# @(#)adlc.make	1.2 00/01/07

# This makefile (adlc.make) is included from the adlc.make in the
# build directories.
# It knows how to compile, link, and run the adlc.

# #########################################################################

# OUTDIR must be the same as AD_Dir = $(GENERATED)/adfiles in top.make:
OUTDIR  = ../generated/adfiles

ARCH = $(Platform_arch)
OS = $(Platform_os_family)

SOURCE.AD = $(OUTDIR)/$(OS)_$(ARCH).ad 

SOURCES.AD = $(GAMMADIR)/src/cpu/$(ARCH)/vm/$(ARCH).ad \
	     $(GAMMADIR)/src/os_cpu/$(OS)_$(ARCH)/vm/$(OS)_$(ARCH).ad 

Src_Dirs += $(GAMMADIR)/src/share/vm/adlc

EXEC	= $(OUTDIR)/adlc

all:

# #########################################################################
# THE FOLLOWING IS TAKEN FROM VM.MAKE

# tell make that .cpp is important

.SUFFIXES: .cpp $(SUFFIXES)


# set VPATH so make knows where to look for source files

Src_Dirs_V = ${Src_Dirs} $(GENERATED)/incls
VPATH    += $(Src_Dirs_V:%=%:)


# set INCLUDES for C preprocessor

Src_Dirs_I = ${Src_Dirs} $(GENERATED)
INCLUDES += $(Src_Dirs_I:%=-I%)

# Suppress warnings (for now)
CFLAGS += -w

# #########################################################################


#  /usr/ccs/bin/make
#  /usr/dist/share/devpro/5.x-sparc/bin/CC
#  man -M /usr/dist/share/devpro/5.x-sparc/man CC

CPP       = g++
CPPFLAGS  = -DASSERT -g -o $@
# -o      object file-name
# +w      Additional warnings: Nonportable, Likely mistakes or inefficient
# -xwe    Treat all warnings as errors (skipped for now)
# +p      Use -features=no%anachronisms.
# -E      Runs a source file through the preprocessor, output to stdout
# 
# $<      first component of target name
# $@      target name

CPPFLAGS += ${SYSDEFS} ${INCLUDES}

OBJECTNAMES = \
	adlparse.o \
	archDesc.o \
	arena.o \
	dfa.o \
	dict2.o \
	filebuff.o \
	forms.o \
	formsopt.o \
	formssel.o \
	main.o \
	adlc-opcodes.o \
	output_c.o \
	output_h.o \

OBJECTS = $(OBJECTNAMES:%=$(OUTDIR)/%)

# Random dependencies:
$(OBJECTS): opcodes.hpp classes.hpp adlc.hpp adlcVMDeps.hpp adlparse.hpp archDesc.hpp arena.hpp dict2.hpp filebuff.hpp forms.hpp formsopt.hpp formssel.hpp

# #########################################################################

all: $(EXEC)

$(EXEC) : $(OBJECTS)
	@echo Making adlc
	@$(CPP) $(DFLAGS) $(LFLAGS) $(OBJECTS) -o $(EXEC)

# The source files refer to ostream.h, which sparcworks calls iostream.h
$(OBJECTS): ostream.h
ostream.h :
	@echo >$@ '#include <iostream.h>'

dump:
	: OUTDIR=$(OUTDIR)
	: OBJECTS=$(OBJECTS)
	: products = $(OUTDIR)/ad_$(ARCH).cpp $(OUTDIR)/ad_$(ARCH).hpp $(OUTDIR)/dfa_$(ARCH).cpp


all: $(OUTDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).cpp $(OUTDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/dfa_$(ARCH).cpp

$(OUTDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).cpp $(OUTDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/dfa_$(ARCH).cpp: refresh_adfiles

# Get a unique temporary directory name, so multiple makes can run in parallel.
# Note that product files are updated via "mv", which is atomic.
TEMPDIR := $(OUTDIR)/mktmp$(shell echo $$$$)

verbose-echo = echo
verbose-echo$(MAKE_VERBOSE) = :

# This action refreshes them all at once:
refresh_adfiles: $(EXEC) $(SOURCE.AD)
	@rm -rf $(TEMPDIR); mkdir $(TEMPDIR)
	@$(verbose-echo) \
	 $(EXEC) -q -T $(SOURCE.AD) \
 -c$(TEMPDIR)/ad_$(ARCH).cpp -h$(TEMPDIR)/ad_$(ARCH).hpp -a$(TEMPDIR)/dfa_$(ARCH).cpp -v$(TEMPDIR)/adGlobals_$(ARCH).hpp
	@$(EXEC) -q -T $(SOURCE.AD) \
 -c$(TEMPDIR)/ad_$(ARCH).cpp -h$(TEMPDIR)/ad_$(ARCH).hpp -a$(TEMPDIR)/dfa_$(ARCH).cpp -v$(TEMPDIR)/adGlobals_$(ARCH).hpp \
	    || { rm -rf $(TEMPDIR); exit 1; }
	@[ -f $(OUTDIR)/ad_$(ARCH).hpp ] \
		&& cmp -s $(TEMPDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).hpp \
		|| ( [ -f $(OUTDIR)/ad_$(ARCH).hpp ]; \
		     echo Updating $(OUTDIR)/ad_$(ARCH).hpp ; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).hpp)

	@[ -f $(OUTDIR)/ad_$(ARCH).cpp ] \
		&& cmp -s $(TEMPDIR)/ad_$(ARCH).cpp $(OUTDIR)/ad_$(ARCH).cpp \
		|| ( [ -f $(OUTDIR)/ad_$(ARCH).cpp ]; \
		     echo Updating $(OUTDIR)/ad_$(ARCH).cpp ; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/ad_$(ARCH).cpp $(OUTDIR)/ad_$(ARCH).cpp)

	@[ -f $(OUTDIR)/adGlobals_$(ARCH).hpp ] \
		&& cmp -s $(TEMPDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/adGlobals_$(ARCH).hpp \
		|| ( [ -f $(OUTDIR)/adGlobals_$(ARCH).hpp ]; \
			echo "Updating adGlobals_$(ARCH).hpp --- Prepare to compile many dependent files."; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/adGlobals_$(ARCH).hpp)

	@[ -f $(OUTDIR)/dfa_$(ARCH).cpp ] \
		&& cmp -s $(TEMPDIR)/dfa_$(ARCH).cpp $(OUTDIR)/dfa_$(ARCH).cpp \
		|| ( [ -f $(OUTDIR)/dfa_$(ARCH).cpp ]; \
		     echo Updating $(OUTDIR)/dfa_$(ARCH).cpp ; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/dfa_$(ARCH).cpp $(OUTDIR)/dfa_$(ARCH).cpp)

	@[ -f $(TEMPDIR)/made-change ] \
		|| echo "Rescanned $(SOURCE.AD) but encountered no changes."

	@rm -rf $(TEMPDIR)
		       

# #########################################################################

$(SOURCE.AD): $(SOURCES.AD)
	@cat $(SOURCES.AD) > $(SOURCE.AD)

COMPILE   = ${CPP} ${CPPFLAGS} ${CFLAGS}

COMPILE.o = ${COMPILE} -c
COMPILE.i = ${COMPILE} -E

verbose-echo = echo
verbose-echo$(MAKE_VERBOSE) = :

$(OUTDIR)/%.o: %.cpp
	@echo Compiling $< ; \
	rm -f $@ ; \
	$(verbose-echo) \
	${COMPILE.o} $< ; \
	${COMPILE.o} $< && \
	case "$(MFLAGS)" in *j*) echo Done with $<;; esac

# Some object files are given a prefix, to disambiguate
# them from objects of the same name built for the VM.
$(OUTDIR)/adlc-%.o: %.cpp
	@echo Compiling $< ; \
	rm -f $@ ; \
	$(verbose-echo) \
	${COMPILE.o} $< ; \
	${COMPILE.o} $<

# Intermediate files (for debugging macros)

%.i: %.cpp
	@echo Preprocessing $< to $@; \
	$(verbose-echo) \
	${COMPILE.i} $< >$@ ; \
	${COMPILE.i} $< >$@

# #########################################################################

clean	:
	rm $(OBJECTS)

cleanall :
	rm $(OBJECTS) $(EXEC)

# #########################################################################


.PHONY: all dump refresh_adfiles clean cleanall
