/*	Copyright 2001 Sun Microsystems, Inc. All rights reserved.	*/
/*	Copyright 2001 Sun Microsystems, Inc. Tous droits rservs.	*/

#include <fcntl.h>
#include <stdio.h>
#include <libelf.h>
#include <sys/elf_SPARC.h>
#include <stdlib.h>
#include <string.h>

static int   fd;
static Elf*  elfD;

void cleanUp(int i) {
   if (elfD != NULL)
      elf_end(elfD);
   if (fd != -1)
      close(fd);
#ifdef VERBOSE
   printf("Cleaned up after file # %d\n", i);
#endif
}

void main(int argc, char** argv) {

  int   i;
  Elf32_Ehdr *elfHdr;
  off_t sz;
  
  if (argc <= 1){
     printf("Usage: %s <file1> [file2] ...\n", argv[0]);
     exit(-1);
  }

  if (elf_version(EV_CURRENT) == EV_NONE) {
     printf("Out of date ELF library\n");
     exit(-1);
  }

  for (i = 1; i< argc; cleanUp(i++)) { /* for each file in list */

     if ((fd = open(argv[i], O_RDWR)) == -1) {
        perror("");
        printf("%s: Unable to open file", argv[i]);
        continue;  /* try next file if any */
     }
     if ((elfD = elf_begin(fd, ELF_C_RDWR, NULL)) == NULL) {
        perror("");
        printf("%s: Unable to obtain ELF descriptor\n", argv[i]);
        continue;
     }
     if ((elfHdr = elf32_getehdr(elfD)) == NULL) {
        perror("");
        printf("%s: Unable to read ELF header\n", argv[i]);
        continue;
     }
     printf("ELF Header of %s:\n", argv[i]);
     printf("\t Read:\te_machine = 0x%x \t e_flags = 0x%x\n",
                         elfHdr->e_machine, elfHdr->e_flags);
     if (elfHdr->e_machine == EM_SPARC32PLUS) {
        elfHdr->e_machine = EM_SPARC;
        elfHdr->e_flags &= (~ EF_SPARC_32PLUS_MASK);
        printf("\t Wrote:\te_machine = 0x%x \t e_flags = 0x%x\n",
                           elfHdr->e_machine, elfHdr->e_flags);
        if ((sz = elf_update(elfD, ELF_C_WRITE)) == -1) {
           perror("");
           printf("\t Surgery failed, patient may be dead\n");
     	} else printf("\t Surgery Complete [0x%x]\n", (int)sz);
     } else printf("\t No surgery necessary\n");
  }
#ifdef VERBOSE
  printf("Done.\n");
#endif VERBOSE
  exit(0);
}
