# @(#)adlc.make	1.33 00/04/19 14:56:17

# This makefile (adlc.make) is included from the adlc.make in the
# build directories.
# It knows how to compile, link, and run the adlc.

include $(GAMMADIR)/build/solaris/makefiles/rules.make

# #########################################################################

# OUTDIR must be the same as AD_Dir = $(GENERATED)/adfiles in top.make:
OUTDIR  = ../generated/adfiles

ARCH = $(Platform_arch)
OS = $(Platform_os_family)

SOURCE.AD = $(OUTDIR)/$(OS)_$(ARCH).ad 

SOURCES.AD = $(GAMMADIR)/src/cpu/$(ARCH)/vm/$(ARCH).ad \
	     $(GAMMADIR)/src/os_cpu/$(OS)_$(ARCH)/vm/$(OS)_$(ARCH).ad 

Src_Dirs += $(GAMMADIR)/src/share/vm/adlc

EXEC	= $(OUTDIR)/adlc

# set VPATH so make knows where to look for source files
Src_Dirs_V = ${Src_Dirs} $(GENERATED)/incls
VPATH    += $(Src_Dirs_V:%=%:)

# set INCLUDES for C preprocessor
Src_Dirs_I = ${Src_Dirs} $(GENERATED)
INCLUDES += $(Src_Dirs_I:%=-I%)

# Force assertions on.
SYSDEFS		+= -DASSERT
CPPFLAGS	= $(SYSDEFS) $(INCLUDES)

# CFLAGS_WARN holds compiler options to suppress/enable warnings.
CFLAGS		+= $(CFLAGS_WARN)

OBJECTNAMES = \
	adlparse.o \
	archDesc.o \
	arena.o \
	dfa.o \
	dict2.o \
	filebuff.o \
	forms.o \
	formsopt.o \
	formssel.o \
	main.o \
	adlc-opcodes.o \
	output_c.o \
	output_h.o \

OBJECTS = $(OBJECTNAMES:%=$(OUTDIR)/%)

# #########################################################################

all: $(EXEC)

$(EXEC) : $(OBJECTS)
	@echo Making adlc
ifeq ("${Platform_arch}", "i486")
	$(QUIETLY) $(LINK_NOPROF.CC) -o $(EXEC) $(OBJECTS)
else
	$(QUIETLY) $(LINK.CC) -o $(EXEC) $(OBJECTS)
endif

# Random dependencies:
$(OBJECTS): opcodes.hpp classes.hpp adlc.hpp adlcVMDeps.hpp adlparse.hpp archDesc.hpp arena.hpp dict2.hpp filebuff.hpp forms.hpp formsopt.hpp formssel.hpp

# The source files refer to ostream.h, which sparcworks calls iostream.h
$(OBJECTS): ostream.h

ostream.h :
	@echo >$@ '#include <iostream.h>'

dump:
	: OUTDIR=$(OUTDIR)
	: OBJECTS=$(OBJECTS)
	: products = $(OUTDIR)/ad_$(ARCH).cpp $(OUTDIR)/ad_$(ARCH).hpp $(OUTDIR)/dfa_$(ARCH).cpp

all: $(OUTDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).cpp $(OUTDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/dfa_$(ARCH).cpp

$(OUTDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).cpp $(OUTDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/dfa_$(ARCH).cpp: refresh_adfiles

# Get a unique temporary directory name, so multiple makes can run in parallel.
# Note that product files are updated via "mv", which is atomic.
TEMPDIR := $(OUTDIR)/mktmp$(shell echo $$$$)

# This action refreshes them all at once:
refresh_adfiles: $(EXEC) $(SOURCE.AD)
	@rm -rf $(TEMPDIR); mkdir $(TEMPDIR)
	$(QUIETLY) $(EXEC) -q -T $(SOURCE.AD) \
 -c$(TEMPDIR)/ad_$(ARCH).cpp -h$(TEMPDIR)/ad_$(ARCH).hpp -a$(TEMPDIR)/dfa_$(ARCH).cpp -v$(TEMPDIR)/adGlobals_$(ARCH).hpp \
	    || { rm -rf $(TEMPDIR); exit 1; }
	$(QUIETLY) [ -f $(OUTDIR)/ad_$(ARCH).hpp ] \
		&& cmp -s $(TEMPDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).hpp \
		|| ( [ -f $(OUTDIR)/ad_$(ARCH).hpp ]; \
		     echo Updating $(OUTDIR)/ad_$(ARCH).hpp ; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/ad_$(ARCH).hpp $(OUTDIR)/ad_$(ARCH).hpp)
	$(QUIETLY) [ -f $(OUTDIR)/ad_$(ARCH).cpp ] \
		&& cmp -s $(TEMPDIR)/ad_$(ARCH).cpp $(OUTDIR)/ad_$(ARCH).cpp \
		|| ( [ -f $(OUTDIR)/ad_$(ARCH).cpp ]; \
		     echo Updating $(OUTDIR)/ad_$(ARCH).cpp ; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/ad_$(ARCH).cpp $(OUTDIR)/ad_$(ARCH).cpp)
	$(QUIETLY) [ -f $(OUTDIR)/adGlobals_$(ARCH).hpp ] \
		&& cmp -s $(TEMPDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/adGlobals_$(ARCH).hpp \
		|| ( [ -f $(OUTDIR)/adGlobals_$(ARCH).hpp ]; \
			echo "Updating adGlobals_$(ARCH).hpp --- Prepare to compile many dependent files."; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/adGlobals_$(ARCH).hpp $(OUTDIR)/adGlobals_$(ARCH).hpp)
	$(QUIETLY) [ -f $(OUTDIR)/dfa_$(ARCH).cpp ] \
		&& cmp -s $(TEMPDIR)/dfa_$(ARCH).cpp $(OUTDIR)/dfa_$(ARCH).cpp \
		|| ( [ -f $(OUTDIR)/dfa_$(ARCH).cpp ]; \
		     echo Updating $(OUTDIR)/dfa_$(ARCH).cpp ; \
		     touch $(TEMPDIR)/made-change ; \
		     mv $(TEMPDIR)/dfa_$(ARCH).cpp $(OUTDIR)/dfa_$(ARCH).cpp)
	$(QUIETLY) [ -f $(TEMPDIR)/made-change ] \
		|| echo "Rescanned $(SOURCE.AD) but encountered no changes."
	$(QUIETLY) rm -rf $(TEMPDIR)


# #########################################################################

$(SOURCE.AD): $(SOURCES.AD)
	$(QUIETLY) cat $(SOURCES.AD) > $(SOURCE.AD)

$(OUTDIR)/%.o: %.cpp
	@echo Compiling $<
	$(QUIETLY) $(COMPILE.CC) -o $@ $<
	$(COMPILE_DONE_MSG)

# Some object files are given a prefix, to disambiguate
# them from objects of the same name built for the VM.
$(OUTDIR)/adlc-%.o: %.cpp
	@echo Compiling $<
	$(QUIETLY) $(COMPILE.CC) -o $@ $<
	$(COMPILE_DONE_MSG)

# #########################################################################

clean	:
	rm $(OBJECTS)

cleanall :
	rm $(OBJECTS) $(EXEC)

# #########################################################################

.PHONY: all dump refresh_adfiles clean cleanall
