# Compiler-specific flags for sparcworks.

# tell make which C and C++ compilers to use
CC	= cc
CPP	= CC

# Get the last thing on the line that looks like x.x+ (x is a digit).
COMPILER_REV := \
$(shell $(CPP) -V 2>&1 | sed -e 's/^.*\([1-9]\.[0-9][0-9]*\).*/\1/')

ENFORCE_COMPILER_REV${ENFORCE_COMPILER_REV} := 5.0
ifneq (${COMPILER_REV},${ENFORCE_COMPILER_REV})
dummy_target_to_enforce_compiler_rev:
	@echo "Wrong ${CPP} version:  ${COMPILER_REV}. " \
	"Use version ${ENFORCE_COMPILER_REV}, or set" \
	"ENFORCE_COMPILER_REV=${COMPILER_REV}."
	@exit 1
endif

# Some interfaces (_lwp_create) changed with LP64 and Solaris 7
SOLARIS_7_OR_LATER := \
$(shell uname -r | awk -F. '{ if ($$2 >= 7) print "-DSOLARIS_7_OR_LATER"; }')
CFLAGS += ${SOLARIS_7_OR_LATER}

ifeq	(${COMPILER_REV}, 5.0)
# BobV says we need this, I haven't seen any evidence why
CFLAGS += -D__STD_CSIGNAL
CFLAGS += -library=iostream
LFLAGS += -library=iostream  -library=Crun
LIBS += -library=iostream -library=Crun -lCrun
endif	# COMPILER_REV

ifeq ("${COMPILER_REV}", "4.2")
# 4.2 COMPILERS SHOULD NO LONGER BE USED
_JUNK_ := $(shell echo >&2 \
       "*** ERROR: SC4.2 compilers are not supported by this code base!")
	@exit 1
endif

# PIC is safer for SPARC, and is considerably slower
# a file foo.o which wants to compile -pic can set "PICFLAG/foo.o = -PIC"
PICFLAG/DEFAULT = -PIC
PICFLAG/BETTER  = -pic
PICFLAG/BYFILE  = $(PICFLAG/$@)$(PICFLAG/DEFAULT$(PICFLAG/$@))

# need position-indep-code for shared libraries
# (ild appears to get errors on PIC code, so we'll try non-PIC for debug)
ifeq ($(PICFLAGS),DEFAULT)
PICFLAG/LIBJVM  = $(PICFLAG/DEFAULT)
else
PICFLAG/LIBJVM  = $(PICFLAG/BYFILE)
endif
PICFLAG/AOUT    =
CFLAGS += $(PICFLAG/$(LINK_INTO))

# less dynamic linking (no PLTs, please)
#LIB_FLAGS += $(LINK_MODE)
# %%%%% despite -znodefs, -Bsymbolic gets link errors -- Rose

LINK_MODE = $(LINK_MODE/$(VERSION))
LINK_MODE/debug     =
LINK_MODE/optimized = -Bsymbolic -znodefs

# no more exceptions
CFLAGS += -noex

# Have thread local errnos
CFLAGS += -D_REENTRANT

# Flags for Debugging
DEBUG_CFLAGS = -g

# Flags for Optimization
OPT_CFLAGS=-xO4 -xarch=v8 -xchip=ultra2 $(EXTRA_OPT_CFLAGS)
ifeq ("${Platform_arch}", "i486")
OPT_CFLAGS=-xtarget=pentium $(EXTRA_OPT_CFLAGS)
ifeq ("${COMPILER_REV}", "5.0")
# SC5.0 tools on x86 are flakey at -xO4
OPT_CFLAGS+=-xO3
else
OPT_CFLAGS+=-xO4
endif
endif

# Use these to work around compiler bugs:
OPT_CFLAGS/SLOWER = -xO3 -xarch=v8 -xchip=ultra2
OPT_CFLAGS/O2     = -xO2 -xarch=v8 -xchip=ultra2

# The following options run into misaligned ldd problem (raj)
#OPT_CFLAGS = -fast -O4 -xarch=v8 -xchip=ultra

# Uncomment or 'gmake CFLAGS_BROWSE=-sbfast' to get source browser information.
# CFLAGS_BROWSE	= -sbfast
CFLAGS		+= $(CFLAGS_BROWSE)

# use ild when debugging (but when optimizing we want reproducible results)
ILDFLAG = $(ILDFLAG/$(VERSION))
ILDFLAG/debug     = -xildon
ILDFLAG/optimized =
AOUT_FLAGS += $(ILDFLAG)

# Where to put the *.o files (a.out, or shared library)?
LINK_INTO = $(LINK_INTO/$(VERSION))
LINK_INTO/debug = AOUT
LINK_INTO/optimized = LIBJVM

# Solaris platforms collect lots of redundant file-ident lines,
# to the point of wasting a significant percentage of file space.
# (The text is stored in ELF .comment sections, contributed by
# all "#pragma ident" directives in header and source files.)
# This command "compresses" the .comment sections simply by
# removing repeated lines.  The data can be extracted from
# binaries in the field by using "mcs -p libjvm.so" or the older
# command "what libjvm.so".
LINK_LIB.CC/POST_HOOK += mcs -c $@ ;
# (The semicolon is necessary, since the hook must terminate
# itself as a valid command.)

# Also, strip debug and line number information (worth about 1.7Mb).
STRIP_LIB.CC/POST_HOOK = strip -x $@ ;
# STRIP_LIB.CC/POST_HOOK is incorporated into LINK_LIB.CC/POST_HOOK
# in certain configurations, such as product.make.  Other configurations,
# such as debug.make, do not include the strip operation.

