//
// @(#)c1_CodeEmitter_i486.cpp	1.297 01/03/27 14:26:33
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_CodeEmitter_i486.cpp.incl"
#include <math.h>

#define __ _masm->

void PatchingDesc::set_hi_word() { _field_offset = 1; }  // marks move that moves Hi-word
void PatchingDesc::set_lo_word() { _field_offset = 0; }  // marks move that moves Lo-word

//-------------------------------------------------------------

void ConstantTable::emit_entries(MacroAssembler* masm) {
  for (int i = 0; i < _entries->length(); i++) {
    FPUConstEntry* fp = _entries->at(i);;
    if (fp->is_double()) {
      masm->align(wordSize*2); // double word alignment
      jlong temp = jlong_cast(fp->double_value());
      fp->set_pc((address)masm->pc());
      masm->a_long(low(temp));
      masm->a_long(high(temp));
    } else {
      masm->align(wordSize); // probably always is
      assert(fp->is_float(), "invalid entry");
      jint temp = jint_cast(fp->float_value());
      fp->set_pc((address)masm->pc());
      masm->a_long(temp);
    }
  }
}


// Specify all special register usage here
const Register IC_Klass    = eax;   // where the IC klass is cached
const Register SHIFT_count = ecx;   // where count for shift operations must be
const Register SYNC_header = eax;   // synchronization header
const Register RECEIVER    = ecx;   // where receiver is preloaded
const Register EXCEPT_oop  = eax;   // holding the exception oop

// other constants
const int MAX_OFFSET = (max_jint - 1) / wordSize; // this offset is used for field accesses, guaranteeing a 4-word offset size


// Creation

CodeEmitter::CodeEmitter(
    Compilation* compilation,
    ConstantTable* table,
    JVMCI_OffsetDesc* offsets
  )
  : _compilation(compilation)
  , _masm(compilation->masm())
  , _method(compilation->method())
  , _frame_map(compilation->frame_map())
  , _const_table(table)
  , _offsets(offsets)
  , _esp_offset(0)
  , _need_debug_info(false)
  , _bailout(false)
  , _cc_locked(false)
  , _info(compilation->info())
  , _linenumbers_offset(NULL)
{
  _slow_case_stubs = new GrowableArray<CodeStub*>();
  _call_stubs      = new GrowableArray<CodeStub*>();
}


//------------------------------------------------------
//          helpers
//------------------------------------------------------

Register CodeEmitter::receiver_reg() const {
  return RECEIVER;
}

//--------------Labels----------------------------------

void CodeEmitter::bind_label(Label* label) {
  __ bind(*label);
}


//--------------FPU Stack----------------------------------

// For INTEL we simulate the pushing and popping of elements on FPU stack.
// This way we know the offsets between FPU registers. This also
// means that any FPU register involved must have ref-count 1, i.e, is
// read-once.

void CodeEmitter::fpu_pop(RInfo reg) {
  frame()->fpu_stack()->pop(reg.fpu());
}


void CodeEmitter::fpu_push(RInfo reg) {
  frame()->fpu_stack()->push(reg.fpu());
}


// forces reg to be on FPU TOS
void CodeEmitter::fpu_on_tos(RInfo reg) {
  if (frame()->fpu_stack()->is_stack_pos(reg.fpu(), 0)) return; // register already on TOS
  int delta = frame()->fpu_stack()->bring_on_tos_0(reg.fpu());
  __ fxch(delta);
  assert(frame()->fpu_stack()->is_stack_pos(reg.fpu(), 0), "should be on TOS?");
}


// if two regs are not on TOS return false;
// if two regs are on TOS and in right order return false;
// if two regs are on TOS and in reverse order, return true
bool CodeEmitter::must_swap_two_on_tos(RInfo tos0, RInfo tos1) {
  bool tos1_is0 = frame()->fpu_stack()->is_stack_pos(tos1.fpu(), 0); 
  bool tos0_is1 = frame()->fpu_stack()->is_stack_pos(tos0.fpu(), 1); 
  return tos0_is1 && tos1_is0;
}


// Forces tos0 and tos1 to be on FPU TOS; if "must_be_ordered" is true,
// then tos0 must be in ST(0) and tos1 must be in ST(1)
void CodeEmitter::fpu_two_on_tos(RInfo tos0, RInfo tos1, bool must_be_ordered) {
  if (!must_be_ordered) {
    bool tos1_is1 = frame()->fpu_stack()->is_stack_pos(tos1.fpu(), 1); 
    bool tos1_is0 = frame()->fpu_stack()->is_stack_pos(tos1.fpu(), 0); 
    bool tos0_is1 = frame()->fpu_stack()->is_stack_pos(tos0.fpu(), 1); 
    bool tos0_is0 = frame()->fpu_stack()->is_stack_pos(tos0.fpu(), 0); 
    if (tos1_is1 && tos0_is0) {
      return; // Done
    } else if (tos1_is0 && tos0_is1) {
      frame()->fpu_stack()->swap_two_on_tos(); // fake exchange; no fxch emitted
      return; // Done!
    }
  }
  if (!frame()->fpu_stack()->is_stack_pos(tos1.fpu(), 1)) {
    fpu_on_tos(tos1);
    frame()->fpu_stack()->swap_two_on_tos();
    __ fxch(1);
  }
  assert(frame()->fpu_stack()->is_stack_pos(tos1.fpu(), 1), "did not work?");
  fpu_on_tos(tos0);
}


// Simulates pushing of result on ST(0)
void CodeEmitter::set_fpu_result(RInfo reg) {
  frame()->fpu_stack()->push(reg.fpu());
}


// Removes value on ST(0), ST(1) becomes ST(0)
void CodeEmitter::remove_fpu_result(RInfo reg) {
  frame()->fpu_stack()->pop(reg.fpu());
  __ fpop();
}

void CodeEmitter::fpop() {
  __ fpop();
}


bool CodeEmitter::is_empty_fpu_stack() {
  return frame()->fpu_stack()->is_empty();
}


void CodeEmitter::set_fpu_stack_empty() {
  frame()->fpu_stack()->clear();
}


// Copy item, without removing it from fpu stack; the value at ST(x) is pushed as ST(0)
// The new value is in "reg"
void CodeEmitter::copy_fpu_item(RInfo reg, Item* item) {
  RInfo from_reg = item->get_register();
  assert(from_reg.is_float_kind(), "wrong item register");
  int offset = frame()->fpu_stack()->tos_offset_for(from_reg.fpu());
  __ fld_s(offset);
  frame()->fpu_stack()->push(reg.fpu());
}


//---------------------item conversion--------------------------


Address CodeEmitter::item2addr_with_offset(Item* item, int offset) {
  assert(item->is_register() && item->type()->is_single_word() && item->type()->is_object(), "cannot handle other cases");
  return Address(item2reg(item), offset*wordSize);
}


Address CodeEmitter::item2addr_with_offsetLO(Item* item, int offset) {
  return item2addr_with_offset(item, offset);
}


Address CodeEmitter::item2addr_with_offsetHI(Item* item, int offset) {
  assert(item->is_register() && item->type()->is_single_word() && item->type()->is_object(), "cannot handle other cases");
  return Address(item2reg(item), (offset+1)*wordSize);
}


//-----------------------------------------------------------------------


Address::ScaleFactor CodeEmitter::array_element_size(BasicType type) const {
  int elem_size = type2aelembytes[type];
  switch (elem_size) {
    case 1: return Address::times_1;
    case 2: return Address::times_2;
    case 4: return Address::times_4;
    case 8: return Address::times_8;
  }
  ShouldNotReachHere();
  return Address::no_scale;
}


Address CodeEmitter::item2array_addr(Item* array, Item* index, BasicType dst_type, int offset) {
  assert(array->is_register(), "item in wrong form");
  Register array_reg = array->get_register().as_register();
  if (index->is_register()) {
    Address::ScaleFactor factor = array_element_size(dst_type);
    Register index_reg = index->get_register().as_register();
    return Address(array_reg, index_reg, factor, arrayOopDesc::base_offset_in_bytes(dst_type) + offset);
  } else {
    assert(index->is_constant(), "index can be either register or constant");
    int elem_size = type2aelembytes[dst_type];
    int adr_offset = item2int(index) * elem_size + arrayOopDesc::base_offset_in_bytes(dst_type) + offset;
    return Address(array_reg, adr_offset);
  }
}


Address CodeEmitter::item2array_addrLO(Item* array, Item* index, BasicType dst_type) {
  return item2array_addr(array, index, dst_type);
}


Address CodeEmitter::item2array_addrHI(Item* array, Item* index, BasicType dst_type) {
  return item2array_addr(array, index, dst_type, wordSize);
}


jobject CodeEmitter::item2jobject(Item* item) {
  assert(item->is_constant() && item->type()->is_object(), "item is not an object constant");
  return item->get_jobject_constant()->encoding();
}


jint CodeEmitter::item2int(Item* item) {
  assert(item->is_constant() && !item->type()->is_object() && item->type()->is_single_word(), "item is not an int constant");
  ValueTag tag = item->type()->tag();
  jint res;
  switch(item->type()->tag()) {
    case intTag     : res = item->get_jint_constant(); break;
    case addressTag : res = item->get_address_constant(); break;
    case floatTag   : 
      {
        // Note: do not change the following code "style"; without 
        // explicit assignment, VC 5.0 generates wrong code in the release 
        // version
        jfloat f = item->get_jfloat_constant();
        res = jint_cast(f);
      }
      break;
    default: ShouldNotReachHere();
  }
  return res;
}


jint CodeEmitter::item2intLO(Item* item) {
  assert(item->is_constant() && item->type()->is_double_word(), "wrong item");
  jint res;
  if (item->type()->is_long()) {
    res = low(item->get_jlong_constant());
  } else {
    // Note: do not change the following code "style"; without 
    // explicit assignment, VC 5.0 generates wrong code in the release 
    // version
    jdouble d = item->get_jdouble_constant();
    jlong   l = jlong_cast(d);
    res = low(l);
  }
  return res;
}


jint CodeEmitter::item2intHI(Item* item) {
  assert(item->is_constant() && item->type()->is_double_word(), "wrong item");
  jint res;
  if (item->type()->is_long()) {
    // Note: do not change the following code "style"; without 
    // explicit assignment, VC 5.0 generates wrong code in the release 
    // version
    jlong l = item->get_jlong_constant();
    res = high(l);
  } else {
    // Note: do not change the following code "style"; without 
    // explicit assignment, VC 5.0 generates wrong code in the release 
    // version
    jdouble d = item->get_jdouble_constant();
    jlong   l = jlong_cast(d);
    res = high(l);
  }
  return res;
}


// Note: NEVER push values directly, but only through following push_xxx functions;
//       This helps us to track the esp changes compared to the entry esp (->_esp_offset)


void CodeEmitter::push_int(jint i) {
  _esp_offset++;
  __ pushl(i);
}


void CodeEmitter::push_oop(jobject o) {
  _esp_offset++;
  __ pushl(o);
}


void CodeEmitter::push_addr(Address a) {
  _esp_offset++;
  __ pushl(a);
}


void CodeEmitter::push_reg(Register r) {
  _esp_offset++;
  __ pushl(r);
}


void CodeEmitter::pop(Register r) {
  _esp_offset--;
  assert(_esp_offset >= 0, "stack offset underflow");
  __ popl(r);
}


void CodeEmitter::dec_stack(int nof_words) {
  _esp_offset -= nof_words;
  assert(_esp_offset >= 0, "stack offset underflow");
  __ addl(esp, wordSize * nof_words);
}


void CodeEmitter::dec_stack_after_call(int nof_words) {
  _esp_offset -= nof_words;
  assert(_esp_offset >= 0, "stack offset underflow");
}


void CodeEmitter::remove_all_stack() {
  __ addl (esp, _esp_offset * wordSize);
  _esp_offset = 0;
}


// This specifies the esp decrement needed to build the frame
int CodeEmitter::stack_offset_in_bytes(int nof_spills) {
  return (frame()->size_of_stack_decrement() + nof_spills ) * wordSize;
}


// In prolog esp points to return address, and frame has not been built yet
Address CodeEmitter::receiver_address_in_prolog() {
  return Address(esp, method()->arg_size() * wordSize);
}


// inline cache check; done before the frame is built.
void CodeEmitter::check_icache() {
  // receiver is already in ecx
  __ verify_oop(RECEIVER);
  // check if receiver is truly in ecx
  if (VerifyReceiver) {
    Label L;
    // frame does not exist yet; esp points to the return address
    __ cmpl(RECEIVER, receiver_address_in_prolog());
    __ jcc(Assembler::equal, L);
    __ stop("nmethod ic check: receiver must be in ecx");
    __ bind(L);
  }
  // explicit NULL check not needed since load from [0x4] causes a trap  
  // check against inline cache
  __ cmpl(IC_Klass, Address(RECEIVER, oopDesc::klass_offset_in_bytes())); 
  // if icache check fails, then jump to runtime routine
  // Note: ecx must still contain the receiver!
  __ jcc(Assembler::notEqual, Runtime1::entry_for(Runtime1::handle_ic_miss_id), relocInfo::runtime_call_type); 
}


void CodeEmitter::bang_stack_with_offset(int offset) {
  __ movl (Address(esp, offset), eax);
}


void CodeEmitter::bang_stack() {
  bang_stack_with_offset(-os::vm_page_size());
}

void CodeEmitter::build_frame(int max_spills) {
#ifdef ASSERT
  if (method()->is_native()) {
    if (method()->is_static()) {
      assert(max_spills == 1, "static native methods need 1 slot for mirror");
    } else {
      assert(max_spills == 0, "non-static native methods must have no spill slots");
    }
  }
#endif
  __ enter();
  int stack_offset = stack_offset_in_bytes(max_spills);
  __ decrement(esp, stack_offset); // does not emit code for stack_offset == 0

  // Note: os::vm_page_size() returns size in bytes!!
  if (method()->is_native()) {
    // make sure that no stack overflow can happen in the natives
    bang_stack();
  }
  // must ping each page of stack (INTEL specific)
  // and extra bang for possible exception processing
  int end = - os::bang_stack_size_for_exceptions();
  for( int offset = stack_offset - os::vm_page_size(); offset > end; offset -= os::vm_page_size() ) {
    bang_stack_with_offset(offset);
  }
}


void CodeEmitter::init_local(int local_index) {
  __ movl(frame()->local_address(local_index, false), (jobject)NULL); // this is done only for possible oop locations
}


void CodeEmitter::restore_cached_receiver() {
  load_receiver(RECEIVER);
}


// call after frame has been built
void CodeEmitter::load_receiver(Register reg) {
  __ movl(reg, frame()->local_address(0, false));	
}


void CodeEmitter::move_reg(Register dst, Register src) {
  if (dst != src) __ movl(dst, src);
}


void CodeEmitter::swap_reg(Register a, Register b) {
  push_reg(a);
  __ movl (a, b);
  pop     (b);
}


void CodeEmitter::swap_reg_with_tmp(Register a, Register b, Register tmp) {
  __ movl (tmp, a);
  __ movl (a, b);
  __ movl (b, tmp);
}



// try the fast way top get Object.hashCode, before the frame is constructed
void CodeEmitter::try_fast_ObjectHashCode() {
  // get the header
  if (compilation()->jvmpi_event_method_enabled()) return;
  Label slowCase;
  Register hdr = eax; // will also be 
  __ movl(hdr, Address(RECEIVER, oopDesc::mark_offset_in_bytes()));

  // check if locked
  __ testl (hdr, markOopDesc::unlocked_value);
  __ jcc (Assembler::zero, slowCase);

  // get hash
  __ andl (hdr, markOopDesc::hash_mask_in_place);
  // test if hashCode exists
  __ jcc  (Assembler::zero, slowCase);
  __ shrl (hdr, markOopDesc::hash_shift);
  int arg_size = (UseC2CallingConventions) ? 0 : method()->arg_size() * wordSize;

  safepoint_return(arg_size);
  __ bind (slowCase);
}


void CodeEmitter::method_entry(GrowableArray<int>* init_vars, int max_spills) {
  // to check: make alignment 16 bytes for Pentium Pro.
  // verify this and measure it: we hope that thanks to CHA, we can almost
  // always jump to the verified entry point anyway.
  const AlignEntryCode = 4;

  // We misalign the entry point so that the verified entry point will be
  // aligned and no nops between check_icache and the verified entry. The code
  // in the unverified entry is known and fixed and so we can pad with these
  // nops.
  // At this point only native method stubs need this alignment since only
  // they can be zombied with threads active. When c1 deoptimizes this
  // will have to happen for all deoptimization candidates.
  //
  if (method()->is_native() && needs_icache()) {
    __ align(AlignEntryCode);
    // Add nops to misalign entry...
    if (!C1Breakpoint) __ nop();
    __ nop();
    __ nop();
  }

  // entry point
  _offsets->_ep_offset = __ offset();
  if (needs_icache()) {
    // Note: even if this is a final method, we may need IC check
    // as the final method may be one of many methods in the hierarchy
    if (C1Breakpoint) __ int3();
    check_icache();
  }

  // verified entry point
  // Do not align vep for non-finals as we mostly enter through ep anyway
  if (method()->is_final_method() || method()->is_native()) {
    __ align(AlignEntryCode);
  }
  _offsets->_vep_offset = __ offset();
  _offsets->_iep_offset = __ offset();

  // Native method stubs can be zombied at non-safepoints so they need an
  // instruction large enough for the patching in patch_verified_entry
  // (5+ bytes)
  if (method()->is_native()) {
    __ fat_nop();
  }
  if (C1Breakpoint) __ int3();

  // build frame
  _offsets->_code_offset = __ offset();
  __ verify_FPU(0, "method_entry");
  if (!method()->is_static() && VerifyReceiver) {
    Label L;
    // frame does not exist yet; esp points to the return address
    __ cmpl(ecx, receiver_address_in_prolog());
    __ jcc(Assembler::equal, L);
    __ stop("nmethod vep check: receiver must be in ecx");
    __ bind(L);
  }

  if (method()->intrinsic_id() == methodOopDesc::_hash) {
    try_fast_ObjectHashCode();
  }
  build_frame(max_spills);

  // clear necessary variables; must happen before synchronization code
  // because synchronization entry may block
  if (init_vars != NULL) {
    int i = init_vars->length();
    while (i-- > 0) init_local(init_vars->at(i));
  }

  if (compilation()->jvmpi_event_method_entry_enabled() ||
      compilation()->jvmpi_event_method_entry2_enabled()) {
    // we need the map for arguments only, therefore we use
    // SynchronizationEntryBCI, although there may be no synchronization in
    // this method
    CodeEmitInfo info(SynchronizationEntryBCI, NULL, NULL);
    jvmpi_method_entry(&info);
  }

  if (method()->is_synchronized() && GenerateSynchronizationCode) {
    const Register obj = RECEIVER; // object to lock
    if (method()->is_static()) {
      __ movl(obj, method()->holder()->java_mirror()->encoding());
    } else {
      // receiver is already loaded
    }
    Register lock_reg = esi;
    CodeEmitInfo info(SynchronizationEntryBCI, NULL, NULL);
    monitorenter(obj, lock_reg, SYNC_header, 0, &info); // sync. slot is 0, however should really use a function here
  }
}


void CodeEmitter::spill (int spill_ix, Item* item ) {
  assert(item->is_register() , "why spill if not a register?");
  ValueTag tag = item->type()->tag();
  if (tag == floatTag || tag == doubleTag) {
    fpu_on_tos(item->get_register());
    Address dst_addr = frame()->spill_address(spill_ix, tag == doubleTag);
    if (tag == floatTag) __ fstp_s (dst_addr);
    else                 __ fstp_d (dst_addr);
    fpu_pop   (item->get_register());
  } else {
    if (tag == longTag) {
      Address dstLO = frame()->spill_address(spill_ix, true, false);
      Address dstHI = frame()->spill_address(spill_ix, true, true);
      __ movl (dstLO, item2regLO(item));
      __ movl (dstHI, item2regHI(item));
    } else {
      Address dst = frame()->spill_address(spill_ix, false);
      __ movl (dst, item2reg(item));
    }
  }
}


void CodeEmitter::monitorenter(Register obj_reg, Register lock_reg, Register hdr, int monitor_no, CodeEmitInfo* info) {
  if (!GenerateSynchronizationCode) return;
  
  // We need to bang stack before attempting a lock, because the topmost frame
  // will be lost when handling stack overflow error;
  // A better solution would be to bang stack only once in the beginning if method is synchronized
  // or has monitors
  bang_stack();

  // setup registers (hdr must be eax for lock_object)
  assert(obj_reg != SYNC_header && lock_reg != SYNC_header, "eax must be available here");
  assert(hdr == SYNC_header, "wrong header register");
  // compute pointer to BasicLock
  __ leal(lock_reg, frame()->monitor_address(monitor_no));
  __ verify_oop(obj_reg);
  // lock object
  MonitorAccessStub* slow_case = new MonitorEnterStub(obj_reg, lock_reg, info, esp_offset());
  _slow_case_stubs->append(slow_case);
  if (UseFastLocking) {
    // try inlined fast locking first, revert to slow locking if it fails
    // note: lock_reg points to the displaced header since the displaced header offset is 0!
    assert(BasicLock::displaced_header_offset_in_bytes() == 0, "lock_reg must point to the displaced header");
    add_debug_info_here(info);
    __ lock_object(hdr, obj_reg, lock_reg, *slow_case->entry());
  } else {
    Unimplemented(); // slow case is not completely implemented
    // always do slow locking
    // note: the slow locking code could be inlined here, however if we use
    //       slow locking, speed doesn't matter anyway and this solution is
    //       simpler and requires less duplicated code - additionally, the
    //       slow locking code is the same in either case which simplifies
    //       debugging
    __ jmp(*slow_case->entry());
  }
  // done
  __ bind(*slow_case->continuation());
}


void CodeEmitter::monitorexit(Register obj_reg, Register lock_reg, Register new_hdr, int monitor_no, Register exception) {
  if (!GenerateSynchronizationCode) return;
  // preserve exception if any
  // note: the monitor_exit runtime call is a leaf routine
  //       and cannot block => no GC can happen
  if (exception.isValid()) push_reg(exception);
  // setup registers (lock_reg must be eax for lock_object)
  assert(obj_reg != SYNC_header && lock_reg != SYNC_header, "eax must be available here");
  Register hdr = lock_reg;
  assert(new_hdr == SYNC_header, "wrong register");
  lock_reg = new_hdr;
  // compute pointer to BasicLock
  Address lock_addr = frame()->monitor_address(monitor_no);
  __ leal(lock_reg, lock_addr);
  __ verify_oop(obj_reg);
  // unlock object
  MonitorAccessStub* slow_case = new MonitorExitStub(obj_reg, lock_reg, UseFastLocking, lock_addr);
  // _slow_case_stubs->append(slow_case);
  // temporary fix: must be created after exceptionhandler, therefore as call stub
  _call_stubs->append(slow_case);
  if (UseFastLocking) {
    // try inlined fast unlocking first, revert to slow locking if it fails
    // note: lock_reg points to the displaced header since the displaced header offset is 0!
    assert(BasicLock::displaced_header_offset_in_bytes() == 0, "lock_reg must point to the displaced header");
    __ unlock_object(hdr, obj_reg, lock_reg, *slow_case->entry());
  } else {
    Unimplemented(); // slow case is not fully implemented
    // always do slow unlocking
    // note: the slow unlocking code could be inlined here, however if we use
    //       slow unlocking, speed doesn't matter anyway and this solution is
    //       simpler and requires less duplicated code - additionally, the
    //       slow unlocking code is the same in either case which simplifies
    //       debugging
    __ jmp(*slow_case->entry());
  }
  // done
  __ bind(*slow_case->continuation());
  // restore exception if any
  if (exception.isValid()) pop(exception);
}


void CodeEmitter::clear_local(int local_index) {
  __ movl (frame()->local_address(local_index, false), 0);
}


void CodeEmitter::local_2_reg(RInfo reg, int local_index, BasicType t) {
  if (reg.is_word()) {
    Address adr = frame()->local_address(local_index, false);
    __ movl (reg.as_register(), adr);
  } else {
    Unimplemented();
  }
}


void CodeEmitter::reg_2_local(int local_index, RInfo reg, BasicType t) {
  if (reg.is_word()) {
    Address adr = frame()->local_address(local_index, false);
    __ movl (adr, reg.as_register());
  } else {
    Unimplemented();
  }
}


void CodeEmitter::nops_for_zombie() {
  while ( __ offset() - _offsets->_vep_offset < NativeCall::instruction_size) {
    __ nop(); 
  }
}

// j represents a class; if it is NULL, we need to patch the move and insert the class at runtime
void CodeEmitter::jobject_2_reg_with_patching(RInfo r, ciObject* obj, CodeEmitInfo* info) {
  nops_for_zombie();
  Register reg = r.as_register();
  if (!obj->is_loaded()) {
    PatchingDesc patch;
    set_patching_desc(&patch);
    __ movl(reg, (jobject)NULL);
    patching_epilog(&patch, info, PatchingStub::load_klass_id);
  } else {
    // no patching needed
    __ movl(reg, obj->encoding());
  } 
}


void CodeEmitter::item_2_reg_with_patching(RInfo reg, Item* klass_item, CodeEmitInfo* info) {
  assert(klass_item->type()->is_object() && klass_item->is_constant(), "wrong item");
  // we expect that next instruction will be patched and will be responsible of
  // of fixing up the move
  jobject_2_reg_with_patching(reg, klass_item->get_jobject_constant(), info);
}


void CodeEmitter::item_2_reg (RInfo reg, Item* item) {
  ValueTag tag = item->type()->tag();
  if (tag == floatTag || tag == doubleTag)  {
    if (item->is_register()) {
      fpu_on_tos (item->get_register());
      fpu_pop    (item->get_register());
    } else if (item->is_stack()) {
      Address src = stack_item2addr(item);
      if (tag == floatTag) __ fld_s(src);
      else                 __ fld_d(src);
    } else if (item->is_constant()) {
      // later we are going to fetch it from constant table
      // make room on stack or use scratch area
      if (tag == floatTag) { 
        jfloat f = item->get_jfloat_constant();
        if (is_zero_float(f)) {
          __ fldz();
        } else if (is_one_float(f)) {
          __ fld1();
        } else {
          address const_addr = _const_table->address_of_float_constant(f);
          if (const_addr == NULL) {
            assert(!UseFPConstTables, "if using const tables, entry should exist");
            jint i = jint_cast(f);
            push_int(i);
            __ fld_s(Address(esp));
            dec_stack(1);
          } else {
            __ fld_s (Address((int)const_addr, relocInfo::internal_word_type));
          }
        }
      } else { 
        jdouble d = item->get_jdouble_constant();

        if (is_zero_double(d)) {
          __ fldz();
        } else if (is_one_double(d)) {
          __ fld1();
        } else {
          address const_addr = _const_table->address_of_double_constant(d);
          if (const_addr == NULL) {
            assert(!UseFPConstTables, "if using const tables, entry should exist");
            jlong l = jlong_cast(d);
            push_int(high(l));
            push_int(low(l));
            __ fld_d(Address(esp));
            dec_stack(2);
          } else {
            __ fld_d (Address((int)const_addr, relocInfo::internal_word_type));
          }
        }
      }
    } else ShouldNotReachHere()
    fpu_push(reg);
  } else if (tag == longTag) {
    Register rlo = reg.as_register_lo();
    Register rhi = reg.as_register_hi();
    if (item->is_register()) {
      if (item2regHI(item) == rlo) {
        // do not overwrite item register first
        assert(rhi != item2regLO(item), "wrong regalloc");
        move_reg (rhi, item2regHI(item));
        move_reg (rlo, item2regLO(item));
      } else {
        move_reg (rlo, item2regLO(item));
        move_reg (rhi, item2regHI(item));
      }
    } else if (item->is_stack()) {
      __ movl(rlo, stack_item2addrLO(item));
      __ movl(rhi, stack_item2addrHI(item));
    } else if (item->is_constant()) {
      __ movl(rlo, item2intLO(item));
      __ movl(rhi, item2intHI(item));
    } else {
      ShouldNotReachHere();
    }
  } else {
    assert(item->type()->is_single_word(), "cannot be");
    Register r = reg.as_register();
    if (item->is_register()) {
      move_reg (r, item2reg(item));
    } else if (item->is_stack()) {
      __ movl (r, stack_item2addr(item));
    } else if (item->is_constant()) {
      if (tag == objectTag) {
        __ movl(r, item2jobject(item));
      } else {
        jint const_val = item2int(item);
        if (!is_cc_locked() && const_val == 0) {
          __ xorl(r, r);
        } else {
          __ movl(r, item2int(item));
        }
      }
    } else {
      ShouldNotReachHere();
    }
  }
}


void CodeEmitter::item_2_local(int local_index, Item* item) {
  ValueTag tag = item->type()->tag();
  bool is_double_word = tag == doubleTag || tag == longTag;
  if (item->type()->is_double()) {
    check_double_address(local_index);
  }
  Address dst_addr = frame()->local_address(local_index, is_double_word);
  if (item->is_register()) {
    if (tag == floatTag || tag == doubleTag) {
      fpu_on_tos(item->get_register());
      if (tag == floatTag) __ fstp_s (dst_addr);
      else                 __ fstp_d (dst_addr);
      fpu_pop(item->get_register());
    } else if(tag == longTag) {
      Address dst_addr_LO = frame()->local_address(local_index, true, false);
      Address dst_addr_HI = frame()->local_address(local_index, true, true);
      __ movl (dst_addr_LO, item2regLO(item));
      __ movl (dst_addr_HI, item2regHI(item));
    } else {
      __ movl (dst_addr, item2reg(item));
    }
  } else if (item->is_constant()) {
    if (is_double_word) {    
      Address dst_addr_LO = frame()->local_address(local_index, true, false);
      Address dst_addr_HI = frame()->local_address(local_index, true, true);
      __ movl(dst_addr_LO, item2intLO(item));
      __ movl(dst_addr_HI, item2intHI(item));
    } else if(tag == objectTag)   {
      __ movl(dst_addr, item2jobject(item));
    } else {
      __ movl(dst_addr, item2int(item));
    }
  } else {
    // cannot handle mem2mem moves
    ShouldNotReachHere();
  }
}


// currently only one variant is of interest
void CodeEmitter::pop_item(Item* item) {
  ValueTag tag = item->type()->tag();
  if (item->is_register()) {
    if (tag ==  floatTag || tag == doubleTag) {
      Unimplemented();
    } else if (tag == longTag) {
      Unimplemented();
    } else {
      pop(item2reg(item));
    }
  } else {
    Unimplemented();
  }
}


void CodeEmitter::push_item(Item* item) {
  ValueTag tag = item->type()->tag();
  if (item->is_register()) {
    if (tag == floatTag || tag == doubleTag) {
      fpu_on_tos(item->get_register());
      if (tag == floatTag) { push_reg(eax); __ fstp_s (Address(esp)); }
      else                 { push_reg(eax); push_reg(eax); __ fstp_d (Address(esp)); }
      fpu_pop(item->get_register());
    } else if (tag == longTag) {
      push_reg(item2regHI(item));
      push_reg(item2regLO(item));
    } else {
      push_reg(item2reg(item));
    }
  } else if (item->is_stack()) {
    if (tag == doubleTag || tag == longTag) {
      push_addr(stack_item2addrHI(item));
      push_addr(stack_item2addrLO(item));
    } else {
      push_addr(stack_item2addr(item));
    }
  } else if (item->is_constant()) {
    if (tag == longTag || tag == doubleTag) {
      push_int(item2intHI(item));
      push_int(item2intLO(item));
    } else if (tag == objectTag)  {
      push_oop(item2jobject(item));
    } else {
      push_int(item2int(item));
    }
  }
}


void CodeEmitter::push_item_with_rinfo(Item* item, RInfo reg) {
  assert(!item->is_register(), "must be constant or stack");
  if (item->is_stack() && reg.is_word()) {
    __ movl (reg.as_register(), stack_item2addr(item));
    push_reg(reg.as_register());
  } else {
    assert(item->is_stack(), "wrong item form");
    __ movl (reg.as_register_hi(), stack_item2addrHI(item));
    __ movl (reg.as_register_lo(), stack_item2addrLO(item));
    push_reg(reg.as_register_hi());
    push_reg(reg.as_register_lo());
  }
}


void CodeEmitter::push_jint(jint val) {
  push_int(val);
}


void CodeEmitter::move_spill (int to_spill_ix, int from_spill_ix, ValueType* type, RInfo tmp) {
  bool is_double_word = type->is_double_word();
  Address from_addr = frame()->spill_address(from_spill_ix, is_double_word);
  Address   to_addr = frame()->spill_address(to_spill_ix,   is_double_word);
  if (tmp.is_float_kind()) {
    if (is_double_word) {
      __ fld_s  (from_addr);
      __ fstp_s (to_addr);
    } else {
      __ fld_d  (from_addr);
      __ fstp_d (to_addr);
    }
  } else if (tmp.is_word()) {
    __ movl (tmp.as_register(), from_addr);
    __ movl (to_addr, tmp.as_register());
  } else if (tmp.is_long()) {
    Address from_addrLO = frame()->spill_address(from_spill_ix, is_double_word, false);
    Address from_addrHI = frame()->spill_address(from_spill_ix, is_double_word, true);
    Address   to_addrLO = frame()->spill_address(to_spill_ix, is_double_word, false);
    Address   to_addrHI = frame()->spill_address(to_spill_ix, is_double_word, true);
    __ movl (tmp.as_register_lo(), from_addrLO);
    __ movl (tmp.as_register_hi(), from_addrHI);
    __ movl (to_addrLO, tmp.as_register_lo());
    __ movl (to_addrHI, tmp.as_register_hi());
  }
}


// Adds nops if the patched instruction is too small for a call instruction.
// Sets field offset so that a high or low word move can be recognized (offset is 4 words wide)
// Creates the patching stub
void CodeEmitter::patching_epilog(PatchingDesc* patch, CodeEmitInfo* info, PatchingStub::PatchID id ) {
  // we must have enough patching space so that call can be inserted
  add_call_info(patch->pc_offset() + NativeCall::instruction_size, info);

  while ((int)__ pc() - patch->pc_start() < NativeCall::instruction_size) {
    __ nop(); 
  }
  if (id == PatchingStub::access_field_id) {
    NativeMovRegMemPatching* n_move = nativeMovRegMemPatching_at((address)patch->pc_start());
    n_move->set_offset(patch->field_offset());
  }
  int instr_size = (int) __ pc() - patch->pc_start();
  PatchingStub* stub = new PatchingStub(patch->pc_start(), noreg, noreg, instr_size, info, id, esp_offset());
  _slow_case_stubs->append(stub);
}


// handle long, float, double, oop and int
void CodeEmitter::field_load(RInfo dst, BasicType dst_type, Item* object, int f_offset, bool needs_patching, bool is_loaded, CodeEmitInfo* info) {
  assert(object->type()->is_object(), "not an object");

  PatchingDesc patch;
  if (needs_patching) {
#if 0
    // There is a possibility to use a short move in some cases by generating a
    // "short" patch when the offset is known and is small.  The only time that this
    // situation could arise, however, is when the class has been loaded but not
    // initialized.  This is generally rare unless there is some type of eager loading
    // going on.  In most cases, the class is not loaded until it is needed, and it is
    // therefore initialized quickly thereafter.
    //
    // If you want to try this, delete these comments (those above and including the
    // dashed line.  Then delete the #if 0 and the #endif and also the assignment
    // "f_offset = MAX_OFFSET" that was AFTER the #endif that you just deleted.
    // There is likely other code that will need to be modified to support this, especially
    // in patch_code().
    //
    // --------------------------------------------------------------------------------------
    //
    // If the class is loaded then the value in f_offset can be trusted and should be used.
    // Otherwise, we must assume the worst case - that it will be a large offset.
    // The only exception is that it is allowed to be -1 if we are running with CompileTheWorld.
    // However, even in that case, if the class is not loaded then offset will get MAX_OFFSET.
    // But that is harmless.
    if (!is_loaded) {
      f_offset = MAX_OFFSET;
    }
#endif
    f_offset = MAX_OFFSET;
    set_patching_desc(&patch);
  }
  assert(f_offset >= 0 || (f_offset == -1 && CompileTheWorld), "field offset cannot be negative");

  add_debug_info_here(info);
  Address src_addr = item2addr_with_offset(object, f_offset);
  if (dst_type == T_FLOAT) {
    __ fld_s(src_addr);
    fpu_push(dst);
  } else if (dst_type == T_DOUBLE) {
    __ fld_d(src_addr);
    fpu_push(dst);
  } else if (dst_type == T_LONG) {
    Register rlo = dst.as_register_lo();
    Register rhi = dst.as_register_hi();
    assert(rlo != rhi, "error in regalloc");
    if (item2reg(object) == rlo) {
      // do not overwrite object register too early
      __ movl(rhi, item2addr_with_offsetHI(object, f_offset));
      if (needs_patching) {
        patch.set_hi_word(); // patching hi word
        patching_epilog(&patch, info, PatchingStub::access_field_id);
        set_patching_desc(&patch);
        patch.set_lo_word(); // patching lo word
      }
      __ movl(rlo, item2addr_with_offsetLO(object, f_offset));
    } else {
      __ movl(rlo, item2addr_with_offsetLO(object, f_offset));
      if (needs_patching) {
        patch.set_lo_word(); // patching lo word
        patching_epilog(&patch, info, PatchingStub::access_field_id);
        set_patching_desc(&patch);
        patch.set_hi_word(); // patching hi word
      }
      __ movl(rhi, item2addr_with_offsetHI(object, f_offset));
    }
  } else {
    __ movl(dst.as_register(), src_addr);
  }

  if (needs_patching) patching_epilog(&patch, info, PatchingStub::access_field_id);
}


// handle long, float, double, oop and int
// we kill the register of "object"
void CodeEmitter::field_store(BasicType value_type, Item* object, int f_offset, Item* value, bool needs_patching, bool is_loaded, CodeEmitInfo* info, RInfo tmp) {
  assert(object->type()->is_object() && object->is_register(), "wrong object type");
  bool is_oop = value_type == T_OBJECT || value_type == T_ARRAY;

  PatchingDesc patch;
  if (needs_patching) {
#if 0
    // There is a possibility to use a short move in some cases by generating a
    // "short" patch when the offset is known and is small.  The only time that this
    // situation could arise, however, is when the class has been loaded but not
    // initialized.  This is generally rare unless there is some type of eager loading
    // going on.  In most cases, the class is not loaded until it is needed, and it is
    // therefore initialized quickly thereafter.
    //
    // If you want to try this, delete these comments (those above and including the
    // dashed line.  Then delete the #if 0 and the #endif and also the assignment
    // "f_offset = MAX_OFFSET" that was AFTER the #endif that you just deleted.
    // There is likely other code that will need to be modified to support this, especially
    // in patch_code().
    //
    // --------------------------------------------------------------------------------------
    //
    // If the class is loaded then the value in f_offset can be trusted and should be used.
    // Otherwise, we must assume the worst case - that it will be a large offset.
    // The only exception is that it is allowed to be -1 if we are running with CompileTheWorld.
    // However, even in that case, if the class is not loaded then offset will get MAX_OFFSET.
    // But that is harmless.
    if (!is_loaded) {
      f_offset = MAX_OFFSET;
    }
#endif
    f_offset = MAX_OFFSET;
    set_patching_desc(&patch);
  }
  assert(f_offset >= 0 || (f_offset == -1 && CompileTheWorld), "field offset cannot be negative");

  add_debug_info_here(info); // this is for implicit exception
  Address dst_addr = item2addr_with_offset(object, f_offset);
  if (value->is_constant()) {
    assert(!needs_patching, "cannot handle code patching with inlined constants");
    if (value_type == T_DOUBLE || value_type == T_LONG) {
      Address dst_addrLO = item2addr_with_offsetLO(object, f_offset);
      Address dst_addrHI = item2addr_with_offsetHI(object, f_offset);
      __ movl (dst_addrLO, item2intLO(value));
      __ movl (dst_addrHI, item2intHI(value));
    } else if (value_type == T_OBJECT || value_type == T_ARRAY) {
      __ movl (dst_addr, item2jobject(value));
    } else {
      __ movl (dst_addr, item2int(value));
    }
  } else if (value->is_stack()) {
    assert(!needs_patching, "cannot handle code patching with value on stack");
    assert(value_type == T_FLOAT || value_type == T_DOUBLE, "only for floats and doubles");
    if (value_type == T_FLOAT) {
      __ movl (tmp.as_register(), stack_item2addr(value));
      __ movl (dst_addr, tmp.as_register());
    } else {
      Address dst_addrLO = item2addr_with_offsetLO(object, f_offset);
      Address dst_addrHI = item2addr_with_offsetHI(object, f_offset);
      __ movl (tmp.as_register_lo(), stack_item2addrLO(value));
      __ movl (tmp.as_register_hi(), stack_item2addrHI(value));
      __ movl (dst_addrLO, tmp.as_register_lo());
      __ movl (dst_addrHI, tmp.as_register_hi());    
    }
  } else {
    // if patching, only this else clause is relevant!
    assert(value->is_register(), "wrong type");
    int pc_start = (int)__ pc();
    if (value_type == T_FLOAT || value_type == T_DOUBLE) {
      fpu_on_tos(value->get_register());
      if (value_type == T_FLOAT)  __ fstp_s(dst_addr);
      else                        __ fstp_d(dst_addr);
      fpu_pop(value->get_register());
    } else if (value_type == T_LONG)   {
      Address dst_addrLO = item2addr_with_offsetLO(object, f_offset);
      Address dst_addrHI = item2addr_with_offsetHI(object, f_offset);
      __ movl (dst_addrLO, item2regLO(value));
      if (needs_patching) {
        patch.set_lo_word(); // patching lo word move
        patching_epilog(&patch, info, PatchingStub::access_field_id);
        set_patching_desc(&patch);
        patch.set_hi_word();
      }
      __ movl (dst_addrHI, item2regHI(value));
    } else {
      __ movl (dst_addr, item2reg(value));
    }
    if (needs_patching) patching_epilog(&patch, info, PatchingStub::access_field_id);
  }
  if (is_oop) {
    assert(object->destroys_register(), "check");
    if (value->is_constant() && item2jobject(value) == NULL) {
      // no store barrier for NULL stores
    } else {
      __ store_check(object->get_register().as_register());
    }
  }
}


// handles byte, char, short, int, long, float, double, oop
void CodeEmitter::indexed_load(RInfo dst, BasicType dst_type, Item* array, Item* index, CodeEmitInfo* info) {
  Address src_addr = item2array_addr(array, index, dst_type);
  int pc_offset = __ offset();

  if (dst_type == T_INT || dst_type == T_OBJECT || dst_type == T_ARRAY) {
    Register dst_reg = dst.as_register();
    __ movl (dst_reg, src_addr);
  } else if (dst_type == T_BYTE) {
    Register dst_reg = dst.as_register();
    if (CodeForP6) {
      pc_offset = __ load_signed_byte(dst_reg, src_addr);
    } else {
      // Is this code necessary??
      if (dst_reg.hasByteRegister()) {
        pc_offset = __ load_signed_byte(dst_reg, src_addr);
      } else {
        // do an explicit NULL check, and then push eax and use eax
        add_debug_info_here(info);
        if (GenerateCompilerNullChecks)__ null_check(item2reg(array));
        Register array_reg = item2reg(array);
        Register index_reg = index->is_constant() ? noreg : item2reg(index);
        Register tmp = noreg;
        for (int i = 0; i < Register::nofRegisters; i++) {
          if (i != array_reg.number() && (index_reg == noreg || i != index_reg.number())) {
            tmp = Register(i, 'c');
            if (tmp.hasByteRegister()) break;
          }
        }
        assert(tmp.hasByteRegister(), "could not find a free byte register");
        push_reg (tmp);
        pc_offset = __ load_signed_byte(tmp, src_addr);
        move_reg(dst_reg, tmp);
        pop (tmp);
      }
    }
  } else if (dst_type == T_CHAR) {
    Register dst_reg = dst.as_register();
    pc_offset = __ load_unsigned_word(dst_reg, src_addr);
  } else if (dst_type == T_SHORT) {
    Register dst_reg = dst.as_register();
    pc_offset = __ load_signed_word(dst_reg, src_addr);
  } else if (dst_type == T_LONG) {
    Address src_addrLO = item2array_addrLO(array, index, dst_type);
    Address src_addrHI = item2array_addrHI(array, index, dst_type);
    Register lo_reg = dst.as_register_lo();
    Register hi_reg = dst.as_register_hi();
    Register arr_reg = item2reg(array);
    Register index_reg = index->is_constant() ? noreg : item2reg(index);
    if (hi_reg == arr_reg || hi_reg == index_reg) {
      // load first into lo_reg, and then destroy hi_reg
      assert(lo_reg != arr_reg && lo_reg != index_reg, "cannot emit correct code");
      __ movl(lo_reg, src_addrLO);
      __ movl(hi_reg, src_addrHI);
    } else {
      // destroy hi_reg first
      __ movl(hi_reg, src_addrHI);
      __ movl(lo_reg, src_addrLO);
    }
  } else if (dst_type == T_FLOAT) {
    __ fld_s(src_addr);
    fpu_push(dst);
  } else if (dst_type == T_DOUBLE) {
    __ fld_d(src_addr);
    fpu_push(dst);
  } else {
    ShouldNotReachHere();
  }
  add_debug_info(pc_offset, info);
}


// handles byte, char, short, int, long, float, double, oop
void CodeEmitter::indexed_store(BasicType dst_type, Item* array, Item* index, Item* value, RInfo tmp, CodeEmitInfo* info) {
  Address dst_addr = item2array_addr(array, index, dst_type);
  int pc_offset = __ offset();
  add_debug_info_here(info);
  if (value->is_constant()) {
    if (value->type()->is_double_word()) {
      Address dst_addrLO = item2array_addrLO(array, index, dst_type);
      Address dst_addrHI = item2array_addrHI(array, index, dst_type);
      __ movl(dst_addrLO, item2intLO(value));
      __ movl(dst_addrHI, item2intHI(value));
    } else if (dst_type == T_BYTE) {
      __ movb (dst_addr, item2int(value) & 0xFF);
    } else if (dst_type == T_INT || dst_type == T_FLOAT) {
      __ movl (dst_addr, item2int(value));
    } else if (dst_type == T_OBJECT || dst_type == T_ARRAY) {
      __ movl (dst_addr, item2jobject(value));
    } else {
      // missing movw with address in assembler
      Unimplemented();
    }
  } else {
    assert(value->is_register(), "can handle only constant and register values");
    if (dst_type == T_BYTE) {
      Register vreg = item2reg(value);
      __ movb (dst_addr, vreg);
    } else if (dst_type == T_CHAR) {
      Register vreg = item2reg(value);
      __ movw (dst_addr, vreg);
    } else if (dst_type == T_SHORT) {
      Register vreg = item2reg(value);
      __ sign_extend_short(vreg);
      __ movw (dst_addr, vreg);
    } else if (dst_type == T_INT || dst_type == T_OBJECT || dst_type == T_ARRAY) {
      Register vreg = item2reg(value);
      __ movl (dst_addr, vreg);
    } else if (dst_type == T_LONG) {
      Address dst_addrLO = item2array_addrLO(array, index, dst_type);
      Address dst_addrHI = item2array_addrHI(array, index, dst_type);

      Register lo_reg = item2regLO(value);
      Register hi_reg = item2regHI(value);

      __ movl(dst_addrLO, lo_reg);
      __ movl(dst_addrHI, hi_reg);

    } else if (dst_type == T_FLOAT || dst_type == T_DOUBLE) {
      fpu_on_tos(value->get_register());
      if (dst_type == T_FLOAT) __ fstp_s(dst_addr);
      else                     __ fstp_d(dst_addr);
      fpu_pop(value->get_register());
    } else {
      Unimplemented();
    }
  }
  if (dst_type == T_ARRAY || dst_type == T_OBJECT) {
    assert(array->destroys_register(), "check");
    if (value->is_constant() && item2jobject(value) == NULL)  {
      // no store barrier for NULL stores
    } else {
      Register reg = item2reg(array);
      Address adr = item2array_addr(array, index, dst_type);
      __ leal (reg,adr);
      __ store_check(reg);
    }
  }
}


void CodeEmitter::monitor_enter(Item* object, RInfo lock, RInfo hdr, RInfo scratch, int monitor_no, CodeEmitInfo* info) {
  monitorenter(item2reg(object), lock.as_register(), hdr.as_register(), monitor_no, info);
}


void CodeEmitter::monitor_exit(Item* object, RInfo lock, RInfo hdr, int monitor_no)  {
  monitorexit(item2reg(object), lock.as_register(), hdr.as_register(), monitor_no, noreg);
}


void CodeEmitter::negate(RInfo dst, Item* value) {
  assert(value->is_register(), "can only handle registers");
  assert(value->destroys_register(), "check");
  ValueTag tag = value->type()->tag();
  if (tag == intTag) {
    __ negl(item2reg(value));
    move_reg(dst.as_register(), item2reg(value));
  } else if (tag == floatTag || tag == doubleTag) {
    fpu_on_tos(value->get_register());
    fpu_pop(value->get_register());
    __ fchs();
    fpu_push(dst);
  } else if (tag == longTag) {
    Register lo = item2regLO(value);
    Register hi = item2regHI(value);
    __ lneg(hi, lo);
    if (dst.as_register_lo() == hi) {
      assert(dst.as_register_hi() != lo, "destroying register");
      move_reg (dst.as_register_hi(), hi);
      move_reg (dst.as_register_lo(), lo);
    } else {
      move_reg (dst.as_register_lo(), lo);
      move_reg (dst.as_register_hi(), hi);
    }
  } else {
    ShouldNotReachHere();
  }
}


void CodeEmitter::math_intrinsic(ciMethod::IntrinsicId id, RInfo dst, Item* value, RInfo unused) {
  assert(value->is_register(), "can only handle registers");
  assert(value->destroys_register(), "check");
  ValueTag tag = value->type()->tag();
  // Note : stack banging  happens in the MathTrigStub

  // Step 1: prepare the slow case stub if needed
  MathStub* trig_stub = NULL;
  if (id == methodOopDesc::_dsin || id == methodOopDesc::_dcos) {
    assert(tag == doubleTag, "wrong type");
    address runtime_entry = NULL;
    switch (id) {
      case methodOopDesc::_dsin: runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dsin); break;
      case methodOopDesc::_dcos: runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dcos); break;
      default                  : ShouldNotReachHere();
    }
    trig_stub = new MathStub(runtime_entry);
    _slow_case_stubs->append(trig_stub);
  } 

  // Step 2: emit code using the generated stub
  if (tag == doubleTag) {
    fpu_on_tos(value->get_register());
    fpu_pop(value->get_register());
    switch(id) {
      case methodOopDesc::_dsqrt: __ fsqrt();                                     break;
      case methodOopDesc::_dsin : __ fsin (); __ jC2(noreg, *trig_stub->entry()); break;
      case methodOopDesc::_dcos : __ fcos (); __ jC2(noreg, *trig_stub->entry()); break;
      default                   : ShouldNotReachHere();
    }

    if (trig_stub != NULL) {
      __ bind(*trig_stub->continuation());
    }
    fpu_push(dst);
  } else {
    Unimplemented();
  }
}


// for: _iadd, _imul, _isub, 
void CodeEmitter::arithmetic_op_int(Bytecodes::Code code, Item* result, Item* left, Item* right, RInfo tmp) {
  assert(left->is_register(), "wrong items state");
  assert(left->destroys_register(), "check");
  Register lreg = item2reg(left);
  if (right->is_register()) {
    Register rreg = item2reg(right);
    switch (code) {
      case Bytecodes::_iadd:  __ addl(lreg, rreg); break;
      case Bytecodes::_imul:  __ imull(lreg, rreg); break;
      case Bytecodes::_isub:  __ subl(lreg, rreg); break;
      default: ShouldNotReachHere();
    }
    assert(result->is_register(), "cannot handle non-register results");
  } else if (right->is_constant()) {
    int c = item2int(right);
    switch (code) {
      case Bytecodes::_iadd:  
        { 
          switch (c) {
            case  1: __ incl(lreg);    break;
            case -1: __ decl(lreg);    break;
            default: __ addl(lreg, c); break;
          }
        }
        break;
      case Bytecodes::_imul: 
        {
          if (is_power_of_2(c)) {
            __ shll (lreg, log2(c));
          } else if (is_power_of_2(c + 1)) {
            Register treg = tmp.as_register();
            __ movl (treg, lreg);
            __ shll (lreg, log2(c + 1));
            __ subl (lreg, treg);
          } else if (is_power_of_2(c - 1)) {
            Register treg = tmp.as_register();
            __ movl (treg, lreg);
            __ shll (lreg, log2(c - 1));
            __ addl (lreg, treg);
          } else {
            Unimplemented();
          }
        }
        break;
      case Bytecodes::_isub:  
        {
          switch (c) {
            case  1: __ decl(lreg);    break;
            case -1: __ incl(lreg);    break;
            default: __ subl(lreg, c); break;
          }
        }
        break;
      default: ShouldNotReachHere();
    }
  } else if (right->is_stack()) {
    Address raddr = stack_item2addr(right);
    switch (code) {
      case Bytecodes::_iadd: __ addl(lreg, raddr); break;
      case Bytecodes::_imul: Unimplemented();      break;
      case Bytecodes::_isub: __ subl(lreg, raddr); break;
      default              : ShouldNotReachHere();
    }
  } else {
    Unimplemented();
  }
  move_reg(item2reg(result), lreg);
}


//      _ladd,      , _lsub, 
void CodeEmitter::arithmetic_op_long(Bytecodes::Code code, Item* result, Item* left, Item* right) {
  assert(left->is_register() && right->is_register(), "wrong item state");
  assert(left->destroys_register(), "check");
  Register op1_lo = item2regLO(left);
  Register op1_hi = item2regHI(left);
  Register op2_lo = item2regLO(right);
  Register op2_hi = item2regHI(right);

  if (code == Bytecodes::_ladd) {
    assert(op2_hi != op1_lo, "overwriting registers");
    __ addl(op1_lo, op2_lo); 
    __ adcl(op1_hi, op2_hi); 
  } else {
    assert(op2_hi != op1_lo, "overwriting registers");
    __ subl(op1_lo, op2_lo); 
    __ sbbl(op1_hi, op2_hi); 
  }
  if (item2regLO(result) == op1_hi) {
    assert(item2regHI(result) != op1_lo, "overwriting registers");
    move_reg (item2regHI(result), op1_hi);
    move_reg (item2regLO(result), op1_lo);
  } else {
    assert(item2regLO(result) != op1_hi, "overwriting registers");
    move_reg (item2regLO(result), op1_lo);
    move_reg (item2regHI(result), op1_hi);
  }
}


// parameters on stack; result in eax/edx, must preserve ecx/ebx
void CodeEmitter::long_mul() {
  int y_esp_offset = 2;
  int x_esp_offset = y_esp_offset + 2;
  push_reg(ecx);
  push_reg(ebx);
  __ lmul(x_esp_offset * wordSize, y_esp_offset * wordSize);
  pop(ebx);
  pop(ecx);
  __ addl (esp, 4 * wordSize);
  dec_stack_after_call(4);
}


//      _fadd, _fmul, _fsub, _fdiv, _frem
//      _dadd, _dmul, _dsub, _ddiv, _drem
void CodeEmitter::arithmetic_op_fpu(Bytecodes::Code code, Item* result, Item* left, Item* right) {
  assert(left->is_register() && result->is_register(), "wrong item state");
  assert(left->destroys_register() && right->destroys_register(), "check");
  if (right->is_register()) {
    bool must_be_ordered = true;
    switch (code) {
      case Bytecodes::_dmul: // fall through
      case Bytecodes::_fmul: // fall through
      case Bytecodes::_dadd: // fall through
      case Bytecodes::_fadd: must_be_ordered = false; break;
    }

    fpu_two_on_tos(right->get_register(), left->get_register(), must_be_ordered);
    int index = 1;
    switch (code) {
      case Bytecodes::_dadd: // fall through
      case Bytecodes::_fadd: __ faddp(index);            break;
      case Bytecodes::_dsub: // fall through
      case Bytecodes::_fsub: __ fsubp(index);            break;
      case Bytecodes::_dmul: // fall through
      case Bytecodes::_fmul: __ fmulp(index);            break;
      case Bytecodes::_ddiv: // fall through
      case Bytecodes::_fdiv: __ fdivp(index);            break;
      case Bytecodes::_drem: // fall through
      case Bytecodes::_frem: __ fxch(); __ fremr(noreg); break;
      default              : ShouldNotReachHere();
    }
    fpu_pop(right->get_register());
    fpu_pop(left->get_register());
  } else if (right->is_stack()) {
    fpu_on_tos(left->get_register());
    Address addr = stack_item2addr(right);
    switch (code) {
      case Bytecodes::_dadd: __ fadd_d(addr); break;
      case Bytecodes::_fadd: __ fadd_s(addr); break;
      case Bytecodes::_dsub: __ fsub_d(addr); break;
      case Bytecodes::_fsub: __ fsub_s(addr); break;
      case Bytecodes::_dmul: __ fmul_d(addr); break;
      case Bytecodes::_fmul: __ fmul_s(addr); break;
      case Bytecodes::_ddiv: __ fdiv_d(addr); break;
      case Bytecodes::_fdiv: __ fdiv_s(addr); break;
      default              : ShouldNotReachHere();
    }
    fpu_pop(left->get_register());
  } else {
    Unimplemented();
  }
  fpu_push(result->get_register());
}


// we assume that eax and edx can be overwritten
void CodeEmitter::arithmetic_idiv(Bytecodes::Code code, Item* result, Item* left, Item* right, RInfo scratch, CodeEmitInfo* info) {
  assert(left->is_register() && right->is_register() && result->is_register(), "wrong item state");
  assert(left->destroys_register() && right->destroys_register(), "check");
  assert(item2reg(left) == eax && item2reg(right) != edx, "wrong items");
  Register left_reg = item2reg(left);
  move_reg(eax, left_reg);
  int idivl_offset = __ corrected_idivl(item2reg(right));
  add_debug_info(idivl_offset, info);
  Register res_reg = item2reg(result);
  if (code == Bytecodes::_irem) {
    // result is in edx
    move_reg(res_reg, edx);
  } else {
    move_reg(res_reg, eax);
  }
}


// preserve registers across calls (NOTE: we may push less registers as some are callee saved anyway)
void CodeEmitter::arithmetic_call_prolog(ValueType* result) {
  push_reg(esi);
  push_reg(edi);
  push_reg(ecx);
  push_reg(ebx);
  if (result->is_float() || result->is_double()) {
    push_reg(edx);
    push_reg(eax);
  } else if (!result->is_long()) {
    push_reg(edx);
  }
}


// preserve registers across calls (NOTE: we may push less registers as some are callee saved anyway)
void CodeEmitter::arithmetic_call_epilog(ValueType* result) {
  if (result->is_float() || result->is_double()) {
    pop(eax);
    pop(edx);
  } else if (!result->is_long()) {
    pop(edx);
  }
  pop (ebx);
  pop (ecx);
  pop (edi);
  pop (esi);
}


void CodeEmitter::arithmetic_call_op(Bytecodes::Code code, RInfo unused) {
  address entry = NULL;
  bang_stack(); // make sure that we do not throw a StackOverflow in a leaf routine
  int size = -1; // parameter size in words
  switch (code) {
    case Bytecodes::_lrem: entry = CAST_FROM_FN_PTR(address, SharedRuntime::lrem); size = 4; break; // check if dividend is 0 is done elsewhere
    case Bytecodes::_ldiv: entry = CAST_FROM_FN_PTR(address, SharedRuntime::ldiv); size = 4; break; // check if dividend is 0 is done elsewhere
    case Bytecodes::_lmul: entry = CAST_FROM_FN_PTR(address, SharedRuntime::lmul); size = 4; break;
    case Bytecodes::_frem: entry = CAST_FROM_FN_PTR(address, SharedRuntime::frem); size = 2; break;
    case Bytecodes::_drem: entry = CAST_FROM_FN_PTR(address, SharedRuntime::drem); size = 4; break;
    default: ShouldNotReachHere();
  }

  __ call_VM_leaf(entry, size);
  dec_stack_after_call(size);
}


// _ishl, _lshl, _ishr, _lshr, _iushr, _lushr
void CodeEmitter::shift_op(Bytecodes::Code code, RInfo dst_reg, Item* value, Item* count, RInfo tmp) {
  if (value->type()->is_single_word()) {
    assert(value->destroys_register(), "check");
    Register vreg = item2reg(value);
    if (count->is_register()) {
      bool restore_shift_count = false;
      Register creg = item2reg(count);
      Register val  = vreg;
      Register count= creg;
      Register treg = noreg;
      if (val == SHIFT_count) {
        // tmp <- val   ecx <- count
        assert(tmp.is_valid(), "not allocated properly");
        treg = tmp.as_register();
        // make ecx free
        __ movl (treg, val);
        val = treg;
        // move coun tinto ecx
        assert(count != SHIFT_count, "error");
        __ movl (SHIFT_count, count);
        count = SHIFT_count;
      } else if (creg != SHIFT_count) {
        // tmp <- ecx   ecx <- count
        assert(tmp.is_valid(), "not allocated properly");
        restore_shift_count = true;
        treg = tmp.as_register();
        __ movl (treg, SHIFT_count);
        __ movl (SHIFT_count, creg);
        count = SHIFT_count;
      }
      assert(val != SHIFT_count && count == SHIFT_count, "ecx is count register");
      switch (code) {
        case Bytecodes::_ishl:  __ shll(val); break;
        case Bytecodes::_ishr:  __ sarl(val); break;
        case Bytecodes::_iushr: __ shrl(val); break;
        default: ShouldNotReachHere();
      }
      if (restore_shift_count) {
        // ecx <- tmp
        __ movl (SHIFT_count, treg);
      }
      move_reg(dst_reg.as_register(), val);
    } else if (count->is_constant()) {
      int cint = item2int(count) & 0x1F;
      switch (code) {
        case Bytecodes::_ishl:  __ shll(vreg, cint); break;
        case Bytecodes::_ishr:  __ sarl(vreg, cint); break;
        case Bytecodes::_iushr: __ shrl(vreg, cint); break;
        default: ShouldNotReachHere();
      }
      move_reg(dst_reg.as_register(), vreg);
    } else ShouldNotReachHere();
  } else {
    assert(count->is_register(), "count for long shift must be in register");
    assert(value->destroys_register(), "check");
    Register creg = item2reg(count);
    Register lo = item2regLO(value);
    Register hi = item2regHI(value);
    Register lo_val = lo;
    Register hi_val = hi;
    bool lo_swapped = false;
    bool hi_swapped = false;
    if (hi_val == SHIFT_count) {
      hi_swapped = true;
      swap_reg(hi, creg);
      hi_val = creg;
    } else if (lo_val == SHIFT_count) {
      lo_swapped = true;
      swap_reg(lo, creg);
      lo_val = creg;
    } else if (creg != SHIFT_count) {
      push_reg(SHIFT_count);
      __ movl (SHIFT_count, creg);
    }
    assert(lo_val != SHIFT_count && hi_val != SHIFT_count, "ecx is count register");
    switch (code) {
      case Bytecodes::_lshl:  __ lshl(hi_val, lo_val);        break;
      case Bytecodes::_lshr:  __ lshr(hi_val, lo_val, true);  break;
      case Bytecodes::_lushr: __ lshr(hi_val, lo_val, false); break;
      default: ShouldNotReachHere();
    }
    if (hi_swapped) {
      swap_reg(hi, creg);
    } else if (lo_swapped) {
      swap_reg(lo, creg);
    } else if (creg != SHIFT_count) {
      pop(SHIFT_count);
    }
  }
}


// _iand, _land, _ior, _lor, _ixor, _lxor
void CodeEmitter::logic_op(Bytecodes::Code code, RInfo dst_reg, Item* left, Item* right) {
  assert(left->destroys_register(), "check");
  if (left->type()->is_single_word()) {
    Register reg = item2reg(left);
    if (right->is_constant()) {
      int val = item2int(right);
      switch (code) {
        case Bytecodes::_iand: __ andl (reg, val); break;
        case Bytecodes::_ior : __ orl  (reg, val); break;
        case Bytecodes::_ixor: __ xorl (reg, val); break;
        default: ShouldNotReachHere();
      }
    } else {
      Register rright = item2reg(right);
      switch (code) {
        case Bytecodes::_iand: __ andl (reg, rright); break;
        case Bytecodes::_ior : __ orl  (reg, rright); break;
        case Bytecodes::_ixor: __ xorl (reg, rright); break;
        default: ShouldNotReachHere();
      }
    }
    move_reg(dst_reg.as_register(), reg);
  } else {
    Register l_lo = item2regLO(left);
    Register l_hi = item2regHI(left);
    if (right->is_constant()) {
      int r_lo = item2intLO(right);
      int r_hi = item2intHI(right);
      switch (code) {
        case Bytecodes::_land: 
          __ andl(l_lo, r_lo);
          __ andl(l_hi, r_hi);
          break;
        case Bytecodes::_lor:
          __ orl(l_lo, r_lo);
          __ orl(l_hi, r_hi);
          break;
        case Bytecodes::_lxor:
          __ xorl(l_lo, r_lo);
          __ xorl(l_hi, r_hi);
          break;
        default: ShouldNotReachHere();
      }
    } else {
      Register r_lo = item2regLO(right);
      Register r_hi = item2regHI(right);
      assert(l_lo != r_hi, "overwriting registers");
      switch (code) {
        case Bytecodes::_land: 
          __ andl(l_lo, r_lo);
          __ andl(l_hi, r_hi);
          break;
        case Bytecodes::_lor:
          __ orl(l_lo, r_lo);
          __ orl(l_hi, r_hi);
          break;
        case Bytecodes::_lxor:
          __ xorl(l_lo, r_lo);
          __ xorl(l_hi, r_hi);
          break;
        default: ShouldNotReachHere();
      }
    }
    if (dst_reg.as_register_lo() == l_hi) {
      assert(dst_reg.as_register_hi() != l_lo, "overwriting registers");
      move_reg(dst_reg.as_register_hi(), l_hi);
      move_reg(dst_reg.as_register_lo(), l_lo);
    } else {
      assert(dst_reg.as_register_lo() != l_hi, "overwriting registers");
      move_reg(dst_reg.as_register_lo(), l_lo);
      move_reg(dst_reg.as_register_hi(), l_hi);
    }
  }
}


// _lcmp, _fcmpl, _fcmpg, _dcmpl, _dcmpg
void CodeEmitter::compare_op(Bytecodes::Code code, RInfo dst_reg, Item* left, Item* right) {
  if (left->type()->is_float() || left->type()->is_double()) {
    assert(left->destroys_register() && right->destroys_register(), "check");
    fpu_two_on_tos(left->get_register(), right->get_register(), true);
    fpu_pop(left->get_register());
    fpu_pop(right->get_register());
    __ fcmp2int(dst_reg.as_register(), code == Bytecodes::_fcmpl || code == Bytecodes::_dcmpl);
  } else if (code == Bytecodes::_lcmp) {
    assert(left->destroys_register(), "check");
    __ lcmp2int(item2regHI(left), item2regLO(left), item2regHI(right), item2regLO(right));
    move_reg(dst_reg.as_register(), item2regHI(left));
  } else {
    ShouldNotReachHere();
  }
}


void CodeEmitter::call_convert_prolog(int nof_floats_to_save, ValueType* result) {
  // first the floats
  for (int i = 0; i < nof_floats_to_save; i++) {
    Unimplemented();
  }
  // second cpu
  arithmetic_call_prolog(result);
}


void CodeEmitter::call_convert_epilog(int nof_floats_to_restore, ValueType* result) {
  // first cpu
  arithmetic_call_epilog(result);
  // second floats
  for (int i = 0; i < nof_floats_to_restore; i++) {
    Unimplemented();
  }
}


// _l2f, _l2d, _f2l, _f2i, _d2l, _d2i
void CodeEmitter::call_convert_op(Bytecodes::Code code, RInfo unused) {
  address entry = NULL;
  int in_size = 0;
  bang_stack(); // make sure that we do not throw a StackOverflow in a leaf routine
  switch (code) {
    case Bytecodes::_l2f: entry = CAST_FROM_FN_PTR(address, SharedRuntime::l2f); in_size = 2; break;
    case Bytecodes::_l2d: entry = CAST_FROM_FN_PTR(address, SharedRuntime::l2d); in_size = 2; break;
    case Bytecodes::_f2l: entry = CAST_FROM_FN_PTR(address, SharedRuntime::f2l); in_size = 1; break;
    case Bytecodes::_f2i: entry = CAST_FROM_FN_PTR(address, SharedRuntime::f2i); in_size = 1; break;
    case Bytecodes::_d2l: entry = CAST_FROM_FN_PTR(address, SharedRuntime::d2l); in_size = 2; break;
    case Bytecodes::_d2i: entry = CAST_FROM_FN_PTR(address, SharedRuntime::d2i); in_size = 2; break;
    default: ShouldNotReachHere();
  }
  __ call_VM_leaf(entry, in_size);
  dec_stack_after_call(in_size);
}


// _i2l, _i2f, _i2d, _l2i, _f2d, _d2f
// _i2b, _i2c, _i2s
void CodeEmitter::convert_op(Bytecodes::Code code, RInfo dst, Item* value) {
  if (value->type()->is_float() || value->type()->is_double()) {
    assert(value->destroys_register(), "check");
    fpu_on_tos(value->get_register());
    fpu_pop(value->get_register());
  }
  if (dst.is_float_kind()) {
    fpu_push(dst);
  }
  switch (code) {
    case Bytecodes::_i2l: 
      move_reg(dst.as_register_lo(), item2reg(value));
      move_reg(dst.as_register_hi(), item2reg(value));
      __ sarl (dst.as_register_hi(), 31);
      break;
    case Bytecodes::_i2d:
    case Bytecodes::_i2f:
      if (value->is_register()) {
        push_reg  (item2reg(value));
        __ fild_s (Address(esp));
        dec_stack(1);
      } else if (value->is_stack()) {
        Address src = stack_item2addr(value);
        __ fild_s (src);
      } else {
        Unimplemented();
      }
      break;
    case Bytecodes::_l2i:
      move_reg (dst.as_register(), item2regLO(value));
      break;
    case Bytecodes::_d2f:
    case Bytecodes::_f2d:
      // do nothing (must round later)
      assert (value->is_register(), "wrong value state");
      // d2f is rounded through spilling
      break;
    case Bytecodes::_i2b:
      move_reg (dst.as_register(), item2reg(value));
      __ sign_extend_byte(dst.as_register());
      break;
    case Bytecodes::_i2c:
      move_reg (dst.as_register(), item2reg(value));
      __ andl (dst.as_register(), 0xFFFF);
      break;
    case Bytecodes::_i2s:
      move_reg (dst.as_register(), item2reg(value));
      __ sign_extend_short(dst.as_register());
      break;
    default: ShouldNotReachHere();
  }
}



// We use eax to store result; input is an FPU register
void CodeEmitter::fast_call_convert_op(Bytecodes::Code code, Item* val, RInfo dst, bool is_32bit) {
  address entry = NULL;
  // Note : stack banging  happens in the ConvertStub
  switch (code) {
    // The ConvertStub, converts the double to a float
    case Bytecodes::_f2i: entry = CAST_FROM_FN_PTR(address, SharedRuntime::f2i);  break;
    case Bytecodes::_d2i: entry = CAST_FROM_FN_PTR(address, SharedRuntime::f2i);  break;
    default: ShouldNotReachHere();
  }
  fpu_on_tos(val->get_register());

  ConvertStub* convert = new ConvertStub(entry, dst.as_register());
  _slow_case_stubs->append(convert);
  Label done;
  __ fldcw(Address((int)StubRoutines::addr_fpu_cntrl_wrd_trunc(), relocInfo::none));
  __ pushl(eax); // make room on stack
  __ fist_s(Address(esp));
  __ popl(eax);
  __ cmpl(eax, 0x80000000);
  __ jcc(Assembler::equal, *convert->entry());
  __ fpop();
  __ bind(*convert->continuation());
  if (is_32bit) {
    set_32bit_fpu_precision();
  } else {
    restore_fpu_precision();
  }
  fpu_pop(val->get_register());
}


void CodeEmitter::call_op(Bytecodes::Code code, const GrowableArray<BasicType>* sig_types, int size_of_arguments, CodeEmitInfo* info, bool optimized, bool needs_null_check, Item* receiver) {
  assert(receiver == NULL, "not implemented");
  nops_for_zombie();

  switch (code) {
    case Bytecodes::_invokestatic:
      { StaticCallStub* stub = new StaticCallStub(__ pc(), NULL);
        _call_stubs->append(stub);
        __ call(Runtime1::entry_for(Runtime1::resolve_invokestatic_id), relocInfo::static_call_type);
      }
      break;
    case Bytecodes::_invokespecial:
    case Bytecodes::_invokevirtual:
    case Bytecodes::_invokeinterface:
      // for final target we stil produce an inline cache, in order
      // to be able to call mixed mode
      if (needs_null_check && GenerateCompilerNullChecks) {
        add_debug_info_here(info);
        __ null_check(RECEIVER);
      }
      if (code == Bytecodes::_invokespecial || optimized) {
        StaticCallStub* stub = new StaticCallStub(__ pc(), NULL);
        _call_stubs->append(stub);
        __ call(Runtime1::entry_for(Runtime1::resolve_invoke_id), relocInfo::opt_virtual_call_type);
      } else {
        RelocationHolder rh = virtual_call_Relocation::spec(__ pc());
        __ movl(IC_Klass, (jobject)NULL);
        __ call(Runtime1::entry_for(Runtime1::resolve_invoke_id), rh);
      }
      break;
    default:
      ShouldNotReachHere();
      break;
  }
  add_call_info(__ offset(), info);
  // remove parameters if necessary
  if (UseC2CallingConventions) {
    const int max_poppable_size = 2;
    if (size_of_arguments <= max_poppable_size) {
      int i = size_of_arguments;
      while (i-- > 0) pop (ebx);
    } else {
      dec_stack(size_of_arguments);
    }
  } else {
    dec_stack_after_call(size_of_arguments);
  }
}


void CodeEmitter::new_instance(RInfo dst, ciInstanceKlass* klass, RInfo tmp1, RInfo tmp2, RInfo tmp3, RInfo klass_reg, CodeEmitInfo* info) {
  const Register obj = dst.as_register(); 
  assert(obj == eax, "must be eax");
  const Register t1 = tmp1.as_register(); // ebx;
  const Register t2 = tmp2.as_register(); // ecx;
  const Register kl = klass_reg.as_register();
  const Register i_size = tmp3.as_register(); // edi
  NewInstanceStub* slow_case = new NewInstanceStub(kl, info, esp_offset());
  _slow_case_stubs->append(slow_case);
  jobject_2_reg_with_patching(klass_reg, klass, info);
  // If klass is not loaded we do not know if the klass has finalizers:
  if (UseFastNewInstance && klass->is_loaded() && !klass->has_finalizer()) {
    // make sure klass is fully initialized
    if (!klass->is_initialized()) {
      __ cmpl(Address(kl, instanceKlass::init_state_offset_in_bytes() + sizeof(oopDesc)), instanceKlass::fully_initialized);
      __ jcc(Assembler::notEqual, *slow_case->entry());
    }
    if (!klass->is_loaded()) {
      // Note: we will never get here because we do not know if a klass has finalizers, until
      // the class is loaded
      ShouldNotReachHere();
    } else {
      // allocate space for instance
      assert(klass->size_helper() > 0, "size_helper should be >0 for instances");
      const int instance_size = align_object_size(klass->size_helper());
      __ allocate_object(obj, t1, t2, oopDesc::header_size(), instance_size , kl, *slow_case->entry());
    }
  } else {
    __ jmp(*slow_case->entry());
  }
  __ bind(*slow_case->continuation());
  // allocation done
  // result is in obj
  __ verify_oop(obj);
  move_reg(dst.as_register(), obj);
}


void CodeEmitter::new_type_array(RInfo dst, BasicType elem_type, Item* length, RInfo tmp1, RInfo tmp2, RInfo tmp3, RInfo tmp4, RInfo klass_r, CodeEmitInfo* info) {
  const Register obj = dst.as_register(); 
  assert(obj == eax, "must be eax");
  const Register len = tmp1.as_register(); //ebx;
  const Register t = tmp2.as_register(); // ecx;
  const Register k = klass_r.as_register(); // edx;
  move_reg(len, item2reg(length));
  NewTypeArrayStub* slow_case = new NewTypeArrayStub(elem_type, len, noreg, info, esp_offset());
  _slow_case_stubs->append(slow_case);
  ciTypeArrayKlass* klass = ciTypeArrayKlass::make(elem_type);
  assert (klass != NULL, "type array klasses exist alread");
  if (UseFastNewTypeArray && !UseSlowPath) {
    // allocate space for array
    __ movl(k, klass->encoding());
    __ allocate_array(obj, len, t, arrayOopDesc::header_size(elem_type), array_element_size(elem_type), k, *slow_case->entry());
  } else {
    __ jmp(*slow_case->entry());
  }
  __ bind(*slow_case->continuation());
  // allocation done
  // result is in obj
  __ verify_oop(obj);
  move_reg(dst.as_register(), obj);
}


void CodeEmitter::new_object_array(RInfo dst, ciKlass* elem_klass, Item* length, RInfo tmp1, RInfo tmp2, RInfo tmp3, RInfo tmp4, RInfo klass_reg, CodeEmitInfo* info) {
  const Register obj = dst.as_register(); 
  assert(obj == eax, "must be eax");
  const Register len = tmp1.as_register(); // ebx;
  const Register t = tmp2.as_register(); // ecx;
  const Register tmp_reg = tmp3.as_register(); // esi;
  move_reg(len, item2reg(length));
  NewObjectArrayStub* slow_case = new NewObjectArrayStub(tmp_reg, len, noreg, info, esp_offset());
  _slow_case_stubs->append(slow_case);
  jobject_2_reg_with_patching(tmp3, ciObjArrayKlass::make(elem_klass), info);
  if (UseFastNewObjectArray) {
    // allocate space for array
    assert(oopSize == 4, "change scale factor below");
    __ allocate_array(obj, len, t, arrayOopDesc::header_size(T_OBJECT), Address::times_4, tmp_reg, *slow_case->entry());
  } else {
    __ jmp(*slow_case->entry());
  }
  __ bind(*slow_case->continuation());
  // allocation done
  // result is in obj
  __ verify_oop(obj);
  move_reg(dst.as_register(), obj);
}


void CodeEmitter::new_multi_array(RInfo dst, ciKlass* klass, int rank, RInfo tmp, CodeEmitInfo* info) {
  Register dst_reg = dst.as_register();
  Register tmp_reg = tmp.as_register();
  assert(tmp_reg != dst_reg, "wrong registers"); 
  __ movl(tmp_reg, esp);
  // use dst register to hold class temporary
  jobject_2_reg_with_patching(dst, klass, info);
  push_reg(dst_reg);
  // push_oop((oop)klass);
  push_int(rank);
  push_reg(tmp_reg);
  nops_for_zombie();
  __ call(Runtime1::entry_for(Runtime1::new_multi_array_id), relocInfo::runtime_call_type);
  add_call_info(__ offset(), info);
  dec_stack_after_call(3);
  dec_stack(rank);
  move_reg(dst_reg, eax);
}


void CodeEmitter::throw_op(Item* exception, CodeEmitInfo* info) {
  move_reg(EXCEPT_oop, item2reg(exception));

  if (GenerateCompilerNullChecks) {
    add_debug_info_here(info);
    __ null_check(EXCEPT_oop);
  }

  // get current pc information
  __ nop(); // prevent ambiguities (overlapping  info)
  int pc_for_athrow  = (int)__ pc();
  int pc_for_athrow_offset = __ offset();
  __ nop(); // pc_for_athrow can not point to itself (relocInfo restriction)
  __ leal(edx, Address(pc_for_athrow, relocInfo::internal_word_type));
  add_call_info(pc_for_athrow_offset, info); // for exception handler
  __ pushl(edx); // pc passed on stack

  // exception pc is on stack
  // if we jump, we would escape the method, use a compiler safepoint
  // and mark exception oop as object
  if (UseCompilerSafepoints) {
    // account for return address
    _esp_offset++;
    RInfoCollection* oop_regs = new RInfoCollection();
    oop_regs->append(FrameMap::_eaxRInfo);
    add_debug_info_for_branch(info->_bci, oop_regs);
    _esp_offset--;
  }
  __ jmp(_throw_entry_label);
  // enough room for two byte trap
  __ nop();
}


// With an additional register we could cache the constant in register
// thus producing shorter code
void CodeEmitter::checkcast_op(RInfo dst_rinfo, Item* obj_item, ciKlass* k, RInfo k_RInfo, RInfo klass_RInfo, CodeEmitInfo* info) {
  assert(obj_item->destroys_register(), "check");
  Register obj = item2reg(obj_item);
  Register obj_klass = obj;
  Register dst = dst_rinfo.as_register();
  Register test_klass = klass_RInfo.as_register();
  assert(test_klass != obj && test_klass != dst, "wrong registers");

  Label done;
  assert(dst.number() != obj.number(), "must be different");
  jobject_2_reg_with_patching(klass_RInfo, k, info); 
  __ verify_oop(obj);
  __ movl  (dst, obj);
  __ testl (obj, obj);
  __ jcc   (Assembler::zero, done);
  __ movl  (obj_klass, Address(obj, oopDesc::klass_offset_in_bytes()));  // get object klass
  // Note: dst contains object to check against class in test_klass
  CheckCastStub* stub = new CheckCastStub(test_klass, info, dst, esp_offset());
  _slow_case_stubs->append(stub);
  if (UseSlowPath) {
    __ jmp (*stub->entry());
  } else {
    if (k->is_loaded() && k->is_instance_klass() && k->as_instance_klass()->is_final()) {
      // compare class equality
      __ cmpl  (obj_klass, test_klass);
    } else {
      __ cmpl  (test_klass, Address(obj_klass, Klass::is_a_cache_1_offset_in_bytes() + sizeof(oopDesc)));
      __ jcc   (Assembler::equal, done);
      __ cmpl  (test_klass, Address(obj_klass, Klass::is_a_cache_2_offset_in_bytes() + sizeof(oopDesc)));
    }
    __ jcc (Assembler::notEqual, *stub->entry()); // <--- jcc notEqual 
  }
  __ bind(*stub->continuation());
  __ bind  (done);
}


// With an additional register we could cache the constant in register
// thus producing shorter code
void CodeEmitter::instanceof_op(RInfo dst_reg, Item* obj_item, ciKlass* k, RInfo k_RInfo, RInfo klass_RInfo, CodeEmitInfo* info) {
  assert(obj_item->destroys_register(), "check");
  Register obj = item2reg(obj_item);
  Register result = dst_reg.as_register();
  Register test_klass = klass_RInfo.as_register();
  assert(result != obj, "overwriting  result register");
  assert(test_klass != result && test_klass != obj, "wrong registers");
  Label done;
  __ verify_oop(obj);
  __ xorl (result, result);
  __ testl(obj, obj);
  __ jcc  (Assembler::zero, done); // return 0 if obj is NULL
  __ movl (obj, Address(obj, oopDesc::klass_offset_in_bytes()));  // get object klass
  jobject_2_reg_with_patching(klass_RInfo, k, info); 
  InstanceOfStub* stub = new InstanceOfStub(obj, result, test_klass, info, esp_offset());
  _slow_case_stubs->append(stub);
  if (UseSlowPath) {
    __ jmp (*stub->entry());
  } else {
    if (k->is_loaded() && k->is_instance_klass() && k->as_instance_klass()->is_final()) {
      __ cmpl (obj, test_klass); // compare for equality
      __ jcc  (Assembler::notEqual, done);
      __ incl (result);
    } else {
      __ incl  (result);         // set result to 1
      __ cmpl  (test_klass, Address(obj, Klass::is_a_cache_1_offset_in_bytes() + sizeof(oopDesc)));
      __ jcc   (Assembler::equal, done);
      __ cmpl  (test_klass, Address(obj, Klass::is_a_cache_2_offset_in_bytes() + sizeof(oopDesc)));
      __ jcc   (Assembler::notEqual, *stub->entry());
    }
  }
  __ bind  (done);
  __ bind  (*stub->continuation());

}



// This has not yet been tested.
void CodeEmitter::instance_of_test_op(RInfo dst_reg, Item* obj_item, ciKlass* k, RInfo klass_ri, BlockBegin* is_instance, BlockBegin* not_is_instance, bool default_true, CodeEmitInfo* info) {
  Unimplemented();
  assert(obj_item->destroys_register(), "check");
  Register obj = item2reg(obj_item);
  Register result = dst_reg.as_register();
  Register test_klass = klass_ri.as_register();
  assert(test_klass != obj, "wrong registers");
  __ verify_oop(obj);
  if (k->is_loaded() && k->is_instance_klass() && k->as_instance_klass()->is_final()) {
    __ xorl (result, result);
    __ testl(obj, obj);
    __ jcc  (Assembler::zero, *(not_is_instance->label())); 
    __ movl (obj, Address(obj, oopDesc::klass_offset_in_bytes()));  // get object klass
    jobject_2_reg_with_patching(klass_ri, k, info); 
    __ cmpl (obj, test_klass); // compare for equality
    if (default_true) {
      __ jcc  (Assembler::notEqual, *(not_is_instance->label()));
    } else {
      __ jcc  (Assembler::equal, *(is_instance->label()));
    }
  } else {
    __ testl (obj, obj);
    __ jcc   (Assembler::zero, *(not_is_instance->label())); 
    __ movl  (obj, Address(obj, oopDesc::klass_offset_in_bytes()));  // get object klass
    jobject_2_reg_with_patching(klass_ri, k, info); 
    __ incl  (result);         // set result to 1
    __ cmpl  (test_klass, Address(obj, Klass::is_a_cache_1_offset_in_bytes() + sizeof(oopDesc)));
    __ jcc   (Assembler::equal, *(is_instance->label()));
    __ cmpl  (test_klass, Address(obj, Klass::is_a_cache_2_offset_in_bytes() + sizeof(oopDesc)));
    InstanceOfStub* stub = new InstanceOfStub(obj, result, test_klass, info, esp_offset());
    _slow_case_stubs->append(stub);
    __ jcc (Assembler::equal, *(is_instance->label()));
    __ jmp (*stub->entry());
    __ bind  (*stub->continuation());
    __ testl (result, result);
    if (default_true) {
      __ jcc (Assembler::zero, *(not_is_instance->label()));
    } else {
      __ jcc (Assembler::notZero, *(is_instance->label()));
    }
  }
}



void CodeEmitter::goto_op(BlockBegin* dst, int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
  __ jmp (*(dst->label()));
}


void CodeEmitter::jsr_op (BlockBegin* dst, int bci) {
  Label L1, L2;
  // needed for compiler safepoints
  if (UseCompilerSafepoints) {
    // make sure that the pc is not in safepoint codebuffer
    __ code()->relocate(__ pc(), relocInfo::safepoint_type);
    CodeEmitInfo info(bci, NULL, NULL);
    add_debug_info_here(&info);
  }
  __ call(L1, relocInfo::none); // push return value on stack
  __ jmp(L2); // continue by jumping over branch to jsr dest
  __ bind(L1);
  __ popl( eax ); // return address in eax

  __ jmp(*(dst->label())); // no safepoint here!!
  __ bind(L2);
}


void CodeEmitter::ret_op (int local_ix, RInfo cached_in_reg, int bci)  {
  if (cached_in_reg.is_illegal()) {
    __ movl (eax, frame()->local_address(local_ix, false));
  } else {
    __ movl(eax, cached_in_reg.as_register());
  }
  if (UseCompilerSafepoints) {
    add_debug_info_for_branch(bci, NULL);
  }
  __ jmp  (eax);
  if (UseCompilerSafepoints) {
    __ nop();
  }
}


If::Condition CodeEmitter::if_op(int phase, If::Condition cond, Item* x, Item* y, BlockBegin* t_dest, BlockBegin* f_dest, BlockBegin* u_dest,
                                 int bci, bool is_backward_branch, const RInfoCollection* oop_regs ) {
  ValueTag cmpTag = y->type()->tag();
  bool use_test_op = false;
  if (y->is_constant()) {
    if (cond == If::eql || cond == If::neq) {
      if (cmpTag == objectTag)  {
        use_test_op = item2jobject(y) == (jobject)NULL;
      } else if (cmpTag == intTag) {
        use_test_op = item2int(y) == 0;
      }
    }
  }
  if (phase == 1) {
    if (y->is_constant()) {
      Register xreg = item2reg(x);
      if (use_test_op) {
        __ testl (xreg, xreg);
      } else if (cmpTag == objectTag) {
        __ cmpl (xreg, item2jobject(y)); 
      } else if (cmpTag = intTag) { 
        __ cmpl (xreg, item2int(y));          
      } else {
        Unimplemented();
      }
    } else if (y->is_stack()) {
      Register xreg = item2reg(x);
      if (cmpTag == intTag || cmpTag == objectTag) {
        Address adr = stack_item2addr(y);
        __ cmpl (xreg, adr);
      } else {
        Unimplemented();
      }
    } else {
      if (cmpTag == objectTag || cmpTag == intTag) {
        Register xreg = item2reg(x);
        __ cmpl (xreg, item2reg(y));
      } else if (cmpTag == longTag) {
        Register xlo = item2regLO(x);
        Register xhi = item2regHI(x);
        Register ylo = item2regLO(y);
        Register yhi = item2regHI(y);
        assert(x->destroys_register(), "check");
        __ subl(xlo, ylo);
        __ sbbl(xhi, yhi);
        if (cond == If::eql || cond == If::neq) {
          __ orl (xhi, xlo);
        }
      } else if (cmpTag == floatTag || cmpTag == doubleTag) {
        assert(x->destroys_register() && y->destroys_register(), "check");
        if (must_swap_two_on_tos(x->get_register(), y->get_register())) {
          fpu_two_on_tos(y->get_register(), x->get_register(), true);
          fpu_pop(y->get_register());
          fpu_pop(x->get_register());
          cond = Instruction::mirror(cond);
        } else {
          fpu_two_on_tos(x->get_register(), y->get_register(), true);
          fpu_pop(x->get_register());
          fpu_pop(y->get_register());
        }
        __ fcmp(noreg);
        // CF (corresponds to C0) if x < y
        // PF (corresponds to C2) if unordered
        // ZF (corresponds to C3) if x = y
      } else {
        Unimplemented();
      }
    }
  } else if (phase == 2) {
    assert(bci >= 0, "bci must be passed in");
    Assembler::Condition acond;
    if (cmpTag == floatTag || cmpTag == doubleTag) {
      assert(u_dest != NULL, "must have unordered successor");
      __ jcc(Assembler::parity, *(u_dest->label()));
      switch (cond) {
        case If::eql: acond = Assembler::equal;        break;
        case If::neq: acond = Assembler::notEqual;     break;
        case If::lss: acond = Assembler::below;         break;
        case If::leq: acond = Assembler::belowEqual;    break;
        case If::geq: acond = Assembler::aboveEqual; break;
        case If::gtr: acond = Assembler::above;      break;
        default: ShouldNotReachHere();
      };
    } else {
      switch (cond) {
        case If::eql: acond = Assembler::equal;        break;
        case If::neq: acond = Assembler::notEqual;     break;
        case If::lss: acond = Assembler::less;         break;
        case If::leq: acond = Assembler::lessEqual;    break;
        case If::geq: acond = Assembler::greaterEqual; break;
        case If::gtr: acond = Assembler::greater;      break;
        default: ShouldNotReachHere();
      };
      assert (cmpTag != longTag || 
              acond == Assembler::equal || 
              acond == Assembler::notEqual ||
              acond == Assembler::less ||
              acond == Assembler::greaterEqual, "illegal condition for long compares");
    }
    if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
    __ jcc  (acond, *(t_dest->label()));
  } else {
    ShouldNotReachHere(); // only phases 1 and 2 exist
  }
  return cond;
}


void CodeEmitter::tableswitch_op(Item* tag, int match, BlockBegin* dest, int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  __ cmpl (item2reg(tag), match);
  if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
  __ jcc  (Assembler::equal, *(dest->label()));
}


void CodeEmitter::lookupswitch_op(Item* tag, int key, BlockBegin* dest, int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  __ cmpl (item2reg(tag), key);
  if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
  __ jcc  (Assembler::equal, *(dest->label()));
}


void CodeEmitter::lookupswitch_range_op(Item* tag, int low_key, int high_key, BlockBegin* dest, int bci, 
					bool is_backward_branch, const RInfoCollection* oop_regs) {
  assert(low_key <= high_key, "wrong range");
  if (low_key == high_key) {
    __ cmpl (item2reg(tag),low_key);
    if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
    __ jcc  (Assembler::equal, *(dest->label()));
  } else if (high_key - low_key == 1) {
    __ cmpl (item2reg(tag),low_key);
    if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
    __ jcc  (Assembler::equal, *(dest->label()));
    __ cmpl (item2reg(tag),high_key);
    if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
    __ jcc  (Assembler::equal, *(dest->label()));
  } else {
    Label L;
    __ cmpl (item2reg(tag), low_key);
    __ jcc  (Assembler::less, L);
    __ cmpl (item2reg(tag), high_key);
    if (is_backward_branch) add_debug_info_for_branch(bci, oop_regs);
    __ jcc  (Assembler::lessEqual,  *(dest->label()));
    __ bind (L);
  }
}


void CodeEmitter::ifop_phase1(Item* x, Item* y) {
  __ cmpl(item2reg(x), item2reg(y));
}



Assembler::Condition CodeEmitter::icond2acond(Instruction::Condition cond) const {
  switch (cond) {
    case If::eql: return Assembler::equal;
    case If::neq: return Assembler::notEqual;     
    case If::lss: return Assembler::less;         
    case If::leq: return Assembler::lessEqual;    
    case If::geq: return Assembler::greaterEqual; 
    case If::gtr: return Assembler::greater;      
  };
  ShouldNotReachHere();
  return Assembler::zero;
}


void CodeEmitter::ifop_phase2(RInfo dst, Item* tval, Item* fval, Instruction::Condition cond) {
  item_2_reg(dst, tval);
  Assembler::Condition acond = icond2acond(cond);
  Label L;
  __ jcc(acond, L);
  item_2_reg(dst, fval);
  __ bind(L);
}


// receiver may be used by the monitorexit; it is set if the receiver
// is cached and therefore can be referenced without loading it from frame
void CodeEmitter::return_op_prolog(int monitor_no, RInfo receiver) {
  assert(method()->is_synchronized(), "must be synchronized");
  // do not use eax/edx
  Register obj_reg  = ecx;
  Register lock_reg = esi;
  assert(monitor_no >= 0, "monitor slot must exist");
  if (method()->is_static()) {
    __ movl (obj_reg, method()->holder()->java_mirror()->encoding());
  } else {
    if (receiver.is_illegal()) {
      // receiver is not cached, must load it
      load_receiver(obj_reg);
    } else {
      // receiver is ready
      obj_reg = receiver.as_register();
    }
  }
  monitorexit(obj_reg, lock_reg, SYNC_header, monitor_no, noreg);
}


void CodeEmitter::jvmpi_method_exit() {
  assert(jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_EXIT), "wrong call");
  // preserve arguments
  bang_stack();
  __ push_CPU_state();
  __ movl (eax, method()->encoding());
  __ call(Runtime1::entry_for(Runtime1::jvmpi_method_exit), relocInfo::runtime_call_type);
  __ pop_CPU_state();
}


void CodeEmitter::jvmpi_method_entry(CodeEmitInfo* info) {
  assert (jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY) || jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY2), "wrong call");
  if (method()->is_static()) {
    __ movl(ecx, (jobject)NULL);
  } else {
    __ movl(ecx, RECEIVER);
  }
  __ movl (eax, method()->encoding());
  __ call(Runtime1::entry_for(Runtime1::jvmpi_method_entry), relocInfo::runtime_call_type);
  add_call_info_here(info);
  restore_cached_receiver();
}


void CodeEmitter::safepoint_return(int argSize) {
  if (compilation()->jvmpi_event_method_exit_enabled()) {
    jvmpi_method_exit();
  }
  address ret_addr = __ pc();
  __ ret (argSize);
  // for compatibility reasons with compiler 2, we must emit ret_start + 1
  // as pc address; safepoint handling is substracting one from that relocInfo (!!!)
  // we do not need oop maps, and pc/bci mapping here
  __ code()->relocate(ret_addr + 1, relocInfo::return_type);
  // add two nops so that return can be patched for compiler safepoint
  // Note: this may not be needed if arg_size > 0
  __ nop ();
  __ nop ();
}

void CodeEmitter::return_op(Item* result) {
  __ leave();
  // remove arguments if necessary
  int arg_size = (UseC2CallingConventions) ? 0 : method()->arg_size() * wordSize;
  assert (arg_size >= 0, "wrong result size");
  // Note: we do not need to round double result; float result has the right precision
  safepoint_return(arg_size);
  if (result != NULL && (result->type()->is_float() || result->type()->is_double())) {
    fpu_pop(result->get_register());
  }
}


void CodeEmitter::implicit_null_check (Item* item, CodeEmitInfo* info) {
  if (GenerateCompilerNullChecks) {
    add_debug_info_here(info);
    __ null_check(item2reg(item));
  }
}


void CodeEmitter::explicit_null_check (Item* item, CodeEmitInfo* info) {
  assert(item->is_register() && item->type()->is_object(), "cannot test non register item");
  Register reg = item2reg(item);
  Label contL;
  __ nop(); // make sure no conflicts with oop maps
  int pc_offset = __ offset();
  NullExceptionStub* stub = new NullExceptionStub(pc_offset);
  _slow_case_stubs->append(stub);
  add_call_info(pc_offset, info);
  __ testl (reg, reg);
  __ jcc   (Assembler::zero, *(stub->entry()));
}


void CodeEmitter::explicit_div_by_zero_check (Item* item, CodeEmitInfo* info) {
  __ nop(); // make sure no conflicts with oop maps
  int pc_offset = __ offset();
  DivByZeroStub* stub = new DivByZeroStub(pc_offset);
  _slow_case_stubs->append(stub);
  add_call_info(pc_offset, info); // an exception looks like a call from here
  if (item->is_register()) {
    if (item->type()->is_double_word()) {
      assert(item->destroys_register(), "check");
      __ orl (item2regLO(item), item2regHI(item));
    } else {
      Register reg = item2reg(item);
      __ testl (reg, reg);
    }
    __ jcc   (Assembler::zero, *(stub->entry()));
  } else if (item->is_constant()) {
    if (item->type()->is_long()) {
      if (item2intLO(item) == 0 && item2intHI(item) == 0) {
        __ jmp   (*(stub->entry()));
      }
    } else {
      if (item2int(item) == 0) {
        __ jmp   (*(stub->entry()));
      }
    }
  } else {
    Unimplemented();
  }
}


void CodeEmitter::range_check (Item* array, Item* index, CodeEmitInfo* info) {
  assert(array->is_register() && array->type()->is_object() && index->type()->is_int(), "wrong item state");
  // must implement stub creation for range checks
  int offset = __ offset();
  // implicit null check 
  add_debug_info_here(info);
  RangeCheckStub* stub = NULL;
  if (index->is_constant()) {
    int c = item2int(index);

    // JLS 15.12.1, must check array before index. Don't get fancy
    // with constant negative offsets.

    __ cmpl(Address(item2reg(array),  arrayOopDesc::length_offset_in_bytes()), c); 
    offset = __ offset();  
    stub = new RangeCheckStub(offset);
    __ jcc (Assembler::belowEqual, *(stub->entry()));
  } else {    
    __ cmpl(item2reg(index), Address(item2reg(array),  arrayOopDesc::length_offset_in_bytes())); 
    offset = __ offset();
    stub = new RangeCheckStub(offset);
    __ jcc (Assembler::aboveEqual, *(stub->entry()));
  }
  _slow_case_stubs->append(stub);
  add_call_info(offset, info); // an exception looks like a call from here
}


// Checks that value can be stored into array: 
//   value, index and array registers may not be destroyed and must be preserved in slow case !

void CodeEmitter::array_store_check(Item* array, Item* value, Item* index, RInfo rtmp1, RInfo rtmp2, CodeEmitInfo* info) {
  if (!GenerateArrayStoreCheck) return;
  assert(value->is_register() && array->is_register() && array->type()->is_object() && value->type()->is_object(), "wrong item state");
  Label done, doneAndPop;
  Register value_reg = item2reg(value);
  Register array_reg = item2reg(array);
  Register index_reg = index->is_constant() ?  noreg : item2reg(index);
  Register tmp1  = rtmp1.as_register(); // value_reg;
  Register tmp2  = rtmp2.as_register(); // array_reg;

  int index_const = index->is_constant() ? item2int(index) : -1;
  ArrayStoreCheckStub* stub = new ArrayStoreCheckStub(array_reg, value_reg, index_reg, index_const, info, esp_offset());
  _slow_case_stubs->append(stub);
  if (UseSlowPath) {
    __ jmp(*stub->entry());
  } else {
    __ testl (value_reg, value_reg);
    __ jcc   (Assembler::zero, done);

    __ movl  (tmp2, Address(array_reg, oopDesc::klass_offset_in_bytes()));
    __ movl  (tmp1, Address(value_reg, oopDesc::klass_offset_in_bytes()));
    // get instance klass
    __ movl  (tmp2, Address(tmp2, objArrayKlass::element_klass_offset_in_bytes() + sizeof(oopDesc)));

    __ cmpl  (tmp2, Address(tmp1, Klass::is_a_cache_1_offset_in_bytes() + sizeof(oopDesc)));
    __ jcc   (Assembler::equal, done);
  
    __ cmpl  (tmp2, Address(tmp1, Klass::is_a_cache_2_offset_in_bytes() + sizeof(oopDesc)));
    __ jcc (Assembler::notEqual, *stub->entry()); // <--- jcc notEqual 
  }
  __ bind(*stub->continuation());

  __ bind  (done);
}


void CodeEmitter::emit_slow_case_stubs() {
// slow cases do not belong to the stubs of CodeBuffer
  for (int i = 0; i < _slow_case_stubs->length(); i++) {
    if (!must_bailout()) {
      check_codespace();
      CodeStub* s = _slow_case_stubs->at(i);
      s->emit_code(this);
  #ifdef ASSERT
      s->assert_no_unbound_labels();
  #endif
    }
  }
}


void CodeEmitter::emit_call_stubs() {
  __ code()->set_stubs_begin(__ pc());
  for (int i = 0; i < _call_stubs->length(); i++) {
    if (!must_bailout()) {
      check_codespace();
      CodeStub* s = _call_stubs->at(i);
      s->emit_code(this);
#ifdef ASSERT
      s->assert_no_unbound_labels();
#endif
    }
  }
  __ code()->set_stubs_end( __ pc());
}


void CodeEmitter::emit_constants() {
  _const_table->emit_entries(masm());
}


int CodeEmitter::emit_exception_handler(int max_spills, RInfo cached_receiver) {
  if (!_throw_entry_label.is_unused()) {
    assert(_throw_entry_label.is_unbound(), "who did it?");
    __ bind (_throw_entry_label);
    __ popl (edx); // exception pc in edx
  }
  int offset = __ offset();
  // Temporary solution: always generate exception handlers
  // check if exception handler exist
  // tos: last argument (if the exception happened at a call and there are arguments)
  // eax: exception
  // edx: throwing pc

  if (method()->has_exception_handlers()) {
    if (compilation()->scan_result()->has_floats() || compilation()->scan_result()->has_doubles()) {
      __ empty_FPU_stack();
    }
    // exception handler exist
    // => check if handler for particular exception exists,
    //    continue at exception handler if so, return otherwise
    // note: need to provide esp_offset so handle_exception can
    //       reset esp to its 'home' position (i.e., we discard
    //       any arguments on the stack) *after* looking up the
    //       concrete exception handler.
    __ movl(ecx, stack_offset_in_bytes(max_spills));
    // eax: exception
    // ecx: esp offset
    // edx: throwing pc 
    __ call(Runtime1::entry_for(Runtime1::handle_exception_id), relocInfo::runtime_call_type);
    // exception handler for particular exception doesn't exist
    // => unwind activation and forward exception to caller
    // eax: exception
  } else {
    // There is no handler in this method, we continue in caller
    // reset the implicit flag.
    // Note: comment why do we need to reset the implicit  flag
    __ get_thread(esi);
    __ movl(Address(esi,JavaThread::is_handling_implicit_exception_offset_in_bytes()), NULL);
  }

  // unlock the receiver/klass if necessary
  // eax: exception
  if (method()->is_synchronized()) {
    if (method()->is_static()) {
      __ movl(ebx, method()->holder()->java_mirror()->encoding());
    } else {
      load_receiver(ebx);
    }
    monitorexit(ebx, ecx, SYNC_header, 0, eax);
  }

  if (compilation()->jvmpi_event_method_exit_enabled()) {
    jvmpi_method_exit();
  }
  // unwind activation and forward exception to caller
  // eax: exception
  __ jmp(Runtime1::entry_for(Runtime1::unwind_exception_id), relocInfo::runtime_call_type);
  return offset;
}


void CodeEmitter::emit_osr_entry(int max_spills, int number_of_locks, Label* continuation, int osr_bci) {
  _offsets->_osr_offset = __ offset();
  // we jump here if osr happens with the interpreter
  // state set up to continue at the beginning of the
  // loop that triggered osr - in particular, we have
  // the following registers setup:
  //
  // edi: interpreter locals pointer
  // ebp: interpreter frame pointer
  // esp: interpreter stack pointer
  //
  // keep old ebp around if we have locks to copy
  if (number_of_locks > 0) {
    __ movl(ebx, ebp);
  }
  // build frame
  ciMethod* m = method();
  { __ subl(esp, m->arg_size() * BytesPerWord);                        // space for arguments
    __ pushl((int)Runtime1::entry_for(Runtime1::osr_frame_return_id)); // osr adapter frame return address
    build_frame(max_spills);                                           // space for rest of frame
    // copy locals
    for (int i = 0; i < m->max_locals(); i++) {
      __ movl(eax, Address(edi, -i*BytesPerWord));
      __ movl(frame()->local_address(i, false), eax);
    }
  }
  // note: we do osr only if the expression stack at the loop beginning is empty,
  //       in which case the spill area is empty too and we don't have to setup
  //       spilled locals
  //
  // copy monitors
  // ebx: old ebp value
  { assert(frame::interpreter_frame_monitor_size() == BasicObjectLock::size(), "adjust code below");
    assert(BasicLock::displaced_header_offset_in_bytes() == 0, "adjust code below");
    for (int i = 0; i < number_of_locks; i++) {
      int slot_offset = frame::interpreter_frame_monitor_block_bottom_offset - (i+1) * BasicObjectLock::size();
#ifdef ASSERT
      { Label L;
        __ cmpl(Address(ebx, slot_offset * BytesPerWord + BasicObjectLock::obj_offset_in_bytes()), (int)NULL);
        __ jcc(Assembler::notZero, L);
        __ stop("locked object is NULL");
        __ bind(L);
      }
#endif
      __ movl(eax, Address(ebx, slot_offset * BytesPerWord + BasicObjectLock::lock_offset_in_bytes()));
      __ movl(frame()->monitor_address(i), eax);
    }
  }
  // convert bci's to return addresses if within subroutines
  if (AcceptJsrForOSR) {
    GrowableArray<int>* adr_locals = compilation()->get_address_map(osr_bci);
    for (int i = 0; i < adr_locals->length(); i++) {
      int local_index = adr_locals->at(i);
      Address locAddr = frame()->local_address(local_index, false);
      __ movl(eax, locAddr);
      __ call(Runtime1::entry_for(Runtime1::address_bci_2_pc_id), relocInfo::runtime_call_type);
      __ movl(locAddr, eax);
    }
  }
  // sometimes the receiver is expected in ecx, copy always if any
  if (m->max_locals() > 0) {
    __ movl(ecx, frame()->local_address(0, false));
  }
  // continue at loop header/body corresponding to the osr'ed interpreter loop
  __ jmp(*continuation);
}


void CodeEmitter::add_nop() {
  __ nop();
}


// use this to mark a continuation attached to a bci (used for jsr) 
void CodeEmitter::add_nop_with_info(CodeEmitInfo* info) {
  add_debug_info_here(info);
  __ nop();
}


// Here the esp points at the frame + native arguments (corresponds to oop map at last_native_pc)
void CodeEmitter::native_method_exit (CodeEmitInfo* info) {
  // change thread state
  Register thread = ebx;
  __ get_thread(thread);
  __ movl(Address(thread, JavaThread::thread_state_offset_in_bytes()), _thread_in_native_trans);

  // handle result
  const BasicType return_type = method()->return_type();
  switch (return_type) {
    case T_ARRAY  : // fall through (result is handle)
    case T_OBJECT :
      { Label L;
        __ testl(eax, eax);
        __ jcc(Assembler::zero, L);
        __ movl(eax, Address(eax)); 
        __ bind(L);
        __ verify_oop(eax);
      }
      break;
    case T_BOOLEAN: __ c2bool(eax);            break;
    case T_CHAR   : __ andl(eax, 0xFFFF);      break;
    case T_BYTE   : __ sign_extend_byte (eax); break;
    case T_SHORT  : __ sign_extend_short(eax); break;
    case T_FLOAT  : // fall through (no rounding for floats)
    case T_DOUBLE : // fall through (no rounding for double)
    case T_VOID   : // fall through
    case T_LONG   : // fall through
    case T_INT    : /* do nothing */           break;
    default       : ShouldNotReachHere();
  }
  
  // check for safepoint operation in progress
  { Label L;
    __ cmpl(Address((int)SafepointSynchronize::address_of_state(), relocInfo::none), SafepointSynchronize::_not_synchronized);
    __ jcc(Assembler::equal, L);
    // Note: result object is upacked her
    BasicType type = method()->return_type();
    bool result_is_oop = type == T_OBJECT || type == T_ARRAY;
    if (result_is_oop) {
      // preserve potential oop result
      __ movl(Address(thread, JavaThread::vm_result_offset_in_bytes()), eax);
    }

    // Do not change esp, as the oop map for last_native_pc would be wrong
    // eax/edx are preserved in the runtime stub

    // Nobody changes the last_Java_xx since the call to native
    __ call (Runtime1::entry_for(Runtime1::safepoint_block_id), relocInfo::runtime_call_type);
    add_call_info_here(info);
    __ get_thread(thread);
    if (result_is_oop) {
      // restore preserved oop
      __ movl(eax, Address(thread, JavaThread::vm_result_offset_in_bytes()));
      __ movl(Address(thread, JavaThread::vm_result_offset_in_bytes()), (int)NULL);
    }
    __ bind(L);
  }

  // change thread state
  __ movl(Address(thread, JavaThread::thread_state_offset_in_bytes()), _thread_in_Java);

  // Clear last_native_pc after call. This is crucial, so we can detect if the last_native_pc is
  // used or not. It is, e.g., not used when calling native code directly from C++ code.
  __ movl(Address(thread, JavaThread::last_native_pc_offset_in_bytes()), (int)NULL);  

  // reset_last_Java_frame
  __ movl(Address(thread, JavaThread::last_Java_sp_offset_in_bytes()), (int)NULL);
  // must clear fp, so that compiled frames are not confused; it is possible
  // that we need it only for debugging
  __ movl(Address(thread, JavaThread::last_Java_fp_offset_in_bytes()), (int)NULL);

  // handle exceptions (exception handling will handle unlocking!)
  { Label L;
    __ cmpl(Address(thread, Thread::pending_exception_offset_in_bytes()), (int)NULL);
    __ jcc(Assembler::zero, L);
    // make sure to unlock the object before exiting to exception handler
    __ movl(eax, Address(thread, Thread::pending_exception_offset_in_bytes()));
    // Note: pending exception is cleared in StubRoutines::forward_exception_entry
    // push address of native stub instruction so that the exception handler of
    // native is invoked in case it needs unlocking
    // note that pc  in native does not have correct oop map, but we cannot have GC at exit
    assert(StubRoutines::forward_exception_entry() != NULL, "must be created");
    __ call(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);

    __ bind(L);
  }

  RInfo dummy_receiver; // there is no receiver caching in natives
  if (method()->is_synchronized()) { 
    // must preserve results across monitor_exit
    push_reg(eax);
    push_reg(edx);
    return_op_prolog(0, dummy_receiver);
    // restore potential results (synchronization exit cannot block, but destroys register)
    pop(edx);
    pop(eax);
  }

  return_op(NULL);
  set_esp_offset(0);
  // esp offset is not zero (see comment in native_call)
  // assert(esp_offset() == 0, "must remove stack");
}


// When passing parameters to natives, the stack looks as follows
//
// +------------------------------+ <- esp
// | JNI Env                      |
// +------------------------------+
// | receiver class mirror handle |
// +------------------------------+ 
// | parameter 0                  |
// +------------------------------+
// | parameter 1                  |
// : ::::::::: :                  :
// +------------------------------+
// | parameter n-1                |
// +------------------------------+
// | mirror class                 | NOTE: Only for static method, otherwise not existent
// +------------------------------+ <- ebp
// | old ebp                      |
// +------------------------------+
// | return address               |
// +------------------------------+ <-+
// | argument n-1                 |   |
// : :::::::: :::                 :   :
// +------------------------------+   |
// | argument 2                   |   |
// +------------------------------+   | = method()->arg_size()
// | argument 1                   |   |
// +------------------------------+   |  
// | argument 0 (receiver if any) |   |
// +------------------------------+ <-+
// |                              |

void CodeEmitter::native_call(address native_entry, CodeEmitInfo* info) {
  // get current pc information
  int pc_in_native        = (int)__ pc();
  int pc_in_native_offset = __ offset();
  // pass parameters
  int src_offset = method()->arg_size(); // must include receiver!
  int dst_offset = 0;
  { const Register lo_word = eax;
    const Register hi_word = edx;
    // push parameters (reversed order, i.e., parameter 0 is on TOS)
    ciSignature* sig = method()->signature();
    for (int i = sig->count(); i-- > 0;) { // note: sig->count() excludes receiver!
      __ movl(lo_word, frame()->local_address(--src_offset, false));
      if (sig->is_oop_at(i)) {
        // use NULL for NULL oops, use handle otherwise
        Label L;
        __ testl(lo_word, lo_word);
        __ jcc(Assembler::zero, L);
        __ leal(lo_word, frame()->local_address(src_offset, false));
        __ bind(L);
      } else if (sig->is_two_word_at(i)) {
        // push high word first
        __ movl(hi_word, frame()->local_address(--src_offset, false));
        push_reg(hi_word); dst_offset++;
      }
      push_reg(lo_word); dst_offset++;
    }
  }
  // push "receiver"
  { const Register handle = eax;
    const Register mirror = edx;
    if (method()->is_static()) {
      // and install mirror
      __ movl(mirror, method()->holder()->java_mirror()->encoding());
      __ movl(frame()->spill_address(0, false), mirror);
      // get mirror handle
      __ leal(handle, frame()->spill_address(0, false));
    } else {
      // get receiver handle
      // note: receiver cannot be NULL at this place!
      __ leal(handle, frame()->local_address(--src_offset, false));
    }
    push_reg(handle); dst_offset++;
  }
  assert(src_offset == 0, "must have iterated through all arguments");
  // push JNIEnv (must be on TOS before the call)
  Register thread = ebx;
  Register t      = edx;
  __ get_thread(thread);
  __ leal(t, Address(thread, JavaThread::jni_environment_offset_in_bytes()));
  push_reg(t); dst_offset++;

  // reset handle block
  __ movl(t, Address(thread, JavaThread::active_handles_offset_in_bytes()));
  __ movl(Address(t, JNIHandleBlock::top_offset_in_bytes()), 0);

  // set_last_Java_frame_before_call
  __ movl(Address(thread, JavaThread::last_Java_fp_offset_in_bytes()), ebp);
  __ movl(Address(thread, JavaThread::last_Java_sp_offset_in_bytes()), esp);

  // change thread state

  // Push return address on stack. We need to do this before we change thread-state, to
  // make sure that the safepoint code will always see a valid return address on the stack.
  
  // Change state to native (we save the return address in the thread, since it might not
  // be pushed on the stack when we do a a stack traversal). It is enough that the pc()
  // points into the right code segment. It does not have to be the correct return pc.

  __ leal(t, Address(pc_in_native, relocInfo::internal_word_type));
  __ movl(Address(thread, JavaThread::last_native_pc_offset_in_bytes()), t);  
  __ movl(Address(thread, JavaThread::thread_state_offset_in_bytes()), _thread_in_native); 
  
  // This outcommented code may be used for debugging purposes
  // __ empty_FPU_stack();

  __ call(native_entry, relocInfo::runtime_call_type);
  // result potentially in edx:eax or ST0

  // last_native_pc is used to build this frame, therefore set debug info at last_native_pc
  add_call_info(pc_in_native_offset, info); 
  add_call_info_here(info);

  // The esp must be constants (including the arguments) all along, otherwise
  // The gc cannot work: The native is popping the arguments
  // Restore pc to state before call (i.e., frame + arguments)
  __ decrement(esp, dst_offset * wordSize); // does not emit code for stack_offset == 0
}


void CodeEmitter::increment_reg(RInfo reg) {
  __ incl(reg.as_register());
}


void CodeEmitter::decrement_reg(RInfo reg) {
  __ decl(reg.as_register());
}


void CodeEmitter::increment_local(int local_index) {
  Address dst_addr = frame()->local_address(local_index, false);
  __ incl(dst_addr);
}


void CodeEmitter::decrement_local(int local_index) {
  Address dst_addr = frame()->local_address(local_index, false);
  __ decl(dst_addr);
}


void CodeEmitter::breakpoint() {
  __ int3();
}


void CodeEmitter::align_backward_branch() {
  __ align (4);
}


void CodeEmitter::set_32bit_fpu_precision() {
  __ fldcw(Address((int)StubRoutines::addr_fpu_cntrl_wrd_24(), relocInfo::none));
}


void CodeEmitter::restore_fpu_precision() {
  __ fldcw(Address((int)StubRoutines::addr_fpu_cntrl_wrd_std(), relocInfo::none));
}


void CodeEmitter::check_codespace() {
  if (_masm->code()->code_end() + 1*K >= _masm->code()->code_limit()) {
    set_bailout("CodeBuffer overflow");
  }
}


#ifndef PRODUCT
void CodeEmitter::print() { 
  tty->print("ESPoff: %d ", _esp_offset); 
  tty->print("FPUstack: %s ", frame()->fpu_stack()->is_empty() ? "empty" : "not empty");
  frame()->fpu_stack()->print();
  tty->cr();
}
#endif


// ---------------arraycopy helpers-----------------------

// reg must be positive or zero, otherwise jump to l
void CodeEmitter::ac_check_for_negative(Register reg, int v, Label* l) {
  if (reg == noreg) {
    if (v < 0) {
      __ jmp (*l);
    }
  } else {
    __ cmpl (reg, 0);
    __ jcc  (Assembler::less, *l);
  }
}


// (lng + pos) <= arr.length, otherwise jump to l 
void CodeEmitter::ac_check_length(Register arr, Register lng_reg, Register pos_reg, int pos_v, Register tmp, Label* l) {
  Address lng_address = Address(arr, arrayOopDesc::length_offset_in_bytes());
  if (pos_reg == noreg) {
    __ movl (tmp, lng_reg);
    if (pos_v != 0) {
      __ addl (tmp, pos_v);
    }
  } else {
    assert(pos_reg != noreg && lng_reg != noreg, "check");
    __ movl (tmp, pos_reg);
    __ addl (tmp, lng_reg);
  }
  __ cmpl (tmp, lng_address);
  // must use unsigned test: if  is greater than lng_address -> go to slow case
  __ jcc  (Assembler::above, *l);
}



void CodeEmitter::ac_memory_start(Register arr, Register pos_reg, int pos_v, Register tmp, BasicType type) {
  if (pos_reg == noreg) {
    __ movl(tmp, pos_v);
    __ leal (arr, Address(arr, tmp, Address::times_2, arrayOopDesc::base_offset_in_bytes(type)));
  } else {
    __ leal (arr, Address(arr, pos_reg, Address::times_2, arrayOopDesc::base_offset_in_bytes(type)));
  }
}


// This is the inlined version of java.lang.System.arraycopy;
// Currently we do not know the type of arrays and must do tests (later IR will provide us
// with that information).
// There are three levels of array copy:
//   - inlined tight loop or rep_move for character arrays;
//   - simple leaf routine(s) (SharedRuntime) that copies the array (all pre-checks succeeded)
//   - call to native: used if any of the pre-checks failed

void CodeEmitter::arraycopy(Item* src, Item* src_pos, Item* dst, Item* dst_pos, Item* length, RInfo tmpRinfo, RInfo unused1, RInfo unused2, CodeEmitInfo* info) {
  Register src_reg = src->get_register().as_register();
  Register dst_reg = dst->get_register().as_register();
  Register tmp = tmpRinfo.as_register();
  Register src_pos_reg = noreg;
  int src_pos_const = -1;
  if (src_pos->is_register()) {
    src_pos_reg = src_pos->get_register().as_register();
  } else {
    src_pos_const = item2int(src_pos);
  }
  Register dst_pos_reg = noreg;
  int dst_pos_const = -1;
  if (dst_pos->is_register()) {
    dst_pos_reg = dst_pos->get_register().as_register();
  } else {
    dst_pos_const = item2int(dst_pos);
  }
  Register length_reg = length->get_register().as_register();

  ArrayCopyDesc* desc = new ArrayCopyDesc();
  desc->set_src(src_reg);
  desc->set_src_pos_reg(src_pos_reg);
  desc->set_dst(dst_reg);
  desc->set_dst_pos_reg(dst_pos_reg);
  desc->set_length_reg(length_reg);
  desc->set_dst_pos_const(dst_pos_const);
  desc->set_src_pos_const(src_pos_const);
  desc->set_length_const(-1);

  ArrayCopyStub* stub = new ArrayCopyStub(desc, info, esp_offset());
  _slow_case_stubs->append(stub);
  // once we know the type of arrays, here will come a fast loop

  //------------------pre-checks-------------------------
  // test for NULL
  __ testl(src_reg, src_reg);
  __ jcc (Assembler::zero, *stub->entry());
  __ testl(dst_reg, dst_reg);
  __ jcc (Assembler::zero, *stub->entry());

  // test if the classes are same
  __ movl (tmp, Address(src_reg, oopDesc::klass_offset_in_bytes()));
  __ cmpl (tmp, Address(dst_reg, oopDesc::klass_offset_in_bytes()));
  __ jcc  (Assembler::notEqual, *stub->entry());
 
  // test length 
  ac_check_for_negative(length_reg, -1, stub->entry());
  ac_check_for_negative(src_pos_reg, src_pos_const, stub->entry());
  ac_check_for_negative(dst_pos_reg, dst_pos_const, stub->entry());

  ac_check_length(src_reg, length_reg, src_pos_reg, src_pos_const, tmp, stub->entry());
  ac_check_length(dst_reg, length_reg, dst_pos_reg, dst_pos_const, tmp, stub->entry());

  //----------------check if can be inlined---------------

  // test if character array
  __ movl (tmp, Address(src_reg, oopDesc::klass_offset_in_bytes()));
  __ cmpl (tmp, ciTypeArrayKlass::make(T_CHAR)->encoding());
  // go to array copy without checks
  __ jcc  (Assembler::notEqual, *stub->noCheckEntry());

  // cannot do copy within the same array
  __ cmpl (src_reg, dst_reg);
  __ jcc (Assembler::equal, *stub->noCheckEntry());


  //----------------inlined character copy------------------

  // here we destroy registers

  // get start addresses to copy
  ac_memory_start(src_reg, src_pos_reg, src_pos_const, tmp, T_CHAR);
  ac_memory_start(dst_reg, dst_pos_reg, dst_pos_const, tmp, T_CHAR);


  //-----T_CHAR---copy
  Label loopTest, loopStart, evenL;

  // check if we need to copy first a char (16-bit)
  __ testl (length_reg, 0x1);
  __ jcc   (Assembler::zero, evenL);

  // move one word
  __ load_unsigned_word(tmp, Address(src_reg));
  __ addl(src_reg, wordSize / 2); // 2 for T_CHAR
  __ movw (Address(dst_reg), tmp);
  __ addl(dst_reg, wordSize / 2); // 2 for T_CHAR

  __ bind  (evenL);

  __ shrl(length_reg, 1); // divide by two for character copy using words

  __ cmpl (length_reg, 4);
  __ jcc  (Assembler::lessEqual, loopTest);
  __ pushl(src_reg);
  __ pushl(dst_reg);
  __ pushl(length_reg);
  __ popl (ecx);
  __ popl (edi);
  __ popl (esi);
  __ rep_move();

  Label done;
  __ jmp(done);
  
  __ jmp (loopTest);
  __ bind(loopStart);
  // wordwise moves
  __ decl (length_reg);
  __ movl (tmp, Address(src_reg, length_reg, Address::times_4, 0));
  __ movl (Address(dst_reg, length_reg, Address::times_4, 0), tmp);
  __ bind(loopTest);
  __ cmpl(length_reg, 0);
  __ jcc (Assembler::greater, loopStart);
  
  __ bind(*stub->continuation());

  __ bind(done);
}


// This is the fast version of java.lang.String.compare; it has not
// OSR-entry and therefore, we generate a slow version for OSR's
void CodeEmitter::string_compare() {
  assert(OptimizeLibraryCalls, "should not call this");
  // Load input parameters
  Address in0 = frame()->local_address(0, false);
  Address in1 = frame()->local_address(1, false);
  __ movl (ebx, ecx); // receiver is in ecx
  __ movl (eax, in1);
  
  // Get addresses of first characters from both Strings
  { CodeEmitInfo info(0, NULL, NULL);
    add_debug_info_here(&info);
  }
  __ movl (esi, Address(eax, java_lang_String::value_offset_in_bytes()));
  __ movl (ecx, Address(eax, java_lang_String::offset_offset_in_bytes()));
  __ leal (esi, Address(esi, ecx, Address::times_2, arrayOopDesc::base_offset_in_bytes(T_CHAR)));


  // ebx may be NULL
  { CodeEmitInfo info(0, NULL, NULL);
    add_debug_info_here(&info);
  }
  __ movl (edi, Address(ebx, java_lang_String::value_offset_in_bytes()));
  __ movl (ecx, Address(ebx, java_lang_String::offset_offset_in_bytes()));
  __ leal (edi, Address(edi, ecx, Address::times_2, arrayOopDesc::base_offset_in_bytes(T_CHAR)));  

  // compute minimum length (in eax) and difference of lengths (on top of stack)
  if (CodeForP6) {
    __ movl (ebx, Address(ebx, java_lang_String::count_offset_in_bytes()));
    __ movl (eax, Address(eax, java_lang_String::count_offset_in_bytes()));
    __ movl (ecx, ebx);
    __ subl (ebx, eax); // subtract lengths
    __ pushl(ebx);      // result
    __ cmovl(Assembler::lessEqual, eax, ecx);
  } else {
    Label L;
    __ movl (ebx, Address(ebx, java_lang_String::count_offset_in_bytes()));
    __ movl (ecx, Address(eax, java_lang_String::count_offset_in_bytes()));
    __ movl (eax, ebx);
    __ subl (ebx, ecx);
    __ pushl(ebx);
    __ jcc  (Assembler::lessEqual, L);
    __ movl (eax, ecx);
    __ bind (L);
  }
  // is minimum length 0?
  Label noLoop, haveResult;
  __ testl (eax, eax);
  __ jcc (Assembler::zero, noLoop);

  // compare first characters
  __ load_unsigned_word(ecx, Address(edi));
  __ load_unsigned_word(ebx, Address(esi));
  __ subl(ecx, ebx);
  __ jcc(Assembler::notZero, haveResult);
  // starting loop
  __ decl(eax); // we already tested index: skip one  
  __ jcc(Assembler::zero, noLoop);

  // set esi.edi to the end of the arrays (arrays have same length)
  // negate the index

  __ leal(esi, Address(esi, eax, Address::times_2, type2aelembytes[T_CHAR]));
  __ leal(edi, Address(edi, eax, Address::times_2, type2aelembytes[T_CHAR]));
  __ negl(eax);

  // compare the strings in a loop

  Label loop;
  __ align(wordSize);
  __ bind(loop);
  __ load_unsigned_word(ecx, Address(edi, eax, Address::times_2, 0));
  __ load_unsigned_word(ebx, Address(esi, eax, Address::times_2, 0));
  __ subl(ecx, ebx);
  __ jcc(Assembler::notZero, haveResult);
  __ incl(eax);
  __ jcc(Assembler::notZero, loop);

  // strings are equal up to min length

  __ bind(noLoop);
  __ popl(eax);
  __ leave();
  safepoint_return(2*wordSize);

  __ bind(haveResult);
  // leave instruction is going to discard the TOS value
  __ movl (eax, ecx); // result of call is in eax
  __ leave();
  safepoint_return(2*wordSize);
}


void CodeEmitter::triglib(ciMethod::IntrinsicId trig_id) {
  assert(OptimizeLibraryCalls, "should not call this");
  // get argument
  __ fld_d(frame()->local_address(0, true));
  // compute trigonometric value (may fail)
  address slow_case = NULL;
  switch (trig_id) {
    case methodOopDesc::_dsin: __ fsin(); slow_case = CAST_FROM_FN_PTR(address, SharedRuntime::dsin); break;
    case methodOopDesc::_dcos: __ fcos(); slow_case = CAST_FROM_FN_PTR(address, SharedRuntime::dcos); break;
    default                  : Unimplemented();
  }
  // check if computation failed (i.e., C2 is set)
  { Label L;
    __ jnC2(eax, L);
    // failed -> slow case
    bang_stack();
    __ addl (esp, -2*wordSize);
    __ fstp_d(Address(esp));
    __ call_VM_leaf(slow_case, 2);
    __ bind(L);
  }
  // result in FPU ST(0)
  __ leave();
  safepoint_return(2*wordSize);
}


void CodeEmitter::handler_entry() {
  __ nop();
}


int CodeEmitter::frame_size() {
  return no_frame_size; // make sure it is not used
}


void CodeEmitter::emit_accessor_code(int offset, BasicType type) {
  const AlignEntryCode = 4;

  // entry point
  _offsets->_ep_offset = __ offset();
  if (C1Breakpoint) __ int3();
  check_icache();

  // verified entry point
  // Do not align vep for non-finals as we mostly enter through ep anyway
  if (method()->is_final_method()) {
    __ align(AlignEntryCode);
  }
  _offsets->_vep_offset = __ offset();
  _offsets->_iep_offset = __ offset();
  if (C1Breakpoint) __ int3();
  _offsets->_code_offset = __ offset();

  Address src_addr = Address(RECEIVER, offset*wordSize);
  if (type == T_FLOAT) {
    __ fld_s(src_addr);
  } else if (type == T_DOUBLE) {
    __ fld_d(src_addr);
  } else if (type == T_LONG) {
    Address src_addr_hi = Address(RECEIVER, (offset+1)*wordSize);
    Address src_addr_lo = Address(RECEIVER, offset*wordSize);
    __ movl (edx, src_addr_hi);
    __ movl (eax, src_addr_lo);
  } else {
    __ movl (eax, src_addr);
  }

  int arg_size = (UseC2CallingConventions) ? 0 : method()->arg_size() * wordSize;
  assert (arg_size >= 0, "wrong result size");
  safepoint_return(arg_size);
  assert (!compilation()->jvmpi_event_method_enabled(), "does not work with jvmpi") ;
}

