// @(#)c1_CodeGenerator_i486.cpp	1.236 00/06/09 17:15:30
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_CodeGenerator_i486.cpp.incl"


// Notes to OopMap generations:
// - because the pc-offset is attached to each OopMap, we must create
//   a new OopMap for each gc-point

//--------------------------------------------------------------
//               ValueGen
//--------------------------------------------------------------


RInfo ValueGen::noRInfo()         { return FrameMap::_noRInfo;      }
RInfo ValueGen::return1RInfo()    { return FrameMap::_eaxRInfo;     }
RInfo ValueGen::divInRInfo()      { return FrameMap::_eaxRInfo;     }
RInfo ValueGen::divOutRInfo()     { return FrameMap::_eaxRInfo;     }
RInfo ValueGen::remOutRInfo()     { return FrameMap::_edxRInfo;     }
RInfo ValueGen::shiftCountRInfo() { return FrameMap::_ecxRInfo;     }
RInfo ValueGen::return2RInfo()    { return FrameMap::_eax_edxRInfo; } // for long result Lo:eax/Hi:edx
RInfo ValueGen::returnF0RInfo()   { return FrameMap::_f0RInfo;      }
RInfo ValueGen::returnD0RInfo()   { return FrameMap::_d0RInfo;      }
RInfo ValueGen::receiverRInfo()   { return FrameMap::_ecxRInfo;     }
RInfo ValueGen::syncTempRInfo()   { return FrameMap::_eaxRInfo;     }
RInfo ValueGen::nonReturnRInfo()  { return FrameMap::_esiRInfo;     }

//-----------------------spilling--------------------------------------------------------


// returns true if reg could be smashed by a callee.
// If is_C_call, return true when register could be smashed by a C-call.
bool ValueGen::is_caller_save_register(RInfo reg, bool is_C_call) {
  return true; // all registers are caller saved on INTEL
}


//-----------------------register allocation---------------------------------------------

RInfo ValueGen::result_register_for(ValueType* type, bool callee) {
  ValueTag tag = type->tag();
  switch (tag) {
    case intTag:
    case objectTag:
    case addressTag: return return1RInfo();

    case longTag:    return return2RInfo();

    case floatTag:   return returnF0RInfo();
    case doubleTag:  return returnD0RInfo();

    default: ShouldNotReachHere(); return noRInfo();
  }
}


//--------- loading items into registers --------------------------------


// for correct fpu stack simulation we must dereference
// an fpu register explictly, by copying it into a new register;
// in such case, item will be set to a new register
void ValueGen::check_float_register(Item* item) {
  if (item->type()->is_float_kind()) {
    if (ra()->get_register_rc(item->get_register()) > 1) {
      // we need a new register to hold the value
      RInfo reg = lock_free_rinfo(item->value(), item->type());
      if (gen_code()) {
        code_emit()->copy_fpu_item(reg, item);
      }
      rfree(item);
      item->set_register(reg);
      set_maynot_spill(item);
    }
  }
}


// iyem will be loaded in a byte register
void ValueGen::load_byte_item(Item* item) {
  item->update();
  set_maynot_spill(item);
  debug_only(check_item_load(item);)
  if (item->is_register() && FrameMap::is_byte_rinfo(item->get_register()) && !must_copy_register(item)) {
    // it is a byte register: we are fine
    // make sure that it is a byte register
  } else {
    assert(!item->type()->is_float() && !item->type()->is_double() , "canot load floats in byte register");
    RInfo reg = lock_free_rinfo(item->value(), FrameMap::byte_reg_mask());
    if (gen_code()) {
      code_emit()->item_2_reg(reg, item);
    }
    item_free(item);
    item->set_register(reg);
    set_maynot_spill(item);
  }
}


// Handle fanout of fpu registers: return false if no fanout;
// If fanout than, we copy the value of float register into a new one,
// so that the new FPU register has ref-count 1
bool ValueGen::fpu_fanout_handled(Item* item) {
  if (item->is_register() && (item->type()->is_float() || item->type()->is_double()) &&  (ra()->get_register_rc(item->get_register()) > 1)) {
    // The item is float or double register with a use_count > 1
    RInfo reg = lock_free_rinfo(item->value(), item->type());
    if (gen_code()) {
      code_emit()->copy_fpu_item(reg, item);
    }
    item_free(item);
    item->set_register(reg);
    return true;
  }
  return false;
}


//---------------------------------------------------------------


// This check is done at item loads
void ValueGen::check_item_load(Item* item) {
  if (item->type()->is_float_kind()) {
    assert(item->destroys_register(), "FPU registers are always destroyed");
  }
  check_item(item);
}



//----------------------------------------------------------------------
//             visitor functions
//----------------------------------------------------------------------


// Phi that represent the TOS is in register; all others are in spill area
// TOS(1) element is at spill index 0.
void ValueGen::do_Phi(Phi* x) {
  assert(x->is_root(), "phi must be root");
  assert(x->item() != NULL, "phi's item must be set by block prolog");
}


void ValueGen::do_Constant(Constant* x) {
  // value of constant is known through the referenced instruction
  _result->set_constant();
}


void ValueGen::do_LoadLocal(LoadLocal* x) {
  RInfo r;
  if (cached_cpu_local(x->index(), r)) {
    _result->set_register(r, true);
  } else {
    _result->set_stack(x->index());
  }
}


// call this for efficient handling arithmetic and logical operations
//
//   Load n     any expression             <-- use count of load is 1
//     \        /
//      \      /
//        Op (and, or, xor, add, sub)
//         |
//       Store n


bool ValueGen::canbe_short_form_arithmetic(StoreLocal* x) {
  Op2* op2 = x->value()->as_Op2();
  if (op2 != NULL && !op2->is_root() && op2->type()->is_int()) { 
    LoadLocal* left = op2->x()->as_LoadLocal();
    if (left == NULL || left->index() != x->index()) return false;
    // left local is the same as the store
    RInfo r;
    bool is_cached = cached_cpu_local(x->index(), r);
    if (!is_cached) return false; // it brings currently something only for cached locals
    // if it was pinned by store then a copy has been already generated (see finish_root)
    // we are going to reuse 
    if (!is_cached) {
      if (left->use_count() > 1) {
        // we may overwrite it if it was not pinned by store (i.e., it is alive past the store)
        if (left->is_pinned_by_store()) return false;
      }
    }

    if (op2->as_ArithmeticOp() != NULL) {
      ArithmeticOp* a_op = op2->as_ArithmeticOp();
      if (a_op->op() == Bytecodes::_iadd || a_op->op() == Bytecodes::_isub) {
        return true;
      }
    } // later also logical ops
  }
  return false;
}


void ValueGen::do_StoreLocal(StoreLocal* x) {
  HintItem theHint(x->type());
  // if this is store local, and we are caching it than we pass a hint
  set_no_result(x);
  bool short_form = canbe_short_form_arithmetic(x);
  if (short_form) {
    ArithmeticOp* a_op = x->value()->as_ArithmeticOp();
    assert(a_op != NULL, "check failed");
    RInfo r;
    bool is_cached = cached_cpu_local(x->index(), r);
    Item value(a_op);
    do_ShortArithmeticOp(a_op, r);

    if (is_cached) {
      // do nothing, result is already in right location
    } else {
      if (gen_code()) {
        code_emit()->item_2_local(x->index(), &value);
      }
    }
  } else {
    Item value(x->value());
    value.handle_float_kind();

    ValueGen v  (&value, &theHint, this);
    if (!value.is_constant()) {
      load_item(&value);
      rfree(&value);
    }
 
    if (gen_code()) {
      RInfo r;
      if (cached_cpu_local(x->index(), r)) {
        code_emit()->item_2_reg(r, &value);
      } else {
        code_emit()->item_2_local(x->index(), &value);
      }
    }
  }
}


void ValueGen::do_LoadField(LoadField* x) {
  bool needs_patching = PatchALot || !x->is_initialized() || !x->is_loaded();
  Item     object(x->obj());
  ValueGen o(&object, HintItem::no_hint(), this);
  if (needs_patching && x->is_static()) {
    load_item_patching(&object);
  } else {
    load_item(&object);
  }
  const RInfoCollection* oop_regs = needs_patching ? get_registers_with_oops() : NULL;
  rfree(&object);
  // result should be hint
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL); 
      code_emit()->explicit_null_check(&object, &info);
    }
    if (PrintNotLoaded && needs_patching) {
      tty->print_cr("   ###class not loaded at load_%s bci %d", x->is_static() ?  "static" : "field", x->bci());
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
    code_emit()->field_load(reg, x->field_type(), &object, x->offset(), needs_patching, x->is_loaded(), &info);
  }
}


void ValueGen::do_StoreField(StoreField* x) {
  bool needs_patching = PatchALot || !x->is_initialized() || !x->is_loaded();
  Item object(x->obj());
  Item value(x->value());
  value.handle_float_kind();
  ValueTag valueTag = x->type()->tag();
  if (valueTag == objectTag) {
    object.set_destroys_register();
  }
  ValueGen o(&object, HintItem::no_hint(), this);
  ValueGen v(&value,  HintItem::no_hint(), this);
  if (needs_patching && x->is_static()) {
    load_item_patching(&object);
  } else {
    load_item(&object);
  }
  RInfo tmp;
  if (!value.is_constant() || needs_patching) {
    // load item if field not initialized
    // load item if field not constant
    // because of code patching we cannot inline constants
    if (!needs_patching && value.is_stack() && (valueTag == floatTag || valueTag == doubleTag)) {
      // use integer registers for floats and doubles
      // ValueType* type = (valueTag == floatTag) ? intType : longType;
      ValueType* type = NULL;
      if (valueTag == floatTag) {
        type = intType;
      } else {
        type = longType;
      }
      tmp = get_free_rinfo(type);
    } else {
      load_item(&value);
    }
  } else {
    dont_load_item(&value);
  }
  const RInfoCollection* oop_regs = needs_patching ? get_registers_with_oops() : NULL;
  item_free(&value);
  item_free(&object);
  set_no_result(x);
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->explicit_null_check(&object, &info);
    }
    if (PrintNotLoaded && needs_patching) {
      tty->print_cr("   ###class not loaded at store_%s bci %d", x->is_static() ?  "static" : "field", x->bci());
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
    code_emit()->field_store(x->field_type(), &object, x->offset(), &value, needs_patching, x->is_loaded(), &info, tmp);
  }
}


void ValueGen::do_ArrayLength(ArrayLength* x) {
  Item array(x->array());
  ValueGen a(&array, HintItem::no_hint(), this);
  load_item(&array);
  rfree(&array);
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->explicit_null_check(&array, &info);
    }
    const int array_length_offset = arrayOopDesc::length_offset_in_bytes() / wordSize;
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
    code_emit()->field_load(reg, T_INT, &array, array_length_offset, false, true, &info);
  }
}


void ValueGen::do_LoadIndexed(LoadIndexed* x) {
  Item array(x->array());
  Item index(x->index());
  ValueGen a(&array, HintItem::no_hint(), this);
  ValueGen i(&index, HintItem::no_hint(), this);
  load_item(&array);
  if (!index.is_constant()) {
    load_item(&index);
    rfree(&index);
  }
  // delayed register release for LONG types, otherwise array or index register may be overwritten
  // before both loads are completed
  if (!x->type()->is_long()) rfree(&array);
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->explicit_null_check(&array, &info);
    }
    if (GenerateRangeChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->range_check(&array, &index, &info);
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
    code_emit()->indexed_load(reg, x->elt_type(), &array, &index, &info);
  }
  if (x->type()->is_long()) rfree(&array);
}


void ValueGen::do_StoreIndexed(StoreIndexed* x) {
  assert(x->is_root(),"");
  bool obj_store = x->elt_type() == T_ARRAY || x->elt_type() == T_OBJECT;
  Item  array(x->array());
  if (obj_store) {
    array.set_destroys_register();
  }
  Item  index(x->index());
  Item  value(x->value());
  value.handle_float_kind();
  ValueGen a(&array, HintItem::no_hint(), this);
  ValueGen i(&index, HintItem::no_hint(), this);
  // for T_BYTE element type, we must have a byte register free
  ValueGen v(&value, HintItem::no_hint(), this);
  load_item(&array);
  if (!index.is_constant()) {
    load_item(&index);
  }
 
  bool must_load = true;
  if (x->elt_type() == T_SHORT || x->elt_type() == T_CHAR) {
    // there is no immediate move of word values in asembler_i486.?pp
    must_load = true;
  } else {
    if (obj_store && value.is_constant() && !value.get_jobject_constant()->is_loaded()) {
      // do nothing; do not load (NULL object)
      must_load = false;
    } else if (value.is_constant() && !obj_store) {
      // array store check needs a register, otherwise do not load a constant
      must_load = false;
    }
  }
  if (must_load) {
    if (x->elt_type() == T_BYTE) {
      load_byte_item(&value);
    } else {
      load_item(&value);
    }
  }

  const RInfoCollection* oop_regs = obj_store ? get_registers_with_oops() : NULL;
  RInfo tmp1;
  RInfo tmp2;
  {
    HideReg hr1(this, objectType);
    HideReg hr2(this, objectType);
    tmp1 = hr1.reg();
    tmp2 = hr2.reg();
    item_free(&index);
    item_free(&value);
    rfree(&array);
  }

  set_no_result(x);
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->explicit_null_check(&array, &info);
    }
    if (GenerateRangeChecks) { 
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->range_check(&array, &index, &info);
    }
    if (obj_store) {
      if (value.is_constant() && !value.get_jobject_constant()->is_loaded()) {
        // skip store check
      } else {
        assert(!tmp1.is_same(tmp2), "wrong allocation");
        CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
        code_emit()->array_store_check(&array, &value, &index, tmp1, tmp2, &info);
      }
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
    RInfo tmp; // tmp is not used
    code_emit()->indexed_store(x->elt_type(), &array, &index, &value, tmp, &info);
  }
}


void ValueGen::do_MonitorEnter(MonitorEnter* x) {
  spill_values_on_stack(x->stack());
  lock_spill_rinfo(NULL, syncTempRInfo(), 1);
  assert(x->is_root(),"");
  Item obj(x->obj());
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item(&obj);
  RInfo lock = get_free_rinfo(intType);
  rfree(&obj);
  ra()->free_reg(syncTempRInfo());
  assert(ra()->are_all_registers_free(), "slow case does not preserve registers");
  set_no_result(x);
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      code_emit()->explicit_null_check(&obj, &info);
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), /*x->stack()*/ NULL);
    RInfo scratch; // not used
    code_emit()->monitor_enter(&obj, lock, syncTempRInfo(), scratch, x->monitor_no(), &info);
  }
}


void ValueGen::do_MonitorExit(MonitorExit* x) {
  spill_values_on_stack(x->stack());
  lock_spill_rinfo(NULL, syncTempRInfo(), 1);
  assert(x->is_root(),"");
  Item obj(x->obj());
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item(&obj);
  RInfo lock = get_free_rinfo(intType);
  rfree(&obj);
  ra()->free_reg(syncTempRInfo());
  assert(ra()->are_all_registers_free(), "slow case does not preserve registers");
  set_no_result(x);
  if (gen_code()) {
    code_emit()->monitor_exit(&obj, lock, syncTempRInfo(), x->monitor_no());
  }
}


// returns true if either strictFP is true or 32-bit rounding precision is
// required.
// Note: this leads to excessive spilling in strictFP mode; it could be solved
// with an extension to items
bool ValueGen::must_round(Value x, const Item* hint) {
  assert(x->as_Intrinsic() != NULL || x->as_NegateOp() != NULL || x->as_ArithmeticOp() != NULL || x->as_Convert() != NULL, "wrong operation");
  if (method()->is_strict()) return true;
  if (x->type()->is_float()) {
    if (x->is_root() || x->use_count() > 1 || (hint->is_round32() && !ra()->is_32bit_precision()) ) {
      return !is_32bit_mode();
    }
  }
  return false;
}


bool ValueGen::is_32bit_mode() {
  return (block()->block_item() != NULL && block()->block_item()->is_32bit_precision());
}


// _ineg, _lneg, _fneg, _dneg 
void ValueGen::do_NegateOp(NegateOp* x) {
  Item value(x->x());
  value.set_destroys_register();
  ValueGen v(&value, hint(), this);
  load_item_hint(&value, hint());
  rfree(&value);
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    code_emit()->negate(reg, &value);
  }
  if (must_round(x, hint())) {
    spill_item(_result);
    assert(_result->is_spilled(), "spilling failed");
  }
}



// for  _fadd, _fmul, _fsub, _fdiv, _frem
//      _dadd, _dmul, _dsub, _ddiv, _drem
void ValueGen::do_ArithmeticOp_FPU(ArithmeticOp* x) {
  Item left_item(x->x());
  Item right_item(x->y());
  // Because of the way we simulate FPU  regsiter location in code emitter, we must
  // mark both items as being destroyed(i.e., they are going to be removed from the FPU stack)
  right_item.set_destroys_register();
  left_item.set_destroys_register();
  // missing test if instr is commutative and if we should swap
  HintItem left_hint(x->x()->type()); left_hint.set_from_item(hint());
  HintItem right_hint(x->y()->type()); right_hint.set_from_item(HintItem::no_hint());
  if (x->type()->is_float()) {
    left_hint.set_round32();
    right_hint.set_round32();
  }
  ValueGen l(&left_item,  &left_hint, this);
  ValueGen r(&right_item, &right_hint, this);
  Item* left_arg  = &left_item;
  Item* right_arg = &right_item;
  if (x->is_commutative() && (left_item.is_stack() || left_item.is_cached()) && 
                              right_item.is_register() && !right_item.is_cached() ) {
      // swap them if left is real stack (or cached) and right is real register(not cached)
    left_arg = &right_item;
    right_arg = &left_item;
  }
  load_item_hint(left_arg, &left_hint);
  // do not need to load right, as we can handle stack and constants
  if ( x->y()->type()->is_constant() || x->op() == Bytecodes::_frem || x->op() == Bytecodes::_drem) {
    // cannot handle inlined constants
    load_item(right_arg);
  } else {
    dont_load_item(right_arg);
  }
  rfree(left_arg);
  item_free(right_arg);
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    code_emit()->arithmetic_op_fpu(x->op(), _result, left_arg, right_arg);
  }
  
  if (must_round(x, hint())) {
    spill_item(_result);
    assert(_result->is_spilled(), "spilling failed");
  }
}



// for  _ladd, _lmul, _lsub, _ldiv, _lrem
void ValueGen::do_ArithmeticOp_Long(ArithmeticOp* x) {
  if (x->op() == Bytecodes::_ldiv || x->op() == Bytecodes::_lrem ) {
    // this is a call
    Item left(x->x());
    Item right(x->y());
    right.set_destroys_register(); // actually: only the div_by_zero destroys register
    ValueGen l(&left,  HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);
    if (gen_code()) {
      code_emit()->arithmetic_call_prolog(x->type()); // preserve registers across call
    }
    load_item(&left);  rfree(&left);
    if (gen_code()) {
      code_emit()->push_item(&left);
    }
    load_item(&right); rfree(&right);
    if (gen_code()) {
      code_emit()->push_item(&right);
      if (x->op() == Bytecodes::_ldiv || x->op() == Bytecodes::_lrem) {
        CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
        code_emit()->explicit_div_by_zero_check(&right, &info);
      }
    }
    RInfo reg = rlock_call_result(x);
    set_result(x, reg);
    if (gen_code()) {
      RInfo unused;
      code_emit()->arithmetic_call_op(x->op(), unused);
      code_emit()->arithmetic_call_epilog(x->type()); // restore registers across call
    }
  } else if (x->op() == Bytecodes::_lmul) {
    Item left(x->x());
    Item right(x->y());
    ValueGen l(&left,  HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);
    load_item(&left);  rfree(&left);
    if (gen_code()) {
      code_emit()->push_item(&left);
    }
    load_item(&right); rfree(&right);
    if (gen_code()) {
      code_emit()->push_item(&right);
    }
    RInfo reg = rlock_call_result(x);
    set_result(x, reg);

    if (gen_code()) {
      // (destroys contents of eax, ebx, ecx and edx)
      // eax/edx are locked as result; preserve ebx/ecx
      code_emit()->long_mul();
    }

  } else {
    Item left(x->x());
    Item right(x->y());
    left.set_destroys_register();
    // missing test if instr is commutative and if we should swap
    ValueGen l(&left, HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);
    load_item_hint(&left, hint());
    load_item(&right);
    // WARNING: for longs, we must guarantee (algorithmically) that the locked lo result
    //          register is not the same as the left-HI register , otherwise we 
    //          would overwrite the results.
    // 
    rfree(&left);
    item_free(&right);
    RInfo reg = rlock(x, hint());
    set_result(x, reg);
    if (gen_code()) {
      code_emit()->arithmetic_op_long(x->op(), _result, &left, &right);
    }
  }
}



// for: _iadd, _imul, _isub, _idiv, _irem
void ValueGen::do_ArithmeticOp_Int(ArithmeticOp* x) {
  if (x->op() == Bytecodes::_idiv || x->op() == Bytecodes::_irem) {
    // The requirements for division and modulo
    // input : eax: dividend                         min_int
    //         reg: divisor   (may not be eax/edx)   -1
    //
    // output: eax: quotient  (= eax idiv reg)       min_int
    //         edx: remainder (= eax irem reg)       0

    // eax and edx will be destroyed

    Item left(x->x());
    Item right(x->y());
    left.set_destroys_register();
    right.set_destroys_register();
    HintItem hint(x->type(), divInRInfo()); // eax
    // Note: does this invalidate the spec ???
    ValueGen r(&right, HintItem::no_hint(), this);
    ValueGen l(&left , &hint,       this);   // visit left second, so that the is_register test is valid
    // By forcing left in eax and right in edx, we guarantee that the
    // registers are available
    if (left.is_register() && left.get_register().is_same(hint.get_register())) {
      if (ra()->get_register_rc(left.get_register()) > 1) {
        // somebody else is using the register that we are going to destroy
        assert(x->x()->is_root(), "we must spill the root item");
        spill_item(x->x()->item());
        load_item_force(&left, hint.get_register());  // load "left" into eax
      } else {
        dont_load_item(&left);
      }
    } else {
      load_item_force(&left, hint.get_register()); // load "left" into eax 
    }

    { HideReg hr(this, remOutRInfo(), true); // needed for computation
      load_item(&right);
      item_free(&left);
      item_free(&right);
    }

    if (x->op() == Bytecodes::_idiv) {
      lock_spill_rinfo(x, divOutRInfo(), x->use_count());
      set_result(x, divOutRInfo());
    } else {
      lock_spill_rinfo(x, remOutRInfo(), x->use_count());
      set_result(x, remOutRInfo());
    }
    if (gen_code()) {
      if (!ImplicitDiv0Checks) {
        CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
        code_emit()->explicit_div_by_zero_check(&right, &info);
      }
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
      RInfo tmp; // not used
      code_emit()->arithmetic_idiv(x->op(), _result, &left, &right, tmp, &info);
    }
  } else {
    Item left_item(x->x());
    Item right_item(x->y());
    // missing test if instr is commutative and if we should swap
    ValueGen l(&left_item, hint(), this);
    ValueGen r(&right_item, HintItem::no_hint(), this);
    Item* left_arg = &left_item;
    Item* right_arg = &right_item;
    if (x->is_commutative() && (left_item.is_stack() || left_item.is_cached()) && 
                                right_item.is_register() && !right_item.is_cached() ) {
      // swap them if left is real stack (or cached) and right is real register(not cached)
      left_arg = &right_item;
      right_arg = &left_item;
    }
    left_arg->set_destroys_register();
    load_item_hint(left_arg, hint());
    // do not need to load right, as we can handle stack and constants
    if (x->op() == Bytecodes::_imul ) {
      // check if we can use shift instead
      RInfo tmp;
      bool use_constant = false;
      bool use_tmp = false;
      if (right_arg->is_constant()) {
        int iconst = right_arg->get_jint_constant();
        if (iconst > 0) {
          if (is_power_of_2(iconst)) {
            use_constant = true;
          } else if (is_power_of_2(iconst - 1) || is_power_of_2(iconst + 1)) {
            use_constant = true;
            use_tmp = true;
          }
        }
      }
      if (use_constant) {
        dont_load_item(right_arg);
      } else {
        load_item(right_arg);
      }
      if (use_tmp) {
        tmp = get_free_rinfo(intType);
      }
      rfree(left_arg);
      item_free(right_arg);
      RInfo reg = rlock(x, hint());
      set_result(x, reg);
      if (gen_code()) {
        code_emit()->arithmetic_op_int(x->op(), _result, left_arg, right_arg, tmp);
      }
    } else {
      dont_load_item(right_arg);
      rfree(left_arg);
      item_free(right_arg);
      RInfo reg = rlock(x, hint());
      set_result(x, reg);
      if (gen_code()) {
        RInfo tmp;
        code_emit()->arithmetic_op_int(x->op(), _result, left_arg, right_arg, tmp);
      }
    }
  }
}


// Call this for arithmetic expressions of form : x = x op y
// This function is called from store op;
// r is valid if we are storing into a cached value
void ValueGen::do_ShortArithmeticOp(ArithmeticOp* x, RInfo r) {
  assert(x->op() == Bytecodes::_iadd || x->op() == Bytecodes::_isub, "not implemented for other bytecodes yet");

  assert(!hint()->has_hint() && !x->is_root(), "what hint, what root??");
  if (r.is_valid()) {
    // it is cached, left is cached value, and will be destroyed by the successor StorLocal
    // in order to release register from left, we must visit it and free it
    Item left(x->x());
    Item right(x->y());
    left.set_destroys_register();
    ValueGen right_gen(&right, HintItem::no_hint(), this);
    ValueGen left_gen(&left, HintItem::no_hint(), this);
    dont_load_item(&right);
    dont_load_item_nocheck(&left); // use nocheck because we may fail the assertion as we overwrite; but that is fine as it has been checked

    item_free(&right);
    item_free(&left);

    RInfo tmp;
    // left_direct is used to pass in the operation
    HintItem left_direct(x->x()->type(), r);
    left_direct.set_destroys_register();
    if (gen_code()) {
      code_emit()->arithmetic_op_int(x->op(), &left_direct, &left_direct, &right, tmp);
    }
  } else {
    Item left(x->x());
    Item right(x->y());
    left.set_destroys_register();
    // Visit first right
    ValueGen rgen(&right, HintItem::no_hint(), this);
    ValueGen lgen(&left,  HintItem::no_hint(), this);
    dont_load_item(&right);
    dont_load_item_nocheck(&left); // use nocheck because we may fail the assertion as we overwrite; but that is fine as it has been checked

    RInfo tmp;
    if (left.is_register()) {
      // cached?? do not copy!
    } else {
      // check if actually it is inc or dec operation
      load_item(&left);
    }

    item_free(&right);
    item_free(&left);

    assert(left.is_register(), "invalid form");
    x->item()->set_register(left.get_register());
    if (gen_code()) {
      code_emit()->arithmetic_op_int(x->op(), &left, &left, &right, tmp);
    }
  }
}


void ValueGen::do_ArithmeticOp(ArithmeticOp* x) {
  ValueTag tag = x->type()->tag();
  assert(x->x()->type()->tag() == tag && x->y()->type()->tag() == tag, "wrong parameters");
  switch (tag) {
    case floatTag:
    case doubleTag:  do_ArithmeticOp_FPU(x);  return;
    case longTag:    do_ArithmeticOp_Long(x); return;
    case intTag:     do_ArithmeticOp_Int(x);  return;
  }
  ShouldNotReachHere();
}


// _ishl, _lshl, _ishr, _lshr, _iushr, _lushr
void ValueGen::do_ShiftOp(ShiftOp* x) {
  Item value(x->x());
  Item count(x->y());
  value.set_destroys_register();
  HintItem count_hint(x->y()->type(), shiftCountRInfo());
  // count must always be in ecx
  ValueGen v(&value, hint(),    this);
  ValueGen c(&count, &count_hint, this);
  ValueTag elemType = x->type()->tag();
  bool must_load_count = !count.is_constant() || elemType == longTag;
  if (must_load_count) {
    // count for long must be in register
    load_item_hint(&count, &count_hint);
  } else {
    dont_load_item(&count);
  }
  load_item_hint(&value, hint());
  RInfo tmp;
  if (elemType == intTag && count.is_register()) {
    // in case we cache the count, we want always to have a register free
    // (without caching, the count may be loaded in ecx; with the caching count,
    // the count register may not be ecx
    tmp = get_free_rinfo(intType);
  }
  item_free(&count);
  rfree(&value);
  RInfo reg = rlock(x, &value);
  set_result(x, reg);
  if (gen_code()) {
    code_emit()->shift_op(x->op(), reg, &value, &count, tmp);
  }
}


// _iand, _land, _ior, _lor, _ixor, _lxor
void ValueGen::do_LogicOp(LogicOp* x) {
  Item left(x->x());
  Item right(x->y());
  left.set_destroys_register();
  // missing test if instr is commutative and if we should swap
  ValueGen l(&left, hint(), this);
  ValueGen r(&right, HintItem::no_hint(), this);
  load_item_hint (&left, hint());
  if (!right.is_constant()) {
    load_item(&right);
  } else {
    dont_load_item(&right);
  }
  rfree(&left);
  if (!right.is_constant()) {
    rfree(&right);
  }
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    code_emit()->logic_op(x->op(), reg, &left, &right);
  }
}



// _lcmp, _fcmpl, _fcmpg, _dcmpl, _dcmpg
void ValueGen::do_CompareOp(CompareOp* x) {
  Item left(x->x());
  Item right(x->y());
  ValueTag tag = x->x()->type()->tag();
  if (tag == floatTag || tag == doubleTag) {
    left.set_destroys_register();
    right.set_destroys_register();
    // missing test if instr is commutative and if we should swap
    HintItem left_hint(x->x()->type()); left_hint.set_from_item(HintItem::no_hint());
    HintItem right_hint(x->y()->type()); right_hint.set_from_item(HintItem::no_hint());
    if (tag == floatTag) {
      left_hint.set_round32();
      right_hint.set_round32();
    }
    ValueGen l(&left,  &left_hint,  this);
    ValueGen r(&right, &right_hint, this);
    load_item(&left);
    load_item(&right);
  } else {
    if (tag == longTag) {
      left.set_destroys_register();
    }
    // NOTE: missing test if instr is commutative and if we should swap (optimization)
    ValueGen l(&left,  HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);
    load_item(&left);
    load_item(&right);
  }
  rfree(&left);
  rfree(&right);
  RInfo reg = rlock(x, hint());
  set_result(x, reg);
  if (gen_code()) {
    code_emit()->compare_op(x->op(), reg, &left, &right);
  }
}


// Code for  :  x->x() {x->cond()} x->y() ? x->tval() : x->fval()
void ValueGen::do_IfOp(IfOp* x) {
  Item left(x->x());
  Item right(x->y());
  { ValueTag tag = x->x()->type()->tag();
    assert(tag == intTag || tag == objectTag, "cannot handle others");
  }
  { ValueTag tag = x->tval()->type()->tag();
    assert(tag == addressTag || tag == intTag || tag == objectTag || tag == longTag, "cannot handle others");
    assert(x->tval()->type()->tag() == x->fval()->type()->tag(), "cannot handle others");
  }
  HintItem left_hint(x->x()->type());  left_hint.set_from_item (HintItem::no_hint());
  HintItem right_hint(x->y()->type()); right_hint.set_from_item(HintItem::no_hint());
  if (x->x()->type()->is_float()) {
    left_hint.set_round32();
    right_hint.set_round32();
  }

  ValueGen l(&left,  &left_hint, this);
  ValueGen r(&right, &right_hint, this);
  load_item(&left);
  load_item(&right);
  rfree(&left);
  rfree(&right);
  if (gen_code()) {
    assert(!code_emit()->is_cc_locked(), "cc register must be unlocked");
    code_emit()->ifop_phase1(&left, &right);
    code_emit()->lock_cc();
  }

  Item t_val(x->tval());
  Item f_val(x->fval());
  // only one of the values will be take
  HintItem tval_hint(x->tval()->type());
  if (x->tval()->as_LoadLocal() != NULL || x->tval()->as_Constant() != NULL) {
    tval_hint.set_from_item(hint());
  } else {
    tval_hint.set_from_item(HintItem::no_hint());
  }
  HintItem fval_hint(x->fval()->type());
  if (x->fval()->as_LoadLocal() != NULL || x->fval()->as_Constant() != NULL) {
    fval_hint.set_from_item(hint());
  } else {
    fval_hint.set_from_item(HintItem::no_hint());
  }
  ValueGen tv(&t_val, &tval_hint, this);  // use hint() if we make sure that input is local or constant
  ValueGen fv(&f_val, &fval_hint, this);  // use hint() if we make sure that input is local or constant
  dont_load_item(&t_val);
  dont_load_item(&f_val);
  RInfo reg;
  if (x->fval()->type()->tag() == longTag) {
    // must lock before releasing
    reg = rlock(x, hint());
    set_result(x, reg);
  }
  item_free(&t_val);
  item_free(&f_val);
  if (x->fval()->type()->tag() != longTag) {
    reg = rlock(x, hint());
    set_result(x, reg);
  }
  if (gen_code()) {
    code_emit()->ifop_phase2(reg, &t_val, &f_val, x->cond());
    code_emit()->unlock_cc();
  }
}


void ValueGen::do_Intrinsic(Intrinsic* x) {
  if (   x->id() == methodOopDesc::_dsqrt 
      || x->id() == methodOopDesc::_dsin 
      || x->id() == methodOopDesc::_dcos ) {
    assert(x->number_of_arguments() == 1, "wrong type");
    Item value(x->argument_at(0));
    value.set_destroys_register();
    ValueGen v(&value, hint(), this);
    load_item_hint(&value, hint());
    rfree(&value);
    RInfo reg = rlock(x, hint());
    set_result(x, reg);
    if (gen_code()) {
      code_emit()->math_intrinsic(x->id(), reg, &value);
    }
    if (must_round(x, hint())) {
      spill_item(_result);
      assert(_result->is_spilled(), "spilling failed");
    }
  } else if (x->id() == methodOopDesc::_arraycopy) {
    spill_values_on_stack(x->stack());
    assert(x->number_of_arguments() == 5, "wrong type");
    Item src     (x->argument_at(0));
    Item src_pos (x->argument_at(1));
    Item dst     (x->argument_at(2));
    Item dst_pos (x->argument_at(3));
    Item length  (x->argument_at(4));
    ValueGen src_v(&src, HintItem::no_hint(), this);
    ValueGen src_pos_v(&src_pos, HintItem::no_hint(), this);
    ValueGen dst_v(&dst, HintItem::no_hint(), this);
    ValueGen dst_pos_v(&dst_pos, HintItem::no_hint(), this);
    ValueGen length_v(&length, HintItem::no_hint(), this);

    load_item(&src);
    if (!src_pos.is_constant()) {
      load_item(&src_pos); // can be constant
    } 
    load_item(&dst);
    if (!dst_pos.is_constant()) {
      load_item(&dst_pos);
    }
    load_item(&length);
    RInfo tmp = get_free_rinfo(intType);
    item_free(&src);
    item_free(&src_pos);
    item_free(&dst);
    item_free(&dst_pos);
    item_free(&length);
    set_no_result(x);
    if (gen_code()) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack()); // we may want to have stack (deoptimization?)
      RInfo unused;
      code_emit()->arraycopy(&src, &src_pos, &dst, &dst_pos, &length, tmp, unused, unused, &info); // does add_safepoint
    }
  } else {
    Unimplemented();
  }
}


// _i2l, _i2f, _i2d, _l2i, _l2f, _l2d, _f2i, _f2l, _f2d, _d2i, _d2l, _d2f
// _i2b, _i2c, _i2s
void ValueGen::do_Convert(Convert* x) {
  HintItem value_hint(x->value()->type()); value_hint.set_from_item(HintItem::no_hint());
  if (x->value()->type()->is_float()) {
    value_hint.set_round32();
  }
  if (x->op() == Bytecodes::_f2i || x->op() == Bytecodes::_d2i) {
    Item value(x->value());
    ValueGen v(&value, &value_hint, this);
    value.set_destroys_register();
    load_item(&value);
    rfree(&value);
    RInfo reg = rlock_call_result(x);
    set_result(x, reg);
    if (gen_code()) {
      code_emit()->fast_call_convert_op(x->op(), &value, reg, is_32bit_mode());
    }
  } else if (x->op() == Bytecodes::_l2f || x->op() == Bytecodes::_l2d ||
      x->op() == Bytecodes::_f2l || x->op() == Bytecodes::_d2l ) {
    // must call into runtime
    Item value(x->value());
    value.handle_float_kind();
    ValueGen v(&value, HintItem::no_hint(), this);
    if (gen_code()) {
      code_emit()->call_convert_prolog(0, x->type());
    }
    load_item(&value);
    rfree(&value);
    RInfo reg = rlock_call_result(x);
    set_result(x, reg);
    if (gen_code()) {
      code_emit()->push_item(&value);
      RInfo unused;
      code_emit()->call_convert_op(x->op(), unused);
      code_emit()->call_convert_epilog(0, x->type());
      if (x->type()->is_float() || x->type()->is_double()) {
        code_emit()->set_fpu_result(_result->get_register());
      }
    }
    if (x->op() == Bytecodes::_l2d || x->op() == Bytecodes::_l2f) {
      // round double or float
      // Note: This may not be necessary if not strictFP; however, some JCK tests fail (Srdjan check it out 5-21-99)
      spill_item(_result);
      assert(_result->is_spilled(), "spilling failed");
    }
  } else {
    Item value(x->value());
    value.handle_float_kind();
    ValueGen v(&value, &value_hint, this);
    if (x->op() == Bytecodes::_i2b) {
      load_byte_item(&value);
    } else if (value.is_constant()) {
      load_item(&value);
    } else if (x->op() != Bytecodes::_i2f && x->op() != Bytecodes::_i2d) {
      load_item(&value);
    } else {
      dont_load_item(&value);
    }
    item_free(&value);
    RInfo reg = rlock(x);
    set_result(x, reg);
    if (gen_code()) {
      code_emit()->convert_op(x->op(), reg, &value);
    }
    if ( x->op() == Bytecodes::_d2f || must_round(x, hint())) {
      // Note: This may not be necessary for d2f if not strictFP; however, some JCK tests fail (Srdjan check it out 5-21-99)
      spill_item(_result);
      assert(_result->is_spilled(), "spilling failed");
    }
  }
}


void ValueGen::do_JsrContinuation(JsrContinuation* x) {
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
    code_emit()->add_nop_with_info(&info);
  }
}


void ValueGen::do_NullCheck(NullCheck* x) {
  Item value(x->obj());
  ValueGen v(&value, HintItem::no_hint(), this);
  load_item(&value);
  item_free(&value);
  if (gen_code() && item_may_be_null(&value)) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
    code_emit()->implicit_null_check(&value, &info);
  }
}


void ValueGen::do_LoopEnter(LoopEnter* x) {
  if (gen_code()) {
    BlockItem* bi = block()->block_item();
    // setting precision
    bool instr_added = false;
    if (bi !=NULL && bi->is_32bit_precision()) {
      code_emit()->set_32bit_fpu_precision();
      instr_added = true;
    }
    if (bi == NULL || bi->map_length() == 0) {
      // loop optimization failed (we should not have added the blocks at all)
      // must emit some code, otherwise jump to jump elimination will cause bad code
      if (!instr_added) code_emit()->add_nop();
    } else {
      int n = bi->map_length() - 1;
      for(; n >= 0; n--) {
        RInfo reg = bi->map_local(n);
        if (!reg.is_illegal()) {
          assert(reg.is_word(), "cannot handle others yet");
          code_emit()->local_2_reg(reg, n);
        }
      }
    }
  }
}


void ValueGen::do_LoopExit(LoopExit* x) {
  if (gen_code()) {
    BlockItem* bi = block()->block_item();
    bool instr_added = false;
    if (bi !=NULL && bi->is_32bit_precision()) {
      code_emit()->restore_fpu_precision();
      instr_added = true;
    }
    if (bi == NULL || bi->map_length() == 0) {
      // must emit some code, otherwise jump to jump elimination will cause bad code
      if (!instr_added) code_emit()->add_nop();
    } else {
      int n = bi->map_length() - 1;
      for(; n >= 0; n--) {
        RInfo reg = bi->map_local(n);
        if (!reg.is_illegal()) {
          assert(reg.is_word(), "cannot handle others yet");
          code_emit()->reg_2_local(n, reg);
        }
      }
    }
  }
}


// handle arguments, without receiver
void ValueGen::invoke_do_arguments(Invoke* x) {
  // handle arguments if any
  for (int i = 0; i < x->number_of_arguments(); i++) {
    Item param(x->argument_at(i));
    param.handle_float_kind();
    ValueGen p(&param, HintItem::no_hint(), this);
    // here comes special handling for constants and addresses (this)
    ValueTag tag = param.type()->tag();
    RInfo helper_reg; /// used to move double arguments
    if (tag == longTag) {
      // This should be done for doubles too, but using CPU registers
      load_item(&param);
    } else {
      // must do that because of item_free
      dont_load_item(&param);
      // no difference was observed using the outcommented optimization
      
      if ((tag == doubleTag || tag == floatTag) && param.is_stack()) { // implement it also for constants !
        ValueType* type = NULL;
        if (tag == floatTag) {
          type = intType;
        } else {
          type = longType;
        }
        helper_reg = get_free_rinfo(type);
      }      
    }

    item_free(&param);
    if (gen_code()) {
      if (helper_reg.is_illegal()) {
        code_emit()->push_item(&param);
      } else {
        code_emit()->push_item_with_rinfo(&param, helper_reg);
      }
    }
  }
}


// Visits all arguments, returns appropriate items without loading them
GrowableArray<Item*>* ValueGen::invoke_visit_arguments(Invoke* x, CallingConvention* unused) {
  GrowableArray<Item*>* argument_items = new GrowableArray<Item*>();
  for (int i = 0; i < x->number_of_arguments(); i++) {
    Item* param = new Item(x->argument_at(i));
    argument_items->append(param);
    ValueGen p(param, HintItem::no_hint(), this);
  }
  return argument_items;
}


void ValueGen::invoke_load_arguments(Invoke* x, GrowableArray<Item*>* args, CallingConvention* unused) {
  // we need to lock a register so that we can transport values on stack
  for (int i = 0; i < x->number_of_arguments(); i++) {
    Item* param = args->at(i);
    param->handle_float_kind();
    ValueTag tag = param->type()->tag();
    RInfo helper_reg; /// used to move double arguments
    if (tag == longTag) {
      // This should be done for doubles too, but using CPU registers
      load_item(param);
    } else {
      dont_load_item(param);
      // Check this out again: no difference was observed using the outcommented optimization
      /*
      if (tag == doubleTag && param->is_stack()) { // implement it also for constants !
        helper_reg = lock_free_rinfo(NULL, longType);
        ra()->free_reg(helper_reg);
      }
      */
    }
    item_free(param);

    if (gen_code()) {
      if (helper_reg.is_illegal()) {
        code_emit()->push_item(param);
      } else {
        code_emit()->push_item_with_rinfo(param, helper_reg);
      }
    }
    delete param;
  }
}


void ValueGen::invoke_do_spill(Invoke* x, RInfo hide_reg) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());
}


void ValueGen::invoke_do_result(Invoke* x, bool needs_null_check, Item* receiver) {
  assert(ra()->are_all_registers_free(), "all registers must be free across calls");
  // setup result register
  if (x->type()->is_void()) {
    set_no_result(x);
  } else {
    RInfo reg = rlock_call_result(x);
    set_result(x, reg);
  }
  // emit invoke code
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack());
    bool optimized = x->target_is_loaded() && x->target_is_final();
    code_emit()->call_op(x->code(), NULL, x->size_of_arguments(), &info, optimized, needs_null_check, NULL); // does add_safepoint
    if (x->type()->is_float() || x->type()->is_double()) {
      code_emit()->set_fpu_result(_result->get_register());
    }
  }
}


void ValueGen::do_Invoke(Invoke* x) {
  // We want to spill roots only late so that any roots with use count > 1, may have
  // its registers used before being spilled
  // invoke_do_spill(x);
  if (x->has_receiver()) {
    bool needs_null_check = false;
    HintItem hint(x->receiver()->type(), receiverRInfo());
    Item receiver(x->receiver());
    // make sure that ref count of register holding the receiver has use
    // use count 1 by marking receiver as register-destroying
    receiver.set_destroys_register(); 
    ValueGen r(&receiver, &hint, this);
    // visit first all arguments, so that the register containing receiver (and is going to be
    // hidden) will certainly not be needed; we hide the receiver with NULL instruction, and
    // therefore we cannot spill it if it is needed by somebody else
    GrowableArray<Item*>* args = invoke_visit_arguments(x, NULL);
    load_item_hint(&receiver, &hint);
    rfree(&receiver);
    if (gen_code()) {
      if (!ImplicitNullChecks) {
        CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
        code_emit()->explicit_null_check(&receiver, &info);
      } 
      code_emit()->push_item(&receiver);
      if (item_may_be_null(&receiver)) {
        if (x->code() == Bytecodes::_invokespecial || 
            !x->target_is_loaded() || x->target_is_final()) {
          // emit receiver NULL check if the class of target is not loaded (i.e., we cannot know if it is final)
          // or if the target us final
          if (PrintNotLoaded && !x->target_is_loaded()) {
            tty->print_cr("   ### class not loaded at invoke bci %d", x->bci());
          }
          needs_null_check = true;
          // CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
          // code_emit()->implicit_null_check(&receiver, &info);
        }
      }
    }
    // After freeing the register used by receiver, we are going to lock it with HideReg;
    // By specifying the Invoke as the locking instructions, we guarantee that it will not be
    // spilled (the spill order is defined by the bci of instructions)
    { HideReg reg(this, receiver.get_register());
      invoke_load_arguments(x, args, NULL);
    }
    // the invoke spill does not lock new registers, therefore
    // it is safe to spill while receiver register is not locked
    debug_only(ra()->lock_locking(true);)
    invoke_do_spill(x);
    debug_only(ra()->lock_locking(false);)
    if (!receiver.get_register().is_same(hint.get_register())) {
      if (gen_code()) {
        code_emit()->item_2_reg(hint.get_register(), &receiver);
      }
    }
    invoke_do_result(x, needs_null_check);
  } else {
    invoke_do_arguments(x);
    invoke_do_spill(x);
    invoke_do_result(x, false);
  }
}


void ValueGen::do_NewInstance(NewInstance* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());
  assert(ra()->are_all_registers_free(), "all registers must be free across calls");
  RInfo reg = result_register_for(x->type());
  ra()->lock_reg(x, reg, x->use_count());
  set_result(x, reg);
  RInfo tmp1;
  RInfo tmp2;
  RInfo tmp3;
  RInfo klassR;
  {
    HideReg hr1(this, objectType); tmp1 = hr1.reg();
    HideReg hr2(this, objectType); tmp2 = hr2.reg();
    HideReg hr3(this, objectType); tmp3 = hr3.reg();
    HideReg hr4(this, objectType); klassR = hr4.reg();

  }
  if (gen_code()) {
    if (PrintNotLoaded && !x->klass()->is_loaded()) {
      tty->print_cr("   ###class not loaded at new bci %d", x->bci());
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack());
    code_emit()->new_instance(reg, x->klass(), tmp1, tmp2, tmp3, klassR, &info);
  }
}


void ValueGen::do_NewTypeArray(NewTypeArray* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());
  Item length(x->length());
  ValueGen l(&length, HintItem::no_hint(), this);
  load_item(&length);
  rfree(&length);
  assert(ra()->are_all_registers_free(), "all registers must be free across calls");
  RInfo reg = result_register_for(x->type());
  ra()->lock_reg(x, reg, x->use_count());
  set_result(x, reg);
  RInfo tmp1;
  RInfo tmp2;
  RInfo klassR;
  {
    HideReg hr1(this, objectType); tmp1 = hr1.reg();
    HideReg hr2(this, objectType); tmp2 = hr2.reg();
    HideReg hr3(this, objectType); klassR = hr3.reg();
  }
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack());
    RInfo unused;
    code_emit()->new_type_array(reg, x->elt_type(), &length, tmp1, tmp2, unused, unused, klassR, &info);
  }
}


void ValueGen::do_NewObjectArray(NewObjectArray* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());
  Item length(x->length());
  ValueGen l(&length, HintItem::no_hint(), this);
  load_item(&length);
  rfree(&length);
  assert(ra()->are_all_registers_free(), "all registers must be free across calls");
  RInfo reg = result_register_for(x->type());
  ra()->lock_reg(x, reg, x->use_count());
  set_result(x, reg);
  RInfo tmp1;
  RInfo tmp2;
  RInfo tmp3;
  {
    HideReg hr1(this, objectType); tmp1 = hr1.reg();
    HideReg hr2(this, objectType); tmp2 = hr2.reg();
    HideReg hr3(this, objectType); tmp3 = hr3.reg();
  }
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack());
    RInfo unused;
    code_emit()->new_object_array(reg, x->klass(), &length, tmp1, tmp2, tmp3, unused, unused, &info);
  }
}


void ValueGen::do_NewMultiArray(NewMultiArray* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());
  Values* dims = x->dims();
  int i = dims->length();
  while (i-- > 0) {
    Item size(dims->at(i));
    ValueGen s(&size, HintItem::no_hint(), this);
    load_item(&size);
    rfree(&size);
    if (gen_code()) {
      code_emit()->push_item(&size);
    }
  }
  assert(ra()->are_all_registers_free(), "all registers must be free across calls");
  RInfo reg = result_register_for(x->type());
  ra()->lock_reg(x, reg, x->use_count());
  set_result(x, reg);
  RInfo tmp = get_free_rinfo(intType);
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack());
    code_emit()->new_multi_array(reg, x->klass(), x->rank(), tmp, &info);
  }
}


void ValueGen::do_BlockBegin(BlockBegin* x) {
  // nothing to do for now
}


void ValueGen::do_Throw(Throw* x) {
  if (!x->stack()->stack_is_empty()) {
    // release all roots (computed values)
    release_roots(x->stack());
  }
  Item exception(x->exception());
  {
    ValueGen e(&exception, HintItem::no_hint(), this);
    load_item(&exception);
    rfree(&exception);
    set_no_result(x);
  }
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), NULL, NULL); 
      code_emit()->explicit_null_check(&exception, &info);
    }
    CodeEmitInfo info(x->bci(), NULL, NULL);
    code_emit()->throw_op(&exception, &info);
  }
  assert(ra()->are_all_free(),"Unimplemented: should just release all registers and spilled locations");
  assert(!gen_code() || code_emit()->is_empty_fpu_stack(), "still some values on FPu stack");
}


void ValueGen::do_CheckCast(CheckCast* x) {
  spill_values_on_stack(x->stack());
  Item obj(x->obj());
  obj.set_destroys_register();
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item(&obj);
  const RInfoCollection* oop_regs = NULL;
  if (gen_code() && !x->klass()->is_loaded()) {
    // must do this before locking the destination register as an oop register
    oop_regs = get_registers_with_oops();
  }
  RInfo reg = rlock(x);
  set_result(x, reg);
  RInfo tmp = get_free_rinfo(objectType);

  // make sure dst_reg is not equal as obj_reg
  rfree(&obj);
  assert(!obj.get_register().is_same(reg), "check");
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    RInfo unused;
    code_emit()->checkcast_op(reg, &obj, x->klass(), unused, tmp, &info);
  }
}


void ValueGen::do_InstanceOf(InstanceOf* x) {
  spill_values_on_stack(x->stack());
  Item obj(x->obj());
  obj.set_destroys_register();
  ValueGen o(&obj, HintItem::no_hint(), this);
  // result and test object may not be in same register
  RInfo reg = rlock(x);
  set_result(x, reg);
  load_item(&obj);
  const RInfoCollection* oop_regs = NULL;
  if (gen_code() && !x->klass()->is_loaded()) {
    // must do this before locking the destination register as an oop register
    oop_regs = get_registers_with_oops();
  }
  RInfo tmp = get_free_rinfo(objectType);
  rfree(&obj);
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    RInfo unused;
    code_emit()->instanceof_op(reg, &obj, x->klass(), unused, tmp, &info);
  }
}


RInfoCollection* ValueGen::get_block_item_oop_regs() {
  BlockItem* bi = block()->block_item();
  RInfoCollection* res = NULL;
  if (bi != NULL) {
    int lng = bi->map_length();
    for (int i = 0; i < lng; i++) {
      if (bi->is_oop(i)) {
        if (res == NULL) res = new RInfoCollection();
        res->append(bi->map_local(i));
      }
    }
  }
  if (res == NULL) {
    return NULL;
  } else {
    return res;
  }
}


RInfoCollection* ValueGen::get_registers_with_oops() {
  RInfoCollection* res = ra()->oops_in_registers();
  return res;
  /*RInfoCollection* tmp = get_block_item_oop_regs();
  if (tmp != NULL) res->appendAll(tmp);
  return res;*/
}


// do a jump to default successot, except if it is the immediate successor in _code (sequence of blocks)
void ValueGen::goto_default_successor(BlockEnd* block_end) {
  bool is_bb = is_backward_branch(block_end, block_end->default_sux());
  const RInfoCollection* oop_regs = NULL;
  if (is_bb) {
    oop_regs = get_block_item_oop_regs(); // for cached locals
  }
  if (block()->weight() + 1 == block_end->default_sux()->weight()) {
    // default successor is next block; no goto necessary
    assert(!is_bb, "cannot be backward branch");
  } else {
    code_emit()->goto_op(block_end->default_sux(), block_end->bci(), is_bb, oop_regs);
  }
}


void ValueGen::do_Goto(Goto* x) {
  bool is_bb = is_backward_branch(x, x->default_sux());
  set_no_result(x);
  move_to_phi(x->stack());
  if (gen_code()) {
    if (is_bb && !x->stack()->stack_is_empty()) {
      // we cannot emit safepoint data if expression stack is empty,
      code_emit()->set_bailout("stack not empty at backward branch");
    }
    // check that this is endless goto (add nop)
    // Note: this is probably not needed as we have switched off EliminateJumpToJumps
    bool endless_loop = is_bb && x->default_sux()->bci() == x->bci();
    if (endless_loop) {
      code_emit()->add_nop();
    }
    goto_default_successor(x);
  }
}


void ValueGen::do_Jsr(Jsr* x) {
  set_no_result(x);
  if (!x->stack()->stack_is_empty()) {
    move_to_phi(x->stack());
  }
  if (gen_code()) {
    if (!x->stack()->stack_is_empty()) {
      code_emit()->set_bailout("stack not empty on jsr");
    }
    code_emit()->jsr_op(x->subroutine(), x->bci());
  }
}


void ValueGen::do_Ret(Ret* x) {
  set_no_result(x);
  if (gen_code()) {
    if (!x->stack()->stack_is_empty()) {
      code_emit()->set_bailout("stack not empty on ret");
    }
    RInfo r;
    cached_cpu_local(x->index(), r);
    code_emit()->ret_op(x->index(), r, x->bci());
  }
}


void ValueGen::do_If(If* x) {
  assert(x->number_of_sux() == 2, "inconsistency");
  ValueTag tag = x->x()->type()->tag();
  bool is_bb = is_backward_branch(x, x->sux_at(0)) || is_backward_branch(x, x->sux_at(1));
  HintItem left_hint(x->x()->type());  left_hint.set_from_item (HintItem::no_hint());
  HintItem right_hint(x->y()->type()); right_hint.set_from_item(HintItem::no_hint());
  if (x->x()->type()->is_float()) {
    left_hint.set_round32();
    right_hint.set_round32();
  }

  Item xitem(x->x());
  Item yitem(x->y());
  Item* xin = &xitem;
  Item* yin = &yitem;
  If::Condition cond = x->cond();
  {  // call destructors before doing move to phi

    ValueGen xg(&xitem, &left_hint, this);
    ValueGen yg(&yitem, &right_hint, this);
    if (tag == longTag) {
      // for longs, only conditions "eql", "neq", "lss", "geq" are valid;
      // mirror for other conditions
      if (cond == If::gtr || cond == If::leq) {
        cond = Instruction::mirror(cond);
        xin = &yitem;
        yin = &xitem;
      }
      xin->set_destroys_register();
    } if (tag == floatTag || tag == doubleTag) {
      xin->set_destroys_register();
      yin->set_destroys_register();
    }
    load_item(xin);
    if (tag == longTag || tag == floatTag || tag == doubleTag) {
      // longs cannot handle constants at right side
      load_item(yin);
    } else {
      dont_load_item(yin);
    }

    rfree(xin);
    item_free(yin);
    set_no_result(x);
  }

  // note that the condition test must happen before the
  // moves into Phi area happen, and that the control flow
  // jump must happen after the moves into the phi area
  assert(!gen_code() || !code_emit()->is_cc_locked(), "cc register must be unlocked");

  const RInfoCollection* oop_regs = NULL;
  if (gen_code() && is_bb) { oop_regs = get_block_item_oop_regs(); }
  for (int phase = 1; phase <= 2; phase++) {
    if (phase == 2)  move_to_phi(x->stack());
    if (gen_code()) {
      if (is_bb && !x->stack()->stack_is_empty()) { code_emit()->set_bailout("stack not empty on backward branch");  }
      // update cond variable for next iteration
      cond = code_emit()->if_op(phase, cond, xin, yin, x->tsux(), x->fsux(), x->usux(),
                         x->bci(), is_bb, oop_regs);
      if (phase == 1) code_emit()->lock_cc();
      else            code_emit()->unlock_cc();
    }
  }
  if (gen_code())  goto_default_successor(x);
  assert(!gen_code() || !code_emit()->is_cc_locked(), "cc must be unlocked");
}


void ValueGen::do_IfInstanceOf(IfInstanceOf* x) {
  Unimplemented();
  spill_values_on_stack(x->stack());
  Item obj (x->obj());
  obj.set_destroys_register();
  ValueGen o(&obj, HintItem::no_hint(), this);
  // result and test object may not be in same register
  RInfo reg = rlock(x);
  set_result(x, reg);
  load_item(&obj);
  const RInfoCollection* oop_regs = NULL;
  if (gen_code() && !x->klass()->is_loaded()) {
    // must do this before locking the destination register as an oop register
    oop_regs = get_registers_with_oops();
  }
  RInfo tmp = get_free_rinfo(objectType);
  rfree(&obj);
  if (gen_code()) {
    CodeEmitInfo info(x->instanceof_bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    code_emit()->instance_of_test_op(reg, &obj, x->klass(), tmp, x->tsux(), x->fsux(), !x->test_is_instance(), &info);
  }
}


// May change the content of tag
void ValueGen::setup_phis_for_switch(Item* tag, ValueStack* stack) {
  // if stack() exists, then move_to_phi may destroy the tag register as
  // the tag register is not locked anymore.
  if (stack != NULL && stack->stack_size() > 0 ) {
    // stack exists, move values into phi locations
    // preserve the tag
    if (gen_code()) {
      // save
      code_emit()->push_item(tag);
    }
    move_to_phi(stack);
    { HideReg hr(this, nonReturnRInfo()); // use one register that is not a result register
      tag->set_register(hr.reg());
      if (gen_code()) {
        // restore (do not use result registers)
        code_emit()->pop_item(tag);
      }
    }
  }
}


void ValueGen::do_TableSwitch(TableSwitch* x) {
  Item tag(x->tag());
  { ValueGen v(&tag, HintItem::no_hint(), this);
    load_item(&tag);
    rfree(&tag);
    set_no_result(x);
  }

  setup_phis_for_switch(&tag, x->stack());
  if (gen_code()) {
    int lo_key = x->lo_key();
    int hi_key = x->hi_key();
    int len = x->length();
    for (int i = 0; i < len && !code_emit()->must_bailout(); i++) {
      int matchVal = i + lo_key;
      bool is_bb = is_backward_branch(x, x->sux_at(i));
      if (is_bb && !x->stack()->stack_is_empty()) {
        // we cannot emit safepoint data if expoession stack is empty,
        code_emit()->set_bailout("stack not empty on tableswitch");
      }
      const RInfoCollection* oop_regs = NULL;
      if (is_bb) {
        oop_regs = get_block_item_oop_regs();
      }
      code_emit()->tableswitch_op(&tag, matchVal, x->sux_at(i), x->bci(), is_bb, oop_regs);
      code_emit()->check_codespace();
    }
    bool is_bb = is_backward_branch(x, x->default_sux());
    if (is_bb && !x->stack()->stack_is_empty()) {
      // we cannot emit safepoint data if expoession stack is empty,
      code_emit()->set_bailout("stack not empty on tableswitch");
    }
    const RInfoCollection* oop_regs = NULL;
    if (is_bb) {
      oop_regs = get_block_item_oop_regs();
    }
    code_emit()->goto_op(x->default_sux(), x->bci(), is_bb, oop_regs);
  }
}


// we expect the keys to be sorted by increasing value
GrowableArray<LookupRange*>* create_lookup_ranges(LookupSwitch* x) {
  GrowableArray<LookupRange*>* res = new GrowableArray<LookupRange*>();
  int len = x->length();
  if (len > 0) {
    int key = x->key_at(0);
    BlockBegin* sux = x->sux_at(0);
    LookupRange* range = new LookupRange(key, sux);
    for (int i = 1; i < len; i++) {
      int new_key = x->key_at(i);
      BlockBegin* new_sux = x->sux_at(i);
      if (key+1 == new_key && sux == new_sux) {
        // still in same range
        range->set_high_key(new_key);
      } else {
        res->append(range);
        range = new LookupRange(new_key, new_sux);
      }
      key = new_key;
      sux = new_sux;
    }
    if (res->length() == 0 || res->last() != range)  res->append(range);
  }
  return res;
}


void ValueGen::do_LookupSwitch_key(Item* tag, LookupSwitch* x, LookupRange* range) {
  bool is_bb = is_backward_branch(x, range->sux());
  if (is_bb && !x->stack()->stack_is_empty()) {
    // we cannot emit safepoint data if expoession stack is empty,
    code_emit()->set_bailout("stack not empty on lookupswitch");
  }
  const RInfoCollection* oop_regs = NULL;
  if (is_bb) {
    oop_regs = get_block_item_oop_regs();
  }
  code_emit()->lookupswitch_range_op(tag, range->low_key(), range->high_key(), range->sux(), x->bci(), is_bb, oop_regs);        
}


void ValueGen::do_LookupSwitch_default(LookupSwitch* x) {
  bool is_bb = is_backward_branch(x, x->default_sux());
  if (is_bb && !x->stack()->stack_is_empty()) {
    // we cannot emit safepoint data if expoession stack is empty,
    code_emit()->set_bailout("stack not empty on lookupswitch");
  }
  const RInfoCollection* oop_regs = NULL;
  if (is_bb) {
    oop_regs = get_block_item_oop_regs();
  }
  code_emit()->goto_op(x->default_sux(), x->bci(), is_bb, oop_regs);
}


void ValueGen::do_LookupSwitch(LookupSwitch* x) {
  Item tag(x->tag());
  { ValueGen v(&tag, HintItem::no_hint(), this);
    load_item(&tag);
    rfree(&tag);
    set_no_result(x);
  }
  setup_phis_for_switch(&tag, x->stack());
  if (UseTableRanges) {
    GrowableArray<LookupRange*>* ranges = create_lookup_ranges(x);
    if (gen_code()) {
      int lng = ranges->length();
      for (int i = 0; i < lng && !code_emit()->must_bailout(); i++) {
        LookupRange* one_range = ranges->at(i);
        do_LookupSwitch_key(&tag, x, one_range);
      }
      do_LookupSwitch_default(x);
      code_emit()->check_codespace();
    } // gen_code
  } else {
    if (gen_code()) {
      int len = x->length();
      for (int i = 0; i < len && !code_emit()->must_bailout(); i++) {
        bool is_bb = is_backward_branch(x, x->sux_at(i));
        if (is_bb && !x->stack()->stack_is_empty()) {
          // we cannot emit safepoint data if expoession stack is empty,
          code_emit()->set_bailout("stack not empty on lookupswitch");
        }
        const RInfoCollection* oop_regs = NULL;
        if (is_bb) {
          oop_regs = get_block_item_oop_regs();
        }
        code_emit()->lookupswitch_op(&tag, x->key_at(i), x->sux_at(i), x->bci(), is_bb, oop_regs);
        code_emit()->check_codespace();
      }
      bool is_bb = is_backward_branch(x, x->default_sux());
      if (is_bb && !x->stack()->stack_is_empty()) {
        // we cannot emit safepoint data if expoession stack is empty,
        code_emit()->set_bailout("stack not empty on lookupswitch");
      }
      const RInfoCollection* oop_regs = NULL;
      if (is_bb) {
        oop_regs = get_block_item_oop_regs();
      }
      code_emit()->goto_op(x->default_sux(), x->bci(), is_bb, oop_regs);
    }
  }
}


void ValueGen::do_Return(Return* x) {
  if (x->type()->is_void()) {
    if (gen_code()) {
      if (method()->is_synchronized()) {
	RInfo receiver;
	// get cached receiver; receiver is illegal if there is no cached receiver
	cached_cpu_local(0, receiver); 
	code_emit()->return_op_prolog(x->monitor_no(), receiver);
      }
      code_emit()->return_op(NULL);
    }
  } else {
    RInfo reg = result_register_for(x->type());
    Item result(x->result());
    result.handle_float_kind();
    HintItem result_hint(x->type(), reg);
    if (x->type()->is_float()) {
      result_hint.set_round32();
    }
    ValueGen r(&result, &result_hint, this);
    bool result_is_local0 = false;
    
    if (!method()->is_static()) {
      LoadLocal* load = x->result()->as_LoadLocal();
      // Note: this is not very safe; it works because the register optimization
      //       will put receiver in register only if nobody else uses
      //       local 0
      if (load != NULL && load->index() == 0) {
        result_is_local0 = true;
      }
    }

    bool result_released = false;
    bool cachedReceiver = result_is_local0 && result.is_register();
    if (cachedReceiver) {   
      // Release result item, which is being held in register, as the 
      // result will be reloaded from the local 0; 
      // otherwise we would unnecesarily spill.
      load_item(&result);
      item_free(&result);
      result_released = true;
    }

    if (x->is_synchronized()) {
      if (result.is_register() && !result_is_local0) {
        // spill the temporary result except if it is the receiver
        spill_value(x->result());
        result.update();
      }
      lock_spill_rinfo(NULL, syncTempRInfo(), 1);
      ra()->free_reg(syncTempRInfo());

      if (gen_code()) {
	RInfo receiver;
	// get cached receiver; receiver is illegal if there is no cached receiver
	cached_cpu_local(0, receiver); 
        code_emit()->return_op_prolog(x->monitor_no(), receiver);
      }
    }

    if (!cachedReceiver) {
      assert(!result_released, "this is the inverted condition A"); 
      if (result.type()->is_long() && result.is_register()) {
        // load_item_force has problems if only one of registers in 'reg'
        // is the same as one of registers in 'result' -> spill it
        RInfo res_reg = result.get_register();
        if (!res_reg.is_same(reg)) {
          if (res_reg.overlaps(reg)) {
            spill_value(x->result());
            result.update();
          }
        }
      }
      load_item_force(&result, reg);
      rfree(&result);
    } else if (!result_released) {
      ShouldNotReachHere(); // result_released must have been set
      item_free(&result);
    }
    if (gen_code()) {
      if (result_is_local0) {
        code_emit()->load_receiver(reg.as_register());
      } 
      code_emit()->return_op(&result);
    }
  }
  set_no_result(x);
}


void ValueGen::do_Base(Base* x) {
  // nothing to do for now
}

