// @(#)c1_CodeStubs_i486.cpp	1.41 00/04/11 10:07:17
//
// Copyright 1991-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_c1_CodeStubs_i486.cpp.incl"


#define __ ce->masm()->


void StaticCallStub::emit_code(CodeEmitter* ce) {
  __ relocate(static_stub_Relocation::spec(_call_pc));
  __ movl(ebx, (jobject)NULL);
  // must be set to -1 at code generation time
  __ jmp((address)-1, relocInfo::runtime_call_type);
}


void RangeCheckStub::emit_code(CodeEmitter* ce) {
  __ bind(_entry);
  __ movl(eax, _pc_offset);
  __ call(Runtime1::entry_for(Runtime1::range_check_failed_id), relocInfo::runtime_call_type);
}


void DivByZeroStub::emit_code(CodeEmitter* ce) {
  __ bind(_entry);
  __ movl(eax, _pc_offset);
  __ call(Runtime1::entry_for(Runtime1::throw_div0_exception_id), relocInfo::runtime_call_type);
}


void NullExceptionStub::emit_code(CodeEmitter* ce) {
  __ bind(_entry);
  __ movl(eax, _pc_offset);
  __ call(Runtime1::entry_for(Runtime1::throw_null_exception_id), relocInfo::runtime_call_type);
}


// Implementation of NewInstanceStub

NewInstanceStub::NewInstanceStub(Register klass, CodeEmitInfo* info, int esp_offset) {
  _klass = klass;
  _info = new CodeEmitInfo(info);
  _esp_offset = esp_offset;
  assert(esp_offset == 0, "not tested");
}


void NewInstanceStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  __ movl(eax, _klass);
  __ call(Runtime1::entry_for(Runtime1::new_instance_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  __ jmp(_continuation);
}


// Implementation of NewTypeArrayStub

NewTypeArrayStub::NewTypeArrayStub(BasicType elt_type, Register length, Register result, CodeEmitInfo* info, int esp_offset) {
  _elt_type = elt_type;
  _length = length;
  _info = new CodeEmitInfo(info);
  _esp_offset = esp_offset;
  assert(esp_offset == 0, "not tested");
}


void NewTypeArrayStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  ce->push_int((int)_elt_type);
  ce->push_reg(_length);
  __ call(Runtime1::entry_for(Runtime1::new_type_array_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  ce->dec_stack_after_call(2);
  __ jmp(_continuation);
}


// Implementation of NewObjectArrayStub

NewObjectArrayStub::NewObjectArrayStub(Register klass, Register length, Register result, CodeEmitInfo* info, int esp_offset) {  
  _klass = klass;
  _result = result;
  _length = length;
  _info = new CodeEmitInfo(info);
  _esp_offset = esp_offset;
  assert(esp_offset == 0, "not tested");
}


void NewObjectArrayStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  ce->push_reg(_klass);
  ce->push_reg(_length);
  __ call(Runtime1::entry_for(Runtime1::new_object_array_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  ce->dec_stack_after_call(2);
  __ jmp(_continuation);
}


// Implementation of MonitorAccessStubs

MonitorEnterStub::MonitorEnterStub(Register obj_reg, Register lock_reg, CodeEmitInfo* info, int esp_offset)
: MonitorAccessStub(obj_reg, lock_reg)
{ _info = new CodeEmitInfo(info);
  _esp_offset = esp_offset;
  assert(esp_offset == 0, "not tested else");
}


void MonitorEnterStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  ce->push_reg(_obj_reg);
  ce->push_reg(_lock_reg);
  __ call(Runtime1::entry_for(Runtime1::monitorenter_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  ce->dec_stack_after_call(2);
  // preserve receiver in ecx in case the method is virtual
  if (!ce->method()->is_static()) {
    // cached receiver is in ecx
    ce->load_receiver(ce->receiver_reg());
  }
  __ jmp(_continuation);
}


void MonitorExitStub::emit_code(CodeEmitter* ce) {
  __ bind(_entry);
  if (_compute_lock) {
    // lock_reg was destroyed by fast unlocking attempt => recompute it
    __ leal(_lock_reg, _lock_addr);
  }
  ce->push_reg(_obj_reg);
  ce->push_reg(_lock_reg);
  // note: non-blocking leaf routine => no call info needed
  __ call(Runtime1::entry_for(Runtime1::monitorexit_id), relocInfo::runtime_call_type);
  ce->dec_stack_after_call(2);
  __ jmp(_continuation);
}


// Implementation of patching: 
// - Copy the code at given offset to an inlined buffer (first the bytes, then the number of bytes)
// - Replace original code with a call to the stub
// At Runtime:
// - call to stub, jump to runtime 
// - in runtime: preserve all registers (especially objects, i.e., source and destination object)
// - in runtime: after initializing class, restore original code, reexecute instruction 

void PatchingStub::emit_code(CodeEmitter* ce) {
  // copy original code here
  // for ( int i = 0; i < NativeCall::instruction_size; i++) {
  assert(NativeCall::instruction_size <= _bytes_to_copy && _bytes_to_copy <= 0xFF, "not enough room for call");
  for ( int i = 0; i < _bytes_to_copy; i++) {
    address ptr = (address)(_pc_start + i);
    int a_byte = (*ptr) & 0xFF;
    __ a_byte (a_byte);
  }
  __ a_byte(_bytes_to_copy);
  address entry = __ pc();
  NativeCall::insert((address)_pc_start, entry);
  address target = NULL;
  switch (_id) {
    case access_field_id:  target = Runtime1::entry_for(Runtime1::init_check_patching_id); break;
    case load_klass_id:    target = Runtime1::entry_for(Runtime1::load_klass_patching_id); break;
    default: ShouldNotReachHere();
  }
  __ jmp(target, relocInfo::runtime_call_type);
  if (_id == load_klass_id) {
    CodeBuffer* cb = __ code();
    RelocIterator iter(cb, (address)_pc_start, (address)(_pc_start + 1));
    relocInfo::change_reloc_info_for_address(&iter, (address) _pc_start, relocInfo::oop_type, relocInfo::none);
  }

}


CheckCastStub::CheckCastStub(Register k, CodeEmitInfo* info, Register dst, int esp_offset): 
_k(k), _info(NULL), _esp_offset(esp_offset), _dst(dst) {
  _info = new CodeEmitInfo(info);
}


void CheckCastStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  ce->push_reg(_dst);
  ce->push_reg (_k);
  __ call  (Runtime1::entry_for(Runtime1::checkcast_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  // object is in eax
  ce->dec_stack_after_call(2);
  // ce->push_reg (eax);
  ce->move_reg(_dst, eax);
  __ jmp(_continuation);
  assert(ce->esp_offset() == _esp_offset, "must be the same when returning");
  ce->set_esp_offset(0);
}


InstanceOfStub::InstanceOfStub(Register obj_klass, Register result, Register k, CodeEmitInfo* info, int esp_offset): 
_obj_klass(obj_klass), _result(result), _k(k), _info(NULL), _esp_offset(esp_offset) {
  _info = new CodeEmitInfo(info);
}


void InstanceOfStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  ce->push_reg (_obj_klass);
  ce->push_reg (_k);
  __ call  (Runtime1::entry_for(Runtime1::instanceof_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  // result is in eax
  ce->dec_stack_after_call(2);
  ce->move_reg (_result, eax);
  __ jmp(_continuation);
  assert(ce->esp_offset() == _esp_offset, "must be the same when returning");
  ce->set_esp_offset(0);
}



ArrayStoreCheckStub::ArrayStoreCheckStub(Register array, Register value, Register index, int index_const, CodeEmitInfo* info, int esp_offset):
_array(array), _value(value), _index(index), _index_const(index_const), _esp_offset(esp_offset) {
  _info = new CodeEmitInfo(info);
}


void ArrayStoreCheckStub::emit_code(CodeEmitter* ce) {
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  ce->push_reg (_value);
  ce->push_reg (_array);
  if (_index == noreg) {
    ce->push_int(_index_const);
  } else {
    ce->push_reg(_index); // index to preserve
  }
  __ call  (Runtime1::entry_for(Runtime1::store_check_id), relocInfo::runtime_call_type);
  ce->add_call_info_here(_info);
  ce->dec_stack(3);
  // Note: runtime stub is preserving the registers, therefore we do not need to pop them
  __ jmp(_continuation);
  assert(ce->esp_offset() == _esp_offset, "must be the same when returning");
  ce->set_esp_offset(0);
}


// Enter: value is on tos of FPU stack
// Exit:  eax contains result
// Note:  preserves all other registers
void ConvertStub::emit_code(CodeEmitter* ce) {
  assert(_result == eax, "wrong result set");
  __ bind(_entry);
  ce->bang_stack();
  __ pushl(esi);
  __ pushl(edi);
  __ pushl(ecx);
  __ pushl(ebx);
  __ pushl(edx);
  __ pushl(edx); // make room for parameter
  __ fstp_s(Address(esp));
  __ call (_runtime, relocInfo::runtime_call_type);
  __ popl (edx); // discard parameter
  __ popl (edx);
  __ popl (ebx);
  __ popl (ecx);
  __ popl (edi);
  __ popl (esi);
  // result is in eax
  __ jmp(_continuation);
}



ArrayCopyStub::ArrayCopyStub(ArrayCopyDesc* desc, CodeEmitInfo* info, int esp_offset):
      _desc(desc), _esp_offset(esp_offset) {
  _info = new CodeEmitInfo(info);
}


void ArrayCopyStub::push_params(CodeEmitter* ce) {
  // push parameters
  ce->push_reg(src());
  if (src_pos_reg() == noreg) {
    ce->push_int(src_pos_const());
  } else {
    ce->push_reg(src_pos_reg());
  }
  ce->push_reg(dst());
  if (dst_pos_reg() == noreg) {
    ce->push_int(dst_pos_const());
  } else {
    ce->push_reg(dst_pos_reg());
  }
  if (length_reg() == noreg) {
    ce->push_int(length_const());
  } else {
    ce->push_reg(length_reg());
  }
}


void ArrayCopyStub::emit_code(CodeEmitter* ce) {
  const int nof_params = 5;
  ce->set_esp_offset(_esp_offset);
  __ bind(_entry);
  Label slowCase;

  //---------------slow case: call to native-----------------
  push_params(ce);
  { StaticCallStub* stub = new StaticCallStub(__ pc(), NULL);
    // This append is OK, as ArrayCopyStub is not part of call_stubs
    ce->_call_stubs->append(stub);
    __ bind(slowCase);
    __ call(Runtime1::entry_for(Runtime1::resolve_invokestatic_id), relocInfo::static_call_type);
    ce->add_call_info(__ offset(), _info);
  }

  assert(!UseC2CallingConventions, "not yet implemented");
  ce->dec_stack_after_call(nof_params);

  __ jmp(_continuation);
  assert(ce->esp_offset() == _esp_offset, "must be the same when returning");
  ce->set_esp_offset(0);

  //-----------------fast case: call to runtime, no checks -----------------------------------------
  // for array copy without checks
  ce->set_esp_offset(_esp_offset);
  __ bind(_noCheckEntry);
  push_params(ce);
  // make a copy in case simple_array_copy_destroys it
  // no esp offset needed
  push_params(ce);
  __ call (CAST_FROM_FN_PTR(address, SharedRuntime::simple_array_copy_no_checks), relocInfo::runtime_call_type);
  ce->dec_stack(nof_params); // remove the copy of parameters; leave first parameters in case of slow case

  // result is in eax: -1 means -> failed
  { __ testl(eax, eax);
    __ jcc (Assembler::negative, slowCase);
    // fast case
    ce->dec_stack(nof_params);
    __ jmp(_continuation);
    assert(ce->esp_offset() == _esp_offset, "must be the same when returning");
    ce->set_esp_offset(0);
  }

}


// State at entry:
//   C2 flag is set, value is on TOS of FPU stack.
// 
//   Note: some values in CPU registers may be alive
void MathStub::emit_code(CodeEmitter* ce) {
  __ bind(_entry);
  ce->bang_stack();
  __ pushad();
  __ addl (esp, -2*wordSize);
  __ fstp_d(Address(esp));
  __ call (_rt_addr, relocInfo::runtime_call_type); 
  __ addl (esp, 2*wordSize);
  __ popad();
  __ jmp(_continuation);
}


#undef __
