// @(#)c1_FrameMap_i486.cpp	1.39 00/03/09 11:08:09
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_FrameMap_i486.cpp.incl"



// some useful constant RInfo's:
RInfo FrameMap::_noRInfo;
RInfo FrameMap::_esiRInfo;
RInfo FrameMap::_ediRInfo;
RInfo FrameMap::_ebxRInfo;
RInfo FrameMap::_eaxRInfo;
RInfo FrameMap::_edxRInfo;
RInfo FrameMap::_ecxRInfo;
RInfo FrameMap::_eax_edxRInfo;
RInfo FrameMap::_f0RInfo;
RInfo FrameMap::_d0RInfo;

c1_RegMask FrameMap::_byte_reg_mask;


//--------------------------------------------------------
//               FrameMap::FpuStack
//--------------------------------------------------------


FrameMap::FpuStack::FpuStack() {
  for (int i = 0; i < nof_fpu_regs; i++) _offset[i] = 0;
  _nof_elems = 0;
}


int FrameMap::FpuStack::find_tos(int offset) {
  int goal = _nof_elems - offset;
  for (int i = 0; i < nof_fpu_regs; i++) {
    if (_offset[i] == goal) return i;
  }
  return -1;
}


void FrameMap::FpuStack::swap(int a, int b) {
  debug_only(fpu_range_check(a);)
  debug_only(fpu_range_check(b);)
  int temp   = _offset[a];
  _offset[a] = _offset[b];
  _offset[b] = temp;
}


int FrameMap::FpuStack::bring_on_tos_0(int rnr) {
  debug_only(fpu_range_check(rnr);)
  assert(_nof_elems > 0 && _offset[rnr] > 0, "rnr is not on stack");
  if (_offset[rnr] != _nof_elems) {
    int tos_reg = find_tos(0);
    swap (tos_reg, rnr);
    return _nof_elems - _offset[tos_reg];
  }
  assert(_offset[rnr] == _nof_elems, "wrong result");
  return 0;
}


void FrameMap::FpuStack::push (int rnr) {
  debug_only(fpu_range_check(rnr);)
  assert( _nof_elems < nof_fpu_regs, "stack is full");
  _nof_elems++;
  _offset[rnr] = _nof_elems;
}


void FrameMap::FpuStack::pop (int rnr) {
  debug_only(fpu_range_check(rnr);)
  assert(_nof_elems > 0, "stack empty");
  assert(_offset[rnr] == _nof_elems, "register is not on TOS: cannot pop");
  _nof_elems--;
  _offset[rnr] = 0;
}


void FrameMap::FpuStack::swap_two_on_tos () {
  assert(_nof_elems >= 2, "not enough space");
  int tos_0 = -1;
  int tos_1 = -1;
  int goal_0 = _nof_elems;
  int goal_1 = _nof_elems - 1;
  for (int i = 0; i < nof_fpu_regs; i++) {
    if (_offset[i] == goal_0) tos_0 = i;
    if (_offset[i] == goal_1) tos_1 = i;
  }
  swap (tos_0, tos_1);
}


bool FrameMap::FpuStack::is_stack_pos(int rnr, int offset) {
  debug_only(fpu_range_check(rnr);)
  assert(_nof_elems > 0 && offset < _nof_elems, "stack is empty or wrong offset");
  return _offset[rnr] == _nof_elems - offset;
}


bool FrameMap::FpuStack::is_empty() {
#ifdef ASSERT
  if (_nof_elems == 0) {
    for (int i = 0; i < nof_fpu_regs; i++) assert(_offset[i] == 0, "illegal state");
  }
#endif
  return _nof_elems == 0;
}


void FrameMap::FpuStack::clear() {
  for (int i = 0; i < nof_fpu_regs; i++) _offset[i] = 0;
  _nof_elems = 0;
}


#ifndef PRODUCT

void FrameMap::FpuStack::print() {
  tty->print(" N=%d[", _nof_elems);
  for (int i = 0; i < nof_fpu_regs; i++) tty->print(" %d", _offset[i]);
  tty->print(" ]");
}

#endif



//--------------------------------------------------------
//               FrameMap
//--------------------------------------------------------


FrameMap::FrameMap(int size_spill, const CachedLocals* cached_locals) {
  _size_locals = 0;
  _size_monitors = 0;
  _size_arguments = 0;
  _size_spills = size_spill;
  _fixed_framesize = -1;
  _max_stack_size = 0;
  _cached_locals = cached_locals;
  if (!_init_done) init();
}


void FrameMap::init() {
  if (_init_done) return;

  assert(nof_cpu_regs == 6, "wrong number of CPU registers");
  c1_RegMask::init_masks(nof_cpu_regs);

  _cpu_regs[0] = esi;  _esiRInfo.set_word_reg(0);
  _cpu_regs[1] = edi;  _ediRInfo.set_word_reg(1);
  _cpu_regs[2] = ebx;  _ebxRInfo.set_word_reg(2);
  _cpu_regs[3] = eax;  _eaxRInfo.set_word_reg(3);
  _cpu_regs[4] = edx;  _edxRInfo.set_word_reg(4);
  _cpu_regs[5] = ecx;  _ecxRInfo.set_word_reg(5);

  _noRInfo.set_no_rinfo();
  _eax_edxRInfo.set_long_reg(3 /*eax*/, 4 /*edx*/);
  _f0RInfo.set_float_reg(0);
  _d0RInfo.set_double_reg(0);

  _byte_reg_mask.add_reg(_eaxRInfo);
  _byte_reg_mask.add_reg(_ecxRInfo);
  _byte_reg_mask.add_reg(_edxRInfo);
  _byte_reg_mask.add_reg(_ebxRInfo);

  _init_done = true;
}


CallingConvention* FrameMap::calling_convention(bool is_static, const GrowableArray<BasicType>& signature, GrowableArray<int>* reg_args /* = NULL */) {
  Unimplemented();
  return NULL;
}


int FrameMap::om_framesize (int esp_offset) const {
  assert(_fixed_framesize == _size_locals + _size_spills + _size_monitors + 2 - _size_arguments, "not set properly");
  return esp_offset + _fixed_framesize;
}


bool FrameMap::is_byte_rinfo (RInfo reg) {
  return reg.is_word() && cpu_rnr2reg(reg.reg()).hasByteRegister();
}


int FrameMap::size_of_stack_decrement () const {
  // size of spilling is not known at the code emission time;
  // therefore we need to patch the stack decrement if any spilling
  // has happened
  int size = _size_locals - _size_arguments + _size_monitors;
  return size >= 0 ? size : 0;
}



// ----------------mapping-----------------------
// all mapping is based on ebp addressing, except for simple leaf methods where we access 
// the locals esp based (and no frame is built)


// Frame for simple leaf methods (quick entries)
//
//   +----------+
//   | ret addr |   <- TOS
//   +----------+
//   | args     |   
//   | ......   |

// Frame for standard methods
// 
//   | .........|  <- TOS
//   | locals   |
//   +----------+
//   | old ebp  |  <- EBP
//   +----------+
//   | ret addr |
//   +----------+
//   |  args    |
//   | .........|



// Maps java_index to an ebp (for standard methods) or esp (for simple leaf methods) offset
int FrameMap::real_index(int java_index) const {
  return _size_arguments - java_index + (java_index < _size_arguments ? 1 : -1); 
}


Address FrameMap::frame_slot_address(int slot_index) const {
  assert(slot_index >= 0, "wrong slot_index");
  return Address(ebp, real_index(slot_index) * wordSize);
}


bool FrameMap::are_adjacent_indeces(int jix1, int jix2) const {
  return abs(real_index(jix1) - real_index(jix2)) <= 1;
}


Address FrameMap::local_address(int local_index, bool is_two_word, bool for_hi_word) const {
  int index = local_index;
  if (is_two_word && !for_hi_word) index++; // i486, little endian
  return frame_slot_address(index);
}


Address FrameMap::monitor_address(int monitor_index) const {
  assert(sizeof(BasicLock)/wordSize == 1, "not tested for this BasicLock");
  int index = monitor_index * (sizeof(BasicLock)/wordSize) + _size_locals;
  return frame_slot_address(index);
}


Address FrameMap::spill_address(int spill_ix, bool is_two_words, bool for_hi_word) const {
  int index = _size_locals + _size_monitors + spill_ix;
  if (is_two_words && !for_hi_word) index++;
  return frame_slot_address(index);
}


// Map local_index to the esp offset 
// (example with 10 locals, including 5 arguments and esp_offset is 3)
//
//               arguments    ret ebp   locals       spill  monitor  esp_offset
//             [..............| . | . |............|......|.........|..........]
// local index  0  1  2  3  4   x   x   5  6  7  8 
//

int FrameMap::esp_offset_for_local(int local_index, int esp_offset) const {
  int tot_frame_size = om_framesize(esp_offset) + _size_arguments;
  int local_offset = tot_frame_size - local_index - 1;
  if (local_index < _size_arguments) local_offset -= 2; // correct for ret and ebp word
  return local_offset;
}

// The mapping of locals, registers and arguments is currently twisted
// (requirement by Compiler2): This is the way that the frame elements 
// must be passed in
// 
//   registers   stack0      esp[0]
// [...............|............|.....................]   RegName's
//                  arguments   
//                    ..  2 1 0 e e..    ... 6 5 4 x x local indeces 
//  (2 x's stand for old-fp and ret-addr)
//  (2 e's stand for esp-offset)
//
// Note that there is a possible esp-offset value that moves all the
// non-arguments locals to the right by esp_offset amount; 


OptoReg::Name FrameMap::regname_for(int in_index, int esp_offset) const {
  int index = -1;
  if (in_index < _size_arguments) {
    // it is an argument
    index = _size_arguments - in_index - 1;
    assert(0 <= index && index < _size_arguments, "wrong index");
  } else {
    // not an argument
#ifdef ASSERT
    int index_slow = (_size_arguments + om_framesize(esp_offset)) -
            (in_index + 2) + _size_arguments - 1; // two stands for the saved ebp and return address (factor out)
#endif
    index = 2 * _size_arguments - 3 + om_framesize(esp_offset) - in_index;
    assert(index_slow == index, "wrong oop map computation");
    assert (_size_arguments + esp_offset <= index  && 
            index < om_framesize(esp_offset) + _size_arguments, "wrong index");
  }
  assert(index >= 0, "wrong index computed");
  return SharedInfo::stack2reg(index);
}


OptoReg::Name FrameMap::local_regname(int local_index, int esp_offset) const {
  return regname_for(local_index, esp_offset);
}


OptoReg::Name FrameMap::spill_regname(int spill_index, int esp_offset) const {
  int index = _size_locals + _size_monitors + spill_index; // FACTOR THIS OUT !
  return regname_for(index, esp_offset);
}


OptoReg::Name FrameMap::register_regname (RInfo reg, int esp_offset) const {
  return OptoReg::Name(reg.as_register().number());
}


OopMap* FrameMap::new_oopmap(int esp_offset) {
  return new OopMap(om_framesize(esp_offset), _size_arguments);
}


Register FrameMap::first_register() { 
  init();
  return _cpu_regs[0]; 
}


Register FrameMap::first_byte_register() {
  init();
  return _cpu_regs[2]; 
}



//--------------------------------------------------------
//               ArgumentLocation
//--------------------------------------------------------


void ArgumentLocation::set_stack_location (int offset) {
  Unimplemented();
}


void ArgumentLocation::set_register_location (int number) {
  Unimplemented();
}


bool ArgumentLocation::is_register_arg () const {
  Unimplemented();
  return false;
}


bool ArgumentLocation::is_stack_arg () const {
  Unimplemented();
  return false;
}


int ArgumentLocation::stack_offset_in_words () const {
  Unimplemented();
  return 0;
}


Address ArgumentLocation::incoming_stack_location () const {
  Unimplemented();
  return Address();
}


Address ArgumentLocation::outgoing_stack_location () const {
  Unimplemented();
  return Address();
}


RInfo ArgumentLocation::incoming_reg_location () const {
  Unimplemented();
  return FrameMap::_noRInfo;
}


RInfo ArgumentLocation::outgoing_reg_location () const {
  Unimplemented();
  return FrameMap::_noRInfo;
}
