// @(#)c1_FrameMap_i486.hpp	1.31 00/04/17 11:53:44
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2




//  On i486 the frame looks as follows:
//
//  +----------------+---------+----------------------------+----------------+-----------
//  | size_arguments | 2 words | size_locals-size_arguments | _size_monitors | spilling .
//  +----------------+---------+----------------------------+----------------+-----------
//
//  The FPU registers are mapped with their offset from TOS; therefore the
//  status of FPU stack must be updated during code emission.


 public:
  enum {
    pd_stack0 = 32
  };

//  Simulates the FPU stack and maintains mapping [fpu-register -> stack offset]
//  FPU registers are described as numbers from 0..nof_fpu_regs-1

  class FpuStack VALUE_OBJ_CLASS_SPEC {
   private:
    int _offset [nof_fpu_regs]; // offset is 1..nof_fpu_regs; 0 means not on stack
    int _nof_elems;                // number of elements on stack

    int  find_tos (int offset);
    void swap     (int rnr1, int rnr2);

   public:
    FpuStack ();

#ifndef PRODUCT
    void fpu_range_check (int rnr) const { assert(0 <= rnr && rnr < nof_fpu_regs, "fpu register number is too big"); }
#endif
    int  tos_offset_for  (int rnr) const { assert(_offset[rnr] > 0, "not on stack"); return _nof_elems - _offset[rnr]; }
    void pop             (int rnr); // rnr must be on tos
    void push            (int rnr);
    int  bring_on_tos_0  (int rnr);
    bool is_stack_pos    (int rnr, int offset); // offset 0 means TOS
    void swap_two_on_tos (); 
    bool is_empty ();
    void clear ();
    void print () PRODUCT_RETURN;
  };

 private:
  int                _fixed_framesize;
  FpuStack           _fpu_stack;
  static c1_RegMask  _byte_reg_mask;
  
  Address frame_slot_address (int slot_index) const;
  int real_index (int java_index) const;


 public:
  static RInfo _noRInfo;
  static RInfo _esiRInfo;
  static RInfo _ediRInfo;
  static RInfo _ebxRInfo;
  static RInfo _eaxRInfo;
  static RInfo _edxRInfo;
  static RInfo _ecxRInfo;
  static RInfo _eax_edxRInfo;
  static RInfo _f0RInfo;
  static RInfo _d0RInfo;


  // no callee save registers on i486
  static c1_RegMask callee_save_regs()  { ShouldNotReachHere(); return c1_RegMask(); }

  FpuStack* fpu_stack () { return &_fpu_stack; }

  void set_fixed_framesize() { _fixed_framesize = _size_locals + _size_spills + _size_monitors + 2 - _size_arguments; } // 2 stands for return address and saved ebp 
  int size_of_stack_decrement() const; // for method entry

  static Register first_register();
  static Register first_byte_register();
  static c1_RegMask byte_reg_mask () { return _byte_reg_mask; }
  static bool is_byte_rinfo(RInfo reg);
