// @(#)c1_RegAlloc_i486.cpp	1.55 00/04/03 17:04:46
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_RegAlloc_i486.cpp.incl"


//--------------------------------------------------------
//               c1_AllocTable
//--------------------------------------------------------


int c1_AllocTable::_reg_mask [c1_AllocTable::nofRegs];
int c1_AllocTable::_free_reg [c1_AllocTable::nofRegsExp2];


c1_AllocTable::c1_AllocTable (int size) {
  assert(size == nofRegs, "sanity check");
  _state    = allFreeState;
  _or_state = _state;
  _lock_out = allFreeState;
}


void c1_AllocTable::init_tables () {
  for (int rnr = 0; rnr < nofRegs; rnr++) {
    _reg_mask[rnr] = 0x1 << rnr;
  }

  _free_reg[0] = FrameMap::cpu_reg2rnr(FrameMap::first_register()); // if all free use esi
  _free_reg[nofRegsExp2 - 1] = nofRegs;
  for (int i = 1; i < nofRegsExp2; i++) {
    for (int r = 0; r < nofRegs; r++) {
      if (!is_set_nth_bit(i, r)) {
        _free_reg[i] = r; break;
      }
    }
  }
}


bool c1_AllocTable::are_all_free () const {
    return _state == allFreeState;
}


bool c1_AllocTable::has_one_free () const {
  return _state != noneFreeState;
}


bool c1_AllocTable::has_two_free () const {   // for longs
  if (!has_one_free()) return false;
  c1_AllocTable* self = (c1_AllocTable*)this; // force cast to non-const type
  int r = self->get_free();
  self->set_locked(r);
  bool has_two = self->has_one_free();
  self->set_free(r);
  return has_two;
}


bool c1_AllocTable::has_one_free_masked (c1_RegMask m) const {
  return (_state | (~m._mask & noneFreeState)) != noneFreeState;
}


bool c1_AllocTable::is_free (int rnr) const {
  debug_only(reg_check(rnr););
  return (_state & _reg_mask[rnr]) == 0;
}


bool c1_AllocTable::did_use_register (int rnr) const {
  debug_only(reg_check(rnr););
  return (_or_state & _reg_mask[rnr]) != 0;
}


bool c1_AllocTable::are_free (int r1, int r2) const {
  return is_free(r1) && is_free(r2);
}


void c1_AllocTable::set_free (int rnr) {
  assert(!is_free(rnr), "not locked");
  _state = _state & ~_reg_mask[rnr];
}


void c1_AllocTable::set_locked (int rnr) { 
  assert(is_free(rnr) , "already locked"); 
  _state    = _state | _reg_mask[rnr];  
  _or_state = _or_state | _state;
  assert((_lock_out & _state) == allFreeState, "lock out check failed");
}

  
int c1_AllocTable::get_free () {
  assert(has_one_free(), "no free reg"); 
  int rnr = _free_reg[_state];
  _or_state = _or_state | _reg_mask[rnr];
#ifdef ASSERT
  int tmp_state = _state | _reg_mask[rnr];
  assert((_lock_out & tmp_state) == allFreeState, "lock out check failed");
#endif
  return rnr; 
}


int c1_AllocTable::get_free_masked (c1_RegMask m) {
  assert(has_one_free_masked(m), "no free reg in mask");
  int tmp_state = _state | (~m._mask & noneFreeState);
  int rnr = _free_reg[tmp_state];
  _or_state = _or_state | _reg_mask[rnr];
#ifdef ASSERT
  tmp_state = _state | _reg_mask[rnr];
  assert((_lock_out & tmp_state) == allFreeState, "lock out check failed");
  assert(m.contains(rnr), "sanity check");
#endif
  return rnr;
}


c1_RegMask c1_AllocTable::used_registers () const {
  c1_RegMask rm = c1_RegMask::empty_set();
  for (int rnr = 0; rnr < nofRegs; rnr++) {
    if ((_or_state & _reg_mask[rnr]) != 0) {
      rm.add_reg(rnr);
    }
  }
  return rm;
}


c1_RegMask c1_AllocTable::free_registers () const {
  c1_RegMask rm = c1_RegMask::empty_set();
  for (int rnr = 0; rnr < nofRegs; rnr++) {
    if ((_or_state & _reg_mask[rnr]) == 0) {
      rm.add_reg(rnr);
    }
  }
  return rm;
}


void c1_AllocTable::set_lockout (c1_RegMask lockout) {
  _lock_out = lockout._mask;
}


//---------------------------------------------------------
//          RegAlloc
//--------------------------------------------------------


void RegAlloc::init_register_allocation() {
  FrameMap::init();
  c1_AllocTable::init_tables();
}


bool RegAlloc::has_free_reg(ValueType* type) const {
  if (type->is_float() || type->is_double()) {
    return _fpu_alloc_table.has_one_free();
  } else if (type->is_long()) {
    return _cpu_alloc_table.has_two_free();
  } else {
    return _cpu_alloc_table.has_one_free();
  }
}


RInfo RegAlloc::get_free_reg (ValueType* type) {
  assert(has_free_reg(type), "no free register");
  RInfo rinfo;
  switch (type->tag()) {
  case intTag:
  case objectTag:
  case addressTag:
    rinfo = RInfo::word_reg(_cpu_alloc_table.get_free());
    break;
  case longTag:
    {
      int rnr_lo = _cpu_alloc_table.get_free(); ((RegAlloc*)this)->set_locked_cpu(rnr_lo, NULL, 1); // temp lock
      int rnr_hi = _cpu_alloc_table.get_free(); ((RegAlloc*)this)->set_free_cpu(rnr_lo);
      rinfo.set_long_reg(rnr_lo, rnr_hi);
    }
    break;  
  case floatTag:
    rinfo.set_float_reg(_fpu_alloc_table.get_free());
    break;
  case doubleTag:
    rinfo.set_double_reg(_fpu_alloc_table.get_free());
    break;
  default:
    ShouldNotReachHere();
    break;
  };
  return rinfo;
}


bool RegAlloc::is_free_double (int rnr) const {
  return is_free_fpu(rnr);
}


void RegAlloc::set_free_double (int rnr) {
  set_free_fpu(rnr);
}


void RegAlloc::set_locked_double (int rnr, Value instr, int rc) {
  set_locked_fpu(rnr, instr, rc);
}


void RegAlloc::set_double_reg (int rnr, int rc, Value value) {
  set_fpu_reg(rnr, rc, value);
}


int RegAlloc::get_double_rc (int rnr) const {
  return get_fpu_rc(rnr);
}


Value RegAlloc::get_double_val (int rnr) const {
  return get_fpu_val(rnr);
}


bool RegAlloc::is_locked_double_spill_count (int rnr) const {
  return is_locked_fpu_spill_count(rnr);
}


void RegAlloc::change_double_spill_count (int rnr, int d) {
  change_fpu_spill_count(rnr, d);
}


Value RegAlloc::get_smallest_value_to_spill(ValueType* type) const {
  bool is_fpu = type->is_float() || type->is_double();
  Value smallest_v = NULL;
  for (int i = 0; i < c1_AllocTable::nofRegs; i++) {
    Value v = NULL;
    if      ( is_fpu && !is_free_fpu(i) && !is_locked_fpu_spill_count(i)) v = _fpu_values[i];
    else if (!is_fpu && !is_free_cpu(i) && !is_locked_cpu_spill_count(i)) v = _cpu_values[i];
    // searching for value with smallest root
    // Sometimes we have a value whose item is not a register: this happens when
    // we are just handling the value and therefore do not spill it
    if (v != NULL && v->item() != NULL && v->item()->is_register()) {
      assert(!is_spill_locked(v->item()->get_register()), "is spill-locked");
      if (smallest_v == NULL) smallest_v = v;
      if (smallest_v->bci() > v->bci()) {
        smallest_v = v;
      }
    }
  }
  assert(smallest_v != NULL, "no spillable root found");
  assert(smallest_v->item()->is_register(), "not a register!");
  return smallest_v;
}
