// @(#)c1_Runtime1_i486.cpp	1.94 01/02/09 14:42:51
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_c1_Runtime1_i486.cpp.incl"


// Implementation of StubAssembler

int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, int args_size) {
  // setup registers
  const Register thread = edi; // is callee-saved register (Visual C++ calling conventions)
  assert(!(oop_result1.isValid() || oop_result2.isValid()) || oop_result1 != oop_result2, "registers must be different");
  assert(oop_result1 != thread && oop_result2 != thread, "registers must be different");
  assert(args_size >= 0, "illegal args_size");
  // push java thread (becomes first argument of C function)
  get_thread(thread);
  pushl(thread);
  // set last Java frame before call
  movl(Address(thread, JavaThread::last_Java_fp_offset_in_bytes()), ebp);
  movl(Address(thread, JavaThread::last_Java_sp_offset_in_bytes()), esp);
  // do the call
  call(entry, relocInfo::runtime_call_type);
  int call_offset = offset();
  // verify callee-saved register
#ifdef ASSERT
  guarantee(thread != eax, "change this code");
  pushl(eax);
  { Label L;
    get_thread(eax);
    cmpl(thread, eax);
    jcc(Assembler::equal, L);
    stop("StubAssembler::call_RT: edi not callee saved?");
    bind(L);
  }
  popl(eax);
#endif
  // reset last Java frame after call
  movl(Address(thread, JavaThread::last_Java_sp_offset_in_bytes()), (int)NULL);
  movl(Address(thread, JavaThread::last_Java_fp_offset_in_bytes()), (int)NULL);
  // discard thread and arguments
  addl(esp, (1 + args_size)*BytesPerWord);
  // check for pending exceptions
  { Label L;
    cmpl(Address(thread, Thread::pending_exception_offset_in_bytes()), (int)NULL);
    jcc(Assembler::equal, L);
    // exception pending => remove activation and forward to exception handler
    movl(eax, Address(thread, Thread::pending_exception_offset_in_bytes()));
    // make sure that the vm_results are cleared
    if (oop_result1.isValid()) {
      movl(Address(thread, JavaThread::vm_result_offset_in_bytes()), (int)NULL);
    }
    if (oop_result2.isValid()) {
      movl(Address(thread, JavaThread::vm_result_2_offset_in_bytes()), (int)NULL);
    }
    leave();
    jmp(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);
    bind(L);
  }
  // get oop results if there are any and reset the values in the thread
  if (oop_result1.isValid()) {
    movl(oop_result1, Address(thread, JavaThread::vm_result_offset_in_bytes()));
    movl(Address(thread, JavaThread::vm_result_offset_in_bytes()), (int)NULL);
    verify_oop(oop_result1);
  }
  if (oop_result2.isValid()) {
    movl(oop_result2, Address(thread, JavaThread::vm_result_2_offset_in_bytes()));
    movl(Address(thread, JavaThread::vm_result_2_offset_in_bytes()), (int)NULL);
    verify_oop(oop_result2);
  }
  return call_offset;
}


int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1) {
  pushl(arg1);
  return call_RT(oop_result1, oop_result2, entry, 1);
}


int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1, Register arg2) {
  pushl(arg2);
  pushl(arg1);
  return call_RT(oop_result1, oop_result2, entry, 2);
}


int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1, Register arg2, Register arg3) {
  pushl(arg3);
  pushl(arg2);
  pushl(arg1);
  return call_RT(oop_result1, oop_result2, entry, 3);
}


// Implementation of StubFrame

class StubFrame: public StackObj {
 private:
  StubAssembler* _sasm;
  int            _args_size;

 public:
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments);
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1);
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2);
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2, Register arg3);

  ~StubFrame();
};


#define __ _sasm->
StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments) {
  _sasm = sasm;
  _args_size = 0;
  __ set_info(name, must_gc_arguments);
  __ enter();
}


StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1) {
  _sasm = sasm;
  _args_size = 1;
  __ set_info(name, must_gc_arguments);
  __ enter();
  // tos + 0: link
  //     + 1: return address
  //     + 2: arg1
  __ movl(arg1, Address(ebp, 2*BytesPerWord));
}


StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2) {
  _sasm = sasm;
  _args_size = 2;
  __ set_info(name, must_gc_arguments);
  assert(arg1 != arg2, "registers must be different");
  __ enter();
  // tos + 0: link
  //     + 1: return address
  //     + 2: arg2
  //     + 3: arg1
  __ movl(arg2, Address(ebp, 2*BytesPerWord));
  __ movl(arg1, Address(ebp, 3*BytesPerWord));
}


StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2, Register arg3) {
  _sasm = sasm;
  _args_size = 3;
  __ set_info(name, must_gc_arguments);
  assert(arg1 != arg2 && arg1 != arg3 && arg2 != arg3, "registers must be different");
  __ enter();
  // tos + 0: link
  //     + 1: return address
  //     + 2: arg3
  //     + 3: arg2
  //     + 4: arg1
  __ movl(arg3, Address(ebp, 2*BytesPerWord));
  __ movl(arg2, Address(ebp, 3*BytesPerWord));
  __ movl(arg1, Address(ebp, 4*BytesPerWord));
}


StubFrame::~StubFrame() {
  __ leave();
  __ ret(_args_size * BytesPerWord);
}
#undef __


// Implementation of Runtime1

void Runtime1::generate_exception_throw(StubAssembler* sasm, const char* msg, address runtime_call) {
#define __ sasm->
  const bool dont_gc_arguments = false;
  // eax: pc offset where exception happened
  // TOS: address pointing into method
  __ popl (edx);
  __ call_VM_leaf(CAST_FROM_FN_PTR(address, compute_exception_pc), eax, edx);
  // eax is the address of exception
  __ pushl(eax);
  // it looks as we were called from the exception point
  // Note: in case of a call, do not gc arguments as they will be
  // thrown away
  { StubFrame f(sasm, msg, dont_gc_arguments); 
    // execute a throw
    __ call_RT(noreg, noreg, runtime_call);
    __ should_not_reach_here();
  }
#undef __
}


OopMapSet* Runtime1::generate_store_check(StubAssembler* sasm, address target) {
  enum layout {
    thread_off    = 0,       // last_java_sp
    fpu_state_off = FPUStateSizeInWords,
    flags_off,
    edi_off,         
    esi_off,
    ebp_off,
    esp_off,                
    ebx_off,
    edx_off,
    ecx_off,
    eax_off,            
    saved_ebp_off,
    return_off,              // slot for return address
    index_off,
    array_off,
    value_off,
    framesize
  };

#define __ sasm->
  __ pushad();
  __ pushfd();
  __ push_FPU_state();
  __ movl (eax, Address(esp, (array_off - 1) * BytesPerWord)); // array
  __ movl (ebx, Address(esp, (value_off - 1) * BytesPerWord)); // value

  // the register contents are preserved, actually do not need to return the objects
  int call_offset = __ call_RT(eax, ebx, target, eax, ebx);
  __ pop_FPU_state();
  __ popfd();
  __ popad();
  __ leave();
  __ ret(0);

  OopMapSet* oop_maps = new OopMapSet();
  OopMap* map =  new OopMap( framesize, 0 );  
  // add 2 as call_RT is pushing two paramters and thus the esp offset increases by two
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+eax_off + 2), framesize, 0, OptoReg::Name(eax.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+ecx_off + 2), framesize, 0, OptoReg::Name(ecx.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+edx_off + 2), framesize, 0, OptoReg::Name(edx.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+ebx_off + 2), framesize, 0, OptoReg::Name(ebx.number()));  
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+esi_off + 2), framesize, 0, OptoReg::Name(esi.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+edi_off + 2), framesize, 0, OptoReg::Name(edi.number()));    
  oop_maps->add_gc_map( call_offset, true, map);

#undef __
  return oop_maps;
}



OopMapSet* Runtime1::generate_patching(StubAssembler* sasm, address target) {
  enum layout {
    thread_off    = 0,       // last_java_sp
    fpu_state_off = FPUStateSizeInWords,
    flags_off,
    edi_off,         
    esi_off,
    ebp_off,
    esp_off,                
    ebx_off,
    edx_off,
    ecx_off,
    eax_off,            
    saved_ebp_off,
    return_off,              // slot for return address
    framesize
  };
#define __ sasm->
  __ pushad();
  __ pushfd();
  __ push_FPU_state();
  int call_offset = __ call_RT(noreg, noreg, target);
  // correct the address so that it is reexecuted
  __ movl (eax, Address(esp, (return_off - 1)*BytesPerWord)); // we have popped the thread
  __ subl (eax, NativeCall::instruction_size);
  __ movl (Address(esp, (return_off - 1)*BytesPerWord), eax);
  // patch return address so that the instruction is reexecuted
  __ pop_FPU_state();
  __ popfd();
  __ popad();
  __ leave();
  __ ret(0);

  OopMapSet* oop_maps = new OopMapSet();
  OopMap* map =  new OopMap( framesize, 0 );  
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+eax_off), framesize, 0, OptoReg::Name(eax.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+ecx_off), framesize, 0, OptoReg::Name(ecx.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+edx_off), framesize, 0, OptoReg::Name(edx.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+ebx_off), framesize, 0, OptoReg::Name(ebx.number()));  
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+esi_off), framesize, 0, OptoReg::Name(esi.number()));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0+edi_off), framesize, 0, OptoReg::Name(edi.number()));    
  oop_maps->add_gc_map( call_offset, true, map);

#undef __
  return oop_maps;
}


OopMapSet* Runtime1::generate_code_for(StubID id, StubAssembler* sasm) {
#define __ sasm->

  // for better readability
  const bool must_gc_arguments = true;
  const bool dont_gc_arguments = false;

  // stub code & info for the different stubs
  OopMapSet* oop_maps = NULL;
  switch (id) {
    case new_instance_id:
      { StubFrame f(sasm, "new_instance", dont_gc_arguments);
        // eax: klassOop
        __ call_RT(eax, noreg, CAST_FROM_FN_PTR(address, new_instance), eax);
        // eax: new instance
      }
      break;

    case new_type_array_id:
      { StubFrame f(sasm, "new_type_array", dont_gc_arguments, eax, ebx);
        // eax: elt_type
        // ebx: length
        __ call_RT(eax, noreg, CAST_FROM_FN_PTR(address, new_type_array), eax, ebx);
        // eax: new type array
      }
      break;

    case new_object_array_id:
      { StubFrame f(sasm, "new_object_array", dont_gc_arguments, eax, ebx);
        // eax: klass
        // ebx: length
        __ call_RT(eax, noreg, CAST_FROM_FN_PTR(address, new_object_array), eax, ebx);
        // eax: new object array
      }
      break;

    case new_multi_array_id:
      { StubFrame f(sasm, "new_multi_array", dont_gc_arguments, eax, ebx, ecx);
        // eax: klass
        // ebx: rank
        // ecx: address of 1st dimension
        __ call_RT(eax, noreg, CAST_FROM_FN_PTR(address, new_multi_array), eax, ebx, ecx);
        // eax: new multi array
      }
      break;

    case resolve_invokestatic_id:
      { StubFrame f(sasm, "resolve_invokestatic", must_gc_arguments);
        __ xorl(ecx, ecx); // recv must be NULL
        __ call_RT(ebx, noreg, CAST_FROM_FN_PTR(address, resolve_invoke), ecx);
        // eax: interpreter or verified entry point
        // ebx: callee method (always)
        __ leave();
        __ jmp(eax);
      }
      break;

    case resolve_invoke_id:
      { StubFrame f(sasm, "resolve_invoke", must_gc_arguments);
        // ecx: receiver
        __ call_RT(ebx, ecx, CAST_FROM_FN_PTR(address, resolve_invoke), ecx);
        // eax: interpreter or verified entry point
        // ebx: callee method (always)
        // ecx: receiver
        __ leave();
        __ jmp(eax);
      }
      break;

    case handle_ic_miss_id:
      { StubFrame f(sasm, "handle_ic_miss", must_gc_arguments);
        // eax: ic klass
        // ecx: receiver
        __ call_RT(ebx, ecx, CAST_FROM_FN_PTR(address, handle_ic_miss), ecx);
        // eax: interpreter or verified entry point
        // ebx: callee method (always)
        // ecx: receiver
        __ leave();
        __ jmp(eax);
      }
      break;

    case handle_wrong_static_method_id:
      { StubFrame f(sasm, "handle_wrong_static_method", must_gc_arguments);
        __ xorl(ecx, ecx); // recv must be NULL
        __ call_RT(ebx, ecx, CAST_FROM_FN_PTR(address, handle_wrong_method), ecx);
        // eax: interpreter or verified entry point
        // ebx: callee method (always)
        __ leave();
        __ jmp(eax);
      }
      break;

    case handle_wrong_method_id:
      { StubFrame f(sasm, "handle_wrong_method", must_gc_arguments);
        // ecx: receiver
        __ call_RT(ebx, ecx, CAST_FROM_FN_PTR(address, handle_wrong_method), ecx);
        // eax: interpreter or verified entry point
        // ebx: callee method (always)
        // ecx: receiver
        __ leave();
        __ jmp(eax);
      }
      break;

    case entering_non_entrant_id:
      { StubFrame f(sasm, "entering_non_entrant", must_gc_arguments);
        __ stop ("we entered a non entrant method: internal error");
      }
      break;

    case range_check_failed_id:
      generate_exception_throw(sasm, "range_check_failed", CAST_FROM_FN_PTR(address, throw_range_check_exception));
      break;

    case arithmetic_call_id:
      { StubFrame f(sasm, "arithmetic_call", dont_gc_arguments);
        __ stop("arithmetic call is not implemented");
      }
      break;

    case throw_div0_exception_id:
      generate_exception_throw(sasm, "throw_div0_exception", CAST_FROM_FN_PTR(address, throw_div0_exception));
      break;

    case throw_null_exception_id:
      generate_exception_throw(sasm, "throw_null_exception_at_call", CAST_FROM_FN_PTR(address, throw_null_exception_at_call));
      break;

    case throw_abstract_method_error_id:
      // exception PC is on TOS
      { StubFrame f(sasm, "throw_abstract_method_exception", dont_gc_arguments); // arguments will be discarded
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_abstract_method_error));
        __ should_not_reach_here();
      }
      break;

    case throw_exception_id:
      { StubFrame f(sasm, "throw_exception", dont_gc_arguments);
        // eax: exception
        __ call_RT(ebx, noreg, CAST_FROM_FN_PTR(address, throw_exception), eax);
        __ should_not_reach_here();
      }
      break;

    case handle_exception_id:
      { StubFrame f(sasm, "handle_exception", dont_gc_arguments);
        // tos: last argument (if the exception happened at a call and there are arguments)
        // eax: exception
        // ecx: esp decrement
        // edx: throwing pc
        // compute exception handler
        __ movl(ebx, Address(esp, 1*BytesPerWord));   // get real return address (to exception handler)
        __ movl(Address(esp, 1*BytesPerWord), edx);   // use throwing pc as return address (has bci & oop map)
        __ pushl(ecx);                                // save esp offset
        __ pushl(ebx);                                // save real return address
        // note: this call assumes any arguments to be on the stack if the
        //       exception happened at a call site - otherwise GC will not
        //       work correctly
        __ call_RT(ecx, noreg, CAST_FROM_FN_PTR(address, exception_handler_for_pc), eax, edx);
        __ popl(ebx);                                 // restore real return address
        __ popl(esi);                                 // restore esp decrement
        // eax: handler address or NULL if no handler exists
        // ebx: real return address
        // ecx: exception
        // esi: esp decrement
        __ verify_oop(ecx);
        // move continuation address into ebx
        // => use handler address if any, use ebx otherwise
        __ testl(eax, eax);
        if (CodeForP6) {
          __ cmovl(Assembler::notZero, ebx, eax);
        } else {
          Label L;
          __ jcc(Assembler::zero, L);
          __ movl(ebx, eax);
          __ bind(L);
        }
        __ movl(eax, ecx);   // restore exception in eax
        // eax: exception
        // ebx: continuation (exception handler or unwind code)
        __ leave();          // manually remove frame since we do a jmp at end
        __ popl(edx);        // remove return address
        __ movl(esp, ebp);   // remove arguments by resetting esp
        __ subl(esp, esi);   // esi contains esp decrement
        __ jmp(ebx);
      }
      break;

    case unwind_exception_id:
      { __ set_info("unwind_exception", dont_gc_arguments);
        // note: no stubframe since we are about to leave the current
        //       activation and we are calling a leaf VM function only.
        // eax: exception
        // leave activation
        __ leave();
        __ movl(edx, Address(esp));
        __ pushl(eax);       // save exception (no GC can happen due to leaf call)
        __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address), edx);
        // tos + 0: exception
        //     + 1: return address
        // eax    : exception handler
        __ movl(ebx, eax);   // set exception handler
        __ popl(eax);        // restore exception
        __ popl(edx);        // get throwing pc (= return address)
        // continue at exception handler (return address removed)
        // note: do *not* remove arguments when unwinding the
        //       activation since the caller assumes having
        //       all arguments on the stack when entering the
        //       runtime to determine the exception handler
        //       (GC happens at call site with arguments!)
        // eax: exception
        // ebx: exception handler
        // edx: throwing pc
        __ verify_oop(eax);
        __ jmp(ebx);
      }
      break;

    case store_check_id:
      { StubFrame f(sasm, "store_check", dont_gc_arguments);
        // tos + 0: link
        //     + 1: return address
        //     + 2: index 
        //     + 3: array
        //     + 4: value
        // preserve oops over call
        oop_maps = generate_store_check(sasm, CAST_FROM_FN_PTR(address, store_check));
      }
      break;

    case checkcast_id:
      { StubFrame f(sasm, "checkcast", dont_gc_arguments, eax, ebx);
        // eax: object
        // ebx: klass
        __ call_RT(eax, noreg, CAST_FROM_FN_PTR(address, checkcast), eax, ebx);
        // eax: object (preserved)
      }
      break;

    case instanceof_id:
      { StubFrame f(sasm, "instanceof", dont_gc_arguments, eax, ebx);
        // eax: object
        // ebx: klass
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, instanceof), eax, ebx);
        // eax: result (0 or 1)
      }
      break;

    case monitorenter_id:
      { StubFrame f(sasm, "monitorenter", dont_gc_arguments, eax, ebx);
        // eax: object
        // ebx: lock address
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, monitorenter), eax, ebx);
      }
      break;

    case monitorexit_id:
      { StubFrame f(sasm, "monitorexit", dont_gc_arguments, eax, ebx);
        // eax: object
        // ebx: lock address
        // note: really a leaf routine but must setup last java sp
        //       => use call_RT for now (speed can be improved by
        //       doing last java sp setup manually)
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, monitorexit), eax, ebx);
      }
      break;

    case implicit_div0_exception_id: // fall through
    case implicit_null_exception_id:
      { // tos + 0: exception filter return address
        //     + 1: tos at exception point
        // all registers are as at the exception point (except eip & flags)
        //
        // how it works: 1) we check if there is a valid return address on the stack
        // (that is the case if the exception happened in an inline cache check);
        // if not, we compute a fake return address and use it instead (i.e., we
        // replace the filter's return address with the new address and don't pop
        // it afterwards) - 2) once we have the return address we throw the exception
        // via a runtime call; i.e., from the Java execution point-of-view it looks
        // like we called the VM to explicitly throw the exception
        //
        // note 1: for now the filter never returns but handles the exception always
        //         (in a 2-compiler system we would need to return if the exception
        //         happened outside the code generated by C1)
        //
        // note 2: div0 exceptions could be handled slightly simpler as they never
        //         occur at an invoke and thus always need a fake return address;
        //         i.e., no need to call return_address_for_exception except that
        //         we need to do the set_is_handling_implicit_exception(true) anyway
        //
        // step 1: determine fake return address and push it if necessary
        { Label L;
          __ pushad();                           // save registers
          __ call_VM_leaf(CAST_FROM_FN_PTR(address, return_address_for_exception));
          __ movl(Address(esp, Register::nofRegisters * BytesPerWord), eax); // potential return address
          __ testl(eax, eax);                    // check for fake return address
          __ popad();                            // restore registers
          __ jcc(Assembler::notZero, L);         // we need a fake return address => don't pop it
          __ addl(esp, 1*BytesPerWord);          // we have a real return address => pop fake address
          __ bind(L);
        }
        // step 2: throw the exception via standard runtime call
        // tos + 0: real/fake return address used as exception pc
        //     + 1: tos at exception point
        // all registers are as at the exception point (except eip & flags)
        if (id == implicit_div0_exception_id) {
          StubFrame f(sasm, "implicit_div0_exception", dont_gc_arguments);
          __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_div0_exception));
        } else {
          StubFrame f(sasm, "implicit_null_exception", dont_gc_arguments);
          __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_null_exception));
        }
        __ should_not_reach_here();
      }
      break;

    case interpreter_entries_id:
      { __ set_info("interpreter_entries", dont_gc_arguments);
        __ stop("do not call interpreter entries entry directly");
        // generate entries for all method kinds
        for (int i = 0; i < AbstractInterpreter::number_of_method_entries; i++) {
          AbstractInterpreter::MethodKind kind = (AbstractInterpreter::MethodKind)i;

          // entry point to methods that are final
          int virtual_final_call_offset = __ offset();
          // get ic method
          // tos + 0: return address
          //       1: last argument
          // eax    : compiledICHolderOop
          // ecx    : receiver
          Label continueL;
          __ movl(ebx, Address(eax, compiledICHolderOopDesc::holder_method_offset()));
          __ verify_oop(eax);
          __ verify_oop(ebx);
          __ verify_oop(ecx);
          __ jmp (continueL);

          // entry point for virtual calls
          int virtual_call_offset = __ offset();
          // get ic method & klass
          // tos + 0: return address
          //       1: last argument
          // eax    : compiledICHolderOop
          // ecx    : receiver
          __ movl(edx, Address(eax, compiledICHolderOopDesc::holder_klass_offset ()));
          __ movl(ebx, Address(eax, compiledICHolderOopDesc::holder_method_offset()));
          __ verify_oop(eax);
          __ verify_oop(ebx);
          __ verify_oop(ecx);
          __ verify_oop(edx);
#ifdef ASSERT
          // check that receiver is truly in ecx
          if (VerifyReceiver) {
            Label L;
            __ load_unsigned_word(eax, Address(ebx, methodOopDesc::size_of_parameters_offset_in_bytes()));
            __ cmpl(ecx, Address(esp, eax, Address::times_4));
            __ jcc(Assembler::equal, L);
            __ stop("interpreter IC check: receiver must be in ecx");
            __ bind(L);
          }
#endif
          // do ic check
          // ebx: ic method
          // ecx: receiver
          // edx: ic klass
          __ cmpl(edx, Address(ecx, oopDesc::klass_offset_in_bytes()));
          __ jcc(Assembler::notEqual, entry_for(handle_ic_miss_id));
          // fall through

          __ bind(continueL);
          // entry point for optimized virtual calls
          int optimized_call_offset = __ offset();
          // ebx: methodOop
          // ecx: receiver
          __ cmpl(Address(ebx, methodOopDesc::compiled_code_offset_in_bytes()), NULL);
          __ jcc(Assembler::notEqual, entry_for(handle_wrong_method_id), relocInfo::runtime_call_type);
          __ jmp(AbstractInterpreter::entry_for_kind(kind), relocInfo::runtime_call_type);

          // entry point for static calls
          int static_call_offset = __ offset();
          // ebx: methodOop
          __ cmpl(Address(ebx, methodOopDesc::compiled_code_offset_in_bytes()), NULL);
          __ jcc(Assembler::notEqual, entry_for(handle_wrong_static_method_id), relocInfo::runtime_call_type);
          __ jmp(AbstractInterpreter::entry_for_kind(kind), relocInfo::runtime_call_type);

          // setup entries
          _ientries[i] = iEntries(static_call_offset, optimized_call_offset, virtual_call_offset, virtual_final_call_offset);
        }
      }
      break;

    case stack_overflow_exception_id:
      // All registers are available.
      // First we remove the topmost frame, making the exception state as it
      // looks when it happens at a call.
      __ leave();
      // PC on TOS: it looks as if a call has been made to this routine
      { StubFrame f(sasm, "stack_overflow_exception", dont_gc_arguments);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_stack_overflow));
        __ should_not_reach_here();
      }
      break;

    case init_check_patching_id:
      { StubFrame f(sasm, "init_check_patching", dont_gc_arguments);
        // we should set up register map
        oop_maps = generate_patching(sasm, CAST_FROM_FN_PTR(address, init_check_patching));
      }
      break;
      
    case load_klass_patching_id:
      { StubFrame f(sasm, "load_klass_patching", dont_gc_arguments);
        // we should set up register map
        oop_maps = generate_patching(sasm, CAST_FROM_FN_PTR(address, move_klass_patching));
      }
      break;

    case osr_frame_return_id:
      { __ set_info("osr_frame_return", dont_gc_arguments);
        // continuation point after returning from an osr'ed method -
        // the current frame is the interpreter frame in the state when
        // it was osr'ed, with esp & ebp setup correctly and the osr'ed
        // method's result in eax and/or edx or FPU ST0 (=> do not touch
        // these registers!)
        assert(!UseC2CallingConventions, "new calling conventions not supported");
        __ movl(edi, Address(ebp, frame::interpreter_frame_locals_offset * BytesPerWord)); // restore locals ptr
        __ leave();                              // remove frame anchor
        __ popl(ecx);                            // get return address
        __ leal(esp, Address(edi, BytesPerWord));    // set sp to sender sp
        __ jmp(ecx);                             // return
      }
      break;

    case osr_unwind_exception_id:
      { __ set_info("osr_unwind_exception", dont_gc_arguments);
        // exception handler entry point for an osr'ed method -
        // the current frame is the interpreter frame in the state when
        // it was osr'ed, with esp & ebp setup correctly and the exception
        // in eax => ignore this frame & further unwind the stack and pro-
        // pagate the excepion back
        //
        // note: do not remove the arguments when removing the activation
        //       since the callee may be compiled code which expects the
        //       arguments on the stack when handling the exception at a
        //       call site! (i.e., use the new calling conventions!)
        //
        // remove activation
        __ movl(ebx, Address(ebp, frame::interpreter_frame_sender_sp_offset * BytesPerWord)); // get caller sp
        __ leave();                              // remove frame anchor
        __ popl(edx);                            // get return address
        __ movl(esp, ebx);                       // set sp to caller sp (w/ arguments untouched!)
        // inbetween activations - previous activation type unknown yet
        // compute continuation point - the continuation point expects
        // the following registers set up:
        //
        // eax: exception
        // edx: return address/pc that threw exception
        // esp: expression stack of caller
        // ebp: ebp of caller
        __ pushl(eax);                           // save exception
        __ pushl(edx);                           // save return address
        __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address), edx);
        __ movl(ebx, eax);                       // save exception handler
        __ popl(edx);                            // restore return address
        __ popl(eax);                            // restore exception
        // continue at exception handler (return address removed)
        // note: do *not* remove arguments when unwinding the
        //       activation since the caller assumes having
        //       all arguments on the stack when entering the
        //       runtime to determine the exception handler
        //       (GC happens at call site with arguments!)
        // eax: exception
        // ebx: exception handler
        // edx: throwing pc
        __ verify_oop(eax);
        __ jmp(ebx);
      }
      break;

    case illegal_instruction_handler_id:
      { __ set_info("illegal_instruction_handler", dont_gc_arguments);
        const Register java_thread = edi; // callee-saved for VC++
        address start   = __ pc();  
        address call_pc = NULL;  
        enum layout {
          thread_off    = 0,       // last_java_sp
          fpu_state_off = FPUStateSizeInWords,
          flags_off,
          edi_off,         
          esi_off,
          ebp_off,
          esp_off,                
          ebx_off,
          edx_off,
          ecx_off,
          eax_off,            
          saved_ebp_off,
          return_off,              // slot for return address
          framesize
        };

        assert(FPUStateSizeInWords == 27, "update stack layout");
        Label noException, returnType;

        // save registers, fpu state, and flags  
        __ pushl(ebx);             // Make room for return address (or push it again for ret patching)
        // Build C1 frame
        __ enter();

        __ pushad();        
        __ pushfd();        
        __ push_FPU_state();

        // The following is basically a call_VM. However, we need the precise address of the call in order to
        // generate an oopmap. Hence, we do all the work ourselves.

        // push thread argument and setup last_Java_sp
        __ get_thread(java_thread);
        __ pushl(java_thread);
        __ movl(Address(java_thread, JavaThread::last_Java_sp_offset_in_bytes()), esp);
        __ movl(Address(java_thread, JavaThread::last_Java_fp_offset_in_bytes()), ebp);

        // do the call
        __ call(CAST_FROM_FN_PTR(address, SafepointSynchronize::handle_illegal_instruction_exception), relocInfo::runtime_call_type);
        // eax contains the number of arguments that must be popped
        call_pc = __ pc();
        __ movl(ecx, eax); // move result into ecx, as eax may be loaded with the result of return
        __ popl(eax);      // remove thread argument

        // clear last_Java_sp again
        __ get_thread(java_thread);
        __ movl(Address(java_thread, JavaThread::last_Java_fp_offset_in_bytes()), NULL);
        __ movl(Address(java_thread, JavaThread::last_Java_sp_offset_in_bytes()), NULL);


        __ cmpl(Address(java_thread, Thread::pending_exception_offset_in_bytes()), NULL);
        __ jcc(Assembler::equal, noException);

        // path for handling exception at safepoint
        __ pop_FPU_state();
        __ popfd();
        __ popad();
        __ leave();
        __ get_thread(java_thread);
        // get exception oop, exception pc is on TOS 
        __ movl (eax, Address(java_thread, JavaThread::pending_exception_offset_in_bytes())); 

        // this is untested, unimplemented code
        assert(StubRoutines::forward_exception_entry() != NULL, "must be generated before");  
        __ jmp(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);
    
        // path for continue normal execution
        // register restoring and exit  
        __ bind(noException);
        __ testl (ecx, ecx);
        __ jcc (Assembler::notZero, returnType);

        __ pop_FPU_state();
        __ popfd();
        __ popad();

        __ leave();
        __ ret(0);

        // return operation is not reexecuted; if result og
        __ bind (returnType);
        // ecx is the size of arguments in bytes that have to be removed

        // restore potential result (eax.edx), without destroying ecx with popad
        // Note: thread has already been popped, therefore (reg_off - 1)
        __ movl (eax, Address(esp, (eax_off - 1) * BytesPerWord));
        __ movl (edx, Address(esp, (edx_off - 1) * BytesPerWord));
        __ pop_FPU_state();
        __ popfd();
        // skip popad
        // remove frame
        __ leave();
        __ popl (ebx);     // get return address
        __ addl(esp, ecx); // remove arguments
        __ jmp (ebx);      // continue in caller

        oop_maps = new OopMapSet();
        OopMap* map = new OopMap(framesize, 0);  
        map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + eax_off), framesize, 0, OptoReg::Name(eax.number()));
        map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + ecx_off), framesize, 0, OptoReg::Name(ecx.number()));
        map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + edx_off), framesize, 0, OptoReg::Name(edx.number()));
        map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + ebx_off), framesize, 0, OptoReg::Name(ebx.number()));  
        map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + esi_off), framesize, 0, OptoReg::Name(esi.number()));
        map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + edi_off), framesize, 0, OptoReg::Name(edi.number()));    
        oop_maps->add_gc_map(call_pc - start, true, map);
      }
      break;

      case address_bci_2_pc_id:
        // we are calling into a leaf, do not need block descriptor
        // bci is in eax;
        // address of nmethod is on TOS
        { StubFrame f(sasm, "bci_2_pc", dont_gc_arguments);
          __ movl (edx, Address(esp, wordSize)); // ebp has been pushed too 
          __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, bci_2_pc), eax, edx);
          // result is in eax
        }
        break;
      case safepoint_block_id:
        {
          // ebx: thread
          // edx: preserve result
          // eax: preserve result (don't worry about oop, it is still a handle)
          __ pushl (eax);
          __ pushl (edx);
          __ pushl (ebx);
          __ call(CAST_FROM_FN_PTR(address, SafepointSynchronize::block), relocInfo::runtime_call_type);
          __ popl  (ebx);
          __ popl  (edx);
          __ popl  (eax);
          __ ret   (0);
        }
        break;
      case jvmpi_method_entry:
        { // eax: methodOop; ecx: receiver or NULL
          StubFrame f(sasm, "jvmpi_method_entry", dont_gc_arguments);

          __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_entry), eax, ecx);
        }
        break;
      case jvmpi_method_exit:
        { // eax: methodOop
          StubFrame f(sasm, "jvmpi_method_exit", dont_gc_arguments);

          __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_exit), eax);
        }
        break;
    default:
      { StubFrame f(sasm, "unimplemented entry", dont_gc_arguments);
        __ movl(eax, (int)id);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, unimplemented_entry), eax);
        __ should_not_reach_here();
      }
      break;
  }
#undef __
  return oop_maps;
}


JRT_ENTRY(void, Runtime1::prepare_interpreter_call(JavaThread* thread, methodOop method))
  // not used for i486
  Unimplemented();
JRT_END


void GC_Support::preserve_callee_argument_oops(const frame fr, symbolHandle signature, bool is_static, void f(oop*)) {
  // not used for i486
  Unimplemented();
}


jint* GC_Support::get_argument_addr_at(const frame fr, ArgumentLocation location) {
  // not used for i486
  Unimplemented();
  return NULL;
}


jint* GC_Support::get_stack_argument_addr_at(const frame fr, int offset_from_sp_in_words) {
  // not used for i486
  Unimplemented();
  return NULL;
}


jint* GC_Support::get_register_argument_addr_at(const frame fr, Register r) {
  // not used for i486
  Unimplemented();
  return NULL;
}
