/*
 * @(#)frame_i486.cpp	1.146 01/03/12 12:28:59
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_frame_i486.cpp.incl"


// Profiling/safepoint support

bool JavaThread::get_top_frame(frame* _fr, ExtendedPC* _addr, bool for_profile_only) {
  jint*   sp;
  jint*   fp;

  ExtendedPC addr = os::fetch_top_frame(this, &sp, &fp);

  if (addr.contained_pc() != NULL) {
  /* Some return paths do not require valid fp or sp values,
     so they are not checked here */
      frame fr(sp, fp, addr.pc());
      *_fr = fr;
      if (_addr != NULL) *_addr = addr;
      return true;
   }
   return false;
}

// Called by the jvmpi profiler to get the sp and pc
// for a thread_in_Java
// match Sparc which can not call fetch_top_frame 
// - if called on other  threads will call _thr_getstate 
// which assumes thread is t_stop or stopallmutators, 
// and it is not
bool JavaThread::profile_top_frame(frame* _fr, ExtendedPC* _addr) {
  bool gotframe = false;;
  ThreadState state = osthread()->state();

  // self suspension saves needed state.
  if ((state == SUSPENDED || state == CONDVAR_WAIT) 
     && has_last_Java_frame()) {
     *_fr = pd_last_frame();
     gotframe = true;
     
  }
  return gotframe;
}

bool JavaThread::profile_last_Java_frame(frame* _fr, ExtendedPC* _addr) {
  // Fine-grained profiling of the VM code is currently not implemented on i486
  // and *_addr is not set. It will eventually be set to the exact native PC if the
  // tick lands inside the VM code.
  if (_fr != NULL) {
    *_fr = last_frame();
    return true;
  } 
  return false;
}


// sender_sp

jint* frame::interpreter_frame_sender_sp() const {
  assert(is_interpreted_frame(), "interpreted frame expected");
  return (jint*) at(interpreter_frame_sender_sp_offset);
}

void frame::set_interpreter_frame_sender_sp(jint* sender_sp) {
  assert(is_interpreted_frame(), "interpreted frame expected");
  int_at_put(interpreter_frame_sender_sp_offset, (jint) sender_sp);
}


// monitor elements

BasicObjectLock* frame::interpreter_frame_monitor_begin() const {
  return (BasicObjectLock*) addr_at(interpreter_frame_monitor_block_bottom_offset);
}

BasicObjectLock* frame::interpreter_frame_monitor_end() const {
  BasicObjectLock* result = (BasicObjectLock*) *(jint*)addr_at(interpreter_frame_monitor_block_top_offset);
  // make sure the pointer points inside the frame
  assert((int) fp() >  (int) result, "result must <  than frame pointer");
  assert((int) sp() <= (int) result, "result must >= than stack pointer");
  return result;
}

void frame::interpreter_frame_set_monitor_end(BasicObjectLock* value) {
  *((BasicObjectLock**)addr_at(interpreter_frame_monitor_block_top_offset)) = value;
}

frame frame::sender_for_entry_frame(RegisterMap* map) const {
  assert(map != NULL, "map must be set");
  // Java frame called from C; skip all C frames and return top C
  // frame of that chunk as the sender
  JavaCallWrapper* jcw = entry_frame_call_wrapper();
  assert(!entry_frame_is_first(), "next Java fp must be non zero");
  assert(jcw->last_Java_sp() > _sp, "must be above this frame on stack");  
  frame fr(jcw->last_Java_sp(), jcw->last_Java_fp());  
  map->clear(jcw->not_at_call_sp()); 
  assert(map->include_argument_oops(), "should be set by clear");
  return fr;
}

frame frame::sender_for_interpreter_frame(RegisterMap* map) const {
  jint* sp = (jint*) at(interpreter_frame_sender_sp_offset);
  // We do not need to update the callee-save register mapping because above
  // us is either another interpreter frame or a converter-frame, but never
  // directly a compiled frame.
  return frame(sp, link(), sender_pc());
}

frame frame::sender_for_raw_compiled_frame(RegisterMap* map) const {
#ifndef CORE
  CodeBlob* stub_cb = CodeCache::find_blob(pc());
  assert(stub_cb != NULL, "wrong pc");
  return sender_for_compiled_frame(map, stub_cb, false);
#else
  ShouldNotReachHere();
  return frame();
#endif
}

#ifndef CORE
#ifdef COMPILER2

//------------------------------link_offset-------------------------------------
// Find the where EBP got saved in the frame.  Cached and lazily computed.
int CodeBlob::link_offset( ) {
  // Magic value -2 means Not Yet Computed; other values are the actual result.
  if( _link_offset == not_yet_computed ) {

    // No prior EBP saved for nmethods (prior frame is NOT an interpreted).
    if( is_nmethod() ) {
      _link_offset = undefined;
    } else {
      // The Adaptor spills EBP so it must be in a stack location somewhere.
      // Find it.
      OopMap *map = oop_maps()->at(0);// Should be in first map

      ResourceMark rm;
      OopMapValue omv;
      for(OopMapStream oms(map,OopMapValue::callee_saved_value); !oms.is_done(); oms.next()) {
	omv = oms.current();
	if(omv.content_reg() == Matcher::interpreter_frame_pointer_reg()) {
	  _link_offset = omv.stack_offset();
	  break;
	}
      }
      assert( _link_offset != not_yet_computed, "didn't find EBP in first oopmap?" );
    }
  }
  return _link_offset;
}
#endif // COMPILER2

//------------------------------sender_for_compiled_frame-----------------------
frame frame::sender_for_compiled_frame(RegisterMap* map, CodeBlob* cb, bool adjusted) const {
  assert(map != NULL, "map must be set");

  // frame owned by optimizing compiler 
  address* return_addr_addr = NULL;
  jint*    sender_sp  = NULL;
#ifdef COMPILER1
  return_addr_addr = (address*)(fp() + 1);
  sender_sp = fp() + 2;
#else
  assert(cb->return_address_offset() >= 0, "Compiled by C1: Do not use return address offset here");
  return_addr_addr = (address*)(sp() + cb->return_address_offset());
  assert(cb->frame_size() >= 0, "Compiled by Compiler1: do not use");
  sender_sp = sp() + cb->frame_size();
#endif

  // On Intel the return_address is always the word on the stack
  assert(((address *)return_addr_addr) + 1 == (address *)sender_sp, "sanity check");

  if (map->update_map() && cb->oop_maps() != NULL) {
    OopMapSet::update_register_map(this, cb, map);
  }

  // Move this here for C1 and collecting oops in arguments (According to Rene)
  COMPILER1_ONLY(map->set_include_argument_oops(cb->caller_must_gc_arguments(map->thread()));)

  jint *saved_fp = NULL;
  COMPILER1_ONLY(saved_fp = (jint*) (*fp());)
#ifdef COMPILER2
  int llink_offset = cb->link_offset();
  if (llink_offset >= 0) {    
    // Restore base-pointer, since next frame might be an interpreter frame.    
    jint* fp_addr = sp() + llink_offset;
#if defined(ASSERT) && defined(COMPILER2)
    // Check to see if regmap has same info
    if (map->update_map()) {
      address fp2_addr = map->location(VMReg::Name(EBP_num));
      assert(fp2_addr == NULL || fp2_addr == (address)fp_addr, "inconsistent framepointer info.");          
    }
#endif       
    saved_fp = (jint*)*fp_addr;
  }
#endif // COMPILER2

  // Update sp (e.g. osr adapter needs to get saved_esp)
  int sp_offset = cb->sp_offset();
  if (sp_offset >= 0) {
  jint* sp_addr = sp() + sp_offset;  
#if defined(ASSERT) && defined(COMPILER2)
    if (map->update_map()) {
      address sp2_addr = map->location(VMReg::Name(ESP_num));
      assert(sp2_addr == (address)sp_addr, "inconsistent framepointer info");          
    }
#endif
    sender_sp = (jint*)*sp_addr;    
  }

  address sender_pc = *return_addr_addr;
#ifndef CORE
  if (adjusted) {
    // Adjust the sender_pc if it points into a temporary codebuffer.      
    sender_pc = map->thread()->safepoint_state()->compute_adjusted_pc(sender_pc);
  }
#endif

  return frame(sender_sp, saved_fp, sender_pc);
}

frame frame::sender_for_deoptimized_frame(RegisterMap* map, CodeBlob* cb) const {
  guarantee(map != NULL, "map must be present");
  JavaThread *thread = map->thread();
  guarantee(thread != NULL, "thread not set");
  vframeArray* vf = thread->vframe_array_for(sp());
  if (map->update_map()) {
    vf->update_register_map(map);
  }      
  COMPILER1_ONLY(ShouldNotReachHere();)
  jint* sender_sp = sp() + vf->frame_size();
  return frame(sender_sp, NULL, (address) *(sender_sp - 1));
}
#endif

frame frame::sender(RegisterMap* map, CodeBlob *cb) const {  
  // Default is we done have to follow them. The sender_for_xxx will
  // update it accordingly
  map->set_include_argument_oops(false);

  if (is_entry_frame())       return sender_for_entry_frame(map);
  if (is_interpreted_frame()) return sender_for_interpreter_frame(map);
#ifdef COMPILER1
  // Note: this test has to come before CodeCache::find_blob(pc())
  //       call since the code for osr adapter frames is contained
  //       in the code cache, too!
  if (is_osr_adapter_frame()) return sender_for_interpreter_frame(map);
#endif // COMPILER1
#ifndef CORE  
  if(cb == NULL) {
    cb = CodeCache::find_blob(pc());
  } else {
    assert(cb == CodeCache::find_blob(pc()),"Must be the same");
  }

  if (cb  != NULL) {
    // Deoptimized frame?
    if (is_deoptimized_frame()) {
      return sender_for_deoptimized_frame(map, cb);
    } else {
      // Returns adjusted pc if it was pointing into a temp. safepoint codebuffer.
      return sender_for_compiled_frame(map, cb, true);      
    }
  }
#endif // CORE
  // Must be native-compiled frame, i.e. the marshaling code for native
  // methods that exists in the core system.
  return frame(sender_sp(), link(), sender_pc());
}


// Adjust index for the 2 words containing old_fp and ret_adr
int frame::adjust_offset(methodOop method, int index) {
  int size_of_parameters = method->size_of_parameters();
  assert (size_of_parameters >= 0, "illegal number of parameters in method");
  return (size_of_parameters - index + (index < size_of_parameters ? 1 : -1));
}

bool frame::interpreter_frame_equals_unpacked_fp(jint* fp) {
  assert(is_interpreted_frame(), "must be interpreter frame");
  methodOop method = interpreter_frame_method();
  // When unpacking an optimized frame the frame pointer is
  // adjusted with: 
  int diff = method->max_locals() - method->size_of_parameters();
  return _fp == (fp - diff);
}

void frame::pd_gc_epilog() {
  // nothing done here now
}

bool frame::is_interpreted_frame_valid() const {
  assert(is_interpreted_frame(), "Not an interpreted frame");
  // These are reasonable sanity checks
  if (fp() == 0 || (int(fp()) & 0x3) != 0) {
    return false;
  }
  if (sp() == 0 || (int(sp()) & 0x3) != 0) {
    return false;
  }
  if (fp() + interpreter_frame_initial_sp_offset < sp()) {
    return false;
  }
  // These are hacks to keep us out of trouble.
  // The problem with these is that they mask other problems
  if (fp() <= sp()) {        // this attempts to deal with unsigned comparison above
    return false;
  }
  if (fp() - sp() > 4096) {  // stack frames shouldn't be large.
    return false;
  }
  return true;
}


#ifdef COMPILER1
void frame::set_sender_pc_for_unpack(frame caller, int exec_mode) {
  Unimplemented();
}
#endif

#ifdef COMPILER2
void frame::set_sender_pc_for_unpack(frame caller, int exec_mode) {
  switch(exec_mode) {
  case Deoptimization::Unpack_deopt:
  case Deoptimization::Unpack_exception: {
    assert(OptoRuntime::deoptimization_blob()->return_address_offset() >= 0, "Compiled by C1: Do not use return address ofsfet here");
    jint* return_address = sp() + OptoRuntime::deoptimization_blob()->return_address_offset();
    *return_address = (jint) caller.pc();
    // Stuff the returned frame pointer into the uncommon_trap frame.
    // Helps PSF do stack crawls.
    assert( OptoRuntime::deoptimization_blob()->link_offset() >= 0, "" );
    jint *fp = sp() + OptoRuntime::deoptimization_blob()->link_offset();
    *fp = (jint) caller.fp();
  } 
  break;
  case Deoptimization::Unpack_uncommon_trap: {
    assert(OptoRuntime::uncommon_trap_blob()->return_address_offset() >= 0, "Compiled by C1: Do not use return address ofsfet here");
    jint* return_address = sp() + OptoRuntime::uncommon_trap_blob()->return_address_offset();
    *return_address = (jint) caller.pc();
    // Stuff the returned frame pointer into the uncommon_trap frame.
    // Helps PSF do stack crawls.
    assert( OptoRuntime::uncommon_trap_blob()->link_offset() >= 0, "" );
    jint *fp = sp() + OptoRuntime::uncommon_trap_blob()->link_offset();
    *fp = (jint) caller.fp();
  } 
  break;
  default:
    ShouldNotReachHere(); 
    break;
  }
}
#endif

// Typed get/set local from a frame
// Todo: make work for compiled frames

oop frame::get_local_object(jint slot) {
 return (oop) interpreter_frame_local_at(slot); 
}

jint frame::get_local_int(jint slot) {
  return (jint) interpreter_frame_local_at(slot);
}

jlong frame::get_local_long(jint slot) {
  jint *local_addr = *interpreter_frame_locals_addr() - slot - 1;
  return JNITypes::get_long(local_addr);
}

jfloat frame::get_local_float(jint slot) {
  jint *local_addr = *interpreter_frame_locals_addr() - slot;
  return JNITypes::get_float(local_addr);
}

jdouble frame::get_local_double(jint slot) {
  jint *local_addr = *interpreter_frame_locals_addr() - slot - 1;
  return JNITypes::get_double(local_addr);
}

void frame::set_local_object(jint slot, oop     obj) {
 ( (oop&) interpreter_frame_local_at(slot)) = obj;
}

void frame::set_local_int(jint slot, jint    i) {
 ( (jint&) interpreter_frame_local_at(slot)) = i;
}

void frame::set_local_long(jint slot, jlong   l) {
  jint *local_addr = *interpreter_frame_locals_addr() - slot - 1;
  JNITypes::put_raw_long(&l, local_addr);
}

void frame::set_local_float(jint slot, jfloat  f) {
  jint *local_addr = *interpreter_frame_locals_addr() - slot;
  JNITypes::put_float(&f, local_addr);
}

void frame::set_local_double(jint slot, jdouble d) {
  jint *local_addr = *interpreter_frame_locals_addr() - slot - 1;
  JNITypes::put_raw_double(&d, local_addr);
}

