/*
 * @(#)interp_masm_i486.cpp	1.97 01/02/09 14:43:01
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_interp_masm_i486.cpp.incl"


// Implementation of InterpreterMacroAssembler

void InterpreterMacroAssembler::null_check(Register obj, int offset) {
  if (GenerateInterpreterNullChecks) {
    // generate explicit NULL check
    testl(obj, obj);    
    jcc(Assembler::zero, Interpreter::throw_NullPointerException_entry(), relocInfo::none);
  } else {
    // use NULL-trap mechanism
    MacroAssembler::null_check(obj, offset);
  }
}


void InterpreterMacroAssembler::call_VM_leaf_base(
  address entry_point,
  int     number_of_arguments
) {
  // interpreter specific
  //
  // Note: No need to save/restore bcp & locals (esi & edi) pointer
  //       since these are callee saved registers and no blocking/
  //       GC can happen in leaf calls.
#ifdef ASSERT
  save_bcp();
#endif
  // super call
  MacroAssembler::call_VM_leaf_base(entry_point, number_of_arguments);
  // interpreter specific
#ifdef ASSERT
  { Label L;
    cmpl(esi, Address(ebp, frame::interpreter_frame_bcx_offset * wordSize));
    jcc(Assembler::equal, L);
    stop("InterpreterMacroAssembler::call_VM_leaf_base: esi not callee saved?");
    bind(L);
  }
  { Label L;
    cmpl(edi, Address(ebp, frame::interpreter_frame_locals_offset * wordSize));
    jcc(Assembler::equal, L);
    stop("InterpreterMacroAssembler::call_VM_leaf_base: edi not callee saved?");
    bind(L);
  }
#endif
}


void InterpreterMacroAssembler::call_VM_base(
  Register oop_result,
  Register java_thread,
  Register last_java_sp,
  address  entry_point,
  int      number_of_arguments
) {
  // interpreter specific
  //
  // Note: Could avoid restoring locals ptr (callee saved) - however doesn't
  //       really make a difference for these runtime calls, since they are
  //       slow anyway. Btw., bcp must be saved/restored since it may change
  //       due to GC.
  assert(java_thread == noreg , "not expecting a precomputed java thread");
  save_bcp();
  // super call
  MacroAssembler::call_VM_base(oop_result, java_thread, last_java_sp, entry_point, number_of_arguments);
  // interpreter specific
  restore_bcp();
  restore_locals();
}


void InterpreterMacroAssembler::get_unsigned_2_byte_index_at_bcp(Register reg, int bcp_offset) {
  assert(bcp_offset >= 0, "bcp is still pointing to start of bytecode");
  movl(reg, Address(esi, bcp_offset));
  bswap(reg);
  shrl(reg, 16);
}


void InterpreterMacroAssembler::get_cache_and_index_at_bcp(Register cache, Register index, int bcp_offset) {
  assert(bcp_offset > 0, "bcp is still pointing to start of bytecode");
  assert(cache != index, "must use different registers");
  load_unsigned_word(index, Address(esi, bcp_offset));
  movl(cache, Address(ebp, frame::interpreter_frame_cache_offset * wordSize));
  assert(sizeof(ConstantPoolCacheEntry) == 4*wordSize, "adjust code below");
  shll(index, 2); // convert from field index to ConstantPoolCacheEntry index
}


void InterpreterMacroAssembler::get_cache_entry_pointer_at_bcp(Register cache, Register tmp, int bcp_offset) {
  assert(bcp_offset > 0, "bcp is still pointing to start of bytecode");
  assert(cache != tmp, "must use different register");
  load_unsigned_word(tmp, Address(esi, bcp_offset));
  assert(sizeof(ConstantPoolCacheEntry) == 4*wordSize, "adjust code below");
                               // convert from field index to ConstantPoolCacheEntry index
                               // and from word offset to byte offset
  shll(tmp, 2 + LogBytesPerWord);
  movl(cache, Address(ebp, frame::interpreter_frame_cache_offset * wordSize));
                               // skip past the header
  addl(cache, constantPoolCacheOopDesc::base_offset_in_bytes());
  addl(cache, tmp);            // construct pointer to cache entry
}


void InterpreterMacroAssembler::f2ieee() {
  if (IEEEPrecision) {
    fstp_s(Address(esp));
    fld_s(Address(esp));
  }
}


void InterpreterMacroAssembler::d2ieee() {
  if (IEEEPrecision) {
    fstp_d(Address(esp));
    fld_d(Address(esp));
  }
}


void InterpreterMacroAssembler::pop(TosState state) {
  switch (state) {
    case atos: /* treat the same as itos */                  // fall through
    case itos: popl(eax);                                    break;
    case ltos: popl(eax);           popl(edx);               break;
    case ftos: fld_s(Address(esp)); addl(esp, 1 * wordSize); break;
    case dtos: fld_d(Address(esp)); addl(esp, 2 * wordSize); break;
    case vtos: /* nothing to do */                           break;
    default  : ShouldNotReachHere();
  }
  verify_oop(eax, state);
}


void InterpreterMacroAssembler::push(TosState state) {
  verify_oop(eax, state);
  switch (state) {
    case atos: /* treat the same as itos */                   // fall through
    case itos: pushl(eax);                                    break;
    case ltos: pushl(edx);              pushl(eax);           break;
    case ftos: subl(esp, 1 * wordSize); fstp_s(Address(esp)); break; // Do not schedule for no AGI! Never write beyond esp!
    case dtos: subl(esp, 2 * wordSize); fstp_d(Address(esp)); break; // Do not schedule for no AGI! Never write beyond esp!
    case vtos: /* nothing to do */                            break;
    default  : ShouldNotReachHere();
  }
}


void InterpreterMacroAssembler::super_call_VM_leaf(address entry_point, Register arg_1) {
  pushl(arg_1);  
  MacroAssembler::call_VM_leaf_base(entry_point, 1);
}


void InterpreterMacroAssembler::super_call_VM(Register oop_result, Register last_java_sp, address entry_point, Register arg_1, Register arg_2) {
  pushl(arg_2);  
  pushl(arg_1);
  MacroAssembler::call_VM_base(oop_result, noreg, last_java_sp, entry_point, 2);
}


// The following two routines provide a hook so that an implementation
// can schedule the dispatch in two parts.  Intel does not do this.
void InterpreterMacroAssembler::dispatch_prolog(TosState state, int step) {
  // Nothing to be done here
}

void InterpreterMacroAssembler::dispatch_epilog(TosState state, int step) {
  dispatch_next(state, step);
}

void InterpreterMacroAssembler::dispatch_base(TosState state, address* table) {
  verify_FPU(1, state);
  if (VerifyActivationFrameSize) {
    Label L;
    movl(ecx, ebp);
    subl(ecx, esp);
    int min_frame_size = (frame::link_offset - frame::interpreter_frame_initial_sp_offset) * wordSize;
    cmpl(ecx, min_frame_size);
    jcc(Assembler::greaterEqual, L);
    stop("broken stack frame");
    bind(L);
  }
  verify_oop(eax, state);
  jmp(Address(noreg, ebx, Address::times_4, (int)table));
}


void InterpreterMacroAssembler::dispatch_only(TosState state) {
  dispatch_base(state, Interpreter::dispatch_table(state));
}

void InterpreterMacroAssembler::dispatch_only_normal(TosState state) {
  dispatch_base(state, Interpreter::normal_table(state));
}


void InterpreterMacroAssembler::dispatch_next(TosState state, int step) {
  // load next bytecode (load before advancing esi to prevent AGI)
  load_unsigned_byte(ebx, Address(esi, step));
  // advance esi
  increment(esi, step);
  dispatch_base(state, Interpreter::dispatch_table(state));
}


void InterpreterMacroAssembler::dispatch_via(TosState state, address* table) {
  // load current bytecode
  load_unsigned_byte(ebx, Address(esi));
  dispatch_base(state, table);
}


void InterpreterMacroAssembler::remove_activation(TosState state, Register ret_addr, bool throw_monitor_exception) {
  // Note: Registers eax, edx and FPU ST(0) may be in use for the result
  // check if synchronized method  
  Label unlocked, unlock;
  movl(ebx, Address(ebp, frame::interpreter_frame_method_offset * wordSize)); // get method access flags
  movl(ecx, Address(ebx, methodOopDesc::access_flags_offset_in_bytes()));
  testl(ecx, JVM_ACC_SYNCHRONIZED);
  jcc(Assembler::zero, unlocked);
  
  // unlock monitor
  push(state);                                   // save result
    
  // BasicObjectLock will be first in list, since this is a synchronized method. However, need
  // to check that the object has not been unlocked by an explicit monitorexit bytecode.  
  const Address monitor(ebp, frame::interpreter_frame_initial_sp_offset * wordSize - (int)sizeof(BasicObjectLock));
  leal  (edx, monitor);                          // address of first monitor
  
  movl  (eax, Address(edx, BasicObjectLock::obj_offset_in_bytes()));
  testl (eax, eax);
  jcc   (Assembler::notZero, unlock);
                                  
  pop(state);
  if (throw_monitor_exception) {
    // Entry already unlocked, need to throw exception
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
    should_not_reach_here();
  } else {
    // Monitor already unlocked during a stack unroll. Install a
    // illegal_monitor_state_exception and continue with stack unrolling.
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::new_illegal_monitor_state_exception));
    jmp(unlocked);
  }

  bind(unlock);  
  unlock_object(edx);              
  pop(state);

  // Check that for block-structured locking (i.e., that all locked objects has been unlocked)  
  bind(unlocked);  

  // eax, edx: Might contain return value

  // Check that all monitors are unlocked
  {
    Label loop, exception, entry, restart;
    const int entry_size               = frame::interpreter_frame_monitor_size()           * wordSize;
    const Address monitor_block_top(ebp, frame::interpreter_frame_monitor_block_top_offset * wordSize);
    const Address monitor_block_bot(ebp, frame::interpreter_frame_initial_sp_offset        * wordSize);
    
    bind(restart);
    movl(ecx, monitor_block_top);             // points to current entry, starting with top-most entry
    leal(ebx, monitor_block_bot);             // points to word before bottom of monitor block
    jmp(entry);
          
    // Entry already locked, need to throw exception
    bind(exception); 

    if (throw_monitor_exception) {
      // Throw exception      
      MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
      should_not_reach_here();
    } else {
      // Stack unrolling. Unlock object and install illegal_monitor_exception
      // Unlock does not block, so don't have to worry about the frame
      // We don't have to preserve eax, edx since we are going to throw an exception

      movl(edx, ecx);
      unlock_object(edx);
      
      call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::new_illegal_monitor_state_exception));

      jmp(restart);
    }
  
    bind(loop);
    cmpl(Address(ecx, BasicObjectLock::obj_offset_in_bytes()), (int)NULL);  // check if current entry is used
    jcc(Assembler::notEqual, exception);
          
    addl(ecx, entry_size);                       // otherwise advance to next entry
    bind(entry);
    cmpl(ecx, ebx);                              // check if bottom reached
    jcc(Assembler::notEqual, loop);              // if not at bottom then check this entry      
  }        

  // jvmdi/jvmpi support
  notify_method_exit(state);                     // preserve TOSCA

  // remove activation
  // C1       : do not remove arguments (i.e., use new calling conventions)
  //            if we are unwinding the frame because of exception forwarding:
  //            this happens if and only if !throw_monitor_exception
  if (UseC2CallingConventions COMPILER1_ONLY(|| !throw_monitor_exception)) {
    movl(ebx, Address(ebp, frame::interpreter_frame_sender_sp_offset * wordSize)); // get sender sp
    leave();                                     // remove frame anchor
    popl(ret_addr);                              // get return address
    movl(esp, ebx);                              // set sp to sender sp
  } else {
    movl(ebx, Address(ebp, frame::interpreter_frame_locals_offset * wordSize)); // get sender sp
    leave();                                     // remove frame anchor
    popl(ret_addr);                              // get return address
    leal(esp, Address(ebx, wordSize));           // set sp to sender sp
  }
}


// Lock object
//
// Argument: edx : Points to BasicObjectLock to be used for locking. Must
// be initialized with object to lock
void InterpreterMacroAssembler::lock_object(Register lock_reg) {
  assert(lock_reg == edx, "The argument is only for looks. It must be edx");

  if (UseHeavyMonitors) {
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorenter), lock_reg);
  } else {

    Label done;

    const Register swap_reg = eax;  // Must use eax for cmpxchg instruction
    const Register obj_reg  = ecx;  // Will contain the oop

    const obj_offset = BasicObjectLock::obj_offset_in_bytes();
    const lock_offset = BasicObjectLock::lock_offset_in_bytes ();
    const mark_offset = lock_offset + BasicLock::displaced_header_offset_in_bytes(); 

    // Load object pointer into obj_reg %ecx
    movl(obj_reg, Address(lock_reg, obj_offset));

   if (UseSpecLock) { 
        Label non_speclock;
        
        unimplemented("The spec lock bit has moved, this code needs change");

        // Load mark and check for speclocking (see comments in unlock for details) 
        movl(swap_reg, Address(obj_reg));  
        testl(swap_reg, markOopDesc::lock_mask_in_place);
        jcc(Assembler::zero, non_speclock);
        testl(swap_reg, markOopDesc::speclock_mask_in_place);
        jcc(Assembler::zero, non_speclock);
    
        // Get object owner (thread id)  and compare with current tid
        get_thread_id(swap_reg);
        cmpl(swap_reg, Address(obj_reg, SpecLock::speclock_field_offset()));
        jcc(Assembler::zero, done);
        pushl(lock_reg); 
        // Else, call a safepoint for fixup 
        call_VM(obj_reg, CAST_FROM_FN_PTR(address, InterpreterRuntime::fixup_speclock), obj_reg);
        popl(lock_reg);
        bind(non_speclock);  
    }

   // Load immediate 1 into swap_reg %eax
    movl(swap_reg, 1);

    // Load (object->mark() | 1) into swap_reg %eax
    orl(swap_reg, Address(obj_reg));

    // Save (object->mark() | 1) into BasicLock's displaced header
    movl(Address(lock_reg, mark_offset), swap_reg);

    assert(lock_offset == 0, "displached header must be first word in BasicObjectLock");
    if (os::is_MP()) {
      lock();
    }    
    cmpxchg(lock_reg, Address(obj_reg));    
    jcc(Assembler::zero, done);

    // Test if the oopMark is an obvious stack pointer, i.e.,
    //  1) (mark & 3) == 0, and
    //  2) esp <= mark < mark + os::pagesize()
    //
    // These 3 tests can be done by evaluating the following 
    // expression: ((mark - esp) & (3 - os::vm_page_size())),
    // assuming both stack pointer and pagesize have their
    // least significant 2 bits clear.
    // NOTE: the oopMark is in swap_reg %eax as the result of cmpxchg
    subl(swap_reg, esp);
    andl(swap_reg, 3 - os::vm_page_size());

    // Save the test result, for recursive case, the result is zero
    movl(Address(lock_reg, mark_offset), swap_reg);

    jcc(Assembler::zero, done);

    // Call the runtime routine for slow case
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorenter), lock_reg);

    bind(done);
  }   
}


// Unlocks an object. Used in monitorexit bytecode and remove_activation.
//
// Argument: edx : Points to BasicObjectLock structure for lock
// Throw an IllegalMonitorException if object is not locked by current thread
// 
// Uses: eax, ebx, ecx, edx
void InterpreterMacroAssembler::unlock_object(Register lock_reg) {
  assert(lock_reg == edx, "The argument is only for looks. It must be edx");

  if (UseHeavyMonitors) {
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorexit), lock_reg);
  } else {
    Label done;

    const Register swap_reg   = eax;  // Must use eax for cmpxchg instruction
    const Register header_reg = ebx;  // Will contain the old oopMark
    const Register obj_reg    = ecx;  // Will contain the oop

    save_bcp(); // Save in case of exception

    // Convert from BasicObjectLock structure to object and BasicLock structure
    // Store the BasicLock address into %eax
    leal(swap_reg, Address(lock_reg, BasicObjectLock::lock_offset_in_bytes()));

    // Load oop into obj_reg(%ecx)
    movl(obj_reg, Address(lock_reg, BasicObjectLock::obj_offset_in_bytes ()));

    if (UseSpecLock) {
      Label non_speclock;

      unimplemented("The spec lock bit has moved, this code needs change");

      // Check if object is speculatively locked
      movl(header_reg, Address(obj_reg));
      
      // Check for bit 0, bit 1 is '11'. This test is not strictly
      // neccesary when valid stack poiners is below 2Gb. This is
      // typically true in WinNT. However, certain enterprise versions
      // can have 3Gb user space.
      testl(header_reg, markOopDesc::lock_mask_in_place);
      jcc(Assembler::zero, non_speclock);

      // Check for high bit set
      testl(header_reg, markOopDesc::speclock_mask_in_place);
      jcc(Assembler::zero, non_speclock);
      
      // If speculatively locked, zero out data on stack 
      // (it is checked by an assertion later to ensure correct nesting)
      movl(Address(lock_reg, BasicObjectLock::obj_offset_in_bytes()), (int)NULL);
      jmp(done);

      bind(non_speclock);
    }

    // Load the old header from BasicLock structure
    movl(header_reg, Address(swap_reg, BasicLock::displaced_header_offset_in_bytes()));

    // Free entry
    movl(Address(lock_reg, BasicObjectLock::obj_offset_in_bytes()), (int)NULL);

    // Test for recursion
    testl(header_reg, header_reg);

    // zero for recursive case
    jcc(Assembler::zero, done);
    
    // Atomic swap back the old header
    if (os::is_MP()) lock();
    cmpxchg(header_reg, Address(obj_reg));

    // zero for recursive case
    jcc(Assembler::equal, done);

    // Call the runtime routine for slow case.
    movl(Address(lock_reg, BasicObjectLock::obj_offset_in_bytes()), obj_reg); // restore obj
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorexit), lock_reg);

    bind(done);

    restore_bcp();
  }
}


void InterpreterMacroAssembler::verify_oop(Register reg, TosState state) {
  if (state == atos) MacroAssembler::verify_oop(reg);
}


void InterpreterMacroAssembler::verify_FPU(int stack_depth, TosState state) {
  if (state == ftos || state == dtos) MacroAssembler::verify_FPU(stack_depth);
}

 
void InterpreterMacroAssembler::notify_method_entry() {
  if (jvmdi::enabled()) {
    Label L;
    movl(ecx, Address(jvmdi::get_method_entry_on_addr(), relocInfo::none));
    testl(ecx,ecx);
    jcc(Assembler::zero, L);
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_method_entry));
    bind(L);
  }
  { Label E;
    Label S;
    cmpl(Address((int)jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_ENTRY ), relocInfo::none), (int)JVMPI_EVENT_ENABLED);
    jcc(Assembler::equal, S);
    cmpl(Address((int)jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_ENTRY2), relocInfo::none), (int)JVMPI_EVENT_ENABLED);
    jcc(Assembler::notEqual, E);
    bind(S);
    // notify method entry
    { get_method(ebx);
      // get receiver
      { Label L;
        xorl(ecx, ecx);             // receiver = NULL for a static method
        movl(eax, Address(ebx, methodOopDesc::access_flags_offset_in_bytes()));
        testl(eax, JVM_ACC_STATIC); // check if method is static
        jcc(Assembler::notZero, L); // if static we're done
        movl(ecx, Address(edi));    // otherwise get receiver
        bind(L);
      }
      call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_entry), ebx, ecx);
    }
    bind(E);
  }
}

 
void InterpreterMacroAssembler::notify_method_exit(TosState state) {
  if (jvmdi::enabled()) {
    Label L;
    push(state);     
    movl(ecx, Address(jvmdi::get_method_exit_on_addr(), relocInfo::none));
    testl(ecx,ecx);
    jcc(Assembler::zero, L);
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_method_exit));
    bind(L);
    pop(state);     
  }
  { Label E;
    cmpl(Address((int)jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_EXIT), relocInfo::none), (int)JVMPI_EVENT_ENABLED);
    jcc(Assembler::notEqual, E);
    // notify method exit
    { push(state);
      get_method(ebx);
      call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_exit), ebx);
      pop(state);
    }
    bind(E);
  }
}
