/*
 * @(#)interp_masm_i486.hpp	1.53 00/06/28 10:25:44
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// This file specializes the assember with interpreter-specific macros


class InterpreterMacroAssembler: public MacroAssembler {
 protected:
  // Interpreter specific version of call_VM_base
  virtual void call_VM_leaf_base(
    address entry_point,
    int     number_of_arguments
  );

  virtual void call_VM_base(
    Register oop_result,
    Register java_thread,
    Register last_java_sp,
    address  entry_point,
    int      number_of_arguments
  );

  // base routine for all dispatches
  void dispatch_base(TosState state, address* table);

 public:
  InterpreterMacroAssembler(CodeBuffer* code) : MacroAssembler(code) {}

  // Interpreter-specific registers
  void save_bcp()                                          { movl(Address(ebp, frame::interpreter_frame_bcx_offset * wordSize), esi); }
  void restore_bcp()                                       { movl(esi, Address(ebp, frame::interpreter_frame_bcx_offset * wordSize)); }
  void restore_locals()                                    { movl(edi, Address(ebp, frame::interpreter_frame_locals_offset * wordSize)); }

  // Helpers for runtime call arguments/results
  void get_method(Register reg)                            { movl(reg, Address(ebp, frame::interpreter_frame_method_offset * wordSize)); }
  void get_constant_pool(Register reg)                     { get_method(reg); movl(reg, Address(reg, methodOopDesc::constants_offset_in_bytes())); }
  void get_constant_pool_cache(Register reg)               { get_constant_pool(reg); movl(reg, Address(reg, constantPoolOopDesc::cache_offset_in_bytes())); }
  void get_unsigned_2_byte_index_at_bcp(Register reg, int bcp_offset);
  void get_cache_and_index_at_bcp(Register cache, Register index, int bcp_offset);
  void get_cache_entry_pointer_at_bcp(Register cache, Register tmp, int bcp_offset);

  // Expression stack
  void f2ieee();                                           // truncate ftos to 32bits
  void d2ieee();                                           // truncate dtos to 64bits
  void pop (TosState state);                               // transition vtos -> state
  void push(TosState state);                               // transition state -> vtos
  void empty_expression_stack()                            { movl(esp, Address(ebp, frame::interpreter_frame_monitor_block_top_offset * wordSize)); }

  // Exceptions
  void null_check(Register obj, int offset = -1);

  // Super call_VM calls - correspond to MacroAssembler::call_VM(_leaf) calls
  void super_call_VM_leaf(address entry_point, Register arg_1);
  void super_call_VM(Register oop_result, Register last_java_sp, address entry_point, Register arg_1, Register arg_2);

  // Dispatching
  void dispatch_prolog(TosState state, int step = 0);
  void dispatch_epilog(TosState state, int step = 0);
  void dispatch_only(TosState state);                      // dispatch via ebx (assume ebx is loaded already)
  void dispatch_only_normal(TosState state);               // dispatch normal table via ebx (assume ebx is loaded already)
  void dispatch_next(TosState state, int step = 0);        // load ebx from [esi + step] and dispatch via ebx
  void dispatch_via (TosState state, address* table);      // load ebx from [esi] and dispatch via ebx and table

  // Returning from interpreted functions
  //
  // Removes the current activation (incl. unlocking of monitors)
  // and sets up the return address.  This code is also used for
  // exception unwindwing. In that case, we do not want to throw 
  // IllegalMonitorStateExceptions, since that might get us into an 
  // infinite rethrow exception loop.
  void remove_activation(TosState state, Register ret_addr, bool throw_monitor_exception = true);

  // Object locking
  void lock_object  (Register lock_reg);
  void unlock_object(Register lock_reg);

  // Debugging
  void verify_oop(Register reg, TosState state = atos);    // only if +VerifyOops && state == atos
  void verify_FPU(int stack_depth, TosState state = ftos); // only if +VerifyFPU  && (state == ftos || state == dtos)
    
  // support for jvmdi/jvmpi
  void notify_method_entry();
  void notify_method_exit(TosState state);
  
};

