/*
 * @(#)interpreter_i486.cpp	1.243 01/02/09 14:43:02
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_interpreter_i486.cpp.incl"

#define __ _masm->

const int method_offset = frame::interpreter_frame_method_offset * wordSize;
const int bci_offset    = frame::interpreter_frame_bcx_offset    * wordSize;
const int locals_offset = frame::interpreter_frame_locals_offset * wordSize;


//------------------------------------------------------------------------------------------------------------------------

address AbstractInterpreterGenerator::generate_StackOverflowError_handler() {
  address entry = __ pc();

  // --- TEMPORARY BUG FIX --------------------------------------------------------------
  // Note: Stack overflow can happen before the current frame is fully setup,
  //       in which case ebp may still point to the previous frame. In that
  //       case esi may not contain the correct bcp; and furthermore, the
  //       previous frame may not be an interpreted frame (bug# 4245436 - gri 7/22/99).
  //       Or, ebp may point to the new frame, but the frame is not completely
  //       setup, in which case we have to revert to the previous frame, too.
  //       For now we assume the previous frame is also interpreted and just make
  //       sure that esi contains the correct bcp of the caller. This is not a
  //       correct fix but probably works in most cases; it also fixes the above
  //       mentioned bug. We must find a correct solution, eventually.
  //
  // 1) if ebp has been set to new frame, make sure frame has minimal size
  { Label L;
    __ leal(eax, Address(ebp, frame::interpreter_frame_monitor_block_top_offset * wordSize));
    __ cmpl(eax, esp);                // eax = maximal esp for current ebp (stack grows negative)
    __ jcc(Assembler::aboveEqual, L); // check if frame is complete
    __ leave();                       // frame is not complete => revert to previous frame
    __ bind(L);
    // if the current frame is an interpreter frame, we have a completely setup frame now
  }
  // 2) restore bcp under the assumption that the current frame is still interpreted
  //    (this assumption can be wrong and then this will fail - see comment above)
  __ restore_bcp();
  // ------------------------------------------------------------------------------------

  // expression stack must be empty before entering the VM if an exception happened
  __ empty_expression_stack();
  __ empty_FPU_stack();
  // create exception
  __ call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::create_StackOverflowError));
  // throw exception
  __ jmp(Interpreter::throw_exception_entry(), relocInfo::none);
  return entry;
}


address AbstractInterpreterGenerator::generate_exception_handler(const char* name, const char* message) {
  address entry = __ pc();
  // expression stack must be empty before entering the VM if an exception happened
  __ empty_expression_stack();
  __ empty_FPU_stack();
  // setup parameters
  __ movl(eax, (int)name);
  __ movl(ebx, (int)message);
  __ call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::create_exception), eax, ebx);
  // throw exception
  __ jmp(Interpreter::throw_exception_entry(), relocInfo::none);
  return entry;
}


address AbstractInterpreterGenerator::generate_continuation_for(TosState state) {
  address entry = __ pc();
  __ dispatch_next(state);
  return entry;
}


address AbstractInterpreterGenerator::generate_return_entry_for(TosState state, int step) {
  address entry = __ pc();
  __ restore_bcp();
  __ restore_locals();
  if (UseC2CallingConventions) {
    __ get_cache_and_index_at_bcp(ebx, ecx, 1);
    __ movl(ebx, Address(ebx, ecx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize));
    __ andl(ebx, 0xFF);
    __ leal(esp, Address(esp, ebx, Address::times_4));
  }
  __ dispatch_next(state, step);
  return entry;
}


#ifndef CORE
address AbstractInterpreterGenerator::generate_deopt_entry_for(TosState state, int step) {
  address entry = __ pc();
  __ restore_bcp();
  __ restore_locals();
  // handle exceptions
  { Label L;
    const Register thread = ecx;
    __ get_thread(thread);
    __ cmpl(Address(thread, Thread::pending_exception_offset_in_bytes()), (int)NULL);
    __ jcc(Assembler::zero, L);    
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_pending_exception));
    __ should_not_reach_here();
    __ bind(L);
  }
  __ dispatch_next(state, step);
  return entry;
}
#endif


int AbstractInterpreter::BasicType_as_index(BasicType type) {
  int i = 0;
  switch (type) {
    case T_BOOLEAN: i = 0; break;
    case T_CHAR   : i = 1; break;
    case T_BYTE   : i = 2; break;
    case T_SHORT  : i = 3; break;
    case T_INT    : // fall through
    case T_LONG   : // fall through
    case T_VOID   : i = 4; break;
    case T_FLOAT  : // fall through
    case T_DOUBLE : i = 5; break;
    case T_OBJECT : // fall through
    case T_ARRAY  : i = 6; break;
    default       : ShouldNotReachHere();
  }
  assert(0 <= i && i < AbstractInterpreter::number_of_result_handlers, "index out of bounds");
  return i;
}


address AbstractInterpreterGenerator::generate_result_handler_for(BasicType type) {
  address entry = __ pc();
  switch (type) {
    case T_BOOLEAN: __ c2bool(eax);            break;
    case T_CHAR   : __ andl(eax, 0xFFFF);      break;
    case T_BYTE   : __ sign_extend_byte (eax); break;
    case T_SHORT  : __ sign_extend_short(eax); break;
    case T_INT    : /* nothing to do */        break;
    case T_FLOAT  :
      { const Register t = InterpreterRuntime::SignatureHandlerGenerator::temp();
        __ popl(t);                            // remove return address first
        __ pop(dtos);                          // restore ST0        
        __ pushl(t);                           // restore return address
      }
      break;
    case T_OBJECT :
      { Label L;
        __ testl(eax, eax);                    // test if NULL handle
        __ jcc(Assembler::zero, L);            // if not then
        __ movl(eax, Address(eax));            // unbox result
        __ verify_oop(eax);                    // and verify it
        __ bind(L);
      }
      break;
    default       : ShouldNotReachHere();
  }
  __ ret(0);                                   // return from result handler
  return entry;
}


address AbstractInterpreterGenerator::generate_slow_signature_handler() {
  address entry = __ pc();
  // ebx: method
  // ecx: temporary
  // edi: pointer to locals
  // esp: end of copied parameters area
  __ movl(ecx, esp);
  __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::slow_signature_handler), ebx, edi, ecx);
  __ ret(0);
  return entry;
}


address AbstractInterpreterGenerator::generate_safept_entry_for(TosState state, address runtime_entry) {
  address entry = __ pc();
  __ push(state);
  __ call_VM(noreg, runtime_entry);
  __ dispatch_via(vtos, Interpreter::_normal_table.table_for(vtos));
  return entry;
}


//------------------------------------------------------------------------------------------------------------------------
// Entry points
//
// Arguments:
//
// ebx: methodOop
//
//
// Stack layout immediately after call:
//
// [ return address     ] <--- esp
// [ parameter n        ]
//   ...
// [ parameter 1        ]
// [ expression stack   ]
//
//
// Stack layout immediately before execution of first bytecode:
// (local variables follow incoming parameters immediately; i.e.
// the return address is moved to the end of the locals).
//
// [ monitor entry      ] <--- esp
//   ...
// [ monitor entry      ]
// [ expr. stack bottom ]
// [ saved esi          ]
// [ current edi        ]
// [ methodOop          ]
// [ saved ebp          ] <--- ebp
// [ return address     ]
// [ local variable m   ]
//   ...
// [ local variable 1   ]
// [ parameter n        ]
//   ...
// [ parameter 1        ] <--- edi

address AbstractInterpreterGenerator::generate_method_entry(AbstractInterpreter::MethodKind kind) {
  // determine code generation flags
  bool inc_counter  = UseRecompilation || CountCompiledCalls;
  bool synchronized = false;
  bool zerolocals   = false;
  bool native_call  = false;
  bool math_call    = false;
  switch (kind) {    
    case Interpreter::zerolocals             : zerolocals  = true;                      break;
    case Interpreter::zerolocals_synchronized: zerolocals  = true; synchronized = true; break;
    case Interpreter::native                 : native_call = true;                      break;
    case Interpreter::native_synchronized    : native_call = true; synchronized = true; break;
    case Interpreter::empty                  :                                          break;
    case Interpreter::accessor               :                                          break;
    case Interpreter::abstract               :                                          break;
    case Interpreter::java_lang_math_sin     : math_call = true;                        break;
    case Interpreter::java_lang_math_cos     : math_call = true;                        break;
    case Interpreter::java_lang_math_sqrt    : math_call = true;                        break;
    default                                  : ShouldNotReachHere();                    break;
  }

  // ebx: methodOop
  // ecx: receiver
  address entry_point = __ pc();

  // abstract method entry
  if (kind == Interpreter::abstract) {    
    // remove return address. Not really needed, since exception handling throws away expression stack
    __ popl(ebx);             
    // throw exception
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_abstract_method_error));
    // the call_VM checks for exception, so we should never return here.
    __ should_not_reach_here();

    return entry_point;
  }

  // mathematical functions inlined by compiler
  // (interpreter must provide identical implementation
  // in order to avoid monotonicity bugs when switching
  // from interpreter to compiler in the middle of some
  // computation)
  //
  // stack: [ ret adr ] <-- esp
  //        [ lo(arg) ]
  //        [ hi(arg) ]
  if (math_call && InlineIntrinsics) {
    if (Universe::is_jdk12x_version()) {
      // Note: For JDK 1.2 StrictMath doesn't exist and Math.sin/cos/sqrt are
      //       native methods.  Interpreter::method_kind(...) does a check for
      //       native methods first before checking for intrinsic methods and
      //       thus will never select this entry point.  Make sure it is not
      //       called accidentally since the SharedRuntime entry points will
      //       not work for JDK 1.2.
      __ should_not_reach_here();
    } else {
      // Note: For JDK 1.3 StrictMath exists and Math.sin/cos/sqrt are
      //       java methods.  Interpreter::method_kind(...) will select
      //       this entry point for the corresponding methods in JDK 1.3.
      __ fld_d(Address(esp, 1*wordSize)); // get argument
      switch (kind) {
	// The original fast sin/cos routines believed the Intel manual and
	// trusted fsin/fcos to return 1 ULP precision or flag if they could
	// not.  Alas, this is not so - outside of PI/4 they need accurate
	// argument reduction or they get increasing less accurate.  

	// The new technique, from Joe Darcy, is to test for being in this
	// narrow range (+/- PI/4).  If so, the Intel hardware instruction is
	// used directly.  If not, the argument must be accurately reduced
	// with a reasonably fast runtime call and then the hardware
	// instruction can be used.  This is faster than calling StrictMath
	// but still fairly meaty.  As always, the interpreter and all
	// compilers must use identical methodology to avoid monotonicity
	// problems when flipping between interpreted and compiled modes.

        // Note that we must not use call_VM_leaf here to call the runtime
        // routines since  call_VM_leaf requires a complete interpreter
        // activation frame in debug mode (was bug 4387334 - gri 12/1/99).
        case Interpreter::java_lang_math_sin :
#ifndef CORE
          __ call(StubRoutines::i486::dsin_fast_wrapper(), relocInfo::runtime_call_type);
#else
          __ push(dtos);     // push double argument on stack
          __ call(CAST_FROM_FN_PTR(address, SharedRuntime::dsin), relocInfo::runtime_call_type);
          __ popl(ebx);      // pop argument (lo word)
          __ popl(ecx);      // pop argument (hi word)
#endif
	  break;
        case Interpreter::java_lang_math_cos :
#ifndef CORE
          __ call(StubRoutines::i486::dcos_fast_wrapper(), relocInfo::runtime_call_type);
#else
          __ push(dtos);     // push double argument on stack
          __ call(CAST_FROM_FN_PTR(address, SharedRuntime::dcos), relocInfo::runtime_call_type);
          __ popl(ebx);      // pop argument (lo word)
          __ popl(ecx);      // pop argument (hi word)
#endif
	  break;
        case Interpreter::java_lang_math_sqrt: 
	  __ fsqrt();
	  break;
        default                              : 
	  ShouldNotReachHere();
      }

      // done, result in FPU ST(0)
      if (UseC2CallingConventions) {
        __ ret(0);           // return w/o popping argument
      } else {
        __ ret(2*wordSize);  // return and pop double argument
      }
    }
    return entry_point;    
  }

  // do nothing for empty methods (do not even increment invocation counter)
  if (kind == Interpreter::empty && UseFastEmptyMethods) {
    // Code: _return
    // _return
    if (UseC2CallingConventions) {
      // return w/o popping parameters
      __ ret(0);
    } else {
      // Note: This works now for empty methods with any parameter list.
      //       However, empty methods mostly occur as initializers that
      //       have always one parameter (the receiver) only. Maybe one
      //       should use the fast empty method optimization only for
      //       methods with exactly one parameter (i.e., the frequent
      //       case) and then one could save 3 instructions here.
      //       Difference is probably not measurable - if we eventually
      //       go with the old calling conventions again, we should choose
      //       the simple "90% there" solution here.
      // get parameter size
      __ load_unsigned_word(ecx, Address(ebx, methodOopDesc::size_of_parameters_offset_in_bytes()));
      // pop return address
      __ popl(edx);
      // pop parameters
      __ leal(esp, Address(esp, ecx, Address::times_4));
      // return
      __ jmp(edx);
    }
    return entry_point;
  }

  // do fastpath for resolved accessor methods
  if (kind == Interpreter::accessor && UseFastAccessorMethods) {
    // Code: _aload_0, _(i|a)getfield, _(i|a)return or any rewrites thereof; parameter size = 1
    // Note: We can only use this code if the getfield has been resolved
    //       and if we don't have a null-pointer exception => check for
    //       these conditions first and use slow path if necessary.
    Label slow_path;
    // ebx: method
    // ecx: receiver
    __ movl(eax, Address(esp, wordSize));
    __ movl(edi, Address(ebx, methodOopDesc::constants_offset_in_bytes()));
    // read first instruction word and extract bytecode @ 1 and index @ 2
    __ movl(esi, Address(ebx, methodOopDesc::codes_offset_in_bytes()));
    __ movl(edx, esi);
    __ shrl(esi, 1*BitsPerByte);
    __ andl(esi, 0xFF);
    assert(ConstantPoolCacheEntry::size() == 4, "adjust shift below");
    __ shrl(edx, 2*BitsPerByte - 2);
    __ andl(edx, 0xFFFF << 2);
    __ movl(edi, Address(edi, constantPoolOopDesc::cache_offset_in_bytes()));
    // eax: local 0
    // ebx: method
    // ecx: receiver - do not destroy since it is needed for slow path!
    // edx: constant pool cache index
    // edi: constant pool cache
    // esi: bytecode @ 1

    // check if getfield has been resolved and read constant pool cache entry
    __ cmpl(esi, Bytecodes::_getfield);
    __ jcc(Assembler::equal, slow_path);
    // Note: constant pool entry is not valid before bytecode is resolved
    __ movl(esi, Address(edi, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + ConstantPoolCacheEntry::f2_offset_in_bytes()));

    // check if local 0 != NULL and read field
    __ testl(eax, eax);
    __ jcc(Assembler::zero, slow_path);
    __ movl(eax, Address(eax, esi, Address::times_4));

    // _ireturn/_areturn
    if (UseC2CallingConventions) {
      __ ret(0);
    } else {
      __ ret(wordSize);
    }

    // slow implementation
    // ebx: method
    // ecx: receiver
    __ bind(slow_path);
  }  

#ifndef CORE
  // check if compiled code exists
  Label run_compiled_code;
  if ((!native_call || !PreferInterpreterNativeStubs) && !CompileTheWorld) {
    __ movl(edx, Address(ebx, methodOopDesc::compiled_code_offset_in_bytes()));
    __ testl(edx, edx);
    __ jcc(Assembler::notZero, run_compiled_code);
  }
#endif

  const Address codes             (ebx, methodOopDesc::codes_offset_in_bytes());
  const Address size_of_parameters(ebx, methodOopDesc::size_of_parameters_offset_in_bytes());
  const Address size_of_locals    (ebx, methodOopDesc::size_of_locals_offset_in_bytes());
#ifndef CORE
  const Address invocation_counter(ebx, methodOopDesc::invocation_counter_offset_in_bytes() + InvocationCounter::counter_offset_in_bytes());
#endif
  const Address access_flags      (ebx, methodOopDesc::access_flags_offset_in_bytes());

  const Address monitor_block_top (ebp, frame::interpreter_frame_monitor_block_top_offset * wordSize);
  const Address monitor_block_bot (ebp, frame::interpreter_frame_initial_sp_offset        * wordSize);
  const int entry_size            = frame::interpreter_frame_monitor_size() * wordSize;

  // get parameter size (always needed)
  __ load_unsigned_word(ecx, size_of_parameters);

  // ebx: methodOop
  // ecx: size of parameters
  __ popl(eax);                                       // get return address
  if (native_call) {
    __ movl(edx, ecx);                                // for natives the size of locals is zero, but should be same as parameter size
  } else {
    __ load_unsigned_word(edx, size_of_locals);       // get size of locals in words
  }

  // compute beginning of parameters (edi)
  __ leal(edi, Address(esp, ecx, Address::times_4, -wordSize));
  __ movl(esi, esp);                                  // remember sender sp

  // allocate space for locals
  if (zerolocals || ClearInterpreterLocals) {
    // explicitly initialize locals
    Label exit, loop;
    __ subl(edx, ecx);                                // edx = no. of additional locals
    __ jcc(Assembler::lessEqual, exit);               // do nothing if edx <= 0
    __ bind(loop);
    __ pushl((int)NULL);                              // initialize local variables
    __ decl(edx);                                     // until everything initialized
    __ jcc(Assembler::greater, loop);
    __ bind(exit);
  } else {
    // only adjust stack pointer    
    __ negl(edx);
    __ leal(esp, Address(edi, edx, Address::times_4, wordSize));
  }

  // add 2 zero-initialized slots for native calls
  if (native_call) {
    __ pushl((int)NULL);                              // slot for native result type info (setup via runtime)
    __ pushl((int)NULL);                              // slot for static native method holder mirror (setup via runtime)
  }

  // initialize fixed part of activation frame
#ifndef CORE
  if (inc_counter) __ movl(ecx, invocation_counter);  // (pre-)fetch invocation count
#endif
  __ pushl(eax);                                      // save return address  
  __ enter();                                         // save old & set new ebp
  __ pushl(esi);                                      // set sender sp
  __ leal(esi, codes);                                // get code base
  __ pushl(ebx);                                      // save methodOop
  __ movl(edx, Address(ebx, methodOopDesc::constants_offset_in_bytes()));
  __ movl(edx, Address(edx, constantPoolOopDesc::cache_offset_in_bytes()));
  __ pushl(edx);                                      // set constant pool cache
  __ pushl(edi);                                      // set locals pointer
  __ pushl(esi);                                      // set bcp
  __ pushl(0);                                        // reserve word for pointer to expression stack bottom
  __ movl(Address(esp), esp);                         // set expression stack bottom

  // make sure method is not native & not abstract
#ifdef ASSERT
  __ movl(eax, access_flags);
  if (native_call) {
    Label L;
    __ testl(eax, JVM_ACC_NATIVE);
    __ jcc(Assembler::notZero, L);
    __ stop("tried to execute non-native method as native");
    __ bind(L);
  } else {
    Label L;
    __ testl(eax, JVM_ACC_NATIVE);
    __ jcc(Assembler::zero, L);
    __ stop("tried to execute native method as non-native");
    __ bind(L);
  }
  { Label L;
    __ testl(eax, JVM_ACC_ABSTRACT);
    __ jcc(Assembler::zero, L);
    __ stop("tried to execute abstract method in interpreter");
    __ bind(L);
  }
#endif


#ifndef CORE
  // increment invocation count & check for overflow
  //
  // Note: checking for negative value instead of overflow
  //       so we have a 'sticky' overflow test
  //
  // ebx: method
  // ecx: invocation counter
  Label invocation_counter_overflow;
  if (inc_counter) {
#ifdef COMPILER2
    if( CountInterpInvocations ) 
      __ incl(Address(ebx,methodOopDesc::interpreter_invocation_counter_offset_in_bytes()));
#endif
    // Update standard invocation counters
    __ increment(ecx, InvocationCounter::count_increment);
    __ movl(invocation_counter, ecx);            // save invocation count
    __ cmpl(ecx, Address(int(&InvocationCounter::InterpreterInvocationLimit), relocInfo::none));
    __ jcc(Assembler::aboveEqual, invocation_counter_overflow);
  }

#endif

  // check for synchronized methods
  //
  // Note: This must happen *after* invocation counter check, since
  //       when overflow happens, the method should not be locked.
  if (synchronized) {
    // synchronize method
    #ifdef ASSERT
      { Label L;
        __ movl(eax, access_flags);
        __ testl(eax, JVM_ACC_SYNCHRONIZED);
        __ jcc(Assembler::notZero, L);
        __ stop("method doesn't need synchronization");
        __ bind(L);
      }
    #endif
    // get synchronization object
    { Label done;
      const int mirror_offset = klassOopDesc::klass_part_offset_in_bytes() + Klass::java_mirror_offset_in_bytes();
      __ movl(eax, access_flags);
      __ testl(eax, JVM_ACC_STATIC);
      __ movl(eax, Address(edi));                     // get receiver (assume this is frequent case)
      __ jcc(Assembler::zero, done);
      __ movl(eax, Address(ebx, methodOopDesc::constants_offset_in_bytes()));
      __ movl(eax, Address(eax, constantPoolOopDesc::pool_holder_offset_in_bytes()));
      __ movl(eax, Address(eax, mirror_offset));
      __ bind(done);
    }
    // add space for monitor & lock
    __ subl(esp, entry_size);                                             // add space for a monitor entry
    __ movl(monitor_block_top, esp);                                      // set new monitor block top
    __ movl(Address(esp, BasicObjectLock::obj_offset_in_bytes()), eax);   // store object
    __ movl(edx, esp);                                                    // object address
    __ lock_object(edx);          
  } else {
    // no synchronization necessary
#ifdef ASSERT
      { Label L;
        __ movl(eax, access_flags);
        __ testl(eax, JVM_ACC_SYNCHRONIZED);
        __ jcc(Assembler::zero, L);
        __ stop("method needs synchronization");
        __ bind(L);
      }
#endif
  }

  // Quick & dirty stack overflow checking: bang the stack & handle trap.
  // Note that we do the banging after the frame is setup, since the exception handling code expects to 
  // find a valid interpreter frame (with valid ebp) on the stack. Doing the banging earlier fails if
  // the caller frame is not an interpreter frame.
  __ movl(Address(esp, -os::vm_page_size()), eax);

  // start execution
#ifdef ASSERT
  { Label L;
    __ movl(eax, monitor_block_top);
    __ cmpl(eax, esp);
    __ jcc(Assembler::equal, L);
    __ stop("broken stack frame setup in interpreter");
    __ bind(L);
  }
#endif

  // jvmdi/jvmpi support
  __ notify_method_entry();
 
  if (native_call) {
    // work registers
    const Register method = ebx;
    const Register thread = edi;
    const Register t      = ecx;    

    // allocate space for parameters
    __ get_method(method);
    __ verify_oop(method);
    __ load_unsigned_word(t, Address(method, methodOopDesc::size_of_parameters_offset_in_bytes()));
    __ shll(t, 2);
    __ subl(esp, t);

    // get signature handler
    { Label L;
      __ movl(t, Address(method, methodOopDesc::signature_handler_offset_in_bytes()));
      __ testl(t, t);
      __ jcc(Assembler::notZero, L);
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::prepare_native_call), method);
      __ get_method(method);
      __ movl(t, Address(method, methodOopDesc::signature_handler_offset_in_bytes()));
      __ bind(L);
    }

    // call signature handler
    assert(InterpreterRuntime::SignatureHandlerGenerator::from() == edi, "adjust this code");
    assert(InterpreterRuntime::SignatureHandlerGenerator::to  () == esp, "adjust this code");
    assert(InterpreterRuntime::SignatureHandlerGenerator::temp() == t  , "adjust this code");
    // The generated handlers do not touch EBX (the method oop).
    // However, large signatures cannot be cached and are generated 
    // each time here.  The slow-path generator will blow EBX
    // sometime, so we must reload it after the call.
    __ call(t, relocInfo::none);
    __ get_method(method);	// slow path call blows EBX on DevStudio 5.0

    // result handler is in eax
    // set result handler
    __ movl(Address(ebp, (frame::interpreter_frame_mirror_offset + 1)*wordSize), eax);

    // pass mirror handle if static call
    { Label L;
      const int mirror_offset = klassOopDesc::klass_part_offset_in_bytes() + Klass::java_mirror_offset_in_bytes();
      __ movl(t, Address(method, methodOopDesc::access_flags_offset_in_bytes()));
      __ testl(t, JVM_ACC_STATIC);
      __ jcc(Assembler::zero, L);
      // get mirror
      __ movl(t, Address(method, methodOopDesc:: constants_offset_in_bytes()));
      __ movl(t, Address(t, constantPoolOopDesc::pool_holder_offset_in_bytes()));
      __ movl(t, Address(t, mirror_offset));
      // copy mirror into activation frame
      __ movl(Address(ebp, frame::interpreter_frame_mirror_offset * wordSize), t);
      // pass handle to mirror
      __ leal(t, Address(ebp, frame::interpreter_frame_mirror_offset * wordSize));
      __ pushl(t);
      __ bind(L);
    }

    // get native function entry point
    { Label L;
      __ movl(eax, Address(method, methodOopDesc::native_function_offset_in_bytes()));
      __ testl(eax, eax);
      __ jcc(Assembler::notZero, L);
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::prepare_native_call), method);
      __ get_method(method);
      __ verify_oop(method);
      __ movl(eax, Address(method, methodOopDesc::native_function_offset_in_bytes()));
      __ bind(L);
    }

    // pass JNIEnv
    __ get_thread(thread);
    __ leal(t, Address(thread, JavaThread::jni_environment_offset_in_bytes()));
    __ pushl(t);

    // reset handle block
    __ movl(t, Address(thread, JavaThread::active_handles_offset_in_bytes()));
    __ movl(Address(t, JNIHandleBlock::top_offset_in_bytes()), 0);

    // set_last_Java_frame_before_call
    __  movl(Address(thread, JavaThread::last_Java_fp_offset_in_bytes()), ebp);
    __  movl(Address(thread, JavaThread::last_Java_sp_offset_in_bytes()), esp);

    // change thread state
#ifdef ASSERT
    { Label L;
      __ movl(t, Address(thread, JavaThread::thread_state_offset_in_bytes()));
      __ cmpl(t, _thread_in_Java);
      __ jcc(Assembler::equal, L);
      __ stop("Wrong thread state in native stub");
      __ bind(L);
    }
#endif
    // Push return address on stack. We need to do this before we change thread-state, to
    // make sure that the safepoint code will always see a valid return address on the stack.
    
    // Change state to native (we save the return address in the thread, since it might not
    // be pushed on the stack when we do a a stack traversal). It is enough that the pc()
    // points into the right code segment. It does not have to be the correct return pc.
    __ movl(Address(thread, JavaThread::last_native_pc_offset_in_bytes()), (int) __ pc());  
    __ movl(Address(thread, JavaThread::thread_state_offset_in_bytes()), _thread_in_native);    

    __ call(eax, relocInfo::none);

    // result potentially in edx:eax or ST0
    __ get_method(method);    
    __ get_thread(thread);

    // restore esi to have legal interpreter frame, i.e., bci == 0 <=> esi == code_base()
    __ leal(esi, Address(method, methodOopDesc::codes_offset_in_bytes()));

    // save potential result in ST(0) & edx:eax
    // (if result handler is the T_FLOAT handler (which is the same as the T_DOUBLE handler), result must be in ST0 -
    // the check is necessary to avoid potential Intel FPU overflow problems by saving/restoring 'empty' FPU registers)
    // It is safe to do this push because state is _thread_in_native and return address will be found
    // via _last_native_pc and not via _last_jave_sp

    { Label L;
      __ cmpl(Address(ebp, (frame::interpreter_frame_mirror_offset + 1)*wordSize), (int)AbstractInterpreter::result_handler(T_FLOAT));
      __ jcc(Assembler::notEqual, L);
      __ push(dtos);      
      __ bind(L);
    }
    __ push(ltos);

    // change thread state
    __ movl(Address(thread, JavaThread::thread_state_offset_in_bytes()), _thread_in_native_trans);

    // check for safepoint operation in progress
    { Label L;
      __ cmpl(Address((int)SafepointSynchronize::address_of_state(), relocInfo::none), SafepointSynchronize::_not_synchronized);
      __ jcc(Assembler::equal, L);
      // Don't use call_VM as it will see a possible pending exception and forward it
      // and never return here preventing us from clearing _last_native_pc down below.
      // Also can't use call_VM_leaf either as it will check to see if esi & edi are
      // preserved and correspond to the bcp/locals pointers. So we do a runtime call
      // by hand.
      //
      __ pushl(thread);
      __ call(CAST_FROM_FN_PTR(address, SafepointSynchronize::block), relocInfo::runtime_call_type);
      __ increment(esp, wordSize);

      __ get_method(method);
      __ get_thread(thread);
      __ bind(L);
    }

    // change thread state
    __ movl(Address(thread, JavaThread::thread_state_offset_in_bytes()), _thread_in_Java);

    // Clear last_native_pc after call. This is crucial, so we can detect if the last_native_pc is
    // used or not. It is, e.g., not used when calling native code directly from C++ code.
    __ movl(Address(thread, JavaThread::last_native_pc_offset_in_bytes()), (int)NULL);  

    // reset_last_Java_frame
    __ movl(Address(thread, JavaThread::last_Java_sp_offset_in_bytes()), (int)NULL);
    // must clear fp, so that compiled frames are not confused; it is possible
    // that we need it only for debugging
    __ movl(Address(thread, JavaThread::last_Java_fp_offset_in_bytes()), (int)NULL);

    // handle exceptions (exception handling will handle unlocking!)
    { Label L;
      __ cmpl(Address(thread, Thread::pending_exception_offset_in_bytes()), (int)NULL);
      __ jcc(Assembler::zero, L);
      // Note: At some point we may want to unify this with the code used in call_VM_base();
      //       i.e., we should use the StubRoutines::forward_exception code. For now this
      //       doesn't work here because the esp is not correctly set at this point.
      __ MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_pending_exception));
      __ should_not_reach_here();
      __ bind(L);
    }

    // do unlocking if necessary
    { Label L;
      __ movl(t, Address(method, methodOopDesc::access_flags_offset_in_bytes()));
      __ testl(t, JVM_ACC_SYNCHRONIZED);
      __ jcc(Assembler::zero, L);
      // the code below should be shared with interpreter macro assembler implementation
      { Label unlock;
        // BasicObjectLock will be first in list, since this is a synchronized method. However, need
        // to check that the object has not been unlocked by an explicit monitorexit bytecode.        
        const Address monitor(ebp, frame::interpreter_frame_initial_sp_offset * wordSize - (int)sizeof(BasicObjectLock));                            
        __ leal(edx, monitor);                   // address of first monitor
  
        __ movl(t, Address(edx, BasicObjectLock::obj_offset_in_bytes()));
        __ testl(t, t);
        __ jcc(Assembler::notZero, unlock);
                                  
        // Entry already unlocked, need to throw exception
        __ MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
        __ should_not_reach_here();
    
        __ bind(unlock);        
        __ unlock_object(edx);             
      }
      __ bind(L);
    }    

    // jvmdi/jvmpi support
    // Note: This must happen _after_ handling/throwing any exceptions since
    //       the exception handler code notifies the runtime of method exits
    //       too. If this happens before, method entry/exit notifications are
    //       not properly paired (was bug - gri 11/22/99).
    __ notify_method_exit(vtos);

    // restore potential result in edx:eax, call result handler to restore potential result in ST0 & handle result
    __ pop(ltos);
    __ movl(t, Address(ebp, (frame::interpreter_frame_mirror_offset + 1)*wordSize));
    __ call(t, relocInfo::none);

    // remove activation
    if (UseC2CallingConventions) {
      __ movl(t, Address(ebp, frame::interpreter_frame_sender_sp_offset * wordSize)); // get sender sp
      __ leave();                                // remove frame anchor
      __ popl(esi);                              // get return address
      __ movl(esp, t);                           // set sp to sender sp
    } else {
      __ movl(t, Address(ebp, frame::interpreter_frame_locals_offset * wordSize)); // get sender sp
      __ leave();                                // remove frame anchor
      __ popl(esi);                              // get return address
      __ leal(esp, Address(t, wordSize));        // set sp to sender sp
    }
    __ jmp(esi);

  } else {
    __ dispatch_next(vtos);
  }

#ifndef CORE
  // invocation counter overflow
  if (inc_counter) {
    __ bind(invocation_counter_overflow);
    // InterpreterRuntime::invocation_counter_overflow takes two arguments,
    // the first indicates if the counter overflow occurs at a backwards branch (false)
    // and the second is only used when the first is true.  We pass zero for both.
    // The call returns the address of the verified entry point for the method or NULL
    // if the compilation did not complete (either went background or bailed out).
    __ movl(eax, (int)false);
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::invocation_counter_overflow), eax, eax);
    __ movl(ebx, Address(ebp, method_offset));   // restore methodOop
  }
#endif

  // method has been compiled - remove activation frame
  // (leave return address on stack) and continue at
  // verified entry point (eax).
  //
  // Note: continuation at verified entry point works if the method that has been
  //       compiled is the right one (in case of virtual calls); i.e., the inline
  //       cache check must have happened before the invocation counter overflow
  //       check.
  __ load_unsigned_word(ecx, size_of_parameters);     // get size of parameters in words
  __ leave();                                         // remove stack frame
  __ popl(edx);                                       // get return address
  __ negl(ecx);                                       // so we can subtract in next step
  __ leal(esp, Address(edi, ecx, Address::times_4, wordSize));  // adjust esp so that it points to last argument
  __ pushl(edx);                                      // push return address after arguments

  __ movl(ecx, Address(edi));                         // restore potential receiver

  // Preserve invariant that esi/edi contain bcp/locals of sender frame  
  __ jmp(entry_point, relocInfo::none);

#ifndef CORE
  // compiled code for method exists
  // ebx: methodOop
  // edx: nmethod
  __ bind(run_compiled_code);         

  // make sure interpreter entry point exists
  { Label L;
    __ movl(eax, Address(edx, nmethod::interpreter_entry_point_offset()));
    __ testl(eax, eax);
    __ jcc(Assembler::notZero, L);

#ifdef COMPILER1
    __ should_not_reach_here();
#endif
    __ leal(eax, Address(esp, wordSize));             // set last_Java_sp to last argument (return address on tos)

    // Do not use __ call_VM here, since it will call to the InterpreterMacroAssembler, which
    // will save and restore edi/esi relative to the current ebp (which could be compiled)
    __ super_call_VM(ebx, eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::nmethod_entry_point), ebx, edx);    

    // Note: At this point, the nmethod may have been recompiled once more; i.e., if one would
    //       look at the nmethod's interpreter entry point, it could be NULL! However, since
    //       the interpreter entry point is simply the corresponding i2c adapter, which is
    //       dependent of the method's signature only, we don't care. If the nmethod has been
    //       deoptimized, we will end up in a zombie nmethod, which is ok, too.
    __ bind(L);
    __ movl(edx, eax);
    // ebx: methodOop
    // edx: interpreter nmethod entry

#ifdef ASSERT
    { Label L;
      __ testl(edx, edx);
      __ jcc(Assembler::notZero, L);
      __ stop("interpreter entry point == NULL");
      __ bind(L);
    }
#endif
  }
  // ebx: methodOop
  // edx: interpreter nmethod entry
  __ movl(eax, ebx);                                  // compiled code expects methodOop in eax
  __ jmp(edx);                                        // enter compiled code via i2c adapter
  
#endif

  return entry_point;
}


#ifndef CORE
// This method tells the deoptimizer how big an interpreted frame must be:
int AbstractInterpreter::size_activation(methodOop method, int tempcount, int moncount, int callee_param_size, int callee_locals) {
  // fixed size of an interpreter frame:
  int locals = method->max_locals() - method->size_of_parameters();
  int overhead = frame::sender_sp_offset - frame::interpreter_frame_initial_sp_offset;
  int size = locals + overhead + moncount*frame::interpreter_frame_monitor_size() + tempcount;
  return size;
}


int AbstractInterpreter::layout_activation_impl(methodOop method, int tempcount, int moncount, frame* f, int callee_param_size, int callee_locals) {
  // Note: This calculation must exactly parallel the frame setup
  // in AbstractInterpreterGenerator::generate_method_entry.
  // If f!=NULL, set up the method, locals, and monitors.
  // The frame f, if not NULL, is guaranteed to be the right size,
  // as determined by a previous call to this method.

  // fixed size of an interpreter frame:
  int max_locals = method->max_locals();
  int overhead = frame::sender_sp_offset - frame::interpreter_frame_initial_sp_offset;
  int size = overhead + max_locals - method->size_of_parameters() + moncount*frame::interpreter_frame_monitor_size() + tempcount;
  if (f != NULL) {
    frame caller_frame = (*f);

    // compute frame for the interpreter
    jint* fp = caller_frame.sp()
	     - max_locals           // make room for locals
             + method->size_of_parameters()
	     - 1                    // make room for the return address
	     - 1;                   // make room for link  
    jint* sp = caller_frame.sp()
	     - size;

    address random_pc = return_entry(vtos, 0);   // will be overwritten

    frame interpreter_frame(sp, fp, random_pc);

    (*f) = interpreter_frame;        // copy out the interpreter frame

    interpreter_frame.interpreter_frame_set_method(method);
    jint* locals = interpreter_frame.sender_sp() + max_locals - 1;

    interpreter_frame.interpreter_frame_set_locals(locals);
    BasicObjectLock* montop = interpreter_frame.interpreter_frame_monitor_begin();
    interpreter_frame.interpreter_frame_set_monitor_end(montop - moncount);

    interpreter_frame.set_interpreter_frame_sender_sp(caller_frame.sp());
    *interpreter_frame.interpreter_frame_cache_addr() = 
      method->constants()->cache();
  }
  return size;
}
#endif


//------------------------------------------------------------------------------------------------------------------------
// Exceptions

void AbstractInterpreterGenerator::generate_throw_exception() {
  // Entry point in previous activation (i.e., if the caller was interpreted)
  Interpreter::_rethrow_exception_entry = __ pc();
  // eax: exception
  // edx: return address/pc that threw exception
  __ restore_bcp();                              // esi points to call/send
  __ restore_locals();

  // Entry point for exceptions thrown within interpreter code
  Interpreter::_throw_exception_entry = __ pc();  
  // expression stack is undefined here
  // eax: exception
  // esi: exception bcp
  __ verify_oop(eax);

  // expression stack must be empty before entering the VM in case of an exception
  __ empty_expression_stack();
  __ empty_FPU_stack();
  // find exception handler address and preserve exception oop
  __ call_VM(edx, CAST_FROM_FN_PTR(address, InterpreterRuntime::exception_handler_for_exception), eax);
  // eax: exception handler entry point
  // edx: preserved exception oop
  // esi: bcp for exception handler
  __ pushl(edx);                                 // push exception which is now the only value on the stack
  __ jmp(eax);                                   // jump to exception handler (may be _remove_activation_entry!)

  // If the exception is not handled in the current frame the frame is removed and
  // the exception is rethrown (i.e. exception continuation is _rethrow_exception).
  //
  // Note: At this point the bci is still the bxi for the instruction which caused
  //       the exception and the expression stack is empty. Thus, for any VM calls
  //       at this point, GC will find a legal oop map (with empty expression stack).

  // In current activation
  // tos: exception
  // esi: exception bcp
  Interpreter::_remove_activation_entry = __ pc();
  
  // preserve exception over this code sequence
  __ popl(eax);
  __ get_thread(ecx);
  __ movl(Address(ecx, JavaThread::vm_result_offset_in_bytes()), eax);
  // remove the activation (without doing throws on illegalMonitorExceptions)
  __ remove_activation(vtos, edx, false);
  // restore exception
  __ get_thread(ecx);
  __ movl(eax, Address(ecx, JavaThread::vm_result_offset_in_bytes()));
  __ movl(Address(ecx, JavaThread::vm_result_offset_in_bytes()), (int)NULL);
  __ verify_oop(eax);

  // Inbetween activations - previous activation type unknown yet
  // compute continuation point - the continuation point expects
  // the following registers set up:
  //
  // eax: exception
  // edx: return address/pc that threw exception
  // esp: expression stack of caller
  // ebp: ebp of caller
  __ pushl(eax);                                 // save exception
  __ pushl(edx);                                 // save return address
  __ super_call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address), edx);
  __ movl(ebx, eax);                             // save exception handler
  __ popl(edx);                                  // restore return address
  __ popl(eax);                                  // restore exception
  // Note that an "issuing PC" is actually the next PC after the call
  __ jmp(ebx);                                   // jump to exception handler of caller
}


//------------------------------------------------------------------------------------------------------------------------
// Helper for vtos entry point generation

void AbstractInterpreterGenerator::set_vtos_entry_points(Template* t, address& iep, address& lep, address& fep, address& dep, address& vep) {
  assert(t->is_valid() && t->tos_in() == vtos, "illegal template");
  Label L;
  fep = __ pc(); __ push(ftos); __ jmp(L);
  dep = __ pc(); __ push(dtos); __ jmp(L);
  lep = __ pc(); __ pushl(edx); // fall through
  iep = __ pc(); __ pushl(eax); // fall through
  vep = __ pc(); __ bind(L);    // fall through
  generate_and_dispatch(t);
}


//------------------------------------------------------------------------------------------------------------------------
// Generation of individual instructions

// helpers for generate_and_dispatch


InterpreterGenerator::InterpreterGenerator(StubQueue* code) 
 : AbstractInterpreterGenerator(code) {
   generate_all(); // down here so it can be "virtual"
}

//------------------------------------------------------------------------------------------------------------------------
// Non-product code
#ifndef PRODUCT

address AbstractInterpreterGenerator::generate_bci_histogram_entry() {
  // This code is only used when PrintBCIHistogram option is set
  address entry = __ pc();
  
  // eax: may contain TOS
  // ebx: free
  // ecx: free
  // edx: may contain TOS
  // edi: pointer to locals
  // esi: current bcp
  __ get_method(ebx);                                                       // get method
  __ movl(ecx, Address(ebx, methodOopDesc::bci_histogram_offset_in_bytes())); // get histo array
  __ subl(ebx, esi);                                                        // ebx = method - bcp
  __ negl(ebx);                                                             // ebx = bcp - method
  // increment array entry, correct base since index in ebx is not bci relative to beginning of method
  const int array_base_offset = 0; // since we are using C arrays, we need no offset
  __ incl(Address(ecx, ebx, Address::times_4, (array_base_offset - methodOopDesc::codes_offset_in_bytes()) * wordSize));

  // return
  __ ret(0);

  return entry;
}
#endif


address AbstractInterpreterGenerator::generate_trace_code(TosState state) {
  address entry = __ pc();

  // prepare expression stack
  __ popl(ecx);         // pop return address so expression stack is 'pure'
  __ push(state);       // save tosca

  // pass arguments & call tracer
  __ call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::trace_bytecode), ecx);
  __ movl(ecx, eax);    // make sure return address is not destroyed by pop(state)

  // restore expression stack
  __ pop(state);        // restore tosca

  // return
  __ jmp(ecx);

  return entry;
}


// Non-product code
#ifndef PRODUCT
void AbstractInterpreterGenerator::count_bytecode() { 
  __ incl(Address((int)&BytecodeCounter::_counter_value, relocInfo::none)); 
}


void AbstractInterpreterGenerator::histogram_bytecode(Template* t) { 
  __ incl(Address((int)&BytecodeHistogram::_counters[t->bytecode()], relocInfo::none));
}


void AbstractInterpreterGenerator::histogram_bci() {
  // Copied from trace_bytecode below, but the purpose here is to
  // gather a per-bci profile.
  address entry = Interpreter::bci_histogram_entry();
  guarantee(entry != NULL, "entry must have been generated");
  __ call(entry, relocInfo::none);
}


void AbstractInterpreterGenerator::histogram_bytecode_pair(Template* t) { 
  __ movl(ebx, Address((int)&BytecodePairHistogram::_index, relocInfo::none));
  __ shrl(ebx, BytecodePairHistogram::log2_number_of_codes);
  __ orl(ebx, ((int)t->bytecode()) << BytecodePairHistogram::log2_number_of_codes);
  __ movl(Address((int)&BytecodePairHistogram::_index, relocInfo::none), ebx);  
  __ incl(Address(noreg, ebx, Address::times_4, (int)BytecodePairHistogram::_counters));
}
#endif


void AbstractInterpreterGenerator::trace_bytecode(Template* t) {
  // Call a little run-time stub to avoid blow-up for each bytecode.
  // The run-time runtime saves the right registers, depending on
  // the tosca in-state for the given template.
  address entry = Interpreter::trace_code(t->tos_in());
  assert(entry != NULL, "entry must have been generated");
  __ call(entry, relocInfo::none);
}


// Non-product code
#ifndef PRODUCT
void AbstractInterpreterGenerator::stop_interpreter_at() {
  Label L;
  __ cmpl(Address(int(&BytecodeCounter::_counter_value), relocInfo::none), StopInterpreterAt);
  __ jcc(Assembler::notEqual, L);
  __ int3();
  __ bind(L);
}
#endif
