/*
 * @(#)nativeInst_i486.cpp	1.43 01/02/09 14:43:03
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_nativeInst_i486.cpp.incl"

void NativeInstruction::wrote(int offset) {
  ICache::invalidate_word(addr_at(offset));
}


void NativeCall::verify() {
  // make sure code pattern is actually a call imm32 instruction
  if ((char_at(0)&0xFF) != instruction_code) {
    tty->print_cr("Addr: 0x%x   Code: 0x%x", instruction_address(), char_at(0)&0xFF);
    fatal("not a call imm32");
  }
}

 
bool NativeCall::is_call_at(address instr) {
  return ((*instr) & 0xFF) == NativeCall::instruction_code;
}


void NativeCall::print() {
  tty->print_cr("0x%x: call 0x%x", instruction_address(), destination());
}


bool NativeCall::destination_is_compiled_verified_entry_point() {
#ifndef CORE
  nmethod* callee = CodeCache::find_nmethod(destination());
  if (callee == NULL) {
    return false;
  } else {
    return destination() == callee->verified_entry_point();
  }
#else
  return false;
#endif
}

bool NativeCall::is_call_to_stub() const {
#ifndef CORE
  nmethod* caller_method = CodeCache::find_nmethod(instruction_address());
  assert(caller_method != NULL, "wrong instruction");
  return (caller_method->stub_contains(destination()));
#else
  return false;
#endif
}

// Inserts a native call instruction at a given pc
void NativeCall::insert(address code_pos, address entry) {
  *code_pos = instruction_code;
  *((long *)(code_pos+1)) = (int)entry - ((int)code_pos + 1 + sizeof(long));

  ICache::invalidate_range(code_pos, NativeCall::instruction_size);
}

// MT-safe patching of a call instruction.
// First patches first word of instruction to two jmp's that jmps to them
// selfs (spinlock). Then patches the last byte, and then atomicly replaces
// the jmp's with the first 4 byte of the new instruction.
void NativeCall::replace_mt_safe(address instr_addr, address code_buffer) {
   assert (instr_addr != NULL, "illegal address for code patching");
#ifdef ASSERT
   NativeCall* n_call =  nativeCall_at (instr_addr); // checking that it is a call 
#endif
      
   // Tempoary code
   unsigned char patch[4];
   assert(sizeof(patch)==sizeof(jint), "sanity check");
   patch[0] = 0xEB;       // jmp rel8
   patch[1] = 0xFE;       // jmp to self
   patch[2] = 0xEB;
   patch[3] = 0xFE;
   
   // First patch dummy jmp in place
   *(jint*)instr_addr = *(jint *)patch;

   // Patch 4th byte
   address byte_4_adr = instr_addr+4;
   *byte_4_adr = code_buffer[4];

   // Patch bytes 0-3
   *(jint*)instr_addr = *(jint *)code_buffer;  

#ifdef ASSERT
   // verify patching
   for ( int i = 0; i < NativeCall::instruction_size; i++) { // bytewise comparing
     address ptr = (address)((int)code_buffer + i);     
     int a_byte = (*ptr) & 0xFF;
     assert(*((address)((int)instr_addr + i)) == a_byte, "mt safe patching failed");
   }
#endif

   ICache::invalidate_range(instr_addr, NativeCall::instruction_size);
}

void NativeMovConstReg::verify() {
  // make sure code pattern is actually a mov reg, imm32 instruction
  u_char test_byte = *(u_char*)instruction_address();
  u_char test_byte_2 = test_byte & ( 0xff ^ register_mask);
  if (test_byte_2 != instruction_code) fatal("not a mov reg, imm32");
}


void NativeMovConstReg::print() {
  tty->print_cr("0x%x: mov reg, 0x%x", instruction_address(), data());
}

//-------------------------------------------------------------------

void NativeMovRegMem::copy_instruction_to(address new_instruction_address) {
  for (int i = 0; i < instruction_size; i++) {
    *(new_instruction_address + i) = *(address(this) + i);
  }
}

void NativeMovRegMem::verify() {
  // make sure code pattern is actually a mov [reg+offset], reg instruction
  u_char test_byte = *(u_char*)instruction_address();
  if ( ! ( (test_byte == instruction_code_reg2mem) 
      || (test_byte == instruction_code_mem2reg) 
      || (test_byte == instruction_code_float_s) 
      || (test_byte == instruction_code_float_d) ) ) {
	  fatal ("not a mov [reg+offs], reg instruction");
  }
}


void NativeMovRegMem::print() {
  tty->print_cr("0x%x: mov reg, [reg + %x]", instruction_address(), offset());
}

//-------------------------------------------------------------------

void NativeLoadAddress::verify() {
  // make sure code pattern is actually a mov [reg+offset], reg instruction
  u_char test_byte = *(u_char*)instruction_address();
  if ( ! (test_byte == instruction_code) ) {
	  fatal ("not a lea reg, [reg+offs] instruction");
  }
}


void NativeLoadAddress::print() {
  tty->print_cr("0x%x: lea [reg + %x], reg", instruction_address(), offset());
}

//--------------------------------------------------------------------------------

void NativeJump::verify() {
  if (*(u_char*)instruction_address() != instruction_code) fatal("not a jump instruction");  
}


void NativeJump::insert(address code_pos, address entry) {
  *code_pos = 0xE9;
  *((long *)(code_pos+1)) = (int)entry - ((int)code_pos + 1 + sizeof(long));
  ICache::invalidate_range(code_pos, NativeCall::instruction_size);
}

// MT safe inserting of a jump over an unknown instruction sequence (used by nmethod::makeZombie)
// The problem: jmp <dest> is a 5-byte instruction. Atomical write can be only with 4 bytes.
// First patches the first word atomically to be a jump to itself.
// Then patches the last byte  and then atomically patches the first word (4-bytes),
// thus inserting the desired jump
// This code is mt-safe with the following conditions: entry point is 4 byte aligned,
// entry point is in same cache line as unverified entry point, and the instruction being 
// patched is >= 5 byte (size of patch).
// 
// In C2 the 5+ byte sized instruction is enforced by code in MachPrologNode::emit.
// In C1 the restriction is enforced by CodeEmitter::method_entry
//
void NativeJump::patch_verified_entry(address entry, address verified_entry, address dest) {
  unsigned char code_buffer[5];
  // complete jump instruction (to be inserted) is in code_buffer;
  address tmp = code_buffer;
  *tmp = 0xE9;
  tmp++;
  *((jint *)(tmp)) = (intptr_t)dest - ((intptr_t)(verified_entry + 1) + sizeof(int));

#ifndef CORE
  // These alignment restrictions are always true for c2, for c1 until it
  // deoptimizes it is only enforced if threads are still running.
#ifndef COMPILER2
  if (!SafepointSynchronize::is_at_safepoint()) {
#endif
      // Patching to not_entrant can happen while activations of the method are
      // in use. The patching in that instance must happen only when certain
      // alignment restrictions are true. These guarantees check those
      // conditions.
      // Must be 4 byte aligned
      guarantee (((uintptr_t)verified_entry & 3) == 0, "illegal address for code patching");
      // Must be in same cache line of entry
      guarantee (((uintptr_t)verified_entry & ~31) == ((uintptr_t)entry & ~31), "illegal address for code patching(2)");
#ifndef COMPILER2
  }
#endif
#endif /* CORE */

  //temporary code

  unsigned char patch[4];
  assert(sizeof(patch)==sizeof(jint), "sanity check");
  patch[0] = 0xEB;       // jmp rel8
  patch[1] = 0xFE;       // jmp to self
  patch[2] = 0xEB;
  patch[3] = 0xFE;

  // First patch dummy jmp in place
  *(jint*)verified_entry = *(jint *)patch;

  // Patch 5th byte (from jump instruction)
  address byte_5_adr = verified_entry + 4;
  *byte_5_adr = code_buffer[4];

  // Patch bytes 0-3 (from jump instruction)
  *(jint*)verified_entry = *(jint *)code_buffer;  

  ICache::invalidate_range(verified_entry, NativeJump::instruction_size);

}

void NativePopReg::insert(address code_pos, Register reg) {
  assert(NativePopReg::instruction_size == sizeof(char), "right address unit for update");
  *code_pos = (char)(instruction_code | reg.number())&0xFF;  
}


void NativeIllegalInstruction::insert(address code_pos) {
  assert(NativeIllegalInstruction::instruction_size == sizeof(short), "right address unit for update");
  *(short *)code_pos = instruction_code;
}

void NativeGeneralJump::verify() {
  assert(((NativeInstruction *)this)->is_jump() ||
	 ((NativeInstruction *)this)->is_cond_jump(), "not a general jump instruction");
}

address NativeGeneralJump::jump_destination() const {  
  int op_code = char_at(0)&0xFF;
  bool is_long = (op_code == 0xEB || op_code == 0x0F);
  int  offset  = (op_code == 0x0F)  ? 2 : 1;
  int  length  = offset + ((is_long) ? 4 : 1);
  
  if (is_long) 
    return addr_at(0) + length + long_at(offset);
  else
    return addr_at(0) + length + ((int)(char_at(offset)&0xFF));
}
