/*
 * @(#)runtime_i486.cpp	1.62 01/04/21 22:46:52
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_runtime_i486.cpp.incl"

DeoptimizationBlob *OptoRuntime::_deopt_blob;
UncommonTrapBlob   *OptoRuntime::_uncommon_trap_blob;
ExceptionBlob      *OptoRuntime::_exception_blob;
SafepointBlob      *OptoRuntime::_illegal_instruction_handler_blob;

#define __ masm->

//------------------------------generate_deopt_blob----------------------------
// Ought to generate an ideal graph & compile, but here's some Intel ASM
// instead.  Sigh, Cliff.
void OptoRuntime::generate_deopt_blob() {
  // allocate space for the code
  ResourceMark rm;
  // setup code generation tools
  CodeBuffer*   buffer = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm = new MacroAssembler(buffer);

  // Capture info about frame layout
  enum layout { arg0_off,       // used for thread
                arg1_off,       // used for sp
                arg2_off,       // used for has_exception
                eax_off,        // slot for java return value
                edx_off,        // slot for java return value
                fp0_off,        // slot for java return value
                fp0_off_2,      // slot for java return value
                ret_double_off, // slot for returns double
                exception_off,  // slot for has_exception
		ebp_off,        // slot for returned frame pointer
                return_off,     // slot for return address
	        framesize };

  OopMapSet *oop_maps = new OopMapSet();

  // -------------
  // This code enters when returning to a de-optimized frame.  The return
  // address has been popped from the stack, and return values are in
  // registers.  At this point we need to de-opt.  We save the argument return
  // registers.  We call the first C routine, fetch_unroll_info().  This
  // routine captures the return values and returns a structure which
  // describes the current frame size and the sizes of all replacement frames.
  // The current frame is compiled code and may contain many inlined
  // functions, each with their own JVM state.  We pop the current frame, then
  // push all the new frames.  Then we call the C routine unpack_frames() to
  // populate these frames.  Finally unpack_frames() returns us the new target
  // address.  Notice that callee-save registers are BLOWN here; they have
  // already been captured in the vframeArray at the time the return PC was
  // patched.
  address start = __ pc();
  Label cont;

  // Add empty oopmap at begining. This can both be at_call and not_at_call,
  // Normally it would be at call. However, it could be at not_at_call, if the
  // thread happended to be blocked at safepoint_node in the frame that got
  // deoptimized.
  OopMap* map = new OopMap( framesize, 0 );
  // First oopmap needs to record EBP so's link_offset() can find it
  map->set_callee_saved( SharedInfo::stack2reg(ebp_off), framesize,0, OptoReg::Name(EBP_num) );
  oop_maps->add_gc_map( 0, true, map );
  oop_maps->add_gc_map( 0, false, new OopMap( framesize, 0) );

  __ subl(esp,framesize*wordSize); // Prolog for non exception case!
  __ movl(Address(esp,exception_off*wordSize), Deoptimization::Unpack_deopt);
  __ jmp(cont);
  
  int exception_offset = __ pc() - start;

  __ subl(esp,framesize*wordSize);	// Prolog for exception case
  __ movl(Address(esp,exception_off*wordSize), Deoptimization::Unpack_exception); 

  __ bind(cont);

  // Save return-value registers
  __ movl(Address(esp,eax_off*wordSize),eax);
  __ movl(Address(esp,edx_off*wordSize),edx);
  __ fstp_d(Address(esp,fp0_off  *wordSize)); // Pop floating point stack and store into local
                                       // If tos is empty, NaN is saved

  // Compiled code leaves the floating point stack dirty, empty it.
  __ empty_FPU_stack();

  // Call C code.  Need thread and this frame, but NOT official VM entry
  // crud.  We cannot block on this call, no GC can happen.  
  __ get_thread(ecx);
  __ movl(Address(esp, arg0_off*wordSize),ecx);
  // fetch_unroll_info needs to call last_java_frame()
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), esp);
  __ call( CAST_FROM_FN_PTR(address, Deoptimization::fetch_unroll_info), relocInfo::runtime_call_type );
  __ get_thread(ecx);
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), NULL);

  // Load UnrollBlock into EDI
  __ movl(edi, eax);

  // Store frame locals in registers
  __ movl(eax, Address(esp,eax_off*wordSize));
  __ movl(edx, Address(esp,edx_off*wordSize));
  __ fld_d(Address(esp,fp0_off*wordSize));    // Push on float stack
  __ movl(ebp, Address(esp,exception_off*wordSize));

  // Pop all the frames we must move/replace. The final frame we pop has space for
  // the return address to c2 code and we must not pop that location (bug 4400303).
  // To solve that problem we pop one less word than expected from the initial frame
  // during the popping stage here and push one less word than expected during the
  // pushing stage.
  // 
  // Pop self-frame.  We have no frame, and must rely only on EAX and ESP.
  __ addl(esp,(framesize-1)*wordSize);     // Epilog!

  // Pop deoptimized frame
  __ addl(esp,Address(edi,Deoptimization::UnrollBlock::size_of_deoptimized_frame_offset_in_bytes()));

  // Pop I2C adapter frame, if any
  __ addl(esp,Address(edi,Deoptimization::UnrollBlock::adjust_i2c_frame_offset_in_bytes()));

  // Load array of frame sizes into ESI
  __ movl(esi,Address(edi,Deoptimization::UnrollBlock::frame_sizes_offset_in_bytes()));

  // Load count of frames into ECX
  __ movl(ecx,Address(edi,Deoptimization::UnrollBlock::number_of_frames_offset_in_bytes()));

  // Push interpreter frames in a loop
  Label loop;
  __ bind(loop);
  __ movl(ebx,Address(esi));	// Load frame size

  __ subl(esp,ebx);		// Prolog!

  __ addl(esi,4);		// Bump array pointer
  __ decl(ecx);                 // decrement counter
  __ jcc(Assembler::notZero, loop);

  // Re-push self-frame (See note above for the -1)
  __ subl(esp,(framesize-1)*wordSize);	// Prolog!

  // Restore frame locals after moving the frame
  __ movl(Address(esp,eax_off*wordSize),eax);
  __ movl(Address(esp,edx_off*wordSize),edx);
  __ fstp_d(Address(esp,fp0_off*wordSize));   // Pop float stack and store in local
  __ movl(Address(esp,exception_off*wordSize),ebp);

  // Store whether the top interpreter frame expects a float or double.
  __ movl(eax, Address(edi,Deoptimization::UnrollBlock::returns_double_offset_in_bytes()));
  __ movl(Address(esp, ret_double_off*wordSize),eax);

  // set last_Java_sp
  __ get_thread(ecx);
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), esp);

  // Call C code.  Need thread but NOT official VM entry
  // crud.  We cannot block on this call, no GC can happen.  Call should
  // restore return values to their stack-slots with the new SP.
  __ movl(Address(esp,arg0_off*wordSize),ecx);
  __ movl(eax, Address(esp,exception_off*wordSize)); 
  __ movl(Address(esp, arg1_off*wordSize),eax);
  __ call( CAST_FROM_FN_PTR(address, Deoptimization::unpack_frames), relocInfo::runtime_call_type );

  // Set an oopmap for the call site
  oop_maps->add_gc_map( __ pc()-start, true, new OopMap( framesize, 0 ));

  __ get_thread(ecx);
  // reset last Java frame
  // we must set sp to zero to clear frame
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), NULL);
  // must clear fp, so that compiled frames are not confused; it is possible
  // that we need it only for debugging
  __ movl(Address(ecx, JavaThread::last_Java_fp_offset_in_bytes()), NULL);

  // unpack_frames stuffed frame pointer of the top interpreter frame.
  __ movl(ebp,Address(esp,ebp_off*wordSize));

  // Collect return values
  __ movl(eax,Address(esp,eax_off*wordSize));
  __ movl(edx,Address(esp,edx_off*wordSize));

  // Clear floating point stack before returning to interpreter
  __ empty_FPU_stack();

  // Check if we should push the float or double return value.
  Label no_double_value;
  __ cmpl(Address(esp, ret_double_off*wordSize), 0);
  __ jcc (Assembler::zero, no_double_value);
  __ fld_d(Address(esp,fp0_off*wordSize)); // Push on float stack
  __ bind(no_double_value);

  // Pop self-frame.
  __ addl(esp,return_off*wordSize);     // Epilog!

  // Jump to interpreter
  __ ret(0);
  
  // -------------
  // make sure all code is generated
  masm->flush();

  _deopt_blob = DeoptimizationBlob::create( buffer, oop_maps, 0, exception_offset, return_off, framesize);
}


//------------------------------generate_uncommon_trap_blob--------------------
// Ought to generate an ideal graph & compile, but here's some Intel ASM
// instead.  Sigh, Cliff.
void OptoRuntime::generate_uncommon_trap_blob() {
  // allocate space for the code
  ResourceMark rm;
  // setup code generation tools
  CodeBuffer*   buffer = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm = new MacroAssembler(buffer);

  enum frame_layout {
    arg0_off,      // thread                     sp + 0 // Arg location for 
    arg1_off,      // unloaded_class_index       sp + 1 // calling C
    fpr4_off,      // callee saved register      sp + 2
    fpr4_off_2,    // callee saved register      sp + 3
    fpr5_off,      // callee saved register      sp + 4
    fpr5_off_2,    // callee saved register      sp + 5
    ebp_off,       // callee saved register      sp + 6
    edi_off,       // callee saved register      sp + 7
    esi_off,       // callee saved register      sp + 8
    return_off,    // slot for return address    sp + 9
    framesize
  };
  
  address start = __ pc();
  // Pop self-frame.
  __ subl(esp,return_off*wordSize);     // Epilog!

  // Save callee saved registers
  if( OptoRuntimeCalleeSavedFloats ) {
     __ fld_s(FPR4L_enc-1);
     __ fstp_d(Address(esp,fpr4_off  *wordSize));
     __ fld_s(FPR5L_enc-1);
     __ fstp_d(Address(esp,fpr5_off  *wordSize));
  }
  __ movl(Address(esp,ebp_off  *wordSize),ebp);
  __ movl(Address(esp,edi_off  *wordSize),edi);
  __ movl(Address(esp,esi_off  *wordSize),esi);

  // Clear the floating point exception stack
  __ empty_FPU_stack();

  // set last_Java_sp
  __ get_thread(edx);
  __ movl(Address(edx,JavaThread::last_Java_sp_offset_in_bytes()), esp);

  // Call C code.  Need thread but NOT official VM entry
  // crud.  We cannot block on this call, no GC can happen.  Call should
  // capture callee-saved registers as well as return values.
  __ movl(Address(esp, arg0_off*wordSize),edx);
  // argument already in ECX 
  __ movl(Address(esp, arg1_off*wordSize),ecx);
  __ call(CAST_FROM_FN_PTR(address, Deoptimization::uncommon_trap), relocInfo::runtime_call_type);

  // Set an oopmap for the call site
  OopMapSet *oop_maps = new OopMapSet();
  OopMap* map =  new OopMap( framesize, 0 );
  // oop_map->set_callee(
  if( OptoRuntimeCalleeSavedFloats ) {
    map->set_callee_saved( SharedInfo::stack2reg(fpr4_off  ), framesize,0, OptoReg::Name(FPR4L_num) );
    map->set_callee_saved( SharedInfo::stack2reg(fpr4_off+1), framesize,0, OptoReg::Name(FPR4H_num) );
    map->set_callee_saved( SharedInfo::stack2reg(fpr5_off  ), framesize,0, OptoReg::Name(FPR5L_num) );
    map->set_callee_saved( SharedInfo::stack2reg(fpr5_off+1), framesize,0, OptoReg::Name(FPR5H_num) );
  }
  map->set_callee_saved( SharedInfo::stack2reg(ebp_off   ), framesize,0, OptoReg::Name(EBP_num  ) );
  map->set_callee_saved( SharedInfo::stack2reg(edi_off   ), framesize,0, OptoReg::Name(EDI_num  ) );
  map->set_callee_saved( SharedInfo::stack2reg(esi_off   ), framesize,0, OptoReg::Name(ESI_num  ) );
  oop_maps->add_gc_map( __ pc()-start, true, map);

  __ get_thread(ecx);

  // reset last Java frame
  // we must set sp to zero to clear frame
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), NULL);
  // must clear fp, so that compiled frames are not confused; it is possible
  // that we need it only for debugging
  __ movl(Address(ecx, JavaThread::last_Java_fp_offset_in_bytes()), NULL);

  // Load UnrollBlock into EDI
  __ movl(edi, eax);

  // Pop all the frames we must move/replace. The final frame we pop has space for
  // the return address to c2 code and we must not pop that location (bug 4400303).
  // To solve that problem we pop one less word than expected from the initial frame
  // during the popping stage here and push one less word than expected during the
  // pushing stage.
  // 
  // Pop self-frame.  We have no frame, and must rely only on EAX and ESP.
  __ addl(esp,(framesize-1)*wordSize);     // Epilog!

  // Pop deoptimized frame
  __ addl(esp,Address(edi,Deoptimization::UnrollBlock::size_of_deoptimized_frame_offset_in_bytes()));

  // Pop I2C adapter frame, if any
  __ addl(esp,Address(edi,Deoptimization::UnrollBlock::adjust_i2c_frame_offset_in_bytes()));

  // Load array of frame sizes into ESI
  __ movl(esi,Address(edi,Deoptimization::UnrollBlock::frame_sizes_offset_in_bytes()));

  // Load count of frames into ECX
  __ movl(ecx,Address(edi,Deoptimization::UnrollBlock::number_of_frames_offset_in_bytes()));

  // Push interpreter frames in a loop
  Label loop;
  __ bind(loop);
  __ movl(ebx,Address(esi));	// Load frame size

  __ subl(esp,ebx);		// Prolog!

  __ addl(esi,4);		// Bump array pointer
  __ decl(ecx);                 // decrement counter
  __ jcc(Assembler::notZero, loop);

  // Re-push self-frame (See note above for the -1)
  __ subl(esp,(framesize-1)*wordSize);	// Prolog!

  // set last_Java_sp
  __ get_thread(edi);
  __ movl(Address(edi, JavaThread::last_Java_sp_offset_in_bytes()), esp);

  // Call C code.  Need thread but NOT official VM entry
  // crud.  We cannot block on this call, no GC can happen.  Call should
  // restore return values to their stack-slots with the new SP.
  __ movl(Address(esp,arg0_off*wordSize),edi);
  __ movl(Address(esp,arg1_off*wordSize), Deoptimization::Unpack_uncommon_trap); 
  __ call(CAST_FROM_FN_PTR(address, Deoptimization::unpack_frames), relocInfo::runtime_call_type);
  // Set an oopmap for the call site
  oop_maps->add_gc_map( __ pc()-start, true, new OopMap( framesize, 0 ) );

  __ get_thread(edi);
  // reset last Java frame
  // we must set sp to zero to clear frame
  __ movl(Address(edi, JavaThread::last_Java_sp_offset_in_bytes()), NULL);
  // must clear fp, so that compiled frames are not confused; it is possible
  // that we need it only for debugging
  __ movl(Address(edi, JavaThread::last_Java_fp_offset_in_bytes()), NULL);

  // unpack_frames stuffed frame pointer of the top interpreter frame.
  __ movl(ebp,Address(esp,ebp_off*wordSize));

  // Pop self-frame.
  __ addl(esp,return_off*wordSize);     // Epilog!

  // Jump to interpreter
  __ ret(0);

  // -------------
  // make sure all code is generated
  masm->flush();

  _uncommon_trap_blob = UncommonTrapBlob::create(buffer, oop_maps, return_off, framesize);
}


//------------------------------fill_in_exception_blob-------------------------
// Ought to generate an ideal graph & compile, but here's some Intel ASM
// instead.  Sigh, Cliff.
int handle_exception_deopt_offset = 0;
int handle_exception_deopt_exception_offset = 0;
int handle_exception_deopt_offset_2 = 0;
int handle_exception_deopt_exception_offset_2 = 0;
int handle_exception_call_pc_offset = 0;

void OptoRuntime::fill_in_exception_blob() {  
  enum layout { 
    thread_off,                 // last_java_sp                
    ebp_off,                
    edi_off,         
    esi_off,
    fpr4_off,                   // callee saved register
    fpr4_off_2,                 // callee saved register
    fpr5_off,                   // callee saved register
    fpr5_off_2,                 // callee saved register
    return_off,                 // slot for return address
    framesize 
  };

   // Patch blob
  assert(handle_exception_stub() != NULL, "exception stub must have been generated");
  assert(handle_exception_call_pc_offset != 0, "");
  assert(handle_exception_deopt_exception_offset != 0, "");
  assert(handle_exception_deopt_offset != 0, "");
  assert(handle_exception_deopt_exception_offset_2 != 0, "");
  assert(handle_exception_deopt_offset_2 != 0, "");

  {
  NativeMovConstReg* move = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_offset);
  move->set_data((int)OptoRuntime::deoptimization_blob()->unpack());

  NativeMovConstReg* move2 = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_exception_offset);
  move2->set_data((int)OptoRuntime::deoptimization_blob()->unpack_with_exception());
  }
  {
  NativeMovConstReg* move = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_offset_2);
  move->set_data((int)OptoRuntime::deoptimization_blob()->unpack());

  NativeMovConstReg* move2 = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_exception_offset_2);
  move2->set_data((int)OptoRuntime::deoptimization_blob()->unpack_with_exception());
  }
  // Set an oopmap for the call site.  This oopmap will only be used if we
  // are unwinding the stack.  Hence, all locations will be dead.
  // Callee-saved registers will be the same as the frame above (i.e.,
  // handle_exception_stub), since they were restored when we got the
  // exception.
  ResourceMark rm;
  OopMapSet *oop_maps = new OopMapSet();
  OopMap* map =  new OopMap( framesize, 0 );        
  map->set_callee_saved( SharedInfo::stack2reg(ebp_off   ), framesize, 0, OptoReg::Name(EBP_num  ) );
  map->set_callee_saved( SharedInfo::stack2reg(edi_off   ), framesize, 0, OptoReg::Name(EDI_num  ) );
  map->set_callee_saved( SharedInfo::stack2reg(esi_off   ), framesize, 0, OptoReg::Name(ESI_num  ) );
  if( OptoRuntimeCalleeSavedFloats ) {
    map->set_callee_saved( SharedInfo::stack2reg(fpr4_off  ), framesize, 0, OptoReg::Name(FPR4L_num) );
    map->set_callee_saved( SharedInfo::stack2reg(fpr4_off+1), framesize, 0, OptoReg::Name(FPR4H_num) );
    map->set_callee_saved( SharedInfo::stack2reg(fpr5_off  ), framesize, 0, OptoReg::Name(FPR5L_num) );
    map->set_callee_saved( SharedInfo::stack2reg(fpr5_off+1), framesize, 0, OptoReg::Name(FPR5H_num) );
  }
  oop_maps->add_gc_map( handle_exception_call_pc_offset, true, map);
  exception_blob()->set_oop_maps(oop_maps);
}


//------------------------------setup_exception_blob---------------------------
// creates exception blob at the end
// Using exception blob, this code is jumped from a compiled method.
// (see emit_exception_handler in i486.ad file)
// It is also the default exception destination for c2i and i2c adapters
// 
// Given an exception pc at a call, we may unwind the stack and jump to
// caller's exception handler. We may also continue into the handler
// This code is entered with a jmp.
// 
// Arguments:
//   eax: exception oop
//   edx: exception pc
//
// Results:
//   eax: exception oop
//   edx: exception pc in caller or ???
//   destination: exception handler of caller
// 
// Note: the exception pc MUST be at a call (precise debug information)
//       Only register eax, edx, ecx are not callee saved.
//

void OptoRuntime::setup_exception_blob() {
  assert(!OptoRuntime::is_callee_saved_register(EDX_num), "");
  assert(!OptoRuntime::is_callee_saved_register(EAX_num), "");
  assert(!OptoRuntime::is_callee_saved_register(ECX_num), "");

  // Capture info about frame layout  
  enum layout { 
    thread_off,                 // last_java_sp                
    ebp_off,                
    edi_off,         
    esi_off,
    fpr4_off,                   // callee saved register
    fpr4_off_2,                 // callee saved register
    fpr5_off,                   // callee saved register
    fpr5_off_2,                 // callee saved register
    return_off,                 // slot for return address
    framesize 
  };

  // allocate space for the code
  ResourceMark rm;
  // setup code generation tools  
  CodeBuffer*   buffer = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm = new MacroAssembler(buffer);


  Label exception_handler_found;

  address start = __ pc();  

  __ pushl(edx);
  __ subl(esp, return_off * wordSize);   // Prolog!

  // Save callee saved registers
  if( OptoRuntimeCalleeSavedFloats ) {
    __ fld_s(FPR4L_enc-1);
    __ fstp_d(Address(esp,fpr4_off  *wordSize));
    __ fld_s(FPR5L_enc-1);
    __ fstp_d(Address(esp,fpr5_off  *wordSize));
  }
  __ movl(Address(esp,ebp_off  *wordSize),ebp);
  __ movl(Address(esp,edi_off  *wordSize),edi);
  __ movl(Address(esp,esi_off  *wordSize),esi);
          
  // Store exception in Thread object. We cannot pass any arguments to the
  // handle_exception call, since we do not want to make any assumption
  // about the size of the frame where the exception happened in.
  __ get_thread(ecx);
  __ movl(Address(ecx, JavaThread::exception_oop_offset_in_bytes()), eax);
  __ movl(Address(ecx, JavaThread::exception_pc_offset_in_bytes()),  edx);

  // This call does all the hard work.  It checks if an exception handler
  // exists in the method.  
  // If so, it returns the handler address.
  // If not, it prepares for stack-unwinding, restoring the callee-save 
  // registers of the frame being removed.
  //  
  __ movl(Address(esp, thread_off * wordSize), ecx); // Thread is first argument
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), esp);
  __ call(CAST_FROM_FN_PTR(address, OptoRuntime::handle_exception_C), relocInfo::runtime_call_type);
  handle_exception_call_pc_offset = __ pc() - start;
  __ get_thread(ecx);
  __ movl(Address(ecx, JavaThread::last_Java_sp_offset_in_bytes()), NULL);


  // Restore callee-saved registers
  if( OptoRuntimeCalleeSavedFloats ) {
    __ fld_d(Address(esp,fpr4_off * 4));
    __ fstp_d(4);
    __ fld_d(Address(esp,fpr5_off * 4));
    __ fstp_d(5);
  }
  __ movl(ebp, Address(esp, ebp_off * wordSize));
  __ movl(edi, Address(esp, edi_off * wordSize));
  __ movl(esi, Address(esp, esi_off * wordSize));
  __ addl(esp, return_off * wordSize);   // Epilog!
  __ popl(edx); // Exception pc


  // eax: 0 if no exception handler for given <exception oop/exception pc>
  // found
  __ testl(eax, eax);
  __ jcc(Assembler::notZero, exception_handler_found);    

  // No exception handler found. We unwind stack one level.
  // All information is stored in thread.
  // thread->exception_oop
  // thread->exception_handler_pc
  // thread->exception_pc;
  // thread->exception_stack_size

  // The registers ebp, edi and esi contain the addresses of their values
  // load the values into the callee saved registers (only if register is non-zero)

  Label skipEBP, skipEDI, skipESI;
  __ testl(ebp, ebp);
  __ jcc  (Assembler::zero, skipEBP);
  __ movl (ebp, Address(ebp));
  __ bind (skipEBP);

  __ testl(edi, edi);
  __ jcc  (Assembler::zero, skipEDI);
  __ movl (edi, Address(edi));
  __ bind (skipEDI);

  __ testl(esi, esi);
  __ jcc  (Assembler::zero, skipESI);
  __ movl (esi, Address(esi));
  __ bind (skipESI);

  // We have to check if the caller frame has been deoptimized;
  // If the caller frame has been deoptimized then we must continue
  // at DeoptimizationBlob::unpack_with_exception address; 
  // If the caller frame is not deoptimized, then we continue at
  // JavaThread::exception_handler_pc_offset_in_bytes.
  // The caller frame state is defined by its return address.

  __ get_thread (ecx);    
  __ movl(edx, Address(ecx, JavaThread::exception_pc_offset_in_bytes()));
  __ movl(eax, Address(ecx, JavaThread::exception_stack_size_offset_in_bytes()));
  //   edx: exception-pc
  //   eax: stack-size
              
  Label endL, deoptL;

  handle_exception_deopt_offset = __ pc() - start;
  __ movl (ecx, (int)-1);  // will be filled in later with fill_in_exception_blob

  __ subl (eax, wordSize); // we need to check the return address, do not remove it from the stack
  __ addl (esp, eax);      // remove frame (excluding return address)

  __ popl (eax);           // get return address to check if the caller frame is deoptimized
  __ cmpl (eax, ecx);
  __ jcc  (Assembler::equal, deoptL);

  __ get_thread(ecx);
  __ movl(eax,Address(ecx,JavaThread::exception_handler_pc_offset_in_bytes()));
  __ jmp (endL);

  __ bind (deoptL);
  handle_exception_deopt_exception_offset = __ pc() - start;
  __ movl (eax, (int)-1);  // will be filled in later with fill_in_exception_blob

  __ bind (endL);
  __ pushl(eax);          // handler-pc of caller is where we continue or the deoptimization blob address
  
  __ get_thread(ecx);
  __ movl(eax, Address(ecx, JavaThread::exception_oop_offset_in_bytes()));   // exception oop 
#ifdef ASSERT
  __ movl(Address(ecx, JavaThread::exception_oop_offset_in_bytes()), NULL);
  __ movl(Address(ecx, JavaThread::exception_handler_pc_offset_in_bytes()), 0);
  __ movl(Address(ecx, JavaThread::exception_pc_offset_in_bytes()), 0);
#endif

  // eax: exception oop
  // edx: exception pc
  // return to handler
 
  __ ret(0);

  // Exception handler found.       
  __ bind (exception_handler_found);  
  
  Label deoptL_2, endL_2;

  // check if top frame has been deoptimized; if yes, exception handler must be
  // deoptimized blob
  // address to check is in edx
  handle_exception_deopt_offset_2 = __ pc() - start;
  __ movl (ecx, (int)-1);  // will be filled in later with fill_in_exception_blob
  __ cmpl (edx, ecx);
  __ jcc  (Assembler::equal, deoptL_2);
  __ movl (ecx,eax);             // Move handler address to ECX
  __ jmp  (endL_2);
  __ bind (deoptL_2);
  handle_exception_deopt_exception_offset_2 = __ pc() - start;
  __ movl (ecx, (int)-1);  // will be filled in later with fill_in_exception_blob
  __ bind (endL_2);

  // ecx contains handler address

  __ get_thread(edx);           // TLS
  __ movl(eax, Address(edx, JavaThread::exception_oop_offset_in_bytes()));   // exception oop    
#ifdef ASSERT
  __ movl(Address(edx, JavaThread::exception_handler_pc_offset_in_bytes()), 0);
  __ movl(Address(edx, JavaThread::exception_pc_offset_in_bytes()), 0); 
  __ movl(Address(edx, JavaThread::exception_oop_offset_in_bytes()), NULL);
#endif

  // eax: exception oop
  // ecx: exception handler
  // edx: not used 
  __ jmp (ecx);

  // -------------
  // make sure all code is generated
  masm->flush();  

  // Set exception blob
  OopMapSet *oop_maps = NULL; // will be set later; currently the register stuff is not yet initialized!
  _exception_blob = ExceptionBlob::create(buffer, oop_maps, return_off, framesize);  
}


//------------------------------generate_illegal_instruction_handler_blob------
//
// Generate a special Compile2Runtime blob that saves all registers, 
// and setup oopmap.
//
void OptoRuntime::generate_illegal_instruction_handler_blob() {
  assert (StubRoutines::forward_exception_entry() != NULL, "must be generated before");  

  // Capture info about frame layout
  enum layout { thread_off      = 0,      // last_java_sp
                fpu_state_off   = FPUStateSizeInWords,
                flags_off,
                edi_off,         
                esi_off,
                ebp_off,
                esp_off,                
                ebx_off,
                edx_off,
                ecx_off,
                eax_off,            
                return_off,      // slot for return address
	        framesize };

  assert(FPUStateSizeInWords == 27, "update stack layout");

  // allocate space for the code
  Label done, noException;
  ResourceMark rm;
  // setup code generation tools  
  CodeBuffer*   buffer = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm = new MacroAssembler(buffer);
  
  const Register java_thread = edi; // callee-saved for VC++
  address start   = __ pc();  
  address call_pc = NULL;  

  // save registers, fpu state, and flags  
  __ pushl(ebx);                            // Make room for return address (or push it again)
  __ pushad();        
  __ pushfd();        
  __ push_FPU_state();
  
  // The following is basically a call_VM. However, we need the precise address of the call in order to
  // generate an oopmap. Hence, we do all the work outselvs.

  // Push thread argument and setup last_Java_sp
  __ get_thread(java_thread);
  __ pushl(java_thread);
  __ movl(Address(java_thread, JavaThread::last_Java_sp_offset_in_bytes()), esp);
  
  // do the call
  __ call(CAST_FROM_FN_PTR(address, SafepointSynchronize::handle_illegal_instruction_exception), relocInfo::runtime_call_type);
  call_pc = __ pc();
  __ movl(ecx, eax); // move result into ecx, as eax may be loaded with the result of return
  __ popl(eax); // Remove thread argument

  // Clear last_Java_sp again
  __ get_thread(java_thread);
  __ movl(Address(java_thread, JavaThread::last_Java_sp_offset_in_bytes()), NULL);

  __ cmpl(Address(java_thread, Thread::pending_exception_offset_in_bytes()), NULL);
  __ jcc(Assembler::equal, noException);

  __ pop_FPU_state();
  __ popfd();
  __ popad();
  __ jmp(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);
      
  // No exception case
  __ bind(noException);
  Label negativeL;
  __ testl (ecx, ecx);
  __ jcc(Assembler::negative, negativeL);

  // Normal exit, register restoring and exit  
  __ bind(done);
  __ pop_FPU_state();
  __ popfd();
  __ popad();

  __ ret(0);
  
  // we have deoptimized at a blocked call, we may not reexecute the
  // instruction as we would skip the call in interpreter; therefore
  // execute the destination of the call; the destination is valid
  // because the receiver was already consumed
  __ bind(negativeL);
  __ pop_FPU_state();
  __ popfd();
  __ popad();
  // now jump to destination (receiver is restored with popad)
  // esi may not be used for passing arguments!!
  __ popl(ebx);
  DeoptimizationBlob* deopt_blob = OptoRuntime::deoptimization_blob();
  assert(deopt_blob != NULL, "deoptimization blob must have been created");
  __ pushl((int)deopt_blob->unpack());
  __ jmp(ebx);


  // Set an oopmap for the call site.  This oopmap will map all oop-registers
  // as callee-saved.
  OopMapSet *oop_maps = new OopMapSet();
  OopMap* map =  new OopMap( framesize, 0 );  
  map->set_callee_saved(SharedInfo::stack2reg(eax_off), framesize, 0, OptoReg::Name(EAX_num));
  map->set_callee_saved(SharedInfo::stack2reg(ecx_off), framesize, 0, OptoReg::Name(ECX_num));
  map->set_callee_saved(SharedInfo::stack2reg(edx_off), framesize, 0, OptoReg::Name(EDX_num));
  map->set_callee_saved(SharedInfo::stack2reg(ebx_off), framesize, 0, OptoReg::Name(EBX_num));  
  map->set_callee_saved(SharedInfo::stack2reg(ebp_off), framesize, 0, OptoReg::Name(EBP_num));
  map->set_callee_saved(SharedInfo::stack2reg(esi_off), framesize, 0, OptoReg::Name(ESI_num));
  map->set_callee_saved(SharedInfo::stack2reg(edi_off), framesize, 0, OptoReg::Name(EDI_num));    
  oop_maps->add_gc_map( call_pc - start, true, map);

  // -------------
  // make sure all code is generated
  masm->flush();  

  // Fill-out other meta info
  _illegal_instruction_handler_blob = SafepointBlob::create(buffer, oop_maps, return_off, framesize);    
}


//------------------------------exception_address_is_unpack_entry--------------
//%note runtime_26
bool DeoptimizationBlob::exception_address_is_unpack_entry(address pc) const {
  return (pc == OptoRuntime::deoptimization_blob()->unpack());
}


//------------------------------pd_unwind_stack--------------------------------
void OptoRuntime::pd_unwind_stack(JavaThread *thread, frame fr, RegisterMap* reg_map) {
  // Update callee-saved registers across frame
  jint* reg_locs[REG_COUNT];
  // Copy values of all callee-saved registers out of frame
  for(int i = 0; i < REG_COUNT; i++) {
    jint* loc = (jint*)reg_map->location(VMReg::Name(i));    
    reg_locs[i] = loc;
  }
  
  // Find oopmap for stub, and update it stack with callee-saved
  // reg. info for next frame.
  frame stub_frame = thread->last_frame();
  CodeBlob* blob = CodeCache::find_blob(stub_frame.pc());
  assert(blob->is_exception_stub(), "sanity check");
  OopMap* map = blob->oop_maps()->singular_oop_map();
  // Update callee-saved registers in stub
  OopMapValue omv;
  for(OopMapStream oms(map,OopMapValue::callee_saved_value); !oms.is_done(); oms.next()) {
    omv = oms.current();
    assert(omv.is_stack_loc(), "sanity check");
    jint* location = (jint*)stub_frame.oopmapreg_to_location(omv.reg(),NULL);
    VMReg::Name reg = omv.content_reg();
    assert(reg >= 0 && reg < REG_COUNT, "reg out of range");
    jint** location2 = (jint**)location;
    *location2 = reg_locs[reg];
  }
}


//
// Generate the on-stack replacement stub, that is used to replace the
// interpreter frame
//

OSRAdapter* OptoRuntime::generate_osr_blob(int frame_size) {
  ResourceMark rm;

  // setup code generation tools
  CodeBuffer*       cb = new CodeBuffer(128, 128, 0, 0, false);
  MacroAssembler* masm = new MacroAssembler(cb);    
  
  // Create oopmap for osr adapter. All it contains is where to find the
  // link offset (ebp) on windows.
  int link_offset = ((frame_size - 2) + frame::link_offset);
  int sp_offset   = ((frame_size - 2) + frame::interpreter_frame_sender_sp_offset);    

  OopMapSet *oop_maps = new OopMapSet();
  OopMap* map =  new OopMap(frame_size, 0 );
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + link_offset), frame_size, 0, OptoReg::Name(EBP_num));
  map->set_callee_saved(OptoReg::Name(SharedInfo::stack0 + sp_offset  ), frame_size, 0, OptoReg::Name(ESP_num));
  oop_maps->add_gc_map(0, true, map);

  if (UseC2CallingConventions) {
    masm->movl(ecx, Address(ebp, frame::interpreter_frame_sender_sp_offset * wordSize)); // get sender sp
    masm->leave();                                // remove frame anchor
    masm->popl(esi);                              // get return address
    masm->movl(esp, ecx);                         // set sp to sender sp
  } else {
    masm->movl(ecx, Address(ebp, frame::interpreter_frame_locals_offset * wordSize)); // get sender sp
    masm->leave();                                // remove frame anchor
    masm->popl(esi);                              // get return address
    masm->leal(esp, Address(ecx, wordSize));      // set sp to sender sp
  }
  masm->jmp(esi);  
  masm->flush();

  return OSRAdapter::new_osr_adapter(cb, oop_maps, frame_size, sp_offset);
}

