/*
 * @(#)templateTable_i486.cpp	1.219 01/02/09 14:43:07
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_templateTable_i486.cpp.incl"

#define __ _masm->

//----------------------------------------------------------------------------------------------------
// Platform-dependent initialization

void TemplateTable::pd_initialize() {
  // No i486 specific initialization
}

//----------------------------------------------------------------------------------------------------
// Address computation

// local variables
static inline Address iaddress(int n)            { return Address(edi, -n*wordSize); }
static inline Address laddress(int n)            { return iaddress(n + 1); }
static inline Address haddress(int n)            { return iaddress(n + 0); }
static inline Address faddress(int n)            { return iaddress(n); }
static inline Address daddress(int n)            { return laddress(n); }
static inline Address aaddress(int n)            { return iaddress(n); }

static inline Address iaddress(Register r)       { return Address(edi, r, Address::times_4); }
static inline Address laddress(Register r)       { return Address(edi, r, Address::times_4, -1 * wordSize); }
static inline Address haddress(Register r)       { return Address(edi, r, Address::times_4,  0 * wordSize); }
static inline Address faddress(Register r)       { return iaddress(r); };
static inline Address daddress(Register r)       { return laddress(r); };
static inline Address aaddress(Register r)       { return iaddress(r); };

// expression stack
// (Note: Must not use symmetric equivalents at_esp_m1/2 since they store
// data beyond the esp which is potentially unsafe in an MT environment;
// an interrupt may overwrite that data.)
static inline Address at_esp   ()                { return Address(esp,  0 * wordSize); }
static inline Address at_esp_p1()                { return Address(esp,  1 * wordSize); }
static inline Address at_esp_p2()                { return Address(esp,  2 * wordSize); }

// Condition conversion
static Assembler::Condition j_not(TemplateTable::Condition cc) {
  switch (cc) {
    case TemplateTable::equal        : return Assembler::notEqual;
    case TemplateTable::not_equal    : return Assembler::equal;
    case TemplateTable::less         : return Assembler::greaterEqual;
    case TemplateTable::less_equal   : return Assembler::greater;
    case TemplateTable::greater      : return Assembler::lessEqual;
    case TemplateTable::greater_equal: return Assembler::less;
  }
  ShouldNotReachHere();
  return Assembler::zero;
}


//----------------------------------------------------------------------------------------------------
// Miscelaneous helper routines

void TemplateTable::div0_check(Register reg) {
  if (GenerateInterpreterDiv0Checks) {
    // generate explicit div0 check
    __ testl(reg, reg);
    __ jcc(Assembler::zero, Interpreter::_throw_ArithmeticException_entry, relocInfo::none);
  }
}


void TemplateTable::null_check(Register obj, int offset) {
  __ null_check(obj, offset);
}


Address TemplateTable::at_bcp(int offset) {
  assert(_desc->uses_bcp(), "inconsistent uses_bcp information");
  return Address(esi, offset);
}


//----------------------------------------------------------------------------------------------------
// Individual instructions

void TemplateTable::nop() {
  transition(vtos, vtos);
  // nothing to do
}

void TemplateTable::shouldnotreachhere() {
  transition(vtos, vtos);
  __ stop("shouldnotreachhere bytecode");
}



void TemplateTable::aconst_null() {
  transition(vtos, atos);
  __ xorl(eax, eax);
}


void TemplateTable::iconst(int value) {
  transition(vtos, itos);
  if (value == 0) {
    __ xorl(eax, eax);
  } else {
    __ movl(eax, value);
  }
}


void TemplateTable::lconst(int value) {
  transition(vtos, ltos);
  if (value == 0) {
    __ xorl(eax, eax);
  } else {
    __ movl(eax, value);
  }
  assert(value >= 0, "check this code");
  __ xorl(edx, edx);
}


void TemplateTable::fconst(int value) {
  transition(vtos, ftos);
         if (value == 0) { __ fldz();
  } else if (value == 1) { __ fld1();
  } else if (value == 2) { __ fld1(); __ fld1(); __ faddp(); // should do a better solution here
  } else                 { ShouldNotReachHere();
  }
}


void TemplateTable::dconst(int value) {
  transition(vtos, dtos);
         if (value == 0) { __ fldz();
  } else if (value == 1) { __ fld1();
  } else                 { ShouldNotReachHere();
  }
}


void TemplateTable::bipush() {
  transition(vtos, itos);
  __ load_signed_byte(eax, at_bcp(1));
}


void TemplateTable::sipush() {
  transition(vtos, itos);
  __ load_unsigned_word(eax, at_bcp(1));
  __ bswap(eax);
  __ sarl(eax, 16);
}


void TemplateTable::ldc() {
  transition(vtos, vtos);
  call_VM(noreg, CAST_FROM_FN_PTR(address,InterpreterRuntime::ldc));
  __ dispatch_next(vtos);
}


void TemplateTable::ldc_w() {
  transition(vtos, vtos);
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::ldc_w));
  __ dispatch_next(vtos);
}


void TemplateTable::ldc2_w() {
  transition(vtos, vtos);
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::ldc2_w));
  __ dispatch_next(vtos);
}


void TemplateTable::fast_ldc(TosState state) {
  transition(vtos, state);
  __ load_unsigned_byte(ebx, at_bcp(1));
  __ get_constant_pool(ecx);
  const int base_offset = constantPoolOopDesc::header_size() * wordSize;
  switch (state) {
    case atos:                                                                 // fall through
    case itos: __ movl(eax, Address(ecx, ebx, Address::times_4, base_offset)); break;
    case ftos: __ fld_s(    Address(ecx, ebx, Address::times_4, base_offset)); break;
    default  : ShouldNotReachHere();
  }
}


void TemplateTable::fast_ldc_w(TosState state) {
  transition(vtos, state);
  __ get_unsigned_2_byte_index_at_bcp(ebx, 1);
  __ get_constant_pool(ecx);
  const int base_offset = constantPoolOopDesc::header_size() * wordSize;
  switch (state) {
    case atos:                                                                 // fall through
    case itos: __ movl(eax, Address(ecx, ebx, Address::times_4, base_offset)); break;
    case ftos: __ fld_s(    Address(ecx, ebx, Address::times_4, base_offset)); break;
    default  : ShouldNotReachHere();
  }
}


void TemplateTable::fast_ldc2_w(TosState state) {
  transition(vtos, state);
  __ get_unsigned_2_byte_index_at_bcp(ebx, 1);
  __ get_constant_pool(ecx);
  const int base_offset = constantPoolOopDesc::header_size() * wordSize;
  switch (state) {
    case ltos: __ movl(eax, Address(ecx, ebx, Address::times_4, base_offset + 0 * wordSize));
               __ movl(edx, Address(ecx, ebx, Address::times_4, base_offset + 1 * wordSize)); break;
    case dtos: __ fld_d(    Address(ecx, ebx, Address::times_4, base_offset               )); break;
    default  : ShouldNotReachHere();
  }
}


void TemplateTable::locals_index(Register reg) {
  __ load_unsigned_byte(reg, at_bcp(1));
  __ negl(reg);	
}


void TemplateTable::iload() {
  transition(vtos, itos);
  locals_index(ebx);
  __ movl(eax, iaddress(ebx));
}


void TemplateTable::lload() {
  transition(vtos, ltos);
  locals_index(ebx);
  __ movl(eax, laddress(ebx));
  __ movl(edx, haddress(ebx));
}


void TemplateTable::fload() {
  transition(vtos, ftos);
  locals_index(ebx);
  __ fld_s(faddress(ebx));
}


void TemplateTable::dload() {
  transition(vtos, dtos);
  locals_index(ebx);
  __ fld_d(daddress(ebx));
}


void TemplateTable::aload() {
  transition(vtos, atos);
  locals_index(ebx);
  __ movl(eax, iaddress(ebx));
}


void TemplateTable::locals_index_wide(Register reg) {
  __ movl(reg, at_bcp(2));
  __ bswap(reg);
  __ shrl(reg, 16);
  __ negl(reg);	
}


void TemplateTable::wide_iload() {
  transition(vtos, itos);
  locals_index_wide(ebx);
  __ movl(eax, iaddress(ebx));
}


void TemplateTable::wide_lload() {
  transition(vtos, ltos);
  locals_index_wide(ebx);
  __ movl(eax, laddress(ebx));
  __ movl(edx, haddress(ebx));
}


void TemplateTable::wide_fload() {
  transition(vtos, ftos);
  locals_index_wide(ebx);
  __ fld_s(faddress(ebx));
}


void TemplateTable::wide_dload() {
  transition(vtos, dtos);
  locals_index_wide(ebx);
  __ fld_d(daddress(ebx));
}


void TemplateTable::wide_aload() {
  transition(vtos, atos);
  locals_index_wide(ebx);
  __ movl(eax, iaddress(ebx));
}


void TemplateTable::index_check(Register array, Register index) {
  // check array
  null_check(array, arrayOopDesc::length_offset_in_bytes());
  // check index
  __ cmpl(index, Address(array, arrayOopDesc::length_offset_in_bytes()));
  __ jcc(Assembler::aboveEqual, Interpreter::_throw_ArrayIndexOutOfBoundsException_entry, relocInfo::none);
}


void TemplateTable::iaload() {
  transition(itos, itos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  __ movl(eax, Address(edx, eax, Address::times_4, arrayOopDesc::base_offset_in_bytes(T_INT)));
}


void TemplateTable::laload() {
  transition(itos, ltos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  __ movl(ebx, eax);
  __ movl(eax, Address(edx, ebx, Address::times_8, arrayOopDesc::base_offset_in_bytes(T_LONG) + 0 * wordSize));
  __ movl(edx, Address(edx, ebx, Address::times_8, arrayOopDesc::base_offset_in_bytes(T_LONG) + 1 * wordSize));
}


void TemplateTable::faload() {
  transition(itos, ftos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  __ fld_s(Address(edx, eax, Address::times_4, arrayOopDesc::base_offset_in_bytes(T_FLOAT)));
}


void TemplateTable::daload() {
  transition(itos, dtos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  __ fld_d(Address(edx, eax, Address::times_8, arrayOopDesc::base_offset_in_bytes(T_DOUBLE)));
}


void TemplateTable::aaload() {
  transition(itos, atos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  __ movl(eax, Address(edx, eax, Address::times_4, arrayOopDesc::base_offset_in_bytes(T_OBJECT)));
}


void TemplateTable::baload() {
  transition(itos, itos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  // can do better code for P5 - fix this at some point
  __ load_signed_byte(ebx, Address(edx, eax, Address::times_1, arrayOopDesc::base_offset_in_bytes(T_BYTE)));
  __ movl(eax, ebx);
}


void TemplateTable::caload() {
  transition(itos, itos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  // can do better code for P5 - may want to improve this at some point
  __ load_unsigned_word(ebx, Address(edx, eax, Address::times_2, arrayOopDesc::base_offset_in_bytes(T_CHAR)));
  __ movl(eax, ebx);
}


void TemplateTable::saload() {
  transition(itos, itos);
  __ popl(edx);
  // eax: index
  // edx: array
  index_check(edx, eax);
  // can do better code for P5 - may want to improve this at some point
  __ load_signed_word(ebx, Address(edx, eax, Address::times_2, arrayOopDesc::base_offset_in_bytes(T_SHORT)));
  __ movl(eax, ebx);
}


void TemplateTable::iload(int n) {
  transition(vtos, itos);
  __ movl(eax, iaddress(n));
}


void TemplateTable::lload(int n) {
  transition(vtos, ltos);
  __ movl(eax, laddress(n));
  __ movl(edx, haddress(n));
}


void TemplateTable::fload(int n) {
  transition(vtos, ftos);
  __ fld_s(faddress(n));
}


void TemplateTable::dload(int n) {
  transition(vtos, dtos);
  __ fld_d(daddress(n));
}


void TemplateTable::aload(int n) {
  transition(vtos, atos);
  __ movl(eax, aaddress(n));
}


void TemplateTable::aload_0() {
  transition(vtos, atos);
  // According to bytecode histograms, the pairs:
  //
  // _aload_0, _fast_igetfield
  // _aload_0, _fast_agetfield
  //
  // occur frequently. If RewriteFrequentPairs is set, the (slow) _aload_0 bytecode checks
  // if the next bytecode is either _fast_igetfield or _fast_agetfield and then rewrites the
  // current bytecode into a pair bytecode; otherwise it rewrites the current bytecode into
  // _fast_aload_0 that doesn't do the pair check anymore.
  //
  // Note: If the next bytecode is _getfield, the rewrite must be delayed, otherwise we
  //       may miss an opportunity for a pair.
  if (RewriteFrequentPairs) {
    Label rewrite, done;
    // get next byte
    __ load_unsigned_byte(ebx, at_bcp(Bytecodes::length_for(Bytecodes::_aload_0)));

    // do actual aload_0
    aload(0);

    // if _getfield then wait with rewrite
    __ cmpl(ebx, Bytecodes::_getfield);
    __ jcc(Assembler::equal, done);

    // if _igetfield then reqrite to _fast_iaccess_0
    assert(Bytecodes::java_code(Bytecodes::_fast_iaccess_0) == Bytecodes::_aload_0, "adjust code below");
    __ cmpl(ebx, Bytecodes::_fast_igetfield);
    __ movl(ecx, Bytecodes::_fast_iaccess_0);
    __ jcc(Assembler::equal, rewrite);

    // if _agetfield then reqrite to _fast_aaccess_0
    assert(Bytecodes::java_code(Bytecodes::_fast_aaccess_0) == Bytecodes::_aload_0, "adjust code below");
    __ cmpl(ebx, Bytecodes::_fast_agetfield);
    __ movl(ecx, Bytecodes::_fast_aaccess_0);
    __ jcc(Assembler::equal, rewrite);

    // else rewrite to _fast_aload0
    assert(Bytecodes::java_code(Bytecodes::_fast_aload_0) == Bytecodes::_aload_0, "adjust code below");
    __ movl(ecx, Bytecodes::_fast_aload_0);

    // rewrite
    // ecx: fast bytecode
    __ bind(rewrite);
    __ movb(at_bcp(0), ecx);

    __ bind(done);
  } else {
    aload(0);
  }
}


void TemplateTable::istore() {
  transition(itos, vtos);
  locals_index(ebx);
  __ movl(iaddress(ebx), eax);
}


void TemplateTable::lstore() {
  transition(ltos, vtos);
  locals_index(ebx);
  __ movl(laddress(ebx), eax);
  __ movl(haddress(ebx), edx);
}


void TemplateTable::fstore() {
  transition(ftos, vtos);
  locals_index(ebx);
  __ fstp_s(faddress(ebx));
}


void TemplateTable::dstore() {
  transition(dtos, vtos);
  locals_index(ebx);
  __ fstp_d(daddress(ebx));
}


void TemplateTable::astore() {
  transition(atos, vtos);
  locals_index(ebx);
  __ movl(aaddress(ebx), eax);
}


void TemplateTable::wide_istore() {
  transition(vtos, vtos);
  __ popl(eax);
  locals_index_wide(ebx);
  __ movl(iaddress(ebx), eax);
}


void TemplateTable::wide_lstore() {
  transition(vtos, vtos);
  __ popl(eax);
  __ popl(edx);
  locals_index_wide(ebx);
  __ movl(laddress(ebx), eax);
  __ movl(haddress(ebx), edx);
}


void TemplateTable::wide_fstore() {
  wide_istore();
}


void TemplateTable::wide_dstore() {
  wide_lstore();
}


void TemplateTable::wide_astore() {
  wide_istore();
}


void TemplateTable::iastore() {
  transition(itos, vtos);
  __ popl(edx);
  __ popl(ebx);
  // eax: value
  // ebx: array
  // edx: index
  index_check(ebx, edx);
  __ movl(Address(ebx, edx, Address::times_4, arrayOopDesc::base_offset_in_bytes(T_INT)), eax);
}


void TemplateTable::lastore() {
  transition(ltos, vtos);
  __ popl(ebx);
  __ popl(ecx);
  // eax: low(value)
  // ebx: index
  // ecx: array
  // edx: high(value)
  index_check(ecx, ebx);
  __ movl(Address(ecx, ebx, Address::times_8, arrayOopDesc::base_offset_in_bytes(T_LONG) + 0 * wordSize), eax);
  __ movl(Address(ecx, ebx, Address::times_8, arrayOopDesc::base_offset_in_bytes(T_LONG) + 1 * wordSize), edx);
}


void TemplateTable::fastore() {
  transition(ftos, vtos);
  __ popl(edx);
  __ popl(eax);
  // eax: array
  // edx: index
  // st0: value
  index_check(eax, edx);
  __ fstp_s(Address(eax, edx, Address::times_4, arrayOopDesc::base_offset_in_bytes(T_FLOAT)));
}


void TemplateTable::dastore() {
  transition(dtos, vtos);
  __ popl(edx);
  __ popl(eax);
  // eax: array
  // edx: index
  // st0: value
  index_check(eax, edx);
  __ fstp_d(Address(eax, edx, Address::times_8, arrayOopDesc::base_offset_in_bytes(T_DOUBLE)));
}


void TemplateTable::aastore() {
  transition(vtos, vtos);
  // stack: ..., array, index, value
  __ movl(edx, at_esp());
  __ movl(ecx, at_esp_p1());
  __ movl(eax, at_esp_p2());
  // eax: array
  // ecx: index
  // edx: value
  index_check(eax, ecx);
  // do array store check - check for NULL value first
  Label store_ok_1, store_ok_2;
  __ testl(edx, edx);
  __ jcc(Assembler::zero, store_ok_1);
  // do fast instanceof cache test
  __ movl(ebx, Address(eax, oopDesc::klass_offset_in_bytes()));
  __ movl(ecx, Address(edx, oopDesc::klass_offset_in_bytes()));
  __ movl(ebx, Address(ebx, sizeof(oopDesc) + objArrayKlass::element_klass_offset_in_bytes()));
  // eax: array
  // ebx: array element klass
  // ecx: value klass
  // edx: value
  __ cmpl(ebx, ecx);    // check if classes are identical
  __ jcc(Assembler::equal, store_ok_2);
  __ cmpl(ebx, Address(ecx, sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes()));
  __ jcc(Assembler::equal, store_ok_2);
  __ cmpl(ebx, Address(ecx, sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes()));
  __ jcc(Assembler::equal, store_ok_2);
  // cache miss, need to restore index
  __ movl(ecx, at_esp_p1());
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::array_store_check), edx, ecx, eax);
  // restore registers
  __ movl(edx, at_esp());
  __ movl(eax, at_esp_p2());
  // store ok, but need to restore index
  __ bind(store_ok_2);
  __ movl(ecx, at_esp_p1());
  // pop values from stack
  __ bind(store_ok_1);
  __ addl(esp, 3 * wordSize);
  // eax: array
  // ecx: index
  // edx: value
  __ leal(ebx, Address(eax, ecx, Address::times_4, arrayOopDesc::base_offset_in_bytes(T_OBJECT)));
  __ movl(Address(ebx), edx);
  __ store_check(ebx);
}


void TemplateTable::bastore() {
  transition(itos, vtos);
  __ popl(edx);
  __ popl(ebx);
  // eax: value
  // ebx: array
  // edx: index
  index_check(ebx, edx);
  __ movb(Address(ebx, edx, Address::times_1, arrayOopDesc::base_offset_in_bytes(T_BYTE)), eax);
}


void TemplateTable::castore() {
  transition(itos, vtos);
  __ popl(edx);
  __ popl(ebx);
  // eax: value
  // ebx: array
  // edx: index
  index_check(ebx, edx);
  __ movw(Address(ebx, edx, Address::times_2, arrayOopDesc::base_offset_in_bytes(T_CHAR)), eax);
}


void TemplateTable::sastore() {
  castore();
}


void TemplateTable::istore(int n) {
  transition(itos, vtos);
  __ movl(iaddress(n), eax);
}


void TemplateTable::lstore(int n) {
  transition(ltos, vtos);
  __ movl(laddress(n), eax);
  __ movl(haddress(n), edx);
}


void TemplateTable::fstore(int n) {
  transition(ftos, vtos);
  __ fstp_s(faddress(n));
}


void TemplateTable::dstore(int n) {
  transition(dtos, vtos);
  __ fstp_d(daddress(n));
}


void TemplateTable::astore(int n) {
  transition(atos, vtos);
  __ movl(aaddress(n), eax);
}


void TemplateTable::pop() {
  transition(vtos, vtos);
  __ popl(eax);
}


void TemplateTable::pop2() {
  transition(vtos, vtos);
  __ popl(eax);
  __ popl(ebx);
}


void TemplateTable::dup() {
  transition(vtos, vtos);
  // stack: ..., a
  __ pushl(Address(esp));
  // stack: ..., a, a
}


void TemplateTable::dup_x1() {
  transition(vtos, vtos);
  // stack: ..., a, b
  __ popl(ebx);
  __ popl(eax);
  __ pushl(ebx);
  __ pushl(eax);
  __ pushl(ebx);
  // stack: ..., b, a, b
}


void TemplateTable::dup_x2() {
  transition(vtos, vtos);
  // stack: ..., a, b, c
  __ popl(ecx);
  __ popl(ebx);
  __ popl(eax);
  __ pushl(ecx);
  __ pushl(eax);
  __ pushl(ebx);
  __ pushl(ecx);
  // stack: ..., c, a, b, c
}


void TemplateTable::dup2() {
  transition(vtos, vtos);
  // stack: ..., a, b
  __ pushl(Address(esp, 1 * wordSize));
  __ pushl(Address(esp, 1 * wordSize));
  // stack: ..., a, b, a, b
}


void TemplateTable::dup2_x1() {
  transition(vtos, vtos);
  // stack: ..., a, b, c
  __ popl(ecx);
  __ popl(ebx);
  __ popl(eax);
  __ pushl(ebx);
  __ pushl(ecx);
  __ pushl(eax);
  __ pushl(ebx);
  __ pushl(ecx);
  // stack: ..., b, c, a, b, c
}


void TemplateTable::dup2_x2() {
  transition(vtos, vtos);
  // stack: ..., a, b, c, d
  __ popl(edx);
  __ popl(ecx);
  __ popl(ebx);
  __ popl(eax);
  __ pushl(ecx);
  __ pushl(edx);
  __ pushl(eax);
  __ pushl(ebx);
  __ pushl(ecx);
  __ pushl(edx);
  // stack: ..., c, d, a, b, c, d
}


void TemplateTable::swap() {
  transition(vtos, vtos);
  // stack: ..., a, b
  __ popl(ebx);
  __ popl(eax);
  __ pushl(ebx);
  __ pushl(eax);
  // stack: ..., b, a
}


void TemplateTable::iop2(Operation op) {
  transition(itos, itos);
  switch (op) {
    case add  :                    __ popl(edx); __ addl (eax, edx); break;
    case sub  : __ movl(edx, eax); __ popl(eax); __ subl (eax, edx); break;
    case mul  :                    __ popl(edx); __ imull(eax, edx); break;
    case _and :                    __ popl(edx); __ andl (eax, edx); break;
    case _or  :                    __ popl(edx); __ orl  (eax, edx); break;
    case _xor :                    __ popl(edx); __ xorl (eax, edx); break;
    case shl  : __ movl(ecx, eax); __ popl(eax); __ shll (eax);      break; // implicit masking of lower 5 bits by Intel shift instr.
    case shr  : __ movl(ecx, eax); __ popl(eax); __ sarl (eax);      break; // implicit masking of lower 5 bits by Intel shift instr.
    case ushr : __ movl(ecx, eax); __ popl(eax); __ shrl (eax);      break; // implicit masking of lower 5 bits by Intel shift instr.
    default   : ShouldNotReachHere();
  }
}


void TemplateTable::lop2(Operation op) {
  transition(ltos, ltos);
  __ popl(ebx);
  __ popl(ecx);
  switch (op) {
    case add : __ addl(eax, ebx); __ adcl(edx, ecx); break;
    case sub : __ subl(ebx, eax); __ sbbl(ecx, edx);
               __ movl(eax, ebx); __ movl(edx, ecx); break;
    case _and: __ andl(eax, ebx); __ andl(edx, ecx); break;
    case _or : __ orl (eax, ebx); __ orl (edx, ecx); break;
    case _xor: __ xorl(eax, ebx); __ xorl(edx, ecx); break;
    default : ShouldNotReachHere();
  }
}


void TemplateTable::idiv() {
  transition(itos, itos);
  __ movl(ecx, eax);
  __ popl(eax);
  div0_check(ecx);
  // Note: could xor eax and ecx and compare with (-1 ^ min_int). If
  //       they are not equal, one could do a normal division (no correction
  //       needed), which may speed up this implementation for the common case.
  //       (see also JVM spec., p.243 & p.271)
  __ corrected_idivl(ecx);
}


void TemplateTable::irem() {
  transition(itos, itos);
  __ movl(ecx, eax);
  __ popl(eax);
  div0_check(ecx);
  // Note: could xor eax and ecx and compare with (-1 ^ min_int). If
  //       they are not equal, one could do a normal division (no correction
  //       needed), which may speed up this implementation for the common case.
  //       (see also JVM spec., p.243 & p.271)
  __ corrected_idivl(ecx);
  __ movl(eax, edx);
}


void TemplateTable::lmul() {
  transition(ltos, ltos);
  __ pushl(edx);
  __ pushl(eax);
  __ lmul(2 * wordSize, 0);
  __ addl(esp, 4 * wordSize);
}


void TemplateTable::ldiv() {
  transition(ltos, ltos);
  __ pushl(edx);
  __ pushl(eax);
  // check if y = 0
  __ orl(eax, edx);
  __ jcc(Assembler::zero, Interpreter::_throw_ArithmeticException_entry, relocInfo::none);
  __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::ldiv));
  __ addl(esp, 4 * wordSize);
}


void TemplateTable::lrem() {
  transition(ltos, ltos);
  __ pushl(edx);
  __ pushl(eax);
  // check if y = 0
  __ orl(eax, edx);
  __ jcc(Assembler::zero, Interpreter::_throw_ArithmeticException_entry, relocInfo::none);
  __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::lrem));
  __ addl(esp, 4 * wordSize);
}


void TemplateTable::lshl() {
  transition(itos, ltos);
  __ movl(ecx, eax);                             // get shift count
  __ popl(eax);                                  // get shift value
  __ popl(edx);
  __ lshl(edx, eax);
}


void TemplateTable::lshr() {
  transition(itos, ltos);
  __ movl(ecx, eax);                             // get shift count
  __ popl(eax);                                  // get shift value
  __ popl(edx);
  __ lshr(edx, eax, true);
}


void TemplateTable::lushr() {
  transition(itos, ltos);
  __ movl(ecx, eax);                             // get shift count
  __ popl(eax);                                  // get shift value
  __ popl(edx);
  __ lshr(edx, eax);
}


void TemplateTable::fop2(Operation op) {
  transition(ftos, ftos);
  switch (op) {
    case add: __ fadd_s (at_esp());                break;
    case sub: __ fsubr_s(at_esp());                break;
    case mul: __ fmul_s (at_esp());                break;
    case div: __ fdivr_s(at_esp());                break;
    case rem: __ fld_s  (at_esp()); __ fremr(eax); break;
    default : ShouldNotReachHere();
  }
  __ f2ieee();
  __ popl(eax);
}


void TemplateTable::dop2(Operation op) {
  transition(dtos, dtos);
  switch (op) {
    case add: __ fadd_d (at_esp());                break;
    case sub: __ fsubr_d(at_esp());                break;
    case mul: __ fmul_d (at_esp());                break;
    case div: __ fdivr_d(at_esp());                break;
    case rem: __ fld_d  (at_esp()); __ fremr(eax); break;
    default : ShouldNotReachHere();
  }
  __ d2ieee();
  __ popl(eax);
  __ popl(edx);
}


void TemplateTable::ineg() {
  transition(itos, itos);
  __ negl(eax);
}


void TemplateTable::lneg() {
  transition(ltos, ltos);
  __ lneg(edx, eax);
}


void TemplateTable::fneg() {
  transition(ftos, ftos);
  __ fchs();
}


void TemplateTable::dneg() {
  transition(dtos, dtos);
  __ fchs();
}


void TemplateTable::iinc() {
  transition(vtos, vtos);
  __ load_signed_byte(edx, at_bcp(2));           // get constant
  locals_index(ebx);
  __ addl(iaddress(ebx), edx);
}


void TemplateTable::wide_iinc() {
  transition(vtos, vtos);
  __ movl(edx, at_bcp(4));                       // get constant
  locals_index_wide(ebx);
  __ bswap(edx);                                 // swap bytes & sign-extend constant
  __ sarl(edx, 16);
  __ addl(iaddress(ebx), edx);
  // Note: should probably use only one movl to get both
  //       the index and the constant -> fix this
}


void TemplateTable::convert() {
  // Checking
#ifdef ASSERT
  { TosState tos_in  = ilgl;
    TosState tos_out = ilgl;
    switch (bytecode()) {
      case Bytecodes::_i2l: // fall through
      case Bytecodes::_i2f: // fall through
      case Bytecodes::_i2d: // fall through
      case Bytecodes::_i2b: // fall through
      case Bytecodes::_i2c: // fall through
      case Bytecodes::_i2s: tos_in = itos; break;
      case Bytecodes::_l2i: // fall through
      case Bytecodes::_l2f: // fall through
      case Bytecodes::_l2d: tos_in = ltos; break;
      case Bytecodes::_f2i: // fall through
      case Bytecodes::_f2l: // fall through
      case Bytecodes::_f2d: tos_in = ftos; break;
      case Bytecodes::_d2i: // fall through
      case Bytecodes::_d2l: // fall through
      case Bytecodes::_d2f: tos_in = dtos; break;
      default             : ShouldNotReachHere();
    }
    switch (bytecode()) {
      case Bytecodes::_l2i: // fall through
      case Bytecodes::_f2i: // fall through
      case Bytecodes::_d2i: // fall through
      case Bytecodes::_i2b: // fall through
      case Bytecodes::_i2c: // fall through
      case Bytecodes::_i2s: tos_out = itos; break;
      case Bytecodes::_i2l: // fall through
      case Bytecodes::_f2l: // fall through
      case Bytecodes::_d2l: tos_out = ltos; break;
      case Bytecodes::_i2f: // fall through
      case Bytecodes::_l2f: // fall through
      case Bytecodes::_d2f: tos_out = ftos; break;
      case Bytecodes::_i2d: // fall through
      case Bytecodes::_l2d: // fall through
      case Bytecodes::_f2d: tos_out = dtos; break;
      default             : ShouldNotReachHere();
    }
    transition(tos_in, tos_out);
  }
#endif

  // Conversion
  // (Note: use pushl(ecx)/popl(ecx) for 1/2-word stack-ptr manipulation)
  switch (bytecode()) {
    case Bytecodes::_i2l:
      __ extend_sign(edx, eax);
      break;
    case Bytecodes::_i2f:
      __ pushl(eax);         // store int on tos
      __ fild_s(at_esp());   // load int to ST0
      __ f2ieee();           // truncate to float size
      __ popl(ecx);          // adjust esp
      break;
    case Bytecodes::_i2d:
      __ pushl(eax);         // add one slot for d2ieee()
      __ pushl(eax);         // store int on tos
      __ fild_s(at_esp());   // load int to ST0
      __ d2ieee();           // truncate to double size
      __ popl(ecx);          // adjust esp
      __ popl(ecx);
      break;
    case Bytecodes::_i2b:
      __ shll(eax, 24);      // truncate upper 24 bits
      __ sarl(eax, 24);      // and sign-extend byte
      break;
    case Bytecodes::_i2c:
      __ andl(eax, 0xFFFF);  // truncate upper 16 bits
      break;
    case Bytecodes::_i2s:
      __ shll(eax, 16);      // truncate upper 16 bits
      __ sarl(eax, 16);      // and sign-extend short
      break;
    case Bytecodes::_l2i:
      /* nothing to do */
      break;
    case Bytecodes::_l2f:
      __ pushl(edx);         // store long on tos
      __ pushl(eax);
      __ fild_d(at_esp());   // load long to ST0
      __ f2ieee();           // truncate to float size
      __ popl(ecx);          // adjust esp
      __ popl(ecx);
      break;
    case Bytecodes::_l2d:
      __ pushl(edx);         // store long on tos
      __ pushl(eax);
      __ fild_d(at_esp());   // load long to ST0
      __ d2ieee();           // truncate to double size
      __ popl(ecx);          // adjust esp
      __ popl(ecx);
      break;
    case Bytecodes::_f2i:
      __ pushl(ecx);         // reserve space for argument
      __ fstp_s(at_esp());   // pass float argument on stack
      __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::f2i), 1);
      break;
    case Bytecodes::_f2l:
      __ pushl(ecx);         // reserve space for argument
      __ fstp_s(at_esp());   // pass float argument on stack
      __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::f2l), 1);
      break;
    case Bytecodes::_f2d:
      /* nothing to do */
      break;
    case Bytecodes::_d2i:
      __ pushl(ecx);         // reserve space for argument
      __ pushl(ecx);
      __ fstp_d(at_esp());   // pass double argument on stack
      __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::d2i), 2);
      break;
    case Bytecodes::_d2l:
      __ pushl(ecx);         // reserve space for argument
      __ pushl(ecx);
      __ fstp_d(at_esp());   // pass double argument on stack
      __ call_VM_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::d2l), 2);
      break;
    case Bytecodes::_d2f:
      __ pushl(ecx);         // reserve space for f2ieee()
      __ f2ieee();           // truncate to float size
      __ popl(ecx);          // adjust esp
      break;
    default             :
      ShouldNotReachHere();
  }
}


void TemplateTable::lcmp() {
  transition(ltos, itos);
  // y = edx:eax
  __ popl(ebx);                   // get x = ecx:ebx
  __ popl(ecx);
  __ lcmp2int(ecx, ebx, edx, eax);// ecx := cmp(x, y)
  __ movl(eax, ecx);
}


void TemplateTable::float_cmp(bool is_float, int unordered_result) {
  if (is_float) {
    __ fld_s(at_esp());
  } else {
    __ fld_d(at_esp());
    __ popl(edx);
  }
  __ popl(ecx);
  __ fcmp2int(eax, unordered_result < 0);
}


void TemplateTable::count_branches(bool taken) {
#ifdef COMPILER2
  const int offset = methodOopDesc::invocation_counter_offset_in_bytes() + InvocationCounter::counter_offset_in_bytes();
  if (CountInterpBranches) {
    Label L;
    // Check invocation counter to see if you want to count the branch
    __ cmpl(Address(ecx, offset), (int)(CallProfile::unprofiled_invocations() * InvocationCounter::count_increment));
    __ jcc(Assembler::below, L);
    // Increment address (freqTable+((BCP&(0x1<<BranchFreqTableSize))*sizeof(FreqPair)))
    __ leal(eax, at_bcp(0));
    __ andl(eax, right_n_bits(BranchFreqTableSize));
    assert(SharedInfo::freqTable != NULL, "should be allocated by now");
    if (taken) {
      __ incl(Address(noreg, eax, Address::times_8, (int)(SharedInfo::freqTable) + FreqPair::taken_offset()));
    } else {
      // Add 4 bytes to the address to ge to the _not_taken field
      __ incl(Address(noreg, eax, Address::times_8, (int)(SharedInfo::freqTable) + FreqPair::not_taken_offset()));
    }
    __ bind(L);
  }
#endif
}

void TemplateTable::branch(bool is_jsr, bool is_wide) {
  __ get_method(ecx);

#ifndef CORE
  const int offset = methodOopDesc::invocation_counter_offset_in_bytes() + InvocationCounter::counter_offset_in_bytes();

  // if counting branches, increment branch taken, ecx has method
  if (UseLoopCounter && !is_jsr) {
    count_branches(true);
  }
#endif

  __ movl(edx, at_bcp(1));
  __ bswap(edx);
  if (!is_wide) __ sarl(edx, 16);
  // factor out the code below
  __ load_unsigned_byte(ebx, Address(esi, edx, Address::times_1, 0));
  if (is_jsr) {
    // compute return address as bci in eax
    __ leal(eax, at_bcp((is_wide ? 5 : 3) - methodOopDesc::codes_offset_in_bytes()));
    __ subl(eax, ecx);
  }
  __ addl(esi, edx);
#ifndef CORE
  assert(UseLoopCounter || !UseOnStackReplacement, "on-stack-replacement requires loop counters");
  if (UseLoopCounter && !is_jsr) {
    // increment invocation counter for backward branches
    // eax: free (since this code is not used for jsr's)
    // ebx: target bytecode
    // ecx: method
    // edx: target offset
    // esi: target bcp
    Label L;
    __ testl(edx, edx);             // check if forward or backward branch
    __ jcc(Assembler::positive, L); // count only if backward branch

    if (UseOnStackReplacement) {
      COMPILER1_ONLY(const int extra_frame_size = 0;);
      COMPILER2_ONLY(const int extra_frame_size = 5;);
      // count & check for overflow
      __ movl(eax, Address(ecx, offset));        // load counter
      __ increment(eax, InvocationCounter::count_increment);
      __ movl(Address(ecx, offset), eax);        // store counter
      __ cmpl(eax, Address(int(&InvocationCounter::InterpreterBackwardBranchLimit),
                           relocInfo::none));
      __ jcc(Assembler::below, L);
      __ movl(eax, (int)true);                   // overflow in loop, pass true
      __ movl(edx, extra_frame_size);            // size of osr parameters
      assert(!Bytecodes::safepoint_safe(bytecode()), "can call OSR method which may not stop");
      call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::invocation_counter_overflow), eax, edx);
      __ load_unsigned_byte(ebx, Address(esi));  // restore target bytecode
      // eax: osr nmethod (osr ok) or NULL (osr not possible)
      // ebx: target bytecode
      // edx: NULL (C1) or osr adapter frame return address (C2)
      // edi: locals pointer
      // esi: bcp
      __ testl(eax, eax);                        // test result
      __ jcc(Assembler::zero, L);                // no osr if null
      // nmethod may have been invalidated (VM bay block upon call_VM return)
      __ movl(ecx, Address(eax, nmethod::entry_bci_offset()));
      __ cmpl(ecx, InvalidOSREntryBci);
      __ jcc(Assembler::equal, L);
      
      // We have the address of an on stack replacement routine in eax, and the 
      // return address in edx (the osr-adapter). We need to prepare for the call. 
#ifdef COMPILER2
      // reserve space for incomming arguments. This is needed since an I2C adapter
      // could be introduced as part of deoptimization. The I2C adapter will think
      // that it owns the incomming arguments area. If we do not reserve space for
      // this explicitly, then the I2C adapter might override the old-ebp/old-esp
      // offsets in the OSR adapter frame, and frame traversal will fail.            
      // Figure out size of outgoing arguments
      __ movl(ebx, Address(eax, nmethod::method_offset_in_bytes()));
      // Make sure not to use ecx in the src part. Otherwise, wrong assembly will be generated
      __ load_unsigned_word(ecx, Address(ebx, methodOopDesc::size_of_parameters_offset_in_bytes()));      
      __ shll(ecx, 2); // Multiply by 4 to get size in bytes
      __ subl(esp, ecx);

      // for C2 we need to set up extra_frame_size words & return
      // address on top of the interpreter activation as follows:
      //
      //     [ retadr ]  osr adapter frame return address
      //  +--[ frame* ]  pointer to VM frame object on the stack
      //  |  [   0    ]  padding slot - may be clobbered by C2
      //  +->[  esp   ]  \
      //     [   0    ]   | VM frame object describing the interpreter frame
      //     [  ebp   ]  /
      assert(extra_frame_size == 5, "adjust constants or code below");
      // construct the VM frame object
      __ pushl(ebp);
      __ pushl(0);
      __ pushl(esp);
      // push padding slot & frame pointer
      __ movl(ecx, esp);
      __ pushl(0);
      __ pushl(ecx);
      // osr adapter frame return address
      __ pushl(edx);
#endif // COMPILER2
      // continue at osr code
      // eax: osr nmethod
      // ebx: target bytecode
      // edx: NULL (C1) or osr adapter frame return address (C2)
      // edi: locals pointer
      // esi: bcp
      __ jmp(Address(eax, nmethod::osr_entry_point_offset()));
    } else {
      // just count
      __ addl(Address(ecx, offset), InvocationCounter::count_increment);
    }
    __ bind(L);
  }
#endif // CORE
  // continue with the bytecode @ target
  // eax: return bci for jsr's, unused otherwise
  // ebx: target bytecode
  // esi: target bcp
  __ dispatch_only(is_jsr ? itos : vtos);
}


void TemplateTable::if_0cmp(Condition cc) {
  transition(itos, vtos);
  // assume branch is more often taken than not (loops use backward branches)
  Label not_taken;
  __ testl(eax, eax);
  __ jcc(j_not(cc), not_taken);
  branch(false, false);
  __ bind(not_taken);
#ifndef CORE
  // if counting branches, increment branch not taken
  __ get_method(ecx);
  count_branches(false);
#endif
}


void TemplateTable::if_icmp(Condition cc) {
  transition(itos, vtos);
  // assume branch is more often taken than not (loops use backward branches)
  Label not_taken;
  __ popl(edx);
  __ cmpl(edx, eax);
  __ jcc(j_not(cc), not_taken);
  branch(false, false);
  __ bind(not_taken);
#ifndef CORE
  // if counting branches, increment branch not taken
  __ get_method(ecx);
  count_branches(false);
#endif
}


void TemplateTable::if_nullcmp(Condition cc) {
  transition(atos, vtos);
  // assume branch is more often taken than not (loops use backward branches)
  Label not_taken;
  __ testl(eax, eax);
  __ jcc(j_not(cc), not_taken);
  branch(false, false);
  __ bind(not_taken);
#ifndef CORE
  // if counting branches, increment branch not taken
  __ get_method(ecx);
  count_branches(false);
#endif
}


void TemplateTable::if_acmp(Condition cc) {
  transition(atos, vtos);
  // assume branch is more often taken than not (loops use backward branches)
  Label not_taken;
  __ popl(edx);
  __ cmpl(edx, eax);
  __ jcc(j_not(cc), not_taken);
  branch(false, false);
  __ bind(not_taken);
#ifndef CORE
  // if counting branches, increment branch not taken
  __ get_method(ecx);
  count_branches(false);
#endif
}


void TemplateTable::ret() {
  transition(vtos, vtos);
  locals_index(ebx);
  __ get_method(eax);
  __ movl(ebx, iaddress(ebx));                   // get return bci, compute return bcp
  __ leal(esi, Address(eax, ebx, Address::times_1, methodOopDesc::codes_offset_in_bytes()));
  __ dispatch_next(vtos);
}


void TemplateTable::wide_ret() {
  transition(vtos, vtos);
  locals_index_wide(ebx);
  __ get_method(eax);
  __ movl(ebx, iaddress(ebx));                   // get return bci, compute return bcp
  __ leal(esi, Address(eax, ebx, Address::times_1, methodOopDesc::codes_offset_in_bytes()));
  __ dispatch_next(vtos);
}


void TemplateTable::tableswitch() {
  Label default_case, continue_execution;
  transition(itos, vtos);
  // align esi
  __ leal(ebx, at_bcp(wordSize));
  __ andl(ebx, -wordSize);
  // load lo & hi
  __ movl(ecx, Address(ebx, 1 * wordSize));
  __ movl(edx, Address(ebx, 2 * wordSize));
  __ bswap(ecx);
  __ bswap(edx);
  // check against lo & hi
  __ cmpl(eax, ecx);
  __ jcc(Assembler::less, default_case);
  __ cmpl(eax, edx);
  __ jcc(Assembler::greater, default_case);
  // lookup dispatch offset
  __ subl(eax, ecx);
  __ movl(edx, Address(ebx, eax, Address::times_4, 3 * wordSize));
  // continue execution
  __ bind(continue_execution);
  __ bswap(edx);
  __ load_unsigned_byte(ebx, Address(esi, edx, Address::times_1));
  __ addl(esi, edx);
  __ dispatch_only(vtos);
  // handle default
  __ bind(default_case);
  __ movl(edx, Address(ebx));
  __ jmp(continue_execution);
}


void TemplateTable::lookupswitch() {
  transition(itos, itos);
  __ stop("lookupswitch bytecode should have been rewritten");
}


void TemplateTable::fast_linearswitch() {
  transition(itos, vtos);
  Label loop_entry, loop, found, continue_execution;  
  // bswap eax so we can avoid bswapping the table entries
  __ bswap(eax);
  // align esi
  __ leal(ebx, at_bcp(wordSize));                // btw: should be able to get rid of this instruction (change offsets below)
  __ andl(ebx, -wordSize);
  // set counter
  __ movl(ecx, Address(ebx, wordSize));  
  __ bswap(ecx);
  __ jmp(loop_entry);
  // table search
  __ bind(loop);
  __ cmpl(eax, Address(ebx, ecx, Address::times_8, 2 * wordSize));
  __ jcc(Assembler::equal, found);
  __ bind(loop_entry);
  __ decl(ecx);
  __ jcc(Assembler::greaterEqual, loop);
  // default case
  __ movl(edx, Address(ebx));
  __ jmp(continue_execution);
  // entry found -> get offset
  __ bind(found);
  __ movl(edx, Address(ebx, ecx, Address::times_8, 3 * wordSize));
  // continue execution
  __ bind(continue_execution);  
  __ bswap(edx);
  __ load_unsigned_byte(ebx, Address(esi, edx, Address::times_1));
  __ addl(esi, edx);
  __ dispatch_only(vtos);
}


void TemplateTable::fast_binaryswitch() {
  transition(itos, vtos);
  // Implementation using the following core algorithm:
  //
  // int binary_search(int key, LookupswitchPair* array, int n) {
  //   // Binary search according to "Methodik des Programmierens" by
  //   // Edsger W. Dijkstra and W.H.J. Feijen, Addison Wesley Germany 1985.
  //   int i = 0;
  //   int j = n;
  //   while (i+1 < j) {
  //     // invariant P: 0 <= i < j <= n and (a[i] <= key < a[j] or Q)
  //     // with      Q: for all i: 0 <= i < n: key < a[i]
  //     // where a stands for the array and assuming that the (inexisting)
  //     // element a[n] is infinitely big.
  //     int h = (i + j) >> 1;
  //     // i < h < j
  //     if (key < array[h].fast_match()) {
  //       j = h;
  //     } else {
  //       i = h;
  //     }
  //   }
  //   // R: a[i] <= key < a[i+1] or Q
  //   // (i.e., if key is within array, i is the correct index)
  //   return i;
  // }

  // register allocation
  const Register key   = eax;                    // already set (tosca)
  const Register array = ebx;
  const Register i     = ecx;
  const Register j     = edx;
  const Register h     = edi;                    // needs to be restored
  const Register temp  = esi;
  // setup array
  __ save_bcp();

  __ leal(array, at_bcp(3*wordSize));            // btw: should be able to get rid of this instruction (change offsets below)
  __ andl(array, -wordSize);
  // initialize i & j
  __ xorl(i, i);                                 // i = 0;
  __ movl(j, Address(array, -wordSize));         // j = length(array);    
  // Convert j into native byteordering  
  __ bswap(j);
  // and start
  Label entry;
  __ jmp(entry);

  // binary search loop
  { Label loop;
    __ bind(loop);
    // int h = (i + j) >> 1;
    __ leal(h, Address(i, j, Address::times_1)); // h = i + j;
    __ sarl(h, 1);                               // h = (i + j) >> 1;
    // if (key < array[h].fast_match()) {
    //   j = h;
    // } else {
    //   i = h;
    // }
    // Convert array[h].match to native byte-ordering before compare
    __ movl(temp, Address(array, h, Address::times_8, 0*wordSize));
    __ bswap(temp);
    __ cmpl(key, temp);
    if (CodeForP6) {
      __ cmovl(Assembler::less        , j, h);   // j = h if (key <  array[h].fast_match())
      __ cmovl(Assembler::greaterEqual, i, h);   // i = h if (key >= array[h].fast_match())
    } else {
      Label set_i, end_of_if;
      __ jcc(Assembler::greaterEqual, set_i);    // {
      __ movl(j, h);                             //   j = h;
      __ jmp(end_of_if);                         // }
      __ bind(set_i);                            // else {
      __ movl(i, h);                             //   i = h;
      __ bind(end_of_if);                        // }
    }
    // while (i+1 < j)
    __ bind(entry);
    __ leal(h, Address(i, 1));                   // i+1
    __ cmpl(h, j);                               // i+1 < j
    __ jcc(Assembler::less, loop);
  }

  // end of binary search, result index is i (must check again!)
  Label default_case;
  // Convert array[i].match to native byte-ordering before compare
  __ movl(temp, Address(array, i, Address::times_8, 0*wordSize));
  __ bswap(temp);
  __ cmpl(key, temp);
  __ jcc(Assembler::notEqual, default_case);

  // entry found -> j = offset    
  __ movl(j , Address(array, i, Address::times_8, 1*wordSize));
  __ bswap(j);
  __ restore_bcp();
  __ restore_locals();                           // restore edi
  __ load_unsigned_byte(ebx, Address(esi, j, Address::times_1));
  
  __ addl(esi, j);
  __ dispatch_only(vtos);

  // default case -> j = default offset
  __ bind(default_case);
  __ movl(j, Address(array, -2*wordSize));
  __ bswap(j);
  __ restore_bcp();
  __ restore_locals();                           // restore edi
  __ load_unsigned_byte(ebx, Address(esi, j, Address::times_1));
  __ addl(esi, j);
  __ dispatch_only(vtos);
}


void TemplateTable::_return(TosState state) {
  transition(state, state);
  assert(_desc->calls_vm(), "inconsistent calls_vm information"); // call in remove_activation
  __ remove_activation(state, esi);
  __ jmp(esi);
}


void TemplateTable::resolve(int byte_no) {
  assert(byte_no == 1 || byte_no == 2, "byte_no out of range");
  const int shift_count = (1 + byte_no)*BitsPerByte;
  Label redo, resolve;
  __ bind(redo);
  __ get_cache_and_index_at_bcp(ecx, edx, 1);
  __ movl(ebx, Address(ecx, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes()));
  __ shrl(ebx, shift_count);
  // do we have new quicker bytecode?
  __ andl(ebx, 0xFF);
  __ jcc(Assembler::zero, resolve);


  Label patch_done;
  if (jvmdi::enabled() && UseFastBreakpoints) {
    Label fast_patch;
    // if a breakpoint is present we can't rewrite the stream directly
    __ movzxb(ecx, at_bcp(0));
    __ cmpl(ecx, Bytecodes::_breakpoint);
    __ jcc(Assembler::notEqual, fast_patch);
    __ get_method(ecx);
    // Let breakpoint table handling rewrite to quicker bytecode 
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::set_original_bytecode_at), ecx, esi, ebx);
    __ jmp(patch_done);
    __ bind(fast_patch);
  }
  // patch bytecode
  __ movb(at_bcp(0), ebx);
  __ bind(patch_done);

  // dispatch via ebx
  __ dispatch_only(vtos);
  // resolve
  address entry;
  switch (bytecode()) {
    case Bytecodes::_getstatic      : // fall through
    case Bytecodes::_putstatic      : // fall through
    case Bytecodes::_getfield       : // fall through
    case Bytecodes::_putfield       : entry = CAST_FROM_FN_PTR(address, InterpreterRuntime::resolve_get_put); break;
    case Bytecodes::_invokevirtual  : // fall through
    case Bytecodes::_invokespecial  : // fall through
    case Bytecodes::_invokestatic   : // fall through
    case Bytecodes::_invokeinterface: entry = CAST_FROM_FN_PTR(address, InterpreterRuntime::resolve_invoke);  break;
    default                         : ShouldNotReachHere();                                 break;
  }
  __ bind(resolve);
  __ movl(ebx, (int)bytecode());
  __ call_VM(noreg, entry, ebx);
  __ jmp(redo);
}


void TemplateTable::fast_accessstatic(bool load) {
  // do the JVM/DI work here to avoid disturbing the register state below
  if (jvmdi::enabled()) {
    if (load) {
      // Check to see if a field access watch has been set before we take
      // the time to call into the VM.
      Label L1;
      __ movl(ecx, Address(jvmdi::get_field_access_count_addr(), relocInfo::none));
      __ testl(ecx, ecx);
      __ jcc(Assembler::zero, L1);
      // access constant pool cache entry
      __ get_cache_entry_pointer_at_bcp(ecx, edx, 1);
      __ movl(edx, 0);  // NULL object reference
      // edx: NULL object pointer
      // ecx: cache entry pointer
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_access), edx, ecx);
      __ bind(L1);
    } else {
      // Check to see if a field modification watch has been set before we take
      // the time to call into the VM.
      Label L2;
      __ movl(ecx, Address(jvmdi::get_field_modification_count_addr(), relocInfo::none));
      __ testl(ecx, ecx);
      __ jcc(Assembler::zero, L2);
      __ subl(esp, sizeof(jvalue));  // add space for a jvalue object
      __ movl(ecx, esp);
      __ movl(ebx, 0);               // use as index and as NULL in call_VM() below
      const Address lo_value(ecx, ebx, Address::times_4, 0*wordSize);
      const Address hi_value(ecx, ebx, Address::times_4, 1*wordSize);
      switch (bytecode()) {          // load values into the jvalue object
      case Bytecodes::_fast_iputstatic: __ movl(lo_value, eax);                         break;
      case Bytecodes::_fast_lputstatic: __ movl(hi_value, edx); __ movl(lo_value, eax); break;
      // need to call fld_s() after fstp_s() to restore the value for below
      case Bytecodes::_fast_fputstatic: __ fstp_s(lo_value); __ fld_s(lo_value);        break;
      // need to call fld_d() after fstp_d() to restore the value for below
      case Bytecodes::_fast_dputstatic: __ fstp_d(lo_value); __ fld_d(lo_value);        break;
      // since ecx is not an object we don't call store_check() here
      case Bytecodes::_fast_aputstatic: __ movl(lo_value, eax);                         break;
      }

      // Save eax and sometimes edx because call_VM() will clobber them,
      // then use them for JVM/DI purposes
      __ pushl(eax);
      if (bytecode() == Bytecodes::_fast_lputstatic) __ pushl(edx);
      // access constant pool cache entry
      __ get_cache_entry_pointer_at_bcp(eax, edx, 1);
      // ebx: NULL object pointer set up above
      // eax: cache entry pointer
      // ecx: jvalue object on the stack
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_modification), ebx, eax, ecx);
      if (bytecode() == Bytecodes::_fast_lputstatic) __ popl(edx);  // restore high value
      __ popl(eax);     // restore lower value   
      __ addl(esp, sizeof(jvalue));  // release jvalue object space
      __ bind(L2);
    }
  }

  // access constant pool cache
  if (bytecode() == Bytecodes::_fast_lputstatic) __ pushl(edx); // save high value
  __ get_cache_and_index_at_bcp(ecx, edx, 1);
  // extract field offset from cache entry
  __ movl(ebx, Address(ecx, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize));
  // replace cache pointer with klass reference
  __ movl(ecx, Address(ecx, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 1*wordSize));
  if (bytecode() == Bytecodes::_fast_lputstatic) __ popl(edx);  // restore high value
  // field addresses
  const Address lo(ecx, ebx, Address::times_4, 0*wordSize);
  const Address hi(ecx, ebx, Address::times_4, 1*wordSize);
  // access static
  switch (bytecode()) {
    // loads
    case Bytecodes::_fast_igetstatic: transition(vtos, itos); __ movl(eax, lo);                          return;
    case Bytecodes::_fast_lgetstatic: transition(vtos, ltos); __ movl(edx, hi); __ movl(eax, lo);        return;
    case Bytecodes::_fast_fgetstatic: transition(vtos, ftos); __ fld_s(lo);                              return;
    case Bytecodes::_fast_dgetstatic: transition(vtos, dtos); __ fld_d(lo);                              return;
    case Bytecodes::_fast_agetstatic: transition(vtos, atos); __ movl(eax, lo); __ verify_oop(eax);      return;
    // stores
    case Bytecodes::_fast_iputstatic: transition(itos, vtos); __ movl(lo, eax);                          return;
    case Bytecodes::_fast_lputstatic: transition(ltos, vtos); __ movl(hi, edx); __ movl(lo, eax);        return;
    case Bytecodes::_fast_fputstatic: transition(ftos, vtos); __ fstp_s(lo);                             return;
    case Bytecodes::_fast_dputstatic: transition(dtos, vtos); __ fstp_d(lo);                             return;
    case Bytecodes::_fast_aputstatic: transition(atos, vtos); __ movl(lo, eax); __ store_check(ecx, lo); return;
  }
  ShouldNotReachHere();
}


void TemplateTable::fast_accessfield(bool load) {
  // Note: does no explicit NULL check (unless +GenerateInterpreterNullChecks);
  //       assumes that the offset is known to be in a range that doesn't need
  //       explicit checks (see also MacroAssembler::null_check(...) for details).

  // do the JVM/DI work here to avoid disturbing the register state below
  if (jvmdi::enabled()) {
    if (load) {
      // Check to see if a field access watch has been set before we take
      // the time to call into the VM.
      Label L1;
      __ movl(ecx, Address(jvmdi::get_field_access_count_addr(), relocInfo::none));
      __ testl(ecx,ecx);
      __ jcc(Assembler::zero, L1);
      // access constant pool cache entry
      __ get_cache_entry_pointer_at_bcp(ecx, edx, 1);
      __ pushl(eax);  // save object pointer before call_VM() clobbers it
      __ verify_oop(eax);
      // eax: object pointer copied above
      // ecx: cache entry pointer
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_access), eax, ecx);
      __ popl(eax);   // restore object pointer
      __ bind(L1);
    } else {
      // Check to see if a field modification watch has been set before we take
      // the time to call into the VM.
      Label L2;
      __ movl(ecx, Address(jvmdi::get_field_modification_count_addr(), relocInfo::none));
      __ testl(ecx,ecx);
      __ jcc(Assembler::zero, L2);
      __ popl(ebx);                  // copy the object pointer from tos
      __ verify_oop(ebx);
      __ pushl(ebx);                 // put the object pointer back on tos
      __ subl(esp, sizeof(jvalue));  // add space for a jvalue object      
      __ movl(ecx, esp);
      __ pushl(ebx);                 // save object pointer so we can steal ebx
      __ movl(ebx, 0);
      const Address lo_value(ecx, ebx, Address::times_4, 0*wordSize);
      const Address hi_value(ecx, ebx, Address::times_4, 1*wordSize);
      switch (bytecode()) {          // load values into the jvalue object
      case Bytecodes::_fast_iputfield: __ movl(lo_value, eax);                         break;
      case Bytecodes::_fast_lputfield: __ movl(hi_value, edx); __ movl(lo_value, eax); break;
      // need to call fld_s() after fstp_s() to restore the value for below
      case Bytecodes::_fast_fputfield: __ fstp_s(lo_value); __ fld_s(lo_value);        break;
      // need to call fld_d() after fstp_d() to restore the value for below
      case Bytecodes::_fast_dputfield: __ fstp_d(lo_value); __ fld_d(lo_value);        break;
      // since ecx is not an object we don't call store_check() here
      case Bytecodes::_fast_aputfield: __ movl(lo_value, eax);                         break;
      }
      __ popl(ebx);  // restore copy of object pointer

      // Save eax and sometimes edx because call_VM() will clobber them,
      // then use them for JVM/DI purposes
      __ pushl(eax);
      if (bytecode() == Bytecodes::_fast_lputfield) __ pushl(edx);
      // access constant pool cache entry
      __ get_cache_entry_pointer_at_bcp(eax, edx, 1);
      __ verify_oop(ebx);
      // ebx: object pointer copied above
      // eax: cache entry pointer
      // ecx: jvalue object on the stack
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_modification), ebx, eax, ecx);
      if (bytecode() == Bytecodes::_fast_lputfield) __ popl(edx);  // restore high value
      __ popl(eax);     // restore lower value   
      __ addl(esp, sizeof(jvalue));  // release jvalue object space
      __ bind(L2);
    }
  }

  // access constant pool cache
  __ get_cache_and_index_at_bcp(ecx, ebx, 1);
  // replace index with field offset from cache entry
  __ movl(ebx, Address(ecx, ebx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize));
  // field addresses
  Address lo, hi;
  if (load) {
    // eax: object
    __ verify_oop(eax);
    null_check(eax, 0);
    lo = Address(eax, ebx, Address::times_4, 0*wordSize);
    hi = Address(eax, ebx, Address::times_4, 1*wordSize);
  } else {
    // tos: object
    __ popl(ecx);
    __ verify_oop(ecx);
    null_check(ecx, 0);
    lo = Address(ecx, ebx, Address::times_4, 0*wordSize);
    hi = Address(ecx, ebx, Address::times_4, 1*wordSize);
  }
  // access field
  switch (bytecode()) {
    // loads
    case Bytecodes::_fast_igetfield: transition(atos, itos); __ movl(eax, lo);                          return;
    case Bytecodes::_fast_lgetfield: transition(atos, ltos); __ movl(edx, hi); __ movl(eax, lo);        return;
    case Bytecodes::_fast_fgetfield: transition(atos, ftos); __ fld_s(lo);                              return;
    case Bytecodes::_fast_dgetfield: transition(atos, dtos); __ fld_d(lo);                              return;
    case Bytecodes::_fast_agetfield: transition(atos, atos); __ movl(eax, lo); __ verify_oop(eax);      return;
    // stores
    case Bytecodes::_fast_iputfield: transition(itos, vtos); __ movl(lo, eax);                          return;
    case Bytecodes::_fast_lputfield: transition(ltos, vtos); __ movl(hi, edx); __ movl(lo, eax);        return;
    case Bytecodes::_fast_fputfield: transition(ftos, vtos); __ fstp_s(lo);                             return;
    case Bytecodes::_fast_dputfield: transition(dtos, vtos); __ fstp_d(lo);                             return;
    case Bytecodes::_fast_aputfield: transition(atos, vtos); __ movl(lo, eax); __ store_check(ecx, lo); return;
  }
  ShouldNotReachHere();
}


void TemplateTable::fast_get(bool is_static) {
  guarantee(!is_static, "static version of this is not used");
  transition(atos, vtos);
  // access constant pool cache
  __ get_cache_and_index_at_bcp(ecx, edx, 1);
  __ movl(ebx, Address(ecx, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize));
  __ movl(ecx, Address(ecx, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize));
  // eax: object
  // ebx: offset in words
  // ecx: flags
  // field addresses
  __ verify_oop(eax);
  __ null_check(eax);
  const Address lo(eax, ebx, Address::times_4, 0*wordSize);
  const Address hi(eax, ebx, Address::times_4, 1*wordSize);
  // access field (assume its 2 words long for better scheduling)
  __ movl(edx, hi);
  __ movl(eax, lo);
  // push field on stack
  { Label one_word;
    __ testl(ecx, 1 << 30);
    __ jcc(Assembler::zero, one_word);
    __ pushl(edx);
    __ bind(one_word);
  }
  __ pushl(eax);
}


void TemplateTable::fast_put(bool is_static) {
  transition(vtos, vtos);
  // access constant pool cache
  __ get_cache_and_index_at_bcp(eax, edx, 1);
  __ movl(ecx, Address(eax, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 1*wordSize));
  __ movl(ebx, Address(eax, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize));
  __ movl(edx, Address(eax, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize));
  // ebx: offset in words
  // ecx: klass
  // edx: flags  
  // field addresses
  const Address lo(ecx, ebx, Address::times_4, 0*wordSize);
  const Address hi(ecx, ebx, Address::times_4, 1*wordSize);
  // assume 1-word case: get value & object (if !is_static)
  __ popl(eax);
  if (!is_static) __ popl(ecx);
  // determine case
  // eax: lo value
  // ebx: offset in words
  // ecx: klass or object  (if !is_static)
  // edx: flags
  Label one_word, one_word_store_check, exit;
  __ testl(edx, 3 << 30);
  __ jcc(Assembler::zero    , one_word);
  __ jcc(Assembler::negative, one_word_store_check);
  // store 2-word value
  if (is_static) {
    __ popl(edx);
  } else {
    __ movl(edx, ecx);
    __ popl(ecx);
    __ null_check(ecx);
  }
  __ verify_oop(ecx);
  __ movl(hi, edx);
  __ movl(lo, eax);
  __ jmp(exit);
  // store check
  __ bind(one_word_store_check);
  __ verify_oop(ecx);
  if (!is_static) null_check(ecx);
  __ movl(lo, eax);
  __ store_check(ecx, lo);
  __ jmp(exit);
  // store 1-word value
  __ bind(one_word);
  __ verify_oop(ecx);
  if (!is_static) null_check(ecx);
  __ movl(lo, eax);
  // done
  __ bind(exit);
}


void TemplateTable::fast_xaccess(TosState state) {
  transition(vtos, state);
  // get receiver
  __ movl(eax, aaddress(0));
  // access constant pool cache
  __ get_cache_and_index_at_bcp(ecx, edx, 2);
  __ movl(ebx, Address(ecx, edx, Address::times_4, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize));
  // make sure exception is reported in correct bcp range (getfield is next instruction)
  __ incl(esi);
  __ null_check(eax, 0);
  __ movl(eax, Address(eax, ebx, Address::times_4));
  __ decl(esi);
}


//----------------------------------------------------------------------------------------------------
// Calls

void TemplateTable::count_calls(Register method, Register temp) {  
#ifdef COMPILER2
  assert(method != temp, "registers must be different");
  const int offset = methodOopDesc::invocation_counter_offset_in_bytes() + InvocationCounter::counter_offset_in_bytes();
  if (CountInterpCalls) {
    Label L;
    // Check invocation counter to see if you want to count the call
    __ verify_oop(method);
    // compare methodoop+offset to target count
    __ cmpl(Address(method, offset), (int)(CallProfile::unprofiled_invocations() * InvocationCounter::count_increment));
    __ jcc(Assembler::below, L);
    // Increment address (callTable+((BCP&(0x1<<BranchFreqTableSize))*sizeof(int)))
    __ leal(temp, at_bcp(0));
    __ andl(temp, right_n_bits(CallFreqTableSize));
    assert(SharedInfo::callTable != NULL, "should be allocated by now");
    __ incl(Address(noreg, temp, Address::times_4, (int)(SharedInfo::callTable )));
    __ bind(L);
  }
#endif
}


static void access_cp_cache(InterpreterMacroAssembler* _masm, Register method, Register itable_index, Register flags, bool is_invokevirtual = false) {
  // setup registers
  const Register cache = ecx;
  const Register index = edx;
  assert(method != flags && method != cache && method != index,                  "registers must be different (1)");
  assert(itable_index != flags && itable_index!= cache && itable_index != index, "registers must be different (2)");
  // determine constant pool cache field offsets
  const int method_offset = constantPoolCacheOopDesc::base_offset_in_bytes() +
    (is_invokevirtual
     ? ConstantPoolCacheEntry::f2_offset_in_bytes()
     : ConstantPoolCacheEntry::f1_offset_in_bytes()
    );
  const int flags_offset = constantPoolCacheOopDesc::base_offset_in_bytes() +
    ConstantPoolCacheEntry::flags_offset_in_bytes();
  // access constant pool cache fields
  const int index_offset = constantPoolCacheOopDesc::base_offset_in_bytes() + ConstantPoolCacheEntry::f2_offset_in_bytes();

  __ get_cache_and_index_at_bcp(cache, index, 1);
  assert(wordSize == 4, "adjust code below");
  __ movl(method, Address(cache, index, Address::times_4, method_offset));
  if (itable_index != noreg) {
    __ movl(itable_index, Address(cache, index, Address::times_4, index_offset ));
  }
  __ movl(flags , Address(cache, index, Address::times_4, flags_offset ));
  
}


static void prepare_invoke(InterpreterMacroAssembler* _masm, Register method, Register index, Bytecodes::Code code) {
  // determine flags
  Bytecodes::Code java_code = Bytecodes::java_code(code);
  const bool is_invokevirtual    = java_code == Bytecodes::_invokevirtual;
  const bool is_invokeinterface  = java_code == Bytecodes::_invokeinterface;
  const bool load_receiver       = java_code != Bytecodes::_invokestatic;
  const bool receiver_null_check = code == Bytecodes::_fast_invokevfinal || code == Bytecodes::_fast_invokespecial;
  // setup registers & access constant pool cache
  const Register recv   = ecx;
  const Register flags  = edx;  
  assert(method != index,                   "registers must be different (1)");
  assert(method != recv && method != flags, "registers must be different (2)");
  assert(index  != recv && index  != flags, "registers must be different (3)");
  access_cp_cache(_masm, method, index, flags, is_invokevirtual);
  // save 'interpreter return address'
  __ save_bcp();
  // compute parameter size if needed
  if (load_receiver) {
    __ movl(recv, flags);
    __ andl(recv, 0xFF);
  }
  // compute return type
  __ shrl(flags, 29);
  // load receiver if needed (note: no return address pushed yet)
  if (load_receiver) {
    __ movl(recv, Address(esp, recv, Address::times_4, -wordSize));
    __ verify_oop(recv);
  }
  // load return address
  { const int table =
      is_invokeinterface
      ? (int)Interpreter::return_5_addrs_by_index_table()
      : (int)Interpreter::return_3_addrs_by_index_table();
    __ movl(flags, Address(noreg, flags, Address::times_4, table));
  }
  // do null check if needed
  if (receiver_null_check) {
    __ null_check(recv);
  }
  // push return address
  __ pushl(flags);
}


void TemplateTable::fast_invokevtable() {
  transition(vtos, vtos);
  prepare_invoke(_masm, ebx, noreg, bytecode());
  // get receiver klass
  __ null_check(ecx, oopDesc::klass_offset_in_bytes());
  __ movl(eax, Address(ecx, oopDesc::klass_offset_in_bytes()));
  __ verify_oop(eax);
  // get target methodOop & entry point
  const int base = instanceKlass::vtable_start_offset() * wordSize;    
  assert(vtableEntry::size() * wordSize == 4, "adjust the scaling in the code below");
  __ movl(ebx, Address(eax, ebx, Address::times_4, base + vtableEntry::method_offset_in_bytes()));  
  __ movl(edx, Address(ebx, methodOopDesc::interpreter_entry_offset_in_bytes()));  
  __ movl(eax, ebx); // the i2c_adapters need methodOop in eax  
  // do the call
#ifdef ASSERT
  { Label L;
    __ testl(edx, edx);
    __ jcc(Assembler::notZero, L);
    __ stop("interpreter entry point is null");
    __ bind(L);
  }
#endif
  count_calls(ebx, eax);
  __ jmp(edx);
}


void TemplateTable::fast_invokevfinal() {
  transition(vtos, vtos);
  prepare_invoke(_masm, ebx, noreg, bytecode());
  // do the call
  __ verify_oop(ebx);
  count_calls(ebx, eax);
  __ jmp(Address(ebx, methodOopDesc::interpreter_entry_offset_in_bytes()));
}


void TemplateTable::fast_invokespecial() {
  transition(vtos, vtos);
  prepare_invoke(_masm, ebx, noreg, bytecode());
  // do the call
  __ verify_oop(ebx);
  count_calls(ebx, eax);
  __ jmp(Address(ebx, methodOopDesc::interpreter_entry_offset_in_bytes()));
}


void TemplateTable::fast_invokestatic() {
  transition(vtos, vtos);
  prepare_invoke(_masm, ebx, noreg, bytecode());
  // do the call
  __ verify_oop(ebx);
  count_calls(ebx, eax);
  __ jmp(Address(ebx, methodOopDesc::interpreter_entry_offset_in_bytes()));
}


void TemplateTable::fast_invokeinterface() {
  transition(vtos, vtos);
  prepare_invoke(_masm, eax, ebx, bytecode());
  
  // eax: Interface
  // ebx: index
  // ecx: receiver    
      
  // Get receiver klass into eax
  __ movl(edx, Address(ecx, oopDesc::klass_offset_in_bytes()));
  __ movl(edi, edx); // Save klassOop in edi
  __ verify_oop(edx);
     
  // Compute start of first itableOffsetEntry (which is at the end of the vtable)
  const int base = instanceKlass::vtable_start_offset() * wordSize;    
  assert(vtableEntry::size() * wordSize == 4, "adjust the scaling in the code below");
  __ movl(esi, Address(edx, instanceKlass::vtable_length_offset() * wordSize)); // Get length of vtable
  __ leal(edx, Address(edx, esi, Address::times_4, base));
  
  Label entry, search;
  
  __ jmp(entry);   
  __ bind(search);
  __ addl(edx, itableOffsetEntry::size() * wordSize);
  
  __ bind(entry);    
#ifdef ASSERT
  // Check that the entry is non-null    
  { Label L;
    __ pushl(edx);
    __ movl(edx, Address(edx, itableOffsetEntry::interface_offset_in_bytes()));
    __ testl(edx, edx);
    __ jcc(Assembler::notZero, L);
    __ stop("null entry point found in itable's offset table");
    __ bind(L);
    __ popl(edx);
  }
#endif
    __ cmpl(eax, Address(edx, itableOffsetEntry::interface_offset_in_bytes()));
    __ jcc(Assembler::notEqual, search);
        
    __ movl(edx, Address(edx, itableOffsetEntry::offset_offset_in_bytes()));      
    __ addl(edx, edi); // Add offset to klassOop
    assert(itableMethodEntry::size() * wordSize == 4, "adjust the scaling in the code below");
    __ movl(ebx, Address(edx, ebx, Address::times_4));
    // ebx: methodOop to call
    // ecx: receiver
    // Check for abstract method error
    // Note: This should be done more efficiently via a throw_abstract_method_error
    //       interpreter entry point and a conditional jump to it in case of a null
    //       method.
    { Label L;
      __ testl(ebx, ebx);
      __ jcc(Assembler::notZero, L);
      // throw exception
	  // note: must restore interpreter registers to canonical
	  //       state for exception handling to work correctly!
	  __ popl(ebx);          // pop return address (pushed by prepare_invoke)
	  __ restore_bcp();      // esi must be correct for exception handler   (was destroyed)
	  __ restore_locals();   // make sure locals pointer is correct as well (was destroyed)
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_abstract_method_error));
      // the call_VM checks for exception, so we should never return here.
      __ should_not_reach_here();
      __ bind(L);
    }
    Label run_interpreted_code;
#ifdef COMPILER1      
  // Check if compiled code exist
  __ movl(edx, Address(ebx, methodOopDesc::compiled_code_offset_in_bytes()));
  __ testl(edx, edx);
  __ jcc(Assembler::zero, run_interpreted_code);

  // Get interpreter entry point into compiled code
  __ movl(edx, Address(edx, nmethod::interpreter_entry_point_offset()));
  
#ifdef ASSERT
  { Label L;
    __ testl(edx, edx);
    __ jcc(Assembler::notZero, L);
    __ stop("nmethod's interpreter entry point is null");
    __ bind(L);
  }
#endif    
  // ecx: receiver
  // ebx: methodOop
  __ jmp(edx);  
#endif // COMPILER1

  __ bind(run_interpreted_code);

  __ movl(edx, Address(ebx, methodOopDesc::interpreter_entry_offset_in_bytes()));  

  COMPILER2_ONLY(__ movl(eax, ebx);) // the i2c_adapters need methodOop in eax      
  // do the call
  #ifdef ASSERT
  { Label L;
    __ testl(edx, edx);
    __ jcc(Assembler::notZero, L);
    __ stop("interpreter entry point is null");
    __ bind(L);
  }
#endif    
  // ecx: receiver
  // ebx: methodOop
  count_calls(ebx, eax);
  __ jmp(edx);  
}

//----------------------------------------------------------------------------------------------------
// Allocation

void TemplateTable::_new() {
  transition(vtos, atos);
  __ get_unsigned_2_byte_index_at_bcp(edx, 1);
  __ get_constant_pool(eax);
  call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::_new), eax, edx);
}


void TemplateTable::fast_new() {
  transition(vtos, atos);
  Label slow_case;
  Label done;
  Label initialize_header;
  Label initialize_object;  // including clearing the fields
  Label allocate_shared;

  // make sure this code should be called
  if (!UseFastInterpreterAllocation) {
    __ should_not_reach_here();
    return;
  }

  // get constant pool & index
  __ get_unsigned_2_byte_index_at_bcp(edx, 1);
  __ get_constant_pool(ecx);
  
  // get instanceKlass
  __ movl(ecx, Address(ecx, edx, Address::times_4, sizeof(constantPoolOopDesc)));

  // make sure klass is initialized & doesn't have has_finalized
#ifdef ASSERT
    // make sure klass is fully initialized
    { Label L;
      __ cmpl(Address(ecx, instanceKlass::init_state_offset_in_bytes() + sizeof(oopDesc)), instanceKlass::fully_initialized);
      __ jcc(Assembler::equal, L);
      __ stop("fast_new executed where klass is not fully initialized");
      __ bind(L);
    }
#endif

  // get size_helper in instanceKlass (already aligned if ALIGN_ALL_OBJECTS
  // is set)
  __ movl(edx, Address(ecx, Klass::size_helper_offset_in_bytes() + sizeof(oopDesc)));
  __ pushl(ecx);  // save the contexts of klass for initializing the header

  // 
  // Allocate the instance
  // 1) Try to allocate in the TLE
  // 2) if fail and the object is large allocate in the shared Eden
  // 3) if the above fails (or is not applicable), go to a slow case
  // (creates a new TLE, etc.)

  if (UseTLE) {
    const Register thread = ecx;

    __ get_thread(thread);
    __ movl(eax, Address(thread, JavaThread::tle_top_offset_in_bytes()));
    __ leal(ebx, Address(eax, edx, Address::times_4));
    __ cmpl(ebx, Address(thread, JavaThread::tle_end_offset_in_bytes()));
    __ jcc(Assembler::above, allocate_shared);
    __ movl(Address(thread, JavaThread::tle_top_offset_in_bytes()), ebx);
    if (ZeroTLE) {
      // the fields have been already cleared
      __ jmp(initialize_header);
    } else {
      // initialize both the header and fields
      __ jmp(initialize_object);
    }

  }
  // Allocation in the shared Eden
  //
  // edx: instance size in words
  __ bind(allocate_shared);
  {
    Label retry;
    __ bind(retry);
    __ movl(eax, Address((int)Universe::new_gen()->eden()->top_addr(), relocInfo::none));
    __ leal(ebx, Address(eax, edx, Address::times_4));
    __ cmpl(ebx, Address((int)Universe::new_gen()->eden()->end_addr(), relocInfo::none));
    __ jcc(Assembler::above, slow_case);

    // Compare eax with the top addr, and if still equal, store the new
    // top addr in ebx at the address of the top addr pointer. Sets ZF if was
    // equal, and clears it otherwise. Use lock prefix for atomicity on MPs.
    //
    // eax: object begin
    // ebx: object end
    // edx: instance size in words
    if (os::is_MP()) __ lock();
    __ cmpxchg(ebx, Address((int)Universe::new_gen()->eden()->top_addr(), relocInfo::none));

    // if someone beat us on the allocation, try again, otherwise continue 
    __ jcc(Assembler::notEqual, retry);
  }

  // The object is initialized before the header.  If the object size is
  // zero, go directly to the header initialization.
  __ bind(initialize_object);
  __ decrement(edx, sizeof(oopDesc) / oopSize);
  __ jcc(Assembler::zero, initialize_header);

  // Initialize topmost object field, divide edx by 2, check if odd and
  // test if zero.
  __ xorl(ecx, ecx);    // use zero reg to clear memory (shorter code)
  __ shrl(edx, 1);      // divide by 2 and set carry flag if odd
  if (ALIGN_ALL_OBJECTS) {
    // edx must have been multiple of 2
#ifdef ASSERT
      // make sure edx was multiple of 2
      Label L;
      __ jcc(Assembler::carryClear, L);
      __ stop("object size is not multiple of 2 - adjust this code");
      __ bind(L);
      // edx must be > 0, no extra check needed here
#endif
  } else {
    // edx could have been odd
    Label even;
    __ jcc(Assembler::carryClear, even);
    // Clear topmost word (no jump needed if conditional assignment would
    // work here).
    __ movl(Address(eax, edx, Address::times_8, sizeof(oopDesc) - 0*oopSize), ecx);
    __ bind(even);
    // edx could be 0, need to check again
    __ jcc(Assembler::zero, initialize_header);
  }

  // initialize remaining object fields: edx is a multiple of 2 now
  { Label loop;
    __ bind(loop);
    __ movl(Address(eax, edx, Address::times_8, sizeof(oopDesc) - 1*oopSize), ecx);
    __ movl(Address(eax, edx, Address::times_8, sizeof(oopDesc) - 2*oopSize), ecx);
    __ decl(edx);
    __ jcc(Assembler::notZero, loop);
  }

  // initialize object header only.
  __ bind(initialize_header);
  __ movl(Address(eax, oopDesc::mark_offset_in_bytes ()),
          (int)markOopDesc::prototype()); // header
  __ popl(ecx);   // get saved klass back in the register.
  __ movl(Address(eax, oopDesc::klass_offset_in_bytes()), ecx);  // klass
  __ jmp(done);

  // slow case
  __ bind(slow_case);
  __ popl(ecx);   // restore stack pointer to what it was when we came in.
  __ get_constant_pool(eax);
  __ get_unsigned_2_byte_index_at_bcp(edx, 1);
  call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::_new), eax, edx);

  // continue
  __ bind(done);
}

void TemplateTable::newarray() {
  transition(itos, atos);
  __ pushl(eax);                                 // make sure everything is on the stack
  __ load_unsigned_byte(edx, at_bcp(1));
  call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::newarray), edx, eax);
  __ popl(edx);                                  // discard size
}


void TemplateTable::anewarray() {
  transition(itos, atos);
  __ get_unsigned_2_byte_index_at_bcp(edx, 1);
  __ get_constant_pool(ecx);
  call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::anewarray), ecx, edx, eax);
}


void TemplateTable::arraylength() {
  transition(atos, itos);
  null_check(eax, arrayOopDesc::length_offset_in_bytes());
  __ movl(eax, Address(eax, arrayOopDesc::length_offset_in_bytes()));
}


void TemplateTable::checkcast() {
  transition(atos, atos);
  Label done, is_null;
  __ testl(eax, eax);
  __ jcc(Assembler::zero, is_null);

  // get obj class & specified class
  __ get_unsigned_2_byte_index_at_bcp(edx, 1);
  __ get_constant_pool(ecx);
  __ movl(ebx, Address(eax, oopDesc::klass_offset_in_bytes()));
  __ movl(ecx, Address(ecx, edx, Address::times_4, sizeof(constantPoolOopDesc)));

  // check if object class' superclass caches hold the specified class
  // eax: object
  // ebx: object class
  // ecx: specified class
  __ cmpl(ecx, ebx);    // check if classes are identical
  __ jcc(Assembler::equal, done);
  __ cmpl(ecx, Address(ebx, sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes()));
  __ jcc(Assembler::equal, done);
  __ cmpl(ecx, Address(ebx, sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes()));
  __ jcc(Assembler::equal, done);

  // check failed, need to call runtime routine
  __ pushl(eax);        // save object (must be on stack for GC)
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::checkcast), eax);
  __ popl(eax);         // restore object

  // Collect counts on whether this check-cast sees NULLs a lot or not.
#ifdef CORE
  __ bind(is_null);
#else
#ifdef COMPILER2
  if (UncommonNullCast) {
    __ jmp(done);
    __ bind(is_null);  
    // Flag as seeing a null cast
    __ get_method(ecx);
    __ movb(Address(ecx,methodOopDesc::null_cast_seen_offset_in_bytes()),1);
  } else {
    __ bind(is_null);
  }
#else
  __ bind(is_null);
#endif
#endif

  __ bind(done);
}


void TemplateTable::instanceof() {
  transition(atos, itos);
  Label done;
  __ testl(eax, eax);
  __ jcc(Assembler::zero, done);

  // get obj class & specified class
  __ get_unsigned_2_byte_index_at_bcp(edx, 1);
  __ get_constant_pool(ecx);
  __ movl(ebx, Address(eax, oopDesc::klass_offset_in_bytes()));
  __ movl(ecx, Address(ecx, edx, Address::times_4, sizeof(constantPoolOopDesc)));

  // check if object class' superclass caches hold the specified class
  // eax: object
  // ebx: object class
  // ecx: specified class
  __ movl(edx, eax);    // save object in case quick tests fail
  __ movl(eax, 1);      // set result assuming quick tests succeed
  __ cmpl(ecx, ebx);    // check if classes are identical
  __ jcc(Assembler::equal, done);
  __ cmpl(ecx, Address(ebx, sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes()));
  __ jcc(Assembler::equal, done);
  __ cmpl(ecx, Address(ebx, sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes()));
  __ jcc(Assembler::equal, done);

  // check failed, need to call runtime routine
  __ pushl(edx);        // save object (must be on stack for GC)
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::instanceof), edx);
  __ popl(edx);         // discard object, result is in eax

  __ bind(done);
  // eax = 0: obj == NULL or  obj is not an instanceof the specified klass
  // eax = 1: obj != NULL and obj is     an instanceof the specified klass
}

//----------------------------------------------------------------------------------------------------
// Breakpoints
void TemplateTable::_breakpoint() {
  
  // Note: We get here even if we are single stepping..
  // jbug inists on setting breakpoints at every bytecode 
  // even if we are in single step mode.  
 
  transition(vtos, vtos);

  // get the unpatched byte code
  __ get_method(ecx);
  __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::get_original_bytecode_at), ecx, esi);
  __ movl(ebx, eax);

  // post the breakpoint event
  __ get_method(ecx);
  __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::_breakpoint), ecx, esi);

  // complete the execution of original bytecode
  __ dispatch_only_normal(vtos);
} 


//----------------------------------------------------------------------------------------------------
// Exceptions

void TemplateTable::athrow() {
  transition(atos, vtos);
  null_check(eax);
  __ jmp(Interpreter::throw_exception_entry(), relocInfo::none);
}


//----------------------------------------------------------------------------------------------------
// Synchronization
//
// Note: monitorenter & exit are symmetric routines; which is reflected
//       in the assembly code structure as well
//
// Stack layout:
//
// [expressions  ] <--- esp               = expression stack top
// ..
// [expressions  ]
// [monitor entry] <--- monitor block top = expression stack bot
// ..
// [monitor entry]
// [frame data   ] <--- monitor block bot
// ...
// [saved ebp    ] <--- ebp


void TemplateTable::monitorenter() {
  transition(atos, vtos);

  // check for NULL object
  null_check(eax);

  const Address monitor_block_top(ebp, frame::interpreter_frame_monitor_block_top_offset * wordSize);
  const Address monitor_block_bot(ebp, frame::interpreter_frame_initial_sp_offset        * wordSize);
  const int entry_size =         (     frame::interpreter_frame_monitor_size()           * wordSize);
  Label allocated;

  // initialize entry pointer
  __ xorl(edx, edx);                             // points to free slot or NULL

  // find a free slot in the monitor block (result in edx)
  { Label entry, loop, exit;
    __ movl(ecx, monitor_block_top);             // points to current entry, starting with top-most entry
    __ leal(ebx, monitor_block_bot);             // points to word before bottom of monitor block
    __ jmp(entry);

    __ bind(loop);
    __ cmpl(Address(ecx, BasicObjectLock::obj_offset_in_bytes()), (int)NULL);  // check if current entry is used

// TODO - need new func here - kbt
    if (os::supports_cmov()) {
      __ cmovl(Assembler::equal, edx, ecx);      // if not used then remember entry in edx
    } else {
      Label L;
      __ jcc(Assembler::notEqual, L);
      __ movl(edx, ecx);                         // if not used then remember entry in edx
      __ bind(L);
    }
    __ cmpl(eax, Address(ecx, BasicObjectLock::obj_offset_in_bytes()));   // check if current entry is for same object
    __ jcc(Assembler::equal, exit);              // if same object then stop searching
    __ addl(ecx, entry_size);                    // otherwise advance to next entry
    __ bind(entry);
    __ cmpl(ecx, ebx);                           // check if bottom reached
    __ jcc(Assembler::notEqual, loop);           // if not at bottom then check this entry
    __ bind(exit);
  }

  __ testl(edx, edx);                            // check if a slot has been found
  __ jcc(Assembler::notZero, allocated);         // if found, continue with that one

  // allocate one if there's no free slot
  { Label entry, loop;
    // 1. compute new pointers                   // esp: old expression stack top
    __ movl(edx, monitor_block_bot);             // edx: old expression stack bottom
    __ subl(esp, entry_size);                    // move expression stack top
    __ subl(edx, entry_size);                    // move expression stack bottom
    __ movl(ecx, esp);                           // set start value for copy loop
    __ movl(monitor_block_bot, edx);             // set new monitor block top
    __ jmp(entry);
    // 2. move expression stack contents
    __ bind(loop);
    __ movl(ebx, Address(ecx, entry_size));      // load expression stack word from old location
    __ movl(Address(ecx), ebx);                  // and store it at new location
    __ addl(ecx, wordSize);                      // advance to next word
    __ bind(entry);
    __ cmpl(ecx, edx);                           // check if bottom reached
    __ jcc(Assembler::notEqual, loop);           // if not at bottom then copy next word
  }
  
  // call run-time routine
  // edx: points to monitor entry
  __ bind(allocated);

  // Increment bcp to point to the next bytecode, so exception handling for async. exceptions work correctly. 
  // The object has already been poped from the stack, so the expression stack looks correct.
  __ incl(esi);

  __ movl(Address(edx, BasicObjectLock::obj_offset_in_bytes()), eax);     // store object  
  __ lock_object(edx);  

  // The bcp has already been incremented. Just need to dispatch to next instruction.
  __ dispatch_next(vtos);
}


void TemplateTable::monitorexit() {
  transition(atos, vtos);

  // check for NULL object
  null_check(eax);

  const Address monitor_block_top(ebp, frame::interpreter_frame_monitor_block_top_offset * wordSize);
  const Address monitor_block_bot(ebp, frame::interpreter_frame_initial_sp_offset        * wordSize);
  const int entry_size =         (     frame::interpreter_frame_monitor_size()           * wordSize);
  Label found;

  // find matching slot
  { Label entry, loop;
    __ movl(edx, monitor_block_top);             // points to current entry, starting with top-most entry
    __ leal(ebx, monitor_block_bot);             // points to word before bottom of monitor block
    __ jmp(entry);

    __ bind(loop);
    __ cmpl(eax, Address(edx, BasicObjectLock::obj_offset_in_bytes()));   // check if current entry is for same object
    __ jcc(Assembler::equal, found);             // if same object then stop searching
    __ addl(edx, entry_size);                    // otherwise advance to next entry
    __ bind(entry);
    __ cmpl(edx, ebx);                           // check if bottom reached
    __ jcc(Assembler::notEqual, loop);           // if not at bottom then check this entry
  }

  // error handling. Unlocking was not block-structured
  Label end;
  __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
  __ should_not_reach_here();

  // call run-time routine
  // ecx: points to monitor entry
  __ bind(found);
  __ pushl(eax);                                 // make sure object is on stack (contract with oopMaps)  
  __ unlock_object(edx);    
  __ popl(eax);                                  // discard object  
  __ bind(end);
}


//----------------------------------------------------------------------------------------------------
// Wide instructions

void TemplateTable::wide() {
  transition(vtos, vtos);
  __ load_unsigned_byte(ebx, at_bcp(1));
  __ jmp(Address(noreg, ebx, Address::times_4, int(Interpreter::_wentry_point)));
  // Note: the esi increment step is part of the individual wide bytecode implementations
}


//----------------------------------------------------------------------------------------------------
// Multi arrays

void TemplateTable::multianewarray() {
  transition(vtos, atos);
  __ load_unsigned_byte(eax, at_bcp(3)); // get number of dimensions
  // last dim is on top of stack; we want address of first one:
  // first_addr = last_addr + (ndims - 1) * wordSize
  __ leal(  eax, Address(esp, eax, Address::times_4, -wordSize));
  call_VM(eax, CAST_FROM_FN_PTR(address, InterpreterRuntime::multianewarray), eax);     // pass in eax
  __ load_unsigned_byte(ebx, at_bcp(3));
  __ leal(esp, Address(esp, ebx, Address::times_4));  // get rid of counts
}
