/*
 * @(#)assembler_sparc.cpp	1.120 00/07/04 23:28:25
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_assembler_sparc.cpp.incl"

# include <sys/trap.h> // for trap numbers

// Implementation of Address

Address::Address( addr_type t, int which ) {
  switch (t) {
   case extra_in_argument:
   case extra_out_argument:
     _base = t == extra_in_argument ? FP : SP;
     _hi   = 0;
     _disp = (which - Argument::n_register_parameters + frame::memory_parameter_word_sp_offset) * BytesPerWord;
    break;
   default:
    ShouldNotReachHere();
    break;
  }
}
   
// Implementation of Register
const int AbstractRegister::nofRegisters = Register::nofRegisters;

const char* registerNames[Register::nofRegisters] = {
  "G0", "G1", "G2", "G3", "G4", "G5", "G6", "G7",
  "O0", "O1", "O2", "O3", "O4", "O5", "SP", "O7",
  "L0", "L1", "L2", "L3", "L4", "L5", "L6", "L7",
  "I0", "I1", "I2", "I3", "I4", "I5", "FP", "I7" };

  // note: above 32, the odd registers do not exist
const char* floatRegisterNames[FloatRegister::nofRegisters] = {
  "F0",  "F1",   "F2",  "F3",   "F4",  "F5",   "F6",  "F7",   "F8",  "F9",
  "F10", "F11",  "F12", "F13",  "F14", "F15",  "F16", "F17",  "F18", "F19",
  "F20", "F21",  "F22", "F23",  "F24", "F25",  "F26", "F27",  "F28", "F29",
  "F30", "F31",  "F32", "F33?", "F34", "F35?", "F36", "F37?", "F38", "F39?",
  "F40", "F41?", "F42", "F43?", "F44", "F45?", "F46", "F47?", "F48", "F49?",
  "F50", "F51?", "F52", "F53?", "F54", "F55?", "F56", "F57?", "F58", "F59?",
  "F60", "F61?", "F62"};
  
  
const char* FloatRegister::name() const { 
  return floatRegisterNames[_number]; 
}


static const char* argumentNames[][2] = {
  {"A0","P0"}, {"A1","P1"}, {"A2","P2"}, {"A3","P3"}, {"A4","P4"},
  {"A5","P5"}, {"A6","P6"}, {"A7","P7"}, {"A8","P8"}, {"A9","P9"},
  {"A(n>9)","P(n>9)"}
};

const char* Argument::name() const {
  int nofArgs = sizeof argumentNames / sizeof argumentNames[0];
  int num = number();
  if (num >= nofArgs)  num = nofArgs - 1;
  return argumentNames[num][is_in() ? 1 : 0];
}




// implementation of Displacement

void Assembler::print_instruction(int inst) {
  const char* s;
  switch (inv_op(inst)) {
  default:         s = "????"; break;
  case call_op:    s = "call"; break;
  case branch_op: 
    switch (inv_op2(inst)) {
      case bpr_op2:    s = "bpr";  break;
      case fb_op2:     s = "fb";   break;
      case fbp_op2:    s = "fbp";  break;
      case br_op2:     s = "br";   break;
      case bp_op2:     s = "bp";   break;
      case cb_op2:     s = "cb";   break;
      default:         s = "????"; break;
    }
  }
  ::tty->print("%s", s);
}

     
// Patch instruction inst at offset inst_pos to refer to dest_pos
// and return the resulting instruction.
// We should have pcs, not offsets, but since all is relative, it will work out
// OK.
int Assembler::patched_branch(int dest_pos, int inst, int inst_pos) {

  int m; // mask for displacement field
  int v; // new value for displacement field
  const int word_aligned_ones = -4;
  switch (inv_op(inst)) {
  default: ShouldNotReachHere();
  case call_op:    m = wdisp(word_aligned_ones, 0, 30);  v = wdisp(dest_pos, inst_pos, 30); break;
  case branch_op:
    switch (inv_op2(inst)) {
      case bpr_op2:    m = wdisp16(word_aligned_ones, 0);      v = wdisp16(dest_pos, inst_pos);     break;
      case fbp_op2:    m = wdisp(  word_aligned_ones, 0, 19);  v = wdisp(  dest_pos, inst_pos, 19); break;
      case bp_op2:     m = wdisp(  word_aligned_ones, 0, 19);  v = wdisp(  dest_pos, inst_pos, 19); break;
      case fb_op2:     m = wdisp(  word_aligned_ones, 0, 22);  v = wdisp(  dest_pos, inst_pos, 22); break;
      case br_op2:     m = wdisp(  word_aligned_ones, 0, 22);  v = wdisp(  dest_pos, inst_pos, 22); break;
      case cb_op2:     m = wdisp(  word_aligned_ones, 0, 22);  v = wdisp(  dest_pos, inst_pos, 22); break;
      default: ShouldNotReachHere();
    }
  }
  return  inst & ~m  |  v;
}


// Return the offset of the branch destionation of instruction inst
// at offset pos.
// Should have pcs, but since all is relative, it works out.
int Assembler::branch_destination(int inst, int pos) {
  int r;
  switch (inv_op(inst)) {
  default: ShouldNotReachHere();
  case call_op:        r = inv_wdisp(inst, pos, 30);  break;
  case branch_op:
    switch (inv_op2(inst)) {
      case bpr_op2:    r = inv_wdisp16(inst, pos);    break;
      case fbp_op2:    r = inv_wdisp(  inst, pos, 19);  break;
      case bp_op2:     r = inv_wdisp(  inst, pos, 19);  break;
      case fb_op2:     r = inv_wdisp(  inst, pos, 22);  break;
      case br_op2:     r = inv_wdisp(  inst, pos, 22);  break;
      case cb_op2:     r = inv_wdisp(  inst, pos, 22);  break;
      default: ShouldNotReachHere();
    }
  }
  return r;
}


int AbstractAssembler::code_fill_byte() {
  return 0x00;			// illegal instruction 0x00000000
}


void Assembler::fnsave(Address dst) {
  // %%%%%%%%% not implemented yet
}


void Assembler::frstor(Address src) {
  // %%%%%%%%% not implemented yet
}

// Generate a bunch 'o stuff (including v9's
#ifndef PRODUCT
void Assembler::test_v9() {
  add(    G0, G1, G2 );
  add(    G3,  0, G4 );
  
  addcc(  G5, G6, G7 );
  addcc(  I0,  1, I1 );
  addc(   I2, I3, I4 );
  addc(   I5, -1, I6 );
  addccc( I7, L0, L1 );
  addccc( L2, (1 << 12) - 2, L3 );

  Label lbl1, lbl2, lbl3;

  bind(lbl1);
  
  bpr( rc_z,    true, pn, L4, pc(),  relocInfo::oop_type );
  delayed()->nop();
  bpr( rc_lez, false, pt, L5, lbl1);
  delayed()->nop();
      
  fb( f_never,     true, pc() + 4,  relocInfo::none);
  delayed()->nop();
  fb( f_notEqual, false, lbl2 );
  delayed()->nop();
  
  fbp( f_notZero,        true, fcc0, pn, pc() - 4,  relocInfo::none);
  delayed()->nop();
  fbp( f_lessOrGreater, false, fcc1, pt, lbl3 );
  delayed()->nop();
  
  br( equal,  true, pc() + 1024, relocInfo::none);
  delayed()->nop();
  br( lessEqual, false, lbl1 );
  delayed()->nop();
  br( never, false, lbl1 );
  delayed()->nop();
    
  bp( less,               true, icc, pn, pc(), relocInfo::none);
  delayed()->nop();
  bp( lessEqualUnsigned, false, xcc, pt, lbl2 );
  delayed()->nop();
  
  call( pc(), relocInfo::none);
  delayed()->nop();
  call( lbl3 );
  delayed()->nop();

 
  casa(  L6, L7, O0 );
  casxa( O1, O2, O3, 0 );
 
  udiv(   O4, O5, O7 );
  udiv(   G0, (1 << 12) - 1, G1 );
  sdiv(   G1, G2, G3 );
  sdiv(   G4, -((1 << 12) - 1), G5 );
  udivcc( G6, G7, I0 );
  udivcc( I1, -((1 << 12) - 2), I2 );
  sdivcc( I3, I4, I5 );
  sdivcc( I6, -((1 << 12) - 0), I7 );
  
  done();
  retry();

  fadd( FloatRegister::S, F0,  F1, F2 );
  fsub( FloatRegister::D, F34, F0, F62 );

  fcmp(  FloatRegister::Q, fcc0, F0, F60);
  fcmpe( FloatRegister::S, fcc1, F31, F30);
  
  ftox( FloatRegister::D, F2, F4 );
  ftoi( FloatRegister::Q, F4, F8 );
  
  ftof( FloatRegister::S, FloatRegister::Q, F3, F12 );
  
  fxtof( FloatRegister::S, F4, F5 );
  fitof( FloatRegister::D, F6, F8 );
  
  fmov( FloatRegister::Q, F16, F20 );
  fneg( FloatRegister::S, F6, F7 );
  fabs( FloatRegister::D, F10, F12 );

  fmul( FloatRegister::Q,  F24, F28, F32 );
  fmul( FloatRegister::S,  FloatRegister::D,  F8, F9, F14 );
  fdiv( FloatRegister::S,  F10, F11, F12 );
  
  fsqrt( FloatRegister::S, F13, F14 );
  
  flush( L0, L1 );
  flush( L2, -1 );

  flushw();
  
  illtrap( (1 << 22) - 2);
   
  impdep1( 17, (1 << 19) - 1 );
  impdep2( 3,  0 );
  
  jmpl( L3, L4, L5 );
  delayed()->nop();
  jmpl( L6, -1, L7, Relocation::spec_simple(relocInfo::none));
  delayed()->nop();


  ldf(    FloatRegister::S, O0, O1, F15 );
  ldf(    FloatRegister::D, O2, -1, F14 );


  ldfsr(  O3, O4 );
  ldfsr(  O5, -1 );
  ldxfsr( O6, O7 );
  ldxfsr( I0, -1 );

  ldfa(  FloatRegister::D, I1, I2, 1, F16 );
  ldfa(  FloatRegister::Q, I3, -1,    F36 );

  ldsb(  I4, I5, I6 );
  ldsb(  I7, -1, G0 );
  ldsh(  G1, G3, G4 );
  ldsh(  G5, -1, G6 );
  ldsw(  G7, L0, L1 );
  ldsw(  L2, -1, L3 );
  ldub(  L4, L5, L6 );
  ldub(  L7, -1, O0 );
  lduh(  O1, O2, O3 );
  lduh(  O4, -1, O5 );
  lduw(  O6, O7, G0 );
  lduw(  G1, -1, G2 );
  ldx(   G3, G4, G5 );
  ldx(   G6, -1, G7 );
  ldd(   I0, I1, I2 );
  ldd(   I3, -1, I4 );
  
  ldsba(  I5, I6, 2, I7 );
  ldsba(  L0, -1, L1 );
  ldsha(  L2, L3, 3, L4 );
  ldsha(  L5, -1, L6 );
  ldswa(  L7, O0, (1 << 8) - 1, O1 );
  ldswa(  O2, -1, O3 );
  lduba(  O4, O5, 0, O6 );
  lduba(  O7, -1, I0 );
  lduha(  I1, I2, 1, I3 );
  lduha(  I4, -1, I5 );
  lduwa(  I6, I7, 2, L0 );
  lduwa(  L1, -1, L2 );
  ldxa(   L3, L4, 3, L5 );
  ldxa(   L6, -1, L7 );
  ldda(   G0, G1, 4, G2 );
  ldda(   G3, -1, G4 );
  
  ldstub(  G5, G6, G7 );
  ldstub(  O0, -1, O1 );

  ldstuba( O2, O3, 5, O4 );
  ldstuba( O5, -1, O6 );

  and3(    I0, L0, O0 );
  and3(    G7, -1, O7 );
  andcc(   L2, I2, G2 );
  andcc(   L4, -1, G4 );
  andn(    I5, I6, I7 );
  andn(    I6, -1, I7 );
  andncc(  I5, I6, I7 );
  andncc(  I7, -1, I6 );
  or3(     I5, I6, I7 );
  or3(     I7, -1, I6 );
  orcc(    I5, I6, I7 );
  orcc(    I7, -1, I6 );
  orn(     I5, I6, I7 );
  orn(     I7, -1, I6 );
  orncc(   I5, I6, I7 );
  orncc(   I7, -1, I6 );
  xor3(    I5, I6, I7 );
  xor3(    I7, -1, I6 );
  xorcc(   I5, I6, I7 );
  xorcc(   I7, -1, I6 );
  xnor(    I5, I6, I7 );
  xnor(    I7, -1, I6 );
  xnorcc(  I5, I6, I7 );
  xnorcc(  I7, -1, I6 );
  
  membar( Membar_mask_bits(StoreStore | LoadStore | StoreLoad | LoadLoad | Sync | MemIssue | Lookaside ) );
  membar( StoreStore );
  membar( LoadStore );
  membar( StoreLoad );
  membar( LoadLoad );
  membar( Sync );
  membar( MemIssue );
  membar( Lookaside );
 
  fmov( FloatRegister::S, f_ordered,  true, fcc2, F16, F17 );    
  fmov( FloatRegister::D, rc_lz, L5, F18, F20 );

  movcc( overflowClear,  false, icc, I6, L4 );
  movcc( f_unorderedOrEqual, true, fcc2, (1 << 10) - 1, O0 );
 
  movr( rc_nz, I5, I6, I7 );
  movr( rc_gz, L1, -1,  L2 );
  
  mulx(  I5, I6, I7 );
  mulx(  I7, -1, I6 );
  sdivx( I5, I6, I7 );
  sdivx( I7, -1, I6 );
  udivx( I5, I6, I7 );
  udivx( I7, -1, I6 );

  umul(   I5, I6, I7 );
  umul(   I7, -1, I6 );
  smul(   I5, I6, I7 );
  smul(   I7, -1, I6 );
  umulcc( I5, I6, I7 );
  umulcc( I7, -1, I6 );
  smulcc( I5, I6, I7 );
  smulcc( I7, -1, I6 );

  mulscc(   I5, I6, I7 );
  mulscc(   I7, -1, I6 );

  nop();
  
  
  popc( G0,  G1);
  popc( -1, G2);
  
  prefetch(   L1, L2,    severalReads );
  prefetch(   L3, -1,    oneRead );
  prefetcha(  O3, O2, 6, severalWritesAndPossiblyReads );
  prefetcha(  G2, -1,    oneWrite );
  
  rett( I7, I7);
  delayed()->nop();
  rett( G0, -1, relocInfo::none);
  delayed()->nop();
  
  save(    I5, I6, I7 );
  save(    I7, -1, I6 );
  restore( I5, I6, I7 );
  restore( I7, -1, I6 );
 
  saved();
  restored();
  
  sethi( 0xaaaaaaaa, I3, Relocation::spec_simple(relocInfo::none));
  
  sll(  I5, I6, I7 );
  sll(  I7, 31, I6 );
  srl(  I5, I6, I7 );
  srl(  I7,  0, I6 );
  sra(  I5, I6, I7 );
  sra(  I7, 30, I6 );
  sllx( I5, I6, I7 );
  sllx( I7, 63, I6 );
  srlx( I5, I6, I7 );
  srlx( I7,  0, I6 );
  srax( I5, I6, I7 );
  srax( I7, 62, I6 );
  
  sir( -1 );
  
  stbar();
  
  stf(    FloatRegister::Q, F40, G0, I7 );
  stf(    FloatRegister::S, F18, I3, -1 );

  stfsr(  L1, L2 );
  stfsr(  I7, -1 );
  stxfsr( I6, I5 );
  stxfsr( L4, -1 );

  stfa(  FloatRegister::D, F22, I6, I7, 7 );
  stfa(  FloatRegister::Q, F44, G0, -1 );

  stb(  L5, O2, I7 );
  stb(  I7, I6, -1 );
  sth(  L5, O2, I7 );
  sth(  I7, I6, -1 );
  stw(  L5, O2, I7 );
  stw(  I7, I6, -1 );
  stx(  L5, O2, I7 );
  stx(  I7, I6, -1 );
  std(  L5, O2, I7 );
  std(  I7, I6, -1 );

  stba(  L5, O2, I7, 8 );
  stba(  I7, I6, -1    );
  stha(  L5, O2, I7, 9 );
  stha(  I7, I6, -1    );
  stwa(  L5, O2, I7, 0 );
  stwa(  I7, I6, -1    );
  stxa(  L5, O2, I7, 11 );
  stxa(  I7, I6, -1     );
  stda(  L5, O2, I7, 12 );
  stda(  I7, I6, -1     );

  sub(    I5, I6, I7 );
  sub(    I7, -1, I6 );
  subcc(  I5, I6, I7 );
  subcc(  I7, -1, I6 );
  subc(   I5, I6, I7 );
  subc(   I7, -1, I6 );
  subccc( I5, I6, I7 );
  subccc( I7, -1, I6 );
  
  swap( I5, I6, I7 );
  swap( I7, -1, I6 );

  swapa(   G0, G1, 13, G2 );
  swapa(   I7, -1,     I6 );

  taddcc(    I5, I6, I7 );
  taddcc(    I7, -1, I6 );
  taddcctv(  I5, I6, I7 );
  taddcctv(  I7, -1, I6 );

  tsubcc(    I5, I6, I7 );
  tsubcc(    I7, -1, I6 );
  tsubcctv(  I5, I6, I7 );
  tsubcctv(  I7, -1, I6 );

  trap( overflowClear, xcc, G0, G1 );
  trap( lessEqual,     icc, I7, 17 );

  bind(lbl2);
  bind(lbl3);

  code()->decode();
}

// Generate a bunch 'o stuff unique to V8
void Assembler::test_v8_onlys() {
  Label lbl1;

  cb( cp_0or1or2, false, pc() - 4, relocInfo::none);
  delayed()->nop();
  cb( cp_never,    true, lbl1);
  delayed()->nop();

  cpop1(1, 2, 3, 4);
  cpop2(5, 6, 7, 8);

  ldc( I0, I1, 31);
  ldc( I2, -1,  0);

  lddc( I4, I4, 30);
  lddc( I6,  0, 1 );

  ldcsr( L0, L1, 0);
  ldcsr( L1, (1 << 12) - 1, 17 );

  stc( 31, L4, L5);
  stc( 30, L6, -(1 << 12) );

  stdc( 0, L7, G0);
  stdc( 1, G1, 0 );

  stcsr( 16, G2, G3);
  stcsr( 17, G4, 1 );

  stdcq( 4, G5, G6);
  stdcq( 5, G7, -1 );

  bind(lbl1);

  code()->decode();
}
#endif

// Implementation of MacroAssembler


void MacroAssembler::null_check(Register reg, int offset) {
  if (needs_explicit_null_check(offset)) {
    // provoke OS NULL exception if reg = NULL by
    // accessing M[reg] w/o changing any registers
    ld(reg, 0, G0);
  }
  else {
    // nothing to do, (later) access of M[reg + offset]
    // will provoke OS NULL exception if reg = NULL
  }
}


bool MacroAssembler::needs_explicit_null_check(int offset) {
  // The first page of virtual addresses is unmapped on SPARC.
  // Thus, any access the VM makes through a null pointer with an offset of
  // less than 4K will get a recognizable SIGSEGV, which the signal handler
  // will transform into a NullPointerException.
  // (Actually, the first 64K or so is unmapped, but it's simpler
  // to depend only on the first 4K or so.)

  bool offset_in_first_page =   0 <= offset  &&  offset < os::vm_page_size();
  return !offset_in_first_page;
}



// convert to C varargs format

void MacroAssembler::set_varargs( Argument inArg, Register d ) {
  // spill register-resident args to their memory slots
  // (SPARC calling convention requires callers to have already preallocated these)
  // Note that the inArg might in fact be an outgoing argument,
  // if a leaf routine or stub does some tricky argument shuffling.
  // This routine must work even though one of the saved arguments
  // is in the d register (e.g., set_varargs(Argument(0, false), O0)).
  for (Argument savePtr = inArg;
       savePtr.is_register();
       savePtr = savePtr.successor()) {
    st(savePtr.as_register(), savePtr.address_in_frame());
  }
  // return the address of the first memory slot
  add(inArg.address_in_frame(), d);
}

// conditional breakpoint (for assertion checks in assembly code)
void MacroAssembler::breakpoint_trap(Condition c, CC cc) {
  trap( c, cc, G0, ST_RESERVED_FOR_USER_0 );
}

// We want to use ST_BREAKPOINT here, but the debugger is confused by it.
void MacroAssembler::breakpoint_trap()    { trap( ST_RESERVED_FOR_USER_0 ); }
void MacroAssembler::flush_windows_trap() { trap( ST_FLUSH_WINDOWS ); }
void MacroAssembler::clean_windows_trap() { trap( ST_CLEAN_WINDOWS ); }
void MacroAssembler::get_psr_trap()       { trap( ST_GETPSR ); }
void MacroAssembler::set_psr_trap()	  { trap( ST_SETPSR ); }

// flush windows (except current) using flushw instruction if avail.
void MacroAssembler::flush_windows() {
  if (VM_Version::v9_instructions_work())  flushw();
  else                                     flush_windows_trap();
}


void MacroAssembler::enter() {
  Unimplemented();
}


void MacroAssembler::leave() {
  Unimplemented();
}


// Calls to C land

#ifdef ASSERT
// a hook for debugging
static Thread* reinitialize_thread() {
  return ThreadLocalStorage::thread();
}
#else
#define reinitialize_thread ThreadLocalStorage::thread
#endif

#ifdef ASSERT
address last_get_thread = NULL;
#endif

// call this when Gthread is not known to be valid
void MacroAssembler::get_thread() {
  save_frame(0); // to avoid clobbering O0
  mov(Garg0, L0); // avoid clobbering Garg0
  mov(Garg1, L1); // avoid clobbering Garg1
  mov(G3, L2); // avoid clobbering G3 also
#ifdef ASSERT
  Address last_get_thread_addr(L3, (address)&last_get_thread);
  get_pc(L4);
  inc(L4, 3 * wordSize);  // skip this & 2 more insns to point at call
  sethi(last_get_thread_addr);
  st(L4, last_get_thread_addr);
#endif
  call(CAST_FROM_FN_PTR(address, reinitialize_thread), relocInfo::runtime_call_type);
  delayed()->nop();
  mov(L0, Garg0);
  mov(L1, Garg1);
  mov(L2, G3);
  restore(O0, 0, Gthread);
}

static Thread* verify_thread_subroutine(Thread* gthread_value) {
  Thread* correct_value = ThreadLocalStorage::thread();
  guarantee(gthread_value == correct_value, "Gthread value must be the thread");
  return correct_value;
}

void MacroAssembler::verify_thread() {
  if (VerifyThread) {
    // make sure Gthread contains the right value
    save_frame_and_mov(0, Lmethod, Lmethod);   // to avoid clobbering O0 (and propagate Lmethod for -Xprof)
    mov(Garg0, L0); // avoid clobbering Garg0
    mov(Garg1, L1); // avoid clobbering Garg1
    call(CAST_FROM_FN_PTR(address, verify_thread_subroutine), relocInfo::runtime_call_type);
    delayed()->mov(Gthread, O0);
    mov(L0, Garg0);
    mov(L1, Garg1);
    restore(O0, 0, Gthread);
  }
}


void MacroAssembler::save_thread(const Register thread_cache) {
  verify_thread();
  if (thread_cache.isValid()) {
    assert(thread_cache.is_local() || thread_cache.is_in(), "bad volatile");
    mov(Gthread, thread_cache);
  }
  if (VerifyThread) {
    // smash Gthread, as if the VM were about to anyway
    set(0x67676767, Gthread);
  }
}


void MacroAssembler::restore_thread(const Register thread_cache) {
  if (thread_cache.isValid()) {
    assert(thread_cache.is_local() || thread_cache.is_in(), "bad volatile");
    mov(thread_cache, Gthread);
    verify_thread();
  } else {
    // do it the slow way
    get_thread();
  }
}



#ifndef CORE
#ifndef PRODUCT

//
// calling trace_method_entry/exit with call_VM implies some default register usage:
//
// call_VM(trace_method_entry/exit,arg0,arg1) modifies registers { G1, G5, O0, O1, O2 }.
// 
// However, G1, G5, and O0-7 are all live when trace_method_entry/exit gets called 
// 
// In order to preserve live values that might be modified by call_VM, 
// we wrap the call_VM with a save_frame/restore which maps:
//
// O0-O7 ==> I0-I7
// L0-L7 ==> previous reg. window
// I0-I7 ==> previous reg. window
//
// In order to preserve G1 & G5, we first copy them to L2 & L3.
// After call_VM, we copy the stored "live" values back to G1 & G5 and continue.
//
// When trace_method_entry/exit is called from the interpreter, the register method_oop
// holds the methodOop to pass to SharedRuntime::trace_method_entry/exit. Since the 
// interpreter doesn't use register O5, we use it to pass the methodOop value between
// register windows.
//
// When trace_method_entry/exit is called from compiled code, method_oop == G0.
//
void MacroAssembler::trace_method_entry(const Register method_oop) {
 if (TraceMethods)
  {    
    if (method_oop != G0)    // *** entrance from interpreter
      {
	mov(method_oop, O5); // ok to trash O5, only O0-1 are live in interpreter
	save_frame(0);       // save topmost register window
	mov(I5,L0);	     // pass method_oop as first parameter to trace_method_entry
      }
    else                     // *** entrance from compiled code
      {	
	save_frame(0);       // save topmost register window
	mov(G0,L0);          // pass 0 as first parameter to trace_method_entry
      }

    rdpc(L1);            // pass (pc) as second parameter to trace_method_entry
    mov(G1,L2);          // G1 is live, save it
    mov(G5,L3);          // G5 is live, save it
    call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::trace_method_entry), L0, L1);
    mov(L2, G1);         // G1 is live, restore it
    mov(L3, G5);         // G5 is live, restore it
    restore();           // restore registers
  }
}

void MacroAssembler::trace_method_exit(const Register method_oop) {    
 if (TraceMethods)
  {
    if (method_oop != G0)    // *** entrance from interpreter
      {
	mov(method_oop, O5); // ok to trash O5, only O0-1 are live in interpreter
	save_frame(0);       // save topmost register window
	mov(I5,L0);	     // pass method_oop as first parameter to trace_method_entry
      }
    else                     // *** entrance from compiled code
      {	
	save_frame(0);       // save topmost register window
	mov(G0,L0);          // pass 0 as first parameter to trace_method_entry
      }

    rdpc(L1);            // pass (pc) as second parameter to trace_method_entry
    mov(G1,L2);          // G1 is live, save it
    mov(G5,L3);          // G5 is live, save it
    call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::trace_method_exit), L0, L1);
    mov(L2, G1);         // G1 is live, restore it
    mov(L3, G5);         // G5 is live, restore it
    restore();           // restore registers
  }
}

#endif
#endif




// %%% maybe get rid of [re]set_last_Java_frame
void MacroAssembler::set_last_Java_frame(Register last_java_sp, Register post_java_state) {
  assert_not_delayed();
  st(last_java_sp,    Address(Gthread, 0, JavaThread::last_Java_sp_offset_in_bytes()));
  st(post_java_state, Address(Gthread, 0, JavaThread::post_Java_state_offset_in_bytes()));
}

void MacroAssembler::reset_last_Java_frame() {
  assert_not_delayed();

  Address sp_addr(Gthread, 0, JavaThread::last_Java_sp_offset_in_bytes());
  Address pj_addr(Gthread, 0, JavaThread::post_Java_state_offset_in_bytes());
  
# ifdef ASSERT
  // check that it WAS previously set
    save_frame_and_mov(0, Lmethod, Lmethod);     // Propagate Lmethod to helper frame for -Xprof
    ld(sp_addr, L0);
    tst(L0);
    breakpoint_trap(Assembler::zero);
    restore();
# endif
  
  st(G0, sp_addr);
  st(G0, pj_addr);
}


void MacroAssembler::call_VM_base(
  Register        oop_result,
  Register        thread_cache,
  Register        last_java_sp,
  address         entry_point,
  int             number_of_arguments,
  bool            check_exceptions)
{
  assert_not_delayed();

  // determine last_java_sp register
  if (!last_java_sp.isValid()) {
    last_java_sp = SP;
  }
  // debugging support
  assert(number_of_arguments >= 0   , "cannot have negative number of arguments");

  set_last_Java_frame(last_java_sp, G0);
  if (VerifyThread)  mov(Gthread, O0); // about to be smashed; pass early
  save_thread(thread_cache);
  // do the call
  call(entry_point, relocInfo::runtime_call_type);
  if (!VerifyThread)
    delayed()->mov(Gthread, O0);  // pass thread as first argument
  else
    delayed()->nop();             // (thread already passed)
  restore_thread(thread_cache);
  reset_last_Java_frame();

  // check for pending exceptions
  if (check_exceptions) {
    Label L;

    Address exception_addr(Gthread,0,Thread::pending_exception_offset_in_bytes());
    ld(exception_addr, Gtemp);
    tst(Gtemp);
    br(equal, false, pt, L);
    delayed()->nop();
    // we use O7 linkage so that forward_exception_entry has the issuing PC
    call(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);
    delayed()->nop();
    bind(L);
  }

  // get oop result if there is one and reset the value in the thread
  if (oop_result.isValid()) {                    // get oop result if there is one and reset it in the thread
    get_vm_result(oop_result);
  }
}


void MacroAssembler::call_VM(Register oop_result, address entry_point, int number_of_arguments) {
  call_VM_base(oop_result, noreg, noreg, entry_point, number_of_arguments, Assembler::_check_for_exceptions_flag);
}


void MacroAssembler::call_VM(Register oop_result, address entry_point, Register arg_1) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  call_VM(oop_result, entry_point, 1);
}


void MacroAssembler::call_VM(Register oop_result, address entry_point, Register arg_1, Register arg_2) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  mov(arg_2, O2); assert(arg_2 != O1, "smashed argument");
  call_VM(oop_result, entry_point, 2);
}


void MacroAssembler::call_VM(Register oop_result, address entry_point, Register arg_1, Register arg_2, Register arg_3) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  mov(arg_2, O2); assert(arg_2 != O1,                "smashed argument");
  mov(arg_3, O3); assert(arg_3 != O1 && arg_3 != O2, "smashed argument");
  call_VM(oop_result, entry_point, 3);
}



// Note: The following call_VM overloadings are useful when a "save"
// has already been performed by a stub, and the last Java frame is
// the previous one.  In that case, last_java_sp must be passed as FP
// instead of SP.


void MacroAssembler::call_VM(Register oop_result, Register last_java_sp, address entry_point, int number_of_arguments) {
  call_VM_base(oop_result, noreg, last_java_sp, entry_point, number_of_arguments, Assembler::_check_for_exceptions_flag);
}


void MacroAssembler::call_VM(Register oop_result, Register last_java_sp, address entry_point, Register arg_1) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  call_VM(oop_result, last_java_sp, entry_point, 1);
}


void MacroAssembler::call_VM(Register oop_result, Register last_java_sp, address entry_point, Register arg_1, Register arg_2) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  mov(arg_2, O2); assert(arg_2 != O1, "smashed argument");
  call_VM(oop_result, last_java_sp, entry_point, 2);
}


void MacroAssembler::call_VM(Register oop_result, Register last_java_sp, address entry_point, Register arg_1, Register arg_2, Register arg_3) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  mov(arg_2, O2); assert(arg_2 != O1,                "smashed argument");
  mov(arg_3, O3); assert(arg_3 != O1 && arg_3 != O2, "smashed argument");
  call_VM(oop_result, last_java_sp, entry_point, 3);
}



void MacroAssembler::call_VM_leaf_base(Register thread_cache, address entry_point, int number_of_arguments) {
  assert_not_delayed();
  save_thread(thread_cache);
  // do the call
  call(entry_point, relocInfo::runtime_call_type);
  delayed()->nop();
  restore_thread(thread_cache);
}


void MacroAssembler::call_VM_leaf(Register thread_cache, address entry_point, int number_of_arguments) {
  call_VM_leaf_base(thread_cache, entry_point, number_of_arguments);
}


void MacroAssembler::call_VM_leaf(Register thread_cache, address entry_point, Register arg_1) {
  mov(arg_1, O0);
  call_VM_leaf(thread_cache, entry_point, 1);
}


void MacroAssembler::call_VM_leaf(Register thread_cache, address entry_point, Register arg_1, Register arg_2) {
  mov(arg_1, O0);
  mov(arg_2, O1); assert(arg_2 != O0, "smashed argument");
  call_VM_leaf(thread_cache, entry_point, 2);
}


void MacroAssembler::call_VM_leaf(Register thread_cache, address entry_point, Register arg_1, Register arg_2, Register arg_3) {
  mov(arg_1, O0);
  mov(arg_2, O1); assert(arg_2 != O0,                "smashed argument");
  mov(arg_3, O2); assert(arg_3 != O0 && arg_3 != O1, "smashed argument");
  call_VM_leaf(thread_cache, entry_point, 3);
}


void MacroAssembler::get_vm_result(Register oop_result) {
  verify_thread();
  Address vm_result_addr(Gthread, 0, JavaThread::vm_result_offset_in_bytes());
  ld(    vm_result_addr, oop_result);
  st(G0, vm_result_addr);
  verify_oop(oop_result);
}


void MacroAssembler::get_vm_result_2(Register oop_result) {
  verify_thread();
  Address vm_result_addr_2(Gthread, 0, JavaThread::vm_result_2_offset_in_bytes());
  ld(    vm_result_addr_2, oop_result);
  st(G0, vm_result_addr_2);
  verify_oop(oop_result);
}


// We require that C code which does not return a value in vm_result will
// leave it undisturbed.
void MacroAssembler::set_vm_result(Register oop_result) {
  verify_thread();
  Address vm_result_addr(Gthread, 0, JavaThread::vm_result_offset_in_bytes());
  verify_oop(oop_result);

# ifdef ASSERT
    // Check that we are not overwriting any other oop.
    save_frame_and_mov(0, Lmethod, Lmethod);     // Propagate Lmethod for -Xprof
    ld(vm_result_addr, L0);
    tst(L0);
    restore();
    breakpoint_trap(notZero);
    // }
# endif

  st(oop_result, vm_result_addr);
}


void MacroAssembler::store_check(Register tmp, Register obj) {
  // Use two shifts to clear out those low order two bits! (Cannot opt. into 1.)

  assert(sizeof(*RememberedSet::byte_map_base) == sizeof(jbyte),
	 "adjust this code");
  srl(obj, RememberedSet::card_shift, obj);
  assert( tmp != obj, "need separate temp reg");
  Address rs(tmp, (address)RememberedSet::byte_map_base);
  load_address(rs);
  stb(G0, rs.base(), obj);
}

void MacroAssembler::store_check(Register tmp, Register obj, Register offset) {
  store_check(tmp, obj);
}

void MacroAssembler::setx( jlong value, Register tmp, Register d, RelocationHolder const& rspec) {
# ifdef CHECK_DELAY
  guarantee( delay_state == no_delay,  "cannot put two instructions in delay slot" );
# endif
  assert(rspec.type() == relocInfo::none, "cannot relocate a 64-bit constant");
  v9_dep();
  int hi = (int)(value >> 32);
  int lo = (int)(value & ~0);
  if (hi == 0 && lo >= 0) {
    sethi(lo, d);
    or3(d, low10(lo), d);
  } 
  else if (hi == -1  &&  lo < 0) {
    sethi(~lo, d);
    xor3(d, low10(lo) ^ ~low10(~0), d);
  } 
  else {
    sethi(hi, tmp);
    or3(tmp, low10(hi), tmp);
    sllx(tmp, 32, tmp);
    sethi(lo, d);
    or3(d, tmp, d);
    or3(d, low10(lo), d);
  }
}

// compute size in bytes of sparc frame, given
// number of extraWords
int MacroAssembler::total_frame_size(int extraWords) {

  int nWords = frame::memory_parameter_word_sp_offset;

  nWords += extraWords;

  if (nWords & 1) ++nWords; // round up to double-word

  return nWords * BytesPerWord;
}


// save_frame: given number of "extra" words in frame,
// issue approp. save instruction (p 200, v8 manual)

void MacroAssembler::save_frame(int extraWords = 0) {
  int delta = -total_frame_size(extraWords);
  if (is_simm13(delta))
    save(SP, delta, SP);
  else {
    set(delta, Gscratch);
    save(SP, Gscratch, SP);
  }
}


void MacroAssembler::save_frame_and_mov(int extraWords,
					Register s1, Register d1,
					Register s2, Register d2) {
  assert_not_delayed();

  // The trick here is to use precisely the same memory word
  // that trap handlers also use to save the register.
  // This word cannot be used for any other purpose, but
  // it works fine to save the register's value, whether or not
  // an interrupt flushes register windows at any given moment!
  Address s1_addr;
  if (s1.isValid() && (s1.is_in() || s1.is_local())) {
    s1_addr = s1.address_in_saved_window();
    st(s1, s1_addr);
  }

  Address s2_addr;
  if (s2.isValid() && (s2.is_in() || s2.is_local())) {
    s2_addr = s2.address_in_saved_window();
    st(s2, s2_addr);
  }

  save_frame(extraWords);

  if (s1_addr.base() == SP) {
    ld(s1_addr.after_save(), d1);
  } else if (s1.isValid()) {
    mov(s1.after_save(), d1);
  }

  if (s2_addr.base() == SP) {
    ld(s2_addr.after_save(), d2);
  } else if (s2.isValid()) {
    mov(s2.after_save(), d2);
  }
}


Address MacroAssembler::allocate_oop_address(jobject obj, Register d) {
  assert(oop_recorder() != NULL, "this assembler needs an OopRecorder");
  int oop_index = oop_recorder()->allocate_index(obj);
  return Address(d, address(obj), oop_Relocation::spec(oop_index));
}


Address MacroAssembler::constant_oop_address(jobject obj, Register d) {
  assert(oop_recorder() != NULL, "this assembler needs an OopRecorder");
  int oop_index = oop_recorder()->find_index(obj);
  return Address(d, address(obj), oop_Relocation::spec(oop_index));
}


void MacroAssembler::align(int modulus) {
  while (offset() % modulus != 0) nop();
}


void MacroAssembler::safepoint() {
  relocate(breakpoint_Relocation::spec(breakpoint_Relocation::safepoint));
}


class RegistersForDebugging : public StackObj {
 public:
  jint    i[8], l[8], o[8], g[8];
  float   f[32];
  double  d[32];
  
  void print(outputStream* s) {
    int j;
    for ( j = 0;  j < 8;  ++j )
      if ( j != 6 ) s->print_cr("i%d = 0x%.8x", j, i[j]);
      else          s->print_cr( "fp = 0x%.8x",    i[j]);
    s->print_cr("");

    for ( j = 0;  j < 8;  ++j )
      s->print_cr("l%d = 0x%.8x", j, l[j]);
    s->print_cr("");

    for ( j = 0;  j < 8;  ++j )
      if ( j != 6 ) s->print_cr("o%d = 0x%.8x", j, o[j]);
      else          s->print_cr( "sp = 0x%.8x",    o[j]);
    s->print_cr("");

    for ( j = 0;  j < 8;  ++j )
      s->print_cr("g%d = 0x%.8x", j, g[j]);
    s->print_cr("");

    // print out floats with compression
    for (j = 0; j < 32; ) {
      jfloat val = f[j];
      int last = j;
      for ( ;  last+1 < 32;  ++last ) {
        char b1[1024], b2[1024];
        sprintf(b1, "%f", val);
        sprintf(b2, "%f", f[last+1]);
        if (strcmp(b1, b2))
          break;
      }
      s->print("f%d", j);
      if ( j != last )  s->print(" - f%d", last);
      s->print(" = %f", val);
      s->fill_to(25);
      s->print_cr(" (0x%x)", val);
      j = last + 1;
    }
    s->print_cr("");

    // and doubles (evens only)
    for (j = 0; j < 32; ) {
      jdouble val = d[j];
      int last = j;
      for ( ;  last+1 < 32;  ++last ) {
        char b1[1024], b2[1024];
        sprintf(b1, "%f", val);
        sprintf(b2, "%f", d[last+1]);
        if (strcmp(b1, b2))
          break;
      }
      s->print("d%d", 2 * j);
      if ( j != last )  s->print(" - d%d", last);
      s->print(" = %f", val);
      s->fill_to(30);
      s->print("(0x%x)", *(int*)&val);
      s->fill_to(42);
      s->print_cr("(0x%x)", *(1 + (int*)&val));
      j = last + 1;
    }
    s->print_cr("");
  }

  static int i_offset(int j) { return (int)&((RegistersForDebugging*)0)->i[j]; }
  static int l_offset(int j) { return (int)&((RegistersForDebugging*)0)->l[j]; }
  static int o_offset(int j) { return (int)&((RegistersForDebugging*)0)->o[j]; }
  static int g_offset(int j) { return (int)&((RegistersForDebugging*)0)->g[j]; }
  static int f_offset(int j) { return (int)&((RegistersForDebugging*)0)->f[j]; }
  static int d_offset(int j) { return (int)&((RegistersForDebugging*)0)->d[j / 2]; }

  // gen asm code to save regs
  static void save_registers(MacroAssembler* a) {
    a->sub( FP, round_to(sizeof(RegistersForDebugging), sizeof(jdouble)), O0 );
    a->flush_windows();
    int i;
    for ( i = 0; i < 8; ++i) { 
      a->ld(Register('i', i).address_in_saved_window().after_save(), L1);  a->st( L1, O0, i_offset(i));
      a->ld(Register('l', i).address_in_saved_window().after_save(), L1);  a->st( L1, O0, l_offset(i));
      a->st( Register('o', i).after_save(), O0, o_offset(i));
      a->st( Register('g', i).after_save(), O0, g_offset(i));
    }
    for ( i = 0;  i < 32; ++i)
      a->stf( FloatRegister::S, FloatRegister(i, ' '), O0, f_offset(i));

    for ( i = 0; i < (VM_Version::v9_instructions_work() ? 64 : 32); i += 2)
      a->stf( FloatRegister::D, FloatRegister(i, ' '), O0, d_offset(i));
  }

  // restore global registers in case C code disturbed them; 
  // struct is in L0
  static void restore_registers(MacroAssembler* a, Register r) {
    for (int i = 1;  i < 8;  ++i) {
      a->ld( r, g_offset(i), Register('g', i));
    }
    for ( int j = 0;  j < 32; ++j)
      a->ldf( FloatRegister::S, O0, f_offset(j), FloatRegister(j, ' '));

    for ( int k = 0; k < (VM_Version::v9_instructions_work() ? 64 : 32); k += 2)
      a->ldf( FloatRegister::D, O0, d_offset(k), FloatRegister(k, ' '));
  }
};

// pushes double TOS element of FPU stack on CPU stack; pops from FPU stack
void MacroAssembler::push_fTOS() {
  // %%%%%% need to implement this
}

// pops double TOS element from CPU stack and pushes on FPU stack 
void MacroAssembler::pop_fTOS() {
  // %%%%%% need to implement this
}

void MacroAssembler::empty_FPU_stack() {
  // %%%%%% need to implement this
}

void MacroAssembler::verify_oop(Register reg, const char* msg) {
  if (!VerifyOops) return;

  if (reg == G0)  return;	// always NULL, which is always an oop

  // plausibility check for oops

  int words_to_allocate = ::round_to(sizeof(RegistersForDebugging) / BytesPerWord, 2);

  // must save the frame, preserve all regs, and pass the selected reg
  Register pass = O0;
  save_frame_and_mov(words_to_allocate, reg, pass);

  // Pass error message to veri_oop_subroutine and possibly
  // to stop_subroutine (in case verify_oops fails, via O1
  char* b = new char[strlen(msg) + 50];
  sprintf(b, "verify_oop: %s: %s", reg.name(), msg);
  sethi(((int)b & ~0x3ff), O1);
  add(O1,((int)b &  0x3ff), O1);

  // call indirectly to solve generation ordering problem
  Address a(O5, (address)StubRoutines::sparc::verify_oop_subroutine_entry_address());
  load_contents(a, a.base());
  jmpl(a.base(), 0, O7);
  delayed()->mov_or_nop(pass, O0);		// pass the argument

  // restore is done by subroutine to save space
}

// side-door communication with signalHandler in os_solaris.cpp
address MacroAssembler::_verify_oop_implicit_branch[3] = { NULL };

// this macro is expanded just once; it creates shared code
// Contract:  receives an oop in O0.  Can smash O registers.
// (Like a leaf routine.)  Must not smash G, L, or I registers.
// Must return to O7 + 12, skipping an embedded (char*)msg word.
void MacroAssembler::verify_oop_subroutine() {
  if (!VerifyOops) return;

  Label succeed, fail, null_or_fail;

  Register obj              = O0;
  Register mask             = O1;
  Register bits             = O2;
  Register mark             = O3;
  Register temp             = O4;
  Register displaced_header = O4;

  // Copy message pointer. O1 is also used as Rmask
  mov(O1, L4);

  // for the sake of the debugger, stick a PC on the current frame
  // (this assumes that the caller has performed an extra "save")
  mov(I7, L7);
  add(O7, -7, I7);

  {
    // count number of verifies
    Register accum = O5;

    Address count_addr( temp, (address) StubRoutines::verify_oop_count_addr() );
    sethi(count_addr);
    ld(count_addr, accum);
    inc(accum);
    st(accum, count_addr);
  }

  {
    // assert((obj & oop_mask) == oop_bits);
    set(Universe::verify_oop_mask(), mask);
    set(Universe::verify_oop_bits(), bits);
    and3(mask, obj, temp);
    cmp(bits,      temp);
    br(notEqual, false, pn, null_or_fail);
    delayed()->nop();

    if ((NULL & Universe::verify_oop_mask()) == Universe::verify_oop_bits()) {
      // the null_or_fail case is useless; must test for null separately
      tst(obj);
      br(zero, false, pn, succeed);
    }
  }

  assert( markOopDesc:: monitor_value != 0, "verify header encoding");
  assert( markOopDesc::  locked_value == 0, "verify header encoding");
  assert( markOopDesc::unlocked_value != 0, "verify header encoding");

  Label get_mark, verify_unlocked_header, get_displaced_header, verify_monitor, verify_klass_oop;

  // mark lower end of faulting range
  assert(_verify_oop_implicit_branch[0] == NULL, "set once");
  _verify_oop_implicit_branch[0] = pc();

  int max_retries = 10;
  Register retry_count = O2;
  set(max_retries, retry_count);

  bind(get_mark);

  // mark = obj->mark();
  ld( Address( obj, 0, oopDesc::mark_offset_in_bytes() ), mark);
  and3(mark, markOopDesc::lock_mask_in_place, temp);

  cmp(temp, markOopDesc::unlocked_value);
  br(equal, false, pt, verify_klass_oop);                   // unlocked case, the mark is valid, go check the klassOop
  delayed()->nop();

  cmp(temp, markOopDesc::locked_value);
  br(equal, false, pt, get_displaced_header);               // locked case, get the displaced header on the stack
  delayed()->nop();

  cmp(temp, markOopDesc::monitor_value);
  br(equal, true, pt, verify_monitor);                      // inflated lock case
  delayed()->and3(mark, ~markOopDesc::monitor_value, mark); // mark now holds a pointer to the monitor structure

  br(always, false, pt, fail);                              // it better be one of these three cases
  delayed()->nop();

  {
    // Lightweight lock case
    // We have to get the displaced header carefully because there is no micro-lock state. The lock may be owned by
    // another thread, in that case, the lock may be released and the frame may be deactivated.
    // We load the displaced header and then load the mark a second time to make sure that it is valid.
    // Also, the mark may be zero while in the process of being inflated, if this is the case we try again.

    Label L;
    Register mark2 = O5;

    bind(get_displaced_header);

    cmp(mark, 0x0);
    br(equal, true, pn, L);
    delayed()->nop();

    ld(Address( mark, 0, BasicLock::displaced_header_offset_in_bytes()), displaced_header);
    ld(Address( obj, 0, oopDesc::mark_offset_in_bytes()), mark2); // get the mark a second time
    cmp(mark, mark2);
    br(equal, false, pt, verify_unlocked_header);           // we have a displaced header, verify that
    delayed()->nop();

    br(always, false, pt, get_mark);                        // we may be in a new state now, try again
    delayed()->nop();

    bind(L);
    { set(1000, temp);
      Label busy;
      bind(busy);
      deccc(temp);
      br(positive, false, pt, busy);
      delayed()->nop();
    }
    cmp(retry_count, 0x0);
    br(notEqual, false, pt, get_mark);
    delayed()->dec(retry_count);

    br(always, false, pt, fail);
    delayed()->nop();
  }

  {
    // Inflated lock case
    // mark holds the pointer to the monitor structure
    // Check the _object and _header fields

    bind(verify_monitor);

    // the object entry in the monitor structure should point back to our oop
    ld(Address(mark, 0, ObjectMonitor::object_offset_in_bytes()), temp);
    cmp(obj, temp);
    br(notEqual, false, pn, fail);
    delayed()->nop();  

    // now check the header inside the monitor structure
    ld(Address(mark, 0, ObjectMonitor::header_offset_in_bytes()), displaced_header);
    br(always, false, pt, verify_unlocked_header);
    delayed()->nop();
  }  

  {
    bind(verify_unlocked_header);

    // assert((mark & mark_mask) == mark_bits);
    assert(Universe::verify_mark_mask() == markOopDesc::lock_mask_in_place, "");
    assert(Universe::verify_mark_bits() == markOopDesc::unlocked_value, "");

    and3(displaced_header, markOopDesc::lock_mask_in_place, temp);
    cmp(temp, markOopDesc::unlocked_value);
    br(notZero, false, pn, fail);
    delayed()->nop(); 

    br(always, false, pt, verify_klass_oop);
    delayed()->nop();
  }

  {
    // Now check the klassOop of this object
    // It should be in the right are of memory and its header should not be locked (although it may be in the monitor
    // state when getting the hash code???)

    bind(verify_klass_oop);

    Register klass = O3;

    ld(Address(obj, 0, oopDesc::klass_offset_in_bytes()), klass);

    // assert((klass & klass_mask) == klass_bits);
    set(Universe::verify_klass_mask(), mask);
    set(Universe::verify_klass_bits(), bits);

    and3(mask, klass, temp);
    cmp(bits, temp);
    br(notZero, false, pn, fail);
    delayed()->nop();

    // assert(klass->_mark->is_unlocked());
    ld( Address( klass, 0, oopDesc::mark_offset_in_bytes() ), mark);
    and3(mark, markOopDesc::lock_mask_in_place, temp);

    cmp(temp, markOopDesc::unlocked_value);
    br(equal, false, pt, succeed);
    delayed()->nop();

    cmp(temp, markOopDesc::monitor_value);
    br(notEqual, false, pn, fail);
    delayed()->nop();

    // we could do some more checking here, but for now this is good enough
  }
  
  // mark upper end of faulting range
  _verify_oop_implicit_branch[1] = pc();

  // all tests pass
  bind(succeed);
  mov(L7, I7);
  retl(frame::pc_return_offset);
  delayed()->restore(); // see verify_oop above

  bind(null_or_fail);
  tst(obj);
  br(zero, false, pt, succeed);
  delayed()->nop();
  //   and fall through to fail...

  bind(fail);
  // report failure:

  _verify_oop_implicit_branch[2] = pc();

  assert(StubRoutines::sparc::stop_subroutine_entry_address(), "hasn't been generated yet");
  // call indirectly to solve generation ordering problem
  mov(L7, I7);

  mov(L4, O1); // Copy message pointer into O1. stop_subroutine sees it in I1

  Address a(O5, (address)StubRoutines::sparc::stop_subroutine_entry_address());
  load_contents(a, a.base());
  jmp(a.base(), 0);
  delayed()->nop();

  // Note: stop_subroutine has the same odd return linkage "jmp [%o7+12]".
}


void MacroAssembler::stop(const char* msg) {
  // save frame first to get O7 for return address
  // add one word to size in case struct is odd number of words long
  // It must be doubleword-aligned for storing doubles into it.

    save_frame(::round_to(sizeof(RegistersForDebugging) / BytesPerWord, 2));

    // stop_subroutine expects message pointer in I1.
    set((int)msg, O1);

    // factor long stop-sequence into subroutine to save space
    assert(StubRoutines::sparc::stop_subroutine_entry_address(), "hasn't been generated yet");

    // call indirectly to solve generation ordering problem
    Address a(O5, (address)StubRoutines::sparc::stop_subroutine_entry_address());
    load_contents(a, O5);
    jmpl(O5, 0, O7);
    delayed()->nop();

    // restore(); done in callee to save space!
}


void MacroAssembler::warn(const char* msg) {
  save_frame(::round_to(sizeof(RegistersForDebugging) / BytesPerWord, 2));
  RegistersForDebugging::save_registers(this);
  mov(O0, L0);
  set((int)msg, O0);
  call( CAST_FROM_FN_PTR(address, warning) );
  delayed()->nop();
//  ret();
//  delayed()->restore();
  RegistersForDebugging::restore_registers(this, L0);
  restore();
}


void MacroAssembler::untested(const char* what) {
  // We must be able to turn interactive prompting off
  // in order to run automated test scripts on the VM
  // Hence the new flag "PromptForDebug" -- Antero
  
  char* b = new char[1024];
  sprintf(b, "untested: %s", what);  

  if ( PromptForDebug )   stop(b);
  else                    warn(b);
}


void MacroAssembler::stop_subroutine() {
  RegistersForDebugging::save_registers(this);

  // for the sake of the debugger, stick a PC on the current frame
  // (this assumes that the caller has performed an extra "save")
  mov(I7, L7);
  add(O7, -7, I7);

  save_frame(); // one more save to free up another O7 register
  mov(I0, O1); // addr of reg save area

  // We expect pointer to message in I1. Caller must set it up in O1
  mov(I1, O0); // get msg

  call (CAST_FROM_FN_PTR(address, MacroAssembler::debug), relocInfo::runtime_call_type);
  delayed()->nop();

  restore();

  RegistersForDebugging::restore_registers(this, O0);

  save_frame(0);
  call(CAST_FROM_FN_PTR(address, breakpoint));
  delayed()->nop();
  restore();

  mov(L7, I7);
  retl(frame::pc_return_offset);
  delayed()->restore(); // see stop above
}


void MacroAssembler::debug(char* msg, RegistersForDebugging* regs) {
  if ( PromptForDebug ) {
      JavaThreadState saved_state = JavaThread::current()->thread_state();
      JavaThread::current()->set_thread_state(_thread_in_vm);
      {
        // %%%% should reconcile this with SharedRuntime::debug
        // In order to get locks work, we need to fake a in_VM state
        MutexLocker m(Event_lock);
        ::tty->print_cr("EXECUTION STOPPED: %s\n", msg);
        if (CountBytecodes || TraceBytecodes || StopInterpreterAt) {
	  ::tty->print_cr("Interpreter::bytecode_counter = %d", BytecodeCounter::counter_value());
        }
        if (os::message_box(msg, "Execution stopped, print registers?"))
          regs->print(::tty);
      }
      ThreadStateTransition::transition(JavaThread::current(), _thread_in_vm, saved_state);
  }
  else
     ::tty->print_cr("=============== DEBUG MESSAGE: %s ================\n", msg);
}


#ifndef PRODUCT
void MacroAssembler::test() {
  ResourceMark rm;

  MacroAssembler* a = new MacroAssembler(new CodeBuffer(10000, 10000, 0, 0, false));
  VM_Version::allow_all();
  a->test_v9();
  a->test_v8_onlys();
  VM_Version::revert();

  StubRoutines::sparc::test_stop_entry()();
}
#endif


void MacroAssembler::calc_mem_param_words(Register Rparam_words, Register Rresult) {
  subcc( Rparam_words, Argument::n_register_parameters, Rresult); // how many mem words?
  Label no_extras;
  br( negative, true, pt, no_extras ); // if neg, clear reg
  delayed()->set( 0, Rresult);         // annuled, so only if taken
  bind( no_extras );
}


void MacroAssembler::calc_frame_size(Register Rextra_words, Register Rresult) {
  add(Rextra_words, frame::memory_parameter_word_sp_offset + 1, Rresult);
  bclr(1, Rresult);
  sll(Rresult, LogBytesPerWord, Rresult);  // Rresult has total frame bytes
}


void MacroAssembler::calc_frame_size_and_save(Register Rextra_words, Register Rresult) {
  calc_frame_size(Rextra_words, Rresult);
  neg(Rresult);
  save(SP, Rresult, SP);
}


// ---------------------------------------------------------


// Warning: the next four routines alter the contents of R2, and the float-point
// source register.

void MacroAssembler::load_unaligned_double(Register r1, Register r2, const Address& d_tmp, FloatRegister d) {
  FloatRegister t = d; // reuse d for temp
  assert_not_delayed();

# ifdef FLOAT_REGS_NOT_ALIASED
  ldf(FloatRegister::S, r1, r2, t);  stf(FloatRegister::S, t, d_tmp);
  inc(r2, wordSize);
  ldf(FloatRegister::S, r1, r2, t);  stf(FloatRegister::S, t, d_tmp, wordSize);
  ldf(FloatRegister::D, d_tmp, d);
# else
  ldf(FloatRegister::S, r1, r2, d);
  inc(r2, wordSize);
  ldf(FloatRegister::S, r1, r2, d.successor());
# endif
}

void MacroAssembler::store_unaligned_double(FloatRegister d, const Address& d_tmp, Register r1, Register r2) {
  FloatRegister t = d; // reuse d for temp
  assert_not_delayed();
# ifdef FLOAT_REGS_NOT_ALIASED
  stf(FloatRegister::D, d, d_tmp);
  ldf(FloatRegister::S, d_tmp, t);           stf(FloatRegister::S, t, r1, r2);
  inc(r2, wordSize);
  ldf(FloatRegister::S, d_tmp, t, wordSize); stf(FloatRegister::S, t, r1, r2);
# else
  stf(FloatRegister::S, d,             r1, r2);
  inc(r2, wordSize);
  stf(FloatRegister::S, d.successor(), r1, r2);
# endif
}


void MacroAssembler::load_unaligned_double(Register r1, int offset, const Address& d_tmp, FloatRegister d) {
  FloatRegister t = d; // reuse d for temp
  assert_not_delayed();
# ifdef FLOAT_REGS_NOT_ALIASED
  ldf(FloatRegister::S, r1, offset,            t);  stf(FloatRegister::S, t, d_tmp);
  ldf(FloatRegister::S, r1, offset + wordSize, t);  stf(FloatRegister::S, t, d_tmp, wordSize);
  ldf(FloatRegister::D, d_tmp, d);
# else
  ldf(FloatRegister::S, r1, offset,            d);
  ldf(FloatRegister::S, r1, offset + wordSize, d.successor());
# endif
}

void MacroAssembler::store_unaligned_double(FloatRegister d, const Address& d_tmp, Register r1, int offset) {
  FloatRegister t = d; // reuse d for temp
  assert_not_delayed();
# ifdef FLOAT_REGS_NOT_ALIASED
  stf(FloatRegister::D, d, d_tmp);
  ldf(FloatRegister::S, d_tmp, t);           stf(FloatRegister::S, t, r1, offset);
  ldf(FloatRegister::S, d_tmp, t, wordSize); stf(FloatRegister::S, t, r1, offset + wordSize);
# else
  stf(FloatRegister::S, d,             r1, offset);
  stf(FloatRegister::S, d.successor(), r1, offset + wordSize);
# endif
}


Assembler::RCondition cond2rcond(Assembler::Condition c) {
  switch (c) {
    /*case zero: */
    case Assembler::equal:        return Assembler::rc_z;
    case Assembler::lessEqual:    return Assembler::rc_lez;
    case Assembler::less:         return Assembler::rc_lz;
    /*case notZero:*/
    case Assembler::notEqual:     return Assembler::rc_nz;
    case Assembler::greater:      return Assembler::rc_gz;
    case Assembler::greaterEqual: return Assembler::rc_gez;
  }
  ShouldNotReachHere();
  return Assembler::rc_z;
}

// compares register with zero and branches
void MacroAssembler::br_zero( Condition c, bool a, Predict p, Register s1, Label& L) {
  tst(s1);
  br (c, a, p, L);
}


// a helper for load/store double:

void MacroAssembler::br_aligned(Register r1, int offset, Label& aligned) {
  btst(LongAlignmentMask, r1);
  br( (offset & LongAlignmentMask) ? notZero : zero,  true, pt, aligned );
}


// generic float register move
void MacroAssembler::mov(FloatRegister::Width w, FloatRegister s, FloatRegister d) {
  // do not bother to move a register to itself
  if (s == d) { assert_not_delayed(); return; }

  if (VM_Version::v9_instructions_work())
    Assembler::fmov (w, s, d);
  else {
    switch (w)
    {
      case FloatRegister::S:
	Assembler::fmov(FloatRegister::S, s, d);
        break;
      case FloatRegister::D:
	Assembler::fmov(FloatRegister::S, s,             d            );
	Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
        break;
      case FloatRegister::Q:
	Assembler::fmov(FloatRegister::S, FloatRegister(s.number(FloatRegister::S) + 0, ' '), FloatRegister(d.number(FloatRegister::D) + 0, ' '));
	Assembler::fmov(FloatRegister::S, FloatRegister(s.number(FloatRegister::S) + 1, ' '), FloatRegister(d.number(FloatRegister::D) + 1, ' '));
	Assembler::fmov(FloatRegister::S, FloatRegister(s.number(FloatRegister::S) + 2, ' '), FloatRegister(d.number(FloatRegister::D) + 2, ' '));
	Assembler::fmov(FloatRegister::S, FloatRegister(s.number(FloatRegister::S) + 3, ' '), FloatRegister(d.number(FloatRegister::D) + 3, ' '));
        break;
      default:
	ShouldNotReachHere();
        break;
    }
  }
}


void MacroAssembler::call_with_next_pc( Register temp, Label& next_pc, address d,  relocInfo::relocType rt ) {
  assert(next_pc.is_bound(), "must be previously defined");
  address return_addr = pc() + 2*wordSize;
  int back_offset = return_addr - target(next_pc);
  if (is_simm13(back_offset)) {
    call(d, rt);
    delayed()->sub(O7, back_offset, O7);
    assert(pc() == return_addr, "return addr prediction correct");
  } else {
    address return_addr_2 = pc() + 4*wordSize;
    int back_offset_2 = return_addr_2 - target(next_pc);
    Address offset_addr(temp, (address)back_offset_2);

    sethi(offset_addr);
    add(offset_addr, temp);
    call(d, rt);
    delayed()->sub(O7, temp, O7);
    assert(pc() == return_addr_2, "return addr prediction correct");
  }
}


// test 2nd register only, also destroys value in 2nd register
void MacroAssembler::load_double(Register r1, Register r2, const Address& d_tmp, FloatRegister d) {
  assert_not_delayed();
  Label aligned;
  br_aligned(r2, 0, aligned);
  delayed()->ldf(FloatRegister::D, r1, r2, d);
  load_unaligned_double(r1, r2, d_tmp, d);
  bind(aligned);
}

void MacroAssembler::load_double(Register r1, int  offset, const Address& d_tmp, FloatRegister d) {
  assert_not_delayed();
  Label aligned;
  br_aligned(r1, offset, aligned);
  delayed()->ldf(FloatRegister::D, r1, offset, d);
  load_unaligned_double(r1, offset, d_tmp, d);
  bind(aligned);
}

// test 2nd register only, also destroys value in 2nd register
void MacroAssembler::store_double(FloatRegister d, const Address& d_tmp, Register r1, Register r2) {
  assert_not_delayed();
  Label aligned;
  br_aligned(r2, 0, aligned);
  delayed()->stf(FloatRegister::D, d, r1, r2);
  store_unaligned_double(d, d_tmp, r1, r2);
  bind(aligned);
}

void MacroAssembler::store_double(FloatRegister d, const Address& d_tmp, Register r1, int offset) {
  assert_not_delayed();
  Label aligned;
  br_aligned(r1, offset, aligned);
  delayed()->stf(FloatRegister::D, d, r1, offset);

  store_unaligned_double(d, d_tmp, r1, offset);
  bind(aligned);
}


void MacroAssembler::load_long(Register r1, Register r2, Register rd) {
  assert_not_delayed();
  if (rd.is_even()) {
    Label aligned;
    br_aligned(r2, 0, aligned);
    delayed()->ldd(r1, r2, rd);
    load_unaligned_long(r1, r2, rd);
    bind (aligned);
  }
  else
    load_unaligned_long(r1, r2, rd);
}

void MacroAssembler::store_long(Register rs, Register r1, Register r2) {
  assert_not_delayed();
  if (rs.is_even()) {
    Label aligned;
    br_aligned(r2, 0, aligned);
    delayed()->std(rs, r1, r2);
    store_unaligned_long(rs, r1, r2);
    bind (aligned);
  }
  else
    store_unaligned_long(rs, r1, r2);
}

void MacroAssembler::load_long(Register r1, int offset, Register rd) {
  assert_not_delayed();
  if (rd.is_even()) {
    Label aligned;
    br_aligned(r1, offset, aligned);
    delayed()->ldd(r1, offset, rd);
    load_unaligned_long(r1, offset, rd);
    bind (aligned);
  }
  else
    load_unaligned_long(r1, offset, rd);
}

void MacroAssembler::store_long(Register rs, Register r1, int offset) {
  assert_not_delayed();
  if (rs.is_even()) {
    Label aligned;
    br_aligned(r1, offset, aligned);
    delayed()->std(rs, r1, offset);
    store_unaligned_long(rs, r1, offset);
    bind (aligned);
  }
  else
    store_unaligned_long(rs, r1, offset);
}

void MacroAssembler::load_unaligned_long(Register r1, Register r2, Register rd) {
  assert_not_delayed();
  ld(r1, r2, rd);
  inc(r2, wordSize);
  ld(r1, r2, rd.successor());
}

void MacroAssembler::store_unaligned_long(Register rd, Register r1, Register r2) {
  assert_not_delayed();
  st(rd, r1, r2);
  inc(r2, wordSize);
  st(rd.successor(), r1, r2);
}

void MacroAssembler::load_unaligned_long(Register r1, int offset, Register rd) {
  assert_not_delayed();
  ld(r1, offset, rd);
  ld(r1, offset + wordSize, rd.successor());
}

void MacroAssembler::store_unaligned_long(Register rd, Register r1, int offset) {
  assert_not_delayed();
  st(rd, r1, offset);
  st(rd.successor(), r1, offset + wordSize);
}

// load a value(word size) from memory(r1,offset) into a register(d)
void MacroAssembler::load_word_with_offset(Register r1, int offset, Register d, Register scratch) {
  assert_not_delayed();
  if (is_simm13(offset))
    ld(r1, offset, d);
  else {
    set(offset, scratch);
    ld(r1, scratch, d);
  }
}

// store a value(word size) from register(d) into memory location(r1, offset)
void MacroAssembler::store_word_with_offset(Register d, Register r1, int offset, Register scratch) {
  assert_not_delayed();
  if (is_simm13(offset))
    st(d, r1, offset);
  else {
    set(offset, scratch);
    st(d, r1, scratch);
  }
}

// load from statically known place
void MacroAssembler::load_static_long(const Address& a, Register rd) {
  assert_not_delayed();
  if ( rd.is_even()  &&  ( a.value() & LongAlignmentMask ) == 0 )  
    ldd( a.base(), a.disp(), rd);
  else
    load_unaligned_long( a.base(), a.disp(), rd);
}


// store to statically known place
void MacroAssembler::store_static_long(Register rd, const Address& a) {
  assert_not_delayed();
  if ( rd.is_even()  &&  ( a.value() & LongAlignmentMask ) == 0 )  
    std( rd, a.base(), a.disp());
  else
    store_unaligned_long( rd, a.base(), a.disp());
}


// load from statically known place
void MacroAssembler::load_static_double(const Address& a, const Address& d_tmp, FloatRegister rd) {
  assert_not_delayed();
  if (a.value() & LongAlignmentMask)  load_unaligned_double( a.base(), a.disp(), d_tmp, rd);
  else                                ldf(FloatRegister::D, a.base(), a.disp(), rd);
}

// store to statically known place
void MacroAssembler::store_static_double(FloatRegister rd, const Address& d_tmp, const Address& a) {
  assert_not_delayed();
  if (a.value() & LongAlignmentMask)  store_unaligned_double( rd, d_tmp, a.base(), a.disp());
  else                                stf(FloatRegister::D, rd, a.base(), a.disp());
}




// instruction sequences factored across compiler & interpreter 


void MacroAssembler::lcmp( Register Ra_hi, Register Ra_low, 
			   Register Rb_hi, Register Rb_low,
			   Register Rresult) {

  Label check_low_parts, done;

  cmp(Ra_hi, Rb_hi );  // compare hi parts
  br(equal, true, pt, check_low_parts);
  delayed()->cmp(Ra_low, Rb_low); // test low parts

  // And, with an unsigned comparison, it does not matter if the numbers
  // are negative or not.
  // E.g., -2 cmp -1: the low parts are 0xfffffffe and 0xffffffff.
  // The second one is bigger (unsignedly). 

  // Other notes:  The first move in each triplet can be unconditional
  // (and therefore probably prefetchable).
  // And the equals case for the high part does not need testing,
  // since that triplet is reached only after finding the high halves differ.

  if (VM_Version::v9_instructions_work()) {
    
                                    mov  (                     -1, Rresult);
    ba( false, done );  delayed()-> movcc(greater, false, icc,  1, Rresult);
  }
  else {
    br(less,    true, pt, done); delayed()-> set(-1, Rresult);
    br(greater, true, pt, done); delayed()-> set( 1, Rresult);
  }

  bind( check_low_parts );

  if (VM_Version::v9_instructions_work()) {
    mov(                               -1, Rresult);
    movcc(equal,           false, icc,  0, Rresult);
    movcc(greaterUnsigned, false, icc,  1, Rresult);
  }
  else {
                                                    set(-1, Rresult);
    br(equal,           true, pt, done); delayed()->set( 0, Rresult);
    br(greaterUnsigned, true, pt, done); delayed()->set( 1, Rresult);
  }
  bind( done );
}


void MacroAssembler::lneg( Register Rhi, Register Rlow ) {
  subcc(  G0, Rlow, Rlow );
  subc(   G0, Rhi,  Rhi  );
}

void MacroAssembler::lshl( Register Rin_high,  Register Rin_low,
			   Register Rcount,
			   Register Rout_high, Register Rout_low,
			   Register Rtemp ) {


  Register Ralt_count = Rtemp;
  Register Rxfer_bits = Rtemp;

  assert( Ralt_count != Rin_high
      &&  Ralt_count != Rin_low
      &&  Ralt_count != Rcount
      &&  Rxfer_bits != Rin_low
      &&  Rxfer_bits != Rin_high
      &&  Rxfer_bits != Rcount
      &&  Rxfer_bits != Rout_low
      &&  Rout_low   != Rin_high,
	"register alias checks");

  Label big_shift, done;

  // This code can be optimized to use the 64 bit shifts in V9.
  // Here we use the 32 bit shifts.

  and3(	Rcount,		0x3f,		Rcount);     // take least significant 6 bits
  subcc(Rcount,		31,		Ralt_count);
  br(greater, true, pn, big_shift);
  delayed()->
  dec(Ralt_count); 

  // shift < 32 bits, Ralt_count = Rcount-31

  // We get the transfer bits by shifting right by 32-count the low
  // register. This is done by shifting right by 31-count and then by one
  // more to take care of the special (rare) case where count is zero
  // (shifting by 32 would not work).

  neg(	Ralt_count				   ); 

  // The order of the next two instructions is critical in the case where
  // Rin and Rout are the same and should not be reversed.

  srl(	Rin_low,	Ralt_count,	Rxfer_bits ); // shift right by 31-count
  sll(	Rin_low,	Rcount,		Rout_low   ); // low half
  sll(	Rin_high,	Rcount, 	Rout_high  );
  srl(	Rxfer_bits,	1, 		Rxfer_bits ); // shift right by one more
  ba (false, done);
  delayed()->
  or3(	Rout_high,	Rxfer_bits,	Rout_high);   // new hi value: or in shifted old hi part and xfer from low

  // shift >= 32 bits, Ralt_count = Rcount-32
  bind(big_shift);
  sll(  Rin_low,        Ralt_count,     Rout_high  );
  clr(	Rout_low				   );

  bind(done);
}


void MacroAssembler::lshr( Register Rin_high,  Register Rin_low,
			   Register Rcount,
			   Register Rout_high, Register Rout_low,
			   Register Rtemp ) {

  Register Ralt_count = Rtemp;
  Register Rxfer_bits = Rtemp;

  assert( Ralt_count != Rin_high
      &&  Ralt_count != Rin_low
      &&  Ralt_count != Rcount
      &&  Rxfer_bits != Rin_low
      &&  Rxfer_bits != Rin_high
      &&  Rxfer_bits != Rcount
      &&  Rxfer_bits != Rout_high
      &&  Rout_high  != Rin_low,
	"register alias checks");

  Label big_shift, done;

  // This code can be optimized to use the 64 bit shifts in V9.
  // Here we use the 32 bit shifts.

  and3(	Rcount,		0x3f,		Rcount);     // take least significant 6 bits
  subcc(Rcount,		31,		Ralt_count);
  br(greater, true, pn, big_shift);
  delayed()->dec(Ralt_count);

  // shift < 32 bits, Ralt_count = Rcount-31

  // We get the transfer bits by shifting left by 32-count the high
  // register. This is done by shifting left by 31-count and then by one
  // more to take care of the special (rare) case where count is zero
  // (shifting by 32 would not work).

  neg(	Ralt_count				    ); 
  srl(	Rin_low,	Rcount,		Rout_low    ); 

  // The order of the next two instructions is critical in the case where
  // Rin and Rout are the same and should not be reversed.

  sll(	Rin_high,	Ralt_count,	Rxfer_bits  ); // shift left by 31-count
  sra(	Rin_high,	Rcount,		Rout_high   ); // high half
  sll(	Rxfer_bits,	1,		Rxfer_bits  ); // shift left by one more
  ba (false, done);
  delayed()->
  or3(	Rout_low,	Rxfer_bits,	Rout_low    ); // new low value: or shifted old low part and xfer from high

  // shift >= 32 bits, Ralt_count = Rcount-32
  bind(big_shift);

  sra(	Rin_high,	Ralt_count,	Rout_low    );
  sra(	Rin_high,	31,		Rout_high   ); // sign into hi

  bind( done );
}



void MacroAssembler::lushr( Register Rin_high,  Register Rin_low,
        	            Register Rcount,
			    Register Rout_high, Register Rout_low,
			    Register Rtemp ) {

  Register Ralt_count = Rtemp;
  Register Rxfer_bits = Rtemp;

  assert( Ralt_count != Rin_high
      &&  Ralt_count != Rin_low
      &&  Ralt_count != Rcount
      &&  Rxfer_bits != Rin_low
      &&  Rxfer_bits != Rin_high
      &&  Rxfer_bits != Rcount
      &&  Rxfer_bits != Rout_high
      &&  Rout_high  != Rin_low,
	"register alias checks");

  Label big_shift, done;

  // This code can be optimized to use the 64 bit shifts in V9.
  // Here we use the 32 bit shifts.

  and3(	Rcount,		0x3f,		Rcount);     // take least significant 6 bits
  subcc(Rcount,		31,		Ralt_count);
  br(greater, true, pn, big_shift);
  delayed()->dec(Ralt_count);

  // shift < 32 bits, Ralt_count = Rcount-31

  // We get the transfer bits by shifting left by 32-count the high
  // register. This is done by shifting left by 31-count and then by one
  // more to take care of the special (rare) case where count is zero
  // (shifting by 32 would not work).

  neg(	Ralt_count				    ); 
  srl(	Rin_low,	Rcount,		Rout_low    ); 

  // The order of the next two instructions is critical in the case where
  // Rin and Rout are the same and should not be reversed.

  sll(	Rin_high,	Ralt_count,	Rxfer_bits  ); // shift left by 31-count
  srl(	Rin_high,	Rcount,		Rout_high   ); // high half
  sll(	Rxfer_bits,	1,		Rxfer_bits  ); // shift left by one more
  ba (false, done);
  delayed()->
  or3(	Rout_low,	Rxfer_bits,	Rout_low    ); // new low value: or shifted old low part and xfer from high

  // shift >= 32 bits, Ralt_count = Rcount-32
  bind(big_shift);

  srl(	Rin_high,	Ralt_count,	Rout_low    );
  clr(	Rout_high				    );

  bind( done );
}


void MacroAssembler::float_cmp( bool is_float, int unordered_result, 
				FloatRegister Fa, FloatRegister Fb,
				Register Rresult) {

  fcmp(is_float ? FloatRegister::S : FloatRegister::D, fcc0, Fa, Fb);

  Condition lt = unordered_result == -1 ? f_unorderedOrLess    : f_less;
  Condition eq =                          f_equal;
  Condition gt = unordered_result ==  1 ? f_unorderedOrGreater : f_greater;

  if (VM_Version::v9_instructions_work()) {

     mov(                   -1, Rresult );
     movcc( eq, true, fcc0,  0, Rresult );
     movcc( gt, true, fcc0,  1, Rresult );
  }
  else {
     Label done;

                                          set( -1, Rresult );
     //fb(lt, true, pn, done); delayed()->set( -1, Rresult );
     fb( eq, true, pn, done);  delayed()->set(  0, Rresult );
     fb( gt, true, pn, done);  delayed()->set(  1, Rresult );

     bind (done);
  }
}

void MacroAssembler::fneg( FloatRegister::Width w, FloatRegister s, FloatRegister d)
{
  if(VM_Version::v9_instructions_work()) {
    Assembler::fneg(w, s, d);
  } else {
    if(w==FloatRegister::S) {
      Assembler::fneg(w, s, d);
    } else if(w==FloatRegister::D) {
      // number() does a sanity check on the alignment.
      assert(((s.number(FloatRegister::D) & 1) == 0) &&
        ((d.number(FloatRegister::D) & 1) == 0), "float register alignment check");

      Assembler::fneg(FloatRegister::S, s, d);
      Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
    } else {
      assert(w==FloatRegister::Q, "Invalid float register width");

      // number() does a sanity check on the alignment.
      assert(((s.number(FloatRegister::D) & 3) == 0) &&
        ((d.number(FloatRegister::D) & 3) == 0), "float register alignment check");

      Assembler::fneg(FloatRegister::S, s, d);
      Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
      Assembler::fmov(FloatRegister::S, s.successor().successor(), d.successor().successor());
      Assembler::fmov(FloatRegister::S, s.successor().successor().successor(), d.successor().successor().successor());
    }
  }
}

void MacroAssembler::fmov( FloatRegister::Width w, FloatRegister s, FloatRegister d)
{
  if(VM_Version::v9_instructions_work()) {
    Assembler::fmov(w, s, d);
  } else {
    if(w==FloatRegister::S) {
      Assembler::fmov(w, s, d);
    } else if(w==FloatRegister::D) {
      // number() does a sanity check on the alignment.
      assert(((s.number(FloatRegister::D) & 1) == 0) &&
        ((d.number(FloatRegister::D) & 1) == 0), "float register alignment check");

      Assembler::fmov(FloatRegister::S, s, d);
      Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
    } else {
      assert(w==FloatRegister::Q, "Invalid float register width");

      // number() does a sanity check on the alignment.
      assert(((s.number(FloatRegister::D) & 3) == 0) &&
        ((d.number(FloatRegister::D) & 3) == 0), "float register alignment check");

      Assembler::fmov(FloatRegister::S, s, d);
      Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
      Assembler::fmov(FloatRegister::S, s.successor().successor(), d.successor().successor());
      Assembler::fmov(FloatRegister::S, s.successor().successor().successor(), d.successor().successor().successor());
    }
  }
}

void MacroAssembler::fabs( FloatRegister::Width w, FloatRegister s, FloatRegister d)
{
  if(VM_Version::v9_instructions_work()) {
    Assembler::fabs(w, s, d);
  } else {
    if(w==FloatRegister::S) {
      Assembler::fabs(w, s, d);
    } else if(w==FloatRegister::D) {
      // number() does a sanity check on the alignment.
      assert(((s.number(FloatRegister::D) & 1) == 0) &&
        ((d.number(FloatRegister::D) & 1) == 0), "float register alignment check");

      Assembler::fabs(FloatRegister::S, s, d);
      Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
    } else {
      assert(w==FloatRegister::Q, "Invalid float register width");

      // number() does a sanity check on the alignment.
      assert(((s.number(FloatRegister::D) & 3) == 0) &&
       ((d.number(FloatRegister::D) & 3) == 0), "float register alignment check");

      Assembler::fabs(FloatRegister::S, s, d);
      Assembler::fmov(FloatRegister::S, s.successor(), d.successor());
      Assembler::fmov(FloatRegister::S, s.successor().successor(), d.successor().successor());
      Assembler::fmov(FloatRegister::S, s.successor().successor().successor(), d.successor().successor().successor());
    }
  }
}

void MacroAssembler::save_all_globals_into_locals() {
  mov(G1,L1);
  mov(G2,L2);
  mov(G3,L3);
  mov(G4,L4);
  mov(G5,L5);
  mov(G6,L6);
  mov(G7,L7);
}

void MacroAssembler::restore_globals_from_locals() {
  mov(L1,G1);
  mov(L2,G2);
  mov(L3,G3);
  mov(L4,G4);
  mov(L5,G5);
  mov(L6,G6);
  mov(L7,G7);
}

void MacroAssembler::cas_under_lock(Register top_ptr_reg, Register top_reg, Register ptr_reg, address lock_addr, bool use_call_vm)
{
  // store ptr_reg as the new top value
  if (VM_Version::v9_instructions_work()) {
    cas(top_ptr_reg, top_reg, ptr_reg);
  } else {

    // If the register is not an out nor global, it is not visible
    // after the save.  Allocate a register for it, save its
    // value in the register save area (the save may not flush
    // registers to the save area).

    Register top_ptr_reg_after_save;
    Register top_reg_after_save;
    Register ptr_reg_after_save;

    if (top_ptr_reg.is_out() || top_ptr_reg.is_global()) {
      top_ptr_reg_after_save = top_ptr_reg.after_save();
    } else {
      Address reg_save_addr = top_ptr_reg.address_in_saved_window();
      top_ptr_reg_after_save = L0;
      st(top_ptr_reg, reg_save_addr);
    }

    if (top_reg.is_out() || top_reg.is_global()) {
      top_reg_after_save = top_reg.after_save();
    } else {
      Address reg_save_addr = top_reg.address_in_saved_window();
      top_reg_after_save = L1;
      st(top_reg, reg_save_addr);
    }

    if (ptr_reg.is_out() || ptr_reg.is_global()) {
      ptr_reg_after_save = ptr_reg.after_save();
    } else {
      Address reg_save_addr = ptr_reg.address_in_saved_window();
      ptr_reg_after_save = L2;
      st(ptr_reg, reg_save_addr);
    }

    const Register& lock_reg = L3;
    const Register& lock_ptr_reg = L4;
    const Register& value_reg = L5;
    const Register& yield_reg = L6;
    const Register& yieldall_reg = L7;

    save_frame();

    if (top_ptr_reg_after_save == L0) {
      ld(top_ptr_reg.address_in_saved_window().after_save(), 
	 top_ptr_reg_after_save);
    }

    if (top_reg_after_save == L1) {
      ld(top_reg.address_in_saved_window().after_save(), 
	 top_reg_after_save);
    }

    if (ptr_reg_after_save == L2) {
      ld(ptr_reg.address_in_saved_window().after_save(), 
	 ptr_reg_after_save);
    }

    Label(retry_get_lock);
    Label(not_same);
    Label(dont_yield);

    assert(lock_addr, "lock_address should be non null for v8");
    set((int)lock_addr, lock_ptr_reg);
    // Initialize yield counter
    mov(G0,yield_reg);
    mov(G0, yieldall_reg);
    set(StubRoutines::sparc::locked, lock_reg);

    bind(retry_get_lock);
    cmp(yield_reg, V8AtomicOperationUnderLockSpinCount);
    br(Assembler::less, false, Assembler::pt, dont_yield);
    delayed()->nop();
    
    if(use_call_vm) {
      Untested("Need to verify global reg consistancy");
      call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::yield_all), yieldall_reg);
    } else {
      // Save the regs and make space for a C call
      save(SP, -96, SP);
      save_all_globals_into_locals();
      call(CAST_FROM_FN_PTR(address, os::yield_all));
      delayed()->mov(yieldall_reg, O0);
      restore_globals_from_locals();
      restore();
    }

    // reset the counter
    mov(G0,yield_reg);
    add(yieldall_reg, 1, yieldall_reg);
    
    bind(dont_yield);
    // try to get lock
    swap(lock_ptr_reg, 0, lock_reg);

    // did we get the lock?
    cmp(lock_reg, StubRoutines::sparc::unlocked);
    br(Assembler::notEqual, true, Assembler::pn, retry_get_lock);
    delayed()->add(yield_reg,1,yield_reg);

    // yes, got lock.  do we have the same top?
    ld(top_ptr_reg_after_save, 0, value_reg);
    cmp(value_reg, top_reg_after_save);
    br(Assembler::notEqual, false, Assembler::pn, not_same);
    delayed()->nop();

    // yes, same top.
    st(ptr_reg_after_save, top_ptr_reg_after_save, 0);
    membar(Assembler::StoreStore); 

    bind(not_same);
    mov(value_reg, ptr_reg_after_save);
    st(lock_reg, lock_ptr_reg, 0); // unlock

    restore();
  }
}

void MacroAssembler::compiler_lock_object(Register Roop, Register Rmark, Register Rbox, Register Rscratch, Label& done) {
   Address mark_addr(Roop, 0, oopDesc::mark_offset_in_bytes());

   verify_oop(Roop);

   // Load markOop from object into Rmark
   ld(mark_addr, Rmark);

   // Save Rbox in Rscratch to be used for the cas operation
   mov(Rbox, Rscratch);

   // set Rmark to markOop | markOopDesc::unlocked_value      
   or3(Rmark, markOopDesc::unlocked_value, Rmark);

   // Initialize the box.  (Must happen before we update the object mark!)
   st(Rmark, Rbox, BasicLock::displaced_header_offset_in_bytes());

   // compare object markOop with Rmark and if equal exchange Rscratch with object markOop
   assert(mark_addr.disp() == 0, "cas must take a zero displacement");
   cas_under_lock(mark_addr.base(), Rmark, Rscratch, (address)StubRoutines::sparc::atomic_memory_operation_lock_addr());

   // if compare/exchange succeeded we found an unlocked object and we now have locked it
   // hence we are done
   cmp(Rmark, Rscratch);
   br(Assembler::equal, false, Assembler::pt, done);
   delayed()->sub(Rscratch, SP, Rscratch);  //pull next instruction into delay slot

   // we did not find an unlocked object so see if this is a recursive case
   // sub(Rscratch, SP, Rscratch);
   assert(os::vm_page_size() > 0xfff, "page size too small - change the constant");
   andcc(Rscratch, 0xfffff003, Rscratch);
   st(Rscratch, Rbox, BasicLock::displaced_header_offset_in_bytes());
}

void MacroAssembler::compiler_unlock_object(Register Roop, Register Rmark, Register Rbox, Register Rscratch, Label& done) {
   Address mark_addr(Roop, 0, oopDesc::mark_offset_in_bytes());

   // Test first if it is a fast recursive unlock
   ld(Rbox, BasicLock::displaced_header_offset_in_bytes(), Rmark);
   cmp(Rmark, G0);
   br(Assembler::equal, false, Assembler::pt, done);
   delayed()->nop();

   // Check if it is still a light weight lock, this is is true if we see
   // the stack address of the basicLock in the markOop of the object
   assert(mark_addr.disp() == 0, "cas must take a zero displacement");
   cas_under_lock(mark_addr.base(), Rbox, Rmark, (address)StubRoutines::sparc::atomic_memory_operation_lock_addr());
   cmp(Rbox, Rmark);
}

void MacroAssembler::print_CPU_state() {
  // %%%%% need to implement this
}

void MacroAssembler::verify_FPU(int stack_depth, const char* s) {
  // %%%%% need to implement this
}

void MacroAssembler::push_IU_state() {
  // %%%%% need to implement this
}


void MacroAssembler::pop_IU_state() {
  // %%%%% need to implement this
}


void MacroAssembler::push_FPU_state() {
  // %%%%% need to implement this
}


void MacroAssembler::pop_FPU_state() {
  // %%%%% need to implement this
}


void MacroAssembler::push_CPU_state() {
  // %%%%% need to implement this
}


void MacroAssembler::pop_CPU_state() {
  // %%%%% need to implement this
}

//Reconciliation History
// 1.38 97/12/04 18:28:46 assembler_i486.cpp
// 1.46 97/12/18 09:51:10 assembler_i486.cpp
// 1.51 98/01/22 18:04:36 assembler_i486.cpp
// 1.53 98/01/28 15:54:16 assembler_i486.cpp
// 1.63 98/02/20 16:37:00 assembler_i486.cpp
// 1.64 98/02/28 16:37:00 assembler_i486.cpp
// 1.52 98/01/30 20:34:04 assembler_i486.cpp
// 1.69 98/03/09 11:37:46 assembler_i486.cpp
// 1.69 98/03/12 11:23:56 assembler_i486.cpp
// 1.72 98/04/08 13:43:19 assembler_i486.cpp
// 1.74 98/04/13 17:41:03 assembler_i486.cpp
// 1.78 98/05/15 16:22:38 assembler_i486.cpp
// 1.79 98/05/21 17:05:47 assembler_i486.cpp
// 1.81 98/06/17 20:35:25 assembler_i486.cpp
// 1.84 98/06/19 17:46:55 assembler_i486.cpp
// 1.90 98/07/06 11:58:52 assembler_i486.cpp
// 1.93 98/07/24 13:32:15 assembler_i486.cpp
// 1.94 98/08/31 13:40:42 assembler_i486.cpp
// 1.96 98/09/25 16:05:16 assembler_i486.cpp
// 1.98 98/09/29 13:16:10 assembler_i486.cpp
// 1.100 98/10/06 14:22:34 assembler_i486.cpp
// 1.103 98/11/06 16:23:13 assembler_i486.cpp
// 1.104 98/11/12 15:31:19 assembler_i486.cpp
// 1.105 98/11/27 13:06:33 assembler_i486.cpp
// 1.106 99/01/26 15:22:36 assembler_i486.cpp
// 1.107 99/03/15 12:35:36 assembler_i486.cpp
// 1.108 99/04/06 14:35:26 assembler_i486.cpp
// 1.112 99/06/29 18:04:31 assembler_i486.cpp
// 1.116 99/08/03 14:58:58 assembler_i486.cpp
// 1.118 99/08/16 13:11:14 assembler_i486.cpp
//End
