/*
 * @(#)c1_CacheLocals_sparc.cpp	1.28 00/07/04 23:31:01
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


# include "incls/_precompiled.incl"
# include "incls/_c1_CacheLocals_sparc.cpp.incl"


//------------------caching locals---------------------



class BlockItemSetter: public BlockClosure {
 private:
  BlockItem* _item;
 public:
  BlockItemSetter(BlockItem* item): _item(item) {}

  virtual void block_do(BlockBegin* block) {
    assert(block->block_item() == NULL, "destroying information");
    block->set_block_item(_item);
  }
};


//-------------------------------------------------------------------------


RInfoCollection* LocalCaching::collect_free_cpu_registers(Loop* loop) {
  Unimplemented();
  return NULL;
}


// Search regs for the register in_reg; if not found, 
// return a non I(n) register; 
RInfo LocalCaching::get_incoming_at_index(RInfoCollection* regs, RInfo in_reg) {
  // try to get the register that belongs to me (mapping I(local_ix) <-> local_ix  )
  if(regs->contains(in_reg)) {
    return in_reg;
  }
  // Otherwise assign a non-incoming register.
  // Search for a non-incoming register.
  for (int i = 0; i < regs->number_of_registers(); i++) {
    RInfo reg = regs->register_at(i);
    if (!reg.as_register().is_in()) return reg;
  }
  // Appropriate register not found
  return norinfo;
}


// Try to use a non incoming register; if none available, use any;
RInfo LocalCaching::get_non_incoming_at_index(RInfoCollection* regs) {
  // search for a non-incoming register
  for (int i = 0; i < regs->number_of_registers(); i++) {
    RInfo reg = regs->register_at(i).as_register();
    if (!reg.as_register().is_in()) return reg;
  }
  // only incoming registers free, get one (
  RInfo first_reg = regs->register_at(0);
  return first_reg;
}


// returns mapping local_index -> RInfo.number() (also marks if it is oop)
void LocalCaching::assign_registers(CachedLocals* locals_desc, const ScanResult& scan,
				    GrowableArray<ALocal*>* ordered_locals, RInfoCollection* free_regs, Loop* loop) {
  RInfo I_rinfo[SPARC_ARGS_IN_REGS_NUM] = 
                     { FrameMap::_I0_RInfo,
		       FrameMap::_I1_RInfo,
		       FrameMap::_I2_RInfo,
		       FrameMap::_I3_RInfo,
		       FrameMap::_I4_RInfo,
		       FrameMap::_I5_RInfo };
  RInfoCollection* mapping = new RInfoCollection();
  GrowableArray<bool>* oops = new GrowableArray<bool>();
  if (PassParametersInRegisters) {
    const bool only_callee_save = true; // we have some VM leaf calls that we do not account for
                                  // scan.has_calls() || scan.has_class_init() ||
	                          // scan.has_slow_cases() || _ir->method()->has_exception_handlers();
    RInfoCollection* usable_regs = NULL;
    // build a list of registers that can be used for caching in this method 
    // (either all of them or only callee_save registers)
    if (only_callee_save) {
      usable_regs = new RInfoCollection();
      int nn = free_regs->number_of_registers();
      for (int i = 0; i < nn; i++) {
        const RInfo rinfo = free_regs->register_at(i);
        if (!FrameMap::is_caller_save_register(rinfo)) {
          usable_regs->append(rinfo);
        }
      }
    } else {
      usable_regs = free_regs;
    }
    free_regs = NULL; // do not use it
    for (int i = 0; i < ordered_locals->length(); i++) {
      if (ordered_locals->at(i)->access_count() == 0) {
        ordered_locals->trunc_to(i);
        break;
      }
    }
    // usable_regs contains a list of registers that we can use
    int lc = min (ordered_locals->length(), usable_regs->number_of_registers());
    int nof_reg_args = FrameMap::nof_reg_args;
    // GrowableArray<int>* reg_args = new GrowableArray<int>(nof_reg_args, nof_reg_args, -1);
    CallingConvention* args = FrameMap::calling_convention(_ir->method());

    // First walk through the lc locals and assign registers to arguments
    for (int i = 0; i < lc; i++) {
      // 0..lc-1 locals can be cached
      int  index  = ordered_locals->at(i)->index();
      assert(index >= 0, "invalid index");
      int  weight = ordered_locals->at(i)->access_count();
      bool is_oop = ordered_locals->at(i)->is_oop();
      assert(weight != 0, "should have been truncated");
      RInfo rinfo;
      if (index < args->length() && args->arg_at(index).is_register_arg()) {
        // the local at index i is an incoming register argument
        // find the corresponding free I register; if none found
        // use non I register (do not destroy a different incoming argument;
        // Note that in some situation we may not be able to cache an incoming argument
        assert(index < SPARC_ARGS_IN_REGS_NUM, "range check");
        rinfo = get_incoming_at_index(usable_regs, I_rinfo[index]);
      } else {
        rinfo = get_non_incoming_at_index(usable_regs);
      }
      if (!rinfo.is_illegal()) {
        usable_regs->remove(rinfo);
      }
      mapping->at_put(index, rinfo);
      oops->at_put_grow(index, is_oop, false);
    }
  } else { // !PassParemeterInRegisters
    // We assign free registers to locals, starting with the locals at index 0 inside
    // array "ordered_locals" (-> contains local indces ordered by their usage)

    int r = 0;
    int nof_free_regs = free_regs->number_of_registers();

    int   I_regs[SPARC_ARGS_IN_REGS_NUM] = { -1, -1, -1, -1, -1, -1 };
    const bool only_callee_save = true; // we have some VM leaf calls that we do not account for
                                  // scan.has_calls() || scan.has_class_init() ||
	                          // scan.has_slow_cases() || _ir->method()->has_exception_handlers();
    int i;
    for (i = 0; i < ordered_locals->length(); i++) {
      // local - build register pair
      int  index  = ordered_locals->at(i)->index();
      int  weight = ordered_locals->at(i)->access_count();
      bool is_oop = ordered_locals->at(i)->is_oop();

      if (weight > 0) {
        RInfo rinfo;
        for (; r < nof_free_regs; r++) {
	  rinfo = free_regs->register_at(r);
          if (!only_callee_save) {
	  // if (!(scan.has_calls() || scan.has_class_init() ||
	  //       scan.has_slow_cases() || _ir->method()->has_exception_handlers())) {
	    break; // if it's a leaf method then any register is fine
	  }
	  if (!FrameMap::is_caller_save_register(rinfo)) {
	    break; // if it's not a leaf then don't use O's or G's (don't use interpreter special registers either)
	  }
        }
        if (r == nof_free_regs) break; // no more free regs
        r++;
        assert(index >= 0, "invalid index");
        mapping->at_put(index, rinfo);
        oops->at_put_grow(index, is_oop, false);

        Register reg = rinfo.as_register();
        if (reg.is_in()) {
          const int ai = reg.number() - Register::in_base;
          assert(0 <= ai && ai < SPARC_ARGS_IN_REGS_NUM, "range check");
	  I_regs[ai] = index;
        }
      }
    }
    // now make sure that all cached local args which are passed in registers
    // (as by the calling convention) get their incoming register (if available)
    int nof_reg_args = FrameMap::nof_reg_args;
    GrowableArray<int> reg_args = GrowableArray<int>(nof_reg_args, nof_reg_args, -1);
    FrameMap::calling_convention(_ir->method(), &reg_args);
    for (i = 0; i < nof_reg_args; i++) {
      int arg = reg_args.at(i);                           // arg is a java local index
      if (arg != -1) {                                    // 1. register "i" is being used to pass a parameter whose index is arg
        if (free_regs->contains(I_rinfo[i])) {            // 2. register "i" is available for global local caching
	  if (arg < mapping->number_of_registers()) {
	    RInfo other = mapping->register_at(arg);
	    if (other.is_valid()) {                       // 3. arg was cached in register "other"
	      if (I_regs[i] != arg) {                     // 4. but not in register "i"
	        // make sure arg gets cached in register i
	        // if some other local was cached in register i then swap assignments
	        assert(arg >= 0, "must be a valid java local index");
	        mapping->at_put(arg, I_rinfo[i]);        // assign register i to arg
	        if (I_regs[i] != -1) {
		  mapping->at_put(I_regs[i], other);
	        }
	      }
	    }
	  }
        }
      }
    }
  } // !PassParemeterInRegisters

  if (PrintCachedLocals) {
    if (mapping->number_of_registers() > 0) {
      tty->print_cr("   Local Index            Cached in           is oop?");
      for (int i = 0; i < mapping->number_of_registers(); i++) {
	tty->print("       %d                    ", i);
        if (mapping->register_at(i).is_illegal()) {
          tty->print("noreg");
        } else {
          mapping->register_at(i).print();
        }
	tty->print_cr("                  %d", oops->at(i));
      }
    } else {
      tty->print_cr("   No locals were cached");
    }
  }
  locals_desc->set_mapping(mapping);
  locals_desc->set_oops_info(oops);
}


void LocalCaching::add_block_items(const RInfoCollection* mapping, GrowableArray<bool>* oops, c1_RegMask lockout, bool set_32bit_precision) {
  BlockItem* item = new BlockItem(mapping, oops, lockout, set_32bit_precision);
  { BlockItemSetter setter(item);
    _ir->code()->iterate_forward(&setter);
  }
}


void LocalCaching::add_block_items(Loop* loop, const RInfoCollection* mapping, GrowableArray<bool>* oops, bool is_32bit_precision) {
  int n = loop->nof_blocks() - 1;
  BlockItem* item = new BlockItem(mapping, oops, loop->lockout(), is_32bit_precision);
  { BlockItemSetter setter(item);
    loop->iterate_blocks(&setter);
  }
}


void LocalCaching::cache_loop_locals() {
  // CacheLoopLocals is x86 only for now
  ShouldNotReachHere();
}


void LocalCaching::cache_receiver(RegAlloc* reg_alloc) {
  // CacheReceiver does not apply to SPARC
  ShouldNotReachHere();
}


CachedLocals* LocalCaching::cache_locals_globally() {
  CachedLocals* locals_desc = new CachedLocals();
  ScanBlocks scan(_ir->code()->collection());
  ScanResult scan_result;
  scan.scan(&scan_result);
  if (TraceCachedLocals) {
    scan.print(&scan_result);
  }

  // get registers that have not been used and lock them
  // (guarantee that they are never used for temps)
  c1_RegMask available_registers = _ir->compilation()->free_registers();

  RInfoCollection* free_regs = RegAlloc::regmask_2_rinfocollection(available_registers);

  if (TraceCachedLocals) {
    tty->print("  Lockout 0x%x; ");
    available_registers.print();
    tty->cr();
    tty->print("  Free registers: ");
    for (int i = 0; i < free_regs->number_of_registers(); i++) {
      RInfo reg = free_regs->register_at(i);
      reg.print();
      tty->print(" ");
    }
    tty->cr();
  }

  GrowableArray<ALocal*>* locals = NULL;
  locals = scan.most_used_locals();

  // locals contains indeces of locals, sorted by their usage
  // describe oops
  if (TraceCachedLocals) {
    tty->print(" LOCALS BY USAGE COUNT[");
    for (int i = 0; i < locals->length(); i++) {
      tty->print("%d:%d ", locals->at(i)->index(), locals->at(i)->access_count());
    }
    tty->print_cr("]");
  }
  assign_registers(locals_desc, scan_result, locals, free_regs, NULL);
  add_block_items(locals_desc->_mapping, locals_desc->_is_oop, available_registers, false /*set_32bit_precision*/);
  return locals_desc;
}
