/*
 * @(#)c1_CodeEmitter_sparc.cpp	1.140 01/03/26 18:50:41
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_CodeEmitter_sparc.cpp.incl"
#include <math.h>
#include <v9/sys/psr_compat.h>


#define __ _masm->

// Big endian
void PatchingDesc::set_hi_word() { _field_offset = 0; }  // marks move that moves Hi-word
void PatchingDesc::set_lo_word() { _field_offset = 1; }  // marks move that moves Lo-word


void ConstantTable::emit_entries(MacroAssembler* masm) {
  for (int i = 0; i < _entries->length(); i++) {
    FPUConstEntry* fp = _entries->at(i);;
    if (fp->is_double()) {
      masm->align(wordSize*2); // double word alignment
      jlong temp = jlong_cast(fp->double_value());
      fp->set_pc((address)masm->pc());
      masm->a_long(high(temp));
      masm->a_long(low(temp));
    } else {
      masm->align(wordSize); // probably always is
      assert(fp->is_float(), "invalid entry");
      jint temp = jint_cast(fp->float_value());
      fp->set_pc((address)masm->pc());
      masm->a_long(temp);
    }
  }
}


//-----------------------------------------------------
// Specify all constant registers here
const Register RECEIVER = O0; // register where receiver is preloaded

// other constants
const int MAX_OFFSET = (max_jint - 1) / wordSize; // this offset is used for field accesses to
						  // guarantee a 4-word offset size


CodeEmitter::CodeEmitter(
    Compilation* compilation,
    ConstantTable* table,
    JVMCI_OffsetDesc* offsets
  )
  : _compilation(compilation)
  , _masm(compilation->masm())
  , _method(compilation->method())
  , _frame_map(compilation->frame_map())
  , _const_table(table)
  , _offsets(offsets)
  , _esp_offset(0)
  , _need_debug_info(false)
  , _bailout(false)
  , _cc_locked(false)
  , _info(compilation->info())
  , _linenumbers_offset(NULL)
{
  _slow_case_stubs = new GrowableArray<CodeStub*>();
  _call_stubs      = new GrowableArray<CodeStub*>();
}


//------------------------------------------------------
//          helpers
//------------------------------------------------------


void CodeEmitter::trace_method_entry(jint con) {
 // load(frame()->local_address(6, false), O0, T_OBJECT);
  __ mov(O0, L0); // preserve O0
  __ mov(O1, L1); // preserve O1
  __ mov(SP, O0);
  __ set(con, O1);

  __ call_VM_leaf(L5, CAST_FROM_FN_PTR(address, Runtime1::trace_method_entry));
  __ mov(L0, O0);
  __ mov(L1, O1);
}


void CodeEmitter::trace_method_exit() {
}


void CodeEmitter::verify_frame_size() {
#ifdef ASSERT
  if (VerifyC1Code) {
    // make sure SP is unchanged (usually tested after a call)
    int frame_size = frame()->om_framesize() * BytesPerWord;
    if (Assembler::is_simm13(frame_size)) {
      __ sub (FP, frame_size, Gscratch);
    } else {
      __ set(frame_size, Gscratch);
      __ sub (FP, Gscratch, Gscratch);
    }
    __ cmp(Gscratch, SP);
    __ breakpoint_trap(Assembler::notEqual);
  }
#endif // ASSERT
}


//--------------Labels----------------------------------

void CodeEmitter::bind_label(Label* label) {
  __ bind(*label);
}


//---------------------item conversion--------------------------



FloatRegister CodeEmitter::item2floatreg(Item* item) {
  assert(item->is_register() && (item->type()->is_float() || item->type()->is_double()), "type error");
  if (item->type()->is_float()) return item->get_register().as_float_reg();
  else return item->get_register().as_double_reg();
}


Address CodeEmitter::item2addr_with_offset(Item* item, int offset) {
  assert(item->is_register() && item->type()->is_single_word() && item->type()->is_object(), "cannot handle other cases");
  Register reg = item2reg(item);
  assert(reg != SP && reg != FP, "address must be in heap, not stack");
  return Address(reg, 0, offset*wordSize);
}


Address CodeEmitter::item2addr_with_offsetLO(Item* item, int offset) {
  // sparc is big endian!
  return item2addr_with_offset(item,offset+1);
}


Address CodeEmitter::item2addr_with_offsetHI(Item* item, int offset) { 
  // sparc is big endian!
  return item2addr_with_offset(item,offset);
}


//-----------------------------------------------------------------------


Address CodeEmitter::item2array_addr(Item* array, Item* index, BasicType dst_type, int offset) {
//   assert(array->is_register() && index->is_register(), "items in wrong form");
//   Address::ScaleFactor factor = array_element_size(dst_type);
//   Register array_reg = array->get_register().as_register();
//   Register index_reg = index->get_register().as_register();
//   return Address(array_reg, index_reg, factor, arrayOopDesc::base_offset_in_bytes(dst_type) + offset);
  Unimplemented();
  return Address();
}


Address CodeEmitter::item2array_addrLO(Item* array, Item* index, BasicType dst_type) {
  return item2array_addr(array, index, dst_type);
}


Address CodeEmitter::item2array_addrHI(Item* array, Item* index, BasicType dst_type) {
  return item2array_addr(array, index, dst_type, wordSize);
}


jobject CodeEmitter::item2jobject(Item* item) {
  assert(item->is_constant() && item->type()->is_object(), "item is not an object constant");
  return item->get_jobject_constant()->encoding();
}


jint CodeEmitter::item2int(Item* item) {
  return item->get_jint_constant();
}


jint CodeEmitter::item2intLO(Item* item) {
  assert(item->is_constant() && item->type()->is_double_word(), "wrong item");
  jint res = low(item->get_jint_constant());
  return res;
}

jint CodeEmitter::item2intHI(Item* item) {
  assert(item->is_constant() && item->type()->is_double_word(), "wrong item");
  jint res = high(item->get_jint_constant());
  return res;
}


// inline cache check
void CodeEmitter::check_icache() {
  Label L;
  const Register recv = O0;
  const Register klass = Ginline_cache_reg;
  const Register temp_reg = Gscratch;
  // Note: needs more testing of out-of-line vs. inline slow case
  if (true) {
    Address ic_miss(temp_reg, Runtime1::entry_for(Runtime1::handle_ic_miss_id));
    __ verify_oop(recv);
    __ ld(recv, oopDesc::klass_offset_in_bytes(), temp_reg);
    __ cmp(temp_reg, klass);
    __ br(Assembler::equal, true, Assembler::pt, L);
    __ delayed()->nop();
    __ jump_to(ic_miss, 0);
    __ delayed()->nop();
    __ align(CodeEntryAlignment);
    __ bind(L);
  } else {
    // receiver is already in O0
    ICacheStub* stub = new ICacheStub();
    _slow_case_stubs->append(stub);
    __ verify_oop(recv);
    __ ld(recv, oopDesc::klass_offset_in_bytes(), temp_reg);
    __ cmp(temp_reg, klass);
    __ br(Assembler::notEqual, false, Assembler::pn, *stub->entry());
    __ delayed()->nop();
  }
}


// Uses G1
void CodeEmitter::interpreter_to_compiler_calling_convention() {
  // Gmethod: method to call
  // Gargs  : last argument
  //
  // Copy arguments if any: Copy last argument first
  // since parameter area may overlap. Use O0 as tmp
  // register, this way the receiver ends up there
  // automatically (receiver caching) since it is copied
  // last.
  //
  // Note: If there are no arguments we should not even
  //       go here but jump to the verified entry point
  //       right away. Fix this eventually.
  if (PassParametersInRegisters) {
    const Register value = O0;
    assert_different_registers(Gmethod, Gargs, value);
    const CallingConvention* cc = FrameMap::calling_convention(method());
    const int n = method()->arg_size();
    for (int i = 0; i < n; i++) {
      ArgumentLocation loc = cc->arg_at(n - 1 - i);
      if (loc.is_register_arg()) {
        __ ld(Gargs, i*BytesPerWord, loc.outgoing_reg_location().as_register());
      } else {
        __ ld(Gargs, i*BytesPerWord, value);
        __ st(value, SP, (frame::memory_parameter_word_sp_offset + i) * BytesPerWord);
      }
    }
  } else {
    const Register value = O0;
    assert_different_registers(Gmethod, Gargs, value);
    const int n = method()->arg_size();
    for (int i = 0; i < n; i++) {
      __ ld(Gargs, i*BytesPerWord, value);
      __ st(value, SP, (frame::memory_parameter_word_sp_offset + i) * BytesPerWord);
    }
  }
  // branch to verified entry point
  __ br(Assembler::always, false, Assembler::pt, _vep_label);
  __ delayed()->nop();
}


void CodeEmitter::build_frame(int extra_words) {
  assert(extra_words == 0, "extra_words argument is unused by build_frame() on SPARC");
  if (method()->is_native()) {
    // A standard C1 frame already includes space for the SPARC ABI's register-save area, accommodating
    // a fixed number of actual parameters.  To pass more, extra frame space is required.  Also, two words
    // are needed for saving a native floating-point result around VM calls in this method's compiled code.
    // The extra words are allocated from the spill area, so the spill area must immediately precede the
    // SPARC ABI words in the frame.
    int fp_result_spill_words;
    switch (method()->return_type()) {
      case T_FLOAT : fp_result_spill_words = 1; break;
      case T_DOUBLE: fp_result_spill_words = 2; break;
      default      : fp_result_spill_words = 0; break;
    }
    NEEDS_CLEANUP // make NativeSignatureIterator export "extra JNI arg words"
    int extra_arg_words = method()->is_static() ? 2 : 1;
    extra_arg_words = max(extra_arg_words + frame()->size_arguments() - Argument::n_register_parameters, 0);
    frame()->add_spill_slots(extra_arg_words + fp_result_spill_words);
  }
  int frame_words = round_to(frame()->om_framesize() + extra_words, 2);
  int frame_size = frame_words * BytesPerWord;
  assert(frame_size > 0, "just checking");
  // We want to bang if our frame is larger than a given amount, or
  // if any synchronization takes place in this method. When we switch
  // to rollforward exception installation, we will no longer need to
  // bang for methods with synchronization.
  bool has_synchronization = method()->is_synchronized() || method()->has_monitor_bytecodes();
  if (frame_size >= os::max_framesize_without_stack_banging() || has_synchronization || method()->is_native()) {
    // Stack banging code to check for stack overflows. We use Gscratch here, this
    // should be the same register used by the stack banging code in the interpreter
    int bang_offset = 0;
    while (bang_offset < frame_size) {
      // We always bang at least once
      bang_offset += os::vm_page_size();
      bang_stack_with_offset(-bang_offset);
    }
    if (has_synchronization) {
      // if the method is synchronized bang one extra page. Otherwise, it
      // is possible that the above bangs do not hit the yellow zone but
      // the monitorenter slow does 
      bang_offset += os::vm_page_size();
      bang_stack_with_offset(-bang_offset);
    }
  }
  // Note: Both om_framesize() and save_frame() add space for the SPARC ABI register-save area;
  // save_frame will re-round up to frame_words.
  NEEDS_CLEANUP
  __ save_frame(frame_words - frame::memory_parameter_word_sp_offset);
}


void CodeEmitter::restore_frame(Register scratch) {
  if (UseC2CallingConventions) {
    int frame_size = frame()->om_framesize() * BytesPerWord;
    if (Assembler::is_simm13(frame_size)) {
      __ sub (FP, frame_size, SP);
    } else {
      assert_different_registers(scratch, SP, FP);
      __ set(frame_size, scratch);
      __ sub (FP, scratch, SP);
    }
  }
  verify_frame_size();
}


void CodeEmitter::init_local(int local_index) {
  const CachedLocals* cached_locals = frame()->cached_locals();
  RInfo reg = cached_locals != NULL ? cached_locals->get_cache_reg(local_index) : FrameMap::_noRInfo; // where parameter n is cached
  if (reg.is_illegal()) { 
    // Note: local initialization happens before the locals are cached
    //       and before synchronization happens
    __ st (G0, frame()->local_address(local_index, false)); // this is done only for possible oop locations
  } else {
    __ clr(reg.as_register()) ; 
  }
}


void CodeEmitter::implicit_null_check (Item* item, CodeEmitInfo* info) {
  if (GenerateCompilerNullChecks) {
    add_debug_info_here(info);
    __ null_check(item2reg(item));
  }
}


// call after frame has been built
void CodeEmitter::load_receiver(Register reg) {
  load(frame()->local_address(0, false), reg, T_OBJECT);
}


void CodeEmitter::swap_reg(Register a, Register b) {
  Unimplemented();
}


void CodeEmitter::try_fast_ObjectHashCode() {

  // For the native method Object.hashCode(), if the object's header can be read
  // while the object is unlocked, and it holds a valid (non-zero) hash code,
  // return the hash value immediately without creating a frame.

  Register receiver = O0;     // guaranteed
  Register header   = Gtemp;
  Register hash     = Gtemp;  // overwrite header register with hash value
  Register mask     = Gtemp2; // for hash field of header
  Label SlowerPath;

  // Read the header and build a mask to get its hash field.  Give up if the object is not unlocked.
  __ ld(receiver, oopDesc::mark_offset_in_bytes(), header);
  __ sethi(markOopDesc::hash_mask_in_place, mask);
  __ btst(markOopDesc::unlocked_value, header);
  __ br(Assembler::zero, false, Assembler::pn, SlowerPath);
  __ delayed()->or3(mask, _masm->low10(markOopDesc::hash_mask_in_place), mask);

  // Check for a valid (non-zero) hash code and get its value.
  __ and3(header, mask, hash);
  __ cmp(hash, G0);
  __ br(Assembler::equal, false, Assembler::pn, SlowerPath);
  __ delayed()->srl(hash, markOopDesc::hash_shift, hash);

  // Safepoint leaf return.
  __ mov(hash, O0);
  __ relocate(relocInfo::return_type);
  __ retl();
  __ delayed()->nop();
  __ nop();

  __ bind(SlowerPath);
}


// Uses G3 and G4
void CodeEmitter::method_entry(GrowableArray<int>* init_vars, int max_spills) {

  // entry point
  __ align(CodeEntryAlignment);
  _offsets->_ep_offset = __ offset();
  if (needs_icache()) {
    // Note: even if this is a final method, we may need IC check
    // as the final method may be one of many methods in the hierarchu
    if (C1Breakpoint)  __ breakpoint_trap();
    check_icache();
  }

  // verified entry point
  _offsets->_vep_offset = __ offset();
  __ bind (_vep_label);
  if (C1Breakpoint) __ breakpoint_trap();

  // Note: every call has a null check in delayed slot; therefore no null check is needed here.
  assert(!needs_null_check_on_entry(), "on sparc, we always do null check at the call site");

  _offsets->_code_offset = __ offset();
  __ verify_FPU(0, "method_entry");

  // The code for Object.hashCode() can be frameless for many receivers.
  if (method()->intrinsic_id() == methodOopDesc::_hash) {
    if (!compilation()->jvmpi_event_method_enabled()) try_fast_ObjectHashCode();
  }

  // Create an activation frame for this method.  All components of the frame must be recorded in a FrameMap,
  // and only FrameMap::om_framesize() is used to compute the frame's size.  On SPARC, build_frame() doesn't
  // use its argument, nor is this function's "max_spills" argument used.
  build_frame(0);

  if (C1TraceMethod) trace_method_entry(0);

  // Initialize oop locals before synchronizing since that may block for GC.
  if (init_vars != NULL) {
    int i = init_vars->length();
    while (i-- > 0) init_local(init_vars->at(i));
  }

  CallingConvention* args = FrameMap::calling_convention(method());

  if (method()->is_synchronized() && GenerateSynchronizationCode) {
    Register obj; // object to lock
    if (method()->is_static()) {
      obj = L0;
      constant_2_reg(method()->holder()->java_mirror(), FrameMap::_L0_RInfo);
    } else {
      ArgumentLocation receiver = args->arg_at(0);
      if (receiver.is_register_arg()) {
	obj = receiver.incoming_reg_location().as_register();
      } else {
	obj = I0;
	load(receiver.incoming_stack_location(), obj, T_OBJECT);
      }
    }
    Register lock_reg = L1;
    CodeEmitInfo info(SynchronizationEntryBCI, NULL, NULL, NULL);
    monitorenter(obj, lock_reg, G4, G3, 0, &info); // sync. slot is 0, however should really use a function here
  }

  if (compilation()->jvmpi_event_method_entry_enabled()  ||
      compilation()->jvmpi_event_method_entry2_enabled()) {
    // we need the map for arguments only, therefore we use
    // SynchronizationEntryBCI, although there may be no synchronization in
    // this method
    CodeEmitInfo info(SynchronizationEntryBCI, NULL, NULL);
    jvmpi_method_entry(&info);
  }

  if (!method()->is_native()) {
    setup_locals(args);
  }
}


// Note: this is used when passing arguments in registers and when caching locals
// Situations:
//    - parameter on stack, local is cached -> move from stack location to register
void CodeEmitter::setup_locals(CallingConvention* args) {
  GrowableArray<BasicType>* sig_types = FrameMap::signature_type_array_for(method());
  const CachedLocals* cached_locals = frame()->cached_locals();
  for (int n = 0; n < args->length(); n++) {
    ArgumentLocation loc = args->arg_at(n); // this is were parameter n goes according to the calling convention
    RInfo cache_reg = cached_locals != NULL ? cached_locals->get_cache_reg(n) : norinfo; // where parameter n is cached
    if (!cache_reg.is_illegal()) {
      // local n is cached in cache_reg
      assert(cache_reg.is_word(), "cannot handle others yet");
      if (loc.is_register_arg()) {
	// it is being passed in an I register
        // it is possible that the incoming parameter is cached in a different 
        // register if that register is not available for caching.
        // The register where the incoming parameter is cached, may not
        // be used for any other parameter, otherwise the register will be destroyed
        /* tty->print ("%d Setup: ", n);
        loc.incoming_reg_location().print();
        tty->print(" -> ");
        cache_reg.print();
        tty->cr(); */
	__ mov(loc.incoming_reg_location().as_register(), cache_reg.as_register());
      } else {
	// it is being passed on the stack
	assert(loc.is_stack_arg(), "invalid mapping of args");
	//assert(loc.stack_offset_in_words() == frame()->local_offset_from_fp(n), "invalid mapping of args");
	local_2_reg(cache_reg, n, sig_types->at(n));
        if (C1InvalidateCachedOopLocation) {
          __ sub (G0, 1, G1);
          // store -1 in local location making that location invalid instead of
          // having a stale copy there
          __ st (G1, frame()->local_address(n, false)); 
        }
      }
    } else {
      // local n is not cached
      if (loc.is_register_arg()) {
	// it is being passed in an I register, store it to its stack location
	reg_2_local(n, loc.incoming_reg_location(), sig_types->at(n));
      } else {
	// it is being passed on the stack, should be in the right location already
	assert(loc.is_stack_arg(), "invalid mapping of args");
	//assert(loc.stack_offset_in_words() == frame()->local_offset_from_fp(n), "invalid mapping of args");
      }
    }
  }
}

 
void CodeEmitter::pass_oop_to_native(int java_arg_index, Address java_arg_addr, Argument jni_arg) {
  // If the outgoing oop is null, pass null, otherwise pass its handle.  For a nonstatic method, the
  // receiver is known to be nonnull, since the caller must check for a NullPointerException.)
  bool check_for_null = method()->is_static() || java_arg_index != 0;
  Register jni_reg = jni_arg.is_register() ? jni_arg.as_register() : Gscratch2;
  if (check_for_null) {
    Label L;
    __ ld(java_arg_addr, Gscratch);
    __ tst(Gscratch);
    __ br(Assembler::notZero, true, Assembler::pt, L);
    __ delayed()->add(java_arg_addr, jni_reg);
    __ mov(G0, jni_reg);
    __ bind(L);
  } else {
    __ add(java_arg_addr, jni_reg);
  }
  if (!jni_arg.is_register()) {
    __ st(jni_reg, jni_arg.as_address());
  }
}


void CodeEmitter::native_call (address native_entry, CodeEmitInfo* info) {
  // Pass JNIEnv
  __ verify_thread();
  __ add(Gthread, JavaThread::jni_environment_offset_in_bytes(), O0);
  int jni_offset = 1;

  // Store a static method holder's mirror in this frame and pass its address.
  // It is "known", and will be reflected in oop maps, that spill slot 0 is used for the mirror.
  NEEDS_CLEANUP
  if (method()->is_static()) {
    int mirror_offset = frame()->spill_offset_from_fp(0, false, false) * BytesPerWord;
    constant_2_reg(method()->holder()->java_mirror(), FrameMap::_L0_RInfo);
    __ add(FP, mirror_offset, O1);
    __ st(L0, FP, mirror_offset);
    jni_offset++;
  }

  // Pass the method's parameters to its native implementation.
  GrowableArray<BasicType>* sig_types = FrameMap::signature_type_array_for(method());
  CallingConvention* args = FrameMap::calling_convention(method()->is_static(), *sig_types);

  int java_arg_index = 0;
  while (java_arg_index < args->length()) {
    BasicType arg_type = sig_types->at(java_arg_index);
    switch (arg_type) {
    case T_OBJECT: // fall through
    case T_ARRAY:
      {
	Argument jni_arg(java_arg_index + jni_offset, false);     // Assembler abstraction; obeys SPARC ABI
	ArgumentLocation java_arg = args->arg_at(java_arg_index); // C1 abstraction for method argument
	if (java_arg.is_register_arg()) {
	  // Flush this argument to its stack location, which is used for a handle.
	  Register java_arg_reg = java_arg.incoming_reg_location().as_register();
	  Address java_arg_addr = frame()->local_address(java_arg_index, false);
	  store(java_arg_reg, java_arg_addr, T_OBJECT);
	  pass_oop_to_native(java_arg_index, java_arg_addr, jni_arg);
	} else {
	  assert(java_arg.is_stack_arg(), "invalid mapping of args");
	  Address java_arg_addr = java_arg.incoming_stack_location();
	  pass_oop_to_native(java_arg_index, java_arg_addr, jni_arg);
	}
      }
      break;
    case T_INT:       // fall through
    case T_BOOLEAN:   // fall through
    case T_CHAR:      // fall through
    case T_BYTE:      // fall through
    case T_SHORT:     // fall through
    case T_FLOAT:
      {
	Argument jni_arg(java_arg_index + jni_offset, false);     // Assembler abstraction; obeys SPARC ABI
	ArgumentLocation java_arg = args->arg_at(java_arg_index); // C1 abstraction for method argument
	if (java_arg.is_register_arg()) {
	  Register java_arg_reg = java_arg.incoming_reg_location().as_register();
	  __ store_argument(java_arg_reg, jni_arg);
	} else {
	  assert(java_arg.is_stack_arg(), "invalid mapping of args");
	  Address java_arg_addr = java_arg.incoming_stack_location();
	  if (jni_arg.is_register()) {
	    load(java_arg_addr, jni_arg.as_register(), T_INT);
	  } else {
	    load(java_arg_addr, Gscratch, T_INT);
	    __ store_argument(Gscratch, jni_arg);
	  }
	}
      }
      break;
    case T_LONG:    // fall through
    case T_DOUBLE:
      {
	ArgumentLocation java_arg_hi = args->arg_at(java_arg_index+1);
	ArgumentLocation java_arg_lo = args->arg_at(java_arg_index+0);
	Argument jni_arg_hi(java_arg_index+1 + jni_offset, false);     // Assembler abstraction; obeys SPARC ABI
	Argument jni_arg_lo(java_arg_index+0 + jni_offset, false);     // Assembler abstraction; obeys SPARC ABI
	Address java_arg_addr_hi = java_arg_hi.incoming_stack_location();
	Address java_arg_addr_lo = java_arg_lo.incoming_stack_location();
	if (jni_arg_hi.is_register()) {
	  load(java_arg_addr_hi, jni_arg_hi.as_register(), T_INT);
	} else {
	  load(java_arg_addr_hi, Gscratch, T_INT);
	  __ store_argument(Gscratch, jni_arg_hi);
	}
	if (jni_arg_lo.is_register()) {
	  load(java_arg_addr_lo, jni_arg_lo.as_register(), T_INT);
	} else {
	  load(java_arg_addr_lo, Gscratch, T_INT);
	  __ store_argument(Gscratch, jni_arg_lo);
	}
      }
      break;
    default:
      ShouldNotReachHere();
      break;
    }
    java_arg_index +=type2size[arg_type];  // either one word or two words
  }

  // Reset local handle block.
  __ ld(Gthread, JavaThread::active_handles_offset_in_bytes(), Gscratch);
  __ st(G0, Gscratch, JNIHandleBlock::top_offset_in_bytes());

  // Save the last Java frame - SP and a PC located somewhere within the method - before
  // entering native.  To get a PC, "call" the next instruction and use the delay slot to
  // encode the "is_pc" flag in the call's return address.  If this frame is scanned by
  // GC before the native function is called, an oop map is needed at this pc reflecting
  // this method's formal parameters and, if it is static, its holder's mirror.
  { Label L;
    add_call_info_here(info);
    __ call(L, relocInfo::none);  // No relocation for call to pc+0x8
    __ delayed()->add(O7, JavaThread::is_pc, Gscratch);
    __ bind(L);
  }
  __ set_last_Java_frame(SP, Gscratch);

  // Transition from _thread_in_Java to _thread_in_native.
  __ set(_thread_in_native, Gscratch);
  __ st(Gscratch, Gthread, JavaThread::thread_state_offset_in_bytes());

  // Save the current thread and call the native function.  An oop map is needed here
  // in case this thread is stopped at the return address.
  __ call(native_entry, relocInfo::runtime_call_type);
  __ delayed()->mov(Gthread, L0);
  add_call_info_here(info);
}


void CodeEmitter::native_method_exit (CodeEmitInfo* info) {
  // Process the native function result: An integer is converted to its Java type and moved to
  // register I0 (and I1 for long) in preparation for restore(), and for a floating-point
  // result, offsets from the frame pointer are computed for saving the result around VM calls.
  int float_spill_offset;
  int float_spill_index = method()->is_static() ? 1 : 0; // for method holder's mirror
  BasicType return_type = method()->return_type();
  switch (return_type) {
    case T_BOOLEAN: __ subcc(G0, O0, G0); __ addc(G0, 0, I0); break; // !0 => true; 0 => false
    case T_CHAR   : __ sll(O0, 16, O0); __ srl(O0, 16, I0);   break; // clear unused bits
    case T_BYTE   : __ sll(O0, 24, O0); __ sra(O0, 24, I0);   break; // sign-extend
    case T_SHORT  : __ sll(O0, 16, O0); __ sra(O0, 16, I0);   break; // sign-extend
    case T_ARRAY  :                                           // fall through (result is a handle)
    case T_OBJECT :                                           // fall through (result is a handle)
    case T_INT    : __ mov(O0, I0);                           break;
    case T_LONG   : __ mov(O0, I0); __ mov(O1, I1);           break;
    case T_VOID   :                                           break;
    case T_DOUBLE :
      float_spill_offset = frame()->spill_offset_from_fp(float_spill_index, true,  true ) * BytesPerWord; break;
    case T_FLOAT  :
      float_spill_offset = frame()->spill_offset_from_fp(float_spill_index, false, false) * BytesPerWord; break;
    default       : ShouldNotReachHere();
  }

  // Change state to _thread_in_native_trans in order to safely poll and then block
  // for a pending GC (thread was saved in L0 before the call to native code.)
  Register Lthread = L0;
  __ set(_thread_in_native_trans, Gscratch);
  __ st(Gscratch, Lthread, JavaThread::thread_state_offset_in_bytes());
  { Label no_block;
    Address sync_state(Gscratch, SafepointSynchronize::address_of_state());
    __ load_contents(sync_state, Gscratch);
    __ cmp(Gscratch, SafepointSynchronize::_not_synchronized);
    __ br(Assembler::equal, false, Assembler::pt, no_block);
    __ delayed()->mov(Lthread, Gthread); // Restore global thread pointer

    // Before blocking, preserve a floating-point result; an integer result is safe in I0/I1
    switch (return_type) {
      case T_DOUBLE: __ stf(FloatRegister::S, F0, FP, float_spill_offset+wordSize);
                     __ stf(FloatRegister::S, F1, FP, float_spill_offset); break;
      case T_FLOAT : __ stf(FloatRegister::S, F0, FP, float_spill_offset); break;
      default      : /* do nothing */                                      break;
    }
    __ call(CAST_FROM_FN_PTR(address, SafepointSynchronize::block));
    __ delayed()->mov(Lthread, O0); 
    add_call_info_here(info);

    // Restore floating-point result
    switch (return_type) {
      case T_DOUBLE: __ ldf(FloatRegister::S, FP, float_spill_offset+wordSize, F0);
                     __ ldf(FloatRegister::S, FP, float_spill_offset,          F1); break;
      case T_FLOAT : __ ldf(FloatRegister::S, FP, float_spill_offset, F0); break;
      default      : /* do nothing */                                      break;
    }

    // Restore global thread pointer
    __ mov(Lthread, Gthread);
    __ bind(no_block);
  }

  // Change state to _thread_in_Java.
  __ set(_thread_in_Java, Gscratch);
  __ st(Gscratch, Lthread, JavaThread::thread_state_offset_in_bytes());

  // Clear "last Java frame" SP and PC.
  __ verify_thread(); // Gthread must be correct
  __ reset_last_Java_frame();

  // Handle exception (will unlock if necessary)
  { Label L;
    __ ld(Gthread, Thread::pending_exception_offset_in_bytes(), Gscratch);
    __ tst(Gscratch);
    __ br(Assembler::equal, false, Assembler::pt, L);
    __ delayed()->nop();
    assert(StubRoutines::forward_exception_entry() != NULL, "must be created");
    __ call(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);
    __ delayed()->nop();
    __ bind(L);
  }

  // Unlock
  if (method()->is_synchronized()) {
    // Preserve double floating-point result through monitor exit code; integer result is safe in I0/I1
    switch (return_type) {
      case T_DOUBLE: __ stf(FloatRegister::S, F0, FP, float_spill_offset+wordSize);
                     __ stf(FloatRegister::S, F1, FP, float_spill_offset); break;
      case T_FLOAT : __ stf(FloatRegister::S, F0, FP, float_spill_offset); break;
      default      : /* do nothing */                                      break;
    }
    RInfo invalid_receiver_reg; // causes receiver to be loaded from memory
    return_op_prolog(0, invalid_receiver_reg);
    switch (return_type) {
      case T_DOUBLE: __ ldf(FloatRegister::S, FP, float_spill_offset+wordSize, F0);
                     __ ldf(FloatRegister::S, FP, float_spill_offset,          F1); break;
      case T_FLOAT : __ ldf(FloatRegister::S, FP, float_spill_offset, F0); break;
      default      : /* do nothing */                                      break;
    }
  }

  // Unpack oop result
  if (return_type == T_OBJECT || return_type == T_ARRAY) {
      Label L;
      __ addcc(G0, I0, G0);
      __ br(Assembler::notZero, true, Assembler::pt, L);
      __ delayed()->ld(I0, 0, I0);
      __ mov(G0, I0);
      __ bind(L);
      __ verify_oop(I0);
  }

  // Return
  __ ret();
  __ delayed()->restore();
}


void CodeEmitter::spill (int spill_ix, Item* item ) {
  assert(item->is_register() , "why spill if not a register?");
  ValueTag tag = item->type()->tag();
  if (tag == floatTag) {
    Address dst_addr = frame()->spill_address(spill_ix, false);
    store(item2floatreg(item), dst_addr, T_FLOAT);
  } else if (tag == doubleTag) {
    Address dst_addrHI = frame()->spill_address(spill_ix, true, true);
    Address dst_addrLO = frame()->spill_address(spill_ix, true, false);
    // may be misaligned
    store(item2floatreg(item),             dst_addrHI, T_FLOAT);
    store(item2floatreg(item).successor(), dst_addrLO, T_FLOAT);
  } else if (tag == longTag) {     
    Address dstLO = frame()->spill_address(spill_ix, true, false);
    Address dstHI = frame()->spill_address(spill_ix, true, true);
    __ st (item2regLO(item), dstLO);
    __ st (item2regHI(item), dstHI);
  } else {                  
    Address dst = frame()->spill_address(spill_ix, false);
    __ st (item2reg(item), dst);
  }                         
}


void CodeEmitter::monitorenter(Register obj_reg, Register lock_reg, Register hdr, Register scratch, int monitor_no, CodeEmitInfo* info) {
  if (!GenerateSynchronizationCode) return;
  bang_stack();
  Address mon_addr = frame()->monitor_address(monitor_no);
  Register reg = mon_addr.base();
  int offset = mon_addr.disp();
  // compute pointer to BasicLock
  if (mon_addr.is_simm13()) {
    __ add(reg, offset, lock_reg);
  } else {
    __ set(offset, lock_reg);
    __ add(reg, lock_reg, lock_reg);
  }
  __ verify_oop(obj_reg);

  // lock object
  MonitorAccessStub* slow_case = new MonitorEnterStub(obj_reg, lock_reg, info);
  _slow_case_stubs->append(slow_case);
  if (UseFastLocking) {
    // try inlined fast locking first, revert to slow locking if it fails
    // note: lock_reg points to the displaced header since the displaced header offset is 0!
    assert(BasicLock::displaced_header_offset_in_bytes() == 0, "lock_reg must point to the displaced header");
    add_debug_info_here(info);
    __ lock_object(hdr, obj_reg, lock_reg, scratch, *slow_case->entry());
  } else {         
    // always do slow locking
    // note: the slow locking code could be inlined here, however if we use
    //       slow locking, speed doesn't matter anyway and this solution is
    //       simpler and requires less duplicated code - additionally, the
    //       slow locking code is the same in either case which simplifies
    //       debugging
    __ br(Assembler::always, false, Assembler::pt, *slow_case->entry());
    __ delayed()->nop();
  }                
  // done          
  __ bind(*slow_case->continuation());
}


void CodeEmitter::monitorexit(Register obj_reg, Register lock_reg, Register hdr, int monitor_no) {
  if (!GenerateSynchronizationCode) return;
  Address mon_addr = frame()->monitor_address(monitor_no);
  Register reg = mon_addr.base();
  int offset = mon_addr.disp();
  // compute pointer to BasicLock
  if (mon_addr.is_simm13()) {
    __ add(reg, offset, lock_reg);
  }
  else {
    __ set(offset, lock_reg);
    __ add(reg, lock_reg, lock_reg);
  }
  __ verify_oop(obj_reg);
  // unlock object
  MonitorAccessStub* slow_case = new MonitorExitStub(obj_reg, lock_reg, UseFastLocking, mon_addr);
  // _slow_case_stubs->append(slow_case);
  // temporary fix: must be created after exceptionhandler, therefore as call stub
  _call_stubs->append(slow_case);
  if (UseFastLocking) {
    // try inlined fast unlocking first, revert to slow locking if it fails
    // note: lock_reg points to the displaced header since the displaced header offset is 0!
    assert(BasicLock::displaced_header_offset_in_bytes() == 0, "lock_reg must point to the displaced header");
    __ unlock_object(hdr, obj_reg, lock_reg, *slow_case->entry());
  } else {         
    // always do slow unlocking
    // note: the slow unlocking code could be inlined here, however if we use
    //       slow unlocking, speed doesn't matter anyway and this solution is
    //       simpler and requires less duplicated code - additionally, the
    //       slow unlocking code is the same in either case which simplifies
    //       debugging
    __ br(Assembler::always, false, Assembler::pt, *slow_case->entry());
    __ delayed()->nop();
  }                
  // done          
  __ bind(*slow_case->continuation());
}


// Helper for load_item_force
// 
// load a floating point stackitem into a non-fp register OR
// load a non floating point stackitem into a fp register
void CodeEmitter::untyped_stackitem_2_reg (RInfo reg, const Item* item) {
  assert(item->is_stack(), "item must be on the stack to do untyped load");
  ValueTag tag = item->type()->tag();
  switch (tag) {
  case floatTag: 
    assert(reg.is_word(), "check");
    load(stack_item2addr(item), reg.as_register(), T_INT);
    break;

  case doubleTag: 
    assert(reg.is_long(), "check");
    load(stack_item2addrHI(item), reg.as_register_hi(), T_INT);
    load(stack_item2addrLO(item), reg.as_register_lo(), T_INT);
    break;

  case longTag: 
    assert(reg.is_double(), "check");
    load(stack_item2addrHI(item), reg.as_double_reg(),             T_FLOAT);
    load(stack_item2addrLO(item), reg.as_double_reg().successor(), T_FLOAT);
    break;

  default: 
    assert(item->type()->is_single_word(), "check");
    assert(reg.is_float(), "check");
    load(stack_item2addr(item), reg.as_float_reg(), T_FLOAT);
  }
}


void CodeEmitter::nops_for_zombie() {
  if ( (int)__ offset() == _offsets->_vep_offset) {
    // make sure that we do not emit oop where a non-entrant patching may happen
    __ nop(); 
  }
}

// obj represents a class.  If it is NULL we need to patch the sethi/add and insert the class at runtime.
void CodeEmitter::jobject_2_reg_with_patching(RInfo reg, ciObject* obj, CodeEmitInfo* info) {
  if (PatchALot || !(obj->is_loaded())) {
    PatchingDesc patch;
    nops_for_zombie();
    set_patching_desc(&patch);
    Register dest = reg.as_register();

    // It may not seem necessary to use a sethi/add pair to load a NULL into dest, but the
    // NULL will be dynamically patched later and the patched value may be large.  We must
    // therefore generate the sethi/nop/add as placeholders (the nop is required for
    // patching - it goes in the delay slot of the call to the patching routine.

    int oop_index = __ oop_recorder()->allocate_index(NULL);
    Address addr = Address(dest, address(NULL), oop_Relocation::spec(oop_index));
    assert(addr.rspec().type() == relocInfo::oop_type, "must be an oop reloc");
    __ sethi(addr);
    __ nop();
    __ add(addr, dest, 0);
    add_call_info(patch.pc_offset() + NativeCall::instruction_size, info);
    patching_epilog(&patch, info, PatchingStub::load_klass_id);
  } else {
    // no patching needed
    constant_2_reg(obj, reg);
  } 
}


void CodeEmitter::item_2_reg_with_patching(RInfo reg, Item* klass_item, CodeEmitInfo* info) {
  assert(klass_item->type()->is_object() && klass_item->is_constant(), "wrong item");
  // We expect that the next instruction will be patched and will be responsible for
  // fixing up the move.
  jobject_2_reg_with_patching(reg, klass_item->get_jobject_constant(), info);
}


// Only case for register currently needed
void CodeEmitter::reg_2_item(Item* item, RInfo reg) {
  ValueTag tag = item->type()->tag();
  if (tag == floatTag)  {
    Unimplemented();
  } else if (tag == doubleTag)  {
    Unimplemented();
  } else if (tag == longTag) {
    Unimplemented();
  } else {
    assert(item->type()->is_single_word(), "must be");
    Register r = reg.as_register();
    if (item->is_register()) {
      __ mov(r, item2reg(item));
    } else {
      Unimplemented();
    }
  }
}


// load an item into a compatible register.
// assert if the two are incompatible (wrong size or type).
void CodeEmitter::item_2_reg(RInfo reg, Item* item) {
  bool is_item_fp = item->type()->is_float() || item->type()->is_double();
  bool is_reg_fp  = reg.is_float() || reg.is_double();
  if (item->is_constant()) {
    constant_2_reg(item->type(), reg);
    return;
  }
  if (is_item_fp != is_reg_fp) {
    assert(item->is_stack(), "can only do untyped load from the stack");
    untyped_stackitem_2_reg(reg, item);
    return;
  }

  ValueTag tag = item->type()->tag();
  if (tag == floatTag)  {
    if (item->is_register()) {
      __ fmov(FloatRegister::S, item2floatreg(item), reg.as_float_reg());
    } else if (item->is_stack()) {
      load( stack_item2addr(item), reg.as_float_reg(), T_FLOAT);
    } else {
      ShouldNotReachHere();
    }
  } else if (tag == doubleTag)  {
    if (item->is_register()) {
      __ fmov(FloatRegister::D, item2floatreg(item), reg.as_double_reg());
    } else if (item->is_stack()) {
      // may be misaligned
      load(stack_item2addrHI(item), reg.as_double_reg(),             T_FLOAT);
      load(stack_item2addrLO(item), reg.as_double_reg().successor(), T_FLOAT);
    } else {
      ShouldNotReachHere();
    }
  } else if (tag == longTag) {
    Register rlo = reg.as_register_lo();
    Register rhi = reg.as_register_hi();
    if (item->is_register()) {
      if (item2regHI(item) == rlo) {
        // do not overwrite item register first
        assert(item2regLO(item) != rhi, "wrong regalloc");
        __ mov(item2regHI(item), rhi);
        __ mov(item2regLO(item), rlo);
      } else {
        __ mov(item2regLO(item), rlo);
        __ mov(item2regHI(item), rhi);
      }
    } else if (item->is_stack()) {
      load(stack_item2addrLO(item), rlo, T_INT);
      load(stack_item2addrHI(item), rhi, T_INT);
    } else {
      ShouldNotReachHere();
    }
  } else {
    assert(item->type()->is_single_word(), "cannot be");
    Register r = reg.as_register();
    if (item->is_register()) {
      __ mov(item2reg(item), r);
    } else if (item->is_stack()) {
      load(stack_item2addr(item), r, item->type()->is_object() ? T_OBJECT : T_INT);
    } else {
      ShouldNotReachHere();
    }
  }
}


void CodeEmitter::constant_2_reg(jint con, RInfo reg) {
  __ set(con, reg.as_register());
}


void CodeEmitter::constant_2_reg(jlong con, RInfo reg) {
  __ set(low(con),  reg.as_register_lo());
  __ set(high(con), reg.as_register_hi());
}


void CodeEmitter::constant_2_reg(ciObject* con, RInfo reg) {
  nops_for_zombie();
  if (!con->is_loaded() || con->is_null_object()) {
    constant_2_reg((jint)NULL, reg);
  } else {
    int oop_index = __ oop_recorder()->find_index(con->encoding());
    RelocationHolder rspec = oop_Relocation::spec(oop_index);
    __ set(NULL, reg.as_register(), rspec); // Will be set when the nmethod is created
  }
}


void CodeEmitter::constant_2_reg(jfloat con, RInfo reg) {
  address const_addr = _const_table->address_of_float_constant(con);
  assert (const_addr != NULL, "must create float constant in the constant table");
  RelocationHolder rspec = internal_word_Relocation::spec(const_addr);
  __ sethi(  (int)const_addr & ~0x3ff, O7, rspec);
  int offset = (int)const_addr & 0x3ff;
  __ relocate(rspec);
  __ ldf (FloatRegister::S, O7, offset, reg.as_float_reg());
}


void CodeEmitter::constant_2_reg(jdouble con, RInfo reg) {
  address const_addr = _const_table->address_of_double_constant(con);
  assert (const_addr != NULL, "must create double constant in the constant table");
  RelocationHolder rspec = internal_word_Relocation::spec(const_addr);
  __ sethi(  (int)const_addr & ~0x3ff, O7, rspec);
  int offset = (int)const_addr & 0x3ff;
  __ relocate(rspec);
  __ ldf (FloatRegister::D, O7, offset, reg.as_double_reg());
}


void CodeEmitter::float_constant_2_cpu_reg(jfloat con, RInfo reg) {
  address const_addr = _const_table->address_of_float_constant(con);
  assert (const_addr != NULL, "must create float constant in the constant table");
  RelocationHolder rspec = internal_word_Relocation::spec(const_addr);
  __ set((int)const_addr, O7, rspec);
  load(O7, 0, reg.as_register(), T_INT);
}


void CodeEmitter::double_constant_2_cpu_reg(jdouble con, RInfo reg) {
  address const_addr = _const_table->address_of_double_constant(con);
  assert (const_addr != NULL, "must create double constant in the constant table");
  RelocationHolder rspec = internal_word_Relocation::spec(const_addr);
  __ set((int)const_addr, O7, rspec);
  load(O7, 0, reg.as_register_hi(), T_INT);
  load(O7, 4, reg.as_register_lo(), T_INT);
}


void CodeEmitter::int_constant_2_fpu_reg(jint con, RInfo reg) {
  assert(reg.is_float(), "wrong register kind");
  __ set(con, O7);
  Address temp_slot(SP, 0, frame::register_save_words * wordSize);
  __ st(O7, temp_slot);
  __ ldf(FloatRegister::S, temp_slot, reg.as_float_reg());
}


void CodeEmitter::long_constant_2_fpu_reg(jlong con, RInfo reg) {
  assert(reg.is_double(), "wrong register kind");
  // sparc is big-endian!
  Address temp_slot_hi(SP, 0, (frame::register_save_words  ) * wordSize);
  Address temp_slot_lo(SP, 0, (frame::register_save_words+1) * wordSize);
  __ set(high(con), O7);
  __ st(O7, temp_slot_hi);
  __ set(low(con),  O7);
  __ st(O7, temp_slot_lo);
  __ ldf(FloatRegister::D, temp_slot_hi, reg.as_double_reg());
}


void CodeEmitter::constant_2_reg(ValueType* type, RInfo reg) {
  assert(type->is_constant(), "type is not a constant");
  switch (type->tag()) {
    case intTag    : constant_2_reg(type->as_IntConstant    ()->value(), reg); return;
    case longTag   : constant_2_reg(type->as_LongConstant   ()->value(), reg); return;
    case floatTag  : constant_2_reg(type->as_FloatConstant  ()->value(), reg); return;
    case doubleTag : constant_2_reg(type->as_DoubleConstant ()->value(), reg); return;
    case objectTag :
      // Note: Check for is_object at the end since all instances are objects!
             if (type->is_array   ()) { constant_2_reg(type->as_ArrayConstant   ()->value(), reg);
      } else if (type->is_instance()) { constant_2_reg(type->as_InstanceConstant()->value(), reg);
      } else if (type->is_class   ()) { constant_2_reg(type->as_ClassConstant   ()->value(), reg);
      } else if (type->is_object  ()) { constant_2_reg(type->as_ObjectConstant  ()->value(), reg);
      } else                          { ShouldNotReachHere();
      }
      return;
    case addressTag: constant_2_reg(type->as_AddressConstant()->value(), reg); return;
  }
  ShouldNotReachHere();
}


void CodeEmitter::local_2_reg(RInfo reg, int local_index, BasicType t) {
  if (reg.is_word()) {
    BasicType local_type;
    switch (t) {
      case T_BOOLEAN: // fall through
      case T_BYTE   : // fall through
      case T_CHAR   : // fall through
      case T_SHORT  : // fall through
      case T_INT    : local_type = T_INT; break;
      case T_OBJECT : // fall through
      case T_ARRAY  : local_type = t;     break;
      default       : ShouldNotReachHere();
    }
    load(frame()->local_address(local_index, false), reg.as_register(), local_type);
  } else {
    Unimplemented();
  }
}


void CodeEmitter::reg_2_local(int local_index, RInfo reg, BasicType t) {
  if (reg.is_word()) {
    BasicType local_type;
    switch (t) {
      case T_BOOLEAN: // fall through
      case T_BYTE   : // fall through
      case T_CHAR   : // fall through
      case T_SHORT  : // fall through
      case T_INT    : local_type = T_INT; break;
      case T_OBJECT : // fall through
      case T_ARRAY  : local_type = t;     break;
      default       : ShouldNotReachHere();
    }
    store(reg.as_register(), frame()->local_address(local_index, false), local_type);
  } else {
    Unimplemented();
  }
}


void CodeEmitter::item_2_local(int local_index, Item* item) {
  if (item->is_register()) {
    switch (item->type()->tag()) {
    case addressTag: // fall through
    case intTag    : store(item2reg     (item), frame()->local_address(local_index, false), T_INT   ); return;
    case objectTag : store(item2reg     (item), frame()->local_address(local_index, false), T_OBJECT); return;
    case floatTag  : store(item2floatreg(item), frame()->local_address(local_index, false), T_FLOAT ); return;
    case doubleTag :
      { Address dst_addr_LO = frame()->local_address(local_index, true, false);
        Address dst_addr_HI = frame()->local_address(local_index, true, true);
        check_double_address(local_index);
        store(item2floatreg(item),             dst_addr_HI, T_FLOAT);
        store(item2floatreg(item).successor(), dst_addr_LO, T_FLOAT);
      }
      return;
    case longTag   :
      { Address dst_addr_LO = frame()->local_address(local_index, true, false);
        Address dst_addr_HI = frame()->local_address(local_index, true, true);
        store(item2regLO(item), dst_addr_LO, T_INT);
        store(item2regHI(item), dst_addr_HI, T_INT);
      }
      return;
    } 
  }
  ShouldNotReachHere();
}


void CodeEmitter::store_stack_parameter (Item *item, int offset_from_sp_in_words) {
  int offset_from_sp = offset_from_sp_in_words * wordSize;
  if (item->is_register()) {
    switch (item->type()->tag()) {
      case intTag   : store(item2reg(item),      SP, offset_from_sp, T_INT   ); return;
      case objectTag: store(item2reg(item),      SP, offset_from_sp, T_OBJECT); return;
      case floatTag : store(item2floatreg(item), SP, offset_from_sp, T_FLOAT ); return;
      case doubleTag: 
        store(item2floatreg(item),             stack_AddressHI(T_DOUBLE, SP, offset_from_sp), T_FLOAT);  
        store(item2floatreg(item).successor(), stack_AddressLO(T_DOUBLE, SP, offset_from_sp), T_FLOAT);  
        return;
      case longTag:   
        store(item2regLO(item), stack_AddressLO(T_LONG, SP, offset_from_sp), T_INT);  
        store(item2regHI(item), stack_AddressHI(T_LONG, SP, offset_from_sp), T_INT);  
        return;
    }
  }
  ShouldNotReachHere();
}


// this is meant to move a floating point value to a  cpu register on sparc
void  CodeEmitter::move_item_to_cpu_register (const Item& src, RInfo dst) {
  BasicType type = as_BasicType(src.type());
  assert((type == T_FLOAT && dst.is_word()) || (type == T_DOUBLE && dst.is_long()), "wrong type");

  if (src.is_constant()) {
    switch (type) {
    case T_FLOAT:   float_constant_2_cpu_reg(src.get_jfloat_constant (), dst); return;
    case T_DOUBLE: double_constant_2_cpu_reg(src.get_jdouble_constant(), dst); return;
    }
  }

  if (src.is_stack()) {
    untyped_stackitem_2_reg(dst, &src);
    return;
  }

  assert (src.is_register(), "src is either constant, stack or register");
  switch (type) {
  case T_FLOAT:
    {
      Address temp_slot = Address(SP, 0, frame::register_save_words * wordSize);
      store(src.get_register().as_float_reg(), temp_slot, T_FLOAT, NULL);
      load(temp_slot, dst.as_register(), T_INT, NULL);
      return;
    }
  case T_DOUBLE:
    {
      // sparc is big-endian!
      Address temp_slot_hi = Address(SP, 0, (frame::register_save_words  ) * wordSize);
      Address temp_slot_lo = Address(SP, 0, (frame::register_save_words+1) * wordSize);
      store(src.get_register().as_double_reg(), temp_slot_hi, T_DOUBLE, NULL);
      load(temp_slot_hi, dst.as_register_hi(), T_INT, NULL);
      load(temp_slot_lo, dst.as_register_lo(), T_INT, NULL);
      return;
    }
  }
}


// this is meant to move an int value to an fpu register on sparc
void  CodeEmitter::move_item_to_fpu_register (const Item& src, RInfo dst) {
  BasicType type = as_BasicType(src.type());
  assert((type == T_INT && dst.is_float()) || (type == T_LONG && dst.is_double()), "wrong type");

  if (src.is_constant()) {
    switch (type) {
    case T_INT:   int_constant_2_fpu_reg(src.get_jint_constant (), dst); return;
    case T_LONG: long_constant_2_fpu_reg(src.get_jlong_constant(), dst); return;
    }
  }

  if (src.is_stack()) {
    untyped_stackitem_2_reg(dst, &src);
    return;
  }

  assert (src.is_register(), "src is either constant, stack or register");
  switch (type) {
  case T_INT:
    {
      Address temp_slot = Address(SP, 0, frame::register_save_words * wordSize);
      store(src.get_register().as_register(), temp_slot, T_INT, NULL);
      load(temp_slot, dst.as_float_reg(), T_FLOAT, NULL);
      return;
    }
  case T_LONG:
    {
      // sparc is big-endian!
      Address temp_slot_hi = Address(SP, 0, (frame::register_save_words  ) * wordSize);
      Address temp_slot_lo = Address(SP, 0, (frame::register_save_words+1) * wordSize);
      store(src.get_register().as_register_hi(), temp_slot_hi, T_INT, NULL);
      store(src.get_register().as_register_lo(), temp_slot_lo, T_INT, NULL);
      load(temp_slot_hi, dst.as_double_reg(), T_DOUBLE, NULL);
      return;
    }
  }
}


void CodeEmitter::move_spill(int to_spill_ix, int from_spill_ix, ValueType* type, RInfo tmp) {
  switch (type->tag()) {
    case addressTag:
    case intTag:
      load(frame()->spill_address(from_spill_ix, false), tmp.as_register(), T_INT);
      store(tmp.as_register(), frame()->spill_address(to_spill_ix, false), T_INT);
      break;
    case objectTag:
      load(frame()->spill_address(from_spill_ix, false), tmp.as_register(), T_OBJECT);
      store(tmp.as_register(), frame()->spill_address(to_spill_ix, false), T_OBJECT);
      break;
    case floatTag:
      load(frame()->spill_address(from_spill_ix, false), tmp.as_float_reg(), T_FLOAT);
      store(tmp.as_float_reg(), frame()->spill_address(to_spill_ix, false), T_FLOAT);
      break;
    case doubleTag:
      {
        Address spill_from_hi = frame()->spill_address(from_spill_ix, true, true );
        Address spill_from_lo = frame()->spill_address(from_spill_ix, true, false);
        Address spill_to_hi   = frame()->spill_address(to_spill_ix,   true, true );
        Address spill_to_lo   = frame()->spill_address(to_spill_ix,   true, false);

        FloatRegister f_hi = tmp.as_double_reg();
        FloatRegister f_lo = tmp.as_double_reg().successor();

        load(spill_from_hi, f_hi, T_FLOAT);
        load(spill_from_lo, f_lo, T_FLOAT);
        store(f_hi, spill_to_hi,  T_FLOAT);
        store(f_lo, spill_to_lo,  T_FLOAT);
      }
      break;
    case longTag:
      load(frame()->spill_address(from_spill_ix, true, false), tmp.as_register_lo(), T_INT);
      store(tmp.as_register_lo(), frame()->spill_address(to_spill_ix, true, false), T_INT);
      load(frame()->spill_address(from_spill_ix, true, true), tmp.as_register_hi(), T_INT);
      store(tmp.as_register_hi(), frame()->spill_address(to_spill_ix, true, true), T_INT);
      break;
    default:
      ShouldNotReachHere();
  }
}


Address CodeEmitter::stack_Address(BasicType type, Register r, int disp_bytes, bool for_hi_word) { 
  assert(r == FP || r == SP, "need stack address");
  switch (type) {
    case T_LONG  : disp_bytes += for_hi_word ? 0        : wordSize ; break; // BigEndian
    case T_DOUBLE: disp_bytes += for_hi_word ? 0        : wordSize ; break; // BigEndian
    default      : assert(!for_hi_word, "Does not have hi word");  
  }
  return Address(r, 0, disp_bytes);
}


Address CodeEmitter::stack_AddressLO(BasicType type, Register r, int disp_bytes) {
  return stack_Address(type,r,disp_bytes,false);
}


Address CodeEmitter::stack_AddressHI(BasicType type, Register r, int disp_bytes) {
  return stack_Address(type,r,disp_bytes,true);
}


Address CodeEmitter::heap_Address(BasicType type, Register r, int disp_bytes, bool for_hi_word) { 
  assert(r != FP || r != SP, "need heap address");
  switch (type) {
    case T_LONG  : disp_bytes += for_hi_word ? 0 : wordSize; break; // BigEndian
    case T_DOUBLE: disp_bytes += for_hi_word ? 0 : wordSize; break; // BigEndian
    default      : assert(!for_hi_word, "Does not have hi word");  
  }
  return Address(r, 0, disp_bytes);
}


Address CodeEmitter::heap_AddressLO(BasicType type, Register r, int disp_bytes) {
  return heap_Address(type,r,disp_bytes,false);
}


Address CodeEmitter::heap_AddressHI(BasicType type, Register r, int disp_bytes) {
  return heap_Address(type, r, disp_bytes, true);
}


// load with 32-bit displacement
void CodeEmitter::load(Register s, int disp, Register d, BasicType ld_type, CodeEmitInfo *info) {
  if (Assembler::is_simm13(disp)) { 
    if (info != NULL) add_debug_info_here(info);
    switch(ld_type) {
      case T_BYTE  : __ ldsb(s, disp, d); break;
      case T_CHAR  : __ lduh(s, disp, d); break;
      case T_SHORT : __ ldsh(s, disp, d); break;
      case T_INT   : __ lduw(s, disp, d); break;
      case T_ARRAY : // fall through
      case T_OBJECT: __ lduw(s, disp, d); break;
      default      : ShouldNotReachHere();
    }
  } else { 
    __ sethi(disp & ~0x3ff, O7);
    __ nop(); // if patching is required, this will need to be here.
    __ add(O7, disp & 0x3ff, O7);
    if (info != NULL) add_debug_info_here(info);
    switch(ld_type) {
      case T_BYTE  : __ ldsb(s, O7, d); break;
      case T_CHAR  : __ lduh(s, O7, d); break;
      case T_SHORT : __ ldsh(s, O7, d); break;
      case T_INT   : __ lduw(s, O7, d); break;
      case T_ARRAY : // fall through
      case T_OBJECT: __ lduw(s, O7, d); break;
      default      : ShouldNotReachHere();
    }
  }
  if (ld_type == T_ARRAY || ld_type == T_OBJECT) __ verify_oop(d);
}


// store with 32-bit displacement
void CodeEmitter::store(Register d, Register s, int disp, BasicType st_type, CodeEmitInfo *info) { 
  if (Assembler::is_simm13(disp)) { 
    if (info != NULL)  add_debug_info_here(info);
    switch (st_type) {
      case T_BYTE  : __ stb(d, s, disp); break;
      case T_CHAR  : __ sth(d, s, disp); break;
      case T_SHORT : __ sth(d, s, disp); break;
      case T_INT   : __ stw(d, s, disp); break;
      case T_ARRAY : // fall through
      case T_OBJECT: __ stw(d, s, disp); break;
      default      : ShouldNotReachHere();
    }
  } else {
    __ sethi(disp & ~0x3ff, O7);
    __ nop(); // if patching is required, this will need to be here.
    __ add(O7, disp & 0x3ff, O7);
    if (info != NULL) add_debug_info_here(info);
    switch (st_type) {
      case T_BYTE  : __ stb(d, s, O7); break;
      case T_CHAR  : __ sth(d, s, O7); break;
      case T_SHORT : __ sth(d, s, O7); break;
      case T_INT   : __ stw(d, s, O7); break;
      case T_ARRAY : //fall through
      case T_OBJECT: __ stw(d, s, O7); break;
      default      : ShouldNotReachHere();
    }
  }
  // Note: Do the store before verification as the code might be patched!
  if (st_type == T_ARRAY || st_type == T_OBJECT) __ verify_oop(d);
} 


// load float with 32-bit displacement
void CodeEmitter::load(Register s, int disp, FloatRegister d, BasicType ld_type, CodeEmitInfo *info) { 
  FloatRegister::Width w;
  switch(ld_type) {
    case T_FLOAT : w = FloatRegister::S; break;
    case T_DOUBLE: w = FloatRegister::D; break;
    default      : ShouldNotReachHere();
  }

  if (Assembler::is_simm13(disp)) { 
    if (info != NULL) add_debug_info_here(info);
    __ ldf(w, s, disp, d); 
  } else { 
    __ sethi(disp & ~0x3ff, O7);
    __ nop(); // if patching is required, this will need to be here.
    __ add(O7, disp & 0x3ff, O7);
    if (info != NULL) add_debug_info_here(info);
    __ ldf(w, s, O7, d); 
  } 
} 


// store float with 32-bit displacement
void CodeEmitter::store(FloatRegister d, Register s, int disp, BasicType st_type, CodeEmitInfo *info) { 
  FloatRegister::Width w;
  switch(st_type) {
    case T_FLOAT : w = FloatRegister::S; break;
    case T_DOUBLE: w = FloatRegister::D; break;
    default      : ShouldNotReachHere();
  }

  if (Assembler::is_simm13(disp)) { 
    if (info != NULL) add_debug_info_here(info);
    __ stf(w, d, s, disp); 
  } else { 
    __ sethi(disp & ~0x3ff, O7);
    __ nop(); // if patching is required, this will need to be here.
    __ add(O7, disp & 0x3ff, O7);
    if (info != NULL) add_debug_info_here(info);
    __ stf(w, d, O7, s); 
  } 
}


// load/store with an Address
void CodeEmitter::load(const Address& a, Register d,  BasicType ld_type, CodeEmitInfo *info, int offset /* = 0 */) { 
  load(a.base(), a.disp() + offset, d, ld_type, info);  
}


void CodeEmitter::store(Register d, const Address& a, BasicType st_type, CodeEmitInfo *info, int offset /* = 0 */) { 
  store(d,a.base(), a.disp() + offset, st_type, info); 
}


// loadf/storef with an Address
void CodeEmitter::load(const Address& a, FloatRegister d, BasicType ld_type, CodeEmitInfo *info, int offset /* = 0 */) { 
  load(a.base(), a.disp() + offset, d, ld_type, info);   
}


void CodeEmitter::store(FloatRegister d, const Address& a, BasicType st_type, CodeEmitInfo *info, int offset /* = 0 */) { 
  store(d,a.base(), a.disp() + offset, st_type, info); 
}


// Creates the patching stub.  Also sets the patch's field offset so that a high or low word 
// move can be recognized (offset is 4 words wide).
void CodeEmitter::patching_epilog(PatchingDesc* patch, CodeEmitInfo* info, PatchingStub::PatchID id ) {

  if (!UsePatching) set_bailout("Patching is disabled");
  if (id == PatchingStub::access_field_id) {
    NativeMovRegMemPatching* n_move = nativeMovRegMemPatching_at((address)patch->pc_start());
    n_move->set_offset(patch->field_offset());
  }

  // We need to figure out how many bytes to patch.  The trouble is that there are two different
  // patch types, and one of them also comes in two flavors - short and long (the short case is
  // rare and we don't currently support it, but we shouldn't preclude it here).  I hate using
  // raw constants in the code, but a lot of code already heavily relies on the presence one of
  // two known and specific code sequences.
  //
  // So anyway, if we've just generated a ld/nop (or st/nop) with a simm13 offset, then we can 
  // patch the ld or st directly and the number of bytes to patch is 4.  Otherwise, we've 
  // generated a sethi/nop/add/ld (or st) and we must patch the first 12 bytes.  The way to tell 
  // them apart is to see if we've generated 8 bytes or 16 bytes since creating the PatchingDesc.

  int instr_size = (int) __ pc() - patch->pc_start();
  int bytes_to_patch = 0;
  if (instr_size == 8) {
     // This handles the short version of the access_field_id case.
     bytes_to_patch = 4;
  } else {
     // This handles both the load_klass_id case and the long version of the access_field_id
     // case.
     bytes_to_patch = 12;
  }
  PatchingStub* stub = new PatchingStub(patch->pc_start(), noreg, noreg, bytes_to_patch, info, id);
  _slow_case_stubs->append(stub);
}


// handle long, float, double, oop and int
void CodeEmitter::field_load(RInfo dst, BasicType dst_type, Item* object, int offset,
			     bool needs_patching, bool is_loaded, CodeEmitInfo* info) {
  assert(object->type()->is_object(), "not an object");
  __ verify_oop(item2reg(object));

  PatchingDesc patch;
  if (needs_patching) {

#if 0
    // There is a possibility to save a few instructions in some cases by generating a
    // "short" patch when the offset is known and is small.  The only time that this
    // situation could arise, however, is when the class has been loaded but not
    // initialized.  This is generally rare unless there is some type of eager loading
    // going on.  In most cases, the class is not loaded until it is needed, and it is
    // therefore initialized quickly thereafter.
    //
    // But if you want to try this, delete these comments (those above and including the
    // dashed line.  Then delete the #if 0 and the #endif and also the assignment
    // "offset = MAX_OFFSET" that was AFTER the #endif that you just deleted.
    // There is likely other code that will need to be modified to support this, especially
    // in patch_code().
    //
    // --------------------------------------------------------------------------------------
    //
    // If the class is loaded then the value in offset can be trusted and should be used.
    // Otherwise, we must assume the worst case - that it will be a large offset.
    // The only exception is that it is allowed to be -1 if we are running with CompileTheWorld.
    // However, even in that case, if the class is not loaded then offset will get MAX_OFFSET.
    // But that is harmless.
    if (!is_loaded) {
      offset = MAX_OFFSET;
    }
#endif
    offset = MAX_OFFSET;
    set_patching_desc(&patch);
    add_call_info(patch.pc_offset() + NativeCall::instruction_size, info);
  }
  assert(offset >= 0 || (offset == -1 && CompileTheWorld), "offset should be non-negative");

  switch (dst_type) {
  case T_DOUBLE: {
    assert(dst.is_double(), "consistency check");

    FloatRegister rdst = dst.as_double_reg();
    Address addr = item2addr_with_offset(object, offset);

    load(addr, rdst, dst_type, info);
    if (needs_patching && addr.is_simm13()) {
      // Add a nop instruction as the load above will be patched to a call.
      __ nop();
    }
    break;
  }

  case T_FLOAT: {
    assert(dst.is_float(), "consistency check");

    FloatRegister rdst = dst.as_float_reg();
    Address addr = item2addr_with_offset(object, offset);
  
    load(addr, rdst, dst_type, info);
    if (needs_patching && addr.is_simm13()) {
      // Add a nop instruction as the load above will be patched to a call.
      __ nop();
    }
    break;
  }

  case T_BOOLEAN:
  case T_BYTE:
  case T_CHAR:
  case T_SHORT:
  case T_OBJECT: 
  case T_ARRAY: 
  case T_INT: {
    assert(dst.is_word(), "consistency check");

    Register rdst = dst.as_register();
    Address addr = item2addr_with_offset(object, offset);

    load(addr, rdst, T_INT, info);
    if (needs_patching && addr.is_simm13()) {
      // Add a nop instruction as the load above will be patched to a call.
      __ nop();
    }
    break;
  }

  case T_LONG: {
    assert(dst.is_long(), "consistency check");

    Register rlo = dst.as_register_lo();
    Register rhi = dst.as_register_hi();
    assert(rlo != rhi, "error in regalloc");

    Address addr_lo = item2addr_with_offsetLO(object, offset);
    Address addr_hi = item2addr_with_offsetHI(object, offset);

    Register robj = item2reg(object);

    // Do not overwrite object register too early.
    if (robj == rlo) {
      load(addr_hi, rhi, T_INT, info);
      if (needs_patching) {
	if (addr_hi.is_simm13()) {
          // Add a nop instruction as the load above will be patched to a call.
	  __ nop();
        }
        patch.set_hi_word();
	patching_epilog(&patch, info, PatchingStub::access_field_id);
	set_patching_desc(&patch);
	patch.set_lo_word();
        add_call_info(patch.pc_offset() + NativeCall::instruction_size, info);
      }
      load(addr_lo, rlo, T_INT);
      if (needs_patching && addr_lo.is_simm13()) {
        // Add a nop instruction as the load above will be patched to a call.
	__ nop();
      }
    } else {  
      load(addr_lo, rlo, T_INT, info);
      if (needs_patching) {
	if (addr_lo.is_simm13()) {
          // Add a nop instruction as the load above will be patched to a call.
	  __ nop();
        }
        patch.set_lo_word();
	patching_epilog(&patch, info, PatchingStub::access_field_id);
	set_patching_desc(&patch);
	patch.set_hi_word();
        add_call_info(patch.pc_offset() + NativeCall::instruction_size, info);
      }
      load(addr_hi, rhi, T_INT);
      if (needs_patching && addr_hi.is_simm13()) {
        // Add a nop instruction as the load above will be patched to a call.
	__ nop();
      }
    }
    break;
  }

  default: ShouldNotReachHere();
  }  

  if (needs_patching) {
    patching_epilog(&patch, info, PatchingStub::access_field_id);
  }
}


// handle long, float, double, oop and int
// we kill the register of "object"
void CodeEmitter::field_store(BasicType value_type, Item* object, int offset, Item* value,
			      bool needs_patching, bool is_loaded, CodeEmitInfo* info, RInfo temp_reg) {
  assert(object->type()->is_object() && object->is_register(), "wrong object type");
  assert(value->is_register(), "wrong type");
  __ verify_oop(item2reg(object));

  PatchingDesc patch;
  if (needs_patching) {

#if 0
    // There is a possibility to save a few instructions in some cases by generating a
    // "short" patch when the offset is known and is small.  The only time that this
    // situation could arise, however, is when the class has been loaded but not
    // initialized.  This is generally rare unless there is some type of eager loading
    // going on.  In most cases, the class is not loaded until it is needed, and it is
    // therefore initialized quickly thereafter.
    //
    // But if you want to try this, delete these comments (those above and including the
    // dashed line.  Then delete the #if 0 and the #endif and also the assignment
    // "offset = MAX_OFFSET" that was AFTER the #endif that you just deleted.
    // There is likely other code that will need to be modified to support this, especially
    // in patch_code().
    //
    // --------------------------------------------------------------------------------------
    //
    // If the class is loaded then the value in offset can be trusted and should be used.
    // Otherwise, we must assume the worst case - that it will be a large offset.
    // The only exception is that it is allowed to be -1 if we are running with CompileTheWorld.
    // However, even in that case, if the class is not loaded then offset will get MAX_OFFSET.
    // But that is harmless.
    if (!is_loaded) {
      offset = MAX_OFFSET;
    }
#endif
    offset = MAX_OFFSET;
    set_patching_desc(&patch);
    add_call_info(patch.pc_offset() + NativeCall::instruction_size, info);
  }
  assert(offset >= 0 || (offset == -1 && CompileTheWorld), "offset should be non-negative");

  switch (value_type) {
  case T_DOUBLE: {
    assert(value->type()->is_double(), "must be double register");
    Address dst_addr = item2addr_with_offset(object, offset);
    store(item2floatreg(value), dst_addr, value_type, info);
    if (needs_patching && dst_addr.is_simm13()) {
      // Add a nop instruction as the store above will be patched to a call.
      __ nop();
    }
    break;
  }

  case T_FLOAT: {
    assert(value->type()->is_float(), "must be float register");
    Address dst_addr = item2addr_with_offset(object, offset);
    store(item2floatreg(value), dst_addr, value_type, info);
    if (needs_patching && dst_addr.is_simm13()) {
      // Add a nop instruction as the store above will be patched to a call.
      __ nop();
    }
    break;
  }

  case T_BOOLEAN:
  case T_BYTE:
  case T_CHAR:
  case T_SHORT:
  case T_OBJECT: 
  case T_ARRAY: 
  case T_INT: {
    Address dst_addr = item2addr_with_offset(object, offset);
    store(item2reg(value), dst_addr, T_INT, info);
    if (needs_patching && dst_addr.is_simm13()) {
      // Add a nop instruction as the store above will be patched to a call.
      __ nop();
    }
    break;
  }

  case T_LONG: { 
    Address dst_addr_lo = item2addr_with_offsetLO(object, offset);
    Address dst_addr_hi = item2addr_with_offsetHI(object, offset);
    Register rlo = item2regLO(value);
    Register rhi = item2regHI(value);

    store(rlo, dst_addr_lo, T_INT, info);
    if (needs_patching) {
      if (dst_addr_lo.is_simm13()) {
        // Add a nop instruction as the store above will be patched to a call.
        __ nop();
      }
      patch.set_lo_word();
      patching_epilog(&patch, info, PatchingStub::access_field_id);
      set_patching_desc(&patch);
      patch.set_hi_word();
      add_call_info(patch.pc_offset() + NativeCall::instruction_size, info);
    }

    store(rhi, dst_addr_hi, T_INT);
    if (needs_patching && dst_addr_hi.is_simm13()) {
      // Add a nop instruction as the store above will be patched to a call.
      __ nop();
    }
    break;
  }

  default: ShouldNotReachHere();
  }

  if (needs_patching) {
    patching_epilog(&patch, info, PatchingStub::access_field_id);
  }

  if (value_type == T_OBJECT || value_type == T_ARRAY) {
    // an object pointer was stored into memory, 
    // emit code that notes this (for gc purposes).
    assert(object->destroys_register(), "check");
    __ store_check(temp_reg.as_register(), object->get_register().as_register());
  }
}


int CodeEmitter::shift_amount(BasicType t) {
  int elem_size = type2aelembytes[t];
  switch (elem_size) {
    case 1 : return 0;
    case 2 : return 1;
    case 4 : return 2; 
    case 8 : return 3; 
  }
  ShouldNotReachHere();
  return -1;
}


void CodeEmitter::constant_address_and_range_check(BasicType dst_type, Register rarr, int index, Register rdst, CodeEmitInfo* info) {
  add_debug_info_here(info); 
  __ ld(rarr, arrayOopDesc::length_offset_in_bytes(), O7);
  __ cmp(O7, index);
  
  int offset = __ offset();
  add_call_info_here(info); // an exception looks like a call from here
  RangeCheckStub* slow_case = new RangeCheckStub(offset);
  _slow_case_stubs->append(slow_case);

  __ br(Assembler::lessEqualUnsigned, false, Assembler::pn, *slow_case->entry());
  int elem_size = type2aelembytes[dst_type];
  int shift = shift_amount(dst_type);
  int array_offset = index * elem_size;
  if (Assembler::is_simm13(array_offset)) {
    __ delayed()->add(rarr, array_offset, rdst);
  } else {
    __ delayed()->mov(index, rdst);
    __ sll(rdst, shift, rdst);
    __ add(rdst, rarr, rdst);
  }
}

// do the range check and compute the indexed address (see also ::indexed_address and ::range_check
// doing it together allows us to fill the delay slot
void CodeEmitter::indexed_address_and_range_check(BasicType dst_type, Register rarr, Register rind, Register rdst, CodeEmitInfo* info) {

  // implicit null check 
  add_debug_info_here(info); 
  __ ld(rarr, arrayOopDesc::length_offset_in_bytes(), O7);
  __ cmp(rind, O7);

  int offset = __ offset();
  add_call_info_here(info); // an exception looks like a call from here
  RangeCheckStub* slow_case = new RangeCheckStub(offset);
  _slow_case_stubs->append(slow_case);

  // if ((uint) rind) >= (uint) O7) then branch (implies (rind < 0) || (rind >= O7))
  __ br(Assembler::carryClear, false, Assembler::pn, *slow_case->entry());

  { int shift = shift_amount(dst_type);
    if (shift > 0) {
      __ delayed()->sll(rind, shift, rdst);  // rdst = rind * 2 ^ shift
      __ add(rdst, rarr, rdst);   // rdst = rdst + rarr
    } else {
      __ delayed()->add(rind, rarr, rdst);   // rdst = rind + rarr
    }  
  }
}


// handles byte, char, short, int, long, float, double, oop
void CodeEmitter::indexed_load(RInfo dst, BasicType dst_type, Item* array, Item* index, CodeEmitInfo* info) {
  int base_offset =  arrayOopDesc::base_offset_in_bytes(dst_type);
  Register rarr = item2reg(array);
  __ verify_oop(rarr);
  if (index->is_constant()) {
    int index_val = item2int(index);
    // O7 = rarr + rind * width
    constant_address_and_range_check(dst_type, rarr, index_val, O7, info);
  } else {
    Register rind = item2reg(index);
    // O7 = rarr + rind * width
    indexed_address_and_range_check(dst_type, rarr, rind, O7, info);
  }
  switch(dst_type) {
    case T_BYTE: 
    case T_CHAR: 
    case T_SHORT: 
    case T_ARRAY:
    case T_OBJECT:
    case T_INT: {
      assert(dst.is_word(), "check");
      load(O7, base_offset, dst.as_register(), dst_type, info);  
      break;
    }  
    case T_FLOAT: {
      assert(dst.is_float(), "check");
      load(O7, base_offset, dst.as_float_reg(), dst_type, info);  
      break;
    }
    case T_DOUBLE: {
      assert(dst.is_double(), "check");
      load(O7, base_offset, dst.as_double_reg(), dst_type, info);  
      break;
    }
    case T_LONG: { 
      assert(dst.is_long(), "check");
      Register rlo = dst.as_register_lo();
      Register rhi = dst.as_register_hi();
      assert(rlo != rhi, "error in regalloc");

      assert(O7 != rlo && O7 != rhi, "O7 is for temps");
      load(heap_AddressLO(dst_type, O7, base_offset), rlo, T_INT, info);
      load(heap_AddressHI(dst_type, O7, base_offset), rhi, T_INT);

      break;
    }
  
    default: ShouldNotReachHere();
  }

}


// handles byte, char, short, int, long, float, double, oop
void CodeEmitter::indexed_store(BasicType dst_type, Item* array, Item* index, Item* value, RInfo temp_reg, CodeEmitInfo* info) {
  assert(value->is_register(), "not in a register");
  int base_offset =  arrayOopDesc::base_offset_in_bytes(dst_type);
  Register rarr = item2reg(array);
  RInfo    rval = value->get_register();
  __ verify_oop(rarr);
  if (index->is_constant()) {
    int index_val = item2int(index);
    // O7 = rarr + rind * width
    constant_address_and_range_check(dst_type, rarr, index_val, O7, info);
  } else {
    Register rind = item2reg(index);
    // O7 = rarr + rind * width
    indexed_address_and_range_check(dst_type, rarr, rind, O7, info);
  }

  switch(dst_type) {
    case T_ARRAY:
    case T_OBJECT:
      __ verify_oop(item2reg(value));
      // fall through
    case T_BYTE: 
    case T_CHAR: 
    case T_SHORT: 
    case T_INT: {
      assert(rval.is_word(), "check");
      store(item2reg(value), O7, base_offset, dst_type, info);  
      break;
    }  
    case T_FLOAT: {
      assert(rval.is_float(), "check");
      store(item2floatreg(value), O7, base_offset, dst_type, info);  
      break;
    }
    case T_DOUBLE: {
      assert(rval.is_double(), "check");
      store(item2floatreg(value), O7, base_offset, dst_type, info);  
      break;
    }
    case T_LONG: { 
      assert(rval.is_long(), "check");
      Register rlo = item2regLO(value);
      Register rhi = item2regHI(value);
      assert(rlo != rhi, "error in regalloc");

      store(rlo, heap_AddressLO(dst_type, O7, base_offset), T_INT, info);
      store(rhi, heap_AddressHI(dst_type, O7, base_offset), T_INT);
      break;
    }  
    default: ShouldNotReachHere();
  }

  if (dst_type == T_ARRAY || dst_type == T_OBJECT) {
    assert(array->destroys_register(), "check");
    __ add(O7, base_offset, O7);
    __ store_check(temp_reg.as_register(), O7);
  }
}


void CodeEmitter::monitor_enter(Item* object, RInfo lock, RInfo hdr, RInfo scratch, int monitor_no, CodeEmitInfo* info) {
  monitorenter(item2reg(object), lock.as_register(), hdr.as_register(), scratch.as_register(), monitor_no, info);
}


void CodeEmitter::monitor_exit(Item* object, RInfo lock, RInfo hdr, int monitor_no)  {
  monitorexit(item2reg(object), lock.as_register(), hdr.as_register(), monitor_no);
}


void CodeEmitter::negate(RInfo dst, Item* value) {
  assert(value->is_register(), "can only handle registers");
  assert(!value->destroys_register(), "check");
  ValueTag tag = value->type()->tag();
  if (tag == intTag) {
    __ neg(item2reg(value), dst.as_register());
  } else if (tag == floatTag) {
    __ fneg(FloatRegister::S, item2floatreg(value), dst.as_float_reg());
  } else if (tag == doubleTag) {
    __ fneg(FloatRegister::D, item2floatreg(value), dst.as_double_reg());
  } else if (tag == longTag) {
    Register Rlow = item2regLO(value);
    Register Rhi = item2regHI(value);
    __ subcc(G0, Rlow, dst.as_register_lo());
    __ subc(G0, Rhi, dst.as_register_hi());
  } else {
    ShouldNotReachHere();
  }
}


// for: _iadd, _imul, _isub, 
void CodeEmitter::arithmetic_op_int(Bytecodes::Code code, Item* result, Item* left, Item* right, RInfo tmp) {
  assert(left  ->is_register(), "wrong items state");
  assert(result->is_register(), "wrong items state");

  Register lreg = item2reg(left);
  Register res  = item2reg(result);

  if (right->is_register()) {
    Register rreg = right->is_register() ? item2reg(right) : noreg;
    switch (code) {
      case Bytecodes::_iadd:  __ add  (lreg, rreg, res); break;
      case Bytecodes::_isub:  __ sub  (lreg, rreg, res); break;
      case Bytecodes::_imul:  
	if(VM_Version::v9_instructions_work()) {
	  __ mulx (lreg, rreg, res); break;
	} else {
	  __ smul (lreg, rreg, res); break;
	}
      default: ShouldNotReachHere();
    }
  } else {
    int simm13 = right->get_jint_constant();
    switch (code) {
      case Bytecodes::_iadd:  __ add  (lreg, simm13, res); break;
      case Bytecodes::_isub:  __ sub  (lreg, simm13, res); break;
      case Bytecodes::_imul:  
	if(VM_Version::v9_instructions_work()) {
	  __ mulx (lreg, simm13, res); break;
	} else {
	  __ smul (lreg, simm13, res); break;
	}
      default: ShouldNotReachHere();
    }
  }

  assert(result->is_register(), "cannot handle non-register results");
}


//      _ladd,      , _lsub, 
void CodeEmitter::arithmetic_op_long(Bytecodes::Code code, Item* result, Item* left, Item* right) {
  assert(left->is_register(),   "wrong item state");
  assert(right->is_register(),  "wrong item state");
  assert(result->is_register(), "wrong item state");

  Register op1_lo = item2regLO(left);
  Register op1_hi = item2regHI(left);
  Register op2_lo = item2regLO(right);
  Register op2_hi = item2regHI(right);
  Register dst_lo = item2regLO(result);
  Register dst_hi = item2regHI(result);

  switch (code) {
  case Bytecodes::_ladd:
    __ addcc(op1_lo, op2_lo, dst_lo); 
    __ addc (op1_hi, op2_hi, dst_hi); 
    break;
  case Bytecodes::_lsub:
    __ subcc(op1_lo, op2_lo, dst_lo); 
    __ subc (op1_hi, op2_hi, dst_hi); 
    break;
  default: ShouldNotReachHere();
  }
}


//      _fadd, _fmul, _fsub, _fdiv
//      _dadd, _dmul, _dsub, _ddiv
void CodeEmitter::arithmetic_op_fpu(Bytecodes::Code code, Item* result, Item* left, Item* right) {
  assert(left  ->is_register(), "wrong items state");
  assert(right ->is_register(), "wrong items state");
  assert(result->is_register(), "wrong items state");

  FloatRegister lreg = item2floatreg(left);
  FloatRegister rreg = item2floatreg(right);
  FloatRegister res  = item2floatreg(result);

  assert(lreg == res ^ !left->destroys_register(), "check");
  assert(rreg == res ^ !right->destroys_register(), "check");

  switch (code) {
  case Bytecodes::_dadd:  __ fadd(FloatRegister::D, lreg, rreg, res); break;
  case Bytecodes::_dsub:  __ fsub(FloatRegister::D, lreg, rreg, res); break;
  case Bytecodes::_dmul:  __ fmul(FloatRegister::D, lreg, rreg, res); break;
  case Bytecodes::_ddiv:  __ fdiv(FloatRegister::D, lreg, rreg, res); break;
  case Bytecodes::_fadd:  __ fadd(FloatRegister::S, lreg, rreg, res); break;
  case Bytecodes::_fsub:  __ fsub(FloatRegister::S, lreg, rreg, res); break;
  case Bytecodes::_fmul:  __ fmul(FloatRegister::S, lreg, rreg, res); break;
  case Bytecodes::_fdiv:  __ fdiv(FloatRegister::S, lreg, rreg, res); break;
  default: ShouldNotReachHere();
  }

  assert(result->is_register(), "cannot handle non-register results");
}


void CodeEmitter::arithmetic_idiv(Bytecodes::Code code, Item* result, Item* left, Item* right, RInfo scratch, CodeEmitInfo* info) {
  Register Rdividend = item2reg(left);
  Register Rdivisor  = item2reg(right);
  Register Rscratch  = scratch.as_register();
  Register Rresult   = item2reg(result);
  assert(Rdividend != Rscratch, "");
  assert(Rdivisor  != Rscratch, "");
  if (code == Bytecodes::_idiv) {
    __ sra(Rdividend, 31, Rscratch);
    __ wry(Rscratch);
    __ nop(); __ nop(); __ nop(); __ nop();

    add_debug_info_here(info);
    __ sdivcc(Rdividend, Rdivisor, Rresult);

    Label skip;
    __ br(Assembler::overflowSet, true, Assembler::pn, skip);
    __ delayed()->sethi(0x80000000, Rresult);
    __ bind(skip);
  } else {
    __ sra(Rdividend, 31, Rscratch);
    __ wry(Rscratch);
    __ nop(); __ nop(); __ nop(); __ nop();

    add_debug_info_here(info);
    __ sdivcc(Rdividend, Rdivisor, Rscratch);

    Label skip;
    __ br(Assembler::overflowSet, true, Assembler::pn, skip);
    __ delayed()->sethi(0x80000000, Rscratch);
    __ bind(skip);

    __ smul(Rscratch, Rdivisor, Rscratch);
    __ sub(Rdividend, Rscratch, Rresult);
  }
}


// preserve registers across calls (NOTE: we may push less registers as some are callee saved anyway)
void CodeEmitter::arithmetic_call_prolog(ValueType* result) {
  ShouldNotReachHere();
}


// preserve registers across calls (NOTE: we may push less registers as some are callee saved anyway)
void CodeEmitter::arithmetic_call_epilog(ValueType* result) {
  ShouldNotReachHere();
}


void CodeEmitter::arithmetic_call_op_check(RInfo left, RInfo right, Item* result) {
  if (left.is_long()) { 
    assert( left.as_register_lo() == O3, "wrong register assignment");
    assert( left.as_register_hi() == O2, "wrong register assignment");
  } else if (left.is_double()) {
    assert( left.as_register_hi() == O1, "wrong register assignment");
    assert( left.as_register_lo() == O0, "wrong register assignment");
  } else if (left.is_float()) {
    assert( left.as_register() == O0, "wrong register assignment");
  } else { ShouldNotReachHere(); }
  
  if (right.is_long()) { 
    assert( right.as_register_lo() == O1, "wrong register assignment");
    assert( right.as_register_hi() == O0, "wrong register assignment");
  } else if (right.is_double()) {
    assert( right.as_register_hi() == O3, "wrong register assignment");
    assert( right.as_register_lo() == O2, "wrong register assignment");
  } else if (right.is_float()) {
    assert( right.as_register() == O1, "wrong register assignment");
  } else { ShouldNotReachHere(); }

  if (result->type()->is_long()) {
    assert(O1 == item2regLO(result), "check" );
    assert(O0 == item2regHI(result), "check" );
  } else if (result->type()->is_double()) {
    assert(F0 == item2floatreg(result), "check" );
  } else if (result->type()->is_float()) {
    assert(F0 == item2floatreg(result), "check" );
  } else { ShouldNotReachHere(); }
}


void CodeEmitter::bang_stack_with_offset(int offset) {
  assert( offset < 0, "must bang with negative offset");
  __ set(offset, Gscratch);
  __ st(G0, SP, Gscratch);
}


void CodeEmitter::bang_stack() {
  bang_stack_with_offset(-os::vm_page_size());
}


void CodeEmitter::arithmetic_call_op(Bytecodes::Code code, RInfo temp_reg) {
  address entry = NULL;
  switch (code) {
  case Bytecodes::_lrem: entry = CAST_FROM_FN_PTR(address, SharedRuntime::lrem); break; // check if dividend is 0 is done elsewhere
  case Bytecodes::_ldiv: entry = CAST_FROM_FN_PTR(address, SharedRuntime::ldiv); break; // check if dividend is 0 is done elsewhere
  case Bytecodes::_lmul: entry = CAST_FROM_FN_PTR(address, SharedRuntime::lmul); break;
  case Bytecodes::_drem: entry = CAST_FROM_FN_PTR(address, SharedRuntime::drem); break;
  case Bytecodes::_frem: entry = CAST_FROM_FN_PTR(address, SharedRuntime::frem); break;
  default: ShouldNotReachHere();
  }
  bang_stack();
  __ call_VM_leaf(temp_reg.as_register(), entry);
}


// _ishl, _lshl, _ishr, _lshr, _iushr, _lushr
void CodeEmitter::shift_op(Bytecodes::Code code, RInfo dst_reg, Item* value, Item* count, RInfo unused) {
  assert(value->is_register(), "value should be in register");
  assert(count->is_register(), "count should be in register");

  switch (code) {
    case Bytecodes::_ishl:  __ sll   (item2reg(value), item2reg(count), dst_reg.as_register()); break;
    case Bytecodes::_ishr:  __ sra   (item2reg(value), item2reg(count), dst_reg.as_register()); break;
    case Bytecodes::_iushr: __ srl   (item2reg(value), item2reg(count), dst_reg.as_register()); break;
    case Bytecodes::_lshl:  __ lshl  (item2regHI(value), item2regLO(value), item2reg(count), dst_reg.as_register_hi(), dst_reg.as_register_lo(), Gscratch); break;
    case Bytecodes::_lshr:  __ lshr  (item2regHI(value), item2regLO(value), item2reg(count), dst_reg.as_register_hi(), dst_reg.as_register_lo(), Gscratch); break;
    case Bytecodes::_lushr: __ lushr (item2regHI(value), item2regLO(value), item2reg(count), dst_reg.as_register_hi(), dst_reg.as_register_lo(), Gscratch); break;
    default: ShouldNotReachHere();
  }
}


// _iand, _land, _ior, _lor, _ixor, _lxor
void CodeEmitter::logic_op(Bytecodes::Code code, RInfo dst_reg, Item* left, Item* right) {
  assert(left->is_register(), "left should be in register");
  assert(right->is_register(), "right should be in register");

  switch (code) {
    case Bytecodes::_iand: __ and3 (item2reg(left), item2reg(right), dst_reg.as_register()); break;
    case Bytecodes::_ior:  __ or3  (item2reg(left), item2reg(right), dst_reg.as_register()); break;
    case Bytecodes::_ixor: __ xor3 (item2reg(left), item2reg(right), dst_reg.as_register()); break;
    case Bytecodes::_land: __ and3 (item2regHI(left), item2regHI(right), dst_reg.as_register_hi()); __ and3(item2regLO(left), item2regLO(right), dst_reg.as_register_lo()); break;
    case Bytecodes::_lor:  __ or3  (item2regHI(left), item2regHI(right), dst_reg.as_register_hi()); __ or3 (item2regLO(left), item2regLO(right), dst_reg.as_register_lo()); break;
    case Bytecodes::_lxor: __ xor3 (item2regHI(left), item2regHI(right), dst_reg.as_register_hi()); __ xor3(item2regLO(left), item2regLO(right), dst_reg.as_register_lo()); break;
    default: ShouldNotReachHere();
  }
}


// _lcmp, _fcmpl, _fcmpg, _dcmpl, _dcmpg
void CodeEmitter::compare_op(Bytecodes::Code code, RInfo dst_reg, Item* left, Item* right) {
  if (left->type()->is_float()) {
    __ float_cmp(true, code == Bytecodes::_fcmpl ? -1 : 1, item2floatreg(left), item2floatreg(right), dst_reg.as_register());
  } else if (left->type()->is_double()) {
    __ float_cmp(false, code == Bytecodes::_dcmpl ? -1 : 1, item2floatreg(left), item2floatreg(right), dst_reg.as_register());
  } else if (code == Bytecodes::_lcmp) {
    __ lcmp(item2regHI(left), item2regLO(left), item2regHI(right), item2regLO(right), dst_reg.as_register());
  } else {
    ShouldNotReachHere();
  }
}


void CodeEmitter::call_convert_prolog(int nof_floats_to_save, ValueType* result) {
  ShouldNotReachHere();
}


void CodeEmitter::call_convert_epilog(int nof_floats_to_restore, ValueType* result) {
  ShouldNotReachHere();
}


void CodeEmitter::call_convert_op_check(Item* value, Item* result) {
  if (value->type()->is_long()) { 
    assert( value->get_register().as_register_lo() == O1, "wrong register assignment");
    assert( value->get_register().as_register_hi() == O0, "wrong register assignment");
  } else if (value->type()->is_double()) {
    assert( value->get_register().as_register_lo() == O1, "wrong register assignment");
    assert( value->get_register().as_register_hi() == O0, "wrong register assignment");
  } else if (value->type()->is_float()) {
    assert( value->get_register().as_register() == O0, "wrong register assignment");
  } else {
    ShouldNotReachHere();
  }

  if (result->type()->is_int()) {   
   assert(O0 == item2reg(result), "check" );
  } else if (result->type()->is_long()) {
   assert(O1 == item2regLO(result), "check" );
   assert(O0 == item2regHI(result), "check" );
  } else if (result->type()->is_double()) {
   assert(F0 == item2floatreg(result), "check" );
  } else if (result->type()->is_float()) {
   assert(F0 == item2floatreg(result), "check" );
  } else {
    ShouldNotReachHere();
  }
}


// _l2f, _l2d, _f2l, _f2i, _d2l, _d2i
void CodeEmitter::call_convert_op(Bytecodes::Code code, RInfo thread_reg) { 
  address entry = NULL;
  switch (code) {
    case Bytecodes::_l2f: entry = CAST_FROM_FN_PTR(address, SharedRuntime::l2f); break;
    case Bytecodes::_l2d: entry = CAST_FROM_FN_PTR(address, SharedRuntime::l2d); break;
    case Bytecodes::_f2l: entry = CAST_FROM_FN_PTR(address, SharedRuntime::f2l); break;
    case Bytecodes::_f2i: entry = CAST_FROM_FN_PTR(address, SharedRuntime::f2i); break;
    case Bytecodes::_d2l: entry = CAST_FROM_FN_PTR(address, SharedRuntime::d2l); break;
    case Bytecodes::_d2i: entry = CAST_FROM_FN_PTR(address, SharedRuntime::d2i); break;
    default: ShouldNotReachHere();
  }
  bang_stack();
  __ call_VM_leaf(thread_reg.as_register(), entry);
}


// _i2l, _i2f, _i2d, _l2i, _f2d, _d2f
// _i2b, _i2c, _i2s
void CodeEmitter::convert_op(Bytecodes::Code code, RInfo dst, Item* value) {
  switch(code) {
    case Bytecodes::_i2l: {
      assert(dst.is_long(), "check");
      Register rlo  = dst.as_register_lo();
      Register rhi  = dst.as_register_hi();
      Register rval = item2reg(value);    
      __ mov(rval, rlo);
      __ sra(rval, BitsPerWord-1, rhi);
      break;
    }
    case Bytecodes::_i2d: 
    case Bytecodes::_i2f: {
      bool is_double = (code == Bytecodes::_i2d);
      assert((!is_double && dst.is_float()) || (is_double && dst.is_double()), "check");
      Address addr;
      if (value->is_cached()) {
        int local_index = value->value()->as_AccessLocal()->index();
        reg_2_local(local_index, value->get_register(), T_INT);
        addr = frame()->local_address(local_index, false);
      } else {
        addr = stack_item2addr(value);
      }
      FloatRegister rdst = is_double ? dst.as_double_reg() : dst.as_float_reg();
      load(addr, rdst, T_FLOAT);
      __ fitof((is_double ? FloatRegister::D : FloatRegister::S), rdst, rdst);
      break;
    }
    case Bytecodes::_f2i:{
      // floating point operand register is overwritten with integer result
      assert(value->destroys_register(), "check");
      FloatRegister rsrc = value->get_register().as_float_reg();
      Register      rdst = dst.as_register();
      Label L;
      // result must be 0 if value is NaN; test by comparing value to itself
      __ fcmp(FloatRegister::S, Assembler::fcc0, rsrc, rsrc);
      if (!VM_Version::v9_instructions_work()) {
        __ nop();
      }
      __ fb(Assembler::f_unordered, true, Assembler::pn, L);  
      __ delayed()->mov(G0, rdst); // annuled if contents of rsrc is not NaN
      __ ftoi(FloatRegister::S, rsrc, rsrc);
      // move integer result from float register to int register
      move_item_to_cpu_register(*value, dst);
      __ bind (L);
      break;
    }
    case Bytecodes::_l2i: {
      assert(dst.is_word(), "check");
      Register rlo  = item2regLO(value);
      Register rhi  = item2regHI(value);
      Register rdst = dst.as_register();
      __ mov(rlo, rdst);
      break;
    } 
    case Bytecodes::_d2f: 
    case Bytecodes::_f2d: {
      bool is_double = (code == Bytecodes::_f2d);
      assert((!is_double && dst.is_float()) || (is_double && dst.is_double()), "check");
      FloatRegister rval = item2floatreg(value);
      FloatRegister rdst = is_double ? dst.as_double_reg() : dst.as_float_reg();
      FloatRegister::Width vw = is_double ? FloatRegister::S : FloatRegister::D;
      FloatRegister::Width dw = is_double ? FloatRegister::D : FloatRegister::S;
      __ ftof(vw, dw, rval, rdst);
      break;
    }
    case Bytecodes::_i2s:
    case Bytecodes::_i2b: {
      assert(dst.is_word(), "check");
      Register rval = item2reg(value);
      Register rdst = dst.as_register();
      int shift = (code == Bytecodes::_i2b) ? (BitsPerWord - T_BYTE_aelem_bytes * BitsPerByte) : (BitsPerWord - BitsPerShort);
      __ sll (rval, shift, rdst);
      __ sra (rdst, shift, rdst);
      break;
    }
    case Bytecodes::_i2c: {
      assert(dst.is_word(), "check");
      Register rval = item2reg(value);
      Register rdst = dst.as_register();
      int shift = BitsPerWord - T_CHAR_aelem_bytes * BitsPerByte;
      __ sll (rval, shift, rdst);
      __ srl (rdst, shift, rdst);
      break;
    }
 
    default: ShouldNotReachHere();
  }
}


void CodeEmitter::call_op(Bytecodes::Code code, const GrowableArray<BasicType>* sig_types, int size_of_arguments, CodeEmitInfo* info, bool optimized, bool needs_null_check, Item* receiver) {
  Register rec_reg = receiver == NULL ? noreg : receiver->get_register().as_register();
  if (UseC2CallingConventions) __ mov(SP, IsavedSP); 
  nops_for_zombie();

  switch (code) {
    case Bytecodes::_invokestatic:
      { StaticCallStub* stub = new StaticCallStub(__ pc(), sig_types);
        _call_stubs->append(stub);
        __ call(Runtime1::entry_for(Runtime1::resolve_invokestatic_id), relocInfo::static_call_type);
        __ delayed()->nop();
      }  
      break;
    case Bytecodes::_invokespecial:
    case Bytecodes::_invokevirtual:
    case Bytecodes::_invokeinterface:
    {
      // The planned technique is as follows:
      //    - if we need a null check, we move the receiver into the
      //      receiver register before the call and we emit the 
      //      null check in the delayed slot
      //    - if the null check is not needed, then we load the 
      //      receiver reg in the delayed slot; NOTE: this does
      //      not work woith safepoint code, as it blocks
      //      at call, does not execute the delayed instruction
      //      but tries to look at the receiver; once this is
      //      fixed, we may emit optimized code
      
      // See comment above:
      // if (needs_null_check) {
        // otherwise this move goes into the delay slot
      //   __ mov (rec_reg, RECEIVER);
      // }
      if (!PassParametersInRegisters) {
	NEEDS_CLEANUP;
	__ mov (rec_reg, RECEIVER);
      }
      if (code == Bytecodes::_invokespecial || optimized) {
        StaticCallStub* stub = new StaticCallStub(__ pc(), sig_types);
        _call_stubs->append(stub);
        __ call(Runtime1::entry_for(Runtime1::resolve_invoke_id), relocInfo::opt_virtual_call_type);
      } else {
        RelocationHolder rspec = virtual_call_Relocation::spec(__ pc());
        __ set_oop(NULL, Ginline_cache_reg);
        __ relocate(rspec);
        __ call(Runtime1::entry_for(Runtime1::resolve_invoke_id), relocInfo::none);
      }
      // null check in the delay slot of the call
      if (needs_null_check) {
        add_debug_info_here(info); // for NULL check
        __ delayed()->ld(RECEIVER, 0, G0);
      } else {
        __ delayed()->nop();
        // See comment above
        // __ delayed()->mov_or_nop(rec_reg, RECEIVER);
      }
    }
    break;
    default:       
      ShouldNotReachHere();
      break;       
  }
  add_call_info_here(info);
  if (UseC2CallingConventions) {
    __ mov(IsavedSP, SP);
    debug_only(__ mov(G0, IsavedSP);) // do not use it
  }
  verify_frame_size();
}


void CodeEmitter::new_instance(RInfo dst, ciInstanceKlass* klass, RInfo scratch1, RInfo scratch2,
			       RInfo scratch3, RInfo klass_reg, CodeEmitInfo* info) {
  assert(dst.as_register() == O0, "wrong register assignment");
  const Register obj = O0;
  const Register Rscratch1 = scratch1.as_register();
  const Register Rscratch2 = scratch2.as_register();
  const Register Rscratch3 = scratch3.as_register();
  const Register Rklass = klass_reg.as_register();
  NewInstanceStub* slow_case = new NewInstanceStub(Rklass, info);
  _slow_case_stubs->append(slow_case);
  jobject_2_reg_with_patching(klass_reg, klass, info);
  if (UseFastNewInstance && klass->is_loaded() && !klass->has_finalizer() && !UseSlowPath) {
    // make sure klass is fully initialized
    if (!klass->is_initialized()) {
      __ ld(Address(Rklass, 0, instanceKlass::init_state_offset_in_bytes() + sizeof(oopDesc)), Rscratch2);
      __ cmp(Rscratch2, instanceKlass::fully_initialized);
      __ br(Assembler::notEqual, true, Assembler::pn, *slow_case->entry());
      __ delayed()->mov_or_nop(Rklass, obj);
    }
    if (!klass->is_loaded()) {
      // Note: we will never get here because we do not know if a klass has finalizers, until
      // the class is loaded
      ShouldNotReachHere();
    } else {
      // allocate space for instance
      const int instance_size_in_words = align_object_size(klass->size_helper());
      __ allocate_object(obj, Rscratch1, Rscratch2, Rscratch3, oopDesc::header_size(), instance_size_in_words, Rklass, *slow_case->entry());
    }
  } else {
    __ br(Assembler::always, false, Assembler::pt, *slow_case->entry());
    __ delayed()->mov_or_nop(Rklass, obj);
  }
  __ bind(*slow_case->continuation());
  // allocation done
  // result is in obj (same as dst)
  __ verify_oop(obj);
}


void CodeEmitter::new_type_array(RInfo dst, BasicType elt_type, Item* length, RInfo scratch1, RInfo scratch2, RInfo scratch3, RInfo scratch4, RInfo klass_reg, CodeEmitInfo* info) {

  const Register Rscratch1 = scratch1.as_register();
  const Register Rscratch2 = scratch2.as_register();
  const Register Rscratch3 = scratch3.as_register();
  const Register Rscratch4 = scratch4.as_register();
  const Register Rklass = klass_reg.as_register();
  const Register obj = dst.as_register();
  const Register len = item2reg(length);
  
  NewTypeArrayStub* slow_case = new NewTypeArrayStub(elt_type, len, obj, info);
  _slow_case_stubs->append(slow_case);
  ciTypeArrayKlass* klass = ciTypeArrayKlass::make(elt_type);
  // We do not need patching as type array klasses are preloaded
  // jobject_2_reg_with_patching(klass_reg, klass, info);
  constant_2_reg(klass, klass_reg);

  if (UseFastNewTypeArray && !UseSlowPath) {
    // allocate space for array
    __ allocate_array(obj, len, Rscratch1, Rscratch2, Rscratch3, Rscratch4, arrayOopDesc::header_size(elt_type), type2aelembytes[elt_type], Rklass, *slow_case->entry());
  } else {
    __ br(Assembler::always, false, Assembler::pt, *slow_case->entry());
    __ delayed()->nop();
  }

  __ bind(*slow_case->continuation());
  // allocation done. result is in obj.
  __ verify_oop(obj);
}


void CodeEmitter::new_object_array(RInfo dst, ciKlass* elem_klass, Item* length, RInfo scratch1, RInfo scratch2, RInfo scratch3, RInfo scratch4, RInfo klass_reg, CodeEmitInfo* info) {
  const Register Rscratch1 = scratch1.as_register();
  const Register Rscratch2 = scratch2.as_register();
  const Register Rscratch3 = scratch3.as_register();
  const Register Rscratch4 = scratch4.as_register();
  const Register Rklass = klass_reg.as_register();
  const Register obj = dst.as_register();
  const Register len = item2reg(length);
  
  NewObjectArrayStub* slow_case = new NewObjectArrayStub(Rklass, len, obj, info);
  _slow_case_stubs->append(slow_case);
  jobject_2_reg_with_patching(klass_reg, ciObjArrayKlass::make(elem_klass), info);

  if (UseFastNewObjectArray && !UseSlowPath) {
    assert(oopSize == 4, "change scale factor below");
    // allocate space for array
    __ allocate_array(obj, len, Rscratch1, Rscratch2, Rscratch3, Rscratch4, arrayOopDesc::header_size(T_OBJECT), oopSize, Rklass, *slow_case->entry());
  } else {
    __ br(Assembler::always, false, Assembler::pt, *slow_case->entry());
    __ delayed()->nop();
  }

  __ bind(*slow_case->continuation());
  // allocation done.
  __ verify_oop(obj);
}


void CodeEmitter::new_multi_array(RInfo dst, ciKlass* klass, int rank, RInfo unused, CodeEmitInfo* info) {
  const Register obj = O0;
  constant_2_reg((jint)rank, FrameMap::_O1_RInfo);
  jobject_2_reg_with_patching(FrameMap::_O0_RInfo, klass, info);
  int offset_from_sp = frame::memory_parameter_word_sp_offset * wordSize;
  // move Address dst_addr(SP, 0, offset_from_sp) --> O2
  Register dim_reg = O2;
  __ mov(SP, O2);
  __ add(O2, offset_from_sp, O2);
  __ call(Runtime1::entry_for(Runtime1::new_multi_array_id), relocInfo::runtime_call_type);
  __ delayed()->nop();
  add_call_info_here(info);
  __ mov(obj, dst.as_register());
  __ verify_oop(obj);
}


// const Register Oexception  = O0; // exception being thrown
// const Register Oissuing_pc = O1; // where the exception is coming from


void CodeEmitter::throw_op(Item* exception, CodeEmitInfo* info) {
  __ mov(item2reg(exception), Oexception);
  add_debug_info_here(info);
  __ null_check(Oexception);
  // make sure that we do not create the pc that points into 
  // safepoint buffer -> safepoint at call!

  address pc_for_athrow  = __ pc();
  int pc_for_athrow_offset = __ offset();
  __ nop(); // pc_for_athrow can not point to itself (relocInfo restriction)

  RelocationHolder rspec = internal_word_Relocation::spec(pc_for_athrow);
  __ set((intptr_t)pc_for_athrow, Oissuing_pc, rspec);
  add_call_info(pc_for_athrow_offset, info); // for exception handler

  // if we jump, we would escape the method, use a compiler safepoint
  // and mark exception oop as object
  if (UseCompilerSafepoints) {
    RInfoCollection* oop_regs = new RInfoCollection();
    oop_regs->append(FrameMap::_O0_RInfo);
    assert(FrameMap::_O0_RInfo.as_register() == Oexception, "consistency check");
    add_debug_info_for_branch(info->_bci, oop_regs);
  }
  __ br(Assembler::always, false, Assembler::pt, _throw_entry_label);
  __ delayed()->nop();
}


void CodeEmitter::checkcast_op(RInfo dst, Item* obj_item, ciKlass* k, RInfo k_RInfo, RInfo klass_RInfo, CodeEmitInfo* info) {
  Register       obj = item2reg(obj_item); // object to test
  Register klass_reg = klass_RInfo.as_register(); // will contain klass of object to test
  Register  cast_reg = k_RInfo.as_register();  // will contain the value of klass k (specified by checkcast instruction)
  Register  temp_reg = O7;   //caller must kill O7!
  Register   dst_reg = dst.as_register();

  // dst_reg and obj may be the same
  assert_different_registers(dst_reg, klass_reg, cast_reg);
  assert_different_registers(obj, klass_reg, cast_reg);
  Label done;

  __ verify_oop(obj);
  __ tst(obj);
  __ br(Assembler::zero, false, Assembler::pn, done);
  __ delayed()->mov_or_nop(obj, dst_reg);

  jobject_2_reg_with_patching(k_RInfo, k, info); // move k -> k_RInfo
  // Note: klass_reg is a global and therefore may be destroyed -> load after eventually patching
  __ ld(Address(obj, 0, oopDesc::klass_offset_in_bytes()), klass_reg);

  int c1_offset = sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes();
  int c2_offset = sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes();

  bool use_ldx = false;
  if (VM_Version::v9_instructions_work()) {
    // save a memory access by loading 2 words at once
    // Note that this forces us to check the cache_2 before the cache_1.
    // Since cache_1 is the most recent miss, while cache_2 is the 2nd
    // most recent miss, I think there's no particular performance loss
    // by inverting the order of checking.
    // (See Klass::compute_and_set_is_subtype_of.)
    assert(c1_offset % longSize == 0, "is_a_cache_1 must be double aligned");
    assert(c2_offset-c1_offset == wordSize, "is_a_cache_2 must follow is_a_cache_1");
    use_ldx = true;
  }
  CheckCastStub* stub = new CheckCastStub(cast_reg, info, dst_reg);
  _slow_case_stubs->append(stub);
  if (UseSlowPath) {
    __ br(Assembler::always, false, Assembler::pt, *stub->entry());
    __ delayed()->nop();
  } else {
    if (k->is_loaded() && k->is_instance_klass() && k->as_instance_klass()->is_final()) {
      __ cmp(cast_reg, klass_reg);
    } else {
      // Check for first cache hit
      if (!use_ldx) {
        __ ld( klass_reg, c1_offset, temp_reg);
      } else {
        __ ldx(klass_reg, c1_offset, temp_reg);
      }
      __ cmp(cast_reg, temp_reg);
      __ br(Assembler::equal, false, Assembler::pt, done);
      if (!use_ldx) {
        __ delayed()->nop();
      } else {
        __ delayed()->srlx(temp_reg, 32, temp_reg);
      }

      // Check for second cache hit
      if (!use_ldx) {
        __ ld(klass_reg, c2_offset, temp_reg);
      }
      __ cmp(cast_reg, temp_reg);
    }
    __ br(Assembler::notEqual, false, Assembler::pn, *stub->entry());
    __ delayed()->nop();
  }
  __ bind(done);
  __ bind(*stub->continuation());
}


void CodeEmitter::instanceof_op(RInfo dst, Item* obj_item, ciKlass* k, RInfo k_RInfo, RInfo klass_RInfo, CodeEmitInfo* info) {
  Register       obj = item2reg(obj_item);
  Register klass_reg = klass_RInfo.as_register();
  Register  cast_reg = k_RInfo.as_register();
  Register  temp_reg = O7;   //caller must kill O7!
  Register   dst_reg = dst.as_register();

  assert_different_registers(dst_reg, obj, klass_reg, cast_reg);

  Label done;

  __ verify_oop(obj);
  __ tst(obj);
  __ br(Assembler::zero, false, Assembler::pn, done);
  __ delayed()->clr(dst_reg);

  jobject_2_reg_with_patching(k_RInfo, k, info);  // move k -> k_RInfo
    // Note: klass_reg is a global and therefore may be destroyed -> load after eventually patching
  __ ld(Address(obj, 0, oopDesc::klass_offset_in_bytes()), klass_reg);

  int c1_offset = sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes();
  int c2_offset = sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes();

  bool use_ldx = false;
  if (VM_Version::v9_instructions_work()) {
    // save a memory access by loading 2 words at once
    // Note that this forces us to check the cache_2 before the cache_1.
    // Since cache_1 is the most recent miss, while cache_2 is the 2nd
    // most recent miss, I think there's no particular performance loss
    // by inverting the order of checking.
    // (See Klass::compute_and_set_is_subtype_of.)
    assert(c1_offset % longSize == 0, "is_a_cache_1 must be double aligned");
    assert(c2_offset-c1_offset == wordSize, "is_a_cache_2 must follow is_a_cache_1");
    use_ldx = true;
  }

  InstanceOfStub* stub = new InstanceOfStub(klass_reg, dst_reg, cast_reg, info);
  _slow_case_stubs->append(stub);
  if (UseSlowPath) {
    __ br(Assembler::always, false, Assembler::pt, *stub->entry());
    __ delayed()->nop();
  } else {
    if (k->is_loaded() && k->is_instance_klass() && k->as_instance_klass()->is_final()) {
      __ cmp(cast_reg, klass_reg);
    } else {
      if (use_ldx) {
        __ ldx(klass_reg, c1_offset, temp_reg);
      } else {
        __ ld( klass_reg, c1_offset, temp_reg);
      }
      // check first cache entry
      __ cmp(cast_reg, temp_reg);
      __ br(Assembler::equal, true, Assembler::pt, done);
      __ delayed()->set(1, dst_reg);

      if (use_ldx) {
        __ srlx(temp_reg, 32, temp_reg);
      } else {
        __ ld(klass_reg, c2_offset, temp_reg);
      }
      // check second cache entry
      __ cmp(cast_reg, temp_reg);
    }
    __ br(Assembler::notEqual, false, Assembler::pn, *(stub->entry()));
    __ delayed()->set(1, dst_reg);
  }

  __ bind(*stub->continuation());
  __ bind(done);
}


void CodeEmitter::goto_op(BlockBegin* dst, int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  if (is_backward_branch) {
    add_debug_info_for_branch(bci, oop_regs);
  }
  __ br(Assembler::always, false, Assembler::pt, *(dst->label()));
  __ delayed()->nop();
}


void CodeEmitter::jsr_op (BlockBegin* dst, int bci) {
  Label Lab1;
  int offset1, offset2;
  // needed for compiler safepoints
  __ code()->relocate(__ pc(), relocInfo::safepoint_type);
  CodeEmitInfo info(bci, NULL, NULL, NULL);
  add_debug_info_here(&info);
  offset1 = __ offset();
  __ call(Lab1, relocInfo::none); // push return value on stack
  __ delayed()->add(O7, 4*wordSize, O0); // return address in O0
  __ bind(Lab1);
  __ br(Assembler::always, false, Assembler::pt, *(dst->label()));
  __ delayed()->nop();
  offset2 = __ offset();
  assert(offset2-offset1==4*wordSize, "change the 4*wordSize in the return address computation above");
}


void CodeEmitter::ret_op (int local_ix, RInfo cached_in_reg, int bci)  {
  // needed for compiler safepoints
  if (cached_in_reg.is_illegal()) {
    __ ld(frame()->local_address(local_ix, false), O0);
  } else {
    // make sure that there is code to patch the safepoint
    __ mov_or_nop(cached_in_reg.as_register(), O0);
  }
  if (UseCompilerSafepoints) {
    add_debug_info_for_branch(bci, NULL);
  }
  __ jmp(O0, G0);
  __ delayed()->nop();
}


If::Condition CodeEmitter::if_op(int phase, If::Condition cond, Item* x, Item* y, BlockBegin* t_dest, BlockBegin* f_dest, BlockBegin* u_dest, 
                     int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  ValueTag cmpTag = y->type()->tag();
  if (phase == 1) {
    switch (cmpTag) {
      case floatTag:  __ fcmp(FloatRegister::S, Assembler::fcc0, item2floatreg(x), item2floatreg(y)); break;
      case doubleTag: __ fcmp(FloatRegister::D, Assembler::fcc0, item2floatreg(x), item2floatreg(y)); break;
      case intTag: {
          if (y->is_constant()) {
            __ cmp(item2reg(x), item2int(y)); 
          } else {
            __ cmp(item2reg(x), item2reg(y)); 
          }
        }
        break;
      case objectTag: {
          if (y->is_constant()) {
            assert(y->get_jobject_constant()->is_null_object(), "wrong constant");
            __ tst(item2reg(x)); 
          } else {
            __ cmp(item2reg(x), item2reg(y)); 
          }
        }
        break;
      case longTag:
        { Register xlo = item2regLO(x);
          Register xhi = item2regHI(x);
          Register ylo = item2regLO(y);
          Register yhi = item2regHI(y);
          assert(x->destroys_register(), "check");
          __ subcc(xlo, ylo, xlo);
          __ subccc(xhi, yhi, xhi);
          if (cond == If::eql || cond == If::neq) {
            __ orcc (xhi, xlo, xhi); // set the condition code
          }
        }
        break;
      default: ShouldNotReachHere();
    }
  } else if (phase == 2) {
    assert(bci >= 0, "bci must be passed in");
    if (is_backward_branch) {
      add_debug_info_for_branch(bci, oop_regs);
    }

    if (cmpTag == floatTag || cmpTag == doubleTag) {
      assert(u_dest != NULL, "must have unordered successor");
      Assembler::Condition acond;
      switch (cond) {
        case If::eql: acond = Assembler::f_equal;    break;
        case If::neq: acond = Assembler::f_notEqual; break;
        case If::lss: acond = (u_dest == t_dest ? Assembler::f_unorderedOrLess           : Assembler::f_less);           break;
        case If::gtr: acond = (u_dest == t_dest ? Assembler::f_unorderedOrGreater        : Assembler::f_greater);        break;
        case If::leq: acond = (u_dest == t_dest ? Assembler::f_unorderedOrLessOrEqual    : Assembler::f_lessOrEqual);    break;
        case If::geq: acond = (u_dest == t_dest ? Assembler::f_unorderedOrGreaterOrEqual : Assembler::f_greaterOrEqual); break;
        default     : ShouldNotReachHere();
      };
      if (!VM_Version::v9_instructions_work()) {
        __ nop();
      }
      __ fb( acond, true, is_backward_branch ? Assembler::pt : Assembler::pn, *(t_dest->label()));  
      __ delayed()->nop();
    } else {
      Assembler::Condition acond;
      switch (cond) {
        case If::eql: acond = Assembler::equal;        break;
        case If::neq: acond = Assembler::notEqual;     break;
        case If::lss: acond = Assembler::less;         break;
        case If::leq: acond = Assembler::lessEqual;    break;
        case If::geq: acond = Assembler::greaterEqual; break;
        case If::gtr: acond = Assembler::greater;      break;
        default     : ShouldNotReachHere();
      };
      assert (cmpTag != longTag            || 
              acond == Assembler::equal    || 
              acond == Assembler::notEqual ||
              acond == Assembler::less     ||
              acond == Assembler::greaterEqual, "illegal condition for long compares");
      __ br (acond, false, is_backward_branch ? Assembler::pt : Assembler::pn, *(t_dest->label()));
      __ delayed()->nop();
    }
  } else {
    ShouldNotReachHere(); // only phases 1 and 2 exist
  }
  return cond;
}


void CodeEmitter::lookupswitch_op(Item* tag, int key, BlockBegin* dest, int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  if (Assembler::is_simm13(key)) {
    __ cmp(item2reg(tag), key);
  }
  else {
    __ set(key, O7);
    __ cmp(item2reg(tag), O7);
  }
  if (is_backward_branch) {
    add_debug_info_for_branch(bci, oop_regs);
  }
  __ br(Assembler::equal, false, Assembler::pt, *(dest->label()));
  __ delayed()->nop();
}


void CodeEmitter::lookupswitch_range_op(Item* tag, int low_key, int high_key, BlockBegin* dest, int bci, 
					bool is_backward_branch, const RInfoCollection* oop_regs) {
  Unimplemented();
}


void CodeEmitter::tableswitch_op(Item* tag, int key, BlockBegin* dest, int bci, bool is_backward_branch, const RInfoCollection* oop_regs) {
  if (Assembler::is_simm13(key)) {
    __ cmp(item2reg(tag), key);
  } else {
    __ set(key, O7);
    __ cmp(item2reg(tag), O7);
  }
  if (is_backward_branch) {
    add_debug_info_for_branch(bci, oop_regs);
  }
  __ br(Assembler::equal, false, Assembler::pt, *(dest->label()));
  __ delayed()->nop();
}


// uses G4 as temp
void CodeEmitter::return_op_prolog(int monitor_no, RInfo receiver) {
  assert (method()->is_synchronized(), "must be synchronized");
  RInfo obj_reg  = FrameMap::_O0_RInfo;
  RInfo lock_reg = FrameMap::_O1_RInfo;
  assert(monitor_no >= 0, "monitor slot must exist");
  if (method()->is_static()) {
    constant_2_reg(method()->holder()->java_mirror(), obj_reg);
  } else {
    if (receiver.is_illegal()) {
      // receiver is not cached, must load it
      load_receiver(obj_reg.as_register());
    } else {
      // receiver is ready
      obj_reg = receiver;
    }
  }
  // pass lock_reg, obj_reg as parameters to this function
  NEEDS_CLEANUP
  if (obj_reg.is_same(lock_reg)) {
    // receiver is in O1
    lock_reg = FrameMap::_O0_RInfo;
  }
  monitorexit(obj_reg.as_register(), lock_reg.as_register(), G4, monitor_no);
}


void CodeEmitter::return_op(Item* result) {
  assert(result == NULL || result->is_register(), "return value must be in register before return");
 
  if (compilation()->jvmpi_event_method_exit_enabled()) {
    jvmpi_method_exit();
  }

  __ restore();
  // We need some extra room in the return instruction, so we can patch it
  // for the safepoint code. We need a total of 4 bytes.
  // Also, we need relocation info. so the return instructions can be found easily    
  __ relocate(relocInfo::return_type);
  __ retl();
  __ delayed()->nop();
  __ nop();
}

void CodeEmitter::jvmpi_method_entry(CodeEmitInfo* info) {
  assert (jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY) || jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY2), "wrong call");
  constant_2_reg (method(), FrameMap::_O0_RInfo);
  if (!method()->is_static()) { 
    __ mov (I0, O1);
  } else {
    __ clr(O1);
  }
  __ call(Runtime1::entry_for(Runtime1::jvmpi_method_entry), relocInfo::runtime_call_type);
  __ delayed()->nop();
  add_call_info_here(info);
}

void CodeEmitter::jvmpi_method_exit() {
  assert(jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_EXIT), "wrong call");
  constant_2_reg (method(), FrameMap::_O0_RInfo);
  __ call(Runtime1::entry_for(Runtime1::jvmpi_method_exit), relocInfo::runtime_call_type);
  __ delayed()->nop();
}

void CodeEmitter::explicit_null_check (Item* item, CodeEmitInfo* info) {
  __ null_check(item2reg(item));
}

void CodeEmitter::explicit_div_by_zero_check (Item* item, CodeEmitInfo* info) {
  int pc_offset = __ offset();
  DivByZeroStub* stub = new DivByZeroStub(pc_offset);
  _slow_case_stubs->append(stub);
  assert(item->is_register(), "item must be in a register");

  add_call_info(pc_offset, info); // an exception looks like a call from here
  if (item->type()->is_double_word()) {
    __ orcc(item2regHI(item), item2regLO(item), G0);
  } else {
    __ orcc(item2reg(item), G0, G0);
  }
  __ br(Assembler::zero, false, Assembler::pn, *stub->entry());
  __ delayed()->nop();
}


void CodeEmitter::range_check (Item* array, Item* index, CodeEmitInfo* info) {
  assert(index->is_register() && array->is_register() && array->type()->is_object() && index->type()->is_int(), "wrong item state");

  // implicit null check 
  add_debug_info_here(info); 

  Register rarr = item2reg(array);
  Register rind = item2reg(index);
  
  __ ld(rarr, arrayOopDesc::length_offset_in_bytes(), O7);
  __ cmp(rind, O7);

  int offset = __ offset();
  add_call_info_here(info); // an exception looks like a call from here
  RangeCheckStub* slow_case = new RangeCheckStub(offset);
  _slow_case_stubs->append(slow_case);

  // if ((uint) rind) >= (uint) O7) then branch (implies (rind < 0) || (rind >= O7))
  __ br(Assembler::carryClear, false, Assembler::pn, *slow_case->entry());
  __ delayed()->nop();

  __ bind(*slow_case->continuation());
}


// Checks that value can be stored into array: 
//   value, index and array registers may not be destroyed and must be preserved in slow case !

void CodeEmitter::array_store_check(Item* array, Item* value, Item* index, RInfo tmp1, RInfo tmp2, CodeEmitInfo* info) {
  // set_bailout("temporary array_store_check bailout");
  if (!GenerateArrayStoreCheck) return;
  assert(value->is_register() && array->is_register() && array->type()->is_object() &&
           value->type()->is_object(), "wrong item state");

  Label    done;
  Register array_reg = item2reg(array);
  Register value_reg = item2reg(value);
  Register index_reg = index->is_constant() ?  noreg : item2reg(index);
  Register temp_reg = O7;
  Register array_klass_reg = tmp1.as_register();
  Register value_klass_reg = tmp2.as_register();

  // Wrong assertion! caller save registers are preserved in slow case
  // assert(index_reg == noreg || !FrameMap::is_caller_save_register(index_reg, true), "index is a caller save register but is not caller-saved");

  int index_const = index->is_constant() ? item2int(index) : -1;

  assert_different_registers(array_reg, value_reg, index_reg, array_klass_reg, value_klass_reg);

  __ tst(value_reg);
  __ br(Assembler::zero, false, Assembler::pn, done);
  __ delayed()->nop();

  add_debug_info_here(info);
  __ ld(Address(array_reg, 0, oopDesc::klass_offset_in_bytes()), array_klass_reg);
  // value_reg cannot be NULL
  __ ld(Address(value_reg, 0, oopDesc::klass_offset_in_bytes()), value_klass_reg);
  // get instance klass
  __ ld(Address(array_klass_reg, 0,
                objArrayKlass::element_klass_offset_in_bytes() + sizeof(oopDesc)), 
                array_klass_reg);

  int c1_offset = sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes();
  int c2_offset = sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes();

  bool use_ldx = false;
  if (VM_Version::v9_instructions_work()) {
    // save a memory access by loading 2 words at once
    // Note that this forces us to check the cache_2 before the cache_1.
    // Since cache_1 is the most recent miss, while cache_2 is the 2nd
    // most recent miss, I think there's no particular performance loss
    // by inverting the order of checking.
    // (See Klass::compute_and_set_is_subtype_of.)
    assert(c1_offset % longSize == 0, "is_a_cache_1 must be double aligned");
    assert(c2_offset-c1_offset == wordSize, "is_a_cache_2 must follow is_a_cache_1");
    use_ldx = true;
  }         
  ArrayStoreCheckStub* stub = new ArrayStoreCheckStub(array_reg, value_reg, index_reg, index_const, info, esp_offset());
  _slow_case_stubs->append(stub);
  if (UseSlowPath) {
    __ br(Assembler::always, false, Assembler::pt, *stub->entry());
    __ delayed()->nop();
  } else {
    if (!use_ldx) {
      __ ld(value_klass_reg, c1_offset, temp_reg);
    } else {
      __ ldx(value_klass_reg, c1_offset, temp_reg);
    }
    __ cmp(array_klass_reg, temp_reg);
    __ br(Assembler::equal, false, Assembler::pn, done);
    if (!use_ldx) {
      __ delayed()->nop();   
    } else {
      __ delayed()->srlx(temp_reg, 32, temp_reg);
    }

    if (!use_ldx) {
      __ ld(value_klass_reg, c2_offset, temp_reg);
    }

    __ cmp(array_klass_reg, temp_reg);
    __ br(Assembler::notEqual, false, Assembler::pn, *stub->entry()); 
    __ delayed()->nop();     
  }
  __ bind(done);
  __ bind(*stub->continuation());
}


void CodeEmitter::emit_slow_case_stubs() {
// slow cases do not belong to the stubs of CodeBuffer
  for (int i = 0; i < _slow_case_stubs->length(); i++) {
    check_codespace();
    if (must_bailout()) break;

    CodeStub* s = _slow_case_stubs->at(i);
    s->emit_code(this);
    debug_only(s->assert_no_unbound_labels();)
  }
}


void CodeEmitter::emit_call_stubs() {
  __ code()->set_stubs_begin(__ pc());

  // Interpreter entry point (argument marshalling)
  _offsets->_iep_offset = __ offset();
  interpreter_to_compiler_calling_convention();

  for (int i = 0; i < _call_stubs->length(); i++) {
    check_codespace();
    if (must_bailout()) break;
    
    CodeStub* s = _call_stubs->at(i);
    s->emit_code(this);
    debug_only(s->assert_no_unbound_labels();)
  }
  __ code()->set_stubs_end( __ pc());
}


void CodeEmitter::emit_constants() {
  _const_table->emit_entries(masm());
}


// When entering the handler, the exception pc is in %O1 and exception oop is in %O0
// In case that there is an exception handler: uses G4 (temp), I0(receiver), L0 (saved exception),
int CodeEmitter::emit_exception_handler(int max_spills, RInfo cached_receiver) {
  int offset = __ offset();
  restore_frame(G4);

  if (!_throw_entry_label.is_unused()) {
    assert(_throw_entry_label.is_unbound(), "who did it?");
    __ bind (_throw_entry_label);
  }

  __ verify_oop(Oexception);
  if (C1TraceMethod) trace_method_entry(1);
  if (method()->has_exception_handlers()) {
    // O0: exception
    // O1: issuing pc
    __ call(Runtime1::entry_for(Runtime1::handle_exception_id), relocInfo::runtime_call_type);
    __ delayed()->nop();
    // exception handler for particular exception doesn't exist
    // => unwind activation and forward exception to caller
    // O0: exception
  } else {
    // There is no handler in this method, we continue in caller
    // reset the implicit flag
    store(G0, Gthread, JavaThread::is_handling_implicit_exception_offset_in_bytes(), T_INT);
  }
  // unlock the receiver/klass if necessary
  // O0: exception
  __ verify_oop(Oexception);
  if (method()->is_synchronized()) {
    RInfo obj_reg = FrameMap::_I0_RInfo;

    // preserve the exception object but make sure the cached receiver is not destroyed
    Register saved_exception = L0;
    if (cached_receiver.is_valid() && cached_receiver.is_same(FrameMap::_L0_RInfo)) {
      // the receiver is cached in L0, use a different register to preserve the exception
      saved_exception = L2;
    }
    __ mov(Oexception, saved_exception); // save exception object
    if (method()->is_static()) {
      constant_2_reg(method()->holder()->java_mirror(), obj_reg);
    } else {    
      if (cached_receiver.is_valid()) {
	__ mov (cached_receiver.as_register(), obj_reg.as_register());
      } else {
	load_receiver(obj_reg.as_register());
      }
    }  
    monitorexit(obj_reg.as_register(), L1, G4, 0);
    __ mov(saved_exception, Oexception); // restore exception object
  }
  __ call(Runtime1::entry_for(Runtime1::unwind_exception_id), relocInfo::runtime_call_type);
  __ delayed()->nop();
  debug_only(__ stop("should have gone to the caller");)

  return offset;
}


void CodeEmitter::emit_osr_entry(int max_spills, int number_of_locks, Label* continuation, int osr_bci) {

  // On-stack-replacement entry sequence (interpreter frame layout described in interpreter_sparc.cpp):
  //
  //   1. Extend the interpreted activation frame to make space for the method's arguments
  //      in the compiled activation, which are stored in its caller's frame.
  //   2. Create a new compiled activation whose return address is the osr_frame_return stub, which will
  //      return from the interpreted activation that was "osr'd" (but not actually replaced).
  //   3. Initialize local variables in the compiled activation.  The expression stack must be empty
  //      at the osr_bci; it is not initialized.
  //   4. Jump to the continuation address in compiled code to resume execution.

  // OSR entry point
  _offsets->_osr_offset = __ offset();

  // Make the compiled activation normally return to the osr_frame_return stub, which returns from the interpreted
  // activation, by pretending the stub follows a normal call to the method.  The "call site" address is passed in O7.
  __ set((int)Runtime1::entry_for(Runtime1::osr_frame_return_id) - frame::pc_return_offset * BytesPerWord , O7);  

  // The osr_frame_return and osr_unwind_exception stubs must restore the SP of the activation that invoked this
  // interpreted activation, since the interpreter extends a caller's frame to allocate a method's non-parameter
  // locals next to its parameters.  Compute the necessary adjustment into a local register that will be saved
  // within the interpreted frame for use by the stubs.
  // Note: This is the same calculation done by InterpreterMacroAssembler::compute_extra_locals_size_in_bytes().
  const Register locals_size = O0;
  const Register args_size   = O1;
  const Register delta       = O2;
  __ lduh(Lmethod, methodOopDesc::size_of_parameters_offset_in_bytes(), args_size);
  __ lduh(Lmethod, methodOopDesc::size_of_locals_offset_in_bytes()    , locals_size);
  __ subcc(locals_size, args_size, delta);                          // adjustment in words
  if (VM_Version::v9_instructions_work()) {
    __ movcc(Assembler::negative, false, Assembler::icc, G0, delta);  // set to 0 if adjustment < 0
  } else {
    Label skip_move;
    // br(test (e.g., negative), annull_delay_slot, target_address)
    __ br(Assembler::negative, true, Assembler::pt, skip_move);
    __ delayed()->mov(G0, delta);
    __ bind(skip_move);
  }
  __ round_to(delta, WordsPerLong);                                 // SP must be 2-word aligned
  __ sll(delta, LogBytesPerWord, Lscratch);                         // adjustment in bytes

  // Allocate the compiled method's parameter locals in the interpreted activation (its "caller".)
  __ sub(SP, round_to(method()->arg_size(), WordsPerLong) * BytesPerWord, SP);

  // Save pointers into the interpreted activation for initializing the compiled activation.
  __ mov(Llocals, Garg0);
  __ mov(Lmonitors, Garg1);

  // Create a frame for the compiled activation.
  build_frame(0);

  // Initialize local variables in the compiled activation.
  int nof_locals = method()->max_locals();
  const CachedLocals* cached_locals = frame()->cached_locals();
  for (int i = 0; i < nof_locals; i++) {      
    RInfo reg = cached_locals != NULL ? cached_locals->get_cache_reg(i) : FrameMap::_noRInfo;
    if (reg.is_illegal()) { 
      __ ld(Garg0, -i * BytesPerWord, O7);
      __ st(O7, frame()->local_address(i, false));
    } else {
      __ ld(Garg0, -i * BytesPerWord, reg.as_register());
    }
  }

  // Initialize monitors in the compiled activation.
  // Note: interpreter uses BasicObjectLock for monitors while compiler1 uses BasicLock
  { assert(frame::interpreter_frame_monitor_size() == BasicObjectLock::size(), "adjust code below");
    int i;
    for (i = 0; i < number_of_locks; i++) {
      Address compiled_monitor_address    = frame()->monitor_address(number_of_locks - i - 1);
      Address interpreter_monitor_address = Address(Garg1, 0, i * BasicObjectLock::size() * BytesPerWord);
#ifdef ASSERT
      // verify the interpreter's monitor has a non-null object
      { Label L;
	load(interpreter_monitor_address, O7, T_INT, NULL, BasicObjectLock::obj_offset_in_bytes());
	__ cmp(G0, O7);
	__ br(Assembler::notEqual, false, Assembler::pt, L);
	__ delayed()->nop();
	__ stop("locked object is NULL");
	__ bind(L);
      }
#endif
      // Copy the lock field into the compiled activation.
      load(interpreter_monitor_address, O7, T_INT, NULL, BasicObjectLock::lock_offset_in_bytes());
      store(O7, compiled_monitor_address,   T_INT, NULL, BasicLock::displaced_header_offset_in_bytes());
    }
  }
  
  // Convert bci's to return addresses if within subroutines.
  if (AcceptJsrForOSR) {
    GrowableArray<int>* adr_locals = compilation()->get_address_map(osr_bci);    
    for (int i = 0; i < adr_locals->length(); i++) {
      int local_index = adr_locals->at(i);
      Address locAddr = frame()->local_address(local_index, false);      
      load(locAddr, O0, T_INT, NULL);
      __ get_pc(O1);
      __ call(Runtime1::entry_for(Runtime1::address_bci_2_pc_id), relocInfo::runtime_call_type);
      __ delayed()->nop();
      {
	CodeEmitInfo info(osr_bci,NULL,NULL,NULL);
        add_call_info_here(&info);
      }
      store(O0, locAddr, T_INT, NULL);
    }
  }

  // Continue execution in compiled code.
  __ br(Assembler::always, false, Assembler::pt, *continuation);
  __ delayed()->nop();
}


void CodeEmitter::add_nop() {
  __ nop();
}


// use this to mark a continuation attached to a bci (used for jsr) 
void CodeEmitter::add_nop_with_info(CodeEmitInfo* info) {
  add_debug_info_here(info);
  __ nop();
}


void CodeEmitter::check_codespace() {
  if (_masm->code()->code_end() + 1*K >= _masm->code()->code_limit()) {
    set_bailout("CodeBuffer overflow");
  }
}


#ifndef PRODUCT
void CodeEmitter::print() {
}
#endif


void CodeEmitter::align_backward_branch() {
  // __ align (OptoLoopAlignment);
}


// This is the fast version of java.lang.String.compare; it has no
// OSR-entry and therefore, we generate a slow version for OSR's
void CodeEmitter::string_compare() {
  assert(OptimizeLibraryCalls, "should not call this");

  Label Ldone;

  Register result = I0;
  {
    // Get a pointer to the first character of string0 in tmp0 and get string0.count in str0
    // Get a pointer to the first character of string1 in tmp1 and get string1.count in str1
    // Also, get string0.count-string1.count in o7 and get the condition code set
    // Note: some instructions have been hoisted for better instruction scheduling

    Register str0 = I0;
    Register str1 = I1;
    Register tmp0 = L0;
    Register tmp1 = L1;
    Register tmp2 = L2;

    CallingConvention* args = FrameMap::calling_convention(method());
    assert(args->length() == 2, "String::compareTo takes only two arguments");

    ArgumentLocation string0 = args->arg_at(0);
    ArgumentLocation string1 = args->arg_at(1);
    if (string0.is_register_arg()) {
      assert(string0.incoming_reg_location().as_register() == str0, "should be in I0");
    } else {
      load(string0.incoming_stack_location(), str0, T_OBJECT);
    }
    if (string1.is_register_arg()) {
      assert(string1.incoming_reg_location().as_register() == str1, "should be in I0");
    } else {
      load(string1.incoming_stack_location(), str1, T_OBJECT);
    }

    int  value_offset = java_lang_String:: value_offset_in_bytes(); // char array
    int offset_offset = java_lang_String::offset_offset_in_bytes(); // first character position
    int  count_offset = java_lang_String:: count_offset_in_bytes();

    __ ld(Address(str0, 0,  value_offset), tmp0);
    __ ld(Address(str0, 0, offset_offset), tmp2);
    __ add(tmp0, arrayOopDesc::base_offset_in_bytes(T_CHAR), tmp0);
    __ ld(Address(str0, 0, count_offset), str0);
    __ sll(tmp2, exact_log2(sizeof(jchar)), tmp2);

    // str1 may be null
    { CodeEmitInfo info(0, NULL, NULL);
      add_debug_info_here(&info);
    }

    __ ld(Address(str1, 0,  value_offset), tmp1);
    __ add(tmp0, tmp2, tmp0);

    __ ld(Address(str1, 0, offset_offset), tmp2);
    __ add(tmp1, arrayOopDesc::base_offset_in_bytes(T_CHAR), tmp1);
    __ ld(Address(str1, 0, count_offset), str1);
    __ sll(tmp2, exact_log2(sizeof(jchar)), tmp2);
    __ subcc(str0, str1, O7);
    __ add(tmp1, tmp2, tmp1);
  }

  {
    // Compute the minimum of the string lengths, scale it and store it in limit
    Register count0 = I0;
    Register count1 = I1;
    Register limit  = L3;

    Label Lskip;
    __ sll(count0, exact_log2(sizeof(jchar)), limit);             // string0 is shorter
    __ br(Assembler::greater, true, Assembler::pt, Lskip);
    __ delayed()->sll(count1, exact_log2(sizeof(jchar)), limit);  // string1 is shorter
    __ bind(Lskip);

    // If either string is empty (or both of them) the result is the difference in lengths
    __ cmp(limit, 0);
    __ br(Assembler::equal, true, Assembler::pn, Ldone);
    __ delayed()->mov(O7, result);  // result is difference in lengths
  }

  {
    // Neither string is empty
    Label Lloop;

    Register base0 = L0;
    Register base1 = L1;
    Register chr0  = I0;
    Register chr1  = I1;
    Register limit = L3;

    // Shift base0 and base1 to the end of the arrays, negate limit
    __ add(base0, limit, base0);
    __ add(base1, limit, base1);
    __ neg(limit);  // limit = -min{string0.count, strin1.count}

    __ lduh(base0, limit, chr0);
    __ bind(Lloop);
    __ lduh(base1, limit, chr1);
    __ subcc(chr0, chr1, chr0);
    __ br(Assembler::notZero, false, Assembler::pt, Ldone);
    assert(chr0 == result, "result must be pre-placed");
    __ delayed()->inccc(limit, sizeof(jchar));
    __ br(Assembler::notZero, false, Assembler::pt, Lloop);
    __ delayed()->lduh(base0, limit, chr0);
  }

  // If strings are equal up to min length, return the length difference.
  __ mov(O7, result);

  // Otherwise, return the difference between the first mismatched chars.
  __ bind(Ldone);
  return_op(NULL);
}


void CodeEmitter::math_intrinsic(ciMethod::IntrinsicId id, RInfo dst, Item* value, RInfo thread_reg) {
  switch (id) {
    case methodOopDesc::_dsin:
    case methodOopDesc::_dcos: {
      assert(thread_reg.is_valid(), "preserve the thread object for performance reasons");
      assert(value->is_register() && value->get_register().is_same(FrameMap::_O0_O1_RInfo), "the C call expects its argument in O0/O1");
      assert(value->destroys_register(), "check");
      assert(dst.is_same(FrameMap::_f0_double_RInfo), "the result will be in f0/f1");
      address runtime_entry = NULL;
      if (id == methodOopDesc::_dsin) {
	runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dsin);
      } else {
	runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dcos);
      }
      bang_stack();
      __ call_VM_leaf(thread_reg.as_register(), runtime_entry);
      break;
    }
    case methodOopDesc::_dsqrt: {
      assert(!thread_reg.is_valid(), "there is no need for a thread_reg for dsqrt");
      FloatRegister src_reg = value->get_register().as_double_reg();
      FloatRegister dst_reg = dst.as_double_reg();
      __ fsqrt(FloatRegister::D, src_reg, dst_reg);
      break;
    }
    default: {
      ShouldNotReachHere();
      break;
    }
  }
}


// java.lang.Math.sin and cos
void CodeEmitter::triglib(ciMethod::IntrinsicId id) {
  assert(OptimizeLibraryCalls, "should not call this");

  CallingConvention* args = FrameMap::calling_convention(method());
  assert(args->length() == 2, "Triglib functions (sine/cosine) take one double argument.");

  // SharedRuntime::dsin/dcos expect their argument in O0/O1, load it from stack
  ArgumentLocation x = args->arg_at(0);
  assert(x.is_stack_arg(), "double values are passed on the stack");

  load(frame()->local_address(0, true /*is_double_word*/, true  /*is_hi_word*/), FrameMap::_O0_O1_RInfo.as_register_hi(), T_INT);
  load(frame()->local_address(0, true /*is_double_word*/, false /*is_hi_word*/), FrameMap::_O0_O1_RInfo.as_register_lo(), T_INT);    

  address runtime_entry = NULL;
  switch (id) {
  case methodOopDesc::_dsin:  runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dsin); break;
  case methodOopDesc::_dcos:  runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dcos); break;
  default: ShouldNotReachHere(); break;
  }

  Register thread_cache = L0;  // preserve Gthread around the C call
  bang_stack();
  __ call_VM_leaf(thread_cache, runtime_entry);

  return_op(NULL);
}
 
 // i486 compatibility
      
void CodeEmitter::set_fpu_stack_empty() {}
void CodeEmitter::set_fpu_result      (RInfo reg) { }
void CodeEmitter::remove_fpu_result   (RInfo reg) { }
void CodeEmitter::restore_cached_receiver()       { }


bool CodeEmitter::is_empty_fpu_stack() {
  return true; // make assertion happy
}


void CodeEmitter::handler_entry() {
  // see code for restore frame
  int framesize = frame()->om_framesize() * wordSize;
  __ add(FP, -framesize, SP);
}


int CodeEmitter::frame_size() {
  assert(frame()->om_framesize() >= 0, "wrong frame_size");
  return frame()->om_framesize();
}

//-----------------arraycopy helpers-----------------------


// This is the inlined version of java.lang.System.arraycopy;
// Currently we do not know the type of arrays and must do tests (later IR will provide us
// with that information).
// There are three levels of array copy:
//   - inlined tight loop or rep_move for character arrays;
//   - simple leaf routine(s) (SharedRuntime) that copies the array (all pre-checks succeeded)
//   - call to native: used if any of the pre-checks failed


void CodeEmitter::arraycopy(Item* src_item, Item* src_pos_item, Item* dst_item, Item* dst_pos_item, Item* length_item, RInfo scratch1, RInfo scratch2, RInfo tmp, CodeEmitInfo* info) {
  assert(!UseC2CallingConventions, "not implemented");
  Register src  = src_item->get_register().as_register();
  Register dst  = dst_item->get_register().as_register();
  Register tmp1 = scratch1.as_register();
  Register thread = tmp.as_register();
  Register src_pos = src_pos_item->get_register().as_register();
  Register dst_pos = dst_pos_item->get_register().as_register();
  Register length  = length_item->get_register().as_register();
  Register src_lng = O7;
  Register dst_lng = scratch2.as_register();;

  ArrayCopyDesc* desc = new ArrayCopyDesc();
  desc->set_src(src);
  desc->set_src_pos_reg(src_pos);
  desc->set_dst(dst);
  desc->set_dst_pos_reg(dst_pos);
  desc->set_length_reg(length);
  desc->set_tmp_reg(thread);
  // we do not currently accept constants in Items

  ArrayCopyStub* stub = new ArrayCopyStub(desc, info, esp_offset());
  _slow_case_stubs->append(stub);

  //------------------pre-checks-------------------------
  // if any of the prechecks fail, we jump to the array copy stub

  // test for NULL;
  __ br_zero(Assembler::zero, false, Assembler::pn, src, *stub->entry());
  __ delayed()->mov(src, O4); 

  __ br_zero(Assembler::zero, false, Assembler::pn, dst, *stub->entry());
  __ delayed()->ld(src, arrayOopDesc::length_offset_in_bytes(), src_lng);  // src is not NULL

  // check if negative
  __ br_zero(Assembler::less, false, Assembler::pn, length, *stub->entry());
  __ delayed()->ld(dst, arrayOopDesc::length_offset_in_bytes(), dst_lng);  // dst is not NULL

  __ br_zero(Assembler::less, false, Assembler::pn, src_pos, *stub->entry());
  __ delayed()->mov(src_pos, O3);

  __ br_zero(Assembler::less, false, Assembler::pn, dst_pos, *stub->entry());
  __ delayed()->mov(dst, O2);

  // check length of arrays
  __ add(src_pos, length, tmp1);
  __ cmp(src_lng, tmp1);
  __ br(Assembler::carrySet, false, Assembler::pn, *stub->entry());
  __ delayed()->mov(dst_pos, O1);

  __ add(dst_pos, length, tmp1);
  __ cmp(dst_lng, tmp1);
  __ br(Assembler::carrySet, false, Assembler::pn, *stub->entry());
  __ delayed()->mov(length,  O0);

  // Now all parameters are set for the call:
  //   length  : O0
  //   dst_pos : O1
  //   dst     : O2
  //   src_pos : O3
  //   src     : O4
  //
  // Note that the arguments for this call a reversed ordered
  address entry = CAST_FROM_FN_PTR(address, Runtime1::arraycopy);
  __ call_VM_leaf(thread, entry);
  
  __ br_zero(Assembler::less, false, Assembler::pn, O0, *stub->entry());
  __ delayed()->nop();

  __ bind(*stub->continuation());

}


void CodeEmitter::emit_accessor_code(int offset, BasicType type) {
  assert (!compilation()->jvmpi_event_method_enabled(), "does not work with jvmpi") ;
  assert(!method()->is_synchronized(), "cannot emit code for synchronized");
  // entry point
  __ align(CodeEntryAlignment);
  _offsets->_ep_offset = __ offset();
  if (C1Breakpoint)  __ breakpoint_trap();
  check_icache();

  // verified entry point
  _offsets->_vep_offset = __ offset();
  __ bind (_vep_label);
  if (C1Breakpoint) __ breakpoint_trap();
  _offsets->_code_offset = __ offset();
  __ verify_FPU(0, "method_entry");
  if (type == T_FLOAT) {
    Address src_addr = Address(O0, 0, offset*wordSize);
    load(src_addr, F0, T_FLOAT, NULL);
  } else if (type == T_DOUBLE) {
    Address src_addr = Address(O0, 0, offset*wordSize);
    load(src_addr, F0, T_DOUBLE, NULL);
  } else if (type == T_LONG) {
    __ ld (O0, offset    *wordSize, O1); // hi word
    __ ld (O0, (offset+1)*wordSize, O0); // lo word
  } else {
    __ ld (O0, offset*wordSize, O0);
  }

  __ relocate(relocInfo::return_type);
  __ retl();
  __ delayed()->nop();
  __ nop();
}


void CodeEmitter::fpop() {
  // do nothing
}

