/*
 * @(#)c1_CodeEmitter_sparc.hpp	1.42 01/02/09 14:43:12
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


 private:
  Label _vep_label;

  void verify_frame_size();
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // 
  // Sparc load/store emission
  //
  // The sparc ld/st instructions cannot accomodate displacements > 13 bits long.
  // The following "pseudo" sparc instructions (load/store) make it easier to use the indexed addressing mode
  // by allowing 32 bit displacements:
  //
  //    When disp <= 13 bits long, a single load or store instruction is emitted with (disp + [d]).
  //    When disp >  13 bits long, code is emitted to set the displacement into the O7 register, 
  //       and then a load or store is emitted with ([O7] + [d]).
  //

  // load/store with 32 bit displacement
  void load(Register s, int disp, Register d, BasicType ld_type, CodeEmitInfo* info = NULL);
  void store(Register d, Register s1, int disp, BasicType st_type, CodeEmitInfo* info = NULL);

  // loadf/storef with 32 bit displacement
  void load(Register s, int disp, FloatRegister d, BasicType ld_type, CodeEmitInfo* info = NULL);
  void store(FloatRegister d, Register s1, int disp, BasicType st_type, CodeEmitInfo* info = NULL);

  // convienence methods for calling load/store with an Address
  void load(const Address& a, Register d, BasicType ld_type, CodeEmitInfo* info = NULL, int offset = 0);
  void store(Register d, const Address& a, BasicType st_type, CodeEmitInfo* info = NULL, int offset = 0);
  void load(const Address& a, FloatRegister d, BasicType ld_type, CodeEmitInfo* info = NULL, int offset = 0);
  void store(FloatRegister d, const Address& a, BasicType st_type, CodeEmitInfo* info = NULL, int offset = 0);

  //
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  void constant_2_reg(jint      con, RInfo reg);
  void constant_2_reg(jfloat    con, RInfo reg);
  void constant_2_reg(jdouble   con, RInfo reg);
  void constant_2_reg(jlong     con, RInfo reg);
  void constant_2_reg(ciObject* con, RInfo reg);

  void  float_constant_2_cpu_reg (jfloat  con, RInfo reg);
  void double_constant_2_cpu_reg (jdouble con, RInfo reg);
  void    int_constant_2_fpu_reg (jint    con, RInfo reg);
  void   long_constant_2_fpu_reg (jlong   con, RInfo reg);

  Address  stack_Address(BasicType type, Register r, int disp_bytes, bool for_hi_word);
  Address  stack_AddressLO(BasicType type, Register r, int disp_bytes);
  Address  stack_AddressHI(BasicType type, Register r, int disp_bytes);

  // For heap objects only (respects endian conventions for heap long and double)
  Address  item2addr_with_offset   (Item* item, int offset);
  Address  item2addr_with_offsetLO (Item* item, int offset);
  Address  item2addr_with_offsetHI (Item* item, int offset);

  Address  item2array_addr   (Item* array, Item* index, BasicType dst_type, int offset = 0);
  Address  item2array_addrLO (Item* array, Item* index, BasicType dst_type);
  Address  item2array_addrHI (Item* array, Item* index, BasicType dst_type);

  Address  heap_Address(BasicType type, Register r, int disp_bytes, bool for_hi_word);
  Address  heap_AddressLO(BasicType type, Register r, int disp_bytes);
  Address  heap_AddressHI(BasicType type, Register r, int disp_bytes);

  // helper for indexed_load/store
  void indexed_address_and_range_check(BasicType dst_type, Register rarr, Register rind, Register rdst, CodeEmitInfo* info);
  void constant_address_and_range_check(BasicType dst_type, Register rarr, int index, Register rdst, CodeEmitInfo* info);

  void untyped_stackitem_2_reg(RInfo reg, const Item* item);

  // code emission
  void nops_for_zombie();

  void interpreter_to_compiler_calling_convention ();
  void setup_locals(CallingConvention* args);
  void pass_oop_to_native(int java_arg_index, Address java_arg_addr, Argument jni_arg);

  void restore_frame(Register scratch);

  void ac_check_for_negative  (Register reg, int v, Label* l);
  void ac_check_length (Register arr, Register lng_reg, Register pos_reg, int pos_v, Register tmp, Label* l);
  void ac_memory_start (Register arr, Register pos_reg, int pos_v, Register tmp, BasicType type);

  void monitorenter(Register obj_reg, Register lock_reg, Register Hdr, Register scratch, int monitor_no, CodeEmitInfo* info);
  void monitorexit (Register obj_reg, Register lock_reg, Register hdr, int monitor_no);

  int shift_amount(BasicType t);


//---------------------------------PUBLIC-------------------------------------
 public:

  // ======================== code checks   ==================================
  void arithmetic_call_op_check(RInfo left, RInfo right, Item* result);
  void call_convert_op_check(Item* value, Item* result);

  // item conversions
  FloatRegister item2floatreg (Item* item);
  void constant_2_reg(ValueType* type, RInfo reg);
