/*
 * @(#)c1_CodeGenerator_sparc.cpp	1.21 01/04/30 16:09:55
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_CodeGenerator_sparc.cpp.incl"


// Notes to OopMap generations:
// - because the pc-offset is attached to each OopMap, we must create
//   a new OopMap for each gc-point

//--------------------------------------------------------------
class SpillLockReg: public StackObj {
 private:
  ValueGen* _vg;
  RInfo     _reg;
 public:
  SpillLockReg::SpillLockReg(ValueGen* vg, RInfo reg): _vg(vg), _reg(reg) {
    vg->ra()->incr_spill_lock(reg);
  }
  SpillLockReg::~SpillLockReg() {
    _vg->ra()->decr_spill_lock(_reg);
  }

};


//--------------------------------------------------------------
//               ValueGen
//--------------------------------------------------------------

// returns true if reg could be smashed by a callee.
// If is_C_call, return true when register could be smashed by a C-call.
bool ValueGen::is_caller_save_register(RInfo reg, bool is_C_call) {
  return FrameMap::is_caller_save_register(reg, is_C_call);
}



RInfo ValueGen::noRInfo()                { return FrameMap::_noRInfo; }
RInfo ValueGen::return1RInfo()           { return FrameMap::_O0_RInfo; }
RInfo ValueGen::callee_return1RInfo()    { return FrameMap::_I0_RInfo; }
RInfo ValueGen::return2RInfo()           { return FrameMap::_O0_O1_RInfo; }
RInfo ValueGen::callee_return2RInfo()    { return FrameMap::_I0_I1_RInfo; }
RInfo ValueGen::returnF0RInfo()          { return FrameMap::_f0_RInfo; }
RInfo ValueGen::returnD0RInfo()          { return FrameMap::_f0_double_RInfo; }
RInfo ValueGen::receiverRInfo()          { return FrameMap::_O0_RInfo; }
RInfo ValueGen::callee_receiverRInfo()   { return FrameMap::_I0_RInfo; }
RInfo ValueGen::nonReturnRInfo()         { return FrameMap::_I0_RInfo; }

//-----------------------spilling--------------------------------------------------------

// spill all values in caller_save registers
void ValueGen::spill_caller_save(bool is_C_call) {
  NEEDS_CLEANUP
  // a more efficient way would be to go through the list of
  // locked registers and ask if they are caller save.
  const RInfoCollection* regs = FrameMap::caller_save_registers(is_C_call);
  for (int i = 0; i < regs->number_of_registers(); i++) {
    RInfo r = regs->register_at(i);
    if (!ra()->is_free_reg(r)) {
      spill_register(r);
    }
  }
}


//-----------------------register allocation---------------------------------------------

RInfo ValueGen::result_register_for(ValueType* type, bool callee) {
  ValueTag tag = type->tag();
  switch (tag) {
    case intTag:
    case objectTag:
    case addressTag: return callee ? callee_return1RInfo() : return1RInfo();

    case longTag:    return callee ? callee_return2RInfo() : return2RInfo();

    case floatTag:   return returnF0RInfo();
    case doubleTag:  return returnD0RInfo();

    default: ShouldNotReachHere(); return noRInfo();
  }
}


//--------- loading items into registers --------------------------------


void ValueGen::check_float_register(Item* item) {
  // do nothing
}


// No FPU register fanout needed
bool ValueGen::fpu_fanout_handled(Item* item) {
  return false;
}


void ValueGen::load_item_with_reg_mask(Item* item, c1_RegMask mask) {
  assert(!item->type()->is_float() && !item->type()->is_double() && !item->type()->is_long(), "only cpu registers for now");
  set_maynot_spill(item);
  item->update();
  debug_only(check_item(item);)
  if (item->is_register() && mask.contains(item->get_register()) && !must_copy_register(item)) {
    return;
  }

  RInfo reg = get_free_rinfo(mask);
  ra()->lock_reg(item->value(), reg);
  if (gen_code()) {
    code_emit()->item_2_reg(reg, item);
  }
  item_free(item);
  item->set_register(reg);
  set_maynot_spill(item);
}


//---------------------------------------------------------------



// This check is done at item loads
void ValueGen::check_item_load(Item* item) {
  check_item(item);
}


//----------------------------------------------------------------------
//             visitor functions
//----------------------------------------------------------------------


// Phi that represent the TOS is in register; all others are in spill area
// TOS(1) element is at spill index 0.
void ValueGen::do_Phi(Phi* x) {
  assert(x->is_root(), "phi must be root");
  assert(x->item() != NULL, "phi's item must be set by block prolog");
}


RInfo ValueGen::rlock_result_with_hint(Value x, const Item* hint) {
  RInfo reg = rlock(x, hint);
  bool is_cached = false;
  if (hint->is_cached() && hint->get_register().is_same(reg)) {
    is_cached = true;
  }
  set_result(x, reg, is_cached);
  return reg;
}


// Performance note: better code may be generated if we do not always
// load a constant into a register, but let it be done by the consumer of
// the 'Constant' node
void ValueGen::do_Constant(Constant* x) {
  // value of constant is known through the referenced instruction
  // we must load constant in register if 'x' is root, 
  // because inreg flag does not work on roots; the only exception is 
  // if the constant has not be loaded (patching); inreg is never true
  // with unloaded constants

  if (x->type()->as_InstanceConstant() != NULL || x->type()->as_ClassConstant() != NULL) {
    // not-loaded/initialized class must come in as a constant in
    // order to be able to do the patching; instance and class constants
    // should never be forced in a register
    _result->set_constant();
  } else if (hint()->is_inreg() || x->is_root()) {
    NEEDS_CLEANUP  // remove in_reg, allow constant always to be constant
    RInfo reg = rlock_result_with_hint(x, hint());
    if (gen_code()) {
      code_emit()->constant_2_reg(x->type(), reg);
    } 
  } else {
    _result->set_constant();
  }
}


void ValueGen::do_LoadLocal(LoadLocal* x) {
  RInfo r;
  if (cached_cpu_local(x->index(), r)) {
    _result->set_register(r, true);
  } else {
    _result->set_stack(x->index());
  }
}


void ValueGen::do_StoreLocal(StoreLocal* x) {
  HintItem theHint(x->type()); 
  // if this is store local, and we are caching it than we pass a hint
  // setup hint
  RInfo r;
  bool is_cached = cached_cpu_local(x->index(), r);
  if (is_cached) {
    theHint.set_register(r, true); // mark is cached
  } else {
    theHint.set_from_item(HintItem::no_hint());
  }

  set_no_result(x);
  Item  value(x->value());
  ValueGen v  (&value, &theHint, this);
  if (!value.is_cached()) {
    load_item_hint(&value, &theHint);
    rfree(&value);
  }
  if (gen_code()) {
    RInfo r;
    if (cached_cpu_local(x->index(), r)) {
      code_emit()->item_2_reg(r, &value);
    } else {
      code_emit()->item_2_local(x->index(), &value);
    }
  }
}


void ValueGen::do_LoadField(LoadField* x) {
  bool needs_patching = !x->is_initialized() || !x->is_loaded() || PatchALot;
  Item     object(x->obj());
  ValueGen o(&object, HintItem::no_hint(), this);
  if (needs_patching && x->is_static()) {
    load_item_patching(&object);
  } else {
    load_item(&object);
  }
  const RInfoCollection* oop_regs = needs_patching ? ra()->oops_in_registers() : NULL;
  rfree(&object);
  // result should be hint
  RInfo reg = rlock_result_with_hint(x, hint());

  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs); 
      code_emit()->explicit_null_check(&object, &info);
    }

    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
    code_emit()->field_load(reg, x->field_type(), &object, x->offset(), needs_patching, x->is_loaded(), &info);
  }
}


void ValueGen::do_StoreField(StoreField* x) {
  bool needs_patching = !x->is_initialized() || !x->is_loaded() || PatchALot;
  Item  object(x->obj());
  Item  value(x->value());
  ValueTag valueTag = x->type()->tag();
  if (valueTag == objectTag) {
    // Note: we could use a G register for preserving the object register instead
    //       of destroying it; this would free up a potentially caller saved register for
    //       caching
    object.set_destroys_register();
  }
  ValueGen o(&object, HintItem::no_hint(), this);
  ValueGen v(&value,  HintItem::no_hint(), this);
  if (needs_patching && x->is_static()) {
    load_item_patching(&object);
  } else {
    load_item(&object);
  }
  load_item(&value);
  const RInfoCollection* oop_regs = needs_patching ? ra()->oops_in_registers() : NULL;
  item_free(&value);
  item_free(&object);

  set_no_result(x);
  {
    if (gen_code()) {
      if (!ImplicitNullChecks) {
        CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
        code_emit()->explicit_null_check(&object, &info);
      }
      if (PrintNotLoaded && needs_patching) {
        tty->print_cr("   ###class not loaded at store_%s bci %d", x->is_static() ?  "static" : "field", x->bci());
      }
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
      code_emit()->field_store(x->field_type(), &object, x->offset(), &value, needs_patching, x->is_loaded(), &info, FrameMap::_g1_RInfo);
    }
  }
}


void ValueGen::do_ArrayLength(ArrayLength* x) {
  Item array(x->array());
  ValueGen a(&array, HintItem::no_hint(), this);
  load_item(&array);
  // Why are oop_regs needed
  const RInfoCollection* oop_regs = ra()->oops_in_registers();
  rfree(&array);
  RInfo reg = rlock_result_with_hint(x, hint());

  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
      code_emit()->explicit_null_check(&array, &info);
    }
    const int array_length_offset = arrayOopDesc::length_offset_in_bytes() / wordSize;
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
    code_emit()->field_load(reg, T_INT, &array, array_length_offset, false,
			    true, // placeholder - not used
			    &info);
  }
}


void ValueGen::do_LoadIndexed(LoadIndexed* x) {
  Item array(x->array());
  Item index(x->index());
  ValueGen a(&array, HintItem::no_hint(), this);
  ValueGen i(&index, HintItem::no_hint(), this);
  load_item(&array);
  if (is_simm13(&index)) {
    dont_load_item(&index);
  } else {
    load_item(&index); 
  }
  // Why do we need oop_regs ?
  const RInfoCollection* oop_regs = ra()->oops_in_registers();
  item_free(&index);
  // delayed register release for LONG types, otherwise array or index register may be overwritten
  // before both loads are completed
  if (!x->type()->is_long()) rfree(&array);
  RInfo reg = rlock_result_with_hint(x, hint());

  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
      code_emit()->explicit_null_check(&array, &info);
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
    code_emit()->indexed_load(reg, x->elt_type(), &array, &index, &info);
  }
  if (x->type()->is_long()) rfree(&array);
}


void ValueGen::do_StoreIndexed(StoreIndexed* x) {
  assert(x->is_root(),"");
  bool obj_store = x->elt_type() == T_ARRAY || x->elt_type() == T_OBJECT;
  Item  array(x->array());
  if (obj_store) {
    // Note: we could use a G register for preserving the object register instead
    //       of destroying it; this would free up a potentially caller saved register for
    //       caching
    array.set_destroys_register();
  }
  if (obj_store) {
    spill_caller_save(true);
  }
  Item  index(x->index());
  Item  value(x->value());
  ValueGen a(&array, HintItem::no_hint(), this);
  ValueGen i(&index, HintItem::no_hint(), this);
  ValueGen v(&value, HintItem::no_hint(), this);
  load_item(&array);
  if (is_simm13(&index)) {
    dont_load_item(&index);
  } else {
    load_item(&index); 
  }
  load_item(&value);
  const RInfoCollection* oop_regs = ra()->oops_in_registers();

  item_free(&index);
  rfree(&value);
  rfree(&array);
  set_no_result(x);

  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
      code_emit()->explicit_null_check(&array, &info);
    }
    if (value.type()->is_object() && GenerateArrayStoreCheck) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
      code_emit()->array_store_check(&array, &value, &index, FrameMap::_g1_RInfo, FrameMap::_g3_RInfo, &info);
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
    code_emit()->indexed_store(x->elt_type(), &array, &index, &value, FrameMap::_g1_RInfo, &info);
  }
}


void ValueGen::do_MonitorEnter(MonitorEnter* x) {
  spill_values_on_stack(x->stack());
  assert(x->is_root(),"");
  Item obj(x->obj());
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item(&obj);

  set_no_result(x);

  RInfo lock    = FrameMap::_g1_RInfo;
  RInfo scratch = FrameMap::_g3_RInfo;
  RInfo hdr     = FrameMap::_g4_RInfo;
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, ra()->oops_in_registers());
      code_emit()->explicit_null_check(&obj, &info);
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), /*x->stack()*/ NULL, ra()->oops_in_registers());
    code_emit()->monitor_enter(&obj, lock, hdr, scratch, x->monitor_no(), &info);
  }
  rfree(&obj); 
}


void ValueGen::do_MonitorExit(MonitorExit* x) {
  spill_values_on_stack(x->stack());
  assert(x->is_root(),"");
  Item obj(x->obj());
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item(&obj);

  set_no_result(x);
  RInfo lock = FrameMap::_g1_RInfo;
  RInfo hdr  = FrameMap::_g3_RInfo;
  if (gen_code()) {
    if (x->monitor_no() == -1)
      warning("Unreachable monitor exit, wait for x86 merge to avoid compiling this bytecode");
    else
      code_emit()->monitor_exit(&obj, lock, hdr, x->monitor_no());
  }

  rfree(&obj); 
}


// _ineg, _lneg, _fneg, _dneg 
void ValueGen::do_NegateOp(NegateOp* x) {
  Item value(x->x());
  ValueGen v(&value, hint(), this);
  load_item_hint(&value, hint());
  RInfo reg = rlock_result_with_hint(x, hint());
  rfree(&value);
  if (gen_code()) {
    code_emit()->negate(reg, &value);
  }
}



// for  _fadd, _fmul, _fsub, _fdiv, _frem
//      _dadd, _dmul, _dsub, _ddiv, _drem
void ValueGen::do_ArithmeticOp_FPU(ArithmeticOp* x) { 
  // don't create any items before possible spill_caller_save
  switch (x->op()) {
  case Bytecodes::_fadd:
  case Bytecodes::_fmul:
  case Bytecodes::_fsub:
  case Bytecodes::_fdiv:
  case Bytecodes::_dadd:
  case Bytecodes::_dmul:
  case Bytecodes::_dsub:
  case Bytecodes::_ddiv: {
    Item left(x->x());
    Item right(x->y());
    ValueGen l(&left,  HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);
    load_item(&left);
    load_item(&right);
    RInfo reg = rlock_result_with_hint(x, hint());
    rfree(&left);
    rfree(&right);
    if (gen_code()) {
      code_emit()->arithmetic_op_fpu(x->op(), _result, &left, &right);
    }
  }
  break;

  case Bytecodes::_frem: 
  case Bytecodes::_drem: {
    spill_caller_save(true); // spill caller save registers before making C call

    Item left (x->x());
    Item right(x->y());
    HintItem left_hint (x->x()->type());
    HintItem right_hint(x->y()->type());
    left_hint. set_from_item(HintItem::no_hint());
    right_hint.set_from_item(HintItem::no_hint());

    ValueGen l(&left,  &left_hint,  this);
    ValueGen r(&right, &right_hint, this);
    
    // There are no sparc machine instructions to move a value from a
    // floating point register to an int register or vice-versa.
    // At this point left & might may reside in fp registers. They have to 
    // be moved to the appropiate O registers for the C call.
    const RInfo left_reg    = (x->op() == Bytecodes::_frem) ? FrameMap::_O0_RInfo : FrameMap::_O0_O1_RInfo;
    const RInfo right_reg   = (x->op() == Bytecodes::_frem) ? FrameMap::_O1_RInfo : FrameMap::_O2_O3_RInfo;
    const RInfo result_reg  = (x->op() == Bytecodes::_frem) ? FrameMap::_f0_RInfo : FrameMap::_f0_double_RInfo;

    dont_load_item(&left);
    dont_load_item(&right);

    lock_spill_rinfo(NULL, left_reg,  1);
    lock_spill_rinfo(NULL, right_reg, 1);

    if (gen_code()) {
      code_emit()->move_item_to_cpu_register (left,   left_reg);
      code_emit()->move_item_to_cpu_register (right, right_reg);
    }

    item_free(&left);
    item_free(&right);

    { HideReg temp(this, FrameMap::callee_save_regs());
      // require the result to use the first float registers
      ra()->lock_reg(x, result_reg, x->use_count());
      set_result(x, result_reg);

      if (gen_code()) {
        //code_emit()->arithmetic_call_op_check(left_reg, right_reg, _result);
        code_emit()->arithmetic_call_op(x->op(), temp.reg());
      }
    }

    ra()->free_reg(left_reg);
    ra()->free_reg(right_reg);
  }
  break;

  default: ShouldNotReachHere();
  }
}


// for  _ladd, _lmul, _lsub, _ldiv, _lrem
void ValueGen::do_ArithmeticOp_Long(ArithmeticOp* x) {
  // don't create any items before possible spill_caller_save
  switch (x->op()) {
  case Bytecodes::_lrem: 
  case Bytecodes::_lmul: 
  case Bytecodes::_ldiv: {
    spill_caller_save(true);     // spill caller save registers before making C call

    Item left(x->x());
    Item right(x->y());
    ValueGen l(&left,  HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);

    const RInfo left_reg   = FrameMap::_O2_O3_RInfo;
    const RInfo right_reg  = FrameMap::_O0_O1_RInfo;
    const RInfo result_reg = FrameMap::_O0_O1_RInfo;
    load_item_force(&left,  left_reg); 
    load_item_force(&right, right_reg); 
    const RInfoCollection* oop_regs = ra()->oops_in_registers();
    rfree(&left);
    rfree(&right);

    if (gen_code()) {
      if (x->op() == Bytecodes::_ldiv || x->op() == Bytecodes::_lrem) {
	CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
        code_emit()->explicit_div_by_zero_check(&right, &info);
      }
    }

    { HideReg temp(this, FrameMap::callee_save_regs());
      ra()->lock_reg(x, result_reg, x->use_count());
      set_result(x,result_reg);

      if (gen_code()) {
        code_emit()->arithmetic_call_op_check(left.get_register(), right.get_register(), _result);
        code_emit()->arithmetic_call_op(x->op(), temp.reg());
      }
    }

    break;
  }
  case Bytecodes::_ladd:
  case Bytecodes::_lsub: {
    Item left(x->x());
    Item right(x->y());
    ValueGen l(&left,  HintItem::no_hint(), this);
    ValueGen r(&right, HintItem::no_hint(), this);
    load_item(&left);
    load_item(&right);
    RInfo reg = rlock_result_with_hint(x, hint());
    rfree(&left);
    rfree(&right);
    if (gen_code()) {
      code_emit()->arithmetic_op_long(x->op(), _result, &left, &right);
    }
    break;
  }
  default: ShouldNotReachHere();
  }
}


bool ValueGen::is_simm13(Item* item) {
  if (item->is_constant() && item->type()->as_IntConstant() != NULL) {
    return Assembler::is_simm13(item->get_jint_constant());
  } else {
    return false;
  }
}


// for: _iadd, _imul, _isub, _idiv, _irem
void ValueGen::do_ArithmeticOp_Int(ArithmeticOp* x) {
  bool is_div_rem = x->op() == Bytecodes::_idiv || x->op() == Bytecodes::_irem;
  Item left(x->x());
  Item right(x->y());
  // missing test if instr is commutative and if we should swap
  ValueGen l(&left, HintItem::no_hint(), this);
  ValueGen r(&right, HintItem::no_hint(), this);
  if (is_div_rem || !is_simm13(&right)) {
    // simm13 constant are allowed for some operations
    load_item(&right);
  } else {
    dont_load_item(&right);
  }
  assert(right.is_constant() || right.is_register(), "wrong state of right");
  load_item(&left);
  const RInfoCollection* oop_regs = ra()->oops_in_registers();
  // Note: division may destroy result before it is used; must add checks
  bool use_hint = true;
  if (is_div_rem && hint()->has_hint()) {
    RInfo hint_reg = hint()->get_register();
    if ( left.get_register().is_same(hint_reg) 
        || (right.is_register() && right.get_register().is_same(hint_reg)) )
          use_hint = false; // result register will be destroyed while computing division
  }
  RInfo reg = rlock_result_with_hint(x, use_hint? hint() : HintItem::no_hint());
  if (is_div_rem) {
    if (gen_code()) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, oop_regs);
      code_emit()->arithmetic_idiv(x->op(), _result, &left, &right, FrameMap::_g1_RInfo, &info);
    }
  } else {
    if (gen_code()) {
      RInfo unused;
      code_emit()->arithmetic_op_int(x->op(), _result, &left, &right, unused);
    }
  }
  rfree(&left);
  item_free(&right);

}


void ValueGen::do_ArithmeticOp(ArithmeticOp* x) {
  ValueTag tag = x->type()->tag();
  assert(x->x()->type()->tag() == tag && x->y()->type()->tag() == tag, "wrong parameters");
  switch (tag) {
    case floatTag:
    case doubleTag:  do_ArithmeticOp_FPU(x);  return;
    case longTag:    do_ArithmeticOp_Long(x); return;
    case intTag:     do_ArithmeticOp_Int(x);  return;
  }
  ShouldNotReachHere();
}


// _ishl, _lshl, _ishr, _lshr, _iushr, _lushr
void ValueGen::do_ShiftOp(ShiftOp* x) {
  Item value(x->x());
  Item count(x->y());
  if (value.type()->is_long()) {
    count.set_destroys_register();
  }
  ValueGen v(&value, HintItem::no_hint(), this);
  ValueGen c(&count, HintItem::no_hint(), this);
  load_item(&value);
  load_item(&count);
  RInfo reg = rlock_result_with_hint(x, hint());
  rfree(&count);
  rfree(&value);
  if (gen_code()) {
    RInfo unused;
    code_emit()->shift_op(x->op(), reg, &value, &count, unused);
  }
}


// _iand, _land, _ior, _lor, _ixor, _lxor
void ValueGen::do_LogicOp(LogicOp* x) {
  Item left(x->x());
  Item right(x->y());
  // missing test if instr is commutative and if we should swap
  ValueGen l(&left, HintItem::no_hint(), this);
  ValueGen r(&right, HintItem::no_hint(), this);
  load_item(&left);
  load_item(&right);
  RInfo reg = rlock_result_with_hint(x, hint());
  rfree(&left);
  rfree(&right);
  if (gen_code()) {
    code_emit()->logic_op(x->op(), reg, &left, &right);
  }
}



// _lcmp, _fcmpl, _fcmpg, _dcmpl, _dcmpg
void ValueGen::do_CompareOp(CompareOp* x) {
  Item left(x->x());
  Item right(x->y());
  ValueGen l(&left,  HintItem::no_hint(), this);
  ValueGen r(&right, HintItem::no_hint(), this);
  load_item(&left);
  load_item(&right);
  RInfo reg = rlock_result_with_hint(x, hint());
  rfree(&left);
  rfree(&right);
  if (gen_code()) {
    code_emit()->compare_op(x->op(), reg, &left, &right);
  }
}


// Code for  :  x->x() {x->cond()} x->y() ? x->tval() : x->fval()
void ValueGen::do_IfOp(IfOp* x) {
  Unimplemented();
}


void ValueGen::do_Intrinsic(Intrinsic* x) {
  switch (x->id()) {
    case methodOopDesc::_dsqrt: {
      assert(x->number_of_arguments() == 1, "wrong type");
      Item value(x->argument_at(0));
      ValueGen v(&value, HintItem::no_hint(), this);
      load_item(&value);
      rfree(&value);
      RInfo result_reg = rlock_result_with_hint(x, hint());
      if (gen_code()) {
	code_emit()->math_intrinsic(x->id(), result_reg, &value);
      }
      break;
    }  
    case methodOopDesc::_dsin: // fall through
    case methodOopDesc::_dcos: {
      assert(x->number_of_arguments() == 1, "wrong type");
      // Note: spill caller save before setting the item
      spill_caller_save(true);  // spill caller save registers before making a C call
      Item value(x->argument_at(0));
      value.set_destroys_register();
      HintItem value_hint(x->argument_at(0)->type());
      value_hint.set_from_item(HintItem::no_hint());
      value_hint.set_inreg(true);
      ValueGen v(&value, &value_hint, this);


      // Make sure value resides on the stack (so it can be forced into an "O" reg for the C call).
      if (value.is_register())  {
	spill_item(value.value()->item());
        value.update();
      }

      assert(value.type()->tag() == doubleTag, "wrong type");

      RInfo value_reg = FrameMap::_O0_O1_RInfo;
      load_item_force(&value, value_reg); 

      rfree(&value);

      RInfo result_reg = FrameMap::_f0_double_RInfo;
      ra()->lock_reg(x, result_reg, x->use_count());
      set_result(x, result_reg);

      { // thread register must be callee saved
	HideReg thread(this, FrameMap::callee_save_regs());
	if (gen_code()) {
	  code_emit()->math_intrinsic(x->id(), result_reg, &value, thread.reg());
	}
      }
      break;
    }
    case methodOopDesc::_arraycopy:
      { assert(x->number_of_arguments() == 5, "wrong type");
        // Note: spill caller save before setting the item
        spill_caller_save(true);  // spill caller save registers before making a C call
        Item src     (x->argument_at(0)); 
        Item src_pos (x->argument_at(1)); 
        Item dst     (x->argument_at(2)); 
        Item dst_pos (x->argument_at(3)); 
        Item length  (x->argument_at(4)); 
        ValueGen src_v(&src, HintItem::no_hint(), this);
        ValueGen src_pos_v(&src_pos, HintItem::no_hint(), this);
        ValueGen dst_v(&dst, HintItem::no_hint(), this);
        ValueGen dst_pos_v(&dst_pos, HintItem::no_hint(), this);
        ValueGen length_v(&length, HintItem::no_hint(), this);
        // load all values in callee_save_registers, as this makes the 
        // parameter passing to the fast case simpler
        load_item_with_reg_mask(&src,     FrameMap::callee_save_regs());
        load_item_with_reg_mask(&src_pos, FrameMap::callee_save_regs()); 
        load_item_with_reg_mask(&dst,     FrameMap::callee_save_regs());
        load_item_with_reg_mask(&dst_pos, FrameMap::callee_save_regs());
        load_item_with_reg_mask(&length,  FrameMap::callee_save_regs());
        { HideReg thread(this, FrameMap::callee_save_regs());
          if (gen_code()) {
            CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack()); // we may want to have stack (deoptimization?)
            code_emit()->arraycopy(&src, &src_pos, &dst, &dst_pos, &length, FrameMap::_g1_RInfo, FrameMap::_g3_RInfo, thread.reg(), &info); // does add_safepoint
          }
        }
        // makes sure that the thread register is not one of the parameter register,
        // as it gets destroyed
        item_free(&src);
        item_free(&src_pos);
        item_free(&dst);
        item_free(&dst_pos);
        item_free(&length);
        set_no_result(x);
      }
      break;
    default:
      Unimplemented();
      break;
  }
}


// _i2l, _i2f, _i2d, _l2i, _l2f, _l2d, _f2i, _f2l, _f2d, _d2i, _d2l, _d2f
// _i2b, _i2c, _i2s
void ValueGen::do_Convert(Convert* x) {
  // don't create any items before possible spill_caller_save
  switch (x->op()) {
    case Bytecodes::_l2f: 
    case Bytecodes::_l2d: 
    case Bytecodes::_f2l: 
    // case Bytecodes::_f2i: 
    case Bytecodes::_d2l: 
    case Bytecodes::_d2i: { 
      spill_caller_save(true);  // spill caller save registers before making C call
      Item value(x->value());
      HintItem value_hint(x->value()->type());
      value_hint.set_from_item(HintItem::no_hint());
      if (value.type()->is_float_kind()) {
        value_hint.set_inreg(true);
      }
      ValueGen v(&value, &value_hint, this);

      // Make sure value resides on the stack (so it can be forced into an "O" reg for the C call).
      if (value.is_register()) {
	spill_item(value.value()->item());
        value.update();
      }
    
      RInfo value_reg;
      switch (value.type()->tag()) {
        case floatTag:  value_reg = FrameMap::_O0_RInfo;    break;
        case doubleTag: value_reg = FrameMap::_O0_O1_RInfo; break;
        case longTag:   value_reg = FrameMap::_O0_O1_RInfo; break;
        default: ShouldNotReachHere();
      }

      // force value into the "O" int registers for the C calling convention
      load_item_force(&value,  value_reg);
      item_free(&value);

      { // thread register must be callee saved
        HideReg thread(this, FrameMap::callee_save_regs());

	// require the result to use the first float registers
	RInfo result_reg;
	switch (x->type()->tag()) {
          case floatTag:   result_reg = FrameMap::_f0_RInfo;        break;
          case doubleTag:  result_reg = FrameMap::_f0_double_RInfo; break;
          case longTag:    result_reg = FrameMap::_O0_O1_RInfo;     break;
          case intTag:     result_reg = FrameMap::_O0_RInfo;        break;
          default: ShouldNotReachHere();
	}

        ra()->lock_reg(x, result_reg, x->use_count());
        set_result(x, result_reg);
        if (gen_code()) {
          code_emit()->call_convert_op_check(&value, _result);
          code_emit()->call_convert_op(x->op(), thread.reg());
        }
      }
    }
    break;

    case Bytecodes::_i2f:
    case Bytecodes::_i2d: {
      Item value(x->value());
      HintItem value_hint(x->value()->type());
      value_hint.set_from_item(HintItem::no_hint());
      value_hint.set_inreg(true);
      ValueGen v(&value,  &value_hint, this);

      // make value reside on the stack so it can get loaded into a fp register
      if (value.is_register() && !value.is_cached()) {
        spill_item(value.value()->item()); 
        value.update();
      }

      RInfo reg = rlock_result_with_hint(x, hint());
      if (value.is_spilled()) sfree(&value);

      if (gen_code()) {
        code_emit()->convert_op(x->op(), reg, &value);
      }  
    }
    break;

    case Bytecodes::_i2l:
    case Bytecodes::_i2b:
    case Bytecodes::_i2c:  
    case Bytecodes::_i2s: 
    case Bytecodes::_l2i: 
    case Bytecodes::_f2d:
    case Bytecodes::_d2f: { // inline code    
      Item value(x->value());
      ValueGen v(&value,  HintItem::no_hint(), this);

      load_item(&value);    

      RInfo reg = rlock_result_with_hint(x, hint());
      rfree(&value);

      if (gen_code()) {
        code_emit()->convert_op(x->op(), reg, &value);
      }  
    }
    break;

    
    case Bytecodes::_f2i: {
      Item value (x->value()); value.set_destroys_register();
      ValueGen v(&value, HintItem::no_hint(), this);
      load_item(&value);
      RInfo reg = rlock_result_with_hint(x, hint());
      rfree(&value);

      if (gen_code()) {
        code_emit()->convert_op(x->op(), reg, &value);
      }  
    }
    break;
    


    default: ShouldNotReachHere();
  }
}


void ValueGen::do_JsrContinuation(JsrContinuation* x) {
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL, ra()->oops_in_registers());
    code_emit()->add_nop_with_info(&info);
  }
}


void ValueGen::do_NullCheck(NullCheck* x) {
  Item value(x->obj());
  ValueGen v(&value, HintItem::no_hint(), this);
  load_item(&value);
  item_free(&value);
  if (gen_code() && item_may_be_null(&value)) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), NULL);
    code_emit()->implicit_null_check(&value, &info);
  }
}


void ValueGen::do_LoopEnter(LoopEnter* x) {
  bool static warn = false;
  if (!warn) {
    warning("ValueGen::do_LoopEnter(...) not implemented yet");
    warn = true;
  }
}


void ValueGen::do_LoopExit(LoopExit* x) {
  bool static warn = false;
  if (!warn) {
    warning("ValueGen::do_LoopExit(...) not implemented yet");
    warn = true;
  }
}

// Visits all arguments, returns appropriate items without loading them
GrowableArray<Item*>* ValueGen::invoke_visit_arguments(Invoke* x, CallingConvention* arg_loc) {
  if (!PassParametersInRegisters) {
    NEEDS_CLEANUP;  // get rid of this once we turn on PassParametersInRegisters

    GrowableArray<Item*>* argument_items = new GrowableArray<Item*>();
    int idx = x->has_receiver() ? 1 : 0;
    for (int i = 0; i < x->number_of_arguments(); i++) {
      Item* param = new Item(x->argument_at(i));
      argument_items->append(param);
      ArgumentLocation loc = arg_loc->arg_at(idx);
      idx += (param->type()->is_double_word() ? 2 : 1);
      Item hint = *HintItem::no_hint();
      if (loc.is_register_arg()) {
	hint.set_register(loc.outgoing_reg_location());
      }
      ValueGen p(param, &hint, this);
    }
    return argument_items;
  } else {
    GrowableArray<Item*>* argument_items = new GrowableArray<Item*>();
    if (x->has_receiver()) {
      Item* receiver = new Item(x->receiver());
      argument_items->append(receiver);
      ArgumentLocation loc = arg_loc->arg_at(0);
      assert(loc.is_register_arg() && loc.outgoing_reg_location().is_same(FrameMap::_O0_RInfo), "receiver must be in O0");
      HintItem hint(objectType, FrameMap::_O0_RInfo);
      ValueGen p(receiver, &hint, this);
    }
    int idx = x->has_receiver() ? 1 : 0;
    for (int i = 0; i < x->number_of_arguments(); i++) {
      Item* param = new Item(x->argument_at(i));
      argument_items->append(param);
      ArgumentLocation loc = arg_loc->arg_at(idx);
      idx += (param->type()->is_double_word() ? 2 : 1);
      HintItem hint(x->argument_at(i)->type());
      if (loc.is_register_arg()) {
	hint.set_register(loc.outgoing_reg_location());
      } else {
	hint.set_from_item(HintItem::no_hint());
      }
      ValueGen p(param, &hint, this);
    }
    return argument_items;
  }
}


void ValueGen::invoke_load_arguments(Invoke* x, GrowableArray<Item*>* args, CallingConvention* arg_loc) {
  if (!PassParametersInRegisters) {
    NEEDS_CLEANUP;  // get rid of this once we turn on PassParametersInRegisters

    int idx = x->has_receiver() ? 1 : 0;
    int i;
    for (i = 0; i < x->number_of_arguments(); i++) {
      Item* param = args->at(i);
      ArgumentLocation loc = arg_loc->arg_at(idx);
      if (loc.is_register_arg()) {
	load_item_force(param, loc.outgoing_reg_location());
	{
	  // the following call to code emitter should be removed once we
	  // have compiler1 C2I adapters in place, besides passing the
	  // argument in a register, we put it on stack in case the callee
	  // is interpreted
	  FrameMap f(0, NULL);
	  f.set_size_arguments(x->signature()->length());
	  f.set_size_locals   (x->signature()->length());
 	  int offset = f.local_offset_from_fp(idx, false, false);
	  if (gen_code()) {
	    code_emit()->store_stack_parameter(param, offset);
	  }
	}
      } else {
	assert(loc.is_stack_arg(), "just checking");
	load_item(param);
	if (gen_code()) {
	  code_emit()->store_stack_parameter(param, loc.stack_offset_in_words());
	}
	item_free(param);
      }
      idx += (param->type()->is_double_word() ? 2 : 1);
    }
  } else {
    GrowableArray<Item*>* register_args = new GrowableArray<Item*>(FrameMap::nof_reg_args);
    int idx = x->has_receiver() ? 1 : 0;
    for (int i = 0; i < x->number_of_arguments(); i++) {
      Item* param = args->at(x->has_receiver() ? i+1 : i);
      ArgumentLocation loc = arg_loc->arg_at(idx);
      if (loc.is_register_arg()) {
	assert(PassParametersInRegisters, "must be set");
	load_item_force(param, loc.outgoing_reg_location());
        // don't free items for register arguments until all other arguments are loaded;
        // otherwise an argument register could be overwritten while initializing a stack argument
        register_args->append(param);
      } else {
	assert(loc.is_stack_arg(), "just checking");
	load_item(param);
	if (gen_code()) {
	  code_emit()->store_stack_parameter (param, loc.stack_offset_in_words());
	}
        item_free(param);
      }
      idx += (param->type()->is_double_word() ? 2 : 1);
    }
    if (x->has_receiver()) {
      Item* receiver = args->at(0);
      ArgumentLocation loc = arg_loc->arg_at(0);
      assert(loc.is_register_arg(), "the receiver must be in O0");
      assert(PassParametersInRegisters, "must be set");
      load_item_force(receiver, loc.outgoing_reg_location());
      item_free(receiver);
    }
    for (int i = 0; i < register_args->length(); i++) {
      item_free(register_args->at(i));
    }
  }
}


// handle arguments, without receiver
void ValueGen::invoke_do_arguments(Invoke* x) {
  ShouldNotReachHere();
}



void ValueGen::invoke_do_spill(Invoke* x, RInfo receiver_reg) {
  // move caller save registers to callee save registers (saves the spill)
  // make sure that caller save registers are spilled
  spill_values_on_stack(x->stack(), receiver_reg, true);
}


void ValueGen::invoke_do_result(Invoke* x, bool needs_null_check, Item* receiver) {
  // assert(ra()->are_all_registers_free(), "all registers must be free across calls");
  // setup result register 
  RInfoCollection* oop_regs = ra()->oops_in_registers();
  if (x->type()->is_void()) {
    set_no_result(x);
  } else {
    RInfo reg = rlock_call_result(x);
    set_result(x, reg);
  }
  // emit invoke code
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    bool optimized = x->target_is_loaded() && x->target_is_final();
    code_emit()->call_op(x->code(), x->signature(), x->signature()->length(), &info, optimized, needs_null_check, receiver); // does add_safepoint
  }
}


// The invoke with receiver has following phases:
//   a) traverse and load/lock receiver; 
//   b) traverse all arguments -> item-array (invoke_visit_argument)
//   c) push receiver on stack
//   d) load each of the items and push on stack 
//   e) unlock receiver
//   f) move receiver into receiver-register %o0
//   g) lock result registers and emit call operation
//
// Before issuing a call, we must spill-save all values on stack
// that are in caller-save register. "spill-save" moves thos registers
// either in a free callee-save register or spills them if no free
// callee save register is available.
//
// The problem is where to invoke spill-save.
// - if invoked between e) and f), we may lock callee save 
//   register in "spill-save" that destroys the receiver register
//   before f) is executed
// - if we rearange the f) to be earlier, by loading %o0, it
//   may destroy a value on the stack that is currently in %o0
//   and is waiting to be spilled
// - if we keep the receiver locked while doing spill-save,
//   we cannot spill it as it is spill-locked
// 
void ValueGen::do_Invoke(Invoke* x) {
  CallingConvention* arg_loc = FrameMap::calling_convention(!x->has_receiver(), *x->signature());
  if (!PassParametersInRegisters) {
    NEEDS_CLEANUP;  // get rid of this once we turn on PassParametersInRegisters

    if (x->has_receiver()) {
      HintItem hint(x->receiver()->type());
      hint.set_register(receiverRInfo());
      Item receiver(x->receiver());
      ValueGen p(&receiver, HintItem::no_hint(), this);
      GrowableArray<Item*>* args = invoke_visit_arguments(x, arg_loc);
      load_item_hint(&receiver, &hint);
      if (gen_code()) {
	int receiver_offset_from_sp_in_words = x->size_of_arguments() - 1 + frame::memory_parameter_word_sp_offset;
	code_emit()->store_stack_parameter(&receiver, receiver_offset_from_sp_in_words);
      }
      invoke_load_arguments(x, args, arg_loc);
      RInfo rec_reg = receiver.get_register();
      rfree(&receiver);
      // receiver is alive but unlocked here; make sure that it is not destroyed
      invoke_do_spill(x, rec_reg);
      bool needs_null_check = false;
      if (item_may_be_null(&receiver)) {
	if (x->code() == Bytecodes::_invokespecial || !x->target_is_loaded() || x->target_is_final()) {
	  // emit receiver NULL check if the class of target is not loaded (i.e., we cannot know if it is final)
	  // or if the target us final
	  if (PrintNotLoaded && !x->target_is_loaded()) {
	    tty->print_cr("   ### class not loaded at invoke bci %d", x->bci());
	  }
	  needs_null_check = true;
	}
      }
      invoke_do_result(x, needs_null_check, &receiver);

    } else {
      GrowableArray<Item*>* args = invoke_visit_arguments(x, arg_loc);
      invoke_load_arguments(x, args, arg_loc);
      invoke_do_spill(x);
      invoke_do_result(x, false);
    }
  } else {
    GrowableArray<Item*>* args = invoke_visit_arguments(x, arg_loc);
    if (x->has_receiver()) {
      Item* receiver = args->at(0);
      invoke_load_arguments(x, args, arg_loc);
      invoke_do_spill(x);

      bool needs_null_check = false;
      if (item_may_be_null(receiver)) {
	if (x->code() == Bytecodes::_invokespecial || !x->target_is_loaded() || x->target_is_final()) {
	  // emit receiver NULL check if the class of target is not loaded (i.e., we cannot know if it is final)
	  // or if the target is final
	  if (PrintNotLoaded && !x->target_is_loaded()) {
	    tty->print_cr("   ### class not loaded at invoke bci %d", x->bci());
	  }
	  needs_null_check = true;
	}
      }
      invoke_do_result(x, needs_null_check, receiver);
    } else {
      invoke_load_arguments(x, args, arg_loc);
      invoke_do_spill(x);
      invoke_do_result(x, false);
    }
  }
}


void ValueGen::do_NewInstance(NewInstance* x) {
  NEEDS_CLEANUP;
  // Note: change this to require any register as result of a new,
  //       instead of specifically %o0; do not spill all values.

  // make sure registers are spilled
  spill_values_on_stack(x->stack());
  const RInfoCollection* oop_regs = ra()->oops_in_registers();
  RInfo reg = FrameMap::_O0_RInfo;
  ra()->lock_reg(x, reg, x->use_count());
  set_result(x, reg);
  if (gen_code()) {
    if (PrintNotLoaded && x->klass() == NULL) {
      tty->print_cr("   ###class not loaded at new bci %d", x->bci());
    }
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    code_emit()->new_instance(reg, x->klass(), FrameMap::_g1_RInfo, FrameMap::_g3_RInfo, FrameMap::_g4_RInfo, FrameMap::_g5_RInfo, &info);
  }
}


void ValueGen::do_NewTypeArray(NewTypeArray* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());

  Item length(x->length());
  ValueGen l(&length, HintItem::no_hint(), this);
  load_item(&length);

  const RInfoCollection* oop_regs = ra()->oops_in_registers();

  HintItem result_hint(x->type(), FrameMap::_O0_RInfo);
  RInfo reg = rlock(x, &result_hint);
  set_result(x, reg);

  SpillLockReg sl(this, reg);
  {
    RInfo t1 = FrameMap::_g1_RInfo;
    RInfo t2 = FrameMap::_g3_RInfo;
    RInfo t3 = FrameMap::_g4_RInfo;
    RInfo t4 = FrameMap::_g5_RInfo;
    HideReg klass_reg(this, objectType);
    rfree(&length);
    if (gen_code()) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
      code_emit()->new_type_array(reg, x->elt_type(), &length, t1, t2, t3, t4, klass_reg.reg(), &info);
    }
  }
}


void ValueGen::do_NewObjectArray(NewObjectArray* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());

  Item length(x->length());
  ValueGen l(&length, HintItem::no_hint(), this);
  load_item(&length);

  const RInfoCollection* oop_regs = ra()->oops_in_registers();

  HintItem result_hint(x->type(), FrameMap::_O0_RInfo);
  RInfo reg = rlock(x, &result_hint);
  set_result(x, reg);

  SpillLockReg sl(this, reg);
  {
    RInfo t1 = FrameMap::_g1_RInfo;
    RInfo t2 = FrameMap::_g3_RInfo;
    RInfo t3 = FrameMap::_g4_RInfo;
    RInfo t4 = FrameMap::_g5_RInfo;
    HideReg klass_reg(this, objectType);
    rfree(&length);
    if (gen_code()) {
      CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
      code_emit()->new_object_array(reg, x->klass(), &length, t1, t2, t3, t4, klass_reg.reg(), &info);
    }
  }
}


void ValueGen::do_NewMultiArray(NewMultiArray* x) {
  // make sure registers are spilled
  spill_values_on_stack(x->stack());

  Values* dims = x->dims();
  for(int i = 0; i < dims->length(); i++) {
    Item size(dims->at(i));
    ValueGen s(&size, HintItem::no_hint(), this);
    load_item(&size);
    rfree(&size);
    if (gen_code()) {
      code_emit()->store_stack_parameter (&size, i + frame::memory_parameter_word_sp_offset);
    }
  }

  const RInfoCollection* oop_regs = ra()->oops_in_registers();

  HintItem result_hint(x->type(), FrameMap::_O0_RInfo);
  RInfo reg = rlock(x, &result_hint);
  set_result(x, reg);

  SpillLockReg sl(this, reg);
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    RInfo unused;
    code_emit()->new_multi_array(reg, x->klass(), x->rank(), unused, &info);
  }
}


void ValueGen::do_BlockBegin(BlockBegin* x) {
}


void ValueGen::do_Throw(Throw* x) {
  if (!x->stack()->stack_is_empty()) {
    // release all roots (computed values)
    release_roots(x->stack());
  }
  Item exception(x->exception());
  {
    ValueGen e(&exception, HintItem::no_hint(), this);
    load_item(&exception);
    rfree(&exception);
    set_no_result(x);
  }
  if (gen_code()) {
    if (!ImplicitNullChecks) {
      CodeEmitInfo info(x->bci(), NULL, NULL, NULL); 
      code_emit()->explicit_null_check(&exception, &info);
    }
    CodeEmitInfo info(x->bci(), NULL, NULL, NULL);
    code_emit()->throw_op(&exception, &info);
  }
  assert(ra()->are_all_free(),"Unimplemented: should just release all registers and spilled locations");
}


void ValueGen::do_CheckCast(CheckCast* x) {
  spill_values_on_stack(x->stack());
  Item obj(x->obj());
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item(&obj);
  RInfoCollection* oop_regs = ra()->oops_in_registers();
  RInfo tmp1;
  RInfo tmp2;
  rfree(&obj);
  const RInfo reg = rlock_result_with_hint(x, hint());
  if (!oop_regs->contains(reg)) {
    // Note: we try to have registers of 'reg' and 'obj' to
    //       be the same; in case that they are not, we must
    //       specify the result register as an oop-register;
    //       this is necessary in the presence of patching
    oop_regs->append(reg);
  }
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    code_emit()->checkcast_op(reg, &obj, x->klass(), FrameMap::_g1_RInfo, FrameMap::_g3_RInfo, &info);
  }
}


void ValueGen::do_InstanceOf(InstanceOf* x) {
  spill_values_on_stack(x->stack());
  Item obj(x->obj());
  ValueGen o(&obj, HintItem::no_hint(), this);
  load_item_with_reg_mask(&obj, FrameMap::callee_save_regs());
  const RInfoCollection* oop_regs = ra()->oops_in_registers();
  RInfo reg = rlock_result_with_hint(x, hint());
  rfree(&obj);
  if (gen_code()) {
    CodeEmitInfo info(x->bci(), ra()->oops_in_spill(), x->stack(), oop_regs);
    code_emit()->instanceof_op(reg, &obj, x->klass(), FrameMap::_g1_RInfo, FrameMap::_g3_RInfo, &info);
  }
}



RInfoCollection* ValueGen::get_block_item_oop_regs() {
  // currently we have no use for this method; it may change in future
  ShouldNotReachHere();
  BlockItem* bi = block()->block_item();
  RInfoCollection* res = NULL;
  if (bi != NULL) {
    int lng = bi->map_length();
    for (int i = 0; i < lng; i++) {
      if (bi->is_oop(i)) {
        if (res == NULL) res = new RInfoCollection();
        res->append(bi->map_local(i));
      }
    }
  }
  if (res == NULL) {
    return NULL;
  } else {
    return res;
  }
}


// do a jump to default successot, except if it is the immediate successor in _code (sequence of blocks)
void ValueGen::goto_default_successor(BlockEnd* block_end) {
  bool is_bb = is_backward_branch(block_end, block_end->default_sux());
  if (block()->weight() + 1 == block_end->default_sux()->weight()) {
    // default successor is next block; no goto necessary
    assert(!is_bb, "cannot be backward branch");
  } else {
    code_emit()->goto_op(block_end->default_sux(), block_end->bci(), is_bb, NULL);
  }
}


void ValueGen::do_Goto(Goto* x) {
  bool is_bb = is_backward_branch(x, x->default_sux());
  set_no_result(x);
  move_to_phi(x->stack());
  if (gen_code()) {
    if (is_bb && !x->stack()->stack_is_empty()) {
      // we cannot emit safepoint data if expression stack is empty,
      code_emit()->set_bailout("stack not empty at backward branch");
    }
    // check that this is endless goto (add nop)
    // Note: this is probably not needed as we have switched off EliminateJumpToJumps
    bool endless_loop = is_bb && x->default_sux()->bci() == x->bci();
    if (endless_loop) {
      code_emit()->add_nop();
    }
    goto_default_successor(x);
  }
}


void ValueGen::do_Jsr(Jsr* x) {
  set_no_result(x);
  if (!x->stack()->stack_is_empty()) {
    move_to_phi(x->stack());
  }
  if (gen_code()) {
    if (!x->stack()->stack_is_empty()) {
      code_emit()->set_bailout("stack not empty on jsr");
    }
    code_emit()->jsr_op(x->subroutine(), x->bci());
  }
}


void ValueGen::do_Ret(Ret* x) {
  set_no_result(x);
  if (gen_code()) {
    if (!x->stack()->stack_is_empty()) {
      code_emit()->set_bailout("stack not empty on ret");
    }
    RInfo r;
    cached_cpu_local(x->index(), r);
    code_emit()->ret_op(x->index(), r, x->bci());
  }
}


void ValueGen::do_If(If* x) {
  bool is_bb = false;
  assert(x->number_of_sux() == 2, "inconsistency");
  ValueTag tag = x->x()->type()->tag();
  is_bb = is_backward_branch(x, x->sux_at(0)) || is_backward_branch(x, x->sux_at(1)) ;
  Item xitem(x->x());
  Item yitem(x->y());
  Item* xin = &xitem;
  Item* yin = &yitem;
  If::Condition cond = x->cond();
  {  // call destructors before doing move to phi
    ValueGen xg(xin, HintItem::no_hint(), this);
    ValueGen yg(yin, HintItem::no_hint(), this);
    if (tag == longTag) {
      // for longs, only conditions "eql", "neq", "lss", "geq" are valid;
      // mirror for other conditions
      if (cond == If::gtr || cond == If::leq) {
        // swap inputs
        cond = Instruction::mirror(cond);
        xin = &yitem;
        yin = &xitem;
      }
      xin->set_destroys_register();
    }
    load_item(xin);
    if (is_simm13(yin)) {
      // inline int constants which are small enough to be immediate operands
      dont_load_item(yin);
    } else if (tag == objectTag && yin->is_constant() && (yin->get_jobject_constant()->is_null_object())) {
      dont_load_item(yin);
    } else {
      load_item(yin);
    }
    item_free(yin);
    rfree(xin);
    set_no_result(x);
  }

  // note that the condition test must happen before the
  // moves into Phi area happen, and that the control flow
  // jump must happen after the moves into the phi area
  //  assert(!gen_code() || !code_emit()->is_cc_locked(), "cc register must be unlocked");

  // Note that in phase 2, the registers holding the testing values may have been destroyed
  for (int phase = 1; phase <= 2; phase++) {
    if (phase == 2) {
      move_to_phi(x->stack());
    }
    if (gen_code()) {
      if (is_bb && !x->stack()->stack_is_empty()) {
        // we cannot emit safepoint data if expression stack is empty,
        code_emit()->set_bailout("stack not empty on backward branch");
      }
      // Note: do not use x->cond, as we may have swapped the arguments
      code_emit()->if_op(phase, cond, xin, yin, x->tsux(), x->fsux(), x->usux(),
                         x->bci(), is_bb, NULL);
      // Note: there are currently no optimizations on SPARC that would profit from
      //       locking of CC information
      if (phase == 1) {
	//        code_emit()->lock_cc();
      } else {
	//        code_emit()->unlock_cc();
      }
    }
  }
  if (gen_code()) {
    goto_default_successor(x);
  }
  //  assert(!gen_code() || !code_emit()->is_cc_locked(), "cc must be unlocked");
}


void ValueGen::do_IfInstanceOf(IfInstanceOf* x) {
  Unimplemented();
}


// May change the content of tag;
void ValueGen::setup_phis_for_switch(Item* tag, ValueStack* stack) {
  // if stack() exists, then move_to_phi may destroy the tag register as
  // the tag register is not locked anymore.
  if (stack != NULL && stack->stack_size() > 0 ) {
    // stack exists, move values into phi locations
    // preserve the tag
    if (gen_code()) {
      // save in G1
      code_emit()->item_2_reg(FrameMap::_g1_RInfo, tag);
    }
    move_to_phi(stack);
    { HideReg hr(this, nonReturnRInfo()); // use one register that is not a return register
      tag->set_register(hr.reg());
      if (gen_code()) {
        // restore from G1(do not use return registers)
        code_emit()->reg_2_item(tag, FrameMap::_g1_RInfo);
      }
    }
  }
}


void ValueGen::do_TableSwitch(TableSwitch* x) {
  Item tag(x->tag());
  { ValueGen v(&tag, HintItem::no_hint(), this);
    load_item(&tag);
    rfree(&tag);
    set_no_result(x);
  }

  setup_phis_for_switch(&tag, x->stack());
  if (gen_code()) {
    int lo_key = x->lo_key();
    int hi_key = x->hi_key();
    int len = x->length();
    for (int i = 0; i < len && !code_emit()->must_bailout(); i++) {
      int matchVal = i + lo_key;
      bool is_bb = is_backward_branch(x, x->sux_at(i));
      if (is_bb && !x->stack()->stack_is_empty()) {
        // we cannot emit safepoint data if expoession stack is empty,
        code_emit()->set_bailout("stack not empty on tableswitch");
      }
      code_emit()->tableswitch_op(&tag, matchVal, x->sux_at(i), x->bci(), is_bb, NULL);
      code_emit()->check_codespace();
    }
    bool is_bb = is_backward_branch(x, x->default_sux());
    if (is_bb && !x->stack()->stack_is_empty()) {
      // we cannot emit safepoint data if expoession stack is empty,
      code_emit()->set_bailout("stack not empty on tableswitch");
    }
    code_emit()->goto_op(x->default_sux(), x->bci(), is_bb, NULL);
  }
}


// we expect the keys to be sorted by increasing value
GrowableArray<LookupRange*>* create_lookup_ranges(LookupSwitch* x) {
  GrowableArray<LookupRange*>* res = new GrowableArray<LookupRange*>();
  int len = x->length();
  if (len > 0) {
    int key = x->key_at(0);
    BlockBegin* sux = x->sux_at(0);
    LookupRange* range = new LookupRange(key, sux);
    for (int i = 1; i < len; i++) {
      int new_key = x->key_at(i);
      BlockBegin* new_sux = x->sux_at(i);
      if (key+1 == new_key && sux == new_sux) {
        // still in same range
        range->set_high_key(new_key);
      } else {
        res->append(range);
        range = new LookupRange(new_key, new_sux);
      }
      key = new_key;
      sux = new_sux;
    }
    if (res->length() == 0 || res->last() != range)  res->append(range);
  }
  return res;
}


void ValueGen::do_LookupSwitch_key(Item* tag, LookupSwitch* x, LookupRange* range) {
  bool is_bb = is_backward_branch(x, range->sux());
  if (is_bb && !x->stack()->stack_is_empty()) {
    // we cannot emit safepoint data if expoession stack is empty,
    code_emit()->set_bailout("stack not empty on lookupswitch");
  }
  code_emit()->lookupswitch_range_op(tag, range->low_key(), range->high_key(), range->sux(), x->bci(), is_bb, NULL);        
}


void ValueGen::do_LookupSwitch_default(LookupSwitch* x) {
  bool is_bb = is_backward_branch(x, x->default_sux());
  if (is_bb && !x->stack()->stack_is_empty()) {
    // we cannot emit safepoint data if expoession stack is empty,
    code_emit()->set_bailout("stack not empty on lookupswitch");
  }
  code_emit()->goto_op(x->default_sux(), x->bci(), is_bb, NULL);
}



void ValueGen::do_LookupSwitch(LookupSwitch* x) {
  Item tag(x->tag());
  { ValueGen v(&tag, HintItem::no_hint(), this);
    load_item(&tag);
    rfree(&tag);
    set_no_result(x);
  }
  setup_phis_for_switch(&tag, x->stack());
  if (UseTableRanges) {
    GrowableArray<LookupRange*>* ranges = create_lookup_ranges(x);
    if (gen_code()) {
      int lng = ranges->length();
      for (int i = 0; i < lng && !code_emit()->must_bailout(); i++) {
        LookupRange* one_range = ranges->at(i);
        do_LookupSwitch_key(&tag, x, one_range);
      }
      do_LookupSwitch_default(x);
      code_emit()->check_codespace();
    } // gen_code
  } else {
    if (gen_code()) {
      int len = x->length();
      for (int i = 0; i < len && !code_emit()->must_bailout(); i++) {
        bool is_bb = is_backward_branch(x, x->sux_at(i));
        if (is_bb && !x->stack()->stack_is_empty()) {
          // we cannot emit safepoint data if expoession stack is empty,
          code_emit()->set_bailout("stack not empty on lookupswitch");
        }
        code_emit()->lookupswitch_op(&tag, x->key_at(i), x->sux_at(i), x->bci(), is_bb, NULL);
        code_emit()->check_codespace();
      }
      bool is_bb = is_backward_branch(x, x->default_sux());
      if (is_bb && !x->stack()->stack_is_empty()) {
        // we cannot emit safepoint data if expoession stack is empty,
        code_emit()->set_bailout("stack not empty on lookupswitch");
      }
      code_emit()->goto_op(x->default_sux(), x->bci(), is_bb, NULL);
    }
  }
}


void ValueGen::do_Return(Return* x) {
  if (x->type()->is_void()) {
    if (gen_code()) {
      if (x->is_synchronized()) {
	RInfo receiver;
	if (!method()->is_static()) {
	  // get cached receiver; receiver is illegal if there is no cached receiver
	  cached_cpu_local(0, receiver);
	}
	code_emit()->return_op_prolog(x->monitor_no(), receiver);
      }
      code_emit()->return_op(NULL);
    }
  } else {
    RInfo reg = result_register_for(x->type(), /*callee=*/true);
    Item result(x->result());
    HintItem result_hint(x->type(), reg);
    if (x->type()->is_float()) {
      result_hint.set_round32();
    }
    ValueGen r(&result, &result_hint, this);
    if (x->is_synchronized()) {
      // we do not need this, as the ??
      if (result.is_register() && is_caller_save_register(result.get_register())) {
        spill_value(x->result());
        result.update();
      }
      if (gen_code()) {
	RInfo receiver;
	if (!method()->is_static()) {
	  // get cached receiver; receiver is illegal if there is no cached receiver
	  cached_cpu_local(0, receiver);
	}
        code_emit()->return_op_prolog(x->monitor_no(), receiver);
      }
    }
    load_item_force(&result, reg);
    rfree(&result);
    if (gen_code()) {
      code_emit()->return_op(&result);
    }
  }
  set_no_result(x);
}


void ValueGen::do_Base(Base* x) {
  // nothing to do for now.
}

