/*
 * @(#)c1_FrameMap_sparc.cpp	1.25 01/02/09 14:43:15
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_FrameMap_sparc.cpp.incl"



//--------------------------------------------------------
//               ArgumentLocation
//--------------------------------------------------------


void ArgumentLocation::set_stack_location (int offset) {
  _n = offset;
  assert(is_stack_arg(), "bad stack argument offset");
}


void ArgumentLocation::set_register_location (int number) {
  _n = number;
  assert(is_register_arg(), "bad register number");
}


bool ArgumentLocation::is_register_arg () const {
  return _n >= 0 && _n < FrameMap::nof_reg_args;
}


bool ArgumentLocation::is_stack_arg () const {
  return _n >= frame::memory_parameter_word_sp_offset;
}


int ArgumentLocation::stack_offset_in_words () const {
  assert(is_stack_arg(), "argument is not on stack");
  return _n;
}


Address ArgumentLocation::incoming_stack_location () const {
  assert(is_stack_arg(), "argument is not on stack");
  return Address(FP, 0, (_n) * wordSize);
}


Address ArgumentLocation::outgoing_stack_location () const {
  assert(is_stack_arg(), "argument is not on stack");
  return Address(SP, 0, (_n) * wordSize);
}


RInfo ArgumentLocation::incoming_reg_location () const {
  assert(is_register_arg(), "argument is not in a register");
  return RInfo(Register('i', _n));
}


RInfo ArgumentLocation::outgoing_reg_location () const {
  assert(is_register_arg(), "argument is not in a register");
  return RInfo(Register('o', _n));
}


#ifndef PRODUCT

void ArgumentLocation::print (bool incoming) const {
  if (is_register_arg()) {
    RInfo r = incoming ? incoming_reg_location() : outgoing_reg_location();
    tty->print(" Register parameter: "); r.print(); tty->cr();
  } else if (is_stack_arg()) {
    tty->print_cr(" Stack parameter: %s + %d", (incoming ? "FP" : "SP"), stack_offset_in_words);
  } else if (is_illegal()) {
    tty->print_cr(" Illegal parameter location");
  } else {
    ShouldNotReachHere();
  }
}

#endif


//--------------------------------------------------------
//               FrameMap
//--------------------------------------------------------

FloatRegister FrameMap::_fpu_regs [FrameMap::nof_fpu_regs];
c1_RegMask    FrameMap::_callee_save_regs;
c1_RegMask    FrameMap::_caller_save_regs;


// some useful constant RInfo's:
RInfo FrameMap::_noRInfo;
RInfo FrameMap::_I0_I1_RInfo;
RInfo FrameMap::_O0_O1_RInfo;
RInfo FrameMap::_O2_O3_RInfo;
RInfo FrameMap::_f0_RInfo;
RInfo FrameMap::_f0_double_RInfo;

RInfo FrameMap::_g0_RInfo;
RInfo FrameMap::_g1_RInfo;
RInfo FrameMap::_g2_RInfo;
RInfo FrameMap::_g3_RInfo;
RInfo FrameMap::_g4_RInfo;
RInfo FrameMap::_g5_RInfo;
RInfo FrameMap::_g6_RInfo;
RInfo FrameMap::_g7_RInfo;
RInfo FrameMap::_O0_RInfo;
RInfo FrameMap::_O1_RInfo;
RInfo FrameMap::_O2_RInfo;
RInfo FrameMap::_O3_RInfo;
RInfo FrameMap::_O4_RInfo;
RInfo FrameMap::_O5_RInfo;
RInfo FrameMap::_O6_RInfo;
RInfo FrameMap::_O7_RInfo;
RInfo FrameMap::_L0_RInfo;
RInfo FrameMap::_L1_RInfo;
RInfo FrameMap::_L2_RInfo;
RInfo FrameMap::_L3_RInfo;
RInfo FrameMap::_L4_RInfo;
RInfo FrameMap::_L5_RInfo;
RInfo FrameMap::_L6_RInfo;
RInfo FrameMap::_L7_RInfo;
RInfo FrameMap::_I0_RInfo;
RInfo FrameMap::_I1_RInfo;
RInfo FrameMap::_I2_RInfo;
RInfo FrameMap::_I3_RInfo;
RInfo FrameMap::_I4_RInfo;
RInfo FrameMap::_I5_RInfo;
RInfo FrameMap::_I6_RInfo;
RInfo FrameMap::_I7_RInfo;


FloatRegister FrameMap::nr2floatreg (int rnr) {
  assert(_init_done, "tables not initialized");
  debug_only(fpu_range_check(rnr);)
  return _fpu_regs[rnr];
}


int FrameMap::om_framesize (int esp_offset) const {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  return round_to(_size_locals - _size_arguments + _size_monitors + _size_spills +
                  _max_stack_size + frame::memory_parameter_word_sp_offset, 2);
}

  
// returns true if reg could be smashed by a callee.
// If is_C_call, return true when register could be smashed by a C-call.
bool FrameMap::is_caller_save_register (RInfo reg, bool is_C_call) {
  if (reg.is_float() || reg.is_double()) { return true; }
  if (reg.is_long()) {
    return is_caller_save_register(reg.as_register_lo(), is_C_call) ||
           is_caller_save_register(reg.as_register_hi(), is_C_call);
  }
  return is_caller_save_register(reg.as_register(), is_C_call);
}


NEEDS_CLEANUP   // once the new calling convention is enabled, we no
		// longer need to treat I5, I4 and L0 specially
// Because the interpreter destroys caller's I5, I4 and L0,
// we must spill them before doing a Java call as we may land in
// interpreter.
bool FrameMap::is_caller_save_register (Register r, bool is_C_call) {
  if ((r.is_global() && (r != G0)) || r.is_out())           { return true; }
  if (UseC2CallingConventions) {
    if ( !is_C_call && ((r == I5) || (r == I4) || (r == L0))) { return true; }
  }
  return false;
}


RInfoCollection* FrameMap::caller_save_registers (bool is_C_call) {
  RInfoCollection* regs = new RInfoCollection();
  int i;
  for (i = 0; i < RegAlloc::nof_cpu_regs; i++) {
    if (is_caller_save_register(_cpu_regs[i], is_C_call)) {
      regs->append(RInfo(_cpu_regs[i]));
    }
  }
  for (i = 0; i < RegAlloc::nof_fpu_regs; i++) {
    regs->append(RInfo(_fpu_regs[i], false));
  }
  return regs;
}


FrameMap::FrameMap(int size_spill, const CachedLocals* cached_locals) {
  _size_locals = 0;
  _size_monitors = 0;
  _size_arguments = 0;
  _size_spills = size_spill;
  _max_stack_size = 0;
  _cached_locals = cached_locals;
  if (!_init_done) init();
}


void FrameMap::init () {
  if (_init_done) return;
  c1_RegMask::init_masks(nof_cpu_regs);

  int i=0;
  // Register usage:
  //  O6: sp
  //  I6: fp
  //  I7: return address
  //  G0: zero
  //  G2: thread
  //  G7: not available
  //  G6: not available
  /*  0 */ _cpu_regs[i++] = L0; // interpreter clobbered when UseC2CallingConventions
  /*  1 */ _cpu_regs[i++] = L1;
  /*  2 */ _cpu_regs[i++] = L2;
  /*  3 */ _cpu_regs[i++] = L3;
  /*  4 */ _cpu_regs[i++] = L4;
  /*  5 */ _cpu_regs[i++] = L5;
  /*  6 */ _cpu_regs[i++] = L6;
  /*  7 */ _cpu_regs[i++] = L7;
  
  /*  8 */ _cpu_regs[i++] = I5; // interpreter clobbered when UseC2CallingConvnetions
  /*  9 */ _cpu_regs[i++] = I4; // interpreter clobbered when UseC2CallingConventions
  /* 10 */ _cpu_regs[i++] = I3;
  /* 11 */ _cpu_regs[i++] = I2;
  /* 12 */ _cpu_regs[i++] = I1;
  /* 13 */ _cpu_regs[i++] = I0;
  /* 14 */ _cpu_regs[i++] = O0;
  /* 15 */ _cpu_regs[i++] = O1;
  /* 16 */ _cpu_regs[i++] = O2;
  /* 17 */ _cpu_regs[i++] = O3;
  /* 18 */ _cpu_regs[i++] = O4;
  /* 19 */ _cpu_regs[i++] = O5; // <- last register visible in RegAlloc (RegAlloc::nof+cpu_regs)
  /* 20 */ _cpu_regs[i++] = G1;
  /* 21 */ _cpu_regs[i++] = G3;
  /* 22 */ _cpu_regs[i++] = G4;
  /* 23 */ _cpu_regs[i++] = G5;
  assert(i == nof_cpu_regs, "number of CPU registers");

  for (i = 0; i < nof_fpu_regs; i++) {
    _fpu_regs[i] = FloatRegister(i, ' ');
  }

  _init_done = true;
  _noRInfo.set_no_rinfo();

  _I0_I1_RInfo.set_long_reg(I1, I0);
  _O0_O1_RInfo.set_long_reg(O1, O0);
  _O2_O3_RInfo.set_long_reg(O3, O2);

  _g0_RInfo.set_word_reg(G0);
  _g1_RInfo.set_word_reg(G1);
  _g2_RInfo.set_word_reg(G2);
  _g3_RInfo.set_word_reg(G3);
  _g4_RInfo.set_word_reg(G4);
  _g5_RInfo.set_word_reg(G5);
  _g6_RInfo.set_word_reg(G6);
  _g7_RInfo.set_word_reg(G7);
  _O0_RInfo.set_word_reg(O0);
  _O1_RInfo.set_word_reg(O1);
  _O2_RInfo.set_word_reg(O2);
  _O3_RInfo.set_word_reg(O3);
  _O4_RInfo.set_word_reg(O4);
  _O5_RInfo.set_word_reg(O5);
  _O6_RInfo.set_word_reg(O6);
  _O7_RInfo.set_word_reg(O7);
  _L0_RInfo.set_word_reg(L0);
  _L1_RInfo.set_word_reg(L1);
  _L2_RInfo.set_word_reg(L2);
  _L3_RInfo.set_word_reg(L3);
  _L4_RInfo.set_word_reg(L4);
  _L5_RInfo.set_word_reg(L5);
  _L6_RInfo.set_word_reg(L6);
  _L7_RInfo.set_word_reg(L7);
  _I0_RInfo.set_word_reg(I0);
  _I1_RInfo.set_word_reg(I1);
  _I2_RInfo.set_word_reg(I2);
  _I3_RInfo.set_word_reg(I3);
  _I4_RInfo.set_word_reg(I4);
  _I5_RInfo.set_word_reg(I5);
  _I6_RInfo.set_word_reg(I6);
  _I7_RInfo.set_word_reg(I7);

  _f0_RInfo.set_float_reg(0);
  _f0_double_RInfo.set_double_reg(0);

  // Add only registers that are visible to Register allocator
  for (i = 0; i < RegAlloc::nof_cpu_regs; i++) {
    if (is_caller_save_register(cpu_rnr2reg(i))) {
      _caller_save_regs.add_reg(i);
    } else {
      _callee_save_regs.add_reg(i);
    }      
  }
}

// ------------ extra spill slots ---------------

void FrameMap::add_spill_slots(int nof_slots) {
  _size_spills += nof_slots;
}

// ------------ argument locations --------------
//
// Maps argument locals to their locations on entry to a C1 method.
//
// The first 6 non-floating-point locals are passed in registers I0 through I5.
// Floating point args and those not-floats that those non-floats that don't make it into
// an I register are passed on the stack.
//
// There is stack space reserved for all arguments wether they are actually passed on
// registers or not. This stack sparc follows the same layout and conventions as the
// interpreter.
//
// For each local arg there is an int describing its location. If it's less than six it
// refers to an I register, otherwise it must be greater than
// frame::memory_parameter_word_sp_offset (23) and it refers to the arg's offset from fp
// in words.
//
// reg_args points to an array of six ints. reg_args[n] gives the local arg index passed in
// the nth I register or -1 if no argument is passed in that register.
//
// Note that double word arguments are referred as two consecutive locals.

CallingConvention* FrameMap::calling_convention(bool is_static, const GrowableArray<BasicType>& signature, GrowableArray<int>* reg_args) {
  CallingConvention* arg_location = new CallingConvention(signature.length());
  FrameMap fm(0, NULL);
  fm.set_size_arguments(signature.length());
  fm.set_size_locals   (signature.length());

  if (reg_args != NULL) {
    for (int reg = 0; reg < nof_reg_args; reg++) {
      reg_args->at_put(reg, -1);
    }
  }

  int idx = 0;
  while (idx < signature.length()) {
    BasicType t = signature.at(idx);
    int size = type2size[t];                       // either one word or two words
    assert(size == 1 || (size == 2 && t == signature.at(idx+1)), "sanity check");
    switch (t) {
    case T_BOOLEAN:
    case T_CHAR:
    case T_BYTE:
    case T_SHORT:
    case T_INT:
    case T_OBJECT:
    case T_ARRAY:
      {
	if (PassParametersInRegisters && idx < nof_reg_args) {
	  if (reg_args != NULL) reg_args->at_put(idx, idx);
	  arg_location->map(idx, ArgumentLocation::new_reg_arg(idx));
	} else {
	  arg_location->map(idx, ArgumentLocation::new_stack_arg(fm.local_offset_from_fp(idx, false, false)));
	}
	break;
      }
    case T_LONG:
      {
	// both halfs on stack (big endian!)
	arg_location->map(idx,   ArgumentLocation::new_stack_arg(fm.local_offset_from_fp(idx, true, true )));
	arg_location->map(idx+1, ArgumentLocation::new_stack_arg(fm.local_offset_from_fp(idx, true, false)));
	break;
      }
    default:
      {
	if (size == 1) {
	  arg_location->map(idx, ArgumentLocation::new_stack_arg(fm.local_offset_from_fp(idx, false, false)));
	} else {
	  arg_location->map(idx,   ArgumentLocation::new_stack_arg(fm.local_offset_from_fp(idx, true, true )));
	  arg_location->map(idx+1, ArgumentLocation::new_stack_arg(fm.local_offset_from_fp(idx, true, false)));
	}
	break;
      }
    }
    idx += size;
  }
  return arg_location;
}



// ----------------mapping-----------------------
// all mapping is based on FP addressing

bool FrameMap::are_adjacent_indeces(int jix1, int jix2) const {
  if (jix1 + 1 == jix2)
    return true;
  ShouldNotReachHere();
  return false;
}


int FrameMap::local_offset_from_fp(int local_index, bool is_two_word, bool for_hi_word) const {
  check_local_index(local_index);
  if (is_two_word && for_hi_word) {
    local_index++;
  }
  if (local_index < _size_arguments)
    return _size_arguments - local_index + frame::memory_parameter_word_sp_offset - 1;
  else
    return -(local_index - _size_arguments + 1);
}


int FrameMap::spill_offset_from_fp(int spill_index, bool is_two_word, bool for_hi_word) const {
  check_spill_index(spill_index);
  if (is_two_word && for_hi_word) {
    spill_index++;
  }
  return -(_size_locals - _size_arguments + _size_monitors + spill_index + 1);
}


int FrameMap::monitor_offset_from_fp(int monitor_index) const {
  check_monitor_index(monitor_index);
  return -(_size_locals - _size_arguments + 1 + monitor_index * (sizeof(BasicLock)/wordSize));
}


Address FrameMap::local_address(int local_index, bool is_two_word, bool for_hi_word) const {
  int offset_from_fp = local_offset_from_fp(local_index, is_two_word, for_hi_word);
  return Address(FP, 0, offset_from_fp * wordSize);
}


Address FrameMap::monitor_address(int monitor_index) const {
  assert(sizeof(BasicLock)/wordSize == 1, "not tested for this BasicLock");
  int offset_from_fp = monitor_offset_from_fp(monitor_index);
  return Address(FP, 0, offset_from_fp * wordSize);
}


Address FrameMap::spill_address(int spill_index, bool is_two_word, bool for_hi_word) const {
  int offset_from_fp = spill_offset_from_fp(spill_index, is_two_word, for_hi_word);
  return Address(FP, 0, offset_from_fp * wordSize);
}


// Map local_index to the esp offset 
// (example with 10 locals, including 5 arguments and esp_offset is 3)
//
//               arguments    register save area           monitor  spill   outgoing arg
//             [..............|................|..........|.......|.......|..............|
// local index  0  1  2  3  4                   5  6  7  8 
//

int FrameMap::esp_offset_for_local(int local_index, int esp_offset) const {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  return om_framesize() + local_offset_from_fp(local_index, false, false);
}

// The mapping of locals, registers and arguments is currently twisted
// (requirement by Compiler2): This is the way that the frame elements 
// must be passed in
// 
//   registers   stack0      esp[0]
// [...............|............|.....................]   RegName's
//                  arguments   
//                    ..  2 1 0 e e..    ... 6 5 4 x x local indeces 
//  (2 x's stand for old-fp and ret-addr)
//  (2 e's stand for esp-offset)
//
// Note that there is a possible esp-offset value that moves all the
// non-arguments locals to the right by esp_offset amount; 


OptoReg::Name FrameMap::regname_for(int in_index, int esp_offset) const {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  int index = -1;
  if (in_index < _size_arguments) {
    // it is an argument
    index = _size_arguments - in_index - 1;
    assert(0 <= index && index < _size_arguments, "wrong index");
  } else {
    // not an argument
#ifdef ASSERT
    int index_slow = (_size_arguments + om_framesize()) - in_index + _size_arguments - 1;
#endif
    index = 2 * _size_arguments - 1 + om_framesize() - in_index;
    assert(index_slow == index, "wrong oop map computation");
    assert (_size_arguments + esp_offset <= index  && 
            index < om_framesize() + _size_arguments, "wrong index");
    index -= frame::memory_parameter_word_sp_offset;
  }
  assert(index >= 0, "wrong index computed");
  return SharedInfo::stack2reg(index);
}


OptoReg::Name FrameMap::local_regname (int local_index, int esp_offset) const {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  return regname_for(local_index);
}


OptoReg::Name FrameMap::spill_regname (int spill_index, int esp_offset) const {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  int index = _size_locals + _size_monitors + spill_index; // FACTOR THIS OUT !
  return regname_for(index);
}


OptoReg::Name FrameMap::register_regname (RInfo reg, int esp_offset) const {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  return OptoReg::Name(reg.as_register().number());
}


OopMap* FrameMap::new_oopmap(int esp_offset) {
  assert(esp_offset == 0, "esp_offset not used for sparc");
  return new OopMap(om_framesize(), _size_arguments);
}
