/*
 * @(#)c1_Runtime1_sparc.cpp	1.56 01/02/09 14:43:18
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_c1_Runtime1_sparc.cpp.incl"
#include <v9/sys/psr_compat.h>


// Implementation of StubAssembler

int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry_point, int number_of_arguments) {
  assert_not_delayed();
  // bang stack before going to runtime
  set(-os::vm_page_size(), Gscratch);
  st(G0, SP, Gscratch);

  // debugging support
  assert(number_of_arguments >= 0   , "cannot have negative number of arguments");

  set_last_Java_frame(SP, G0);
  if (VerifyThread)  mov(Gthread, O0); // about to be smashed; pass early
  save_thread(Lthread_cache);
  // do the call
  call(entry_point, relocInfo::runtime_call_type);
  int call_offset = offset();
  if (!VerifyThread) {
    delayed()->mov(Gthread, O0);  // pass thread as first argument
  } else {
    delayed()->nop();             // (thread already passed)
  }
  restore_thread(Lthread_cache);
  reset_last_Java_frame();

  // check for pending exceptions
  { Label L;
    Address exception_addr(Gthread,0,Thread::pending_exception_offset_in_bytes());
    ld(exception_addr, Gtemp);
    tst(Gtemp);
    br(equal, false, pt, L);
    delayed()->nop();
    Address vm_result_addr(Gthread, 0, JavaThread::vm_result_offset_in_bytes());
    st(G0, vm_result_addr);
    Address vm_result_addr_2(Gthread, 0, JavaThread::vm_result_2_offset_in_bytes());
    st(G0, vm_result_addr_2);

    // we use O7 linkage so that forward_exception_entry has the issuing PC
    call(StubRoutines::forward_exception_entry(), relocInfo::runtime_call_type);
    delayed()->restore();
    bind(L);
  }

  // get oop result if there is one and reset the value in the thread
  if (oop_result1.isValid()) {                    // get oop result if there is one and reset it in the thread
    get_vm_result  (oop_result1);
  }
  Address vm_result_addr(Gthread, 0, JavaThread::vm_result_offset_in_bytes());
  st(G0, vm_result_addr);

  if (oop_result2.isValid()) {
    get_vm_result_2(oop_result2);
  }
  Address vm_result_addr_2(Gthread, 0, JavaThread::vm_result_2_offset_in_bytes());
  st(G0, vm_result_addr_2);

  return call_offset;
}


int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1) {
  // O0 is reserved for the thread
  mov(arg1, O1);
  return call_RT(oop_result1, oop_result2, entry, 1);
}


int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1, Register arg2) {
  // O0 is reserved for the thread
  mov(arg1, O1);
  mov(arg2, O2); assert(arg2 != O1, "smashed argument");
  return call_RT(oop_result1, oop_result2, entry, 2);
}


int StubAssembler::call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1, Register arg2, Register arg3) {
  // O0 is reserved for the thread
  mov(arg1, O1);
  mov(arg2, O2); assert(arg2 != O1,               "smashed argument");
  mov(arg3, O3); assert(arg3 != O1 && arg3 != O2, "smashed argument");
  return call_RT(oop_result1, oop_result2, entry, 3);
}

// Implementation of StubFrame

class StubFrame: public StackObj {
 private:
  StubAssembler* _sasm;
  int            _args_size;

 public:
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments);
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1);
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2);
  StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2, Register arg3);

  ~StubFrame();
};


#define __ _sasm->
StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments) {
  _sasm = sasm;
  _args_size = 0;
  __ set_info(name, must_gc_arguments);
  __ save_frame(0);
}


StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1) {
  _sasm = sasm;
  _args_size = 1;
  __ set_info(name, must_gc_arguments);
  __ save_frame(0);
  // tos + 0: link
  //     + 1: return address
  //     + 2: arg1
  Unimplemented();
  // __ movl(arg1, Address(ebp, 2*BytesPerWord));
}


StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2) {
  _sasm = sasm;
  _args_size = 2;
  __ set_info(name, must_gc_arguments);
  assert(arg1 != arg2, "registers must be different");
  __ save_frame(0);
  // tos + 0: link
  //     + 1: return address
  //     + 2: arg2
  //     + 3: arg1
  Unimplemented();

  // __ movl(arg2, Address(ebp, 2*BytesPerWord));
  // __ movl(arg1, Address(ebp, 3*BytesPerWord));
}


StubFrame::StubFrame(StubAssembler* sasm, const char* name, bool must_gc_arguments, Register arg1, Register arg2, Register arg3) {
  _sasm = sasm;
  _args_size = 3;
  __ set_info(name, must_gc_arguments);
  assert(arg1 != arg2 && arg1 != arg3 && arg2 != arg3, "registers must be different");
  __ save_frame(0);
  // tos + 0: link
  //     + 1: return address
  //     + 2: arg3
  //     + 3: arg2
  //     + 4: arg1
  Unimplemented();

  // __ movl(arg3, Address(ebp, 2*BytesPerWord));
  // __ movl(arg2, Address(ebp, 3*BytesPerWord));
  // __ movl(arg1, Address(ebp, 4*BytesPerWord));
}


StubFrame::~StubFrame() {
  __ ret();
  __ delayed()->restore();
}
#undef __



// Implementation of Runtime1

void Runtime1::generate_exception_throw(StubAssembler* sasm, const char* msg, address runtime_call) {
#define __ sasm->
  const bool dont_gc_arguments = false;
  // G1: pc offset where exception happened
  // O7->I7: address pointing into method
  __ save_frame(0);
  __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, compute_exception_pc), G1, I7);
  // O0 is the address of exception
  __ sub(O0, frame::pc_return_offset * BytesPerWord, O7.after_save());
  __ restore();
  __ set_info(msg, dont_gc_arguments);
  // it looks as we were called from the exception point
  __ save_frame(0);
  // execute a throw
  __ call_RT(noreg, noreg, runtime_call);
  __ should_not_reach_here();
#undef __
}


OopMapSet* Runtime1::generate_illegal_instruction_handler(StubAssembler* sasm, address target) {
#define __ sasm->
  const bool dont_gc_arguments = false;
  // the frame contains all floating point registers and the CC register
  const int frame_size = FrameMap::nof_fpu_regs + 1;
  // make room to save fpu registers
  __ save_frame(frame_size);
  // save fpu registers
  FrameMap fm(0, NULL);
  fm.set_size_arguments(0);
  fm.set_size_locals   (frame_size);

  const Register saved_ccr = L1;
  { Address  adr = fm.local_address(0, false);
    if (VM_Version::v9_instructions_work()) {
      __ rdccr(L1);
#ifdef DEBUG
      // Test code sequence used on V8.
      const Register temp_reg = Gscratch;
      const Register temp_reg2 = Gscratch2;
      Label get_psr_test;
      // Get the condition codes the V8 way.
      __ get_psr_trap();
      __ nop();
      __ set(PSR_ICC, temp_reg);
      __ and3(O0, temp_reg, temp_reg);
      __ srl(temp_reg, PSR_ICC_SHIFT, temp_reg);
      __ mov(L1, temp_reg2);
      // This is a test of V8 which has icc but not xcc 
      // so mask off the xcc bits
      __ and3(temp_reg2, 0xf, temp_reg2);
      // Compare condition codes from the V8 and V9 ways.
      __ subcc(temp_reg2, temp_reg, G0);
      __ br(Assembler::notEqual, true, Assembler::pt, get_psr_test);
      __ delayed()->breakpoint_trap();
      __ bind(get_psr_test);
#endif
    } else {
      // Execute a trap to get the PSR, mask and shift 
      // to get the condition codes.
      __ get_psr_trap();
      __ nop();
      __ set(PSR_ICC, L1);
      __ and3(O0, L1, L1);
      __ srl(L1, PSR_ICC_SHIFT, L1);
    }
    __ stw(L1, adr.base(), adr.disp());
  }
  int i;
  for (i = 0; i < FrameMap::nof_fpu_regs; i++) {
    Address  adr = fm.local_address(i + 1, false);
    __ stf(FloatRegister::S, fm.nr2floatreg(i), adr.base(), adr.disp());
  }

  // setup last_Java_sp
  __ set_last_Java_frame(SP, G0);

  // call into the runtime to handle illegal instructions exception
  __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SafepointSynchronize::handle_illegal_instruction_exception), Gthread);

  // clear last_Java_sp
  __ reset_last_Java_frame();

  // check for pending exceptions
  { Label L;
    Address exception_addr(Gthread,0,Thread::pending_exception_offset_in_bytes());
    __ ld(exception_addr, Gtemp);
    __ tst(Gtemp);
    __ br(Assembler::equal, false, Assembler::pt, L);
    __ delayed()->nop();

    // restore saved state from thread local storage 
    // NOTE: floating point registers are not restored (no need, it's an exception)!
    __ restore();
    const Register temp_reg = Gscratch;

    Address exc(temp_reg, StubRoutines::forward_exception_entry());
    __ jump_to(exc, 0);
    __ delayed()->nop();
    __ bind(L);
  }

  // return to the instruction so that it is reexecuted
  // restore fpu registers
  for (i = 0; i < FrameMap::nof_fpu_regs; i++) {
    Address  adr = fm.local_address(i + 1, false);
    __ ldf(FloatRegister::S, adr.base(), adr.disp(), fm.nr2floatreg(i)); 
  }

  { Address  adr = fm.local_address(0, false);
    __ lduw(adr.base(), adr.disp(), L1);
    if (VM_Version::v9_instructions_work()) {
#ifdef DEBUG
      // Test code sequence used on V8.
      Label set_psr_test;
      const Register temp_reg = Gscratch;
      const Register temp_reg2 = Gscratch2;
      // Write out the saved condition codes the V8 way
      __ sll(L1, PSR_ICC_SHIFT, temp_reg2);
      __ get_psr_trap();
      __ nop();
      __ set(~PSR_ICC, temp_reg);
      __ and3(O0, temp_reg, O0);
      __ or3(O0, temp_reg2, O0);
      __ set_psr_trap();
      __ nop();
      // Read back the condition codes using the V9 instruction
      __ rdccr(temp_reg);
      __ mov(L1, temp_reg2);
      // This is a test of V8 which has icc but not xcc 
      // so mask off the xcc bits
      __ and3(temp_reg2, 0xf, temp_reg2);
      __ and3(temp_reg, 0xf, temp_reg);
      // Compare the V8 way with the V9 way.
      __ subcc(temp_reg2, temp_reg, G0);
      __ br(Assembler::notEqual, true, Assembler::pt, set_psr_test);
      __ delayed()->breakpoint_trap();
      __ bind(set_psr_test);
#endif
      __ wrccr(L1);
    } else {
      const Register temp_reg = Gscratch;
      // Execute a trap to get the PSR, shift back
      // the condition codes, mask the condition codes
      // back into and PSR and trap to write back the
      // PSR.
      __ sll(L1, PSR_ICC_SHIFT, L1);
      __ get_psr_trap();
      __ nop();
      __ set(~PSR_ICC, temp_reg);
      __ and3(O0, temp_reg, O0);
      __ or3(O0, L1, O0);
      __ set_psr_trap();
      __ nop();
    }
  }

  __ ret();
  __ delayed()->restore();

  return NULL;
#undef __
}


OopMapSet* Runtime1::generate_patching(StubAssembler* sasm, address target) {
#define __ sasm->
  const bool dont_gc_arguments = false;
  __ set_info("generate_init_check_patching", dont_gc_arguments);
  // make room to save fpu registers
  __ save_frame(FrameMap::nof_fpu_regs);
  // save fpu registers
  FrameMap fm(0, NULL);
  fm.set_size_arguments(0);
  fm.set_size_locals   (FrameMap::nof_fpu_regs);

  for (int i = 0; i < FrameMap::nof_fpu_regs; i++) {
    Address  adr = fm.local_address(i, false);
    __ stf(FloatRegister::S, fm.nr2floatreg(i), adr.base(), adr.disp());
  }

  int call_offset = __ call_RT(noreg, noreg, target);

  // return to the instruction so that it is reexecuted
  // restore fpu registers
  for (int j = 0; j < FrameMap::nof_fpu_regs; j++) {
    Address  adr = fm.local_address(j, false);
    __ ldf(FloatRegister::S, adr.base(), adr.disp(), fm.nr2floatreg(j)); 
  }

  __ ret(0);
  __ delayed()->restore();
  return NULL;
#undef __
}


OopMapSet* Runtime1::generate_code_for(StubID id, StubAssembler* sasm) {
#define __ sasm->

  OopMapSet* oop_maps = NULL;
  // for better readability
  const bool must_gc_arguments = true;
  const bool dont_gc_arguments = false;

  // stub code & info for the different stubs
  switch (id) {
    case new_instance_id:
      { StubFrame f(sasm, "new_instance", dont_gc_arguments);
        // G4: klassOop
        __ call_RT(I0, noreg, CAST_FROM_FN_PTR(address, new_instance), G4);
        // I0->O0: new instance
      }

      break;

    case new_type_array_id:
      { StubFrame f(sasm, "new_type_array", dont_gc_arguments);
        // G4: elt_type
        // G5: length   
        __ call_RT(G4, noreg, CAST_FROM_FN_PTR(address, new_type_array), G4, G5);
        // G4: new type array
      }
      break;

    case new_object_array_id:
      { // G4: klass
        // G5: length
        __ set_info("new_object_array", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(G4, noreg, CAST_FROM_FN_PTR(address, new_object_array), G4, G5);
	__ ret();
	__ delayed()->restore();
        // G4: new object array
      }
      break;

    case new_multi_array_id:
      { // O0: klass
        // O1: rank
        // O2: address of 1st dimension
        __ set_info("new_multi_array", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(I0, noreg, CAST_FROM_FN_PTR(address, new_multi_array), I0, I1, I2);
	__ ret();
	__ delayed()->restore();
        // I0: new multi array
      }
      break;

    case resolve_invokestatic_id:
      { 
	__ set_info("resolve_invokestatic", must_gc_arguments);
	__ save_frame(0);
	__ call_RT(Gmethod, noreg, CAST_FROM_FN_PTR(address, resolve_invoke), G0); 
	__ jmp(O0, 0);
	__ delayed()->restore();
      }
      break;

    case resolve_invoke_id:
      {
	__ set_info("resolve_invoke", must_gc_arguments);
	__ save_frame(0);
	__ call_RT(Gmethod, noreg, CAST_FROM_FN_PTR(address, resolve_invoke), I0); 
	__ jmp(O0, 0);
	__ delayed()->restore();
      }
      break;

    case handle_ic_miss_id:
      {
        __ set_info("handle_ic_miss", must_gc_arguments);
        // Ginline_cache_reg : ic klass
        // O0 : receiver
        __ save_frame(0);
        __ call_RT(Gmethod, I0, CAST_FROM_FN_PTR(address, handle_ic_miss), I0);
        // O0: interpreter or verified entry point
        // Gmethod: callee method (always)
        // I0: receiver (becomes O0 upon restore)
        __ jmp(O0, 0);
        __ delayed()->restore();
      }
      break;

    case handle_wrong_static_method_id:
      {
        __ set_info("handle_wrong_static_method", must_gc_arguments);
        __ save_frame(0);
        __ call_RT(Gmethod, noreg, CAST_FROM_FN_PTR(address, handle_wrong_method), G0);
        // O0: interpreter or verfied entry point
        // Gmethod: callee method (always)
        __ jmp(O0, 0);
        __ delayed()->restore();
      }
      break;

    case handle_wrong_method_id:
      {
        __ set_info("handle_wrong_method", must_gc_arguments);
        __ save_frame(0);
        __ call_RT(Gmethod, I0, CAST_FROM_FN_PTR(address, handle_wrong_method), I0);
        // O0: interpreter or verfied entry point
        // Gmethod: callee method (always)
        // I0: receiver (becomes O0 upon restore)
        __ jmp(O0, 0);
        __ delayed()->restore();
      }
      break;

    case entering_non_entrant_id:
      { __ set_info("entering_non_entrant", must_gc_arguments);
        __ stop ("we entered a non entrant method: internal error");
      }
      break;

    case arithmetic_call_id:
      __ unimplemented("arithmetic_call_id"); // unused
      break;

    case throw_abstract_method_error_id:
      { StubFrame f(sasm, "throw_abstract_method_exception", dont_gc_arguments); // arguments will be discarded
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_abstract_method_error));
        __ should_not_reach_here();
      }
      break;

    case range_check_failed_id:
      generate_exception_throw(sasm, "range_check_failed", CAST_FROM_FN_PTR(address, throw_range_check_exception));
      break;

    case throw_div0_exception_id:
      generate_exception_throw(sasm, "throw_div0_exception", CAST_FROM_FN_PTR(address, throw_div0_exception));
      break;

    case throw_null_exception_id:
      generate_exception_throw(sasm, "throw_null_exception_at_call", CAST_FROM_FN_PTR(address, throw_null_exception_at_call));
      break;

    case throw_exception_id:
      { // O0: exception
        __ set_info("throw_exception", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(O1, noreg, CAST_FROM_FN_PTR(address, throw_exception), I0);
        __ should_not_reach_here();
      }
      break;

    case handle_exception_id:
      {
        Label no_handler;
        // O0: exception
        // O1: issuing pc
        __ set_info("handle_exception", dont_gc_arguments);
        __ save_frame(0);
        __ mov(O7.after_save(), L0); // save return address
        // use throwing pc as return address (has bci & oop map)
        __ sub(Oissuing_pc.after_save(), frame::pc_return_offset * BytesPerWord, O7.after_save());
        __ call_RT(Oexception.after_save(), noreg, CAST_FROM_FN_PTR(address, exception_handler_for_pc),
                   Oexception.after_save(), Oissuing_pc.after_save());
        __ verify_oop(Oexception.after_save());
        __ mov(L0, O7.after_save()); // restore return address
        __ tst(O0);
        __ br(Assembler::zero, false, Assembler::pn, no_handler);
        __ delayed()->nop();
        __ jmp(O0, 0);
        __ delayed()->restore();
        __ bind(no_handler);
        __ ret();
        __ delayed()->restore();
      }
      break;

    case unwind_exception_id:
      {
        // Upon entry I7 has the return address returning into Java
        // (compiled or interpreted) code. O0 has the exception.

        __ set_info("unwind_exception", dont_gc_arguments); 
        __ mov(Oexception, Oexception .after_save()); // get exception in I0 so it will be 
						      // O0 after restore
        __ add(O7.after_save(), frame::pc_return_offset * BytesPerWord, Oissuing_pc.after_save()); // Likewise for I1
        __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address),
			Oissuing_pc.after_save()); 

        __ jmp(O0, 0);
        __ delayed()->restore();
      }
      break;

    case store_check_id:
      { StubFrame f(sasm, "store_check", dont_gc_arguments);
        // G4 array
        // G5 value
        __ call_RT(G4, G5, CAST_FROM_FN_PTR(address, store_check), G4, G5);
        // G4 array
        // G5 value
      }
      break;

    case checkcast_id:
      { // G4: object
        // G5: klass
        __ set_info("checkcast", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(G5, noreg, CAST_FROM_FN_PTR(address, checkcast), G4, G5);
	__ ret();
	__ delayed()->restore();
        // G5 result object
      }
      break;

    case instanceof_id:
      { // G4: object
        // G5: klass
        __ set_info("instanceof", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, instanceof), G4, G5);
        __ mov(O0, G5);
        __ ret();
        __ delayed()->restore();
	// __ delayed()->restore(O0, 0, O0);
        // G5: result (0 or 1)
      }
      break;

    case monitorenter_id:
      { // O0: object
        // O1: lock address
        __ set_info("monitorenter", dont_gc_arguments);
	__ save_frame(0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, monitorenter), I0, I1);
	__ ret();
	__ delayed()->restore();
      }
      break;

    case monitorexit_id:
      { // G4: object
        // G5: lock address
        // note: really a leaf routine but must setup last java sp
        //       => use call_RT for now (speed can be improved by
        //       doing last java sp setup manually)
        __ set_info("monitorexit", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, monitorexit), G4, G5);
	__ ret();
	__ delayed()->restore();
      }
      break;

    case implicit_div0_exception_id: // fall through
      {
        // we are in the exception handler stub frame
        // O7: filter return address
        __ set_info("implicit_div0_exception", dont_gc_arguments);
        __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, return_address_for_exception));
        __ sub(O0, frame::pc_return_offset * wordSize, I7);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_div0_exception));
        __ should_not_reach_here();
      }
      break;

    case implicit_null_exception_id:
      {
        // we are in the exception handler stub frame
        // O7: filter return address
        Label L;
        __ set_info("implicit_null_exception", dont_gc_arguments);
        __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, return_address_for_exception));
        __ tst(O0);
        __ br(Assembler::notEqual, true, Assembler::pt, L);
        __ delayed()->sub(O0, frame::pc_return_offset * wordSize, I7);
        __ bind(L);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_null_exception));
        __ should_not_reach_here();
      }
      break;

    case interpreter_entries_id:
      { __ set_info("interpreter_entries", PassParametersInRegisters);
        __ stop("do not call interpreter entries entry directly");
        // generate entries for all method kinds
        for (int i = 0; i < AbstractInterpreter::number_of_method_entries; i++) {
          AbstractInterpreter::MethodKind kind = (AbstractInterpreter::MethodKind)i;

	  Register recv       = O0;
	  Register icache     = G5;
          //       Gargs      = G4; 

          //--------------------------------------------------------------------------------
          // entry point to methods that are final
          // (icache contains IC holder oop => get methodOop)
          int virtual_final_call_offset = __ offset();
          Label optimized_call;
          __ verify_oop(recv);
	  __ verify_oop(icache);
          __ br(Assembler::always, false, Assembler::pt, optimized_call);
          __ delayed()->ld(icache, compiledICHolderOopDesc::holder_method_offset(), icache);


          //--------------------------------------------------------------------------------
          // entry point for virtual calls
          // (icache contains IC holder oop => test & get methodOop)
          int virtual_call_offset = __ offset();
	  { const Register klass1 = G1;
            const Register klass2 = G3;
            assert_different_registers(recv, icache, klass1, klass2);

	    __ verify_oop(recv);
	    __ verify_oop(icache);
	    __ ld(recv  , oopDesc::klass_offset_in_bytes              (), klass1);
	    __ ld(icache, compiledICHolderOopDesc::holder_klass_offset(), klass2);
	    __ verify_oop(klass1);
            __ verify_oop(klass2);
	    __ cmp(klass1, klass2);
	    __ br(Assembler::equal, false, Assembler::pt, optimized_call);
	    __ delayed()->ld(icache, compiledICHolderOopDesc::holder_method_offset(), icache);
	    __ jump_to(Address(klass1, entry_for(handle_ic_miss_id)));
	    __ delayed()->nop();
          }

          //--------------------------------------------------------------------------------
          // entry point for optimized virtual calls
          // (icache contains methodOop => test if target is compiled)
          int optimized_call_offset = __ offset();
          Label setup_parameters;
          __ bind(optimized_call);
	  { const Register tmp = G1;
            assert_different_registers(recv, icache, tmp);
            
            Label L;
	    __ verify_oop(recv);
            __ verify_oop(icache);
            __ ld(icache, methodOopDesc::compiled_code_offset_in_bytes(), tmp);
	    __ tst(tmp);
	    __ br(Assembler::zero, false, Assembler::pt, setup_parameters);
	    __ delayed()->nop();
	    __ jump_to(Address(tmp, entry_for(handle_wrong_method_id)));
	    __ delayed()->nop();
          }

          //--------------------------------------------------------------------------------
          // entry point for static calls
          // (icache contains methodOop)
          int static_call_offset = __ offset();
	  { const Register tmp = G1;
            assert_different_registers(recv, icache, tmp);
            
            Label L;
            __ verify_oop(icache);
            __ ld(icache, methodOopDesc::compiled_code_offset_in_bytes(), tmp);
            __ tst(tmp);
            __ br(Assembler::zero, false, Assembler::pt, setup_parameters);
            __ delayed()->nop();
            __ jump_to(Address(tmp, entry_for(handle_wrong_static_method_id)));
            __ delayed()->nop();
          }

          //--------------------------------------------------------------------------------
          // copy parameters on stack if they are passed in registers
          { Label setup_parameter_map;
            const Register map = Gargs;
            const Register dst = G3;
            assert_different_registers(recv, icache, map, dst);

            if (PassParametersInRegisters) {
              // Note: set_info is setup at the beginning of the interpreter entries!
              __ bind(setup_parameter_map);
              __ save_frame(0);
              __ call_RT(icache, noreg, CAST_FROM_FN_PTR(address, prepare_interpreter_call), icache);
              __ restore();
            }

            __ bind(setup_parameters);
            if (PassParametersInRegisters) {
              // setup registers
              __ lduh(icache, methodOopDesc::size_of_parameters_offset_in_bytes(), dst);
              __ lduh(icache, methodOopDesc::parameter_info_offset_in_bytes    (), map);
              __ sll(dst, LogBytesPerWord, dst);      // parameter size in bytes
              __ add(SP, dst, dst);
              // make sure parameter map is setup
              __ tst(map);
              __ br(Assembler::zero, false, Assembler::pn, setup_parameter_map);
              // copy parameters
              const int n = SPARC_ARGS_IN_REGS_NUM;   // number of register that can be used for parameter passing
              const int average_parameter_size = 2;   // average no. of parameters - for branch prediction only
              __   delayed()->btst(1 << 0, map);
              for (int i = 1; i <= n; i++) {
                Label L;
                __ br(Assembler::zero, false, i <= average_parameter_size ? Assembler::pn : Assembler::pt, L);
                __ delayed()->btst(1 << i, map);      // for i == n, this test is not needed, but it doesn't matter
                __ st(Register('o', i - 1), dst, (frame::memory_parameter_word_sp_offset - i) * BytesPerWord);
                __ bind(L);
              }
            }
          }

          // continue in interpreter
          { const Register tmp = G3;
            assert_different_registers(recv, icache, Gargs, tmp);
            __ jump_to(Address(tmp, AbstractInterpreter::entry_for_kind(kind)));
            __ delayed()->add(SP, frame::memory_parameter_word_sp_offset * BytesPerWord, Gargs);
          }

          //--------------------------------------------------------------------------------
          // setup entries
          _ientries[i] = iEntries(static_call_offset, optimized_call_offset, virtual_call_offset, virtual_final_call_offset);
        }
      }
      break;

    case stack_overflow_exception_id:
      { __ set_info("stack_overflow_exception", dont_gc_arguments);
        __ restore();
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, throw_stack_overflow));
        __ should_not_reach_here();
      }
      break;

    case init_check_patching_id:
      { __ set_info("init_check_patching", dont_gc_arguments);
        // we should set up register map
        oop_maps = generate_patching(sasm, CAST_FROM_FN_PTR(address, init_check_patching));
      }
      break;

    case load_klass_patching_id:
      { __ set_info("load_klass_patching", dont_gc_arguments);
        // we should set up register map
        oop_maps = generate_patching(sasm, CAST_FROM_FN_PTR(address, move_klass_patching));
      }
      break;

    case osr_frame_return_id:
      { __ set_info("osr_frame_return", dont_gc_arguments);
	// Continuation point after returning from osr compiled method.
        // Position a potential integer result for returning from the original interpreted activation.
        __ mov(O0, I0);
        __ mov(O1, I1);
        // Return, restoring the caller's register window and the calling activation's SP to
	// its value before it was changed to accommodate the method's non-parameter locals.
	// The SP adjustment was calculated in Lscratch by the osr entry code - see CodeEmitter::emit_osr_entry().
        __ ret();
        __ delayed()->restore(FP, Lscratch, SP);   // callee's FP + Lscratch -> caller's SP
      }
      break;

    case osr_unwind_exception_id:
      { __ set_info("osr_unwind_exception", dont_gc_arguments);
        // Upon entry, Oexception holds the exception, and I7 holds the PC of the call that created this activation.
        // Position the exception oop and calling PC for the unwinding restore instruction, which also protects them
        // from the ensuing VM call.
        __ mov(Oexception, Oexception .after_save());
        __ add(I7, frame::pc_return_offset * BytesPerWord, Oissuing_pc.after_save());

        // Get handler address in O0.
        __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address),
			Oissuing_pc.after_save()); 

        // Return, restoring the caller's register window and the calling activation's SP to
	// its value before it was changed to accomodate the method's non-parameter locals.
	// The SP adjustment was calculated in Lscratch by the osr entry code - see CodeEmitter::emit_osr_entry().
        __ jmp(O0, 0);
        __ delayed()->restore(FP, Lscratch, SP);   // callee's FP + Lscratch -> caller's SP
      }
      break;

    case illegal_instruction_handler_id:
      { __ set_info("illegal_instruction_handler", dont_gc_arguments);
        oop_maps = generate_illegal_instruction_handler(sasm, CAST_FROM_FN_PTR(address, SafepointSynchronize::handle_illegal_instruction_exception));
      }
      break;

    case address_bci_2_pc_id:
      { // O0: bci
        // O1: nmethod_pc
        __ set_info("bci_2_pc", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, bci_2_pc), I0, I1);
        __ ret();
        __ delayed()->restore(O0, 0, O0);
        // O0: pc
      }
      break;

    case safepoint_block_id:
      __ unimplemented("safepoint block id is not implemented");
      break;


    case jvmpi_method_entry:
      { // O0: methodOop
        // O1: receiver or NULL
        __ set_info("jvmpi_method_entry", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_entry), I0, I1);
        __ ret();
        __ delayed()->restore();
      }
      break;

    case jvmpi_method_exit:
      { // O0: methodOop
        __ set_info("jvmpi_method_exit", dont_gc_arguments);
        __ save_frame(0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_exit), I0);
        __ ret();
        __ delayed()->restore();
      }
      break;

    default:
      { __ set_info("unimplemented entry", dont_gc_arguments);
        __ save_frame(0);
        __ set((int)id, O0);
        __ call_RT(noreg, noreg, CAST_FROM_FN_PTR(address, unimplemented_entry), I0);
        __ should_not_reach_here();
      }
      break;
  }
#undef __
  return oop_maps;
}


class ParameterMapper: public SignatureIterator {
 protected:
  int _recv_slot;
  int _map;                                      // bitmap: if i.th bit is set => register Ii is a parameter

  void copy() {
    const int i = _recv_slot + parameter_index();
    if (i < SPARC_ARGS_IN_REGS_NUM) _map |= (1 << i);
  }

 public:
  // Constructor
  ParameterMapper(methodOop method) : SignatureIterator(method->signature()) {
    _recv_slot = method->is_static() ? 0 : 1;
    _map       = 0;
  }

  // Accessors
  int map() {
    // compute _map lazily
    if (_map == 0) {
      // set 1st bit if we have a receiver and set 'termination' bit to make sure _map != 0
      _map = _recv_slot | (1 << SPARC_ARGS_IN_REGS_NUM);
      // set bit for remaining parameters passed in registers
      iterate_parameters();
    }
    return _map;
  }

  // Basic types
  virtual void do_bool  ()                       { copy(); }
  virtual void do_char  ()                       { copy(); }
  virtual void do_float ()                       { /* ignore */ }
  virtual void do_double()                       { /* ignore */ }
  virtual void do_byte  ()                       { copy(); }
  virtual void do_short ()                       { copy(); }
  virtual void do_int   ()                       { copy(); }
  virtual void do_long  ()                       { /* ignore */ }
  virtual void do_void  ()                       { ShouldNotReachHere(); }

  // Object types (begin indexes the first character of the entry, end indexes the first character after the entry)
  virtual void do_object(int begin, int end)     { copy(); }
  virtual void do_array (int begin, int end)     { copy(); }
};


JRT_ENTRY(void, Runtime1::prepare_interpreter_call(JavaThread* thread, methodOop method))
  // Note: This function may be called by more than one thread simultaneously.
  //       However, no locking is required since all threads will compute the
  //       same result.
  assert(PassParametersInRegisters, "must only be called if arguments are passed in registers");
  thread->set_vm_result(method);
  if (method->parameter_info() == 0) {
    // only compute parameter info if it has not been computed
    // before and no thread has won the race in the meantime
    method->set_parameter_info(ParameterMapper(method).map());
  }
JRT_END


// Implementation of GC_Support

class FindSignatureTypes: public SignatureInfo {
 private:
  GrowableArray<BasicType>* _types;
  int  _index;
  bool _is_static;
 public:
   
  FindSignatureTypes(symbolHandle signature, bool is_static, GrowableArray<BasicType>* types): SignatureInfo(signature) {
    _types = types;
    _index = 0;
    _is_static = is_static;
  }

  void set(int size, BasicType type) {
    _types->at_put(_index, type);
    if (size == 2) _types->at_put(_index+1, type);
    _index += size;

  }

  void find() {
    if (!_is_static) {
      _types->at_put(0, T_OBJECT);
      _index ++;
    }
    iterate_parameters();
  }
};


class C1_ArgumentOopFinder: public SignatureInfo {
 private:
  void              (*_f)(oop *); // Method to call
  bool                _is_static;   // true if the callee is a static method
  const frame         _fr;
  CallingConvention*  _calling_convention;
  int                 _idx;

  void set(int size, BasicType type) {
    if (type == T_OBJECT || type == T_ARRAY) {
      oop_arg_do();
    }
    _idx += size;
  }

  void oop_arg_do() {
    ArgumentLocation location = _calling_convention->arg_at(_idx);
    oop* addr = (oop*)GC_Support::get_argument_addr_at(_fr, location);
    _f(addr);
  }

 public:
  C1_ArgumentOopFinder(symbolHandle signature,
		       bool is_static,
		       const frame fr,
		       void f(oop*))
    : SignatureInfo(signature),
      _f(f),
      _is_static(is_static),
      _fr(fr) {
    _idx = 0;
    int args_size = ArgumentSizeComputer(signature).size() + (is_static ? 0 : 1);
    GrowableArray<BasicType> sig_types = GrowableArray<BasicType>(args_size, args_size, T_ILLEGAL);
    FindSignatureTypes type_finder(signature, is_static, &sig_types);
    type_finder.find();
    _calling_convention = FrameMap::calling_convention(is_static, sig_types);
  }

  void oops_do() {
    _idx = 0;
    if (!_is_static) {
      if (PassParametersInRegisters) {
	oop_arg_do();
      } else {
	// get rid of this once we turn on PassParametersInRegisters by default
	// the receiver is in two locations: %o0 and on the stack
	NEEDS_CLEANUP;	 
	oop* addr = (oop*)GC_Support::get_register_argument_addr_at(_fr, O0); // take care of O0 
	_f(addr);      
	oop_arg_do();  // take care of the stack location
      }
      _idx++;
    }
    iterate_parameters();
  }
};


void GC_Support::preserve_callee_argument_oops(const frame fr, symbolHandle signature, bool is_static, void f(oop*)) {
  C1_ArgumentOopFinder finder(signature, is_static, fr, f);
  finder.oops_do();
}


jint* GC_Support::get_argument_addr_at(const frame fr, ArgumentLocation location) {
  if (location.is_stack_arg()) {
    int offset_from_sp_in_words = location.stack_offset_in_words();
    return get_stack_argument_addr_at(fr, offset_from_sp_in_words);
  } else {
    assert(location.is_register_arg(), "sanity check");
    Register r = location.outgoing_reg_location().as_register();
    return get_register_argument_addr_at(fr, r);
  }
}


jint* GC_Support::get_stack_argument_addr_at(const frame fr, int offset_from_sp_in_words) {
  return fr.sp_addr_at(offset_from_sp_in_words);
}


jint* GC_Support::get_register_argument_addr_at(const frame fr, Register r) {
  frame next_younger = fr.after_save();
  return next_younger.register_addr(r.after_save());
}
