/*
 * @(#)frame_sparc.inline.hpp	1.40 00/02/21 16:41:01
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// Inline functions for SPARC frames:

// Constructors

inline frame::frame() { _pc = NULL; _sp = NULL; _younger_sp = NULL; _interpreter_sp_adjustment_offset = 0;}


// Accessors:

inline bool frame::equal(frame other) const {
  return sp() == other.sp()
      && fp() == other.fp()
      && pc() == other.pc();
}

inline int frame::frame_size() const { return sender_sp() - sp(); }

inline jint**   frame::       link_addr() const  { return (jint**)&fp()[ FP.sp_offset_in_saved_window()]; }

inline void frame::set_link(jint* addr) { assert(link()==addr, "frame nesting is controlled by hardware"); }



// return address:

inline address  frame::sender_pc()        const    { return *I7_addr() + (pc_return_offset * wordSize); }

inline void     frame::set_sender_pc(address addr) { *I7_addr() = addr - (pc_return_offset * wordSize); }

inline address* frame::I7_addr() const  { return (address*) &sp()[ I7.sp_offset_in_saved_window()]; }  
inline address* frame::O7_addr() const  { return after_save().I7_addr(); }

inline address* frame::I0_addr() const  { return (address*) &sp()[ I0.sp_offset_in_saved_window()]; }  
inline address* frame::O0_addr() const  { return after_save().I0_addr(); }

inline jint*    frame::sender_sp() const  { return fp(); }

// Used only in frame::oopmapreg_to_location
inline int frame::pd_oop_map_offset_adjustment() const {
  COMPILER1_ONLY(int adjust = memory_parameter_word_sp_offset;)
  NOT_COMPILER1 (int adjust = 0;)
  return adjust + _interpreter_sp_adjustment_offset;
}

#ifndef CORE
inline oop *frame::pd_compiled_argument_to_location(VMReg::Name reg, RegisterMap reg_map, int arg_size) const {
  COMPILER1_ONLY(return (oop *) (arg_size - 1 - reg + sp() + memory_parameter_word_sp_offset);   )
  COMPILER2_ONLY(return oopmapreg_to_location(reg, &reg_map); )
}
#endif

inline jint** frame::interpreter_frame_locals_addr() const { 
  return (jint**) sp_addr_at( Llocals.sp_offset_in_saved_window());
}

inline jint* frame::interpreter_frame_bcx_addr() const {
  // %%%%% reinterpreting Lbcp as a bcx
  return (jint*) sp_addr_at( Lbcp.sp_offset_in_saved_window());
}


inline jint& frame::interpreter_frame_local_at(int index) const {
    return  (*interpreter_frame_locals_addr()) [-index];
}


inline jint* frame::interpreter_frame_expression_stack() const {
  return ((jint*)interpreter_frame_monitors()) - 1;
}

inline jint& frame::interpreter_frame_expression_stack_at(jint offset) const {
  return interpreter_frame_expression_stack()[-offset]; 
}

inline jint frame::interpreter_frame_expression_stack_direction() { return -1; }

// top of expression stack
inline jint* frame::interpreter_frame_tos_address() const {
  return *interpreter_frame_esp_addr() + 1;
} 
inline void frame::interpreter_frame_set_tos_address( jint* x ) { 
  *interpreter_frame_esp_addr() = x - 1; 
}

inline jint frame::interpreter_frame_expression_stack_size() const { 
  return interpreter_frame_expression_stack() - interpreter_frame_tos_address() + 1;
}

inline jint& frame::interpreter_frame_tos_at(jint offset) const { 
  return interpreter_frame_tos_address()[offset];
}

// monitor elements

// in keeping with Intel side: end is lower in memory than begin;
// and beginning element is oldest element
// Also begin is one past last monitor.

inline BasicObjectLock* frame::interpreter_frame_monitor_begin()       const  { 
  int rounded_vm_local_words = round_to(frame::interpreter_frame_vm_local_words, WordsPerLong);
  return (BasicObjectLock *)fp_addr_at(-rounded_vm_local_words);
}

inline BasicObjectLock* frame::interpreter_frame_monitor_end()         const  { 
  return interpreter_frame_monitors();
}


inline void frame::interpreter_frame_set_monitor_end(BasicObjectLock* value) {
  interpreter_frame_set_monitors(value);
}


inline int frame::interpreter_frame_monitor_size() {
  return round_to(BasicObjectLock::size(), WordsPerLong);
}

inline methodOop* frame::interpreter_frame_method_addr() const { 
  return (methodOop*)sp_addr_at( Lmethod.sp_offset_in_saved_window());
}



// Constant pool cache

inline constantPoolCacheOop* frame::interpreter_frame_cache_addr() const {
  return (constantPoolCacheOop*)sp_addr_at( LcpoolCache.sp_offset_in_saved_window());
}


inline JavaCallWrapper* frame::entry_frame_call_wrapper() const {
  // note: adjust this code if the link argument in StubGenerator::call_stub() changes!
  const Argument link = Argument(0, false);
  return (JavaCallWrapper*)sp()[link.as_in().as_register().sp_offset_in_saved_window()];
}


inline int frame::local_offset_for_compiler(int local_index, int nof_args, int max_nof_locals, int max_nof_monitors) {
   // always allocate non-argument locals 0..5 as if they were arguments:
  int allocated_above_frame = nof_args;
  if (allocated_above_frame < callee_register_argument_save_area_words)
    allocated_above_frame = callee_register_argument_save_area_words;
  if (allocated_above_frame > max_nof_locals)
    allocated_above_frame = max_nof_locals;

  // Note: monitors (BasicLock blocks) are never allocated in argument slots
  //assert(local_index >= 0 && local_index < max_nof_locals, "bad local index");
  if (local_index < allocated_above_frame)
    return local_index + callee_register_argument_save_area_sp_offset;
  else
    return local_index - (max_nof_locals + max_nof_monitors*2) + compiler_frame_vm_locals_fp_offset;
}

inline int frame::monitor_offset_for_compiler(int local_index, int nof_args, int max_nof_locals, int max_nof_monitors) {
  assert(local_index >= max_nof_locals && ((local_index - max_nof_locals) & 1) && (local_index - max_nof_locals) < max_nof_monitors*2, "bad monitor index");

  // The compiler uses the __higher__ of two indexes allocated to the monitor.
  // Increasing local indexes are mapped to increasing memory locations,
  // so the start of the BasicLock is associated with the __lower__ index.

  int offset = (local_index-1) - (max_nof_locals + max_nof_monitors*2) + compiler_frame_vm_locals_fp_offset;

  // We allocate monitors aligned zero mod 8:
  assert((offset & 1) == 0, "monitor must be an an even address.");
  // This works because all monitors are allocated after
  // all locals, and because the highest address corresponding to any
  // monitor index is always even.
  assert((compiler_frame_vm_locals_fp_offset & 1) == 0, "end of monitors must be even address");

  return offset;
}

inline int frame::min_local_offset_for_compiler(int nof_args, int max_nof_locals, int max_nof_monitors) {
   // always allocate non-argument locals 0..5 as if they were arguments:
  int allocated_above_frame = nof_args;
  if (allocated_above_frame < callee_register_argument_save_area_words)
    allocated_above_frame = callee_register_argument_save_area_words;
  if (allocated_above_frame > max_nof_locals)
    allocated_above_frame = max_nof_locals;

  int allocated_in_frame = (max_nof_locals + max_nof_monitors*2) - allocated_above_frame;

  return compiler_frame_vm_locals_fp_offset - allocated_in_frame;
}

// On SPARC, the %lN and %iN registers are non-volatile.
inline bool frame::volatile_across_calls(Register reg) {
  // This predicate is (presently) applied only to temporary registers,
  // and so it need not recognize non-volatile globals.
  return reg.is_out() || reg.is_global();
}

inline oop* frame::saved_oop_result_addr() const { return (oop*)O0_addr(); }
inline oop  frame::saved_oop_result() const      { return *saved_oop_result_addr(); }
inline void frame::set_saved_oop_result(oop obj) { *saved_oop_result_addr() = obj; }

inline oop* frame::saved_receiver_addr() const { return (oop*)O0_addr(); }
inline oop frame::saved_receiver() const       { return *saved_receiver_addr(); }
inline void frame::set_saved_receiver(oop obj) { *saved_receiver_addr() = obj; }

//Reconciliation History
// 1.12 97/11/24 10:34:54 frame_i486.inline.hpp
// 1.15 98/01/26 13:03:53 frame_i486.inline.hpp
// 1.17 98/01/26 13:03:53 frame_i486.inline.hpp
// 1.19 98/03/05 05:29:45 frame_i486.inline.hpp
// 1.19 98/03/06 16:35:13 frame_i486.inline.hpp
// 1.21 98/04/30 16:37:50 frame_i486.inline.hpp
// 1.22 98/05/07 14:20:18 frame_i486.inline.hpp
// 1.22 98/05/07 14:20:18 frame_i486.inline.hpp
// 1.23 98/06/24 16:38:47 frame_i486.inline.hpp
// 1.27 98/09/29 15:07:12 frame_i486.inline.hpp
// 1.28 98/11/09 20:04:23 frame_i486.inline.hpp
// 1.32 99/07/12 11:29:47 frame_i486.inline.hpp
// 1.33 99/08/16 13:44:02 frame_i486.inline.hpp
//End

