/*
 * @(#)interp_masm_sparc.cpp	1.111 01/02/09 14:43:23
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_interp_masm_sparc.cpp.incl"
#include <v9/sys/psr_compat.h>

// Implementation of InterpreterMacroAssembler

// This file specializes the assember with interpreter-specific macros

const Address InterpreterMacroAssembler::l_tmp( FP, 0,  frame::interpreter_frame_l_scratch_fp_offset    * wordSize);
const Address InterpreterMacroAssembler::d_tmp( FP, 0,  frame::interpreter_frame_d_scratch_fp_offset    * wordSize);
const Address InterpreterMacroAssembler::d_tmp2(FP, 0, (frame::interpreter_frame_d_scratch_fp_offset+1) * wordSize);


void InterpreterMacroAssembler::compute_extra_locals_size_in_bytes(Register args_size, Register locals_size, Register delta) {
  // Note: this algorithm is also used by C1's OSR entry sequence.
  // Any changes should also be applied to CodeEmitter::emit_osr_entry().
  assert_different_registers(args_size, locals_size);
  subcc(locals_size, args_size, delta);// extra space for non-arguments locals in words
  if (VM_Version::v9_instructions_work()) {
    movcc(Assembler::negative, false, Assembler::icc, G0, delta);  // set to 0 if adjustment < 0
  } else {
    Label skip_move;
    // br(test (e.g., negative), annull_delay_slot, target_address)
    br(Assembler::negative, true, Assembler::pt, skip_move);
    delayed()->mov(G0, delta);
    bind(skip_move);
  }
  round_to(delta, WordsPerLong);       // make multiple of 2 (SP must be 2-word aligned)
  sll(delta, LogBytesPerWord, delta);  // extra space for locals in bytes
}


// Dispatch code executed in the prolog of a bytecode which does not do it's
// own dispatch. The dispatch address is computed and placed in IdispatchAddress
void InterpreterMacroAssembler::dispatch_prolog(TosState state, int bcp_incr) {
  assert_not_delayed();
  ldub( Lbcp, bcp_incr, Lbyte_code);               // load next bytecode
  // dispatch table to use
  Address tbl(Gscratch, (address)Interpreter::dispatch_table(state));

  sethi(tbl);
  sll(Lbyte_code, LogBytesPerWord, Lbyte_code);    // multiply by wordSize
  add(tbl, tbl.base(), 0);
  ld( Gscratch, Lbyte_code, IdispatchAddress);     // get entry addr
}


// Dispatch code executed in the epilog of a bytecode which does not do it's
// own dispatch. The dispatch address in IdispatchAddress is used for the
// dispatch.
void InterpreterMacroAssembler::dispatch_epilog(TosState state, int bcp_incr) {
  assert_not_delayed();
  verify_FPU(1, state);
  verify_oop(Otos_i, state);
  jmp( IdispatchAddress, 0 );
  if (bcp_incr != 0)  delayed()->inc(Lbcp, bcp_incr);
  else                delayed()->nop();
}


void InterpreterMacroAssembler::dispatch_next(TosState state, int bcp_incr) {
  // %%%% consider branching to a single shared dispatch stub (for each bcp_incr)
  assert_not_delayed();
  ldub( Lbcp, bcp_incr, Lbyte_code);               // load next bytecode
  dispatch_Lbyte_code(state, Interpreter::dispatch_table(state), bcp_incr);
}


void InterpreterMacroAssembler::dispatch_via(TosState state, address* table) {
  // load current bytecode       
  assert_not_delayed();
  ldub( Lbcp, 0, Lbyte_code);               // load next bytecode
  dispatch_base(state, table);   
}                                
              

void InterpreterMacroAssembler::call_VM_leaf_base(
  Register java_thread,
  address  entry_point,
  int      number_of_arguments
) {
  if (!java_thread.isValid())
    java_thread = Lthread_cache;
  // super call
  MacroAssembler::call_VM_leaf_base(java_thread, entry_point, number_of_arguments);
}
 
 
void InterpreterMacroAssembler::call_VM_base(
  Register        oop_result,
  Register        java_thread,
  Register        last_java_sp,
  address         entry_point,
  int             number_of_arguments,
  bool		  check_exception
) {
  if (!java_thread.isValid())
    java_thread = Lthread_cache;
  // See class ThreadInVMfromInterpreter, which assumes that the interpreter
  // takes responsibility for setting its own thread-state on call-out.
  // However, ThreadInVMfromInterpreter resets the state to "in_Java".

  //save_bcp();                                  // save bcp
  MacroAssembler::call_VM_base(oop_result, java_thread, last_java_sp, entry_point, number_of_arguments, Assembler::_check_for_exceptions_flag);
  //restore_bcp();                               // restore bcp
  //restore_locals();                            // restore locals pointer
}


void InterpreterMacroAssembler::super_call_VM_leaf(Register thread_cache, address entry_point, Register arg_1) {
  mov(arg_1, O0);
  MacroAssembler::call_VM_leaf_base(thread_cache, entry_point, 1);
}


void InterpreterMacroAssembler::super_call_VM(Register thread_cache, Register oop_result, Register last_java_sp, address entry_point, Register arg_1, Register arg_2, bool check_exception) {
  // O0 is reserved for the thread
  mov(arg_1, O1);
  mov(arg_2, O2);
  MacroAssembler::call_VM_base(oop_result, thread_cache, last_java_sp, entry_point, 2, check_exception);
}


void InterpreterMacroAssembler::dispatch_base(TosState state, address* table) {
  assert_not_delayed();
  dispatch_Lbyte_code(state, table);
}


void InterpreterMacroAssembler::dispatch_normal(TosState state) {
  dispatch_base(state, Interpreter::normal_table(state));
}                                


void InterpreterMacroAssembler::dispatch_only(TosState state) {
  dispatch_base(state, Interpreter::dispatch_table(state));
}

 
// common code to dispatch and dispatch_only
// dispatch value in Lbyte_code and increment Lbcp

void InterpreterMacroAssembler::dispatch_Lbyte_code(TosState state, address* table, int bcp_incr) {
  verify_FPU(1, state);
  // %%%%% maybe implement +VerifyActivationFrameSize here
  //verify_thread(); //too slow; we will just verify on method entry & exit
  verify_oop(Otos_i, state);
  // dispatch table to use
  Address tbl(Gscratch, (address)table);

  sll(Lbyte_code, LogBytesPerWord, Lbyte_code);    // multiply by wordSize
  load_address(tbl);                               // compute addr of table
  ld( Gscratch, Lbyte_code, Gscratch);             // get entry addr
  jmp( Gscratch, 0 );
  if (bcp_incr != 0)  delayed()->inc(Lbcp, bcp_incr);
  else                delayed()->nop();
}


// helpers for expression stack

void InterpreterMacroAssembler::pop_i(Register r) { 
  assert_not_delayed();
  ld( Lesp, wordSize, r);  
  inc(Lesp, wordSize); 
  debug_only(verify_esp(Lesp));
}


void InterpreterMacroAssembler::pop_l(Register r) {
  assert_not_delayed();
  load_unaligned_long(Lesp, 1 * wordSize, r);
  inc(Lesp, 2 * wordSize); 
  debug_only(verify_esp(Lesp));
}


void InterpreterMacroAssembler::pop_f(FloatRegister f) { 
  assert_not_delayed();
  ldf(FloatRegister::S, Lesp, 1 * wordSize, f); 
  inc(Lesp, wordSize); 
  debug_only(verify_esp(Lesp));
}


void InterpreterMacroAssembler::pop_d(FloatRegister f) { 
  assert_not_delayed();
  load_unaligned_double(Lesp, 1 * wordSize, f);
  inc(Lesp, 2 * wordSize); 
  debug_only(verify_esp(Lesp));
}


// (Note use register first, then decrement so dec can be done during store stall)

void InterpreterMacroAssembler::push_i(Register r) {
  assert_not_delayed();
  debug_only(verify_esp(Lesp));
  st(  r,    Lesp, 0);
  dec( Lesp, wordSize); 
}


// remember: our convention for longs in SPARC is:
// O0 (Otos_l1) has high-order part in first word,
// O1 (Otos_l2) has low-order part in second word

void InterpreterMacroAssembler::push_l(Register r) {
  assert_not_delayed();
  debug_only(verify_esp(Lesp));
  // Longs are in stored in memory-correct order, even if unaligned.
  store_unaligned_long(r, Lesp, -1 *wordSize);
  dec(Lesp, 2 * wordSize); 
}


void InterpreterMacroAssembler::push_f(FloatRegister f) { 
  assert_not_delayed();
  debug_only(verify_esp(Lesp));
  stf(FloatRegister::S, f, Lesp, 0); 
  dec(                     Lesp, wordSize); 
}


void InterpreterMacroAssembler::push_d(FloatRegister f)   { 
  assert_not_delayed();
  debug_only(verify_esp(Lesp));
  // Doubles are in stored in memory-correct order, even if unaligned.
  store_unaligned_double(f, Lesp, -1 *wordSize);
  dec(Lesp, 2 * wordSize);
}


void InterpreterMacroAssembler::push(TosState state) {
  verify_oop(Otos_i, state);
  switch (state) {
    case atos:
    case itos: push_i();              break;
    case ltos: push_l();              break;
    case ftos: push_f();              break;
    case dtos: push_d();              break;
    case vtos: /* nothing to do */    break;
    default  : ShouldNotReachHere();
  }
}


void InterpreterMacroAssembler::pop(TosState state) {
  switch (state) {
    case atos:
    case itos: pop_i();              break;
    case ltos: pop_l();              break;
    case ftos: pop_f();              break;
    case dtos: pop_d();              break;
    case vtos: /* nothing to do */   break;
    default  : ShouldNotReachHere();
  }
  verify_oop(Otos_i, state);
}


void InterpreterMacroAssembler::empty_expression_stack() {
  // Reset Lesp.
  sub( Lmonitors, wordSize, Lesp );

  // Reset SP by subtracting more space from Lesp.
  Label done;

  const Address max_stack         (Lmethod, 0, methodOopDesc::max_stack_offset_in_bytes());
  const Address access_flags      (Lmethod, 0, methodOopDesc::access_flags_offset_in_bytes());

  verify_oop(Lmethod);

  assert( Gscratch2    != Gframe_size,
          "Only you can prevent register aliasing!");

  // A native does not need to do this, since its callee does not change SP.
  ld(access_flags, Gframe_size);
  btst(JVM_ACC_NATIVE, Gframe_size);
  br(Assembler::notZero, false, Assembler::pt, done);
  delayed()->nop();

  //
  // Compute max expression stack+register save area
  //
  lduh( max_stack, Gframe_size );
  add( Gframe_size, frame::memory_parameter_word_sp_offset, Gframe_size );

  //
  // now set up a stack frame with the size computed above
  //
  //round_to( Gframe_size, WordsPerLong ); // -- moved down to the "and" below
  sll( Gframe_size, LogBytesPerWord, Gframe_size );
  sub( Lesp, Gframe_size, Gframe_size );
  and3( Gframe_size, -BytesPerLong, Gframe_size ); // align SP (downwards) to an 8-byte boundary
  debug_only(verify_sp(Gframe_size, Gscratch2));
  mov(Gframe_size, SP);

  bind(done);
}


#ifdef ASSERT
void InterpreterMacroAssembler::verify_sp(Register Rsp, Register Rtemp) {
  Label Bad, OK;

  // Saved SP must be aligned.
  btst(LongAlignmentMask, Rsp);
  br(Assembler::notZero, false, Assembler::pn, Bad);
  delayed()->nop();

  // Saved SP, plus register window size, must not be above FP.
  add(Rsp, frame::register_save_words * wordSize, Rtemp);
  cmp(Rtemp, FP);
  br(Assembler::greaterUnsigned, false, Assembler::pn, Bad);
  delayed()->nop();

  // Saved SP must not be ridiculously below current SP.
  int maxstack = max(JavaThread::stack_size_at_create(), 4*K*K);
  set(maxstack, Rtemp);
  sub(SP, Rtemp, Rtemp);
  cmp(Rsp, Rtemp);
  br(Assembler::lessUnsigned, false, Assembler::pn, Bad);
  delayed()->nop();

  br(Assembler::always, false, Assembler::pn, OK);
  delayed()->nop();

  bind(Bad);
  stop("on return to interpreted call, restored SP is corrupted");

  bind(OK);
}


void InterpreterMacroAssembler::verify_esp(Register Resp) {
  // about to read or write Resp[0]
  // make sure it is not in the monitors or the register save area
  Label OK1, OK2;
  cmp(Resp, Lmonitors);
  br(Assembler::lessUnsigned, true, Assembler::pt, OK1);
  delayed()->sub(Resp, frame::memory_parameter_word_sp_offset * wordSize, Resp);
  stop("too many pops:  Lesp points into monitor area");
  bind(OK1);
  cmp(Resp, SP);
  br(Assembler::greaterEqualUnsigned, false, Assembler::pt, OK2);
  delayed()->add(Resp, frame::memory_parameter_word_sp_offset * wordSize, Resp);
  stop("too many pushes:  Lesp points into register window");
  bind(OK2);
}
#endif // ASSERT


void InterpreterMacroAssembler::if_cmp(Condition cc) {
  Label not_taken;

  assert_not_delayed();
  br(cc, false, Assembler::pn, not_taken);
  delayed()->nop();

  // get (wide) offset
  get_2_byte_integer_at_bcp(1, Gscratch, Otos_i, Signed, set_CC);
  add( Otos_i, Lbcp, Lbcp );     // add to bc addr
  // Note:  condition codes are live here, up to the br(greater) below.
  
#ifndef CORE
  bool increment_invocation_counter_for_backward_branches = UseRecompilation && UseLoopCounter;
  if (increment_invocation_counter_for_backward_branches) {
    Label L;
    br(greater, false, Assembler::pn, L);
    delayed()->nop();
    increment_invocation_counter(Gscratch, Gscratch2);
    if(UseOnStackReplacement) {
      test_invocation_counter_for_osr(false, Gscratch, Gscratch2, vtos);
    }
    bind(L);
  }

  // if counting branches, increment branch taken
  if( UseLoopCounter )
    TemplateTable::count_branches(true);
#endif // CORE

  dispatch_next(vtos);

  bind(not_taken);

#ifndef CORE
  // if counting branches, increment branch not taken
  if( UseLoopCounter )
    TemplateTable::count_branches(false);
#endif // CORE
}


void InterpreterMacroAssembler::get_2_byte_integer_at_bcp(
                                  int         bcp_offset, 
                                  Register    Rtmp, 
				  Register    Rdst,
                                  signedOrNot is_signed,
                        	  setCCOrNot  should_set_CC ) {
  assert(Rtmp != Rdst, "need separate temp register");
  assert_not_delayed();
  switch (is_signed) {
   default: ShouldNotReachHere();

   case   Signed:  ldsb( Lbcp, bcp_offset, Rdst  );  break; // high byte
   case Unsigned:  ldub( Lbcp, bcp_offset, Rdst  );  break; // high byte
  }
  ldub( Lbcp, bcp_offset + 1, Rtmp ); // low byte
  sll( Rdst, BitsPerByte, Rdst);
  switch (should_set_CC ) {
   default: ShouldNotReachHere();

   case      set_CC:  orcc( Rdst, Rtmp, Rdst ); break;
   case dont_set_CC:  or3(  Rdst, Rtmp, Rdst ); break;
  }
} 


void InterpreterMacroAssembler::get_4_byte_integer_at_bcp(
                                  int        bcp_offset,
                                  Register   Rtmp, 
				  Register   Rdst,
				  setCCOrNot should_set_CC ) {
  assert(Rtmp != Rdst, "need separate temp register");
  assert_not_delayed();
  add( Lbcp, bcp_offset, Rtmp);
  andcc( Rtmp, 3, G0);
  Label aligned;
  switch (should_set_CC ) {
   default: ShouldNotReachHere();

   case      set_CC: break;
   case dont_set_CC: break;
  }

  br(Assembler::zero, true, Assembler::pn, aligned);
  delayed()->ld(Rtmp, 0, Rdst);

  ldub(Lbcp, bcp_offset + 3, Rdst);
  ldub(Lbcp, bcp_offset + 2, Rtmp);  sll(Rtmp,  8, Rtmp);  or3(Rtmp, Rdst, Rdst);
  ldub(Lbcp, bcp_offset + 1, Rtmp);  sll(Rtmp, 16, Rtmp);  or3(Rtmp, Rdst, Rdst);
  ldub(Lbcp, bcp_offset + 0, Rtmp);  sll(Rtmp, 24, Rtmp);
  or3(Rtmp, Rdst, Rdst );

  bind(aligned);
  if (should_set_CC == set_CC) tst(Rdst);
} 


void InterpreterMacroAssembler::get_cache_and_index_at_bcp(Register cache, Register tmp, int bcp_offset) {
  assert(bcp_offset > 0, "bcp is still pointing to start of bytecode");
  assert(cache != tmp, "registers must be different");
  assert_not_delayed();
  get_2_byte_integer_at_bcp(bcp_offset, cache, tmp, Unsigned);
	      // convert from field index to ConstantPoolCacheEntry index
	      // and from word index to byte offset
  sll(tmp, exact_log2(ConstantPoolCacheEntry::size() * BytesPerWord), tmp);
  add(LcpoolCache, tmp, cache);
}


void InterpreterMacroAssembler::get_cache_entry_pointer_at_bcp(Register cache, Register tmp, int bcp_offset) {
  assert(bcp_offset > 0, "bcp is still pointing to start of bytecode");
  assert(cache != tmp, "registers must be different");
  assert_not_delayed();
  get_2_byte_integer_at_bcp(bcp_offset, cache, tmp, Unsigned);
              // convert from field index to ConstantPoolCacheEntry index
              // and from word index to byte offset
  sll(tmp, exact_log2(ConstantPoolCacheEntry::size() * BytesPerWord), tmp);
              // skip past the header
  add(tmp, constantPoolCacheOopDesc::base_offset_in_bytes(), tmp);
              // construct pointer to cache entry
  add(LcpoolCache, tmp, cache);
}


// Separate these two to allow for delay slot in middle
// These are used to do a test and full jump to exception-throwing code.

// %%%%% Could possibly reoptimize this by testing to see if could use
// a single conditional branch (i.e. if span is small enough.
// If you go that route, than get rid of the split and give up
// on the delay-slot hack. 

void InterpreterMacroAssembler::throw_if_not_1( Condition ok_condition, 
			         	        Label&    ok ) {

  assert_not_delayed();
  br( ok_condition, true, pt, ok);
  // DELAY SLOT
}


void InterpreterMacroAssembler::throw_if_not_2( address  throw_entry_point,
					        Register Rscratch,
					        Label&   ok ) {
  assert(throw_entry_point != NULL, "entry point must be generated by now");
  jump_to(Address( Rscratch, throw_entry_point ));
  delayed()->nop();

  bind(ok);
}


// And if you cannot use the delay slot, here is a shorthand:

void InterpreterMacroAssembler::throw_if_not( Condition ok_condition,
					      address   throw_entry_point,
					      Register  Rscratch ) {
  Label ok;
  if (ok_condition != never) {
    throw_if_not_1( ok_condition, ok);
    delayed()->nop();
  }
  throw_if_not_2( throw_entry_point, Rscratch, ok);
}



void InterpreterMacroAssembler::field_offset_at( int n, Register t, Register d, 
						 Register base) {
  assert(t != d, "need distinct temp reg");
  
  verify_oop(base);
  get_2_byte_integer_at_bcp( n, t, d, Unsigned );
  Label ok;
  tst(base);
  throw_if_not_1(Assembler::notZero, ok);
  delayed()->sll( d, LogBytesPerWord, d);
  throw_if_not_2(Interpreter::_throw_NullPointerException_entry, t, ok);
}  


int InterpreterMacroAssembler::field_offset_at( Register object, address bcp, int offset ) {
  assert( bcp != NULL, "should only be used for snippet templates");
  int field_offset = Bytes::get_native_u2(bcp + offset) * wordSize;
  null_check(object, field_offset);
  return field_offset;
}
 
 
void InterpreterMacroAssembler::fast_iagetfield( address bcp ) {
  if ( bcp == NULL ) {
    // interpreter template
    field_offset_at(1, Gscratch2, Gscratch, Otos_i);
    ld( Otos_i, Gscratch, Otos_i); 
  }
  else {
    // snippet template
    untested("fast_iagetfield snippet case");
    verify_oop(Otos_i);
    load_word_with_offset(Otos_i, field_offset_at( Otos_i, bcp, 1 ), Otos_i, Gscratch);
  }
}


void InterpreterMacroAssembler::fast_iaputfield( address bcp, bool do_store_check ) {
  pop_i(O1); // get object
  if ( bcp == NULL) {
    // interpreter template
    field_offset_at(1, Gscratch2, Gscratch, O1);
    st( Otos_i, O1, Gscratch);
  }
  else {
    // snippet template
    untested("fast_iputfield snippet case");
    verify_oop(Otos_i);
    store_word_with_offset(Otos_i, O1, field_offset_at( O1, bcp, 1), O2);
  }
  if ( do_store_check ) {
     store_check(Gscratch, O1);
  }
}


void InterpreterMacroAssembler::fast_iaaccess(int n, address bcp) {
  assert(n < Argument::n_register_parameters, "would need more code");
  assert_not_delayed();
  Register lreg = Register('i', n);
  if (bcp == NULL) {
    // interpreter template
    field_offset_at(2, Gscratch2, Gscratch, lreg);
    ld( lreg, Gscratch, Otos_i); 
  }
  else {
    // snippet template
    verify_oop(lreg);
    load_word_with_offset(lreg, field_offset_at( lreg, bcp, 2 ), Otos_i, Gscratch);
  }
}


// Check that index is in range for array, then shift index by index_shift, and put arrayOop + shifted_index into res
// Note: res is still shy of address by array offset into object.

void InterpreterMacroAssembler::index_check_without_pop(Register array, Register index, int index_shift, Register tmp, Register res) {
  assert_not_delayed();

  verify_oop(array);

  // check array
  Label ptr_ok;
  tst(array);
  throw_if_not_1( notZero, ptr_ok );
  delayed()->ld( array, arrayOopDesc::length_offset_in_bytes(), tmp ); // check index
  throw_if_not_2( Interpreter::_throw_NullPointerException_entry, Gscratch, ptr_ok);

  Label index_ok;
  cmp(index, tmp);
  throw_if_not_1( lessUnsigned, index_ok );
  if (index_shift > 0)  delayed()->sll(index, index_shift, index);
  else                  delayed()->add(array, index, res); // addr - const offset in index
  throw_if_not_2( Interpreter::_throw_ArrayIndexOutOfBoundsException_entry, Gscratch, index_ok);

  // add offset if didn't do it in delay slot
  if (index_shift > 0)   add(array, index, res); // addr - const offset in index
}


void InterpreterMacroAssembler::index_check(Register array, Register index, int index_shift, Register tmp, Register res) {
  assert_not_delayed();

  // pop array
  pop_i(array);

  // check array
  index_check_without_pop(array, index, index_shift, tmp, res);
}


void InterpreterMacroAssembler::check_newarray(Register Rsize) {
  assert_not_delayed();

  Label ok;
  tst(Otos_i);
  throw_if_not_1( greaterEqual, ok);
  delayed()->mov(Otos_i, Rsize);
  Unimplemented();
  //throw_if_not_2( Interpreter::_throw_NegativeArraySizeException_entry, Gscratch, ok );
}


void InterpreterMacroAssembler::get_constant_pool(Register Rdst) {
  ld(Lmethod, methodOopDesc::constants_offset_in_bytes(), Rdst);
}


void InterpreterMacroAssembler::get_constant_pool_cache(Register Rdst) {
  get_constant_pool(Rdst);
  ld(Rdst, constantPoolOopDesc::cache_offset_in_bytes(), Rdst);
}

#if 0
void InterpreterMacroAssembler::is_a(Label& ok) {
  assert_not_delayed();
  verify_oop(Otos_i);
  tst(Otos_i);
  br(Assembler::zero, false, Assembler::pn, ok);

  delayed()->mov(Otos_i, O3);
  get_constant_pool(O1);
  get_2_byte_integer_at_bcp(1, Gscratch2, O2, Unsigned );

  TemplateTable::call_VM(noreg, (address)InterpreterRuntime::is_a, O1, O2, O3);
  // (use TemplateTable version, because it has extra asserts)
}
#endif


void InterpreterMacroAssembler::unlock_if_synchronized_method(TosState state, bool throw_monitor_exception) {
  Label unlocked, unlock;
  // check if synchronized method
  const Address access_flags(Lmethod, 0, methodOopDesc::access_flags_offset_in_bytes());
  verify_oop(Otos_i, state);
  push(state); // save tos
  ld(access_flags, Gscratch);
  btst(JVM_ACC_SYNCHRONIZED, Gscratch);
  br( zero, false, pt, unlocked);
  delayed()->nop();

  // BasicObjectLock will be first in list, since this is a synchronized method. However, need
  // to check that the object has not been unlocked by an explicit monitorexit bytecode.  

  //Intel: if (throw_monitor_exception) ... else ...
  // Entry already unlocked, need to throw exception
  //...

  // pass top-most monitor elem
  add( top_most_monitor(), O1 );

  ld(O1, BasicObjectLock::obj_offset_in_bytes(), Gscratch);
  tst(Gscratch);
  br(Assembler::notZero, false, pt, unlock);
  delayed()->nop();

  if (throw_monitor_exception) {
    // Entry already unlocked need to throw an exception
    MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
    should_not_reach_here();
  } else {
    // Monitor already unlocked during a stack unroll. Install an
    // illegal_monitor_state_exception and continue with stack unrolling
    MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::new_illegal_monitor_state_exception));
    ba(false, unlocked);
    delayed()->nop();
  }

  bind(unlock);

  unlock_object(O1);

  bind(unlocked);

  // I0, I1: Might contain return value

  // Check that all monitors are unlocked
  { Label loop, exception, entry, restart;

    Register Rmptr   = O0;
    Register Rtemp   = O1;
    Register Rlimit  = Lmonitors;
    const jint delta = frame::interpreter_frame_monitor_size() * wordSize;
    assert( (delta & LongAlignmentMask) == 0, 
	    "sizeof BasicObjectLock must be even number of doublewords");

    #ifdef ASSERT
    add(top_most_monitor(), Rmptr, delta);
    { Label L;
      // ensure that Rmptr starts out above (or at) Rlimit
      cmp(Rmptr, Rlimit);
      br(Assembler::greaterEqualUnsigned, false, pn, L);
      delayed()->nop();
      stop("monitor stack has negative size");
      bind(L);
    }
    #endif
    bind(restart);
    ba(false, entry);
    delayed()->
    add(top_most_monitor(), Rmptr, delta);      // points to current entry, starting with bottom-most entry

    // Entry is still locked, need to throw exception
    bind(exception);
    if (throw_monitor_exception) {
      MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
      should_not_reach_here();
    } else {
      // Stack unrolling. Unlock object and install illegal_monitor_exception
      // Unlock does not block, so don't have to worry about the frame
      unlock_object(Rmptr);
      MacroAssembler::call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::new_illegal_monitor_state_exception));
      ba(false, restart);
      delayed()->nop();
    }

    bind(loop);
    cmp(Rtemp, (int)NULL);                      // check if current entry is used
    br(Assembler::notEqual, false, pn, exception);
    delayed()->
    dec(Rmptr, delta);                          // otherwise advance to next entry
    #ifdef ASSERT
    { Label L;
      // ensure that Rmptr has not somehow stepped below Rlimit
      cmp(Rmptr, Rlimit);
      br(Assembler::greaterEqualUnsigned, false, pn, L);
      delayed()->nop();
      stop("ran off the end of the monitor stack");
      bind(L);
    }
    #endif
    bind(entry);
    cmp(Rmptr, Rlimit);                         // check if bottom reached
    br(Assembler::notEqual, true, pn, loop);    // if not at bottom then check this entry
    delayed()->
    ld(Rmptr, BasicObjectLock::obj_offset_in_bytes() - delta, Rtemp);
  }
  pop(state);
  verify_oop(Otos_i, state);
}


// Lock object
//
// Argument - lock_reg points to the BasicObjectLock to be used for locking,
//            it must be initialized with the object to lock
void InterpreterMacroAssembler::lock_object(Register lock_reg, Register Object) {
  if (UseHeavyMonitors) {
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorenter), lock_reg);
  }
  else {
    Register obj_reg = Object;
    Register mark_reg = Gscratch2;
    Register temp_reg = Gscratch3;
    Address  lock_addr = Address(lock_reg, 0, BasicObjectLock::lock_offset_in_bytes());
    Address  mark_addr = Address(obj_reg, 0, oopDesc::mark_offset_in_bytes());
    Label    done;

    // load markOop from object into mark_reg
    ld(mark_addr, mark_reg);

    // Attempt a spec lock if possible
    if (UseSpecLock) {
      Label   non_speclock;
      Address thrid_addr = Address(obj_reg, 0, SpecLock::speclock_field_offset());

      // First test if the object can be speclocked
      // untested("Speclocking in the interpreter");
      and3(mark_reg, markOopDesc::speclock_pattern, temp_reg);
      cmp(temp_reg, markOopDesc::speclock_pattern);
      br(Assembler::notEqual, false, Assembler::pn, non_speclock);
      delayed()->ld(thrid_addr, temp_reg);

      // now see if the speclock is owned by the executing thread
      cmp(Gthread, temp_reg);
      br(Assembler::equal, false, Assembler::pt, done);
      delayed()->nop();

      // Fixup what appears to be a speclock as a different thread is
      // trying to acquire a lock on the object
      assert(lock_reg != I2, "Incorrect register assignment");
      mov(lock_reg, I2);
      call_VM(obj_reg, CAST_FROM_FN_PTR(address, InterpreterRuntime::fixup_speclock), obj_reg);
      mov(I2, lock_reg);

      bind(non_speclock);
    }
   
    // get the address of basicLock on stack that will be stored in the object
    // we need a temporary register here as we do not want to clobber lock_reg
    // (cas clobbers the destination register)
    mov(lock_reg, temp_reg);
    // set mark reg to be (markOop of object | UNLOCK_VALUE)
    or3(mark_reg, markOopDesc::unlocked_value, mark_reg);
    // initialize the box  (Must happen before we update the object mark!)
    st(mark_reg, lock_addr, BasicLock::displaced_header_offset_in_bytes());
    // compare and exchange object_addr, markOop | 1, stack address of basicLock
    assert(mark_addr.disp() == 0, "cas must take a zero displacement");
    cas_under_lock(mark_addr.base(), mark_reg, temp_reg, (address)StubRoutines::sparc::atomic_memory_operation_lock_addr());
    // if the compare and exchange succeeded we are done (we saw an unlocked object)
    // store the original markOop of the object into the displaced_header field of
    // the basicLock on the stack
    cmp(mark_reg, temp_reg);
    br(Assembler::equal, true, Assembler::pt, done);
    delayed()->nop();

    // We did not see an unlocked object so try the fast recursive case

    // Check if owner is self by comparing the value in the markOop of object
    // with the stack pointer
    sub(temp_reg, SP, temp_reg);
    assert(os::vm_page_size() > 0xfff, "page size too small - change the constant");
    cmp(temp_reg, 0xfff);
    // if condition is true we are done and hece we can store 0 in the displaced
    // header indicating it is a recursive lock and be done
    br(Assembler::lessUnsigned, true, Assembler::pt, done);
    delayed()->st(G0, lock_addr, BasicLock::displaced_header_offset_in_bytes());

    // none of the above fast optimizations worked so we have to get into the
    // slow case of monitor enter
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorenter), lock_reg);

    bind(done);   
  }
}

// Unlocks an object. Used in monitorexit bytecode and remove_activation.
//
// Argument - lock_reg points to the BasicObjectLock for lock
// Throw IllegalMonitorException if object is not locked by current thread
void InterpreterMacroAssembler::unlock_object(Register lock_reg) {
  if (UseHeavyMonitors) {
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorexit), lock_reg);
  } else {
    Register obj_reg = Gscratch;
    Register mark_reg = Gscratch2;
    Register displaced_header_reg = Gscratch3;
    Address  lock_addr = Address(lock_reg, 0, BasicObjectLock::lock_offset_in_bytes());
    Address  lockobj_addr = Address(lock_reg, 0, BasicObjectLock::obj_offset_in_bytes());
    Address  mark_addr = Address(obj_reg, 0, oopDesc::mark_offset_in_bytes());
    Label    done;

    if (UseSpecLock) {
      // untested("SpecLocking unlock in the interpreter");
      ld(lockobj_addr, obj_reg);
      ld(mark_addr, mark_reg);
      // and(mark_reg, Gscratch3, I2);
      and3(mark_reg, markOopDesc::speclock_pattern, Gscratch3);
      cmp(Gscratch3, markOopDesc::speclock_pattern);
      br(Assembler::equal, true, Assembler::pt, done);
      delayed()->st(G0, lockobj_addr);
    }

    // Test first if we are in the fast recursive case
    ld(lock_addr, displaced_header_reg, BasicLock::displaced_header_offset_in_bytes());
    tst(displaced_header_reg);
    br(Assembler::zero, true, Assembler::pn, done);
    delayed()->st(G0, lockobj_addr);  // free entry

    // See if it is still a light weight lock, if so we just unlock
    // the object and we are done
    if (!UseSpecLock) {
      // load the markOop from the object
      ld(lockobj_addr, obj_reg);
    }

    // we have the displaced header in displaced_header_reg
    // we expect to see the stack address of the basicLock in case the
    // lock is still a light weight lock (lock_reg)
    assert(mark_addr.disp() == 0, "cas must take a zero displacement");
    cas_under_lock(mark_addr.base(), lock_reg, displaced_header_reg, (address)StubRoutines::sparc::atomic_memory_operation_lock_addr());
    cmp(lock_reg, displaced_header_reg);
    br(Assembler::equal, true, Assembler::pn, done);
    delayed()->st(G0, lockobj_addr);  // free entry

    // The lock has been converted into a heavy lock and hence
    // we need to get into the slow case

    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::monitorexit), lock_reg);

    bind(done);
  }
}


// add a InterpMonitorElem to stack (see frame_sparc.hpp)

void InterpreterMacroAssembler::add_monitor_to_stack( bool stack_is_empty,
						      Register Rtemp,
						      Register Rtemp2 ) {

  Register Rlimit = Lmonitors;
  const jint delta = frame::interpreter_frame_monitor_size() * wordSize;
  assert( (delta & LongAlignmentMask) == 0, 
	  "sizeof BasicObjectLock must be even number of doublewords");

  sub( SP,        delta, SP);
  sub( Lesp,      delta, Lesp);
  sub( Lmonitors, delta, Lmonitors);

  if (!stack_is_empty) {

    // must copy stack contents down

    Label start_copying, next;

    // untested("monitor stack expansion");
    compute_stack_base(Rtemp);
    ba( false, start_copying );
    delayed()->cmp( Rtemp, Rlimit); // done? duplicated below

    // note: must copy from low memory upwards
    // On entry to loop,
    // Rtemp points to new base of stack, Lesp points to new end of stack (1 past TOS)
    // Loop mutates Rtemp

    bind( next);

    st(Rtemp2, Rtemp, 0);
    inc(Rtemp, wordSize);
    cmp(Rtemp, Rlimit); // are we done? (duplicated above)

    bind( start_copying );

    br( notEqual, true, pn, next );
    delayed()->ld( Rtemp, delta, Rtemp2 );

    // done copying stack
  }
}


void InterpreterMacroAssembler::access_local_int( Register index, Register dst ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  ld(index, 0, dst);
  // Note:  index must hold the effective address--the iinc template uses it
}


void InterpreterMacroAssembler::access_local_long( Register index, Register dst ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  load_unaligned_long(index, -1*wordSize, dst);
}


void InterpreterMacroAssembler::access_local_float( Register index, FloatRegister dst ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  ldf(FloatRegister::S, index, 0, dst);
}


void InterpreterMacroAssembler::access_local_double( Register index, FloatRegister dst ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  load_unaligned_double(index, -1*wordSize, dst);
}


#ifdef ASSERT
void InterpreterMacroAssembler::check_for_regarea_stomp(Register Rindex, int offset, Register Rlimit, Register Rscratch, Register Rscratch1) {
  Label L;

  assert(Rindex != Rscratch, "Registers cannot be same");
  assert(Rindex != Rscratch1, "Registers cannot be same");
  assert(Rlimit != Rscratch, "Registers cannot be same");
  assert(Rlimit != Rscratch1, "Registers cannot be same");
  assert(Rscratch1 != Rscratch, "Registers cannot be same");

  // untested("reg area corruption");
  add(Rindex, offset, Rscratch);
  add(Rlimit, 64, Rscratch1);
  cmp(Rscratch, Rscratch1);
  br(Assembler::greaterEqualUnsigned, false, pn, L);
  delayed()->nop();
  stop("regsave area is being clobbered");
  bind(L);
}
#endif // ASSERT

void InterpreterMacroAssembler::store_local_int( Register index, Register src ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  debug_only(check_for_regarea_stomp(index, 0, FP, Gscratch3, Gscratch2);)
  st(src, index, 0);
}


void InterpreterMacroAssembler::store_local_long( Register index, Register src ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  #ifdef ASSERT
  check_for_regarea_stomp(index, -1 * wordSize, FP, Gscratch3, Gscratch2);
  #endif
  store_unaligned_long(src, index, -1*wordSize);
}


void InterpreterMacroAssembler::store_local_float( Register index, FloatRegister src ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  #ifdef ASSERT
  check_for_regarea_stomp(index, 0, FP, Gscratch3, Gscratch2);
  #endif
  stf(FloatRegister::S, src, index, 0);
}


void InterpreterMacroAssembler::store_local_double( Register index, FloatRegister src ) {
  assert_not_delayed();
  sll(index, LogBytesPerWord, index);
  sub(Llocals, index, index);
  #ifdef ASSERT
  check_for_regarea_stomp(index, -1 * wordSize, FP, Gscratch3, Gscratch2);
  #endif
  store_unaligned_double(src, index, -1*wordSize);
}


Address InterpreterMacroAssembler::top_most_monitor() {
  const jint delta = frame::interpreter_frame_monitor_size() * wordSize;
  int rounded_vm_local_words = ::round_to(frame::interpreter_frame_vm_local_words, WordsPerLong);
  return Address(FP, 0, (-rounded_vm_local_words * wordSize)  -  delta);
}


void InterpreterMacroAssembler::compute_stack_base( Register Rdest ) {
  add( Lesp,      wordSize,                                    Rdest );
}


// iinc to statically known place

void InterpreterMacroAssembler::static_iinc( int which_local, jint increment, Register Rtmp, Register Rtmp2 ) {
  if (increment == 0)   return;

  Address loc( Llocals, 0, -which_local * wordSize );
  Register r;

  untested("static iinc memory");
  r = Rtmp;
  ld(  loc, r);

  if (is_simm13(increment) )          add( r, increment, r );
  else {  set( increment, Rtmp2 );    add( r, Rtmp2,     r ); }

  st(  r, loc);
}


#ifndef CORE
void InterpreterMacroAssembler::increment_invocation_counter( Register Rtmp, Register Rtmp2 ) {
  assert(UseRecompilation, "incrementing must be useful");
  Address counter(Lmethod, 0, methodOopDesc::invocation_counter_offset_in_bytes()
			    + InvocationCounter::counter_offset_in_bytes());
  int delta = InvocationCounter::count_increment;
  ld( counter, Rtmp );
  if ( is_simm13( delta ) )  {
    addcc( Rtmp, delta, Rtmp );
  }
  else {
    set( delta, Rtmp2);
    addcc( Rtmp, Rtmp2, Rtmp );
  }
  st( Rtmp, counter );

  // Note that this macro must leave the value of the invocation counter in Rtmp!
}


void InterpreterMacroAssembler::test_invocation_counter_for_osr( bool load_invocation_count, 
                                                                 Register invocation_count,
                                                                 Register Rtmp,
                                                                 TosState state ) {
  Label did_not_overflow;
  Label overflow_with_error;

  assert(UseOnStackReplacement,"Must UseOnStackReplacement to test_invocation_counter_for_osr");
    
  if(load_invocation_count) {
    Address counter(Lmethod, 0, methodOopDesc::invocation_counter_offset_in_bytes()
                    + InvocationCounter::counter_offset_in_bytes());
    ld( counter, invocation_count );
  }

  Address limit(Rtmp, address(&InvocationCounter::InterpreterBackwardBranchLimit));
  load_contents(limit, Rtmp);
  cmp(invocation_count, Rtmp);
  br(Assembler::lessUnsigned, false, Assembler::pt, did_not_overflow);
  delayed()->nop();

  // We have to save the TosState because we're overwriting the O registers
  // when we call_VM
  push(state);

  // overflow in loop, pass true
  set((int)true, invocation_count);
  set(6,Rtmp);
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::invocation_counter_overflow), invocation_count, Rtmp);

  // Was an OSR adapter generated?
#ifdef COMPILER1
  // O0 = NULL
  // O1 = osr_code
  tst(O1);
#else
  // O0 = osr_adapter_frame_address
  // O1 = osr_code
  tst(O0);
#endif // COMPILER1
  br(Assembler::zero, false, Assembler::pn, overflow_with_error);
  delayed()->nop();

  // Has the nmethod been invalidated already?	
  ld(O1,nmethod::entry_bci_offset(),O2);
  cmp(O2, InvalidOSREntryBci);
  br(Assembler::equal, false, Assembler::pn, overflow_with_error);
  delayed()->nop();

#ifdef COMPILER2
  // reserve space for incoming arguments. This is needed since a C2I adapter
  // could be introduced as part of deoptimization. The C2I adapter will think
  // it owns the incomming arguments area. If we do not reserve space for this
  // explicitly, then the C2I adapter might potentially override stuff in the
  // frame above the OSR adapter frame and frame traversal will fail
  ld(O1, nmethod::method_offset_in_bytes(), O2); // get methodOop
  lduh(Address(O2, 0, methodOopDesc::size_of_parameters_offset_in_bytes()), O2);
  // get #parameters
  round_to(O2, WordsPerLong);                    // align
  sll(O2, LogBytesPerWord, O2);

    // The nmethod seems to be okay, we need to make room for
    // the frame structure, a pointer to the frame struct, and
    // one argument slot, which the OSR compiled frame will stomp.

    // struct is 4 words, plus 1 word for the argument, and 1
    // word to keep things 8 byte aligned.
  add(O2, 24, O2);
  sub(SP, O2, SP);
    
  // Build the frame struct. 
  //
  // Frame slot #4
  // Frame slot #3
  // Frame slot #2
  // Frame slot #1
  // alignment word
  // argument word
  // 16 words preserve area.

  // We include the alignment word here for simplicity
  int argument_words = 2; 

  st(G0, SP, (frame::register_save_words + argument_words + 3) * wordSize);
  st(G0, SP, (frame::register_save_words + argument_words + 2) * wordSize);
  st(G0, SP, (frame::register_save_words + argument_words + 1) * wordSize);
  st(SP, SP, (frame::register_save_words + argument_words + 0) * wordSize);

  // The argument is passed using the interpreter calling convention,
  // So it is stored on stack, instead of o0. 
  add(SP, (frame::register_save_words + argument_words + 0) * wordSize, O2);
  st(O2, SP, frame::register_save_words * wordSize);
    
  // We want to return to the OSRAdaptor, the addr of which is in o0.
  // Due to the way returns are handled in sparc land, we need to set
  // o7 to (address - 8 bytes)
  sub(O0, 2 * wordSize, O7);


  // We have to force all register windows to the stack, including our own
  save_frame(0);
  flush_windows();
  restore();
#endif // COMPILER2

  // Jump to the osr code.
  ld(O1, nmethod::osr_entry_point_offset(),O2);
  jmp(O2,G0);
  delayed()->nop();

  bind(overflow_with_error);
  // If we return to the interpreter, we must restore the TosState
  pop(state);

  bind(did_not_overflow);
}
#endif // not CORE


#ifdef COMPILER2
// The call and type profiles are C2-specific.

void InterpreterMacroAssembler::setup_call_in_profile( Label& skip, Register Rentry, Register Rtmp ) {
  assert(UseTypeProfile || UseCallProfile, "call profiling must be useful");

  Address counter(Lmethod, 0, methodOopDesc::invocation_counter_offset_in_bytes()
			    + InvocationCounter::counter_offset_in_bytes());
  Address table_addr(Rentry, CallProfile::table_base());

  int sub_threshold = CallProfile::unprofiled_invocations();
  assert(sub_threshold >= 0 && sub_threshold <= CompileThreshold, "");
  int scaled_threshold = sub_threshold * InvocationCounter::count_increment;

  Label &done = skip;   // branch here when done

  ld(counter, Rentry);
  if (Assembler::is_simm13(scaled_threshold)) {
    cmp(Rentry, scaled_threshold);
  } else {
    set(scaled_threshold, Rtmp);
    cmp(Rentry, Rtmp);
  }
  br(Assembler::lessUnsigned, false, Assembler::pt, done);
  delayed()->sethi(table_addr);

  // The following is a hand-assembled version of this expression:
  //  entry_addr_at(Lbcp)->record_bcp(Rbcp)

  // Compute the address of the CallProfileEntry into Rentry
  int hash_mask = right_n_bits(exact_log2(CallProfileSize));
  if (Assembler::is_simm13(hash_mask)) {
    and3(Lbcp, hash_mask, Rtmp);
  } else {
    set(hash_mask, Rtmp);
    and3(Lbcp, Rtmp, Rtmp);
  }
  //sethi(table_addr);  // put in delay slot above
  add(table_addr, Rentry);
  sll(Rtmp, exact_log2(CallProfileEntry::size() * wordSize), Rtmp);
  add(Rentry, Rtmp, Rentry);

#ifdef ASSERT
  // These "#ifdef ASSERT" parts help monitor the contents of the profile.
  // They are optional, and need not be ported to other platforms.
  // The bcp and other_bcp fields can be left zero (or garbage) without
  // loss of product functionality.
  record_bcp_in_profile(Rentry, CallProfileEntry::key_offset_in_bytes(), Rtmp);
#endif // ASSERT
}


void InterpreterMacroAssembler::record_bcp_in_profile( Register Rentry, int offset, Register Rtmp ) {
  Address       bcp_addr(Rentry, 0, offset + BCPProfileEntry::bcp_offset_in_bytes());
#ifdef ASSERT
  Address other_bcp_addr(Rentry, 0, offset + BCPProfileEntry::other_bcp_offset_in_bytes());
  Address bcp_count_addr(Rentry, 0, offset + BCPProfileEntry::bcp_count_offset_in_bytes());
#endif // ASSERT

  Label done;
  Label normal_case;

  ld(bcp_addr, Rtmp);
  cmp(Lbcp, Rtmp);
  br(Assembler::equal, false, Assembler::pt, normal_case);
  delayed()->tst(Rtmp);

  //tst(Rtmp);  // put in delay slot above
  br(Assembler::zero, true, Assembler::pt, done);
  delayed()->st(Lbcp, bcp_addr);  // executed only if branch is taken

  // test the saved bcp:  is it already a conflict value? (sign bit flipped)
  xorcc(Rtmp, Lbcp, G0);
  br(Assembler::negative, false, Assembler::pt, done);
  delayed()->neg(Rtmp);  // hoisted into delay slot

#ifdef ASSERT
  // sign bits are the same, so the old _bcp is not yet a conflict
  st(Lbcp, other_bcp_addr);
#endif // ASSERT

  // store the new conflict value
  //neg(Rtmp);  // put in delay slot above
  st(Rtmp, bcp_addr);

#ifdef ASSERT
  ba(false, done);
  delayed()->nop();

  bind(normal_case);
  ld(bcp_count_addr, Rtmp);
  inc(Rtmp);
  st(Rtmp, bcp_count_addr);
#else
  bind(normal_case);
#endif // ASSERT

  bind(done);
}


void InterpreterMacroAssembler::record_static_call_in_profile( Register Rentry, Register Rtmp ) {
  Label done;
  setup_call_in_profile(done, Rentry, Rtmp);
  if (UseCallProfile)  bump_count_in_profile(Rentry, Rtmp);
  bind(done);
}


void InterpreterMacroAssembler::record_receiver_call_in_profile( Register Rklass, Register Rentry, Register Rtmp ) {
  Label done;
  setup_call_in_profile(done, Rentry, Rtmp);
  if (UseCallProfile)  bump_count_in_profile(Rentry, Rtmp);
  if (UseTypeProfile)  update_klass_in_profile(Rklass, Rentry, Rtmp);
  bind(done);
}


void InterpreterMacroAssembler::bump_count_in_profile( Register Rentry, Register Rtmp ) {
  assert(UseCallProfile, "call profiling must be useful");

  Address count_addr(Rentry, 0, CallProfileEntry::count_offset_in_bytes());

  ld(count_addr, Rtmp);
  inc(Rtmp);
  st(Rtmp, count_addr);
}


void InterpreterMacroAssembler::update_klass_in_profile( Register Rklass, Register Rentry, Register Rtmp ) {
  assert(UseTypeProfile, "type profiling must be useful");

  Address       klass_addr(Rentry, 0, CallProfileEntry::receiver_offset_in_bytes() + TypeProfileEntry::      klass_offset_in_bytes());
#ifdef ASSERT
  Address other_klass_addr(Rentry, 0, CallProfileEntry::receiver_offset_in_bytes() + TypeProfileEntry::other_klass_offset_in_bytes());
  Address  mono_count_addr(Rentry, 0, CallProfileEntry::receiver_offset_in_bytes() + TypeProfileEntry:: mono_count_offset_in_bytes());
#endif // ASSERT

  // load the entry and check the klass
  ld(klass_addr, Rtmp);
  cmp(Rtmp, Rklass);

  Label normal_case, done;

  br(Assembler::equal, false, Assembler::pt, normal_case);
  delayed()->tst(Rtmp);

  // the klass interferes somehow with the table entry
  // tst(Rtmp);  // put in delay slot above
  br(Assembler::zero, true, Assembler::pt, done);
  delayed()->st(Rklass, klass_addr);   // is_empty() -> is_klass(Rklass)

  btst(1, Rtmp);
  br(Assembler::notZero, false, Assembler::pt, done);
  delayed()->bset(1, Rtmp);  // is_conflict() -> is_conflict()

  // is_klass(!Rklass) -> is_conflict()
  // bset(1, Rtmp);  // put in delay slot above
  st(Rtmp, klass_addr);

#ifdef ASSERT
  // store the other conflicting value, if possible
  ld(other_klass_addr, Rtmp);
  tst(Rtmp);
  br(Assembler::notZero, true, Assembler::pt, done);
  delayed()->nop();

  ba(false, done);
  delayed()->st(Rklass, other_klass_addr);

  bind(normal_case);
  ld(mono_count_addr, Rtmp);
  inc(Rtmp);
  st(Rtmp, mono_count_addr);
#else
  bind(normal_case);
#endif // ASSERT

  bind(done);
}
#endif // COMPILER2


void InterpreterMacroAssembler::verify_oop(Register reg, TosState state) {
  if (state == atos) { MacroAssembler::verify_oop(reg); }
}


// local helper function for the verify_oop_or_return_address macro
static bool verify_return_address(methodOop m, int bci) {
#ifndef PRODUCT
  address pc = (address)m + methodOopDesc::codes_offset_in_bytes() + bci;
  // assume it is a valid return address if it is inside m and is preceded by a jsr
  if (!m->contains(pc))                                          return false;
  address jsr_pc;
  jsr_pc = pc - Bytecodes::length_for(Bytecodes::_jsr);
  if (*jsr_pc == Bytecodes::_jsr   && jsr_pc >= m->code_base())    return true;
  jsr_pc = pc - Bytecodes::length_for(Bytecodes::_jsr_w);
  if (*jsr_pc == Bytecodes::_jsr_w && jsr_pc >= m->code_base())    return true;
#endif // PRODUCT
  return false;
}


void InterpreterMacroAssembler::verify_oop_or_return_address(Register reg, Register Rtmp) {
  if (!VerifyOops)  return;
  // the VM documentation for the astore[_wide] bytecode allows
  // the TOS to be not only an oop but also a return address
  Label test;
  Label skip;
  // See if it is an address (in the current method):
  mov(reg, Rtmp);
  const int log2_bytecode_size_limit = 16;
  srl(Rtmp, log2_bytecode_size_limit, Rtmp);
  tst(Rtmp);
  br( notZero, false, pt, test );
  delayed()->nop();

  save_frame_and_mov(0, Lmethod, O0, reg, O1);
  save_thread(Lthread_cache);
  call(CAST_FROM_FN_PTR(address, verify_return_address), relocInfo::none);
  delayed()->nop();
  restore_thread(Lthread_cache);
  tst(O0);
  br( notZero, false, pt, skip );
  delayed()->restore();

  // Perform a more elaborate out-of-line call
  // Not an address; verify it:
  bind(test);
  verify_oop(reg);
  bind(skip);
}


void InterpreterMacroAssembler::verify_FPU(int stack_depth, TosState state) {
  if (state == ftos || state == dtos) MacroAssembler::verify_FPU(stack_depth);
}


// Inline assembly for:
//
// if (*jvmdi::get_method_entry_on_addr) {
//   InterpreterRuntime::post_method_entry();
// }
// if (*jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_ENTRY ) ||
//     *jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_ENTRY2)   ) {
//   SharedRuntime::jvmpi_method_entry(method, receiver);
// }

void InterpreterMacroAssembler::notify_method_entry() {
  if (jvmdi::enabled()) {
    Label L;
    Register temp_reg = O5;
    Address get_method_entry_on_addr(temp_reg, (address)jvmdi::get_method_entry_on_addr(), relocInfo::none);
    load_contents(get_method_entry_on_addr, temp_reg);
    tst(temp_reg);
    br(zero, false, pt, L);
    delayed()->nop();
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_method_entry));
    bind(L);
  }
  { Label E;
    Label S;
    Register temp_reg = O5;
    Address event0(temp_reg, (address)jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_ENTRY ), relocInfo::none);
    load_contents(event0, temp_reg);
    cmp(temp_reg, (int)JVMPI_EVENT_ENABLED);
    br(equal, false, pn, S);
    delayed()->nop();
    Address event1(temp_reg, (address)jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_ENTRY2), relocInfo::none);
    load_contents(event1, temp_reg);
    cmp(temp_reg, (int)JVMPI_EVENT_ENABLED);
    br(notEqual, false, pt, E);
    delayed()->nop();
    bind(S);
    // notify method entry
    { Label L;
      const Address access_flags(Lmethod, 0, methodOopDesc::access_flags_offset_in_bytes());
      ld(access_flags, temp_reg);
      and3(temp_reg, JVM_ACC_STATIC, temp_reg);
      cmp(temp_reg, JVM_ACC_STATIC); // check if method is static
      br(equal, true, pn, L);        // if static we're done
      delayed()->mov(G0, temp_reg);  // receiver = NULL for a static method
      mov(I0, temp_reg);             // otherwise get receiver
      bind(L);
      call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_entry), Lmethod, temp_reg);
    }
    bind(E);
  }
}


// Inline assembly for:
//
// if (*jvmdi::get_method_exit_on_addr) {
//   // save result
//   InterpreterRuntime::post_method_exit();
//   // restore result
// }
// if (*jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_EXIT)) {
//   // save result
//   SharedRuntime::jvmpi_method_exit();
//   // restore result
// }
// 
// Native methods have their result stored in d_tmp and l_tmp
// Java methods have their result stored in the expression stack

void InterpreterMacroAssembler::notify_method_exit(bool is_native_method, TosState state) {
  if (jvmdi::enabled()) {
    Label L;
    Register temp_reg = O5;
    Address get_method_exit_on_addr(temp_reg, (address)jvmdi::get_method_exit_on_addr(), relocInfo::none);

    load_contents(get_method_exit_on_addr, temp_reg);
    tst(temp_reg);
    br(zero, false, pt, L);
    delayed()->nop();

    if (is_native_method) {
      stf(FloatRegister::D, F0, d_tmp);
      std(                  O0, l_tmp);
    } else {
      push(state);
    }

    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_method_exit));

    if (is_native_method) {
      ldf(FloatRegister::D, d_tmp, F0);
      ldd(                  l_tmp, O0);
    } else {
      pop(state);
    }

    bind(L);

  }
  { Label E;
    Register temp_reg = O5;
    Address event0(temp_reg, (address)jvmpi::event_flags_array_at_addr(JVMPI_EVENT_METHOD_EXIT ), relocInfo::none);
    load_contents(event0, temp_reg);
    cmp(temp_reg, (int)JVMPI_EVENT_ENABLED);
    br(notEqual, false, pn, E);
    delayed()->nop();
    if (is_native_method) {
      stf(FloatRegister::D, F0, d_tmp);
      std(                  O0, l_tmp);
    } else {
      push(state);
    }
    call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::jvmpi_method_exit), Lmethod);
    if (is_native_method) {
      ldf(FloatRegister::D, d_tmp, F0);
      ldd(                  l_tmp, O0);
    } else {
      pop(state);
    }
    bind(E);
  }
}


//Reconciliation History
// 1.19 97/11/16 18:25:24 interp_masm_i486.cpp
// 1.20 97/11/24 14:07:35 interp_masm_i486.cpp
// 1.24 97/12/11 19:23:37 interp_masm_i486.cpp
// 1.26 98/01/30 10:04:18 interp_masm_i486.cpp
// 1.28 98/02/10 13:33:52 interp_masm_i486.cpp
// 1.29 98/02/23 15:09:33 interp_masm_i486.cpp
// 1.31 98/03/04 13:43:08 interp_masm_i486.cpp
// 1.34 98/03/30 17:20:31 interp_masm_i486.cpp
// 1.39 98/04/22 18:20:45 interp_masm_i486.cpp
// 1.40 98/04/24 12:52:40 interp_masm_i486.cpp
// 1.41 98/04/30 12:10:40 interp_masm_i486.cpp
// 1.42 98/05/11 13:42:33 interp_masm_i486.cpp
// 1.44 98/06/22 17:40:10 interp_masm_i486.cpp
// 1.47 98/07/01 15:42:15 interp_masm_i486.cpp
// 1.49 98/07/24 13:32:32 interp_masm_i486.cpp
// 1.52 98/10/02 15:21:57 interp_masm_i486.cpp
// 1.55 98/10/05 02:31:58 interp_masm_i486.cpp
// 1.57 98/10/21 17:27:40 interp_masm_i486.cpp
// 1.59 98/10/26 10:29:27 interp_masm_i486.cpp
// 1.61 98/11/04 13:20:33 interp_masm_i486.cpp
// 1.63 98/11/25 16:10:35 interp_masm_i486.cpp
// 1.64 98/12/01 16:12:22 interp_masm_i486.cpp
// 1.65 99/01/05 09:25:37 interp_masm_i486.cpp
// 1.66 99/01/13 16:18:05 interp_masm_i486.cpp
// 1.68 99/01/25 14:07:46 interp_masm_i486.cpp
// 1.72 99/03/10 18:06:14 interp_masm_i486.cpp
// 1.75 99/04/19 13:50:32 interp_masm_i486.cpp
// 1.78 99/06/28 09:58:09 interp_masm_i486.cpp
// 1.74 99/04/01 16:52:54 interp_masm_i486.cpp
// 1.75 99/04/05 16:57:56 interp_masm_i486.cpp
// 1.75 99/04/12 12:58:21 interp_masm_i486.cpp
// 1.79 99/07/06 16:02:50 interp_masm_i486.cpp
//End
