/*
 * @(#)interp_masm_sparc.hpp	1.59 00/06/28 08:28:44
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// This file specializes the assember with interpreter-specific macros

const      Register Otos_i  = O0; // tos for ints, etc
const      Register Otos_l  = O0; // for longs
const      Register Otos_l1 = O0; // for 1st part of longs
const      Register Otos_l2 = O1; // for 2nd part of longs
const FloatRegister Ftos_f  = F0; // for floats
const FloatRegister Ftos_d  = F0; // for doubles
const FloatRegister Ftos_d1 = F0; // for 1st part of double
const FloatRegister Ftos_d2 = F1; // for 2nd part of double


class InterpreterMacroAssembler: public MacroAssembler {
 protected:
  // Interpreter specific version of call_VM_base
  virtual void call_VM_leaf_base(
    Register java_thread,
    address  entry_point,
    int      number_of_arguments
  );

  virtual void call_VM_base(
    Register        oop_result,
    Register        java_thread,
    Register        last_java_sp,
    address         entry_point,
    int             number_of_arguments,
    bool	    check_exception=true
  );

  // base routine for all dispatches
  void dispatch_base(TosState state, address* table);

  // Helper routine for call profiling.
  // skip is the label to skip to, if no profiling is to be done.
  // The address of the CallProfileEntry is left in Rentry.
  void setup_call_in_profile( Label &skip, Register Rentry, Register Rtmp );
  void bump_count_in_profile( Register Rentry, Register Rtmp );
  void update_klass_in_profile( Register Rklass, Register Rentry, Register Rtmp );
  void record_bcp_in_profile( Register Rentry, int offset, Register Rtmp );

 public:
  InterpreterMacroAssembler(CodeBuffer* c)
    : MacroAssembler(c) {}

  static const Address l_tmp ;
  static const Address d_tmp ;
  static const Address d_tmp2;

  // helper routine for frame allocation/deallocation
  // compute the delta by which the caller's SP has to
  // be adjusted to accomodate for the non-argument
  // locals
  void compute_extra_locals_size_in_bytes(Register args_size, Register locals_size, Register delta);

  // dispatch routines
  void dispatch_prolog(TosState state, int step = 0);
  void dispatch_epilog(TosState state, int step = 0);
  void dispatch_only(TosState state);
  void dispatch_normal(TosState state);
  void dispatch_next(TosState state, int step = 0);
  void dispatch_via (TosState state, address* table);


 protected:
  void dispatch_Lbyte_code(TosState state, address* table, int bcp_incr = 0);

 public:
  // Super call_VM calls - correspond to MacroAssembler::call_VM(_leaf) calls
  void super_call_VM_leaf(Register thread_cache, address entry_point, Register arg_1);
  void super_call_VM(Register thread_cache, Register oop_result, Register last_java_sp, address entry_point, Register arg_1, Register arg_2, bool check_exception = true);

  // helpers for tossing exceptions

  void throw_if_not(   Condition ok_condition,
		       address   throw_entry_point,
		       Register  Rscratch );

  void throw_if_not_1( Condition ok_condition,
                       Label&    ok );

  void throw_if_not_2( address   throw_entry_point,
		       Register  Rscratch,
		       Label&    ok );

  // helpers for expression stack

  void pop_i(     Register r = Otos_i);
  void pop_l(     Register r = Otos_l1);
  void pop_f(FloatRegister f = Ftos_f);
  void pop_d(FloatRegister f = Ftos_d1);

  void push_i(     Register r = Otos_i);
  void push_l(     Register r = Otos_l1);
  void push_f(FloatRegister f = Ftos_f);
  void push_d(FloatRegister f = Ftos_d1);


  void pop (TosState state);           // transition vtos -> state
  void push(TosState state);           // transition state -> vtos
  void empty_expression_stack();       // resets both Lesp and SP

#ifdef ASSERT
  void verify_sp(Register Rsp, Register Rtemp);
  void verify_esp(Register Resp);      // verify that Lesp points to a word in the temp stack
#endif // ASSERT

 public:
  void if_cmp(Condition cc);

  // Load values from bytecode stream:

  enum signedOrNot { Signed, Unsigned };
  enum setCCOrNot  { set_CC,  dont_set_CC };

  void get_2_byte_integer_at_bcp( int         bcp_offset, 
                                  Register    Rtmp, 
				  Register    Rdst,
                                  signedOrNot is_signed,
                        	  setCCOrNot  should_set_CC = dont_set_CC );

  void get_4_byte_integer_at_bcp( int        bcp_offset,
                                  Register   Rtmp, 
				  Register   Rdst,
				  setCCOrNot should_set_CC = dont_set_CC );

  void get_cache_and_index_at_bcp(Register cache, Register tmp, int bcp_offset);
  void get_cache_entry_pointer_at_bcp(Register cache, Register tmp, int bcp_offset);


  // common code

  void field_offset_at(int n, Register tmp, Register dest, Register base);
  int  field_offset_at(Register object, address bcp, int offset);
  void fast_iaaccess(int n, address bcp);
  void fast_iagetfield(address bcp);
  void fast_iaputfield(address bcp, bool do_store_check );
  
  void index_check(Register array, Register index, int index_shift, Register tmp, Register res);
  void index_check_without_pop(Register array, Register index, int index_shift, Register tmp, Register res);

  void check_newarray(Register Rsize);
  void get_constant_pool(Register Rdst);
  void get_constant_pool_cache(Register Rdst);
  void is_a(Label& L);


  // --------------------------------------------------

  // A family of functions for ld/st doubles, handing misalignment:
  // Here, just supply the scratch area and super on.
  
  void load_double(Register r1, Register r2, FloatRegister d)  { MacroAssembler::load_double( r1, r2,     InterpreterMacroAssembler::d_tmp, d); }
  void load_double(Register r1, int  offset, FloatRegister d)  { MacroAssembler::load_double( r1, offset, InterpreterMacroAssembler::d_tmp, d); }
  void store_double(FloatRegister d, Register r1, Register r2) { MacroAssembler::store_double(d, InterpreterMacroAssembler::d_tmp, r1, r2    ); }
  void store_double(FloatRegister d, Register r1, int offset ) { MacroAssembler::store_double(d, InterpreterMacroAssembler::d_tmp, r1, offset); }

  // helpers:
  void  load_unaligned_double(Register r1,     Register r2, FloatRegister d) { MacroAssembler:: load_unaligned_double(r1, r2,     d_tmp, d); }
  void  load_unaligned_double(Register r1,     int offset,  FloatRegister d) { MacroAssembler:: load_unaligned_double(r1, offset, d_tmp, d); }
  void store_unaligned_double(FloatRegister d, Register r1, Register r2)     { MacroAssembler::store_unaligned_double(d, d_tmp, r1, r2);     }
  void store_unaligned_double(FloatRegister d, Register r1, int offset )     { MacroAssembler::store_unaligned_double(d, d_tmp, r1, offset); }

  void  load_static_double(Address& a,      FloatRegister d) { MacroAssembler:: load_static_double(a, d_tmp, d); }
  void store_static_double(FloatRegister d, const Address& a     ) { MacroAssembler::store_static_double(d, a, d_tmp); }


  void unlock_if_synchronized_method(TosState state, bool throw_monitor_exception = true);

  void add_monitor_to_stack( bool stack_is_empty,
		             Register Rtemp,
			     Register Rtemp2 );

  void access_local_int( Register index, Register dst );
  void access_local_long( Register index, Register dst );
  void access_local_float( Register index, FloatRegister dst );
  void access_local_double( Register index, FloatRegister dst );
#ifdef ASSERT
  void check_for_regarea_stomp( Register Rindex, int offset, Register Rlimit, Register Rscratch, Register Rscratch1);
#endif // ASSERT
  void store_local_int( Register index, Register src );
  void store_local_long( Register index, Register src );
  void store_local_float( Register index, FloatRegister src );
  void store_local_double( Register index, FloatRegister src );


  Address first_local_in_stack();
  Address top_most_monitor();
  void compute_stack_base( Register Rdest );

  enum LoadOrStore { load, store };
  void static_iload_or_store( int which_local, LoadOrStore direction, Register Rtmp );
  void static_aload_or_store( int which_local, LoadOrStore direction, Register Rtmp );
  void static_dload_or_store( int which_local, LoadOrStore direction );

  void static_iinc(           int which_local, jint increment, Register Rtmp, Register Rtmp2 );

  void increment_invocation_counter( Register Rtmp, Register Rtmp2 );
  void test_invocation_counter_for_osr( bool load_invocation_count, Register invocation_count, Register Rtmp, TosState state );

  void record_static_call_in_profile( Register Rentry, Register Rtmp );
  void record_receiver_call_in_profile( Register Rklass, Register Rentry, Register Rtmp );

  // Object locking
  void lock_object  (Register lock_reg, Register obj_reg);
  void unlock_object(Register lock_reg);
  // Debugging
  void verify_oop(Register reg, TosState state = atos);    // only if +VerifyOops && state == atos
  void verify_oop_or_return_address(Register reg, Register rtmp); // for astore
  void verify_FPU(int stack_depth, TosState state = ftos); // only if +VerifyFPU  && (state == ftos || state == dtos)

  // support for jvmdi/jvmpi
  void notify_method_entry();
  void notify_method_exit(bool save_result, TosState state);

};

//Reconciliation History
// 1.16 97/11/16 18:25:23 interp_masm_i486.hpp
// 1.18 97/12/11 17:10:49 interp_masm_i486.hpp
// 1.21 98/02/10 13:33:51 interp_masm_i486.hpp
// 1.22 98/02/23 15:09:35 interp_masm_i486.hpp
// 1.24 98/03/04 13:43:11 interp_masm_i486.hpp
// 1.25 98/03/18 11:21:21 interp_masm_i486.hpp
// 1.27 98/03/26 16:49:51 interp_masm_i486.hpp
// 1.30 98/04/13 13:37:58 interp_masm_i486.hpp
// 1.31 98/05/11 13:42:30 interp_masm_i486.hpp
// 1.33 98/06/19 17:46:59 interp_masm_i486.hpp
// 1.34 98/07/01 15:42:16 interp_masm_i486.hpp
// 1.36 98/07/13 14:03:06 interp_masm_i486.hpp
// 1.37 98/10/02 13:39:53 interp_masm_i486.hpp
// 1.38 98/11/25 16:05:18 interp_masm_i486.hpp
// 1.39 99/01/25 13:50:17 interp_masm_i486.hpp
// 1.43 99/03/10 18:06:14 interp_masm_i486.hpp
// 1.44 99/04/02 16:24:07 interp_masm_i486.hpp
// 1.47 99/06/28 09:58:57 interp_masm_i486.hpp
// 1.44 99/04/01 16:52:59 interp_masm_i486.hpp
// 1.45 99/04/05 16:58:01 interp_masm_i486.hpp
// 1.48 99/07/06 16:02:53 interp_masm_i486.hpp
//End
