/*
 * @(#)interpreter_sparc.cpp	1.135 01/02/09 14:43:25
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_interpreter_sparc.cpp.incl"


// Generation of Interpreter
//
// The InterpreterGenerator generates the interpreter into Interpreter::_code.


#define __ _masm->


//----------------------------------------------------------------------------------------------------


address AbstractInterpreterGenerator::generate_exception_handler(const char* name, const char *message) {
  address entry = __ pc();
  // expression stack must be empty before entering the VM if an exception happened
  __ empty_expression_stack();
  // load exception object
  __ set((int)name, Gscratch);
  __ set((int)message, Gscratch2);
  __ call_VM(Oexception, CAST_FROM_FN_PTR(address, InterpreterRuntime::create_exception), Gscratch, Gscratch2);
  // throw exception
  assert(Interpreter::throw_exception_entry() != NULL, "generate it first");
  __ jump_to (Address(Gscratch, Interpreter::throw_exception_entry()));
  __ delayed()->nop();
  return entry;
}


address AbstractInterpreterGenerator::generate_StackOverflowError_handler() {
  address entry = __ pc();
  // expression stack must be empty before entering the VM if an exception happened
  __ empty_expression_stack();
  // create exception
  __ call_VM(Oexception, CAST_FROM_FN_PTR(address, InterpreterRuntime::create_StackOverflowError));
  // throw exception
  __ jump_to(Address(Gscratch, Interpreter::throw_exception_entry()));
  __ delayed()->nop();
  return entry;
}


address AbstractInterpreterGenerator::generate_return_entry_for(TosState state, int step) {
  address entry = __ pc();

  // (At this point, i486 under UseC2CallingConventions adjusts sp to
  // free the space for the outgoing arguments.  It refers to the C.P. cache
  // to determine how many words to pop.
  // In the case of SPARC we have saved the number of words to pop in the
  // register IsizeCalleeParms and we use it here to pop the parameters
  // (C2 Calling Conventions only)

  if (UseC2CallingConventions) {
    debug_only(__ verify_sp(IsavedSP, Gscratch));
    __ mov(IsavedSP, SP);                   // restore stack pointer
    __ add(Lesp, IsizeCalleeParms, Lesp);   // pop the arguments from the stack
    // %%%%%% could incr bcp first, then jump to common dispatching code
    __ dispatch_next(state, step);

  } else {
    // C1 Calling Conventions:
    //
    // The callee returns with the stack in the same state as
    // before the call. In particular, SP and all interpreter
    // local registers are untouched. Any result is passed
    // back in the O0/O1 or float registers. Before continuing,
    // the arguments must be popped from the java expression
    // stack; i.e., Lesp must be adjusted. (gri 2/25/2000)
    const Register cache = Gscratch;
    const Register size  = Gscratch3;
    __ get_cache_and_index_at_bcp(cache, Gscratch3, 1);
    __ ld(Address(cache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*BytesPerWord), size);
    __ and3(size, 0xFF, size);                   // argument size in words
    __ sll(size, LogBytesPerWord, size);         // argument size in bytes
    __ add(Lesp, size, Lesp);                    // pop arguments
    __ dispatch_next(state, step); 
  }

  return entry;
}

  
#ifndef CORE
address AbstractInterpreterGenerator::generate_deopt_entry_for(TosState state, int step) {
  address entry = __ pc();
  __ get_constant_pool_cache(LcpoolCache); // load LcpoolCache
  { Label L;
    Address exception_addr (Gthread, 0, Thread::pending_exception_offset_in_bytes());

    __ ld(exception_addr, Gtemp);
    __ tst(Gtemp);
    __ br(Assembler::equal, false, Assembler::pt, L);
    __ delayed()->nop();
    // The following line has been deleted from the Intel version:
    __ stop("new exception propagation across deoptimization has not been tested yet");
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_pending_exception));
    __ should_not_reach_here();
    __ bind(L);              
  }
  __ dispatch_next(state, step);
  return entry;
}
#endif // CORE


int AbstractInterpreter::BasicType_as_index(BasicType type) {
  int i = 0;
  switch (type) {
    case T_BOOLEAN: i = 0; break;
    case T_CHAR   : i = 1; break;
    case T_BYTE   : i = 2; break;
    case T_SHORT  : i = 3; break;
    case T_INT    : i = 4; break;
    case T_LONG   : i = 5; break;
    case T_VOID   : i = 6; break;
    case T_FLOAT  : i = 7; break;
    case T_DOUBLE : i = 8; break;
    case T_OBJECT : i = 9; break;
    case T_ARRAY  : i = 9; break;
    default       : ShouldNotReachHere();
  }
  assert(0 <= i && i < AbstractInterpreter::number_of_result_handlers, "index out of bounds");
  return i;
}


// A result handler converts/unboxes a native call result into
// a java interpreter/compiler result. The current frame is an
// interpreter frame. The activation frame unwind code must be
// consistent with that of TemplateTable::_return(...). In the
// case of native methods, the caller's SP was not modified.
address AbstractInterpreterGenerator::generate_result_handler_for(BasicType type) {
  address entry = __ pc();
  Register Itos_i  = Otos_i .after_save();
  Register Itos_l1 = Otos_l1.after_save();
  Register Itos_l2 = Otos_l2.after_save();
  Register rs = G0, rd = G0;   // hook for using the last restore
  switch (type) {
    case T_BOOLEAN: __ subcc(G0, O0, G0); __ addc(G0, 0, Itos_i); break; // !0 => true; 0 => false
    case T_CHAR   : __ sll(O0, 16, O0); __ srl(O0, 16, Itos_i);   break; // cannot use and3, 0xFFFF too big as immediate value!
    case T_BYTE   : __ sll(O0, 24, O0); __ sra(O0, 24, Itos_i);   break;
    case T_SHORT  : __ sll(O0, 16, O0); __ sra(O0, 16, Itos_i);   break;
    case T_INT    : rs = O0; rd = Itos_i;                         break;
    case T_LONG   : rs = O1; rd = Itos_l2; __ mov(O0, Itos_l1);   break;
    case T_VOID   : /* nothing to do */                           break;
    case T_FLOAT  : __ mov(FloatRegister::S, F0, Ftos_f);         break;
    case T_DOUBLE : __ mov(FloatRegister::D, F0, Ftos_d);         break;
    case T_OBJECT :
      { Label L;
	__ addcc(G0, O0, Itos_i);
	__ br(Assembler::notZero, true, Assembler::pt, L);    // if result is not NULL:
	__ delayed()->ld(O0, 0, Itos_i);                      // unbox it
	__ bind(L);
      }
      __ verify_oop(Itos_i);
      break;
    default       : ShouldNotReachHere();
  }
  rd = rd.after_restore();
  __ ret();                           // return from interpreter activation
  __ delayed()->restore(G0, rs, rd);  // remove interpreter frame
  NOT_PRODUCT(__ emit_long(0);)       // marker for disassembly
  return entry;
}


address AbstractInterpreterGenerator::generate_slow_signature_handler() {
  address entry = __ pc();
  Argument argv(1, true);

  __ mov(Lmethod, Gscratch);
  __ mov(Llocals, Gscratch2);
  __ save_frame(0);
  __ add(argv.address_in_frame(), O3);
  __ call_VM(noreg, FP, CAST_FROM_FN_PTR(address, InterpreterRuntime::slow_signature_handler), Gscratch, Gscratch2, O3);
  // load the register arguments (the C code packed them as varargs)
  for (Argument ldarg = argv; ldarg.is_register(); ldarg = ldarg.successor()) {
    __ ld(ldarg.address_in_frame(), ldarg.as_register());
  }
  __ ret();
  __ delayed()->
     restore(O0, 0, Lscratch);  // caller's Lscratch gets the result handler
  return entry;
}


address AbstractInterpreterGenerator::generate_safept_entry_for(TosState state, address runtime_entry) {
  address entry = __ pc();
  __ push(state);
  __ call_VM(noreg, runtime_entry);
  __ dispatch_via(vtos, Interpreter::_normal_table.table_for(vtos));
  return entry;
}


address AbstractInterpreterGenerator::generate_continuation_for(TosState state) {
  address entry = __ pc();
  __ dispatch_next(state);
  return entry;
}


//----------------------------------------------------------------------------------------------------
// Entry points & stack frame layout

address AbstractInterpreterGenerator::generate_method_entry(AbstractInterpreter::MethodKind kind) {
  // determine code generation flags
  bool inc_counter  = UseRecompilation || CountCompiledCalls;
  bool synchronized = false;
  bool zerolocals   = false;
  bool native_call  = false;
  bool math_call    = false;
  switch (kind) {
    case Interpreter::zerolocals             : zerolocals  = true;                      break;
    case Interpreter::zerolocals_synchronized: zerolocals  = true; synchronized = true; break;
    case Interpreter::native                 : native_call = true;                      break;
    case Interpreter::native_synchronized    : native_call = true; synchronized = true; break;
    case Interpreter::empty                  :                                          break;
    case Interpreter::accessor               :                                          break;
    case Interpreter::abstract               :                                          break;
    case Interpreter::java_lang_math_sin     : zerolocals = true;                       break;
    case Interpreter::java_lang_math_cos     : zerolocals = true;                       break;
    case Interpreter::java_lang_math_sqrt    : zerolocals = true;                       break;
    default                                  : ShouldNotReachHere();                    break;
  }                                    

  address entry = __ pc();

  // the following temporary registers are used during frame creation
  const Register Gtmp1 = Gscratch ;
  const Register Gtmp2 = Gscratch3;

  // make sure registers are different!
  if (!UseC2CallingConventions) {
    assert_different_registers(Gthread, Gmethod, Gargs, Gtmp1, Gtmp2);
  }

  // abstract method entry
  if (kind == Interpreter::abstract) {
    // throw exception
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_abstract_method_error));
    // the call_VM checks for exception, so we should never return here.
    __ should_not_reach_here();
    return entry;
  }

  const Address size_of_parameters(Gmethod, 0, methodOopDesc::size_of_parameters_offset_in_bytes());
  const Address compiled_code     (Gmethod, 0, methodOopDesc::compiled_code_offset_in_bytes());
  const Address codes             (Gmethod, 0, methodOopDesc::codes_offset_in_bytes());
  const Address size_of_locals    (Gmethod, 0, methodOopDesc::size_of_locals_offset_in_bytes());
  const Address max_stack         (Gmethod, 0, methodOopDesc::max_stack_offset_in_bytes());
#ifndef CORE
  const Address invocation_counter(Lmethod, 0, methodOopDesc::invocation_counter_offset_in_bytes() + InvocationCounter::counter_offset_in_bytes());
#endif
  const Address access_flags      (Lmethod, 0, methodOopDesc::access_flags_offset_in_bytes());


  // locals_limit points past last Java local ...offset is negative
  const Address locals_limit      (FP,      0, frame::interpreter_frame_vm_locals_fp_offset * BytesPerWord);


  // mathematical functions inlined by compiler
  // (interpreter must provide identical implementation
  // in order to avoid monotonicity bugs when switching
  // from interpreter to compiler in the middle of some
  // computation)
  if (math_call && InlineIntrinsics) {
    assert(kind == Interpreter::java_lang_math_sqrt, "Other intrinsics are not special");
    if (Universe::is_jdk12x_version()) {
      // Note: For JDK 1.2 StrictMath doesn't exist and Math.sin/cos/sqrt are
      //       native methods. Interpreter::method_kind(...) does a check for
      //       native methods first before checking for intrinsic methods and
      //       thus will never select this entry point. Make sure it is not
      //       called accidentally since the SharedRuntime entry points will
      //       not work for JDK 1.2.
      __ should_not_reach_here();
    } else {
      // Note: For JDK 1.3 StrictMath exists and Math.sin/cos/sqrt are
      //       java methods. Interpreter::method_kind(...) will select
      //       this entry point for the corresponding methods in JDK 1.3.
      assert(!UseC2CallingConventions, "UseC2CallingConventions should go away soon");
      __ ldf( FloatRegister::D, Gargs, 0, F0 );
      __ fsqrt( FloatRegister::D, F0, F0 );
      __ retl();
      __ delayed()->nop();
    }
    return entry;
  }


  Label Lentry;
  __ bind(Lentry);

  __ verify_oop(Gmethod);

  const Register Glocals_size = G3;
  assert_different_registers(Glocals_size, Gscratch2, Gframe_size);

  // do nothing for empty methods (do not even increment invocation counter)
  if (kind == Interpreter::empty && UseFastEmptyMethods) {
    // Code: _return
    // _return
    if (UseC2CallingConventions) {
      __ lduh( size_of_parameters, Gscratch2);
      __ mov( SP, IsavedSP );
      __ retl();
      __ delayed()->sll( Gscratch2, LogBytesPerWord, IsizeCalleeParms );
    } else {
      __ retl();
      __ delayed()->nop();
    }

    return entry;
  }

  // do fastpath for resolved accessor methods
  if (kind == Interpreter::accessor && UseFastAccessorMethods) {
    // Code: _aload_0, _(i|a)getfield, _(i|a)return or any rewrites thereof; parameter size = 1
    // Note: We can only use this code if the getfield has been resolved
    //       and if we don't have a null-pointer exception => check for
    //       these conditions first and use slow path if necessary.
    Label slow_path;

    if (UseC2CallingConventions) {
      // Check if getfield has been resolved
      // read first instruction word and extract bytecode @ 1 and index @ 2
      assert(codes.disp() % wordSize == 0, "first byte must be aligned");
      __ ld(codes, Glocals_size);
      __ srl(Glocals_size, 2*BitsPerByte, Gscratch2);
      __ and3(Gscratch2, 0xFF, Gscratch2);
      __ cmp(Gscratch2, Bytecodes::_getfield);
      __ br(Assembler::equal, false, Assembler::pt, slow_path);

      // Check if local 0 != NULL
      __ delayed()->ld(Lesp, wordSize, Gscratch2);
      __ tst(Gscratch2);
      __ br(Assembler::zero, false, Assembler::pt, slow_path);

      // Access the field
      __ delayed()->mov(Gscratch2, Otos_i);
      __ ld(Gmethod, methodOopDesc::constants_offset_in_bytes(), Gscratch2);
      __ sll(Glocals_size, 2*BitsPerByte, Glocals_size);
      __ ld(Gscratch2, constantPoolOopDesc::cache_offset_in_bytes(), Gscratch2);
      __ srl(Glocals_size, 2*BitsPerByte - exact_log2(ConstantPoolCacheEntry::size() * wordSize), Glocals_size);
      __ add(Gscratch2, Glocals_size, Gscratch2);
      __ ld(Address(Gscratch2, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + ConstantPoolCacheEntry::f2_offset_in_bytes()), Gscratch2);
      __ sll(Gscratch2, LogBytesPerWord, Gscratch2);
      __ ld(Otos_i, Gscratch2, Otos_i);

      // _ireturn/_areturn
      __ lduh( size_of_parameters, Gscratch2);
      __ mov( SP, IsavedSP );
      __ retl();
      __ delayed()->sll( Gscratch2, LogBytesPerWord, IsizeCalleeParms );

    } else {
      // read first instruction word and extract bytecode @ 1 and index @ 2
      assert(codes.disp() % BytesPerWord == 0, "first byte must be aligned");
      __ ld(Gmethod, methodOopDesc::codes_offset_in_bytes(), Gtmp2); // get first 4 bytecodes (big endian!)
      __ srl(Gtmp2, 2*BitsPerByte, Gtmp1);                           // move bytecode @ 1 far right
      __ and3(Gtmp1, 0xFF, Gtmp1);                                   // get bytecode @ 1
      __ cmp(Gtmp1, Bytecodes::_getfield);                           // check if resolved
      __ br(Assembler::equal, false, Assembler::pn, slow_path);      // if not resolved go the slow path

      // check if local 0 != NULL
      __ delayed()->ld(Gargs, G0, Otos_i);                           // get local 0
      __ ld(Gmethod, methodOopDesc::constants_offset_in_bytes(), Gtmp1); // get constant pool
      __ tst(Otos_i);                                                // check if local 0 == NULL
      __ br(Assembler::zero, false, Assembler::pn, slow_path);       // if NULL go the slow path

      // read the field
      __ delayed()->sll(Gtmp2, 2*BitsPerByte, Gtmp2);                // move index @ 2 far left
      __ ld(Gtmp1, constantPoolOopDesc::cache_offset_in_bytes(), Gtmp1);  // get constant pool cache
      __ srl(Gtmp2, 2*BitsPerByte - exact_log2(ConstantPoolCacheEntry::size() * BytesPerWord), Gtmp2);
      __ add(Gtmp1, Gtmp2, Gtmp1);                                   // get constant pool cache entry
      __ ld(Gtmp1, constantPoolCacheOopDesc::base_offset_in_bytes() + ConstantPoolCacheEntry::f2_offset_in_bytes(), Gtmp1);
      __ sll(Gtmp1, LogBytesPerWord, Gtmp1);                         // scale word offset

      // _ireturn/_areturn
      __ retl();                                                     // return from leaf routine
      __ delayed()->ld(Otos_i, Gtmp1, Otos_i);                       // with field value
    }

    __ bind(slow_path);
  }

  // make sure method is not native & not abstract
  // rethink these assertions - they can be simplified and shared (gri 2/25/2000)
#ifdef ASSERT
  __ ld(Gmethod, methodOopDesc::access_flags_offset_in_bytes(), Gtmp1);
  if (native_call) {
    Label L;
    __ btst(JVM_ACC_NATIVE, Gtmp1);
    __ br(Assembler::notZero, false, Assembler::pt, L);
    __ delayed()->nop();
    __ stop("tried to execute non-native method as native");
    __ bind(L);
  } else {
    Label L;
    __ btst(JVM_ACC_NATIVE, Gtmp1);
    __ br(Assembler::zero, false, Assembler::pt, L);
    __ delayed()->nop();
    __ stop("tried to execute native method as non-native");
    __ bind(L);
  }
  { Label L;
    __ btst(JVM_ACC_ABSTRACT, Gtmp1);
    __ br(Assembler::zero, false, Assembler::pt, L);
    __ delayed()->nop();
    __ stop("tried to execute abstract method as non-abstract");
    __ bind(L);
  }
#endif // ASSERT

#ifndef CORE
  // check if compiled code exists
  Label run_compiled_code;
  if ((!native_call || !PreferInterpreterNativeStubs) && !CompileTheWorld) {
    __ ld(compiled_code, Gscratch);
    __ tst(Gscratch);
    __ br(Assembler::notZero, false, Assembler::pt, run_compiled_code);
    __ delayed()->nop();
  }
#endif // CORE

  // C2 Calling Conventions:
  //
  // The entry code below assumes that the following registers are set
  // when coming in:
  //    Gmethod: holds the methodOop of the method to call
  //    Lesp:    points to the TOS of the callers expression stack
  //             after having pushed all the parameters
  //
  // The entry code does the following to setup an interpreter frame
  //   pop parameters from the callers stack by adjusting Lesp
  //   set O0 to Lesp
  //   compute X = (max_locals - num_parameters)
  //   bump SP up by X to accomadate the extra locals
  //   compute X = max_expression_stack
  //               + vm_local_words
  //               + 16 words of register save area
  //   save frame doing a save sp, -X, sp growing towards lower addresses
  //   set Lbcp, Lmethod, LcpoolCache
  //   set Llocals to i0
  //   set Lmonitors to FP - rounded_vm_local_words
  //   set Lesp to Lmonitors - 4
  //
  //  The frame has now been setup to do the rest of the entry code

  // Try this optimization:  Most method entries could live in a
  // "one size fits all" stack frame without all the dynamic size
  // calculations.  It might be profitable to do all this calculation
  // statically and approximately for "small enough" methods.

  //-----------------------------------------------------------------------------------------------

  // C1 Calling conventions
  //
  // Upon method entry, the following registers are setup:
  //
  // g2 Gthread: current thread
  // g5 Gmethod: method to activate
  // g4 Gargs  : pointer to last argument
  //
  //
  // Stack:
  //
  // +---------------+ <--- sp
  // |               |
  // : reg save area :
  // |               |
  // +---------------+ <--- sp + 0x40
  // |               |
  // : extra 7 slots :      note: these slots are not really needed for the interpreter (fix later)
  // |               |
  // +---------------+ <--- sp + 0x5c
  // |               |
  // :     free      :
  // |               |
  // +---------------+ <--- Gargs
  // |               |
  // :   arguments   :
  // |               |
  // +---------------+
  // |               |
  //
  //
  // The entry code sets up a new interpreter frame in 4 steps:
  //
  // 1) Increase caller's SP by for the extra local space needed:
  //    Efficient implementation of xload/xstore bytecodes requires
  //    that arguments and non-argument locals are in a contigously
  //    addressable memory block => non-argument locals must be
  //    allocated in the caller's frame.
  //    
  // 2) Create a new stack frame and register window:
  //    The new stack frame must provide space for the standard
  //    register save area, the maximum java expression stack size,
  //    the monitor slots (0 slots initially), and some frame local
  //    scratch locations.
  //
  // 3) The following interpreter activation registers must be setup:
  //    Lesp       : expression stack pointer
  //    Lbcp       : bytecode pointer
  //    Lmethod    : method
  //    Llocals    : locals pointer
  //    Lmonitors  : monitor pointer
  //    LcpoolCache: constant pool cache
  //
  // 4) Initialize the non-argument locals if necessary:
  //    Non-argument locals may need to be initialized to NULL
  //    for GC to work. If the oop-map information is accurate
  //    (in the absence of the JSR problem), no initialization
  //    is necessary.
  //
  // (gri - 2/25/2000)


  int rounded_vm_local_words = round_to( frame::interpreter_frame_vm_local_words, WordsPerLong );
  if (UseC2CallingConventions) {
    //
    // set up register OO so that Llocals of the callee can be set based on it
    //
    __ lduh( size_of_parameters, Glocals_size);
    __ mov( SP, IsavedSP );
    __ sll( Glocals_size, LogBytesPerWord, IsizeCalleeParms );
    __ add( Lesp, IsizeCalleeParms, OparamAddr );

    if (native_call) {
     __ calc_mem_param_words( Glocals_size, Gframe_size );
     __ add( Gframe_size, rounded_vm_local_words + frame::memory_parameter_word_sp_offset + frame::interpreter_frame_extra_outgoing_argument_words, Gframe_size);
    } else {
      //
      // Compute number of locals in method apart from incoming parameters
      //
      __ lduh( size_of_locals, Gscratch2 );
      __ sub( Gscratch2, Glocals_size, Glocals_size );
      __ round_to( Glocals_size, WordsPerLong );
      __ sll( Glocals_size, LogBytesPerWord, Glocals_size );
      //
      // bump SP to accomadate the extra locals
      //
      __ sub( SP, Glocals_size, SP );
      //
      // Compute max expression stack+rounded_vm_local_words+regsiter save area
      //
      __ lduh( max_stack, Gframe_size );
      __ add( Gframe_size, rounded_vm_local_words + frame::memory_parameter_word_sp_offset, Gframe_size );
    }

  } else {
    // determine arguments
    __ lduh(Gmethod, methodOopDesc::size_of_parameters_offset_in_bytes(), Gtmp1);
    __ sll(Gtmp1, LogBytesPerWord, Gtmp2);       // parameter size in bytes
    __ add(Gargs, Gtmp2, Gargs);                 // points to first local + BytesPerWord
    // determine extra space for non-argument locals & adjust caller's SP
    // Gtmp1: parameter size in words
    if (!native_call) {
      __ lduh(Gmethod, methodOopDesc::size_of_locals_offset_in_bytes(), Gtmp2);
      __ compute_extra_locals_size_in_bytes(Gtmp1, Gtmp2, Gtmp2);
      __ sub(SP, Gtmp2, SP);                     // adjust caller's SP
    }
  }

  if (UseC2CallingConventions) {
    //
    // now set up a stack frame with the size computed above
    //
    __ round_to( Gframe_size, WordsPerLong );
    __ sll( Gframe_size, LogBytesPerWord, Gframe_size );
    __ neg( Gframe_size );
    // sub( Gframe_size, 64, Gframe_size );      // (could add extra space here)
    __ save( SP, Gframe_size, SP );

  } else {
    // determine space & setup new stack frame
    // Gtmp1: parameter size in words
    const int extra_space =
      rounded_vm_local_words +                   // frame local scratch space
      frame::memory_parameter_word_sp_offset +   // register save area
      (native_call ? frame::interpreter_frame_extra_outgoing_argument_words : 0);
    if (native_call) {
      __ calc_mem_param_words(Gtmp1, Gtmp2);     // space for native call parameters passed on the stack in words
    } else {
      __ lduh(Gmethod, methodOopDesc::max_stack_offset_in_bytes(), Gtmp2);
    }
    __ neg(Gtmp2);                               // negative space for stack/parameters in words
    __ sub(Gtmp2, extra_space, Gtmp2);           // subtract space for extra space in words
    __ and3(Gtmp2, -WordsPerLong, Gtmp2);        // make multiple of 2 (SP must be 2-word aligned)
    __ sll(Gtmp2, LogBytesPerWord, Gtmp2);       // negative space for frame in bytes
    __ save(SP, Gtmp2, SP);                      // setup new frame and register window
  }

  //
  // now set up all the local cache registers
  //
  if (UseC2CallingConventions) {
    __ add( codes, Lbcp );                       // set Lbcp
    __ mov( Gmethod, Lmethod);                   // set Lmethod
    __ get_constant_pool_cache( LcpoolCache );   // set LcpoolCache
    __ sub( FP, rounded_vm_local_words * wordSize, Lmonitors ); // set Lmonitors
    __ mov( OparamAddr.after_save(), Llocals );  // set Llocals
    __ sub( Lmonitors, wordSize, Lesp );         // set Lesp
    __ clr(IsizeCalleeParms);                    // initialize to 0

  } else {
    // setup interpreter activation registers
    __ add(Gmethod, methodOopDesc::codes_offset_in_bytes(), Lbcp); // set Lbcp
    __ mov(Gmethod, Lmethod);                    // set Lmethod
    __ sub(Gargs, BytesPerWord, Llocals);        // set Llocals
    __ sub(FP, rounded_vm_local_words * BytesPerWord, Lmonitors ); // set Lmonitors
    __ sub(Lmonitors, BytesPerWord, Lesp);       // set Lesp
    __ get_constant_pool_cache(LcpoolCache);     // set LcpoolCache
  }

  // Stack after setup:
  //
  // +---------------+ <--- sp
  // |               |
  // : reg save area :
  // |               |
  // +---------------+ <--- sp + 0x40
  // |               |
  // : extra 7 slots :      note: these slots are not really needed for the interpreter (fix later)
  // |               |
  // +---------------+ <--- sp + 0x5c
  // |               |
  // :               :
  // |               | <--- Lesp
  // +---------------+ <--- Lmonitors (fp - 0x18)
  // |   VM locals   |
  // +---------------+ <--- fp
  // |               |
  // : reg save area :
  // |               |
  // +---------------+ <--- fp + 0x40
  // |               |
  // : extra 7 slots :      note: these slots are not really needed for the interpreter (fix later)
  // |               |
  // +---------------+ <--- fp + 0x5c
  // |               |
  // :     free      :
  // |               |
  // +---------------+
  // |               |
  // : nonarg locals :
  // |               |
  // +---------------+
  // |               |
  // :   arguments   :
  // |               | <--- Llocals
  // +---------------+ <--- Gargs
  // |               |

  //
  // Code to initialize locals
  //
  if (native_call) {
    // this slot will be set later, we initialize it to null here just in
    // case we get a GC before the actual value is stored later
    __ st(G0, FP, frame::interpreter_frame_mirror_offset * BytesPerWord);
  }
  else {
    Register init_value = noreg;    // will be G0 if we must clear locals
    if (zerolocals || ClearInterpreterLocals) {
      // explicitly initialize locals
      init_value = G0;
    } else {
#ifdef ASSERT
      // initialize locals to a garbage pattern for better debugging
      init_value = O3;
      __ set( 0x0F0F0F0F, init_value );
#endif // ASSERT
    }
    if (init_value != noreg) {
      Label clear_loop;

      // NOTE: If you change the frame layout, this code will need to
      // be updated!
      __ lduh( size_of_locals, O2 );
      __ lduh( size_of_parameters, O1 );
      __ sll( O2, LogBytesPerWord, O2);
      __ sll( O1, LogBytesPerWord, O1 );
      __ sub( Llocals, O2, O2 );
      __ sub( Llocals, O1, O1 );

      __ bind( clear_loop );
      __ inc( O2, wordSize );

      __ cmp( O2, O1 );
      __ br( Assembler::lessEqualUnsigned, true, Assembler::pt, clear_loop );
      __ delayed()->st( init_value, O2, 0 );
    }
  }

#ifndef CORE
  // increment invocation counter and check for overflow
  //
  // Note: checking for negative value instead of overflow
  //       so we have a 'sticky' overflow test (may be of
  //       importance as soon as we have true MT/MP)
  Label invocation_counter_overflow; 
  if (inc_counter) {
    // Update standard invocation counters
    __ increment_invocation_counter(O0, Gscratch2);
#ifdef COMPILER2
    if( CountInterpInvocations ) {
      Address interpreter_invocation_counter(Gmethod, 0, methodOopDesc::interpreter_invocation_counter_offset_in_bytes());
      __ ld(interpreter_invocation_counter, Gscratch);
      __ inc(Gscratch);
      __ st(Gscratch, interpreter_invocation_counter);
    }
#endif // COMPILER2
    Address limit(Gscratch, (address)&InvocationCounter::InterpreterInvocationLimit);
    __ sethi(limit);
    __ ld(limit, Gscratch);
    __ cmp(O0, Gscratch);
    __ br(Assembler::greaterEqualUnsigned, false, Assembler::pn, invocation_counter_overflow);
    __ delayed()->nop();
  }
#endif // CORE

  // check for synchronized methods
  // Must happen AFTER invocation_counter check, so method is not locked
  // if counter overflows.

  if (synchronized) {
    __ ld(access_flags, O0);

#ifdef ASSERT
    { Label ok;
      __ btst(JVM_ACC_SYNCHRONIZED, O0);
      __ br( Assembler::notZero, false, Assembler::pt, ok);
      __ delayed()->nop();
      __ stop("method doesn't need synchronization");
      __ bind(ok);
    }
#endif // ASSERT

    // get synchronization object to O0
    { Label done;
      const int mirror_offset = klassOopDesc::klass_part_offset_in_bytes() + Klass::java_mirror_offset_in_bytes();
      __ btst(JVM_ACC_STATIC, O0);
      __ br( Assembler::zero, true, Assembler::pt, done);
      __ delayed()->ld(Llocals, 0, O0); // get receiver for not-static case

      __ ld( Lmethod, methodOopDesc::constants_offset_in_bytes(), O0);
      __ ld( O0, constantPoolOopDesc::pool_holder_offset_in_bytes(), O0);

      // lock the mirror, not the klassOop
      __ ld( O0, mirror_offset, O0);

#ifdef ASSERT
      __ tst(O0);
      __ breakpoint_trap(Assembler::zero);
#endif // ASSERT

      __ bind(done);
    }

    __ add_monitor_to_stack(true, noreg, noreg);  // allocate monitor elem
    __ st( O0, Lmonitors, BasicObjectLock::obj_offset_in_bytes());   // store object
    // __ untested("lock_object from method entry");
    __ lock_object(Lmonitors, O0);
  } else {
#ifdef ASSERT
    { Label ok;
      __ ld(access_flags, O0);
      __ btst(JVM_ACC_SYNCHRONIZED, O0);
      __ br( Assembler::zero, false, Assembler::pt, ok);
      __ delayed()->nop();
      __ stop("method needs synchronization");
      __ bind(ok);
    }
#endif // ASSERT
  }

  // Quick & dirty stack overflow checking: bang the stack & handle trap.
  // Note that we do the banging after the frame is setup, since the exception handling code expects to 
  // find a valid interpreter frame on the stack. Doing the banging earlier fails if
  // the caller frame is not an interpreter frame.
  // (Also, the exception throwing code expects to unlock any synchronized
  // method receiever, so do the banging after locking the receiver.)
  if (UseStackBanging) {
    __ set(-os::vm_page_size(), Gscratch);
    __ st(G0, SP, Gscratch);
  }


  // start execution

  __ verify_thread();

#ifndef CORE
#ifndef PRODUCT
  // generate code that calls tracing subroutine
  __ trace_method_entry(Lmethod);
#endif // PRODUCT
#endif // CORE

  // jvmdi/jvmpi support 
  __ notify_method_entry();

  if (!native_call) {
    // start executing instructions
    __ dispatch_next(vtos);

  } else {
    // native call
    // result potentially in O0/O1: save it across calls in here
    const Address& l_tmp = InterpreterMacroAssembler::l_tmp;

    // result potentially in F0/F1: save it across calls in here
    const Address& d_tmp = InterpreterMacroAssembler::d_tmp;

    // (note that O0 is never an oop--at most it is a handle)
    // It is important not to smash any handles created by this call,
    // until any oop handle in O0 is dereferenced.

    // (note that the space for outgoing params is preallocated)

    // get signature handler
    { Label L;
      __ ld(Address(Lmethod, 0, methodOopDesc::signature_handler_offset_in_bytes()), Gscratch);
      __ tst(Gscratch);
      __ br(Assembler::notZero, false, Assembler::pt, L);
      __ delayed()->nop();
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::prepare_native_call), Lmethod);
      __ ld(Address(Lmethod, 0, methodOopDesc::signature_handler_offset_in_bytes()), Gscratch);
      __ bind(L);
    }

    // call signature handler
    __ callr(Gscratch, 0);
    __ delayed()->nop();

    { Label not_static;

      __ ld(access_flags, O0);
      __ btst(JVM_ACC_STATIC, O0);
      __ br( Assembler::zero, false, Assembler::pt, not_static);
      __ delayed()->
        // get native function entry point(O0 is a good temp until the very end)
         ld(Address(Lmethod, 0, methodOopDesc::native_function_offset_in_bytes()), O0);
      // for static methods insert the mirror argument
      const int mirror_offset = klassOopDesc::klass_part_offset_in_bytes() + Klass::java_mirror_offset_in_bytes();

      __ ld(Address(Lmethod, 0, methodOopDesc:: constants_offset_in_bytes()), O1);
      __ ld(Address(O1, 0, constantPoolOopDesc::pool_holder_offset_in_bytes()), O1);
      __ ld(O1, mirror_offset, O1);
      // where the mirror handle body is allocated:
      Address mirror(FP, 0, frame::interpreter_frame_mirror_offset*wordSize);
#ifdef ASSERT
      if (!PrintSignatureHandlers)  // do not dirty the output with this
      { Label L;
	__ tst(O1);
	__ br(Assembler::notZero, false, Assembler::pt, L);
	__ delayed()->nop();
	__ stop("mirror is missing");
	__ bind(L);
      }
#endif // ASSERT
      __ st(O1, mirror);
      __ add(mirror, O1);
      __ bind(not_static);
    }

    // At this point, arguments have been copied off of stack into
    // their JNI positions, which are O1..O5 and SP[68..].
    // Oops are boxed in-place on the stack, with handles copied to arguments.
    // The result handler is in Lscratch.  O0 will shortly hold the JNIEnv*.

#ifdef ASSERT
    { Label L;
      __ tst(O0);
      __ br(Assembler::notZero, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("native entry point is missing");
      __ bind(L);
    }
#endif // ASSERT

    // reset handle block
    __ ld(Gthread, JavaThread::active_handles_offset_in_bytes(), Gscratch);
    __ st(G0, Gscratch, JNIHandleBlock::top_offset_in_bytes());

    //
    // setup _last_Java_sp and post_Java_state
    //
    // post_Java_state is setup as an is_pc so that an unpatchable frame is
    // created (the next instr. add(O7, JavaThread::is_pc, Gscratch)
    // indicates this)
    //
    // The scavenge function only needs to know that the PC of this frame is
    // in the interpreter method entry code, it doesn't need to know the exact
    // PC and hence we can use O7 which points to the return address from the
    // previous call in the code stream (signature handler function)
    //
    __ add(O7, JavaThread::is_pc, Gscratch);
    __ set_last_Java_frame(SP, Gscratch);

    // transition the thread state from _thread_in_Java to _thread_in_native
    //
    Address thread_state(Gthread,0,JavaThread::thread_state_offset_in_bytes());

#ifdef ASSERT
    { Label L;
      __ ld(thread_state, Gscratch);
      __ cmp(Gscratch, _thread_in_Java);
      __ br(Assembler::equal, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("Wrong thread state in native stub");
      __ bind(L);
    }
#endif // ASSERT

    // Push return address on stack. We need to do this before we change thread-state, to
    // make sure that the safepoint code will always see a valid return address on the stack.
    
    // Change state to native (we save the return address in the thread, since it might not
    // be pushed on the stack when we do a a stack traversal). It is enough that the pc()
    // points into the right code segment. It does not have to be the correct return pc.
    { Label L;
      __ call(L);
      __ delayed()->nop();
      __ bind(L);
      // Don't bother to update last_native_pc_offset:  It is easier
      // just to look at O7.
      // __ st(O7, Gthread, JavaThread::last_native_pc_offset_in_bytes());
    }

    __ set(_thread_in_native, Gscratch);
    __ st(Gscratch, thread_state);
    
    //
    // save the thread to Lthread_cache -- Gthread is now dead until the next restore_thread()
    // 
    __ save_thread(Lthread_cache);

    // do call
    __ callr(O0, 0);
    // and pass JNIEnv
    __ delayed()->
       add(Lthread_cache, JavaThread::jni_environment_offset_in_bytes(), O0);

    __ restore_thread(Lthread_cache);

    // The following code is not needed as Lbcp is not clobbered, It is left
    // here as a reference for reconcilations
    // restore Lbcp to have legal interpreter frame, i.e., bci == 0 <=> bcp == code_base()
    // __ add(Lmethod, methodOopDesc::codes_offset_in_bytes(), Lbcp);


    //
    // Check if we have to block - we need to do this before we clear
    // the last_Java_sp, in order for GC to work
    //
    { Label no_block;
      Address sync_state(Gscratch, SafepointSynchronize::address_of_state());

      __ set(_thread_in_native_trans, Gscratch);
      __ st(Gscratch, thread_state);

      __ load_contents(sync_state, Gscratch);
      __ cmp(Gscratch, SafepointSynchronize::_not_synchronized);
      __ br(Assembler::equal, true, Assembler::pt, no_block);

      // change thread state
      __ delayed()->
         set(_thread_in_Java, Gscratch);  // hoisted from below

      // The next two lines were placed before the branch above to take care of the following (extremely rare) scenario:
      //
      //      -java thread A (in _thread_in_native state) loads the sync_state and gets _not_synchronized
      //      -the vm thread sets sync state to synchronizing and looks at thread A, it lets it run
      //      -thread A doesn't block, sets its state to _thread_in_Java and escapes
      //__ set(_thread_in_native_trans, Gscratch);
      //__ st(Gscratch, thread_state);

      __ stf(FloatRegister::D, F0, d_tmp);
      __ std(                  O0, l_tmp);

      __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SafepointSynchronize::block), Gthread); // use leaf call to not disturb last_Java_frame setup

      __ set(_thread_in_Java, Gscratch);

      __ ldf(FloatRegister::D, d_tmp, F0);
      __ ldd(                  l_tmp, O0);
      __ bind(no_block);
    }


    //
    // clean up -- reset the last and post Java sp
    //
    // set(_thread_in_Java, Gscratch); // pulled into previous delay slot
    __ st(Gscratch, thread_state);

    // The following is copied from the Intel version, for reference:
    //// Clear last_native_pc after call. This is crucial, so we can detect if the last_native_pc is
    //// used or not. It is, e.g., not used when calling native code directly from C++ code.
    //__ movl(Address(thread, JavaThread::last_native_pc_offset_in_bytes()), NULL);  

    __ reset_last_Java_frame();

    // handle exceptions (exception handling will handle unlocking!)
    { Label L;
      Address exception_addr (Gthread, 0, Thread::pending_exception_offset_in_bytes());
  
      __ ld(exception_addr, Gtemp);
      __ tst(Gtemp);
      __ br(Assembler::equal, false, Assembler::pt, L);
      __ delayed()->nop();
      // Note: This could be handled more efficiently since we know that the native
      //       method doesn't have an exception handler. We could directly return
      //       to the exception handler for the caller.
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_pending_exception));
      __ should_not_reach_here();
      __ bind(L);
    }

    // jvmdi/jvmpi support (preserves thread register) 
    __ notify_method_exit(true, ilgl);  

#ifndef CORE
#ifndef PRODUCT
    // native method exit
    __ trace_method_exit(Lmethod);
#endif // PRODUCT
#endif // CORE

    // do unlocking if necessary
    if (synchronized) {
      __ stf(FloatRegister::D, F0, d_tmp);
      __ std(                  O0, l_tmp);

      __ add( __ top_most_monitor(), O1 );
      __ unlock_object(O1);

      __ ldf(FloatRegister::D, d_tmp, F0);
      __ ldd(                  l_tmp, O0);
    }

    // dispose of return address and remove activation
    __ jmp(Lscratch, 0);
    __ delayed()->nop();
  } // end native_call


#ifndef CORE
  // handle invocation counter overflow
  if (inc_counter) {
    __ bind(invocation_counter_overflow);
    // InterpreterRuntime::invocation_counter_overflow takes two arguments,
    // the first indicates if the counter overflow occurs at a backwards branch (false)
    // and the second is only used when the first is true.  We pass zero for both.
    // The call returns the address of the verified entry point for the method or NULL
    // if the compilation did not complete (either went background or bailed out).
    __ set((int)false, O2);
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::invocation_counter_overflow), O2, O2); // returns verified_entry_point or NULL
  }

  if (UseC2CallingConventions) {
    // method has been compiled - remove activation frame
    // (leave return address on stack) and continue at
    // verified entry point
    //
    // Note: continuation at verified entry point works if the method that has been
    //       compiled is the right one (in case of virtual calls); i.e., the inline
    //       cache check must have happened before the invocation counter overflow
    //       check.

    // __ jmp(O0, 0);                               // continue at (compiled) verified entry point
    // __ delayed()->restore(Lmethod, G0, Gmethod); // restore Gmethod and window

    // Preserve invariant that Lbcp/Llocals contain bcp/locals of sender frame
    __ restore(Lmethod, G0, Gmethod);              // get rid of activation frame
    // The interpreter has set up certain values prior to the save
    // assuming the method was going to be interpreted, this needs to
    // be undone as part of removing the activation frame
    __ clr(IsizeCalleeParms);
    debug_only(__ verify_sp(IsavedSP, Gscratch));
    __ ba(false, Lentry);
    __ delayed()->mov(IsavedSP, SP);
  
  } else {
    // C1 Calling Conventions

    // restore O0 (must contain receiver in case of virtual calls)
    __ ld(Llocals, 0, O0.after_save());          // is this needed?

    // restore Gmethod & Gargs
    __ add(Llocals, BytesPerWord, Gargs);
    __ mov(Lmethod, Gmethod);

    // determine arguments
    __ lduh(Gmethod, methodOopDesc::size_of_parameters_offset_in_bytes(), Gtmp1);
    __ sll(Gtmp1, LogBytesPerWord, Gtmp2);       // parameter size in bytes
    __ sub(Gargs, Gtmp2, Gargs);                 // points to last argument

    // determine extra space for non-argument locals
    // Gtmp1: parameter size in words
    if (native_call) {
      // the caller's SP was not adjusted => simply unwind and redo method entry
      __ ba(false, Lentry);
      __ delayed()->restore();
    } else {
      // the caller's SP was adjusted upon method entry to accomodate
      // the callee's non-argument locals => determine the adjustment
      __ lduh(Gmethod, methodOopDesc::size_of_locals_offset_in_bytes(), Gtmp2);
      __ compute_extra_locals_size_in_bytes(Gtmp1, Gtmp2, Gtmp2);
      // restore register window & adjust SP, then redo method entry
      __ ba(false, Lentry);                      // redo method entry
      __ delayed()->restore(FP, Gtmp2, SP);      // restore registers & reset caller's SP
    }

  }

  // compiled code for method exists
  // Gmethod: methodOop
  // Gscratch: nmethod   (NOTE:Gscratch should not clobbered until the call to nmethod_entry_point)
  __ bind(run_compiled_code);

  if (UseC2CallingConventions) {
    // make sure interpreter entry point exists
    { Label L;

      __ ld(Address(Gscratch, 0, nmethod::interpreter_entry_point_offset()), Gscratch2);
      __ tst(Gscratch2);
      __ br(Assembler::notZero, false, Assembler::pn, L);
      __ delayed()->nop();

      COMPILER1_ONLY(__ should_not_reach_here();)

      // Do not use __ call_VM here, since it will call to the InterpreterMacroAssembler, which
      // assumes that the current frame (the caller) is interpreted (but it could be compiled)
      __ save_frame(0);
      __ super_call_VM(Lthread_cache, Gmethod, FP, CAST_FROM_FN_PTR(address, InterpreterRuntime::nmethod_entry_point), Gmethod, Gscratch, !Assembler::_check_for_exceptions_flag);
      __ mov(O0, Gscratch2);
      __ restore();

      // Gmethod: methodOop
      // Gscratch2: interpreter nmethod entry
#ifdef ASSERT
      { Label L;
        __ tst(Gscratch2);
        __ br(Assembler::notZero, false, Assembler::pn, L);
        __ delayed()->nop();
        __ stop("interpreter entry point == NULL");
        __ bind(L);
      }
#endif // ASSERT
      // Note: At this point, the nmethod may have been recompiled once more; i.e., if one would
      //       look at the nmethod's interpreter entry point, it could be NULL! However, since
      //       the interpreter entry point is simply the corresponding i2c adapter, which is
      //       dependent of the method's signature only, we don't care. If the nmethod has been
      //       deoptimized, we will end up in a zombie nmethod, which is ok, too.
      __ bind(L);
    }
    // Gmethod: compiled code expects methodOop in Gmethod 
    // Gscratch2: interpreter nmethod entry
    __ jmp(Gscratch2, 0);                 // enter compiled code via i2c adapter
    __ delayed()->nop();

  } else {
    // C1 Calling Conventions
    // Gmethod: method to be invoked
    // Gargs  : points to last argument
    COMPILER2_ONLY(__ stop("this code needs to be adjusted for C2");)
    assert_different_registers(Gmethod, Gargs, Gscratch, Gscratch3);
    __ ld(Address(Gscratch, 0, nmethod::interpreter_entry_point_offset()), Gscratch3);
    __ jmp(Gscratch3, 0);
    __ delayed()->nop();
  }
#endif // not CORE

  return entry;
}



#ifndef CORE
static inline int calc_mem_param_words(int argument_words) {
  // Parallels MacroAssembler::calc_mem_param_words.
  argument_words -= Argument::n_register_parameters;
  if (argument_words < 0)  return 0;
  return argument_words;
}

static inline int calc_frame_size_in_words(int extra_words) {
  // Parallels MacroAssembler::calc_frame_size, except for the scaling by 4.
  int frame_size = extra_words + frame::memory_parameter_word_sp_offset;
  return round_to(frame_size, WordsPerLong);
}


// This method tells the deoptimizer how big an interpreted frame must be:
int AbstractInterpreter::size_activation(methodOop method, int tempcount, int moncount, int callee_param_size, int callee_locals) {
  // %%% this is different from the Intel version -- which is right?
  return layout_activation_impl(method, tempcount, moncount, (frame*)NULL, callee_param_size, callee_locals);
}


int AbstractInterpreter::layout_activation_impl(methodOop method, int tempcount, int moncount, frame* f, int callee_param_size, int callee_locals_size) {
  // Note: This calculation must exactly parallel the frame setup
  // in AbstractInterpreterGenerator::generate_method_entry.
  // If f!=NULL, set up the following variables:
  //   - Lmethod
  //   - Llocals
  //   - Lmonitors (to the indicated number of monitors)
  //   - Lesp (to the indicated number of temps)
  // The frame f, if not NULL, is guaranteed to be the right size,
  // as determined by a previous call to this method.

  int raw_frame_size;
  int monitor_size           = moncount * frame::interpreter_frame_monitor_size();
  int rounded_vm_local_words = round_to(frame::interpreter_frame_vm_local_words,WordsPerLong);

  assert(monitor_size == round_to(monitor_size, WordsPerLong), "must align");
  callee_locals_size = round_to((callee_locals_size - callee_param_size), WordsPerLong);
  assert(callee_locals_size == round_to(callee_locals_size, WordsPerLong), "must align");
  raw_frame_size = round_to((method->max_stack()
                   + rounded_vm_local_words
                   + frame::memory_parameter_word_sp_offset), WordsPerLong)
                 + callee_locals_size
                 + monitor_size;
  if (f != NULL) {
    frame caller_frame = (*f);

    address random_pc = return_entry(vtos, 0);   // will be overwritten
    jint* sp = caller_frame.sp() - raw_frame_size;
    frame interpreter_frame(sp, frame::unpatchable, random_pc);
    assert(interpreter_frame.fp() == caller_frame.sp(), "fp must match");

    (*f) = interpreter_frame;

    jint* montop   = f->fp() - rounded_vm_local_words;
    jint* esp      = montop - 1;
    jint* monitors = montop;
    jint* locals   = f->fp() + Lesp.sp_offset_in_saved_window();
    jint* psize    = f->fp() + IsizeCalleeParms.sp_offset_in_saved_window();

    // preallocate monitors (cf. __ add_monitor_to_stack)
    monitors -= monitor_size;
    esp      -= monitor_size;

    // preallocate stack space
    esp -= tempcount;

    *f->register_addr( Lmethod  ) = (jint) method;
    *f->register_addr( Llocals  ) = (jint) (*locals + *psize);
    *f->register_addr( Lmonitors) = (jint) monitors;
    *f->register_addr( Lesp     ) = (jint) esp;
    *f->register_addr( IsavedSP ) = (jint) (f->sp() + callee_locals_size);
    *f->register_addr( IsizeCalleeParms) = (jint) (callee_param_size * wordSize);
    *f->register_addr( LcpoolCache ) = (jint) method->constants()->cache();


#ifdef ASSERT
    BasicObjectLock* mp = (BasicObjectLock*)monitors;

    assert(f->interpreter_frame_method() == method, "method matches");
    assert(&f->interpreter_frame_local_at(9) == (long *)((*locals + *psize) - (9 * wordSize)), "locals match");
    assert(f->interpreter_frame_monitor_end()   == mp, "monitor_end matches");
    assert(((jint *)f->interpreter_frame_monitor_begin()) == ((jint *)mp)+monitor_size, "monitor_begin matches");
    assert(f->interpreter_frame_tos_address()-1 == esp, "esp matches");

    // check bounds
    jint* lo = f->sp() + (frame::memory_parameter_word_sp_offset - 1);
    jint* hi = f->fp() - rounded_vm_local_words;
    assert(lo < monitors && montop <= hi, "monitors in bounds");
    assert(lo <= esp && esp < monitors, "esp in bounds");
#endif // ASSERT
  }

  return raw_frame_size;
}
#endif // not CORE


//----------------------------------------------------------------------------------------------------
// Exceptions

void AbstractInterpreterGenerator::generate_throw_exception() {

  // Entry point in previous activation (i.e., if the caller was interpreted)
  Interpreter::_rethrow_exception_entry = __ pc();
  // O0: exception
  
  // entry point for exceptions thrown within interpreter code
  Interpreter::_throw_exception_entry = __ pc();
  __ verify_thread();
  // expression stack is undefined here
  // O0: exception, i.e. Oexception
  // Lbcp: exception bcx
  __ verify_oop(Oexception);
  
  
  // expression stack must be empty before entering the VM in case of an exception 
  __ empty_expression_stack();
  // find exception handler address and preserve exception oop
  // call C routine to find handler and jump to it
  __ call_VM(O1, CAST_FROM_FN_PTR(address, InterpreterRuntime::exception_handler_for_exception), Oexception);
  __ push_i(O1); // push exception for exception handler bytecodes
  
  __ jmp(O0, 0); // jump to exception handler (may be remove activation entry!)
  __ delayed()->nop();


  // if the exception is not handled in the current frame
  // the frame is removed and the exception is rethrown
  // (i.e. exception continuation is _rethrow_exception)
  //
  // Note: At this point the bci is still the bxi for the instruction which caused
  //       the exception and the expression stack is empty. Thus, for any VM calls
  //       at this point, GC will find a legal oop map (with empty expression stack).

  // in current activation
  // tos: exception
  // Lbcp: exception bcp
  Interpreter::_remove_activation_entry = __ pc();

  // preserve exception over this code sequence (remove activation calls the vm, but oopmaps are not correct here)
  __ pop_i(Oexception);                                  // get exception

  // Intel has the following comment:
  //// remove the activation (without doing throws on illegalMonitorExceptions)
  // They remove the activation without checking for bad monitor state.
  // %%% We should make sure this is the right semantics before implementing.

  // %%% changed set_vm_result_2 to set_vm_result and get_vm_result_2 to get_vm_result. Is there a bug here?
  __ set_vm_result(Oexception);
  __ unlock_if_synchronized_method(vtos, /* throw_monitor_exception */ false);

  // jvmdi/jvmpi support (preserves thread register)
  __ notify_method_exit(false, vtos);

  __ get_vm_result(Oexception);
  __ verify_oop(Oexception);

    const int return_reg_adjustment = frame::pc_return_offset * BytesPerWord;
  Address issuing_pc_addr(I7, 0, return_reg_adjustment);

  // We are done with this activation frame; find out where to go next.
  // The continuation point will be an exception handler, which expects
  // the following registers set up:
  //
  // Oexception: exception
  // Oissuing_pc: the local call that threw exception
  // Other On: garbage
  // In/Ln:  the contents of the caller's register window
  //
  // We do the required restore at the last possible moment, because we
  // need to preserve some state across a runtime call.
  // (Remember that the caller activation is unknown--it might not be
  // interpreted, so things like Lscratch are useless in the caller.)

  // Although the Intel version uses call_C, we can use the more
  // compact call_VM.  (The only real difference on SPARC is a
  // harmlessly ignored [re]set_last_Java_frame, compared with
  // the Intel code which lacks this.)
  __ mov(Oexception,      Oexception .after_save());  // get exception in I0 so it will be on O0 after restore
  __ add(issuing_pc_addr, Oissuing_pc.after_save());  // likewise set I1 to a value local to the caller
  __ super_call_VM_leaf(Lthread_cache,
		        CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address),
		        Oissuing_pc.after_save());

  if (UseC2CallingConventions) {
    __ jmpl(O0, G0, G0);
    __ delayed()->restore();

  } else {
    // The caller's SP was adjusted upon method entry to accomodate
    // the callee's non-argument locals. Determine the adjustment.
    const Register Gtmp1 = Gscratch;
    const Register Gtmp2 = Gscratch3;
    __ lduh(Lmethod, methodOopDesc::size_of_parameters_offset_in_bytes(), Gtmp1);
    __ lduh(Lmethod, methodOopDesc::size_of_locals_offset_in_bytes()    , Gtmp2);
    __ compute_extra_locals_size_in_bytes(Gtmp1, Gtmp2, Gtmp2);
    // return, restore register window & adjust SP
    // Gtmp2: extra space allocated for non-argument locals in caller
    __ jmpl(O0, G0, G0);                         // return exception handler in caller
    __ delayed()->restore(FP, Gtmp2, SP);        // restore registers & reset caller's SP
  }
  // (same old exception object is already in Oexception; see above)
  // Note that an "issuing PC" is actually the next PC after the call
}


//------------------------------------------------------------------------------------------------------------------------
// Helper for vtos entry point generation

void AbstractInterpreterGenerator::set_vtos_entry_points(Template* t, address& iep, address& lep, address& fep, address& dep, address& vep) {
  assert(t->is_valid() && t->tos_in() == vtos, "illegal template");
  Label L;
  fep = __ pc(); __ push_f(); __ ba(false, L); __ delayed()->nop();
  dep = __ pc(); __ push_d(); __ ba(false, L); __ delayed()->nop();
  lep = __ pc(); __ push_l(); __ ba(false, L); __ delayed()->nop();
  iep = __ pc(); __ push_i();                  // fall through
  vep = __ pc(); __ bind(L);                   // fall through
  generate_and_dispatch(t);
}

// --------------------------------------------------------------------------------


InterpreterGenerator::InterpreterGenerator(StubQueue* code) 
 : AbstractInterpreterGenerator(code) {
   generate_all(); // down here so it can be "virtual"
}

// --------------------------------------------------------------------------------
// Non-product code
#ifndef PRODUCT
address AbstractInterpreterGenerator::generate_bci_histogram_entry() {
     address entry = __ pc();

  __ untested("bci histogram");
  __ ld( Lmethod, methodOopDesc::bci_histogram_offset_in_bytes(), Lscratch); // get array
  __ sub( Lbcp, methodOopDesc::codes_offset_in_bytes(), Lscratch2); // index
  __ sll( Lscratch2, LogBytesPerWord, Lscratch2 ); // word index
  __ add( Lscratch, Lscratch2, Lscratch ); // address within array
  __ ld(  Lscratch, 0, Lscratch2 );
  __ inc( Lscratch2 );
  __ st(  Lscratch2, Lscratch, 0);
  
  __ retl();  
  __ delayed()->nop();

  return entry;
}
#endif // not PRODUCT


address AbstractInterpreterGenerator::generate_trace_code(TosState state) {
  address entry = __ pc();

  __ push(state);
  __ mov(O7, Lscratch); // protect return address within interpreter
  __ call_VM(noreg, CAST_FROM_FN_PTR(address, SharedRuntime::trace_bytecode), Lmethod, Lbcp);
  __ mov(Lscratch, O7); // restore return address
  __ pop(state);
  __ retl();  
  __ delayed()->nop();

  return entry;
}


// Non-product code
#ifndef PRODUCT
// helpers for generate_and_dispatch

void AbstractInterpreterGenerator::count_bytecode() { 
  Address c(Gscratch, (address)&BytecodeCounter::_counter_value);
  __ load_contents(c, Gscratch2);
  __ inc(Gscratch2);
  __ st(Gscratch2, c);
}


void AbstractInterpreterGenerator::histogram_bytecode(Template* t) { 
  Address bucket( Gscratch, (address) &BytecodeHistogram::_counters[t->bytecode()] );
  __ load_contents(bucket, Gscratch2);
  __ inc(Gscratch2);
  __ st(Gscratch2, bucket);
}


void AbstractInterpreterGenerator::histogram_bytecode_pair(Template* t) { 
  __ untested("histogram_bytecode_pair");
  address index_addr      = (address)&BytecodePairHistogram::_index;
  int     counters_offset = (address) BytecodePairHistogram::_counters - index_addr;
  Address index(Gscratch, index_addr);

  // get index, bring in new bytecode, and store it

  __ load_contents( index,      Gscratch2 );
  __ srl(           Gscratch2, BytecodePairHistogram::log2_number_of_codes, Gscratch2 );
  __ set( ((int)t->bytecode()) << BytecodePairHistogram::log2_number_of_codes,  Gscratch );
  __ or3(            Gscratch,  Gscratch2, Gscratch2 );
  __ store_contents( Gscratch2, index );

  // bump bucket contents

  __ sll( Gscratch2,    LogBitsPerWord, Gscratch2 );
  __ add( index.base(), Gscratch2,      index.base() );
  __ ld(  index,        Gscratch2,      counters_offset);
  __ inc( Gscratch2 );
  __ st(  Gscratch2,    index,          counters_offset );
}


void AbstractInterpreterGenerator::histogram_bci() {
  // Copied from trace_bytecode below, but the purpose here is to
  // gather a per-bci profile.
  address entry = Interpreter::bci_histogram_entry();
  assert(entry != NULL, "entry must have been generated");
  __ call(entry, relocInfo::none);
  __ delayed()->nop();
}
#endif // not PRODUCT


void AbstractInterpreterGenerator::trace_bytecode(Template* t) {
  // Call a little run-time stub to avoid blow-up for each bytecode.
  // The run-time runtime saves the right registers, depending on
  // the tosca in-state for the given template.
  address entry = Interpreter::trace_code(t->tos_in());
  guarantee(entry != NULL, "entry must have been generated");
  __ call(entry, relocInfo::none);
  __ delayed()->nop();
}


// Non-product code
#ifndef PRODUCT
void AbstractInterpreterGenerator::stop_interpreter_at() {
  __ load_contents(Address(Gscratch , (address)&BytecodeCounter::_counter_value), Gscratch );
  __ load_contents(Address(Gscratch2, (address)&StopInterpreterAt              ), Gscratch2);
  __ cmp(Gscratch, Gscratch2);
  __ breakpoint_trap(Assembler::equal);
}
#endif // not PRODUCT

//Reconciliation History
// 1.67 97/11/12 09:26:11 interpreter_i486.cpp
// 1.75 97/12/04 14:39:58 interpreter_i486.cpp
// 1.78 97/12/19 08:49:42 interpreter_i486.cpp
// 1.80 98/01/12 16:14:11 interpreter_i486.cpp
// 1.87 98/02/03 19:06:48 interpreter_i486.cpp
// 1.92 98/02/20 17:07:36 interpreter_i486.cpp
// 1.94 98/02/28 11:28:31 interpreter_i486.cpp
// 1.99 98/03/10 13:46:51 interpreter_i486.cpp
// 1.100 98/03/17 11:02:27 interpreter_i486.cpp
// 1.102 98/03/26 16:49:49 interpreter_i486.cpp
// 1.108 98/04/08 11:09:44 interpreter_i486.cpp
// 1.111 98/04/16 19:07:07 interpreter_i486.cpp
// 1.114 98/04/22 12:43:43 interpreter_i486.cpp
// 1.110 98/04/30 16:37:59 interpreter_i486.cpp
// 1.134 98/05/06 14:32:09 interpreter_i486.cpp
// 1.136 98/05/07 14:49:24 interpreter_i486.cpp
// 1.141 98/05/11 17:34:50 interpreter_i486.cpp
// 1.147 98/05/21 11:56:36 interpreter_i486.cpp
// 1.151 98/05/28 10:48:58 interpreter_i486.cpp
// 1.156 98/06/08 07:59:36 interpreter_i486.cpp
// 1.159 98/06/15 15:36:17 interpreter_i486.cpp
// 1.163 98/06/17 20:35:29 interpreter_i486.cpp
// 1.167 98/06/23 18:19:20 interpreter_i486.cpp
// 1.171 98/07/06 15:08:14 interpreter_i486.cpp
// 1.173 98/07/24 13:32:36 interpreter_i486.cpp
// 1.175 98/08/06 12:57:14 interpreter_i486.cpp
// 1.177 98/08/26 18:28:03 interpreter_i486.cpp
// 1.181 98/09/25 16:05:19 interpreter_i486.cpp
// 1.184 98/09/30 14:24:49 interpreter_i486.cpp
// 1.185 98/10/05 22:30:59 interpreter_i486.cpp
// 1.187 98/10/16 17:13:06 interpreter_i486.cpp
// 1.188 98/10/21 16:49:20 interpreter_i486.cpp
// 1.189 98/11/23 15:35:13 interpreter_i486.cpp
// 1.193 98/11/27 13:06:30 interpreter_i486.cpp
// 1.194 99/01/12 14:22:48 interpreter_i486.cpp
// 1.195 99/01/20 15:59:42 interpreter_i486.cpp
// 1.198 99/02/02 17:54:19 interpreter_i486.cpp
// 1.199 99/02/03 19:49:52 interpreter_i486.cpp
// 1.200 99/02/17 10:14:51 interpreter_i486.cpp
// 1.201 99/03/04 15:52:33 interpreter_i486.cpp
// 1.203 99/03/22 18:07:10 interpreter_i486.cpp
// 1.204 99/04/01 13:51:50 interpreter_i486.cpp
// 1.206 99/04/19 16:49:29 interpreter_i486.cpp
// 1.207 99/06/16 13:37:11 interpreter_i486.cpp
// 1.211 99/06/24 18:35:37 interpreter_i486.cpp
// 1.214 99/06/29 18:05:21 interpreter_i486.cpp
// 1.217 99/07/23 18:43:55 interpreter_i486.cpp
// 1.218 99/08/17 02:39:51 interpreter_i486.cpp
// 1.219 99/08/18 14:09:58 interpreter_i486.cpp
// 1.221 99/08/27 12:55:40 interpreter_i486.cpp
//End
