/*
 * @(#)nativeInst_sparc.cpp	1.54 01/02/09 14:43:27
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_nativeInst_sparc.cpp.incl"


void NativeInstruction::verify() {
  // make sure code pattern is actually an instruction address
  address addr = addr_at(0);
  if (addr == 0 || ((int)addr & 3) != 0)
    fatal("not an instruction address");
}

void NativeInstruction::print() {
  tty->print_cr("0x%x: 0x%x", addr_at(0), long_at(0));
}

void NativeInstruction::set_long_at(int offset, int i) {
  address addr = addr_at(offset);
  *(jint*)addr = i;
  ICache::invalidate_word(addr);
}

bool NativeInstruction::is_zero_test(Register &reg) {
  int x = long_at(0);
  Assembler::op3s temp = (Assembler::op3s) (Assembler::sub_op3 | Assembler::cc_bit_op3);
  if (is_op3(x, temp, Assembler::arith_op) &&
      inv_immed(x) &&  inv_rd(x) == G0) {
      if (inv_rs1(x) == G0) {
        reg = inv_rs2(x);
        return true;  
      }
      else if (inv_rs2(x) == G0) {
        reg = inv_rs1(x);
        return true;  
      }
  }
  return false;
}

bool NativeInstruction::is_load_store_with_small_offset(Register reg) {
  int x = long_at(0);
  if (is_op(x, Assembler::ldst_op) &&
      inv_rs1(x) == reg && inv_immed(x)) {
    return true;
  }
  return false;
}


void NativeCall::verify() {
  NativeInstruction::verify();
  // make sure code pattern is actually a call instruction
  if (!is_op(long_at(0), Assembler::call_op))
    fatal("not a call");
}


bool NativeCall::is_call_at(address instr) {
  return nativeInstruction_at(instr)->is_call();
}


void NativeCall::print() {
  tty->print_cr("0x%x: call 0x%x", instruction_address(), destination());
}

bool NativeCall::destination_is_compiled_verified_entry_point() {
#ifndef CORE
  nmethod* callee = CodeCache::find_nmethod(destination());
  if (callee == NULL) {
    return false;
  } else {
    return destination() == callee->verified_entry_point();
  }
#else
  return false;
#endif
}


bool NativeCall::is_call_to_stub() const {
#ifndef CORE
  nmethod* caller_method = CodeCache::find_nmethod(instruction_address());
  assert(caller_method != NULL, "wrong instruction");
  return (caller_method->stub_contains(destination()));
#else
  return false;
#endif
}


// Inserts a native call instruction at a given pc
void NativeCall::insert(address code_pos, address entry) {
  NativeCall* call = nativeCall_overwriting_at(code_pos, entry);
}


// MT-safe patching of a call instruction (and following word).
// First patches the second word, and then atomicly replaces
// the first word with the first new instruction word.
// Other processors might briefly see the old first word
// followed by the new second word.  This is OK if the old
// second word is a nop, and the new second word may be
// harmlessly executed in the delay slot of the call.
void NativeCall::replace_mt_safe(address instr_addr, address code_buffer) {
   assert (instr_addr != NULL, "illegal address for code patching");
   NativeCall* n_call =  nativeCall_at (instr_addr); // checking that it is a call
   assert(NativeCall::instruction_size == 8, "wrong instruction size; must be 8");
   jint i0 = ((jint*)code_buffer)[0];
   jint i1 = ((jint*)code_buffer)[1];
   jint* contention_addr = (jint*) n_call->addr_at(1*wordSize);
   assert(*contention_addr == nop_instruction(), "must not interfere with original call");
   n_call->set_long_at(1*wordSize, i1);
   n_call->set_long_at(0*wordSize, i0);
   // NOTE:  It is possible that another thread T will execute
   // only the second patched word.
   // In other words, since the original instruction is this
   //    call patching_stub; nop                   (NativeCall)
   // and the new sequence from the buffer is this:
   //    sethi %hi(K), %r; add %r, %lo(K), %r      (NativeMovConstReg)
   // what T will execute is this:
   //    call patching_stub; add %r, %lo(K), %r
   // thereby putting garbage into %r before calling the patching stub.
   // This is OK, because the patching stub ignores the value of %r.

   // Make sure the first-patched instruction, which may co-exist
   // briefly with the call, will do something harmless.
   assert(inv_op(*contention_addr) == Assembler::arith_op || *contention_addr == nop_instruction() || !VM_Version::v9_instructions_work(), 
				"must not interfere with original call");
   // %%%%% Really, we should see if we can get away with patching
   // only a single "call" word, and making sure that the delay slot
   // is harmless.  Why must it start out as a nop?
}

// Code for unit testing implementation of NativeCall class
void NativeCall::test() {
#ifdef ASSERT
  ResourceMark rm;
  CodeBuffer* cb = new CodeBuffer(100, 100, 0, 0, false);
  MacroAssembler* a = new MacroAssembler(cb);
  NativeCall  *nc;
  int idx;
  int offsets[] = {
    0x0,
    0xfffffff0,
    0x7ffffff0,
    0x80000000,
    0x20,
    0x4000,
  };

  VM_Version::allow_all();

  a->call( a->pc(), relocInfo::none );
  a->delayed()->nop();
  nc = nativeCall_at( cb->code_begin() );
  nc->print();

  nc = nativeCall_overwriting_at( nc->next_instruction_address() );
  for (idx=0; idx < (sizeof(offsets) / sizeof(int)); idx++)
  {
    nc->set_destination( cb->code_begin() + offsets[idx] );
    assert(nc->destination() == (cb->code_begin() + offsets[idx]), "check unit test");
    nc->print();
  }

  nc = nativeCall_before( cb->code_begin() + 8 );
  nc->print();

  VM_Version::revert();
#endif
}
// End code for unit testing implementation of NativeCall class

//-------------------------------------------------------------------


void NativeMovConstReg::verify() {
  NativeInstruction::verify();
  // make sure code pattern is actually a "set_oop" synthetic instruction
  // see MacroAssembler::set_oop()
  int i0 = long_at(sethi_offset);
  int i1 = long_at(add_offset);

  // verify the pattern "sethi %hi22(imm), reg ;  add reg, %lo10(imm), reg"
  Register rd = inv_rd(i0);
  if (!(is_op2(i0, Assembler::sethi_op2) && rd != G0 &&
	is_op3(i1, Assembler::add_op3, Assembler::arith_op) &&
	inv_immed(i1) && (unsigned)get_simm13(i1) < (1 << 10) &&
	rd == inv_rs1(i1) && rd == inv_rd(i1)))
    fatal("not a set_oop");
}


void NativeMovConstReg::print() {
  tty->print_cr("0x%x: mov reg, 0x%x", instruction_address(), data());
}


int NativeMovConstReg::data() const {
  return data32(long_at(sethi_offset), long_at(add_offset));
}


void NativeMovConstReg::set_data(int x) {
  set_long_at(sethi_offset, set_data32_sethi(  long_at(sethi_offset), x));
  set_long_at(add_offset,   set_data32_simm13( long_at(add_offset),   x));

#ifndef CORE
  // also store the value into an oop_Relocation cell, if any
  CodeBlob* nm = CodeCache::find_blob(instruction_address());
  if (nm != NULL) {
    RelocIterator iter(nm, instruction_address(), next_instruction_address());
    oop* oop_addr = NULL;
    while (iter.next()) {
      if (iter.type() == relocInfo::oop_type) {
	oop_Relocation *r = iter.oop_reloc();
	if (oop_addr == NULL) {
	  oop_addr = r->oop_addr();
	  *oop_addr = (oop)x;
	} else {
	  assert(oop_addr == r->oop_addr(), "must be only one set-oop here");
	}
      }
    }
  }
#endif
}


// Code for unit testing implementation of NativeMovConstReg class
void NativeMovConstReg::test() {
#ifdef ASSERT
  ResourceMark rm;
  CodeBuffer* cb = new CodeBuffer(100, 100, 0, 0, false);
  MacroAssembler* a = new MacroAssembler(cb);
  NativeMovConstReg* nm;
  int idx;
  int offsets[] = {
    0x0,
    0x7fffffff,
    0x80000000,
    0xffffffff,
    0x20,
    4096,
    4097,
  };

  VM_Version::allow_all();

  a->sethi(0xaaaabbbb, I3, RelocationHolder::none);
  a->add(I3, low10(0xaaaabbbb), I3);
  a->sethi(0xccccdddd, O2, RelocationHolder::none);
  a->add(O2, low10(0xccccdddd), O2);

  nm = nativeMovConstReg_at( cb->code_begin() );
  nm->print();

  nm = nativeMovConstReg_at( nm->next_instruction_address() );
  for (idx=0; idx < (sizeof(offsets) / sizeof(int)); idx++)
  {
    nm->set_data( offsets[idx] );
    assert(nm->data() == offsets[idx], "check unit test");
  }
  nm->print();

  VM_Version::revert();
#endif
}
// End code for unit testing implementation of NativeMovConstReg class

//-------------------------------------------------------------------

void NativeMovConstRegPatching::verify() {
  NativeInstruction::verify();
  // Make sure code pattern is sethi/nop/add.
  int i0 = long_at(sethi_offset);
  int i1 = long_at(nop_offset);
  int i2 = long_at(add_offset);

  // Verify the pattern "sethi %hi22(imm), reg; nop; add reg, %lo10(imm), reg"
  // The casual reader should note that on Sparc a nop is a special case if sethi
  // in which the destination register is %g0.
  Register rd0 = inv_rd(i0);
  Register rd1 = inv_rd(i1);
  if (!(is_op2(i0, Assembler::sethi_op2) && rd0 != G0 &&
	is_op2(i1, Assembler::sethi_op2) && rd1 == G0 &&	// nop is a special case of sethi
	is_op3(i2, Assembler::add_op3, Assembler::arith_op) &&
	inv_immed(i2) && (unsigned)get_simm13(i2) < (1 << 10) &&
	rd0 == inv_rs1(i2) && rd0 == inv_rd(i2)))
    fatal("not a set_oop");
}


void NativeMovConstRegPatching::print() {
  tty->print_cr("0x%x: mov reg, 0x%x", instruction_address(), data());
}


int NativeMovConstRegPatching::data() const {
  return data32(long_at(sethi_offset), long_at(add_offset));
}


void NativeMovConstRegPatching::set_data(int x) {
  set_long_at(sethi_offset, set_data32_sethi(long_at(sethi_offset), x));
  set_long_at(add_offset, set_data32_simm13(long_at(add_offset), x));

#ifndef CORE
  // also store the value into an oop_Relocation cell, if any
  CodeBlob* nm = CodeCache::find_blob(instruction_address());
  if (nm != NULL) {
    RelocIterator iter(nm, instruction_address(), next_instruction_address());
    oop* oop_addr = NULL;
    while (iter.next()) {
      if (iter.type() == relocInfo::oop_type) {
	oop_Relocation *r = iter.oop_reloc();
	if (oop_addr == NULL) {
	  oop_addr = r->oop_addr();
	  *oop_addr = (oop)x;
	} else {
	  assert(oop_addr == r->oop_addr(), "must be only one set-oop here");
	}
      }
    }
  }
#endif
}


// Code for unit testing implementation of NativeMovConstRegPatching class
void NativeMovConstRegPatching::test() {
#ifdef ASSERT
  ResourceMark rm;
  CodeBuffer* cb = new CodeBuffer(100, 100, 0, 0, false);
  MacroAssembler* a = new MacroAssembler(cb);
  NativeMovConstRegPatching* nm;
  int idx;
  int offsets[] = {
    0x0,
    0x7fffffff,
    0x80000000,
    0xffffffff,
    0x20,
    4096,
    4097,
  };

  VM_Version::allow_all();

  a->sethi(0xaaaabbbb, I3, RelocationHolder::none);
  a->nop();
  a->add(I3, low10(0xaaaabbbb), I3);
  a->sethi(0xccccdddd, O2, RelocationHolder::none);
  a->nop();
  a->add(O2, low10(0xccccdddd), O2);

  nm = nativeMovConstRegPatching_at( cb->code_begin() );
  nm->print();

  nm = nativeMovConstRegPatching_at( nm->next_instruction_address() );
  for (idx=0; idx < (sizeof(offsets) / sizeof(int)); idx++)
  {
    nm->set_data( offsets[idx] );
    assert(nm->data() == offsets[idx], "check unit test");
  }
  nm->print();

  VM_Version::revert();
#endif
}
// End code for unit testing implementation of NativeMovConstRegPatching class

//-------------------------------------------------------------------

void NativeMovRegMem::copy_instruction_to(address new_instruction_address) {
  Untested("copy_instruction_to");
  int instruction_size = next_instruction_address() - instruction_address();
  for (int i = 0; i < instruction_size; i += wordSize) {
    *(long*)(new_instruction_address + i) = *(long*)(address(this) + i);
  }
}

void NativeMovRegMem::verify() {
  NativeInstruction::verify();
  // make sure code pattern is actually a "ld" or "st" of some sort.
  int i0 = long_at(0);
  int op3 = inv_op3(i0);

  if (!(is_op(i0, Assembler::ldst_op) &&
	inv_immed(i0) &&
	0 != (op3 < op3_ldst_int_limit
	 ? (1 <<  op3                      ) & (op3_mask_ld  | op3_mask_st)
	 : (1 << (op3 - op3_ldst_int_limit)) & (op3_mask_ldf | op3_mask_stf))))
  {
    int i1 = long_at(ldst_offset);
    Register rd = inv_rd(i0);

    op3 = inv_op3(i1);
    if (!is_op(i1, Assembler::ldst_op) && rd == inv_rs2(i1) && 
	 0 != (op3 < op3_ldst_int_limit
	      ? (1 <<  op3                      ) & (op3_mask_ld  | op3_mask_st)
	      : (1 << (op3 - op3_ldst_int_limit)) & (op3_mask_ldf | op3_mask_stf)))
      fatal("not a ld* or st* op");
  }
}


void NativeMovRegMem::print() {
  if (is_immediate())
    tty->print_cr("0x%x: mov reg, [reg + %x]", instruction_address(), offset());
  else
    tty->print_cr("0x%x: mov reg, [reg + reg]", instruction_address());
}

// Code for unit testing implementation of NativeMovRegMem class
void NativeMovRegMem::test() {
#ifdef ASSERT
  ResourceMark rm;
  CodeBuffer* cb = new CodeBuffer(1000, 1000, 0, 0, false);
  MacroAssembler* a = new MacroAssembler(cb);
  NativeMovRegMem* nm;
  int idx = 0;
  int idx1;
  int offsets[] = {
    0x0,
    0xffffffff,
    0x7fffffff,
    0x80000000,
    4096,
    4097,
    0x20,
    0x4000,
  };

  VM_Version::allow_all();

  a->ldsw( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldsw( G5, I3, G4 ); idx++;
  a->ldsb( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldsb( G5, I3, G4 ); idx++;
  a->ldsh( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldsh( G5, I3, G4 ); idx++;
  a->lduw( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->lduw( G5, I3, G4 ); idx++;
  a->ldub( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldub( G5, I3, G4 ); idx++;
  a->lduh( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->lduh( G5, I3, G4 ); idx++;
  a->ldx( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldx( G5, I3, G4 ); idx++;
  a->ldd( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldd( G5, I3, G4 ); idx++;
  a->ldf( FloatRegister::D, O2, -1, F14 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldf( FloatRegister::S, O0, I3, F15 ); idx++;

  a->stw( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->stw( G5, G4, I3 ); idx++;
  a->stb( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->stb( G5, G4, I3 ); idx++;
  a->sth( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->sth( G5, G4, I3 ); idx++;
  a->stx( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->stx( G5, G4, I3 ); idx++;
  a->std( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->std( G5, G4, I3 ); idx++;
  a->stf( FloatRegister::S, F18, O2, -1 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->add(I3, low10(0xaaaabbbb), I3);
  a->stf( FloatRegister::S, F15, O0, I3 ); idx++;

  nm = nativeMovRegMem_at( cb->code_begin() );
  nm->print();
  nm->set_offset( low10(0) );
  nm->print();
  nm->add_offset_in_words( low10(0xbb) );
  nm->print();

  while (--idx) {
    nm = nativeMovRegMem_at( nm->next_instruction_address() );
    nm->print();
    for (idx1 = 0; idx1 < (sizeof(offsets) / sizeof(int)); idx1++)
    {
      nm->set_offset( nm->is_immediate() ? low10(offsets[idx1]) : offsets[idx1] );
      assert(nm->offset() == (nm->is_immediate() ? low10(offsets[idx1]) : offsets[idx1]),
             "check unit test");
      nm->print();
    }
    nm->add_offset_in_words( low10(0xbb) );
    nm->print();
  }

  VM_Version::revert();
#endif
}
// End code for unit testing implementation of NativeMovRegMem class

//--------------------------------------------------------------------------------

void NativeMovRegMemPatching::copy_instruction_to(address new_instruction_address) {
  Untested("copy_instruction_to");
  int instruction_size = next_instruction_address() - instruction_address();
  for (int i = 0; i < instruction_size; i += wordSize) {
    *(long*)(new_instruction_address + i) = *(long*)(address(this) + i);
  }
}

void NativeMovRegMemPatching::verify() {
  NativeInstruction::verify();
  // make sure code pattern is actually a "ld" or "st" of some sort.
  int i0 = long_at(0);
  int op3 = inv_op3(i0);

  if (!(is_op(i0, Assembler::ldst_op) &&
	inv_immed(i0) &&
	0 != (op3 < op3_ldst_int_limit
	 ? (1 <<  op3                      ) & (op3_mask_ld  | op3_mask_st)
	 : (1 << (op3 - op3_ldst_int_limit)) & (op3_mask_ldf | op3_mask_stf))))
  {
    int i1 = long_at(ldst_offset);
    Register rd = inv_rd(i0);

    op3 = inv_op3(i1);
    if (!is_op(i1, Assembler::ldst_op) && rd == inv_rs2(i1) && 
	 0 != (op3 < op3_ldst_int_limit
	      ? (1 <<  op3                      ) & (op3_mask_ld  | op3_mask_st)
	      : (1 << (op3 - op3_ldst_int_limit)) & (op3_mask_ldf | op3_mask_stf)))
      fatal("not a ld* or st* op");
  }
}


void NativeMovRegMemPatching::print() {
  if (is_immediate())
    tty->print_cr("0x%x: mov reg, [reg + %x]", instruction_address(), offset());
  else
    tty->print_cr("0x%x: mov reg, [reg + reg]", instruction_address());
}

// Code for unit testing implementation of NativeMovRegMemPatching class
void NativeMovRegMemPatching::test() {
#ifdef ASSERT
  ResourceMark rm;
  CodeBuffer* cb = new CodeBuffer(1000, 1000, 0, 0, false);
  MacroAssembler* a = new MacroAssembler(cb);
  NativeMovRegMemPatching* nm;
  int idx = 0;
  int idx1;
  int offsets[] = {
    0x0,
    0xffffffff,
    0x7fffffff,
    0x80000000,
    4096,
    4097,
    0x20,
    0x4000,
  };

  VM_Version::allow_all();

  a->ldsw( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldsw( G5, I3, G4 ); idx++;
  a->ldsb( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldsb( G5, I3, G4 ); idx++;
  a->ldsh( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldsh( G5, I3, G4 ); idx++;
  a->lduw( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->lduw( G5, I3, G4 ); idx++;
  a->ldub( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldub( G5, I3, G4 ); idx++;
  a->lduh( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->lduh( G5, I3, G4 ); idx++;
  a->ldx( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldx( G5, I3, G4 ); idx++;
  a->ldd( G5, low10(0xffffffff), G4 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldd( G5, I3, G4 ); idx++;
  a->ldf( FloatRegister::D, O2, -1, F14 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->ldf( FloatRegister::S, O0, I3, F15 ); idx++;

  a->stw( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->stw( G5, G4, I3 ); idx++;
  a->stb( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->stb( G5, G4, I3 ); idx++;
  a->sth( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->sth( G5, G4, I3 ); idx++;
  a->stx( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->stx( G5, G4, I3 ); idx++;
  a->std( G5, G4, low10(0xffffffff) ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->std( G5, G4, I3 ); idx++;
  a->stf( FloatRegister::S, F18, O2, -1 ); idx++;
  a->sethi(0xaaaabbbb, I3, RelocationHolder::none); a->nop(); a->add(I3, low10(0xaaaabbbb), I3);
  a->stf( FloatRegister::S, F15, O0, I3 ); idx++;

  nm = nativeMovRegMemPatching_at( cb->code_begin() );
  nm->print();
  nm->set_offset( low10(0) );
  nm->print();
  nm->add_offset_in_words( low10(0xbb) );
  nm->print();

  while (--idx) {
    nm = nativeMovRegMemPatching_at( nm->next_instruction_address() );
    nm->print();
    for (idx1 = 0; idx1 < (sizeof(offsets) / sizeof(int)); idx1++)
    {
      nm->set_offset( nm->is_immediate() ? low10(offsets[idx1]) : offsets[idx1] );
      assert(nm->offset() == (nm->is_immediate() ? low10(offsets[idx1]) : offsets[idx1]),
             "check unit test");
      nm->print();
    }
    nm->add_offset_in_words( low10(0xbb) );
    nm->print();
  }

  VM_Version::revert();
#endif
}
// End code for unit testing implementation of NativeMovRegMemPatching class

//--------------------------------------------------------------------------------

void NativeJump::verify() {
  NativeInstruction::verify();
  int i0 = long_at(sethi_offset);
  int i1 = long_at(jmpl_offset);
  // verify the pattern "sethi %hi22(imm), treg ;  jmpl treg, %lo10(imm), lreg"
  Register rd = inv_rd(i0);
  if (!(is_op2(i0, Assembler::sethi_op2) && rd != G0 &&
	is_op3(i1, Assembler::jmpl_op3, Assembler::arith_op) &&
	inv_immed(i1) && (unsigned)get_simm13(i1) < (1 << 10) &&
	rd == inv_rs1(i1)))
    fatal("not a jump_to instruction");  
}

void NativeJump::print() {
  tty->print_cr("0x%x: jmpl reg, 0x%x", instruction_address(), jump_destination());
}

// Code for unit testing implementation of NativeJump class
void NativeJump::test() {
#ifdef ASSERT
  ResourceMark rm;
  CodeBuffer* cb = new CodeBuffer(100, 100, 0, 0, false);
  MacroAssembler* a = new MacroAssembler(cb);
  NativeJump* nj;
  int idx;
  int offsets[] = {
    0x0,
    0xffffffff,
    0x7fffffff,
    0x80000000,
    4096,
    4097,
    0x20,
    0x4000,
  };

  VM_Version::allow_all();

  a->sethi(0x7fffbbbb, I3, RelocationHolder::none);
  a->jmpl(I3, low10(0x7fffbbbb), G0, RelocationHolder::none);
  a->delayed()->nop();
  a->sethi(0x7fffbbbb, I3, RelocationHolder::none);
  a->jmpl(I3, low10(0x7fffbbbb), L3, RelocationHolder::none);
  a->delayed()->nop();

  nj = nativeJump_at( cb->code_begin() );
  nj->print();

  nj = nativeJump_at( nj->next_instruction_address() + 4 );
  for (idx=0; idx < (sizeof(offsets) / sizeof(int)); idx++)
  {
    nj->set_jump_destination( nj->instruction_address() + offsets[idx] );
    assert(nj->jump_destination() == (nj->instruction_address() + offsets[idx]), "check unit test");
    nj->print();
  }

  VM_Version::revert();
#endif
}
// End code for unit testing implementation of NativeJump class


void NativeJump::insert(address code_pos, address entry) {
  Unimplemented();
}

// MT safe inserting of a jump over an unknown instruction sequence (used by nmethod::makeZombie)
// The problem: jump_to <dest> is a 3-word instruction (including its delay slot).
// Atomic write can be only with 1 word.
void NativeJump::patch_verified_entry(address entry, address verified_entry, address dest) {
  // Here's one way to do it:  Pre-allocate a three-word jump sequence somewhere
  // in the header of the nmethod, within a short branch's span of the patch point.
  // Set up the jump sequence using NativeJump::insert, and then use an annulled
  // unconditional branch at the target site (an atomic 1-word update).
  // Limitations:  You can only patch nmethods, with any given nmethod patched at
  // most once, and the patch must be in the nmethod's header.
  // It's messy, but you can ask the CodeCache for the nmethod containing the
  // target address (see is_call_to_stub for a similar use of CodeCache).

  // %%%%% For now, do something MT-stupid:
  ResourceMark rm;
  int code_size = 1 * wordSize;
  CodeBuffer* cb = new CodeBuffer(verified_entry, code_size + 1);
  MacroAssembler* a = new MacroAssembler(cb);
#ifdef COMPILER2
  assert(dest == OptoRuntime::handle_wrong_method_stub(), "The segfault always takes us to OptoRuntime::handle_wrong_method_stub");
#endif
  a->ld(G0, 0, O7);
  ICache::invalidate_range(verified_entry, code_size);
}


void NativeIllegalInstruction::insert(address code_pos) {
  NativeIllegalInstruction* nii = (NativeIllegalInstruction*) nativeInstruction_at(code_pos);
  nii->set_long_at(0, illegal_instruction());
}

static int illegal_instruction_bits = 0;

int NativeInstruction::illegal_instruction() {
  if (illegal_instruction_bits == 0) {
    ResourceMark rm;
    CodeBuffer* cb = new CodeBuffer(20, 20, 0, 0, false);
    MacroAssembler* a = new MacroAssembler(cb);
    address ia = a->pc();
    a->trap(ST_RESERVED_FOR_USER_0 + 1);
    jint bits = *(jint*)ia;
    assert(is_op3(bits, Assembler::trap_op3, Assembler::arith_op), "bad instruction");
    illegal_instruction_bits = bits;
    assert(illegal_instruction_bits != 0, "oops");
  }
  return illegal_instruction_bits;
}

bool NativeInstruction::is_illegal() {
  if (illegal_instruction_bits == 0)
    return false;
  return long_at(0) == illegal_instruction_bits;
}


void NativeGeneralJump::verify() {
  assert(((NativeInstruction *)this)->is_jump() ||
	 ((NativeInstruction *)this)->is_cond_jump(), "not a general jump instruction");
}

//Reconciliation History
// 1.3 97/09/11 17:15:11 nativeInst_i486.cpp
// 1.6 97/10/16 23:44:02 nativeInst_i486.cpp
// 1.7 97/10/21 10:41:49 nativeInst_i486.cpp
// 1.9 97/11/12 12:13:53 nativeInst_i486.cpp
// 1.11 97/12/04 18:26:40 nativeInst_i486.cpp
// 1.13 97/12/17 09:08:56 nativeInst_i486.cpp
// 1.14 98/01/11 16:10:01 nativeInst_i486.cpp
// 1.16 98/02/03 17:56:07 nativeInst_i486.cpp
// 1.18 98/02/28 11:36:21 nativeInst_i486.cpp
// 1.20 98/03/05 17:17:12 nativeInst_i486.cpp
// 1.22 98/04/23 09:39:11 nativeInst_i486.cpp
// 1.22 98/04/23 09:39:11 nativeInst_i486.cpp
// 1.23 98/04/28 18:06:29 nativeInst_i486.cpp
// 1.24 98/05/11 16:35:56 nativeInst_i486.cpp
// 1.25 98/05/12 14:28:39 nativeInst_i486.cpp
// 1.26 98/05/24 18:03:04 nativeInst_i486.cpp
// 1.27 98/06/17 15:58:58 nativeInst_i486.cpp
// 1.28 98/06/23 14:23:12 nativeInst_i486.cpp
// 1.29 98/07/10 13:25:13 nativeInst_i486.cpp
// 1.30 98/08/27 14:42:32 nativeInst_i486.cpp
// 1.31 98/10/05 22:30:54 nativeInst_i486.cpp
// 1.32 98/11/06 11:18:17 nativeInst_i486.cpp
// 1.33 98/11/11 08:47:39 nativeInst_i486.cpp
// 1.36 99/06/28 11:00:57 nativeInst_i486.cpp
//End
