/*
 * @(#)nativeInst_sparc.hpp	1.50 01/02/09 14:43:28
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// We have interface for the following instructions:
// - NativeInstruction
// - - NativeCall
// - - NativeMovConstReg
// - - NativeMovConstRegPatching
// - - NativeMovRegMem
// - - NativeMovRegMemPatching
// - - NativeJump
// - - NativeIllegalOpCode
// The base class for different kinds of native instruction abstractions.
// Provides the primitive operations to manipulate code relative to this.

class NativeInstruction VALUE_OBJ_CLASS_SPEC {
  friend class Relocation;

 public:
  enum Sparc_specific_constants {
    nop_instruction_size        =    4
  };

  bool is_nop()                        { return long_at(0) == nop_instruction(); }
  bool is_call()                       { return is_op(long_at(0), Assembler::call_op); }
  bool sets_cc() {
    // conservative (returns true for some instructions that do not set the
    // the condition code, such as, "save".
    // Does not return true for the deprecated tagged instructions, such as, TADDcc
    long x = long_at(0);
    return (is_op(x, Assembler::arith_op) &&
            (inv_op3(x) & Assembler::cc_bit_op3) == Assembler::cc_bit_op3);
  }
  bool is_illegal();
  bool is_zombie() {
    long x = long_at(0);
    return is_op3(x, Assembler::lduw_op3, Assembler::ldst_op)
        && Assembler::inv_rs1(x) == G0
        && Assembler::inv_rd(x) == O7;
  }
  bool is_useless_load(NativeInstruction *prev_inst) {
    long x = long_at(0);
    long y = *(long*)prev_inst;

    if (inv_op(x) != Assembler::ldst_op ||
        inv_op(y) != Assembler::ldst_op)
      return false;

    if (inv_op3(x) == Assembler::lduw_op3 &&
        inv_op3(y) == Assembler::stw_op3)
      ;
    else
      return false;
    if (!(inv_rd(x) == inv_rd(y) &&
          inv_rs1(x) == inv_rs1(y) &&
          inv_immed(x) == inv_immed(y)))
      return false;

    if (inv_immed(x) &&
        inv_simm(x, 13) != inv_simm(y, 13))
      return false;

    if (!inv_immed(x) &&
        inv_rs2(x) != inv_rs2(y))
      return false;
    return true;
  }

  bool is_return() {
    // is it the output of MacroAssembler::ret or MacroAssembler::retl?
    long x = long_at(0);
    const int nWords = 2;	// displacement used by ret/retl
    return is_op3(x, Assembler::jmpl_op3, Assembler::arith_op)
	&& (inv_rs1(x) == I7 || inv_rs1(x) == O7)
	&& inv_immed(x) && inv_simm(x, 13) == nWords * BytesPerWord
	&& inv_rd(x) == G0;
  }
  bool is_int_jump() {
    // is it the output of MacroAssembler::b?
    long x = long_at(0);
    return is_op2(x, Assembler::bp_op2) || is_op2(x, Assembler::br_op2);
  }
  bool is_float_jump() {
    // is it the output of MacroAssembler::fb?
    long x = long_at(0);
    return is_op2(x, Assembler::fbp_op2) || is_op2(x, Assembler::fb_op2);
  }
  bool is_jump() {
    return is_int_jump() || is_float_jump();
  }
  bool is_cond_jump() {
    long x = long_at(0);
    return (is_int_jump() && Assembler::inv_cond(x) != Assembler::always) ||
           (is_float_jump() && Assembler::inv_cond(x) != Assembler::f_always);
  }

  bool is_stack_bang() {
    long x = long_at(0);
    return is_op3(x, Assembler::stw_op3, Assembler::ldst_op) &&
      (inv_rd(x) == G0) && (inv_rs1(x) == SP) && (inv_rs2(x) == Gscratch); 
  }

  bool is_zero_test(Register &reg);
  bool is_load_store_with_small_offset(Register reg);

 public:
  static int nop_instruction()         { return Assembler::op(Assembler::branch_op) | Assembler::op2(Assembler::sethi_op2); }
  static int illegal_instruction();    // the output of __ breakpoint_trap()
  static int call_instruction(address destination, address pc) { return Assembler::op(Assembler::call_op) | Assembler::wdisp((int)destination, (int)pc, 30); }

  static int branch_instruction(Assembler::op2s op2val, Assembler::Condition c, bool a) {
    return Assembler::op(Assembler::branch_op) | Assembler::op2(op2val) | Assembler::annul(a) | Assembler::cond(c);
  }

  static int op3_instruction(Assembler::ops opval, Register rd, Assembler::op3s op3val, Register rs1, int simm13a) {
    return Assembler::op(opval) | Assembler::rd(rd) | Assembler::op3(op3val) | Assembler::rs1(rs1) | Assembler::immed(true) | Assembler::simm(simm13a, 13);
  }

  static int sethi_instruction(Register rd, int imm22a) {
    return Assembler::op(Assembler::branch_op) | Assembler::rd(rd) | Assembler::op2(Assembler::sethi_op2) | Assembler::hi22(imm22a);
  }

 protected:
  address  addr_at(int offset) const    { return address(this) + offset; }
  int      long_at(int offset) const    { return *(jint*)addr_at(offset); }
  void set_long_at(int offset, int  i);	/* deals with I-cache */

  static bool is_op( int x, Assembler::ops opval)  {
    return Assembler::inv_op(x) == opval;
  }
  static bool is_op2(int x, Assembler::op2s op2val) {
    return Assembler::inv_op(x) == Assembler::branch_op && Assembler::inv_op2(x) == op2val;
  }
  static bool is_op3(int x, Assembler::op3s op3val, Assembler::ops opval) {
    return Assembler::inv_op(x) == opval && Assembler::inv_op3(x) == op3val;
  }

  // utilities to help subclasses decode:
  static Register inv_rd(  int x ) { return Assembler::inv_rd( x); }
  static Register inv_rs1( int x ) { return Assembler::inv_rs1(x); }
  static Register inv_rs2( int x ) { return Assembler::inv_rs2(x); }

  static bool inv_immed( int x ) { return Assembler::inv_immed(x); }
  static bool inv_annul( int x ) { return (Assembler::annul(true) & x) != 0; }
  static int  inv_cond(  int x ) { return Assembler::inv_cond(x); }

  static int inv_op(  int x ) { return Assembler::inv_op( x); }
  static int inv_op2( int x ) { return Assembler::inv_op2(x); } 
  static int inv_op3( int x ) { return Assembler::inv_op3(x); } 

  static int inv_simm(    int x, int nbits ) { return Assembler::inv_simm(x, nbits); }
  static int inv_wdisp(   int x, int nbits ) { return Assembler::inv_wdisp(  x, 0, nbits); }
  static int inv_wdisp16( int x )            { return Assembler::inv_wdisp16(x, 0); }
  static int branch_destination_offset(int x) { return Assembler::branch_destination(x, 0); }
  static int patch_branch_destination_offset(int dest_offset, int x) {
    return Assembler::patched_branch(dest_offset, x, 0);
  }
  void set_annul_bit() { set_long_at(0, long_at(0) | Assembler::annul(true)); }

  // utility for checking if x is either of 2 small constants
  static bool is_either(int x, int k1, int k2) {
    // return x == k1 || x == k2;
    return (1 << x) & (1 << k1 | 1 << k2);
  }

  // utility for checking overflow of signed instruction fields
  static bool fits_in_simm(int x, int nbits) {
    // cf. Assembler::assert_signed_range()
    // return -(1 << nbits-1) <= x  &&  x < ( 1 << nbits-1),
    return (unsigned)(x + (1 << nbits-1)) < (unsigned)(1 << nbits);
  }

  // set a signed immediate field
  static int set_simm(int insn, int imm, int nbits) {
    return (insn &~ Assembler::simm(-1, nbits)) | Assembler::simm(imm, nbits);
  }

  // set a wdisp field (disp should be the difference of two addresses)
  static int set_wdisp(int insn, int disp, int nbits) {
    return (insn &~ Assembler::wdisp(-4, 0, nbits)) | Assembler::wdisp(disp, 0, nbits);
  }

  static int set_wdisp16(int insn, int disp) {
    return (insn &~ Assembler::wdisp16(-4, 0)) | Assembler::wdisp16(disp, 0);
  }

  // get a simm13 field from an arithmetic or memory instruction
  static bool get_simm13(int insn) {
    assert(is_either(Assembler::inv_op(insn),
		     Assembler::arith_op, Assembler::ldst_op) &&
	    (insn & Assembler::immed(true)), "must have a simm13 field");
    return Assembler::inv_simm(insn, 13);
  }

  // set the simm13 field of an arithmetic or memory instruction
  static bool set_simm13(int insn, int imm) {
    get_simm13(insn);		// tickle the assertion check
    return set_simm(insn, imm, 13);
  }

  // combine the fields of a sethi/simm13 pair (simm13 = or, add, jmpl, ld/st)
  static int data32(int sethi_insn, int arith_insn) {
    assert(is_op2(sethi_insn, Assembler::sethi_op2), "must be sethi");
    int hi = Assembler::inv_hi22(sethi_insn);
    int lo = get_simm13(arith_insn);
    assert((unsigned)lo < (1 << 10), "offset field of set_oop must be 10 bits");
    return hi | lo;
  }

  static int set_data32_sethi(int sethi_insn, int imm) {
    // note that Assembler::hi22 clips the low 10 bits for us
    assert(is_op2(sethi_insn, Assembler::sethi_op2), "must be sethi");
    return (sethi_insn &~ Assembler::hi22(-1)) | Assembler::hi22(imm);
  }

  static int set_data32_simm13(int arith_insn, int imm) {
    get_simm13(arith_insn);		// tickle the assertion check
    int imm10 = Assembler::low10(imm);
    return (arith_insn &~ Assembler::simm(-1, 13)) | Assembler::simm(imm10, 13);
  }

  static int low10(int imm) {
    return Assembler::low10(imm);
  }

 public:

  void  verify();
  void  print();

  // unit test stuff
  static void test() {}			// override for testing

  friend NativeInstruction* nativeInstruction_at(address address) {
    NativeInstruction* inst = (NativeInstruction*)address;
    #ifdef ASSERT
      inst->verify();
    #endif
    return inst;
  }

};

//-----------------------------------------------------------------------------

// The NativeCall is an abstraction for accessing/manipulating native call imm32 instructions.
// (used to manipulate inline caches, primitive & dll calls, etc.)
class NativeCall: public NativeInstruction {
 public:
  enum Sparc_specific_constants {
    instruction_size                   = 8,
    return_address_offset              = 8,
    displacement_offset                = 0,
    instruction_offset                 = 0,
    call_displacement_width	       = 30
  };
  address instruction_address() const       { return addr_at(0); }
  address next_instruction_address() const  { return addr_at(instruction_size); }
  address return_address() const            { return addr_at(return_address_offset); }
  address destination() const               { return inv_wdisp(long_at(0), call_displacement_width) + instruction_address(); }
  address displacement_address() const      { return addr_at(displacement_offset); }
  void  set_destination(address dest)       { set_long_at(0, set_wdisp(long_at(0), dest - instruction_address(), call_displacement_width)); }

  bool destination_is_compiled_verified_entry_point();

  bool is_call_to_stub() const;   // returns true if calls to stub (destination in same nmethod as call)

  void  verify();
  void  print();

  // unit test stuff
  static void  test();
 
  // Creation
  friend NativeCall* nativeCall_at(address address) {
    NativeCall* call = (NativeCall*)address;
    #ifdef ASSERT
      call->verify();
    #endif
    return call;
  }

  friend NativeCall* nativeCall_overwriting_at(address address, address destination = NULL) {
    // insert a "blank" call:
    NativeCall* call = (NativeCall*)address;
    call->set_long_at(0 * wordSize, call_instruction(destination, address));
    call->set_long_at(1 * wordSize, nop_instruction());
    assert(call->addr_at(2 * wordSize) - address == instruction_size, "instruction size");
    // check its structure now:
    assert(nativeCall_at(address)->destination() == destination, "correct call destination");
    return call;
  }

  friend NativeCall* nativeCall_before(address return_address) {
    NativeCall* call = (NativeCall*)(return_address - return_address_offset);
    #ifdef ASSERT
      call->verify();
    #endif
    return call;
  }

  static bool is_call_at(address instr);

  // MT-safe patching of a call instruction.
  static void insert(address code_pos, address entry);
  static void replace_mt_safe(address instr_addr, address code_buffer);  
};


// An interface for accessing/manipulating native set_oop imm, reg instructions.
// (used to manipulate inlined data references, etc.)
//      set_oop imm, reg
//	== sethi %hi22(imm), reg ;  add reg, %lo10(imm), reg
class NativeMovConstReg: public NativeInstruction {
 public:
  enum Sparc_specific_constants {
    sethi_offset           = 0,
    add_offset             = 4,
    instruction_size       = 8
  };

  address instruction_address() const       { return addr_at(0); }
  address next_instruction_address() const  { return addr_at(instruction_size); }

  // (The [set_]data accessor respects oop_type relocs also.)
  int data() const;
  void  set_data(int x);

  // report the destination register
  Register destination() { return inv_rd(long_at(sethi_offset)); }

  void  verify();
  void  print();
  
  // unit test stuff
  static void test();

  // Creation
  friend NativeMovConstReg* nativeMovConstReg_at(address address) {
    NativeMovConstReg* test = (NativeMovConstReg*)address;
    #ifdef ASSERT
      test->verify();
    #endif
    return test;
  }


  friend NativeMovConstReg* nativeMovConstReg_before(address address) {
    NativeMovConstReg* test = (NativeMovConstReg*)(address - instruction_size);
    #ifdef ASSERT
      test->verify();
    #endif
    return test;
  }

};


// An interface for accessing/manipulating native set_oop imm, reg instructions.
// (used to manipulate inlined data references, etc.)
//      set_oop imm, reg
//	== sethi %hi22(imm), reg; nop; add reg, %lo10(imm), reg
//
// Note that it is identical to NativeMovConstReg with the exception of a nop between the
// sethi and the add.  The nop is required to be in the delay slot of the call instruction
// which overwrites the sethi during patching.
class NativeMovConstRegPatching: public NativeInstruction {
 public:
  enum Sparc_specific_constants {
    sethi_offset           = 0,
    nop_offset             = 4,
    add_offset             = 8,
    instruction_size       = 12
  };

  address instruction_address() const       { return addr_at(0); }
  address next_instruction_address() const  { return addr_at(instruction_size); }

  // (The [set_]data accessor respects oop_type relocs also.)
  int data() const;
  void  set_data(int x);

  // report the destination register
  Register destination() { return inv_rd(long_at(sethi_offset)); }

  void  verify();
  void  print();
  
  // unit test stuff
  static void test();

  // Creation
  friend NativeMovConstRegPatching* nativeMovConstRegPatching_at(address address) {
    NativeMovConstRegPatching* test = (NativeMovConstRegPatching*)address;
    #ifdef ASSERT
      test->verify();
    #endif
    return test;
  }


  friend NativeMovConstRegPatching* nativeMovConstRegPatching_before(address address) {
    NativeMovConstRegPatching* test = (NativeMovConstRegPatching*)(address - instruction_size);
    #ifdef ASSERT
      test->verify();
    #endif
    return test;
  }

};


// An interface for accessing/manipulating native memory ops
//	ld* [reg + offset], reg
//      st* reg, [reg + offset]
//	sethi %hi(imm), reg; add reg, %lo(imm), reg; ld* [reg1 + reg], reg2
//	sethi %hi(imm), reg; add reg, %lo(imm), reg; st* reg2, [reg1 + reg]
// Ops covered: {lds,ldu,st}{w,b,h}, {ld,st}{d,x}
class NativeMovRegMem: public NativeInstruction {
 public:
  enum Sparc_specific_constants {
    op3_mask_ld = 1 << Assembler::lduw_op3 |
		  1 << Assembler::ldub_op3 |
		  1 << Assembler::lduh_op3 |
		  1 << Assembler::ldd_op3 |
		  1 << Assembler::ldsw_op3 |
		  1 << Assembler::ldsb_op3 |
		  1 << Assembler::ldsh_op3 |
		  1 << Assembler::ldx_op3,
    op3_mask_st = 1 << Assembler::stw_op3 |
		  1 << Assembler::stb_op3 |
		  1 << Assembler::sth_op3 |
		  1 << Assembler::std_op3 |
		  1 << Assembler::stx_op3,
    op3_ldst_int_limit = Assembler::ldf_op3,
    op3_mask_ldf = 1 << (Assembler::ldf_op3  - op3_ldst_int_limit) |
		   1 << (Assembler::lddf_op3 - op3_ldst_int_limit),
    op3_mask_stf = 1 << (Assembler::stf_op3  - op3_ldst_int_limit) |
		   1 << (Assembler::stdf_op3 - op3_ldst_int_limit),

    offset_width    = 13,
    sethi_offset    = 0,
    add_offset      = 4,
    ldst_offset	    = 8
  };
  bool is_immediate() const {
    // check if instruction is ld* [reg + offset], reg or st* reg, [reg + offset]
    int i0 = long_at(0);
    return (is_op(i0, Assembler::ldst_op));
  }

  address instruction_address() const	        { return addr_at(0); }
  address next_instruction_address() const	{ 
    return addr_at(is_immediate()? 4 : 12);
  }
  int   offset() const	                        {
     return is_immediate()? inv_simm(long_at(0), offset_width) :
                            nativeMovConstReg_at(addr_at(0))->data();
  }
  void  set_offset(int x) {
    if (is_immediate()) {
      guarantee(fits_in_simm(x, offset_width), "data block offset overflow");
      set_long_at(0, set_simm(long_at(0), x, offset_width));
    }
    else
      nativeMovConstReg_at(addr_at(0))->set_data(x);
  }

  void  add_offset_in_words(int radd_offset)     {
      set_offset ( ( offset() + radd_offset ) * wordSize);
  }
  void  copy_instruction_to(address new_instruction_address);

  void verify();
  void print ();

  // unit test stuff
  static void test();

 private:
  friend NativeMovRegMem* nativeMovRegMem_at (address address) {
    NativeMovRegMem* test = (NativeMovRegMem*)address;
    #ifdef ASSERT
      test->verify();
    #endif
    return test;
  }
};


// An interface for accessing/manipulating native memory ops
//	ld* [reg + offset], reg
//      st* reg, [reg + offset]
//	sethi %hi(imm), reg; nop; add reg, %lo(imm), reg; ld* [reg1 + reg], reg2
//	sethi %hi(imm), reg; nop; add reg, %lo(imm), reg; st* reg2, [reg1 + reg]
// Ops covered: {lds,ldu,st}{w,b,h}, {ld,st}{d,x}
//
// Note that it is identical to NativeMovRegMem with the exception of a nop between the
// sethi and the add.  The nop is required to be in the delay slot of the call instruction
// which overwrites the sethi during patching.
class NativeMovRegMemPatching: public NativeInstruction {
 public:
  enum Sparc_specific_constants {
    op3_mask_ld = 1 << Assembler::lduw_op3 |
		  1 << Assembler::ldub_op3 |
		  1 << Assembler::lduh_op3 |
		  1 << Assembler::ldd_op3 |
		  1 << Assembler::ldsw_op3 |
		  1 << Assembler::ldsb_op3 |
		  1 << Assembler::ldsh_op3 |
		  1 << Assembler::ldx_op3,
    op3_mask_st = 1 << Assembler::stw_op3 |
		  1 << Assembler::stb_op3 |
		  1 << Assembler::sth_op3 |
		  1 << Assembler::std_op3 |
		  1 << Assembler::stx_op3,
    op3_ldst_int_limit = Assembler::ldf_op3,
    op3_mask_ldf = 1 << (Assembler::ldf_op3  - op3_ldst_int_limit) |
		   1 << (Assembler::lddf_op3 - op3_ldst_int_limit),
    op3_mask_stf = 1 << (Assembler::stf_op3  - op3_ldst_int_limit) |
		   1 << (Assembler::stdf_op3 - op3_ldst_int_limit),

    offset_width    = 13,
    sethi_offset    = 0,
    nop_offset      = 4,
    add_offset      = 8,
    ldst_offset	    = 12
  };
  bool is_immediate() const {
    // check if instruction is ld* [reg + offset], reg or st* reg, [reg + offset]
    int i0 = long_at(0);
    return (is_op(i0, Assembler::ldst_op));
  }

  address instruction_address() const	        { return addr_at(0); }
  address next_instruction_address() const	{ 
    return addr_at(is_immediate()? 4 : 16);
  }
  int   offset() const	                        {
     return is_immediate()? inv_simm(long_at(0), offset_width) :
                            nativeMovConstRegPatching_at(addr_at(0))->data();
  }
  void  set_offset(int x) {
    if (is_immediate()) {
      guarantee(fits_in_simm(x, offset_width), "data block offset overflow");
      set_long_at(0, set_simm(long_at(0), x, offset_width));
    }
    else
      nativeMovConstRegPatching_at(addr_at(0))->set_data(x);
  }

  void  add_offset_in_words(int radd_offset)     {
      set_offset ( ( offset() + radd_offset ) * wordSize);
  }
  void  copy_instruction_to(address new_instruction_address);

  void verify();
  void print ();

  // unit test stuff
  static void test();

 private:
  friend NativeMovRegMemPatching* nativeMovRegMemPatching_at (address address) {
    NativeMovRegMemPatching* test = (NativeMovRegMemPatching*)address;
    #ifdef ASSERT
      test->verify();
    #endif
    return test;
  }
};



// An interface for accessing/manipulating native jumps
//      jump_to addr
//	== sethi %hi22(addr), temp ;  jumpl reg, %lo10(addr), G0 ;  <delay>
//      jumpl_to addr, lreg
//	== sethi %hi22(addr), temp ;  jumpl reg, %lo10(addr), lreg ;  <delay>
class NativeJump: public NativeInstruction {
 private:
  void guarantee_displacement(int disp, int width) {
    guarantee(fits_in_simm(disp, width + 2), "branch displacement overflow");
  }

 public:
  enum Sparc_specific_constants {
    sethi_offset           = 0,
    jmpl_offset            = 4,
    instruction_size       = 8  // does not include delay slot
  };

  address instruction_address() const       { return addr_at(0); }
  address next_instruction_address() const  { return addr_at(instruction_size); }

  address jump_destination() const {
    return (address) data32(long_at(sethi_offset), long_at(jmpl_offset));
  }
  void set_jump_destination(address dest) {
    set_long_at(sethi_offset, set_data32_sethi(  long_at(sethi_offset), (int)dest));
    set_long_at(jmpl_offset,  set_data32_simm13( long_at(jmpl_offset),  (int)dest));
  }

  // Creation
  friend NativeJump* nativeJump_at(address address) {
    NativeJump* jump = (NativeJump*)address;
    #ifdef ASSERT
      jump->verify();
    #endif
    return jump;
  }


  void verify();
  void print();

  // Unit testing stuff
  static void test();

  // Insertion of native jump instruction
  static void insert(address code_pos, address entry);
  // MT-safe insertion of native jump at verified method entry
  static void patch_verified_entry(address entry, address verified_entry, address dest);
};



// Despite the name, handles only simple branches.
class NativeGeneralJump: public NativeInstruction {
 public:
  address instruction_address() const       { return addr_at(0); }  
  address jump_destination()	const       { return addr_at(0) + branch_destination_offset(long_at(0)); }
  void set_jump_destination(address dest) {
    int patched_instr = patch_branch_destination_offset(dest - addr_at(0), long_at(0));
    set_long_at(0, patched_instr);
  }
  void set_annul() { set_annul_bit(); }
  NativeInstruction *delay_slot_instr() { return nativeInstruction_at(addr_at(4));}
  void fill_delay_slot(int instr) { set_long_at(4, instr);}
  Assembler::Condition condition() {
    long x = long_at(0);
    return (Assembler::Condition) Assembler::inv_cond(x);
  }

  // Creation
  friend NativeGeneralJump* nativeGeneralJump_at(address address) {
    NativeGeneralJump* jump = (NativeGeneralJump*)(address);
    #ifdef ASSERT
      jump->verify();
    #endif
    return jump;
  }

  void verify();
};


class NativeIllegalInstruction: public NativeInstruction {
 public:
  enum Sparc_specific_constants {
    instruction_size	        =    4
  };

  // Insert illegal opcode as specific address
  static void insert(address code_pos);
};


//Reconciliation History
// 1.10 97/09/11 17:15:10 nativeInst_i486.hpp
// 1.13 97/10/16 23:44:08 nativeInst_i486.hpp
// 1.14 97/10/21 10:41:52 nativeInst_i486.hpp
// 1.18 97/11/12 12:13:54 nativeInst_i486.hpp
// 1.23 97/12/08 10:15:59 nativeInst_i486.hpp
// 1.28 97/12/17 09:11:54 nativeInst_i486.hpp
// 1.29 98/01/11 16:10:00 nativeInst_i486.hpp
// 1.31 98/02/03 17:56:06 nativeInst_i486.hpp
// 1.30 98/02/04 02:16:11 nativeInst_i486.hpp
// 1.35 98/03/03 03:29:53 nativeInst_i486.hpp
// 1.36 98/03/10 13:46:54 nativeInst_i486.hpp
// 1.37 98/04/07 15:43:20 nativeInst_i486.hpp
// 1.38 98/08/27 14:42:33 nativeInst_i486.hpp
// 1.39 98/10/05 22:30:53 nativeInst_i486.hpp
// 1.40 99/03/11 11:01:48 nativeInst_i486.hpp
// 1.43 99/06/28 11:00:59 nativeInst_i486.hpp
// 1.44 99/07/20 10:52:04 nativeInst_i486.hpp
// 1.46 99/07/30 13:53:04 nativeInst_i486.hpp
//End
