/*
 * @(#)runtime_sparc.cpp	1.62 01/03/05 13:41:11
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_runtime_sparc.cpp.incl"

// Deoptimization

#define __ masm->

UncommonTrapBlob   *OptoRuntime::_uncommon_trap_blob;
DeoptimizationBlob *OptoRuntime::_deopt_blob;
ExceptionBlob      *OptoRuntime::_exception_blob;
SafepointBlob      *OptoRuntime::_illegal_instruction_handler_blob;

int handle_exception_deopt_offset = 0;
int handle_exception_deopt_exception_offset = 0;
int handle_exception_deopt_offset_2 = 0;
int handle_exception_deopt_exception_offset_2 = 0;
int handle_exception_call_pc_offset = 0;

static const int exception_blob_words = 0;

//---------------------------------RegisterSaver-------------------------------
class RegisterSaver {
 private:
  // the offsets (or -1) where the registers are spilled
  int _saved_reg_set[_last_Mach_Reg];

  int _first_saved_reg_offset;
  int _limit_saved_reg_offset;

  MacroAssembler* masm;

  void move_registers(bool is_save);

 public:
  RegisterSaver(MacroAssembler* masm_, bool (*save_this_register)(int reg)) {
    masm = masm_;

    // evaluate the predicate
    for (int reg = 0; reg < _last_Mach_Reg; reg++) {
      _saved_reg_set[reg] = (int) save_this_register(reg);
    }
  }

  int allocate_offsets(int first_saved_reg_offset) {
    _first_saved_reg_offset = first_saved_reg_offset;
    int saved_reg_offset    = first_saved_reg_offset;
    for (int reg = 0; reg < _last_Mach_Reg; reg++) {
      if ((bool) _saved_reg_set[reg]) {
	// align, patch, and increment the offset
	int reg_size = regSize[reg];
	assert(reg_size == 1 || reg_size == 2, "sane regSize");
	saved_reg_offset = align_size_up(saved_reg_offset, reg_size);
	_saved_reg_set[reg] = saved_reg_offset;
	saved_reg_offset += reg_size;
      } else {
	_saved_reg_set[reg] = -1;
      }
    }
    _limit_saved_reg_offset = saved_reg_offset;
    return _limit_saved_reg_offset;
  }

  int num_saved_reg_words() {
    return _limit_saved_reg_offset - _first_saved_reg_offset;
  }

  void save_registers() {
    move_registers(true);
  }

  void restore_registers() {
    move_registers(false);
  }

  void set_callee_saved(OopMap* map, int framesize) {
    for (int reg = 0; reg < _last_Mach_Reg; reg++) {
      int sp_offset = _saved_reg_set[reg];
      if (sp_offset >= 0) {
	map->set_callee_saved(SharedInfo::stack2reg(sp_offset), framesize, 0, OptoReg::Name(reg));
      }
    }
  }
};

void RegisterSaver::move_registers(bool is_save) {
  // save or restore the saved registers
  // (no scratch register worries:  we have all the %o and %l to play with)

  #ifdef ASSERT
  Register ijunk;
  FloatRegister fjunk;
  #endif

  // save the selected registers
  for (int reg = 0; reg < _last_Mach_Reg; reg++) {
    int sp_offset_in_bytes = _saved_reg_set[reg] * wordSize;
    if (sp_offset_in_bytes >= 0) {
      switch (register_save_type[reg]) {
      case Op_RegI:
	{
	  Register ireg((int)Matcher::_regEncode[reg], ' ');
	  if (is_save) {
	    __ st(ireg.after_save(), SP, sp_offset_in_bytes);
	    #ifdef ASSERT
	    // deliberately trash the register
	    if (!ijunk.isValid()) {
	      __ set(0xbaadd00d, ireg.after_save());
	      ijunk = ireg;
	    } else {
	      __ mov(ijunk, ireg.after_save());
	    }
	    #endif
	  } else {
	    __ ld(SP, sp_offset_in_bytes, ireg.after_save());
	  }
	  break;
	}

      case Op_RegF:
	{
	  FloatRegister freg((int)Matcher::_regEncode[reg], ' ');
	  if (is_save) {
	    __ stf(FloatRegister::S, freg, SP, sp_offset_in_bytes);
	    #ifdef ASSERT
	    // deliberately trash the register
	    if (!fjunk.isValid()) {
	      __ st(ijunk, FP, -4);
	      __ ldf(FloatRegister::S, FP, -4, freg);
	      fjunk = freg;
	    } else {
	      __ fmov(FloatRegister::S, fjunk, freg);
	    }
	    #endif
	  } else {
	    __ ldf(FloatRegister::S, SP, sp_offset_in_bytes, freg);
	  }
	  break;
	}

      case Op_RegD:
	{
	  // the 5-bit encoding is in the range [0..32)
	  int enc = (int)Matcher::_regEncode[reg];
	  // the constructor parameter for FloatRegister is even, and in the range [0..64).
	  int num = ((enc & 1) << 5) + (enc &~ 1);
	  FloatRegister dreg(num, ' ');
	  if (is_save) {
	    __ stf(FloatRegister::D, dreg, SP, sp_offset_in_bytes);
	  } else {
	    __ ldf(FloatRegister::D, SP, sp_offset_in_bytes, dreg);
	  }
	  break;
	}
      case Op_RegFlags:
	if (VM_Version::v9_instructions_work()) { 
	  if( !is_save )		// Load old flags into O0
	    __ ld( SP, sp_offset_in_bytes, O0 );
	  if( !strcmp( SharedInfo::regName[reg], "CCR" ) ) { // Check for 'CCR' vs 'FCC'
	    if( is_save ) __ rdccr ( O0 ); // Read CCR's and save
	    else          __ wrccr ( O0 ); // Restore CCR's
	  } else if( !strcmp( SharedInfo::regName[reg], "FCC0" ) ) {
	    if( is_save ) __ rdfprs( O0 ); // Read FCC0's and save
	    else          __ wrfprs( O0 ); // Restore FCC0's
	  } else if( !strcmp( SharedInfo::regName[reg], "FCC1" ) ) {
	    if( is_save ) {
	      __ rdfprs( O0 );	// Read all FCC's
	      __ srlx( O0, 32, O0 ); // Move high-order bits down low
	    } else {
	      __ rdfprs( O1 );	// Read all FCC0 into O1
	      __ srl( O1, 0, O1 ); // Zero FCC1-3 bits
	      __ sllx( O0, 32, O0 ); // Move old FCC1-3 bits up hi
	      __ or3( O0, O1, O0 ); // Combine
	      __ wrfprs( O0 ); // Restore ALL FCC's
	    }
	  } else {		// Save FCC2-FCC3
	    // FCC2 and FCC3 are saved along with FCC1 above
	  }
	  if( is_save )		// Save flags into stack
	    __ st( O0, SP, sp_offset_in_bytes);
	} else {
	  if( !strcmp( SharedInfo::regName[reg], "CCR" ) ) { // Check for 'CCR' vs 'FCC'
	    if( is_save ) {
	      // Store the ICC masked, but unshifted.
	      __ get_psr_trap();
	      __ nop();
	      __ set( PSR_ICC, O1 );
	      __ and3( O0, O1, O1 );
	      __ st( O1, SP, sp_offset_in_bytes);
	    } else { 
	      // Assume stored ICC is masked and unshifted.
	      __ get_psr_trap();
	      __ nop();
	      __ set( ~PSR_ICC, O1 );
	      __ and3( O0, O1, O0 );
	      __ ld( SP, sp_offset_in_bytes, O1 );
	      __ or3( O0, O1, O0 );
	      __ set_psr_trap();
	      __ nop();
 	    }
	  } else if( !strcmp( SharedInfo::regName[reg], "FCC0" ) ) {
	    if( is_save ) {
	      __ stfsr( SP, sp_offset_in_bytes );
	    } else {
	      __ ldfsr( SP, sp_offset_in_bytes );
	    }
	  } else {		// Save FCC1-FCC3
	    // No need to save FCC1-FCC3 on V8
	  }
	}
	break;
      default:
	ShouldNotReachHere();
      }
    }
  }
}


// predicate for RegisterSaver constructor
// selects all registers which are:
//  - not saved in any register window
//  - used by compiled Java code
//  - not the TLS register
// This amounts to 4 %g registers and all the %f/%d registers
static bool all_volatile_registers(int reg) {
  // special cases for particular registers:
  switch (reg) {
  case R_SP_num:
  case R_FP_num:
  case R_I7_num:
  case R_G2_num:  // handled by a different means, below
    // do not preserve these; the stack walker needs them to stay put
    return false;
  }

  // weed out %i/%l/%o registers and %d<odd> pseudo-regs
  // (Note:  we might be saving %o registers, because they can be SOE.)
  switch (register_save_type[reg]) {
  case Op_RegI:
    {
      assert(regSize[reg] == 1, "just checking");
      Register ireg((int)Matcher::_regEncode[reg], ' ');
      if (ireg.is_in() || ireg.is_local() || ireg.is_out())  return false;
      break;
    }

  case Op_RegF:
    assert(regSize[reg] == 1, "just checking");
    break;

  case Op_RegD:
    assert(regSize[reg] == 2, "just checking");
    if (is_odd(reg)) {
      // ignore the high-half registers
      assert(Matcher::_regEncode[reg] == 99, "sparc.ad uses 99 for uncodeable regs");
      return false;
    }
    if (VM_Version::v8_instructions_work()) { // No upper 32 float registers on V8
      return false;
    }
    break;
  case Op_RegFlags:
    return true;
  default:
    ShouldNotReachHere();
  }

  // weed out "always-save" registers
  switch (register_save_policy[reg]) {
  case 'E': // save on entry (callee-saved)
  case 'C': // save on call (caller-saved)
  case 'A': // always save
    // Because we are forcing a control transfer to happen in a place
    // where calls do not ordinarily occur, we must save all registers
    // except %i and %l registers, and certain "don't touch" registers
    // like %g7.
    return true;

  case 'N': // not saved
    return false;
  }

  ShouldNotReachHere();
  return false;
}


//------------------------------setup_exception_blob---------------------------
// creates exception blob at the end
// Using exception blob, this code is jumped from a compiled method.
// (see emit_exception_handler in sparc.ad file)
// It is also the default exception destination for c2i and i2c adapters
// 
// Given an exception pc at a call, we may unwind the stack and jump to
// caller's exception handler. We may also continue into the handler
// This code is entered with a jmp.
// 
// Arguments:
//   O0: exception oop
//   O1: exception pc
//
// Results:
//   O0: exception oop
//   O1: exception pc in caller or ???
//   destination: exception handler of caller
// 
// Note: the exception pc MUST be at a call (precise debug information)
//
void OptoRuntime::setup_exception_blob() {
  // allocate space for code
  ResourceMark rm;

  // setup code generation tools
  CodeBuffer*     buffer   = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm     = new MacroAssembler(buffer);

  Label exception_handler_found;
  Label L;

  address start = __ pc();

  __ verify_thread();
  __ st(Oexception,  Address(Gthread, 0, JavaThread::exception_oop_offset_in_bytes()));
  __ st(Oissuing_pc, Address(Gthread, 0, JavaThread::exception_pc_offset_in_bytes()));

  // This call does all the hard work. It checks if an exception handler
  // exists in the method.  
  // If so, it returns the handler address.
  // If not, it prepares for stack-unwinding, restoring the callee-save 
  // registers of the frame being removed.
  //
  __ save_frame(exception_blob_words);
  __ mov(Gthread, O0);
  __ set_last_Java_frame(SP, G0);
  __ save_thread(Lthread_cache);

  __ call(CAST_FROM_FN_PTR(address, OptoRuntime::handle_exception_C), relocInfo::runtime_call_type);
  __ delayed()->mov(Lthread_cache, O0);
  handle_exception_call_pc_offset = __ pc() - start;

  __ bind(L);
  __ restore_thread(Lthread_cache);
  __ reset_last_Java_frame();
  __ mov(O0, Gscratch);             // Move handler address to temp
  __ restore();

  __ tst(Gscratch);
  __ br(Assembler::notZero, false, Assembler::pn, exception_handler_found);
  __ delayed()->nop();

  // We have to check if the caller frame has been deoptimized;
  // If the caller frame has been deoptimized then we must continue
  // at DeoptimizationBlob::unpack_with_exception address; 
  // If the caller frame is not deoptimized, then we continue at
  // JavaThread::exception_handler_pc_offset_in_bytes.
  // The caller frame state is defined by its return address.

  __ ld(Address(Gthread, 0, JavaThread::exception_oop_offset_in_bytes()),
	Oexception  .after_save());
  __ ld(Address(Gthread, 0, JavaThread::exception_pc_offset_in_bytes()),
	Oissuing_pc .after_save());

  Label endL;

  __ ld(Address(Gthread, 0, JavaThread::exception_handler_pc_offset_in_bytes()), Gscratch);
  handle_exception_deopt_offset = __ pc() - start;
  __ set((unsigned)-1>>1, O5);    // will be filled in later with fill_in_exception_blob
  __ inc(O5, -frame::pc_return_offset*wordSize);
  __ cmp(I7, O5);                 // get return address to check if the caller frame is deoptimized
  __ br(Assembler::notEqual, false, Assembler::pn, endL);
  __ delayed()->nop();

  // deoptimizing this frame:  set Gscratch to point to a stub
  handle_exception_deopt_exception_offset = __ pc() - start;
  __ set((unsigned)-1>>1, Gscratch);  // will be filled in later with fill_in_exception_blob

  __ bind (endL);
  // Gscratch is handler-pc of caller is where we continue or the deoptimization blob address
  // Oexception is live
#ifdef ASSERT
  __ st(G0, Address(Gthread, 0, JavaThread::exception_oop_offset_in_bytes()));
  __ st(G0, Address(Gthread, 0, JavaThread::exception_handler_pc_offset_in_bytes()));
  __ st(G0, Address(Gthread, 0, JavaThread::exception_pc_offset_in_bytes()));
#endif
  __ jmpl(Gscratch, G0, G0);
  __ delayed()->restore();               // remove frame

  // Exception handler found.
  __ bind(exception_handler_found);

  Label endL_2;

  // check if top frame has been deoptimized; if yes, exception handler must be
  // deoptimized blob
  // address to check is in Oissuing_pc
  handle_exception_deopt_offset_2 = __ pc() - start;
  __ set((unsigned)-1>>1, O5);  // will be filled in later with fill_in_exception_blob
  __ inc(O5, -frame::pc_return_offset*wordSize);
  __ cmp(O7, O5);
  __ br(Assembler::notEqual, false, Assembler::pn, endL_2);
  __ delayed()->nop();

  handle_exception_deopt_exception_offset_2 = __ pc() - start;
  __ set((unsigned)-1>>1, Gscratch);  // will be filled in later with fill_in_exception_blob

  __ bind (endL_2);

  // Gscratch contains handler address
  assert(Assembler::is_simm13(JavaThread::exception_oop_offset_in_bytes()), "exception offset overflows simm13, following ld instruction cannot be in delay slot");
#ifdef ASSERT
  __ ld(Address(Gthread, 0, JavaThread::exception_oop_offset_in_bytes()), Oexception);
  __ st(G0, Address(Gthread, 0, JavaThread::exception_handler_pc_offset_in_bytes()));
  __ st(G0, Address(Gthread, 0, JavaThread::exception_pc_offset_in_bytes()));
  __ st(G0, Address(Gthread, 0, JavaThread::exception_oop_offset_in_bytes()));
  __ jmpl(Gscratch, G0, G0);
  __ delayed()->nop();
#else
  __ jmpl(Gscratch, G0, G0);
  __ delayed()->ld(Address(Gthread, 0, JavaThread::exception_oop_offset_in_bytes()), Oexception);
#endif

  // -------------
  // make sure all code is generated
  masm->flush();

  // Fill out other meta info
  OopMapSet *oop_maps = NULL; // will be set later; currently the register stuff is not yet initialized!
  _exception_blob = ExceptionBlob::create(buffer, oop_maps, I7.sp_offset_in_saved_window(), __ total_frame_size(0)/wordSize);
  address xxxyyy = _exception_blob->instructions_begin();
}

//------------------------------fill_in_exception_blob-------------------------
void OptoRuntime::fill_in_exception_blob() {
  int framesize;
  { ResourceMark rm;
    MacroAssembler* masm = new MacroAssembler(new CodeBuffer(NULL, NULL));
    framesize = __ total_frame_size(exception_blob_words)/wordSize;
  }

   // Patch blob
  assert(handle_exception_stub() != NULL, "exception stub must have been generated");
  assert(handle_exception_call_pc_offset != 0, "");
  assert(handle_exception_deopt_exception_offset != 0, "");
  assert(handle_exception_deopt_offset != 0, "");
  assert(handle_exception_deopt_exception_offset_2 != 0, "");
  assert(handle_exception_deopt_offset_2 != 0, "");

  {
  NativeMovConstReg* move = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_offset);
  move->set_data((int)OptoRuntime::deoptimization_blob()->unpack());

  NativeMovConstReg* move2 = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_exception_offset);
  move2->set_data((int)OptoRuntime::deoptimization_blob()->unpack_with_exception());
  }
  {
  NativeMovConstReg* move = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_offset_2);
  move->set_data((int)OptoRuntime::deoptimization_blob()->unpack());

  NativeMovConstReg* move2 = nativeMovConstReg_at(exception_blob()->instructions_begin() + handle_exception_deopt_exception_offset_2);
  move2->set_data((int)OptoRuntime::deoptimization_blob()->unpack_with_exception());
  }
  // Set an oopmap for the call site.  This oopmap will only be used if we
  // are unwinding the stack.  Hence, all locations will be dead.
  // Callee-saved registers will be the same as the frame above (i.e.,
  // handle_exception_stub), since they were restored when we got the
  // exception.
  ResourceMark rm;
  OopMapSet *oop_maps = new OopMapSet();
  // No callee-save registers to save here.
  oop_maps->add_gc_map( handle_exception_call_pc_offset, true, new OopMap(framesize, 0));
  exception_blob()->set_oop_maps(oop_maps);

}


//------------------------------generate_deopt_blob----------------------------
// Ought to generate an ideal graph & compile, but here's some SPARC ASM
// instead.
void OptoRuntime::generate_deopt_blob() {
  // allocate space for the code
  ResourceMark rm;
  // setup code generation tools
  CodeBuffer*     buffer             = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm               = new MacroAssembler(buffer);
  FloatRegister   Freturn0           = F0;
  Register        Oreturn0           = O0;
  Register        Oreturn1           = O1;
  Register        OUnrollBlock       = O2;
  Register        Oframe_size        = O2;
  Register        Oarray             = O3;
  Register        Oarray_size        = O4;
  Register        Oexception_tmp     = O5;
  int             unpack_frame_words = sizeof(double)/wordSize;
  int             framesize          = __ total_frame_size(unpack_frame_words)/wordSize;
  Address         saved_Freturn0_addr(FP, 0, -sizeof(double));
  Label           cont, loop, skip_i2c;

  OopMapSet *oop_maps = new OopMapSet();

  //
  // This is the entry point for code which is returning to a de-optimized
  // frame.
  // The steps taken by this frame are as follows:
  //   - push a dummy "unpack_frame" and save the return values (O0, O1, F0)
  //     which are coming in
  //   - call the C routine: Deoptimization::fetch_unroll_info (this function
  //     returns information about the number and size of interpreter frames
  //     which are equivalent to the frame which is being deoptimized)
  //   - deallocate the unpack frame
  //   - deallocate the deoptimization frame
  //   - in a loop using the information returned in the previous step
  //     push new interpreter frames (take care to propagate the return
  //     values through each new frame pushed)
  //   - create a dummy "unpack_frame" and save the return values (O0, O1, F0)
  //   - call the C routine: Deoptimization::unpack_frames (this function
  //     lays out values on the interpreter frame which was just created)
  //   - deallocate the dummy unpack_frame
  //   - ensure that all the return values are correctly set and then do
  //     a return to the interpreter entry point
  //
  // Refer to the following methods for more information:
  //   - Deoptimization::fetch_unroll_info
  //   - Deoptimization::unpack_frames

  address start = __ pc();

  // Add empty oopmap at begining. This can both be at_call and not_at_call,
  // Normally it would be at call. However, it could be at not_at_call, if the
  // thread happended to be blocked at safepoint_node in the frame that got
  // deoptimized.
  oop_maps->add_gc_map( 0, true, new OopMap( framesize, 0) );
  oop_maps->add_gc_map( 0, false, new OopMap( framesize, 0) );

  __ ba(false, cont);
  __ delayed()->mov(Deoptimization::Unpack_deopt, Oexception_tmp);
  int exception_offset = __ pc() - start;
  __ mov(Deoptimization::Unpack_exception, Oexception_tmp);

  __ bind(cont);
  // push a dummy "unpack_frame" taking care to preserve float return values
  // and call Deoptimization::fetch_unroll_info to get the UnrollBlock
  __ save_frame(unpack_frame_words);
  __ stf(FloatRegister::D, Freturn0, saved_Freturn0_addr);
  __ set_last_Java_frame(SP, G0);
  __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, Deoptimization::fetch_unroll_info), Gthread);
  // Set an oopmap for the call site   
  //%%% This is not the right place.  Is it really needed?
  //oop_maps->add_gc_map( __ pc()-start, true, new OopMap( framesize, 0) );

  __ reset_last_Java_frame();
  __ mov(O0, OUnrollBlock.after_save());
  __ ldf(FloatRegister::D, saved_Freturn0_addr, Freturn0);
  __ restore();

  // %%%%%%% may have to setup some oop maps for the call site if G/O
  //         registers are used in code

  // deallocate the deoptimization frame taking care to preserve the return values
  __ mov(Oreturn0,     Oreturn0.after_save());
  __ mov(Oreturn1,     Oreturn1.after_save());
  __ mov(OUnrollBlock, OUnrollBlock.after_save());
  __ mov(Oexception_tmp, Oexception_tmp.after_save());
  __ restore();

  // Pop an I2C adapter if desired
  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::pop_i2c_frame_offset_in_bytes()), Oarray);
  __ tst(Oarray);
  __ br(Assembler::zero, false, Assembler::pt, skip_i2c);
  __ delayed()-> nop();
  __ mov(Oreturn0,     Oreturn0.after_save());
  __ mov(Oreturn1,     Oreturn1.after_save());
  __ mov(OUnrollBlock, OUnrollBlock.after_save());
  __ mov(Oexception_tmp, Oexception_tmp.after_save());
  __ restore();			// Actually pop I2C frame here
  // Adjust old interpreter frame to make space for new frame's java locals
  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::adjust_i2c_frame_offset_in_bytes()), Oarray);
  __ sub(SP,Oarray,SP);
  __ bind(skip_i2c);

  // loop through the UnrollBlock info and create interpreter frames

  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::number_of_frames_offset_in_bytes()), Oarray_size);
  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::frame_sizes_offset_in_bytes()), Oarray);

  #ifdef ASSERT
  // make sure that there is at least one entry in the array
  __ tst(Oarray_size);
  __ breakpoint_trap(Assembler::zero);
  #endif

  __ bind(loop);

  __ ld(Oarray, 0, Oframe_size);
  __ sub(G0, Oframe_size, Oframe_size);
  __ save(SP, Oframe_size, SP);

  #ifdef ASSERT
  // make sure that the frames are aligned properly
  __ btst(wordSize, SP);
  __ breakpoint_trap(Assembler::notZero);
  #endif


  __ mov(Oarray_size.after_save(), Oarray_size);
  __ mov(Oreturn0.after_save(), Oreturn0);
  __ sub(Oarray_size, 1, Oarray_size);
  __ mov(Oreturn1.after_save(), Oreturn1);
  __ mov(Oarray.after_save(), Oarray);
  __ mov(Oexception_tmp.after_save(), Oexception_tmp);

  #ifdef ASSERT
  // trash registers to show a clear pattern in backtraces
  __ set(0xDEAD0000, I0);
  __ add(I0,  2, I1);
  __ add(I0,  4, I2);
  __ add(I0,  6, I3);
  __ add(I0,  8, I4);
  __ add(I0, 12, I5);
  __ set(0xDEADBEEF, L0);
  __ mov(L0, L1);
  __ mov(L0, L2);
  __ mov(L0, L3);
  __ mov(L0, L4);
  __ mov(L0, L5);

  // trash the return value as there is nothing to return yet
  __ set(0xDEAD0001, O7);
  #endif

  __ tst(Oarray_size);
  __ br(Assembler::notZero, false, Assembler::pn, loop);
  __ delayed()->add(Oarray, wordSize, Oarray);

  // push a dummy "unpack_frame" taking care of float return values and
  // call Deoptimization::unpack_frames to have the unpacker layout
  // information in the interpreter frames just created and then return
  // to the interpreter entry point
  __ save_frame(unpack_frame_words);
  __ stf(FloatRegister::D, Freturn0, saved_Freturn0_addr);
  __ set_last_Java_frame(SP, G0);
  __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, Deoptimization::unpack_frames), Gthread, Oexception_tmp.after_save());
  __ reset_last_Java_frame();
  __ ldf(FloatRegister::D, saved_Freturn0_addr, Freturn0);
  __ ret();
  __ delayed()->restore();

  masm->flush();
  _deopt_blob = DeoptimizationBlob::create(buffer, oop_maps, 0, exception_offset, I7.sp_offset_in_saved_window(), framesize);
}


//------------------------------generate_uncommon_trap_blob--------------------
// Ought to generate an ideal graph & compile, but here's some SPARC ASM
// instead.
void OptoRuntime::generate_uncommon_trap_blob() {
  // allocate space for the code
  ResourceMark rm;
  // setup code generation tools
  OopMapSet *oop_maps          = NULL;        
  CodeBuffer*   buffer               = new CodeBuffer(512, 512, 0, 0, false);
  MacroAssembler* masm               = new MacroAssembler(buffer);
  Register        OUnrollBlock       = O2;
  Register        Oframe_size        = O2;
  Register        Oarray             = O3;
  Register        Oklass_index       = O2;
  Register        Oarray_size        = O4;
  Label           cont, loop, skip_i2c;

  //
  // This is the entry point for all traps the compiler takes when it thinks
  // it cannot handle further execution of compilation code. The frame is
  // deoptimized in these cases and converted into interpreter frames for
  // execution
  // The steps taken by this frame are as follows:
  //   - push a fake "unpack_frame"
  //   - call the C routine Deoptimization::uncommon_trap (this function
  //     packs the current compiled frame into vframe arrays and returns
  //     information about the number and size of interpreter frames which 
  //     are equivalent to the frame which is being deoptimized)
  //   - deallocate the "unpack_frame"
  //   - deallocate the deoptimization frame
  //   - If removing a paired I2C/C2I, deallocate the I2C frame.
  //   - in a loop using the information returned in the previous step
  //     push interpreter frames; includes a C2I frame if needed.
  //   - create a dummy "unpack_frame"
  //   - call the C routine: Deoptimization::unpack_frames (this function
  //     lays out values on the interpreter frame which was just created)
  //   - deallocate the dummy unpack_frame
  //   - return to the interpreter entry point
  //
  //  Refer to the following methods for more information:
  //   - Deoptimization::uncommon_trap
  //   - Deoptimization::unpack_frame

  // the unloaded class index is in O0 (first parameter to this blob)

  // push a dummy "unpack_frame"
  // and call Deoptimization::uncommon_trap to pack the compiled frame into
  // vframe array and return the UnrollBlock information
  __ save_frame(0);
  __ set_last_Java_frame(SP, G0);
  __ mov(I0, Oklass_index);
  __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, Deoptimization::uncommon_trap), Gthread, Oklass_index);
  __ reset_last_Java_frame();
  __ mov(O0, OUnrollBlock.after_save());
  __ restore();

  // %%%%%%% may have to setup some oop maps for the call site if G/O
  //         registers are used in code

  // deallocate the deoptimized frame taking care to preserve the return values
  __ mov(OUnrollBlock, OUnrollBlock.after_save());
  __ restore();

  // Pop an I2C adapter if desired
  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::pop_i2c_frame_offset_in_bytes()), Oarray);
  __ tst(Oarray);
  __ br(Assembler::zero, false, Assembler::pt, skip_i2c);
  __ delayed()->nop();
  __ mov(OUnrollBlock, OUnrollBlock.after_save());
  __ restore();			// Actually pop I2C frame here
  // Adjust old interpreter frame to make space for new frame's java locals
  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::adjust_i2c_frame_offset_in_bytes()), Oarray);
  __ sub(SP,Oarray,SP);
  __ bind(skip_i2c);

  // loop through the UnrollBlock info and create interpreter frames

  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::number_of_frames_offset_in_bytes()), Oarray_size);
  __ ld(Address(OUnrollBlock, 0, Deoptimization::UnrollBlock::frame_sizes_offset_in_bytes()), Oarray);

  #ifdef ASSERT
  // make sure that there is atleast one entry in the array
  __ tst(Oarray_size);
  __ breakpoint_trap(Assembler::zero);
  #endif

  __ bind(loop);

  __ ld(Oarray, 0, Oframe_size);
  __ sub(G0, Oframe_size, Oframe_size);
  __ save(SP, Oframe_size, SP);

  #ifdef ASSERT
  // make sure that the frames are aligned properly
  __ btst(wordSize, SP);
  __ breakpoint_trap(Assembler::notZero);
  #endif

  __ mov(Oarray_size.after_save(), Oarray_size);
  __ mov(Oarray.after_save(), Oarray);
  __ sub(Oarray_size, 1, Oarray_size);

  #ifdef ASSERT
  // trash registers to show a clear pattern in backtraces
  __ set(0xDEAD0000, I0);
  __ add(I0,  2, I1);
  __ add(I0,  4, I2);
  __ add(I0,  6, I3);
  __ add(I0,  8, I4);
  __ add(I0, 12, I5);
  __ set(0xDEADBEEF, L0);
  __ mov(L0, L1);
  __ mov(L0, L2);
  __ mov(L0, L3);
  __ mov(L0, L4);
  __ mov(L0, L5);

  // trash the return value as there is nothing to return yet
  __ set(0xDEAD0001, O7);
  #endif

  __ tst(Oarray_size);
  __ br(Assembler::notZero, false, Assembler::pn, loop);
  __ delayed()->add(Oarray, wordSize, Oarray);

  // push a dummy "unpack_frame" taking care of float return values and
  // call Deoptimization::unpack_frames to have the unpacker layout
  // information in the interpreter frames just created and then return
  // to the interpreter entry point
  __ save_frame(0);
  __ set_last_Java_frame(SP, G0);
  __ mov(Deoptimization::Unpack_uncommon_trap, O3); // indicate it is the uncommon trap case
  __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, Deoptimization::unpack_frames), Gthread, O3);
  __ reset_last_Java_frame();
  __ ret();
  __ delayed()->restore();

  masm->flush();
  _uncommon_trap_blob = UncommonTrapBlob::create(buffer, oop_maps, I7.sp_offset_in_saved_window(), __ total_frame_size(0)/wordSize);
}

//------------------------------generate_illegal_instruction_handler_blob------
//
// Generate a special Compile2Runtime blob that saves all registers, 
// and setup oopmap.
//
// It is important that we do not touch the %i registers before restore and
// %o registers (other than %o7) after restore here (these may also be modified by gc, 
// and we just propagate them up to the responsible Java frame)
//
// This blob is jumped to (via a breakpoint and the signal handler) from a safepoint
// in a thread code buffer.  On entry to this blob, O7 contains the address in the
// original nmethod at which we should resume normal execution.  Thus, this blob looks
// like a subroutine which must preserve lots of registers and return normally.
// Note that O7 is never register-allocated, so it is guaranteed to be free here.
//

void OptoRuntime::generate_illegal_instruction_handler_blob() {
  assert (StubRoutines::forward_exception_entry() != NULL, "must be generated before");  

  // allocate space for the code
  ResourceMark rm;
  // setup code generation tools  
  CodeBuffer*     buffer              = new CodeBuffer(1536, 1536, 0, 0, false);
  MacroAssembler* masm                = new MacroAssembler(buffer);
  
  address start = __ pc();

  // decide which registers we are dumping to stack
  RegisterSaver rs(masm, all_volatile_registers);
  rs.allocate_offsets(frame::memory_parameter_word_sp_offset);

  // setup a frame for saving the state
  __ save_frame(rs.num_saved_reg_words());
  int framesize = __ total_frame_size(rs.num_saved_reg_words())/wordSize;

  rs.save_registers();

  // setup last_Java_sp
  __ set_last_Java_frame(SP, G0);

  // call into the runtime to handle illegal instructions exception
  // Do not use call_VM_leaf, because we need to make a GC map at this call site.
  __ mov(Gthread, O0);
  __ save_thread(Lthread_cache);
  __ call(CAST_FROM_FN_PTR(address, SafepointSynchronize::handle_illegal_instruction_exception));
  __ delayed()->nop();
  address call_pc = __ pc();
  __ restore_thread(Lthread_cache);

  // clear last_Java_sp
  __ reset_last_Java_frame();
 
  Register Otemp = O7;

  // check for pending exceptions
  { Label L;
    Address exception_addr(Gthread, 0, Thread::pending_exception_offset_in_bytes());
    __ ld(exception_addr, Otemp);
    if (VM_Version::v9_instructions_work()) { 
      __ bpr(Assembler::rc_z, false, Assembler::pt, Otemp, L);
    } else {
      __ tst(Otemp);
      __ br(Assembler::equal, false, Assembler::pt, L);
    }
    __ delayed()->nop();

    // Restore all registers, including flags. 
    // This could be merged with the other call to restore_registers() and hoisted 
    // above the check for pending exceptions, if the above tst instruction 
    // didn't clobber the condition codes.
    rs.restore_registers();

    // Tail-call forward_exception_entry, with the issuing PC in O7,
    // so it looks like the original nmethod called forward_exception_entry.
    __ set((int)StubRoutines::forward_exception_entry(), Otemp);
    __ jmpl(Otemp, G0, G0);
    __ delayed()->restore();

    __ bind(L);
  }

  Label negativeL;
  __ tst(O0);
  __ br(Assembler::negative, false, Assembler::pt, negativeL);
  __ delayed()->nop();

  // Restore all registers, including flags
  rs.restore_registers();

  // Return to the original nmethod, as identified by the O7 value
  // (now I7 until the next restore) set up by the trap handler.
  // (probably, that logic should be moved out of the trap handler!)
  __ ret();
  __ delayed()->restore();

  // deoptimization happened at a compiled safepoint at a call; jimp to destination
  // instead of skipping the call in the blocked method; we do not have debug information
  // to reexecute the call
  __ bind(negativeL);
  rs.restore_registers();
  __ restore();
  __ mov(O7, Gscratch); // destination of the call
  DeoptimizationBlob* deopt_blob = OptoRuntime::deoptimization_blob();
  assert(deopt_blob != NULL, "deoptimization blob must have been created");
  __ load_address(Address(O7, deopt_blob->unpack() -  frame::pc_return_offset*wordSize));
  // __ breakpoint_trap();
  __ jmpl(Gscratch, frame::pc_return_offset*wordSize, G0);
  __ delayed()->nop();

  // Set an oopmap for the call site.
  // We need this not only for callee-saved registers, but also for
  // volatile registers that the compiler might be keeping live across a safepoint.
  OopMapSet *oop_maps = new OopMapSet();
  OopMap* map =  new OopMap( framesize, 0 );  
  rs.set_callee_saved(map, framesize);
  oop_maps->add_gc_map( call_pc - start, true, map);

  // -------------
  // make sure all code is generated
  masm->flush();  

  // Set exception blob
  _illegal_instruction_handler_blob = SafepointBlob::create(buffer, oop_maps, I7.sp_offset_in_saved_window(), framesize);
}


//------------------------------exception_address_is_unpack_entry--------------
bool DeoptimizationBlob::exception_address_is_unpack_entry(address pc) const {
  address unpack_pc = OptoRuntime::deoptimization_blob()->unpack();
  return (pc == unpack_pc ||
          (pc + frame::pc_return_offset * wordSize) == unpack_pc);
}


void OptoRuntime::pd_unwind_stack(JavaThread *thread, frame fr, RegisterMap* reg_map) {
}

//
// Generate the on-stack replacement stub, that is used to replace the
// interpreter frame
//
OSRAdapter* OptoRuntime::generate_osr_blob(int frame_size) {
  ResourceMark rm;
  
  // setup code generation tools
  CodeBuffer*       cb = new CodeBuffer(128, 128, 0, 0, false);
  MacroAssembler* masm = new MacroAssembler(cb);    
  
  OopMapSet *oop_maps = new OopMapSet();
  OopMap* map =  new OopMap(frame_size, 0 );
  oop_maps->add_gc_map(0, true, map);
  
  __ mov(O0,I0);
  __ ret();
  __ delayed()->restore(O1,G0,O1);
  
  return OSRAdapter::new_osr_adapter(cb, oop_maps, frame_size, 0);
}

//Reconciliation History
// 1.2 98/06/11 10:29:32 runtime_i486.cpp
// 1.3 98/06/25 16:44:37 runtime_i486.cpp
// 1.6 98/07/24 15:30:03 runtime_i486.cpp
// 1.7 98/08/06 11:43:15 runtime_i486.cpp
// 1.8 98/08/17 10:45:15 runtime_i486.cpp
// 1.12 98/08/28 11:30:59 runtime_i486.cpp
// 1.13 98/09/02 12:58:19 runtime_i486.cpp
// 1.14 98/09/03 20:05:49 runtime_i486.cpp
// 1.23 98/10/02 12:37:40 runtime_i486.cpp
// 1.26 98/10/06 16:50:52 runtime_i486.cpp
// 1.28 98/10/07 11:36:46 runtime_i486.cpp
// 1.30 98/10/09 17:17:32 runtime_i486.cpp
// 1.32 98/10/15 12:51:04 runtime_i486.cpp
// 1.33 98/10/27 13:56:13 runtime_i486.cpp
// 1.35 98/11/09 09:26:27 runtime_i486.cpp
// 1.36 98/11/13 14:04:16 runtime_i486.cpp
// 1.37 99/01/06 16:18:57 runtime_i486.cpp
// 1.40 99/01/21 16:59:49 runtime_i486.cpp
// 1.44 99/05/26 19:34:52 runtime_i486.cpp
// 1.47 99/06/24 14:24:51 runtime_i486.cpp
// 1.42 99/03/16 10:37:21 runtime_i486.cpp
// 1.43 99/04/13 11:57:13 runtime_i486.cpp
// 1.50 99/08/13 17:48:48 runtime_i486.cpp
// 1.52 99/08/26 16:29:56 runtime_i486.cpp
//End
