/*
 * @(#)stubGenerator_sparc.cpp	1.147 00/04/18 15:58:54
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_stubGenerator_sparc.cpp.incl"

// Declaration and definition of StubGenerator (no .hpp file).
// For a more detailed description of the stub routine structure
// see the comment in stubRoutines.hpp.

#define __ _masm->

// Note:  The register L7 is used as Lthread_cache, and may not be used
//        any other way within this module.


static const Register& Lstub_temp = L2;

// -------------------------------------------------------------------------------------------------------------------------
// Stub Code definitions

class StubGenerator: public StubCodeGenerator {
 private:

  enum Ret_type {
    RET_VOID,			// the C stub returns nothing
    RET_INT,			// the C stub returns a word copied to O0 (int)
    RET_OOP,			// the C stub returns a word copied to O0 (oop)
    RET_INT_AND_OOP,		// the C stub returns int (O0) and an oop also
    RET_TWO_OOPS,		// the C stub returns oops in Garg0 and Garg1
    RET_JUMP			// the C stub returns an address to jump to
  };

  enum {
    no_stub_flags	= 00,
    preserve_registers  = 1 << 1, // be careful to preserve the On/Gn registers
    inline_code		= 1 << 2, // don't do a save, a restore, or a ret
    no_thread_arg	= 1 << 3  // do not prepend thread as first C argument
  };

  // --------------------------------------------
  // Most stubs are used to call C from Java.
  // This overloaded helper routine can generate many of them.

  // %%%%% Over time, try to reduce the degrees of freedom in
  // this routine, and eventually move the simplified version
  // into the assembler (as call_VM_with_save or some such).  

  void generate_runtime_stub(
    Ret_type ret_type,
    address runtime_routine,
    int flags,
    Register a0 = noreg,
    Register a1 = noreg,
    Register a2 = noreg,
    Register a3 = noreg) {
    Register vm_result_reg = noreg;
    generate_runtime_stub(ret_type, vm_result_reg,
			  runtime_routine,
			  flags,
			  a0, a1, a2, a3);
  }

  void generate_runtime_stub(
    Ret_type ret_type,
    Register vm_result_reg,
    address runtime_routine,
    int flags,
    Register a0 = noreg,
    Register a1 = noreg,
    Register a2 = noreg,
    Register a3 = noreg) {
    // (Conjecture:  Any stub which could be a leaf-routine could also be inlined.)

    Register vm_result_2_reg = noreg;

    // Registers allocated locally
    const Register& saved_gtemp1_reg = L5;
    const Register& saved_gtemp2_reg = L1;
    const Register& saved_result_reg = L0;
    // these must not equal any of Lstub_temp, which is used below

    const int preserve_register_words = 2;
    Address saved_d0_addr(FP, 0, -preserve_register_words * wordSize);
    // See CodeEmitter::FPU_store_object_offset_fixup for use of saved_d0_addr.

    // RET_TWO_OOPS equivalent to RET_OOP with another oop result in O1
    if (ret_type == RET_TWO_OOPS) {
      ret_type = RET_VOID;
      assert(vm_result_reg == noreg, "");
      vm_result_reg   = Garg0;
      vm_result_2_reg = Garg1;
    }

    // RET_OOP equivalent to RET_VOID with an oop result in O0
    if (ret_type == RET_OOP) {
      ret_type = RET_VOID;
      assert(vm_result_reg == noreg, "");
      vm_result_reg = O0;
    }

    // RET_INT_AND_OOP equivalent to RET_INT with an oop result in O1
    if (ret_type == RET_INT_AND_OOP) {
      ret_type = RET_INT;
      assert(vm_result_reg == noreg, "");
      vm_result_reg = O1;
    }

    // (RET_INT and RET_VOID are essentially the same, also)

    const int na = 1+4;
    int i;
    Register a[na];
    a[0] = a0; a[1] = a1; a[2] = a2; a[3] = a3;
    a[na-1] = noreg;

    // Maybe create a stack frame.
    if (!(flags & inline_code)) {
      if (flags & preserve_registers)
	__ save_frame(preserve_register_words);
      else
	__ save_frame(0);
      for (i = 0; i < na; i++) {
	if (a[i] != noreg) {
	  a[i] = a[i].after_save();
	}
      }
      if (vm_result_reg != noreg)
	vm_result_reg = vm_result_reg.after_save();
      if (vm_result_2_reg != noreg)
	vm_result_2_reg = vm_result_2_reg.after_save();
    }

    // Maybe prepend the thread argument (in a[*]).
    if (!(flags & no_thread_arg)) {
      for (i = na; --i > 0; ) {
	a[i] = a[i-1];
      }
      a[0] = Lthread_cache;
    }

    // marshal the G registers and all I's except one, a[lasti]
    int lasti = -1;

    if (!(flags & no_thread_arg))
      lasti = 0;		// will do delayed()->mov(Gthread, O0)

    for (i = 0; i < na; i++) {
      if (a[i] != noreg) {
	if (!a[i].is_global() && lasti < 0) {
	  lasti = i;
	}
	if (a[i] == I7) {
	  // %%%% kludge:  bias any I7 value passed up to C
	  // this will disappear when return addresses are removed
	  // from the CompilerRuntime API.
	  const int pc_adj = frame::pc_return_offset * BytesPerWord;
	  Register oreg = Register('o',i);
	  __ add(a[i], pc_adj, oreg);
	  a[i] = oreg;
	}
	if (i != lasti)
	  __ mov(a[i], Register('o', i));
      }
    }

    if (flags & preserve_registers) {
      __ mov(Gtemp,  saved_gtemp1_reg);
      __ mov(Gtemp2, saved_gtemp2_reg);
      __ stf(FloatRegister::D, F0, saved_d0_addr);
      // The following code will smash Gn registers; preserve them.
      // (The incoming On are now in non-volatile In, and so are preserved
      // without any special effort.)
    }

    __ set_last_Java_frame(FP, G0);

    __ save_thread(Lthread_cache);

    __ call (runtime_routine, relocInfo::runtime_call_type);
    if (lasti < 0)   __ delayed()->nop();
    else             __ delayed()->mov_or_nop(a[lasti], Register('o', lasti));

    __ restore_thread(Lthread_cache);

    // Where did the call put the result?
    Register int_result_reg = (ret_type == RET_VOID) ? noreg : O0;

    __ reset_last_Java_frame();

    if (flags & preserve_registers) {
      __ mov(saved_gtemp1_reg, Gtemp);
      __ mov(saved_gtemp2_reg, Gtemp2);
      __ ldf(FloatRegister::D, saved_d0_addr, F0);
    }

    if (vm_result_reg != noreg)
      __ get_vm_result(vm_result_reg);

    if (vm_result_2_reg != noreg) 
      __ get_vm_result_2(vm_result_2_reg);

    if (flags & inline_code) {
      switch (ret_type) {
      case RET_VOID:
	return;

      case RET_INT:
	// result in O0; do not return or restore
	__ mov(int_result_reg, O0);
	return;

      case RET_JUMP:
	__ jmp(int_result_reg, 0);
	__ delayed()->restore(); // must restore anyway
	break;

      default:
	ShouldNotReachHere();
	break;
      }
    }

    // else it must restore and return:

    switch (ret_type) {
    case RET_VOID:
      __ ret();
      __ delayed()->restore();
      break;

    case RET_INT:
      __ ret();
      __ delayed()->restore(int_result_reg, G0, O0); // copy result word back to caller
      break;

    case RET_JUMP:
      __ jmp(int_result_reg, 0);
      __ delayed()->restore();
      break;

    default:
      ShouldNotReachHere();
    }
  }

  // Load Oissuing_pc from the given register.
  void load_issuing_pc_from_return_addr(Register return_addr_reg) {
    const int pc_adj = frame::pc_return_offset * BytesPerWord;

    __ add(return_addr_reg, pc_adj, Oissuing_pc);
  }


  //----------------------------------------------------------------------------------------------------
  // Call stubs are used to call Java from C

  address StubGenerator::generate_call_stub(address& return_pc) {
    StubCodeMark mark(this, "StubRoutines", "call_stub");
    address start = __ pc();

    // Incoming arguments:
    //
    // o0         : call wrapper address
    // o1         : result (address)
    // o2         : result type
    // o3         : method
    // o4         : (interpreter) entry point
    // o5         : parameters (address)
    // [sp + 0x5c]: parameter size (in words)
    // [sp + 0x60]: thread
    //
    // +---------------+ <--- sp + 0
    // |               |
    // . reg save area .
    // |               |
    // +---------------+ <--- sp + 0x40
    // |               |
    // . extra 7 slots .
    // |               |
    // +---------------+ <--- sp + 0x5c
    // |  param. size  |
    // +---------------+ <--- sp + 0x60
    // |    thread     |
    // +---------------+
    // |               |

    // note: if the link argument position changes, adjust
    //       the code in frame::entry_frame_call_wrapper()

    const Argument link           = Argument(0, false); // used only for GC
    const Argument result         = Argument(1, false);
    const Argument result_type    = Argument(2, false);
    const Argument method         = Argument(3, false);
    const Argument entry_point    = Argument(4, false);
    const Argument parameters     = Argument(5, false);
    const Argument parameter_size = Argument(6, false);
    const Argument thread         = Argument(7, false);

    // setup thread register
    __ ld(thread.as_address(), Gthread);

#ifdef ASSERT
    // make sure we have no pending exceptions
    { const Register t = Gscratch;
      Label L;
      __ ld(Gthread, Thread::pending_exception_offset_in_bytes(), t);
      __ tst(t);
      __ br(Assembler::equal, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("StubRoutines::call_stub: entered with pending exception");
      __ bind(L);
    }
#endif

    // create activation frame & allocate space for parameters
    { const Register t = Gscratch;
      __ ld(parameter_size.as_address(), t);                    // get parameter size (in words)
      __ add(t, frame::memory_parameter_word_sp_offset, t);     // add space for save area (in words)
      __ round_to(t, WordsPerLong);                             // make sure it is multiple of 2 (in words)
      __ sll(t, LogBytesPerWord, t);                            // compute number of bytes
      __ neg(t);                                                // negate so it can be used with save
      __ save(SP, t, SP);                                       // setup new frame
    }

    // +---------------+ <--- sp + 0
    // |               |
    // . reg save area .
    // |               |
    // +---------------+ <--- sp + 0x40
    // |               |
    // . extra 7 slots .
    // |               |
    // +---------------+ <--- sp + 0x5c
    // |  empty slot   |      (only if parameter size is even)
    // +---------------+
    // |               |
    // .  parameters   .
    // |               |
    // +---------------+ <--- fp + 0
    // |               |
    // . reg save area .
    // |               |
    // +---------------+ <--- fp + 0x40
    // |               |
    // . extra 7 slots .
    // |               |
    // +---------------+ <--- fp + 0x5c
    // |  param. size  |
    // +---------------+ <--- fp + 0x60
    // |    thread     |
    // +---------------+
    // |               |

    // pass parameters if any
    { const Register src = parameters.as_in().as_register();
      const Register dst = Lesp;
      const Register tmp = Gscratch;
      const Register cnt = Gscratch2;

      // test if any parameters & setup of Lesp
      Label exit;
      __ ld(parameter_size.as_in().as_address(), cnt);          // parameter counter
      __ tst(cnt);
      __ br(Assembler::zero, false, Assembler::pn, exit);
      __ delayed()->sub(FP, BytesPerWord, dst);                 // setup Lesp

      // copy parameters if any
      Label loop;
      __ bind(loop);
      __ ld(src, 0, tmp);
      __ add(src, BytesPerWord, src);
      __ st(tmp, dst, 0);
      __ deccc(cnt);
      __ br(Assembler::greater, false, Assembler::pt, loop);
      __ delayed()->sub(dst, BytesPerWord, dst);

      // done
      __ bind(exit);
    }

    // setup parameters, method & call Java function
#ifdef ASSERT
    const Register saved_SP = Lscratch;
    if (!UseC2CallingConventions) {
      __ mov(SP, saved_SP);                                     // keep track of SP before call
    }
#endif
    // setup parameters
    if (!UseC2CallingConventions) {
      const Register t = Gscratch;
      __ ld(parameter_size.as_in().as_address(), t);            // get parameter size (in words)
      __ sll(t, LogBytesPerWord, t);                            // compute number of bytes
      __ sub(FP, t, Gargs);                                     // setup parameter pointer
    }
    // do the call
    //
    // the following register must be setup:
    //
    // Gthread
    // Gmethod
    // Gargs
    __ jmpl(entry_point.as_in().as_register(), G0, O7);
    __ delayed()->mov(method.as_in().as_register(), Gmethod);   // setup method
    return_pc = __ pc();

#ifdef ASSERT
    // make sure callee didn't change SP
    // note: strictly speaking, this doesn't matter for java calls, because
    //       we return anyway - however, it is an indication that something
    //       goes wrong elsewhere
    if (!UseC2CallingConventions) {
      Label L;
      __ cmp(SP, saved_SP);
      __ br(Assembler::equal, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("StubRoutines::call_stub: callee changed SP");
      __ bind(L);
    }
#endif

    // store result depending on type
    // (everything that is not T_LONG, T_FLOAT, or T_DOUBLE is treated as T_INT)
    { const Register addr = result     .as_in().as_register();
      const Register type = result_type.as_in().as_register();
      Label is_long, is_float, is_double, exit;
      __            cmp(type, T_FLOAT ); __ br(Assembler::equal, false, Assembler::pn, is_float );
      __ delayed()->cmp(type, T_DOUBLE); __ br(Assembler::equal, false, Assembler::pn, is_double);
      __ delayed()->cmp(type, T_LONG  ); __ br(Assembler::equal, false, Assembler::pn, is_long  );

      // store int result or higher half of long (non-annulled delay slot)
      __ delayed()->st(O0, addr, G0);

      __ bind(exit);
      __ ret();
      __ delayed()->restore();

      __ bind(is_float);
      __ ba(false, exit);
      __ delayed()->stf(FloatRegister::S, F0, addr, G0);

      __ bind(is_double);
      __ ba(false, exit);
      __ delayed()->stf(FloatRegister::D, F0, addr, G0);

      __ bind(is_long);
      __ ba(false, exit);
      __ delayed()->st(O1, addr, BytesPerWord);                 // store lower half of long
    }

    return start;
  }


  //----------------------------------------------------------------------------------------------------
  // Return point for a Java call if there's an exception thrown in Java code.
  // The exception is caught and transformed into a pending exception stored in
  // JavaThread that can be tested from within the VM.
  //
  // Oexception: exception oop

  address StubGenerator::generate_catch_exception() {
    StubCodeMark mark(this, "StubRoutines", "catch_exception");

    address start = __ pc();
    // verify that thread corresponds
    __ verify_thread();

    const Register& temp_reg = Gtemp;
    Address pending_exception_addr(Gthread, 0, Thread::pending_exception_offset_in_bytes());
    Address exception_file_offset_addr(Gthread, 0, Thread::exception_file_offset_in_bytes());
    Address exception_line_offset_addr(Gthread, 0, Thread::exception_line_offset_in_bytes());

    // set pending exception
    __ verify_oop(Oexception, "generate_catch_exception");
    __ verify_thread();
    __ st(Oexception, pending_exception_addr);
    __ set((int)__FILE__, temp_reg);
    __ st(temp_reg, exception_file_offset_addr);
    __ set((int)__LINE__, temp_reg);
    __ st(temp_reg, exception_line_offset_addr);

    // complete return to VM
    assert(StubRoutines::_call_stub_return_address != NULL, "must have been generated before");

    __ jump_to(Address(temp_reg, StubRoutines::_call_stub_return_address));
    __ delayed()->nop();

    return start;
  }


  //----------------------------------------------------------------------------------------------------
  // Continuation point for runtime calls returning with a pending exception
  // The pending exception check happened in the runtime or native call stub
  // The pending exception in Thread is converted into a Java-level exception
  //
  // Contract with Java-level exception handler: O0 = exception
  //                                  		 O1 = throwing pc

  address StubGenerator::generate_forward_exception() {
    StubCodeMark mark(this, "StubRoutines", "forward_exception");
    address start = __ pc();

    // Upon entry, O7 has the return address returning into Java
    // (interpreted or compiled) code; i.e. the return address
    // becomes the throwing pc.

    const Register& handler_reg = Gtemp;

    Address exception_addr (Gthread, 0, Thread::pending_exception_offset_in_bytes());

#ifdef ASSERT
    // make sure that this code is only executed if there is a pending exception
    { Label L;
      __ ld(exception_addr, Gtemp);
      __ tst(Gtemp);
      __ br(Assembler::notEqual, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("StubRoutines::forward exception: no pending exception (1)");
      __ bind(L);
    }
#endif

    // compute exception handler into handler_reg
    __ get_thread();
    __ ld(exception_addr, Oexception);
    __ verify_oop(Oexception, "generate_forward_exception");
    __ save_frame(0);             // compensates for compiler weakness
    __ add(O7.after_save(), frame::pc_return_offset * BytesPerWord, Lscratch); // save the issuing PC
    __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SharedRuntime::exception_handler_for_return_address), Lscratch);
    __ mov(O0, handler_reg);
    __ restore();                 // compensates for compiler weakness

    __ ld(exception_addr, Oexception);
    __ add(O7, frame::pc_return_offset * BytesPerWord, Oissuing_pc); // save the issuing PC

#ifdef ASSERT
    // make sure exception is set
    { Label L;
      __ tst(Oexception);
      __ br(Assembler::notEqual, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("StubRoutines::forward exception: no pending exception (2)");
      __ bind(L);
    }
#endif
    // jump to exception handler
    __ jmp(handler_reg, 0);
    // clear pending exception
    __ delayed()->st(G0, exception_addr);

    return start;
  }


  //----------------------------------------------------------------------------------------------------
  // The following routine generates a subroutine to throw a null reference exception.
  // 
  //
  // Arguments :
  //     (The O7 value is dug out of the stack by the C code.)
  //
  // Results:
  //
  //  throws a new exception object to the given return address

  // catch exception if receiver is NULL

  address StubGenerator::generate_throw_null_exception() {
    StubCodeMark mark(this, "StubRoutines", "throw_null_exception");
    address start = __ pc();

    __ untested("throw_null_exception");  // not used yet; see Intel code.

    generate_runtime_stub(RET_JUMP, Oexception,
			  CAST_FROM_FN_PTR(address, SharedRuntime::get_null_exception_entry_and_oop),
			  no_stub_flags);
    // Oissuing_pc is still live at the jump, and is used by the handler.

    return start;
  }


  //-------------------------------------------------------------------
  // Called when an implicit stack overflow exception happens
  // %%% The following comments apply to Intel not necessarily SPARC.
  //
  // The stack looks as follows:
  //
  //  +--------+   <- esp
  //  |        |            <== return address offset words
  //  |        |          
  //  +--------+
  //  | retaddr|
  //  +--------+
  //
  //  In other to have correct frame chain, we must first remove
  //  the words between esp, and retaddr, leaving retaddr on top of stack.
  //  The size of that area is defined by the return address offset of
  //  the nmethod that contains saved_eception_pc

#ifdef COMPILER2
  address generate_stack_overflow_stub() {
    StubCodeMark mark(this, "StubRoutines", "handler_for_stack_overflow_stub");
    address start = __ pc();
    if (OptoRuntime::saved_exception_rao_stub() == NULL) {
      __ stop("wrong stack overflow exception");
    } else {
      // need to remove the data from stack first (exposing the return address)
      // __ call_VM_leaf(Lthread_cache, OptoRuntime::saved_exception_rao_stub(), relocInfo::none);
      // O0 is return address offset from SP (in bytes)
      Address error_stub(G1, OptoRuntime::throw_stack_overflow_error_stub());
      __ jump_to(error_stub);
      __ delayed()->restore();
    }
    return start;
  }
#endif

  //------------------------------------------------------------------------------------------------------------------------
  // Handler for implicit exceptions forwarded by the OS
  //
  // How it works: Implicit exceptions are forwarded by the OS to this
  // handler, as if by a simple branch.  All registers remain intact.
  // The trap handler sets JavaThread::saved_exception_pc() to show
  // where the trap came from.
  //
  // Note 1: This solution automatically handles cases where the underlying
  //         nmethod (if any) is deoptimized, since VM calls must be able to
  //         handle such a situation anyway.
  //
  // Note 2: In case of an exception in the interpreter, the Lbcp (L1) value
  //         has to be saved in the stack frame (calling convention for
  //         interpreter calls). This happens automatically as a result
  //         of register window saving.  However, if the call_VM_base
  //         method of InterpreterMacroAssembler were to perform explicit
  //         state saving, it would need to be emulated here also.
  //
  // Note 3: The previous value of O7 is saved in JavaThread::saved_accumulator().
  //         It must used if the null exception arose in the middle of
  //         a call (e.g., a null receiver getting its type checked).
  //
  // Arguments :
  //
  //      trapping PC:  O7 (may need to be adjusted if an inline cache check)
  //
  // Results:
  //     initiates exception processing; does not return
  //
  address generate_handler_for_implicit_exception(address throw_interpreted, address throw_compiled, address *filter, bool is_implicit_null) {
    StubCodeMark mark(this, "StubRoutines", "handler_for_implicit_exception");
    address start = __ pc();

    // In general we don't care about stack contents (or that tos
    // is saved in O0 for the interpreter) since the stack will
    // be reset when the exception is thrown. In particular, we
    // don't need to preserve call arguments accross a GC.

    Address saved_exception_pc(Gthread, 0, JavaThread::saved_exception_pc_offset_in_bytes());

    Label non_interp;
    Label retry;
    Label compiler;

    __ bind(retry);
    __ save_frame(0);         // do not blow any registers!
    __ mov(G1, L1);
    __ mov(G3, L3);
    __ mov(G4, L4);
    __ mov(G5, L5);
    __ verify_thread();
    
    // INTERPRETER CASE
    // decide if exception happened in interpreter
    __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SharedRuntime::is_interpreter_exception));
    __ tst(O0);
    __ br(Assembler::zero, true, Assembler::pn, non_interp);
    __ delayed()->ld(saved_exception_pc, O0);
    __ set((int)throw_interpreted, O1);
    __ mov(L1, G1);
    __ mov(L3, G3);
    __ mov(L4, G4);
    __ mov(L5, G5);
    __ jmp(O1, 0);
    __ delayed()->restore();

    __ bind(non_interp);

    // check if this is an implicit null_check in the 
    // megamorphic_call stub. if so, we need to set the thread's
    // saved_exception_pc to the caller of the stub and remove 
    // the stub's activation frame. this can be found in I7.
    //__ set((int)StubRoutines::_megamorphic_null_exception_address, O1);
    //__ cmp(O0, O1);
    //__ br(Assembler::notEqual, false, Assembler::pt, compiler);
    //__ delayed()->nop();
    //__ restore();
    //__ st(I7, saved_exception_pc, 0);
    //__ br(Assembler::always, false, Assembler::pt, retry);
    //__ delayed()->restore();

    __ bind(compiler);

    // JVMCI EXCEPTION FILTER
    // register values as at the exception point (except for eip & flags) -
    // call exception filter to filter out exceptions happening in external
    // compilers, returns with untouched register values if exception could
    // not be handled
    //
    // note: eventually, with a fully functional JVMCI we need to try out
    //       all 'installed' exception filters here (if we have 2 compilers
    //       in the system, we would have 2 exception filters to try)
    #ifdef COMPILER1
    {
      __ mov(L1, G1);
      __ mov(L3, G3);
      __ mov(L4, G4);
      __ mov(L5, G5);
      __ set((int)filter, O1);
      __ ld(O1, 0, O1);
      __ callr(O1, 0);
      __ delayed()->nop();
      __ should_not_reach_here();
    }
    #endif


    // COMPILED CODE CASE - BOTH IMPLICIT AND INLINE-CACHE/VTABLE-STUB
    // exception happened outside of interpreter => compute exception
    // return address, if any.  Must not blow any registers; the implicit
    // exception acts as if it took a long-latency branch to the final target.

    { Label L;
      if (is_implicit_null) {
        __ mov(1, O0);
      } else {
        __ clr(O0);
      }
      __ call_VM_leaf(Lthread_cache, CAST_FROM_FN_PTR(address, SharedRuntime::compute_exception_return_address), O0);
      __ tst(O0);
      __ br(Assembler::zero, false, Assembler::pn, L);
      __ delayed()->nop();

      __ mov(L1, G1);
      __ mov(L3, G3);
      __ mov(L4, G4);
      __ mov(L5, G5);
      // take a branch (a very slow branch) to the indicated label
      __ jmp(O0, G0);
      __ delayed()->restore();

      __ bind(L);
    }

    // VTABLE-STUB OR INLINE_CACHE CASE

    // An implicit exception in compiled code may happen from the
    // vtable stub or inline cache check.  We handle it similiarly to
    // handle_wrong_method, except that we throw an exception.
    // The runtime code must verify that the exception happened in the stubs.

    if (throw_compiled == NULL) {
      // running with -Xint
      __ stop("wrong implicit exception");
    } else {
       // The exception PC which is the frame above vtable or unverified
       // entry point has been saved in the context save area
       // by the trap Handler just before calling into the stub, we restore
       // it in O7 so that the rest of the exception system can handle
       // the exception correctly
       // restoring %g's not strictly necessary
       __ mov(L1, G1);
       __ mov(L3, G3);
       __ mov(L4, G4);
       __ mov(L5, G5);
       __ set((int)throw_compiled, Gscratch);
       __ jmp(Gscratch, relocInfo::none);
       __ delayed()->restore();
    }

    return start;
  }


  #ifndef CORE
  //----------------------------------------------------------------------------------------------------
  // Explicit throwing of exceptions
  //
  // Arguments :
  //
  //         O7: stub address where called from (to locate the method)
  //         O0: offset of exception occurence (from begin of method code)
  // 
  // Results:
  //
  //         exception pc  in Oissuing_pc
  //         exception oop in Oexception
  //         jumps to exception handler (does not return to the caller)

  address generate_division_by_zero() {
    StubCodeMark mark(this, "StubRoutines", "division_by_zero");
    address start = __ pc();

    __ unimplemented("division_by_zero");
    return start;
  }
  #endif


  // Generate a routine that sets all the registers so we
  // can tell if the stop routine prints them correctly.
  address StubGenerator::generate_test_stop() {
    StubCodeMark mark(this, "StubRoutines", "test_stop");
    address start = __ pc();

    int i;

    __ save_frame(0);

    static jfloat zero = 0.0, one = 1.0;

    // put addr in L0, then load through L0 to F0
    __ set((int)&zero, L0);  __ ldf( FloatRegister::S, L0, 0, F0);
    __ set((int)&one,  L0);  __ ldf( FloatRegister::S, L0, 0, F1); // 1.0 to F1

    // use add to put 2..18 in F2..F18
    for ( i = 2;  i <= 18;  ++i ) 
      __ fadd( FloatRegister::S, F1, FloatRegister(i-1, ' '),  FloatRegister(i, ' '));

    // Now put double 2 in F16, double 18 in F18
    __ ftof( FloatRegister::S, FloatRegister::D, F2, F16 );
    __ ftof( FloatRegister::S, FloatRegister::D, F18, F18 );

    // use add to put 20..32 in F20..F32 
    for (i = 20; i < 32; i += 2)
      __ fadd( FloatRegister::D, F16, FloatRegister(i-2, ' '),  FloatRegister(i, ' '));


    // put 0..7 in i's, 8..15 in l's, 16..23 in o's, 24..31 in g's
    for ( i = 0; i < 8; ++i ) {
      if (i < 6) {
	__ set(     i, Register('i', i));
	__ set(16 + i, Register('o', i));
	__ set(24 + i, Register('g', i));
      }
      __ set( 8 + i, Register('l', i));
    }

    __ stop("testing stop");


    __ ret();
    __ delayed()->restore();

    return start;
  }


  address StubGenerator::generate_stop_subroutine() {
    StubCodeMark mark(this, "StubRoutines", "stop_subroutine");
    address start = __ pc();

    __ stop_subroutine();

    return start;
  }

  address StubGenerator::generate_flush_callers_register_windows() {
    StubCodeMark mark(this, "StubRoutines", "flush_callers_register_windows");
    address start = __ pc();

    __ flush_windows();
    __ retl();
    __ delayed()->mov(FP, O0);
    // The returned value must be a stack pointer whose register save area
    // is flushed, and will stay flushed while the caller executes.

    return start;
  }

  // Helper functions for v8 atomic operations.
  // 
  void StubGenerator::get_v8_oop_lock_ptr(Register lock_ptr_reg, Register mark_oop_reg, Register scratch_reg) {
    if (mark_oop_reg == noreg) {
      address lock_ptr = (address)StubRoutines::sparc::atomic_memory_operation_lock_addr();
      __ set((int)lock_ptr, lock_ptr_reg);
    } else {
      assert(scratch_reg != noreg, "just checking");
      address lock_ptr = (address)StubRoutines::sparc::_v8_oop_lock_cache;
      __ set((int)lock_ptr, lock_ptr_reg);
      __ and3(mark_oop_reg, StubRoutines::sparc::v8_oop_lock_mask_in_place, scratch_reg);
      __ add(lock_ptr_reg, scratch_reg, lock_ptr_reg);
    }
  }

  void StubGenerator::generate_v8_lock_prologue(Register lock_reg, Register lock_ptr_reg, Register yield_reg, Label& retry, Label& dontyield, Register mark_oop_reg = noreg, Register scratch_reg = noreg) {

    get_v8_oop_lock_ptr(lock_ptr_reg, mark_oop_reg, scratch_reg);
    __ set(StubRoutines::sparc::locked, lock_reg);
    // Initialize yield counter
    __ mov(G0,yield_reg);

    __ bind(retry);
    __ cmp(yield_reg, V8AtomicOperationUnderLockSpinCount);
    __ br(Assembler::less, false, Assembler::pt, dontyield);
    __ delayed()->nop();

    // This code can only be called from inside the VM, this
    // stub is only invoked from os:atomic_increment(). We do
    // not want to use call_VM, because _last_java_sp and such
    // must already be set.
    // 
    // Save the regs and make space for a C call
    __ save(SP, -96, SP);
    __ save_all_globals_into_locals();
    __ call(CAST_FROM_FN_PTR(address, os::Solaris::naked_sleep));
    __ delayed()->nop();
    __ restore_globals_from_locals();
    __ restore();
    // reset the counter
    __ mov(G0,yield_reg);

    __ bind(dontyield);

    // try to get lock
    __ swap(lock_ptr_reg, 0, lock_reg);

    // did we get the lock?
    __ cmp(lock_reg, StubRoutines::sparc::unlocked);
    __ br(Assembler::notEqual, true, Assembler::pn, retry);
    __ delayed()->add(yield_reg,1,yield_reg);

    // yes, got lock. do the operation here.
  }

  void StubGenerator::generate_v8_lock_epilogue(Register lock_reg, Register lock_ptr_reg, Register yield_reg, Label& retry, Label& dontyield, Register mark_oop_reg = noreg, Register scratch_reg = noreg) {
    __ st(lock_reg, lock_ptr_reg, 0); // unlock
  }

  // Support for jint os::atomic_exchange(jint* dest, jint exchange_value).
  // Note that the stub arguments are in reverse order.
  //
  // Arguments :
  //
  //      exchange_value: O0
  //      dest:           O1
  //
  // Results:
  //
  //     O0: the value previously stored in dest
  //
  address StubGenerator::generate_atomic_exchange() {
    StubCodeMark mark(this, "StubRoutines", "atomic_exchange");
    address start = __ pc();

    if (UseCASForSwap) {
      // Use CAS instead of swap, just in case the MP hardware
      // prefers to work with just one kind of synch. instruction.
      Label retry;
      __ bind(retry);
      __ mov(O0, O3);       // scratch copy of exchange value
      __ ld(O1, 0, O2);     // observe the previous value
      // try to replace O2 with O3
      __ cas_under_lock(O1, O2, O3, 
      (address)StubRoutines::sparc::atomic_memory_operation_lock_addr(),false);
      __ cmp(O2, O3);
      __ br(Assembler::notEqual, false, Assembler::pn, retry);
      __ delayed()->nop();

      __ retl();
      __ delayed()->mov(O2, O0);  // report previous value to caller

    } else {
      if (VM_Version::v9_instructions_work()) {
        __ retl();
        __ delayed()->swap(O1, 0, O0);
      } else {
        const Register& lock_reg = O2;
        const Register& lock_ptr_reg = O3;
        const Register& yield_reg = O4;
  
        Label retry;
        Label dontyield;

        generate_v8_lock_prologue(lock_reg, lock_ptr_reg, yield_reg, retry, dontyield);
        // got the lock, do the swap
        __ swap(O1, 0, O0);

        generate_v8_lock_epilogue(lock_reg, lock_ptr_reg, yield_reg, retry, dontyield);
        __ retl();
        __ delayed()->nop();
      }
    }

    return start;
  }

  // Support for void os::atomic_store(jint* dest, jint store_value).
  // Note that the stub arguments are in reverse order.
  //
  // Arguments :
  //
  //      store_value: O0
  //      dest:        O1
  //
  address StubGenerator::generate_atomic_store() {
    StubCodeMark mark(this, "StubRoutines", "atomic_store");
    address start = __ pc();

    if (VM_Version::v9_instructions_work()) {
      __ retl();
      __ delayed()->st(O0, O1, 0);
    } else {
      const Register& lock_reg = O2;
      const Register& lock_ptr_reg = O3;
      const Register& yield_reg = O4;

      Label retry;
      Label dontyield;

    // initialize v8_oop_lock_cache
    int i;
    for(i = 0; i < StubRoutines::sparc::nof_v8_oop_lock_cache_entries; i++)
        StubRoutines::sparc::_v8_oop_lock_cache[i] = StubRoutines::sparc::unlocked;
      generate_v8_lock_prologue(lock_reg, lock_ptr_reg, yield_reg, retry, dontyield, O1, O5);
      // got the lock, do the store
      __ st(O0, O1, 0);

      generate_v8_lock_epilogue(lock_reg, lock_ptr_reg, yield_reg, retry, dontyield, O1, O5);
      __ retl();
      __ delayed()->nop();
    }

    return start;
  }

  // NOTE: a new cas_under_lock function in MacroAssembler provides
  // the functionality needed for compare_and_swap for both v8 and
  // v9. This function might be obsoleted in the near future.

  // Support for jint atomic_compare_and_exchange(jint* dest, jint exchange_value, jint compare_value)
  // Note that the first two stub arguments are in reverse order.
  //
  // Arguments :
  //
  //      exchange_value: O0
  //      dest:           O1
  //      compare_value:  O2
  //
  // Results:
  //
  //     O0: the value previously stored in dest
  //
  // Overwrites (v8): O3,O4,O5
  //
  address StubGenerator::generate_atomic_compare_and_exchange() {
    StubCodeMark mark(this, "StubRoutines", "atomic_compare_and_exchange");
    address start = __ pc();

    // compare_and_swap(dest, compare_value, exchange_value)
    __ cas_under_lock(O1, O2, O0,
      (address)StubRoutines::sparc::atomic_memory_operation_lock_addr(),false);
    __ retl();
    __ delayed()->nop();

    return start;
  }


  // Support for jint os::atomic_increment(jint* dest).
  //     and for jint os::atomic_decrement(jint* dest).
  // Note that the stub arguments are in reverse order.
  //
  // Arguments :
  //
  //      inc: O0   (i.e., +1 or -1)
  //      loc: O1
  //
  // Results:
  //
  //     O0: the new value stored in dest
  //
  // Overwrites (v9): O3
  // Overwrites (v8): O3,O4,O5
  //
  address StubGenerator::generate_atomic_increment() {
    StubCodeMark mark(this, "StubRoutines", "atomic_increment");
    address start = __ pc();
    __ bind(_atomic_increment_stub);

    if (VM_Version::v9_instructions_work()) {
      Label(retry);
      __ bind(retry);

      __ ld(O1, 0, O2);
      __ add(O0,   O2, O3);
      __ cas(O1,   O2, O3);
      __ cmp(      O2, O3);
      __ br(Assembler::notEqual, false, Assembler::pn, retry);
      __ delayed()->nop();
      __ retl();
      __ delayed()->add(O0, O2, O0); // note that cas made O2==O3
    } else {
      const Register& lock_reg = O2;
      const Register& lock_ptr_reg = O3;
      const Register& value_reg = O4;
      const Register& yield_reg = O5;

      Label(retry);
      Label(dontyield);

      generate_v8_lock_prologue(lock_reg, lock_ptr_reg, yield_reg, retry, dontyield);
      // got lock, do the increment
      __ ld(O1, 0, value_reg);
      __ add(O0, value_reg, value_reg);
      __ st(value_reg, O1, 0);

      // %%% only for RMO and PSO
      __ membar(Assembler::StoreStore); 

      generate_v8_lock_epilogue(lock_reg, lock_ptr_reg, yield_reg, retry, dontyield);

      __ retl();
      __ delayed()->mov(value_reg, O0);
    }

    return start;
  }
  Label _atomic_increment_stub;  // called from other stubs


  // type:  long long  long_load_atomic( long long* p );
  address StubGenerator::generate_long_load_atomic() {
    StubCodeMark mark(this, "StubRoutines", "long_load_atomic");
    address start = __ pc();

    if (VM_Version::v9_instructions_work()) {
      __ ldx(O0, 0, O1);
      __ retl();
      __ delayed()->srlx(O1, 32, O0); // fetch the high half into O0
    } else {
      __ ldd(O0, 0, O0);
    }

    return start;
  }


  // type: void  long_store_atomic( long long x,  long long* p );
  //   or: void  long_store_atomic( int h, int l, long long* p );
  address StubGenerator::generate_long_store_atomic() {
    StubCodeMark mark(this, "StubRoutines", "long_store_atomic");
    address start = __ pc();

    if (!VM_Version::v8_instructions_work()) {
      // clear the top bits of O1, and scale up O0
      __ clruw(O1, O1);
      __ sllx(O0, 32, O0);
      // combine the two halves into the 64 bits of O0
      __ or3(O0, O1, O0);
      __ retl();
      __ delayed()->stx(O0, O2, 0);
    } else {
      __ std(O0, O2, 0);
    }

    return start;
  }



#ifndef CORE
  //------------------------------------------------------------------------------------------------------------------------
  // The following routine generates a subroutine to throw an array-index-out-of-bounds exception
  // (This is currently a SPARC-only routine, but it could perhaps be
  // carried over to Intel.  It pushes the offending array index into
  // the exception processing machinery, so that the index can appear
  // in the exception's message.)
  //
  //
  // Arguments :
  //
  //      trapping PC:    O7
  //      array length:   Garg0
  //      (index-length): Garg1
  //
  // Results:
  //     initiates exception processing; does not return
  //

  address generate_handler_for_array_index_check() {
    StubCodeMark mark(this, "StubRoutines", "handler_for_array_index_check");
    address start = __ pc();

    const Register length_reg = Garg0;
    const Register diff_reg   = Garg1;
    const Register index_reg  = Garg1;
    const Register temp_reg   = O2;

    __ untested("handler_for_array_index_check");

    __ add(length_reg, diff_reg, index_reg);

    __ save_frame(0);

    const int pc_adj = frame::pc_return_offset * BytesPerWord;
    __ call_VM(temp_reg, CAST_FROM_FN_PTR(address, SharedRuntime::handle_array_index_check), index_reg);
    __ add(I7, pc_adj, Oissuing_pc.after_save());
    __ jmp(O0, 0);
    __ delayed()->restore(temp_reg, 0, Oexception);

    return start;
  }
#endif


  // Called from MacroAssembler::verify_oop
  //
  address generate_verify_oop_subroutine() {
    StubCodeMark mark(this, "StubRoutines", "verify_oop_stub");
     
    address start = __ pc();
    
    __ verify_oop_subroutine();
    
    return start;
  }


  //-----------------------------------------------------------------
  // Called when an interpreter call needs an i2c adapter before
  // continuing in the compiled callee. Thi sis typically called from a vtable
  //
  // 
  // ebx: methodOop
  //

#ifdef COMPILER2
  address generate_lazy_i2c_adapter_entry() {
    StubCodeMark mark(this, "StubRoutines", "lazy_i2c_adapter_entry");
    address start = __ pc();

    generate_runtime_stub(RET_JUMP, Gmethod,
			  CAST_FROM_FN_PTR(address, SharedRuntime::create_i2c_adapter),
			  no_stub_flags,
			  Gmethod);
    // O0:      i2c adapter destination 
    // Gmethod: methodOop

    return start;
  }
#endif // COMPILER2


  void generate_initial() {
    // Generates all stubs and initializes the entry points

    //------------------------------------------------------------------------------------------------------------------------
    // entry points that exist in all platforms
    // Note: This is code that could be shared among different platforms - however the benefit seems to be smaller than
    //       the disadvantage of having a much more complicated generator structure. See also comment in stubRoutines.hpp.
    //StubRoutines::_verify_oop_entry                      = generate_verify_oop(); // this must come first since it is used later
    StubRoutines::_forward_exception_entry                 = generate_forward_exception();

    StubRoutines::_throw_null_exception_entry              = generate_throw_null_exception();

    StubRoutines::_call_stub_entry                         = generate_call_stub(StubRoutines::_call_stub_return_address);    
    StubRoutines::_catch_exception_entry                   = generate_catch_exception();

    COMPILER2_ONLY(StubRoutines::_lazy_i2c_adapter_entry   = generate_lazy_i2c_adapter_entry();)

    //------------------------------------------------------------------------------------------------------------------------
    // entry points that are platform specific
    StubRoutines::sparc::_test_stop_entry                  = generate_test_stop();

    StubRoutines::sparc::_stop_subroutine_entry            = generate_stop_subroutine();
    StubRoutines::sparc::_flush_callers_register_windows_entry = generate_flush_callers_register_windows();

    StubRoutines::_atomic_exchange_entry                   = generate_atomic_exchange();
    StubRoutines::_atomic_store_entry                      = generate_atomic_store();
    StubRoutines::_atomic_compare_and_exchange_entry       = generate_atomic_compare_and_exchange();
    StubRoutines::_atomic_increment_entry                  = generate_atomic_increment();
    StubRoutines::sparc::_long_load_atomic_entry           = generate_long_load_atomic();
    StubRoutines::sparc::_long_store_atomic_entry          = generate_long_store_atomic();
  }


  void generate_all() {
    // Generates all stubs and initializes the entry points

    //------------------------------------------------------------------------------------------------------------------------
    // entry points that are platform specific
    StubRoutines::sparc::_handler_for_divide_by_zero_entry =
      generate_handler_for_implicit_exception(
        Interpreter::throw_ArithmeticException_entry(),
        NOT_COMPILER2(NULL) COMPILER2_ONLY(OptoRuntime::throw_div0_exception_stub()),
        &SharedRuntime::_implicit_div0_exception_filter,
        false
      );
 
    StubRoutines::sparc::_handler_for_null_exception_entry =
      generate_handler_for_implicit_exception(
        Interpreter::throw_NullPointerException_entry(),
        NOT_COMPILER2(NULL) COMPILER2_ONLY(OptoRuntime::throw_null_exception_stub()),
        &SharedRuntime::_implicit_null_exception_filter,
        true
      );
 
    StubRoutines::sparc::_handler_for_stack_overflow_entry =
      generate_handler_for_implicit_exception(
        Interpreter::throw_StackOverflowError_entry(),
        NOT_COMPILER2(NULL) COMPILER2_ONLY(generate_stack_overflow_stub()),
        &SharedRuntime::_stack_ovrflow_exception_filter,
        false
      );

    NOT_CORE(StubRoutines::sparc::_handler_for_array_index_check_entry = generate_handler_for_array_index_check();)


    // support for verify_oop (must happen after universe_init)
    StubRoutines::sparc::_verify_oop_subroutine_entry	   = generate_verify_oop_subroutine();
  }


  class StubMacroAssembler: public MacroAssembler {
    StubGenerator* _sg;

   public:
    StubMacroAssembler(CodeBuffer* code, StubGenerator* sg) : MacroAssembler(code) {
      _sg = sg;
    }

    // Stub generator version of call_VM_base.
    // It differs in two ways from the standard one:
    // First, it uses this module's standard "Lthread_cache".
    // Second, the last_Java_sp to record is actually the FP,
    // since all stubs save their own private register windows.
    virtual void call_VM_base(
      Register        oop_result,
      Register        java_thread,
      Register        last_java_sp,
      address         entry_point,
      int             number_of_arguments,
      bool	      check_exception
    ) {
      assert(!java_thread.isValid(), "checking");
      java_thread = Lthread_cache;

      assert(!last_java_sp.isValid(), "checking");
      last_java_sp = FP;

      MacroAssembler::call_VM_base(oop_result, java_thread, last_java_sp, entry_point, number_of_arguments, Assembler::_check_for_exceptions_flag);

    }
  };
  friend StubMacroAssembler;

 public:
  StubGenerator(CodeBuffer* code, bool all) : StubCodeGenerator(code) {
    // replace the standard masm with a special one:
    _masm = new StubMacroAssembler(code, this);

    _stub_count = !all ? 0x100 : 0x200;
    if (all) {
      generate_all();
    } else {
      generate_initial();
    }

    // make sure this stub is available for all local cals
    if (_atomic_increment_stub.is_unbound()) {
      // generate a second time, if necessary
      (void) generate_atomic_increment();
    }
  }


 private:
  int _stub_count;
  void stub_prolog(StubCodeDesc* cdesc) {
    # ifdef ASSERT
      // put extra information in the stub code, to make it more readable
      __ emit_data((int)cdesc,    relocInfo::none);
      __ emit_data(++_stub_count, relocInfo::none);
    # endif
    align(true);
  }

  void align(bool at_header = false) {
    // %%%%% move this constant somewhere else
    // UltraSPARC cache line size is 8 instructions:
    const unsigned int icache_line_size = 32;
    const unsigned int icache_half_line_size = 16;

    if (at_header) {
      while ((int)(__ pc()) % icache_line_size != 0) {
	__ emit_data(0, relocInfo::none);
      }
    } else {
      while ((int)(__ pc()) % icache_half_line_size != 0) {
	__ nop();
      }
    }
  }

}; // end class declaration


void StubGenerator_generate(CodeBuffer* code, bool all) {
  StubGenerator g(code, all);
}


//Reconciliation History
// 1.11 97/07/03 14:03:24 stubRoutines_i486.cpp
// 1.17 97/07/14 16:43:44 stubRoutines_i486.cpp
// 1.7 97/08/06 11:02:34 stubGenerator_i486.cpp
// 1.15 97/08/20 03:20:39 stubGenerator_i486.cpp
// 1.25 97/09/11 17:20:56 stubGenerator_i486.cpp
// 1.26 97/09/11 18:14:09 stubGenerator_i486.cpp
// 1.30 97/10/06 16:08:07 stubGenerator_i486.cpp
// 1.37 97/10/16 15:47:08 stubGenerator_i486.cpp
// 1.42 97/10/21 11:54:32 stubGenerator_i486.cpp
// 1.54 97/10/28 17:55:26 stubGenerator_i486.cpp
// 1.63 97/11/06 08:49:24 stubGenerator_i486.cpp
// 1.68 97/11/11 14:54:45 stubGenerator_i486.cpp
// 1.81 97/11/19 17:27:45 stubGenerator_i486.cpp
// 1.75 97/12/03 18:51:59 stubGenerator_i486.cpp
// 1.77 97/12/08 10:50:13 stubGenerator_i486.cpp
// 1.92 98/01/06 15:24:49 stubGenerator_i486.cpp
// 1.97 98/01/14 11:14:58 stubGenerator_i486.cpp
// 1.101 98/01/16 23:04:07 stubGenerator_i486.cpp
// 1.112 98/01/28 12:13:26 stubGenerator_i486.cpp
// 1.119 98/02/05 13:40:24 stubGenerator_i486.cpp
// 1.140 98/02/26 18:46:40 stubGenerator_i486.cpp
// 1.146 98/03/12 16:19:59 stubGenerator_i486.cpp
// 1.147 98/03/17 11:33:10 stubGenerator_i486.cpp
// 1.149 98/03/19 18:59:29 stubGenerator_i486.cpp
// 1.151 98/03/26 10:37:35 stubGenerator_i486.cpp
// 1.152 98/03/27 09:49:48 stubGenerator_i486.cpp
// 1.154 98/04/07 15:43:24 stubGenerator_i486.cpp
// 1.155 98/04/09 16:49:29 stubGenerator_i486.cpp
// 1.161 98/04/24 12:52:59 stubGenerator_i486.cpp
// 1.163 98/05/05 14:42:07 stubGenerator_i486.cpp
// 1.168 98/05/11 12:59:19 stubGenerator_i486.cpp
// 1.174 98/05/19 16:56:57 stubGenerator_i486.cpp
// 1.179 98/05/22 11:38:48 stubGenerator_i486.cpp
// 1.182 98/05/26 17:33:00 stubGenerator_i486.cpp
// 1.184 98/06/01 11:28:51 stubGenerator_i486.cpp
// 1.185 98/06/03 09:33:43 stubGenerator_i486.cpp
// 1.194 98/06/17 08:19:08 stubGenerator_i486.cpp
// 1.198 98/06/19 10:07:35 stubGenerator_i486.cpp
// 1.206 98/07/02 13:50:58 stubGenerator_i486.cpp
// 1.209 98/07/24 15:30:08 stubGenerator_i486.cpp
// 1.210 98/08/13 16:30:08 stubGenerator_i486.cpp
// 1.211 98/08/17 17:03:34 stubGenerator_i486.cpp
// 1.215 98/08/28 11:31:05 stubGenerator_i486.cpp
// 1.218 98/09/02 12:58:26 stubGenerator_i486.cpp
// 1.220 98/10/02 15:34:01 stubGenerator_i486.cpp
// 1.222 98/10/06 14:41:06 stubGenerator_i486.cpp
// 1.228 98/10/21 18:36:46 stubGenerator_i486.cpp
// 1.229 98/10/22 17:38:15 stubGenerator_i486.cpp
// 1.230 98/10/27 13:56:30 stubGenerator_i486.cpp
// 1.232 98/11/17 14:00:24 stubGenerator_i486.cpp
// 1.235 98/11/23 10:31:18 stubGenerator_i486.cpp
// 1.236 98/11/24 10:30:45 stubGenerator_i486.cpp
// 1.238 98/12/10 16:47:33 stubGenerator_i486.cpp
// 1.239 99/01/06 16:19:00 stubGenerator_i486.cpp
// 1.240 99/01/08 09:04:34 stubGenerator_i486.cpp
// 1.242 99/01/15 16:45:34 stubGenerator_i486.cpp
// 1.244 99/01/18 17:43:18 stubGenerator_i486.cpp
// 1.247 99/01/25 11:50:15 stubGenerator_i486.cpp
// 1.248 99/03/02 16:52:12 stubGenerator_i486.cpp
// 1.250 99/03/30 14:59:45 stubGenerator_i486.cpp
// 1.252 99/04/06 01:07:28 stubGenerator_i486.cpp
// 1.254 99/06/28 09:59:23 stubGenerator_i486.cpp
// 1.251 99/04/01 16:53:04 stubGenerator_i486.cpp
// 1.252 99/04/06 01:07:28 stubGenerator_i486.cpp
// 1.255 99/07/06 16:03:00 stubGenerator_i486.cpp
// 1.257 99/07/20 10:58:46 stubGenerator_i486.cpp
// 1.2 99/07/30 13:53:10 stubGenerator_i486.cpp
// 1.4 99/08/16 13:11:52 stubGenerator_i486.cpp
//End
