/*
 * @(#)stubRoutines_sparc.hpp	1.45 00/05/08 16:03:38
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// This file holds the platform specific parts of the StubRoutines
// definition. See stubRoutines.hpp for a description on how to
// extend it.

enum /* platform_dependent_constants */ {
  // %%%%%%%% May be able to shrink this a lot 
  code_size1 = 20000,                                        // simply increase if too small (assembler will crash if too small)
  code_size2 = 20000                                         // simply increase if too small (assembler will crash if too small)
};

#undef sparc
class sparc {
 friend class StubGenerator;

 public:
  enum { nof_instance_allocators = 10 };

  // allocator lock values
  enum {
    unlocked = 0,
    locked   = 1
  };

  enum { 
    v8_oop_lock_ignore_bits = 2, 
    v8_oop_lock_bits = 4, 
    nof_v8_oop_lock_cache_entries = 1 << (v8_oop_lock_bits+v8_oop_lock_ignore_bits), 
    v8_oop_lock_mask = right_n_bits(v8_oop_lock_bits), 
    v8_oop_lock_mask_in_place = v8_oop_lock_mask << v8_oop_lock_ignore_bits
  };

  static int _v8_oop_lock_cache[nof_v8_oop_lock_cache_entries];

 private:
  static address _test_stop_entry;
  static address _stop_subroutine_entry;
  static address _verify_oop_subroutine_entry;
  static address _flush_callers_register_windows_entry;

  static address _long_load_atomic_entry;
  static address _long_store_atomic_entry;

  static address _handler_for_divide_by_zero_entry;
  static address _handler_for_null_exception_entry;
  static address _handler_for_stack_overflow_entry;
  static address _handler_for_array_index_check_entry;

  static int _atomic_memory_operation_lock;

 public:
  // %%% global lock for everyone who needs to use atomic_compare_and_exchange
  // %%% or atomic_increment -- should probably use more locks for more 
  // %%% scalability-- for instance one for each eden space or group of

  // address of the lock for atomic_compare_and_exchange
  static int* atomic_memory_operation_lock_addr() { return &_atomic_memory_operation_lock; }

  // accessor and mutator for _atomic_memory_operation_lock
  static int atomic_memory_operation_lock() { return _atomic_memory_operation_lock; }
  static void set_atomic_memory_operation_lock(int value) { _atomic_memory_operation_lock = value; }

  // test assembler stop routine by setting registers
  static void (*test_stop_entry()) ()                     { return CAST_TO_FN_PTR(void (*)(void), _test_stop_entry); }

  // a subroutine for debugging assembler code
  static address stop_subroutine_entry_address()          { return (address)&_stop_subroutine_entry; }

  // a subroutine for debugging the GC
  static address verify_oop_subroutine_entry_address()    { return (address)&_verify_oop_subroutine_entry; }

  // flushes (all but current) register window
  static jint* (*flush_callers_register_windows_func())() { return CAST_TO_FN_PTR(jint* (*)(void), _flush_callers_register_windows_entry); }

  // 64-bit atomic memory references
  static address long_load_atomic_entry()                 { return _long_load_atomic_entry; }
  static address long_store_atomic_entry()                { return _long_store_atomic_entry; }

  static address handler_for_divide_by_zero_entry()       { return _handler_for_divide_by_zero_entry; }
  static address handler_for_null_exception_entry()       { return _handler_for_null_exception_entry; }
  static address handler_for_stack_overflow_entry()       { return _handler_for_stack_overflow_entry; }
  static address handler_for_array_index_check_entry()    { return _handler_for_array_index_check_entry; }



  // tell whether a fixup might smash a given register
  friend class Register;
  static bool volatile_across_fixups(const Register& r);
};

//Reconciliation History
// 1.24 97/12/08 10:50:20 stubRoutines_i486.hpp
// 1.25 98/01/30 16:00:49 stubRoutines_i486.hpp
// 1.30 98/03/10 08:25:57 stubRoutines_i486.hpp
// 1.31 98/04/09 16:49:23 stubRoutines_i486.hpp
// 1.36 98/05/27 11:18:09 stubRoutines_i486.hpp
// 1.37 98/07/07 13:27:29 stubRoutines_i486.hpp
// 1.38 98/08/06 11:43:15 stubRoutines_i486.hpp
// 1.40 98/08/17 17:04:10 stubRoutines_i486.hpp
// 1.41 98/08/19 14:29:52 stubRoutines_i486.hpp
// 1.40 98/10/06 14:41:12 stubRoutines_i486.hpp
// 1.42 99/06/22 16:37:58 stubRoutines_i486.hpp
// 1.43 99/07/16 11:12:12 stubRoutines_i486.hpp
// 1.44 99/08/06 15:47:24 stubRoutines_i486.hpp
//End
