/*
 * @(#)templateTable_sparc.cpp	1.134 01/02/09 14:43:34
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_templateTable_sparc.cpp.incl"

#define __ _masm->


//----------------------------------------------------------------------------------------------------
// Platform-dependent initialization

void TemplateTable::pd_initialize() {
  // (none)
}


//----------------------------------------------------------------------------------------------------
// Condition conversion
Assembler::Condition ccNot(TemplateTable::Condition cc) {
  switch (cc) {
    case TemplateTable::equal        : return Assembler::notEqual;
    case TemplateTable::not_equal    : return Assembler::equal;
    case TemplateTable::less         : return Assembler::greaterEqual;
    case TemplateTable::less_equal   : return Assembler::greater;
    case TemplateTable::greater      : return Assembler::lessEqual;
    case TemplateTable::greater_equal: return Assembler::less;
  }
  ShouldNotReachHere();
  return Assembler::zero;
}

//----------------------------------------------------------------------------------------------------
// Miscelaneous helper routines 


Address TemplateTable::at_bcp(int offset) {
  assert(_desc->uses_bcp(), "inconsistent uses_bcp information");
  return Address( Lbcp, 0, offset);
}


//----------------------------------------------------------------------------------------------------
// Individual instructions

void TemplateTable::nop() {
  transition(vtos, vtos);
  // nothing to do
}

void TemplateTable::shouldnotreachhere() {
  transition(vtos, vtos);
  __ stop("shouldnotreachhere bytecode");
}

void TemplateTable::aconst_null() {
  transition(vtos, atos);
  __ clr(Otos_i);
}


void TemplateTable::iconst(int value) {
  transition(vtos, itos);
  __ set(value, Otos_i);
}


void TemplateTable::lconst(int value) {
  transition(vtos, ltos);
  assert(value >= 0, "check this code");
  __ set(value, Otos_l2);
  __ clr( Otos_l1);
}


void TemplateTable::fconst(int value) {
  transition(vtos, ftos);
  static float zero = 0.0, one = 1.0, two = 2.0;
  float* p;
  switch( value ) {
   default: ShouldNotReachHere();
   case 0:  p = &zero;  break;
   case 1:  p = &one;   break;
   case 2:  p = &two;   break;
  }
  Address a(Gscratch, (address)p);
  __ sethi(a);
  __ ldf(FloatRegister::S, a, Ftos_f);
}


void TemplateTable::dconst(int value) {
  transition(vtos, dtos);
  static double zero = 0.0, one = 1.0;
  double* p;
  switch( value ) {
   default: ShouldNotReachHere();
   case 0:  p = &zero;  break;
   case 1:  p = &one;   break;
  }
  Address a(Gscratch, (address)p);
  __ sethi(a);
  __ ldf(FloatRegister::D, a, Ftos_d);
}


// %%%%% Should factore most snippet templates across platforms 

void TemplateTable::bipush() {
  transition(vtos, itos);
  __ ldsb( at_bcp(1), Otos_i );
}


void TemplateTable::sipush() {
  transition(vtos, itos);
  __ get_2_byte_integer_at_bcp(1, Gscratch, Otos_i, InterpreterMacroAssembler::Signed);
}


// %%%%% FACTOR ALL ldc's! 
void TemplateTable::ldc() {
  transition(vtos, vtos);
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::ldc));
  __ dispatch_next(vtos);
  // (result may be a float, cannot leave it cached)
}



void TemplateTable::ldc_w() {
  transition(vtos, vtos);
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::ldc_w));
  __ dispatch_next(vtos);
}


void TemplateTable::ldc2_w() {
  transition(vtos, vtos);
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::ldc2_w));
  __ dispatch_next(vtos);
}


void TemplateTable::fast_ldc(TosState state) {
  transition(vtos, state);
  __ ldub(Lbcp, 1, O1);
  __ get_constant_pool(O0);
  const int base_offset = constantPoolOopDesc::header_size() * wordSize;
  __ sll(O1, LogBytesPerWord, O1);
  __ add(O0, base_offset, O0);
  switch (state) {
    case atos:                                           // fall through
    case itos: __ ld(                   O0, O1, Otos_i); break;
    case ftos: __ ldf(FloatRegister::S, O0, O1, Ftos_f); break;
    default  : ShouldNotReachHere();
  }
  __ verify_oop(Otos_i, state);
}


void TemplateTable::fast_ldc_w(TosState state) {
  transition(vtos, state);
  __ get_constant_pool(O0);
  __ get_2_byte_integer_at_bcp(1, Gscratch, O1, InterpreterMacroAssembler::Unsigned);
  const int base_offset = constantPoolOopDesc::header_size() * wordSize;
  __ sll(O1, LogBytesPerWord, O1);
  __ add(O0, base_offset, O0);
  switch (state) {
    case atos:                                           // fall through
    case itos: __ ld(                   O0, O1, Otos_i); break;
    case ftos: __ ldf(FloatRegister::S, O0, O1, Ftos_f); break;
    default  : ShouldNotReachHere();
  }
  __ verify_oop(Otos_i, state);
}


void TemplateTable::fast_ldc2_w(TosState state) {
  transition(vtos, state);
  __ get_constant_pool(O0);
  __ get_2_byte_integer_at_bcp(1, Gscratch, O1,  InterpreterMacroAssembler::Unsigned );
  const int base_offset = constantPoolOopDesc::header_size() * wordSize;
  __ sll(O1, LogBytesPerWord, O1);
  __ add(O0, O1, Gscratch);
  switch (state) {
    case ltos: __ load_long  ( Gscratch, base_offset, Otos_l);  break;
    case dtos: __ load_double( Gscratch, base_offset, Ftos_d);  break;
    // A double can be placed at word-aligned locations in the constant pool.
    // Check out Conversions.java for an example.  
    // Also constantPoolOopDesc::header_size() is 20, which makes it very difficult 
    // to double-align double on the constant pool.  SG, 11/7/97
    default  : ShouldNotReachHere();
  }
}


void TemplateTable::locals_index(Register reg) {
  __ ldub( at_bcp(1), reg );
}


void TemplateTable::locals_index_wide(Register reg) {
  // offset is 2, not 1, because Lbcp points to wide prefix code
  __ get_2_byte_integer_at_bcp(2, Gscratch2, reg, InterpreterMacroAssembler::Unsigned);
}


void TemplateTable::iload() {
  transition(vtos, itos);
  locals_index(Gscratch);
  __ access_local_int( Gscratch, Otos_i );
}

void TemplateTable::lload() {
  transition(vtos, ltos);
  locals_index(Gscratch);
  __ access_local_long( Gscratch, Otos_l );
}


void TemplateTable::fload() {
  transition(vtos, ftos);
  locals_index(Gscratch);
  __ access_local_float( Gscratch, Ftos_f );
}


void TemplateTable::dload() {
  // Doubles are stored on the locals stack backwards.
  // It is as if the word order of local stack elements
  // is determined by the JVM local indexes, not the
  // memory addresses.  The deoptimizer expects to extract
  // double halves in this peculiar order!
  transition(vtos, dtos);
  locals_index(Gscratch);
  __ access_local_double( Gscratch, Ftos_d );
}


void TemplateTable::aload() {
  transition(vtos, atos);
  locals_index(Gscratch);
  __ access_local_int( Gscratch, Otos_i);
}


void TemplateTable::wide_iload() {
  transition(vtos, itos);
  locals_index_wide(Gscratch);
  __ access_local_int( Gscratch, Otos_i );
}


void TemplateTable::wide_lload() {
  transition(vtos, ltos);
  locals_index_wide(Gscratch);
  __ access_local_long( Gscratch, Otos_l );
}


void TemplateTable::wide_fload() {
  transition(vtos, ftos);
  locals_index_wide(Gscratch);
  __ access_local_float( Gscratch, Ftos_f );
}


void TemplateTable::wide_dload() {
  transition(vtos, dtos);
  locals_index_wide(Gscratch);
  __ access_local_double( Gscratch, Ftos_d );
}


void TemplateTable::wide_aload() {
  transition(vtos, atos);
  locals_index_wide(Gscratch);
  __ access_local_int( Gscratch, Otos_i );
  __ verify_oop(Otos_i);
}


void TemplateTable::index_check(Register array, Register index) {
  Unimplemented(); // moved to InterpreterMacroAssembler
}


void TemplateTable::iaload() {
  transition(itos, itos);
  // Otos_i: index
  // tos: array
  __ index_check(O2, Otos_i, LogBytesPerWord, Gscratch, O3);
  __ ld(O3, arrayOopDesc::base_offset_in_bytes(T_INT), Otos_i);
}


void TemplateTable::laload() {
  transition(itos, ltos);
  // Otos_i: index
  // O2: array
  __ index_check(O2, Otos_i, LogBytesPerLong, Gscratch, O3);
  __ ldd(O3, arrayOopDesc::base_offset_in_bytes(T_LONG), Otos_l);
}


void TemplateTable::faload() {
  transition(itos, ftos);
  // Otos_i: index
  // O2: array
  __ index_check(O2, Otos_i, LogBytesPerWord, Gscratch, O3);
  __ ldf(FloatRegister::S, O3, arrayOopDesc::base_offset_in_bytes(T_FLOAT), Ftos_f);
}


void TemplateTable::daload() {
  transition(itos, dtos);
  // Otos_i: index
  // O2: array
  __ index_check(O2, Otos_i, LogBytesPerLong, Gscratch, O3);
  __ ldf(FloatRegister::D, O3, arrayOopDesc::base_offset_in_bytes(T_DOUBLE), Ftos_d);
}


void TemplateTable::aaload() {
  transition(itos, atos);
  // Otos_i: index
  // tos: array
  __ index_check(O2, Otos_i, LogBytesPerWord, Gscratch, O3);
  __ ld(O3, arrayOopDesc::base_offset_in_bytes(T_OBJECT), Otos_i);
  __ verify_oop(Otos_i);
}


void TemplateTable::baload() {
  transition(itos, itos);
  // Otos_i: index
  // tos: array
  __ index_check(O2, Otos_i, 0, Gscratch, O3);
  __ ldsb(O3, arrayOopDesc::base_offset_in_bytes(T_BYTE), Otos_i);
}


void TemplateTable::caload() {
  transition(itos, itos);
  // Otos_i: index
  // tos: array
  __ index_check(O2, Otos_i, LogBytesPerShort, Gscratch, O3);
  __ lduh(O3, arrayOopDesc::base_offset_in_bytes(T_CHAR), Otos_i);
}


void TemplateTable::saload() {
  transition(itos, itos);
  // Otos_i: index
  // tos: array
  __ index_check(O2, Otos_i, LogBytesPerShort, Gscratch, O3);
  __ ldsh(O3, arrayOopDesc::base_offset_in_bytes(T_SHORT), Otos_i);
}


void TemplateTable::iload(int n) {
  transition(vtos, itos);
  __ ld( Llocals, -n * BytesPerWord, Otos_i );
}


void TemplateTable::lload(int n) {
  transition(vtos, ltos);
  assert(n+1 < Argument::n_register_parameters, "would need more code");
  __ load_unaligned_long(Llocals, -(n+1)*BytesPerWord, Otos_l);
}


void TemplateTable::fload(int n) {
  transition(vtos, ftos);
  assert(n < Argument::n_register_parameters, "would need more code");
  __ ldf( FloatRegister::S, Llocals, -n * BytesPerWord,     Ftos_f );
}


void TemplateTable::dload(int n) {
  transition(vtos, dtos);
  FloatRegister dst = Ftos_d;
  __ load_unaligned_double(Llocals, -(n+1)*BytesPerWord, dst);
}


void TemplateTable::aload(int n) {
  transition(vtos, atos);
  __ ld( Llocals, -n * BytesPerWord, Otos_i );
}


void TemplateTable::aload_0() {
  transition(vtos, atos);
  // According to bytecode histograms, the pairs:
  //
  // _aload_0, _fast_igetfield
  // _aload_0, _fast_agetfield
  //
  // occur frequently. If RewriteFrequentPairs is set, the (slow) _aload_0 bytecode checks
  // if the next bytecode is either _fast_igetfield or _fast_agetfield and then rewrites the
  // current bytecode into a pair bytecode; otherwise it rewrites the current bytecode into
  // _fast_aload_0 that doesn't do the pair check anymore.
  //
  // Note: If the next bytecode is _getfield, the rewrite must be delayed, otherwise we
  //       may miss an opportunity for a pair.
  if (RewriteFrequentPairs) {
    Label rewrite, done;

    // get next byte
    __ ldub(at_bcp(Bytecodes::length_for(Bytecodes::_aload_0)), Gscratch);

    // do actual aload_0
    aload(0);

    // if _getfield then wait with rewrite
    __ cmp(Gscratch, (int)Bytecodes::_getfield);
    __ br(Assembler::equal, false, Assembler::pn, done);
    __ delayed()->nop();

    // if _igetfield then rewrite to _fast_iaccess_0
    assert(Bytecodes::java_code(Bytecodes::_fast_iaccess_0) == Bytecodes::_aload_0, "adjust code below");
    __ cmp(Gscratch, (int)Bytecodes::_fast_igetfield);
    __ br(Assembler::equal, false, Assembler::pn, rewrite);
    __ delayed()->set(Bytecodes::_fast_iaccess_0, Gscratch2);

    // if _agetfield then rewrite to _fast_aaccess_0
    assert(Bytecodes::java_code(Bytecodes::_fast_aaccess_0) == Bytecodes::_aload_0, "adjust code below");
    __ cmp(Gscratch, (int)Bytecodes::_fast_agetfield);
    __ br(Assembler::equal, false, Assembler::pn, rewrite);
    __ delayed()->set(Bytecodes::_fast_aaccess_0, Gscratch2);

    // else rewrite to _fast_aload0
    assert(Bytecodes::java_code(Bytecodes::_fast_aload_0) == Bytecodes::_aload_0, "adjust code below");
    __ set(Bytecodes::_fast_aload_0, Gscratch2);

    // rewrite
    // Gscratch2: fast bytecode
    __ bind(rewrite);
    __ stb(Gscratch2, at_bcp(0));
    __ bind(done);
  }
  else {
    aload(0);
  }
}


void TemplateTable::istore() {
  transition(itos, vtos);
  locals_index(Gscratch);
  __ store_local_int( Gscratch, Otos_i );
}


void TemplateTable::lstore() {
  transition(ltos, vtos);
  locals_index(Gscratch);
  __ store_local_long( Gscratch, Otos_l );
}


void TemplateTable::fstore() {
  transition(ftos, vtos);
  locals_index(Gscratch);
  __ store_local_float( Gscratch, Ftos_f );
}


void TemplateTable::dstore() {
  transition(dtos, vtos);
  locals_index(Gscratch);
  __ store_local_double( Gscratch, Ftos_d );
}


void TemplateTable::astore() {
  transition(atos, vtos);
  __ verify_oop_or_return_address(Otos_i, Gscratch);
  locals_index(Gscratch);
  __ store_local_int( Gscratch, Otos_i );
}


void TemplateTable::wide_istore() {
  transition(vtos, vtos);
  __ pop_i();
  locals_index_wide(Gscratch);
  __ store_local_int( Gscratch, Otos_i );
}


void TemplateTable::wide_lstore() {
  transition(vtos, vtos);
  __ pop_l();
  locals_index_wide(Gscratch);
  __ store_local_long( Gscratch, Otos_l );
}


void TemplateTable::wide_fstore() {
  transition(vtos, vtos);
  __ pop_f();
  locals_index_wide(Gscratch);
  __ store_local_float( Gscratch, Ftos_f );
}


void TemplateTable::wide_dstore() {
  transition(vtos, vtos);
  __ pop_d();
  locals_index_wide(Gscratch);
  __ store_local_double( Gscratch, Ftos_d );
}


void TemplateTable::wide_astore() {
  transition(vtos, vtos);
  __ pop_i();
  locals_index_wide(Gscratch);
  __ store_local_int( Gscratch, Otos_i );
}


void TemplateTable::iastore() {
  transition(itos, vtos);
  __ pop_i(O2); // index
  // Otos_i: val
  // O3: array
  __ index_check(O3, O2, LogBytesPerWord, Gscratch, O2);
  __ st(Otos_i, O2, arrayOopDesc::base_offset_in_bytes(T_INT));
}


void TemplateTable::lastore() {
  transition(ltos, vtos);
  __ pop_i(O2); // index
  // Otos_l: val
  // O3: array
  __ index_check(O3, O2, LogBytesPerLong, Gscratch, O2);
  __ std(Otos_l, O2, arrayOopDesc::base_offset_in_bytes(T_LONG));
}


void TemplateTable::fastore() {
  transition(ftos, vtos);
  __ pop_i(O2); // index
  // Ftos_f: val
  // O3: array
  __ index_check(O3, O2, LogBytesPerWord, Gscratch, O2);
  __ stf(FloatRegister::S, Ftos_f, O2, arrayOopDesc::base_offset_in_bytes(T_FLOAT));
}


void TemplateTable::dastore() {
  transition(dtos, vtos);
  __ pop_i(O2); // index
  // Fos_d: val
  // O3: array
  __ index_check(O3, O2, LogBytesPerLong, Gscratch, O2);
  __ stf(FloatRegister::D, Ftos_d, O2, arrayOopDesc::base_offset_in_bytes(T_DOUBLE));
}


void TemplateTable::aastore() {
  transition(vtos, vtos);
  __ ld(Lesp, 1 * wordSize, Otos_i); // get value from stack
  __ ld(Lesp, 2 * wordSize, O2);     // get index
  __ ld(Lesp, 3 * wordSize, O3);     // get array
  // Otos_i: val
  // O2: index
  // O3: array
  __ verify_oop(Otos_i);
  __ index_check_without_pop(O3, O2, LogBytesPerWord, Gscratch, O1);
  
  // do array store check - check for NULL value first
  Label store_ok, store_done;
  __ tst(Otos_i);
  __ br(Assembler::zero, false, Assembler::pn, store_ok);

  // do fast instanceof cache test
  __ delayed()->

     ld(O3,     oopDesc::klass_offset_in_bytes(), O4); // get array klass
  __ ld(Otos_i, oopDesc::klass_offset_in_bytes(), O5); // get value klass

  __ ld(O4,     sizeof(oopDesc) + objArrayKlass::element_klass_offset_in_bytes(),  O4);
  
  assert(Otos_i == O0, "just checking");
  
  // Otos_i:    value
  // O1:        addr - offset
  // O2:        index
  // O3:        array
  // O4:        array element klass
  // O5:        value klass
  // Gscratch:  is_a_cache contents of value klass
  Address element(O1, 0, arrayOopDesc::base_offset_in_bytes(T_OBJECT));

  __ cmp(O4, O5);    // check if classes are identical
  __ br(Assembler::equal, true, Assembler::pt, store_done);
  __ delayed()->st(Otos_i, element);

  __ ld(O5,     sizeof(oopDesc) +         Klass::    is_a_cache_1_offset_in_bytes(),  Gscratch);
  __ cmp(O4, Gscratch);
  __ br(Assembler::equal, true, Assembler::pt, store_done);
  __ delayed()->st(Otos_i, element);

  __ ld(O5,     sizeof(oopDesc) +         Klass::    is_a_cache_2_offset_in_bytes(),  Gscratch);
  __ cmp(O4, Gscratch);
  __ br(Assembler::equal, true, Assembler::pt, store_done);
  __ delayed()->st(Otos_i, element);
  
  
  // cache miss

  __ mov(O2, Lscratch);                // save index
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::array_store_check),
	  Otos_i,
	  O2,
	  O3);

  // restore value and array
  __ ld(Lesp, 1 * wordSize, Otos_i); // get value from stack
  __ ld(Lesp, 3 * wordSize, O3);     // get array
  __ add(O3, Lscratch, O1);           // addr - const offset in index

  __ bind(store_ok);
  __ st(Otos_i, O1, arrayOopDesc::base_offset_in_bytes(T_OBJECT));

  __ bind(store_done);
  __ inc(Lesp, 3 * wordSize);     // adj sp (pops array, index and value)
  // Quote from rememberedSet.hpp: For objArrays, the precise card
  // corresponding to the pointer store is dirtied so we don't need to
  // scavenge the entire array.
  __ add(element, O1);              // address the element precisely
  __ store_check(Gscratch, O1);
}


void TemplateTable::bastore() {
  transition(itos, vtos);
  __ pop_i(O2); // index
  // Otos_i: val
  // O3: array
  __ index_check(O3, O2, 0, Gscratch, O2);
  __ stb(Otos_i, O2, arrayOopDesc::base_offset_in_bytes(T_BYTE));
}


void TemplateTable::castore() {
  transition(itos, vtos);
  __ pop_i(O2); // index
  // Otos_i: val
  // O3: array
  __ index_check(O3, O2, LogBytesPerShort, Gscratch, O2);
  __ sth(Otos_i, O2, arrayOopDesc::base_offset_in_bytes(T_CHAR));
}


void TemplateTable::sastore() {
  // %%%%% Factor across platform
  castore();
}


void TemplateTable::istore(int n) {
  transition(itos, vtos);
  __ st(Otos_i, Llocals, -n * BytesPerWord);
}


void TemplateTable::lstore(int n) {
  transition(ltos, vtos);
  assert(n+1 < Argument::n_register_parameters, "only handle register cases");
  __ store_unaligned_long(Otos_l, Llocals, -(n+1)*BytesPerWord);
}


void TemplateTable::fstore(int n) {
  transition(ftos, vtos);
  assert(n < Argument::n_register_parameters, "only handle register cases");
  __ stf(FloatRegister::S, Ftos_f, Llocals, -n * BytesPerWord);
}


void TemplateTable::dstore(int n) {
  transition(dtos, vtos);
  FloatRegister src = Ftos_d;
  __ store_unaligned_double(src, Llocals, -(n+1)*BytesPerWord);
}


void TemplateTable::astore(int n) {
  transition(atos, vtos);
  __ st(Otos_i, Llocals, -n * BytesPerWord);
}


void TemplateTable::pop() {
  transition(vtos, vtos);
  __ inc(Lesp, wordSize);
}


void TemplateTable::pop2() {
  transition(vtos, vtos);
  __ inc(Lesp, 2 * wordSize);
}


void TemplateTable::dup() {
  transition(vtos, vtos);
  // stack: ..., a
  __ ld( Lesp, +wordSize, Otos_i );
  __ push_i();
  // stack: ..., a, a
}


void TemplateTable::dup_x1() {
  transition(vtos, vtos);
  // stack: ..., a, b
  __ ld(Lesp, 2 * wordSize, Otos_l1); // get a
  __ ld(Lesp, 1 * wordSize, Otos_l2); // get b
  __ st(Otos_l2, Lesp, 2 * wordSize); // put b
  __ st(Otos_l1, Lesp, 1 * wordSize); // put a
  __ push_i(Otos_l2);                  // push b
  // stack: ..., b, a, b
}


void TemplateTable::dup_x2() {
  transition(vtos, vtos);
  // stack: ..., a, b, c
  __ ld(Lesp, 3 * wordSize, Otos_l1 );       // get a
  __ ld(Lesp, 2 * wordSize, Otos_l2 );       // get b
  __ ld(Lesp, 1 * wordSize, Gscratch);       // get c

  __ st(Gscratch, Lesp, 3 * wordSize);       // put c
  __ st(Otos_l1,  Lesp, 2 * wordSize);       // put a
  __ st(Otos_l2,  Lesp, 1 * wordSize);       // put b
  __ push_i(Gscratch);                        // push c
  // stack: ..., c, a, b, c
}


void TemplateTable::dup2() {
  transition(vtos, vtos);
  // stack: ..., a, b
  __ ld(Lesp, 1 * wordSize, Otos_l1); // get b
  __ ld(Lesp, 2 * wordSize, Otos_l2); // get a
  __ push_l();
  // stack: ..., a, b, a, b
}


void TemplateTable::dup2_x1() {
  transition(vtos, vtos);
  // stack: ..., a, b, c
  __ ld(Lesp, 3 * wordSize, Gscratch); // get a
  __ ld(Lesp, 2 * wordSize, Otos_l2 ); // get b
  __ ld(Lesp, 1 * wordSize, Otos_l1 ); // get c

  __ st( Otos_l2, Lesp, 3 * wordSize); // put b
  __ st( Otos_l1, Lesp, 2 * wordSize); // put c
  __ st(Gscratch, Lesp, 1 * wordSize); // put a
  __ push_l();                         // put b,c
  // stack: ..., b, c, a, b, c
}


void TemplateTable::dup2_x2() {
  transition(vtos, vtos);
  // stack: ..., a, b, c, d
  __ ld(Lesp, 4 * wordSize, Gscratch ); // get a
  __ ld(Lesp, 3 * wordSize, Gscratch2); // get b
  __ ld(Lesp, 2 * wordSize, Otos_l2  ); // get c
  __ ld(Lesp, 1 * wordSize, Otos_l1  ); // get d

  __ st(  Otos_l2, Lesp, 4 * wordSize); // put c
  __ st(  Otos_l1, Lesp, 3 * wordSize); // put d
  __ st(Gscratch,  Lesp, 2 * wordSize); // put a
  __ st(Gscratch2, Lesp, 1 * wordSize); // put b
  __ push_l();                           // push c,d
  // stack: ..., c, d, a, b, c, d
}


void TemplateTable::swap() {
  transition(vtos, vtos);
  // stack: ..., a, b
  __ ld(Lesp, 2 * wordSize, Otos_l2); // get a
  __ ld(Lesp, 1 * wordSize, Otos_l1); // get b

  __ st(Otos_l1, Lesp, 2 * wordSize); // put b
  __ st(Otos_l2, Lesp, 1 * wordSize); // put a
  // stack: ..., b, a
}


void TemplateTable::iop2(Operation op) {
  transition(itos, itos);
  __ pop_i(O1);
  switch (op) {
   case  add:  __  add(O1, Otos_i, Otos_i);  break;
   case  sub:  __  sub(O1, Otos_i, Otos_i);  break;
     // %%%%% Mul may not exist: better to call .mul? 
   case  mul:  __ smul(O1, Otos_i, Otos_i);  break;
   case _and:  __  and3(O1, Otos_i, Otos_i);  break;
   case  _or:  __   or3(O1, Otos_i, Otos_i);  break;
   case _xor:  __  xor3(O1, Otos_i, Otos_i);  break;
   case  shl:  __  sll(O1, Otos_i, Otos_i);  break;
   case  shr:  __  sra(O1, Otos_i, Otos_i);  break;
   case ushr:  __  srl(O1, Otos_i, Otos_i);  break;
   default: ShouldNotReachHere();
  }
}


void TemplateTable::lop2(Operation op) {
  transition(ltos, ltos);
  __ pop_l(O2);
  switch (op) {
   case  add:  __ addcc(O3, Otos_l2, Otos_l2);  __ addc(O2, Otos_l1, Otos_l1);  break;
   case  sub:  __ subcc(O3, Otos_l2, Otos_l2);  __ subc(O2, Otos_l1, Otos_l1);  break;
   case _and:  __ and3( O3, Otos_l2, Otos_l2);  __ and3(O2, Otos_l1, Otos_l1);  break;
   case  _or:  __  or3( O3, Otos_l2, Otos_l2);  __  or3(O2, Otos_l1, Otos_l1);  break;
   case _xor:  __ xor3( O3, Otos_l2, Otos_l2);  __ xor3(O2, Otos_l1, Otos_l1);  break;
   default: ShouldNotReachHere();
  }
}


void TemplateTable::idiv() {
  // %%%%% Later: ForSPARC/V7 call .sdiv library routine, 
  // %%%%% Use ldsw...sdivx on pure V9 ABI. 64 bit safe.
  
  transition(itos, itos);
  __ pop_i(O1); // get 1st op

  // Y contains upper 32 bits of result, set it to 0 or all ones
  __ wry(G0);
  __ mov(~0, Gscratch);

  __ tst(O1);
     Label neg;
  __ br(Assembler::negative, true, Assembler::pn, neg);
  __ delayed()->wry(Gscratch);
  __ bind(neg);

     Label ok;
  __ tst(Otos_i); 
  __ throw_if_not_1( Assembler::notZero, ok );
  __ delayed()->nop();
  __ throw_if_not_2( Interpreter::_throw_ArithmeticException_entry, Gscratch, ok);

  const int min_int = 0x80000000;
  Label regular;
  __ cmp(Otos_i, -1);
  __ br(Assembler::notEqual, false, Assembler::pt, regular);
  __ delayed()->set(min_int, Gscratch2);

  Label done;
  __ cmp(O1, Gscratch2);
  __ br(Assembler::equal, true, Assembler::pt, done);
  __ delayed()->mov(O1, Otos_i);   // (mov only executed if branch taken)

  __ bind(regular);
  __ sdiv(O1, Otos_i, Otos_i); // note: irem uses O1 after this instruction!
  __ bind(done);
}


void TemplateTable::irem() {
  transition(itos, itos);
  __ mov(Otos_i, O2); // save divisor
  idiv(); 				// %%%% Hack: exploits fact that idiv leaves dividend in O1 
  __ smul(Otos_i, O2, Otos_i);
  __ sub(O1, Otos_i, Otos_i);
}


void TemplateTable::lmul() {
  transition(ltos, ltos);
  __ pop_l(O2);

  __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::lmul));
}


void TemplateTable::ldiv() {
  transition(ltos, ltos);
  __ pop_l(O2);

  // check for zero
  __ orcc(Otos_l1, Otos_l2, G0);
  __ throw_if_not( Assembler::notZero, Interpreter::_throw_ArithmeticException_entry, Gscratch);

  __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::ldiv));
}


void TemplateTable::lrem() {
  transition(ltos, ltos);
  __ pop_l(O2);

  // check for zero
  __ orcc(Otos_l1, Otos_l2, G0);
  __ throw_if_not(Assembler::notZero, Interpreter::_throw_ArithmeticException_entry, Gscratch);
  __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::lrem));
}


void TemplateTable::lshl() {
  transition(itos, ltos); // %%%% could optimize, fill delay slot or opt for ultra 

  __ pop_l(O2);                          // shift value in O2, O3
  __ lshl(O2, O3, Otos_i, Otos_l1, Otos_l2, O4);
}


void TemplateTable::lshr() {
  transition(itos, ltos); // %%%% see lshl comment 

  __ pop_l(O2);                          // shift value in O2, O3
  __ lshr(O2, O3, Otos_i, Otos_l1, Otos_l2, O4);
}



void TemplateTable::lushr() {
  transition(itos, ltos); // %%%% see lshl comment 

  __ pop_l(O2);                          // shift value in O2, O3
  __ lushr(O2, O3, Otos_i, Otos_l1, Otos_l2, O4);
}


void TemplateTable::fop2(Operation op) {
  transition(ftos, ftos);
  switch (op) {
   case  add:  __  pop_f(F4); __ fadd(FloatRegister::S, F4, Ftos_f, Ftos_f);  break;
   case  sub:  __  pop_f(F4); __ fsub(FloatRegister::S, F4, Ftos_f, Ftos_f);  break;
   case  mul:  __  pop_f(F4); __ fmul(FloatRegister::S, F4, Ftos_f, Ftos_f);  break;
   case  div:  __  pop_f(F4); __ fdiv(FloatRegister::S, F4, Ftos_f, Ftos_f);  break;
   case  rem:  
     assert(Ftos_f == F0, "just checking");
     __ pop_i(O0);
     __ stf(FloatRegister::S, Ftos_f, __ d_tmp);
     __ ld( __ d_tmp, O1 );
     __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::frem));
     if (Ftos_f != F0)
       __ mov(FloatRegister::S, F0, Ftos_f);
     break;

   default: ShouldNotReachHere();
  }
}


void TemplateTable::dop2(Operation op) {
  transition(dtos, dtos);
  switch (op) {
   case  add:  __  pop_d(F4); __ fadd(FloatRegister::D, F4, Ftos_d, Ftos_d);  break;
   case  sub:  __  pop_d(F4); __ fsub(FloatRegister::D, F4, Ftos_d, Ftos_d);  break;
   case  mul:  __  pop_d(F4); __ fmul(FloatRegister::D, F4, Ftos_d, Ftos_d);  break;
   case  div:  __  pop_d(F4); __ fdiv(FloatRegister::D, F4, Ftos_d, Ftos_d);  break;
   case  rem:  // Sreeram: pass arguments correctly
     // Pass arguments in O0O1, O2O3
     __ stf(FloatRegister::D, Ftos_f, __ d_tmp);
     __ ldd( __ d_tmp, O2 );
     __ pop_d(Ftos_f);
     __ stf(FloatRegister::D, Ftos_f, __ d_tmp);
     __ ldd( __ d_tmp, O0 );
     __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::drem));
     if (Ftos_d != F0)
       __ mov(FloatRegister::D, F0, Ftos_d);
     break;

   default: ShouldNotReachHere();
  }
}


void TemplateTable::ineg() {
  transition(itos, itos);
  __ neg(Otos_i);
}


void TemplateTable::lneg() {
  transition(ltos, ltos);
  __ lneg(Otos_l1, Otos_l2);
}


void TemplateTable::fneg() {
  transition(ftos, ftos);
  __ fneg(FloatRegister::S, Ftos_f);
}


void TemplateTable::dneg() {
  transition(dtos, dtos);
  // v8 has fnegd if source and dest are the same
  __ fneg(FloatRegister::D, Ftos_f);
}


void TemplateTable::iinc() {
  transition(vtos, vtos);
  locals_index(Gscratch);
  __ ldsb(Lbcp, 2, O2);  // load constant
  __ access_local_int(Gscratch, Otos_i);
  __ add(Otos_i, O2, Otos_i);
  __ st(Otos_i, Gscratch, 0);    // access_local_int puts E.A. in Gscratch
}


void TemplateTable::wide_iinc() {
  transition(vtos, vtos);
  locals_index_wide(Gscratch);
  __ get_2_byte_integer_at_bcp( 4,  O2, O3, InterpreterMacroAssembler::Signed);
  __ access_local_int(Gscratch, Otos_i);
  __ add(Otos_i, O3, Otos_i);
  __ st(Otos_i, Gscratch, 0);    // access_local_int puts E.A. in Gscratch
}


void TemplateTable::convert() {
// %%%%% Factor this first part accross platforms 
  #ifdef ASSERT
    TosState tos_in  = ilgl;
    TosState tos_out = ilgl;
    switch (bytecode()) {
      case Bytecodes::_i2l: // fall through
      case Bytecodes::_i2f: // fall through
      case Bytecodes::_i2d: // fall through
      case Bytecodes::_i2b: // fall through
      case Bytecodes::_i2c: // fall through
      case Bytecodes::_i2s: tos_in = itos; break;
      case Bytecodes::_l2i: // fall through
      case Bytecodes::_l2f: // fall through
      case Bytecodes::_l2d: tos_in = ltos; break;
      case Bytecodes::_f2i: // fall through
      case Bytecodes::_f2l: // fall through
      case Bytecodes::_f2d: tos_in = ftos; break;
      case Bytecodes::_d2i: // fall through
      case Bytecodes::_d2l: // fall through
      case Bytecodes::_d2f: tos_in = dtos; break;
      default             : ShouldNotReachHere();
    }
    switch (bytecode()) {
      case Bytecodes::_l2i: // fall through
      case Bytecodes::_f2i: // fall through
      case Bytecodes::_d2i: // fall through
      case Bytecodes::_i2b: // fall through
      case Bytecodes::_i2c: // fall through
      case Bytecodes::_i2s: tos_out = itos; break;
      case Bytecodes::_i2l: // fall through
      case Bytecodes::_f2l: // fall through
      case Bytecodes::_d2l: tos_out = ltos; break;
      case Bytecodes::_i2f: // fall through
      case Bytecodes::_l2f: // fall through
      case Bytecodes::_d2f: tos_out = ftos; break;
      case Bytecodes::_i2d: // fall through
      case Bytecodes::_l2d: // fall through
      case Bytecodes::_f2d: tos_out = dtos; break;
      default             : ShouldNotReachHere();
    }
    transition(tos_in, tos_out);
  #endif

  
  // Conversion
  Label done;
  switch (bytecode()) {
   case Bytecodes::_i2l: 
    __ addcc(Otos_i, 0, Otos_l2);  
    __ br(Assembler::greaterEqual, true, Assembler::pt, done);  
    __ delayed()->clr(Otos_l1); 
    __ set(~0, Otos_l1);  
    break;
    
   case Bytecodes::_i2f:
    __ st(Otos_i, __ d_tmp );
    __ ldf(FloatRegister::S,  __ d_tmp, F0);
    __ fitof(FloatRegister::S, F0, Ftos_f);
    break;
    
   case Bytecodes::_i2d:
    __ st(Otos_i, __ d_tmp);
    __ ldf(FloatRegister::S,  __ d_tmp, F0);
    __ fitof(FloatRegister::D, F0, Ftos_f);
    break;
    
   case Bytecodes::_i2b: 
    __ sll(Otos_i, 24, Otos_i); 
    __ sra(Otos_i, 24, Otos_i); 
    break;
    
   case Bytecodes::_i2c:
    __ sll(Otos_i, 16, Otos_i); 
    __ srl(Otos_i, 16, Otos_i); 
    break;
   
   case Bytecodes::_i2s: 
    __ sll(Otos_i, 16, Otos_i);
    __ sra(Otos_i, 16, Otos_i);
    break;

   case Bytecodes::_l2i:
    __ mov(Otos_l2, Otos_i);
    break;
    
   case Bytecodes::_l2f:
   case Bytecodes::_l2d:
    __ std(Otos_l1, __ d_tmp);
    __ ldf(FloatRegister::D, __ d_tmp, Ftos_d);

    if (VM_Version::v9_instructions_work()) {
      if (bytecode() == Bytecodes::_l2f)    
	__ fxtof(FloatRegister::S, Ftos_d, Ftos_f);
      else 
	__ fxtof(FloatRegister::D, Ftos_d, Ftos_d);
    } else {
      __ call_VM_leaf(Lscratch,
		      bytecode() == Bytecodes::_l2f  ? CAST_FROM_FN_PTR(address, SharedRuntime::l2f)
		      : CAST_FROM_FN_PTR(address, SharedRuntime::l2d));
    }
    break;
    
  case Bytecodes::_f2i: 
    __ ftoi(FloatRegister::S, Ftos_f, F30);
    __ stf(FloatRegister::S, F30, __ d_tmp);
    __ ld(__ d_tmp, Otos_i);
    __ set(0x7fffffff, Gscratch);	// max int
    __ cmp(Otos_i, Gscratch);
    __ br(Assembler::notEqual, false, Assembler::pt, done);
    __ delayed()->nop();
    // NaN, +infinity, large positive
    __ stf(FloatRegister::S, Ftos_f, __ d_tmp); 
    __ ld(__ d_tmp, Gscratch);
    __ srl(Gscratch, 0x17, Gscratch2);		// get exponent
    __ cmp(Gscratch2, 0xff);			// if ( exp != 0xff ) it's not a NaN
    __ br(Assembler::notEqual, false, Assembler::pt, done);
    __ delayed()->nop();
    __ set(0x7fffff, Gscratch2);		// NaN or infinity
    __ andcc(Gscratch2, Gscratch, Gscratch2);	// get fraction
    __ br(Assembler::zero, false, Assembler::pt, done); // if(fraction == 0) it's an  infinity
    __ delayed()->nop();      
    __ clr(Otos_i);				// NaN
    break;
     
   case Bytecodes::_f2l: 
    // must uncache tos
    __ push_f();
    __ pop_i(O0);
    __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::f2l));
    break;

   case Bytecodes::_f2d: 
    __ ftof( FloatRegister::S, FloatRegister::D, Ftos_f, Ftos_f);
    break;

   case Bytecodes::_d2i:
    __ ftoi( FloatRegister::D, Ftos_d, F30);
    __ stf(FloatRegister::S, F30, __ d_tmp);
    __ ld(__ d_tmp, Otos_i);
    __ set(0x7fffffff, Gscratch);	// max int
    __ cmp(Otos_i, Gscratch);
    __ br(Assembler::notEqual, false, Assembler::pt, done);
    __ delayed()->nop();
    // NaN, +infinity, large positive
    __ stf(FloatRegister::D, Ftos_d, __ d_tmp); 
    __ ld(__ d_tmp, Gscratch);
    __ srl(Gscratch, 0x14, Gscratch2);		// get exponent
    __ cmp(Gscratch2, 0x7ff);			// if ( exp != 0x7ff ) it's not a NaN
    __ br(Assembler::notEqual, false, Assembler::pt, done);
    __ delayed()->nop();
    __ ld(__ d_tmp, Gscratch2,wordSize);
    __ set(0xfffff, O7);
    __ and3(Gscratch, O7, Gscratch);		// get fraction
    __ orcc(Gscratch, Gscratch2, Gscratch);	// or lsw or fraction
    __ br(Assembler::zero, false, Assembler::pt, done); // it's an  infinity
    __ delayed()->nop();
    __ clr(Otos_i);				// NaN
    break;
    
   case Bytecodes::_d2l:
    // must uncache tos
    __ push_d();
    __ pop_i(O0);
    __ pop_i(O1);
    __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::d2l));
    break;
    
    case Bytecodes::_d2f:
    if (VM_Version::v9_instructions_work()) {
      __ ftof( FloatRegister::D, FloatRegister::S, Ftos_d, Ftos_f);
    }
    else {
      // must uncache tos
      __ push_d();
      __ pop_i(O0);
      __ pop_i(O1);
      __ call_VM_leaf(Lscratch, CAST_FROM_FN_PTR(address, SharedRuntime::d2f));
    }
    break;
    
    default: ShouldNotReachHere();
  }
  __ bind(done);
}


void TemplateTable::lcmp() {
  transition(ltos, itos);

  __ pop_l(O2); // cmp O2,3 to O0,1
  __ lcmp( O2, O3, Otos_l1, Otos_l2, Otos_i );
}


void TemplateTable::float_cmp(bool is_float, int unordered_result) {

  if (is_float) __ pop_f(F2);
  else          __ pop_d(F2);

  assert(Ftos_f == F0  &&  Ftos_d == F0,  "alias checking:");

  __ float_cmp( is_float, unordered_result, F2, F0, Otos_i );
}

void TemplateTable::count_branches(bool taken) {
#ifdef COMPILER2
  if (CountInterpBranches) {
    Address invocation_counter(Lmethod, 0, methodOopDesc::invocation_counter_offset_in_bytes() + InvocationCounter::counter_offset_in_bytes());

    assert(SharedInfo::freqTable != NULL, "should be allocated by now");
    address table_base = (address) SharedInfo::freqTable;
    if (taken)
      table_base += FreqPair::taken_offset();
    else
      table_base += FreqPair::not_taken_offset();
    Address table_addr(Gscratch2, table_base);


    Label L;
    // Check invocation counter to see if you want to count the branch
    __ ld(invocation_counter, Gscratch);

    int sub_threshold = CallProfile::unprofiled_invocations();
    assert(sub_threshold >= 0 && sub_threshold <= CompileThreshold, "");
    int scaled_threshold = sub_threshold * InvocationCounter::count_increment;
    if (Assembler::is_simm13(scaled_threshold)) {
      __ cmp(Gscratch, scaled_threshold);
    } else {
      Register Rscratch = table_addr.base();  // table_addr not live yet
      __ set(scaled_threshold, Rscratch);
      __ cmp(Gscratch, Rscratch);
    }

    __ br(Assembler::lessUnsigned, false, Assembler::pt, L);
    __ delayed()->sethi(table_addr);
    // Increment freqTable[BCP % BranchFreqTableSize].{_taken|_not_taken}
    __ and3(Lbcp, right_n_bits(BranchFreqTableSize), Gscratch);
    __ add(table_addr, Gscratch2);
    __ sll(Gscratch, exact_log2(sizeof(FreqPair)), Gscratch);
    __ ld(Gscratch2, Gscratch, Gscratch3);
    __ inc(Gscratch3);
    __ st(Gscratch3, Gscratch2, Gscratch);
    __ bind(L);
  }
#endif /* COMPILER2 */
}

void TemplateTable::branch(bool is_jsr, bool is_wide) {
  // Note: on SPARC, we use InterpreterMacroAssembler::if_cmp also.

  __ verify_oop(Lmethod);
  __ verify_thread();

#ifdef COMPILER2
  // if counting branches, increment branch taken, ecx has method
  if( UseLoopCounter && !is_jsr )
    count_branches(true);
  // Note: Caller of branch() must also instrument not-taken case, if any.
#endif

  {
    if (is_jsr) {
      // compute return address as bci in Otos_i
      __ sub(Lbcp, Lmethod, Gscratch);
      __ sub(Gscratch, methodOopDesc::codes_offset_in_bytes() - (is_wide ? 5 : 3), Otos_i);
    }

    bool increment_invocation_counter_for_backward_branches = UseRecompilation && UseLoopCounter && !is_jsr;
    InterpreterMacroAssembler::setCCOrNot scc = increment_invocation_counter_for_backward_branches
             ?  InterpreterMacroAssembler::     set_CC
             :  InterpreterMacroAssembler::dont_set_CC;
        
    if (is_wide)  __ get_4_byte_integer_at_bcp( 1,  Gscratch2, Gscratch,                         scc);
    else          __ get_2_byte_integer_at_bcp( 1,  Gscratch2, Gscratch, InterpreterMacroAssembler::Signed, scc);

    __ add(Lbcp, Gscratch, Lbcp);

#ifndef CORE
    assert(UseLoopCounter || !UseOnStackReplacement, "on-stack-replacement requires loop counters");
    if ( increment_invocation_counter_for_backward_branches ) {
      Label L;
      // check branch direction
      __ br( Assembler::positive, false,  Assembler::pt, L );
      __ delayed()->nop();

      // %%%% BUG: Consult InterpreterBackwardBranchLimit here,
      // %%%% not InterpreterInvocationLimit.
      __ increment_invocation_counter( Gscratch, Gscratch2 );

      if(UseOnStackReplacement) {
	assert(!Bytecodes::safepoint_safe(bytecode()), "can call OSR method which may not stop");
        __ test_invocation_counter_for_osr( false, Gscratch, Gscratch2, is_jsr ? itos : vtos );
      }

      __ bind(L);
    }
#endif
    // %%%%%%%%%% Need to add some code here for UseOnStackReplacement on SPARC
  }
  // continue with bytecode @ target
  // %%%%% Like Intel, could speed things up by moving bytecode fetch to code above,
  // %%%%% and changing dispatch_next to dispatch_only 

  __ dispatch_next(is_jsr ? itos : vtos);
}


// Note Condition in argument is TemplateTable::Condition
// arg scope is within class scope

void TemplateTable::if_0cmp(Condition cc) {
  transition(itos, vtos);
  // assume branch is more often taken than not (loops use backward branches)
  __ cmp( Otos_i, 0);
  __ if_cmp(ccNot(cc));
}


void TemplateTable::if_icmp(Condition cc) {
  transition(itos, vtos);
  __ pop_i(O1);
  __ cmp(O1, Otos_i);
  __ if_cmp(ccNot(cc));
}


void TemplateTable::if_nullcmp(Condition cc) {
  transition(atos, vtos);
  __ tst(Otos_i);
  __ if_cmp(ccNot(cc));
}


void TemplateTable::if_acmp(Condition cc) {
  transition(atos, vtos);
  __ pop_i(O1);
  __ verify_oop(O1);
  __ verify_oop(Otos_i);
  __ cmp(O1, Otos_i);
  __ if_cmp(ccNot(cc));
}



void TemplateTable::ret() {
 
  transition(vtos, vtos);
  locals_index(Gscratch);
  __ access_local_int(Gscratch, Otos_i);
  // Otos_i contains the bci, compute the bcp from that
  __ add(Lmethod, Otos_i, Gscratch);
  __ add(Gscratch, methodOopDesc::codes_offset_in_bytes(), Lbcp);
  __ dispatch_next(vtos);
}


void TemplateTable::wide_ret() {
  transition(vtos, vtos);
  locals_index_wide(Gscratch);
  __ access_local_int(Gscratch, Otos_i);
  // Otos_i contains the bci, compute the bcp from that
  __ add(Lmethod, Otos_i, Gscratch);
  __ add(Gscratch, methodOopDesc::codes_offset_in_bytes(), Lbcp);
  __ dispatch_next(vtos);
}


void TemplateTable::tableswitch() {
  transition(itos, vtos);
  Label default_case, continue_execution;

  // align bcp
  __ add(Lbcp, wordSize, O1);
  __ and3(O1, -wordSize, O1);
  // load lo, hi
  __ ld(O1, 1 * wordSize, O2);
  __ ld(O1, 2 * wordSize, O3);
  // check against lo & hi
  __ cmp( Otos_i, O2);
  __ br( Assembler::less, false, Assembler::pn, default_case);
  __ delayed()->cmp( Otos_i, O3 );
  __ br( Assembler::greater, false, Assembler::pn, default_case);
  // lookup dispatch offset
  __ delayed()->sub(Otos_i, O2, O2);
  __ sll(O2, LogBytesPerWord, O2);
  __ add(O2, 3 * wordSize, O2);
  __ ba(false, continue_execution);
  __ delayed()->ld(O1, O2, O2);
  // handle default
  __ bind(default_case);
  __ ld(O1, 0, O2); // get default offset
  // continue execution
  __ bind(continue_execution);
  __ add(Lbcp, O2, Lbcp);
  __ dispatch_next(vtos);
}


void TemplateTable::lookupswitch() {
  transition(itos, itos);
  __ stop("lookupswitch bytecode should have been rewritten");
}  

void TemplateTable::fast_linearswitch() {
  transition(itos, vtos);
    Label loop_entry, loop, found;
  // align bcp
  __ add(Lbcp, wordSize, O1);
  __ and3(O1, -wordSize, O1);
 // set counter
  __ ld(O1, wordSize, O2);
  __ sll(O2, LogBytesPerWord + 1, O2); // in word-pairs
  __ add(O1, 2 * wordSize, O3); // set first pair addr
  __ ba(false, loop_entry);
  __ delayed()->add(O3, O2, O2); // counter now points past last pair
  
  // table search
  __ bind(loop);
  __ cmp(O4, Otos_i);
  __ br(Assembler::equal, true, Assembler::pn, found);
  __ delayed()->ld(O3, wordSize, O4); // offset -> O4
  __ inc(O3, 2 * wordSize);

  __ bind(loop_entry);
  __ cmp(O2, O3);
  __ br(Assembler::greaterUnsigned, true, Assembler::pt, loop);
  __ delayed()->ld(O3, 0, O4);
  
  // default case
  __ ld(O1, 0, O4); // get default offset

  // entry found -> get offset
  __ bind(found);
  __ add(Lbcp, O4, Lbcp);
  __ dispatch_next(vtos);
}


void TemplateTable::fast_binaryswitch() {
  transition(itos, vtos);
  // Implementation using the following core algorithm: (copied from Intel)
  //
  // int binary_search(int key, LookupswitchPair* array, int n) {
  //   // Binary search according to "Methodik des Programmierens" by
  //   // Edsger W. Dijkstra and W.H.J. Feijen, Addison Wesley Germany 1985.
  //   int i = 0;
  //   int j = n;
  //   while (i+1 < j) {
  //     // invariant P: 0 <= i < j <= n and (a[i] <= key < a[j] or Q)
  //     // with      Q: for all i: 0 <= i < n: key < a[i]
  //     // where a stands for the array and assuming that the (inexisting)
  //     // element a[n] is infinitely big.
  //     int h = (i + j) >> 1;
  //     // i < h < j
  //     if (key < array[h].fast_match()) {
  //       j = h;
  //     } else {
  //       i = h;
  //     }
  //   }
  //   // R: a[i] <= key < a[i+1] or Q
  //   // (i.e., if key is within array, i is the correct index)
  //   return i;
  // }
  
  // register allocation
  assert(Otos_i == O0, "alias checking");
  const Register Rkey     = Otos_i;                    // already set (tosca)
  const Register Rarray   = O1;
  const Register Ri       = O2;
  const Register Rj       = O3;
  const Register Rh       = O4; 
  const Register Rscratch = O5;
  
  const int log_entry_size = 3;
  const int entry_size = 1 << log_entry_size;
  
  Label found;
  // Find Array start
  __ add(Lbcp, 3 * wordSize, Rarray);
  __ and3(Rarray, -wordSize, Rarray);
  // initialize i & j (in delay slot)
  __ clr( Ri );

  // and start
  Label entry;
  __ ba(false, entry);
  __ delayed()->ld( Rarray, -wordSize, Rj);
  // (Rj is already in the native byte-ordering.)
  
  // binary search loop
  { Label loop;
    __ bind( loop );
    // int h = (i + j) >> 1;
    __ sra( Rh, 1, Rh );
    // if (key < array[h].fast_match()) {
    //   j = h;
    // } else {
    //   i = h;
    // }
    __ sll( Rh, log_entry_size, Rscratch );
    __ ld( Rarray, Rscratch, Rscratch );
    // (Rscratch is already in the native byte-ordering.)
    __ cmp( Rkey, Rscratch );
    if ( VM_Version::v9_instructions_work() ) {
      __ movcc( Assembler::less,         false, Assembler::icc, Rh, Rj );  // j = h if (key <  array[h].fast_match())
      __ movcc( Assembler::greaterEqual, false, Assembler::icc, Rh, Ri );  // i = h if (key >= array[h].fast_match())
    }
    else {
      Label end_of_if;
      __ br( Assembler::less, true, Assembler::pt, end_of_if );
      __ delayed()->mov( Rh, Rj ); // if (<) Rj = Rh
      __ mov( Rh, Ri ); 	   // else i = h 
      __ bind(end_of_if);          // }
    }
    
    // while (i+1 < j)
    __ bind( entry );
    __ add( Ri, 1, Rscratch );
    __ cmp(Rscratch, Rj);
    __ br( Assembler::less, true, Assembler::pt, loop );
    __ delayed()->add( Ri, Rj, Rh ); // start h = i + j  >> 1;
  }
  
  // end of binary search, result index is i (must check again!)
  Label default_case;
  __ sll( Ri, log_entry_size, Ri );
  __ ld( Rarray, Ri, Rscratch );
  // (Rscratch is already in the native byte-ordering.)
  __ cmp( Rkey, Rscratch );
  __ br( Assembler::notEqual, true, Assembler::pn, default_case );
  __ delayed()->ld( Rarray, -2 * wordSize, Rj ); // load default offset -> j
  
  // entry found -> j = offset
  __ inc( Ri, wordSize );
  __ ld( Rarray, Ri, Rj );
  // (Rj is already in the native byte-ordering.)

  __ bind(default_case); // fall through
  __ add( Lbcp, Rj, Lbcp );
  __ dispatch_next( vtos );
}


void TemplateTable::_return(TosState state) {

#ifndef CORE
#ifndef PRODUCT
  // __ trace_method_exit(Lmethod);
#endif
#endif

  transition(state, state);
  assert(_desc->calls_vm(), "inconsistent calls_vm information");
  __ unlock_if_synchronized_method(state, /* throw_monitor_exception */ true);

  // save result (push state before jvmdi/jvmpi call and pop it afterwards) and notify jvmdi/jvmpi
  __ notify_method_exit(false, state); 

  __ verify_oop(Otos_i, state);
  __ verify_oop(Lmethod);
  __ verify_thread();

  if (UseC2CallingConventions) {
    // return tos
    switch (state) {
     case atos:
     case itos:
      __ ret();
      __ delayed()->restore(Otos_i, G0, Otos_i);
      break;

     case ltos: 
      __ mov(Otos_l1, Otos_l1.after_save()); // after_save turns Ox into Ix
      __ ret();
      __ delayed()->restore(Otos_l2, G0, Otos_l2);
      break;

     case ftos:
     case dtos:
     case vtos:
      __ ret();
      __ delayed()->restore(G0, G0, G0);
      break;

      default: ShouldNotReachHere();
    }

  } else {
    // return tos
    assert(Otos_l1 == Otos_i, "adjust code below");
    switch (state) {
      case ltos: __ mov(Otos_l2, Otos_l2.after_save()); // fall through
      case atos:                                        // fall through
      case itos: __ mov(Otos_l1, Otos_l1.after_save()); break;
      case ftos:                                        // fall through
      case dtos:                                        // fall through
      case vtos: /* nothing to do */                    break;
      default  : ShouldNotReachHere();
    }

    // The caller's SP was adjusted upon method entry to accomodate
    // the callee's non-argument locals. Determine the adjustment.
    const Register Gtmp1 = Gscratch;
    const Register Gtmp2 = Gscratch3;
    __ lduh(Lmethod, methodOopDesc::size_of_parameters_offset_in_bytes(), Gtmp1);
    __ lduh(Lmethod, methodOopDesc::size_of_locals_offset_in_bytes()    , Gtmp2);
    __ compute_extra_locals_size_in_bytes(Gtmp1, Gtmp2, Gtmp2);
    // return, restore register window & adjust SP
    // Gtmp2: extra space allocated for non-argument locals in caller
    __ ret();                                    // return to caller
    __ delayed()->restore(FP, Gtmp2, SP);        // restore registers & reset caller's SP
  }
}



void TemplateTable::resolve(int byte_no) {
  assert(byte_no == 1 || byte_no == 2, "byte_no out of range");
  const int shift_count = (1 + byte_no)*BitsPerByte;
  Label resolve, redo;
  Register Rcache = Gscratch;

  __ bind(redo);
  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes()), Lbyte_code);

  __ srl(   Lbyte_code, shift_count, Lbyte_code );
  __ andcc( Lbyte_code,        0xFF, Lbyte_code );
  __ br( Assembler::zero, true, Assembler::pt, resolve);
  __ delayed()->set((int)bytecode(), O1);

  // update the code to be faster next time
  // (this update is not optional, because we will immediately re-dispatch
  // on the quickened bytecode, to get to the correct template)
  Label patch_done;
  if (jvmdi::enabled() && UseFastBreakpoints) {
    Label fast_patch;
    __ ldub(at_bcp(0), Gscratch2);
    __ cmp(Gscratch2, Bytecodes::_breakpoint);
    __ br(Assembler::notEqual, false, Assembler::pt, fast_patch);
    __ delayed()->nop();  // don't bother to hoist the stb here
    // perform the quickening, slowly, in the bowels of the breakpoint table
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::set_original_bytecode_at), Lmethod, Lbcp, Lbyte_code);
    __ ba(false, patch_done);
    __ delayed()->nop();
    __ bind(fast_patch);
  }
  __ stb(Lbyte_code, at_bcp(0));
  __ bind(patch_done);

  // dispatch via Lbyte_code
  __ dispatch_only(vtos);

  address entry;
  switch (bytecode()) {
    case Bytecodes::_getstatic      : // fall through
    case Bytecodes::_putstatic      : // fall through
    case Bytecodes::_getfield       : // fall through
    case Bytecodes::_putfield       : entry = CAST_FROM_FN_PTR(address, InterpreterRuntime::resolve_get_put); break;
    case Bytecodes::_invokevirtual  : // fall through
    case Bytecodes::_invokespecial  : // fall through
    case Bytecodes::_invokestatic   : // fall through
    case Bytecodes::_invokeinterface: entry = CAST_FROM_FN_PTR(address, InterpreterRuntime::resolve_invoke);  break;
    default                         : ShouldNotReachHere();                                 break;
  }
  __ bind(resolve);
  // first time invocation - must resolve first
  // set((int)bytecode(), O1);
  __ call_VM(noreg, entry, O1);
  __ ba(false, redo);
  __ delayed()->nop();
}

void TemplateTable::fast_accessstatic(bool load) {
  // do the JVM/DI work here to avoid disturbing the register state below
  if (jvmdi::enabled()) {
    if (load) {
      // Check to see if a field access watch has been set before we take
      // the time to call into the VM.
      Label L1;
      Address get_field_access_count_addr(Gscratch2, (address)jvmdi::get_field_access_count_addr(), relocInfo::none);
      __ load_contents(get_field_access_count_addr, Gscratch2);
      __ tst(Gscratch2);
      __ br(Assembler::zero, false, Assembler::pt, L1);
      __ delayed()->nop();
      __ get_cache_entry_pointer_at_bcp(Gscratch, Gscratch2, 1);
      __ clr(Gscratch2);
      // Gscratch2: NULL object pointer
      // Gscratch:  cache entry pointer
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_access), Gscratch2, Gscratch);
      __ bind(L1);
    } else {
      // Check to see if a field modification watch has been set before we take
      // the time to call into the VM.
      Label L2;
      Address get_field_modification_count_addr(Gscratch2, (address)jvmdi::get_field_modification_count_addr(), relocInfo::none);
      __ load_contents(get_field_modification_count_addr, Gscratch2);
      __ tst(Gscratch2);
      __ br(Assembler::zero, false, Assembler::pt, L2);
      __ delayed()->nop();
      // Save tos values before call_VM() clobbers them. Since we have
      // to do it for every data type, we use the saved values as the
      // jvalue object.
      switch (bytecode()) {  // save tos values before call_VM() clobbers them
      case Bytecodes::_fast_aputstatic: // fall through
      case Bytecodes::_fast_iputstatic: __ push_i(Otos_i); break;
      case Bytecodes::_fast_dputstatic: __ push_d(Ftos_d); break;
      case Bytecodes::_fast_fputstatic: __ push_f(Ftos_f); break;
      // get words in right order for use as jvalue object
      case Bytecodes::_fast_lputstatic: __ push_i(Otos_l2); __ push_i(Otos_l1); break;
      }
      // setup pointer to jvalue object
      __ mov(Lesp, Gscratch3); __ inc(Gscratch3, wordSize);
      // access constant pool cache entry
      __ get_cache_entry_pointer_at_bcp(Gscratch, Gscratch2, 1);
      __ clr(Gscratch2);
      // Gscratch2: NULL object pointer set above
      // Gscratch:  cache entry pointer
      // Gscratch3: jvalue object on the stack
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_modification), Gscratch2, Gscratch, Gscratch3);
      switch (bytecode()) {             // restore tos values
      case Bytecodes::_fast_aputstatic: // fall through
      case Bytecodes::_fast_iputstatic: __ pop_i(Otos_i); break;
      case Bytecodes::_fast_dputstatic: __ pop_d(Ftos_d); break;
      case Bytecodes::_fast_fputstatic: __ pop_f(Ftos_f); break;
      case Bytecodes::_fast_lputstatic: __ pop_i(Otos_l1); __ pop_i(Otos_l2); break;
      }
      __ bind(L2);
    }
  }

  Register Rcache = Gscratch;
  Register Rclass = Rcache;
  Register Roffset= Gscratch2;

  // access constant pool cache
  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize), Roffset);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 1*wordSize), Rclass);

  // field addresses
  __ sll(Roffset, LogBytesPerWord, Roffset);

  // access static
  __ verify_oop(Rclass);
  switch (bytecode()) {
    // loads
    case Bytecodes::_fast_igetstatic:
      transition(vtos, itos);
      __ ld(Rclass, Roffset, Otos_i);
      break;
    case Bytecodes::_fast_lgetstatic:
      transition(vtos, ltos);
      __ ldd(Rclass, Roffset, Otos_l);
      break;
    case Bytecodes::_fast_fgetstatic:
      transition(vtos, ftos);
      __ ldf(FloatRegister::S, Rclass, Roffset, Ftos_f);
      break;
    case Bytecodes::_fast_dgetstatic:
      transition(vtos, dtos);
      __ ldf(FloatRegister::D, Rclass, Roffset, Ftos_d);
      break;
    case Bytecodes::_fast_agetstatic:
      transition(vtos, atos);
      __ ld(Rclass, Roffset, Otos_i);
      __ verify_oop(Otos_i);
      break;

    // stores
    case Bytecodes::_fast_iputstatic:
      transition(itos, vtos);
      __ st(Otos_i, Rclass, Roffset);
      break;
    case Bytecodes::_fast_lputstatic:
      transition(ltos, vtos);
      __ std(Otos_l, Rclass, Roffset);
      break;
    case Bytecodes::_fast_fputstatic:
      transition(ftos, vtos);
      __ stf(FloatRegister::S, Ftos_f, Rclass, Roffset);
      break;
    case Bytecodes::_fast_dputstatic:
      transition(dtos, vtos);
      __ stf(FloatRegister::D, Ftos_d, Rclass, Roffset);
      break;
    case Bytecodes::_fast_aputstatic:
      transition(atos, vtos);
      __ st(Otos_i, Rclass, Roffset);
      __ store_check(Gscratch3, Rclass, Roffset);
      break;
    default:
      ShouldNotReachHere();
      break;
  }
}

void TemplateTable::fast_accessfield(bool load) {
  // Note: does no explicit NULL check (unless +GenerateInterpreterNullChecks);
  //       assumes that the offset is known to be in a range that doesn't need
  //       explicit checks (see also MacroAssembler::null_check(...) for details).

  // do the JVM/DI work here to avoid disturbing the register state below
  if (jvmdi::enabled()) {
    if (load) {
      // Check to see if a field access watch has been set before we take
      // the time to call into the VM.
      Label L1;
      Address get_field_access_count_addr(Gscratch2, (address)jvmdi::get_field_access_count_addr(), relocInfo::none);
      __ load_contents(get_field_access_count_addr, Gscratch2);
      __ tst(Gscratch2);
      __ br(Assembler::zero, false, Assembler::pt, L1);
      __ delayed()->nop();
      __ get_cache_entry_pointer_at_bcp(Gscratch2, Gscratch3, 1);
      __ mov(Otos_i, Lscratch);  // save object pointer before call_VM() clobbers it
      __ verify_oop(Otos_i);
      // Otos_i:    object pointer
      // Gscratch2: cache entry pointer
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_access), Otos_i, Gscratch2);
      __ mov(Lscratch, Otos_i);  // restore object pointer
      __ verify_oop(Otos_i);
      __ bind(L1);
    } else {
      // Check to see if a field modification watch has been set before we take
      // the time to call into the VM.
      Label L2;
      Address get_field_modification_count_addr(Gscratch2, (address)jvmdi::get_field_modification_count_addr(), relocInfo::none);
      __ load_contents(get_field_modification_count_addr, Gscratch2);
      __ tst(Gscratch2);
      __ br(Assembler::zero, false, Assembler::pt, L2);
      __ delayed()->nop();
      __ pop_i(Gscratch2);       // copy the object pointer from tos
      __ verify_oop(Gscratch2);
      __ push_i(Gscratch2);      // put the object pointer back on tos
      __ get_cache_entry_pointer_at_bcp(Gscratch, Gscratch3, 1);
      // Save tos values before call_VM() clobbers them. Since we have
      // to do it for every data type, we use the saved values as the
      // jvalue object.
      switch (bytecode()) {  // save tos values before call_VM() clobbers them
      case Bytecodes::_fast_aputfield: // fall through
      case Bytecodes::_fast_iputfield: __ push_i(Otos_i); break;
      case Bytecodes::_fast_dputfield: __ push_d(Ftos_d); break;
      case Bytecodes::_fast_fputfield: __ push_f(Ftos_f); break;
      // get words in right order for use as jvalue object
      case Bytecodes::_fast_lputfield: __ push_i(Otos_l2); __ push_i(Otos_l1); break;
      }
      // setup pointer to jvalue object
      __ mov(Lesp, Gscratch3); __ inc(Gscratch3, wordSize);
      // Gscratch2:  object pointer
      // Gscratch: cache entry pointer
      // Gscratch3: jvalue object on the stack
      __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::post_field_modification), Gscratch2, Gscratch, Gscratch3);
      switch (bytecode()) {             // restore tos values
      case Bytecodes::_fast_aputfield: // fall through
      case Bytecodes::_fast_iputfield: __ pop_i(Otos_i); break;
      case Bytecodes::_fast_dputfield: __ pop_d(Ftos_d); break;
      case Bytecodes::_fast_fputfield: __ pop_f(Ftos_f); break;
      case Bytecodes::_fast_lputfield: __ pop_i(Otos_l1); __ pop_i(Otos_l2); break;
      }
      __ bind(L2);
    }
  }

  Register Rcache = Gscratch;
  Register Roffset = Gscratch;
  Register Robj = Gscratch2;

  // access constant pool cache
  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  // replace cache address with field offset
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize), Roffset);
  __ sll(Roffset, LogBytesPerWord, Roffset);

  // offset = 0 => assumes MacroAssembler::needs_explicit_null_check(offset) = false
  if (load) {
    // TOS cache has the object
    __ verify_oop(Otos_i);
    __ null_check(Otos_i, 0);
  } else {
    // TOS cache has the value to store, pop the object out
    __ pop_i(Robj);
    __ verify_oop(Robj);
    __ null_check(Robj, 0);
  }

  // access field
  switch (bytecode()) {
    // loads
    case Bytecodes::_fast_igetfield:
      transition(atos, itos);
      __ ld(Otos_i, Roffset, Otos_i);
      break;
    case Bytecodes::_fast_lgetfield:
      transition(atos, ltos);
      __ ldd(Otos_i, Roffset, Otos_l);
      break;
    case Bytecodes::_fast_fgetfield:
      transition(atos, ftos);
      __ ldf(FloatRegister::S, Otos_i, Roffset, Ftos_f);
      break;
    case Bytecodes::_fast_dgetfield:
      transition(atos, dtos);
      __ ldf(FloatRegister::D, Otos_i, Roffset, Ftos_d);
      break;
    case Bytecodes::_fast_agetfield:
      transition(atos, atos);
      __ ld(Otos_i, Roffset, Otos_i);
      break;

    // stores
    case Bytecodes::_fast_iputfield:
      transition(itos, vtos);
      __ st(Otos_i, Robj, Roffset);
      break;
    case Bytecodes::_fast_lputfield:
      transition(ltos, vtos);
      __ std(Otos_l, Robj, Roffset);
      break;
    case Bytecodes::_fast_fputfield:
      transition(ftos, vtos);
      __ stf(FloatRegister::S, Ftos_f, Robj, Roffset);
      break;
    case Bytecodes::_fast_dputfield:
      transition(dtos, vtos);
      __ stf(FloatRegister::D, Ftos_d, Robj, Roffset);
      break;
    case Bytecodes::_fast_aputfield:
      transition(atos, vtos);
      __ st(Otos_i, Robj, Roffset);
      __ store_check(Gscratch3, Robj, Roffset);
      break;
    default:
      ShouldNotReachHere();
      break;
  }
}

void TemplateTable::fast_get(bool is_static) {
  guarantee(!is_static, "static version of this is not used");
  transition(atos, vtos);
  Register Rcache = Gscratch;
  Register Roffset= Gscratch2;
  Register RFlags = Gscratch;
  Label    one_word;

  // access constant pool cache
  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2 * wordSize), Roffset);
  __ sll(Roffset, LogBytesPerWord, Roffset);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3 * wordSize), RFlags);

  __ verify_oop(Otos_i);
  __ null_check(Otos_i);

  __ ld(Otos_i, Roffset, O1);

  // %%%% maybe load flags with a ldub, to avoid the following sethi:
  __ set((1 << 30), Gscratch3);
  __ btst(Gscratch3, RFlags);
  __ br(Assembler::zero, false, Assembler::pt, one_word);
  __ delayed() -> nop();

  __ add( Roffset, wordSize, Roffset);

  // For jdouble & jlong, O0 = Hi address in Memory O1 = Lo address in Memory
  __ ld(Otos_i, Roffset, O0);

  __ push_i(O0);      // word at high stack address
  __ bind(one_word);
  __ push_i(O1);      // word at low stack address
}

void TemplateTable::fast_put(bool is_static) {
  transition(vtos, vtos); 
  Register Rcache  = Gscratch;
  Register Rklass  = Gscratch;
  Register Roffset = Gscratch2;
  Register RFlags  = Gscratch3;
  Label    not_final, one_word, one_word_store_check, exit;

  // access constant pool cache
  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3 * wordSize), RFlags);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2 * wordSize), Roffset);
  __ sll(Roffset, LogBytesPerWord, Roffset);
  if (is_static)
    __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 1 * wordSize), Rklass);

  __ pop_i(O1);       //  either a 1-word value, or the low word of a double
  if (!is_static)
    __ pop_i(Rklass); //  (watch out: might be the high word of a double)  

  __ set((3 << 30), Lscratch); // %%%% maybe load flags with a ldub, to avoid the following sethi:
  __ btst(Lscratch, RFlags);

  __ br(Assembler::zero, false, Assembler::pt, one_word);
  __ delayed()->nop();

  __ br(Assembler::negative, false, Assembler::pt, one_word_store_check);
  __ delayed()->nop();

  // In the 2-word case, the high word was pushed first, then the low word.
  // (Cf. pop_l and pop_d.)  The high word is stored in the lower address.
  if (is_static)
    __ pop_i(O0);          // the high word of the double
  else {
    __ mov(Rklass, O0);    // fixup after earlier misconception
    __ pop_i(Rklass);      // load Rklass for a two word value
    __ null_check(Rklass); 
  }
  __ verify_oop(Rklass);

  // Swap (why?) for both longs and doubles
  __ mov(O0,        Gscratch3 );
  __ mov(O1,        O0 );
  __ mov(Gscratch3, O1 );

  __ std(O0, Rklass, Roffset); // might be jlong or jdouble!
  __ br(Assembler::always, false, Assembler::pt, exit);
  __ delayed()->nop();

  __ bind(one_word_store_check);
  __ verify_oop(Rklass);
  if (!is_static)
    __ null_check(Rklass);

  __ st(O1, Rklass, Roffset);
  __ store_check(Lscratch, Rklass, Roffset);
  __ br(Assembler::always, false, Assembler::pt, exit);
  __ delayed()->nop();

  __ bind(one_word);
  __ verify_oop(Rklass);
  if (!is_static)
    __ null_check(Rklass);

  __ st(O1, Rklass, Roffset);
  __ bind(exit);
}

void TemplateTable::fast_xaccess(TosState state) {
  transition(vtos, state);
  Register Rcache = Gscratch;
  Register Roffset = Gscratch;
  Register Rreceiver = Lscratch;

  __ ld(Llocals, 0, Rreceiver);

  // access constant pool cache
  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 2);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2 * wordSize), Roffset);
  __ sll(Roffset, LogBytesPerWord, Roffset);
  __ add(Lbcp, 1, Lbcp);         // needed to report exception at the correct bcp
  // offset = 0 => assumes MacroAssembler::needs_explicit_null_check(offset) = false
  __ verify_oop(Rreceiver);
  __ null_check(Rreceiver, 0);
  __ ld(Rreceiver, Roffset, Otos_i);
  __ verify_oop(Otos_i, state);
  __ sub(Lbcp, 1, Lbcp);
}

//----------------------------------------------------------------------------------------------------
// Calls

void TemplateTable::count_calls(Register method, Register temp) {
  // implemented elsewhere
  ShouldNotReachHere();
}


void TemplateTable::fast_invokevtable() {
  transition(vtos, vtos);
  Register Rcache = Gscratch;
  Register Rtemp = Gscratch2;
  Register Rret = Lscratch;
  Register Rrecv = Gmethod;

  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize), Rret);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize), Rcache);
  // __ save_bcp();

  __ and3(Rret, 0xFF, Gscratch2);      // gets number of parameters

  __ sll(Gscratch2, LogBytesPerWord, Gscratch2);
  __ ld(Lesp, Gscratch2, O0);          // gets receiver Oop

  // receiver is in O0
  __ verify_oop(O0);

  // get return address
  const int table = (int)Interpreter::return_3_addrs_by_index_table();
  __ load_address(Address(Rtemp, (address)table));
  __ srl(Rret, 29, Rret);		// get return type
  __ sll(Rret,  2, Rret);
  __ ld(Rtemp, Rret, Rret);		// get return address

  // get receiver klass
  __ null_check(O0, oopDesc::klass_offset_in_bytes());
  __ ld(Address(O0, 0, oopDesc::klass_offset_in_bytes()), Rrecv);
  __ verify_oop(Rrecv);

  // get target methodOop & entry point
  const int base = instanceKlass::vtable_start_offset() * wordSize;
  if (vtableEntry::size() % 3 == 0) {
    // scale the vtable index by 12:
    int one_third = vtableEntry::size() / 3;
    __ sll(Rcache, exact_log2(one_third * 1 * wordSize), Rtemp);
    __ sll(Rcache, exact_log2(one_third * 2 * wordSize), Rcache);
    __ add(Rcache, Rtemp, Rcache);
  } else {
    // scale the vtable index by 8:
    __ sll(Rcache, exact_log2(vtableEntry::size() * wordSize), Rcache);
  }

#ifdef COMPILER2
  if (UseCallProfile || UseTypeProfile) {
    __ record_receiver_call_in_profile(Rrecv, O4, O5);
  }
#endif // COMPILER2

  __ add(Rrecv, Rcache, Rrecv);
  __ ld(Rrecv, base + vtableEntry::method_offset_in_bytes(), Gmethod);
  __ ld(Gmethod, methodOopDesc::interpreter_entry_offset_in_bytes(), Rcache);
   // the i2c_adapters need methodOop in Gmethod (right? %%%)

  // do the call
#ifdef ASSERT
  __ tst(Rcache);
  __ breakpoint_trap(Assembler::zero);
#endif // ASSERT
  if (!UseC2CallingConventions) __ add(Lesp, BytesPerWord, Gargs); // setup parameter pointer
  __ jmp(Rcache, 0);
  __ delayed()->add(Rret, -frame::pc_return_offset * wordSize, O7);
}


void TemplateTable::fast_invokevfinal() {
  transition(vtos, vtos);
  Register Rcache = Gscratch;
  Register Rtemp = Gscratch2;
  Register Rret = Lscratch;

  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize), Rret);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize), Gmethod);
  // __ save_bcp();

  __ verify_oop(Gmethod);

  //%%% use the cache entry (already in a register) to get the param size
  __ lduh(Address(Gmethod, 0, methodOopDesc::size_of_parameters_offset_in_bytes()), Gscratch2);

  __ sll(Gscratch2, LogBytesPerWord, Gscratch2);
  __ ld(Lesp, Gscratch2, O0);           // gets receiver Oop

  
  // receiver NULL check 
  __ null_check(O0);

  // get return address
  const int table = (int)Interpreter::return_3_addrs_by_index_table();
  __ load_address(Address(Rtemp, (address)table));
  __ srl(Rret, 29, Rret);		// get return type
  __ sll(Rret,  2, Rret);
  __ ld(Rtemp, Rret, Rret);		// get return address

#ifdef COMPILER2
  if (UseCallProfile) {
    __ record_static_call_in_profile(O4, O5);
  }
#endif // COMPILER2

  // do the call
  __ ld(Address(Gmethod, 0, methodOopDesc::interpreter_entry_offset_in_bytes()), Rcache);
  if (!UseC2CallingConventions) __ add(Lesp, BytesPerWord, Gargs); // setup parameter pointer
  __ jmp(Rcache, 0);
  __ delayed()->add(Rret, -frame::pc_return_offset * wordSize, O7);
}


void TemplateTable::fast_invokespecial() {
  transition(vtos, vtos);
  Register Rcache = Gscratch;
  Register Rtemp = Gscratch2;
  Register Rret = Lscratch;

  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize), Rret);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 1*wordSize), Gmethod);
  // __ save_bcp();

  __ verify_oop(Gmethod);

  // get parameters
  //%%% use the cache entry (already in a register) to get the param size
  __ lduh(Address(Gmethod, 0, methodOopDesc::size_of_parameters_offset_in_bytes()), Gscratch2);

  __ sll(Gscratch2, LogBytesPerWord, Gscratch2);
  __ ld(Lesp, Gscratch2, O0);          // gets receiver Oop

  // receiver NULL check
  __ null_check(O0);

  // get return address
  const int table = (int)Interpreter::return_3_addrs_by_index_table();
  __ load_address(Address(Rtemp, (address)table));
  __ srl(Rret, 29, Rret);		// get return type
  __ sll(Rret,  2, Rret);
  __ ld(Rtemp, Rret, Rret);		// get return address

#ifdef COMPILER2
  if (UseCallProfile) {
    __ record_static_call_in_profile(O4, O5);
  }
#endif // COMPILER2

  // do the call
  __ ld(Address(Gmethod, 0, methodOopDesc::interpreter_entry_offset_in_bytes()), Rcache);
  if (!UseC2CallingConventions) __ add(Lesp, BytesPerWord, Gargs); // setup parameter pointer
  __ jmp(Rcache, 0);
  __ delayed()->add(Rret, -frame::pc_return_offset * wordSize, O7);
}


void TemplateTable::fast_invokestatic() {
  transition(vtos, vtos);
  Register Rcache = Gscratch;
  Register Rtemp = Gscratch2;
  Register Rret = Lscratch;

  __ get_cache_and_index_at_bcp(Rcache, Gscratch2, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize), Rret);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 1*wordSize), Gmethod);
  // __ save_bcp();

  __ verify_oop(Gmethod);

  // get return address
  const int table = (int)Interpreter::return_3_addrs_by_index_table();
  __ load_address(Address(Rtemp, (address)table));
  __ srl(Rret, 29, Rret);		// get return type
  __ sll(Rret,  2, Rret);
  __ ld(Rtemp, Rret, Rret);		// get return address

#ifdef COMPILER2
  if (UseCallProfile) {
    __ record_static_call_in_profile(O4, O5);
  }
#endif // COMPILER2

  // do the call
  __ ld(Address(Gmethod, 0, methodOopDesc::interpreter_entry_offset_in_bytes()), Rcache);
  if (!UseC2CallingConventions) __ add(Lesp, BytesPerWord, Gargs); // setup parameter pointer
  __ jmpl(Address(Rcache, 0, 0), G0, 0);
  __ delayed()->add(Rret, -frame::pc_return_offset * wordSize, O7);
}


void TemplateTable::fast_invokeinterface() {
  transition(vtos, vtos);

  Register Rcache = Gscratch;
  Register Rscratch = Gscratch2;
  Register Rret = Lscratch;
  Register Rindex = Gscratch;
  Register Rinterface = Gscratch3;
  Register RklassOop = Gmethod;
  Register Rtemp = O1;
  assert(Rscratch != Gmethod, "Invalid register specification");

  __ get_cache_and_index_at_bcp(Rcache, Rscratch, 1);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 3*wordSize), Rret);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 1*wordSize), Rinterface);
  __ ld(Address(Rcache, 0, constantPoolCacheOopDesc::base_offset_in_bytes() + 2*wordSize), Rindex);
  // _save_bcp();

  __ verify_oop(Rinterface);

  // get receiver
  __ and3(Rret, 0xFF, Rscratch);       // gets number of parameters
  __ sll(Rscratch, LogBytesPerWord, Rscratch);
  __ ld(Lesp, Rscratch, O0);           // gets receiver Oop
  __ verify_oop(O0);

  // get return address
  const int table = (int)Interpreter::return_5_addrs_by_index_table();
  __ load_address(Address(Rscratch, (address)table));
  __ srl(Rret, 29, Rret);		// get return type
  __ sll(Rret,  2, Rret);
  __ ld(Rscratch, Rret, Rret);	// get return address

  // get receiver klass
  __ null_check(O0, oopDesc::klass_offset_in_bytes());
  __ ld(O0, oopDesc::klass_offset_in_bytes(), RklassOop);
  __ verify_oop(RklassOop);

#ifdef COMPILER2
  if (UseCallProfile || UseTypeProfile) {
    __ record_receiver_call_in_profile(RklassOop, O4, O5);
  }
#endif // COMPILER2

  //
  // find entry point to call
  //

  // compute start of first itableOffsetEntry (which is at end of vtable)
  const int base = instanceKlass::vtable_start_offset() * wordSize;
  Label search;

  __ ld(Address(RklassOop, 0, instanceKlass::vtable_length_offset() * wordSize), Rtemp);
  if (align_object_offset(1) > 1) {
    __ round_to(Rtemp, align_object_offset(1));
  }
  __ sll(Rtemp, LogBytesPerWord, Rtemp);   // Rscratch *= 4;
  if (Assembler::is_simm13(base)) {
    __ add(Rtemp, base, Rtemp);
  } else {
    __ set(base, Rscratch);
    __ add(Rscratch, Rtemp, Rtemp);
  }
  __ add(RklassOop, Rtemp, Rscratch);

  __ bind(search);

  __ ld(Rscratch, itableOffsetEntry::interface_offset_in_bytes(), Rtemp); 
#ifdef ASSERT
  {
    Label ok;

    // Check that entry is non-null and an Oop
    __ tst(Rtemp);
    __ br(Assembler::notZero, false, Assembler::pt, ok);
    __ delayed()->nop();
    __ stop("null interface Oop found in itable's offset table");
    __ bind(ok);
    __ verify_oop(Rtemp);
  }
#endif // ASSERT

  __ cmp(Rinterface, Rtemp);
  __ br(Assembler::notEqual, true, Assembler::pn, search);
  __ delayed()->add(Rscratch, itableOffsetEntry::size() * wordSize, Rscratch);

  // entry found and Rscratch points to it
  __ ld(Rscratch, itableOffsetEntry::offset_offset_in_bytes(), Rscratch);

  assert(itableMethodEntry::method_offset_in_bytes() == 0, "adjust instruction below");
  __ sll(Rindex, exact_log2(itableMethodEntry::size() * wordSize), Rindex);       // Rindex *= 8;
  __ add(Rscratch, Rindex, Rscratch);
  __ ld(RklassOop, Rscratch, Gmethod);

  // Check for abstract method error.
  {
    Label ok;
    __ tst(Gmethod);
    __ br(Assembler::notZero, false, Assembler::pt, ok);
    __ delayed()->nop();
    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_abstract_method_error));
    __ should_not_reach_here();
    __ bind(ok);
  }

  __ verify_oop(Gmethod);
  __ ld(Gmethod, methodOopDesc::interpreter_entry_offset_in_bytes(), Rscratch);

  // do the call (Rscratch has the entry point now)
#ifdef ASSERT
  {
    Label ok;
    __ tst(Rscratch);
    __ br(Assembler::notZero, false, Assembler::pt, ok);
    __ delayed()->nop();
    __ stop("null entry point found in itable's vtable");
    __ bind(ok);
  }
#endif // ASSERT
  if (!UseC2CallingConventions) {
    NEEDS_CLEANUP
    __ mov(Rscratch, Rtemp);
    __ add(Lesp, BytesPerWord, Gargs); // setup parameter pointer
    __ jmp(Rtemp, 0);
    __ delayed()->add(Rret, -frame::pc_return_offset * wordSize, O7);
    return;
  }
  __ jmp(Rscratch, 0);
  __ delayed()->add(Rret, -frame::pc_return_offset * wordSize, O7);
}

//----------------------------------------------------------------------------------------------------
// Allocation

void TemplateTable::_new() {
  transition(vtos, atos);
  // %%%%% factor across plats-
  // %%%%%% The Intel version has hand-written assembly code here, but I do not believe it is worthwhile.
  // %%%%%% At the very most, the stub-generated code should be reused here. 

  __ get_2_byte_integer_at_bcp(1, Gscratch, O2, InterpreterMacroAssembler::Unsigned);
  __ get_constant_pool(O1);
     call_VM(Otos_i, CAST_FROM_FN_PTR(address, InterpreterRuntime::_new), O1, O2);
}


void TemplateTable::fast_new() {
  transition(vtos, atos);
  Label slow_case;
  Label done;
  Label initialize_header;
  Label initialize_object;  // including clearing the fields
  
  Register RallocatedObject = Otos_i;
  Register RinstanceKlass = O1;
  Register RinstanceSize = O2;
  Register Roffset = O3;
  Register Rscratch = O4;
  
  // make sure this code should be called
  if (!UseFastInterpreterAllocation) {
    __ should_not_reach_here();
    return;
  } 

  __ get_2_byte_integer_at_bcp(1, Rscratch, Roffset, InterpreterMacroAssembler::Unsigned);
  __ sll(Roffset, LogBytesPerWord, Roffset);
  __ get_constant_pool(Rscratch);
  __ add(Roffset, sizeof(constantPoolOopDesc), Roffset);
  __ ld(Rscratch, Roffset, RinstanceKlass);

  // make sure klass is initialized:
  #ifdef ASSERT
    // make sure klass is fully initialized
    { Label L;
      __ ld(RinstanceKlass, instanceKlass::init_state_offset_in_bytes() + sizeof(oopDesc), Gscratch);
      __ cmp(Gscratch, instanceKlass::fully_initialized);

      __ br(Assembler::equal, false, Assembler::pt, L);
      __ delayed()->nop();
      __ stop("fast_new executed where klass is not fully initialized");
      __ bind(L);
    }
  #endif
    
  // get instance size in instanceKlass (already aligned if ALIGN_ALL_OBJECTS is set)
  __ ld(RinstanceKlass, Klass::size_helper_offset_in_bytes() + sizeof(oopDesc), RinstanceSize);
  __ sll(RinstanceSize, LogBytesPerWord, Roffset);
 
  // allocate the instance
  // 1) Try to allocate in the TLE
  // 2) if fail and the object is large allocate in in the shared Eden
  // 3) if the above fails (or is not applicable), go to a slow case (creates a new TLE, etc.)

  if(UseTLE) {
    Register RoldTopValue = RallocatedObject;
    Register RtopAddr = Gscratch, RtleSizeValue = Gscratch;
    Register RnewTopValue = Gscratch3;
    Register RendValue = Rscratch;
 
    // check if we can allocate in the TLE
    __ ld(Gthread, JavaThread::tle_top_offset_in_bytes(), RoldTopValue); // sets up RalocatedObject
    __ ld(Gthread, JavaThread::tle_end_offset_in_bytes(), RendValue);
    __ add(RoldTopValue, Roffset, RnewTopValue);   

    // if there is enough space, we do not CAS and do not clear
    __ cmp(RnewTopValue, RendValue);
    if(ZeroTLE) {
      // the fields have been lready cleared
      __ br(Assembler::lessEqualUnsigned, true, Assembler::pt, initialize_header);
    }
    else {
      // initialize both the header and fields
      __ br(Assembler::lessEqualUnsigned, true, Assembler::pt, initialize_object);
    }
    __ delayed()->st(RnewTopValue, Gthread, JavaThread::tle_top_offset_in_bytes());

    // "large" object that do not fit allocated in the shared space
    __ ld(Gthread, JavaThread::tle_size_offset_in_bytes(), RtleSizeValue);
    __ srl(RtleSizeValue, ThreadLocalEden::log2_fragmentation_ratio(), RtleSizeValue);  // large object?
    __ cmp(RtleSizeValue, Roffset);
    __ br(Assembler::greaterEqualUnsigned, false, Assembler::pt, slow_case); // "small" object does not fit
    __ delayed()->nop();
    
  }

  // Allocation in the shared Eden
  {
    Register RoldTopValue = Gscratch3;
    Register RtopAddr = Gscratch;
    Register RnewTopValue = RallocatedObject;
    Register RendValue = Rscratch;

    __ set((int)Universe::new_gen()->eden()->top_addr(), RtopAddr);

    Label retry;
    __ bind(retry);
    __ set((int)Universe::new_gen()->eden()->end_addr(), RendValue);
    __ ld(RendValue, 0, RendValue);
    __ ld(RtopAddr, 0, RoldTopValue);
    __ add(RoldTopValue, Roffset, RnewTopValue);
 
    // RnewTopValue contains the top address after the new object
    // has been allocated.
    __ cmp(RnewTopValue, RendValue);
    __ br(Assembler::greaterUnsigned, false, Assembler::pn, slow_case);
    __ delayed()->nop();

    __ cas_under_lock(RtopAddr, RoldTopValue, RnewTopValue,
      VM_Version::v9_instructions_work() ? NULL :
      (address)StubRoutines::sparc::atomic_memory_operation_lock_addr());

    // if someone beat us on the allocation, try again, otherwise continue
    __ cmp(RoldTopValue, RnewTopValue);
    __ br(Assembler::notEqual, false, Assembler::pn, retry);
    __ delayed()->nop();
  }

  // clear object fields
  __ bind(initialize_object);
  __ deccc(RinstanceSize, sizeof(oopDesc) / oopSize);
  __ br(Assembler::zero, false, Assembler::pt, initialize_header);
  __ delayed()->nop();

  __ add(RallocatedObject, sizeof(oopDesc), Gscratch);
  __ sll(RinstanceSize, LogBytesPerWord, RinstanceSize);
 
  // initialize remaining object fields
  { Label loop;
    __ bind(loop);
    __ sub(RinstanceSize, wordSize, Roffset);
    __ st(G0, Gscratch, Roffset);
    __ subcc(RinstanceSize, wordSize, RinstanceSize);
    __ br(Assembler::equal, false, Assembler::pt, initialize_header);
    __ delayed()->nop();
    __ br(Assembler::always, false, Assembler::pt, loop);
    __ delayed()->nop();
  }

  // slow case
  __ bind(slow_case);
  __ get_2_byte_integer_at_bcp(1, Gscratch, O2, InterpreterMacroAssembler::Unsigned);
  __ get_constant_pool(O1);

  call_VM(Otos_i, CAST_FROM_FN_PTR(address, InterpreterRuntime::_new), O1, O2);

  __ br(Assembler::always, false, Assembler::pt, done);
  __ delayed()->nop();

  // Initialize the header: mark, klass
  __ bind(initialize_header);

  __ set((int)markOopDesc::prototype(), Gscratch2);
  __ st(Gscratch2, RallocatedObject, oopDesc::mark_offset_in_bytes());       // mark
  __ st(RinstanceKlass, RallocatedObject, oopDesc::klass_offset_in_bytes()); // klass  
  

  // continue
  __ bind(done);
}



void TemplateTable::newarray() {
  transition(itos, atos);
  __ ldub(Lbcp, 1, O1);
     call_VM(Otos_i, CAST_FROM_FN_PTR(address, InterpreterRuntime::newarray), O1, Otos_i);
}


void TemplateTable::anewarray() {
  transition(itos, atos);
  __ get_constant_pool(O1);
  __ get_2_byte_integer_at_bcp(1, Gscratch2, O2, InterpreterMacroAssembler::Unsigned);
     call_VM(Otos_i, CAST_FROM_FN_PTR(address, InterpreterRuntime::anewarray), O1, O2, Otos_i);
}


void TemplateTable::arraylength() {
  transition(atos, itos);
  Label ok;
  __ verify_oop(Otos_i);
  __ tst(Otos_i);
  __ throw_if_not_1( Assembler::notZero, ok );
  __ delayed()->ld(Otos_i, arrayOopDesc::length_offset_in_bytes(), Otos_i);
  __ throw_if_not_2( Interpreter::_throw_NullPointerException_entry, Gscratch, ok);
}


void TemplateTable::checkcast() {
  transition(atos, atos);
  Label done, is_null;
  Register Roffset = Gscratch3;
  Register RobjKlass = Gscratch2;
  Register RspecifiedKlass = Gscratch;

  __ tst(Otos_i);
  __ br(Assembler::zero, false, Assembler::pn, is_null);
  __ delayed()->nop();

  __ ld(Otos_i, oopDesc::klass_offset_in_bytes(), RobjKlass); // get value klass
  __ get_2_byte_integer_at_bcp(1, Lscratch, Roffset, InterpreterMacroAssembler::Unsigned);
  __ sll(Roffset, LogBytesPerWord, Roffset);
  __ add(Roffset, sizeof(constantPoolOopDesc), Roffset);
  __ get_constant_pool(Lscratch);
  __ ld(Lscratch, Roffset, RspecifiedKlass);
  __ cmp(RspecifiedKlass, RobjKlass);
  __ br(Assembler::equal, false, Assembler::pt, done);
  __ delayed()->ld(Address(RobjKlass, 0, sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes()), Lscratch);
  __ cmp(RspecifiedKlass, Lscratch);
  __ br(Assembler::equal, false, Assembler::pt, done);
  __ delayed()->ld(Address(RobjKlass, 0, sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes()), Lscratch);
  __ cmp(RspecifiedKlass, Lscratch);
  __ br(Assembler::equal, false, Assembler::pt, done);
  __ delayed()->nop();

  __ push(atos); // save receiver for result, and for GC
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::checkcast), Otos_i);
  __ pop(atos); // restore receiver
#ifdef CORE
  __ bind(is_null);
#else
  // Collect counts on whether this check-cast sees NULLs a lot or not.
#ifdef COMPILER2
  if (UncommonNullCast) {
    __ ba(false, done);
    __ delayed()->nop();

    __ bind(is_null);
    // Flag as seeing a null cast
    __ set(1, Roffset);
    __ stb(Roffset, Lmethod, methodOopDesc::null_cast_seen_offset_in_bytes());
  } else {
    __ bind(is_null);
  }
#else
  __ bind(is_null);
#endif
#endif /* not CORE */
  __ bind(done);
}


void TemplateTable::instanceof() {
  transition(atos, itos);
  Label done;
  Register Roffset = Gscratch3;
  Register RobjKlass = Gscratch2;
  Register RspecifiedKlass = Gscratch;

  __ tst(Otos_i);
  __ br(Assembler::zero, false, Assembler::pt, done);
  __ delayed()->nop();

  __ ld(Otos_i, oopDesc::klass_offset_in_bytes(), RobjKlass); // get value klass
  __ get_2_byte_integer_at_bcp(1, Lscratch, Roffset, InterpreterMacroAssembler::Unsigned);
  __ sll(Roffset, 2, Roffset);
  __ add(Roffset, sizeof(constantPoolOopDesc), Roffset);
  __ get_constant_pool(Lscratch);
  __ ld(Lscratch, Roffset, RspecifiedKlass);
  __ mov(Otos_i, Gscratch3);  // save object in case quick tests fail
  __ or3(G0, 1, Otos_i);      // set result assuming quick tests succeed
  __ cmp(RspecifiedKlass, RobjKlass);
  __ br(Assembler::equal, false, Assembler::pt, done);
  __ delayed()->ld(Address(RobjKlass, 0, sizeof(oopDesc) + Klass::is_a_cache_1_offset_in_bytes()), Lscratch);
  __ cmp(RspecifiedKlass, Lscratch);
  __ br(Assembler::equal, false, Assembler::pt, done);
  __ delayed()->ld(Address(RobjKlass, 0, sizeof(oopDesc) + Klass::is_a_cache_2_offset_in_bytes()), Lscratch);
  __ cmp(RspecifiedKlass, Lscratch);
  __ br(Assembler::equal, false, Assembler::pt, done);
  __ delayed()->nop();

  __ push_i(Gscratch3); // save receiver for result, and for GC
  call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::instanceof), Gscratch3);
  __ mov(O0, Otos_i);
  __ pop_i(Gscratch3); // discard object, result is in Otos_i

  __ bind(done);
}

void TemplateTable::_breakpoint() {

   // Note: We get here even if we are single stepping..
   // jbug inists on setting breakpoints at every bytecode 
   // even if we are in single step mode.  
 
   transition(vtos, vtos);
   // get the unpatched byte code
   __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::get_original_bytecode_at), Lmethod, Lbcp);
   __ mov(O0, Lbyte_code);

   // post the breakpoint event
   __ call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::_breakpoint), Lmethod, Lbcp);

   // complete the execution of original bytecode
   __ dispatch_normal(vtos);
}


//----------------------------------------------------------------------------------------------------
// Exceptions

void TemplateTable::athrow() {
  transition(atos, vtos);

  // This works because exception is cached in Otos_i which is same as O0,
  // which is same as what throw_exception_entry_expects 
  assert(Otos_i == Oexception, "see explanation above");

  __ verify_oop(Otos_i);
  __ null_check(Otos_i);
  __ throw_if_not(Assembler::never, Interpreter::throw_exception_entry(), Gscratch);
}


//----------------------------------------------------------------------------------------------------
// Synchronization


// See frame_sparc.hpp for monitor block layout.
// Monitor elements are dynamically allocated by growing stack as needed.

void TemplateTable::monitorenter() {
  transition(atos, vtos);
  __ verify_oop(Otos_i);
  // Try to acquire a lock on the object
  // Repeat until succeeded (i.e., until
  // monitorenter returns true).

  {   Label ok;
    __ tst(Otos_i);
    __ throw_if_not_1( Assembler::notZero,  ok);
    __ delayed()->mov(Otos_i, Lscratch); // save obj
    __ throw_if_not_2( Interpreter::_throw_NullPointerException_entry, Gscratch, ok);
  }

  assert(O0 == Otos_i, "Be sure where the object to lock is");

  // find a free slot in the monitor block


  // initialize entry pointer
  __ clr(O1); // points to free slot or NULL

  {
    Label entry, loop, exit;
    __ add( __ top_most_monitor(), O2 ); // last one to check
    __ ba( false, entry );
    __ delayed()->mov( Lmonitors, O3 ); // first one to check


    __ bind( loop );

    { Label done;
      __ tst(O4); // is this entry unused?
      __ br( Assembler::notZero, false, Assembler::pt, done);
      __ delayed()->nop();
      __ bind(done);
    }

    __ verify_oop(O4);		// verify each monitor's oop
    __ tst(O4); // is this entry unused?
    if (VM_Version::v9_instructions_work())
      __ movcc( Assembler::zero, false, Assembler::icc, O3, O1);
    else {
      Label L;
      __ br( Assembler::zero, true, Assembler::pn, L );
      __ delayed()->mov(O3, O1); // rememeber this one if match
      __ bind(L);        
    }

    { Label done;
      __ cmp(O4, O0);
      __ br( Assembler::notEqual, false, Assembler::pt, done);
      __ delayed()->nop();
      __ bind(done);
    }
    __ cmp(O4, O0); // check if current entry is for same object
    __ br( Assembler::equal, false, Assembler::pn, exit );
    __ delayed()->inc( O3, frame::interpreter_frame_monitor_size() * wordSize ); // check next one


    __ bind( entry );

    __ cmp( O3, O2 );
    __ br( Assembler::lessEqualUnsigned, true, Assembler::pt, loop );
    __ delayed()->ld(O3, BasicObjectLock::obj_offset_in_bytes(), O4);


    __ bind( exit );
  }

  { Label allocated;

    __ tst(O1); // found free slot?
    __ br(Assembler::notZero, false, Assembler::pn, allocated);
    __ delayed()->nop();

    __ add_monitor_to_stack( false, O2, O3 );
    __ mov(Lmonitors, O1);

    __ bind(allocated);
  }

  // Increment bcp to point to the next bytecode, so exception handling for async. exceptions work correctly. 
  // The object has already been poped from the stack, so the expression stack looks correct.
  __ inc(Lbcp);
  
  __ st(O0, O1, BasicObjectLock::obj_offset_in_bytes()); // store object
  __ lock_object(O1, O0);

  // The bcp has already been incremented. Just need to dispatch to next instruction.
  __ dispatch_next(vtos); 
}


void TemplateTable::monitorexit() {
  transition(atos, vtos);
  __ verify_oop(Otos_i);
  __ tst(Otos_i);
  __ throw_if_not( Assembler::notZero,  Interpreter::_throw_NullPointerException_entry,
		   Gscratch );

  assert(O0 == Otos_i, "just checking");

  { Label entry, loop, found;
    __ add( __ top_most_monitor(), O2 ); // last one to check
    __ ba(false, entry );
    // use Lscratch to hold monitor elem to check, start with most recent monitor,
    // By using a local it survives the call to the C routine.
    __ delayed()->mov( Lmonitors, Lscratch ); 

    __ bind( loop );

    __ verify_oop(O4);		// verify each monitor's oop
    __ cmp(O4, O0); // check if current entry is for desired object
    __ br( Assembler::equal, true, Assembler::pt, found );
    __ delayed()->mov(Lscratch, O1); // pass found entry as argument to monitorexit

    __ inc( Lscratch, frame::interpreter_frame_monitor_size() * wordSize ); // advance to next

    __ bind( entry );

    __ cmp( Lscratch, O2 );
    __ br( Assembler::lessEqualUnsigned, true, Assembler::pt, loop );
    __ delayed()->ld(Lscratch, BasicObjectLock::obj_offset_in_bytes(), O4);

    call_VM(noreg, CAST_FROM_FN_PTR(address, InterpreterRuntime::throw_illegal_monitor_state_exception));
    __ should_not_reach_here();

    __ bind(found);
  }
  __ unlock_object(O1);
}


//----------------------------------------------------------------------------------------------------
// Wide instructions

void TemplateTable::wide() {
  transition(vtos, vtos);
  __ ldub(Lbcp, 1, Gscratch);// get next bc
  __ sll(Gscratch, LogBytesPerWord, Gscratch); 
  Address ep(Gscratch2, (address)Interpreter::_wentry_point);
  __ load_address(ep);
  __ ld(ep.base(), Gscratch, Gscratch);
  __ jmp(Gscratch, G0);
  __ delayed()->nop();
  // Note: the Lbcp increment step is part of the individual wide bytecode implementations
}


//----------------------------------------------------------------------------------------------------
// Multi arrays

void TemplateTable::multianewarray() {
  transition(vtos, atos);
     // put ndims * wordSize into Lscratch
  __ ldub( Lbcp,     3,               Lscratch);
  __ sll(  Lscratch, LogBytesPerWord, Lscratch);
     // Lesp points past last_dim, so set to O1 to first_dim address
  __ add(  Lesp,     Lscratch,        O1); 
     call_VM(Otos_i, CAST_FROM_FN_PTR(address, InterpreterRuntime::multianewarray), O1);
  __ add(  Lesp,     Lscratch,        Lesp); // pop all dimensions off the stack
}


//Reconciliation History
// 1.81 97/11/12 09:26:42 templateTable_i486.cpp
// 1.88 97/12/04 10:19:22 templateTable_i486.cpp
// 1.89 97/12/08 13:19:30 templateTable_i486.cpp
// 1.90 97/12/12 15:43:37 templateTable_i486.cpp
// 1.91 98/01/12 11:39:11 templateTable_i486.cpp
// 1.98 98/02/03 19:06:46 templateTable_i486.cpp
// 1.105 98/02/11 12:07:56 templateTable_i486.cpp
// 1.107 98/03/03 14:38:36 templateTable_i486.cpp
// 1.115 98/03/18 13:14:33 templateTable_i486.cpp
// 1.124 98/03/30 17:20:33 templateTable_i486.cpp
// 1.142 98/04/08 13:43:13 templateTable_i486.cpp
// 1.159 98/04/16 13:20:45 templateTable_i486.cpp
// 1.167 98/05/04 11:55:17 templateTable_i486.cpp
// 1.170 98/05/19 18:08:50 templateTable_i486.cpp
// 1.172 98/05/29 17:36:52 templateTable_i486.cpp
// 1.173 98/06/22 14:49:10 templateTable_i486.cpp
// 1.175 98/07/24 13:34:39 templateTable_i486.cpp
// 1.176 98/07/31 11:41:18 templateTable_i486.cpp
// 1.177 98/08/28 11:31:14 templateTable_i486.cpp
// 1.179 98/10/02 15:22:49 templateTable_i486.cpp
// 1.180 98/10/05 02:32:12 templateTable_i486.cpp
// 1.181 98/10/05 16:30:27 templateTable_i486.cpp
// 1.183 98/10/15 14:08:32 templateTable_i486.cpp
// 1.184 98/10/26 11:37:16 templateTable_i486.cpp
// 1.185 98/11/12 11:58:41 templateTable_i486.cpp
// 1.186 98/12/17 15:37:41 templateTable_i486.cpp
// 1.187 99/01/05 17:32:03 templateTable_i486.cpp
// 1.188 99/01/06 11:43:54 templateTable_i486.cpp
// 1.189 99/01/20 15:59:45 templateTable_i486.cpp
// 1.190 99/02/19 12:51:05 templateTable_i486.cpp
// 1.192 99/04/01 13:02:53 templateTable_i486.cpp
// 1.199 99/07/13 11:16:59 templateTable_i486.cpp
// 1.201 99/07/19 10:21:37 templateTable_i486.cpp
// 1.198 99/06/02 15:27:40 templateTable_i486.cpp
// 1.200 99/06/24 14:25:02 templateTable_i486.cpp
// 1.207 99/07/23 18:43:52 templateTable_i486.cpp
// 1.207 99/08/17 02:40:41 templateTable_i486.cpp
// 1.208 99/08/17 11:36:54 templateTable_i486.cpp
// 1.208 99/08/26 16:15:39 templateTable_i486.cpp
// 1.209 99/08/27 12:55:50 templateTable_i486.cpp
// 1.208 99/10/19 15:05:40 templateTable_i486.cpp
//End

//TO DO: 1.204 99/07/20 10:58:53 templateTable_i486.cpp
