/*
 * @(#)vm_version_sparc.cpp	1.29 00/09/29 14:51:33
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_vm_version_sparc.cpp.incl"

bool VM_Version::_v8_instructions_work = false;
bool VM_Version::_v9_instructions_work = false;

static jlong stubCode[10]; // want doubleword alignment
extern "C" {
    typedef int (*_getpsr_stub_t)(void);
}
static _getpsr_stub_t getpsr_stub = NULL; 


class VM_Version_StubGenerator: public StubCodeGenerator {
 public:

  VM_Version_StubGenerator(CodeBuffer *c) : StubCodeGenerator(c) {}

  address generate_getpsr() {
    StubCodeMark mark(this, "VM_Version", "getpsr_stub");
#   define __ _masm->
    address start = __ pc();

    __ get_psr_trap();
    __ retl();
    __ delayed()->nop();

#   undef __
  
    return start;
  };
};


void VM_Version::initialize() {
  ResourceMark rm;
  // Making this stub must be FIRST use of assembler
  CodeBuffer* c = new CodeBuffer(address(stubCode), sizeof(stubCode));
  VM_Version_StubGenerator g(c);
  getpsr_stub = CAST_TO_FN_PTR(_getpsr_stub_t, g.generate_getpsr());
  determine_sparc_version();
  if (VM_Version::v9_instructions_work()) {
    // Thse settings were determined to be optimal for ultra3.
    // These flags must stay zero on v8 systems, and have little 
    // effect on ultra2.
    PrefetchScavengeFieldsAhead = 1;
    PrefetchCopyIntervalInBytes = 512;
  } else {
    // saving of ccr register is not implemented for V8
    // note: cannot set UseCompilerSafepoints in arguments_sparc.cpp
    //       since the v9_instructions_work() flag is only set now
    UseCompilerSafepoints = false;  
  }
}

void VM_Version::determine_sparc_version() {
  // According to Sreeram Duvurru
  // Version = 0xfe	-> ultraSparc
  // Version = 0x41	-> superSparc
  // Version = 0x4	-> microSparc

  if (UseV8InstrsOnly) {
     _v8_instructions_work = true;
     _v9_instructions_work = false;
     if (Verbose)  tty->print_cr("Version is Forced-V8");
     return;
  } 

  // According to v8 manual:
  u_char vers = (getpsr_stub() >> 28) & 0xf;
  if (Verbose) {
    tty->print_cr("PSR (Version) = 0x%x", vers);
  }

  switch ( vers ) {
   case 0x0:
   case 0x1:
   case 0x4:
    if (Verbose)  tty->print_cr("Version is V8");
    _v8_instructions_work = true;
    _v9_instructions_work = false;
    break;

   case 0xf:
    if (Verbose)  tty->print_cr("Version is V9");
    _v8_instructions_work = false;
    _v9_instructions_work = true;
    break;

  default:
    warning("Cannot recognize SPARC version (0x%x). Default to V9", vers);
    _v9_instructions_work = true;
    _v8_instructions_work = false;
    break;
  }

  if (Verbose) {
    tty->print("This CPU supports SPARC %s instructions.\n",
      	_v8_instructions_work ? "V8" : _v9_instructions_work ? "V9" : "Unknown-Version");
  }
}

static bool had_v8;
static bool had_v9;

void VM_Version::allow_all() {
  had_v8 = _v8_instructions_work;  _v8_instructions_work = true;
  had_v9 = _v9_instructions_work;  _v9_instructions_work = true;
}

void VM_Version::revert() {
  _v8_instructions_work = had_v8;
  _v9_instructions_work = had_v9;
}


void VM_Version_init() {
  VM_Version::initialize();
}
