/*
 * @(#)vtableStubs_sparc.cpp	1.32 00/04/18 17:46:14
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_vtableStubs_sparc.cpp.incl"

// machine-dependent part of VtableStubs: create vtableStub of correct size and
// initialize its code

#define __ masm->


#ifndef PRODUCT
extern "C" void bad_compiled_vtable_index(JavaThread* thread, oop receiver, int index);
#endif


// used by compiler only; may use only caller saved, non-argument registers
// NOTE:  %%%% if any change is made to this stub make sure that the function
//             pd_code_size_limit is changed to ensure the correct size for VtableStub
VtableStub* VtableStubs::create_vtable_stub(int vtable_index, int receiver_location) {
  const int sparc_code_length = VtableStub::pd_code_size_limit(true);
  VtableStub* s = new(sparc_code_length) VtableStub(true, vtable_index, receiver_location);
  ResourceMark rm;
  MacroAssembler* masm = new MacroAssembler(new CodeBuffer(s->entry_point(), sparc_code_length));

#ifndef PRODUCT
#ifdef COMPILER2
  if (CountCompiledCalls) {
    Address ctr(Gscratch2, OptoRuntime::nof_megamorphic_calls_addr());
    __ sethi(ctr);
    __ ld(ctr, Gscratch);
    __ inc(Gscratch);
    __ st(Gscratch, ctr);
  }
#endif
#endif

  COMPILER1_ONLY(assert(receiver_location == 0,        "receiver expected in O0");)
  COMPILER2_ONLY(assert(receiver_location == R_O0_num, "receiver expected in O0");)

  // get receiver klass
  address npe_addr = __ pc();
  __ ld(O0, oopDesc::klass_offset_in_bytes(), Gscratch);

  // set methodOop (in case of interpreted method), and destination address
  int entry_offset = instanceKlass::vtable_start_offset() + vtable_index*vtableEntry::size();
#ifndef PRODUCT
  if (DebugVtables) { 
    Label L;
    // check offset vs vtable length
    __ ld(Gscratch, instanceKlass::vtable_length_offset()*wordSize, Gscratch2);
    __ cmp(Gscratch2, vtable_index*vtableEntry::size());
    __ br(Assembler::greaterUnsigned, false, Assembler::pt, L);
    __ delayed()->nop();
    __ set(vtable_index, O2);
    __ call_VM(noreg, CAST_FROM_FN_PTR(address, bad_compiled_vtable_index), O0, O2);
    __ bind(L);
  }
#endif
  Address entry_addr(Gscratch, 0, entry_offset*wordSize);
  int slop = 2*wordSize;
  if (!entry_addr.is_simm13(vtableEntry::method_offset_in_bytes())) {
    __ split_disp(entry_addr, Gscratch2); // add long part of offset to Gscratch
    slop = 0;
  }
  __ ld(entry_addr, Gmethod, vtableEntry::method_offset_in_bytes());

  #ifndef PRODUCT
  if (DebugVtables) {
    Label L;
    __ cmp(Gmethod, NULL);
    __ br(Assembler::notZero, false, Assembler::pt, L);
    __ delayed()->nop();
    __ stop("Vtable entry is ZERO");
    __ bind(L);
  }
  #endif

  address ame_addr = __ pc();  // if the vtable entry is null, the method is abstract
  __ ld(Gmethod, methodOopDesc::from_compiled_code_entry_point_offset_in_bytes(), Gscratch);

  // jump to target (either compiled code or c2iadapter)
  __ jmp(Gscratch, 0);
  // load methodOop (in case we call c2iadapter)
  __ delayed()->nop();

  masm->flush();
  s->set_exception_points(npe_addr, ame_addr);
  return s;
}


// NOTE:  %%%% if any change is made to this stub make sure that the function
//             pd_code_size_limit is changed to ensure the correct size for VtableStub
VtableStub* VtableStubs::create_itable_stub(int vtable_index, int receiver_location) {
  const int sparc_code_length = VtableStub::pd_code_size_limit(false);
  VtableStub* s = new(sparc_code_length) VtableStub(false, vtable_index, receiver_location);
  ResourceMark rm;
  MacroAssembler* masm = new MacroAssembler(new CodeBuffer(s->entry_point(), sparc_code_length));

  Register Rscratch = Gscratch2;
  Register RklassOop = Gscratch;
  Register Rtemp = Gscratch3;
  Register Rinterface = Gmegamorphic_method;
  Label search;
  assert(Gmethod != Rscratch, "Invalid register specification");

  #ifndef PRODUCT
  #ifdef COMPILER2
  if (CountCompiledCalls) {
    Address ctr(Gscratch2, OptoRuntime::nof_megamorphic_calls_addr());
    __ sethi(ctr);
    __ ld(ctr, Gscratch);
    __ inc(Gscratch);
    __ st(Gscratch, ctr);
  }
  #endif
  #endif

  COMPILER1_ONLY(assert(receiver_location == 0,        "receiver expected in O0");)
  COMPILER2_ONLY(assert(receiver_location == R_O0_num, "receiver expected in O0");)

  // Entry arguments:
  //  Rinterface: Interface
  //  O0:         Receiver

  // get receiver klass (also an implicit null-check)
  address npe_addr = __ pc();
  __ ld(O0, oopDesc::klass_offset_in_bytes(), RklassOop);
  #ifdef PRODUCT
  __ verify_oop(RklassOop);
  #endif

  // load start of itable entries into Rscratch register
  const int base = instanceKlass::vtable_start_offset() * wordSize;    
  __ ld(Address(RklassOop, 0, instanceKlass::vtable_length_offset() * wordSize), Rscratch);

  // %%% Could store the aligned, prescaled offset in the klassoop.
  __ sll(Rscratch, exact_log2(vtableEntry::size() * wordSize), Rscratch);
  // see code for instanceKlass::start_of_itable!
  const int vtable_alignment = align_object_offset(1);
  assert(vtable_alignment == 1 || vtable_alignment == 2, "");
  const int odd_bit = vtableEntry::size() * wordSize;
  if (vtable_alignment == 2) {
    __ and3(Rscratch, odd_bit, Rtemp);  // isolate the odd bit
  }
  __ add(RklassOop, Rscratch, Rscratch);
  if (vtable_alignment == 2) {
    __ add(Rscratch, Rtemp, Rscratch);  // double the odd bit, to align up
  }

  // Loop over all itable entries until desired interfaceOop(Rinterface) found
  __ bind(search);

  // %%%% Could load both offset and interface in one ldx, if they were
  // in the opposite order.  This would save a load.
  __ ld(Rscratch, base + itableOffsetEntry::interface_offset_in_bytes(), Rtemp);
#ifdef ASSERT
  Label ok;
#ifdef COMPILER2
  // Check that entry is non-null and an Oop
  __ tst(Rtemp);
  __ br(Assembler::notZero, false, Assembler::pt, ok);
  __ delayed()->nop();
  __ stop("null entry point found in itable's offset table");
  __ bind(ok);
  __ verify_oop(Rtemp);
#endif // COMPILER2
#endif // ASSERT

  __ cmp(Rinterface, Rtemp);
  __ br(Assembler::notEqual, true, Assembler::pn, search);
  __ delayed()->add(Rscratch, itableOffsetEntry::size() * wordSize, Rscratch);

  // entry found and Rscratch points to it, move offset of vtable for interface into Rscratch
  __ ld(Rscratch, base + itableOffsetEntry::offset_offset_in_bytes(), Rscratch);

  // Compute itableMethodEntry and get methodOop(Gmethod)  and entrypoint(Rscratch) for compiler
  const int method_offset = (itableMethodEntry::size() * wordSize * vtable_index) + itableMethodEntry::method_offset_in_bytes();
  __ add(RklassOop, Rscratch, Rtemp);
  __ ld(Rtemp, method_offset, Gmethod);

#ifdef COMPILER1
  Label notZeroL;
  __ tst(Gmethod);
  __ br(Assembler::notZero, false, Assembler::pt, notZeroL);
  __ delayed()->nop();
  Address abstract_error(Gscratch, Runtime1::entry_for(Runtime1::throw_abstract_method_error_id));
  __ jump_to(abstract_error, 0);
  __ delayed()->nop();
  __ bind(notZeroL);
#endif // COMPILER1

  #ifndef PRODUCT
  if (DebugVtables) {
    Label L01, L02;

    __ ld(Rtemp, method_offset, Gmethod);
    __ tst(Gmethod);
    __ br(Assembler::notZero, false, Assembler::pt, L01);
    __ delayed()->nop();
    __ stop("methodOop is null");
    __ bind(L01);
    __ verify_oop(Gmethod);
  }
  #endif

  address ame_addr = __ pc();  // if the vtable entry is null, the method is abstract
  __ ld(Gmethod, methodOopDesc::from_compiled_code_entry_point_offset_in_bytes(), Rscratch);

  // Gmethod:  methodOop
  // O0:       Receiver
  // Rscratch: entry point
  __ jmp(Rscratch, 0);
  __ delayed()->nop();

  masm->flush();
  s->set_exception_points(npe_addr, ame_addr);
  return s;
}


int VtableStub::pd_code_size_limit(bool is_vtable_stub) {
  if (DebugVtables || CountCompiledCalls || VerifyOops) return 999;
  else {
    const int slop = 2*wordSize; // sethi;add  (needed for long offsets)
    if (is_vtable_stub) {
      const int basic = 5*wordSize; // ld;ld;ld,jmp,nop
      return basic + slop;
    } else {
#ifdef COMPILER1    
      const int basic = 23*wordSize; 
#else 
#ifdef ASSERT
      return 999;
#endif // ASSERT
      const int basic = 16*wordSize; // ld, ld, sll, and, add, add, ld , cmp, br, add, ld, add, ld, ld, jmp, nop
#endif // COMPILER1
      return (basic + slop);
    }
  }
}


int VtableStub::pd_code_alignment() {
  // UltraSPARC cache line size is 8 instructions:
  const unsigned int icache_line_size = 32;
  return icache_line_size;
}


//Reconciliation History
// 1.2 97/12/09 17:13:31 vtableStubs_i486.cpp
// 1.4 98/01/21 19:18:37 vtableStubs_i486.cpp
// 1.5 98/02/13 16:33:55 vtableStubs_i486.cpp
// 1.7 98/03/05 17:17:28 vtableStubs_i486.cpp
// 1.9 98/05/18 09:26:17 vtableStubs_i486.cpp
// 1.10 98/05/26 16:28:13 vtableStubs_i486.cpp
// 1.11 98/05/27 08:51:35 vtableStubs_i486.cpp
// 1.12 98/06/15 15:04:12 vtableStubs_i486.cpp
// 1.13 98/07/28 18:44:22 vtableStubs_i486.cpp
// 1.15 98/08/28 11:31:19 vtableStubs_i486.cpp
// 1.16 98/09/02 12:58:31 vtableStubs_i486.cpp
// 1.17 98/09/04 12:15:52 vtableStubs_i486.cpp
// 1.18 98/11/19 11:55:24 vtableStubs_i486.cpp
// 1.19 99/01/12 14:57:56 vtableStubs_i486.cpp
// 1.20 99/01/19 17:42:52 vtableStubs_i486.cpp
// 1.22 99/01/21 10:29:25 vtableStubs_i486.cpp
// 1.30 99/06/02 15:27:39 vtableStubs_i486.cpp
// 1.26 99/06/24 14:25:07 vtableStubs_i486.cpp
// 1.23 99/02/22 14:37:52 vtableStubs_i486.cpp
// 1.28 99/06/29 18:06:17 vtableStubs_i486.cpp
// 1.29 99/07/22 17:03:44 vtableStubs_i486.cpp
// 1.30 99/08/11 09:33:27 vtableStubs_i486.cpp
//End
