/*
 * @(#)hpi_linux.hpp	1.2 00/05/13
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

//
// Because the interruptible IO has been dropped for HotSpot/Linux,
// the following HPI interface is very different from HotSparc.
//

#include <unistd.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <sys/ioctl.h>

// HPI_FileInterface

inline int hpi::close(int fd) {
  return ::close(fd);
}

inline size_t hpi::read(int fd, void *buf, unsigned int nBytes) {
  return (size_t) ::read(fd, buf, (size_t) nBytes);
}

inline size_t hpi::write(int fd, const void *buf, unsigned int nBytes) {
  return (size_t) ::write(fd, buf, (size_t) nBytes);
}


// HPI_SocketInterface

inline int hpi::socket_close(int fd) {
  return ::close(fd);
}

inline int hpi::socket(int domain, int type, int protocol) {
  return ::socket(domain, type, protocol);
}

inline int hpi::recv(int fd, char *buf, int nBytes, int flags) {
  return ::recv(fd, buf, nBytes, (unsigned int) flags);
}

inline int hpi::send(int fd, char *buf, int nBytes, int flags) {
  return ::send(fd, buf, nBytes, (unsigned int) flags);
}

inline int hpi::timeout(int fd, long timeout) {
  for (;;) {
    struct pollfd pfd;

    pfd.fd = fd;
    pfd.events = POLLIN | POLLERR;
  
    int res = ::poll(&pfd, 1, timeout);
    if (res == -1 && errno == EINTR) {
      continue;
    }
    return res;
  }
}

inline int hpi::listen(int fd, int count) {
  return ::listen(fd, count);
}

inline int hpi::connect(int fd, struct sockaddr *him, int len) {
  return ::connect(fd, him, len);
}

inline int hpi::accept(int fd, struct sockaddr *him, int *len) {
  return ::accept(fd, him, len);
}

inline int hpi::recvfrom(int fd, char *buf, int nBytes, int flags,
		         sockaddr *from, int *fromlen) {
  return ::recvfrom(fd, buf, nBytes, (unsigned int) flags, from, fromlen);
}

inline int hpi::sendto(int fd, char *buf, int len, int flags,
			struct sockaddr *to, int tolen) {
  return ::sendto(fd, buf, len, (unsigned int) flags, to, tolen);
}

inline int hpi::socket_available(int fd, long *pbytes) {
  int ret = ::ioctl(fd, FIONREAD, pbytes);

  //%% note ioctl can return 0 when successful, JVM_SocketAvailable
  // is expected to return 0 on failure and 1 on success to the jdk.
  return (ret < 0) ? 0 : 1;
}


// following methods have been updated to avoid problems in 
// hpi's sockets calls based on sys_api_td.c (JDK1.3)

/* 
HPIDECL(get_host_by_name, "get_host_by_name", _socket, GetHostByName,
        struct hostent *, "(struct hostent *)%p",
        (char *name),
        ("%s", name),
        (name));
	*/
inline struct hostent * hpi::get_host_by_name(char *hostname) {
  return ::gethostbyname(hostname);
}

/*
HPIDECL(socket_shutdown, "socket_shutdown", _socket, SocketShutdown, 
        int, "%d",
        (int fd, int howto),
        ("fd = %d, howto = %d", fd, howto),
        (fd, howto));
	*/
inline int hpi::socket_shutdown(int fd, int howto){
  return ::shutdown(fd, howto);
}

/*
HPIDECL(bind, "bind", _socket, Bind, 
        int, "%d",
        (int fd, struct sockaddr *him, int len),
        ("fd = %d, him = %p, len = %d",
         fd, him, len),
        (fd, him, len));
*/
inline int hpi::bind(int fd, struct sockaddr *him, int len){
  return ::bind(fd, him, len);
}

/*
HPIDECL(get_sock_name, "get_sock_name", _socket, GetSocketName, 
        int, "%d",
        (int fd, struct sockaddr *him, int *len),
        ("fd = %d, him = %p, len = %p",
         fd, him, len),
        (fd, him, len));
	*/
inline int hpi::get_sock_name(int fd, struct sockaddr *him, int *len){
  return ::getsockname(fd, him, len);
}

/*
HPIDECL(get_host_name, "get_host_name", _socket, GetHostName, int, "%d",
        (char *hostname, int namelen),
        ("hostname = %p, namelen = %d",
         hostname, namelen),
        (hostname, namelen));
	*/
inline int hpi::get_host_name(char* name, int namelen){
  return ::gethostname(name, namelen);
}

/*
HPIDECL(get_host_by_addr, "get_host_by_addr", _socket, GetHostByAddr, 
        struct hostent *, "(struct hostent *)%p",
        (const char* name, int len, int type),
        ("name = %p, len = %d, type = %d",
         name, len, type),
        (name, len, type));
	*/
inline struct hostent* hpi::get_host_by_addr(const char* name, int len, int type){
  return ::gethostbyaddr(name, len, type);
}

/*
HPIDECL(get_sock_opt, "get_sock_opt", _socket, SocketGetOption, int, "%d",
        (int fd, int level, int optname, char *optval, int* optlen),
        ("fd = %d, level = %d, optname = %d, optval = %p, optlen = %p",
         fd, level, optname, optval, optlen),
        (fd, level, optname, optval, optlen));
	*/
inline int hpi::get_sock_opt(int fd, int level, int optname, 
			     char *optval, int* optlen){
  return ::getsockopt(fd, level, optname, optval, optlen);
}

/*
HPIDECL(set_sock_opt, "set_sock_opt", _socket, SocketSetOption, int, "%d",
        (int fd, int level, int optname, const char *optval, int optlen),
        ("fd = %d, level = %d, optname = %d, optval = %p, optlen = %d",
         fd, level, optname, optval, optlen),
        (fd, level, optname, optval, optlen));
	*/
inline int hpi::set_sock_opt(int fd, int level, int optname, 
			     const char *optval, int optlen){
  return ::setsockopt(fd, level, optname, optval, optlen);
}

/*
HPIDECL(get_proto_by_name, "get_proto_by_name", _socket, GetProtoByName, 
        struct protoent *, "(struct protoent *)%p",
        (char* name),
        ("name = %p",
         name),
        (name));
	*/
inline struct protoent* hpi::get_proto_by_name(char* name){
  return ::getprotobyname(name);
}


// Reconciliation History
// hpi_solaris.hpp	1.9 99/08/30 16:31:23
// End
