/*
 * @(#)osThread_linux.hpp	1.10 01/03/23
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

 private:
  int _thread_type;

 public:

  int thread_type() const {
    return _thread_type;
  }
  void set_thread_type(int type) {
    _thread_type = type;
  }

 private:

  pid_t     _thread_id;
  pthread_t _pthread_id;

 public:

  pid_t thread_id() const {
    return _thread_id;
  }
#ifndef PRODUCT
  // Used for debugging, return a unique integer for each thread.
  int thread_identifier() const   { return _thread_id; }
#endif
  void set_thread_id(pid_t id) {
    _thread_id = id;
  }
  pthread_t pthread_id() const {
    return _pthread_id;
  }
  void set_pthread_id(pthread_t tid) {
    _pthread_id = tid;
  }

  // ***************************************************************
  // suspension support.
  // ***************************************************************

private:

  int _suspend_action;
  int _suspend_count;

  void* _siginfo;
  void* _ucontext;

  // LinuxThreads uses a signal to restart a thread that is waiting
  // for a contended mutex. Thread suspend/resume on Linux also uses
  // a signal. When a Linux thread is suspended it can be granted
  // ownership of a mutex because LinuxThreads knows nothing of how
  // suspend/resume is implemented by the application (HotSpot).
  // This flag is set before we try to enter a mutex and is cleared
  // once we obtain the mutex. If we contend on the mutex and we are
  // suspended while we are contended, then the flag will remain set
  // while SR_handler() is running. This allows SR_handler() to detect
  // being granted the mutex while in the suspend/resume loop.
  bool _try_mutex_enter;

public:

  int suspend_action() const {
    return _suspend_action;
  }
  void set_suspend_action(int act) {
    _suspend_action = act;
  }
// suspend_action values:
#define SR_NONE          (0)  // thread is not suspended
#define SR_SUSPEND       (1)  // other thread suspend request
#define SR_SUSPEND_SELF  (2)  // self suspend request
#define SR_SUSPENDED     (3)  // thread is (about to be) suspended
#define SR_CONTINUE      (4)  // resume request

  int suspend_count() const {
    return _suspend_count;
  }
  void set_suspend_count(int count) {
    _suspend_count = count;
  }

  void* siginfo() const {
    return _siginfo;
  }
  void set_siginfo(void* ptr) {
    _siginfo = ptr;
  }

  void* ucontext() const {
    return _ucontext;
  }
  void set_ucontext(void* ptr) {
    _ucontext = ptr;
  }

  bool is_try_mutex_enter() const {
    return _try_mutex_enter;
  }
  void clear_try_mutex_enter() {
    _try_mutex_enter = false;
  }
  void set_try_mutex_enter() {
    _try_mutex_enter = true;
  }

  // ***************************************************************
  // interrupt support.  interrupts (using signals) are used to get
  // the thread context (fetch_top_frame), to set the thread context
  // (set_thread_pc), and to implement java.lang.Thread.interrupt.
  // this implementation is borrowed from Win32 classic vm.
  // ***************************************************************

private:

  os::Linux::Event* _interrupt_event;
  bool  _interrupted;

public:

  bool interrupted() const {
    return _interrupted;
  }
  void set_interrupted(bool n) {
    _interrupted = n;
  }

  os::Linux::Event* interrupt_event() const {
    return _interrupt_event;
  }
  void set_interrupt_event(os::Linux::Event* ptr) {
    _interrupt_event = ptr;
  }

  // ***************************************************************
  // Platform dependent initialization and cleanup
  // ***************************************************************

private:

  void pd_initialize();
  void pd_destroy();

// Reconciliation History
// osThread_solaris.hpp	1.24 99/08/27 13:11:54
// End
