/*
 * @(#)os_linux.cpp	1.62 01/04/17
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
// do not include  precompiled  header file
# include "incls/_os_linux.cpp.incl"

// put OS-includes here
# include <sys/types.h>
# include <sys/mman.h>
# include <pthread.h>
# include <signal.h>
# include <errno.h>
# include <dlfcn.h>
# include <stdio.h>
# include <unistd.h>
# include <sys/resource.h>
# include <pthread.h>
# include <sys/stat.h>
# include <sys/time.h>
# include <sys/utsname.h>
# include <sys/socket.h>
# include <pwd.h>
# include <poll.h>
# include <semaphore.h>

#define MAX_PATH    (2 * K)

// define _USE_PTHREAD_SPECIFIC in Makefile if you want to use 
// pthread_getspecific and pthread_setspecific for thread local 
// storage, even when we're running  with fixed stack pthread 
// library. DO NOT just define it in this file.

////////////////////////////////////////////////////////////////////////////////
// global variables

int  os::Linux::_processor_count = 0;

static Mutex* createThread_lock = NULL;

static pthread_t main_thread;

static int page_size = -1;

static double start_time = 0;

/* Signal number used to suspend/resume a thread */

/* do not use any signal number less than SIGSEGV, see 4355769 */
static int SR_signum = SIGUSR2;

// HACK: libpthread doesn't export which signal it is using for restart
// which grants a mutex to a thread. This selection logic is copied from
// version 2.2 of the LinuxThreads.

#ifndef __SIGRTMIN
static int thread_sig_restart = SIGUSR1;
#else
#if __SIGRTMAX - __SIGRTMIN >= 3
static int pthread_sig_restart = __SIGRTMIN;
#else
static int pthread_sig_restart = SIGUSR1;
#endif
#endif
// END HACK


////////////////////////////////////////////////////////////////////////////////
// utility functions

static inline intptr_t align(intptr_t ptr, intptr_t size) {
  size = size - 1;
  return ((ptr + size) & ~size);
}

static int SR_initialize();
static int SR_finalize();

////////////////////////////////////////////////////////////////////////////////
// environment support

bool os::getenv(const char* name, char* buf, int len) {
  const char* val = ::getenv(name);
  if (val != NULL && strlen(val) < len) {
    strcpy(buf, val);
    return true;
  }
  if (len > 0) buf[0] = 0;  // return a null string
  return false;
}

props_md_t* os::get_system_properties() {
  if (_system_properties != NULL) {
    return _system_properties;
  }

  // create properties structure and fill it
  props_md_t* props = new props_md_t();
  memset(props, '\0', sizeof(*props));

//  char arch[12];
//  sysinfo(SI_ARCHITECTURE, arch, sizeof(arch));

  // The following extra steps are taken in the debugging version:
  //
  // Starting from the binary containing this code, look for a
  // symbolic link to JAVA_HOME.  If this code is linked into
  // gamma_g, for example, look for gamma_g.JAVA_HOME/{bin,lib}....
  //
  // If that fails, obtain the JAVA_HOME from the location of
  // libjvm(_g).so. This library should be located at
  // JAVA_HOME/lib/{sparc|i386}/hotspot/libjvm.so.
  // (This step is taken in the product version also.)
  //
  // If that fails, look for the environment variable $JAVA_HOME.
  // Note that this does __not__ override the result of the previous
  // step, if successful.  (This is different from the 1.1 JDK.)
  //
  // Important note. if the location of libjvm.so changes this 
  // code needs to be changed accordingly.

  // HotSpot integration note:
  // code below matches code in JDK1.2fcs K build.
  // (Some '#ifndef PRODUCT' lines are introduced.)
  // "$JDK/src/solaris/javavm/runtime/javai_md.c":80 (ver. 1.7 98/09/15)
  // The next few definitions allow the code to be verbatim:
  props_md_t& sprops = *props;
#define GetPropertiesMD os::get_system_properties
#define NATIVE 1
#define malloc(n) (char*)NEW_C_HEAP_ARRAY(char, (n))
#define getenv(n) ::getenv(n)

#define RT_JAR         "/lib/rt.jar"
#define I18N_JAR       "/lib/i18n.jar"
#define SUNRSASIGN_JAR "/lib/sunrsasign.jar"
#define CLASSES_DIR    "/classes"
#define EXTENSIONS_DIR "/lib/ext"

  char *v; /* tmp var */
  {
    /* sysclasspath, java_home, dll_dir */
    {
        char *home_path;
	char *sysclasspath;
	char *dll_path;
        char buf[MAXPATHLEN];
#if 1 //HotSpot change
	os::jvm_path(buf, sizeof(buf));
#else //HotSpot change
	Dl_info dlinfo;
	dladdr((void *)GetPropertiesMD, &dlinfo);
	realpath((char *)dlinfo.dli_fname, buf);
#endif //HotSpot change
	*(strrchr(buf, '/')) = '\0';	/* get rid of /libjvm.so */
	*(strrchr(buf, '/')) = '\0';    /* get rid of classic    */
	dll_path = malloc(strlen(buf) + 1);
	if (dll_path == 0)
	    return NULL;
	strcpy(dll_path, buf);
	sprops.dll_dir = dll_path;

	*(strrchr(buf, '/')) = '\0';
	*(strrchr(buf, '/')) = '\0';

	home_path = malloc(strlen(buf) + 1);
	if (home_path == 0)
	    return NULL;
	strcpy(home_path, buf);
	sprops.java_home = home_path;

	// Four nulls from the #defines are enough for 3 colons + 1 null below.
	sysclasspath = malloc(strlen(home_path) * 4 + 
 			      sizeof(RT_JAR) + sizeof(I18N_JAR) +
 			      sizeof(SUNRSASIGN_JAR) + sizeof(CLASSES_DIR));
	if (sysclasspath == 0) {
          return NULL;
	}
	sprintf(sysclasspath, "%s" RT_JAR ":%s" I18N_JAR ":%s" SUNRSASIGN_JAR
		":%s" CLASSES_DIR, home_path, home_path, home_path, home_path);
	sprops.sysclasspath = sysclasspath;
    }

    /* Where to look for native libraries */
    {
	/*
	 * Get the user setting of LD_LIBRARY_PATH
	 */
	v = getenv("LD_LIBRARY_PATH");

	if (v == NULL) {
	    v = "";
	}
	
	sprops.library_path = strdup(v);
    }

    /* Extensions directories */
    {
	char * buf;
	buf = malloc(strlen(sprops.java_home) + sizeof(EXTENSIONS_DIR));
	sprintf(buf, "%s" EXTENSIONS_DIR, sprops.java_home);
	sprops.ext_dirs = buf;
    }
  }
#undef GetPropertiesMD
#undef NATIVE
#undef malloc
#undef getenv
#undef RT_JAR
#undef I18N_JAR
#undef SUNRSASIGN_JAR
#undef CLASSES_DIR
#undef EXTENSIONS_DIR

  // Done
  _system_properties = props;
  return props;
}

////////////////////////////////////////////////////////////////////////////////
// breakpoint support

void os::breakpoint() {
  BREAKPOINT;
}

extern "C" void breakpoint() {
  // use debugger to set breakpoint here
}

////////////////////////////////////////////////////////////////////////////////
// signal support

// These are signals that are unblocked while a thread is running Java.
// (For some reason, they get blocked by default.)
sigset_t* os::Linux::unblocked_signals() {
  static sigset_t sigs;
  static bool done = false;
  if (!done) {
    sigemptyset(&sigs);
    sigaddset  (&sigs, SIGINT);
    sigaddset  (&sigs, SIGCHLD);
    sigaddset  (&sigs, SIGTERM);
    done = true;                // it's OK to re-enter this
  }
  return &sigs;
}

static void hotspot_sigmask(bool main) {
  sigset_t sigset;

  sigemptyset(&sigset);
  if (!ReduceSignalUsage) {
    // Do not change the blocked/unblocked status of these signals if
    // -Xrs is specified. See signal_sets_init, os_solaris.cpp.
    sigaddset  (&sigset, SHUTDOWN1_SIGNAL);
    sigaddset  (&sigset, SHUTDOWN2_SIGNAL);
    sigaddset  (&sigset, SHUTDOWN3_SIGNAL);
    sigaddset  (&sigset, BREAK_SIGNAL);
  }
  // These are needed for correctness of the VM.
  sigaddset  (&sigset, SIGCHLD);
  sigaddset  (&sigset, SR_signum);
  pthread_sigmask(SIG_UNBLOCK, &sigset, NULL);

  if (!ReduceSignalUsage) {
    if (!main) {
      sigemptyset(&sigset);
      sigaddset  (&sigset, BREAK_SIGNAL);
      pthread_sigmask(SIG_BLOCK, &sigset, NULL);
    }
  }
}

/* pthread_getattr_np comes with LinuxThreads-0.9-7 on RedHat 7.1 */
typedef int (*pthread_getattr_func_type) (pthread_t, pthread_attr_t *);
static pthread_getattr_func_type pthread_getattr_func = NULL;
bool is_floating_stack = false;

/* Initial thread (i.e., the main thread of the process, not necessarily  */
/* JVM's main_thread) is created by kernel. Its stack is growing downwards*/
/* from 0xC0000000. The stack size shall be determined through getrlimit. */
/*                                                                        */
/* NOTE : THIS WILL NOT WORK IF KERNEL CHANGES STACK ALLOCATION BEHAVIOR !*/
/*                                                                        */
#define INITIAL_THREAD_STACK_BASE ((address)0xC0000000)
static address   initial_thread_stack_bottom = NULL;
static uintptr_t initial_thread_stack_size   = 0;

void current_stack_region(address *, size_t *);

////////////////////////////////////////////////////////////////////////////////
// thread creation support

// Thread start routine for all newly created threads

#define THREAD_SAFETY_MARGIN (50*K*K)

static void* _start(Thread* thread) {
 if (!is_floating_stack) {
  /* don't do this with floating stack, threads stacks are no longer */
  /* allocated at high memory areas.                                 */
  static uintptr_t thread_limit = NULL;

  if(thread_limit == (uintptr_t)NULL) {
      // Get the top of the "mmap area".  Use a big value to ensure we don't
      // fall in a hole.  There shouldn't be anything above this until the
      // initial stack.
      uintptr_t addr = mmap(0, THREAD_SAFETY_MARGIN, 0,
                               MAP_PRIVATE|MAP_ANONYMOUS|MAP_NORESERVE, -1, 0);

      if(addr == (uintptr_t)MAP_FAILED) {
         // As we asked it not to check available memory this should only happen when
         // we have a large maximum heap setting which has grabbed most of the VM.
         // If it does fail we don't want it printed as an internal hotspot error
         // so don't use fatal()...
         tty->print_cr("Insufficent virtual memory for thread stacks.");
         tty->print_cr("Try reducing your maximum heap size.");
         os::abort();
      }

      munmap(addr, THREAD_SAFETY_MARGIN);

      // Add a safety margin onto the address to allow for expansion while the VM
      // is running (extra shared libs, etc).
      thread_limit = addr + THREAD_SAFETY_MARGIN;
  }

  uintptr_t bottom;
  size_t size;
  current_stack_region((address *)&bottom, &size);

  // Our stack extends below the safety margin - abort the thread
  if(bottom < thread_limit) {
    {
      MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
      thread->osthread()->set_state(ZOMBIE);
      return 0;
    }
  }
 }

  // NOTE: it is safe to do this on linux.
  ThreadLocalStorage::set_thread(thread);

  OSThread* osthread = thread->osthread();

  // setup linux process id for thread
  osthread->set_thread_id(::getpid());

  int need_altstack =
    (osthread->thread_type() != os::vm_thread &&
     osthread->thread_type() != os::compiler_thread);
  
  if (need_altstack) {
    // Make sure the thread has an alternate signal stack
    os::Linux::install_alternate_signal_stack();
  }
  
  // initialize signal mask for this thread
  hotspot_sigmask(pthread_self() == main_thread);
  
  // initialize floating point control register
  os::Linux::init_thread_fpu_state();

  // notify parent thread
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    osthread->set_state(INITIALIZED);
    osthread->set_state_under_suspend(INITIALIZED);
  }

  // wait until notified.
  ((os::Linux::Event*) osthread->interrupt_event())->down();

  // call one more level start routine
  thread->run();
  
  if (need_altstack) {
    // Make sure remove the alternate signal stack
    os::Linux::remove_alternate_signal_stack();
  }
  
  ThreadLocalStorage::set_thread(NULL);

  return 0;
}

void os::initialize_thread() {
  // noop
}

// bootstrap the main thread, need extra thoughts???
bool os::create_main_thread(Thread* thread) {
  assert(main_thread == pthread_self(), "should be called inside main thread");
  return create_attached_thread(thread);
}

bool os::create_attached_thread(Thread* thread) {
  // Allocate the OSThread object
  OSThread* osthread = new OSThread(NULL, NULL);

  if (osthread == NULL) {
    return false;
  }

  // Store pthread info into the OSThread
  osthread->set_thread_id(::getpid());
  osthread->set_pthread_id(::pthread_self());

  // Make sure the thread has an alternate signal stack
  os::Linux::install_alternate_signal_stack();

  // initialize signal mask for this thread
  hotspot_sigmask(pthread_self() == main_thread);

  // initialize floating point control register
  os::Linux::init_thread_fpu_state();

  // Initial thread state is RUNNABLE
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    osthread->set_state(RUNNABLE);
    osthread->set_state_under_suspend(RUNNABLE);
  }

  thread->set_osthread(osthread);

  return true;
}

bool os::create_thread(Thread* thread, ThreadType thr_type) {
  assert(thread->osthread() == NULL, "caller responsible");

  // Allocate the OSThread object
  OSThread* osthread = new OSThread(NULL, NULL);
  if (osthread == NULL) {
    return false;
  }
  
  // set the correct thread state
  osthread->set_thread_type(thr_type);
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    osthread->set_state(ALLOCATED);
    osthread->set_state_under_suspend(ALLOCATED);
  }
  thread->set_osthread(osthread);
  
  // Create the Linux thread
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

  // Lock so that we don't create another thread while we're still waiting
  // for a thread to start.  Avoids the race condition where multiple
  // pending threads can extend below our virtual memory safety margin
  // (see _start).

  createThread_lock->lock();

  pthread_t tid;
  int ret = pthread_create(&tid, &attr, (void* (*)(void*)) _start, thread);
  
  pthread_attr_destroy(&attr);
  
  if (ret != 0) {
    if (Verbose || WizardMode) {
      perror("pthread_create()");
    }
    // Need to clean up stuff we've allocated so far
    thread->set_osthread(NULL);
    delete osthread;
    return false;
  }

  // Store pthread info into the OSThread
  osthread->set_pthread_id(tid);

  // wait until thread is either initialized or aborted
  ThreadState state;
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    state = osthread->state();
  }

  for (int i = 0; state == ALLOCATED; i++) {
    os::yield_all(i);
    {
      MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
      state = osthread->state();
    }
  }

  // Thread has started - allow another thread to be created.
  createThread_lock->unlock();

  // Aborted due to thread limit being reached
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    if (osthread->state() == ZOMBIE) {
      thread->set_osthread(NULL);
      delete osthread;
      return false;
    }
  }

  // the child thread should be ready
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    assert(osthread->state() == INITIALIZED, "race condition");
  }

  // The thread is returned suspended (in state INITIALIZED),
  // and is started higher up in the call chain
  return true;
}

// The INITIALIZED state is distinguished from the SUSPENDED state because the
// conditions in which a thread is first started are different from those in which
// a suspension is resumed.  These differences make it hard for us to apply the
// tougher checks when starting threads that we want to do when resuming them.
// However, when start_thread is called as a result of Thread.start, on a Java
// thread, the operation is synchronized on the Java Thread object.  So there
// cannot be a race to start the thread and hence for the thread to exit while
// we are working on it.  Non-Java threads that start Java threads either have
// to do so in a context in which races are impossible, or should do appropriate
// locking.

void os::start_thread(Thread* thread) {
// guards suspend/resume
  MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
  OSThread* osthread = thread->osthread();
  assert(osthread->state() == INITIALIZED || 
         osthread->state_before_suspend() == INITIALIZED, "can only start thread in INITIALIZED state");

  // A thread may be suspended in the presence of the profiler.
  // If so, it will also resume it.
  if(osthread->suspend_depth() > 0) {
    // transition into a "true" SUSPENDED state
    osthread->set_state(SUSPENDED);
    osthread->set_state_before_suspend(RUNNABLE);
    osthread->set_physically_suspended();
    return;
  }

  osthread->set_state(RUNNABLE);

  ((os::Linux::Event*) osthread->interrupt_event())->set();
}

// this may cause fatal error is some cases???
void os::exit_thread() {
  free_thread(Thread::current()->osthread());
  Thread::current()->set_osthread(NULL);

  pthread_exit(NULL);

  ShouldNotReachHere();
}
  
// Free Linux resources related to the OSThread
void os::free_thread(OSThread* osthread) {
  assert(osthread != NULL, "osthread not set");
  delete osthread;
}

////////////////////////////////////////////////////////////////////////////////
// thread local storage, stack information, alternative signal stack support

// NOTE: (on fixed stack implementation, which is outdated since RedHat 7.1)
// The standard linuxthreads uses 2MB thread stack for all threads other
// than main thread. The main thread uses 4MB to 16MB stack, depending on
// ulimit setting. Since the current linuxthreads does not support ucontext,
// alternative signal stack, or fast pthread_getspecific(). We have to do
// our own hack here.
//
// NOTE on floating stack implementation :
//
// There are some major changes between RedHat 7.0 and RedHat 7.1 with the
// new 2.4 kernel:
//
// a) Alternate signal stack works (before 7.0, alternate signal stack must
//    be allocated within the 2M stack region).
// b) pthread descriptor is stored in a dedicated register (gs on i486).
//    The original problem of slow pthread_self, pthread_getspecific should
//    disappear.
// c) Thread stack size is determined by ulimit setting and may not be
//    2M. The initial thread is created by kernel, its stack starts from
//    0xC0000000. All other threads are created by the manager thread,
//    the stacks are allocated by mmap(0,...), by default, it uses low
//    memory area first.
// d) LinuxThreads library provides pthread_getattr_np() that can be used
//    to obtain stack region of a running thread.
//
// The approach we use here are:
// a) If we are on fixed stack, we assume stack size is 2M and obtain the
//    stack bottom by (sp & ~(2M-1)); otherwise we obtain stack bottom and
//    stack size through pthread_getattr_np().
// b) Leave [bottom, bottom + HOTSPOT_ALTSTACK_START) not used. This is
//    necessary for RedHat 7.0, as glibc will put a guard page at the 2M
//    boundary. 7.0 uses fixed stack, so we will calculate stack bottom
//    below the glibc guard page.
// c) Use the next 40K for alternate signal stack.
// d) Put our own guard pages on top of alternate signal stack.
//
// Here is the diagram that shows stack region as being used on floating
// stack; with fixed stack before RedHat 7.0, there is no glibc guard page;
// with RedHat 7.0, glibc guard page (4K) is overlapped with Unused Space.
//
//      +------------------------+
//      |                        |\
//      |    Glibc Guard Page    | - only exists with LinuxThreads 0.9+
//      |                        |/
//      +------------------------+ stack low addr (stack bottom)
//      |                        |\  
//      |      Unused Space      | - (necessary for RedHat 7.0)
//      |                        |/ 
//      +------------------------+ low + HOTSPOT_ALTSTACK_START
//      |                        |\
//      | Alternate Signal Stack | - HOTSPOT_ALTSTACK_SIZE
//      |                        |/
//      +------------------------+ low + HOTSPOT_GUARD_START
//      |                        |\
//      |  HotSpot Guard Pages   | - HOTSPOT_GUARD_SIZE
//      |                        |/
//      +------------------------+ low + HOTSPOT_STACK_START
//      |                        | \
//      |      Normal Stack      |  -
//      |                        | /
//      +------------------------+ stack base = stack bottom + stack size


#ifdef HOTSPOT_STACK_SIZE
#error "please update the following code carefully"
#else
#ifndef _USE_PTHREAD_SPECIFIC
  #define HOTSPOT_THREAD_COUNT       (  2048)
  #define HOTSPOT_STACK_SHIFT        (    21)
  #define HOTSPOT_STACK_SIZE         (2048*K)
#endif

  #define HOTSPOT_ALTSTACK_START     (  16*K)
  #define HOTSPOT_ALTSTACK_SIZE      (  40*K)
  #define HOTSPOT_GUARD_START        (HOTSPOT_ALTSTACK_START+HOTSPOT_ALTSTACK_SIZE)
  #define HOTSPOT_GUARD_SIZE         (   8*K)
  #define HOTSPOT_STACK_START        (HOTSPOT_GUARD_START+HOTSPOT_GUARD_SIZE)
#endif

#ifndef _USE_PTHREAD_SPECIFIC
extern "C" void*  hotspot_thread_local_storage[HOTSPOT_THREAD_COUNT] = {0};
extern "C" int    hotspot_thread_local_storage_shift = HOTSPOT_STACK_SHIFT;
#endif

const size_t hotspot_altstack_size = HOTSPOT_ALTSTACK_SIZE;

/* HACK: LinuxThreads does not provide the function similar to thr_main   */
/*       on Solaris. We have to play the trick here. One way to tell if   */
/*       we are in the process's initial thread is to check current stack */
/*       region. The other way is to check if pthread_self() returns a    */
/*       specific thread number, which may be different among pthread     */
/*       libraries.                                                       */
static bool is_initial_thread(void) {
  char dummy;
  assert(initial_thread_stack_bottom != NULL &&
         initial_thread_stack_size   != 0, 
         "os_init did not locate initial thread's stack region");
  if (&dummy >= initial_thread_stack_bottom &&
      &dummy < initial_thread_stack_bottom + initial_thread_stack_size)
       return true;
  else return false;
}

static void current_stack_region(address * bottom, size_t * size) {
  if (is_initial_thread()) {
     *bottom = initial_thread_stack_bottom;
     *size   = initial_thread_stack_size;
  } else {
     if (is_floating_stack) {
        assert(pthread_getattr_func != NULL, "floating stack pthread must have pthread_getattr_np");
        pthread_attr_t attr;
        pthread_t tid = pthread_self();
        int rslt = pthread_getattr_func(tid, &attr);
        if (rslt!=0) {
           /* what else can we do?? */
           fatal("Can not locate current stack region!");
        }

        void * base;
        pthread_attr_getstackaddr(&attr, &base);
        pthread_attr_getstacksize(&attr, size);
        *bottom = (address)align((uintptr_t)base - *size, page_size);
        *size   = (address)base - *bottom;
     } else {
        // fixed stack implementation, thread stacks are 2M in size
        // and aligned at 2M boundary.
        char dummy;
        *size = 2 * K * K;
        *bottom = (address)(((uintptr_t)&dummy) & ~(2 * K * K - 1));
     }
  }
}

int os::Linux::check_stack_overflow(address addr) {
  // calculate current stack location
  uintptr_t bottom;
  size_t    size;
  current_stack_region((address *)&bottom, &size);

  // use uintptr_t for fancy arithmatic operation
  uintptr_t a = (uintptr_t) addr;
  // check fault address against current stack
  if (a > bottom && a <= bottom + size) {
    // stack overflow case
    if (a >= bottom + HOTSPOT_STACK_START) {
      // the stack banging code touch the current stack limit.
      // enfore stack expanding manually. (Dangerous!!!)
      *addr = 0;
      return 2;
    } else if (a > bottom + HOTSPOT_GUARD_START) {
      // we have enough space left
      a = (a & ~(page_size - 1));
      if (::mprotect(a, page_size, PROT_READ|PROT_WRITE|PROT_EXEC)) {
        fatal("can not allocate space to cover stack overflow");
      }
      // we released the guard page.
      return 1;
    }
    // we really run of space
    return -1;
  }
  // null pointer case
  return 0;
}

void os::Linux::install_alternate_signal_stack() {
  stack_t stack;

/*
  HACK: this following code does not work on linux!!!
    The child thread inherits signal stack from parent.

  if (::sigaltstack(NULL, &stack) != 0) {
    warning("sigaltstack() failed (check linux version)");
    return;
  }

  if ((stack.ss_flags & SS_DISABLE) == 0) {
    warning("sigaltstack already installed");
    return;
  }
*/

  uintptr_t bottom;
  size_t    size;
  current_stack_region((address *)&bottom, &size);

  /* check if we have enough space to set up guard page and alternate */
  /* signal stack.                                                    */
  const uintptr_t safety_margin = 16 * K;
  uintptr_t current_sp = (uintptr_t)os::current_stack_pointer();
  if (bottom + HOTSPOT_STACK_START + safety_margin > current_sp) {
     /* the thread is created or attached with stack size too small. */
     /* either way, we don't want to print this as a hotspot internal*/
     /* error. so don't fatal(...).                                  */
     tty->print_cr("Fatal: Stack size too small. Use 'ulimit -s' to "
                   "increase default stack size.");
     os::abort();
  }

  /*
  /* setup guard page */
  /*
  NOTE on glibc guard page:
    Since glibc 2.1, LinuxThreads will put a one-page-size guard page 
    at the end of thread stack. However, due to a bug in LinuxThreads 
    prior to version 0.9, guard page will NEVER be allocated. This bug 
    appears to be fixed in RedHat 7.0.
    Still, main thread stack is NOT protected; pthread with user stack
    is NOT protected.
    Once we have stack bottom in hand, it's most convenient to manage
    guard page ourselves.
  */
  uintptr_t start = bottom + HOTSPOT_GUARD_START;

  /*
  NOTE: up until LinuxThreads 0.8, thread stacks are mmapped with
    initial size of 4K and grows downwards when necessary. Since
    LinuxThreads 0.9, thread stacks are mmapped with full stack
    size once the thread is created. commit_memory() is not necessary
    for system with LinuxThreads 0.9, but it's harmless.
  */
  if (!os::commit_memory((char*) start, HOTSPOT_GUARD_SIZE)) {
    warning("can not allocate stack guard pages to prevent stack overflow");
    return;
  }

  if (::mprotect((void*) start, HOTSPOT_GUARD_SIZE, PROT_NONE)) {
    warning("can not guard stack space to prevent stack overflow");
    return;
  }

  /* setup alternate signal stack */
  /*
  NOTE: prior to LinuxThreads 0.9 (as shipped with RedHat 7.1),
        stack size is fixed to be 2M. Alternate signal stack
        MUST be installed within the 2M memory slot. If the
        alternate signal stack is installed outside the 2M thread
        stack region, system will hang upon signals!!
        This bug appears to have been fixed in LinuxThreads 0.9,
        we may use mmap(0,...) to allocate alternate signal
        stack in the future.
  */
  uintptr_t astart = bottom + HOTSPOT_ALTSTACK_START;

  if (!os::commit_memory((char*) astart, HOTSPOT_ALTSTACK_SIZE)) {
    warning("can not allocate alternative signal stack");
    return;
  }

  stack.ss_sp    = astart;
  stack.ss_size  = HOTSPOT_ALTSTACK_SIZE;
  stack.ss_flags = 0;

  if (::sigaltstack(&stack, NULL)) {
    warning("Cannot install alternative signal stack");
  }
}

void os::Linux::remove_alternate_signal_stack() {
  stack_t old_stack;

  ::sigaltstack(NULL, &old_stack);
  if ((old_stack.ss_flags & SS_DISABLE) == 0) {
    stack_t new_stack;

    new_stack.ss_sp    = NULL;
    new_stack.ss_size  = 0;
    new_stack.ss_flags = SS_DISABLE;

    if (sigaltstack(&new_stack, NULL) != 0) {
      warning("cannot uninstall alt signal stack");
    }
    // TODO: shall we munmap the guard page and altstack.
    // I am not sure the linuxthreads will do it for us.
  }
}

address os::current_stack_pointer() {
  char dummy;
  return (address) &dummy;
}

address os::current_stack_base() {
  address bottom;
  size_t size;
  current_stack_region(&bottom, &size);
  return (bottom + size);
}

// this does not include the stuff we put at the end of thread stack
// e.g. alternate signal stack, guard pages and unused space.
int os::current_stack_size() {
  address bottom;
  size_t size;
  current_stack_region(&bottom, &size);
  return (size - HOTSPOT_STACK_START);
}

int os::allocate_thread_local_storage() {
#ifndef _USE_PTHREAD_SPECIFIC
  if (is_floating_stack) {
#endif
    pthread_key_t key;
    int rslt = pthread_key_create(&key, NULL);
    assert(rslt == 0, "cannot allocate thread local storage");
    return (int)key;
#ifndef _USE_PTHREAD_SPECIFIC
  } else {
    assert(sizeof(void*) == 4, "need 64-bit update");
    return 0;
  }
#endif
}

void os::free_thread_local_storage(int index) {
#ifndef _USE_PTHREAD_SPECIFIC
  if (is_floating_stack) {
#endif
    int rslt = pthread_key_delete((pthread_key_t)index);
    assert(rslt == 0, "invalid index");
#ifndef _USE_PTHREAD_SPECIFIC
  } else {
    assert(index == 0, "invalid index");
  }
#endif
}

void os::thread_local_storage_at_put(int index, void* value) {
#ifndef _USE_PTHREAD_SPECIFIC
  if (is_floating_stack) {
#endif
    int rslt = pthread_setspecific((pthread_key_t)index, value);
    assert(rslt == 0, "pthread_setspecific failed");
#ifndef _USE_PTHREAD_SPECIFIC
  } else {
    assert(index == 0, "invalid index");
    uintptr_t esp = (uintptr_t) os::current_stack_pointer();
    uintptr_t idx = esp / ((uintptr_t) HOTSPOT_STACK_SIZE);
    hotspot_thread_local_storage[(int) idx] = value;
  }
#endif
}

void* os::thread_local_storage_at(int index) {
#ifndef _USE_PTHREAD_SPECIFIC 
  if (is_floating_stack) {
#endif
    return pthread_getspecific((pthread_key_t)index);
#ifndef _USE_PTHREAD_SPECIFIC
  } else { 
    assert(index == 0, "invalid index");
    uintptr_t esp = (uintptr_t) os::current_stack_pointer();
    uintptr_t idx = esp / ((uintptr_t) HOTSPOT_STACK_SIZE);
    return hotspot_thread_local_storage[(int) idx];
  }
#endif
}

Thread* ThreadLocalStorage::thread() {
  return (Thread *) os::thread_local_storage_at(thread_index());
}

extern "C" Thread* get_thread() {
  return ThreadLocalStorage::thread();
}

// ISM not avaliable for linux
char* os::reserve_memory_special(size_t bytes) {
  fatal("ISM is not supported for this operating system.");
  return NULL;
}

////////////////////////////////////////////////////////////////////////////////
// time support

// Time since start-up in seconds to a fine granularity.
// Used by VMSelfDestructTimer and the MemProfiler.
double os::elapsedTime() {
  double time = (double) os::elapsed_counter();
  time = (time - start_time) * 0.000001;
  return time; 
}

jlong os::elapsed_counter() {
  timeval time;
  int status = gettimeofday(&time, NULL);
  return jlong(time.tv_sec) * K * K + jlong(time.tv_usec);
}

jlong os::elapsed_frequency() {
  return (K * K);
}

jlong os::javaTimeMillis() {
  timeval time;
  int status = gettimeofday(&time, NULL);
  assert(status != -1, "linux error");
  return jlong(time.tv_sec) * 1000  +  jlong(time.tv_usec / 1000);
}

////////////////////////////////////////////////////////////////////////////////
// runtime exit support

// On most platforms, we have a problem: threads (including
// daemon threads) continue to run during exit, causing all sorts of
// bad results.  Jump through some hoops to try to avoid crashing.

class VM_Exit: public VM_Operation {
 private:
  int     _code;
  Thread* _thread;
 public:
  VM_Exit(int code, Thread* thread) {
    _code   = code; 
    _thread = thread;
  }  
  void doit() {
    // make doubly sure that a thread doesn't wake up at an inconvenient time
    // Note: this is necessary or the threads continue to run
    assert(Threads_lock->owner() == Thread::current(), "must have threads lock");
    for (JavaThread *thr = Threads::first(); thr != NULL; thr = thr->next()) {
      OSThread *osthread = ((Thread *)thr)->osthread();
      // Don't suspend a thread that is blocked on trying to enter a
      // mutex. The SR_handler() will block all signals except for
      // resume and mutex grant and the call to pthread_exit_process()
      // via exit() will hang.
      if (!osthread->is_try_mutex_enter()) {
        thr->shutdown_suspend();
      }
    }

    // Check for exit hook
    exit_hook_t exit_hook = Arguments::exit_hook();
    if (exit_hook != NULL) {
      exit_hook(_code);
    } else {
      ::exit(_code);
    }
  }
  char* name() const {
    return "exit";
  }
};

void os::exit(int code) {  
  Thread* thread = ThreadLocalStorage::get_thread_slow();

  if (thread == NULL) {
    // we have serious problems -- just exit
    ::exit(code);
  }

  // Fire off a VM_Operation to suspend all remaining threads
  if (VMThread::vm_thread() != NULL) {
    VM_Exit op(code, thread);
    // %%%%% integrate this with vm_exit !
    if (thread->is_Java_thread()) {
      ((JavaThread*)thread)->set_thread_state(_thread_in_vm);
    }
    VMThread::execute(&op);
  } else {
    ::exit(code);
  }
}

void os::abort(bool dump_core) {
  // Check for abort hook
  abort_hook_t abort_hook = Arguments::abort_hook();
  if (abort_hook != NULL) {
    abort_hook();
  }
  if (dump_core) {
    NOT_PRODUCT(tty->print("Current thread is %p\n", pthread_self()));
    NOT_PRODUCT(tty->print("Dumping core ...\n"));
    ::abort(); // dump core
  }
  ::exit(1);
}

int os::current_process_id()
{
  return getpid();
}

// DLL functions

const char* os::dll_file_extension() { return ".so"; }

const char* os::get_temp_directory() { return "/tmp/"; }

// Returns true if the given PC was in the VM DLL; otherwise returns false.
bool os::dll_address_is_in_vm(address addr) {
  // These tests should be unified with the code in find(int, bool) in
  // debug.cpp.

  // Check generated code
  if (AbstractInterpreter::contains(addr)) {
    return true;
  }

  if (StubRoutines::contains(addr)) {
    return true;
  }

#ifndef CORE
  if (CodeCache::contains(addr)) {
    return true;
  }

  if (VtableStubs::contains(addr)) {
    return true;
  }
#endif

  // Check if inside libjvm
  Dl_info dlinfo;
  if (dladdr(CAST_FROM_FN_PTR(void *, os::dll_address_is_in_vm), &dlinfo)){
      void * fbase = dlinfo.dli_fbase;
      assert(fbase !=NULL, "Cannot obtain base address for libjvm");
      if (dladdr((void *)addr, &dlinfo)){
          if (fbase == dlinfo.dli_fbase) return true;
      }
  }
  else assert(NULL != NULL, "Cannot obtain base address for libjvm");

  return false;
}

// Returns the symbol nearest to the specified addres or NULL if it's
// is outside all known mapped object. Used by profiler
const char* os::dll_address_to_function_name(address addr){
  Dl_info dlinfo;
  const char* symbol = (char *)NULL;

  if (dladdr((void*)addr, &dlinfo)) {
    symbol = dlinfo.dli_sname;
  }

  return symbol;
}

// Returns the name of the library that contains the specified addres
// or NULL if it's outside all known mapped object.
const char* os::dll_address_to_library_name(address addr){
  Dl_info dlinfo;
  const char* lib = (char *)NULL;

  if (dladdr((void*)addr, &dlinfo)){
     lib = dlinfo.dli_fname;
  }
  return lib;
}

// Maps PC to line number and source file name. Not implemented for
// Linux
int os::dll_address_to_line_number(address addr, char ** filename) {
  if (filename) *filename = NULL;
  return -1;
}

/*
 * splits a string _str_ into substrings with delimiter _delim_ by replacing old
 * delimiters with _new_delim_ (ideally, '\0'). the address of each substring
 * is stored in array _ptrs_ as the return value. the maximum capacity of _ptrs_
 * array is specified by parameter _n_.
 * RETURN VALUE: total number of substrings (always <= _n_)
 * NOTE: string _str_ is modified if _delim_!=_new_delim_
 */
static int split_n_str(char * str, int n, char ** ptrs, char delim, char new_delim)
{
   int i;
   for(i = 0; i < n; i++) ptrs[i] = NULL;
   if (str == NULL || n < 1 ) return 0;

   i = 0;

   // skipping leading blanks
   while(*str&&*str==delim) str++;

   while(*str&&i<n){
     ptrs[i++] = str;
     while(*str&&*str!=delim) str++;
     while(*str&&*str==delim) *(str++) = new_delim;
   }

   return i;
}

/*
 * fgets without storing '\n' at the end of the string
 */
static char * fgets_no_cr(char * buf, int n, FILE *fp)
{
   char * rslt = fgets(buf, n, fp);
   if (rslt && buf && *buf){
       char *p = strchr(buf, '\0');
       if (*--p=='\n') *p='\0';
   }
   return rslt;
}

/*
 * search string _str_ in a string list _strlist_. _n_ is the size of the list
 * RETURN VALUE: index in the list if found; otherwise -1.
 */
static int find_string(char ** strlist, int n, char * str)
{
   int i;
   for(i=0; i<n; i++)
       if (strcmp(strlist[i], str)==0)
           return i;
   return -1;
}


#define PRINT_PROC_MAP    /* do not attempt to analyze /proc/<pid>/maps, directly print the */
                          /* file to st. it's desirable for fatal error handling, as it's   */
                          /* less error-prone and actually can provide more information     */
                          /* for seasoned debuggers.                                        */
void os::dump_dll_info(outputStream *st) {
       pid_t pid = getpid();

       char fname[32];
       char buf[256];
       FILE *fp = NULL;

#ifndef PRINT_PROC_MAP
#      define MAX_HISTORY_SIZE 32
       char *history[MAX_HISTORY_SIZE];
       int  history_size = 0;
       char *execfile = "";
#endif

       st->print_cr("Dynamic libraries:");

       sprintf(fname, "/proc/%d/maps", pid);
       fp = fopen(fname, "r");

       if (fp == NULL){
           st->print("Can not get information for pid = %d\n", pid);
           return;
       }

       while(fgets_no_cr(buf, 256, fp)){
#ifndef PRINT_PROC_MAP
           char * word[6];
           int nwords = split_n_str(buf, 6, word, ' ', '\0');
#          define EXE_LOC "08048000-"
           if (strncmp(buf, EXE_LOC, strlen(EXE_LOC)) == 0){
                execfile = strdup(word[5]);
           }
           else if (nwords > 5 && strcmp(word[5], execfile)!=0
                    && find_string(history, history_size, word[5])<0){
                st->print("lib %s\n", word[5]); st->flush();
                if (history_size<MAX_HISTORY_SIZE-1)
                    history[history_size++] = strdup(word[5]);
           }
#else
           if (strlen(buf)>42){
             /* skip not so useful non-dll blocks */
              st->print_cr(buf);  st->flush();
           }
#endif
       }
       fclose(fp);
#undef EXE_LOC
#undef MAX_HISTORY_SIZE
}

static char saved_jvm_path[MAXPATHLEN] = {0};

// Find the full path to the current module, libjvm.so or libjvm_g.so
void os::jvm_path(char *buf, jint len) {
  // Error checking.
  if (len < MAXPATHLEN) {
    assert(false, "must use a large-enough buffer");
    buf[0] = '\0';
    return;
  }
  // Lazy resolve the path to current module.
  if (saved_jvm_path[0] == 0) {
    Dl_info dlinfo;
    int ret = dladdr(CAST_FROM_FN_PTR(void *, os::jvm_path), &dlinfo);
    if (ret == 0) {
      tty->print_cr("Can not locate current module (%s)!!!", dlerror());
    } else {
      ::realpath(dlinfo.dli_fname, saved_jvm_path);
    }
  }
  // Copy into caller provided buffer.
  strcpy(buf, saved_jvm_path);
}
 
void os::print_jni_name_prefix_on(outputStream* st, int args_size) {
  // no prefix required, not even "_"
}

void os::print_jni_name_suffix_on(outputStream* st, int args_size) {
  // no suffix required
}

////////////////////////////////////////////////////////////////////////////////
// sun.misc.Signal support

static void
UserHandler(int sig, void *siginfo, void *context) {
  os::signal_notify(sig);
}

void* os::user_handler() {
  return CAST_FROM_FN_PTR(void*, UserHandler);
}

extern "C" {
  typedef void (*sa_handler_t)(int);
  typedef void (*sa_sigaction_t)(int, siginfo_t *, void *);
}

void* os::signal(int signal_number, void* handler) {
  struct sigaction sigAct, oldSigAct;

  sigfillset(&(sigAct.sa_mask));
  sigAct.sa_flags   = SA_RESTART|SA_SIGINFO;
  sigAct.sa_handler = CAST_TO_FN_PTR(sa_handler_t, handler);

  if (sigaction(signal_number, &sigAct, &oldSigAct)) {
    fatal2("sigaction: cannot set signal %d (%s)", signal_number, strerror(errno));
  }

  return CAST_FROM_FN_PTR(void*, oldSigAct.sa_handler);
}

void os::signal_raise(int signal_number) {
  ::raise(signal_number);
}

/*
 * The following code is moved from os.cpp for making this
 * code platform specific, which it is by its very nature.
 */

// a counter for each possible signal value
static jint pending_signals[NSIG+1] = { 0 };

// Linux(POSIX) specific hand shaking semaphore.
static sem_t sig_sem;

void os::signal_init_pd() {
  // Initialize signal structures
  ::memset(pending_signals, 0, sizeof(pending_signals));

  // Initialize signal semaphore
  ::sem_init(&sig_sem, 0, 0);
}

void os::signal_notify(int sig) {
  atomic::increment(&pending_signals[sig]);
  ::sem_post(&sig_sem);
}  

static int check_pending_signals(bool wait) {
  for (;;) {
    for (int i = 0; i < NSIG + 1; i++) {
      jint n = pending_signals[i];
      if (n > 0 && n == atomic::compare_and_exchange(&pending_signals[i], n - 1, n)) {
        return i;
      }
    }
    if (!wait) {
      return -1;
    }
    ThreadBlockInVM tib(JavaThread::current());
    ::sem_wait(&sig_sem);
  }
}

int os::signal_lookup() {
  return check_pending_signals(false);
}

int os::signal_wait() {
  return check_pending_signals(true);
}

////////////////////////////////////////////////////////////////////////////////
// Virtual Memory


int os::vm_page_size() {
  assert(page_size != -1, "must call os::init");
  return page_size;
}

/* NOTE: Linux kernel does not really reserve the pages for us. */
/*       All it does is to check if there are enough free pages */
/*       left at the time of mmap(). This could be a potential  */
/*       problem.                                               */
bool os::commit_memory(char* addr, size_t size) {
  return ::mmap(addr, size, 
                PROT_READ|PROT_WRITE|PROT_EXEC,
                MAP_PRIVATE|MAP_FIXED|MAP_ANONYMOUS, -1, 0)
    != MAP_FAILED;
}

bool os::uncommit_memory(char* addr, size_t size) {
  return ::mmap(addr, size,
                PROT_READ|PROT_WRITE|PROT_EXEC,
                MAP_PRIVATE|MAP_FIXED|MAP_NORESERVE|MAP_ANONYMOUS, -1, 0)
    != MAP_FAILED;
}

bool os::release_memory(char* addr, size_t size) {
  int ret = ::munmap(addr, size);
  return ret == 0;
}

bool os::guard_memory(char* addr, size_t size) {
  // int ret = ::mprotect(addr, size, PROT_NONE);
  // HACK: need to be fixed
  return true;
}

bool os::unguard_memory(char* addr, size_t size) {
  // int ret = ::mprotect(addr, size, PROT_READ|PROT_WRITE|PROT_EXEC);
  // HACK: need to be fixed
  return true;
}
/*
static int os_sleep(jlong millis) {
  const jlong limit = INT_MAX;
  while (millis > limit) {
    if (os_sleep(limit) == -1) {
      return -1;
    }
    millis -= limit;
  }

  struct timeval tv;
  tv.tv_sec  = millis / 1000;
  tv.tv_usec = (millis % 1000) * 1000;
  return select(0, NULL, NULL, NULL, &tv);
}
*/
int os::sleep(Thread* thread, jlong millis, bool interruptible) {
  assert(thread == Thread::current(),  "thread consistency check");

  OSThread* osthread = thread->osthread();
  if (interruptible) {
    if (os::is_interrupted(thread, true)) {
      return OS_INTRPT;
    }
    {
      ThreadBlockInVM tb((JavaThread*) thread);
      OSThreadWaitState osts(thread->osthread());
      Linux::Event* event = (Linux::Event*) osthread->interrupt_event();
      event->down(millis);
    }
    if (os::is_interrupted(thread, true)) {
      return OS_INTRPT;
    }
  } else {
    OSThreadWaitState osts(thread->osthread());
    Linux::Event* event = (Linux::Event*) osthread->interrupt_event();
    event->down(millis);
  }

  return OS_TIMEOUT;
}

// Used to convert frequent JVM_Yield() to nops
bool os::dont_yield() {
  return true;
}

void os::yield() {
  sched_yield();
}

void os::yield_all(int attempts) {
  // Yields to all threads, including threads with lower priorities
  if (attempts < 10) {
    ::sched_yield();
  } else {
    struct timespec req;
    struct timespec rem;
    req.tv_sec = 0;
    req.tv_nsec = 1000000;
    ::nanosleep(&req, &rem);
  }
}

// Called from the tight loops to possibly influence time-sharing heuristics
void os::loop_breaker(int attempts) {
  os::yield_all(attempts);
}

////////////////////////////////////////////////////////////////////////////////
// thread priority support

// TODO: the entire priority issue needs to be overhauled.

#define T_PRI_MIN   1
#define T_PRI_NORM  1
#define T_PRI_MAX   1

int os::java_to_os_priority[MaxPriority + 1] = {
  0,              // 0 Entry should never be used

  1,              // 1 MinPriority
  2,              // 2
  3,              // 3

  4,              // 4
  5,              // 5 NormPriority
  6,              // 6

  7,              // 7
  8,              // 8
  9,              // 9 NearMaxPriority

  10              // 10 MaxPriority
};

OSReturn os::set_native_priority(Thread* thread, int newpri) {
  pthread_t thr = thread->osthread()->pthread_id();
  int policy = SCHED_OTHER;
  struct sched_param param;
  param.sched_priority = newpri;
  int ret = pthread_getschedparam(thr, &policy, &param);
  return (ret == 0) ? OS_OK : OS_ERR;
}

OSReturn os::get_native_priority(Thread* thread, int *priority_ptr) {
  pthread_t thr = thread->osthread()->pthread_id();
  int policy = SCHED_OTHER;
  struct sched_param param;
  int ret = pthread_getschedparam(thr, &policy, &param);
  if (param.sched_priority < MinPriority) {
    *priority_ptr = MinPriority;
  } else if (param.sched_priority > MaxPriority) {
    *priority_ptr = MaxPriority;
  } else {
    *priority_ptr = param.sched_priority;
  }
  return (ret == 0) ? OS_OK : OS_ERR;
}

// Priority boosting/inheritence/inversion needed by safepoint code
bool os::boost_priority(Thread *thread, int max_java_priority) {
  return true;
}

bool os::deflate_priority(Thread *thread, int min_java_priority) {
  return true;
}

////////////////////////////////////////////////////////////////////////////////
// suspend/resume/interrupt support

static void resume_clear_context(OSThread *osthread) {
      osthread->set_ucontext(0);
      osthread->set_siginfo(0);

      // notify the suspend action is completed, we have now resumed
      osthread->set_suspend_action(SR_NONE);
}

static void suspend_save_context(OSThread *osthread, siginfo_t* siginfo, ucontext_t* context) {
     osthread->set_ucontext(context);
     osthread->set_siginfo(siginfo);
}

//
// Handler function invoked when a thread's execution is suspended or
// resumed. We have to be careful that only async-safe functions are
// called here. When this handler is invoked, SR_signum (suspend/resume)
// and pthread_sig_restart (mutex grant) are blocked.
//
// Note: sigwait() is a more natural fit than sigsuspend() from an
// interface point of view, but sigwait() prevents the signal hander
// from being run. libpthread would get very confused by not having
// its signal handlers run and prevents sigwait()'s use with the
// mutex granting granting signal.
//
static void SR_handler(int sig, siginfo_t* siginfo, ucontext_t* context) {
  Thread* thread = Thread::current();
  OSThread* osthread = thread->osthread();

  // read current suspend action
  int action = osthread->suspend_action();
  if (action == SR_SUSPEND || action == SR_SUSPEND_SELF) {

    suspend_save_context(osthread, siginfo, context);

    // Notify the suspend action is about to be completed. do_suspend()
    // waits until SR_SUSPENDED is set and then returns. In the
    // self-suspension case, do_suspend() will return and its caller
    // will do an Interrupt_lock.wait() and that completes the
    // self-suspend action. In the suspend-other case, we will wait
    // here for a resume signal and that completes the suspend-other
    // action. do_suspend() will return to eventually release the
    // Interrupt_lock and return to the caller. At that point it is
    // possible for a racing resume request to get the Interrupt_lock
    // and send a resume signal before we call sigsuspend() below.
    // Fortunately, the resume signal is blocked until we call
    // sigsuspend().
    //
    // Another interesting race is that we might have been granted a
    // mutex between the time SR_handler() is entered due to a suspend
    // signal and the time we call sigsuspend() below. Fortunately,
    // the mutex grant signal is blocked until we call sigsuspend().

    osthread->set_suspend_action(SR_SUSPENDED);

    if (action == SR_SUSPEND_SELF) {
      // self-suspension admin is done and can now do Interrupt_lock.wait()
      return;
    }

    sigset_t suspend_set;  // signals for sigsuspend()

    if (osthread->is_try_mutex_enter()) {
      // Block all signals except for resume and mutex grant. We can
      // identify a resume signal by looking at the suspend_action
      // field. We have no way to positively identify the mutex granting
      // signal so we rely on the process of elimination, i.e., if it is
      // not a resume signal, then it must be a mutex granting signal.
      // This strict handling of signals prevents other signal handlers
      // from running while we are in sigsuspend(). This complicates
      // VM_Exit()::doit().
      sigfillset(&suspend_set);
      sigdelset(&suspend_set, SR_signum);
      sigdelset(&suspend_set, pthread_sig_restart);

      sigsuspend(&suspend_set);

      // When we return from sigsuspend(), then we got a resume signal
      // and/or a mutex granting signal. If we got a resume signal, then
      // suspend_action has been set to SR_CONTINUE by the resumer thread
      // and to SR_NONE by the resume signal's call to SR_handler(). We
      // need Interrupt_lock to protect our query of the suspend_action
      // field. The Interrupt_lock is held by the resumer thread until
      // sometime after SR_NONE is set. It is possible for a resume
      // operation to still be holding the Interrupt_lock when we get
      // here, but it will drop the lock soon so retrying is okay.
      //
      // If we got a mutex granting signal, then we need Interrupt_lock
      // to protect our query and set of the suspend_action field. It is
      // possible for an unrelated thread to be temporarily holding
      // Interrupt_lock, but that grab is transient so retrying is okay.
      //
      // If only one signal brought us out of sigsuspend(), then it is
      // possible to receive the other signal as pending before we have
      // managed to grab Interrupt_lock. In the rare cases of:
      //
      // resume then mutex grant)
      //   We don't need to call sigsuspend() for the pending mutex grant
      //   signal. libpthread will take care of that when we return from
      //   the handler. We can safely retry the Interrupt_lock grab.
      // mutex grant then resume)
      //   We call sigsuspend() again to handle the resume signal. The
      //   resulting call to SR_handler() will allow the resumer thread
      //   to finish and we can safely retry the Interrupt_lock grab.

      sigset_t pending_set;  // signals for sigpending()
      int      retries = 0;
      while (!Interrupt_lock->try_lock()) {
        if (sigpending(&pending_set) == 0
        && sigismember(&pending_set, SR_signum)) {
          // resume signal came in so deal with it
          sigsuspend(&suspend_set);
        }
        // always yield to give the Interrupt_lock owner a chance to run
        os::yield_all(retries++);
      }

      bool mutex_granted = false;
      if (osthread->suspend_action() == SR_SUSPENDED) {
        // We are still suspended so a mutex granting signal caused
        // us to return from the first sigsuspend() call above.
        mutex_granted = true;
      }

      resume_clear_context(osthread);

      if (mutex_granted) {
        // We are still suspended, but we were granted a mutex
        // (libpthread doesn't know about suspension). We need to
        // partially resume so we can unlock the mutex and let someone
        // else have it. We restore our "suspended" state so that
        // ObjectMonitor::raw_enter() knows what is happening.
        osthread->set_suspend_action(SR_SUSPENDED);
      }

      Interrupt_lock->unlock();
    } else {
      // We are doing a plain and simple suspend so we will wait here
      // until we are resumed.

      // get current set of blocked signals and unblock resume signal
      pthread_sigmask(SIG_BLOCK, NULL, &suspend_set);
      sigdelset(&suspend_set, SR_signum);

      do {
        sigsuspend(&suspend_set);
        // ignore all returns until we get a resume signal
      } while (osthread->suspend_action() != SR_CONTINUE);

      resume_clear_context(osthread);
    }
  } else if (action == SR_CONTINUE && osthread->is_try_mutex_enter()) {
    // Normally, we receive a resume signal in the third sigsuspend()
    // call above and the SR_handler() call resulting from the resume
    // signal does no work because is_try_mutex_enter() is false.
    //
    // However, if we were suspended while trying to enter a mutex,
    // then we have work to do in three cases:
    // 1) If we were resumed while we were in the first sigsuspend()
    //    call above, the SR_handler() call for the resume signal
    //    results in marking the thread as resumed. That sigsuspend()
    //    call is typically where we are resumed if we are still trying
    //    to enter the mutex.
    // 2) If we were resumed while we were in the second sigsuspend()
    //    call above, the SR_handler() call for the resume signal
    //    results in marking the thread as resumed. That sigsuspend()
    //    call is where we are resumed if there was a race between the
    //    resume signal and the mutex granting signal.
    // 3) If we were granted the mutex while suspended and we didn't
    //    get resumed before we left the handler, then we returned
    //    back to ObjectMonitor::raw_enter() to unlock the mutex.
    //    When the subsequent resume signal comes in, we catch it
    //    here and mark the thread as resumed.
    osthread->set_suspend_action(SR_NONE);
  }
}


static int SR_initialize() {
  struct sigaction act;
  char *s;
  /* Get signal number to use for suspend/resume */
  if ((s = ::getenv("_JAVA_SR_SIGNUM")) != 0) {
    int sig = ::strtol(s, 0, 10);
    if (sig > 0 || sig < _NSIG) {
	SR_signum = sig;
    }
  }

  assert(SR_signum > SIGSEGV && SR_signum > SIGBUS,
        "SR_signum must be greater than max(SIGSEGV, SIGBUS), see 4355769");

  /* Set up signal handler for suspend/resume */
  act.sa_flags = SA_RESTART|SA_SIGINFO;
  act.sa_handler = (void (*)(int)) SR_handler;
  // SR_signum is blocked by default. Also block mutex grant signal.
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask, pthread_sig_restart);
  // HACK: use __sigaction() to bypass linuxthreads because
  // it does not support siginfo and ucontext at this moment.
  if (__sigaction(SR_signum, &act, 0) == -1) {
    return -1;
  }
  return 0;
}

static int SR_finalize() {
  return 0;
}


long do_suspend(OSThread* osthread, int action) {
  long ret;

  // set suspend action and send signal
  osthread->set_suspend_action(action);

  ret = pthread_kill(osthread->pthread_id(), SR_signum);
  // check return code and wait for notification

  if (ret == 0) {
    for (int i = 0; osthread->suspend_action() != SR_SUSPENDED; i++) {
      os::yield_all(i);
    }
  }
  return ret;
}

// Suspend another thread by one level. The VM code tracks suspend
// nesting and handles self-suspension via wait on Interrupt_lock.
// Returns zero on success.
// Solaris found that non-compiler threads also needed ThreadCritical
// May also need to free malloc, etc. locks here. fence currently always true.
long os::pd_suspend_thread(Thread* thread, bool fence) {
    long ret;
    OSThread *osthread = thread->osthread();

    if (fence) {
      ThreadCritical tc;
      ret = do_suspend(osthread, SR_SUSPEND);
    } else {
      ret = do_suspend(osthread, SR_SUSPEND);
    }
    return ret;
}
   
// Resume a thread by one level.  This method assumes that consecutive
// suspends nest and require matching resumes to fully resume.  Note that
// this is different from Java's Thread.resume, which always resumes any
// number of nested suspensions.  The ability to nest suspensions is used
// by other facilities like safe points.  
// Resuming a thread that is not suspended is a no-op.
// Returns zero on success.

long os::pd_resume_thread(Thread* thread) {
  OSThread *osthread = thread->osthread();
  long ret;
  int i;

  // higher layers should catch resume of a thread that is not suspended
  assert(osthread->suspend_action() == SR_SUSPENDED, "thread should be suspended");
  if (osthread->suspend_action() != SR_SUSPENDED) {
    return -1;  // robustness
  }

  osthread->set_suspend_action(SR_CONTINUE);

  ret = pthread_kill(osthread->pthread_id(), SR_signum);

  if (ret == 0) {
    for (i = 0; osthread->suspend_action() != SR_NONE; i++) {
      os::yield_all(i);
    }
  }
  return ret;
}


// Part I of self-suspension. 
// Part I of self-suspension sends a signal to ourselves
// so the SR_handler will save our ucontext and siginfo. 
// Then return to the caller who will do a wait on the Interrupt_Lock
// as the common self-suspension implementation.
// Part III: after resume, clear saved ucontext and siginfo
bool os::pd_pre_self_suspend(OSThread* osthread) {
   do_suspend(osthread, SR_SUSPEND_SELF);
  
   return true;
}

// Part III of self-suspension. 
// clear saved ucontext and siginfo
void os::pd_post_self_suspend(OSThread* osthread) {
   resume_clear_context(osthread);
}


void os::interrupt(Thread* thread) {
  assert(Thread::current() == thread || Threads_lock->owned_by_self(),
    "possibility of dangling Thread pointer");

  OSThread* osthread = thread->osthread();

  if (!osthread->interrupted()) {
    Linux::Event* event = (Linux::Event*) osthread->interrupt_event();
    osthread->set_interrupted(true);
    event->set();
  }
}

bool os::is_interrupted(Thread* thread, bool clear_interrupted) {
  assert(Thread::current() == thread || Threads_lock->owned_by_self(),
    "possibility of dangling Thread pointer");

  OSThread* osthread = thread->osthread();

  bool interrupted = osthread->interrupted();

  if (interrupted && clear_interrupted) {
    Linux::Event* event = (Linux::Event*) osthread->interrupt_event();
    osthread->set_interrupted(false);
    event->reset();
  }

  return interrupted;
}

ExtendedPC os::fetch_top_frame(Thread* thread, jint** sp, jint** fp) {
  OSThread* osthread = thread->osthread();
  assert(thread->osthread()->state() == SUSPENDED, "must be suspended");

#if defined(IA32)
  ucontext_t* context = (ucontext*) osthread->ucontext();
  *sp = context->uc_mcontext.gregs[UESP];
  *fp = context->uc_mcontext.gregs[EBP];
  ExtendedPC pc((address) context->uc_mcontext.gregs[EIP]);
  return pc;
#else
  #error "add processor specific code here";
#endif
}

// return true if successul, false otherwise
bool os::set_thread_pc(Thread* thread, ExtendedPC addr) {
  OSThread* osthread = thread->osthread();
  assert(thread->osthread()->state() == SUSPENDED, "must be suspended");

#if defined(IA32)
  ucontext_t* context = (ucontext*) osthread->ucontext();
  context->uc_mcontext.gregs[EIP] = addr.pc();
  return true;
#else
  #error "add processor specific code here";
#endif
}

////////////////////////////////////////////////////////////////////////////////
// grody low-level lock required because of malloc bootstrapping!

static pthread_t        tc_owner = 0;
static pthread_mutex_t  tc_mutex = PTHREAD_MUTEX_INITIALIZER;
static int              tc_count = 0;

void ThreadCritical::initialize() {
}

void ThreadCritical::release() {
}

ThreadCritical::ThreadCritical() {
  pthread_t self = pthread_self();
  if (self != tc_owner) {
    int ret = pthread_mutex_lock(&tc_mutex);
    assert(ret == 0, "fatal error with pthread_mutex_lock()");
    tc_owner = self;
  }
  tc_count++;
}

ThreadCritical::~ThreadCritical() {
  assert(tc_owner == pthread_self(), "must have correct owner");
  assert(tc_count > 0, "must have correct count");
  if ((--tc_count) == 0) {
    tc_owner = 0;
    int ret = pthread_mutex_unlock(&tc_mutex);
    assert(ret == 0, "fatal error with pthread_mutex_unlock()");
  }
}

// Used for PostMortemDump
extern "C" void safepoints();
extern "C" void find(int x);
extern "C" void findpc(int x);
extern "C" void events();
extern void set_error_reported();
#ifndef PRODUCT
extern "C" void ps();
#endif

///////////////////////////////////////////////////////////////////////////////////
// signal handling (expect suspend/resume/interrupt)

#ifndef PRODUCT
void continue_with_dump(void) {
  tty->print_cr("Dumping thread state before exit....");
  ps();
  fatal("Exiting after thread state dump.");
}
#endif


// This routine may be used by user applications as a "hook" to catch signals.
// The user-defined signal handler must pass unrecognized signals to this
// routine, and if it returns true (non-zero), then the signal handler must
// return immediately.  If the flag "abort_if_unrecognized" is true, then this
// routine will never retun false (zero), but instead will execute a VM panic
// routine kill the process.
//
// If this routine returns false, it is OK to call it again.  This allows
// the user-defined signal handler to perform checks either before or after
// the VM performs its own checks.  Naturally, the user code would be making
// a serious error if it tried to handle an exception (such as a null check
// or breakpoint) that the VM was generating for its own correct operation.
//
// This routine may recognize any of the following kinds of signals:
//    SIGBUS, SIGSEGV, SIGILL, SIGFPE, SIGQUIT, SIGPIPE, SIGUSR1.
// It should be consulted by handlers for any of those signals.
//
// The caller of this routine must pass in the three arguments supplied
// to the function referred to in the "sa_sigaction" (not the "sa_handler")
// field of the structure passed to sigaction().  This routine assumes that
// the sa_flags field passed to sigaction() includes SA_SIGINFO and SA_RESTART.
//
// In order for stack overflow checking to operate correctly, the handler
// for SIGSEGV should also have to specify the SA_ONSTACK bit.
//
// Note that the VM will print warnings if it detects conflicting signal
// handlers, unless invoked with the option "-XX:+AllowUserSignalHandlers".
//
extern "C" int
JVM_handle_linux_signal(int signo, siginfo_t* siginfo,
                        void* ucontext, int abort_if_unrecognized);

void signalHandler(int sig, siginfo_t* info, ucontext_t* uc) {
  assert(info != NULL && uc != NULL, "it must be old kernel");
  JVM_handle_linux_signal(sig, info, uc, true);
}


// This boolean allows users to forward their own non-matching signals
// to JVM_handle_linux_signal, harmlessly.
bool os::Linux::signal_handlers_are_installed = false;

// For signal-chaining
struct sigaction os::Linux::sigact[MAXSIGNUM];
unsigned int os::Linux::sigs = 0;
bool os::Linux::libjsig_is_loaded = false;
typedef struct sigaction *(*get_signal_t)(int);
get_signal_t os::Linux::get_signal_action = NULL;

struct sigaction* os::Linux::get_chained_signal_action(int sig) {
  struct sigaction *actp = NULL;
 
  if (libjsig_is_loaded) {
    // Retrieve the old signal handler from libjsig
    actp = (*get_signal_action)(sig);
  }
  if (actp == NULL) {
    // Retrieve the preinstalled signal handler from jvm
    actp = get_preinstalled_handler(sig);
  }

  return actp;
}

bool os::Linux::chained_handler(struct sigaction *actp, int sig,
                                  siginfo_t *siginfo, void *context) {
  // Call the old signal handler
  if (actp->sa_handler == SIG_DFL) {
    // It's more reasonable to let jvm treat it as an unexpected exception
    // instead of taking the default action.
    return false;
  } else if (actp->sa_handler != SIG_IGN) {
    if ((actp->sa_flags & SA_NODEFER) == 0) {
      // automaticlly block the signal
      sigaddset(&(actp->sa_mask), sig);
    }

    sa_handler_t hand;
    sa_sigaction_t sa;
    bool siginfo_flag_set = (actp->sa_flags & SA_SIGINFO) != 0;
    // retrieve the chained handler
    if (siginfo_flag_set) {
      sa = actp->sa_sigaction;
    } else {
      hand = actp->sa_handler;
    }

    if ((actp->sa_flags & SA_RESETHAND) != 0) {
      actp->sa_handler = SIG_DFL;
    }

    // try to honor the signal mask
    sigset_t oset;
    pthread_sigmask(SIG_SETMASK, &(actp->sa_mask), &oset);

    // call into the chained handler
    if (siginfo_flag_set) {
      (*sa)(sig, siginfo, context);
    } else {
      (*hand)(sig);
    }

    // restore the signal mask
    pthread_sigmask(SIG_SETMASK, &oset, 0);
  }
  // Tell jvm's signal handler the signal is taken care of.
  return true;
}

struct sigaction* os::Linux::get_preinstalled_handler(int sig) {
  if ((( (unsigned int)1 << sig ) & sigs) != 0) {
    return &sigact[sig];
  }
  return NULL;
}

void os::Linux::save_preinstalled_handler(int sig, struct sigaction& oldAct) {
  assert(sig > 0 && sig < MAXSIGNUM, "vm signal out of expected range");
  sigact[sig] = oldAct;
  sigs |= (unsigned int)1 << sig;
}

void os::Linux::set_signal_handler(int sig, bool set_installed) {
  // Check for overwrite.
  struct sigaction oldAct;
  __sigaction(sig, (struct sigaction*)NULL, &oldAct);

  void* oldhand = oldAct.sa_sigaction
                ? CAST_FROM_FN_PTR(void*,  oldAct.sa_sigaction)
		: CAST_FROM_FN_PTR(void*,  oldAct.sa_handler);
  if (oldhand != CAST_FROM_FN_PTR(void*, SIG_DFL) &&
      oldhand != CAST_FROM_FN_PTR(void*, SIG_IGN) &&
      oldhand != CAST_FROM_FN_PTR(void*, signalHandler)) {
    if (AllowUserSignalHandlers || !set_installed) {
      // Do not overwrite; user takes responsibility to forward to us.
      return;
    } else if (UseSignalChaining) {
      // save the old handler in jvm
      save_preinstalled_handler(sig, oldAct);
      // libjsig also interposes the __sigaction() call below and saves the
      // old sigaction on it own.
    } else {
      fatal2("Encountered unexpected pre-existing sigaction handler %#lx for signal %d.", (long)oldhand, sig);
    }
  }

  struct sigaction sigAct;
  sigfillset(&(sigAct.sa_mask));
  sigAct.sa_handler = SIG_DFL;
  if (!set_installed) {
    sigAct.sa_flags = SA_SIGINFO|SA_RESTART;
  } else {
    sigAct.sa_sigaction = signalHandler;
    sigAct.sa_flags = SA_SIGINFO|SA_RESTART;
    if (sig == SIGCHLD) {
      sigAct.sa_flags |= SA_NOCLDSTOP;
    } else if (sig == SIGSEGV) {
      sigAct.sa_flags = SA_SIGINFO|SA_RESTART|SA_ONSTACK;
    }
  }

  int ret = __sigaction(sig, &sigAct, &oldAct);
  assert(ret == 0, "check");

  void* oldhand2  = oldAct.sa_sigaction
                  ? CAST_FROM_FN_PTR(void*, oldAct.sa_sigaction)
                  : CAST_FROM_FN_PTR(void*, oldAct.sa_handler);
  assert(oldhand2 == oldhand, "no concurrent signal handler installation");
}

// install signal handlers for signals that HotSpot needs to
// handle in order to support Java-level exception handling.

void os::Linux::install_signal_handlers() {
  if (!signal_handlers_are_installed) {
    signal_handlers_are_installed = true;

    // signal-chaining
    typedef void (*signal_setting_t)();
    signal_setting_t begin_signal_setting = NULL;
    signal_setting_t end_signal_setting = NULL;
    begin_signal_setting = (signal_setting_t) dlsym(RTLD_DEFAULT,
                                       "JVM_begin_signal_setting");
    if (begin_signal_setting != NULL) {
      end_signal_setting = (signal_setting_t) dlsym(RTLD_DEFAULT,
                                       "JVM_end_signal_setting");
      get_signal_action = (get_signal_t) dlsym(RTLD_DEFAULT,
                                      "JVM_get_signal_action");
      libjsig_is_loaded = true;
      assert(UseSignalChaining, "should enable signal-chaining");
    }
    if (libjsig_is_loaded) {
      // Tell libjsig jvm is setting signal handlers
      (*begin_signal_setting)();
    }

    set_signal_handler(SIGSEGV, true);
    set_signal_handler(SIGPIPE, true);
    set_signal_handler(SIGCHLD, true);
    set_signal_handler(SIGBUS, true);
    set_signal_handler(SIGILL, true);
    set_signal_handler(SIGFPE, true);

    if (libjsig_is_loaded) {
      // Tell libjsig jvm finishes setting signal handlers
      (*end_signal_setting)();
    }
  }
}

extern void report_error(char* file_name, int line_no, char* title, char* format, ...);

const char * os::exception_name(int exception_code)
{
  return NULL;
}

extern "C"{
static void exception_handler_during_fatal_error(int);
}

/* a very simple exception handler designed to print out at least some information */
/* when the fatal error handler crashes.                                           */
static void exception_handler_during_fatal_error(int sig)
{
   os::handle_recursive_fatal_error(sig);
   /* won't reach here */
   os::abort();
}

void os::set_exception_handler_during_fatal_error(int sig, int is_install)
{
   if (is_install)
        os::signal(sig, CAST_FROM_FN_PTR(void *, exception_handler_during_fatal_error));
   else os::signal(sig, CAST_FROM_FN_PTR(void *, SIG_DFL));
}

// this is called _before_ the global arguments have been parsed
void os_init()
{
//  first_hrtime = gethrtime();

  os::init_random(1234567);

  ThreadCritical::initialize();

  page_size = sysconf(_SC_PAGESIZE);

  if (page_size == -1) {
    fatal1("os_linux.cpp: os_init: sysconf failed (%s)", strerror(errno));
  }

  os::Linux::_processor_count = ::sysconf(_SC_NPROCESSORS_CONF);
  assert(os::Linux::_processor_count > 0, "linux error");
  
  // main_thread points to the aboriginal thread
  main_thread = pthread_self();

  // locate pthread_getattr_np and test if we are on floating stack
  void * h = dlopen(NULL, RTLD_LAZY);
  assert(h != NULL, "just checking");
  pthread_getattr_func = (pthread_getattr_func_type)dlsym(h, "pthread_getattr_np");
  if (pthread_getattr_func != NULL) is_floating_stack = true;
  else is_floating_stack = false;

  // locate initial thread's stack location
  struct rlimit rlim;
  getrlimit(RLIMIT_STACK, &rlim);
  // 4441425: avoid crash with "unlimited" stack size on SuSE 7.1 or Redhat
  //   7.1, in both cases we will get 2G in return value.
  if (rlim.rlim_cur > 8 * K * K) rlim.rlim_cur = 8 * K * K;
  initial_thread_stack_size   = rlim.rlim_cur & ~(page_size - 1);
  initial_thread_stack_bottom = INITIAL_THREAD_STACK_BASE - initial_thread_stack_size;

  start_time = (double) os::elapsed_counter();
}

// this is called _after_ the global arguments have been parsed
jint os_init_2()
{
  if (Verbose || WizardMode) {
     if (is_floating_stack) {
       tty->print_cr("[HotSpot is running with floating stack LinuxThreads]");
     } else {
       tty->print_cr("[HotSpot is running with fixed stack LinuxThreads]");
     }
  }

  if (UseTopLevelExceptionFilter) {
    os::Linux::install_signal_handlers();
  }

  // initialize suspend/resume support
  if (SR_initialize() != 0) {
    perror("SR_initialize failed");
    return JNI_ERR;
  }

  if (MaxFDLimit) {
    // set the number of file descriptors to max. print out error 
    // if getrlimit/setrlimit fails but continue regardless. 
    struct rlimit nbr_files;
    int status = getrlimit(RLIMIT_NOFILE, &nbr_files);
    if (status != 0) {
      if (Verbose || WizardMode)
        perror("os::os_init_2 getrlimit failed");
    } else {
      nbr_files.rlim_cur = nbr_files.rlim_max;
      status = setrlimit(RLIMIT_NOFILE, &nbr_files);
      if (status != 0) {
        if (Verbose || WizardMode)
          perror("os::os_init_2 setrlimit failed");
      }
    }
  }

  // Initialise lock used to serialise thread creation (see os::create_thread)
  createThread_lock = new Mutex(Mutex::leaf, "createThread_lock", false);

  // Initialize HPI.
  jint hpi_result = hpi::initialize();
  if (hpi_result != JNI_OK) {
    tty->print_cr("There was an error trying to initialize the HPI library.");
    tty->print_cr("Please check your installation, HotSpot does not work correctly");
    tty->print_cr("when installed in the JDK 1.2 Linux Production Release, or");
    tty->print_cr("with any JDK 1.1.x release.");
    return hpi_result;
  }

  return JNI_OK;
}

void os_exit() {
  // cleanup in the reverse order

  SR_finalize();

  // Make sure remove the alternate signal stack
  os::Linux::remove_alternate_signal_stack();

  ThreadCritical::release();
}

////////////////////////////////////////////////////////////////////////////////
// atomic function support

// these routines are the initial value of atomic_exchange_entry(),
// atomic_compare_and_exchange_entry(), and atomic_increment_entry
// until initialization is complete (see stubRoutines_<arch>.cpp)
// TODO - factor (or see win32 inline model?)

typedef jint exchange_func_t            (jint, jint*);
typedef jint compare_and_exchange_func_t(jint, jint*, jint);
typedef jint increment_func_t           (jint, jint*);

jint os::Linux::atomic_exchange_bootstrap(jint exchange_value, jint* dest) {
  // try to use the stub:
  exchange_func_t* func = CAST_TO_FN_PTR(exchange_func_t*, StubRoutines::atomic_exchange_entry());

  if (func != NULL) {
    os::Linux::atomic_exchange_func = func;
    return (*func)(exchange_value, dest);
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  jint old_value = *dest;
  *dest = exchange_value;
  return old_value;
}

jint os::Linux::atomic_compare_and_exchange_bootstrap(jint exchange_value, jint* dest, jint compare_value) {
  // try to use the stub:
  compare_and_exchange_func_t* func = CAST_TO_FN_PTR(compare_and_exchange_func_t*,
						       StubRoutines::atomic_compare_and_exchange_entry());

  if (func != NULL) {
    os::Linux::atomic_compare_and_exchange_func = func;
    return (*func)(exchange_value, dest, compare_value);
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  jint old_value = *dest;
  if (old_value == compare_value)
    *dest = exchange_value;
  return old_value;
}

jint os::Linux::atomic_increment_bootstrap(jint inc, jint* loc) {
  // try to use the stub:
  increment_func_t* func = CAST_TO_FN_PTR(increment_func_t*, StubRoutines::atomic_increment_entry());

  if (func != NULL) {
    os::Linux::atomic_increment_func = func;
    return (*func)(inc, loc);
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  return (*loc) += inc;
}

exchange_func_t* os::Linux::atomic_exchange_func 
  = os::Linux::atomic_exchange_bootstrap;
compare_and_exchange_func_t* os::Linux::atomic_compare_and_exchange_func
  = os::Linux::atomic_compare_and_exchange_bootstrap;
increment_func_t* os::Linux::atomic_increment_func
  = os::Linux::atomic_increment_bootstrap;

////////////////////////////////////////////////////////////////////////////////
// debug support

#ifndef PRODUCT
static address same_page(address x, address y) {
  int page_bits = -os::vm_page_size();
  if ((jint(x) & page_bits) == (jint(y) & page_bits))
    return x;
  else if (x > y)
    return (address)(jint(y) | ~page_bits) + 1;
  else
    return (address)(jint(y) & page_bits);
}

bool os::find(address addr) {
  Dl_info dlinfo;
  memset(&dlinfo, 0, sizeof(dlinfo));
  if (dladdr(addr, &dlinfo)) {
    tty->print("0x%08x: ", addr);
    if (dlinfo.dli_sname != NULL)
      tty->print("%s+%#x", dlinfo.dli_sname, addr-(int)dlinfo.dli_saddr);
    else if (dlinfo.dli_fname)
      tty->print("<offset %#x>", addr-(int)dlinfo.dli_fbase);
    else
      tty->print("<absolute address>");
    if (dlinfo.dli_fname)  tty->print(" in %s", dlinfo.dli_fname);
    if (dlinfo.dli_fbase)  tty->print(" at 0x%08x", dlinfo.dli_fbase);
    tty->cr();

    if (Verbose) {
      // decode some bytes around the PC
      address begin = same_page(addr-40, addr);
      address end   = same_page(addr+40, addr);
      address       lowest = (address) dlinfo.dli_sname;
      if (!lowest)  lowest = (address) dlinfo.dli_fbase;
      if (begin < lowest)  begin = lowest;
      Dl_info dlinfo2;
      if (dladdr(end, &dlinfo2) && dlinfo2.dli_saddr != dlinfo.dli_saddr
	  && end > dlinfo2.dli_saddr && dlinfo2.dli_saddr > begin)
        end = (address) dlinfo2.dli_saddr;
      Disassembler::decode(begin, end);
    }
    return true;
  }
  return false;
}
 
#endif

////////////////////////////////////////////////////////////////////////////////
// misc

// This does not do anything on Linux. This is basically a hook for being
// able to use structured exception handling (thread-local exception filters)
// on, e.g., Win32.
void
os::os_exception_wrapper(java_call_t f, JavaValue* value, methodHandle* method,
                         JavaCallArguments* args, Thread* thread) {
  f(value, method, args, thread);  
}

void os::print_statistics() {
}

int os::message_box(const char* title, const char* message) {
  ::write(2, title, strlen(title));
  ::write(2, "\n", 1);
  ::write(2, message, strlen(message));
  ::write(2, "\n", 1);

  char buf[256];
  // Prevent process from exiting upon "read error" without consuming all CPU
  while (::read(0, buf, 256) <= 0) { ::sleep(100); }

  return buf[0] == 'y' || buf[0] == 'Y';
}

int os::stat(const char *path, struct stat *sbuf) {
  char pathbuf[MAX_PATH];
  if (strlen(path) > MAX_PATH - 1) {
    errno = ENAMETOOLONG;
    return -1;
  }
  hpi::native_path(strcpy(pathbuf, path));
  return ::stat(pathbuf, sbuf);
}

bool os::check_heap(bool force) {
  return true;
}

int local_vsnprintf(char* buf, size_t count, const char* format, va_list args) {
  return ::vsnprintf(buf, count, format, args);
}

// JVMPI code
jlong os::thread_cpu_time() {
  //
  // gethrtime() is used in Solaris for this function. However, we
  // don't have gethrtime(). It is also used in the Solaris version
  // of os::elapsed_counter() so we leverage off our implementation
  // of os::elapsed_counter().
  //
  return os::elapsed_counter();
}

bool os::thread_is_running(JavaThread* tp) {
  Unimplemented();
  return false;
}

#ifndef PRODUCT
void os::Linux::Event::verify() {
  guarantee(Universe::perm_gen() == NULL ||
            !Universe::is_reserved_heap((oop)this),
            "Mutex must be in C heap only.");
}

void os::Linux::OSMutex::verify() {
  guarantee(Universe::perm_gen() == NULL || !Universe::is_reserved_heap((oop)this), "OSMutex must be in C heap only.");
}

void os::Linux::OSMutex::verify_locked() {
  pthread_t my_id = pthread_self();
  assert(_is_owned, "OSMutex should be locked");
  assert(pthread_equal(_owner, my_id), "OSMutex should be locked by me");
}
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * NOTE: the following code is to keep the green threads code
 * in the libjava.so happy. Once the green threads is removed,
 * these code will no longer be needed. 
 */
int
jdk_waitpid(pid_t pid, int* status, int options) {
    return waitpid(pid, status, options);
}

int
fork1() {
    return fork();
}

int
jdk_sem_init(sem_t *sem, int pshared, unsigned int value) {
    return sem_init(sem, pshared, value);
}

int
jdk_sem_post(sem_t *sem) {
    return sem_post(sem);
}

int
jdk_sem_wait(sem_t *sem) {
    return sem_wait(sem);
}

int
jdk_pthread_sigmask(int how , const sigset_t* newmask, sigset_t* oldmask) {
    return pthread_sigmask(how , newmask, oldmask);
}

#ifdef __cplusplus
}
#endif

// Reconciliation History
// 
// End
