/*
 * @(#)hpi_solaris.hpp	1.10 00/05/08 16:22:51
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

//
// Parts of the HPI interface for which the HotSparc does not use the
// HPI (because the interruptible IO mechanims used are different).
//

#include <sys/socket.h>
#include <sys/poll.h>
#include <sys/filio.h>
#include <unistd.h>
#include <netdb.h>

// HPI_FileInterface

inline int    hpi::close(int fd) {
  return ::close(fd);
}

inline size_t hpi::read(int fd, void *buf, unsigned int nBytes) {
  INTERRUPTIBLE_RETURN_INT(::read(fd, buf, nBytes), os::Solaris::clear_interrupted);
}

inline size_t hpi::write(int fd, const void *buf, unsigned int nBytes) {
  INTERRUPTIBLE_RETURN_INT(::write(fd, buf, nBytes), os::Solaris::clear_interrupted);
}


// HPI_SocketInterface

inline int    hpi::socket_close(int fd) {
  return ::close(fd);
}

inline int    hpi::socket(int domain, int type, int protocol) {
  return ::socket(domain, type, protocol);
}

inline int    hpi::recv(int fd, char *buf, int nBytes, int flags) {
  INTERRUPTIBLE_RETURN_INT(::recv(fd, buf, nBytes, flags), os::Solaris::clear_interrupted);
}

inline int    hpi::send(int fd, char *buf, int nBytes, int flags) {
  INTERRUPTIBLE_RETURN_INT(::send(fd, buf, nBytes, flags), os::Solaris::clear_interrupted);
}

inline int    hpi::timeout(int fd, long timeout) {
#ifndef USE_SELECT
  struct pollfd pfd;
  
  pfd.fd = fd;
  pfd.events = POLLIN;
  
  INTERRUPTIBLE_RETURN_INT(::poll(&pfd, 1, timeout), os::Solaris::clear_interrupted);
#else
  fd_set tbl;
  struct timeval t;
  
  t.tv_sec = timeout / 1000;
  t.tv_usec = (timeout % 1000) * 1000;
  
  FD_ZERO(&tbl);
  FD_SET(fd, &tbl);
  
  INTERRUPTIBLE_RETURN_INT(::select(fd + 1, &tbl, 0, 0, &t), os::Solaris::clear_interrupted);
#endif
}

inline int    hpi::listen(int fd, int count) {
  if (fd < 0) 
    return -1;
  
  return ::listen(fd, count);
}

inline int    hpi::connect(int fd, struct sockaddr *him, int len) {
  INTERRUPTIBLE_RETURN_INT(::connect(fd, him, len), os::Solaris::clear_interrupted);
}

// Note: this looks very suspicious, why the loop?
inline int    _hpi_accept(int fd, struct sockaddr *him, int *len) {
  int result;
  //%note jvm_r10
  while((result = accept(fd, him, len)) == -1 && errno == EINTR)
    ;
  return result;
}

inline int    hpi::accept(int fd, struct sockaddr *him, int *len) {
  if (fd < 0)
    return -1;
  INTERRUPTIBLE_RETURN_INT(_hpi_accept(fd, him, (int*)len), os::Solaris::clear_interrupted);
}

inline int    hpi::recvfrom(int fd, char *buf, int nBytes, int flags,
			    sockaddr *from, int *fromlen) {
  //%%note jvm_r11
  INTERRUPTIBLE_RETURN_INT((int)::recvfrom(fd, buf, nBytes, flags, from, fromlen), os::Solaris::clear_interrupted);
}

inline int    hpi::sendto(int fd, char *buf, int len, int flags,
			  struct sockaddr *to, int tolen) {
  //%%note jvm_r11
  INTERRUPTIBLE_RETURN_INT((int)::sendto(fd, buf, len, flags, to, tolen), os::Solaris::clear_interrupted);
}

inline int    hpi::socket_available(int fd, long *pbytes) {
  if (fd < 0)
    return 0;

  int ret = ::ioctl(fd, FIONREAD, pbytes);

  if (ret < 0)
    return 0;

  //%% note ioctl can return 0 when successful, JVM_SocketAvailable
  // is expected to return 0 on failure and 1 on success to the jdk.
  return 1;
}


// following methods have been updated to avoid problems in 
// hpi's sockets calls based on sys_api_td.c (JDK1.3)

/* 
HPIDECL(get_host_by_name, "get_host_by_name", _socket, GetHostByName,
        struct hostent *, "(struct hostent *)%p",
        (char *name),
        ("%s", name),
        (name));
	*/
inline struct hostent * hpi::get_host_by_name(char *hostname) {
  return ::gethostbyname(hostname);
}

/*
HPIDECL(socket_shutdown, "socket_shutdown", _socket, SocketShutdown, 
        int, "%d",
        (int fd, int howto),
        ("fd = %d, howto = %d", fd, howto),
        (fd, howto));
	*/
inline int hpi::socket_shutdown(int fd, int howto){
  return ::shutdown(fd, howto);
}

/*
HPIDECL(bind, "bind", _socket, Bind, 
        int, "%d",
        (int fd, struct sockaddr *him, int len),
        ("fd = %d, him = %p, len = %d",
         fd, him, len),
        (fd, him, len));
*/
inline int hpi::bind(int fd, struct sockaddr *him, int len){
  INTERRUPTIBLE_RETURN_INT(::bind(fd, him, len),os::Solaris::clear_interrupted);
}

/*
HPIDECL(get_sock_name, "get_sock_name", _socket, GetSocketName, 
        int, "%d",
        (int fd, struct sockaddr *him, int *len),
        ("fd = %d, him = %p, len = %p",
         fd, him, len),
        (fd, him, len));
	*/
inline int hpi::get_sock_name(int fd, struct sockaddr *him, int *len){
  return ::getsockname(fd, him, len);
}

/*
HPIDECL(get_host_name, "get_host_name", _socket, GetHostName, int, "%d",
        (char *hostname, int namelen),
        ("hostname = %p, namelen = %d",
         hostname, namelen),
        (hostname, namelen));
	*/
inline int hpi::get_host_name(char* name, int namelen){
  return ::gethostname(name, namelen);
}

/*
HPIDECL(get_host_by_addr, "get_host_by_addr", _socket, GetHostByAddr, 
        struct hostent *, "(struct hostent *)%p",
        (const char* name, int len, int type),
        ("name = %p, len = %d, type = %d",
         name, len, type),
        (name, len, type));
	*/
inline struct hostent* hpi::get_host_by_addr(const char* name, int len, int type){
  return ::gethostbyaddr(name, len, type);
}

/*
HPIDECL(get_sock_opt, "get_sock_opt", _socket, SocketGetOption, int, "%d",
        (int fd, int level, int optname, char *optval, int* optlen),
        ("fd = %d, level = %d, optname = %d, optval = %p, optlen = %p",
         fd, level, optname, optval, optlen),
        (fd, level, optname, optval, optlen));
	*/
inline int hpi::get_sock_opt(int fd, int level, int optname, 
			     char *optval, int* optlen){
  return ::getsockopt(fd, level, optname, optval, optlen);
}

/*
HPIDECL(set_sock_opt, "set_sock_opt", _socket, SocketSetOption, int, "%d",
        (int fd, int level, int optname, const char *optval, int optlen),
        ("fd = %d, level = %d, optname = %d, optval = %p, optlen = %d",
         fd, level, optname, optval, optlen),
        (fd, level, optname, optval, optlen));
	*/
inline int hpi::set_sock_opt(int fd, int level, int optname, 
			     const char *optval, int optlen){
  return ::setsockopt(fd, level, optname, optval, optlen);
}

/*
HPIDECL(get_proto_by_name, "get_proto_by_name", _socket, GetProtoByName, 
        struct protoent *, "(struct protoent *)%p",
        (char* name),
        ("name = %p",
         name),
        (name));
	*/
inline struct protoent* hpi::get_proto_by_name(char* name){
  return ::getprotobyname(name);
}


//Reconciliation History
// 1.3 98/10/21 18:17:14 hpi_win32.hpp
// 1.6 99/06/28 11:01:36 hpi_win32.hpp
//End
