// @(#)os_share_solaris.hpp	1.8 01/02/14 09:12:04
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

// Defines the interfaces to Solaris operating systems that vary across platforms

// Asynchronous: cancelled by the target thread
class Interrupt_Callback : public OSThread::InterruptCallback {
 public:
  Interrupt_Callback(int priority = OSThread::InterruptCallback::normal) : 
    OSThread::InterruptCallback(priority, false) {}

  void execute(OSThread::InterruptArguments *args);
};

// Asynchronous: cancelled by the target thread
class SetThreadPC_Callback : public OSThread::InterruptCallback {
 private:
  ExtendedPC _addr;

 public:
  SetThreadPC_Callback(ExtendedPC addr, int priority = OSThread::InterruptCallback::normal) : 
    OSThread::InterruptCallback(priority, false) {
    _addr = addr;
  }

  void execute(OSThread::InterruptArguments *args);
};

class FetchTopFrame_SuspendThreadSelf_Callback : public SuspendThreadSelf_Callback {
 private:
  bool* _is_done;
 public:
  FetchTopFrame_SuspendThreadSelf_Callback(bool* is_done) { _is_done = is_done; }
  bool before_wait(OSThread* osthread) { 
    *_is_done = true;
    Interrupt_lock->notify_all();
    return true;
  }
};

// Synchronous: signal sender waits for the frame. processes it and then cancels the callback
class FetchTopFrame_Callback : public OSThread::InterruptCallback {
 private:
  ExtendedPC _addr;
  jint*      _sp;
#ifndef SPARC
  jint*      _fp;  // Intel solaris only
#endif
  bool       _is_done;

  static FetchTopFrame_Callback* _cancelled_list;

 public:
  jint*      sp()   const { return _sp;   }
#ifndef SPARC
  jint*      fp()   const { return _fp;   }
#endif
  ExtendedPC addr() const { return _addr; }

  FetchTopFrame_Callback(int priority = OSThread::InterruptCallback::normal) : OSThread::InterruptCallback(priority, true) { 
    _is_done = false;
  }

  // call the following factory routine instead of "new" when calling
  // in a situation where a suspended thread might hold the malloc
  // lock. Keep a statically allocated buffer of callbacks, currently 2 - one
  // for the VM thread and one for watcher thread.

  static FetchTopFrame_Callback* create(int priority = OSThread::InterruptCallback::normal) {
    // There is a race condition here if fprofiler is active. Wathcer thread resumes 
    // before callback cancell in the interrupt handler is executed and the callback
    // is returned to the free list. The watcher thread may run out callbacks 
    // (and does so occasionally on regression tests for threaded programs).
    assert(_cancelled_list != NULL, "no free callbacks");

    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);

    FetchTopFrame_Callback* cb = _cancelled_list;
    _cancelled_list = (FetchTopFrame_Callback*)cb->next();

    cb->_is_done = false;

    return cb;
  }

  void move_to_cancelled_list() {
    assert((Interrupt_lock == NULL) || Interrupt_lock->owned_by_self(), "Must have an interrupt lock");
    set_next(_cancelled_list);
    _cancelled_list = this;
  }

  bool is_done() const { return _is_done; }

  void execute(OSThread::InterruptArguments *args);
};

// This is a simple callback that just fetches a PC for an interrupted thread.
// The thread need not be suspended and the fetched PC is just a hint.
// Returned PC and nPC are not necessarily consecutive.
// This one is currently used for profiling the VMThread ONLY!

// Should be synchronous
class GetThreadPC_Callback : public OSThread::InterruptCallback {
 private:
  ExtendedPC _addr;
  bool       _is_done;

 public:
  ExtendedPC addr() const { return _addr; }

  // synchrounous
  // GetThreadPC_Callback(int priority = OSThread::InterruptCallback::normal) : OSThread::InterruptCallback(priority, true);
  GetThreadPC_Callback(int priority = OSThread::InterruptCallback::normal);

  // call the following factory routine instead of "new".
  static GetThreadPC_Callback* create(OSThread* osthread, int priority = OSThread::InterruptCallback::normal);

  // Nothing to cancel, there is a single token callback
  void move_to_cancelled_list() {}

  void set_addr(ExtendedPC addr) { _addr = addr; }
  bool is_done() const { return _is_done; }

  void execute(OSThread::InterruptArguments *args);
};

// misc
extern "C" {
  void signalHandler(int, siginfo_t*, void*);
}
void resolve_lwp_exit_calls(void);
int send_interrupt(Thread*);
void handle_unexpected_exception(Thread* thread, int sig, siginfo_t* info, address pc, address adjusted_pc);
#ifndef PRODUCT
void continue_with_dump(void);
#endif

#define PROCFILE_LENGTH 128

