/*
 * @(#)os_solaris.cpp	1.177 01/03/16 12:16:56
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
// do not include  precompiled  header file
# include "incls/_os_solaris.cpp.incl"

// put OS-includes here
# include <sys/types.h>
# include <sys/mman.h>
# include <pthread.h>
# include <signal.h>
# include <errno.h>
# include <dlfcn.h>
# include <stdio.h>
# include <unistd.h>
# include <sys/resource.h>
# include <thread.h>
# include <sys/stat.h>
# include <sys/time.h>
# include <sys/filio.h>
# include <sys/utsname.h>
# include <sys/systeminfo.h>
# include <sys/socket.h>
# include <sys/lwp.h>
# include <pwd.h>
# include <poll.h>
# include <sys/lwp.h>
# include <sys/shm.h>
# include <sys/ipc.h>
# include <link.h>

# define _STRUCTURED_PROC 1  //  this gets us the new structured proc interfaces of 5.6 & later
# include <sys/procfs.h>     //  see comment in <sys/procfs.h>

# ifdef PCSREG               //  5.6 or newer
# define SOLARIS_STRUCTURED_PROC
# endif

#define MAX_PATH (2 * K)

address os::Solaris::handler_start;  // start pc of thr_sighndlrinfo
address os::Solaris::handler_end;    // end pc of thr_sighndlrinfo

address os::Solaris::_main_stack_base = NULL;  // 4352906 workaround

// "default" initializers for missing libc APIs
extern "C" {
  static int lwp_mutex_init(mutex_t *mx, int scope, void *arg) { memset(mx, 0, sizeof(mutex_t)); return 0; }
  static int lwp_mutex_destroy(mutex_t *mx)                 { return 0; }

  static int lwp_cond_init(cond_t *cv, int scope, void *arg){ memset(cv, 0, sizeof(cond_t)); return 0; }
  static int lwp_cond_destroy(cond_t *cv)                   { return 0; }
}

// "default" initializers for pthread-based synchronization
extern "C" {
  static int pthread_mutex_default_init(mutex_t *mx, int scope, void *arg) { memset(mx, 0, sizeof(mutex_t)); return 0; }

  static int pthread_cond_default_init(cond_t *cv, int scope, void *arg){ memset(cv, 0, sizeof(cond_t)); return 0; }
}


void os::Solaris::install_alternate_signal_stack() {
  stack_t alt_stack;

  assert(UseBoundThreads,
         "Shouldn't attempt to install alternate signal stack");

  sigaltstack(NULL, &alt_stack);
  assert(alt_stack.ss_flags & SS_DISABLE,
         "Shouldn't already have an alternate stack!");
  if (alt_stack.ss_flags & SS_DISABLE) {
    assert(AltStackSize >= MINSIGSTKSZ,
          "Alt sig stack size must be larger than MINSIGSTKSZ");
    int *alt_stack_base = (int *)memalign(vm_page_size(), AltStackSize);
    if (alt_stack_base == NULL) {
      debug_only(perror("memalign"));
      assert(false, "Couldn't allocate alternate signal stack");
      warning("Cannot allocate alt signal stack");
      return;
    }

    alt_stack.ss_sp = (char *)alt_stack_base;
    alt_stack.ss_size = AltStackSize;
    alt_stack.ss_flags = 0;

    if (sigaltstack((const stack_t *)&alt_stack, NULL) != 0) {
      debug_only(perror("sigaltstack"));
      assert(false, "Cannot install alt signal stack");
      warning("Cannot install alt signal stack");
    }
  }
}

void os::Solaris::remove_alternate_signal_stack() {
  stack_t alt_stack;

  assert(UseBoundThreads,
         "Shouldn't attempt to uninstall (absent) alternate signal stack");

  sigaltstack(NULL,&alt_stack);
  assert(!(alt_stack.ss_flags & SS_DISABLE) && alt_stack.ss_sp != NULL,
         "Should have an alternate signal stack");
  if(!(alt_stack.ss_flags & SS_DISABLE)) {
    stack_t new_stack;
    
    new_stack.ss_sp = NULL;
    new_stack.ss_size = 0;
    new_stack.ss_flags = SS_DISABLE;

    if (sigaltstack((const stack_t *)&new_stack,NULL) != 0) {
      debug_only(perror("sigaltstack"));
      assert(false, "Cannot uninstall alt signal stack");
      warning("Cannot uninstall alt signal stack");
    } else {
      // Return via libC free() what was obtained above via
      // memalign(). Our memory handling isn't smart about
      // doling out aligned memory.
      // XXX: The following comment seems obsolete today 2000/06/09,
      //  because we shouldn't be trying to remove an alternate
      // signal stack even before our memory handling is initialized;
      // The latter is done single-threaded before any java threads
      // have started to exist (so clearly they can't be dying):
      // "We must use the libC free, our memory handling isn't initialized
      //  when this is called the first time."
      ::free((void *)alt_stack.ss_sp);
    }
  }
}

address os::current_stack_base() {
  bool is_primordial_thread = thr_main();

  // Workaround 4352906, avoid calls to thr_stksegment by
  // thr_main after the first one (it looks like we trash
  // some data, causing the value for ss_sp to be incorrect).
  if (!is_primordial_thread || os::Solaris::_main_stack_base == NULL) {
    stack_t st;
    int retval;
    retval = thr_stksegment(&st);
    assert(retval == 0, "incorrect return value from thr_stksegment");
    assert((address)&st < (address)st.ss_sp, "Invalid stack base returned");
    assert((address)&st > (address)st.ss_sp-st.ss_size, "Invalid stack size returned");
    if (is_primordial_thread) {
      // cache initial value of stack base
      os::Solaris::_main_stack_base = (address)st.ss_sp;
    }
    return (address)st.ss_sp;
  } else {
    guarantee(os::Solaris::_main_stack_base != NULL, "Attempt to use null cached stack base");
    return os::Solaris::_main_stack_base;
  }
}

int os::current_stack_size() {
  int size;

  if(!thr_main()) {
    stack_t st;
    int retval;

    retval = thr_stksegment(&st);
    assert(retval == 0, "incorrect return value from thr_stksegment");
    assert((address)&st < (address)st.ss_sp, "Invalid stack base returned");
    assert((address)&st > (address)st.ss_sp-st.ss_size, "Invalid stack size returned");
    size = st.ss_size;
  } else {
    struct rlimit limits;
    getrlimit(RLIMIT_STACK,&limits);
    size = (int)limits.rlim_cur;
  }

  return size;
}

// interruptible infrastructure

// setup_interruptible saves the thread state before going into an
// interruptible system call.  The thread's post java sp is set up
// (save_state_before_block as in StateSaver) and the thread is
// transitioned to the _thread_blocked state (as in
// ThreadBlockInVM). The saved state is used to restore the thread to
// its former state whether or not an interrupt is received.  
//
// In short, the setup_interruptible/cleanup_interruptible pair is
// "morally equivalent" to enclosing the same region is
// StateSaver/ThreadBlockInVM bracket objects.

void os::Solaris::setup_interruptible(JavaThread* thread) {
  if(ParkTLE) {
    assert(UseTLE, "UseTLE must be on");
    thread->tle().park();                        // Make TLE available to others while blocked
  } 
 
  jint* state = thread->save_state_before_block();
  JavaThreadState thread_state = thread->thread_state();

  OSThread* osthread = thread->osthread();

  osthread->set_saved_interrupt_state(state);
  osthread->set_saved_interrupt_thread_state(thread_state);

  assert(thread_state != _thread_blocked, "Coming from the wrong thread");
  ThreadStateTransition::transition(thread, thread_state, _thread_blocked);
}

JavaThread* os::Solaris::setup_interruptible() {
  JavaThread* thread = (JavaThread*)ThreadLocalStorage::thread();
  setup_interruptible(thread);
  return thread;
}

// cleanup_interruptible reverses the effects of setup_interruptible

void os::Solaris::cleanup_interruptible(JavaThread* thread) {
  OSThread* osthread = thread->osthread();

  if(ParkTLE) {
    thread->tle().unpark();
  }

  ThreadStateTransition::transition(thread, _thread_blocked, osthread->saved_interrupt_thread_state());
  thread->restore_state_after_block(osthread->saved_interrupt_state());
}

/*
The distinct watcher support in the os level has gone away.  If
we need something like that we need an abstraction over the native
facilities.

void real_time_tick(int delay_time);

// The sole purpose of the watcher thread is simulating
// timer interrupts.

int WatcherMain(void*) {
  const int delay_interval = 10; // Delay 10 ms
  struct timeval delay_interval_tv;
  delay_interval_tv.tv_sec = 0;
  delay_interval_tv.tv_usec = delay_interval * 1000;
  while(1) {
    select(0, 0, 0, 0, &delay_interval_tv);
      break;
    real_time_tick(delay_interval);
  }
  return 0;
}

*/


static int _processors_online = 0; 
int os::Solaris::_processor_count = 0;
jint os::Solaris::_os_thread_limit = 0;
jint os::Solaris::_os_thread_count = 0;

static hrtime_t first_hrtime = 0;
static const hrtime_t hrtime_hz = 1000*1000*1000;


void os::Solaris::initialize_system_info() {

  int _processor_configured;
  int _processor_online;

  _processor_configured = sysconf(_SC_NPROCESSORS_CONF);
  _processor_online     = sysconf(_SC_NPROCESSORS_ONLN);
  _processor_count = _processor_configured;
  _processors_online 	= _processor_online; 
}


bool os::getenv(const char* name, char* buffer, int len) {
  char* val = ::getenv( name );
  if ( val == NULL 
  ||   strlen(val) + 1  >  len ) {
    if (len > 0)  buffer[0] = 0; // return a null string
    return false;
  }
  strcpy( buffer, val );
  return true;
}


static char* get_property(char* name, char* buffer, int buffer_size) {
  if (os::getenv(name, buffer, buffer_size)) {
    return buffer;
  }
  static char empty[] = "";
  return empty;
}


props_md_t* os::get_system_properties() {
  if (_system_properties != NULL) {
    return _system_properties;
  }
  // create properties structure and fill it
  props_md_t* props = new props_md_t();
  memset(props, '\0', sizeof(*props));

  char arch[12];
  sysinfo(SI_ARCHITECTURE, arch, sizeof(arch));

  // The following extra steps are taken in the debugging version:
  //
  // Look for the environment variable $JAVA_HOME to override
  // location of libjvm.so, if defined.
  // Important note. if the location of libjvm.so changes this 
  // code needs to be changed accordingly.

  // HotSpot integration note:
  // code below matches code in JDK1.3fcs O build.
  // (Some '#ifndef PRODUCT' lines are introduced.)
  // synced with src/solaris/javavm/runtime/javai_md.c:
  // D 1.11 99/09/14 13:49:34 luehe 13 12	00011/00004/00220

  // The next few definitions allow the code to be verbatim:
  props_md_t& sprops = *props;
#define GetPropertiesMD os::get_system_properties
#define NATIVE 1
#define malloc(n) (char*)NEW_C_HEAP_ARRAY(char, (n))
#define getenv(n) ::getenv(n)

#define RT_JAR         "/lib/rt.jar"
#define I18N_JAR       "/lib/i18n.jar"
#define SUNRSASIGN_JAR "/lib/sunrsasign.jar"
#define CLASSES_DIR    "/classes"
#define DEFAULT_LD_LIBRARY_PATH "/usr/lib" /* See ld.so.1(1) */
#define EXTENSIONS_DIR "/lib/ext"

  char *v; /* tmp var */
  {
    /* sysclasspath, java_home, dll_dir */
    {
        char *home_path;
	char *sysclasspath;
	char *dll_path;
        char buf[MAXPATHLEN];
#if 1 //HotSpot change
	os::jvm_path(buf, sizeof(buf));
#else //HotSpot change
	Dl_info dlinfo;
	dladdr((void *)GetPropertiesMD, &dlinfo);
	realpath((char *)dlinfo.dli_fname, buf);
#endif //HotSpot change
	*(strrchr(buf, '/')) = '\0';	/* get rid of /libjvm.so */
	*(strrchr(buf, '/')) = '\0';    /* get rid of classic    */
	dll_path = malloc(strlen(buf) + 1);
	if (dll_path == 0)
	    return NULL;
	strcpy(dll_path, buf);
	sprops.dll_dir = dll_path;

	*(strrchr(buf, '/')) = '\0';
	*(strrchr(buf, '/')) = '\0';

	home_path = malloc(strlen(buf) + 1);
	if (home_path == 0)
	    return NULL;
	strcpy(home_path, buf);
	sprops.java_home = home_path;

	// Four nulls from the #defines are enough for 3 colons + 1 null below.
	sysclasspath = malloc(strlen(home_path) * 4 + 
 			      sizeof(RT_JAR) + sizeof(I18N_JAR) +
 			      sizeof(SUNRSASIGN_JAR) + sizeof(CLASSES_DIR));
	if (sysclasspath == 0) {
          return NULL;
	}
	sprintf(sysclasspath, "%s" RT_JAR ":%s" I18N_JAR ":%s" SUNRSASIGN_JAR
		":%s" CLASSES_DIR, home_path, home_path, home_path, home_path);
	sprops.sysclasspath = sysclasspath;
    }

    /* Where to look for native libraries */
    {
	/* Get the user setting of LD_LIBRARY_PATH */
	v = getenv("LD_LIBRARY_PATH");

	if (v == NULL) {
	    /* This is never the case. */
	    v = (char *) malloc(sizeof(DEFAULT_LD_LIBRARY_PATH));
	    strcpy(v, DEFAULT_LD_LIBRARY_PATH);
	} else {
	    char *ld_library_path = (char *)
		malloc(sizeof(DEFAULT_LD_LIBRARY_PATH) 	/* enough for '\0' */ +
		       1 				/* colon */ +
		       strlen(v));
	    strcpy(ld_library_path, v);
	    strcat(ld_library_path, ":" DEFAULT_LD_LIBRARY_PATH);
	    v = ld_library_path;
	}
	sprops.library_path = v;
    }

    /* Extensions directories */
    {
	char * buf;
	buf = malloc(strlen(sprops.java_home) + sizeof(EXTENSIONS_DIR));
	sprintf(buf, "%s" EXTENSIONS_DIR, sprops.java_home);
	sprops.ext_dirs = buf;
    }
  }

#undef GetPropertiesMD
#undef NATIVE
#undef malloc
#undef getenv
#undef RT_JAR
#undef I18N_JAR
#undef SUNRSASIGN_JAR
#undef CLASSES_DIR
#undef DEFAULT_LD_LIBRARY_PATH
#undef EXTENSIONS_DIR

  // Done
  _system_properties = props;
  return props;
}

void os::breakpoint() {
  BREAKPOINT;
}

bool os::obsolete_option(JavaVMOption *option)
{
  if (!strncmp(option->optionString, "-Xt", 3)) {
    return true;
  } else if (!strncmp(option->optionString, "-Xtm", 4)) {
    return true;
  } else if (!strncmp(option->optionString, "-Xverifyheap", 12)) {
    return true;
  } else if (!strncmp(option->optionString, "-Xmaxjitcodesize", 16)) {
    return true;
  } 
  return false;
}

#ifndef PRODUCT
void os::Solaris::Event::verify() {
  guarantee(Universe::perm_gen() == NULL || !Universe::is_reserved_heap((oop)this), "Event must be in C heap only.");
}

void os::Solaris::OSMutex::verify() {
  guarantee(Universe::perm_gen() == NULL || !Universe::is_reserved_heap((oop)this), "OSMutex must be in C heap only.");
}

void os::Solaris::OSMutex::verify_locked() {
  int my_id = thr_self();
  assert(_is_owned, "OSMutex should be locked");
  assert(_owner == my_id, "OSMutex should be locked by me");
}
#endif

extern "C" void breakpoint() {
  // use debugger to set breakpoint here
}

// Returns an estimate of the current stack pointer. Result must be guaranteed to
// point into the calling threads stack, and be no lower than the current stack 
// pointer.
address os::current_stack_pointer() {
  int dummy;
  address sp = (address)&dummy + 8;	// %%%% need to confirm if this is right
  return sp;
}

static thread_t main_thread;

// Thread start routine for all new Java threads
extern "C" {
  static void* _start(void* data) {
    // %%%%% should do the os::initialize_thread actions in here
    Thread* thread = (Thread*)data;
    OSThread* osthr = thread->osthread();
    
    assert(osthr->state() == RUNNABLE, "invalid os thread state");
    
    thread->run();

    // One less thread is executing
    atomic::decrement(&os::Solaris::_os_thread_count);

    if (UseDetachedThreads) {
      thr_exit(NULL);
      ShouldNotReachHere();
    }
    return NULL;
  }
}


static OSThread* create_os_thread(thread_t thread_id) {
  // Allocate the OSThread object
  OSThread* osthread = new OSThread(NULL, NULL);
  if (osthread == NULL) return NULL;

  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);  
    // Initial state is ALLOCATED but not INITIALIZED
    osthread->set_state(ALLOCATED);
    
    // Store info on the Solaris thread into the OSThread
    osthread->set_thread_id(thread_id);
    
    // Initial thread state is INITIALIZED, not SUSPENDED
    osthread->set_state(INITIALIZED);
  }
  return osthread;
}


bool os::create_attached_thread(Thread* thread) {
  OSThread* osthread = create_os_thread(thr_self());
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    osthread->set_state(RUNNABLE);   
    osthread->set_state_under_suspend(RUNNABLE);   
  }
  thread->set_osthread(osthread);
  return osthread != NULL;
}

bool os::create_main_thread(Thread* thread) {
  if (_starting_thread == NULL) {
    _starting_thread = create_os_thread(main_thread);
  }
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);  
    // The primodial thread is runnable from the start
    _starting_thread->set_state_under_suspend(RUNNABLE);
  }
  thread->set_osthread(_starting_thread);
  return _starting_thread != NULL;
}

// _mutator_libthread is true if we have dynamically detected the presense of
// the new SunSoft libthread.so (2.8)
bool os::Solaris::_mutator_libthread = false;

bool os::create_thread(Thread* thread, ThreadType thr_type)  {
  // Allocate the OSThread object
  OSThread* osthread = new OSThread(NULL, NULL);
  bool is_vm_thread = (thr_type == vm_thread);
  if (osthread == NULL) {
    return false;
  }
  
  size_t stack_size = 0;
  if (thr_type == java_thread && JavaThread::stack_size_at_create() > 0) {
    size_t min_stack = thr_min_stack();
    stack_size = max(JavaThread::stack_size_at_create(), min_stack);
  }
  else if (is_vm_thread && VMThreadStackSize > 0) {
    size_t min_stack = thr_min_stack();
    stack_size = max((size_t)(VMThreadStackSize * K), min_stack);
  }
#ifdef COMPILER2
  // Compiler2 requires a large stack size to handle recursive routines.
  else if (thr_type == compiler_thread) {
    size_t min_stack = thr_min_stack();
    // stack size in units of Kbytes; 2M total needed as default.
    int default_size = (CompilerThreadStackSize > 0) ? CompilerThreadStackSize : 2 * K;
    stack_size = max((size_t)(default_size * K), min_stack);
  }
#endif
  
  // Initial state is ALLOCATED but not INITIALIZED
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    osthread->set_state(ALLOCATED);
    osthread->set_state_under_suspend(ALLOCATED);
  }
  
  if (os::Solaris::_os_thread_count > os::Solaris::_os_thread_limit) {
    // We got lots of threads. Check if we still have some address space left.
    // Need to be at least 5Mb of unreserved address space. We do check by
    // trying to reserve some.
    const size_t VirtualMemoryBangSize = 20*K*K;
    char* mem = os::reserve_memory(VirtualMemoryBangSize);
    if (mem == NULL) { 
      delete osthread;    
      return false;
    } else {
      // Release the memory again
      os::release_memory(mem, VirtualMemoryBangSize);
    }
  }

  // Setup osthread because the child thread may need it.
  thread->set_osthread(osthread);

  // Create the Solaris thread
  thread_t tid;
  long     flags = (UseDetachedThreads ? THR_DETACHED : 0) | THR_SUSPENDED
                   | ((UseBoundThreads || (is_vm_thread && BackgroundCompilation)) ? THR_BOUND : 0);
  int      status;

  // On multiprocessors systems, libthread sometimes under-provisions our
  // process with LWPs.  On a 30-way systems, for instance, we could have
  // 50 user-level threads in ready state and only 2 or 3 LWPs assigned
  // to our process.  This can result in under utilization of PEs. 
  // I suspect the problem is related to libthread's LWP
  // pool management and to the kernel's SIGBLOCKING "last LWP parked" 
  // upcall policy. 
  // 
  // The following code is palliative -- it attempts to ensure that our 
  // process has sufficient LWPs to take advantage of multiple PEs.
  // Proper long-term cures include using user-level threads bound to LWPs 
  // (THR_BOUND) or using LWP-based synchronization.  Note that there is a 
  // slight timing window with respect to sampling _os_thread_count, but
  // the race is benign.  Also, we should periodically recompute
  // _processors_online as the min of SC_NPROCESSORS_ONLN and the 
  // the number of PEs in our partition.  You might be tempted to use
  // THR_NEW_LWP here, but I'd recommend against it as that could
  // result in undesirable growth of the libthread's LWP pool.
  // The fix below isn't sufficient; for instance, it doesn't take into count
  // LWPs parked on IO.  It does, however, help certain CPU-bound benchmarks.
  //
  // Some pathologies this scheme doesn't handle:
  // *	Threads can block, releasing the LWPs.  The LWPs can age out. 
  //	When a large number of threads become ready again there aren't
  //	enough LWPs available to service them.  This can occur when the
  //	number of ready threads oscillates. 
  // *	LWPs/Threads park on IO, thus taking the LWP out of circulation.
  // 
  // Finally, we should call thr_setconcurrency() periodically to refresh
  // the LWP pool and thwart the LWP age-out mechanism. 
  // The "+3" term provides a little slop -- we want to slightly overprovision.

  if (AdjustConcurrency && os::Solaris::_os_thread_count < (_processors_online+3)) { 
	if (!(flags & THR_BOUND)) { 
		thr_setconcurrency (os::Solaris::_os_thread_count) ; 	// avoid starvation
	}
  }

  status = thr_create(NULL, stack_size, _start, thread, flags, &tid);
  if (status != 0) {
    if (Verbose || WizardMode) {
      perror("os::create_thread");
    }
    thread->set_osthread(NULL);
    // Need to clean up stuff we've allocated so far
    delete osthread;
    return false;
  }
  
  atomic::increment(&os::Solaris::_os_thread_count);
  
  // Store info on the Solaris thread into the OSThread
  osthread->set_thread_id(tid);

  // Initial thread state is INITIALIZED, not SUSPENDED
  {
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    osthread->set_state(INITIALIZED);
    osthread->set_state_under_suspend(INITIALIZED);
  }
  
  // The thread is returned suspended (in state INITIALIZED), and is started higher up in the call chain
  return true;
}

debug_only(static bool signal_sets_initialized = false);
static sigset_t unblocked_sigs, vm_sigs;

static void signal_sets_init() {
  // Should also have an assertion stating we are still single-threaded.
  assert(!signal_sets_initialized, "Already initialized");
  // Fill in signals that are necessarily unblocked for all threads in
  // the VM. Currently, we unblock the following signals:
  // SHUTDOWN{1,2,3}_SIGNAL: for shutdown hooks support (unless over-ridden
  //                         by -Xrs (=ReduceSignalUsage));
  // BREAK_SIGNAL which is unblocked only by the VM thread and blocked by all
  // other threads. The "ReduceSignalUsage" boolean tells us not to alter
  // the dispositions or masks wrt these signals.
  // Programs embedding the VM that want to use the above signals for their
  // own purposes must, at this time, use the "-Xrs" option to prevent
  // interference with shutdown hooks and BREAK_SIGNAL thread dumping.
  // (See bug 4345157, and other related bugs).
  // In reality, though, unblocking these signals is really a nop, since
  // these signals are not blocked by default.
  sigemptyset(&unblocked_sigs);
  if (!ReduceSignalUsage) {
    sigaddset(&unblocked_sigs, SHUTDOWN1_SIGNAL);
    sigaddset(&unblocked_sigs, SHUTDOWN2_SIGNAL);
    sigaddset(&unblocked_sigs, SHUTDOWN3_SIGNAL);
  }
  // Fill in signals that are blocked by all but the VM thread.
  sigemptyset(&vm_sigs);
  if (!ReduceSignalUsage)
    sigaddset(&vm_sigs, BREAK_SIGNAL);
  debug_only(signal_sets_initialized = true);
}

// These are signals that are unblocked while a thread is running Java.
// (For some reason, they get blocked by default.)
sigset_t* os::Solaris::unblocked_signals() {
  assert(signal_sets_initialized, "Not initialized");
  return &unblocked_sigs;
}

// These are the signals that are blocked while a (non-VM) thread is
// running Java. Only the VM thread handles these signals.
static sigset_t* vm_signals() {
  assert(signal_sets_initialized, "Not initialized");
  return &vm_sigs;
}

// First crack at OS-specific initialization, from inside the new thread.
void os::initialize_thread() {
  // This is used to force the primordial thread to map enough stack so
  // we can install guard pages. To make things more consistent, we only
  // map ThreadStackSize worth of stack, even though the main thread could
  // map more.
  if (thr_main()) {
    JavaThread* jt = (JavaThread *)Thread::current();
    assert(jt != NULL,"Sanity check");
    // Use 2MB to allow for Solaris 7 64 bit mode.
    size_t stack_size = (JavaThread::stack_size_at_create() == 0) ?
                              2048*K : JavaThread::stack_size_at_create();
    if (stack_size > jt->stack_size()) {
      NOT_PRODUCT(
        struct rlimit limits;
        getrlimit(RLIMIT_STACK, &limits);
        assert(limits.rlim_cur == jt->stack_size(),
               "Stack size problem in main thread");
      )
      tty->print_cr(
        "Stack size of %d Kb exceeds current limit of %d Kb.\n"
        "(Stack sizes are rounded up to a multiple of the system page size.)\n"
        "See limit(1) to increase the stack size limit.",
        stack_size / K, jt->stack_size() / K);
      os::exit(1);
    }
    assert(jt->stack_size() >= stack_size,
          "Attempt to map more stack than was allocated");
    address stk_end = (address)((unsigned int)jt->stack_base() -
                                (stack_size - 16));
    Thread::force_stack_page_mapping_to(stk_end);

    jt->set_stack_size(stack_size);
  }

  // If using BOUND threads, install an alternate signal stack.
  // Because alternate stacks associate with LWPs on Solaris,
  // see sigaltstack(2), if using UNBOUND threads, we prefer
  // to explicitly stack bang. The alternative is to interpose
  // on _lwp_create to associate an alt sig stack with each LWP,
  // and this could be a problem when the JVM is embedded.
  if (UseBoundThreads) {
    os::Solaris::install_alternate_signal_stack();
  } else {
    // We already checked this in Arguments::check_vm_args_consistency()
    // but this is just re-emphasizes the reason for that check.
    assert(UseStackBanging, "Must stack bang when -UseBoundThreads");
  }

  thr_sigsetmask(SIG_UNBLOCK, os::Solaris::unblocked_signals(), NULL);
  if (!ReduceSignalUsage) {
    if (Thread::current()->is_VM_thread()) {
      // Only the VM thread handles BREAK_SIGNAL ...
      thr_sigsetmask(SIG_UNBLOCK, vm_signals(), NULL);
    } else {
      // ... all other threads block BREAK_SIGNAL
      thr_sigsetmask(SIG_BLOCK, vm_signals(), NULL);
    }
  }
  os::Solaris::init_thread_fpu_state();
}



// Free Solaris resources related to the OSThread
void os::free_thread(OSThread* osthread) {
  assert(osthread != NULL, "os::free_thread but osthread not set");
  // We are told to free resources of the argument thread, 
  // but we can only really operate on the current thread.
  guarantee(Thread::current()->osthread() == osthread, "os::free_thread but not current thread");
  if (UseBoundThreads) {
    os::Solaris::remove_alternate_signal_stack();
  }
  delete osthread;
}


void os::exit_thread() {
  free_thread(Thread::current()->osthread());

  thr_exit(NULL);

  ShouldNotReachHere();
}

 // ***************************************************************
 // Platform dependent Suspend and Resume
 // ***************************************************************

// this is for the normal thr_suspend case for non-self suspension
// Returns 0 for success
long os::pd_suspend_thread(Thread* thread, bool fence) {
    // fence off shared mutexes prior to suspension of target to avoid
    // any deadlocks
    OSThread* osthread = thread->osthread();
    long sts;
    if (fence) {
      ThreadCritical tc;

      sts = thr_suspend(osthread->thread_id());
    } else {
      sts = thr_suspend(osthread->thread_id());
    }
    return sts;
}



// Resume a thread by one level.  
// Return 0 for success
// Nesting of suspend/resume is handled
// by Thread::suspend_thread_impl, Thread::resume_thread_impl

// resume_thread

long os::pd_resume_thread(Thread* thread) {
    return thr_continue(thread->osthread()->thread_id()); 
}

// Part I of self-suspension. 
// Part I on Solaris does nothing
// Then return to the caller who will do a wait on the Interrupt_Lock
// as the common self-suspension implementation.
// Part III on Solaris does nothing.
bool os::pd_pre_self_suspend(OSThread* osthread) {
   return true;
}

// Part III of self-suspension. 
void os::pd_post_self_suspend(OSThread* osthread) {
}

// The INITIALIZED state is distinguished from the SUSPENDED state because the
// conditions in which a thread is first started are different from those in which
// a suspension is resumed.  These differences make it hard for us to apply the
// tougher checks when starting threads that we want to do when resuming them.
// However, when start_thread is called as a result of Thread.start, on a Java
// thread, the operation is synchronized on the Java Thread object.  So there
// cannot be a race to start the thread and hence for the thread to exit while
// we are working on it.  Non-Java threads that start Java threads either have
// to do so in a context in which races are impossible, or should do appropriate
// locking.

void os::start_thread(Thread* thread) {
  MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag); // guards suspend/resume
  assert(thread->osthread()->state() == INITIALIZED || 
	 thread->osthread()->state_before_suspend() == INITIALIZED, "can only start thread in INITIALIZED state");

  // A thread may be suspended in the presence of the profiler. If so, it will also resume it.
  if(thread->osthread()->suspend_depth() > 0) {
    // transition into a "true" SUSPENDED state
    thread->osthread()->set_state(SUSPENDED);
    thread->osthread()->set_state_before_suspend(RUNNABLE);
    thread->osthread()->set_physically_suspended();
    return;
  }

  thread->osthread()->set_state(RUNNABLE);
  int status = thr_continue(thread->osthread()->thread_id());
  assert(status == 0, "thr_continue failed");

  // the new thread's state is set to RUNNABLE in _start() when it is actually resumed
  // the child thread's state should not be modified by the parent at or after this point.
}

int os::current_thread_id() {
  Unimplemented();  return 0;
}

int os::current_process_id() {
  return getpid();
}

int os::allocate_thread_local_storage() {
  // %%%       in Win32 this allocates a memory segment pointed to by a
  //           register.  Dan Stein can implement a similar feature in
  //           Solaris.  Alternatively, the VM can do the same thing
  //           explicitly: malloc some storage and keep the pointer in a
  //           register (which is part of the thread's context) (or keep it
  //           in TLS).
  // %%%       In current versions of Solaris, thr_self and TSD can
  //	       be accessed via short sequences of displaced indirections.
  //	       The value of thr_self is available as %g7(36).
  //	       The value of thr_getspecific(k) is stored in %g7(12)(4)(k*4-4),
  //	       assuming that the current thread already has a value bound to k.
  //	       It may be worth experimenting with such access patterns,
  //	       and later having the parameters formally exported from a Solaris
  //	       interface.  I think, however, that it will be faster to
  //	       maintain the invariant that %g2 always contains the
  //	       JavaThread in Java code, and have stubs simply
  //	       treat %g2 as a caller-save register, preserving it in a %lN.
  thread_key_t tk;
  if (thr_keycreate( &tk, NULL ) )
    fatal1("os::allocate_thread_local_storage: thr_keycreate failed (%s)", strerror(errno));
  return int(tk);
}

void os::free_thread_local_storage(int index) {
  // %%% don't think we need anything here
  // if ( pthread_key_delete((pthread_key_t) tk) )
  //   fatal("os::free_thread_local_storage: pthread_key_delete failed");
}

void os::thread_local_storage_at_put(int index, void* value) {
  // %%% this is used only in threadLocalStorage.cpp
  if (thr_setspecific((thread_key_t)index, value))
    fatal1("os::thread_local_storage_at_put: thr_setspecific failed (%s)", strerror(errno));
}

void* os::thread_local_storage_at(int index) {
  // %%% this is used only in threadLocalStorage.cpp
  void* r = NULL;
  if (thr_getspecific((thread_key_t)index, &r))
    fatal1("os::thread_local_storage_at: thr_getspecific failed (%s)", strerror(errno));
  return r;
}


// Time since start-up in seconds to a fine granularity.
// Used by VMSelfDestructTimer and the MemProfiler.
double os::elapsedTime() {
  return (double)(gethrtime() - first_hrtime) / (double)hrtime_hz;
}

jlong os::elapsed_counter() {
  return gethrtime() - first_hrtime;
}

jlong os::elapsed_frequency() {
   return hrtime_hz;
}

jlong os::javaTimeMillis() {
  timeval t;
  static const char* aNull = 0;
  if (gettimeofday( &t, &aNull) == -1)
    fatal1("os::javaTimeMillis: gettimeofday (%s)", strerror(errno));
  return jlong(t.tv_sec) * 1000  +  jlong(t.tv_usec) / 1000;
}

//
// On Solaris (2.6 at least), we have a problem: threads (including
// daemon threads) continue to run during exit, causing all sorts of
// bad results.  Jump through some hoops to try to avoid crashing.

class VM_Exit: public VM_Operation {
 private:
  int     _code;
  Thread* _thread;
 public:
  VM_Exit(int code, Thread* thread) {
    _code   = code; 
    _thread = thread;
  }  
  void doit() {
    // make doubly sure that a thread doesn't wake up at an inconvenient time
    // Note: this is necessary or the threads continue to run
    assert(Threads_lock->owner() == Thread::current(), "must have threads lock");
    for(JavaThread *thread = Threads::first(); thread != NULL; thread = thread->next ()) {
      thread->shutdown_suspend();
    }

    // Check for exit hook
    exit_hook_t exit_hook = Arguments::exit_hook();
    if (exit_hook != NULL) {
      exit_hook(_code);
    } else {
      ::exit(_code);
    }
  }
  const char* name() const { return "exit"; }
};

void os::exit(int code) {  
  Thread* thread = ThreadLocalStorage::thread_index() == 0 ? NULL
    : ThreadLocalStorage::get_thread_slow();
  if (thread == NULL) {
    // we have serious problems -- just exit
    ::exit(code);
  }

  // Fire off a VM_Operation to suspend all remaining threads
  if (VMThread::vm_thread() != NULL) {
    VM_Exit op(code, thread);
    // %%%%% integrate this with vm_exit !
    if (thread->is_Java_thread())
      ((JavaThread*)thread)->set_thread_state(_thread_in_vm);
    VMThread::execute(&op);
  } else {
    ::exit(code);
  }
}

void os::abort(bool dump_core) {

  if (UseISM) os::release_memory_special();
  
  // Check for abort hook
  abort_hook_t abort_hook = Arguments::abort_hook();
  if (abort_hook != NULL) {
    abort_hook();
  }
  if (dump_core) {
    NOT_PRODUCT(tty->print_cr("Dumping core...."));
    struct sigaction noAct;
    memset(&noAct, 0, sizeof(noAct));
    noAct.sa_handler = SIG_DFL;
    sigaction(SIGILL, &noAct, (struct sigaction*)NULL);
    ::abort(); // dump core (for debugging)
  }
  ::exit(1);
}

// DLL functions

const char* os::dll_file_extension() { return ".so"; }

const char* os::get_temp_directory() { return "/tmp/"; }

// Returns true if the given PC was in the VM DLL; otherwise returns false.
bool os::dll_address_is_in_vm(address addr) {
  // These tests should be unified with the code in find(int, bool) in
  // debug.cpp.

  // Check generated code
  if (AbstractInterpreter::contains(addr)) {
    return true;
  }

  if (StubRoutines::contains(addr)) {
    return true;
  }

#ifndef CORE
  if (CodeCache::contains(addr)) {
    return true;
  }

  if (VtableStubs::contains(addr)) {
    return true;
  }
#endif

  // Check if inside libjvm
  Dl_info dlinfo;
  if (dladdr(CAST_FROM_FN_PTR(void *, os::dll_address_is_in_vm), &dlinfo)){
      void * fbase = dlinfo.dli_fbase;
      assert(fbase !=NULL, "Cannot obtain base address for libjvm");
      if (dladdr((void *)addr, &dlinfo)){
          if (fbase == dlinfo.dli_fbase) return true;
      }
  }
  else assert(NULL != NULL, "Cannot obtain base address for libjvm");

  return false;
}


// Returns the symbol nearest to the specified addres or NULL if it's 
// is outside all known mapped object. Used by profiler
const char* os::dll_address_to_function_name(address addr){
  Dl_info dlinfo;
  const char* symbol = (char *)NULL;

  if(dladdr((void *)addr, &dlinfo)){
    symbol = dlinfo.dli_sname;
  }
  return symbol;
}

// Returns the name of the library that contains the specified addres
// or NULL if it's outside all known mapped object.
const char* os::dll_address_to_library_name(address addr){
  Dl_info dlinfo;
  const char* lib = (char *)NULL;

  if (dladdr((void*)addr, &dlinfo)){
     lib = dlinfo.dli_fname;
  }
  return lib;
}

// Map PC to line number. Not implemented for Solaris
int os::dll_address_to_line_number(address addr, char ** filename){
  if (filename) *filename = NULL;
  return -1;
}

// Prints the names and full paths of all opened dynamic libraries
// for current process
void os::dump_dll_info(outputStream * st) {
    Dl_info dli;
    void *handle;
    Link_map *map;
    Link_map *p;

    st->print_cr("Dynamic libraries:"); st->flush();

    if (!dladdr(CAST_FROM_FN_PTR(void *, os::dump_dll_info), &dli)) {
        st->print_cr("Error: Cannot print dynamic libraries: os::dump_dll_info() must be inside a DLL");
        return;
    }
    handle = dlopen(dli.dli_fname, RTLD_LAZY);
    if (handle == NULL) {
        st->print_cr("Error: Cannot print dynamic libraries.");
        return;
    }
    dlinfo(handle, RTLD_DI_LINKMAP, &map);
    if (map == NULL) {
        st->print_cr("Error: Cannot print dynamic libraries.");
        return;
    }

    while (map->l_prev != NULL)
        map = map->l_prev;

    while (map != NULL) {
        st->print_cr("0x%p \t%s", map->l_addr, map->l_name);
        map = map->l_next;
    }

    dlclose(handle);
}

static char* saved_jvm_path = NULL;


// Find the pathname JAVA_HOME/lib/<arch>/hotspot/libjvm{,_g}.so.
//
// In the debugging build, the libjvm_g.so might be in a non-standard
// place, so we might determine JAVA_HOME by other means, and append a
// fake suffix hotspot/libjvm_g.so.
//
void os::jvm_path(char *buf, jint buflen) {
  //char buf[MAXPATHLEN];
  assert(buflen >= MAXPATHLEN, "must use a large-enough buffer");

  if (saved_jvm_path != NULL) {
    strcpy(buf, saved_jvm_path);
    return;
  }

  Dl_info dlinfo;
  dladdr(CAST_FROM_FN_PTR(void *, os::jvm_path), &dlinfo);

  #ifdef PRODUCT
  realpath((char *)dlinfo.dli_fname, buf);

  #else //PRODUCT

  char cpu_arch[12];

  sysinfo(SI_ARCHITECTURE, cpu_arch, sizeof(cpu_arch));

  strcpy(buf, (char *)dlinfo.dli_fname);

  // if it has no directory component, and does not exist, walk $PATH,
  // under the assumption that it is really an a.out
  if (strchr(buf, '/') == NULL && 0 != access(buf, F_OK)) {
    char* tpath = ::getenv("PATH");
    const char * path = (!tpath) ? "" : tpath;
    char pathbuf[MAXPATHLEN*2];
    for (const char* pp = path; pp && *pp; pp = strchr(pp, ':')) {
      if (*pp == ':')  pp++;
      strcpy(pathbuf, pp);
      char* ep = strchr(pathbuf, ':');
      if (!ep)  ep = pathbuf + strlen(pathbuf);
      sprintf(ep, "/%s", buf);
      if (0 == access(pathbuf, X_OK)) {
	strcpy(buf, pathbuf);
	break;
      }
    }
  }

  realpath((char *)dlinfo.dli_fname, buf);
  char* java_home_var = ::getenv("JAVA_HOME");

  if (java_home_var != NULL && java_home_var[0] != 0) {
    realpath(java_home_var, buf);
    // first try $JAVA_HOME/jre/lib/<arch>
    sprintf(buf+strlen(buf), "/jre/lib/%s", cpu_arch);
    if (0 == access(buf, F_OK)) {
      sprintf(buf+strlen(buf), "/hotspot/libjvm%s.so", debug_only("_g")"");
    } else {
      realpath(java_home_var, buf);
      // now try $JAVA_HOME/build/solaris/lib/<arch>
      sprintf(buf+strlen(buf), "/build/solaris/lib/%s", cpu_arch);
      if (0 == access(buf, F_OK)) {
        sprintf(buf+strlen(buf), "/hotspot/libjvm%s.so", debug_only("_g")"");
      } else {
        // Go back to path of .so
        realpath((char *)dlinfo.dli_fname, buf);
      }
    }
  } 
  #endif // PRODUCT

  saved_jvm_path = (char*) malloc(strlen(buf)+1);
  strcpy(saved_jvm_path, buf);
}

 
void os::print_jni_name_prefix_on(outputStream* st, int args_size) {
  // no prefix required, not even "_"
}


void os::print_jni_name_suffix_on(outputStream* st, int args_size) {
  // no suffix required
}


// sun.misc.Signal

extern "C" {
  static void UserHandler(int sig, void *siginfo, void *context) {
    os::signal_notify(sig);
    // We do not need to reinstate the signal handler each time...
  }
}

void* os::user_handler() {
  return CAST_FROM_FN_PTR(void*, UserHandler);
}
extern "C" {
  typedef void       (*sa_handler_t)(int);
  typedef void      (*sa_sigaction_t)(int, siginfo_t *, void *);
}

void* os::signal(int signal_number, void* handler) {
  struct sigaction sigAct, oldSigAct;
  sigfillset(&(sigAct.sa_mask));
  sigAct.sa_flags = SA_RESTART & ~SA_RESETHAND;
  sigAct.sa_handler = CAST_TO_FN_PTR(sa_handler_t, handler);

  if (sigaction(signal_number, &sigAct, &oldSigAct))
    fatal2("sigaction: cannot set signal %d (%s)", signal_number, strerror(errno));

  return CAST_FROM_FN_PTR(void*, oldSigAct.sa_handler);
}

void os::signal_raise(int signal_number) {
  raise(signal_number);
}

/*
 * The following code is moved from os.cpp for making this
 * code platform specific, which it is by its very nature.
 */

// a counter for each possible signal value
static int pending_signals[NSIG+1] = { 0 };

void os::signal_init_pd() {
  // Initialize signal structures
  memset(pending_signals, 0, sizeof(pending_signals));
}

void os::signal_notify(int signal_number) {
#ifdef ASSERT
  // We are not guarenteed that the thread calling this entry
  // point is a known thread. In debug mode the Mutex code
  // expects a Thread to be present for various assertions
  // to pass, so we make one available (debug mode only).
  Thread* fake_thread = NULL;
  if (ThreadLocalStorage::thread() == NULL) {
    fake_thread = new Thread();
    fake_thread->initialize_thread_local_storage();
  }
#endif
  { 
    MutexLocker mu(Signal_lock);
    pending_signals[signal_number]++;
    Signal_lock->notify_all();
  }
#ifdef ASSERT
  if (fake_thread != NULL) {
    delete fake_thread;
  }
#endif
}   

static int check_pending_signals(bool wait_for_signal) {
  int i;
  MutexLocker mu(Signal_lock);
  while (true) {
    for (i = 0; i < NSIG + 1; i++) {
      if (pending_signals[i]) {
        pending_signals[i]--;
        return i;
      }
    }
    if (!wait_for_signal) {
      return -1;
    }
    Signal_lock->wait();
  }
}

int os::signal_lookup() {
  return check_pending_signals(false);
}

int os::signal_wait() {
  return check_pending_signals(true);
}

// Virtual Memory

static int page_size = -1;

int os::vm_page_size() {
  assert(page_size != -1, "must call os::init");
  return page_size;
}

// The mmap functions commit_memory, uncommit_memory, 
// release_memory, guard memory, unguard memory just return true for
// objects allocated in ISM
bool os::commit_memory(char* addr, size_t bytes) {
  size_t size = bytes;
  if ((UseISM || UsePermISM) && os::Solaris::withinISMRange(addr, bytes)) {
    return true;
  } else {
    return 
      NULL != Solaris::mmap_chunk( addr, size, MAP_PRIVATE|MAP_FIXED,
				   PROT_READ | PROT_WRITE | PROT_EXEC );
  }
}


bool os::uncommit_memory(char* addr, size_t bytes) {
  size_t size = bytes;
  if ((UseISM || UsePermISM) && os::Solaris::withinISMRange(addr, bytes)) {
    return true;
  } else {
    return
      NULL != Solaris::mmap_chunk(addr, size, MAP_PRIVATE|MAP_FIXED|MAP_NORESERVE, 
				  PROT_READ | PROT_WRITE | PROT_EXEC );
  }
}

bool os::release_memory(char* addr, size_t bytes) {
  size_t size = bytes;
  if ((UseISM || UsePermISM) && os::Solaris::withinISMRange(addr, bytes)) {
    return true;
  } else {
    return munmap(addr, size) == 0;
  }
}

bool os::guard_memory(char* addr, size_t bytes) {
  if ((UseISM || UsePermISM) && os::Solaris::withinISMRange(addr, bytes)) {
    fatal("guard_memory should not happen in ISM");
  } else {
    int retVal = mprotect(addr, (size_t)bytes, PROT_NONE);
    return retVal == 0;
  }
}

bool os::unguard_memory(char* addr, size_t bytes) {
  if ((UseISM || UsePermISM) && os::Solaris::withinISMRange(addr, bytes)) {
    return true;
  } else {
    int retVal = mprotect(addr, (size_t)bytes, PROT_READ|PROT_WRITE|PROT_EXEC);
    return retVal == 0;
  }
}

// ISM init
char* os::Solaris::_ism_start_addr = NULL;
char* os::Solaris::_ism_end_addr = NULL;
int os::Solaris::_ism_shmid = -1;

// Checks to see if the address is within the ISM allocated region
bool os::Solaris::withinISMRange(char* addr, size_t size) {
  // Make sure address and size are within ISM segment
  if ( (os::Solaris::ism_start_addr() <= addr) && 
       (addr + size <= os::Solaris::ism_end_addr()) ) {
    return true;
  } else {
    assert(addr + size <= os::Solaris::ism_start_addr() ||
	   addr >= os::Solaris::ism_end_addr(), "overlaps ISM segment");
    return false;
  }
}

char* os::reserve_memory_special(size_t bytes) {
  size_t size = bytes;
  char* retAddr = NULL;
  int shmid;
  key_t ismKey;

  // If more than one ISM call occurs, assert should fail
  assert(os::Solaris::ism_shmid() == -1, "ISM should only be called once");
  
  // UsePermISM will use the file and int combo to determine the key
  // so that subsequent use of permanent ISM will be mapped to the same
  // memory region. Only using UsePermISM, only 1 VM can be running
  // at the same time or else the VMs will try to share the memory region.
  if (UsePermISM) {
    ismKey = ftok("/dev/zero", 23098);
  } else {
    ismKey = IPC_PRIVATE;
  }
  // Create a large shared memory region to attach to based on size.
  // Currently, size is the total size of the heap
  shmid = shmget(ismKey, size, SHM_R | SHM_W | IPC_CREAT);
  if (shmid == -1){
    fatal1("Could not get ISM region: %s", strerror(errno));
  }
  // Save the shmid for future removal of ISM from system
  os::Solaris::set_ism_shmid(shmid);
      
  // Attach to the region
  retAddr = (char *) shmat(shmid, 0, SHM_SHARE_MMU | SHM_R | SHM_W);
  if (retAddr == (char *) -1) {
    fatal1("Could not attach to ISM region: %s", strerror(errno));
  }
      
  // Save the start and end address for the ISM region based on the size
  // of the heap. This is done so that future mmap calls for addresses
  // within the ISM region are ignored.  The end address needs to be 
  // a multiple of 4MB.
  os::Solaris::set_ism_start_addr(retAddr);
  os::Solaris::set_ism_end_addr((char *) (retAddr + round_to(size, 4*M)));
  
  return retAddr;
}

// Removes ISM from the system.
void os::release_memory_special() {
  int shmid = os::Solaris::ism_shmid();
  if (shmid >= 0) {
    shmid_ds *buf = NULL;
    if (shmctl(shmid, IPC_RMID, buf) != 0) {
      warning("Unable to remove ISM segment with shmid=%d", shmid);
    }
  }
}

static int os_sleep(jlong millis) {
  const jlong limit = INT_MAX;
  while (millis > limit) {
    if (os_sleep(limit) == -1)
      return -1;
    millis -= limit;
  }

  struct timeval tv;
  tv.tv_sec  = millis / 1000;
  tv.tv_usec = (millis % 1000) * 1000;
  return select(0, NULL, NULL, NULL, &tv);
}

int os::Solaris::naked_sleep() {
  // %% make the sleep time an interger flag. for now use 1 millisec.
  return os_sleep(1);
}

int os::sleep(Thread* thread, jlong millis, bool interruptible) {
  assert(thread == Thread::current(),  "thread consistency check");

  // On Solaris machines (especially 2.5.1) we found that sometimes the VM gets into a live lock
  // situation with a JavaThread being starved out of a lwp. The kernel doesn't seem to generate
  // a SIGWAITING signal which would enable the threads library to create a new lwp for the starving
  // thread. We suspect that because the Watcher thread keeps waking up at periodic intervals the kernel
  // is fooled into believing that the system is making progress. In the code below we block the
  // the watcher thread while safepoint is in progress so that it would not appear as though the
  // system is making progress.
  if (thread->is_Watcher_thread() && SafepointSynchronize::is_synchronizing() && !Arguments::has_profile()) {
    // We now try to acquire the threads lock. Since this lock is held by the VM thread during
    // the entire safepoint, the watcher thread will  line up here during the safepoint.
    Threads_lock->lock_without_safepoint_check();
    Threads_lock->unlock(); 
  }

  if (millis <= 0) {
    // NOTE: workaround for bug 4338139
    if (thread->is_Java_thread()) {
      StateSaver sv(thread);
      ThreadBlockInVM tbiv((JavaThread*) thread);
      thr_yield();
      return 0;
    }

    thr_yield();
    return 0;
  }

  OSThreadWaitState osts(thread->osthread());

  if (!interruptible) {
    // Following assert fails for os::yield_all:
    //assert(!thread->is_Java_thread(), "must not be java thread");
    return os_sleep(millis);
  }

  assert(thread->is_Java_thread(), "must be java thread");
  INTERRUPTIBLE_RETURN_INT(os_sleep(millis), os::Solaris::clear_interrupted);
  return 0;
}

// Used to convert frequent JVM_Yield() to nops
bool os::dont_yield() {
  if (DontYieldALot) {  
    static hrtime_t last_time = 0;
    hrtime_t diff = gethrtime() - last_time;
    
    if (diff < DontYieldALotInterval * 1000000)
      return true;
    
    last_time += diff;
    
    return false;
  }
  else {
    return false;
  }
}

void os::yield() {  
  // Yields to all threads with same or greater priority
  os::sleep(Thread::current(), 0, false);
}


// On Solaris we found that yield_all doesn't always yield to all other threads.
// There have been cases where there is a thread ready to execute but it doesn't
// get an lwp as the VM thread continues to spin with sleeps of 1 millisecond.
// The 1 millisecond wait doesn't seem long enough for the kernel to issue a
// SIGWAITING signal which will cause a new lwp to be created. So we count the
// number of times yield_all is called in the one loop and increase the sleep
// time after 8 attempts. If this fails too we increase the concurrency level
// so that the starving thread would get an lwp

void os::yield_all(int attempts) {
  // Yields to all threads, including threads with lower priorities
  if (attempts == 0) {
    os::sleep(Thread::current(), 1, false);
  } else {
    int iterations = attempts % 30;
    if (iterations == 0) {
      int noofLWPS = thr_getconcurrency();
      if (noofLWPS < (Threads::number_of_threads() + 2)) {
        thr_setconcurrency(thr_getconcurrency() + 1);
      }
    } else if (iterations < 25) {
      os::sleep(Thread::current(), 1, false);
    } else {
      os::sleep(Thread::current(), 10, false);
    }
  }
}

// Called from the tight loops to possibly influence time-sharing heuristics
void os::loop_breaker(int attempts) {
  os::yield_all(attempts);
}

// see thr_setprio(3T) for the basis of these numbers
#define T_PRI_MIN    0
#define T_PRI_NORM  64
#define T_PRI_MAX  127

// Solaris only gives access to 128 real priorities at a time,
// so we expand Java's ten to fill this range.  This would be better
// if we dynamically adjusted relative priorities. 

int os::java_to_os_priority[MaxPriority + 1] = {
  -99999,         // 0 Entry should never be used

  0,              // 1 MinPriority
  10,             // 2
  20,             // 3

  63,             // 4
  64,             // 5 NormPriority
  65,             // 6

  90,             // 7
  100,            // 8
  110,            // 9 NearMaxPriority

  127             // 10 MaxPriority
};


OSReturn os::set_native_priority(Thread* thread, int newpri) {
  assert(newpri >= T_PRI_MIN && newpri <= T_PRI_MAX, "bad priority mapping");
  int status = thr_setprio(thread->osthread()->thread_id(), newpri);
  return (status == 0) ? OS_OK : OS_ERR;
}


OSReturn os::get_native_priority(Thread* thread, int *priority_ptr) {
  int p;
  int status = thr_getprio(thread->osthread()->thread_id(), &p);
  if (status != 0) {
    assert(false, "thr_getprio failed");
    return OS_ERR;
  }
  *priority_ptr = p;
  return OS_OK;
}


// Interrupt support

int os::Solaris::send_interrupt(Thread* thread) {
  OSThread* osthread = thread->osthread();

  // interrupting the VM thread is a special case
  if(thread->is_VM_thread()){
    assert(osthread->interrupt_in_progress() == false, "Nested signals to VM thread");
    osthread->set_interrupt_in_progress(true);
    int status = thr_kill(osthread->thread_id(), INTERRUPT_SIGNAL);
    assert(status == 0, "thr_kill != 0");
    
    return status;    
  }
  else {
    if (osthread->send_interrupt()) {
      int status = thr_kill(osthread->thread_id(), INTERRUPT_SIGNAL);
      assert(status == 0, "thr_kill != 0");
      
      return status;
    }
  }
  
  return 0;
}

// From the man page pthread_setschedparam(3T):
// 
// By default, the scope for newly-created threads are unbound,
// or intra-process, and their setting is PTHREAD_SCOPE_PROCESS
// or NULL. An unbound thread  is  scheduled  by  libthread  or
// libpthread  on  an underlying LWP, which competes with other
// LWPs in the same process.
// 
// The following dynamic scheduling functions  should  be  used
// only   with   unbound   threads:    pthread_setschedparam(),
// pthread_getschedparam(), thr_setprio(), and thr_getprio().
// 
inline int OSThread_priority(OSThread* t) {
  int prio = 0;
  int sts = thr_getprio(t->thread_id(), &prio);
  assert(sts == 0, "getprio failed");
  return prio;
}
inline void OSThread_set_priority(OSThread* t, int prio) {
  int sts = thr_setprio(t->thread_id(), prio);
  assert(sts == 0, "setprio failed");
}


// Increase priority of a thread by one OS level, but not higher than
// the os-level corresponding to the given java priority
bool os::boost_priority(Thread *thread, int max_java_priority) {
  int cur_os_prio   = OSThread_priority(thread->osthread());
  int max_os_prio   = java_to_os_priority[max_java_priority];
  
  int new_os_prio = min(cur_os_prio + 1, T_PRI_MAX);

  // Check if boosting is allowed
  if (new_os_prio > max_os_prio) return false;

  OSThread_set_priority(thread->osthread(), new_os_prio);
  return true;
}

// Decrease priority of a thread by one OS level, but not lower than
// the os-level corresponding to the given java priority
bool os::deflate_priority(Thread *thread, int min_java_priority) {
  int cur_os_prio   = OSThread_priority(thread->osthread());
  int min_os_prio = java_to_os_priority[min_java_priority];

  int new_os_prio = max(cur_os_prio - 1, T_PRI_MIN);

  if (new_os_prio < min_os_prio) return false;

  OSThread_set_priority(thread->osthread(), new_os_prio);
  return true;
}

void os::interrupt(Thread* thread) {
  assert(Thread::current() == thread || Threads_lock->owned_by_self(), "possibility of dangling Thread pointer");

  OSThread* osthread = thread->osthread();

  if (!osthread->interrupted()) {
    osthread->set_interrupted(true);
    atomic::membar();
    osthread->interrupt_event()->set();
    if (osthread->jump_buffer() != NULL) {
      osthread->add_interrupt_callback(new Interrupt_Callback());
      os::Solaris::send_interrupt(thread);
      // callback is cancelled by the target 
    }
  }
}


bool os::is_interrupted(Thread* thread, bool clear_interrupted) {
  assert(Thread::current() == thread || Threads_lock->owned_by_self(), "possibility of dangling Thread pointer");

  OSThread* osthread = thread->osthread();

  bool res = osthread->interrupted();

  // NOTE that since there is no "lock" around these two operations,
  // there is the possibility that the interrupted flag will be
  // "false" but that the interrupt event will be set. This is
  // intentional. The effect of this is that Object.wait() will appear
  // to have a spurious wakeup, which is not harmful, and the
  // possibility is so rare that it is not worth the added complexity
  // to add yet another lock. It has also been recommended not to put
  // the interrupted flag into the os::Solaris::Event structure,
  // because it hides the issue.
  if (res && clear_interrupted) {
    osthread->set_interrupted(false);
    osthread->interrupt_event()->reset();
  }
  return res;
}

// Use signals to set up pc for a suspended thread
bool os::Solaris::set_thread_pc_slow(Thread* thread, ExtendedPC addr) {
  OSThread* osthread = thread->osthread();

  osthread->add_interrupt_callback(new SetThreadPC_Callback(addr));
  send_interrupt(thread);
  // callback is cancelled by the target
  return true;
}

// return true if successul, false otherwise
bool os::set_thread_pc(Thread* thread, ExtendedPC addr) {
  assert(thread->osthread()->state() == SUSPENDED, "must be suspended");

  // Fast version can be called (currently) on 5.6 or later
  if (os::Solaris::_thr_setstate != NULL && GetThreadState) {
    return os::Solaris::set_thread_pc_fast(thread, addr);
  }
  else {
    return os::Solaris::set_thread_pc_slow(thread, addr);
  }
}

ExtendedPC os::fetch_top_frame(Thread* thread, jint** ret_younger_sp, jint** ret_sp) {
  // There is a lurking race condition with setting osthread()->state(). 
  // We would like to profile JDK even before it is fixed.
  if(!ProfilerLight) {
    ThreadState state = thread->osthread()->state();
    assert(state == SUSPENDED || state == ALLOCATED || state == INITIALIZED, "must be suspended");
  }
  
  // Fast version can be called (currently) on 5.6 or later
  if (os::Solaris::_thr_getstate != NULL && GetThreadState) {
    return os::Solaris::fetch_top_frame_fast(thread, ret_younger_sp, ret_sp);
  }
  else {
    return os::Solaris::fetch_top_frame_slow(thread, ret_younger_sp, ret_sp);
  }
}


void os::print_statistics() {
}

int os::message_box(const char* title, const char* message) {
  fprintf(stderr, "%s\n%s\n", title, message);
  char buf[1024];
  // Prevent process from exiting upon "read error" without consuming all CPU
  while (fgets(buf, sizeof(buf), stdin) == NULL) { ::sleep (100); }
  return buf[0] == 'y'  ||  buf[0] == 'Y';
}


// grody low-level implem required because of malloc bootstrapping!
static  mutex_t  global_mut = {0}; // already initialized if all zeroes
static  thread_t global_mut_owner = -1;
static  int      global_mut_count = 0;
typedef void     ThreadCritical_construct_func ();
typedef void     ThreadCritical_destruct_func ();

static void ThreadCritical_construct_bootstrap() {
   assert (Threads::number_of_threads() == 0, "valid only during initialization");
}

static void ThreadCritical_destruct_bootstrap() {
   assert (Threads::number_of_threads() == 0, "valid only during initialization");
}

static void ThreadCritical_construct_regular() {
   thread_t owner = thr_self();
   if (global_mut_owner != owner) {
      if (os::Solaris::mutex_lock(&global_mut))
        fatal1("ThreadCritical::ThreadCritical: mutex_lock failed (%s)", strerror(errno));
      assert(global_mut_count == 0, "must have clean count");
      assert(global_mut_owner == -1, "must have clean owner");
   }
   global_mut_owner = owner;
   ++global_mut_count;
}

static void ThreadCritical_destruct_regular() {
   assert(global_mut_owner == thr_self(), "must have correct owner");
   assert(global_mut_count > 0, "must have correct count");
   --global_mut_count;
   if (global_mut_count == 0) {
     global_mut_owner = -1;
     if (os::Solaris::mutex_unlock(&global_mut))
        fatal1("ThreadCritical::~ThreadCritical: mutex_unlock failed (%s)", strerror(errno));
   }
}

static  ThreadCritical_construct_func *ThreadCritical_construct = ThreadCritical_construct_bootstrap;
static  ThreadCritical_destruct_func  *ThreadCritical_destruct = ThreadCritical_destruct_bootstrap;


void ThreadCritical::initialize() { 
}

void ThreadCritical::release() {
}

ThreadCritical::ThreadCritical() {
   (*ThreadCritical_construct)();
}

ThreadCritical::~ThreadCritical() {
   (*ThreadCritical_destruct)();
}



// This does not do anything on Solaris. This is basically a hook for being
// able to use structured exception handling (thread-local exception filters) on, e.g., Win32.
void os::os_exception_wrapper(java_call_t f, JavaValue* value, methodHandle* method, JavaCallArguments* args, Thread* thread) {
  f(value, method, args, thread);  
}


// Used for PostMortemDump
extern "C" void safepoints();
extern "C" void find(int x);
extern "C" void findpc(int x);
extern "C" void findpc(int x);
extern "C" void events();
extern void set_error_reported();
#ifndef PRODUCT
extern "C" void ps();
#endif

#ifndef PRODUCT
void continue_with_dump(void) {
  tty->print_cr("Dumping thread state before exit....");
  ps();
  fatal("Exiting after thread state dump.");
}
#endif


// This routine may be used by user applications as a "hook" to catch signals.
// The user-defined signal handler must pass unrecognized signals to this
// routine, and if it returns true (non-zero), then the signal handler must
// return immediately.  If the flag "abort_if_unrecognized" is true, then this
// routine will never retun false (zero), but instead will execute a VM panic
// routine kill the process.
//
// If this routine returns false, it is OK to call it again.  This allows
// the user-defined signal handler to perform checks either before or after
// the VM performs its own checks.  Naturally, the user code would be making
// a serious error if it tried to handle an exception (such as a null check
// or breakpoint) that the VM was generating for its own correct operation.
//
// This routine may recognize any of the following kinds of signals:
//    SIGBUS, SIGSEGV, SIGILL, SIGFPE, BREAK_SIGNAL, SIGPIPE, INTERRUPT_SIGNAL.
// It should be consulted by handlers for any of those signals.
//
// The caller of this routine must pass in the three arguments supplied
// to the function referred to in the "sa_sigaction" (not the "sa_handler")
// field of the structure passed to sigaction().  This routine assumes that
// the sa_flags field passed to sigaction() includes SA_SIGINFO and SA_RESTART.
//
// Note that the VM will print warnings if it detects conflicting signal
// handlers, unless invoked with the option "-XX:+AllowUserSignalHandlers".
//
extern "C" int JVM_handle_solaris_signal(int signo, siginfo_t* siginfo, void* ucontext, int abort_if_unrecognized);


void signalHandler(int sig, siginfo_t* info, void* ucVoid) {
  JVM_handle_solaris_signal(sig, info, ucVoid, true);
}


// This boolean allows users to forward their own non-matching signals
// to JVM_handle_solaris_signal, harmlessly.
bool os::Solaris::signal_handlers_are_installed = false;

// For signal-chaining
struct sigaction os::Solaris::sigact[MAXSIGNUM];
unsigned int os::Solaris::sigs = 0;
bool os::Solaris::libjsig_is_loaded = false;
typedef struct sigaction *(*get_signal_t)(int);
get_signal_t os::Solaris::get_signal_action = NULL;

struct sigaction* os::Solaris::get_chained_signal_action(int sig) {
  struct sigaction *actp = NULL;
 
  if (libjsig_is_loaded) {
    // Retrieve the old signal handler from libjsig
    actp = (*get_signal_action)(sig);
  }
  if (actp == NULL) {
    // Retrieve the preinstalled signal handler from jvm
    actp = get_preinstalled_handler(sig);
  }

  return actp;
}

bool os::Solaris::chained_handler(struct sigaction *actp, int sig,
                                  siginfo_t *siginfo, void *context) {
  // Call the old signal handler
  if (actp->sa_handler == SIG_DFL) {
    // It's more reasonable to let jvm treat it as an unexpected exception
    // instead of taking the default action.
    return false;
  } else if (actp->sa_handler != SIG_IGN) {
    if ((actp->sa_flags & SA_NODEFER) == 0) {
      // automaticlly block the signal
      sigaddset(&(actp->sa_mask), sig);
    }

    sa_handler_t hand;
    sa_sigaction_t sa;
    bool siginfo_flag_set = (actp->sa_flags & SA_SIGINFO) != 0;
    // retrieve the chained handler
    if (siginfo_flag_set) {
      sa = actp->sa_sigaction;
    } else {
      hand = actp->sa_handler;
    }

    if ((actp->sa_flags & SA_RESETHAND) != 0) {
      actp->sa_handler = SIG_DFL;
    }

    // try to honor the signal mask
    sigset_t oset;
    thr_sigsetmask(SIG_SETMASK, &(actp->sa_mask), &oset);

    // call into the chained handler
    if (siginfo_flag_set) {
      (*sa)(sig, siginfo, context);
    } else {
      (*hand)(sig);
    }

    // restore the signal mask
    thr_sigsetmask(SIG_SETMASK, &oset, 0);
  }
  // Tell jvm's signal handler the signal is taken care of.
  return true;
}

struct sigaction* os::Solaris::get_preinstalled_handler(int sig) {
  if ((( (unsigned int)1 << sig ) & sigs) != 0) {
    return &sigact[sig];
  }
  return NULL;
}

void os::Solaris::save_preinstalled_handler(int sig, struct sigaction& oldAct) {
  assert(sig > 0 && sig < MAXSIGNUM, "vm signal out of expected range");
  sigact[sig] = oldAct;
  sigs |= (unsigned int)1 << sig;
}

void os::Solaris::set_signal_handler(int sig, bool set_installed) {
  // Check for overwrite.
  struct sigaction oldAct;
  sigaction(sig, (struct sigaction*)NULL, &oldAct);
  void* oldhand = oldAct.sa_sigaction ? CAST_FROM_FN_PTR(void*,  oldAct.sa_sigaction)
				      : CAST_FROM_FN_PTR(void*,  oldAct.sa_handler);
  if (oldhand != CAST_FROM_FN_PTR(void*, SIG_DFL) &&
      oldhand != CAST_FROM_FN_PTR(void*, SIG_IGN) &&
      oldhand != CAST_FROM_FN_PTR(void*, signalHandler)) {
    if (AllowUserSignalHandlers || !set_installed) {
      // Do not overwrite; user takes responsibility to forward to us.
      return;
    } else if (UseSignalChaining) {
      // save the old handler in jvm
      save_preinstalled_handler(sig, oldAct);
      // libjsig also interposes the sigaction() call below and saves the
      // old sigaction on it own.
    } else {
      fatal2("Encountered unexpected pre-existing sigaction handler %#lx for signal %d.", (long)oldhand, sig);
    }
  }

  struct sigaction sigAct;
  sigfillset(&(sigAct.sa_mask));
  sigAct.sa_handler = SIG_DFL;

  sigAct.sa_sigaction = signalHandler;
  // Handle SIGSEGV on alternate signal stack if
  // running with bound threads.
  if (UseBoundThreads && sig == SIGSEGV) {
    sigAct.sa_flags = SA_SIGINFO | SA_RESTART | SA_ONSTACK;
  } else {
    sigAct.sa_flags = SA_SIGINFO | SA_RESTART;
  }

  sigaction(sig, &sigAct, &oldAct);

  void* oldhand2 = oldAct.sa_sigaction ? CAST_FROM_FN_PTR(void*, oldAct.sa_sigaction)
				       : CAST_FROM_FN_PTR(void*, oldAct.sa_handler);
  assert(oldhand2 == oldhand, "no concurrent signal handler installation");
}


void os::Solaris::install_signal_handlers() {
  signal_handlers_are_installed = true;

  // signal-chaining
  typedef void (*signal_setting_t)();
  signal_setting_t begin_signal_setting = NULL;
  signal_setting_t end_signal_setting = NULL;
  begin_signal_setting = (signal_setting_t) dlsym(RTLD_DEFAULT,
                                       "JVM_begin_signal_setting");
  if (begin_signal_setting != NULL) {
    end_signal_setting = (signal_setting_t) dlsym(RTLD_DEFAULT,
                                       "JVM_end_signal_setting");
    get_signal_action = (get_signal_t) dlsym(RTLD_DEFAULT,
                                      "JVM_get_signal_action");
    libjsig_is_loaded = true;
    assert(UseSignalChaining, "should enable signal-chaining");
  }
  if (libjsig_is_loaded) {
    // Tell libjsig jvm is setting signal handlers
    (*begin_signal_setting)();
  }

  set_signal_handler(SIGSEGV, true);
  set_signal_handler(SIGPIPE, true);
  set_signal_handler(SIGBUS, true);
  set_signal_handler(SIGILL, true);
  set_signal_handler(SIGFPE, true);
  set_signal_handler(INTERRUPT_SIGNAL, true);

  if (UseISM) {
    set_signal_handler(SIGINT, false);
    set_signal_handler(SIGHUP, false);
    set_signal_handler(SIGTERM, false);
  }

  if (libjsig_is_loaded) {
    // Tell libjsig jvm finishes setting signal handlers
    (*end_signal_setting)();
  }
}


void report_error(const char* file_name, int line_no, const char* title, const char* format, ...);

const char * os::exception_name(int exception_code)
{
  return NULL;
}

extern "C"{
static void exception_handler_during_fatal_error(int);
}

/* a very simple exception handler designed to print out at least some information */
/* when the fatal error handler crashes.                                           */
static void exception_handler_during_fatal_error(int sig)
{
   os::handle_recursive_fatal_error(sig);
   /* won't reach here */
   os::abort();
}

void os::set_exception_handler_during_fatal_error(int sig, int is_install)
{
   if (is_install)
        os::signal(sig, CAST_FROM_FN_PTR(void *, exception_handler_during_fatal_error));
   else os::signal(sig, CAST_FROM_FN_PTR(void *, SIG_DFL));
}

// (Static) wrappers for the new libthread API
int_fnP_thread_t_iP_uP_stack_tP_gregset_t os::Solaris::_thr_getstate;
int_fnP_thread_t_i_gregset_t os::Solaris::_thr_setstate;
int_fnP_thread_t_i os::Solaris::_thr_setmutator;
int_fnP_thread_t os::Solaris::_thr_suspend_mutator;
int_fnP_thread_t os::Solaris::_thr_continue_mutator;

static address resolve_symbol(const char *name) {
  address addr;

  addr = (address) dlsym(RTLD_DEFAULT, name);
  if(addr == NULL) {
    // RTLD_DEFAULT was not defined on some early versions of 2.5.1
    addr = (address) dlsym(RTLD_NEXT, name);    
    if(addr == NULL) {
      fatal(dlerror());
    }
  }
  return addr;
}

void os::Solaris::libthread_init() {
  address func = (address) dlsym(RTLD_DEFAULT, "_thr_suspend_allmutators");
  bool fast_pc_funcs = false;

  // RTLD_DEFAULT was not defined on some early versions of 5.5.1
  if(func == NULL) {
    func = (address) dlsym(RTLD_NEXT, "_thr_suspend_allmutators");  
  }
  os::Solaris::set_mutator_libthread(func != NULL);

  // Turn off fast_pc functions if this VM is running on an old OS (prior to 5.6)
  if (os::Solaris::mutator_libthread()) {
    char procdir[1000];
    sprintf(procdir, "/proc/%d/.", getpid());
    fast_pc_funcs = true;
    if (access(procdir, F_OK) != 0) {
        fast_pc_funcs = false;
        if(GetThreadState && UseCompilerSafepoints && ShowSafepointMsgs) {
          tty->print_cr("For Solaris 5.5.1 and older compiled safepoints are disabled.");
        }
    }
    else {
#ifndef SOLARIS_STRUCTURED_PROC
      // This is the case when we were built on 2.5.1, but are executing on 2.6 or later
      fast_pc_funcs = false;
      if(GetThreadState && UseCompilerSafepoints && ShowSafepointMsgs) {
        tty->print_cr("For VM built on Solaris 5.5.1 and older compiled safepoints are disabled.");
      }
#endif
    }
  }

  // Initialize the new libthread getstate API wrappers
  if(fast_pc_funcs) {
    func = resolve_symbol("thr_getstate");
    os::Solaris::set_thr_getstate(CAST_TO_FN_PTR(int_fnP_thread_t_iP_uP_stack_tP_gregset_t, func));

    func = resolve_symbol("thr_setstate");
    os::Solaris::set_thr_setstate(CAST_TO_FN_PTR(int_fnP_thread_t_i_gregset_t, func));
    
    func = resolve_symbol("thr_setmutator");
    os::Solaris::set_thr_setmutator(CAST_TO_FN_PTR(int_fnP_thread_t_i, func));

    func = resolve_symbol("thr_suspend_mutator");
    os::Solaris::set_thr_suspend_mutator(CAST_TO_FN_PTR(int_fnP_thread_t, func));

    func = resolve_symbol("thr_continue_mutator");
    os::Solaris::set_thr_continue_mutator(CAST_TO_FN_PTR(int_fnP_thread_t, func));

    long size;
    void (*handler_info_func)(address *, long *);
    handler_info_func = CAST_TO_FN_PTR(void (*)(address *, long *), resolve_symbol("thr_sighndlrinfo"));
    handler_info_func(&handler_start, &size);
    handler_end = handler_start + size;
  }
  else {
    if(GetThreadState && UseCompilerSafepoints && ShowSafepointMsgs) {
        tty->print_cr("Compiled code safepoints disabled.");
    }
    os::Solaris::set_thr_getstate(NULL);
    os::Solaris::set_thr_setstate(NULL);
    os::Solaris::set_thr_setmutator(NULL);
    os::Solaris::set_thr_suspend_mutator(NULL);
    os::Solaris::set_thr_continue_mutator(NULL);
  }
}


int_fnP_mutex_tP os::Solaris::_mutex_lock;
int_fnP_mutex_tP os::Solaris::_mutex_trylock;
int_fnP_mutex_tP os::Solaris::_mutex_unlock;
int_fnP_mutex_tP_i_vP os::Solaris::_mutex_init;
int_fnP_mutex_tP os::Solaris::_mutex_destroy;
int os::Solaris::_mutex_scope = USYNC_THREAD;

int_fnP_cond_tP_mutex_tP_timestruc_tP os::Solaris::_cond_timedwait;
int_fnP_cond_tP_mutex_tP os::Solaris::_cond_wait;
int_fnP_cond_tP os::Solaris::_cond_signal;
int_fnP_cond_tP os::Solaris::_cond_broadcast;
int_fnP_cond_tP_i_vP os::Solaris::_cond_init;
int_fnP_cond_tP os::Solaris::_cond_destroy;
int os::Solaris::_cond_scope = USYNC_THREAD;

void os::Solaris::synchronization_init() {
  if(UseLWPSynchronization) {
    os::Solaris::set_mutex_lock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("_lwp_mutex_lock")));
    os::Solaris::set_mutex_trylock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("_lwp_mutex_trylock")));
    os::Solaris::set_mutex_unlock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("_lwp_mutex_unlock")));
    os::Solaris::set_mutex_init(lwp_mutex_init);
    os::Solaris::set_mutex_destroy(lwp_mutex_destroy);
    os::Solaris::set_mutex_scope(USYNC_PROCESS);
    
    os::Solaris::set_cond_timedwait(CAST_TO_FN_PTR(int_fnP_cond_tP_mutex_tP_timestruc_tP, resolve_symbol("_lwp_cond_timedwait")));
    os::Solaris::set_cond_wait(CAST_TO_FN_PTR(int_fnP_cond_tP_mutex_tP, resolve_symbol("_lwp_cond_wait")));
    os::Solaris::set_cond_signal(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("_lwp_cond_signal")));
    os::Solaris::set_cond_broadcast(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("_lwp_cond_broadcast")));
    os::Solaris::set_cond_init(lwp_cond_init);        
    os::Solaris::set_cond_destroy(lwp_cond_destroy);        
    os::Solaris::set_cond_scope(USYNC_PROCESS);
  }
  else {
    os::Solaris::set_mutex_scope(USYNC_THREAD);
    os::Solaris::set_cond_scope(USYNC_THREAD);

    if(UsePthreads) {
      os::Solaris::set_mutex_lock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("pthread_mutex_lock")));
      os::Solaris::set_mutex_trylock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("pthread_mutex_trylock")));
      os::Solaris::set_mutex_unlock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("pthread_mutex_unlock")));
      os::Solaris::set_mutex_init(pthread_mutex_default_init);
      os::Solaris::set_mutex_destroy(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("pthread_mutex_destroy")));
      
      os::Solaris::set_cond_timedwait(CAST_TO_FN_PTR(int_fnP_cond_tP_mutex_tP_timestruc_tP, resolve_symbol("pthread_cond_timedwait")));
      os::Solaris::set_cond_wait(CAST_TO_FN_PTR(int_fnP_cond_tP_mutex_tP, resolve_symbol("pthread_cond_wait")));
      os::Solaris::set_cond_signal(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("pthread_cond_signal")));
      os::Solaris::set_cond_broadcast(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("pthread_cond_broadcast")));
      os::Solaris::set_cond_init(pthread_cond_default_init);    
      os::Solaris::set_cond_destroy(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("pthread_cond_destroy")));
    }   
    else {
      os::Solaris::set_mutex_lock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("mutex_lock")));
      os::Solaris::set_mutex_trylock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("mutex_trylock")));
      os::Solaris::set_mutex_unlock(CAST_TO_FN_PTR(int_fnP_mutex_tP, resolve_symbol("mutex_unlock")));
      os::Solaris::set_mutex_init(::mutex_init);
      os::Solaris::set_mutex_destroy(::mutex_destroy);
      
      os::Solaris::set_cond_timedwait(CAST_TO_FN_PTR(int_fnP_cond_tP_mutex_tP_timestruc_tP, resolve_symbol("cond_timedwait")));
      os::Solaris::set_cond_wait(CAST_TO_FN_PTR(int_fnP_cond_tP_mutex_tP, resolve_symbol("cond_wait")));
      os::Solaris::set_cond_signal(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("cond_signal")));
      os::Solaris::set_cond_broadcast(CAST_TO_FN_PTR(int_fnP_cond_tP, resolve_symbol("cond_broadcast")));
      os::Solaris::set_cond_init(::cond_init);    
      os::Solaris::set_cond_destroy(::cond_destroy);        
    }
  }
}

int os::Solaris::_dev_zero_fd = -1;

// this is called _before_ the global arguments have been parsed
void os_init() {
  first_hrtime = gethrtime();

  os::init_random(1234567);
  ThreadCritical::initialize();

  page_size = sysconf(_SC_PAGESIZE);
  if (page_size == -1)
    fatal1("os_solaris.cpp: os_init: sysconf failed (%s)", strerror(errno));

  os::Solaris::initialize_system_info();

  int fd = open("/dev/zero", O_RDWR);
  if (fd < 0) {
    fatal1("os_init: cannot open /dev/zero (%s)", strerror(errno));
  } else {
    os::Solaris::set_dev_zero_fd(fd);
  }

  // cache a FetchTopFrame_Callback so we don't need to call "new"
  // from the callback -- this is a problem if the suspended thread we
  // are suspending is holding the malloc lock (one for the VMThread
  // and one for the WatcherThread)

  (new FetchTopFrame_Callback())->move_to_cancelled_list();
  (new FetchTopFrame_Callback())->move_to_cancelled_list();

  // Setup a unique callback for interrupting the VM thread during profiling
  OSThread::set_vm_interrupt_callback(new GetThreadPC_Callback());

  // main_thread points to the aboriginal thread
  ThreadCritical_construct = ThreadCritical_construct_regular;
  ThreadCritical_destruct  = ThreadCritical_destruct_regular;
  main_thread = thr_self();
}

// To install release_memory_special function for atexit system call
extern "C" {
  static void release_memory_special_helper() {
    os::release_memory_special();
  }
}

// this is called _after_ the global arguments have been parsed
jint os_init_2() {

  size_t threadStackSizeInBytes = ThreadStackSize * K;
  if (threadStackSizeInBytes != 0 &&
    threadStackSizeInBytes < (64 * K)) {
    tty->print_cr("\nThe stack size specified is too small, Specify at least 64k");
    return JNI_ERR;
  }
  // Make the stack size a multiple of the page size so that
  // the yellow/red zones can be guarded.

  JavaThread::set_stack_size_at_create(round_to(threadStackSizeInBytes,
        os::vm_page_size()));

  os::Solaris::libthread_init();
  if(os::Solaris::mutator_libthread() == NULL) {
    if (Arguments::has_profile()) {
      tty->print_cr("\nThe libthread.so on your  system  is an older  version that does not");
      tty->print_cr("support -Xprof. Please read the install documentation for patch");
      tty->print_cr("installation instructions.");
      return JNI_ERR;
    }
    tty->print_cr("Warning! The libthread.so on your system is an older version than");
    tty->print_cr("the one this VM was tested with. Please read the install documentation for");
    tty->print_cr("patch installation instructions.");

    // This is a temporary workaround, 5.5.1's libthread patch
    // will not be available at fcs.
    struct utsname un;
    strcpy(un.release, "");
    uname(&un);
    if(ExitOnBadLibThread && strcmp(un.release,"5.5.1") != 0) {
      return JNI_ERR;
    }
  }

  signal_sets_init();
  if (UseTopLevelExceptionFilter) {
    os::Solaris::install_signal_handlers();
  }

  // initialize synchronization primitives to use either thread or
  // lwp synchronization (controlled by UseLWPSynchronization)
  os::Solaris::synchronization_init();

  if (MaxFDLimit) {
    // set the number of file descriptors to max. print out error 
    // if getrlimit/setrlimit fails but continue regardless. 
    struct rlimit nbr_files;
    int status = getrlimit(RLIMIT_NOFILE, &nbr_files);
    if (status != 0) {
      if (Verbose || WizardMode)
        perror("os::os_init_2 getrlimit failed");
    } else {
      nbr_files.rlim_cur = nbr_files.rlim_max;
      status = setrlimit(RLIMIT_NOFILE, &nbr_files);
      if (status != 0) {
        if (Verbose || WizardMode)
          perror("os::os_init_2 setrlimit failed");
      }
    }
  }

  // Initialize HPI.
  jint hpi_result = hpi::initialize();
  if (hpi_result != JNI_OK) {
    tty->print_cr("There was an error trying to initialize the HPI library.");
    tty->print_cr("Please check your installation, HotSpot does not work correctly");
    tty->print_cr("when installed in the JDK 1.2 Solaris Production Release, or");
    tty->print_cr("with any JDK 1.1.x release.");
    return hpi_result;
  }

  // Calculate theoretical max. size of Threads to guard gainst
  // artifical out-of-memory situations, where all available address-
  // space has been reserved by thread stacks. Default stack size is 1Mb.
  int pre_thread_stack_size = (JavaThread::stack_size_at_create()) ?
    JavaThread::stack_size_at_create() : (1*K*K);
  assert(pre_thread_stack_size != 0, "Must have a stack");
  // Solaris has a maximum of 4Gb of user programs. Calculate the thread limit when
  // we should start doing Virtual Memory banging. Currently when the threads will
  // have used all but 200Mb of space.
  size_t max_address_space = ((unsigned int)4 * K * K * K) - (200 * K * K);
  os::Solaris::_os_thread_limit = max_address_space / pre_thread_stack_size;

  // Install cleanup functions called during VM exit
  if (UseISM) {
    if (atexit(release_memory_special_helper) != 0) {
      perror("os::os_init2 atexit(release_memory_special) failed");
    }
  }

  return JNI_OK;
}


void os_exit() {
  int fd = os::Solaris::get_dev_zero_fd();
  if (fd >=0 ) {
    close(fd);
    os::Solaris::set_dev_zero_fd(-1);
  }
  ThreadCritical::release();
}


// OS interface.

int os::stat(const char *path, struct stat *sbuf) {
  char pathbuf[MAX_PATH];
  if (strlen(path) > MAX_PATH - 1) {
    errno = ENAMETOOLONG;
    return -1;
  }
  hpi::native_path(strcpy(pathbuf, path));
  return ::stat(pathbuf, sbuf);
}


bool os::check_heap(bool force) { return true; }

typedef int (*vsnprintf_t)(char* buf, size_t count, const char* fmt, va_list argptr);
static vsnprintf_t sol_vsnprintf = NULL;

int local_vsnprintf(char* buf, size_t count, const char* fmt, va_list argptr) {
  if (!sol_vsnprintf) {
    //search  for the named symbol in the objects that were loaded after libjvm
    void* where = RTLD_NEXT;
    if ((sol_vsnprintf = CAST_TO_FN_PTR(vsnprintf_t, dlsym(where, "__vsnprintf"))) == NULL)
	sol_vsnprintf = CAST_TO_FN_PTR(vsnprintf_t, dlsym(where, "vsnprintf"));
    if (!sol_vsnprintf){
      //search  for the named symbol in the objects that were loaded before libjvm
      where = RTLD_DEFAULT;
      if ((sol_vsnprintf = CAST_TO_FN_PTR(vsnprintf_t, dlsym(where, "__vsnprintf"))) == NULL)
	sol_vsnprintf = CAST_TO_FN_PTR(vsnprintf_t, dlsym(where, "vsnprintf"));
      assert(sol_vsnprintf != NULL, "vsnprintf not found");
    }
  }
  return (*sol_vsnprintf)(buf, count, fmt, argptr);
}


#ifndef PRODUCT
#ifdef INTERPOSE_ON_SYSTEM_SYNCH_FUNCTIONS
// Turn this on if you need to trace synch operations.
// Set RECORD_SYNCH_LIMIT to a large-enough value,
// and call record_synch_enable and record_synch_disable
// around the computation of interest.

void record_synch(char* name, bool returning);  // defined below

class RecordSynch {
  char* _name;
 public:
  RecordSynch(char* name) :_name(name)
                 { record_synch(_name, false); }
  ~RecordSynch() { record_synch(_name,   true);  }
};

#define CHECK_SYNCH_OP(ret, name, params, args, inner)		\
extern "C" ret name params {					\
  typedef ret name##_t params;					\
  static name##_t* implem = NULL;				\
  static int callcount = 0;					\
  if (implem == NULL) {						\
    implem = (name##_t*) dlsym(RTLD_NEXT, #name);		\
    if (implem == NULL)  fatal(dlerror());			\
  }								\
  ++callcount;							\
  RecordSynch _rs(#name);					\
  inner;							\
  return implem args;						\
}
// in dbx, examine callcounts this way:
// for n in $(eval whereis callcount | awk '{print $2}'); do print $n; done

#define CHECK_POINTER_OK(p) \
  (Universe::perm_gen() == NULL || !Universe::is_reserved_heap((oop)(p)))
#define CHECK_MU \
  if (!CHECK_POINTER_OK(mu)) fatal("Mutex must be in C heap only.");
#define CHECK_CV \
  if (!CHECK_POINTER_OK(cv)) fatal("Condvar must be in C heap only.");
#define CHECK_P(p) \
  if (!CHECK_POINTER_OK(p))  fatal(false,  "Pointer must be in C heap only.");

#define CHECK_MUTEX(mutex_op) \
CHECK_SYNCH_OP(int, mutex_op, (mutex_t *mu), (mu), CHECK_MU);

CHECK_MUTEX(   mutex_lock)
CHECK_MUTEX(  _mutex_lock)
CHECK_MUTEX( mutex_unlock)
CHECK_MUTEX(_mutex_unlock)
CHECK_MUTEX( mutex_trylock)
CHECK_MUTEX(_mutex_trylock)

#define CHECK_COND(cond_op) \
CHECK_SYNCH_OP(int, cond_op, (cond_t *cv, mutex_t *mu), (cv, mu), CHECK_MU;CHECK_CV);

CHECK_COND( cond_wait);
CHECK_COND(_cond_wait);
CHECK_COND(_cond_wait_cancel);

#define CHECK_COND2(cond_op) \
CHECK_SYNCH_OP(int, cond_op, (cond_t *cv, mutex_t *mu, timestruc_t* ts), (cv, mu, ts), CHECK_MU;CHECK_CV);

CHECK_COND2( cond_timedwait);
CHECK_COND2(_cond_timedwait);
CHECK_COND2(_cond_timedwait_cancel);

// do the _lwp_* versions too
#define mutex_t lwp_mutex_t
#define cond_t  lwp_cond_t
CHECK_MUTEX(  _lwp_mutex_lock)
CHECK_MUTEX(  _lwp_mutex_unlock)
CHECK_MUTEX(  _lwp_mutex_trylock)
CHECK_MUTEX( __lwp_mutex_lock)
CHECK_MUTEX( __lwp_mutex_unlock)
CHECK_MUTEX( __lwp_mutex_trylock)
CHECK_MUTEX(___lwp_mutex_lock)
CHECK_MUTEX(___lwp_mutex_unlock)

CHECK_COND(  _lwp_cond_wait);
CHECK_COND( __lwp_cond_wait);
CHECK_COND(___lwp_cond_wait);

CHECK_COND2(  _lwp_cond_timedwait);
CHECK_COND2( __lwp_cond_timedwait);
#undef mutex_t
#undef cond_t

CHECK_SYNCH_OP(int, _lwp_suspend2,       (int lwp, int *n), (lwp, n), 0);
CHECK_SYNCH_OP(int,__lwp_suspend2,       (int lwp, int *n), (lwp, n), 0);
CHECK_SYNCH_OP(int, _lwp_kill,           (int lwp, int n),  (lwp, n), 0);
CHECK_SYNCH_OP(int,__lwp_kill,           (int lwp, int n),  (lwp, n), 0);
CHECK_SYNCH_OP(int, _lwp_sema_wait,      (lwp_sema_t* p),   (p),  CHECK_P(p));
CHECK_SYNCH_OP(int,__lwp_sema_wait,      (lwp_sema_t* p),   (p),  CHECK_P(p));
CHECK_SYNCH_OP(int, _lwp_cond_broadcast, (lwp_cond_t* cv),  (cv), CHECK_CV);
CHECK_SYNCH_OP(int,__lwp_cond_broadcast, (lwp_cond_t* cv),  (cv), CHECK_CV);


// recording machinery:

enum { RECORD_SYNCH_LIMIT = 200 };
char* record_synch_name[RECORD_SYNCH_LIMIT];
void* record_synch_arg0ptr[RECORD_SYNCH_LIMIT];
bool record_synch_returning[RECORD_SYNCH_LIMIT];
thread_t record_synch_thread[RECORD_SYNCH_LIMIT];
int record_synch_count = 0;
bool record_synch_enabled = false;

// in dbx, examine recorded data this way:
// for n in name arg0ptr returning thread; do print record_synch_$n[0..record_synch_count-1]; done

void record_synch(char* name, bool returning) {
  if (record_synch_enabled) {
    if (record_synch_count < RECORD_SYNCH_LIMIT) {
      record_synch_name[record_synch_count] = name;
      record_synch_returning[record_synch_count] = returning;
      record_synch_thread[record_synch_count] = thr_self();
      record_synch_arg0ptr[record_synch_count] = &name;
      record_synch_count++;
    }
    // put more checking code here:
    // ...
  }
}

void record_synch_enable() {
  // start collecting trace data, if not already doing so
  if (!record_synch_enabled)  record_synch_count = 0;
  record_synch_enabled = true;
}

void record_synch_disable() {
  // stop collecting trace data
  record_synch_enabled = false;
}

#endif // INTERPOSE_ON_SYSTEM_SYNCH_FUNCTIONS
#endif // PRODUCT


// these routines are the initial value of atomic_exchange_entry(),
// atomic_compare_and_exchange_entry(), and atomic_increment_entry
// until initialization is complete (see stubRoutines_<arch>.cpp)
// TODO - factor (or see win32 inline model?)

typedef jint exchange_func_t            (jint, jint*);
typedef jint compare_and_exchange_func_t(jint, jint*, jint);
typedef jint increment_func_t           (jint, jint*);

jint os::Solaris::atomic_exchange_bootstrap(jint exchange_value, jint* dest) {
  // try to use the stub:
  exchange_func_t* func = CAST_TO_FN_PTR(exchange_func_t*, StubRoutines::atomic_exchange_entry());

  if (func != NULL) {
    os::Solaris::atomic_exchange_func = func;
    return (*func)(exchange_value, dest);
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  jint old_value = *dest;
  *dest = exchange_value;
  return old_value;
}

jint os::Solaris::atomic_compare_and_exchange_bootstrap(jint exchange_value, jint* dest, jint compare_value) {
  // try to use the stub:
  compare_and_exchange_func_t* func = CAST_TO_FN_PTR(compare_and_exchange_func_t*,
						       StubRoutines::atomic_compare_and_exchange_entry());

  if (func != NULL) {
    os::Solaris::atomic_compare_and_exchange_func = func;
    return (*func)(exchange_value, dest, compare_value);
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  jint old_value = *dest;
  if (old_value == compare_value)
    *dest = exchange_value;
  return old_value;
}

jint os::Solaris::atomic_increment_bootstrap(jint inc, jint* loc) {
  // try to use the stub:
  increment_func_t* func = CAST_TO_FN_PTR(increment_func_t*, StubRoutines::atomic_increment_entry());

  if (func != NULL) {
    os::Solaris::atomic_increment_func = func;
    return (*func)(inc, loc);
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  return (*loc) += inc;
}


exchange_func_t* os::Solaris::atomic_exchange_func
  = os::Solaris::atomic_exchange_bootstrap;
compare_and_exchange_func_t* os::Solaris::atomic_compare_and_exchange_func
  = os::Solaris::atomic_compare_and_exchange_bootstrap;
increment_func_t* os::Solaris::atomic_increment_func
  = os::Solaris::atomic_increment_bootstrap;


// JVMPI code
jlong os::thread_cpu_time() {
  return (jlong)gethrtime();
}

FetchTopFrame_Callback* FetchTopFrame_Callback::_cancelled_list = NULL;

//---------------------------------------------------------------------------------
#ifndef PRODUCT

static address same_page(address x, address y) {
  int page_bits = -os::vm_page_size();
  if ((jint(x) & page_bits) == (jint(y) & page_bits))
    return x;
  else if (x > y)
    return (address)(jint(y) | ~page_bits) + 1;
  else
    return (address)(jint(y) & page_bits);
}

bool os::find(address addr) {
  Dl_info dlinfo;
  memset(&dlinfo, 0, sizeof(dlinfo));
  if (dladdr(addr, &dlinfo)) {
    tty->print("0x%08x: ", addr);
    if (dlinfo.dli_sname != NULL)
      tty->print("%s+%#x", dlinfo.dli_sname, addr-(int)dlinfo.dli_saddr);
    else if (dlinfo.dli_fname)
      tty->print("<offset %#x>", addr-(int)dlinfo.dli_fbase);
    else
      tty->print("<absolute address>");
    if (dlinfo.dli_fname)  tty->print(" in %s", dlinfo.dli_fname);
    if (dlinfo.dli_fbase)  tty->print(" at 0x%08x", dlinfo.dli_fbase);
    tty->cr();

    if (Verbose) {
      // decode some bytes around the PC
      address begin = same_page(addr-40, addr);
      address end   = same_page(addr+40, addr);
      address       lowest = (address) dlinfo.dli_sname;
      if (!lowest)  lowest = (address) dlinfo.dli_fbase;
      if (begin < lowest)  begin = lowest;
      Dl_info dlinfo2;
      if (dladdr(end, &dlinfo2) && dlinfo2.dli_saddr != dlinfo.dli_saddr
	  && end > dlinfo2.dli_saddr && dlinfo2.dli_saddr > begin)
        end = (address) dlinfo2.dli_saddr;
      Disassembler::decode(begin, end);
    }
    return true;
  }
  return false;
}
  
#endif


// Following function has been added to support HotSparc's libjvm.so running 
// under Solaris production JDK 1.2.2 / 1.3.0.  These came from 
// src/solaris/hpi/native_threads in the EVM codebase.
//
// NOTE: This is no longer needed in the 1.3.1 and 1.4 production release
// libraries and should thus be removed. We will leave it behind for a while
// until we no longer want to able to run on top of 1.3.0 Solaris production
// JDK. See 4341971.

#define STACK_SLACK 0x800

extern "C" {
  int sysThreadAvailableStackWithSlack(){
    stack_t st;
    int retval, stack_top;
    retval = thr_stksegment(&st);
    assert(retval == 0, "incorrect return value from thr_stksegment");
    assert((address)&st < (address)st.ss_sp, "Invalid stack base returned");
    assert((address)&st > (address)st.ss_sp-st.ss_size, "Invalid stack size returned");
    stack_top=(int)st.ss_sp-st.ss_size; 
    return ((int)&stack_top - stack_top - STACK_SLACK);
  }
}

//Reconciliation History
// 1.18 97/07/17 18:50:28 os_win32.cpp
// 1.30 97/08/12 17:07:10 os_win32.cpp
// 1.49 97/09/26 12:52:54 os_win32.cpp
// 1.63 97/10/08 19:13:30 os_win32.cpp
// 1.74 97/10/31 14:58:14 os_win32.cpp
// 1.82 97/11/11 10:42:57 os_win32.cpp
// 1.92 97/11/19 17:49:49 os_win32.cpp
// 1.100 97/12/04 17:40:10 os_win32.cpp
// 1.102.1.1 97/12/09 13:30:13 os_win32.cpp
// 1.116 98/01/06 15:45:21 os_win32.cpp
// 1.124 98/01/14 11:13:57 os_win32.cpp
// 1.129 98/01/16 23:12:17 os_win32.cpp
// 1.132 98/01/21 16:38:21 os_win32.cpp
// 1.133 98/01/21 17:19:12 os_win32.cpp
// 1.139 98/01/28 12:13:10 os_win32.cpp
// 1.145 98/02/05 13:38:29 os_win32.cpp
// 1.159 98/02/28 11:29:29 os_win32.cpp
// 1.162 98/03/11 12:10:42 os_win32.cpp
// 1.164 98/03/17 11:03:21 os_win32.cpp
// 1.165 98/03/18 13:50:16 os_win32.cpp
// 1.166 98/03/19 18:12:25 os_win32.cpp
// 1.170 98/03/30 17:01:30 os_win32.cpp
// 1.176 98/04/02 10:33:50 os_win32.cpp
// 1.182 98/04/15 09:36:44 os_win32.cpp
// 1.178 98/04/30 16:44:29 os_win32.cpp
// 1.179 98/05/04 16:06:10 os_win32.cpp
// 1.186 98/05/07 14:17:46 os_win32.cpp
// 1.188 98/05/26 15:54:13 os_win32.cpp
// 1.194 98/06/16 15:10:34 os_win32.cpp
// 1.197 98/06/20 13:44:49 os_win32.cpp
// 1.200 98/07/02 13:51:09 os_win32.cpp
// 1.204 98/07/24 15:16:59 os_win32.cpp
// 1.208 98/08/07 16:48:17 os_win32.cpp
// 1.209 98/08/13 16:24:18 os_win32.cpp
// 1.210 98/08/26 18:22:18 os_win32.cpp
// 1.211 98/09/03 20:06:02 os_win32.cpp
// 1.222 98/09/28 18:47:04 os_win32.cpp
// 1.227 98/10/02 14:14:46 os_win32.cpp
// 1.228 98/10/06 14:41:17 os_win32.cpp
// 1.234 98/10/08 14:28:45 os_win32.cpp
// 1.235 98/10/09 01:10:53 os_win32.cpp
// 1.236 98/10/09 14:53:27 os_win32.cpp
// 1.241 98/10/12 11:10:48 os_win32.cpp
// 1.242 98/10/16 16:41:24 os_win32.cpp
// 1.243 98/10/22 17:38:18 os_win32.cpp
// 1.245 98/10/27 13:57:22 os_win32.cpp
// 1.247 98/11/09 09:26:29 os_win32.cpp
// 1.250 98/11/16 08:26:49 os_win32.cpp
// 1.251 98/11/19 11:55:30 os_win32.cpp
// 1.252 98/11/25 15:47:57 os_win32.cpp
// 1.255 98/12/28 20:17:22 os_win32.cpp
// 1.259 99/01/06 10:00:50 os_win32.cpp
// 1.261 99/01/07 16:53:57 os_win32.cpp
// 1.262 99/01/19 11:31:07 os_win32.cpp
// 1.267 99/01/30 18:18:36 os_win32.cpp
// 1.273 99/02/18 16:58:11 os_win32.cpp
// 1.278 99/03/10 15:40:05 os_win32.cpp
// 1.286 99/03/30 19:32:30 os_win32.cpp
// 1.287 99/04/19 13:50:56 os_win32.cpp
// 1.291 99/06/02 18:15:39 os_win32.cpp
// 1.292 99/06/28 10:06:44 os_win32.cpp
// 1.287 99/04/01 18:31:08 os_win32.cpp
// 1.288 99/04/12 12:56:48 os_win32.cpp
// 1.293 99/07/06 16:36:49 os_win32.cpp
// 1.294 99/07/13 11:41:18 os_win32.cpp
// 1.297 99/07/20 15:49:56 os_win32.cpp
// 1.296 99/08/05 16:45:47 os_win32.cpp
// 1.297 99/08/25 13:15:30 os_win32.cpp
//End
