/*
 * @(#)os_solaris.inline.hpp	1.26 00/06/23 00:53:55
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

inline const char* os::file_separator() { return "/"; }
inline const char* os::line_separator() { return "\n"; }
inline const char* os::path_separator() { return ":"; }

inline const char* os::jlong_format_specifier()   { return "%lld"; }
inline const char* os::julong_format_specifier()  { return "%llu"; }

// Tells whether we're running on an MP machine
inline bool os::is_MP() { return Solaris::is_MP();}

// File names are case-sensitive on windows only
inline int os::file_name_strcmp(char* s1, char* s2) { return strcmp(s1, s2); }

inline int os::bang_stack_size_for_exceptions() {
  const exception_stack_allowance = 8*K;
  if (!UseBoundThreads && UseStackBanging)
    return ((int)exception_stack_allowance);
  return 0;
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// macros for interruptible io and system calls 

#define _INTERRUPTIBLE(_setup, _cmd, _result, _thread, _clear, _before, _after) \
do { \
  _setup; \
  _before; \
  OSThread* _osthread = _thread->osthread(); \
  if (_thread->has_last_Java_frame()) { \
    /* this is java interruptible io stuff */ \
    sigjmp_buf _jmpbuf; \
    int _jmpres = sigsetjmp(_jmpbuf, 1); \
    if (_jmpres == 0) { \
      _osthread->set_jump_buffer(&_jmpbuf); \
      atomic::membar(); \
      if (os::is_interrupted(_thread, _clear)) { \
        _result = OS_INTRPT; \
      } else { \
        _cmd; \
      } \
    } \
    _osthread->set_jump_buffer(NULL); \
    if (_jmpres != 0) { \
      os::is_interrupted(_thread, _clear); \
      _result = OS_INTRPT; \
    } \
  } else { \
    /* this is normal blocking io stuff */ \
    _cmd; \
  } \
  _after; \
} while(false)

#define INTERRUPTIBLE(_cmd, _result, _clear) \
  _INTERRUPTIBLE(JavaThread* _thread = os::Solaris::setup_interruptible(), _result = _cmd, _result, _thread, _clear, , os::Solaris::cleanup_interruptible(_thread) )

#define INTERRUPTIBLE_RETURN_INT(_cmd, _clear) do { \
  int _result; \
  INTERRUPTIBLE(_result = _cmd, _result, _clear); \
  return _result; \
} while(false)
