/*
 * @(#)safepoint_solaris.cpp	1.16 01/02/09 14:43:56
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_safepoint_solaris.cpp.incl"


bool SafepointSynchronize::can_be_at_safepoint_before_suspend(JavaThread *thread, JavaThreadState state) {
  switch(state) {
   case _thread_in_native:
     // threads in native code have not yet saved state
     return false;

   // New thread may have resources during creation. (ie. malloc lock)
   case _thread_new:
     return false;

   // blocked threads have saved state
   // new threads have no state to save
   case _thread_blocked:
   case _thread_in_native_blocked:
     return true;

   default:
    ShouldNotReachHere();
  }
}


bool SafepointSynchronize::can_be_at_safepoint_after_suspend(JavaThread *thread, JavaThreadState state) {
  switch(state) {
   case _thread_in_vm_trans:
     // threads in vm code have not yet saved state
     return false;

   default:
    ShouldNotReachHere();
  }
}


// On Solaris we found that we sometimes run into live lock situations where
// 3 or more threads are doing useless work but they hog onto lwps and since
// the system is not blocked the threads library does not create new lwps. This
// results in threads which need lwps getting startved and the application does
// not make much progress (This is espcially true of GUI applications). So over
// here we check if the -Xbatch option was not set and if not we trigget periodic
// safepoints which suspends some threads and resumes them which results in the
// system geting a shakeup and makes the GUI applications behave better)
bool SafepointSynchronize::pd_is_cleanups_needed() {
  if ( BackgroundCompilation == true ) {
    // This code previously checked for the DynamicVMThreadPriority
    // and LimitVMThreadPriorityBoosting flags.  These have been
    // removed.  Quaere: does the new CIDynamicCompilePriority flag
    // need to be checked for here?
    return true;
  }
  return false;
}
