// @(#)atomic_win32.inline.hpp	1.5 01/04/23
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


// The following alternative implementations are needed because
// Windows 95 doesn't support (some of) the corresponding Windows NT
// calls. Furthermore, these versions allow inlining in the caller.
// (More precisely: The documentation for InterlockedExchange says
// it is supported for Windows 95. However, when single-stepping
// through the assembly code we cannot step into the routine and
// when looking at the routine address we see only garbage code.
// Better safe then sorry!). Was bug 7/31/98 (gri).
//
// Performance note: On uniprocessors, the 'lock' prefixes are not
// necessary (and expensive). We should generate separate cases if
// this becomes a performance problem.

#pragma warning(disable: 4035) // Disables warnings reporting missing return statement

inline jint atomic::add(jint* dest, jint add_value) {
  __asm {
    mov edx, dest;
    mov eax, add_value;
    mov ecx, eax;
    lock xadd dword ptr [edx], eax;
    add eax, ecx;
  }
}

inline void atomic::increment(jint* dest) {
  // alternative for InterlockedIncrement
  __asm {
    mov edx, dest;
    lock inc dword ptr [edx];
  }
}

inline void atomic::decrement(jint* dest) {
  // alternative for InterlockedDecrement
  __asm {
    mov edx, dest;
    lock dec dword ptr [edx];
  }
}

inline jint atomic::exchange(jint* dest, jint exchange_value) {
  // alternative for InterlockedExchange
  __asm {
    mov eax, exchange_value;
    mov ecx, dest;
    xchg eax, dword ptr [ecx];
  }
}

inline void atomic::store(jint* dest, jint exchange_value) {
  *dest = exchange_value;
}

inline jint atomic::compare_and_exchange(jint* dest, jint exchange_value, jint compare_value) {
  // alternative for InterlockedCompareExchange
  __asm {
    mov edx, dest
    mov ecx, exchange_value
    mov eax, compare_value
    lock cmpxchg dword ptr [edx], ecx   
  }
}

inline intptr_t atomic::add_ptr(intptr_t* dest, intptr_t add_value) {
  __asm {
    mov edx, dest;
    mov eax, add_value;
    mov ecx, eax;
    lock xadd dword ptr [edx], eax;
    add eax, ecx;
  }
}

inline void atomic::increment_ptr(intptr_t* dest) {
  // alternative for InterlockedIncrement
  __asm {
    mov edx, dest;
    lock inc dword ptr [edx];
  }
}

inline void atomic::decrement_ptr(intptr_t* dest) {
  // alternative for InterlockedDecrement
  __asm {
    mov edx, dest;
    lock dec dword ptr [edx];
  }
}

inline intptr_t atomic::exchange_ptr(intptr_t* dest, intptr_t exchange_value) {
  // alternative for InterlockedExchange
  __asm {
    mov eax, exchange_value;
    mov ecx, dest;
    xchg eax, dword ptr [ecx];
  }
}

inline void atomic::store_ptr(intptr_t* dest, intptr_t exchange_value) {
  *dest = exchange_value;
}

// If compare_value == [dest], then [dest] = exchange_value, return exchange_value
// else return compare_value
inline intptr_t atomic::compare_and_exchange_ptr(intptr_t* dest, intptr_t exchange_value, intptr_t compare_value) {
  // alternative for InterlockedCompareExchange
  __asm {
    mov edx, dest
    mov ecx, exchange_value
    mov eax, compare_value
    lock cmpxchg dword ptr [edx], ecx   
  }
}

inline void atomic::membar() {
#ifdef _M_ALPHA
  __MB();
#endif/*_M_ALPHA*/
}

#pragma warning(default: 4035) // Enables warnings reporting missing return statement

inline void atomic::prefetch_read(void *address) {
  // no-op
}

inline void atomic::prefetch_write(void *address) {
  // no-op
}
