/*
 * @(#)chaitin_win32.cpp	1.8 99/06/22 16:38:58
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2 

#include "incls/_precompiled.incl"
#include "incls/_chaitin_win32.cpp.incl"

// Disallow the use of the frame pointer (EBP) for implicit null exceptions
// on win95/98.  If we do not do this, the OS gets confused and gives a stack
// error.
void PhaseChaitin::pd_preallocate_hook() {
  if (ImplicitNullChecks && !os::win32::is_nt()) {
    for (uint block_num=1; block_num<_cfg._num_blocks; block_num++) {
      Block *block = _cfg._blocks[block_num];
      
      MachNode *block_end = block->end()->is_Mach();
      if (block_end && block_end->ideal_Opcode() != Op_Con) {
	MachNullCheckNode *null_check = block_end->is_MachNullCheck();
	if (null_check) {
	  // The last instruction in the block is an implicit null check.
	  // Fix its input so that it does not load into the frame pointer.
	  _matcher.pd_implicit_null_fixup(null_check->in(1)->is_Mach(),
					  null_check->_vidx);
	}
      }
    }
  }
}

#ifdef ASSERT
// Verify that no implicit null check uses the frame pointer (EBP) as
// its register on win95/98.  Use of the frame pointer in an implicit
// null check confuses the OS, yielding a stack error.
void PhaseChaitin::pd_postallocate_verify_hook() {
  if (ImplicitNullChecks && !os::win32::is_nt()) {
    for (uint block_num=1; block_num<_cfg._num_blocks; block_num++) {
      Block *block = _cfg._blocks[block_num];

      MachNode *block_end = block->_nodes[block->_nodes.size()-1]->is_Mach();
      if (block_end && block_end->ideal_Opcode() != Op_Con) {
        MachNullCheckNode *null_check = block_end->is_MachNullCheck();
        if (null_check) {
          // The last instruction in the block is an implicit
          // null check.  Verify that this instruction does not
          // use the frame pointer.
          int reg = get_register(null_check->in(1)->in(null_check->_vidx));
          assert(reg != EBP_num,
                 "implicit null check using frame pointer on win95/98");
        }
      }
    }
  }
}
#endif
