//
// @(#)os_linux_i486.cpp	1.24 01/03/09
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2
// do not include  precompiled  header file
# include "incls/_os_linux_i486.cpp.incl"
# include "incls/_os_pd.hpp.incl"

// put OS-includes here
# include <sys/types.h>
# include <sys/mman.h>
# include <pthread.h>
# include <signal.h>
# include <errno.h>
# include <dlfcn.h>
# include <stdio.h>
# include <unistd.h>
# include <sys/resource.h>
# include <pthread.h>
# include <sys/stat.h>
# include <sys/time.h>
# include <sys/utsname.h>
# include <sys/socket.h>
# include <sys/wait.h>
# include <pwd.h>
# include <poll.h>
# include <ucontext.h>

static int cpu_level = -1;

static void init_cpu_level() {
  struct utsname buf;
  if (uname(&buf)) {
    warning("system call uname() failed");
    cpu_level = 0;
    return;
  }
  if (buf.machine[0] == 'i'
      && buf.machine[2] == '8'
      && buf.machine[3] == '6'
      && buf.machine[4] == '\0') {
    cpu_level = buf.machine[1];
  } else {
    cpu_level = 0;
  }
}

bool os::is_P6() {
  if (cpu_level < 0) {
    init_cpu_level();
  }
  return cpu_level >= '6';
}

bool os::supports_cmov() {
  return is_P6();
}

int os::max_framesize_without_stack_banging() {
  assert(StackYellowPages > 0,"Sanity check");
  assert(StackRedPages > 0,"Sanity check");

  return vm_page_size();
}

char* os::reserve_memory(size_t size) {
  char *addr = ::mmap(0, size, PROT_READ|PROT_WRITE|PROT_EXEC,
                      MAP_PRIVATE|MAP_NORESERVE|MAP_ANONYMOUS, -1, 0);

  return addr == MAP_FAILED ? NULL : addr;
}
  
char* os::non_memory_address_word() {
  // Must never look like an address returned by reserve_memory,
  // even in its subfields (as defined by the CPU immediate fields,
  // if the CPU splits constants across multiple instructions).

  return (char*) -1;
}

// A lightweight implementation that does not suspend the target thread and
// thus returns only a hint. Used for profiling only!
ExtendedPC os::get_thread_pc(Thread* thread) {
  // Make sure that it is called by the watcher and the Threads lock is owned.
  assert(Thread::current()->is_Watcher_thread(), "Must be watcher and own Threads_lock");
  // For now, is only used to profile the VM Thread
  assert(thread->is_VM_thread(), "Can only be called for VMThread");

  ExtendedPC epc;

  thread->suspend_other(false);  // internal suspend request
  OSThread* osthread = thread->osthread();
  ucontext_t* context = (ucontext_t*) osthread->ucontext();
#if defined(IA32)
  epc = (address) context->uc_mcontext.gregs[EIP];
#else
  #error "should not reach here"
#endif
  thread->resume(false, false);  // don't yield, internal resume request

  return epc;
}

extern "C" void findpc(int x);
extern void set_error_reported();
extern void signalHandler(int sig, siginfo_t* info, ucontext_t* context);

extern "C" int 
JVM_handle_linux_signal(int sig,
                        siginfo_t* info,
                        void* ucVoid,
                        int abort_if_unrecognized)
{
  ucontext_t* uc = (ucontext_t*) ucVoid;

  if (sig == SIGCHLD) {
    // cleanup zombie pid for child thread
    ::waitpid(-1, 0, WNOHANG);
    return 1;
  }

  JavaThread* thread = NULL;
  VMThread* vmthread = NULL;
  if (os::Linux::signal_handlers_are_installed) {
    // Thread* t = Thread::current();
    Thread* t = ThreadLocalStorage::get_thread_slow();  // slow & steady
    if (t != NULL ){
      if(t->is_Java_thread()) {
        thread = (JavaThread*)t;
      }
      else if(t->is_VM_thread()){
        vmthread = (VMThread *)t;
      }
    }
  }
/*
  NOTE: does not seem to work on linux.
  if (info == NULL || info->si_code <= 0 || info->si_code == SI_NOINFO) {
    // can't decode this kind of signal
    info = NULL;
  } else {
    assert(sig == info->si_signo, "bad siginfo");
  }
*/
  // decide if this trap can be handled by a stub
  address stub = NULL;

  address pc          = NULL;
  address adjusted_pc = NULL;

  //%note os_trap_1
  if (info != NULL && thread != NULL) {
    // factor me: getPCfromContext
    pc = (address) uc->uc_mcontext.gregs[EIP];
    #ifndef CORE
    adjusted_pc = thread->safepoint_state()->compute_adjusted_pc(pc);
    #else
    adjusted_pc = pc;
    #endif

    // Handle ALL stack overflow variations here
    if (sig == SIGSEGV && thread != NULL) {
      // HACK: HACK: HACK: cleanup this code.
      // Linux does not support stack banging/probing,
      // and it does not support stack overflow either.
      // Therefore I (hz) manually manage the stack.
      address addr = (address) uc->uc_mcontext.cr2;
      int res = os::Linux::check_stack_overflow(addr);
      switch (res) {
      case -1:
        // stack overflow failed
        tty->print_cr("An irrecoverable stack overflow has occurred.");
        break;
      case 0:
        // null pointer exception
        break;
      case 1:
        // stack overflow succeed
        if (thread->thread_state() != _thread_in_Java) {
          // can not recover from native code stack overflow
          return 1;
        } else {
          // Throw a stack overflow exception
          stub = StubRoutines::i486::handler_for_stack_overflow();
        }
        break;
      case 2:
        // NOTE: check_stack_overflow() successfully expanded the stack.
        return 1;
      default:
        ShouldNotReachHere();
        break;
      }
    }

    // if (thread->last_Java_sp() != NULL)
    if (thread->thread_state() == _thread_in_Java) {
      // Java thread running in Java code => find exception handler if any
      // a fault inside compiled code, the interpreter, or a stub

#ifndef CORE
      if (sig == SIGILL && nativeInstruction_at(pc)->is_illegal()) {
#ifdef COMPILER1
       stub =  Runtime1::entry_for(Runtime1::illegal_instruction_handler_id);
#else
        // debugging trap, or a safepoint
        assert(OptoRuntime::illegal_exception_handler_blob() != NULL, "stub not created yet");
        stub = OptoRuntime::illegal_exception_handler_blob()->instructions_begin();
#endif
        CompiledCodeSafepointHandler *handler = (CompiledCodeSafepointHandler *) thread->safepoint_state()->handle();
        if (handler != NULL && thread->is_in_compiled_safepoint()) {
          nmethod* nm = handler->get_nmethod();
          assert(nm != NULL, "safepoint handler is not setup correctly");

          relocInfo::relocType type = nm->reloc_type_for_address(adjusted_pc);
          assert( ((NativeInstruction*)adjusted_pc)->is_call() || 
        	  (type == relocInfo::return_type) || (type == relocInfo::safepoint_type), 
        	  "Only calls, returns, and backward branches are patched at safepoint");

          if(type == relocInfo::return_type) {
            // The retl case: restore has already happened. The safepoint blob frame will appear 
            // immediately below caller of the orignal method that we have patched. The stack could 
            // be walked properly (the frame of the patched method is already popped). 
            // Will need to be revisited if and when we put restore in the delay slot of ret.
          }
          else {
            // This is the case when either a call or a branch was patched with an illegal instruction.
            // At this point we are done with the patched method and would like things to
            // appear as if the orignal method simply called the safepoint blob at a "safe point".
            // To achieve this, we set up the correct linkage by placing the adjusted trap pc in O7
            // and then "returning" from this trap handler to the safepoint blob.

            // QQQ TODO
            // uc->uc_mcontext.gregs[REG_O7] = (greg_t) adjusted_pc;
          }
        }
      }
      else 
#endif // NOT CORE 

      if (sig == SIGFPE /* && info->si_code == FPE_INTDIV */) {
        // HACK: si_code does not work on linux 2.2.12-20!!!
        int op = pc[0];
        if (op == 0xDB) {
          // FIST
          // TODO: The encoding of D2I in i486.ad can cause an exception
          // prior to the fist instruction if there was an invalid operation
          // pending. We want to dismiss that exception. From the win_32
          // side it also seems that if it really was the fist causing
          // the exception that we do the d2i by hand with different
          // rounding. Seems kind of weird.
          // NOTE: that we take the exception at the NEXT floating point instruction.
          assert(pc[0] == 0xDB, "not a FIST opcode");
          assert(pc[1] == 0x14, "not a FIST opcode");
          assert(pc[2] == 0x24, "not a FIST opcode");
          return true;
        } else if (op == 0xF7) {
          // IDIV
          stub = StubRoutines::i486::handler_for_divide_by_zero();
        } else {
          // TODO: handle more cases if we are using other x86 instructions
          //   that can generate SIGFPE signal on linux.
          tty->print_cr("unknown opcode 0x%X with SIGFPE.", op);
          fatal("please update this code.");
        }
      }

#ifdef COMPILER2
      else if (sig == SIGSEGV && OptoRuntime::illegal_exception_handler_blob() != NULL &&
               OptoRuntime::illegal_exception_handler_blob()->contains(pc)) {
        // This is the case when we use a trap to restore the context at the end of safepoint
        // taken in compiled code. The "return" address is passed in O7
        // QQQ TODO
        ShouldNotReachHere();

#if 0
        greg_t ret_pc = uc->uc_mcontext.gregs[EIP];
        // Recover global regs saved at the original illegal instruction trap. 
        // (%o are handled "automatically" and are possibly modified by the gc - don't touch them)
        thread->restore_global_regs_from_saved_context(uc);
        uc->uc_mcontext.gregs[EIP] =  ret_pc;
        return true;
#endif
      }
#endif

      else if (sig == SIGSEGV &&
               !MacroAssembler::needs_explicit_null_check(uc->uc_mcontext.cr2)) {
        // an access to the first page of VM--assume it is a null pointer
        stub = StubRoutines::i486::handler_for_null_exception();
      }
    }
  }

  if (stub != NULL) {
    // save all thread context in case we need to restore it

    thread->set_saved_exception_pc(pc);
    // TODO: 
    // Fastlane -- is this all? Do we need to save off any context?
    // Currently, no, but might need to if we'll be returning to
    // the context where the trap occurred. Will need to revisit
    // this for compiled safepoint stubs.  factor me: setPC
    uc->uc_mcontext.gregs[EIP] = (greg_t)stub;
    return true;
  }

  // signal-chaining
  if (UseSignalChaining) {
    bool chained = false;
    struct sigaction *actp = os::Linux::get_chained_signal_action(sig);
    if (actp != NULL) {
      chained = os::Linux::chained_handler(actp, sig, info, ucVoid);
    }
    if (chained) {
      // signal-chaining in effect. Continue.
      return true;
    }
  }

  if (sig == SIGPIPE) {
    if (WizardMode || Verbose) {
      warning("Ignoring SIGPIPE - see bug 4229104");
    }
    return 1;
  }
  
  if (!abort_if_unrecognized) {
    // caller wants another chance, so give it to him
    return false;
  }

  if (pc == NULL && uc != NULL) {
    pc = (address) uc->uc_mcontext.gregs[EIP];
  }

#if 0
  #ifndef PRODUCT
  // tell the system that it is about to die
  set_error_reported();

  // Remove this handler now.  It is of no more use.
  os::Linux::set_signal_handler(sig, false);

  if (WizardMode) {
    if (pc != NULL) {
       printf("\n");
       printf("PC information:\n");
       findpc((int)pc);
    }
    if (info != NULL) {
       printf("\n");
       printf("siginfo_t dump:\n");
       for (int i = 0; i < sizeof(info[0])/sizeof(int); i++) {
         printf(" 0x%08X", ((int*)info)[i]);
         if ((i + 1) % 4 == 0)  printf("\n");
       }
       printf("\n");
    }
    if (uc != NULL) {
       printf("\n");
       printf("ucontext_t dump:\n");
       for (int i = 0; i < sizeof(uc[0])/sizeof(int); i++) {
         printf(" 0x%08X", ((int*)uc)[i]);
         if ((i + 1) % 4 == 0)  printf("\n");
       }
       printf("\n");

       tty->print_cr("Unexpected exception occured: signal=%d, pc=%lx", sig, (long)pc);

       // QQQ will we need a stub to bridge into continue_with_dump so that a reasonable
       // return address will be present and the stack looks ok.
       //
       uc->uc_mcontext.gregs[EIP ] = (greg_t)&continue_with_dump;
       return true;
    }
  }
  #endif
#endif

  os::handle_unexpected_exception(thread, sig, pc, (void *)info);

  // reaches here only when the user wants to debug the problem in
  // -XX:+ShowMessageBoxOnError mode. But we can't launch debugger
  // as Windows does, just os::abort.
  os::abort();
}

void os::Linux::init_thread_fpu_state(void) {
  // Hardcoded to set fpu to 53 bit precision. This happens too early to use a stub.
  //
  // pushl  $0x27f
  // fldcw  0(%esp)
  // popl   %eax
  // ret
  // int 3

  static int code_template[] = {
    0x00027f68, 0x246cd900, 0x90c35800, 0xCCCCCCCC
  };

  static void (*fixcw)(void) = CAST_TO_FN_PTR(void (*)(void), code_template);

  fixcw();
}

// This is split from os_linux.cpp because of platform issues but is related to os_win32.cpp
//Reconciliation History
// 1.1 99/05/04 15:28:19 os_linux_i486.cpp        
//End
