// @(#)os_solaris_i486.cpp	1.47 01/03/09 08:17:31
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2
// do not include  precompiled  header file
# include "incls/_os_solaris_i486.cpp.incl"
# include "incls/_os_pd.hpp.incl"

// put OS-includes here
# include <sys/types.h>
# include <sys/mman.h>
# include <pthread.h>
# include <signal.h>
# include <errno.h>
# include <dlfcn.h>
# include <stdio.h>
# include <unistd.h>
# include <sys/resource.h>
# include <thread.h>
# include <sys/stat.h>
# include <sys/time.h>
# include <sys/filio.h>
# include <sys/utsname.h>
# include <sys/systeminfo.h>
# include <sys/socket.h>
# include <sys/lwp.h>
# include <pwd.h>
# include <poll.h>
# include <sys/lwp.h>

# define _STRUCTURED_PROC 1  //  this gets us the new structured proc interfaces of 5.6 & later
# include <sys/procfs.h>     //  see comment in <sys/procfs.h>


# ifdef PCSREG               //  5.6 or newer
# define SOLARIS_STRUCTURED_PROC
# endif

#define MAX_PATH (2 * K)
#define INTERRUPT_SIGNAL SIGUSR1

int os::max_framesize_without_stack_banging() {
  assert(StackYellowPages > 0,"Sanity check");
  assert(StackRedPages > 0,"Sanity check");
    
  // If we aren't using bound threads, then we have to use stack banging
  // because the lwp we are on might not have an alternate signal stack.
  if (!UseBoundThreads) {
      return 0;
  }
  // This needs to be less than min(StackYellowPages,StackRedPages)
  // always. For sparc, we also need to make sure that we
  // cant creep past the gaurd pages with unflushed register
  // windows.
  int yellowSize = StackYellowPages * vm_page_size();
  int redSize = StackRedPages * vm_page_size();
  int size = min(yellowSize,redSize);
  
  return size;
}

// Thanks, Mario Wolzcko for the code below: (Ungar 5/97)

char* os::Solaris::mmap_chunk(char *addr, size_t size, int flags, int prot) {
  char *b = mmap(addr, size, prot, flags, os::Solaris::_dev_zero_fd, 0);

  if (b == MAP_FAILED) {
    return NULL;
  }

  // QQQ TODO this could be moved back to os_solaris.cpp if high_half_mask was factored instead
  // of this entire routine and reserve_memory was also moved backed to os_solaris.cpp

  // Ensure MP-correctness when we patch instructions containing addresses.
  const int high_half_mask = -1 << 24;   // misalignment can separate high byte
  guarantee(((long)b & high_half_mask) != ((long)os::non_memory_address_word() & high_half_mask), "high half of address must not be all-zero");
  
  return b;
}

static size_t reserved_size = 0;
static size_t reserved_increment = 0x1000000;

char* os::reserve_memory(size_t bytes) {
  size_t size = bytes;
  int flags = MAP_PRIVATE | MAP_NORESERVE;
  char* addr = (char*)reserved_size;

  addr += 0x40000000; // eases debugging (wastes address space though)

  reserved_size += size;
  reserved_size = (reserved_size + reserved_increment - 1) & ~(reserved_increment - 1);
  return Solaris::mmap_chunk( addr, size,  flags, PROT_READ | PROT_WRITE | PROT_EXEC);
}
  

char* os::non_memory_address_word() {
  // Must never look like an address returned by reserve_memory,
  // even in its subfields (as defined by the CPU immediate fields,
  // if the CPU splits constants across multiple instructions).

  // On Intel, virtual addresses never have the sign bit set.
  return (char*) -1;
}

// Asynchronous: cancelled by the target thread
void Interrupt_Callback::execute(OSThread::InterruptArguments *args) {
  Thread*     thread = args->thread();

  assert(UseInterruptibleIO, "just checking");
  assert(thread->is_Java_thread(), "can only interrupt a java thread");

  errno = EINTR;

  OSThread* osthread = thread->osthread();

  cancel_at_interrupt();
  // Make sure we are not running on the alternate stack
  assert((address)&thread < (address)thread->stack_base(), "We seem to be running SIGUSR1 on the alternate stack");
  assert((address)&thread > (address)(thread->stack_base() - thread->stack_size()), "We seem to be running SIGUSR1 on the alternate stack");
  #ifdef ASSERT
  stack_t st;
  int retval;
  retval = thr_stksegment(&st);
  assert(retval == 0, "incorrect return value from thr_stksegment");
  assert(st.ss_flags != SS_ONSTACK, "We seem to be running SIGUSR1 on the alternate stack");
  #endif

  sigjmp_buf* buf = (sigjmp_buf*) osthread->jump_buffer();
  if (buf != NULL) {
    siglongjmp(*buf, 1);
  }
}


// Asynchronous: cancelled by the target thread
void SetThreadPC_Callback::execute(OSThread::InterruptArguments *args) {
  Thread*     thread = args->thread();
  ucontext_t* uc     = args->ucontext();

// TODO -- needs work - fastlane
  uc->uc_mcontext.gregs[EIP]     = (greg_t)_addr.pc();
}


// Synchronous: signal sender waits for the frame. processes it and then cancels the callback
void FetchTopFrame_Callback::execute(OSThread::InterruptArguments *args) {
  Thread*     thread = args->thread();
  ucontext_t* uc     = args->ucontext();

// TODO -- needs work - fastlane
  // fastlane always uses callback for safepoints
  assert(!GetThreadState || Arguments::has_profile(), "just checking");
    
  _sp   = (jint*)uc->uc_mcontext.gregs[UESP];
  if (uc->uc_stack.ss_flags != args->ucontext()->uc_stack.ss_flags ||
      uc->uc_stack.ss_sp    != args->ucontext()->uc_stack.ss_sp    ||
      uc->uc_stack.ss_size  != args->ucontext()->uc_stack.ss_size  ) {
    // we have received a corrupted ucontext from the signal handler -- bail

    _sp = NULL;
    ShouldNotReachHere();
  } else {
    ExtendedPC new_addr((address)uc->uc_mcontext.gregs[EIP]);

    _sp   = (jint*)uc->uc_mcontext.gregs[UESP];
    _fp   = (jint*)uc->uc_mcontext.gregs[EBP];
    _addr = new_addr;
  }

  FetchTopFrame_SuspendThreadSelf_Callback cb(&_is_done);
  thread->suspend_thread_self(cb);
}

GetThreadPC_Callback::GetThreadPC_Callback(int priority) : OSThread::InterruptCallback(priority, true) { 

  ExtendedPC addr;
  _is_done = false;
  _addr = addr;
}

GetThreadPC_Callback* GetThreadPC_Callback::create(OSThread* osthread, int priority) {
  GetThreadPC_Callback* cb = (GetThreadPC_Callback *) OSThread::vm_interrupt_callback();
  cb->_is_done = false;
  cb->_priority = priority;
  ExtendedPC addr;
  cb->set_addr(addr);
  cb->set_osthread(osthread);
  return cb;
}

// This is a simple callback that just fetches a PC for an interrupted thread.
// The thread need not be suspended and the fetched PC is just a hint.
// Returned PC and nPC are not necessarily consecutive.
// This one is currently used for profiling the VMThread ONLY!

// Should be synchronous
void GetThreadPC_Callback::execute(OSThread::InterruptArguments *args) {
  Thread*     thread = args->thread();
  ucontext_t* uc     = args->ucontext();
  jint* sp;

  assert(ProfileVM && thread->is_VM_thread(), "just checking");
    
  ExtendedPC new_addr((address)uc->uc_mcontext.gregs[EIP]);
  _addr = new_addr;
  
  // Notify the waiting watcher thread and continue executing 
  // The returned PC is just a hint
  {
    MutexLockerEx ml(ProfileVM_lock, Mutex::_no_safepoint_check_flag);
    _is_done = true;
    ProfileVM_lock->notify_all();
  }
    
  return;
}

// A lightweight implementation that does not suspend the target thread and
// thus returns only a hint. Used for profiling only!
ExtendedPC os::get_thread_pc(Thread* thread) {
  // Make sure that it is called by the watcher and the Threads lock is owned.
  assert(Thread::current()->is_Watcher_thread(), "Must be watcher and own Threads_lock");
  // For now, is only used to profile the VM Thread
  assert(thread->is_VM_thread(), "Can only be called for VMThread");
  ExtendedPC epc;

  // Watcher thread: add GetThreadPC_Callback to the queue for "thread" (may be VM Thread!)
  // Watcher thread: send a signal to "thread" and block until PC is returned
  // "thread": signal handler calls GetThreadPC_Callback which obtains the PC
  // "thread": set the callback to done and notify the waiting watcher thread
  // "thread": continue executing
  // Watcher thread: wake up, get the needed pc from the callback
  // Watcher thread: cancel the callback
  
  GetThreadPC_Callback*  cb = GetThreadPC_Callback::create(thread->osthread());
  Solaris::send_interrupt(thread);

  {      
    // don't use safepoint check because we are the watcher thread and stopping
    // the VMThread
    MutexLockerEx ml(ProfileVM_lock, Mutex::_no_safepoint_check_flag);
    
    while(!cb->is_done()) {
      ProfileVM_lock->wait(Mutex::_no_safepoint_check_flag);
    }
    
    epc = cb->addr();
    thread->osthread()->set_interrupt_in_progress(false);

    assert(cb->is_synchronous(), "must be a sycnhronous callback");
    // Do not really need to cancel anything, since there is a single callback
    // for interrupting the VM thread
  }

  return epc;
}  

#ifdef SOLARIS_STRUCTURED_PROC            // Solaris 5.6 and younger

static int threadgetstate(thread_t tid, int *flags, lwpid_t *lwp, stack_t *ss, gregset_t rs, lwpstatus_t *lwpstatus) {
  char lwpstatusfile[PROCFILE_LENGTH];
  int lwpfd, err;

  if (err = os::Solaris::thr_getstate(tid, flags, lwp, ss, rs))
    return (err);
  if (*flags == TRS_LWPID) {
    sprintf(lwpstatusfile, "/proc/%d/lwp/%d/lwpstatus", getpid(),
	    *lwp);
    if ((lwpfd = open(lwpstatusfile, O_RDONLY)) < 0) {
      perror("thr_mutator_status: open lwpstatus");
      return (EINVAL);
    }
    if (pread(lwpfd, lwpstatus, sizeof (lwpstatus_t), (off_t)0) !=
	sizeof (lwpstatus_t)) {
      perror("thr_mutator_status: read lwpstatus");
      (void) close(lwpfd);
      return (EINVAL);
    }
    (void) close(lwpfd);
  }
  return (0);
}

#endif // SOLARIS_STRUCTURED_PROC

static ucontext_t* get_ucontext_from_siglwp_handler(Thread *thread, address s_pc, jint* s_sp, jint* s_fp);

// two versions os set_thread_pc corresponding to two libthread APIs 
// return false on failure
bool os::Solaris::set_thread_pc_fast(Thread* thread, ExtendedPC addr) {
#ifndef  SOLARIS_STRUCTURED_PROC     // Solaris 5.5.1 or older, no structured /proc 
  ShouldNotReachHere();
  return false;
#else
  // Solaris 5.6 or newer:  structured /proc interface
  assert(os::Solaris::mutator_libthread() && GetThreadState, "Must use new libthread API");

  char lwpctl_file[PROCFILE_LENGTH];
  int flag;
  int lwpctl_fd;
  lwpid_t lwpid;
  gregset_t rs;
  lwpstatus_t lwpstatus;
  int res;

  thread_t tid = thread->osthread()->thread_id();
  res = threadgetstate(tid, &flag, &lwpid, NULL, rs, &lwpstatus);
  assert(res == 0, "threadgetstate() failure");
  if(res) {
    return false;  // it is best to be careful here
  }
 
  assert((flag == TRS_VALID) || (flag == TRS_LWPID) || (flag == TRS_NONVOLATILE), "invalid thread state in set_thread_pc_fast");
 
  switch(flag) {
  case TRS_VALID:         //  really fast cases - register state cached by libthread
    {
      assert(rs[UESP] != NULL, "stack point shouldn't be null in TRS_VALID case");
      rs[EIP] =  (greg_t) addr.pc();
      if( os::Solaris::thr_setstate(tid, flag, rs) == EINVAL ) {
	assert(false, "thr_setstate failed");
	return false;
      }
    }
  break;

  case TRS_LWPID:        // thread stopped while on lwp 
    // Another solution is to try the slow interrupt/based case instead
    {
      assert(lwpstatus.pr_reg[UESP] != 0, "stack point shouldn't be null in TRS_LWPID case");
      
      // Set pc/npc by sending a message to an lwp control file
      lwpstatus.pr_reg[EIP]  = (greg_t) addr.pc();

      sprintf(lwpctl_file, "/proc/%d/lwp/%d/lwpctl", getpid(), lwpid);
      
      if ((lwpctl_fd = open(lwpctl_file, O_WRONLY)) < 0) {
	assert(false, "set_thread_pc_fast: open lwpctl");
	return false;
      }

      // setup the control message and write it to the lwp control file
      lwpstatus.pr_instr = PCSREG;   // set general registers
      if (write(lwpctl_fd, &lwpstatus.pr_instr, sizeof (long) + sizeof (prgregset_t)) !=
	  sizeof (long) + sizeof (prgregset_t)) {
	assert(false, "write to lwpctl failed");
	(void) close(lwpctl_fd);
	return false;
      }
      (void) close(lwpctl_fd);
      return true;
    }
  break;

  case TRS_NONVOLATILE:
    {
      // This is the case when the thread was suspended by interrupting it with a SIGLWP
      // Change eip in SIGLWP's handler ucontext to let it resume at the new address
      assert(rs[UESP] != NULL, "Invalid UESP in set_thread_pc_fast");

      ucontext_t *uc = NULL;

      // get_ucontext_from_siglwp_handler is somewhat dangerous to call. We must call it
      // when thread state is _thread_in_Java if we expect to be able to reach a
      // safepoint with a long (infinite) loop in Java code for other states we just
      // wait and try again.
      // 
      if (thread->is_Java_thread()) {
	JavaThread* jthread = (JavaThread *)thread;
	if (jthread->thread_state() == _thread_in_Java) {
	  uc = get_ucontext_from_siglwp_handler(thread, (address)rs[EIP], (jint*)rs[UESP], (jint*)rs[EBP]);
	}
      }

      if(uc) {
        uc->uc_mcontext.gregs[EIP] =  (greg_t) addr.pc();
        return true;
      } else {
        return false;
      }
    }
    break;

  case TRS_INVALID:
    ShouldNotReachHere();
    return false;
    break;

  default:
    ShouldNotReachHere();
    return false;
    break;
  }
#endif // SOLARIS_STRUCTURED_PROC
}

// Returns a pointer to ucontext from SIGLWP handler by walking up the stack
// NULL indicates failure
static ucontext_t* get_ucontext_from_siglwp_handler(Thread* thread, address s_pc, jint* s_sp, jint* s_fp) {

// This routine is relatively fragile.
// If we wander back into a c2 compiled frame, we can't trust our frame pointer
// any more, as the compiler can use ebp as a general register.
// As a result we try very hard to ensure that no references will cause
// an accidental segv.
//
   ucontext_t *uc = NULL;
   address stackStart = (address) thread->stack_base();
   address stackEnd   = stackStart - thread->stack_size();
   int i = 0;
   jint* s_oldfp = s_fp - 1;  // initialize to smaller value
   intptr_t page_mask = (intptr_t) ~(os::vm_page_size() - 1);

   // walk back on stack until the signal handler is found
   // only follow legitimate frame pointers. Note: this can
   // still be fooled but as long as we don't segv we are ok.
   while (1) {

     // Check for bad pc's and bad relationship between sp and fp.
     // Check alignment and range on fp and sp

     if ( s_pc == (address) -1 || s_sp > s_fp || s_oldfp >= s_fp ||
	 (intptr_t) s_sp & 0x3 || (address) s_sp > stackStart - wordSize || (address) s_sp <= stackEnd ||
	 (intptr_t) s_fp & 0x3 || (address) s_fp > stackStart - wordSize || (address) s_fp <= stackEnd ) {
         // The real stackStart seems to be one word less that what is returned by thr_stksegment.
       return NULL;
     }

     // The siglwp handler is known to be 4-7 frames back
     if (++i > 10) {
       return NULL;
     }
     if (s_pc >= os::Solaris::handler_start && s_pc < os::Solaris::handler_end) {
       frame sighandler_frame(s_sp, s_fp, s_pc);
       return *(ucontext_t **)sighandler_frame.native_param_addr(2);
     }

     frame fr(s_sp, s_fp, s_pc);
     s_sp = fr.sender_sp();
     s_oldfp = s_fp;

     s_fp = fr.link();
     s_pc = fr.sender_pc();
  }
  ShouldNotReachHere();

  return NULL;
}

// This is a "fast" implementation of fetch_top_frame that relies on the new libthread
// APIs to get pc for a suspended thread
ExtendedPC os::Solaris::fetch_top_frame_fast(Thread* thread, jint** sp, jint** fp) {
#ifndef SOLARIS_STRUCTURED_PROC       // 5.5.1 or older
  ShouldNotReachHere();
  ExtendedPC noaddr;
  return noaddr;
#else
  int flag;
  lwpid_t lwpid;
  gregset_t rs;
  lwpstatus_t lwpstatus;
  int res;
  ExtendedPC addr;
  
  #ifdef ASSERT
  memset(&rs[0], 0xff, sizeof(gregset_t));
  assert(os::Solaris::mutator_libthread() && GetThreadState, "Must use new libthread API");
  #endif

  thread_t tid = thread->osthread()->thread_id();
  res = threadgetstate(tid, &flag, &lwpid, NULL, rs, &lwpstatus);
  // Let caller know about failure 
  // thread self-suspension uses wait on Interrupt_Lock, so
  // thread is not t_stop or t_stopallmutators
//  assert(res == 0, "threadgetstate() failure"); // guarantee
  if(res) {
    ExtendedPC new_addr(NULL);
    *sp   = (jint*)0;
    *fp   = (jint*)0;
    addr = new_addr;
    return addr;  // bail out
  }
  
  switch(flag) {
  case TRS_NONVOLATILE:   
      // called when thread voluntarily gave up control via thr_suspend().
    // Even worse, the register values that libthread returns are for the libthread's
    // own SIGLWP handler - a suspended thread sleeps on a synchronization object
    // inside this handler... 
    if (Arguments::has_profile() || UseStrictCompilerSafepoints || jvmpi::enabled()) {
	ucontext_t *uc = NULL;

	// get_ucontext_from_siglwp_handler is too dangerous to call unless thread state
	// is _thread_in_Java. It is dangerous then too but we must use it if we expect
	// to be able to reach a safepoint with a long (infinite) loop in Java code.
	// 
	if (thread->is_Java_thread()) {
	  JavaThread* jthread = (JavaThread *)thread;
	  if (jthread->thread_state() == _thread_in_Java) {
	    uc = get_ucontext_from_siglwp_handler(thread, (address)rs[EIP], (jint*)rs[UESP], (jint*)rs[EBP]);
	  }
	}
        // if we can not find the signal handler on the stack,
        // set new_addr to null, which get_top_frame will use
        // to return false, which causes examine_state_of_thread to
        // roll_forward and SafepointSynchronize::begin() will
        // retry this thread again.
        if (uc == NULL) {
            ExtendedPC new_addr(NULL);
            *sp   = (jint*)0;
            *fp   = (jint*)0;
            addr = new_addr;
        }
        else {
            ExtendedPC new_addr((address)uc->uc_mcontext.gregs[EIP]);
            addr = new_addr;
	    assert(uc->uc_mcontext.gregs[EIP] != NULL, "cannot be null");
	    *sp = (jint *)uc->uc_mcontext.gregs[UESP];
	    *fp = (jint *)uc->uc_mcontext.gregs[EBP];
        }
        break;
    }
    else
      ;// fallthrough to invalid case
  case TRS_INVALID:
    {
      ExtendedPC new_addr(NULL);
      *sp   = (jint*)0;
      *fp   = (jint*)0;
      addr = new_addr;
    }
  break;


  case TRS_VALID:         // the entire register set is cached for us by libthread
    {
      assert(rs[UESP] != NULL, "stack point shouldn't be null in TRS_VALID case");
      ExtendedPC new_addr((address)rs[EIP]);
      *sp   = (jint*)rs[UESP];
      *fp   = (jint*)rs[EBP];
      assert(rs[EIP] != NULL, "cannot be null");
      addr = new_addr;
    }
  break;

  case TRS_LWPID:        // got a full register set from the /proc interface
    // Fastlane?
    {
      assert(lwpstatus.pr_reg[UESP] != 0, "stack point shouldn't be null in TRS_LWPID case");
      ExtendedPC new_addr((address)lwpstatus.pr_reg[EIP]);
      assert(lwpstatus.pr_reg[EIP] != NULL, "cannot be null");
      *sp   = (jint*)lwpstatus.pr_reg[UESP];
      *fp   = (jint*)lwpstatus.pr_reg[EBP];
      addr = new_addr;
    }
  break;
  default:
    break;
  }
  
  return addr;

#endif   // SOLARIS_STRUCTURED_PROC
}


// An alternative implementation that obtains a pc for a suspended thread by sending
// a signal to a thread and examining its context in the signal handler.
ExtendedPC os::Solaris::fetch_top_frame_slow(Thread* thread, jint** sp, jint** fp) {
  // NOTE: For Solaris/Intel, we always get the information we need by forcing
  // the thread of interest to be the currently running process, and then
  // getting the context from it. Can we find this context in another way?
  
  if ((!UseCompilerSafepoints || GetThreadState) && !Arguments::has_profile()) {
    // just a kludge that won't crash, but sets the pc to NULL
    // the vm will restart the thread and stop it again later
    
    // When GetThreadState is set and we get here, it means that our intention probably
    // was to use the new libthread, but it was not found on LD_LIBRARY_PATH 
    *sp = *fp = NULL;
    
    ExtendedPC addr;
    return addr;
  }

  // The requesting thread is either the VMThread or the WatcherThread
  // for flat profiling. Here is the dance that we do to fetch_top_frame()
  // using interrupt callbacks:
  // 
  // req thread: <already has the JavaThread suspended before we get here>
  // req thread: add the FetchTopFrame_Callback to the JavaThread
  // req thread: send a signal to the JavaThread and resume it
  // req thread: wait while the callback is not done
  // JavaThread: get the signal and call the FetchTopFrame_Callback
  //             (which stashes the info)
  // JavaThread: set the callback to done and notify the req thread
  // JavaThread: suspend_thread_self
  // req thread: wake up and install compiled code traps
  // req thread: <resumes the JavaThread when we return from here>
  // JavaThread: wakes up and hits the trap, vectoring to the correct
  //             safepoint code
  
  FetchTopFrame_Callback* cb = FetchTopFrame_Callback::create();
  
  OSThread *osthread = thread->osthread();
  
  osthread->add_interrupt_callback(cb);
  send_interrupt(thread);
  
  thread->resume(false, false);  // don't yield, internal resume request
  
  ExtendedPC addr;
  {
    // don't use safepoint check because we might be the WatcherThread.
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    
    while(!cb->is_done()) {
      Interrupt_lock->wait(Mutex::_no_safepoint_check_flag);
    }
    
    // copy the needed info from the callback
    *fp = cb->fp();
    *sp = cb->sp();
    addr = cb->addr();
    
    // and now cancel the callback
    assert(cb->is_synchronous(), "must be a synchronous callback");
    cb->cancel();
  }
  return addr;
}  

extern "C" void findpc(int x);
extern void set_error_reported();

extern "C" int JVM_handle_solaris_signal(int signo, siginfo_t* siginfo, void* ucontext, int abort_if_unrecognized);

int JVM_handle_solaris_signal(int sig, siginfo_t* info, void* ucVoid, int abort_if_unrecognized) {
  ucontext_t* uc = (ucontext_t*) ucVoid;

  if ((sig == SIGINT) || (sig == SIGHUP) || (sig == SIGTERM)) { 
    // Handled to cleanup ISM if necessary.  
    assert(UseISM && !UsePermISM, "These signals are caught only when we are using non-sticky ISM");
    os::exit(1);
  }

  JavaThread* thread = NULL;
  VMThread* vmthread = NULL;
  if (os::Solaris::signal_handlers_are_installed) {
    // Thread* t = Thread::current();
    Thread* t = ThreadLocalStorage::get_thread_slow();  // slow & steady
    if (t != NULL ){
      if(t->is_Java_thread()) {
	thread = (JavaThread*)t;
      }
      else if(t->is_VM_thread()){
	vmthread = (VMThread *)t;
      }
    }
  }

  if (sig == INTERRUPT_SIGNAL) {
    if(thread){
      OSThread::InterruptArguments args(thread, uc);
      thread->osthread()->do_interrupt_callbacks_at_interrupt(&args);
    } 
    else if(vmthread){
      OSThread::InterruptArguments args(vmthread, uc);
      vmthread->osthread()->do_vm_interrupt_callback_at_interrupt(&args);      
    }
    return true;
  }

  if (info == NULL || info->si_code <= 0 || info->si_code == SI_NOINFO) {
    // can't decode this kind of signal
    info = NULL;
  } else {
    assert(sig == info->si_signo, "bad siginfo");
  }

  // decide if this trap can be handled by a stub
  address stub = NULL;

  address pc          = NULL;
  address adjusted_pc = NULL;

  //%note os_trap_1
  if (info != NULL && thread != NULL) {
    // factor me: getPCfromContext
    pc = (address) uc->uc_mcontext.gregs[EIP];
    #ifndef CORE
    adjusted_pc = thread->safepoint_state()->compute_adjusted_pc(pc);
    #else
    adjusted_pc = pc;
    #endif

    // Handle ALL stack overflow variations here
    if (UseStackBanging && sig == SIGSEGV && thread) {
      bool is_yellow = thread->in_stack_yellow_zone((address)info->si_addr);
      bool is_red = thread->in_stack_red_zone((address)info->si_addr);
      // QQQ - Fastlane - track the real fix for this one which is
      // to disable the yellow_zone, determine the frame in which
      // it can be re-enabled and do so once that frame is 
      // reentered. Coomes email 12/10/99
      if(is_yellow) {
        // Temporary fix for bug 4296554--keep yellow zone enabled.
	// Bad. We can't keep it enabled because then we can segv
	// until we reach the red zone and then we die. This isn't
	// by any means a real fix for stackover problem since we
	// really need to re-enable the yellow zone but this is
	// a complex problem.
	// 
        thread->disable_stack_yellow_zone();
        // Yellow zone access violations can be recovered from.
        // If the thread isn't in java land, all we can do is return.
        if (thread->thread_state() != _thread_in_Java) {
          return true; // Try to finish
        } else {
          // Throw a stack overflow exception
          stub = StubRoutines::i486::handler_for_stack_overflow();
        }
      } else if(is_red) {
        // Any red zone access violation is fatal.
        tty->print_cr("An irrecoverable stack overflow has occurred.");
        thread->disable_stack_yellow_zone();
        thread->disable_stack_red_zone();
      }
    }

    // if (thread->last_Java_sp() != NULL)
    if (thread->thread_state() != _thread_in_Java) {
      // a fault inside the VM, or native code.
      if (UseStackBanging && sig == SIGSEGV && info->si_code == SEGV_ACCERR
          && thread && thread->in_stack_yellow_zone((address)info->si_addr)) { 
        // For now, all we can do is remove the yellow pages and hope for the
        // best. In the future, we should use the vm exit macros to check for
        // missing yellow pages and reinsert them.
        thread->disable_stack_yellow_zone();
        return true;
      } else if (UseStackBanging && sig == SIGSEGV && info->si_code == SEGV_ACCERR
          && thread && thread->in_stack_red_zone((address)info->si_addr)) { 
        // This is always a fatal error. Do we want to do anything more here?
        tty->print_cr("An irrecoverable stack overflow has occurred.");
      }
    } else {
      // Java thread running in Java code => find exception handler if any
      // a fault inside compiled code, the interpreter, or a stub
#ifndef CORE
      if (sig == SIGILL && nativeInstruction_at(pc)->is_illegal()) {
#ifdef COMPILER1
       stub =  Runtime1::entry_for(Runtime1::illegal_instruction_handler_id);
#else
	// debugging trap, or a safepoint
	assert(OptoRuntime::illegal_exception_handler_blob() != NULL, "stub not created yet");
	stub = OptoRuntime::illegal_exception_handler_blob()->instructions_begin();
#endif

	CompiledCodeSafepointHandler *handler = (CompiledCodeSafepointHandler *) thread->safepoint_state()->handle();
	if (handler != NULL && thread->is_in_compiled_safepoint()) {
	  nmethod* nm = handler->get_nmethod();
	  assert(nm != NULL, "safepoint handler is not setup correctly");

	  relocInfo::relocType type = nm->reloc_type_for_address(adjusted_pc);
	  assert( ((NativeInstruction*)adjusted_pc)->is_call() || 
		  (type == relocInfo::return_type) || (type == relocInfo::safepoint_type), 
		  "Only calls, returns, and backward branches are patched at safepoint");

	    // if(type == relocInfo::return_type) {
	    // The retl case: restore has already happened. The safepoint blob frame will appear 
	    // immediately below caller of the orignal method that we have patched. The stack could 
	    // be walked properly (the frame of the patched method is already popped). 
	    // Sparc: Will need to be revisited if and when we put restore in the delay slot of ret.
	    // }
	    // else {
	    // This is the case when either a call or a branch was patched with an illegal instruction.
	    // At this point we are done with the patched method and would like things to
	    // appear as if the orignal method simply called the safepoint blob at a "safe point".
	    // Sparc: To achieve this, we set up the correct linkage by placing the adjusted trap pc in O7
	    // and then "returning" from this trap handler to the safepoint blob.
            // Intel: to achieve this the handle_illegal_instruction_exception code 
            // called by the stub will place saved_exception_pc into %sp and return there

	    // }
	}
      }
      else if (sig == SIGFPE && info->si_code == FPE_INTDIV) {
#else
      if (sig == SIGFPE && info->si_code == FPE_INTDIV) {
#endif
// TODO -- needs work - fastlane
// is_zombie needs factoring?
	// integer divide by zero
// TODO -- needs work - fastlane
        stub = StubRoutines::i486::handler_for_divide_by_zero();
      }
      else if (sig == SIGFPE && info->si_code == FPE_FLTDIV) {
	// floating-point divide by zero
// TODO -- needs work - fastlane
        stub = StubRoutines::i486::handler_for_divide_by_zero();
      }
      else if (sig == SIGFPE && info->si_code == FPE_FLTINV) {
	// The encoding of D2I in i486.ad can cause an exception prior
	// to the fist instruction if there was an invalid operation 
	// pending. We want to dismiss that exception. From the win_32
	// side it also seems that if it really was the fist causing
	// the exception that we do the d2i by hand with different
	// rounding. Seems kind of weird. QQQ TODO
	// Note that we take the exception at the NEXT floating point instruction.
	if (pc[0] == 0xDB) {
	    assert(pc[0] == 0xDB, "not a FIST opcode");
	    assert(pc[1] == 0x14, "not a FIST opcode");
	    assert(pc[2] == 0x24, "not a FIST opcode");
	    return true;
	} else {
	    assert(pc[-3] == 0xDB, "not an flt invalid opcode");
	    assert(pc[-2] == 0x14, "not an flt invalid opcode");
	    assert(pc[-1] == 0x24, "not an flt invalid opcode");
	}
      }
      else if (sig == SIGFPE ) {
        tty->print_cr("caught SIGFPE, info 0x%x.", info->si_code);
      }
	// QQQ It doesn't seem that we need to do this on x86 because we should be able
	// to return properly from the handler without this extra stuff on the back side.

#ifdef COMPILER2
      else if (sig == SIGSEGV && OptoRuntime::illegal_exception_handler_blob() != NULL &&
               OptoRuntime::illegal_exception_handler_blob()->contains(pc)) {
	// This is the case when we use a trap to restore the context at the end of safepoint
	// taken in compiled code. The "return" address is passed in O7
	// QQQ TODO
        ShouldNotReachHere();

#if 0
	greg_t ret_pc = uc->uc_mcontext.gregs[EIP];
	// Recover global regs saved at the original illegal instruction trap. 
	// (%o are handled "automatically" and are possibly modified by the gc - don't touch them)
	thread->restore_global_regs_from_saved_context(uc);
	uc->uc_mcontext.gregs[EIP] =  ret_pc;
	return true;
#endif
      }
#endif

      else if (sig == SIGSEGV && info->si_code > 0 && !MacroAssembler::needs_explicit_null_check((int)info->si_addr)) {
	// an access to the first page of VM--assume it is a null pointer
        stub = StubRoutines::i486::handler_for_null_exception();
      }
    }
  }

  if (stub != NULL) {
    // save all thread context in case we need to restore it

    thread->set_saved_exception_pc(pc);
    // 12/02/99: On Sparc it appears that the full context is alsosaved
    // but as yet, no one looks at or restores that saved context
    // factor me: setPC
    uc->uc_mcontext.gregs[EIP] = (greg_t)stub;
    return true;
  }

  // signal-chaining
  if (UseSignalChaining) {
    bool chained = false;
    struct sigaction *actp = os::Solaris::get_chained_signal_action(sig);
    if (actp != NULL) {
      chained = os::Solaris::chained_handler(actp, sig, info, ucVoid);
    }
    if (chained) {
      // signal-chaining in effect. Continue.
      return true;
    }
  }

  if(sig == SIGPIPE) {
    if (WizardMode || Verbose) {
      warning("Ignoring SIGPIPE - see bug 4229104");
    }
    return true;
  }

  if (!abort_if_unrecognized) {
    // caller wants another chance, so give it to him
    return false;
  }

  if (!os::Solaris::libjsig_is_loaded) {
    struct sigaction oldAct;
    sigaction(sig, (struct sigaction *)0, &oldAct);
    if (oldAct.sa_sigaction != signalHandler) {
      void* sighand = oldAct.sa_sigaction ? CAST_FROM_FN_PTR(void*,  oldAct.sa_sigaction)
					  : CAST_FROM_FN_PTR(void*, oldAct.sa_handler);
      warning("Unexpected Signal %d occured under user-defined signal handler %#lx", sig, (long)sighand);
    }
  }

  if (pc == NULL && uc != NULL) {
    pc = (address) uc->uc_mcontext.gregs[EIP];
  }

  #ifndef PRODUCT
  // tell the system that it is about to die
  set_error_reported();

  // Remove this handler now.  It is of no more use.
  os::Solaris::set_signal_handler(sig, false);

  if (WizardMode) {
    if (pc != NULL) {
       printf("\n");
       printf("PC information:\n");
       findpc((int)pc);
    }
    if (info != NULL) {
       printf("\n");
       printf("siginfo_t dump:\n");
       for (int i = 0; i < sizeof(info[0])/sizeof(int); i++) {
	 printf(" 0x%08X", ((int*)info)[i]);
	 if ((i + 1) % 4 == 0)  printf("\n");
       }
       printf("\n");
    }
    if (uc != NULL) {
       printf("\n");
       printf("ucontext_t dump:\n");
       for (int i = 0; i < sizeof(uc[0])/sizeof(int); i++) {
	 printf(" 0x%08X", ((int*)uc)[i]);
	 if ((i + 1) % 4 == 0)  printf("\n");
       }
       printf("\n");

       tty->print_cr("Unexpected exception occured: signal=%d, pc=%lx", sig, (long)pc);

       // QQQ will we need a stub to bridge into continue_with_dump so that a reasonable
       // return address will be present and the stack looks ok.
       //
       uc->uc_mcontext.gregs[EIP ] = (greg_t)&continue_with_dump;
       return true;
    }
  }
  #endif

  os::handle_unexpected_exception(thread, sig, pc, (void *)info);

  // reaches here only when the user wants to debug the problem in
  // -XX:+ShowMessageBoxOnError mode. But we can't launch debugger
  // as Windows does, just os::abort.
  os::abort();
}

// Should really be in safepoint_solaris_i486.cpp but this would be all it contained
// and all the other platforms would need a safepoint_<os>_<cpu>.cpp which seems like
// overkill.
// The patching of the return instruction is machine-specific
void SafepointSynchronize::patch_return_instruction_md(address cb_pc) {
  // The relocation info. is actually after the ret (i.e., where we are going to patch with
  // an illegal instruction.
  cb_pc -= NativeReturn::instruction_size;

  assert(((NativeInstruction*)cb_pc)->is_return(), "must be a return instruction");

  // In C1  code is part of ret eventually
  NOT_COMPILER1(assert(((NativeInstruction*)(cb_pc+1))->is_nop(), "must be a nop instruction");)
  NOT_COMPILER1(assert(((NativeInstruction*)(cb_pc+2))->is_nop(), "must be a nop instruction");)
  assert(NativePopReg::instruction_size + NativeIllegalInstruction::instruction_size == 3, "wrong code format");
  NativePopReg::insert(cb_pc, ebx); // Pop return address (it is getting pushed by the SafepointBlob again)
  NativeIllegalInstruction::insert(cb_pc + NativePopReg::instruction_size);        
}

void SafepointSynchronize::save_thread_state_at_safepoint(JavaThread *thread) {
  // Nothing to do on Intel Solaris, no thread state to cache, windows to flush.
}

void os::Solaris::init_thread_fpu_state(void) {
  // Hardcoded to set fpu to 53 bit precision. This happens too early
  // to use a stub.
  //
  // pushl  $0x27f
  // fldcw  0(%esp)
  // popl   %eax
  // ret
  // nop ...

  static int code_template[] = {
    0x00027f68, 0x246cd900, 0x90c35800, 0x90909090
  };
  static void (*fixcw)(void) = CAST_TO_FN_PTR(void (*)(void), code_template);

  fixcw();
}

// JVMPI code
bool os::thread_is_running(JavaThread* tp) {
  int         flag;
  lwpid_t     lwpid;
  gregset_t   reg;
  lwpstatus_t lwpstatus;
  int         res;
  thread_t    tid = tp->osthread()->thread_id();
  res = threadgetstate(tid, &flag, &lwpid, NULL, reg, &lwpstatus);
  assert(res == 0, "threadgetstate() failure");
  if(res != 0) return false; // Safe return value

  unsigned int sum = 0;
  // give TRS_NONVOLATILE doesn't trust any other registers, just use these
  sum += reg[UESP];  sum += reg[EIP];   sum += reg[EBP];

  if (tp->last_sum() == sum) {
    return false;
  } else {
    tp->set_last_sum(sum);
    return true;
  }
}

// This is split from os_solaris.cpp because of platform issues but is related to os_win32.cpp
//Reconciliation History
// 1.287 99/04/01 18:31:08 os_win32.cpp
// 1.288 99/04/12 12:56:48 os_win32.cpp
// 1.293 99/07/06 16:36:49 os_win32.cpp
// 1.294 99/07/13 11:41:18 os_win32.cpp
//End
