// @(#)os_solaris_sparc.cpp	1.44 01/03/09 08:17:43
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2
// do not include  precompiled  header file

# include <signal.h>        // needed first to avoid name collision for "std" with SC 5.0

# include "incls/_os_solaris_sparc.cpp.incl"

// put OS-includes here
# include <sys/types.h>
# include <sys/mman.h>
# include <pthread.h>
# include <errno.h>
# include <dlfcn.h>
# include <stdio.h>
# include <unistd.h>
# include <sys/resource.h>
# include <thread.h>
# include <sys/stat.h>
# include <sys/time.h>
# include <sys/filio.h>
# include <sys/utsname.h>
# include <sys/systeminfo.h>
# include <sys/socket.h>
# include <sys/lwp.h>
# include <pwd.h>
# include <poll.h>
# include <sys/lwp.h>

# define _STRUCTURED_PROC 1  //  this gets us the new structured proc interfaces of 5.6 & later
# include <sys/procfs.h>     //  see comment in <sys/procfs.h>

# ifdef PCSREG               //  5.6 or newer
# define SOLARIS_STRUCTURED_PROC
# endif

#define MAX_PATH (2 * K)
#define INTERRUPT_SIGNAL SIGUSR1

typedef void store_func_t               (jint, jint*);

store_func_t* os::Solaris::atomic_store_func
  = os::Solaris::atomic_store_bootstrap;

int os::Solaris::max_register_window_saves_before_flushing() {
  // We should detect this at run time. For now, filling
  // in with a constant.
  return 8;
}

int os::max_framesize_without_stack_banging() {
  assert(StackYellowPages > 0,"Sanity check");
  assert(StackRedPages > 0,"Sanity check");
  assert(os::Solaris::max_register_window_saves_before_flushing() > 0,"Sanity check");
  
  // If we aren't using bound threads, then we have to use stack banging
  // because the lwp we are on might not have an alternate signal stack.
  if (!UseBoundThreads) {
      return 0;
  }
  
  // This needs to be less than min(StackYellowPages,StackRedPages)
  // always. For sparc, we also need to make sure that we
  // cant creep past the gaurd pages with unflushed register
  // windows.
  int yellowSize = StackYellowPages * vm_page_size();
  int redSize = StackRedPages * vm_page_size();
  int size = min(yellowSize,redSize);
  size /= os::Solaris::max_register_window_saves_before_flushing();
  
  return size;
}

static void handle_unflushed_register_windows(gwindows_t *win) {
  int restore_count = win->wbcnt;
  int i;

  for(i=0; i<restore_count; i++) {
    address sp = (address)win->spbuf[i];
    address reg_win = (address)&win->wbuf[i];
    memcpy(sp,reg_win,sizeof(struct rwindow));
  }
}


// Thanks, Mario Wolzcko for the code below: (Ungar 5/97)

char* os::Solaris::mmap_chunk(char *addr, size_t size, int flags, int prot) {
  char *b = mmap(addr, size, prot, flags, os::Solaris::_dev_zero_fd, 0);

  if (b == MAP_FAILED) {
    return NULL;
  }

  // QQQ TODO this could be moved back to os_solaris.cpp if high_half_mask was factored instead
  // of this entire routine

  // Ensure MP-correctness when we patch instructions containing addresses.
  const int high_half_mask = -1 << 10;   // sethi has a 22-bit immediate
  guarantee(((long)b & high_half_mask) != ((long)os::non_memory_address_word() & high_half_mask), "high half of address must not be all-zero");

  return b;
}

static size_t reserved_size = 0;
static size_t reserved_increment = 0x1000000;

char* os::reserve_memory(size_t bytes) {
  size_t size = bytes;

  // On Sparc, addr is not critical,
  // for Intel Solaris, though, must ensure heap is at address with
  // sentinel bit set -- assuming stack is up there, too.
  // - Ungar 11/97

  int flags = MAP_PRIVATE | MAP_NORESERVE;
  char* addr = (char*)reserved_size;

  addr += 0x40000000; // eases debugging

  reserved_size += size;
  reserved_size = (reserved_size + reserved_increment - 1) & ~(reserved_increment - 1);
  return Solaris::mmap_chunk( addr, size,  flags, PROT_READ | PROT_WRITE | PROT_EXEC);
}

char* os::non_memory_address_word() {
  // Must never look like an address returned by reserve_memory,
  // even in its subfields (as defined by the CPU immediate fields,
  // if the CPU splits constants across multiple instructions).
  // On SPARC, 0 != %hi(any real address), because there is no
  // allocation in the first 1Kb of the virtual address space.
  return (char*) 0;
}

void Interrupt_Callback::execute(OSThread::InterruptArguments *args) {
    Thread*     thread = args->thread();

    assert(UseInterruptibleIO, "just checking");
    assert(thread->is_Java_thread(), "can only interrupt a java thread");

    errno = EINTR;

    OSThread* osthread = thread->osthread();

    cancel_at_interrupt();
    // Make sure we are not running on the alternate stack
    assert((address)&thread < (address)thread->stack_base(), "We seem to be running SIGUSR1 on the alternate stack");
    assert((address)&thread > (address)(thread->stack_base() - thread->stack_size()), "We seem to be running SIGUSR1 on the alternate stack");
    #ifdef ASSERT
    stack_t st;
    int retval;
    retval = thr_stksegment(&st);
    assert(retval == 0, "incorrect return value from thr_stksegment");
    assert(st.ss_flags != SS_ONSTACK, "We seem to be running SIGUSR1 on the alternate stack");
    #endif

    sigjmp_buf* buf = (sigjmp_buf*) osthread->jump_buffer();
    if (buf != NULL) {
      siglongjmp(*buf, 1);
    }
}


// Asynchronous: cancelled by the target thread
void SetThreadPC_Callback::execute(OSThread::InterruptArguments *args) {
    Thread*     thread = args->thread();
    ucontext_t* uc     = args->ucontext();

    // fastlane always uses callback for safepoints
    assert(UseCompilerSafepoints || Arguments::has_profile(), "just checking");

    uc->uc_mcontext.gregs[REG_PC ] = (greg_t)_addr.pc();
    uc->uc_mcontext.gregs[REG_nPC] = (greg_t)_addr.npc();
}

// Synchronous: signal sender waits for the frame. processes it and then cancels the callback
void FetchTopFrame_Callback::execute(OSThread::InterruptArguments *args) {
  Thread*     thread = args->thread();
  ucontext_t* uc     = args->ucontext();

// TODO -- needs work - fastlane
  // fastlane always uses callback for safepoints
  assert(!GetThreadState || Arguments::has_profile(), "just checking");
    
  _sp   = (jint*)uc->uc_mcontext.gregs[REG_SP];
  if (uc->uc_stack.ss_flags != args->ucontext()->uc_stack.ss_flags ||
      uc->uc_stack.ss_sp    != args->ucontext()->uc_stack.ss_sp    ||
      uc->uc_stack.ss_size  != args->ucontext()->uc_stack.ss_size  || 
      (thread->is_Java_thread() &&
       !frame::is_valid_stack_pointer(((JavaThread*)thread)->base_of_stack_pointer(), NULL, _sp))) {
    // we have received a corrupted ucontext from the signal handler -- bail

    _sp = NULL;
  }
  else {
    ExtendedPC new_addr((address)uc->uc_mcontext.gregs[REG_PC],
			(address)uc->uc_mcontext.gregs[REG_nPC]);

    _sp   = (jint*)uc->uc_mcontext.gregs[REG_SP];
    _addr = new_addr;
  }

  FetchTopFrame_SuspendThreadSelf_Callback cb(&_is_done);
  thread->suspend_thread_self(cb);
}


GetThreadPC_Callback::GetThreadPC_Callback(int priority) : OSThread::InterruptCallback(priority, true) { 
  ExtendedPC addr(NULL,NULL);

  _is_done = false;
  _addr = addr;
}

GetThreadPC_Callback* GetThreadPC_Callback::create(OSThread* osthread, int priority) {
  GetThreadPC_Callback* cb = (GetThreadPC_Callback *) OSThread::vm_interrupt_callback();
  cb->_is_done = false;
  cb->_priority = priority;
  ExtendedPC addr(NULL,NULL);
  cb->set_addr(addr);
  cb->set_osthread(osthread);
  return cb;
}

void GetThreadPC_Callback::execute(OSThread::InterruptArguments *args) {
  Thread*     thread = args->thread();
  ucontext_t* uc     = args->ucontext();
  jint* sp;

  assert(ProfileVM && thread->is_VM_thread(), "just checking");
    
  // Skip the mcontext corruption verification. If if occasionally
  // things get corrupt, it is ok for profiling - we will just get an unresolved 
  // function name
  ExtendedPC new_addr((address)uc->uc_mcontext.gregs[REG_PC],
		      (address)uc->uc_mcontext.gregs[REG_nPC]);
  _addr = new_addr;
    
  // Notify the waiting watcher thread and continue executing 
  // The returned PC is just a hint
  {
    MutexLockerEx ml(ProfileVM_lock, Mutex::_no_safepoint_check_flag);
    _is_done = true;
    ProfileVM_lock->notify_all();
  }
  
  return;
}

// A lightweight implementation that does not suspend the target thread and
// thus returns only a hint. Used for profiling only!
ExtendedPC os::get_thread_pc(Thread* thread) {
  // Make sure that it is called by the watcher and the Threads lock is owned.
  assert(Thread::current()->is_Watcher_thread(), "Must be watcher and own Threads_lock");
  // For now, is only used to profile the VM Thread
  assert(thread->is_VM_thread(), "Can only be called for VMThread");
  ExtendedPC epc(NULL, NULL);

  // Watcher thread: add GetThreadPC_Callback to the queue for "thread" (may be VM Thread!)
  // Watcher thread: send a signal to "thread" and block until PC is returned
  // "thread": signal handler calls GetThreadPC_Callback which obtains the PC
  // "thread": set the callback to done and notify the waiting watcher thread
  // "thread": continue executing
  // Watcher thread: wake up, get the needed pc from the callback
  // Watcher thread: cancel the callback
  
  GetThreadPC_Callback*  cb = GetThreadPC_Callback::create(thread->osthread());
  Solaris::send_interrupt(thread);

  {      
    // don't use safepoint check because we are the watcher thread and stopping
    // the VMThread
    MutexLockerEx ml(ProfileVM_lock, Mutex::_no_safepoint_check_flag);
    
    while(!cb->is_done()) {
      ProfileVM_lock->wait(Mutex::_no_safepoint_check_flag);
    }
    
    epc = cb->addr();
    thread->osthread()->set_interrupt_in_progress(false);

    assert(cb->is_synchronous(), "must be a sycnhronous callback");
    // Do not really need to cancel anything, since there is a single callback
    // for interrupting the VM thread
  }

  return epc;
}  

#ifdef SOLARIS_STRUCTURED_PROC            // Solaris 5.6 and younger

static int threadgetstate(thread_t tid, int *flags, lwpid_t *lwp, stack_t *ss, gregset_t rs, lwpstatus_t *lwpstatus) {
  char lwpstatusfile[PROCFILE_LENGTH];
  int lwpfd, err;

  if (err = os::Solaris::thr_getstate(tid, flags, lwp, ss, rs))
    return (err);
  if (*flags == TRS_LWPID) {
    sprintf(lwpstatusfile, "/proc/%d/lwp/%d/lwpstatus", getpid(),
	    *lwp);
    if ((lwpfd = open(lwpstatusfile, O_RDONLY)) < 0) {
      perror("thr_mutator_status: open lwpstatus");
      return (EINVAL);
    }
    if (pread(lwpfd, lwpstatus, sizeof (lwpstatus_t), (off_t)0) !=
	sizeof (lwpstatus_t)) {
      perror("thr_mutator_status: read lwpstatus");
      (void) close(lwpfd);
      return (EINVAL);
    }
    (void) close(lwpfd);
  }
  return (0);
}

#endif // SOLARIS_STRUCTURED_PROC

static ucontext_t* get_ucontext_from_siglwp_handler(jint* s_sp, Thread* thread);

// two versions of set_thread_pc corresponding to two libthread APIs 
// return false on failure
bool os::Solaris::set_thread_pc_fast(Thread* thread, ExtendedPC addr) {
#ifndef  SOLARIS_STRUCTURED_PROC     // Solaris 5.5.1 or older, no structured /proc 
  ShouldNotReachHere();
  return false;
#else
  // Solaris 5.6 or newer:  structured /proc interface
  assert(os::Solaris::mutator_libthread() && GetThreadState, "Must use new libthread API");

  char lwpctl_file[PROCFILE_LENGTH];
  int flag;
  int lwpctl_fd;
  lwpid_t lwpid;
  gregset_t rs;
  lwpstatus_t lwpstatus;
  int res;

  thread_t tid = thread->osthread()->thread_id();
  res = threadgetstate(tid, &flag, &lwpid, NULL, rs, &lwpstatus);
  assert(res == 0, "threadgetstate() failure");
  if(res) {
    return false;  // it is best to be careful here
  }
  
  assert((flag == TRS_VALID) || (flag == TRS_LWPID) || (flag == TRS_NONVOLATILE), "invalid thread state in set_thread_pc_fast");
  
  switch(flag) {
  case TRS_VALID:         //  really fast cases - register state cached by libthread
    {
      assert(rs[REG_SP] != NULL, "stack point shouldn't be null in TRS_VALID case");
      rs[REG_PC] =  (greg_t) addr.pc();
      rs[REG_nPC] = (greg_t) addr.npc();
      if( os::Solaris::thr_setstate(tid, flag, rs) == EINVAL ) {
	assert(false, "thr_setstate failed");
	return false;
      }
      return true;
    }
  break;

  case TRS_LWPID:        // thread stopped while on lwp 
    // Another solution is to try the slow interrupt/based case instead
    {
      assert(lwpstatus.pr_reg[R_SP] != 0, "stack point shouldn't be null in TRS_LWPID case");
      
      // Set pc/npc by sending a message to an lwp control file
      lwpstatus.pr_reg[R_PC]  = (greg_t) addr.pc();
      lwpstatus.pr_reg[R_nPC] = (greg_t) addr.npc(); 

      sprintf(lwpctl_file, "/proc/%d/lwp/%d/lwpctl", getpid(), lwpid);
      
      if ((lwpctl_fd = open(lwpctl_file, O_WRONLY)) < 0) {
	assert(false, "set_thread_pc_fast: open lwpctl");
	return false;
      }

      // setup the control message and write it to the lwp control file
      lwpstatus.pr_instr = PCSREG;   // set general registers
      if (write(lwpctl_fd, &lwpstatus.pr_instr, sizeof (long) + sizeof (prgregset_t)) !=
	  sizeof (long) + sizeof (prgregset_t)) {
	assert(false, "write to lwpctl failed");
	(void) close(lwpctl_fd);
	return false;
      }
      (void) close(lwpctl_fd);
      return true;
    }
  break;

  case TRS_NONVOLATILE:   
    {
      // This is the case when the thread was suspended by interrupting it with a SIGLWP
      // Change pc/npc in SIGLWP's handler ucontext to let it resume at the new address
      assert(rs[REG_SP] != NULL, "Invalid REG_SP in set_thread_pc_fast");
      ucontext_t *uc = get_ucontext_from_siglwp_handler((jint*)rs[REG_SP], thread);
      if(uc) {
	uc->uc_mcontext.gregs[REG_PC] =  (greg_t) addr.pc();
	uc->uc_mcontext.gregs[REG_nPC] = (greg_t) addr.npc();
	return true;
      }
      else {
	return false;
      }
    }
    break;

  case TRS_INVALID:
    ShouldNotReachHere();
    return false;
    break;

  default:
    ShouldNotReachHere();
    return false;
    break;
  }
#endif // SOLARIS_STRUCTURED_PROC
}

// Returns a pointer to ucontext from SIGLWP handler by walking up the stack
// NULL indicates failure
static ucontext_t*
get_ucontext_from_siglwp_handler(jint* s_sp, Thread* thread) {
  ucontext_t *uc = NULL;

  address s_pc       = os::Solaris::handler_end;  // outside the needed range
  jint*   younger_sp = NULL;
  jint*   orig_sp    = s_sp;
  address stackStart = (address)thread->stack_base();
  address stackEnd   = (address)(stackStart - (address)thread->stack_size());
  int     i          = 0;
  
  // walk back on stack until the signal handler is found
  while (1) {
    // Temporary workaround for libthread bug:
    // 4335248 thr_getstate returns bad sp for flag TRS_NONVOLATILE
    // If we start with a bogus (usually somewhat stale) SP we may soon
    // go astray; so use a few sanity checks to detect and dodge such
    // situations:
    if (s_sp == NULL || s_pc == NULL ||
        (UseThrGetStateWorkaround &&
         ((int)s_sp & 0x7 != 0 || (int)s_pc & 0x3 != 0 ||
          (address)s_sp > stackStart - wordSize || (address)s_sp < stackEnd)) ||
          // The real stackStart seems to be one word less that what is returned by thr_stksegment.
        ++i > 10) {
      return NULL;
    }

    if (s_pc >= os::Solaris::handler_start &&
        s_pc <  os::Solaris::handler_end) {
      frame sighandler_frame(s_sp, younger_sp);
      uc = *(ucontext_t **)sighandler_frame.param_addr(2, true);
      // Sometimes the register windows are not properly flushed.
      if(uc && uc->uc_mcontext.gwins != NULL)
        ::handle_unflushed_register_windows(uc->uc_mcontext.gwins);
      return uc;
    }
    younger_sp = s_sp;
    frame fr(s_sp, frame::unpatchable, s_pc);
    s_sp = fr.sender_sp();
    s_pc = fr.sender_pc();
  }
  ShouldNotReachHere();
}


// This is a "fast" implementation of fetch_top_frame that relies on the new libthread
// APIs to get pc for a suspended thread
ExtendedPC os::Solaris::fetch_top_frame_fast(Thread* thread, jint** ret_younger_sp, jint** ret_sp) {
#ifndef SOLARIS_STRUCTURED_PROC       // 5.5.1 or older
  ShouldNotReachHere();
  ExtendedPC noaddr;
  return noaddr;
#else
  jint* sp = NULL;
  int flag;
  lwpid_t lwpid;
  gregset_t rs;
  lwpstatus_t lwpstatus;
  int res;
  ExtendedPC addr;
  
  #ifdef ASSERT
  memset(&rs[0], 0xff, sizeof(gregset_t));
  assert(os::Solaris::mutator_libthread() && GetThreadState, "Must use new libthread API");
  #endif

  thread_t tid = thread->osthread()->thread_id();
  res = threadgetstate(tid, &flag, &lwpid, NULL, rs, &lwpstatus);
//  assert(res == 0, "threadgetstate() failure");
  // Let caller know about failure so profiler can use 0
  // and safepoint can roll-forward and retry
  // thread self-suspension uses wait on Interrupt_Lock, so
  // thread is not t_stop or t_stopallmutators
  if(res) {
    ExtendedPC new_addr(NULL, NULL);
    sp   = (jint*)0;
    addr = new_addr;
    *ret_younger_sp = NULL; 
    *ret_sp = sp;
    return addr;  // bail out
  }
  
  switch(flag) {
  case TRS_NONVOLATILE:   
    // Even worse, the register values that libthread returns are for the libthread's
    // own SIGLWP handler - a suspended thread sleeps on a synchronization object
    // inside this handler... 
    if (Arguments::has_profile() || UseStrictCompilerSafepoints || jvmpi::enabled()) {
      ucontext_t *uc = NULL;
     
      uc = get_ucontext_from_siglwp_handler((jint*)rs[REG_SP], thread);

      if (uc == NULL) {
        ExtendedPC new_addr(NULL, NULL);
        sp   = (jint*)0;
        addr = new_addr;
      }
      else {
	address npc;
	if(UseStrictCompilerSafepoints) {
	  npc = (address) uc->uc_mcontext.gregs[REG_nPC];
	}
	else {
	  // set npc to zero to avoid using it for safepoint (good for profiling only)
	  npc = (address) 0;
	}
        ExtendedPC new_addr((address)uc->uc_mcontext.gregs[REG_PC], npc);
        addr = new_addr;
	sp = (jint *)uc->uc_mcontext.gregs[REG_SP];
      }
      break;
    }
    else
      ;// fallthrough to invalid case
  case TRS_INVALID:
    {
      ExtendedPC new_addr(NULL, NULL);
      sp   = (jint*)0;
      addr = new_addr;
    }
  break;
  case TRS_VALID:         // the entire register set is cached for us by libthread
    {
      // TODO -- needs work - fastlane
      assert(rs[REG_SP]!= NULL, "stack point shouldn't be null in TRS_VALID case");
      ExtendedPC new_addr((address)rs[REG_PC], (address)rs[REG_nPC]);
      sp   = (jint*)rs[REG_SP];
      addr = new_addr;
    }
  break;
  case TRS_LWPID:        // got a full register set from the /proc interface
    // Fastlane?
    {
      assert(lwpstatus.pr_reg[R_SP] != 0, "stack point shouldn't be null in TRS_LWPID case");
      ExtendedPC new_addr((address)lwpstatus.pr_reg[R_PC], (address)lwpstatus.pr_reg[R_nPC]);
      sp   = (jint*)lwpstatus.pr_reg[R_SP ];
      addr = new_addr;
    }
  break;
  default:
    break;
  }
  
  *ret_younger_sp = NULL; 
  *ret_sp = sp;

  return addr;

#endif   // SOLARIS_STRUCTURED_PROC
}


// An alternative implementation that obtains a pc for a suspended thread by sending
// a signal to a thread and examining its context in the signal handler.
ExtendedPC os::Solaris::fetch_top_frame_slow(Thread* thread, jint** ret_younger_sp, jint** ret_sp) {
  // TODO -- needs work - fastlane
  // NOTE: For Solaris/Intel, we always get the information we need by forcing
  // the thread of interest to be the currently running process, and then
  // getting the context from it. Can we find this context in another way?
  if ((!UseCompilerSafepoints || GetThreadState) && !Arguments::has_profile()) {
    // just a kludge that won't crash, but sets the pc to NULL
    // the vm will restart the thread and stop it again later
    
    // When GetThreadState is set and we get here, it means that our intention probably
    // was to use the new libthread, but it was not found on LD_LIBRARY_PATH 
    *ret_younger_sp = *ret_sp = NULL;
    
    ExtendedPC addr(NULL, NULL);
    return addr;
  }
  
  // The requesting thread is either the VMThread or the WatcherThread
  // for flat profiling. Here is the dance that we do to fetch_top_frame()
  // using interrupt callbacks:
  // 
  // req thread: <already has the JavaThread suspended before we get here>
  // req thread: add the FetchTopFrame_Callback to the JavaThread
  // req thread: send a signal to the JavaThread and resume it
  // req thread: wait while the callback is not done
  // JavaThread: get the signal and call the FetchTopFrame_Callback
  //             (which stashes the info)
  // JavaThread: set the callback to done and notify the req thread
  // JavaThread: suspend_thread_self
  // req thread: wake up and install compiled code traps
  // req thread: <resumes the JavaThread when we return from here>
  // JavaThread: wakes up and hits the trap, vectoring to the correct
  //             safepoint code
  
  FetchTopFrame_Callback* cb = FetchTopFrame_Callback::create();
  
  OSThread *osthread = thread->osthread();
  
  osthread->add_interrupt_callback(cb);
  send_interrupt(thread);
  
  thread->resume(false, false);  // don't yield, internal resume request
  
  ExtendedPC addr;
  {
    // don't use safepoint check because we might be the WatcherThread.
    MutexLockerEx ml(Interrupt_lock, Mutex::_no_safepoint_check_flag);
    
    while(!cb->is_done()) {
      Interrupt_lock->wait(Mutex::_no_safepoint_check_flag);
    }
    
    // copy the needed info from the callback
    *ret_younger_sp = NULL;
    *ret_sp = cb->sp();
    addr = cb->addr();
    
    // and now cancel the callback
    assert(cb->is_synchronous(), "must be a synchronous callback");
    cb->cancel();
  }
  return addr;
}  

extern "C" void findpc(int x);
extern void set_error_reported();

extern "C" int JVM_handle_solaris_signal(int signo, siginfo_t* siginfo, void* ucontext, int abort_if_unrecognized);

int JVM_handle_solaris_signal(int sig, siginfo_t* info, void* ucVoid, int abort_if_unrecognized) {
  ucontext_t* uc = (ucontext_t*) ucVoid;

  if ((sig == SIGINT) || (sig == SIGHUP) || (sig == SIGTERM)) { 
    // Handled to cleanup ISM if necessary.  
    assert(UseISM && !UsePermISM, "These signals are caught only when we are using non-sticky ISM");
    os::exit(1);
  }

  JavaThread* thread = NULL;
  VMThread* vmthread = NULL;
  if (os::Solaris::signal_handlers_are_installed) {
    // Thread* t = Thread::current();
    Thread* t = ThreadLocalStorage::get_thread_slow();  // slow & steady
    if (t != NULL ){
      if(t->is_Java_thread()) {
	thread = (JavaThread*)t;
      }
      else if(t->is_VM_thread()){
	vmthread = (VMThread *)t;
      }
    }
  }

  if (sig == INTERRUPT_SIGNAL) {
    if(thread){
      OSThread::InterruptArguments args(thread, uc);
      thread->osthread()->do_interrupt_callbacks_at_interrupt(&args);
    } 
    else if(vmthread){
      OSThread::InterruptArguments args(vmthread, uc);
      vmthread->osthread()->do_vm_interrupt_callback_at_interrupt(&args);      
    }
    return true;
  }

  if (info == NULL || info->si_code <= 0 || info->si_code == SI_NOINFO) {
    // can't decode this kind of signal
    info = NULL;
  } else {
    assert(sig == info->si_signo, "bad siginfo");
  }

  // decide if this trap can be handled by a stub
  address stub = NULL;

  address pc          = NULL;
  address adjusted_pc = NULL;

  //%note os_trap_1
  if (info != NULL && thread != NULL) {
    // factor me: getPCfromContext
    pc = (address) uc->uc_mcontext.gregs[REG_PC];
    #ifndef CORE
    adjusted_pc = thread->safepoint_state()->compute_adjusted_pc(pc);
    #else
    adjusted_pc = pc;
    #endif

    // Handle ALL stack overflow variations here
    if (UseStackBanging && sig == SIGSEGV && thread) {
      bool is_yellow = thread->in_stack_yellow_zone((address)info->si_addr);
      bool is_red = thread->in_stack_red_zone((address)info->si_addr);
      if(is_yellow) {
        thread->disable_stack_yellow_zone();
        // Sometimes the register windows are not properly flushed.
        if(uc->uc_mcontext.gwins != NULL) {
          ::handle_unflushed_register_windows(uc->uc_mcontext.gwins);
        }
        // Yellow zone access violations can be recovered from.
        // If the thread isn't in java land, all we can do is return.
        if (thread->thread_state() != _thread_in_Java) {
          return true; // Try to finish
        } else {
          // Throw a stack overflow exception
          stub = StubRoutines::sparc::handler_for_stack_overflow_entry();
        }
      } else if(is_red) {
        // Any red zone access violation is fatal.
        tty->print_cr("An irrecoverable stack overflow has occurred.");
        thread->disable_stack_yellow_zone();
        thread->disable_stack_red_zone();
        // Sometimes the register windows are not properly flushed.
        if(uc->uc_mcontext.gwins != NULL) {
          ::handle_unflushed_register_windows(uc->uc_mcontext.gwins);
        }
      }
    }

    // if (thread->last_Java_sp() != NULL)
    if (thread->thread_state() != _thread_in_Java) {
      // a fault inside the VM, or native code.
      // QQQ need this on x86 side
      if (UseStackBanging && sig == SIGSEGV && info->si_code == SEGV_ACCERR
          && thread && thread->in_stack_yellow_zone((address)info->si_addr)) { 
        // For now, all we can do is remove the yellow pages and hope for the
        // best. In the future, we should use the vm exit macros to check for
        // missing yellow pages and reinsert them.
        thread->disable_stack_yellow_zone();
        return true;
      } else if (UseStackBanging && sig == SIGSEGV && info->si_code == SEGV_ACCERR
          && thread && thread->in_stack_red_zone((address)info->si_addr)) { 
        // This is always a fatal error. Do we want to do anything more here?
        tty->print_cr("An irrecoverable stack overflow has occurred.");
      }
    } else {
      // Java thread running in Java code => find exception handler if any
      // a fault inside compiled code, the interpreter, or a stub
#ifndef CORE
      if (sig == SIGILL && nativeInstruction_at(pc)->is_illegal()) {
	// debugging trap, or a safepoint
	COMPILER1_ONLY(stub = Runtime1::entry_for(Runtime1::illegal_instruction_handler_id);)
	COMPILER2_ONLY(stub = OptoRuntime::illegal_exception_handler_blob()->instructions_begin();)

	CompiledCodeSafepointHandler *handler = (CompiledCodeSafepointHandler *) thread->safepoint_state()->handle();
	if (handler != NULL && thread->is_in_compiled_safepoint()) {
	  nmethod* nm = handler->get_nmethod();
	  assert(nm != NULL, "safepoint handler is not setup correctly");

	  relocInfo::relocType type = nm->reloc_type_for_address(adjusted_pc);
	  assert( ((NativeInstruction*)adjusted_pc)->is_call() || 
		  (type == relocInfo::return_type) || (type == relocInfo::safepoint_type), 
		  "Only calls, returns, and backward branches are patched at safepoint");

	  if(type == relocInfo::return_type) {
	    // The retl case: restore has already happened. The safepoint blob frame will appear 
	    // immediately below caller of the orignal method that we have patched. The stack could 
	    // be walked properly (the frame of the patched method is already popped). 
	    // Will need to be revisited if and when we put restore in the delay slot of ret.
	  }
	  else {
	    // This is the case when either a call or a branch was patched with an illegal instruction.
	    // At this point we are done with the patched method.
	    // The appropriate handle_illegal_instruction_exception method
	    // will compute the real continuation point, and patch it
	    // into the frame.  No more action is required here.
	    #ifdef ASSERT
	    // The outgoing O7 will be used to link back to the real method.
	    // Make sure we do not depend on its value henceforth.
	    uc->uc_mcontext.gregs[REG_O7] = (greg_t) 0xbaadc0de;
	    #endif
	  }
	}
      }
#endif

      // Not needed on x86 solaris because verify_oops doesn't generate
      // SEGV/BUS like sparc does.
      if ( (sig == SIGSEGV || sig == SIGBUS)
	   && pc >= MacroAssembler::_verify_oop_implicit_branch[0]
	   && pc <  MacroAssembler::_verify_oop_implicit_branch[1] ) {
	stub     =  MacroAssembler::_verify_oop_implicit_branch[2];
	warning("fixed up memory fault in +VerifyOops at address 0x%x", (int)info->si_addr);
      }
      #ifndef CORE
      address implicit_null_handler;
      // This is not factored because on x86 solaris the patching for
      // zombies does not generate a SEGV.
      if (sig == SIGSEGV && nativeInstruction_at(pc)->is_zombie()) {
        // zombie method (ld [%g0],%o7 instruction)
        // "handle wrong method stub not implemented"
#ifdef COMPILER1
        nmethod* nm = CodeCache::find_nmethod(adjusted_pc);
        assert(nm != NULL, "no nmethod");
        // Note: we cannot ask method()->is_static() here because nmethod::_method is
        // set to NULL when nmethod is marked as unloaded.
        stub = nm->entry_point() == nm->verified_entry_point() ?
          Runtime1::entry_for(Runtime1::handle_wrong_static_method_id) :
          Runtime1::entry_for(Runtime1::handle_wrong_method_id);
#endif
	COMPILER2_ONLY(stub = OptoRuntime::handle_wrong_method_stub();)
        // At the stub it needs to look like a call from the caller of this
        // method (not a call from the segv site).
        pc = (address)uc->uc_mcontext.gregs[REG_O7];
      }   
      #endif

      else if (sig == SIGFPE && info->si_code == FPE_INTDIV) {
	// integer divide by zero
	stub = StubRoutines::sparc::handler_for_divide_by_zero_entry();
      }
      else if (sig == SIGFPE && info->si_code == FPE_FLTDIV) {
	// floating-point divide by zero
	stub = StubRoutines::sparc::handler_for_divide_by_zero_entry();
      }
#ifdef COMPILER2
      else if (sig == SIGSEGV && OptoRuntime::illegal_exception_handler_blob() != NULL &&
               OptoRuntime::illegal_exception_handler_blob()->contains(pc)) {
	ShouldNotReachHere();
      }
#endif

      else if (sig == SIGSEGV && info->si_code > 0 && !MacroAssembler::needs_explicit_null_check((int)info->si_addr)) {
	// an access to the first page of VM--assume it is a null pointer
	stub = StubRoutines::sparc::handler_for_null_exception_entry();
      }
    }
  }

  if (stub != NULL) {
    // save all thread context in case we need to restore it

    thread->set_saved_exception_pc(pc);

    // simulate a branch to the stub (a "call" in the safepoint stub case)
    // factor me: setPC
    uc->uc_mcontext.gregs[REG_PC ] = (greg_t)stub;
    uc->uc_mcontext.gregs[REG_nPC] = (greg_t)(stub + 4);

    return true;
  }

  // signal-chaining
  if (UseSignalChaining) {
    bool chained = false;
    struct sigaction *actp = os::Solaris::get_chained_signal_action(sig);
    if (actp != NULL) {
      chained = os::Solaris::chained_handler(actp, sig, info, ucVoid);
    }
    if (chained) {
      // signal-chaining in effect. Continue.
      return true;
    }
  }

  if(sig == SIGPIPE) {
    if (WizardMode || Verbose) {
      warning("Ignoring SIGPIPE - see bug 4229104");
    }
    return true;
  }

  if (!abort_if_unrecognized) {
    // caller wants another chance, so give it to him
    return false;
  }

  if (!os::Solaris::libjsig_is_loaded) {
    struct sigaction oldAct;
    sigaction(sig, (struct sigaction *)0, &oldAct);
    if (oldAct.sa_sigaction != signalHandler) {
      void* sighand = oldAct.sa_sigaction ? CAST_FROM_FN_PTR(void*, oldAct.sa_sigaction)
					  : CAST_FROM_FN_PTR(void*, oldAct.sa_handler);
      warning("Unexpected Signal %d occured under user-defined signal handler %#lx", sig, (long)sighand);
    }
  }

  if (pc == NULL && uc != NULL) {
    pc = (address) uc->uc_mcontext.gregs[REG_PC];
  }

  #ifndef PRODUCT
  // tell the system that it is about to die
  set_error_reported();

  // Remove this handler now.  It is of no more use.
  os::Solaris::set_signal_handler(sig, false);

  if (WizardMode) {
    if (pc != NULL) {
       printf("\n");
       printf("PC information:\n");
       findpc((int)pc);
    }
    if (info != NULL) {
       printf("\n");
       printf("siginfo_t dump:\n");
       for (int i = 0; i < sizeof(info[0])/sizeof(int); i++) {
	 printf(" 0x%08X", ((int*)info)[i]);
	 if ((i + 1) % 4 == 0)  printf("\n");
       }
       printf("\n");
    }
    if (uc != NULL) {
       printf("\n");
       printf("ucontext_t dump:\n");
       for (int i = 0; i < sizeof(uc[0])/sizeof(int); i++) {
	 printf(" 0x%08X", ((int*)uc)[i]);
	 if ((i + 1) % 4 == 0)  printf("\n");
       }
       printf("\n");

       tty->print_cr("Unexpected exception occured: signal=%d, pc=%lx", sig, (long)pc);

       uc->uc_mcontext.gregs[REG_PC ] = (greg_t)&continue_with_dump;
       uc->uc_mcontext.gregs[REG_nPC] = uc->uc_mcontext.gregs[REG_PC] + 4;
       return true;
    }
  }
  #endif

  os::handle_unexpected_exception(thread, sig, pc, (void *)info);

  // reaches here only when the user wants to debug the problem in
  // -XX:+ShowMessageBoxOnError mode. But we can't launch debugger
  // as Windows does, just os::abort.
  os::abort();
}

void os::Solaris::atomic_store_bootstrap(jint store_value, jint* dest) {
  // try to use the stub:
  store_func_t* func = CAST_TO_FN_PTR(store_func_t*, StubRoutines::atomic_store_entry());

  if (func != NULL) {
    os::Solaris::atomic_store_func = func;
    (*func)(store_value, dest);
    return;
  }
  assert(Threads::number_of_threads() == 0, "for bootstrap only");

  *dest = store_value;
}

void os::Solaris::init_thread_fpu_state(void) {
    // Nothing needed on Sparc.
}

// JVMPI code
bool os::thread_is_running(JavaThread* tp) {
  int         flag;
  lwpid_t     lwpid;
  gregset_t   reg;
  lwpstatus_t lwpstatus;
  int         res;
  thread_t    tid = tp->osthread()->thread_id();
  res = threadgetstate(tid, &flag, &lwpid, NULL, reg, &lwpstatus);
  assert(res == 0, "threadgetstate() failure");
  if(res != 0) return false; // Safe return value

  unsigned int sum = 0;
  sum += reg[R_SP];  sum += reg[R_PC];

  sum += reg[R_G1];  sum += reg[R_G2];  sum += reg[R_G3];  sum += reg[R_G4];
    
  sum += reg[R_O0];  sum += reg[R_O1];  sum += reg[R_O2];  sum += reg[R_O3];
  sum += reg[R_O4];  sum += reg[R_O5];

  sum += reg[R_I0];  sum += reg[R_I1];  sum += reg[R_I2];  sum += reg[R_I3];
  sum += reg[R_I4];  sum += reg[R_I5];  sum += reg[R_I6];  sum += reg[R_I7];

  sum += reg[R_L0];  sum += reg[R_L1];  sum += reg[R_L2];  sum += reg[R_L3];
  sum += reg[R_L4];  sum += reg[R_L5];  sum += reg[R_L6];  sum += reg[R_L7];

  if (tp->last_sum() == sum) {
    return false;
  } else {
    tp->set_last_sum(sum);
    return true;
  }
}

// Should really be in safepoint_solaris_sparc.cpp but this is all it would contain
// and all the other platforms would need a safepoint_<os>_<cpu>.cpp which seems like
// overkill.
// The patching of the return instruction is machine-specific
void SafepointSynchronize::patch_return_instruction_md(address cb_pc) {
  // In the case of SPARC the relocation info. is on the ret instruction
  // which is where we are going to patch
  assert(((NativeInstruction*)cb_pc)->is_return(), "must be a return instruction");
  assert(((NativeInstruction*)(cb_pc+NativeInstruction::nop_instruction_size))->is_nop(), "delay slot must be a nop instruction");
  NativeIllegalInstruction::insert(cb_pc);
}

void SafepointSynchronize::save_thread_state_at_safepoint(JavaThread *thread) {
  if (thread->osthread()->state() == SUSPENDED && thread->has_last_Java_frame()) {
    // cause pd state to be cached
    // the state saver recognizes that since we are suspended that our
    // register windows are flushed
    (void)thread->last_frame();
  }
}


// This is split from os_solaris.cpp because of platform issues but is related to os_win32.cpp
//Reconciliation History
// 1.287 99/04/01 18:31:08 os_win32.cpp
// 1.288 99/04/12 12:56:48 os_win32.cpp
// 1.293 99/07/06 16:36:49 os_win32.cpp
// 1.294 99/07/13 11:41:18 os_win32.cpp
// 1.296 99/08/05 16:45:47 os_win32.cpp
// 1.297 99/08/25 13:15:30 os_win32.cpp
//End
