/*
 * @(#)thread_solaris_sparc.hpp	1.43 00/04/28 04:42:53
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

public:

  enum {
    is_pc       = 1,
    has_flushed = 2
  };

private:

  void pd_initialize() {
    _post_Java_state              = NULL;
    _base_of_stack_pointer        = NULL;
  }

  inline jint* pd_post_Java_sp() {
    assert(!pd_post_Java_state_is_pc(), "must be a stack pointer");
    return (jint*)((unsigned long)_post_Java_state & ~(is_pc | has_flushed));
  }

  inline address pd_post_Java_pc() {
    assert(pd_post_Java_state_is_pc(), "must be a program counter");
    return (address)((unsigned long)_post_Java_state & ~(is_pc | has_flushed));
  }

  inline bool pd_post_Java_state_is_pc() {
    return ((unsigned long)_post_Java_state & is_pc) != 0;
  }

  inline bool pd_post_Java_state_has_flushed() {
    return ((unsigned long)_post_Java_state & has_flushed) != 0;
  }

  inline void pd_set_post_Java_state_has_flushed(bool flushed) {
    if (flushed)
      _post_Java_state = (jint*)((unsigned long)_post_Java_state |  has_flushed);
    else
      _post_Java_state = (jint*)((unsigned long)_post_Java_state & ~has_flushed);
  }

  inline void pd_set_post_Java_state_is_pc(bool is_pc_val) {
    if (is_pc_val)
      _post_Java_state = (jint*)((unsigned long)_post_Java_state |  is_pc);
    else
      _post_Java_state = (jint*)((unsigned long)_post_Java_state & ~is_pc);
  }

  // Because sparc threads need to scan for a younger sp
  // It is hard to get the last_frame of another thread,
  // so when blocking, cause state to be saved, since it
  // is relatively easy from inside the blocking thread.

  jint* pd_cached_state() const          { return _post_Java_state; }
  void  pd_set_cached_state(jint* state) { _post_Java_state = state; }

  #ifdef ASSERT
  bool pd_has_cached_state() {
    if (pd_post_Java_state_is_pc()) {
      assert(pd_post_Java_pc() != NULL || !pd_post_Java_state_has_flushed(), "");
    }
    else {
      assert(pd_post_Java_sp() != NULL || !pd_post_Java_state_has_flushed(), "");
    }

    return _post_Java_state != NULL && pd_post_Java_state_has_flushed();
  }
  #endif

  // pd_cache_state -- this is called as part of the process of making
  // a thread ready to take go to a safepoint.  This is usually called
  // when this == JavaThread::current() (from save_state_before_block
  // [i.e. StateSaver]) but can also in a certain circumstance (see
  // the state() == SUSPENDED case) be called from the vm thread.

  // Note -- We can completely skip flushing register windows provided
  // we: 1. always suspend all java threads at a safepoint (a la
  // thr_suspendallmutators) and 2. this kind of suspension is
  // guaranteed to flush the thread's register windows and 3. we can
  // get the thread's current sp from another thread (again a la the
  // new libthread).  If these are true, then all of this work (in
  // pd_cache_state) can be done from the vm thread at safepoint time
  // and no flushing should be necessary.

  void pd_cache_state(bool ris_definitely_current_thread) { 
    if (!has_last_Java_frame()) {
      // we have no state to cache -- return
      return;

    } else if (!ris_definitely_current_thread && osthread()->state() == SUSPENDED) {
      if (_post_Java_state == NULL) {
        jint* sp;
        jint* younger_sp;
        ExtendedPC addr  = os::fetch_top_frame(this, &younger_sp, &sp);
        if (sp == NULL) {
          return;
        }
        _post_Java_state = frame::next_younger_sp(last_Java_sp(), sp);
      }
      // A side effect (undocumented) of suspending a Solaris thread
      // is that all of the register windows get flushed.  Therefore,
      // if another thread (i.e. the vm thread) has suspended a
      // thread, it can call pd_cache_state to ensure that it is noted
      // that the windows are flushed.  The alternative would be to
      // send a signal to the thread and have it flush its own
      // windows.  Suspension seems to be faster.
      pd_set_post_Java_state_has_flushed(true);

    } else if (_post_Java_state == NULL) {
      // The state has not been cached -- cache it -- i.e. find the
      // frame one younger than the last Java sp.  This frame's
      // register save area holds interesting things like the last
      // java frame's 'o' registers.

      assert(Thread::current() == this, "only current thread can flush its own register windows");
      _post_Java_state = frame::next_younger_sp(last_Java_sp());

      // a side effect of calling next_younger_sp is that the register
      // windows get flushed -- note this.

      pd_set_post_Java_state_has_flushed(true);

    } else if (!pd_post_Java_state_has_flushed()) {
      // Sometimes we note the post java state but don't flush the
      // windows because it would be too expensive to do so all of the
      // time.  Flush them now.

      assert(Thread::current() == this, "only current thread can flush its own register windows");
      (void)StubRoutines::sparc::flush_callers_register_windows_func()();
      pd_set_post_Java_state_has_flushed(true);
    }

    assert(pd_has_cached_state(), "sanity check");
  }


  frame pd_last_frame() {
    assert(has_last_Java_frame(), "must have last_Java_sp() when suspended");
    assert(pd_post_Java_state_has_flushed(), "thread has not dumped its register windows yet");

    if (pd_post_Java_state_is_pc())
      return frame(last_Java_sp(), frame::unpatchable, pd_post_Java_pc());
    else
      return frame(last_Java_sp(), pd_post_Java_sp());
  }

  // Some runtime calls need the caller pc. This gives them a faster alternative
  // to flushing the register window and reading the stack
  address _stub_caller_pc;

  // cache of the younger_sp derived from last_Java_sp
  // also, if it is non-null, we know we have flushed register windows
  jint* _post_Java_state;

  // a stack pointer older than any java frame stack pointer.  It is
  // used to validate stack pointers in frame::next_younger_sp (it
  // provides the upper bound in the range check).  This is necessary
  // on Solaris/SPARC since the ucontext passed to a signal handler is
  // sometimes corrupt and we need a way to check the extracted sp.
  jint* _base_of_stack_pointer;

public:

  jint* last_Java_fp()                       { return NULL; }
  void  set_last_Java_fp(jint* fp)           { assert(fp == NULL, ""); }

  static int post_Java_state_offset_in_bytes()                                             { return (int)&((JavaThread*)NULL)->_post_Java_state; }

  static int stub_caller_pc_offset_in_bytes()                                              { return (int)&((JavaThread*)NULL)->_stub_caller_pc; }

  address stub_caller_pc() { return _stub_caller_pc; }

  // It can be very expensive to create a last_frame just so we
  // can extract the PC.  (SPARC requires a register window flush.)
  // Instead, it is better to have the stub save its own PC into
  // TLS, as the "stub_caller_pc".
  address return_pc_from_stub() { 
    return stub_caller_pc() + frame::pc_return_offset * wordSize;
  }

public:

  jint* base_of_stack_pointer() { return _base_of_stack_pointer; }

  void set_base_of_stack_pointer(jint* base_sp) { 
    _base_of_stack_pointer = base_sp; 
  }

  void record_base_of_stack_pointer() { 
    jint *sp = StubRoutines::sparc::flush_callers_register_windows_func()();  
    jint *ysp;
    while((ysp = (jint*)sp[FP.sp_offset_in_saved_window()]) != NULL) {
      sp = ysp;
    }
    _base_of_stack_pointer = sp;
  }

  static frame current_frame_guess() { 
    jint* sp = StubRoutines::sparc::flush_callers_register_windows_func()(); 
    frame fr(sp, frame::unpatchable, NULL); 
    return fr; 
  }

# ifdef ASSERT
  virtual bool state_is_saved_before_block() {
    return  (jvmdi::enabled()) || (thread_state() == _thread_new_trans) || (thread_state() == _thread_new) || (pd_has_cached_state() && pd_post_Java_state_has_flushed() && Thread::state_is_saved_before_block());
  }
# endif

  int pd_get_fast_thread_id() {
    return (int) ThreadLocalStorage::thread();
  }

//Reconciliation History
// 1.2 99/08/25 13:31:32 thread_win32_i486.hpp
// 1.3 99/08/26 10:31:50 thread_win32_i486.hpp
//End
