/*
 * @(#)threadLS_win32_i486.cpp	1.7 99/08/03 15:02:16
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Provides an entry point we can link against and
// a buffer we can emit code into. The buffer is
// filled by ThreadLocalStorage::generate_code_for_get_thread
// and called from ThreadLocalStorage::thread()

#include "incls/_precompiled.incl"
#include "incls/_threadLS_win32_i486.cpp.incl"

int ThreadLocalStorageOffset = 0;

// This code is win32 specific
int ThreadLocalStorage::array_offset() {
  // see definition of struct _NT_TIB in WINNT.H
  return 0x18;
}

int ThreadLocalStorage::base_offset() {
  return os::win32::is_nt()
       ? 0xe10  // found by looking at TLS code on NT
       :  0x88; // found by looking at TLS code on Windows95
}

// This code is win32/x86 specific
void ThreadLocalStorage::generate_code_for_get_thread() {
  ::ThreadLocalStorageOffset = base_offset() + thread_offset();

  // We have to generate code for get_thread since the
  // tls index is resolved at load time.
  // The reason we cannot use the nice assembler class is
  // it uses resource object which in turn uses get_thread
  // for allocation.
  address buffer = (address) &get_thread;

  if (os::win32::is_nt()) {
    // mov eax, dword ptr fs:[ThreadLocalStorageOffset]
    // 64 A1 ThreadLocalStorageOffset
    *buffer++ = 0x64;
    *buffer++ = 0xA1;

    *((int*) buffer) = ThreadLocalStorageOffset;
    buffer += sizeof(int);
  } else {
    // mov eax, dword ptr fs:[18H];
    // 64 A1 18 00 00 00
    *buffer++ = 0x64;
    *buffer++ = 0xA1;
    *buffer++ = 0x18;
    *buffer++ = 0x00;
    *buffer++ = 0x00;
    *buffer++ = 0x00;

    // mov eax, dword ptr [eax+ThreadLocalStorageOffset];
    // 8B 80 ThreadLocalStorageOffset
    *buffer++ = 0x8B;
    *buffer++ = 0x80;
    *((int*) buffer) = ThreadLocalStorageOffset;
    buffer += sizeof(int);
  }

  // ret
  // C3
  *buffer++ = 0xC3;

  const int get_thread_size = 16; // constant copied from getThread_i486.cpp
  guarantee((buffer - (address) &get_thread) <= get_thread_size, "check boundary");
}
