/*
 * @(#)main.cpp	1.80 01/02/09 15:31:09
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// MAIN.CPP - Entry point for the Architecture Description Language Compiler
#include "adlc.hpp"

//------------------------------Prototypes-------------------------------------
static void  usage(ArchDesc& AD);          // Print usage message and exit
static int   open_files(ArchDesc& AD);     // Open Input/Output files
static void  close_files(int delete_out, ArchDesc& AD); // Close I/O files
static char *strip_ext(char *fname);       // Strip off name extension
static char *base_plus_suffix(const char* base, const char *suffix);// New concatenated string 

//------------------------------main-------------------------------------------
int main(int argc, char *argv[])
{
  // ResourceMark  mark;
  ADLParser    *ADL_Parse;      // ADL Parser object to parse AD file
  ArchDesc      AD;             // Architecture Description object

  // Check for proper arguments
  if( argc == 1 ) usage(AD);    // No arguments?  Then print usage

  // Read command line arguments and file names
  for( int i = 1; i < argc; i++ ) { // For all arguments
    register char *s = argv[i];	// Get option/filename

    if( *s++ == '-' ) {         // It's a flag? (not a filename)
      if( !*s ) {		// Stand-alone `-' means stdin
	//********** INSERT CODE HERE **********	
      } else while (*s != '\0') { // While have flags on option
	switch (*s++) { 	// Handle flag
	case 'd':               // Debug flag
	  AD._dfa_debug += 1;   // Set Debug Flag
	  break;
	case 'g':               // Debug ad location flag
	  AD._adlocation_debug += 1;       // Set Debug ad location Flag
	  break;
	case 'o':               // No Output Flag
	  AD._no_output ^= 1;   // Toggle no_output flag
	  break;
	case 'q':               // Quiet Mode Flag
	  AD._quiet_mode ^= 1;  // Toggle quiet_mode flag
	  break;
	case 'w':               // Disable Warnings Flag
	  AD._disable_warnings ^= 1; // Toggle disable_warnings flag
	  break;
	case 'T':               // Option to make DFA as many subroutine calls.
	  AD._dfa_small += 1;   // Set Mode Flag
	  break;
	case 'c':               // Set C++ Output file name
	  AD._CPP_file._name = s; s += strlen(s);
	  break;
	case 'h':               // Set C++ Output file name
	  AD._HPP_file._name = s; s += strlen(s);
	  break;
	case 'v':               // Set C++ Output file name
	  AD._VM_file._name = s; s += strlen(s);
	  break;
	case 'a':               // Set C++ Output file name
	  AD._DFA_file._name = s;
	  AD._bug_file._name = s;
	  s += strlen(s);
	  break;
	case '#':               // Special internal debug flag
	  AD._adl_debug++;      // Increment internal debug level
	  break;
	case 's':               // Output which instructions are cisc-spillable
	  AD._cisc_spill_debug = true;
	  break;
	default:		// Unknown option
	  usage(AD);            // So print usage and exit
	}			// End of switch on options...
      } 			// End of while have options...

    } else {			// Not an option; must be a filename
      AD._ADL_file._name = argv[i]; // Set the input filename

      // // Files for storage, based on input file name
      const char *base = strip_ext(strdup(argv[i]));
      char       *temp = base_plus_suffix("dfa_",base);
      AD._DFA_file._name = base_plus_suffix(temp,".cpp");
      delete temp;
      temp = base_plus_suffix("ad_",base);
      AD._CPP_file._name = base_plus_suffix(temp,".cpp");
      AD._HPP_file._name = base_plus_suffix(temp,".hpp");
      delete temp;
      temp = base_plus_suffix("adGlobals_",base);
      AD._VM_file._name = base_plus_suffix(temp,".hpp");
      delete temp;      
      temp = base_plus_suffix("bugs_",base);
      AD._bug_file._name = base_plus_suffix(temp,".out");
      delete temp;
    }				// End of files vs options...
  }				// End of while have command line arguments

  // Open files used to store the matcher and its components
  if (open_files(AD) == 0) return 1; // Open all input/output files

  // Build the File Buffer, Parse the input, & Generate Code
  FileBuff  ADL_Buf(&AD._ADL_file, AD); // Create a file buffer for input file

  ADL_Parse = new ADLParser(ADL_Buf, AD); // Create a parser to parse the buffer
  ADL_Parse->parse();           // Parse buffer & build description lists

  if( AD._dfa_debug >= 1 ) {    // For higher debug settings, print dump
    AD.dump();
  }

  delete ADL_Parse;             // Delete parser

  // Verify that the results of the parse are consistent
  AD.verify();

  // Prepare to generate the result files: 
  AD.generateMatchLists();
  AD.identify_cisc_spill_instructions();
  AD.identify_short_branches();
  // Generate the result files: 
  // enumerations, class definitions, object generators, and the DFA
  // file containing enumeration of machine operands & instructions (hpp)
  AD.addSunCopyright(AD._HPP_file._fp);           // .hpp
  AD.buildMachOperEnum(AD._HPP_file._fp);         // .hpp
  AD.buildMachOpcodesEnum(AD._HPP_file._fp);      // .hpp
  AD.buildMachRegisterNumbers(AD._VM_file._fp);   // VM file
  AD.buildMachRegisterEncodes(AD._HPP_file._fp);  // .hpp file
  AD.declareRegSizes(AD._HPP_file._fp);           // .hpp
  AD.build_pipeline_enums(AD._HPP_file._fp);      // .hpp
  // output definition of class "State"
  AD.defineStateClass(AD._HPP_file._fp);          // .hpp
  // file declaring the Mach* classes derived from MachOper and MachNode
  AD.declareClasses(AD._HPP_file._fp);
  // files declaring and defining generators for Mach* Objects (hpp,cpp)
  AD.addSunCopyright(AD._CPP_file._fp);           // .cpp
  AD.machineDependentIncludes(AD._CPP_file);      // .cpp
  // declare and define maps: in the .hpp and .cpp files respectively
  AD.buildReduceMaps(AD._HPP_file._fp, AD._CPP_file._fp);
  AD.build_rematerialize_map(AD._HPP_file._fp, AD._CPP_file._fp);
  AD.buildMustCloneMap(AD._HPP_file._fp, AD._CPP_file._fp);
  // build CISC_spilling oracle and MachNode::cisc_spill() methods
  AD.build_cisc_spill_instructions(AD._HPP_file._fp, AD._CPP_file._fp);
  // define methods for machine dependent State, MachOper, and MachNode classes
  AD.defineClasses(AD._CPP_file._fp);
  AD.buildMachOperGenerator(AD._CPP_file._fp);    // .cpp
  AD.buildMachNodeGenerator(AD._CPP_file._fp);    // .cpp
  // define methods for machine dependent instruction matching
  AD.buildInstructMatchChecks(AD._CPP_file._fp);  // .cpp
  // define methods for machine dependent frame management
  AD.buildFrameMethods(AD._CPP_file._fp);         // .cpp

  // define the finite automata that selects lowest cost production
  AD.addSunCopyright(AD._DFA_file._fp);           // .cpp
  AD.machineDependentIncludes(AD._DFA_file);      // .cpp
  AD.buildDFA(AD._DFA_file._fp);

  close_files(0, AD);               // Close all input/output files

  // Final printout and statistics
  // cout << program;

  if( AD._dfa_debug & 2 ) {    // For higher debug settings, print timing info
    //    Timer t_stop;
    //    Timer t_total = t_stop - t_start; // Total running time
    //    cerr << "\n---Architecture Description Totals---\n";
    //    cerr << ", Total lines: " << TotalLines;
    //    float l = TotalLines;
    //    cerr << "\nTotal Compilation Time: " << t_total << "\n";
    //    float ft = (float)t_total;
    //    if( ft > 0.0 ) fprintf(stderr,"Lines/sec: %#5.2f\n", l/ft);
  }
  return (AD._syntax_errs + AD._semantic_errs + AD._internal_errs); // Bye Bye!!
}

//------------------------------usage------------------------------------------
static void usage(ArchDesc& AD)
{
  printf("Architecture Description Language Compiler\n\n");
  printf("Usage: adl [-doqw] [-cFILENAME] [-hFILENAME] [-aDFAFILE] ADLFILE\n");
  printf(" d  produce DFA debugging info\n");
  printf(" o  no output produced, syntax and semantic checking only\n");
  printf(" q  quiet mode, supresses all non-essential messages\n"); 
  printf(" w  suppress warning messages\n");
  printf(" c  specify CPP file name (default: %s)\n", AD._CPP_file._name);
  printf(" h  specify HPP file name (default: %s)\n", AD._HPP_file._name);
  printf(" a  specify DFA output file name\n");
  printf("\n");
}

//------------------------------open_files-------------------------------------
static int open_files(ArchDesc& AD)
{
  if (AD._ADL_file._name == NULL)
  { printf("ERROR: No ADL input file specified\n"); return 0; }
  if ((AD._ADL_file._fp = fopen(AD._ADL_file._name, "r")) == NULL) {
    printf("ERROR: Cannot open ADL input file: %s\n", AD._ADL_file._name);
    close_files(1, AD);
    return 0;
  }
  if (!AD._no_output &&
      (AD._DFA_file._fp = fopen(AD._DFA_file._name, "w")) == NULL) {
    printf("ERROR: Cannot open DFA output file: %s\n", AD._DFA_file._name);
    close_files(1, AD);
    return 0;
  }
  if (!AD._no_output &&
      (AD._HPP_file._fp = fopen(AD._HPP_file._name, "w")) == NULL) {
    printf("ERROR: Cannot open HPP output file: %s\n", AD._HPP_file._name);
    close_files(1, AD);
    return 0;
  }
  if (!AD._no_output &&
      (AD._CPP_file._fp = fopen(AD._CPP_file._name, "w")) == NULL) {
    printf("ERROR: Cannot open CPP output file: %s\n", AD._CPP_file._name);
    close_files(1, AD);
    return 0;
  }
  if (!AD._no_output &&
      (AD._VM_file._fp = fopen(AD._VM_file._name, "w")) == NULL) {
    printf("ERROR: Cannot open VM globals output file: %s\n", AD._VM_file._name);
    close_files(1, AD);
    return 0;
  }
  if (AD._dfa_debug &&
      (AD._bug_file._fp = fopen(AD._bug_file._name, "w")) == NULL) {
    printf("ERROR: Cannot open DFA output file: %s\n", AD._bug_file._name);
    close_files(1, AD);
    return 0;
  }
   
  return 1;
}

//------------------------------close_files------------------------------------
static void close_files(int delete_out, ArchDesc& AD)
{
  if (AD._ADL_file._fp) fclose(AD._ADL_file._fp);
  if (AD._CPP_file._fp) {
    fclose(AD._CPP_file._fp);
    if (delete_out) remove(AD._CPP_file._name);
  }
  if (AD._HPP_file._fp) {
    fclose(AD._HPP_file._fp);
    if (delete_out) remove(AD._HPP_file._name);
  }
  if (AD._DFA_file._fp) {
    fclose(AD._DFA_file._fp);
    if (delete_out) remove(AD._DFA_file._name);
  }
  if (AD._bug_file._fp) {
    fclose(AD._bug_file._fp);
    if (delete_out) remove(AD._bug_file._name);
  }

  if (!AD._quiet_mode) {
    printf("\n");
    if (AD._no_output || delete_out) {
      if (AD._ADL_file._name) printf("%s: ", AD._ADL_file._name);
      printf("No output produced");
    }
    else {
      if (AD._ADL_file._name) printf("%s --> ", AD._ADL_file._name);
      printf("%s, %s, %s", AD._CPP_file._name, AD._HPP_file._name, AD._DFA_file._name);
    }
    printf("\n");
  }
}

//------------------------------strip_ext--------------------------------------
static char *strip_ext(char *fname)
{
  char *ep;
  char *sp;
   
  if (fname) {
    for (sp = fname; *sp; sp++)
      if (*sp == '/')  fname = sp+1;
    ep = fname;                    // start at first character and look for '.'
    while (ep <= (fname + strlen(fname) - 1) && *ep != '.') ep++;
    if (*ep == '.')	*ep = '\0'; // truncate string at '.' 
  }
  return fname;
}

//------------------------------base_plus_suffix-------------------------------
// New concatenated string 
static char *base_plus_suffix(const char* base, const char *suffix)
{
  int len = strlen(base) + strlen(suffix) + 1;

  char* fname = new char[len];
  sprintf(fname,"%s%s",base,suffix);
  return fname;
}


void *operator new( size_t size, int, const char *, int ) {
  return ::operator new( size );
}
