/*
 * @(#)assembler.cpp	1.18 00/03/21 16:16:27
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_assembler.cpp.incl"


// Implementation of Register

const char* Register::name() const {
  return isValid() ? registerNames[_number] : "noreg";
}


// Implementation of AbstractAssembler
//
// The AbstractAssembler is generating code into a CodeBuffer. To make code generation faster,
// the assembler keeps a copy of the code buffers boundaries & modifies them when
// emitting bytes rather than using the code buffers accessor functions all the time.
// The code buffer is updated via set_code_end(...) after emiting a whole instruction.

AbstractAssembler::AbstractAssembler(CodeBuffer* code) {
  _code	       = code;
  _code_begin  = code->code_begin();
  _code_limit  = code->code_limit();
  _code_pos    = code->code_end();
  _inst_mark   = NULL;
  _oop_recorder= code->oop_recorder();
}


void AbstractAssembler::flush() {
  ICache::invalidate_range(addr_at(0), offset());
}


void AbstractAssembler::a_byte(int x) {
  emit_byte(x);
}


void AbstractAssembler::a_long(jint x) {
  emit_long(x);
}




// Labels refer to positions in the (to be) generated code.
// There are bound, unbound and undefined labels.
//
// Bound labels refer to known positions in the already
// generated code. pos() is the position the label refers to.
//
// Unbound labels refer to unknown positions in the code
// to be generated; pos() is the position of the 32bit
// Displacement of the last instruction using the label.
//
// Undefined labels are labels that haven't been used yet.
// They refer to no position at all.


#ifndef PRODUCT
void AbstractAssembler::print(Label& L) {
  if (L.is_unused()) {
    tty->print_cr("unused label");
  } else if (L.is_bound()) {
    tty->print_cr("bound label to %d", L.pos());
  } else if (L.is_unbound()) {
    Label l = L;
    tty->print_cr("unbound label");
    while (l.is_unbound()) {
      Displacement disp = disp_at(l);
      tty->print("@ %d ", l.pos());
      disp.print();
      tty->cr();
      disp.next(l);
    }
  } else {
    tty->print_cr("label in inconsistent state (pos = %d)", L._pos);
  }
}
#endif // PRODUCT


void AbstractAssembler::bind_to(Label& L, int pos) {
  assert(0 <= pos && pos <= offset(), "must have a valid binding position");
  while (L.is_unbound()) {
    disp_at(L).bind(L, pos, this);
  }
  L.bind_to(pos);
}


void AbstractAssembler::bind(Label& L) {
  assert(!L.is_bound(), "label can only be bound once");
  bind_to(L, offset());
}



