// @(#)c1_CacheLocals.hpp	1.17 00/04/25 14:39:24
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2



class IR;
class Loop;
class RegAlloc;
class ALocal;
class CachedLocals;
class RInfoCollection;
class BlockItem;
class ScanResult;


// This class is used to map locals to register (cache locals into registers)
// There are 2 situtations where this can be done:
//    - cache receiver: on INTEL, receiver is always passed in a certin register (ecx)
//                      if no instruction can destroy that register, we want to cache
//                      the receiver throughout the whole method.
//    - cache loop locals: using a simple heuristic, locals inside a leaf loop (no calls..)
//                         will be cached in the free registers

class LocalCaching : public StackObj {
 private:
  IR* _ir;

  void assign_registers(CachedLocals* cached_locals, const ScanResult& scan,
			GrowableArray<ALocal*>* ordered, RInfoCollection* free_regs, Loop* loop);
  RInfoCollection* collect_free_cpu_registers(Loop* loop);
  void add_block_items(const RInfoCollection* mapping, GrowableArray<bool>* oops, c1_RegMask lockout, bool set_32bit_precision);
  void add_block_items(Loop* loop, const RInfoCollection* mapping, GrowableArray<bool>* oops, bool set_32bit_precision);
  void selective_caching(BlockItem* item);
  bool can_cache_receiver(ScanResult* scan_result);
  RInfo get_incoming_at_index(RInfoCollection* regs, RInfo in_reg);
  RInfo get_non_incoming_at_index(RInfoCollection* regs);

 public:
  LocalCaching(IR* ir): _ir(ir) {}
  CachedLocals* cache_locals_globally();
  void cache_loop_locals();
  // use cached receiver to eliminate loads; reg_alloc specifies registers used/free
  void cache_receiver(RegAlloc* reg_alloc);
};


class CachedLocals: public ResourceObj {
  friend LocalCaching;
 private:
  const RInfoCollection* _mapping; // maps local index to a rinfo
  GrowableArray<bool>*   _is_oop;
  void set_mapping(const RInfoCollection* locals)  { _mapping = locals; }
  void set_oops_info(GrowableArray<bool>* is_oop)  { _is_oop = is_oop; }

 public:
  CachedLocals() : _mapping(NULL), _is_oop(NULL) {}
  bool  is_oop   (int local_index) const {
    if (_is_oop != NULL && local_index < _is_oop->length()) {
      return _is_oop->at(local_index);
    } else {
      return false;
    }
  }
  RInfo get_cache_reg (int local_index) const {
    if (_mapping != NULL && local_index < _mapping->number_of_registers()) {
      return _mapping->register_at(local_index);
    } else {
      // illegal reg
      return norinfo;
    }
  }
  bool is_local_cached  (int local_index) const { return get_cache_reg(local_index).is_valid(); }
  int max_oop_local() const { return _is_oop->length() - 1 ; }
};
