/*
 * @(#)c1_CodeEmitter.cpp	1.18 00/04/27 08:46:30
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_CodeEmitter.cpp.incl"


#define __ _masm->


//------------------ConstantTable----------------------------------


ConstantTable::ConstantTable() {
  _entries = new GrowableArray<FPUConstEntry*>();
}


void ConstantTable::append_float(jfloat f) {
  _entries->append(new FPUConstEntry(f));
}


void ConstantTable::append_double(jdouble d) {
  _entries->append(new FPUConstEntry(d));
}

// Note: use JCK test to verify if this can be simplified to use only test
//       of values produced by jint_cast and jlong_cast.

address ConstantTable::address_of_float_constant(jfloat f) {
  for (int i = 0; i < _entries->length(); i++) {
    FPUConstEntry* fp = _entries->at(i);;
    // searching for floats
    if (fp->is_float()) {
      if ( g_isnan(fp->float_value()) || g_isnan(f) ) {
        if ( g_isnan(fp->float_value()) && g_isnan(f) ) {
          assert(fp->pc() != NULL, "entry not emitted");
          return fp->pc();
        }
      } else {
        if (jint_cast(fp->float_value())==jint_cast(f))   { // bitwise check
          assert(fp->pc() != NULL, "entry not emitted");
          return fp->pc(); 
        }
      }
    }
  }
  return NULL;
}


address ConstantTable::address_of_double_constant(jdouble d) {
  for (int i = 0; i < _entries->length(); i++) {
    FPUConstEntry* fp = _entries->at(i);;
    // searching for doubles
    if (fp->is_double()) {
      if ( g_isnan(fp->double_value()) || g_isnan(d) ) {
        if ( g_isnan(fp->double_value()) && g_isnan(d) ) {
          assert(fp->pc() != NULL, "entry not emitted");
          return fp->pc();
        }
      } else {
        if (jlong_cast(fp->double_value())==jlong_cast(d))  {  // bitwise check
          assert(fp->pc() != NULL, "entry not emitted");
          return fp->pc(); 
        }
      }
    }
  }
  return NULL;
}


#ifndef PRODUCT

void ConstantTable::print() {
  tty->print_cr("Constant table (%d entries)", _entries->length());
  for (int i = 0; i < _entries->length(); i++) {
    FPUConstEntry* fp = _entries->at(i);;
    if (fp->is_double()) {
      tty->print_cr("  double: %f ", *((jdouble*)fp->pc()));
    } else {
      tty->print_cr("  float : %f ", *((jfloat*)fp->pc()));
    }
  }
}

#endif

//-----------------------------------------------------

// Sets debug information for patching: the patch instruction has the form of a call.
void CodeEmitter::set_patching_desc(PatchingDesc* patch) {
  patch->set((int) __ pc(), __ offset());
}


// The recognition of float and double constants 0.0 and 1.0
// must be completed and properly implemented; currently
// only float 0.0 is recognized

// watch out for negative zero, nan and doubles

bool CodeEmitter::is_zero_float(jfloat f) const {
  jfloat ok = 0.0f;
  return jint_cast(f) == jint_cast(ok);
}


bool CodeEmitter::is_one_float(jfloat f) const {
  return !g_isnan(f) && g_isfinite(f) && f == 1.0;
}


bool CodeEmitter::is_zero_double(jdouble d) const {
  jdouble ok = 0.0;
  return jlong_cast(d) == jlong_cast(ok);
}


bool CodeEmitter::is_one_double(jdouble d) const {
  return !g_isnan(d) && g_isfinite(d) && d == 1.0;
}

bool CodeEmitter::needs_icache() const {
  return !method()->is_static();
}


bool CodeEmitter::needs_null_check_on_entry() const {
  // a method that is in a final class is classified as final method
  // even if static;  we need null receiver checks only for virtual methods
  return !NullCheckAtCaller && !method()->is_static() && method()->is_final_method();
}


// The local_index and local_index+1 addresses must be adjacent on stack
// (no link or return address may be in between)
void CodeEmitter::check_double_address(int local_index) {
  if (!frame()->are_adjacent_indeces(local_index, local_index+1)) {
    set_bailout("double elements not adjacent" );
  }
}



void CodeEmitter::add_call_info(int pc_offset, CodeEmitInfo* cinfo) {
  add_debug_info(pc_offset, cinfo, true);
}


void CodeEmitter::add_debug_info(int pc_offset, CodeEmitInfo* cinfo, bool at_call) {
  if (GenerateOopMaps) {
    OopMap* map = NULL;
    if (PassParametersInRegisters) {
      if (cinfo->_bci == SynchronizationEntryBCI) {
        // use this also for native methods at synchronization point
        map = compilation()->create_oop_map_for_own_signature(cinfo->_spilled_oops, esp_offset());
      } else if (method()->is_native()) {
        // use this after the oops have been stored on stack
        map = compilation()->create_oop_map_inside_natives(cinfo->_spilled_oops, esp_offset());
      } else {
        map = compilation()->create_oop_map(cinfo->_bci, cinfo->_spilled_oops, esp_offset());
      }
    } else {
      map = (cinfo->_bci == SynchronizationEntryBCI || method()->is_native()) ?
        compilation()->create_oop_map_for_own_signature(cinfo->_spilled_oops, esp_offset()) :
        compilation()->create_oop_map(cinfo->_bci, cinfo->_spilled_oops, esp_offset());
    }
    if (cinfo->has_register_oops()) {
      assert(!method()->is_native(), "internal error");
      compilation()->add_registers_to_oop_map(map, cinfo->register_oops(), esp_offset());
    }
    JVMCI_DebugInfo* elem = new JVMCI_DebugInfo(method(), pc_offset, cinfo->_bci, map, at_call);
    if (need_debug_info()) {
      Unimplemented();
      /*
      GrowableArray<ScopeValue*>*   locarray = method()->create_locarray(cinfo->_bci, esp_offset());
      GrowableArray<ScopeValue*>*   exparray = method()->create_exparray(cinfo->_stack, esp_offset()); // put in ValueStack
      GrowableArray<MonitorValue*>* monarray = method()->create_monarray(cinfo->_monstack, esp_offset()); // put in MonitorStack
      elem->set_locations(locarray, exparray, monarray);
      */
    }
    info()->append(elem);
  } else {
    assert(cinfo->_spilled_oops == NULL, "we should not generate spilled oops");
  }
}


void CodeEmitter::add_debug_info_for_branch(int bci, const RInfoCollection* oop_regs) {
  if (UseCompilerSafepoints)  {
    __ code()->relocate(__ pc(), relocInfo::safepoint_type);
    CodeEmitInfo info(bci, NULL, NULL, oop_regs);
    add_debug_info_here(&info);
  }
}


//---------------------item conversion--------------------------


Register CodeEmitter::item2reg(Item* item) {
  assert(item->is_register() && !item->type()->is_double_word() && !item->type()->is_float(), "type error");
  return item->get_register().as_register();
}


Register CodeEmitter::item2regLO(Item* item) {
  assert(item->is_register() && item->type()->is_double_word() && !item->type()->is_double(), "type error");
  return item->get_register().as_register_lo();
}


Register CodeEmitter::item2regHI(Item* item) {
  assert(item->is_register() && item->type()->is_double_word() && !item->type()->is_double(), "type error");
  return item->get_register().as_register_hi();
}



Address CodeEmitter::stack_item2addr(const Item* item, bool for_hi_word) {
  assert(item->is_stack(),"error");
  if (item->is_spilled()) {
    return frame()->spill_address(item->get_spilled_index(), item->type()->is_double_word(), for_hi_word);
  } else {
    if (item->type()->is_double()) {
      check_double_address(item->get_stack());
    }
    return frame()->local_address(item->get_stack(), item->type()->is_double_word(), for_hi_word);
  }
}


Address CodeEmitter::stack_item2addrLO(const Item* item) {
  return stack_item2addr(item, false);
}


Address CodeEmitter::stack_item2addrHI(const Item* item) {
  return stack_item2addr(item, true);
}


void CodeEmitter::set_bailout(const char* msg) {
  if (PrintBailouts && !_bailout) { // print only first time
    tty->print(" Bailout reason: %s", msg);
  }
  _bailout = true;  
}


void CodeEmitter::add_call_info_here(CodeEmitInfo* info) {
  add_call_info(__ offset(), info);
}


void CodeEmitter::add_debug_info_here(CodeEmitInfo* info) {
  add_debug_info(__ offset(), info);
}

//-----------------------------------------------
//------------------LineNumbers------------------------------------


class LineNumbersOffset: public ResourceObj {
 private:
  enum { unused = -1 };
  GrowableArray<LinePair*>* _pairs;
  GrowableArray<int>*       _bci_offset; // array[bci] == pc_offset

 public:
  LineNumbersOffset(Compilation* compilation, ciMethod* m);
  void add_offset(int bci, int offset);
  JVMPI_Lineno* get_lineno_pairs();
  int get_lineno_count() const;
};


LineNumbersOffset::LineNumbersOffset(Compilation* compilation, ciMethod* m) {
  _pairs = compilation->get_method_line_numbers(m);
  int n = _pairs->length();
  int size = m->code_size();
  _bci_offset  = new GrowableArray<int>(size, size, unused);
}


void LineNumbersOffset::add_offset(int bci, int offset) {
  if (_bci_offset->at(bci) == unused) {
    // use only first reference to that bci
    _bci_offset->at_put_grow(bci, offset);
  }
}


JVMPI_Lineno* LineNumbersOffset::get_lineno_pairs() {
  int n = _pairs->length();
  JVMPI_Lineno* res = NEW_RESOURCE_ARRAY(JVMPI_Lineno, n);
  for (int i = 0; i < n; i++) {
    int bci  = _pairs->at(i)->bci();
    res[i].offset = _bci_offset->at(bci);
    res[i].lineno = _pairs->at(i)->line();
  }
  return res;
}


int LineNumbersOffset::get_lineno_count() const{
  return _pairs->length();
}


//-----------------------------------------------

void CodeEmitter::mark_offset_for_bci(int bci) {
  if (_linenumbers_offset == NULL) {
    _linenumbers_offset = new LineNumbersOffset(compilation(), method());
  }
  _linenumbers_offset->add_offset(bci, __ offset());
}


JVMCI_LineNumbersDesc* CodeEmitter::get_jvmpi_lineno_table() {
  JVMCI_LineNumbersDesc* res = new JVMCI_LineNumbersDesc();
  if (_linenumbers_offset != NULL) {
    res->_nof_lines =  _linenumbers_offset->get_lineno_count();
    res->_line_numbers = _linenumbers_offset->get_lineno_pairs();
  }
  return res;
}
