// @(#)c1_CodeGenerator.hpp	1.110 00/06/09 17:15:22
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

// The classes responsible for code emission and register allocation



class LookupRange: public ResourceObj {
 public:
  int _low_key;
  int _high_key;
  BlockBegin* _sux;
 public:
  LookupRange(int start_key, BlockBegin* sux): _low_key(start_key), _high_key(start_key), _sux(sux) {}
  void set_high_key(int key) { _high_key = key; }

  int high_key() const { return _high_key; }
  int low_key() const { return _low_key; }
  BlockBegin* sux() const { return _sux; }
};


class ValueGenInvariant: public StackObj {
 private:
  RegAlloc*     _ra;        // register allocator
  CodeEmitter*  _emit;      // code emitter
  ciMethod*     _method;    // method that we are compiling
  BlockBegin*   _block;
 public:
  ValueGenInvariant(ciMethod* m, RegAlloc* ra, CodeEmitter* ce) : _method(m), _emit(ce), _ra(ra), _block(NULL) {}
  RegAlloc*     reg_alloc() const { return _ra;     }
  CodeEmitter*  code_emit() const { return _emit;   }
  ciMethod*     method() const    { return _method; }
  BlockBegin*   block() const     { return _block;  }
  void set_block(BlockBegin* block) { _block = block; }
};


// only the classes below belong in the same file
class ValueGen: public InstructionVisitor {
 private:
  // ---------------------------STATICS-------------------------------
  static bool  _init_done;

 private:
  Compilation*  _compilation;

  // data
  const Item*   _hint;      // hints what result should look like (e.g, preferred register)
  Item*         _result;    // describes the result of do_xxx function
  ValueGenInvariant* _data; // data that is invariant 

  // accessors
  Compilation*  compilation() const { return _compilation; }
  const Item*   hint()      const { return _hint;      }
  Item*         result()    const { return _result;    }
  RegAlloc*     ra()        const { return _data->reg_alloc();  } 
  bool          gen_code()  const { return code_emit() != NULL; }
  CodeEmitter*  code_emit() const { return _data->code_emit();  }
  ciMethod*     method()    const { return _data->method();     }
  BlockBegin*   block()     const { return _data->block();      }

  bool is_strict_fp()       const { return method()->is_strict(); }

  // register allocation
  RInfo rlock                      (Value instr);                    // lock a free register, spill if necessary
  RInfo rlock                      (Value instr, const Item* hint);  // lock a free register, spill if necessary, use hint if possible
  RInfo rlock_call_result          (Value instr);                    // lock the return value register for a call, spill if necessary
  RInfo rlock_result_with_hint     (Value instr, const Item* hint);

  void  lock_rinfo                 (Value instr, RInfo reg);         // lock a specific register, it must be free
  void  lock_spill_rinfo           (Value instr, RInfo reg, int rc); // lock a specific register, if it's not free spill
  RInfo lock_free_rinfo            (Value instr, ValueType* type);   // lock a free register of a specific type, spill if necessary
  RInfo lock_free_rinfo            (Value instr, c1_RegMask mask);   // lock a free register from the register set, spill if necessary

  RInfo get_free_rinfo (ValueType* type);      // does not lock
  RInfo get_free_rinfo (c1_RegMask mask);      // does not lock

  void  set_maynot_spill(Item* item);

  void  rfree(Item* item);
  void  raw_rfree(Item* item);

  bool  is_free_rinfo    (RInfo reg);

  void  set_result(Value x, RInfo reg, bool is_cached = false)  {  result()->set_register(reg, is_cached); }
  void  set_no_result(Value x);

  bool  cached_cpu_local (int index, RInfo& r); // returns true if value v is cached in register r

  friend class HideReg;
  friend class TempReg;
  friend class SpillLockReg;

  RInfo result_register_for(ValueType* type, bool callee = false);

  void item_free(Item* item);
  void release_item(Item* item);

  // spilling
  void  sfree(Item* item);

  void  spill_one      (ValueType* type);
  void  spill_register (RInfo reg);
  void  spill_item     (Item* item); 
  void  spill_value    (Value value);
  void  spill_values_on_stack (ValueStack* stack, RInfo hide_reg = RInfo(), bool caller_to_callee = false);
  void  spill_caller_save   (bool is_C_call);
  bool  try_caller_to_callee(Value value, RInfo hide_reg);

  void  move_to_phi(ValueStack* stack);
  void  setup_phis_for_switch(Item* tag, ValueStack* stack);


  bool  item_may_be_null(Item* item);


  // code emission
  void  finish_root          (Value instr);
  void  check_float_register (Item* item );
  bool  must_copy_register   (Item* item );

  void  dont_load_item         (Item* item);
  void  dont_load_item_nocheck (Item* item);

  void  load_item       (Item* item);
  void  load_item_patching(Item* item);
  void  load_byte_item  (Item* item);

  bool  fpu_fanout_handled (Item* item);

  void  load_item_hint  (Item* item, const Item* hint);
  void  load_item_force (Item* item, RInfo force_reg);
  void  move_spill_to   (int spill_ix, int least_spill_ix, Value v);

  void  load_item_with_reg_mask(Item* item, c1_RegMask mask);

  void  check_item      (Item* item);
  void  check_item_load (Item* item);

  void  check_result_usage (Value instr); // release reg/spill if instr->use_count is 0
  void  release_roots(ValueStack* stack);

  void do_ArithmeticOp_Long   (ArithmeticOp*    x);
  void do_ArithmeticOp_Int    (ArithmeticOp*    x);
  void do_ArithmeticOp_FPU    (ArithmeticOp*    x);

  // statics
  static RInfo noRInfo();
  static RInfo return1RInfo();
  static RInfo callee_return1RInfo();
  static RInfo divInRInfo();
  static RInfo divOutRInfo();
  static RInfo remOutRInfo();
  static RInfo shiftCountRInfo();
  static RInfo return2RInfo();         // for long result
  static RInfo callee_return2RInfo();
  static RInfo returnF0RInfo();
  static RInfo returnD0RInfo();
  static RInfo callee_receiverRInfo();
  static RInfo syncTempRInfo();
  static RInfo nonReturnRInfo();       // an rinfo that is not a return register

  static bool is_caller_save_register(RInfo reg, bool is_C_call = false);

  void invoke_do_arguments (Invoke* x);
  void invoke_do_spill     (Invoke* x, RInfo receiver_reg = RInfo());
  void invoke_do_result    (Invoke* x, bool needs_null_check, Item* receiver = NULL);

  GrowableArray<Item*>* invoke_visit_arguments(Invoke* x, CallingConvention* arg_loc);
  void invoke_load_arguments(Invoke* x, GrowableArray<Item*>* args, CallingConvention* arg_loc);

  void do_ShortArithmeticOp(ArithmeticOp*  x, RInfo r);
  void goto_default_successor(BlockEnd* b);

  //---------
  Value compute_phi_arrays(ValueStack* stack, GrowableArray<Value>* phi_values, 
                                              GrowableArray<int>* spill_ixs);

  RInfoCollection* get_block_item_oop_regs();
  RInfoCollection* get_registers_with_oops();

  bool is_backward_branch(Value instr, BlockBegin* dest) const;
  bool must_round(Value x, const Item* hint);
  bool canbe_short_form_arithmetic(StoreLocal* x);
  bool is_32bit_mode();

  void do_LookupSwitch_key(Item* tag, LookupSwitch* x, LookupRange* range);
  void do_LookupSwitch_default(LookupSwitch* x);

  void print_at_do_root(Value instr, bool before_visit) PRODUCT_RETURN;
  void print_at_walk(Value instr, bool before_visit) PRODUCT_RETURN;
  void print_spilling(int bci, int spill_ix) PRODUCT_RETURN;

  bool is_simm13(Item* item); // for SPARC

  void check_for_spill(Item* item, RInfo reg);


 public:
  static RInfo receiverRInfo();
  static void init_value_gen();

  // This constructor is called as the main root object that passes
  // Code Emitter and register allocation structure to other ValueGen objects
  ValueGen::ValueGen(Compilation* compilation, ValueGenInvariant* vgi,  bool gen_code) :
    _compilation(compilation), _result(NULL), _hint(NULL), _data(vgi)
  {  assert(_init_done, "ValueGen is not initialized");  }

  // This is called for each instruction and is used to visit each node;
  // It returns the state in _result
  ValueGen::ValueGen(Item* result, const Item* hint, ValueGen* parent):
    _compilation(parent->compilation()), _hint(hint), _result(result), _data(parent->_data)
  { assert(_init_done, "ValueGen is not initialized");
    walk(result->value());
  }

  void set_invariant(ValueGenInvariant* vgi) { _data = vgi; }
  void set_block(BlockBegin* block) { _data->set_block(block); }

  bool is_fpu_stack_free()            { return !gen_code() || code_emit()->is_empty_fpu_stack();}
  bool is_cpu_stack_free()            { return !gen_code() || code_emit()->esp_offset() == 0;   }
  void block_prolog(BlockBegin* block);
  void block_epilog(BlockBegin* block);

  void do_root (Instruction* instr);
  void walk    (Instruction* instr);

  void bind_label(Label* label);
  void align_block(BlockBegin* block);

  void exception_handler_start(int bci);

  // visitor functionality
  virtual void do_Phi            (Phi*             x);
  virtual void do_Constant       (Constant*        x);
  virtual void do_LoadLocal      (LoadLocal*       x);
  virtual void do_StoreLocal     (StoreLocal*      x);
  virtual void do_LoadField      (LoadField*       x);
  virtual void do_StoreField     (StoreField*      x);
  virtual void do_ArrayLength    (ArrayLength*     x);
  virtual void do_LoadIndexed    (LoadIndexed*     x);
  virtual void do_StoreIndexed   (StoreIndexed*    x);
  virtual void do_LoopEnter      (LoopEnter*       x);
  virtual void do_LoopExit       (LoopExit*        x);
  virtual void do_NegateOp       (NegateOp*        x);
  virtual void do_ArithmeticOp   (ArithmeticOp*    x);
  virtual void do_ShiftOp        (ShiftOp*         x);
  virtual void do_LogicOp        (LogicOp*         x);
  virtual void do_CompareOp      (CompareOp*       x);
  virtual void do_IfOp           (IfOp*            x);
  virtual void do_Convert        (Convert*         x);
  virtual void do_NullCheck      (NullCheck*       x);
  virtual void do_JsrContinuation(JsrContinuation* x);
  virtual void do_Jsr            (Jsr*             x);
  virtual void do_Invoke         (Invoke*          x);
  virtual void do_NewInstance    (NewInstance*     x);
  virtual void do_NewTypeArray   (NewTypeArray*    x);
  virtual void do_NewObjectArray (NewObjectArray*  x);
  virtual void do_NewMultiArray  (NewMultiArray*   x);
  virtual void do_CheckCast      (CheckCast*       x);
  virtual void do_InstanceOf     (InstanceOf*      x);
  virtual void do_MonitorEnter   (MonitorEnter*    x);
  virtual void do_MonitorExit    (MonitorExit*     x);
  virtual void do_Intrinsic      (Intrinsic*       x);
  virtual void do_BlockBegin     (BlockBegin*      x);
  virtual void do_Goto           (Goto*            x);
  virtual void do_If             (If*              x);
  virtual void do_IfInstanceOf   (IfInstanceOf*    x);
  virtual void do_TableSwitch    (TableSwitch*     x);
  virtual void do_LookupSwitch   (LookupSwitch*    x);
  virtual void do_Ret            (Ret*             x);
  virtual void do_Return         (Return*          x);
  virtual void do_Throw          (Throw*           x);
  virtual void do_Base           (Base*            x);
};


class CodeGenerator: public BlockClosure {
 private:
  ValueGen* _gen;
  int       _max_spills;
  ValueGenInvariant* _vgi;

  void block_do_prolog(BlockBegin* block);
  void block_do_epilog(BlockBegin* block);

 public:
  CodeGenerator(ValueGen* gen, ValueGenInvariant* _vgi);

  void block_do(BlockBegin* block);

  bool has_spills() const { return _max_spills > 0; }
  int  max_spills() const { return _max_spills;     }

  static void clear_instruction_items(BlockBegin* b);
};


class HideReg: public StackObj {
 private:
  ValueGen* _vg;
  RInfo     _reg;
 public:
  HideReg(ValueGen* vg, RInfo reg, bool spill = false);
  // spills, locks
  HideReg(ValueGen* vg, ValueType* t);
  HideReg(ValueGen* vg, c1_RegMask mask);
  ~HideReg();

  RInfo reg() const { return _reg; }
};







