// @(#)c1_Compilation.hpp	1.25 00/04/26 12:03:27
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


class Compilation {
 private:
  // compilation specifics
  ciEnv*             _env;
  ciMethod*          _method;
  int                _osr_bci;
  IR*                _hir;
  ScanResult         _scan_result;               // needs to go into 'Scope' for Merlin
  int                _max_spills;
  c1_RegMask         _used_registers;            // unavailable for global local caching
  c1_RegMask         _free_registers;            //   available for global local caching
  ciOopMap*          _oop_map;
  FrameMap*          _frame_map;
  C1_MacroAssembler* _masm;
  const char*        _bailout_msg;

  GrowableArray<JVMCI_DebugInfo*>* _info;
  JVMCI_LineNumbersDesc*           _jvmpi_lines;

  // JVMPI specifics
  bool _jvmpi_event_compiled_method_enabled;
  bool _jvmpi_event_method_entry_enabled;
  bool _jvmpi_event_method_entry2_enabled;
  bool _jvmpi_event_method_exit_enabled;

  // compilation helpers
  void initialize();
  void initialize_oop_maps();
  void build_hir();
  void scan_method();
  void allocate_registers();
  const CachedLocals* cache_locals();
  void emit_code_prolog(FrameMap* map);
  void emit_code_epilog(CodeEmitter* emit, int max_spills, RInfo cached_receiver = FrameMap::_noRInfo);
  GrowableArray<int>* get_init_vars();
  int  emit_code_body(JVMCI_OffsetDesc* offsets, const CachedLocals* cached_locals);
  int  emit_accessor_body(JVMCI_OffsetDesc* offsets, AccessorInfo* info);
  int  compile_java_method(JVMCI_OffsetDesc& offsets);
  void emit_code_for_native(address native_entry, JVMCI_OffsetDesc* offsets);
  int  compile_native_method(JVMCI_OffsetDesc& offsets);
  ciMethod::IntrinsicId library_method_id() const;
  int  compile_library_method(JVMCI_OffsetDesc& offsets);
  void install_code(JVMCI_OffsetDesc& offsets, int frame_size);
  void compile_method();
  void enter_in_oop_map(OopMap* map, ArgumentLocation loc, int local_ix, int esp_offset);

 public:
  // The following functionality should be inherited from AbstractCompilation
  // as soon as we have that class instead of ciEnv.
  GrowableArray<LinePair*>* Compilation::get_method_line_numbers(ciMethod *m);

 public:
  // creation
  Compilation(ciEnv* env, ciMethod* method, int osr_bci, C1_MacroAssembler* masm);

  // accessors
  ciMethod* method() const                       { return _method; }
  int osr_bci() const                            { return _osr_bci; }
  bool is_osr_compile() const                    { return osr_bci() >= 0; }
  IR* hir() const                                { return _hir; }
  ScanResult* scan_result()                      { return &_scan_result; }
  int max_spills() const                         { return _max_spills; }
  c1_RegMask used_registers() const              { return _used_registers; }
  c1_RegMask free_registers() const              { return _free_registers; }
  ciOopMap* oop_map() const                      { return _oop_map; }
  FrameMap* frame_map() const                    { return _frame_map; }
  CodeBuffer* code() const                       { return _masm->code(); }
  C1_MacroAssembler* masm() const                { return _masm; }
  GrowableArray<JVMCI_DebugInfo*>* info() const  { return _info; }
  JVMCI_LineNumbersDesc* jvmpi_lines() const     { return _jvmpi_lines; }

  // jvmpi flags
  bool jvmpi_event_compiled_method_enabled()const{ return _jvmpi_event_compiled_method_enabled; };
  bool jvmpi_event_method_entry_enabled() const  { return _jvmpi_event_method_entry_enabled;}
  bool jvmpi_event_method_entry2_enabled() const { return _jvmpi_event_method_entry2_enabled;}
  bool jvmpi_event_method_exit_enabled() const   { return _jvmpi_event_method_exit_enabled;}
  bool jvmpi_event_method_enabled() const {
    return
      jvmpi_event_method_entry_enabled()  ||
      jvmpi_event_method_entry2_enabled() ||
      jvmpi_event_method_exit_enabled();
  }

  // oop maps / address maps
  OopMap* create_oop_map          (int bci, GrowableArray<int>* spilled_oops, int esp_offset = 0);
  OopMap* create_oop_map_for_own_signature (GrowableArray<int>* spilled_oops, int esp_offset = 0);
  OopMap* create_oop_map_inside_natives    (GrowableArray<int>* spilled_oops, int esp_offset = 0);
  
  void add_registers_to_oop_map  (OopMap* map, const RInfoCollection* reg_oops, int esp_offset);
  GrowableArray<int>* get_address_map(int bci);

  // error handling
  void bailout(const char* msg);
  bool bailed_out() const                        { return _bailout_msg != NULL; }
  const char* bailout_msg() const                { return _bailout_msg; }
};
