// @(#)c1_Compiler.cpp	1.64 00/01/26 17:14:55
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_c1_Compiler.cpp.incl"


Compiler::Compiler() {
  _code_buffer_size = 0;
  _locs_buffer_size = 0;
  _code_buffer      = NULL;
  _locs_buffer      = NULL;
}


Compiler::~Compiler() {
  Unimplemented();
}


void Compiler::initialize() {
  // setup CodeBuffer
  assert(_code_buffer == NULL, "initialization should happen only once");
  _code_buffer_size = CodeBuffer::insts_memory_size(desired_max_code_buffer_size);
  _locs_buffer_size = CodeBuffer::locs_memory_size (desired_max_locs_buffer_size);
  _code_buffer = NEW_C_HEAP_ARRAY(u_char, _code_buffer_size);
  _locs_buffer = NEW_C_HEAP_ARRAY(u_char, _locs_buffer_size);
  // install exception filters
  // note: this should be done once for all (C1) Compilers
  //       instead of once for every (C1) Compiler!
  JVMCI_InstallImplicitNullExceptionFilter(Runtime1::entry_for(Runtime1::implicit_null_exception_id));
  JVMCI_InstallImplicitDiv0ExceptionFilter(Runtime1::entry_for(Runtime1::implicit_div0_exception_id));
  JVMCI_InstallStackOvrFlowExceptionFilter(Runtime1::entry_for(Runtime1::stack_overflow_exception_id));
  // done with installation
  mark_initialized();
}


void Compiler::compile_method(ciEnv* env, ciScope* scope, ciMethod* method, int entry_bci, bool reuse_env) {
  // setup assembler
  CodeBuffer*        code = new CodeBuffer(_code_buffer_size, _locs_buffer_size, 0, 0, true, _code_buffer, _locs_buffer);
  C1_MacroAssembler* masm = new C1_MacroAssembler(code);
  // invoke compilation
  Compilation c(env, method, entry_bci, masm);
}


void Compiler::print_timers() {
  Unimplemented();
}


void compiler1_init() {
  // This is needed as runtime1 stubs are generated always
  NEEDS_CLEANUP
#ifndef SPARC  // expires 9/15/1999
  SharedInfo::set_stack0(32);
#else
  SharedInfo::set_stack0(96);
#endif
  // No initialization needed if interpreter only
  if (Arguments::mode() != Arguments::_int) {
    RegAlloc::init_register_allocation();
  }
}

