/*
 * @(#)c1_FrameMap.cpp	1.5 00/03/09 11:00:43
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


# include "incls/_precompiled.incl"
# include "incls/_c1_FrameMap.cpp.incl"



//-----------------------------------------------------

// Helper routine to simplify transition from old
// jvmci interface to new CI interface. Eventually
// we should adjust the code to use ciSignature*
// instead (gri 1/28/2000).

GrowableArray<BasicType>* FrameMap::signature_type_array_for(const ciMethod* method) {
  ciSignature* sig = method->signature();
  GrowableArray<BasicType>* sta = new GrowableArray<BasicType>(method->arg_size());
  // add receiver, if any
  if (!method->is_static()) sta->append(T_OBJECT);
  // add remaining arguments
  for (int i = 0; i < sig->count(); i++) {
    /* add basic type once */   sta->append(sig->type_at(i));
    if (sig->is_two_word_at(i)) sta->append(sig->type_at(i));
  }
  // done
  assert(sta->is_full(), "should have used all slots");
  return sta;
}


//--------------------------------------------------------
//               FrameMap
//--------------------------------------------------------

bool      FrameMap::_init_done = false;
Register  FrameMap::_cpu_regs [FrameMap::nof_cpu_regs];


Register FrameMap::cpu_rnr2reg (int rnr) {
  assert(_init_done, "tables not initialized");
  debug_only(cpu_range_check(rnr);)
  return _cpu_regs[rnr];
}


// This is slow, called rarely
int FrameMap::cpu_reg2rnr (Register reg) {
  assert(_init_done, "tables not initialized");
  for (int i = 0; i < nof_cpu_regs; i++) {
    if (cpu_rnr2reg(i).number() == reg.number()) return i;
  }
  return -1;
}


CallingConvention* FrameMap::calling_convention (const ciMethod* method, GrowableArray<int>* reg_args /* = NULL */) {
  const GrowableArray<BasicType>* signature = signature_type_array_for(method);
  return calling_convention (method->is_static(), *signature, reg_args);
}
