// @(#)c1_GraphBuilder.hpp	1.39 00/04/10 17:47:15
// 
// Copyright 1993-1999 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


class BlockListBuilder VALUE_OBJ_CLASS_SPEC {
 private:
  IRScope*    _scope;
  BlockList*  _bci2block;
  BlockBegin* _std_entry;
  BlockBegin* _osr_entry;

  // helper functions
  BlockBegin* new_block_at(int bci, BlockBegin::Flag f = BlockBegin::no_flag);
  void set_leaders();
  void set_xhandler_entries();

 public:
  // creation
  BlockListBuilder(IRScope* scope, int osr_bci);

  // accessors
  IRScope*      scope() const                    { return _scope; }
  ciMethod*     method() const                   { return scope()->method(); }
  XHandlers*    xhandlers() const                { return scope()->xhandlers(); }
  BlockList*    bci2block() const                { return _bci2block; }
  BlockBegin*   std_entry() const                { return _std_entry; }
  BlockBegin*   osr_entry() const                { return _osr_entry; }
};


class GraphBuilder VALUE_OBJ_CLASS_SPEC {
 private:
  // for all GraphBuilders
  static bool       _is_initialized;             // true if trap tables were initialized, false otherwise
  static bool       _can_trap[Bytecodes::number_of_java_codes];
  static bool       _is_async[Bytecodes::number_of_java_codes];

  // for each instance of GraphBuilder
  Compilation*      _compilation;                // the current compilation
  IRScope*          _scope;                      // the current scope
  BlockList*        _bci2block;                  // the mapping of bci's -> basic blocks
  ValueMap*         _vmap;                       // the map of values encountered (for CSE)
  bool              _has_handler;                // true if the method has exception handlers
  bool              _found_jsr;                  // true if GraphBuilder encountered a jsr, sticky
  const char*       _bailout_msg;                // non-null if GraphBuilder bailed out, sticky

  // for each call to build_transitive_closure
  BlockBegin*       _block;                      // the current block
  ValueStack*       _state;                      // the current execution state for scope
  ciBytecodeStream* _stream;                     // the bytecodes
  Instruction*      _last;                       // the last instruction added

  // accessors
  Compilation*      compilation() const          { return _compilation; }
  IRScope*          scope() const                { return _scope; }
  BlockList*        bci2block() const            { return _bci2block; }
  ValueMap*         vmap() const                 { return _vmap; }
  bool              has_handler() const          { return _has_handler; }
  ciMethod*         method() const               { return scope()->method(); }
  XHandlers*        xhandlers() const            { return scope()->xhandlers(); }

  ValueStack*       state() const                { return _state; }
  ciBytecodeStream* stream() const               { return _stream; }
  Instruction*      last() const                 { return _last; }
  Bytecodes::Code   code() const                 { return stream()->code(); }
  int               bci() const                  { return stream()->bci(); }
  
  // stack manipulation helpers
  void ipush(Value t) const                      { state()->ipush(t); }
  void lpush(Value t) const                      { state()->lpush(t); }
  void fpush(Value t) const                      { state()->fpush(t); }
  void dpush(Value t) const                      { state()->dpush(t); }
  void apush(Value t) const                      { state()->apush(t); }
  void  push(ValueType* type, Value t) const     { state()-> push(type, t); }

  Value ipop() const                             { return state()->ipop(); }
  Value lpop() const                             { return state()->lpop(); }
  Value fpop() const                             { return state()->fpop(); }
  Value dpop() const                             { return state()->dpop(); }
  Value apop() const                             { return state()->apop(); }
  Value  pop(ValueType* type) const              { return state()-> pop(type); }

  // instruction helpers
  void load_constant();
  void load_local(ValueType* type, int index);
  void store_local(ValueType* type, int index);
  void load_indexed (BasicType type);
  void store_indexed(BasicType type);
  void stack_op(Bytecodes::Code code);
  void arithmetic_op(ValueType* type, Bytecodes::Code code);
  void negate_op(ValueType* type);
  void shift_op(ValueType* type, Bytecodes::Code code);
  void logic_op(ValueType* type, Bytecodes::Code code);
  void compare_op(ValueType* type, Bytecodes::Code code);
  void convert(Bytecodes::Code op, BasicType from, BasicType to);
  void increment();
  void if_node(Value x, If::Condition cond, Value y);
  void if_zero(ValueType* type, If::Condition cond);
  void if_null(ValueType* type, If::Condition cond);
  void if_same(ValueType* type, If::Condition cond);
  void jsr(int dest);
  void table_switch();
  void lookup_switch();
  void method_return(Value x);
  void access_field(Bytecodes::Code code);
  void invoke(Bytecodes::Code code);
  void new_instance(int klass_index);
  void new_type_array();
  void new_object_array();
  void check_cast(int klass_index);
  void instance_of(int klass_index);
  void monitorenter(Value x);
  void monitorexit(Value x);
  void new_multi_array(int dimensions);

  // stack/code manipulation helpers
  Instruction* append_base(Instruction* instr);
  Instruction* append(Instruction* instr);
  Instruction* append_split(StateSplit* instr);

  // other helpers
  static bool is_initialized()                   { return _is_initialized; }
  static bool can_trap(Bytecodes::Code code) {
    assert(0 <= code && code < Bytecodes::number_of_java_codes, "illegal bytecode");
    return _can_trap[code];
  }
  static bool is_async(Bytecodes::Code code) {
    assert(0 <= code && code < Bytecodes::number_of_java_codes, "illegal bytecode");
    return _is_async[code];
  }
  void bailout(const char* msg);
  BlockBegin* block_at(int bci)                  { return _bci2block->at(bci); }
  void handle_exception(bool is_async);
  BlockEnd* connect_to_end(BlockBegin* beg);
  void build_transitive_closure(BlockBegin* b);

  friend class Inliner;

 public:
  // initialization
  static void initialize();

  // creation
  GraphBuilder(Compilation* compilation, IRScope* scope, BlockList* bci2block, BlockBegin* start);
  bool found_jsr() const                         { return _found_jsr; }
  bool bailed_out() const                        { return _bailout_msg != NULL; }
  const char* bailout_msg() const                { return _bailout_msg; }
};

