// @(#)c1_Inliner.hpp	1.11 00/03/21 16:16:34
// 
// Copyright 1993-1999 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


class GraphBuilder;

class Inliner: public StackObj {
 private:
  GraphBuilder*   _gb;
  const char*     _bailout_msg;                  // non-null if Inliner bailed out, sticky

  // accessors
  GraphBuilder*   gb() const                     { return _gb; }
  Compilation*    compilation() const            { return gb()->compilation(); }
  IRScope*        scope() const                  { return gb()->scope(); }
  ciMethod*       method() const                 { return gb()->method(); }

  ValueStack*     state() const                  { return gb()->state(); }
  Instruction*    last() const                   { return gb()->last(); }
  Bytecodes::Code code() const                   { return gb()->code(); }
  int             bci() const                    { return gb()->bci(); }

  // helpers
  void bailout(const char* msg);

  // inliners
  bool try_inline_intrinsics(ciMethod* callee);
  bool try_inline_simple    (ciMethod* callee);

 public:
  Inliner(GraphBuilder* gb) : _gb(gb), _bailout_msg(NULL) {
    assert(gb != NULL, "GraphBuilder must exist");
  }

  const char* bailout_msg() const                { return _bailout_msg; }
  bool  bailed_out() const                       { return bailout_msg() != NULL; }

  bool try_inline(ciMethod* callee);
};

