/*
 * @(#)c1_Instruction.cpp	1.49 00/06/09 11:06:29
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_c1_Instruction.cpp.incl"


// Implementation of Instruction


int Instruction::_next_id = 0;


Instruction::Condition Instruction::mirror(Condition cond) {
  switch (cond) {
    case eql: return eql;
    case neq: return neq;
    case lss: return gtr;
    case leq: return geq;
    case gtr: return lss;
    case geq: return leq;
  }
  ShouldNotReachHere();
  return eql;
}


Instruction::Condition Instruction::negate(Condition cond) {
  switch (cond) {
    case eql: return neq;
    case neq: return eql;
    case lss: return geq;
    case leq: return gtr;
    case gtr: return leq;
    case geq: return lss;
  }
  ShouldNotReachHere();
  return eql;
}


Instruction* Instruction::prev(BlockBegin* block) {
  Instruction* p = NULL;
  Instruction* q = block;
  while (q != this) {
    assert(q != NULL, "this is not in the block's instruction list");
    p = q; q = q->next();
  }
  return p;
}


#ifndef PRODUCT
void Instruction::print() {
  InstructionPrinter ip;
  print(ip);
}


void Instruction::print(InstructionPrinter& ip) {
  ip.print_head();
  ip.print_line(this);
  tty->cr();
}
#endif // PRODUCT


// Implementation of StoreIndexed

void StoreIndexed::state_values_do(void f(Value*)) { 
  if (state() != NULL) state()->values_do(f);
}


// Implementation of ArithmeticOp

bool ArithmeticOp::is_commutative() const {
  switch (op()) {
    case Bytecodes::_iadd: // fall through
    case Bytecodes::_ladd: // fall through
    case Bytecodes::_fadd: // fall through
    case Bytecodes::_dadd: // fall through
    case Bytecodes::_imul: // fall through
    case Bytecodes::_lmul: // fall through
    case Bytecodes::_fmul: // fall through
    case Bytecodes::_dmul: return true; 
  }
  return false;
}


bool ArithmeticOp::can_trap() const {
  switch (op()) {
    case Bytecodes::_idiv: // fall through
    case Bytecodes::_ldiv: // fall through
    case Bytecodes::_irem: // fall through
    case Bytecodes::_lrem: return true;
  }
  return false;
}


// Implementation of LogicOp

bool LogicOp::is_commutative() const {
#ifdef ASSERT
  switch (op()) {
    case Bytecodes::_iand: // fall through
    case Bytecodes::_land: // fall through
    case Bytecodes::_ior : // fall through
    case Bytecodes::_lor : // fall through
    case Bytecodes::_ixor: // fall through
    case Bytecodes::_lxor: break;
    default              : ShouldNotReachHere();
  }
#endif
  // all LogicOps are commutative
  return true;
}


// Implementation of IfOp

bool IfOp::is_commutative() const {
  return cond() == eql || cond() == neq;
}


// Implementation of StateSplit

void StateSplit::state_values_do(void f(Value*)) { 
  if (state() != NULL) state()->values_do(f);
}


// Implementation of Invoke

int Invoke::size_of_arguments() const {
  int s = has_receiver() ? 1 : 0;
  int i = number_of_arguments();
  while (i-- > 0) s += argument_at(i)->type()->size();
  return s;
}


GrowableArray<BasicType>* Invoke::signature() const {
  GrowableArray<BasicType>* signature = new GrowableArray<BasicType>();
  if (has_receiver()) {
    signature->append(as_BasicType(receiver()->type()));
  }
  int nof_args = number_of_arguments();
  for (int i = 0; i < nof_args; i++) {
    ValueType* t = argument_at(i)->type();
    signature->append(as_BasicType(t));
    if (t->is_double_word()) {
      signature->append(as_BasicType(t));
    }
  }
  return signature;
}


// Implementation of BlockBegin

int BlockBegin::_next_block_id = 0;


void BlockBegin::set_weight(int start_distance) {
  // computes the weight of a block by looking at its bci, it's distance from the
  // start block and some other attributes - the word's bits are used as follows:
  //
  //  3 3 2 2222222221111111111 0000000000
  //  1 0 9 8765432109876543210 9876543210
  // [0|h|g|       bci         | distance ]
  //
  // the g & h bits are set to force the block to the end of a method
  assert(BitsPerWord == 32, "adjust this code");
  _weight = ((bci() & 0x7FFFF) << 10) | (start_distance & 0x3FF);
  if (end() != NULL && (end()->as_Throw() != NULL || end()->as_Return() != NULL) && !is_set(BlockBegin::std_entry_flag)) {
    // blocks that end with a Throw or Return instruction and that are not the first block
    // (method entry) are given a huge weight (h) to make them move to the end of the method -
    // Throws are always following after Returns
    _weight |= 1 << (end()->as_Throw() != NULL ? 30 : 29);
  };
  assert(_weight >= 0, "weight should be positive");
}


void BlockBegin::add_subroutine(BlockBegin* b) {
  assert(b != NULL && b->is_set(subroutine_entry_flag), "subroutine must exist");
  // add only if not in the list already
  if (!_subroutines.contains(b)) _subroutines.append(b);
}


void BlockBegin::add_exception_handler(BlockBegin* b) {
  assert(b != NULL && b->is_set(exception_entry_flag), "exception handler must exist");
  // add only if not in the list already
  if (!_exception_handlers.contains(b)) _exception_handlers.append(b);
}


void BlockBegin::collect_ends(GrowableArray<bool>& mark, EndList* endlist, bool& bailout) const {
  if (!mark.at(block_id())) {
    mark.at_put(block_id(), true);
    BlockEnd* e = end();
    if (e == NULL) {
      // Note: If e is NULL, we have encountered a block that has not been
      //       completely constructed yet which is the case if it contains
      //       a jsr that lead us to here. In that case, we cannot compute
      //       the endlist correctly and bail out for now (was bug - gri 3/20/2000).
      bailout = true;
      return;
    }
    if (e->number_of_sux() == 0) {
      assert(!endlist->contains(e), "element added before");
      endlist->append(e);
    } else {
      // Note: To be correct we should also find non-Ret ControlFlowEnds of subgraphs
      //       called via a jsr. Currently we use endlist only to find Ret's though;
      //       thus currently it doesn't matter that we ignore these subgraphs. In the
      //       long run we should inline jsr's to avoid all of the jsr-related problems.
      for (int j = e->number_of_sux() - 1; j >= 0; j--) e->sux_at(j)->collect_ends(mark, endlist, bailout);
    }
  }
}


// Note: Currently the endlist is computed every single time it is needed.
//       One could compute the list lazily and store the result - however,
//       it would need to be invalidated after CFG changes. For now we re-
//       compute as this is more robust and it is only called once per jsr.

EndList* BlockBegin::endlist() const {
  GrowableArray<bool> mark(number_of_blocks(), number_of_blocks(), false);
  EndList* endlist = new GrowableArray<BlockEnd*>(1);
  bool bailout = false;
  collect_ends(mark, endlist, bailout);
  return bailout ? NULL : endlist;
}


void BlockBegin::iterate_preorder(GrowableArray<bool>& mark, BlockClosure* closure) {
  if (!mark.at(block_id())) {
    mark.at_put(block_id(), true);
    closure->block_do(this);
    BlockEnd* e = end(); // must do this after block_do because block_do may change it!
    { for (int i = number_of_exception_handlers() - 1; i >= 0; i--) exception_handler_at(i)->iterate_preorder(mark, closure); }
    { for (int i = number_of_subroutines       () - 1; i >= 0; i--) subroutine_at       (i)->iterate_preorder(mark, closure); }
    { for (int i = e->number_of_sux            () - 1; i >= 0; i--) e->sux_at           (i)->iterate_preorder(mark, closure); }
  }
}


void BlockBegin::iterate_postorder(GrowableArray<bool>& mark, BlockClosure* closure) {
  if (!mark.at(block_id())) {
    mark.at_put(block_id(), true);
    BlockEnd* e = end();
    { for (int i = number_of_exception_handlers() - 1; i >= 0; i--) exception_handler_at(i)->iterate_postorder(mark, closure); }
    { for (int i = number_of_subroutines       () - 1; i >= 0; i--) subroutine_at       (i)->iterate_postorder(mark, closure); }
    { for (int i = e->number_of_sux            () - 1; i >= 0; i--) e->sux_at           (i)->iterate_postorder(mark, closure); }
    closure->block_do(this);
  }
}


void BlockBegin::iterate_preorder(BlockClosure* closure) {
  GrowableArray<bool> mark(number_of_blocks(), number_of_blocks(), false);
  iterate_preorder(mark, closure);
}


void BlockBegin::iterate_postorder(BlockClosure* closure) {
  GrowableArray<bool> mark(number_of_blocks(), number_of_blocks(), false);
  iterate_postorder(mark, closure);
}


void BlockBegin::iterate_topological(BlockClosure* closure) {
  BlockCollection bc(number_of_blocks());
  iterate_postorder(&bc);
  bc.iterate_backward(closure);
}


void BlockBegin::block_values_do(void f(Value*)) {
  for (Instruction* n = this; n != NULL; n = n->next()) n->values_do(f);
}


bool BlockBegin::try_join(ValueStack* s) {
  if (state() == NULL) {
    // first visit of block => must initialize state
    set_state(new ValueStack(s));
    return true;
  } else {
    // not first visit to this block => check if states correspond
    return state()->is_same(s);
  }
}


static void resolve_substituted_value(Value* x)  { *x = (*x)->subst(); }

void BlockBegin::resolve_substitution() {
  block_values_do(resolve_substituted_value);
}


#ifndef PRODUCT
void BlockBegin::print_block() {
  InstructionPrinter ip;
  print_block(ip);
}


void BlockBegin::print_block(InstructionPrinter& ip) {
  ip.print_instr(this); tty->cr();
  ip.print_stack(this->stack()); tty->cr();
  ip.print_head();
  for (Instruction* n = next(); n != NULL; n = n->next()) ip.print_line(n);
  tty->cr();
}
#endif // PRODUCT


// Implementation of BlockCollection

void BlockCollection::iterate_forward (BlockClosure* closure) {
  const int l = collection()->length();
  for (int i = 0; i < l; i++) closure->block_do(collection()->at(i));
}


void BlockCollection::iterate_backward(BlockClosure* closure) {
  for (int i = collection()->length() - 1; i >= 0; i--) closure->block_do(collection()->at(i));
}


void BlockCollection::blocks_do(void f(BlockBegin*)) {
  for (int i = collection()->length() - 1; i >= 0; i--) f(collection()->at(i));
}


void BlockCollection::values_do(void f(Value*)) {
  for (int i = collection()->length() - 1; i >= 0; i--) collection()->at(i)->block_values_do(f);
}


#ifndef PRODUCT
void BlockCollection::print(bool cfg_only) {
  InstructionPrinter ip;
  for (int i = 0; i < collection()->length(); i++) {
    BlockBegin* block = collection()->at(i);
    if (cfg_only) {
      ip.print_instr(block); tty->cr();
    } else {
      block->print_block();
    }
  }
}
#endif // PRODUCT


// Implementation of BlockEnd

void BlockEnd::substitute_sux(BlockBegin* old_sux, BlockBegin* new_sux) {
  for (int i = 0; i < number_of_sux(); i++) {
    BlockBegin** sa = addr_sux_at(i);
    if (*sa == old_sux) *sa = new_sux;
  }
}
