/*
 * @(#)c1_Instruction.hpp	1.105 00/06/19 13:26:48
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// Predefined classes
class Item;
class BlockItem;
class ValueStack;
class InstructionPrinter;
class IRScope;


// Instruction class hierarchy
//
// All leaf classes in the class hierarchy are concrete classes
// (i.e., are instantiated). All other classes are abstract and
// serve factoring.

class Instruction;
class   HiWord;
class   Phi;
class   Constant;
class   AccessLocal;
class     LoadLocal;
class     StoreLocal;
class   AccessField;
class     LoadField;
class     StoreField;
class   AccessArray;
class     ArrayLength;
class     AccessIndexed;
class       LoadIndexed;
class       StoreIndexed;
class   AccessLoop;
class     LoopEnter;
class     LoopExit;
class   NegateOp;
class   Op2;
class     ArithmeticOp;
class     ShiftOp;
class     LogicOp;
class     CompareOp;
class     IfOp;
class   Convert;
class   NullCheck;
class   JsrContinuation;
class   StateSplit;
class     Jsr;
class     Invoke;
class     NewInstance;
class     NewArray;
class       NewTypeArray;
class       NewObjectArray;
class       NewMultiArray;
class     TypeCheck;
class       CheckCast;
class       InstanceOf;
class     AccessMonitor;
class       MonitorEnter;
class       MonitorExit;
class     Intrinsic;
class     BlockBegin;
class     BlockEnd;
class       Goto;
class       If;
class       IfInstanceOf;
class       Switch;
class         TableSwitch;
class         LookupSwitch;
class       Ret;
class       Return;
class       Throw;
class       Base;


// A Value is a reference to the instruction creating the value
typedef Instruction* Value;
typedef GrowableArray<Value> Values;


// Frequently used lists
typedef GrowableArray<BlockBegin*> BlockList;
typedef GrowableArray<BlockEnd*> EndList;


// InstructionVisitors provide type-based dispatch for instructions.
// For each concrete Instruction class X, a virtual function do_X is
// provided. Functionality that needs to be implemented for all classes
// (e.g., printing, code generation) is factored out into a specialised
// visitor instead of added to the Instruction classes itself.
 
class InstructionVisitor: StackObj {
 public:
          void do_HiWord         (HiWord*          x) { ShouldNotReachHere(); }
  virtual void do_Phi            (Phi*             x) = 0;
  virtual void do_Constant       (Constant*        x) = 0;
  virtual void do_LoadLocal      (LoadLocal*       x) = 0;
  virtual void do_StoreLocal     (StoreLocal*      x) = 0;
  virtual void do_LoadField      (LoadField*       x) = 0;
  virtual void do_StoreField     (StoreField*      x) = 0;
  virtual void do_ArrayLength    (ArrayLength*     x) = 0;
  virtual void do_LoadIndexed    (LoadIndexed*     x) = 0;
  virtual void do_StoreIndexed   (StoreIndexed*    x) = 0;
  virtual void do_LoopEnter      (LoopEnter*       x) = 0;
  virtual void do_LoopExit       (LoopExit*        x) = 0;
  virtual void do_NegateOp       (NegateOp*        x) = 0;
  virtual void do_ArithmeticOp   (ArithmeticOp*    x) = 0;
  virtual void do_ShiftOp        (ShiftOp*         x) = 0;
  virtual void do_LogicOp        (LogicOp*         x) = 0;
  virtual void do_CompareOp      (CompareOp*       x) = 0;
  virtual void do_IfOp           (IfOp*            x) = 0;
  virtual void do_Convert        (Convert*         x) = 0;
  virtual void do_NullCheck      (NullCheck*       x) = 0;
  virtual void do_JsrContinuation(JsrContinuation* x) = 0;
  virtual void do_Jsr            (Jsr*             x) = 0;
  virtual void do_Invoke         (Invoke*          x) = 0;
  virtual void do_NewInstance    (NewInstance*     x) = 0;
  virtual void do_NewTypeArray   (NewTypeArray*    x) = 0;
  virtual void do_NewObjectArray (NewObjectArray*  x) = 0;
  virtual void do_NewMultiArray  (NewMultiArray*   x) = 0;
  virtual void do_CheckCast      (CheckCast*       x) = 0;
  virtual void do_InstanceOf     (InstanceOf*      x) = 0;
  virtual void do_MonitorEnter   (MonitorEnter*    x) = 0;
  virtual void do_MonitorExit    (MonitorExit*     x) = 0;
  virtual void do_Intrinsic      (Intrinsic*       x) = 0;
  virtual void do_BlockBegin     (BlockBegin*      x) = 0;
  virtual void do_Goto           (Goto*            x) = 0;
  virtual void do_If             (If*              x) = 0;
  virtual void do_IfInstanceOf   (IfInstanceOf*    x) = 0;
  virtual void do_TableSwitch    (TableSwitch*     x) = 0;
  virtual void do_LookupSwitch   (LookupSwitch*    x) = 0;
  virtual void do_Ret            (Ret*             x) = 0;
  virtual void do_Return         (Return*          x) = 0;
  virtual void do_Throw          (Throw*           x) = 0;
  virtual void do_Base           (Base*            x) = 0;
};


// Hashing support
//
// Note: This hash functions affect the performance
//       of ValueMap - make changes carefully!

#define HASH1(x1            )                    ((int)(x1))
#define HASH2(x1, x2        )                    ((HASH1(x1        ) << 7) ^ HASH1(x2))
#define HASH3(x1, x2, x3    )                    ((HASH2(x1, x2    ) << 7) ^ HASH1(x3))
#define HASH4(x1, x2, x3, x4)                    ((HASH3(x1, x2, x3) << 7) ^ HASH1(x4))


// The following macros are used to implement instruction-specific hashing.
// By default, each instruction implements hash() and is_equal(Value), used
// for value numbering/common subexpression elimination. The default imple-
// mentation disables value numbering. Each instruction which can be value-
// numbered, should define corresponding hash() and is_equal(Value) functions
// via the macros below. The f arguments specify all the values/op codes, etc.
// that need to be identical for two instructions to be identical.
//
// Note: The default implementation of hash() returns 0 in order to indicate
//       that the instruction should not be considered for value numbering.
//       The currently used hash functions do not guarantee that never a 0
//       is produced. While this is still correct, it may be a performance
//       bug (no value numbering for that node). However, this situation is
//       so unlikely, that we are not going to handle it specially. 

#define HASHING1(class_name, enabled, f1)             \
  virtual int hash() const {                          \
    return (enabled) ? HASH2(name(), f1) : 0;         \
  }                                                   \
  virtual bool is_equal(Value v) const {              \
    if (!(enabled)  ) return false;                   \
    class_name* _v = v->as_##class_name();            \
    if (_v == NULL  ) return false;                   \
    if (f1 != _v->f1) return false;                   \
    return true;                                      \
  }                                                   \


#define HASHING2(class_name, enabled, f1, f2)         \
  virtual int hash() const {                          \
    return (enabled) ? HASH3(name(), f1, f2) : 0;     \
  }                                                   \
  virtual bool is_equal(Value v) const {              \
    if (!(enabled)  ) return false;                   \
    class_name* _v = v->as_##class_name();            \
    if (_v == NULL  ) return false;                   \
    if (f1 != _v->f1) return false;                   \
    if (f2 != _v->f2) return false;                   \
    return true;                                      \
  }                                                   \


#define HASHING3(class_name, enabled, f1, f2, f3)     \
  virtual int hash() const {                          \
    return (enabled) ? HASH4(name(), f1, f2, f3) : 0; \
  }                                                   \
  virtual bool is_equal(Value v) const {              \
    if (!(enabled)  ) return false;                   \
    class_name* _v = v->as_##class_name();            \
    if (_v == NULL  ) return false;                   \
    if (f1 != _v->f1) return false;                   \
    if (f2 != _v->f2) return false;                   \
    if (f3 != _v->f3) return false;                   \
    return true;                                      \
  }                                                   \


// The mother of all instructions...

class Instruction: public ResourceObj {
 private:
  static int   _next_id;                         // the node counter

  int          _id;                              // the unique instruction id
  int          _bci;                             // the instruction bci
  int          _use_count;                       // the number of instructions refering to this value (w/o prev/next); only roots can have use count = 0 or > 1
  bool         _is_pinned;                       // true if the instruction is pinned
  ValueType*   _type;                            // the instruction value type
  Instruction* _next;                            // the next instruction if any (NULL for BlockEnd instructions)
  Instruction* _subst;                           // the substitution instruction if any
  Item*        _item;                            // the machine specific information

  friend class UseCountComputer;
  friend class Inliner;

 protected:
  void set_bci(int bci)                          { assert(bci == SynchronizationEntryBCI || bci >= 0, "illegal bci"); _bci = bci; }

 public:
  // 'globally' used condition values
  enum Condition {
    eql, neq, lss, leq, gtr, geq
  };

  static Condition mirror(Condition cond);
  static Condition negate(Condition cond);

  // initialization
  static void initialize()                       { _next_id = 0; }
  static int number_of_instructions()            { return _next_id; }

  // creation
  Instruction(ValueType* type)
  : _id(_next_id++)
  , _bci(-1)
  , _use_count(0)
  , _is_pinned(false)
  , _type(type)
  , _next(NULL)
  , _subst(NULL)
  , _item(NULL) 
  {
    assert(type != NULL, "type must exist");
  }

  // accessors
  int id() const                                 { return _id; }
  int bci() const                                { return _bci; }
  int use_count() const                          { return _use_count; }
  bool is_pinned() const                         { return _is_pinned || PinAllInstructions; }
  ValueType* type() const                        { return _type; }
  Instruction* prev(BlockBegin* block);          // use carefully, expensive operation
  Instruction* next() const                      { return _next; }
  Instruction* subst()                           { return _subst == NULL ? this : _subst->subst(); }
  Item* item() const                             { return _item; }

  bool has_uses() const                          { return use_count() > 0; }
  bool is_root() const                           { return is_pinned() || use_count() > 1; }

  // manipulation
  void pin()                                     { _is_pinned = true; }

  Instruction* set_next(Instruction* next, int bci) {
    if (next != NULL) {
      assert(as_BlockEnd() == NULL, "BlockEnd instructions must have no next");
      next->set_bci(bci);
    }
    _next = next;
    return next;
  }

  void set_subst(Instruction* subst)             { _subst = subst; }

  // machine-specifics
  void set_item(Item* item)                      { assert(item != NULL, "item must exist"); _item = item; }
  void clear_item()                              { _item = NULL; }

  // generic
  virtual Instruction*      as_Instruction()     { return this; } // to satisfy HASHING1 macro
  virtual HiWord*           as_HiWord()          { return NULL; }
  virtual Phi*              as_Phi()             { return NULL; }
  virtual Constant*         as_Constant()        { return NULL; }
  virtual AccessLocal*      as_AccessLocal()     { return NULL; }
  virtual LoadLocal*        as_LoadLocal()       { return NULL; }
  virtual StoreLocal*       as_StoreLocal()      { return NULL; }
  virtual AccessField*      as_AccessField()     { return NULL; }
  virtual LoadField*        as_LoadField()       { return NULL; }
  virtual StoreField*       as_StoreField()      { return NULL; }
  virtual ArrayLength*      as_ArrayLength()     { return NULL; }
  virtual AccessIndexed*    as_AccessIndexed()   { return NULL; }
  virtual LoadIndexed*      as_LoadIndexed()     { return NULL; }
  virtual StoreIndexed*     as_StoreIndexed()    { return NULL; }
  virtual LoopEnter*        as_LoopEnter()       { return NULL; }
  virtual LoopExit*         as_LoopExit()        { return NULL; }
  virtual NegateOp*         as_NegateOp()        { return NULL; }
  virtual Op2*              as_Op2()             { return NULL; }
  virtual ArithmeticOp*     as_ArithmeticOp()    { return NULL; }
  virtual ShiftOp*          as_ShiftOp()         { return NULL; }
  virtual LogicOp*          as_LogicOp()         { return NULL; }
  virtual CompareOp*        as_CompareOp()       { return NULL; }
  virtual IfOp*             as_IfOp()            { return NULL; }
  virtual Convert*          as_Convert()         { return NULL; }
  virtual NullCheck*        as_NullCheck()       { return NULL; }
  virtual JsrContinuation*  as_JsrContinuation() { return NULL; }
  virtual StateSplit*       as_StateSplit()      { return NULL; }
  virtual Jsr*              as_Jsr()             { return NULL; }
  virtual Invoke*           as_Invoke()          { return NULL; }
  virtual NewInstance*      as_NewInstance()     { return NULL; }
  virtual NewArray*         as_NewArray()        { return NULL; }
  virtual NewTypeArray*     as_NewTypeArray()    { return NULL; }
  virtual NewObjectArray*   as_NewObjectArray()  { return NULL; }
  virtual NewMultiArray*    as_NewMultiArray()   { return NULL; }
  virtual TypeCheck*        as_TypeCheck()       { return NULL; }
  virtual CheckCast*        as_CheckCast()       { return NULL; }
  virtual InstanceOf*       as_InstanceOf()      { return NULL; }
  virtual AccessMonitor*    as_AccessMonitor()   { return NULL; }
  virtual MonitorEnter*     as_MonitorEnter()    { return NULL; }
  virtual MonitorExit*      as_MonitorExit()     { return NULL; }
  virtual Intrinsic*        as_Intrinsic()       { return NULL; }
  virtual BlockBegin*       as_BlockBegin()      { return NULL; }
  virtual BlockEnd*         as_BlockEnd()        { return NULL; }
  virtual Goto*             as_Goto()            { return NULL; }
  virtual If*               as_If()              { return NULL; }
  virtual IfInstanceOf*     as_IfInstanceOf()    { return NULL; }
  virtual TableSwitch*      as_TableSwitch()     { return NULL; }
  virtual LookupSwitch*     as_LookupSwitch()    { return NULL; }
  virtual Ret*              as_Ret()             { return NULL; }
  virtual Return*           as_Return()          { return NULL; }
  virtual Throw*            as_Throw()           { return NULL; }
  virtual Base*             as_Base()            { return NULL; }

  virtual void visit(InstructionVisitor* v)      = 0;

  virtual bool can_trap() const                  { return false; }

  virtual void input_values_do(void f(Value*))   = 0;
  virtual void state_values_do(void f(Value*))   { /* usually no state - override on demand */ }
          void       values_do(void f(Value*))   { input_values_do(f); state_values_do(f); }

  // hashing
  virtual const char* name() const               = 0;
  HASHING1(Instruction, false, id())             // hashing disabled by default

  // debugging
  void print()                                   PRODUCT_RETURN;
  void print(InstructionPrinter& ip)             PRODUCT_RETURN;
};


// The following macros are used to define base (i.e., non-leaf)
// and leaf instruction classes. They define class-name related
// generic functionality in one place.

#define BASE(class_name, super_class_name)       \
  class class_name: public super_class_name {    \
   public:                                       \
    virtual class_name* as_##class_name()        { return this; }              \


#define LEAF(class_name, super_class_name)       \
  BASE(class_name, super_class_name)             \
   public:                                       \
    virtual const char* name() const             { return #class_name; }       \
    virtual void visit(InstructionVisitor* v)    { v->do_##class_name(this); } \


// Debugging support

#ifdef ASSERT
  static void assert_value(Value* x)             { assert((*x) != NULL, "value must exist"); }
  #define ASSERT_VALUES                          values_do(assert_value);
#else
  #define ASSERT_VALUES
#endif


// A HiWord occupies the 'high word' of a 2-word
// expression stack entry. Hi & lo words must be
// paired on the expression stack (otherwise the
// bytecode sequence is illegal). Note that 'hi'
// refers to the IR expression stack format and
// does *not* imply a machine word ordering.

LEAF(HiWord, Instruction)
 private:
  Value _lo_word;

 public:
  // creation
  HiWord(Value lo_word) : Instruction(lo_word->type()), _lo_word(lo_word) {
    assert(lo_word->type()->is_double_word(), "HiWord must be used for 2-word values only");
  }

  // accessors
  Value lo_word() const                          { return _lo_word; }

  // generic
  virtual void input_values_do(void f(Value*))   { ShouldNotReachHere(); }
};


// A Phi instruction is a placeholder for a value
// coming in from one or more predecessor blocks.
// Phi instructions can only appear in the state
// array of a BlockBegin. Currently Phis are used
// only for stack values.

LEAF(Phi, Instruction)
 private:
  int    _index;
  int    _stack_size;

 public:
  // creation
  Phi(ValueType* type, int index, int stack_size) : Instruction(type->base()), _index(index), _stack_size(stack_size) {
    assert(stack_size > 0, "illegal stack size");
    assert(0 <= index && index < stack_size, "illegal index");
    // Note: If the type of a phi is a constant type, the phi's constant
    //       value may be used by the canonicalizer. However, currently,
    //       we do not correctly compute the merged phi type, thus we
    //       always use the non-constant base type for phis for now -
    //       was bug - gri 6/23/99.
    pin();
  }

  // accessors
  int index() const                              { return _index; }
  int stack_size() const                         { return _stack_size; }

  // generic
  virtual void input_values_do(void f(Value*))   { /* no values */ }
  HASHING1(Phi, true, index());
};


LEAF(Constant, Instruction)
 public:
  // creation
  Constant(ValueType* type) : Instruction(type) {
    assert(type->is_constant(), "must be a constant");
  }

  // generic
  virtual void input_values_do(void f(Value*))   { /* no values */ }
  HASHING1(Constant, type()->as_IntConstant() != NULL, type()->as_IntConstant()->value())
};


BASE(AccessLocal, Instruction)
 private:
  int   _index;
  bool  _pinned_by_store;

 public:
  // creation
  AccessLocal(ValueType* type, int index) : Instruction(type), _index(index), _pinned_by_store(false) {}

  // accessors
  int index() const                              { return _index; }

  void set_pinned_by_store()                     { _pinned_by_store = true; }
  // note: returns true for now due to bug in setup - this is a conservative
  //       solution that always works (comes with a minor code quality degradation)
  bool is_pinned_by_store() const                { return true; /* _pinned_by_store */ ; }

  // generic
  virtual void input_values_do(void f(Value*))   { /* no values */ }
};


LEAF(LoadLocal, AccessLocal)
 public:
  // creation
  LoadLocal(ValueType* type, int index) : AccessLocal(type, index) {
    assert(!type->is_constant(), "must not be constant type");
  }

  // Note: LoadLocal is currently not used with value numbering as
  //       load elimination is taking care of it - eventually, value
  //       numbering could do this as well.
};


LEAF(StoreLocal, AccessLocal)
 private:
  Value _value;

 public:
  // creation
  StoreLocal(ValueType* type, int index, Value value) : AccessLocal(type, index), _value(value) {
    ASSERT_VALUES
    pin();
  }

  // accessors
  Value value() const                            { return _value; }

  // generic
  virtual void input_values_do(void f(Value*))   { AccessLocal::input_values_do(f); f(&_value); }
};


BASE(AccessField, Instruction)
 private:
  Value     _obj;
  int       _offset;
  BasicType _field_type;
  bool      _is_static;
  bool      _is_loaded;
  bool      _is_initialized;

 public:
  // creation
  AccessField(Value obj, int offset, BasicType field_type, bool is_static, bool is_loaded, bool is_initialized)
  : Instruction(as_ValueType(field_type))
  , _obj(obj)
  , _offset(offset)
  , _field_type(field_type)
  , _is_static(is_static)
  , _is_loaded(is_loaded)
  , _is_initialized(is_initialized)
  {
    ASSERT_VALUES
    if (!is_loaded || !is_initialized) pin(); // in this case we have patching at runtime => needs pinning
  }

  // accessors
  Value obj() const                              { return _obj; }
  int offset() const                             { return _offset; }
  BasicType field_type() const                   { return _field_type; }
  bool is_static() const                         { return _is_static; }
  bool is_loaded() const                         { return _is_loaded; }
  bool is_initialized() const                    { return _is_initialized; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { f(&_obj); }
};


LEAF(LoadField, AccessField)
 public:
  // creation
  LoadField(Value obj, int offset, BasicType field_type, bool is_static, bool is_loaded, bool is_initialized)
  : AccessField(obj, offset, field_type, is_static, is_loaded, is_initialized)
  {}

  // generic
  HASHING2(LoadField, is_loaded(), obj(), offset())  // cannot be eliminated if not yet loaded
};


LEAF(StoreField, AccessField)
 private:
  Value _value;
  bool  _needs_write_barrier;

 public:
  // creation
  StoreField(Value obj, int offset, BasicType field_type, Value value, bool is_static, bool is_loaded, bool is_initialized)
  : AccessField(obj, offset, field_type, is_static, is_loaded, is_initialized)
  , _value(value)
  , _needs_write_barrier(as_ValueType(field_type)->is_object())
  {
    ASSERT_VALUES
    pin();
  }

  // accessors
  Value value() const                            { return _value; }
  bool needs_write_barrier() const               { return _needs_write_barrier; }

  // generic
  virtual void input_values_do(void f(Value*))   { AccessField::input_values_do(f); f(&_value); }
};


BASE(AccessArray, Instruction)
 private:
  Value _array;

 public:
  // creation
  AccessArray(ValueType* type, Value array) : Instruction(type), _array(array) {
    ASSERT_VALUES
  }

  // accessors
  Value array() const                            { return _array; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { f(&_array); }
};


LEAF(ArrayLength, AccessArray)
 public:
  // creation
  ArrayLength(Value array) : AccessArray(intType, array) {}

  // generic
  HASHING1(ArrayLength, true, array())
};


BASE(AccessIndexed, AccessArray)
 private:
  Value     _index;
  BasicType _elt_type;
  bool      _needs_range_check;

 public:
  // creation
  AccessIndexed(Value array, Value index, BasicType elt_type)
  : AccessArray(as_ValueType(elt_type), array)
  , _index(index)
  , _elt_type(elt_type)
  , _needs_range_check(true)
  {
    ASSERT_VALUES
  }

  // accessors
  Value index() const                            { return _index; }
  BasicType elt_type() const                     { return _elt_type; }
  bool needs_range_check() const                 { return _needs_range_check; }

  // generic
  virtual void input_values_do(void f(Value*))   { AccessArray::input_values_do(f); f(&_index); }
};


LEAF(LoadIndexed, AccessIndexed)
 public:
  // creation
  LoadIndexed(Value array, Value index, BasicType elt_type) : AccessIndexed(array, index, elt_type) {}

  // generic
  HASHING2(LoadIndexed, true, array(), index())
};


LEAF(StoreIndexed, AccessIndexed)
 private:
  Value       _value;
  ValueStack* _state;
  bool        _needs_write_barrier;
  bool        _needs_store_check;

 public:
  // creation
  StoreIndexed(Value array, Value index, BasicType elt_type, Value value)
  : AccessIndexed(array, index, elt_type)
  , _value(value)
  , _state(NULL)
  , _needs_write_barrier(as_ValueType(elt_type)->is_object())
  , _needs_store_check  (as_ValueType(elt_type)->is_object())
  {
    ASSERT_VALUES
    pin();
  }

  // accessors
  Value value() const                            { return _value; }
  ValueStack* stack() const                      { return _state; }
  ValueStack* state() const                      { return _state; }
  bool needs_write_barrier() const               { return _needs_write_barrier; }
  bool needs_store_check() const                 { return _needs_store_check; }

  // manipulation
  void set_state(ValueStack* state)              { _state = state; }

  // generic
  virtual void input_values_do(void f(Value*))   { AccessIndexed::input_values_do(f); f(&_value); }
  virtual void state_values_do(void f(Value*));
};


BASE(AccessLoop, Instruction)
 private:
  int _loop_id;

 public:
  // creation
  AccessLoop(int loop_id) : Instruction(illegalType), _loop_id(loop_id) {
    assert(loop_id >= 0, "illegal loop id");
    pin();
  }

  // accessors
  int loop_id() const                            { return _loop_id; }

  // generic
  virtual void input_values_do(void f(Value*))   { /* no values */ }
};


LEAF(LoopEnter, AccessLoop)
 public:
  // creation
  LoopEnter(int loop_id) : AccessLoop(loop_id) {}
};


LEAF(LoopExit, AccessLoop)
 public:
  // creation
  LoopExit(int loop_id) : AccessLoop(loop_id) {}
};


LEAF(NegateOp, Instruction)
 private:
  Value _x;

 public:
  // creation
  NegateOp(Value x) : Instruction(x->type()->base()), _x(x) {
    ASSERT_VALUES
  }

  // accessors
  Value x() const                                { return _x; }

  // generic
  virtual void input_values_do(void f(Value*))   { f(&_x); }
};


BASE(Op2, Instruction)
 private:
  Bytecodes::Code _op;
  Value           _x;
  Value           _y;

 public:
  // creation
  Op2(ValueType* type, Bytecodes::Code op, Value x, Value y) : Instruction(type), _op(op), _x(x), _y(y) {
    ASSERT_VALUES
  }

  // accessors
  Bytecodes::Code op() const                     { return _op; }
  Value x() const                                { return _x; }
  Value y() const                                { return _y; }

  // manipulators
  void swap_operands() {
    assert(is_commutative(), "operation must be commutative");
    Value t = _x; _x = _y; _y = t;
  }

  // generic
  virtual bool is_commutative() const            { return false; }
  virtual void input_values_do(void f(Value*))   { f(&_x); f(&_y); }
};


LEAF(ArithmeticOp, Op2)
 public:
  // creation
  ArithmeticOp(Bytecodes::Code op, Value x, Value y) : Op2(x->type()->meet(y->type()), op, x, y) {
    if (can_trap()) pin();
  }

  // generic
  virtual bool is_commutative() const;
  virtual bool can_trap() const;
  HASHING3(Op2, true, op(), x(), y())
};


LEAF(ShiftOp, Op2)
 public:
  // creation
  ShiftOp(Bytecodes::Code op, Value x, Value s) : Op2(x->type()->base(), op, x, s) {}

  // generic
  HASHING3(Op2, true, op(), x(), y())
};


LEAF(LogicOp, Op2)
 public:
  // creation
  LogicOp(Bytecodes::Code op, Value x, Value y) : Op2(x->type()->meet(y->type()), op, x, y) {}

  // generic
  virtual bool is_commutative() const;
  HASHING3(Op2, true, op(), x(), y())
};


LEAF(CompareOp, Op2)
 public:
  // creation
  CompareOp(Bytecodes::Code op, Value x, Value y) : Op2(intType, op, x, y) {}

  // generic
  HASHING3(Op2, true, op(), x(), y())
};


LEAF(IfOp, Op2)
 private:
  Value _tval;
  Value _fval;

 public:
  // creation
  IfOp(Value x, Condition cond, Value y, Value tval, Value fval)
  : Op2(tval->type()->meet(fval->type()), (Bytecodes::Code)cond, x, y)
  , _tval(tval)
  , _fval(fval)
  {
    ASSERT_VALUES
    assert(tval->type()->tag() == fval->type()->tag(), "types must match");
  }

  // accessors
  virtual bool is_commutative() const;
  Bytecodes::Code op() const                     { ShouldNotCallThis(); return Bytecodes::_illegal; }
  Condition cond() const                         { return (Condition)Op2::op(); }
  Value tval() const                             { return _tval; }
  Value fval() const                             { return _fval; }

  // generic  
  virtual void input_values_do(void f(Value*))   { Op2::input_values_do(f); f(&_tval); f(&_fval); }
};


LEAF(Convert, Instruction)
 private:
  Bytecodes::Code _op;
  Value           _value;

 public:
  // creation
  Convert(Bytecodes::Code op, Value value, ValueType* to_type) : Instruction(to_type), _op(op), _value(value) {
    ASSERT_VALUES
  }

  // accessors
  Bytecodes::Code op() const                     { return _op; }
  Value value() const                            { return _value; }

  // generic
  virtual void input_values_do(void f(Value*))   { f(&_value); }
  HASHING2(Convert, true, op(), value())
};


LEAF(NullCheck, Instruction)
 private:
  Value _obj;

 public:
  // creation
  NullCheck(Value obj) : Instruction(obj->type()), _obj(obj) {
    ASSERT_VALUES
    assert(_obj->type()->is_object(), "null check must be applied to objects only");
    pin();
  }

  // accessors
  Value obj() const                              { return _obj; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { f(&_obj); }
};


LEAF(JsrContinuation, Instruction)
 public:
  // creation
  JsrContinuation() : Instruction(illegalType) {
    pin();
  }

  // generic
  virtual void input_values_do(void f(Value*))   { /* no values */ }
};


BASE(StateSplit, Instruction)
 private:
  ValueStack* _state;

 public:
  // creation
  StateSplit(ValueType* type) : Instruction(type), _state(NULL) {
    pin();
  }

  // accessors
  ValueStack* stack() const                      { return _state; }
  ValueStack* state() const                      { return _state; }

  // manipulation
  void set_state(ValueStack* state)              { _state = state; }

  // generic
  virtual void input_values_do(void f(Value*))   { /* no values */ }
  virtual void state_values_do(void f(Value*));
};


LEAF(Jsr, StateSplit)
 private:
  BlockBegin* _subroutine;

 public:
  // creation
  Jsr(BlockBegin* subroutine) : StateSplit(illegalType), _subroutine(subroutine) {
    assert(subroutine != NULL, "subroutine must exist");
  }

  // accessors
  BlockBegin* subroutine() const                 { return _subroutine; }
};


LEAF(Invoke, StateSplit)
 private:
  Bytecodes::Code           _code;
  Value                     _recv;
  Values*                   _args;
  bool                      _target_is_final;
  bool                      _target_is_loaded;

 public:
  // creation
  Invoke(Bytecodes::Code code, ValueType* result_type, Value recv, Values* args,
	 bool target_is_final, bool target_is_loaded)
  : StateSplit(result_type)
  , _code(code)
  , _recv(recv)
  , _args(args)
  , _target_is_final(target_is_final)
  , _target_is_loaded(target_is_loaded)
  {
    assert(args != NULL, "args must exist");
    ASSERT_VALUES
  }

  // accessors
  Bytecodes::Code code() const                   { return _code; }
  Value receiver() const                         { return _recv; }
  bool has_receiver() const                      { return receiver() != NULL; }
  int number_of_arguments() const                { return _args->length(); }
  int size_of_arguments() const;
  Value argument_at(int i) const                 { return _args->at(i); }
  GrowableArray<BasicType>* signature() const;

  bool target_is_final() const                   { return _target_is_final; }
  bool target_is_loaded() const                  { return _target_is_loaded; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*)) {
    StateSplit::input_values_do(f);
    if (has_receiver()) f(&_recv);
    for (int i = 0; i < _args->length(); i++) f(_args->adr_at(i));
  } 
};


LEAF(NewInstance, StateSplit)
 private:
  ciInstanceKlass* _klass;

 public:
  // creation
  NewInstance(ciInstanceKlass* klass) : StateSplit(instanceType), _klass(klass) {}

  // accessors
  ciInstanceKlass* klass() const                 { return _klass; }

  // generic
  virtual bool can_trap() const                  { return true; }
};


BASE(NewArray, StateSplit)
 private:
  Value _length;

 public:
  // creation
  NewArray(Value length) : StateSplit(objectType), _length(length) {
    ASSERT_VALUES
  }

  // accessors
  Value length() const                           { return _length; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { StateSplit::input_values_do(f); f(&_length); }
};


LEAF(NewTypeArray, NewArray)
 private:
  BasicType _elt_type;

 public:
  // creation
  NewTypeArray(Value length, BasicType elt_type) : NewArray(length), _elt_type(elt_type) {}

  // accessors
  BasicType elt_type() const                     { return _elt_type; }
};


LEAF(NewObjectArray, NewArray)
 private:
  ciKlass* _klass;

 public:
  // creation
  NewObjectArray(ciKlass* klass, Value length) : NewArray(length), _klass(klass) {}

  // accessors
  ciKlass* klass() const                         { return _klass; }
};


LEAF(NewMultiArray, NewArray)
 private:
  ciKlass* _klass;
  Values*  _dims;

 public:
  // creation
  NewMultiArray(ciKlass* klass, Values* dims) : NewArray(dims->at(0)), _klass(klass), _dims(dims) {
    ASSERT_VALUES
  }

  // accessors
  ciKlass* klass() const                         { return _klass; }
  Values* dims() const                           { return _dims; }
  int rank() const                               { return dims()->length(); }

  // generic
  virtual void input_values_do(void f(Value*)) {
    NewArray::input_values_do(f);
    // Note: do not visit dims->at(0) again, since it is visited by the superclass (length)!
    for (int i = 1; i < _dims->length(); i++) f(_dims->adr_at(i));    
  }
};


BASE(TypeCheck, StateSplit)
 private:
  ciKlass* _klass;
  Value    _obj;

 public:
  // creation
  TypeCheck(ciKlass* klass, Value obj, ValueType* type) : StateSplit(type), _klass(klass), _obj(obj) {
    ASSERT_VALUES
  }

  // accessors
  ciKlass* klass() const                         { return _klass; }
  Value obj() const                              { return _obj; }
  bool is_loaded() const                         { return klass() != NULL; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { StateSplit::input_values_do(f); f(&_obj); }
};


LEAF(CheckCast, TypeCheck)
 public:
  // creation
  CheckCast(ciKlass* klass, Value obj) : TypeCheck(klass, obj, objectType) {}
};


LEAF(InstanceOf, TypeCheck)
 public:
  // creation
  InstanceOf(ciKlass* klass, Value obj) : TypeCheck(klass, obj, intType) {}
};


BASE(AccessMonitor, StateSplit)
 private:
  Value _obj;
  int   _monitor_no;

 public:
  // creation
  AccessMonitor(Value obj, int monitor_no) : StateSplit(illegalType), _obj(obj), _monitor_no(monitor_no) {
    ASSERT_VALUES
  }

  // accessors
  Value obj() const                              { return _obj; }
  int monitor_no() const                         { return _monitor_no; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { StateSplit::input_values_do(f); f(&_obj); }
};


LEAF(MonitorEnter, AccessMonitor)
 public:
  // creation
  MonitorEnter(Value obj, int monitor_no) : AccessMonitor(obj, monitor_no) {}
};


LEAF(MonitorExit, AccessMonitor)
 public:
  // creation
  MonitorExit(Value obj, int monitor_no) : AccessMonitor(obj, monitor_no) {}
};


LEAF(Intrinsic, StateSplit)
 private:
  ciMethod::IntrinsicId _id;
  Values*               _args;

 public:
  Intrinsic(ValueType* type, ciMethod::IntrinsicId id, Values* args) : StateSplit(type), _id(id), _args(args) {
    assert(args != NULL, "args must exist");
    ASSERT_VALUES
  }

  // accessors
  ciMethod::IntrinsicId id() const               { return _id; }
  int number_of_arguments() const                { return _args->length(); }
  Value argument_at(int i) const                 { return _args->at(i); }

  // generic
  virtual void input_values_do(void f(Value*)) {
    StateSplit::input_values_do(f);
    for (int i = 0; i < _args->length(); i++) f(_args->adr_at(i));
  }
};


// BlockClosure is the base class for block traversal/iteration.

class BlockClosure: public ResourceObj {
 public:
  virtual void block_do(BlockBegin* block)       = 0;
};


LEAF(BlockBegin, StateSplit)
 private:
  static int _next_block_id;                     // the block counter

  int        _block_id;                          // the unique block id
  int        _weight;                            // the block weight, used for block ordering
  int        _flags;                             // the flags associated with this block
  IRScope*   _scope;                             // the scope to which this block belongs
  BlockEnd*  _end;                               // the last instruction of this block
  BlockList  _subroutines;                       // the subroutines called by this block
  BlockList  _exception_handlers;                // the exception handlers potentially invoked by this block
  BlockItem* _block_item;                        // the machine-specific information for this block
  Label      _label;                             // the label associated with this block

  void collect_ends(GrowableArray<bool>& mark, EndList* endlist, bool& bailout) const;
  void iterate_preorder (GrowableArray<bool>& mark, BlockClosure* closure);
  void iterate_postorder(GrowableArray<bool>& mark, BlockClosure* closure);

  friend class SuxAndWeightAdjuster;

 public:
  // initialization/counting
  static void initialize()                       { _next_block_id = 0; }
  static int  number_of_blocks()                 { return _next_block_id; }

  // creation
  BlockBegin(IRScope* scope, int bci)
  : StateSplit(illegalType)
  , _block_id(_next_block_id++)
  , _weight(0)
  , _flags(0)
  , _scope(scope)
  , _end(NULL)
  , _subroutines(1)
  , _exception_handlers(1)
  , _block_item(NULL)
  {
    assert(scope != NULL, "scope must exist");
    set_bci(bci);
  }

  // accessors
  int block_id() const                           { return _block_id; }
  int weight() const                             { return _weight; }
  IRScope* scope() const                         { return _scope; }
  BlockEnd* end() const                          { return _end; }
  BlockItem* block_item() const                  { return _block_item; }
  Label* label()                                 { return &_label; }
  EndList* endlist() const;                      // the BlockEnds reachable from this block, NULL for bailout

  // manipulation
  void set_weight(int start_distance);
  void set_end(BlockEnd* end)                    { _end = end; }
  void set_block_item(BlockItem* block_item)     { _block_item = block_item; }

  // subroutines called by this block
  void add_subroutine(BlockBegin* b);
  int number_of_subroutines() const              { return _subroutines.length(); }
  BlockBegin* subroutine_at(int i) const         { return _subroutines.at(i); }

  // exception handlers potentially invoked by this block
  void add_exception_handler(BlockBegin* b);
  int number_of_exception_handlers() const       { return _exception_handlers.length(); }
  BlockBegin* exception_handler_at(int i) const  { return _exception_handlers.at(i); }

  // flags
  enum Flag {
    no_flag                     = 0,
    std_entry_flag              = 1 << 0,
    osr_entry_flag              = 1 << 1,
    exception_entry_flag        = 1 << 2,
    subroutine_entry_flag       = 1 << 3,
    backward_branch_target_flag = 1 << 4,
    was_visited_flag            = 1 << 5
  };

  void set(Flag f)                               { _flags |= f; }
  void clear(Flag f)                             { _flags &= ~f; }
  bool is_set(Flag f) const                      { return (_flags & f) != 0; }
  bool is_entry_block() const {
    const int entry_mask = std_entry_flag | osr_entry_flag | exception_entry_flag | subroutine_entry_flag;
    return (_flags & entry_mask) == 0;
  }

  // block ordering in final block list (before code generation)
  bool is_after_block(BlockBegin* b)             { return weight() >= b->weight(); }

  // iteration
  void iterate_preorder   (BlockClosure* closure);
  void iterate_postorder  (BlockClosure* closure);
  void iterate_topological(BlockClosure* closure);

  void block_values_do(void f(Value*));

  // merging
  bool try_join(ValueStack* s);                  // same as join, but returns false if join fails
  void join(ValueStack* s)                       { bool b = try_join(s); assert(b, "join failed"); }
  void resolve_substitution();                   // resolve value references to substituted values
                                                 // for all instructions in this block

  // debugging
  void print_block()                             PRODUCT_RETURN;
  void print_block(InstructionPrinter& ip)       PRODUCT_RETURN;
};


// BlockCollection can be used to collect blocks using any traversal algorithm.

class BlockCollection: public BlockClosure {
 private:
  BlockList* _collection;

 public:
  BlockCollection(int n)                         { _collection = new GrowableArray<BlockBegin*>(n); }
  BlockList* collection() const                  { return _collection; }
  virtual void block_do(BlockBegin* block)       { _collection->append(block); }

  // iteration
  void iterate_forward (BlockClosure* closure);
  void iterate_backward(BlockClosure* closure);

  void blocks_do(void f(BlockBegin*));
  void values_do(void f(Value*));

  // printing
  void print(bool cfg_only)                      PRODUCT_RETURN;
};


BASE(BlockEnd, StateSplit)
 private:
  BlockList* _sux;

 protected:
  BlockList* sux() const                         { return _sux; }

  void set_sux(BlockList* sux) {
#ifdef ASSERT
    assert(sux != NULL, "sux must exist");
    for (int i = sux->length() - 1; i >= 0; i--) assert(sux->at(i) != NULL, "sux must exist");
#endif
    _sux = sux;
  }

 public:
  // creation
  BlockEnd(ValueType* type) : StateSplit(type), _sux(NULL) {}

  // successors
  int number_of_sux() const                      { return _sux != NULL ? _sux->length() : 0; }
  BlockBegin* sux_at(int i) const                { return _sux->at(i); }
  BlockBegin* default_sux() const                { return sux_at(number_of_sux() - 1); }
  BlockBegin** addr_sux_at(int i) const          { return _sux->adr_at(i); }
  int sux_index(BlockBegin* sux) const           { return _sux->find(sux); }
  void substitute_sux(BlockBegin* old_sux, BlockBegin* new_sux);
};


LEAF(Goto, BlockEnd)
 public:
  // creation
  Goto(BlockBegin* sux) : BlockEnd(illegalType) {
    BlockList* s = new GrowableArray<BlockBegin*>(1);
    s->append(sux);
    set_sux(s);
  }
};


LEAF(If, BlockEnd)
 private:
  Value       _x;
  Condition   _cond;
  bool        _unordered_is_true;                // valid for float/double compares only
  Value       _y;

 public:
  // creation
  If(Value x, Condition cond, bool unordered_is_true, Value y, BlockBegin* tsux, BlockBegin* fsux)
  : BlockEnd(illegalType)
  , _x(x)
  , _cond(cond)
  , _unordered_is_true(unordered_is_true)
  , _y(y)
  {
    ASSERT_VALUES
    assert(x->type()->tag() == y->type()->tag(), "types must match");
    BlockList* s = new GrowableArray<BlockBegin*>(2);
    s->append(tsux);
    s->append(fsux);
    set_sux(s);
  }

  // accessors
  Value x() const                                { return _x; }
  Condition cond() const                         { return _cond; }
  bool unordered_is_true() const                 { return _unordered_is_true; }
  Value y() const                                { return _y; }
  BlockBegin* sux_for(bool is_true) const        { return sux_at(is_true ? 0 : 1); }
  BlockBegin* tsux() const                       { return sux_for(true); }
  BlockBegin* fsux() const                       { return sux_for(false); }
  BlockBegin* usux() const                       { return sux_for(unordered_is_true()); }

  // manipulation
  void swap_operands() {
    Value t = _x; _x = _y; _y = t;
    _cond = mirror(_cond);
  }

  void swap_sux() {
    assert(number_of_sux() == 2, "wrong number of successors");
    BlockList* s = sux();
    BlockBegin* t = s->at(0); s->at_put(0, s->at(1)); s->at_put(1, t);
    _cond = negate(_cond);
    _unordered_is_true = !_unordered_is_true;
  }
    
  // generic  
  virtual void input_values_do(void f(Value*))   { BlockEnd::input_values_do(f); f(&_x); f(&_y); }
};


LEAF(IfInstanceOf, BlockEnd)
 private:
  ciKlass* _klass;
  Value    _obj;
  bool     _test_is_instance;                    // jump if instance
  int      _instanceof_bci;

 public:
  IfInstanceOf(ciKlass* klass, Value obj, bool test_is_instance, int instanceof_bci, BlockBegin* tsux, BlockBegin* fsux)
  : BlockEnd(illegalType)
  , _klass(klass)
  , _obj(obj)
  , _test_is_instance(test_is_instance)
  , _instanceof_bci(instanceof_bci)
  {
    ASSERT_VALUES
    assert(instanceof_bci >= 0, "illegal bci");
    BlockList* s = new GrowableArray<BlockBegin*>(2);
    s->append(tsux);
    s->append(fsux);
    set_sux(s);
  }

  // accessors
  //
  // Note 1: If test_is_instance() is true, IfInstanceOf tests if obj *is* an
  //         instance of klass; otherwise it tests if it is *not* and instance
  //         of klass.
  //
  // Note 2: IfInstanceOf instructions are created by combining an InstanceOf
  //         and an If instruction. The IfInstanceOf bci() corresponds to the
  //         bci that the If would have had; the (this->) instanceof_bci() is
  //         the bci of the original InstanceOf instruction.
  ciKlass* klass() const                         { return _klass; }
  Value obj() const                              { return _obj; }
  int instanceof_bci() const                     { return _instanceof_bci; }
  bool test_is_instance() const                  { return _test_is_instance; }
  BlockBegin* sux_for(bool is_true) const        { return sux_at(is_true ? 0 : 1); }
  BlockBegin* tsux() const                       { return sux_for(true); }
  BlockBegin* fsux() const                       { return sux_for(false); }

  // manipulation
  void swap_sux() {
    assert(number_of_sux() == 2, "wrong number of successors");
    BlockList* s = sux();
    BlockBegin* t = s->at(0); s->at_put(0, s->at(1)); s->at_put(1, t);
    _test_is_instance = !_test_is_instance;
  }
    
  // generic  
  virtual void input_values_do(void f(Value*))   { BlockEnd::input_values_do(f); f(&_obj); }
};


BASE(Switch, BlockEnd)
 private:
  Value _tag;

 public:
  // creation
  Switch(Value tag, BlockList* sux) : BlockEnd(illegalType), _tag(tag) {
    ASSERT_VALUES
    set_sux(sux);
  }

  // accessors
  Value tag() const                              { return _tag; }
  int length() const                             { return number_of_sux() - 1; }

  // generic
  virtual void input_values_do(void f(Value*))   { BlockEnd::input_values_do(f); f(&_tag); }
};


LEAF(TableSwitch, Switch)
 private:
  int _lo_key;

 public:
  // creation
  TableSwitch(Value tag, BlockList* sux, int lo_key) : Switch(tag, sux), _lo_key(lo_key) {}

  // accessors
  int lo_key() const                             { return _lo_key; }
  int hi_key() const                             { return _lo_key + length() - 1; }
};


LEAF(LookupSwitch, Switch)
 private:
  GrowableArray<int>* _keys;

 public:
  // creation
  LookupSwitch(Value tag, BlockList* sux, GrowableArray<int>* keys) : Switch(tag, sux), _keys(keys) {
    assert(keys != NULL, "keys must exist");
    assert(keys->length() == length(), "sux & keys have incompatible lengths");
  }

  // accessors
  int key_at(int i) const                        { return _keys->at(i); }
};


LEAF(Ret, BlockEnd)
 private:
  int _index;

 public:
  // creation
  Ret(int index) : BlockEnd(illegalType), _index(index) {}

  // accessors
  int index() const                              { return _index; }
};


LEAF(Return, BlockEnd)
 private:
  Value _result;
  int   _monitor_no;

 public:
  // creation
  Return(Value result, int monitor_no) : BlockEnd(result == NULL ? voidType : result->type()), _result(result), _monitor_no(monitor_no) {}

  // accessors
  Value result() const                           { return _result; }
  bool has_result() const                        { return result() != NULL; }
  int monitor_no() const                         { return _monitor_no; }
  bool is_synchronized() const                   { return monitor_no() >= 0; }

  // generic
  virtual void input_values_do(void f(Value*)) {
    BlockEnd::input_values_do(f); 
    if (has_result()) f(&_result);
  }
};


LEAF(Throw, BlockEnd)
 private:
  Value _exception;

 public:
  // creation
  Throw(Value exception) : BlockEnd(illegalType), _exception(exception) {
    ASSERT_VALUES
  }

  // accessors
  Value exception() const                        { return _exception; }

  // generic
  virtual bool can_trap() const                  { return true; }
  virtual void input_values_do(void f(Value*))   { BlockEnd::input_values_do(f); f(&_exception); }
};


LEAF(Base, BlockEnd)
 public:
  // creation
  Base(BlockBegin* std_entry, BlockBegin* osr_entry) : BlockEnd(illegalType) {
    assert(std_entry->is_set(BlockBegin::std_entry_flag), "std entry must be flagged");
    assert(osr_entry == NULL || osr_entry->is_set(BlockBegin::osr_entry_flag), "osr entry must be flagged");
    BlockList* s = new GrowableArray<BlockBegin*>(2);
    if (osr_entry != NULL) s->append(osr_entry);
    s->append(std_entry); // must be default sux!
    set_sux(s);
  }

  // accessors
  BlockBegin* std_entry() const                  { return default_sux(); }
  BlockBegin* osr_entry() const                  { return number_of_sux() < 2 ? NULL : sux_at(0); }
};


#undef ASSERT_VALUES
