// @(#)c1_InstructionPrinter.cpp	1.77 00/04/26 22:06:07
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_c1_InstructionPrinter.cpp.incl"


#ifndef PRODUCT

const char* InstructionPrinter::basic_type_name(BasicType type) {
  switch (type) {
    case T_BOOLEAN: return "boolean";
    case T_BYTE   : return "byte";
    case T_CHAR   : return "char";
    case T_SHORT  : return "short";
    case T_INT    : return "int";
    case T_LONG   : return "long";
    case T_FLOAT  : return "float";
    case T_DOUBLE : return "double";
    case T_ARRAY  : return "array";
    case T_OBJECT : return "object";
    default       : return "???";
  }
}


const char* InstructionPrinter::cond_name(If::Condition cond) {
  switch (cond) {
    case If::eql: return "==";
    case If::neq: return "!=";
    case If::lss: return "<";
    case If::leq: return "<=";
    case If::gtr: return ">";
    case If::geq: return ">=";
  }
  ShouldNotReachHere();
  return NULL;
}


const char* InstructionPrinter::op_name(Bytecodes::Code op) {
  switch (op) {
    // arithmetic ops
    case Bytecodes::_iadd : // fall through
    case Bytecodes::_ladd : // fall through
    case Bytecodes::_fadd : // fall through
    case Bytecodes::_dadd : return "+";
    case Bytecodes::_isub : // fall through
    case Bytecodes::_lsub : // fall through
    case Bytecodes::_fsub : // fall through
    case Bytecodes::_dsub : return "-";
    case Bytecodes::_imul : // fall through
    case Bytecodes::_lmul : // fall through
    case Bytecodes::_fmul : // fall through
    case Bytecodes::_dmul : return "*";
    case Bytecodes::_idiv : // fall through
    case Bytecodes::_ldiv : // fall through
    case Bytecodes::_fdiv : // fall through
    case Bytecodes::_ddiv : return "/";
    case Bytecodes::_irem : // fall through
    case Bytecodes::_lrem : // fall through
    case Bytecodes::_frem : // fall through
    case Bytecodes::_drem : return "%";
    // shift ops
    case Bytecodes::_ishl : // fall through
    case Bytecodes::_lshl : return "<<";
    case Bytecodes::_ishr : // fall through
    case Bytecodes::_lshr : return ">>";
    case Bytecodes::_iushr: // fall through
    case Bytecodes::_lushr: return ">>>";
    // logic ops
    case Bytecodes::_iand : // fall through
    case Bytecodes::_land : return "&";
    case Bytecodes::_ior  : // fall through
    case Bytecodes::_lor  : return "|";
    case Bytecodes::_ixor : // fall through
    case Bytecodes::_lxor : return "^";
  }
  return Bytecodes::name(op);
}


void InstructionPrinter::print_klass(ciKlass* klass) {
  tty->print("<klass 0x%x>", klass);
}


void InstructionPrinter::print_object(Value obj) {
  ValueType* type = obj->type();
  if (type->as_ObjectConstant() != NULL) {
    ciObject* value = type->as_ObjectConstant()->value();
    if (value->is_null_object()) {
      tty->print("null");
    } else {
      tty->print("<object 0x%x>", value->encoding());
    }
  } else if (type->as_InstanceConstant() != NULL) {
    tty->print("<instance 0x%x>", type->as_InstanceConstant()->value()->encoding());
  } else if (type->as_ArrayConstant() != NULL) {
    tty->print("<array 0x%x>", type->as_ArrayConstant()->value()->encoding());
  } else if (type->as_ClassConstant() != NULL) {
    ciKlass* klass = type->as_ClassConstant()->value();
    if (!klass->is_loaded()) {
      tty->print("<unloaded class>");
    } else {
      tty->print("<class 0x%x>", klass->encoding());
    }
  } else {
    tty->print("???");
  }
}


void InstructionPrinter::print_temp(Value value) {
  tty->print("%c%d", value->type()->tchar(), value->id());
}


void InstructionPrinter::print_local(AccessLocal* local) {
  tty->print("L%d", local->index());
}


void InstructionPrinter::print_field(AccessField* field) {
  print_value(field->obj());
  tty->print("._%d", field->offset());
}


void InstructionPrinter::print_indexed(AccessIndexed* indexed) {
  print_value(indexed->array());
  tty->put('[');
  print_value(indexed->index());
  tty->put(']');
}


void InstructionPrinter::print_monitor(AccessMonitor* monitor) {
  tty->print("monitor[%d](", monitor->monitor_no());
  print_value(monitor->obj());
  tty->put(')');
}


void InstructionPrinter::print_loop(AccessLoop* loop) {
  tty->print("loop[%d]", loop->loop_id());
}


void InstructionPrinter::print_op2(Op2* instr) {
  print_value(instr->x());
  tty->print(" %s ", op_name(instr->op()));
  print_value(instr->y());
}


void InstructionPrinter::print_value(Value value) {
  if (value == NULL) {
    tty->print("NULL");
  } else {
    print_temp(value);
  }
}


void InstructionPrinter::print_instr(Instruction* instr) {
  instr->visit(this);
}


void InstructionPrinter::print_stack(ValueStack* stack) {
  if (stack->stack_is_empty()) {
    tty->print("empty stack");
  } else {
    tty->print("stack [");
    for (int i = 0; i < stack->stack_size();) {
      if (i > 0) tty->print(", ");
      tty->print("%d:", i);
      print_value(stack->stack_at_inc(i));
    }
    tty->put(']');
  }
}


void InstructionPrinter::fill_to(int pos, char filler) {
  while (tty->position() < pos) tty->put(filler);
}


void InstructionPrinter::print_head() {
  const char filler = '_';
  fill_to(bci_pos  , filler); tty->print("bci"  );
  fill_to(use_pos  , filler); tty->print("use"  );
  fill_to(temp_pos , filler); tty->print("tid"  );
  fill_to(instr_pos, filler); tty->print("instr");
  fill_to(end_pos  , filler);
  tty->cr();
}


void InstructionPrinter::print_line(Instruction* instr) {
  // print instruction data on one line
  if (instr->is_pinned()) tty->put('.');
  fill_to(bci_pos  ); tty->print("%d", instr->bci());
  fill_to(use_pos  ); tty->print("%d", instr->use_count());
  fill_to(temp_pos ); print_temp(instr);
  fill_to(instr_pos); print_instr(instr);
  tty->cr();
  // add a line for StateSplit instructions w/ non-empty stacks
  // (make it robust so we can print incomplete instructions)
  StateSplit* split = instr->as_StateSplit();
  if (split != NULL && split->stack() != NULL && !split->stack()->stack_is_empty()) {
    fill_to(instr_pos); print_stack(split->stack());
    tty->cr();
  }
}


void InstructionPrinter::do_Phi(Phi* x) {
  tty->print("phi[%d of %d]", x->index(), x->stack_size());
}


void InstructionPrinter::do_Constant(Constant* x) {
  ValueType* t = x->type();
  switch (t->tag()) {
    case intTag   : tty->print("%d"  , t->as_IntConstant   ()->value()); break;
    case longTag  : tty->print("%ldL", t->as_LongConstant  ()->value()); break;
    case floatTag : tty->print("%g"  , t->as_FloatConstant ()->value()); break;
    case doubleTag: tty->print("%gD" , t->as_DoubleConstant()->value()); break;
    case objectTag: print_object(x);                                     break;
    default       : tty->print("???"                                  ); break;
  }
}


void InstructionPrinter::do_LoadLocal(LoadLocal* x) {
  print_local(x);
}


void InstructionPrinter::do_StoreLocal(StoreLocal* x) {
  print_local(x);
  tty->print(" := ");
  print_value(x->value());
}


void InstructionPrinter::do_LoadField(LoadField* x) {
  print_field(x);
}


void InstructionPrinter::do_StoreField(StoreField* x) {
  print_field(x);
  tty->print(" := ");
  print_value(x->value());
}


void InstructionPrinter::do_ArrayLength(ArrayLength* x) {
  print_value(x->array());
  tty->print(".length");
}


void InstructionPrinter::do_LoadIndexed(LoadIndexed* x) {
  print_indexed(x);
}


void InstructionPrinter::do_StoreIndexed(StoreIndexed* x) {
  print_indexed(x);
  tty->print(" := ");
  print_value(x->value());
}


void InstructionPrinter::do_LoopEnter(LoopEnter* x) {
  tty->print("enter ");
  print_loop(x);
}


void InstructionPrinter::do_LoopExit(LoopExit* x) {
  tty->print("exit ");
  print_loop(x);
}


void InstructionPrinter::do_NegateOp(NegateOp* x) {
  tty->put('-');
  print_value(x->x());
}


void InstructionPrinter::do_ArithmeticOp(ArithmeticOp* x) {
  print_op2(x);
}


void InstructionPrinter::do_ShiftOp(ShiftOp* x) {
  print_op2(x);
}


void InstructionPrinter::do_LogicOp(LogicOp* x) {
  print_op2(x);
}


void InstructionPrinter::do_CompareOp(CompareOp* x) {
  print_op2(x);
}


void InstructionPrinter::do_IfOp(IfOp* x) {
  print_value(x->x());
  tty->print(" %s ", cond_name(x->cond()));
  print_value(x->y());
  tty->print(" ? ");
  print_value(x->tval());
  tty->print(" : ");
  print_value(x->fval());
}


void InstructionPrinter::do_Convert(Convert* x) {
  tty->print("%s(", Bytecodes::name(x->op()));
  print_value(x->value());
  tty->put(')');
}


void InstructionPrinter::do_NullCheck(NullCheck* x) {
  tty->print("null_check(");
  print_value(x->obj());
  tty->put(')');
}


void InstructionPrinter::do_JsrContinuation(JsrContinuation* x) {
  tty->print("jsr continuation (bci = %d)", x->bci());
}


void InstructionPrinter::do_Invoke(Invoke* x) {
  if (x->receiver() != NULL) {
    print_value(x->receiver());
    tty->print(".");
  }
  tty->print("%s(", Bytecodes::name(x->code()));
  for (int i = 0; i < x->number_of_arguments(); i++) {
    if (i > 0) tty->print(", ");
    print_value(x->argument_at(i));
  }
  tty->put(')');
}


void InstructionPrinter::do_NewInstance(NewInstance* x) {
  tty->print("new instance ");
  print_klass(x->klass());
}


void InstructionPrinter::do_NewTypeArray(NewTypeArray* x) {
  tty->print("new %s array [", basic_type_name(x->elt_type()));
  print_value(x->length());
  tty->put(']');
}


void InstructionPrinter::do_NewObjectArray(NewObjectArray* x) {
  tty->print("new object array [");
  print_value(x->length());
  tty->print("] ");
  print_klass(x->klass());
}


void InstructionPrinter::do_NewMultiArray(NewMultiArray* x) {
  tty->print("new multi array [");
  Values* dims = x->dims();
  for (int i = 0; i < dims->length(); i++) {
    if (i > 0) tty->print(", ");
    print_value(dims->at(i));
  }
  tty->print("] ");
  print_klass(x->klass());
}


void InstructionPrinter::do_MonitorEnter(MonitorEnter* x) {
  tty->print("enter ");
  print_monitor(x);
}


void InstructionPrinter::do_MonitorExit(MonitorExit* x) {
  tty->print("exit ");
  print_monitor(x);
}


void InstructionPrinter::do_Intrinsic(Intrinsic* x) {
  const char* name = "<unknown intrinsic>";
  switch (x->id()) {
    case methodOopDesc::_dsin     : name = "dsin"     ; break;
    case methodOopDesc::_dcos     : name = "dcos"     ; break;
    case methodOopDesc::_dsqrt    : name = "dsqrt"    ; break;
    case methodOopDesc::_arraycopy: name = "arraycopy"; break;
  }
  tty->print("%s(", name);
  for (int i = 0; i < x->number_of_arguments(); i++) {
    if (i > 0) tty->print(", ");
    print_value(x->argument_at(i));
  }
  tty->put(')');
}


void InstructionPrinter::do_BlockBegin(BlockBegin* x) {
  // print block extension
  BlockEnd* end = x->end();
  tty->print("B%d [%d, %d]", x->block_id(), x->bci(), (end == NULL ? -1 : end->bci()));
  // print block successors
  if (end != NULL && end->number_of_sux() > 0) {
    tty->print(" ->");
    for (int i = 0; i < end->number_of_sux(); i++) {
      tty->print(" B%d", end->sux_at(i)->block_id());
    }
  }
  // print subroutines
  if (x->number_of_subroutines() > 0) {
    tty->print(" (");
    for (int i = 0; i < x->number_of_subroutines();  i++) {
      if (i > 0) tty->print(" ");
      tty->print("B%d", x->subroutine_at(i)->block_id());
    }
    tty->put(')');
  }
}


void InstructionPrinter::do_CheckCast(CheckCast* x) {
  tty->print("checkcast(");
  print_value(x->obj());
  tty->print(") ");
  print_klass(x->klass());
}


void InstructionPrinter::do_InstanceOf(InstanceOf* x) {
  tty->print("instanceof(");
  print_value(x->obj());
  tty->print(") ");
  print_klass(x->klass());
}


void InstructionPrinter::do_Goto(Goto* x) {
  tty->print("goto B%d", x->default_sux()->block_id());
}


void InstructionPrinter::do_Jsr(Jsr* x) {
  tty->print("jsr B%d", x->subroutine()->block_id());
}


void InstructionPrinter::do_If(If* x) {
  tty->print("if ");
  print_value(x->x());
  tty->print(" %s ", cond_name(x->cond()));
  print_value(x->y());
  tty->print(" then B%d else B%d", x->sux_at(0)->block_id(), x->sux_at(1)->block_id());
}


void InstructionPrinter::do_IfInstanceOf(IfInstanceOf* x) {
  tty->print("<IfInstanceOf>");
}


void InstructionPrinter::do_TableSwitch(TableSwitch* x) {
  tty->print("tableswitch ");
  print_value(x->tag());
  tty->cr();
  int l = x->length();
  for (int i = 0; i < l; i++) {
    fill_to(instr_pos);
    tty->print_cr("case %5d: B%d", x->lo_key() + i, x->sux_at(i)->block_id());
  }
  fill_to(instr_pos);
  tty->print("default   : B%d", x->default_sux()->block_id());
}


void InstructionPrinter::do_LookupSwitch(LookupSwitch* x) {
  tty->print("lookupswitch ");
  print_value(x->tag());
  tty->cr();
  int l = x->length();
  for (int i = 0; i < l; i++) {
    fill_to(instr_pos);
    tty->print_cr("case %5d: B%d", x->key_at(i), x->sux_at(i)->block_id());
  }
  fill_to(instr_pos);
  tty->print("default   : B%d", x->default_sux()->block_id());
}


void InstructionPrinter::do_Ret(Ret* x) {
  tty->print("ret L%d", x->index());
}


void InstructionPrinter::do_Return(Return* x) {
  if (x->result() == NULL) {
    tty->print("return");
  } else {
    tty->print("%creturn ", x->type()->tchar());
    print_value(x->result());
  }
}


void InstructionPrinter::do_Throw(Throw* x) {
  tty->print("throw ");
  print_value(x->exception());
}


void InstructionPrinter::do_Base(Base* x) {
  tty->print("std entry B%d", x->std_entry()->block_id());
  if (x->number_of_sux() > 1) {
    tty->print(" osr entry B%d", x->osr_entry()->block_id());
  }
}

#endif // PRODUCT

