// @(#)c1_Miscellaneous.hpp	1.4 00/03/27 18:39:50
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


// set frame size and return address offset to this values, if the
// compiled frame uses ebp as link pointer; otherwise, the frame size
// must be fixed

# include "incls/_c1_Miscellaneous_pd.hpp.incl"

enum {
  no_return_address_offset = -1,
  no_frame_size            = -1
};

enum {
  // Advertise here if the CPU requires explicit rounding operations to implement the strictFP mode.
  strict_fp_requires_explicit_rounding = pd_strict_fp_requires_explicit_rounding
};


// This is the structure generated by Compiler1; it is converted inside
// JVMCI_install_code into the VM debug information
class JVMCI_DebugInfo: public ResourceObj {
public:
  ciMethod* method;
  int       pc_offset;
  int       bci;
  bool      at_call;
  OopMap*   oop_map;
  // locals, expressions and monitors come later
  GrowableArray<ScopeValue*>* locarray;
  GrowableArray<ScopeValue*>* exparray;
  GrowableArray<MonitorValue*>* monarray;

  JVMCI_DebugInfo(ciMethod* m, int p_pc, int p_bci, OopMap* p_map, bool p_at_call):
        method(m), pc_offset(p_pc), bci(p_bci), at_call(p_at_call), oop_map(p_map),
        locarray(NULL), exparray(NULL), monarray(NULL) {}
  void set_locations(GrowableArray<ScopeValue*>* l_array, GrowableArray<ScopeValue*>* e_array, GrowableArray<MonitorValue*>* m_array) {
    locarray = l_array;
    exparray = e_array;
    monarray = m_array;
  }
};


class JVMCI_OffsetDesc: public StackObj {
public:
  int _iep_offset;
  int _ep_offset;
  int _vep_offset;
  int _code_offset;
  int _osr_offset;
  JVMCI_OffsetDesc(): _iep_offset(0), _ep_offset(0), _vep_offset(0), _code_offset(0), _osr_offset(0) {}
};


class JVMCI_LineNumbersDesc: public ResourceObj {
public:
  JVMPI_Lineno* _line_numbers;
  int           _nof_lines;
  JVMCI_LineNumbersDesc(): _line_numbers(NULL), _nof_lines(0) {}
};



//-----------------Runtime-------------------------------------

// The following routines allow the installation of filter functions
// handling implicit exceptions. The filter functions are called when
// an implicit exception happened outside the interpreter. All registers
// except eip & flags are untouched, i.e. contain the values they had at
// the exception pc, the stack is untouched but contains the filter's
// return address on top. If the filter cannot handle the exception, it
// is supposed to return w/o changing any state (besides popping the
// return address).

JNIEXPORT void JNICALL
JVMCI_InstallImplicitNullExceptionFilter(address filter);

JNIEXPORT void JNICALL
JVMCI_InstallImplicitDiv0ExceptionFilter(address filter);

JNIEXPORT void JNICALL
JVMCI_InstallStackOvrFlowExceptionFilter(address filter);
