/*
 * @(#)c1_RInfo.cpp	1.8 00/04/18 12:29:22
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_c1_RInfo.cpp.incl"


//-------------------------------------------------------
//                 RInfo
//-------------------------------------------------------

bool RInfo::overlaps (const RInfo other) const {
  if (this->is_word()) {
    if (other.is_word()) return this->reg() == other.reg();
    if (other.is_long()) return this->reg() == other.reg_lo() || this->reg() == other.reg_hi();
    return false;
  } else if (this->is_long()) {
    if (other.is_long()) {
      if (this->reg_lo() == other.reg_lo()) return true;
      if (this->reg_lo() == other.reg_hi()) return true;
      if (this->reg_hi() == other.reg_lo()) return true;
      if (this->reg_hi() == other.reg_hi()) return true;
      return false;
    }
    if (other.is_word()) return this->reg_lo() == other.reg() || this->reg_hi() == other.reg();
    return false;
  }
  return false;
}

Register RInfo::as_register() const {
  return FrameMap::cpu_rnr2reg(reg());
}

Register RInfo::as_register_lo() const {
  return FrameMap::cpu_rnr2reg(reg_lo());
}

Register RInfo::as_register_hi() const {
  return FrameMap::cpu_rnr2reg(reg_hi());
}

#ifndef PRODUCT

void RInfo::print() const {
  if (is_word()) {
    tty->print(as_register().name());
  } else if (is_long()) {
    const char* nameLO = as_register_lo().name();
    const char* nameHI = as_register_hi().name();
    tty->print("(", nameLO, ",", nameHI, ")");
  } else if (is_float() || is_double()) {
    pd_print_fpu();
  } else {
    assert(is_illegal(), "");
    tty->print("Illegal");
  }
}


void RInfo::print_raw() const {
  tty->print_cr(" RInfo - type:%d reg1:%d reg2:%d", type(), reg1(), reg2());
}


void RInfo::print(char* msg) const {
  tty->print("XX %s", msg); print();
}

#endif


//--------------------------------------------------------
//               c1_RegMask
//--------------------------------------------------------


int              c1_RegMask::_size      = 0;
const c1_RegMask c1_RegMask::_empty_set = c1_RegMask ();


void c1_RegMask::init_masks(int size) {
  //assert(size > 0 && size <= sizeof(c1_RegMask::_mask) * BitsPerByte, "wrong size");
  _size = size;
}


int c1_RegMask::get_first() const {
  assert(!is_empty(), "");
  for (int rnr = 0; rnr < _size; rnr++) {
    if (contains(rnr)) {
      return rnr;
    }
  }
  return -1;
}


#ifndef PRODUCT

void c1_RegMask::print() const {
  for (int rnr = 0; rnr < _size; rnr++) {
    if (contains(rnr)) {
      RInfo::word_reg(rnr).print();
      tty->print(", ");
    }
  }
}

#endif
