// @(#)c1_Runtime1.cpp	1.126 00/10/28 17:30:48
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_c1_Runtime1.cpp.incl"


// Implementation of StubAssembler

StubAssembler::StubAssembler(CodeBuffer* code) : C1_MacroAssembler(code) {
  _name = NULL;
  _must_gc_arguments = false;
}


void StubAssembler::set_info(const char* name, bool must_gc_arguments) {
  _name = name;
  _must_gc_arguments = must_gc_arguments;
}


// Implementation of iEntries

iEntries::iEntries() {
  _static_call_offset    = 0;
  _optimized_call_offset = 0;
  _virtual_call_offset   = 0;
  _virtual_final_call_offset  = 0;
}


iEntries::iEntries(int static_call_offset, int optimized_call_offset, int virtual_call_offset, int virtual_final_call_offset) {
  _static_call_offset    = static_call_offset;
  _optimized_call_offset = optimized_call_offset;
  _virtual_call_offset   = virtual_call_offset;
  _virtual_final_call_offset  = virtual_final_call_offset;
}


void iEntries::set_base(address base) {
  assert(base != NULL, "base must exist");
  _base = base;
}


// Implementation of Runtime1

bool      Runtime1::_is_initialized = false;
CodeBlob* Runtime1::_blobs[Runtime1::number_of_ids];
iEntries  Runtime1::_ientries[AbstractInterpreter::number_of_method_entries];


// statistics
int Runtime1::_resolve_invoke_cnt = 0;
int Runtime1::_handle_wrong_method_cnt = 0;
int Runtime1::_ic_miss_cnt = 0;


void Runtime1::generate_blob_for(StubID id) {
  assert(0 <= id && id < number_of_ids, "illegal stub id");
  ResourceMark rm;
  // create code buffer for code storage
  int code_size = CodeBuffer::insts_memory_size(VerifyOops ? 8*K : 4*K);
  int locs_size = CodeBuffer::locs_memory_size(1*K);
  
  CodeBuffer* code = new CodeBuffer(code_size, locs_size, 0, 0, false);
  // create assembler for code generation
  StubAssembler* sasm = new StubAssembler(code);
  // generate code
  // create runtime stub
  OopMapSet* oop_maps = generate_code_for(id, sasm);
  // align so printing shows nop's instead of random code at the end (SimpleStubs are aligned)
  sasm->align(BytesPerWord);
  // make sure all code is in code buffer
  sasm->flush();
  // create blob - distinguish a few special cases
  CodeBlob* blob = NULL;
  switch (id) {
    case illegal_instruction_handler_id:
      blob = SafepointBlob::create(code, oop_maps, no_return_address_offset, no_frame_size);
      break;

    default:
      { RuntimeStub* stub = RuntimeStub::new_runtime_stub(code, no_return_address_offset, no_frame_size, oop_maps, sasm->must_gc_arguments());
        stub->set_name(sasm->name());
        blob = stub;
      }
      break;
  }
  // install blob
  assert(blob != NULL, "blob must exist");
  _blobs[id] = blob;
}


void Runtime1::initialize() {
  // Warning: If we have more than one compilation running in parallel, we
  //          need a lock here with the current setup (lazy initialization).
  if (!is_initialized()) {
    _is_initialized = true;
    // generate stubs
    for (int id = 0; id < number_of_ids; id++) generate_blob_for((StubID)id);
    // 'relocate' interpreter entries
    address base = blob_for(interpreter_entries_id)->instructions_begin();
    for (int j = 0; j < AbstractInterpreter::number_of_method_entries; j++) {
      _ientries[j].set_base(base);
    }
    // printing
    if (PrintSimpleStubs) {
      tty->print_cr("PrintSimpleStub is not implemented");
    }
  }
}


CodeBlob* Runtime1::blob_for(StubID id) {
  assert(0 <= id && id < number_of_ids, "illegal stub id");
  if (!is_initialized()) initialize();
  return _blobs[id];
}


iEntries* Runtime1::ientries_for(methodHandle m) {
  if (!is_initialized()) initialize();
  AbstractInterpreter::MethodKind kind = AbstractInterpreter::method_kind(m);
  assert(0 <= kind && kind < AbstractInterpreter::number_of_method_entries, "illegal index");
  return &_ientries[kind];
}


// runtime entry points

// Finds receiver, CallInfo (i.e. receiver method), and calling bytecode)
// for a call current in progress, i.e., arguments has been pushed on stack
// put callee has not been invoked yet.  Used by: resolve virtual/static,
// vtable updates, etc.  Caller frame must be compiled
static Handle compute_call_info(JavaThread* thread, Bytecodes::Code& invoke_code, CallInfo& callinfo, TRAPS) {
  Handle receiver = NULL;
  ResourceMark rm;

  // last java frame on stack (which includes native call frames)
  vframeStream vfst(thread, true);  // Do not skip and javaCalls
  assert(!vfst.at_end(), "Java frame must exist");
  
  // get caller and bci
  methodHandle caller = vfst.method();
  int bci = vfst.bci();
  
  // get invoke bytecode and index
  Bytecode_invoke* invoke = Bytecode_invoke_at(caller, bci);
  invoke_code = invoke->adjusted_invoke_code();
  int invoke_index = invoke->index();

  // get receiver, if any
  if (invoke_code != Bytecodes::_invokestatic) {
    // This register map must be update since we need to find the receiver for
    // compiled frames. The receiver might be in a register.
    RegisterMap reg_map(thread); 
    frame stubFrame   = thread->last_frame();
    frame caller_frame = stubFrame.real_sender(&reg_map);

    methodHandle callee = invoke->static_target(CHECK_0);
    if (callee.is_null()) {
      THROW_0(vmSymbols::java_lang_NoSuchMethodException());
    }  
#ifdef SPARC
    receiver = caller_frame.saved_receiver();
#else
    symbolHandle signature = callee->signature();
    receiver = caller_frame.interpreter_callee_receiver(signature);
#endif
    if (receiver.is_null()) {
      THROW_0(vmSymbols::java_lang_NullPointerException());
    }
  }      

  // Resolve method. This is parameterized by bytecode.  
  constantPoolHandle constants = caller->constants();
  assert (receiver.is_null() || receiver->is_oop(), "wrong receiver");
  LinkResolver::resolve_invoke(callinfo, receiver, constants, invoke_index, invoke_code, CHECK_0);

#ifdef ASSERT
  // Check that the receiver klass is of the right subtype and that it is initialized for virtual calls  
  if (invoke_code != Bytecodes::_invokestatic) {    
    assert(receiver.not_null(), "should have thrown exception");
    KlassHandle receiver_klass = receiver->klass();
    KlassHandle static_receiver_klass = constants->klass_ref_at(invoke_index, CHECK_0); // klass is already loaded
    assert(receiver_klass->is_subtype_of(static_receiver_klass()), "actual receiver must be subclass of static receiver klass");
    if (receiver_klass->oop_is_instance()) {
      if (instanceKlass::cast(receiver_klass())->is_not_initialized()) {
        tty->print_cr("ERROR: Klass not yet initialized!!");
        receiver_klass.print();
      }
      assert (!instanceKlass::cast(receiver_klass())->is_not_initialized(), "receiver_klass must be initialized");
    }
  }
#endif

  return receiver;
}


static void update_verified_code_entry(methodHandle callee_method, Handle recv, bool lazy_entry) {
  // Not used anymore
}


static bool is_native_static(address pc) {
  pc = pc - NativeCall::instruction_size;
  nmethod* nm = CodeCache::find_nmethod(pc);
  RelocIterator iter(nm, pc, pc+1);
  bool ret = iter.next();
  assert(ret == true, "relocInfo must exist at this address");    
  assert(iter.addr() == pc, "must find call");
  if (iter.type() == relocInfo::virtual_call_type) {      
    return false;
  } else {
    assert(iter.type() == relocInfo::opt_virtual_call_type, "must be a virtual call");
    return true;
  }
}


JRT_ENTRY(address, Runtime1::resolve_invoke(JavaThread* thread, oop receiver))
  ResourceMark rm;
#ifdef ASSERT
  _resolve_invoke_cnt++;
  RegisterMap debug_cbl_map(thread, false);
  frame debug_caller = thread->last_frame().sender(&debug_cbl_map);  
  assert(thread->vm_result() == NULL, "who did it?")
  assert(thread->vm_result_2() == NULL, "who did it?")
#endif
  Handle recv(thread, receiver);

  // make sure caller is not getting deoptimized
  // and removed before we are done with it.
  RegisterMap cbl_map(thread, false);
  frame caller = thread->last_frame().sender(&cbl_map);  
  nmethodLocker caller_lock(caller.pc());
  
  // determine call info & receiver
  // note: a) receiver is NULL for static calls
  //       b) an exception is thrown if receiver is NULL for non-static calls
  CallInfo call_info;
  Bytecodes::Code invoke_code = Bytecodes::_illegal;
  { // computes invoke_code and call_info
    Handle r = compute_call_info(thread, invoke_code, call_info, CHECK_0);
    assert(r() == recv(), "receivers must be identical");
  }
  methodHandle callee_method = call_info.selected_method();
  bool is_virtual = (invoke_code == Bytecodes::_invokevirtual || invoke_code == Bytecodes::_invokeinterface);
  // for now: we do not optimize any optimizable virtual calls
  // bool is_optimized = Klass::can_be_statically_bound(call_info.resolved_method()());

  // The only possible optimization is currently for finals
  bool is_optimized = call_info.resolved_method()->is_final_method();

  // Note: the following problem exists with Compiler1:
  //       - at compile time we may or may not know if the destination is final
  //       - if we know that the destination is final, we will emit an optimized
  //         virtual call (no inline cache), and need a methodOop to make a call to
  //         interpreter
  //       - if we do not know if the destination is final, we emit a standard virtual
  //         call, and use CompiledICHolder to call interpreted code (no static call
  //         stub has been generated)
  //       That is why we need to determine if the call is static in its native form.

  // static_call is true for invokestatic, invokespecial and invokevirtuals to known final methods
  // static call does not have an inline cache
  bool static_call = is_virtual ? is_native_static(caller.pc()) : true;

  // tracing/debugging
#ifndef PRODUCT
  if (TraceCallFixup) {
    ResourceMark rm;
    tty->print("resolving %s to: ", Bytecodes::name(invoke_code));
    callee_method->print_short_name(tty);
    tty->print_cr(" (nmethod = 0x%x)", callee_method->code());                  
  }
#endif

  // Compute entry points. This might require generation of C2I converter
  // frames, so we cannot be holding any locks here. Furthermore, the
  // computation of the entry points is independent of patching the call.  We
  // always return the entry-point, but we only patch the stub if the call has
  // not been deoptimized.  Return values: For a virtual call this is an
  // (cached_oop, destination address) pair. For a static call/optimized
  // virtual this is just a destination address.

  StaticCallInfo static_call_info;
  CompiledICInfo virtual_call_info;  
  address  entry = NULL;  
  Handle   oop_result;
  Mutex*   lock;
  
  // Make sure the callee nmethod does not get deoptimized and removed before
  // we are done patching the code. callee_nm is NULL if we are calling an 
  // interpreted method
  nmethodLocker nl_callee(callee_method->code());

#ifdef ASSERT
  nmethod* nm = callee_method->code();
  address dest_entry_point = (nm == NULL) ? 0 : nm->entry_point(); // used below
#endif

  if (is_virtual && !static_call) {
    // We cannot compute this info for optimized virtual calls (needs a methodOop in 
    // info instead of the CompiledICHolder)
    assert(recv.not_null(), "sanity check");
    // for now: we do not optimize any optimizable virtual calls
    bool static_bound = is_optimized; //Klass::can_be_statically_bound(call_info.resolved_method()());
    CompiledIC::compute_monomorphic_entry(callee_method, recv->klass(), is_optimized, static_bound, virtual_call_info, CHECK_0);
    entry = virtual_call_info.entry();
    oop_result = virtual_call_info.cached_oop();
    lock = CompiledIC_lock;    
  } else {
    // static call or optimized virtual calls (use methodOop in info)
    CompiledStaticCall::compute_entry(callee_method, static_call_info);    
    entry = static_call_info.entry();
    oop_result = callee_method(); // Only used when calling interpreter
    lock = Patching_lock;
  }      

  // grab lock, check for deoptimization and potentially patch caller
  { MutexLocker ml_patch(lock);
    // Reread frame since deoptimization could have happened while we
    // waited for the lock
    RegisterMap map(thread, false);
    frame caller_frame = thread->last_frame().sender(&map);  

#ifdef ASSERT
    // We must not try to patch to jump to an already unloaded method.
        
    if (dest_entry_point != 0) {
      assert(CodeCache::find_blob(dest_entry_point) != NULL, "should not unload nmethod while locked");
    }
#endif

    if (!caller_frame.is_deoptimized_frame()) {
      if (is_virtual && !static_call) {
        CompiledIC* inline_cache = CompiledIC_before(caller_frame.pc());  
        if (inline_cache->is_clean()) inline_cache->set_to_monomorphic(virtual_call_info);
      } else {
        CompiledStaticCall* ssc = compiledStaticCall_before(caller_frame.pc());            
        if (ssc->is_clean()) ssc->set(static_call_info);      
      }
    }
  }

  // return callee methodOop, recv and interpreter or verified entry point
  // (resolution was exact)
  update_verified_code_entry(callee_method, recv, false);
  thread->set_vm_result(callee_method());
  thread->set_vm_result_2(recv());
  return callee_method->verified_code_entry();
JRT_END


JRT_ENTRY(address, Runtime1::handle_ic_miss(JavaThread* thread, oop receiver))
  ResourceMark rm;  
  Handle recv(thread, receiver);

  CallInfo call_info;
  Bytecodes::Code bc;  

  // receiver is NULL for static calls. An exception is thrown for NULL receivers for non-static calls
  { Handle r = compute_call_info(thread, bc, call_info, CHECK_0);
    assert(r() == recv(), "receivers must be identical");
  }
  methodHandle callee_method = call_info.selected_method();

#ifndef PRODUCT
  // Statistics & Tracing
  _ic_miss_cnt++;
  if (TraceICmisses) { tty->print_cr("IC miss"); }
#endif

  // Update inline cache to megamorphic. Skip update if caller has been deoptimized or we 
  // are called from interpreted)
  { MutexLocker ml_patch (CompiledIC_lock);
    RegisterMap reg_map(thread, false);
    frame caller_frame = thread->last_frame().sender(&reg_map);  
    assert(!caller_frame.is_interpreted_frame(), "interpreted methods never call to inline cache entry (without deoptimization)");
    if (caller_frame.is_compiled_frame()) {
      // Not a deoptimized frame, so find inline_cache 
      CompiledIC* inline_cache = CompiledIC_before(caller_frame.pc());    
      if (!inline_cache->is_megamorphic() && !inline_cache->is_clean()) {
        // Change to megamorphic
        inline_cache->set_to_megamorphic(&call_info, (bc == Bytecodes::_invokeinterface), CHECK_0); 
      } else {
        // Either clean or megamorphic
      }
    }  
  } // Release CompiledIC_lock

  // Return methodOop & recv through TLS
  update_verified_code_entry(callee_method, recv, false);
  thread->set_vm_result(callee_method());
  thread->set_vm_result_2(recv());
  // Return address to jump to
  return callee_method->verified_code_entry();
JRT_END


// Note: There are no c2r, c2i or i2c adapters; this code should be cleaned up.
JRT_ENTRY(address, Runtime1::handle_wrong_method(JavaThread* thread, oop receiver))
  ResourceMark rm;
  Handle recv(thread, receiver);
  RegisterMap reg_map(thread, false);
  // Check if the caller is a compiled method or an I2C adapter
  // Note: We can get an deoptimized frame on the stack - if we first get
  // into a lazy_c2i_adapter stub - that will deoptimize the calling frame.
  //assert(check_compiled_frame(thread, true, false, true), "incorrect caller");
  frame c2r_frame = thread->last_frame();  
  frame caller_frame = c2r_frame.sender(&reg_map);
  
  //Force resolving of caller (if we called from compiled frame)        
  if (caller_frame.is_compiled_frame()) {
    Recompilation::reresolve_call_site(caller_frame);   
  }
#ifndef PRODUCT
  _handle_wrong_method_cnt++;
#endif
  
  methodHandle callee_method;

  // We need first to check if any Java activations (compiled, interpreted, deoptimized)
  // exist on the stack since last JavaCall. If not, we need to get the target method 
  // and receiver from the JavaCall wrapper.    
  vframeStream vfst(thread, true);  // Do not skip and javaCalls
  if (vfst.at_end()) {    
    // No Java frames was found on stack since we did the JavaCall.
    // Hence, the stack can only contain an entry_frame and possible some  
    // and i2c_adapter. We need to find the target method and receviver from
    // the stub frame
    RegisterMap reg_map(thread, false);
    frame fr = thread->last_frame();
    assert(fr.is_c2runtime_frame(), "must be a runtimeStub");
    fr = fr.sender(&reg_map);
    while(!fr.is_entry_frame())  {
      assert(fr.is_c2i_frame() || fr.is_i2c_frame(), "sanity check");
      fr = fr.sender(&reg_map);
    }
    // fr is now pointing to the entry frame.    
    callee_method = methodHandle(THREAD, fr.entry_frame_call_wrapper()->callee_method());        
    { Handle r(THREAD, fr.entry_frame_call_wrapper()->receiver());
      assert(r() == recv(), "receivers must be identical");
    }
  } else {
    Bytecodes::Code bc;
    CallInfo callinfo;
    { Handle r = compute_call_info(thread, bc, callinfo, CHECK_0);
      assert(r() == recv(), "receivers must be identical");
    }
    callee_method = callinfo.selected_method();
  }
  assert(recv.is_null() || !callee_method->is_static(), "non-null receiver for static call?");

  update_verified_code_entry(callee_method, recv, false); 
  thread->set_vm_result(callee_method());
  thread->set_vm_result_2(recv());

  // Return address to jump to
  return callee_method->verified_code_entry();
JRT_END


JRT_ENTRY(void, Runtime1::new_instance(JavaThread* thread, klassOop klass))
  assert(oop(klass)->is_klass(), "not a class");
  instanceKlassHandle h(thread, klass);
  // make sure we are not instantiating an abstract klass
  if (h->is_interface() || h->is_abstract()) {
    ResourceMark rm;
    THROW_MSG(vmSymbols::java_lang_InstantiationError(), h->name()->as_C_string());
  }
  // make sure klass is initialized
  h->initialize(CHECK);
  // allocate instance and return via TLS
  oop obj = h->allocate_instance(CHECK);
  thread->set_vm_result(obj);
JRT_END


JRT_ENTRY(void, Runtime1::new_type_array(JavaThread* thread, BasicType elt_type, jint length))
  oop obj = oopFactory::new_typeArray(elt_type, length, CHECK);
  thread->set_vm_result(obj);
JRT_END


JRT_ENTRY(void, Runtime1::new_object_array(JavaThread* thread, klassOop array_klass, jint length))
  // Note: no handle for klass needed since they are not used
  //       anymore after new_objArray() and no GC can happen before.
  //       (This may have to change if this code changes!)       
  assert(oop(array_klass)->is_klass(), "not a class");
  klassOop elem_klass = objArrayKlass::cast(array_klass)->element_klass();
  objArrayOop obj = oopFactory::new_objArray(elem_klass, length, CHECK);
  thread->set_vm_result(obj);
JRT_END


JRT_ENTRY(void, Runtime1::new_multi_array(JavaThread* thread, klassOop klass, int rank, jint* dims))
  assert(oop(klass)->is_klass(), "not a class");
  assert(rank >= 1, "rank must be nonzero");
  oop obj = arrayKlass::cast(klass)->multi_allocate(rank, dims, 1, CHECK);
  thread->set_vm_result(obj);
JRT_END


JRT_ENTRY(void, Runtime1::unimplemented_entry(JavaThread* thread, StubID id))
  tty->print_cr("Runtime1::entry_for(%d) returned unimplemented entry point", id);
JRT_END


JRT_ENTRY(void, Runtime1::store_check(JavaThread* thread, oop array_oop, oop value_oop))
  Events::log("store_check @ %#x", array_oop);
  Handle array(thread, array_oop);
  Handle value(thread, value_oop);
  if (!value.is_null()) {
    klassOop element_klass = objArrayKlass::cast(array->klass())->element_klass();
    bool is_ok = value->klass()->klass_part()->is_subtype_of(element_klass);
    if (!is_ok) {
      thread->set_vm_result  (NULL); 
      thread->set_vm_result_2(NULL); 
      THROW(vmSymbols::java_lang_ArrayStoreException());
    }
  }
  thread->set_vm_result(array());
  thread->set_vm_result_2(value());
JRT_END


JRT_ENTRY(void, Runtime1::checkcast(JavaThread* thread, oop object_oop, oop test_klass_oop))
  Events::log("check_cast @ %#x", object_oop);
  Handle obj (object_oop);
  KlassHandle test_klass  (test_klass_oop);
  bool is_ok = Klass::cast(obj->klass())->is_subtype_of(test_klass());
  if (!is_ok) {    
    ResourceMark rm;
    THROW_MSG(vmSymbols::java_lang_ClassCastException(), Klass::cast(obj->klass())->external_name());    
  }
  thread->set_vm_result(obj());
JRT_END


JRT_ENTRY(int, Runtime1::instanceof(JavaThread* thread, oop object_klass_oop, oop test_klass_oop))
  KlassHandle obj_klass(object_klass_oop);
  KlassHandle test_klass(test_klass_oop);
  bool is_ok = Klass::cast(obj_klass())->is_subtype_of(test_klass());
  return is_ok ? 1 : 0;
JRT_END


JRT_ENTRY(void, Runtime1::throw_exception(JavaThread* thread, oop exception))  
  Handle exc_h(exception);
  THROW_OOP(exc_h());
JRT_END


static void set_pc_not_at_call_for_caller(JavaThread* thread) {
  // if we throw an implicit exception we need to set the
  // 'pc_not_at_call' bit for the caller frame, as the pc
  // information doesn't correspond to a (assembly level)
  // call instruction but another instruction.
  if (thread->is_handling_implicit_exception()) {
    RegisterMap reg_map(thread); 
    frame stub_frame = thread->last_frame();
    frame caller_frame = stub_frame.real_sender(&reg_map);
    thread->set_pc_not_at_call_for_frame(caller_frame.sp());
  }
}

static address fast_find_exception_handler(JavaThread* thread, Handle exception, address pc) {
  assert(!thread->is_handling_implicit_exception() && UseFastExceptionHandling, "should not enter");
  address adjust_pc = thread->safepoint_state()->compute_adjusted_pc(pc);
  nmethod* nm = CodeCache::find_nmethod(adjust_pc);
  assert(nm != NULL, "this is not an nmethod");
  address continuation = nm->handler_for_exception_and_pc(exception, adjust_pc);
  return continuation;
}


extern void vm_exit(int code);

// Enter this method from compiled code only if there is a Java exception handler
// in the method handling the exception

JRT_ENTRY(address, Runtime1::exception_handler_for_pc(JavaThread* thread, oop ex, address pc))
  Handle exception(thread, ex);
#ifdef ASSERT
  assert(exception.not_null(), "NULL exceptions should be handled by throw_exception");
  assert(exception->is_oop(), "just checking");
  // Check that exception is a subclass of Throwable, otherwise we have a VerifyError
  if (!(exception->is_a(SystemDictionary::throwable_klass()))) {
    if (ExitVMOnVerifyError) vm_exit(-1);
    ShouldNotReachHere();
  }
#endif
  bool entered_as_implicit = thread->is_handling_implicit_exception();
  pc = thread->safepoint_state()->compute_adjusted_pc(pc);
  // too noisy event
  // Events::log("exception_handler_for_pc @ %#x oop %#x impl %d", pc, ex, entered_as_implicit);
  // ExceptionCache is used only for exceptions at call and not for implicit exceptions
  if (!entered_as_implicit && UseFastExceptionHandling) {
    address fast_continuation = fast_find_exception_handler(thread, ex, pc);
    if (fast_continuation != NULL) {
      if (fast_continuation == ExceptionCache::unwind_handler()) fast_continuation = NULL;
      thread->set_vm_result(exception());
      return fast_continuation;
    }
  }

  set_pc_not_at_call_for_caller(thread);             // needed for implicit exceptions
  thread->set_is_handling_implicit_exception(false); // reset implicit exception flag
  // get method & bci
  vframeStream vfst(thread, true);
  assert(!vfst.at_end(), "Java frame must exist");
  methodHandle method(thread, vfst.method());
  assert(method->has_exception_handler(), "performance bug: should only be called if there is a handler");
  int bci = vfst.bci();
  // debugging support
  // tracing
  if (TraceExceptions) {
    ThreadCritical tc;
    ResourceMark rm;
    tty->print_cr("Exception <%s> thrown in compiled method <%s> for thread 0x%x",
                   exception->print_value_string(), method->print_value_string(), thread);
  }

  bool should_repeat;
  int handler_bci;

  do {
    should_repeat = false;
    // exception handler lookup
    handler_bci = method->fast_exception_handler_bci_for(exception->klass(), bci, true, THREAD);
    if (HAS_PENDING_EXCEPTION) {
      // We threw an exception while trying to find the exception handler.
      // Transfer the new exception to the exception handle which will
      // be set into thread local storage, and do another lookup for an
      // exception handler for this exception, this time starting at the
      // BCI of the exception handler which caused the exception to be
      // thrown (bug 4307310).
      exception = Handle(PENDING_EXCEPTION);
      CLEAR_PENDING_EXCEPTION;
      if (handler_bci >= 0) {
	bci = handler_bci;
	should_repeat = true;
      }
    }
  } while (should_repeat == true);

  address continuation = NULL;
  if (handler_bci < 0) {
    // no handler in this method => forward exception to callee
    continuation = NULL;
    if (!entered_as_implicit) {
      nmethod* nm = CodeCache::find_nmethod(pc);
      assert(nm->method() == method(), "wrong nmethod");
      nm->add_handler_for_exception_and_pc(exception, pc, ExceptionCache::unwind_handler());
    }
  } else {
    // handler in this method => continue at exception handler
    // find pc offset for the handler bci
    // (note: this is a slow implementation, but it happens once
    //        per exception that is handled - tune if necessary)

    // because of OSR method, method->code() may not be the same
    // as the method that we are working with
    nmethod* nm = CodeCache::find_nmethod(pc);
    assert(nm->method() == method(), "wrong nmethod");

    // note that in C1 also the stubs have pc_descs and can handle exceptions
    // as the stubs are also slow cases
    assert(nm->code_contains(pc) || nm->stub_contains(pc), "wrong nmethod");
    int offset = nm->pc_offset_for_bci(handler_bci);
    assert(offset >= 0, "wrong bci -> pc offset mapping");
    continuation = nm->instructions_begin() + offset;
    if (!entered_as_implicit) {
      // the exception cache is used only by non-implicit exceptions
      assert (continuation != NULL, "must have compiled handler");
      nm->add_handler_for_exception_and_pc(exception, pc, continuation);
    }
  }
  thread->set_vm_result(exception());
  return continuation;
JRT_END


JRT_LEAF(address, Runtime1::return_address_for_exception())
  // Compute the (corrected) issuing pc for an exception pc.
  // Caution: This routine must be a leaf routine (no GCs. no locking!)
  //          because we are in the middle of exception resolving
  //
  // Note: The pc is __not__ a conventional next_pc value (as from
  // frame::pc), but the real address of the trapping instruction.
  //
  // compute the corrected pc (if the exception happened in a temporary code
  // buffer used for safepoint synchronization, we need to adjust it first)
  JavaThread* thread = JavaThread::current();
  assert(!AbstractInterpreter::contains(thread->saved_exception_pc()), "interpreter exceptions must be handled already");
  assert(!thread->is_handling_implicit_exception(), "implicit exception handling bit should not be set");
  address pc = thread->safepoint_state()->compute_adjusted_pc(thread->saved_exception_pc());
  // check if exception happened outside nmethods
  if (VtableStubs::contains(pc) || blob_for(interpreter_entries_id)->contains(pc)) {
    // exception happened in vtable stub or interpreter entry code
    // => must be a NULL receiver => use return address
    // pushed by caller => don't push another return address
    return NULL;
  }
  // exception must have happened inside an nmethod
  CodeBlob* cb = CodeCache::find_blob(pc);
  guarantee(cb != NULL && cb->is_nmethod(), "exception happened outside interpreter, nmethods and vtable stubs");
  nmethod* nm = (nmethod*)cb;
  if (nm->inlinecache_check_contains(pc)) {
    // exception happened inside inline-cache check code
    // => the nmethod is not yet active (i.e., the frame
    // is not set up yet) => use return address pushed by
    // caller => don't push another return address
    return NULL;
  }
  // the exception happened inside an nmethod and is
  // not at a call => set implicit exception bit and
  // push exception pc as return address
  //
  // note: the implicit exception bit is reset when
  //       the exception is handled
  JavaThread::current()->set_is_handling_implicit_exception(true);
  return pc;
JRT_END


JRT_LEAF(address, Runtime1::compute_exception_pc(int pc_offset, address pc_addr))
  // compute the corrected pc (if the exception happened in a temporary code
  // buffer used for safepoint synchronization, we need to adjust it first)
  JavaThread* thread = JavaThread::current();
  address pc_adj = thread->safepoint_state()->compute_adjusted_pc(pc_addr);
  // compute exception pc
  nmethod* nm = CodeCache::find_nmethod(pc_adj);
  assert(nm != NULL, "nmethod not found");
  return nm->instructions_begin() + pc_offset;
JRT_END


JRT_ENTRY(void, Runtime1::throw_range_check_exception(JavaThread* thread))
  Events::log("throw_range_check");
  THROW(vmSymbols::java_lang_ArrayIndexOutOfBoundsException());
JRT_END


JRT_ENTRY(void, Runtime1::throw_div0_exception(JavaThread* thread))  
  set_pc_not_at_call_for_caller(thread); // needed for implicit div0 exceptions
  THROW_MSG(vmSymbols::java_lang_ArithmeticException(), "/ by zero");
JRT_END


// Used only by implicit null exceptions
JRT_ENTRY(void, Runtime1::throw_null_exception(JavaThread* thread))
  set_pc_not_at_call_for_caller(thread); // needed for implicit null exceptions
  THROW(vmSymbols::java_lang_NullPointerException());
JRT_END


// Used by explicit Null exceptions
JRT_ENTRY(void, Runtime1::throw_null_exception_at_call(JavaThread* thread))
  THROW(vmSymbols::java_lang_NullPointerException());
JRT_END


JRT_ENTRY(void, Runtime1::throw_stack_overflow(JavaThread* thread))
  THROW(vmSymbols::java_lang_StackOverflowError());
JRT_END


JRT_ENTRY(void, Runtime1::throw_abstract_method_error(JavaThread* thread))
  THROW(vmSymbols::java_lang_AbstractMethodError());
JRT_END


JRT_ENTRY_FOR_MONITORENTER(void, Runtime1::monitorenter(JavaThread* thread, oop obj, BasicLock* lock))
  Handle h_obj(thread, obj);  
  assert(h_obj.is_null() || Universe::is_heap(h_obj()), "must be NULL or an object");
  ObjectSynchronizer::slow_enter(h_obj, lock, CHECK);
JRT_END


JRT_LEAF(void, Runtime1::monitorexit(JavaThread* thread, oop obj, BasicLock* lock))
  // note: we change the thread-state to thread_in_comp_leaf => this allows
  // the safepoint code to insert safepoints in the compiled code which calls
  // monitorexit - must set last java sp to be able to change the thread state
  assert(thread == JavaThread::current(), "threads must correspond");
  assert(thread->last_Java_sp(), "last_Java_sp must be set");
  StateSaver sv(thread);  // cache post java sp
  ThreadInCompiledLeaf cl(thread);  
  // monitorexit is non-blocking (leaf routine) => no exceptions can be thrown
  EXCEPTION_MARK;
  assert(obj == NULL || Universe::is_heap(obj), "must be NULL or an object");
  if(CompSafepointALot) {
    // try to increase the comp. leaf window to catch more safepoints
    // do it carefully though - we do not want to slow down things too much
    if(!os::dont_yield()) {
      os::yield_all();
    }
  }

  ObjectSynchronizer::slow_exit(obj, lock, THREAD);
JRT_END


int resolve_field_return_offset(methodHandle caller, int bci, TRAPS) {
  Bytecode_field* field_access = Bytecode_field_at(caller(), caller->bcp_from(bci));
  // This can be static or non-static field access
  Bytecodes::Code code       = field_access->code();

  // We must load class, initialize class and resolvethe field
  FieldAccessInfo result; // initialize class if needed
  LinkResolver::resolve_field(result, caller->constants(), field_access->index(), Bytecodes::java_code(code), false, CHECK_0);
  return result.field_offset();
}


klassOop resolve_field_return_klass(methodHandle caller, int bci, TRAPS) {
  Bytecode_field* field_access = Bytecode_field_at(caller(), caller->bcp_from(bci));
  // This can be static or non-static field access
  Bytecodes::Code code       = field_access->code();

  // We must load class, initialize class and resolvethe field
  FieldAccessInfo result; // initialize class if needed
  LinkResolver::resolve_field(result, caller->constants(), field_access->index(), Bytecodes::java_code(code), false, CHECK_0);
  return result.klass()();
}


void Runtime1::patch_code(JavaThread* thread, StubID stub_id, TRAPS) {
  ResourceMark rm;
  RegisterMap reg_map(thread, false);
  frame runtime_frame = thread->last_frame();  
  frame caller_frame = runtime_frame.sender(&reg_map);

  // last java frame on stack
  vframeStream vfst(thread, true);  
  assert(!vfst.at_end(), "Java frame must exist");

  methodHandle caller_method = vfst.method();
  // Note that caller_method->code() may not be same as caller_code because of OSR's
  nmethod* caller_code = CodeCache::find_nmethod(caller_frame.pc());
  assert(caller_code != NULL, "nmethod not found");
  assert(caller_method() == caller_code->method(), "wrong nmethod");
  int bci = vfst.bci();

  Events::log("patch_code @ %#x", caller_frame.pc());

  Bytecodes::Code code = Bytecode_at(caller_method->bcp_from(bci))->java_code();
  int patch_field_offset = -1;
  KlassHandle klass(klassOop(NULL));
  if (stub_id == init_check_patching_id) {
    patch_field_offset = resolve_field_return_offset(caller_method, bci, CHECK);
  } else if (stub_id == load_klass_patching_id) {
    switch (code) {
      case Bytecodes::_putstatic:
      case Bytecodes::_getstatic:
        { klass = resolve_field_return_klass(caller_method, bci, CHECK);
        }
        break;
      case Bytecodes::_new:
        { Bytecode_new* bnew = Bytecode_new_at(caller_method->bcp_from(bci));
          klass = caller_method->constants()->klass_at(bnew->index(), CHECK);
        }
        break;
      case Bytecodes::_multianewarray:
        { Bytecode_multianewarray* mna = Bytecode_multianewarray_at(caller_method->bcp_from(bci));
          klass = caller_method->constants()->klass_at(mna->index(), CHECK);
        }
        break;
      case Bytecodes::_instanceof:
        { Bytecode_instanceof* io = Bytecode_instanceof_at(caller_method->bcp_from(bci));
          klass = caller_method->constants()->klass_at(io->index(), CHECK);
        }
        break;
      case Bytecodes::_checkcast:
        { Bytecode_checkcast* cc = Bytecode_checkcast_at(caller_method->bcp_from(bci));
          klass = caller_method->constants()->klass_at(cc->index(), CHECK);
        }
        break;
      case Bytecodes::_anewarray:
        { Bytecode_anewarray* anew = Bytecode_anewarray_at(caller_method->bcp_from(bci));
          klassOop ek = caller_method->constants()->klass_at(anew->index(), CHECK);
          klass = Klass::cast(ek)->array_klass(CHECK);
        }
        break;
      default: Unimplemented();
    }
  } else {
    ShouldNotReachHere();
  }

  // Now copy code back

  { 
    MutexLocker ml_patch (Patching_lock);
    address instr_pc = caller_frame.pc() - NativeCall::instruction_size;
    if (NativeCall::is_call_at(instr_pc)) {
      // the call has not been patched yet
      NativeCall* call = nativeCall_at(instr_pc);
      // The call destination is slow case and therefore not part of the stubs
      // (stubs are only for StaticCalls)
      
      // format of buffer
      //    ....
      //    instr byte 0     <-- copy_buff
      //    instr byte 1
      //    ..
      //    instr byte n-1
      //      n
      //    ....             <-- call destination

// FIX ME: These should either be computed elsewhere or we should use Srdjan's suggestion
// to pad the byte_count to a full word on x86 as it is (and must be) on Sparc.
#ifdef SPARC
      int* byte_count = (int*) (call->destination() - 4);
      address copy_buff = call->destination() - 4 - *byte_count;
#else
      jbyte* byte_count = (jbyte*) (call->destination() - 1);
      address copy_buff = call->destination() - 1 - *byte_count;
#endif
      if (TracePatching) {
        tty->print_cr(" Patching %s at bci %d at address 0x%x  (%s)", Bytecodes::name(code), bci, 
		      instr_pc, (stub_id == init_check_patching_id) ? "field" : "klass");
        OopMap* map = caller_code->oop_map_for_return_address(caller_frame.pc(), true);
        assert(map != NULL, "null check");
        map->print();
        tty->cr();
      }
      if (stub_id == init_check_patching_id) {
        // The offset may not be correct if the class was not loaded at code generation time.
        // Set it now.
        NativeMovRegMemPatching* n_move = nativeMovRegMemPatching_at(copy_buff);
        assert(n_move->offset() == 0 || n_move->offset() == 1, "illegal uninitialized offset");
        // the offset is either 0 or wordSize
        assert(patch_field_offset >= 0, "illegal offset");
        n_move->add_offset_in_words(patch_field_offset);
      } else if (stub_id == load_klass_patching_id) {
        // patch the instruction <move reg, klass>
        NativeMovConstRegPatching* n_copy = nativeMovConstRegPatching_at(copy_buff);
        assert(n_copy->data() == 0, "illegal init value");
        assert(klass() != NULL, "klass not set");
        n_copy->set_data((int) (klass()));

        // update relocInfo to oop
        nmethod* nm = CodeCache::find_nmethod(instr_pc);
        assert(nm != NULL, "invalid nmethod_pc");

        RelocIterator iter(nm, (address)instr_pc, (address)(instr_pc + 1));
        relocInfo::change_reloc_info_for_address(&iter, (address) instr_pc, relocInfo::none, relocInfo::oop_type);
#ifdef SPARC
        { address instr_pc2 = instr_pc + 2*wordSize;
          RelocIterator iter2(nm, instr_pc2, instr_pc2 + 1);
          relocInfo::change_reloc_info_for_address(&iter2, (address) instr_pc2, relocInfo::none, relocInfo::oop_type);
        }
#endif
      } else {
        ShouldNotReachHere();
      }
      // replace instructions
      // first replace the tail, then the call
      for (int i = NativeCall::instruction_size; i < *byte_count; i++) {
        address ptr = (address)(copy_buff + i);
        int a_byte = (*ptr) & 0xFF;
        address dst = (address)(instr_pc + i);
        *(jbyte*)dst = (jbyte) a_byte;
      }
      NativeCall::replace_mt_safe(instr_pc, copy_buff);
#ifdef SPARC
      if (stub_id == load_klass_patching_id) { 
        // Factor this out!
        // update relocInfo to oop
        nmethod* nm = CodeCache::find_nmethod(instr_pc);
        assert(nm != NULL, "invalid nmethod_pc");

        RelocIterator oops(nm, instr_pc, instr_pc + 1);
        bool found = false;
        while (oops.next() && !found) {
          if (oops.type() == relocInfo::oop_type) {
            oop_Relocation* r = oops.oop_reloc();
            oop* oop_adr = r->oop_addr();
            *oop_adr = klass();
            r->fix_oop_relocation();
            found = true;
          }
        }
        assert(found, "the oop must exist!");
      }
#endif
    }
  }
}

JRT_ENTRY(void, Runtime1::move_klass_patching(JavaThread* thread))
  patch_code(thread, load_klass_patching_id, CHECK);
JRT_END


JRT_ENTRY(void, Runtime1::init_check_patching(JavaThread* thread))
  patch_code(thread, init_check_patching_id, CHECK);
JRT_END


JRT_ENTRY(address, Runtime1::bci_2_pc(JavaThread* thread, int bci, address nmethod_pc))
  nmethod* nm = CodeCache::find_nmethod(nmethod_pc);
  assert(nm != NULL, "invalid nmethod_pc");
  int offset = nm->pc_offset_for_bci(bci);
  assert(offset >= 0, "wrong bci -> pc offset mapping");
  address res = nm->instructions_begin() + offset;
  return res;
JRT_END


JRT_LEAF(void, Runtime1::trace_method_entry(jint v1, jint v2))
  tty->print_cr("Entering a method 0x%x 0x%x", v1, v2);
JRT_END


JRT_LEAF(void, Runtime1::trace_method_exit(jint value))
  Unimplemented();
JRT_END


// fast and direct copy of arrays; returning -1, means that an exception may be thrown
// and we did not copy anything
JRT_LEAF(int, Runtime1::arraycopy(int length, int dst_pos, oop dst, int src_pos, oop src))
  enum {
    ac_failed = -1, // arraycopy failed
    ac_ok = 0       // arraycopy succeeded
  };

  if (!dst->is_array() || !src->is_array()) return ac_failed;
  assert(dst->is_array() && src->is_array(), "wrong objects");
  assert (src != NULL && dst != NULL &&  src_pos >= 0 && dst_pos >= 0 && length >= 0, "prechecks failed");
  assert((unsigned int) arrayOop(src)->length() >= (unsigned int)src_pos + (unsigned int)length, "precheck failed");
  assert((unsigned int) arrayOop(dst)->length() >= (unsigned int)dst_pos + (unsigned int)length, "precheck failed");

  if (length == 0) return ac_ok;
  if (src->is_typeArray()) {
    const klassOop klass_oop = src->klass();
    if (klass_oop != dst->klass()) return ac_failed;
    typeArrayKlass* klass = typeArrayKlass::cast(klass_oop);
    const int sc  = klass->scale();
    const int ihs = klass->array_header_in_bytes() / wordSize;
    char* src_addr = (char*) ((oop*)src + ihs) + (src_pos * sc);
    char* dst_addr = (char*) ((oop*)dst + ihs) + (dst_pos * sc);
    // Potential problem: memmove is not guaranteed to be word atomic
    // Revisit in Merlin
    memmove(dst_addr, src_addr, length * sc);
    return ac_ok;
  } else {
    assert(src->is_objArray(), "what array type is it else?");
    oop* src_addr = objArrayOop(src)->obj_at_addr(src_pos);
    oop* dst_addr = objArrayOop(dst)->obj_at_addr(dst_pos);
    if (src == dst) {
      // same object, no check
      Memory::copy_oops_overlapping_atomic(src_addr, dst_addr, length);
      RememberedSet::record_array_store(dst_addr, dst_addr + length);
      return ac_ok;
    } else {
      klassOop bound = objArrayKlass::cast(dst->klass())->element_klass();
      klassOop stype = objArrayKlass::cast(src->klass())->element_klass();
      if (stype == bound || Klass::cast(stype)->is_subtype_of(bound)) {
        // elements are guaranteed to be subtypes, so no check necessary
        Memory::copy_oops_overlapping_atomic(src_addr, dst_addr, length);
        RememberedSet::record_array_store(dst_addr, dst_addr + length);
        return ac_ok;
      }
    }
  }
  return ac_failed;
JRT_END


#ifndef PRODUCT
void Runtime1::print_statistics() {
  tty->print_cr("C1 Runtime statistics:");
  tty->print_cr("    _resolve_invoke_cnt:      %d", _resolve_invoke_cnt);
  tty->print_cr("    _handle_wrong_method_cnt: %d", _handle_wrong_method_cnt);
  tty->print_cr("    _ic_miss_cnt:             %d", _ic_miss_cnt);
  tty->cr();
}
#endif // PRODUCT
