// @(#)c1_Runtime1.hpp	1.76 01/02/09 14:44:31
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


// A StubAssembler is a MacroAssembler w/ extra functionality for runtime
// stubs. Currently it 'knows' some stub info. Eventually, the information
// may be set automatically or can be asserted when using specialised
// StubAssembler functions.

class StubAssembler: public C1_MacroAssembler {
 private:
  const char* _name;
  bool  _must_gc_arguments;

 public:
  // creation
  StubAssembler(CodeBuffer* code);
  void set_info(const char* name, bool must_gc_arguments);

  // accessors
  const char* name() const                       { return _name; }
  bool  must_gc_arguments() const                { return _must_gc_arguments; }

  // runtime calls (return offset of call to be used by GC map)
  int call_RT(Register oop_result1, Register oop_result2, address entry, int args_size = 0);
  int call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1);
  int call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1, Register arg2);
  int call_RT(Register oop_result1, Register oop_result2, address entry, Register arg1, Register arg2, Register arg3);
};


// iEntries is a holder for interpreter entry points entered from
// compiled code. Before invoking the interpreter they check if
// the callee has been compiled in the meantime (besides inline
// cache check, if necessary). If the callee has been compiled,
// the call site needs to be updated.

class iEntries VALUE_OBJ_CLASS_SPEC {
 private:
  address _base;
  int _static_call_offset;
  int _optimized_call_offset;
  int _virtual_call_offset;
  int _virtual_final_call_offset;

 public:
  // creation
  iEntries();
  iEntries(int static_call_offset, int optimized_call_offset, int virtual_call_offset, int virtual_final_call_offset);
  void set_base(address base);

  // accessors
  address static_call_entry() const              { return _base + _static_call_offset; }
  address optimized_call_entry() const           { return _base + _optimized_call_offset; }
  address virtual_call_entry() const             { return _base + _virtual_call_offset;  }
  address virtual_final_call_entry() const       { return _base + _virtual_final_call_offset; }
};


// The Runtime1 holds all assembly stubs and VM
// runtime routines needed by code code generated
// by the Compiler1.

class Runtime1: public AllStatic {
  friend class VMStructs;
 public:
  enum StubID {
    new_instance_id,
    new_type_array_id,
    new_object_array_id,
    new_multi_array_id,
    resolve_invokestatic_id,
    resolve_invoke_id,
    handle_ic_miss_id,
    handle_wrong_static_method_id,
    handle_wrong_method_id,
    throw_abstract_method_error_id,
    must_compile_method_id,
    entering_non_entrant_id,
    arithmetic_call_id,
    range_check_failed_id,
    throw_div0_exception_id,
    throw_null_exception_id,
    throw_exception_id,
    handle_exception_id,
    unwind_exception_id,
    store_check_id,
    checkcast_id,
    instanceof_id,
    monitorenter_id,
    monitorexit_id,
    implicit_null_exception_id,
    implicit_div0_exception_id,
    stack_overflow_exception_id,
    interpreter_entries_id,
    init_check_patching_id,
    load_klass_patching_id,
    osr_frame_return_id,
    osr_unwind_exception_id,
    illegal_instruction_handler_id,
    address_bci_2_pc_id,
    safepoint_block_id,
    jvmpi_method_entry,
    jvmpi_method_exit,
    number_of_ids
  };

 private:
  static bool      _is_initialized;
  static CodeBlob* _blobs[number_of_ids];
  static iEntries  _ientries[AbstractInterpreter::number_of_method_entries];

  // statistics
  static int _resolve_invoke_cnt;
  static int _handle_wrong_method_cnt;
  static int _ic_miss_cnt;

  static void generate_exception_throw (StubAssembler* sasm, const char* msg, address runtime_call);
  static void generate_blob_for(StubID id);
  static OopMapSet* generate_code_for(StubID id, StubAssembler* masm);
  static OopMapSet* generate_patching(StubAssembler* sasm, address target);
  static OopMapSet* generate_store_check(StubAssembler* sasm, address target);
  static OopMapSet* generate_illegal_instruction_handler(StubAssembler* sasm, address target);

  static bool is_initialized()                   { return _is_initialized; }
  static void initialize();

  // runtime entry points
  static void new_instance    (JavaThread* thread, klassOop klass);
  static void new_type_array  (JavaThread* thread, BasicType elt_type, jint length);
  static void new_object_array(JavaThread* thread, klassOop klass, jint length);
  static void new_multi_array (JavaThread* thread, klassOop klass, int rank, jint* dims);

  static address resolve_invoke     (JavaThread* thread, oop recv);
  static address handle_ic_miss     (JavaThread* thread, oop recv);
  static address handle_wrong_method(JavaThread* thread, oop recv);
  static void unimplemented_entry   (JavaThread* thread, StubID id);

  static void store_check(JavaThread* thread, oop array, oop value);
  static void checkcast  (JavaThread* thread, oop object_oop, oop test_klass_oop);
  static int  instanceof (JavaThread* thread, oop object_oop, oop test_klass_oop);

  static void throw_exception(JavaThread* thread, oop exception);
  static address exception_handler_for_pc(JavaThread* thread, oop exception, address pc);
  static address return_address_for_exception();

  static address compute_exception_pc(int pc_offset, address pc_addr);
  static void throw_range_check_exception(JavaThread* thread);
  static void throw_div0_exception(JavaThread* thread);
  static void throw_null_exception(JavaThread* thread);
  static void throw_null_exception_at_call(JavaThread* thread);
  static void throw_stack_overflow(JavaThread* thread);
  static void throw_abstract_method_error(JavaThread* thread);

  static void monitorenter(JavaThread* thread, oop obj, BasicLock* lock);
  static void monitorexit (JavaThread* thread, oop obj, BasicLock* lock);

  static void init_check_patching(JavaThread* thread);
  static void move_klass_patching(JavaThread* thread);

  static address bci_2_pc(JavaThread* thread, int bci, address nmethod_pc);

  static void patch_code(JavaThread* thread, StubID stub_id, TRAPS);

  static void prepare_interpreter_call(JavaThread* thread, methodOop method);

 public:
  // stubs
  static CodeBlob* blob_for (StubID id);
  static address   entry_for(StubID id)          { return blob_for(id)->instructions_begin(); }

  // osr support
  static bool returns_to_osr_adapter(address pc) { return entry_for(osr_frame_return_id) == pc; }

  // interpreter entry points for compiled code
  static iEntries* ientries_for(methodHandle m);

  // method tracing
  static void trace_method_entry(jint v1, jint v2);
  static void trace_method_exit(jint value);

  // directly accessible leaf routine
  static int  arraycopy(int length, int dst_pos, oop dst, int src_pos, oop src);

  static void print_statistics()                 PRODUCT_RETURN;
};


class GC_Support: public AllStatic {
 public:
  static void  preserve_callee_argument_oops(const frame fr, symbolHandle signature, bool is_static, void f(oop*));
  static jint* get_argument_addr_at         (const frame fr, ArgumentLocation location);
  static jint* get_stack_argument_addr_at   (const frame fr, int offset_from_sp_in_words);
  static jint* get_register_argument_addr_at(const frame fr, Register r);
};
