/*
 * @(#)c1_ScanBlocks.cpp	1.22 01/02/09 14:44:32
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


# include "incls/_precompiled.incl"
# include "incls/_c1_ScanBlocks.cpp.incl"


inline void inc_ga_count(GrowableArray<int>* ga, int index, int count) {
  int cnt = ga->at_grow(index, 0) + count;
  ga->at_put(index, cnt);
}


inline int ga_count_at(GrowableArray<int>* ga, int index) {
  if (index < ga->length()) {
    return ga->at(index);
  } else {
    return 0;
  }
}


inline bool ga_is_zero(GrowableArray<int>* ga, int index) {
  return ga_count_at(ga, index) == 0;
}


// Increment count in _xxx_acces_count
void ScanBlocks::accumulate_access(int index, ValueTag tag, int count) {
  switch (tag) {
  case intTag:     inc_ga_count(    _int_access_count, index, count);  break;
  case longTag:    inc_ga_count(   _long_access_count, index, count);  break;
  case floatTag:   inc_ga_count(  _float_access_count, index, count);  break;
  case doubleTag:  inc_ga_count( _double_access_count, index, count);  break;
  case objectTag:  inc_ga_count(    _obj_access_count, index, count);  break;
  case addressTag: inc_ga_count(_address_access_count, index, count);  break;
  default: 
    ShouldNotReachHere();
    break;
  }
}


// Here we set all the flags
void ScanBlocks::scan_block(BlockBegin* block, ScanResult* desc) {
  for (Instruction* n = block; n != NULL; n = n->next()) {
    ValueTag tag = n->type()->tag();
    if (tag == floatTag) desc->set_has_floats(true);
    else if (tag == doubleTag) desc->set_has_doubles(true);
    if (n->as_StateSplit() != NULL) {
      if (n->as_Invoke() != NULL || n->as_Jsr() != NULL) {
        if (n->as_Jsr() != NULL) desc->set_has_jsr(true);
        desc->set_has_calls(true);
      }
      if (n->as_NewArray() || n->as_NewInstance() || n->as_TypeCheck() || n->as_AccessMonitor()) {
        desc->set_has_slow_cases(true);
      }  else if(n->as_Intrinsic() != NULL) {
        Intrinsic* i = n->as_Intrinsic();
        if (i->id() == methodOopDesc::_arraycopy) desc->set_has_slow_cases(true);
      }
    } else if (n->as_AccessField() != NULL) {
      AccessField* af = n->as_AccessField();
      if (!af->is_initialized() || !af->is_loaded()) desc->set_has_class_init(true);
    } else if (n->as_AccessLocal() != NULL) {
      AccessLocal* local = n->as_AccessLocal();
      int use_count = local->as_StoreLocal() != NULL ? 1 : n->use_count();
      ValueType* type = local->type();
      accumulate_access(local->index(), type->tag(), use_count);
      if (type->is_double_word()) {
        // for local caching, ensure detection of words that hold values of different types
        accumulate_access(local->index() + 1, type->tag(), use_count);
      }
      if (local->index() == 0 && local->as_StoreLocal() != NULL) desc->set_has_store_0(true);
    } else if(n->as_StoreIndexed() != NULL) {
      StoreIndexed* si = n->as_StoreIndexed();
      if (si->type()->tag() == objectTag) desc->set_has_slow_cases(true);
    }
#ifdef SPARC
    else {
      if (n->as_Convert() != NULL) {
        Convert* conv = n->as_Convert();
        switch (conv->op()) {
          case Bytecodes::_l2f: 
          case Bytecodes::_l2d: 
          case Bytecodes::_f2l: 
          case Bytecodes::_f2i: 
          case Bytecodes::_d2l: 
          case Bytecodes::_d2i: { desc->set_has_calls(true); break; }
        }
      } else if (n->as_ArithmeticOp() != NULL) {
        ArithmeticOp* arith = n->as_ArithmeticOp();
        switch (arith->op()) {
          case Bytecodes::_lrem:  
          case Bytecodes::_ldiv:  
          case Bytecodes::_lmul: 
          case Bytecodes::_drem: 
          case Bytecodes::_frem: { desc->set_has_calls(true); break; }
        }
      }
    }
#endif
  }
}


ScanBlocks::ScanBlocks(GrowableArray<BlockBegin*>* blocks) {
  _blocks = blocks;
  _int_access_count     = new GrowableArray<int>();
  _long_access_count    = new GrowableArray<int>();
  _float_access_count   = new GrowableArray<int>();
  _double_access_count  = new GrowableArray<int>();
  _obj_access_count     = new GrowableArray<int>();
  _address_access_count = new GrowableArray<int>();
}


void ScanBlocks::scan(ScanResult* desc) {
  int n = _blocks->length() - 1;
  for (; n >= 0; n--) {
    scan_block(_blocks->at(n), desc);
  }
}


// Sort int locals by their usage
GrowableArray<ALocal*>* ScanBlocks::most_used_int_locals() {
  ShouldNotReachHere();
  return NULL;
}


int ScanBlocks::int_count_at(int index) const {
  return ga_count_at(_int_access_count, index);
}


int ScanBlocks::long_count_at(int index) const {
  return ga_count_at(_long_access_count, index);
}


int ScanBlocks::float_count_at(int index) const {
  return ga_count_at(_float_access_count, index);
}


int ScanBlocks::double_count_at(int index) const {
  return ga_count_at(_double_access_count, index);
}


int ScanBlocks::obj_count_at(int index) const {
  return ga_count_at(_obj_access_count, index);
}


int ScanBlocks::address_count_at(int index) const {
  return ga_count_at(_address_access_count, index);
}


static int sort_ALocal(ALocal** a, ALocal** b) {
  return (*b)->access_count() - (*a)->access_count();
}


// Sorts all CPU locals by their usage; access frequency is stored separately for
// objects and integers
GrowableArray<ALocal*>* ScanBlocks::most_used_locals() {
  // check that no local is object and integer
  int lng = max(_int_access_count->length(), _obj_access_count->length());
  GrowableArray<ALocal*>* res = new GrowableArray<ALocal*>();
  // build array of ALocal's if they are exclusively obj or int's
  for (int i = 0; i < lng; i++) {
    int icnt = int_count_at(i);
    int ocnt = obj_count_at(i);
    ALocal* alocal = NULL;
    if (icnt > 0 && is_int_only(i)) {
      alocal = new ALocal(i, false, icnt);
    } else if (ocnt > 0 && is_obj_only(i)) {
      alocal = new ALocal(i, true, ocnt);
    }
    if (alocal != NULL) res->append(alocal);
  }

  res->sort(sort_ALocal);
  return res;
}


bool ScanBlocks::is_int_only(int index) const {
  return ga_is_zero(_long_access_count, index) && ga_is_zero(_float_access_count, index) &&
    ga_is_zero(_double_access_count, index) && ga_is_zero(_obj_access_count, index) &&
    ga_is_zero(_address_access_count, index);
}


bool ScanBlocks::is_long_only(int index) const {
  return ga_is_zero(_int_access_count, index) && ga_is_zero(_float_access_count, index) &&
    ga_is_zero(_double_access_count, index) && ga_is_zero(_obj_access_count, index) &&
    ga_is_zero(_address_access_count, index);
}


bool ScanBlocks::is_float_only(int index) const {
  return ga_is_zero(_int_access_count, index) && ga_is_zero(_long_access_count, index) &&
    ga_is_zero(_double_access_count, index) && ga_is_zero(_obj_access_count, index) &&
    ga_is_zero(_address_access_count, index);
}


bool ScanBlocks::is_double_only(int index) const {
  return ga_is_zero(_int_access_count, index) && ga_is_zero(_long_access_count, index) &&
    ga_is_zero(_float_access_count, index) && ga_is_zero(_obj_access_count, index) &&
    ga_is_zero(_address_access_count, index);
}


bool ScanBlocks::is_obj_only(int index) const {
  return ga_is_zero(_int_access_count, index) && ga_is_zero(_long_access_count, index) &&
    ga_is_zero(_float_access_count, index) && ga_is_zero(_double_access_count, index) &&
    ga_is_zero(_address_access_count, index);
}


bool ScanBlocks::is_address_only(int index) const {
  return ga_is_zero(_int_access_count, index) && ga_is_zero(_long_access_count, index) &&
    ga_is_zero(_float_access_count, index) && ga_is_zero(_double_access_count, index) &&
    ga_is_zero(_obj_access_count, index);
}


#ifndef PRODUCT
void ScanBlocks::print(ScanResult* r) {
  tty->print("Scan block analysis: ");
  if (r != NULL) {
    if (r->has_calls()) tty->print(" calls "); 
    if (r->has_slow_cases()) tty->print(" slow-case "); 
    if (r->has_floats()) tty->print(" floats "); 
    if (r->has_doubles()) tty->print(" doubles "); 
    if (r->has_class_init()) tty->print(" class-init ");
  }
  tty->print("; ");
  tty->print(" INT[");
  int i;
  for (i = 0; i < _int_access_count->length(); i++) {
    tty->print("L%d:%d ", i, _int_access_count->at(i));
  }
  tty->print("] OBJ[");
  for (i = 0; i < _obj_access_count->length(); i++) {
    tty->print("L%d:%d ", i, _obj_access_count->at(i));
  }
  tty->print("]");
  tty->cr();
}
#endif

