// @(#)c1_ScanBlocks.hpp	1.10 00/02/04 11:13:53
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


class BlockBegin;

class ALocal: public ResourceObj {
 private:
  int  _index;
  bool _is_oop;
  int  _access_count;
 public:
  ALocal(int index, bool is_oop, int access_count): _index(index), _is_oop(is_oop), _access_count(access_count) {}
  int  index() const  { return _index;  }
  bool is_oop() const { return _is_oop; }
  int  access_count() const { return _access_count; }
};

class ScanResult: public ResourceObj {
 private:
  bool _has_calls;
  bool _has_slow_cases;
  bool _has_doubles;
  bool _has_floats;
  bool _has_class_init;
  bool _has_store_0; // has stores to local 0
  bool _has_jsr;
 public:
  ScanResult(): 
       _has_calls(false)
     , _has_slow_cases(false)
     , _has_doubles(false)
     , _has_floats(false)
     , _has_class_init(false)
     , _has_store_0(false) 
     , _has_jsr(false)
       {}
  bool has_calls() const      { return _has_calls;      }
  bool has_slow_cases() const { return _has_slow_cases; }
  bool has_doubles() const    { return _has_doubles;    }
  bool has_floats() const     { return _has_floats;     }
  bool has_class_init() const { return _has_class_init; }
  bool has_store_0() const    { return _has_store_0;    }
  bool has_jsr() const        { return _has_jsr;        }

  void set_has_calls(bool f)       { _has_calls = f;      }
  void set_has_slow_cases(bool f)  { _has_slow_cases = f; }
  void set_has_doubles(bool f)     { _has_doubles = f;    }
  void set_has_floats(bool f)      { _has_floats = f;     }
  void set_has_class_init(bool f)  { _has_class_init = f; }
  void set_has_store_0(bool f)     { _has_store_0 = f;    }
  void set_has_jsr(bool f)         { _has_jsr = f;        }

  bool can_cache_locals() const { return !has_calls() && !has_class_init() && !has_slow_cases(); }

};


// Analyzes specified blocks for various properties (see fields)
class ScanBlocks: public StackObj {
 private:
  GrowableArray<BlockBegin*>* _blocks;
  ScanResult* _desc;
  GrowableArray<int>* _int_access_count;
  GrowableArray<int>* _long_access_count;
  GrowableArray<int>* _float_access_count;
  GrowableArray<int>* _double_access_count;
  GrowableArray<int>* _obj_access_count;
  GrowableArray<int>* _address_access_count;

  void accumulate_access(int index, ValueTag tag, int count);
  void scan_block(BlockBegin* b, ScanResult* desc);

  int int_count_at     (int ix) const;
  int long_count_at    (int ix) const;
  int float_count_at   (int ix) const;
  int double_count_at  (int ix) const;
  int obj_count_at     (int ix) const;
  int address_count_at (int ix) const;

 public:
  ScanBlocks(GrowableArray<BlockBegin*>* blocks);

  void scan(ScanResult* desc);

  // return sorted list of locals
  GrowableArray<ALocal*>* most_used_int_locals();
  GrowableArray<ALocal*>* most_used_locals();


  bool is_int_only     (int index) const;
  bool is_long_only    (int index) const;
  bool is_float_only   (int index) const;
  bool is_double_only  (int index) const;
  bool is_obj_only     (int index) const;
  bool is_address_only (int index) const;


  void print(ScanResult* r) PRODUCT_RETURN;
};
