/*
 * @(#)ciConstant.hpp	1.3 99/12/14 11:14:56
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// ciConstant
//
// This class represents a constant value.
class ciConstant VALUE_OBJ_CLASS_SPEC {
private:
  CI_PACKAGE_ACCESS

  BasicType _type;
  union {
    jint      _int;
    jlong     _long;
    jint      _long_half[2];
    jfloat    _float;
    jdouble   _double;
    ciObject* _object;
  } _value;

  // Implementation of the print method.
  void print_impl();


  ciConstant() {
    _type = T_ILLEGAL; _value._long = -1;
  }
  ciConstant(BasicType type, jint value) {
    assert(type != T_LONG && type != T_DOUBLE && type != T_FLOAT,
           "using the wrong ciConstant constructor");
    _type = type; _value._int = value;
  }
  ciConstant(jlong value) { 
    _type = T_LONG; _value._long = value;
  }
  ciConstant(jfloat value) { 
    _type = T_FLOAT; _value._float = value;
  }
  ciConstant(jdouble value) { 
    _type = T_DOUBLE; _value._double = value;
  }
  ciConstant(BasicType type, ciObject* p) { 
    _type = type, _value._object = p;
  }

public:
  BasicType type() const { return _type; }

  jboolean  as_boolean() {
    assert(type() == T_BOOLEAN, "wrong type");
    return (jboolean)_value._int;
  }
  jchar     as_char() {
    assert(type() == T_CHAR, "wrong type");
    return (jchar)_value._int;
  }
  jbyte     as_byte() {
    assert(type() == T_BYTE, "wrong type");
    return (jbyte)_value._int;
  }
  jshort    as_short() {
    assert(type() == T_SHORT, "wrong type");
    return (jshort)_value._int;
  }
  jint      as_int() {
    assert(type() == T_BOOLEAN || type() == T_CHAR  ||
           type() == T_BYTE    || type() == T_SHORT ||
           type() == T_INT, "wrong type");
    return _value._int;
  }
  jlong     as_long() {
    assert(type() == T_LONG, "wrong type");
    return _value._long;
  }

  // These low/high conversions are machine specific and should not be used
  // Use low/high in globalDefinitions.hpp to convert
  jint      as_long_low() {
    assert(type() == T_LONG, "wrong type");
    return _value._long_half[0];
  }
  jint      as_long_high() {
    assert(type() == T_LONG, "wrong type");
    return _value._long_half[1];
  }

  jfloat    as_float() {
    assert(type() == T_FLOAT, "wrong type");
    return _value._float;
  }
  jdouble   as_double() {
    assert(type() == T_DOUBLE, "wrong type");
    return _value._double;
  }
  ciObject* as_object() const {
    assert(type() == T_OBJECT || type() == T_ARRAY, "wrong type");
    return _value._object;
  }

  // Debugging output
  void print();
};

