/*
 * @(#)ciEnv.cpp	1.27 00/07/21 09:50:59
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_ciEnv.cpp.incl"

// ciEnv
//
// This class is the top level broker for requests from the compiler
// to the VM.

ciObject*              ciEnv::_null_object_instance;
ciMethodKlass*         ciEnv::_method_klass_instance;
ciSymbolKlass*         ciEnv::_symbol_klass_instance;
ciKlassKlass*          ciEnv::_klass_klass_instance;
ciInstanceKlassKlass*  ciEnv::_instance_klass_klass_instance;
ciTypeArrayKlassKlass* ciEnv::_type_array_klass_klass_instance;
ciObjArrayKlassKlass*  ciEnv::_obj_array_klass_klass_instance;
 
ciInstanceKlass* ciEnv::_ArrayStoreException;
ciInstanceKlass* ciEnv::_Class;
ciInstanceKlass* ciEnv::_ClassCastException;
ciInstanceKlass* ciEnv::_Object;
ciInstanceKlass* ciEnv::_OutOfMemoryError;
ciInstanceKlass* ciEnv::_String;

// ------------------------------------------------------------------
// ciEnv::ciEnv
ciEnv::ciEnv(JNIEnv* jni_env, int class_load_counter, bool break_at_compile) {
  VM_ENTRY_MARK;

  CompilerThread* current_thread = (CompilerThread*)JavaThread::current();
  current_thread->set_data(NULL);
  current_thread->set_env(this);
  _invalidated = false;
  _success = false;
  _break_at_compile = break_at_compile;

  _jni_env = jni_env;
  _class_load_counter = class_load_counter;
  
  // Temporary buffer for creating symbols and such.
  _name_buffer = NULL;
  _name_buffer_len = 0;

  _arena = new Arena();
  _factory = new (_arena) ciObjectFactory(_arena, 128);

  // Preload commonly referenced system ciObjects.

  // During VM initialization, these instances have not yet been created.
  // Assertions ensure that these instances are not accessed before
  // their initialization.

  oop o = Universe::null_ptr_exception_instance();
  if (o != NULL) {
    _NullPointerException_instance =
      get_object(o)->as_instance();
  } else {
    _NullPointerException_instance = NULL;
  }
  o = Universe::arithmetic_exception_instance();
  if (o != NULL) {
    _ArithmeticException_instance =
      get_object(o)->as_instance();
  } else {
    _ArithmeticException_instance = NULL;
  }
}

ciEnv::~ciEnv() {
  delete _arena;
}

// ------------------------------------------------------------------
// ciEnv::get_method_from_handle
ciMethod* ciEnv::get_method_from_handle(jobject method) {
  VM_ENTRY_MARK;
  return get_object(JNIHandles::resolve(method))->as_method();
}

// ------------------------------------------------------------------
// ciEnv::check_klass_accessiblity
//
// Note: the logic of this method should mirror the logic of
// constantPoolOopDesc::verify_constant_pool_resolve.
bool ciEnv::check_klass_accessibility(ciKlass* accessing_klass,
				      klassOop resolved_klass) {
  if (accessing_klass == NULL || !accessing_klass->is_loaded()) {
    return true;
  }
  if (accessing_klass->is_obj_array()) {
    accessing_klass = accessing_klass->as_obj_array_klass()->base_element_klass();
  }
  if (!accessing_klass->is_instance_klass()) {
    return true;
  }

  if (resolved_klass->klass_part()->oop_is_objArray()) {
    // Find the element klass, if this is an array.
    resolved_klass = objArrayKlass::cast(resolved_klass)->bottom_klass();
  }
  if (resolved_klass->klass_part()->oop_is_instance()) {
    return Reflection::verify_class_access(accessing_klass->get_klassOop(),
					   resolved_klass,
					   true);
  }
  return true;
}

// ------------------------------------------------------------------
// ciEnv::get_klass_by_name_impl
ciKlass* ciEnv::get_klass_by_name_impl(ciKlass* accessing_klass,
                                       ciSymbol* name) {
  EXCEPTION_CONTEXT;
  ASSERT_IN_VM;
  symbolOop sym = name->get_symbolOop();
  if (sym->byte_at(0) == 'L' &&
    sym->byte_at(sym->utf8_length()-1) == ';') {
    // This is a name from a signature.  Strip off the trimmings.
    sym = oopFactory::new_symbol(sym->as_utf8()+1,
                                 sym->utf8_length()-2,
                                 KILL_COMPILE_ON_ANY);
  }
  oop loader = NULL;
  oop domain = NULL;
  if (accessing_klass != NULL) {
    loader = accessing_klass->loader();
    domain = accessing_klass->protection_domain();
  }
  klassOop klass =
    SystemDictionary::find_instance_or_array_klass(sym, loader, domain, KILL_COMPILE_ON_ANY);
  if (klass != NULL) {
    // We've found the klassOop.
    if (check_klass_accessibility(accessing_klass, klass)) {
      return get_object(klass)->as_klass();
    }
  } else if( sym->byte_at(0) == '[' ) {	// Failed!  Check for array-ness
    // We have an unloaded array.  Try to make it.
    // First compute the element name
    symbolOop elem_sym = oopFactory::new_symbol( sym->as_utf8()+1, 
						 sym->utf8_length()-1, 
						 KILL_COMPILE_ON_ANY);
    ciSymbol *ci_sym = get_object(elem_sym)->as_symbol();
    // Get element ciKlass recursively
    ciKlass *elem_klass = get_klass_by_name_impl( accessing_klass, ci_sym );
    // Now make an array for it
    return ciObjArrayKlass::make_impl( elem_klass );
  }
  
  // Make a representative for this unloaded klass.
  return get_unloaded_klass(accessing_klass, name);
}

// ------------------------------------------------------------------
// ciEnv::get_klass_by_name
ciKlass* ciEnv::get_klass_by_name(ciKlass* accessing_klass,
                                  ciSymbol* klass_name) {
  GUARDED_VM_ENTRY(return get_klass_by_name_impl(accessing_klass,
                                                 klass_name);)
}

// ------------------------------------------------------------------
// ciEnv::get_klass_by_index_impl
//
// Implementation of get_klass_by_index.
ciKlass* ciEnv::get_klass_by_index_impl(ciInstanceKlass* accessor,
                                        int index) {
  assert(accessor->get_instanceKlass()->is_linked(), "must be linked before accessing constant pool");
  constantPoolHandle cpool = accessor->get_instanceKlass()->constants();
  klassOop klass = constantPoolOopDesc::klass_at_if_loaded(cpool, index);
  symbolHandle klass_name;
  if (klass == NULL) {
    EXCEPTION_CONTEXT;
    // The klass has not been inserted into the constant pool.
    // Try to look it up by name.
    {
      // We have to lock the cpool to keep the oop from being resolved
      // while we are accessing it.
      ObjectLocker ol(cpool(), KILL_COMPILE_ON_ANY);
      
      constantTag tag = cpool->tag_at(index);
      if (tag.is_klass()) {
	// The klass has been inserted into the constant pool
	// very recently.
	klass = cpool->resolved_klass_at(index);
      } else if (tag.is_symbol()) {
	klass_name = cpool->symbol_at(index);
      } else {
	assert(cpool->tag_at(index).is_unresolved_klass(), "wrong tag");
	klass_name = cpool->unresolved_klass_at(index);
      }
    }
  }

  if (klass == NULL) {
    return get_klass_by_name_impl(accessor,
                                  get_object(klass_name())->as_symbol());
  } else {
    return get_object(klass)->as_klass();
  }
}

// ------------------------------------------------------------------
// ciEnv::get_klass_by_index
//
// Get a klass from the constant pool.
ciKlass* ciEnv::get_klass_by_index(ciInstanceKlass* accessor,
                                   int index) {
  GUARDED_VM_ENTRY(return get_klass_by_index_impl(accessor, index);)
}

// ------------------------------------------------------------------
// ciEnv::get_constant_by_index_impl
//
// Implementation of get_constant_by_index().
ciConstant ciEnv::get_constant_by_index_impl(instanceKlass* accessor,
					     int index) {
  EXCEPTION_CONTEXT;
  assert(accessor->is_linked(), "must be linked before accessing constant pool");
  constantPoolOop cpool = accessor->constants();
  constantTag tag = cpool->tag_at(index);
  if (tag.is_int()) {
    return ciConstant(T_INT, (jint)cpool->int_at(index));
  } else if (tag.is_long()) {
    return ciConstant((jlong)cpool->long_at(index));
  } else if (tag.is_float()) {
    return ciConstant((jfloat)cpool->float_at(index));
  } else if (tag.is_double()) {
    return ciConstant((jdouble)cpool->double_at(index));
  } else if (tag.is_string() || tag.is_unresolved_string()) {
    oop string = cpool->string_at(index, KILL_COMPILE_ON_ANY);
    ciObject* constant = get_object(string);
    assert (constant->is_instance(), "must be an instance, or not? ");
    return ciConstant(T_OBJECT, constant);
  } else {
    ShouldNotReachHere();
    return ciConstant();
  }
}

// ------------------------------------------------------------------
// ciEnv::is_unresolved_string_impl
//
// Implementation of is_unresolved_string().
bool ciEnv::is_unresolved_string_impl(instanceKlass* accessor, int index) const {
  EXCEPTION_CONTEXT;
  assert(accessor->is_linked(), "must be linked before accessing constant pool");
  constantPoolOop cpool = accessor->constants();
  constantTag tag = cpool->tag_at(index);
  return tag.is_unresolved_string();
}

// ------------------------------------------------------------------
// ciEnv::get_constant_by_index
//
// Pull a constant out of the constant pool.  How appropriate.
//
// Implementation note: this query is currently in no way cached.
ciConstant ciEnv::get_constant_by_index(ciInstanceKlass* accessor,
					int index) {
  GUARDED_VM_ENTRY(return get_constant_by_index_impl(accessor->get_instanceKlass(),
                                                     index); )
}

// ------------------------------------------------------------------
// ciEnv::is_unresolved_string
//
// Check constant pool
//
// Implementation note: this query is currently in no way cached.
bool ciEnv::is_unresolved_string(ciInstanceKlass* accessor,
					int index) const {
  GUARDED_VM_ENTRY(return is_unresolved_string_impl(accessor->get_instanceKlass(), index); )
}

// ------------------------------------------------------------------
// ciEnv::get_field_by_index_impl
//
// Implementation of get_field_by_index.
//
// Implementation note: the results of field lookups are cached
// in the accessor klass.
ciField* ciEnv::get_field_by_index_impl(ciInstanceKlass* accessor,
					int index) {
  ciConstantPoolCache* cache = accessor->field_cache();
  if (cache == NULL) {
    ciField* field = new (arena()) ciField(accessor, index);
    return field;
  } else {
    ciField* field = (ciField*)cache->get(index);
    if (field == NULL) {
      field = new (arena()) ciField(accessor, index);
      cache->insert(index, field);
    }
    return field;
  }
}

// ------------------------------------------------------------------
// ciEnv::get_field_by_index
//
// Get a field by index from a klass's constant pool.
ciField* ciEnv::get_field_by_index(ciInstanceKlass* accessor,
				   int index) {
  GUARDED_VM_ENTRY(return get_field_by_index_impl(accessor, index);)
}

// ------------------------------------------------------------------
// ciEnv::lookup_method
//
// Perform an appropriate method lookup based on accessor, holder,
// name, signature, and bytecode.
methodOop ciEnv::lookup_method(instanceKlass*  accessor,
			       instanceKlass*  holder,
			       symbolOop       name,
			       symbolOop       sig,
			       Bytecodes::Code bc) {
  EXCEPTION_CONTEXT;
  LinkResolver::check_klass_accessability(accessor, holder, KILL_COMPILE_ON_FATAL_(NULL));
  methodHandle dest_method;
  switch (bc) {
  case Bytecodes::_invokestatic:
    dest_method = 
      LinkResolver::resolve_static_call_or_null(holder, name, sig, accessor); 
    break;
  case Bytecodes::_invokespecial:
    dest_method = 
      LinkResolver::resolve_special_call_or_null(holder, name, sig, accessor); 
    break;
  case Bytecodes::_invokeinterface: 
    dest_method =
      LinkResolver::linktime_resolve_interface_method_or_null(holder, name, sig,
							      accessor, true);
    break;
  case Bytecodes::_invokevirtual:
    dest_method = 
      LinkResolver::linktime_resolve_virtual_method_or_null(holder, name, sig,
							    accessor, true);
    break;
  default: ShouldNotReachHere();
  }

  return dest_method();
}


// ------------------------------------------------------------------
// ciEnv::get_method_by_index_impl
ciMethod* ciEnv::get_method_by_index_impl(ciInstanceKlass* accessor,
                                     int index, Bytecodes::Code bc) {
  // Get the method's declared holder.
                       
  assert(accessor->get_instanceKlass()->is_linked(), "must be linked before accessing constant pool");
  constantPoolHandle cpool = accessor->get_instanceKlass()->constants();
  int holder_index = cpool->klass_ref_index_at(index);
  ciInstanceKlass* declared_holder =
    get_klass_by_index_impl(accessor, holder_index)->as_instance_klass();

  // Get the method's name and signature.
  int nt_index = cpool->name_and_type_ref_index_at(index);
  int sig_index = cpool->signature_ref_index_at(nt_index);
  symbolOop name_sym = cpool->name_ref_at(index);
  symbolOop sig_sym = cpool->symbol_at(sig_index);

  if (declared_holder->is_loaded()) { // Our declared holder is loaded.
    instanceKlass *lookup = declared_holder->get_instanceKlass();
    methodOop m = lookup_method(accessor->get_instanceKlass(), lookup, name_sym, sig_sym, bc);
    if (m != NULL) {
      // We found the method.
      return get_object(m)->as_method();
    }
  }

  // Either the declared holder was not loaded, or the method could
  // not be found.  Create a dummy ciMethod to represent the failed
  // lookup.

  return get_unloaded_method(declared_holder,
                             get_object(name_sym)->as_symbol(),
                             get_object(sig_sym)->as_symbol());
}

// ------------------------------------------------------------------
// ciEnv::get_method_by_index
ciMethod* ciEnv::get_method_by_index(ciInstanceKlass* accessor,
                                     int index, Bytecodes::Code bc) {
  GUARDED_VM_ENTRY(return get_method_by_index_impl(accessor, index, bc);)
}

// ------------------------------------------------------------------
// ciEnv::name_buffer
char *ciEnv::name_buffer(int req_len) {
  if (_name_buffer_len < req_len) {
    if (_name_buffer == NULL) {
      _name_buffer = (char*)arena()->Amalloc(sizeof(char)*req_len);
      _name_buffer_len = req_len;
    } else {
      _name_buffer =
	(char*)arena()->Arealloc(_name_buffer, _name_buffer_len, req_len);
      _name_buffer_len = req_len;
    }
  }
  return _name_buffer;
}

// ------------------------------------------------------------------
// ciEnv::is_in_vm
bool ciEnv::is_in_vm() {
  return JavaThread::current()->thread_state() == _thread_in_vm;
} 

// ------------------------------------------------------------------
// ciEnv::call_has_multiple_targets
int ciEnv::call_has_multiple_targets(instanceKlass* current,
                              symbolHandle   method_name,
                              symbolHandle   method_sig,
                              bool&           found) {
  methodOop target = current->find_method(method_name, method_sig);
  if (target != NULL && !target->is_abstract()) {
    // We have found an implementation.
    if (found) {
      // There is more than one concrete target for the virtual call.
      return true;
    } else {
      // We have found one concrete target for the virtual call.
      found = true;
    }
    
    if (target->is_final_method() || target->is_private()) {
      // This method cannot be overridden, so we do not need to check
      // our subclasses.
      return false;
    }
  }
  
  // Check recursively.
  for (Klass* s = current->subklass(); s != NULL; s = s->next_sibling()) {
    if (!s->is_interface() && s->oop_is_instance()) {
      if (call_has_multiple_targets((instanceKlass*)s, method_name, method_sig, found)) {
        return true;
      }
    }
  }
  return false;
}

// ------------------------------------------------------------------
// ciEnv::is_dependence_violated
bool ciEnv::is_dependence_violated(klassOop k, methodOop m) {
  if (Klass::cast(k)->is_interface() &&
      instanceKlass::cast(k)->nof_implementors() > 1) {
    return true;
  }
  
  instanceKlass* ik = instanceKlass::cast(k);
  if (m == NULL) {
    // Subtype dependence
    return (ik->subklass() != NULL);
  } else {
    symbolHandle h_name(m->name());
    symbolHandle h_sig (m->signature());
    bool found = false;
    return call_has_multiple_targets(ik, h_name, h_sig, found);
  }
  return false;
}

// ------------------------------------------------------------------
// ciEnv::register_method
void ciEnv::register_method(ciMethod* target,
			    int entry_bci,
			    int iep_offset,
			    int ep_offset,
			    int vep_offset,
			    int code_offset,
			    int osr_offset,
			    DebugInformationRecorder* recorder,
			    CodeBuffer* code_buffer,
			    int frame_words,
			    int return_address_offset,
			    OopMapSet* oop_map_set,
			    ExceptionHandlerTable* handler_table,
			    ImplicitNullCheckTable* inc_table) {
  VM_ENTRY_MARK;
 // To prevent compile queue updates.
  MutexLocker locker(MethodCompileQueue_lock);

  _success = true;

  // We need this lock to access SystemDictionary::number_of_adds().
  MutexLocker ml(Compile_lock);
  methodOop method = target->get_methodOop();

  // Were there any class loads during the compilation?
  if (_class_load_counter != SystemDictionary::number_of_adds()) {
    int first = recorder->first_dependent();
    int limit = first + recorder->number_of_dependents();
    OopRecorder* oop_recorder= recorder->oop_recorder();
    for (int index = first; index < limit; index += 2) {
      klassOop klass = klassOop(JNIHandles::resolve(oop_recorder->handle_at(index-1)));
      assert(klass->is_klass(), "type check");
      methodOop method = methodOop(JNIHandles::resolve(oop_recorder->handle_at(index)));
      assert(!method || method->is_method(), "type check");
      if (is_dependence_violated(klass, method)) {
	if (PrintCompilation) {
	  tty->print_cr("COMPILATION KILLED BY CONCURRENT CLASS LOADING : index = %d", index);
	}
	_invalidated = true;
	return;
      }
    }	
  }
  
  nmethod *nm =  nmethod::new_nmethod(method, 
                                      entry_bci,
				      iep_offset,
                                      ep_offset, vep_offset, code_offset, osr_offset,
                                      recorder, code_buffer, 
                                      frame_words, return_address_offset, oop_map_set, 
                                      handler_table, inc_table);
  
  if (nm == NULL) {
    // The CodeCache is full.  Print out warning and disable compilation.
    UseInterpreter = true;
    if (UseRecompilation || AlwaysCompileLoopMethods ) {
#ifndef PRODUCT
      warning("CodeCache is full. Compiler has been disabled");     
#endif
      UseRecompilation           = false;    
      AlwaysCompileLoopMethods   = false;
    }
  } else {
    // JVMPI -- compiled method notification
    post_compiled_method_load_event(nm);

    if (entry_bci == InvocationEntryBci) {
      method->set_code(nm);
    } else {
      instanceKlass::cast(method->method_holder())->add_osr_nmethod(nm);
    }
  }
}

// ------------------------------------------------------------------
// ciEnv::register_i2c_adapter
#ifdef COMPILER2
void ciEnv::register_i2c_adapter(ciMethod* target,
				 OopMapSet *oopmaps,
				 CodeBuffer* cb,
				 int frame_words,
				 int return_address_offset) {
  VM_ENTRY_MARK;
  MutexLocker locker(AdapterCompileQueue_lock);
  methodHandle method = target->get_methodOop();
  
  AdapterInfo info(method, true);
  if (I2CAdapterGenerator::_cache->lookup(&info) != NULL) {
    return;
  }

  // Make the NMethod
  I2CAdapter* adapter = I2CAdapter::new_i2c_adapter( cb, 
						     oopmaps, 
						     return_address_offset,
						     frame_words);  
  assert(adapter != NULL && adapter->is_i2c_adapter(), "sanity check");
  adapter->set_prototypical_signature(method()->signature(), method()->is_static());
  I2CAdapterGenerator::_cache->insert(&info, adapter);
}
#endif


// ------------------------------------------------------------------
// ciEnv::register_c2i_adapter
#ifdef COMPILER2
void ciEnv::register_c2i_adapter(ciMethod* target,
				OopMapSet* oopmaps,
                                CodeBuffer* cb,
				int first_block_size,
				int frame_words,
				int return_address_offset) {
  VM_ENTRY_MARK;
  MutexLocker locker(AdapterCompileQueue_lock);
  methodOop method = target->get_methodOop();

  AdapterInfo info(method, false);
  if (C2IAdapterGenerator::_cache->lookup(&info) != NULL) {
    return;
  }

  C2IAdapter* adapter = C2IAdapter::new_c2i_adapter(cb,
                                             first_block_size,
                                             return_address_offset,
                                             oopmaps,
                                             frame_words );  
  assert(adapter != NULL && adapter->is_c2i_adapter(), "sanity check");
  
  adapter->set_prototypical_signature(method->signature(), method->is_static());
  C2IAdapterGenerator::_cache->insert(&info, adapter);
}
#endif // COMPILER2


// ------------------------------------------------------------------
// ciEnv::post_compiled_method_load_event
// new method for install_code() path
// Transfer information from compilation to jvmpi
// NOTE: lineno_table must be from long-term storage
void ciEnv::post_compiled_method_load_event(nmethod *nm) {
  // !!!!! !!!!! !!!!!
//   // if (nm != NULL && jvmpi::is_event_enabled(JVMPI_EVENT_COMPILED_METHOD_LOAD)) {
//   if( nm != NULL && _linenumbers_offset != NULL ) {
//     JVMCI_LineNumbersDesc *lineno_table = get_jvmpi_lineno_table();
//     set_jvmpi_lines( lineno_table );
//     JVMCI_LineNumbersDesc *line_numbers = jvmpi_lines();
//     compiled_method_t compiled_method;
//     compiled_method.method    = nm->method();
//     compiled_method.code_addr = nm->code_begin();
//     compiled_method.code_size = nm->code_size();
//     if (line_numbers == NULL) {
//       compiled_method.lineno_table_len = 0;
//       compiled_method.lineno_table = NULL;
//     } else {
//       compiled_method.lineno_table_len = line_numbers->_nof_lines;
//       compiled_method.lineno_table     = line_numbers->_line_numbers;
//     }
//     jvmpi::post_compiled_method_load_event(&compiled_method);
//   }
}

// ------------------------------------------------------------------
// ciEnv::get_method_number_of_lines
jint ciEnv::get_method_number_of_lines(ciMethod *m) {
  // Compute size of table
  jint num_entries = 0;
  if (m->has_linenumber_table()) {
    CompressedLineNumberReadStream stream(m->compressed_linenumber_table());
    while (stream.read_pair()) {
      num_entries++;
    }
  }
  return num_entries;
}

// ------------------------------------------------------------------
// ciEnv::fill_method_line_numbers
void ciEnv::fill_method_line_numbers(ciMethod *m, GrowableArray<LinePair*>* line_pairs) {
  if (line_pairs != NULL && line_pairs->length() > 0) {
    int index = 0;
    CompressedLineNumberReadStream stream(m->compressed_linenumber_table());
    while (stream.read_pair()) {
      line_pairs->at(index)->set(stream.line(), stream.bci());
      index++;
    }
    assert(index == line_pairs->length(), "sanity check");
  }
}


// ------------------------------------------------------------------
// ciEnv::find_system_klass
ciKlass* ciEnv::find_system_klass(ciSymbol* klass_name) {
  VM_ENTRY_MARK;
  return get_klass_by_name_impl(NULL, klass_name);
}

// ------------------------------------------------------------------
// ciEnv::find_system_klass
ciKlass* ciEnv::find_system_klass(const char* klass_name) {
  return find_system_klass(ciSymbol::make(klass_name));
}

// ------------------------------------------------------------------
// ciEnv::request_eager_c2i_adapter
#ifdef COMPILER2
void ciEnv::request_eager_c2i_adapter(ciMethod* target) {
  if (target == NULL || !target->is_loaded()) {
    return;
  }
  VM_ENTRY_MARK;
  methodHandle method(target->get_methodOop());
  CompileBroker::compile_c2i_adapter_for(method, false);
}
#endif

// ------------------------------------------------------------------
// ciEnv::compile_id
uint ciEnv::compile_id() {
  return ((CompilerThread*)JavaThread::current())->task()->compile_id();
}
