/*
 * @(#)ciEnv.hpp	1.15 00/02/14 15:25:57
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

class LinePair;  // forward def

// ciEnv
//
// This class is the top level broker for requests from the compiler
// to the VM.
class ciEnv : StackObj {
  CI_PACKAGE_ACCESS
  friend class CompileBroker;

private:
  JNIEnv*          _jni_env;
  Arena*           _arena;
  int              _class_load_counter;
  ciObjectFactory* _factory;
  bool             _invalidated;
  bool             _success;
  bool             _break_at_compile;

  char* _name_buffer;
  int   _name_buffer_len;

  // Distinguished instances of certain ciObjects..
  static ciObject*              _null_object_instance;
  static ciMethodKlass*         _method_klass_instance;
  static ciSymbolKlass*         _symbol_klass_instance;
  static ciKlassKlass*          _klass_klass_instance;
  static ciInstanceKlassKlass*  _instance_klass_klass_instance;
  static ciTypeArrayKlassKlass* _type_array_klass_klass_instance;
  static ciObjArrayKlassKlass*  _obj_array_klass_klass_instance;

  static ciInstanceKlass* _ArrayStoreException;
  static ciInstanceKlass* _Class;
  static ciInstanceKlass* _ClassCastException;
  static ciInstanceKlass* _Object;
  static ciInstanceKlass* _OutOfMemoryError;
  static ciInstanceKlass* _String;

  ciInstance* _NullPointerException_instance;
  ciInstance* _ArithmeticException_instance;

  // Look up a klass by name from some class loader.
  ciKlass* get_klass_by_name(ciKlass* accessing_klass,
                             ciSymbol* klass_name);

  // Constant pool access.
  ciKlass*   get_klass_by_index(ciInstanceKlass* loading_klass,
                                int klass_index);
  ciConstant get_constant_by_index(ciInstanceKlass* loading_klass,
                                   int constant_index);
  bool       is_unresolved_string(ciInstanceKlass* loading_klass,
                                   int constant_index) const;
  ciField*   get_field_by_index(ciInstanceKlass* loading_klass,
                                int field_index);
  ciMethod*  get_method_by_index(ciInstanceKlass* loading_klass,
                                 int method_index, Bytecodes::Code bc);

  // Implementation methods for loading and constant pool access.
  ciKlass* get_klass_by_name_impl(ciKlass* accessing_klass,
                                  ciSymbol* klass_name);
  ciKlass*   get_klass_by_index_impl(ciInstanceKlass* loading_klass,
                                     int klass_index);
  ciConstant get_constant_by_index_impl(instanceKlass* loading_klass,
					int constant_index);
  bool       is_unresolved_string_impl (instanceKlass* loading_klass,
					int constant_index) const;
  ciField*   get_field_by_index_impl(ciInstanceKlass* loading_klass,
				     int field_index);
  ciMethod*  get_method_by_index_impl(ciInstanceKlass* loading_klass,
				      int method_index, Bytecodes::Code bc);

  // Helper methods
  bool       check_klass_accessibility(ciKlass* accessing_klass,
				      klassOop resolved_klassOop);
  methodOop  lookup_method(instanceKlass*  accessor,
			   instanceKlass*  holder,
			   symbolOop       name,
			   symbolOop       sig,
			   Bytecodes::Code bc);

  // Get a ciObject from the object factory.  Ensures uniqueness
  // of ciObjects.
  ciObject* get_object(oop o) {
    if (o == NULL) {
      return _null_object_instance;
    } else {
      return _factory->get(o);
    }
  }

  ciMethod* get_method_from_handle(jobject method);

  // Get a ciMethod representing either an unfound method or
  // a method with an unloaded holder.  Ensures uniqueness of
  // the result.
  ciMethod* get_unloaded_method(ciInstanceKlass* holder,
                                ciSymbol*        name,
                                ciSymbol*        signature) {
    return _factory->get_unloaded_method(holder, name, signature);
  }

  // Get a ciKlass representing an unloaded klass.
  // Ensures uniqueness of the result.
  ciKlass* get_unloaded_klass(ciKlass* accessing_klass,
                              ciSymbol* name) {
    return _factory->get_unloaded_klass(accessing_klass, name);
  }

  // General utility : get a buffer of some required length.
  // Used in symbol creation.
  char* name_buffer(int req_len);

  // Is this thread currently in the VM state?
  static bool is_in_vm();

  // Helper routines for determining the validity of a compilation
  // with respect to concurrent class loading.
  static int call_has_multiple_targets(instanceKlass* current,
                                symbolHandle   method_name,
                                symbolHandle   method_sig,
                                bool&          found);
  static bool is_dependence_violated(klassOop k, methodOop m);

public:
  enum {
    not_adapter = -1,
    i2c = 0,
    c2i = 1
  };

  ciEnv(JNIEnv* jni_env, int class_load_counter, bool break_at_compile);
  ~ciEnv();

  bool is_invalidated() { return _invalidated; }

  bool is_success() { return _success; }

  bool break_at_compile() { return _break_at_compile; }

  uint compile_id();

  // Register the result of a compilation.
  void register_method(ciMethod*                 target,
                       int                       entry_bci,
                       int                       iep_offset,
                       int                       ep_offset,
                       int                       vep_offset,
                       int                       code_offset,
                       int                       osr_offset,
                       DebugInformationRecorder* recorder,
                       CodeBuffer*               code_buffer,
                       int                       frame_words,
                       int                       return_address_offset,
                       OopMapSet*                oop_map_set,
                       ExceptionHandlerTable*    handler_table,
                       ImplicitNullCheckTable*   inc_table);

#ifdef COMPILER2
  // Register compiled code for an i2c adapter.
  void register_i2c_adapter(ciMethod*   target,
                            OopMapSet*  oop_map_set,
                            CodeBuffer* code_buffer,
                            int         frame_words,
                            int         return_address_offset);

  // Register compiled code for a c2i adapter.
  void register_c2i_adapter(ciMethod*   target,
                            OopMapSet*  oop_map_set,
                            CodeBuffer* code_buffer,
                            int         first_block_size,
                            int         frame_words,
                            int         return_address_offset);
#endif // COMPILER2

  // JVMPI
  void post_compiled_method_load_event(nmethod *nm);
  // merge with corresponding JVMCI_ methods
  jint get_method_number_of_lines(ciMethod *m);
  void fill_method_line_numbers  (ciMethod *m, GrowableArray<LinePair*>* line_pairs);

  // Access to certain well known ciObjects.
  ciInstanceKlass* ArrayStoreException_klass() {
    return _ArrayStoreException;
  }
  ciInstanceKlass* Class_klass() {
    return _Class;
  }
  ciInstanceKlass* ClassCastException_klass() {
    return _ClassCastException;
  }
  ciInstanceKlass* Object_klass() {
    return _Object;
  }
  ciInstanceKlass* OutOfMemoryError_klass() {
    return _OutOfMemoryError;
  }
  ciInstanceKlass* String_klass() {
    return _String;
  }
  ciInstance* NullPointerException_instance() {
    assert(_NullPointerException_instance != NULL, "initialization problem");
    return _NullPointerException_instance;
  }
  ciInstance* ArithmeticException_instance() {
    assert(_ArithmeticException_instance != NULL, "initialization problem");
    return _ArithmeticException_instance;
  }

  ciKlass*  find_system_klass(ciSymbol* klass_name);
  ciKlass*  find_system_klass(const char* klass_name);

  // Access to the compile-lifetime allocation arena.
  Arena*    arena() { return _arena; }

  // What is the current compilation environment?
  static ciEnv* current() { return ((CompilerThread*)JavaThread::current())->env(); }

  static void set_data(void *data) {
    ((CompilerThread*)JavaThread::current())->set_data(data);
  }

  static void* data() {
    return ((CompilerThread*)JavaThread::current())->data();
  }

#ifdef COMPILER2
  void request_eager_c2i_adapter(ciMethod* dest_method);
#endif
};

// JVMPI - support
class LinePair: public ResourceObj {
 private:
  jint _line;
  jint _bci;
 public:
  LinePair(): _line(-1), _bci(-1) {}
  void set(jint line, jint bci) { _line = line; _bci = bci; }
  jint bci() const  { return _bci; }
  jint line() const { return _line; }
};


