/*
 * @(#)ciFieldLayout.cpp	1.3 99/12/01 15:43:40
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_ciFieldLayout.cpp.incl"

// ciFieldLayout
//
// This class represents the layout of the fields of an instanceKlass.

// ------------------------------------------------------------------
// ciFieldLayout::ciFieldLayout
ciFieldLayout::ciFieldLayout(ciInstanceKlass* my_klass) {
  assert(my_klass->is_loaded(), "must be loaded");
  ASSERT_IN_VM;

  klassOop klass = my_klass->get_klassOop();

  Arena* arena = CURRENT_ENV->arena();
  GrowableArray<BasicType>* fields =
    new (arena) GrowableArray<BasicType>(arena, 8, 0, T_VOID);

  int pos = 0;

  fill_in_header_fields(fields, pos);
  _header_count = pos;
  fill_in_instance_fields(fields, pos, klass);

  _elements = fields;
}

// ------------------------------------------------------------------
// ciFieldLayout::fill_in_header_fields
//
// Set up the header fields in this ciFieldLayout.
void ciFieldLayout::fill_in_header_fields(GrowableArray<BasicType>* fields,
					  int& pos) {
  assert(pos++*wordSize == oopDesc::mark_offset_in_bytes(),
	 "mark offset failure");
  fields->append(T_INT);
  assert(pos++*wordSize == oopDesc::klass_offset_in_bytes(),
	 "klass offset failure");
  fields->append(T_OBJECT);

  // Fill in any extra header fields with nothing.
  uint header_size_in_words = (uint)instanceOopDesc::header_size();
  for (; (uint)pos < header_size_in_words; pos++) {
    fields->append(T_VOID);
  }
}

// ------------------------------------------------------------------
// ciFieldLayout::fill_in_instance_fields
//
// Set up the instance fields in this ciFieldLayout.
void ciFieldLayout::fill_in_instance_fields(GrowableArray<BasicType>* fields,
					    int& pos,
					    klassOop current) {
  Klass* k = current->klass_part();
  assert(k->oop_is_instance(), "must be instanceKlass");
  instanceKlass* ik = (instanceKlass*)k;
  klassOop super = k->super();
  if (super) {
    fill_in_instance_fields(fields, pos, super);
  }

  // Fill in this klass's instance fields.
  typeArrayOop field_list = ik->fields();
  constantPoolOop cpool = ik->constants();
  uint num_fields = field_list->length();

  for (uint i = 0; i < num_fields; i += 5) {
    AccessFlags flags;
    flags.set_flags(field_list->short_at(i));
    
    if (!flags.is_static()) {
      // This is an instance field.  Add it to our list.
      uint field_offset = field_list->ushort_at(i+4);
      
      // Add the type to our list.
      symbolOop type_sym = cpool->symbol_at(field_list->short_at(i+2));
      BasicType field_type = FieldType::basic_type(type_sym);
      fields->at_put_grow(field_offset, field_type, T_VOID);
      pos += type2size[field_type];
    }
  }
}

// ------------------------------------------------------------------
// ciFieldLayout::print_layout
void ciFieldLayout::print_layout() {
  for (ciFieldLayoutStream ls(this); !ls.is_done(); ls.next()) {
    tty->print("%c", basictype_to_char(ls.type()));
  }  
}

// ------------------------------------------------------------------
// ciFieldLayout::print
void ciFieldLayout::print() {
  tty->print("<ciLayout layout=");
  print_layout();
  tty->print(" address=0x%x>", (address)this);
}


