/*
 * @(#)ciInstanceKlass.hpp	1.8 01/02/09 14:44:34
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// ciInstanceKlass
//
// This class represents a klassOop in the HotSpot virtual machine
// whose Klass part is an instanceKlass.  It may or may not
// be loaded.
class ciInstanceKlass : public ciKlass {
  CI_PACKAGE_ACCESS

private:
  bool                   _is_shared;

  jobject                _loader;
  jobject                _protection_domain;

  bool                   _is_initialized;
  bool                   _has_finalizer;
  bool                   _has_subklass;
  ciFlags                _flags;
  jint                   _size_helper;
  
  // Lazy fields get filled in only upon request.
  ciInstanceKlass*       _super;
  ciFieldLayout*         _layout;
  ciInstance*            _java_mirror;

  ciConstantPoolCache*   _field_cache;

protected:
  ciInstanceKlass(klassOop k);
  ciInstanceKlass(ciSymbol* name, jobject loader, jobject protection_domain)
    : ciKlass(name, ciInstanceKlassKlass::make()) {
    assert(name->byte_at(0) != '[', "not an instance klass");
    _loader = loader;
    _protection_domain = protection_domain;
    _is_initialized = false;
    _field_cache = NULL;
  }

  instanceKlass* get_instanceKlass() {
    return (instanceKlass*)get_Klass();
  }

  oop loader();
  jobject loader_handle();

  oop protection_domain();
  jobject protection_domain_handle();

  const char* type_name() { return "ciInstanceKlass"; }

  void print_impl();

  ciConstantPoolCache* field_cache();

  bool compute_shared_is_initialized();
  bool compute_shared_has_subklass();

public:
  // Has this klass been initialized?
  bool                   is_initialized() {
    if (_is_shared && !_is_initialized) {
      return compute_shared_is_initialized();
    }
    return _is_initialized;
  }

  // General klass information.
  ciFlags                flags()          {
    assert(is_loaded(), "must be loaded");
    return _flags;
  }
  bool                   has_finalizer()  {
    assert(is_loaded(), "must be loaded");
    return _has_finalizer; }
  bool                   has_subklass()   {
    assert(is_loaded(), "must be loaded");
    if (_is_shared && !_has_subklass) {
      if (flags().is_final()) {
        return false;
      } else {
        return compute_shared_has_subklass();
      }
    }
    return _has_subklass;
  }
  jint                   size_helper()  {
    assert(is_loaded(), "must be loaded");
    return _size_helper; }
  ciInstanceKlass*       super();
  ciFieldLayout*         field_layout();

  ciInstanceKlass* get_canonical_holder(int offset);

  ciInstanceKlass* unique_concrete_subklass();

  // Get the instance of java.lang.Class corresponding to
  // this klass.  This instance is used for locking of
  // synchronized static methods of this klass.
  ciInstance*            java_mirror();

  // Java access flags
  bool is_public      () { return flags().is_public(); }
  bool is_final       () { return flags().is_final(); }
  bool is_super       () { return flags().is_super(); }
  bool is_interface   () { return flags().is_interface(); }
  bool is_abstract    () { return flags().is_abstract(); }

  ciMethod* find_method(ciSymbol* name, ciSymbol* signature);
  ciMethod* find_method(const char* name, const char* signature);

  // Is the defining class loader of this class the default loader?
  bool uses_default_loader();

  bool is_java_lang_Object();

  // What kind of ciObject is this?
  bool is_instance_klass() { return true; }
  bool is_java_klass()     { return true; }
};

