/*
 * @(#)ciKlass.cpp	1.5 00/02/15 17:18:41
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_ciKlass.cpp.incl"

// ciKlass
//
// This class represents a klassOop in the HotSpot virtual
// machine.

// ------------------------------------------------------------------
// ciKlass::ciKlass
ciKlass::ciKlass(klassOop k) : ciObject(k) {
  assert(get_oop()->is_klass(), "wrong type");
  symbolOop klass_name = get_Klass()->name();
  assert(klass_name != NULL, "wrong ciKlass constructor");
  _name = CURRENT_ENV->get_object(klass_name)->as_symbol();
}

// ------------------------------------------------------------------
// ciKlass::ciKlass
//
// Nameless klass variant.
ciKlass::ciKlass(klassOop k, ciSymbol* name) : ciObject(k) {
  assert(get_oop()->is_klass(), "wrong type");
  _name = name;
}

// ------------------------------------------------------------------
// ciKlass::ciKlass
//
// Unloaded klass variant.
ciKlass::ciKlass(ciSymbol* name, ciKlass* klass) : ciObject(klass) {
  _name = name;
}

// ------------------------------------------------------------------
// ciKlass::is_subtype_of
bool ciKlass::is_subtype_of(ciKlass* that) {
  assert(is_loaded() && that->is_loaded(), "must be loaded");
  assert(is_java_klass() && that->is_java_klass(), "must be java klasses");
  // Check to see if the klasses are identical.
  if (this == that) {
    return true;
  }

  VM_ENTRY_MARK;
  Klass* this_klass = get_Klass();
  klassOop that_klass = that->get_klassOop();
  bool result = this_klass->is_subtype_of(that_klass);

  return result;
}

// ------------------------------------------------------------------
// ciKlass::is_subclass_of
bool ciKlass::is_subclass_of(ciKlass* that) {
  assert(is_loaded() && that->is_loaded(), "must be loaded");
  assert(is_java_klass() && that->is_java_klass(), "must be java klasses");
  // Check to see if the klasses are identical.

  VM_ENTRY_MARK;
  Klass* this_klass = get_Klass();
  klassOop that_klass = that->get_klassOop();
  bool result = this_klass->is_subclass_of(that_klass);

  return result;
}

// ------------------------------------------------------------------
// ciKlass::least_common_ancestor
//
// Get the shared parent of two klasses.
//
// Implementation note: this method currently goes "over the wall"
// and does all of the work on the VM side.  It could be rewritten
// to use the super() method and do all of the work (aside from the
// lazy computation of super()) in native mode.  This may be
// worthwhile if the compiler is repeatedly requesting the same lca
// computation or possibly if most of the superklasses have already
// been created as ciObjects anyway.  Something to think about...
ciKlass*
ciKlass::least_common_ancestor(ciKlass* that) {
  assert(is_loaded() && that->is_loaded(), "must be loaded");
  assert(is_java_klass() && that->is_java_klass(), "must be java klasses");
  // Check to see if the klasses are identical.
  if (this == that) {
    return this;
  }

  VM_ENTRY_MARK;
  Klass* this_klass = get_Klass();
  Klass* that_klass = that->get_Klass();
  Klass* lca        = this_klass->LCA(that_klass);

  // Many times the LCA will be either this_klass or that_klass.
  // Treat these as special cases.
  if (lca == that_klass) {
    return that;
  }
  if (this_klass == lca) {
    return this;
  }

  // Create the ciInstanceKlass for the lca.
  ciKlass* result =
    CURRENT_ENV->get_object(lca->as_klassOop())->as_klass();

  return result;
}

// ------------------------------------------------------------------
// ciKlass::always_speculatively_locked
bool ciKlass::always_speculatively_locked() {
  assert(is_loaded(), "must be loaded");
  VM_ENTRY_MARK;
  return SpecLock::is_speclock_class(get_klassOop());
}

// ------------------------------------------------------------------
// ciKlass::never_speculatively_locked
bool ciKlass::never_speculatively_locked() {
  assert(is_loaded(), "must be loaded");
  VM_ENTRY_MARK;
  return SpecLock::is_not_speclock_class(get_klassOop());
}

// ------------------------------------------------------------------
// ciKlass::find_klass
//
// Find a klass using this klass's class loader.
ciKlass* ciKlass::find_klass(ciSymbol* klass_name) {
  assert(is_loaded(), "cannot find_klass through an unloaded klass");
  return CURRENT_ENV->get_klass_by_name(this,
					klass_name);
}

// ------------------------------------------------------------------
// ciKlass::print_impl
//
// Implementation of the print method
void ciKlass::print_impl() {
  tty->print(" name=");
  print_name();
}

// ------------------------------------------------------------------
// ciKlass::print_name
//
// Print the name of this klass
void ciKlass::print_name() {
  name()->print_symbol();
}

