// @(#)ciOopMap.hpp	1.4 00/01/24 12:06:12
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


// Note: This classes have been derived from class in oopMapCache.?pp;
//       We need to factorize this operations if possible

// This class holds oop maps generated in the compiler interface
// No oops allowed inside this structure as, it is passed and referenced
// by the compiler

// The class contains two structures, one describing the bci that has 
// the oop map and the other describing the oop-map; the oop map is compressed
// and contains nof_gc_points x max_nof_locals entries, with each entry being 
// either 1 or 2 bit wide; this is an array of masks, all of fixed length 
// (number of bits = max_nof_locals*entry_size).; there are nof_gc_points
// masks
class ciOopMap: public ResourceObj {
 public:
  enum {
#ifdef ENABLE_ZAP_DEAD_LOCALS
    bits_per_entry   = 2,
    dead_bit_number  = 1,
#else
    bits_per_entry   = 1,
#endif
    oop_bit_number   = 0
  };

 private:
  Arena*        _arena;
  const int     _max_nof_locals;
  const int     _nof_gc_points;
  int           _nof_entries;
  unsigned int* _bci;
  unsigned int* _map;
  int           _mask_size; // in bits
  int           _mask_words;

  int           _nof_initialize;
  unsigned int* _locals;

  GrowableArray<GrowableArray<int>*>* _cached_oops;

  int  mask_size() const { return _mask_size; }
  unsigned int entry_at(int map_index, int offset)  const; 

  void set_init_local(int index, int local_index);
  void set_nof_initialize(int n);
  void set_nof_entries(int n) { _nof_entries = n; }

  friend class ciGenerateOopMap;

 public:
  ciOopMap(Arena* arena, int max_nof_locals, int nof_gc_points);

  bool is_value(int map_index, int offset)   { return (entry_at(map_index, offset) & (1 << oop_bit_number )) == 0; }
  bool is_oop  (int map_index, int offset)   { return (entry_at(map_index, offset) & (1 << oop_bit_number )) != 0; }
#ifdef ENABLE_ZAP_DEAD_LOCALS
  bool is_dead (int map_index, int offset)   { return (entry_at(map_index, offset) & (1 << dead_bit_number)) != 0; }
#endif

  void set_bci_for_index(int map_index, unsigned int bci);

  // map_index must be : 0 .. _nof_gc_points - 1
  unsigned int* map_for_index (int map_index)  const;
  unsigned int  bci_for_index (int map_index)  const;

  // int nof_gc_points()  const { return _nof_gc_points;  }
  int nof_entries()    const { return _nof_entries;    }
  int max_nof_locals() const { return _max_nof_locals; }

  // info about locals to initialize
  int nof_locals_to_initialize()  const { return _nof_initialize; }
  int local_to_initialize(int index) const { assert(0<=index && index<_nof_initialize,"range chek"); return _locals[index]; }

  void init_oops_cache(int code_size);
  GrowableArray<int>* ciOopMap::get_oops_for_bci(int bci);

  void print();
};


class ciGenerateOopMap: public GenerateOopMap {
 private:
  Arena*         _arena;
  bool           _has_exceptions_handlers;
  bool           _is_synchronized;
  GrowableArray<bool>* _is_gc_point; // for exception handler starts and for jsr return points
  int            _nof_gc_points;
  ciOopMap*      _oop_maps;

  int            _fill_counter;

  virtual bool possible_gc_point        (BytecodeStream *bcs); // tells if current operation is a GC point
  virtual void fill_stackmap_prolog     (int nof_gc_points);   // reports nof of gc point
  virtual bool report_results           () const               { return true;  }
  virtual bool report_init_vars         () const               { return true;  }
  virtual void fill_stackmap_epilog     ()                     { _oop_maps->set_nof_entries(_fill_counter); }

  virtual void fill_stackmap_for_opcodes(BytecodeStream *bcs,
                                         CellTypeState* vars, 
                                         CellTypeState* stack, 
                                         int stackTop) ;
  virtual void fill_init_vars           (GrowableArray<int> *init_vars);

  void find_jsr_return_points(methodHandle method);

 public:
  ciGenerateOopMap(Arena* arena, methodHandle m);

  ciOopMap* get_oop_maps() const { return _oop_maps; }
};


class ciAddressMap: public CHeapObj {
public:
  enum {
    bits_per_entry   = 1,
    address_bit_number   = 0
  };
private:
  Arena*        _arena;
  unsigned int* _map;
  int           _mask_size; // in bits
  int           _mask_words;
  int           _nof_locals;

  unsigned int entry_at(int offset) const;

public:
  ciAddressMap(Arena* arena, int nof_locals);
  ~ciAddressMap();
  unsigned int* get_map_raw() const { return _map; }
  bool is_address  (int offset)   { return (entry_at(offset) & (1 << address_bit_number )) != 0; }
  int nof_locals() const { return _nof_locals; }
};


// returns an address map (index i is true if local i is of type Address)
class ciAddressMapGenerator: public GenerateOopMap {
 private:
  Arena*        _arena;
  int           _for_bci;
  ciAddressMap* _address_map;

  virtual bool possible_gc_point          (BytecodeStream *bcs);     // tells if current operation is a GC point
  virtual void fill_stackmap_prolog       (int nof_gc_points);       // reports nof of gc point
  virtual void fill_stackmap_epilog       ()                         {}
  virtual bool report_results             () const                   { return true;  }

  virtual void fill_stackmap_for_opcodes  (BytecodeStream *bcs,
                                           CellTypeState* vars, 
                                           CellTypeState* stack, 
                                           int stackTop) ;
  virtual void fill_init_vars             (GrowableArray<int> *init_vars) {}


public:
  ciAddressMapGenerator(Arena* arena, methodHandle m, int for_bci);
  ciAddressMap* get_address_map() const                              { return _address_map; }
};
