/*
 * @(#)ciSignature.cpp	1.5 00/02/15 17:19:05
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_ciSignature.cpp.incl"

// ciSignature
//
// This class represents the signature of a method.

// ------------------------------------------------------------------
// ciSignature::ciSignature
ciSignature::ciSignature(ciKlass* accessing_klass, ciSymbol* symbol) {
  ASSERT_IN_VM;
  _accessing_klass = accessing_klass;
  _symbol = symbol;
  _klasses = NULL;

  Arena* arena = CURRENT_ENV->arena();
  _types = new (arena) GrowableArray<BasicType>(arena, 8, 0, T_ILLEGAL);

  int size = 0;
  int count = 0;
  SignatureStream ss(symbol->get_symbolOop());
  for (; !ss.at_return_type(); ss.next()) {
    // Process one element of the signature
    BasicType type = ss.type();
    _types->append(type);
    size += type2size[type];
    count++;
  }
  _size = size;
  _count = count;

  // Process the return type
  _types->append(ss.type());
}

// ------------------------------------------------------------------
// ciSignature::get_all_klasses
//
// Parse the symbol representing this signature and fill in the
// _klasses array.
void ciSignature::get_all_klasses() {
  VM_ENTRY_MARK;
  assert(_accessing_klass->is_loaded(),
	 "cannot find_klass through an unloaded klass");
  Arena* arena = CURRENT_ENV->arena();
  _klasses = new (arena) GrowableArray<ciKlass*>(arena, _types->length(), 0, NULL);

  SignatureStream ss(_symbol->get_symbolOop());
  for (; !ss.is_done(); ss.next()) {
    // Process one element of the signature
    if (ss.is_object()) {
      symbolOop name = ss.as_symbol(KILL_COMPILE_ON_ANY);
      ciSymbol* klass_name = CURRENT_ENV->get_object(name)->as_symbol();
      ciKlass* klass = CURRENT_ENV->get_klass_by_name_impl(_accessing_klass, klass_name);
      _klasses->append(klass);
    } else {
      _klasses->append(NULL);
    }
  }
}  

// ------------------------------------------------------------------
// ciSignature::return_klass
//
// What is the return klass, if any, of this signature?
ciKlass* ciSignature::return_klass() {
  if (_klasses == NULL) {
    get_all_klasses();
  }
  ciKlass* return_klass = _klasses->at(_count);
  assert(return_klass != NULL, "not a klass");
  return return_klass;
}

// ------------------------------------------------------------------
// ciSignature::klass_at
//
// What is the klass, if any, of the index'th element of this
// signature?
ciKlass* ciSignature::klass_at(int index) {
  assert(index < _count, "out of bounds");
  if (_klasses == NULL) {
    get_all_klasses();
  }
  // The first _klasses element holds the return klass.
  ciKlass* klass = _klasses->at(index);
  assert(klass != NULL, "not a klass");
  return klass;
}

// ------------------------------------------------------------------
// ciSignature::print_signature
void ciSignature::print_signature() {
  _symbol->print_symbol();
}

// ------------------------------------------------------------------
// ciSignature::print
void ciSignature::print() {
  tty->print("<ciSignature symbol=");
  print_signature();
 tty->print(" accessing_klass=");
  _accessing_klass->print();
  tty->print(" address=0x%x>", (address)this);
}

