/*
 * @(#)ciSignature.hpp	1.4 00/01/31 13:46:50
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// ciSignature
//
// This class represents the signature of a method.
class ciSignature : public ResourceObj {
private:
  ciSymbol* _symbol;
  ciKlass*  _accessing_klass;

  GrowableArray<BasicType>* _types;
  int _size;
  int _count;

  // Filled in upon demand.
  GrowableArray<ciKlass*>*  _klasses;

  friend class ciMethod;

  ciSignature(ciKlass* accessing_klass, ciSymbol* signature);

  void get_all_klasses();

  symbolOop get_symbolOop() const                { return _symbol->get_symbolOop(); }

public:
  ciSymbol* as_symbol() const                    { return _symbol; }

  BasicType return_type() const                  { return _types->at(_count); }
  ciKlass*  return_klass();

  BasicType type_at(int index) const { 
    assert(0 <= index && index < _count, "index out of bounds");
    return _types->at(index);
  }

  ciKlass*  klass_at(int index);

  bool      is_oop_at(int index)                 { BasicType t = type_at(index); return t == T_OBJECT || t == T_ARRAY; }
  bool      is_one_word_at(int index)            { return !is_two_word_at(index); }
  bool      is_two_word_at(int index)            { BasicType t = type_at(index); return t == T_LONG || t == T_DOUBLE; }

  int       size() const                         { return _size; }
  int       count() const                        { return _count; }

  void print_signature();
  void print();
};



