/*
 * @(#)codeCache.hpp	1.38 01/02/09 14:44:38
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// The CodeCache implements the code cache for various pieces of generated
// code, e.g., compiled java methods, runtime stubs, transition frames, etc.
// The entries in the CodeCache are all CodeBlob's.

// Implementation:
//   - Each CodeBlob occupies one chunk of memory.
//   - Like the offset table in oldspace the zone has at table for
//     locating a method given a addess of an instruction.

class CodeCache : AllStatic {
  friend class VMStructs;
 private:
  static CodeHeap _heap;
  static int _number_of_blobs;
  static int _number_of_nmethods_with_dependencies;
  
  static void verify_if_often() PRODUCT_RETURN;
 public:     

  // Initialization
  static void initialize();

  // Allocation/administration
  static CodeBlob* allocate(int size);              // allocates a new CodeBlob
  static void commit(CodeBlob* cb);                 // called when the allocated CodeBlob has been filled
  static int alignment_unit();                      // guaranteed alignment of all CodeBlobs
  static int alignment_offset();                    // guaranteed offset of first CodeBlob byte within alignment unit (i.e., allocation header)
  static void free(CodeBlob* cb);                   // frees a CodeBlob
  static void flush();                              // flushes all CodeBlobs
  static bool contains(void *p);                    // returns whether p is included  
  static void blobs_do(void f(CodeBlob* cb));       // iterates over all CodeBlobs 
  static void nmethods_do(void f(nmethod* nm));     // iterates over all nmethods 

  // Lookup
  static CodeBlob* find_blob(void* start);         
  static nmethod*  find_nmethod(void* start);

  // Lookup that does not fail if you lookup a zombie method (if you call this, be sure to know
  // what you are doing)
  static CodeBlob* find_blob_unsafe(void* start);

  // Iteration
  static CodeBlob* first();
  static CodeBlob* next (CodeBlob* cb);  
  static CodeBlob* alive(CodeBlob *cb);
  static int       nof_blobs()                 { return _number_of_blobs; }

  // GC support
  static void gc_epilogue();
  static void gc_prologue();
  static void follow_roots_phase1() {};
  static void follow_roots_phase2(bool unloading_occurred, bool& marked_for_unloading);
  static void adjust_pointers();
  
  // Printing/debugging
  static void print()   PRODUCT_RETURN;          // prints summary 
  static void print_internals();
  static void verify()  PRODUCT_RETURN;          // verifies the code cache
  
  // Profiling
  static address first_address();                // first address used for CodeBlobs
  static address last_address();                 // last  address used for CodeBlobs
  static int     capacity();
  static int     max_capacity();
  static int     uncommited_capacity()           { return max_capacity() - capacity(); }  

#ifndef CORE
  static void clear_inline_caches()                       CORE_RETURN;         // clear all inline caches

  // Deoptimization
  static int  mark_for_deoptimization(klassOop dependee)  CORE_RETURN;
  static int  mark_all_for_deoptimization()               CORE_RETURN;
  static void make_marked_nmethods_zombies()              CORE_RETURN;  
  static void make_marked_nmethods_unloaded()             CORE_RETURN;

    // tells how many nmethods have dependencies
  static int number_of_nmethods_with_dependencies();
#endif
};
