/*
 * @(#)icBuffer.hpp	1.20 99/06/22 16:39:45
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
//
// For CompiledIC's:
//
// In cases where we do not have MT-safe state transformation,
// we go to a transition state, using ICStubs. At a safepoint,
// the inline caches are transferred from the transitional code:
//
//    instruction_address --> 01 set xxx_oop, Ginline_cache_klass
//                            23 jump_to Gtemp, yyyy
//                            4  nop

class ICStub;

class InlineCacheBuffer: public AllStatic {
 private:
  // friends  
  friend class ICStub;

  static const int ic_stub_code_size;

  static StubQueue* _buffer;
  static ICStub*    _next_stub;

  static StubQueue* buffer()                         { return _buffer;         }
  static void       set_next_stub(ICStub* next_stub) { _next_stub = next_stub; }
  static ICStub*    get_next_stub()                  { return _next_stub;      }

  static void       init_next_stub();

  static ICStub* new_ic_stub();
 

  // Machine-dependent implementation of ICBuffer
  static void    assemble_ic_buffer_code(address code_begin, oop cached_oop, address entry_point);
  static address ic_buffer_entry_point  (address code_begin); 
  static oop     ic_buffer_cached_oop   (address code_begin);

 public:
 
    // Initialization; must be called before first usage
  static void initialize();

  // Access
  static bool contains(address instruction_address);

    // removes the ICStubs after backpatching
  static void update_inline_caches();

  // for debugging
  static bool is_empty();


  // New interface
  static void    create_transition_stub(CompiledIC *ic, oop cached_oop, address entry);
  static address ic_destination_for(CompiledIC *ic);
  static oop     cached_oop_for(CompiledIC *ic);
};

