/*
 * @(#)location.hpp	1.26 99/06/29 18:11:12
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// A Location describes a concrete machine variable location
// (such as integer or floating point register or a stack-held
// variable). Used when generating debug-information for nmethods.
//
// Encoding:
//
// bits:
//  Where   [0]
//  Type:   [1..2]
//  Offset  [3..15]

class Location VALUE_OBJ_CLASS_SPEC { 
 public:
  enum Where {
    on_stack,
    in_register
  };

  enum Type {
    normal,                     // Ints, floats, double halves
    oop,                        // Oop (please GC me!)
    float_in_dbl,               // Float held in double register
    invalid                     // Invalid location
  };

 private:
   jchar _offset:13,            // 
         _type:2,               // Type of value (normal, oop, float-in-dbl)
         _where:1;              // Where value is (stack or register)
        

  // Create a bit-packed Location
  Location(Where where_, Type type_, unsigned offset_) 
    : _where(where_), _type(type_), _offset(offset_) { 
      assert( where () == where_ , "" );
      assert( type  () == type_  , "" );
      assert( offset() == offset_, "" );
}

 public:

  // Stack location Factory.  Offset is word aligned; remove low bits
  static Location new_stk_loc( Type t, int offset ) { return Location(on_stack,t,offset>>2); }
  // Register location Factory
  static Location new_reg_loc( Type t, int regnum ) { return Location(in_register,t,regnum); }
  // Invalid 
  static Location invalid_loc(                    ) { return Location(in_register,invalid,-1); }
  // Default constructor
  Location() { _type=invalid; }


  // Bit field accessors
  Where where()  const { return (Where)_where;}
  Type  type()   const { return (Type) _type; }
  unsigned offset() const { return (unsigned) _offset; }

  // Accessors
  bool is_register() const    { return where() == in_register; }
  bool is_stack() const       { return where() == on_stack;    }

  bool holds_oop() const      { return type() == oop;          }
  bool holds_float() const    { return type() == float_in_dbl; }
  bool is_illegal() const     { return type() == invalid;      }

  int stack_offset() const    { assert(where() == on_stack,    "wrong Where"); return offset()<<2; }
  int register_number() const { assert(where() == in_register, "wrong Where"); return offset()   ; }
  
  // Printing
  void print_on(outputStream* st) const;

  // Serialization of debugging information
  Location(DebugInfoReadStream* stream);
  void write_on(DebugInfoWriteStream* stream);
};
