/*
 * @(#)onStackReplacement.hpp	1.12 00/05/19 08:05:20
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


#ifdef COMPILER2
// Class OSRAdapter describes an interpreter frame that has been 
// on-stack replaced. All oops will be dead in this frame. 
// This adapter frame keeps track of the size of the frame, and
// where the link (ebp on Win32) is stored, so stack-traversal
// works.
//
class OSRAdapter: public CodeBlob {
 private:
  int _sp_offset;
 
  OSRAdapter::OSRAdapter(CodeBuffer *cb, OopMapSet *oop_maps, int size, int frame_size, int sp_offset);  
  void* operator new(size_t s, unsigned size);

 public:
  // Creation
  static OSRAdapter *new_osr_adapter(CodeBuffer *cb, OopMapSet *oop_maps, int frame_size, int sp_offset);

  // Define Codeblob behaviour
  bool is_osr_adapter()                  const  { return true; }
  bool is_alive()                        const  { return true; }
  int  sp_offset()                       const  { return _sp_offset; }
  void adjust_pointers()                        { /* do-nothing*/ }  
  void preserve_callee_argument_oops          (frame fr, const RegisterMap *reg_map, void f(oop*))  { /* nothing to do */ }  
  void follow_roots_or_mark_for_unloading(bool unloading_occurred, bool& marked_for_unloading)     { /* do-nothing*/ }
        
  // Debugging code
  void verify()                                PRODUCT_RETURN; 
  void print()                          const  PRODUCT_RETURN;
  void print_value_on(outputStream* st) const  PRODUCT_RETURN; 
};
#endif // COMPILER2


// Namespace for OnStackReplacement functionality

class OnStackReplacement: public AllStatic {
#ifdef COMPILER2
 private:
  // List of adapters for different frame sizes.
  static GrowableArray<OSRAdapter*>* _osr_adapters;

  // List of on-stack replacement methods
  static GrowableArray<nmethod*>* _osr_nmethods;

  enum Constants {    
    InitialAdapterVectorSize   = 64,     // Initial size of adapter array
    MinTypicalAdapterSize      = 6,      // The typical range is precomputed 
    MaxTypicalAdapterSize      = 16      //   (framesize is in words)
  };
#endif // COMPILER2

 public:
#ifdef COMPILER2
  // Initialization
  static void initialize();  

  // Returns an OSRAdapter for the given framesize
  static OSRAdapter* get_osr_adapter(int framesize);
#endif // COMPILER2

  // Returns an osr nmethod if possible; returns NULL otherwise.
  static nmethod* get_osr_code(methodHandle method, int bci, TRAPS);
};


