/*
 * @(#)abstractCompiler.hpp	1.6 00/03/21 16:16:44
 *
 * Copyright 1991-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

class AbstractCompiler : public CHeapObj {
 private:
  bool _is_initialized;

 public:
  AbstractCompiler() : _is_initialized(false)    {}

  // Name of this compiler
  virtual const char* name()                     { return "compiler"; }

  // Missing feature tests
  virtual bool supports_native()                 { return true; }
  virtual bool supports_osr   ()                 { return true; } 

  // Customization
  virtual bool needs_preloaded_classes()         = 0;
  virtual bool needs_inlining_scope   ()         = 0;
  virtual bool needs_adapters         ()         = 0;
  virtual bool needs_stubs            ()         = 0;

  void mark_initialized()                        { _is_initialized = true; }
  bool is_initialized()                          { return _is_initialized; }

  virtual void initialize()                      = 0;

  // Compilation entry point for methods
  virtual void compile_method(ciEnv* env,
			      ciScope* scope,
			      ciMethod* target,
			      int entry_bci,
                              bool reuse_env) {
    ShouldNotReachHere();
  }
  
  // Compilation entry point for adapters
  virtual void compile_adapter(ciEnv* env,
		               ciMethod* callee,
		               int kind,
                               bool reuse_env) {
    ShouldNotReachHere();
  }

  // Print compilation timers and statistics
  virtual void print_timers() {
    ShouldNotReachHere();
  }
};
