/*
 * @(#)compilationScope.cpp	1.157 99/11/17 15:22:02
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_compilationScope.cpp.incl"


CompilationScope::CompilationScope(methodHandle method, Scope* caller, int callerBCI) : Scope(method, caller, callerBCI) {
  assert(!method->is_abstract(), "cannot generate code for abstract method"); 
  assert(!method->is_native(), "no scopes for native methods");
}


void CompilationScope::inline_scopes() {
  // Determine (recursively) which callees should be inlined.
  // After this pass, all inlining decisions have been made, and
  // subsequent passes obey these decisions.

  if (Inline || InlineAccessors) {
    InliningClosure c(this);
    iterate(&c);
    if (PrintScopeTree) {
      print_tree();
    }
  }
}


class ResolveStringsClosure: public ScopeClosure {
 private:
  int       _nesting;
  Thread*   _thread;

 public:
  ResolveStringsClosure(Scope* s, Thread* thread, int n = 0) : ScopeClosure(s) { _nesting = n; _thread = thread; }
  void enter_scope () {         
    if (!_thread->has_pending_exception()) {
      _scope->method()->constants()->resolve_string_constants(_thread);
    }
  } 
  ScopeClosure* clone_for_subiteration(Scope* subscope) { return new ResolveStringsClosure(subscope, _thread, _nesting + 1); }
};


// TEMP METHOD: This should be removed when JVMCI is in place.
void CompilationScope::resolve_string_constants(TRAPS) {  
  ResolveStringsClosure rsc(this, THREAD);
  iterate(&rsc);
  // Could have a pending exception at this point
}

//--------------------------------------------------------------------------------------------
#ifndef PRODUCT

void CompilationScope::print(int nesting) {
  tty->print("%*s", nesting * 2, " ");
  method()->print_name(tty);
  tty->fill_to(68);
  tty->print("(CompilationScope*)%#x", this);
  tty->cr();
}

#endif

