/*
 * @(#)compilerOracle.cpp	1.15 00/03/24 11:26:28
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_compilerOracle.cpp.incl"

class MethodMatcher : public CHeapObj {
 private:
  jobject        _class_name;
  jobject        _method_name;
  MethodMatcher* _next;
 public:
  MethodMatcher(symbolOop class_name, symbolOop method_name, MethodMatcher* next);
  bool match(methodOop method);
};

MethodMatcher::MethodMatcher(symbolOop class_name, symbolOop method_name, MethodMatcher* next) {
  _class_name  = JNIHandles::make_global(class_name,  false);
  _method_name = JNIHandles::make_global(method_name, false);
  _next        = next;
}


bool MethodMatcher::match(methodOop method) {
  symbolOop class_name  = Klass::cast(method->method_holder())->name();
  symbolOop method_name = method->name();
  for (MethodMatcher* current = this; current != NULL; current = current->_next) {
    if ( JNIHandles::resolve(current->_class_name)  == class_name
      && JNIHandles::resolve(current->_method_name) == method_name) {
      return true;
    }
  }
  return false;
}

static MethodMatcher* should_exclude_list  = NULL;
static MethodMatcher* should_inline_list  = NULL;
static MethodMatcher* should_break_at_list = NULL;
static MethodMatcher* should_print_list = NULL;

void CompilerOracle::add_should_break_at(symbolOop class_name, symbolOop method_name) {
  should_break_at_list = new MethodMatcher(class_name, method_name, should_break_at_list);
}

void CompilerOracle::add_should_exclude(symbolOop class_name, symbolOop method_name) {
  should_exclude_list = new MethodMatcher(class_name, method_name, should_exclude_list);
}

void CompilerOracle::add_should_inline(symbolOop class_name, symbolOop method_name) {
  should_inline_list = new MethodMatcher(class_name, method_name, should_inline_list);
}

void CompilerOracle::add_should_print(symbolOop class_name, symbolOop method_name) {
  should_print_list = new MethodMatcher(class_name, method_name, should_print_list);
}

bool CompilerOracle::should_exclude(methodOop method) {
  return (should_exclude_list != NULL)
      && should_exclude_list->match(method);
}

bool CompilerOracle::should_inline(methodOop method) {
  return (should_inline_list != NULL)
      && should_inline_list->match(method);
}

bool CompilerOracle::should_print(methodOop method) {
  return (should_print_list != NULL) && (method != NULL)
      && should_print_list->match(method);
}

bool CompilerOracle::should_break_at(methodOop method) {
  return (should_break_at_list != NULL)
      && should_break_at_list->match(method);
}

void CompilerOracle::read_from_line(char* line) {
  if (line[0] == '\0') return;
  if (line[0] == '#')  return;

  #define RANGE "[abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/$_<>]"
  char class_name[256];
  char method_name[256];
  int result = sscanf(line, "exclude %" RANGE " %"  RANGE, class_name, method_name);
  if (result == 2) {
    EXCEPTION_MARK;
    symbolHandle c_name  = oopFactory::new_symbol(class_name, CATCH);
    symbolHandle m_name = oopFactory::new_symbol(method_name, CATCH);
    add_should_exclude(c_name(), m_name());
    tty->print_cr("CompilerOracle: exclude %s %s", class_name, method_name);
    return;
  }

  if (sscanf(line, "break %" RANGE " %"  RANGE, class_name, method_name) == 2) {
    EXCEPTION_MARK;
    symbolHandle c_name  = oopFactory::new_symbol(class_name, CATCH);
    symbolHandle m_name = oopFactory::new_symbol(method_name, CATCH);
    add_should_break_at(c_name(), m_name());
    tty->print_cr("CompilerOracle: break %s %s", class_name, method_name);
    return;
  }

  if (sscanf(line, "inline %" RANGE " %"  RANGE, class_name, method_name) == 2) {
    EXCEPTION_MARK;
    symbolHandle c_name  = oopFactory::new_symbol(class_name, CATCH);
    symbolHandle m_name = oopFactory::new_symbol(method_name, CATCH);
    add_should_inline(c_name(), m_name());
    tty->print_cr("CompilerOracle: inline %s %s", class_name, method_name);
    return;
  }

  if (sscanf(line, "print %" RANGE " %"  RANGE, class_name, method_name) == 2) {
    EXCEPTION_MARK;
    symbolHandle c_name  = oopFactory::new_symbol(class_name, CATCH);
    symbolHandle m_name = oopFactory::new_symbol(method_name, CATCH);
    add_should_print(c_name(), m_name());
    tty->print_cr("CompilerOracle: print %s %s", class_name, method_name);
    return;
  }

  tty->print_cr("CompilerOracle: unrecognized line");
  tty->print_cr("\"%s\"", line);
}

const char* CompilerOracle::file_name = ".hotspot_compiler";

void CompilerOracle::parse_from_file() {
  FILE* stream = fopen(file_name, "rt");
  if (stream == NULL) return;

  char token[1024];
  int  pos = 0;
  int  c = getc(stream);
  while(c != EOF) {
    if (c == '\n') {
      token[pos++] = '\0';
      read_from_line(token);
      pos = 0;
    } else {
      token[pos++] = c;
    }
    c = getc(stream);
  }   
  token[pos++] = '\0';
  read_from_line(token);

  fclose(stream);
}

void CompilerOracle::append_comment_to_file(const char* message) {
  fileStream stream(fopen(file_name, "at"));
  stream.print("# ");
  for (int index = 0; message[index] != '\0'; index++) {
    stream.put(message[index]);
    if (message[index] == '\n') stream.print("# ");
  }
  stream.cr();
}

void CompilerOracle::append_exclude_to_file(methodOop method) {
  fileStream stream(fopen(file_name, "at"));
  stream.print("exclude ");
  Klass::cast(method->method_holder())->name()->print_symbol_on(&stream);
  stream.print(" ");
  method->name()->print_symbol_on(&stream);
  stream.cr();
  stream.cr();
}


void compilerOracle_init() {
  CompilerOracle::parse_from_file();
}


