// @(#)callProfile.cpp	1.9 00/04/10 17:51:57
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_callProfile.cpp.incl"


#ifndef CORE

CallProfileEntry* CallProfile::_table;


void CallProfile::initialize() {
  if (UseCallProfile || UseTypeProfile) {
    _table = NEW_C_HEAP_ARRAY(CallProfileEntry, CallProfileSize);
    assert(is_initialized(), "must be set up");
    clear();
  }
}


void CallProfile::clear() {
  if (!is_initialized())  return;

  Memory::set_words((jint*)table_base(), table_length() * CallProfileEntry::size(), 0);
}



methodHandle CallProfile::resolve_method(methodHandle method, int bci, KlassHandle& target_klass) {
  assert(UseCallProfile || UseTypeProfile, "No use in this unless profiling is enabled");
  CallProfileEntry e = CallProfile::entry_at(method, bci);
  if (e.is_monomorphic()) {
    // a single klass:  it is monomorphic in the type profile
    KlassHandle prof_klass  = e.receiver().klass();
    Bytecode_invoke* invoke = Bytecode_invoke_at(method, bci);
    symbolOop name          = invoke->name();
    symbolOop signature     = invoke->signature();
    assert(!prof_klass->is_interface(), "must be exactly-typed class");
    assert(!prof_klass->is_abstract(),  "must be call to real object");
    KlassHandle link_klass = prof_klass;  // fake a link-time class
    if (!link_klass->oop_is_instance()) {
      assert(link_klass->oop_is_array(), "method receiver must be instance or array");
      link_klass = SystemDictionary::object_klass();
    }
    methodHandle prof_method = LinkResolver::resolve_virtual_call_or_null(link_klass, link_klass, name, signature, method->method_holder());
    if (prof_method.not_null()) {
#ifdef ASSERT
      if (WizardMode && e.key().bcp() != method->bcp_from(bci)) {
	tty->print("Warning: BCI conflict detected in type profile:  ");
	e.print();
	tty->cr();
      }
#endif
      // return two values
      target_klass = prof_klass;
      return prof_method;
    }
  }
  // failure
  return (methodOop)NULL;
}


// bcp tracking

static bool bcp_is_conflict_value(address bcp) {
  // %%% assert(Universe::verify_klass_mask() < 0, "sign bit must be identically zero or one for klass pointers");
  jint differing_bits = (jint)bcp ^ Universe::verify_klass_bits();
  differing_bits &= Universe::verify_klass_mask(); // ignore insignificant bits
  return differing_bits < 0; // was the sign bit of the klass ptr flipped?
}

static address set_bcp_conflict_value(address bcp) {
  if (bcp_is_conflict_value(bcp))
    return bcp;
  else
    return (address)(-(jint)bcp);
}

static address clear_bcp_conflict_value(address bcp) {
  if (!bcp_is_conflict_value(bcp))
    return bcp;
  else
    return (address)(-(jint)bcp);
}

address BCPProfileEntry::bcp() {
  return clear_bcp_conflict_value(_bcp);
}

bool BCPProfileEntry::is_bcp_conflict() {
  return bcp_is_conflict_value(_bcp);
}

void BCPProfileEntry::record_bcp(address bcp) {
  // For debugging only.
  // Record the bcp which gave rise to the entry.
  // If there is a conflict, flip the sign bit.
  address prev_bcp = _bcp;
  if (bcp != prev_bcp) {
    if (prev_bcp == NULL) {
      _bcp = bcp;
    } else if (!bcp_is_conflict_value(prev_bcp)) {
      debug_only(_other_bcp = bcp);
      _bcp = set_bcp_conflict_value(prev_bcp);
    }
  } else {
    debug_only(++_bcp_count);
  }
}

static bool decode_bcp(address bcp, methodOop &m, int &bci) {
  oop p = Universe::slow_object_start((oop*)bcp);
  if (p != NULL && p->is_method()) {
    m = (methodOop)p;
    if (m->contains(bcp)) {
      bci = m->bci_from(bcp);
      return true;
    }
  }
  return false;
}

static symbolOop decode_call(methodOop m, int bci) {
  Bytecode_invoke* invoke = Bytecode_invoke_at(m, bci);
  if (!invoke->is_valid())  return NULL;
  return invoke->name();
#if 0
  symbolOop name          = invoke->name();
  symbolOop signature     = invoke->signature();
  klassOop  klass         = ??;
  methodHandle callee     = LinkResolver::resolve_virtual_call_or_null(klass, klass, name, signature, m->method_holder());
  return callee();
#endif
}

static void print_bcp(address bcp) {
  methodOop m;
  int bci;
  if (decode_bcp(bcp, m, bci)) {
    m->print_short_name(tty);
    tty->print("@%d [IC=%d%s]", bci, m->invocation_count(), m->invocation_counter()->carry() ? "*" : "");
    symbolOop callee = decode_call(m, bci);
    if (callee != NULL) {
      tty->print("  callee: %s", callee->as_C_string());
      //callee->print_short_name(tty);
      //tty->print(" [IC=%d%s]", callee->invocation_count(), callee->invocation_counter()->carry() ? "*" : "");
    }
  } else {
    tty->print("@(0x%x)", (jint)bcp);
  }
}


// GC support

void CallProfile::gc_prologue() {
  if (!is_initialized())  return;

#ifndef PRODUCT
  if (WizardMode && PrintGC) {
    tty->print("[clearing ");
    print_summary_line();
    tty->print_cr("]");
  }
#endif

  clear();
}

void CallProfile::gc_epilogue() {
  if (!is_initialized())  return;

}

void CallProfile::oops_do(void f(oop*)) {
  if (!is_initialized())  return;

  for (int i = 0; i < table_length(); i++) {
    CallProfileEntry &e = _table[i];
    if (e._receiver._klass)  f((oop*) &e._receiver._klass);
  }
}



#ifndef PRODUCT

static void print_klass(oop k) {
  if (k->is_klass())
    tty->print("%s", Klass::cast(klassOop(k))->internal_name());
  else
    k->print_value();
}

void CallProfileEntry::print() {
  tty->print("count: %4d", count());
  if (!receiver().is_empty()) {
    tty->print("  ");
    receiver().print();
  }

#ifdef ASSERT
  tty->print("  at bcp: ");
  key().print();
#endif
}

void BCPProfileEntry::print() {
  print_bcp(bcp());
  if (is_bcp_conflict()) {
    tty->print(" (BCP CONFLICT ");
    debug_only(print_bcp(_other_bcp));
    tty->print(")");
  }
}

void TypeProfileEntry::print() {
  tty->print("klass: ");
  klassOop k = (klassOop)( (jint)_klass & ~1 );
  print_klass(k);

  if (is_conflict()) {
    tty->print(" (KLASS CONFLICT ");
    debug_only(print_klass(_other_klass));
    tty->print(")");
  }

  debug_only(tty->print(" mono_count: %4d", _mono_count));
}

void CallProfile::print_summary_line() {
  int non_empty = 0;
  int conflicts = 0;
  int bcp_confl = 0;
  for (int i = 0; i < table_length(); i++) {
    CallProfileEntry e = _table[i];
    if (e.is_empty())  continue;
    non_empty++;
    if (e.receiver().is_conflict())  conflicts++;
#ifdef ASSERT
    if (e.key().is_bcp_conflict())   bcp_confl++;
#endif
  }
  tty->print("CallProfile %d entries (%.1f%% full), %d receiver conflicts (%.1f%% of non-empty), %d BCP conflicts (%.1f%% of non-empty)",
		non_empty, 100.0 * non_empty / CallProfile::table_length(),
		conflicts, 100.0 * conflicts / (non_empty?non_empty:1),
		bcp_confl, 100.0 * bcp_confl / (non_empty?non_empty:1));
}

void CallProfile::print() {
  print_summary_line();
  tty->cr();
  for (int i = 0; i < table_length(); i++) {
    CallProfileEntry e = _table[i];
    if (e.is_empty()) {
      continue;
    }
    tty->print("%4d: ", i);
    e.print();
    tty->cr();
  }
}


void CallProfileEntry::verify() {
  // %%%%
  debug_only(key().verify());
  receiver().verify();
}

void BCPProfileEntry::verify() {
  // %%%%
}

void TypeProfileEntry::verify() {
  // %%%%
}

void CallProfile::verify() {
  for (int i = 0; i < table_length(); i++) {
    _table[i].verify();
  }
}

#endif //PRODUCT


#endif //CORE
