/*
 * @(#)interpreterRuntime.cpp	1.360 01/02/28 17:11:00
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_interpreterRuntime.cpp.incl"


//------------------------------------------------------------------------------------------------------------------------
// Constants

void InterpreterRuntime::ldc_helper(Bytecodes::Code bytecode, int index, Bytecodes::Code ildc, Bytecodes::Code fldc, Bytecodes::Code aldc, TRAPS) {
  // access constant pool
  constantPoolOop pool      = method()->constants();
  constantTag     tag       = pool->tag_at(index);
  Bytecodes::Code fast_code = Bytecodes::_xxxunusedxxx; // for debugging
  // resolve constant & new bytecode
         if (tag.is_int   ()) { pool->int_at   (index); fast_code = ildc;
  } else if (tag.is_float ()) { pool->float_at (index); fast_code = fldc;
  } else if (tag.is_unresolved_string()) { pool->string_at(index, CHECK); fast_code = aldc;
  } else if (tag.is_string()) { pool->resolved_string_at(index); fast_code = aldc;
  } else                      { tag.print_on(tty); ShouldNotReachHere();
  }
  // patch bytecode
  assert(Bytecodes::java_code(fast_code) == bytecode, "just checkin'");

  // If a breakpoint is set, patch the original byte code in the BP instead.
  if (!is_breakpoint()) {
    Bytecode_at(bcp())->set_code(fast_code);
  } else {
    method()->set_orig_bytecode_at(method()->bci_from(bcp()), fast_code);
  }
}


IRT_ENTRY(void, InterpreterRuntime::ldc(JavaThread* thread))
  ldc_helper(Bytecodes::_ldc  , one_byte_index(), Bytecodes::_fast_ildc  , Bytecodes::_fast_fldc  , Bytecodes::_fast_aldc, CHECK);
IRT_END


IRT_ENTRY(void, InterpreterRuntime::ldc_w(JavaThread* thread))
  ldc_helper(Bytecodes::_ldc_w, two_byte_index(), Bytecodes::_fast_ildc_w, Bytecodes::_fast_fldc_w, Bytecodes::_fast_aldc_w, CHECK);
IRT_END


IRT_ENTRY(void, InterpreterRuntime::ldc2_w(JavaThread* thread))
  // access costant pool
  int             index     = two_byte_index();
  constantPoolOop pool      = method()->constants();
  constantTag     tag       = pool->tag_at(index);
  Bytecodes::Code fast_code = Bytecodes::_xxxunusedxxx; // for debugging
  // resolve constant & new bytecode
         if (tag.is_long  ()) { pool->long_at  (index); fast_code = Bytecodes::_fast_lldc_w;
  } else if (tag.is_double()) { pool->double_at(index); fast_code = Bytecodes::_fast_dldc_w;
  } else                      { ShouldNotReachHere();
  }
  // patch bytecode
  assert(Bytecodes::java_code(fast_code) == Bytecodes::_ldc2_w, "just checkin'");

  // If a breakpoint is set, patch the original byte code in the BP instead.
  if (!is_breakpoint()) {
    Bytecode_at(bcp())->set_code(fast_code);
  } else {
    method()->set_orig_bytecode_at(method()->bci_from(bcp()), fast_code);
  }
  // redo new bytecode
IRT_END


//------------------------------------------------------------------------------------------------------------------------
// Allocation

IRT_ENTRY(void, InterpreterRuntime::_new(JavaThread* thread, constantPoolOop pool, int index))
  instanceKlassHandle klass = pool->klass_at(index, CHECK);

  // Make sure we are not instantiating an abstract klass
  klass->check_valid_for_instantiation(true, CHECK);

  // Make sure klass is initialized
  klass->initialize(CHECK);    

  // At this point the class may not be fully initialized
  // because of recursive initialization. If it is fully
  // initialized & has_finalized is not set, we rewrite
  // it into its fast version (Note: no locking is needed
  // here since this is an atomic byte write and can be
  // done more than once).
  //
  // Note: In case of classes with has_finalized we don't
  //       rewrite since that saves us an extra check in
  //       the fast version which then would call the
  //       slow version anyway (and do a call back into
  //       Java).
  //       If we have a breakpoint, then we don't rewrite
  //       because the _breakpoint bytecode would be lost.
  if (UseFastInterpreterAllocation 
    && !is_breakpoint()
    && !already_patched(Bytecodes::_new) 
    && !(UseSpecLock && SpecLock::is_speclock_class(klass()))
    && klass->is_initialized() && !klass->has_finalizer()) {
    // patch bytecode - doesn't matter if it happens more than once
    Bytecode_at(bcp())->set_code(Bytecodes::_fast_new);
  }
  oop obj = klass->allocate_instance(CHECK);
  thread->set_vm_result(obj);  
IRT_END


IRT_ENTRY(void, InterpreterRuntime::fixup_speclock(JavaThread* thread, oop lockedObject))
  assert(UseSpecLock, "Speclock object even though no speclocking in use?");
  Handle h_obj(lockedObject);
  SpecLock::trigger_fixup(h_obj, CHECK);
  thread->set_vm_result(h_obj());
IRT_END

IRT_ENTRY(void, InterpreterRuntime::newarray(JavaThread* thread, BasicType type, jint size))
  oop obj = oopFactory::new_typeArray(type, size, CHECK);
  thread->set_vm_result(obj);
IRT_END


IRT_ENTRY(void, InterpreterRuntime::anewarray(JavaThread* thread, constantPoolOop pool, int index, jint size))
  // Note: no oopHandle for pool & klass needed since they are not used
  //       anymore after new_objArray() and no GC can happen before.
  //       (This may have to change if this code changes!)       
  klassOop  klass = pool->klass_at(index, CHECK);    
  objArrayOop obj = oopFactory::new_objArray(klass, size, CHECK);
  thread->set_vm_result(obj);
IRT_END


IRT_ENTRY(void, InterpreterRuntime::multianewarray(JavaThread* thread, jint* first_size_address))
  // We may want to pass in more arguments - could make this slightly faster
  constantPoolOop constants = method()->constants();
  int          i = two_byte_index();
  klassOop klass = constants->klass_at(i, CHECK);
  int   nof_dims = number_of_dimensions();
  assert(oop(klass)->is_klass(), "not a class");
  assert(nof_dims >= 1, "multianewarray rank must be nonzero");
  oop obj = arrayKlass::cast(klass)->
              multi_allocate(nof_dims, 
                             first_size_address, 
                             frame::interpreter_frame_expression_stack_direction(),
                             CHECK);
  thread->set_vm_result(obj);
IRT_END


IRT_ENTRY(void, InterpreterRuntime::array_store_check(JavaThread* thread, oop value, int index, oop array))
  if (!value->is_a(objArrayKlass::cast(array->klass())->element_klass())) {
    THROW(vmSymbols::java_lang_ArrayStoreException());  
  }
IRT_END


IRT_ENTRY(void, InterpreterRuntime::checkcast(JavaThread* thread, oop obj))
  assert(obj != NULL, "must be a real object");
  Handle h_obj(thread, obj);
  klassOop klass = method()->constants()->klass_at(two_byte_index(), CHECK);
  if (!h_obj->is_a(klass)) {    
    ResourceMark rm;
    THROW_MSG(vmSymbols::java_lang_ClassCastException(), Klass::cast(h_obj->klass())->external_name());
  }
IRT_END


IRT_ENTRY(jint, InterpreterRuntime::instanceof(JavaThread* thread, oop obj))
  assert(obj != NULL, "must be a real object");
  Handle h_obj(thread, obj);    
  klassOop klass = method()->constants()->klass_at(two_byte_index(), CHECK_0);
  return h_obj()->is_a(klass) ? 1 : 0;  
IRT_END


//------------------------------------------------------------------------------------------------------------------------
// Exceptions

// Special handling for stack overflow: since we don't have any (java) stack
// space left we use the pre-allocated & pre-initialized StackOverflowError
// klass to create an stack overflw error instance. We do not call its
// constructor for the same reason (it is empty, anyway).

IRT_ENTRY(void, InterpreterRuntime::create_StackOverflowError(JavaThread* thread))
  // get klass
  instanceKlass* klass = instanceKlass::cast(SystemDictionary::StackOverflowError_klass());
  assert(klass->is_initialized(), "StackOverflowError klass should have been initialized during VM initialization");
  // create instance - do not call constructor since we have no (java) stack space left
  Handle exception(thread, klass->allocate_instance(thread));
  // check if another exception happened in the meantime
  if (thread->has_pending_exception()) {
    // another exception happened => use that one
    exception = Handle(thread, thread->pending_exception());
    thread->clear_pending_exception();
  } else {
    // no other exception happened => fill in stack trace if needed
    if (StackTraceInThrowable) {
      java_lang_Throwable::fill_in_stack_trace(exception);
    }
  }
  // return exception
  thread->set_vm_result(exception());
IRT_END


IRT_ENTRY(void, InterpreterRuntime::create_exception(JavaThread* thread, char* name, char* message))
  // lookup exception klass
  symbolHandle s = oopFactory::new_symbol(name, CHECK);
  // create exception 
  Handle exception = Exceptions::new_exception(thread, s, message, NULL, NULL);  
  thread->set_vm_result(exception());
IRT_END


// exception_handler_for_exception(...) returns the continuation address,
// the exception oop (via TLS) and sets the bci/bcp for the continuation.
// The exception oop is returned to make sure it is preserved over GC (it
// is only on the stack if the exception was thrown explicitly via athrow).
// During this operation, the expression stack contains the values for the
// bci where the exception happened. If the exception was propagated back
// from a call, the expression stack contains the values for the bci at the
// invoke w/o arguments (i.e., as if one were inside the call).
IRT_ENTRY(address, InterpreterRuntime::exception_handler_for_exception(JavaThread* thread, oop exception))
  Handle             h_exception(thread, exception);
  methodHandle       h_method   (thread, method());
  constantPoolHandle h_constants(thread, h_method->constants());
  typeArrayHandle    h_extable  (thread, h_method->exception_table());
  bool               should_repeat;
  int                handler_bci;
  int                current_bci = bcp() - h_method->code_base();
  
  do {
    should_repeat = false;

    // assertions
#ifdef ASSERT
    assert(h_exception.not_null(), "NULL exceptions should be handled by athrow");
    assert(h_exception->is_oop(), "just checking");
    // Check that exception is a subclass of Throwable, otherwise we have a VerifyError
    if (!(h_exception->is_a(SystemDictionary::throwable_klass()))) {
      if (ExitVMOnVerifyError) vm_exit(-1);
      ShouldNotReachHere();
    }
#endif

    // tracing
    if (TraceExceptions) {
      ThreadCritical tc;
      ResourceMark rm;
      tty->print_cr("Exception <%s> thrown in interpreter method <%s> for thread 0x%x",
		    h_exception->print_value_string(), h_method->print_value_string(), thread);
    } else if (h_extable->length() == 0) {
      // disabled for now - interpreter is not using shortcut yet
      // (shortcut is not to call runtime if we have no exception handlers)
      // warning("performance bug: should not call runtime if method has no exception handlers");
    }

    // exception handler lookup
    handler_bci = h_method->fast_exception_handler_bci_for(h_exception->klass(), current_bci, true, THREAD);
    if (HAS_PENDING_EXCEPTION) {
      // We threw an exception while trying to find the exception handler.
      // Transfer the new exception to the exception handle which will
      // be set into thread local storage, and do another lookup for an
      // exception handler for this exception, this time starting at the
      // BCI of the exception handler which caused the exception to be
      // thrown (bug 4307310).
      h_exception = Handle(PENDING_EXCEPTION);
      CLEAR_PENDING_EXCEPTION;
      if (handler_bci >= 0) {
	current_bci = handler_bci;
	should_repeat = true;
      }
    }
  } while (should_repeat == true);

  // notify jvmdi of an exception throw; jvmdi will detect if this is a first 
  // time throw or a stack unwinding throw and accordingly notify the debugger
  if (jvmdi::enabled()) {
    jvmdi::post_exception_throw_event(thread, h_method(), bcp(), h_exception());
  }

  address continuation = NULL;
  address handler_pc = NULL;
  if (handler_bci < 0) {
    // no handler in this method => forward exception to callee, leave bci/bcp untouched
    continuation = Interpreter::remove_activation_entry();

    // Count this for compilation purposes
    COMPILER2_ONLY(h_method->interpreter_throwout_increment());
  } else {
    // handler in this method => change bci/bcp to handler bci/bcp and continue there
    handler_pc = h_method->code_base() + handler_bci;
    set_bcp(handler_pc);
    continuation = Interpreter::dispatch_table(vtos)[*handler_pc];
  }
  // notify debugger of an exception catch 
  // (this is good for exceptions caught in native methods as well)
  if (jvmdi::enabled()) {
    jvmdi::post_exception_catch_event(thread, h_method(), handler_pc, h_exception());
  }

  thread->set_vm_result(h_exception());
  return continuation;
IRT_END


IRT_ENTRY(void, InterpreterRuntime::throw_pending_exception(JavaThread* thread))  
  assert(thread->has_pending_exception(), "must only ne called if there's an exception pending");
  // nothing to do - eventually we should remove this code entirely (see comments @ call sites)
IRT_END


IRT_ENTRY(void, InterpreterRuntime::throw_abstract_method_error(JavaThread* thread))          
  THROW(vmSymbols::java_lang_AbstractMethodError());
IRT_END


//------------------------------------------------------------------------------------------------------------------------
// Fields
//
// Rewrite a get/put field/static bytecode into the corresponding
// fast version of the bytecode.
//
// Note: The fast_xget/xputfield instructions do an implicit
//       NULL check which requires the offset to be within
//       a certain range. If the offset is outside this
//       range, an explicit NULL check is needed. Since this
//       is a rare case, a slow implementation for that case
//       is ok.

static Bytecodes::Code fast_get_code_for(bool is_static, TosState state, bool explicit_null_check) {
  if (is_static) {
    // no null check needed => ignore explicit_null_check flag
    // use type specific version
    switch (state) {
      case itos: return Bytecodes::_fast_igetstatic;
      case ltos: return Bytecodes::_fast_lgetstatic;
      case ftos: return Bytecodes::_fast_fgetstatic;
      case dtos: return Bytecodes::_fast_dgetstatic;
      case atos: return Bytecodes::_fast_agetstatic;
    }
  } else {
    if (explicit_null_check) {
      // no type-specific version
      return Bytecodes::_fast_getfield;
    } else {
      // use type specific version
      switch (state) {
        case itos: return Bytecodes::_fast_igetfield;
        case ltos: return Bytecodes::_fast_lgetfield;
        case ftos: return Bytecodes::_fast_fgetfield;
        case dtos: return Bytecodes::_fast_dgetfield;
        case atos: return Bytecodes::_fast_agetfield;
      }
    }
  }
  ShouldNotReachHere();
  return Bytecodes::_illegal;
}


static Bytecodes::Code fast_put_code_for(bool is_static, TosState state, bool explicit_null_check) {
  if (is_static) {
    // no null check needed => ignore explicit_null_check flag        
    // use type specific version
    switch (state) {
      case itos: return Bytecodes::_fast_iputstatic;
      case ltos: return Bytecodes::_fast_lputstatic;
      case ftos: return Bytecodes::_fast_fputstatic;
      case dtos: return Bytecodes::_fast_dputstatic;
      case atos: return Bytecodes::_fast_aputstatic;
    }    
  } else {
    if (explicit_null_check) {
      // no type-specific version
      return Bytecodes::_fast_putfield;
    } else {
      // use type specific version
      switch (state) {
        case itos: return Bytecodes::_fast_iputfield;
        case ltos: return Bytecodes::_fast_lputfield;
        case ftos: return Bytecodes::_fast_fputfield;
        case dtos: return Bytecodes::_fast_dputfield;
        case atos: return Bytecodes::_fast_aputfield;
      }
    }
  }
  ShouldNotReachHere();
  return Bytecodes::_illegal;
}


IRT_ENTRY(void, InterpreterRuntime::resolve_get_put(JavaThread* thread, Bytecodes::Code bytecode))
  // resolve field
  FieldAccessInfo info;
  constantPoolHandle pool(thread, method()->constants());
  bool is_static = (bytecode == Bytecodes::_getstatic || bytecode == Bytecodes::_putstatic);
  if (jvmdi::enabled()) {
    jvmdi::hide_single_stepping(thread);
  }
  LinkResolver::resolve_field(info, pool, two_byte_index(), bytecode, false, CHECK);
  if (jvmdi::enabled()) {
    jvmdi::expose_single_stepping(thread);
  }
  if (already_patched(bytecode)) return; // check if link resolution caused patching
  // compute auxiliary field attributes
  TosState state  = as_TosState(info.field_type());
  bool null_check = MacroAssembler::needs_explicit_null_check(info.field_offset() * wordSize);  
  // set field cache entry
  cache_entry()->set_field(
    fast_get_code_for(is_static, state, null_check),
    fast_put_code_for(is_static, state, null_check),
    info.klass(),
    info.field_index(),
    info.field_offset(),
    state,
    info.access_flags().is_final()
  );
IRT_END


//------------------------------------------------------------------------------------------------------------------------
// Synchronization
//
// The interpreter's synchronization code is factored out so that it can
// be shared by method invocation and synchronized blocks.
//%note synchronization_3

static void trace_locking(Handle& h_locking_obj, bool is_locking) {
  NOT_CORE(ObjectSynchronizer::trace_locking(h_locking_obj, false, true, is_locking);)
}


//%note monitor_1
IRT_ENTRY_FOR_MONITORENTER(void, InterpreterRuntime::monitorenter(JavaThread* thread, BasicObjectLock* elem))
#ifdef ASSERT
  thread->last_frame().interpreter_frame_verify_monitor(elem);
#endif
  Handle h_obj(elem->obj());  
  assert(h_obj.is_null() || Universe::is_heap(h_obj()), "must be NULL or an object");
  ObjectSynchronizer::slow_enter(h_obj, elem->lock(), CHECK);
  assert(elem->obj() == NULL || Universe::is_heap(elem->obj()), "must be NULL or an object");
#ifdef ASSERT
  thread->last_frame().interpreter_frame_verify_monitor(elem);
#endif
IRT_END


//%note monitor_1
IRT_ENTRY(void, InterpreterRuntime::monitorexit(JavaThread* thread, BasicObjectLock* elem))
#ifdef ASSERT
  thread->last_frame().interpreter_frame_verify_monitor(elem);
#endif
  Handle h_obj(elem->obj());  
  assert(h_obj.is_null() || Universe::is_heap(h_obj()), "must be NULL or an object");
  if (elem == NULL || h_obj()->is_unlocked()) {
    THROW(vmSymbols::java_lang_IllegalMonitorStateException());
  }
  ObjectSynchronizer::slow_exit(h_obj(), elem->lock(), thread);
  // Free entry. This must be done here, since a pending exception might be installed on
  // exit. If it is not cleared, the exception handling code will try to unlock the monitor again.
  elem->set_obj(NULL); 
#ifdef ASSERT
  thread->last_frame().interpreter_frame_verify_monitor(elem);
#endif
IRT_END


IRT_ENTRY(void, InterpreterRuntime::throw_illegal_monitor_state_exception(JavaThread* thread))  
  THROW(vmSymbols::java_lang_IllegalMonitorStateException());
IRT_END


IRT_ENTRY(void, InterpreterRuntime::new_illegal_monitor_state_exception(JavaThread* thread))
  // Returns an illegal exception to install into the current thread. The pending_exception
  // flag is cleared so normal exception handling does not trigger. Any current installed
  // exception will be overwritten. This method will be called during an exception unwind.  
  assert(!HAS_PENDING_EXCEPTION, "no pending exception");
  Handle exception(thread, thread->vm_result());
  assert(exception() != NULL, "vm result should be set");
  if (!exception->is_a(SystemDictionary::threaddeath_klass())) {        
    exception = Exceptions::new_exception(thread, 
                                          vmSymbols::java_lang_IllegalMonitorStateException(),
                                          NULL,                                          
                                          NULL,
                                          NULL);
  }
  thread->set_vm_result(exception());
IRT_END


//------------------------------------------------------------------------------------------------------------------------
// Invokes

static Bytecodes::Code fast_invoke_code_for(Bytecodes::Code bytecode, bool can_be_statically_bound) {
  switch (bytecode) {
    case Bytecodes::_invokevirtual  : return can_be_statically_bound ? Bytecodes::_fast_invokevfinal : Bytecodes::_fast_invokevtable;
    case Bytecodes::_invokestatic   : return Bytecodes::_fast_invokestatic;
    case Bytecodes::_invokespecial  : return Bytecodes::_fast_invokespecial;
    case Bytecodes::_invokeinterface: return Bytecodes::_fast_invokeinterface;
    default:                          break;
  }
  ShouldNotReachHere();
  return Bytecodes::_illegal;
}

IRT_ENTRY(Bytecodes::Code, InterpreterRuntime::get_original_bytecode_at(JavaThread* thread, methodOop method, address bcp))
  return method->orig_bytecode_at(method->bci_from(bcp));
IRT_END

IRT_ENTRY(void, InterpreterRuntime::set_original_bytecode_at(JavaThread* thread, methodOop method, address bcp, Bytecodes::Code new_code))
  method->set_orig_bytecode_at(method->bci_from(bcp), new_code);
IRT_END

IRT_ENTRY(void, InterpreterRuntime::_breakpoint(JavaThread* thread, methodOop method, address bcp))
  Bytecodes::Code orig_code = method->orig_bytecode_at(method->bci_from(bcp));
  if (orig_code == Bytecodes::_ldc ||
      orig_code == Bytecodes::_ldc_w || 
      orig_code == Bytecodes::_ldc2_w) {
    // ldc opcode will be patched to fast ldc opcode and another breakpoint will be posted 
    // for the same bcp at that time.  So skip not to post this breakpoint.
    return;
  }
  
  JvmdiThreadState *state = thread->jvmdi_thread_state();
  state->compare_and_set_current_location(method, bcp, JVMDI_EVENT_BREAKPOINT);
  if (!state->breakpoint_posted()) {
    ThreadState old_state = thread->osthread()->get_state();
    thread->osthread()->set_state(BREAKPOINTED);
    jvmdi::post_breakpoint_event(thread, method, bcp);
    thread->osthread()->set_state(old_state);
  }
IRT_END

IRT_ENTRY(void, InterpreterRuntime::resolve_invoke(JavaThread* thread, Bytecodes::Code bytecode))  
  // extract receiver from the outgoing argument list if necessary
  Handle receiver(thread, NULL);  
  if (bytecode == Bytecodes::_invokevirtual || bytecode == Bytecodes::_invokeinterface) {
    ResourceMark rm;    
    methodHandle m = method();    
    int bci = m->bci_from(bcp());    
    Bytecode_invoke* call = Bytecode_invoke_at(m, bci);    
    symbolHandle signature = call->signature();        
    receiver = thread->last_frame().interpreter_callee_receiver(signature);
    assert(Universe::is_heap_or_null(receiver()), "sanity check");    
    assert(receiver.is_null() || Universe::is_heap(receiver->klass()), "sanity check");
  }  

  // resolve method
  CallInfo info;
  constantPoolHandle pool(thread, method()->constants());

  if (jvmdi::enabled()) {
    jvmdi::hide_single_stepping(thread);
  }
  LinkResolver::resolve_invoke(info, receiver, pool, two_byte_index(), bytecode, CHECK);
  if (jvmdi::enabled()) {
    jvmdi::expose_single_stepping(thread);
  }
  if (already_patched(bytecode)) return; // check if link resolution caused patching

  if (bytecode == Bytecodes::_invokeinterface) {    
    if (Klass::can_be_statically_bound(info.resolved_method()())) {
      cache_entry()->set_method(
        fast_invoke_code_for(bytecode, true), // Statically bound
        info.selected_method(),
        info.vtable_index()
      ); 
    } else {          
      if (TraceItables && Verbose) {
        ResourceMark rm;
        tty->print_cr("Resolving: klass: %s to method: %s", info.resolved_klass()->name()->as_C_string(), info.resolved_method()->name()->as_C_string());
      }      
      if (info.resolved_method()->method_holder() != SystemDictionary::object_klass()) {
        // Setup itable entry      
        int index = klassItable::compute_itable_index(info.resolved_method()());
        cache_entry()->set_interface_call(info.resolved_method(), index);
      } else {
        // NOTE: THIS IS A TEMPORARY FIX FOR A CORNER CASE PROBLEM (see also cpCacheOop.cpp for details)
        assert(info.vtable_index() == -1 && info.resolved_method()->vtable_index() != -1, "sanity check");
        cache_entry()->set_method(
          fast_invoke_code_for(Bytecodes::_invokevirtual, false),
          info.resolved_method(),
          info.resolved_method()->vtable_index(),
          true // see cpCacheOop.cpp for details
        );
        // set_method rewrites the invokeinterface into an invokvirtual which leaves
        // two extra bytes after the rewrite. In order to ensure correct behaviour,
        // we need to rewrite them into nop bytecodes. The last bytecode is always
        // zero according do JVM Spec; so we only need to take care of the 2nd last
        // bytecode.
        bcp()[3] = (char)Bytecodes::_nop;
        assert(bcp()[4] == (char)Bytecodes::_nop, "must be a nop bytecode according to JVM Spec");
      }
    }
  } else {    
    // For CheckItables, the method must be the one which method_holder() is the interface we
    // are calling through. This branch can be removed when CheckItables goes away.
    cache_entry()->set_method(
      fast_invoke_code_for(bytecode, Klass::can_be_statically_bound(info.resolved_method()())),
      info.resolved_method(),
      info.vtable_index()
    );     
  }
IRT_END


#ifndef CORE
IRT_ENTRY(address, InterpreterRuntime::nmethod_entry_point(JavaThread* thread, methodOop method, nmethod* nm))  
  methodHandle m(thread, method);    
  { debug_only(nmethod* nm2 = m->code());
    // Note: nm2 could be null, if an uncommon trap for the method happened right before we entered here
    assert(nm != NULL && (nm2 == NULL || nm == nm2), "nmethods must match");
  }
  // Sets the interpreter_entry_point as a side effect.
  address i2c_entry = nm->interpreter_entry_point();
  thread->set_vm_result(m());
  return i2c_entry;
IRT_END
#endif


//------------------------------------------------------------------------------------------------------------------------
// Miscellaneous


#ifndef CORE
// This method returns continuation information in the event of an invocation counter
// overflow.  For normal compilations, this returns a jlong whose low word is the
// continuation address.  For on stack replacement continuation, this code returns a
// jlong whose high word is the osr_adapter_entry and whose low word is the on stack
// replacement nmethod.
IRT_ENTRY(jlong, InterpreterRuntime::invocation_counter_overflow(JavaThread* thread, bool in_loop, int extra_frame_size))
  frame fr = thread->last_frame();
  assert(fr.is_interpreted_frame(), "must come from interpreter");
  methodHandle method(thread, fr.interpreter_frame_method());  
  const int bci = fr.interpreter_frame_bcp() - method->code_base();
  // trace overflow
  if (TraceInvocationCounterOverflow) {
    InvocationCounter* c = method->invocation_counter();
    ResourceMark rm;
    const char* msg =
      in_loop
        ? "cntr ovfl @ %d in loop of "
        : "cntr ovfl @ %d in entry of ";
    tty->print(msg, bci);
    method->print_value();
    tty->cr();
    c->print();
  }
  // invoke invocation counter state specific action
  if (in_loop) {
    // invocation counter overflow in a loop => try to do on-stack-replacement
    nmethod* osr_code = OnStackReplacement::get_osr_code(method, bci, CHECK_0);
    if (osr_code != NULL) {
      if (UseSpecLock) {
        if (PrintSpecLock) { tty->print_cr("fixup all objects before OSR"); } 
        SpecLock::fixup_all_objects_on_stack(thread);
      }
  
      // continue w/ on-stack-replacement code
#ifdef COMPILER1
      // no osr_adapter_frame_return_address needed
      address osr_adapter_frame_return_address = NULL;
#else // COMPILER2
      // compute osr_adapter_frame_return_address
      int osr_frame_size = fr.frame_size() + extra_frame_size + method->size_of_parameters();
      OSRAdapter* osr = OnStackReplacement::get_osr_adapter(osr_frame_size);
      address osr_adapter_frame_return_address = osr->instructions_begin();
#endif // COMPILER1
      // return 2 values as 64bit entity
      return jlong_from((jint)osr_adapter_frame_return_address, (jint)osr_code);
    }
    // on-stack-replacement not possible

    {  // Invocation counter is in methodOop, so make sure to read right before use.
       InvocationCounter* c = method->invocation_counter();
       c->set( c->state(), CompileThreshold );
    }
    return NULL;
  }
  // handle 'normal' invocation counter overflow
  return (jlong)method->invocation_counter()->action()(method, CHECK_0);
IRT_END
#endif // CORE


IRT_ENTRY(void, InterpreterRuntime::at_safepoint(JavaThread* thread))
  // We used to need an explict preserve_arguments here for invoke bytecodes. However,
  // stack traversal automatically takes care of preserving arguments for invoke, so
  // this is no longer needed.

  // IRT_END does an implicit safepoint check, hence we are guaranteed to block
  // if this is called during a safepoint

  if (jvmdi::enabled() &&
      (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_SINGLE_STEP) ||
      (!UseFastBreakpoints && jvmdi::is_bytecode_stepping()))) {
    // We are called during regular safepoints and when the VM is
    // single stepping. If this thread is marked for single stepping,
    // then we have JVM/DI work to do.
    // If we aren't using fast breakpoints, then we have JVM/DI work
    // to do for every thread.
    jvmdi::at_single_stepping_point(thread, method(), bcp());
  }
IRT_END

IRT_ENTRY(void, InterpreterRuntime::post_field_access(JavaThread *thread, oop obj,
ConstantPoolCacheEntry *cp_entry))

  // check the access_flags for the field in the klass
  instanceKlass* ik = instanceKlass::cast((klassOop)cp_entry->f1());
  typeArrayOop fields = ik->fields();
  int index = cp_entry->holder_index();
  assert(index < fields->length(), "holder index is out of range");
  // bail out if field accesses are not watched
  if ((fields->ushort_at(index) & JVM_ACC_FIELD_ACCESS_WATCHED) == 0) return;

  bool is_static = false;

  // make sure we have the right bytecode and set is_static
  switch (cp_entry->bytecode_1()) {
    case Bytecodes::_fast_getfield:   // fall through
    case Bytecodes::_fast_igetfield:  // fall through
    case Bytecodes::_fast_lgetfield:  // fall through
    case Bytecodes::_fast_fgetfield:  // fall through
    case Bytecodes::_fast_dgetfield:  // fall through
    case Bytecodes::_fast_agetfield:  break;
    case Bytecodes::_fast_putstatic:  // fall through
    case Bytecodes::_fast_igetstatic: // fall through
    case Bytecodes::_fast_lgetstatic: // fall through
    case Bytecodes::_fast_fgetstatic: // fall through
    case Bytecodes::_fast_dgetstatic: // fall through
    case Bytecodes::_fast_agetstatic: is_static = true; break;
    default:                          ShouldNotReachHere(); return;
  }

  HandleMark hm(thread);

  if (is_static) {
    // static field accessors have no object
    KlassHandle h_klass(cp_entry->f1());
    jvmdi::post_field_access_event(thread, method(), bcp(), h_klass, NULL,
      jfieldIDWorkaround::to_jfieldID((klassOop)cp_entry->f1(), cp_entry->f2(), true));
  } else {
    // non-static field accessors have an object, but we need a handle
    assert(obj != NULL, "non-static needs an object");
    KlassHandle h_klass(cp_entry->f1());
    Handle h_obj(thread, obj);
    jvmdi::post_field_access_event(thread, method(), bcp(), h_klass, h_obj,
      jfieldIDWorkaround::to_jfieldID((klassOop)cp_entry->f1(), cp_entry->f2(), false));
  }
IRT_END

IRT_ENTRY(void, InterpreterRuntime::post_field_modification(JavaThread *thread,
  oop obj, ConstantPoolCacheEntry *cp_entry, jvalue *value))

  klassOop k = (klassOop)cp_entry->f1();

  // check the access_flags for the field in the klass
  instanceKlass* ik = instanceKlass::cast(k);
  typeArrayOop fields = ik->fields();
  int index = cp_entry->holder_index();
  assert(index < fields->length(), "holder index is out of range");
  // bail out if field modifications are not watched
  if ((fields->ushort_at(index) & JVM_ACC_FIELD_MODIFICATION_WATCHED) == 0) return;

  bool   is_static = false;
  char   sig_type = NULL;

  // make sure we have the right bytecode and set bytecode specific values
  switch (cp_entry->bytecode_2()) {
    // XXX - fix _fast_putfield
    case Bytecodes::_fast_putfield:   sig_type = 'X';                   break;
    case Bytecodes::_fast_iputfield:  sig_type = 'I';                   break;
    case Bytecodes::_fast_lputfield:  sig_type = 'J';                   break;
    case Bytecodes::_fast_fputfield:  sig_type = 'F';                   break;
    case Bytecodes::_fast_dputfield:  sig_type = 'D';                   break;
    case Bytecodes::_fast_aputfield:  sig_type = 'L';                   break;
    // XXX - fix _fast_putstatic
    case Bytecodes::_fast_putstatic:  sig_type = 'X'; is_static = true; break;
    case Bytecodes::_fast_iputstatic: sig_type = 'I'; is_static = true; break;
    case Bytecodes::_fast_lputstatic: sig_type = 'J'; is_static = true; break;
    case Bytecodes::_fast_fputstatic: sig_type = 'F'; is_static = true; break;
    case Bytecodes::_fast_dputstatic: sig_type = 'D'; is_static = true; break;
    case Bytecodes::_fast_aputstatic: sig_type = 'L'; is_static = true; break;
    default:                          ShouldNotReachHere(); return;
  }

  HandleMark hm(thread);
  KlassHandle h_klass(k);
  jfieldID fid = jfieldIDWorkaround::to_jfieldID(k, cp_entry->f2(), is_static);

  if (sig_type == 'I') {
    // 'I' instructions are used for byte, char, short and int.
    // determine which it really is, and convert
    fieldDescriptor fd;
    bool found = JvmdiInternal::get_field_descriptor(k, fid, &fd);
    // should be found (if not, leave as is)
    if (found) {
      jint ival = value->i;
      // convert value from int to appropriate type
      switch (fd.field_type()) {   
      case T_BYTE:
        sig_type = 'B';
        value->i = 0; // clear it
        value->b = (jbyte)ival;
        break;
      case T_CHAR:
        sig_type = 'C';
        value->i = 0; // clear it
        value->c = (jchar)ival;
        break;
      case T_SHORT:
        sig_type = 'S';
        value->i = 0; // clear it
        value->s = (jshort)ival;
        break;
      case T_INT:
        // nothing to do
        break;
      default:
        // this is an integer instruction, should be one of above
        ShouldNotReachHere();
        break;
      }
    }
  }
  if (is_static) {
    // static field setters have no object
    jvmdi::post_field_modification_event(thread, method(), bcp(), h_klass, NULL,
                                         fid, sig_type, value);
  } else {
    // non-static field accessors have an object, but we need a handle
    assert(obj != NULL, "non-static needs an object");
    Handle h_obj(thread, obj);
    jvmdi::post_field_modification_event(thread, method(), bcp(), h_klass, h_obj,
                                         fid, sig_type, value);
  }
IRT_END

IRT_ENTRY(void, InterpreterRuntime::post_method_entry(JavaThread *thread))
  jvmdi::post_method_entry_event(thread, InterpreterRuntime::method(), InterpreterRuntime::last_frame());
IRT_END


IRT_ENTRY(void, InterpreterRuntime::post_method_exit(JavaThread *thread))
  jvmdi::post_method_exit_event(thread, InterpreterRuntime::method(), InterpreterRuntime::last_frame());
IRT_END

