/*
 * @(#)interpreterRuntime.hpp	1.104 00/06/29 14:40:47
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// The InterpreterRuntime is called by the interpreter for everything
// that cannot/should not be dealt with in assembly and needs C support.

class InterpreterRuntime: AllStatic {
  friend class BytecodeClosure; // for method and bcp
  friend class PrintingClosure; // for method and bcp

 private:
  // Helper functions to access current interpreter state
  static frame     last_frame()                  { return JavaThread::active()->last_frame(); }  
  static methodOop method()                      { return                       last_frame().interpreter_frame_method(); }
  static address   bcp()                         { return                       last_frame().interpreter_frame_bcp(); }
  static void      set_bcp(address bcp)          {                              last_frame().interpreter_frame_set_bcp(bcp); }
  static Bytecodes::Code code()                  { return Bytecodes::code_at(bcp()); }
  static bool      already_patched(Bytecodes::Code old_code) { return code() != old_code; }
  static int       one_byte_index()              { return bcp()[1]; }
  static int       two_byte_index()              { return Bytes::get_Java_u2(bcp() + 1); }
  static int       number_of_dimensions()        { return bcp()[3]; }
  static ConstantPoolCacheEntry* cache_entry()   { return method()->constants()->cache()->entry_at(Bytes::get_native_u2(bcp() + 1)); }

  // Miscellanous
  static void    ldc_helper(Bytecodes::Code bytecode, int index, Bytecodes::Code ildc, Bytecodes::Code fldc, Bytecodes::Code aldc, TRAPS);

 public:
  // Constants
  static void    ldc   (JavaThread* thread);
  static void    ldc_w (JavaThread* thread);
  static void    ldc2_w(JavaThread* thread);

  // Allocation
  static void    _new          (JavaThread* thread, constantPoolOop pool, int index);
  static void    newarray      (JavaThread* thread, BasicType type, jint size);
  static void    anewarray     (JavaThread* thread, constantPoolOop pool, int index, jint size);
  static void    multianewarray(JavaThread* thread, jint* first_size_address);

  // Type tests
  static void    array_store_check(JavaThread* thread, oop value, int index, oop array);
  static void    checkcast        (JavaThread* thread, oop obj);
  static jint    instanceof       (JavaThread* thread, oop obj);

  // Exceptions
  static void    throw_abstract_method_error(JavaThread* thread);
  static void    create_StackOverflowError(JavaThread* thread);
  static void    create_exception(JavaThread* thread, char* name, char* message);
  static address exception_handler_for_exception(JavaThread* thread, oop exception);
  static void    throw_pending_exception(JavaThread* thread);

  // Statics & fields
  static void    resolve_get_put(JavaThread* thread, Bytecodes::Code bytecode);  
  
  // Synchronization
  static void    monitorenter(JavaThread* thread, BasicObjectLock* elem);
  static void    monitorexit (JavaThread* thread, BasicObjectLock* elem);
  
  static void    fixup_speclock(JavaThread* thread, oop obj);

  static void    throw_illegal_monitor_state_exception(JavaThread* thread);
  static void    new_illegal_monitor_state_exception(JavaThread* thread);

  // Calls
  static void    resolve_invoke     (JavaThread* thread, Bytecodes::Code bytecode);

  // Breakpoints
  static void _breakpoint(JavaThread* thread, methodOop method, address bcp);
  static Bytecodes::Code get_original_bytecode_at(JavaThread* thread, methodOop method, address bcp);
  static bool is_breakpoint() { return Bytecodes::code_or_bp_at(bcp()) == Bytecodes::_breakpoint; }
  static void            set_original_bytecode_at(JavaThread* thread, methodOop method, address bcp, Bytecodes::Code new_code);

#ifndef CORE
  static address nmethod_entry_point(JavaThread* thread, methodOop m, nmethod* nm);

  // Invocation counter overflow
  static jlong invocation_counter_overflow(JavaThread* thread, bool in_loop, int extra_frame_size);  
#endif

  // Safepoints
  static void    at_safepoint(JavaThread* thread);

  // Debugger support
  static void post_field_access(JavaThread *thread, oop obj,
    ConstantPoolCacheEntry *cp_entry);
  static void post_field_modification(JavaThread *thread, oop obj,
    ConstantPoolCacheEntry *cp_entry, jvalue *value);
  static void post_method_entry(JavaThread *thread);
  static void post_method_exit (JavaThread *thread);

  // Platform dependent stuff
  #include "incls/_interpreterRT_pd.hpp.incl"
};

