/*
 * @(#)shared.hpp	1.16 99/08/18 16:13:35
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

//------------------------------OptoReg----------------------------------------
// We eventually need Registers for the Real World.  Registers are essentially
// non-SSA names.  A Register is represented as a number.  Non-regular values
// (e.g., Control, Memory, I/O) use the Special register.  The actual machine
// registers (as described in the ADL file for a machine) start at zero.
// Stack-slots (spill locations) start at the nest Chunk past the last machine
// register.
//
// Note that stack spill-slots are treated as a very large register set.
// They have all the correct properties for a Register: not aliased (unique
// named).  There is some simple mapping from a stack-slot register number
// to the actual location on the stack; this mapping depends on the calling
// conventions and is described in the ADL.
//
// This silly enum is my attempt to be both efficient and type-safe.  This
// really should be a C++ class, but passing and returning structures is
// usually much worse than passing/returning an int - even when the structure
// is merely a wrapper for an int.  Class OptoReg is merely used as a way
// to control the C++ namespace.
class OptoReg VALUE_OBJ_CLASS_SPEC { 
 public:
  enum Name {
    // Chunk 0
#ifdef COMPILER2
    Physical = AdlcVMDeps::Physical, // Start of physical regs 
#endif
    // A few oddballs at the edge of the world
    Special = -2,		// All special (not allocated) values
    Bad = -1			// Not a register
  };

  // Increment a register number.  As in:
  //    "for ( OptoReg::Name i; i=Control; i = add(i,1) ) ..."
  static Name add( Name x, int y ) { return Name(x+y); }

  // (We would like to have an operator+ for RegName, but it is not
  // a class, so this would be illegal in C++.)

  static void dump( int );
};

//------------------------------VMReg------------------------------------------
// The VM uses 'unwarped' stack slots; the compiler uses 'warped' stack slots.
// Register numbers below SharedInfo::stack0 are the same for both.  Register
// numbers above stack0 are either warped (in the compiler) or unwarped
// (in the VM).  Unwarped numbers represent stack indices, offsets from
// the current stack pointer.  Warped numbers are required during compilation
// when we do not yet know how big the frame will be.
class VMReg VALUE_OBJ_CLASS_SPEC { 
public:
  enum Name { };
};


//------------------------------FreqPair--------------------------------------
// The Interpreter collects branch frequencies, and the compiler uses them to
// guide optimizations and register allocation.  Each branch is associated by
// BCP to a slot in a global table of FreqPairs.  These pairs contain counts
// of how many times a branch was taken, and how many times it was not taken.
// For simplicity & convenience, these pairs are pairs of uints.
struct FreqPair {
  unsigned int _taken;       // Count of times that branch is taken
  unsigned int _not_taken;   // Count of times that branch is not taken
  static int taken_offset() { return (int)&((FreqPair*)NULL)->_taken; }
  static int not_taken_offset() { return (int)&((FreqPair*)NULL)->_not_taken; }
};

//------------------------------SharedInfo------------------------------------
// Interesting bits shared between the compiled High Performance system
// and the core interpreted-only system.  For the interpreted-only system
// these bits take on default values (usually zero).  For the compiled
// system, the compiler initialization overwrites them.

class SharedInfo : AllStatic {
 public:
  // Names for registers
  static const char *regName[128];

  // Stack pointer register
  static OptoReg::Name c_frame_pointer;

  static OptoReg::Name stack0;

  // Convert register numbers to stack slots and vice versa
  static OptoReg::Name stack2reg( int idx ) { 
    return OptoReg::Name(stack0+idx); }

  static unsigned reg2stack( OptoReg::Name r ) {
    assert( r >=stack0, "Not a stack-based register" );
    return r - stack0; 
  }

  static void set_stack0(int n);
  static void set_regName();

  // Branch Frequency Tracking Table
  static FreqPair *freqTable;

  // Flush the Branch Frequency Tracking Table
  static void flush_branch_freq_table(void);
  // Dump the Branch Frequency Tracking Table
  static void dump_freqTable(void);

  // Call Frequency Tracking Table
  static int *callTable;

  // Flush the Branch Frequency Tracking Table
  static void flush_call_freq_table(void);
  // Dump the Branch Frequency Tracking Table
  static void dump_callTable(void);

};
