/*
 * @(#)ageTable.cpp	1.13 00/08/10 18:02:23
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

/*  Copyright 1992-2001 Sun Microsystems, Inc. and Stanford University.
   See the LICENSE file for license information. */

# include "incls/_precompiled.incl"
# include "incls/_ageTable.cpp.incl"

void ageTable::clear() {
  Memory::set_words(&sizes[0], table_size, 0);
}

int ageTable::compute_tenuring_threshold(size_t survivor_capacity) {
  size_t desired_survivor_size = (((double) survivor_capacity)*TargetSurvivorRatio)/100;
  size_t total = 0;
  int age = 1;
  assert(sizes[0] == 0, "no objects with age zero should be recorded");
  while (age < table_size) {
    total += sizes[age];
    // check if including objects of age 'age' made us pass the desired
    // size, if so 'age' is the new threshold
    if (total > desired_survivor_size) break;
    age++;
  }
  int result = age < MaxTenuringThreshold ? age : MaxTenuringThreshold;

  if (PrintTenuringDistribution) {
    tty->cr();
    tty->print_cr("Desired survivor size %d bytes, new threshold %d (max %d)", 
      desired_survivor_size*oopSize, result, MaxTenuringThreshold);
    total = 0;
    age = 1;
    while (age < table_size) {
      total += sizes[age];
      if (sizes[age] > 0) {
        tty->print_cr("- age %3d: %8d bytes, %8d total", age, sizes[age]*oopSize, total*oopSize);
      }
      age++;
    }
  }

  return result;
}
