/*
 * @(#)allocation_inline.hpp	1.8 00/03/21 16:16:57
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Explicit C-heap memory management

void trace_heap_malloc(int size, const char* name, void *p);
void trace_heap_free(void *p);
void vm_exit_out_of_memory(int size, const char* name);


// allocate using malloc; will fail if no memory available
inline char* AllocateHeap(int size, const char* name = NULL) {
  char* p = (char*) os::malloc(size);
  #ifdef ASSERT
  if (PrintHeapAllocation) trace_heap_malloc(size, name, p);
  #else
  Unused_Variable(name);
  #endif
  if (p == NULL) vm_exit_out_of_memory(size, name);
  return p;
}

inline char* ReallocateHeap(char *old, int size, const char* name = NULL) {
  char* p = (char*) os::realloc(old,size);
  #ifdef ASSERT
  if (PrintHeapAllocation) trace_heap_malloc(size, name, p);
  #else
  Unused_Variable(name);
  #endif
  if (p == NULL) vm_exit_out_of_memory(size, name);
  return p;
}

inline void FreeHeap(void* p) {
  #ifdef ASSERT
  if (PrintHeapAllocation) trace_heap_free(p);
  #endif
  os::free(p);
}

#ifdef ASSERT
inline void* Arena::malloc(size_t size) {
  assert(UseMallocOnly, "shouldn't call");
  // use malloc, but save pointer in res. area for later freeing
  char** save = (char**)internal_malloc_4(sizeof(char*));
  return (*save = (char*)os::malloc(size));
}

// for debugging with UseMallocOnly
inline void* Arena::internal_malloc_4(size_t x) { 
  assert( (x&3) == 0, "misaligned size" );
  if (_hwm + x > _max) {
    return grow(x);
  } else {
    char *old = _hwm; 
    _hwm += x; 
    return old; 
  }
}
#endif


