/*
 * @(#)carRememberedSet.hpp	1.15 99/07/06 16:40:44
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// Remembered set for single car. Records areas in higher cars or perm space
// holding references into this car. The entries are pointers into the card
// marking array, with the recorded cards mapping to the areas holding references.
// The information is thus not exact, requiring scanning of a small memory area
// to locate the actual reference(s).
//
// Implemented as a closed hash table.

class CarRememberedSet: public CHeapObj {
 private:
  enum SomeConstants {
    initial_capacity = 512
  };
  int     _size;              // number of elements inserted
  int     _capacity_mask;     // set capacity minus one
  jbyte** _set;               // array
  bool    _being_processed;   // set during scavenge of this set

  void allocate(int capacity);
  void grow_and_rehash(int grow_factor);

  inline int index_for(jbyte* card) const { return (juint) card & _capacity_mask; }
  inline int next_index(int index) const  { return (index + 1)  & _capacity_mask; }

 public:
  // Constructors
  CarRememberedSet()    { allocate(initial_capacity); }
  ~CarRememberedSet();
  // Current number of elements
  int size() const      { return _size; }
  // Set capacity
  int capacity() const  { return _capacity_mask + 1; }
  // Insert new entry (card corresponding to root from)
  void add_reference(oop* from);
  // Reset
  void clear();
  void reinitialize(bool already_empty);
  // Garbage collection support
  void scavenge_higher_train_recorded_stores();
  void scavenge_same_train_recorded_stores();
  bool has_other_train_references(Train* t);
  void scavenge_special_nonoop_recorded_stores(bool exclude_lowest_car, bool exclude_lowest_train);

  // Debugging
  void verify(CarSpace* holder) PRODUCT_RETURN;

#ifndef PRODUCT
  bool contains_reference(oop* from) const;
  void verify_is_cleared() const;
  void print(CarSpace* holder) const;
#endif
};
