/*
 * @(#)cartable.cpp	1.22 99/06/24 14:26:18
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_cartable.cpp.incl"


const julong CarTableDesc::tenuredgen_train_number = (julong)-2;
const julong CarTableDesc::permgen_train_number    = (julong)-1;


void CarTableDesc::initialize(julong train_number, juint car_number, Train* train, Space* sp, bool should_scavenge) {
  _train_number = train_number;
  _car_number = car_number;
  _train = train;
  _space = sp;
  _should_scavenge = should_scavenge;
}


CarTableDesc* CarTable::_table = NULL;
CarTableDesc* CarTable::_table_base;
int           CarTable::_table_size;


static inline int get_index(oop* p) {
  assert(((juint) p & ((1 << LogOfCarSpaceSize)-1)) == 0, "boundary pointer must be valid for CarTable");
  return (juint) p >> LogOfCarSpaceSize;
}


void CarTable::initialize() {
  // Skip if already initialized (bootstrapping problem)
  if (_table != NULL) return;

  Space* eden = Universe::new_gen()->eden();
  Space* from = Universe::new_gen()->from();
  Space* to   = Universe::new_gen()->to();

  int newgen_low_index   = get_index( Universe::new_gen()->low_boundary   );
  int newgen_high_index  = get_index( Universe::new_gen()->high_boundary  );
  int oldgen_low_index   = get_index( Universe::old_gen()->low_boundary   );
  int oldgen_high_index  = get_index( Universe::old_gen()->high_boundary  );
  int permgen_low_index  = get_index( Universe::perm_gen()->low_boundary  );
  int permgen_high_index = get_index( Universe::perm_gen()->high_boundary );
  assert(newgen_high_index == oldgen_low_index, "checking index");
  assert(oldgen_high_index == permgen_low_index, "checking index");

  _table_size = permgen_high_index - newgen_low_index;
  _table      = NEW_C_HEAP_ARRAY(CarTableDesc, _table_size);
  _table_base = _table - newgen_low_index;

  // Initialize table for new gen
  initialize_new_generation();
  
  // Initialize table for old gen
  int n;
  if (UseTrainGC) {
    for (n = oldgen_low_index; n < oldgen_high_index; n++) {
      _table_base[n].initialize(CarTableDesc::invalid_train_number, CarTableDesc::invalid_car_number, NULL, NULL, false);
    }
  } else {
    for (n = oldgen_low_index; n < oldgen_high_index; n++) {
      _table_base[n].initialize(CarTableDesc::tenuredgen_train_number, CarTableDesc::invalid_car_number, NULL, Universe::tenured_gen()->the_space(), false);
    }
  }

  // Initialize table for perm gen
  for (n = permgen_low_index; n < permgen_high_index; n++) {
    _table_base[n].initialize(CarTableDesc::permgen_train_number, CarTableDesc::invalid_car_number, NULL, Universe::perm_gen()->the_space(), false);
  }
}


void CarTable::initialize_new_generation() {
  NewGeneration* newgen = Universe::new_gen();
  Space* eden = newgen->eden();
  Space* from = newgen->from();
  Space* to   = newgen->to();

  int eden_low_index     = get_index( eden->bottom() );
  int eden_high_index    = get_index( eden->end()    );
  int from_low_index     = get_index( from->bottom() );
  int from_high_index    = get_index( from->end()    );
  int to_low_index       = get_index( to->bottom()   );
  int to_high_index      = get_index( to->end()      );
  int newgen_high_index  = get_index( newgen->high_boundary  );
  assert(eden_high_index == from_low_index && from_high_index == to_low_index, "just checking");
  assert(to_high_index <= newgen_high_index, "just checking");

  // Initialize table for eden
  int n;
  for (n = eden_low_index; n < eden_high_index; n++) {
    _table_base[n].initialize(CarTableDesc::newgen_train_number, CarTableDesc::invalid_car_number, NULL, eden, true);
  }

  // Initialize table for from
  for (n = from_low_index; n < from_high_index; n++) {
    _table_base[n].initialize(CarTableDesc::newgen_train_number, CarTableDesc::invalid_car_number, NULL, from, true);
  }

  // Initialize table for to
  for (n = to_low_index; n < to_high_index; n++) {
    _table_base[n].initialize(CarTableDesc::newgen_train_number, CarTableDesc::invalid_car_number, NULL, to, false);
  }

  // Initialize table for uncommitted part (for debugging only)
  for (n = to_high_index; n < newgen_high_index; n++) {
    _table_base[n].initialize(CarTableDesc::invalid_train_number, CarTableDesc::invalid_car_number, NULL, NULL, false);
  }
}


void CarTable::update_entry(CarSpace* car, julong train_number, juint car_number, Train* train) {
  CarTableDesc* d = car->desc();
  int blocks = car->blocks();
  while (blocks--) {
    d->initialize(train_number, car_number, train, car, true);
    d++;
  }
}


void CarTable::clear_entry(CarSpace* car) {
  CarTableDesc* d = car->desc();
  int blocks = car->blocks();
  while (blocks--) {
    d->initialize(CarTableDesc::invalid_train_number, CarTableDesc::invalid_car_number, NULL, NULL, false);
    d++;
  }
}


void CarTable::update_after_scavenge() {
  Space* from = Universe::new_gen()->from();
  Space* to = Universe::new_gen()->to();
  int from_low_index     = get_index( from->bottom() );
  int from_high_index    = get_index( from->end()    );
  int to_low_index       = get_index( to->bottom()   );
  int to_high_index      = get_index( to->end()      );
  assert(from_high_index == to_low_index || to_high_index == from_low_index, "just checking");
  // Set scavenge flag for from space, and update space pointer
  int n;
  for (n = from_low_index; n < from_high_index; n++) {
    _table_base[n].set_should_scavenge(true);
    _table_base[n].set_space(from);
  }
  // Clear scavenge flag for to space, and update space pointer
  for (n = to_low_index; n < to_high_index; n++) {
    _table_base[n].set_should_scavenge(false);
    _table_base[n].set_space(to);
  }
}


#ifndef PRODUCT

void CarTable::verify() {
  Space* eden = Universe::new_gen()->eden();
  Space* from = Universe::new_gen()->from();
  Space* to   = Universe::new_gen()->to();

  int eden_low_index     = get_index( eden->bottom() );
  int eden_high_index    = get_index( eden->end()    );
  int from_low_index     = get_index( from->bottom() );
  int from_high_index    = get_index( from->end()    );
  int to_low_index       = get_index( to->bottom()   );
  int to_high_index      = get_index( to->end()      );
  int newgen_low_index   = get_index( Universe::new_gen()->low_boundary   );
  int newgen_high_index  = get_index( Universe::new_gen()->high_boundary  );
  int oldgen_low_index   = get_index( Universe::old_gen()->low_boundary   );
  int oldgen_high_index  = get_index( Universe::old_gen()->high_boundary  );
  int permgen_low_index  = get_index( Universe::perm_gen()->low_boundary  );
  int permgen_high_index = get_index( Universe::perm_gen()->high_boundary );
  guarantee(eden_high_index == from_low_index || eden_high_index == to_low_index, "just checking");
  guarantee(from_high_index == to_low_index || to_high_index == from_low_index, "just checking");
  guarantee(newgen_high_index <= oldgen_low_index, "checking index");
  guarantee(oldgen_high_index == permgen_low_index, "checking index");

  // Verify table for eden
  int n;
  for (n = eden_low_index; n < eden_high_index; n++) {
    CarTableDesc* d = &(_table_base[n]);
    guarantee(d->train_number() == CarTableDesc::newgen_train_number,  "Illegal car table entry");
    guarantee(d->car_number() == CarTableDesc::invalid_car_number,     "Illegal car table entry");
    guarantee(d->train() == NULL,                     "Illegal car table entry");
    guarantee(d->space() == eden,                     "Illegal car table entry");
    guarantee(d->should_scavenge(),                   "Illegal car table entry");
  }

  // Verify table for from
  for (n = from_low_index; n < from_high_index; n++) {
    CarTableDesc* d = &(_table_base[n]);
    guarantee(d->train_number() == CarTableDesc::newgen_train_number,  "Illegal car table entry");
    guarantee(d->car_number() == CarTableDesc::invalid_car_number,     "Illegal car table entry");
    guarantee(d->train() == NULL,                     "Illegal car table entry");
    guarantee(d->space() == from,                     "Illegal car table entry");
    guarantee(d->should_scavenge(),                   "Illegal car table entry");
  }

  // Verify table for to
  for (n = to_low_index; n < to_high_index; n++) {
    CarTableDesc* d = &(_table_base[n]);
    guarantee(d->train_number() == CarTableDesc::newgen_train_number,  "Illegal car table entry");
    guarantee(d->car_number() == CarTableDesc::invalid_car_number,     "Illegal car table entry");
    guarantee(d->train() == NULL,                     "Illegal car table entry");
    guarantee(d->space() == to,                       "Illegal car table entry");
    guarantee(!d->should_scavenge(),                  "Illegal car table entry");
  }

  // Verify table for uncommitted part of new generation
  for (n = max(from_high_index, to_high_index); n < newgen_high_index; n++) {
    CarTableDesc* d = &(_table_base[n]);
    guarantee(d->train_number() == CarTableDesc::invalid_train_number, "Illegal car table entry");
    guarantee(d->car_number() == CarTableDesc::invalid_car_number,     "Illegal car table entry");
    guarantee(d->train() == NULL,                     "Illegal car table entry");
    guarantee(d->space() == NULL,                     "Illegal car table entry");
    guarantee(!d->should_scavenge(),                  "Illegal car table entry");
  }

  // Verify table for old gen
  if (UseTrainGC) {
    for (n = oldgen_low_index; n < oldgen_high_index; n++) {
      CarTableDesc* d = &(_table_base[n]);
      if (UseTrainGC) {
        if (d->train() == NULL) {
          guarantee(d->train_number() == CarTableDesc::invalid_train_number, "Illegal car table entry");
          guarantee(d->car_number() == CarTableDesc::invalid_car_number,     "Illegal car table entry");
          guarantee(d->space() == NULL,                                      "Illegal car table entry");
        } else {
          guarantee(d->train_number() >= CarTableDesc::special_nonoop_train_number, "Illegal car table entry");
          guarantee(d->car_number() >= CarTableDesc::initial_car_number,     "Illegal car table entry");
          guarantee(((CarSpace*) d->space())->desc()->equals(d),             "Illegal car table entry");
        }
      } else {
        guarantee(d->train_number() == CarTableDesc::tenuredgen_train_number,  "Illegal car table entry");
        guarantee(d->car_number() == CarTableDesc::invalid_car_number,      "Illegal car table entry");
        guarantee(d->train() == NULL,                                 "Illegal car table entry");
        guarantee(d->space() == Universe::tenured_gen()->the_space(), "Illegal car table entry");
      }
    }
  }

  // Verify table for perm gen
  for (n = permgen_low_index; n < permgen_high_index; n++) {
    CarTableDesc* d = &(_table_base[n]);
    guarantee(d->train_number() == CarTableDesc::permgen_train_number,  "Illegal car table entry");
    guarantee(d->car_number() == CarTableDesc::invalid_car_number,      "Illegal car table entry");
    guarantee(d->train() == NULL,                              "Illegal car table entry");
    guarantee(d->space() == Universe::perm_gen()->the_space(), "Illegal car table entry");
  }
}

#endif
