/*
 * @(#)javaClasses.cpp	1.153 01/02/09 14:45:04
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_javaClasses.cpp.incl"


oop java_lang_String::basic_create(typeArrayOop buffer, bool tenured, TRAPS) {
  typeArrayHandle h_buffer(THREAD, buffer);
  oop obj;
  if (tenured) {
    obj = instanceKlass::cast(SystemDictionary::string_klass())->allocate_permanent_instance(CHECK_0);
  } else {
    obj = instanceKlass::cast(SystemDictionary::string_klass())->allocate_instance(CHECK_0);
  }
  obj->obj_field_put(oopDesc::header_size() + value_offset,  h_buffer());
  obj->int_field_put(oopDesc::header_size() + offset_offset, 0);
  obj->int_field_put(oopDesc::header_size() + count_offset, h_buffer()->length());

  return obj;
}

oop java_lang_String::create_from_unicode(jchar* unicode, int length, TRAPS) {
  typeArrayOop buffer = oopFactory::new_charArray(length, CHECK_0);
  for (int index = 0; index < length; index++) {
    buffer->char_at_put(index, unicode[index]);
  }
  return basic_create(buffer, false, CHECK_0);
}

oop java_lang_String::create_tenured_from_unicode(jchar* unicode, int length, TRAPS) {
  typeArrayOop buffer = oopFactory::new_permanent_charArray(length, CHECK_0);
  for (int index = 0; index < length; index++) {
    buffer->char_at_put(index, unicode[index]);
  }
  return basic_create(buffer, true, CHECK_0);
}

oop java_lang_String::create_from_str(const char* utf8_str, TRAPS) {
  if (utf8_str == NULL) return NULL;
  typeArrayOop buffer = oopFactory::new_charArray(utf8_str, CHECK_0);
  return basic_create(buffer, false, CHECK_0);
}

// Converts a C string to a Java String based on current encoding
oop java_lang_String::create_from_platform_depended_str(const char* str, TRAPS) {
  assert(str != NULL, "bad arguments");

  typedef jstring (*to_java_string_fn_t)(JNIEnv*, const char *);
  static to_java_string_fn_t _to_java_string_fn = NULL;

  if (_to_java_string_fn == NULL) {
    void *lib_handle = os::native_java_library();
    _to_java_string_fn = CAST_TO_FN_PTR(to_java_string_fn_t, hpi::dll_lookup(lib_handle, "NewStringPlatform"));
    if (_to_java_string_fn == NULL) fatal("NewStringPlatform missing");
  }

  jstring js = NULL;
  { JavaThread* thread = (JavaThread*)THREAD;
    assert(thread->is_Java_thread(), "must be java thread");
    ThreadToNativeFromVM ttn(thread);
    HandleMark hm(thread);    
    StateSaver nsv(thread);                  // we must have post Java state set while in native
    js = (_to_java_string_fn)(thread->jni_environment(), str);
  }
  return JNIHandles::resolve(js);
}


oop java_lang_String::create_from_symbol(symbolHandle symbol, TRAPS) {
  int length = UTF8::unicode_length((char*)symbol->bytes(), symbol->utf8_length());
  typeArrayOop value = oopFactory::new_charArray(length, CHECK_0);
  if (length > 0) {
    UTF8::convert_to_unicode((char*)symbol->bytes(), value->char_at_addr(0), length);
  }
  return basic_create(value, false, CHECK_0);
}


oop java_lang_String::char_converter(oop java_string, jchar from_char, jchar to_char, TRAPS) {
  // Converts all '/' to '.' in string.
  typeArrayOop value  = (typeArrayOop)java_string->obj_field(oopDesc::header_size() + value_offset);
  int          offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int          length = java_string->int_field(oopDesc::header_size() + count_offset);

  // First check if any from_char exist
  bool found = false;
  int index = 0; // Declared outside for portability
  for (index = 0; index < length && !found; index++) {
    if (value->char_at(index + offset) == from_char) found = true;;
  }
  // No '/' so do not copy.
  if (!found) return java_string;

  // Create new UNICODE buffer
  typeArrayHandle from_buffer(THREAD, value);
  typeArrayHandle to_buffer  = oopFactory::new_charArray(length, CHECK_0);

  // Copy contents  
  for (index = 0; index < length; index++) {
    jchar c = from_buffer->char_at(index + offset);
    if (c == from_char) c = to_char;
    to_buffer->char_at_put(index, c);
  }  
        
  return basic_create(to_buffer(), false, CHECK_0);  
}


jchar* java_lang_String::as_unicode_string(oop java_string, int& length) {
  assert(is_instance(java_string), "must be java_string");
  oop value  = java_string->obj_field(oopDesc::header_size() + value_offset);
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  length = java_string->int_field(oopDesc::header_size() + count_offset);

  jchar* result = NEW_RESOURCE_ARRAY(jchar, length);
  for (int index = 0; index < length; index++) {
    result[index] = typeArrayOop(value)->char_at(index + offset);
  }
  return result;
}


int java_lang_String::length(oop java_string) {
  assert(is_instance(java_string), "must be java_string");
  return java_string->int_field(oopDesc::header_size() + count_offset);
}


int java_lang_String::offset(oop java_string) {
  assert(is_instance(java_string), "must be java_string");
  return java_string->int_field(oopDesc::header_size() + offset_offset);
}


typeArrayOop java_lang_String::value(oop java_string) {
  assert(is_instance(java_string), "must be java_string");
  return (typeArrayOop) java_string->obj_field(oopDesc::header_size() + value_offset);
}

int java_lang_String::value_offset_in_bytes() {
  return (oopDesc::header_size() + value_offset) * wordSize;
}


int java_lang_String::offset_offset_in_bytes() {
  return (oopDesc::header_size() + offset_offset) * wordSize;
}


int java_lang_String::count_offset_in_bytes() {
  return (oopDesc::header_size() + count_offset) * wordSize;
}


symbolOop java_lang_String::as_symbol(oop java_string, TRAPS) {
  assert(is_instance(java_string), "must be java_string");
  typeArrayOop value = typeArrayOop(java_string->obj_field(oopDesc::header_size() + value_offset));
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int length = java_string->int_field(oopDesc::header_size() + count_offset);

  ResourceMark rm;
  symbolOop result;

  if (length > 0) {
    int utf8_length = UNICODE::utf8_length(value->char_at_addr(offset), length);
    char* chars = NEW_RESOURCE_ARRAY(char, utf8_length + 1);
    UNICODE::convert_to_utf8(value->char_at_addr(offset), length, chars);
    // Allocate the symbol
    result = oopFactory::new_symbol(chars, utf8_length, CHECK_0);  
  } else {
    result = oopFactory::new_symbol("", 0, CHECK_0);  
  }
  return result;
}

int java_lang_String::utf8_length(oop java_string) {
  assert(is_instance(java_string), "must be java_string");
  oop value  = java_string->obj_field(oopDesc::header_size() + value_offset);
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int length = java_string->int_field(oopDesc::header_size() + count_offset);
  jchar* position = (length == 0) ? NULL : typeArrayOop(value)->char_at_addr(offset);
  return UNICODE::utf8_length(position, length);
}

char* java_lang_String::as_utf8_string(oop java_string) {
  assert(is_instance(java_string), "must be java_string");
  oop value  = java_string->obj_field(oopDesc::header_size() + value_offset);
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int length = java_string->int_field(oopDesc::header_size() + count_offset);
  jchar* position = (length == 0) ? NULL : typeArrayOop(value)->char_at_addr(offset);
  return UNICODE::as_utf8(position, length);
}


char* java_lang_String::as_utf8_string(oop java_string, int start, int len) {
  assert(is_instance(java_string), "must be java_string");
  oop value  = java_string->obj_field(oopDesc::header_size() + value_offset);
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int length = java_string->int_field(oopDesc::header_size() + count_offset);
  assert(start+len <= length, "just checking");
  jchar* position = typeArrayOop(value)->char_at_addr(offset+start);
  return UNICODE::as_utf8(position, len);
}

  
bool java_lang_String::is_instance(oop obj) {
  return obj != NULL && obj->klass() == SystemDictionary::string_klass();
}

bool java_lang_String::equals(oop java_string, jchar* chars, int len) {
  assert(java_string->klass() == SystemDictionary::string_klass(), "must be java_string");
  oop value  = java_string->obj_field(oopDesc::header_size() + value_offset);
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int length = java_string->int_field(oopDesc::header_size() + count_offset);
  if (length != len) return false;
  for (int i = 0; i < len; i++) {
    if (typeArrayOop(value)->char_at(i + offset) != chars[i]) {
      return false;
    }
  }
  return true;
}

void java_lang_String::print(oop java_string, outputStream* st) {
  assert(java_string->klass() == SystemDictionary::string_klass(), "must be java_string");
  oop value  = java_string->obj_field(oopDesc::header_size() + value_offset);
  int offset = java_string->int_field(oopDesc::header_size() + offset_offset);
  int length = java_string->int_field(oopDesc::header_size() + count_offset);

  int end = min(length, 100); 
  st->print("\"");
  for (int index = 0; index < length; index++) {
    st->print("%c", typeArrayOop(value)->char_at(index + offset));
  }
  st->print("\"");
}


oop java_lang_Class::create_mirror(KlassHandle k, TRAPS) {
  assert(k->java_mirror() == NULL, "should only assign mirror once");
  if (SystemDictionary::class_klass_loaded()) {
    // Allocate mirror (java.lang.Class instance)
    oop mirror = instanceKlass::cast(SystemDictionary::class_klass())->allocate_permanent_instance(CHECK_0);
    // Setup indirections
    mirror->obj_field_put(oopDesc::header_size() + klass_offset,  k());
    k->set_java_mirror(mirror);
    return mirror;
  } else {
    return NULL;
  }
}


oop java_lang_Class::create_basic_type_mirror(const char* basic_type_name, TRAPS) {
  // This should be improved by introducing a new VM klass
  return instanceKlass::cast(SystemDictionary::class_klass())->allocate_permanent_instance(CHECK_0);
}


klassOop java_lang_Class::as_klassOop(oop java_class) {
  //%note memory_2
  klassOop k = klassOop(java_class->obj_field(oopDesc::header_size() + klass_offset));
  assert(k == NULL || k->is_klass(), "type check");
  return k;
}


methodOop java_lang_Class::resolved_constructor(oop java_class) {
  oop constructor = java_class->obj_field(oopDesc::header_size() + resolved_constructor_offset);
  assert(constructor == NULL || constructor->is_method(), "should be method");
  return methodOop(constructor);
}


void java_lang_Class::set_resolved_constructor(oop java_class, methodOop constructor) {
  assert(constructor->is_method(), "should be method");
  java_class->obj_field_put(oopDesc::header_size() + resolved_constructor_offset, constructor);
}


klassOop java_lang_Class::fastpath_caller_class(oop java_class) {
  oop caller = java_class->obj_field(oopDesc::header_size() + fastpath_caller_class_offset);
  assert(caller == NULL || (caller->is_klass() && Klass::cast(klassOop(caller))->oop_is_instance()), "should be instanceKlass");
  return klassOop(caller);
}


void java_lang_Class::set_fastpath_caller_class(oop java_class, klassOop caller) {
  assert(caller->is_klass() && Klass::cast(caller)->oop_is_instance(), "should be instanceKlass");
  java_class->obj_field_put(oopDesc::header_size() + fastpath_caller_class_offset, caller);
}

int java_lang_Class::klass_offset_in_bytes() {
  return (oopDesc::header_size() + klass_offset) * wordSize;
}

int java_lang_Class::fastpath_caller_class_offset_in_bytes() {
  return (oopDesc::header_size() + fastpath_caller_class_offset) * wordSize;
}

bool java_lang_Class::is_primitive(oop java_class) {
  klassOop k = klassOop(java_class->obj_field(oopDesc::header_size() + klass_offset)); 
  assert(   k != NULL
         || java_class == SystemDictionary::int_mirror()
         || java_class == SystemDictionary::float_mirror()
         || java_class == SystemDictionary::double_mirror()
         || java_class == SystemDictionary::byte_mirror()
         || java_class == SystemDictionary::bool_mirror()
         || java_class == SystemDictionary::char_mirror()
         || java_class == SystemDictionary::short_mirror()
         || java_class == SystemDictionary::void_mirror()
         || java_class == SystemDictionary::long_mirror(), "Consistency check");
  return k == NULL;
}


// Note: JDK1.1 and before had a privateInfo_offset field which was used for the
//       platform thread structure, and a eetop offset which was used for thread
//       local storage (and unused by the HotSpot VM). In JDK1.2 the two structures 
//       merged, so in the HotSpot VM we just use the eetop field for the thread 
//       instead of the privateInfo_offset.

int java_lang_Thread::_name_offset = 0;
int java_lang_Thread::_group_offset = 0;
int java_lang_Thread::_inheritedAccessControlContext_offset = 0;
int java_lang_Thread::_priority_offset = 0;
int java_lang_Thread::_eetop_offset = 0;
int java_lang_Thread::_daemon_offset = 0;
int java_lang_Thread::_stillborn_offset = 0;


void java_lang_Thread::compute_offsets() {
  assert(_group_offset == 0, "offsets should be initialized only once");
  fieldDescriptor fd;
  instanceKlass* ik = instanceKlass::cast(SystemDictionary::thread_klass());

  if (!ik->find_local_field(vmSymbols::name_name(), vmSymbols::char_array_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _name_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::group_name(), vmSymbols::threadgroup_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _group_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::inheritedAccessControlContext_name(), vmSymbols::accesscontrolcontext_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _inheritedAccessControlContext_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::priority_name(), vmSymbols::int_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _priority_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::daemon_name(), vmSymbols::bool_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _daemon_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::eetop_name(), vmSymbols::long_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _eetop_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::stillborn_name(), vmSymbols::bool_signature(), &fd)) {
    fatal("Invalid layout of java.lang.Thread");
  }
  _stillborn_offset = fd.offset();
}


JavaThread* java_lang_Thread::thread(oop java_thread) {
  return (JavaThread*) java_thread->int_field(_eetop_offset);
}


void java_lang_Thread::set_thread(oop java_thread, JavaThread* thread) {
  java_thread->int_field_put(_eetop_offset, (jint) thread);
}


typeArrayOop java_lang_Thread::name(oop java_thread) {
  oop name = java_thread->obj_field(_name_offset);  
  assert(name == NULL || (name->is_typeArray() && typeArrayKlass::cast(name->klass())->type() == T_CHAR), "just checking");
  return typeArrayOop(name);
}


void java_lang_Thread::set_name(oop java_thread, typeArrayOop name) {
  assert(java_thread->obj_field(_name_offset) == NULL, "name should be NULL");
  java_thread->obj_field_put(_name_offset, name);
}


ThreadPriority java_lang_Thread::priority(oop java_thread) {
  return (ThreadPriority)java_thread->int_field(_priority_offset);
}


void java_lang_Thread::set_priority(oop java_thread, ThreadPriority priority) {
  java_thread->int_field_put(_priority_offset, priority);
}


oop java_lang_Thread::threadGroup(oop java_thread) {
  return java_thread->obj_field(_group_offset);
}


bool java_lang_Thread::is_stillborn(oop java_thread) {
  return java_thread->bool_field(_stillborn_offset);
}


// We never have reason to turn the stillborn bit off
void java_lang_Thread::set_stillborn(oop java_thread) {
  java_thread->bool_field_put(_stillborn_offset, true);
}


bool java_lang_Thread::is_alive(oop java_thread) {
  JavaThread* thr = java_lang_Thread::thread(java_thread);
  return (thr != NULL);
}

bool java_lang_Thread::is_daemon(oop java_thread) {
  return java_thread->bool_field(_daemon_offset);
}


void java_lang_Thread::set_daemon(oop java_thread) {
  java_thread->bool_field_put(_daemon_offset, true);
}


oop java_lang_Thread::inherited_access_control_context(oop java_thread) {
  return java_thread->obj_field(_inheritedAccessControlContext_offset);
}


oop  java_lang_ThreadGroup::parent(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return java_thread_group->obj_field(oopDesc::header_size() + parent_offset);
}

// ("name as oop" accessor is not necessary)

typeArrayOop java_lang_ThreadGroup::name(oop java_thread_group) {
  oop name = java_thread_group->obj_field(oopDesc::header_size() + name_offset);
  assert(name != NULL, "threadgroups should have a name");
  return java_lang_String::value(name);
}

int java_lang_ThreadGroup::nthreads(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return java_thread_group->int_field(oopDesc::header_size() + nthreads_offset);
}

objArrayOop java_lang_ThreadGroup::threads(oop java_thread_group) {
  oop threads = java_thread_group->obj_field(oopDesc::header_size() + threads_offset);
  assert(threads != NULL, "threadgroups should have threads");
  assert(threads->is_objArray(), "just checking"); // Todo: Add better type checking code
  return objArrayOop(threads);
}

int java_lang_ThreadGroup::ngroups(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return java_thread_group->int_field(oopDesc::header_size() + ngroups_offset);
}

objArrayOop java_lang_ThreadGroup::groups(oop java_thread_group) {
  oop groups = java_thread_group->obj_field(oopDesc::header_size() + groups_offset);
  assert(groups == NULL || groups->is_objArray(), "just checking"); // Todo: Add better type checking code
  return objArrayOop(groups);
}

ThreadPriority java_lang_ThreadGroup::maxPriority(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return (ThreadPriority) java_thread_group->int_field(oopDesc::header_size() + maxPriority_offset);
}

bool java_lang_ThreadGroup::is_destroyed(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return java_thread_group->bool_field(oopDesc::header_size() + destroyed_offset);
}

bool java_lang_ThreadGroup::is_daemon(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return java_thread_group->bool_field(oopDesc::header_size() + daemon_offset);
}

bool java_lang_ThreadGroup::is_vmAllowSuspension(oop java_thread_group) {
  assert(java_thread_group->is_oop(), "thread group must be oop");
  return java_thread_group->bool_field(oopDesc::header_size() + vmAllowSuspension_offset);
}


oop java_lang_Throwable::backtrace(oop throwable) {
  return throwable->obj_field(oopDesc::header_size() + backtrace_offset);
}


void java_lang_Throwable::set_backtrace(oop throwable, oop value) {
  throwable->obj_field_put(oopDesc::header_size() + backtrace_offset, value);
}


oop java_lang_Throwable::message(oop throwable) {
  return throwable->obj_field(oopDesc::header_size() + detailMessage_offset);
}


oop java_lang_Throwable::message(Handle throwable) {
  return throwable->obj_field(oopDesc::header_size() + detailMessage_offset);
}

void java_lang_Throwable::set_message(oop throwable, oop value) {
  throwable->obj_field_put(oopDesc::header_size() + detailMessage_offset, value);
}


void java_lang_Throwable::print(oop throwable, outputStream* st) {
  ResourceMark rm;
  klassOop k = throwable->klass();
  assert(k != NULL, "just checking");
  st->print("%s", instanceKlass::cast(k)->external_name());
  oop msg = message(throwable);
  if (msg != NULL) {
    st->print(": %s", java_lang_String::as_utf8_string(msg));
  }
}


void java_lang_Throwable::print(Handle throwable, outputStream* st) {
  ResourceMark rm;
  klassOop k = throwable->klass();
  assert(k != NULL, "just checking");
  st->print("%s", instanceKlass::cast(k)->external_name());
  oop msg = message(throwable);
  if (msg != NULL) {
    st->print(": %s", java_lang_String::as_utf8_string(msg));
  }
}

void java_lang_Throwable::print_stack_element(Handle stream, methodOop method, int bci) {  
  ResourceMark rm;
  // Get strings and string lengths
  instanceKlass* klass = instanceKlass::cast(method->method_holder());
  const char* klass_name  = klass->external_name();
  int buf_len = strlen(klass_name);
  char* source_file_name;
  if (klass->source_file_name() == NULL) {
    source_file_name = NULL;
  } else {
    source_file_name = klass->source_file_name()->as_C_string();
    buf_len += strlen(source_file_name);
  }
  char* method_name = method->name()->as_C_string();
  buf_len += strlen(method_name);

  // Allocate temporary buffer with extra space for formatting and line number
  char* buf = NEW_RESOURCE_ARRAY(char, buf_len + 64);

  // Print stack trace line in buffer
  sprintf(buf, "\tat %s.%s", klass_name, method_name);
  if (method->is_native()) {
    strcat(buf, "(Native Method)");
  } else {    
    int line_number = method->line_number_from_bci(bci);
    if (source_file_name != NULL && (line_number != -1)) {
      // Sourcename and linenumber
      sprintf(buf + strlen(buf), "(%s:%d)", source_file_name, line_number);
    } else if (source_file_name != NULL) {
      // Just sourcename
      sprintf(buf + strlen(buf), "(%s)", source_file_name);      
    } else {
      // Neither soucename and linenumber
      sprintf(buf + strlen(buf), "(Unknown Source)");
    }
    if (WizardMode && method->code() != NULL) {
      sprintf(buf + strlen(buf), "(nmethod %#x)", method->code());
    }
  }

  // Print buffer on stream
  print_to_stream(stream, buf);
}


void java_lang_Throwable::print_stack_usage(Handle stream) {  
  print_to_stream(stream, "\t<<no stack trace available>>");
}


void java_lang_Throwable::print_to_stream(Handle stream, const char* str) {
  if (stream == NULL) {
    tty->print_cr("%s", str);
  } else {
    EXCEPTION_MARK;
    JavaValue result(T_VOID);
    Handle arg = oopFactory::new_charArray(str, THREAD);
    if (!HAS_PENDING_EXCEPTION) {
      JavaCalls::call_virtual(&result, 
                              stream, 
                              stream->klass(),
                              vmSymbolHandles::println_name(), 
                              vmSymbolHandles::char_array_void_signature(), 
                              arg, 
                              THREAD);
    }
    // Ignore any exceptions. we are in the middle of exception handling. Same as classic VM.
    if (HAS_PENDING_EXCEPTION) CLEAR_PENDING_EXCEPTION;
  }

}


void java_lang_Throwable::print_stack_trace(oop throwable, oop print_stream) {
  Handle stream(print_stream);
  objArrayHandle result = objArrayOop(backtrace(throwable));
  if (result.is_null()) {
    print_stack_usage(stream);
    return;
  }
  
  while (result.not_null()) {
    objArrayHandle methods = objArrayOop(result->obj_at(0));
    typeArrayHandle bcis = typeArrayOop(result->obj_at(1));

    if (methods.is_null() || bcis.is_null()) {
      print_stack_usage(stream);
      return;
    }

    int length = methods()->length();
    for (int index = 0; index < length; index++) {
      methodOop method = methodOop(methods()->obj_at(index));
      if (method == NULL) return;
      int bci = bcis->ushort_at(index);
      print_stack_element(stream, method, bci);
    }
    result = objArrayOop(result->obj_at(2));
  }
}


void java_lang_Throwable::fill_in_stack_trace(Handle throwable, TRAPS) {
  if (!StackTraceInThrowable) return;
  ResourceMark rm;
  int total_count = 0;
  int chunk_count = 0;
  const int chunk_size = 35;
  objArrayHandle  head    = oopFactory::new_objArray(SystemDictionary::object_klass(), 3,          CHECK);
  objArrayHandle  methods = oopFactory::new_objArray(SystemDictionary::object_klass(), chunk_size, CHECK);
  typeArrayHandle bcis    = oopFactory::new_shortArray(chunk_size, CHECK);
  head->obj_at_put(0, methods());
  head->obj_at_put(1, bcis());

  set_backtrace(throwable(), head());

  vframeStream st((JavaThread*) THREAD); 

  // skip fillInStackTrace
  if (!st.at_end() && (st.method()->name() == vmSymbols::fillInStackTrace_name())) {
    st.next();
  }

  // skip <init> methods of the exceptions klass. If there is <init> methods that
  // belongs to a superclass of the exception  we are going to skipping them in the
  // stack trace. This is simlar to classic VM.
  
  while (!st.at_end() && 
          (st.method()->name() == vmSymbols::object_initializer_name() &&  
           throwable->is_a(st.method()->method_holder())
          )
        ) {    
    st.next();
  }

  // collect the rest
  for (;!st.at_end(); st.next()) {    
    methodOop m = st.method(); // The method is not stored GC safe
    int bci     = st.bci();

    if (chunk_count >= chunk_size) {
      // Need to preserve m across GC points
      methodHandle mh(THREAD, m);
      // Create a new chunk
      objArrayHandle  new_head = oopFactory::new_objArray(SystemDictionary::object_klass(), 3, CHECK);
      head->obj_at_put(2, new_head());
      methods = oopFactory::new_objArray(SystemDictionary::object_klass(), chunk_size, CHECK);
      bcis    = oopFactory::new_shortArray(chunk_size, CHECK);
      new_head->obj_at_put(0, methods());
      new_head->obj_at_put(1, bcis());
      chunk_count = 0;
      head = new_head;
      m = mh(); // Update m after GC points
    }

    // add element
    bcis->ushort_at_put(chunk_count, bci);
    methods->obj_at_put(chunk_count, m);

    chunk_count++;
    total_count++;

    // Bail-out case for too deep stacks
    if (MaxJavaStackTraceDepth == total_count) return;
  }
}


void java_lang_Throwable::fill_in_stack_trace(Handle throwable) {
  // No-op if stack trace is disabled
  if (!StackTraceInThrowable) return;

  // Disable stack traces for OutOfMemoryErrors
  if (throwable == Universe::out_of_memory_error_instance()) return;

  PRESERVE_EXCEPTION_MARK;

  JavaThread* thread = JavaThread::active();
  fill_in_stack_trace(throwable, thread);
  // ignore exceptions thrown during stack trace filling
  CLEAR_PENDING_EXCEPTION;  
}

jboolean java_lang_reflect_AccessibleObject::override(oop reflect) {
  return (jboolean) reflect->int_field(oopDesc::header_size() + override_offset);
}

void java_lang_reflect_AccessibleObject::set_override(oop reflect, jboolean value) {
  reflect->int_field_put(oopDesc::header_size() + override_offset, (int) value);
}

oop java_lang_reflect_Method::create(TRAPS) {  
  klassOop klass = SystemDictionary::reflect_method_klass();
  // This class is eagerly initialized during VM initialization, since we keep a refence
  // to one of the methods
  assert(instanceKlass::cast(klass)->is_initialized(), "must be initialized");  
  return instanceKlass::cast(klass)->allocate_instance(CHECK_0);
}

oop java_lang_reflect_Method::clazz(oop reflect) {
  return reflect->obj_field(oopDesc::header_size() + clazz_offset);
}

void java_lang_reflect_Method::set_clazz(oop reflect, oop value) {
   reflect->obj_field_put(oopDesc::header_size() + clazz_offset, value);
}

int java_lang_reflect_Method::slot(oop reflect) {
  return reflect->int_field(oopDesc::header_size() + slot_offset);
}

void java_lang_reflect_Method::set_slot(oop reflect, int value) {
  reflect->int_field_put(oopDesc::header_size() + slot_offset, value);
}

oop java_lang_reflect_Method::name(oop method) {
  return method->obj_field(oopDesc::header_size() + name_offset);
}

void java_lang_reflect_Method::set_name(oop method, oop value) {
  method->obj_field_put(oopDesc::header_size() + name_offset, value);
}

oop java_lang_reflect_Method::return_type(oop method) {
  return method->obj_field(oopDesc::header_size() + returnType_offset);
}

void java_lang_reflect_Method::set_return_type(oop method, oop value) {
  method->obj_field_put(oopDesc::header_size() + returnType_offset, value);
}

oop java_lang_reflect_Method::parameter_types(oop method) {
  return method->obj_field(oopDesc::header_size() + parameterTypes_offset);
}

void java_lang_reflect_Method::set_parameter_types(oop method, oop value) {
  method->obj_field_put(oopDesc::header_size() + parameterTypes_offset, value);
}

oop java_lang_reflect_Method::exception_types(oop method) {
  return method->obj_field(oopDesc::header_size() + exceptionTypes_offset);
}

void java_lang_reflect_Method::set_exception_types(oop method, oop value) {
  method->obj_field_put(oopDesc::header_size() + exceptionTypes_offset, value);
}

int java_lang_reflect_Method::modifiers(oop method) {
  return method->int_field(oopDesc::header_size() + modifiers_offset);
}

void java_lang_reflect_Method::set_modifiers(oop method, int value) {
  method->int_field_put(oopDesc::header_size() + modifiers_offset, value);
}

oop java_lang_reflect_Constructor::create(TRAPS) {
  symbolHandle name = vmSymbolHandles::java_lang_reflect_Constructor();
  instanceKlassHandle klass  = SystemDictionary::resolve_or_fail(name, NULL, NULL, true, CHECK_0);
  // Ensure it is initialized
  klass->initialize(CHECK_0);
  return klass->allocate_instance(CHECK_0);
}

oop java_lang_reflect_Constructor::clazz(oop reflect) {
  return reflect->obj_field(oopDesc::header_size() + clazz_offset);
}

void java_lang_reflect_Constructor::set_clazz(oop reflect, oop value) {
   reflect->obj_field_put(oopDesc::header_size() + clazz_offset, value);
}

oop java_lang_reflect_Constructor::parameter_types(oop constructor) {
  return constructor->obj_field(oopDesc::header_size() + parameterTypes_offset);
}

void java_lang_reflect_Constructor::set_parameter_types(oop constructor, oop value) {
  constructor->obj_field_put(oopDesc::header_size() + parameterTypes_offset, value);
}

oop java_lang_reflect_Constructor::exception_types(oop constructor) {
  return constructor->obj_field(oopDesc::header_size() + exceptionTypes_offset);
}

void java_lang_reflect_Constructor::set_exception_types(oop constructor, oop value) {
  constructor->obj_field_put(oopDesc::header_size() + exceptionTypes_offset, value);
}

int java_lang_reflect_Constructor::slot(oop reflect) {
  return reflect->int_field(oopDesc::header_size() + slot_offset);
}

void java_lang_reflect_Constructor::set_slot(oop reflect, int value) {
  reflect->int_field_put(oopDesc::header_size() + slot_offset, value);
}

int java_lang_reflect_Constructor::modifiers(oop constructor) {
  return constructor->int_field(oopDesc::header_size() + modifiers_offset);
}

void java_lang_reflect_Constructor::set_modifiers(oop constructor, int value) {
  constructor->int_field_put(oopDesc::header_size() + modifiers_offset, value);
}

oop java_lang_reflect_Field::create(TRAPS) {  
  symbolHandle name = vmSymbolHandles::java_lang_reflect_Field();
  instanceKlassHandle klass = SystemDictionary::resolve_or_fail(name, NULL, NULL, true, CHECK_0);
  // Ensure it is initialized
  klass->initialize(CHECK_0);
  return klass->allocate_instance(CHECK_0);
}

oop java_lang_reflect_Field::clazz(oop reflect) {
  return reflect->obj_field(oopDesc::header_size() + clazz_offset);
}

void java_lang_reflect_Field::set_clazz(oop reflect, oop value) {
   reflect->obj_field_put(oopDesc::header_size() + clazz_offset, value);
}

oop java_lang_reflect_Field::name(oop field) {
  return field->obj_field(oopDesc::header_size() + name_offset);
}

void java_lang_reflect_Field::set_name(oop field, oop value) {
  field->obj_field_put(oopDesc::header_size() + name_offset, value);
}

oop java_lang_reflect_Field::type(oop field) {
  return field->obj_field(oopDesc::header_size() + type_offset);
}

void java_lang_reflect_Field::set_type(oop field, oop value) {
  field->obj_field_put(oopDesc::header_size() + type_offset, value);
}

int java_lang_reflect_Field::slot(oop reflect) {
  return reflect->int_field(oopDesc::header_size() + slot_offset);
}

void java_lang_reflect_Field::set_slot(oop reflect, int value) {
  reflect->int_field_put(oopDesc::header_size() + slot_offset, value);
}

int java_lang_reflect_Field::modifiers(oop field) {
  return field->int_field(oopDesc::header_size() + modifiers_offset);
}

void java_lang_reflect_Field::set_modifiers(oop field, int value) {
  field->int_field_put(oopDesc::header_size() + modifiers_offset, value);
}


oop java_lang_boxing_object::initialize_and_allocate(klassOop k, TRAPS) {
 instanceKlassHandle h = k;
 if (!h->is_initialized()) h->initialize(CHECK_0);
 oop instance = h->allocate_instance(THREAD);
 return instance;
}

oop java_lang_boxing_object::create(BasicType type, jvalue* value, TRAPS) {
  oop box;
  switch (type) {
    case T_BOOLEAN:
      box = initialize_and_allocate(SystemDictionary::boolean_klass(), CHECK_0);
      box->bool_field_put(oopDesc::header_size() + value_offset, value->z);
      break;
    case T_CHAR:
      box = initialize_and_allocate(SystemDictionary::char_klass(), CHECK_0);
      box->char_field_put(oopDesc::header_size() + value_offset, value->c);
      break;
    case T_FLOAT:
      box = initialize_and_allocate(SystemDictionary::float_klass(), CHECK_0);
      box->float_field_put(oopDesc::header_size() + value_offset, value->f);
      break;
    case T_DOUBLE:
      box = initialize_and_allocate(SystemDictionary::double_klass(), CHECK_0);
      box->double_field_put(oopDesc::header_size() + value_offset, value->d);
      break;
    case T_BYTE:
      box = initialize_and_allocate(SystemDictionary::byte_klass(), CHECK_0);
      box->byte_field_put(oopDesc::header_size() + value_offset, value->b);
      break;
    case T_SHORT:
      box = initialize_and_allocate(SystemDictionary::short_klass(), CHECK_0);
      box->short_field_put(oopDesc::header_size() + value_offset, value->s);
      break;
    case T_INT:
      box = initialize_and_allocate(SystemDictionary::int_klass(), CHECK_0);
      box->int_field_put(oopDesc::header_size() + value_offset, value->i);
      break;
    case T_LONG:
      box = initialize_and_allocate(SystemDictionary::long_klass(), CHECK_0);
      box->long_field_put(oopDesc::header_size() + value_offset, value->j);
      break;
    default:
      return NULL;
  }
  return box;
}


BasicType java_lang_boxing_object::get_value(oop box, jvalue* value) {
  klassOop k = box->klass();
  if (k == SystemDictionary::boolean_klass()) {
    value->z = box->bool_field(oopDesc::header_size() + value_offset);
    return T_BOOLEAN;
  }
  if (k == SystemDictionary::char_klass()) {
    value->c = box->char_field(oopDesc::header_size() + value_offset);
    return T_CHAR;
  }
  if (k == SystemDictionary::float_klass()) {
    value->f = box->float_field(oopDesc::header_size() + value_offset);
    return T_FLOAT;
  }
  if (k == SystemDictionary::double_klass()) {
    value->d = box->double_field(oopDesc::header_size() + value_offset);
    return T_DOUBLE;
  }
  if (k == SystemDictionary::byte_klass()) {
    value->b = box->byte_field(oopDesc::header_size() + value_offset);
    return T_BYTE;
  }
  if (k == SystemDictionary::short_klass()) {
    value->s = box->short_field(oopDesc::header_size() + value_offset);
    return T_SHORT;
  }
  if (k == SystemDictionary::int_klass()) {
    value->i = box->int_field(oopDesc::header_size() + value_offset);
    return T_INT;
  }
  if (k == SystemDictionary::long_klass()) {
    value->j = box->long_field(oopDesc::header_size() + value_offset);
    return T_LONG;
  }
  return T_ILLEGAL;
}


BasicType java_lang_boxing_object::set_value(oop box, jvalue* value) {
  klassOop k = box->klass();
  if (k == SystemDictionary::boolean_klass()) {
    box->bool_field_put(oopDesc::header_size() + value_offset, value->z);
    return T_BOOLEAN;
  }
  if (k == SystemDictionary::char_klass()) {
    box->char_field_put(oopDesc::header_size() + value_offset, value->c);
    return T_CHAR;
  }
  if (k == SystemDictionary::float_klass()) {
    box->float_field_put(oopDesc::header_size() + value_offset, value->f);
    return T_FLOAT;
  }
  if (k == SystemDictionary::double_klass()) {
    box->double_field_put(oopDesc::header_size() + value_offset, value->d);
    return T_DOUBLE;
  }
  if (k == SystemDictionary::byte_klass()) {
    box->byte_field_put(oopDesc::header_size() + value_offset, value->b);
    return T_BYTE;
  }
  if (k == SystemDictionary::short_klass()) {
    box->short_field_put(oopDesc::header_size() + value_offset, value->s);
    return T_SHORT;
  }
  if (k == SystemDictionary::int_klass()) {
    box->int_field_put(oopDesc::header_size() + value_offset, value->i);
    return T_INT;
  }
  if (k == SystemDictionary::long_klass()) {
    box->long_field_put(oopDesc::header_size() + value_offset, value->j);
    return T_LONG;
  }
  return T_ILLEGAL;
}


// Support for java_lang_ref_Reference

void java_lang_ref_Reference::set_referent(oop ref, oop value) {
  ref->obj_field_put(oopDesc::header_size() + referent_offset, value);
}

oop* java_lang_ref_Reference::referent_addr(oop ref) {
  return ref->obj_field_addr(oopDesc::header_size() + referent_offset);
}

void java_lang_ref_Reference::set_next(oop ref, oop value) {
  ref->obj_field_put(oopDesc::header_size() + next_offset, value);
}

oop* java_lang_ref_Reference::next_addr(oop ref) {
  return ref->obj_field_addr(oopDesc::header_size() + next_offset);
}

oop* java_lang_ref_Reference::pending_list_lock_addr() {
  instanceKlass* ik = instanceKlass::cast(SystemDictionary::reference_klass());
  return ik->start_of_static_fields() + static_lock_offset;
}

oop* java_lang_ref_Reference::pending_list_addr() {
  instanceKlass* ik = instanceKlass::cast(SystemDictionary::reference_klass());
  return ik->start_of_static_fields() + static_pending_offset;
}


// Support for java_lang_ref_SoftReference

jlong java_lang_ref_SoftReference::timestamp(oop ref) {
  return ref->long_field(oopDesc::header_size() + timestamp_offset);
}

jlong java_lang_ref_SoftReference::clock() {
  instanceKlass* ik = instanceKlass::cast(SystemDictionary::soft_reference_klass());
  int offset = ik->offset_of_static_fields() + static_clock_offset;

  return SystemDictionary::soft_reference_klass()->long_field(offset);
}

void java_lang_ref_SoftReference::set_clock(jlong value) {
  instanceKlass* ik = instanceKlass::cast(SystemDictionary::soft_reference_klass());
  int offset = ik->offset_of_static_fields() + static_clock_offset;

  SystemDictionary::soft_reference_klass()->long_field_put(offset, value);
}
  
// Support for java_io_FileDescriptor

long java_io_FileDescriptor::fd(oop java_filedescriptor) {
  assert(java_filedescriptor->is_oop(), "illegal java_filedescriptor");
  return java_filedescriptor->int_field(oopDesc::header_size() + fd_offset);
}

void java_io_FileDescriptor::set_fd(oop java_filedescriptor, long fd) {
  assert(java_filedescriptor->is_oop(), "illegal java_filedescriptor");
  java_filedescriptor->int_field_put(oopDesc::header_size() + fd_offset, fd);
} 


// Support for java_security_AccessControlContext

int java_security_AccessControlContext::_context_offset = 0;
int java_security_AccessControlContext::_privilegedContext_offset = 0;
int java_security_AccessControlContext::_isPrivileged_offset = 0;


void java_security_AccessControlContext::compute_offsets() {
  assert(_isPrivileged_offset == 0, "offsets should be initialized only once");
  fieldDescriptor fd;
  instanceKlass* ik = instanceKlass::cast(SystemDictionary::AccessControlContext_klass());

  if (!ik->find_local_field(vmSymbols::context_name(), vmSymbols::protectiondomain_signature(), &fd)) {
    fatal("Invalid layout of java.security.AccessControlContext");
  }
  _context_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::privilegedContext_name(), vmSymbols::accesscontrolcontext_signature(), &fd)) {
    fatal("Invalid layout of java.security.AccessControlContext");
  }
  _privilegedContext_offset = fd.offset();

  if (!ik->find_local_field(vmSymbols::isPrivileged_name(), vmSymbols::bool_signature(), &fd)) {
    fatal("Invalid layout of java.security.AccessControlContext");
  }
  _isPrivileged_offset = fd.offset();
}


oop java_security_AccessControlContext::create(objArrayHandle context, bool isPrivileged, Handle privileged_context, TRAPS) {  
  assert(_isPrivileged_offset != 0, "offsets should have been initialized");
  // Ensure klass is initialized
  instanceKlass::cast(SystemDictionary::AccessControlContext_klass())->initialize(CHECK_0);
  // Allocate result
  oop result = instanceKlass::cast(SystemDictionary::AccessControlContext_klass())->allocate_instance(CHECK_0);
  // Fill in values
  result->obj_field_put(_context_offset, context());
  result->obj_field_put(_privilegedContext_offset, privileged_context());
  result->bool_field_put(_isPrivileged_offset, isPrivileged);
  return result;
}


// Support for java_lang_ClassLoader

oop java_lang_ClassLoader::parent(oop loader) {
  assert(loader->is_oop(), "loader must be oop");
  return loader->obj_field(oopDesc::header_size() + parent_offset);
}


bool java_lang_ClassLoader::is_trusted_loader(oop loader) {
  oop cl = SystemDictionary::java_system_loader();
  while(cl != NULL) {
    if (cl == loader) return true;
    cl = parent(cl);
  }
  return false;
}


// Compute non-hard-coded field offsets of all the classes in this file

void JavaClasses::compute_offsets() {
  java_lang_Thread::compute_offsets();
  java_security_AccessControlContext::compute_offsets();
}


#ifndef PRODUCT

// These functions exist to assert the validity of hard-coded field offsets to guard 
// against changes in the class files

bool JavaClasses::check_offset(const char *klass_name, int hardcoded_offset, const char *field_name, const char* field_sig) {
  EXCEPTION_MARK;
  fieldDescriptor fd;
  symbolHandle klass_sym = oopFactory::new_symbol(klass_name, CATCH);
  klassOop klass = SystemDictionary::resolve_or_fail(klass_sym, NULL, NULL, true, CATCH);
  instanceKlassHandle h_klass(klass);
  symbolOop f_name = oopFactory::new_symbol(field_name, CATCH);
  symbolOop f_sig  = oopFactory::new_symbol(field_sig, CATCH);
  if (!h_klass->find_local_field(f_name, f_sig, &fd)) {
    tty->print_cr("Nonstatic field %s.%s not found", klass_name, field_name);
    return false;
  }
  if (fd.is_static()) {
    tty->print_cr("Nonstatic field %s.%s appears to be static", klass_name, field_name);
    return false;
  }
  if (fd.offset() == hardcoded_offset + instanceOopDesc::header_size()) {
    return true;
  } else {
    tty->print_cr("Offset of nonstatic field %s.%s is hardcoded as %d but should really be %d.", klass_name, field_name, hardcoded_offset, fd.offset() - instanceOopDesc::header_size());
    return false;
  }
}


bool JavaClasses::check_static_offset(const char *klass_name, int hardcoded_offset, const char *field_name, const char* field_sig) {
  EXCEPTION_MARK;
  fieldDescriptor fd;
  symbolHandle klass_sym = oopFactory::new_symbol(klass_name, CATCH);
  klassOop klass = SystemDictionary::resolve_or_fail(klass_sym, NULL, NULL, true, CATCH);
  instanceKlassHandle h_klass(klass);
  symbolOop f_name = oopFactory::new_symbol(field_name, CATCH);
  symbolOop f_sig  = oopFactory::new_symbol(field_sig, CATCH);
  if (!h_klass->find_local_field(f_name, f_sig, &fd)) {
    tty->print_cr("Static field %s.%s not found", klass_name, field_name);
    return false;
  }
  if (!fd.is_static()) {
    tty->print_cr("Static field %s.%s appears to be nonstatic", klass_name, field_name);
    return false;
  }
  if (fd.offset() == hardcoded_offset + h_klass->offset_of_static_fields()) {
    return true;
  } else {
    tty->print_cr("Offset of static field %s.%s is hardcoded as %d but should really be %d.", klass_name, field_name, hardcoded_offset, fd.offset() - h_klass->offset_of_static_fields());
    return false;
  }
}


// Check the hard-coded field offsets of all the classes in this file

void JavaClasses::check_offsets() {
  bool valid = true;

#define CHECK_OFFSET(klass_name, cpp_klass_name, field_name, field_sig) \
  valid &= check_offset(klass_name, cpp_klass_name :: field_name ## _offset, #field_name, field_sig)

#define CHECK_STATIC_OFFSET(klass_name, cpp_klass_name, field_name, field_sig) \
  valid &= check_static_offset(klass_name, cpp_klass_name :: static_ ## field_name ## _offset, #field_name, field_sig)

  // java.lang.String

  CHECK_OFFSET("java/lang/String", java_lang_String, value, "[C");
  CHECK_OFFSET("java/lang/String", java_lang_String, offset, "I");
  CHECK_OFFSET("java/lang/String", java_lang_String, count, "I");
  
  // java.lang.ThreadGroup

  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, parent,  "Ljava/lang/ThreadGroup;");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, name,    "Ljava/lang/String;");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, threads, "[Ljava/lang/Thread;");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, groups,  "[Ljava/lang/ThreadGroup;");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, maxPriority,       "I");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, destroyed,         "Z");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, daemon,            "Z");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, vmAllowSuspension, "Z");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, nthreads,          "I");
  CHECK_OFFSET("java/lang/ThreadGroup", java_lang_ThreadGroup, ngroups,           "I");

  // java.lang.Class

  // CHECK_OFFSET("java/lang/Class", klass); // %%% this needs to be checked
  // CHECK_OFFSET("java/lang/Class", resolved_constructor); // %%% this needs to be checked

  // java.lang.Throwable

  CHECK_OFFSET("java/lang/Throwable", java_lang_Throwable, backtrace, "Ljava/lang/Object;");
  CHECK_OFFSET("java/lang/Throwable", java_lang_Throwable, detailMessage, "Ljava/lang/String;");

  // Boxed primitive objects (java_lang_boxing_object)

  CHECK_OFFSET("java/lang/Boolean",   java_lang_boxing_object, value, "Z");
  CHECK_OFFSET("java/lang/Character", java_lang_boxing_object, value, "C");
  CHECK_OFFSET("java/lang/Float",     java_lang_boxing_object, value, "F");
  CHECK_OFFSET("java/lang/Double",    java_lang_boxing_object, value, "D");
  CHECK_OFFSET("java/lang/Byte",      java_lang_boxing_object, value, "B");
  CHECK_OFFSET("java/lang/Short",     java_lang_boxing_object, value, "S");
  CHECK_OFFSET("java/lang/Integer",   java_lang_boxing_object, value, "I");
  CHECK_OFFSET("java/lang/Long",      java_lang_boxing_object, value, "J");

  // java.io.FileDescriptor

  CHECK_OFFSET("java/io/FileDescriptor", java_io_FileDescriptor, fd, "I");

  // java.lang.ClassLoader

  CHECK_OFFSET("java/lang/ClassLoader", java_lang_ClassLoader, parent,      "Ljava/lang/ClassLoader;");

  // java.lang.reflect.AccessibleObject

  CHECK_OFFSET("java/lang/reflect/AccessibleObject", java_lang_reflect_AccessibleObject, override, "Z");

  // java.lang.reflect.Method

  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, clazz, "Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, name, "Ljava/lang/String;");
  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, returnType, "Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, parameterTypes, "[Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, exceptionTypes, "[Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, slot, "I");
  CHECK_OFFSET("java/lang/reflect/Method", java_lang_reflect_Method, modifiers, "I");

  // java.lang.reflect.Constructor

  CHECK_OFFSET("java/lang/reflect/Constructor", java_lang_reflect_Constructor, clazz, "Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Constructor", java_lang_reflect_Constructor, parameterTypes, "[Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Constructor", java_lang_reflect_Constructor, exceptionTypes, "[Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Constructor", java_lang_reflect_Constructor, slot, "I");
  CHECK_OFFSET("java/lang/reflect/Constructor", java_lang_reflect_Constructor, modifiers, "I");

  // java.lang.reflect.Field

  CHECK_OFFSET("java/lang/reflect/Field", java_lang_reflect_Field, clazz, "Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Field", java_lang_reflect_Field, name, "Ljava/lang/String;");
  CHECK_OFFSET("java/lang/reflect/Field", java_lang_reflect_Field, type, "Ljava/lang/Class;");
  CHECK_OFFSET("java/lang/reflect/Field", java_lang_reflect_Field, slot, "I");
  CHECK_OFFSET("java/lang/reflect/Field", java_lang_reflect_Field, modifiers, "I");

  // java.lang.ref.Reference

  CHECK_OFFSET("java/lang/ref/Reference", java_lang_ref_Reference, referent, "Ljava/lang/Object;");
  CHECK_OFFSET("java/lang/ref/Reference", java_lang_ref_Reference, queue, "Ljava/lang/ref/ReferenceQueue;");
  CHECK_OFFSET("java/lang/ref/Reference", java_lang_ref_Reference, next, "Ljava/lang/ref/Reference;");
  CHECK_STATIC_OFFSET("java/lang/ref/Reference", java_lang_ref_Reference, lock, "Ljava/lang/ref/Reference$Lock;");
  CHECK_STATIC_OFFSET("java/lang/ref/Reference", java_lang_ref_Reference, pending, "Ljava/lang/ref/Reference;");

  // java.lang.ref.SoftReference

  CHECK_OFFSET("java/lang/ref/SoftReference", java_lang_ref_SoftReference, timestamp, "J");
  CHECK_STATIC_OFFSET("java/lang/ref/SoftReference", java_lang_ref_SoftReference, clock, "J");

  if (!valid) vm_exit_during_initialization("Hard-coded field offset verification failed");
}

#endif // PRODUCT


void javaClasses_init() {
  JavaClasses::compute_offsets();
  JavaClasses::check_offsets();
}
