/*
 * @(#)javaClasses.hpp	1.91 01/02/09 14:45:05
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Interface for manipulating the basic Java classes.
//
// All dependencies on layout of actual Java classes should be kept here.
// If the layout of any of the classes above changes the offsets must be adjusted.
//
// For most classes we hardwire the offsets for performance reasons. In certain
// cases (e.g. java.security.AccessControlContext) we compute the offsets at
// startup since the layout here differs between JDK1.2 and JDK1.3.
// 
// Note that fields (static and non-static) are arranged with oops before non-oops
// on a per class basis. The offsets below have to reflect this ordering.
//
// When editing the layouts please update the check_offset verification code 
// correspondingly. The names in the enums must be identical to the actual field 
// names in order for the verification code to work.


// Interface to java.lang.String objects

class java_lang_String : AllStatic {
 private:
  enum {
    value_offset  = 0,
    offset_offset = 1,
    count_offset  = 2
  };
  static oop basic_create(typeArrayOop buffer, bool tenured, TRAPS);
  static oop char_converter(oop java_string, jchar from_char, jchar to_char, TRAPS);
 public:
  // Instance creation
  static oop create_from_unicode(jchar* unicode, int len, TRAPS);
  static oop create_tenured_from_unicode(jchar* unicode, int len, TRAPS);
  static oop create_from_str(const char* utf8_str, TRAPS);
  static oop create_from_platform_depended_str(const char* str, TRAPS);
  static oop create_from_symbol(symbolHandle symbol, TRAPS);  
  static int length(oop java_string);
  static int offset(oop java_string);
 
  static int value_offset_in_bytes();
  static int count_offset_in_bytes();
  static int offset_offset_in_bytes();

  static typeArrayOop value(oop java_string);
  static int utf8_length(oop java_string);
  static char* as_utf8_string(oop java_string);
  static char* as_utf8_string(oop java_string, int start, int len);
  static jchar* as_unicode_string(oop java_string, int& length);

  static bool equals(oop java_string, jchar* chars, int len);

  // Conversion between '.' and '/' formats
  static oop externalize_classname(oop java_string, TRAPS) { return char_converter(java_string, '/', '.', THREAD); }
  static oop internalize_classname(oop java_string, TRAPS) { return char_converter(java_string, '.', '/', THREAD); }    

  // Conversion
  static symbolOop as_symbol(oop java_string, TRAPS);

  // Testers
  static bool is_instance(oop obj);
  // Debugging
  static void print(oop java_string, outputStream* st);
  friend class JavaClasses;
};


// Interface to java.lang.Class objects

class java_lang_Class : AllStatic {
 private:
  // The fake offsets are added by the class loader when java.lang.Class is loaded 
  enum {
   klass_offset  = 0,
   resolved_constructor_offset  = 1,
   fastpath_caller_class_offset = 2,
   number_of_fake_oop_fields = 3
  };

 public:
  // Instance creation
  static oop  create_mirror(KlassHandle k, TRAPS);
  static oop  create_basic_type_mirror(const char* basic_type_name, TRAPS);
  // Conversion
  static klassOop as_klassOop(oop java_class);
  // Testing
  static bool is_primitive(oop java_class);  
  // JVM_NewInstance support
  static methodOop resolved_constructor(oop java_class);
  static void set_resolved_constructor(oop java_class, methodOop constructor);
  static klassOop fastpath_caller_class(oop java_class);
  static void set_fastpath_caller_class(oop java_class, klassOop caller);
  // compiler support for JVM_NewInstance
  static int klass_offset_in_bytes();
  static int fastpath_caller_class_offset_in_bytes();
  // Debugging
  friend class JavaClasses;
  friend class instanceKlass;   // verification code accesses offsets
  friend class ClassFileParser; // access to number_of_fake_fields
};


// Interface to java.lang.Thread objects

class java_lang_Thread : AllStatic {
 private:
  // Note that for this class the layout changed between JDK1.2 and JDK1.3,
  // so we compute the offsets at startup rather than hard-wiring them.
  static int _name_offset;
  static int _group_offset;
  static int _inheritedAccessControlContext_offset;
  static int _priority_offset;
  static int _eetop_offset;
  static int _daemon_offset;
  static int _stillborn_offset;

  static void compute_offsets();

 public:
  // Instance creation
  static oop create();
  // Returns the JavaThread associated with the thread obj
  static JavaThread* thread(oop java_thread);
  // Set JavaThread for instance
  static void set_thread(oop java_thread, JavaThread* thread);
  // Name
  static typeArrayOop name(oop java_thread);
  static void set_name(oop java_thread, typeArrayOop name);
  // Priority
  static ThreadPriority priority(oop java_thread);
  static void set_priority(oop java_thread, ThreadPriority priority);
  // Thread group
  static oop  threadGroup(oop java_thread);
  // Stillborn
  static bool is_stillborn(oop java_thread);
  static void set_stillborn(oop java_thread);
  // Alive (NOTE: this is not really a field, but provides the correct
  // definition without doing a Java call)
  static bool is_alive(oop java_thread);
  // Daemon
  static bool is_daemon(oop java_thread);
  static void set_daemon(oop java_thread);
  // Control context
  static oop inherited_access_control_context(oop java_thread);
  // Debugging
  friend class JavaClasses;
};



// Interface to java.lang.ThreadGroup objects

class java_lang_ThreadGroup : AllStatic {
 private:
  enum {
   parent_offset            = 0,
   name_offset              = 1,
   threads_offset           = 2,
   groups_offset            = 3,
   maxPriority_offset       = 4,
   destroyed_offset         = 5,
   daemon_offset            = 6,
   vmAllowSuspension_offset = 7,
   nthreads_offset          = 8,
   ngroups_offset           = 9
  };
 
 public:  
  // parent ThreadGroup
  static oop  parent(oop java_thread_group);
  // name
  static typeArrayOop name(oop java_thread_group);
  // ("name as oop" accessor is not necessary)
  // Number of threads in group
  static int nthreads(oop java_thread_group);
  // threads
  static objArrayOop threads(oop java_thread_group);
  // Number of threads in group
  static int ngroups(oop java_thread_group);
  // groups
  static objArrayOop groups(oop java_thread_group);
  // maxPriority in group
  static ThreadPriority maxPriority(oop java_thread_group);
  // Destroyed
  static bool is_destroyed(oop java_thread_group);
  // Daemon
  static bool is_daemon(oop java_thread_group);
  // vmAllowSuspension
  static bool is_vmAllowSuspension(oop java_thread_group);
  // Debugging
  friend class JavaClasses;
};
  


// Interface to java.lang.Throwable objects

class java_lang_Throwable: AllStatic {
 private:
  enum {
    backtrace_offset     =  0,
    detailMessage_offset =  1
  };
 
  // Printing
  static void print_to_stream(Handle stream, const char* str);

 public:
  // Backtrace
  static oop backtrace(oop throwable);
  static void set_backtrace(oop throwable, oop value);
  // Message
  static oop message(oop throwable);
  static oop message(Handle throwable);
  static void set_message(oop throwable, oop value);
  // Print stack trace stored in exception by call-back to Java
  static void print_stack_trace(oop throwable, oop print_stream);
  static void print_stack_element(Handle stream, methodOop method, int bci);
  static void print_stack_usage(Handle stream);
  // Fill in current stack trace, can cause GC
  static void fill_in_stack_trace(Handle throwable, TRAPS);
  static void fill_in_stack_trace(Handle throwable);
  // Printing
  static void print(oop throwable, outputStream* st);
  static void print(Handle throwable, outputStream* st);
  // Debugging
  friend class JavaClasses;
};


// Interface to java.lang.reflect.AccessibleObject objects

class java_lang_reflect_AccessibleObject: AllStatic {
 private:
  enum {
   override_offset  =  0
  };
 public:
  // Accessors
  static jboolean override(oop reflect);
  static void set_override(oop reflect, jboolean value);

  // Debugging
  friend class JavaClasses;
};


// Interface to java.lang.reflect.Method objects

class java_lang_reflect_Method : public java_lang_reflect_AccessibleObject {
 private:
  enum {
    clazz_offset          =  1,
    name_offset           =  2,
    returnType_offset     =  3,
    parameterTypes_offset =  4,
    exceptionTypes_offset =  5,
    slot_offset           =  6,
    modifiers_offset      =  7
  };
 public:
  // Allocation
  static oop create(TRAPS);

  // Accessors
  static oop clazz(oop reflect);
  static void set_clazz(oop reflect, oop value);

  static oop name(oop method);
  static void set_name(oop method, oop value);

  static oop return_type(oop method);
  static void set_return_type(oop method, oop value);

  static oop parameter_types(oop method);
  static void set_parameter_types(oop method, oop value);

  static oop exception_types(oop method);
  static void set_exception_types(oop method, oop value);

  static int slot(oop reflect);
  static void set_slot(oop reflect, int value);

  static int modifiers(oop method);
  static void set_modifiers(oop method, int value);

  // Debugging
  friend class JavaClasses;
};


// Interface to java.lang.reflect.Constructor objects

class java_lang_reflect_Constructor : public java_lang_reflect_AccessibleObject {
 private:
  enum {
    clazz_offset          =  1,
    parameterTypes_offset =  2,
    exceptionTypes_offset =  3,
    slot_offset           =  4,
    modifiers_offset      =  5
  };
 public:
  // Allocation
  static oop create(TRAPS);

  // Accessors
  static oop clazz(oop reflect);
  static void set_clazz(oop reflect, oop value);

  static oop parameter_types(oop constructor);
  static void set_parameter_types(oop constructor, oop value);

  static oop exception_types(oop constructor);
  static void set_exception_types(oop constructor, oop value);

  static int slot(oop reflect);
  static void set_slot(oop reflect, int value);

  static int modifiers(oop constructor);
  static void set_modifiers(oop constructor, int value);

  // Debugging
  friend class JavaClasses;
};


// Interface to java.lang.reflect.Field objects

class java_lang_reflect_Field : public java_lang_reflect_AccessibleObject {
 private:
  enum {
    clazz_offset          =  1,
    name_offset           =  2,
    type_offset           =  3,
    slot_offset           =  4,
    modifiers_offset      =  5
  };
 public:
  // Allocation
  static oop create(TRAPS);

  // Accessors
  static oop clazz(oop reflect);
  static void set_clazz(oop reflect, oop value);

  static oop name(oop field);
  static void set_name(oop field, oop value);

  static oop type(oop field);
  static void set_type(oop field, oop value);

  static int slot(oop reflect);
  static void set_slot(oop reflect, int value);

  static int modifiers(oop field);
  static void set_modifiers(oop field, int value);

  // Debugging
  friend class JavaClasses;
}; 


// Interface to java.lang primitive type boxing objects:
//  - java.lang.Boolean
//  - java.lang.Character
//  - java.lang.Float
//  - java.lang.Double
//  - java.lang.Byte
//  - java.lang.Short
//  - java.lang.Integer
//  - java.lang.Long

// This could be separated out into 8 individual classes.

class java_lang_boxing_object: AllStatic {
 private:
  enum {
   value_offset  =  0
  };
  static oop initialize_and_allocate(klassOop klass, TRAPS);
 public:
  // Allocation. Returns a boxed value, or NULL for invalid type.
  static oop create(BasicType type, jvalue* value, TRAPS);
  // Accessors. Returns the basic type being boxed, or T_ILLEGAL for invalid oop.
  static BasicType get_value(oop box, jvalue* value);
  static BasicType set_value(oop box, jvalue* value);

  // Debugging
  friend class JavaClasses;
};



// Interface to java.lang.ref.Reference objects

class java_lang_ref_Reference: AllStatic {
 public:
  enum {
   referent_offset                      =  0,
   queue_offset                         =  1,
   next_offset                          =  2
  };
  enum {
   static_lock_offset                   =  0,
   static_pending_offset                =  1
  };
 
  // Accessors
  static oop referent(oop ref)        { return *referent_addr(ref); }
  static void set_referent(oop ref, oop value);
  static oop* referent_addr(oop ref);

  static oop next(oop ref)            { return *next_addr(ref); }
  static void set_next(oop ref, oop value);
  static oop* next_addr(oop ref);

  // Accessors for statics
  static oop  pending_list_lock()     { return *pending_list_lock_addr(); }
  static oop  pending_list()          { return *pending_list_addr(); }

  static oop* pending_list_lock_addr();
  static oop* pending_list_addr();
};


// Interface to java.lang.ref.SoftReference objects

class java_lang_ref_SoftReference: public java_lang_ref_Reference {
 public:
  enum {
   timestamp_offset                     = ALIGN_LONG_FIELDS ? 4 : 3
  };
  enum {
   static_clock_offset                  = 0
  };

  // Accessors
  static jlong timestamp(oop ref);

  // Accessors for statics
  static jlong clock();
  static void set_clock(jlong value);
};


// Interface to java.io.FileDescriptor objects

class java_io_FileDescriptor: AllStatic {
 private:
  enum {
   fd_offset = 0 // There are more fields, but this is the only one we currently use
  };
 public:
  static long     fd(oop java_filedescriptor);
  static void set_fd(oop java_filedescriptor, long fd);

  // Debugging
  friend class JavaClasses;
};


// Interface to java.security.AccessControlContext objects

class java_security_AccessControlContext: AllStatic {
 private:
  // Note that for this class the layout changed between JDK1.2 and JDK1.3,
  // so we compute the offsets at startup rather than hard-wiring them.
  static int _context_offset;
  static int _privilegedContext_offset;
  static int _isPrivileged_offset;

  static void compute_offsets();
 public:
  static oop create(objArrayHandle context, bool isPrivileged, Handle privileged_context, TRAPS);  

  // Debugging/initialization
  friend class JavaClasses;
};


// class java.lang.ClassLoader objects

class java_lang_ClassLoader : AllStatic {
 private:
  enum {
   parent_offset            = 0
  };
  static oop parent(oop loader);

 public:
  static bool is_trusted_loader(oop loader);
  // Debugging
  friend class JavaClasses;
};


// Interface to hard-coded offset checking


class JavaClasses : AllStatic {
 private:
  static bool check_offset(const char *klass_name, int offset, const char *field_name, const char* field_sig) PRODUCT_RETURN0;
  static bool check_static_offset(const char *klass_name, int hardcoded_offset, const char *field_name, const char* field_sig) PRODUCT_RETURN0;
 public:
  static void compute_offsets();
  static void check_offsets() PRODUCT_RETURN;
};
