/*
 * @(#)markSweep.cpp	1.161 01/02/14 09:19:41
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_markSweep.cpp.incl"

bool                    MarkSweep::_is_active           = false;
int                     MarkSweep::_invoke_count        = 0;
elapsedTimer            MarkSweep::_accumulated_time;
jlong                   MarkSweep::_time_of_last_gc     = 0;


GrowableArray<oop>*     MarkSweep::_marking_stack       = NULL;
GrowableArray<Klass*>*  MarkSweep::_revisit_klass_stack = NULL;

GrowableArray<oop>*     MarkSweep::_preserved_oop_stack = NULL;
GrowableArray<markOop>* MarkSweep::_preserved_mark_stack= NULL;
int			MarkSweep::_preserved_count = 0;
int			MarkSweep::_preserved_count_max = 0;
PreservedMark*          MarkSweep::_preserved_marks = NULL;


#ifdef VALIDATE_MARK_SWEEP
GrowableArray<oop*>*    MarkSweep::_root_refs_stack = NULL;
GrowableArray<oop> *    MarkSweep::_live_oops = NULL;
GrowableArray<oop> *    MarkSweep::_live_oops_moved_to = NULL;
GrowableArray<int> *    MarkSweep::_live_oops_size = NULL;
int                     MarkSweep::_live_oops_index = 0;
int                     MarkSweep::_live_oops_index_at_perm = 0;
GrowableArray<oop*>*    MarkSweep::_other_refs_stack = NULL;
GrowableArray<oop*>*    MarkSweep::_adjusted_pointers = NULL;
bool 			MarkSweep::_pointer_tracking = false;
bool 			MarkSweep::_root_tracking = true;
#endif

void MarkSweep::revisit_weak_klass_link(Klass* k) {
  _revisit_klass_stack->push(k);
}


void MarkSweep::follow_weak_klass_links() {
  // All klasses on the revisit stack are marked at this point.
  // Update and follow all subklass, sibling and implementor links.
  for (int i=0; i<_revisit_klass_stack->length(); i++) {
    _revisit_klass_stack->at(i)->follow_weak_klass_links();
  }
  follow_stack();
}


void MarkSweep::invoke(int size_to_be_allocated, MarkSweepInvokeCause cause, TRAPS) {
  COMPILER2_ONLY(if (OptoNoGC) return;)

  if (GC_locker::is_active()) return;

  VM_MarkSweep op(size_to_be_allocated, cause);
  VMThread::execute(&op);
}


class JVMPI_Object_Free : public ObjectClosure {
  void do_object(oop obj) {
    if (!obj->mark()->is_marked()) {
      jvmpi::post_object_free_event(obj);
    }
  }
};


void MarkSweep::invoke_at_safepoint(int size_to_be_allocated, bool deferred, bool clear_all_softrefs,
                                    bool& notify_ref_lock, MarkSweepInvokeCause cause) {
  assert(SafepointSynchronize::is_at_safepoint(), "should be at safepoint");
  NOT_CORE(assert(InlineCacheBuffer::is_empty(), "should have cleaned up ICBuffer");)
  assert(Thread::current() == VMThread::vm_thread(), "should be in vm thread");
  guarantee(!is_active(), "mark-sweep is not reentrant");

  if (GC_locker::is_active()) return;

  _invoke_count++;
  NOT_PRODUCT(if (_invoke_count == GCBreakAt) BREAKPOINT;)
    
  // Ensure eden top is correct
  Universe::new_gen()->eden()->ensure_proper_top();

  if (UseTLE) {
    Universe::fill_all_tles();
  }

  // JVMPI notification
  if (jvmpi::is_event_enabled(JVMPI_EVENT_GC_START)) {
    jvmpi::post_gc_start_event();
  }

  // Reset scavenge flag
  Scavenge::set_last_scavenge_near_limit(false);

  if ((VerifyBeforeScavenge || VerifyBeforeGC) && _invoke_count >= VerifyGCStartAt) {
    // We have to update the offset arrays for verify to work. The offset arrays 
    // are actually not needed during mark-sweep and are up-to-date afterwards.
    Universe::update_offset_tables();
    Universe::verify(true);
  }
  NOT_PRODUCT(instanceRefKlass::verify_no_references_recorded());
  
  bool clear_remset;
  bool marked_for_unloading;  // did phase1 mark any methods for unloading

  {
    ResourceMark rm;  // ResourceMark is here for post_class_unload_events()
    {
      FlagSetting fl(_is_active, true);
      EventMark m("Mark Sweep");
  
      // Timing
      TraceTime t("Full GC", PrintGC);
      if (TraceMarkSweepTime) accumulated_time()->start();
  
      VALIDATE_MARK_SWEEP_ONLY(if (PrintGC && ValidateMarkSweep) tty->print(" (Validated)"));
      size_t prev_used, prev_permanent_used;
      if (PrintGC || PrintHeapUsageOverTime) {
        prev_used = Universe::mem_used();
        prev_permanent_used = Universe::perm_gen()->used();
      }
  
      HandleMark hm;
      No_GC_Verifier ngc;
  
      if (PrintGC && Verbose) {
        tty->print(" invoke=%d size=%d <%s>", _invoke_count, size_to_be_allocated*wordSize, invoke_cause_name(cause));
      }
      // Call allocation profiler
      AllocationProfiler::iterate_since_last_gc();
  
      if (UseTrainGC) Universe::train_gen()->full_gc_prologue();
      if (deferred) instanceRefKlass::disable_refs_discovery(); // temporary functionality        
  
      // Flush the Branch/Call Frequency Tables
      COMPILER2_ONLY(if(CountInterpBranches)    SharedInfo::flush_branch_freq_table();)
      COMPILER2_ONLY(if(CountInterpCalls)       SharedInfo::flush_call_freq_table();)
  
      NOT_CORE(CodeCache::gc_prologue();)
      NOT_CORE(CallProfile::gc_prologue();)
      Threads::gc_prologue();
  
      allocate_stacks();	    
  
      COMPILER2_ONLY(DerivedPointerTable::clear();)
      RememberedSet::clear();        // clear remembered set; it is used for object sizes
  
      mark_sweep_phase1(marked_for_unloading, clear_all_softrefs);
  
      if (jvmpi::is_event_enabled(JVMPI_EVENT_OBJECT_FREE)) {
        JVMPI_Object_Free clo;
        Universe::object_iterate(&clo);
      }
  
      mark_sweep_phase2();
  
      COMPILER2_ONLY(DerivedPointerTable::deactivate();) // Don't add anymore derived pointers during phase3
  
      mark_sweep_phase3();    
  
      VALIDATE_MARK_SWEEP_ONLY(if (ValidateMarkSweep) guarantee(_root_refs_stack->length() == 0, "should be empty by now"));
  
      mark_sweep_phase4();
  
      VALIDATE_MARK_SWEEP_ONLY(if (ValidateMarkSweep) guarantee(_live_oops->length() == _live_oops_moved_to->length(), "should be the same size"));
  
      clear_remset = UseTrainGC ? false : (Universe::new_gen()->used() == 0);
  
      restore_marks();
  
      Threads::gc_epilogue();
      NOT_CORE(CallProfile::gc_epilogue();)
      NOT_CORE(CodeCache::gc_epilogue();)            
      COMPILER2_ONLY(DerivedPointerTable::update_pointers();)
  
      // Set saved marks for allocation profiler
      Universe::set_generation_saved_marks_to_top();
  
      // Compute new heap sizes
      Universe::compute_new_generation_sizes();
    
      // Enqueue reference objects found. Note that we use |= here, since scavenge may already have set the variable to true.
      notify_ref_lock |= instanceRefKlass::enqueue_discovered_references();
  
      if (clear_remset) {
        RememberedSet::clear();      // clear remembered set; we have no pointers from old to new
      } else {
        RememberedSet::invalidate(); // invalidate remembered set; we may have pointers from old to new
        if (UseTrainGC) Universe::train_gen()->clear_remembered_sets();
      }
  
      if (UseTrainGC) Universe::train_gen()->full_gc_epilogue();
  
      deallocate_stacks();	    
  
      // Timing
      if (TraceMarkSweepTime) accumulated_time()->stop();
  
      if (PrintGC) {
        tty->print(" %uK->%uK(%uK)", prev_used / K, Universe::mem_used() / K, Universe::mem_capacity() / K);
        if (Verbose) {
          tty->print(" permanent %dK->%dK(%dK)", prev_permanent_used / K, Universe::perm_gen()->used() / K, Universe::perm_gen()->capacity() / K);
        }
      }
      if (PrintHeapUsageOverTime) Universe::print_heap_usage_stamp(prev_used, invoke_cause_name(cause));
  
    }

    // Information about classes unloaded by GC may have been saved by
    // SystemDictionary::follow_roots_phase2(). We post CLASS_UNLOAD
    // events from here because we can lock GC. We always call this
    // function to make sure that the saved memory is released.
    jvmpi::post_class_unload_events();
  }

  // Check whether any nmethods got marked for deoptimization when traversing CodeCache
  // We will only mark methods that has been unloaded. There will be no active activations
  // on the stack so we do not need to deoptimize anything. We just mark the nmethods as 
  // unloaded. The NmethodSweeper will make sure that to unload them when no more inline caches, etc.
  // are pointing to them. We do not make them zombies here, since that will require the Compile_lock
  NOT_CORE(if (marked_for_unloading) CodeCache::make_marked_nmethods_unloaded();)
    
  Scavenge::reset_tick_adjustment();

  // Update timestamp
  reset_time_since_last_gc();

  if ((VerifyAfterScavenge || VerifyAfterGC) && _invoke_count >= VerifyGCStartAt) {
    Universe::verify(!clear_remset);
  }
  NOT_PRODUCT(instanceRefKlass::verify_no_references_recorded());

  if (jvmpi::is_event_enabled(JVMPI_EVENT_GC_FINISH)) {
    jvmpi::post_gc_finish_event(Universe::mem_used(), Universe::mem_capacity());
  }
  if (Universe::jvmpi_alloc_event_enabled()) {
    Scavenge::fill_newgen();
  }

  Universe::set_heap_capacity_at_last_gc(Universe::mem_capacity());
  Universe::set_heap_free_at_last_gc(Universe::mem_free());
  Universe::set_heap_used_at_last_gc(Universe::mem_used());

  // Last ditch bail-out for out-of-memory conditions.
  // The check for clear_all_softrefs prevents recursion.
  size_t new_gen_free = Universe::new_gen()->free() / wordSize;
  size_t old_gen_free = Universe::old_gen()->free() / wordSize;
  if (!clear_all_softrefs && (new_gen_free <= (size_t)size_to_be_allocated ||
                              old_gen_free <= (size_t)size_to_be_allocated)) {
    invoke_at_safepoint(size_to_be_allocated, deferred, true, notify_ref_lock, cause);
  }
}


void MarkSweep::allocate_stacks() {
  Space * to_space = Universe::new_gen()->to();
  _preserved_marks = (PreservedMark*) to_space->top();
  _preserved_count = 0;

  _preserved_count_max = (to_space->end() - to_space->top()) * sizeof(oop*) / sizeof(PreservedMark);
  _preserved_mark_stack = NULL;
  _preserved_oop_stack = NULL;
  if (to_space->top() > to_space->bottom()) {
    trace("to_space not empty");
  }

  _marking_stack       = new GrowableArray<oop>(4000, true);

  int size = SystemDictionary::number_of_classes() * 2;
  _revisit_klass_stack = new GrowableArray<Klass*>(size, true);

#ifdef VALIDATE_MARK_SWEEP
  if (ValidateMarkSweep) {
    _root_refs_stack    = new GrowableArray<oop*>(100, true);
    _other_refs_stack   = new GrowableArray<oop*>(100, true);
    _adjusted_pointers  = new GrowableArray<oop*>(100, true);
    _live_oops          = new GrowableArray<oop>(100, true);
    _live_oops_moved_to = new GrowableArray<oop>(100, true);
    _live_oops_size     = new GrowableArray<int>(100, true);
  }
#endif
}


void MarkSweep::deallocate_stacks() {
  if (_preserved_oop_stack) {
    _preserved_mark_stack->clear_and_deallocate();
    delete _preserved_mark_stack;
    _preserved_mark_stack = NULL;
    _preserved_oop_stack->clear_and_deallocate();
    delete _preserved_oop_stack;
    _preserved_oop_stack = NULL;
  }

  _marking_stack->clear_and_deallocate();
  _revisit_klass_stack->clear_and_deallocate();

#ifdef VALIDATE_MARK_SWEEP
  if (ValidateMarkSweep) {
    _root_refs_stack->clear_and_deallocate();
    _other_refs_stack->clear_and_deallocate();
    _adjusted_pointers->clear_and_deallocate();
    _live_oops->clear_and_deallocate();
    _live_oops_size->clear_and_deallocate();
    _live_oops_moved_to->clear_and_deallocate();
    _live_oops_index = 0;
    _live_oops_index_at_perm = 0;
  }
#endif
}


inline void MarkSweep::mark_object(oop obj) {
  // some marks may contain information we need to preserve so we store them away
  // and overwrite the mark.  We'll restore it at the end of markSweep.
  markOop mark = obj->mark();
  obj->set_mark(markOopDesc::prototype()->set_marked());

  if (mark != markOopDesc::prototype()) {
    // we may need to preserve this mark
    if (mark->must_be_preserved()) {
      // the only marks we don't have to preserve are unlocked ones with no hash.
      // Anything else needs to be preserved
      preserve_mark(obj, mark);
    }
  }
}



void MarkSweep::mark_and_follow(oop* p) {
  assert(Universe::is_reserved_heap((oop)p), "we should only be traversing objects here");
  oop m = *p;
  if (m && !m->mark()->is_marked()) {
    mark_object(m);
    m->follow_contents();  // Follow contents of the marked object
  }
}


void MarkSweep::_mark_and_push(oop* p) {
  // Push marked object, contents will be followed later
  oop m = *p;
  mark_object(m);
  _marking_stack->push(m);
}


void MarkSweep::follow_root(oop* p) {
  assert(!Universe::is_reserved_heap((oop)p), 
         "roots shouldn't be things within the heap");
#ifdef VALIDATE_MARK_SWEEP
  if (ValidateMarkSweep) {
    guarantee(!_root_refs_stack->contains(p), "should only be in here once");
    _root_refs_stack->push(p);
  }
#endif
  oop m = *p;
  if (m && !m->mark()->is_marked()) {
    mark_object(m);
    m->follow_contents();  // Follow contents of the marked object
  }
  follow_stack();
}


void MarkSweep::follow_stack() {
  while (!_marking_stack->is_empty()) {
    oop obj = _marking_stack->pop();
    assert (obj->is_gc_marked(), "p must be marked");
    obj->follow_contents();
  }
}

// We preserve the mark which should be replaced at the end and the location that it
// will go.  Note that the object that this markOop belongs isn't currently at that
// address but it will be after phase4
void MarkSweep::preserve_mark(oop obj, markOop mark) {
  // we try to store preserved marks in the to space of the new generation since this
  // is storage which should be available.  Most of the time this should be sufficient
  // space for the marks we need to preserve but if it isn't we fall back in using
  // GrowableArrays to keep track of the overflow.
  if (_preserved_count < _preserved_count_max) {
    _preserved_marks[_preserved_count++].init(obj, mark);
  } else {
    if (_preserved_mark_stack == NULL) {
      _preserved_mark_stack = new GrowableArray<markOop>(40, true);
      _preserved_oop_stack = new GrowableArray<oop>(40, true);
    }
    _preserved_mark_stack->push(mark);
    _preserved_oop_stack->push(obj);
  }
}


void MarkSweep::adjust_marks() {
  assert(_preserved_oop_stack == NULL ||
	 _preserved_oop_stack->length() == _preserved_mark_stack->length(),
	 "inconsistent preserved oop stacks");

  // adjust the oops we saved earlier
  for (int i = 0; i < _preserved_count; i++) {
    _preserved_marks[i].adjust_pointer();
  }

  // deal with the overflow stack
  if (_preserved_oop_stack) {
    for (int i = 0; i < _preserved_oop_stack->length(); i++) {
      oop* p = _preserved_oop_stack->adr_at(i);
      adjust_pointer(p);
    }
  }
}


void MarkSweep::restore_marks() {
  assert(_preserved_oop_stack == NULL ||
	 _preserved_oop_stack->length() == _preserved_mark_stack->length(),
	 "inconsistent preserved oop stacks");
  if (PrintGC && Verbose) {
    tty->print_cr("Restoring %d marks", _preserved_count +
		  (_preserved_oop_stack ? _preserved_oop_stack->length() : 0));
  }

  // restore the marks we saved earlier
  for (int i = 0; i < _preserved_count; i++) {
    _preserved_marks[i].restore();
  }

  // deal with the overflow
  if (_preserved_oop_stack) {
    for (int i = 0; i < _preserved_oop_stack->length(); i++) {
      oop obj       = _preserved_oop_stack->at(i);
      markOop mark  = _preserved_mark_stack->at(i);
      obj->set_mark(mark);      
    }
  }
}


#ifdef VALIDATE_MARK_SWEEP

void MarkSweep::track_adjusted_pointer(oop* p, oop newobj, bool isroot) {
  if (!ValidateMarkSweep)
    return;

  if (!isroot) {
    if (_pointer_tracking) {
      guarantee(_adjusted_pointers->contains(p), "should have seen this pointer");
      _adjusted_pointers->remove(p);
    }
  } else {
    int index = _root_refs_stack->find(p);
    if (index != -1) {
      int l = _root_refs_stack->length();
      if (l > 0 && l - 1 != index) {
	oop* last = _root_refs_stack->pop();
	assert(last != p, "should be different");
	_root_refs_stack->at_put(index, last);
      } else {
	_root_refs_stack->remove(p);
      }
    }
  }
}


void MarkSweep::check_adjust_pointer(oop* p) {
  _adjusted_pointers->push(p);
}


class AdjusterTracker: public OopClosure {
 public:
  AdjusterTracker() {};
  virtual void do_oop(oop* o)          { MarkSweep::check_adjust_pointer(o); }
};


void MarkSweep::track_interior_pointers(oop obj) {
  if (ValidateMarkSweep) {
    _adjusted_pointers->clear();
    _pointer_tracking = true;
    
    AdjusterTracker checker;
    obj->oop_iterate(&checker);
  }
}


void MarkSweep::check_interior_pointers() {
  if (ValidateMarkSweep) {
    _pointer_tracking = false;
    guarantee(_adjusted_pointers->length() == 0, "should have processed the same pointers");
  }
}


void MarkSweep::reset_live_oop_tracking(bool at_perm) {
  if (ValidateMarkSweep) {
    guarantee(_live_oops->length() == _live_oops_index, "should be at end of live oops");
    _live_oops_index = at_perm ? _live_oops_index_at_perm : 0;
  }
}


void MarkSweep::register_live_oop(oop p, int size) {
  if (ValidateMarkSweep) {
    _live_oops->push(p);
    _live_oops_size->push(size);
    _live_oops_index++;
  }
}

void MarkSweep::validate_live_oop(oop p, int size) {
  if (ValidateMarkSweep) {
    oop obj = _live_oops->at(_live_oops_index);
    guarantee(obj == p, "should be the same object");
    guarantee(_live_oops_size->at(_live_oops_index) == size, "should be the same size");
    _live_oops_index++;
  }
}

void MarkSweep::live_oop_moved_to(oop* q, int size, oop* compaction_top) {
  assert(oop(q)->forwardee() == NULL || oop(q)->forwardee() == oop(compaction_top),
	 "should be moved to forwarded location");
  if (ValidateMarkSweep) {
    MarkSweep::validate_live_oop(oop(q), size);
    _live_oops_moved_to->push(oop(compaction_top));
  }
}
#endif //VALIDATE_MARK_SWEEP

bool MarkSweep::mark_sweep_is_alive(oop p) {
  return p->is_gc_marked();
}


void MarkSweep::mark_sweep_keep_alive(oop* p) {
#ifdef VALIDATE_MARK_SWEEP
  if (ValidateMarkSweep) {
    if (!Universe::is_reserved_heap((oop)p)) {
      _root_refs_stack->push(p);
    } else {
      _other_refs_stack->push(p);
    }
  }
#endif
  mark_and_push(p);
}


void MarkSweep::mark_sweep_phase1(bool& marked_for_unloading, bool clear_all_softrefs) {
  // Recursively traverse all live objects and mark them
  EventMark m("1 mark object");
  TraceTime tm("phase 1", PrintGC && Verbose);
  trace(" 1");

  VALIDATE_MARK_SWEEP_ONLY(reset_live_oop_tracking(false));

  NOT_CORE(CallProfile::oops_do(&follow_root);)
  Threads::oops_do(&follow_root);
  ObjectSynchronizer::oops_do(&follow_root);

  Universe::oops_do(&follow_root);
  FlatProfiler::oops_do(&follow_root);
  if (jvmdi::enabled()) { 
    jvmdi::oops_do(&follow_root);
  }
  vmSymbols::follow_roots();
  JNIHandles::oops_do(&follow_root);  // Global (strong) JNI handles

  NOT_CORE(CodeCache::follow_roots_phase1();)
  SystemDictionary::follow_roots_phase1();

  // Process reference objects found during marking
  ReferencePolicy *soft_ref_policy;
  if (clear_all_softrefs) {
    soft_ref_policy = new AlwaysClearPolicy();
  } else {
    NOT_COMPILER2(soft_ref_policy = new LRUCurrentHeapPolicy();)
    COMPILER2_ONLY(soft_ref_policy = new LRUMaxHeapPolicy();)
  }
  assert(soft_ref_policy != NULL,"No soft reference policy");
  instanceRefKlass::process_discovered_references(soft_ref_policy, &mark_sweep_is_alive,
                                                  &mark_sweep_keep_alive, &follow_stack);

  // Follow system dictionary roots and unload classes
  bool purged_class = SystemDictionary::follow_roots_phase2();
  assert(_marking_stack->is_empty(), "stack should be empty by now");

  // Follow code cache roots (has to be done after system dictionary, assumes all live klasses are marked)
  NOT_CORE(CodeCache::follow_roots_phase2(purged_class, marked_for_unloading);)   // did we mark any nmethods for unloading?

  // Update subklass/sibling/implementor links of live klasses
  follow_weak_klass_links();

  // Visit symbol and interned string tables and delete unmarked oops
  SymbolTable::follow_roots();
  StringTable::follow_roots();

  assert(_marking_stack->is_empty(), "stack should be empty by now");
}



void MarkSweep::mark_sweep_phase2() {
  // Now all live objects are marked, compute the new object addresses.

  // It is imperative that we traverse perm_gen LAST. If dead space is allowed a range of dead
  // object may get overwritten by a dead int array. If perm_gen is not traversed last a klassOop
  // may get overwritted. This is fine since it is dead, but if the class has dead instances we
  // have to skip them, and in order to find their size we need the klassOop!
  
  // It is not required that we traverse spaces in the same order in phase2, phase3 and phase4,
  // but the ValidateMarkSweep live oops tracking expects us to do so. See comment under phase4.
  
  EventMark m("2 compute new addresses");
  TraceTime tm("phase 2", PrintGC && Verbose);
  trace("2");

  VALIDATE_MARK_SWEEP_ONLY(reset_live_oop_tracking(false));

  if (UseTrainGC) {
    // If we use train gc we currently have to invalidate the remembered
    // set anyway, so we might as well compact new_gen locally.
    Universe::train_gen()->prepare_for_compaction();
    Universe::new_gen()->prepare_for_compaction();
  } else {
    Universe::tenured_gen()->prepare_for_compaction();  // will traverse new_gen as well
  }
  VALIDATE_MARK_SWEEP_ONLY(_live_oops_index_at_perm = _live_oops_index;)
  Universe::perm_gen()->prepare_for_compaction();
}

bool always_true(oop obj) {
  return true;
}

void MarkSweep::mark_sweep_phase3() {
  // Adjust the pointers to reflect the new locations
  EventMark m("3 adjust pointers");
  TraceTime tm("phase 3", PrintGC && Verbose);
  trace("3");

  VALIDATE_MARK_SWEEP_ONLY(reset_live_oop_tracking(false));

  Threads::oops_do(&adjust_root_pointer);
  ObjectSynchronizer::oops_do(&adjust_root_pointer);

  Universe::oops_do(&adjust_root_pointer);
  FlatProfiler::oops_do(&adjust_root_pointer);
  if (jvmdi::enabled()) {
    jvmdi::oops_do(&adjust_root_pointer);
  }
  vmSymbols::adjust_roots();
  JNIHandles::oops_do(&adjust_root_pointer);  // Global (strong) JNI handles
  JNIHandles::weak_oops_do(&always_true, &adjust_root_pointer);  // Global (weak) JNI handles

  NOT_CORE(CodeCache::adjust_pointers();)
  SystemDictionary::adjust_pointers();
  SymbolTable::adjust_pointers();
  StringTable::adjust_pointers();

  instanceRefKlass::adjust_roots();
  adjust_marks();

  if (UseTrainGC) {
    Universe::train_gen()->adjust_pointers();
    Universe::new_gen()->adjust_pointers();
  } else {
    Universe::tenured_gen()->adjust_pointers();  // will traverse new_gen as well
  }
  Universe::perm_gen()->adjust_pointers();
}

void MarkSweep::mark_sweep_phase4() {
  // All pointers are now adjusted, move objects accordingly

  // It is imperative that we traverse perm_gen first in phase4. All classes must be allocated
  // earlier than their instances, and traversing perm_gen first makes sure that all klassOops
  // have moved to their new location before any instance does a dispatch through it's klass!

  // The ValidateMarkSweep live oops tracking expects us to traverse spaces in the same order 
  // in phase2, phase3 and phase4. We don't quite do that here (perm_gen first rather than last),
  // so we tell the validate code to use a higher index (saved from phase2) when verifying perm_gen.

  EventMark m("4 compact heap");
  TraceTime tm("phase 4", PrintGC && Verbose);
  trace("4");

  VALIDATE_MARK_SWEEP_ONLY(reset_live_oop_tracking(true));

  Universe::perm_gen()->compact();
  
  VALIDATE_MARK_SWEEP_ONLY(reset_live_oop_tracking(false));
  
  if (UseTrainGC) {
    Universe::train_gen()->compact();
    Universe::new_gen()->compact();
  } else {
    Universe::tenured_gen()->compact();  // will traverse new_gen as well
  }
}


jlong MarkSweep::time_since_last_gc() { 
  return os::javaTimeMillis() - _time_of_last_gc; 
}


void MarkSweep::reset_time_since_last_gc() { 
  _time_of_last_gc = os::javaTimeMillis(); 
}


void marksweep_init() {
  MarkSweep::reset_time_since_last_gc();
}


#ifndef PRODUCT

void MarkSweep::trace(const char* msg) {
  if (TraceMarkSweep)
    tty->print("%s", msg);
}


const char* MarkSweep::invoke_cause_name(MarkSweepInvokeCause cause) {
  switch (cause) {
    case _java_lang_system_gc:
      return "System.gc()";
    case _tenured_generation_full:
      return "tenured generation full";
    case _permanent_generation_full:
      return "permanent generation full";
    case _train_generation_full:
      return "train generation full";
    case _old_generation_expanded_on_last_scavenge:
      return "old generation expanded on last scavenge";
    case _old_generation_too_full_to_scavenge:
      return "old generation too full to scavenge";
    case _full_gc_alot:
      return "FullGCAlot";
    default: 
      break;
  }
  ShouldNotReachHere();
  return NULL;
}

#endif

