/*
 * @(#)oopFactory.cpp	1.55 00/03/21 16:17:01
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_oopFactory.cpp.incl"


typeArrayOop oopFactory::new_charArray(const char* utf8_str, TRAPS) {
  int length = utf8_str == NULL ? 0 : UTF8::unicode_length(utf8_str);
  typeArrayOop result = new_charArray(length, CHECK_0);
  if (length != 0) {
    UTF8::convert_to_unicode(utf8_str, result->char_at_addr(0), length);
  }
  return result;
}

typeArrayOop oopFactory::new_permanent_charArray(int length, TRAPS) {
  return typeArrayKlass::cast(Universe::charArrayKlassObj())->allocate_permanent(length, THREAD);
}

typeArrayOop oopFactory::new_permanent_byteArray(int length, TRAPS) {
  return typeArrayKlass::cast(Universe::byteArrayKlassObj())->allocate_permanent(length, THREAD);
}


typeArrayOop oopFactory::new_permanent_shortArray(int length, TRAPS) {
  return typeArrayKlass::cast(Universe::shortArrayKlassObj())->allocate_permanent(length, THREAD);
}


typeArrayOop oopFactory::new_permanent_intArray(int length, TRAPS) {
  return typeArrayKlass::cast(Universe::intArrayKlassObj())->allocate_permanent(length, THREAD);
}


typeArrayOop oopFactory::new_typeArray(BasicType type, int length, TRAPS) {
  switch (type) {
    case T_BOOLEAN : return new_boolArray  (length, THREAD);
    case T_CHAR    : return new_charArray  (length, THREAD);
    case T_FLOAT   : return new_singleArray(length, THREAD);
    case T_DOUBLE  : return new_doubleArray(length, THREAD);
    case T_BYTE    : return new_byteArray  (length, THREAD);
    case T_SHORT   : return new_shortArray (length, THREAD);
    case T_INT     : return new_intArray   (length, THREAD);
    case T_LONG    : return new_longArray  (length, THREAD);
  }
  fatal("Wrong array type"); 
  return NULL;
}


objArrayOop oopFactory::new_objArray(klassOop klass, int length, TRAPS) {
  assert(klass->is_klass(), "must be instance class");
  if (klass->klass_part()->oop_is_array()) {
    return ((arrayKlass*)klass->klass_part())->allocate_arrayArray(1, length, THREAD);
  } else {
    assert (klass->klass_part()->oop_is_instance(), "new object array with klass not an instanceKlass");
    return ((instanceKlass*)klass->klass_part())->allocate_objArray(1, length, THREAD);
  }
}

symbolOop oopFactory::new_symbol(const char* utf8_buffer, int utf8_length, TRAPS) {
  assert(utf8_buffer != NULL, "just checking");
  return SymbolTable::lookup(utf8_buffer, utf8_length, CHECK_0);
}

objArrayOop oopFactory::new_system_objArray(int length, TRAPS) {
  int size = objArrayOopDesc::object_size(length);
  objArrayOop o = (objArrayOop)Universe::clear_allocate_permanent_array(Universe::systemObjArrayKlassObj(), size, length, CHECK_0);
  // initialization not needed, allocated cleared
  return o;
}


constantPoolOop oopFactory::new_constantPool(int length, TRAPS) {
  constantPoolKlass* ck = constantPoolKlass::cast(Universe::constantPoolKlassObj());
  return ck->allocate(length, CHECK_0);
}


constantPoolCacheOop oopFactory::new_constantPoolCache(int length, TRAPS) {
  constantPoolCacheKlass* ck = constantPoolCacheKlass::cast(Universe::constantPoolCacheKlassObj());
  return ck->allocate(length, CHECK_0);
}


klassOop oopFactory::new_instanceKlass(int vtable_len, int itable_len, int static_field_size, 
                                       int nonstatic_oop_map_size, ReferenceType rt, TRAPS) {
  instanceKlassKlass* ikk = instanceKlassKlass::cast(Universe::instanceKlassKlassObj());
  return ikk->allocate_instance_klass(vtable_len, itable_len, static_field_size, nonstatic_oop_map_size, rt, CHECK_0);
}


methodOop oopFactory::new_method(int byte_code_size, AccessFlags access_flags, int compressed_line_number_size, int localvariable_table_length, int checked_exceptions_length, TRAPS) {
  methodKlass* mk = methodKlass::cast(Universe::methodKlassObj());
  return mk->allocate(byte_code_size, access_flags, compressed_line_number_size, localvariable_table_length, checked_exceptions_length, CHECK_0);
}


oop oopFactory::new_thread(TRAPS) {
  KlassHandle k =  SystemDictionary::resolve_or_fail(vmSymbolHandles::java_lang_Thread(), NULL, NULL, true, CHECK_0);

  // DO never change this code (corresponds to .class specs)
  Handle obj = (oop)Universe::allocate(k, 12, CHECK_0);    // 2 words for mark and klass fields of oop
  Handle nameOop = vmSymbols::main_name();                 // it is an array of characters (not a String)
    
  int	priority	= 5;
  oop	threadQOop	= NULL;                           // hidden (links the thread)
  int	privateInfo	= 0;                              // hidden
  int	eetop		= 0;                              // hidden
  bool	singleStep	= false;                          // hidden
  bool	daemon		= false;
  bool	stillborn	= false;                          // hidden
  oop	targetOop	= NULL;                           // Runnable that should run
  oop	threadGroupOop  = oopFactory::new_threadGroup(CHECK_0);
   
  oop o = obj();
  o->obj_field_put(oopDesc::header_size() + 0, nameOop());
  o->int_field_put(oopDesc::header_size() + 1, priority);
  o->obj_field_put(oopDesc::header_size() + 2, threadQOop);
  o->int_field_put(oopDesc::header_size() + 3, privateInfo);
  o->int_field_put(oopDesc::header_size() + 4, eetop);
  o->int_field_put(oopDesc::header_size() + 5, singleStep);
  o->int_field_put(oopDesc::header_size() + 6, daemon);
  o->int_field_put(oopDesc::header_size() + 7, stillborn);
  o->obj_field_put(oopDesc::header_size() + 8, targetOop);
  o->obj_field_put(oopDesc::header_size() + 9, threadGroupOop);
  
  return o;
}


oop oopFactory::new_threadGroup(TRAPS) {  
  KlassHandle h_k = SystemDictionary::resolve_or_fail(vmSymbolHandles::java_lang_ThreadGroup(), NULL, NULL, true, CHECK_0);

  // DO never change this code (corresponds to .class specs)
  Handle h_obj = (oop)Universe::allocate(h_k, 12, CHECK_0);

  oop	parentOop	= NULL;
  oop	nameOop		= java_lang_String::create_from_str("main", CHECK_0);
  int	maxPriority	= 9;
  bool	destroyed	= false;
  bool	daemon		= false;
  bool  vmAllowSuspension = false;
  int	nthreads	= 0;
  oop	threadsOop	= NULL; // array of all threads contained
  int	ngroups		= 0;
  oop	groupsOop	= NULL; // contains all ThreadGroups

  oop obj = h_obj();
  obj->obj_field_put(oopDesc::header_size() + 0, parentOop);
  obj->obj_field_put(oopDesc::header_size() + 1, nameOop);
  obj->int_field_put(oopDesc::header_size() + 2, maxPriority);
  obj->int_field_put(oopDesc::header_size() + 3, destroyed);
  obj->int_field_put(oopDesc::header_size() + 4, daemon);
  obj->int_field_put(oopDesc::header_size() + 5, vmAllowSuspension);
  obj->int_field_put(oopDesc::header_size() + 6, nthreads);
  obj->obj_field_put(oopDesc::header_size() + 7, threadsOop);
  obj->int_field_put(oopDesc::header_size() + 8, ngroups);
  obj->obj_field_put(oopDesc::header_size() + 9, groupsOop);  
  return obj;
}


#ifndef CORE
compiledICHolderOop oopFactory::new_compiledICHolder(methodOop method, klassOop klass, TRAPS) {
  compiledICHolderKlass* ck = (compiledICHolderKlass*) Universe::compiledICHolderKlassObj()->klass_part();
  compiledICHolderOop c = ck->allocate(CHECK_0);
  c->set_holder_method(method);
  c->set_holder_klass(klass);
  return c;
}
#endif

