/*
 * @(#)scavenge.cpp	1.80 01/02/09 14:45:11
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_scavenge.cpp.incl"


ageTable              Scavenge::_age_table;
int                   Scavenge::_tenuring_threshold = 0;
int                   Scavenge::_invoke_count = 0;
elapsedTimer          Scavenge::_accumulated_time;
bool                  Scavenge::_is_active = false;
bool                  Scavenge::_deferred_pending = false;
int                   Scavenge::_full_gc_revert_count = -1;
bool                  Scavenge::_last_scavenge_near_limit = false;

int                   Scavenge::_train_invoke_count = 0;
bool                  Scavenge::_reference_to_first_train_found = false;
int                   Scavenge::_tick_interval;
int                   Scavenge::_next_invoke_count;
int                   Scavenge::_delay_adjustment_count;

jlong                 Scavenge::_total_promoted = 0;
jlong                 Scavenge::_total_processed = 0;
jlong                 Scavenge::_delta_promoted;
jlong                 Scavenge::_delta_processed;

void                  (*Scavenge::scavenge)(oop*) = NULL;

void                  (*Scavenge::_scavenge_nontenured)(oop*)            = NULL;
void                  (*Scavenge::_scavenge_nontenured_with_check)(oop*) = NULL;
void                  (*Scavenge::_scavenge_tenured)(oop*)               = NULL;


void Scavenge::reset_tick_adjustment() {
  _tick_interval = DefaultTickInterval;
  _next_invoke_count = invoke_count() + 1;
  _delay_adjustment_count = DelayTickAdjustment;
  _delta_promoted = _total_promoted;
  _delta_processed = _total_processed;
}


void Scavenge::compute_tick_adjustment() {
  if (_delay_adjustment_count > 0) {
    _delay_adjustment_count--;
    _next_invoke_count = _invoke_count + 1;
    _delta_promoted = _total_promoted;
    _delta_processed = _total_processed;
  } else {
    int prev_tick_interval = _tick_interval;
    jlong promoted = _total_promoted - _delta_promoted;
    jlong processed = _total_processed - _delta_processed;
    if (promoted * ProcessingToTenuringRatio > processed) {
      // increase speed
      if (_tick_interval > MinTickInterval) {
        _tick_interval--;
      }
    } else {
      // decrease speed
      if (_tick_interval < MaxTickInterval) {
        _tick_interval++;
      } else {
        _delta_promoted = _total_promoted;
        _delta_processed = _total_processed;
      }
    }
    if (PrintScavenge && Verbose && prev_tick_interval != _tick_interval) {
      tty->print(" (tick interval %d->%d) ", prev_tick_interval, _tick_interval);
    }
    if (_tick_interval < 1) {
      // decrease eden size temporarily
      assert(Universe::new_gen()->eden()->is_empty(), "eden should be empty here");
      int factor = 2 - _tick_interval;
      assert(factor > 1, "sanity check");
      Universe::new_gen()->eden()->allocate_temporary_filler(factor);
      if (PrintScavenge && Verbose) {
        tty->print(" (decreasing eden by factor of %d) ", factor);
      }
    }
    _next_invoke_count = _invoke_count + (_tick_interval > 0 ? _tick_interval : 1);
  }
}


void Scavenge::scavenge_oop(oop* p) {
  assert(!oop(p)->is_old_or_perm(), "old ref not expected");
  oop obj = *p;
  assert(should_scavenge(obj), "invalid ref");
  if (obj->is_forwarded()) {
    *p = obj->forwardee();
  } else {        
    *p = obj->copy_to_survivor_space(p);
  }
}


void Scavenge::scavenge_oop_with_check(oop* p) {
  oop obj = *p;
  if (should_scavenge(obj)) {
    if (obj->is_forwarded()) {
      *p = obj->forwardee();
    } else {        
      *p = obj->copy_to_survivor_space(NULL);
    }
  }
}

void Scavenge::train_scavenge_oop(oop* p) {
  assert(!oop(p)->is_old_or_perm(), "old ref not expected");
  oop obj = *p;
  assert(should_scavenge(obj), "invalid ref");
  if (obj->is_forwarded()) {
    *p = obj->forwardee();
  } else {        
    CarTableDesc* to_desc = CarTable::desc_for((oop*) obj);
    if (to_desc->is_new()) {
      *p = obj->copy_to_survivor_space(p);
    } else {
      assert(to_desc->is_train(), "invalid ref");
      CarTableDesc* first_desc = Universe::train_gen()->first_car_desc();
      julong to_desc_train_number = to_desc->train_number();
      if (to_desc_train_number <= first_desc->train_number()) {
        if (to_desc_train_number == CarTableDesc::special_nonoop_train_number) {
          ((CarSpace*) to_desc->space())->set_marked(true);
        } else {
          assert(to_desc_train_number == first_desc->train_number(), "invalid train number");
          _reference_to_first_train_found = true;
          if (first_desc->car_number() == to_desc->car_number()) {
            *p = obj->copy_to_train(p);
          }
        }
      }
    }
  }
}


void Scavenge::skip_train_scavenge_oop(oop* p) {
  assert(!oop(p)->is_old_or_perm(), "old ref not expected");
  oop obj = *p;
  assert(should_scavenge(obj), "invalid ref");
  if (obj->is_forwarded()) {
    *p = obj->forwardee();
  } else {        
    CarTableDesc* to_desc = CarTable::desc_for((oop*) obj);
    if (to_desc->is_new()) {
      *p = obj->copy_to_survivor_space(p);
    } else {
      assert(to_desc->is_train(), "invalid ref");
    }
  }
}


void Scavenge::train_scavenge_oop_with_check(oop* p) {
  if (should_scavenge(*p)) {
    oop obj = *p;
    if (obj->is_forwarded()) {
      *p = obj->forwardee();
    } else {        
      CarTableDesc* to_desc = CarTable::desc_for((oop*) obj);
      if (to_desc->is_new()) {
        *p = obj->copy_to_survivor_space(NULL);
      } else {
        assert(to_desc->is_train(), "invalid ref");
        CarTableDesc* first_desc = Universe::train_gen()->first_car_desc();
        julong to_desc_train_number = to_desc->train_number();
        if (to_desc_train_number<= first_desc->train_number()) {
          if (to_desc_train_number == CarTableDesc::special_nonoop_train_number) {
            ((CarSpace*) to_desc->space())->set_marked(true);
          } else {
            assert(to_desc_train_number == first_desc->train_number(), "invalid train number");
            _reference_to_first_train_found = true;
            if (first_desc->car_number() == to_desc->car_number()) {
              *p = obj->copy_to_train(NULL);
            }
          }
        }
      }
    }
  }
}


void Scavenge::skip_train_scavenge_oop_with_check(oop* p) {
  if (should_scavenge(*p)) {
    oop obj = *p;
    if (obj->is_forwarded()) {
      *p = obj->forwardee();
    } else {        
      CarTableDesc* to_desc = CarTable::desc_for((oop*) obj);
      if (to_desc->is_new()) {
        *p = obj->copy_to_survivor_space(NULL);
      } else {
        assert(to_desc->is_train(), "invalid ref");
      }
    }
  }
}


void Scavenge::scavenge_tenured_oop(oop* p) {
  assert(oop(p)->is_old_or_perm(), "old ref expected");
  oop obj = *p;
  debug_only(oop prev_obj = obj);
  assert(should_scavenge(obj), "invalid ref");
  if (obj->is_forwarded()) {
    obj = obj->forwardee();
  } else {
    obj = obj->copy_to_survivor_space(p);
  }
  *p = obj;
  RememberedSet::record_gc_store(p, obj);
}


void Scavenge::train_scavenge_tenured_oop(oop* p) {
  assert(oop(p)->is_old_or_perm(), "old ref expected");
  oop obj = *p;
  debug_only(oop prev_obj = obj);
  assert(should_scavenge(obj), "invalid ref");
  CarTableDesc* to_desc = CarTable::desc_for((oop*) obj);
  if (obj->is_forwarded()) {
    obj = obj->forwardee();
  } else {
    if (to_desc->is_new()) {
      obj = obj->copy_to_survivor_space(p);
    } else {
      assert(to_desc->is_train(), "invalid ref");
      CarTableDesc* first_desc = Universe::train_gen()->first_car_desc();
      julong to_desc_train_number = to_desc->train_number();
      if (to_desc_train_number <= first_desc->train_number()) {
        if (to_desc_train_number == CarTableDesc::special_nonoop_train_number) {
          ((CarSpace*) to_desc->space())->set_marked(true);
          RememberedSet::record_train_gc_store(p, obj);
          return;
        } else {
          assert(to_desc_train_number == first_desc->train_number(), "invalid train number");
          _reference_to_first_train_found = true;
          if (first_desc->car_number() == to_desc->car_number()) {
            obj = obj->copy_to_train(p);
          } else {
            RememberedSet::record_train_gc_store(p, obj);
            return;
          }
        }
      }
    }
  }
  *p = obj;
  RememberedSet::record_train_gc_store(p, obj);
}


void Scavenge::skip_train_scavenge_tenured_oop(oop* p) {
  assert(oop(p)->is_old_or_perm(), "old ref expected");
  oop obj = *p;
  debug_only(oop prev_obj = obj);
  assert(should_scavenge(obj), "invalid ref");
  CarTableDesc* to_desc = CarTable::desc_for((oop*) obj);
  if (obj->is_forwarded()) {
    obj = obj->forwardee();
    *p = obj;
  } else {
    if (to_desc->is_new()) {
      obj = obj->copy_to_survivor_space(p);
      *p = obj;
  } else {
      assert(to_desc->is_train(), "invalid ref");
    }
  }
  RememberedSet::record_train_gc_store(p, obj);
}


bool Scavenge::scavenge_is_alive(oop p) {
  if (!should_scavenge(p) || p->is_forwarded()) return true;
  return !p->is_new();
}


void Scavenge::scavenge_keep_alive(oop* p) {
  if (should_scavenge(*p)) {
    // Note: This function is used for processing soft/weak/final/phantom references
    // and also JNI weak references and monitor cache weak references. The last two
    // are weak roots outside the heap, whereas the first four are weak roots inside
    // the heap. If we have a weak heap root in an older generation we have to call
    // scavenge_tenured (since that will so the store check), and for roots outside
    // the heap we have to call scavenge_nontenured (since doing a store check would
    // be invalid).
    if (oop(p)->is_old_or_perm()) { // do not change to is_new with cases reverted!
      _scavenge_tenured(p);
    } else {
      _scavenge_nontenured(p);
    }
  }
}


bool Scavenge::train_scavenge_is_alive(oop p) {
  // Note: cannot merely call should_scavenge here, since should_scavenge will return true
  // for all cars in the first train. We want to know whether p is in eden space, from space
  // or in the lowest car.
  if (!should_scavenge(p) || p->is_forwarded()) return true;
  // p is in eden/from or in first train
  CarTableDesc* p_desc  = CarTable::desc_for((oop*) p);
  if (p_desc->is_new()) return false;
  CarTableDesc* first_desc = Universe::train_gen()->first_car_desc();
  return !p_desc->equals(first_desc);
}


bool Scavenge::entire_train_scavenge_is_alive(oop p) {
  if (!should_scavenge(p) || p->is_forwarded()) return true;
  CarTableDesc* p_desc  = CarTable::desc_for((oop*) p);
  if (p_desc->is_new()) return false;
  CarTableDesc* first_desc = Universe::train_gen()->first_car_desc();
  return first_desc->train_number() != p_desc->train_number();
}


void Scavenge::compute_scavenge_functions() {
  // Compute the scavenge functions used in different phases of scavenge
  if (UseTrainGC) {
    if (do_train_tick()) {
      _scavenge_nontenured            = &train_scavenge_oop;
      _scavenge_nontenured_with_check = &train_scavenge_oop_with_check;
      _scavenge_tenured               = &train_scavenge_tenured_oop;
    } else {
      _scavenge_nontenured            = &skip_train_scavenge_oop;
      _scavenge_nontenured_with_check = &skip_train_scavenge_oop_with_check;
      _scavenge_tenured               = &skip_train_scavenge_tenured_oop;
    }
  } else {
    _scavenge_nontenured            = &scavenge_oop;
    _scavenge_nontenured_with_check = &scavenge_oop_with_check;
    _scavenge_tenured               = &scavenge_tenured_oop;
  }
}


#ifndef PRODUCT

static int _starting_length = 1;
static int _process_count   = 0;

void Scavenge::print_train_processing_statictics(julong prev_first_number) {
  Train* t = Universe::train_gen()->first_train();
  if (t->train_number() != prev_first_number) {
    tty->cr();
    double passes = (double) _process_count / _starting_length;
    tty->print("Train ");
    tty->print_julong(prev_first_number);
    tty->print_cr(" processed in %d/%d=%lf passes", _process_count, _starting_length, passes);
    _process_count = 0;
    _starting_length = t->length();
    assert(_starting_length > 0, "just checking");
    while (t != NULL) {
      tty->print_cr("- Train ");
      tty->print_julong(t->train_number());
      tty->print_cr(" length %d", t->length());
      t = t->next_train();
    }
  }
}

#endif // PRODUCT


void Scavenge::invoke(int size_to_be_allocated, TRAPS) {
  if (!GC_locker::is_active()) {
    VM_Scavenge op(size_to_be_allocated);
    VMThread::execute(&op);
  }
}


oop* Scavenge::invoke_and_allocate(int size_to_be_allocated, bool is_large_typearray, TRAPS) {
  if (!Universe::jvmpi_alloc_event_enabled() && !last_scavenge_near_limit()) {
    invoke(size_to_be_allocated, CHECK_0);
  }
  oop* obj = Universe::new_gen()->eden()->allocate(size_to_be_allocated);
  if (obj == NULL) {
    obj = Universe::new_gen()->from()->allocate(size_to_be_allocated);
    if (obj == NULL) {
      if (is_large_typearray) {
        obj = Universe::old_gen()->allocate_large_typearray(size_to_be_allocated, CHECK_0);
      } else {
        obj = Universe::old_gen()->allocate(size_to_be_allocated, CHECK_0);
      }
    }
  }
  return obj;
}


void Scavenge::invoke_at_safepoint(int size_to_be_allocated, bool deferred, bool& notify_ref_lock) {
  assert(SafepointSynchronize::is_at_safepoint(), "should be at safepoint");
  assert(Thread::current() == VMThread::vm_thread(), "should be in vm thread");
  guarantee(!is_active(), "scavenge is not reentrant");
  guarantee(!MarkSweep::is_active(), "scavenge cannot occur during mark-sweep");

  _deferred_pending = false;
  if (GC_locker::is_active()) return;

  // Ensure eden top is correct (can be set too high by compiled code)
  Universe::new_gen()->eden()->ensure_proper_top();

  // Check whether we should revert to full mark-sweep on this scavenge
  if (_full_gc_revert_count != -1) {
    // Make sure that full gc was not invoked from elsewhere in the meantime
    if (_full_gc_revert_count == MarkSweep::invoke_count()) {
      MarkSweep::invoke_at_safepoint(size_to_be_allocated, deferred, false, notify_ref_lock, _old_generation_expanded_on_last_scavenge);
      _full_gc_revert_count = -1; // Reset count
      return;
    }
    _full_gc_revert_count = -1;   // Reset count
  }

  // Check whether scavenge is guaranteed to succeed, otherwise revert to full mark-sweep also
  // Note that we require contiguous space to be available to avoid any fragmentation problems (train gc only)
  size_t worst_case_evacuation = Universe::new_gen()->used();
  NOT_CORE(if (UseTrainGC) worst_case_evacuation += Universe::train_gen()->first_train()->first_car()->used();)
  size_t contiguous_available = Universe::old_gen()->contiguous_available();

  // When using extremely large edens, we effectively lose a large amount of old space.
  // Use the "MaxLiveObjectEvacuationRatio" flag to reduce the minimum evacuation space
  // requirements. If there is not enough space to evacuate eden during a scavenge, the
  // VM will immediately exit with an out of memory error.
  assert(MaxLiveObjectEvacuationRatio == 100 || !UseTrainGC,"Cannot use train gc with MaxLiveObjectEvacuationRatio");
  worst_case_evacuation = (size_t)((double)worst_case_evacuation * (double)MaxLiveObjectEvacuationRatio / 100.0); 

  if (contiguous_available < worst_case_evacuation) {
    // Not enough old generation capacity to ensure scavenge can complete, bail out
    MarkSweep::invoke_at_safepoint(size_to_be_allocated, deferred, false, notify_ref_lock, _old_generation_too_full_to_scavenge);
    size_t new_contiguous_available = Universe::old_gen()->contiguous_available();
    if (new_contiguous_available < worst_case_evacuation) {
      set_last_scavenge_near_limit(true);
    }
    return;
  }

  FlagSetting fl(_is_active, true);
  EventMark m("scavenging");

  // JVMPI notification
  if (jvmpi::is_event_enabled(JVMPI_EVENT_GC_START)) {
    jvmpi::post_gc_start_event();
  }

  // Timing
  TraceTime t("GC", PrintScavenge);
  if (TraceScavengeTime) accumulated_time()->start();

  _invoke_count++;
  NOT_PRODUCT(if (_invoke_count == ScavengeBreakAt) BREAKPOINT;)

  size_t prev_old_capacity  = Universe::old_gen()->capacity();
  bool release_lowest_train = false;

  if(UseTLE) {
    Universe::fill_all_tles();
  }  
  
  // Printing
  size_t prev_used, prev_eden_used, prev_from_used, prev_old_used;
  if (PrintScavenge || PrintHeapUsageOverTime) {
    prev_used = Universe::mem_used();
    if (PrintScavenge) {
      prev_eden_used = Universe::new_gen()->eden()->used();
      prev_from_used = Universe::new_gen()->from()->used();
      prev_old_used  = Universe::old_gen()->used();
      if (Verbose) tty->print(" %sinvoke=%d size=%d threshold=%d", deferred ? "deferred " : "", _invoke_count, size_to_be_allocated*wordSize, tenuring_threshold());
    }
  }
  if (PrintRemsetAtScavenge) RememberedSet::print();    
  {
    ResourceMark rm;
    HandleMark hm;

    // Make sure offset arrays are up to date (must be done before verify)
    Universe::update_offset_tables();

    // Verify 
    if (VerifyBeforeScavenge && _invoke_count >= VerifyScavengeStartAt) Universe::verify(true);
    NOT_PRODUCT(instanceRefKlass::verify_no_references_recorded());

    // Call allocation profiler
    AllocationProfiler::iterate_since_last_gc();

    // Getting ready for scavenge
    if (deferred) instanceRefKlass::disable_refs_discovery();  // temporary functionality
    bool do_tick = do_train_tick();
    compute_scavenge_functions();
    age_table()->clear();
    Universe::new_gen()->to()->clear();
    COMPILER2_ONLY(DerivedPointerTable::clear();)
    Universe::new_gen()->set_saved_marks_to_top();
    Universe::old_gen()->set_saved_marks_to_top();
    Universe::perm_gen()->set_saved_marks_to_top();
    if (UseTrainGC) {
      if (do_tick) {
        NOT_PRODUCT(_process_count++;)
        _reference_to_first_train_found = false;
        _train_invoke_count++;
        // Ensure that at least two trains exist
        if (Universe::train_gen()->first_train() == Universe::train_gen()->last_train()) {
          Universe::train_gen()->add_new_highest_train(CarSpace::car_size_in_oops());
        }
      }
      Universe::train_gen()->first_train()->set_should_scavenge(true);
      Universe::train_gen()->tail_trains_set_should_scavenge(true);
      Universe::train_gen()->special_nonoop_train_set_should_scavenge(true);
    }

    // Scavenge remembered set roots
    scavenge = _scavenge_tenured;
    if (UseTrainGC) {
      Universe::train_gen()->scavenge_recorded_stores(!do_tick);
    } else {
      Universe::tenured_gen()->scavenge_recorded_stores();
    }
    Universe::perm_gen()->scavenge_recorded_stores();

    // Scavenge lowest train remembered set roots
    if (do_tick) {
      Universe::train_gen()->tail_trains_set_should_scavenge(false);
      Universe::train_gen()->first_train()->first_car()->remembered_set()->scavenge_higher_train_recorded_stores();
      Universe::train_gen()->tail_trains_set_should_scavenge(true);
    }

    // Complete evacuation of followers
    evacuate_followers();

    // Scavenge all basic roots
    scavenge = _scavenge_nontenured_with_check;
    Universe::oops_do(scavenge);
    SystemDictionary::young_oops_do(scavenge);
    JNIHandles::oops_do(scavenge);   // Global (strong) JNI handles
    Threads::oops_do(scavenge);
    ObjectSynchronizer::oops_do(scavenge);
    FlatProfiler::oops_do(scavenge);
    // Complete evacuation of followers again
    evacuate_followers();
    
    if (do_tick) {
      scavenge = _scavenge_tenured;
      NOT_COMPILER2(ReferencePolicy *soft_ref_policy = new LRUCurrentHeapPolicy();)
      COMPILER2_ONLY(ReferencePolicy *soft_ref_policy = new LRUMaxHeapPolicy();)
      Train* first_train = Universe::train_gen()->first_train();
      // Check whether the remaining cars in this train has any references into them.
      // This can be done by simply looking through their remembered sets and checking
      // the card locations.
      if (!_reference_to_first_train_found && !first_train->tail_has_other_train_references()) {
        // No references to entire lowest train, we may be able to reclaim entire train
        // Now process reference objects discovered during scavenge
        instanceRefKlass::process_discovered_references(soft_ref_policy, &entire_train_scavenge_is_alive,
                                                        &scavenge_keep_alive, &evacuate_followers);
        if (!_reference_to_first_train_found) {
          // Still no references to entire lowest train, we can reclaim it
          release_lowest_train = true;
        } else {
          // Processing references encountered reference to lowest train, cannot reclaim it after all
          Universe::train_gen()->first_train()->first_car()->remembered_set()->scavenge_same_train_recorded_stores();
          // Evacuate followers from the above operation
          evacuate_followers();
        }
      } else {
        // We have references into lowest train, evacuate objects reachable from higher cars in lowest train
        Universe::train_gen()->first_train()->first_car()->remembered_set()->scavenge_same_train_recorded_stores();
        // Evacuate followers from the above operation
        evacuate_followers();
        // Process reference objects discovered during scavenge
        instanceRefKlass::process_discovered_references(soft_ref_policy, &train_scavenge_is_alive, 
                                                        &scavenge_keep_alive, &evacuate_followers);
      }
    } else {
      NOT_COMPILER2(ReferencePolicy *soft_ref_policy = new LRUCurrentHeapPolicy();)
      COMPILER2_ONLY(ReferencePolicy *soft_ref_policy = new LRUMaxHeapPolicy();)
      // Process reference objects discovered during scavenge
      instanceRefKlass::process_discovered_references(soft_ref_policy, &scavenge_is_alive,
                                                      &scavenge_keep_alive, &evacuate_followers);
    }

    // Special large object handling
    if (UseTrainGC) Universe::train_gen()->scavenge_special_nonoop_train_recorded_stores(do_tick, release_lowest_train);

    // Enqueue reference objects discovered during scavenge.
    notify_ref_lock = instanceRefKlass::enqueue_discovered_references();

    // Swap the survivor spaces
    Universe::new_gen()->eden()->clear();
    Universe::new_gen()->from()->clear();
    Universe::new_gen()->swap_spaces();
    assert(Universe::new_gen()->to()->is_empty(), "to space should be empty now");

    // Release freed train area
    if (do_tick) {
      julong prev_first = Universe::train_gen()->first_train()->train_number();
      if (release_lowest_train) {
        if (PrintScavenge && Verbose) {
          Universe::train_gen()->first_train()->print_short();
        }
        add_total_processed(Universe::train_gen()->release_lowest_train());
      } else {
        if (PrintScavenge && Verbose) {
          Universe::train_gen()->first_train()->first_car()->print_short();
        }
        add_total_processed(Universe::train_gen()->release_lowest_car());
      }
      Universe::train_gen()->release_special_nonoop_train_contents();
      if (PrintTrainGCProcessingStats) print_train_processing_statictics(prev_first);
    }
    // Set the desired survivor size to half the real survivor space
    _tenuring_threshold = age_table()->compute_tenuring_threshold(Universe::new_gen()->to()->capacity()/oopSize);

    // Update data structures after scavenge
    if (do_tick) compute_tick_adjustment();
    CarTable::update_after_scavenge();
    COMPILER2_ONLY(DerivedPointerTable::update_pointers();)

    // Timing
    if (TraceScavengeTime) accumulated_time()->stop();

    // Verify again
    if (VerifyAfterScavenge && _invoke_count >= VerifyScavengeStartAt) Universe::verify(false);
    NOT_PRODUCT(instanceRefKlass::verify_no_references_recorded());
    
    // Printing
    if (PrintScavenge) {
      tty->print(" %uK->%uK(%uK)", prev_used / K, Universe::mem_used() / K, Universe::mem_capacity() / K);
      if (Verbose) {
        int from_used = Universe::new_gen()->from()->used();
        int diff_old_used  = Universe::old_gen()->used() - prev_old_used;
        tty->print(" eden %uK->0K survivor %uK->%uK tenured %uK", 
          prev_eden_used / K, prev_from_used / K, from_used / K, diff_old_used / K);
      }
    }
    if (PrintRemsetAtScavenge) RememberedSet::print();
    if (PrintHeapUsageOverTime) Universe::print_heap_usage_stamp(prev_used, do_tick ? "trainscavenge" : "scavenge");
  }

  // Check if we had to expand old generation in order to complete scavenge
  // If so, do full gc on next scavenge and (possibly) shrink old generation again
  if (!UseTrainGC) {
    ptrdiff_t actual_old_capacity_expansion = Universe::old_gen()->capacity() - prev_old_capacity;
    if (actual_old_capacity_expansion > 0) {
      _full_gc_revert_count = MarkSweep::invoke_count();
    }
  }

  if (jvmpi::is_event_enabled(JVMPI_EVENT_GC_FINISH)) {
    jvmpi::post_gc_finish_event(Universe::mem_used(), Universe::mem_capacity());
  }
  if (Universe::jvmpi_alloc_event_enabled()) {
    fill_newgen();
  }
  
  Universe::set_heap_capacity_at_last_gc(Universe::mem_capacity());
  Universe::set_heap_free_at_last_gc(Universe::mem_free());
  Universe::set_heap_used_at_last_gc(Universe::mem_used());
}


void Scavenge::fill_newgen() {
  // fill eden and from space with a dummy int[] object
  EdenSpace* eden     = Universe::new_gen()->eden();
  SurvivorSpace* from = Universe::new_gen()->from();
  SurvivorSpace* to   = Universe::new_gen()->to();
  assert(eden->is_empty() && from->is_empty() && to->is_empty(), "newgen should be empty");
  eden->allocate_temporary_filler(0);
  from->allocate_temporary_filler(0);
  assert(eden->free() == 0 && from->free() == 0, "newgen should be full");
}


void Scavenge::evacuate_followers() {
  // Scavenge evacuated contents in to_space, old_gen and perm_gen until done.
  do {
    scavenge = _scavenge_nontenured;
    Universe::new_gen()->scavenge_contents_from_saved_marks();

    scavenge = _scavenge_tenured;
    Universe::old_gen()->scavenge_contents_from_saved_marks();
    Universe::perm_gen()->scavenge_contents_from_saved_marks();

  } while (!(Universe::new_gen()->top_at_saved_marks() 
             && Universe::old_gen()->top_at_saved_marks() 
             && Universe::perm_gen()->top_at_saved_marks()));  
}


void scavenge_init() {
  Scavenge::reset_tick_adjustment();
  Scavenge::_tenuring_threshold = MaxTenuringThreshold; // Set after argument parsing
}

