/*
 * @(#)universe.hpp	1.126 01/02/09 14:45:19
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Universe is a name space holding known system classes and objects in the VM.
// 
// Loaded classes are accessible through the SystemDictionary.
// 
// The object heap is allocated and accessed through Universe, and various allocation
// support is provided. Allocation by the interpreter and compiled code is done inline
// and bails out to Scavenge::invoke_and_allocate.


class Universe: AllStatic {
  friend class MarkSweep;
  friend class oopDesc;
  friend class ClassLoader;
  friend class Arguments;
  friend class SystemDictionary;
  friend class VMStructs;
  friend void  universe_init();
  friend void  universe2_init();
  friend void  universe_post_init();

 private:
  // Known classes in the VM
  static klassOop _boolArrayKlassObj;
  static klassOop _byteArrayKlassObj;
  static klassOop _charArrayKlassObj;
  static klassOop _intArrayKlassObj;
  static klassOop _shortArrayKlassObj;
  static klassOop _longArrayKlassObj;
  static klassOop _singleArrayKlassObj;
  static klassOop _doubleArrayKlassObj;

  static klassOop _symbolKlassObj;
  static klassOop _methodKlassObj;
  static klassOop _klassKlassObj;
  static klassOop _arrayKlassKlassObj;
  static klassOop _objArrayKlassKlassObj;
  static klassOop _typeArrayKlassKlassObj;
  static klassOop _instanceKlassKlassObj;
  static klassOop _constantPoolKlassObj;
  static klassOop _constantPoolCacheKlassObj;
#ifndef CORE
  static klassOop _compiledICHolderKlassObj;
#endif
  static klassOop _systemObjArrayKlassObj;

  // Known objects in the VM
  static oop          _main_thread_group;             // Reference to the main thread group object
  static oop          _system_thread_group;           // Reference to the system thread group object

  static typeArrayOop _the_empty_short_array;         // Canonicalized short array
  static typeArrayOop _the_empty_int_array;           // Canonicalized int array
  static objArrayOop  _the_empty_system_obj_array;    // Canonicalized system obj array
  static objArrayOop  _the_empty_class_klass_array;   // Canonicalized obj array of type java.lang.Class
  static methodOop    _finalizer_register_method;     // static method for registering finalizable objects
  static methodOop    _reflect_invoke_method;         // method for security checks
  static methodOop    _loader_addClass_method;        // method for registering loaded classes in class loader vector
  static oop          _out_of_memory_error_instance;  // preallocated error object
  static oop          _null_ptr_exception_instance;   // preallocated exception object
  static oop          _arithmetic_exception_instance; // preallocated exception object
  static oop          _vm_exception;                  // The object used as an exception dummy when exceptions are
                                                      //  thrown for the  vm thread
  debug_only(static objArrayOop _fullgc_alot_dummy_array;) // array of dummy objects used with +FullGCAlot
  debug_only(static int         _fullgc_alot_dummy_next;)  // index of next entry to clear

  // Object heap sizes
  static size_t  _new_size;
  static size_t  _reserved_new_size;
  static size_t  _old_size;
  static size_t  _reserved_old_size;
  static size_t  _perm_size;
  static size_t  _reserved_perm_size;
  static int  _large_typearray_limit;

  // Object heap
  static NewGeneration*  _new_gen;
  static OldGeneration*  _old_gen;
  static PermGeneration* _perm_gen;

  // Compiler/dispatch support
  static int  _base_vtable_size;                      // Java vtbl size of klass Object (in words)

  // Version checking
  static bool _is_jdk12x_version;

  // Initialization
  static bool _bootstrapping;                         // true during genesis
  static bool _fully_initialized;                     // true after universe_init and initialize_vtables called

  // JVMPI support (cached values for performance)
  static bool _jvmpi_alloc_event_enabled;
  static bool _jvmpi_move_event_enabled;
  static bool _jvmpi_jni_global_alloc_event_enabled;
  static bool _jvmpi_jni_global_free_event_enabled;
  static bool _jvmpi_jni_weak_global_alloc_event_enabled;
  static bool _jvmpi_jni_weak_global_free_event_enabled;

  // Historic gc information
  static size_t _heap_capacity_at_last_gc;
  static size_t _heap_used_at_last_gc;
  static size_t _heap_free_at_last_gc;

  static void initialize_heap();
  static void fixup_mirrors(TRAPS);

  static void reinitialize_vtable_of(Klass* k, TRAPS);
  static void reinitialize_itables();
  static void compute_base_vtable_size();             // compute vtable size of class Object

  static void genesis(TRAPS);                         // Create the initial world

  // Debugging
  static int _verify_count;                           // number of verifies done

  debug_only(static void check_for_valid_allocation_state(Thread* t);)
  static void compute_verify_oop_data();

  // Allocation support
  static void post_allocation_setup      (KlassHandle klass, oop* obj, int size            );
  static void post_allocation_setup_array(KlassHandle klass, oop* obj, int size, int length);

 public:
  // Known classes in tbe VM
  static klassOop boolArrayKlassObj()                 { return _boolArrayKlassObj;   }
  static klassOop byteArrayKlassObj()                 { return _byteArrayKlassObj;   }
  static klassOop charArrayKlassObj()                 { return _charArrayKlassObj;   }
  static klassOop intArrayKlassObj()                  { return _intArrayKlassObj;    }
  static klassOop shortArrayKlassObj()                { return _shortArrayKlassObj;  }
  static klassOop longArrayKlassObj()                 { return _longArrayKlassObj;   }
  static klassOop singleArrayKlassObj()               { return _singleArrayKlassObj; }
  static klassOop doubleArrayKlassObj()               { return _doubleArrayKlassObj; }

  static klassOop symbolKlassObj()                    { return _symbolKlassObj;            }
  static klassOop methodKlassObj()                    { return _methodKlassObj;            }
  static klassOop klassKlassObj()                     { return _klassKlassObj;             }
  static klassOop arrayKlassKlassObj()                { return _arrayKlassKlassObj;        }
  static klassOop objArrayKlassKlassObj()             { return _objArrayKlassKlassObj;     }
  static klassOop typeArrayKlassKlassObj()            { return _typeArrayKlassKlassObj;    }
  static klassOop instanceKlassKlassObj()             { return _instanceKlassKlassObj;     }
  static klassOop constantPoolKlassObj()              { return _constantPoolKlassObj;      }
  static klassOop constantPoolCacheKlassObj()         { return _constantPoolCacheKlassObj; }
#ifndef CORE
  static klassOop compiledICHolderKlassObj()          { return _compiledICHolderKlassObj;  }
#endif
  static klassOop systemObjArrayKlassObj()            { return _systemObjArrayKlassObj;    }

  // Known objects in tbe VM
  static oop      main_thread_group()                 { return _main_thread_group; }
  static void set_main_thread_group(oop group)        { _main_thread_group = group;}

  static oop      system_thread_group()               { return _system_thread_group; }
  static void set_system_thread_group(oop group)      { _system_thread_group = group;}

  static typeArrayOop the_empty_short_array()         { return _the_empty_short_array;         }
  static typeArrayOop the_empty_int_array()           { return _the_empty_int_array;           }
  static objArrayOop  the_empty_system_obj_array ()   { return _the_empty_system_obj_array;    }  
  static objArrayOop  the_empty_class_klass_array ()  { return _the_empty_class_klass_array;   }  
  static methodOop    finalizer_register_method()     { return _finalizer_register_method;     }  
  static methodOop    reflect_invoke_method()         { return _reflect_invoke_method;         }
  static methodOop    loader_addClass_method()        { return _loader_addClass_method;        }
  static oop          out_of_memory_error_instance()  { return _out_of_memory_error_instance;  }
  static oop          null_ptr_exception_instance()   { return _null_ptr_exception_instance;   }
  static oop          arithmetic_exception_instance() { return _arithmetic_exception_instance; }
  static oop          vm_exception()                  { return _vm_exception; }

  // Accessors needed for fast allocation
  static klassOop* boolArrayKlassObj_addr()           { return &_boolArrayKlassObj;   }
  static klassOop* byteArrayKlassObj_addr()           { return &_byteArrayKlassObj;   }
  static klassOop* charArrayKlassObj_addr()           { return &_charArrayKlassObj;   }
  static klassOop* intArrayKlassObj_addr()            { return &_intArrayKlassObj;    }
  static klassOop* shortArrayKlassObj_addr()          { return &_shortArrayKlassObj;  }
  static klassOop* longArrayKlassObj_addr()           { return &_longArrayKlassObj;   }
  static klassOop* singleArrayKlassObj_addr()         { return &_singleArrayKlassObj; }
  static klassOop* doubleArrayKlassObj_addr()         { return &_doubleArrayKlassObj; }

  static void Universe::fill_all_tles();              // Make Eden look contiguous

  // Object heap access
  static NewGeneration*  new_gen()                    { return _new_gen; }
  static OldGeneration*  old_gen()                    { return _old_gen; }
  static PermGeneration* perm_gen()                   { return _perm_gen; }

  static TrainGeneration* train_gen() {
    assert(UseTrainGC, "Should only be called when train GC is used");
    return (TrainGeneration*) _old_gen;
  }

  static TenuredGeneration* tenured_gen() {
    assert(!UseTrainGC, "Should not be called when train GC is used");
    return (TenuredGeneration*) _old_gen;
  }

  // Object heap sizes
  static size_t new_size()                               { return _new_size;            }
  static size_t reserved_new_size()                      { return _reserved_new_size;   }
  static size_t old_size()                               { return _old_size;            }
  static size_t reserved_old_size()                      { return _reserved_old_size;   }
  static size_t perm_size()                              { return _perm_size;           }
  static size_t reserved_perm_size()                     { return _reserved_perm_size;  }
  static int large_typearray_limit()                  { return _large_typearray_limit;}

  // Object heap size calculations
  static size_t mem_capacity();                          // returns size in bytes, not oops
  static size_t mem_free();                              // returns size in bytes, not oops
  static size_t mem_used();                              // returns size in bytes, not oops

  static size_t get_heap_capacity_at_last_gc()         { return _heap_capacity_at_last_gc; }
  static size_t get_heap_free_at_last_gc()             { return _heap_free_at_last_gc; }
  static size_t get_heap_used_at_last_gc()             { return _heap_used_at_last_gc; }
  static void set_heap_capacity_at_last_gc(size_t val) { assert(val > 0, "Sanity"); _heap_capacity_at_last_gc = val; }
  static void set_heap_free_at_last_gc(size_t val)     { _heap_free_at_last_gc = val; }
  static void set_heap_used_at_last_gc(size_t val)     { /* Can't assert due to init */ _heap_used_at_last_gc = val; }

  // JVMPI support
  static bool jvmpi_alloc_event_enabled()             { return _jvmpi_alloc_event_enabled; }
  static void set_jvmpi_alloc_event_enabled(bool b)   { _jvmpi_alloc_event_enabled = b;    }
  static bool jvmpi_move_event_enabled()              { return _jvmpi_move_event_enabled; }
  static void set_jvmpi_move_event_enabled(bool b)    { _jvmpi_move_event_enabled = b;    }

  static bool jvmpi_jni_global_alloc_event_enabled()                 { return _jvmpi_jni_global_alloc_event_enabled; }
  static void set_jvmpi_jni_global_alloc_event_enabled(bool b)       { _jvmpi_jni_global_alloc_event_enabled = b;    }
  static bool jvmpi_jni_global_free_event_enabled()                  { return _jvmpi_jni_global_free_event_enabled; }
  static void set_jvmpi_jni_global_free_event_enabled(bool b)        { _jvmpi_jni_global_free_event_enabled = b;    }

  static bool jvmpi_jni_weak_global_alloc_event_enabled()            { return _jvmpi_jni_weak_global_alloc_event_enabled; }
  static void set_jvmpi_jni_weak_global_alloc_event_enabled(bool b)  { _jvmpi_jni_weak_global_alloc_event_enabled = b;    }
  static bool jvmpi_jni_weak_global_free_event_enabled()             { return _jvmpi_jni_weak_global_free_event_enabled; }
  static void set_jvmpi_jni_weak_global_free_event_enabled(bool b)   { _jvmpi_jni_weak_global_free_event_enabled = b;    }

  // Normal allocation (uninitialized)  
  static oop* allocate      (KlassHandle klass, int size,             TRAPS);
  static oop* allocate_array(KlassHandle klass, int size, int length, TRAPS);
  
  // Normal allocation in shared Eden (initialized)  
  static oop* clear_allocate_shared      (KlassHandle klass, int size, TRAPS);
  static oop* clear_allocate_array_shared(KlassHandle klass, int size, int length, TRAPS);
  // Normal allocation in Thread Local Eden (initialized), for oop or array
  static oop* clear_allocate_local(KlassHandle klass, int size, int length, bool is_array, TRAPS);
 
  // Normal allocation (initialized)
  static oop* clear_allocate      (KlassHandle klass, int size,             TRAPS);
  static oop* clear_allocate_array(KlassHandle klass, int size, int length, TRAPS);
 
  // Tenured allocation (uninitialized)  
  static oop* allocate_tenured(KlassHandle klass, int size, TRAPS);

  // Tenured allocation (initialized)  
  static oop* clear_allocate_tenured(KlassHandle klass, int size, TRAPS);
  
  // Tenured allocation of large type arrays (uninitialized)
  static oop* allocate_large_typearray(KlassHandle klass, int size, int length, TRAPS);

  // Tenured allocation of large type arrays (initialized)
  static oop* clear_allocate_large_typearray(KlassHandle klass, int size, int length, TRAPS);
  
  // Permanent allocation (uninitialized)  
  static oop* allocate_permanent      (KlassHandle klass, int size,             TRAPS);
  static oop* allocate_permanent_array(KlassHandle klass, int size, int length, TRAPS);
   
  // Permanent allocation (initialized)  
  static oop* clear_allocate_permanent      (KlassHandle klass, int size,             TRAPS);
  static oop* clear_allocate_permanent_array(KlassHandle klass, int size, int length, TRAPS);
  
  // Testers
  static bool is_bootstrapping()                      { return _bootstrapping; }
  static bool is_fully_initialized()                  { return _fully_initialized; }
  static bool is_jdk12x_version()                     { return _is_jdk12x_version; }

  // Test whether p points into a valid heap area (use for asserts only)
#ifndef PRODUCT
  static inline bool is_in_heap(oop p);
  static inline bool is_heap(oop p);
  static inline bool is_heap_or_null(oop p);
#endif

  // Test whether p points into the reserved heap area (fast)
  static inline bool is_reserved_heap(oop p);

  static inline bool element_type_should_be_aligned(BasicType type);
  static inline bool field_type_should_be_aligned(BasicType type);
  static bool        on_page_boundary(void* addr);
  static void check_alignment(size_t size, size_t alignment, const char* name);

  // GC support
  static void run_finalizers_on_exit();
  static void set_generation_saved_marks_to_top();
  static void compute_new_generation_sizes();

  // Object start calculation - very slow
  static oop slow_object_start(oop* p)     PRODUCT_RETURN0;

  // Fast object start calculation, works only for old objects
  static oop fast_object_start(oop* p)     PRODUCT_RETURN0;

  // Update offset arrays (needed before verify and before scavenge)
  static void update_offset_tables();

  // Iteration
  static void oops_do(void f(oop*));
  static void basic_type_classes_do(void f(klassOop));// iterate over all klasses for basic types (classes not present in SystemDictionary)
  static void system_classes_do(void f(klassOop));    // iterate over all system klasses (classes not present in SystemDictionary)

  // Heap iteration
  static void object_iterate(ObjectClosure* blk);
  static void space_iterate(SpaceClosure* blk);

  // Debugging
  static void verify(bool allow_dirty = true, bool silent = false) PRODUCT_RETURN;
  static int  verify_count()                  { return _verify_count; }
  static void print()                         PRODUCT_RETURN;
  static void print_heap_usage_header()       PRODUCT_RETURN;
  static void print_heap_usage_stamp(size_t prev_mem_used, const char* type) PRODUCT_RETURN;
  static bool no_gc_in_progress()             PRODUCT_RETURN0;
  debug_only(static bool release_fullgc_alot_dummy();)
  static void update_max_heap_capatity()      PRODUCT_RETURN;
  static void print_max_heap_capacity()       PRODUCT_RETURN;

  // The non-oop pattern (see compiledIC.hpp, etc)
  static void*   non_oop_word();

  // Detect if either the high or low bits of x are a non-oop pattern
  // is_non_oop is true of the non_oop_word, and of any half-and-half
  // combination of a real oop and the non_oop_word that may arise
  // due to race conditions between patching of oop-bearing instructions,
  // and processors reading those instructions.  Thus, the non_oop_word
  // is a good initial value for an oop in code, if it is eventually
  // to be patched to a real oop.  Then, is_non_oop can detect both
  // the initial state and any transient intermediate states.
  static bool is_non_oop(oop x);

  // Oop verification (see MacroAssembler::verify_oop)
  static jint verify_oop_mask()               PRODUCT_RETURN0;
  static jint verify_oop_bits()               PRODUCT_RETURN0;
  static jint verify_mark_bits()              PRODUCT_RETURN0;
  static jint verify_mark_mask()              PRODUCT_RETURN0;
  static jint verify_klass_mask()             PRODUCT_RETURN0;
  static jint verify_klass_bits()             PRODUCT_RETURN0;

#ifndef CORE
  // Flushing and deoptimization
  static void flush_dependents_on(instanceKlassHandle dependee);
#endif

  // Compiler support
  static int base_vtable_size()               { return _base_vtable_size; }

  // jvmpi support
  static void jvmpi_object_alloc(oop obj, int bytesize);
  static void jvmpi_object_move(oop from, oop to);
  static int addr_to_arena_id(void *);

  static void print_bci_histogram()           PRODUCT_RETURN;
};
