/*
 * @(#)vmSymbols.hpp	1.71 00/01/28 17:13:25
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// The classes vmSymbols and vmSymbolHandles are a name spaces for fast lookup of 
// symbols commonly used in the VM. The first class return a symbolOop, while the
// second class returns a SymbolHandle. The underlying data structure is shared
// between the two classes.
//
// Sample usage:
//
//   symbolOop obj       = vmSymbols::java_lang_Object()();
//   SymbolHandle handle = vmSymbolHandles::java_lang_Object();


// Mapping function names to values. New entries should be added below.

#define VM_SYMBOLS_DO(template)                                                                   \
  /* commonly used class names */                                                                 \
  template(java_lang_System,                          "java/lang/System")                         \
  template(java_lang_Object,                          "java/lang/Object")                         \
  template(java_lang_Class,                           "java/lang/Class")                          \
  template(java_lang_String,                          "java/lang/String")                         \
  template(java_lang_Thread,                          "java/lang/Thread")                         \
  template(java_lang_ThreadGroup,                     "java/lang/ThreadGroup")                    \
  template(java_lang_Cloneable,                       "java/lang/Cloneable")                      \
  template(java_lang_Throwable,                       "java/lang/Throwable")                      \
  template(java_lang_ClassLoader,                     "java/lang/ClassLoader")                    \
  template(java_lang_ClassLoader_NativeLibrary,       "java/lang/ClassLoader\x024NativeLibrary")  \
  template(java_lang_ThreadDeath,                     "java/lang/ThreadDeath")                    \
  template(java_lang_Boolean,                         "java/lang/Boolean")                        \
  template(java_lang_Character,                       "java/lang/Character")                      \
  template(java_lang_Float,                           "java/lang/Float")                          \
  template(java_lang_Double,                          "java/lang/Double")                         \
  template(java_lang_Byte,                            "java/lang/Byte")                           \
  template(java_lang_Short,                           "java/lang/Short")                          \
  template(java_lang_Integer,                         "java/lang/Integer")                        \
  template(java_lang_Long,                            "java/lang/Long")                           \
  template(java_lang_Shutdown,                        "java/lang/Shutdown")                       \
  template(java_lang_ref_Reference,                   "java/lang/ref/Reference")                  \
  template(java_lang_ref_SoftReference,               "java/lang/ref/SoftReference")              \
  template(java_lang_ref_WeakReference,               "java/lang/ref/WeakReference")              \
  template(java_lang_ref_FinalReference,              "java/lang/ref/FinalReference")             \
  template(java_lang_ref_PhantomReference,            "java/lang/ref/PhantomReference")           \
  template(java_lang_ref_Finalizer,                   "java/lang/ref/Finalizer")                  \
  template(java_lang_reflect_Method,                  "java/lang/reflect/Method")                 \
  template(java_lang_reflect_Constructor,             "java/lang/reflect/Constructor")            \
  template(java_lang_reflect_Field,                   "java/lang/reflect/Field")                  \
  template(java_lang_StringBuffer,                    "java/lang/StringBuffer")                   \
  template(java_security_AccessControlContext,        "java/security/AccessControlContext")       \
  template(java_security_ProtectionDomain,            "java/security/ProtectionDomain")           \
  template(java_io_OutputStream,                      "java/io/OutputStream")                     \
  template(java_io_Reader,                            "java/io/Reader")                           \
  template(java_io_BufferedReader,                    "java/io/BufferedReader")                   \
  template(java_io_FileInputStream,                   "java/io/FileInputStream")                  \
  template(java_io_ByteArrayInputStream,              "java/io/ByteArrayInputStream")             \
  template(java_io_Serializable,                      "java/io/Serializable")                     \
  template(java_util_Properties,                      "java/util/Properties")                     \
  template(java_util_Vector,                          "java/util/Vector")                         \
  template(java_util_AbstractList,                    "java/util/AbstractList")                   \
  template(java_util_Hashtable,                       "java/util/Hashtable")                      \
  template(java_lang_Compiler,                        "java/lang/Compiler")                       \
  template(sun_misc_Signal,                           "sun/misc/Signal")                          \
  /* class file format tags */                                                                    \
  template(tag_source_file,                           "SourceFile")                               \
  template(tag_inner_classes,                         "InnerClasses")                             \
  template(tag_constant_value,                        "ConstantValue")                            \
  template(tag_code,                                  "Code")                                     \
  template(tag_exceptions,                            "Exceptions")                               \
  template(tag_line_number_table,                     "LineNumberTable")                          \
  template(tag_local_variable_table,                  "LocalVariableTable")                       \
  template(tag_synthetic,                             "Synthetic")                                \
  /* exception klasses: at least all exceptions thrown by the VM have entries here */             \
  template(java_lang_ArithmeticException,             "java/lang/ArithmeticException")            \
  template(java_lang_ArrayIndexOutOfBoundsException,  "java/lang/ArrayIndexOutOfBoundsException") \
  template(java_lang_ArrayStoreException,             "java/lang/ArrayStoreException")            \
  template(java_lang_ClassCastException,              "java/lang/ClassCastException")             \
  template(java_lang_ClassNotFoundException,          "java/lang/ClassNotFoundException")         \
  template(java_lang_CloneNotSupportedException,      "java/lang/CloneNotSupportedException")     \
  template(java_lang_IllegalAccessException,          "java/lang/IllegalAccessException")         \
  template(java_lang_IllegalArgumentException,        "java/lang/IllegalArgumentException")       \
  template(java_lang_IllegalMonitorStateException,    "java/lang/IllegalMonitorStateException")   \
  template(java_lang_IllegalThreadStateException,     "java/lang/IllegalThreadStateException")    \
  template(java_lang_IndexOutOfBoundsException,       "java/lang/IndexOutOfBoundsException")      \
  template(java_lang_InstantiationException,          "java/lang/InstantiationException")         \
  template(java_lang_InstantiationError,              "java/lang/InstantiationError")             \
  template(java_lang_InterruptedException,            "java/lang/InterruptedException")           \
  template(java_lang_LinkageError,                    "java/lang/LinkageError")                   \
  template(java_lang_NegativeArraySizeException,      "java/lang/NegativeArraySizeException")     \
  template(java_lang_NoSuchFieldException,            "java/lang/NoSuchFieldException")           \
  template(java_lang_NoSuchMethodException,           "java/lang/NoSuchMethodException")          \
  template(java_lang_NullPointerException,            "java/lang/NullPointerException")           \
  template(java_lang_StringIndexOutOfBoundsException, "java/lang/StringIndexOutOfBoundsException")\
  template(java_lang_InvalidClassException,           "java/lang/InvalidClassException")          \
  template(java_lang_reflect_InvocationTargetException, "java/lang/reflect/InvocationTargetException") \
  template(java_lang_Exception,                       "java/lang/Exception")                      \
  template(java_lang_RuntimeException,                "java/lang/RuntimeException")               \
  template(java_io_IOException,                       "java/io/IOException")                      \
  template(java_security_PrivilegedActionException,   "java/security/PrivilegedActionException")  \
  /* error klasses: at least all error thrown by the VM have entries here */                      \
  template(java_lang_AbstractMethodError,             "java/lang/AbstractMethodError")            \
  template(java_lang_ClassCircularityError,           "java/lang/ClassCircularityError")          \
  template(java_lang_ClassFormatError,                "java/lang/ClassFormatError")               \
  template(java_lang_UnsupportedClassVersionError,    "java/lang/UnsupportedClassVersionError")   \
  template(java_lang_Error,                           "java/lang/Error")                          \
  template(java_lang_ExceptionInInitializerError,     "java/lang/ExceptionInInitializerError")    \
  template(java_lang_IllegalAccessError,              "java/lang/IllegalAccessError")             \
  template(java_lang_IncompatibleClassChangeError,    "java/lang/IncompatibleClassChangeError")   \
  template(java_lang_InternalError,                   "java/lang/InternalError")                  \
  template(java_lang_NoClassDefFoundError,            "java/lang/NoClassDefFoundError")           \
  template(java_lang_NoSuchFieldError,                "java/lang/NoSuchFieldError")               \
  template(java_lang_NoSuchMethodError,               "java/lang/NoSuchMethodError")              \
  template(java_lang_OutOfMemoryError,                "java/lang/OutOfMemoryError")               \
  template(java_lang_UnsatisfiedLinkError,            "java/lang/UnsatisfiedLinkError")           \
  template(java_lang_VerifyError,                     "java/lang/VerifyError")                    \
  template(java_lang_StackOverflowError,              "java/lang/StackOverflowError")             \
  /* handling native inlining */                                                                  \
  template(java_lang_Math,                            "java/lang/Math")                           \
  template(java_lang_Math_sin_Name,                   "sin")                                      \
  template(java_lang_Math_sin_Signature,              "(D)D")                                     \
  template(java_lang_Math_cos_Name,                   "cos")                                      \
  template(java_lang_Math_cos_Signature,              "(D)D")                                     \
  template(java_lang_Math_tan_Name,                   "tan")                                      \
  template(java_lang_Math_tan_Signature,              "(D)D")                                     \
  template(java_lang_Math_atan2_Name,                 "atan2")                                    \
  template(java_lang_Math_atan2_Signature,            "(DD)D")                                   \
  template(java_lang_Math_sqrt_Name,                  "sqrt")                                     \
  template(java_lang_Math_sqrt_Signature,             "(D)D")                                     \
  template(java_lang_System_arraycopy_Name,           "arraycopy")                                \
  template(java_lang_System_arraycopy_Signature,      "(Ljava/lang/Object;ILjava/lang/Object;II)V") \
  template(java_lang_Object_hashCode_Name,            "hashCode")                                 \
  template(java_lang_Object_hashCode_Signature,       "()I")                                      \
  template(java_lang_String_compareTo_Name,           "compareTo")                                 \
  template(java_lang_String_compareTo_Signature,      "(Ljava/lang/String;)I")                    \
  template(java_util_Vector_elementAt_Name,            "elementAt")                               \
  template(java_util_Vector_elementAt_Signature,       "(I)Ljava/lang/Object;")                               \
  /* common method names */                                                                       \
  template(object_initializer_name,                   "<init>")                                   \
  template(class_initializer_name,                    "<clinit>")                                 \
  template(println_name,                              "println")                                  \
  template(printStackTrace_name,                      "printStackTrace")                          \
  template(main_name,                                 "main")                                     \
  template(name_name,                                 "name")                                     \
  template(priority_name,                             "priority")                                 \
  template(stillborn_name,                            "stillborn")                                \
  template(group_name,                                "group")                                    \
  template(daemon_name,                               "daemon")                                   \
  template(eetop_name,                                "eetop")                                    \
  template(run_method_name,                           "run")                                      \
  template(exit_method_name,                          "exit")                                     \
  template(shutdown_method_name,                      "shutdown")                                 \
  template(finalize_method_name,                      "finalize")                                 \
  template(register_method_name,                      "register")                                 \
  template(reference_lock_name,                       "lock")                                     \
  template(run_finalizers_on_exit_name,               "runFinalizersOnExit")                      \
  template(uncaughtException_name,                    "uncaughtException")                        \
  template(initializeSystemClass_name,                "initializeSystemClass")                    \
  template(loadClass_name,                            "loadClass")                                \
  template(loadClassInternal_name,                    "loadClassInternal")                        \
  template(put_name,                                  "put")                                      \
  template(type_name,                                 "type")                                     \
  template(findNative_name,                           "findNative")                               \
  template(deadChild_name,                            "deadChild")                                \
  template(invoke_name,                               "invoke")                                   \
  template(addClass_name,                             "addClass")                                 \
  template(getFromClass_name,                         "getFromClass")                             \
  template(dispatch_name,                             "dispatch")                                 \
  template(getSystemClassLoader_name,                 "getSystemClassLoader")                     \
  template(fillInStackTrace_name,                     "fillInStackTrace")                         \
  template(setProperty_name,                          "setProperty")                              \
  template(context_name,                              "context")                                  \
  template(privilegedContext_name,                    "privilegedContext")                        \
  template(inheritedAccessControlContext_name,        "inheritedAccessControlContext")            \
  template(isPrivileged_name,                         "isPrivileged")                             \
  template(compareTo_name,                            "compareTo")                                \
  template(wait_name,                                 "wait")                                     \
  template(checkPackageAccess_name,                   "checkPackageAccess")                       \
  /* common signatures */                                                                         \
  template(void_method_signature,                     "()V")                                      \
  template(int_void_signature,                        "(I)V")                                     \
  template(byte_signature,                            "B")                                        \
  template(char_signature,                            "C")                                        \
  template(double_signature,                          "D")                                        \
  template(float_signature,                           "F")                                        \
  template(int_signature,                             "I")                                        \
  template(long_signature,                            "J")                                        \
  template(short_signature,                           "S")                                        \
  template(bool_signature,                            "Z")                                        \
  template(void_signature,                            "V")                                        \
  template(char_array_signature,                      "[C")                                       \
  template(register_method_signature,                 "(Ljava/lang/Object;)V")                    \
  template(string_void_signature,                     "(Ljava/lang/String;)V")                    \
  template(string_int_signature,                      "(Ljava/lang/String;)I")                    \
  template(throwable_void_signature,                  "(Ljava/lang/Throwable;)V")                       \
  template(class_void_signature,                      "(Ljava/lang/Class;)V")                           \
  template(throwable_string_void_signature,           "(Ljava/lang/Throwable;Ljava/lang/String;)V")     \
  template(string_array_void_signature,               "([Ljava/lang/String;)V")                         \
  template(string_array_string_array_void_signature,  "([Ljava/lang/String;[Ljava/lang/String;)V")      \
  template(thread_throwable_void_signature,           "(Ljava/lang/Thread;Ljava/lang/Throwable;)V")     \
  template(threadgroup_runnable_void_signature,       "(Ljava/lang/ThreadGroup;Ljava/lang/Runnable;)V") \
  template(threadgroup_string_void_signature,         "(Ljava/lang/ThreadGroup;Ljava/lang/String;)V")   \
  template(string_class_signature,                    "(Ljava/lang/String;)Ljava/lang/Class;")          \
  template(object_object_object_signature,            "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;") \
  template(string_string_string_signature,            "(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;") \
  template(classloader_string_long_signature,         "(Ljava/lang/ClassLoader;Ljava/lang/String;)J")   \
  template(byte_array_void_signature,                 "([B)V")                                          \
  template(char_array_void_signature,                 "([C)V")                                          \
  template(int_int_void_signature,                    "(II)V")                                          \
  template(void_classloader_signature,                "()Ljava/lang/ClassLoader;")                      \
  template(void_object_signature,                     "()Ljava/lang/Object;")                           \
  template(void_class_signature,                      "()Ljava/lang/Class;")                            \
  template(object_array_object_object_signature,      "(Ljava/lang/Object;[Ljava/lang/Object;)Ljava/lang/Object;") \
  template(exception_void_signature,                  "(Ljava/lang/Exception;)V")                \
  template(protectiondomain_signature,                "[Ljava/security/ProtectionDomain;")       \
  template(accesscontrolcontext_signature,            "Ljava/security/AccessControlContext;")    \
  template(class_protectiondomain_signature,          "(Ljava/lang/Class;Ljava/security/ProtectionDomain;)V")    \
  template(threadgroup_signature,                     "Ljava/lang/ThreadGroup;")


// Name macros

#define VM_SYMBOL_NAME_POSTFIX       _enum
#define VM_SYMBOL_ENUM_NAME(name)    name##VM_SYMBOL_NAME_POSTFIX
#define VM_SYMBOL_ENUM(name, string) VM_SYMBOL_ENUM_NAME(name),


// Declaration macros

#define VM_SYMBOL_DECLARE(name, string) \
  static symbolOop name() { return _symbols[VM_SYMBOL_ENUM_NAME(name)]; }

#define VM_SYMBOL_HANDLE_DECLARE(name, string) \
  static symbolHandle name() { return symbolHandle(&vmSymbols::_symbols[vmSymbols::VM_SYMBOL_ENUM_NAME(name)], false); }


// Class vmSymbols

class vmSymbols: AllStatic {
 friend class vmSymbolHandles;
 public:
  // enum for figuring positions and size of array holding symbolOops
  enum {
    VM_SYMBOLS_DO(VM_SYMBOL_ENUM)
    vm_symbols_terminating_enum
  };
 private:
  // The symbol array
  static symbolOop _symbols[];

 public:
  // Initialization
  static void initialize(TRAPS);
  // Accessing
  VM_SYMBOLS_DO(VM_SYMBOL_DECLARE)
  // GC support
  static void follow_roots();
  static void adjust_roots();
  static void oops_do(void f(oop*));
};


// Class vmSymbolHandles

class vmSymbolHandles: AllStatic {
 public:
  // Accessing
  VM_SYMBOLS_DO(VM_SYMBOL_HANDLE_DECLARE)
};

#undef VM_SYMBOL_NAME_POSTFIX
#undef VM_SYMBOL_ENUM
#undef VM_SYMBOL_DECLARE
#undef VM_SYMBOL_HANDLE_DECLARE

