/*
 * @(#)arrayKlass.cpp	1.64 01/02/09 14:45:20
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_arrayKlass.cpp.incl"

int arrayKlass::object_size(int header_size) const {
  // size of an array klass object
  assert(header_size <= instanceKlass::header_size(), "bad header size");
  // If this assert fails, see comments in base_create_array_klass.
  header_size = instanceKlass::header_size();
  int size = header_size + vtable_length();
  return align_object_size(size);
}


oop arrayKlass::multi_allocate(int rank, jint* sizes, int next_dim_step, TRAPS) {
  ShouldNotReachHere();
  return NULL;
}

methodOop arrayKlass::uncached_lookup_method(symbolOop name, symbolOop signature) const {
  // There are no methods in an array klass but the super class (Object) has some
  assert(super(), "super klass must be present");
  return Klass::cast(super())->uncached_lookup_method(name, signature);
}


klassOop arrayKlass::base_create_array_klass(
const Klass_vtbl& cplusplus_vtbl, int header_size, KlassHandle klass, TRAPS) {
  // Allocation
  // Note: because the Java vtable must start at the same offset in all klasses,
  // we must insert filler fields into arrayKlass to make it the same size as instanceKlass.
  // If this assert fails, add filler to instanceKlass to make it bigger.
  assert(header_size <= instanceKlass::header_size(),
         "array klasses must be same size as instanceKlass");
  header_size = instanceKlass::header_size();
  // Arrays don't add any new methods, so their vtable is the same size as
  // the vtable of klass Object.
  int vtable_size = Universe::base_vtable_size();
  arrayKlassHandle k = Klass::base_create_klass(klass, header_size + vtable_size, cplusplus_vtbl, CHECK_0);
  
  k->set_super(Universe::is_bootstrapping() ? NULL : SystemDictionary::object_klass());
  k->set_size_helper(0);
  k->set_dimension(1);
  k->set_higher_dimension(NULL);
  k->set_lower_dimension(NULL);
  k->set_vtable_length(vtable_size);
  k->set_is_cloneable(); // All arrays are considered to be cloneable (See JLS 20.1.5)

  // Make sure size calculation is right
  assert(k()->size() == align_object_size(header_size + vtable_size), "wrong size for object");
  
  return k();
}

// Initialization of vtables and mirror object is done separatly from base_create_array_klass,
// since a GC can happen. At this point all instance variables of the arrayKlass must be setup.
void arrayKlass::complete_create_array_klass(arrayKlassHandle k, TRAPS) {
  ResourceMark rm;    
  k->vtable()->initialize_vtable();  
  java_lang_Class::create_mirror(k, CHECK);     
}

bool arrayKlass::compute_is_subtype_of(klassOop k) {
  // An array is a subtype of Serializable, Clonable, and Object
  return    k == SystemDictionary::object_klass()
         || k == SystemDictionary::cloneable_klass()
         || k == SystemDictionary::serializable_klass();
}


inline int* arrayKlass::start_of_vtable() const {
  // all vtables start at the same place, that's why we use instanceKlass::header_size here
  return ((int*)as_klassOop()) + instanceKlass::header_size(); 
}


klassVtable* arrayKlass::vtable() const {
  return new klassVtable(as_klassOop(), start_of_vtable(), vtable_length() / vtableEntry::size());
}


void arrayKlass::oop_follow_contents(oop obj) {
  assert (obj->is_array(), "obj must be array");
  obj->follow_header();
}


int arrayKlass::oop_scavenge_contents(oop obj) {
  assert (obj->is_array(), "obj must be array");
  return oop_size(obj);
}


int arrayKlass::oop_scavenge_contents(oop obj, oop* bottom, oop* top) {
  assert (obj->is_array(), "obj must be array");
  return oop_size(obj);
}


int arrayKlass::oop_adjust_pointers(oop obj) {
  int size = oop_size(obj);
  assert (obj->is_array(), "obj must be array");
  obj->adjust_header();
  return size;
}


void arrayKlass::oop_oop_iterate(oop obj, OopClosure* blk) {
  assert (obj->is_array(), "obj must be array");
  obj->oop_iterate_header(blk);
}


objArrayOop arrayKlass::allocate_arrayArray(int n, int length, TRAPS) {
  if (length < 0) THROW_0(vmSymbols::java_lang_NegativeArraySizeException());
  if (length > arrayOopDesc::max_array_length(T_ARRAY)) {
    THROW_OOP_0(Universe::out_of_memory_error_instance());
  }
  int size = objArrayOopDesc::object_size(length);
  arrayKlassHandle ak = array_klass(n+dimension(), CHECK_0);
  objArrayOop o = (objArrayOop)Universe::clear_allocate_array(ak, size, length, CHECK_0);
  // initialization to NULL not necessary, area already cleared
  return o;
}


void arrayKlass::array_klasses_do(void f(klassOop k)) {
  klassOop k = as_klassOop();
  // Iterate over this array klass and all higher dimensions
  while (k != NULL) {
    f(k);
    k = arrayKlass::cast(k)->higher_dimension();
  }
}


void arrayKlass::with_array_klasses_do(void f(klassOop k)) {
  array_klasses_do(f);
}

// JVM support

jint arrayKlass::modifier_flags(TRAPS) const {
  return JVM_ACC_ABSTRACT | JVM_ACC_FINAL | JVM_ACC_PUBLIC;
}

// JVMDI support

jint arrayKlass::class_status() const {
  return JVMDI_CLASS_STATUS_VERIFIED | JVMDI_CLASS_STATUS_PREPARED | JVMDI_CLASS_STATUS_INITIALIZED;
}

#ifndef PRODUCT

// Printing

void arrayKlass::oop_print_on(oop obj, outputStream* st) {
  assert(obj->is_array(), "must be array");
  Klass::oop_print_on(obj, st);
  st->print_cr(" - length: %d", arrayOop(obj)->length());
}

#endif

