/*
 * @(#)arrayKlass.hpp	1.37 00/10/07 11:46:43
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// arrayKlass is the abstract baseclass for all array classes

class arrayKlass: public Klass {
  friend class VMStructs;
 private:
  int      _dimension;         // This is n'th-dimensional array.
  klassOop _higher_dimension;  // Refers the (n+1)'th-dimensional array (if present).
  klassOop _lower_dimension;   // Refers the (n-1)'th-dimensional array (if present).
  int      _vtable_len;        // size of vtable for this klass  
  juint    _alloc_size;        // allocation profiling support
  int      _array_header_in_bytes; // offset of first element, including any
				// padding for the sake of alignment

 public:
  // Testing operation
  bool oop_is_array() const { return true; }

  // Instance variables
  int dimension() const                 { return _dimension;      }
  void set_dimension(int dimension)     { _dimension = dimension; }

  klassOop higher_dimension() const     { return _higher_dimension; }
  void set_higher_dimension(klassOop k) { oop_store_without_check((oop*) &_higher_dimension, (oop) k); }
  oop* adr_higher_dimension()           { return (oop*)&this->_higher_dimension;}

  klassOop lower_dimension() const      { return _lower_dimension; }
  void set_lower_dimension(klassOop k)  { oop_store_without_check((oop*) &_lower_dimension, (oop) k); }
  oop* adr_lower_dimension()            { return (oop*)&this->_lower_dimension;}

  // Allocation profiling support
  juint alloc_size() const              { return _alloc_size; }
  void set_alloc_size(juint n)          { _alloc_size = n; }

  int  array_header_in_bytes() const    { return _array_header_in_bytes; }
  void set_array_header_in_bytes(int s) { _array_header_in_bytes = s;    }

  // Allocation
  // Sizes points to the first dimension of the array, subsequent dimensions
  // are in higher memory if next_dim_step == 1, or lower memory if next_dim_step == -1
  virtual oop multi_allocate(int rank, jint* sizes, int next_dim_step, TRAPS);
  objArrayOop allocate_arrayArray(int n, int length, TRAPS);

  // Lookup operations
  methodOop uncached_lookup_method(symbolOop name, symbolOop signature) const;

  // Casting from klassOop
  static arrayKlass* cast(klassOop k) {
    assert(k->klass_part()->oop_is_array(), "cast to arrayKlass");
    return (arrayKlass*) k->klass_part(); 
  }

  bool compute_is_subtype_of(klassOop k);

  // Sizing
  static int header_size()                 { return oopDesc::header_size() + sizeof(arrayKlass)/wordSize; }
  int object_size(int header_size) const;

  // Java vtable
  klassVtable* vtable() const;             // return new klassVtable
  int  vtable_length() const               { return _vtable_len; }
  static int base_vtable_length()          { return Universe::base_vtable_size(); }
  void set_vtable_length(int len)          { assert(len == base_vtable_length(), "bad length"); _vtable_len = len; }
 protected:
  inline int* start_of_vtable() const;

 public:
  // Garbage collection
  void oop_follow_contents(oop obj);
  int  oop_scavenge_contents(oop obj);
  int  oop_scavenge_contents(oop obj, oop* bottom, oop* top);
  int  oop_adjust_pointers(oop obj);

  // Iterators
  void oop_oop_iterate(oop obj, OopClosure* blk);
  void array_klasses_do(void f(klassOop k));
  void with_array_klasses_do(void f(klassOop k));

  // Shared creation method
  static klassOop base_create_array_klass(const Klass_vtbl& vtbl,
                                          int header_size, KlassHandle klass,
                                          TRAPS);
  static void     complete_create_array_klass(arrayKlassHandle k, TRAPS);

 public:
   // jvm support
   jint modifier_flags(TRAPS) const;

 public:
   // jvmdi support
   jint class_status() const;

#ifndef PRODUCT
 public:
  // Printing
  void oop_print_on(oop obj, outputStream* st);
#endif
};

