/*
 * @(#)arrayOop.hpp	1.20 01/02/09 14:45:21
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// arrayOopDesc is the abstract baseclass for all arrays.

class arrayOopDesc : public oopDesc {
  friend class VMStructs;
 private:
  int _length; // number of elements in the array

 public:
  // Returns the address of the first element.
  void* base(BasicType type) const              { return (void*) (((int) this) + base_offset_in_bytes(type)); }

  // Tells whether index is within bounds.
  bool is_within_bounds(int index) const	{ return 0 <= index && index < length(); }

  // Accessores for instance variable
  int length() const				{ return _length;   }
  void set_length(int length)			{ _length = length; }

  // Header size computation. 
  // Should only be called with constants as argument (will not constant fold otherwise)
  static int header_size(BasicType type) {
    return Universe::element_type_should_be_aligned(type) 
      ? align_size_up(sizeof(arrayOopDesc)/wordSize, 2) 
      : sizeof(arrayOopDesc)/wordSize; 
  }

  // Interpreter/Compiler offsets
  static int length_offset_in_bytes()             { return (int)&(arrayOop(NULL)->_length); }
  static int base_offset_in_bytes(BasicType type) { return header_size(type) * wordSize; }

  // This method returns the  maximum length that can passed into object_size(scale, length, type) without
  // causing an overflow. We substract an extra 2*wordSize to guard against double word alignments.
  // It gets the scale from the type2aelembytes array.
  static int max_array_length(BasicType type) { 
    assert(type >= 0 && type < T_CONFLICT, "wrong type");
    assert(type2aelembytes[type] != 0, "wrong type");
    return (((unsigned)1 << (BitsPerWord-1)) - wordSize * header_size(type) - 2*wordSize) / type2aelembytes[type];
  }

};
