/*
 * @(#)cpCacheOop.cpp	1.24 01/02/09 14:45:24
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_cpCacheOop.cpp.incl"


// Implememtation of ConstantPoolCacheEntry

void ConstantPoolCacheEntry::set_initial_state(int index) {
  assert(0 <= index && index < 0x10000, "sanity check");
  _indices = index;
}


static int as_flags(TosState state, bool is_final) {
  int f;
  switch (state) {
    case itos: f = 0; break;
    case ftos: f = 1; break;
    case ltos: f = 2; break;
    case dtos: f = 3; break;
    case atos: f = 4; break;
    case vtos: f = 5; break;
    default  : ShouldNotReachHere();
  }
  f <<= 1;
  if (is_final) f |= 1;
  return f << 28;
}


void ConstantPoolCacheEntry::set_bytecode_1(Bytecodes::Code code) {
  // %%% need a memory synchronization here
  assert(bytecode_1() == 0 || bytecode_1() == code, "update must be consistent");
  _indices |= (u_char)code << 16;
}

void ConstantPoolCacheEntry::set_bytecode_2(Bytecodes::Code code) {
  // %%% need a memory synchronization here
  assert(bytecode_2() == 0 || bytecode_2() == code, "update must be consistent");
  _indices |= (u_char)code << 24;
}

#ifdef ASSERT
// It is possible to have two different dummy methodOops created
// when the resolve code for invoke interface executes concurrently
// Hence the assertion below is weakened a bit for the invokeinterface
// case.
bool ConstantPoolCacheEntry::same_methodOop(oop cur_f1, oop f1) {
  return (cur_f1 == f1 || ((methodOop)cur_f1)->name() ==
	 ((methodOop)f1)->name() || ((methodOop)cur_f1)->signature() == 
	 ((methodOop)f1)->signature());
}
#endif

// Note that concurrent update of both bytecodes can leave one of them
// reset to zero.  This is harmless; the interpreter will simply re-resolve
// the damaged entry.  More seriously, the memory synchronization is needed
// to flush other fields (f1, f2) completely to memory before the bytecodes
// are updated, lest other processors see a non-zero bytecode but zero f1/f2.
void ConstantPoolCacheEntry::set_field(Bytecodes::Code get_code, Bytecodes::Code put_code,
KlassHandle field_holder, int field_index, int field_offset, TosState field_type, bool is_final) {
  set_f1(field_holder());
  set_f2(field_offset);
  assert(field_index <= 0xFFFF, "field index does not fit in low flag bits");
  set_flags(as_flags(field_type, is_final) | (field_index & 0xFFFF));
  set_bytecode_1(get_code);
  set_bytecode_2(put_code);
  verify(tty);
}


void ConstantPoolCacheEntry::set_method(Bytecodes::Code invoke_code, methodHandle method, int vtable_index, bool change_to_virtual) {  
  assert(method->interpreter_entry() != NULL, "should have been set at this point");

  int byte_no = -1;
  switch (invoke_code) {
    case Bytecodes::_fast_invokevtable   : set_f2(vtable_index);  byte_no = 2; break;
    case Bytecodes::_fast_invokevfinal   : set_f2((int)method()); byte_no = 2; break;
    case Bytecodes::_fast_invokespecial  : // fall through
    case Bytecodes::_fast_invokestatic   : // fall through
    case Bytecodes::_fast_invokeinterface: set_f1(method());      byte_no = 1; break;
    default                              : ShouldNotReachHere();          break;
  }
  set_flags(as_flags(as_TosState(method->result_type()), method->is_final_method()) | method()->size_of_parameters());
  // Note:  byte_no also appears in TemplateTable::resolve.
  if (byte_no == 1) {
    set_bytecode_1(invoke_code);
  } else if (byte_no == 2)  {
    set_bytecode_2(invoke_code);
    if (change_to_virtual) {
      // NOTE: THIS IS A HACK - BE VERY CAREFUL!!!
      //
      // Workaround for the case where we encounter an invokeinterface, but we
      // should really have an invokevirtual since the resolved method is a 
      // virtual method in java.lang.Object. This is a corner case in the spec
      // but is presumably legal. javac does not generate this code.
      //
      // We set the invokeinterface byte (byte 1) to _fast_invokevtable as well.
      // When we return back to the interpreter, the resolve code for invokeinterface
      // will patch the bytecode to _fast_invokevtable (not _fast_invokeinterface!).
      // When re-executing the bytecode, we continue with a virtual call. Note that
      // invokeinterface is 5 bytes long, while invokevirtual is only 3 bytes long.
      // This leaves 2 bytes unused after the patch. The last byte of invokeinterface
      // must always be 0 (which translates into a nop bytecode). The 2nd last byte
      // may not be zero and we need to patch it (the interpreter doesn't use it).
      // See also interpreterRuntime.cpp. (8/25/2000)
      assert(invoke_code == Bytecodes::_fast_invokevtable, "flag not applicable");
      set_bytecode_1(invoke_code);
    }
  } else {
    ShouldNotReachHere();
  }
  verify(tty);
}


void ConstantPoolCacheEntry::set_interface_call(methodHandle method, int index) {
  klassOop interf = method->method_holder();
  assert(instanceKlass::cast(interf)->is_interface(), "must be an interface");
  set_f1(interf);
  set_f2(index);
  set_flags(as_flags(as_TosState(method->result_type()), method->is_final_method()) | method()->size_of_parameters());
  set_bytecode_1(Bytecodes::_fast_invokeinterface);  
}


class LocalOopClosure: public OopClosure {
 private:
  void (*_f)(oop*);

 public:
  LocalOopClosure(void f(oop*))        { _f = f; }
  virtual void do_oop(oop* o)          { _f(o); }
};


void ConstantPoolCacheEntry::oops_do(void f(oop*)) {
  LocalOopClosure blk(f);
  oop_iterate(&blk);
}


void ConstantPoolCacheEntry::oop_iterate(OopClosure* blk) {
  assert(size() == 4, "check code below - may need adjustment");
  // field[1] is always oop or NULL
  blk->do_oop(&_f1);
  // field[2] is oop if bytecode_2 is _fast_invokevfinal
  if (bytecode_2() == Bytecodes::_fast_invokevfinal) {
    blk->do_oop((oop*)&_f2);
  }
}


void ConstantPoolCacheEntry::follow_contents() {
  assert(size() == 4, "check code below - may need adjustment");
  // field[1] is always oop or NULL
  MarkSweep::mark_and_push(&_f1);
  // field[2] is oop if bytecode_2 is _fast_invokevfinal
  if (bytecode_2() == Bytecodes::_fast_invokevfinal) {
    MarkSweep::mark_and_push((oop*)&_f2);
  }
}


void ConstantPoolCacheEntry::adjust_pointers() {
  assert(size() == 4, "check code below - may need adjustment");
  // field[1] is always oop or NULL
  MarkSweep::adjust_pointer(&_f1);
  // field[2] is oop if bytecode_2 is _fast_invokevfinal
  if (bytecode_2() == Bytecodes::_fast_invokevfinal) {
    MarkSweep::adjust_pointer((oop*)&_f2);
  }
}


void ConstantPoolCacheEntry::print(outputStream* st, int index) const {
  // print separator
  if (index == 0) tty->print_cr("                 -------------");
  // print entry
  tty->print_cr("%3d  (%08x)  [%02x|%02x|%5d]", index, this, bytecode_2(), bytecode_1(), constant_pool_index());
  tty->print_cr("                 [   %08x]", _f1);
  tty->print_cr("                 [   %08x]", _f2);
  tty->print_cr("                 [   %08x]", _flags);
  tty->print_cr("                 -------------");
}


void ConstantPoolCacheEntry::verify(outputStream* st) const {
  // not implemented yet
}


// Implementation of ConstantPoolCache

void constantPoolCacheOopDesc::initialize(GrowableArray<int>* inverse_index_map) {
  assert(inverse_index_map->length() == length(), "inverse index map must have same length as cache");
  for (int i = 0; i < length(); i++) entry_at(i)->set_initial_state(inverse_index_map->at(i));
}

