/*
 * @(#)instanceRefKlass.cpp	1.42 01/03/28 13:21:30
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_instanceRefKlass.cpp.incl"

oop  instanceRefKlass::_discoveredSoftRefs;
oop  instanceRefKlass::_discoveredWeakRefs;
oop  instanceRefKlass::_discoveredFinalRefs;
oop  instanceRefKlass::_discoveredPhantomRefs;
bool instanceRefKlass::_processing_refs = false;
oop  instanceRefKlass::_sentinel_ref;

void instanceRefKlass::initializeRef() {
  EXCEPTION_MARK;
  _sentinel_ref = instanceKlass::cast(SystemDictionary::object_klass())->allocate_permanent_instance(THREAD);

  // Initialize the master soft ref clock.
  java_lang_ref_SoftReference::set_clock(os::javaTimeMillis());

  if (HAS_PENDING_EXCEPTION) {
      Handle ex(THREAD, PENDING_EXCEPTION);
      vm_exit_during_initialization(ex);
  }
  _discoveredSoftRefs    = _sentinel_ref;
  _discoveredWeakRefs    = _sentinel_ref;
  _discoveredFinalRefs   = _sentinel_ref;
  _discoveredPhantomRefs = _sentinel_ref;
}


bool instanceRefKlass::record_and_enqueue_reference(oop obj) {
  oop* next_addr = java_lang_ref_Reference::next_addr(obj);
  // We only enqueue active references, and only if we are not already processing discovered refs
  if (*next_addr != NULL || _processing_refs || !RegisterReferences) {
    return false;
  }
  oop* list;
  switch (reference_type()) {
    case REF_OTHER:
      return false;   // Unknown reference type, no special treatment
    case REF_SOFT:
      list = &_discoveredSoftRefs;
      break;
    case REF_WEAK:
      list = &_discoveredWeakRefs;
      break;
    case REF_FINAL:
      list = &_discoveredFinalRefs;
      break;
    case REF_PHANTOM:
      list = &_discoveredPhantomRefs;
      break;
    case REF_NONE:
      // we should not reach here if we are an instanceRefKlass
    default:
      ShouldNotReachHere();
  }
  // link reference object to appropriate list
  assert(*next_addr == NULL, "next should be NULL when enqueueing reference");
  assert(*list != NULL, "cannot set next to NULL, will look like non-enqueued reference");
  *next_addr = *list; // we do a raw store here, the field will be visited later
  *list = obj;
  if (TraceReferenceGC) {
    oop referent = java_lang_ref_Reference::referent(obj);
    tty->print_cr("Enqueued reference (0x%x)", obj);
  }
  return true;
}


int instanceRefKlass::oop_scavenge_contents(oop obj) {
  oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
  if (Scavenge::should_scavenge(*referent_addr)) {
    if (record_and_enqueue_reference(obj)) {
      // reference already enqueued, referent and next will be traversed later
      return instanceKlass::oop_scavenge_contents(obj);
    } else {
      // treat referent as normal oop
      Scavenge::scavenge(referent_addr);
    }
  }
  // treat next as normal oop
  oop* next_addr = java_lang_ref_Reference::next_addr(obj);
  if (Scavenge::should_scavenge(*next_addr)) {
    Scavenge::scavenge(next_addr);
  }
  return instanceKlass::oop_scavenge_contents(obj);
}


int instanceRefKlass::oop_scavenge_contents(oop obj, oop* bottom, oop* top) {
  oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
  if (referent_addr >= bottom && referent_addr < top && Scavenge::should_scavenge(*referent_addr)) {
    if (record_and_enqueue_reference(obj)) {
      // reference already enqueued, referent and next will be traversed later
      return instanceKlass::oop_scavenge_contents(obj, bottom, top);
    } else {
      // treat referent as normal oop
      Scavenge::scavenge(referent_addr);
    }
  }
  // treat next as normal oop
  oop* next_addr = java_lang_ref_Reference::next_addr(obj);
  if (next_addr >= bottom && next_addr < top && Scavenge::should_scavenge(*next_addr)) {
    Scavenge::scavenge(next_addr);
  }
  return instanceKlass::oop_scavenge_contents(obj, bottom, top);
}


void instanceRefKlass::oop_follow_contents(oop obj) {
  oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
  oop referent = *referent_addr;
  if (referent != NULL) {
    if (!referent->is_gc_marked() && record_and_enqueue_reference(obj)) {
      // reference already enqueued, referent will be traversed later
      instanceKlass::oop_follow_contents(obj);
      return;
    } else {
      // treat referent as normal oop      
      MarkSweep::mark_and_push(referent_addr);
    }
  }
  // treat next as normal oop
  oop* next_addr = java_lang_ref_Reference::next_addr(obj);
  MarkSweep::mark_and_push(next_addr);
  instanceKlass::oop_follow_contents(obj);
}


int instanceRefKlass::oop_adjust_pointers(oop obj) {
  int size = size_helper();
  instanceKlass::oop_adjust_pointers(obj);
  
  oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
  MarkSweep::adjust_pointer(referent_addr);
  oop* next_addr = java_lang_ref_Reference::next_addr(obj);
  MarkSweep::adjust_pointer(next_addr);
  return size;
}

void instanceRefKlass::oop_oop_iterate(oop obj, OopClosure* closure) {
  instanceKlass::oop_oop_iterate(obj, closure);
  
  oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
  closure->do_oop(referent_addr);

  // treat next as normal oop
  oop* next_addr = java_lang_ref_Reference::next_addr(obj);
  closure->do_oop(next_addr);
}

void instanceRefKlass::process_discovered_reflist(oop* refs_list_addr,
						  ReferencePolicy *policy, 
                                                  bool clear_referent,
						  bool is_alive(oop),
                                                  void keep_alive(oop*),
						  void complete_gc(void)) {
  debug_only(oop first_seen = NULL;) // Used for cyclic linked list check
    
  bool kept_some_alive = false;

  oop* prev_next = refs_list_addr;
  oop obj = *refs_list_addr;
  debug_only(first_seen = obj;) // cyclic linked list check

  // (SoftReferences only) Traverse the list and remove any SoftReferences whose
  // referents are not alive, but that should be kept alive for policy reasons.
  // Keep alive the transitive closure of all such referents.
  if (policy != NULL) {
    // Decide which softly reachable refs should be kept alive.
    while (obj != _sentinel_ref) {
      oop* next_addr = java_lang_ref_Reference::next_addr(obj);
      oop next = *next_addr;
      oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
      oop referent = *referent_addr;
      assert(Universe::is_heap_or_null(referent), "Wrong oop found in java.lang.Reference object");
      if (!is_alive(referent) && !policy->should_clear_reference(obj)) {
        // Make the Reference object active again
        java_lang_ref_Reference::set_next(obj, NULL);
        keep_alive(referent_addr);    // keep the referent around
        kept_some_alive = true; 
        if (TraceReferenceGC) {
          tty->print_cr("Dropping reference (0x%x) by policy", obj);
        }
        // Remove Reference object from list
        *prev_next = next;
      } else {
        prev_next = next_addr;
      }
      obj = next;
      assert(obj != first_seen, "cyclic ref_list found");
    }
    if (kept_some_alive) {
      complete_gc();
      kept_some_alive = false;
    }

    // Update the soft ref master clock field. This must be done
    // after processing the soft ref list.
    java_lang_ref_SoftReference::set_clock(os::javaTimeMillis());
  }

  // Traverse the list and remove any refs whose referents are alive.
  prev_next = refs_list_addr;
  obj = *refs_list_addr;
  while (obj != _sentinel_ref) {
    oop* next_addr = java_lang_ref_Reference::next_addr(obj);
    oop next = *next_addr;
    oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
    oop referent = *referent_addr;
    assert(Universe::is_heap_or_null(referent), "Wrong oop found in java.lang.Reference object");
    if (is_alive(referent)) {
      // The referent is reachable after all, make the Reference object active again
      java_lang_ref_Reference::set_next(obj, NULL);
      keep_alive(referent_addr);    // keep the referent around
      // Note that we do not have to set kept_some_alive to true here, since the
      // referent must already be traversed
      if (TraceReferenceGC) {
        tty->print_cr("Dropping strongly reachable reference (0x%x)", obj);
      }
      // Remove Reference object from list
      *prev_next = next;
    } else {
      prev_next = next_addr;
    }
    obj = next;
    assert(obj != first_seen, "cyclic ref_list found");
  }

  // Now traverse list and process list pointers themselves
  prev_next = refs_list_addr;
  obj = *refs_list_addr;
  debug_only(first_seen = obj;) // cyclic linked list check
  while (obj != _sentinel_ref) {
    keep_alive(prev_next);
    prev_next = java_lang_ref_Reference::next_addr(obj);
    oop* referent_addr = java_lang_ref_Reference::referent_addr(obj);
    oop referent = *referent_addr;
    assert(Universe::is_heap_or_null(referent), "Wrong oop found in java.lang.Reference object");
    if (clear_referent) {
      // NULL out referent pointer
      *referent_addr = NULL;
    } else {
      // keep the referent around
      keep_alive(referent_addr);
      kept_some_alive = true;
    }
    if (TraceReferenceGC) {
      tty->print_cr("Adding %sreference (0x%x) as pending", clear_referent ? "cleared " : "", obj);
    }
    obj = *prev_next;
    assert(obj != first_seen, "cyclic ref_list found");
  }
  // Remember to keep sentinel pointer around
  keep_alive(prev_next);
  if (kept_some_alive) {
    complete_gc();
  }
}


void instanceRefKlass::process_discovered_references(ReferencePolicy *policy, bool is_alive(oop), void keep_alive(oop*), void complete_gc(void)) {
  // Set flag so no additional discovered refs are enqueued (by traversing the referent fields) 
  _processing_refs = true;
  
  // Soft references
  if (TraceReferenceGC) tty->print_cr("Processing discovered soft references");
  process_discovered_reflist(&_discoveredSoftRefs, policy, true, is_alive, keep_alive, complete_gc);

  // Weak references
  if (TraceReferenceGC) tty->print_cr("Processing discovered weak references");
  process_discovered_reflist(&_discoveredWeakRefs, NULL, true, is_alive, keep_alive, complete_gc);

  // Final references
  if (TraceReferenceGC) tty->print_cr("Processing discovered final references");
  process_discovered_reflist(&_discoveredFinalRefs, NULL, false, is_alive, keep_alive, complete_gc);

  // Phantom references
  if (TraceReferenceGC) tty->print_cr("Processing discovered phantom references");
  process_discovered_reflist(&_discoveredPhantomRefs, NULL, false, is_alive, keep_alive, complete_gc);

  // Weak global JNI references. It would make more sense (semantically) to traverse these
  // simultaneously with the regular weak references above, but that is not how the JDK1.2
  // speficication is. See #4126360. Native code can thus use JNI weak references to circumvent 
  // the phantom references and resurrect a "post-mortem" object.
  if (TraceReferenceGC) tty->print_cr("Processing JNI weak references");
  JNIHandles::weak_oops_do(is_alive, keep_alive);

  // Finally remember to keep sentinel around
  keep_alive(&_sentinel_ref);
  complete_gc();
};


void instanceRefKlass::enqueue_discovered_reflist(oop refs_list, oop* pending_list_addr) {
  oop obj = refs_list;
  debug_only(oop prev_obj = NULL);
  while (obj != _sentinel_ref) {
    assert(obj->is_instanceRef(), "should be reference object");
    oop next = java_lang_ref_Reference::next(obj);
    // Hook up to pending list
    oop pending_list = *pending_list_addr;
    if (pending_list == NULL) {
      // Make "self-reference" end of list marker
      pending_list = obj;
    }
    java_lang_ref_Reference::set_next(obj, pending_list);
    oop_store(pending_list_addr, obj);
    debug_only(prev_obj = obj);
    obj = next;
  }
};


bool instanceRefKlass::enqueue_discovered_references() {
  // Remember old value of pending references list
  oop* pending_list_addr = java_lang_ref_Reference::pending_list_addr();
  oop old_pending_list_value = *pending_list_addr;

  // Enqueue references that are not made active again
  enqueue_discovered_reflist(_discoveredSoftRefs, pending_list_addr);
  enqueue_discovered_reflist(_discoveredWeakRefs, pending_list_addr);
  enqueue_discovered_reflist(_discoveredFinalRefs, pending_list_addr);
  enqueue_discovered_reflist(_discoveredPhantomRefs, pending_list_addr);

  // Clear lists for next GC
  _discoveredSoftRefs    = _sentinel_ref;
  _discoveredWeakRefs    = _sentinel_ref;
  _discoveredFinalRefs   = _sentinel_ref;
  _discoveredPhantomRefs = _sentinel_ref;

  // Clear flag for next GC
  _processing_refs = false;

  // Return true if new pending references were added
  return old_pending_list_value != *pending_list_addr;
};

void instanceRefKlass::adjust_roots() {
  MarkSweep::adjust_root_pointer((oop*)&_discoveredSoftRefs);
  MarkSweep::adjust_root_pointer((oop*)&_discoveredWeakRefs);
  MarkSweep::adjust_root_pointer((oop*)&_discoveredFinalRefs);
  MarkSweep::adjust_root_pointer((oop*)&_discoveredPhantomRefs);
  MarkSweep::adjust_root_pointer((oop*)&_sentinel_ref);
}


void instanceRefKlass::update_nonstatic_oop_maps(klassOop k) {
  // Clear the nonstatic oop-map entries corresponding to referent and nextPending field.
  // They are treated specially by the garbage collector.
  instanceKlass* ik = instanceKlass::cast(k);

  // Check that we have the right class
  debug_only(static bool first_time = true);
  assert(k == SystemDictionary::reference_klass() && first_time, "Invalid update of maps");
  debug_only(first_time = false);
  assert(ik->nonstatic_oop_map_size() == 1, "just checking");

  OopMapBlock* map = ik->start_of_nonstatic_oop_maps();

  // Check that the current map is (2,3)
  debug_only(int offset = java_lang_ref_Reference::referent_offset + instanceOopDesc::header_size());
  debug_only(int length = java_lang_ref_Reference::next_offset - java_lang_ref_Reference::referent_offset + 1);
  assert(map->offset() == offset && map->length() == length, "just checking");

  // Update map to (3,1)
  map->set_offset(java_lang_ref_Reference::queue_offset + instanceOopDesc::header_size());
  map->set_length(1);
}



#ifndef PRODUCT

// Verification

void instanceRefKlass::oop_verify_on(oop obj, outputStream* st) {
  instanceKlass::oop_verify_on(obj, st);
  // Verify referent field
  oop referent = java_lang_ref_Reference::referent(obj);
  if (referent != NULL) {
    guarantee(referent->safepoint_is_oop(), "referent field heap failed");    
    if (obj->is_old_or_perm()) {
      // We do a specific remembered set check here since the referent field is 
      // not part of the oop mask and therefore skipped by the regular verify code.
      obj->verify_old_oop(java_lang_ref_Reference::referent_addr(obj), true);
    }
  }
  // Verify next field
  oop next = java_lang_ref_Reference::next(obj);
  if (next != NULL) {
    guarantee(next->safepoint_is_oop(), "next field verify failed");    
    guarantee(next->is_instanceRef(), "next field verify failed");
    if (obj->is_old()) {
      // We do a specific remembered set check here since the next field is 
      // not part of the oop mask and therefore skipped by the regular verify code.
      obj->verify_old_oop(java_lang_ref_Reference::next_addr(obj), true);
    }
  }
}


void instanceRefKlass::verify_no_references_recorded() {
  guarantee(
    _discoveredSoftRefs == _sentinel_ref &&
    _discoveredWeakRefs == _sentinel_ref &&
    _discoveredFinalRefs == _sentinel_ref &&
    _discoveredPhantomRefs == _sentinel_ref &&
    _processing_refs == false, "verify_no_references_recorded failed");
}

#endif

void instanceRefKlass_init() {
  instanceRefKlass::initializeRef();
}
