/*
 * @(#)instanceRefKlass.hpp	1.27 01/03/28 13:21:30
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// An instanceRefKlass is a specialized instanceKlass for Java 
// classes that are subclasses of java/lang/ref/Reference.
//
// These classes are used to implement soft/weak/final/phantom 
// references and finalization, and need special treatment by the
// garbage collector.
//
// During GC discovered reference objects are added (chained) to one
// of the four lists below, depending on the type of reference.
// The linked occurs through the next field in class java/lang/ref/Reference.
//
// Afterwards, the discovered references are processed in decreasing
// order of reachability. Reference objects eligible for notification
// are linked to the static pending_list in class java/lang/ref/Reference,
// and the pending list lock object in the same class is notified.


class instanceRefKlass: public instanceKlass {
  friend class VMStructs;
 private:
  // Lists of reference objects found during GC
  static oop  _discoveredSoftRefs;
  static oop  _discoveredWeakRefs;
  static oop  _discoveredFinalRefs;
  static oop  _discoveredPhantomRefs;
  // Flag
  static bool _processing_refs;
  // End of list marker
  static oop  _sentinel_ref;

  // Check whether reference obj should be enqueued
  bool record_and_enqueue_reference(oop obj);
  // Process references with a certain reachability level.
  static void process_discovered_reflist(oop* refs_list_addr,
					 ReferencePolicy *policy,
					 bool clear_referent,
					 bool is_alive(oop),
					 void keep_alive(oop*),
					 void complete_gc(void));
  // Enqueue references with a certain reachability level
  static void enqueue_discovered_reflist(oop refs_list, oop* pending_list_addr);

 public:
  // Type testing
  bool oop_is_instanceRef() const             { return true; }

  // Casting from klassOop
  static instanceRefKlass* cast(klassOop k) {
    assert(k->klass_part()->oop_is_instanceRef(), "cast to instanceRefKlass");
    return (instanceRefKlass*) k->klass_part(); 
  }

  // allocation
  DEFINE_ALLOCATE_PERMANENT(instanceRefKlass);

  // Garbage collection
  int oop_scavenge_contents(oop obj);
  int oop_scavenge_contents(oop obj, oop* bottom, oop* top);
  int oop_adjust_pointers(oop obj);
  void oop_follow_contents(oop obj);
  void oop_oop_iterate(oop obj, OopClosure* blk);

  // Disable reference discovery for single gc (temporary functionality)
  static void disable_refs_discovery()        { _processing_refs = true; }

  // Process references found during GC (called by the garbage collector)
  static void process_discovered_references(ReferencePolicy *policy, bool is_alive(oop), void keep_alive(oop*), void complete_gc(void));
         
  // Enqueue references at end of GC (called by the garbage collector)
  static bool enqueue_discovered_references();

  // Mark-sweep support
  static void adjust_roots();

  // Update non-static oop maps so 'referent' and 'nextPending' will look like non-oops
  static void update_nonstatic_oop_maps(klassOop k);

  // Initialization
  static void initializeRef();

#ifndef PRODUCT
 public:
  // Verification
  void oop_verify_on(oop obj, outputStream* st);
  static void verify_no_references_recorded();
#endif
};

