/*
 * @(#)klass.cpp	1.72 01/02/09 14:45:32
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_klass.cpp.incl"

bool Klass::compute_and_set_is_subtype_of(klassOop k) {
  bool result = compute_is_subtype_of(k);
  if (result) {
    set_subtype_cache_2(subtype_cache_1());
    set_subtype_cache_1(k);
  }
  return result;
}

bool Klass::is_subclass_of(klassOop k) const {
  // Run up the super chain and check
  klassOop t = as_klassOop();
    
  if (t == k) return true;
  t = Klass::cast(t)->super();

  while (t != NULL) {  
    if (t == k) return true;
    t = Klass::cast(t)->super();
  }
  return false;
}

// Return self, except for abstract classes with exactly 1
// implementor.  Then return the 1 concrete implementation.
Klass *Klass::up_cast_abstract() {
  Klass *r = this;
  while( r->is_abstract() ) {   // Receiver is abstract?
    Klass *s = r->subklass();   // Check for exactly 1 subklass
    if( !s || s->next_sibling() ) // Oops; wrong count; give up
      return this;              // Return 'this' as a no-progress flag
    r = s;                    // Loop till find concrete class
  }
  return r;                   // Return the 1 concrete class
}

// Find LCA in class heirarchy
Klass *Klass::LCA( Klass *k2 ) {
  Klass *k1 = this;
  while( 1 ) {
    if( k1->is_subtype_of(k2->as_klassOop()) ) return k2;
    if( k2->is_subtype_of(k1->as_klassOop()) ) return k1;
    k1 = k1->super()->klass_part();
    k2 = k2->super()->klass_part();
  }
}

bool Klass::can_be_statically_bound(methodOop m) {
  if (!RespectFinal) return false;
  if (m->is_final_method()) return true;
  // if it doesn't have a vtable entry, it's statically bound
  ResourceMark rm;
  klassVtable* vt = Klass::cast(m->method_holder())->vtable();
  return vt->index_of(m) == -1;
}


void Klass::check_valid_for_instantiation(bool throwError, TRAPS) {
  ResourceMark rm;
  THROW_MSG(throwError ? vmSymbols::java_lang_InstantiationError()
	    : vmSymbols::java_lang_InstantiationException(), external_name());
}


void Klass::copy_array(arrayOop s, int src_pos, arrayOop d, int dst_pos, int length, TRAPS) {
  THROW(vmSymbols::java_lang_ArrayStoreException());
}


void Klass::initialize(TRAPS) {
  ShouldNotReachHere();
}

bool Klass::compute_is_subtype_of(klassOop k) {
  assert(k->is_klass(), "argument must be a class");
  return is_subclass_of(k);
}

methodOop Klass::lookup_method(symbolOop name, symbolOop signature) const {
  return uncached_lookup_method(name, signature);
}

methodOop Klass::uncached_lookup_method(symbolOop name, symbolOop signature) const {
#ifdef ASSERT
  tty->print_cr("Error: uncached_lookup_method called on a klass oop."
                " Likely error: reflection method does not correctly"
                " wrap return value in a mirror object.");
#endif
  ShouldNotReachHere();
  return NULL;
}

klassOop Klass::base_create_klass(KlassHandle& klass, int size, 
                                  const Klass_vtbl& vtbl, TRAPS) {
  size = align_object_size(size);
  // allocate and initialize vtable
  Klass*   kl = (Klass*) vtbl.allocate_permanent(klass, size, CHECK_0);
  klassOop k  = kl->as_klassOop();

  kl->set_super(NULL);

  // kl->set_subtype_cache(NULL);
  kl->set_subtype_cache_1(k); // subtype cache must contain a valid klass; compiler needs it.
  kl->set_subtype_cache_2(k); // subtype cache must contain a valid klass; compiler needs it.
  kl->set_java_mirror(NULL);
  kl->set_size_helper(0);
  kl->set_name(NULL);
  AccessFlags af;
  af.set_flags(0);
  kl->set_access_flags(af);
  kl->set_subklass(NULL);
  kl->set_next_sibling(NULL);
  kl->set_alloc_count(0);
  kl->set_alloc_size(0);
  return k;
}

void* Klass_vtbl::operator new(size_t ignored, KlassHandle& klass,
                               int size, TRAPS) {
  klassOop k = (klassOop) Universe::allocate_permanent(klass, size, CHECK_0);
  return k->klass_part();
}

Klass* Klass::subklass() const {
  return _subklass == NULL ? NULL : Klass::cast(_subklass);
}

instanceKlass* Klass::superklass() const {
  assert(super() == NULL || super()->klass_part()->oop_is_instance(), "must be instance klass");
  return _super == NULL ? NULL : instanceKlass::cast(_super);
}

Klass* Klass::next_sibling() const {
  return _next_sibling == NULL ? NULL : Klass::cast(_next_sibling);
}

void Klass::set_subklass(klassOop s) {
  assert(s != as_klassOop(), "sanity check");
  _subklass = s;
}

void Klass::set_next_sibling(klassOop s) {
  assert(s != as_klassOop(), "sanity check");
  _next_sibling = s;
}

void Klass::append_to_sibling_list() {
  debug_only(as_klassOop()->verify();)
  // add ourselves to superklass' subklass list
  instanceKlass* super = superklass();
  if (super == NULL) return;        // special case: class Object
  assert(super->is_interface() || super->superklass() == NULL || !is_interface(), 
    "an interface can only be a subklass of another interface or of Object (not of a klass)");
  klassOop prev_first_subklass = super->subklass_oop();
  if (prev_first_subklass != NULL) {
    // set our sibling to be the superklass' previous first subklass
    set_next_sibling(prev_first_subklass);
  }
  // make ourselves the superklass' first subklass
  super->set_subklass(as_klassOop());
  debug_only(as_klassOop()->verify();)
}

void Klass::remove_from_sibling_list() {
  // remove receiver from sibling list
  instanceKlass* super = superklass();
  assert(super != NULL || as_klassOop() == SystemDictionary::object_klass(), "should have super");
  if (super == NULL) return;        // special case: class Object
  if (super->subklass() == this) {
    // first subklass
    super->set_subklass(_next_sibling);
  } else {
    Klass* sib = super->subklass(); 
    while (sib->next_sibling() != this) {
      sib = sib->next_sibling();
    };
    sib->set_next_sibling(_next_sibling);
  }
}


void Klass::follow_weak_klass_links() {
  // This klass is alive but the subklass and siblings are not followed/updated.
  // We update the subklass link and the subklass' sibling links here. 
  // Our own sibling link will be updated by our superclass (which must be alive 
  // since we are).
  assert(as_klassOop()->is_gc_marked(), "just checking");
  if (ClassUnloading) {
    klassOop sub = subklass_oop();
    if (sub != NULL && !sub->is_gc_marked()) {
      // first subklass not alive, find first one alive
      do {
#ifndef PRODUCT
        if (TraceClassUnloading && WizardMode) {
          ResourceMark rm;
          tty->print_cr("[Unlinking class (subclass) %s]", sub->klass_part()->external_name());
        }
#endif
        sub = sub->klass_part()->next_sibling_oop();
      } while (sub != NULL && !sub->is_gc_marked());
      set_subklass(sub);
    }
    // sub now contains our first live subklass, follow it.
    MarkSweep::mark_and_push(adr_subklass());
    // now update the subklass' sibling list
    while (sub != NULL) {
      klassOop next = sub->klass_part()->next_sibling_oop();
      if (next != NULL && !next->is_gc_marked()) {
        // first sibling not alive, find first one alive
        do {
#ifndef PRODUCT
          if (TraceClassUnloading && WizardMode) {
            ResourceMark rm;
            tty->print_cr("[Unlinking class (sibling) %s]", next->klass_part()->external_name());
          }
#endif
          next = next->klass_part()->next_sibling_oop();
        } while (next != NULL && !next->is_gc_marked());
        sub->klass_part()->set_next_sibling(next);
      }
      // next now contains the subklass' first live sibling, follow it.
      MarkSweep::mark_and_push(sub->klass_part()->adr_next_sibling());
      sub = next;
    }
  } else {
    // Always follow subklass and sibling link. This will prevent any klasses from 
    // being unloaded (all classes are transitively linked from java.lang.Object).
    MarkSweep::mark_and_push(adr_subklass());
    MarkSweep::mark_and_push(adr_next_sibling());
  }
}


bool Klass::oop_being_unloaded(oop obj) {
  assert(!obj->is_gc_marked(), "should not be marked");
  return true;
}


klassOop Klass::array_klass_or_null(int rank) {
  EXCEPTION_MARK;
  // No exception can be thrown by array_klass_impl when called with or_null == true. 
  // (In anycase, the execption mark will fail if it do so)
  klassOop result = array_klass_impl(true, rank, THREAD);
  return result;
}


klassOop Klass::array_klass_or_null() {
  EXCEPTION_MARK;
  // No exception can be thrown by array_klass_impl when called with or_null == true. 
  // (In anycase, the execption mark will fail if it do so)
  klassOop result = array_klass_impl(true, THREAD);
  return result;
}


klassOop Klass::array_klass_impl(bool or_null, int rank, TRAPS) {
  fatal("array_klass should be dispatched to instanceKlass, objArrayKlass or typeArrayKlass");
  return NULL;
}


klassOop Klass::array_klass_impl(bool or_null, TRAPS) {
  fatal("array_klass should be dispatched to instanceKlass, objArrayKlass or typeArrayKlass");
  return NULL;
}


void Klass::with_array_klasses_do(void f(klassOop k)) {
  f(as_klassOop());
}


const char* Klass::external_name() const {
  ShouldNotReachHere();
  return NULL;
}

char* Klass::signature_name() const {
  return name()->as_C_string();
}

// Unless overridden, modifier_flags is 0.
jint Klass::modifier_flags(TRAPS) const {
  return 0;
}

// Unless overridden, class_status is 0.
jint Klass::class_status() const {
  return 0;
}

#ifndef PRODUCT

// Printing

void Klass::oop_print_on(oop obj, outputStream* st) {
  ResourceMark rm;
  // print title
  st->print_cr("%s ", internal_name());
  obj->print_address_on(st);
  
  if (WizardMode) {
     // print header
     obj->mark()->print_on(st);
  }

  // print class
  st->print(" - klass: ");
  obj->klass()->print_value_on(st);
  st->cr();
}


void Klass::oop_print_value_on(oop obj, outputStream* st) {
  // print title
  st->print("%s", internal_name());
  obj->print_address_on(st);
}


// Verification

void Klass::oop_verify_on(oop obj, outputStream* st) {
  guarantee(obj->safepoint_is_oop(),  "should be oop");  
  guarantee(obj->klass()->is_perm(),  "should be in permspace");
  guarantee(obj->klass()->is_klass(), "klass field is not a klass");
}


void Klass::oop_verify_old_oop(oop obj, oop* p, bool allow_dirty) {
  RememberedSet::verify_old_oop(obj, p, allow_dirty, false);
}

#endif
