/*
 * @(#)klassKlass.cpp	1.36 00/03/21 16:17:21
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_klassKlass.cpp.incl"

int klassKlass::oop_size(oop obj) const {
  assert (obj->is_klass(), "must be a klassOop");
  return klassOop(obj)->klass_part()->klass_oop_size();
}

klassOop klassKlass::create_klass(TRAPS) {
  KlassHandle h_this_klass;
  klassKlass o;
  klassOop k = base_create_klass(h_this_klass, header_size(), o.vtbl_value(), CHECK_0);
  k->set_klass(k); // point to thyself
  // Do not try to allocate mirror, java.lang.Class not loaded at this point.
  // See Universe::fixup_mirrors()
  return k;
}

void klassKlass::oop_follow_contents(oop obj) {
  Klass* k = Klass::cast(klassOop(obj));
  // If we are alive it is valid to keep our superclass and subtype caches alive
  MarkSweep::mark_and_push(k->adr_super());
  MarkSweep::mark_and_push(k->adr_subtype_cache_1());
  MarkSweep::mark_and_push(k->adr_subtype_cache_2());
  MarkSweep::mark_and_push(k->adr_java_mirror());
  MarkSweep::mark_and_push(k->adr_name());
  // We follow the subklass and sibling links at the end of the 
  // marking phase, since otherwise following them will prevent
  // class unloading (all classes are transitively linked from
  // java.lang.Object).
  MarkSweep::revisit_weak_klass_link(k);
  obj->follow_header();
}


void klassKlass::oop_oop_iterate(oop obj, OopClosure* blk) {
  Klass* k = Klass::cast(klassOop(obj));

  blk->do_oop(k->adr_super());
  blk->do_oop(k->adr_subtype_cache_1());
  blk->do_oop(k->adr_subtype_cache_2());
  blk->do_oop(k->adr_java_mirror());
  blk->do_oop(k->adr_name());
  blk->do_oop(k->adr_subklass());
  blk->do_oop(k->adr_next_sibling());

  obj->oop_iterate_header(blk);
}


int klassKlass::oop_scavenge_contents(oop obj) {
  return oop_size(obj);
}


int klassKlass::oop_scavenge_contents(oop obj, oop* bottom, oop* top) {
  return oop_size(obj);
}


int klassKlass::oop_adjust_pointers(oop obj) {
  int size = oop_size(obj);
  obj->adjust_header();

  Klass* k = Klass::cast(klassOop(obj));

  MarkSweep::adjust_pointer(k->adr_super());
  MarkSweep::adjust_pointer(k->adr_subtype_cache_1());
  MarkSweep::adjust_pointer(k->adr_subtype_cache_2());
  MarkSweep::adjust_pointer(k->adr_java_mirror());
  MarkSweep::adjust_pointer(k->adr_name());
  MarkSweep::adjust_pointer(k->adr_subklass());
  MarkSweep::adjust_pointer(k->adr_next_sibling());
  return size;
}


#ifndef PRODUCT

// Printing

void klassKlass::oop_print_on(oop obj, outputStream* st) {
  Klass::oop_print_on(obj, st);
}


void klassKlass::oop_print_value_on(oop obj, outputStream* st) {
  Klass::oop_print_value_on(obj, st);
}


const char* klassKlass::internal_name() const {
  return "{other class}";
}


// Verification

void klassKlass::oop_verify_on(oop obj, outputStream* st) {
  Klass::oop_verify_on(obj, st);
  guarantee(obj->is_perm(),                      "should be in permspace");
  guarantee(obj->is_klass(),                     "should be klass");

  Klass* k = Klass::cast(klassOop(obj));
  if (k->super() != NULL) {
    guarantee(k->super()->is_perm(),             "should be in permspace");
    guarantee(k->super()->is_klass(),            "should be klass");
  }
  if (k->subtype_cache_1() != NULL) {
    guarantee(k->subtype_cache_1()->is_perm(),     "should be in permspace");
    guarantee(k->subtype_cache_1()->is_klass(),    "should be klass");
  }
  if (k->subtype_cache_2() != NULL) {
    guarantee(k->subtype_cache_2()->is_perm(),     "should be in permspace");
    guarantee(k->subtype_cache_2()->is_klass(),    "should be klass");
  }  
  if (k->java_mirror() != NULL || (k->oop_is_instance() && instanceKlass::cast(klassOop(obj))->is_loaded())) {
    guarantee(k->java_mirror() != NULL,          "should be allocated");
    guarantee(k->java_mirror()->is_perm(),       "should be in permspace");
    guarantee(k->java_mirror()->is_instance(),   "should be instance");
  }
  if (k->name() != NULL) {
    guarantee(k->name()->is_perm(),              "should be in permspace");
    guarantee(k->name()->is_symbol(),            "should be symbol");
  }
}

#endif
