/*
 * @(#)markOop_alignment.hpp	1.6 99/06/22 16:41:35
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Included into markOop.hpp.

// Implementation of inline functions for storing pointers in markOops
// for mark-sweep, scavenger, train GC. Not used by locking code.

// This version uses the low-order bits because on SPARC
// (and other boxes) all Oops must be aligned but there is NO
// guarantee that would let us use the sentinel bit
// (i.e. the MSB of an oop's address may be set if the oop is in the stack).

// Since scavenging, train gc, and mark-sweep only happen at safepoints
// and since the busy() lock state may not happen at safepoints,
// we reuse that state to encode a pointer into a mark.

// This encoding distinguishes pointers-in-marks from locked marks,
// (locking does NOT follow the protocol in this file).
// This distinction is needed for scavenging and train, which
// may have to deal with locked (but not busy!) objects.
// It also could be used for an assert in mark-sweep.

// Ungar - 11/97

// See interface comments in markOop.hpp.

inline bool markOopDesc::has_pointer() { return is_marked(); }

inline bool markOopDesc::assert_not_real_mark() { return true; } // nothing to check
inline bool markOopDesc::assert_real_mark()     { return true; } // nothing to check

inline markOop markOopDesc::set_no_pointer() { return this; }

inline markOop markOopDesc::encode_pointer_as_mark(void* p) {
  return markOop(p)->set_marked();
}

inline void* markOopDesc::decode_pointer() {
  return clear_lock_bits();
}

