/*
 * @(#)methodKlass.cpp	1.63 00/04/25 10:13:53
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_methodKlass.cpp.incl"

klassOop methodKlass::create_klass(TRAPS) {
  methodKlass o;
  KlassHandle h_this_klass(THREAD, Universe::klassKlassObj());  
  KlassHandle k = base_create_klass(h_this_klass, header_size(), o.vtbl_value(), CHECK_0);
  // Make sure size calculation is right
  assert(k()->size() == align_object_size(header_size()), "wrong size for object");
  java_lang_Class::create_mirror(k, CHECK_0); // Allocate mirror
  return k();
}


int methodKlass::oop_size(oop obj) const {
  assert(obj->is_method(), "must be method oop");
  return methodOop(obj)->object_size();
}


methodOop methodKlass::allocate(int byte_code_size, AccessFlags access_flags, int compressed_line_number_size, int localvariable_table_length, int checked_exceptions_length, TRAPS) {
  int size = methodOopDesc::object_size(byte_code_size, access_flags.is_native(), compressed_line_number_size, localvariable_table_length, checked_exceptions_length);
  KlassHandle h_k(THREAD, as_klassOop());
  methodOop m = (methodOop)Universe::allocate_permanent(h_k, size, CHECK_0);
  m->set_access_flags(access_flags);
  m->set_method_size(size);
  m->set_code_size(byte_code_size);
  m->set_parameter_info(0);
  m->set_name_index(0);
  m->set_signature_index(0);
  m->set_constants(NULL);
  m->set_max_stack(0);
  m->set_max_locals(0);
#ifdef COMPILER2
  m->set_null_cast_cache(methodOopDesc::Null_Cast_Cache_Empty);
  m->clear_null_cast_seen();
  m->set_interpreter_throwout_count(0);
#endif
  m->set_vtable_index(-1);  
  m->init_code();
  if (access_flags.is_native()) {
    m->set_native_function(NULL);
    m->set_signature_handler(NULL);
  }
  m->set_interpreter_entry(NULL);
  m->set_exception_table(NULL);
  m->set_inlined_tables_length(checked_exceptions_length, compressed_line_number_size, localvariable_table_length); // updates access flags as well
  NOT_CORE(NOT_PRODUCT(m->set_compiled_invocation_count(0);))
  COMPILER2_ONLY(m->set_interpreter_invocation_count(0);)
  NOT_CORE(m->invocation_counter()->reset();)
  m->alloc_bci_histogram();
  assert(m->size() == size, "wrong size for object");
  return m;
}


void methodKlass::oop_follow_contents(oop obj) {
  assert (obj->is_method(), "object must be method");
  methodOop m = methodOop(obj);

  obj->follow_header();
  MarkSweep::mark_and_push(m->adr_exception_table());
  MarkSweep::mark_and_push(m->adr_constants());  
}


void methodKlass::oop_oop_iterate(oop obj, OopClosure* blk) {
  assert (obj->is_method(), "object must be method");
  methodOop m = methodOop(obj);

  obj->oop_iterate_header(blk);
  blk->do_oop(m->adr_exception_table());
  blk->do_oop(m->adr_constants());
}


int methodKlass::oop_scavenge_contents(oop obj) {
  assert(obj->is_method(), "should be method");
  return oop_size(obj);
}


int methodKlass::oop_scavenge_contents(oop obj, oop* bottom, oop* top) {
  assert(obj->is_method(), "should be method");
  return oop_size(obj);
}

int methodKlass::oop_adjust_pointers(oop obj) {
  assert(obj->is_method(), "should be method");
  int size = oop_size(obj);
  methodOop m = methodOop(obj);

  obj->adjust_header();
  MarkSweep::adjust_pointer(m->adr_exception_table());
  MarkSweep::adjust_pointer(m->adr_constants());

  return size;
}


#ifndef PRODUCT

// Printing

void methodKlass::oop_print_on(oop obj, outputStream* st) {
  ResourceMark rm;
  assert(obj->is_method(), "must be method");
  Klass::oop_print_on(obj, st);
  methodOop m = methodOop(obj);
  st->print   (" - method holder:     ");    m->method_holder()->print_value_on(st); st->cr();
  st->print   (" - constants:         ");    m->constants()->print_value_on(st); st->cr();
  st->print   (" - access:            0x%x  ", m->access_flags().as_int()); m->access_flags().print_on(st); st->cr();
  st->print   (" - name:              ");    m->name()->print_value_on(st); st->cr();
  st->print   (" - signature:         ");    m->signature()->print_value_on(st); st->cr();
  st->print_cr(" - max stack:         %d",   m->max_stack());
  st->print_cr(" - max locals:        %d",   m->max_locals());
  st->print_cr(" - size of params:    %d",   m->size_of_parameters());
  st->print_cr(" - method size:       %d",   m->method_size());
  st->print_cr(" - vtable index:      %d",   m->vtable_index());
  st->print   (" - exceptions:        ");    m->exception_table()->print_value_on(st); st->cr();
  st->print_cr(" - code size:         %d",   m->code_size());
  st->print_cr(" - code start:        0x%x", m->code_base());
  st->print_cr(" - code end (excl):   0x%x", m->code_base() + m->code_size());
  st->print_cr(" - checked ex length: %d",   m->checked_exceptions_length());
  if (m->checked_exceptions_length() > 0) {
    CheckedExceptionElement* table = m->checked_exceptions_start();
    st->print_cr(" - checked ex start:  0x%x", table);
    if (Verbose) {
      for (int i = 0; i < m->checked_exceptions_length(); i++) {
        st->print_cr("   - throws %s", m->constants()->printable_name_at(table[i].class_cp_index));
      }
    }
  }
  if (m->has_linenumber_table()) {
    u_char* table = m->compressed_linenumber_table();
    st->print_cr(" - linenumber start:  0x%x", table);
    if (Verbose) {
      CompressedLineNumberReadStream stream(table);
      while (stream.read_pair()) {
        st->print_cr("   - line %d: %d", stream.line(), stream.bci());
      }
    }
  }
  st->print_cr(" - localvar length:   %d",   m->localvariable_table_length());
  if (m->localvariable_table_length() > 0) {
    LocalVariableTableElement* table = m->localvariable_table_start();
    st->print_cr(" - localvar start:    0x%x", table);
    if (Verbose) {
      for (int i = 0; i < m->localvariable_table_length(); i++) {
        int bci = table[i].start_bci;
        int len = table[i].length;
        const char* name = m->constants()->printable_name_at(table[i].name_cp_index);
        const char* desc = m->constants()->printable_name_at(table[i].descriptor_cp_index);
        int slot = table[i].slot;
        st->print_cr("   - %s %s bci=%d len=%d slot=%d", desc, name, bci, len, slot);
      }
    }
  }
#ifndef CORE
  if (m->code() != NULL) {
    st->print   (" - compiled code: ");
    m->code()->print_value_on(st);
    st->cr();
  }
#endif
}


void methodKlass::oop_print_value_on(oop obj, outputStream* st) {
  assert(obj->is_method(), "must be method");
  Klass::oop_print_value_on(obj, st);
  methodOop m = methodOop(obj);
  st->print(" ");
  m->name()->print_value_on(st);
  st->print(" ");
  m->signature()->print_value_on(st);
  st->print(" in ");
  m->method_holder()->print_value_on(st);
  if (WizardMode) st->print("[%d,%d]", m->size_of_parameters(), m->max_locals());
  if (WizardMode && m->code() != NULL) st->print(" ((nmethod*)%#x)", m->code());
}


const char* methodKlass::internal_name() const {
  return "{method}";
}


// Verification

void methodKlass::oop_verify_on(oop obj, outputStream* st) {
  Klass::oop_verify_on(obj, st);
  guarantee(obj->is_method(), "object must be method");
  methodOop m = methodOop(obj);
  guarantee(m->is_perm(),                                    "should be in permspace");
  guarantee(m->name()->is_perm(),                          "should be in permspace");
  guarantee(m->name()->is_symbol(),                        "should be symbol");
  guarantee(m->signature()->is_perm(),                     "should be in permspace");
  guarantee(m->signature()->is_symbol(),                   "should be symbol");
  guarantee(m->constants()->is_perm(),                     "should be in permspace");
  guarantee(m->constants()->is_constantPool(),             "should be constant pool");
  guarantee(m->exception_table()->is_perm(),               "should be in permspace");
  guarantee(m->exception_table()->is_typeArray(),          "should be type array");
  address m_end = (address)((oop*) m + m->size());
  address compressed_table_start = m->inlined_table_start();
  guarantee(compressed_table_start <= m_end, "invalid method layout");
  address compressed_table_end = compressed_table_start;
  // Verify line number table
  if (m->access_flags().has_linenumber_table()) {
    CompressedLineNumberReadStream stream(m->compressed_linenumber_table());
    while (stream.read_pair()) {
      guarantee(stream.bci() >= 0 && stream.bci() <= m->code_size(), "invalid bci in line number table");
    }
    compressed_table_end += stream.position();
  }
  guarantee(compressed_table_end <= m_end, "invalid method layout");
  // Verify checked exceptions and local variable tables
  if (m->access_flags().has_checked_exceptions()) {
    u2* addr = m->checked_exceptions_length_addr();
    guarantee(*addr > 0 && (address) addr >= compressed_table_end && (address) addr < m_end, "invalid method layout");
  }
  if (m->access_flags().has_localvariable_table()) {
    u2* addr = m->localvariable_table_length_addr();
    guarantee(*addr > 0 && (address) addr >= compressed_table_end && (address) addr < m_end, "invalid method layout");
  }
  // Check compressed_table_end relative to uncompressed_table_start
  u2* uncompressed_table_start;
  if (m->access_flags().has_localvariable_table()) {
    uncompressed_table_start = (u2*) m->localvariable_table_start();
  } else {
    if (m->access_flags().has_checked_exceptions()) {
      uncompressed_table_start = (u2*) m->checked_exceptions_start();
    } else {
      uncompressed_table_start = (u2*) m_end;
    }
  }
  int gap = (int) uncompressed_table_start - (int) compressed_table_end;
  int max_gap = align_object_size(1)*BytesPerWord;
  guarantee(gap >= 0 && gap < max_gap, "invalid method layout");
}

#endif // PRODUCT

