/*
 * @(#)methodOop.cpp	1.201 01/04/06 12:59:23
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_methodOop.cpp.incl"


// Implementation of methodOopDesc

int  methodOopDesc::fast_exception_handler_bci_for(KlassHandle ex_klass, int throw_bci, bool load_classes, TRAPS) {
  // exception table holds quadruple entries of the form (beg_bci, end_bci, handler_bci, klass_index)
  const int beg_bci_offset     = 0;
  const int end_bci_offset     = 1;
  const int handler_bci_offset = 2;
  const int klass_index_offset = 3;
  const int entry_size         = 4;
  // access exception table
  typeArrayHandle table = exception_table();
  int length = table->length();
  assert(length % entry_size == 0, "exception table format has changed");
  // iterate through all entries sequentially
  constantPoolHandle pool(constants());
  for (int i = 0; i < length; i += entry_size) {
    int beg_bci = table->int_at(i + beg_bci_offset);
    int end_bci = table->int_at(i + end_bci_offset);
    assert(beg_bci <= end_bci, "inconsistent exception table");
    if (beg_bci <= throw_bci && throw_bci < end_bci) {
      // exception handler bci range covers throw_bci => investigate further
      int handler_bci = table->int_at(i + handler_bci_offset);
      int klass_index = table->int_at(i + klass_index_offset);
      if (klass_index == 0) {
        return handler_bci;
      } else if (ex_klass == NULL) {
        return handler_bci;
      } else {
        // we know the exception class => get the constraint class
        KlassHandle klass;
        klass = constantPoolOopDesc::klass_at_if_loaded(pool, klass_index);
        if (klass.is_null() && load_classes) {
          // the class has not been loaded yet => load the class
          klass = pool->klass_at(klass_index, CHECK_(handler_bci));
        }
        if (klass.is_null()) {
          // we don't know the constraint class
          if (!load_classes) {
            return handler_bci;
          } else {
            // error happened during class loading
            // => assume constraint class is not supertype
            // => ignore this handler
          }
        } else if (ex_klass->is_subtype_of(klass())) {
          return handler_bci;
        }
      }
    }
  }

  return -1;
}

methodOop methodOopDesc::method_from_bcp(address bcp) {
  debug_only(static int count = 0; count++);
  assert(Universe::perm_gen()->contains((oop*)bcp), "bcp not in perm_gen");
  oop p = Universe::perm_gen()->fast_object_start((oop*)bcp);
  assert(p->is_method(), "not a method");
  return methodOop(p);
}


OopMapCacheEntry* methodOopDesc::mask_for(int bci) {
    
#ifdef ASSERT
  bool is_VM_thread = Thread::current()->is_VM_thread();
  if (!is_VM_thread) {
    if (!VerifyStack && !VerifyLastFrame) {
      // verify stack calls this outside VM thread
      warning("oopmap should only be accessed by the VM thread (or during debugging)");  
      OopMapCacheEntry* mask = instanceKlass::cast(method_holder())->mask_for(this, bci, true);
      mask->print();
    }
  }
#endif
  OopMapCacheEntry* mask = instanceKlass::cast(method_holder())->mask_for(this, bci, true);
  if (mask == NULL) {
    // This code should be removed. We can always pretend that we are in the middel of a VM 
    // operation during debugging. Also the 'true' argument to the above call should be removed.
    ShouldNotReachHere();
    // Mask wasn't cached and couldn't be computed because outside of a VM thread. Use
    // a VM operation to enforce a safepoint and compute the mask inside a VM thread.
    //
    // Note: With a re-entrant oop map generator and a lock around the oop map cache
    //       one could do this w/o explicit synchronization of all other threads.
    //       However, the explicit synchronization may slow down the common (GC) case.
    assert(!is_VM_thread, "mask should have been computed if in VM thread");
    VM_GenerateOopMap op(this, bci);
    VMThread::execute(&op);
    mask = op.mask();
  }
  return mask;
}


int methodOopDesc::bci_from(address bcp) const {  
  assert(is_native() && bcp == code_base() || contains(bcp), "bcp doesn't belong to this method");
  return bcp - code_base();
}


address methodOopDesc::bcp_from(int bci) const {
  assert(is_native() && bci == 0 || 0 <= bci && bci < code_size(), "illegal bci");
  address bcp = code_base() + bci;
  assert(is_native() && bcp == code_base() || contains(bcp), "bcp doesn't belong to this method");
  return bcp;
}


int methodOopDesc::object_size(int code_size, bool is_native, int compressed_line_number_size, int local_variable_table_length, int checked_exceptions_length) {
  int extra_bytes;
  if (is_native) {
    assert(code_size == 0, "native methods should not contain byte codes");
    extra_bytes = 2*sizeof(address*);  // pointers for native_function and signature_handler
  } else {
    extra_bytes = code_size;
  }
  if (compressed_line_number_size > 0) {
    extra_bytes += compressed_line_number_size;
  }
  if (checked_exceptions_length > 0) {
    extra_bytes += sizeof(u2) + (checked_exceptions_length * sizeof(CheckedExceptionElement));
  }
  if (local_variable_table_length > 0) {
    extra_bytes += sizeof(u2) + (local_variable_table_length * sizeof(LocalVariableTableElement));
  }
  int extra_words = align_size_up(extra_bytes, BytesPerWord) / BytesPerWord;
  return align_object_size(header_size() + extra_words);
}


void methodOopDesc::set_inlined_tables_length(int checked_exceptions_len, int compressed_line_number_size, int localvariable_table_len) {
  // Must be done in the order below, otherwise length_addr accessors will not work. Only set bit in header if length is positive.
  if (compressed_line_number_size > 0) {
    _access_flags.set_has_linenumber_table();
  }
  if (checked_exceptions_len > 0) {
    // Make sure that this is the first time setting the length, or otherwise that the length is unchanged (method rewriting)    
    _access_flags.set_has_checked_exceptions();
    *(checked_exceptions_length_addr()) = checked_exceptions_len;
  }
  if (localvariable_table_len > 0) {    
    _access_flags.set_has_localvariable_table();
    *(localvariable_table_length_addr()) = localvariable_table_len;
  }
}


int methodOopDesc::checked_exceptions_length() const {
  return _access_flags.has_checked_exceptions() ? *(checked_exceptions_length_addr()) : 0;
}


u2* methodOopDesc::checked_exceptions_length_addr() const {
  assert(_access_flags.has_checked_exceptions(), "should only be called if table is present");
  return last_u2_element();
}


CheckedExceptionElement* methodOopDesc::checked_exceptions_start() const {
  u2* addr = checked_exceptions_length_addr();
  u2 length = *addr;
  assert(length > 0, "should only be called if table is present");
  addr -= length * sizeof(CheckedExceptionElement) / sizeof(u2);
  return (CheckedExceptionElement*) addr;
}


u_char* methodOopDesc::compressed_linenumber_table() const {
  assert(has_linenumber_table(), "should only be called if table is present");
  return inlined_table_start();
}


int methodOopDesc::localvariable_table_length() const {
  return _access_flags.has_localvariable_table() ? *(localvariable_table_length_addr()) : 0;
}


u2* methodOopDesc::localvariable_table_length_addr() const {
  assert(_access_flags.has_localvariable_table(), "should only be called if table is present");
  if (_access_flags.has_checked_exceptions()) {
    return (u2*) checked_exceptions_start() - 1;  // if checked exceptions are present the localvariable table length is just before them
  } else {
    return last_u2_element(); // if no line number are present the linenumber table is last
  }
}


LocalVariableTableElement* methodOopDesc::localvariable_table_start() const {
  u2* addr = localvariable_table_length_addr();
  u2 length = *addr;
  assert(length > 0, "should only be called if table is present");
  addr -= length * sizeof(LocalVariableTableElement) / sizeof(u2);
  return (LocalVariableTableElement*) addr;
}


symbolOop methodOopDesc::klass_name() const {
  klassOop k = method_holder();
  assert(k->is_klass(), "must be klass");
  instanceKlass* ik = (instanceKlass*) k->klass_part();
  return ik->name();
}


#ifndef CORE

bool methodOopDesc::was_executed_more_than(int n) const {
  // Invocation counter is reset when the methodOop is compiled.
  // If the method has compiled code we therefore assume it has
  // be excuted more than n times.
  if (is_accessor() || is_empty_method() || (code() != NULL)) {
    // interpreter doesn't bump invocation counter of trivial methods
    // compiler does not bump invocation counter of compiled methods
    return true;
  } else if (_invocation_counter.carry()) {
    // The carry bit is set when the counter overflows and causes
    // a compilation to occur.  We don't know how many times
    // the counter has been reset, so we simply assume it has
    // been executed more than n times.
    return true;
  } else {
    return invocation_count() > n;
  }
}

#ifndef PRODUCT
void methodOopDesc::print_invocation_count() const {
  tty->print ("%8d ", invocation_count());
  tty->print ("%8d ", compiled_invocation_count());
  if (is_static()) tty->print("static ");
  if (is_final()) tty->print("final ");
  if (is_synchronized()) tty->print("synchronized ");
  if (is_native()) tty->print("native ");
  name()->print_value();
  tty->print(" ");
  tty->fill_to(48);
  method_holder()->print_value();
  if (WizardMode) {
    // dump the size of the byte codes
    tty->print(" {%d}", code_size());
  }
  tty->cr();
}
#endif
#endif


void methodOopDesc::cleanup_inline_caches() {
  // The current system doesn't use inline caches in the interpreter
  // => nothing to do (keep this method around for future use)
}


void methodOopDesc::compute_size_of_parameters() {
  ArgumentSizeComputer asc(signature());
  set_size_of_parameters(asc.size() + (is_static() ? 0 : 1));
}


BasicType methodOopDesc::result_type() const {
  ResultTypeFinder rtf(signature());
  return rtf.type();
}


bool methodOopDesc::is_empty_method() const {
  return  code_size() == 1
      && *code_base() == Bytecodes::_return;
}


bool methodOopDesc::is_vanilla_constructor() const {
  // Returns true if this method is a vanilla constructor, i.e. an "<init>" "()V" method
  // which only calls the superclass vanilla constructor and possibly does stores of
  // zero constants to local fields:
  //
  //   aload_0
  //   invokespecial
  //   indexbyte1
  //   indexbyte2
  // 
  // followed by an (optional) sequence of:
  //
  //   aload_0
  //   aconst_null / iconst_0 / fconst_0 / dconst_0
  //   putfield
  //   indexbyte1
  //   indexbyte2
  //
  // followed by:
  //
  //   return

  assert(name() == vmSymbols::object_initializer_name(),    "Should only be called for default constructors");
  assert(signature() == vmSymbols::void_method_signature(), "Should only be called for default constructors");
  int size = code_size();
  // Check if size match
  if (size == 0 || size % 5 != 0) return false;
  address cb = code_base();
  int last = size - 1;
  if (cb[0] != Bytecodes::_aload_0 || cb[1] != Bytecodes::_invokespecial || cb[last] != Bytecodes::_return) {
    // Does not call superclass default constructor
    return false;
  }
  // Check optional sequence
  for (int i = 4; i < last; i += 5) {
    if (cb[i] != Bytecodes::_aload_0) return false;
    if (!Bytecodes::is_zero_const(Bytecodes::cast(cb[i+1]))) return false;
    if (cb[i+2] != Bytecodes::_putfield) return false;
  }
  return true;
}


bool methodOopDesc::compute_has_loops_flag() {  
  BytecodeStream bcs(this);
  Bytecodes::Code bc;
    
  while ((bc = bcs.next()) >= 0) {    
    switch( bc ) {        
      case Bytecodes::_ifeq: 
      case Bytecodes::_ifnull: 
      case Bytecodes::_iflt: 
      case Bytecodes::_ifle: 
      case Bytecodes::_ifne: 
      case Bytecodes::_ifnonnull: 
      case Bytecodes::_ifgt: 
      case Bytecodes::_ifge: 
      case Bytecodes::_if_icmpeq: 
      case Bytecodes::_if_icmpne: 
      case Bytecodes::_if_icmplt: 
      case Bytecodes::_if_icmpgt: 
      case Bytecodes::_if_icmple: 
      case Bytecodes::_if_icmpge: 
      case Bytecodes::_if_acmpeq:
      case Bytecodes::_if_acmpne:
      case Bytecodes::_goto: 
      case Bytecodes::_jsr:     
        if( bcs.dest() < bcs.next_bci() ) _access_flags.set_has_loops();
        break;

      case Bytecodes::_goto_w:       
      case Bytecodes::_jsr_w:        
        if( bcs.dest_w() < bcs.next_bci() ) _access_flags.set_has_loops(); 
        break;
    }  
  }
  _access_flags.set_loops_flag_init();
  return _access_flags.has_loops(); 
}


bool methodOopDesc::is_final_method() const {
  return is_final() || Klass::cast(method_holder())->is_final();
}


bool methodOopDesc::is_strict_method() const {
  return is_strict();
}


bool methodOopDesc::is_accessor() const {
  if (code_size() != 5) return false;
  if (size_of_parameters() != 1) return false;
  if (Bytecodes::java_code(Bytecodes::cast(code_base()[0])) != Bytecodes::_aload_0 ) return false;
  if (Bytecodes::java_code(Bytecodes::cast(code_base()[1])) != Bytecodes::_getfield) return false;
  if (Bytecodes::java_code(Bytecodes::cast(code_base()[4])) != Bytecodes::_areturn &&
      Bytecodes::java_code(Bytecodes::cast(code_base()[4])) != Bytecodes::_ireturn ) return false;
  return true;
}


bool methodOopDesc::is_initializer() const {
  return name() == vmSymbols::object_initializer_name() || name() == vmSymbols::class_initializer_name();
}


objArrayHandle methodOopDesc::resolved_checked_exceptions_impl(methodOop this_oop, TRAPS) {
  int length = this_oop->checked_exceptions_length();
  if (length == 0) {
    return Universe::the_empty_class_klass_array();  // common case
  } else {
    methodHandle h_this(THREAD, this_oop);
    objArrayOop array = oopFactory::new_objArray(SystemDictionary::class_klass(), length, CHECK_(objArrayOop(NULL)));
    objArrayHandle mirrors(THREAD, array);
    for (int i = 0; i < length; i++) {
      CheckedExceptionElement* table = h_this->checked_exceptions_start(); // recompute on each iteration, not gc safe
      klassOop k = h_this->constants()->klass_at(table[i].class_cp_index, CHECK_(objArrayOop(NULL)));
      assert(Klass::cast(k)->is_subclass_of(SystemDictionary::throwable_klass()), "invalid exception class");
      mirrors->obj_at_put(i, Klass::cast(k)->java_mirror());
    }
    return mirrors;
  }
};


int methodOopDesc::line_number_from_bci(int bci) const {
  NOT_CORE(if (bci == SynchronizationEntryBCI) bci = 0;)
  assert (bci == 0 || (bci > 0 && bci < code_size()), "illegal bci");
  int best_bci  =  0;
  int best_line = -1;

  if (has_linenumber_table()) {
    // The line numbers are a short array of 2-tuples [start_pc, line_number].
    // Not necessarily sorted and not necessarily one-to-one.
    CompressedLineNumberReadStream stream(compressed_linenumber_table());
    while (stream.read_pair()) {
      if (stream.bci() == bci) {
        // perfect match
        return stream.line();
      } else {
        // update best_bci/line
        if (stream.bci() < bci && stream.bci() >= best_bci) {
          best_bci  = stream.bci();
          best_line = stream.line();
        }
      }
    }
  }
  return best_line;
}


bool methodOopDesc::is_klass_loaded_by_klass_index(int klass_index) const {
  if( _constants->tag_at(klass_index).is_unresolved_klass() ) {
    symbolOop klass_name = _constants->unresolved_klass_at(klass_index);
    oop loader = instanceKlass::cast(method_holder())->class_loader();
    return SystemDictionary::find(klass_name, loader, Klass::cast(method_holder())->protection_domain()) != NULL;
  } else {
    return true;
  }
}


bool methodOopDesc::is_klass_loaded(int refinfo_index, bool must_be_resolved) const {
  int klass_index = _constants->klass_ref_index_at(refinfo_index);
  if (must_be_resolved) {
    // Make sure klass is resolved in constantpool.   
    if (constants()->tag_at(klass_index).is_unresolved_klass()) return false;
  }  
  return is_klass_loaded_by_klass_index(klass_index);
}


void methodOopDesc::set_native_function(address function) {
  address* native_function = native_function_addr();
  assert((address)native_function < method_end(), "just checking");
  *native_function = function;
#ifndef CORE
  // This function can be called more than once. We must make sure that we always
  // use the latest registered method -> check if a stub already has been generated.
  // If so, we have to make it not_entrant.
  nmethod* nm = code(); // Put it into local variable to guard against concurrent updates
  if (nm != NULL) {
    nm->make_not_entrant();
  }  
#endif
}


void methodOopDesc::set_signature_handler(address handler) { 
  address* signature_handler =  signature_handler_addr();
  assert((address)signature_handler < method_end(), "just checking");
  *signature_handler = handler;
}


bool methodOopDesc::is_not_compilable() const {
#ifndef CORE
  return _invocation_counter.state() == InvocationCounter::wait_for_nothing;
#else
  return false;
#endif
}

// call this when compiler finds that this method is not compilable
void methodOopDesc::set_not_compilable() {
#ifndef CORE  
  invocation_counter()->set_state(InvocationCounter::wait_for_nothing);
#endif
}

// This is called before the access flags have been set, i.e.,during method allocation
// The _from_compiled_code_entry_point is updated when a class is linked  
void methodOopDesc::init_code() {
  _code = NULL;  
  NOT_CORE(_from_compiled_code_entry_point = NULL;)
}

// Called when the method_holder is getting linked. Setup entrypoints so the method
// is ready to be called from interpreter, compiler, and vtables.
void methodOopDesc::link_method() {
  NOT_CORE(assert(_from_compiled_code_entry_point == NULL, "should only be called once");)
  assert(_interpreter_entry              == NULL, "should only be called once");

  // Setup interpreter entrypoint
  address entry = AbstractInterpreter::entry_for_method(this);
  assert(entry != NULL, "interpreter entry must be non-null");
  set_interpreter_entry(entry);

  // Setup compiler entrypoint. Default entry is lazy, so we do
  // not eagerly create I2C adapters.
  update_compiled_code_entry_point(true);
}

// Returns verifyed code entry, and updates compiler entrypoint to be eagerly
// resolved. The verified_code_entry() must be called when a invoke is resolved
// on this method
#ifndef CORE
address methodOopDesc::verified_code_entry() {
  update_compiled_code_entry_point(false);
  assert(_from_compiled_code_entry_point != NULL, "must be set");
  return _from_compiled_code_entry_point;
}
#endif

#ifndef CORE
// Check that if an nmethod ref exists, it has a backlink to this.
// Not inline to avoid circular ref.
bool methodOopDesc::check_code() const {
  return !_code || _code->method() == this; 
}
#endif

// Install compiled code for this nmethod
void methodOopDesc::set_code(nmethod* code) {
#ifdef COMPILER2
  NOT_CORE(assert(code == NULL || !code->is_osr_method(), "osr code should not be used here");)
#endif
  _code = code;
  // Update compiler entrypoint
  update_compiled_code_entry_point(true);
}

void methodOopDesc::update_compiled_code_entry_point(bool lazy) {
#ifndef CORE
  if (_code != NULL) {
    _from_compiled_code_entry_point = _code->verified_entry_point();
    return;
  }

  if (is_abstract()) {
    COMPILER1_ONLY(_from_compiled_code_entry_point = Runtime1::entry_for(Runtime1::throw_abstract_method_error_id);)
    COMPILER2_ONLY(_from_compiled_code_entry_point = OptoRuntime::handle_abstract_method_stub();)
    return;
  }  

#ifdef COMPILER1       
  // There is no lazy entrypoints for Compiler1 since it does not
  // use adapter frames.
  methodHandle m(methodOop(this));
  _from_compiled_code_entry_point = is_static()
    ? Runtime1::ientries_for(m)->static_call_entry()
    : Runtime1::ientries_for(m)->optimized_call_entry();    
#endif

#ifdef COMPILER2
  methodHandle m(methodOop(this));  
  if (lazy) {  
    m->_from_compiled_code_entry_point = C2IAdapterGenerator::lazy_std_verified_entry(m);
  } else {
    m->_from_compiled_code_entry_point = C2IAdapterGenerator::std_verified_entry(m);
  }
#endif
#endif
}


bool methodOopDesc::is_overridden_in(klassOop k) const {
  instanceKlass* ik = instanceKlass::cast(k);

  if (ik->is_interface()) return false;

  // If method is an interface, we skip it - except if it
  // is a miranda method
  if (instanceKlass::cast(method_holder())->is_interface()) {
    // Check that method is not a miranda method
    if (ik->lookup_method(name(), signature()) == NULL) {
      // No implementation exist - so miranda method
      return false;
    }
    return true;
  }  

  assert(ik->is_subclass_of(method_holder()), "should be subklass");
  ResourceMark rm;
  assert(ik->vtable() != NULL, "vtable should exist");
  if (_vtable_index == -1) {
    return false;
  } else {
    methodOop vt_m = ik->vtable()->method_at(_vtable_index);
    return vt_m != this;
  }
}


methodHandle methodOopDesc:: clone_with_new_data(methodHandle m, u_char* new_code, int new_code_length, 
                                                u_char* new_compressed_linenumber_table, int new_compressed_linenumber_size, TRAPS) {
  // Code below does not work for native methods - they should never get rewritten anyway
  assert(!m->is_native(), "cannot rewrite native methods");
  // Allocate new methodOop
  AccessFlags flags = m->access_flags();
  int checked_exceptions_len = m->checked_exceptions_length();
  int localvariable_len = m->localvariable_table_length();
  methodHandle newm = oopFactory::new_method(new_code_length, flags, new_compressed_linenumber_size, localvariable_len, checked_exceptions_len, CHECK_(methodOop(NULL)));
  int new_method_size = newm->method_size();
  // Create a shallow copy of methodOopDesc part
  memcpy(newm(), m(), sizeof(methodOopDesc));
  // Reset correct bytecode method size, method size, and parameter info
  newm->set_code_size(new_code_length);
  newm->set_parameter_info(0);
  newm->set_method_size(new_method_size);
  newm->set_parameter_info(0);
  // Copy new byte codes
  memcpy(newm->code_base(), new_code, new_code_length);
  // Copy line number table
  if (new_compressed_linenumber_size > 0) {
    memcpy(newm->compressed_linenumber_table(), new_compressed_linenumber_table, new_compressed_linenumber_size);
  }
  // Copy checked_exceptions
  if (checked_exceptions_len > 0) {
    memcpy(newm->checked_exceptions_start(), m->checked_exceptions_start(), checked_exceptions_len * sizeof(CheckedExceptionElement));
  }
  // Copy local variable number table
  if (localvariable_len > 0) {
    memcpy(newm->localvariable_table_start(), m->localvariable_table_start(), localvariable_len * sizeof(LocalVariableTableElement));
  }
  return newm;
}


JNIid* methodOopDesc::jni_id() {
  instanceKlass* ik = instanceKlass::cast(method_holder());
  objArrayOop methods = ik->methods();
  int len = methods->length();
  int index = 0; // Defined outside of for loop for portability
  for (; index < len; index++) {
    if (methods->obj_at(index) == this) {
      break;
    }
  }
  assert(index<len, "should have found method");
  JNIid* id = ik->jni_id_for(index);
  debug_only(id->set_is_method_id();)
  return id;
}


methodOopDesc::IntrinsicId methodOopDesc::intrinsic_id() const {
  // if loader is not the default loader (i.e., != NULL), we can't know the intrinsics
  // because we are not loading from core libraries
  if (instanceKlass::cast(method_holder())->class_loader() != NULL) return _none;
  // return intrinsic id if any
  symbolOop klass_name = instanceKlass::cast(method_holder())->name();
  if (klass_name == vmSymbols::java_lang_Object() && !is_static() && !is_synchronized()) {
    if (is_same(vmSymbols::java_lang_Object_hashCode_Name (), vmSymbols::java_lang_Object_hashCode_Signature ())) return _hash;
  } else if (klass_name == vmSymbols::java_lang_Math() && is_static() && !is_synchronized()) {
    if (is_same(vmSymbols::java_lang_Math_sin_Name        (), vmSymbols::java_lang_Math_sin_Signature        ())) return _dsin;
    if (is_same(vmSymbols::java_lang_Math_cos_Name        (), vmSymbols::java_lang_Math_cos_Signature        ())) return _dcos;
    if (is_same(vmSymbols::java_lang_Math_sqrt_Name       (), vmSymbols::java_lang_Math_sqrt_Signature       ())) return _dsqrt;
  } else if (klass_name == vmSymbols::java_lang_System() && is_static() && !is_synchronized()) {
    if (is_same(vmSymbols::java_lang_System_arraycopy_Name(), vmSymbols::java_lang_System_arraycopy_Signature())) return _arraycopy;
  } else if (klass_name == vmSymbols::java_lang_String() && !is_static() && !is_synchronized()) {
    if (is_same(vmSymbols::compareTo_name()                 , vmSymbols::string_int_signature                ())) return _compareTo;
  }
  return _none;
}


#ifndef CORE
// These two methods are static since a GC may move the methodOopDesc
bool methodOopDesc::load_signature_classes(methodHandle m, TRAPS) {
  bool sig_is_loaded = true;
  Handle class_loader(THREAD, instanceKlass::cast(m->method_holder())->class_loader());
  Handle protection_domain(THREAD, Klass::cast(m->method_holder())->protection_domain());
  for(SignatureStream ss(m->signature()); !ss.is_done(); ss.next()) {
    if (ss.is_object()) {
      symbolHandle name = ss.as_symbol(CHECK_(false));
      klassOop klass = SystemDictionary::resolve_or_null(name, class_loader, protection_domain, CHECK_(false));
      if( klass == NULL ) { sig_is_loaded = false; }
    }
  }
  return sig_is_loaded;
}

bool methodOopDesc::has_unloaded_classes_in_signature(methodHandle m, TRAPS) {
  Handle class_loader(THREAD, instanceKlass::cast(m->method_holder())->class_loader());
  Handle protection_domain(THREAD, Klass::cast(m->method_holder())->protection_domain());
  for(SignatureStream ss(m->signature()); !ss.is_done(); ss.next()) {
    if (ss.type() == T_OBJECT) {
      symbolHandle name(THREAD, ss.as_symbol_or_null());
      if (name() == NULL) return true;   
      klassOop klass = SystemDictionary::find(name, class_loader, protection_domain);
      if (klass == NULL) return true;
    }
  }
  return false;
}
#endif // not CORE


// Exposed so field engineers can debug VM
void methodOopDesc::print_short_name(outputStream* st) {
  ResourceMark rm;
#ifdef PRODUCT
  st->print(" %s::", method_holder()->klass_part()->external_name());
#else
  st->print(" %s::", method_holder()->klass_part()->internal_name());
#endif
  name()->print_symbol_on(st);
  if (WizardMode) signature()->print_symbol_on(st);
}


//-----------------------------------------------------------------------------------
// Non-product code

#ifndef PRODUCT
class SignatureTypePrinter : public SignatureTypeNames {
 private:
  outputStream* _st;
  bool _use_separator;

  void type_name(const char* name) {
    if (_use_separator) _st->print(", ");
    _st->print(name);
    _use_separator = true;
  }

 public:
  SignatureTypePrinter(symbolOop signature, outputStream* st) : SignatureTypeNames(signature) {
    _st = st;
    _use_separator = false;
  }

  void print_parameters()              { _use_separator = false; iterate_parameters(); }
  void print_returntype()              { _use_separator = false; iterate_returntype(); }
};


void methodOopDesc::print_name(outputStream* st) {
  SignatureTypePrinter sig(signature(), st);
  st->print("%s ", is_static() ? "static" : "virtual");
  sig.print_returntype();
  st->print(" %s.", method_holder()->klass_part()->internal_name());
  name()->print_symbol_on(st);
  st->print("(");
  sig.print_parameters();
  st->print(")");
}


void methodOopDesc::print_codes() const {
  ResourceMark rm;
  if (UseNewStylePrintCodes) {
    BytecodeStream s((methodOop)this);
    while (s.next() >= 0) BytecodeTracer::std_closure()->trace((methodOop)this, s.bcp());
  } else {
    MethodPrinterClosure p;
    MethodIterator iter((methodOop)this);
    iter.iterate(&p);
  }
}


void methodOopDesc::alloc_bci_histogram() { 
  if (!PrintBCIHistogram) {
    _bci_histogram = NULL;
    return;
  }

  _bci_histogram = (int*) os::malloc(code_size() * sizeof(int));
  for (int i = 0;  i < code_size();  ++i)
    _bci_histogram[i] = 0;
}
#endif // not PRODUCT


// Simple compression of line number tables. We use a regular compressed stream, except that we compress deltas
// between (bci,line) pairs since they are smaller. If (bci delta, line delta) fits in (5-bit unsigned, 3-bit unsigned)
// we save it as one byte, otherwise we write a 0xFF escape character and use regular compression. 0x0 is used
// as end-of-stream terminator.


CompressedLineNumberWriteStream::CompressedLineNumberWriteStream(int initial_size) : CompressedWriteStream(initial_size) {
  _bci = 0;
  _line = 0;
};


void CompressedLineNumberWriteStream::write_pair(int bci, int line) {
  int bci_delta = bci - _bci;
  int line_delta = line - _line;
  _bci = bci;
  _line = line;
  // Skip (0,0) deltas - they do not add information and conflict with terminator.
  if (bci_delta == 0 && line_delta == 0) return;
  // Check if bci is 5-bit and line number 3-bit unsigned.
  if (((bci_delta & ~0x1F) == 0) && ((line_delta & ~0x7) == 0)) {
    // Compress into single byte.
    jubyte value = ((jubyte) bci_delta << 3) | (jubyte) line_delta;
    // Check that value doesn't match escape character.
    if (value != 0xFF) {
      write_byte(value);
      return;
    }
  }
  // bci and line number does not compress into single byte.
  // Write out escape character and use regular compression for bci and line number.
  write_byte((jubyte)0xFF);
  write_int(bci_delta);
  write_int(line_delta);
}


CompressedLineNumberReadStream::CompressedLineNumberReadStream(u_char* buffer) : CompressedReadStream(buffer) {
  _bci = 0;
  _line = 0;
};


bool CompressedLineNumberReadStream::read_pair() {
  jubyte next = read_byte();
  // Check for terminator
  if (next == 0) return false;
  if (next == 0xFF) {
    // Escape character, regular compression used
    _bci  += read_int();
    _line += read_int();
  } else {
    // Single byte compression used
    _bci  += next >> 3;
    _line += next & 0x7;
  }
  return true;
}


/// Fast Breakpoints.

// If this structure gets more complicated (because bpts get numerous),
// move it into its own header.

// There is presently no provision for concurrent access
// to breakpoint lists, which is only OK for JVMDI because
// breakpoints are written only at safepoints, and are read
// concurrently only outside of safepoints.

class BreakpointInfo : public CHeapObj {
 private:
  Bytecodes::Code  _orig_bytecode;
  int              _bci;
  u2               _name_index;       // of method
  u2               _signature_index;  // of method
  BreakpointInfo*  _next;             // simple storage allocation

 public:
  BreakpointInfo(methodOop m, int bci) {
    _bci = bci;
    _name_index = m->name_index();
    _signature_index = m->signature_index();
    _orig_bytecode = (Bytecodes::Code) *m->bcp_from(_bci);
    if (_orig_bytecode == Bytecodes::_breakpoint)
      _orig_bytecode = m->orig_bytecode_at(_bci);
    _next = NULL;
  }

  // accessors
  Bytecodes::Code orig_bytecode()                     { return _orig_bytecode; }
  void        set_orig_bytecode(Bytecodes::Code code) { _orig_bytecode = code; }

  BreakpointInfo*          next() const               { return _next; }
  void                 set_next(BreakpointInfo* n)    { _next = n; }

  // helps for searchers
  bool match(methodOop m, int bci) {
    return bci == _bci && match(m);
  }

  bool match(methodOop m) {
    return _name_index == m->name_index() &&
      _signature_index == m->signature_index();
  }

  void set(methodOop method) {
#ifdef ASSERT
    {
      Bytecodes::Code code = (Bytecodes::Code) *method->bcp_from(_bci);
      if (code == Bytecodes::_breakpoint)
        code = method->orig_bytecode_at(_bci);
      assert(orig_bytecode() == code, "original bytecode must be the same");
    }
#endif
    *method->bcp_from(_bci) = Bytecodes::_breakpoint;
  }

  void clear(methodOop method) {
    *method->bcp_from(_bci) = orig_bytecode();
  }
};


Bytecodes::Code methodOopDesc::orig_bytecode_at(int bci) {
  BreakpointInfo* bp = instanceKlass::cast(method_holder())->breakpoints();
  for (; bp != NULL; bp = bp->next()) {
    if (bp->match(this, bci)) {
      return bp->orig_bytecode();
    }
  }
  ShouldNotReachHere();
  return Bytecodes::_shouldnotreachhere;
}

void methodOopDesc::set_orig_bytecode_at(int bci, Bytecodes::Code code) {
  assert(code != Bytecodes::_breakpoint, "cannot patch breakpoints this way");
  BreakpointInfo* bp = instanceKlass::cast(method_holder())->breakpoints();
  for (; bp != NULL; bp = bp->next()) {
    if (bp->match(this, bci)) {
      bp->set_orig_bytecode(code);
      // and continue, in case there is more than one
    }
  }
}

void methodOopDesc::set_breakpoint(int bci) {
  instanceKlass* ik = instanceKlass::cast(method_holder());
  BreakpointInfo *bp = new BreakpointInfo(this, bci);
  bp->set_next(ik->breakpoints());
  ik->set_breakpoints(bp);
  // do this last:
  bp->set(this);
}

static void clear_matches(methodOop m, int bci) {
  instanceKlass* ik = instanceKlass::cast(m->method_holder());
  BreakpointInfo* prev_bp = NULL;
  BreakpointInfo* next_bp;
  for (BreakpointInfo* bp = ik->breakpoints(); bp != NULL; bp = next_bp) {
    next_bp = bp->next();
    if (bci >= 0 ? bp->match(m, bci) : bp->match(m)) {
      // do this first:
      bp->clear(m);
      // unhook it
      if (prev_bp != NULL)
        prev_bp->set_next(next_bp);
      else
        ik->set_breakpoints(next_bp);
      delete bp;
    } else {
      // This one is a keeper.
      prev_bp = bp;
    }
  }
}

void methodOopDesc::clear_breakpoint(int bci) {
  assert(bci >= 0, "");
  clear_matches(this, bci);
}

void methodOopDesc::clear_all_breakpoints() {
  clear_matches(this, -1);
}
