/*
 * @(#)oop.cpp	1.72 99/12/07 10:24:51
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_oop.cpp.incl"


#ifdef PRODUCT
void oopDesc::print_on(outputStream* st) const {}
void oopDesc::print_value_on(outputStream* st) const {}
void oopDesc::print_address_on(outputStream* st) const {}
char* oopDesc::print_value_string() { return NULL; }
char* oopDesc::print_string() { return NULL; }
void oopDesc::print()         {}
void oopDesc::print_value()   {}
void oopDesc::print_address() {}
void oopDesc::verify() {}
void oopDesc::verify_old_oop(oop* p, bool allow_dirty) {}
#else
void oopDesc::print_on(outputStream* st) const {
  if (this == NULL) {
    st->print_cr("NULL");
  } else {
    blueprint()->oop_print_on(oop(this), st);
  }
}

void oopDesc::print_value_on(outputStream* st) const {
  oop obj = oop(this);
  if (this == NULL) {
    st->print("NULL");
  } else if (java_lang_String::is_instance(obj)) {
    java_lang_String::print(obj, st);
    if (PrintOopAddress) print_address_on(st);
#ifdef ASSERT
  } else if (!Universe::is_heap(obj) || !Universe::is_heap(klass())) {
    st->print("### BAD OOP %#x ###", obj);
#endif
  } else {
    blueprint()->oop_print_value_on(obj, st);
  }
}

void oopDesc::print_address_on(outputStream* st) const {
  if (PrintOopAddress) {
    st->print("{");
    if (PrintOopAddress) {
      st->print("0x%lx", this);
    }
    st->print("}");
  }
}

void oopDesc::print()         { print_on(tty);         }

void oopDesc::print_value()   { print_value_on(tty);   }

void oopDesc::print_address() { print_address_on(tty); }

char* oopDesc::print_string() { 
  stringStream* st = new stringStream();
  print_on(st);
  return st->as_string();
}

char* oopDesc::print_value_string() { 
  stringStream* st = new stringStream();
  print_value_on(st);
  return st->as_string();
}


void oopDesc::verify_on(outputStream* st) {
  if (this != NULL) {
    blueprint()->oop_verify_on(this, st);
  }
}


void oopDesc::verify() { 
  verify_on(tty); 
}


void oopDesc::verify_old_oop(oop* p, bool allow_dirty) {
  blueprint()->oop_verify_old_oop(this, p, allow_dirty);
}


bool oopDesc::partially_loaded() {
  return blueprint()->oop_partially_loaded(this);
}


void oopDesc::set_partially_loaded() {
  blueprint()->oop_set_partially_loaded(this);
}
#endif // PRODUCT


int oopDesc::slow_identity_hash() {
  // slow case; we have to acquire the micro lock in order to locate the header  
  ResetNoHandleMark rnm; // Might be called from LEAF/QUICK ENTRY
  HandleMark hm;
  Handle object(this);
  return ObjectSynchronizer::identity_hash_value_for(object);
}


void oopDesc::oop_iterate(OopClosure* blk) {
  blueprint()->oop_oop_iterate(this, blk);
}


oop oopDesc::copy_to_survivor_space(oop* from) {
  assert(Scavenge::should_scavenge(this) && !is_forwarded(), "shouldn't be scavenging this oop"); 
  int s = size();
  oop obj = NULL;
  bool obj_is_tenured = false;
  
  // Try allocating obj in to-space (unless too old or won't fit or JVMPI enabled)
  if (age() < Scavenge::tenuring_threshold() && !Universe::jvmpi_alloc_event_enabled()) {
    obj = (oop) Universe::new_gen()->to()->safepoint_allocate(s);
  }

  // Otherwise try allocating obj tenured
  if (obj == NULL) {
    if (is_typeArray() && s >= Universe::large_typearray_limit()) {
      obj = (oop) Universe::old_gen()->safepoint_allocate_large_typearray(s);
    } else {
      if (UseTrainGC) {
        Train* t = from == NULL ? NULL : CarTable::train_for(oop(from));
        if (t == NULL) {
          obj = (oop) Universe::train_gen()->safepoint_allocate(s);
        } else {
          obj = (oop) (t->safepoint_allocate(s));
        }
        Scavenge::add_total_promoted(s);
      } else {
        obj = (oop) Universe::tenured_gen()->safepoint_allocate(s);
      }
    }
    obj_is_tenured = true;
  }
  assert(obj != NULL, "allocation should have succeeded");

  if (Universe::jvmpi_move_event_enabled()) {
    Universe::jvmpi_object_move(this, obj);
  }

  // Copy obj
  Memory::copy_oops_aligned((oop*) this, (oop*) obj, s);

  debug_only(if (TraceScavenge)
    tty->print_cr("{%s %s 0x%x -> 0x%x (%d)}", obj->is_new() ? "copying" : "tenuring", obj->blueprint()->internal_name(), this, obj, s));

  // Increment age if obj still in new generation
  if (!obj_is_tenured) { 
    obj->incr_age(); 
    Scavenge::age_table()->add(obj, s); 
  }
  // Done, insert forward pointer to obj in this header
  forward_to(obj);
  return obj;
}


oop oopDesc::copy_to_train(oop* from) {
  assert(Scavenge::should_scavenge(this) && !is_forwarded(), "shouldn't be scavenging this oop"); 
  assert(Universe::train_gen()->first_train()->first_car()->contains(this), "should be in first car");
  int s = size();
  // Figure out where to copy to
  Train* t = from == NULL ? NULL : CarTable::train_for(oop(from));
  if (t == NULL) t = Universe::train_gen()->last_train();

  // Allocate obj
  oop obj = (oop) (t->safepoint_allocate(s));
  assert(obj != NULL, "allocation should have succeeded");

  // Copy obj
  assert(!Universe::train_gen()->first_train()->first_car()->contains(obj), "shouldn't copy to same car");
  Memory::copy_oops_aligned((oop*) this, (oop*) obj, s);

  debug_only(if (TraceScavenge)
    tty->print_cr("{copying %s %x -> 0x%x (%d)}", obj->blueprint()->internal_name(), this, obj, s));

  // Done, insert forward pointer to obj in this header
  forward_to(obj);
  return obj;
}
