/*
 * @(#)oop.hpp	1.60 01/02/09 14:45:40
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// oopDesc is the top baseclass for objects classes.  The {name}Desc classes describe
// the format of Java objects so the fields can be accessed from C++.
// oopDesc is abstract.
// (see oopHierarchy for complete oop class hierarchy)
//
// no virtual functions allowed

// store into oop with store check
inline void oop_store(oop* p, oop v);

// store into oop without store check
inline void oop_store_without_check(oop* p, oop v);

class oopDesc {
  friend class VMStructs;
 private:
  markOop  _mark;
  klassOop _klass;

 public:
  markOop  mark() const         { return _mark; }
  markOop* mark_addr() const    { return (markOop*) &_mark; }

  void set_mark()               { init_mark();	        }
  void set_mark_speclock()      { init_mark_speclock(); }
  void set_mark(markOop m)      { _mark = m;	        }
  void set_mark(oop p)	        { set_mark(markOop(p)); }
  void set_mark(oop* p)         { set_mark(markOop(p)); }
  inline void set_mark_atomic(markOop m);

  inline void init_mark();
  inline void init_mark_speclock();

  klassOop klass() const        { return _klass; }
  oop* klass_addr() const       { return (oop*) &_klass; }

  inline void set_klass(klassOop k);

  // size of object header
  static int header_size()      { return sizeof(oopDesc)/wordSize; }

  inline Klass* blueprint() const;

  // Returns whether this is an instance of k or an instance of a subclass of k
  inline bool is_a(klassOop k)  const;

  // Returns the actual oop size of the object
  inline int size();

  // type test operations (inlined in oop.inline.h)
  inline bool is_instance()           const;
  inline bool is_instanceRef()        const;
  inline bool is_array()              const;
  inline bool is_objArray()	      const;
  inline bool is_symbol()             const;
  inline bool is_klass()              const;
  inline bool is_thread()	      const;
  inline bool is_method()             const;
  inline bool is_constantPool()       const;
  inline bool is_constantPoolCache()  const;
  inline bool is_typeArray()          const;
  inline bool is_compiledICHolder()   const;

 private:
  // field addresses in oop
  // byte/char/bool/short fields are always stored as full words
  inline void*     field_base(int offset)        const;
  inline jint*     byte_field_addr(int offset)   const;
  inline jint*     char_field_addr(int offset)   const;
  inline jint*     bool_field_addr(int offset)   const;
  inline jint*     int_field_addr(int offset)    const;
  inline jint*     short_field_addr(int offset)  const;
  inline jlong*    long_field_addr(int offset)   const;
  inline jfloat*   float_field_addr(int offset)  const;
  inline jdouble*  double_field_addr(int offset) const;

 public:
  // need this as public for garbage collection
  inline oop* obj_field_addr(int offset) const;

  inline oop obj_field(int offset) const;
  inline void obj_field_put(int offset, oop value);

  inline jbyte byte_field(int offset) const;
  inline void byte_field_put(int offset, jbyte contents);

  inline jchar char_field(int offset) const;
  inline void char_field_put(int offset, jchar contents);

  inline jboolean bool_field(int offset) const;
  inline void bool_field_put(int offset, jboolean contents);

  inline jint int_field(int offset) const;
  inline void int_field_put(int offset, jint contents);

  inline jshort short_field(int offset) const;
  inline void short_field_put(int offset, jshort contents);

  inline jlong long_field(int offset) const;
  inline void long_field_put(int offset, jlong contents);

  inline jfloat float_field(int offset) const;
  inline void float_field_put(int offset, jfloat contents);

  inline jdouble double_field(int offset) const;
  inline void double_field_put(int offset, jdouble contents);

  // printing functions for VM debugging
  void print_on(outputStream* st) const;         // First level print 
  void print_value_on(outputStream* st) const;   // Second level print.
  void print_address_on(outputStream* st) const; // Address printing

  // printing on default output stream
  void print();
  void print_value();
  void print_address();

  // return the print strings
  char* print_string();
  char* print_value_string();

  // verification operations
  void verify_on(outputStream* st);
  void verify();
  void verify_old_oop(oop* p, bool allow_dirty);

  // tells whether this oop is partially constructed (gc during class loading)
  bool partially_loaded();
  void set_partially_loaded();

  // locking operations
  inline bool is_locked()   const;
  inline bool is_unlocked() const;
  inline bool is_not_unlocked() const; // locked or busy

  // asserts
#ifndef PRODUCT
  inline bool is_oop() const;
  inline bool safepoint_is_oop() const;
  inline bool safepoint_is_oop_or_null() const;
  inline bool is_unlocked_oop() const;
  inline bool is_oop_or_link() const;
#endif

  // garbage collection
  inline bool is_gc_marked() const;
  inline void follow_contents();
  inline void follow_header();
  inline bool being_unloaded();

  inline bool is_new() const;
  inline bool is_old() const;
  inline bool is_perm() const;
  inline bool is_old_or_perm() const;

  // Forward pointer operations for scavenge
  inline bool is_forwarded() const;
  inline void forward_to(oop p);
  inline oop forwardee() const;

  // Age of object during scavenge
  inline int age() const;
  inline void incr_age();

  // Scavenge all pointers in this object (using Scavenge::scavenge) 
  // and return the oop size
  inline int scavenge_contents();
  // Scavenge all pointers in this object (using Scavenge::scavenge)
  // and return the oop size. Only oops between bottom (inclusive) and top (exclusive)
  // are considered.
  inline int scavenge_contents(oop* bottom, oop* top);

  // Adjust all pointers in this object to point at it's forwarded location and
  // return the size of this oop.  This is used by the MarkSweep collector.
  inline int adjust_pointers();
  inline void adjust_header();

  // Copy this object to survivor space and return the new address (called by scavenge)
  oop copy_to_survivor_space(oop* from);
  oop copy_to_train(oop* from);

  // mark-sweep support
  inline void follow_body(int begin, int end);

  // iterators
  void oop_iterate(OopClosure* blk);
  inline void oop_iterate_header(OopClosure* blk);

  // identity hash; returns the identity hash key (computes it if necessary)
  inline int identity_hash();
  int slow_identity_hash();

  // marks are forwarded to stack when object is locked
  inline bool     has_displaced_mark() const;
  inline markOop  displaced_mark() const;
  inline void     set_displaced_mark(markOop m);

  // for code generation
  static int klass_offset_in_bytes()   { return (int)&(oop(NULL)->_klass); }
  static int mark_offset_in_bytes()    { return (int)&(oop(NULL)->_mark); }
};

