/*
 * @(#)oop.inline.hpp	1.70 00/08/03 15:04:47
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Implementation of all inlined member functions defined in oop.hpp
// We need a separate file to avoid circular references


inline void oopDesc::set_mark_atomic(markOop m) {
  atomic::store_ptr((intptr_t*) &_mark, (intptr_t) m);
}

inline void oopDesc::set_klass(klassOop k) {
  // since klasses are promoted no store check is needed
  assert(Universe::is_bootstrapping() || k != NULL, "must be a real klassOop");
  assert(Universe::is_bootstrapping() || k->is_klass(), "not a klassOop");
  oop_store_without_check((oop*) &_klass, (oop) k);
}

inline void oopDesc::init_mark()              { set_mark(markOopDesc::prototype()); }
inline void oopDesc::init_mark_speclock()     { set_mark(markOopDesc::prototype_speclock()); }
inline Klass* oopDesc::blueprint() 	     const { return klass()->klass_part(); }

inline bool oopDesc::is_a(klassOop k)        const { return blueprint()->is_subtype_of(k); }

inline bool oopDesc::is_instance()	     const { return blueprint()->oop_is_instance(); }
inline bool oopDesc::is_instanceRef()        const { return blueprint()->oop_is_instanceRef(); }
inline bool oopDesc::is_array()		     const { return blueprint()->oop_is_array(); }
inline bool oopDesc::is_objArray()	     const { return blueprint()->oop_is_objArray(); }
inline bool oopDesc::is_typeArray()	     const { return blueprint()->oop_is_typeArray(); }
inline bool oopDesc::is_symbol()	     const { return blueprint()->oop_is_symbol(); }
inline bool oopDesc::is_klass()		     const { return blueprint()->oop_is_klass(); }
inline bool oopDesc::is_thread()	     const { return blueprint()->oop_is_thread(); }
inline bool oopDesc::is_method()	     const { return blueprint()->oop_is_method(); }
inline bool oopDesc::is_constantPool()	     const { return blueprint()->oop_is_constantPool(); }
inline bool oopDesc::is_constantPoolCache()  const { return blueprint()->oop_is_constantPoolCache(); }
inline bool oopDesc::is_compiledICHolder()   const { return blueprint()->oop_is_compiledICHolder(); }

// Note that byte/char/bool/short fields are always stored as full words
inline void*     oopDesc::field_base(int offset)        const { return (void*)&((jint*)this)[offset]; }
inline oop*      oopDesc::obj_field_addr(int offset)    const { return (oop*)     field_base(offset); }
inline jint*     oopDesc::byte_field_addr(int offset)   const { return (jint*)    field_base(offset); }
inline jint*     oopDesc::char_field_addr(int offset)   const { return (jint*)    field_base(offset); }
inline jint*     oopDesc::bool_field_addr(int offset)   const { return (jint*)    field_base(offset); }
inline jint*     oopDesc::int_field_addr(int offset)    const { return (jint*)    field_base(offset); }
inline jint*     oopDesc::short_field_addr(int offset)  const { return (jint*)    field_base(offset); }
inline jlong*    oopDesc::long_field_addr(int offset)   const { return (jlong*)   field_base(offset); }
inline jfloat*   oopDesc::float_field_addr(int offset)  const { return (jfloat*)  field_base(offset); }
inline jdouble*  oopDesc::double_field_addr(int offset) const { return (jdouble*) field_base(offset); }

inline oop oopDesc::obj_field(int offset) const                     { return *obj_field_addr(offset);             }
inline void oopDesc::obj_field_put(int offset, oop value)           { oop_store(obj_field_addr(offset), value);   }

inline jbyte oopDesc::byte_field(int offset) const                  { return (jbyte) *byte_field_addr(offset);    }
inline void oopDesc::byte_field_put(int offset, jbyte contents)     { *byte_field_addr(offset) = (jint) contents; }

inline jboolean oopDesc::bool_field(int offset) const               { return (jboolean) *bool_field_addr(offset); }
inline void oopDesc::bool_field_put(int offset, jboolean contents)  { *bool_field_addr(offset) = (jint) contents; }

inline jchar oopDesc::char_field(int offset) const                  { return (jchar) *char_field_addr(offset);    }
inline void oopDesc::char_field_put(int offset, jchar contents)     { *char_field_addr(offset) = (jint) contents; }

inline jint oopDesc::int_field(int offset) const                    { return *int_field_addr(offset);        }
inline void oopDesc::int_field_put(int offset, jint contents)       { *int_field_addr(offset) = contents;    }

inline jshort oopDesc::short_field(int offset) const                { return (jshort) *short_field_addr(offset);  }
inline void oopDesc::short_field_put(int offset, jshort contents)   { *short_field_addr(offset) = (jint) contents;}

inline jlong oopDesc::long_field(int offset) const                  { return *long_field_addr(offset);       }
inline void oopDesc::long_field_put(int offset, jlong contents)     { *long_field_addr(offset) = contents;   }

inline jfloat oopDesc::float_field(int offset) const                { return *float_field_addr(offset);      }
inline void oopDesc::float_field_put(int offset, jfloat contents)   { *float_field_addr(offset) = contents;  }

inline jdouble oopDesc::double_field(int offset) const              { return *double_field_addr(offset);     }
inline void oopDesc::double_field_put(int offset, jdouble contents) { *double_field_addr(offset) = contents; }


inline int oopDesc::size()  {
  Klass* klass = blueprint();
  int s = klass->size_helper();

  // s is now a value computed at class initialization that may hint
  // at the size.  For instances, this is positive and equal to the
  // size.  For arrays, this is negative and one minus log2 of the
  // array element size.  For other oops, it is zero and thus requires
  // a virtual call.
  //
  // We go to all this trouble because the size computation is at the
  // heart of phase 2 of mark-compaction, and called for every object,
  // alive or dead.  So the speed here is equal in importance to the
  // speed of allocation.

  if (s <= 0) {    // The most common case is instances; fall through if so.
    if (s < 0) {
      // Second most common case is arrays.  We have to fetch the
      // length of the array, shift (multiply) it appropriately, 
      // up to wordSize, add the header, and align to object size.
      s = ((arrayOop)this)->length() << (-1-s);
      s += ((arrayKlass*)klass)->array_header_in_bytes();

      // This code could be simplified, but by keeping array_header_in_bytes
      // in units of bytes and doing it this way we can round up just once,
      // skipping the intermediate round to wordSize.
      if (ALIGN_ALL_OBJECTS) {
	s = ((s + (wordSize*2-1)) & ~(wordSize*2-1)) / wordSize;
      } else {
	s = (s + (wordSize-1)) / wordSize;
      }
      assert(s == klass->oop_size(this), "wrong array object size");
    } else {
      // Must be zero, so bite the bullet and take the virtual call.
      s = klass->oop_size(this);
    }
  }

  assert(!ALIGN_ALL_OBJECTS || s % 2 == 0, "alignment check");
  return s;
}


inline void oop_store(oop* p, oop v) {
  *p = v;
  if (v != NULL && !v->is_perm() && !oop(p)->is_new()) {
    RememberedSet::record_runtime_store(p, v);
  }
}


inline void oop_store_without_check(oop* p, oop v) {
  assert(v == NULL || v->is_perm() || oop(p)->is_new(), "oop store without store check failed");
  *p = v;
}


// Used only for markSweep, scavenging, train
inline bool oopDesc::is_gc_marked() const {
  return mark()->is_marked();
}

inline bool oopDesc::is_locked() const {
  return mark()->is_locked();
}

inline bool oopDesc::is_unlocked() const {
  return mark()->is_unlocked();
}

// Can be locked or busy
inline bool oopDesc::is_not_unlocked() const    { return !mark()->is_unlocked(); }


#ifndef PRODUCT

// used only for asserts
inline bool oopDesc::is_oop_or_link() const { 
  return markOop(this)->assert_not_real_mark(); 
}


inline bool check_obj_alignment(oop obj) {
  int alignment_size = ALIGN_ALL_OBJECTS ? (2*oopSize) : oopSize;
  return (intptr_t)obj % alignment_size == 0;
}


// used only for asserts
inline bool oopDesc::is_oop() const {
  oop obj = (oop) this;
  if (!check_obj_alignment(obj)) return false;
  if (!Universe::is_heap(obj)) return false;
  // obj is aligned and accessible in heap
  // try to find metaclass cycle safely without seg faulting on bad input
  // we should reach klassKlassObj by following klass link at most 3 times
  for (int i = 0; i < 3; i++) {
    obj = obj->klass();
    // klass should be aligned and in permspace
    if (!check_obj_alignment(obj)) return false;
    if (!Universe::perm_gen()->the_space()->contains(obj)) return false;
  }
  if (obj != Universe::klassKlassObj()) return false;
  // Don't try to verify header, it might be changing  
  return true;
}


// used only for asserts
inline bool oopDesc::safepoint_is_oop() const {
  if (!is_oop()) return false;
  // Try to verify header
  if (mark() == NULL) return false;
  return has_displaced_mark() ? displaced_mark()->assert_real_mark() : mark()->assert_real_mark();
}


// used only for asserts
inline bool oopDesc::safepoint_is_oop_or_null() const {
  return this == NULL ? true : safepoint_is_oop();
    
}


// used only for asserts
inline bool oopDesc::is_unlocked_oop() const {
  if (!Universe::is_reserved_heap((oop) this)) return false;
  markOop m = mark();
  return m->is_locked() ? false : m->assert_real_mark();
}


#endif // PRODUCT

inline void oopDesc::follow_header() { 
  MarkSweep::mark_and_push((oop*)&_klass);
}


inline void oopDesc::follow_contents() {
  assert (is_gc_marked(), "should be marked");
  blueprint()->oop_follow_contents(this);
}


inline bool oopDesc::being_unloaded() {
  // Tells whether we should discard this object in order not to 
  // prevent a klass from being unloaded.
  return is_gc_marked() ? false : blueprint()->oop_being_unloaded(this);
}


// Used by scavenger and train GC:

inline bool oopDesc::is_forwarded() const { 
  // The extra heap check is needed since the obj might be locked, in which case the
  // mark would point to a stack location and have the sentinel bit cleared
  return mark()->is_marked();
}


// Used by scavenger and train gc
inline void oopDesc::forward_to(oop p) {
  assert(Universe::is_reserved_heap(p), "forwarding to something not in heap");
  markOop m = markOopDesc::encode_pointer_as_mark(p);
  assert(m->has_pointer()  &&  m->decode_pointer() == p,
	 "mark must be distinguishable and encoding reversable");
  set_mark(m);
}


// Note that the forwardee is not the same thing as the displaced_mark.
// The forwardee is used when copying on scavenge.  It does not need to worry
// about clearing the waiters bit of the lock bits (not used for locking).

// But, it does need to worry about clearing the busy lock bits for
// those platforms that use the (low-order) lock bits instead of the
// high-order sentinal bit.
inline oop oopDesc::forwardee() const           { return (oop) mark()->decode_pointer(); }


inline bool oopDesc::has_displaced_mark() const {
  return !is_unlocked();
}

inline markOop oopDesc::displaced_mark() const {
  return mark()->displaced_mark_helper();
}

inline void oopDesc::set_displaced_mark(markOop m) {
  mark()->set_displaced_mark_helper(m);
}

inline int oopDesc::age() const {
  if (has_displaced_mark()) {
    return displaced_mark()->age();
  } else {
    return mark()->age();
  }
}


inline void oopDesc::incr_age() {
  if (has_displaced_mark()) {
    set_displaced_mark(displaced_mark()->incr_age());
  } else {
    set_mark(mark()->incr_age());
  }
}


inline int oopDesc::identity_hash() {
  // Fast case; if the object is unlocked and the hash value is set, no locking is needed
  // Note: The mark must be read into local variable to avoid concurrent updates.
  markOop mrk = mark();
  return (mrk->is_unlocked() && !mrk->has_no_hash()) ? mrk->hash() : slow_identity_hash();
}


inline bool oopDesc::is_new() const { 
  return (oop*)this < Universe::new_gen()->high_boundary;
}


inline bool oopDesc::is_old() const { 
  return (oop*)this >= Universe::old_gen()->low_boundary && (oop*)this < Universe::old_gen()->high_boundary;
}


inline bool oopDesc::is_perm() const { 
  return (oop*)this >= Universe::perm_gen()->low_boundary && (oop*)this < Universe::perm_gen()->high_boundary;
}


inline bool oopDesc::is_old_or_perm() const { 
  return (oop*)this >= Universe::old_gen()->low_boundary && (oop*)this < Universe::perm_gen()->high_boundary;
}


inline void oopDesc::oop_iterate_header(OopClosure* blk) {
  blk->do_oop((oop*)&_klass);
}


inline int oopDesc::scavenge_contents() {
  return blueprint()->oop_scavenge_contents(this);
}


inline int oopDesc::scavenge_contents(oop* bottom, oop* top) {
  return blueprint()->oop_scavenge_contents(this, bottom, top);
}


inline int oopDesc::adjust_pointers() {
  debug_only(int check_size = size());
  int s = blueprint()->oop_adjust_pointers(this);
  assert(s == check_size, "should be the same");
  return s;
}


inline void oopDesc::adjust_header() {
  MarkSweep::adjust_pointer((oop*)&_klass);
}

