/*
 * @(#)symbolKlass.cpp	1.29 00/03/21 16:17:25
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_symbolKlass.cpp.incl"

symbolOop symbolKlass::allocate_symbol(u1* name, int len, TRAPS) {
  int size = symbolOopDesc::object_size(len);
  symbolKlassHandle h_k(THREAD, as_klassOop());
  symbolOop sym = (symbolOop)Universe::allocate_permanent(h_k, size, CHECK_0);
  sym->set_next(NULL);
  sym->set_utf8_length(len);
  for (int i = 0; i < len; i++) {
    sym->byte_at_put(i, name[i]);
  }
  return sym;
}

klassOop symbolKlass::create_klass(TRAPS) {
  symbolKlass o;
  KlassHandle h_this_klass(THREAD, Universe::klassKlassObj());
  KlassHandle k = base_create_klass(h_this_klass, header_size(), o.vtbl_value(), CHECK_0);
  // Make sure size calculation is right
  assert(k()->size() == align_object_size(header_size()), "wrong size for object");
//  java_lang_Class::create_mirror(k, CHECK_0); // Allocate mirror
  return k();
}

int symbolKlass::oop_size(oop obj) const { 
  assert (obj->is_symbol(),"must be a symbol");
  symbolOop t = symbolOop(obj);
  int size = symbolOopDesc::object_size(t->utf8_length());
  return size; 
}


void symbolKlass::oop_follow_contents(oop obj) {
  assert (obj->is_symbol(), "object must be symbol");
  obj->follow_header();
  // Note: do not follow next link here (see SymbolTable::follow_contents)
}


void symbolKlass::oop_oop_iterate(oop obj, OopClosure* blk) {
  assert (obj->is_symbol(), "object must be symbol");
  obj->oop_iterate_header(blk);
  blk->do_oop((oop*) symbolOop(obj)->next_addr());
}


int symbolKlass::oop_scavenge_contents(oop obj) {
  assert(obj->is_symbol(), "should be symbol");
  return oop_size(obj);
}


int symbolKlass::oop_scavenge_contents(oop obj, oop* bottom, oop* top) {
  assert(obj->is_symbol(), "should be symbol");
  return oop_size(obj);
}


int symbolKlass::oop_adjust_pointers(oop obj) {
  assert(obj->is_symbol(), "should be symbol");
  int size = oop_size(obj);
  obj->adjust_header();
  MarkSweep::adjust_pointer((oop*) symbolOop(obj)->next_addr());
  return size;
}


#ifndef PRODUCT
// Printing

void symbolKlass::oop_print_on(oop obj, outputStream* st) {
  st->print("Symbol: '");
  symbolOop sym = symbolOop(obj);
  for (int i = 0; i < sym->utf8_length(); i++) {
    st->print("%c", sym->byte_at(i));
  }
  st->print("'");
}

void symbolKlass::oop_print_value_on(oop obj, outputStream* st) {
  symbolOop sym = symbolOop(obj);
  st->print("'");
  for (int i = 0; i < sym->utf8_length(); i++) {
    st->print("%c", sym->byte_at(i));
  }
  st->print("'");
}

const char* symbolKlass::internal_name() const {
  return "{symbol}";
}
#endif //PRODUCT
