/*
 * @(#)bytecodeInfo.cpp	1.77 01/02/09 14:45:51
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_bytecodeInfo.cpp.incl"

// These variables are declared in parse1.cpp
extern int  explicit_null_checks_inserted;
extern int  explicit_null_checks_elided;
extern int  explicit_null_checks_inserted_old;
extern int  explicit_null_checks_elided_old;
extern int  methods_parsed_old;
extern int  methods_parsed;
extern int  methods_seen;
extern int  methods_seen_old;


//=============================================================================
//------------------------------InlineTree-------------------------------------
InlineTree::InlineTree( ciEnv* env, ciMethod* m, int caller_bci, float site_invoke_ratio ) : _env(env), _method(m), _caller_bci(caller_bci), _site_invoke_ratio(site_invoke_ratio) {
  // Array of CFG predecessor counts.  This needs to go into an
  // abstract data structure as well.  This array and the following
  // ones are all given 1 extra padding word of space to allow for the
  // case where the bytecodes have some unreachable code that falls off
  // the end.  The first pass will end up incrementing the predecessor
  // code of a mythical next bytecode 1 past the end of the code_size.
  _preds = NEW_RESOURCE_ARRAY(int16, _method->code_size()+1);
  memset(_preds, 0, sizeof(int16) *  _method->code_size()  );
  // Array of bit-flags, one per bytecode.
  // 00001111 - 4 bits for opcode BasicType
  // 00110000 - 2 bits for result size (0, 1, 2)
  // 01000000 - 1 bit  for merge points
  // 10000000 - 1 bit  for cloning blocks
  _bits = NEW_RESOURCE_ARRAY(uint8, _method->code_size()+1);
  memset(_bits, 0, sizeof(uint8) *  _method->code_size()  );
  // Number of arguments consumed
  _args = NEW_RESOURCE_ARRAY(uint8, _method->code_size()+1);

  _cnt_inlines = _cnt_inline_bcs = 0;

  _scope = NULL;
}

//------------------------------osr_copy---------------------------------------
// Make a top-level copy for OSR purposes.  No inlining in the OSR parse.
InlineTree *InlineTree::osr_copy() const {
  InlineTree *ilt = new InlineTree( _env, _method, _caller_bci, _site_invoke_ratio );
  memcpy(ilt->_preds, _preds, sizeof(int16) * _method->code_size());
  memcpy(ilt->_bits , _bits , sizeof(uint8) * _method->code_size());
  memcpy(ilt->_args , _args , sizeof(uint8) * _method->code_size());
  ilt->_scope = _scope;
  return ilt;
}

//------------------------------deep_copy--------------------------------------
// Deep copy fields for cloning whilst inlining
InlineTree *InlineTree::deep_copy() const {
  InlineTree *ilt = osr_copy();
  for( int i = 0; i < _subtrees.length(); i++ ) 
    ilt->_subtrees.append( _subtrees.at(i)->deep_copy() );
  return ilt;
}

//------------------------------inline_policy----------------------------------
// Build an InlineTree, given an inline decision tree.
void InlineTree::inline_policy( ciScope *scope ) {
  if (UseInlineImage || DumpInlineImage)  _scope = scope;
  // We will attempt to see if a class/field/etc got properly loaded.  If it
  // did not, it may attempt to throw an exception during our probing.  Catch
  // and ignore such exceptions and do not attempt to compile the method.
  EXCEPTION_MARK;
  for( int i = 0; i < scope->number_of_subscopes(); i++ ) {
    // Create all the subtrees of an InlineTree by inspecting the
    // current inline decisions.
    ciScope* sub = scope->subscope_at(i);

    // No inlining here today, please
    if( sub->method()->should_exclude() )
      continue;

    // Called method invocation counts
    int invcnt = 1;
    if( CountInterpInvocations ) {
      invcnt = sub->method()->interpreter_invocation_count();
      // if this method has not been seen yet just set value to one
      if( invcnt == 0 ) invcnt = 1;
    }
    // Call site invocation counts
    int cnt = 0;
    if( CountInterpCalls ) {
      cnt = scope->method()->interpreter_call_site_count(sub->caller_bci());
      // Could be a GC has cleared the table, or that there are fewer
      // than the cutoff for counting
      if( !cnt ) cnt = 1;
    }
    float freq = (float)cnt/(float)invcnt;
    // Call-site count / interpreter invocation count, scaled recursively.
    // Always between 0.0 and 1.0.  Represents the percentage of the method's
    // total execution time used at this call site.
    float recur_freq = freq * _site_invoke_ratio;

    // Attempt inlining; gather _preds, _args, _bits info
    InlineTree *ilt = new InlineTree( _env, sub->method(), sub->caller_bci(), recur_freq );
    // Fill in the _preds, _args and _bits fields.
    // Check for bailout conditions (generally unloadable classes).
    if( ilt->find_block_starts( 0, THREAD ) == Parse::Bailout_none ) {  
      _subtrees.append( ilt );
      ilt->inline_policy( sub );
      _cnt_inlines += ilt->_cnt_inlines + 1;
      _cnt_inline_bcs += ilt->_cnt_inline_bcs + ilt->_method->code_size();
    } else {                    // Clear potential exception
      CLEAR_PENDING_EXCEPTION;  
    } 
  }
}

//------------------------------subtree_at_bci---------------------------------
InlineTree *InlineTree::subtree_at_bci( uint bci ) const {
  // Could be binary search, but who cares?
  for( int i = 0; i < _subtrees.length(); i++ ) {
    InlineTree *ilt = _subtrees.at(i);
    if( ilt->_caller_bci == (int)bci )
      return ilt;
  }
  return NULL;
}

//------------------------------set_type_and_size------------------------------
void InlineTree::set_type_and_size(int bci, Bytecodes::Code bc,BasicType type){
  bits_type_set(bci,type);
  char x = type2size[type];
  bits_size_set(bci,x);
  _args[bci] = x - Bytecodes::depth(bc);
}

//------------------------------find_block_starts------------------------------
Parse::BailoutConstants InlineTree::find_block_starts( uint entry_bci, TRAPS) {

  Compile* compile = Compile::current();

  // Exception handlers require loading the exception klass from the oop
  if( _method->has_exception_handlers() )
    compile->set_alias_index( TypeOopPtr::HEADER ); 

  _preds[entry_bci]++;          // entry point has additional predecessor

  // Check all the bytecodes
  ciByteCodeStream i(_method);
  Bytecodes::Code bc;  

  while( (bc=i.next()) != ciByteCodeStream::EOBC ) {
    int cur_bci = i.cur_bci();
    // By default, assume we fall into the next bytecode.
    // Bytecodes that do NOT fall into the next path will fixup.
    _preds[i.next_bci()]++;     // Fall into next bytecode
    BasicType t = Bytecodes::result_type(bc);
    if (t == T_ILLEGAL) t = T_VOID; // type will be fixed later
    assert (t > 0, "");
    bits_type_set(cur_bci,t);
    char x = type2size[t];
    bits_size_set(cur_bci,x);
    _args[cur_bci] = x - Bytecodes::depth(bc);    

    switch( bc ) {
    case Bytecodes::_invokeinterface: // Fall through
    case Bytecodes::_invokevirtual:   // Fall through
    case Bytecodes::_invokespecial:   // Fall through
    case Bytecodes::_invokestatic: {
      // If there is a local exception handler, inlined methods 
      // get their thrown exception headers inspected.
      if( _method->has_exception_handlers() )
        compile->set_alias_index( TypeOopPtr::HEADER );
      // Source of 2 exceptions: One for the receiver NULL check, one
      // for the call itself
      ciMethod* callee = i.get_method();
      // Check for unloaded classes
      if( !callee->holder()->is_loaded() ||
	  // Also check for uninitialized static calls
	  (bc == Bytecodes::_invokestatic && 
	   !callee->holder()->is_initialized()) ) {
	_preds[i.next_bci()]--;
	bits_type_set(cur_bci,0); // Flag for pass 2
      } else if (!callee->will_link(_method->holder(), 
				    i.get_declared_method_holder(), bc)) {
        return Parse::Bailout_exception_thrown;
      } else {
	int sig_arg_size = callee->arg_size();

        // Keep track of argument size for final regmask bailout
        if ( compile->_max_arg_size < sig_arg_size )
          compile->_max_arg_size = sig_arg_size;
        // Check if too many arguments for register representation
        // Note: >>1 reflects the fact that adapters will need space
        //       on the stack for both incoming and outgoing args.
        // NOTE: -1 in computation reflects the usage of the last
        //       bit of the regmask as an infinite stack flag.
        int old_SP = SharedInfo::stack2reg( round_to( compile->in_preserve_stack_slots() + (compile->_sync * compile->sync_stack_slots()), WordsPerLong ) );
        if( sig_arg_size >= (int)(((RM_SIZE*32 - (old_SP + (2 * compile->out_preserve_stack_slots())))>>1)-1) )
          return Parse::Bailout_too_many_args;

	_args[cur_bci] = sig_arg_size;	
        BasicType rettype = callee->return_type();
        bits_type_set(cur_bci, rettype);
        bits_size_set(cur_bci, type2size[rettype]);
        if (SpecialStringCompareTo) {
          if (callee->equals(compile->get_String_compareTo())) {
            // We will be inlining a specialized version of String.compareTo().
            // Define the memory types which we will need.
            
	    ciInstanceKlass* klass = _env->String_klass();
            const TypeOopPtr* string_type =
              TypeOopPtr::make(TypePtr::BotPTR, klass, NULL, 0);
            
            const int value_offset = java_lang_String::value_offset_in_bytes();
            const int count_offset = java_lang_String::count_offset_in_bytes();
            const int offset_offset = java_lang_String::offset_offset_in_bytes();
            
            compile->set_alias_index(TypeAryPtr::CHARS);
            compile->set_alias_index(string_type->add_offset(value_offset));
            compile->set_alias_index(string_type->add_offset(count_offset));
            compile->set_alias_index(string_type->add_offset(offset_offset));
          }
        }
        if( InlineArrayCopy ) {
          ciMethod* m = compile->get_System_arraycopy();
          if (m != NULL && callee->equals(m)) {
            compile->set_alias_index( TypeOopPtr::HEADER ); 
            compile->set_alias_index(TypeAryPtr::RANGE);
          }
        }
        if( InlineObjectHash ) {
          if (callee->equals(compile->get_Object_hashCode()) ||
	      callee->equals(compile->get_System_identityHashCode())) {
            compile->set_alias_index( TypeOopPtr::BOTTOM ); // mark of hash'd object
            compile->set_alias_index( TypeOopPtr::HEADER ); // klass of hash'd object
          }
        }
      }
    }
    break;

    case Bytecodes::_multianewarray: {
      // Also must init all fields
      ciArrayKlass* klass = i.get_klass()->as_array_klass();
      // uncommon_trap if unloaded or uninitialized
      if( klass->is_loaded() ) {
        int ndim = i.get_dimensions();
        if (ndim > 5)
          return Parse::Bailout_unhandled_bytecode;
        _args[cur_bci] = ndim;    // Uses dimension elements from stack
      } else {
	bits_type_set(cur_bci,0); // Flag for pass 2
      }
      break;
    }


    case Bytecodes::_goto_w: 
      _preds[i.next_bci()]--;   // Will never hit the next bytecode
      _preds[i.get_far_dest()]++;
      if( i.get_far_dest() < cur_bci ) compile->_has_loops = true;
      break;

    // Jump to here
    case Bytecodes::_goto:   
      _preds[i.next_bci()]--;   // Will never hit the next bytecode
    // Fall thru
    case Bytecodes::_ifnull: 
    case Bytecodes::_ifnonnull: 
    case Bytecodes::_if_acmpeq:
    case Bytecodes::_if_acmpne:
    case Bytecodes::_ifeq: 
    case Bytecodes::_iflt: 
    case Bytecodes::_ifle: 
    case Bytecodes::_ifne: 
    case Bytecodes::_ifgt: 
    case Bytecodes::_ifge: 
    case Bytecodes::_if_icmpeq: 
    case Bytecodes::_if_icmpne: 
    case Bytecodes::_if_icmplt: 
    case Bytecodes::_if_icmpgt: 
    case Bytecodes::_if_icmple: 
    case Bytecodes::_if_icmpge: 
      // Already bumped the _next_ bytecode, just need to 
      // bump the branch target.
      _preds[i.get_dest()]++;
      if( i.get_dest() < cur_bci ) compile->_has_loops = true;
      break;

    case Bytecodes::_jsr:
    case Bytecodes::_jsr_w: {
      // Not sure why Bytecodes:: has this as T_INT
      bits_type_set( cur_bci, T_ADDRESS );
      int target_bci = (bc==Bytecodes::_jsr) ? i.get_dest() : i.get_far_dest();
      _preds[target_bci]++;

      Bytecodes::Code bc_target = _method->java_code_at_bci(target_bci);

      if (!Bytecodes::is_astore(bc_target))
        return Parse::Bailout_unhandled_bytecode;
      break; }

    case Bytecodes::_ret: 
      _preds[i.next_bci()]--;   // Will never hit the next bytecode
      // Treated as return - nothing to do 
      break;      

    case Bytecodes::_tableswitch: {
      _preds[i.next_bci()]--;   // Will never hit the next bytecode
      int default_dest = i.get_dest_table(0);
      if (SwitchDecisionTrees)
	_preds[default_dest] += 2;   // Up to two paths to default case
      else
	_preds[default_dest]++;      // One path to default case
      int len = i.get_tableswitch_length();
      for( int j = 0; j < len; j++ ) 
        _preds[i.get_dest_table(j+3)]++;
      break;
    }

    case Bytecodes::_fast_binaryswitch:
    case Bytecodes::_fast_linearswitch:
    case Bytecodes::_lookupswitch: { // fall through
      _preds[i.next_bci()]--;   // Will never hit the next bytecode
      int default_dest = i.get_dest_table(0);
      int len = i.get_int_table(1);
      if (SwitchDecisionTrees)
	_preds[default_dest] += len+1;  // Up to N+1 paths to default case
      else
	_preds[default_dest]++;         // One path to default case
      for( int j = 0; j < len; j++ ) 
        _preds[i.get_dest_table(2+j+j+1)]++;
      break;
    }

    case Bytecodes::_getstatic: 
    case Bytecodes::_getfield: {       
      ciField* field = i.get_field();
      ciInstanceKlass* holder = field->holder();
      if( holder->is_loaded() && 
	  (bc == Bytecodes::_getfield || holder->is_initialized()) ) {
        if (!field->will_link(_method->holder(), bc))
          // We bailout if there is a problem in link resolution.
          return Parse::Bailout_exception_thrown;
        t = field->field_type();
	uint offset = field->offset() * wordSize;
	const TypeJavaPtr *tjp = (bc == Bytecodes::_getfield)
	  ? (const TypeJavaPtr *) TypeOopPtr::make( TypePtr::NotNull, holder, offset )
	  : (const TypeJavaPtr *) TypeKlassPtr::make( holder, offset );
        if( t == T_LONG )       // Longs have other word unaliased
          compile->set_alias_index( tjp->add_offset(wordSize) );
	compile->set_alias_index( tjp );
        bits_type_set( cur_bci, t );
        bits_size_set( cur_bci, type2size[t] );
      } else {
	bits_type_set(cur_bci,0); // Flag for pass 2
      }
      break;
    }

    case Bytecodes::_putstatic: 
    case Bytecodes::_putfield: {       
      ciField* field = i.get_field();
      ciInstanceKlass* holder = field->holder();
      if( holder->is_loaded() &&
	  (bc == Bytecodes::_putfield || holder->is_initialized()) ) {
        if (!field->will_link(_method->holder(), bc))
          // We bailout if there is a problem in link resolution.
          return Parse::Bailout_exception_thrown;
        t = field->field_type();
	uint offset = field->offset() * wordSize;
	const TypeJavaPtr *tjp = (bc == Bytecodes::_putfield)
	  ? (const TypeJavaPtr *) TypeOopPtr::make( TypePtr::NotNull, holder, offset )
	  : (const TypeJavaPtr *) TypeKlassPtr::make( holder, offset );
        if( t == T_LONG )       // Longs have other word unaliased
          compile->set_alias_index( tjp->add_offset(wordSize) );
	compile->set_alias_index( tjp );
        bits_type_set( cur_bci, t );
        bits_size_set( cur_bci, 0 );
        _args[cur_bci] = type2size[t]-1-Bytecodes::depth(bc);
        if( t == T_OBJECT ||       // Store barrier    
            t == T_ARRAY )
          compile->set_alias_index( TypeRawPtr::BOTTOM );
      } else {
	bits_type_set(cur_bci,0); // Flag for pass 2
      }
      break;
    }

    case Bytecodes::_athrow:
    case Bytecodes::_return:
    case Bytecodes::_ireturn:
    case Bytecodes::_areturn:
    case Bytecodes::_dreturn:
    case Bytecodes::_lreturn:
    case Bytecodes::_freturn:
      _preds[i.next_bci()]--;   // Will never hit the next bytecode
      bits_type_set( cur_bci, T_VOID );// Despite having a type, always return VOID
      break;

    // Count float operations that produce new values
    case Bytecodes::_fsub:
    case Bytecodes::_fadd:
    case Bytecodes::_fmul:
    case Bytecodes::_fdiv:
    case Bytecodes::_fneg:
    case Bytecodes::_frem:
    case Bytecodes::_i2f:
    case Bytecodes::_l2f: 
      break;

    // Count all double operations that may use FPU
    case Bytecodes::_dstore:
    case Bytecodes::_dstore_0:
    case Bytecodes::_dstore_1:
    case Bytecodes::_dstore_2:
    case Bytecodes::_dstore_3:
    case Bytecodes::_dconst_0: 
    case Bytecodes::_dconst_1: 
    case Bytecodes::_dload_0:
    case Bytecodes::_dload_1:
    case Bytecodes::_dload_2:
    case Bytecodes::_dload_3:
    case Bytecodes::_dload:

    case Bytecodes::_dsub:
    case Bytecodes::_dadd:
    case Bytecodes::_dmul:
    case Bytecodes::_ddiv:
    case Bytecodes::_dneg:
    case Bytecodes::_drem:
    case Bytecodes::_dcmpl:
    case Bytecodes::_dcmpg:

    case Bytecodes::_i2d:
    case Bytecodes::_l2d:
    case Bytecodes::_d2l:
    case Bytecodes::_d2i:
      break;

    // Count float and double operations
    case Bytecodes::_d2f:
      // fall through to count as double also.
    case Bytecodes::_f2d:
      break;

    case Bytecodes::_ldc:
    case Bytecodes::_ldc_w:
    case Bytecodes::_ldc2_w: { // Check that we parse this type
      ciConstant value = i.get_constant();
      set_type_and_size(cur_bci, bc, value.type());
      break;
    }

    case Bytecodes::_aastore: {
      compile->set_alias_index( TypeAryPtr::RANGE ); // length check
      compile->set_alias_index( TypeOopPtr::HEADER ); // array store check
      // More array store check: the element type fetch
      int element_klass_offset = objArrayKlass::element_klass_offset_in_bytes() + sizeof(oopDesc);
      ciKlass* k = TypeAryPtr::OOPS->klass();
      if( !k || !k->is_loaded() ) { // Only fails for some -Xcomp runs
        k = TypeOopPtr::BOTTOM->klass();
	bits_type_set(cur_bci,0); // Flag for pass 2
      }
      const TypeKlassPtr *kary = TypeKlassPtr::make( TypePtr::NotNull, k, element_klass_offset );
      compile->set_alias_index( kary ); // element type fetch
      compile->set_alias_index( TypeAryPtr::OOPS ); // array body
      compile->set_alias_index( TypeRawPtr::BOTTOM ); // store barrier
      break;
    }

    case Bytecodes::_aaload:
      compile->set_alias_index( TypeAryPtr::OOPS ); // array body
      compile->set_alias_index( TypeAryPtr::RANGE ); // length check
      break;
    case Bytecodes::_baload:
    case Bytecodes::_bastore:
      compile->set_alias_index( TypeAryPtr::BYTES );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;
    case Bytecodes::_caload:
    case Bytecodes::_castore:
      compile->set_alias_index( TypeAryPtr::CHARS );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;
    case Bytecodes::_daload:
    case Bytecodes::_dastore:
      compile->set_alias_index( TypeAryPtr::DOUBLES );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;
    case Bytecodes::_faload:
    case Bytecodes::_fastore:
      compile->set_alias_index( TypeAryPtr::FLOATS );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;
    case Bytecodes::_iaload:
    case Bytecodes::_iastore:
      compile->set_alias_index( TypeAryPtr::INTS );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;
    case Bytecodes::_laload:
    case Bytecodes::_lastore:
      compile->set_alias_index( TypeAryPtr::LONGS );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;
    case Bytecodes::_saload:
    case Bytecodes::_sastore:
      compile->set_alias_index( TypeAryPtr::SHORTS );
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;

    case Bytecodes::_arraylength:
      compile->set_alias_index( TypeAryPtr::RANGE );
      break;

    case Bytecodes::_monitorenter:
    case Bytecodes::_monitorexit:
    
    case Bytecodes::_irem:
    case Bytecodes::_idiv:
    case Bytecodes::_lrem:
    case Bytecodes::_ldiv:

    case Bytecodes::_aload_0:
    case Bytecodes::_aload_1:      
    case Bytecodes::_aload_2:      
    case Bytecodes::_aload_3:      
    case Bytecodes::_aload:      
    case Bytecodes::_astore_0:
    case Bytecodes::_istore_0:    
    case Bytecodes::_fstore_0:
    case Bytecodes::_istore_1:
    case Bytecodes::_astore_1:
    case Bytecodes::_fstore_1:
    case Bytecodes::_istore_2:
    case Bytecodes::_astore_2:
    case Bytecodes::_fstore_2:
    case Bytecodes::_istore_3:
    case Bytecodes::_astore_3:
    case Bytecodes::_fstore_3:
    case Bytecodes::_istore:
    case Bytecodes::_astore:
    case Bytecodes::_fstore:
    case Bytecodes::_iinc:
    case Bytecodes::_lstore_0:
    case Bytecodes::_lstore_1:
    case Bytecodes::_lstore_2:
    case Bytecodes::_lstore_3:
    case Bytecodes::_lstore:
    case Bytecodes::_nop: 
    case Bytecodes::_lconst_0: 
    case Bytecodes::_lconst_1: 
    case Bytecodes::_fconst_0: 
    case Bytecodes::_fconst_1: 
    case Bytecodes::_fconst_2: 
    case Bytecodes::_iconst_m1:
    case Bytecodes::_iconst_0: 
    case Bytecodes::_iconst_1: 
    case Bytecodes::_iconst_2: 
    case Bytecodes::_iconst_3: 
    case Bytecodes::_iconst_4: 
    case Bytecodes::_iconst_5: 
    case Bytecodes::_bipush:   
    case Bytecodes::_sipush:   
    case Bytecodes::_aconst_null: 
    case Bytecodes::_iload_0:    
    case Bytecodes::_fload_0:
    case Bytecodes::_iload_1:    
    case Bytecodes::_fload_1:
    case Bytecodes::_iload_2:    
    case Bytecodes::_fload_2:
    case Bytecodes::_iload_3:    
    case Bytecodes::_fload_3:
    case Bytecodes::_iload:    
    case Bytecodes::_fload:
    case Bytecodes::_lload_0:
    case Bytecodes::_lload_1:
    case Bytecodes::_lload_2:
    case Bytecodes::_lload_3:
    case Bytecodes::_lload:
    case Bytecodes::_pop:
    case Bytecodes::_pop2:
    case Bytecodes::_dup:
    case Bytecodes::_dup_x1:
    case Bytecodes::_dup_x2:
    case Bytecodes::_dup2:
    case Bytecodes::_dup2_x1:
    case Bytecodes::_dup2_x2:

    case Bytecodes::_ineg:
    case Bytecodes::_imul:
    case Bytecodes::_iadd:
    case Bytecodes::_isub:
    case Bytecodes::_iand:
    case Bytecodes::_ior:
    case Bytecodes::_ixor:
    case Bytecodes::_ishl:
    case Bytecodes::_ishr:
    case Bytecodes::_iushr:

    case Bytecodes::_fcmpl:
    case Bytecodes::_fcmpg:

    case Bytecodes::_land:
    case Bytecodes::_lor:
    case Bytecodes::_lxor:
    case Bytecodes::_lshr:
    case Bytecodes::_lushr:
    case Bytecodes::_lshl:
    case Bytecodes::_i2l:
    case Bytecodes::_l2i:

    case Bytecodes::_lmul:
    case Bytecodes::_ladd:
    case Bytecodes::_lcmp:
    case Bytecodes::_lneg:
    case Bytecodes::_lsub:

    case Bytecodes::_swap:

    // conversion operations
    case Bytecodes::_f2l:
    case Bytecodes::_f2i:
    case Bytecodes::_i2b:
    case Bytecodes::_i2c:
    case Bytecodes::_i2s:
      break;

    case Bytecodes::_anewarray:
      if( !i.get_klass()->is_loaded() ) {
	bits_type_set(cur_bci,0); // Flag for pass 2
        break;
      }
      t = T_OBJECT;
      goto handle_array;
    case Bytecodes::_newarray: 
      t = (BasicType)i.get_index();
      goto handle_array;
      {
handle_array: 
      compile->set_alias_index( TypeRawPtr::BOTTOM );
      compile->set_alias_index( TypeOopPtr::BOTTOM );
      compile->set_alias_index( TypeOopPtr::HEADER );
      compile->set_alias_index( TypeAryPtr::RANGE );
      const TypePtr *tp;
      switch( t ) {
      case T_OBJECT:
      case T_ARRAY:   tp = TypeAryPtr::OOPS  ; break;
      case T_BYTE:    tp = TypeAryPtr::BYTES ; break;
      case T_BOOLEAN: tp = TypeAryPtr::BYTES ; break;
      case T_SHORT:   tp = TypeAryPtr::SHORTS; break;
      case T_CHAR:    tp = TypeAryPtr::CHARS ; break;
      case T_INT:     tp = TypeAryPtr::INTS  ; break;
      case T_LONG:    tp = TypeAryPtr::LONGS ; break;
      case T_FLOAT:   tp = TypeAryPtr::FLOATS; break;
      case T_DOUBLE:  tp = TypeAryPtr::DOUBLES;break;
      default: ShouldNotReachHere();
      }
      compile->set_alias_index( tp->add_offset( arrayOopDesc::base_offset_in_bytes(t) ) );
      break;
    }   
      
    case Bytecodes::_new: {
      compile->set_alias_index( TypeRawPtr::BOTTOM );
      compile->set_alias_index( TypeOopPtr::BOTTOM );
      compile->set_alias_index( TypeOopPtr::HEADER );
      // Also must init all fields
      ciKlass* klass = i.get_klass();
      // uncommon_trap if unloaded or uninitialized
      if( klass->is_loaded() &&
          klass->is_instance_klass() &&
          klass->as_instance_klass()->is_initialized() ) {
        ciInstanceKlass* ik = klass->as_instance_klass();
        const TypeOopPtr *toop = TypeOopPtr::make(TypePtr::NotNull, klass);
        const uint size = (uint)ik->size_helper();
        for( uint j = instanceOopDesc::header_size(); j<size;j++)
          compile->set_alias_index( toop->add_offset( j*oopSize ) );
      } else {
	bits_type_set(cur_bci,0); // Flag for pass 2
      }
      break;
    }
    case Bytecodes::_checkcast:
    case Bytecodes::_instanceof: {
      // Also must init all fields
      ciKlass* klass = i.get_klass();
      // uncommon_trap if unloaded or uninitialized
      if( klass->is_loaded() ) {
        compile->set_alias_index( TypeOopPtr::HEADER );
      } else {
	bits_type_set(cur_bci,0); // Flag for pass 2
      }
      break;
    }

    case Bytecodes::_fast_igetfield: 
    case Bytecodes::_fast_agetfield: 
    case Bytecodes::_fast_lgetfield:
    case Bytecodes::_fast_iputfield: 
    case Bytecodes::_fast_aputfield: 
    case Bytecodes::_fast_fputfield: 
    case Bytecodes::_fast_lputfield: 
    case Bytecodes::_fast_dputfield: 
      ShouldNotReachHere();


    default:                    // Ignore most bytecodes      
      return Parse::Bailout_unhandled_bytecode;
    } // End of switch on opcode

  } // End of while more bytecodes

  // Also the exception table entries are branch targets
  //
  // We add an extra input to prevent these targets from ever believing
  // they have seen all predecessors.  Also, exception paths are always
  // treated as 'cloning' inputs, meaning that we are adding a new path
  // in, instead of taking one of the pre-counted paths.
  for (ciExceptionHandlerStream hs(_method); !hs.is_done(); hs.next()) {
    _preds[hs.handler()->handler_bci()]++;
  }

  return Parse::Bailout_none;
}


// ----------------------------------------------------------------------------
#ifndef PRODUCT

static void per_method_stats() {
  // Compute difference between this method's cumulative totals and old totals
  int explicit_null_checks_cur = explicit_null_checks_inserted - explicit_null_checks_inserted_old;
  int elided_null_checks_cur = explicit_null_checks_elided - explicit_null_checks_elided_old;

  // Print differences
  if( explicit_null_checks_cur )
    tty->print_cr("XXX Explicit NULL checks inserted: %d", explicit_null_checks_cur);
  if( elided_null_checks_cur )
    tty->print_cr("XXX Explicit NULL checks removed at parse time: %d", elided_null_checks_cur);

  // Store the current cumulative totals
  methods_parsed_old = methods_parsed;
  methods_seen_old = methods_seen;
  explicit_null_checks_inserted_old = explicit_null_checks_inserted;
  explicit_null_checks_elided_old = explicit_null_checks_elided;
}  


void Parse::show_bailout_info() {
  
  if( (_bailout != Bailout_compile_only) && (PrintOptoBailouts || PrintOpto || PrintOptoParseInfo) ){
    // Print status information for each method parsed
    tty->print_cr("");
    tty->print("Bailed out: ");
    _method->print_name();
    switch(_bailout) {
      case Bailout_node_limit_exceeded: tty->print_cr(" due to exceeding node limit"); break;
      case Bailout_shared_handler:      tty->print_cr(" due to shared exception handler in/out of jsr...ret"); break;
      case Bailout_unhandled_bytecode:  tty->print_cr(" due to unhandled bytecode"); break;
      case Bailout_monitor_matching:    tty->print_cr(" due to wrongly nested monitorenter/monitorexit"); break;
      case Bailout_exception_thrown:    tty->print_cr(" due to exception thrown during parsing"); break;
    }        
    per_method_stats();
  }

  if( PrintOpto ) {           // Print that we bailed out
    tty->print("- "); 
    _method->print_short_name();
    tty->print_cr("");
  }  
}

#endif
