/*
 * @(#)callnode.cpp	1.184 01/02/09 14:45:54
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
// Portions of code courtesy of Clifford Click

// Optimization - Graph Style

#include "incls/_precompiled.incl"
#include "incls/_callnode.cpp.incl"

//=============================================================================
uint StartNode::size_of() const { return sizeof(*this); }
uint StartNode::hash() const { 
  // We do not create equivalent starts, just add to Node::in(control)
  return (uint)in(TypeFunc::Control) + (uint)_domain; 
}
uint StartNode::cmp( const Node &n ) const
{ return _domain == ((StartNode&)n)._domain; }
const Type *StartNode::bottom_type() const { return _domain; }
const Type *StartNode::Value(PhaseTransform *phase) const { return _domain; }
#ifndef PRODUCT
void StartNode::dump_spec() const { tty->print(" #"); _domain->dump();}
#endif

//------------------------------Ideal------------------------------------------
Node *StartNode::Ideal(PhaseGVN *phase, PhaseDefUse *du){
  return remove_dead_region(phase,du) ? this : NULL;
}

//------------------------------calling_convention-----------------------------
void StartNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  Matcher::calling_convention( parm_regs, argcnt, false );
}

//------------------------------Registers--------------------------------------
const RegMask &StartNode::in_RegMask(uint) const { 
  return RegMask::Empty;
}

//------------------------------match------------------------------------------
// Construct projections for incoming parameters, and their RegMask info
Node *StartNode::match( const ProjNode *proj, const Matcher *match ) {
  switch( proj->_con ) {
  case TypeFunc::Control: 
  case TypeFunc::I_O:
  case TypeFunc::Memory:
    return new (1) MachProjNode(this,proj->_con,RegMask::Empty,MachProjNode::unmatched_proj);
  case TypeFunc::FramePtr:
    return new (1) MachProjNode(this,proj->_con,Matcher::c_frame_ptr_mask, Op_RegP);
  case TypeFunc::ReturnAdr:
    return new (1) MachProjNode(this,proj->_con,match->_return_addr_mask,Op_RegP);
  case TypeFunc::Parms:
  default: {
      uint parm_num = proj->_con - TypeFunc::Parms;
      const Type *t = _domain->_fields[proj->_con];
      if( t->_base == Type::Half )  // 2nd half of Longs and Doubles
        return new (1) ConNode( Type::TOP );
      uint ideal_reg = Matcher::base2reg[t->_base];
      RegMask &rm = match->_calling_convention_mask[parm_num];
      return new (1) MachProjNode(this,proj->_con,rm,ideal_reg);
    }
  } 
  return NULL;
}

//------------------------------StartC2INode----------------------------------
// The method start node for a Compiled to Interpreter frame converter

//------------------------------c2i_domain-----------------------------
// Construct the new incoming signature, 
// Since the MethodKlassHolder parameter is split outside of ideal world
// Expand signature to include inline_cached_klass and method_oop
const TypeTuple *StartC2INode::c2i_domain( const TypeTuple *domain ) {
  uint j_parm_cnt     = domain->_cnt;
  uint start_parm_cnt = j_parm_cnt + 1;
  const Type     **fields = TypeTuple::fields(start_parm_cnt);
  for( uint i = 0; i < j_parm_cnt; i++ )
    fields[i] = domain->_fields[i];
  // either method oop or MethodKlassHolder converted to method_oop by prefix
  fields[j_parm_cnt + 0] = TypeOopPtr::BOTTOM;  // method_oop
  return TypeTuple::make( start_parm_cnt, fields );
}

//------------------------------calling_convention----------------------------
// Frame converters from compiled code to interpreter 
// pass the last parameter in the inline_cache_reg, 
// not through the standard calling convention
void StartC2INode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  // Use the standard compiled-java calling convention with one additional
  // argument in the inline_cache_reg()
  assert( argcnt >= 1, "Need method_oop parameters");
  uint java_arg_count = argcnt - 1;
  Matcher::calling_convention( parm_regs, java_arg_count, false );

  // Set up register assignments for extra parameter
  parm_regs[ java_arg_count + 0 ] = Matcher::inline_cache_reg();
}

//------------------------------match------------------------------------------
// Construct projections for incoming parameters, and their RegMask info
// A StartC2INode takes an extra parameter in Matcher::inline_cache_reg()
// that is the methodOop of the callee
Node *StartC2INode::match( const ProjNode *proj, const Matcher *match ) {
  // Check for projections describing the method's signature
  uint parm_num = proj->_con;
  if( parm_num < (_domain->_cnt - 1) ) {
    return StartNode::match( proj, match );
  } 
  // special MachProjNode, method_oop in Matcher::inline_cache_reg()
  else if( parm_num == (_domain->_cnt - 1) ) {
    const RegMask &rm  = match->inline_cache_reg_mask();
    return  new (1) MachProjNode(this,proj->_con,rm,Op_RegP);
  } else {
    ShouldNotReachHere();
  }
  return NULL;
}


//------------------------------StartI2CNode----------------------------------
// The method start node for an Interpreter to Compiled frame converter

//------------------------------i2c_domain-----------------------------
const TypeTuple *StartI2CNode::i2c_domain( const TypeTuple *domain ) {
  uint j_parm_cnt     = domain->_cnt;
  uint start_parm_cnt = j_parm_cnt + 1;
  const Type **fields = TypeTuple::fields(start_parm_cnt);
  for( uint i=0; i<j_parm_cnt; i++ )
    fields[i] = domain->_fields[i];
  fields[j_parm_cnt + 0] = TypeOopPtr::BOTTOM;  // methodOop
  return TypeTuple::make( start_parm_cnt, fields );
}

//------------------------------calling_convention-----------------------------
// Frame converters from interpreter to compiled code
// pass the last parameter in the inline_cache_reg, 
// not through the standard calling convention
void StartI2CNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  // Use the standard compiler calling convention 
  // for all arguments except the last, nmethod 
  assert( argcnt > 0, "Need the nmethod");
  uint java_arg_count = argcnt - 1;
  Matcher::interpreter_calling_convention( parm_regs, java_arg_count, false );

  // Set up register assignments for extra parameter, nmethod
  parm_regs[ java_arg_count + 0 ] = Matcher::inline_cache_reg();
}

//------------------------------match------------------------------------------
// Construct projections for incoming parameters, and their RegMask info
// StartI2CNodes take an extra parameter in: Matcher::inline_cache_reg()
// that describes the methodOop being called.
Node *StartI2CNode::match( const ProjNode *proj, const Matcher *match ) {

  switch( proj->_con ) {
  case TypeFunc::Control: 
  case TypeFunc::I_O:
  case TypeFunc::Memory:
  case TypeFunc::FramePtr:
  case TypeFunc::ReturnAdr:
    return StartNode::match( proj, match );

  // Check for projections describing the method's signature
  case TypeFunc::Parms:
  default:
    if( proj->_con != _domain->_cnt - 1 ) {
      return StartNode::match( proj, match );
    } else {
      // The last parameter is passed by interpreter in inline_cache_reg
      const RegMask &rm  = match->inline_cache_reg_mask();
      return new (1) MachProjNode(this,proj->_con,rm,Op_RegP);
    }
  } 
  return NULL;
}

//------------------------------StartOSRNode----------------------------------
// The method start node for an on stack replacement adapter

//------------------------------osr_domain-----------------------------
const TypeTuple *StartOSRNode::osr_domain() {
  const int total_parm_count = TypeFunc::Parms+1;
  const Type **fields = TypeTuple::fields(total_parm_count);
  fields[TypeFunc::Parms+0] = TypeRawPtr::BOTTOM;  // address of frame object

  return TypeTuple::make(total_parm_count, fields);
}

//------------------------------calling_convention-----------------------------
// On stack replacement calls use the interpreter calling convention.
void StartOSRNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  // On stack replacement code takes its single argument in the
  // interpreter's calling convention.
  assert( argcnt > 0, "Need the nmethod");
  Matcher::interpreter_calling_convention( parm_regs, argcnt, false );
}

//=============================================================================
const char * const ParmNode::names[TypeFunc::Parms+1] = {
  "Control", "I_O", "Memory", "FramePtr", "ReturnAdr", "Parms"
};

#ifndef PRODUCT
void ParmNode::dump_spec() const {
  if( _con < TypeFunc::Parms ) {
    tty->print(names[_con]);
  } else {
    tty->print("Parm%d: ",_con-TypeFunc::Parms);
    // Verbose and WizardMode dump bottom_type for all nodes
    if( !Verbose && !WizardMode )   bottom_type()->dump();
  }
}
#endif

uint ParmNode::ideal_reg() const {
  switch( _con ) {
  case TypeFunc::Control  : // fall through
  case TypeFunc::I_O      : // fall through
  case TypeFunc::Memory   : return 0;
  case TypeFunc::FramePtr : // fall through
  case TypeFunc::ReturnAdr: return Op_RegP;      
  default                 : assert( _con > TypeFunc::Parms, "" ); 
    // fall through
  case TypeFunc::Parms    : {
    // Register or stack slot the parameter is being passed in
    Compile* compile = Compile::current();
    int reg = compile->_matcher->_parm_regs[_con - TypeFunc::Parms];
    // Type of argument being passed
    const Type *t = in(0)->is_Start()->_domain->_fields[_con];
    // Check to see if being passed in a register and is a float type
    if (reg < SharedInfo::stack0 && (t->_base == Type::FloatBot || t->_base == Type::DoubleBot)) {
      // %%%% Checks for floating point only to avoid thinking of oop
      // parameters as Op_RegI. Need a better check.
      // Passed in a register; return the register type.
      // Can be different from the argument type if, for example, you
      // want to pass float arguments in integer registers.
      if (compile->_matcher->_register_save_type[reg] == Op_RegI) return Op_RegI;
      return t->_base == Type::FloatBot ? Op_RegF : Op_RegD;
    } else {
      // Passed on the stack or not a float; use the argument type
      return Matcher::base2reg[t->_base];
    }
  }
  }
  ShouldNotReachHere();
  return 0;
}

//------------------------------Ideal-----------------------------------------
// If the StartNode was inlined in another procedure, then we are a copy
Node *ParmNode::Identity( PhaseTransform *phase ) {
  Node *start = _in[0];
  Node *ctrl = start->in(0);
  if( ctrl != start ) {
    // Start is for an inlined method
    if( start->in(0)->bottom_type() == Type::TOP ) return ctrl;
    return start->in(_con);
  } else {
    return this;
  }
}


//=============================================================================
ReturnNode::ReturnNode(Node *cntrl, Node *i_o, Node *memory, Node *frameptr, Node *retadr ) : Node(TypeFunc::Parms) { 
  set_req(TypeFunc::Control,cntrl); 
  set_req(TypeFunc::I_O,i_o); 
  set_req(TypeFunc::Memory,memory); 
  set_req(TypeFunc::FramePtr,frameptr);
  set_req(TypeFunc::ReturnAdr,retadr); 
}

Node *ReturnNode::Ideal(PhaseGVN *phase, PhaseDefUse *du){
  return remove_dead_region(phase,du) ? this : NULL; 
}

const Type *ReturnNode::Value( PhaseTransform *phase ) const { 
  return ( phase->type(in(TypeFunc::Control)) == Type::TOP)
    ? Type::TOP
    : Type::BOTTOM;
}

// Do we Match on this edge index or not?  No edges on return nodes
uint ReturnNode::match_edge(uint idx) const {
  return 0;
}


#ifndef PRODUCT
void ReturnNode::dump_req() const {
  // Dump the required inputs, enclosed in '(' and ')'
  uint i;                       // Exit value of loop
  for( i=0; i<req(); i++ ) {    // For all required inputs
    if( i == TypeFunc::Parms ) tty->print("returns");
    if( in(i) ) tty->print("%c%d ", Compile::current()->_arena.contains(in(i)) ? ' ' : 'o', in(i)->_idx);
    else tty->print("_ ");
  }
}
#endif

//=============================================================================
RethrowNode::RethrowNode(
  Node* cntrl,
  Node* i_o,
  Node* memory,
  Node* frameptr,
  Node* ret_adr,
  Node* exception
) : Node(TypeFunc::Parms + 1) { 
  set_req(TypeFunc::Control  , cntrl    ); 
  set_req(TypeFunc::I_O      , i_o      ); 
  set_req(TypeFunc::Memory   , memory   ); 
  set_req(TypeFunc::FramePtr , frameptr );
  set_req(TypeFunc::ReturnAdr, ret_adr);
  set_req(TypeFunc::Parms    , exception);
}

Node *RethrowNode::Ideal(PhaseGVN *phase, PhaseDefUse *du){
  return remove_dead_region(phase,du) ? this : NULL; 
}

const Type *RethrowNode::Value( PhaseTransform *phase ) const { 
  return (phase->type(in(TypeFunc::Control)) == Type::TOP)
    ? Type::TOP
    : Type::BOTTOM;
}

uint RethrowNode::match_edge(uint idx) const {
  return 0;
}

#ifndef PRODUCT
void RethrowNode::dump_req() const {
  // Dump the required inputs, enclosed in '(' and ')'
  uint i;                       // Exit value of loop
  for( i=0; i<req(); i++ ) {    // For all required inputs
    if( i == TypeFunc::Parms ) tty->print("exception");
    if( in(i) ) tty->print("%c%d ", Compile::current()->_arena.contains(in(i)) ? ' ' : 'o', in(i)->_idx);
    else tty->print("_ ");
  }
}
#endif

//=============================================================================
TailCallNode::TailCallNode(Node *cntrl, Node *i_o, Node *memory, Node *frameptr, Node *retadr, Node *target, Node *moop) : ReturnNode( cntrl, i_o, memory, frameptr, retadr ) {
  add_req(target);
  add_req(moop);
}

// Do we Match on this edge index or not?  Match only target address & method
uint TailCallNode::match_edge(uint idx) const {
  return TypeFunc::Parms <= idx  &&  idx <= TypeFunc::Parms+1;
}

//=============================================================================
TailJumpNode::TailJumpNode(Node *cntrl, Node *i_o, Node *memory, Node *frameptr, Node *target, Node *ex_oop) : ReturnNode( cntrl, i_o, memory, frameptr, Compile::current()->_top_node ) {
  add_req(target);
  add_req(ex_oop);
}

// Do we Match on this edge index or not?  Match only target address & oop
uint TailJumpNode::match_edge(uint idx) const {
  return TypeFunc::Parms <= idx  &&  idx <= TypeFunc::Parms+1;
}

//=============================================================================
JVMStateObj::JVMStateObj(ciMethod* method, int bci, uint16 sp) : 
  _method(method), _bci(bci) {
    _locoff = TypeFunc::Parms;
    _spoff = _locoff + (_method ? _method->max_locals() : 0);
    _monoff = _spoff + sp;
    _endoff = _monoff;
    _next = NULL;
    _sp = 0;
}

//------------------------------format_helper----------------------------------
// Given an allocation (a Chaitin object) and a Node decide if the Node carries
// any defined value or not.  If it does, print out the register or constant.
#ifndef PRODUCT
  static void format_helper( PhaseChaitin *chaitin, Node *n, const char *msg, uint i ) {
  if (n == NULL) { tty->print(" NULL"); return; }
  if( chaitin->Find(n) ) { // Check for undefined
    char buf[50];
    chaitin->dump_register(n,buf);
    tty->print(" %s%d=%s",msg,i,buf);
  } else {                      // No register, but might be constant  
    const Type *t = n->bottom_type();
    switch( t->_base ) {
    case Type::Int:  
      tty->print(" %s%d=#%d",msg,i,t->is_int()->get_con()); 
      break;
    case Type::AnyPtr: 
      assert( t == TypePtr::NULL_PTR, "" );
      tty->print(" %s%d=#NULL",msg,i);
      break;
    case Type::AryPtr: 
    case Type::KlassPtr:
    case Type::OopPtr: 
      tty->print(" %s%d=#Ptr0x%x",msg,i,t->isa_javaptr()->const_oop());
      break;
    case Type::DoubleCon:
      tty->print(" %s%d=#%fD",msg,i,t->is_double_constant()->_d);
      break;
    case Type::FloatCon:
      tty->print(" %s%d=#%fF",msg,i,t->is_float_constant()->_f);
      break;
    case Type::Long:
      tty->print(" %s%d=#%ld",msg,i,t->is_long()->get_con());
      break;
    case Type::Half:
    case Type::Top:  
      tty->print(" %s%d=_",msg,i);
      break;
    default: ShouldNotReachHere();
    }
  }
}
#endif

//------------------------------format-----------------------------------------
#ifndef PRODUCT
void JVMStateObj::format( PhaseChaitin *chaitin, const Node *n ) const {
  tty->print("        #");
  if( _method ) {
    _method->print_short_name();
    tty->print(" @ bci:%d ",_bci);
  } else {
    tty->print_cr(" runtime stub ");
    return;
  }
  MachNode *mach = ((Node *)n)->is_Mach();
  MachSafePointNode *mcall = mach->is_MachSafePoint();
  if (mcall) {
    uint16 i;
    // Print locals
    for( i = 0; i < (uint16)_method->max_locals(); i++ ) 
      format_helper( chaitin, mcall->local(this, i), "L", i );
    // Print stack
    for( i = 0; i < (_monoff-_spoff); i++ ) {
      if ((uint)(_spoff + i) >= mcall->len()) 
        tty->print(" oob ");
      else
       format_helper( chaitin, mcall->stack(this, i), "STK", i );
    }
    for( i = 0; (int)i < nof_monitors(); i++ ) {
      Node *box = mcall->monitor_box(this, i);
      Node *obj = mcall->monitor_obj(this, i);
      if ( chaitin->Find(box) ) {
        while( !box->is_BoxLock() )  box = box->in(1);
        format_helper( chaitin, box, "MON-BOX", i );
      } else {
        OptoReg::Name box_reg = BoxLockNode::stack_slot(box);
        tty->print(" MON-BOX%d=%s+%d",i,SharedInfo::regName[SharedInfo::c_frame_pointer],chaitin->reg2offset(box_reg));
      }
      format_helper( chaitin, obj, "MON-OBJ", i );      
    }
  }
  tty->print_cr("");
  if( next() ) next()->format(chaitin, n);
}
#endif

#ifndef PRODUCT
void JVMStateObj::dump_spec() const { 
  if( _method ) {
    _method->print_short_name();
    tty->print(" @ bci:%d ",_bci);
  } else {
    tty->print(" runtime stub ");
  }
  if( _next ) _next->dump_spec();
}
#endif

//------------------------------clone------------------------------------------
JVMStateObj *JVMStateObj::clone() const {
  JVMStateObj *n = new JVMStateObj(_method, _bci, _monoff-_spoff);
  n->set_locoff(_locoff);
  n->set_spoff(_spoff);
  n->set_monoff(_monoff);
  n->set_endoff(_endoff);
  return n;
}

//=============================================================================
uint CallNode::hash() const { return Node::hash() + (int)_func + (int)_jvms; }
uint CallNode::cmp( const Node &n ) const
{ return _func == ((CallNode&)n)._func && _jvms == ((CallNode&)n)._jvms; }
#ifndef PRODUCT
void CallNode::dump_req() const { 
  // Dump the required inputs, enclosed in '(' and ')'
  uint i;                       // Exit value of loop
  for( i=0; i<req(); i++ ) {    // For all required inputs
    if( i == TypeFunc::Parms ) tty->print("(");
    if( in(i) ) tty->print("%c%d ", Compile::current()->_arena.contains(in(i)) ? ' ' : 'o', in(i)->_idx);
    else tty->print("_ ");
  }
  tty->print(")");
}

void CallNode::dump_spec() const { 
  tty->print(" "); 
  _func->dump();
  if( _jvms ) _jvms->dump_spec();
}
#endif

const Type *CallNode::bottom_type() const { return _func->_range; }
const Type *CallNode::Value(PhaseTransform *phase) const { 
  if( phase->type(_in[0]) == Type::TOP ) return Type::TOP;
  return _func->_range; 
}

//------------------------------calling_convention-----------------------------
void CallNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  // Use the standard compiler calling convention 
  Matcher::calling_convention( parm_regs, argcnt, true );
}


//------------------------------fix_mispaired_doubles_longs--------------------
// Normally, double and long values are carried around in register pairs or
// stack slot pairs, and represented by masks with two adjacent bits, the
// first of which is even.  (This is recognized by RegMask::is_evenodd.)
//
// However, OS-specific calling conventions will often dictate that the
// halves of a double/long must be transmitted in some other way, for
// example:
//   - in adjacent "misaligned" stack slots, the first of which is odd
//   - in two integer registers (e.g., Sparc %o0/%o1)
//   - some even stranger pair of locations (e.g., Sparc %o5/[%sp+92])
// To cope with this variety, we reserve the right to break up outgoing
// double and long values and dispose of the halves independently of each
// other.  The break-up is managed by the quasi-projections DoubleHalf[01].
//
// The node DoubleHalf0 takes as an input a double value and produces as its
// output the first 32-bit word, as a RegI.  (By "first" we mean first in
// memory order.)  Likewise, a DoubleHalf1 produces the second 32-bit word.
//
// Note that Java calling conventions must not ever contain mispaired
// doubles--CallNode::fix_mispaired_doubles is intended only for C calls.
//
void CallNode::fix_mispaired_double_longs() {
  // If CPU can load and store mis-aligned doubles directly then no fixup is 
  // needed.  Else we split the double into 2 integer pieces and move it 
  // piece-by-piece.  Only happens when passing doubles into C code as the 
  // Java calling convention forces doubles to be aligned.
  if( Matcher::misaligned_doubles_ok ) return;

  // Build signature type array
  const TypeTuple *domain = _func->_domain;
  int c_argcnt = domain->_cnt - TypeFunc::Parms;
  uint16 *parm_regs = NEW_RESOURCE_ARRAY( uint16, c_argcnt );
  for( int i = 0; i < c_argcnt; i++ ) 
    parm_regs[i] = Matcher::base2reg[domain->_fields[TypeFunc::Parms + i]->_base];
  // Convert signature type to a calling convention
  calling_convention( parm_regs, c_argcnt );
  // Check the calling convention for usage of double/long register pairs
  for( int j = 0; j < c_argcnt; j++ ) {
    const Type *t_arg = domain->_fields[TypeFunc::Parms+j];
    if( t_arg == Type::DOUBLE || t_arg == TypeLong::LONG ) {
      assert(domain->_fields[TypeFunc::Parms+j+1] == Type::HALF, "");
      // Do we desire the double/long in a misaligned or not-adjacent place?
      // Or are we doing a cross-register-file transfer?
      if( (parm_regs[j] < (unsigned int)SharedInfo::stack0 &&   // 1st word is in a non-D register
	   !Matcher::idealreg2regmask[t_arg == Type::DOUBLE ? Op_RegD : Op_RegL]->Member((OptoReg::Name)parm_regs[j])) ||
	 !is_even(parm_regs[j]) ||             // 1st word is not aligned?  OR
          parm_regs[j]+1 != parm_regs[j+1] ) {  // next word is not adjacent?
	// We must go back and split the double/long into 32-bit halves.
	Node *val = in(j+TypeFunc::Parms);
	Node *s0, *s1;
	if( t_arg == Type::DOUBLE ) {
	  s0 = new (2) DoubleHalf0Node(val);
	  s1 = new (2) DoubleHalf1Node(val);
	} else {
	  s0 = new (2) LongHalf0Node(val);
	  s1 = new (2) LongHalf1Node(val);
	}
	set_req(j  +TypeFunc::Parms, s0 );
	set_req(j+1+TypeFunc::Parms, s1 );
      }
    }
  }
}


//------------------------------match------------------------------------------
// Construct projections for control, I/O, memory-fields, ..., and
// return result(s) along with their RegMask info
Node *CallNode::match( const ProjNode *proj, const Matcher *match ) {
  switch( proj->_con ) {
  case TypeFunc::Control: 
  case TypeFunc::I_O:
  case TypeFunc::Memory:
    return new (1) MachProjNode(this,proj->_con,RegMask::Empty,MachProjNode::unmatched_proj);

  case TypeFunc::Parms+1:	// For LONG & DOUBLE returns
    assert( _func->_range->_fields[TypeFunc::Parms+1] == Type::HALF, "" );
    // 2nd half of doubles and longs
    return new (1) MachProjNode(this,proj->_con, RegMask::Empty, (uint)OptoReg::Bad);

  case TypeFunc::Parms: {       // Normal returns
    uint ideal_reg = Matcher::base2reg[_func->_range->_fields[TypeFunc::Parms]->_base];
    OptoReg::Name machine_reg = is_CallRuntime() 
      ? match->c_return_value(ideal_reg)  // Calls into C runtime
      : match->  return_value(ideal_reg); // Calls into compiled Java code
    RegMask rm = Matcher::mreg2regmask[machine_reg];
    // Doubles & longs need 2 bits in the main returned value
    const Type *ret = _func->_range->_fields[TypeFunc::Parms];
    if( ret == Type::DOUBLE || ret == TypeLong::LONG )
      rm.Insert( OptoReg::add(machine_reg,1) );
    return new (1) MachProjNode(this,proj->_con,rm,ideal_reg);
  }

  case TypeFunc::ReturnAdr:
  case TypeFunc::FramePtr:
  default:
    ShouldNotReachHere();
  }
  return NULL;
}

// Do we Match on this edge index or not?  Match no edges
uint CallNode::match_edge(uint idx) const {
  return 0;
}

//=============================================================================
uint CallJavaNode::size_of() const { return sizeof(*this); }
uint CallJavaNode::hash() const { return CallNode::hash() + (int)_method; }
uint CallJavaNode::cmp( const Node &n ) const { 
  CallJavaNode &call = (CallJavaNode&)n;
  return CallNode::cmp(call) && _method == call._method; 
}
#ifndef PRODUCT
void CallJavaNode::dump_spec() const { 
  if( _method ) _method->print_short_name();
  CallNode::dump_spec();
}
#endif

//=============================================================================
uint CallStaticJavaNode::size_of() const { return sizeof(*this); }
uint CallStaticJavaNode::hash() const {
  // Since calls produce control, distinguishing control is sufficient
  // Node::in(control) + CallNode::_func + CallJavaNode_method + this::_name
  return (uint)in(TypeFunc::Control) + (uint)_func + (uint)_method + (uint)_name; 
}
uint CallStaticJavaNode::cmp( const Node &n ) const { 
  CallStaticJavaNode &call = (CallStaticJavaNode&)n;
  return CallJavaNode::cmp(call); 
}
#ifndef PRODUCT
void CallStaticJavaNode::dump_spec() const { 
  tty->print("# Static ");
  if( _name ) {
    tty->print(_name);
    tty->print(" ");
  }
  CallJavaNode::dump_spec();
}
#endif

//=============================================================================
uint CallDynamicJavaNode::size_of() const { return sizeof(*this); }
uint CallDynamicJavaNode::hash() const { 
  // Since calls produce control, distinguishing control is sufficient
  // Node::in(control) + CallNode::_func + CallJavaNode_method
  return (uint)in(TypeFunc::Control) + (uint)_func + (uint)_method; 
}
uint CallDynamicJavaNode::cmp( const Node &n ) const { 
  CallDynamicJavaNode &call = (CallDynamicJavaNode&)n;
  return CallJavaNode::cmp(call); 
}
#ifndef PRODUCT
void CallDynamicJavaNode::dump_spec() const { 
  tty->print("# Dynamic ");
  CallJavaNode::dump_spec();
}
#endif

//=============================================================================
uint CallCompiledJavaNode::size_of() const { return sizeof(*this); }
#ifndef PRODUCT
void CallCompiledJavaNode::dump_spec() const { 
  tty->print("# CompiledJava ");
  CallNode::dump_spec();
}
#endif

//------------------------------calling_convention-----------------------------
void CallCompiledJavaNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  // Use the standard compiler calling convention 
  // for all arguments except the last, method_oop
  assert( argcnt > 0, "Need the method_oop in inline_cache_reg");
  uint java_arg_count = argcnt - 1;
  Matcher::calling_convention( parm_regs, java_arg_count, true );

  // Set up register assignments for extra parameter
  parm_regs[ java_arg_count + 0 ] = Matcher::inline_cache_reg();
}


//=============================================================================
uint CallInterpreterNode::size_of() const { return sizeof(*this); }
#ifndef PRODUCT
void CallInterpreterNode::dump_spec() const { 
  tty->print("# Interpreter ");
  CallNode::dump_spec();
}
#endif

//------------------------------calling_convention-----------------------------
void CallInterpreterNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  assert( argcnt > 0, "Must have method_oop");
  uint java_arg_count = argcnt - 1;
  Matcher::interpreter_calling_convention( parm_regs, java_arg_count, true );

  // Set up register assignments for extra parameter
  parm_regs[ java_arg_count + 0 ] = Matcher::interpreter_method_oop_reg();
}


//=============================================================================
uint CallRuntimeNode::size_of() const { return sizeof(*this); }
uint CallRuntimeNode::hash() const { return CallNode::hash() + _name[0]; }
uint CallRuntimeNode::cmp( const Node &n ) const { 
  CallRuntimeNode &call = (CallRuntimeNode&)n;
  return CallNode::cmp(call) && !strcmp(_name,call._name);
}
#ifndef PRODUCT
void CallRuntimeNode::dump_spec() const { 
  tty->print("# "); 
  tty->print(_name);
  CallNode::dump_spec();
}
#endif

//------------------------------calling_convention-----------------------------
void CallRuntimeNode::calling_convention( uint16 *parm_regs, uint argcnt ) const {
  Matcher::c_calling_convention( parm_regs, argcnt );
}

//=============================================================================
#ifndef PRODUCT
void CallLeafNode::dump_spec() const { 
  tty->print("# "); 
  tty->print(_name);
  CallNode::dump_spec();
}
#endif

//=============================================================================
SafePointNode::SafePointNode(uint edges, JVMStateObj *jvms) 
: MultiNode( edges ), _jvms(jvms), _jvmtail(jvms), _oop_map(NULL) {
}

uint SafePointNode::size_of() const { return sizeof(*this); }
uint SafePointNode::hash() const { return Node::hash(); }
uint SafePointNode::cmp( const Node &n ) const { 
  return (&n == this);          // Always fail except on self
}

//------------------------------Ideal------------------------------------------
// Skip over any collapsed Regions
Node *SafePointNode::Ideal(PhaseGVN *phase, PhaseDefUse *du) {
  if( remove_dead_region(phase,du) ) return this;

  return NULL; 
}

//------------------------------Identity---------------------------------------
// Remove obviously duplicate safepoints
Node *SafePointNode::Identity( PhaseTransform *phase ) {
  return this;
}

//------------------------------Value------------------------------------------
const Type *SafePointNode::Value( PhaseTransform *phase ) const {
  if( phase->type(in(0)) == Type::TOP ) return Type::TOP;
  if( phase->eqv( in(0), this ) ) return Type::TOP; // Dead infinite loop
  return TypeTuple::MEMBAR;
}

#ifndef PRODUCT
void SafePointNode::dump_spec() const { 
  tty->print(" SafePoint "); 
}
#endif

const RegMask &SafePointNode::in_RegMask(uint idx) const { 
  if( idx < TypeFunc::Parms ) return RegMask::Empty;
  // Values outside the domain represent debug info
  return *(Compile::current()->_matcher->idealreg2debugmask[in(idx)->ideal_reg()]);
}
const RegMask &SafePointNode::out_RegMask() const {
  return RegMask::Empty;
}

void SafePointNode::push_monitor( const FastLockNode *lock ) {
  // Add a FastLockNode, which points to both the original BoxLockNode (the
  // stack space for the monitor) and the Object being locked.
  if( GenerateSynchronizationCode ) {
    add_req(lock->box_node());
    add_req(lock->obj_node());
  } else {
    add_req(NULL);
    add_req(NULL);
  }
  _jvms->set_endoff(_jvms->endoff()+2);
}

void SafePointNode::pop_monitor() {
  // Delete last monitor from debug info
  del_req( _jvms->endoff()-1 );
  del_req( _jvms->endoff()-2 );
  _jvms->set_endoff(_jvms->endoff()-2);
}

Node *SafePointNode::peek_monitor_box() const {
  return in( _jvms->endoff()-2 );
}

Node *SafePointNode::peek_monitor_obj() const {
  return in( _jvms->endoff()-1 );
}

// Do we Match on this edge index or not?  Match no edges
uint SafePointNode::match_edge(uint idx) const {
  return 0;
}


