/*
 * @(#)callnode.hpp	1.146 01/02/09 14:45:55
 *
 * Copyright 1991-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
// Portions of code courtesy of Clifford Click

// Optimization - Graph Style

class Chaitin;
class MultiNode;
class  SafePointNode;
class   CallNode;
class     CallJavaNode;
class       CallStaticJavaNode;
class       CallDynamicJavaNode;
class     CallCompiledJavaNode;
class     CallInterpreterNode;
class     CallRuntimeNode;
class       CallLeafNode;
class       CallNativeNode;
class JVMStateObj;
class OopMap;
class State;
class StartNode;
class   StartI2CNode;
class   StartC2INode;
class MachCallNode;
class FastLockNode;

//------------------------------StartNode--------------------------------------
// The method start node
class StartNode : public MultiNode {
  virtual uint hash() const;    // Check the type
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const; // Size is bigger
public:
  const TypeTuple *_domain;
  StartNode( Node *root, const TypeTuple *domain ) : MultiNode(2), _domain(domain) { set_req(0,this); set_req(1,root); }
  virtual int Opcode() const;
  virtual int pinned() const { return 1; };
  virtual bool  is_CFG        () const { return true; }
  virtual bool depends_only_on_test() const { return false; }
  virtual int   is_block_start() const { return 1; }
  virtual const Type *bottom_type() const;
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual StartNode *is_Start() { return this; }
  virtual void  calling_convention( uint16 *parm_reg, uint length ) const;
  virtual const RegMask &in_RegMask(uint) const;
  virtual Node *match( const ProjNode *proj, const Matcher *m );
  virtual uint ideal_reg() const { return 0; }
#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};

//------------------------------StartC2INode-----------------------------------
// The method start node for a Compiled to Interpreter frame converter
class StartC2INode : public StartNode {
public:
  StartC2INode( Node *root, const TypeTuple *domain ) : StartNode(root, domain) {}
  virtual int   Opcode() const;
  static  const TypeTuple *c2i_domain( const TypeTuple *domain );
  virtual void  calling_convention( uint16 *parm_reg, uint length ) const;
  virtual Node *match( const ProjNode *proj, const Matcher *m );
};


//------------------------------StartI2CNode-----------------------------------
// The method start node for an Interpreter to Compiled frame converter
class StartI2CNode : public StartNode {
public:
  StartI2CNode( Node *root, const TypeTuple *domain ) : StartNode(root, domain) {}
  virtual int   Opcode() const;
  static  const TypeTuple *i2c_domain( const TypeTuple *domain );
  virtual void  calling_convention( uint16 *parm_regs, uint argcnt ) const;
  virtual Node *match( const ProjNode *proj, const Matcher *m );
};


//------------------------------StartOSRNode-----------------------------------
// The method start node for on stack replacement code
class StartOSRNode : public StartNode {
public:
  StartOSRNode( Node *root, const TypeTuple *domain ) : StartNode(root, domain) {}
  virtual int   Opcode() const;
  static  const TypeTuple *osr_domain();
  virtual void  calling_convention( uint16 *parm_regs, uint argcnt ) const;
  // Use the StartNode implementation of match
};


//------------------------------ParmNode---------------------------------------
// Incoming parameters
class ParmNode : public ProjNode {
  static const char * const names[TypeFunc::Parms+1];
public:
  ParmNode( StartNode *src, uint con ) : ProjNode(src,con) {}
  virtual int Opcode() const;
  virtual bool  is_CFG() const { return (_con == TypeFunc::Control); }
  virtual bool depends_only_on_test() const { return false; }
  virtual uint ideal_reg() const;
  virtual Node *Identity( PhaseTransform *phase );
#ifndef PRODUCT
  virtual void dump_spec() const;
#endif
};


//------------------------------ReturnNode-------------------------------------
// Return from subroutine node
class ReturnNode : public Node {
public:
  ReturnNode( Node *cntrl, Node *i_o, Node *memory, Node *retadr, Node *frameptr );
  virtual int Opcode() const;
  virtual bool  is_CFG() const { return true; }
  virtual bool depends_only_on_test() const { return false; }
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint ideal_reg() const { return NotAMachineReg; }
  virtual uint match_edge(uint idx) const;
#ifndef PRODUCT
  virtual void dump_req() const;
#endif
};


//------------------------------RethrowNode------------------------------------
// Rethrow of exception at call site.  Ends a procedure before rethrowing;
// ends the current basic block like a ReturnNode.  Restores registers and
// unwinds stack.  Rethrow happens in the caller's method.
class RethrowNode : public Node {
 public:
  RethrowNode( Node *cntrl, Node *i_o, Node *memory, Node *frameptr, Node *ret_adr, Node *exception );
  virtual int Opcode() const;
  virtual bool  is_CFG() const { return true; }
  virtual bool depends_only_on_test() const { return false; }
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint match_edge(uint idx) const;
  virtual uint ideal_reg() const { return NotAMachineReg; }
#ifndef PRODUCT
  virtual void dump_req() const;
#endif
};


//------------------------------TailCallNode-----------------------------------
// Pop stack frame and jump indirect
class TailCallNode : public ReturnNode {
public:
  TailCallNode( Node *cntrl, Node *i_o, Node *memory, Node *retadr, Node *frameptr, Node *target, Node *moop );

  virtual int Opcode() const;
  virtual uint match_edge(uint idx) const;
};

//------------------------------TailJumpNode-----------------------------------
// Pop stack frame and jump indirect
class TailJumpNode : public ReturnNode {
public:
  TailJumpNode( Node *cntrl, Node *i_o, Node *memory, Node *frameptr, Node *target, Node *ex_oop);

  virtual int Opcode() const;
  virtual uint match_edge(uint idx) const;
};

//------------------------------JVMStateObj-----------------------------------
// JVM State Nodes capture Java Virtual Machine state within one scope, and
// represent a way to map the optimized program back into the JVM.
class JVMStateObj : public ResourceObj {
private:
  JVMStateObj      *_next;      // List pointer for forming scope chains
  uint16            _locoff;    // Offset to locals in input edge mapping
  uint16            _spoff;     // Offset to stack in input edge mapping
  uint16            _monoff;    // Offset to monitors in input edge mapping
  uint16            _endoff;    // Offset to end of input edge mapping
  uint16            _sp;        // Jave Expression Stack Pointer for this state
  int               _bci;       // Byte Code Index of this JVM point
  ciMethod*         _method;    // Method Pointer
  Node             *_map;       // Map node associated with this scope
public:
  friend class Compile;

  // Because JVMStateObj objects live over the entire lifetime of the
  // Compile object, they are allocated into the _comp_arena, which
  // does not get resource marked or reset during the compile process
  void *operator new( size_t x ) { return Compile::current()->_comp_arena.Amalloc(x); }
  void operator delete( void * ) { } // fast deallocation

  // Create a new JVMStateObj.  Each of these must have at lease a scope
  // and a bci.
  JVMStateObj(ciMethod* method, int bci, uint16 sp);

  // List handling functions
  JVMStateObj      *next() const { return _next; }
  void              set_next(JVMStateObj *next) { _next = next; }

  const JVMStateObj*tail() const { return _next == NULL ? this : _next->tail(); }

  // Access functions for the JVM
  uint16            locoff() const { return _locoff; }
  uint16            spoff()  const { return _spoff; }
  uint16            monoff() const { return _monoff; }
  uint16            endoff() const { return _endoff; }
  uint16            oopoff() const { assert(!_next, "must be last jvms in chain"); return endoff(); }
  uint16            sp()     const { return _sp; }
  int               bci()    const { return _bci; }
  ciMethod*         method() const { return _method; }
  Node             *map()    const { return _map; }

  // Monitors (monitors are stored as (boxNode, objNode) pairs
  enum { logMonitorEdges = 1 };
  int  nof_monitors()              const { return (endoff() - monoff()) >> logMonitorEdges; }
  int  monitor_box_offset(int idx) const { return monoff() + (idx << logMonitorEdges) + 0; }
  int  monitor_obj_offset(int idx) const { return monoff() + (idx << logMonitorEdges) + 1; }
  bool is_monitor_use(uint16 off)  const { return (off >= monoff() && off < endoff()
						   && 0 == bitfield(off - monoff(), 0, logMonitorEdges))
					     || (next() && next()->is_monitor_use(off)); }

  // Initialization functions for the JVM
  void              set_locoff(uint16 off) { _locoff = off; }
  void              set_spoff(uint16 off) { _spoff = off; }
  void              set_monoff(uint16 off) { _monoff = off; }
  void              set_endoff(uint16 off) { _endoff = off; }
  void              set_map(Node *map) { _map = map; }
  void              set_sp(uint16 sp) { _sp = sp; }
  void              set_bci(int bci) { _bci = bci; }

  // Miscellaneous utility functions
  virtual JVMStateObj *clone() const;

#ifndef PRODUCT
  virtual void      format(PhaseChaitin *chaitin, const Node *n) const;
  virtual void      dump_spec() const;
#endif
};

//------------------------------SafePointNode----------------------------------
// A SafePointNode is a subclass of a MultiNode for convenience (and
// potential code sharing) only - conceptually it is independent of
// the Node semantics.
class SafePointNode : public MultiNode {
  virtual uint           hash() const;          // Check the type
  virtual uint           cmp( const Node &n ) const;
  virtual uint           size_of() const;       // Size is bigger

public:
  SafePointNode(uint edges, JVMStateObj *jvms);
  
  OopMap           *_oop_map;   // Array of OopMap info (8-bit char) for GC
  JVMStateObj      *_jvms;      // Pointer to list of JVM State Objects
  JVMStateObj      *_jvmtail;   // Pointer to tail of list of JVM State Objects
  virtual JVMStateObj *jvms() const { return _jvms; }
  virtual JVMStateObj *jvmtail() const { return _jvmtail; }
  OopMap *oop_map() const { return _oop_map; }
  void set_oop_map(OopMap *om) { _oop_map = om; }

  // Functionality from old debug nodes which has changed
  Node *local(JVMStateObj *jvms, uint idx) const {
    return in(jvms->locoff() + idx);
  }
  Node *stack(JVMStateObj *jvms, uint idx) const {
    return in(jvms->spoff() + idx);
  }
  Node *monitor_box(JVMStateObj *jvms, uint idx) const {
    return in(jvms->monitor_box_offset(idx));
  }
  Node *monitor_obj(JVMStateObj *jvms, uint idx) const {
    return in(jvms->monitor_obj_offset(idx));
  }
  void  set_local(JVMStateObj *jvms, uint idx, Node *c) {
    set_req(jvms->locoff() + idx, c);
  }
  void  set_stack(JVMStateObj *jvms, uint idx, Node *c) {
    set_req(jvms->spoff() + idx, c);
  }
  // Handle monitor stack
  void push_monitor( const FastLockNode *lock );
  void pop_monitor ();
  Node *SafePointNode::peek_monitor_box() const;
  Node *SafePointNode::peek_monitor_obj() const;

  // Access functions for the JVM
  Node *control  () const { return in(TypeFunc::Control  ); }
  Node *i_o      () const { return in(TypeFunc::I_O      ); }
  Node *memory   () const { return in(TypeFunc::Memory   ); }
  Node *returnadr() const { return in(TypeFunc::ReturnAdr); }
  Node *frameptr () const { return in(TypeFunc::FramePtr ); }

  void set_control  ( Node *c ) { set_req(TypeFunc::Control,c); }
  void set_i_o      ( Node *c ) { set_req(TypeFunc::I_O    ,c); }
  void set_memory   ( Node *c ) { set_req(TypeFunc::Memory ,c); }

  // Standard Node stuff
  virtual int               Opcode() const;
  virtual const Type       *Value( PhaseTransform *phase ) const;
  virtual const Type       *bottom_type() const { return TypeTuple::MEMBAR; }
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual Node *Identity( PhaseTransform *phase );
  virtual uint              ideal_reg() const { return 0; }
  virtual const RegMask    &in_RegMask(uint) const;
  virtual const RegMask    &out_RegMask() const;
  virtual SafePointNode    *is_SafePoint() { return this; }
  virtual uint match_edge(uint idx) const;

#ifndef PRODUCT
  virtual void              dump_spec() const;
#endif
};

//------------------------------CallNode---------------------------------------
// Call nodes now subsume the function of debug nodes at callsites, so they
// contain the functionality of a full scope chain of debug nodes.
class CallNode : public SafePointNode {
public:
  const TypeFunc *_func;        // Function type
  const address   _entrypoint;  // Address of method being called
  virtual JVMStateObj *jvms() const { return _jvms; }

  CallNode( const TypeFunc *tf , address addr )
    : SafePointNode(tf->_domain->_cnt, NULL), _func(tf), _entrypoint(addr) {}

  virtual const Type *bottom_type() const;
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual Node *Identity( PhaseTransform *phase ) { return this; }
  virtual uint        hash() const;
  virtual uint        cmp( const Node &n ) const;
  virtual uint        size_of() const = 0;
  virtual int         Opcode() const;
  virtual void        calling_convention( uint16 *parm_regs, uint argcnt ) const;
  virtual Node       *match( const ProjNode *proj, const Matcher *m );
  virtual uint        ideal_reg() const { return NotAMachineReg; }

  // Node casting functions
  virtual       CallNode             *is_Call() { return this; }
  virtual const CallJavaNode         *is_CallJava() const { return 0; }
  virtual const CallStaticJavaNode   *is_CallStaticJava() const { return 0; }
  virtual const CallDynamicJavaNode  *is_CallDynamicJava() const { return 0; }
  virtual const CallCompiledJavaNode *is_CallCompiledJava() const { return 0; }
  virtual const CallInterpreterNode  *is_CallInterpreter() const { return 0; }
  virtual const CallRuntimeNode      *is_CallRuntime() const { return 0; }
  virtual const CallLeafNode         *is_CallLeaf() const { return 0; }  

  // Detect and repair outgoing double/long args which are not in
  // even/odd register or stackslot pairs.  (Use Half[01] projections.)
  void fix_mispaired_double_longs();

  virtual uint match_edge(uint idx) const;

#ifndef PRODUCT
  virtual void        dump_req()  const;
  virtual void        dump_spec() const;
#endif
};

//------------------------------CallJavaNode-----------------------------------
// Make a static or dynamic subroutine call node using Java calling
// convention.  (The "Java" calling convention is the compiler's calling
// convention, as opposed to the interpreter's or that of native C.)
class CallJavaNode : public CallNode {
protected:
  virtual uint hash() const;    // Check the type
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const; // Size is bigger

  bool    _optimized_virtual;
public:
  ciMethod*  _method;      // Method being direct called
  const int  _bci;         // Byte Code Index of call byte code
  CallJavaNode( const TypeFunc *tf , address addr, ciMethod* method, int bci ) : CallNode(tf,addr), _method(method), _bci(bci), _optimized_virtual(false) { }

  virtual int   Opcode() const;
  virtual const CallJavaNode *is_CallJava() const { return this; }
  void  set_optimized_virtual(bool f)     { _optimized_virtual = f; }
  bool  is_optimized_virtual() const      { return _optimized_virtual; }

#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};

//------------------------------CallStaticJavaNode-----------------------------
// Make a direct subroutine call using Java calling convention (for static
// calls and optimized virtual calls, plus calls to wrappers for run-time
// routines); generates static stub.
class CallStaticJavaNode : public CallJavaNode {
  virtual uint hash() const;    // Check the type
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const; // Size is bigger
public:
  CallStaticJavaNode( const TypeFunc *tf, address addr, ciMethod* method, int bci ) : CallJavaNode(tf,addr,method,bci), _name(0) {}
  CallStaticJavaNode( const TypeFunc *tf, address addr, const char *name, int bci ) : CallJavaNode(tf,addr,0,bci), _name(name) {}  
  const char *_name;            // Runtime wrapper name

  virtual int         Opcode() const;
  virtual const       CallStaticJavaNode *is_CallStaticJava() const { return this; }
#ifndef PRODUCT
  virtual void        dump_spec() const;
#endif
};

//------------------------------CallDynamicJavaNode----------------------------
// Make a dispatched call using Java calling convention.
class CallDynamicJavaNode : public CallJavaNode {
  virtual uint hash() const;    // Check the type
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const; // Size is bigger
public:
  CallDynamicJavaNode( const TypeFunc *tf , address addr, ciMethod* method, int bci ) : CallJavaNode(tf,addr,method,bci) {}

  virtual int   Opcode() const;
  virtual const CallDynamicJavaNode *is_CallDynamicJava() const { return this; }
#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};

//------------------------------CallCompiledJavaNode---------------------------
// Make a direct subroutine call node from interpreter to compiled java
class CallCompiledJavaNode : public CallNode {
  virtual uint size_of() const;
public:
  CallCompiledJavaNode( const TypeFunc *tf , address addr ) : CallNode(tf,addr) {}

  virtual int   Opcode() const;
  virtual const CallCompiledJavaNode *is_CallCompiledJava() const { return this; }
  virtual void  calling_convention( uint16 *parm_regs, uint argcnt ) const;
#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};

//------------------------------CallInterpreterNode----------------------------
// Make a direct subroutine call node into the interpreter
class CallInterpreterNode : public CallNode {
  virtual uint size_of() const;
public:
  CallInterpreterNode( const TypeFunc *tf , address addr ) : CallNode(tf,addr) {}

  virtual int   Opcode() const;
  virtual const CallInterpreterNode *is_CallInterpreter() const { return this; }
  // Handle additional parameters, inline_cached_klass and method_oop
  virtual void  calling_convention( uint16 *parm_regs, uint argcnt ) const;
#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};

//------------------------------CallRuntimeNode--------------------------------
// Make a direct subroutine call node into compiled C++ code.
class CallRuntimeNode : public CallNode {
  virtual uint hash() const;    // Check the type
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const; // Size is bigger
public:
  CallRuntimeNode( const TypeFunc *tf, address addr, const char *name ) : CallNode(tf,addr), _name(name) {}

  const char *_name;            // Printable name, if _method is NULL
  virtual int   Opcode() const;
  virtual const CallRuntimeNode *is_CallRuntime() const { return this; }
  virtual void  calling_convention( uint16 *parm_regs, uint argcnt ) const;
#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};

//------------------------------CallLeafNode-----------------------------------
// Make a direct subroutine call node into compiled C++ code, without
// safepoints
class CallLeafNode : public CallRuntimeNode {
public:
  CallLeafNode( const TypeFunc *tf, address addr, const char *name ) : 
      CallRuntimeNode(tf,addr,name){}
  virtual int   Opcode() const;
  virtual const CallLeafNode *is_CallLeaf() const { return this; }
#ifndef PRODUCT
  virtual void  dump_spec() const;
#endif
};


//------------------------------CallNativeNode---------------------------------
// Make a direct subroutine call node into Native code.
class CallNativeNode : public CallRuntimeNode {
public:
  CallNativeNode( const TypeFunc *tf, address addr, const char *name ) : CallRuntimeNode(tf,addr,name) {}

  virtual int Opcode() const;
};



