/*
 * @(#)compile.cpp	1.479 01/03/05 10:09:01
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_compile.cpp.incl"

ciMethod* Compile::get_String_compareTo() {
  if (_String_compareTo == NULL) {
    _String_compareTo =
      env()->String_klass()->find_method("compareTo", "(Ljava/lang/String;)I");
    assert(_String_compareTo != NULL, "must be able to find");
  }
  return _String_compareTo;
}


ciMethod* Compile::get_Object_hashCode() {
  if (_Object_hashCode == NULL) {
    _Object_hashCode =
      env()->Object_klass()->find_method("hashCode", "()I");
    assert(_Object_hashCode != NULL, "must be able to find");
  }
  return _Object_hashCode;
}


ciMethod* Compile::get_System_identityHashCode() {
  if (_System_identityHashCode == NULL) {
    ciInstanceKlass* system =
      env()->find_system_klass("java/lang/System")->as_instance_klass();
    assert(system->is_loaded(), "must be able to find");
    _System_identityHashCode =
      system->find_method(ciSymbol::make("identityHashCode"),
			  ciSymbol::make("(Ljava/lang/Object;)I"));
    assert(_System_identityHashCode != NULL, "must be able to find");
  }
  return _System_identityHashCode;
}


ciMethod* Compile::get_System_arraycopy() {
  if (_System_arraycopy == NULL) {
    ciInstanceKlass* system =
      env()->find_system_klass("java/lang/System")->as_instance_klass();
    assert(system->is_loaded(), "must be able to find");
    _System_arraycopy =
      system->find_method("arraycopy",
                          "(Ljava/lang/Object;ILjava/lang/Object;II)V");
    assert(_System_arraycopy != NULL, "must be able to find");
  }
  return _System_arraycopy;
}



ciInstanceKlass* Compile::get_Math() {
  if (_Math == NULL) {
    _Math = env()->find_system_klass("java/lang/Math")->as_instance_klass();
  }
  return _Math;
}

ciMethod* Compile::get_Math_sin() {
  if (_Math_sin == NULL) {
    _Math_sin = get_Math()->find_method("sin", "(D)D");
    assert(_Math_sin != NULL, "must be able to find");
  }
  return _Math_sin;
}


ciMethod* Compile::get_Math_cos() {
  if (_Math_cos == NULL) {
    _Math_cos = get_Math()->find_method("cos", "(D)D");
    assert(_Math_cos != NULL, "must be able to find");
  }
  return _Math_cos;
}


ciMethod* Compile::get_Math_tan() {
  if (_Math_tan == NULL) {
    _Math_tan = get_Math()->find_method("tan", "(D)D");
    assert(_Math_tan != NULL, "must be able to find");
  }
  return _Math_tan;
}


ciMethod* Compile::get_Math_atan2() {
  if (_Math_atan2 == NULL) {
    _Math_atan2 = get_Math()->find_method("atan2", "(DD)D");
    assert(_Math_atan2 != NULL, "must be able to find");
  }
  return _Math_atan2;
}


ciMethod* Compile::get_Math_sqrt() {
  if (_Math_sqrt == NULL) {
    _Math_sqrt = get_Math()->find_method("sqrt", "(D)D");
    assert(_Math_sqrt != NULL, "must be able to find");
  }
  return _Math_sqrt;
}

ciMethod* Compile::get_Class_newInstance() {
  if (_Class_newInstance == NULL) {
    _Class_newInstance =
      env()->Class_klass()->find_method("newInstance", "()Ljava/lang/Object;");
    assert(_Class_newInstance != NULL, "must be able to find");
  }
  return _Class_newInstance;
}


ciMethod* Compile::get_Class_newInstance0() {
  if (_Class_newInstance0 == NULL) {
    _Class_newInstance0 =
      env()->Class_klass()->find_method("newInstance0", "()Ljava/lang/Object;");
    assert(_Class_newInstance0 != NULL, "must be able to find");
  }
  return _Class_newInstance0;
}

// Support for bundling info
Bundle* Compile::node_bundling(const Node *n) {
  return &_node_bundling[n->_idx];
}

bool Compile::valid_bundle_info(const Node *n) {
  return (_node_bundle_limit > n->_idx);
}

// ============================================================================
//------------------------------CompileWrapper---------------------------------
class CompileWrapper : public StackObj {
  Compile *const _compile;
 public:
  CompileWrapper(Compile* compile);

  ~CompileWrapper();
};
  
CompileWrapper::CompileWrapper(Compile* compile) : _compile(compile) {
  COMPILER1_ONLY(ShouldNotReachHere());
  ciEnv* env = compile->env();
  assert(env->data() == NULL, "compile already active?");
  ciEnv::set_data(compile);

  compile->set_typedict(NULL);
  compile->set_type_arena(new Arena());
  compile->set_type_hwm(NULL);
  compile->set_type_last_size(0);
  compile->set_node_unique_top(NULL);
  compile->set_indexSet_arena(NULL);
  compile->set_indexSet_free_block_list(NULL);
  Type::Initialize(compile);
}
CompileWrapper::~CompileWrapper() {
  delete _compile->type_arena();
  ciEnv::set_data(NULL);
}

// This method is called during the compilation of on stack replacement code.
void Compile::reset_state_for_second_parse() {
  _unique  = 0;
  _chaitin = 0;
  _sync    = 0;

  // Globally visible Nodes
  _root = new RootNode();
  _top_node = new (1) ConNode(Type::TOP);
}


//------------------------------find_deopts------------------------------------
// Find if we can _ever_ de-optimize in this method.  Very conservative;
// requires _no_ inlining of monomorphic not-final calls.  Even then, it only
// cuts out debug info at SafePoints.  Calls still need debug info because a
// not-final monomorphic leaf call might use CHA to optimize the call site.
// Class loading would then attempt to deoptimize us so the call site can be
// recompiled.
static bool find_deopts( ciScope *scope ) {
  if( !scope->method()->flags().is_final() &&
      !scope->method()->holder()->flags().is_final() &&
      !scope->method()->flags().is_private() )
    return true;
  const int len = scope->number_of_subscopes();
  for( int i = 0; i < len; i++ ) 
    if( find_deopts( scope->subscope_at(i) ) ) 
      return true;
  return false;
}

//------------------------------bailouts_and_breaks----------------------------
// Check for various debugging bail-out conditions and breakpoints.
InlineTree *Compile::bailouts_and_breaks() {
#ifndef PRODUCT
  // Check if recompiling
  if( _subsume_loads == false && (PrintOpto || PrintOptoBailouts) ) {
    // Recompiling without allowing machine instructions to subsume loads
    tty->print_cr("*********************************************************");
    tty->print_cr("** Bailout: Recompile without subsuming loads          **");
    tty->print_cr("*********************************************************");
  }
  if (env()->break_at_compile()) {
    // Open the debugger when compiing this method.
    tty->print("### Breaking when compiling: ");
    method()->print_short_name();
    tty->cr();
    BREAKPOINT;
  }

  //if (UseInlineImage) {
  //  if (InlineImage::query_method(_method) == NULL) {
  //    // This method should not be compiled.
  //    tty->print("### Excluding compile: ");
  //    _method->print_short_name(tty);
  //    tty->cr();
  //    _root = NULL;
  //    return NULL;    
  //  }
  //}

  if( PrintOpto ) {
    if (is_osr_compilation()) {
      tty->print("[OSR]%3d", _compile_id);
    } else {
      tty->print("%3d", _compile_id);
    }
  }
#endif
  
  // We will attempt to see if a class/field/etc got properly loaded.  If it
  // did not, it may attempt to throw an exception during our probing.  Catch
  // and ignore such exceptions and do not attempt to compile the method.
  EXCEPTION_MARK;
  InlineTree *ilt = new InlineTree(env(), method(), -1, 1.0f);
  if( ilt->find_block_starts( _entry_bci == -1 ? 0 : _entry_bci, THREAD ) != Parse::Bailout_none ) {
    // Top level pass1 parse failed
    CLEAR_PENDING_EXCEPTION;
    _root = NULL;
    return NULL;
  }

  // Top-level pass1 parse succeeds
  return ilt;
}

// ============================================================================
//------------------------------Compile standard-------------------------------
debug_only( int Compile::_debug_idx = 0; )

// Compile a method.  entry_bci is -1 for normal compilations and indicates
// the continuation bci for on stack replacement.


Compile::Compile( ciEnv* ci_env, ciScope *ci_scope, ciMethod* target, int osr_bci, bool subsume_loads, bool reuse_env )
                : Phase(Compiler),
                  _env(ci_env),
		  _compile_id(ci_env->compile_id()),
		  _name(NULL),
		  _C_function(NULL),
                  _method(target),
                  _scope(ci_scope),
                  _sync(0),
                  _max_arg_size(0),
                  _entry_bci(osr_bci),
                  _has_loops(false),
                  _has_split_ifs(false),
                  _deopt_happens(true),
                  _subsume_loads(subsume_loads),
                  _result(Comp_success),
                  _node_size(NULL),
                  _node_bundle_limit(0),
                  _node_bundling(NULL),
                  _map_adr_mem(Thread::current()->resource_area()) {
  CompileWrapper cw(this);
#ifndef PRODUCT
  if (TimeCompiler2) {
    tty->print(" ");
    target->holder()->name()->print();
    tty->print(".");
    target->print_short_name();
    tty->print("  ");
  }
#endif
  TraceTime t1("Total compilation time", &_t_totalCompilation, TimeCompiler, TimeCompiler2);
  TraceTime t2(NULL, &_t_methodCompilation, TimeCompiler, false);
  NOT_PRODUCT( FlagSetting print_opto_assembly(PrintOptoAssembly, (PrintOptoAssembly || _method->should_print_assembly())); )

  init(is_osr_compilation() ? 1 : AliasLevel);

  // Check for various bail-out flags and conditions.
  InlineTree *ilt = bailouts_and_breaks();
  if( !ilt ) return;

  // Pass-1 of the parser for inlines.
  ilt->inline_policy( scope() );
  // If NO memory addresses are used, make at least one so
  // MergeMem nodes have at least 1 input
  if( _cnt_map_adr_mem <= 1 )
    set_alias_index( TypeRawPtr::BOTTOM );

#define MINIMUM_NODE_HASH  1023
  uint estimated_size = method()->code_size()*4+64;
  estimated_size = (estimated_size < MINIMUM_NODE_HASH ? MINIMUM_NODE_HASH : estimated_size);
  PhaseGVN gvn( &_arena, estimated_size);

  { // Scope for timing the parser
    TraceTime t3(NULL, &_t_parser, TimeCompiler, 0);

    OSRParseInfo *osr_info = NULL;

    // If we are performing a compilation for on stack replacement, we first
    // perform a throwaway parse to determine the type information at our
    // entry bci.
    if (is_osr_compilation()) {
      EXCEPTION_MARK;
      InlineTree *ilt_osr = new InlineTree(env(), method(), -1, 1.0f);
      if( ilt_osr->find_block_starts( 0, THREAD ) != Parse::Bailout_none ) {
        // Top level pass1 parse failed
        CLEAR_PENDING_EXCEPTION;
        _root = NULL;
        return;
      }
      osr_info = new OSRParseInfo(method(), entry_bci());

      // Throwaway parse.
      PhaseGVN gvn2( Thread::current()->resource_area(), method()->code_size()*4+64);
      set_parse_gvn(&gvn2);
      Unique_Node_List throwaway_node_list;   
      Parse first_parser(env(), ilt_osr, gvn2, osr_info, throwaway_node_list);
      
      if( TraceOptoParse ) { tty->print_cr("\nFinished first_osr_parse\n"); }

      // Check to see if we encountered trouble during our throwaway parse.
      if (!osr_info->is_complete()) {
        if (PrintOpto) {
          tty->print("---"); 
          method()->print_short_name();
          tty->cr();
        }
        _tf = first_parser._tf;
        _root = NULL;
        return;
      }

      // Start the second parse with a clean slate.
      reset_state_for_second_parse();
    }

    // Node list that Iterative GVN will start with
    set_parse_gvn(&gvn);
    Unique_Node_List for_igvn(&_comp_arena);
    set_for_igvn( &for_igvn );

    // This is the real parse.
    Parse parser(env(), ilt, gvn, osr_info, for_igvn);
    _tf = parser._tf;             // Grab signature for matching/allocation
    if( !parser.parse_succeeded() ) {
      _root = NULL;               // Bailed out during parsing
      return;
    }
    _sync = parser._sync_max;
  }

  // Give up now on large methods
  if( _unique > (uint)NodeCountCutoff ) _root = NULL;

  // !!!!! !!!!! !!!!! DEBUG - Dump the Call Frequency Table
  if (TraceFrequencyInlining && CountInterpCalls && Verbose) {
    SharedInfo::dump_callTable();
  }

  // !!!!! !!!!! !!!!! DEBUG - Dump the Branch Frequency Table
  if (TraceFrequencyInlining && CountInterpBranches && Verbose) {
    int zcnt = 0, tcnt = 0, ncnt = 0;
    for(int x = 0; x < (0x1 << BranchFreqTableSize); x++) {
      if ((SharedInfo::freqTable[x]._taken == 0) &&
           (SharedInfo::freqTable[x]._not_taken == 0)) {
	zcnt++;
      }
      if ((SharedInfo::freqTable[x]._taken != 0) &&
	  (SharedInfo::freqTable[x]._not_taken == 0)) {
	tcnt++;
      }
      if ((SharedInfo::freqTable[x]._taken == 0) &&
	  (SharedInfo::freqTable[x]._not_taken != 0)) {
	ncnt++;
      }
    }
    double zpercent = ((double)zcnt / (double)(0x1 << BranchFreqTableSize)) * 100;
    double tpercent = ((double)tcnt / (double)(0x1 << BranchFreqTableSize)) * 100;
    double npercent = ((double)ncnt / (double)(0x1 << BranchFreqTableSize)) * 100;
    tty->print("Branch Table Density = %f%% empty,", zpercent);
    tty->print(" %f%% taken,", tpercent);
    tty->print(" %f%% not taken\n", npercent);
  }

  // Now optimize and generate code
  if (_root) Optimize();
  if (_root) Code_Gen(false);

  // Check if we want to skip execution of all compiled code.  
  if (OptoNoExecute) _root = NULL;  // Flag as failed

  if ( _root ) {
    int return_address_offset = _frame_words - 1;

    const int ep_offset   = 0;
    const int vep_offset  = is_osr_compilation() ? 0 : _first_block_size;
    const int code_offset = _first_block_size;
    const int osr_offset  = is_osr_compilation() ? _first_block_size : 0;
    env()->register_method(_method, 
                           _entry_bci,
                           ep_offset, ep_offset, vep_offset, code_offset, osr_offset,
                           _recorder, _code_buffer, 
                           _frame_words, return_address_offset, _oop_map_set, 
                           &_handler_table, &_inc_table);
  }
}

//------------------------------Compile----------------------------------------
// Compile a runtime stub
Compile::Compile( ciEnv* ci_env,
		  TypeFunc_generator generator,
		  address C_function,
		  const char *name,
		  int is_fancy_jump,
		  bool pass_tls,
		  bool save_arg_registers,
		  bool return_pc )
  : Phase(Compiler),
    _env(ci_env),
    _compile_id(-1),
    _sync(0),
    _max_arg_size(0),
    _has_loops(false),
    _has_split_ifs(false),
    _deopt_happens(true),
    _method(NULL),
    _name(name),
    _C_function(C_function),
    _scope(NULL),
    _entry_bci(InvocationEntryBci),
    _subsume_loads(true),
    _result(Comp_success),
    _node_bundle_limit(0),
    _node_bundling(NULL),
    _map_adr_mem(Thread::current()->resource_area()) {

  TraceTime t1(NULL, &_t_totalCompilation, TimeCompiler, false);
  TraceTime t2(NULL, &_t_stubCompilation, TimeCompiler, false);
  CompileWrapper cw(this);
  init(AliasLevel);
  const TypeFunc* j_sig = (*generator)();
  Generate_Stub_Graph( j_sig, C_function, name, NULL, NULL, is_fancy_jump, false, false, pass_tls, return_pc );
  _tf = j_sig;
  Code_Gen(save_arg_registers);
}

//------------------------------Compile an adapter----------------------------
// Compile an adapter routine.
Compile::Compile( ciEnv* ci_env, ciMethod* method,
		  int /*AdapterKind*/ kind,
		  bool reuse_env )
  : Phase(Compiler),
    _env(ci_env),
    _compile_id(ci_env->compile_id()),
    _sync(0),
    _max_arg_size(0),
    _has_loops(false),
    _has_split_ifs(false),
    _deopt_happens(true),
    _method(NULL),
    _name(NULL),
    _C_function(NULL),
    _scope(NULL),
    _entry_bci(InvocationEntryBci),
    _subsume_loads(true),
    _result(Comp_success),
    _node_bundle_limit(0),
    _node_bundling(NULL),
    _map_adr_mem(Thread::current()->resource_area()) {
  TraceTime t1(NULL, &_t_totalCompilation, TimeCompiler, false);
  TraceTime t2(NULL, &_t_adapterCompilation, TimeCompiler, false);
  CompileWrapper cw(this);
  init(AliasLevel);
  // Check for too many adapter arguments.

  int old_SP = SharedInfo::stack2reg( round_to( in_preserve_stack_slots() + (_sync * sync_stack_slots()), WordsPerLong ) );
  if( method->arg_size() >= (int)(((RM_SIZE*32 - (old_SP + (2 * out_preserve_stack_slots())))>>1)-1) )
	return;

  const TypeFunc* j_sig = TypeFunc::make(method);
  if (kind == ciEnv::i2c) {
    _name = "i2c";
#ifndef PRODUCT
    if(PrintOpto && Verbose) {
    tty->print("i2c   ");
      j_sig->print_flattened();
    tty->cr();
    }
#endif
    Generate_Interpreter_To_Compiled_Graph(j_sig);
    Code_Gen(false);
    int return_address_offset = _frame_words - 1;
    env()->register_i2c_adapter(method,
                                _oop_map_set,
			        _code_buffer,
			        _frame_words,
			        return_address_offset);
  } else {
    assert (kind == ciEnv::c2i, "unknown adapter kind");
    _name = "c2i";
#ifndef PRODUCT
    if(PrintOpto && Verbose) {
      tty->print("c2i   ");
      j_sig->print_flattened();
      tty->cr();
    }
#endif
    Generate_Compiled_To_Interpreter_Graph(j_sig, method->interpreter_entry());
    Code_Gen(false);

    // On Intel, the return address is always on the stack as the first
    // blob in the 'preserve' area.  This is not true in general; we need
    // to get the return address via a more generic mechanism.
    int return_address_offset = _frame_words - 1;
    env()->register_c2i_adapter(method,
                                _oop_map_set,
                                _code_buffer,
                                _first_block_size,
                                _frame_words,
                                return_address_offset);
  }
}

//------------------------------Compile native---------------------------------
// Compile a native method
Compile::Compile( ciEnv* ci_env, ciMethod* method )
		: Phase(Compiler),
                  _env(ci_env),
		  _compile_id(ci_env->compile_id()),
		  _sync(0),
                  _max_arg_size(0),
		  _has_loops(false),
                  _has_split_ifs(false),
		  _deopt_happens(true),
		  _method(method),
		  _name(NULL),
		  _C_function(NULL),
		  _scope(NULL),
		  _entry_bci(InvocationEntryBci),
		  _subsume_loads(true),
		  _result(Comp_success),
                  _node_bundle_limit(0),
                  _node_bundling(NULL),
		  _map_adr_mem(Thread::current()->resource_area()) {
  TraceTime t1(NULL, &_t_totalCompilation, TimeCompiler, false);
  TraceTime t2(NULL, &_t_nativeCompilation, TimeCompiler, false);
  CompileWrapper cw(this);
  init(AliasLevel);
  assert( method->is_native(), "sanity check" );

#ifndef PRODUCT
  if (env()->break_at_compile()) {
    // Open the debugger when compiling this method.
    tty->print("### Breaking when compiling: ");
    method->print_short_name();
    tty->cr();
    BREAKPOINT;
  }

  if( PrintOpto ) {             // Print when we attempt compilation
    tty->print("%3d*%c", _compile_id,
		 method->flags().is_synchronized() ? 's' : ' ');
    method->print_short_name();
    tty->cr();    
  }  
#endif

  const char *rstr = method->name()->as_utf8();
  char *str = NEW_ARENA_ARRAY( (&_comp_arena), char, strlen(rstr) + 1 );
  strcpy( str, rstr );

  // Compute Java signature
  _tf = TypeFunc::make(method);
  
  address native_entry = method->native_entry();
  assert(native_entry != NULL, "bad native entry");
  Generate_Stub_Graph( _tf, native_entry, str, method, 
               method->flags().is_static() ? method->holder() : NULL,
               false, true, true, false, false );
  Code_Gen(false);

    int return_address_offset = _frame_words - 1;

    const int ep_offset   = 0;
    const int vep_offset  = is_osr_compilation() ? 0 : _first_block_size;
    const int code_offset = _first_block_size;
    const int osr_offset  = is_osr_compilation() ? _first_block_size : 0;
    env()->register_method(method, 
                           _entry_bci,
                           ep_offset, ep_offset, vep_offset, code_offset, osr_offset,
                           _recorder, _code_buffer, 
                           _frame_words, return_address_offset, _oop_map_set, 
                           &_handler_table, &_inc_table);
}

void Compile::print_codes() { 
  if( scope() )
    scope()->print_codes(1); 
}

//------------------------------init-------------------------------------------
// Prepare for a single compilation
void Compile::init( int aliaslevel ) {
  _unique  = 0;
  _chaitin = 0;

  set_24_bit_selection_and_mode(Use24BitFP, false);

  // Capture status of JVMPI once for the compilation
  _jvmpi_compiled_method_event = jvmpi::is_event_enabled(JVMPI_EVENT_COMPILED_METHOD_LOAD);
  _jvmpi_method_entry_event    = jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY);
  _jvmpi_method_entry2_event   = jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY2);
  _jvmpi_method_exit_event     = jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_EXIT);

  // Globally visible Nodes
  _root = new RootNode();
  _top_node = new (1) ConNode(Type::TOP);
  // Create Debug Information Recorder to record scopes, oopmaps, etc.
  _recorder = new DebugInformationRecorder(new OopRecorder(env()->arena()));

  // // -- Initialize types before each compile --
  // // Update cached type information
  // if( _method && _method->constants() ) 
  //   Type::update_loaded_types(_method, _method->constants());

  // Init the map_adr_mem; the mapping from address types to memory Nodes.
  _map_adr_mem.map(0,NULL);
  _cnt_map_adr_mem = 1;
  _AliasLevel = aliaslevel;
  switch( _AliasLevel ) {
  case 0: 
    set_alias_index( TypeRawPtr::BOTTOM );
    break;
  case 1:                       // Flatten to: oop, static, field or array
    set_alias_index( TypeRawPtr::BOTTOM );
    set_alias_index( TypeOopPtr::BOTTOM );
    set_alias_index( TypeKlassPtr::OBJECT );
    set_alias_index( TypeAryPtr::RANGE );
    break;
  case 2:                       // Bonus spare type for any missed bits
    set_alias_index( TypeRawPtr::BOTTOM );
    break;
  }

  // Walk the inline tree and see if any methods _can_ be overridden.  If the
  // answer is NO, then we do not need debug info at safepoints.  We iterate
  // over the top level because we do not care if the top level can be
  // de-opt'd; that will happen in HIS caller.
  if( _scope && !DeoptimizeALot ) {
    _deopt_happens = false;
    const int len = _scope->number_of_subscopes();
    for( int i = 0; i < len; i++ ) 
      if( find_deopts( _scope->subscope_at(i) ) ) 
        _deopt_happens = true;
  }

  // JVMPI support
  set_jvmpi_lines( NULL );
  _linenumbers_offset = (is_set_jvmpi_method_event() && _method != NULL) ?
    new LineNumbersOffset( this ) :
    NULL;

  _String_compareTo = NULL;
  _Object_hashCode = NULL;
  _System_identityHashCode = NULL;
  _System_arraycopy = NULL;
  _Class_newInstance = NULL;
  _Class_newInstance0 = NULL;
  _Math = NULL;
  _Math_sin = NULL;
  _Math_cos = NULL;
  _Math_tan = NULL;
  _Math_atan2 = NULL;
  _Math_sqrt = NULL;
}

StartNode *Compile::start() const { 
  if( !_root ) return NULL;
  Node *rethrow_or_ret = _root->in(1);
  Node *frame = rethrow_or_ret->in(TypeFunc::FramePtr);
  StartNode *start = frame->in(0)->is_Start();
  assert( start, "Expect a start node");
  return start;
}


//------------------------------flatten_alias_type-----------------------------
const TypePtr *Compile::flatten_alias_type( const TypePtr *tj ) const {
  int offset = tj->_offset;
  TypePtr::PTR ptr = tj->_ptr;

  // Array pointers need some flattening
  const TypeAryPtr *ta = tj->isa_aryptr();
  if( ta && _AliasLevel >= 2 ) {
    // For arrays indexed by constant indices, we flatten the alias
    // space to include all of the array body.  Only the header, klass
    // and array length can be accessed un-aliased.
    if( offset != Type::OffsetBot ) {
      if( offset == arrayOopDesc::length_offset_in_bytes() ) {
        // range is OK as-is.
        tj = ta = TypeAryPtr::RANGE; 
      } else if( offset == oopDesc::klass_offset_in_bytes() ) {
        tj = TypeOopPtr::HEADER; // all klass loads look alike
        ta = TypeAryPtr::RANGE; // generic ignored junk
        ptr = TypePtr::BotPTR;
      } else if( offset == oopDesc::mark_offset_in_bytes() ) {
        tj = TypeOopPtr::BOTTOM->add_offset( offset );
        ta = TypeAryPtr::RANGE; // generic ignored junk
        ptr = TypePtr::BotPTR;
      } else {                  // Random constant offset into array body 
        offset = Type::OffsetBot;   // Flatten constant access into array body
        tj = ta = TypeAryPtr::make(ptr,ta->_ary,ta->klass(),Type::OffsetBot);
      }
    }
    // Arrays of fixed size alias with arrays of unknown size.
    if( ta->_ary->_size != TypeInt::POS ) {
      const TypeAry *tary = TypeAry::make( ta->_ary->_elem, TypeInt::POS );
      tj = ta = TypeAryPtr::make(ptr,tary,ta->klass(),offset);
    }
    // Arrays of known objects become arrays of unknown objects.
    if( ta->_ary->_elem->isa_javaptr() && ta->_ary->_elem!=TypeOopPtr::BOTTOM){
      const TypeAry *tary = TypeAry::make( TypeOopPtr::BOTTOM,ta->_ary->_size);
      tj = ta = TypeAryPtr::make(ptr,tary,NULL,offset);
    }
    // Arrays of bytes and of booleans both use 'bastore' and 'baload' so
    // cannot be distinguished by bytecode alone.
    if( ta->_ary->_elem == TypeInt::BOOL ) {
      const TypeAry *tary = TypeAry::make( TypeInt::BYTE, ta->_ary->_size );
      tj = ta = TypeAryPtr::make(ptr,tary,NULL,offset);
    }
    // During the 2nd round of IterGVN, NotNull castings are removed.
    // Make sure the Bottom and NotNull variants alias the same.
    if( ptr == TypePtr::NotNull ) 
      tj = ta = TypeAryPtr::make(TypePtr::BotPTR,ta->_ary,NULL,offset);
  }

  // Oop pointers need some flattening
  const TypeOopPtr *to = tj->isa_oopptr();
  if( to && _AliasLevel >= 2 ) {
    if( ptr == TypePtr::Constant ) {
      // No constant oop pointers (such as Strings); they alias with
      // unknown strings.
      tj = to = TypeOopPtr::make(TypePtr::BotPTR,to->klass(),0,offset);
    } else if( ptr == TypePtr::NotNull ) {
      // During the 2nd round of IterGVN, NotNull castings are removed.
      // Make sure the Bottom and NotNull variants alias the same.
      tj = to = TypeOopPtr::make(TypePtr::BotPTR,to->klass(),0,offset);
    }
    // Canonicalize the the holder of this field
    ciInstanceKlass *k = to->klass()->as_instance_klass();
    assert(offset % 4 == 0, "offset must be multiple of 4");
    ciInstanceKlass *canonical_holder = k->get_canonical_holder(offset/4);
    if (!k->equals(canonical_holder)) {
      tj = to = TypeOopPtr::make(TypePtr::BotPTR, canonical_holder, 0, offset);
    }
  }

  // Klass pointers to object array klasses need some flattening
  const TypeKlassPtr *tk = tj->isa_klassptr();
  if( tk ) {
    ciKlass* klass = tk->klass();
    if( klass->is_obj_array_klass() ) {
      ciKlass* k = TypeAryPtr::OOPS->klass();
      if( !k || !k->is_loaded() )                  // Only fails for some -Xcomp runs
        k = TypeOopPtr::BOTTOM->klass();
      tj = tk = TypeKlassPtr::make( TypePtr::NotNull, k, offset );
    }
  }

  // Flatten all Raw pointers together.
  if( tj->_base == Type::RawPtr )
    tj = TypeRawPtr::BOTTOM;

  assert( tj->_base != Type::AnyPtr, "Bare Ptr not allowed" );

  // Flatten all to bottom for now
  switch( _AliasLevel ) {
  case 0: 
    tj = TypePtr::BOTTOM; 
    break;
  case 1:                       // Flatten to: oop, static, field or array
    switch( tj->_base ) {
    //case Type::AryPtr: tj = TypeAryPtr::RANGE;    break;
    case Type::RawPtr:   tj = TypeRawPtr::BOTTOM;   break;
    case Type::AryPtr:   // OSR methods cannot distinguish arrays at all
    case Type::OopPtr:   tj = TypeOopPtr::BOTTOM;   break;
    case Type::KlassPtr: tj = TypeKlassPtr::OBJECT; break;
    default: ShouldNotReachHere();
    }
    break;
  case 2:                       // No collasping at level 2; keep all splits
    break;
  default:
    Unimplemented();
  } 

  offset = tj->_offset;
  assert( offset != Type::OffsetTop, "Offset has fallen from constant" );
  
  assert( (offset != Type::OffsetBot && tj->_base != Type::AryPtr) ||
          (offset == Type::OffsetBot && tj->_base == Type::AryPtr) ||
          (offset == 0 && tj->_base == Type::AryPtr) ||
          (offset == 4 && tj->_base == Type::AryPtr) ||
          (offset == 8 && tj->_base == Type::AryPtr)  , 
          "For oops, klasses, raw offset must be constant; for arrays the offset is never known" );
  assert( tj->_ptr != TypePtr::TopPTR &&
          tj->_ptr != TypePtr::AnyNull &&          
          tj->_ptr != TypePtr::Null, "No imprecise addresses" );
  assert( tj->_ptr != TypePtr::Constant ||
          tj->_base == Type::RawPtr ||
          tj->_base == Type::KlassPtr, "No constant oop addresses" );

  return tj;
}

//------------------------------get_alias_index--------------------------------
// Return the alias index from this address type
uint Compile::get_alias_index( const TypePtr *tj ) const {

  // Flatten alias type into disjoint equivalence classes
  const TypePtr *flat = flatten_alias_type( tj );

  // Linear scan.  Could be a hash, but I am assuming that in general I'll
  // have so few unique address types that the scan will be fast.  Average 
  // size from _213_javac looks to be about 6.
  for( uint i = 1; i < _cnt_map_adr_mem; i++ ) 
    if( _map_adr_mem.fast_lookup(i) == flat )
      return i;

  // Not found.  Hitting this means I missed an alias type during
  // the pass-1 bytecode parsing.
#ifndef PRODUCT
  if ( PrintOpto && Verbose ) {
    // %%%% This should be a hard assert, not just a warning. %%%%
    tty->print_cr("Found a new type in parser pass 2 not recorded in pass 1");
    for(uint i = 1; i < _cnt_map_adr_mem; i++) {
      tty->print("%i) ",i);
      _map_adr_mem.fast_lookup(i)->dump();
      tty->cr();
    }
    tty->print("Original  type: "); tj  ->dump(); tty->cr();
    tty->print("Flattened type: "); flat->dump(); tty->cr();
  }
#endif

  // Index 1 is for leftover bits
  return 1;
}

//------------------------------set_alias_index--------------------------------
// Record the alias index for this address type
void Compile::set_alias_index( const TypePtr *tj ) {
  // Flatten alias type into disjoint equivalence classes
  tj = flatten_alias_type( tj );

  // Linear scan.  Could be a hash, but I am assuming that in general I'll
  // have so few unique address types that the scan will be fast.  Average 
  // size from _213_javac looks to be about 6.
  for( uint i = 1; i < _cnt_map_adr_mem; i++ ) 
    if( _map_adr_mem.fast_lookup(i) == tj )
      return;

  // Not found.  Insert into table.
  _map_adr_mem.map(_cnt_map_adr_mem++,tj);
}

//------------------------------Optimize---------------------------------------
// Given a graph, optimize it.
void Compile::Optimize() {
  TraceTime t1(NULL, &_t_optimizer, TimeCompiler, false);

#ifndef PRODUCT
  if (env()->break_at_compile()) {
    BREAKPOINT;
  }

#ifdef ASSERT
  if( VerifyOpto && WizardMode) { // Build Def-Use edges for debugging
    PhaseDefUse du(_unique,_root);
    assert( true, "du.dump()");
  }
#endif
#endif

  ResourceMark rm;
  _major_progress = false;

  // Iterative Global Value Numbering, including ideal transforms
  // Initialize IterGVN with types and values from parse-time GVN
  PhaseIterGVN igvn(get_parse_gvn());
  igvn.optimize();

  // Loop transforms on the ideal graph.  Range Check Elimination,
  // peeling, unrolling, etc.
  if( _has_loops || (SplitIfBlocks && _has_split_ifs) ) {
    PhaseIdealLoop ideal_loop( igvn, NULL );
  }

  // Conditional Constant Propagation;
  PhaseCCP ccp( &igvn ); 
  assert( true, "Break here to ccp.dump_nodes_and_types(_root,999,1)");
  ccp.do_transform();
  assert( true, "Break here to ccp.dump_old2new_map()");
    
  // Iterative Global Value Numbering, including ideal transforms
  igvn = ccp;
  igvn.optimize();

  // Now cycle if we are making progress
  debug_only( int cnt = 0; )
  while( _major_progress ) {
    assert( cnt++ < 40, "infinite cycle in optimizer" );
    _major_progress = false;
    // Loop transforms on the ideal graph.  Range Check Elimination,
    // peeling, unrolling, etc.
    if( _has_loops || (SplitIfBlocks && _has_split_ifs) ) {
      PhaseIdealLoop ideal_loop( igvn, NULL );
      // Do not repeat loop optimizations if irreducible loops are present
      if( ideal_loop._has_irreducible_loops )
	_major_progress = false;
    }
  } 

  // A method with only infinite loops has no edges entering loops from root
  if( final_graph_reshaping(igvn._du) ) {
    if( PrintOpto || PrintOptoBailouts ) {
      tty->print_cr("Warning: Bail out for infinite loop");
    }
    _root = NULL;
    return;
  }


#ifdef ASSERT
    if( VerifyOpto ) {
      // Build Def-Use edges for debugging
      PhaseDefUse du(_unique,_root);
      assert( true, "du.dump()");
      _root->verify(&du);
    }
#endif
}


//------------------------------Code_Gen---------------------------------------
// Given a graph, generate code for it
void Compile::Code_Gen(bool save_argument_registers) {
  // Perform instruction selection.  You might think we could reclaim Matcher
  // memory PDQ, but actually the Matcher is used in generating spill code.
  // Internals of the Matcher (including some VectorSets) must remain live
  // for awhile - thus I cannot reclaim Matcher memory lest a VectorSet usage
  // set a bit in reclaimed memory.
  Node_List proj_list;
  Matcher m(proj_list,save_argument_registers);
  _matcher = &m;
  // In debug mode can dump m._nodes.dump() for mapping of ideal to machine
  // nodes.  Mapping is only valid at the root of each matched subtree.

  // If you have too many nodes, or if matching has failed, bail out
  if( _unique > 57000 ) _root = NULL;
  if( _root == NULL ) return;

  // Build a proper-looking CFG
  PhaseCFG cfg(&_arena,_root,m);
  _cfg = &cfg;
  {
    TraceTime t2(NULL, &_t_scheduler, TimeCompiler, false);
    cfg.Dominators();
    cfg.Find_Inner_Loops();
    cfg.Estimate_Block_Frequency();
    cfg.GlobalCodeMotion(m,_unique,proj_list);
    if( cfg._root == NULL ) {
      _root = NULL;
      return;
    }
#ifdef ASSERT
    cfg.verify();
#endif
  }

  PhaseChaitin regalloc(_unique,cfg,m,save_argument_registers);
  _chaitin = &regalloc;
  {
    TraceTime t2(NULL, &_t_registerAllocation, TimeCompiler, false);
    // Perform any platform dependent preallocation actions.  This is used,
    // for example, to avoid taking an implicit null pointer exception
    // using the frame pointer on win95.
    _chaitin->pd_preallocate_hook();

    // Perform register allocation.  After Chaitin, use-def chains are
    // no longer accurate (at spill code) and so must be ignored.
    // Node->LRG->reg mappings are still accurate.
    _chaitin->Register_Allocate();

    // Bail out if the allocator builds too many nodes
    if( _root == NULL ) return;
  }

  // Prior to register allocation we kept empty basic blocks in case the
  // the allocator needed a place to spill.  After register allocation we 
  // are not adding any new instructions.  If any basic block is empty, we 
  // can now safely remove it.
  cfg.RemoveEmpty();

#ifdef ASSERT
  // Perform any platform dependent postallocation verifications.
  _chaitin->pd_postallocate_verify_hook();
#endif

  // Apply peephole optimizations
  if( OptoPeephole ) {
    PhasePeephole peep( _chaitin, cfg);
    peep.do_transform();
  }

  // Convert Nodes to instruction bits into a buffer
  {
    TraceTime t2(NULL, &_t_output, TimeCompiler, false);
    Output();
  }

  // He's dead, Jim.
  _cfg     = (PhaseCFG*)0xdeadbeef;
  _chaitin = (PhaseChaitin*)0xdeadbeef;
}

//------------------------------new_runtime_stub-------------------------------
RuntimeStub *Compile::new_runtime_stub(bool save_argument_registers) {
  if( !_root ) return NULL;     // Bailed out
  assert(_sync == 0, "no synchronization is used for runtime stubs");

  // On Intel, the return address is always on the stack as the first
  // blob in the 'preserve' area.  This is not true in general; we need
  // to get the return address via a more generic mechanism.
  int return_address_offset = _frame_words - 1;
        
  // Make the NMethod
  RuntimeStub *rs = RuntimeStub::new_runtime_stub( _code_buffer, return_address_offset, _frame_words, _oop_map_set, save_argument_registers);
  assert(rs != NULL && rs->is_runtime_stub(), "sanity check");
  return rs;
}


//------------------------------dump_asm---------------------------------------
// Dump formatted assembly
#ifndef PRODUCT
void Compile::dump_asm() { 
  tty->print_cr("#");
  tty->print("#  ");  _tf->dump();  tty->cr();
  tty->print_cr("#");

  // For all blocks
  int pc = 0x0;                 // Program counter
  bool starts_bundle = false;
  _chaitin->dump_frame();
  for( uint i=0; i<_cfg->_num_blocks; i++ ) {
    Block *b = _cfg->_blocks[i];
    tty->print("%3.3x   ",pc);
    b->dump_head( &_cfg->_bbs );

    // For all instructions
    Node *delay = NULL;
    for( uint j = 0; j<b->_nodes.size(); j++ ) {
      Node *n = b->_nodes[j];
      if (valid_bundle_info(n)) {
        Bundle *bundle = node_bundling(n);
        if (bundle->used_in_unconditional_delay()) {
          delay = n;
          continue;
        }
        if (bundle->instr_count() > 0)
          starts_bundle = true;
      }

      if( n->size(_chaitin) ||
          (n->is_Mach() &&
           n->is_Mach()->is_MachNullCheck()) ) {
        tty->print("%3.3x %c ",pc, starts_bundle ? '+' : ' ');
        starts_bundle = false;
        pc += n->size(_chaitin);
        tty->print("\t");
        n->format(_chaitin);
        tty->print_cr("");
      }

      // If we have an instruction with a delay slot, and have seen a delay,
      // then back up and print it
      if (valid_bundle_info(n) && node_bundling(n)->use_unconditional_delay()) {
        assert(delay != NULL, "no unconditional delay instruction");
        if (node_bundling(delay)->instr_count() > 0)
          starts_bundle = true;
        uint delay_size = delay->size(_chaitin);
        tty->print("%3.3x %c ", pc - delay_size, starts_bundle ? '+' : ' ');
        starts_bundle = false;
        tty->print("\t");
        delay->format(_chaitin);
        tty->print_cr("");
        delay = NULL;
      }

      // Dump the exception table as well
      if( n->is_Catch() && (Verbose || WizardMode) ) {
        // Print the exception table for this offset
        _handler_table.print_subtable_for(pc);
      }
    }
    tty->print_cr("%3.3x ",pc);

    assert(delay == NULL, "no unconditional delay branch");

  } // End of per-block dump
  tty->print_cr("");
}
#endif

// register information defined by ADLC
extern const char register_save_policy[];
extern const int  register_save_type[];


// !!!!! The following method initializes the runtime stubs used by compiler2.
// The code here should be moved to an initialization routine in C2Compiler.
void compiler2_init() {
  // No initialization needed if interpreter only
  if (Arguments::mode() == Arguments::_int) return;

  // Check that runtime and architecture description agree on callee-saved-floats
  bool callee_saved_floats = false;
  for( OptoReg::Name i=OptoReg::Name(0); i<OptoReg::Name(_last_Mach_Reg); i = OptoReg::add(i,1) ) {
    // Is there a callee-saved float or double?
    if( register_save_policy[i] == 'E' /* callee-saved */ && 
       (register_save_type[i] == Op_RegF || register_save_type[i] == Op_RegD) ) {
      callee_saved_floats = true;
    }
  }
  assert( OptoRuntimeCalleeSavedFloats == callee_saved_floats, "Runtime and Architecture Description must agree");

  Compile::pd_compiler2_init();

  if (UseInlineImage) {    
    InlineImage::parse_from_file();
    InlineImage::sort_by_addr();
  }
  if (DumpInlineImage) {
    InlineImage::init_file();
  }

  JNIEnv* jni_env = JavaThread::current()->jni_environment();
  { ThreadToNativeFromVM ttn(JavaThread::current());

    HandleMark  handle_mark(JavaThread::current());
    ciEnv ci_env(jni_env, SystemDictionary::number_of_adds(), false);

    // -- Also Initialize types before each compile --
    // Compile all the Runtime stubs
    OptoRuntime::generate(&ci_env);
  }
}

//---------------------------argcnt--------------------------------------------
uint Compile::argcnt() const { 
  return _tf->_domain->_cnt - TypeFunc::Parms; 
}

//------------------------------Final_Reshape_Counts---------------------------
// This class defines counters to help identify when a method 
// may/must be executed using hardware with only 24-bit precision.
struct Final_Reshape_Counts : public StackObj {
  int  _call_count;             // count non-inlined 'common' calls 
  int  _float_count;            // count float ops requiring 24-bit precision
  int  _double_count;           // count double ops requiring more precision
  PhaseDefUse &_du;             // DU info 
  VectorSet _visited;           // Visitation flags
  Node_List _tests;		// Set of IfNodes & PCTableNodes

  Final_Reshape_Counts( PhaseDefUse &du ) : 
    _call_count(0), _float_count(0), _double_count(0), _du(du),
    _visited( Thread::current()->resource_area() ) { }

  void inc_call_count  () { _call_count  ++; }
  void inc_float_count () { _float_count ++; }
  void inc_double_count() { _double_count++; }

  int  get_call_count  () const { return _call_count  ; }
  int  get_float_count () const { return _float_count ; }
  int  get_double_count() const { return _double_count; }
};

static bool oop_offset_is_sane(const TypeOopPtr* tp) {
  ciInstanceKlass *k = tp->klass()->as_instance_klass();
  // Make sure the offset goes inside the instance layout.
  return (uint)tp->_offset < (uint)k->size_helper()*wordSize;
  // Note that OffsetBot and OffsetTop are very negative.
}

//------------------------------final_graph_reshaping_impl----------------------
// Implement items 1-5 from final_graph_reshaping below.
static void final_graph_reshaping_impl( Node *n, Final_Reshape_Counts &fpu ) {

  uint nop = n->Opcode();

  // Check for 2-input instruction with "last use" on right input.
  // Swap to left input.  Implements item (2).
  if( n->req() == 3 &&          // two-input instruction
      fpu._du.cnt(n->in(1)) > 1 &&   // left use is NOT a last use
      fpu._du.cnt(n->in(2)) == 1 &&  // right use IS a last use
      !n->in(2)->is_Con() ) {   // right use is not a constant
    // Check for commutative opcode
    switch( nop ) {
    case Op_AddI:  case Op_AddF:  case Op_AddD:  case Op_AddL:
    case Op_MulI:  case Op_MulF:  case Op_MulD:  case Op_MulL:
    case Op_AndL:  case Op_XorL:  case Op_OrL: 
    case Op_AndI:  case Op_XorI:  case Op_OrI: {
      // Move "last use" input to left by swapping inputs
      Node *t = n->in(1);
      n->set_req(1,n->in(2));
      n->set_req(2,t);
      break;
    }
    default:
      break;
    }
  }

  // Count FPU ops and common calls, implements item (3)
  switch( nop ) {
  // Count all float operations that may use FPU
  case Op_AddF:
  case Op_SubF:
  case Op_MulF:
  case Op_DivF:
  case Op_NegF:
  case Op_ModF:
  case Op_ConvI2F:
  // case Op_ConvL2F: // longs are split into 32-bit halves
    fpu.inc_float_count();
    break;

  case Op_ConvF2D:
  case Op_ConvD2F:
    fpu.inc_float_count();
    fpu.inc_double_count();
    break;

  // Count all double operations that may use FPU
  case Op_AddD:
  case Op_SubD:
  case Op_MulD:
  case Op_DivD:
  case Op_NegD:
  case Op_ModD:
  case Op_ConvI2D:
  case Op_ConvD2I:
  // case Op_ConvL2D: // handled by leaf call
  // case Op_ConvD2L: // handled by leaf call
  case Op_ConD:
  case Op_CmpDLess:
  case Op_CmpDGreater:
    fpu.inc_double_count();
    break;
  case Op_Opaque1:              // Remove Opaque Nodes before matching
  case Op_Opaque2:              // Remove Opaque Nodes before matching
    fpu._du.replace_node( n, n->in(1) );
    break;
  case Op_Phi:                  // Clone initializing constants to loops
    if( n->in(0)->is_Loop() &&  // These will split & rematerialize anyways
        n->in(1)->is_Con() &&   // so we save the allocator some work here.
        fpu._du.cnt(n->in(1)) > 1 ) {
      n->set_req( 1, n->in(1)->clone() );
    }
    break;

  case Op_CallStaticJava:
  case Op_CallJava:
  case Op_CallDynamicJava:
  case Op_CallCompiledJava:
  case Op_CallInterpreter:
  case Op_CallRuntime:
  case Op_CallLeaf:
  case Op_CallNative:
  case Op_Call: {
    assert( n->is_Call(), "" );
    CallNode *call = (CallNode*)n;
    // Count call sites where the FP mode bit would have to be flipped.
    // Do not count uncommon runtime calls:
    // uncommon_trap, _complete_monitor_locking, _complete_monitor_unlocking, 
    // _new_Java, _new_typeArray, _new_objArray, _rethrow_Java, ...
    const CallStaticJavaNode *static_call = call->is_CallStaticJava();
    if( !static_call || !static_call->_name ) {
      fpu.inc_call_count();   // Count the call site
    } else {                  // See if uncommon argument is shared
      Node *n = call->in(TypeFunc::Parms);
      int nop = n->Opcode();
      // Clone shared simple arguments to uncommon calls, item (1).
      if( fpu._du.cnt(n) > 1 && 
          !n->is_Proj() && 
	  nop != Op_CreateEx && 
	  nop != Op_CheckCastPP && 
	  !n->is_Mem() ) {
        Node *x = n->clone();
        call->set_req( TypeFunc::Parms, x );
      }
    }
    break;
  }

  // This "look_for_insane_oop_offset" mode is a conservative check
  // to avoid certain bugs in computation of OSR typestate, in which
  // the fetch_oop calls return a too-low type (such as Object:BotPtr).
  // This bug makes the optimizer forward-propagate the low type after CCP
  // which then causes alias analysis to fail (during anti-dep checking).
  case Op_StoreD:
  case Op_LoadD:
    fpu.inc_double_count();
    goto handle_mem;
  case Op_StoreF:
  case Op_LoadF:
    fpu.inc_float_count();
    goto handle_mem;

  case Op_Store:
  case Op_StoreB:
  case Op_StoreC:
  case Op_StoreI:
  case Op_StoreL:
  case Op_StoreP:
  case Op_LoadB:
  case Op_LoadC:
  case Op_LoadI:
  case Op_LoadKlass:
  case Op_LoadL:
  case Op_LoadP:
  case Op_LoadRange:
  case Op_LoadS:
  case Op_Load: {
  handle_mem:
#ifdef ASSERT
    if( VerifyOptoOopOffsets ) {
      assert( n->is_Mem(), "" );
      MemNode *mem  = (MemNode*)n;
      // Check to see if address types have grounded out somehow.
      // This can happen due to imperfect OSR typestate prediction .
      const TypeOopPtr *tp = mem->in(MemNode::Address)->bottom_type()->isa_oopptr();
      assert( !tp || oop_offset_is_sane(tp), "" );
    }
#endif
    break;
  }
  case Op_If:
  case Op_CountedLoopEnd:
    fpu._tests.push(n);		// Collect CFG split points
    break;

  default: 
    assert( !n->is_Call(), "" );
    assert( !n->is_Mem(), "" );
    if( n->is_If() || n->is_PCTable() ) 
      fpu._tests.push(n);	// Collect CFG split points
    break;
  }
}

//------------------------------final_graph_reshaping_walk---------------------
// Small method to do recursive walk
static void final_graph_reshaping_walk( Node *n, Final_Reshape_Counts &fpu ) {
  // Standard visitation checks for doing a post-order walk
  if( !n ) return;
  if( fpu._visited.test_set(n->_idx) ) return;

  // Do a post-order reverse walk (cause its easy)
  for( uint i = 0; i < n->req(); i++ ) 
    final_graph_reshaping_walk( n->in(i), fpu );

  final_graph_reshaping_impl( n, fpu );
}

//------------------------------final_graph_reshaping--------------------------
// Final Graph Reshaping.  
//
// (1) Clone simple inputs to uncommon calls, so they can be scheduled late
//     and not commoned up and forced early.  Must come after regular 
//     optimizations to avoid GVN undoing the cloning.  Clone constant
//     inputs to Loop Phis; these will be split by the allocator anyways.
//     Remove Opaque nodes.
// (2) Move last-uses by commutative operations to the left input to encourage
//     Intel update-in-place two-address operations and better register usage
//     on RISCs.  Must come after regular optimizations to avoid GVN Ideal
//     calls canonicalizing them back.
// (3) Count the number of double-precision FP ops, single-precision FP ops
//     and call sites.  On Intel, we can get correct rounding either by
//     forcing singles to memory (requires extra stores and loads after each
//     FP bytecode) or we can set a rounding mode bit (requires setting and
//     clearing the mode bit around call sites).  The mode bit is only used
//     if the relative frequency of single FP ops to calls is low enough.
//     This is a key transform for SPEC mpeg_audio.
// (4) Detect infinite loops; blobs of code reachable from above but not 
//     below.  Several of the Code_Gen algorithms fail on such code shapes,
//     so we simply bail out.  Happens a lot in ZKM.jar, but also happens
//     from time to time in other codes (such as -Xcomp finalizer loops, etc).
//     Detection is by looking for IfNodes where only 1 projection is
//     reachable from below or CatchNodes missing some targets.
// (5) Assert for insane oop offsets in debug mode.

bool Compile::final_graph_reshaping( PhaseDefUse &du ) {
  // an infinite loop may have been eliminated by the optimizer,
  // in which case the graph will be empty.
  if( _root->req() == 1 ) 
    return true;

  Final_Reshape_Counts fpu(du);

  // Visit everybody reachable!
  final_graph_reshaping_walk( _root, fpu );

  // Check for unreachable (from below) code (i.e., infinite loops).
  for( uint i = 0; i < fpu._tests.size(); i++ ) {
    Node *n = fpu._tests[i];
    const PCTableNode *pct = n->is_PCTable();
    assert( pct || n->is_If(), "either PCTables or IfNodes" );
    // Get number of CFG targets; 2 for IfNodes or _size for PCTables.
    // Note that PCTables include exception targets after calls.
    uint expected_kids = pct ? pct->_size : 2;
    if( du.cnt(n) != expected_kids ) {
      // Check for a few special cases.  Rethrow Nodes never take the
      // 'fall-thru' path, so expected kids is 1 less.
      if( pct && pct->in(0) && pct->in(0)->in(0) ) {
	CallNode *call = pct->in(0)->in(0)->is_Call();
	if( call && call->_entrypoint == OptoRuntime::rethrow_stub() )
	  expected_kids--;	// Rethrow always has 1 less kid
      }
      // Recheck with a better notion of 'expected_kids'
      if( du.cnt(n) != expected_kids ) 
	return true;		// Not all targets reachable!
    }
    // Check that I actually visited all kids.  Unreached kids
    // must be infinite loops.
    Node **outp = du.out(n);
    for( uint j = 0; j < expected_kids; j++ ) 
      if( !fpu._visited.test(outp[j]->_idx) )
        return true;            // Found unvisited kid; must be unreach
  }

  // If original bytecodes contained a mixture of floats and doubles
  // check if the optimizer has made it homogenous, item (3).
  if( Use24BitFPMode && Use24BitFP && 
      fpu.get_float_count() > 32 && 
      fpu.get_double_count() == 0 &&
      (10 * fpu.get_call_count() < fpu.get_float_count()) ) {
    set_24_bit_selection_and_mode( false,  true );
  }

  // No infinite loops, no reason to bail out.
  return false;
}

//------------------------------JVMPI_line_numbers---------------------------
// JVMPI support -- line number -- code offset mapping
//

LineNumbersOffset::LineNumbersOffset(Compile *c) {
  _pairs = c->get_line_number_pairs();
  int n = _pairs->length();
  int size = c->method()->code_size();  // byte code length
  _bci_offset  = new GrowableArray<int>(size, size, unused);
}


void LineNumbersOffset::add_offset(int bci, int offset) {
  if (_bci_offset->at(bci) == unused) {
    // use only first reference to that bci
    _bci_offset->at_put_grow(bci, offset);
  }
}


JVMPI_Lineno* LineNumbersOffset::get_lineno_pairs() {
  int n = _pairs->length();
  JVMPI_Lineno* res = NEW_RESOURCE_ARRAY(JVMPI_Lineno, n);
  for (int i = 0; i < n; i++) {
    int bci    = _pairs->at(i)->bci();
    int line   = _pairs->at(i)->line();
    int offset = _bci_offset->at(bci);
    res[i].offset = offset;
    res[i].lineno = line;
  }
  return res;
}


int LineNumbersOffset::get_lineno_count() const{
  return _pairs->length();
}


JVMCI_LineNumbersDesc* Compile::get_jvmpi_lineno_table() {
  JVMCI_LineNumbersDesc* res = new JVMCI_LineNumbersDesc();
  if (_linenumbers_offset != NULL) {
    res->_nof_lines    = _linenumbers_offset->get_lineno_count();
    res->_line_numbers = _linenumbers_offset->get_lineno_pairs();
  }
  return res;
}

GrowableArray<LinePair*>* Compile::get_line_number_pairs() {
  assert( _method != NULL, "Only called on methods");
  int n = env()->get_method_number_of_lines(_method);
  GrowableArray<LinePair*>* res = new GrowableArray<LinePair*>(n, n, NULL);
  // do all allocation on this side of JVMCI
  for (int i = 0; i < n; i++) res->at_put(i, new LinePair());
  env()->fill_method_line_numbers(_method, res);
  return res;
}

