/*
 * @(#)compile.hpp	1.140 01/02/09 14:46:08
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

class Block;
class Bundle;
class CompilationScope;
class InlineTree;
class Int_Array;
class Matcher;
class MachNode;
class Node;
class Node_Array;
class OptoReg;
class PhaseCFG;
class PhaseChaitin;
class PhaseDefUse;
class PhaseGVN;
class RootNode;
class Scope;
class StartNode;
class TypeFunc;
class Unique_Node_List;
class nmethod;

//----------------LineNumbers--------------------------------------
// JVMPI support
class LineNumbersOffset;

class JVMCI_LineNumbersDesc: public ResourceObj {
public:
  JVMPI_Lineno* _line_numbers;
  int           _nof_lines;
  JVMCI_LineNumbersDesc(): _line_numbers(NULL), _nof_lines(0) {}
};


// JVMPI support
class LineNumbersOffset: public ResourceObj {
 private:
  enum { unused = -1 };
  GrowableArray<LinePair*>* _pairs;      // array[i]   == <line,bci>
  GrowableArray<int>*       _bci_offset; // array[bci] == pc_offset

 public:
  LineNumbersOffset(Compile *a_compilation);
  void add_offset(int bci, int offset);
  JVMPI_Lineno* get_lineno_pairs();
  int get_lineno_count() const;
};




//------------------------------Compile----------------------------------------
// This class defines a top-level Compiler invocation. 

class Compile : public Phase {
private:
  void init( int aliaslevel );  // Prepare for a single compilation
  void Optimize();              // Given a graph, optimize it
  void Code_Gen(bool save_argument_registers);   // Generate code from a graph

  // capture JVMPI status once for the compilation
private:
  bool _jvmpi_compiled_method_event;
  bool _jvmpi_method_entry_event;
  bool _jvmpi_method_entry2_event;
  bool _jvmpi_method_exit_event;
public:
  bool is_set_jvmpi_compiled_method_event() { return _jvmpi_compiled_method_event; }
  bool is_set_jvmpi_method_entry_event()    { return _jvmpi_method_entry_event;    }
  bool is_set_jvmpi_method_entry2_event()   { return _jvmpi_method_entry2_event;   }
  bool is_set_jvmpi_method_entry_exit()     { return _jvmpi_method_exit_event;     }
  bool is_set_jvmpi_method_event() {
    return _jvmpi_method_entry_event
      || _jvmpi_method_entry2_event
      || _jvmpi_method_exit_event;
  }

  // JVMPI support -- line numbers
  GrowableArray<LinePair*> *get_line_number_pairs();
  // from c1_CodeEmitter.hpp
  LineNumbersOffset        *_linenumbers_offset;
  JVMCI_LineNumbersDesc    *get_jvmpi_lineno_table();
  void                      mark_offset_for_bci(int bci);
  // from c1_Compile1.hpp
private:
  JVMCI_LineNumbersDesc    *_jvmpi_lines;
  void set_jvmpi_lines(JVMCI_LineNumbersDesc *jvmpi_lines) { _jvmpi_lines = jvmpi_lines; }
public:
  JVMCI_LineNumbersDesc    *jvmpi_lines() const     { return _jvmpi_lines; }

private:
  ciMethod* _method;
  ciScope*  _scope;
  ciEnv*    _env;

  ciMethod* _String_compareTo;
  ciMethod* _Object_hashCode;
  ciMethod* _System_identityHashCode;
  ciMethod* _System_arraycopy;
  ciMethod* _Class_newInstance;
  ciMethod* _Class_newInstance0;

  ciInstanceKlass* _Math;

  ciMethod* _Math_sin;
  ciMethod* _Math_cos;
  ciMethod* _Math_tan;
  ciMethod* _Math_atan2;
  ciMethod* _Math_sqrt;

  // Remember the reason for compiler bailout, default is Comp_success
public:
  enum CompResult {
    Comp_no_retry                   =-1, // Do not attempt to recompile this method
    Comp_success                    = 0, // Successful compile
    Comp_subsumed_load_conflict     = 1  // Retry matching and scheduling without subsuming loads
  };

  void          set_result(CompResult result) { assert( _result == Comp_success, "Record one result"); _result = result; }
  CompResult    result() { return _result; }
  // Check for various bail-out causes and breakpoints
  InlineTree *bailouts_and_breaks();
  // Pass along information about the size of calls for bailing out
  int          _max_arg_size;

private:
  CompResult   _result;

  // Does this compilation allow instructions to subsume loads?  User
  // instructions that subsume a load may result in an unschedulable
  // instruction sequence.
  const bool   _subsume_loads;
  // ID for this compilation.  Useful for setting breakpoints in the debugger.
  const int _compile_id;
public:
  bool	subsume_loads() 	{ return _subsume_loads; }

  ciMethod* method() { return _method; }
  ciScope*  scope () { return _scope; }
  ciEnv*    env   () { return _env; }

  ciMethod* get_String_compareTo();
  ciMethod* get_Object_hashCode();
  ciMethod* get_System_identityHashCode();
  ciMethod* get_System_arraycopy();
  ciMethod* get_Class_newInstance();
  ciMethod* get_Class_newInstance0();

  ciInstanceKlass* get_Math();

  ciMethod* get_Math_sin();
  ciMethod* get_Math_cos();
  ciMethod* get_Math_tan();
  ciMethod* get_Math_atan2();
  ciMethod* get_Math_sqrt();

  // Major entry point.  Given a Scope, compile the associated method.
  // For normal compilations, entry_bci is InvocationEntryBci.  For on stack
  // replacement, entry_bci indicates the bytecode for which to compile a
  // continuation.
  Compile( ciEnv* ci_env, ciScope* ci_scope, ciMethod* target,
           int entry_bci, bool subsume_loads, bool reuse_env );

  // Second major entry point.  From the TypeFunc signature, generate code
  // to pass arguments from the Java calling convention to the C calling
  // convention.  
  Compile( ciEnv* ci_env, const TypeFunc *(*gen)(), address C_function,
	         const char *name, int is_fancy_jump, bool pass_tls,
	         bool save_arg_registers, bool return_pc );

  void Generate_Stub_Graph( const TypeFunc *java_sig,
			    address C_function, const char *name,
			    ciMethod* method, ciInstanceKlass* static_klass,
			    int is_fancy_jump, bool wrap_oops,
			    bool jni_sig, bool pass_tls, bool return_pc );

  // Third major entry point.  Compile a native method.
  Compile( ciEnv* ci_env, ciMethod* method );

  // Fourth major entry point.  Compile an adapter.
  Compile( ciEnv* ci_env, ciMethod* method, int /*AdapterType*/ type, bool reuse_env );

  // From the TypeFunc signature, generate code to pass arguments
  // from Compiled calling convention to Interpreter's calling convention
  void Generate_Compiled_To_Interpreter_Graph( const TypeFunc *tf, address interpreter_entry );
  // From the TypeFunc signature, generate code to pass arguments
  // from Interpreter's calling convention to Compiler's calling convention
  void Generate_Interpreter_To_Compiled_Graph( const TypeFunc *tf );

  address         _C_function;  // VM entry for stub being compiled, or NULL
  const char*     _name;        // Name of stub or adapter being compiled, or NULL
  const TypeFunc *_tf;          // My kind of signature
  int             _sync;        // Max depth of monitor stack.

  int             _entry_bci;   // entry bci for osr methods.

  int   entry_bci() { return _entry_bci; }

  Arena _arena;                 // Arena for new-space Nodes
  Arena _old_arena;             // Arena for old-space Nodes, lifetime during xform
  Arena _comp_arena;            // Arena with lifetime equivalent to Compile

  uint _unique;                 // Counter for unique Node indices

  bool _has_loops;              // True if the method _may_ have some loops
  bool _has_split_ifs;          // True if the method _may_ have some split-if
  bool _major_progress;         // True if something big happened
  bool _deopt_happens;          // TRUE if de-optimization CAN happen
  int  _AliasLevel;             // Local version of the aliasing flag

  // After building we hang onto the Root instruction.
  // The RootNode instruction is where the whole program begins.  It produces
  // the initial Control and BOTTOM for everybody else.
  RootNode  *_root;
  StartNode *start() const;

  // Handy undefined Node
  Node *_top_node;

  // Remember if this compilation changes hardware mode to 24-bit precision
private:
  bool  _in_24_bit_fp_mode;
  // Remember if this compilation selects instructions that generate 24-bit results
  bool  _select_24_bit_instr;
public:
  void set_24_bit_selection_and_mode(bool selection, bool mode) { Compile::_select_24_bit_instr = selection; Compile::_in_24_bit_fp_mode = mode; }
  bool select_24_bit_instr()  { return _select_24_bit_instr; }
  bool in_24_bit_fp_mode()    { return _in_24_bit_fp_mode; }

  // Used during the compilation of on stack replacement code to make
  // the compiler ready for a clean parse.
  void reset_state_for_second_parse();
  bool is_osr_compilation()          { return _entry_bci != InvocationEntryBci; }

  // Final graph reshaping, post the regular optimizer
  bool final_graph_reshaping(PhaseDefUse &du);

  // Results of parse-time PhaseGVN
  PhaseGVN *_parse_gvn;
  void      set_parse_gvn(PhaseGVN *gvn) { _parse_gvn = gvn; }
  PhaseGVN *get_parse_gvn() { return _parse_gvn; }

  // Nodes that need to be transformed during next round of Iterative GVN
  Unique_Node_List *_for_igvn;
  void      set_for_igvn(Unique_Node_List *for_igvn) { _for_igvn = for_igvn; }
  Unique_Node_List *get_for_igvn() { 
    assert(_for_igvn != NULL, "insanity check");
    return _for_igvn;
  }


  // Array to map address types to memories.  Mapping is created during 
  // parsing.  The index of a particular address type in the
  // array is also the index of the defining Node in the merge_mem Node.
  Type_Array _map_adr_mem;
  uint _cnt_map_adr_mem;
  const TypePtr *flatten_alias_type( const TypePtr * ) const;
  void set_alias_index ( const TypePtr * );
  uint get_alias_index ( const TypePtr * ) const;
  
  // Results of CFG finding
  // Handy place to keep them around, since they are so long lived.
  PhaseCFG *_cfg;

  // Matching and other machine-specific hooks
  Matcher *_matcher;
  RegMask FIRST_STACK_mask;
  // Count of incoming arguments
  uint argcnt() const;

  // Results of register allocation.
  // Handy place to keep them around, since they are so long lived.
  PhaseChaitin *_chaitin;
  
  // Convert Nodes to instruction bits and pass off to the VM
  int         _method_size;
  int         _frame_words;      // Size of total frame in words
  CodeBuffer *_code_buffer;
  ExceptionHandlerTable _handler_table;
  ImplicitNullCheckTable _inc_table;
  address     _code_base;
  int         _first_block_size;	// Size of unvalidated entry point code / osr poison code

  // Information for instruction sizes and bundling, indexed by the nodes
  unsigned short *_node_size;
  uint _node_bundle_limit;
  Bundle *_node_bundling;

  Bundle* node_bundling(const Node *n);
  bool valid_bundle_info(const Node *n);

  // Driver for converting compiler's IR into machine code bits
  void Output();

  // Schedule and Bundle the instructions
  void ScheduleAndBundle();

  // Append debug info for the node to the array
  int FillLocArray( Node *local, GrowableArray<ScopeValue*> *array );
  // Write out basic block data to code buffer
  void Fill_buffer( uint *blk_starts );
  // Compute size of each basic block
  uint *Compute_size();

  // Compute bundling information
  bool Compute_Bundled_Size(Block *b, Node *n, uint &j, Node * &delay_slot, MachNode *_nop_list[],
    uint blk_start, uint &blk_size, uint &last_bundle_resources);

  // ScopeDesc/Garbage Collection/Debug Information
  DebugInformationRecorder *_recorder;

  OopMapSet    *_oop_map_set;
  
  // Create special derived classes of CodeBlob
  RuntimeStub *new_runtime_stub(bool save_argument_registers);
  I2CAdapter  *new_i2c_adapter(methodHandle m);
  C2IAdapter  *new_c2i_adapter(methodHandle m);

  // Stack slots that may be unused by the calling convention but must
  // otherwise be preserved.  On Intel this includes the return address.
  // On PowerPC it includes the 4 words holding the old TOC & LR glue.
  uint in_preserve_stack_slots();
  // "Top of Stack" slots that may be unused by the calling convention but must
  // otherwise be preserved.  
  // On Intel these are not necessary and the value can be zero.
  // On Sparc this describes the words reserved for storing a register window
  // when an interrupt occurs.
  static uint out_preserve_stack_slots();
  // Number of outgoing stack slots killed above the out_preserve_stack_slots
  // for calls to C.  Supports the var-args backing area for register parms.
  uint varargs_C_out_slots_killed() const;
  // Number of Stack Slots consumed by a synchronization entry
  int sync_stack_slots() const;
 public:
  // Print bytecodes, including the scope inlining tree
  void print_codes();

# ifdef ENABLE_ZAP_DEAD_LOCALS
  static bool is_node_getting_a_safepoint(Node*);

  void Insert_zap_nodes();
  Node* call_zap_node(Node* n, int block_no);

  static int  CompiledZap_count;

# endif
  // The current active Compile
  static Compile* current() {
    Compile* current = (Compile*)ciEnv::data();
    assert(current != NULL, "no active Compile");
    return current;
  }

#define COMPILE_DATA(name, type) \
 private:                        \
  type _##name;                  \
 public:                         \
  type name () {                 \
    return _##name;              \
  }                              \
  void set_##name (type value) { \
    _##name = value;             \
  }

  COMPILE_DATA(type_arena, Arena*)
  COMPILE_DATA(typedict, Dict*)
  COMPILE_DATA(type_hwm, void*)
  COMPILE_DATA(type_last_size, size_t)

  COMPILE_DATA(node_unique_top, const Node *)

  COMPILE_DATA(indexSet_arena, Arena*)
  COMPILE_DATA(indexSet_free_block_list, void*)

 public:
#ifndef PRODUCT
  // Dump formatted assembly
  void dump_asm();
#endif
  debug_only( static int _debug_idx; )  // node counter that is not reset

  // Definitions of pd methods
  static void pd_compiler2_init();
};

