/*
 * @(#)connode.hpp	1.117 01/02/09 14:46:11
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


class PhaseTransform;
class PhaseDefUse;

//------------------------------ConNode----------------------------------------
// Simple constants
class ConNode : public TypeNode {
public:
  ConNode( const Type *t ) : TypeNode(t,1) { set_req(0,(Node*)Compile::current()->_root);}
  virtual int  Opcode() const;
  virtual uint hash() const;
  virtual uint is_Con() const { return 1; }
  virtual const RegMask &out_RegMask() const { return RegMask::Empty; }
  virtual const RegMask &in_RegMask(uint) const { return RegMask::Empty; }
  static ConNode *make( const Type *t );
};

//------------------------------ConINode---------------------------------------
// Simple integer constants
class ConINode : public ConNode {
public:
  ConINode( const TypeInt *t ) : ConNode(t) {}
  virtual int Opcode() const;
  virtual int get_int( jint *con ) const { *con = _type->is_int()->get_con(); return true; }
};

//------------------------------ConPNode---------------------------------------
// Simple pointer constants
class ConPNode : public ConNode {
public:
  ConPNode( const TypePtr *t ) : ConNode(t) {}
  virtual int Opcode() const;
};


//------------------------------ConLNode---------------------------------------
// Simple long constants
class ConLNode : public ConNode {
public:
  ConLNode( const TypeLong *t ) : ConNode(t) {}
  virtual int Opcode() const;
};

//------------------------------ConFNode---------------------------------------
// Simple float constants
class ConFNode : public ConNode {
public:
  ConFNode( const TypeF *t ) : ConNode(t) {}
  virtual int Opcode() const;
};

//------------------------------ConDNode---------------------------------------
// Simple double constants
class ConDNode : public ConNode {
public:
  ConDNode( const TypeD *t ) : ConNode(t) {}
  virtual int Opcode() const;
};

//------------------------------CopyNode---------------------------------------
// Simple copy
class CopyNode : public TypeNode {
public:
  CopyNode( Node *n, const Type *t ) : TypeNode(t,2) { set_req(0,NULL); set_req(1,n); }
  virtual int Opcode() const;
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual Node *Identity( PhaseTransform *phase );
  virtual uint is_Copy() const { return 1; }
  static Node *make( Node *n, const Type *t );
};

//------------------------------CopyINode--------------------------------------
class CopyINode : public CopyNode {
public:
  CopyINode( Node *n ) : CopyNode( n, TypeInt::INT ) { }
  virtual int Opcode() const;
};

//------------------------------CopyLNode--------------------------------------
class CopyLNode : public CopyNode {
public:
  CopyLNode( Node *n ) : CopyNode( n, TypeLong::LONG ) { }
  virtual int Opcode() const;
};

//------------------------------CopyPNode--------------------------------------
class CopyPNode : public CopyNode {
public:
  CopyPNode( Node *n ) : CopyNode( n, TypeOopPtr::BOTTOM ) { }
  virtual int Opcode() const;
};

//------------------------------CopyFNode--------------------------------------
class CopyFNode : public CopyNode {
public:
  CopyFNode( Node *n ) : CopyNode( n, Type::FLOAT ) { }
  virtual int Opcode() const;
};

//------------------------------CopyDNode--------------------------------------
class CopyDNode : public CopyNode {
public:
  CopyDNode( Node *n ) : CopyNode( n, Type::DOUBLE ) { }
  virtual int Opcode() const;
};

//------------------------------CMovePairNode----------------------------------
// Place holder for the 2 conditional inputs to a CMove.  CMove needs 4
// inputs: the Bool (for the lt/gt/eq/ne bits), the flags (result of some
// compare), and the 2 values to select between.  The Matcher requires a
// binary tree so we break it down like this: 
//     (CMove (CMovePair bol cmp) (CMovePair src1 src2))
class CMovePairNode : public Node {
public:
  CMovePairNode( Node *n1, Node *n2 ) : Node(0,n1,n2) { }
  virtual int Opcode() const;
  virtual uint ideal_reg() const { return 0; }
};

//------------------------------CMoveNode--------------------------------------
// Conditional move
class CMoveNode : public TypeNode {
public:
  enum { Control,               // When is it safe to do this cmove?
         Condition,             // Condition controlling the cmove
         IfFalse,               // Value if condition is false
         IfTrue };              // Value if condition is true
  CMoveNode( Node *bol, Node *left, Node *right, const Type *t ) : TypeNode(t,4) { set_req(Control,NULL); set_req(Condition,bol); set_req(IfFalse,left); set_req(IfTrue,right); }
  virtual int Opcode() const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual Node *Identity( PhaseTransform *phase );
  static CMoveNode *make( Node *c, Node *bol, Node *left, Node *right, const Type *t );
  virtual CMoveNode *is_CMove() { return this; }
};

//------------------------------CMoveDNode-------------------------------------
class CMoveDNode : public CMoveNode {
public:
  CMoveDNode( Node *bol, Node *left, Node *right ) : CMoveNode(bol,left,right,Type::DOUBLE){}
  virtual int Opcode() const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
};

//------------------------------CMoveFNode-------------------------------------
class CMoveFNode : public CMoveNode {
public:
  CMoveFNode( Node *bol, Node *left, Node *right ) : CMoveNode(bol,left,right,Type::FLOAT) {}
  virtual int Opcode() const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
};

//------------------------------CMoveINode-------------------------------------
class CMoveINode : public CMoveNode {
public:
  CMoveINode( Node *bol, Node *left, Node *right ) : CMoveNode(bol,left,right,TypeInt::INT){}
  virtual int Opcode() const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
};

//------------------------------CMoveLNode-------------------------------------
class CMoveLNode : public CMoveNode {
public:
  CMoveLNode(Node *bol, Node *left, Node *right ) : CMoveNode(bol,left,right,TypeLong::LONG){}
  virtual int Opcode() const;
};

//------------------------------CMovePNode-------------------------------------
class CMovePNode : public CMoveNode {
public:
  CMovePNode( Node *c, Node *bol, Node *left, Node *right ) : CMoveNode(bol,left,right,TypeOopPtr::BOTTOM) { set_req(Control,c); }
  virtual int Opcode() const;
};

//------------------------------CastPPNode-------------------------------------
// cast pointer to pointer (different type)
class CastPPNode: public TypeNode {
public:
  CastPPNode (Node *n, const Type *t ): TypeNode(t,2) { set_req(0, NULL); set_req(1, n); }
  virtual Node *Identity( PhaseTransform *phase );
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual int Opcode() const;
  virtual uint ideal_reg() const { return Op_RegP; }
  virtual Node *Ideal_DU_postCCP( PhaseCCP * );
};

//------------------------------CheckCastPPNode--------------------------------
// for _checkcast, cast pointer to pointer (different type), without JOIN,
class CheckCastPPNode: public Node {
  virtual uint hash() const;    // Check the type
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const; // Size is bigger
public:
  const Type *_type;
  CheckCastPPNode( Node *c, Node *n, const Type *t ) : Node(c,n), _type(t) { }
  virtual Node *Identity( PhaseTransform *phase );
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual int   Opcode() const;
  virtual uint  ideal_reg() const { return Op_RegP; }
  // No longer remove CheckCast after CCP as it gives me a place to hang
  // the proper address type - which is required to compute anti-deps.
  //virtual Node *Ideal_DU_postCCP( PhaseCCP * );
  virtual const Type *bottom_type() const { return _type; }
#ifndef PRODUCT
  virtual void dump_spec() const;
#endif
};

//------------------------------Conv2BNode-------------------------------------
// Convert int/pointer to a Boolean.  Map zero to zero, all else to 1.
class Conv2BNode : public Node {
public:
  Conv2BNode( Node *i ) : Node(0,i) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return TypeInt::BOOL; }
  virtual Node *Identity( PhaseTransform *phase );
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegI; }
};


// The conversions operations are all Alpha sorted.  Please keep it that way!
//------------------------------ConvD2FNode------------------------------------
// Convert double to float
class ConvD2FNode : public Node {
public:
  ConvD2FNode( Node *in1 ) : Node(0, in1) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return Type::FLOAT; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegF; }
};

//------------------------------ConvD2INode------------------------------------
// Convert Double to Integer
class ConvD2INode : public Node {
public:
  ConvD2INode( Node *in1 ) : Node(0, in1) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegI; }
};

//------------------------------ConvD2LNode------------------------------------
// Convert Double to Long
class ConvD2LNode : public Node {
public:
  ConvD2LNode( Node *dbl ) : Node( 0, dbl ) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return TypeLong::LONG; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint ideal_reg() const { return Op_RegL; }
};

//------------------------------ConvF2DNode------------------------------------
// Convert Float to a Double.
class ConvF2DNode : public Node {
public:
  ConvF2DNode( Node *in1 ) : Node(0, in1) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return Type::DOUBLE; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegD; }
};

//------------------------------ConvF2INode------------------------------------
// Convert float to integer
class ConvF2INode : public Node {
public:
  ConvF2INode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegI; }
};

//------------------------------ConvF2LNode------------------------------------
// Convert float to long
class ConvF2LNode : public Node {
public:
  ConvF2LNode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return TypeLong::LONG; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegL; }
};

//------------------------------ConvI2DNode------------------------------------
// Convert Integer to Double
class ConvI2DNode : public Node {
public:
  ConvI2DNode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return Type::DOUBLE; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegD; }
};

//------------------------------ConvI2FNode------------------------------------
// Convert Integer to Float
class ConvI2FNode : public Node {
public:
  ConvI2FNode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return Type::FLOAT; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegF; }
};

//------------------------------ConvI2LNode------------------------------------
// Convert integer to long
class ConvI2LNode : public Node {
public:
  ConvI2LNode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return TypeLong::LONG; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegL; }
};

//------------------------------ConvL2DNode------------------------------------
// Convert Long to Double
class ConvL2DNode : public Node {
public:
  ConvL2DNode( Node *in1 ) : Node( 0, in1 ) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return Type::DOUBLE; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint ideal_reg() const { return Op_RegD; }
};

//------------------------------ConvL2FNode------------------------------------
// Convert Long to Float
class ConvL2FNode : public Node {
public:
  ConvL2FNode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return Type::FLOAT; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegF; }
};

//------------------------------ConvL2INode------------------------------------
// Convert long to integer
class ConvL2INode : public Node {
public:
  ConvL2INode( Node *in1 ) : Node(0, in1) {}
  virtual int         Opcode() const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual uint  ideal_reg() const { return Op_RegI; }
};

//------------------------------DoubleHalfNode---------------------------------
// Convert doubles to a pair of integers.  Used in native calling conventions.
class DoubleHalfNode : public Node {
public:
  DoubleHalfNode( Node *n ) : Node( 0, n ) { }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
  virtual const RegMask &out_RegMask() const;
  virtual const RegMask &in_RegMask(uint) const;
  virtual uint ideal_reg() const { return Op_RegI; }
};
class DoubleHalf0Node : public DoubleHalfNode {
public:
  DoubleHalf0Node( Node *n ) : DoubleHalfNode(n) {}
  virtual int Opcode() const;
};
class DoubleHalf1Node : public DoubleHalfNode {
public:
  DoubleHalf1Node( Node *n ) : DoubleHalfNode(n) {}
  virtual int Opcode() const;
};

//------------------------------LongHalfNode---------------------------------
// Convert Longs to a pair of integers.  Used in native calling conventions.
class LongHalfNode : public Node {
public:
  LongHalfNode( Node *n ) : Node( 0, n ) { }
  virtual const Type *Value( PhaseTransform *phase ) const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
  virtual const RegMask &out_RegMask() const;
  virtual const RegMask &in_RegMask(uint) const;
  virtual uint ideal_reg() const { return Op_RegI; }
};
class LongHalf0Node : public LongHalfNode {
public:
  LongHalf0Node( Node *n ) : LongHalfNode(n) {}
  virtual int Opcode() const;
};
class LongHalf1Node : public LongHalfNode {
public:
  LongHalf1Node( Node *n ) : LongHalfNode(n) {}
  virtual int Opcode() const;
};

//------------------------------SpillCopyNode----------------------------------
// Copy-spill.  Move from any old machine register to any other, including
// stack spill-slots.
class SpillCopyNode : public CopyNode {
  const RegMask *_in;		// RegMask for input
  const RegMask *_out;		// RegMask for output
public:
  SpillCopyNode( Node *n, const RegMask &in, const RegMask &out ) : 
    CopyNode( n, n->bottom_type() ), _in(&in), _out(&out) { }
  virtual SpillCopyNode *is_SpillCopy() { return this; }
  virtual int Opcode() const;
  void set_out_RegMask(const RegMask &out) { _out = &out; }
  void set_in_RegMask(const RegMask &in) { _in = &in; }
  virtual const RegMask &out_RegMask() const { return *_out; }
  virtual const RegMask &in_RegMask(uint) const { return *_in; }
  class Node *copy_make( Node *src ) const;
  class Node *load_make( Node *mem, Node *stk ) const;
  class Node *store_make( Node *mem, Node *stk, Node *val ) const;
  virtual uint size_of() const { return sizeof(*this); }
};

//------------------------------MemMoveNode------------------------------------
// Memory to memory move.  Inserted very late, after allocation.
class MemMoveNode : public Node {
public:
  MemMoveNode( Node *dst, Node *src ) : Node(0,dst,src) {}
  virtual int Opcode() const;
};

//------------------------------ThreadLocalNode--------------------------------
// Ideal Node which returns the base of ThreadLocalStorage.
class ThreadLocalNode : public Node {
public:
  ThreadLocalNode( ) : Node((Node*)Compile::current()->_root) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return TypeRawPtr::BOTTOM;}
  virtual uint ideal_reg() const { return Op_RegP; }
};

//------------------------------AtomicAdd--------------------------------------
// Do an atomic add to memory.  Must be multi-processor safe.  The updated 
// location is not continously updated so it is safe to spin-retry.  Machines
// are expected to use something like a load-locked/store-conditional or a
// compare-exchange operation.  AtomicAdd returns the new contents of memory.
class AtomicAddNode : public Node {
  virtual uint cmp( const Node &n ) const;
  virtual bool depends_only_on_test() const { return false; }
public:
  AtomicAddNode( Node *control, Node *oopmem, Node *adr, Node *con ) : Node( control, oopmem, adr, con ) {}
  virtual int Opcode() const;
  virtual Node *Ideal(PhaseGVN *phase,PhaseDefUse *du);
  virtual const Type *bottom_type() const { return TypeRawPtr::BOTTOM;}
  virtual uint ideal_reg() const { return Op_RegP; }  
  // Do not match memory edge
  virtual uint match_edge(uint idx) const;
};

//------------------------------BoxNode----------------------------------------
// Take the address of a value.  Since values have no location, the "box" 
// requires its input to be stack-based.  It returns the stack address.  Spill
// code will "load" the box if needed.  Since stack locations are treated as 
// unaliased values, the user of this Node needs to make sure the boxed value
// remains alive past the last use of the box.
class BoxNode : public Node {
public:
  BoxNode( Node *value ) : Node( 0, value ) {}
  virtual int Opcode() const;
  virtual void emit(CodeBuffer &cbuf, PhaseChaitin *pc_) const;
  virtual uint size(PhaseChaitin *pc_) const;

  virtual const RegMask &in_RegMask(uint) const;
  virtual const RegMask &out_RegMask() const;

  virtual const class Type *bottom_type() const { return TypeRawPtr::BOTTOM; }
  virtual uint ideal_reg() const { return Op_RegP; }
#ifndef PRODUCT
  virtual void format( PhaseChaitin * ) const;
#endif
};

//------------------------------LoadPCNode-------------------------------------
class LoadPCNode: public Node {
public:
  LoadPCNode(Node *c) : Node(c) { }
  virtual int Opcode() const;
  virtual uint ideal_reg() const { return Op_RegI; }
};


//------------------------------LoadReturnPCNode-------------------------------

class LoadReturnPCNode: public Node {
public:
  LoadReturnPCNode(Node *c) : Node(c) { }
  virtual int Opcode() const;
  virtual uint ideal_reg() const { return Op_RegI; }
};


//-----------------------------RoundFloatNode----------------------------------
class RoundFloatNode: public Node {
public:
  RoundFloatNode(Node* c, Node *in1): Node(c, in1) {}
  virtual int   Opcode() const;
  virtual const Type *bottom_type() const { return Type::FLOAT; }
  virtual uint  ideal_reg() const { return Op_RegF; }
  virtual Node *Identity( PhaseTransform *phase );
  virtual const Type *Value( PhaseTransform *phase ) const;
};


//-----------------------------RoundDoubleNode---------------------------------
class RoundDoubleNode: public Node {
public:
  RoundDoubleNode(Node* c, Node *in1): Node(c, in1) {}
  virtual int   Opcode() const;
  virtual const Type *bottom_type() const { return Type::DOUBLE; }
  virtual uint  ideal_reg() const { return Op_RegD; }
  virtual Node *Identity( PhaseTransform *phase );
  virtual const Type *Value( PhaseTransform *phase ) const;
};

//------------------------------Opaque1Node------------------------------------
// A node to prevent unwanted optimizations.  Allows constant folding.
// Stops value-numbering, Ideal calls or Identity functions.
class Opaque1Node : public Node {
  virtual uint cmp( const Node &n ) const;
public:
  Opaque1Node( Node *n ) : Node(0,n) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
  virtual Node *Identity( PhaseTransform *phase );
};

//------------------------------Opaque2Node------------------------------------
// A node to prevent unwanted optimizations.  Allows constant folding.  Stops
// value-numbering, most Ideal calls or Identity functions.  This Node is
// specifically designed to prevent the pre-increment value of a loop trip
// counter from being live out of the bottom of the loop (hence causing the
// pre- and post-increment values both being live and thus requiring an extra
// temp register and an extra move).  If we "accidentally" optimize through
// this kind of a Node, we'll get slightly pessimal, but correct, code.  Thus
// it's OK to be slightly sloppy on optimizations here.
class Opaque2Node : public Node {
  virtual uint cmp( const Node &n ) const;
public:
  Opaque2Node( Node *n ) : Node(0,n) {}
  virtual int Opcode() const;
  virtual const Type *bottom_type() const { return TypeInt::INT; }
};
