/*
 * @(#)doCall.cpp	1.104 01/02/09 14:46:13
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_doCall.cpp.incl"

//------------------------------do_call----------------------------------------
// Handle your basic call.  Inline if we can & want to, else just setup call.
void Parse::do_call( ciByteCodeStream* stream, Bytecodes::Code bc, uint cloning ) {
  uint bci = stream->cur_bci();
  uint next_bci = stream->next_bci();

  // Find target being called
  ciMethod* dest_method = stream->get_method();

  // Check for unloaded or uninitialized
  if( bits_type(bci) == 0 ) {
    // Do not know how to optimize!
    if( !dest_method->is_loaded() )
      uncommon_trap( stream->get_method_holder_index()   , bci );
    else
      uncommon_trap( Deoptimization::Deopt_uninitialized, bci );
    return;                     // Bail out!
  }
  assert( dest_method->is_loaded(), "" );
  assert( dest_method->holder()->is_loaded(), "" );
  assert( (bc != Bytecodes::_invokestatic ||
	   dest_method->holder()->is_initialized() ), "" );

  // Set frequently used booleans
  bool is_virtual;
  bool has_receiver;
  bool is_invokeinterface;
  switch( bc ) {
  case Bytecodes::_invokestatic:
    is_virtual = false;  has_receiver = false;  is_invokeinterface = false;
    break;
  case Bytecodes::_invokespecial:
    is_virtual = false;  has_receiver = true;   is_invokeinterface = false;
    break;
  case Bytecodes::_invokeinterface:
    is_virtual = true;   has_receiver = true;   is_invokeinterface = true;
    break;
  case Bytecodes::_invokevirtual:
    is_virtual = true;   has_receiver = true;   is_invokeinterface = false;
    break;
  default:
    ShouldNotReachHere();
  }

  uint nargs = _args[bci];
  // Call_exit is the Node that produces results for a call.  It is the
  // CallNode itself for regular Calls or the ReturnNode of an inlined call.
  Node *call_exit;

  ciInstanceKlass* holder_klass = dest_method->holder();
  ciInstanceKlass* klass = stream->get_declared_method_holder();
  ciInstanceKlass* calling_klass = _klass->as_instance_klass();
  
  const TypeFunc *tf = TypeFunc::make(dest_method);

  // If the call's return value is an unloaded class or destination not found
  // throw uncommon_trap at call's bytecode, with JVM state before the call.
  // Only need to check for return value being unloaded if it is an oop
  if( tf->_range->_cnt > TypeFunc::Parms ) {
    const Type       *tresult = tf->_range->_fields[TypeFunc::Parms];
    const TypeOopPtr *toop    = tresult->isa_oopptr();
    if( toop && !toop->is_loaded() ) {
      // The class will have been loaded when returned by the call,
      // but may not have been loaded by our class_loader
      uncommon_trap( stream->get_method_signature_index(), bci );
      return;
    }
  }  

  // ---------------------
  // Does Class Hierarchy Analysis reveal only a single target of a v-call?
  // Then we may either inline or make a static call, but in no case do we
  // make a v-call.  Thus we have a dependency on there being only 1 target.

  bool cha_is_monomorphic = false;
  ciMethod* cha_monomorphic_target = NULL;
  ciMethod* optimized_virtual_method = NULL;
  Node *receiver_nonnull_after_call = NULL;  // set this after an implicit null check

  ciInstanceKlass* actual_receiver = klass;
  // try to get the most accurate receiver type
  // also, add receiver null_check for _invokespecial and optimized virtuals
  if (is_virtual || is_invokeinterface) {
    const TypeOopPtr* receiver_type = _gvn.type(stack(_sp - nargs))->isa_oopptr();
    if (receiver_type != NULL) {
      ciKlass* kl = receiver_type->klass();
      if (!kl->is_interface() && kl != klass && kl->is_subclass_of(klass)) {
        // this is a better type than the static receiver type
        actual_receiver = kl->as_instance_klass();
      }
    }
    cha_monomorphic_target =
      dest_method->find_monomorphic_target(calling_klass, klass, actual_receiver);
    cha_is_monomorphic = (cha_monomorphic_target != NULL);
    if (cha_is_monomorphic) {
      optimized_virtual_method = cha_monomorphic_target;

      // We skip the standard null check for an optimized virtual call,
      // or for optimized invokeinterface.
      // This call might also be inlined.  Virtuals assume the receiver is 
      // not null, so we need an explicit null check.
      do_null_check(stack(_sp - nargs), bci, T_OBJECT);

    // If CHA fails, try the type profile.
    } else if ((UseTypeProfile || UseInlineImage) && !cha_is_monomorphic) {
      ciInstanceKlass* prof_klass  = NULL;
      ciMethod*        prof_method = NULL;
      // !!!!! UseInlineImage has been disabled as part of the compiler interface
      // integration.
      //
      //#ifndef PRODUCT
      //      if (UseInlineImage) {
      //        prof_method = InlineImage::resolve_virtual_method(_ilt->_scope, bci);
      //        if (prof_method.not_null())
      //          prof_klass = prof_method->method_holder();
      //      }
      //      else
      //#endif
      ciCallProfile profile = _method->call_profile_at_bci(bci);
      if (profile.is_valid() && profile.receiver()->is_instance_klass()) {
	prof_method = profile.method();
        prof_klass  = profile.receiver()->as_instance_klass();
      }
      if (DeutschShiffmanALot && prof_method == NULL) {
	// fake a type prediction
	prof_method = dest_method;
	prof_klass = dest_method->holder();
	if (prof_method->is_abstract() || prof_klass->is_abstract()) {
	  prof_method = NULL;
	  prof_klass = NULL;
	}
      }
      if (prof_method != NULL) {
	// need a type check here
#ifndef PRODUCT
	if (TraceTypeProfile) {
	  if (!PrintOpto) {
	    tty->print("  ");
	    _method->print_short_name();
	    tty->print(" -> ");
	  } else {
	    for( int i = 0; i < _depth; i++ ) tty->print("  ");
	  }
	  tty->print("@ %d  ", bci);
	  prof_method->print_short_name();
	  tty->print("  >>TypeProfile = ");
          prof_klass->name()->print_symbol();
          tty->print_cr(" (%d bytes)", prof_method->code_size());
	}
#endif
	// Do Deutsch-Shiffman inlining of the expected method.
	// (Even if the inliner refuses to inline it, make a static call.)
	Node* recv = stack(_sp - nargs);
	Node* prof_k = _gvn.makecon(TypeKlassPtr::make(prof_klass));
	recv = do_null_check(recv, bci, T_OBJECT);
	if( !_map ) return;  // non-null case of null check folded away!
	Node* p = _gvn.transform( new (4) AddPNode( recv, recv, _gvn.intcon(oopDesc::klass_offset_in_bytes())) );
	Node* recv_klass = _gvn.transform(new (3) LoadKlassNode(0, memory(p), p, TypeKlassPtr::OBJECT));
	Node *cmp = _gvn.transform(new (3) CmpPNode( recv_klass, prof_k));
	Node *bol = _gvn.transform(new (2) BoolNode( cmp, BoolTest::eq));
	IfNode *iff = create_and_xform_if( control(), bol, 0.9999f, -1.0f );
	set_control(_gvn.transform(new (1) IfFalseNode(iff)));
	{ PreserveJVMState pjvms(this, bci);
	  uncommon_trap(Deoptimization::Deopt_unexpected_klass, bci);
	}
	// fall through if the instance exactly matches the desired type
	set_control(_gvn.transform(new (1) IfTrueNode (iff)));

	// %%% The following type is not as exact as it could be
	const Type* recv_exact_type = TypeOopPtr::make(TypePtr::NotNull, prof_klass);
	// %%% We should subsume downstream occurrences of recv with a cast to
	// recv_exact_type, since now we know what the type will be.
	// This would cause subsequent type checks to collapse.
	recv = _gvn.transform( new(2) CheckCastPPNode(control(), recv, recv_exact_type) );
	set_stack(_sp - nargs, recv);  // record the type narrowing

	if( control() == _top ) return;
	optimized_virtual_method = prof_method;
      }
    }

    // At this point, optimized_virtual_method has been set if possible.

    if( optimized_virtual_method == NULL ) {  
      // This will be a regular virtual call, which does a null check for us.
      // Flag that we can assert receiver-not-null after the call.
      receiver_nonnull_after_call = stack(_sp - nargs);
    }

  } else if ( has_receiver ) {
    // Invoke special is a direct call, so needs a null check
    do_null_check(stack(_sp - nargs), bci, T_OBJECT);
  }

  if( !_map ) return;  // non-null case of null check folded away!

  // ---------------------
  // Round any double arguments before the call
  for( uint j = 0; j < nargs; j++ ) {
    const Type *targ = tf->_domain->_fields[j + TypeFunc::Parms];
    if( targ->_base == Type::DoubleTop || targ->_base == Type::DoubleCon || targ->_base == Type::DoubleBot ) {  
      // If any parameters are doubles, they must be rounded before
      // the call, dstore_rounding does gvn.transform
      Node *arg = stack(_sp - nargs + j);
      arg = dstore_rounding(arg);
      set_stack( _sp - nargs + j, arg );
    }
  }
  uint arg_start_idx = _map->_jvms->spoff() + _sp - nargs;

  // ---------------------
  // Need to construct the JVMStateObj list now in case inlining does occur.
  JVMStateObj *jhead = _jhead;
  JVMStateObj *jtail = _jtail;
  if( _jtail ) {
    jtail->set_next(_map->_jvms);
    jtail = _map->_jvms;
  } else {
    jhead = jtail = _map->_jvms; 
  }
  _sp -= nargs;                 // Temporarily pop args for JVM state of call
  _map->_jvms->set_sp(_sp);     // Record the new sp in the JVMState Object
  _map->_jvms->set_bci(bci);    // Record current bci

  // ---------------------
  // Special case the handling of certain common, profitable library
  // methods.  If these methods are replaced with specialized code,
  // then we exit do_call.
  if( inline_library_methods( dest_method, nargs, tf, bc, bci)){
    if( _jtail )                // Reset the end of the jvms list.
      _jtail->set_next( NULL );
    return;
  }

  // ---------------------
  // Attempt to inline.
  // Here is the decision to inline, we will try to inline if ilt != NULL
  InlineTree* ilt = NULL;
  // Stop inlining into giant methods
  if (C->_unique <= (uint)NodeCountInliningCutoff) {
    // Get any inline policy decision
    ilt = _ilt->subtree_at_bci(bci);
    if (ilt) {
      if (cloning || !is_jsr_stack_empty() ) // Cloning?
        ilt = ilt->deep_copy(); // Make private copy of _preds, etc info
      // if (PrintOpto && Verbose) {
      //   for(int i = 0; i < _depth; i++ ) tty->print("  ");
      //   tty->print("@ %d", bci);
      // }
    }
  }

#ifndef PRODUCT
  // bump global counters for calls
  if( CountCompiledCalls ) {
    if (ilt) {
      switch (bc) {
       case Bytecodes::_invokevirtual:   increment_counter(OptoRuntime::nof_inlined_calls_addr()); break;
       case Bytecodes::_invokeinterface: increment_counter(OptoRuntime::nof_inlined_interface_calls_addr()); break;
       case Bytecodes::_invokestatic:  
       case Bytecodes::_invokespecial:   increment_counter(OptoRuntime::nof_inlined_static_calls_addr()); break;
       default: fatal("unexpected call bytecode");
      }
    } else {
      switch (bc) {
       case Bytecodes::_invokevirtual:   increment_counter(OptoRuntime::nof_normal_calls_addr()); break;
       case Bytecodes::_invokeinterface: increment_counter(OptoRuntime::nof_interface_calls_addr()); break;
       case Bytecodes::_invokestatic:  
       case Bytecodes::_invokespecial:   increment_counter(OptoRuntime::nof_static_calls_addr()); break;
       default: fatal("unexpected call bytecode");
      }
    }
  }
#endif

#ifndef PRODUCT
  // !!!!! DumpInlineImage has been disabled as part of the compiler interface
  // integration.
  //
  // Record a resolved virtual call site
  // If it is inlined (ilt != NULL), the parser will record it instead
  // if (DumpInlineImage) {
  //   if (ilt == NULL && !is_first_osr_parse() && is_virtual && optimized_virtual_method.not_null())
  //     InlineImage::append_call_begin_to_file(optimized_virtual_method, _depth, bci);
  // }
#endif

  // CHA has decided to call the optimized_virtual_method, but an earlier decision was made
  // (by the ilt) to inline a different method.  This may happen because Deutsch-Schiffman
  // inlining predicts a random receiver type for this call site.  In any case, we need to
  // trust the optimized_virtual_method, not what the ilt says.
  if ( ilt && optimized_virtual_method != NULL && !optimized_virtual_method->equals(ilt->_method) ) {
    assert(UseTypeProfile, "ilt must be exact unless we are using a type profile");
    // Do a plain out-of-line optimized virtual call.
    ilt = NULL;
  }

  // BugId 4356206: inlining decision recorded by InlineTree became invalid
  //                due to class loading before doCall's CHA analysis
  //                and type profile did not produce Deutsch-Schiffman check.
  //                When CHA and Deutsch-Schiffman decide not to inline,
  //                they set receiver_nonnull_after_call.
  if( ilt && receiver_nonnull_after_call ) {
    ilt = NULL;
  }

  ::Parse parser( env(), ilt, jhead, jtail, _sync_max, _depth+1, _gvn, _for_igvn, _map, arg_start_idx, _has_ex_handler );

#ifndef PRODUCT
  // !!!!! DumpInlineImage has been disabled as part of the compiler interface
  // integration.
  //
  // if (DumpInlineImage) {
  //   if (ilt == NULL && !is_first_osr_parse() && is_virtual && optimized_virtual_method.not_null())
  //     InlineImage::append_scope_end_to_file();
  // }
#endif

  // Pull inlined return from RootNode
  for( uint i = 1; i < _root->req(); i++ ) {
    if( _root->in(i) == parser._exit_return ||
        _root->in(i) == parser._exit_rethrow )
      _root->del_req(i--);
  }

  // Did parser build a graph we can inline here?
  if( parser._bailout == Bailout_none ) {

    if( is_virtual ) {          // Inlining a virtual?
      // If we inlined because CHA revealed only a single target method,
      // then we are dependent on that target method not getting overridden
      // by dynamic class loading.  Be sure to test the "static" receiver
      // dest_method here, as opposed to the actual receiver, which may
      // falsely lead us to believe that the receiver is final or private.
      if( cha_is_monomorphic &&
          !dest_method->is_final() &&
          !dest_method->is_private() ) 
        C->_recorder->add_dependent(actual_receiver, cha_monomorphic_target);
    }

    // ---------------------
    // Yeah!  "Link" things together - knit together the outgoing arguments 
    // with the inlined StartNode, and returned result with the inlined
    // ReturnNode.
    call_exit  = parser._exit_return;

    if (parser._sync_max > _sync_max) {
      _sync_max = parser._sync_max;
    }

    // ------------------
  } else if ( parser._bailout < 0 ) { // Need to recursively bail out
      _bailout = parser._bailout;
      return;
  } else {                      // Else no inlining, insert a call
    // Break out 4 cases: virtual/optimized virtual/static/static-with-this.
    if( is_virtual ) {

      // Does Class Hierarchy Analysis reveal only a single target?
      if( optimized_virtual_method != NULL ) {
        // Make a static call
#ifndef PRODUCT
        if (CountCompiledCalls) {
          increment_counter(OptoRuntime::nof_optimized_calls_addr());
        }
#endif
        CallStaticJavaNode *call = new CallStaticJavaNode(tf, OptoRuntime::resolve_opt_virtual_call_Java(), optimized_virtual_method, bci);
        call_exit = call;
        call->set_optimized_virtual(true);
        if (cha_is_monomorphic)
          C->_recorder->add_dependent(actual_receiver, cha_monomorphic_target);
      } else {
        // Normal inline cache used for call
        assert(is_virtual, "shouldn't generate DynamicJavaNode");        
        call_exit = new CallDynamicJavaNode(tf, OptoRuntime::resolve_virtual_call_Java(), dest_method, bci);
      }      
    } else {                    // Else static call, with or without receiver
      if (has_receiver) {
        address target = OptoRuntime::resolve_opt_virtual_call_Java();
        CallStaticJavaNode *call = new CallStaticJavaNode (tf, target, dest_method, bci);
        call_exit = call;
        call->set_optimized_virtual(true); 
        // no dependency needed here--guaranteed by JVM (invoke_special et al)
      } else {
        address target = OptoRuntime::resolve_static_call_Java();
        call_exit = new CallStaticJavaNode(tf, target, dest_method, bci);
      }
    } // End if virtual or not

    //----------------
    // Add JVM state to the call node.
    // Walk the inline list to fill in the correct set of JVMStateObj's
    // Also fill in the associated edges for each JVMStateObj.
    CallNode *call_temp = call_exit->is_Call();
    assert(call_temp != NULL, "call_entry must be a call node");
    add_debug_info(call_temp,jhead);

    // Common parts of both regular calls and inlined calls
    // Set fixed predefined input arguments
    call_temp->set_req( TypeFunc::Control, control   () );
    call_temp->set_req( TypeFunc::I_O    , i_o       () );
    call_temp->set_req( TypeFunc::Memory , all_memory() );
    call_temp->set_req( TypeFunc::ReturnAdr, _top       );
    call_temp->set_req( TypeFunc::FramePtr, frameptr () );
    // pop and thread regular input args
    for( uint j = 0; j < nargs; j++ ) 
      call_temp->set_req( j + TypeFunc::Parms, stack(_sp + j) );
    assert( _sp >= 0, "" );
    _gvn.transform(call_temp);

  } // End if inlining or not

  //---------------------

  // Set fixed predefined return values
  set_control  (_gvn.transform(new(1) ProjNode(call_exit,TypeFunc::Control) ));
  set_i_o      (_gvn.transform(new(1) ProjNode(call_exit,TypeFunc::I_O    ) ));
  set_all_memory_call(call_exit);
    // Set the normal returned results also
  switch( bits_size(bci) ) {
  case 2: 
    // Push high value, then low value. Must be consistent with implementation
    // of _lload, _lstore, _lreturn, _dload, _dstore, and _dreturn
    assert( bits_type(bci) == T_DOUBLE || bits_type(bci) == T_LONG, "bits_type must match");
    push( _gvn.transform(new(1)ProjNode(call_exit,TypeFunc::Parms)) );
    push( _top );
    break;
  case 1: 
    push( _gvn.transform(new(1)ProjNode(call_exit,TypeFunc::Parms  )));
  case 0: 
    break;
  default: ShouldNotReachHere();
  }

  // Split handling depending on inlining or not
  if( parser._bailout == Bailout_none ) {
    // Can the inlined code throw any exceptions?
    if(parser._exit_rethrow && parser._exit_rethrow->bottom_type()!=Type::TOP){
      // Catch all thrown exceptions from inlined code
      handlers_for_rethrow( parser._exit_rethrow, bci );
    }
  } else {
    // determine potential exception handlers for this call
    ciExceptionHandlerStream handlers(_method, bci);
  
    // Make the CatchNode use I/O from the call as if we are passing the
    // exception status thru I/O.
    Node *catch_ = catch_exceptions(&handlers,bci,i_o());
    // No inlining, get control from the CatchNode.  
    set_control(_gvn.transform(new(1)CatchProjNode(catch_, CatchProjNode::fall_through_index, CatchProjNode::no_handler_bci)));
  }    

  // If the inlined code never returns (ala throw uncommon trap) then the
  // bytecodes following the call cannot be reached.  Flag them all as dead.
  if( control() == _top ) {
    _map = NULL;
  } else {
    // Receiver null_check?  Cast receiver to not-null on this path
    if( receiver_nonnull_after_call ) cast_not_null(receiver_nonnull_after_call);
  }

  // Unhook current jvms from list
  if( _jtail )
    _jtail->set_next( NULL );   // Reset end of list to _jtail

}

//------------------------------catch_exceptions-------------------------------
// Put a Catch and CatchProj nodes behind a just-created call.
// Send their caught exceptions to the proper handler.
Node *Parse::catch_exceptions(ciExceptionHandlerStream* handlers, int bci, Node *call) {
  // Add a CatchNode.
  GrowableArray<int>* bcis = new (&C->_arena) GrowableArray<int>(&C->_arena, 8, 0, -1);

  while (!handlers->is_done()) {
    bcis->append_if_missing(handlers->handler()->handler_bci());
    handlers->next();
  }

  int len = bcis->length();
  CatchNode *cn = new CatchNode(control(), call, len+1);
  Node *catch_ = _gvn.transform(cn);

  // now branch with the exception state to each of the (potential)
  // handlers
  for(int i=0; i < len; i++) {
    // Setup JVM state to enter the handler.
    PreserveJVMState pjvms(this, bci);
    // Locals are just copied from before the call.
    // Get control from the CatchNode.
    int handler_bci = bcis->at(i);
    Node* ctrl = _gvn.transform( new(1)CatchProjNode(catch_, i+1,handler_bci));
    // This handler cannot happen?
    if( ctrl == _top ) continue;
    set_control(ctrl);

    // Create exception oop
    _sp = 0;                    // Exception oop is only stack item
    Node *ex_oop = _gvn.transform(new(2)CreateExNode(ctrl,call));
    // go to the exception handler
    if( handler_bci < 0 ) {     // merge with corresponding rethrow node
      // JVMPI -- record exit from a method if compiled while JVMPI is turned on
      if( C->is_set_jvmpi_method_event() ) {
        make_jvmpi_method_exit(bci);
      }
      rethrow(ex_oop, false);
    } else {                      // Else jump to corresponding handle
      push(ex_oop);
      merge(handler_bci, true);
    }
  }

  return catch_;
}


//------------------------------handlers_for_rethrow---------------------------
// Handle all exceptions thrown by the inlined method.  
// Common case 1: we have no handler, so all exceptions merge right into
// the rethrow case.
// Case 2: we have some handlers, with loaded exception klasses that have
// no subklasses.  We do a Deutsch-Shiffman style type-check on the incoming
// exception oop and branch to the handler directly.
// Case 3: We have some handlers with subklasses or are not loaded at
// compile-time.  We have to call the runtime to resolve the exception.
// So we insert a RethrowCall and all the logic that goes with it.
void Parse::handlers_for_rethrow( Node *exit_rethrow, int throw_bci ) {
  // Remember the current, non-excepting, map
  PreserveJVMState pjvms(this, throw_bci);

  // The exit_rethrow defines a new map
  _sp = 0;                      // No expression stack in handlers
  uint j;
  for( j = TypeFunc::Control; j < TypeFunc::Parms; j++ ) 
    _map->set_req(j,exit_rethrow->in(j));
  set_all_memory( _map->in(TypeFunc::Memory) );

  // Get the exception oop as known at compile time.
  Node *ex_node = exit_rethrow->in( TypeFunc::Parms );
  // Get the exception oop klass from its header
  Node *koff = _gvn.intcon(oopDesc::klass_offset_in_bytes());
  Node* p = _gvn.transform( new (4) AddPNode( ex_node, ex_node, koff ) );
  Node *mem = NULL;
  Node* ex_klass_node = NULL;
  if( _has_ex_handler ) {
    mem = memory(TypeOopPtr::HEADER);
    ex_klass_node = _gvn.transform(new (3) LoadKlassNode(0, mem, p, TypeKlassPtr::OBJECT));

    // Compute the exception klass a little more cleverly.
    // Obvious solution is to simple do a LoadKlass from the 'ex_node'.
    // However, if the ex_node is a PhiNode, I'm going to do a LoadKlass for
    // each arm of the Phi.  If I know something clever about the exceptions
    // I'm loading the class from, I can replace the LoadKlass with the
    // klass constant for the exception oop.
    if( ex_node->is_Phi() && mem->is_Phi() && mem->in(0) == ex_node->in(0) ) {
      ex_klass_node = new PhiNode( ex_node->in(0), TypeKlassPtr::OBJECT );
      for( uint i = 1; i < ex_node->req(); i++ ) {
        Node* p = _gvn.transform( new (4) AddPNode( ex_node->in(i), ex_node->in(i), koff ) );
        Node* k = _gvn.transform(new (3) LoadKlassNode(0, mem->in(i), p, TypeKlassPtr::OBJECT));
        ex_klass_node->set_req( i, k );
      }
      _gvn._types.map( ex_klass_node->_idx, TypeKlassPtr::OBJECT );
      
    }
  }

  ciExceptionHandlerStream handlers(_method, throw_bci);

  // Scan the exception table for applicable handlers.
  // If none, we can call rethrow() and be done!
  // If precise (loaded with no subklasses), insert a D.S. style
  // pointer compare to the correct handler and loop back.
  // If imprecise, switch to the Rethrow VM-call style handling.

  // iterate through all entries sequentially
  for (;!handlers.is_done(); handlers.next()) {
    // Do nothing if turned off
    if( !DeutschShiffmanExceptions ) break;
    ciExceptionHandler* handler = handlers.handler();

    if( handler->is_rethrow() ) {
      // JVMPI -- record exit from a method if compiled while JVMPI is turned on
      if( C->is_set_jvmpi_method_event() ) {
        make_jvmpi_method_exit(throw_bci);
      }
      // If we fell off the end of the table without finding an imprecise
      // exception klass (and without finding a generic handler) then we
      // know this exception is not handled in this method.  We just rethrow
      // the exception into the caller.
      rethrow( ex_node, false );
      return;
    }

    // exception handler bci range covers throw_bci => investigate further
    int handler_bci = handler->handler_bci();
    
    if( handler->is_catch_all() ) { // catches all exceptions => final candidate
      push(ex_node);            // Push exception oop for handler
      merge(handler_bci, true); // jump to handler
      return;                   // No more handling to be done here!
    }
    ciInstanceKlass* klass = handler->catch_klass();

    // Get the handler's klass
    if( !klass->is_loaded() )   // klass is not loaded?
      break;                    // Must call Rethrow!
    // See if the loaded exception klass has no subtypes
    if( klass->has_subklass() )        // Has subklasses?
      break;                    // Cannot easily do precise test ==> Rethrow

    // Add a dependency; if any subclass added we need to recompile
    if( !klass->is_final() )
      C->_recorder->add_dependent(klass, NULL);

    // Implement precise test 
    const TypeKlassPtr *tk = TypeKlassPtr::make(klass);
    Node *con = _gvn.makecon( tk );
    Node *cmp = _gvn.transform(new (3) CmpPNode( ex_klass_node, con ) );
    Node *bol = _gvn.transform(new (2) BoolNode( cmp, BoolTest::ne));
    IfNode *iff = new (2) IfNode( control(), bol, 0.7f, -1.0f );
    _gvn.transform( iff );
    // Restore state after merge
    { PreserveJVMState pjvms(this, throw_bci);
      set_control( _gvn.transform( new (1) IfFalseNode(iff) ) );
      push(ex_node);            // Push exception oop for handler
      merge( handler_bci, true );
    }
    // Come here if exception does not match handler.
    // Carry on with more handler checks.
    set_control( _gvn.transform( new (1) IfTrueNode(iff) ) );

  }

  // Oops, need to call into the VM to resolve the klasses at runtime.
  CallNode *call = new CallStaticJavaNode(OptoRuntime::rethrow_Type(), OptoRuntime::rethrow_stub(), OptoRuntime::stub_name(OptoRuntime::rethrow_stub()), throw_bci);

  for( j = TypeFunc::Control; j < TypeFunc::Parms; j++ ) 
    call->set_req(j, _map->in(j) );
  call->set_req( TypeFunc::Parms, ex_node );
      
  // Add the safepoint edges to the call
  add_safepoint_edges(call, throw_bci);
  Node* c = _gvn.transform(call);
  // Rethrow is a pure call, no side effects, only a result.
  // The result cannot be allocated, so we use I_O
  Node *result = _gvn.transform(new(1) ProjNode(c, TypeFunc::I_O));
  set_control( _gvn.transform( new (1) ProjNode(c, TypeFunc::Control) ) );
  set_all_memory( all_memory() );

  // Catch exceptions from the rethrow
  catch_exceptions(&handlers,throw_bci,result);

}


//------------------------------add_debug_info---------------------------------
void Parse::add_debug_info( SafePointNode *call, JVMStateObj *jhead ) {
  JVMStateObj *jtail = NULL;

  // Do we need debug info here?  If it is a SafePoint and this method
  // cannot de-opt, then we do NOT need any debug info.
  bool x = (C->_deopt_happens || call->Opcode() != Op_SafePoint);

  while( jhead ) {
    // Loop over the _map input edges associated with jhead, add them
    // to the call node, & reset all offsets to match call node array.
    uint newlocoff = call->req();
    // Add the Locals
    uint q;
    for( q = jhead->locoff(); q < jhead->spoff(); q++ ) 
      call->add_req(x ? jhead->map()->in(q) : _top);
    // Add the Expression Stack
    // The (uint) cast is needed because MSC++ is broken
    for( q = jhead->spoff(); q < jhead->spoff()+ (uint)jhead->sp(); q++ ) 
      call->add_req(x ? jhead->map()->in(q) : _top);
    // Add the Monitors
    for( q = jhead->monoff(); q < jhead->endoff(); q++ ) 
      call->add_req(jhead->map()->in(q));
    // Clone the jvms on the list
    JVMStateObj *jtmp = (JVMStateObj *)jhead->clone(); 
    jtmp->set_map(call);
    jtmp->set_locoff(newlocoff);
    // Reset the spoff in the jvms to the current sp
    jtmp->set_spoff(newlocoff + (jhead->spoff() - jhead->locoff()));
    jtmp->set_monoff(jtmp->spoff() + jhead->sp());
    jtmp->set_endoff(jtmp->monoff() + (jhead->endoff() - jhead->monoff()));

    if( jtail ) jtail->set_next(jtmp);    // Hook new jvms onto list
    else        call->_jvms = jtmp;       // Start jvms list for call node
    jtail = jtmp;                         // Save pointer to new list tail
    jhead = jhead->next();                // Iterate
  }
  call->_jvmtail = jtail;            // Set the tail pointer
}
