// @(#)generateOptoStub.cpp	1.23 01/02/09 14:46:17
//
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_generateOptoStub.cpp.incl"

//------------------------------Parse------------------------------------------
// Temp parser used in making Native graphs

Parse::Parse( SafePointNode *map, PhaseGVN &gvn, Unique_Node_List &for_igvn ) : Phase(Phase::Parser), _method(NULL), _gvn(gvn), _for_igvn(for_igvn), _sync_base(0), _sync_max(0), _jhead(0), _jtail(0), _sp(0), _map(map) {
  _klass = NULL; // Otherwise _klass has a bogus value
  _top = makecon( Type::TOP );
  _null = makecon( TypePtr::NULL_PTR );
}

//-----------------------plus_adr----------------------------------------------
static Node *plus_adr( Node *ptr, int con, Node *top ) { 
  Node *connode = new (1) ConINode( TypeInt::make(con) );
  return new (4) AddPNode( top/*No base for non-oop ptrs*/, ptr, connode );
}

//-----------------------Generate_Stub_Graph-----------------------------------
void Compile::Generate_Stub_Graph(const TypeFunc *java_sig, 
                                  address C_function,
                                  const char *name, 
                                  ciMethod* method, 
                                  ciInstanceKlass* static_klass, 
                                  int is_fancy_jump,
                                  bool wrap_oops, 
                                  bool jni_sig, 
                                  bool pass_tls,
                                  bool return_pc ) {
  assert( !(jni_sig && pass_tls), "JNI calls use handles, do not pass oop thru TLS" );
  const TypeTuple *jdomain = java_sig->_domain;
  // The procedure start
  StartNode *start = new StartNode( _root, jdomain );

  // Make a map, with JVM state
  uint parm_cnt = jdomain->_cnt;
  uint max_map = max(parm_cnt+1, java_sig->_range->_cnt);
  JVMStateObj *jvms = new JVMStateObj(NULL,SynchronizationEntryBCI,max_map - TypeFunc::Parms);
  SafePointNode *map = new SafePointNode( max_map, jvms );
  jvms->set_map(map);

  // Make up the parameters
  uint i;
  for( i = 0; i < parm_cnt; i++ ) 
    map->set_req(i, new (1) ParmNode(start, i));
  for( ; i<map->req(); i++ )
    map->set_req(i, NULL);      // For nicer debugging
  set_alias_index( TypeRawPtr::BOTTOM );


  // Put names on a few parameters for easier manipulation
  Node *memory  = map->in(TypeFunc::Memory);
  Node *frameptr  = map->in(TypeFunc::FramePtr);
  Node *null = new (1) ConPNode( TypePtr::NULL_PTR );
  // Make a class-mirror constant oop Node for static natives.
  // Used to synchronize and as an argument to static natives.
  Node *mirror_node = NULL;
  Node *mirror_box = NULL;
  if( static_klass ) {
    ciInstance* mirror = static_klass->java_mirror();
    const TypeOopPtr *tk = TypeOopPtr::make(mirror);
    mirror_node = new (1) ConPNode( tk );
    if( jni_sig ) 		// JNI style call or not?
      // Static JNI calls pass the static_klass mirror.  They need it
      // boxed.  The box has to update the stack prior to setting
      // thread_in_native in order to be GC safe.
      mirror_box = new (2) BoxNode( mirror_node );
  }

  // Do we need to wrap incoming arguments in handles?
  // Generate code to put the oop on the stack and return its address.
  Node *unwrapped_this = map->in(TypeFunc::Parms+0);
  if( wrap_oops ) {
    for( i = TypeFunc::Parms; i < parm_cnt; i++ ) 
      if( jdomain->_fields[i]->isa_oop_ptr() ) {
        // replace parameter with handle
        // if oop is null, handle must be null !

        // Always create handle, so that oop is always set to correct value
        // even if the oop is null; otherwise we will have an uninitialized
        // oop location
        BoxNode *box = new (2) BoxNode( map->in(i) );

        // we use box node as input to test, to guarantee that it is not
        // moved and that is executed regardless of testing.  BoxNode has the
        // side effect of setting the memory location where it points to.
        Node* test_val = new (3) LoadPNode(map->in(TypeFunc::Control), map->in(TypeFunc::Memory), box,  TypeRawPtr::BOTTOM);
        CmpPNode *cmp = new (3) CmpPNode( test_val, null );
        BoolNode *b = new (2) BoolNode( cmp, BoolTest::ne );
        IfNode *iff = new (2) IfNode( map->in(TypeFunc::Control), b, 0.9f, -1.0f );
        IfFalseNode *if_null     = new (1) IfFalseNode( iff );
        IfTrueNode  *if_not_null = new (1) IfTrueNode ( iff );
        RegionNode *region = new RegionNode(3);
        region->set_req(1, if_not_null );
        region->set_req(2, if_null );
        PhiNode *phi = new PhiNode(region, TypeRawPtr::BOTTOM);
        phi->set_req(1,box);
        // if oop is null, set handle to null.
        phi->set_req(2,test_val);
        map->set_req(i,phi);
        map->set_req(TypeFunc::Control,region);
      }
  }

  // Do we need to lock argument 0?  Is this a synchronized native call?
  FastLockNode *flock = NULL;
  if( method && method->flags().is_synchronized() ) {
    ResourceMark rm;
    // Create a temp parser to emit locking code
    PhaseGVN gvn(Thread::current()->resource_area(),255);
    Unique_Node_List for_igvn(&_comp_arena);
    Parse native_parser( map, gvn, for_igvn );
    // Lock either the class-mirror or the regular 'this' pointer
    Node *lock_obj = static_klass ? mirror_node : unwrapped_this;
    gvn._types.map(start->_idx,start->bottom_type());
    gvn._types.map(map->memory()->_idx,Type::MEMORY);
    // Emit code for locking natives
    flock = native_parser.shared_lock( lock_obj, SynchronizationEntryBCI );
    // Reload local variable holding memory state
    memory = map->memory();
    _sync = native_parser._sync_max;
  }
  assert(_sync == (method && method->flags().is_synchronized() ? 1 : 0), "sync slots counted right");

  // Get base of thread-local storage area
  Node *thread = new (1) ThreadLocalNode( );

  // reset handle block
  if( jni_sig ) {
    Node *adr_hb = plus_adr( thread, JavaThread::active_handles_offset_in_bytes(), _top_node );
    Node *aho = new (3) LoadPNode(0, memory, adr_hb, TypePtr::BOTTOM);
    Node *adr_to = plus_adr( aho,JNIHandleBlock::top_offset_in_bytes(), _top_node);
    memory = new (4) StorePNode(0,memory,adr_to,null,Type::MEMORY);
  }

  // Drop in the last_Java_sp.  last_Java_fp is not touched.
  // Done for both JNI and runtime calls.
  Node *adr_sp = plus_adr( thread, JavaThread::last_Java_sp_offset_in_bytes(), _top_node);
  memory = new (4) StorePNode(0,memory, adr_sp, frameptr, Type::MEMORY);

#ifdef SPARC  // EXPIRES 3/1/1999
  Node *adr_post_sp = plus_adr( thread, JavaThread::post_Java_state_offset_in_bytes(), _top_node);
  if (!jni_sig) {
    // Reset post_Java_sp
    memory = new (4) StorePNode(0,memory, adr_post_sp, null, Type::MEMORY);
  }
#endif

  // Set _thread_in_native
  Node *adr_state = plus_adr(thread,JavaThread::thread_state_offset_in_bytes(), _top_node);
  Node *adr_last_native_pc = plus_adr(thread,JavaThread::last_native_pc_offset_in_bytes(), _top_node);
  if( jni_sig ) {    
    // The order of stores into TLS is critical!  First fill the
    // last_native_pc and then the thread state.
    Node *pc_node = new (1) LoadPCNode( 0 );    
    memory = new (4) StoreINode( 0, memory, adr_last_native_pc, pc_node, Type::MEMORY );

    // For all platforms, setting _thread_in_native is a flag indicating that
    // we are now GC safe until the flag is cleared.  This means we canNOT have
    // any uninitialized oopy stack slots.  We used to fail when calling static
    // natives because the store of the klass-mirror constant to the stack 
    // looked independent to the optimizer and it would slide the store past
    // setting _thread_in_native.  This, in turn, gave us a few instruction 
    // window where this thread could be halted for a GC, but the stack slot
    // was uninitialized and the GC chased junk and went out to lunch.
    if( static_klass ) {
      assert( mirror_box, "" );
      Node* loadklasscon = new (3) LoadPNode(0, memory, mirror_box,  TypeRawPtr::BOTTOM);
      memory = new (4) StorePNode(0, memory, mirror_box, loadklasscon, Type::MEMORY);
    }

#ifndef SPARC   // EXPIRES 3/1/1999
    memory = new (4) StoreINode( 0, memory, adr_state, Parse::intcon(_thread_in_native), Type::MEMORY );
#endif
  }
 
  map->set_req( TypeFunc::Memory , memory );

  //-----------------------------
  // Compute signature for C call.  Varies from the Java signature!
  const Type **fields = TypeTuple::fields(parm_cnt+2);
  uint cnt = TypeFunc::Parms;
  bool saw_jni_double_long = false;
  if( jni_sig ) {               // JNI style call or not?
    i = TypeFunc::Parms;
    // NOTE: All calls get the JNIEnv as an extra leading argument.
    // Static calls get the class oop where virtual calls get the reciever
    fields[cnt++] = TypeRawPtr::BOTTOM; // JNI Env pointer
    fields[cnt++] = (static_klass != NULL)
      ? TypeRawPtr::BOTTOM      // klass mirror handle
      : jdomain->_fields[i++];  // Else copy reciever type
    // Copy the remaining fields.  
    for( ciSignatureStream ss(method->signature()); !ss.at_return_type(); ss.next()) {
      BasicType bt = ss.type();
      if( bt == T_DOUBLE || bt == T_LONG ) {
        fields[cnt++] = jdomain->_fields[i++];
        assert( jdomain->_fields[i] == Type::HALF, "" );
        fields[cnt++] = jdomain->_fields[i++];
	saw_jni_double_long = true;
      } else {
        fields[cnt++] = jdomain->_fields[i++];
      }
    }
    assert( i == parm_cnt, "" );

  } else {
    // The C routines gets the base of thread-local storage passed in as an
    // extra argument.  Not all calls need it, but its cheap to add here.
    for( ; cnt<parm_cnt; cnt++ )
      fields[cnt] = jdomain->_fields[cnt];
    fields[cnt++] = TypeRawPtr::BOTTOM; // Thread-local storage
    // Also pass in the caller's PC, if asked for.
    if( return_pc )
      fields[cnt++] = TypeRawPtr::BOTTOM; // Return PC
  }
  const TypeTuple *domain = TypeTuple::make(cnt,fields);
  // The C routine we are about to call cannot return an oop; it can block on
  // exit and a GC will trash the oop while it sits in C-land.  Instead, we
  // return the oop through TLS for runtime calls and use a handle for JNI
  // calls.
  const TypeTuple *range = java_sig->_range;
  const Type *retval = range->_cnt == TypeFunc::Parms ? Type::TOP : range->_fields[TypeFunc::Parms];
  int retval_ptr = retval->isa_oop_ptr();
  if( retval_ptr ) {
    const Type **fields = TypeTuple::fields(1);
    if( jni_sig ) {             // JNI unboxes returned result
      fields[TypeFunc::Parms] = TypeRawPtr::BOTTOM; // return Handle
    } else {                    // Else C-land signature is void or address
      assert( pass_tls, "Oop must be returned thru TLS" );
      // Fancy-jumps return address; others return void
      fields[TypeFunc::Parms] = is_fancy_jump ? TypeRawPtr::BOTTOM : Type::TOP;
    }
    range = TypeTuple::make(TypeFunc::Parms+1,fields);
  }
  // Final C signature
  const TypeFunc *c_sig = TypeFunc::make(domain,range);

  //-----------------------------
  // Make the call node
  CallNode *call = jni_sig 
    ? new CallNativeNode ( c_sig, C_function, name )
    : new CallRuntimeNode( c_sig, C_function, name );
  //-----------------------------

  // Fix-up the debug info for the call
  call->_jvms = new JVMStateObj(method,0,0);
  call->_jvmtail = call->_jvms;
  call->_jvms->set_locoff(cnt);
  call->_jvms->set_spoff(cnt);
  call->_jvms->set_monoff(cnt);
  call->_jvms->set_endoff(cnt);
  
  // Set fixed predefined input arguments
  cnt = 0;
  for( i=0; i<TypeFunc::Parms; i++ )
    call->set_req( cnt++, map->in(i) );
  // A little too aggressive on the parm copy; return address is not an input
  call->set_req( TypeFunc::ReturnAdr, _top_node );
  // Add JNI required arguments
  if( jni_sig ) {               // JNI style call or not?
    Node *box = plus_adr(thread,JavaThread::jni_environment_offset_in_bytes(), _top_node);
    call->set_req(cnt++, box ); // Pass handle to JNIEnv
    if( static_klass )   	// Pass klass mirror handle for statics
      call->set_req(cnt++, mirror_box );
  }
  for( ; i<parm_cnt; i++ )      // Regular input arguments
    call->set_req( cnt++, map->in(i) );
  if( saw_jni_double_long )
    call->fix_mispaired_double_longs();
  if( !jni_sig ) {              // Runtime calls get the extra thread parm
    call->set_req( cnt++, thread );
    if( return_pc )             // Return PC, if asked for
      call->set_req( cnt++, map->in(TypeFunc::ReturnAdr) );
  }

  //-----------------------------
  // Now set up the return results
  map->set_req( TypeFunc::Control, new (1) ProjNode(call,TypeFunc::Control) );
  map->set_req( TypeFunc::I_O    , new (1) ProjNode(call,TypeFunc::I_O    ) );
  memory =                         new (1) ProjNode(call,TypeFunc::Memory )  ;
  map->set_req( TypeFunc::Memory , memory );
  // Capture 0, 1 or 2 return values.
  for( i = TypeFunc::Parms; i < range->_cnt; i++ ) {
    Node *retnode = new (1) ProjNode(call,i );
    // C-land is allowed to return sub-word values.  Convert to integer type.
    assert( retval != Type::TOP, "" );
    if( retval == TypeInt::BOOL ) 
      retnode = new (3) AndINode( retnode, Parse::intcon(0xFF) );
    else if( retval == TypeInt::CHAR ) 
      retnode = new (3) AndINode( retnode, Parse::intcon(0xFFFF) );
    else if( retval == TypeInt::BYTE ) 
      retnode = new (3) RShiftINode( new (3) LShiftINode( retnode, Parse::intcon(24) ), Parse::intcon(24) );
    else if( retval == TypeInt::SHORT ) 
      retnode = new (3) RShiftINode( new (3) LShiftINode( retnode, Parse::intcon(16) ), Parse::intcon(16) );
    map->set_req( i, retnode );
  }

  //-----------------------------

  if( jni_sig ) {    

    // Mark thread in transition
    memory = new (4) StoreINode( 0, memory, adr_state, Parse::intcon(_thread_in_native_trans), Type::MEMORY );

    map->set_req( TypeFunc::Memory, memory );

    // If a safepoint operation is in progress then we have to block
    // before entering Java code again
    Node *adr_as = new (1) ConPNode( TypeRawPtr::make( SafepointSynchronize::address_of_state() ) );
    Node *sstate = new (3) LoadINode( 0, memory, adr_as );
    CmpINode *cmp = new (3) CmpINode( sstate, Parse::intcon(SafepointSynchronize::_not_synchronized ) );
    BoolNode *b = new (2) BoolNode( cmp, BoolTest::ne );
    IfNode *iff = new (2) IfNode( map->in(TypeFunc::Control), b, 0.000001f, -1.0f );
    IfFalseNode *if_block    = new (1) IfFalseNode( iff );
    IfTrueNode  *if_no_block = new (1) IfTrueNode ( iff );

    // Create signature for call to "block"
    const Type **fields = TypeTuple::fields(1);
    fields[TypeFunc::Parms+0] = TypeRawPtr::BOTTOM; // Thread
    const TypeTuple *domain = TypeTuple::make(TypeFunc::Parms+1,fields);
    fields = TypeTuple::fields(1);
    fields[TypeFunc::Parms+0] = Type::TOP;
    const TypeTuple *range = TypeTuple::make(TypeFunc::Parms+1,fields);
    // Make a call to the runtime to "block"
    CallNode *block = new CallRuntimeNode(TypeFunc::make(domain,range), 
                                          CAST_FROM_FN_PTR(address, SafepointSynchronize::block),
                                          "SafepointSynchronize::block" );
    // The debug info is the only real input to this call.

    // Create debug-info for call, so ScopeDesc's are getting properly
    // recorded.  This code should be factored somehow.
    block->_jvms = new JVMStateObj(method,0,0);
    block->_jvmtail = block->_jvms;            
    block->_jvms->set_locoff(block->_func->_domain->_cnt);
    block->_jvms->set_spoff(block->_func->_domain->_cnt);
    block->_jvms->set_monoff(block->_func->_domain->_cnt);
    block->_jvms->set_endoff(block->_func->_domain->_cnt);
    
    // Set up inputs to call to "block"
    block->set_req( TypeFunc::Control  , if_no_block                );
    block->set_req( TypeFunc::I_O      , map->in( TypeFunc::I_O   ) );
    block->set_req( TypeFunc::Memory   , map->in( TypeFunc::Memory) );
    block->set_req( TypeFunc::FramePtr , frameptr                   );
    block->set_req( TypeFunc::ReturnAdr, _top_node                  );
    block->set_req( TypeFunc::Parms+0  , thread                     );

    // Merge control flow post call
    RegionNode *region = new RegionNode(3);
    region->set_req( 1, new (1) ProjNode( block, TypeFunc::Control ) );
    region->set_req( 2, if_block );
    map->set_req( TypeFunc::Control, region );

    // Setup merge point post the call
    for( i = TypeFunc::I_O; i < TypeFunc::FramePtr; i++ ) {
      PhiNode *phi = new PhiNode(region, map->in(i)->bottom_type());
      phi->set_req( 1, new (1) ProjNode( block,i ) );
      phi->set_req( 2, map->in(i) );
      map->set_req( i, phi );
    }

    //-----------------------------
    // Mark thread in Java code
    memory = map->in( TypeFunc::Memory );
    memory = new(4) StoreINode(0,memory, adr_state, Parse::intcon(_thread_in_Java), Type::MEMORY );

    //-----------------------------
    // Crucial - otherwise we cannot detect when native code is called
    // directly from the VM.  Clear last_native_pc
    memory = new (4) StorePNode( 0, memory, adr_last_native_pc, null, Type::MEMORY);

    //-----------------------------
    // Clear last_Java_sp
    memory = new (4) StorePNode( 0, memory, adr_sp, null, Type::MEMORY);

#ifdef SPARC    // EXPIRES 3/1/1999
    // Clear post_Java_sp
    memory = new (4) StorePNode( 0, memory, adr_post_sp, null, Type::MEMORY);
#endif

    // Clear last _last_native_pc
    memory = new (4) StoreINode( 0, memory, adr_last_native_pc, Parse::intcon(0), Type::MEMORY );
    
    //-----------------------------
    // JNI calls return boxed results.  Unbox.
    if( retval_ptr ) {
      // Test oop for NULL at runtime; unbox handle if not null
      CmpPNode *cmp = new (3) CmpPNode( map->in(TypeFunc::Parms), null );
      BoolNode *b = new (2) BoolNode( cmp, BoolTest::ne );
      IfNode *iff = new (2) IfNode( map->in(TypeFunc::Control), b, 0.1f, -1.0f );
      IfFalseNode *if_null     = new (1) IfFalseNode( iff );
      IfTrueNode  *if_not_null = new (1) IfTrueNode ( iff );
      RegionNode *region = new RegionNode(3);
      region->set_req(1, if_not_null );
      region->set_req(2, if_null );
      PhiNode *phi = new PhiNode(region, retval);
      Node *unbox = new (3) LoadPNode( if_not_null, memory, map->in(TypeFunc::Parms), retval->isa_ptr() );
      phi->set_req(1,unbox);
      phi->set_req(2,map->in(TypeFunc::Parms));
      map->set_req(TypeFunc::Parms,phi);
      map->set_req(TypeFunc::Control,region);
    }
  } // End of if JNI call
  else {    
    //-----------------------------
    // Clear last_Java_sp
    memory = new (4) StorePNode( 0, memory, adr_sp, null, Type::MEMORY);
#ifdef SPARC    // EXPIRES 3/1/1999
    // Clear post_Java_sp
    memory = new (4) StorePNode( 0, memory, adr_post_sp, null, Type::MEMORY);
#endif
  }

  // For is-fancy-jump, the C-return value is also the branch target
  Node *target = map->in(TypeFunc::Parms);
  // Runtime call returning oop in TLS?  Fetch it out
  if( pass_tls ) {
    Node *adr = plus_adr( thread, JavaThread::vm_result_offset_in_bytes(), _top_node);
    Node *vm_result = new (3) LoadPNode(0,memory, adr, retval->isa_ptr());
    map->set_req( TypeFunc::Parms, vm_result ); // vm_result passed as result
    // clear thread-local-storage(tls)
    memory = new (4) StorePNode(0, memory, adr, null, Type::MEMORY);
  }

  //-----------------------------
  // Unlock
  if( method && method->flags().is_synchronized() ) {
    // Synchronize cached version of memory state and 'map' version
    map->set_req( TypeFunc::Memory , memory ); 
    // Create a temp parser to emit locking code
    PhaseGVN gvn(Thread::current()->resource_area(),255);
    Unique_Node_List for_igvn(&_comp_arena);
    Parse native_parser( map, gvn, for_igvn );
    gvn._types.map(start->_idx,start->bottom_type());
    gvn._types.map(map->memory()->_idx,Type::MEMORY);
    native_parser.shared_unlock( flock->box_node(), flock->obj_node(), SynchronizationEntryBCI );
    // Reload local variable cache of memory state
    memory = map->in(TypeFunc::Memory);
  }

  //-----------------------------
  // check exception
  Node *adr = plus_adr( thread, Thread::pending_exception_offset_in_bytes(), _top_node);
  Node *pending = new (3) LoadPNode(0, memory, adr, TypePtr::BOTTOM);

  CmpPNode *cmp = new (3) CmpPNode( pending, null );
  BoolNode *bo  = new (2) BoolNode( cmp, BoolTest::ne );
  IfNode   *iff = new (2) IfNode( map->in(TypeFunc::Control), bo, 0.000001f, -1.0f );

  IfFalseNode *if_null     = new (1) IfFalseNode( iff );
  IfTrueNode  *if_not_null = new (1) IfTrueNode ( iff );

  assert (StubRoutines::forward_exception_entry() != NULL, "must be generated before");
  Node *exc_target = new (1) ConPNode( TypeRawPtr::make( StubRoutines::forward_exception_entry() ) );
  Node *to_exc = new TailCallNode(if_not_null,
                                  map->in(TypeFunc::I_O),
                                  memory,
                                  frameptr,
                                  map->in(TypeFunc::ReturnAdr),
                                  exc_target, null);
  _root->add_req(to_exc);

  //-----------------------------
  // If this is a normal subroutine return, issue the return and be done.
  Node *ret;
  switch( is_fancy_jump ) {
  case 0:                       // Make a return instruction
    // Return to caller, free any space for return address
    ret = new ReturnNode(if_null,
                         map->in(TypeFunc::I_O),
                         memory,
                         frameptr,
                         map->in(TypeFunc::ReturnAdr));
    for( i=TypeFunc::Parms; i<java_sig->_range->_cnt; i++ )
      ret->add_req( map->in(i) );
    break;
  case 1:    // This is a fancy tail-call jump.  Jump to computed address.
    // Jump to new callee; leave old return address alone.
    ret = new TailCallNode(if_null,
                           map->in(TypeFunc::I_O),
                           memory,
                           frameptr,
                           map->in(TypeFunc::ReturnAdr),
                           target, map->in(TypeFunc::Parms));
    break;
  case 2:                       // Pop return address & jump
    // Throw away old return address; jump to new computed address
    //assert(C_function == CAST_FROM_FN_PTR(address, OptoRuntime::rethrow_C), "fancy_jump==2 only for rethrow");
    ret = new TailJumpNode(if_null,
                           map->in(TypeFunc::I_O),
                           memory,
                           frameptr,
                           target, map->in(TypeFunc::Parms));
    break;
  default:
    ShouldNotReachHere();
  }
  _root->add_req(ret);
}

//------------------------------Generate--------------------------------------
// Create a Java-calling-convention-compatible piece of code.  This code calls
// C++ function with the same signature but using the C calling convention.
// This code marshals the arguments between Java and C.  It knows that C
// returns oops in thread local storage.  It knows to set the last_Java_sp so
// the stack crawlers can get their starting point.  It can box oops for 
// native routines (can box for runtime calls as well, but they generally do
// not need it).

// If 'is_fancy_jump' is set, then this code makes a fancy jump, instead of
// being a true subroutine; i.e. I do the classic tail-call optimization also
// known as a interprocedural-jump.  


//------------------------------Generate_Compiled_To_Interpreter_Graph--------
// From the TypeFunc signature, generate code to pass arguments 
// from the Java calling convention to the Interpreter's calling convention.
// 
// Compiled code always passes the MethodKlassHolder
void Compile::Generate_Compiled_To_Interpreter_Graph( const TypeFunc *java_sig, address interpreter_entry ) {
  uint       i;
  ciMethod* method   = NULL;

  // Expand signature to include inline_cached_klass and method_oop
  // Caller provides MethodKlassHolder, mkh.
  // at entry point, expand mkh into inline-cache-class and method-oop
  const TypeTuple *domain   = StartC2INode::c2i_domain( java_sig->_domain );
  const TypeFunc *start_sig = TypeFunc::make( domain, java_sig->_range );

  uint j_parm_cnt           = java_sig->_domain->_cnt;
  uint start_parm_cnt       = domain->_cnt;

  // The procedure start
  StartNode *start = new StartC2INode( _root, start_sig->_domain );

  // Create projections from start for incoming parameters in signature
  Node *map = new Node( max(start_parm_cnt, java_sig->_range->_cnt) );
  for( i=0; i<start_parm_cnt; i++ ) 
    map->set_req(i,new (1) ParmNode(start,i));

  // ---------- inline cache check ----------
  // implement using a MachUEPNode, prefixed to code in output.cpp
  // and preceeded by a MachMKHSplitNode

  // ---------- access components of MethodKlassHolder ----------
  // implement using a MachMKHSplitNode,
  // prefixed to unverified entry and verified entry points in output.cpp

  // ---------- collect information for oop-map on callee saved registers

  // SafePointNode *safe_point =new SafePointNode(new JVMStateObj(method,0,0));
  // for( i = 0; i < TypeFunc::Parms; i++ )
  //   safe_point->set_req( i, map->in(i) );
  // map->set_req( TypeFunc::Control, safe_point );


  // ---------- call interpreter ----------

  // Parameter Copy, from compiled convention to interpreter convention,
  // is done by the register allocator using the RegMask assignment
  // for incoming and outgoing parameters
  const TypeFunc  *call_sig  = start_sig;
  uint interp_parm_cnt       = start_parm_cnt;

  // Make the call node
  CallNode *call = new CallInterpreterNode( call_sig, interpreter_entry );

  // Fix-up the debug info for the call
  call->_jvms    = new JVMStateObj(method,0,0);
  call->_jvmtail = call->_jvms;
  call->_jvms->set_locoff( interp_parm_cnt );
  call->_jvms->set_spoff(interp_parm_cnt);
  call->_jvms->set_monoff(interp_parm_cnt);
  call->_jvms->set_endoff(interp_parm_cnt);

  // Set fixed predefined input arguments
  for( i=0; i<interp_parm_cnt; i++ )
    call->set_req( i, map->in(i) );
  // The interpreter's calling convention may require the storage of
  // misaligned doubles, which appear here as odd/even stack slot pairs.
  call->fix_mispaired_double_longs();
  // A little too aggressive on the parm copy; return address is not an input
  call->set_req( TypeFunc::ReturnAdr, _top_node );

  // Now set up the return results
  map->set_req( TypeFunc::Control, new (1) ProjNode(call,TypeFunc::Control) );
  map->set_req( TypeFunc::I_O    , new (1) ProjNode(call,TypeFunc::I_O    ) );
  map->set_req( TypeFunc::Memory , new (1) ProjNode(call,TypeFunc::Memory ) );
  // Capture 0, 1 or 2 return values.
  for( i=TypeFunc::Parms; i<java_sig->_range->_cnt; i++ ) {
    map->set_req(    i           , new (1) ProjNode(call,i )                );
  }

  // ---------- return instruction ----------

  // Make a return instruction
  Node *ret = new ReturnNode(map->in(TypeFunc::Control),
                             map->in(TypeFunc::I_O),
                             map->in(TypeFunc::Memory),
                             map->in(TypeFunc::FramePtr),
                             map->in(TypeFunc::ReturnAdr));
  for( i=TypeFunc::Parms; i<java_sig->_range->_cnt; i++ )
    ret->add_req( map->in(i) );
  _root->add_req(ret);       // Set compiler to new graph
  _tf  = start_sig;          // Set compiler to new type function
}


//------------------------------Generate_Interpreter_To_Compiled_Graph---------
// From the TypeFunc signature, generate code to pass arguments from the
// Interpreter's calling convention to the Compiler's convention.  Interpreter
// never passes MethodKlassHolder, inline_cached_klass or method_oop
void Compile::Generate_Interpreter_To_Compiled_Graph(const TypeFunc *java_sig){
  uint i;
  ciMethod*  method = NULL;
  
  // Update routine's signature for all components of compiler
  const TypeTuple *domain   = StartI2CNode::i2c_domain( java_sig->_domain );
  const TypeFunc *start_sig = TypeFunc::make( domain, java_sig->_range );

  uint j_parm_cnt     = java_sig->_domain->_cnt;
  uint start_parm_cnt = domain->_cnt;

  // ---------- StartI2CNode and its parameters

  // routine's signature is the same
  // The procedure start
  StartNode *start = new StartI2CNode( _root, start_sig->_domain );

  // Create projections from start for incoming parameters in signature
  Node *map = new Node( max(start_parm_cnt, java_sig->_range->_cnt) );
  for( i=0; i<start_parm_cnt; i++ ) 
    map->set_req( i, new (1) ParmNode( start, i ) );
  // The extra parameter, nmethod, is recognized by "StartNode::match"

  // ---------- collect information for oop-map

  // SafePointNode *safe_point =new SafePointNode(new JVMStateObj(method,0,0));
  // for( i = 0; i < TypeFunc::Parms; i++ )
  //   safe_point->set_req( i, map->in(i) );
  // map->set_req( TypeFunc::Control, safe_point );

  // ---------- call compiled ----------

  // Parameter Copy, from compiled convention to interpreter convention,
  // is done by the register allocator using signatures
  // The compiler's signature is the same as the interpreter's

  // Make the call node
  CallNode *call = new CallCompiledJavaNode( start_sig, NULL );

  // Fix-up the debug info for the call
  call->_jvms    = new JVMStateObj(method,0,0);
  call->_jvmtail = call->_jvms;
  call->_jvms->set_locoff( start_parm_cnt );
  call->_jvms->set_spoff(start_parm_cnt);
  call->_jvms->set_monoff(start_parm_cnt);
  call->_jvms->set_endoff(start_parm_cnt);

  // Set fixed predefined input arguments
  for( i=0; i<start_parm_cnt; i++ )
    call->set_req( i, map->in(i) );
  // return address is not an input
  call->set_req( TypeFunc::ReturnAdr, _top_node );

  // Now set up the return results
  map->set_req( TypeFunc::Control, new (1) ProjNode(call,TypeFunc::Control) );
  map->set_req( TypeFunc::I_O    , new (1) ProjNode(call,TypeFunc::I_O    ) );
  map->set_req( TypeFunc::Memory , new (1) ProjNode(call,TypeFunc::Memory ) );
  // Capture 0, 1 or 2 return values.
  for( i=TypeFunc::Parms; i<java_sig->_range->_cnt; i++ ) {
    map->set_req(    i           , new (1) ProjNode(call,i )                );
  }

  // ---------- return instruction ----------

  // Make a return instruction
  Node *ret = new ReturnNode(map->in(TypeFunc::Control),
                             map->in(TypeFunc::I_O),
                             map->in(TypeFunc::Memory),
                             map->in(TypeFunc::FramePtr),
                             map->in(TypeFunc::ReturnAdr));
  for( i = TypeFunc::Parms; i < java_sig->_range->_cnt; i++ ) {
    ret->add_req( map->in(i) );
  }
  _root->add_req(ret);       // Set compiler to new graph
  _tf  = start_sig;          // Set compiler to new type function
  return;
}
