/*
 * @(#)library_call.cpp	1.38 01/02/09 14:46:21
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_library_call.cpp.incl"

//------------------------------inline_library_methods--------------------
bool Parse::inline_library_methods(ciMethod* method,
                                   int nargs,
                                   const TypeFunc *tf,
                                   Bytecodes::Code bc,
                                   int bci) {
  // Test to see if the method uses the system class loader.  This test
  // is redundant to the tests below and should be removed if there is no
  // apparent efficiency benefit.
  ciInstanceKlass* holder = method->holder();
  if( !holder->uses_default_loader() ) {
    // We only inline library methods loaded by the system class loader.
    return false;
  }

  // JVMPI -- Turn off inlining
  if( C->is_set_jvmpi_method_event() ) {
    return false;
  }

  if (SpecialStringCompareTo) {
    if ( method->equals(C->get_String_compareTo()) ) {
      return inline_string_compareTo(method, tf, bci);
    }
  }

  if( !InlineNatives ) {        // Check for intrinsics
    return false;
  }

  // See if the method is one of our special cases.
  if( InlineArrayCopy &&
      method->equals(C->get_System_arraycopy()) ) {
    return inline_arraycopy(method, tf, bci);

  }
  ciInstanceKlass* math = C->get_Math();
  if( InlineMathNatives &&
      math->is_loaded() && holder->equals(math) ) {
    return inline_math_natives(method, nargs);

  }
  if( InlineObjectHash &&
      method->equals(C->get_Object_hashCode()) ) {
    return inline_native_hashcode(method, tf, bc, bci);
  }
  if( InlineObjectHash &&
      method->equals(C->get_System_identityHashCode()) ) {
    return inline_native_hashcode(method, tf, bc, bci);
  }
  if( InlineNewInstance &&
      method->equals(C->get_Class_newInstance0()) ) {
    return inline_native_newinstance(method, tf, bc, bci);

  }

  return false;
}

//------------------------------inline_string_compareTo------------------------
bool Parse::inline_string_compareTo(ciMethod* method,
				    const TypeFunc* tf,
				    int bci) {

  const int value_offset = java_lang_String::value_offset_in_bytes();
  const int count_offset = java_lang_String::count_offset_in_bytes();
  const int offset_offset = java_lang_String::offset_offset_in_bytes();

  _sp += 2;
  Node *argument = pop();  // pop non-receiver first:  it was pushed second
  Node *receiver = pop();
  
  // Null check on self without removing any arguments.  The argument
  // null check technically happens in the wrong place, which can lead to
  // invalid stack traces when string compare is inlined into a method
  // which handles NullPointerExceptions.
  _sp += 2;
  receiver = do_null_check(receiver, bci, T_OBJECT);
  if( !_map ) {  
    _sp -= 2;
    return true;
  }
  argument = do_null_check(argument, bci, T_OBJECT);
  _sp -= 2;
  if( !_map ) {
    return true;
  }
  
  ciInstanceKlass* klass = env()->String_klass();
  const TypeOopPtr* string_type =
    TypeOopPtr::make(TypePtr::BotPTR, klass, NULL, 0);
  
  Node* compare =
    _gvn.transform(new (7) StrCompNode(
			control(), 
			memory(TypeAryPtr::CHARS),
			memory(string_type->add_offset(value_offset)),
			memory(string_type->add_offset(count_offset)),
			memory(string_type->add_offset(offset_offset)),
			receiver, 
			argument));
  push(compare);
  return true;
}

//------------------------------inline_math_native-----------------------------
bool Parse::inline_math_natives(ciMethod* method, int nargs) {
  if (method->equals(C->get_Math_sin())) {
    if (Matcher::sinDSupported()) {
      _sp += nargs;		// restore stack poiinter
      pop(); Node* b = pop();  
      Node* a = _gvn.transform( new (2) SinDNode(0, b) );
      push( a ); push( _top );
      return true;
    }
  } else if
     (method->equals(C->get_Math_cos())) {
    if (Matcher::cosDSupported()) {
      _sp += nargs;		// restore stack pointer
      pop(); Node* b = pop();  
      Node* a = _gvn.transform( new (2) CosDNode(0, b) );
      push( a ); push( _top );
      return true;
    }
  } else if
     (method->equals(C->get_Math_tan())) {
    if (Matcher::tanDSupported()) {
      if (PrintInlining) {
        tty->print_cr("Cannot inline math native java.lang.Math.tanD()");
      }
      return false; // not yet correctly implemented
      _sp += nargs;                 // restore stack pointer
      pop(); Node* b = pop();  
      Node* a = _gvn.transform( new (2) TanDNode(0, b) );
      push( a ); push( _top );
      return true;
    }
  } else if
     (method->equals(C->get_Math_atan2())) {
    if (Matcher::atanDSupported()) {
      if (PrintInlining) {
        tty->print_cr("Cannot inline math native java.lang.Math.atanD()");
      }
      return false; // not yet correctly implemented
      _sp += nargs;                 // restore stack pointer
      pop(); Node* a = pop();
      pop(); Node* b = pop();  
      a = _gvn.transform( new (3) AtanDNode(0, a, b) );
      push( a ); push( _top );
      return true;
    }
  } else if
     (method->equals(C->get_Math_sqrt())) {
    if (Matcher::sqrtDSupported()) {
      _sp += nargs;                 // restore stack pointer
      pop(); Node* b = pop();  
      Node* a = _gvn.transform( new (2) SqrtDNode(0, b) );
      push( a ); push( _top );
      return true;
    }
  } else {
    if (PrintInlining) {
      tty->print("Cannot inline math native ");
      method->name()->print();
      tty->cr();
    }
  }
  return false;
}

//------------------------------generate_guard---------------------------
// Helper function for generating guarded fast-slow graph structures.
void Parse::generate_guard(Node *test,
                           RegionNode *slow_region) {
  // Build an if node and its projections.
  // If test is true we take the slow path, which we assume is uncommon.
  // IfNode *iff = create_and_map_if( &_gvn, control(), test );
  // replaced with create_and_map_if()
  IfNode *iff = new (2) IfNode( control(), test, (float)0.001, (float)-1.0 );
  _gvn._types.map(iff->_idx, iff->Value(&_gvn));

  Node *if_slow = _gvn.transform( new (1) IfTrueNode(iff) );
  if( if_slow == _top ) {
    // The slow branch is never taken.  No need to build this guard.
    return;
  }
  slow_region->add_req(if_slow);
  
  Node *if_fast = _gvn.transform( new (1) IfFalseNode(iff) );
  set_control(if_fast);

  return;
}

//------------------------------inline_native_hashcode--------------------
// Build special case code for calls to hashCode on an object.
bool Parse::inline_native_hashcode(ciMethod* method,
                                   const TypeFunc *tf,
                                   Bytecodes::Code bc,
                                   int bci) {
  _sp++;
  Node *obj = pop();
  bool is_static = method->is_static();
  RegionNode *zero_result = NULL;

  if (!is_static) {
    _sp++;                        // Restore the stack
    // Null check without removing any arguments.
    obj = do_null_check(obj, bci, T_OBJECT);
    // Check for locking null object
    if( !_map ) {  return true;  }
    // NOW we can remove the argument
    _sp--;
  } else {
    // Do a null check, and return zero if null.
    zero_result = new RegionNode(3 + 1);
    obj = null_check_oop(zero_result, obj, _null, false, bci);
    set_control(obj->in(0));
    if( control() == _top ) {
      // System.identityHashCode(null) == 0
      push(_gvn.intcon(0));
      set_control(zero_result->in(3));
      return true;
    }
    obj = _gvn.transform(obj);
  }

  // This call may be virtual (invokevirtual) or bound (invokespecial).
  // For each case we generate slightly different code.
  bool is_virtual = (bc == Bytecodes::_invokevirtual);
  assert( is_static ? (bc == Bytecodes::_invokestatic) : (is_virtual || bc == Bytecodes::_invokespecial),
          "expected invokevirtual or invokespecial" );

  // We only go to the fast case code if we pass a number of guards.  The
  // paths which do not pass are accumulated in the slow_region.
  RegionNode *slow_region = new RegionNode(1);
  _for_igvn.push(slow_region);

  // If this is a virtual call, we generate a funny guard.  We pull out
  // the vtable entry corresponding to hashCode() from the target object.
  // If the target method which we are calling happens to be the native
  // Object hashCode() method, we pass the guard.  We do not need this
  // guard for non-virtual calls -- the caller is known to be the native
  // Object hashCode().
  if (is_virtual) {
    // Get the klass of the argument.
    Node *klass_addr = basic_plus_adr( obj, obj, oopDesc::klass_offset_in_bytes() );
    Node* obj_klass = _gvn.transform(new (3) LoadKlassNode(0, memory(klass_addr), klass_addr, TypeKlassPtr::OBJECT));

    // Get the methodOop out of the appropriate vtable entry.
    int vtable_index  = method->vtable_index();
    int entry_offset  = (instanceKlass::vtable_start_offset() +
                       vtable_index*vtableEntry::size()) * wordSize +
                       vtableEntry::method_offset_in_bytes();
    Node *entry_addr  = basic_plus_adr( obj_klass, obj_klass, entry_offset );
    Node *target_call = _gvn.transform(new (3) LoadPNode(0, memory(entry_addr), entry_addr, TypeOopPtr::NOTNULL));

    // Compare the target call with the native hashCode
    // const TypeKlassPtr* native_call_addr = TypeKlassPtr::make(method->klass());
    const TypeOopPtr*   native_call_addr = TypeOopPtr::make(method);

    Node *native_call = makecon(native_call_addr);
    Node *chk_native  = _gvn.transform( new (3) CmpPNode( target_call, native_call));
    Node *test_native = _gvn.transform( new (2) BoolNode( chk_native, BoolTest::ne) );
    
    generate_guard(test_native, slow_region);
  }

  // Get the header out of the object, use LoadMarkNode when available
  Node *header_addr = basic_plus_adr( obj, obj, oopDesc::mark_offset_in_bytes());
  Node *header = _gvn.transform( new (3) LoadINode(0, memory(header_addr), header_addr, TypeInt::INT));

  // Test the header to see if it is unlocked.
  Node *lock_mask      = _gvn.intcon(markOopDesc::lock_mask_in_place);
  Node *lmasked_header = _gvn.transform( new (3) AndINode(header, lock_mask) );
  Node *unlocked_val   = _gvn.intcon(markOopDesc::unlocked_value);
  Node *chk_unlocked   = _gvn.transform( new (3) CmpINode( lmasked_header, unlocked_val));
  Node *test_unlocked  = _gvn.transform( new (2) BoolNode( chk_unlocked, BoolTest::ne) );

  generate_guard(test_unlocked, slow_region);

  // Get the hash value and check to see that it has been properly assigned.
  Node *hash_mask      = _gvn.intcon(markOopDesc::hash_mask_in_place);
  Node *hash_shift     = _gvn.intcon(markOopDesc::hash_shift);
  Node *hmasked_header = _gvn.transform( new (3) AndINode(header, hash_mask) );
  Node *hash_val       = _gvn.transform( new (3) URShiftINode(hmasked_header,
                                                              hash_shift) );
  Node *no_hash_val    = _gvn.intcon(markOopDesc::no_hash);
  Node *chk_assigned   = _gvn.transform( new (3) CmpINode( hash_val, no_hash_val));
  Node *test_assigned  = _gvn.transform( new (2) BoolNode( chk_assigned, BoolTest::eq) );

  generate_guard(test_assigned, slow_region);

  Node *fast_control = control();

  Node *slow_call;
  Node *slow_control;

  // Generate code for the slow case.  We make a call to hashCode().
  if (is_virtual) {
    slow_call = new CallDynamicJavaNode(tf, OptoRuntime::resolve_virtual_call_Java(),
                                        method, bci);
  } else if (is_static) {
    slow_call = new CallStaticJavaNode(tf, OptoRuntime::resolve_static_call_Java(),
				       method, bci);
  } else {
    CallStaticJavaNode *call = new CallStaticJavaNode(tf,
                                       OptoRuntime::resolve_opt_virtual_call_Java(),
                                       method, bci);
    call->set_optimized_virtual(true);
    slow_call = call;
  }

  // Set fixed predefined input arguments
  slow_call->set_req( TypeFunc::Control,   _gvn.transform(slow_region) );
  slow_call->set_req( TypeFunc::I_O    ,   i_o() );
  slow_call->set_req( TypeFunc::Memory ,   all_memory() );
  slow_call->set_req( TypeFunc::ReturnAdr, _top );
  slow_call->set_req( TypeFunc::FramePtr,  frameptr() );
  slow_call->set_req( TypeFunc::Parms+0,   obj);

  add_safepoint_edges(slow_call->is_SafePoint(),bci);

  slow_call = _gvn.transform(slow_call);

  set_control   (_gvn.transform(new(1) ProjNode(slow_call,TypeFunc::Control)));
  set_i_o       (_gvn.transform(new(1) ProjNode(slow_call,TypeFunc::I_O    )));
  set_all_memory_call(slow_call);

  ciExceptionHandlerStream handlers(_method, bci);
  Node *call_io = _gvn.transform(new(1) ProjNode(slow_call, TypeFunc::I_O) );
  Node *catch_ = catch_exceptions(&handlers,bci,call_io);
  slow_control = _gvn.transform(new(1)CatchProjNode(catch_, CatchProjNode::fall_through_index, CatchProjNode::no_handler_bci));

  // Make the merge region/phi, which combine the fast and slow
  // controls/results.
  RegionNode *result_region = zero_result ? zero_result : new RegionNode(3);
  _for_igvn.push(result_region);
  result_region->set_req(1,slow_control);
  result_region->set_req(2,fast_control);
  _gvn._types.map(result_region->_idx,result_region->bottom_type());
  set_control(result_region);

  Node *phi = new PhiNode(result_region,Type::ABIO);
  phi->set_req(1,i_o()                      );
  phi->set_req(2,slow_call->in(TypeFunc::I_O));
  if (zero_result)  phi->set_req(3,phi->in(2));
  _gvn._types.map(phi->_idx,Type::ABIO);
  set_i_o(_gvn.transform(phi));

  phi = new PhiNode(result_region,Type::MEMORY);
  phi->set_req(1,all_memory()                                  );
  phi->set_req(2,_gvn.transform(slow_call->in(TypeFunc::Memory)));
  if (zero_result)  phi->set_req(3,phi->in(2));
  _gvn._types.map(phi->_idx,Type::MEMORY);
  set_all_memory(_gvn.transform(phi));

  Node *phi_res = new PhiNode(result_region,TypeInt::INT);
  phi_res->set_req(1,_gvn.transform(new(1) ProjNode(slow_call,TypeFunc::Parms+0)));
  phi_res->set_req(2,hash_val);
  if (zero_result)  // System.identityHashCode(null) == 0
    phi_res->set_req(3,_gvn.intcon(0));

  push(_gvn.transform( phi_res ));

  return true;
}

//------------------------------inline_native_newinstance----------------------
// Build special case code for calls to newInstance0()  on a Class object.

// This call only occurs inside Class.newInstance().  The newInstance0()
// method does some fairly expensive security check which are based on the
// class of the method calling newInstance().  If the security check passes
// and the class has a vanilla constructor (it's constructor and all
// superclass constructors don't do anything), the calling class is cached in
// the Class object.  If we are inlining newInstance(), we know the calling
// class, so we can allocate the object directly if the calling class matches
// the cached class.  The "class has a vanilla constructor" test excludes
// things with finalizers or Arrays.
bool Parse::inline_native_newinstance( ciMethod* method, const TypeFunc *tf, Bytecodes::Code bc, int bci ) {
  assert(_method->equals(C->get_Class_newInstance()), "make sure we are in Class.newInstance()");

  // make sure we are inlining
  if (_depth < 2)
    return false;

  _sp += 1;
  Node *receiver = pop();

  uint raw_idx = C->get_alias_index( TypeRawPtr::BOTTOM );
  Node *old_raw_mem, *mem;

  old_raw_mem = memory(raw_idx);

  // Get the caller klass
  ciInstanceKlass  *caller_klass = _jhead->method()->holder();
  // Get the cached caller klass of the reciever
  Node *klass_node = makecon(TypeKlassPtr::make(caller_klass));

  int  cached_klass_offs = java_lang_Class::fastpath_caller_class_offset_in_bytes();
  Node *cached_klass_addr = basic_plus_adr( receiver, receiver, cached_klass_offs);
  Node* cached_klass = _gvn.transform(new (3) LoadPNode(0,
                    memory(cached_klass_addr), cached_klass_addr, TypeKlassPtr::OBJECT));

  //  Region to merge the 3 paths.
  //  Inputs:  1 - slow allocation call, 2 - fast path allocation, 3 - newInstance0() call
  RegionNode *merge_region = new RegionNode(4);
  Node *phi_result = new PhiNode(merge_region, TypeRawPtr::NOTNULL);
  Node *phi_mem = new PhiNode( merge_region, Type::MEMORY );
  Node *phi_io = new PhiNode( merge_region, Type::ABIO);

  // compare caller class to cached klass
  Node *cmp_klasses  = _gvn.transform( new (3) CmpPNode( klass_node, cached_klass));
  Node *test_klasses = _gvn.transform( new (2) BoolNode( cmp_klasses, BoolTest::ne) );
  IfNode *iff = new (2) IfNode( control(), test_klasses, (float)0.001, (float)-1.0 );
  _gvn._types.map(iff->_idx, iff->Value(&_gvn));
  Node *if_slow = _gvn.transform( new (1) IfTrueNode(iff) );
  Node *if_fast = _gvn.transform( new (1) IfFalseNode(iff) );
  set_control(if_fast);

  // compute size of object to allocate
  int klass_offset = java_lang_Class::klass_offset_in_bytes();
  Node *klass_addr = basic_plus_adr( receiver, receiver,  klass_offset);
  Node *klass = _gvn.transform(new (3) LoadPNode(0,
                    memory(klass_addr), klass_addr, TypeKlassPtr::OBJECT));
  int size_helper_offset = klassOopDesc::klass_part_offset_in_bytes() +
                           Klass::size_helper_offset_in_bytes();
  Node *size_helper_addr = basic_plus_adr( klass, klass,  size_helper_offset);
  Node *instance_words = _gvn.transform( new (3) LoadINode(0, memory(size_helper_addr), size_helper_addr));
  Node *instance_size = _gvn.transform( new (3) LShiftINode(instance_words, _gvn.intcon(log2(wordSize))));

  Node *newoop, *slow_alloc_ctrl;
  Node *post_alloc_mem;

  //
  // fast path inline allocation
  //
  newoop = fast_path_allocate(merge_region, instance_size, klass, &slow_alloc_ctrl);
  post_alloc_mem = memory(raw_idx);

  // Set control to the fast path to pin the stores there
  set_control( merge_region->in(2) );

  init_object(klass, newoop, false);
  mem = memory(raw_idx);

  if(!ZeroTLE) {
    Node* data_offset = _gvn.intcon(oopDesc::header_size() * wordSize);
    Node* dataP = _gvn.transform(new (4) AddPNode(newoop, newoop, data_offset));
    Node* dataSize = _gvn.transform(new (3)
                     SubINode(instance_words, _gvn.intcon(oopDesc::header_size())));
    mem = _gvn.transform(new (4) ClearArrayNode(control(),mem,dataSize,dataP));
  }
  // merge_region input 2 was set by fast_path_allocate()
  phi_result->set_req(2, newoop);
  phi_mem->set_req(2, mem);
  phi_io->set_req(2, i_o());

  //
  // if fast path allocation overflowed heap area, make call to new_Java()
  // Note: we can't just go to the newInstance0() call on an overflow
  //       because the heap top has been incremented past the end.
  //
  set_control( slow_alloc_ctrl);
  set_memory( post_alloc_mem, raw_idx );

  // Make the slow-path GC stuff
  make_slow_call( OptoRuntime::new_Type(), OptoRuntime::new_Java(), NULL,
                  bci, slow_alloc_ctrl, newoop, klass );
  Node *sacall = control()->in(0);
  make_slow_call_ex( sacall, bci, env()->OutOfMemoryError_klass());
  merge_region->set_req(1, control());
  phi_result->set_req(1, _gvn.transform( new (1) ProjNode(sacall,TypeFunc::Parms)));
  phi_mem->set_req(1, memory(raw_idx));
  phi_io->set_req(1, i_o());


  //
  // Call to newInstance0()
  //
  set_control( if_slow);
  set_memory( old_raw_mem, raw_idx );
  CallStaticJavaNode *call = new CallStaticJavaNode(tf,
              OptoRuntime::resolve_opt_virtual_call_Java(), method, bci);
  call->set_req( TypeFunc::Parms+0,   receiver);
  // Set fixed predefined input arguments
  call->set_req( TypeFunc::Control,   control() );
  call->set_req( TypeFunc::I_O    ,   i_o() );
  call->set_req( TypeFunc::Memory ,   all_memory() );
  call->set_req( TypeFunc::ReturnAdr, _top );
  call->set_req( TypeFunc::FramePtr,  frameptr() );
  call->set_optimized_virtual(true);

  add_safepoint_edges(call->is_SafePoint(),bci);

  call = (CallStaticJavaNode *) _gvn.transform(call);

  set_control   (_gvn.transform(new(1) ProjNode(call,TypeFunc::Control)));
  set_i_o       (_gvn.transform(new(1) ProjNode(call,TypeFunc::I_O    )));
  set_all_memory_call(call);

  ciExceptionHandlerStream handlers(_method, bci);
  Node *call_io = _gvn.transform(new(1) ProjNode(call, TypeFunc::I_O) );
  Node *catch_ = catch_exceptions(&handlers,bci,call_io);

  merge_region->set_req(3, _gvn.transform(
         new(1)CatchProjNode(catch_, CatchProjNode::fall_through_index, CatchProjNode::no_handler_bci)));
  phi_result->set_req(3, _gvn.transform( new (1) ProjNode(call,TypeFunc::Parms)));
  phi_mem->set_req(3, memory(raw_idx));
  phi_io->set_req(3, i_o());

  //
  // merge region
  //
  set_control( _gvn.transform(merge_region) );
  _for_igvn.push(merge_region);

  newoop = _gvn.transform(phi_result);

  set_all_memory( _gvn.transform(phi_mem));

  _gvn._types.map(phi_io->_idx,Type::ABIO);
  set_i_o(_gvn.transform(phi_io));

  // Cast to correct type
  newoop = _gvn.transform( new (2) CheckCastPPNode( control(), newoop,
                          TypeJavaPtr::make_from_klass(method->return_klass())));

  // Push resultant oop onto stack
  push( newoop );

  return true;
}

//------------------------------type2basic-------------------------------------
BasicType type2basic(const Type *t) {
  if (t == TypeInt::CHAR) {
    return T_CHAR;
  } else if (t == TypeInt::INT) {
    return T_INT;
  } else if (t == TypeInt::BYTE || t == TypeInt::BOOL) {
    return T_BYTE;
  } else if (t == TypeInt::SHORT) {
    return T_SHORT;
  } else if (t == TypeInt::DOUBLE) {
    return T_DOUBLE;
  } else if (t == TypeInt::FLOAT) {
    return T_FLOAT;
  } else if (t == TypeLong::LONG) {
    return T_LONG;
  } else if (t->isa_aryptr()) {
    assert( t->isa_oopptr() == NULL, "instance and array oops are not ambiguous")
    return T_ARRAY;
  } else if (t->isa_oopptr()) {
    return T_OBJECT;
  } else if (t->isa_klassptr()) {
    ShouldNotReachHere();
    return T_OBJECT;
  }

  return T_VOID;
}

//------------------------------inline_arraycopy-----------------------
bool Parse::inline_arraycopy(ciMethod* method,
                             const TypeFunc *tf,
                             int bci) {


  // Restore the stack and pop off the arguments.
  _sp += 5;
  Node *length = pop();
  Node *dest_offset = pop();
  Node *dest = pop();
  Node *src_offset = pop();
  Node *src = pop();

  // Compile time checks.  If any of these checks cannot be verified at compile time,
  // we do not make a fast path for this call.  Instead, we let the call remain as it
  // is.  The checks we choose to mandate at compile time are:
  //
  // (1) src is an array.
  // (2) dest is an array.
  // (3) The component types of the src and dest arrays are verifiably compatible.
  //     Specifically:
  //     (a) if both component types are primitive types, then they are the same
  //         primitive type.
  //     (b) if both component types are reference types, generate a runtime check
  //         that the src and dest types are identical.
  //     (c) the component types of src and dest are not mixed primitive/reference.

  // Are both src and dest verifiably arrays?
  const Type *src_type = src->Value(&_gvn);
  const Type *dest_type = dest->Value(&_gvn);
  const TypeAryPtr *top_src = src_type->isa_aryptr();
  const TypeAryPtr *top_dest = dest_type->isa_aryptr();
  if (!top_src || !top_dest) {
    // The top level type of either src or dest is not known to be an
    // array.  Punt.
    return false;
  }

  // Walk the array chain until we get to a component type.

  // !!!!! Currently our comparison of array component types is done based on
  // the compiler's type system rather than the VM's type system.  This could
  // get us in trouble if, for example, the compiler figures out that an array
  // of ints is only assigned values in byte range.  The solution for this is
  // to use the VM's type system for this check.  Unfortunately, the TypeAry
  // objects do not contain pointers to Klasses in the VM type system.  These
  // members should be added and this test should be modified.
  const TypeAryPtr *p_src = top_src;
  const TypeAryPtr *p_dest = top_dest;
  const Type *t_src = NULL;
  const Type *t_dest = NULL;
  bool generate_type_check = false;
  do {
    t_src = p_src->_ary->_elem;
    t_dest = p_dest->_ary->_elem;
    p_src = t_src->isa_aryptr();
    p_dest = t_dest->isa_aryptr();
  } while (p_src && p_dest);

  if (!p_src && !p_dest) {
    // Neither t_src nor t_dest is an array type.

    const TypeOopPtr *oop_src = t_src->isa_oopptr();
    const TypeOopPtr *oop_dest = t_dest->isa_oopptr();
    if (oop_src && oop_dest) {
      generate_type_check = true;
    } else if (oop_src || oop_dest) {
      // One of the arrays has a primitive component type, the other has
      // a reference component type.  Punt.
      return false;
    } else {
      // Both arrays have primitive component type.  Check to see that these
      // types are identical.
      BasicType dest_type = type2basic(t_dest);
      if (type2basic(t_src) != dest_type  ||
          T_VOID == dest_type) {
        // The component types are not the same or are not recognized.  Punt.
        return false;
      }
    }
  } else {
    // One of t_src and t_dest is not an array type.  The only case we
    // allow here is the case where the destination array has a component
    // type of java.lang.Object and the source array has an array component
    // type.
    const TypeOopPtr *oop_dest = t_dest->isa_oopptr();

    if (!oop_dest || oop_dest->klass()->is_java_lang_Object()) {
      // The destination array does not have a java.lang.Object component type.
      return false;
    } else {
      generate_type_check = true;
    }
  }

  // Figure out the size and type of the elements we will be copying.
  bool is_primitive;
  size_t elem_size;
  const Type *elem_type = top_dest->_ary->_elem;
  BasicType basic_elem_type = type2basic(elem_type);

  // Is the first level component type a primitive type?  Note that
  // isa_oop_ptr returns true for array ptrs as well.
  is_primitive = !elem_type->isa_oop_ptr();
  elem_size = type2aelembytes[basic_elem_type];
  if (basic_elem_type == T_VOID) {
    // We do not recognize the top level array component type.
    // This case will probably be caught by the not equal T_VOID test in
    // the preceding loop, but this test is here for robust and obvious
    // correctness.
    return false;
  }

  //---------------------------------------------------------------------------
  // We will make a fast path for this call to arraycopy.

  // We have the following tests left to perform:
  // 
  // (1) src must not be null.
  // (2) dest must not be null.
  // (3) src klassOop and dest klassOop must be identical
  // (4) src_offset must not be negative.
  // (5) dest_offset must not be negative.
  // (6) length must not be negative.
  // (7) src_offset + length must not exceed length of src.
  // (8) dest_offset + length must not exceed length of dest.

  // !!!!! Null Checks (1) and (2)
  // We currently perform our null checks with the do_null_check routine.
  // This means that the null exceptions will be reported in the caller
  // rather than (correctly) reported inside of the native arraycopy call.
  // This should be corrected, given time.  We do our null check with the
  // stack pointer restored.
  _sp += 5;
  src = do_null_check(src, bci, T_ARRAY);
  // Check for locking null object
  if( !_map ) {
    _sp -= 5;
    return true;
  }
  dest = do_null_check(dest, bci, T_ARRAY);
  _sp -= 5;
  // Check for locking null object
  if( !_map ) {  return true;  }

  RegionNode *slow_region = new RegionNode(1);
  _for_igvn.push(slow_region);

  // (3) src klassOop and dest klassOop must be identical
  if (generate_type_check) {
    // Get the klassOop for both src and dest
    Node* klass_offset = _gvn.intcon(oopDesc::klass_offset_in_bytes());

    Node* p1 = _gvn.transform( new (4) AddPNode(src , src , klass_offset) );
    Node* p2 = _gvn.transform( new (4) AddPNode(dest, dest, klass_offset) );
    Node* src_klass  = _gvn.transform(new (3) LoadKlassNode(0, memory(p1), p1, TypeKlassPtr::OBJECT));
    Node* dest_klass = _gvn.transform(new (3) LoadKlassNode(0, memory(p2), p2, TypeKlassPtr::OBJECT));

    // Check that src_klass is a subclass of dest_klass
    Node *check = _gvn.transform( new (3) CheckTypeNode( control(), src_klass, dest_klass, (float)0.000001, (float)-1.0 ) );
    IfNode *subclass_iff = check->is_If();
    assert( subclass_iff, "Expect an IfNode");

    // Build slow_path and fast_path projections when test is not decidable
    Node *subclass_slow = _gvn.transform( new (1) IfTrueNode(subclass_iff) );
    if( subclass_slow != _top ) {
      slow_region->add_req( subclass_slow );
      Node *subclass_fast = _gvn.transform( new (1) IfFalseNode(subclass_iff) );
      set_control( subclass_fast );
    }
  }

  // (4) src_offset must not be negative.
  Node *cmp  = _gvn.transform( new (3) CmpINode( src_offset, _gvn.intcon(0)));
  Node *test = _gvn.transform( new (2) BoolNode( cmp, BoolTest::lt) );
  generate_guard(test, slow_region);

  // (5) dest_offset must not be negative.
  cmp  = _gvn.transform( new (3) CmpINode( dest_offset, _gvn.intcon(0)));
  test = _gvn.transform( new (2) BoolNode( cmp, BoolTest::lt) );
  generate_guard(test, slow_region);

  // (6) length must not be negative.
  cmp  = _gvn.transform( new (3) CmpINode( length, _gvn.intcon(0)));
  test = _gvn.transform( new (2) BoolNode( cmp, BoolTest::lt) );
  generate_guard(test, slow_region);

  // (7) src_offset + length must not exceed length of src.
  Node *last_access  = _gvn.transform( new (3) AddINode(src_offset, length));
  Node *base         = basic_plus_adr( src, src, arrayOopDesc::length_offset_in_bytes());
  Node *array_length = _gvn.transform( new (3) LoadRangeNode(0, memory(TypeAryPtr::RANGE), base, TypeInt::POS));

  cmp  = _gvn.transform( new (3) CmpUNode( array_length, last_access));
  test = _gvn.transform( new (2) BoolNode( cmp, BoolTest::lt) );
  generate_guard(test, slow_region);

  // (8) dest_offset + length must not exceed length of dest.
  last_access  = _gvn.transform( new (3) AddINode(dest_offset, length));
  base         = basic_plus_adr( dest, dest, arrayOopDesc::length_offset_in_bytes());
  array_length = _gvn.transform( new (3) LoadRangeNode(0, memory(TypeAryPtr::RANGE), base, TypeInt::POS));

  cmp  = _gvn.transform( new (3) CmpUNode( array_length, last_access));
  test = _gvn.transform( new (2) BoolNode( cmp, BoolTest::lt) );
  generate_guard(test, slow_region);

  Node          *slow_call;
  Node          *slow_control;
  SafePointNode *slow_map;
  {
    PreserveJVMState pjvms(this, bci);

    // Generate code for the slow case.  We make a call to the native arraycopy method.

    slow_call = new CallStaticJavaNode(tf, OptoRuntime::resolve_static_call_Java(),
                                       method, bci);

    // Set fixed predefined input arguments
    slow_call->set_req( TypeFunc::Control,   _gvn.transform(slow_region) );
    slow_call->set_req( TypeFunc::I_O    ,   i_o      () );
    slow_call->set_req( TypeFunc::Memory ,   all_memory() );
    slow_call->set_req( TypeFunc::ReturnAdr, _top );
    slow_call->set_req( TypeFunc::FramePtr,  frameptr() );
    slow_call->set_req( TypeFunc::Parms+0,   src);
    slow_call->set_req( TypeFunc::Parms+1,   src_offset);
    slow_call->set_req( TypeFunc::Parms+2,   dest);
    slow_call->set_req( TypeFunc::Parms+3,   dest_offset);
    slow_call->set_req( TypeFunc::Parms+4,   length);

    add_safepoint_edges(slow_call->is_SafePoint(), bci);
    slow_call = _gvn.transform(slow_call);

    set_control (_gvn.transform(new(1) ProjNode(slow_call,TypeFunc::Control)));
    set_i_o     (_gvn.transform(new(1) ProjNode(slow_call,TypeFunc::I_O    )));
    set_all_memory_call(slow_call);

    ciExceptionHandlerStream handlers(_method, bci);
    Node *call_io = _gvn.transform(new(1) ProjNode(slow_call, TypeFunc::I_O) );
    Node *catch_ = catch_exceptions(&handlers,bci,call_io);
    slow_control = _gvn.transform(new(1)CatchProjNode(catch_, CatchProjNode::fall_through_index, CatchProjNode::no_handler_bci));

    slow_map = _map;
  }

  // Compute the starting address in each array by scaling the offset
  // and adding it to the array base.
  Node *src_base  = basic_plus_adr( src, src,
                                    arrayOopDesc::base_offset_in_bytes(basic_elem_type) );
  Node *dest_base = basic_plus_adr( dest, dest,
                                    arrayOopDesc::base_offset_in_bytes(basic_elem_type) );

  Node *scale              = _gvn.intcon(elem_size);
  Node *scaled_src_offset  = _gvn.transform( new (3) MulINode(src_offset, scale) );
  Node *scaled_dest_offset = _gvn.transform( new (3) MulINode(dest_offset, scale) );

  Node *src_start  = _gvn.transform( new (4) AddPNode( src, src_base, scaled_src_offset) );
  Node *dest_start = _gvn.transform( new (4) AddPNode( dest, dest_base, scaled_dest_offset) );

  // We call a different routine for object array copies and primitive array copies.
  CallNode *fast_call;
  if (is_primitive) {
    // Call a traditional memmove.
    Node *scaled_length = _gvn.transform( new (3) MulINode( length, scale ) );

    fast_call = new CallLeafNode(OptoRuntime::primitive_arraycopy_Type(),
                                 CAST_FROM_FN_PTR(address, OptoRuntime::primitive_arraycopy),
                                 "primitive_arraycopy");

    fast_call->set_req( TypeFunc::Parms+0,   src_start);
    fast_call->set_req( TypeFunc::Parms+1,   dest_start);
    fast_call->set_req( TypeFunc::Parms+2,   scaled_length); 
  } else {
    fast_call = new CallLeafNode(OptoRuntime::oop_arraycopy_Type(),
                                 CAST_FROM_FN_PTR(address, OptoRuntime::oop_arraycopy),
                                 "oop_arraycopy");
    fast_call->set_req( TypeFunc::Parms+0,   src_start);
    fast_call->set_req( TypeFunc::Parms+1,   dest_start);
    fast_call->set_req( TypeFunc::Parms+2,   length);
  }
  set_predefined_input_for_runtime_call(fast_call);

  Node *fast_call2 = _gvn.transform(fast_call);

  // Set fixed predefined return values
  set_predefined_output_for_runtime_call(fast_call2);

  // Make the merge region, which combines the fast and slow controls.
  RegionNode *result_region = new RegionNode(3);
  _for_igvn.push(result_region);
  result_region->set_req(1,slow_control);
  result_region->set_req(2,control());
  set_control(_gvn.transform(result_region));

  Node *phi = new PhiNode(result_region,Type::ABIO);
  phi->set_req(1,slow_map->in(TypeFunc::I_O));
  phi->set_req(2,i_o()                      );
  _gvn._types.map(phi->_idx,Type::ABIO);
  set_i_o(_gvn.transform(phi));

  phi = new PhiNode(result_region,Type::MEMORY);
  phi->set_req(1,_gvn.transform(slow_map->in(TypeFunc::Memory)));
  phi->set_req(2,all_memory()                  );
  _gvn._types.map(phi->_idx,Type::MEMORY);
  set_all_memory(_gvn.transform(phi));

  return true;
}

