/*
 * @(#)locknode.cpp	1.28 01/02/09 14:46:24
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_locknode.cpp.incl"

//=============================================================================
const RegMask &BoxLockNode::in_RegMask(uint i) const {
  return _inmask;
}

const RegMask &BoxLockNode::out_RegMask() const { 
  return *Matcher::idealreg2regmask[Op_RegP];
}

uint BoxLockNode::size_of() const { return sizeof(*this); }

BoxLockNode::BoxLockNode( int lock ) : Node( Compile::current()->_root ), _locknum(lock) {
  OptoReg::Name reg = SharedInfo::stack2reg(_locknum * Compile::current()->sync_stack_slots());
  _inmask.Insert(reg);
}

//------------------------------cmp--------------------------------------------
uint BoxLockNode::cmp( const Node &n ) const {
  const BoxLockNode &bn = (const BoxLockNode &)n;
  return bn._locknum == _locknum;
}

OptoReg::Name BoxLockNode::stack_slot(Node* box_node) {
  // Chase down the BoxNode 
  while (!box_node->is_BoxLock()) {
    //    if (box_node->is_SpillCopy()) {
    //      MachNode *m = box_node->in(1)->is_Mach();
    //      if (m && m->ideal_Opcode() == Op_StoreP) {
    //        box_node = m->in(m->operand_index(2));
    //        continue;
    //      }
    //    }
    assert(box_node->is_SpillCopy() || box_node->is_Phi(), "Bad spill of Lock.");
    box_node = box_node->in(1);
  }
  return box_node->in_RegMask(0).find_first_elem();
}

//=============================================================================
//------------------------------do_monitor_enter-------------------------------
void Parse::do_monitor_enter( int bci ) {
  
  // Null check; get casted pointer.
  Node *obj = do_null_check( peek(), bci, T_OBJECT );
  // Check for locking null object
  if( !_map ) return;

  // the monitor object is not part of debug info expression stack
  pop(); 

  // Insert a FastLockNode which takes as arguments the current thread pointer,
  // the obj pointer & the address of the stack slot pair used for the lock.
  shared_lock(obj, bci);
}

//------------------------------do_monitor_exit--------------------------------
void Parse::do_monitor_exit( int bci ) {
  pop();                        // Pop oop to unlock
  // Because monitors are guarenteed paired (else we bail out), we know
  // the matching Lock for this Unlock.  Hence we know there is no need
  // for a null check on Unlock.
  shared_unlock( _map->peek_monitor_box(), _map->peek_monitor_obj(), bci );
} 


//=============================================================================

uint FastLockNode::size_of() const { return sizeof(*this); }

uint FastLockNode::cmp( const Node &n ) const { 
  FastLockNode& arg = (FastLockNode &) n;
  return ((arg._is_speclock == _is_speclock) 
      && (arg._maybe_speclock == _maybe_speclock)); 
}

//------------------------------Ideal------------------------------------------
Node *FastLockNode::Ideal(PhaseGVN *phase,PhaseDefUse *du){
  if (!_maybe_speclock) {
    return NULL;
  }

  Node *obj = in(1);
  const Type *tobj = phase->type(obj);
  if (tobj == NULL) {
    return NULL;
  }
  
  const TypeOopPtr *toop = tobj->isa_oopptr();
  
  if( toop == NULL ) {
    return NULL;
  }

  // Get the object class
  ciKlass* klass = toop->klass();
  assert( klass->is_loaded(), "Only expecting loaded classes");
  if (PrintSpecLock) {  
    tty->print(" Lock ideal: %0x ", _idx); 
    NOT_PRODUCT(toop->dump();)
    tty->cr(); 
  }
  
  // Modify flags based on class
  if (klass->always_speculatively_locked()) {
    _maybe_speclock = false;
    _is_speclock = true;
    if (PrintSpecLock) { tty->print_cr("     changing to is_speclock"); }
    return this;
  } else if (klass->never_speculatively_locked()) {
    _maybe_speclock = false;
    _is_speclock = false;
    if (PrintSpecLock) { tty->print_cr("    changing to not speclocked"); }
    return this;
  } 
 
  assert(!(_maybe_speclock && _is_speclock), "Must specify whether or not speclocked");
  
  return NULL;
}

//=============================================================================

uint FastUnlockNode::size_of() const { return sizeof(*this); }

uint FastUnlockNode::cmp( const Node &n ) const { 
  FastUnlockNode& arg = (FastUnlockNode &) n;
 return ((arg._is_speclock == _is_speclock) 
      && (arg._maybe_speclock == _maybe_speclock));  
}

//------------------------------Ideal------------------------------------------
Node *FastUnlockNode::Ideal(PhaseGVN *phase,PhaseDefUse *du){
   if (!_maybe_speclock) {
    return NULL;
  }

  Node *obj = in(1);
  const Type *tobj = phase->type(obj);
  if (tobj == NULL) {
    return NULL;
  }
  
  const TypeOopPtr *toop = tobj->isa_oopptr();
  
  if( toop == NULL ) {
    return NULL;
  }

  // Get the object class
  ciKlass* klass = toop->klass();
  assert( klass->is_loaded(), "Only expecting loaded classes");
  if (PrintSpecLock && PrintCompilation) {  
    tty->print(" UnLock ideal: %0x ", _idx); 
    NOT_PRODUCT(toop->dump();)
    tty->cr(); 
  }
  
  // Modify flags based on class
  if (klass->always_speculatively_locked()) {
    _maybe_speclock = false;
    _is_speclock = true;
    if (PrintSpecLock) { tty->print_cr("     changing to is_speclock"); }
    return this;
  } else if (klass->never_speculatively_locked()) {
    _maybe_speclock = false;
    _is_speclock = false;
    if (PrintSpecLock) { tty->print_cr("    changing to not speclocked"); }
    return this;
  } 
 
  assert(!(_maybe_speclock && _is_speclock), "Must specify whether or not speclocked");
  
  return NULL;
}

