/*
 * @(#)locknode.hpp	1.17 01/02/09 14:46:25
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

//------------------------------BoxLockNode------------------------------------
class BoxLockNode : public Node {
public:
  int _locknum;
  RegMask _inmask;

  BoxLockNode( int lock );
  virtual int Opcode() const;
  virtual void emit(CodeBuffer &cbuf, PhaseChaitin *pc_) const;
  virtual uint size(PhaseChaitin *pc_) const;
  virtual bool rematerialize() const { return true; }
  virtual const RegMask &in_RegMask(uint) const;
  virtual const RegMask &out_RegMask() const;
  virtual uint size_of() const;
  virtual uint hash() const { return Node::hash() + _locknum; }
  virtual uint cmp( const Node &n ) const;
  virtual const class Type *bottom_type() const { return TypeRawPtr::BOTTOM; }
  virtual uint ideal_reg() const { return Op_RegP; }
  virtual const BoxLockNode *is_BoxLock() const { return this; }

  static OptoReg::Name stack_slot(Node* box_node);

#ifndef PRODUCT
  virtual void format( PhaseChaitin * ) const;
  virtual void dump_spec() const { tty->print("  Lock %d",_locknum); }
#endif
};

//------------------------------FastLockNode-----------------------------------
class FastLockNode: public Node {
private:
  bool _is_speclock;
  bool _maybe_speclock;
  
  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const;
public:
  FastLockNode(Node *ctrl, Node *oop, Node *box) 
    : Node(ctrl,oop,box) {
    _is_speclock = false;
    _maybe_speclock = false;
  }

  FastLockNode(Node *ctrl, Node *oop, Node *box, bool is_speclock, bool maybe_speclock) 
    : Node(ctrl,oop,box) {
    _is_speclock = is_speclock;
    _maybe_speclock = maybe_speclock;
  }
  virtual int Opcode() const;
  virtual const FastLockNode *is_FastLock() const { return this; }
  virtual const bool  is_SpecLock() const { return _is_speclock; }
  virtual const bool  maybe_SpecLock() const { return _maybe_speclock; }
  Node* obj_node() const { return in(1); }
  Node* box_node() const { return in(2); }
  virtual uint ideal_reg() const { return Op_RegFlags; }
  virtual Node* Ideal(PhaseGVN *phase, PhaseDefUse *du);

};


//------------------------------FastUnlockNode---------------------------------
class FastUnlockNode: public Node {
private:
  bool _maybe_speclock;
  bool _is_speclock;

  virtual uint cmp( const Node &n ) const;
  virtual uint size_of() const;

public:
  FastUnlockNode(Node *ctrl, Node *oop, Node *box) 
    : Node(ctrl,oop,box) {
    _maybe_speclock = false;
    _is_speclock = false;
  }

  FastUnlockNode(Node *ctrl, Node *oop, Node *box, bool is_speclock, bool maybe_speclock) 
    : Node(ctrl,oop,box) {
    _maybe_speclock = maybe_speclock;
    _is_speclock = is_speclock;
  }

  virtual int   Opcode() const;
  virtual uint  ideal_reg() const { return Op_RegFlags; }
  virtual const bool  maybe_SpecLock() const { return _maybe_speclock; }
  virtual const bool  is_SpecLock() const { return _is_speclock; }
  virtual Node* Ideal(PhaseGVN *phase, PhaseDefUse *du);
};


//------------------------------LoadThreadIDNode--------------------------
// Used for loading the thread ID from the register
class LoadThreadIDNode : public Node {
public:
  LoadThreadIDNode() : Node ((Node*)Compile::current()->_root) {/*set_req(0,Compile::current()->_root);*/}
  virtual int Opcode() const;
  virtual const class Type *bottom_type() const { return TypeInt::INT; }
  virtual uint ideal_reg() const { return Op_RegI; }
};

