// @(#)loopopts.cpp	1.147 01/02/14 09:21:00
//
//  Copyright 1997-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_loopopts.cpp.incl"

//=============================================================================
//------------------------------split_thru_phi---------------------------------
// Split Node 'n' through merge point if there is enough win.
Node *PhaseIdealLoop::split_thru_phi( Node *n, Node *region, int policy ) {
  int wins = 0;
  assert( !n->is_CFG(), "" );
  assert( region->is_Region(), "" );
  Node *phi = new PhiNode( region, n->bottom_type() );
  uint old_unique = C->_unique;
  for( uint i = 1; i < region->req(); i++ ) {
    Node *x;
    if( region->in(i) == C->_top_node ) {
      x = C->_top_node;		// Dead path?  Use a dead data op
    } else {
      x = n->clone();		// Else clone up the data op
      // Alter data node to use pre-phi inputs
      if( n->in(0) == region )
	x->set_req( 0, region->in(i) );
      for( uint j = 1; j < n->req(); j++ ) {
	Node *in = n->in(j);
	if( in->is_Phi() && in->in(0) == region ) 
	  x->set_req( j, in->in(i) ); // Use pre-Phi input for the clone
      }
    }
    // Check for a 'win' on some paths
    const Type *t = x->Value(&_igvn);

    // Make sure that the type array is big enough for
    // our new node, even though we may throw the node away.
    // CheckCastPP::Identity uses phase->type(this) and
    // so the type array has to be big enough.
    _igvn._types.map(x->_idx,t);

    if( t->singleton() ) {
      wins++;
      x = ((PhaseGVN&)_igvn).makecon(t);
    } else {
      Node *y = x->Identity(&_igvn);
      if( y != x ) {
        wins++;
        x = y;
      } else {
        y = _igvn.hash_find(x);
        if( y ) {
          wins++;
          x = y;
        } else {
          // Else x is a new node we are keeping
          _igvn.register_new_node_with_optimizer(x);
        }
      }
    }
    phi->set_req( i, x );
  }
  // Too few wins?
  if( wins <= policy ) {
    for( uint i = 1; i < phi->req(); i++ )
      if( _igvn._du.cnt(phi->in(i)) == 0 )
        _igvn.remove_dead_node(phi->in(i));
    return NULL;
  }

  // Record Phi 
  register_new_node( phi, region );

  for( uint i2 = 1; i2 < phi->req(); i2++ ) {
    Node *x = phi->in(i2);
    // If we commoned up the cloned 'x' with another existing Node, 
    // the existing Node picks up a new use.  We need to make the
    // existing Node occur higher up so it dominates its uses.
    Node *old_ctrl;
    IdealLoopTree *old_loop;

    // The occasional new node
    if( x->_idx >= old_unique ) {   // Found a new, unplaced node?  
      old_ctrl = x->is_Con() ? C->_root : NULL;
      old_loop = NULL;              // Not in any prior loop
    } else {
      old_ctrl = get_ctrl( x->_idx );
      old_loop = get_loop( old_ctrl );  // Get prior loop
    }
    // New late point must dominate new use
    Node *new_ctrl = dom_lca( old_ctrl, region->in(i2) );
    // Set new location
    set_ctrl( x->_idx, new_ctrl );
    IdealLoopTree *new_loop = get_loop( new_ctrl );
    // If changing loop bodies, see if we need to collect into new body
    if( old_loop != new_loop ) {
      if( old_loop && !old_loop->_child ) 
        old_loop->_body.yank(x);
      if( !new_loop->_child )
        new_loop->_body.push(x);  // Collect body info
    }
  }

  return phi;
}

//------------------------------dominated_by------------------------------------
// Replace the dominated test with an obvious true or false.  Place it on the
// IGVN worklist for later cleanup.  Move control-dependent data Nodes on the
// live path up to the dominating control.
void PhaseIdealLoop::dominated_by( Node *prevdom, Node *iff ) {
#ifndef PRODUCT
  if( VerifyLoopOptimizations && PrintOpto ) tty->print_cr("dominating test");
#endif
 

  // prevdom is the dominating projection of the dominating test.
  assert( iff->is_If(), "" );
  int pop = prevdom->Opcode();
  assert( pop == Op_IfFalse || Op_IfTrue, "" );
  // 'con' is set to true or false to kill the dominated test.
  Node *con = _igvn.makecon(pop == Op_IfTrue ? TypeInt::ONE : TypeInt::ZERO);
  set_ctrl(con->_idx, C->_root); // Constant gets a new use
  // Hack the dominated test
  _igvn.hash_delete(iff);
  iff->set_req(1, con, &_igvn._du);
  _igvn._worklist.push(iff);

  // If I dont have a reachable TRUE and FALSE path following the IfNode then
  // I can assume this path reaches an infinite loop.  In this case it's not
  // important to optimize the data Nodes - either the whole compilation will
  // be tossed or this path (and all data Nodes) will go dead.
  if( _igvn._du.cnt(iff) != 2 ) return;

  // Make control-dependent data Nodes on the live path (path that will remain
  // once the dominated IF is removed) become control-dependent on the 
  // dominating projection.
  Node *dp = _igvn._du.out(iff)[0];
  if( dp->Opcode() != pop ) {
    dp = _igvn._du.out(iff)[1];
    assert( dp->Opcode() == pop, "found the dominated projection" );
  }
  IdealLoopTree *old_loop = get_loop(dp);

  uint    max = _igvn._du.cnt( dp );
  Node **outp = _igvn._du.out( dp );
  for( uint i = 0; i < max; i++ ) {
    Node *cd = outp[i];		// Control-dependent node
    if( cd->depends_only_on_test() ) {
      assert( cd->in(0) == dp, "" );
      _igvn.hash_delete( cd );
      cd->set_req( 0, prevdom, &_igvn._du );
      set_early_ctrl( cd );
      _igvn._worklist.push(cd);
      IdealLoopTree *new_loop = get_loop(get_ctrl(cd->_idx));
      if( old_loop != new_loop ) {
	if( !old_loop->_child ) old_loop->_body.yank(cd);
	if( !new_loop->_child ) new_loop->_body.push(cd);
      }
      i--;
      max--;
      assert( max == _igvn._du.cnt(dp), "funny DU mod" );
      outp = _igvn._du.out(dp);	// Reload outp
    }
  }
}

//------------------------------has_local_phi_input----------------------------
// Return TRUE if 'n' has Phi inputs from its local block and no other
// block-local inputs (all non-local-phi inputs come from earlier blocks)
Node *PhaseIdealLoop::has_local_phi_input( Node *n ) {
  Node *n_ctrl = get_ctrl(n->_idx);
  // See if some inputs come from a Phi in this block, or from before
  // this block.
  uint i;
  for( i = 1; i < n->req(); i++ ) {
    Node *phi = n->in(i)->is_Phi();
    if( phi && phi->in(0) == n_ctrl )
      break;
  }
  if( i >= n->req() )
    return NULL;		// No Phi inputs; nowhere to clone thru

  // Check for inputs created between 'n' and the Phi input.  These
  // must split as well; they have already been given the chance
  // (courtesy of a post-order visit) and since they did not we must
  // recover the 'cost' of splitting them by being very profitable
  // when splitting 'n'.  Since this is unlikely we simply give up.
  for( i = 1; i < n->req(); i++ ) {
    Node *m = n->in(i);
    if( get_ctrl( m->_idx ) == n_ctrl && !m->is_Phi() ) {
      // We allow the special case of AddP's with no local inputs.
      // This allows us to split-up address expressions.
      if( m->is_AddP() && 
	  get_ctrl( m->in(2)->_idx ) != n_ctrl &&
	  get_ctrl( m->in(3)->_idx ) != n_ctrl ) {
	// Move the AddP up to dominating point
	set_ctrl( m->_idx, idom(n_ctrl->_idx) );
	continue;
      }
      return NULL;
    }
  }

  return n_ctrl;
}

//------------------------------remix_address_expressions----------------------
// Rework addressing expressions to get the most loop-invariant stuff 
// moved out.  We'd like to do all associative operators, but it's especially
// important (common) to do address expressions.
Node *PhaseIdealLoop::remix_address_expressions( Node *n ) {
  if( !has_ctrl(n->_idx) ) return NULL;
  Node *n_ctrl = get_ctrl(n->_idx);
  IdealLoopTree *n_loop = get_loop(n_ctrl ); 

  // See if 'n' mixes loop-varying and loop-invariant inputs and
  // itself is loop-varying.

  // Only interested in binary ops (and AddP)
  if( n->req() < 3 || n->req() > 4 ) return NULL;

  Node *n1_ctrl = get_ctrl( n->in(                    1)->_idx );
  Node *n2_ctrl = get_ctrl( n->in(                    2)->_idx );
  Node *n3_ctrl = get_ctrl( n->in(n->req() == 3 ? 2 : 3)->_idx );
  IdealLoopTree *n1_loop = get_loop( n1_ctrl ); 
  IdealLoopTree *n2_loop = get_loop( n2_ctrl );
  IdealLoopTree *n3_loop = get_loop( n3_ctrl ); 

  // Does one of my inputs spin in a tighter loop than self?
  if( (n_loop->is_member( n1_loop ) && n_loop != n1_loop) ||
      (n_loop->is_member( n2_loop ) && n_loop != n2_loop) ||
      (n_loop->is_member( n3_loop ) && n_loop != n3_loop) )
    return NULL;                // Leave well enough alone

  // Is at least one of my inputs loop-invariant?
  if( n1_loop == n_loop &&
      n2_loop == n_loop &&
      n3_loop == n_loop )
    return NULL;                // No loop-invariant inputs


  int n_op = n->Opcode();

  // Replace expressions like ((V+I) << 2) with (V<<2 + I<<2).
  if( n_op == Op_LShiftI ) {
    // Scale is loop invariant
    Node *scale = n->in(2);
    Node *scale_ctrl = get_ctrl(scale->_idx);
    IdealLoopTree *scale_loop = get_loop(scale_ctrl ); 
    if( n_loop == scale_loop || !scale_loop->is_member( n_loop ) )
      return NULL;
    // Add must vary with loop (else shift would be loop-invariant)
    Node *add = n->in(1);
    if( add->Opcode() != Op_AddI ) return NULL;
    Node *add_ctrl = get_ctrl(add->_idx);
    IdealLoopTree *add_loop = get_loop(add_ctrl ); 
    assert( n_loop == add_loop, "" );
    // See if one add input is loop invariant
    Node *add_var = add->in(1);
    Node *add_var_ctrl = get_ctrl(add_var->_idx);
    IdealLoopTree *add_var_loop = get_loop(add_var_ctrl ); 
    Node *add_invar = add->in(2);
    Node *add_invar_ctrl = get_ctrl(add_invar->_idx);
    IdealLoopTree *add_invar_loop = get_loop(add_invar_ctrl ); 
    if( add_var_loop == n_loop ) {
    } else if( add_invar_loop == n_loop ) {
      // Swap to find the invariant part
      add_invar = add_var;
      add_invar_ctrl = add_var_ctrl;
      add_invar_loop = add_var_loop;
      add_var = add->in(2);
      Node *add_var_ctrl = get_ctrl(add_var->_idx);
      IdealLoopTree *add_var_loop = get_loop(add_var_ctrl ); 
    } else			// Else neither input is loop invariant
      return NULL;
    if( n_loop == add_invar_loop || !add_invar_loop->is_member( n_loop ) )
      return NULL;              // No invariant part of the add?

    // Yes!  Reshape address expression!
    Node *inv_scale = new (3) LShiftINode( add_invar, scale );
    register_new_node( inv_scale, add_invar_ctrl );
    Node *var_scale = new (3) LShiftINode( add_var, scale );
    register_new_node( var_scale, n_ctrl );
    Node *var_add = new (3) AddINode( var_scale, inv_scale );
    register_new_node( var_add, n_ctrl );
    _igvn.hash_delete( n );
    _igvn.subsume_node( n, var_add );
    return var_add;
  }

  // Replace (I+V) with (V+I)
  if( n_op == Op_AddI ) {
    if( n2_loop == n_loop ) {
      assert( n1_loop != n_loop, "" );
      Node *tmp = n->in(1);
      n->set_req( 1, n->in(2) );
      n->set_req( 2, tmp );
    }
  }

  // Replace ((I1 +p V) +p I2) with ((I1 +p I2) +p V),
  // but not if I2 is a constant.
  if( n_op == Op_AddP ) {
    if( n2_loop == n_loop && n3_loop != n_loop ) {
      if( n->in(2)->Opcode() == Op_AddP && !n->in(3)->is_Con() ) {
        Node *n22_ctrl = get_ctrl( n->in(2)->in(2)->_idx );
        Node *n23_ctrl = get_ctrl( n->in(2)->in(3)->_idx );
        IdealLoopTree *n22loop = get_loop( n22_ctrl ); 
        IdealLoopTree *n23_loop = get_loop( n23_ctrl );
        if( n22loop != n_loop && n22loop->is_member(n_loop) && 
            n23_loop == n_loop ) {
          Node *add1 = new (4) AddPNode( n->in(1), n->in(2)->in(2), n->in(3) );
          register_new_node( add1, n22_ctrl );
          Node *add2 = new (4) AddPNode( n->in(1), add1, n->in(2)->in(3) );
          register_new_node( add2, n_ctrl );
          _igvn.hash_delete( n );
          _igvn.subsume_node( n, add2 );
          return add2;
        }
      }
    }

    // Replace (I1 +p (I2 + V)) with ((I1 +p I2) +p V)
    if( n2_loop != n_loop && n3_loop == n_loop ) {
      if( n->in(3)->Opcode() == Op_AddI ) {
        Node *V = n->in(3)->in(1);
        Node *I = n->in(3)->in(2);
        if( get_loop(get_ctrl(V->_idx)) == n_loop ) {
        } else {
          Node *tmp = V; V = I; I = tmp;
        }
        if( get_loop(get_ctrl(I->_idx)) != n_loop ) {
	  Node *add1 = new (4) AddPNode( n->in(1), n->in(2), I );
	  // Stuff new AddP in the loop preheader
	  register_new_node( add1, n_loop->_head->in(LoopNode::EntryControl) );
	  Node *add2 = new (4) AddPNode( n->in(1), add1, V );
	  register_new_node( add2, n_ctrl );
	  _igvn.hash_delete( n );
	  _igvn.subsume_node( n, add2 );
	  return add2;
	}
      }
    }
  }

  return NULL;
}

//------------------------------conditional_move-------------------------------
// Attempt to replace a Phi with a conditional move.  We have some pretty
// strict profitability requirements.  All Phis at the merge point must
// be converted, so we can remove the control flow.  We need to limit the
// number of c-moves to a small handful.  All code that was in the side-arms
// of the CFG diamond is now speculatively executed.  This code has to be 
// "cheap enough".  We are pretty much limited to CFG diamonds that merge
// 1 or 2 items with a total of 1 or 2 ops executed speculatively.
Node *PhaseIdealLoop::conditional_move( Node *region ) {

  assert( region->is_Region(), "sanity check" );
  if( region->req() != 3 ) return NULL;

  // Check for CFG diamond
  Node *lp = region->in(1);
  Node *rp = region->in(2);
  if( !lp || !rp ) return NULL;
  if( lp->in(0) != rp->in(0) ) return NULL;
  if( !lp->in(0) ) return NULL;
  IfNode *iff = lp->in(0)->is_If();
  if( !iff ) return NULL;

  // Check for highly predictable branch.  No point in CMOV'ing if
  // we are going to predict accurately all the time.
  if( iff->_prob < 0.001 ||
      iff->_prob > 0.999 )
    return NULL;

  // Check for ops pinned in an arm of the diamond.
  // Can't remove the control flow in this case
  PhaseDefUse &du = _igvn._du;
  if( du.cnt(lp) > 1 ) return NULL;
  if( du.cnt(rp) > 1 ) return NULL;

  // Check profitability
  int cost = 0;
  uint   max  = du.cnt(region);
  Node **outp = du.out(region);
  for( uint i = 0; i < max; i++ ) {
    PhiNode *phi = outp[i]->is_Phi();
    if( !phi ) continue;	// Ignore other control edges,etc
    switch( phi->_type->_base ) {
    case Type::Long:
      cost++;			// Probably encodes as 2 CMOV's
    case Type::Int:		// These all CMOV fine
    case Type::FloatBot:
    case Type::DoubleBot:
    case Type::KlassPtr:
    case Type::AnyPtr:
    case Type::RawPtr:
    case Type::OopPtr:
    case Type::AryPtr:
      cost++;
      break;
    default:
      return NULL;		// In particular, can't do memory or I/O
    }
    // Add in cost any speculative ops
    for( uint j = 1; j < region->req(); j++ ) {
      Node *proj = region->in(j);
      Node *inp = phi->in(j);
      if( get_ctrl(inp->_idx) == proj ) { // Found local op
	cost++;
	// Check for a chain of dependent ops; these will all become
	// speculative in a CMOV.
	for( uint k = 1; k < inp->req(); k++ ) 
	  if( get_ctrl( inp->in(k)->_idx ) == proj )
	    return NULL;	// Too much speculative goo
      }
    }
    // See if the Phi is used by a Cmp.  This will likely Split-If, a
    // higher-payoff operation.
    uint   max2  = du.cnt(phi);
    Node **outp2 = du.out(phi);
    for( uint k = 0; k < max2; k++ ) {
      Node *use = outp2[k];
      if( use->is_Cmp() )
	return NULL;
    }
  }
  if( cost >= ConditionalMoveLimit ) return NULL; // Too much goo

  // --------------
  // Now replace all Phis with CMOV's
  Node *cmov_ctrl = iff;
  uint flip = (lp->Opcode() == Op_IfTrue);
  while( 1 ) {
    uint   max  = du.cnt(region);
    Node **outp = du.out(region);
    uint i;
    for( i = 0; i < max; i++ ) 
      if( outp[i]->is_Phi() ) 
	break;
    if( i >= max ) break;
    Node *phi = outp[i];
#ifndef PRODUCT
    if( PrintOpto && VerifyLoopOptimizations ) tty->print_cr("CMOV");
#endif
    // Move speculative ops
    for( uint j = 1; j < region->req(); j++ ) {
      Node *proj = region->in(j);
      Node *inp = phi->in(j);
      if( get_ctrl(inp->_idx) == proj ) { // Found local op
#ifndef PRODUCT
	if( PrintOpto && VerifyLoopOptimizations ) {
	  tty->print("  speculate: ");
	  inp->dump();
	}
#endif
	set_ctrl( inp->_idx, cmov_ctrl );
      }
    }
    Node *cmov = CMoveNode::make( cmov_ctrl->in(0), iff->in(1), phi->in(1+flip), phi->in(2-flip), _igvn.type(phi) );
    register_new_node( cmov, cmov_ctrl );
    _igvn.hash_delete(phi);
    _igvn.subsume_node( phi, cmov );
#ifndef PRODUCT
    if( VerifyLoopOptimizations ) verify();
#endif
  }

  // The useless CFG diamond will fold up later; see the optimization in
  // RegionNode::Ideal.
  _igvn._worklist.push(region);

  return iff->in(1);
}

//------------------------------split_if_with_blocks_pre-----------------------
// Do the real work in a non-recursive function.  Data nodes want to be
// cloned in the pre-order so they can feed each other nicely.
Node *PhaseIdealLoop::split_if_with_blocks_pre( Node *n ) {
  // Cloning these guys is unlikely to win
  int n_op = n->Opcode();
  if( n_op == Op_MergeMem ) return n;
  if( n_op == Op_FastLock ) return n;
  if( n_op == Op_FastUnlock ) return n;
  if( n_op == Op_Proj ) return n;
  // These are part of the 'split-if' transform
  if( n_op == Op_CmpI || n_op == Op_CmpU || n_op == Op_CmpP ) return n;
  // Do not clone-up CmpL or CmpFXXX variations, as these are always
  // followed by a CmpI
  if( n->is_Cmp() ) return n;
  // Attempt to use a conditional move instead of a phi/branch
  if( ConditionalMoveLimit > 0 && n_op == Op_Region ) {
    Node *cmov = conditional_move( n );
    if( cmov ) return cmov;
  }
  if( n->is_CFG() ) return n;
  if( n_op == Op_Opaque1 ||     // Opaque nodes cannot be mod'd
      n_op == Op_Opaque2 ) {
    if( !C->_major_progress )   // If chance of no more loop opts...
      _igvn._worklist.push(n);  // maybe we'll remove them
    return n;
  }

  Node *n_ctrl = get_ctrl(n->_idx);
  if( !n_ctrl ) return n;	// Dead node

  // Attempt to remix address expressions for loop invariants
  Node *m = remix_address_expressions( n );
  if( m ) return m;

  // Determine if the Node has inputs from some local Phi.
  // Returns the block to clone thru.
  Node *n_blk = has_local_phi_input( n );
  if( !n_blk ) return n;
  // Do not clone the trip counter through on a CountedLoop
  // (messes up the canonical shape).
  CountedLoopNode *cl = n_blk->is_CountedLoop();
  if( cl && n->Opcode() == Op_AddI ) return n;

  // Check for having no control input; not pinned.  Allow 
  // dominating control.
  if( n->in(0) ) {
    Node *dom = idom(n_blk->_idx);
    if( dom_lca( n->in(0), dom ) != n->in(0) )
      return n;
  }
  // Policy: when is it profitable.  You must get more wins than
  // policy before it is considered profitable.  Policy is usually 0,
  // so 1 win is considered profitable.  Big merges will require big
  // cloning, so get a larger policy.
  int policy = n_blk->req() >> 2;

  // Split 'n' through the merge point if it is profitable
  Node *phi = split_thru_phi( n, n_blk, policy );
  if( !phi ) return n;

  // Found a Phi to split thru!
  // Replace 'n' with the new phi
  _igvn.hash_delete(n);
  _igvn.subsume_node( n, phi );
  // Moved a load around the loop, 'en-registering' something.
  if( n_blk->Opcode() == Op_Loop && n->is_Load() && 
      !phi->in(LoopNode::LoopBackControl)->is_Load() )
    C->_major_progress = true;

  return phi;
}

//------------------------------merge_point_too_heavy--------------------------
static bool merge_point_too_heavy(Compile* C, Node* region, PhaseDefUse &du) {
  // Bail out if the region and its phis have too many users.
  uint   max  = du.cnt(region);
  Node **outp = du.out(region);
  int weight = 0;
  for (uint i = 0; i < max; i++) {
    weight += du.cnt(outp[i]);
  }
  int nodes_left = 65000 - C->_unique;
  if (weight * 8 > nodes_left) {
#ifndef PRODUCT
    if (PrintOpto)
      tty->print_cr("*** Split-if bails out:  %d nodes, region weight %d", C->_unique, weight);
#endif
    return true;
  } else {
    return false;
  }
}

//------------------------------place_near_use---------------------------------
// Place some computation next to use but not inside inner loops.
// For inner loop uses move it to the preheader area.
Node *PhaseIdealLoop::place_near_use( Node *useblock ) const {
  IdealLoopTree *u_loop = get_loop( useblock );
  return (u_loop->_irreducible || u_loop->_child) 
    ? useblock
    : u_loop->_head->in(LoopNode::EntryControl);
}


//------------------------------split_if_with_blocks_post----------------------
// Do the real work in a non-recursive function.  CFG hackery wants to be
// in the post-order, so it can dirty the I-DOM info and not use the dirtied
// info.
void PhaseIdealLoop::split_if_with_blocks_post( Node *n ) {

  // Check for an IF ready to split; one that has its 
  // condition codes input coming from a Phi at the block start.
  int n_op = n->Opcode();

  // Cloning Cmp through Phi's involves the split-if transform.
  if( (n_op == Op_CmpI || n_op == Op_CmpU || n_op == Op_CmpP) ) {
    if( C->_unique > 35000 ) return; // Method too big

    Node *n_ctrl = get_ctrl(n->_idx);
    // Determine if the Node has inputs from some local Phi.
    // Returns the block to clone thru.
    Node *n_blk = has_local_phi_input( n );
    if( n_blk != n_ctrl ) return;

    if( merge_point_too_heavy(C, n_ctrl, _igvn._du) )
      return;

    if( _igvn._du.cnt(n) != 1 ) return; // Multiple bool's from 1 compare?
    Node *bol = _igvn._du.out(n)[0];
    assert( bol->is_Bool(), "" );
    if( _igvn._du.cnt(bol) != 1 ) return;// Multiple branches from 1 compare?
    Node *iff = _igvn._du.out(bol)[0];

    // Check some safety conditions
    if( iff->in(0) ) {		// Classic split-if?
      if( iff->in(0) != n_ctrl ) return; // Compare must be in same blk as if
    } else {			// Trying to split-up a CMOVE
      if( get_ctrl(iff->in(2)->_idx) == n_ctrl ||
	  get_ctrl(iff->in(3)->_idx) == n_ctrl )
	return;			// Inputs not yet split-up
    }

    // Do not do 'split-if' if some paths are dead.  First do dead code
    // elimination and then see if its still profitable.
    for( uint i = 1; i < n_ctrl->req(); i++ )
      if( n_ctrl->in(i) == C->_top_node )
        return;

    // When is split-if profitable?  Every 'win' on means some control flow
    // goes dead, so it's almost always a win.
    int policy = 0;
    // If trying to do a 'Split-If' at the loop head, it is only
    // profitable if the cmp folds up on BOTH paths.  Otherwise we
    // risk peeling a loop forever.
    // CNC - Disabled for now.
    if( n_ctrl->is_Loop() )
      policy = 999;             // Policy requires BOTH paths to win  

    // Split compare 'n' through the merge point if it is profitable
    Node *phi = split_thru_phi( n, n_ctrl, policy );
    if( !phi ) return;

    // Found a Phi to split thru!
    // Replace 'n' with the new phi
    _igvn.hash_delete(n);
    _igvn.subsume_node( n, phi );

    // Now split the bool up thru the phi
    Node *bolphi = split_thru_phi( bol, n_ctrl, -1 );
    _igvn.hash_delete(bol);
    _igvn.subsume_node( bol, bolphi );
    assert( iff->in(1) == bolphi, "" );
    if( bolphi->Value(&_igvn)->singleton() ) 
      return;

    // Conditional-move?  Must split up now
    if( !iff->is_If() ) {
      Node *cmovphi = split_thru_phi( iff, n_ctrl, -1 );      
      _igvn.hash_delete(iff);
      _igvn.subsume_node( iff, cmovphi );
      return;
    }

    // Now split the IF
    do_split_if( iff );
    return;
  }

  
  // CheckTypeNode's have 2 inputs, both must feed from Phis at the
  // same merge point or before.
  if( n_op == Op_CheckType && n->in(0)->is_Region() ) {
    Node *n0 = n->in(0);
    Node *n1 = n->in(1);
    Node *n2 = n->in(2);
    // Check for input from local Phi or before
    if( !(n1->is_Phi() && n1->in(0) == n0) && get_ctrl(n1->_idx) == n0 )
      return;			// Local input that was not cloned-thru
    if( !(n2->is_Phi() && n2->in(0) == n0) && get_ctrl(n2->_idx) == n0 )
      return;			// Local input that was not cloned-thru
    if( merge_point_too_heavy(C, n0, _igvn._du) )
      return;

    // Now check for profitability.  See if there is a dominating CheckType.
    for( uint i = 1; i < n0->req(); i++ ) {
      Node *n0i = n0->in(i);
      if( n0i->Opcode() == Op_IfFalse ) {
        Node *n0i0 = n0i->in(0);
        // One arm of the Region has a checktype-false with identical inputs
        if( n0i0->Opcode() == Op_CheckType &&
            n0i0->in(1) == n1 &&
            n0i0->in(2) == n2 ) {
          do_split_if( n );
          return;
        }
      }
    }
  }

  // Check for an IF being dominated by another IF same test
  if( n_op == Op_If ) {
    Node *bol = n->in(1);
    uint   max  = _igvn._du.cnt(bol);
    Node **outp = _igvn._du.out(bol);
    // Check for same test used more than once?
    if( n_op == Op_If && max > 1 && bol->is_Bool() ) { 
      // Search up IDOMs to see if this IF is dominated.
      Node *cutoff = get_ctrl(bol->_idx);

      // Now search up IDOMs till cutoff, looking for a dominating test
      Node *prevdom = n;
      Node *dom = idom( prevdom->_idx );
      while( dom != cutoff ) {
        if( dom->req() > 1 && dom->in(1) == bol && prevdom->in(0) == dom ) {
          // Replace the dominated test with an obvious true or false.
          // Place it on the IGVN worklist for later cleanup.
          C->_major_progress = true;
	  dominated_by( prevdom, n );
#ifndef PRODUCT
	  if( VerifyLoopOptimizations ) verify();
#endif
          return;
        }
        prevdom = dom;
        dom = idom( prevdom->_idx );
      }
    }
  }

  // See if a shared loop-varying computation has no loop-varying uses.
  // Happens if something is only used for JVM state in uncommon trap exits,
  // like various versions of induction variable+offset.  Clone the 
  // computation per usage to allow it to sink out of the loop.
  if( has_ctrl(n->_idx) && !n->in(0) ) {
    IdealLoopTree *n_loop = get_loop( get_ctrl(n->_idx) );
    if( n_loop != _ltree_root ) {
      uint   max  = _igvn._du.cnt(n);
      Node **outp = _igvn._du.out(n);
      uint i;
      for( i = 0; i < max; i++ ) {
        Node *u = outp[i];
        if( !has_ctrl(u->_idx) ) break; // Found control user
        IdealLoopTree *u_loop = get_loop( get_ctrl(u->_idx) );
        if( u_loop == n_loop ) break; // Found loop-varying use
        if( n_loop->is_member( u_loop ) ) break; // Found use in inner loop
      }
      if( i == max && max > 1 ) { // All uses in outer loops!
        while( _igvn._du.cnt(n) ) { // Clone private computation per use
          Node *u = _igvn._du.out(n)[0];
          _igvn.hash_delete(u);
          _igvn._worklist.push(u);
          Node *x = n->clone(); // Clone computation
	  if( u->is_Phi() ) {
	    // Replace all uses of normal nodes.  Replace Phi uses 
	    // individually, so the seperate Nodes can sink down
	    // different paths.
	    uint j = 1;
	    while( u->in(j) != n ) j++;
	    // x goes next to Phi input path
	    register_new_node( x, place_near_use(u->in(0)->in(j)));
	    u->set_req( j, x, &_igvn._du );
	  } else {		// Normal use
	    // Assume 'x' goes next to use but not inside inner loops.
	    // For inner loop uses move 'x' to the preheader area.
	    register_new_node( x, place_near_use(get_ctrl(u->_idx)));
	    for( uint j = 0; j < u->req(); j++ )
	      if( u->in(j) == n ) // Replace all uses
		u->set_req( j, x, &_igvn._du );
          }
          _igvn._worklist.yank(x);
        }
      }
    }
  }

}

//------------------------------split_if_with_blocks---------------------------
// Check for aggressive application of 'split-if' optimization,
// using basic block level info.
void PhaseIdealLoop::split_if_with_blocks( Node *n, VectorSet &visited ) {
  if( !_igvn._du.cnt(n) ) return; // Dead node
  // Standard visitation check
  if( visited.test_set( n->_idx ) ) return;

  // Now do pre-visit work
  n = split_if_with_blocks_pre( n );

  // Visit recursively
  for( uint i = 0; i < _igvn._du.cnt(n); i++ )
    split_if_with_blocks( _igvn._du.out(n)[i], visited );

  // Now do post-visit work
  if( !_igvn._du.cnt(n) ) return; // Dead node
  assert( _nodes[n->_idx], "no dead nodes" );
  if( !SplitIfBlocks ) return;	// Disabled
  split_if_with_blocks_post( n );
}


//=============================================================================
//
//                   C L O N E   A   L O O P   B O D Y
//

//------------------------------clone_iff--------------------------------------
// Passed in a Phi merging (recursively) some nearly equivalent Bool/Cmps.
// "Nearly" because all Nodes have been cloned from the original in the loop,
// but the fall-in edges to the Cmp are different.  Clone bool/Cmp pairs 
// through the Phi recursively, and return a Bool.
BoolNode *PhaseIdealLoop::clone_iff( PhiNode *phi, IdealLoopTree *loop ) {

  // Convert this Phi into a Phi merging Bools
  uint i;
  for( i = 1; i < phi->req(); i++ ) {
    Node *b = phi->in(i);
    PhiNode *pp = b->is_Phi();
    if( pp ) {
      _igvn.hash_delete(phi);
      _igvn._worklist.push(phi);
      phi->set_req( i, clone_iff( pp, loop ), &_igvn._du );
    } else {
      assert( b->is_Bool(), "" );
    }
  }

  Node *sample_bool = phi->in(1);
  Node *sample_cmp  = sample_bool->in(1);

  // Make Phis to merge the Cmp's inputs.
  Node *phi1 = new PhiNode( phi->in(0), Type::BOTTOM );
  Node *phi2 = new PhiNode( phi->in(0), Type::BOTTOM );
  for( i = 1; i < phi->req(); i++ ) {
    phi1->set_req( i, phi->in(i)->in(1)->in(1) );
    phi2->set_req( i, phi->in(i)->in(1)->in(2) );
  }
  // See if these Phis have been made before.
  // Register with optimizer
  Node *hit1 = _igvn.hash_find_insert(phi1);
  if( hit1 ) {			// Hit, toss just made Phi
    phi1 = hit1;		// Use existing phi
  } else {			// Miss
    _igvn.register_new_node_with_optimizer(phi1);
  }
  Node *hit2 = _igvn.hash_find_insert(phi2);
  if( hit2 ) {			// Hit, toss just made Phi
    phi2 = hit2;		// Use existing phi
  } else {			// Miss
    _igvn.register_new_node_with_optimizer(phi2);
  }
  // Register Phis with loop/block info
  set_ctrl( phi1->_idx, phi->in(0) );
  set_ctrl( phi2->_idx, phi->in(0) );
  // Make a new Cmp
  Node *cmp = sample_cmp->clone();
  cmp->set_req( 1, phi1 );
  cmp->set_req( 2, phi2 );
  _igvn.register_new_node_with_optimizer(cmp);
  set_ctrl( cmp->_idx, phi->in(0) );

  // Make a new Bool
  Node *b = sample_bool->clone();
  b->set_req(1,cmp);
  _igvn.register_new_node_with_optimizer(b);
  set_ctrl( b->_idx, phi->in(0) );
  
  assert( b->is_Bool(), "" );
  return (BoolNode*)b;
}

//------------------------------clone_bool-------------------------------------
// Passed in a Phi merging (recursively) some nearly equivalent Bool/Cmps.
// "Nearly" because all Nodes have been cloned from the original in the loop,
// but the fall-in edges to the Cmp are different.  Clone bool/Cmp pairs 
// through the Phi recursively, and return a Bool.
SubNode *PhaseIdealLoop::clone_bool( PhiNode *phi, IdealLoopTree *loop ) {

  // Convert this Phi into a Phi merging Bools
  for( uint i = 1; i < phi->req(); i++ ) {
    Node *b = phi->in(i);
    PhiNode *pp = b->is_Phi();
    if( pp ) {
      _igvn.hash_delete(phi);
      _igvn._worklist.push(phi);
      phi->set_req( i, clone_bool( pp, loop ), &_igvn._du );
    } else {
      assert( b->is_Cmp() || b == C->_top_node, "inputs are all Cmp or TOP" );
    }
  }

  Node *sample_cmp = phi->in(1);

  // Make Phis to merge the Cmp's inputs.
  Node *phi1 = new PhiNode( phi->in(0), Type::BOTTOM );
  Node *phi2 = new PhiNode( phi->in(0), Type::BOTTOM );
  for( uint j = 1; j < phi->req(); j++ ) {
    Node *cmp_top = phi->in(j);	// Inputs are all Cmp or TOP
    phi1->set_req( j, cmp_top->is_Cmp() ? cmp_top->in(1) : cmp_top );
    phi2->set_req( j, cmp_top->is_Cmp() ? cmp_top->in(2) : cmp_top );
  }
  // See if these Phis have been made before.
  // Register with optimizer
  Node *hit1 = _igvn.hash_find_insert(phi1);
  if( hit1 == NULL ) { 
    _igvn.register_new_node_with_optimizer(phi1);  // Register new phi
  } else {                                         // or
    phi1 = hit1;                                   // Use existing phi
  }
  Node *hit2 = _igvn.hash_find_insert(phi2);
  if( hit2 == NULL ) { 
    _igvn.register_new_node_with_optimizer(phi2);  // Register new phi
  } else {                                         // or
    phi2 = hit2;                                   // Use existing phi
  }
  // Register Phis with loop/block info
  set_ctrl( phi1->_idx, phi->in(0) );
  set_ctrl( phi2->_idx, phi->in(0) );
  // Make a new Cmp
  Node *cmp = sample_cmp->clone();
  cmp->set_req( 1, phi1 );
  cmp->set_req( 2, phi2 );
  _igvn.register_new_node_with_optimizer(cmp);
  set_ctrl( cmp->_idx, phi->in(0) );

  assert( cmp->is_Cmp(), "" );
  return (SubNode*)cmp;
}

//------------------------------sink_use---------------------------------------
// If 'use' was in the loop-exit block, it now needs to be sunk
// below the post-loop merge point.
void PhaseIdealLoop::sink_use( Node *use, Node *post_loop ) {
  if( !use->is_CFG() && get_ctrl( use->_idx ) == post_loop->in(2) ) {
    set_ctrl( use->_idx, post_loop );
    PhaseDefUse &du = _igvn._du;
    uint   max  = du.cnt(use);
    Node **outp = du.out(use);
    for( uint j = 0; j < max; j++ )
      sink_use( outp[j], post_loop );
  }
}

//------------------------------clone_loop-------------------------------------
//
//                   C L O N E   A   L O O P   B O D Y
//
// This is the basic building block of the loop optimizations.  It clones an
// entire loop body.  It makes an old_new loop body mapping; with this mapping
// you can find the new-loop equivalent to an old-loop node.  All new-loop 
// nodes are exactly equal to their old-loop counterparts, all edges are the 
// same.  All exits from the old-loop now have a RegionNode that merges the
// equivalent new-loop path.  This is true even for the normal "loop-exit"
// condition.  All uses of loop-invariant old-loop values now come from (one
// or more) Phis that merge their new-loop equivalents.  
//
// This operation leaves the graph in an illegal state: there are two valid 
// control edges coming from the loop pre-header to both loop bodies.  I'll
// definitely have to hack the graph after running this transform.
//
// From this building block I will further edit edges to perform loop peeling
// or loop unrolling or iteration splitting (Range-Check-Elimination), etc.
void PhaseIdealLoop::clone_loop( IdealLoopTree *loop, Node_List &old_new ) {

  // Step 1: Clone the loop body.  Make the old->new mapping.
  uint i;
  for( i = 0; i < loop->_body.size(); i++ ) {
    Node *old = loop->_body.at(i);
    Node *nnn = old->clone();
    old_new.map( old->_idx, nnn );
    _igvn.register_new_node_with_optimizer(nnn);
  }


  // Step 2: Fix the edges in the new body.  If the old input is outside the 
  // loop use it.  If the old input is INside the loop, use the corresponding
  // new node instead.
  PhaseDefUse &du = _igvn._du;
  for( i = 0; i < loop->_body.size(); i++ ) {
    Node *old = loop->_body.at(i);
    Node *nnn = old_new[old->_idx];
    // Fix CFG/Loop controlling the new node
    if( has_ctrl(old->_idx) ) {
      set_ctrl( nnn->_idx, old_new[get_ctrl(old->_idx)->_idx] );
    } else {
      set_loop( nnn->_idx, loop->_parent );
      if( du.cnt(old) ) {
        set_idom( nnn->_idx, old_new[idom(old->_idx)->_idx],
                  dom_depth(old->_idx) );
      }
    }
    // Correct edges to the new node
    for( uint j = 0; j < nnn->req(); j++ ) {
	Node *n = nnn->in(j);
	if( n ) {
	  IdealLoopTree *old_in_loop = get_loop( has_ctrl(n->_idx) ? get_ctrl(n->_idx) : n );
	  if( loop->is_member( old_in_loop ) )
	    nnn->set_req( j, old_new[n->_idx], &_igvn._du );
	}
    }
    _igvn.hash_find_insert(nnn);
  }
  Node *newhead = old_new[loop->_head->_idx];
  set_idom( newhead->_idx, newhead->in(LoopNode::EntryControl), dom_depth(loop->_head->_idx) );

  
  // Step 3: Now fix control uses.  Loop varying control uses have already
  // been fixed up (as part of all input edges in Step 2).  Loop invariant
  // control uses must be either an IfFalse or an IfTrue.  Make a merge
  // point to merge the old and new IfFalse/IfTrue nodes; make the use 
  // refer to this.
  Node_List worklist(Thread::current()->resource_area());
  uint new_counter = C->_unique;
  for( i = 0; i < loop->_body.size(); i++ ) {
    Node* old = loop->_body.at(i);
    if( !old->is_CFG() ) continue;
    Node* nnn = old_new[old->_idx];

    // Copy uses to a worklist, so I can munge the def-use info
    // with impunity.
    uint   max  = du.cnt(old);
    Node **outp = du.out(old);
    for( uint j = 0; j < max; j++ )
      worklist.push(*outp++);
    
    while( worklist.size() ) {	// Visit all uses
      Node *use = worklist.pop();
      if( !_nodes[use->_idx] ) continue; // Ignore dead nodes
      IdealLoopTree *use_loop = get_loop( has_ctrl(use->_idx) ? get_ctrl(use->_idx) : use );
      if( !loop->is_member( use_loop ) && use->is_CFG() ) {
        // Both OLD and USE are CFG nodes here.
        assert( use->is_Proj(), "" );

        // Clone the loop exit control projection
        Node *newuse = use->clone();
        newuse->set_req(0,nnn);
        _igvn.register_new_node_with_optimizer(newuse);
	set_loop( newuse->_idx, use_loop );
        set_idom( newuse->_idx, nnn, dom_depth(nnn->_idx)+1 );

        // We need a Region to merge the exit from the peeled body and the
        // exit from the old loop body.
        RegionNode *r = new RegionNode(3);
        du.grow_nodes(r->_idx);
        // Map the old use to the new merge point
        old_new.map( use->_idx, r );

        // The original user of 'use' uses 'r' instead.
        while( du.cnt(use) ) {
          Node *useuse = du.out(use)[0];
          _igvn.hash_delete(useuse);
          _igvn._worklist.push(useuse);
	  if( useuse->in(0) == use ) {
	    useuse->set_req( 0, r, &du );
	    set_idom( useuse->_idx, r, dom_depth(loop->_head->_idx)+1 );
          }
          for( uint k = 1; k < useuse->req(); k++ )
            if( useuse->in(k) == use )
              useuse->set_req( k, r, &du );
        }

        // Now finish up 'r'       
        r->set_req( 1, newuse );
        r->set_req( 2,    use );
        _igvn.register_new_node_with_optimizer(r);
	set_loop( r->_idx, use_loop );
        set_idom( r->_idx, newuse->in(0), dom_depth(loop->_head->_idx)+1 );
      } // End of if a loop-exit test
    }
  }

  // Step 4: If loop-invariant use is not control, it must be dominated by a
  // loop exit IfFalse/IfTrue.  Find "proper" loop exit.  Make a Region
  // there if needed.  Make a Phi there merging old and new used values.
  Node_List *split_if_set = NULL;
  Node_List *split_bool_set = NULL;
  Node_List *split_cex_set = NULL;
  for( i = 0; i < loop->_body.size(); i++ ) {
    Node* old = loop->_body.at(i);
    Node* nnn = old_new[old->_idx];
    // Copy uses to a worklist, so I can munge the def-use info
    // with impunity.
    uint   max  = du.cnt(old);
    Node **outp = du.out(old);
    for( uint j = 0; j < max; j++ )
      worklist.push(*outp++);

    while( worklist.size() ) {
      Node *use = worklist.pop();
      if( !_nodes[use->_idx] ) continue; // Ignore dead nodes
      if( use->in(0) == C->_top_node ) continue;
      IdealLoopTree *use_loop = get_loop( has_ctrl(use->_idx) ? get_ctrl(use->_idx) : use );
      // Check for data-use outside of loop - at least one of OLD or USE
      // must not be a CFG node.
      if( !loop->is_member( use_loop ) && (!old->is_CFG() || !use->is_CFG())) {

	// If the Data use is an IF, that means we have an IF outside of the
	// loop that is switching on a condition that is set inside of the 
	// loop.  Happens if people set a loop-exit flag; then test the flag
	// in the loop to break the loop, then test is again outside of the
	// loop to determine which way the loop exited.
        if( (use->is_If() && use->Opcode() != Op_CheckType) ||
	    use->is_CMove() ) {
	  // Since this code is highly unlikely, we lazily build the worklist
	  // of such Nodes to go split.
	  if( !split_if_set ) 
	    split_if_set = new Node_List(Thread::current()->resource_area());
	  split_if_set->push(use);
        }
        if( use->is_Bool() ) {
	  if( !split_bool_set ) 
	    split_bool_set = new Node_List(Thread::current()->resource_area());
	  split_bool_set->push(use);
        }
        if( use->Opcode() == Op_CreateEx ) {
	  if( !split_cex_set ) 
	    split_cex_set = new Node_List(Thread::current()->resource_area());
	  split_cex_set->push(use);
        }


        // Get "block" use is in
        uint idx = 0; 
        while( use->in(idx) != old ) idx++;
        Node *prev = use->is_CFG() ? use : get_ctrl(use->_idx);
        assert( !loop->is_member( get_loop( prev ) ), "" );
        Node *cfg = prev->_idx >= new_counter 
	  ? prev->in(2) 
	  : idom(prev->_idx);
        if( use->is_Phi() )     // Phi use is in prior block
          cfg = prev->in(idx);  // NOT in block of Phi itself
	if( cfg == C->_top_node ) { // Use is dead?
	  _igvn.hash_delete(use);
	  _igvn._worklist.push(use);
	  use->set_req( idx, C->_top_node, &du );
	  continue;
	}

        while( !loop->is_member( get_loop( cfg ) ) ) {
          prev = cfg;
          cfg = cfg->_idx >= new_counter ? cfg->in(2) : idom(cfg->_idx);
        }
        // If the use occurs after merging several exits from the loop, then
        // old value must have dominated all those exits.  Since the same old 
        // value was used on all those exits we did not need a Phi at this
        // merge point.  NOW we do need a Phi here.  Each loop exit value
        // is now merged with the peeled body exit; each exit gets its own
        // private Phi and those Phis need to be merged here.
        Node *phi;
        if( prev->is_Region() ) {
          if( idx == 0 ) {      // Updating control edge?  
            phi = prev;         // Just use existing control
          } else {              // Else need a new Phi
            phi = new PhiNode( prev, old->bottom_type() );
            // Now recursively fix up the new uses of old!
            for( uint i = 1; i < prev->req(); i++ ) {
              phi->set_req( i, old );     // Fixup phi edge to old
              worklist.push(phi); // Onto worklist once for each 'old' input
            }
          }
        } else {
          // Get new RegionNode merging old and new loop exits
          prev = old_new[prev->_idx];
          assert( prev, "just made this in step 7" );
          if( idx == 0 ) {      // Updating control edge?  
            phi = prev;         // Just use existing control
          } else {              // Else need a new Phi
            // Make a new Phi merging data values properly
            phi = new PhiNode( prev, old->bottom_type() );
            phi->set_req( 1, nnn );
            phi->set_req( 2, old );
          }
        }
        // If inserting a new Phi, check for prior hits
        if( idx != 0 ) {
          Node *hit = _igvn.hash_find_insert(phi);
          if( hit == NULL ) { 
           _igvn.register_new_node_with_optimizer(phi); // Register new phi
          } else {                                      // or
            phi = hit;                                  // Use existing phi
          }
          set_ctrl( phi->_idx, prev );
        }
        // Make 'use' use the Phi instead of the old loop body exit value
        _igvn.hash_delete(use);
        _igvn._worklist.push(use);
        use->set_req( idx, phi, &du );
	if( use->_idx >= new_counter ) { // If updating new phis
	  // Not needed for correctness, but prevents a weak assert
	  // in AddPNode from tripping (when we end up with different
	  // base & derived Phis that will become the same after
	  // IGVN does CSE).
	  Node *hit = _igvn.hash_find_insert(use);
	  if( hit )		// Go ahead and re-hash for hits.
	    _igvn.subsume_node( use, hit );
	}
	
        // If 'use' was in the loop-exit block, it now needs to be sunk
        // below the post-loop merge point.
	sink_use( use, prev );
      }
    }
  }

  // Check for IFs that need splitting/cloning.  Happens if an IF outside of
  // the loop uses a condition set in the loop.  The original IF probably
  // takes control from one or more OLD Regions (which in turn get from NEW
  // Regions).  In any case, there will be a set of Phis for each merge point
  // from the IF up to where the original BOOL def exists the loop.
  if( split_if_set ) {
    while( split_if_set->size() ) {
      Node *iff = split_if_set->pop();
      PhiNode *phi = iff->in(1)->is_Phi();
      if( phi ) {
	BoolNode *b = clone_iff( phi, loop );
	_igvn.hash_delete(iff);
	_igvn._worklist.push(iff);
	iff->set_req(1,b,&du);
      }
    }
  }
  if( split_bool_set ) {
    while( split_bool_set->size() ) {
      Node *b = split_bool_set->pop();
      Node *phi = b->in(1);
      assert( phi->is_Phi(), "" );
      SubNode *cmp = clone_bool( (PhiNode*)phi, loop );
      _igvn.hash_delete(b);
      _igvn._worklist.push(b);
      b->set_req(1,cmp,&du);
    }
  }
  if( split_cex_set ) {
    while( split_cex_set->size() ) {
      Node *b = split_cex_set->pop();
      assert( b->in(0)->is_Region(), "" );
      assert( b->in(1)->is_Phi(), "" );
      assert( b->in(0)->in(0) == b->in(1)->in(0), "" );
      split_up( b, b->in(0), NULL );
    }
  }

}

//------------------------------reorg_offsets----------------------------------
// Reorganize offset computations to lower register pressure.  Mostly
// prevent loop-fallout uses of the pre-incremented trip counter (which are
// then alive with the post-incremented trip counter forcing an extra
// register move)
void PhaseIdealLoop::reorg_offsets( IdealLoopTree *loop ) {

  assert( loop->_head->is_CountedLoop(), "" );
  CountedLoopNode *cl = (CountedLoopNode*)loop->_head;
  PhaseDefUse &du = _igvn._du;
  CountedLoopEndNode *cle = cl->loopexit();
  if( !cle ) return;		// The occasional dead loop
  // Find loop exit control
  Node *exit = du.out(cle)[0];
  if( exit->Opcode() == Op_IfTrue )
    exit = du.out(cle)[1];
  assert( exit->Opcode() == Op_IfFalse, "" );

  // Check for the special case of folks using the pre-incremented
  // trip-counter on the fall-out path (forces the pre-incremented
  // and post-incremented trip counter to be live at the same time).
  // Fix this by adjusting to use the post-increment trip counter.
  Node *phi = cl->phi();
  uint    max = du.cnt( phi );
  Node **outp = du.out( phi );
  for( uint i = 0; i < max; i++ ) {
    Node *use = outp[i];        // User of trip-counter
    if( !has_ctrl(use->_idx) ) continue;
    Node *u_ctrl = get_ctrl(use->_idx);
    if( use->is_Phi() ) {
      u_ctrl = NULL;
      for( uint j = 1; j < use->req(); j++ )
        if( use->in(j) == phi )
          u_ctrl = dom_lca( u_ctrl, use->in(0)->in(j) );
    }
    IdealLoopTree *u_loop = get_loop(u_ctrl);
    // Look for loop-invariant use
    if( u_loop == loop ) continue;
    if( loop->is_member( u_loop ) ) continue;
    // Check that use is live out the bottom.  Assuming the trip-counter
    // update is right at the bottom, uses of of the loop middle are ok.
    if( dom_lca( exit, u_ctrl ) != exit ) continue;
    // Hit!  Refactor use to use the post-incremented tripcounter.
    // Compute a post-increment tripcounter.
    Node *opaq = new (2) Opaque2Node( cle->incr() );
    register_new_node( opaq, u_ctrl );
    Node *post = new (3) AddINode( opaq, _igvn.intcon(-cle->stride_con()));
    register_new_node( post, u_ctrl );
    _igvn.hash_delete(use);
    _igvn._worklist.push(use);
    for( uint j = 1; j < use->req(); j++ )
      if( use->in(j) == phi )
        use->set_req( j, post, &du );
    // Since DU info changed, rerun loop
    max = du.cnt(phi);
    outp = du.out(phi);
    i = (uint)-1;
  }

}

//=============================================================================
//------------------------------reorg_offsets----------------------------------
// Reorganize uses of 'q' and 'q+con' such that only 1 is live at a time.  
// This lowers register pressure.  I can map a use of 'q' to 'q+con' by
// subtracting 'con' and vice-versa.  Partition the uses of 'q' and 'q+con'
// into 'soft' and 'hard' uses.  Soft uses can be moved between 'q' and 
// 'q+con' with little (zero?) cost.  Hard uses cost a full add are not worth 
// moving.  Soft uses are all uses outside the current use (the extra add or 
// sub is also outside the loop), addressing expressions (extra offset folds in 
// cheaply) or CmpI.  Hard uses define hard limits on motion.

// Next determine where the 'q+con' add goes.  Moving it above 'q's hard uses
// is not worth it.  Moving the add below the add's hard limit is not worth 
// it.  These limits define a range to move the add in.  Move the add to 
// the add's hard limit.  If this is below or at 'q's hard limit then we can
// make progress.  
//
// Look at each soft use: if the use occurs before the add's block, make the
// use come from 'q'; if the use occurs after the add's block make the use
// come from the add.  If the use occurs in the add's block, make it come 
// from 'q' and assume the block can be scheduled so the add is late.

  // Check for constant offsets being alive at the same time as the base
  // and reorganize.  Changes "A[i++]" code like this:
  //   -OptoReorgOffsets        +OptoReorgOffsets
  //
  //   MOV   AX,BX              CMPu  BX,[range]
  //   INC   AX                 Jge   range-check
  //   CMPu  BX,[range]         INC   BX
  //   Jge   range-check
  //   MOV   BX,AX
  //   ...                    range-check:
  // range-check:               INC   BX
  //   CALL  uncommon_trap      CALL  uncommon_trap
  //
  // In the few loops I studied hard I saw the transform remove one or
  // two copies and 1 or 2 spills.  On the Specs it appears to slow down
  // all Specs by 0.25% to 0.5% - within noise but always in the wrong
  // direction.  It causes CaffeineMarks 'loop' benchmark to slow down
  // by 10% - but it only moves a few instructions around.  In other words,
  // the scheduling change was worth a 10% loss!  Turned off for now.
  // CNC 7/30/1999
/*
static Node *get_use_blk( Node *use, uint uidx, PhaseIdealLoop *phase ) {
  return use->is_Phi() ? use->in(0)->in(uidx) : phase->get_ctrl( use->_idx );
}

static bool is_soft_op( Node *use ) {
  uint uop = use->Opcode();
  switch( uop ) {
  case Op_AddP:
  case Op_AddI:
  case Op_CmpI:
    return true;
  }
  return false;
}

// Compute LCA of hard uses
static Node *hard_uses( Node *x, IdealLoopTree *loop, PhaseIdealLoop *phase, PhaseDefUse *du ) {
  Node *lca = NULL;
  int    max  = du->cnt(x);
  Node **outp = du->out(x);
  for( int i = 0; i < max; i++ ) {  // Check all uses
    Node *use = outp[i];
    if( is_soft_op( use ) )     // Soft use?  
      continue;                 // Skip it
    // Check for hard use(s)
    if( use->is_Phi() ) {
      for( uint j = 1; j < use->req(); j++ ) 
        if( use->in(j) == x ) {
          Node *use_blk = use->in(0)->in(j);
          if( use_blk == phase->C->_top_node ) continue;
          if( phase->is_member( loop, use_blk ) )
            lca = phase->dom_lca( lca, use_blk );
        }
    } else {                     // Normal (non-phi) use
      Node *use_blk = use->is_CFG() ? use : phase->get_ctrl( use->_idx );
      if( phase->is_member( loop, use_blk ) )
        lca = phase->dom_lca( lca, use_blk );
    }
  }
  return lca;
}

// Move this use to either 'add' or 'q'.
static void alter_use( Node *use, uint idx, Node *use_blk, Node *x, Node *x_blk, Node *con, IdealLoopTree *loop, PhaseIdealLoop *phase, PhaseIterGVN *igvn, const char *msg ) {
#ifndef PRODUCT
  if( PrintOpto ) tty->print( msg );
#endif
  igvn->hash_delete( use );

  // Is use a CmpI, with a loop-invariant other input?
  // Then move adjustment out of the loop
  if( use->Opcode() == Op_CmpI ) {
    Node *li = use->in(3-idx);  // The loop-invariant other input
    Node *li_blk = li->is_CFG() ? li : phase->get_ctrl(li->_idx);
    if( !phase->is_member( loop, li_blk ) ) {
#ifndef PRODUCT
      if( PrintOpto ) tty->print("(cmp twist)");
#endif
      // Subtract from L.I.
      Node *sub = new (3) SubINode( li, con );
      igvn->register_new_node_with_optimizer( sub );
      // Locate subtract next to L.I.
      phase->set_ctrl( sub->_idx, li_blk );
      // Use SUB instead of L.I.
      use->set_req( 3-idx, sub, &igvn->_du );
      // Use X instead of previous add
      use->set_req( idx, x, &igvn->_du );
      igvn->_worklist.push( use );
      return;
    }
  } 

  // 'opaque' is used to prevent the optimizer from undoing my 
  // remapping of uses.
  Node *opaque = new (2) OpaqueNode( x );
  igvn->register_new_node_with_optimizer( opaque );
  phase->set_ctrl( opaque->_idx, x_blk );

  Node *new_add = new (3) AddINode( opaque, con );
  igvn->register_new_node_with_optimizer( new_add );
  phase->set_ctrl( new_add->_idx, use_blk );
  use->set_req( idx, new_add, &igvn->_du );
  igvn->_worklist.push( use );
}

static void do_remap_soft_use( Node *use, uint idx, Node *use_blk, Node *q, Node *add, Node *q_ctrl, Node *a_hard, IdealLoopTree *loop, PhaseIdealLoop *phase, PhaseDefUse *du, PhaseIterGVN *igvn ) {

  if( a_hard == phase->dom_lca( use_blk, a_hard ) && use != add ) {
    // Use will come from 'add'
    if( use->in(idx) == q ) { // Use comes from 'q', must move it
      Node *conode = ((PhaseGVN*)igvn)->intcon( -add->in(2)->get_int() );
      alter_use( use, idx, use_blk, add, a_hard, conode, loop, phase, igvn, ", move to add" );
    }
  } else {
    // Use will come from 'q'
    if( use->in(idx) == add ) {   // Use comes from 'add', must move it  
      alter_use( use, idx, use_blk, q, q_ctrl, add->in(2), loop, phase, igvn, ", move to q" );
    }
  }
}

// Remap soft uses ONLY.  Hard uses (ones that cost to relocate) remain put.
// Soft uses are relocated to either 'q' or 'add' from their current location
// by adding or subtracting the appropriate constant.
static void remap_soft_uses( Node *x, Node *q, Node *add, Node *q_ctrl, Node *a_hard, IdealLoopTree *loop, PhaseIdealLoop *phase, PhaseDefUse *du, PhaseIterGVN *igvn ) {
  Node *uses[256];
  int max  = du->cnt(x);
  if( max > 256 ) max = 256;
  Node **outp = du->out(x);
  for( int j = 0; j < max; j++) // Copy all uses
    uses[j] = outp[j];

  for( int i = 0; i < max; i++ ) {  // Check all uses
    Node *use = uses[i];
    // Check for hard use(s)
    if( use->is_Phi() ) {       // Phis must check all inputs
      for( uint j = 1; j < use->req(); j++ ) 
        if( use->in(j) == x ) {
          Node *use_blk = use->in(0)->in(j);
          if( use_blk == phase->C->_top_node ) continue;
          if( !phase->is_member( loop, use_blk ) )
            do_remap_soft_use(use,j,use_blk,q,add,q_ctrl,a_hard,loop,phase,du,igvn);
        }
    } else {                    // Normal (non-phi) use
      Node *use_blk = use->is_CFG() ? use : phase->get_ctrl( use->_idx );
      if( !use_blk ) continue;  // The occasional dead node
      uint j;
      for( j = 1; j < use->req(); j++ ) 
        if( use->in(j) == x ) 
          break;
      if( !phase->is_member( loop, use_blk ) || is_soft_op( use ) )
        do_remap_soft_use(use,j,use_blk,q,add,q_ctrl,a_hard,loop,phase,du,igvn);
    }
  }
}

void IdealLoopTree::reorg_offsets( Node *add, PhaseIdealLoop *phase ) {
  PhaseIterGVN *igvn = &phase->_igvn;
  PhaseDefUse *du = &igvn->_du;

  // The 'q' being added
  Node *q = add->in(1);

  // Compute hard limit - the LCA of hard uses
  Node *q_hard = hard_uses(   q, this, phase, du );
  Node *a_hard = hard_uses( add, this, phase, du );
  Node *q_ctrl = phase->get_ctrl(  q->_idx);
  Node *a_ctrl = phase->get_ctrl(add->_idx);

  // Lift a_hard out of a loop back edge, but not past 'q'
  if( a_hard == _head->in(LoopNode::LoopBackControl) &&
      q_ctrl != a_hard &&       // Do not try to slip past 'q'
      q_ctrl != a_hard->in(0) &&
      q_ctrl != a_hard->in(0)->in(0) )
    a_hard = a_hard->in(0)->in(0);

  // Check for being forced to have q live past add
  // because q's hard uses are dominated by add's hard uses.
  if( q_hard ) {
    Node *n = q_hard;
    while( n != q_ctrl ) {
      n = phase->idom_at(n->_idx);
      if( n == a_hard )         // q MUST overlap add's lifetime
        return;                 // No progress
    }
  }

  // Move the add to its hard limit.  If it actually moves then call
  // 'remap_soft_uses'.  Remapping soft uses makes uses that are
  // above the add's new location use 'q' instead.
  if( a_hard != a_ctrl ) {      // Add is going to move?
#ifndef PRODUCT
    tty->print("sinking add");
#endif
    phase->set_ctrl( add->_idx, a_hard );
    remap_soft_uses( add, q, add, q_ctrl, a_hard, this, phase, du, igvn );
  }
  // Now remap all q's soft uses to use add if possible or q otherwise
  remap_soft_uses( q, q, add, q_ctrl, a_hard, this, phase, du, igvn );

#ifndef PRODUCT
  tty->cr();
#endif
}
*/

