/*
 * @(#)machnode.cpp	1.126 01/02/09 14:46:31
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_machnode.cpp.incl"

//=============================================================================
// Return the value requested
// result register lookup, corresponding to int_format
int MachOper::reg(PhaseChaitin *pc_, const Node *node)   const { return 0x00; }
// input register lookup, corresponding to ext_format
int MachOper::reg(PhaseChaitin *pc_, const Node *node, int idx)   const { return 0x00; }
int  MachOper::constant() const { return 0x00; }
bool MachOper::constant_is_oop() const { return false; }
jdouble MachOper::constantD() const { ShouldNotReachHere(); return 0.0; }
jfloat  MachOper::constantF() const { ShouldNotReachHere(); return 0.0; }
jlong   MachOper::constantL() const { ShouldNotReachHere(); return 0L ; }
TypeOopPtr *MachOper::oop() const { return NULL; }
int MachOper::ccode() const { return 0x00; }
// A zero, default, indicates this value is not needed.
// May need to lookup the base register, as done in int_ and ext_format
int MachOper::base (PhaseChaitin *pc_, const Node *node, int idx)  const { return 0x00; }
int MachOper::index(PhaseChaitin *pc_, const Node *node, int idx)  const { return 0x00; }
int MachOper::scale()  const { return 0x00; }
int MachOper::disp (PhaseChaitin *pc_, const Node *node, int idx)  const { return 0x00; }
int MachOper::constant_disp()  const { return 0; }
// Check for PC-Relative displacement
bool MachOper::disp_is_oop() const { return false; }
// Return the label's address
int MachOper::label()  const { return 0x00; }

//------------------------------negate-----------------------------------------
// Negate conditional branches.  Error for non-branch operands
void MachOper::negate() {
  ShouldNotCallThis();
}

//-----------------------------type--------------------------------------------
const Type *MachOper::type() const {
  return Type::BOTTOM;
}

//------------------------------in_RegMask-------------------------------------
const RegMask *MachOper::in_RegMask(int index) const { 
  ShouldNotReachHere(); 
  return NULL; 
}

//------------------------------dump_spec--------------------------------------
// Print any per-operand special info
#ifndef PRODUCT
void MachOper::dump_spec() const { }
#endif

//------------------------------hash-------------------------------------------
// Print any per-operand special info
uint MachOper::hash() const {
  ShouldNotCallThis();
  return 5;
}

//------------------------------cmp--------------------------------------------
// Print any per-operand special info
uint MachOper::cmp( const MachOper &oper ) const {
  ShouldNotCallThis();
  return opcode() == oper.opcode();
}

//------------------------------hash-------------------------------------------
// Print any per-operand special info
uint labelOper::hash() const {
  ShouldNotCallThis();
  return 5;
}

//------------------------------cmp--------------------------------------------
// Print any per-operand special info
uint labelOper::cmp( const MachOper &oper ) const {
  ShouldNotCallThis();
  return opcode() == oper.opcode();
}


//=============================================================================
//------------------------------MachNode---------------------------------------
MachNode::MachNode() : Node((uint)0), _opnds(0) {}
        
//------------------------------emit-------------------------------------------
void MachNode::emit(CodeBuffer &cbuf, PhaseChaitin *pc_) const {
  #ifdef ASSERT
  tty->print("missing MachNode emit function: ");
  dump();
  #endif
  ShouldNotCallThis();
}

//------------------------------size-------------------------------------------
// Size of instruction in bytes
uint MachNode::size(PhaseChaitin *pc_) const {
  ResourceMark rm;

  CodeBuffer buf(256 + 128 + 128,256,128,128,true);

  // Very sleazy; emit into a trash buffer and count bytes emitted.
  emit(buf, pc_);
  return buf.code_end() - buf.code_begin();
}


//------------------------------primary secondary tertiary---------------------
// Return components of the opcode:
// 0xFFFFFFFF indicates that this value is not used.
uint  MachNode::primary()   const { return 0xFFFFFFFF; };
uint  MachNode::secondary() const { return 0xFFFFFFFF; };
uint  MachNode::tertiary()  const { return 0xFFFFFFFF; };

//------------------------------hash-------------------------------------------
uint MachNode::hash() const {
  uint no = num_opnds();
  uint sum = rule();
  for( uint i=0; i<no; i++ )
    sum += _opnds[i]->hash();
  return sum+Node::hash();
}

//-----------------------------cmp---------------------------------------------
uint MachNode::cmp( const Node &node ) const {
  MachNode& n = *((Node&)node).is_Mach();
  uint no = num_opnds();
  if( no != n.num_opnds() ) return 0;
  if( rule() != n.rule() ) return 0;
  for( uint i=0; i<no; i++ )    // All operands must match
    if( !_opnds[i]->cmp( *n._opnds[i] ) )
      return 0;                 // mis-matched operands
  return 1;                     // match
}

// Return an equivalent instruction using memory for cisc_operand position
MachNode *MachNode::cisc_version(int offset) {
  ShouldNotCallThis();
  return NULL;
}

void MachNode::use_cisc_RegMask() {
  ShouldNotReachHere();
}


//-----------------------------in_oper_RegMask---------------------------------
// cisc-spillable instructions redefine for use by in_RegMask
const RegMask *MachNode::in_oper_RegMask(uint idx, uint operand, uint skipped) const {
  return _opnds[operand]->in_RegMask(idx-skipped);
}


//-----------------------------in_RegMask--------------------------------------
const RegMask &MachNode::in_RegMask( uint idx ) const {
  uint numopnds = num_opnds();        // Virtual call for number of operands
  uint skipped   = oper_input_base(); // Sum of leaves skipped so far
  if( idx < skipped ) {
    assert( ideal_Opcode() == Op_AddP, "expected base ptr here" );
    assert( idx == 1, "expected base ptr here" );
    // debug info can be anywhere
    return *Compile::current()->_matcher->idealreg2spillmask[Op_RegP];
  }
  uint opcnt     = 1;                 // First operand
  uint num_edges = _opnds[1]->num_edges(); // leaves for first operand 
  while( idx >= skipped+num_edges ) {
    skipped += num_edges;
    opcnt++;                          // Bump operand count
    assert( opcnt < numopnds, "Accessing non-existent operand" );
    num_edges = _opnds[opcnt]->num_edges(); // leaves for next operand
  }

  return *in_oper_RegMask(idx, opcnt, skipped);
}


//-----------------------------get_base_and_disp----------------------------
const Node* MachNode::get_base_and_disp(int &offset) const {
  return (Node*)NULL;  // no address in most nodes
}


//-----------------------------operand_index---------------------------------
int MachNode::operand_index( uint operand ) const {
  if( operand < 1 )  return -1;
  assert(operand < num_opnds(), "oob");
  if( _opnds[operand]->num_edges() == 0 )  return -1;

  uint skipped   = oper_input_base(); // Sum of leaves skipped so far
  for (uint opcnt = 1; opcnt < operand; opcnt++) {
    uint num_edges = _opnds[opcnt]->num_edges(); // leaves for operand
    skipped += num_edges;
  }
  return skipped;
}


//------------------------------negate-----------------------------------------
// Negate conditional branches.  Error for non-branch Nodes
void MachNode::negate() {
  ShouldNotCallThis();
}

//------------------------------peephole---------------------------------------
// Apply peephole rule(s) to this instruction
MachNode *MachNode::peephole( Block *block, int block_index, PhaseChaitin *pc_, int &deleted ) {
  return NULL;
}

//------------------------------label_addr-------------------------------------
// Set the address for a LabelOper, if an operand for this instruction
void MachNode::label_addr( int addr ) {
  ShouldNotCallThis();
}

//------------------------------label_block------------------------------------
// Set the block number associated with this label
void MachNode::label_block( int target, int block_number ) {
  ShouldNotCallThis();
}

//------------------------------rematerialize----------------------------------
bool MachNode::rematerialize() const {
  uint r = rule();              // Match rule
  if( r <  Matcher::_begin_rematerialize ||
      r >= Matcher::_end_rematerialize )
    return false;

  // For 2-address instructions, the input live range is also the output
  // live range.  Remateralizing does not make progress on the that live range.
  if( two_adr() )  return false;

  // Check for rematerializing float constants, or not
  if( !Matcher::rematerialize_float_constants ) {
    int op = ideal_Opcode();
    if( op == Op_ConF || op == Op_ConD )
      return false;
  }

  if( req() > 2 && ideal_reg() != Op_RegFlags )
    return false;

  return true;
}

#ifndef PRODUCT
//------------------------------dump_spec--------------------------------------
// Print any per-operand special info
void MachNode::dump_spec() const {
  uint cnt = num_opnds();
  for( uint i=0; i<cnt; i++ )
    _opnds[i]->dump_spec();
}

//------------------------------dump_format------------------------------------
// access to virtual
void MachNode::dump_format(PhaseChaitin *pc) const {
  format(pc); // access to virtual
}
#endif

//=============================================================================
MachNullCheckNode::MachNullCheckNode( Node *ctrl, Node *memop, uint vidx ) : _vidx(vidx) {
  add_req(ctrl);
  add_req(memop);
}

#ifndef PRODUCT
void MachNullCheckNode::format( PhaseChaitin *pc_ ) const {
  int reg = pc_->get_register(in(1)->in(_vidx));
  tty->print("%s %s", Name(), Matcher::regName[reg]);
}
#endif

void MachNullCheckNode::emit(CodeBuffer &cbuf, PhaseChaitin *pc_) const {
  // only emits entries in the null-pointer exception handler table
}

const RegMask &MachNullCheckNode::in_RegMask( uint idx ) const {
  if( idx == 0 ) return RegMask::Empty;
  else return in(1)->is_Mach()->out_RegMask();
}

//=============================================================================
//------------------------------CastIP-----------------------------------------
CastIPNode::CastIPNode( Node *control, Node *oopmem, Node *n ) : MachIdealNode( ) {
  add_req(control);
  add_req(oopmem);
  add_req(n);
}

//------------------------------Idealize---------------------------------------
// Magically we "know" that all StoreB's reached from here are actually
// card-marks AND all card-marks can be freely reordered.
Node *CastIPNode::Ideal( PhaseGVN *phase, PhaseDefUse *du ) {
  if( in(0) && remove_dead_region(phase,du) ) return this;
  if( in(1)->Opcode() != Op_StoreB ) return NULL;
  if( in(1)->in(MemNode::Memory) == in(1) ) return NULL; // no progress
  assert( phase->type( in(1)->in(MemNode::ValueIn) ) == TypeInt::ZERO, "" );
  set_req( 1, in(1)->in(MemNode::Memory), du );
  return this;
}

//------------------------------Value------------------------------------------
const Type *CastIPNode::Value( PhaseTransform *phase ) const {
  // Detune optimizer to fix a bug with constant folding of klass oops
  // into card mark offsets.
  // return phase->type( _in[2] );
  return TypeInt::INT;
}

//------------------------------out_RegMask------------------------------------
const RegMask &CastIPNode::out_RegMask() const {
  return *Matcher::idealreg2regmask[Op_RegI];
}

//------------------------------in_RegMask-------------------------------------
const RegMask &CastIPNode::in_RegMask(uint) const {
  return *Matcher::idealreg2regmask[Op_RegP];
}

//------------------------------format-----------------------------------------
#ifndef PRODUCT
void CastIPNode::format( PhaseChaitin *pc ) const {
  int reg = pc->get_register(this);
  assert( pc->get_register(in(2)) == reg, "must be 2-adr" );
  tty->print("cast %s from ptr to int",Matcher::regName[reg]);
}
#endif


//=============================================================================
const Type *MachProjNode::bottom_type() const { 
  if( _ideal_reg == fat_proj ) return Type::BOTTOM;
  // Try the normal mechanism first
  const Type *t = _in[0]->bottom_type();
  if( t->_base == Type::Tuple ) {
    const TypeTuple *tt = t->is_tuple();
    if( _con < tt->_cnt )
      return tt->_fields[_con]; 
  }
  // Else use generic type from ideal register set
  assert((uint)_ideal_reg < (uint)_last_machine_leaf && Type::mreg2type[_ideal_reg], "in bounds");
  return Type::mreg2type[_ideal_reg];
}

#ifndef PRODUCT
void MachProjNode::dump_spec() const {
  ProjNode::dump_spec();
  if (WizardMode) {
    switch (_ideal_reg) {
      case unmatched_proj:  tty->print("/unmatched");         break;
      case fat_proj:        tty->print("/fat"); _out.dump();  break;
    }
  }
}
#endif

//=============================================================================
#ifndef PRODUCT
void MachIfNode::dump_spec() const {
  tty->print("P=%f, C=%f",_prob, _cnt);
}
#endif

//=============================================================================
uint MachReturnNode::size_of() const { return sizeof(*this); }

//------------------------------Registers--------------------------------------
const RegMask &MachReturnNode::in_RegMask( uint idx ) const { 
  return _in_rms[idx];
}


//=============================================================================
MachSafePointNode::MachSafePointNode() :
  _oop_map(NULL), _jvms(NULL), _jvmtail(NULL), _jvmadj(0) {}

const Type *MachSafePointNode::bottom_type() const {  return TypeTuple::MEMBAR; }

//------------------------------Registers--------------------------------------
const RegMask &MachSafePointNode::in_RegMask( uint idx ) const { 
  // Values in the domain use the users calling convention, embodied in the
  // _in_rms array of RegMasks.
  if( idx < TypeFunc::Parms ) return _in_rms[idx];
  // Values outside the domain represent debug info
  return *Compile::current()->_matcher->idealreg2spillmask[in(idx)->ideal_reg()];
}


//=============================================================================
MachCallNode::MachCallNode() : MachSafePointNode() {}

uint MachCallNode::hash() const { return Node::hash() + (int)_func; }
uint MachCallNode::cmp( const Node &n ) const
{ return _func == ((MachCallNode&)n)._func; }
const Type *MachCallNode::bottom_type() const { return _func->_range; }
const Type *MachCallNode::Value(PhaseTransform *phase) const { return _func->_range; }

#ifndef PRODUCT
void MachCallNode::dump_spec() const { 
  tty->print("# "); 
  _func->dump();
  if( _jvms ) _jvms->dump_spec();
}
#endif

bool MachCallNode::returns_float_or_double() const {
  const Type *return_val = _func->_range->_cnt == TypeFunc::Parms ? Type::TOP 
    : _func->_range->_fields[TypeFunc::Parms];
  bool  float_or_double = (return_val != NULL) 
    && ((return_val->_base == Type::FloatBot) || (return_val->_base == Type::DoubleBot));
  return float_or_double;
}

//------------------------------Registers--------------------------------------
const RegMask &MachCallNode::in_RegMask( uint idx ) const { 
  // Values in the domain use the users calling convention, embodied in the
  // _in_rms array of RegMasks.
  if( idx < _func->_domain->_cnt ) return _in_rms[idx];
  // Values outside the domain represent debug info
  return *Compile::current()->_matcher->idealreg2debugmask[in(idx)->ideal_reg()];
}

//=============================================================================
uint MachCallJavaNode::size_of() const { return sizeof(*this); }
uint MachCallJavaNode::hash() const { return MachCallNode::hash() + _method->hash(); }
uint MachCallJavaNode::cmp( const Node &n ) const { 
  MachCallJavaNode &call = (MachCallJavaNode&)n;
  return MachCallNode::cmp(call) && _method->equals(call._method); 
}
#ifndef PRODUCT
void MachCallJavaNode::dump_spec() const { 
  if( _method ) {
    _method->print_short_name();
    tty->print(" ");
  }
  MachCallNode::dump_spec();
}
#endif

//=============================================================================
uint MachCallStaticJavaNode::size_of() const { return sizeof(*this); }
uint MachCallStaticJavaNode::hash() const { return MachCallJavaNode::hash() + _name[0]; }
uint MachCallStaticJavaNode::cmp( const Node &n ) const { 
  MachCallStaticJavaNode &call = (MachCallStaticJavaNode&)n;
  return MachCallJavaNode::cmp(call) && _name == call._name; 
}
#ifndef PRODUCT
void MachCallStaticJavaNode::dump_spec() const { 
  tty->print("Static ");
  if( _name ) tty->print("wrapper for: %s ", _name );
  MachCallJavaNode::dump_spec();
}
#endif

//=============================================================================
#ifndef PRODUCT
void MachCallDynamicJavaNode::dump_spec() const { 
  tty->print("Dynamic ");
  MachCallJavaNode::dump_spec();
}
#endif
//=============================================================================
#ifndef PRODUCT
void MachCallCompiledJavaNode::dump_spec() const { 
  tty->print("CompiledJava ");
  MachCallNode::dump_spec();
}
#endif
//=============================================================================
#ifndef PRODUCT
void MachCallInterpreterNode::dump_spec() const { 
  tty->print("Interpreter ");
  MachCallNode::dump_spec();
}
#endif
//=============================================================================
uint MachCallRuntimeNode::size_of() const { return sizeof(*this); }
uint MachCallRuntimeNode::hash() const { return MachCallNode::hash() + _name[0]; }
uint MachCallRuntimeNode::cmp( const Node &n ) const { 
  MachCallRuntimeNode &call = (MachCallRuntimeNode&)n;
  return MachCallNode::cmp(call) && !strcmp(_name,call._name);
}
#ifndef PRODUCT
void MachCallRuntimeNode::dump_spec() const { 
  tty->print("%s ",_name);
  MachCallNode::dump_spec();
}
#endif
//=============================================================================
// A shared JVMStateObj for all HaltNodes.  Indicates the start of debug info
// is at TypeFunc::Parms.  Only required for SOE register spill handling - 
// to indicate where the stack-slot-only debug info inputs begin.
// There is no other JVM state needed here.
JVMStateObj jvms_for_throw(0,0,0);
JVMStateObj *MachHaltNode::jvms() const {
  return &jvms_for_throw;
}

//=============================================================================
void labelOper::int_format(PhaseChaitin *pc, const MachNode *node) const {
  if( _label == 0 ) {
    tty->print("$$label");
  } else if ( !_block_num ) {
    tty->print("#0x%03d", _label);
  } else {
    if( !node->is_pc_relative() ) Unimplemented();
    tty->print("B%d", _block_num);
    tty->print("\t# 0x%3.3x", (int)_target);
  }
}
