/*
 * @(#)matcher.cpp	1.277 01/02/09 14:46:34
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_matcher.cpp.incl"


static Node * const unmatchable_mem = (Node*)-1;

const int Matcher::base2reg[Type::lastype] = {
  Node::NotAMachineReg,0,0, Op_RegI, Op_RegL, 0, 
  Node::NotAMachineReg, Node::NotAMachineReg, /* tuple, array */
  Op_RegP, Op_RegP, Op_RegP, Op_RegP, Op_RegP, /* the pointers */
  0, 0/*abio*/,
  Op_RegP /* Return address */, 0, /* the memories */
  Op_RegF, Op_RegF, Op_RegF, Op_RegD, Op_RegD, Op_RegD, 
  0  /*bottom*/
};

const RegMask *Matcher::idealreg2regmask[_last_machine_leaf];
RegMask Matcher::mreg2regmask[_last_Mach_Reg];
RegMask Matcher::STACK_ONLY_mask;
RegMask Matcher::c_frame_ptr_mask;
const uint Matcher::_begin_rematerialize = _BEGIN_REMATERIALIZE;
const uint Matcher::_end_rematerialize   = _END_REMATERIALIZE;

//---------------------------Matcher-------------------------------------------
Matcher::Matcher( Node_List &proj_list, bool save_argument_registers ) : 
  PhaseTransform( Phase::Ins_Select ),
#ifdef ASSERT
  _old2new_map(&C->_comp_arena),
#endif
  _reduceOp(reduceOp), _leftOp(leftOp), _rightOp(rightOp), 
  _swallowed(swallowed), 
  _begin_inst_chain_rule(_BEGIN_INST_CHAIN_RULE), 
  _end_inst_chain_rule(_END_INST_CHAIN_RULE), 
  _must_clone(must_clone), _proj_list(proj_list),
  _register_save_policy(register_save_policy),
  _c_reg_save_policy(c_reg_save_policy),
  _register_save_type(register_save_type),
  _can_be_arg(can_be_arg),
  _ruleName(ruleName), 
  _allocation_started(false),
  _states(Chunk::medium_size),
  _visited(&_states),
  _shared(&_states),
  _dontcare(&_states) {
  C->_matcher = this;

  idealreg2spillmask[Op_RegI] = NULL;
  idealreg2spillmask[Op_RegF] = NULL;
  idealreg2spillmask[Op_RegD] = NULL;
  idealreg2spillmask[Op_RegP] = NULL;
  idealreg2spillmask[Op_RegL] = NULL;
  idealreg2debugmask[Op_RegI] = NULL;
  idealreg2debugmask[Op_RegF] = NULL;
  idealreg2debugmask[Op_RegD] = NULL;
  idealreg2debugmask[Op_RegP] = NULL;
  idealreg2debugmask[Op_RegL] = NULL;

  // One-time initialization of some register masks.
  init_spill_mask( C->_root->in(1) );
  _return_addr_mask = return_addr(); 

  // Map a Java-signature return type into return register-value
  // machine registers for 0, 1 and 2 returned values.
  const TypeTuple *range = C->_tf->_range;
  switch( range->_cnt ) {
  case TypeFunc::Parms+2: 	// Longs return 2 words
    _return_value_register[0] 
      = callee_return_value(base2reg[range->_fields[TypeFunc::Parms]->_base]);
    _return_value_mask[0] = RegMask(_return_value_register[0]);
    // Doubles & longs return 1 value but two words.  
    // So 1 edge into the ReturnNode but 2 bits in the result mask.
    _return_value_mask[0].Insert(OptoReg::add(_return_value_register[0],1));
    assert(_return_value_mask[0].is_bound2(), "valid return mask");
    // Throw away the other edge
    assert( range->_fields[TypeFunc::Parms+1] == Type::HALF, "" );
    _return_value_register[1] = OptoReg::Bad;                                 
    _return_value_mask[1] = RegMask::Empty;
    break;
  case TypeFunc::Parms+1:	// Normal case
    _return_value_register[0] 
      = callee_return_value(base2reg[range->_fields[TypeFunc::Parms]->_base]);
    _return_value_mask[0] = RegMask(_return_value_register[0]);
    break;
  case TypeFunc::Parms+0:      // No returned values
    break;
  default:
    ShouldNotReachHere();
  }

  // ---------------
  // Frame Layout

  // Need the method signature to determine the incoming argument types,
  // because the types determine which registers the incoming arguments are
  // in, and this affects the matched code.
  const TypeTuple *domain = C->_tf->_domain;
  uint argcnt = C->argcnt();
  _parm_regs = NEW_RESOURCE_ARRAY( uint16, argcnt );
  _calling_convention_mask = NEW_RESOURCE_ARRAY( RegMask, argcnt );
  uint i;
  for( i = 0; i<argcnt; i++ )
    _parm_regs[i] = base2reg[domain->_fields[i+TypeFunc::Parms]->_base];

  // Pass array of ideal registers and length to USER code (from the AD file)
  // that will convert this to an array of register numbers.
  const StartNode *start = C->start();
  start->calling_convention( _parm_regs, argcnt );
#ifdef ASSERT
  // Sanity check users' calling convention.  Real handy while trying to 
  // get the initial port correct.
  if ( C->_C_function == CAST_FROM_FN_PTR(address, OptoRuntime::rethrow_C) )
    assert((int)_parm_regs[0] == pd_rethrow_oop_register(), "rethrow register must agree");
  { for (uint i = 0; i<argcnt; i++) {
      assert((int)_parm_regs[i] >= SharedInfo::stack0 ||
              _can_be_arg[_parm_regs[i]] ||
              C->_C_function == CAST_FROM_FN_PTR(address, OptoRuntime::rethrow_C) ||
	     (int)_parm_regs[i] == inline_cache_reg(),"parameters in register must be preserved by runtime stubs");
      for (uint j = 0; j < i; j++) {
        assert(_parm_regs[i] != _parm_regs[j], "calling conv. must produce distinct regs");
      }
    }
  }
#endif

  // Do some initial frame layout.

  // Compute the old incoming SP (may be called FP) as
  //   SharedInfo::stack0 + locks + in_preserve_stack_slots + pad2.
  _old_SP = SharedInfo::stack2reg( round_to( C->in_preserve_stack_slots() + (C->_sync * C->sync_stack_slots()), WordsPerLong ) );
  assert( is_even(_old_SP), "must be even" );

  // Compute highest incoming stack argument as
  //   _old_SP + out_preserve_stack_slots + incoming argument size.
  _in_arg_limit = OptoReg::add(_old_SP, C->out_preserve_stack_slots());
  assert( is_even(_in_arg_limit), "out_preserve must be even" );
  for( i = 0; i < argcnt; i++ ) {
    // calling_convention returns stack arguments as a count of
    // slots beyond SharedInfo::stack0.  We need to convert this to
    // the allocators point of view, taking into account all the
    // preserve area, locks & pad2.
    OptoReg::Name reg = OptoReg::Name(_parm_regs[i]);
    if( reg >= SharedInfo::stack0 ) {  // Stack slot argument?
      reg = OptoReg::add(_old_SP, SharedInfo::reg2stack(reg) );
      reg = OptoReg::add(reg, C->out_preserve_stack_slots());
      _parm_regs[i] = reg;    // Saved biased stack-slot register number
      if( reg >= _in_arg_limit )
        _in_arg_limit = OptoReg::add(reg,1); // Bump max stack slot seen
    }
    _calling_convention_mask[i] = RegMask(reg);

    // Doubles need 2 bits in the first value, and undefined in the 2nd
    // An exception: Doubles are passed by halves into an I2C converter.
    // This is because the interpreter stack is not 8-byte aligned.
    if( i > 0 && 
	(domain->_fields[i-1+TypeFunc::Parms] == Type::DOUBLE ||
	 domain->_fields[i-1+TypeFunc::Parms] == TypeLong::LONG) ) {
      assert(domain->_fields[i+TypeFunc::Parms] == Type::HALF, "");
      _calling_convention_mask[i-1].Insert(reg);
      _calling_convention_mask[i  ].Remove(reg);
      assert( start->Opcode() == Op_StartI2C || 
              _calling_convention_mask[i-1].is_bound2(), 
              "must be a proper pair");
    }

  }

  // Finally, make sure the incoming arguments take up an even number of
  // words, in case the arguments or locals need to contain doubleword stack 
  // slots.  The rest of the system assumes that stack slot pairs (in 
  // particular, in the spill area) which look aligned will in fact be 
  // aligned relative to the stack pointer in the target machine.  Double 
  // stack slots will always be allocated aligned.
  _new_SP = OptoReg::Name(round_to( _in_arg_limit, WordsPerLong ));

  // Compute highest outgoing stack argument as
  //   _new_SP + out_preserve_stack_slots + max(outgoing argument size).
  _out_arg_limit = OptoReg::add(_new_SP, C->out_preserve_stack_slots());
  assert( is_even(_out_arg_limit), "out_preserve must be even" );


  // ---------------
  // Collect roots of matcher trees.  Every node for which
  // _shared[_idx] is cleared is guaranteed to not be shared, and thus
  // be a valid interior of some tree.
  set_shared( C->_root );
  set_shared( C->_top_node );

  // Swap out to old-space; emptying new-space 
  Arena *old = C->_arena.move_contents(&(C->_old_arena));
  C->_unique = 0;		// Reset unique Node counter

  // Recursively match trees from old space into new space.
  // Correct leaves of new-space Nodes; they point to old-space.
  _visited.Clear();		// Clear visit bits for xform call
  C->_top_node = xform( C->_top_node );
  C->_root     = xform( C->_root     )->is_Root();
  if (SoftMatchFailure
      && (C->_top_node == NULL || C->_root == NULL)) {
    // delete old;
    old->destruct_contents();
    C->_root = NULL; // Bail out
    return;
  }
  assert( C->_top_node, "" );
  assert( C->_root, "" );
  validate_null_checks();
  // Now smoke old-space
  // delete old;
  old->destruct_contents();

  // ------------------------
  // Set up save-on-entry registers
  Fixup_Save_On_Entry( save_argument_registers );
}


//------------------------------Fixup_Save_On_Entry----------------------------
// The stated purpose of this routine is to take care of save-on-entry 
// registers.  However, the overall goal of the Match phase is to convert into
// machine-specific instructions which have RegMasks to guide allocation.
// So what this procedure really does is put a valid RegMask on each input
// to the machine-specific variations of all Return, TailCall and Halt 
// instructions.  It also adds edgs to define the save-on-entry values (and of
// course gives them a mask).

static RegMask *init_input_masks( uint size, RegMask &ret_adr, RegMask &fp ) {
  RegMask *rms = NEW_RESOURCE_ARRAY( RegMask, size );
  // Do all the pre-defined register masks
  rms[TypeFunc::Control  ] = RegMask::Empty;
  rms[TypeFunc::I_O      ] = RegMask::Empty;
  rms[TypeFunc::Memory   ] = RegMask::Empty;
  rms[TypeFunc::ReturnAdr] = ret_adr;
  rms[TypeFunc::FramePtr ] = fp;
  return rms;
}

//---------------------------init_first_stack_mask-----------------------------
// Create the initial stack mask used by values spilling to the stack.
// Disallow any debug info in outgoing argument areas by setting the
// initial mask accordingly.
void Matcher::init_first_stack_mask() {

  // Allocate storage for spill masks as masks for the appropriate load type.
  RegMask *rms = (RegMask*)C->_comp_arena.Amalloc_4(sizeof(RegMask)*10);
  idealreg2spillmask[Op_RegI] = &rms[0];
  idealreg2spillmask[Op_RegF] = &rms[1];
  idealreg2spillmask[Op_RegD] = &rms[2];
  idealreg2spillmask[Op_RegP] = &rms[3];
  idealreg2spillmask[Op_RegL] = &rms[4];
  idealreg2debugmask[Op_RegI] = &rms[5];
  idealreg2debugmask[Op_RegF] = &rms[6];
  idealreg2debugmask[Op_RegD] = &rms[7];
  idealreg2debugmask[Op_RegP] = &rms[8];
  idealreg2debugmask[Op_RegL] = &rms[9];

  OptoReg::Name i;

  // At first, start with the empty mask
  C->FIRST_STACK_mask.Clear();

  // Add in locks, pad2; the space between register 0 and the start of
  // the in_preserve area.
  OptoReg::Name max  = OptoReg::add(_old_SP, -(int)C->in_preserve_stack_slots());

  // Add in the incoming argument area
  OptoReg::Name init = OptoReg::add(_old_SP, C->out_preserve_stack_slots());
  max  = _in_arg_limit;
  for( i = init; i < max; i = OptoReg::add(i,1) )
    C->FIRST_STACK_mask.Insert(i);

  // Add in all bits past the outgoing argument area
  init = _out_arg_limit;
  max = OptoReg::Name(RM_SIZE<<5);
  for( i = init; i < max; i = OptoReg::add(i,1) )
    C->FIRST_STACK_mask.Insert(i);

  // Make spill masks.  Registers for their class, plus FIRST_STACK_mask.
  *idealreg2spillmask[Op_RegI] = *idealreg2regmask[Op_RegI]; 
   idealreg2spillmask[Op_RegI]->OR(C->FIRST_STACK_mask);
  *idealreg2spillmask[Op_RegF] = *idealreg2regmask[Op_RegF]; 
   idealreg2spillmask[Op_RegF]->OR(C->FIRST_STACK_mask);
  *idealreg2spillmask[Op_RegD] = *idealreg2regmask[Op_RegD]; 
   idealreg2spillmask[Op_RegD]->OR(C->FIRST_STACK_mask);
  *idealreg2spillmask[Op_RegP] = *idealreg2regmask[Op_RegP]; 
   idealreg2spillmask[Op_RegP]->OR(C->FIRST_STACK_mask);
  *idealreg2spillmask[Op_RegL] = *idealreg2regmask[Op_RegL]; 
   idealreg2spillmask[Op_RegL]->OR(C->FIRST_STACK_mask);

  // Make up debug masks.  Any spill slot plus callee-save registers.
  // Caller-save registers are assumed to be trashable by the various
  // inline-cache fixup routines.
  *idealreg2debugmask[Op_RegI]= *idealreg2spillmask[Op_RegI];
  *idealreg2debugmask[Op_RegF]= *idealreg2spillmask[Op_RegF];
  *idealreg2debugmask[Op_RegD]= *idealreg2spillmask[Op_RegD];
  *idealreg2debugmask[Op_RegP]= *idealreg2spillmask[Op_RegP];
  *idealreg2debugmask[Op_RegL]= *idealreg2spillmask[Op_RegL];

  for( i=OptoReg::Name(0); i<OptoReg::Name(_last_Mach_Reg); i = OptoReg::add(i,1) ) {
    // registers the caller has to save do not work 
    if( _register_save_policy[i] == 'C' ||  
	_register_save_policy[i] == 'A' ) { 
      idealreg2debugmask[Op_RegI]->Remove(i); // Exclude save-on-call 
      idealreg2debugmask[Op_RegF]->Remove(i); // registers from debug
      idealreg2debugmask[Op_RegD]->Remove(i); // masks
      idealreg2debugmask[Op_RegP]->Remove(i); 
      idealreg2debugmask[Op_RegL]->Remove(i); 
    }
  }
}

void Matcher::Fixup_Save_On_Entry( bool save_argument_registers ) {
  init_first_stack_mask();

  Node *root = C->_root;	// Short name for root  
  // Count number of save-on-entry registers.  
  uint soe_cnt = number_of_saved_registers();
  uint i;

  // Find the procedure Start Node
  StartNode *start = C->start();
  assert( start, "Expect a start node" );
  
  // Save argument registers in the trampolining stubs
  if( save_argument_registers ) 
    for( i = 0; i < _last_Mach_Reg; i++ ) 
      if( _can_be_arg[i] ) 
        soe_cnt++;

  // Input RegMask array shared by all Returns.
  uint ret_edge_cnt = C->_tf->_range->_cnt;
  RegMask *ret_rms  = init_input_masks( ret_edge_cnt + soe_cnt, _return_addr_mask, c_frame_ptr_mask );
  // Returns have 0-2 returned values depending on call signature. 
  // Return registers are specified by return_value in the AD file.  
  switch( ret_edge_cnt ) {
  case TypeFunc::Parms+2: ret_rms[TypeFunc::Parms+1] = _return_value_mask[1];
  case TypeFunc::Parms+1: ret_rms[TypeFunc::Parms+0] = _return_value_mask[0];
  case TypeFunc::Parms+0: break;
  default: ShouldNotReachHere();
  }
  
  // Input RegMask array shared by all Rethrows.
  uint reth_edge_cnt = TypeFunc::Parms+1;
  RegMask *reth_rms  = init_input_masks( reth_edge_cnt + soe_cnt, _return_addr_mask, c_frame_ptr_mask );
  // Rethrow takes exception oop only, but in the argument 0 slot.
  reth_rms[TypeFunc::Parms] = mreg2regmask[pd_rethrow_oop_register()];
  
  // Input RegMask array shared by all TailCalls
  uint tail_call_edge_cnt = TypeFunc::Parms+2;
  RegMask *tail_call_rms = init_input_masks( tail_call_edge_cnt + soe_cnt, _return_addr_mask, c_frame_ptr_mask );

  // Input RegMask array shared by all TailJumps
  uint tail_jump_edge_cnt = TypeFunc::Parms+2;
  RegMask *tail_jump_rms = init_input_masks( tail_jump_edge_cnt + soe_cnt, _return_addr_mask, c_frame_ptr_mask );

  // TailCalls have 2 returned values (target & moop), whose masks come
  // from the usual MachNode/MachOper mechanism.  Find a sample
  // TailCall to extract these masks and put the correct masks into
  // the tail_call_rms array.
  for( i=1; i < root->req(); i++ ) {
    MachReturnNode *m = root->in(i)->is_Mach()->is_MachReturn();
    if( m->ideal_Opcode() == Op_TailCall ) {
      tail_call_rms[TypeFunc::Parms+0] = m->MachNode::in_RegMask(TypeFunc::Parms+0);
      tail_call_rms[TypeFunc::Parms+1] = m->MachNode::in_RegMask(TypeFunc::Parms+1);
      break;
    }
  }

  // TailJumps have 2 returned values (target & ex_oop), whose masks come
  // from the usual MachNode/MachOper mechanism.  Find a sample
  // TailJump to extract these masks and put the correct masks into
  // the tail_jump_rms array.
  for( i=1; i < root->req(); i++ ) {
    MachReturnNode *m = root->in(i)->is_Mach()->is_MachReturn();
    if( m->ideal_Opcode() == Op_TailJump ) {
      tail_jump_rms[TypeFunc::Parms+0] = m->MachNode::in_RegMask(TypeFunc::Parms+0);
      tail_jump_rms[TypeFunc::Parms+1] = m->MachNode::in_RegMask(TypeFunc::Parms+1);
      break;
    }
  }

  // Input RegMask array shared by all Halts
  uint halt_edge_cnt = TypeFunc::Parms;
  RegMask *halt_rms = init_input_masks( halt_edge_cnt + soe_cnt, _return_addr_mask, c_frame_ptr_mask );

  // Capture the return input masks into each exit flavor
  for( i=1; i < root->req(); i++ ) {
    MachReturnNode *exit = root->in(i)->is_Mach()->is_MachReturn();
    switch( exit->ideal_Opcode() ) {
      case Op_Return   : exit->_in_rms = ret_rms;  break;
      case Op_Rethrow  : exit->_in_rms = reth_rms; break;
      case Op_TailCall : exit->_in_rms = tail_call_rms; break;
      case Op_TailJump : exit->_in_rms = tail_jump_rms; break;
      case Op_Halt     : exit->_in_rms = halt_rms; break;
      default          : ShouldNotReachHere();
    }
  }

  // Next unused projection number from Start.
  int proj_cnt = C->_tf->_domain->_cnt;  
  
  // Do all the save-on-entry registers.  Make projections from Start for 
  // them, and give them a use at the exit points.  To the allocator, they 
  // look like incoming register arguments.
  for( i = 0; i < _last_Mach_Reg; i++ ) {
    if( _register_save_policy[i] == 'E' ||
	_register_save_policy[i] == 'A' ||	// Save-on-entry register?
        // Also save argument registers in the trampolining stubs
        (save_argument_registers && _can_be_arg[i]) ) {

      // Add the save-on-entry to the mask array
      ret_rms      [      ret_edge_cnt] = mreg2regmask[i];
      reth_rms     [     reth_edge_cnt] = mreg2regmask[i];
      tail_call_rms[tail_call_edge_cnt] = mreg2regmask[i];
      tail_jump_rms[tail_jump_edge_cnt] = mreg2regmask[i];
      // Halts need the SOE registers, but only in the stack as debug info.
      // A just-prior uncommon-trap or deoptimization will use the SOE regs.
      halt_rms     [     halt_edge_cnt] = *idealreg2spillmask[_register_save_type[i]];

      Node *mproj;

      // Is this a RegF low half of a RegD?  Double up 2 adjacent RegF's
      // into a single RegD.
      if( (i&1) == 0 &&
          _register_save_type[i  ] == Op_RegF &&
          _register_save_type[i+1] == Op_RegF ) {
        // Add other bit for double
        ret_rms      [      ret_edge_cnt].Insert(OptoReg::Name(i+1));
        reth_rms     [     reth_edge_cnt].Insert(OptoReg::Name(i+1));
        tail_call_rms[tail_call_edge_cnt].Insert(OptoReg::Name(i+1));
        tail_jump_rms[tail_jump_edge_cnt].Insert(OptoReg::Name(i+1));
        halt_rms     [     halt_edge_cnt].Insert(OptoReg::Name(i+1));
        mproj = new (1) MachProjNode( start, proj_cnt, ret_rms[ret_edge_cnt], Op_RegD );
        proj_cnt += 2;          // Skip 2 for doubles
      } 
      else if( (i&1) == 1 &&    // Else check for high half of double
               _register_save_type[i-1] == Op_RegF &&
               _register_save_type[i  ] == Op_RegF ) {
        ret_rms      [      ret_edge_cnt] = RegMask::Empty;
        reth_rms     [     reth_edge_cnt] = RegMask::Empty;
        tail_call_rms[tail_call_edge_cnt] = RegMask::Empty;
        tail_jump_rms[tail_jump_edge_cnt] = RegMask::Empty;
        halt_rms     [     halt_edge_cnt] = RegMask::Empty;
        mproj = C->_top_node;
      } else {
        // Make a projection for it off the Start
        mproj = new (1) MachProjNode( start, proj_cnt++, ret_rms[ret_edge_cnt], _register_save_type[i] );
      }

      ret_edge_cnt ++;
      reth_edge_cnt ++;
      tail_call_edge_cnt ++;
      tail_jump_edge_cnt ++;
      halt_edge_cnt ++;

      // Add a use of the SOE register to all exit paths
      for( uint j=1; j < root->req(); j++ ) 
        root->in(j)->add_req(mproj); 
    } // End of if a save-on-entry register
  } // End of for all machine registers
}

//------------------------------init_spill_mask--------------------------------
void Matcher::init_spill_mask( Node *ret ) {
  if( idealreg2regmask[Op_RegI] ) return; // One time only init

  // Stack registers start at next register past last machine register
  SharedInfo::stack0 = OptoReg::Name(round_to(_last_Mach_Reg,WordsPerLong));

  SharedInfo::c_frame_pointer = c_frame_pointer();
  c_frame_ptr_mask = c_frame_pointer();

  // Start at SharedInfo::stack0
  STACK_ONLY_mask.Clear();
  OptoReg::Name init = OptoReg::Name(SharedInfo::stack0);
  OptoReg::Name max = OptoReg::Name(RM_SIZE<<5);
  // STACK_ONLY_mask is all stack bits
  OptoReg::Name i;
  for( i = init; i < max; i = OptoReg::add(i,1) )
    STACK_ONLY_mask.Insert(i);

  // Copy the register names over into the shared world
  for( i=OptoReg::Name(0); i<OptoReg::Name(_last_Mach_Reg); i = OptoReg::add(i,1) ) {
    SharedInfo::regName[i] = regName[i];
    // Handy RegMasks per machine register
    mreg2regmask[i].Insert(i);
  }

  // Grab the Frame Pointer
  Node *fp = ret->in(TypeFunc::FramePtr);
  Node *mem= ret->in(TypeFunc::Memory);
  // Compute a generic offset
  ConINode *offset = new (1) ConINode(TypeInt::make(8));
  AddPNode *adr = new (4) AddPNode(C->_top_node,fp,offset);

  // Share frame pointer while making spill ops
  _shared.set(fp->_idx);

  // Get the ADLC notion of the right regmask.
  idealreg2regmask[Op_RegI] = &match_tree(new (3) LoadINode(NULL,mem,adr))->out_RegMask();

  // Get the ADLC notion of the right regmask.
  idealreg2regmask[Op_RegF] = &match_tree(new (3) LoadFNode(NULL,mem,adr))->out_RegMask();

  // Get the ADLC notion of the right regmask.
  idealreg2regmask[Op_RegD] = &match_tree(new (3) LoadDNode(NULL,mem,adr))->out_RegMask();

  // Get the ADLC notion of the right regmask.
  idealreg2regmask[Op_RegP] = &match_tree(new (3) LoadPNode(NULL,mem,adr,TypeOopPtr::BOTTOM))->out_RegMask();

  // Get the ADLC notion of the right regmask.
  idealreg2regmask[Op_RegL] = &match_tree(new (3) LoadLNode(NULL,mem,adr))->out_RegMask();
}

//------------------------------xform------------------------------------------
// Given a Node in old-space, Match him (Label/Reduce) to produce a machine
// Node in new-space.  Given a new-space Node, recursively walk his children.
Node *Matcher::transform( Node *n ) { ShouldNotCallThis(); return n; }
Node *Matcher::xform( Node *n ) {
  // Old-space or new-space check
  if( !C->_arena.contains(n) ) {
    // Old space!
    Node *m = _nodes[n->_idx];	// Get x-formed new-space equivalent
    if( !m ) {			// Not yet Label/Reduced
      if( !_dontcare.test(n->_idx)) {// Matcher can match this guy
	// Calls match special.  They match alone with no children.
	// Their children, the incoming arguments, match normally.
	SafePointNode *sfpt = n->is_SafePoint();
	m = sfpt ? match_sfpt(sfpt) : match_tree(n);
        if (SoftMatchFailure && m == NULL) return NULL;
      } else {			// Nothing the matcher cares about
	ProjNode *proj = n->is_Proj();
	if( proj ) {		// Projections?
	  // Convert to machine-dependent projection
	  m = proj->in(0)->is_Multi()->match( proj, this );
          collect_null_checks(m);
	} else {		// Else just a regular 'ol guy
	  m = n->clone();	// So just clone into new-space
	}
      }

      _nodes.map(n->_idx,m);	// Map old to new
    }
    n = m;			// n is now a new-space node
  }

  // New space!
  if( _visited.test_set(n->_idx) ) return n;

  // Now walk his children, and convert his inputs to new-space.
  // For constant debug info, I'd rather have unmatched constants.
  uint cnt = n->req();
  JVMStateObj *jvms = n->jvms();
  uint debug_cnt = jvms ? jvms->locoff() : cnt;

  uint i;
  for( i=0; i<debug_cnt; i++ ) { // For all normal inputs do
    Node *m = n->in(i);		// Get input
    if( !m ) continue;		// 
    m = xform(m);
    if( SoftMatchFailure && !m ) return NULL;
    n->set_req(i, m);
  }

  // Now do only debug info.  Clone constants rather than matching.
  // Constants are represented directly in the debug info without
  // the need for executable machine instructions.
  // Monitor boxes are also represented directly.
  for( ; i<cnt; i++ ) {         // For all debug inputs do
    Node *m = n->in(i);		// Get input
    int op = m->Opcode();
    assert((op == Op_BoxLock) == jvms->is_monitor_use(i), "boxes only at monitor sites");
    if( op == Op_ConI || op == Op_ConP ||
        op == Op_ConF || op == Op_ConD || op == Op_ConL
        // || op == Op_BoxLock  // %%%% enable this and remove (+++) in chaitin.cpp
	) {
      m = m->clone();
      m->set_req(0,xform(m->in(0)));
    } else m = xform(m);
    if( SoftMatchFailure && !m ) return NULL;
    n->set_req(i, m);
  }

  return n;			// Return new-space Node
}

//------------------------------match_sfpt-------------------------------------
// Helper function to match call instructions.  Calls match special.
// They match alone with no children.  Their children, the incoming
// arguments, match normally.
MachNode *Matcher::match_sfpt( SafePointNode *sfpt ) {
  MachSafePointNode *msfpt = NULL;
  MachCallNode      *mcall = NULL;
  uint               cnt;
  // Split out case for SafePoint vs Call
  CallNode *call = sfpt->is_Call();
  const TypeTuple *domain = (call != NULL) ? call->_func->_domain : NULL;
  if( call != NULL ) {
    cnt = domain->_cnt;

    // Match just the call, nothing else
    MachNode *m = match_tree(call);
    if (SoftMatchFailure && m == NULL) return NULL;

    // Copy data from the Ideal SafePoint to the machine version
    MachNode *mtmp = m->is_Mach();
    mcall = m->is_Mach()->is_MachCall();

    mcall->_func = call->_func;
    mcall->_entrypoint = call->_entrypoint;
  
    MachCallRuntimeNode     *mcall_runtime = mcall->is_MachCallRuntime();
    MachCallJavaNode        *mcall_java    = mcall->is_MachCallJava();
    if( mcall_java ) {
      const CallJavaNode *call_java = call->is_CallJava();
      mcall_java->_method = call_java->_method;
      mcall_java->_bci = call_java->_bci;
      mcall_java->_optimized_virtual = call_java->is_optimized_virtual(); 
      MachCallStaticJavaNode *mcalls_java = mcall_java->is_MachCallStaticJava();
      if( mcalls_java ) 
	mcalls_java->_name = call_java->is_CallStaticJava()->_name;
    } 
    else if( mcall_runtime ) {
      const CallRuntimeNode *call_runtime = call->is_CallRuntime();
      mcall_runtime->_name = call_runtime->_name;
    }
    msfpt = mcall;
  }
  // This is a non-call safepoint
  else {
    MachNode *mn = match_tree(sfpt);
    msfpt = mn->is_MachSafePoint();
    cnt = TypeFunc::Parms;
  }

  // Allocate a private array of RegMasks.  These RegMasks are not shared.
  msfpt->_in_rms = NEW_RESOURCE_ARRAY( RegMask, cnt );
  // Empty them all.
  memset( msfpt->_in_rms, 0, sizeof(RegMask)*cnt );

  // Do all the pre-defined non-Empty register masks
  msfpt->_in_rms[TypeFunc::ReturnAdr] = _return_addr_mask;
  msfpt->_in_rms[TypeFunc::FramePtr ] = c_frame_ptr_mask;

  // Do the normal argument list (parameters) register masks
  int argcnt = cnt - TypeFunc::Parms;
  if( argcnt > 0 ) {          // Skip it all if we have no args
    uint16 *parm_regs = NEW_RESOURCE_ARRAY( uint16, argcnt );
    int i;
    for( i = 0; i < argcnt; i++ ) {
      parm_regs[i] = base2reg[domain->_fields[i+TypeFunc::Parms]->_base];
    }
    // V-call to pick proper calling convention
    call->calling_convention( parm_regs, argcnt );

#ifdef ASSERT
    // Sanity check users' calling convention.  Really handy during 
    // the initial porting effort.
    { for (int i = 0; i<argcnt; i++) {
        for (int j = 0; j < i; j++) {
          assert(parm_regs[i] != parm_regs[j], "calling conv. must produce distinct regs");
        }
      }
    }
#endif

    // Place first outgoing argument can possibly be put.
    OptoReg::Name begin_out_arg_area = OptoReg::add(_new_SP, C->out_preserve_stack_slots());
    assert( is_even(begin_out_arg_area), "" );
    // Compute max outgoing register number per call site.  
    OptoReg::Name out_arg_limit_per_call = begin_out_arg_area;
    // Calls to C may hammer extra stack slots above and beyond any arguments.
    // These are usually backing store for register arguments for varargs.
    if( call->is_CallRuntime() )
      out_arg_limit_per_call = OptoReg::add(out_arg_limit_per_call,C->varargs_C_out_slots_killed());

    // Visit each argument.  Compute its outgoing register mask.
    // Compute max over all outgoing arguments both per call-site
    // and over the entire method.
    for( i = 0; i < argcnt; i++ ) {
      // Convert outgoing argument location to a pre-biased stack offset
      int stk_slot_num = parm_regs[i] - SharedInfo::stack0;
      OptoReg::Name out_reg = OptoReg::Name(parm_regs[i]);
      if( stk_slot_num >= 0 ) { // Check for being in stack or register
        // Adjust the stack slot offset to be the register number used
        // by the allocator.
        parm_regs[i] = out_reg = OptoReg::add(begin_out_arg_area,stk_slot_num);
        // Keep track of the largest numbered stack slot used for an arg.
        // Largest used slot per call-site indicates the amount of stack
        // that is killed by the call.
        if( out_reg >= out_arg_limit_per_call ) 
          out_arg_limit_per_call = OptoReg::add(out_reg,1);
      } 
      // Address of incoming argument mask to fill in
      RegMask *rm = &mcall->_in_rms[i+TypeFunc::Parms];
      // Else fill in a register mask
      rm->Insert(out_reg);

      // Doubles need 2 bits in the first argument word and nothing in the 2nd.
      // If the double is aligned or we can directly load misaligned doubles
      // with little extra cost we leave the argument alone.  Otherwise it
      // got split into high and low pieces.
      if( i > 0 && 
	  (domain->_fields[i-1+TypeFunc::Parms] == Type::DOUBLE ||
	   domain->_fields[i-1+TypeFunc::Parms] == TypeLong::LONG) ) {
        assert(domain->_fields[i+TypeFunc::Parms] == Type::HALF, "");
        // Was this double split into 2 ints?
	const Type *t2 = mcall->in(i+TypeFunc::Parms)->bottom_type();
	// It is unsplit if and only if the second word is TOP or HALF.
        if( t2 == Type::TOP || t2 == Type::HALF ) {
          // Add 2nd half to lower regmask
          mcall->_in_rms[i-1+TypeFunc::Parms].Insert( out_reg );
          // Clear out 2nd half regmask
          rm->Remove(out_reg);
          assert( mcall->_in_rms[i-1+TypeFunc::Parms].is_bound2(), "" );
        } else {
	  // The outgoing double value is split into two non-paired registers.
#ifdef ASSERT
     	  Node* n0 = mcall->in(i-1+TypeFunc::Parms);
 	  Node* n1 = mcall->in(i  +TypeFunc::Parms);
 	  assert(n0->Opcode() == Op_DoubleHalf0 || n0->Opcode() == Op_LongHalf0 || n0->is_Con() || n0->is_Phi(), "");
 	  assert(n1->Opcode() == Op_DoubleHalf1 || n1->Opcode() == Op_LongHalf1 || n1->is_Con() || n1->is_Phi(), "");
#endif
        }
      } // End of special misaligned double handling
    } // End of for all arguments

    // Compute the max stack slot killed by any call.  These will not be 
    // available for debug info, and will be used to adjust FIRST_STACK_mask 
    // after all call sites have been visited.
    if( out_arg_limit_per_call > _out_arg_limit ) 
      _out_arg_limit = out_arg_limit_per_call;

    // Compute number of stack slots needed to restore stack in case of
    // Pascal-style argument popping.
    mcall->_argsize = out_arg_limit_per_call - begin_out_arg_area;

    // Kill the outgoing argument area, including any non-argument holes and
    // any legacy C-killed slots.  Use Fat-Projections to do the killing.
    // Since the max-per-method covers the max-per-call-site and debug info
    // is excluded on the max-per-method basis, debug info cannot land in
    // this killed area.
    uint r_cnt = mcall->_func->_range->_cnt;
    MachProjNode *proj = new (1) MachProjNode( mcall, r_cnt+10000, RegMask::Empty, MachProjNode::fat_proj );
    for( i = begin_out_arg_area; i < out_arg_limit_per_call; i++ ) 
      proj->_out.Insert(OptoReg::Name(i));
    if( proj->_out.is_NotEmpty() )
      _proj_list.push(proj);
  }

  // Transfer the safepoint information from the call to the mcall
  // Move the JVMStateObj list
  msfpt->_jvms = sfpt->_jvms;
  msfpt->_jvmtail = sfpt->_jvmtail;
  for( JVMStateObj *j = msfpt->_jvms; j; j = j->next() )
    j->set_map(sfpt);

  // Debug inputs begin just after the last incoming parameter
  assert( (mcall == NULL) || (mcall->_jvms == NULL) ||
          (mcall->_jvms->locoff() + mcall->_jvmadj == mcall->_func->_domain->_cnt), "" );

  // Move the OopMap
  msfpt->_oop_map = sfpt->_oop_map;

  // Registers killed by the call are set in the local scheduling pass
  // of Global Code Motion.
  return msfpt;
}

//---------------------------match_tree----------------------------------------
// Match a Ideal Node DAG - turn it into a tree; Label & Reduce.  Used as part
// of the whole-sale conversion from Ideal to Mach Nodes.  Also used for
// making GotoNodes while building the CFG and CopyNodes during spilling.
MachNode *Matcher::match_tree( const Node *n ) {
  assert( n->Opcode() != Op_Phi, "cannot match" );
  assert( !n->is_block_start(), "cannot match" );
  // Set the mark for all locally allocated State objects.
  // When this call returns, the _states arena will be reset
  // freeing all State objects.
  ResourceMark rm( &_states );

  // StoreNodes require their Memory input to match any LoadNodes
  Node *mem = n->is_Store() ? n->in(MemNode::Memory) : (Node*)1 ;

  // State object for root node of match tree
  State s;
  s._kids[0] = s._kids[1] = NULL;
  s._leaf = (Node*)n;
  // Label the input tree, allocating labels from top-level arena
  Label_Root( n, &s, n->in(0), mem );
  
  // The minimum cost match for the whole tree is found at the root State
  uint mincost = 0xFFFFFFFF;
  uint cost = 0xFFFFFFFF;
  uint i;
  for( i = 0; i < NUM_OPERANDS; i++ ) {
    if( s.valid(i) &&                // valid entry and
        s._cost[i] < cost &&         // low cost and 
	s._rule[i] >= NUM_OPERANDS ) // not an operand
      cost = s._cost[mincost=i];
  }
  if (SoftMatchFailure && (mincost == 0xFFFFFFFF)) {
    tty->print("No matching rule for:");
    s.dump();
    return NULL;
  }

  // Reduce input tree based upon the state labels to machine Nodes
  MachNode *m = ReduceInst( &s, s._rule[mincost], mem );
#ifdef ASSERT
  _old2new_map.map(n->_idx, m);
#endif
  
  // Add any Matcher-ignored edges
  uint cnt = n->req();
  uint start = 1;
  if( mem != (Node*)1 ) start = MemNode::Memory+1;
  if( n->Opcode() == Op_AddP ) {
    assert( mem == (Node*)1, "" );
    start = AddPNode::Base+1;
  }
  for( i = start; i < cnt; i++ ) 
    if( !n->match_edge(i) )
      m->ins_req( i, n->in(i) );	
  

  return m;
}

//------------------------------Instruction Selection--------------------------
// Label method walks a "tree" of nodes, using the ADLC generated DFA to match
// ideal nodes to machine instructions.  Trees are delimited by shared Nodes,
// things the Matcher does not match (e.g., Memory), and things with different
// Controls (hence forced into different blocks).  We pass in the Control 
// selected for this entire State tree.

// The Matcher works on Trees, but an Intel add-to-memory requires a DAG: the
// Store and the Load must have identical Memories (as well as identical 
// pointers).  Since the Matcher does not have anything for Memory (and 
// does not handle DAGs), I have to match the Memory input myself.  If the
// Tree root is a Store, I require all Loads to have the identical memory.
Node *Matcher::Label_Root( const Node *n, State *svec,Node *control,Node *mem){
  uint care = 0;		// Edges matcher cares about
  uint cnt = n->req();
  int n_op = n->Opcode();
  uint i = 0;

  // Examine children for memory state
  // Can only subsume a child into your match-tree if that child's memory state
  // is not modified along the path to another input.  
  // It is unsafe even if the other inputs are separate roots.
  Node *input_mem = NULL;
  for( i = 1; i < cnt; i++ ) { 
    if( !n->match_edge(i) ) continue;
    Node *m = n->in(i);		// Get ith input
    if( m->is_Load() ) {
      Node *mem = m->in(MemNode::Memory);
      if( input_mem == NULL ) {
        input_mem = mem;
      } else if( input_mem != mem ) { 
        input_mem = unmatchable_mem;
      }
    }
  }

  for( i = 1; i < cnt; i++ ){// For my children
    if( !n->match_edge(i) ) continue;
    Node *m = n->in(i);		// Get ith input
    assert( m, "expect non-null children" );
    // Allocate states out of a private arena
    State *s = svec->_kids[care++] = new (&_states) State;
    assert( care <= 2, "binary only for now" );

    // Recursively label the State tree.
    s->_kids[0] = s->_kids[1] = NULL;
    s->_leaf = (Node*)m;
    
    // Check for leaves of the State Tree; things that cannot be a part of 
    // the current tree.  If it finds any, that value is matched as a 
    // register operand.  If not, then the normal matching is used.
    if( _shared.test(m->_idx) || 	// Shared; stop recursion
        // Also stop recursion if they have different Controls.
        // Slot 0 of constants is not really a Control.
        (control && m->in(0) && control != m->in(0) && !m->is_Con()) ||
	// 
	// Stores should share the constant being stored.  Don't share the
	// zero, it's cheaper on Sparc to use G0 (and worse on Intel!)
	(i == MemNode::ValueIn && n->is_Store() && m->bottom_type() != TypeInt::ZERO && m->is_Con()) ||
        // 
        // Finally, stop recursion if this a LoadNode and the root of
        // this tree is a StoreNode and the load & store have different
        // memories.
        ((mem!=(Node*)1) && m->is_Load() && m->in(MemNode::Memory) != mem) ||
        // Can NOT include the match of a subtree when its memory state
        // is used by any of the other subtrees
        (input_mem == unmatchable_mem) ) {
      // Print when we exclude matching due to different memory states at input-loads
      if( PrintOpto && (Verbose && WizardMode) && (input_mem == unmatchable_mem) 
        && !((mem!=(Node*)1) && m->is_Load() && m->in(MemNode::Memory) != mem) ) { 
        tty->print_cr("invalid input_mem"); 
      }
      // Switch to a register-only opcode; this value must be in a register
      // and cannot be subsumed as part of a larger instruction.
      s->DFA( m->ideal_reg(), m );
    
    } else {
      // If match tree has no control and we do, adopt it for entire tree
      if( !control && m->in(0) && m->req() > 1 )
        control = m->in(0);         // Pick up control
      // Else match as a normal part of the match tree.
      control = Label_Root(m,s,control,mem);
    }
  }

  
  // Call DFA to match this node, and return
  bool success = svec->DFA( n_op, n );

#ifdef ASSERT
  uint x;
  for( x = 0; x < _LAST_MACH_OPER; x++ )
    if( svec->valid(x) )
      break;

  if (!  (x < _LAST_MACH_OPER))  svec->dump();
  assert( x < _LAST_MACH_OPER, "bad AD file" );
#endif
  return control;
}


//------------------------------ReduceInst-------------------------------------
// Reduce a State tree (with given Control) into a tree of MachNodes.
// This routine (and it's cohort ReduceOper) convert Ideal Nodes into 
// complicated machine Nodes.  Each MachNode covers some tree of Ideal Nodes.
// Each MachNode has a number of complicated MachOper operands; each 
// MachOper also covers a further tree of Ideal Nodes.  

// The root of the Ideal match tree is always an instruction, so we enter
// the recursion here.  After building the MachNode, we need to recurse
// the tree checking for these cases:
// (1) Child is an instruction - 
//     Build the instruction (recursively), add it as an edge.
//     Build a simple operand (register) to hold the result of the instruction.
// (2) Child is an interior part of an instruction -
//     Skip over it (do nothing)
// (3) Child is the start of a operand -
//     Build the operand, place it inside the instruction
//     Call ReduceOper.
MachNode *Matcher::ReduceInst( State *s, int rule, Node *&mem ) {
  assert( rule >= NUM_OPERANDS, "called with operand rule" );

  if (rule == _PSEUDO_RULE_FOR_PRESET) {
    MachNode* m = s->_leaf->is_Mach();
    assert(m != NULL, "bad preset");
    return m;
  }

  // Build the object to represent this state & prepare for recursive calls
  MachNode *mach = s->MachNodeGenerator( rule );
  Node *leaf = s->_leaf;
  // Check for instruction or instruction chain rule
  if( rule >= _END_INST_CHAIN_RULE || rule < _BEGIN_INST_CHAIN_RULE ) {
    // Instruction
    mach->add_req( s->_leaf->in(0) );// Set initial control
    mach->_opnds[0] = s->MachOperGenerator( _reduceOp[rule], mach );
    assert( mach->_opnds[0], "Missing result operand" );
    // Reduce interior of complex instruction
    ReduceInst_Interior( s, rule, mach, 1, mem );
  } else {
    // Instruction chain rules are data-dependent on their inputs
    mach->add_req(0);                   // Set initial control to none
    mach->_opnds[0] = s->MachOperGenerator( _reduceOp[rule], mach );
    assert( mach->_opnds[0], "Missing result operand" );
    ReduceInst_Chain_Rule( s, rule, mach, mem );
  }

  // If a Memory was used, insert a Memory edge
  if( mem != (Node*)1 )
    mach->ins_req(MemNode::Memory,mem);

  // If the _leaf is an AddP, insert the base edge
  if( leaf->Opcode() == Op_AddP )
    mach->ins_req(AddPNode::Base,leaf->in(AddPNode::Base));

  int num_proj = _proj_list.size();

  // Perform any 1-to-many expansions required
  MachNode *ex = mach->Expand(s,_proj_list);
  if( ex != mach ) {
    if( ex->in(1)->is_Con() )
      ex->in(1)->set_req(0,C->_root);
  }

  // PhaseChaitin::fixup_spills will sometimes generate spill code
  // via the matcher.  By the time, nodes have been wired into the CFG,
  // and any further nodes generated by expand rules will be left hanging
  // in space, and will not get emitted as output code.  Catch this.
  // Also, catch any new register allocation constraints ("projections")
  // generated belatedly during spill code generation.
  if (_allocation_started) {
    guarantee(ex == mach, "no expand rules during spill generation");
    int new_proj = _proj_list.size() - num_proj;
    guarantee(new_proj == 0, "no allocation during spill generation");
  }

  return ex;
}

void Matcher::ReduceInst_Chain_Rule( State *s, int rule, MachNode *mach, Node *&mem ) {
  if( s->_leaf->is_Load() ) {
    // !!!!! !!!!!
    // tty->print("\nChain Rule %s on result of leaf: ",ruleName[rule]);
    // s->_leaf->dump();
    // tty->print_cr("");
  }

  // 'op' is what I am expecting to receive
  int op = _leftOp[rule];
  // Operand type to catch childs result
  // This is what my child will give me.
  int opnd_class_instance = s->_rule[op];
  // Choose between operand class or not.
  // This is what I will recieve.
  int catch_op = (FIRST_OPERAND_CLASS <= op && op < NUM_OPERANDS) ? opnd_class_instance : op;
  // New rule for child.  Chase operand classes to get the actual rule.
  int newrule = s->_rule[catch_op];

  if( newrule < NUM_OPERANDS ) {
    // Chain from operand or operand class, may be output of shared node
    assert( 0 <= opnd_class_instance && opnd_class_instance < NUM_OPERANDS,
            "Bad AD file: Instruction chain rule must chain from operand");
    MachOper *oper = s->MachOperGenerator( opnd_class_instance, mach);
    // Insert operand into array of operands for this instruction
    mach->_opnds[1] = oper;

    ReduceOper( s, newrule, mach, mem );
    // delete s;  // Done by ReduceOper
  } else {
    // Chain from the result of an instruction
    assert( newrule >= _LAST_MACH_OPER, "Do NOT chain from internal operand");
    MachOper *oper = s->MachOperGenerator( _reduceOp[catch_op], mach);
    mach->_opnds[1] = oper;
    Node *mem = (Node*)1;
    mach->add_req( ReduceInst(s, newrule, mem) );
    // delete s;  // Done by ReduceInst_Interior
  }
  return;
}


uint Matcher::ReduceInst_Interior( State *s, int rule, MachNode *mach, uint num_opnds, Node *&mem ) {
  if( s->_leaf->is_Load() ) {
    Node *mem2 = s->_leaf->in(MemNode::Memory);
    assert( mem == (Node*)1 || mem == mem2, "multiple Memories being matched at once?" );
    mem = mem2;
  }
  if( s->_leaf->in(0) && s->_leaf->req() > 1) {
    if( !mach->in(0) ) 
      mach->set_req(0,s->_leaf->in(0));
    else {
      assert( s->_leaf->in(0) == mach->in(0), "same instruction, differing controls?" );
    }
  }

  // Now recursively walk the state tree & add operand list.
  for( uint i=0; i<2; i++ ) {	// binary tree
    State *newstate = s->_kids[i];
    if( !newstate ) break;	// Might only have 1 child
    // 'op' is what I am expecting to receive
    int op = (i?_rightOp:_leftOp)[rule];
    // Operand type to catch childs result
    // This is what my child will give me.
    int opnd_class_instance = newstate->_rule[op];
    // Choose between operand class or not.
    // This is what I will receive.
    int catch_op = (op >= FIRST_OPERAND_CLASS && op < NUM_OPERANDS) ? opnd_class_instance : op;
    // New rule for child.  Chase operand classes to get the actual rule.
    int newrule = newstate->_rule[catch_op];

    if( newrule < NUM_OPERANDS ) { // Operand/operandClass or internalOp/instruction?
      // Operand/operandClass
      MachOper *oper = newstate->MachOperGenerator( opnd_class_instance, mach);
      // Insert operand into array of operands for this instruction
      mach->_opnds[num_opnds++] = oper;
      ReduceOper( newstate, newrule, mach, mem );

    } else {			// Child is internal operand or new instruction
      if( newrule < _LAST_MACH_OPER ) { // internal operand or instruction?
	// internal operand --> call ReduceInst_Interior
	// Interior of complex instruction.  Do nothing but recurse.
	num_opnds = ReduceInst_Interior( newstate, newrule, mach, num_opnds, mem );
      } else {
	// instruction --> call build operand(  ) to catch result
	//             --> ReduceInst( newrule )
	mach->_opnds[num_opnds++] = s->MachOperGenerator( _reduceOp[catch_op], mach );
	Node *mem = (Node*)1;
	mach->add_req( ReduceInst( newstate, newrule, mem ) );
      }
    }
    assert( mach->_opnds[num_opnds-1], "" );
  }
  delete s;
  return num_opnds;
}

// This routine walks the interior of possible complex operands.
// At each point we check our children in the match tree:
// (1) No children -
//     We are a leaf; add _leaf field as an input to the MachNode
// (2) Child is an internal operand -
//     Skip over it ( do nothing )
// (3) Child is an instruction -
//     Call ReduceInst recursively and
//     and instruction as an input to the MachNode
void Matcher::ReduceOper( State *s, int rule, MachNode *mach, Node *&mem ) {
  assert( rule < _LAST_MACH_OPER, "called with operand rule" );
  assert( !s->_kids[0] || !s->_leaf->in(0), "internal operands have no control" );

  // Leaf?  And not subsumed?
  if( !s->_kids[0] && !_swallowed[rule] ) {
    mach->add_req( s->_leaf );	// Add leaf pointer
    delete s;
    return;			// Bail out
  }

  if( s->_leaf->is_Load() ) {
    assert( mem == (Node*)1, "multiple Memories being matched at once?" );
    mem = s->_leaf->in(MemNode::Memory);
  }
  if( s->_leaf->in(0) && s->_leaf->req() > 1) {
    if( !mach->in(0) ) 
      mach->set_req(0,s->_leaf->in(0));
    else {
      assert( s->_leaf->in(0) == mach->in(0), "same instruction, differing controls?" );
    }
  }

  for( uint i=0; i<2; i++ ) {	// binary tree
    if( !s->_kids[i] ) break;	// Might be only 1 child
    int op = (i?_rightOp:_leftOp)[rule];
    int newrule = s->_kids[i]->_rule[op];
    if( newrule < _LAST_MACH_OPER ) { // Operand or instruction?
      // Internal operand; recurse but do nothing else
      ReduceOper( s->_kids[i], newrule, mach, mem );

    } else {			// Child is a new instruction
      // Reduce the instruction, and add a direct pointer from this
      // machine instruction to the newly reduced one.
      Node *mem = (Node*)1;
      mach->add_req( ReduceInst( s->_kids[i], newrule, mem ) );
    }
  }
  delete s;
}


// -------------------------------------------------------------------------
// Java-Java calling convention
// (what you use when Java calls Java)

//------------------------------find_receiver----------------------------------
// For a given signature, return the OptoReg for parameter 0.
// Note that we have to construct the entire array of ideal registers, as the
// calling convention might be weird: arg 0 might go in a register for some
// signatures and elsewhere for other signatures.
VMReg::Name Matcher::find_receiver(symbolOop sig) {
  ResourceMark rm;
  int arg_size;
  uint16 *regs = find_callee_arguments(sig, false, &arg_size);
  assert(arg_size > 0, "there must at least be a receiver");
  // Return argument 0 register  
  return VMReg::Name(regs[0]);
}

uint16 *Matcher::find_callee_arguments(symbolOop sig, bool is_static, int* arg_size) {  
  // This method is returning a data structure allocating as a
  // ResourceObject, so do not put any ResourceMarks in here.
  char *s = sig->as_C_string();
  int len = strlen(s);
  *s++; len--;			// Skip opening paren
  char *t = s+len;
  while( *(--t) != ')' ) ;	// Find close paren

  uint16 *regs = NEW_RESOURCE_ARRAY( uint16, 256 );
  int cnt = 0;
  if (!is_static) {    
    regs[cnt++] = Op_RegP;	// Receiver is argument 0; not in signature
  }

  while( s < t ) {
    switch( *s++ ) {		// Switch on signature character
    case 'B': regs[cnt++] = Op_RegI; break;
    case 'C': regs[cnt++] = Op_RegI; break;
    case 'D': regs[cnt++] = Op_RegD; regs[cnt++] = 0; break;
    case 'F': regs[cnt++] = Op_RegF; break;
    case 'I': regs[cnt++] = Op_RegI; break;
    case 'J': regs[cnt++] = Op_RegL; regs[cnt++] = 0; break;
    case 'S': regs[cnt++] = Op_RegI; break;
    case 'Z': regs[cnt++] = Op_RegI; break;
    case 'V': regs[cnt++] = 0;       break;
    case 'L':			// Oop
      while( *s++ != ';'  ) ;	// Skip signature
      regs[cnt++] = Op_RegP; 
      break;
    case '[': {			// Array
      do {			// Skip optional size
	while( *s >= '0' && *s <= '9' ) s++;
      } while( *s++ == '[' );	// Nested arrays?
      // Skip element type
      if( s[-1] == 'L' )
	while( *s++ != ';'  ) ;	// Skip signature
      regs[cnt++] = Op_RegP; 
      break;
    }
    default : ShouldNotReachHere();
    }
  }
  assert( cnt < 256, "grow table size" );

  // Pass array of ideal registers and length to USER code (from the AD file)
  // that will convert this to an array of register numbers.
  // The "true" means report arguments from the caller's point of view.
  // The name "callee arguments" means arguments to the current
  // frame's call, but named according the caller's point of view.
  calling_convention(regs,cnt,true);

  for (int i = 0; i < cnt; i++) {
    if( regs[i] >= (uint16)SharedInfo::stack0 )  // Stack slot argument?
      regs[i] = OptoReg::add(OptoReg::Name(regs[i]), Compile::out_preserve_stack_slots());
  }

  // results
  *arg_size = cnt;
  return regs;
}

// A method-klass-holder may be passed in the inline_cache_reg
// and then expanded into the inline_cache_reg and a method_oop register
//   defined in ad_<arch>.cpp



//------------------------------set_shared-------------------------------------
// Set bits if Node is shared or otherwise a root
void Matcher::set_shared( Node *n ) {
  _visited.set(n->_idx);	// Flag as visited now
  bool mem_op = false;
  
  switch( n->Opcode() ) {	// Handle some opcodes special
  case Op_AtomicAdd:            // fall through: needs memory edge
    _shared.set(n->_idx);
    break;
  case Op_Phi:			// Treat Phis as shared roots
  case Op_Parm:
  case Op_Proj:			// All handled specially during matching
    _shared.set(n->_idx);
    _dontcare.set(n->_idx);
    break;
  case Op_CastIP:               // Convert pointers to ints for store barriers 
    _shared.set(n->_idx);       // It cannot be matched because it is a 2-adr  
    _dontcare.set(n->_idx);     // op with different types for src & dst       
    break;
    // Convert (If (Bool (CmpX A B))) into (If (Bool) (CmpX A B)).  Helps
    // with matching cmp/branch in 1 instruction.  The Matcher needs the Bool
    // and CmpX side-by-side, because it can only get at constants that are
    // at the leaves of Match trees, and the Bool's condition acts as a
    // constant here.  
  case Op_If:
  case Op_CountedLoopEnd:
    set_if_shared(n->in(0));	// Visit control input
    set_shared(n->in(1));	// Clone the Bool

    // We cannot remove the Cmp input from the Bool here, as the Bool may be
    // shared and all users of the Bool need to move the Cmp in parallel.
    // This leaves both the Bool and the If pointing at the Cmp.  To 
    // prevent the Matcher from trying to Match the Cmp along both paths
    // BoolNode::match_edge always returns a zero.

    // We reorder the Op_If in a pre-order manner, so we can visit without
    // accidently sharing the Cmp (the Bool and the If make 2 users).
    n->add_req( n->in(1)->in(1) ); // Add the Cmp next to the Bool
    return;
  case Op_Box:			// Cant match until we get stack-regs in ADLC
  case Op_BoxLock:	        // Cant match until we get stack-regs in ADLC
  case Op_IfFalse:
  case Op_IfTrue:
  case Op_MachProj:
  case Op_MergeMem:
  case Op_Catch:
  case Op_CatchProj:
  case Op_Jump:
  case Op_CProj:
  case Op_RethrowLocal:
  case Op_NeverBranch:
    _dontcare.set(n->_idx);
    break;
  case Op_StrComp:
    _shared.set(n->_idx);	// Force result into register (it will be anyways)
    break;
  case Op_ConP: {               // Convert pointers above the centerline to NUL
    TypeNode *tn = n->is_Type(); // Constants derive from type nodes
    assert( tn->_type != NULL, "insanity check");
    const TypePtr *tp = tn->_type->is_ptr();
    if( tp->_ptr == TypePtr::AnyNull ) {
      tn->_type = TypePtr::NULL_PTR;
    }
    break;
  }
  case Op_CMovePair:		// Match these, despite no ideal reg
    break;
  case Op_Store:		// Do match these, despite no ideal reg
  case Op_StoreB:
  case Op_StoreC:
  case Op_StoreD:
  case Op_StoreF:
  case Op_StoreI:
  case Op_StoreL:
  case Op_StoreP:
  case Op_ClearArray:
  case Op_SafePoint:
    mem_op = true;
    break;
  case Op_Load:
  case Op_LoadB:
  case Op_LoadC:
  case Op_LoadD:
  case Op_LoadF:
  case Op_LoadI:
  case Op_LoadKlass:
  case Op_LoadL:
  case Op_LoadS:
  case Op_LoadP:
  case Op_LoadRange:
    mem_op = true;
    // Must be root of match tree due to prior load conflict
    if( C->subsume_loads() == false ) {
      _shared.set(n->_idx);
    }
    // Fall into default case
  default:
    if( !n->ideal_reg() )
      _dontcare.set(n->_idx);	// Unmatchable Nodes
  } // end_switch

  uint cnt = n->req();
  for( uint i=0; i<cnt; i++ ) { // For my children
    Node *m = n->in(i);		// Get ith input
    if( !m ) continue;		// Ignore NULLs
    uint mop = m->Opcode();

    // Must clone all producers of flags, or we will not match correctly.
    // Suppose a compare setting int-flags is shared (e.g., a switch-tree)
    // then it will match into an ideal Op_RegFlags.  Alas, the fp-flags
    // are also there, so we may match a float-branch to int-flags and
    // expect the allocator to haul the flags from the int-side to the
    // fp-side.  No can do.
    if( _must_clone[mop] ) {
      set_shared(m);
      continue;
    } 

    // Nearly always clone integer constants: e.g., add+1 or
    // shift-by-2.  Cloning the constant means we can match it
    // directly into the machine instruction instead of loading the
    // constant into a register and use a reg/reg operation.

    // Also clone the NULL pointer.  It has a short form on most chips
    // (think: test vs NULL).  Do NOT clone other raw pointer
    // constants, like the card-mark and heap-top.  Clone the zero Long
    // and the Zero float & double.
    const Type *t = m->bottom_type();
    if( t == TypeInt::ZERO ||	// Always clone zero constants
	t == TypeLong::ZERO ||
	t == TypePtr::NULL_PTR ||
	t == TypeF::ZERO ||
	t == TypeD::ZERO ||
	mop == Op_ConI ) {	// Clone all integer constants
      set_shared(m);
      continue;
    }

    // Clone addressing expressions as they are "free" in most instructions
    if( mem_op && i == MemNode::Address && mop == Op_AddP ) {
      Node *adr = m->in(AddPNode::Address);
      Node *off = m->in(AddPNode::Offset);
      if( off->is_Con() ) {
        _visited.set(m->_idx);	// Flag as visited now
        set_if_shared( m->in(AddPNode::Base) );
        // Clone X+offset as it also folds into most addressing expressions
        set_shared( m->in(AddPNode::Offset) );
        // Intel, ARM and friends can handle 2 adds in addressing mode
        if( clone_shift_expressions && adr->Opcode() == Op_AddP &&
            // AtomicAdd is not an addressing expression.
            // Cheap to find it by looking for screwy base.
            adr->in(AddPNode::Base) != C->_top_node ) {
          _visited.set(adr->_idx);	// Flag as visited now
          set_if_shared( adr->in(AddPNode::Base   ) );
          set_if_shared( adr->in(AddPNode::Address) );
          Node *shift = adr->in(AddPNode::Offset);
          // Check for shift by small constant as well
          if( shift->Opcode() == Op_LShiftI && shift->in(2)->is_Con() && 
              shift->in(2)->get_int() <= 3 ) { 
            _visited.set(shift->_idx);	// Flag as visited now
            set_if_shared( shift->in(1) );
            set_shared( shift->in(2) );
          } else {
            set_if_shared( shift );
          }
        } else {                        // Sparc, Alpha, PPC and friends
          set_if_shared( adr );
        }
        continue;
      }
    }
    // Check for 'm' being shared or not.  This is an inlined version of
    // set_if_shared; inlined to reduce stack depth during deep recursion
    // when walking deep graphs.
    if( _visited.test(m->_idx) )   // Visited already?  (Shared?)
      // Node is shared and has no reason to clone.  Flag it as shared.  This 
      // causes it to match into a register for the sharing.
      _shared.set(m->_idx);	// Flag as shared
    else
      set_shared(m);               // Not already visited; so visit now
  }

  // Now hack a few special opcodes
  switch( n->Opcode() ) {	// Handle some opcodes special
  case Op_CMoveD:		// Convert trinary to binary-tree
  case Op_CMoveF:
  case Op_CMoveI:
  case Op_CMoveL:
  case Op_CMoveP: {
    // Restructure into a binary tree for Matching.  It's possible that
    // we could move this code up next to the graph reshaping for IfNodes
    // or vice-versa, but I do not want to debug this for Ladybird.
    // 10/2/2000 CNC.
    Node *pair1 = new (3) CMovePairNode(n->in(1),n->in(1)->in(1));
    n->set_req(1,pair1);
    Node *pair2 = new (3) CMovePairNode(n->in(2),n->in(3));
    n->set_req(2,pair2);
    n->del_req(3);
    return;
  }
  default:
    break;
  }
}

#ifdef ASSERT
// machine-independent root to machine-dependent root
void Matcher::dump_old2new_map() {
  _old2new_map.dump();
}
#endif

//---------------------------collect_null_checks-------------------------------
// Find null checks in the ideal graph; write a machine-specific node for
// it.  Used by later implicit-null-check handling.  Actually collects
// either an IfTrue or IfFalse for the common NOT-null path, AND the ideal
// value being tested.
void Matcher::collect_null_checks( Node *proj ) {
  Node *iff = proj->in(0);
  if( iff->Opcode() == Op_If ) {
    // During matching If's have Bool & Cmp side-by-side
    BoolNode *b = iff->in(1)->is_Bool();
    Node *cmp = iff->in(2);
    if( cmp->Opcode() == Op_CmpP ) {
      if( cmp->in(2)->bottom_type() == TypePtr::NULL_PTR ) {

        if( proj->Opcode() == Op_IfTrue ) {
          extern int all_null_checks_found;
          all_null_checks_found++;
          if( b->_test._test == BoolTest::ne ) {
            _null_check_tests.push(proj);
            _null_check_tests.push(cmp->in(1));
          }
        } else {
          assert( proj->Opcode() == Op_IfFalse, "" );
          if( b->_test._test == BoolTest::eq ) {
            _null_check_tests.push(proj);
            _null_check_tests.push(cmp->in(1));
          }
        }
      }
    }
  }
}

//---------------------------validate_null_checks------------------------------
// Its possible that the value being NULL checked is not the root of a match
// tree.  If so, I cannot use the value in an implicit null check.
void Matcher::validate_null_checks( ) {
  uint cnt = _null_check_tests.size();
  for( uint i=0; i < cnt; i+=2 ) {
    Node *test = _null_check_tests[i];
    Node *val = _null_check_tests[i+1];
    Node *mval = _nodes[val->_idx];
    if( mval ) {
      // Is a match-tree root, so replace with the matched value 
      _null_check_tests.map(i+1,mval);
    } else {
      // Yank from candidate list
      _null_check_tests.map(i+1,_null_check_tests[--cnt]);
      _null_check_tests.map(i,_null_check_tests[--cnt]);
      _null_check_tests.pop();
      _null_check_tests.pop();
      i-=2;
    }
  }  
}

//=============================================================================
//---------------------------State---------------------------------------------
State::State(void) {
#ifdef ASSERT
  _id = 0;
  _kids[0] = _kids[1] = (State*)0xcafebabe;
  _leaf = (Node*)0xbaadf00d;
  //memset(_cost, -1, sizeof(_cost));
  //memset(_rule, -1, sizeof(_rule));
#endif
  memset(_valid, 0, sizeof(_valid));
}

State::~State() {
#ifdef ASSERT
  _id = 99;
  _kids[0] = _kids[1] = (State*)0xcafebabe;
  _leaf = (Node*)0xbaadf00d;
  memset(_cost, -3, sizeof(_cost));
  memset(_rule, -3, sizeof(_rule));
#endif
}

void State::preset(MachNode* n) {
  _leaf = n;
  _kids[0] = _kids[1] = NULL;
  _rule[0] = _PSEUDO_RULE_FOR_PRESET;
  _cost[0] = 1;                             // min cost
  set_valid(0);
}

//---------------------------dump----------------------------------------------
void State::dump() { 
  tty->print("\n");
  dump(0); 
}

void State::dump(int depth) { 
  for( int j = 0; j < depth; j++ )
    tty->print("   ");
  tty->print("--N: ");
#ifndef PRODUCT
  _leaf->dump();
#endif
  uint i;
  for( i = 0; i < _LAST_MACH_OPER; i++ )
    // Check for valid entry
    if( valid(i) ) {
      for( int j = 0; j < depth; j++ )
	tty->print("   ");
      if( i == 0 && _rule[i] == _PSEUDO_RULE_FOR_PRESET ) {
	tty->print_cr("PRESET");
      } else {
        assert(_cost[i] != 0xFFFFFFFF, "cost must be a valid value");
	assert(_rule[i] < _last_Mach_Node, "rule[i] must be valid rule");
	tty->print_cr("%s  %d  %s", 
		      ruleName[i], _cost[i], ruleName[_rule[i]] );
      }
    }
  tty->print_cr("");

  for( i=0; i<2; i++ )
    if( _kids[i] )
      _kids[i]->dump(depth+1);
}
