/*
 * @(#)matcher.hpp	1.136 01/02/09 14:46:36
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

class Compile;
class Node;
class MachNode;
class MachOper;

//---------------------------Matcher-------------------------------------------
class Matcher : public PhaseTransform {
  // Private arena of State objects
  ResourceArea _states;

  VectorSet   _visited;         // Visit bits

  // Private methods which perform the actual matching and reduction
  // Walks the label tree, generating machine nodes
  MachNode *ReduceInst( State *s, int rule, Node *&mem);
  void ReduceInst_Chain_Rule( State *s, int rule, MachNode *mach, Node *&mem);
  uint ReduceInst_Interior(State *s, int rule, MachNode *mach, uint num_opnds, Node *&mem);
  void ReduceOper( State *s, int newrule, MachNode *mach, Node *&mem );

  // Convert a dense opcode number to an expanded rule number
  const int *_reduceOp;
  const int *_leftOp;
  const int *_rightOp;

  // Map dense opcode number to info on when rule is swallowed constant.
  const bool *_swallowed;
  
  // Map dense rule number to determine if this is an instruction chain rule
  const uint _begin_inst_chain_rule;
  const uint _end_inst_chain_rule;
  
  // We want to clone constants and possible CmpI-variants.
  // If we do not clone CmpI, then we can have many instances of
  // condition codes alive at once.  This is OK on some chips and
  // bad on others.  Hence the machine-dependent table lookup.
  const char *_must_clone;

  // Find shared Nodes, or Nodes that otherwise are Matcher roots
  void set_shared( Node *n );
  // Recurse set_shared if not already visited.  Marked as inlined
  // and made tail-recursive to reduce stack depth during recursion.
  void Matcher::set_if_shared( Node *n ) {
    if( _visited.test(n->_idx) )   // Visited already?  (Shared?)
      // Node is shared and has no reason to clone.  Flag it as shared.  This 
      // causes it to match into a register for the sharing.
      _shared.set(n->_idx);	// Flag as shared
    else
      set_shared(n);               // Not already visited; so visit now
  }

  // Node labeling iterator for instruction selection
  Node *Label_Root( const Node *n, State *svec, Node *control, Node *mem );
  
  Node *transform( Node *dummy );

  Node_List &_proj_list;        // For Machine nodes killing many values

  debug_only(Node_Array _old2new_map;)   // Map roots of ideal-trees to machine-roots

public:
  static const int base2reg[];        // Map Types to machine register types
  // Convert ideal machine register to a register mask for spill-loads
  static const RegMask *idealreg2regmask[];
  RegMask *idealreg2spillmask[_last_machine_leaf];
  RegMask *idealreg2debugmask[_last_machine_leaf];
  void init_spill_mask( Node *ret );
  // Convert machine register number to register mask
  static uint mreg2regmask_max;
  static RegMask mreg2regmask[];
  static RegMask STACK_ONLY_mask;

  // Used to control the Label pass
  VectorSet   _shared;          // Shared Ideal Node
  VectorSet   _dontcare;        // Nothing the matcher cares about

  // Mode bit to tell DFA and expand rules whether we are running after
  // (or during) register selection.  Usually, the matcher runs before,
  // but it will also get called to generate post-allocation spill code.
  // In this situation, it is a deadly error to attempt to allocate more
  // temporary registers.
  bool _allocation_started;

  // Machine register names
  static const char *regName[];
  // Machine register encodings
  static const char _regEncode[];
  // Machine Node names
  const char **_ruleName;
  // Rules that are cheaper to rematerialize than to spill
  static const uint _begin_rematerialize;
  static const uint _end_rematerialize;

  // An array of chars, from 0 to _last_Mach_Reg.
  // No Save       = 'N' (for register windows)
  // Save on Entry = 'E'
  // Save on Call  = 'C'
  // Always Save   = 'A' (same as SOE + SOC)
  const char *_register_save_policy;
  const char *_c_reg_save_policy;
  // Convert a machine register to a machine register type, so-as to
  // properly match spill code.
  const int *_register_save_type;
  // Maps from machine register to boolean; true if machine register can
  // be holding a call argument in some signature.
  const bool *_can_be_arg;

  // List of IfFalse or IfTrue Nodes that indicate a taken null test.
  // List is valid in the post-matching space.
  Node_List _null_check_tests;
  void collect_null_checks( Node *proj );
  void validate_null_checks( );

  Matcher( Node_List &proj_list, bool save_argument_registers );

  // Transform, then walk.  Does implicit DCE while walking.
  // Name changed from "transform" to avoid it being virtual.
  Node *xform( Node *old_space_node );

  // Match a single Ideal Node - turn it into a 1-Node tree; Label & Reduce.
  MachNode *match_tree( const Node *n );
  MachNode *match_sfpt( SafePointNode *sfpt );

  // Initialize first stack mask and related masks.
  void init_first_stack_mask();

  // Fixup the save-on-entry registers
  void Fixup_Save_On_Entry( bool save_argument_registers );

  // --- Frame handling ---

  // Register number of the stack slot corresponding to the incoming SP.
  // Per the Big Picture in the AD file, it is:
  //   SharedInfo::stack0 + locks + in_preserve_stack_slots + pad2.
  OptoReg::Name _old_SP;

  // Register number of the stack slot corresponding to the highest incoming
  // argument on the stack.  Per the Big Picture in the AD file, it is:
  //   _old_SP + out_preserve_stack_slots + incoming argument size.
  OptoReg::Name _in_arg_limit;

  // Register number of the stack slot corresponding to the new SP.
  // Per the Big Picture in the AD file, it is:
  //   _in_arg_limit + pad0
  OptoReg::Name _new_SP;

  // Register number of the stack slot corresponding to the highest outgoing
  // argument on the stack.  Per the Big Picture in the AD file, it is:
  //   _new_SP + max outgoing arguments of all calls
  OptoReg::Name _out_arg_limit;

  uint16  *_parm_regs;          // Array of machine registers per argument
  RegMask *_calling_convention_mask; // Array of RegMasks per argument

  static const bool modFSupported(void);
  static const bool modDSupported(void);
  static const bool sinDSupported(void);
  static const bool cosDSupported(void);
  static const bool tanDSupported(void);
  static const bool atanDSupported(void);
  static const bool sqrtDSupported(void);

  // These calls are all generated by the ADLC

  // TRUE - grows up, FALSE - grows down (Intel)
  virtual bool stack_direction() const;

  // Java-Java calling convention
  // (what you use when Java calls Java)

  // Alignment of stack in bits, standard Intel word alignment is 5.
  // Sparc probably wants at least double-word (6).
  virtual uint stack_alignment() const;
  // Array mapping arguments to registers.  Argument 0 is usually the 'this' 
  // pointer.  Registers can include stack-slots and regular registers.
  static void calling_convention( uint16 *, uint len, bool is_outgoing );
  static VMReg::Name find_receiver(symbolOop sig);
  static uint16*               find_callee_arguments(symbolOop sig, bool is_static, int *arg_size);
  // Return address register.  On Intel it is a stack-slot.  On PowerPC
  // it is the Link register.  On Sparc it is r31?
  virtual OptoReg::Name return_addr() const;
  RegMask _return_addr_mask;
  // Return value register.  On Intel it is EAX.  On Sparc i0/o0.
  virtual OptoReg::Name return_value(int) const;
  virtual OptoReg::Name callee_return_value(int) const;
  OptoReg::Name _return_value_register[2];
  RegMask _return_value_mask[2];
  // Inline Cache Register
  static OptoReg::Name inline_cache_reg();
  static const RegMask        &inline_cache_reg_mask();
  static int                   inline_cache_reg_encode();
  

  // Java-Interpreter calling convention
  // (what you use when calling between compiled-Java and Interpreted-Java

  // Convert an array of types to an array mapping arguments to stack-slots
  // and registers. Argument 0 is usually the 'this' pointer.
  // 
  // Call originating in frame converter, to interpreter
  static void interpreter_calling_convention( uint16 *, uint, bool is_outgoing );

  // Number of callee-save + always-save registers 
  // Ignores frame pointer and "special" registers
  static int  number_of_saved_registers();

  // The Method-klass-holder may be passed in the inline_cache_reg
  // and then expanded into the inline_cache_reg and a method_oop register
  static OptoReg::Name interpreter_method_oop_reg();
  static const RegMask        &interpreter_method_oop_reg_mask();
  static int                   interpreter_method_oop_reg_encode();

  static OptoReg::Name compiler_method_oop_reg();
  static const RegMask        &compiler_method_oop_reg_mask();
  static int                   compiler_method_oop_reg_encode();

  // Interpreter's Frame Pointer Register
  static OptoReg::Name interpreter_frame_pointer_reg();
  static const RegMask        &interpreter_frame_pointer_reg_mask();

  // Java-Native calling convention 
  // (what you use when intercalling between Java and C++ code)

  // Alignment of stack in bits, standard Intel word alignment is 5.
  // Sparc probably wants at least double-word (6).
  virtual uint c_stack_alignment() const;
  // Array mapping arguments to registers.  Argument 0 is usually the 'this' 
  // pointer.  Registers can include stack-slots and regular registers.
  static void c_calling_convention( uint16 *, uint );
  // Return value register.  On Intel it is AX.  On Sparc i0.
  virtual OptoReg::Name c_return_value(int ideal_reg) const;
  // Frame pointer. The frame pointer is kept at the base of the stack
  // and so is probably the stack pointer for most machines.  On Intel
  // it is ESP.  On the PowerPC it is R1.  On Sparc it is SP.
  OptoReg::Name c_frame_pointer() const;
  static RegMask c_frame_ptr_mask;

  // !!!!! Special stuff for building ScopeDescs
  virtual int      regnum_to_fpu_offset(int regnum);

  // Is this branch offset small enough to be addressed by a short branch?
  bool is_short_branch_offset(int offset);

  // Should the Matcher clone shifts on addressing modes, expecting them to
  // be subsumed into complex addressing expressions or compute them into
  // registers?  True for Intel but false for most RISCs
  static const bool clone_shift_expressions;
  
  // Is it better to copy float constants, or load them directly from memory?
  // Intel can load a float constant from a direct address, requiring no
  // extra registers.  Most RISCs will have to materialize an address into a
  // register first, so they may as well materialize the constant immediately.
  static const bool rematerialize_float_constants;

  // If CPU can load and store mis-aligned doubles directly then no fixup is 
  // needed.  Else we split the double into 2 integer pieces and move it 
  // piece-by-piece.  Only happens when passing doubles into C code as the 
  // Java calling convention forces doubles to be aligned.
  static const bool misaligned_doubles_ok;

  // Perform a platform dependent implicit null fixup.  This is needed on windows95
  // to take care of some unusual register constraints.
  void pd_implicit_null_fixup(MachNode *load, uint idx);

  // Which register is used to pass the oop to the exception rethrow stub?
  int pd_rethrow_oop_register();

  Node* pd_check_special_copy(const RegMask &dmask, const RegMask &umask, Node* def, Node* n, uint inpidx);

  // Advertise here if the CPU requires explicit rounding operations
  // to implement the UseStrictFP mode.
  static const bool strict_fp_requires_explicit_rounding;

  // Do floats take an entire double register or just half?
  static const bool float_in_double;

#ifdef ASSERT
  void dump_old2new_map();      // machine-independent to machine-dependent
#endif
};
