/*
 * @(#)memnode.cpp	1.144 01/03/12 16:56:29
 *
 * Copyright 1991-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
// Portions of code courtesy of Clifford Click

// Optimization - Graph Style

#include "incls/_precompiled.incl"
#include "incls/_memnode.cpp.incl"

//=============================================================================
uint MemNode::size_of() const { return sizeof(*this); }
uint MemNode::hash() const { return Node::hash() + _type->hash(); }
uint MemNode::cmp( const Node &n ) const
{ return !Type::cmp( _type, ((MemNode&)n)._type ); }
const Type *MemNode::bottom_type() const { return _type; }
uint MemNode::ideal_reg() const { 
  return Matcher::base2reg[_type->_base];
}

#ifndef PRODUCT
void MemNode::dump_spec() const { 
  if( !Verbose && !WizardMode ) {
    // standard dump does this in Verbose and WizardMode
    tty->print(" #"); _type->dump();
  }
}
#endif

//------------------------------Ideal_DU_postCCP-------------------------------
// Find any cast-away of null-ness and keep its control.  Null cast-aways are
// going away in this pass and we need to make this memory op depend on the
// gating null check.

// I tried to leave the CastPP's in.  This makes the graph more accurate in
// some sense; we get to keep around the knowledge that an oop is not-null
// after some test.  Alas, the CastPP's interfere with GVN (some values are
// the regular oop, some are the CastPP of the oop, all merge at Phi's which
// cannot collapse, etc).  This cost us 10% on SpecJVM, even when I removed
// some of the more trivial cases in the optimizer.  Removing more useless
// Phi's started allowing Loads to illegally float above null checks.  I gave
// up on this approach.  CNC 10/20/2000
Node *MemNode::Ideal_DU_postCCP( PhaseCCP *ccp ) {
  Node *ctr = _in[MemNode::Control];
  Node *mem = _in[MemNode::Memory];
  Node *adr = _in[MemNode::Address];
  // Need a null check?  Regular static accesses do not because they are 
  // from constant addresses.  Array ops are gated by the range check (which
  // always includes a NULL check).  Just check field ops.
  if( !ctr ) {
    // Compute base address
    if( adr->is_AddP() )
      adr = adr->in(AddPNode::Base);
    switch( adr->Opcode() ) {
      // CastPP is going away in this pass!  We need this memory op to be
      // control-dependent on the test that is guarding the CastPP
    case Op_CastPP:
      // These usually stick around to change address type, however a
      // useless one can be elided and we still need to pick up a control edge
    case Op_CheckCastPP:
      // CastPP's can be merged into a Phi.  When the CastPP's go away, the
      // Phi might as well, so this memory op cannot be allowed to float
      // above the merge point.
    case Op_Phi:
      ccp->hash_delete(this);
      set_req(MemNode::Control, adr->in(0), &(ccp->_du));
      ccp->hash_insert(this);
      return this;
      break;
      // List of "safe" opcodes; those that implicitly block the memory
      // op below any null check.
    case Op_Parm:               // 'this' pointer is not null
    case Op_LoadP:              // Loading from within a klass
    case Op_LoadKlass:          // Loading from within a klass
    case Op_ConP:               // Loading from a klass
    case Op_CreateEx:           // Sucking up the guts of an exception oop
    case Op_Con:		// Reading from TLS
      break;
    case Op_CMoveP:		// CMoveP is pinned
      break;
    case Op_Proj:               // Direct call to an allocation routine
#ifdef ASSERT
      {
        ProjNode* p = adr->is_Proj();
        assert(p->_con == TypeFunc::Parms, "must be return value");
        const CallNode* call = adr->in(0)->is_Call();
        if (call->is_CallStaticJava()) {
          const CallStaticJavaNode* call_java = call->is_CallStaticJava();
          assert(call_java && call_java->_method == NULL, "must be runtime call");
          // We further presume that this is one of new_slow_Java,
          // new_Java, new_objArray_Java, new_typeArray_Java, or
          // the like, but do not assert for this.
        } else if (!call->is_CallLeaf()) {
          // Projections from fetch_oop (OSR) are allowed as well.
          ShouldNotReachHere();
        }
      }
#endif
    break;
    default:
      ShouldNotReachHere();
    }
  }

  return  NULL;               // No progress
}


//=============================================================================
//------------------------------hash-------------------------------------------
uint LoadNode::hash() const {
  // unroll addition of interesting fields
  return (uint)in(Control) + (uint)in(Memory) + (uint)in(Address);
}

//------------------------------Identity---------------------------------------
// Loads are identity if previous store is to same address
Node *LoadNode::Identity( PhaseTransform *phase ) {
  // If the previous store-maker is the right kind of Store, and the store is
  // to the same address, then we are equal to the value stored.
  Node *mem = _in[MemNode::Memory];
  const StoreNode *st = mem->is_Store();
  int op = Opcode();
  if( st && phase->eqv( st->in(MemNode::Address), _in[MemNode::Address] ) ) {
    // byte, short & char stores truncate naturally.
    // A load has to load the truncated value which requires
    // some sort of masking operation and that requires an
    // Ideal call instead of an Identity call.
    if( op == Op_LoadS || op == Op_LoadC || op == Op_LoadB ) {
      if( !phase->type(st->in(MemNode::ValueIn))->higher_equal( phase->type(this)) )
	return this;
    }
    return st->in(MemNode::ValueIn);
  }
  return this;
}

//------------------------------Ideal------------------------------------------
// If the load is from Field memory and the pointer is non-null, we can
// zero out the control input.
Node *LoadNode::Ideal( PhaseGVN *phase, PhaseDefUse *du ) {
  if( remove_dead_region(phase,du) ) return this;

  Node *address = in(MemNode::Address);
  const Type *t_adr = phase->type( address );
  if( t_adr == Type::TOP ) return NULL;

  // Ignore if memory is dead
  Node *mem = in(MemNode::Memory);
  if( phase->type( mem ) == Type::TOP ) return NULL;

  // Check for useless memory edge in some common special cases
  if( in(MemNode::Control) ) {
    Node *adr = address->is_AddP() ? address->in(AddPNode::Base) : address;
    ProjNode *p = adr->is_Proj();
    if( p && p->_con == TypeFunc::Parms && p->in(0)->is_Start() && 
	phase->type(p)->is_ptr()->_ptr == TypePtr::NotNull ) {
      set_req( MemNode::Control, NULL, du );
    }
  }

  // Avoid independent memory operations
  if( mem->Opcode() == Op_MergeMem ) {
    const TypePtr *tp = t_adr->is_ptr();
    // TypeOopPtr::NOTNULL+any is an OOP with unknown offset - generally
    // means an array I have not precisely typed yet.  Do not do any
    // alias stuff with it any time soon.
    const TypeOopPtr *toop = tp->isa_oopptr();
    if( tp->_base != Type::AnyPtr &&
        (!toop ||
        !toop->klass()->is_java_lang_Object() ||
         toop->_offset != Type::OffsetBot) ) {
      set_req( MemNode::Memory, 
               mem->in(phase->C->get_alias_index(tp)), du );
      return this;
    }
  }

  // Check for prior array store with a different offset; 
  // make Load independent.
  if( mem->is_Store() && address->is_AddP() ) {
    Node *st_adr = mem->in(MemNode::Address);
    if( st_adr->is_AddP() &&
        address->in(2) == st_adr->in(2) &&
	phase->type( address->in(2) ) != Type::TOP &&
	address->in(3)->is_Con() &&
	st_adr ->in(3)->is_Con() &&
	phase->type( address->in(3) ) != phase->type( st_adr->in(3) ) &&
	mem != mem->in(Memory) ) {
      set_req( Memory, mem->in(Memory), du );
      return this;
    }
  }

  return NULL;			// No further progress
}

//------------------------------Value-----------------------------------------
const Type *LoadNode::Value( PhaseTransform *phase ) const {
  // Either input is TOP ==> the result is TOP
  const Type *t1 = phase->type( _in[MemNode::Memory] );
  if( t1 == Type::TOP ) return Type::TOP;
  Node *adr = _in[MemNode::Address];
  const Type *t2 = phase->type( adr );
  if( t2 == Type::TOP ) return Type::TOP;
  const TypePtr *tp = t2->is_ptr();
  if( tp->_ptr == TypePtr::TopPTR ||
      tp->_ptr == TypePtr::AnyNull ) return Type::TOP;
  int off = tp->_offset;
  // If the offset is 'top', then the load is undefined
  if( off == Type::OffsetTop ) return Type::TOP;

  // Try to guess loaded type from pointer type
  if( tp->_base == Type::AryPtr ) {
    const TypeAry *tary = tp->is_aryptr()->_ary;
    const Type *t = tary->_elem;
    if( t->_base == Type::Long && Opcode() == Op_LoadI )
      t = TypeInt::INT;
    return t;

  } else if( tp->_base == Type::OopPtr ) {
    assert( off != Type::OffsetBot ||
            // arrays can be cast to Objects
            tp->is_oopptr()->klass()->is_java_lang_Object(),
            "Field accesses must be precise" );
    // For oop loads, we expect the _type to be precise
  } else if( tp->_base == Type::KlassPtr ) {
    assert( off != Type::OffsetBot ||
            // arrays can be cast to Objects
            tp->is_klassptr()->klass()->is_java_lang_Object(),
            "Field accesses must be precise" );
    // For klass/static loads, we expect the _type to be precise
  }
  return _type;
}

//------------------------------match_edge-------------------------------------
// Do we Match on this edge index or not?  Match only the address.
uint LoadNode::match_edge(uint idx) const {
  return idx == MemNode::Address;
}


//--------------------------LoadBNode::Ideal--------------------------------------
//
//  If the previous store is to the same address as this load,
//  and the value stored was larger than a byte, replace this load
//  with the value stored truncated to a byte.  If no truncation is
//  needed, the replacement is done in LoadNode::Identity().
//
Node *LoadBNode::Ideal(PhaseGVN *phase, PhaseDefUse *du ) {
  Node *mem = in(MemNode::Memory);
  const StoreNode *st = mem->is_Store();

  if( st && phase->eqv( st->in(MemNode::Address), in(MemNode::Address) ) ) {
    Node *stVal = st->in(MemNode::ValueIn);
    if( !phase->type(stVal)->higher_equal( _type ) ) {
      Node *result = phase->transform( new (3) LShiftINode(stVal, phase->intcon(24)) );
      return new (3) RShiftINode(result, phase->intcon(24));
    }
    // Identity call will handle the case where truncation is not needed.
  }
  return LoadNode::Ideal(phase, du);
}

//--------------------------LoadCNode::Ideal--------------------------------------
//
//  If the previous store is to the same address as this load,
//  and the value stored was larger than a char, replace this load
//  with the value stored truncated to a char.  If no truncation is
//  needed, the replacement is done in LoadNode::Identity().
//
Node *LoadCNode::Ideal(PhaseGVN *phase, PhaseDefUse *du) {
  Node *mem = in(MemNode::Memory);
  const StoreNode *st = mem->is_Store();

  if( st && phase->eqv( st->in(MemNode::Address), in(MemNode::Address) ) ) {
    Node *stVal = st->in(MemNode::ValueIn);
    if( !phase->type(stVal)->higher_equal( _type ) ) {
      return new (3) AndINode(stVal,phase->intcon(0xFFFF));
    }
    // Identity call will handle the case where truncation is not needed.
  }
  return LoadNode::Ideal(phase, du);
}

//--------------------------LoadSNode::Ideal--------------------------------------
//
//  If the previous store is to the same address as this load,
//  and the value stored was larger than a short, replace this load
//  with the value stored truncated to a short.  If no truncation is
//  needed, the replacement is done in LoadNode::Identity().
//
Node *LoadSNode::Ideal(PhaseGVN *phase, PhaseDefUse *du ) {
  Node *mem = in(MemNode::Memory);
  const StoreNode *st = mem->is_Store();
  
  if( st && phase->eqv( st->in(MemNode::Address), in(MemNode::Address) ) ) {
    Node *stVal = st->in(MemNode::ValueIn);
    if( phase->type(stVal)->higher_equal( _type ) ) {
      Node *result = phase->transform( new (3) LShiftINode(stVal, phase->intcon(16)) );
      return new (3) RShiftINode(result, phase->intcon(16));
    }
    // Identity call will handle the case where truncation is not needed.
  }
  return LoadNode::Ideal(phase, du);
}

//=============================================================================
//------------------------------Value-----------------------------------------
// Must return TypeInt::INT when used to access an oop's mark field
const Type *LoadINode::Value( PhaseTransform *phase ) const {
  // Either input is TOP ==> the result is TOP
  const Type *t1 = phase->type( _in[MemNode::Memory] );
  if( t1 == Type::TOP ) return Type::TOP;
  Node *adr = _in[MemNode::Address];
  const Type *t2 = phase->type( adr );
  if( t2 == Type::TOP ) return Type::TOP;
  const TypePtr *tp = t2->is_ptr();
  if( tp->_ptr == TypePtr::TopPTR ||
      tp->_ptr == TypePtr::AnyNull ) return Type::TOP;
  int off = tp->_offset;
  // If the offset is 'top', then the load is undefined
  if( off == Type::OffsetTop ) return Type::TOP;

  // Try to guess loaded type from pointer type
  if( tp->_base == Type::AryPtr ) {
    const TypeAry *tary = tp->is_aryptr()->_ary;
    const Type *t = tary->_elem;
    // Check for offset == mark_offset_in_bytes()
    // can use LoadMarkNode in Parse::inline_native_hashcode.
    if( t->_base == Type::Long || (off == oopDesc::mark_offset_in_bytes()) ) {
      t = TypeInt::INT;
    }
    return t;

  } else if( tp->_base == Type::OopPtr ) {
    assert( off != Type::OffsetBot ||
            // arrays can be cast to Objects
            tp->is_oopptr()->klass()->is_java_lang_Object(),
            "Field accesses must be precise" );
    // For oop loads, we expect the _type to be precise
  } 
  return _type;
}

//=============================================================================
//------------------------------Value------------------------------------------
const Type *LoadKlassNode::Value( PhaseTransform *phase ) const {
  // Either input is TOP ==> the result is TOP
  const Type *t1 = phase->type( _in[MemNode::Memory] );
  if( t1 == Type::TOP ) return Type::TOP;
  Node *adr = _in[MemNode::Address];
  const Type *t2 = phase->type( adr );
  if( t2 == Type::TOP ) return Type::TOP;
  const TypePtr *tp = t2->is_ptr();
  if( tp->_ptr == TypePtr::TopPTR ||
      tp->_ptr == TypePtr::AnyNull ||
      tp->_ptr == TypePtr::Null ) return Type::TOP;

  // Return a more precise klass, if possible
  const TypeOopPtr *toop = tp->isa_oopptr();
  if( toop != NULL ) {
    ciInstanceKlass* ik = toop->klass()->as_instance_klass();
    if( !ik->is_loaded() )
      return _type;		// Bail out if not loaded
    // See if we can become precise: no subklasses and no interface
    if( !ik->is_interface() && !ik->has_subklass() ) {
      // Add a dependence; if any subclass added we need to recompile
      if( !ik->flags().is_final() ) 
	phase->C->_recorder->add_dependent(ik, NULL);
      // Return precise klass
      return TypeKlassPtr::make(ik);
    }

    // Return root of possible klass
    return TypeKlassPtr::make(TypePtr::NotNull, ik, 0/*offset*/);
  } 

  // Check for loading klass from an array
  const TypeAryPtr *tary = tp->isa_aryptr();
  if( tary != NULL ) {
    ciKlass *tary_klass = tary->klass();
    if( tary_klass != NULL ) { // can be NULL when at BOTTOM or TOP
      ciArrayKlass *ak = tary->klass()->as_array_klass();
      // If the klass is an object array, we defer the question to the
      // array component klass.
      if( ak->is_obj_array_klass() ) {
	assert( ak->is_loaded(), "" );
        ciKlass *base_k = ak->as_obj_array_klass()->base_element_klass();
        if( base_k->is_loaded() && base_k->is_instance_klass() ) {
          ciInstanceKlass* ik = base_k->as_instance_klass();
          // See if we can become precise: no subklasses and no interface
          if( !ik->is_interface() && !ik->has_subklass() ) {
            // Add a dependence; if any subclass added we need to recompile
            if( !ik->flags().is_final() ) 
              phase->C->_recorder->add_dependent(ik, NULL);
            // Return precise array klass
            return TypeKlassPtr::make(ak);
          }
        }
        return TypeKlassPtr::make(TypePtr::NotNull, ak, 0/*offset*/);
      } else {			// Found a type-array?
        assert( ak->is_type_array_klass(), "" );
        return TypeKlassPtr::make(ak); // These are always precise
      }
    }
  }

  // Check for loading klass from an array klass
  const TypeKlassPtr *tkls = tp->isa_klassptr();
  if( tkls != NULL ) {
    ciKlass* klass = tkls->klass();
    if( !klass->is_loaded() )
      return _type;		// Bail out if not loaded
    if( klass->is_obj_array_klass() &&
	(uint)tkls->_offset == objArrayKlass::element_klass_offset_in_bytes() + sizeof(oopDesc)) {
      ciKlass* elem = klass->as_obj_array_klass()->element_klass();
      // // Always returning precise element type is incorrect, 
      // // e.g., element type could be object and array may contain strings
      // return TypeKlassPtr::make(TypePtr::Constant, elem, 0);

      // The array's TypeKlassPtr was declared 'precise' or 'not precise'
      // according to the element type's subclassing.
      return TypeKlassPtr::make(tkls->_ptr, elem, 0/*offset*/);
    }
  }

  // Bailout case
  return _type;
}

//------------------------------Value-----------------------------------------
const Type *LoadRangeNode::Value( PhaseTransform *phase ) const {
  // Either input is TOP ==> the result is TOP
  const Type *t1 = phase->type( _in[MemNode::Memory] );
  if( t1 == Type::TOP ) return Type::TOP;
  Node *adr = _in[MemNode::Address];
  const Type *t2 = phase->type( adr );
  if( t2 == Type::TOP ) return Type::TOP;
  const TypePtr *tp = t2->is_ptr();
  if( tp->_ptr == TypePtr::TopPTR ||
      tp->_ptr == TypePtr::AnyNull ) return Type::TOP;
  const TypeAryPtr *tap = tp->isa_aryptr();
  if( !tap ) return _type;
  return tap->_ary->_size;
}

//=============================================================================
//------------------------------hash-------------------------------------------
uint StoreNode::hash() const {
  // unroll addition of interesting fields
  return (uint)in(Control) + (uint)in(Memory) + (uint)in(Address) + (uint)in(ValueIn);
}

//------------------------------Ideal------------------------------------------
// If the store is from to memory and the pointer is non-null, we can
// zero out the control input.
Node *StoreNode::Ideal( PhaseGVN *phase, PhaseDefUse *du ) {
  // If our control input is a dead region, kill all below the region
  if( in(MemNode::Control) && remove_dead_region(phase,du) ) 
    return this;

  Node *address = in(MemNode::Address);
  const Type *t_adr = phase->type( address );
  if( t_adr == Type::TOP ) return NULL;

  Node *mem = in(MemNode::Memory);
  if( mem == this ) return NULL;

  if( mem->Opcode() == Op_MergeMem ) {
    const TypePtr *tp = t_adr->is_ptr();
    // TypeOopPtr::NOTNULL+any is an OOP with unknown offset - generally
    // means an array I have not precisely typed yet.  Do not do any
    // alias stuff with it any time soon.
    const TypeOopPtr *toop = tp->isa_oopptr();
    if( tp->_base != Type::AnyPtr &&
        (!toop ||
         !toop->klass()->is_java_lang_Object() ||
         toop->_offset != Type::OffsetBot) ) {
      set_req( MemNode::Memory, 
               mem->in(phase->C->get_alias_index(t_adr->is_ptr())), du );
      return this;
    }
  }

  // Back-to-back stores to same address?  Fold em up.
  // Generally unsafe if I have intervening uses...
  const StoreNode *st = mem->is_Store();
  if( du && st && phase->eqv( st->in(MemNode::Address), address ) ) {
    if( in(MemNode::Memory) == st->in(MemNode::Memory) ) {
      // Did 'else' branch during previous Ideal call
      return NULL;
    } 
    // Mis-matched stores on the same memory edge happen during
    // initialization, where everything is cleared with 32-bit wide
    // stores.  It would be easy enough to combine short and byte
    // stores here, but _they_ only happen into arrays.  In other
    // words, we could optimize the clearing of a short fixed-size
    // array and it's initial filling, but this is a very low frequency
    // event.
    else if( Opcode() == st->Opcode() ) {
      // Check for intervening uses
      uint   max  = du->cnt(st);
      Node **outp = du->out(st);
      uint i;
      for( i=0; i < max; i++ )
        if( outp[i]->is_Load() )
          break;
      if( i == max ) {          // No load uses

        // OUCH! Cannot have multiple copies of store alive at the same time.
        // Happens in _201_compress::decompress at least.  Require exactly
        // ONE user, the 'this' store, until such time as I clone 'st' for
        // each of 'st's uses (thus making the exactly-1-user-rule hold true).
        if( max == 1 ) {
          set_req_X( MemNode::Memory, st->in(MemNode::Memory), phase->is_IterGVN() );
          return this;
        }
      }
    }
  }

  return NULL;			// No further progress
}

//------------------------------Value-----------------------------------------
const Type *StoreNode::Value( PhaseTransform *phase ) const {
  // Either input is TOP ==> the result is TOP
  const Type *t1 = phase->type( _in[MemNode::Memory] );
  if( t1 == Type::TOP ) return Type::TOP;
  const Type *t2 = phase->type( _in[MemNode::Address] );
  if( t2 == Type::TOP ) return Type::TOP;
  const Type *t3 = phase->type( _in[MemNode::ValueIn] );
  if( t3 == Type::TOP ) return Type::TOP;
  return _type;
}

//------------------------------Identity---------------------------------------
Node *StoreNode::Identity( PhaseTransform *phase ) {
  const LoadNode *ld = _in[MemNode::ValueIn]->is_Load();
  // Load then Store?  Then the Store is useless
  if( ld && 
      phase->eqv( ld->in(MemNode::Address), _in[MemNode::Address] ) &&
      phase->eqv( ld->in(MemNode::Memory ), _in[MemNode::Memory ] ) )
    return _in[MemNode::Memory];
  return this;
}

//------------------------------match_edge-------------------------------------
// Do we Match on this edge index or not?  Match only memory & value
uint StoreNode::match_edge(uint idx) const {
  return idx == MemNode::Address || idx == MemNode::ValueIn;
}

//------------------------------cmp--------------------------------------------
// Do not common stores up together.  They generally have to be split
// back up anyways, so do not bother.
uint StoreNode::cmp( const Node &n ) const { 
  return (&n == this);          // Always fail except on self
}

//=============================================================================
//------------------------------Ideal------------------------------------------
// If the store is from an AND mask that leaves the low bits untouched, then
// we can skip the AND operation.  If the store is from a sign-extension
// (a left shift, then right shift) we can skip both.
Node *StoreBNode::Ideal(PhaseGVN *phase,PhaseDefUse *du){
  Node *val = _in[MemNode::ValueIn];
  if( val->Opcode() == Op_AndI ) {
    const TypeInt *t = phase->type( val->in(2) )->is_int();
    if( t->is_con() && (t->get_con() & 0xFF) == 0xFF ) {
      set_req( MemNode::ValueIn, val->in(1), du );
      return this;
    }
  }

  if( val->Opcode() == Op_RShiftI ) {
    const TypeInt *t = phase->type( val->in(2) )->is_int();
    if( t->is_con() && (t->get_con() <= 24) ) {
      Node *shl = val->in(1);
      if( shl->Opcode() == Op_LShiftI ) {
	const TypeInt *t2 = phase->type( shl->in(2) )->is_int();
	if( t2->is_con() && (t2->get_con() == t->get_con()) ) {
	  set_req( MemNode::ValueIn, shl->in(1), du );
	  return this;
	}
      }
    }
  }

  // Finally check the default case
  return StoreNode::Ideal(phase,du);
}

const RegMask &StoreBNode::out_RegMask() const {
  return RegMask::Empty;
}

//=============================================================================
//------------------------------Ideal------------------------------------------
// If the store is from an AND mask that leaves the low bits untouched, then
// we can skip the AND operation
Node *StoreCNode::Ideal(PhaseGVN *phase,PhaseDefUse *du){
  Node *val = _in[MemNode::ValueIn];
  if( val->Opcode() == Op_AndI ) {
    const TypeInt *t = phase->type( val->in(2) )->is_int();
    if( t->is_con() && (t->get_con() & 0xFFFF) == 0xFFFF ) {
      set_req( MemNode::ValueIn, val->in(1), du );
      return this;
    }
  }

  // Check for useless sign-extension before a word store
  if( val->Opcode() == Op_RShiftI ) {
    const TypeInt *t = phase->type( val->in(2) )->is_int();
    if( t->is_con() && (t->get_con() <= 16) ) {
      Node *shl = val->in(1);
      if( shl->Opcode() == Op_LShiftI ) {
	const TypeInt *t2 = phase->type( shl->in(2) )->is_int();
	if( t2->is_con() && (t2->get_con() == t->get_con()) ) {
	  set_req( MemNode::ValueIn, shl->in(1), du );
	  return this;
	}
      }
    }
  }

  // Finally check the default case
  return StoreNode::Ideal(phase,du);
}

//=============================================================================
const RegMask &StoreCNode::out_RegMask() const {
  return RegMask::Empty;
}

//=============================================================================
const RegMask &StoreINode::out_RegMask() const {
  return RegMask::Empty;
}

//=============================================================================
const RegMask &StoreLNode::out_RegMask() const {
  return RegMask::Empty;
}

//=============================================================================
const RegMask &StoreFNode::out_RegMask() const {
  return RegMask::Empty;
}

//=============================================================================
const RegMask &StoreDNode::out_RegMask() const {
  return RegMask::Empty;
}

//=============================================================================
const RegMask &StorePNode::out_RegMask() const {
  return RegMask::Empty;
}


//=============================================================================
//------------------------------match_edge-------------------------------------
// Do we Match on this edge index or not?  Do not match memory
uint ClearArrayNode::match_edge(uint idx) const {
  return idx > 1;
}

//------------------------------Identity---------------------------------------
// Clearing a zero length array does nothing
Node *ClearArrayNode::Identity( PhaseTransform *phase ) {
  return phase->type(in(2))->higher_equal(TypeInt::ZERO)  ? in(1) : this;
}

//------------------------------Idealize---------------------------------------
// Clearing a short array is faster with stores
Node *ClearArrayNode::Ideal(PhaseGVN *phase,PhaseDefUse *du){
  const TypeInt *t = phase->type(in(2))->isa_int();
  if( !t ) return NULL;
  if( !t->is_con() ) return NULL;
  int con = t->get_con();
  // Length too long; use fast hardware clear
  if( con > 8 ) return NULL;
  // Clearing nothing uses the Identity call.
  // Negative clears are possible on dead ClearArrays
  // (see jck test stmt114.stmt11402.val).
  if( con <= 0 ) return NULL;
  Node *mem = in(1);
  if( phase->type(mem)==Type::TOP ) return NULL;
  Node *adr = in(3);
  if( phase->type(adr)==Type::TOP ) return NULL;
  // Get base for derived pointer purposes
  if( adr->Opcode() != Op_AddP ) Unimplemented();
  Node *base = adr->in(1);
  Node *zero = phase->intcon(0);
  Node *off  = phase->intcon(sizeof(int));
  mem = new (4) StoreINode(in(0),mem,adr,zero,Type::MEMORY);
  con--;
  while( con-- ) {
    mem = phase->transform(mem);
    adr = phase->transform(new (4) AddPNode(base,adr,off));
    mem = new (4) StoreINode(in(0),mem,adr,zero,Type::MEMORY);
  }
  return mem;
}

//=============================================================================
// Do we match on this edge? No memory edges
uint StrCompNode::match_edge(uint idx) const {
  return idx == 5 || idx == 6;
}

//------------------------------Ideal------------------------------------------
// Return a node which is more "ideal" than the current node.  Strip out 
// control copies
Node *StrCompNode::Ideal(PhaseGVN *phase,PhaseDefUse *du){
  return remove_dead_region(phase,du) ? this : NULL;
}


//=============================================================================
MemBarNode::MemBarNode(Node *control, Node *memory) : MultiNode(TypeFunc::Parms) {
  Node *top = Compile::current()->_top_node;
  set_req(TypeFunc::Control,control);
  set_req(TypeFunc::I_O,top);
  set_req(TypeFunc::Memory,memory);
  set_req(TypeFunc::FramePtr,top);
  set_req(TypeFunc::ReturnAdr,top);
}

//------------------------------cmp--------------------------------------------
uint MemBarNode::cmp( const Node &n ) const { 
  return (&n == this);		// Always fail except on self
}

//------------------------------Ideal------------------------------------------
// Return a node which is more "ideal" than the current node.  Strip out 
// control copies
Node *MemBarNode::Ideal( PhaseGVN *phase, PhaseDefUse *du ) {
  if( remove_dead_region(phase,du) ) return this;
  return NULL;
}

//------------------------------Value------------------------------------------
const Type *MemBarNode::Value( PhaseTransform *phase ) const {
  if( !_in[0] ) return Type::TOP;
  if( phase->type(_in[0]) == Type::TOP )
    return Type::TOP;
  return TypeTuple::MEMBAR;
}

//------------------------------match------------------------------------------
// Construct projections for memory.
Node *MemBarNode::match( const ProjNode *proj, const Matcher *m ) {
  switch( proj->_con ) {
  case TypeFunc::Control:
  case TypeFunc::Memory:
    return new (1) MachProjNode(this,proj->_con,RegMask::Empty,MachProjNode::unmatched_proj);
  }
  ShouldNotReachHere();
  return NULL;
}

//=============================================================================
//------------------------------MergeMemNode-----------------------------------
MergeMemNode::MergeMemNode( Node *def, uint cnt ) : Node(cnt) { 
  set_req(0,NULL); 
  if( def && def->Opcode() == Op_MergeMem ) {
    for( uint i = 1; i < cnt; i++ ) {
      assert( def->in(i)->Opcode() != Op_MergeMem, "" );
      set_req(i,def->in(i));
    }
  } else {
    for( uint i = 1; i < cnt; i++ )
      set_req(i,def);
  }
}

//------------------------------cmp--------------------------------------------
uint MergeMemNode::cmp( const Node &n ) const { 
  return (&n == this);		// Always fail except on self
}

//------------------------------Identity---------------------------------------
Node *MergeMemNode::Identity( PhaseTransform *phase ) {
  // Identity if this merge point does not record any interesting memory 
  // disambiguations.
  if( req() <= 1 ) return this;
  Node *def = in(1);
  for( uint i = 2; i < req(); i++ )
    if( in(i) != def )
      return this;              // Many memory splits; no change
  return def;                   // No memory splits; ID on the one true input
}

//------------------------------Ideal------------------------------------------
Node *MergeMemNode::Ideal( PhaseGVN *phase, PhaseDefUse *du ) {

  // Remove chain'd MergeMems.  For a closed cycle containing self, always
  // make the MergeMem point to self, i.e. make it trivially obvious that
  // there's a cycle.  Report NO-PROGRESS on closed cycles ofMergeMem's.
  const int TooManyMergeMems=10;
  Node *progress = NULL;
  for( uint i = 1; i < req(); i++ ) {
    Node *m = in(i);
    if( m != this && m->Opcode() == Op_MergeMem ) { // Fast cutout of common case
      int cnt;
      m = m->in(i);             // Skip through the MergeMem
      for( cnt = 0; cnt < TooManyMergeMems; cnt++ ) {
        if( m->Opcode() != Op_MergeMem )
          break;                // Skip thru up to 10 MergeMems
        if( m == this ) break;  // Handle infinite loop containing self
        m = m->in(i);           // Skip through the MergeMem
      }
      if( cnt >= 10 )           // Apparent infinite loop of MMs?
        continue;               // Dont bother reporting progress
      if( m != in(i) ) {        // Progress?
        set_req( i, m, du );    // "Peek" through the nested MergeMems
        progress = this;        // Report progress
      }
    }
  }
  return progress;
}

//------------------------------out_RegMask------------------------------------
const RegMask &MergeMemNode::out_RegMask() const {
  return RegMask::Empty;
}

//------------------------------dump_spec--------------------------------------
#ifndef PRODUCT
void MergeMemNode::dump_spec() const { 
  for( uint i = 1; i < req(); i++ ) {
    tty->print( " N%d:", in(i)->_idx );
    Compile::current()->_map_adr_mem[i]->dump();
  }
}
#endif


