/*
 * @(#)multnode.cpp	1.37 01/02/09 14:46:41
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_multnode.cpp.incl"

//=============================================================================
//------------------------------MultiNode--------------------------------------
MultiNode *MultiNode::is_Multi() { return this; }

const RegMask &MultiNode::out_RegMask() const { 
  return RegMask::Empty;
}

Node *MultiNode::match( const ProjNode *proj, const Matcher *m ) { return proj->clone(); }

//=============================================================================
//------------------------------ProjNode---------------------------------------
uint ProjNode::hash() const { 
  // only one input
  return (uint)in(TypeFunc::Control) + (uint)_con; 
}
uint ProjNode::cmp( const Node &n ) const { return _con == ((ProjNode&)n)._con; }
uint ProjNode::size_of() const { return sizeof(ProjNode); }

ProjNode *ProjNode::is_Proj() { return this; }

// Test if we propagate interesting control along this projection
bool ProjNode::is_CFG() const {
  Node *def = _in[0];
  return (_con == TypeFunc::Control && def->is_CFG());
}

const Type *ProjNode::bottom_type() const { 
  const Type *tb = _in[0]->bottom_type();
  if( tb == Type::TOP ) return Type::TOP;
  if( tb == Type::BOTTOM ) return Type::BOTTOM;
  const TypeTuple *t = tb->is_tuple();
  assert( _con < t->_cnt, "" );
  return t->_fields[_con]; 
}
int ProjNode::pinned() const { return _in[0]->pinned(); }
#ifndef PRODUCT
void ProjNode::dump_spec() const { tty->print("#%d",_con); }
#endif

//------------------------------Identity---------------------------------------
Node *ProjNode::Identity( PhaseTransform *phase ) {
  Node *safe = in(0);
  int op = safe->Opcode();
  // If we have inlined, then I put call projections on the downside of a
  // ReturnNode.  Just optimize it all away.
  if( op == Op_Return ) return safe->in(_con);
  // Check for useless safepoint & remove
  if( op == Op_SafePoint ) {
    Node *n = safe->in(0);

    if( n->is_Proj() ) {
      Node *n0 = n->in(0);
      // Check if he is a call projection
      if( n0->is_Call() || n0->is_Catch() ) 
        // Useless Safepoint, so remove it
        return safe->in(_con);
      // If you have back to back safepoints, remove one
      if( n0->is_SafePoint() && n0->req() >= TypeFunc::Parms ) 
        return safe->in(_con);
    }
    // If you have all JVM state info stripped, remove self
    if( safe->req() < TypeFunc::Parms ) 
      return safe->in(_con);
  }

  return this;
}

//------------------------------Value------------------------------------------
const Type *ProjNode::Value( PhaseTransform *phase ) const {
  if( !_in[0] ) return Type::TOP;
  const Type *t = phase->type(_in[0]);
  if( t == Type::TOP ) return t;
  if( t == Type::BOTTOM ) return t;
  return t->is_tuple()->_fields[_con];
}

//------------------------------out_RegMask------------------------------------
// Pass the buck uphill
const RegMask &ProjNode::out_RegMask() const {
  return RegMask::Empty;
}

//------------------------------ideal_reg--------------------------------------
uint ProjNode::ideal_reg() const { 
  return Matcher::base2reg[bottom_type()->_base];
}


