/*
 * @(#)node.cpp	1.168 01/02/09 14:46:43
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_node.cpp.incl"

class RegMask;
// #include "phase.hpp"
class PhaseTransform;
class PhaseGVN;

// Arena we are currently building Nodes in
const uint Node::NotAMachineReg = 0xffff0000;

#ifdef ASSERT
//-------------------------- construct_node------------------------------------
// Set a breakpoint here to identify where a particular node index is built.
void Node::verify_construction( ) {
  set_debug_idx( Compile::_debug_idx++ );
  assert( Compile::current()->_unique < 65400 /*1<<(8*sizeof(_idx))*/, "Node limit exceeded" );
}
#endif

//------------------------------Node-------------------------------------------
// Create a Node, with a given number of required edges.
Node::Node( uint req ) : _idx(Compile::current()->_unique++), _cnt(req), _max(req) {
  assert( req < 64000 /*1<<(8*sizeof(_cnt))*/, "Input limit exceeded" );
  debug_only( verify_construction() );
  _in = (Node**)Compile::current()->_arena.Amalloc(req*sizeof(Node*)); // Input array
}

//------------------------------Node-------------------------------------------
Node::Node( Node *n0 ) : _idx( Compile::current()->_unique++ ), _cnt(1), _max(1) {
  debug_only( verify_construction() );
  // Assert we allocated space for input array already
  assert( ((void**)this)[-1] == this, "Must pass arg count to 'new'" );
  _in = ((Node**)this) - 1;
  _in[0] = n0;
}

//------------------------------Node-------------------------------------------
Node::Node( Node *n0, Node *n1 ) : _idx( Compile::current()->_unique++ ), _cnt(2), _max(2) {
  debug_only( verify_construction() );
  // Assert we allocated space for input array already
  assert( ((void**)this)[-2] == this, "Must pass arg count to 'new'" );
  _in = ((Node**)this) - 2;
  _in[0] = n0;
  _in[1] = n1;
}

//------------------------------Node-------------------------------------------
Node::Node( Node *n0, Node *n1, Node *n2 ) : 
  _idx( Compile::current()->_unique++ ), _cnt(3), _max(3) {
  debug_only( verify_construction() );
  // Assert we allocated space for input array already
  assert( ((void**)this)[-3] == this, "Must pass arg count to 'new'" );
  _in = ((Node**)this) - 3;
  _in[0] = n0;
  _in[1] = n1;
  _in[2] = n2;
}

//------------------------------Node-------------------------------------------
Node::Node( Node *n0, Node *n1, Node *n2, Node *n3 ) : 
  _idx( Compile::current()->_unique++ ), _cnt(4), _max(4) {
  debug_only( verify_construction() );
  // Assert we allocated space for input array already
  assert( ((void**)this)[-4] == this, "Must pass arg count to 'new'" );
  _in = ((Node**)this) - 4;
  _in[0] = n0;
  _in[1] = n1;
  _in[2] = n2;
  _in[3] = n3;
}

//------------------------------Node-------------------------------------------
Node::Node( Node *n0, Node *n1, Node *n2, Node *n3,
		    Node *n4, Node *n5, Node *n6 ) : 
  _idx( Compile::current()->_unique++ ), _cnt(7), _max(7) {
  debug_only( verify_construction() );
  // Assert we allocated space for input array already
  assert( ((void**)this)[-7] == this, "Must pass arg count to 'new'" );
  _in = ((Node**)this) - 7;
  _in[0] = n0;
  _in[1] = n1;
  _in[2] = n2;
  _in[3] = n3;
  _in[4] = n4;
  _in[5] = n5;
  _in[6] = n6;
}

//------------------------------clone------------------------------------------
// Clone a Node.  
Node *Node::clone() const {
  Compile *compile = Compile::current();
  uint s = size_of();           // Size of inherited Node
  Node *n = (Node*)compile->_arena.Amalloc_4(size_of() + _max*sizeof(Node*));
  memmove(n, this, s);          // Copy structure bits
  // Address of new input pointer array
  Node **nn = (Node**)(((char*)n)+s);
  n->_in = nn;                  // Set the new input pointer array
  // Copy the input pointer bits as well
  memmove(nn, _in, _max*sizeof(Node*));
  n->_idx = compile->_unique++;       // Get new unique index as well
  debug_only( n->verify_construction() );
  debug_only( n->set_debug_idx( debug_idx() ) );
  return n;                     // Return the clone
}

//------------------------------~Node------------------------------------------
// Fancy destructor; eagerly attempt to reclaim Node numberings and storage
extern int reclaim_idx ;
extern int reclaim_in  ;
extern int reclaim_node;
void Node::destruct() { 
  // Eagerly reclaim unique Node numberings
  Compile* compile = Compile::current();
  if( (uint)_idx+1 == compile->_unique ) {
    compile->_unique--; 
#ifdef ASSERT
    reclaim_idx++;
#endif
  }
  // See if the input array was allocated just prior to the object
  int edge_size = _max*sizeof(void*);
  char *edge_end = ((char*)_in) + edge_size;
  int node_size = size_of();
  if( edge_end == (char*)this ) {
#ifdef ASSERT
    if( edge_end+node_size == compile->_arena.hwm() ) {
      reclaim_in  += edge_size;
      reclaim_node+= node_size;
    }
#endif
    // It was; free the input array and object all in one hit
    compile->_arena.Afree(_in,edge_size+node_size);
  } else {
    // Free just the input array
#ifdef ASSERT
    if( edge_end == compile->_arena.hwm() ) 
      reclaim_in  += edge_size;
#endif
    compile->_arena.Afree(_in,edge_size);
    // Free just the object
#ifdef ASSERT
    if( ((char*)this) + node_size == compile->_arena.hwm() ) 
      reclaim_node+= node_size;
#endif
    compile->_arena.Afree(this,node_size);
  }
}

//------------------------------grow-------------------------------------------
// Grow the input array, making space for more edges
void Node::grow( uint len ) {
  uint new_max = _max;
  if( !new_max ) new_max = 2;
  // Is _max a uint8 or a uint16?  Be generic.
  const uint limit = 64000; // 1<<(8*sizeof(_max));
  // Check for growing too large
  if( len >= limit ) Unimplemented();
  while( new_max <= len ) new_max <<= 1; // Find next power-of-2
  // Trimming to limit allows a uint8 to handle up to 255 edges.
  // Previously I was using only powers-of-2 which peaked at 128 edges.
  if( new_max >= limit ) new_max = limit-1;
  _in =(Node**)Compile::current()->_arena.Arealloc(_in,_max*sizeof(Node*),new_max*sizeof(Node*));
  memset(&_in[_max],0,(new_max-_max)*sizeof(Node*));    // NULL all new space
  _max = new_max;               // Record new max length
}

//------------------------------add_req----------------------------------------
// Add a new required input at the end
void Node::add_req( Node *n ) {

  // Look to see if I can move precedence down one without reallocating
  if( _cnt >= _max || _in[_max-1] ) 
    grow( _max+1 );

  // Find a precedence edge to move
  if( _in[_cnt] ) {             // Next precedence edge is busy?
    uint i;
    for( i=_cnt; i<_max; i++ ) 
      if( !_in[i] )             // Find the NULL at end of prec edge list
        break;                  // There must be one, since we grew the array
    _in[i] = _in[_cnt];         // Move prec over, making space for req edge
  }
  
  _in[_cnt++] = n;              // Stuff over old prec edge
}

//------------------------------del_req----------------------------------------
// Delete the required edge and compact the edge array
void Node::del_req( uint idx ) {
  _in[idx] = _in[--_cnt];
  _in[_cnt] = NULL;
}

//------------------------------ins_req----------------------------------------
// Insert a new required input at the end
void Node::ins_req( uint idx, Node *n ) {
  add_req(NULL);                // Make space
  assert( idx < _max, "Must have allocated enough space");
  // Slide over
  memmove(&_in[idx+1],&_in[idx],(_cnt-idx-1)*sizeof(Node*));
  _in[idx] = n;                 // Stuff over old required edge
}

//------------------------------pack_req---------------------------------------
// Remove inputs from this Node that are NULL in the region Node.
// "this" may or may not be equal to "region".
void Node::pack_req( const Node *region ) {
  uint j=1;
  uint i;
  // Copy and compact the live inputs
  for( i=1; i<region->req(); i++ ) 
    if( region->in(i) ) 
      set_req( j++, _in[i] );
  _cnt = j;
  // Copy down any precedence edges
  for( ; i<len(); i++ )
    _in[j++] = _in[i];
  // Zap the rest
  for( ; j<len(); j++ )
    _in[j] = NULL;
}

//------------------------------add_prec---------------------------------------
// Add a new precedence input.  Precedence inputs are unordered, with 
// duplicates removed and NULLs packed down at the end.
void Node::add_prec( Node *n ) {

  // Check for NULL at end
  if( _cnt >= _max || _in[_max-1] ) 
    grow( _max+1 );

  // Find a precedence edge to move
  uint i = _cnt;
  while( _in[i] ) i++;
  _in[i] = n;                   // Stuff prec edge over NULL
}

//------------------------------rm_prec----------------------------------------
// Remove a precedence input.  Precedence inputs are unordered, with 
// duplicates removed and NULLs packed down at the end.
void Node::rm_prec( uint j ) {

  // Find end of precedence list to pack NULLs
  uint i;
  for( i=j; i<_max; i++ ) 
    if( !_in[i] )               // Find the NULL at end of prec edge list
      break;
  _in[j] = _in[--i];            // Move last element over removed guy
  _in[i] = NULL;                // NULL out last element
}

//------------------------------edge_copy--------------------------------------
void Node::edge_copy(Node **edges, int length) const {
  assert(length == (int)_max, "Copying edge array into wrongly sized space\n");
  memmove(edges, _in, _max*sizeof(Node *));
}

// -----------------------------Name-------------------------------------------
extern const char *NodeClassNames[];
const char *Node::Name() const { return NodeClassNames[Opcode()]; }

//------------------------------size_of----------------------------------------
uint Node::size_of() const { return sizeof(*this); }

//------------------------------ideal_reg--------------------------------------
uint Node::ideal_reg() const { return 0; }

//------------------------------jvms-------------------------------------------
JVMStateObj *Node::jvms() const { return NULL; }
JVMStateObj *Node::jvmtail() const { return NULL; }

//------------------------------oop_map----------------------------------------
OopMap *Node::oop_map() const { ShouldNotCallThis(); return NULL; }

//------------------------------set_oop_map------------------------------------
void Node::set_oop_map(OopMap *) { ShouldNotCallThis(); }

//------------------------------format-----------------------------------------
// Print as assembly
void Node::format( PhaseChaitin * ) const {}
//------------------------------emit-------------------------------------------
// Emit bytes starting at parameter 'ptr'.  
void Node::emit(CodeBuffer &cbuf, PhaseChaitin *pc_) const {} 
//------------------------------size-------------------------------------------
// Size of instruction in bytes
uint Node::size(PhaseChaitin *pc_) const { return 0; }

//------------------------------CFG Construction-------------------------------
// Nodes that begin basic blocks, e.g. RootNode, RegionNode, StartNode.
// I also need the machine specific versions of these.
int Node::is_block_start() const { return 0; }

// Nodes that end basic blocks, e.g. IfTrue/IfFalse, JumpProjNode, Root,
// Goto and Return.  
const Node *Node::is_block_proj() const { return 0; }

// Minimum guaranteed type
const Type *Node::bottom_type() const { return Type::BOTTOM; }

//------------------------------Identity---------------------------------------
// Return a node that the given node is equivalent to.
Node *Node::Identity( PhaseTransform * ) {
  return this;                  // Default to no identities
}

//------------------------------Value------------------------------------------
// Compute a new Type for a node using the Type of the inputs.
const Type *Node::Value( PhaseTransform * ) const {
  return bottom_type();         // Default to worst-case Type
}

//------------------------------Ideal------------------------------------------
//
// 'Idealize' the graph rooted at this Node.
//
// In order to be efficient and flexible there are some subtle invariants
// these Ideal calls need to hold.  Running with '+VerifyIterativeGVN' checks
// these invariants, although its too slow to have on by default.  If you are
// hacking an Ideal call, be sure to test with +VerifyIterativeGVN!
//
// The Ideal call almost arbitrarily reshape the graph rooted at the 'this'
// pointer.  If ANY change is made, it must return the root of the reshaped
// graph - even if the root is the same Node.  Example: swapping the inputs
// to an AddINode gives the same answer and same root, but you still have to
// return the 'this' pointer instead of NULL.
//
// You cannot return an OLD Node, except for the 'this' pointer.  Use the
// Identity call to return an old Node; basically if Identity can find 
// another Node have the Ideal call make no change and return NULL.
// Example: AddINode::Ideal must check for add of zero; in this case it
// returns NULL instead of doing any graph reshaping.
//
// You cannot modify any old Nodes except for the 'this' pointer.  Due to
// sharing there may be other users of the old Nodes relying on their current
// semantics.  Modifying them will break the other users.
// Example: when reshape "(X+3)+4" into "X+7" you must leave the Node for
// "X+3" unchanged in case it is shared.
//
// If you modify the 'this' pointer's inputs, you must use 'set_req' with
// def-use info.  If you are making a new Node (either as the new root or
// some new internal piece) you must NOT use set_req with def-use info.
// You can make a new Node with either 'new' or 'clone'.  In either case,
// def-use info is (correctly) not generated.  
// Example: reshape "(X+3)+4" into "X+7":
//    set_req(1,in(1)->in(1) /* grab X */, du /* must use DU on 'this' */);
//    set_req(2,phase->intcon(7),du);
//    return this;
// Example: reshape "X*4" into "X<<1"
//    return new (2) LShiftINode( in(1), phase->intcon(1) );
// 
// You must call 'phase->transform(X)' on any new Nodes X you make, except
// for the returned root node.  Example: reshape "X*31" with "(X<<5)-1".
//    Node *shift =phase->transform(new(2)LShiftINode(in(1),phase->intcon(5)));
//    return new (2) AddINode(shift, phase->intcon(-1));
//
// When making a Node for a constant use 'phase->makecon' or 'phase->intcon'.
// These forms are faster than 'phase->transform(new (1) ConNode())' and Do
// The Right Thing with def-use info.
//
// You cannot bury the 'this' Node inside of a graph reshape.  If the reshaped
// graph uses the 'this' Node it must be the root.  If you want a Node with 
// the same Opcode as the 'this' pointer use 'clone'.
//
Node *Node::Ideal( PhaseGVN *phase, PhaseDefUse *du ) {
  return NULL;                  // Default to being Ideal already
}

//------------------------------remove_dead_region-----------------------------
// This control node is dead.  Follow the subgraph below it making everything
// using it dead as well.  This will happen normally via the usual IterGVN
// worklist but this call is more efficient.  Do not update use-def info
// inside the dead region, just at the borders.
static int kill_dead_code( Node *dead, PhaseIterGVN *igvn ) {
  // Con's are a popular node to re-hit in the hash table again.
  if( dead->is_Con() ) return 0;
  Node *top = igvn->C->_top_node;
  igvn->_worklist.remove(dead);
  igvn->hash_delete(dead);
  igvn->_types.map(dead->_idx,Type::TOP);
  PhaseDefUse *du = &igvn->_du;
  int progress = 0;

  // For all Users of the Dead...    ;-)
  while( du->cnt(dead) ) {
    Node *use = du->out(dead)[0];
    if( use->in(0) == dead ) {  // Found another dead node
      progress |= kill_dead_code(use,igvn); // Recurse
    } else {                    // Else found a not-dead user
      igvn->hash_delete(use);   // Yank from hash table prior to mod
      for( uint j = 1; j < use->req(); j++ )
        if( use->in(j) == dead ) // Turn all dead uses into TOP uses
          use->set_req( j, top, du );
      igvn->_worklist.push(use);
    }
  }
  // Kill all inputs to the dead guy
  for( uint i=0; i < dead->req(); i++ ) {
    Node *n = dead->in(i);      // Get input to dead guy
    if( n && n != top ) {       // Input is valid?
      progress = 1;
      dead->set_req( i, top );  // Smash input away
      du->delete_use_attempt( n, dead );// Clear it out of DU info
      if( du->cnt(n) == 0 )     // Input also goes dead?
        kill_dead_code( n, igvn );// Clear it out as well
    }
  }
  return progress;
}

//------------------------------remove_dead_region-----------------------------
bool Node::remove_dead_region( PhaseGVN *phase, PhaseDefUse *du ) {
  Node *n = in(0);
  if( !n ) return false;
  // Lost control into this guy?  I.e., it became unreachable?
  // Aggressively kill all unreachable code.
  if( du && n == phase->C->_top_node ) {
    return kill_dead_code(this, phase->is_IterGVN());
  }

  if( n->is_Region() && !n->in(0) ) {
    Node *m = n->nonnull_req();
    set_req(0,m,du);
    return true;
  }
  return false;
}

//------------------------------Ideal_DU_postCCP-------------------------------
// Idealize graph, using DU info.  Must clone result into new-space
Node *Node::Ideal_DU_postCCP( PhaseCCP * ) {
  return NULL;                 // Default to no change
}

//------------------------------hash-------------------------------------------
// Hash function over Nodes.
uint Node::hash() const {
  uint sum = 0;
  for( uint i=0; i<_cnt; i++ )  // Add in all inputs
    sum = (sum<<1)-(uint)_in[i];        // Ignore embedded NULLs
  return (sum>>2) + _cnt + Opcode();
}

//------------------------------cmp--------------------------------------------
// Compare special parts of simple Nodes
uint Node::cmp( const Node &n ) const {
  return 1;                     // Must be same
}


// Get an integer constant from a ConstNode.
// Returns True if it is an integer ConstNode and puts the constant
// into 'con', otherwise it returns False.
int Node::get_int( jint * ) const { return false; }

jint Node::get_int( ) const { 
  if( Opcode() == Op_ConI )
    return ((ConINode*)this)->_type->is_int()->get_con(); 
  else {
    assert( Opcode() == Op_ConP, "" ); 
    return ((ConPNode*)this)->_type->is_ptr()->get_con(); 
  }
}

// Get a long constant from a ConstNode.
jlong Node::get_long( ) const { 
  assert( Opcode() == Op_ConL, "" ); 
  return ((ConLNode*)this)->_type->is_long()->get_con(); 
}

// Get a double constant from a ConstNode.
// Returns the constant if it is a double ConstNode
jdouble Node::getd( ) const { 
  assert( Opcode() == Op_ConD, "" ); 
  return ((ConDNode*)this)->_type->is_double_constant()->getd(); 
}

// Get a float constant from a ConstNode.
// Returns the constant if it is a float ConstNode
jfloat Node::getf( ) const { 
  assert( Opcode() == Op_ConF, "" ); 
  return ((ConFNode*)this)->_type->is_float_constant()->getf(); 
}


//------------------------------find------------------------------------------
// Find a neighbor of this Node with the given _idx
static void find_recur( Node* &result, Node *n, int idx, bool only_ctrl, 
                        VectorSet &old_space, VectorSet &new_space ) {
  if( !n ) return;
  // Contained in new_space or old_space?
  VectorSet *v = Compile::current()->_arena.contains(n) ? &new_space : &old_space;
  if( v->test(n->_idx) ) return;
  if( (int)n->_idx == idx) {
    if (result != NULL)
      tty->print("find: 0x%x and 0x%x both have idx==%d\n",
		 (uint)result, (uint)n, idx);
    result = n;
  }
  v->set(n->_idx);
  for( uint i=0; i<n->len(); i++ ) {
    if( only_ctrl && !(n->is_Region()) && (n->Opcode() != Op_Root) && (i != TypeFunc::Control) ) continue;
    find_recur( result, n->in(i), idx, only_ctrl, old_space, new_space );
  }
}

#ifndef PRODUCT
// call this from debugger:
Node* find_node(Node* n, int idx) {
  return n->find(idx);
}

//------------------------------find-------------------------------------------
Node* Node::find(int idx) const {
  VectorSet old_space(Thread::current()->resource_area()), new_space(Thread::current()->resource_area());
  Node* result = NULL;
  find_recur( result, (Node*) this, idx, false, old_space, new_space );
  return result;
}

//------------------------------find_ctrl--------------------------------------
// Find an ancestor to this node in the control history with given _idx
Node* Node::find_ctrl(int idx) const {
  VectorSet old_space(Thread::current()->resource_area()), new_space(Thread::current()->resource_area());
  Node* result = NULL;
  find_recur( result, (Node*) this, idx, true, old_space, new_space );
  return result;
}
#endif



#ifndef PRODUCT
//------------------------------dump------------------------------------------
// Dump a Node
void Node::dump() const {
  tty->print("%c%d\t%s\t<== ",
    Compile::current()->_arena.contains(this) ? ' ' : 'o', _idx, Name());

  // Dump the required and precedence inputs
  dump_req();
  dump_prec();

  // Dump node-specific info
  dump_spec();
#ifdef ASSERT
  // Dump the non-reset _debug_idx
  if( Verbose && WizardMode ) {
    tty->print("  <%d>",debug_idx());
  }
#endif

  const Type *t = NULL;
  if( ((Node*)this)->is_Proj() ) {
    if( in(0) && in(0)->bottom_type()->_base == Type::Tuple ) 
      t = bottom_type();
  } else if( Opcode() == Op_AddP && in(2)->bottom_type() == Type::TOP ) {
    // A dead AddP will fail looking up bottom_type, accessed during debugging
    /* NOTHING */
  } else {
    t = bottom_type();
  }

  if( t && (t->isa_oopptr() || t->isa_klassptr()) ) {
    const TypeOopPtr   *toop = t->isa_oopptr();
    const TypeKlassPtr *tkls = t->isa_klassptr();
    ciKlass*           klass = toop ? toop->klass() : (tkls ? tkls->klass() : NULL );
    if( klass && klass->is_interface() ) {
      tty->print("  Interface:");
    } else if( toop ) {
      tty->print("  Oop:");
    } else if( tkls ) {
      tty->print("  Klass:");
    }
    t->dump();
  } else if( Verbose || WizardMode ) {
    tty->print("  Type:");
    if( t ) {
      t->dump();
    } else {
      tty->print("no type");
    }
  }
  tty->print("\n");
}

//------------------------------dump_req--------------------------------------
void Node::dump_req() const {
  // Dump the required input edges
  uint i;                       // Exit value of loop
  for( i=0; i<req(); i++ ) {    // For all required inputs
    if( (int)in(i) == 0xabababab ) {
      tty->print("Uninitialized ");
    }
    else {
      if( in(i) )
        tty->print("%c%d ", Compile::current()->_arena.contains(in(i)) ? ' ' : 'o', in(i)->_idx);
      else tty->print("_ ");
    }
  }
}

//------------------------------dump_prec-------------------------------------
void Node::dump_prec() const {
  // Dump the precedence edges
  uint i = req();
  int any_prec = 0;
  for( ; i<len(); i++ ) {       // For all precedence inputs
    if( in(i) ) {
      if( !any_prec++ ) tty->print(" |");
      tty->print("%c%d ", Compile::current()->_arena.contains(in(i)) ? ' ' : 'o', in(i)->_idx);
    }
  }
}

//------------------------------dump_recur-------------------------------------
static void dump_recur( const Node *n, uint d, bool only_ctrl,
                        VectorSet &old_space, VectorSet &new_space ) {
  if( !n ) return;
  if( !d ) return;
  // Contained in new_space or old_space?
  VectorSet *v = Compile::current()->_arena.contains(n) ? &new_space : &old_space;
  if( v->test_set(n->_idx) ) return;

  for( uint i=0; i<n->len(); i++ ) {
    if( only_ctrl && !(n->is_Region()) && (n->Opcode() != Op_Root) && (i != TypeFunc::Control) ) continue;
    dump_recur( n->in(i), d-1, only_ctrl, old_space, new_space );
  }
  n->dump();
}

//------------------------------dump-------------------------------------------
void Node::dump(int d) const {
  VectorSet old_space(Thread::current()->resource_area()), new_space(Thread::current()->resource_area());
  dump_recur( this, d, false, old_space, new_space );
}

//------------------------------dump_ctrl--------------------------------------
// Dump a Node's control history to depth
void Node::dump_ctrl(int d) const {
  //this was a merge-o: //unique_top = NULL;
  VectorSet old_space(Thread::current()->resource_area()), new_space(Thread::current()->resource_area());
  dump_recur( this, d, true, old_space, new_space );
}

//------------------------------verify_recur-----------------------------------
void Node::verify_recur( const Node *n, int verify_depth, PhaseDefUse *du,
			 VectorSet &old_space, VectorSet &new_space ) {
  if ( verify_depth == 0 )  return;
  if (verify_depth > 0)  --verify_depth;

  // Contained in new_space or old_space?
  VectorSet *v = du->C->_arena.contains(n) ? &new_space : &old_space;
  // Check for visited in the proper space.  Numberings are not unique
  // across spaces so we need a seperate VectorSet for each space.
  if( v->test_set(n->_idx) ) return;

  if( n == du->C->_top_node ) {
    assert( du->C->node_unique_top() == NULL || du->C->node_unique_top() == n, "TOP must be unique");
    du->C->set_node_unique_top(n); 
  }
  if( n->is_Con() && ((ConNode*)n)->_type == Type::TOP ) {
    assert( n == du->C->_top_node, "TOP must be unique");
  }

  for( uint i = 0; i < n->len(); i++ ) {
    Node *x = n->in(i);
    if( !x ) continue;
    if( x == du->C->_top_node ) continue;

    // Verify my input has a def-use edge to me
    if( du ) {
      // Count use-def edges from n to x
      int cnt = 0;
      for( uint j = 0; j < n->len(); j++ ) 
        if( n->in(j) == x )
          cnt++;
      // Count def-use edges from x to n
      uint    max = du->cnt(x);
      Node **outp = du->out(x);
      for( uint k = 0; k < max; k++ )
        if( outp[k] == n )
          cnt--;
      assert( cnt == 0, "mismatched def-use edge counts" );
    }

    verify_recur( x, verify_depth, du, old_space, new_space );
  }

  // Verify each of my uses has an input edge to me
  if( du ) du->verify_uses( n );
}

//------------------------------verify-----------------------------------------
// Check Def-Use info for my subgraph
void Node::verify(PhaseDefUse *du) const {
  du->C->set_node_unique_top(NULL); // we expect to set this non-null just once
  VectorSet old_space(Thread::current()->resource_area()), new_space(Thread::current()->resource_area());
  verify_recur( this, -1, du, old_space, new_space );
}
#endif


//------------------------------walk-------------------------------------------
// Graph walk, with both pre-order and post-order functions
void Node::walk(NFunc pre, NFunc post, void *env) {
  VectorSet visited(Thread::current()->resource_area()); // Setup for local walk
  walk_(pre, post, env, visited); 
}

void Node::walk_(NFunc pre, NFunc post, void *env, VectorSet &visited) {
  if( visited.test_set(_idx) ) return;
  pre(*this,env);               // Call the pre-order walk function
  for( uint i=0; i<_max; i++ )
    if( _in[i] )		// Input exists and is not walked?
      _in[i]->walk_(pre,post,env,visited); // Walk it with pre & post functions
  post(*this,env);              // Call the post-order walk function
}

void Node::nop(Node &, void*) {}

void Node::packregion( Node &n, void* ) {
  if( n.is_Region() ) // Remove dead control paths from Regions
    n.pack_req( &n );
}

//------------------------------Registers--------------------------------------
// Do we Match on this edge index or not?  Generally false for Control
// and true for everything else.  Weird for calls & returns.
uint Node::match_edge(uint idx) const {
  return idx;                   // True for other than index 0 (control)
}

// Register classes are defined for specific machines
const RegMask &Node::out_RegMask() const { 
  ShouldNotCallThis();
  return *(new RegMask());
}

const RegMask &Node::in_RegMask(uint) const { 
  ShouldNotCallThis();
  return *(new RegMask());
}

//=============================================================================
//-----------------------------------------------------------------------------
void Node_Array::reset( Arena *new_arena ) {
  _a->Afree(_nodes,_max*sizeof(Node*));
  _max   = 0;
  _nodes = NULL;
  _a     = new_arena;
}

//------------------------------clear------------------------------------------
// Clear all entries in _nodes to NULL but keep storage
void Node_Array::clear( ) {
  memset( _nodes, 0, _max*sizeof(Node*) );
}

//-----------------------------------------------------------------------------
void Node_Array::grow( uint i ) {
  if( !_max ) { 
    _max = 1;
    _nodes = (Node**)_a->Amalloc( _max * sizeof(Node*) );
    _nodes[0] = NULL;
  }
  uint old = _max;
  while( i >= _max ) _max <<= 1;        // Double to fit
  _nodes = (Node**)_a->Arealloc( _nodes, old*sizeof(Node*),_max*sizeof(Node*));
  memset( &_nodes[old], 0, (_max-old)*sizeof(Node*) );
}

//-----------------------------------------------------------------------------
void Node_Array::insert( uint i, Node *n ) {
  if( _nodes[_max-1] ) grow(_max);      // Get more space if full
  memmove( &_nodes[i+1], &_nodes[i], (_max-i-1)*sizeof(Node*) );
  _nodes[i] = n;
}

//-----------------------------------------------------------------------------
void Node_Array::remove( uint i ) {
  memmove( &_nodes[i], &_nodes[i+1], (_max-i-1)*sizeof(Node*) );
  _nodes[_max-1] = NULL;
}

//-----------------------------------------------------------------------------
void Node_Array::sort( C_sort_func_t func) {
  qsort( _nodes, _max, sizeof( Node* ), func );
}

//-----------------------------------------------------------------------------
void Node_Array::dump() const {
#ifndef PRODUCT
  for( uint i = 0; i < _max; i++ ) {
    Node *nn = _nodes[i];
    if( nn != NULL ) {
      tty->print("%5d--> ",i); nn->dump();
    }
  }
#endif
}

//=============================================================================
//------------------------------yank-------------------------------------------
// Find and remove
void Node_List::yank( Node *n ) {
  uint i;
  for( i = 0; i < _cnt; i++ )
    if( _nodes[i] == n )
      break;

  if( i < _cnt )
    _nodes[i] = _nodes[--_cnt];
}

//------------------------------dump-------------------------------------------
void Node_List::dump() const {
#ifndef PRODUCT
  for( uint i = 0; i < _cnt; i++ )
    if( _nodes[i] ) {
      tty->print("%5d--> ",i); 
      _nodes[i]->dump();
    }
#endif
}

//=============================================================================
//------------------------------remove-----------------------------------------
void Unique_Node_List::remove( Node *n ) {
  if( _in_worklist[n->_idx] ) {
    for( uint i = 0; i < size(); i++ )
      if( _nodes[i] == n ) {
        map(i,Node_List::pop());
        _in_worklist >>= n->_idx;
        return;
      }
    ShouldNotReachHere();
  }
}

//=============================================================================
uint TypeNode::size_of() const { return sizeof(*this); }
#ifndef PRODUCT
void TypeNode::dump_spec() const { 
  if( !Verbose && !WizardMode ) {
    // standard dump does this in Verbose and WizardMode
    tty->print(" #"); _type->dump();
  }
}
#endif
uint TypeNode::hash() const { 
  return Node::hash() + _type->hash(); 
}
uint TypeNode::cmp( const Node &n ) const
{ return !Type::cmp( _type, ((TypeNode&)n)._type ); }
const Type *TypeNode::bottom_type() const { return _type; }
const Type *TypeNode::Value( PhaseTransform * ) const { return _type; }

//------------------------------ideal_reg--------------------------------------
uint TypeNode::ideal_reg() const { 
  return Matcher::base2reg[_type->_base];
}


