/*
 * @(#)output.cpp	1.182 01/03/20 10:06:31
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_output.cpp.incl"

extern uint size_java_to_interp();
extern uint reloc_java_to_interp();
extern uint size_exception_handler();
extern int  size_sync_prologue();
extern int  size_sync_epilogue();

#ifndef PRODUCT
#define DEBUG_ARG(x) , x
#else
#define DEBUG_ARG(x)
#endif

extern void emit_exception_handler(CodeBuffer &cbuf);

//------------------------------Output-----------------------------------------
// Convert Nodes to instruction bits and pass off to the VM
void Compile::Output() {
  // RootNode goes
  assert( _cfg->_broot->_nodes.size() == 0, "" );

  // Make sure I can find the Start Node
  Block_Array& bbs = _cfg->_bbs;
  Block *entry = _cfg->_blocks[1];
  Block *broot = _cfg->_broot;

  const StartNode *start = entry->_nodes[0]->is_Start();

  // Replace StartNode with prolog
  MachPrologNode *prolog = new MachPrologNode();
  entry->_nodes.map( 0, prolog );
  bbs.map( prolog->_idx, entry );
  
  // Virtual methods need an unverified entry point

  if( is_osr_compilation() ) {
    if( PoisonOSREntry ) {
      // TODO: Should use a ShouldNotReachHereNode...
      _cfg->insert( broot, 0, new MachBreakpointNode() );
    }
  } else {
    if( _method && !_method->flags().is_static() ) {
      // Insert unvalidated entry point 
      _cfg->insert( broot, 0, new MachUEPNode() );

      if( OptoBreakpoint && !_method->flags().is_native())      // Break before unverified entry point chk
        _cfg->insert( broot, 0, new MachBreakpointNode() );
    } 

    // Frame Converter: Compiled to Interpreter
    else if( start && start->Opcode() == Op_StartC2I ) {
      _cfg->insert( broot, 0, new MachC2IEntriesNode() );
      _cfg->insert( entry, 0, new MachC2IcheckICNode() );

      if( OptoBreakpointC2I )      // Break before unverified entry point check
        _cfg->insert( broot, 0, new MachBreakpointNode() );
    }
  #ifdef ASSERT
    // Frame Converter: Interpreter to Compiled
    if( start && start->Opcode() == Op_StartI2C ) {
      assert( start == start, "debug point");
      // NO unverified entry point,
      // Standard Java-Signature plus methodOop passed as parameters
    }
    else if( _method && _method->flags().is_native()) {
      assert( start == start, "debug point");
    }
  #endif
  }


  // Break before main entry point
  if( (OptoBreakpoint && _method && !_method->flags().is_native())
    ||(OptoBreakpointOSR && is_osr_compilation())
    ||(OptoBreakpointC2N && _method && _method->flags().is_native()) 
    ||(OptoBreakpointC2R && !_method && (start->Opcode() != Op_StartC2I) && (start->Opcode() != Op_StartI2C)) 
    ||(OptoBreakpointC2I && start && start->Opcode() == Op_StartC2I)
    ||(OptoBreakpointI2C && start && start->Opcode() == Op_StartI2C) ){
    // checking for _method means that OptoBreakpoint does not apply to 
    // runtime stubs or frame converters
    _cfg->insert( entry, 1, new MachBreakpointNode() );
  }

  // Insert epilogs before every return
  for( uint i=0; i<_cfg->_num_blocks; i++ ) { 
    Block *b = _cfg->_blocks[i];
    if( b->_succs[0] == _cfg->_broot ) { // Found a program exit point?
      MachNode *mach = b->end()->is_Mach();
      if( mach && mach->ideal_Opcode() != Op_Halt ) {
        MachEpilogNode *epilog = new MachEpilogNode();
        b->add_inst( epilog );
        bbs.map(epilog->_idx, b);
	_chaitin->set_register(epilog, OptoReg::Bad);
      }
    }
  }

# ifdef ENABLE_ZAP_DEAD_LOCALS
  if ( ZapDeadCompiledLocals )  Insert_zap_nodes();
# endif
  
  ScheduleAndBundle();

  uint *blk_starts = Compute_size();

  // Compute size of root block; this is the unvalidated entry point (or osr poison) size
#ifdef ASSERT
  _first_block_size = 0;
  for( uint j = 0; j<broot->_nodes.size(); j++ ) {
    _first_block_size += broot->_nodes[j]->is_Mach()->size(_chaitin);
  }
  assert((uint)_first_block_size == blk_starts[1] - blk_starts[0], "1st block size correct");
#endif
  _first_block_size = blk_starts[1] - blk_starts[0];

  Fill_buffer(blk_starts);
}


# ifdef ENABLE_ZAP_DEAD_LOCALS


// In order to catch compiler oop-map bugs, we have implemented
// a debugging mode called ZapDeadCompilerLocals.
// This mode causes the compiler to insert a call to a runtime routine,
// "zap_dead_locals", right before each place in compiled code
// that could potentially be a gc-point (i.e., a safepoint or oop map point).
// The runtime routine checks that locations mapped as oops are really
// oops, that locations mapped as values do not look like oops,
// and that locations mapped as dead are not used later
// (by zapping them to an invalid address).

int Compile::CompiledZap_count = 0;

void Compile::Insert_zap_nodes() {
  bool skip = false;


  // Dink with static counts because code code without the extra
  // runtime calls is MUCH faster for debugging purposes

       if ( CompileZapFirst  ==  0  ) ; // nothing special
  else if ( CompileZapFirst  >  CompiledZap_count )  skip = true;
  else if ( CompileZapFirst  == CompiledZap_count )
    warning("starting zap compilation after skipping");

       if ( CompileZapLast  ==  -1  ) ; // nothing special
  else if ( CompileZapLast  <   CompiledZap_count )  skip = true;
  else if ( CompileZapLast  ==  CompiledZap_count )
    warning("about to compile last zap");

  ++CompiledZap_count; // counts skipped zaps, too

  if ( skip )  return;

  
  if ( _method == NULL )
    return; // no safepoints/oopmaps emitted for calls in stubs,so we don't care

  // But only for real methods (not stubs) following logic around "add_safepoint" below
  if ( _method->flags().is_native() )
    return; // cannot do calls in native stubs yet; thread is in wrong state for JRT_ENTRY

  // Insert call to zap runtime stub before every node with an oop map
  for( uint i=0; i<_cfg->_num_blocks; i++ ) { 
    Block *b = _cfg->_blocks[i];
    for ( uint j = 0;  j < b->_nodes.size();  ++j ) {
      Node *n = b->_nodes[j];

      // Determining if we should insert a zap-a-lot node in output.
      // We do that for all nodes that has oopmap info, except for calls
      // to allocation.  Calls to allocation passes in the old top-of-eden pointer
      // and expect the C code to reset it.  Hence, there can be no safepoints between
      // the inlined-allocation and the call to new_Java, etc.
      // We also cannot zap monitor calls, as they must hold the microlock
      // during the call to Zap, which also wants to grab the microlock.
      bool insert = is_node_getting_a_safepoint(n) && (n->oop_map() != NULL);
      if ( insert ) {
        MachNode* mn = n->is_Mach();
        MachCallNode* call = (mn != NULL)?mn->is_MachCall():NULL;
        MachReturnNode* mrn = (mn != NULL)?mn->is_MachReturn():NULL;
        MachSafePointNode* safepoint = (mrn != NULL)?mrn->is_MachSafePoint():NULL;
        if ( call == NULL && safepoint != NULL ) {
          insert = false;
        } else if ( call != NULL ) {
          if (call->_entrypoint == OptoRuntime::new_Java() ||
              call->_entrypoint == OptoRuntime::new_slow_Java() ||
              call->_entrypoint == OptoRuntime::new_typeArray_Java() ||
              call->_entrypoint == OptoRuntime::new_objArray_Java() ||
              call->_entrypoint == OptoRuntime::multianewarray1_Java() ||
              call->_entrypoint == OptoRuntime::multianewarray2_Java() ||
              call->_entrypoint == OptoRuntime::multianewarray3_Java() ||
              call->_entrypoint == OptoRuntime::multianewarray4_Java() ||
              call->_entrypoint == OptoRuntime::multianewarray5_Java() ||
              call->_entrypoint == OptoRuntime::complete_monitor_locking_Java() 
              ) {
            insert = false;
          }
        }
        if (insert) {
          Node *zap = call_zap_node(n, i);
          b->_nodes.insert( j, zap );
          _cfg->_bbs.map( zap->_idx, b );
          ++j;
        }
      }
    }
  }
}


Node* Compile::call_zap_node(Node* node_to_check, int block_no) {
  CallStaticJavaNode* ideal_node = 
    new CallStaticJavaNode( OptoRuntime::zap_dead_locals_Type(), 
                            OptoRuntime::zap_dead_locals_stub(_method->flags().is_native()), 
                            "call zap dead locals stub", 0);
  // We need to copy the OopMap from the site we're zapping at.
  // We have to make a copy, because the zap site might not be
  // a call site, and zap_dead is a call site.
  OopMap* clone = node_to_check->oop_map()->deep_copy();

  // Add the cloned OopMap to the zap node
  ideal_node->set_oop_map(clone);
  return _matcher->match_sfpt(ideal_node);
}

//------------------------------is_node_getting_a_safepoint--------------------
bool Compile::is_node_getting_a_safepoint( Node* n) {
  // This code duplicates the logic prior to the call of add_safepoint
  // below in this file.
  MachNode          *mach = n->is_Mach();
  if( mach == NULL ) return false;
  MachSafePointNode *sfpt = mach->is_MachSafePoint();
  if( sfpt != NULL) return true;
  return false;
}

# endif // ENABLE_ZAP_DEAD_LOCALS

//------------------------------Compute_Bundled_Size---------------------------
bool Compile::Compute_Bundled_Size(
  Block *b,
  Node *n,
  uint &j,
  Node * &delay_slot,
  MachNode *_nop_list[],
  uint blk_start,
  uint &blk_size,
  uint &last_bundle_resources
  )
{
  // See if node bundling is supported
  if (!valid_bundle_info(n))
    return true;

  Bundle *bundle = node_bundling(n);

  // If this node will be used in an unconditional delay slot,
  // then it must be 1 instruction long. Don't count it, as
  // we will later stuff it over a NOP in the code
  if (bundle->used_in_unconditional_delay()) {
#ifndef PRODUCT
    uint delay_size = n->size(_chaitin);
    assert( delay_size == Pipeline::_instr_unit_size, "delay slot instruction wrong size");
#endif
    delay_slot = n;
    return false;
  }

  // If this is disabled, give up
  if (!OptoBundling)
    return true;

  // See if we want to bundle instructions. This means we check
  // to see if need to force the alignment to be different
  uint instr_count = bundle->instr_count();

#ifndef PRODUCT
  Scheduling::increment_instructions_per_bundle(instr_count);
#endif

  if (instr_count > 1) {
    uint current_pc = blk_start + blk_size;
    uint instr_align = (current_pc >> Pipeline::_instr_fetch_unit_shift) &
      Pipeline::_instr_fetch_unit_mask;

    uint fetch_instrs = Pipeline::_instr_fetch_unit_size / Pipeline::_instr_unit_size;
    uint nop_count = (fetch_instrs - instr_align) &
      Pipeline::_instr_fetch_unit_mask;

    if (nop_count > 0 && nop_count < instr_count) {

      // Verify that we can generate appropriate nop(s)
      uint nops_found = 0;
      for ( uint l = 0; nop_count > nops_found && l < Bundle::_nop_count; l++ )
        if ((_nop_list[l]->pipeline()->resourcesUsed() & last_bundle_resources) == 0)
          nops_found++;

      if (nop_count == nops_found) {
        nops_found = 0;
        for ( uint l = 0; nops_found < nop_count; l++ ) {
          MachNode *nop = _nop_list[l]; 

          // If this overlaps used functional units, then this will cause a
          // stall. Find a different nop.
          if ((nop->pipeline()->resourcesUsed() & last_bundle_resources) != 0)
            continue;

          // Found a legal nop, insert it
          b->_nodes.insert( j++, nop );
          uint nop_size = nop->size(_chaitin);
          blk_size += nop_size;

#ifndef PRODUCT
          Scheduling::increment_nop_size(nop_size);

          if (TraceOptoOutput) {
            tty->print("### NOP Generated at %d: ", j-1);
            nop->dump();
          }
#endif
          // Increase the number of nops and the number of instrs and current instr
          nops_found++;
        }
      }
#ifndef PRODUCT
      else {
        if (TraceOptoOutput) {
          tty->print("WARNING: could not generate %d nops for resources 0x%0x at idx %d\n",
            nop_count, last_bundle_resources, n->_idx);
          n->dump();
        }
      }
#endif
    }
  }

  // Keep a pointer to the node starting the previous bundle
  if (bundle->instr_count() > 0)
    last_bundle_resources = bundle->resources_used();

  // This instruction is not to be skipped.
  return true;
}

//------------------------------Compute_size-----------------------------------
uint *Compile::Compute_size() {

  // fill in the nop array for bundling computations
  MachNode *_nop_list[Bundle::_nop_count];
  Bundle::initialize_nops(_nop_list);

  // Compute prolog code size
  _method_size = 0;
  _frame_words = SharedInfo::reg2stack(_matcher->_old_SP)+_chaitin->_framesize;
  assert( _frame_words >= 0 && _frame_words < 1000000, "sanity check" );

  // Count and start of implicit null check instructions
  uint inct_cnt = 0;
  uint *inct_starts = NEW_RESOURCE_ARRAY(uint,_cfg->_num_blocks+1);

  // ------------------
  // Compute size of each block, method size, and relocation information size
  uint *jmp_end = NEW_RESOURCE_ARRAY(uint,_cfg->_num_blocks);
  uint *blk_starts = NEW_RESOURCE_ARRAY(uint,_cfg->_num_blocks+1);
  uint *last_native_pc = NEW_RESOURCE_ARRAY(uint,_cfg->_num_blocks+1);
  blk_starts[0] = _method_size;

  int  stub_size  = 0;          // Size in bytes of all stub entries
  // Size in bytes of all relocation entries, including those in local stubs.
  // !!!!! Start with 2-bytes of reloc info for the unvalidated entry point
  int  reloc_size = 1;		// Number of relocation entries
  int  const_size = 0;          // size of fp constants in words


  // Check to see if short branches are in the architecture description
  // for this machine.  If they are, replace eligible long branches with
  // short branches.
  //
  // NOTE: The two branches in the if, below, share a good deal of code
  // and could use some cleanup/refactoring.
  if( _matcher->is_short_branch_offset(0) ) {
    // The architecture description provides short branch variants
    // for some long branch instructions.  Make three passes.  The
    // first computes pessimistic blk_starts, relative jmp_end,
    // reloc_size and const_size information.  The second performs
    // short branch substitution using the pessimistic sizing.  The
    // third computes the real blk_starts and absolute jmp_end
    // information.

    // Step one, perform a pessimistic sizing pass.
    uint i;
    for( i=0; i<_cfg->_num_blocks; i++ ) { // For all blocks
      Block *b = _cfg->_blocks[i];

      // Sum all instruction sizes to compute block size
      uint last_inst = b->_nodes.size();
      uint blk_size = 0;
      // Track state: { have not seen LoadPC |
      //   have seen LoadPC, record offset from start of block and look for call }
      MachNode *last_native_pc_node   = NULL;// LNPC
      uint      last_native_pc_offset = 0;// LNPC
      uint      inst_start            = 0;// LNPC
      for( uint j = 0; j<last_inst; j++ ) {
        inst_start = blk_size;// LNPC
        blk_size += b->_nodes[j]->size(_chaitin);
        // Handle machine instruction nodes
        MachNode *mach = b->_nodes[j]->is_Mach();
        if( mach ) {
          reloc_size += mach->reloc();
          const_size += mach->const_size();
          MachCallNode *call = mach->is_MachCall();
          if( call ) {
            // if we have seen a LoadPC node// LNPC
            // Backpatch the offset to the call's return address// LNPC
            if( last_native_pc_node != NULL ) {// LNPC
              int relative_offset = inst_start + call->ret_addr_offset() - last_native_pc_offset;// LNPC
              last_native_pc_node->label_addr(relative_offset);// LNPC
              last_native_pc_node = NULL;// LNPC
            }// LNPC
            // This destination address is NOT PC-relative
            call->label_addr((int)call->_entrypoint);
            MachCallJavaNode *jcall = call->is_MachCallJava();
            if( jcall && jcall->_method ) {
              stub_size  += size_java_to_interp();
              reloc_size += reloc_java_to_interp();
            }
          }
          // Remember the most recently seen LoadPC Node// LNPC
          else if( mach->ideal_Opcode() == Op_LoadPC ) {// LNPC
            last_native_pc_node = mach;// LNPC
            last_native_pc_offset = inst_start;// LNPC
          }// LNPC
        }
      }
      // During short branch replacement, we store the relative (to blk_starts)
      // end of jump in jmp_end, rather than the absolute end of jump.  This
      // is so that we do not need to recompute sizes of all nodes when we compute
      // correct blk_starts in our next sizing pass.
      jmp_end[i] = blk_size;

      // When the next block starts a loop, we may insert pad NOP
      // instructions.  Since we cannot know our future alignment,
      // assume the worst.
      if( i<_cfg->_num_blocks-1 ) {
        Block *nb = _cfg->_blocks[i+1];
	int max_loop_pad = nb->code_alignment()-relocInfo::addr_unit();
        if( max_loop_pad > 0 ) {
	  assert(is_power_of_2(max_loop_pad+relocInfo::addr_unit()), "");
          blk_size += max_loop_pad;
        }
      }

      // Save block size; update total method size
      blk_starts[i+1] = blk_starts[i]+blk_size;
    }

    // Step two, replace eligible long jumps.
    for( i=0; i<_cfg->_num_blocks; i++ ) {
      Block *b = _cfg->_blocks[i];

      int j;
      // Find the branch; ignore trailing NOPs.
      for( j = b->_nodes.size()-1; j>=0; j-- ) {
        MachNode *mach = b->_nodes[j]->is_Mach();
        if( !mach || mach->ideal_Opcode() != Op_Con ) 
          break;
      }

      if (j >= 0) {
        MachNode *mach = b->_nodes[j]->is_Mach();
        if( mach && mach->may_be_short_branch() ) {
          // This requires the TRUE branch target be in succs[0]
          uint16 bnum = b->_succs[0]->_pre_order;
          uint32 target = blk_starts[bnum];
          if( mach->is_pc_relative() ) {
            int offset = target-(blk_starts[i] + jmp_end[i]);
            if (_matcher->is_short_branch_offset(offset)) {
              // We've got a winner.  Replace this branch.
              MachNode *replacement = mach->short_branch_version();
              b->_nodes.map(j, replacement);
            
              // Update the jmp_end size to save time in our
              // next pass.
              jmp_end[i] -= (mach->size(_chaitin) - replacement->size(_chaitin));
            }
          } else {
#ifndef PRODUCT
            mach->dump(3);
#endif
            Unimplemented();
          }
        }
      }
    }

    // Step three, perform an accurate sizing pass.
    for( i=0; i<_cfg->_num_blocks; i++ ) { // For all blocks
      Block *b = _cfg->_blocks[i];

      // The relative jump end was tucked away in jmp_end during the last pass.
      uint blk_size = jmp_end[i];
      uint j = b->_nodes.size();
      if( j ) {
        MachNode *mach = b->_nodes[--j]->is_Mach();
        if( mach && mach->is_MachNullCheck() ) {
          int nul_chk_offset = blk_starts[i]+blk_size;
          // go back to start of prior instruction which is the
          // one doing the hardware nul check.  If its mods flags
          // there may be some mach-proj's in the way.
          while( b->_nodes[--j]->is_Proj() ) ;
          Node *load = b->_nodes[j];
#ifndef PRODUCT
	  if( mach->in(1) != load ) {
	    b->dump();
	    assert(mach->in(1) == load, "load is expected to immediately precede null check");
	  }
#endif
          nul_chk_offset -= load->size(_chaitin);
          inct_starts[inct_cnt++] = nul_chk_offset;
        }
      }
      jmp_end[i] = blk_starts[i] + blk_size;

      // If the next block _starts_ a loop, pad this block out to align
      // the loop start a little. Helps prevent pipe stalls at loop starts
      if( i<_cfg->_num_blocks-1 ) {
        Block *nb = _cfg->_blocks[i+1];
	int max_loop_pad = nb->code_alignment()-relocInfo::addr_unit();
        if( max_loop_pad > 0 ) {
	  assert(is_power_of_2(max_loop_pad+relocInfo::addr_unit()), "");
          while( (_method_size + blk_size) & max_loop_pad ) {
            MachNode *nop = new MachNopNode();
            b->_nodes.insert( b->_nodes.size(), nop );
            _cfg->_bbs.map( nop->_idx, b );
            blk_size += nop->size(_chaitin);
          }
        }
      }

      // Save block size; update total method size
      blk_starts[i+1] = blk_starts[i]+blk_size;
      _method_size += blk_size;
    } // end step three, for_each_block

  } else {
    // The architecture description does not provide short branches.
    // Compute blk_starts, jmp_end, reloc_size, and const_size in
    // a single pass.
    uint  node_bundle_limit = _node_bundle_limit;
    uint  last_bundle_resources = 0;

    for( uint i=0; i<_cfg->_num_blocks; i++ ) { // For all blocks
      Block *b = _cfg->_blocks[i];

      // Sum all instruction sizes to compute block size
      uint blk_size = 0;
      // Track state: { have not seen LoadPC |
      //   have seen LoadPC, record offset from start of block and look for call }
      MachNode *last_native_pc_node   = NULL;// LNPC
      uint      last_native_pc_offset = 0;// LNPC
      uint      inst_start            = 0;// LNPC
      Node*     delay_slot            = NULL;
      for( uint j = 0; j<b->_nodes.size(); j++ ) {

        // Get the node
        Node *n = b->_nodes[j];

        // See how bundling effects the size, including nops
        if (!Compute_Bundled_Size(b, n, j, delay_slot, _nop_list, blk_starts[i], blk_size, last_bundle_resources))
          continue;

        // Handle machine instruction nodes
        inst_start = blk_size;
        blk_size += n->size(_chaitin);
        MachNode *mach = n->is_Mach();
        if( mach ) {
          reloc_size += mach->reloc();
          const_size += mach->const_size();
          MachCallNode *call = mach->is_MachCall();
          if( call ) {
            // This destination address is NOT PC-relative
            call->label_addr((int)call->_entrypoint);
            MachCallJavaNode *jcall = call->is_MachCallJava();
            if( jcall && jcall->_method ) {
              stub_size  += size_java_to_interp();
              reloc_size += reloc_java_to_interp();
            }
            // if we have seen a LoadPC node// LNPC
            // Backpatch the offset to the call's return address// LNPC
            if( last_native_pc_node != NULL ) {// LNPC
              int relative_offset = inst_start + call->ret_addr_offset() - last_native_pc_offset;// LNPC
              last_native_pc_node->label_addr(relative_offset);// LNPC
              last_native_pc_node = NULL;// LNPC
            }// LNPC

          } else if( mach->is_MachNullCheck() ) {
            int nul_chk_offset = blk_starts[i]+blk_size;
            // go back to start of prior instruction which is the
            // one doing the hardware nul check
            nul_chk_offset -= b->_nodes[j-1]->size(_chaitin);
            inct_starts[inct_cnt++] = nul_chk_offset;
          }
          // Remember the most recently seen LoadPC Node// LNPC
          else if( mach->ideal_Opcode() == Op_LoadPC ) {// LNPC
            last_native_pc_node = mach;// LNPC
            last_native_pc_offset = inst_start;// LNPC
          }// LNPC
        }

        // See if this has a filled delay slot
        if (valid_bundle_info(n) && node_bundling(n)->use_unconditional_delay()) {
          assert(delay_slot != NULL, "expecting delay slot node");

          if (delay_slot->_idx < node_bundle_limit)
            last_bundle_resources = node_bundling(delay_slot)->resources_used();

          // Handle machine nodes in the delay slot
          MachNode *mach = delay_slot->is_Mach();
          if( mach ) {
            reloc_size += mach->reloc();
            const_size += mach->const_size();

            // Remember the most recently seen LoadPC Node// LNPC
            if( mach->ideal_Opcode() == Op_LoadPC ) {// LNPC
              last_native_pc_node = mach;// LNPC
              last_native_pc_offset = inst_start - Pipeline::_instr_unit_size; //LNPC
            }// LNPC
          }

          delay_slot = NULL;
        }
      }

      jmp_end[i] = blk_starts[i] + blk_size;

      // If the next block _starts_ a loop, pad this block out to align
      // the loop start a little. Helps prevent pipe stalls at loop starts
      if( i<_cfg->_num_blocks-1 ) {
        Block *nb = _cfg->_blocks[i+1];
	int max_loop_pad = nb->code_alignment()-relocInfo::addr_unit();
        if( max_loop_pad > 0 ) {
	  assert(is_power_of_2(max_loop_pad+relocInfo::addr_unit()), "");
          while( (_method_size + blk_size) & max_loop_pad ) {
            MachNode *nop = new MachNopNode();
            b->_nodes.insert( b->_nodes.size(), nop );
            _cfg->_bbs.map( nop->_idx, b );
            blk_size += nop->size(_chaitin);
          }
        }
      }

      // Save block size; update total method size
      blk_starts[i+1] = blk_starts[i]+blk_size;
      _method_size += blk_size;
    } // End of for all blocks compute block size
  }

  assert( _method_size < 500000, "method is unreasonably large");

  // ------------------
  // Fill in branch offsets && exception table entries.
  _inc_table.set_size(inct_cnt);
  inct_cnt = 0;
  uint i;
  for( i=0; i<_cfg->_num_blocks; i++ ) {
    Block *b = _cfg->_blocks[i];
    int j;
    // Find the branch; ignore trailing NOPs.
    for( j = b->_nodes.size()-1; j>=0; j-- ) {
      MachNode *mach = b->_nodes[j]->is_Mach();
      if( !mach || mach->ideal_Opcode() != Op_Con ) 
        break;
    }
    // Found last non-NOP node.  Check for machine branch
    if( j >= 0 ) {
      MachNode *mach = b->_nodes[j]->is_Mach();
      if( mach && mach->is_Branch() ) {
        // This requires the TRUE branch target be in succs[0]
        uint16 bnum = b->_succs[0]->_pre_order;
        uint32 target = blk_starts[bnum];
        mach->label_block( target, bnum );
        if( mach->is_pc_relative() ) {
          mach->label_addr( target-jmp_end[i] );
        } else {
#ifndef PRODUCT
          mach->dump(3);
#endif
          Unimplemented();
        }
      }
    }

    // Compute ExceptionHandlerTable subtable entry and add it
    // (skip empty blocks)
    if( j >= 0 ) {
      const Node* catch_ = b->_nodes[j];
      if( catch_->is_Catch() ) {
#ifdef ASSERT
        while( b->_nodes[--j]->Opcode() == Op_MachProj ) ;
        assert( b->_nodes[j]->is_Call(), "CatchProj must follow call" );
#endif
        // last instruction is a CatchNode, find it's CatchProjNodes
        int nof_succs = b->_num_succs;
        // allocate space 
        GrowableArray<int> handler_bcis(nof_succs);
        GrowableArray<int> handler_pcos(nof_succs);
        // iterate through all successors
        for (int j = 0; j < nof_succs; j++) {
          Block* s = b->_succs[j];
          bool found_p = false;
          for( uint k = 1; k < s->num_preds(); k++ ) {
            const CatchProjNode* p = s->pred(k)->is_CatchProj();
            if( p && p->in(0) == catch_ ) {
              found_p = true;
              // add the corresponding handler bci & pco information
              if( p->_con != CatchProjNode::fall_through_index ) {
                // p leads to an exception handler (and is not fall through)
                assert(s == _cfg->_blocks[s->_pre_order],"bad numbering");
                // no duplicates, please
                if( !handler_bcis.contains(p->handler_bci()) ) {
                  handler_bcis.append(p->handler_bci());
                  handler_pcos.append(blk_starts[s->_pre_order]);
                }
              }
            }
          }
          assert(found_p, "no matching predecessor found");
          // Note:  Due to empty block removal, one block may have
          // several CatchProj inputs, from the same Catch.
        }
        // add subtable information for this CatchNode
        Node         *proj = catch_->in(0);
        MachNode     *mach = NULL;
        MachCallNode *call = NULL;
        int call_return_pc = jmp_end[i];
        if( proj && proj->is_Proj() && (mach = proj->in(0)->is_Mach()) && (call = mach->is_MachCall()) ) {
          int call_trailer = call->size(_chaitin) - call->ret_addr_offset();
          call_return_pc  -= call_trailer;
        } else {
          assert( proj->is_Region() || proj->in(0)->is_Start(), "insanity check");
        }
        _handler_table.add_subtable(call_return_pc, &handler_bcis, &handler_pcos);
      }
    }

    // Handle implicit null exception table updates
    if( j >= 0 ) {
      MachNode* nul = b->_nodes[j]->is_Mach();
      if( nul && nul->is_MachNullCheck() ) {
        _inc_table.append(inct_starts[inct_cnt++],
                          blk_starts[b->_succs[0]->_pre_order]);
      }
    }
  } // End of for all blocks fill in branch offset & exception table entries

#ifndef PRODUCT
  // Information on the size of the method, without the extraneous code
  Scheduling::increment_method_size(_method_size);
#endif

  // ------------------
  // Allocate code buffer

  // Allocate space for the exception handler at code end
  _method_size += size_exception_handler();

  reloc_size += 1;		// Relo entry for exception handler

  // Allocate code buffer object
  _method_size += stub_size;
  // Adjust reloc_size to number of bytes of relocation info
  // Min is 2 bytes, max is probably 6 or 8, with a tax up to 25% for
  // a relocation index.
  reloc_size   *= 10;
  // Adjust const_size to number of bytes
  const_size   *= 2*wordSize; // both float and double take two words per entry
 _method_size += const_size;

  // nmethod and CodeBuffer count stubs as part of method's code.
  _code_buffer  = new CodeBuffer(_method_size, reloc_size, stub_size, const_size, false);
  _code_base    = _code_buffer->code_begin();
  _code_buffer->set_oop_recorder(_recorder->oop_recorder());

  return blk_starts;
}

//------------------------------FillLocArray-----------------------------------
// Create a bit of debug info and append it to the array.  The mapping is from
// Java local or expression stack to constant, register or stack-slot.  For
// doubles, insert 2 mappings and return 1 (to tell the caller that the next 
// entry has been taken care of and caller should skip it).
static LocationValue *new_loc_value( PhaseChaitin *pc, OptoReg::Name regnum, Location::Type l_type ) {
  return (regnum < SharedInfo::stack0)
    ? new LocationValue(Location::new_reg_loc(l_type,                regnum) )
    : new LocationValue(Location::new_stk_loc(l_type, pc->reg2offset(regnum)));
}

int Compile::FillLocArray( Node *local, GrowableArray<ScopeValue*> *array ) {
  assert( local, "use _top instead of null" );
  const Type *t = local->bottom_type();

  // Grab the register number for the local
  OptoReg::Name regnum = _chaitin->get_register(local);
  if( regnum != OptoReg::Bad ) {// Got a register?
    assert( _chaitin->Find(local), "live range exists" );
    // Record the double as two float registers.
    // The register mask for such a value always specifies two adjacent
    // float registers, with the lower register number even.
    // Normally, the allocation of high and low words to these registers
    // is irrelevant, because nearly all operations on register pairs
    // (e.g., StoreD) treat them as a single unit.
    // Here, we assume in addition that the words in these two registers
    // stored "naturally" (by operations like StoreD and double stores
    // within the interpreter) such that the lower-numbered register
    // is written to the lower memory address.  This may seem like
    // a machine dependency, but it is not--it is a requirement on
    // the author of the <arch>.ad file to ensure that, for every
    // even/odd double-register pair to which a double may be allocated,
    // the word in the even single-register is stored to the first
    // memory word.  (Note that register numbers are completely
    // arbitrary, and are not tied to any machine-level encodings.)
    if( t->_base == Type::DoubleBot || t->_base == Type::DoubleCon || t->_base == Type::Long ) { 
    // Repack the double/long as two jints.
    // The convention the interpreter uses is that the second local
    // holds the first raw word of the native double representation.
    // This is actually reasonable, since locals and stack arrays
    // grow downwards in all implementations.
    // (If, on some machine, the interpreter's Java locals or stack
    // were to grow upwards, the embedded doubles would be word-swapped.)
      array->append(new_loc_value( _chaitin, OptoReg::add(regnum,1), Location::normal ));
      array->append(new_loc_value( _chaitin,              regnum   , Location::normal ));
      return 1;                 // Yes a double/long
    } else if( (t->_base == Type::FloatBot || t->_base == Type::FloatCon) &&
               regnum < SharedInfo::stack0 ) { 
      array->append(new_loc_value( _chaitin, regnum, Matcher::float_in_double 
                                   ? Location::float_in_dbl : Location::normal ));
    } else {
      array->append(new_loc_value( _chaitin, regnum, _chaitin->is_oop(local) ? Location::oop : Location::normal ));
    }
    return 0;                   // Not a double
  }
  assert( !_chaitin->Find(local), "live range does not exist" );

  // No register.  It must be constant data.
  switch( t->_base ) {
  case Type::Half:              // Second half of a double
    ShouldNotReachHere();       // Caller should skip 2nd halves
    break;
  case Type::AnyPtr:
    array->append(new ConstantOopWriteValue(NULL));
    break;
  case Type::AryPtr: 
  case Type::OopPtr: 
  case Type::KlassPtr:          // fall through
    array->append(new ConstantOopWriteValue(t->isa_javaptr()->const_oop()->encoding()));
    break;
  case Type::Int:    
    array->append(new ConstantIntValue(t->is_int()->get_con())); 
    break;
  case Type::FloatCon: {
    float f = t->is_float_constant()->getf();
    array->append(new ConstantIntValue(jint_cast(f)));
    break;
  }
  case Type::DoubleCon: {
    jdouble d = t->is_double_constant()->getd();
    // Repack the double as two jints.
    // The convention the interpreter uses is that the second local
    // holds the first raw word of the native double representation.
    // This is actually reasonable, since locals and stack arrays
    // grow downwards in all implementations.
    // (If, on some machine, the interpreter's Java locals or stack
    // were to grow upwards, the embedded doubles would be word-swapped.)
    jint   *dp = (jint*)&d;
    array->append(new ConstantIntValue(dp[1]));
    array->append(new ConstantIntValue(dp[0]));
    return 1;
  }
  case Type::Long: {
    jlong d = t->is_long()->get_con();
    // Repack the long as two jints.
    // The convention the interpreter uses is that the second local
    // holds the first raw word of the native double representation.
    // This is actually reasonable, since locals and stack arrays
    // grow downwards in all implementations.
    // (If, on some machine, the interpreter's Java locals or stack
    // were to grow upwards, the embedded doubles would be word-swapped.)
    jint *dp = (jint*)&d;
    array->append(new ConstantIntValue(dp[1]));
    array->append(new ConstantIntValue(dp[0]));
    return 1;
  }
  case Type::Top:               // Add an illegal value here
    array->append(new LocationValue(Location())); 
    break;
  default:
    ShouldNotReachHere();
    break;
  }                     
  return 0;
}

//------------------------------Fill_buffer------------------------------------
void Compile::Fill_buffer( uint *blk_starts ) {
  // Create oopmap set. 
  _oop_map_set = new OopMapSet();

  // !!!!! This preserves old handling of oopmaps for now
  _recorder->set_oopmaps(_oop_map_set);

  // Dump the assembly code, including basic-block numbers
  if( (PrintOptoAssembly && _method) || PrintFrameConverterAssembly ) {
    if( _method ) {
      _method->print_oop();
      print_codes();
    }
#ifndef PRODUCT
    dump_asm();
#endif
  }

  // ------------------
  // Now fill in the code buffer

  Node *delay_slot = NULL;

  for( uint i=0; i<_cfg->_num_blocks; i++ ) {
    Block *b = _cfg->_blocks[i];
    uint last_inst = b->_nodes.size();

    guarantee(_code_buffer->code_end() == _code_buffer->code_begin() + blk_starts[i], "block layout must be predicted accurately");

    // Emit block normally, except for last instruction.
    // Emit means "dump code bits into code buffer".
    for( uint j = 0; j<last_inst; j++ ) {

      // Get the node
      Node *n = b->_nodes[j];

      // See if node bundling is supported
      if (valid_bundle_info(n) &&
          node_bundling(n)->used_in_unconditional_delay()) {
        assert(delay_slot == NULL, "no use of delay slot node");
        assert(n->size(_chaitin) == Pipeline::_instr_unit_size, "delay slot instruction wrong size");
        delay_slot = n;
        continue;
      }

      // Special handling for SafePoint/Call Nodes

      // The following logic is duplicated in the code ifdeffed for
      // ENABLE_ZAP_DEAD_LOCALS which apppears above in this file.  It
      // should be factored out.  Or maybe dispersed to the nodes?

      MachNode *mach = n->is_Mach();
      MachSafePointNode *sfn = NULL;
      MachCallNode *mcall = NULL;
      if( mach ) {
	sfn = mach->is_MachSafePoint();
	mcall = mach->is_MachCall();
        if (mcall && !mcall->is_safepoint_node()) {
          mcall = NULL;
          sfn = NULL;
        }
      }
      // sfn will be valid whenever mcall is valid now because of inheritance
      if( sfn || mcall ) {
	// Grab the PC (ie - offset into the code buffer) of this safe point
	int pc = _code_buffer->code_end()-_code_buffer->code_begin();

	// Handle special safepoint nodes for synchronization
	if( mcall == NULL ) {
	  // !!!!! Stubs only need an oopmap right now, so bail out
	  if( sfn->_jvms->method() == NULL) {
	    // Write the oopmap directly to the code blob??!!
#           ifdef ENABLE_ZAP_DEAD_LOCALS
            assert( !is_node_getting_a_safepoint(sfn),  "logic does not match; false positive");
#           endif
	    continue;
	  }
	} // End synchronization

        // JVMPI -- line number to offset mapping
        if( _method != NULL && !_method->flags().is_native() && is_set_jvmpi_method_event() ) {
          JVMStateObj *jvms = sfn->_jvms;
          if( jvms != NULL ) {
            int bci = jvms->bci();
            if( bci != SynchronizationEntryBCI ) {
              assert( _linenumbers_offset != NULL, "insanity check");
              _linenumbers_offset->add_offset(bci, pc/* code offset */);
            }
          }
        }

#       ifdef ENABLE_ZAP_DEAD_LOCALS
        assert( is_node_getting_a_safepoint(n),  "logic does not match; false negative");
#       endif

	// Add the safepoint in the DebugInfoRecorder
        if( mcall == NULL ) {
	  _recorder->add_safepoint(pc, false, sfn->_oop_map);
        } else {
	  _recorder->add_safepoint(pc+mcall->ret_addr_offset(), true , mcall->_oop_map);
        }

	// Loop over the JVMStateList to add scope information
        // Do not skip safepoints with a NULL method, they need monitor info
	JVMStateObj *jvms = sfn->_jvms;
	while( jvms /* && jvms->method() != NULL */) {
          int idx;
          ciMethod* method = jvms->method();
          // Safepoints that do not have method() set only provide oop-map and monitor info
          // to support GC and SpecLocking; these do not support deoptimization.
          int num_locs = (method == NULL) ? 0 : (method->max_locals());
          int num_exps = (method == NULL) ? 0 : (jvms->monoff() - jvms->spoff());
          int num_mon  = jvms->nof_monitors();

          // Add Local and Expression Stack Information

          // Insert locals into the locarray
	  GrowableArray<ScopeValue*> *locarray = new GrowableArray<ScopeValue*>(num_locs);
	  for( idx = 0; idx < num_locs; idx++ ) 
	    idx += FillLocArray( sfn->local(jvms, idx), locarray );

          // Insert expression stack entries into the exparray
	  GrowableArray<ScopeValue*> *exparray = new GrowableArray<ScopeValue*>(num_exps);
	  for( idx = 0; idx < num_exps; idx++ ) 
	    idx += FillLocArray( sfn->stack(jvms, idx), exparray );

	  // Add in mappings of the monitors
          assert( !method ||
                  !method->is_synchronized() || 
                  method->is_native() || 
                  num_mon > 0 || 
                  (mcall && mcall->_entrypoint == OptoRuntime::jvmpi_method_entry_Java()) ||
                  (mcall && mcall->_entrypoint == OptoRuntime::jvmpi_method_exit_Java()) ||
                  !GenerateSynchronizationCode, 
                  "monitors must always exist for synchronized methods");

	  // Build the growable array of ScopeValues for exp stack
	  GrowableArray<MonitorValue*> *monarray = new GrowableArray<MonitorValue*>(num_mon);
	  	  
          // Loop over monitors and insert into array
          for(idx = 0; idx < num_mon; idx++) {
	    // Grab the node that defines this monitor
            Node* box_node;
            Node* obj_node;
	    box_node = sfn->monitor_box(jvms, idx);
	    obj_node = sfn->monitor_obj(jvms, idx);

            // Create ScopeValue for object
            ScopeValue *scval = NULL;
            if( _chaitin->Find_const(obj_node) ) {
              OptoReg::Name obj_reg = _chaitin->get_register(obj_node);
              scval = new_loc_value( _chaitin, obj_reg, Location::oop );
            } else {
              scval = new ConstantOopWriteValue(obj_node->bottom_type()->is_oopptr()->const_oop()->encoding());
            }

            OptoReg::Name box_reg = BoxLockNode::stack_slot(box_node);
            monarray->append(new MonitorValue(scval, Location::new_stk_loc(Location::normal,_chaitin->reg2offset(box_reg))));
	  }	

	  // Build first class objects to pass to scope
	  DebugToken *locvals = _recorder->create_scope_values(locarray);
	  DebugToken *expvals = _recorder->create_scope_values(exparray);
	  DebugToken *monvals = _recorder->create_monitor_values(monarray);

          // Make method available for all Safepoints
          ciMethod* scope_method = method ? method : _method;
	  // Describe the scope here
          _recorder->describe_scope(scope_method,jvms->_bci,locvals,expvals,monvals);
	  // Step to next in list
	  jvms = jvms->next();
	} // End jvms loop
      } // End if safepoint
      
      // "Normal" instruction case
      n->emit(*_code_buffer, _chaitin);

      // See if this instruction has a delay slot
      if ( valid_bundle_info(n) && node_bundling(n)->use_unconditional_delay()) {
        assert(delay_slot != NULL, "expecting delay slot node");
  
        // Back up 1 instruction
        _code_buffer->set_code_end(
          _code_buffer->code_end()-Pipeline::_instr_unit_size);
  
        // Insert the delay slot instruction
        delay_slot->emit(*_code_buffer, _chaitin);

        // Don't reuse it
        delay_slot = NULL;
      }
    } // End for all instructions in block
  } // End of for all blocks

  // Emit the exception handler code
  _code_buffer->set_exception_offset(_code_buffer->code_size());
  emit_exception_handler(*_code_buffer);

  // Generate the relocation info for stubs, where reloc info was out-of-line
  _code_buffer->relocate_stubs();

  // Include stubs in the code area
  _code_buffer->set_code_end_after_constants();
}


// Static Variables
#ifndef PRODUCT
uint Scheduling::_total_nop_size = 0;
uint Scheduling::_total_method_size = 0;
uint Scheduling::_total_branches = 0;
uint Scheduling::_total_unconditional_delays = 0;
uint Scheduling::_total_instructions_per_bundle[Pipeline::_max_instrs_per_bundle+1];
#endif

// Initializer for class Scheduling

Scheduling::Scheduling(Arena *arena, Compile &compile)
  : _arena(arena), 
    _cfg(compile._cfg), 
    _bbs(compile._cfg->_bbs), 
    _chaitin(compile._chaitin),
    _bundle_instr_count(0),
    _bundle_cycle_number(0),
    _scheduled(arena),
    _available(arena),
    _mach_projs(arena),
    _next_assignment(arena),
    _writes_oop(arena),
    _spillcopy_write(arena),
    _next_call(NULL),
    _next_node(NULL),
    _frame_pointer(NULL),
    _bundle_use(0, 0, resource_count, &_bundle_use_elements[0])
#ifndef PRODUCT
  , _branches(0)
  , _unconditional_delays(0)
#endif
{
  uint i;

  // Create a MachNopNode
  _nop = new MachNopNode();

  // Now that the nops are in the array, save the count
  // (but allow entries for the nops)
  compile._node_bundle_limit = _node_bundle_limit = compile._unique;

  // Compute the number of unsigned ints needed to represent the node count
  // as a bit vector
  uint bit_vector_count = (_node_bundle_limit + 31) >> 5;
  _bit_vector_size = bit_vector_count * sizeof(uint);

  // This one is persistant within the Compile class
  compile._node_bundling = _node_bundling =
    NEW_ARENA_ARRAY((&compile._comp_arena), Bundle, _node_bundle_limit);

  // Allocate space for fixed-size arrays
  _node_latency    = NEW_ARENA_ARRAY(arena, unsigned short, _node_bundle_limit);
  _uses            = NEW_ARENA_ARRAY(arena, short,          _node_bundle_limit);
  _current_latency = NEW_ARENA_ARRAY(arena, unsigned short, _node_bundle_limit);
  _visited         = NEW_ARENA_ARRAY(arena, uint,           bit_vector_count);
  _found           = NEW_ARENA_ARRAY(arena, uint,           bit_vector_count);

  // Clear the arrays
  memset(_node_bundling,   0, _node_bundle_limit * sizeof(Bundle));
  memset(_node_latency,    0, _node_bundle_limit * sizeof(unsigned short));
  memset(_uses,            0, _node_bundle_limit * sizeof(short));
  memset(_current_latency, 0, _node_bundle_limit * sizeof(unsigned short));
  memset(_visited,         0, _bit_vector_size);
  memset(_found,           0, _bit_vector_size);

  // Clear the bundling information
  memcpy(_bundle_use_elements,
    Pipeline_Use::elaborated_elements,
    sizeof(Pipeline_Use::elaborated_elements));

  // Compute the number of entry instructions
  Block *bb = _cfg->_blocks[1];
  uint bb_size = bb->_nodes.size();

  // Skip prologs and breakpoints
  for (i = 0; i < bb_size; i++)
    if (bb->_nodes[i]->Opcode() == Op_MachProj)
      break;

  // Skip MachProj's.  Also find the frame pointer.
  for (; i < bb_size; i++) {
    ProjNode *n = bb->_nodes[i]->is_Proj();
    if( !n ) break;		// Not a MachProj
    assert( n->Opcode() == Op_MachProj, "" );
    if( n->_con == TypeFunc::FramePtr )
      _frame_pointer = n;
  }

  assert(_frame_pointer != NULL, "no frame pointer MachProj found");

  _entry_instructions = i;

  // Get the last node
  bb = _cfg->_blocks[_cfg->_blocks.size()-1];

  _next_node = bb->_nodes[bb->_nodes.size()-1];
}

// Scheduling destructor
Scheduling::~Scheduling() {
#ifndef PRODUCT
  _total_branches             += _branches;
  _total_unconditional_delays += _unconditional_delays;
#endif
}

// Step ahead "i" cycles
void Scheduling::step(uint i) {

  // Update the bundle record, but leave the flags information alone
  if (_bundle_instr_count > 0) {
    Bundle *bundle = node_bundling(_next_node);
    bundle->set_instr_count(_bundle_instr_count);
    bundle->set_resources_used(_bundle_use.resourcesUsed());
  }

  // Update the state information
  _bundle_instr_count = 0;
  _bundle_cycle_number += i;
  _bundle_use.step(i);
}

void Scheduling::step_and_clear() {
  if (_bundle_instr_count > 0) {

    // Update the bundle record
    Bundle *bundle = node_bundling(_next_node);
    bundle->set_instr_count(_bundle_instr_count);
    bundle->set_resources_used(_bundle_use.resourcesUsed());

    _bundle_cycle_number += 1;
  }

  // Clear the bundling information
  _bundle_instr_count = 0;
  _bundle_use.reset();

  memcpy(_bundle_use_elements,
    Pipeline_Use::elaborated_elements,
    sizeof(Pipeline_Use::elaborated_elements));
}

//------------------------------ScheduleAndBundle------------------------------
// Perform instruction scheduling and bundling over the sequence of
// instructions in backwards order.
void Compile::ScheduleAndBundle() {

  // Don't optimize this if it isn't a method
  if (!_method)
    return;

  // Don't optimize this if scheduling is disabled
  if (!OptoScheduling)
    return;

  TraceTime t2(NULL, &_t_instrSched, TimeCompiler, false);

  // Create a data structure for all the scheduling information
  Scheduling scheduling(Thread::current()->resource_area(), *this);

  // Walk backwards over each basic block, computing the needed alignment
  // Walk over all the basic blocks
  scheduling.DoScheduling();
}

//------------------------------ComputeLocalLatenciesForward-------------------
// Compute the latency of all the instructions. This is fairly simple,
// because we already have a legal ordering. Walk over the instructions
// from first to last, and compute the latency of the instruction based
// on the latency of the preceeding instruction(s).
void Scheduling::ComputeLocalLatenciesForward(const Block *bb) {
#ifndef PRODUCT
  if (TraceOptoOutput)
    tty->print("# -> ComputeLocalLatenciesForward\n");
#endif

  uint nodes_size = bb->_nodes.size();

  // Walk over all the instructions
  for( uint j=0; j < nodes_size; j++ ) {

    // This is a kludge, forcing all latency calculations to start at 1.
    // Used to allow latency 0 to force an instruction to the beginning
    // of the bb
    uint latency = 1;
    Node *use = bb->_nodes[j];
    uint nlen = use->len();

    // Walk over all the inputs
    for ( uint k=0; k < nlen; k++ ) {
      Node *def = use->in(k);
      if (!def)
        continue;

      if (def->is_SpillCopy()) {
        do {
          def = def->in(MemNode::Memory);
        } while (def->is_SpillCopy());
      }

      if (not_found(def->_idx))
        continue;

      uint l = _node_latency[def->_idx] + use->latency(k);
      if (latency < l)
        latency = l;
    }

    _node_latency[use->_idx] = latency;

#ifndef PRODUCT
    if (TraceOptoOutput) {
      tty->print("# latency %4d: ", latency);
      use->dump();
    }
#endif
  }

#ifndef PRODUCT
  if (TraceOptoOutput)
    tty->print("# <- ComputeLocalLatenciesForward\n");
#endif

} // end ComputeLocalLatenciesForward

// See if this node fits into the present instruction bundle
bool Scheduling::NodeFitsInBundle(Node *n) {
  uint n_idx = n->_idx;

  // If this is the unconditional delay instruction, then it fits
  if (n == _unconditional_delay_slot) {
#ifndef PRODUCT
    if (TraceOptoOutput)
      tty->print("#     NodeFitsInBundle [%4d]: TRUE; is in unconditional delay slot\n", n->_idx);
#endif
    return (true);
  }

  // If the node cannot be scheduled this cycle, skip it
  if (_current_latency[n_idx] > _bundle_cycle_number) {
#ifndef PRODUCT
    if (TraceOptoOutput)
      tty->print("#     NodeFitsInBundle [%4d]: FALSE; latency %4d > %d\n",
        n->_idx, _current_latency[n_idx], _bundle_cycle_number);
#endif
    return (false);
  }

  const Pipeline *node_pipeline = n->pipeline();

  uint instruction_count = node_pipeline->instructionCount();
  if (node_pipeline->mayHaveNoCode() && n->size(_chaitin) == 0)
    instruction_count = 0;
  else if (node_pipeline->hasBranchDelay() && !_unconditional_delay_slot)
    instruction_count++;

  if (_bundle_instr_count + instruction_count > Pipeline::_max_instrs_per_bundle) {
#ifndef PRODUCT
    if (TraceOptoOutput)
      tty->print("#     NodeFitsInBundle [%4d]: FALSE; too many instructions: %d > %d\n",
        n->_idx, _bundle_instr_count + instruction_count, Pipeline::_max_instrs_per_bundle);
#endif
    return (false);
  }

  // Don't allow non-machine nodes to be handled this way
  if (!n->is_Mach() && instruction_count == 0)
    return (false);

  // See if there is any overlap
  uint delay = _bundle_use.full_latency(0, node_pipeline->resourceUse());

  if (delay > 0) {
#ifndef PRODUCT
    if (TraceOptoOutput)
      tty->print("#     NodeFitsInBundle [%4d]: FALSE; functional units overlap\n", n_idx);
#endif
    return false;
  }

#ifndef PRODUCT
  if (TraceOptoOutput)
    tty->print("#     NodeFitsInBundle [%4d]:  TRUE\n", n_idx);
#endif

  return true;
}

Node * Scheduling::ChooseNodeToBundle() {
  uint siz = _available.size();

  if (siz == 0) {

#ifndef PRODUCT
    if (TraceOptoOutput)
      tty->print("#   ChooseNodeToBundle: NULL\n");
#endif
    return (NULL);
  }

  // Fast path, if only 1 instruction in the bundle
  if (siz == 1) {
#ifndef PRODUCT
    if (TraceOptoOutput) {
      tty->print("#   ChooseNodeToBundle (only 1): ");
      _available[0]->dump();
    }
#endif
    return (_available[0]);
  }

  // Don't bother, if the bundle is already full
  if (_bundle_instr_count < Pipeline::_max_instrs_per_bundle) {
    for ( uint i = 0; i < siz; i++ ) {
      Node *n = _available[i];

      // Skip projections, we'll handle them another way
      if (n->is_Proj())
        continue;

      // This presupposed that instructions are inserted into the
      // available list in a legality order; i.e. instructions that
      // must be inserted first are at the head of the list
      if (NodeFitsInBundle(n)) {
#ifndef PRODUCT
        if (TraceOptoOutput) {
          tty->print("#   ChooseNodeToBundle: ");
          n->dump();
        }
#endif
        return (n);
      }
    }
  }

  // Nothing fits in this bundle, choose the highest priority
#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("#   ChooseNodeToBundle: ");
    _available[0]->dump();
  }
#endif

  return _available[0];
}

void Scheduling::AddNodeToAvailableList(Node *n) {
#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("#   AddNodeToAvailableList: ");
    n->dump();
  }
#endif

  if (n->Opcode() == Op_MachProj) {
    _mach_projs.push(n);
    return;
  }

  MachNode *mach = n->is_Mach();
  int iop = mach ? mach->ideal_Opcode() : 0;

  // These go last in the available list
  if( n->is_Proj()         || // Projections always win
      n->Opcode()== Op_Con || // So does constant 'Top'
      iop == Op_CreateEx      // Create-exception must start block
      ) {
    _node_latency[n->_idx] = 0;
    _available.push(n);
    return;
  }

  // If this is not a machine node, then push it on the end of the list
  // and ignore latencies
  if (!mach) {
    _available.push(n);
    return;
  }

  // If the list is empty, then push it on the list
  if (_available.size() == 0) {
    _available.push(n);
    return;
  }

  int latency = _current_latency[n->_idx];

  // Skip the head of the available list if it must be last
  uint i = 0;
  if (_available[0]->is_last_instr())
    i = 1;

  // See if there is a place to insert it

  for (; i < _available.size(); i++) {
    Node *p = _available[i];
    MachNode *m = p->is_Mach();

    // Everything remaining on the worklist is not a MachNode, and this is
    if (!m)
      break;

    // If the latency of this instruction is greater, insert it
    int l = _current_latency[p->_idx];

    // Check to see if the list item must be cloned; don't insert
    // this node ahead of something that is must_clone.
    if (must_clone[m->ideal_Opcode()]) {
      if (must_clone[iop] && l > latency)
        break;
      continue;
    }

    if (l > latency)
      break;
    
    // When the latency is the same, see if this is must_clone
    if (l == latency && must_clone[iop]) 
      break;
  }

  // Insert the node in the available list
  if ( i < _available.size() )
    _available.insert(i, n);
  else
    _available.push(n);

#ifndef PRODUCT
  if (TraceOptoOutput)
    dump_available();
#endif
}

void Scheduling::DecrementUseCounts(Node *n) {

  uint i = 0;
  if (n->Opcode() == Op_MachProj)
    i = 1;
  uint len = n->len();

  for ( ; i < len; i++ ) {
    Node *def = n->in(i);
    if (!def || def == n)
      continue;

    if (def->is_Start() || def->is_Root())
      def = _cfg->_blocks[1]->_nodes[0];

    if (def->is_SpillCopy()) {
      do {
        def = def->in(MemNode::Memory);
      } while (def->is_SpillCopy());
    }

    uint def_idx = def->_idx;

    if (not_found(def_idx))
      continue;

    // Compute the latency
    uint l = _bundle_cycle_number + n->latency(i);
    if (_current_latency[def_idx] < l)
      _current_latency[def_idx] = l;

    // If this does not have uses then schedule it
    if ((--_uses[def_idx]) == 0)
      AddNodeToAvailableList(def);

    // If this is a machine projection, then propagate the
    // usage thru to the base instruction
    if (def->Opcode() == Op_MachProj) {
      Node *base = def->in(0);
      if (!base->is_Start() && !base->is_Root() && (--_uses[base->_idx]) == 0)
        AddNodeToAvailableList(base);
    }
  }
}

void Scheduling::AddNodeToBundle(Node *n) {
#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("#   AddNodeToBundle: ");
    n->dump();
  }
#endif

  int i;
  uint mpstart = 0;
  uint mpcount = 0;

  // If the instruction has any machine projections following it,
  // then schedule them
  for ( i = _mach_projs.size()-1; i >= 0; i-- ) {
    Node *a = _mach_projs[i];
    // If this is this instruction, then schedule it
    if (a->in(0) == n) {
      if (mpcount == 0)
        mpstart = _scheduled.size();
      mpcount++;
      _scheduled.push(a);
      _mach_projs.remove(i);
    }
  }
 
  // Remove this from the available list
  int asize = _available.size();

  for (i = 0; i < asize; i++)
    if (_available[i] == n) {
      _available.remove(i);
      break;
  }

#ifndef PRODUCT
  assert(i < asize, "entry in _available list not found");
#endif

  // See if this fits in the current bundle
  const Pipeline *node_pipeline = n->pipeline();
  const Pipeline_Use& node_usage = node_pipeline->resourceUse();

  // Check for instructions to be placed in the delay slot. We
  // do this before we actually schedule the current instruction,
  // because the delay slot follows the current instruction.
  if (Pipeline::_branch_has_delay_slot &&
      node_pipeline->hasBranchDelay() &&
      !_unconditional_delay_slot) {

    uint siz = _available.size();

    // Conditional branches can support an instruction that
    // is unconditionally executed and not dependant by the
    // branch, OR a conditionally executed instruction if
    // the branch is taken. In practice, this means that
    // the first instruction at the branch target is
    // copied to the delay slot, and the branch goes to
    // the instruction after that at the branch target
    MachNode *mach = n->is_Mach();
    if ( mach && mach->is_Branch() ) {

      assert( !mach->is_MachNullCheck(), "should not look for delay slot for Null Check" );
      assert( !mach->is_Catch(),         "should not look for delay slot for Catch" );

#ifndef PRODUCT
      _branches++;
#endif

      // At least 1 instruction is on the available list
      // that is not dependant on the branch
      for (uint i = 0; i < siz; i++) {
        Node *d = _available[i];
        const Pipeline *avail_pipeline = d->pipeline();

        // Don't allow safepoints in the branch shadow, that will
        // cause a number of difficulties
        if ( avail_pipeline->instructionCount() == 1 &&
             d->size(_chaitin) == Pipeline::_instr_unit_size &&
            !avail_pipeline->hasMultipleBundles() &&
            !avail_pipeline->hasBranchDelay() &&
            NodeFitsInBundle(d) &&
            !node_bundling(d)->used_in_delay()) {
          MachNode *mach = d->is_Mach();

          if (mach && (!mach->is_MachSafePoint()) && (mach->ideal_Opcode() != Op_LoadPC) ) {
            // A node that fits in the delay slot was found, so we need to
            // set the appropriate bits in the bundle pipeline information so
            // that it correctly indicates resource usage.  Later, when we
            // attempt to add this instruction to the bundle, we will skip
            // setting the resource usage.
            _unconditional_delay_slot = d;
            node_bundling(n)->set_use_unconditional_delay();
            node_bundling(d)->set_used_in_unconditional_delay();
            _bundle_use.add_usage(avail_pipeline->resourceUse());
            _current_latency[d->_idx] = _bundle_cycle_number;
            _next_node = d;
            ++_bundle_instr_count;
#ifndef PRODUCT
            _unconditional_delays++;
#endif
            break;
          }
        }
      }
    }

    // No delay slot, add a nop to the usage
    if (!_unconditional_delay_slot) {
      // See if adding an instruction in the delay slot will overflow
      // the bundle.
      if (!NodeFitsInBundle(_nop)) {
#ifndef PRODUCT
        if (TraceOptoOutput)
          tty->print("#  *** STEP(1 instruction for delay slot) ***\n");
#endif
        step(1);
      }

      _bundle_use.add_usage(_nop->pipeline()->resourceUse());
      _next_node = _nop;
      ++_bundle_instr_count;
    }

    // See if the instruction in the delay slot requires a
    // step of the bundles
    if (!NodeFitsInBundle(n)) {
#ifndef PRODUCT
        if (TraceOptoOutput)
          tty->print("#  *** STEP(branch won't fit) ***\n");
#endif
        // Update the state information
        _bundle_instr_count = 0;
        _bundle_cycle_number += 1;
        _bundle_use.step(1);
    }
  }

  // Get the number of instructions
  uint instruction_count = node_pipeline->instructionCount();
  if (node_pipeline->mayHaveNoCode() && n->size(_chaitin) == 0)
    instruction_count = 0;

  // Compute the latency information
  uint delay = 0;

  if (instruction_count > 0 || !node_pipeline->mayHaveNoCode()) {
    int relative_latency = _current_latency[n->_idx] - _bundle_cycle_number;
    if (relative_latency < 0)
      relative_latency = 0;

    delay = _bundle_use.full_latency(relative_latency, node_usage);

    // Does not fit in this bundle, start a new one
    if (delay > 0) {
      step(delay);

#ifndef PRODUCT
      if (TraceOptoOutput)
        tty->print("#  *** STEP(%d) ***\n", delay);
#endif
    }
  }

  // If this was placed in the delay slot, ignore it
  if (n != _unconditional_delay_slot) {

    if (delay == 0) {
      if (node_pipeline->hasMultipleBundles()) {
#ifndef PRODUCT
        if (TraceOptoOutput)
          tty->print("#  *** STEP(multiple instructions) ***\n");
#endif
        step(1);
      }

      else if (instruction_count + _bundle_instr_count > Pipeline::_max_instrs_per_bundle) {
#ifndef PRODUCT
        if (TraceOptoOutput)
          tty->print("#  *** STEP(%d >= %d instructions) ***\n",
            instruction_count + _bundle_instr_count,
            Pipeline::_max_instrs_per_bundle);
#endif
        step(1);
      }
    }

    if (node_pipeline->hasBranchDelay() && !_unconditional_delay_slot)
      _bundle_instr_count++;

    // Set the node's latency
    _current_latency[n->_idx] = _bundle_cycle_number;

    // Now merge the functional unit information
    if (instruction_count > 0 || !node_pipeline->mayHaveNoCode())
      _bundle_use.add_usage(node_usage);

    // Increment the number of instructions in this bundle
    _bundle_instr_count += instruction_count;

    // Remember this node for later
    if (n->is_Mach())
      _next_node = n;
  }

  // Put the instruction in the schedule list
  _scheduled.push(n);

#ifndef PRODUCT
  if (TraceOptoOutput)
    dump_available();
#endif

  // Walk all the definitions, decrementing use counts, and
  // if a definition has a 0 use count, place it in the available list.

  // Process additional inputs to a MachProj
  for ( ; mpcount > 0; mpcount-- )
    DecrementUseCounts(_scheduled[mpstart++]);

  // Decrement Use Counts for the node
  DecrementUseCounts(n);
}

// This method sets the use count within a basic block. We will ignore all uses
// outside the current basic block. As we are doing a backwards walk, any node
// we reach that has a use count of 0 may be scheduled. This also avoids the
// problem of cyclic references from phi nodes, as long as phi nodes are at the
// front of the basic block. This method also initializes the available list
// to the set of instructions that have no uses within this basic block.
void Scheduling::ComputeUseCount(const Block *bb) {
#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("# -> ComputeUseCount\n");
  }
#endif

  int j;

  // Clear the list of available and scheduled instructions, just in case
  _available.clear();
  _mach_projs.clear();
  _scheduled.clear();

  // No delay slot specified
  _unconditional_delay_slot = NULL;

  // Don't walk over the leading instruction(s)
  int lb = (bb != _cfg->_blocks[1]) ? 1 : _entry_instructions;

  // Iterate backwards over the instructions in the block
  // (ignoring branch projections)
  for ( j=bb->_nodes.size()-1; j >= lb; j-- ) {
    Node *n = bb->_nodes[j];

    // Mark as visited; this is to ignore backwards edges *KLUDGE*
    set_visited(n->_idx);

    uint k = 0;

    // Special case for MachProj, skip the referenced node
    if (n->Opcode() == Op_MachProj)
      k = 1;

    for ( ; k < n->len(); k++ ) {
      Node *inp = n->in(k);
      if (!inp)
        continue;

      if (!valid_bundle_info(inp))
        tty->print("ERROR - inp->_idx=%d >= _node_bundle_limit=%d\n", inp->_idx, _node_bundle_limit);

      if (inp == n)
        continue;

      if (inp->is_SpillCopy()) {
        do {
          inp = inp->in(MemNode::Memory);
        } while (inp->is_SpillCopy());
      }

      // I really don't like this, but it handles references to
      // the original start node by using the new prolog node.
      if (inp->is_Start() || inp->is_Root()) {
        inp = bb->_nodes[0];

        if (!valid_bundle_info(inp))
          tty->print("ERROR - inp->_idx=%d >= _node_bundle_limit=%d\n", inp->_idx, _node_bundle_limit);
#ifndef PRODUCT
        if (TraceOptoOutput) {
          tty->print("#   * replaced with ");
          inp->dump();
        }
#endif
      }

      if (not_found(inp->_idx))
        continue;

      // If the node was already visited, don't count it *KLUDGE*
      if (not_visited(inp->_idx))
        ++_uses[inp->_idx];

      if (inp->Opcode() == Op_MachProj) {
        Node *base = inp->in(0);
#ifndef PRODUCT
        if (!valid_bundle_info(base))
          tty->print("ERROR - in(0)->_idx=%d >= _node_bundle_limit=%d\n",
            base->_idx, _node_bundle_limit);
        if (TraceOptoOutput) {
          tty->print("#   *Scanning thru MachProj %4d[%d]: ", inp->_idx, 0);
          base->dump();
        }
#endif
        // If the node was already visited, don't count it *KLUDGE*
        if (not_visited(base->_idx))
          ++_uses[base->_idx];
      }
    }

    // If this instruction has a 0 use count, then it is available
    if (!_uses[n->_idx]) { 
      _current_latency[n->_idx] = _bundle_cycle_number;
      AddNodeToAvailableList(n);
    }

#ifndef PRODUCT
    if (TraceOptoOutput) {
      tty->print("#   uses: %3d: ", _uses[n->_idx]);
      n->dump();
    }
#endif
  }

  // Force the use count of the start of the block so that it is never
  // scheduled
  for ( j = 0; j < lb; j++ )
    _uses[bb->_nodes[j]->_idx] = -1;

#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("# <- ComputeUseCount\n");
  }
#endif
}

// This routine performs scheduling on each basic block in reverse order,
// using instruction latencies and taking into account function unit
// availability.
void Scheduling::DoScheduling() {
#ifndef PRODUCT
  if (TraceOptoOutput)
    tty->print("# -> DoScheduling\n");
#endif

  Block *succ_bb = NULL;
  Block *bb;

  // Walk over all the basic blocks in reverse order
  for( int i=_cfg->_num_blocks-1; i >= 0; succ_bb = bb, i-- ) {
    bb = _cfg->_blocks[i];

#ifndef PRODUCT
    if (TraceOptoOutput) {
      tty->print("#  Schedule BB#%03d (initial)\n", i);
      for (uint j = 0; j < bb->_nodes.size(); j++)
        bb->_nodes[j]->dump();
    }
#endif

    // On the head node, skip processing
    if( bb == _cfg->_broot )
      continue;

    // If the following block is reachable from some other block than
    // this one, then reset the pipeline information
    if (succ_bb && (succ_bb->num_preds() > 1 || _bbs[succ_bb->pred(1)->_idx] != bb)) {
#ifndef PRODUCT
      if (TraceOptoOutput) {
        tty->print("*** bundle start of next BB, node %d, for %d instructions\n",
          _next_node->_idx, _bundle_instr_count);
      }
#endif
      step_and_clear();
    }

    // Compute the register antidependencies for the basic block
    ComputeRegisterAntidependencies(bb);

    // Compute intra-bb latencies for the nodes
    ComputeLocalLatenciesForward(bb);

    // Compute the usage within the block, and set the list of all nodes
    // in the block that have no uses within the block.
    ComputeUseCount(bb);

    // Schedule the last instruction in the basic block.
    if( _available.size() > 0 ) {// Might be none
      Node *end = _available[0];
      AddNodeToBundle(end);

      // Final call in a block must be adjacent to 'catch'
      if (end->is_Catch()) {
	Node *p = end->in(0)->in(0);
	if (!_uses[p->_idx])
	  AddNodeToBundle(p);
      }
      
      else {
	MachNode *emach = end->is_Mach();
	
	if (emach) {
	  
	  // Memory op for an implicit null check has to be at the end of the block
	  if (emach->is_MachNullCheck()) {
	    Node *p = end->in(1);
	    if (!_uses[p->_idx])
	      AddNodeToBundle(p);
	  }
	  
	  // Memory op for an implicit null check has to be at the end of the block
	  else if (emach->is_MachIf()) {
	    for (uint k = emach->oper_input_base(); k > emach->req(); k++) {
	      Node *inp = emach->in(k);
	      if (inp && inp->is_Mach() && must_clone[inp->is_Mach()->ideal_Opcode()] && !_uses[inp->_idx]) {
		AddNodeToBundle(inp);
		break;
	      }
	    }
	  }
	}
      }
      
      // Schedule the remaining instructions in the block
      while ( _available.size() > 0 ) {
	Node *n = ChooseNodeToBundle();
	AddNodeToBundle(n);
      }
    }

    // Now copy the instructions (in reverse order) back to the block
    int ubi = bb->_nodes.size();
    int lbi = (i != 1) ? 1 : _entry_instructions;

    for ( int k = lbi; k < ubi; k++ )
      bb->_nodes.map(k, _scheduled[ubi-k-1]);

#ifndef PRODUCT
    if (TraceOptoOutput) {
      tty->print("#  Schedule BB#%03d (final)\n", i);
      uint current = 0;
      for (uint j = 0; j < bb->_nodes.size(); j++) {
        Node *n = bb->_nodes[j];
        Bundle *bundle = node_bundling(n);
        if (bundle->instr_count() > 0 || bundle->flags() > 0) {
          tty->print("*** Bundle: ");
          bundle->dump();
        }
        n->dump();
      }
    }
#endif
  }

#ifndef PRODUCT
  if (TraceOptoOutput)
    tty->print("# <- DoScheduling\n");
#endif
} // end DoScheduling

//------------------------------AddAntidependency---------------------------
void Scheduling::AddAntidependency(Node *src, Node *dst
				   DEBUG_ARG(OptoReg::Name nam)
				   DEBUG_ARG(const char * msg) ) {
  if (!src || !dst)
    return;

  for (int i = src->len()-1; i >= 0; i--)
    if (src->in(i) == dst)
      return;

  src->add_prec(dst);  

#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("  insert %s antidependence from [%d] %s to [%d] %s",
      msg, src->_idx, src->Name(), dst->_idx, dst->Name());
    if (nam != OptoReg::Bad && nam < REG_COUNT)
      tty->print(" for %s", Matcher::regName[nam]);
    tty->print("\n");
    }
#endif
}

//------------------------------ReadRegister--------------------------------
void Scheduling::ReadRegister(OptoReg::Name regnum, Node *n
			      DEBUG_ARG(const char * comment) ) {
#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("#     reads  %s (%d)\n", regnum < REG_COUNT ? Matcher::regName[regnum] : "", regnum);
  }
#endif

  Node *next = _next_assignment[regnum];

#ifndef PRODUCT
  if (TraceOptoOutput && next) {
    tty->print("#   _next_assignment[%d]: %d\n", regnum, next->_idx);
  }
#endif

  // Incorporate the next call into the antidependence list
  if (!next && _next_call) {
    next = _next_call;
#ifndef PRODUCT
    if (TraceOptoOutput && _next_call) {
      tty->print("#   _next_call: %d\n", regnum, _next_call->_idx);
    }
#endif
  }

  AddAntidependency(next, n
      DEBUG_ARG(regnum)
      DEBUG_ARG(comment)
    );
}
 
//------------------------------WriteRegister-------------------------------
void Scheduling::WriteRegister(OptoReg::Name r, Node *n
			       DEBUG_ARG(const char *comment) ) {
#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("#     writes %s (%d)\n", r < REG_COUNT ? Matcher::regName[r] : "", r);
  }
#endif

  // Add a write->write antidependence
  AddAntidependency(_next_assignment[r], n
      DEBUG_ARG(r)
      DEBUG_ARG(comment)
    );

#ifndef PRODUCT
  if (TraceOptoOutput) {
    tty->print("#   _next_assignment[%d] = %d (%s)\n", r, n->_idx, comment);
  }
#endif

  // Update the assignment map
  _next_assignment.map(r, n);
}

//------------------------------OffsetToReg---------------------------------
OptoReg::Name Scheduling::OffsetToReg(int offset) const {
   // Attempt to use a shift to avoid a divide
   int delta;

   if (sizeof(jint) == 4)
     delta = (offset >= 0) ? offset >> 2 : ((offset - 1) >> 2) + 1;
   else if (sizeof(jint) == 8)
     delta = (offset >= 0) ? offset >> 3 : ((offset - 1) >> 3) + 1;
   else
     delta = offset / sizeof(jint);

   // Get the pseudo-register from the stack offset
   delta += (delta >= 0) ? _chaitin->_matcher._new_SP : SharedInfo::stack0 - _chaitin->_framesize;

   assert(delta > REG_COUNT, "bad register number for stack offset");

   return ((OptoReg::Name)delta);
}

//------------------------------get_register--------------------------------
OptoReg::Name Scheduling::get_register(Node *n) const {
  OptoReg::Name regnum = _chaitin->get_register(n);
  int offset, idx;

  // If it did not write a value, then check for stores to the stack
  if (regnum != OptoReg::Bad)
    return regnum;

  // Now, it must be a machine node or it's bad
  MachNode *m = n->is_Mach();

  if (!m)
    return regnum;

  // If this is a store to the frame, then return the pseudo number for the stack
  switch (m->ideal_Opcode()) {

  case Op_MemMove:
    offset = m->_opnds[1]->constant_disp();
    idx = m->operand_index(1);
    if (idx < 0 || _frame_pointer != m->in(idx))
      break;
    regnum = OffsetToReg(offset);
    break;

  case Op_Store:
  case Op_StoreB:
  case Op_StoreC:
  case Op_StoreD:
  case Op_StoreF:
  case Op_StoreI:
  case Op_StoreL:
  case Op_StoreP:
    if (_frame_pointer != m->get_base_and_disp(offset))
      break;

    // Compute a register number from the offset
    regnum = OffsetToReg(offset);
    break;

  default:
    break;
  }

  return regnum;
}

OptoReg::Name Scheduling::get_register(Node *n, bool &is_pair) const {
  OptoReg::Name regnum = _chaitin->get_register(n);
  int offset, idx;

  is_pair = false;

  // If it did not write a value, then check for stores to the stack
  if (regnum != OptoReg::Bad) {

    // Check to see if the register is actually a register pair. Sigh.
    uint n_ideal_reg = n->ideal_reg();
    if (n_ideal_reg == Op_RegL || n_ideal_reg == Op_RegD)
      is_pair = true;

    return regnum;
  }

  // Now, it must be a machine node or it's bad
  MachNode *m = n->is_Mach();

  if (!m)
    return regnum;

  // If this is a store to the frame, then return the pseudo number for the stack
  switch (m->ideal_Opcode()) {

  case Op_MemMove:
    offset = m->_opnds[1]->constant_disp();
    idx = m->operand_index(1);
    if (idx < 0 || _frame_pointer != m->in(idx))
      break;
    regnum = OffsetToReg(offset);
    break;

  case Op_StoreD:
  case Op_StoreL:
#ifdef LP64
  case Op_StoreP:
#endif
    is_pair = true;
    /* fall thru */

  case Op_Store:
  case Op_StoreB:
  case Op_StoreC:
  case Op_StoreF:
  case Op_StoreI:
#ifndef LP64
  case Op_StoreP:
#endif
    if (_frame_pointer != m->get_base_and_disp(offset))
      break;

    // Compute a register number from the offset
    regnum = OffsetToReg(offset);
    break;

  default:
    break;
  }

  return regnum;
}

//------------------------------InsertAntidependenciesForSpillCopy----------
void Scheduling::InsertAntidependenciesForSpillCopy( const Block *bb, uint node_idx, Node *spillcopy, Node *upper_limit ) {
  uint ub = bb->_nodes.size();
  Node *n = bb->_nodes[node_idx];

  if (upper_limit) {
    while (bb->_nodes[--ub] != upper_limit && ub > 0);
  }

#ifndef PRODUCT
  if (TraceOptoOutput)
    tty->print("# InsertAntidependenceForSpillCopy: from %d to %d, spillcopy %d\n", 
      bb->_nodes[node_idx+1]->_idx, bb->_nodes[ub-1]->_idx, spillcopy->_idx);
#endif

  // Walk forward over all the nodes after this one in the
  // block, adding dependencies from nodes that read the
  // spillcopy to this one.
  for (uint j=node_idx+1; j < ub; j++) {
    MachNode *m = bb->_nodes[j]->is_Mach();
    if (!m)
      continue;

    for (uint k = m->oper_input_base(); k < m->req(); k++) {
      if (m->in(k) == spillcopy) {
#ifndef PRODUCT
        if (TraceOptoOutput)
          tty->print("# InsertAntidependenceForSpillCopy: %d dependent on %d\n", m->_idx, n->_idx);
#endif
        m->add_prec(n);
      }
    }
  }
}

//------------------------------ComputeRegisterAntidependences-----------------
// We insert antidependences between the reads and following write of
// allocated registers to prevent illegal code motion. Hopefully, the
// number of added references should be fairly small, especially as we
// are only adding references within the current basic block.
void Scheduling::ComputeRegisterAntidependencies(const Block *b) {
  int offset, idx;
  OptoReg::Name stackreg;
  bool is_pair;

  // Clear the values but keep the storage
  _next_assignment.clear();
  _spillcopy_write.clear();
  _writes_oop     .clear();
  _next_call = NULL;
  clear_found_vector();

  // Walk backwards over all the instructions
  for( int i=b->_nodes.size()-1; i >= 0; i-- ) {
    Node     *n = b->_nodes[i];
    MachNode *m = n->is_Mach();

#ifndef PRODUCT
    if (TraceOptoOutput) {
      if (m) {
        for (uint j = m->oper_input_base(); j < m->req(); j++) {
          Node *inp = m->in(j);
          if (inp && inp->is_SpillCopy()) {
            OptoReg::Name result = get_register(inp);
            tty->print("# %-6s ",
              result >= 0 && result < REG_COUNT ? Matcher::regName[result] : "", result);
            inp->dump();
          }
        }
      }

      OptoReg::Name result = get_register(n);
      tty->print("# %-6s ",
        result >= 0 && result < REG_COUNT ? Matcher::regName[result] : "", result);
      n->dump();
    }
#endif

    // Indicate that this instruction has been seen
    set_found(n->_idx);

    // Handle serializing nodes
    AddAntidependency(_next_call, n
        DEBUG_ARG(OptoReg::Bad)
        DEBUG_ARG("serialization")
      );

    // A little bit too much of a special case for me
    if (n->is_BoxLock()) {
      _next_assignment.clear();
      _spillcopy_write.clear();
      _next_call = n;
      continue;
    }

    // Same here
    if (n->Opcode() == Op_MachProj) {

      // Check the output registers
      RegMask outputs = n->out_RegMask();

      if (outputs.is_NotEmpty()) {
        OptoReg::Name lb = outputs.find_first_elem();
        OptoReg::Name ub = outputs.find_last_elem();

        for (OptoReg::Name r = lb; r <= ub; r = OptoReg::add(r,1)) {
          if (outputs.Member(r)) {
            WriteRegister(r, n
                DEBUG_ARG("MachProj")
              );
          }
        }
      }

      continue;
    }

    // Ignore non-machine nodes
    if (!m)
      continue;

    // Build anti-dependencies for all inputs to this instruction
    // that are written to in a succeeding instruction. We only
    // look at the inputs that correspond to operands
    for (uint j = m->oper_input_base(); j < m->req(); j++) {
      Node *inp = m->in(j);

      if (!inp)
        continue;

      OptoReg::Name regnum = get_register(inp, is_pair);

      if( regnum != OptoReg::Bad ) {

        if (inp->is_SpillCopy()) {
#ifndef PRODUCT
          if (TraceOptoOutput) {
            tty->print("#     *** in(%d): %d is SpillCopy\n", j, inp->_idx);
          }
#endif
          Node *old_write = _spillcopy_write[regnum];
          if (old_write != inp) {
            if (old_write)
              InsertAntidependenciesForSpillCopy(b, i, old_write, _next_assignment[regnum]);

#ifndef PRODUCT
            if (TraceOptoOutput) {
              tty->print("#     _spillcopy_write[%d] = %d\n", regnum, inp->_idx);
            }
#endif
            _spillcopy_write.map(regnum, inp);
          }

          if (is_pair) {
            Node *old_write = _spillcopy_write[regnum+1];
            if (old_write != inp) {
              if (old_write)
                InsertAntidependenciesForSpillCopy(b, i, old_write, _next_assignment[regnum+1]);

#ifndef PRODUCT
              if (TraceOptoOutput) {
                tty->print("#     _spillcopy_write[%d] = %d\n", regnum+1, inp->_idx);
              }
#endif
              _spillcopy_write.map(regnum+1, inp);
            }
          }
        }

        // Create antidependences for the register read
        ReadRegister(regnum, m 
            DEBUG_ARG("read->write")
          );

        if (is_pair)
          ReadRegister((OptoReg::Name)(regnum+1), m 
               DEBUG_ARG("read->write")
            );
      }
    }
 
    // If this is a load from the frame, then check for memory read->write antidependences
    switch (m->ideal_Opcode()) {

    case Op_MemMove:
      offset = m->_opnds[2]->constant_disp();
      idx = m->operand_index(2);
      if (idx < 0 || _frame_pointer != m->in(idx))
        break;
      goto load_case;

    case Op_Load:
    case Op_LoadB:
    case Op_LoadC:
    case Op_LoadF:
    case Op_LoadI:
    case Op_LoadRange:
    case Op_LoadS:
    case Op_LoadThreadID:
#ifndef LP64
    case Op_LoadKlass:
    case Op_LoadP:
    case Op_LoadPC:
#endif
      if (_frame_pointer != m->get_base_and_disp(offset))
        break;

    load_case:
      // Compute a register number from the offset
      stackreg = OffsetToReg(offset);

      if (stackreg == OptoReg::Bad)
        break;

      // Create antidependences for the memory read
#ifndef PRODUCT
      if (TraceOptoOutput) {
        tty->print("#     *** load from offset %d, register %d\n", offset, stackreg);
      }
#endif
      ReadRegister(stackreg, m 
          DEBUG_ARG("read->write (frame)")
        );
      break;

    case Op_LoadD:
    case Op_LoadL:
#ifdef LP64
    case Op_LoadKlass:
    case Op_LoadP:
    case Op_LoadPC:
#endif
      if (_frame_pointer != m->get_base_and_disp(offset))
        break;

      // Compute a register number from the offset
      stackreg = OffsetToReg(offset);

      if (stackreg == OptoReg::Bad)
        break;

      // Create antidependences for the memory read
#ifndef PRODUCT
      if (TraceOptoOutput) {
        tty->print("#     *** load from offset %d, register %d/%d\n", offset, stackreg, stackreg+1);
      }
#endif
      ReadRegister(stackreg, m 
          DEBUG_ARG("read->write (frame)")
        );
      ReadRegister((OptoReg::Name)(stackreg+1), m
          DEBUG_ARG("read->write (frame)")
        );
      break;

    default:
      break;
    }

    // If this is node has a result, then map it
    OptoReg::Name result = get_register(n, is_pair);

    if ( result != OptoReg::Bad ) {

      // Really gross KLUDGE, due to problems with SpillCopy
      // Insert antidependences to any following node in the BB
      // that reads a value from a SpillCopy that has the
      // result written here.
      Node *spillcopy = _spillcopy_write[result];
      if (spillcopy)
        InsertAntidependenciesForSpillCopy(b, i, spillcopy, _next_assignment[result]);

      WriteRegister(result, n
          DEBUG_ARG("write->write")
        );

      // Remember all nodes that write Oops, so that we can prevent motion over safepoints
      const Type *bt = n->bottom_type();

      if (bt && bt->isa_oopptr()) {
        _writes_oop.push(n);
#ifndef PRODUCT
        if (TraceOptoOutput) {
          tty->print("* _writes_oop.push: ");
          n->dump();
        }
#endif
      }

#ifndef PRODUCT
      if (TraceOptoOutput) {
        tty->print("#     _spillcopy_write[%d] = NULL\n", result);
      }
#endif

      // Clear the mapping
      _spillcopy_write.map(result, NULL);

      // Check to see if the register is actually a register pair. Sigh.
      // If so, then create antidependences for the other register read 
      if (is_pair) {
        Node *spillcopy = _spillcopy_write[result+1];
        if (spillcopy)
          InsertAntidependenciesForSpillCopy(b, i, spillcopy, _next_assignment[result+1]);

        WriteRegister((OptoReg::Name)(result+1), n
            DEBUG_ARG("write->write")
          );

#ifndef PRODUCT
        if (TraceOptoOutput) {
          tty->print("#     _spillcopy_write[%d] = NULL\n", result+1);
        }
#endif

        // Clear the mapping
        _spillcopy_write.map(result+1, NULL);
      }
    }

    // If this is a call or a safepoint, then force ordering
    if (m->is_MachSafePoint() || m->is_MachEpilog()) {

      // Set to Null all the assignments written by this
      // node, and set _next_call in it's place
      for ( uint j = i+1; j < b->_nodes.size(); j++ ) {
        Node *p = b->_nodes[j];
        if (p->Opcode() != Op_MachProj)
          break;

        // Check the output registers
        RegMask outputs = p->out_RegMask();

        if (outputs.is_NotEmpty()) {
          OptoReg::Name lb = outputs.find_first_elem();
          OptoReg::Name ub = outputs.find_last_elem();

          for (OptoReg::Name r = lb; r <= ub; r = OptoReg::add(r,1))
            if (outputs.Member(r)) 
              _next_assignment.map(r, NULL);
        }
      }

      // Now scan for any Oops - they cannot be moved
      //   (Fortunately, they are always a single register)
      while ( _writes_oop.size() > 0 ) {
        Node *w = _writes_oop.pop();
        AddAntidependency(w, n
          DEBUG_ARG(get_register(w))
          DEBUG_ARG("SafePoint -> Write Oop")
        );
      }

      _writes_oop.clear();
      _next_call = n;
    }
  }
}

#ifndef PRODUCT

// Print Scheduling Statistics
void Scheduling::print_statistics() {
  // Print the size added by nops for bundling
  tty->print("Nops added %d bytes to total of %d bytes",
    _total_nop_size, _total_method_size);
  if (_total_method_size > 0)
    tty->print(", for %.2f%%",
      ((double)_total_nop_size) / ((double) _total_method_size) * 100.0);
  tty->print("\n");

  // Print the number of branch shadows filled
  if (Pipeline::_branch_has_delay_slot) {
    tty->print("Of %d branches, %d had unconditional delay slots filled",
      _total_branches, _total_unconditional_delays);
    if (_total_branches > 0)
      tty->print(", for %.2f%%",
        ((double)_total_unconditional_delays) / ((double)_total_branches) * 100.0);
    tty->print("\n");
  }

  uint total_instructions = 0, total_bundles = 0;

  for (uint i = 1; i <= Pipeline::_max_instrs_per_bundle; i++) {
    uint bundle_count   = _total_instructions_per_bundle[i];
    total_instructions += bundle_count * i;
    total_bundles      += bundle_count;
  }

  if (total_bundles > 0)
    tty->print("Average ILP (excluding nops) is %.2f\n",
      ((double)total_instructions) / ((double)total_bundles));
}
#endif
