/*
 * @(#)parse1.cpp	1.385 01/02/09 14:46:51
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_parse1.cpp.incl"

// Static array so we can figure out which bytecodes stop us from compiling
// the most. Some of the non-static variables are needed in bytecodeInfo.cpp
// and eventually should be encapsulated in a proper class (gri 8/18/98).

int methods_parsed             = 0; int methods_parsed_old             = 0;
int methods_seen               = 0; int methods_seen_old               = 0;

int explicit_null_checks_inserted = 0, explicit_null_checks_inserted_old = 0;
int explicit_null_checks_elided   = 0, explicit_null_checks_elided_old   = 0;
int all_null_checks_found         = 0, implicit_null_checks              = 0;
int implicit_null_throws          = 0;

int parse_idx = 0, parse_arena = 0;
int reclaim_idx  = 0;
int reclaim_in   = 0;
int reclaim_node = 0;

//------------------------------print_statistics-------------------------------
#ifndef PRODUCT
void Parse::print_statistics() {
  tty->print_cr("--- Compiler Statistics ---");
  tty->print("Methods seen: %d  Methods parsed: %d", methods_seen, methods_parsed);
  debug_only(tty->print("  Nodes created: %d", Compile::_debug_idx) );
  tty->cr();
  if (methods_seen != methods_parsed)
    tty->print_cr("Reasons for parse failures (NOT cumulative):");

  if( explicit_null_checks_inserted )
    tty->print_cr("%d original NULL checks - %d elided (%2d%%); optimizer leaves %d,", explicit_null_checks_inserted, explicit_null_checks_elided, (100*explicit_null_checks_elided)/explicit_null_checks_inserted, all_null_checks_found);
  if( all_null_checks_found )
    tty->print_cr("%d made implicit (%2d%%)", implicit_null_checks,
                  (100*implicit_null_checks)/all_null_checks_found);
  if( implicit_null_throws )
    tty->print_cr("%d implicit null exceptions at runtime",
                  implicit_null_throws);
}
#endif

//-----------------------------------------------------------------------------
// Supporting data structures

void Bci2Node::copy(Bci2Node* dst, Bci2Node* src) {
  if (dst == src) return; 
  dst->_bci       = src->_bci;
  dst->_sp        = src->_sp;
  dst->_map       = src->_map;
  dst->_jsr_stack = src->_jsr_stack ? src->_jsr_stack->copy() : NULL;
}

Bci2NodeArray::Bci2NodeArray() {
  _stack    = NEW_RESOURCE_ARRAY(Bci2Node, InitialSize);  
  _max_size = InitialSize;  
  for( int i = 0; i < _max_size; i++ ) 
    at(i)->clear();
}

// Initializes _map, _sp, and _jsr_stack in the Parse objects. Returns the bci
int Bci2NodeArray::copy_to_parser(int idx, Parse *p) {
  Bci2Node *n   = at(idx);
  p->_map       = n->map();
  p->_sp        = n->sp();
  p->_jsr_stack = n->jsr_stack() ? n->jsr_stack()->copy() : NULL;
  return n->bci();
}

void Bci2NodeArray::copy_from_parser(int idx, Parse* p, int bci) {
  Bci2Node *n = at(idx);
  n->set_map(p->_map);
  n->set_sp(p->_sp);
  n->set_jsr_stack( p->_jsr_stack ? p->_jsr_stack->copy() : NULL);
  n->set_bci(bci);
}

bool Bci2NodeArray::equals(int idx, Parse* p, int bci) const {
  Bci2Node* n = at(idx);  
  if (bci != n->bci()) return false;    
  return jsr_stack_equals(idx, p);
}

bool Bci2NodeArray::jsr_stack_equals(int idx, Parse *p) const {
  Bci2Node* n = at(idx);  
  if (p->_jsr_stack == NULL && n->jsr_stack() == NULL) return true;
  if (p->jsr_stack_depth() != n->jsr_stack_depth()) return false;    
  return (p->_jsr_stack->equals(n->jsr_stack()));
}

void Bci2NodeArray::grow() {
  int new_max = 2*_max_size;  
  Bci2Node* new_stack = NEW_RESOURCE_ARRAY(Bci2Node, new_max);
  int i;
  for(i = 0; i < _max_size; i++) {
    Bci2Node::copy(&new_stack[i], &_stack[i]);  
  }
  for(i = _max_size; i < new_max; i++) {
    new_stack[i].clear();
  }
  _stack    = new_stack;
  _max_size = new_max;
}

//------------------------------ON STACK REPLACEMENT---------------------------

// Construct a node which can be used to get incoming state for
// on stack replacement.
Node *Parse::fetch_interpreter_state(int index,
                                     const Type *type,
                                     Node *frame) {
  CallNode *fetch;

  // Call a runtime routine to retrieve the value.  We call
  // a routine which will fetch a value of the appropriate type.

  // We use fetch_oop to fetch any sort of pointer.  The CastPPNode
  // will take care of turning it into the correct type.
  if (type->isa_ptr()) {
    // Oops, arrays.

    // create input type (domain)
    const Type **fields = TypeTuple::fields(2);          
    fields[TypeFunc::Parms+0] = TypeRawPtr::BOTTOM;
    fields[TypeFunc::Parms+1] = TypeInt::INT;       // index
    const TypeTuple *domain = TypeTuple::make(TypeFunc::Parms+2, fields);
    
    // create result type
    fields = TypeTuple::fields(1);
    fields[TypeFunc::Parms+0] = type;
    const TypeTuple *range = TypeTuple::make(TypeFunc::Parms+1, fields);
    const TypeFunc *tf = TypeFunc::make(domain, range);  
    
    fetch = new CallLeafNode(tf,CAST_FROM_FN_PTR(address, OptoRuntime::fetch_oop), "fetch_oop");
  } else if (type->isa_int()) {
    fetch = new CallLeafNode(OptoRuntime::fetch_int_Type(),
                            CAST_FROM_FN_PTR(address, OptoRuntime::fetch_int),
                            "fetch_int");
  } else if (type->isa_double_constant() || type == Type::DOUBLE) {
    fetch = new CallLeafNode(OptoRuntime::fetch_double_Type(),
                            CAST_FROM_FN_PTR(address, OptoRuntime::fetch_double),
                            "fetch_double");
  } else if (type->isa_long()) {
    fetch = new CallLeafNode(OptoRuntime::fetch_long_Type(),
                            CAST_FROM_FN_PTR(address, OptoRuntime::fetch_long),
                            "fetch_long");
  } else if (type->isa_float_constant() || type == Type::FLOAT) {
    fetch = new CallLeafNode(OptoRuntime::fetch_float_Type(),
                            CAST_FROM_FN_PTR(address, OptoRuntime::fetch_float),
                            "fetch_float");
  } else {
    // What type is this?
    ShouldNotReachHere();
  }

  // Inputs
  set_predefined_input_for_runtime_call(fetch);
  fetch->set_req(TypeFunc::Parms+0, frame);
  fetch->set_req(TypeFunc::Parms+1, _gvn.intcon(index));

  Node *fetch2 = _gvn.transform(fetch);

  // Outputs
  set_predefined_output_for_runtime_call(fetch2);
  return _gvn.transform(new(1) ProjNode(fetch2,TypeFunc::Parms+0));
}

// Construct a graph to fetch a locked oop from the calling interpreter frame.
// Used for on stack replacement.
Node *Parse::fetch_monitor(int index, Node *box, Node *frame) {
  CallNode *fetch = new CallLeafNode(OptoRuntime::fetch_monitor_Type(),
                            CAST_FROM_FN_PTR(address, OptoRuntime::fetch_monitor),
                            "fetch_monitor");

  // Inputs
  set_predefined_input_for_runtime_call(fetch);
  fetch->set_req(TypeFunc::Parms+0, frame);
  fetch->set_req(TypeFunc::Parms+1, _gvn.intcon(index));
  fetch->set_req(TypeFunc::Parms+2, box);

  Node *fetch2 = _gvn.transform(fetch);

  // Outputs
  set_predefined_output_for_runtime_call(fetch2);
  Node *value = _gvn.transform(new(1) ProjNode(fetch2,TypeFunc::Parms+0));

  return value;
}

// Helper routine which sets up elements of the initial parser map when
// performing a parse for on stack replacement.
void Parse::initialize_osr_map(StartNode *start) {
  uint index;
  uint max_locals = _method->max_locals();
  uint max_stack  = _method->max_stack();

  // Set initial stack depth.
  _sp = _osr_info->stack_depth();

  for (index = 0; index < max_locals+max_stack; index++) {
    _map->add_req(_top);
  }

  // Our first parameter contains the address of a frame object describing
  // the interpreter frame which we are replacing.
  ParmNode *frame =  new (1) ParmNode(start, TypeFunc::Parms+0);

  _gvn._types.map(frame->_idx,  frame->bottom_type());
  _for_igvn.push(frame);  

  // Commute monitors from interpreter frame to compiler frame.
  assert(_sync_base == 0, "should be no active locks at beginning of osr");
  for (index = 0; index < (uint)_osr_info->monitor_depth(); index++) {
    // Make a BoxLockNode for the monitor.
    Node *box = _gvn.transform(new (1) BoxLockNode(next_monitor()));

    // Fetch the locked oop from the interpreter frame and commute the
    // monitor to our box node.
    Node *lock_object = fetch_monitor(index, box, frame);

    // Build a bogus FastLockNode (no code will be generated) and push the
    // monitor into our debug info.
    const FastLockNode *flock = _gvn.transform(new (3) FastLockNode( 0, lock_object, box ))->is_FastLock();
    _map->push_monitor(flock);

    // If the lock is our method synchronization lock, tuck it away in
    // _sync_lock for return and rethrow exit paths.
    if( index == 0 && _method->flags().is_synchronized() ) {
      _synch_lock = flock;
    }
  }

  const TypeTuple *domain = _osr_info->domain();

  uint first_local = TypeFunc::Parms;
  // Extract the needed locals from the interpreter frame.
  for (index = 0; index < max_locals; index++) {
    const Type *type = domain->_fields[first_local+index];
    if (type != Type::TOP && type != Type::HALF) {
      // Construct code to access the appropriate local.
      Node *value = fetch_interpreter_state(index, type, frame);
      _map->set_req(first_local+index, value);
    }
  }

  uint first_stack = first_local + max_locals;
  // Extract the needed stack entries from the interpreter frame.
  for (index = 0; index < max_stack; index++) {
    const Type *type = domain->_fields[first_stack+index];
    if (type != Type::TOP) {

      // Currently the compiler bails out when attempting to on stack replace
      // at a bci with a non-empty stack.  We should not reach here.
      ShouldNotReachHere();

      // Construct code to access the appropriate stack entry.
      //   Node *value = fetch_interpreter_stack_state(index, type, frame);
      //   _map->set_req(first_stack+index, value);
    }
  }
}


// Collect information from the current parser state to be used in the
// second osr parse.
void Parse::collect_osr_info(int cloning) {
  // Check bailouts.  We currently do not perform on stack replacement
  // of loops in catch blocks or loops which branch with a non-empty stack.
  if (_sp != 0 || cloning != 0) {
    return;
  }

  // Record the expression and monitor stack depths.
  _osr_info->set_stack_depth(_sp);
  _osr_info->set_monitor_depth((_map->_jvms->endoff() - _map->_jvms->monoff())/2);

  // Build a type domain corresponding to the current JVM map.
  int max_parms = _method->max_locals() + _method->max_stack();
  const Type **domain_array = TypeTuple::fields(max_parms);

  BitMap oops = _osr_info->oops();

  for (int i=TypeFunc::Parms; i < (TypeFunc::Parms + max_parms); i++) {
    domain_array[i] = _map->in(i)->bottom_type();

    // Check for T_CONFLICT; type is locally an oop but globally not an oop.
    if( domain_array[i]->isa_oop_ptr() &&
	!oops.at(i-TypeFunc::Parms) ) {
      domain_array[i] = Type::TOP;
    }
  }

  const TypeTuple *domain = TypeTuple::make(TypeFunc::Parms + max_parms, domain_array);

  // Record the domain.
  _osr_info->set_domain(domain);
}


//------------------------------not_obviously_loopback-------------------------
// javac emits loops with a test at the bottom and an initial branch to the
// loop bottom to cover the initial test.  This makes for lousy looking loops
// (the loop-exit test is at the start of the loop instead of at the end).
// I clone at the bytecode level to get a zero-trip-count test and a nice loop.
// If I clone loops with 'continue' statements I might make an irreducible 
// loop, so I check for the obvious loop-back edge before cloning.

int Parse::not_obviously_loopback( ciByteCodeStream &iter, uint new_bci ) {
  // Do not clone inside of JSRs
  if( !is_jsr_stack_empty() ) return true;
  iter.reset_to_bci(new_bci);   // Set iterator to start of merge point
  // Do not clone very large exit blocks
  int counter = 0;
  while( 1 ) {
    switch( (int)iter.next() ) {
    case Bytecodes::_goto_w:
    case Bytecodes::_goto: 
      return true; 
    case Bytecodes::_ifnull:    
    case Bytecodes::_ifnonnull: 
    case Bytecodes::_if_acmpeq: 
    case Bytecodes::_if_acmpne: 
    case Bytecodes::_if_icmpeq: 
    case Bytecodes::_if_icmpne: 
    case Bytecodes::_if_icmplt: 
    case Bytecodes::_if_icmple: 
    case Bytecodes::_if_icmpgt: 
    case Bytecodes::_if_icmpge: 
    case Bytecodes::_ifeq: 
    case Bytecodes::_ifne: 
    case Bytecodes::_iflt: 
    case Bytecodes::_ifle: 
    case Bytecodes::_ifgt: 
    case Bytecodes::_ifge: 
      // Do not clone unless we backwards branch to BEFORE the 
      // test&exit area.
      return iter.get_dest() >= (int)new_bci;
    case Bytecodes::_return:
    case Bytecodes::_ireturn:
    case Bytecodes::_areturn:
    case Bytecodes::_freturn:
    case Bytecodes::_dreturn:
    case Bytecodes::_lreturn:
    case Bytecodes::_athrow:
    case Bytecodes::_ret:
    case Bytecodes::_jsr:
    case Bytecodes::_jsr_w:
    case Bytecodes::_tableswitch:
    case Bytecodes::_fast_binaryswitch:
    case Bytecodes::_fast_linearswitch:
    case Bytecodes::_lookupswitch: 
      return true;
    case ciByteCodeStream::EOBC:
      ShouldNotReachHere(); // We should never fall off the end
    default: 
      break;                // Ignore most bytecodes until control flow
    }
    if( counter++ > 50 )
      return true;
  }

  return false;
}

#ifndef PRODUCT
// Record if uncommon trap bailout inserted, min_jint indicates no bailouts
void Parse::record_deopt(int deopt_cause) { 
  if( (deopt_cause == Deoptimization::Deopt_uninitialized) || (deopt_cause == Deoptimization::Deopt_unloaded_array_class)
    || (deopt_cause > Deoptimization::Deopt_unloaded)  ) _deopt_cause = deopt_cause; 
}
#endif

//------------------------------check_monitor_matching-------------------------
// Check that monitorenter/monitorexit is properly nested in current
// method. Builds map mapping monitorexit bci's to monitorenter bci's
Parse::BailoutConstants Parse::check_monitor_matching() {
  if (_method->has_balanced_monitors()) {
    return Bailout_none;
  } else {
    return Bailout_monitor_matching;
  }
}

//------------------------------Parse------------------------------------------
// Top level parser; used for first and second on stack replacement parses.
Parse::Parse( ciEnv* env,
	      InlineTree* ilt,
	      PhaseGVN &gvn,
	      OSRParseInfo *osr_info, 
	      Unique_Node_List &for_igvn ) :
    Phase(Phase::Parser),
    _env(env),
    _gvn(gvn),
    _for_igvn(for_igvn),
    _osr_info(osr_info),
    _tf(NULL),
    _exit_rethrow(NULL),
    _exit_return(NULL) {

  Init( ilt, NULL, NULL, 0, 1, NULL, 0, ilt->_method->has_exception_handlers());
}

//------------------------------Parse------------------------------------------
// Recursive parser; record shared GVN object
Parse::Parse( ciEnv* env,
	      InlineTree* ilt,
	      JVMStateObj *jhead,
	      JVMStateObj *jtail,
	      int sync,
	      int depth,
	      PhaseGVN &gvn,
	      Unique_Node_List &for_igvn,
	      Node *map,
	      uint sp_off,
              bool has_ex_handler ) : 
  Phase(Phase::Parser), 
  _env(env),
  _gvn(gvn), 
  _for_igvn(for_igvn), 
  _osr_info(NULL),
  _tf(NULL),
  _exit_rethrow(NULL),
  _exit_return(NULL) {

  if( !ilt ) {                // Not inlining here
    _bailout = Bailout_compile_only;
    return;
  }
  // JVMPI -- Turn off inlining
  if( C->is_set_jvmpi_method_event() ) {
    _bailout = Bailout_compile_only;
    return;
  }
  Init( ilt, jhead, jtail, sync, depth, map, sp_off, 
        has_ex_handler || ilt->_method->has_exception_handlers());
}

//------------------------------Init-------------------------------------------
void Parse::Init( InlineTree* ilt,
		  JVMStateObj *jhead,
		  JVMStateObj *jtail,
		  int sync,
		  int depth,
		  Node *map,
		  uint sp_off,
                  bool has_ex_handler) {
  // Init some variables
  _map = 0;
  _depth = depth;
  _ilt = ilt;
  _ready   = 0;
  _reached = 0;
  _looped = 0;
  _sync_base = sync;
  _sync_max = _sync_base;
  _jhead = jhead;
  _jtail = jtail;
  _jsr_stack = NULL;
  _null = NULL;
  _has_ex_handler = has_ex_handler;
#ifndef PRODUCT
  _deopt_cause = No_deopt_in_method;
  _max_switch_depth = 0;
  _est_switch_depth = 0;
#endif

  // Is this a first osr parse of a method?
  _is_first_osr_parse = _osr_info != NULL && !_osr_info->is_complete();

  // Allocate bci2node list
  _bci2node = new Bci2NodeArray();

  _method = _ilt->_method;
  _klass  = _method->holder();

  methods_seen++;

  // Check for bailout due to wrongly nested monitorenter/monitorexit
  _bailout = check_monitor_matching();
  if (_bailout != Bailout_none) {
    show_bailout_info(); 
    return;
  }

  // Record locally the interesting bits
  _preds = ilt->_preds;
  _bits = ilt->_bits;
  _args = ilt->_args;

  methods_parsed++;
  // add method size here to guarantee that inlined methods are added too
  if (TimeCompiler) 
    _total_bytes_compiled += _method->code_size();

#ifndef PRODUCT
  if (PrintCompilation && Verbose) {
    if (_depth == 1) {
      if( ilt->_cnt_inlines ) {
        tty->print("    __inlined %d (%d bytes)", ilt->_cnt_inlines,
                     ilt->_cnt_inline_bcs);
        tty->cr();
      }
    } else {
      if( _method->flags().is_synchronized() ) tty->print("s");
      if( _method->has_exception_handlers() )            tty->print("!");
      // Check this is not the final compiled version
      if( _deopt_cause != No_deopt_in_method ) {
        tty->print("-");
      } else {
        tty->print(" ");
      }
      _method->print_short_name();
      if (is_second_osr_parse()) {
        tty->print(" @ %d", entry_bci());
      }
      tty->print(" (%d bytes)",_method->code_size());
      if( ilt->_cnt_inlines )
        tty->print(" __inlined %d (%d bytes)", ilt->_cnt_inlines,
                   ilt->_cnt_inline_bcs);
      tty->cr();
    }
  }
  if( (PrintOpto) && !is_first_osr_parse() && (_depth == 1 || PrintOptoInlining)) {
    // Print that we succeeded; suppress this message on the first osr parse.

    if( _method->flags().is_synchronized() ) tty->print("s");
    if( _method->has_exception_handlers() )            tty->print("!");
    // Check this is not the final compiled version
    if( _deopt_cause != No_deopt_in_method ) {
      tty->print("-");
    } else {
      tty->print(" ");
    }
    for( int i = 1; i < _depth; ++i ) { tty->print("  "); }
    _method->print_short_name();
    if (is_second_osr_parse()) {
      tty->print(" @ %d", entry_bci());
    }
    if (ilt->_caller_bci != -1) {
      tty->print(" @ %d", _ilt->_caller_bci);
    }
    tty->print(" (%d bytes)",_method->code_size());
    if( ilt->_cnt_inlines )
      tty->print(" __inlined %d (%d bytes)", ilt->_cnt_inlines,
                 ilt->_cnt_inline_bcs);
    tty->cr();
  }  
#endif     

  // Record merge points
  for( int j = 0; j < _method->code_size(); j++ ) 
    if( _preds[j] > 1 ) 
      bits_merge_set(j);

  _root = C->_root;
  _gvn._types.map(_root->_idx,_root->bottom_type());
  _top = _gvn.transform(C->_top_node);
  // Create a handy NULL pointer
  _null = makecon( TypePtr::NULL_PTR );

  // Create the input domain and start node.
  StartNode *start = build_start_node();
  // Bailout if an I2C or C2I adapter would require more argument
  // slots than are available in the Register Mask
  // Note: >>1 reflects the fact that adapters will need space
  //       on the stack for both incoming and outgoing args.
  // NOTE: -1 in computation reflects the usage of the last
  //       bit of the regmask as an infinite stack flag.
  int argwords = (int)(_tf->_domain->_cnt - TypeFunc::Parms);
  int old_SP = SharedInfo::stack2reg( round_to( C->in_preserve_stack_slots() + (_sync_max * C->sync_stack_slots()), WordsPerLong ) );
  if( argwords >= (int)(((RM_SIZE*32 - (old_SP + C->varargs_C_out_slots_killed() + (2 * C->out_preserve_stack_slots())))>>1)-1) ) {
    _bailout = Bailout_too_many_args;
    show_bailout_info(); 
    return;
  }

  // Check for really stupid bail-out cases.  
  if (_method->max_locals() + _method->max_stack() + TypeFunc::Parms >= 32760) { 
    _bailout = Bailout_too_many_args;
    show_bailout_info(); 
    return;
  }

  // Setup the initial JVM state map.
  JVMStateObj *jvms = initialize_map(start,map,sp_off);

#ifndef PRODUCT
  // bump invocation counter if top method (for statistics)
  if( CountCompiledCalls && _ilt->_caller_bci == -1 ) {
    const TypeOopPtr* addr_type = TypeOopPtr::make(_method);
    Node* adr1 = makecon(addr_type);
    Node* adr2 = basic_plus_adr(adr1, adr1, methodOopDesc::compiled_invocation_counter_offset_in_bytes());
    Node* cnt  = _gvn.transform(new (3) LoadINode(0, memory(TypeRawPtr::BOTTOM), adr2));
    Node* incr = _gvn.transform(new (3) AddINode(cnt, _gvn.intcon(1)));
    Node* upd  = _gvn.transform(new (4) StoreINode(0, memory(TypeRawPtr::BOTTOM), adr2, incr, Type::MEMORY));
    // record the update (otherwise code gets optimized away)
    set_memory(upd,TypeRawPtr::BOTTOM);
  }
#endif
  
  // Merge point for all normal exits
  { RegionNode* region = new RegionNode(1);
    _for_igvn.push(region);
    // normal exit
    _exit_return =
      new ReturnNode(
        region,
        new PhiNode(region, Type::ABIO),
        new MergeMemNode( NULL, C->_cnt_map_adr_mem ),
        frameptr(),
        returnadr()
      );
    // Merge point for normal memory exit
    Node *mm = _exit_return->in(TypeFunc::Memory);
    uint i;
    for( i = 1; i < C->_cnt_map_adr_mem; i++ ) 
      mm->set_req( i, new PhiNode(region, Type::MEMORY ) );
    // Add zero, 1 or 2 return values
    for( i = TypeFunc::Parms; i < _tf->_range->_cnt; i++) 
      _exit_return->add_req(new PhiNode(region, _tf->_range->_fields[i]));
    // bind to root
    _root->add_req(_exit_return);
  }

  // We begin parsing as if we have just encountered a jump to the
  // method entry.
  merge( entry_bci(), 0 );

#ifndef PRODUCT
  //if (DumpInlineImage && !is_first_osr_parse())
  //  InlineImage::append_scope_begin_to_file(_ilt->_scope, _depth-1);
#endif

  // Iterator over bytecodes
  ciByteCodeStream iter(_method);

  // Pull from the ready list if possible, else pull from the reached list.
  while( _reached && (_bailout == Bailout_none)) {
    if( _ready ) {
      while( _ready ) {
        // Capture ready block data in local variables
        int bci = _bci2node->copy_to_parser(--_ready, this);
        _bci2node->copy(_ready  , --_reached);
        _bci2node->copy(_reached, --_looped);

        // Parse this ready block
        bits_clone_clr(bci);    // mark as not cloning!
        uint cloning = 0;
        do_one_block( iter, bci, cloning );
      }
    } else {                    // Nothing ready.  Must be a loop.
      // Pull a reached, but unready, block from the list.  Find the smallest
      // bci as the block most likely to preserve the topological order.
      int midx = 0;
      for( int i = 1; i < _reached; i++ )
        if( _bci2node->at(i)->bci() < _bci2node->at(midx)->bci() )
          midx = i;
      // If nessecary, swap the min BCI to the end of the reached list
      if( midx != _reached-1 ) {
        Bci2Node tmp;
        tmp.put( _bci2node->at(midx) );
        _bci2node->at(midx)->put( _bci2node->at(_reached-1) );
        _bci2node->at(_reached-1)->put(&tmp);
      }

      int bci = _bci2node->copy_to_parser(--_reached, this);

      // Always clone inside of inlined jsr region.  // !!!!! !!!!!

      // Cloning check: was the prior block cloned?
      if( ( bits_clone(bci) ||    // Was cloned; do not clone more
            not_obviously_loopback(iter, bci) ) ) {
        bits_clone_clr(bci);    // mark as not cloning!
        // Make all entries (except monitors) into Phis.
        Node *region = control();
        merge_value( _map, TypeFunc::I_O, region );
        // Effectively "merge_value" on all memories
        Node *mem = _map->memory();
	uint i;
        for( i = 1; i < mem->req(); i++ )
          merge_value( mem, i, region );
        // Monitors must nest nicely and not get confused amongst themselves.
        uint cnt = _map->_jvms->monoff();
        for( i = TypeFunc::Parms; i<cnt; i++ ) 
          merge_value(_map,i,region);
      } else {                  // Clone this block.  Makes nicer loops.
        bits_clone_set(bci);    // mark as cloning!
        // Remove the reached guy altogether; compact array
        _bci2node->copy(_reached, --_looped);
      }

      // Clone the map; original map marks block entry, clone is used for block
      _map = clone_map(bci);
      kill_dead_locals(bci);

      uint cloning = bits_clone(bci);
      // Parse this reached block
      do_one_block( iter, bci, cloning );
    }
  }

  // Check if too many arguments for register representation
  // Note: >>1 reflects the fact that adapters will need space
  //       on the stack for both incoming and outgoing args.
  // NOTE: -1 in computation reflects the usage of the last
  //       bit of the regmask as an infinite stack flag.
  old_SP = SharedInfo::stack2reg( round_to( C->in_preserve_stack_slots() + (_sync_max * C->sync_stack_slots()), WordsPerLong ) );
  if( C->_max_arg_size >= (int)(((RM_SIZE*32 - (old_SP + (2 * C->out_preserve_stack_slots())))>>1)-1) ) {
    _bailout = Bailout_too_many_args;
    show_bailout_info();
    return;
  }

#ifndef PRODUCT
  if (DumpInlineImage && !is_first_osr_parse())
    InlineImage::append_scope_end_to_file();
#endif

  // Check for bailout during conversion to graph
  if (_bailout != Bailout_none) {
    if (_bailout != Bailout_throwaway_parse) {
      show_bailout_info(); 
    }
    return;
  }

  // Now peephole on the return bits
  _exit_return->set_req(0,_gvn.transform(_exit_return->in(0)));
  Node *mem = _exit_return->in(TypeFunc::Memory);
  assert( mem->Opcode() == Op_MergeMem, "" );
  uint i;
  for( i=1; i < C->_cnt_map_adr_mem; i++ )
    mem->set_req(i,_gvn.transform(mem->in(i)));
  for( i=1; i<_exit_return->req(); i++ ) 
    _exit_return->set_req(i,_gvn.transform(_exit_return->in(i)));

  // Optimize the return
  _for_igvn.push(_exit_return);
  _gvn.transform_no_reclaim(_exit_return);

  // The exception-exit path is lazyily built to speed up methods with
  // no exception exits.
  if( _exit_rethrow ) {
    // GVN transform the combined null exception region and its phis
    Node *ctrl      = _exit_rethrow->in(TypeFunc::Control);
    Node *io_phi    = _exit_rethrow->in(TypeFunc::I_O    );
    Node *mem       = _exit_rethrow->in(TypeFunc::Memory );
    Node *ex_oop_phi= _exit_rethrow->in(TypeFunc::Parms  );

    ctrl      ->set_req(1, _gvn.transform(ctrl      ->in(1)));
    io_phi    ->set_req(1, _gvn.transform(io_phi    ->in(1)));
    ex_oop_phi->set_req(1, _gvn.transform(ex_oop_phi->in(1)));

    // Combined null exception memories
    assert( mem->Opcode() == Op_MergeMem, "" );
    for( uint i=1; i < C->_cnt_map_adr_mem; i++ ) {
      Node *mem_phi = mem->in(i);
      mem_phi->set_req(1,_gvn.transform(mem_phi->in(1)));
    }

    // Now the not-null-exception path (i.e., the normal exception path)
    ctrl = _gvn.transform(ctrl);
    _exit_rethrow->set_req( TypeFunc::Control, ctrl );
    if( ctrl != _top ) {
      assert( ctrl->bottom_type() == Type::CONTROL, "" );

      // Peephole the inputs to the memory
      for( i=1; i < Compile::current()->_cnt_map_adr_mem; i++ )
        mem->set_req(i,_gvn.transform(mem->in(i)));

      // Peephole the inputs to the rethrow
      _exit_rethrow->set_req(TypeFunc::I_O   , _gvn.transform(io_phi    ));
      _exit_rethrow->set_req(TypeFunc::Parms , _gvn.transform(ex_oop_phi));

      // Synchronized methods must unlock on exception exit
      if( _method->flags().is_synchronized() ) {
        // Use the empty locked JVM state map to do unlocking
        jvms->set_endoff(jvms->monoff());
        _map = new SafePointNode(TypeFunc::Parms, jvms);
        jvms->set_map(_map);
	uint i;
        // Collect the memory state into the state map
        for( i = 0; i < TypeFunc::Parms; i++ )
          _map->set_req(i,_exit_rethrow->in(i));
        // No locals or expression stack
        for( ; i < jvms->monoff(); i++ )
          _map->add_req(_top);
        // Add on the synchronized-method box/object combo
        _map->push_monitor( _synch_lock );
        // Unlock!
        if( GenerateSynchronizationCode )
          shared_unlock( _synch_lock->box_node(), _synch_lock->obj_node(), SynchronizationEntryBCI );
        // Take the final state into the rethrow Node
        assert( _map->in(TypeFunc::Memory)->Opcode() == Op_MergeMem, "" );
        for( i = 0; i < TypeFunc::Parms; i++ ) 
          _exit_rethrow->set_req(i,_map->in(i));
        mem = _map->in(TypeFunc::Memory);
      }

      _exit_rethrow->set_req(TypeFunc::Memory, _gvn.transform(mem ));
    }

    // Optimize the rethrow
    _for_igvn.push(_exit_rethrow);
    _gvn.transform_no_reclaim(_exit_rethrow);
  }

  parse_idx += C->_unique;
  parse_arena += C->_arena.used();
}

//------------------------------build_start_node------------------------------
// Create a start node with the appropriate entry domain.
StartNode *Parse::build_start_node() {
  StartNode *start;
  if( is_second_osr_parse() ) {
    const TypeTuple *domain = StartOSRNode::osr_domain();
    const TypeTuple *range = TypeTuple::make_range(_method->signature());
    _tf = TypeFunc::make(domain, range);  

    start = new StartOSRNode( _root, domain );
  } else {
    // Check if TypeFunc has been constructed at inlining site,
    // expensive to reconstruct
    _tf = TypeFunc::make(_method);

    // See if any arguments are abstract with up-casting possible
    //
    // Note: If *all* TypeFunc construction can benefit from upcasting
    // then we should move this operation into TypeFunc::make.
    const TypeTuple *domain = _tf->_domain;
    for( uint i = TypeFunc::Parms; i < domain->_cnt; i++ ) {
      const TypeOopPtr *tjp = domain->_fields[i]->isa_oopptr();
      if( tjp ) {		// Oopish argument?
        ciKlass *t = tjp->klass();
        if( t && t->is_instance_klass() ) {
          ciInstanceKlass* it = t->as_instance_klass();
          ciKlass *s = it->unique_concrete_subklass();
          if( s != NULL ) {          // Up-casting an abstract class with 1 subkls?
            // Replace starting signature with upcast one
            tjp = TypeOopPtr::make( TypePtr::BotPTR, s );
            C->_recorder->add_dependent(it,NULL);
            const Type **f = TypeTuple::fields(domain->_cnt-TypeFunc::Parms);
            memcpy( f, domain->_fields, domain->_cnt*sizeof(Type*) );
            f[i] = tjp;
            domain = TypeTuple::make( domain->_cnt, f );
            _tf = TypeFunc::make( domain, _tf->_range );
          }
        }
      }
    }

    start = new StartNode( _root, _tf->_domain );
  }

  _gvn._types.map(start->_idx, start->bottom_type());
  return start;
}

//------------------------------initialize_map--------------------------------
// Initialize our parser map to contain the types/monitors at method entry.
JVMStateObj *Parse::initialize_map( StartNode *start, Node *map, uint sp_off ){
  // Create an initial safepoint to hold JVM state during parsing
  JVMStateObj *jvms = new JVMStateObj(_method, 0, _method->max_stack());
  _map = new SafePointNode(TypeFunc::Parms, jvms);
  jvms->set_map(_map);

  if( is_second_osr_parse() ) {
    assert( !map, "OSR is not inlined" );

    // Initialize the predefined input parameters.
    for( uint i=0; i < TypeFunc::Parms; i++ ) {
      ParmNode *p = new (1) ParmNode(start, i);
      _gvn._types.map(p->_idx, p->bottom_type());
      _map->set_req(i, p);
      _for_igvn.push(p);
    }
    // Insert the memory aliasing node
    set_all_memory( all_memory() );

    // On stack replacement code extracts its state from the calling
    // interpreter frame.
    initialize_osr_map(start);

  } else {
    uint i;
    // Have we been passed in an inlined map?  Then we do not need to
    // make a new one
    if( map ) {
      // Pass thru the predefined input parameters.
      for( i=0; i < TypeFunc::Parms; i++ )
        _map->set_req(i, map->in(i));
      for( ; i < _tf->_domain->_cnt; i++ ) 
        _map->add_req(map->in(sp_off+ (i-TypeFunc::Parms)));
      assert( _map->in(TypeFunc::Memory)->Opcode() == Op_MergeMem, "" );
    } else {
      // Initialize the predefined input parameters.
      for( i=0; i < TypeFunc::Parms; i++ ) {
        ParmNode *p = new (1) ParmNode(start, i);
        _gvn._types.map(p->_idx, p->bottom_type());
        _map->set_req(i, p);
        _for_igvn.push(p);
      }
      // Insert the memory aliasing node
      set_all_memory( all_memory() );
      // Match up the parameters, stack and all that to the parser Node.
      for( ; i < _tf->_domain->_cnt; i++ ) {
        ParmNode *p = new (1) ParmNode(start, i);
        _gvn._types.map(p->_idx, p->bottom_type());
        _map->add_req(p);
        _for_igvn.push(p);
      }
    }

    // Add space for locals and expression stack
    for( ; i < (uint)(TypeFunc::Parms+_method->max_locals() + _method->max_stack()); i++ )
      _map->add_req(_top);

    _sp = 0;                      // Java Stack Pointer

    Node *receiver = (_method != NULL && !_method->is_static()) ? local(0) : NULL;

    // JVMPI -- record entry to a method if compiled while JVMPI is turned on
    if( C->is_set_jvmpi_method_event() ) {
      make_jvmpi_method_entry();
    }

    // If the method is synchronized, we need to construct a lock node, attach
    // it to the Start node, and pin it there.  
    if( _method->flags().is_synchronized()) {
      // Insert a FastLockNode right after the Start which takes as arguments
      // the current thread pointer, the "this" pointer & the address of the
      // stack slot pair used for the lock.  The "this" pointer is a projection
      // off the start node, but the locking spot has to be constructed by
      // creating a ConLNode of 0, and boxing it with a BoxNode.  The BoxNode
      // becomes the second argument to the FastLockNode call.  The
      // FastLockNode becomes the new control parent to pin it to the start.

      // Setup Object Pointer
      Node *lock_obj = NULL;
      if(_method->flags().is_static()) {
	ciInstance* mirror = _method->holder()->java_mirror();
        const TypeOopPtr *t_lock = TypeOopPtr::make(mirror);
        lock_obj = makecon(t_lock);
      } else {                  // Else pass the "this" pointer, 
        lock_obj = receiver;    // which is Parm0 from StartNode
      }
      // Build the FastLockNode
      _synch_lock = shared_lock(lock_obj, SynchronizationEntryBCI);
    }
  }
  return jvms;
}

//------------------------------increment_counter------------------------------
// for statistics: increment a VM counter by 1

void Parse::increment_counter(address counter_addr) {
  Node* adr1 = makecon(TypeRawPtr::make(counter_addr));
  Node* cnt  = _gvn.transform(new (3) LoadINode (NULL, memory(TypeRawPtr::BOTTOM), adr1));
  Node* incr = _gvn.transform(new (3) AddINode  (cnt, _gvn.intcon(1)));
  Node* upd  = _gvn.transform(new (4) StoreINode(NULL, memory(TypeRawPtr::BOTTOM), adr1, incr, Type::MEMORY));
  // record the update (otherwise code gets optimized away)
  set_memory(upd,TypeRawPtr::BOTTOM);
}


//------------------------------do_one_block-----------------------------------

void Parse::do_one_block( ciByteCodeStream &iter, uint bci, uint cloning ) {
  if( TraceOptoParse ) {
    tty->print("Parsing block at bci:%d", bci);
    if( cloning != 0 ) tty->print("  cloning");
    tty->print("  _ready:%d, _reached:%d, _looped:%d", _ready, _reached, _looped);
    tty->cr();
  }
  iter.reset_to_bci(bci);       // Set iterator to start of merge point
  // Parse bytecodes
  // uint cloning = bits_clone(bci);
  for( Bytecodes::Code bc = iter.next(); _map && (_bailout == Bailout_none); bc = iter.next() )
    do_one_bytecode( bc, iter, cloning );
}


//------------------------------merge------------------------------------------
// Merge the current mapping into the basic block starting at bci
void Parse::merge( uint bci, uint cloning ) {
  assert(bci < (uint)_method->code_size(), "bci outside method");
  // If this is within _jsr..._ret, always clone
  if( !is_jsr_stack_empty() ) 
    cloning = 1;

  if( TraceOptoParse ) {
    tty->print("Merging state at bci:%d", bci);
  }
  // Zap extra stack slots to top  
  int i;
  for(i = _sp; i < _method->max_stack(); i++ )  set_stack(i, _top);

  // Get any prior map at this bci
  for( i = _ready; i < _looped; i++ ) {
    Bci2Node* n = _bci2node->at(i);  
    if( bci == (uint)n->bci() ) { // Same BCI?
      if( _bci2node->jsr_stack_equals(i, this) ) // And same JSR stack?
        break;                  // Then found same!
      // If at different stack depths, then we musta bailed out of a JSR
      // without noticing.
      else if( jsr_stack_depth() != n->jsr_stack_depth() ) {
        assert(_bailout == Bailout_shared_handler || _bailout == Bailout_none, "only one state");
        _bailout = Bailout_shared_handler;
        show_bailout_info();
        // _reached = _ready = _looped = 0;
        _map = NULL;
        return;
      }
    }
  }

  int pnum;                     // Count of predecessors == path to merge into
  if( i == _looped ) {          // No prior mapping at this bci
    if( TraceOptoParse ) { tty->print(" with empty state");  }

    pnum = _preds[bci];         // Predecessor number; count of preds
    if( cloning ) {             // Cloning inputs ?
      bits_clone_set(bci);      // Mark as cloning
      pnum++;                   // One more pred path coming in
    } else {
      _preds[bci]--;            // Merged one more path
      assert(_preds[bci] >= 0, "cannot have negative predecessor count");
    }
    // If this path is dead, do not bother capturing it as a merge.
    // It is "as if" we had 1 fewer predecessors from the beginning.
    if( _map->control() == _top ) {
      if( TraceOptoParse ) { tty->print_cr(", but path is dead and doesn't count"); }
      _map = NULL;
      return;
    }

    // Convert the existing Parser mapping into a mapping at this bci.
    if( pnum > 1 ) {            // Merging anything here?
      // Add a Region to start the new basic block.  Phis will be added
      // later lazily.
      Node *r = new RegionNode( pnum+1 );
      _gvn._types.map(r->_idx, Type::CONTROL);
      _for_igvn.push(r);
      // zap all inputs to NULL for debugging
      for (int j = 1; j < pnum+1; j++) { r->set_req(j, NULL); }
      r->set_req(pnum, control());
      set_control(r);
    } else {
      assert(pnum >= 0, "can not have negative predecessor count");
    }
    // Now consider this block reached

    // grow the _bci2node data structure (double size)
    if (_looped >= _bci2node->max_size()) _bci2node->grow();
    i = _reached;               // Leave i pointing at where we find this block
    _bci2node->copy(_looped++, _reached);
    _bci2node->copy_from_parser(_reached, this, bci);
    _reached++;


  } else {                      // Prior mapping at this bci
    if( TraceOptoParse ) {  tty->print(" with previous state"); }

    Node *curmap = _map;        // Hang on to current mapping
    _map = _bci2node->at(i)->map();   // Get prior mapping
    assert( _sp == _bci2node->at(i)->sp(), "mismatched stack depth" );
    assert( _bci2node->jsr_stack_equals(i, this), "mismatched jsr_bci");
            
    // Iterate over my current mapping and the old mapping.
    // Where different, insert Phi functions.
    // Use any existing Phi functions.
    Node *r = _map->in(TypeFunc::Control);// old region node
    uint preds = r->req();      // Number of predecessor paths

    // Compute where to merge into
    // Merge incoming control path
    if( cloning ) {             // Cloning inputs ?
      bits_clone_set(bci);      // Mark as cloning
      pnum = preds;             // Merge at the very end
      // Expand the region and all Phis for the new incoming path
      r->add_req(curmap->in(TypeFunc::Control));
      for( uint i = 1; i < curmap->req(); i++ ) {
        PhiNode *phi = _map->in(i)->is_Phi();
        if( phi && phi->in(0) == r && phi->req() <= preds )
          phi->add_req(NULL);
      }
      // Expand Phi's for memory as well
      Node *mem = _map->memory();
      for( i = 1; i < mem->req(); i++ ) {
        PhiNode *phi = mem->in(i)->is_Phi();
        if( phi && phi->in(0) == r && phi->req() <= preds )
          phi->add_req(NULL);
      }
    } else {
      pnum = _preds[bci]--;     // Predecessor number
      assert( pnum, "too many merges" );
      r->set_req(pnum, curmap->in(TypeFunc::Control));
    }

    if( pnum == 1 ) {           // Last merge for this Region?
      _gvn.transform_no_reclaim(r);
      _for_igvn.push(r);
    }

    for( uint j = 1; j < curmap->req(); j++ ) {
      Node *n = curmap->in(j);
      Node *p = n;              // Potential Phi
      if( _map->in(j) != n ) {  // Different; must merge
        switch( j ) {
        // Frame pointer and Return Address never changes
        case TypeFunc::FramePtr:
        case TypeFunc::ReturnAdr:
          p = _map->in(j);      // Use the original value
          break;
        case TypeFunc::Memory: {// Merge inputs to the MergeMem node
          Node *mem = _map->in(j);
          assert( mem->Opcode() == Op_MergeMem, "" );
          assert( n  ->Opcode() == Op_MergeMem, "" );
          // Merge the inputs to the MergeMem
          for( uint k = 1; k < mem->req(); k++ ) {
            Node *p = mem->in(k);
            Node *q = n  ->in(k);
            while( q->Opcode() == Op_MergeMem )
              q = q->in(k);
            if( p != q )        // Force a Phi if required
              p = merge_value( mem, k, r );
            // Insert q into local phi
            if( p->is_Phi() && p->in(0) == r ) {
              p->set_req( pnum, q );
              if( pnum == 1 ) {
                _for_igvn.push(p);
                mem->set_req(k,_gvn.transform_no_reclaim(p));
              }
            }
          }
          continue;
        }
        default:                // All normal stuff
          p = merge_value(_map,j,_map->in(0));   // Merge; turn into a Phi
          // If either is TOP, then the value is undefined on that path.
          // By the Java standard, it must be unused (until defined again).
          if( (n == _top || _map->in(j)== _top) && _map->in(0) != _top ) {
            _map->set_req(j, _top);
            // Dead Phi, so degrade the Phi to TOP
            if( p->Opcode() == Op_Phi &&  // Phi already?
              p->in(0) == r ) {           // And of this merge point?
              p->set_req(0, NULL);        // Unique value; degrade to Copy
              p->set_req(1, _top);
              _gvn._types.map(p->_idx,Type::TOP);
            }
            continue;
          }
          break;
        }

      }
      if( n == _top ) continue; // ignore equal TOPs

      PhiNode *phi = p->is_Phi();
      if( phi &&                // Phi already?
          p->in(0) == r ) {     // And of this merge point?
        phi->set_req(pnum, n);  // Then add 'n' to the merge

	// So far, Phis have had an overly optimistic type (equal to their
	// first input type).  Stuff in the merge of current inputs.  I
	// cannot jam a BOTTOM in here, 'cause pessimistically Phis of
	// pointers cannot loose the basic pointer type.
	const Type *t = phi->_type->meet( n->bottom_type() );

	// If the type falls to bottom, then this must be a local that
	// is mixing ints and oops or some such.  Forcing it to top
	// makes it go dead.
	if( t == Type::BOTTOM ) {
	  // set PHI's result and inputs to TOP to notify existing uses
	  t = Type::TOP;
	  for( uint l = 0; l < phi->req(); l++ )  phi->set_req( l, _top );
	  _map->set_req(j,_top);
	}
	phi->_type = t;
	// Last input to phi?
	if( pnum == 1 ) {
          _map->set_req(j,_gvn.transform_no_reclaim(phi));
	  _for_igvn.push(phi);	
        }
      }
    } // End of for all values to be merged

    if( pnum == 1 && !r->in(0) ) // The occasional useless Region
      set_control(r->nonnull_req());

    delete curmap;              // This map is now subsumed into the lazy merge
  }
  if( TraceOptoParse ) {
    tty->print(" on path %d", pnum);
    if( cloning != 0 ) tty->print(" while cloning");
    tty->cr();
  }
  // If this is our entry bci for on stack replacement, and we are doing
  // a throwaway first osr parse, collect the osr entry information.
  if (UseOnStackReplacement &&
      is_first_osr_parse() &&
      bci == (uint)_osr_info->bci()) {

    collect_osr_info(cloning);

    if (_preds[bci] == 0) {
      // We will not visit this bci again.  The type information is complete
      // and accurate and it is safe to bail out of the throwaway parse early.
      _bailout = Bailout_throwaway_parse;
      _map = NULL;
      return;
    }
  }

  if( _preds[bci] == 0 ) {      // All preds seen?  Move to ready list
    if( i < _reached ) {        // Also must not have already done it
      Bci2Node tmp;
      _bci2node->at(i)->get(&tmp);       // get reached value
      _bci2node->copy(i, _ready); // swap with end of ready list
      _bci2node->at(_ready++)->put(&tmp);
    } else {                    // previously visited block now has all preds
      _bci2node->copy(i, --_looped);
    }
  }

  _map = NULL;
}

//------------------------------merge_value------------------------------------
// Turn the idx'th entry of the current map into a Phi
Node *Parse::merge_value( Node *map, uint idx, Node *region ) {
  Node *o = map->in(idx);
  if( o == _top ) return o;     // TOP always merges into TOP
  if( o->Opcode() == Op_Phi &&  // Phi already?
      o->in(0) == region )      // And of this merge point?
    return o;
  
  uint preds = region->req();   // Number of predecessor paths
  const Type *t = o->bottom_type();
  switch( t->_base ) {          // Flatten worst-case type for Phis
  case Type::DoubleCon: t = Type::DOUBLE;   break;
  case Type::FloatCon:  t = Type::FLOAT;    break;
  case Type::Int:       t = TypeInt::INT;   break;
  case Type::Long:      t = TypeLong::LONG; break;
  case Type::AnyPtr:    
  case Type::OopPtr:
  case Type::KlassPtr: // fall through
  case Type::AryPtr:    t = TypeOopPtr::BOTTOM; break;
  default: break;
  }
  Node *p = new PhiNode(region, t);
  _gvn._types.map(p->_idx, t);
  for( uint j = 1; j < preds; j++ ) // Fill in all prior inputs
    p->set_req(j, region->in(j) ? o : NULL ); 
  map->set_req( idx, p );       // Now use a Phi here for merging
  return p;
}

//------------------------------return_current---------------------------------
// Append current _map to _exit_return
void Parse::return_current( Node *parm0, int bci ) {
  if( _method->is_synchronized() && GenerateSynchronizationCode ) {
    shared_unlock( _synch_lock->box_node(), _synch_lock->obj_node(), SynchronizationEntryBCI);
  }
  // JVMPI -- record exit from a method if compiled while JVMPI is turned on
  if( C->is_set_jvmpi_method_event() ) {
    make_jvmpi_method_exit(bci);
  }
  _exit_return->in( TypeFunc::Control  )->add_req( control() );
  _exit_return->in( TypeFunc::I_O      )->add_req( i_o    () );
  Node *mem = _exit_return->in( TypeFunc::Memory   );
  for( uint i = 1; i < C->_cnt_map_adr_mem; i++ ) 
    mem->in(i)->add_req( memory(i) );
  
  // frame pointer is always same, already captured
  if( parm0 ) {
    // If returning oops to an interface-return, there is a silent free
    // cast from oop to interface allowed by the Verifier.  Make it explicit
    // here.
    Node *phi = _exit_return->in(TypeFunc::Parms);
    const TypeOopPtr *tr = phi->bottom_type()->isa_oopptr();
    if( tr && tr->klass()->is_loaded() &&
	tr->klass()->is_interface() ) {
      const TypeOopPtr *tp = parm0->bottom_type()->isa_oopptr();
      if( tp && tp->klass()->is_loaded() &&
	  !tp->klass()->is_interface() ) {
        parm0 = _gvn.transform(new (2) CheckCastPPNode(0,parm0,tr));
      }
    }
    phi->add_req( parm0 );
  }
  _map = NULL;                  // This CFG path dies here
}


//------------------------------clone_map--------------------------------------
// Implementation of PreserveJVMState
//
// Only clone_map(...) here. If this function is only used in the
// PreserveJVMState class we may want to get rid of this extra
// function eventually and do it all there.

SafePointNode *Parse::clone_map( int bci) {
  if( !_map ) return NULL;
  
  // Clone the memory edge first
  Node *curmem = _map->memory();
  Node *mem = (curmem->Opcode() == Op_MergeMem) 
    ? curmem->clone()
    : new MergeMemNode(curmem,C->_cnt_map_adr_mem);
  _gvn._types.map(mem->_idx, mem->bottom_type());

  SafePointNode *clonemap = (SafePointNode*)_map->clone();
  clonemap->set_req(TypeFunc::Memory,mem);
  clonemap->_jvms = _map->_jvms->clone();
  clonemap->_jvms->set_map(clonemap);
  clonemap->_jvms->set_bci(bci);
  clonemap->_jvms->set_sp(_map->_jvms->sp());
  _gvn._types.map(clonemap->_idx, clonemap->bottom_type());
  return clonemap;
}

//------------------------------add_safepoint----------------------------------
void Parse::add_safepoint(ciByteCodeStream &iter) {
  // See if we can avoid this safepoint.  No need for a SafePoint immediately
  // after a Call or another SafePoint.
  Node *proj = control();
  if( proj->is_Proj() ) {
    Node *n0 = proj->in(0);
    if( n0->is_Call() || n0->is_Catch() ) return;
    if( n0->is_SafePoint() && n0->req() >= TypeFunc::Parms ) 
      return;
  }

  // Clone the JVM State
  SafePointNode *sfpnt = new SafePointNode(TypeFunc::Parms, NULL);

  // Capture memory state BEFORE a SafePoint.  Since we can block at a
  // SafePoint we need our GC state to be safe; i.e. we need all our current
  // write barriers (card marks) to not float down after the SafePoint.
  // Likewise we need all oop stores.  If deopt can happen, we need ALL stores.

  // We do not need to WRITE the memory state after a SafePoint (except for
  // raw memory because eden pointers will have moved).  Anti-dependencies
  // will keep card-marks and oop-stores from floating up from below a
  // SafePoint and our true dependency added here will keep them from floating
  // down below a SafePoint.

  // Clone the current memory state
  Node *mem = new MergeMemNode(_map->memory(),C->_cnt_map_adr_mem);

  mem = _gvn.transform(mem);
  
  // Pass control through the safepoint
  sfpnt->set_req(TypeFunc::Control  , control());
  // Fix edges normally used by a call
  sfpnt->set_req(TypeFunc::I_O      , _top);
  sfpnt->set_req(TypeFunc::Memory   , mem );
  sfpnt->set_req(TypeFunc::ReturnAdr, _top);
  sfpnt->set_req(TypeFunc::FramePtr , _top);

  // Fix up the JVM State edges
  add_safepoint_edges(sfpnt, iter.cur_bci());
  Node *s = _gvn.transform(sfpnt);

  // Memory barrier to avoid floating raw memory (eden) pass a safepoint.
  // Cannot use "_gvn.transform()" here because we may attempt to remove
  // an unneeded safepoint in ProjNode::Identity and that will try to 
  // hand the call's output memory to set_memory.
  uint raw_idx = C->get_alias_index(TypeRawPtr::BOTTOM);
  mem = new (1) ProjNode(s, TypeFunc::Memory );
  _gvn._types.map(mem->_idx,Type::MEMORY);
  _for_igvn.push(mem);		// Clean out useless safepoints later
  set_memory (mem, raw_idx );
  set_control(_gvn.transform( new (1) ProjNode(s, TypeFunc::Control) ));
}

#ifndef PRODUCT
//------------------------------dump-------------------------------------------
// Dump information associated with the bytecodes of current _method
void Parse::dump() {
  if( _method != NULL ) {
    // Iterate over bytecodes
    ciByteCodeStream iter(_method);
    for( Bytecodes::Code bc = iter.next(); bc != ciByteCodeStream::EOBC ; bc = iter.next() ) {
      dump_bci( iter.cur_bci() );
      tty->cr();
    }
  }
}

// Dump information associated with a byte code index, 'bci'
void Parse::dump_bci(int bci) {
  // Output info on merge-points, cloning, and within _jsr..._ret
  tty->print("%s", bits_merge(bci) ? "M" : " ");
  tty->print("%s", bits_clone(bci) ? "C" : " ");
  if( jsr_stack_depth() != 0 ) {
    for(int i = 0; i < jsr_stack_depth(); i++) {
      tty->print("J%-d ", jsr_return_at(i));
    }
  } else {
    tty->print("      ");
  }
  tty->print(" bci:%d", bci);
  if( _preds[bci] != 1 ) tty->print(" preds:%d", _preds[bci]);
}

#endif
