/*
 * @(#)parse2.cpp	1.281 01/03/21 17:31:39
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_parse2.cpp.incl"

extern int explicit_null_checks_inserted, 
           explicit_null_checks_elided;

//------------------------------basic_plus_adr---------------------------------
Node *Parse::basic_plus_adr( Node *base, Node *ptr, int offset ) {
  return _gvn.transform( new (4) AddPNode(base,ptr,_gvn.intcon(offset)) );
}

//------------------------------array_addressing-------------------------------
// Pull array and index from the stack.  Compute pointer-to-element.
Node *Parse::array_addressing( uint stride, BasicType type, int bci, int vals ) {
  Node *idx   = stack(_sp-1-vals);   // Get from stack without popping
  Node *ary   = stack(_sp-2-vals);   // in case of exception

  // Null check the array base, with correct stack contents
  ary = do_null_check(ary, bci, T_ARRAY);
  // Compile-time detect of null-exception?
  if( !_map ) return _top;

  if( !_gvn.type(ary)->isa_aryptr() ) { // Not yet typed as an array?
    const TypePtr *t_a = TypeAryPtr::make( TypePtr::BotPTR, TypeAry::make(Type::get_const_basic_type(type),TypeInt::POS), 0, 0 );
    ary = cast_up( ary, t_a );
  }

  // Check for big class initializers with all constant offsets 
  // feeding into a known-size array.
  const TypeAryPtr *ta = _gvn.type(ary)->is_aryptr();
  const TypeInt *ts = ta->_ary->_size;

  // Do the range check
  if( GenerateRangeChecks &&
      // Check for parse-time-known no-range-check failure & skip it
      !(ts->is_con() && idx->is_Con() && (uint)idx->get_int() < (uint)ts->get_con()) ) {
    Node *r_adr = basic_plus_adr( ary, ary, arrayOopDesc::length_offset_in_bytes());
    // Range is constant in array-oop, so we can use the original state of mem
    Node *len   = _gvn.transform( new (3) LoadRangeNode(0, memory(TypeAryPtr::RANGE), r_adr, TypeInt::POS));
    // Test length vs index (standard trick using unsigned compare)
    Node *chk   = _gvn.transform( new (3) CmpUNode( idx, len) );
    BoolTest::mask btest = BoolTest::lt;
    Node *tst   = _gvn.transform( new (2) BoolNode( chk, btest) );
    // Branch to failure if out of bounds
    IfNode *iff = create_and_map_if( control(), tst, 0.999999f, -1.0f );
    Node *iffalse = _gvn.transform( new (1) IfFalseNode(iff) );
    // Add exception exit path - just uncommon trap
    set_control(iffalse);
    { PreserveJVMState pjvms(this, bci);
      // End the current control-flow path
      uncommon_trap(Deoptimization::Deopt_range_check,bci); 
    }
    
    // Else fall through
    Node *iftrue = _gvn.transform( new (1) IfTrueNode(iff) );
    set_control(iftrue);
  }
  // Check for always knowing you are throwing a range-check exception
  if( control() == _top ) return _top;

  // must be correct type for alignment purposes
  Node *base  = basic_plus_adr( ary, ary, arrayOopDesc::base_offset_in_bytes(type)); 
  Node *c4    = _gvn.intcon(stride);
  Node *scale = _gvn.transform( new (3) MulINode( idx, c4 ) );
  Node *ptr   = _gvn.transform( new (4) AddPNode( ary, base, scale ) );
  return ptr;
}

Node *Parse::array_addressing_hi_word( Node *ptr ) {
  AddPNode *addp = ptr->is_AddP();
  if( !addp ) return _top;      // Handle dead long word load or store

  Node *scale = ptr->in(AddPNode::Offset);
  Node *base  = ptr->in(AddPNode::Address);
  Node *word  = _gvn.intcon(sizeof(jint));
  Node *scaleplus = _gvn.transform( new (3) AddINode( scale, word) );
  return _gvn.transform( new (4) AddPNode( ptr->in(AddPNode::Base), base, scaleplus ) );    
}


//------------------------------store_barrier----------------------------------
// Insert a write-barrier store.  This is to let generational GC work; we have 
// to flag all oop-stores before the next GC point.
void Parse::store_barrier(Node* adr, Node* val) {
  // No store check needed if we're storing a NULL or an old object
  // (latter case is probably a string constant.)
  if (val->is_Con()) {
    // must be either an oop or NULL
    const Type* t = val->bottom_type();
    assert( t == TypePtr::NULL_PTR || t->isa_javaptr()->const_oop() != NULL, 
	    "must be either a constant oop or NULL");
    // no store barrier needed, because no old-to-new ref created
    return;
  }
  // Get the alias_index for raw card-mark memory
  uint alias_idx = C->get_alias_index(TypeRawPtr::BOTTOM);
  Node *mem = memory(alias_idx);
  // Convert the pointer to an int prior to doing math on it
  Node *cast = _gvn.transform(new (3) CastIPNode(control(), mem, adr));
  // Divide by card size
  Node *a = _gvn.transform(new (3) URShiftINode( cast, _gvn.intcon(RememberedSet::card_shift) ));
  // We store into a byte array, so do not bother to left-shift by zero
  Node *b = a;
  // Get base of card map
  assert(sizeof(*RememberedSet::byte_map_base) == sizeof(jbyte), "adjust this code");
  Node *c = makecon(TypeRawPtr::make((address)RememberedSet::byte_map_base));
  // Combine
  Node *d = _gvn.transform(new (4) AddPNode( _top/*no base ptr*/, c, b ));
  // Smash zero into card
  Node *card_mark = _gvn.transform(new(4) StoreBNode(control(), mem, d, _gvn.intcon(0), Type::MEMORY) );
  set_memory( card_mark, alias_idx );
  // Check for back-to-back identical card marks
  if( mem->req() > MemNode::Address && d == mem->in(MemNode::Address) )
    _for_igvn.push( card_mark );
}


//------------------------------Long Shift-------------------------------------
// Helper function for long shift right

// returns IfNode
IfNode* Parse::jump_if_fork_int(Node* a, Node* b, BoolTest::mask mask) {
  Node   *cmp = _gvn.transform( new (3) CmpINode( a, b)); // two cases: shiftcount > 32 and shiftcount <= 32
  Node   *tst = _gvn.transform( new (2) BoolNode( cmp, mask));
  IfNode *iff = create_and_map_if( control(), tst, ((mask == BoolTest::eq) ? 0.1f : 0.5f), -1.0f );
  return iff;
}

// return Region node
Node* Parse::jump_if_join(Node* iffalse, Node* iftrue) {
  Node *region  = new RegionNode(3); // 2 results
  _for_igvn.push(region);
  region->set_req(1, iffalse);
  region->set_req(2, iftrue );
  _gvn._types.map(region->_idx, region->bottom_type());
  region = _gvn.transform(region);
  set_control (region);
  return region;
}


//------------------------------helper for tableswitch-------------------------
void Parse::jump_if_true_fork(IfNode *iff, int dest_bci_if_true, uint cloning){
  // True branch, use existing _map info
  { PreserveJVMState pjvms(this, _map->_jvms->bci());
    Node *iftrue  = _gvn.transform( new (1) IfTrueNode (iff) );
    set_control( iftrue );
    merge( dest_bci_if_true, cloning );
  }

  // False branch
  Node *iffalse = _gvn.transform( new (1) IfFalseNode(iff) );
  set_control( iffalse );
}

void Parse::jump_if_false_fork(IfNode *iff, int dest_bci_if_true, uint cloning){
  // True branch, use existing _map info
  { PreserveJVMState pjvms(this, _map->_jvms->bci());
    Node *iffalse  = _gvn.transform( new (1) IfFalseNode (iff) );
    set_control( iffalse );
    merge( dest_bci_if_true, cloning );
  }

  // False branch
  Node *iftrue = _gvn.transform( new (1) IfTrueNode(iff) );
  set_control( iftrue );
}

void Parse::jump_if_always_fork(int dest_bci, uint cloning) {
  // False branch, use existing _map and control()
  merge( dest_bci, cloning );
}


extern "C" {
  static int jint_cmp(const void *i, const void *j) {
    int a = *(jint *)i;
    int b = *(jint *)j;
    return a > b ? 1 : a < b ? -1 : 0;
  }
}


class SwitchRange : public StackObj {
  // a range of integers coupled with a bci destination
  jint _lo;			// inclusive lower limit
  jint _hi;			// inclusive upper limit
  int _dest;

public:
  jint lo() const              { return _lo;   }
  jint hi() const              { return _hi;   }
  int  dest() const            { return _dest; }
  bool is_singleton() const    { return _lo == _hi; }

  void setRange(jint lo, jint hi, int dest) {
    assert(lo <= hi, "must be a non-empty range");
    _lo = lo, _hi = hi; _dest = dest;
  }
  bool adjoinRange(jint lo, jint hi, int dest) {
    assert(lo <= hi, "must be a non-empty range");
    if (lo == _hi+1 && dest == _dest) {
      _hi = hi;
      return true;
    }
    return false;
  }

  void set   (jint value, int dest) {           setRange(value, value, dest); }
  bool adjoin(jint value, int dest) { return adjoinRange(value, value, dest); }

  void print(ciEnv* env) {
    if (is_singleton())
      tty->print(" {%d}=>%d", lo(), dest());
    else if (lo() == min_jint)
      tty->print(" {..%d}=>%d", hi(), dest());
    else if (hi() == max_jint)
      tty->print(" {%d..}=>%d", lo(), dest());
    else
      tty->print(" {%d..%d}=>%d", lo(), hi(), dest());
  }
};


void Parse::jump_switch_ranges(int bci, Node* key_val, SwitchRange *lo, SwitchRange *hi, uint cloning, int switch_depth) {
  if (switch_depth == 0) {
    // Maybe prune the inputs, based on the type of key_val.
    jint min_val = min_jint;
    jint max_val = max_jint;
    const TypeInt* ti = key_val->bottom_type()->isa_int();
    if (ti != NULL) {
      min_val = ti->_lo;
      max_val = ti->_hi;
      assert(min_val <= max_val, "invalid int type");
    }
    while (lo->hi() < min_val)  lo++;
    if (lo->lo() < min_val)  lo->setRange(min_val, lo->hi(), lo->dest());
    while (hi->lo() > max_val)  hi--;
    if (hi->hi() > max_val)  hi->setRange(hi->lo(), max_val, hi->dest());
  }

#ifndef PRODUCT
  if (switch_depth == 0) {
    _max_switch_depth = 0;
    _est_switch_depth = log2((hi-lo+1)-1)+1;
  }
#endif

  assert(lo <= hi, "must be a non-empty set of ranges");
  if (lo == hi) {
    jump_if_always_fork(lo->dest(), cloning);
  } else {
    assert(lo->hi() == (lo+1)->lo()-1, "contiguous ranges");
    assert(hi->lo() == (hi-1)->hi()+1, "contiguous ranges");
    int nr = hi - lo + 1;

    SwitchRange* mid = lo + nr/2;
    // if there is an easy choice, pivot at a singleton:
    if (nr > 3 && !mid->is_singleton() && (mid-1)->is_singleton())  mid--;

    assert(lo < mid && mid <= hi, "good pivot choice");
    assert(nr != 2 || mid == hi,   "should pick higher of 2");
    assert(nr != 3 || mid == hi-1, "should pick middle of 3");

    Node *test_val = _gvn.intcon(mid->lo());

    if (mid->is_singleton()) {
      IfNode *iff_ne = jump_if_fork_int(key_val, test_val, BoolTest::ne);
      jump_if_false_fork( iff_ne, mid->dest(), cloning );

      // Special Case:  If there are exactly three ranges, and the high
      // and low range each go to the same place, omit the "gt" test,
      // since it will not discriminate anything.
      bool eq_test_only = (hi == lo+2 && hi->dest() == lo->dest());
      if (eq_test_only) {
	assert(mid == hi-1, "");
	//_preds[lo->dest()] -= 1;
      }

      // if there is a higher range, test for it and process it:
      if (mid < hi && !eq_test_only) {
	// two comparisons of same values--should enable 1 test for 2 branches
        // Use BoolTest::le instead of BoolTest::gt
	IfNode *iff_le  = jump_if_fork_int(key_val, test_val, BoolTest::le);
	Node   *iftrue  = _gvn.transform( new (1) IfTrueNode(iff_le) );
	Node   *iffalse = _gvn.transform( new (1) IfFalseNode(iff_le) );
	{ PreserveJVMState pjvms(this, bci);
	  set_control(iffalse);
	  jump_switch_ranges(bci, key_val, mid+1, hi, cloning, switch_depth+1);
	}
	set_control(iftrue);
      }

    } else {
      // mid is a range, not a singleton, so treat mid..hi as a unit
      IfNode *iff_ge = jump_if_fork_int(key_val, test_val, BoolTest::ge);

      // if there is a higher range, test for it and process it:
      if (mid == hi) {
	jump_if_true_fork( iff_ge, mid->dest(), cloning );
      } else {
	Node *iftrue  = _gvn.transform( new (1) IfTrueNode(iff_ge) );
	Node *iffalse = _gvn.transform( new (1) IfFalseNode(iff_ge) );
	{ PreserveJVMState pjvms(this, bci);
	  set_control(iftrue);
	  jump_switch_ranges(bci, key_val, mid, hi, cloning, switch_depth+1);
	}
	set_control(iffalse);
      }
    }

    // in any case, process the lower range
    jump_switch_ranges(bci, key_val, lo, mid-1, cloning, switch_depth+1);
  }

#ifndef PRODUCT
  _max_switch_depth = max(switch_depth, _max_switch_depth);
  if (TraceOptoParse && Verbose && WizardMode && switch_depth == 0) {
    SwitchRange* r;
    int nsing = 0;
    for( r = lo; r <= hi; r++ ) {
      if( r->is_singleton() )  nsing++;
    }
    tty->print(">>> ");
    _method->print_short_name();
    tty->print_cr(" switch decision tree");
    tty->print_cr("    %d ranges (%d singletons), max_depth=%d, est_depth=%d",
		  hi-lo+1, nsing, _max_switch_depth, _est_switch_depth);
    if (_max_switch_depth > _est_switch_depth) {
      tty->print_cr("******** BAD SWITCH DEPTH ********");
    }
    tty->print("   ");
    for( r = lo; r <= hi; r++ ) {
      r->print(env());
    }
    tty->print_cr("");
  }
#endif
}

// Reading and setting the memory state is way conservative here.
// The real problem is that I am not doing real Type analysis on memory,
// so I cannot distinguish OOP stores from other stores.  Across a GC
// point the Store Barrier and the OOP memory has to agree.  I cannot have
// an OOP store and its barrier split across the GC point from either above
// or below.  Here I get that to happen by reading ALL of memory.  
// A better answer would be to seperate out OOPs from other memory.
void Parse::set_predefined_input_for_runtime_call(SafePointNode* call) {
  // Set fixed predefined input arguments
  call->set_req( TypeFunc::Control, control() );
  call->set_req( TypeFunc::I_O    , _top );        // does no i/o
  call->set_req( TypeFunc::Memory , all_memory() ); // may gc ptrs
  call->set_req( TypeFunc::ReturnAdr, _top );
  call->set_req( TypeFunc::FramePtr, frameptr() );
}

void Parse::set_predefined_output_for_runtime_call(Node* call) {
  // no i/o
  set_control    (_gvn.transform(new (1) ProjNode(call,TypeFunc::Control) ));
  set_all_memory_call(call);
}

void Parse::modf() {
  Node *f2 = pop();
  Node *f1 = pop();
  CallNode *call = new CallLeafNode(OptoRuntime::modf_Type(), CAST_FROM_FN_PTR(address, SharedRuntime::frem), "frem");

  set_predefined_input_for_runtime_call(call);
  call->set_req( TypeFunc::Parms+0, f1 );
  call->set_req( TypeFunc::Parms+1, f2 );

  Node* c = _gvn.transform(call);
  // Set fixed predefined return values
  set_predefined_output_for_runtime_call(c);
  Node* res = _gvn.transform(new (1) ProjNode(c, TypeFunc::Parms + 0));

  push(res);
}

void Parse::modd() {
  pop(); 
  Node *d2 = pop();
  pop(); 
  Node *d1 = pop();
  CallNode *call = new CallLeafNode(OptoRuntime::modd_Type(), CAST_FROM_FN_PTR(address, SharedRuntime::drem), "drem");

  set_predefined_input_for_runtime_call(call);
  call->set_req( TypeFunc::Parms+0, d1 );
  call->set_req( TypeFunc::Parms+1, _top );
  call->set_req( TypeFunc::Parms+2, d2 );
  call->set_req( TypeFunc::Parms+3, _top );

  // The C calling convention may require the putting the doubles in
  // unusual places.
  call->fix_mispaired_double_longs();
 
  Node* c = _gvn.transform(call);
  // Set fixed predefined return values
  set_predefined_output_for_runtime_call(c);
  Node* res_d   = _gvn.transform(new (1) ProjNode(c, TypeFunc::Parms + 0));
  Node* res_top = _gvn.transform(new (1) ProjNode(c, TypeFunc::Parms + 1));

  push(res_d);
  push(res_top);
}

// Handle jsr and jsr_w bytecode
void Parse::do_jsr(ciByteCodeStream &iter) {
  Bytecodes::Code bc = iter.cur_bc();
  assert(bc == Bytecodes::_jsr || bc == Bytecodes::_jsr_w, "wrong bytecode");

  // Store information about current state, tagged with new _jsr_bci
  int return_bci = iter.next_bci();
  int jsr_bci    = (bc == Bytecodes::_jsr) ? iter.get_dest() : iter.get_far_dest();

  // Remember return address
  push_jsr_return(return_bci);

  // Effect on jsr on stack
  push(_gvn.intcon(return_bci));

  // Change iterator to start parsing finally block
  iter.reset_to_bci(jsr_bci);        
}

// Handle ret bytecode
void Parse::do_ret(ciByteCodeStream &iter) {
  assert( !is_jsr_stack_empty(), "_ret must be preceeded by a _jsr");
  
  // Set iterator to continue at return point
  int return_bci = pop_jsr_return();
  iter.reset_to_bci(return_bci);    
}


// rounding for strict float precision conformance
Node* Parse::precision_rounding(Node* n) {
  return ((UseStrictFP && _method->flags().is_strict()) || EnforceStrictFP ) 
         && Matcher::strict_fp_requires_explicit_rounding
    ? _gvn.transform( new (2) RoundFloatNode(0, n) )
    : n;
}

// rounding for strict double precision conformance
Node* Parse::dprecision_rounding( Node* n ) {
  return ((UseStrictFP && _method->flags().is_strict()) || EnforceStrictFP ) 
         && Matcher::strict_fp_requires_explicit_rounding
    ? _gvn.transform( new (2) RoundDoubleNode(0, n) )
    : n;
}

// rounding for non-strict double stores
Node *Parse::dstore_rounding(Node *n) {
  return Matcher::strict_fp_requires_explicit_rounding
    ? _gvn.transform( new (2) RoundDoubleNode(0, n) )
    : n;
}

//------------------------------branch_prediction------------------------------
// Try to gather dynamic branch prediction behavior.  Return a probability
// of the branch being taken and set the "cnt" field.  Returns a -1.0
// if we need to use static prediction for some reason.
static float branch_prediction( int bcp, int cloning, float &cnt, float site_invoke_ratio ) {
  cnt  = -1.0f;
  // Use branch frequency information if it is available
  if( !CountInterpBranches ) return -1.0f;

  // Mask bcp into table
  bcp = bcp & right_n_bits(BranchFreqTableSize);

  // index into table to get taken and not taken values
  int     taken = SharedInfo::freqTable[bcp].    _taken;
  int not_taken = SharedInfo::freqTable[bcp]._not_taken;

  // Give up if too few counts to be meaningful
  if( taken + not_taken < 40 ) return -1.0f;

  // Count is always the sum of the counters, normalized by percent of
  // execution taken up by this inlining 
//  cnt = (float)(taken + not_taken) * site_invoke_ratio;
  cnt = (float)(taken + not_taken) / (float)FreqCountInvocations;

  // Pin probability to sane limits
  float prob;
  if( !taken )
    prob = 0.000001f;
  else if( !not_taken )
    prob = 0.999999f;
  else                          // Compute probability of true path
    prob = (float)taken / (float)(taken + not_taken);


  assert((cnt == -1.0f  || cnt > 0.0f) && (prob > 0.0f),
         "Bad frequency assignment in if");

  // If cloning, then the counts are NOT true (count in the cloned code
  // is probable actually 1; the cloned code occurs before, not in, a loop).
  // However, the relative branch predictions are probably OK; keep the
  // prob value but smash the cnt value.
  if( cloning ) cnt = -1.0f;

  return prob;
}

//------------------------------do_one_bytecode--------------------------------
// Parse this bytecode, and alter the Parsers JVM->Node mapping
void Parse::do_one_bytecode( Bytecodes::Code bc, ciByteCodeStream &iter, uint cloning ) {
  Node *a, *b, *c, *d;          // Handy temps
  BoolTest::mask btest;
  int i;

  if( _root && C->_unique > 60000 ) {
    _bailout = Bailout_node_limit_exceeded;
    return;
  }

#ifdef ASSERT
  // for setting breakpoints
  int bci = iter.cur_bci();
  if( TraceOptoParse ) {
    tty->print("  ");
    dump_bci(bci);
    if( cloning != 0 ) tty->print("  cloning");
    tty->cr();
  }
#endif

  switch( bc ) {
  case Bytecodes::_nop:
    // do nothing
    break;
  case Bytecodes::_lconst_0:
    push(makecon(TypeLong::ZERO));
    push(_top);
    break;

  case Bytecodes::_lconst_1:
    push(makecon(TypeLong::ONE));
    push(_top);
    break;

  case Bytecodes::_fconst_0:
    push(makecon(TypeF::ZERO));
    break;

  case Bytecodes::_fconst_1:
    push(makecon(TypeF::ONE));
    break;

  case Bytecodes::_fconst_2:
    a = _gvn.transform(new ConFNode(TypeF::make(2.0f)));
    push( a );
    break;

  case Bytecodes::_dconst_0:
    push(makecon(TypeD::ZERO));
    push(_top);
    break;

  case Bytecodes::_dconst_1:
    push(makecon(TypeD::ONE));
    push(_top);
    break;

  case Bytecodes::_iconst_m1:push( _gvn.intcon(-1)); break;
  case Bytecodes::_iconst_0: push( _gvn.intcon( 0)); break;
  case Bytecodes::_iconst_1: push( _gvn.intcon( 1)); break;
  case Bytecodes::_iconst_2: push( _gvn.intcon( 2)); break;
  case Bytecodes::_iconst_3: push( _gvn.intcon( 3)); break;
  case Bytecodes::_iconst_4: push( _gvn.intcon( 4)); break;
  case Bytecodes::_iconst_5: push( _gvn.intcon( 5)); break;
  case Bytecodes::_bipush:   push( _gvn.intcon( iter.get_byte())); break;
  case Bytecodes::_sipush:   push( _gvn.intcon( iter.get_short())); break;
  case Bytecodes::_aconst_null: 
    push( _null );
    break;
    // !!!!!
  case Bytecodes::_ldc:
  case Bytecodes::_ldc_w:
  case Bytecodes::_ldc2_w: {
    if( iter.is_unresolved_string() ) {
      uncommon_trap( Deoptimization::Deopt_uninitialized, iter.cur_bci() );
      break;
    }
    if( method()->is_not_reached(iter.cur_bci()) && 
	CompileThreshold > 100 && !CompileTheWorld ) {
      uncommon_trap(Deoptimization::Deopt_uninitialized, iter.cur_bci());
      break;
    }

    ciConstant constant = iter.get_constant();
    switch(constant.type()) {
    case T_INT:
      push( _gvn.intcon( constant.as_int() ));
      break;
    case T_FLOAT: {
      jfloat fval = constant.as_float();
      a = _gvn.transform(new ConFNode(TypeF::make(fval)));
      push( a );
      break;
    }
    case T_OBJECT:
      push( _gvn.makecon(TypeOopPtr::make(constant.as_object())) );
      break;
    case T_LONG: {
      jlong lval = constant.as_long();
      a = _gvn.transform(new ConLNode(TypeLong::make(lval)));
      push( a );
      push(_top);
      break;
    }
    case T_DOUBLE: {
      jdouble dval = constant.as_double();
      a = _gvn.transform(new ConDNode(TypeD::make(dval)));
      push( a );
      push(_top);
      break;
    }
    default:
      tty->print_cr("Unimplemented CONSTANT_* type==%d", constant.type());
      ShouldNotReachHere();
      break;
    }                  
    break;
  }

  case Bytecodes::_aload_0:
    push( local(0) );
    break;
  case Bytecodes::_aload_1:
    push( local(1) );
    break;
  case Bytecodes::_aload_2:
    push( local(2) );
    break;
  case Bytecodes::_aload_3:
    push( local(3) );
    break;
  case Bytecodes::_aload:
    push( local(iter.get_index()) );
    break;

  case Bytecodes::_fload_0:
  case Bytecodes::_iload_0:
    push( local(0) );
    break;
  case Bytecodes::_fload_1:
  case Bytecodes::_iload_1:
    push( local(1) );
    break;
  case Bytecodes::_fload_2:
  case Bytecodes::_iload_2:
    push( local(2) );
    break;
  case Bytecodes::_fload_3:
  case Bytecodes::_iload_3:
    push( local(3) );
    break;
  case Bytecodes::_fload:
  case Bytecodes::_iload:    
    push( local(iter.get_index()) );
    break;
  case Bytecodes::_lload_0:
    push( local(0) );
    push( local(1) );
    break;
  case Bytecodes::_lload_1:
    push( local(1) );
    push( local(2) );
    break;
  case Bytecodes::_lload_2:
    push( local(2) );
    push( local(3) );
    break;
  case Bytecodes::_lload_3:
    push( local(3) );
    push( local(4) );
    break;
  case Bytecodes::_lload:
    push(local(iter.get_index()  ));
    push(local(iter.get_index()+1));
    break;

  case Bytecodes::_dload_0:
    push( local(0) );
    push( _top );
    break;
  case Bytecodes::_dload_1:
    push( local(1) );
    push( _top );
    break;
  case Bytecodes::_dload_2:
    push( local(2) );
    push( _top );
    break;
  case Bytecodes::_dload_3:
    push( local(3) );
    push( _top );
    break;
  case Bytecodes::_dload:
    push( local(iter.get_index()) );
    push( _top );
    break;
  case Bytecodes::_fstore_0:
  case Bytecodes::_istore_0:
  case Bytecodes::_astore_0:
    set_local( 0, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_fstore_1:
  case Bytecodes::_istore_1:
  case Bytecodes::_astore_1:
    set_local( 1, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_fstore_2:
  case Bytecodes::_istore_2:
  case Bytecodes::_astore_2:
    set_local( 2, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_fstore_3:
  case Bytecodes::_istore_3:
  case Bytecodes::_astore_3:
    set_local( 3, stack(_sp-1) ); _sp--;
    break; 
  case Bytecodes::_fstore:
  case Bytecodes::_istore:
  case Bytecodes::_astore:
    set_local( iter.get_index(), stack(_sp-1) ); _sp--;
    break; 
  // long stores 
  case Bytecodes::_lstore_0:
    set_local( 1, stack(_sp-1) ); _sp--;
    set_local( 0, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_lstore_1:
    set_local( 2, stack(_sp-1) ); _sp--;
    set_local( 1, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_lstore_2:
    set_local( 3, stack(_sp-1) ); _sp--;
    set_local( 2, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_lstore_3:
    set_local( 4, stack(_sp-1) ); _sp--;
    set_local( 3, stack(_sp-1) ); _sp--;
    break;
  case Bytecodes::_lstore:
    set_local( iter.get_index()+1, stack(_sp-1) ); _sp--;
    set_local( iter.get_index()+0, stack(_sp-1) ); _sp--;
    break;

  // double stores 
  case Bytecodes::_dstore_0:
    pop();
    set_local( 0, dstore_rounding(stack(_sp-1)) ); _sp--;
    break;
  case Bytecodes::_dstore_1:
    pop();
    set_local( 1, dstore_rounding(stack(_sp-1)) ); _sp--;
    break;
  case Bytecodes::_dstore_2:
    pop();
    set_local( 2, dstore_rounding(stack(_sp-1)) ); _sp--;
    break;
  case Bytecodes::_dstore_3:
    pop();
    set_local( 3, dstore_rounding(stack(_sp-1)) ); _sp--;
    break;
  case Bytecodes::_dstore:
    pop();
    set_local( iter.get_index()+0, dstore_rounding(stack(_sp-1)) ); _sp--;
    break;

  case Bytecodes::_pop:  _sp -= 1;   break;
  case Bytecodes::_pop2: _sp -= 2;   break;
  case Bytecodes::_swap: 
    a = pop();
    b = pop();
    push(a);
    push(b);
    break;
  case Bytecodes::_dup:
    a = pop();
    push(a);
    push(a);
    break;
  case Bytecodes::_dup_x1: 
    a = pop();
    b = pop();
    push( a );
    push( b );
    push( a );
    break;
  case Bytecodes::_dup_x2: 
    a = pop();
    b = pop();
    c = pop();
    push( a );
    push( c );
    push( b );
    push( a );
    break;
  case Bytecodes::_dup2: 
    a = pop();
    b = pop();
    push( b );
    push( a );
    push( b );
    push( a );
    break;

  case Bytecodes::_dup2_x1:
    // before: .. c, b, a
    // after:  .. b, a, c, b, a
    // not tested
    a = pop();
    b = pop();
    c = pop();
    push( b );
    push( a );
    push( c );
    push( b );
    push( a );
    break;
  case Bytecodes::_dup2_x2:
    // before: .. d, c, b, a
    // after:  .. b, a, d, c, b, a
    // not tested
    a = pop();
    b = pop();
    c = pop();
    d = pop();
    push( b );
    push( a );
    push( d );
    push( c );
    push( b );
    push( a );
    break;

  case Bytecodes::_arraylength: {
    // Must do null-check with value on expression stack
    Node *ary = do_null_check(peek(),iter.cur_bci(), T_ARRAY); 
    // Compile-time detect of null-exception?
    if( !_map ) return;
    pop(); 
    Node *base  = basic_plus_adr( ary, ary, arrayOopDesc::length_offset_in_bytes());
    push( _gvn.transform( new (3) LoadRangeNode(0, memory(TypeAryPtr::RANGE), base, TypeInt::POS)));
    break;
  } 

  case Bytecodes::_aaload: {
    Node *ary   = stack(_sp-2);
    const Type *t = ary->bottom_type();
    const TypePtr *tp = ( t->_base == Type::AryPtr ) 
      ? t->is_aryptr()->_ary->_elem->isa_javaptr()
      : TypeOopPtr::BOTTOM;
    a = array_addressing( oopSize, T_OBJECT, iter.cur_bci(), 0 );
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadPNode(control(),memory(TypeAryPtr::OOPS),a,tp)));
    break;
  } 
  case Bytecodes::_baload: {
    a = array_addressing( sizeof(jbyte), T_BYTE, iter.cur_bci(), 0 );
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadBNode(control(),memory(TypeAryPtr::BYTES),a)));
    break;
  }
  case Bytecodes::_caload: {
    a = array_addressing(sizeof(jchar), T_CHAR, iter.cur_bci(), 0 );
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadCNode(control(),memory(TypeAryPtr::CHARS),a)));
    break;
  }
  case Bytecodes::_iaload: {
    a = array_addressing(sizeof(jint), T_INT, iter.cur_bci(), 0 );
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadINode(control(),memory(TypeAryPtr::INTS),a)));
    break;
  }
  case Bytecodes::_laload: {
    a = array_addressing(sizeof(jlong), T_LONG, iter.cur_bci(), 0);
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadLNode(control(),memory(TypeAryPtr::LONGS),a)));
    push(_top);
    break;
  }
  case Bytecodes::_saload: {
    a = array_addressing(sizeof(jshort), T_SHORT, iter.cur_bci(), 0);
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadSNode(control(),memory(TypeAryPtr::SHORTS),a)));
    break;
  }
  case Bytecodes::_faload: {
    a = array_addressing(sizeof(jfloat), T_FLOAT, iter.cur_bci(), 0);
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadFNode(control(),memory(TypeAryPtr::FLOATS),a)));
    break;
  }
  case Bytecodes::_daload: {
    a = array_addressing(sizeof(jdouble), T_DOUBLE, iter.cur_bci(), 0);
    if( !_map ) return;         // guarenteed null or range check
    _sp -= 2;			// Pop array and index
    push( _gvn.transform(new (3) LoadDNode(control(),memory(TypeAryPtr::DOUBLES),a)));
    push(_top);
    break;
  }
  case Bytecodes::_aastore: {
    a = array_addressing(oopSize, T_OBJECT, iter.cur_bci(),1);
    if( !_map ) return;         // guarenteed null or range check
    array_store_check(iter.cur_bci());
    c = pop();			// Oop to store
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::OOPS);
    set_memory( _gvn.transform( new (4) StorePNode(control(),memory(alias_idx),a,c,Type::MEMORY)),alias_idx);                                                   
    store_barrier(a, c);                                                        
    break;
  }
  case Bytecodes::_bastore: {
    a = array_addressing(sizeof(jbyte), T_BYTE, iter.cur_bci(),1);
    if( !_map ) return;         // guarenteed null or range check
    c = pop();
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::BYTES);
    set_memory( _gvn.transform( new (4) StoreBNode(control(),memory(alias_idx),a,c,Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_castore: {
    a = array_addressing(sizeof(jchar), T_CHAR, iter.cur_bci(),1);
    if( !_map ) return;         // guarenteed null or range check
    c = pop();
    // must truncate in case that it is an int
    d = _gvn.transform( new (3) AndINode(c,_gvn.intcon(0xFFFF)) ) ;
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::CHARS);
    set_memory( _gvn.transform( new (4) StoreCNode(control(),memory(alias_idx),a,d,Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_iastore: {
    a = array_addressing(sizeof(jint), T_INT, iter.cur_bci(),1);
    if( !_map ) return;         // guarenteed null or range check
    c = pop();
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::INTS);
    set_memory( _gvn.transform( new (4) StoreINode(control(),memory(alias_idx),a,c,Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_lastore: {
    a = array_addressing(sizeof(jlong), T_LONG, iter.cur_bci(),2);
    if( !_map ) return;         // guarenteed null or range check
    d = pop();			// T_CONFLICT
    c = pop();
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::LONGS);
    set_memory( _gvn.transform( new (4) StoreLNode(control(),memory(alias_idx),a,c,Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_sastore: {
    a = array_addressing(sizeof(jshort), T_SHORT, iter.cur_bci(),1);
    if( !_map ) return;         // guarenteed null or range check
    c = pop();
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::SHORTS);
    set_memory( _gvn.transform( new (4) StoreCNode(control(),memory(alias_idx),a,c,Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_dastore: {
    a = array_addressing(sizeof(jdouble), T_DOUBLE, iter.cur_bci(),2);
    if( !_map ) return;         // guarenteed null or range check
    d = pop();			// T_CONFLICT
    c = pop();
    _sp -= 2;			// Pop array and index
    c = dstore_rounding(c);
    uint alias_idx = C->get_alias_index(TypeAryPtr::DOUBLES);
    set_memory( _gvn.transform( new (4) StoreDNode(control(),memory(alias_idx),a,c,Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_fastore: {
    a = array_addressing(sizeof(jfloat), T_FLOAT, iter.cur_bci(),1);
    if( !_map ) return;         // guarenteed null or range check
    c = pop();
    _sp -= 2;			// Pop array and index
    uint alias_idx = C->get_alias_index(TypeAryPtr::FLOATS);
    set_memory( _gvn.transform( new (4) StoreFNode(control(), memory(alias_idx), a, c, Type::MEMORY)),alias_idx);
    break;
  }
  case Bytecodes::_getfield:
    do_get_xxx( &iter, true );
    break;

  case Bytecodes::_getstatic:
    do_get_xxx( &iter, false );
    break;

  case Bytecodes::_putfield:
    do_putfield( &iter );
    break;

  case Bytecodes::_putstatic:
    do_putstatic( &iter );
    break;

  case Bytecodes::_irem: {
    // Must keep both values on the expression-stack during null-check
    do_null_check(peek(), iter.cur_bci(), T_INT); 
    // Compile-time detect of null-exception?
    if( !_map ) return;
    b = pop();
    a = pop();  

    const Type *t = _gvn.type( b );
    if( t != Type::TOP ) {
      const TypeInt *ti = t->is_int();
      if( ti->is_con() ) {
        int divisor = ti->get_con();
        // check for positive power of 2
        if( divisor > 0 &&
            (divisor & ~(divisor-1)) == divisor ) {
          // yes !  
          Node *mask = _gvn.intcon((divisor - 1));
          // Sigh, must handle negative dividends
          Node *zero = _gvn.intcon(0);
          IfNode *ifff = jump_if_fork_int( a, zero, BoolTest::lt );
          Node *iff = _gvn.transform( new (1) IfFalseNode( ifff ) );
          Node *ift = _gvn.transform( new (1) IfTrueNode ( ifff ) );
          Node *reg = jump_if_join( ift, iff );
          Node *phi = new (3) PhiNode( reg, TypeInt::INT );
          // Negative path; negate/and/negate
          Node *neg = _gvn.transform( new (3) SubINode( zero, a ) );
          Node *andn= _gvn.transform( new (3) AndINode( neg, mask) );
          Node *negn= _gvn.transform( new (3) SubINode( zero, andn ) );
          phi->set_req( 1, negn );
          // Fast positive case
          Node *andx = _gvn.transform( new (3) AndINode( a, mask) );
          phi->set_req( 2, andx );
          // Push the merge
          push( _gvn.transform( phi ) );
          break;
        }
      }
    }
    // Default case
    push( _gvn.transform( new (3) ModINode(control(),a,b) ) );
    break;
  }
  case Bytecodes::_idiv:
    // Must keep both values on the expression-stack during null-check
    do_null_check(peek(), iter.cur_bci(), T_INT); 
    // Compile-time detect of null-exception?
    if( !_map ) return;
    b = pop();
    a = pop();  
    push( _gvn.transform( new (3) DivINode(control(),a,b) ) );
    break;
  case Bytecodes::_imul:
    b = pop(); a = pop();  
    push( _gvn.transform( new (3) MulINode(a,b) ) );
    break;
  case Bytecodes::_iadd:
    b = pop(); a = pop();  
    push( _gvn.transform( new (3) AddINode(a,b) ) );
    break;
  case Bytecodes::_ineg:
    a = pop();
    push( _gvn.transform( new (3) SubINode(_gvn.intcon(0),a)) );
    break;
  case Bytecodes::_isub:
    b = pop(); a = pop();  
    push( _gvn.transform( new (3) SubINode(a,b) ) );
    break;
  case Bytecodes::_iand:
    b = pop(); a = pop();  
    push( _gvn.transform( new (3) AndINode(a,b) ) );
    break;
  case Bytecodes::_ior:
    b = pop(); a = pop();  
    push( _gvn.transform( new (3) OrINode(a,b) ) );
    break;
  case Bytecodes::_ixor:
    b = pop(); a = pop();  
    push( _gvn.transform( new (3) XorINode(a,b) ) );
    break;
  case Bytecodes::_ishl:
    b = pop(); a = pop();
    push( _gvn.transform( new (3) LShiftINode(a,b) ) );
    break;
  case Bytecodes::_ishr:
    b = pop(); a = pop();
    push( _gvn.transform( new (3) RShiftINode(a,b) ) );
    break;
  case Bytecodes::_iushr:
    b = pop(); a = pop();
    push( _gvn.transform( new (3) URShiftINode(a,b) ) );
    break;

  case Bytecodes::_fneg:  
    a = pop();
    b = _gvn.transform(new (2) NegFNode (a));
    push(b);
    break;

  case Bytecodes::_fsub:
    b = pop();  
    a = pop();  
    c = _gvn.transform( new (3) SubFNode(a,b) );
    d = precision_rounding(c);
    push( d );
    break;

  case Bytecodes::_fadd:
    b = pop();  
    a = pop();  
    c = _gvn.transform( new (3) AddFNode(a,b) );
    d = precision_rounding(c);
    push( d );
    break;

  case Bytecodes::_fmul:
    b = pop();  
    a = pop();  
    c = _gvn.transform( new (3) MulFNode(a,b) );
    d = precision_rounding(c);
    push( d );
    break;

  case Bytecodes::_fdiv:
    b = pop();
    a = pop();  
    c = _gvn.transform( new (3) DivFNode(0,a,b) );
    d = precision_rounding(c);
    push( d );
    break;

  case Bytecodes::_frem:
    if (Matcher::modFSupported()) {
      // Generate a ModF node.
      b = pop();
      a = pop();  
      c = _gvn.transform( new (3) ModFNode(0,a,b) );
      d = precision_rounding(c);
      push( d );
    }
    else {
      // Generate a call.
      modf();
    }
    break;
    
  case Bytecodes::_fcmpl:
    b = pop();
    a = pop();
    c = _gvn.transform( new (3) CmpFLessNode( a, b)); 
    push(c);
    break;
  case Bytecodes::_fcmpg:
    b = pop();
    a = pop();
    c = _gvn.transform( new (3) CmpFGreaterNode( a, b)); 
    push(c);
    break;

  case Bytecodes::_f2i:
    push(_gvn.transform(new (2) ConvF2INode(pop())));
    break;

  case Bytecodes::_d2i:
    pop();
    push(_gvn.transform(new (2) ConvD2INode(pop())));
    break;

  case Bytecodes::_f2d:
    a = pop();
    b = _gvn.transform( new (2) ConvF2DNode(a));
    push ( b );
    push ( _top );
    break;

  case Bytecodes::_d2f:
    pop(); a = pop();
    b = _gvn.transform( new (2) ConvD2FNode(a));
    // This breaks _227_mtrt (speed & correctness) and _222_mpegaudio (speed)
    //c = _gvn.transform(new (2) RoundFloatNode(0, b) );
    //push ( c );
    push ( b );
    break;

  case Bytecodes::_l2f:
    pop(); 
    b = _gvn.transform( new (2) ConvL2FNode( pop() ) );
    c = precision_rounding(b);
    push(c);
    break;

  case Bytecodes::_l2d:
    pop(); 
    b = _gvn.transform( new (2) ConvL2DNode( pop() ) );
    c = dprecision_rounding(b);
    push(c);
    push ( _top );
    break;

  case Bytecodes::_f2l:
    b = _gvn.transform( new (2) ConvF2LNode( pop() ) );
    push(b);
    push ( _top );
    break;

  case Bytecodes::_d2l:
    pop(); 
    b = _gvn.transform( new (2) ConvD2LNode( pop() ) );
    push(b);
    push ( _top );
    break;

  case Bytecodes::_dsub:
    pop(); b = pop();  
    pop(); a = pop();  
    c = _gvn.transform( new (3) SubDNode(a,b) );
    d = dprecision_rounding(c);
    push( d );
    push( _top );
    break;

  case Bytecodes::_dadd:
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) AddDNode(a,b) );
    d = dprecision_rounding(c);
    push( d );
    push( _top );
    break;

  case Bytecodes::_dmul:
    pop(); b = pop();  
    pop(); a = pop();  
    c = _gvn.transform( new (3) MulDNode(a,b) );
    d = dprecision_rounding(c);
    push( d );
    push( _top );
    break;

  case Bytecodes::_ddiv:
    pop(); b = pop();
    pop(); a = pop();  
    c = _gvn.transform( new (3) DivDNode(0,a,b) );
    d = dprecision_rounding(c);
    push( d );
    push( _top );
    break;

  case Bytecodes::_dneg: 
    pop(); a = pop();
    b = _gvn.transform(new (2) NegDNode (a));
    push(b);
    push( _top );
    break;

  case Bytecodes::_drem:
    // %%%% EXPIRATION 9/07/98 move this transformation downstream
    if (Matcher::modDSupported()) {
      // Generate a ModD node.
      pop(); b = pop();
      pop(); a = pop();  
      // a % b
    
      c = _gvn.transform( new (3) ModDNode(0,a,b) );
      d = dprecision_rounding(c);
      push( d );
      push( _top ); 
    }
    else {
      // Generate a call.
      modd();
    }
    break;
    
  case Bytecodes::_dcmpl:
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) CmpDLessNode( a, b)); 
    push(c);
    break;

  case Bytecodes::_dcmpg:
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) CmpDGreaterNode( a, b)); 
    push(c);
    break;

   
    // Note for longs -> lo word is on TOS, hi word is on TOS - 1
  case Bytecodes::_land:
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) AndLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lor:
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) OrLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lxor:
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) XorLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lshl:
    b = pop();			// the shift count
    pop(); a = pop();		// value to be shifted
    c = _gvn.transform( new (3) LShiftLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lshr:
    b = pop();			// the shift count
    pop(); a = pop();		// value to be shifted
    c = _gvn.transform( new (3) RShiftLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lushr: 
    b = pop();			// the shift count
    pop(); a = pop();		// value to be shifted
    c = _gvn.transform( new (3) URShiftLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lmul: 
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) MulLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lrem: 
    // Must keep both values on the expression-stack during null-check
    do_null_check(peek(1), iter.cur_bci(), T_LONG); 
    // Compile-time detect of null-exception?
    if( !_map ) return;
    pop(); b = pop();
    pop(); a = pop();
    push( _gvn.transform( new (3) ModLNode(control(),a,b) ) );
    push( _top );
    break;
  case Bytecodes::_ldiv: 
    // Must keep both values on the expression-stack during null-check
    do_null_check(peek(1), iter.cur_bci(), T_LONG); 
    // Compile-time detect of null-exception?
    if( !_map ) return;
    pop(); b = pop();
    pop(); a = pop();
    push( _gvn.transform( new (3) DivLNode(control(),a,b) ) );
    push( _top );
    break;

  case Bytecodes::_ladd: 
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) AddLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lsub: 
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) SubLNode(a,b) );
    push( c );
    push( _top );
    break;
  case Bytecodes::_lcmp: 
    pop(); b = pop();
    pop(); a = pop();
    c = _gvn.transform( new (3) CmpLNode( a, b )); 
    push(c);
    break;

  case Bytecodes::_lneg: 
    pop(); a = pop();
    push( _gvn.transform( new (3) SubLNode(_gvn.makecon(TypeLong::ZERO),a)) );
    push ( _top );
    break;
  case Bytecodes::_l2i:
    pop(); a = pop();
    push( _gvn.transform( new (2) ConvL2INode( a ) ) );
    break;
  case Bytecodes::_i2l:
    b = _gvn.transform( new (2) ConvI2LNode( pop() ) );
    push(b);
    push ( _top );
    break;
  case Bytecodes::_i2b:
    // Sign extend
    a = pop();
    a = _gvn.transform( new (3) LShiftINode(a,_gvn.intcon(24)) );
    a = _gvn.transform( new (3) RShiftINode(a,_gvn.intcon(24)) );
    push( a );
    break;
  case Bytecodes::_i2s:
    a = pop();
    a = _gvn.transform( new (3) LShiftINode(a,_gvn.intcon(16)) );
    a = _gvn.transform( new (3) RShiftINode(a,_gvn.intcon(16)) );
    push( a );
    break;
  case Bytecodes::_i2c:
    a = pop();
    push( _gvn.transform( new (3) AndINode(a,_gvn.intcon(0xFFFF)) ) );
    break;

  case Bytecodes::_i2f:
    a = pop();
    b = _gvn.transform( new (2) ConvI2FNode(a) ) ;
    c = precision_rounding(b);
    push (b);
    break;

  case Bytecodes::_i2d:
    a = pop();
    push( _gvn.transform( new (2) ConvI2DNode(a) ) );
    push ( _top );
    break;

  case Bytecodes::_iinc:        // Increment local
    i = iter.get_index();       // Get local index
    set_local( i, _gvn.transform( new (3) AddINode( _gvn.intcon(iter.get_iinc_con()), local(i) ) ) );
    break;

  // Exit points of synchronized methods must have an unlock node
  case Bytecodes::_return:
    return_current( NULL, iter.cur_bci() );
    break;

  case Bytecodes::_dreturn:
  case Bytecodes::_lreturn:
    pop();                      // Drop other half of double
                                // Fall into regular case
  case Bytecodes::_ireturn:
  case Bytecodes::_areturn:
  case Bytecodes::_freturn:
    return_current( pop(), iter.cur_bci() );
    break;

  case Bytecodes::_athrow:
    // null exception oop throws NULL pointer exception
    do_null_check(peek(),iter.cur_bci(), T_OBJECT); 
    if( !_map ) return;
    do_athrow(Deoptimization::Deopt_athrow, iter.cur_bci());
    break;

  case Bytecodes::_goto:   // fall through
  case Bytecodes::_goto_w: {
    int target_bci = (bc == Bytecodes::_goto) ? iter.get_dest() : iter.get_far_dest();
    if( is_jsr_stack_empty() || _preds[target_bci] > 1 ) {
      // If this is a backwards branch in the bytecodes, add Safepoint
      if( UseLoopSafepoints && target_bci <= iter.cur_bci() ) 
        add_safepoint(iter);
      // Merge the current control into the target basic block
      merge(target_bci, cloning);
    } else {
      // Check for bad structuring of _jsr and _ret
      if ( iter.cur_bci() < target_bci ) {
        _bailout = Bailout_shared_handler;
        return;
      }
      // No other predecessor, continue inlining code between _jsr and _ret
      // Reset iterator to parse at the destination
      iter.reset_to_bci(target_bci);
    }
    break;
  }

  case Bytecodes::_ifnull:    btest = BoolTest::eq; goto handle_ifyy;
  case Bytecodes::_ifnonnull: btest = BoolTest::ne; goto handle_ifyy;
  { handle_ifyy: 
    // If this is a backwards branch in the bytecodes, add Safepoint
    if( UseLoopSafepoints && iter.get_dest() <= iter.cur_bci() )
      add_safepoint(iter);
    explicit_null_checks_inserted++;
    a = _null;
    b = pop();  
    c = _gvn.transform( new (3) CmpPNode( b, a ) );
    // Make a cast-away-nullness that is control dependent on the test
    const Type *t = _gvn.type(b);
    const Type *t_not_null = t->join(TypePtr::NOTNULL);
    Node *cast = new (2) CastPPNode(b,t_not_null);

    // Generate real control flow
    Node   *tst = _gvn.transform( new (2) BoolNode( c, btest ) );
    float cnt;
    float prob = branch_prediction( (int)iter.cur_bcp(), cloning, cnt, _ilt->_site_invoke_ratio );

    // If prob is -1.0f, switch to static prediction
    if( prob == -1.0f )
      prob =  (btest == BoolTest::eq) ? 0.1f : 0.9f;

    // Sanity check the probability value
    assert(prob > 0.0f,"Bad probability in Parser");
   // Need xform to put node in hash table
    IfNode *iff = create_and_xform_if( control(), tst, prob, cnt );
    assert(iff->_prob > 0.0f,"Optimizer made bad probability in parser");
    // True branch
    { PreserveJVMState pjvms(this, iter.next_bci());
      Node *iftrue  = _gvn.transform( new (1) IfTrueNode (iff) );
      set_control( iftrue );
      if( iftrue == _top ) {    // Path is dead?
        explicit_null_checks_elided++;
      } else {                  // Else path is live
        Node *checked_value;
        if( btest == BoolTest::eq ) {
          checked_value = _null;
        } else {
          cast->set_req(0,control());
          checked_value = _gvn.transform(cast);
        }
        // Make instances of value not-null on this path
        for( uint i=0; i < _map->req(); i++ ) 
          if( _map->in(i) == b ) 
            _map->set_req(i,checked_value);
      }
      merge( iter.get_dest(), cloning );
      // Believe it or not, but it's possible for b's type to have
      // improved by the merge() call, which can improve c's type
      // to the point that the test goes dead.  Bug 4428483.
      Node *c_prime = _gvn.transform(c);
      if( c_prime != c ) {
        c = c_prime;
        tst->set_req(1,c);
        tst = _gvn.transform(tst);
        iff->set_req(1,tst);
        _gvn.transform(iff);
      }
    }

    // False branch
    Node *iffalse = _gvn.transform( new (1) IfFalseNode(iff) );
    set_control( iffalse );
    if( iffalse == _top ) {     // Path is dead?
      explicit_null_checks_elided++;
    } else {                    // Path is live?
      if( btest == BoolTest::eq ) {
        cast->set_req(0,control());
        cast = _gvn.transform(cast);
      } else {
        cast = _null;
      }
      // Make instances of value not-null on this path
      for( i=0; i < (int)_map->req(); i++ ) 
        if( _map->in(i) == b ) 
          _map->set_req(i,cast);
    }
    // If I am cloning, stop it!  Following code becomes a new block.
    if( cloning ) {                    // Cloning bytecodes
      assert( iter.get_dest() < iter.next_bci(), "made irreducible" );
      bits_merge_set(iter.next_bci()); // Flag as next point being a merge
    }
    break;
  }

  case Bytecodes::_if_acmpeq: btest = BoolTest::eq; goto handle_if_acmp;
  case Bytecodes::_if_acmpne: btest = BoolTest::ne; goto handle_if_acmp;
  handle_if_acmp:
    // If this is a backwards branch in the bytecodes, add Safepoint
    if( UseLoopSafepoints && iter.get_dest() <= iter.cur_bci() ) 
      add_safepoint(iter);
    a = pop(); 
    b = pop();  
    c = _gvn.transform( new (3) CmpPNode( b, a ) );
    goto handle_if;

  case Bytecodes::_ifeq: btest = BoolTest::eq; goto handle_ifxx;
  case Bytecodes::_ifne: btest = BoolTest::ne; goto handle_ifxx;
  case Bytecodes::_iflt: btest = BoolTest::lt; goto handle_ifxx;
  case Bytecodes::_ifle: btest = BoolTest::le; goto handle_ifxx;
  case Bytecodes::_ifgt: btest = BoolTest::gt; goto handle_ifxx;
  case Bytecodes::_ifge: btest = BoolTest::ge; goto handle_ifxx;
  handle_ifxx:
    // If this is a backwards branch in the bytecodes, add Safepoint
    if( UseLoopSafepoints && iter.get_dest() <= iter.cur_bci() ) 
      add_safepoint(iter);
    a = _gvn.intcon(0);
    b = pop();  
    c = _gvn.transform( new (3) CmpINode( b, a ) );
    goto handle_if;

  case Bytecodes::_if_icmpeq: btest = BoolTest::eq; goto handle_if_icmp;
  case Bytecodes::_if_icmpne: btest = BoolTest::ne; goto handle_if_icmp;
  case Bytecodes::_if_icmplt: btest = BoolTest::lt; goto handle_if_icmp;
  case Bytecodes::_if_icmple: btest = BoolTest::le; goto handle_if_icmp;
  case Bytecodes::_if_icmpgt: btest = BoolTest::gt; goto handle_if_icmp;
  case Bytecodes::_if_icmpge: btest = BoolTest::ge; goto handle_if_icmp;
  handle_if_icmp: 
    // If this is a backwards branch in the bytecodes, add Safepoint
    if( UseLoopSafepoints && iter.get_dest() <= iter.cur_bci() ) 
      add_safepoint(iter);
    a = pop(); 
    b = pop();  
    c = _gvn.transform( new (3) CmpINode( b, a ) );
    goto handle_if;

  handle_if: {
    float cnt;
    float prob = branch_prediction( (int)iter.cur_bcp(), cloning, cnt, _ilt->_site_invoke_ratio );

    // If prob is -1.0f, switch to static prediction
    if( prob == -1.0f ) {
      prob = 0.5f;                // Set defualt value
      if( btest == BoolTest::eq ) // Exactly equal test?
	prob = 0.1f;              // Assume its relatively infrequent
      else if( btest == BoolTest::ne )
	prob = 0.9f;              // Assume its relatively frequent

      // If this is a backwards branch in the bytecodes, add Safepoint
      if( UseLoopSafepoints && !cloning &&
	  iter.get_dest() < iter.cur_bci() ) {
	// This is a conditional test guarding a backwards branch.
	// Assume its a loop-back edge.  Make it a likely taken branch.
	// Also, use '0.901' vs '0.9' so a later heuristic can find 
	// textually-backward-branches from forward branches that lead to
	// unconditional backward branches.  THIS test is likely to be a
	// real nested-loop backedge.  The other case (forward branch to
	// unconditional back branch) occurs with switch-tables in tight 
	// loops.  In _213_javac, xscan function has this problem in spades.
	prob = 0.901f;            // Likely to take backwards branch
      }
    }

    // Sanity check the probability value
    assert(prob > 0.0f,"Bad probability in Parser");
    // Generate real control flow
    Node *tst = _gvn.transform( new (2) BoolNode( c, btest ) );
    IfNode *iff = create_and_map_if( control(), tst, prob, cnt );
    assert(iff->_prob > 0.0f,"Optimizer made bad probability in parser");
    // True branch
    { PreserveJVMState pjvms(this, iter.next_bci());
      Node *iftrue  = _gvn.transform( new (1) IfTrueNode (iff) );
      set_control( iftrue );
      // Check for equal-to-constant case, and "know" that the compared
      // value is a constant on the taken branch.
      if( btest == BoolTest::eq && c->is_Cmp()) {
        Node *con = c->in(1);
        Node *xxx = c->in(2);
        if( con->is_Con() ) {
        } else if( xxx->is_Con() ) {
          con = c->in(2);
          xxx = c->in(1);
        } else {
          con = xxx = NULL;
        }
        for( uint i = 1; i < _map->req(); i++ )
          if( _map->in(i) == xxx )
            _map->set_req(i,con);
      }
      merge( iter.get_dest(), cloning );
    }

    // False branch
    Node *iffalse = _gvn.transform( new (1) IfFalseNode(iff) );
    set_control( iffalse );
    // Check for not-equal-to-constant case, and "know" that the compared
    // value is a constant on the not-taken branch.
    if( btest == BoolTest::ne && c->is_Cmp()) {
      Node *con = c->in(1);
      Node *xxx = c->in(2);
      if( con->is_Con() ) {
      } else if( xxx->is_Con() ) {
        con = c->in(2);
        xxx = c->in(1);
      } else {
        con = xxx = NULL;
      }
      for( uint i = 1; i < _map->req(); i++ )
        if( _map->in(i) == xxx )
          _map->set_req(i,con);
    }
    
    // If I am cloning, stop it!  Following code becomes a new block.
    if( cloning ) {                    // Cloning bytecodes
      assert( iter.get_dest() < iter.next_bci(), "made irreducible" );
      bits_merge_set(iter.next_bci()); // Flag as next point being a merge
    }
    break;
  }

  case Bytecodes::_tableswitch: {
    if( UseLoopSafepoints && !cloning ) 
      add_safepoint(iter);	// Just always insert
    a = pop();			// table index
    // Get information about tableswitch
    int default_dest = iter.get_dest_table(0);
    int lo_index     = iter.get_int_table(1);
    Node *lo         = _gvn.intcon(lo_index);
    int hi_index     = iter.get_int_table(2);
    int len          = hi_index - lo_index + 1;

    if (SwitchDecisionTrees && len > 1) {
      // generate decision tree, using trichotomy when possible
      Node *lookup = a;
      int rnum = len+2;
      SwitchRange* ranges = NEW_RESOURCE_ARRAY(SwitchRange, rnum);
      int rp = -1;
      if( lo_index-1 != min_jint ) {
	ranges[++rp].setRange(min_jint, lo_index-1, default_dest);
      }
      for( int j = 0; j < len; j++ ) {
	jint match_int = lo_index+j;
	int  dest      = iter.get_dest_table(j+3);
	if( rp < 0 || !ranges[rp].adjoin(match_int, dest) ) {
	  ranges[++rp].set(match_int, dest);
	}
	//_preds[dest] -= 1;
      }
      //_preds[default_dest] -= 1; // two paths to default case?
      jint highest = lo_index+(len-1);
      assert(ranges[rp].hi() == highest, "");
      if( highest != max_jint 
	  && !ranges[rp].adjoinRange(highest+1, max_jint, default_dest) ) {
	ranges[++rp].setRange(highest+1, max_jint, default_dest);
      }
      assert(rp < len+2, "not too many ranges");
      assert(ranges[0 ].lo() == min_jint, "");
      assert(ranges[rp].hi() == max_jint, "");
      jump_switch_ranges( iter.cur_bci(), lookup, &ranges[0], &ranges[rp], cloning );
      break;
    }

    // Generate control flow for   0 <= (index - lo) <= (hi - lo)
    Node *val        = _gvn.transform( new (3) SubINode(a,lo) );
    int j;
    for( j = 0; j < len; j++ ) {
      // Check for (index - lo) == j
      int      dest     = iter.get_dest_table(j+3);
      Node    *test_val = _gvn.intcon(j);
      IfNode  *iff_ne   = jump_if_fork_int(val, test_val, BoolTest::ne );
      jump_if_false_fork( iff_ne, dest, cloning );
    }
    // Generate control flow for last case, guaranteed to be true,
    // Control dependent on previous iffalseNode
    jump_if_always_fork( default_dest, cloning );
    break;
  }

  case Bytecodes::_fast_binaryswitch:
  case Bytecodes::_fast_linearswitch:
  case Bytecodes::_lookupswitch: {
    if( UseLoopSafepoints && !cloning ) 
      add_safepoint(iter);	// Just always insert
    Node *lookup = pop();	// lookup value
    // Get information about lookupswitch
    int default_dest = iter.get_dest_table(0);
    int len          = iter.get_int_table(1);

    if (SwitchDecisionTrees && len > 1) {
      // generate decision tree, using trichotomy when possible
      jint* table = NEW_RESOURCE_ARRAY(jint, len*2);
      {
	for( int j = 0; j < len; j++ ) {
	  table[j+j+0] = iter.get_int_table(2+j+j);
	  table[j+j+1] = iter.get_dest_table(2+j+j+1);
	}
	qsort( table, len, 2*sizeof(table[0]), jint_cmp );
      }

      int rnum = len*2+1;
      SwitchRange* ranges = NEW_RESOURCE_ARRAY(SwitchRange, rnum);
      int rp = -1;
      for( int j = 0; j < len; j++ ) {
	jint match_int = table[j+j+0];
	int  dest      = table[j+j+1];
	int next_lo    = rp < 0 ? min_jint : ranges[rp].hi()+1;
	if( match_int != next_lo ) {
	  ranges[++rp].setRange(next_lo, match_int-1, default_dest);
	}
	if( rp < 0 || !ranges[rp].adjoin(match_int, dest) ) {
	  ranges[++rp].set(match_int, dest);
	}
	//_preds[dest] -= 1;
      }
      //_preds[default_dest] -= 1; // one path to default case
      jint highest = table[2*(len-1)];
      assert(ranges[rp].hi() == highest, "");
      if( highest != max_jint 
	  && !ranges[rp].adjoinRange(highest+1, max_jint, default_dest) ) {
	ranges[++rp].setRange(highest+1, max_jint, default_dest);
      }
      assert(rp < rnum, "not too many ranges");
      assert(ranges[0 ].lo() == min_jint, "");
      assert(ranges[rp].hi() == max_jint, "");
      jump_switch_ranges( iter.cur_bci(), lookup, &ranges[0], &ranges[rp], cloning );
      break;
    }

    // Generate control flow for matches
    int j;
    for( j = 0; j < len; j++ ) {
      // Check for  (lookup_value  == match_val)
      int     match_int = iter.get_int_table(2+j+j);
      Node   *match  = _gvn.intcon(match_int);
      int     dest   = iter.get_dest_table(2+j+j+1);
      IfNode *iff_ne = jump_if_fork_int(lookup, match, BoolTest::ne);
      jump_if_false_fork( iff_ne, dest, cloning );
    }

    // Generate control flow for unmatched case, default_dest
    // Control dependent on Parse::control(), the previous iffalseNode
    jump_if_always_fork( default_dest, cloning );
    break;
  }

  case Bytecodes::_invokestatic:
  case Bytecodes::_invokespecial:
  case Bytecodes::_invokevirtual: 
  case Bytecodes::_invokeinterface:
    do_call( &iter, bc, cloning );
    break;
  case Bytecodes::_checkcast:
    do_checkcast(&iter);
    break;
  case Bytecodes::_instanceof:
    do_instanceof(&iter);
    break;
  case Bytecodes::_anewarray: 
    do_anewarray(&iter);
    break;
  case Bytecodes::_newarray:
    do_newarray((BasicType)iter.get_index(), iter.cur_bci());
    break;
  case Bytecodes::_multianewarray:
    do_multianewarray(&iter);
    break;
  case Bytecodes::_new:
    do_new(&iter);
    break;

  case Bytecodes::_jsr:
  case Bytecodes::_jsr_w:
    do_jsr(iter);
    break;
      
  case Bytecodes::_ret:
    do_ret(iter);
    break;


  case Bytecodes::_monitorenter:
    do_monitor_enter(iter.cur_bci());
    break;

  case Bytecodes::_monitorexit:
    do_monitor_exit(iter.cur_bci());
    break;

  default:
#ifndef PRODUCT
    _map->dump(99);
#endif
    tty->print("\nUnhandled bytecode %s\n", Bytecodes::name(bc) );
    ShouldNotReachHere();
  }

  if( _map && control() == _top ) // We became unreachable somehow?
    _map = NULL;                // Clear the Parser map
  
  // Fall into next bytecode.  Each bytecode normally has 1 sequential
  // successor which is typically made ready by visiting this bytecode.
  // If the successor has several predecessors, then it is a merge
  // point, starts a new basic block, and is handled like other basic blocks.  
  uint nextbci = iter.next_bci();
  if( _map &&                   // If control falls into next bytecode
      bits_merge(nextbci) ) 
    merge( nextbci, cloning );
}
