/*
 * @(#)parse3.cpp	1.212 01/02/09 14:46:55
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_parse3.cpp.incl"

void Parse::kill_dead_locals( int bci ) {
  if (_method == NULL) {
    // We are building a graph for a call to a native method.
    // All locals are live.
    return;
  }

  ResourceMark rm;

  // Consult the liveness information for the locals.  If any
  // of them are unused, then they can be replaced by _top.  This
  // should help register allocation time and cut down on the size
  // of the deoptimization information.
  BitMap liveLocals = _method->liveness_at_bci(bci);
  
  // This could perhaps be made more zippy.
  int first = (bci == SynchronizationEntryBCI && C->is_set_jvmpi_method_event()) ? 1 : 0;
  int len = liveLocals.size();
  for (int local = first; local < len; local++) {
    if (liveLocals.at(local) == 0) {
      set_local(local, _top);
    }
  }
}

// Helper function for adding debug information to node
void Parse::add_safepoint_edges(SafePointNode *sfpnt, int bci) {
  // Add the safepoint edges to the sfpnt
  JVMStateObj *jhead  = _jhead;
  JVMStateObj *jtail  = _jtail;
  _map->_jvms->set_bci(bci);

  // Consult the liveness information for the locals.  If any
  // of them are unused, then they can be replaced by _top.  This
  // should help register allocation time and cut down on the size
  // of the deoptimization information.
  ResourceMark rm;

  // bci can be -1 (SynchronizationEntryBCI).  We return the entry
  // liveness for the method.
  kill_dead_locals((bci == SynchronizationEntryBCI)? 0 : bci);
  
  // Place this on inline list
  if (jtail != NULL) {
    jtail->set_next(_map->_jvms); // Add to tail of list
  } else {
    jhead = jtail = _map->_jvms;  // Setup tail of list
  }

  _map->_jvms->set_sp(_sp);     // Record the new sp in the JVMState Object

  // Walk the inline list to fill in the correct set of JVMStateObj's
  // Also fill in the associated edges for each JVMStateObj.
  add_debug_info(sfpnt,jhead);
  // Finally, unhook current jvms from list
  if (_jtail != NULL)
    _jtail->set_next( NULL );          // Reset end of list to _jtail
}


//=============================================================================
// helper methods for unloaded and uninitialized class uncommon_trap
bool Parse::do_unloaded( ciByteCodeStream* stream, ciKlass* klass ) {
  int bci = stream->cur_bci();
  if( bits_type(bci) == 0 ) {
    uncommon_trap(stream->get_klass_index(), bci);
    return true;                // Generated uncommon_trap for unloaded class
  }
  assert( klass->is_loaded(), "" );
  return false;			// Do not throw uncommon trap, class is loaded
}

bool Parse::do_unloaded_or_uninitialized(ciByteCodeStream* stream, ciKlass* klass) {
  int bci = stream->cur_bci();
  if( bits_type(bci) == 0 ) {
    if( !klass->is_loaded() ) 
      uncommon_trap( stream->get_klass_index(), bci );
    else 
      uncommon_trap( Deoptimization::Deopt_uninitialized, bci );
    return true;                // Generated uncommon_trap for unloaded class
  }
  assert( klass->is_loaded(), "" );
  assert( !klass->is_instance_klass() ||
	  klass->as_instance_klass()->is_initialized(), "" );
  return false;			// Do not throw uncommon trap, class is loaded
}


//=============================================================================
// Helper methods for _get* and _put* bytecodes
//=============================================================================
void Parse::do_get_xxx(ciByteCodeStream* stream, bool is_field) {
  int cur_bci = stream->cur_bci();

  if( method()->is_not_reached(cur_bci) && 
      CompileThreshold > 100 && !CompileTheWorld ) {
    uncommon_trap(Deoptimization::Deopt_uninitialized, cur_bci);
    return;
  }

  ciField *field = stream->get_field();
  ciInstanceKlass *field_holder = field->holder();
  // Is the field holder loaded?
  if( bits_type(cur_bci) == 0 ) {
    if( !field_holder->is_loaded() || is_field )
      uncommon_trap( stream->get_field_holder_index(), cur_bci );
    else
      uncommon_trap(Deoptimization::Deopt_uninitialized, cur_bci);
    return;
  }
  assert(field_holder->is_instance_klass(),
	 "only instance klasses have fields");
  assert( field_holder->is_loaded(), "" );
  assert( is_field || field_holder->as_instance_klass()->is_initialized(), "");

  // Has the field's type been loaded?  Works for arrays as well.
  BasicType field_type = field->field_type();
  ciKlass* field_klass = NULL;
  if (field_type == T_OBJECT || field_type == T_ARRAY) {
    field_klass = field->field_klass();
    if (!field_klass->is_loaded()) {
      uncommon_trap(stream->get_field_signature_index(), cur_bci);
      return;
    }
  }
  const TypeKlassPtr *t_klass = NULL;
  // Generate code for the object pointer.
  Node *obj = NULL;
  if (is_field) {
    obj = do_null_check(stack(_sp-1), cur_bci, T_OBJECT);
    _sp--;

    // Compile-time detect of null-exception?
    if( !_map ) return;
  } else {
    // Does this field have a constant value?  If so, just push the value.
    if (field->is_constant()) {
      ciConstant constant = field->constant_value();
      switch(constant.type()) {
      case T_BOOLEAN:
      case T_INT:    
        push( _gvn.intcon(constant.as_int()) );
        return;
      case T_CHAR:
        push( _gvn.intcon(constant.as_char()) );
        return;
      case T_BYTE:
        push( _gvn.intcon(constant.as_byte()) );
        return;
      case T_SHORT:
        push( _gvn.intcon(constant.as_short()) );
        return;
      case T_FLOAT:
        push( _gvn.makecon(TypeF::make(constant.as_float())) );
        return;
      case T_DOUBLE:
        push( _gvn.makecon(TypeD::make(constant.as_double())) );
        push( _top );
        return;
      case T_LONG:
        push( _gvn.makecon(TypeLong::make(constant.as_long())) );
        push( _top );
        return;
      case T_ARRAY:
      case T_OBJECT: {
        // CI_Constant::is_constant() makes sure that the oop is in perm space.
        ciObject* oop_constant = constant.as_object();
        if (oop_constant->is_null_object()) {
          push( _null );
        } else {
          push( _gvn.makecon(TypeJavaPtr::make_from_constant(oop_constant)) );
        }
        return;
      }
      default:
        ShouldNotReachHere();
      }
    }

    // The value was not a constant.  We will load from the klass.
    t_klass = TypeKlassPtr::make(field_holder);
    obj = _gvn.makecon(t_klass);
  }

  int offset = field->offset()*wordSize;

  // !!!!! !!!!!
  // Danger, Will Robison, Danger!

  // The putfield or getfield here basically asserts that the incoming
  // pointer is of the correct type (the verified told us so).  The parser
  // has more limited knowledge than the verify (it does not do a full flow
  // analysis) so it must use pessimistic types at Phis.  This code "lifts"
  // the Phis pessimistic types to be at least as high as the getfield or
  // putfield requires.  This lifting is nessecary so alias analysis in the
  // optimizer works.  Unlike the parser, the optimizer must use the
  // incoming address types (instead of querying the constant pool), and if
  // those addresses are too pessimistic the optimizer will use the wrong
  // memory edges for aliasing.  So we "lift" the address here.
  uint alias_idx;
  uint alias_idx2 = 0;

  // Compute alias index and memory edge
  Node *mem2 = NULL;
  if( is_field ) {
    const TypeJavaPtr* field_holder_type =
      TypeOopPtr::make(TypePtr::NotNull, stream->get_declared_field_holder());
    if (_gvn.type(obj) != field_holder_type ) 
      obj = cast_up( obj, field_holder_type );
    // Prevent any volatile memory ops from commoning up
    if( field->is_volatile() ) 
      insert_mem_bar();
    alias_idx =C->get_alias_index(field_holder_type->add_offset(offset));
    if (bits_type(cur_bci) == T_LONG) {
      alias_idx2 = C->get_alias_index( field_holder_type->add_offset( offset + wordSize) );
      mem2 = memory(alias_idx2);
    }
  } else {
    // Prevent any volatile memory ops from commoning up
    if( field->is_volatile() ) 
      insert_mem_bar();
    alias_idx = C->get_alias_index( t_klass->add_offset( offset ) );
    if (bits_type(cur_bci) == T_LONG) {
      alias_idx2 = C->get_alias_index( t_klass->add_offset( offset + wordSize) );
      mem2 = memory(alias_idx2);
    }
  }
  Node *mem = memory(alias_idx);

  // Build address expression
  Node *adr = basic_plus_adr(obj, obj, offset);

  // Build the load
  Node *l;
  switch( bits_type(cur_bci) ) {
  case T_BOOLEAN:
  case T_CHAR:
  case T_BYTE:
  case T_SHORT:
  case T_INT:    l = new (3) LoadINode(NULL, mem, adr); break;
  case T_FLOAT:  l = new (3) LoadFNode(NULL, mem, adr); break;
  case T_DOUBLE: l = new (3) LoadDNode(NULL, mem, adr); push( _gvn.transform( l ) ); l = _top;  break;
  case T_LONG  : l = new (3) LoadLNode(NULL, mem, adr); push( _gvn.transform( l ) ); l = _top;  break;
  case T_ARRAY:
  case T_OBJECT: {
    // The field's klass may be an abstract class with a unique
    // concrete subklass.  In this case, we can note a class hierarchy
    // dependence and let the field assume the concrete type.
    if (field_klass->is_instance_klass()) {
      ciInstanceKlass* instance_field_klass = field_klass->as_instance_klass();
      ciKlass* concrete_field_klass = instance_field_klass->unique_concrete_subklass();
      if (concrete_field_klass != NULL) {
	C->_recorder->add_dependent(instance_field_klass, NULL);
	field_klass = concrete_field_klass;
      }
    }
    const TypeJavaPtr *type = TypeJavaPtr::make_from_klass(field_klass);
    l = new (3) LoadPNode(NULL, mem, adr, type);
    break;
  }
  default: ShouldNotReachHere();
  }
  // push result of load onto stack
  push( _gvn.transform( l ) );
} // end _get_xxx


void Parse::do_putstatic(ciByteCodeStream* stream) {
  int cur_bci = stream->cur_bci();
  if( method()->is_not_reached(cur_bci) && 
      CompileThreshold > 100 && !CompileTheWorld ) {
    uncommon_trap(Deoptimization::Deopt_uninitialized, cur_bci);
    return;
  }

  ciField *field = stream->get_field();

  // Is the field holder loaded?
  ciInstanceKlass *field_holder = field->holder();

  // Is the field holder loaded?
  if( bits_type(cur_bci) == 0 ) {
    if( !field_holder->is_loaded() ) 
      uncommon_trap( stream->get_field_holder_index(), cur_bci );
    else
      uncommon_trap(Deoptimization::Deopt_uninitialized, cur_bci);
    return;
  }
  assert(field_holder->is_instance_klass(),
	 "only instance klasses have fields");
  assert( field_holder->is_loaded(), "" );
  assert( field_holder->as_instance_klass()->is_initialized(), "" );

  // We can only get fields from an instance klass
  assert(field_holder->is_instance_klass(),
	 "field holder must be instance klass");
  guarantee(field->is_static(),
	    "should throw java_lang_IncompatibleClassChangeError");  

  int   con = field->offset()*wordSize;
  Node *obj = _gvn.makecon( TypeKlassPtr::make(field_holder) );
  // Prevent any volatile memory ops from commoning up
  if( field->is_volatile() )
    insert_mem_bar();
  // Compute alias index and memory edge
  Node *adr = basic_plus_adr(obj, obj, con);
  Node *l;
  Node *val1 = pop();
  Node *val2 = (_args[cur_bci] == 2) ? pop() : NULL;

  uint alias_idx = C->get_alias_index( TypeKlassPtr::make(field_holder, con) );
  Node *mem = memory(alias_idx);

  switch( bits_type(cur_bci) ) {
  case T_BOOLEAN:
  case T_INT:  
  case T_SHORT: 
  case T_BYTE: 
  case T_CHAR:
    l = new (4) StoreINode(control(), mem, adr, val1, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break;
  case T_ARRAY: 
  case T_OBJECT: 
    l = new (4) StorePNode(control(), mem, adr, val1, Type::MEMORY );           
    set_memory( _gvn.transform( l ), alias_idx ); // Set memory for store barrier                                                                               
    store_barrier(obj, val1);                                                   
    break;
  case T_LONG: 
    l = new (4) StoreLNode(control(), mem, adr, val2, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break;
  case T_FLOAT:
    l = new (4) StoreFNode(control(), mem, adr, val1, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break;
  case T_DOUBLE:
    val2 = dstore_rounding(val2);
    l = new (4) StoreDNode(control(), mem, adr, val2, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break;

  default: ShouldNotReachHere();
  }
  if( mem->req() > MemNode::Address && adr == mem->in(MemNode::Address) )
    _for_igvn.push( l );
} // end _putstatic


void Parse::do_putfield(ciByteCodeStream* stream) {
  int cur_bci = stream->cur_bci();
  if( method()->is_not_reached(cur_bci) &&
      CompileThreshold > 100 && !CompileTheWorld ) {
    uncommon_trap(Deoptimization::Deopt_uninitialized, cur_bci);
    return;
  }

  ciField *field = stream->get_field();
  ciInstanceKlass *field_holder = field->holder();

  // Is the field holder loaded?
  if( bits_type(cur_bci) == 0 ) {
    uncommon_trap( stream->get_field_holder_index(), cur_bci );
    return;
  }
  assert(field_holder->is_instance_klass(), 
	 "only instance klasses have fields");
  assert( field_holder->is_loaded(), "" );

  // Should we add code to detect field->klass() unloaded?
  
  Node *obj = do_null_check(stack(_sp-_args[cur_bci]), cur_bci, T_OBJECT);
  // Compile-time detect of null-exception?
  if (!_map) return;
  
  Node *val1 = pop();
  Node *val2 = (_args[cur_bci] == 3) ? pop() : NULL;
  _sp--;

  int offset = field->offset()*wordSize;

  // !!!!! !!!!!
  // Danger, Will Robison, Danger!
  
  // The putfield or getfield here basically asserts that the incoming
  // pointer is of the correct type (the verified told us so).  The parser
  // has more limited knowledge than the verify (it does not do a full flow
  // analysis) so it must use pessimistic types at Phis.  This code "lifts"
  // the Phis pessimistic types to be at least as high as the getfield or
  // putfield requires.  This lifting is nessecary so alias analysis in the
  // optimizer works.  Unlike the parser, the optimizer must use the
  // incoming address types (instead of querying the constant pool), and if
  // those addresses are too pessimistic the optimizer will use the wrong
  // memory edges for aliasing.  So we "lift" the address here.
  const TypeOopPtr *tjp = TypeOopPtr::make(TypePtr::NotNull,stream->get_declared_field_holder());
  if( _gvn.type(obj) != tjp ) {
    obj = cast_up( obj, tjp );
  }
  // Prevent any volatile memory ops from commoning up
  if( field->is_volatile() ) 
    insert_mem_bar();
  // Compute alias index and memory edge
  uint alias_idx = C->get_alias_index( tjp->add_offset( offset ) );
  Node *mem = memory(alias_idx);

  Node *adr = basic_plus_adr(obj, obj, offset);
  Node *l;
  switch( bits_type(cur_bci) ) {
  case T_BOOLEAN:
  case T_INT: 
  case T_CHAR:
  case T_SHORT:
  case T_BYTE:
    l = new (4) StoreINode(control(), mem, adr, val1, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break;
  case T_ARRAY:
  case T_OBJECT: 
    l = new (4) StorePNode(control(), mem, adr, val1, Type::MEMORY );
    set_memory( _gvn.transform( l ), alias_idx );
    store_barrier(obj, val1);   
    break;
  case T_LONG: 
    l = new (4) StoreLNode(control(), mem, adr, val2, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break; 
  case T_FLOAT:
    l = new (4) StoreFNode(control(), mem, adr, val1, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break; 
  case T_DOUBLE:
    val2 = dstore_rounding(val2);
    l = new (4) StoreDNode(control(), mem, adr, val2, Type::MEMORY);
    set_memory( _gvn.transform( l ), alias_idx );
    break; 
    
  default: ShouldNotReachHere();
  }
  // Back-to-back putfields can only remove intermediate store with DU info
  if( mem->req() > MemNode::Address && adr == mem->in(MemNode::Address) )
    _for_igvn.push( l );
}


//=============================================================================
void Parse::do_anewarray(ciByteCodeStream* stream) {
  int bci = stream->cur_bci();
  ciKlass* klass = stream->get_klass();

  // Uncommon Trap when class that array contains is not loaded
  // we need the loaded class for the rest of graph; do not
  // initialize the container class (see Java spec)!!!
  if( do_unloaded(stream, klass) ) return;
  
  ciObjArrayKlass* array_klass = ciObjArrayKlass::make(klass);
  // Check that array_klass object is loaded
  if( !array_klass->is_loaded() ) {
    // Generate uncommon_trap for unloaded array_class
    uncommon_trap(Deoptimization::Deopt_unloaded_array_class, bci);
    return;
  }
  
  const Type*         element_type     = TypeJavaPtr::make_from_klass(klass);  
  const TypeKlassPtr* array_klass_type = TypeKlassPtr::make(array_klass);
  Node *count_val = pop();
  do_Xnewarray(count_val, T_OBJECT, element_type, array_klass_type, bci);
}


void Parse::do_newarray(BasicType elem_type, int bci) {
  Node*   count_val = pop();
  const Type* etype = Type::get_const_basic_type(elem_type);
  // try to get a better type than POS for the size
  const TypeKlassPtr* array_klass = TypeKlassPtr::make(ciTypeArrayKlass::make(elem_type));
  do_Xnewarray(count_val, elem_type, etype, array_klass, bci);
}


void Parse::do_multianewarray(ciByteCodeStream* stream) {
  int cur_bci = stream->cur_bci();
  int ndimensions = stream->get_dimensions();

  // the m-dimensional array                      
  ciArrayKlass* array_klass = stream->get_klass()->as_array_klass();

  // Throw uncommon trap if class is not loaded or initialized
  if( do_unloaded_or_uninitialized(stream, array_klass) )
    return;

  // Can use _multianewarray instead of _anewarray or _newarray
  // if only one dimension
  if( ndimensions == 1 && array_klass->is_type_array_klass() ) {
    // If this is for a basic type, call code for do_newarray instead
    BasicType element_type = array_klass->as_type_array_klass()->element_type();
    do_newarray( element_type, cur_bci );
    return;
  }

  ciObjArrayKlass* obj_array_klass = array_klass->as_obj_array_klass();

  // find the element type (etype)
  ciKlass* element_klass = obj_array_klass->base_element_klass();
  // Base_element is either an instance-klass or a type-array but NOT
  // a basic type.  We really wanted the klass of a basic type; since that's
  // not available we have to test for type-array here.
  const Type* element_type = element_klass->is_type_array_klass()
    ? Type::get_const_basic_type(element_klass->as_type_array_klass()->element_type())
    : TypeOopPtr::make(TypePtr::BotPTR, element_klass->as_instance_klass());

  int mdimensions = obj_array_klass->dimension();

  // get the lengths from the stack (last dimension is on top)
  Node** length = NEW_RESOURCE_ARRAY(Node*, ndimensions);
  for (int j = 0; j < ndimensions; j++) length[j] = pop();

  // construct the array type
  const Type* prev_type = element_type;

  // fill the lowest dimensions with unknown sizes
  for (int index = 0; index < mdimensions - ndimensions; index++) {
    const TypeAry* arr0 = TypeAry::make(prev_type, TypeInt::POS);
    prev_type = TypeAryPtr::make( TypePtr::BotPTR, arr0, NULL, 0 );
  }

  // Fill in the dimensions with known sizes (passed in the JVM stack)
  for (int i = 0; i < ndimensions; i++) {
    Node* count_val = length[i];
    // try to get a better type than POS for the size
    const Type* count_type = count_val->bottom_type()->join(TypeInt::POS);
    assert(count_type->is_int(), "must be integer");
    const TypeAry* arr0 = TypeAry::make(prev_type, (TypeInt*)count_type);
    prev_type = TypeAryPtr::make( TypePtr::NotNull, arr0, NULL, 0 );
  }
  const TypeAryPtr* arr = (const TypeAryPtr*)prev_type;

  address fun = NULL;
  switch (ndimensions) {
   case 1: fun = OptoRuntime::multianewarray1_Java(); break;
   case 2: fun = OptoRuntime::multianewarray2_Java(); break;
   case 3: fun = OptoRuntime::multianewarray3_Java(); break;
   case 4: fun = OptoRuntime::multianewarray4_Java(); break;
   case 5: fun = OptoRuntime::multianewarray5_Java(); break;
   default: ShouldNotReachHere();
  };
  CallNode* call = new CallStaticJavaNode(OptoRuntime::multianewarray_Type( ndimensions), fun, OptoRuntime::stub_name(fun), cur_bci);
  set_predefined_input_for_runtime_call(call);
  call->set_req(TypeFunc::Parms+0, makecon(TypeKlassPtr::make(array_klass)));
  for (int k = 0; k < ndimensions; k++) {
    call->set_req(TypeFunc::Parms + (ndimensions - k), length[k]);
  }
  
  add_safepoint_edges(call, cur_bci);
  Node* c = _gvn.transform(call);
  set_predefined_output_for_runtime_call(c);
  Node *res = _gvn.transform(new (1) ProjNode(c, TypeFunc::Parms));
  Node *cast = _gvn.transform( new (2) CheckCastPPNode(control(), res, arr) );
  push( cast );
}


//------------------------------do_null_check----------------------------------
// Helper function to do a NULL pointer check.  Returned value is 
// the incoming address with NULL casted away.  You are allowed to use the
// not-null value only if you are control dependent on the test.
extern int explicit_null_checks_inserted, 
           explicit_null_checks_elided;
Node *Parse::do_null_check( Node *value, int bci, BasicType type ) {
  if( !GenerateCompilerNullChecks ) return value;
  explicit_null_checks_inserted++;

  // Construct NULL check
  Node *chk = NULL;
  switch(type) {
  case T_LONG: 
    value = _gvn.transform( new (3) CmpLNode( value, _gvn.makecon(TypeLong::ZERO)) );
    // fall into next case
    
  case T_INT   : chk = new (3) CmpINode( value, _gvn.intcon(0)); break;
  case T_OBJECT: // fall through
  case T_ARRAY : {
    const Type *t = _gvn.type( value );
    // See if mixing in the NULL pointer changes type.
    // If so, then the NULL pointer was not allowed in the original
    // type.  In other words, "value" was not-null.
    if( t->meet(TypePtr::NULL_PTR) != t ) {
      explicit_null_checks_elided++;  
      return value;           // Elided null check quickly!
    }
    chk = new (3) CmpPNode( value, _null);
    break;    
  }
  default      : ShouldNotReachHere();
  }
  assert(chk != NULL, "sanity check"); 
  chk = _gvn.transform(chk);

  BoolTest::mask btest = BoolTest::ne;
  BoolNode *btst = new (2) BoolNode( chk, btest);
  Node   *tst = _gvn.transform( btst );

  //-----------
  // if peephole optimizations occured, a prior test existed.
  // If a prior test existed, maybe it dominates as we can avoid this test.
  if( tst != btst && (type == T_ARRAY || type == T_OBJECT) ) {
    // At this point we want to scan up the CFG to see if we can
    // find an identical test (and so avoid this test altogether).
    Node *cfg = control();
    int depth = 0;
    while( depth < 16 ) {	// Limit search depth for speed
      if( !cfg->in(0) ) {
	cfg = cfg->nonnull_req();
	continue;
      }
      if( cfg->Opcode() == Op_IfTrue &&
	  cfg->in(0)->in(1) == tst ) {
	// Found prior test.  Use "cast_not_null" to construct an identical
	// CastPP (and hence hash to) as already exists for the prior test.
	// Return that casted value.
	Node *oldcontrol = control();
	set_control(cfg);
	Node *res = cast_not_null(value);
	set_control(oldcontrol);
        explicit_null_checks_elided++;  
	return res;
      }
      Node *next = cfg->in(0);	// Follow up the CFG chain
      if( next == cfg ) break;	// Quit at region nodes
      cfg = next;
      depth++;
    }
  }

  //-----------
  // Branch to failure if null
  IfNode *iff = create_and_map_if( control(), tst, (float)0.999999, -1.0F );
  Node *iffalse = _gvn.transform( new (1) IfFalseNode(iff) );

  // Done for the rare not-zero checks
  // Check for optimizer eliding test at parse time
  if( iffalse == _top ) {
    // Failure not possible; do not bother making uncommon trap.
    // Do not change the control().
    explicit_null_checks_elided++;
  } else {
    // Add exception exit path - just uncommon trap
    set_control(iffalse);
    { PreserveJVMState pjvms(this, bci);
    do_athrow( (type == T_ARRAY || type == T_OBJECT)
	       ? Deoptimization::Deopt_null_check
	       : Deoptimization::Deopt_div0_check, bci );
    }

    // Else fall through
    Node *iftrue = _gvn.transform( new (1) IfTrueNode(iff) );
    set_control(iftrue);
    // Must throw exception, fall-thru not possible?
    if( iftrue == _top ) {
      _map = NULL;                // Zap map
      return _top;                // No result
    }
  }

  // Cast obj to not-null on this path
  return (type == T_ARRAY || type == T_OBJECT) ? cast_not_null(value) : value;
}


//------------------------------cast_up----------------------------------------
// A putfield or getfield here basically asserts that the incoming
// pointer is of the correct type (the verified told us so).  The parser
// has more limited knowledge than the verify (it does not do a full flow
// analysis) so it must use pessimistic types at Phis.  This code "lifts"
// the Phis pessimistic types to be at least as high as the getfield or
// putfield requires.  This lifting is nessecary so alias analysis in the
// optimizer works.  Unlike the parser, the optimizer must use the
// incoming address types (instead of querying the constant pool), and if
// those addresses are too pessimistic the optimizer will use the wrong
// memory edges for aliasing.  So we "lift" the address here.
Node *Parse::cast_up( Node *adr, const TypePtr *tp ) {
  tp = _gvn.type(adr)->join(tp)->isa_javaptr();
  TypeNode *tn = adr->is_Type();
  if( tn ) {
    bool was_hashed = _gvn.hash_delete(adr); // Type changes so hash changes
    tn->_type = tp;             // Record better type
    if( was_hashed )            // Put it back as you found it
      _gvn.hash_find_insert( adr );
    // Type changed; prior children need to be revisited in IterGVN
    _for_igvn.push(adr);        
    
  }
  MemNode *mem = adr->is_Mem();
  if( mem ) {
    _gvn.hash_delete(adr);      // Type changes so hash changes
    mem->_type = tp;            // Record better type
    _gvn.hash_find_insert( adr );
    // Type changed; prior children need to be revisited in IterGVN
    _for_igvn.push(adr);    
  }
  // Just record better type in GVN
  _gvn._types.map( adr->_idx, tp );
  return adr;
}

//------------------------------cast_not_null----------------------------------
// Cast obj to not-null on this path
Node *Parse::cast_not_null( Node *obj ) {
  const Type *t = _gvn.type(obj);
  const Type *t_not_null = t->join(TypePtr::NOTNULL);
  // Object is already not-null?
  if( t == t_not_null ) return obj;

  Node *cast = new (2) CastPPNode(obj,t_not_null);
  cast->set_req(0,control());
  cast = _gvn.transform( cast );

  // Scan for instances of 'obj' in the current JVM mapping.
  // These instances are known to be not-null after the test.
  for( uint i=0; i < _map->req(); i++ ) 
    if( _map->in(i) == obj ) 
      _map->set_req(i,cast);

  return cast;			// Return casted value
}

//------------------------------athrow-----------------------------------------

void Parse::do_athrow(int klass_index, int bci) {
  debug_only(int sp = _sp;)

  // get exception node
  Node* ex_node = NULL;
  switch (klass_index) {
    case Deoptimization::Deopt_null_check:
    case Deoptimization::Deopt_div0_check:
      // Note: This is currently broken in the way that no new instance is
      //       created for each null pointer/arithmetic exception, and thus
      //       no stack trace is filled in. However, creating a new exception
      //       here is cause for some of the big slowdows in compilation due
      //       to the many runtime calls that need full debugging information
      //       and thus blow up register allocation time.
      //
      //       THIS NEEDS TO BE FIXED
      { ciInstance* ex_obj =
          klass_index == Deoptimization::Deopt_null_check
            ? env()->NullPointerException_instance()
            : env()->ArithmeticException_instance();
        const TypeOopPtr* t = TypeOopPtr::make(ex_obj);
        ex_node = _gvn.transform(new(1)ConPNode(t));
      }
      break;
    case Deoptimization::Deopt_athrow:
      ex_node = peek(); // do not change expression stack
      break;
    default:
      ShouldNotReachHere();
  }

  // handle exceptions via exception handler or rethrow code if possible

  // determine exception klass if possible
  ciInstanceKlass* ex_klass = NULL;
  { const TypeOopPtr* t = ex_node->bottom_type()->isa_oopptr();
  if( t != NULL && t != TypeOopPtr::NOTNULL ) ex_klass = t->klass()->as_instance_klass();
  // !!!!! TEST
  assert( (t != NULL) || (ex_node->bottom_type()->isa_klassptr() == NULL), "insanity check");
  }
  bool ex_klass_is_exact = false;
  // determine potential exception handlers
  ciExceptionHandlerStream handlers(_method, bci, ex_klass, ex_klass_is_exact);

  // There is always at least one handler.
  ciExceptionHandler* first_handler = handlers.handler();
  handlers.next();
  if (handlers.is_done()) {
    // There is a unique handler for this exception.
    _sp = 0;                  // Exception oop is only stack item
    // go to the exception handler
    int handler_bci = first_handler->handler_bci();
    if (handler_bci < 0) {
      // JVMPI -- record exit from a method if compiled while JVMPI is turned on
      if( C->is_set_jvmpi_method_event() ) {
        make_jvmpi_method_exit(bci);
      }
      // merge with corresponding rethrow node
      rethrow(ex_node, klass_index == Deoptimization::Deopt_null_check);
    } else {
      push(ex_node);
      merge(handler_bci, true);
    }
    assert(_map == NULL, "just checkin'");
    return;
  } else {
    // more then 1 potential handler => cannot deal with this for now
  }

  assert(sp == _sp, "just checkin'");
  // all other cases: handle exceptions via runtime throw or uncommon trap

  // if we have an exception handler in this nmethod
  // in the 'path' of the exception
  // => bail out to uncommon trap
  if( _has_ex_handler ) {
    // this scope's method has an exception handler
    // => bail out to uncommon trap for now
    uncommon_trap(klass_index, bci);
    return;
  }

  // we don't have an exception handler in this method
  // in the path of the exception
  // => it is safe to throw the exception w/o deoptimization
  if (control()->bottom_type() == Type::CONTROL) {
    // athrow is reachable
    switch (klass_index) {
      case Deoptimization::Deopt_null_check: /* nothing to do */    break;
      case Deoptimization::Deopt_div0_check: /* nothing to do */    break;
      case Deoptimization::Deopt_athrow    : pop(); /* remove exception obj */ break;
      default                              : ShouldNotReachHere();
    }

    CallNode* call =
      new CallStaticJavaNode(
        OptoRuntime::athrow_Type(),
        OptoRuntime::throw_exception_stub(),
        "athrow",
        bci
      );

    set_predefined_input_for_runtime_call(call);
    call->set_req(TypeFunc::I_O, i_o());

    // Pass the exception
    //
    // Note: Currently we pass a 'constant' exception oop instead of a
    //       freshly created exception for null and div0 exceptions.
    //
    //       Eventually this has to be cleaned up, by either a) using different
    //       runtime calls, or b) getting rid of this code alltogether.
    call->set_req(TypeFunc::Parms, ex_node);

    // The debug info is the only real input to this call.
    add_safepoint_edges(call, bci);

    Node* c = _gvn.transform(call);
    Node *ctrl = _gvn.transform(new (1) ProjNode(c,TypeFunc::Control));

    // Halt-and-catch fire here. The above call should never return!
    HaltNode *halt = new HaltNode(ctrl,frameptr());
    _gvn._types.map( halt->_idx, halt->bottom_type());
    _root->add_req(halt);
  }
  _map = NULL;
}


//------------------------------uncommon_trap----------------------------------
// Bail out to the interpreter in mid-method.  Implemented by calling the
// uncommon_trap blob.  This helper function inserts a runtime call with the
// right debug info.  
void Parse::uncommon_trap(int klass_index, int bci) {
  if( control()->bottom_type() == Type::CONTROL ) { // trap reachable?
#ifndef PRODUCT
    // Record trap type for debugging
    debug_only( Parse::record_deopt(klass_index); );
#endif

    // Make sure any guarding test views this path as very unlikely
    if( control()->in(0) ) {
      IfNode *iff = control()->in(0)->is_If();
      if( iff ) {               // Found a guarding if test?
        float f = iff->_prob;   // Get prob
        if( control()->Opcode() == Op_IfTrue ) {
          if( f > 0.0001 )        
            iff->_prob = 0.000001f;
        } else {
          if( f < 0.9999 )        
            iff->_prob = 0.999999f;
        }
      }
    }

    // Now insert the uncommon trap subroutine call
    CallNode *call =
      new CallStaticJavaNode(
        OptoRuntime::uncommon_trap_Type(),
        OptoRuntime::uncommon_trap_blob()->instructions_begin(),
        "uncommon_trap", 
        bci
      );

    set_predefined_input_for_runtime_call(call);
    call->set_req(TypeFunc::I_O, i_o());
    call->set_req(TypeFunc::ReturnAdr, returnadr());
    
    // Pass the index of the class to be loaded
    call->set_req(TypeFunc::Parms, _gvn.intcon(klass_index));
    
    // The debug info is the only real input to this call.
    add_safepoint_edges(call,bci);
    
    Node *c = _gvn.transform(call);
    Node *ctrl = _gvn.transform(new (1) ProjNode(c,TypeFunc::Control));

    // Halt-and-catch fire here.  The above call should never return!
    HaltNode *halt = new HaltNode(ctrl,frameptr());
    _gvn._types.map( halt->_idx, halt->bottom_type());
    _root->add_req(halt);
  }

  _map = NULL;
}
                                   

Node *Parse::intcon( int con ) { 
  return new (1) ConINode( TypeInt::make(con) );
}

Node *Parse::plus_adr( Node *ptr, int con ) const { 
  return new (4) AddPNode( _top/*No base for non-oop ptrs*/, ptr, intcon(con));
}

