/*
 * @(#)parseHelper.cpp	1.133 01/03/21 13:55:10
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_parseHelper.cpp.incl"

//------------------------------rethrow----------------------------------------
// merge current _map into _exit_rethrow node.  Build _exit_rethrow
// if needed.  Merge into the private null-ptr exception if asked.
void Parse::rethrow( Node *ex_oop, bool is_null_exception ) {
  // Have we built the exception-exit path yet?
  if( !_exit_rethrow ) {        // If not, built it now!
    RegionNode* region = new RegionNode(1);
    _for_igvn.push(region);
    PhiNode *ex_phi = new PhiNode(region, TypeOopPtr::NOTNULL);
    _gvn._types.map( ex_phi->_idx, ex_phi->_type );

    _exit_rethrow = new RethrowNode(region,
                                    new PhiNode(region, Type::ABIO),
                                    new MergeMemNode(NULL,C->_cnt_map_adr_mem),
                                    frameptr(),
                                    returnadr(),
                                    // like a return but with exception input
                                    ex_phi
                                    );
    // bind it to root
    _root->add_req(_exit_rethrow);

    // Also build a private merge path for the ever-common null exception
    RegionNode* r = new RegionNode(1);
    _for_igvn.push(r);
    _exit_rethrow->in(TypeFunc::Control)->add_req(r);
    _exit_rethrow->in(TypeFunc::I_O    )->add_req(new PhiNode(r, Type::ABIO  ));
    _exit_rethrow->in(TypeFunc::Parms  )->add_req(new PhiNode(r, TypeOopPtr::NOTNULL));
    Node *mm = _exit_rethrow->in(TypeFunc::Memory );
    for( uint i = 1; i < C->_cnt_map_adr_mem; i++ ) {
      Node *mphi = new PhiNode(region, Type::MEMORY);
      mphi->set_req( 1, new PhiNode(r, Type::MEMORY) );
      mm->set_req( i, mphi );
    }
  }
  Node *mm = _exit_rethrow->in(TypeFunc::Memory );
  if( is_null_exception ) {
    _exit_rethrow->in(TypeFunc::Control)->in(1)->add_req(control   ());
    _exit_rethrow->in(TypeFunc::I_O    )->in(1)->add_req(i_o       ());
    _exit_rethrow->in(TypeFunc::Parms  )->in(1)->add_req(ex_oop      );
    for( uint i = 1; i < C->_cnt_map_adr_mem; i++ )
      mm->in(i)->in(1)->add_req( memory(i) );
  } else {
    _exit_rethrow->in(TypeFunc::Control)       ->add_req(control   ());
    _exit_rethrow->in(TypeFunc::I_O    )       ->add_req(i_o       ());
    _exit_rethrow->in(TypeFunc::Parms  )       ->add_req(ex_oop      );
    for( uint i = 1; i < C->_cnt_map_adr_mem; i++ )
      mm->in(i)->       add_req( memory(i) );
  }

  _map = NULL; // this path ends here
}

//=============================================================================
// Generate a fast path/slow path idiom.  Graph looks like:
// [foo] indicates that 'foo' is a parameter
//
//              [in]     NULL
//                 \    /
//                  CmpP
//                  Bool ne
//                   If
//                  /  \
//              True    False-<2>
//              / |        
//             /  cast_not_null
//           Load  |    |   ^
//        [fast_test]   |   |
// gvn to   opt_test    |   |
//          /    \      |  <1>
//      True     False  |
//        |         \\  |
//   [slow_call]     \[fast_result]
//    Ctl   Val       \      \
//     |               \      \ 
//    Catch       <1>   \      \
//   /    \        ^     \      \
//  Ex    No_Ex    |      \      \
//  |       \   \  |       \ <2>  \
//  ...      \  [slow_res] |  |    \   [null_result]
//            \         \--+--+---  |  |
//             \           | /    \ | /
//              --------Region     Phi
//
//=============================================================================
// Code is structured as a series of driver functions all called 'do_XXX' that 
// call a set of helper functions.  Helper functions first, then drivers.

//------------------------------null_check_oop---------------------------------
// Null check oop.  Set null-path control into Region in slot 1.
// Make a cast-not-nullness use the other not-null control.  Return cast.
Node *Parse::null_check_oop( RegionNode *region, Node *oop, Node *null, bool never_see_null, int bci ) {

  // Initial NULL check
  Node   *cmp = _gvn.transform( new (3) CmpPNode( oop, null ) );
  Node   *tst = _gvn.transform( new (2) BoolNode( cmp, BoolTest::ne) );
  IfNode *iff = create_and_xform_if( control(), tst, never_see_null ? 0.99999f : 0.999f, -1.0f );

  // Initial NULL check taken path
  Node *null_true = _gvn.transform( new (1) IfFalseNode(iff) );
  if( never_see_null ) {
    PreserveJVMState pjvms(this, bci);
    set_control(null_true);
    uncommon_trap(Deoptimization::Deopt_unexpected_null_cast,bci);
    region->set_req(3,_top);    // NULL path is dead
  } else {
    region->set_req(3,null_true); 
  }

  // NULL check not-taken path
  Node *null_false = _gvn.transform( new (1) IfTrueNode(iff) );

  // Cast away null-ness on the result
  const Type *t = _gvn.type(oop);
  const Type *t_not_null = t->join(TypePtr::NOTNULL);
  Node *cast = new (2) CastPPNode(oop,t_not_null);
  cast->set_req(0,null_false);
  return cast;
}

//------------------------------make_check_type--------------------------------
IfNode *Parse::make_check_type( Node *control, Node *fieldmem, Node *cast, Node *slow_arg, float prob, float cnt ) {
  // Load the klass from the oop
  int con = oopDesc::klass_offset_in_bytes();
  Node* p = _gvn.transform( new (4) AddPNode( cast, cast, _gvn.intcon(con)) );
  Node* obj_klass = _gvn.transform(new (3) LoadKlassNode(0, fieldmem, p, TypeKlassPtr::OBJECT));
  IfNode *ccn = new (3) CheckTypeNode(control,obj_klass,slow_arg, prob, cnt);
  return ccn;
}

//------------------------------opt_iff----------------------------------------
// Optimize the fast-check IfNode.  Set the fast-path region slot 2.
// Return slow-path control.
Node *Parse::opt_iff( Node *region, Node *iff ) {
  Node *opt_test = _gvn.transform(iff);
  assert( opt_test->is_If(), "Expect an IfNode");
  IfNode *opt_iff = (IfNode*)opt_test;

  // Fast path taken; set region slot 2
  Node *fast_taken = _gvn.transform( new (1) IfFalseNode(opt_iff) );
  region->set_req(2,fast_taken); // Capture fast-control 

  // Fast path not-taken, i.e. slow path
  Node *slow_taken = _gvn.transform( new (1) IfTrueNode(opt_iff) );
  return slow_taken;
}

//------------------------------make_jvmpi_method_entry------------------------
// JVMPI -- record entry to a method if compiled while JVMPI is turned on
void Parse::make_jvmpi_method_entry() {
  const TypeFunc *call_type    = OptoRuntime::jvmpi_method_entry_Type();
  address         call_address = OptoRuntime::jvmpi_method_entry_Java();
  const char     *call_name    = OptoRuntime::stub_name( call_address );
  int             call_bci     = SynchronizationEntryBCI;
  const TypeOopPtr *method_type = TypeOopPtr::make(TypePtr::Constant, _method->klass(), _method, 0);
  Node *methodOop_node = _gvn.transform( new ConPNode(method_type) );
  Node *receiver_node  = (_method && !_method->is_static()) // IF  (virtual call)
    ? _map->in( TypeFunc::Parms )                           // THEN 'this' pointer, receiver,
    : _null;                                                // ELSE NULL

  make_slow_call( call_type, call_address, NULL, call_bci, control(), methodOop_node, receiver_node);
}

//------------------------------make_jvmpi_method_exit-------------------------
// JVMPI -- record entry to a method if compiled while JVMPI is turned on
void Parse::make_jvmpi_method_exit(int bci) {
  const TypeFunc *call_type    = OptoRuntime::jvmpi_method_exit_Type();
  address         call_address = OptoRuntime::jvmpi_method_exit_Java();
  const char     *call_name    = OptoRuntime::stub_name( call_address );
  int             call_bci     = bci;
  const TypeOopPtr*   method_type = TypeOopPtr::make(TypePtr::Constant, _method->klass(), _method, 0);
  Node *method_node = _gvn.transform( new ConPNode(method_type) );

  make_slow_call( call_type, call_address, NULL, call_bci, control(), method_node, _null );

}

//------------------------------make_slow_call---------------------------------
Node *Parse::make_slow_call( const TypeFunc *slow_call_type, address slow_call, const char *leaf_name, int bci, Node *slow_path, Node *parm0, Node *parm1 ) {

  Node *fast_mem = _map->memory(); // Capture and return old memory

  // Slow-path call
  CallNode *call = leaf_name 
    ? (CallNode*)new CallLeafNode      ( slow_call_type, slow_call, leaf_name )
    : (CallNode*)new CallStaticJavaNode( slow_call_type, slow_call, OptoRuntime::stub_name(slow_call), bci );

  // Slow path call has no side-effects, uses few values
  call->set_req( TypeFunc::Control,slow_path);
  call->set_req( TypeFunc::I_O    , _top );        // does no i/o
  call->set_req( TypeFunc::Memory , all_memory() );
  call->set_req( TypeFunc::ReturnAdr, _top );
  call->set_req( TypeFunc::FramePtr, frameptr() );
  call->set_req( TypeFunc::Parms+0,parm0);
  call->set_req( TypeFunc::Parms+1,parm1);
  if( !leaf_name ) add_safepoint_edges(call,bci);

  Node *c = _gvn.transform(call);
  // First clone the existing memory state
  set_all_memory( fast_mem );
  // Make memory for the call
  Node *mem = _gvn.transform( new (1) ProjNode(c, TypeFunc::Memory) );
  // Set the RawPtr memory state only.  This covers all the
  // heap top/GC stuff
  set_memory( mem, TypeRawPtr::BOTTOM );

  // Slow path call has no side-effects, sets few values
  set_control( _gvn.transform( new (1) ProjNode(c,TypeFunc::Control)) );
  // Return old memory to call
  return fast_mem;
}

//------------------------------make_merge_mem---------------------------------
// Merge fast path memory with slow path memory
void Parse::make_merge_mem( Node *region, Node *call, Node *fast_mem ) {

  // Slow path memory comes from the current map (which is from a slow call)
  Node *slow_mem = _map->memory();
  assert( slow_mem->Opcode() == Op_MergeMem, "" );
  // Fast path/null path memory comes from the call's input
  assert( fast_mem->Opcode() == Op_MergeMem, "" );

  // Merge the other fast-memory inputs with the new slow-default memory.
  for( uint k = 1; k < fast_mem->req(); k++ ) {
    if( slow_mem->in(k) != fast_mem->in(k) ) {
      Node *phi = new PhiNode( region, Type::MEMORY );
      phi->set_req( 1, slow_mem->in(k) ); // slow path mem comes from call
      phi->set_req( 2, fast_mem->in(k) ); // fast path mem is calls input
      if( region->req() == 4 )    // null path mem is calls input
        phi->set_req( 3, fast_mem->in(k) );
      _gvn._types.map(phi->_idx, Type::MEMORY);
      phi = _gvn.transform(phi);
      slow_mem->set_req(k,phi);
    }
  }
}

//------------------------------make_slow_call_ex------------------------------
// Make the exception handler hookups for the slow call
void Parse::make_slow_call_ex( Node *call, int bci, ciInstanceKlass* ex_klass ) {
  // Determine potential exception handlers for the cast
  ciExceptionHandlerStream handlers(method(), bci, ex_klass, false);
  
  // Pass the exception control on to any targets
  Node *result = _gvn.transform( new (1) ProjNode(call, TypeFunc::I_O));
  Node *catch_ = catch_exceptions(&handlers,bci,result);

  // Get the no-exception control from the CatchNode. 
  set_control( _gvn.transform( new (1) CatchProjNode(catch_, CatchProjNode::fall_through_index, CatchProjNode::no_handler_bci)) );
}  


//=============================================================================
//------------------------------do_checkcast-----------------------------------
void Parse::do_checkcast( ciByteCodeStream* stream ) {
  C->_has_split_ifs = true; // Has chance for split-if optimization
  int bci = stream->cur_bci();
  int bcp = (int)stream->cur_bcp();
  ciKlass* klass = stream->get_klass();

  // Throw uncommon trap if class is not loaded
  if( do_unloaded(stream, klass) ) return;

  Node *obj = peek();
  const Type *tk   = TypeKlassPtr::make(klass);
  const Type *toop = TypeJavaPtr::make_from_klass(klass)->meet(TypePtr::NULL_PTR);

  // Make the merge point
  RegionNode *region = new RegionNode(4);

  // Use null-cast information if it is available
  bool never_see_null = UncommonNullCast && // Collected this info?
    !_method->null_cast_seen()&&// And it was zero?
    obj != _null &&             // And not the -Xcomp stupid case?
    // Make sure we have not failed here before
    _method->null_cast_cache() != methodOopDesc::Null_Cast_Cache_Full && 
    _method->null_cast_cache() != bci;

  // Null check; get casted pointer; set region slot 3
  Node *cast = null_check_oop( region, obj, _null, never_see_null, bci );

  pop();                        // Pop from stack AFTER do_null_check

  // If cast is dead, only null-path is taken
  Node *not_null_ctrl = cast->in(0);
  if( not_null_ctrl == _top ) {
    push(_null);
    return;
  }
  cast = _gvn.transform(cast);

  // Set null path value
  Node *phi = new PhiNode(region,toop);
  phi->set_req(3,_null);

  // LoadKlass and make CheckType.  Optimize it.  Set region slot 2
  IfNode *iff = make_check_type( not_null_ctrl, memory(TypeOopPtr::HEADER), cast, makecon(tk), 0.000001f, -1.0f );
  Node *slow_path = opt_iff(region,iff);
  Node *fast_result = _gvn.transform(new (2) CheckCastPPNode(region->in(2),cast,toop));
  phi->set_req(2,fast_result);

  // Check out the fast test.  If the CheckTypeNode got replaced with
  // a simple pointer compare, then we have a Yes/No answer instead
  // of a Yes/Maybe answer and we no longer need a slow path call.
  Node *slow_result;
  if( slow_path->in(0)->Opcode() == Op_CheckType ) {
    Node *f_mem = make_slow_call( OptoRuntime::check_cast_Type(), OptoRuntime::check_cast_stub(), NULL, bci, slow_path, makecon(tk), cast );
    Node *call = control()->in(0); 
    assert( call->is_Call(), "" );
    make_slow_call_ex( call, bci, env()->ClassCastException_klass());
    region->set_req(1,control());
    make_merge_mem( region, call, f_mem );
    slow_result = new (2) CheckCastPPNode(control(),cast,toop);

  } else {                      // Deopt for check-cast throwing exception
    PreserveJVMState pjvms(this, bci);
    // End the current control-flow path
    push(obj);
    set_control(slow_path);
    uncommon_trap(Deoptimization::Deopt_cast_check,bci); 
    slow_result = _top; 
    region->set_req(1,_top);
  }
  // Capture slow path
  phi->set_req(1,_gvn.transform(slow_result));

  // Return final merged results
  set_control( _gvn.transform(region) );
  _for_igvn.push(region);
  // Push the casted result back on stack
  push( _gvn.transform(phi) );
}


//------------------------------do_instanceof----------------------------------
void Parse::do_instanceof(ciByteCodeStream* stream) {
  C->_has_split_ifs = true; // Has chance for split-if optimization
  int bci = stream->cur_bci();
  ciKlass* klass = stream->get_klass();

  // Throw uncommon trap if class is not loaded
  if( do_unloaded(stream, klass) ) return;

  Node *obj = pop();

  const TypeKlassPtr *tk   = TypeKlassPtr::make(klass);
  const Type         *toop = TypeJavaPtr::make_from_klass(klass);

  // Make the merge point
  RegionNode *region = new RegionNode(4);
  // Null check; get casted pointer; set region slot 1
  Node *cast = null_check_oop( region, obj, _null, false, bci );
  Node *not_null_ctrl = cast->in(0);
  // If cast is dead, only null-path is taken
  if( not_null_ctrl == _top ) {
    push(_gvn.intcon(0));
    return;
  }
  cast = _gvn.transform(cast);
  // Set null path value
  Node *phi = new PhiNode(region,TypeInt::BOOL);
  phi->set_req(3,_gvn.intcon(0));

  // LoadKlass and make CheckType.  Optimize it.  Set region slot 2
  IfNode *iff = make_check_type( not_null_ctrl, memory(TypeOopPtr::HEADER), cast, makecon(tk), 0.3f, -1.0f );
  Node *slow_path = opt_iff(region,iff);
  phi->set_req(2,_gvn.intcon(1));  // Capture fast-result

  // Check out the fast test.  If the CheckTypeNode got replaced with
  // a simple pointer compare, then we have a Yes/No answer instead
  // of a Yes/Maybe answer and we no longer need a slow path call.
  Node *slow_result;
  if( slow_path->in(0)->Opcode() == Op_CheckType ) {
    Node *f_mem = make_slow_call( OptoRuntime::instance_of_Type(), OptoRuntime::instance_of_stub(), NULL, bci, slow_path, makecon(tk), cast );
    Node *call = control()->in(0); 
    slow_result = _gvn.transform( new (1) ProjNode(call,TypeFunc::Parms));
    region->set_req(1,control());
    make_merge_mem( region, call, f_mem );

  } else {
    region->set_req(1,slow_path);
    slow_result = _gvn.intcon(0);  // Instance-of returns 0
  }
  // Capture slow path
  phi->set_req(1,_gvn.transform(slow_result));

  // Return final merged results
  set_control( _gvn.transform(region) );
  _for_igvn.push(region);
  // Push the bool result back on stack
  push( _gvn.transform(phi) );
}


//------------------------------array_store_check------------------------------
// pull array from stack and check that the store is valid
void Parse::array_store_check(int bci ) {
  C->_has_split_ifs = true; // Has chance for split-if optimization
  // Shorthand access to array store elements
  Node *obj = stack(_sp-1);
  Node *idx = stack(_sp-2);
  Node *ary = stack(_sp-3);

  // Extract the array element class
  int klass_offset = oopDesc::klass_offset_in_bytes();
  Node* p = basic_plus_adr( ary, ary, klass_offset );
  // p's type is array-of-OOPS plus klass_offset
  Node* array_klass =  _gvn.transform(new (3) LoadKlassNode(0, memory(TypeOopPtr::HEADER), p, TypeKlassPtr::OBJECT));
  int element_klass_offset = objArrayKlass::element_klass_offset_in_bytes() + sizeof(oopDesc);
  p = basic_plus_adr(array_klass, array_klass, element_klass_offset);
  Node *slow_arg = _gvn.transform(new (3) LoadKlassNode(0, memory(p), p, TypeKlassPtr::OBJECT));

  // Make the merge point
  RegionNode *region = new RegionNode(4);
  // Null check; get casted pointer; set region slot 3
  Node *cast = null_check_oop( region, obj, _null, false, bci );
  Node *not_null_ctrl = cast->in(0);
  // If control is dead, we are storing a NULL pointer
  if( not_null_ctrl == _top )
    return;                     // No store-check needed
  cast = _gvn.transform(cast);

  // LoadKlass and make CheckType; set region slot 2
  IfNode *iff = make_check_type( not_null_ctrl, memory(TypeOopPtr::HEADER), cast, slow_arg, 0.000001f, -1.0f );
  Node *slow_path = opt_iff(region,iff);

  // Check out the fast test.  If the CheckTypeNode got replaced with
  // a simple pointer compare, then we have a Yes/No answer instead
  // of a Yes/Maybe answer and we no longer need a slow path call.
  if( slow_path->in(0)->Opcode() == Op_CheckType ) {
    Node *f_mem = make_slow_call( OptoRuntime::check_cast_Type(), OptoRuntime::array_store_check_stub(), NULL, bci, slow_path, slow_arg, cast );
    Node *call = control()->in(0); 
    make_slow_call_ex( call,bci,env()->ArrayStoreException_klass() );
    region->set_req( 1,control() );
    make_merge_mem( region, call, f_mem );
  } else {
    // Incorrect to not throw the exception when we fail array_store_check
    // region->set_req(1,slow_path); // Else Slow path returns a NO result

    // Else: Deoptimize so interpreter throws an ArrayStoreException
    PreserveJVMState pjvms(this, bci);
    set_control(slow_path);
    uncommon_trap(Deoptimization::Deopt_cast_check,bci); 
    // End the current control-flow path
    region->set_req(1,_top);
  }
  // Capture slow path
  set_control( _gvn.transform(region) );
  _for_igvn.push(region);
}


//------------------------------next_monitor-----------------------------------
// What number should be given to the next monitor?
int Parse::next_monitor() {
  int next = _sync_base + _map->jvms()->nof_monitors();
  if (next+1 > _sync_max) {
    _sync_max = next+1;
  }
  return next;
}

//------------------------------insert_mem_bar---------------------------------
// Memory barrier to avoid floating things around
void Parse::insert_mem_bar() {
  Node *membar = _gvn.transform(new MemBarNode(control(), all_memory()));
  set_control(_gvn.transform(new (1) ProjNode(membar,TypeFunc::Control) ));
  set_all_memory_call(membar);
}
  
//------------------------------shared_lock------------------------------------
// Emit locking code.
FastLockNode *Parse::shared_lock( Node *obj, uint bci ) {
  if( !GenerateSynchronizationCode )
    return NULL;                // Not locking things?
  if( control() == _top )       // Dead monitor?
    return NULL;

  // Memory barrier to avoid floating things do into the locked region
  insert_mem_bar();
  
  // Box the stack location
  Node *box = _gvn.transform(new (1) BoxLockNode(next_monitor()));

  // Make the merge point
  Node *region = new RegionNode(3);
  // If converting bytecodes, push onto worklist for later transform
  _for_igvn.push(region);
  // Make the fast lock test.
  bool maybe_speclock = false;
  bool is_speclock = false;
  
  if (UseSpecLock) {
    // Spec locking is only used in known classes with no native methods
    if (!_klass->is_loaded()) {
      maybe_speclock = true;
    } else if (_klass->always_speculatively_locked() &&
      (bci == SynchronizationEntryBCI)) {
      // This is a synchronized method of a spec lock class
      is_speclock = true;
    } else if (bci != SynchronizationEntryBCI) {
      // Any synchronized block. Since we currently don't know the
      // type of the lock object, must emit dynamic check (maybe_speclock)
      maybe_speclock = true;
    }
  }

  if (UseSpecLock && PrintSpecLock && PrintCompilation) {
    tty->print_cr(" Lock: maybe speclock?%d is speclock?%d BCI=%d",
      maybe_speclock, is_speclock, bci); 
  }

  FastLockNode* flock = new (3) FastLockNode( 0, obj, box, is_speclock, maybe_speclock);
  

  _gvn.transform( flock );
  Node *bol = _gvn.transform(new (2) BoolNode(flock,BoolTest::ne));
  Node *iff = new (2) IfNode( control(), bol, 0.000001f, -1.0f );
  // Optimize test; set region slot 2
  Node *slow_path = opt_iff(region,iff);
  
  // Add monitor to debug info for the slow path.  If we block inside the
  // slow path and de-opt, we need the monitor hanging around
  map()->push_monitor( flock );

  // Make slow path call
  Node *f_mem = make_slow_call( OptoRuntime::complete_monitor_enter_Type(), OptoRuntime::complete_monitor_locking_Java(), NULL, bci, slow_path, obj, box );
  Node *call = control()->in(0); 
  // Get current call memory
  Node *mem = memory( TypeRawPtr::BOTTOM );
  // Pretend slow lock alters all memory to preserve lock semantics
  for( uint i = 1; i < f_mem->req(); i++ )
    set_memory( mem, i );

  // Slow path can only throw asynchronous exceptions, which are always
  // de-opted.  So the compiler thinks the slow-call can never throw an
  // exception.  If it DOES throw an exception we would need the debug
  // info removed first (since if it throws there is no monitor).

  // Capture slow path
  region->set_req(1,control());
  // Optimize control
  region = _gvn.transform(region);
  set_control(region);
  // Hack memory state
  make_merge_mem( region, call, f_mem );

  return flock;
}


//------------------------------shared_unlock----------------------------------
// Emit unlocking code.
void Parse::shared_unlock( Node *box, Node *obj, uint bci ) {
  if( !GenerateSynchronizationCode )
    return;
  if( control() == _top ) {     // Dead monitor?
    _map->pop_monitor( );       // Kill monitor from debug info
    return;
  }

  // No need for a null check on unlock

  // Memory barrier to avoid floating things do into the locked region
  insert_mem_bar();

  // Make the merge point
  RegionNode *region = new RegionNode(3);
  // If converting bytecodes, push onto worklist for later transform
  _for_igvn.push(region);
 // Make the fast lock test.
  bool maybe_speclock = false;
  bool is_speclock = false;
 
  if (UseSpecLock) {
    // See comments in lock
    if (!_klass->is_loaded()) {
      maybe_speclock = true;  
    } else if (_klass->always_speculatively_locked() &&
      (bci == SynchronizationEntryBCI)) {
      is_speclock = true;
    } else if (bci != SynchronizationEntryBCI) {
      maybe_speclock = true;
    }
  }
  
  if (UseSpecLock && PrintSpecLock && PrintCompilation) {
    tty->print_cr(" Unlock: maybe speclock?%d is speclock?%d BCI=%d",
      maybe_speclock, is_speclock, bci); 
  } 

  FastUnlockNode *funlock = new (3) FastUnlockNode( 0, obj, box, is_speclock, maybe_speclock );
 
  Node *tmp_funlock = _gvn.transform( funlock );

  // Verify that node will be a fast unlock
  assert( tmp_funlock->Opcode() == Op_FastUnlock, "must do a fast unlock");
  funlock = (FastUnlockNode*)tmp_funlock;
  Node *bol = _gvn.transform(new (2) BoolNode(funlock,BoolTest::ne));
  Node *iff = new (2) IfNode( control(), bol, 0.000001f, -1.0f );
  // Optimize test; set region slot 2
  Node *slow_path = opt_iff(region,iff);

  Node *f_mem = make_slow_call( OptoRuntime::complete_monitor_exit_Type(), CAST_FROM_FN_PTR(address, OptoRuntime::complete_monitor_unlocking_C), "complete_monitor_unlocking_C", bci, slow_path, obj, box );
  
  Node *call = control()->in(0); 
  // Get current call memory
  Node *mem = memory( TypeRawPtr::BOTTOM );
  // Pretend slow lock alters all memory to preserve lock semantics
  for( uint i = 1; i < f_mem->req(); i++ )
    set_memory( mem, i );

  // No exceptions for unlocking

  // Capture slow path
  region->set_req(1,control());
  // Optimize final control
  set_control( _gvn.transform(region) );
  // Hack memory state
  make_merge_mem( region, call, f_mem );

  // Kill monitor from debug info
  map()->pop_monitor( );
}


//=============================================================================
//------------------------------test_and_bump_heap-----------------------------
// fast case: test and bump heap top
IfNode *Parse::test_and_bump_heap( Node* size ) {
  // Code issued is:
  //   Atomic_add of Eden::top and oop_size
  //   Compare Eden::top to Eden::end
  //   Conditional call to GC stub

  // Note that concurrent allocation attempts can run off the end. The GC
  // stub requires the old Eden::top, since only for the allocation attempt
  // that ran off the end of Eden (using Atomic_add) do we know the last
  // valid Eden::top.

  // Atomic increment of heap top by oop size.  Return new heap top.
  address top_adr = (address)Universe::new_gen()->eden()->top_addr();
  Node *eden_top_adr = makecon(TypeRawPtr::make(top_adr));
  Node *aadd = _gvn.transform(new (4) AtomicAddNode(control(), memory(TypeRawPtr::BOTTOM), eden_top_adr, size ));
    
  // Compare new heap top to heap max for overflow (and GC)
  address max_adr = (address)Universe::new_gen()->eden()->end_addr();
  Node* eden_max_adr = basic_plus_adr( eden_top_adr, eden_top_adr, max_adr - top_adr );
  Node *eden_max = _gvn.transform(new (3) LoadPNode(0, memory(TypeRawPtr::BOTTOM), eden_max_adr, TypeRawPtr::BOTTOM));
    
  // Note: need unsigned pointer compare for portability (if eden lives in
  // upper half of address space)
  Node   *cmp = _gvn.transform(new (3) CmpPNode( aadd, eden_max ));
  Node   *b   = _gvn.transform(new (2) BoolNode( cmp, BoolTest::gt ));
  IfNode *iff = new (2) IfNode( control(), b, 0.00001f, -1.0f );
  return iff;
}

// Similar to above, but avoids atomic add (by allocating in the thread-local space)
IfNode *Parse::bump_tle( Node* size, Node** newoop) {
  // see test_and_bump_heap

  Node *eden_top_adr, *eden_max_adr;
  Node *eden_top, *eden_max;
  Node *new_top;
  
  // Thread-local Eden. Use a regular add node to compute the new Eden top
  Node *thread = _gvn.transform(new (1) ThreadLocalNode());
  eden_top_adr = _gvn.transform( new (4) AddPNode(_top/*not oop*/ , thread, _gvn.intcon( JavaThread::tle_top_offset_in_bytes())));
  eden_max_adr = _gvn.transform( new (4) AddPNode(_top/*not oop*/, thread, _gvn.intcon( JavaThread::tle_end_offset_in_bytes())));
  eden_top = _gvn.transform(new (3) LoadPNode(control(), memory(TypeRawPtr::BOTTOM), eden_top_adr, TypeRawPtr::BOTTOM));
  eden_max = _gvn.transform(new (3) LoadPNode(control(), memory(TypeRawPtr::BOTTOM), eden_max_adr, TypeRawPtr::BOTTOM));
  
  new_top = _gvn.transform(new (4) AddPNode(_top/*not oop*/, eden_top, size));  // new top
   // Get the alias_index for raw memory
  uint alias_idx = C->get_alias_index(TypeRawPtr::BOTTOM);
  Node *mem = memory(alias_idx); 
 
  set_memory(_gvn.transform(new (4) StorePNode(control(), mem, eden_top_adr, new_top, Type::MEMORY)), eden_top_adr);
  (*newoop) = eden_top;    // return the old top
  
  Node   *cmp = _gvn.transform(new (3) CmpPNode( new_top, eden_max ));
  Node   *b   = _gvn.transform(new (2) BoolNode( cmp, BoolTest::gt ));
  IfNode *iff = new (2) IfNode( control(), b, 0.00001f, -1.0f );
  return iff;
}


   
//------------------------------init_object------------------------------------
// Initialize mark and klass words
void Parse::init_object( Node *klass_node, Node *newoop, bool init_speclock ) {
  Node *adr = basic_plus_adr(newoop, newoop, oopDesc::mark_offset_in_bytes());
  int m;
  if (init_speclock) {
    m = (int) markOopDesc::prototype_speclock();
  } else {
    m = (int) markOopDesc::prototype();
  }
  set_memory(_gvn.transform(new (4) StoreINode(control(), memory(adr), adr, _gvn.intcon(m), Type::MEMORY)), adr );
  adr = basic_plus_adr(newoop, newoop, oopDesc::klass_offset_in_bytes());
  set_memory(_gvn.transform(new (4) StorePNode(control(), memory(adr), adr, klass_node, Type::MEMORY)), adr);
}

//------------------------------fast_path_allocate-----------------------------
Node * Parse::fast_path_allocate( RegionNode *region, Node *alloc_size,
				  Node *klass_node, Node **slow_ctrl_p) {
  Node *newoop;
  if (UseTLE) {
    // fast case: test and bump heap
    IfNode *iff = bump_tle(alloc_size, &newoop);  // sets newoop 
    // Make true/false Nodes on iff.
    *slow_ctrl_p = opt_iff( region, iff );
  } else {
    // fast case: test and bump heap
    IfNode *iff = test_and_bump_heap(alloc_size);
    // Make true/false Nodes on iff.
    *slow_ctrl_p = opt_iff( region, iff );
    // Get atomic-add result
    Node *aadd = iff->in(1)->in(1)->in(1);
    // Back down from heap top to get oop start
    Node *neg_size = _gvn.transform( new (3) SubINode(_gvn.intcon(0),alloc_size));
    newoop = _gvn.transform(new (4) AddPNode(_top/*not oop*/, aadd, neg_size));
  }
  return newoop;
}

//------------------------------do_new-----------------------------------------
void Parse::do_new( ciByteCodeStream* stream ) {
  // The allocator will coalesce int->oop copies away.  See comment in
  // coalesce.cpp about how this works.  It depends critically on the exact
  // code shape produced here, so if you are changing this code shape
  // make sure the GC info for the heap-top is correct in and around the
  // slow-path call.

  int bci = stream->cur_bci();
  ciInstanceKlass* klass = stream->get_klass()->as_instance_klass();

  // uncommon_trap if unloaded or uninitialized
  if( do_unloaded_or_uninitialized(stream, klass) ) return;
  // Need klass node constant for either fast or slow path
  Node *klass_node = makecon(TypeKlassPtr::make(klass));
  // Check to see if the oop has a finalizer.  If so, it must be created
  // with a slow-path call only so it gets registered.
  const TypeOopPtr *toop = TypeOopPtr::make(TypePtr::NotNull, klass);
  if( klass->has_finalizer() ) {
    // only slow case: call out-of-line code in OptoRuntime
    CallNode *call = new CallStaticJavaNode(OptoRuntime::new_slow_Type(), OptoRuntime::new_slow_Java(), OptoRuntime::stub_name(OptoRuntime::new_slow_Java()), bci);
    // set inputs for call
    set_predefined_input_for_runtime_call(call);
    call->set_req( TypeFunc::Parms+0, klass_node );
    // Add the safepoint edges to the call
    add_safepoint_edges(call, bci);
    // Register call with optimizer
    Node *c = _gvn.transform(call);
    // Set fixed predefined return values
    set_predefined_output_for_runtime_call(c);
    make_slow_call_ex( c, bci, env()->OutOfMemoryError_klass());
    // Pick up return oop
    Node *newoop = _gvn.transform(new (1) ProjNode(c, TypeFunc::Parms));
    // Cast to correct type
    Node *cast = _gvn.transform( new (2) CheckCastPPNode( control(), newoop, toop ) );
    // Push resultant oop on JVM stack
    push( cast );
    return;
  }

  // Generate fast-path/slow-path idiom.

  // Make the merge point
  RegionNode *region = new RegionNode(3);
  // Make merge for fast/slow oop address
  Node *phi = new PhiNode(region, TypeRawPtr::NOTNULL);

  // Generate allocation code (heap pointer increment & check)
  int size_in_bytes = align_object_size(klass->size_helper()) * oopSize;

  Node *newoop, *slow_ctrl;
  newoop = fast_path_allocate(region, _gvn.intcon(size_in_bytes), klass_node, &slow_ctrl);

  // Capture fast-path result.
  phi->set_req( 2, newoop );

  // Set control to the fast path to pin the stores there
  set_control( region->in(2) );
  // We'll need to reset the state of raw memory to before the stores
  // for the slow-path side
  uint raw_idx = C->get_alias_index( TypeRawPtr::BOTTOM );
  Node *old_raw_mem = memory( raw_idx );
  // Now initialize the object on the fast-path only
  bool use_speclock = UseSpecLock && klass->always_speculatively_locked();
  init_object( klass_node, newoop, use_speclock );

  // Raw memory cache
  Node *mem = memory( raw_idx );

  // Reset the raw-memory state to before the stores
  set_memory( old_raw_mem, raw_idx );

  if (!ZeroTLE) {
    // zero out instance fields into raw memory
    Node *zero = _gvn.intcon(0);
    ciFieldLayout* layout = klass->field_layout();
    ciFieldLayoutStream ls(layout);
    ls.skip_header();
    for ( ; !ls.is_done(); ls.next()) {
      Node *fld_adr = basic_plus_adr(newoop, newoop, ls.pos_in_bytes());
      switch (ls.type()) {
      case T_BOOLEAN:
      case T_CHAR:
      case T_BYTE:
      case T_SHORT:
      case T_INT:
	mem = new (4) StoreINode(control(), mem, fld_adr, zero, Type::MEMORY); 
	break;
      case T_LONG:
	mem = _gvn.transform(new (4) StoreINode(control(), mem, fld_adr, zero, Type::MEMORY));
	fld_adr = basic_plus_adr(newoop, newoop, ls.pos_in_bytes() + wordSize);
	mem = new (4) StoreINode(control(), mem, fld_adr,  zero, Type::MEMORY);
	break;
      case T_ARRAY:
      case T_OBJECT:
	mem = new (4) StorePNode(control(), mem, fld_adr, _null, Type::MEMORY);
	break;
      case T_FLOAT:
	mem = new (4) StoreFNode(control(), mem, fld_adr, makecon(TypeF::ZERO), Type::MEMORY); 
	break;
      case T_DOUBLE:
	mem = new (4) StoreDNode(control(), mem, fld_adr, makecon(TypeD::ZERO), Type::MEMORY); 
	break;
      case T_VOID:
	// Skip to next field in the object.
	continue;
      default:
	ShouldNotReachHere();
      }
      mem = _gvn.transform(mem);
    }
  }

  // If spec-locking, shove the Thread ID into the spec-lock field
  if (use_speclock) {
    Node *lock_adr = basic_plus_adr(newoop, newoop, SpecLock::speclock_field_offset());
    Node *tid = _gvn.transform(new (1) LoadThreadIDNode());
    mem = _gvn.transform( new(4) StoreINode(control(), mem, lock_adr, tid, Type::MEMORY) );
  }

  // Make the slow-path GC stuff
  make_slow_call( OptoRuntime::new_Type(), OptoRuntime::new_Java(), NULL, bci, slow_ctrl, newoop, klass_node );
  Node *call = control()->in(0); 
  make_slow_call_ex( call, bci, env()->OutOfMemoryError_klass());
  // Grab slow-path control at merge point
  region->set_req(1,control());

  // Fixup memory at merge point.  Only Raw memory has changed, so 
  // only need to merge Raw memory.
  Node *phi_mem = new PhiNode( region, Type::MEMORY );
  phi_mem->set_req( 1, memory( raw_idx ) );
  phi_mem->set_req( 2, mem );
  _gvn._types.map(phi_mem->_idx, Type::MEMORY);
  set_memory( _gvn.transform(phi_mem), raw_idx );

  // Issue a prefetch on the fast-only-path
  if( AllocatePrefetchStyle ) {
    Node *prefetch_adr = basic_plus_adr(newoop, newoop, size_in_bytes + AllocatePrefetchDistance);
    Node *pre = _gvn.transform( new (3) PrefetchNode( call->in(TypeFunc::I_O), prefetch_adr ) );
    Node *phi_i_o = new PhiNode( region, Type::ABIO );
    phi_i_o->set_req( 1, i_o() );
    phi_i_o->set_req( 2, pre );
    _gvn._types.map(phi_i_o->_idx, Type::ABIO);
    set_i_o( phi_i_o );
  }

  // Capture slow path result (the new oop) at merge point
  phi->set_req(1,_gvn.transform( new (1) ProjNode(call,TypeFunc::Parms)));
  // Return final merged results
  set_control( _gvn.transform(region) );
  _for_igvn.push(region);
  // Oop, post merging fast/slow path
  newoop = _gvn.transform(phi);
  // Cast to correct type
  newoop = _gvn.transform( new (2) CheckCastPPNode( control(), newoop, toop ));
  
  // Push resultant oop onto stack
  push( newoop );
}

//------------------------------do_Xnewarray-----------------------------------
// helper for both newarray and anewarray
void Parse::do_Xnewarray(Node* count_val,       // number of elements
                         BasicType elem_type,   // BasicType of elememts
                         const Type* etype,     // Compiler type of elements
                         const TypeKlassPtr* array_klass, // array klass
                         int bci) {

  // Check for negative array length.  Too large just forces a slow-path GC 
  // which can throw the too-large exception just fine.
  Node *cmp   = _gvn.transform(new (3) CmpUNode( count_val, _gvn.intcon(arrayOopDesc::max_array_length(elem_type)) ));
  Node *b     = _gvn.transform(new (2) BoolNode( cmp, BoolTest::le ));
  IfNode *iff = create_and_map_if( control(), b, 0.999999f, -1.0f );

  // If negative, add another exception exit path
  Node* error = _gvn.transform(new (1) IfFalseNode(iff));
  set_control(error);
  { PreserveJVMState pjvms(this, bci);
    push(count_val);            // Push back size for trap            
    uncommon_trap(Deoptimization::Deopt_negative_array_length,bci);
  }

  // Else fall through
  set_control(_gvn.transform(new (1) IfTrueNode (iff)));
  // Funny JCK test must throw negative size exception
  if( control() == _top ) return;

  // Try to get a better type than POS for the size
  const Type* count_type = _gvn.type(count_val)->join(TypeInt::POS);
  assert(count_type->is_int(), "must be integer");

  // --- Vast Annoying Size Computation ---

  // compute array_words = size in words of count_val elements,
  // (elem_byte_size * len + 3) >> 2 (see typeArrayOopDesc::word_size)
  Node *array_words;
  if( type2aelembytes[elem_type] == oopSize ) { 
    array_words = count_val;    // Skip rounding for popular size 4 elements
  } else {
    Node* elem_size = _gvn.intcon(type2aelembytes[elem_type]);
    Node* mul = _gvn.transform(new (3) MulINode( elem_size, count_val));
    Node* add = _gvn.transform(new (3) AddINode( mul, _gvn.intcon(3)));
    array_words = _gvn.transform(new (3) URShiftINode(add, _gvn.intcon(log2(oopSize))));
  }

  // base_words = size in words without array elements
  int base_words = typeArrayOopDesc::header_size(elem_type);
  // Add array header size
  Node* alloc_words = _gvn.transform(new (3) AddINode( _gvn.intcon(base_words), array_words));
  // align object size (see align_object_size())
  Node* alloc_words_aligned;
  if (ALIGN_ALL_OBJECTS) {
    Node* a1 = _gvn.transform(new (3) AddINode( alloc_words, _gvn.intcon(1)));
    alloc_words_aligned = _gvn.transform(new (3) AndINode( a1, _gvn.intcon(~1)));
  } else {
    alloc_words_aligned = alloc_words;
  }
  // scale to bytes
  Node* alloc_size = _gvn.transform(new (3) MulINode(alloc_words_aligned, _gvn.intcon(oopSize)));

  // --- Generate fast-path/slow-path idiom. ---

  // Make the merge point
  RegionNode *region = new RegionNode(3);
  // Make merge for fast/slow oop address
  Node *phi = new PhiNode(region, TypeRawPtr::NOTNULL);

  Node *slow_ctrl, *aadd, *newoop;
  // fast case: test and bump heap
  if (UseTLE) {
    iff = bump_tle(alloc_size, &newoop);  // sets newoop 
    // Make true/false Nodes on iff.
    slow_ctrl = opt_iff( region, iff );
  }
  else {
    iff = test_and_bump_heap(alloc_size);
    // Make true/false Nodes on iff.
    slow_ctrl = opt_iff( region, iff );
    // Get atomic-add result
    aadd = iff->in(1)->in(1)->in(1);
    // Back down from heap top to get oop start
    Node* neg_alloc_size = _gvn.transform(new (3) SubINode( _gvn.intcon(0), alloc_size));
    newoop = _gvn.transform(new (4) AddPNode(_top/*not oop*/, aadd, neg_alloc_size));
  }

  // Capture fast-path result.
  phi->set_req( 2, newoop );

  // Set control to the fast path to pin the stores there
  set_control( region->in(2) );
  // We'll need to reset the state of raw memory to before the stores
  // for the slow-path side
  uint raw_idx = C->get_alias_index( TypeRawPtr::BOTTOM );
  Node *old_raw_mem = memory( raw_idx );
  // Initialize the array
  Node* klass_node = makecon(array_klass);
  init_object( klass_node, newoop, false );

  // Raw memory cache
  Node *mem = memory( raw_idx );

  // Reset the raw-memory state to before the stores
  set_memory( old_raw_mem, raw_idx );

  // init length field
  Node *adr = basic_plus_adr(newoop, newoop, arrayOopDesc::length_offset_in_bytes());
  uint alias_idx = C->get_alias_index(TypeAryPtr::RANGE);
  mem = _gvn.transform(new (4) StoreINode(control(), mem, adr, count_val, Type::MEMORY));
  // there may be a padding word after the length, but it doesn't
  // need to be initialized

  if(!ZeroTLE) {
    Node* array_offset = _gvn.intcon(arrayOopDesc::base_offset_in_bytes(elem_type));
    Node* dataP = _gvn.transform(new (4) AddPNode(newoop, newoop, array_offset));
    mem = _gvn.transform(new (4) ClearArrayNode(control(),mem,array_words,dataP));
  }

  // slow case: call out-of-line code in OptoRuntime
  if (elem_type == T_OBJECT) {
    assert( etype->isa_oopptr() || etype->isa_aryptr(), "" );
    const TypeJavaPtr *jetype = etype->isa_javaptr();
    ciKlass* klass = jetype->klass();
    Node *elem_klass_node = makecon( TypeKlassPtr::make(klass) );
    make_slow_call( OptoRuntime::new_objArray_Type(), OptoRuntime::new_objArray_Java(), NULL, bci, slow_ctrl, elem_klass_node, count_val );
  } else {
    make_slow_call( OptoRuntime::new_typeArray_Type(), OptoRuntime::new_typeArray_Java(), NULL, bci, slow_ctrl, _gvn.intcon(elem_type), count_val );
  }
  Node *call = control()->in(0); 
  _gvn.hash_delete(call);
  call->set_req(TypeFunc::Parms+2, newoop);
  make_slow_call_ex( call, bci, env()->OutOfMemoryError_klass() );

  // Grab slow-path control at merge point
  region->set_req(1,control());

  // Fixup memory at merge point.  Only Raw memory has changed, so 
  // only need to merge Raw memory.
  Node *phi_mem = new PhiNode( region, Type::MEMORY );
  phi_mem->set_req( 1, memory( raw_idx ) );
  phi_mem->set_req( 2, mem );
  _gvn._types.map(phi_mem->_idx, Type::MEMORY);
  set_memory( _gvn.transform(phi_mem), raw_idx );

  // Issue a prefetch on the fast-only-path
  if( AllocatePrefetchStyle ) {
    Node *oop_end = _gvn.transform(new (4) AddPNode(_top/*not oop*/, newoop, alloc_size));
    Node *prefetch_adr = basic_plus_adr(oop_end, oop_end, AllocatePrefetchDistance);
    Node *pre = _gvn.transform( new (3) PrefetchNode( call->in(TypeFunc::I_O), prefetch_adr ) );
    Node *phi_i_o = new PhiNode( region, Type::ABIO );
    phi_i_o->set_req( 1, i_o() );
    phi_i_o->set_req( 2, pre );
    _gvn._types.map(phi_i_o->_idx, Type::ABIO);
    set_i_o( phi_i_o );
  }

  // Capture slow path result (the new oop) at merge point
  phi->set_req(1,_gvn.transform( new (1) ProjNode(call,TypeFunc::Parms)));
  // Return final merged results
  set_control( _gvn.transform(region) );
  _for_igvn.push(region);
  // Oop, post merging fast/slow path
  newoop = _gvn.transform(phi);
  // Cast to correct type
  const TypeAry* arr0 = TypeAry::make(etype, (TypeInt*)count_type);
  const TypeAryPtr* tarr = TypeAryPtr::make(TypePtr::NotNull, arr0, array_klass->klass(), 0);
  newoop = _gvn.transform( new (2) CheckCastPPNode( control(), newoop, tarr ));

  // Push resultant oop onto stack
  push( newoop );
}

//=============================================================================
//------------------------------all_memory-------------------------------------
Node *Parse::all_memory() const {
  return _gvn.transform( _map->memory() ); 
}

//------------------------------set_all_memory---------------------------------
void Parse::set_all_memory( Node *newmem ) {
  Node *mergemem = new MergeMemNode(newmem, C->_cnt_map_adr_mem);
  _map->set_memory(mergemem);
  _gvn._types.map(mergemem->_idx, mergemem->bottom_type());
}

//------------------------------set_all_memory_call----------------------------
void Parse::set_all_memory_call( Node *call ) {
  set_all_memory( _gvn.transform( new (1) ProjNode(call, TypeFunc::Memory) ) );
}

#ifndef PRODUCT
//------------------------------dump_map_adr_mem-------------------------------
// Debug dump of the mapping from address types to MergeMemNode indices.
void Parse::dump_map_adr_mem() const {
  tty->print_cr("--- Mapping from address types to memory Nodes ---");
  Node *mem = NULL;
  if( _map && _map->req() > 1 )
    mem = _map->in(1);
  for( uint i = 1; i < C->_cnt_map_adr_mem; i++ ) {
    tty->print("%d) ",i);
    C->_map_adr_mem[i]->dump();
    tty->print("\t");
    // Node mapping, if any
    if( mem && C->_cnt_map_adr_mem < mem->req() )
      mem->in(i)->dump();
    else
      tty->cr();
  }
}

#endif
