/*
 * @(#)phase.cpp	1.35 01/02/09 14:46:57
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_phase.cpp.incl"

const char * const Phase::names[Phase::last_phase] = {
  "Compiler",
  "Parse",
  "Optimistic",
  "Global Value Numbering",
  "Instruction Selection",
  "Copy Elimination",
  "Dead Code Elimination",
  "Conditional Constant Propagation",
  "CFG",
  "DefUse",
  "Register Allocation",
  "LIVE",
  "Interference Graph",
  "Coalesce",
  "Conditional Constant Propagation",
  "Find Idealized Loops"
};

elapsedTimer Phase::_t_totalCompilation;
elapsedTimer Phase::_t_methodCompilation;
elapsedTimer Phase::_t_nativeCompilation;
elapsedTimer Phase::_t_stubCompilation;
elapsedTimer Phase::_t_adapterCompilation;

elapsedTimer Phase::_t_parser;
elapsedTimer Phase::_t_optimizer;
elapsedTimer Phase::_t_scheduler;
elapsedTimer Phase::_t_registerAllocation;
elapsedTimer Phase::_t_output;
elapsedTimer Phase::_t_instrSched;

int Phase::_total_bytes_compiled = 0;
int Phase::_total_nmethods_size = 0;
int Phase::_total_code_size = 0;

//------------------------------Phase------------------------------------------
Phase::Phase( PhaseNumber pnum ) : _pnum(pnum), C( pnum == Compiler ? NULL : Compile::current()) { 
}

//------------------------------Phase------------------------------------------

void Phase::add_code_size(nmethod* nm) {
  _total_nmethods_size += nm->size(); 
  _total_code_size += nm->code_size();
}

#ifndef PRODUCT
void Phase::print_timers() {
  tty->print_cr ("Accumulated compiler times:");
  tty->print_cr ("---------------------------");
  tty->print_cr ("  Total compilation: %3.3f sec.", Phase::_t_totalCompilation.seconds());
  tty->print    ("    method compilation : %3.3f sec", Phase::_t_methodCompilation.seconds());
  tty->print    ("/%d bytes",_total_bytes_compiled);
  tty->print_cr (" (%3.0f bytes per sec) ", Phase::_total_bytes_compiled / Phase::_t_methodCompilation.seconds());
  tty->print_cr ("       [%3.1f nmethod per method code bytes - including debug info etc..] ", (float)Phase::_total_nmethods_size / (float)Phase::_total_bytes_compiled );
  tty->print_cr ("       [%3.1f nmethod assembly bytes per method bytes] ", (float)Phase::_total_code_size / (float)Phase::_total_bytes_compiled );
  tty->print_cr ("    native compilation : %3.3f sec.", Phase::_t_nativeCompilation.seconds());
  tty->print_cr ("    stub compilation   : %3.3f sec.", Phase::_t_stubCompilation.seconds());
  tty->print_cr ("    adapter compilation: %3.3f sec.", Phase::_t_adapterCompilation.seconds());
  tty->print_cr ("  Phases:");
  tty->print_cr ("    parse     : %3.3f sec", Phase::_t_parser.seconds());
  tty->print_cr ("    optimizer : %3.3f sec", Phase::_t_optimizer.seconds());
  tty->print_cr ("    scheduler : %3.3f sec", Phase::_t_scheduler.seconds());
  tty->print_cr ("    regalloc  : %3.3f sec", Phase::_t_registerAllocation.seconds());
  tty->print_cr ("    output    : %3.3f sec", Phase::_t_output.seconds());
  tty->print_cr ("      isched  : %3.3f sec", Phase::_t_instrSched.seconds());
}
#endif
