/*
 * @(#)phaseX.cpp	1.197 01/02/09 14:46:59
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_phaseX.cpp.incl"

//=============================================================================
#define NODE_HASH_MINIMUM_SIZE    255
//------------------------------NodeHash---------------------------------------
NodeHash::NodeHash(uint est_max_size) : 
  _max( round_up(est_max_size < NODE_HASH_MINIMUM_SIZE ? NODE_HASH_MINIMUM_SIZE : est_max_size) ),
  _a(Thread::current()->resource_area()),
  _table( NEW_ARENA_ARRAY( _a , Node* , _max ) ), // (Node**)_a->Amalloc(_max * sizeof(Node*)) ), 
  _inserts(0), _insert_limit( insert_limit() ),
  _look_probes(0), _lookup_hits(0), _lookup_misses(0), 
  _total_insert_probes(0), _total_inserts(0), 
  _insert_probes(0), _grows(0) {
  // _sentinel must be in the current node space
  _sentinel = new (1) ProjNode(NULL, TypeFunc::Control);
  memset(_table,0,sizeof(Node*)*_max);
}

//------------------------------NodeHash---------------------------------------
NodeHash::NodeHash(Arena *arena, uint est_max_size) : 
  _max( round_up(est_max_size < NODE_HASH_MINIMUM_SIZE ? NODE_HASH_MINIMUM_SIZE : est_max_size) ),
  _a(arena),
  _table( NEW_ARENA_ARRAY( _a , Node* , _max ) ), // (Node**)_a->Amalloc(_max * sizeof(Node*)) ), 
  _inserts(0), _insert_limit( insert_limit() ),
  _look_probes(0), _lookup_hits(0), _lookup_misses(0), 
  _delete_probes(0), _delete_hits(0), _delete_misses(0), 
  _total_insert_probes(0), _total_inserts(0), 
  _insert_probes(0), _grows(0) {
  // _sentinel must be in the current node space
  _sentinel = new (1) ProjNode(NULL, TypeFunc::Control);
  memset(_table,0,sizeof(Node*)*_max);
}

//------------------------------NodeHash---------------------------------------
NodeHash::NodeHash(NodeHash *nh) : _max(nh->_max),
  _a(nh->_a), _table(nh->_table), 
  _inserts(nh->_inserts), _insert_limit(nh->_insert_limit),
  _look_probes(nh->_look_probes), _lookup_hits(nh->_lookup_hits), _lookup_misses(nh->_lookup_misses), 
  _delete_probes(nh->_delete_probes), _delete_hits(nh->_delete_hits), _delete_misses(nh->_delete_misses), 
  _total_insert_probes(nh->_total_insert_probes), _total_inserts(nh->_total_inserts), 
  _insert_probes(nh->_insert_probes), _grows(nh->_grows) {
  // nh->_sentinel must be in the current node space
  _sentinel = nh->_sentinel;
}

//------------------------------hash_find--------------------------------------
// Find in hash table
Node *NodeHash::hash_find( const Node *n ) {
  // ((Node*)n)->set_hash( n->hash() );
  uint key = n->hash() & (_max-1); 
  uint stride = key | 0x01;
  debug_only( _look_probes++ );
  Node *k = _table[key];        // Get hashed value
  if( !k ) {                    // ?Miss?
    debug_only( _lookup_misses++ );
    return NULL;                // Miss!
  }
  
  int op = n->Opcode();
  uint req = n->req();
  while( 1 ) {                  // While probing hash table
    if( k->req() == req &&      // Same count of inputs
        k->Opcode() == op ) {   // Same Opcode
      for( uint i=0; i<req; i++ )
        if( n->in(i)!=k->in(i)) // Different inputs?
          goto collision;       // "goto" is a speed hack...
      if( n->cmp(*k) ) {        // Check for any special bits
        debug_only( _lookup_hits++ );
        return k;               // Hit!
      }
    }
  collision:
    debug_only( _look_probes++ );
    key = (key + stride/*7*/) & (_max-1); // Stride through table with relative prime
    k = _table[key];            // Get hashed value
    if( !k ) {                  // ?Miss?
      debug_only( _lookup_misses++ );
      return NULL;              // Miss!
    }
  }
  ShouldNotReachHere();
  return NULL;
}

//------------------------------hash_find_insert-------------------------------
// Find in hash table, insert if not already present
// Used to preserve unique entries in hash table
Node *NodeHash::hash_find_insert( Node *n ) {
  // n->set_hash( );
  uint key = n->hash() & (_max-1); 
  uint stride = key | 0x01;     // stride must be relatively prime to table siz
  uint first_sentinel = 0;      // replace a sentinel if seen.
  debug_only( _look_probes++ );
  Node *k = _table[key];        // Get hashed value
  if( !k ) {                    // ?Miss?
    debug_only( _lookup_misses++ );
    _table[key] = n;            // Insert into table!
    check_grow();               // Grow table if insert hit limit
    return NULL;                // Miss!
  }
  else if( k == _sentinel ) {
    first_sentinel = key;      // Can insert here
  }
  
  int op = n->Opcode();
  uint req = n->req();
  while( 1 ) {                  // While probing hash table
    if( k->req() == req &&      // Same count of inputs
        k->Opcode() == op ) {   // Same Opcode
      for( uint i=0; i<req; i++ )
        if( n->in(i)!=k->in(i)) // Different inputs?
          goto collision;       // "goto" is a speed hack...
      if( n->cmp(*k) ) {        // Check for any special bits
        debug_only( _lookup_hits++ );
        return k;               // Hit!
      }
    }
  collision:
    debug_only( _look_probes++ );
    key = (key + stride) & (_max-1); // Stride through table w/ relative prime
    k = _table[key];            // Get hashed value
    if( !k ) {                  // ?Miss?
      debug_only( _lookup_misses++ );
      key = (first_sentinel == 0) ? key : first_sentinel; // ?saw sentinel?
      _table[key] = n;          // Insert into table!
      check_grow();             // Grow table if insert hit limit
      return NULL;              // Miss!
    }
    else if( first_sentinel == 0 && k == _sentinel ) {
      first_sentinel = key;    // Can insert here
    }
      
  }
  ShouldNotReachHere();
  return NULL;
}

//------------------------------hash_insert------------------------------------
// Insert into hash table
void NodeHash::hash_insert( Node *n ) {
  // // "conflict" comments -- print nodes that conflict
  // bool conflict = false;
  check_grow();
  // n->set_hash();
  uint key = n->hash() & (_max-1); 
  uint stride = key | 0x01;
  
  while( 1 ) {                  // While probing hash table
    debug_only( _insert_probes++ );
    Node *k = _table[key];      // Get hashed value
    if( !k || (k == _sentinel) ) break;       // Found a slot
    assert( k != n, "already inserted" );
    // if( PrintOptoStatistics && PrintCompilation ) { tty->print("  conflict: "); k->dump(); conflict = true; }
    key = (key + stride) & (_max-1); // Stride through table w/ relative prime
  }
  _table[key] = n;              // Insert into table!
  // if( conflict ) { n->dump(); }
}

//------------------------------hash_delete------------------------------------
// Replace in hash table with sentinal
bool NodeHash::hash_delete( const Node *n ) {
  Node *k;
  uint key = n->hash() & (_max-1); 
  uint stride = key | 0x01;
  debug_only( uint counter = 0; );
  for( ; /* (k != NULL) && (k != _sentinal) */; ) {
    debug_only( counter++ );
    debug_only( _delete_probes++ );
    k = _table[key];            // Get hashed value
    if( !k ) {                  // Miss?
      debug_only( _delete_misses++ );
#ifdef ASSERT
      if( VerifyOpto ) {
        for( uint i=0; i < _max; i++ )
          assert( _table[i] != n, "changed edges with rehashing" );
      }
#endif
      return false;             // Miss! Not in chain
    }
    else if( n == k ) {
      debug_only( _delete_hits++ );
      _table[key] = _sentinel;  // Hit! Label as deleted entry
      return true;
    }
    else {
      // collision: move through table with prime offset
      key = (key + stride/*7*/) & (_max-1);
      assert( counter <= _insert_limit, "Cycle in hash-table");
    }
  }
  ShouldNotReachHere();
  return false;
}

//------------------------------round_up---------------------------------------
// Round up to nearest power of 2
uint NodeHash::round_up( uint x ) {
  x += (x>>2);                  // Add 25% slop
  if( x <16 ) return 16;        // Small stuff
  uint i=16;
  while( i < x ) i <<= 1;       // Double to fit
  return i;                     // Return hash table size
}

//------------------------------grow-------------------------------------------
// Grow _table to next power of 2 and insert old entries
void  NodeHash::grow() {
  // Record old state
  uint   old_max   = _max;
  Node **old_table = _table;
  // Construct new table with twice the space
  _grows++;
  _total_inserts       += _inserts;
  _total_insert_probes += _insert_probes;
  _inserts         = 0;
  _insert_probes   = 0;
  _max     = _max << 1;
  _table   = NEW_ARENA_ARRAY( _a , Node* , _max ); // (Node**)_a->Amalloc( _max * sizeof(Node*) );
  memset(_table,0,sizeof(Node*)*_max);
  _insert_limit = insert_limit();
  // Insert old entries into the new table
  for( uint i = 0; i < old_max; i++ ) {
    Node *m = *old_table++;
    if( !m || m == _sentinel ) continue;
    hash_insert(m);
  }
}

//------------------------------clear------------------------------------------
// Clear all entries in _table to NULL but keep storage
void  NodeHash::clear() {
  memset( _table, NULL, _max * sizeof(Node*) );
}

#ifndef PRODUCT  
//------------------------------dump-------------------------------------------
// Dump statistics for the hash table
void NodeHash::dump() {
  _total_inserts       += _inserts;
  _total_insert_probes += _insert_probes;
  if( PrintCompilation && PrintOptoStatistics && (_inserts > 0) ) { // PrintOptoGVN
    if( PrintCompilation2 ) {
      for( uint i=0; i<_max; i++ )
      if( _table[i] ) 
        tty->print("%d/%d/%d ",i,_table[i]->hash()&(_max-1),_table[i]->_idx);
    }
    tty->print("\nGVN Hash stats:  %d grows to %d max_size\n", _grows, _max);
    tty->print("  %d/%d (%8.1f%% full)\n", _inserts, _max, (double)_inserts/_max*100.0);
    tty->print("  %dp/(%dh+%dm) (%8.2f probes/lookup)\n", _look_probes, _lookup_hits, _lookup_misses, (double)_look_probes/(_lookup_hits+_lookup_misses));
    tty->print("  %dp/%di (%8.2f probes/insert)\n", _total_insert_probes, _total_inserts, (double)_total_insert_probes/_total_inserts);
    // sentinels increase lookup cost, but not insert cost
    assert((_lookup_misses+_lookup_hits)*4+100 >= _look_probes, "bad hash function");
    assert( _inserts+(_inserts>>3) < _max, "table too full" );
    assert( _inserts*3+100 >= _insert_probes, "bad hash function" );
  }
}

Node *NodeHash::find_index(uint idx) { // For debugging
  // Find an entry by its index value
  for( uint i = 0; i < _max; i++ ) {
    Node *m = _table[i];
    if( !m || m == _sentinel ) continue;
    if( m->_idx == (uint)idx ) return m;
  }
  return NULL;
}
#endif


//=============================================================================
//------------------------------PhaseTransform---------------------------------
PhaseTransform::PhaseTransform( PhaseNumber pnum ) : Phase(pnum), 
  _arena(Thread::current()->resource_area()),
  _nodes(_arena),
  _types(_arena)
{
#ifndef PRODUCT
  clear_progress();
  clear_transforms();
  set_allow_progress(true);
#endif
  // Force allocation for currently existing nodes
  _types.map( C->_unique, NULL );
}

//------------------------------PhaseTransform---------------------------------
PhaseTransform::PhaseTransform( Arena *arena, PhaseNumber pnum ) : Phase(pnum), 
  _arena(arena),
  _nodes(arena),
  _types(arena)
{
#ifndef PRODUCT
  clear_progress();
  clear_transforms();
  set_allow_progress(true);
#endif
  // Force allocation for currently existing nodes
  _types.map( C->_unique, NULL );
}

//------------------------------PhaseTransform---------------------------------
// Initialize with previously generated type information
PhaseTransform::PhaseTransform( PhaseTransform *pt, PhaseNumber pnum ) : Phase(pnum), 
  _arena(pt->_arena),
  _nodes(pt->_nodes),
  _types(pt->_types)
{
#ifndef PRODUCT
  clear_progress();
  clear_transforms();
  set_allow_progress(true);
#endif
}

int PhaseTransform::eqv( const Node *, const Node * ) const { 
  ShouldNotCallThis();
  return 0;                     // Never reached
}

#ifndef PRODUCT
void PhaseTransform::dump_old2new_map() const {
  _nodes.dump();
}

void PhaseTransform::dump_new( uint nidx ) const {
  for( uint i=0; i<_nodes.Size(); i++ )
    if( _nodes[i] && _nodes[i]->_idx == nidx ) {
      _nodes[i]->dump();
      tty->cr();
      tty->print_cr("Old index= %d",i);
      return;
    }
  tty->print_cr("Node %d not found in the new indices", nidx);
}

//------------------------------dump_types-------------------------------------
void PhaseTransform::dump_types( ) const {
  _types.dump();
}

//------------------------------dump_nodes_and_types---------------------------
void PhaseTransform::dump_nodes_and_types(const Node *root, uint depth, bool only_ctrl) {
  VectorSet visited(Thread::current()->resource_area());
  dump_nodes_and_types_recur( root, depth, only_ctrl, visited );
}

//------------------------------dump_nodes_and_types_recur---------------------
void PhaseTransform::dump_nodes_and_types_recur( const Node *n, uint depth, bool only_ctrl, VectorSet &visited) {
  if( !n ) return;
  if( depth == 0 ) return;
  if( visited.test_set(n->_idx) ) return;
  for( uint i=0; i<n->len(); i++ ) {
    if( only_ctrl && !(n->is_Region()) && i != TypeFunc::Control ) continue;
    dump_nodes_and_types_recur( n->in(i), depth-1, only_ctrl, visited );
  }
  n->dump();
  if( _types[n->_idx] != NULL ) {
    tty->print("      "); _types[n->_idx]->dump(); tty->cr();
  }
}

#endif


//=============================================================================
//------------------------------PhaseValues------------------------------------
// Set minimum table size to "255"
PhaseValues::PhaseValues( Arena *arena, uint est_max_size ) : PhaseTransform(arena, GVN), _table(arena, est_max_size) {
  memset(_icons,0,sizeof(_icons));
  NOT_PRODUCT( clear_new_values(); )
}

//------------------------------PhaseValues------------------------------------
// Set minimum table size to "255"
PhaseValues::PhaseValues( PhaseValues *ptv ) : PhaseTransform( ptv, GVN ),
  _table(&ptv->_table) {
  memset(_icons,0,sizeof(_icons));
  NOT_PRODUCT( clear_new_values(); )
}

//------------------------------~PhaseValues-----------------------------------
PhaseValues::~PhaseValues() {
#ifndef PRODUCT
  _table.dump();

  // Statistics for value progress and efficiency
  if( PrintCompilation && Verbose ) {
    tty->print("\n%sValues: %d nodes ---> %d/%d (%d)",
      is_IterGVN() ? "Iter" : "    ", C->_unique, made_progress(), made_transforms(),_count_new_values);
    if( made_transforms() != 0 ) { 
      tty->print_cr("  ratio %f", made_progress()/(float)made_transforms() );
    } else {
      tty->cr();
    }
  }
#endif
}

//------------------------------makecon----------------------------------------
// Make an idealized constant - one of ConINode, ConPNode, etc.
ConNode *PhaseValues::makecon( const Type *t ) {
  assert( t->singleton(), "must be a constant" );
  if( t == Type::TOP ) return (ConNode*)C->_top_node;
  ConNode *x;
  switch( t->_base ) {
  case Type::Int:      return PhaseValues::intcon( t->is_int()->get_con() );
  case Type::AnyPtr:
  case Type::RawPtr:
  case Type::OopPtr: 
  case Type::KlassPtr:
  case Type::AryPtr:   x = new ConPNode   ( t->is_ptr()            ); break;
  case Type::Half:
  case Type::Top:      x = new ConNode    ( Type::TOP              ); break;
  case Type::FloatCon: x = new ConFNode   ( t->is_float_constant() ); break;
  case Type::DoubleCon:x = new ConDNode   ( t->is_double_constant()); break;
  case Type::Long:     x = new ConLNode   ( t->is_long()           ); break;
  default:             ShouldNotReachHere();
  }

  ConNode *k = (ConNode*)hash_find_insert(x); // Value numbering
  if( !k ) {
    _types.map( x->_idx, t );   // Missed, provide type mapping
  } else {
    x->destruct();              // Hit, destroy duplicate constant
    x = k;                      // use existing constant
  }
  return x;
}

//------------------------------intcon-----------------------------------------
// Fast integer constant.  Same as "transform(new ConINode(TypeInt::make(i)))"
ConINode *PhaseValues::intcon( int i ) {
  // Small integer?  Check cache! Check that cached node is not dead
  if( i >= -4 && i <= 32 && _icons[i+4] && (_icons[i+4]->in(TypeFunc::Control) != NULL) )
    return _icons[i+4];
  const TypeInt *t = TypeInt::make(i);
  ConINode *n = new (1) ConINode(t);
  ConINode *k = (ConINode*)hash_find_insert(n); // Insert into value-table if new
  if( k == NULL ) {
    _types.map( n->_idx, t );   // Missed, provide type mapping
  } else {
    n->destruct();              // Hit, destroy duplicate constant
    n = k;                      // use existing constant
  }
  if( i >= -4 && i <= 32 )      // Cache small integers
    _icons[i+4] = n;
  return n;
}

//------------------------------eqv--------------------------------------------
int PhaseValues::eqv( const Node *n1, const Node *n2 ) const {
  return n1 == n2;
}


//=============================================================================
//------------------------------transform--------------------------------------
// Return a node which computes the same function as this node, but in a
// faster or cheaper fashion.  The Node passed in here must have no other 
// pointers to it, as its storage will be reclaimed if the Node can be
// optimized away.
Node *PhaseGVN::transform( Node *n ) {
  NOT_PRODUCT( set_transforms(); )
  
  // Apply the Ideal call in a loop until it no longer applies
  Node *k = n;
  NOT_PRODUCT( int loop_count = 0; )
  while( 1 ) {
    Node *i = k->Ideal(this,NULL);
    if( !i ) break;
    assert( i->_idx >= k->_idx, "Idealize should return new nodes, use Identity to return old nodes" );
    // Can never reclaim storage for Ideal calls, because the Ideal call
    // returns a new Node, bumping the High Water Mark and our old Node
    // is caught behind the new one.
    //if( k != i ) {
    //k->destruct();            // Reclaim storage for recent node
    k = i;
    //}
    assert(loop_count++ < K, "infinite loop in PhaseGVN::transform");
  }
  NOT_PRODUCT( if( loop_count != 0 ) { set_progress(); } )


  // Since I just called 'Value' to compute the set of run-time values
  // for this Node, and 'Value' is non-local (and therefore expensive) I'll
  // cache Value.  Later requests for the local phase->type of this Node can
  // use the cached Value instead of suffering with 'bottom_type'.
  const Type *t = k->Value(this); // Get runtime Value set
  if( t != _types[k->_idx] ) {
#ifndef PRODUCT
    debug_only(++_count_new_values;)
    set_progress();
#endif
    _types.map( k->_idx, t );
    // If k is a TypeNode, capture any more-precise type permanently into Node
    TypeNode *tn = k->is_Type();
    if( tn ) tn->_type = t;
  }

  if( t->singleton() && !k->is_Con() ) {
    //k->destruct();              // Reclaim storage for recent node
    NOT_PRODUCT( set_progress(); )
    return makecon(t);          // Turn into a constant
  }

  // Now check for Identities
  Node *i = k->Identity(this);  // Look for a nearby replacement
  if( i != k ) {                // Found? Return replacement!
    //k->destruct();              // Reclaim storage for recent node
    NOT_PRODUCT( set_progress(); )
    return i;
  }

  // Try Global Value Numbering
  i = hash_find_insert(k);      // Found older value when i != NULL
  if( i && i != k ) {           // Hit? Return the old guy
    NOT_PRODUCT( set_progress(); )
    return i;
  }

  // Return Idealized original
  return k;
}

//------------------------------transform--------------------------------------
// Return a node which computes the same function as this node, but
// in a faster or cheaper fashion.  
Node *PhaseGVN::transform_no_reclaim( Node *n ) {
  NOT_PRODUCT( set_transforms(); )

  // Apply the Ideal call in a loop until it no longer applies
  Node *k = n;
  NOT_PRODUCT( int loop_count = 0; )
  while( 1 ) {
    Node *i = k->Ideal(this,NULL);
    if( !i ) break;
    assert( i->_idx >= k->_idx, "Idealize should return new nodes, use Identity to return old nodes" );
    k = i;
    assert(loop_count++ < K, "infinite loop in PhaseGVN::transform");
  }
  NOT_PRODUCT( if( loop_count != 0 ) { set_progress(); } )


  // Since I just called 'Value' to compute the set of run-time values
  // for this Node, and 'Value' is non-local (and therefore expensive) I'll
  // cache Value.  Later requests for the local phase->type of this Node can
  // use the cached Value instead of suffering with 'bottom_type'.
  const Type *t = k->Value(this); // Get runtime Value set
  if( t != _types[k->_idx] ) {
#ifndef PRODUCT
    debug_only(++_count_new_values;)
    set_progress();
#endif
    _types.map( k->_idx, t );
    // If k is a TypeNode, capture any more-precise type permanently into Node
    TypeNode *tn = k->is_Type();
    if( tn ) tn->_type = t;
  }

  if( t->singleton() && !k->is_Con() ) {
    NOT_PRODUCT( set_progress(); )
    return makecon(t);          // Turn into a constant
  }

  // Now check for Identities
  Node *i = k->Identity(this);  // Look for a nearby replacement
  if( i != k ) {                // Found? Return replacement!
    NOT_PRODUCT( set_progress(); )
    return i;
  }

  // Global Value Numbering
  i = hash_find_insert(k);      // Insert if new
  if( i && (i != k) ) {
    // Return the pre-existing node
    NOT_PRODUCT( set_progress(); )
    return i;
  }

  // Return Idealized original
  return k;
}


//=============================================================================
//------------------------------PhaseIterGVN-----------------------------------
// Initialize with previous PhaseIterGVN info; used by PhaseCCP
PhaseIterGVN::PhaseIterGVN( PhaseIterGVN *igvn ) : PhaseGVN(igvn), 
  _worklist( igvn->_worklist ), 
  _du( igvn->_du ) {

  // Ensure def-use edge from 'root' to 'top'
  if( C->_top_node->in(0) &&
      !_du.def_use_exists( C->_root, C->_top_node ) )
    _du.insert_use( C->_root, C->_top_node );
}

//------------------------------PhaseIterGVN-----------------------------------
// Initialize with previous PhaseGVN info from Parser
PhaseIterGVN::PhaseIterGVN( PhaseGVN *gvn ) : PhaseGVN(gvn), 
  _worklist(*C->get_for_igvn()),
  _du(Thread::current()->resource_area(), C->_unique, C->_root) {
  // TEMPORARY: use thread's resource area for new data structures
  _arena = Thread::current()->resource_area();

  // Ensure def-use edge from 'root' to 'top'
  if( C->_top_node->in(0) &&
      !_du.def_use_exists( C->_root, C->_top_node ) )
    _du.insert_use( C->_root, C->_top_node );

  // Dead nodes in the hash table inherited from GVN were not treated as
  // roots during def-use info creation; hence they represent an invisible
  // use.  Clear them out.
  uint max = _table.size();
  for( uint i = 0; i < max; ++i ) {
    Node *n = _table.at(i);
    if( n && n != _table.sentinel() && !_du.cnt(n) )
      hash_delete(n);
  }

  // Any Phis or Regions on the worklist probably had uses that could not
  // make more progress because the uses were made while the Phis and Regions 
  // were in half-built states.  Put all uses of Phis and Regions on worklist.
  max = _worklist.size();
  for( uint j = 0; j < max; j++ ) {
    Node *n = _worklist.at(j);
    uint uop = n->Opcode();
    if( uop == Op_Phi || uop == Op_Region ||
        n->is_Type() ||
        n->is_Mem() )
      add_users_to_worklist(n);
  }
}


#ifndef PRODUCT
void PhaseIterGVN::verify_step(Node* n) {
  _verify_window[_verify_counter % _verify_window_size] = n;
  ++_verify_counter;
  VectorSet old_space(Thread::current()->resource_area()), new_space(Thread::current()->resource_area());
  if (C->_unique < 1000 ||
      0 == _verify_counter % (C->_unique < 10000 ? 10 : 100)) {
    ++_verify_full_passes;
    Node::verify_recur( C->_root, -1, &_du, old_space, new_space );
  }
  const verify_depth = 4;
  for ( int i = 0; i < _verify_window_size; i++ ) {
    Node* n = _verify_window[i];
    if ( n == NULL )  continue;
    if( n->in(0) == (Node*)-1 ) {  // xform_idom
      _verify_window[i] = n->in(1);
      --i; continue;
    }
    // Typical fanout is 1-2, so this call visits about 6 nodes.
    Node::verify_recur( n, verify_depth, &_du, old_space, new_space );
  }
}
#endif


//------------------------------init_worklist----------------------------------
// Initialize worklist for each node.
void PhaseIterGVN::init_worklist( Node *n ) {
  if( _worklist.member(n) ) return;
  _worklist.push(n);
  uint cnt = n->req();
  for( uint i =0 ; i < cnt; i++ ) {
    Node *m = n->in(i);
    if( m ) init_worklist(m);
  }
}

//------------------------------optimize---------------------------------------
void PhaseIterGVN::optimize() {
  debug_only(uint num_processed  = 0;);
#ifndef PRODUCT
  {
    _verify_counter = 0;
    _verify_full_passes = 0;
    for ( int i = 0; i < _verify_window_size; i++ ) {
      _verify_window[i] = NULL;
    }
  }
#endif

  // Pull from worklist; transform node; 
  // If node has changed: update edge info and put uses on worklist.
  while( _worklist.size() ) {
    Node *n  = _worklist.pop();
    if( TraceIterativeGVN ) {
      tty->print("  Pop ");
      NOT_PRODUCT( n->dump(); )
      debug_only(if( (num_processed++ % 100) == 0 ) _worklist.print_set();)
    }

    if( _du.cnt(n) != 0 ) {
      Node *nn = transform_old(n);
      if( VerifyIterativeGVN && nn != n ) {
#ifndef PRODUCT
	verify_step((Node*) NULL);  // ignore n, it might be subsumed
#endif
      }
    } else {
      remove_dead_node(n);
    }
  }

#ifndef PRODUCT
  if( VerifyOpto && allow_progress() ) {
    // Must turn off allow_progress to enable assert and break recursion
    C->_root->verify(&_du);
    { // Check if any progress was missed using IterGVN
      // Def-Use info enables transformations not attempted in wash-pass
      // e.g. Region/Phi cleanup, ...
      // Null-check elision -- may not have reached fixpoint
      //                       do not propagate to dominated nodes
      ResourceMark rm;
      PhaseIterGVN igvn2(this);
      // Fill worklist completely
      igvn2._worklist.clear();
      igvn2.init_worklist(C->_root);
      // Rebuild DU info completely
      PhaseDefUse du((C->_unique + (C->_unique >> 1)),C->_root);
      igvn2._du = du;

      igvn2.set_allow_progress(false);
      igvn2.optimize();
      igvn2.set_allow_progress(true);
    } 
  }
  if ( VerifyIterativeGVN && PrintOpto ) {
    if ( _verify_counter == _verify_full_passes )
      tty->print_cr("VerifyIterativeGVN: %d transforms and verify passes",
		    _verify_full_passes);
    else
      tty->print_cr("VerifyIterativeGVN: %d transforms, %d full verify passes",
		  _verify_counter, _verify_full_passes);
  }
#endif
}


//------------------register_new_node_with_optimizer---------------------------
// Register a new node with the optimizer.  Update the types array, the def-use
// info.  Put on worklist.
Node *PhaseIterGVN::register_new_node_with_optimizer( Node *n ) {
  _types.map(n->_idx,n->bottom_type());
  _du.grow_nodes(n->_idx);
  // Put in initial def-use info
  for( uint i = 0; i < n->req(); i++ )
    if( n->in(i) )
      _du.insert_use( n->in(i), n );
  _worklist.push(n);
  return n;
}

//------------------------------transform--------------------------------------
// Non-recursive: idealize Node 'n' with respect to its inputs and its value
Node *PhaseIterGVN::transform( Node *n ) {
  // If brand new node, make space in DU array.  I'm using square braces
  // instead of fast_lookup so I get a NULL back instead of a range-check
  // error when doing the array lookup.
  if( !_types[n->_idx] ) {
    register_new_node_with_optimizer( n );
    _worklist.remove(n);
  }

  return transform_old(n);
}

//------------------------------transform_old----------------------------------
Node *PhaseIterGVN::transform_old( Node *n ) {
#ifndef PRODUCT
  debug_only(int loop_count = 0;);
  set_transforms();
#endif
  // Remove 'n' from hash table in case it gets modified
  _table.hash_delete(n);
  if( VerifyIterativeGVN ) {
   assert( !_table.find_index(n->_idx), "found duplicate entry in table");
  }

  // Apply the Ideal call in a loop until it no longer applies
  Node *k = n;
  Node *i = k->Ideal(this,&_du);
#ifndef PRODUCT
  if( VerifyIterativeGVN )
    verify_step(k);
  if( i && VerifyOpto ) {
    if( !allow_progress() ) {
      if( i->is_Add() && (_du.cnt(i) == 1) ) {
        // Switched input to left side because this is the only use
      } else if( i->is_If() && (i->in(0) == NULL) ) {
        // This IF is dead because it is dominated by an equivalent IF When
        // dominating if changed, info is not propagated sparsely to 'this'
        // Propagating this info further will spuriously identify other
        // progress.
        return i;
      } else
        set_progress();
    } else
      set_progress();
  }
#endif

  while( i ) {
#ifndef PRODUCT
    debug_only( if( loop_count >= K ) i->dump(4); )
    assert(loop_count < K, "infinite loop in PhaseGVN::transform");
    debug_only( loop_count++; )
#endif
    assert( i->_idx >= k->_idx, "Idealize should return new nodes, use Identity to return old nodes" );
    // Made a change; put users of original Node on worklist
    add_users_to_worklist( k );
    // Replacing root of transform tree?  
    if( k != i ) {
      // Since root is new, add def-use to his inputs
      for( uint j = 0; j < i->req(); j++ )
        if( i->in(j) )
          _du.insert_use( i->in(j), i );
      // Make users of old Node now use new.
      subsume_node( k, i );
      k = i;
    }
    // Try idealizing again
    i = k->Ideal(this,&_du);
#ifndef PRODUCT
    if( VerifyIterativeGVN )
      verify_step(k);
    if( i && VerifyOpto ) set_progress();
#endif
  }

  // See what kind of values 'k' takes on at runtime
  const Type *t = k->Value(this);
  
  // Since I just called 'Value' to compute the set of run-time values
  // for this Node, and 'Value' is non-local (and therefore expensive) I'll
  // cache Value.  Later requests for the local phase->type of this Node can
  // use the cached Value instead of suffering with 'bottom_type'.
  const Type *old_t = _types[k->_idx];
  if( t != old_t ) {
    NOT_PRODUCT( set_progress(); )
    debug_only(++_count_new_values;)
    _types.map( k->_idx, t );
    // If k is a TypeNode, capture any more-precise type permanently into Node
    TypeNode *tn = k->is_Type();
    if( tn ) tn->_type = t;
    // Move users of node to worklist
    add_users_to_worklist( k );
  }

  // If 'k' computes a constant, replace it with a constant
  if( t->singleton() && !k->is_Con() ) {
    NOT_PRODUCT( set_progress(); )
    Node *con = makecon(t);     // Make a constant
    add_users_to_worklist( k );
    subsume_node( k, con );     // Everybody using k now uses con
    return con;
  } 

  // Now check for Identities
  i = k->Identity(this);        // Look for a nearby replacement
  if( i != k ) {                // Found? Return replacement!
    NOT_PRODUCT( set_progress(); )
    add_users_to_worklist( k );
    subsume_node( k, i );       // Everybody using k now uses i
    return i;
  }

  // Global Value Numbering
  i = hash_find_insert(k);      // Check for pre-existing node
  if( i && (i != k) ) {
    // Return the pre-existing node if it isn't dead
    NOT_PRODUCT( set_progress(); )
    add_users_to_worklist( k );
    subsume_node( k, i );       // Everybody using k now uses i
    return i;
  }

  // Return Idealized original
  return k;
}

//------------------------------remove_globally_dead_node----------------------
// Kill a globally dead Node.  All uses are also globally dead and are
// aggressively trimmed.
void PhaseIterGVN::remove_globally_dead_node( Node *dead ) {
  assert( dead != C->_root, "killing root, eh?" );
  if( dead == C->_top_node ) return;
  NOT_PRODUCT( set_progress(); )
  // Remove from iterative worklist
  _worklist.remove(dead);
  // Remove from hash table
  _table.hash_delete( dead );
  // Smash all inputs to 'dead', isolating him completely
  for( uint i = 0; i < dead->req(); i++ ) {
    Node *in = dead->in(i);
    if( in ) {                 // Points to something?
      _du.delete_use_attempt( in, dead );
      dead->set_req(i,NULL);  // Kill the edge
      if( _du.cnt(in) == 0 && in != C->_top_node) {// Made input go dead?
	remove_dead_node(in); // Recursively remove
      } else if( _du.cnt(in) <= 2 && dead->is_Phi() ) {
	if( in->Opcode() == Op_Region )
	  _worklist.push(in);
	else if( in->is_Store() ) {
	  _worklist.push( _du.out(in)[0] );
	  if(_du.cnt(in) == 2 )
	    _worklist.push( _du.out(in)[1] );
	}
      }
    }
  }

  // Aggressively kill globally dead uses
  while( _du.cnt(dead) )
    remove_globally_dead_node( _du.out(dead)[0] );
}

//------------------------------subsume_node-----------------------------------
// Remove users from node 'old' and add them to node 'nn'.
void PhaseIterGVN::subsume_node( Node *old, Node *nn ) {
  assert( old != hash_find(old), "should already been removed" );
  // Guaranty that the possibly new node 'nn' has an entry in PhaseDefUse
  _du.grow_nodes(nn->_idx);
  // Move users of node 'old' to node 'nn'
  uint max = _du.cnt(old);      // Number of uses
  uint i;
  for( i=0; i<max; i++ ) { // For all uses
    Node *use = _du.out(old)[i];
    // Add use to 'nn'
    _du.insert_use(nn,use);
    // use will need re-hashing
    _table.hash_delete( use );
    // Update use-def info as well
    for( uint j=0; j<use->len(); j++ ) {
      if( use->in(j) == old ) {
        use->set_req(j,nn);
      }
    }
    // Insert into GVN hash table if unique
    // If a duplicate, 'use' will be cleaned up when pulled off worklist
    hash_find_insert(use);
  }
  // Remove all uses from 'old'
  _du.delete_all_uses(old);

  // Smash all inputs to 'old', isolating him completely
  _du.insert_use(nn,nn);        // Add a cycle to nn to prevent him from dying
  remove_dead_node( old );
  _du.delete_use(nn,nn);        // Yank bogus edge
#ifndef PRODUCT
  if( VerifyIterativeGVN ) {
    for ( int i = 0; i < _verify_window_size; i++ ) {
      if ( _verify_window[i] == old )
	_verify_window[i] = nn;
    }
  }
#endif
}

//------------------------------add_users_to_worklist--------------------------
void PhaseIterGVN::add_users_to_worklist0( Node *n ) {
  uint max    = _du.cnt(n);     // Number of uses
  Node **outp = _du.out(n);
  for( uint i=0; i<max; i++ )   // For all uses
    _worklist.push( outp[i] );  // Push on worklist
}

void PhaseIterGVN::add_users_to_worklist( Node *n ) {
  add_users_to_worklist0(n);

  // Move users of node to worklist
  uint max    = _du.cnt(n);    // Number of uses
  Node **outp = _du.out(n);
  for( uint i=0; i<max; i++ ) { // For all uses
    Node *use = outp[i];        // Get use

    if( use->is_Multi() ||      // Multi-definer?  Push projs on worklist
        use->is_Store() )       // Enable store/load same address
      add_users_to_worklist0(use); 

    if( use->is_Cmp() ) {       // Enable CMP/BOOL optimization
      add_users_to_worklist(use); // Put Bool on worklist
      // Look for the 'is_x2logic' pattern: "x ? : 0 : 1" and put the
      // phi merging either 0 or 1 onto the worklist
      if( _du.cnt(use) ) {
        Node *bol = _du.out(use)[0];
        if( _du.cnt(bol) ) {
          Node *iff = _du.out(bol)[0];
          if( _du.cnt(iff) == 2 ) {
            Node *ifproj0 = _du.out(iff)[0];
            Node *ifproj1 = _du.out(iff)[1];
            if( _du.cnt(ifproj0) && _du.cnt(ifproj1) ) {
              Node *region0 = _du.out(ifproj0)[0];
              Node *region1 = _du.out(ifproj1)[0];
              if( region0 == region1 )
                add_users_to_worklist0(region0);
            }
          }
        }
      }
    }
  
    uint use_op = use->Opcode();
    // If changed Cast input, check Phi users for simple cycles
    if( use_op == Op_CastPP ) {
      uint max2    = _du.cnt(use);    // Number of uses
      Node **outp2 = _du.out(use);
      for( uint i2=0; i2<max2; i2++ ) 
        if( outp2[i2]->is_Phi() )
          _worklist.push( outp2[i2] );
    }
    // If changed LShift inputs, check RShift users for useless sign-ext
    if( use_op == Op_LShiftI ) {
      uint max2    = _du.cnt(use);    // Number of uses
      Node **outp2 = _du.out(use);
      for( uint i2=0; i2<max2; i2++ ) 
        if( outp2[i2]->Opcode() == Op_RShiftI )
          _worklist.push( outp2[i2] );
    }
    // If changed AddP inputs, check Stores for loop invariant
    if( use_op == Op_AddP ) {
      uint max2    = _du.cnt(use);    // Number of uses
      Node **outp2 = _du.out(use);
      for( uint i2=0; i2<max2; i2++ ) 
        if( outp2[i2]->is_Mem() )
          _worklist.push( outp2[i2] );
    }
  }
}

//------------------------------makecon----------------------------------------
// Make an idealized constant, i.e., one of ConINode, ConPNode, ConFNode, etc
ConNode *PhaseIterGVN::makecon( const Type *t ) {
  if( t == Type::TOP ) return (ConNode*)C->_top_node;
  uint unique_hwm = C->_unique;
  ConNode *con = PhaseValues::makecon(t);
  _du.grow_nodes(con->_idx);
  if( con->_idx == unique_hwm ) _du.insert_use( C->_root, con );
  return con;
}

//------------------------------intcon-----------------------------------------
// Fast integer constant. Same as "transform(new ConINode(TypeInt::make(i)))"
ConINode *PhaseIterGVN::intcon( int i ) {
  uint unique_hwm = C->_unique;
  ConINode *con = PhaseValues::intcon(i);
  _du.grow_nodes(con->_idx);
  if( con->_idx == unique_hwm ) _du.insert_use( C->_root, con );
  return con;
}


//=============================================================================
#ifndef PRODUCT
uint PhaseCCP::_total_invokes   = 0;
uint PhaseCCP::_total_constants = 0;
#endif
//------------------------------PhaseCCP---------------------------------------
// Conditional Constant Propagation, ala Wegman & Zadeck
PhaseCCP::PhaseCCP( PhaseIterGVN *igvn ) : PhaseIterGVN(igvn) {
  NOT_PRODUCT( clear_constants(); )
  assert( _worklist.size() == 0, "" );
  // Clear out _nodes from IterGVN.  Must be clear to transform call.
  _nodes.clear();               // Clear out from IterGVN
  analyze();
}

//------------------------------~PhaseCCP--------------------------------------
PhaseCCP::~PhaseCCP() {
#ifndef PRODUCT
  inc_invokes();
  _total_constants += count_constants();
#endif
}


//------------------------------analyze----------------------------------------
void PhaseCCP::analyze() {
  // Initialize all types to TOP, optimistic analysis
  for( int i = C->_unique - 1; i >= 0; i-- ) 
    _types.map(i,Type::TOP);

  // Push root onto worklist
  Unique_Node_List worklist;
  worklist.push(C->_root);

  // Pull from worklist; compute new value; push changes out.
  // This loop is the meat of CCP.
  while( worklist.size() ) {
    Node *n = worklist.pop();
    const Type *t = n->Value(this);
    if( t != type(n) ) {
      assert( t->meet(type(n)) == t, "Not monotonic" );
      if( TracePhaseCCP ) {
#ifndef PRODUCT
        t->dump();
        tty->print(" ");
        n->dump();
#endif
      }
      _types.map(n->_idx,t);
      uint   max  = _du.cnt(n); // Number of uses
      Node **outp = _du.out(n); // Start of uses
      for( uint i=0; i<max; i++ ) { // For all uses, propagate changes
        Node *m = *outp++;      // Get user
        if( m->is_Region() ) {  // New path to Region?  Must recheck Phis too
          uint max2 = _du.cnt(m);       // Number of uses
          Node **outp2 = _du.out(m);    // Start of uses
          for( uint i2=0; i2<max2; i2++ ) { // For all uses, propagate changes
            Node *p = *outp2++;
            worklist.push(p);   // Propagate change to user
          }
        }
        // If we changed the reciever type to a call, we need to revisit
        // the Catch following the call.  It's looking for a non-NULL
        // receiver to know when to enable the regular fall-through path
        // in addition to the NullPtrException path
        CallNode *call = m->is_Call();
        if( call ) {
          uint   max2  = _du.cnt(call);    // Number of uses
          Node **outp2 = _du.out(call);    // Start of uses
          for( uint i2=0; i2 < max2; i2++){// For all uses, propagate changes
            ProjNode *p = outp2[i2]->is_Proj();
            if( p && p->_con == TypeFunc::Control && _du.cnt(p) == 1 )
              worklist.push(_du.out(p)[0]);
          }
        }
        worklist.push(m);       // Propagate change to user
      }
    }
  }
}

//------------------------------do_transform-----------------------------------
// Top level driver for the recursive transformer
void PhaseCCP::do_transform() {
  // Correct leaves of new-space Nodes; they point to old-space.
  C->_root     = transform( C->_root     )->is_Root();
  assert( C->_top_node, "missing TOP node" );
  assert( C->_root, "missing root" );
}

//------------------------------transform--------------------------------------
// Given a Node in old-space, clone him into new-space.
// Convert any of his old-space children into new-space children.
Node *PhaseCCP::transform( Node *n ) {
  Node *nn = _nodes[n->_idx]; // Check for transformed node
  if( nn != NULL ) return nn; // Been there, done that, return old answer
  nn = transform_once(n);     // Check for constant
  _nodes.map( n->_idx, nn );  // Flag as having been cloned

  uint cnt = nn->req();
  for( uint i=0; i<cnt; i++ ) { // For all inputs do
    Node *m = nn->in(i);        // Get input
    if( !m ) continue;          // Ignore NULLs
    Node *input = transform(m);
    assert( input == nn->in(i), "insanity check");
    nn->set_req(i,input);// Recursively xform inputs to new-space child
  }
  return nn;
}


//------------------------------transform_once---------------------------------
// For PhaseCCP, transformation is IDENTITY unless Node computed a constant.
Node *PhaseCCP::transform_once( Node *n ) {
  const Type *t = _types[n->_idx];
  // Constant?  Use constant Node instead
  if( t->singleton() ) {
    Node *nn = n;               // Default is to return the original constant
    if( t == Type::TOP ) {
      if( C->_top_node == NULL || C->_top_node->in(0) == NULL ) {
        C->_top_node = ConNode::make(Type::TOP);
        _types.map( C->_top_node->_idx, Type::TOP );
      }
      nn = C->_top_node;
    } 
    if( !n->is_Con() ) { 
      add_users_to_worklist(n); // Users of about-to-be-constant 'n'
      nn = makecon(t);          // ConNode::make(t);
      hash_delete(n);           // Removed 'n' from table before subsuming it
      subsume_node(n,nn);       // Update DefUse edges for new constant
      NOT_PRODUCT( inc_constants(); )
    }
    return nn;
  }

  // If x is a TypeNode, capture any more-precise type permanently into Node
  TypeNode *tn = n->is_Type();
  if( tn ) {
    hash_delete(n);
    tn->_type = t;
    hash_insert(n);
  } else {
    MemNode *mn = n->is_Mem();
    if( mn ) {
      hash_delete(n);
      mn->_type = t;
      hash_insert(n);
    }
  }

  // Idealize graph using DU info.  Must clone() into new-space.
  // DU info is generally used to show profitability, progress or safety
  // (but generally not needed for correctness).
  Node *nn = n->Ideal_DU_postCCP(this);

  // TEMPORARY fix to ensure that 2nd GVN pass eliminates NULL checks
  switch( n->Opcode() ) {
  case Op_If:
  case Op_CheckType:
  case Op_CountedLoopEnd:
  case Op_Region:
  case Op_Loop:
  case Op_CountedLoop:
  case Op_Conv2B:
  case Op_Opaque1:
  case Op_Opaque2:
    _worklist.push(n);
    break;
  default:
    break;
  }
  if( nn ) {
    _worklist.push(n);
    // Put users of 'n' onto worklist for second igvn transform
    add_users_to_worklist(n);
    return nn;
  }

  return  n;
}


//------------------------------print_statistics-------------------------------
#ifndef PRODUCT
void PhaseCCP::print_statistics() {
  tty->print_cr("CCP: %d  constants found: %d", _total_invokes, _total_constants);
}
#endif


//=============================================================================
#ifndef PRODUCT
uint PhasePeephole::_total_peepholes = 0;
#endif
//------------------------------PhasePeephole----------------------------------
// Conditional Constant Propagation, ala Wegman & Zadeck
PhasePeephole::PhasePeephole( PhaseChaitin *chaitin, PhaseCFG &cfg )
  : PhaseTransform(Peephole), _chaitin(chaitin), _cfg(cfg) {
  NOT_PRODUCT( clear_peepholes(); )
}

//------------------------------~PhasePeephole---------------------------------
PhasePeephole::~PhasePeephole() {
  NOT_PRODUCT( _total_peepholes += count_peepholes(); )
}

//------------------------------transform--------------------------------------
Node *PhasePeephole::transform( Node *n ) {
  ShouldNotCallThis();
  return NULL;
}

//------------------------------do_transform-----------------------------------
void PhasePeephole::do_transform() {
  bool method_name_not_printed = true;

  // Examine each basic block
  for( uint block_number = 1; block_number < _cfg._num_blocks; ++block_number ) {
    Block *block = _cfg._blocks[block_number];
    bool block_not_printed = true;

    // and each instruction within a block
    uint end_index = block->_nodes.size() - 1; // zero-based 
    // block->end_idx() not valid after PhaseChaitin
    for( uint instruction_index = 1; instruction_index < end_index; ++instruction_index ) {
      Node     *n = block->_nodes.at(instruction_index);
      MachNode *m = n->is_Mach();
      if( m ) {
        int deleted_count = 0;
        // check for peephole opportunities
        MachNode *m2 = m->peephole( block, instruction_index, _chaitin, deleted_count );
        if( m2 != NULL ) {
#ifndef PRODUCT
          if( PrintOptoPeephole ) {
            // Print method, first time only
            if( C->method() && method_name_not_printed ) {
              C->method()->print_short_name(); tty->cr();
              method_name_not_printed = false;
            }
            // Print this block
            if( Verbose && block_not_printed) {
              tty->print_cr("in block");
              block->dump();
              block_not_printed = false;
            }
            // Print instructions being deleted
            for( int i = (deleted_count - 1); i >= 0; --i ) {
              block->_nodes.at(instruction_index-i)->is_Mach()->format(_chaitin); tty->cr();
            }
            tty->print_cr("replaced with");
            // Print new instruction
            m2->format(_chaitin);
            tty->print("\n\n");
          }
#endif
          // Remove old nodes from basic block and update instruction_index
          // (old nodes still exist and may have edges pointing to them
          //  as register allocation info is stored in the allocator using
          //  the node index to live range mappings.)
          uint safe_instruction_index = (instruction_index - deleted_count);
          for( ; (instruction_index > safe_instruction_index); --instruction_index ) {
            block->_nodes.remove( instruction_index );
          }
          // install new node after safe_instruction_index
          block->_nodes.insert( safe_instruction_index + 1, m2 );
          end_index = block->_nodes.size() - 1; // Recompute new block size
          NOT_PRODUCT( inc_peepholes(); )
        }
      }
    }
  }
}

//------------------------------print_statistics-------------------------------
#ifndef PRODUCT
void PhasePeephole::print_statistics() {
  tty->print_cr("Peephole: peephole rules applied: %d",  _total_peepholes);
}
#endif


//=============================================================================
//------------------------------PhaseDCE---------------------------------------
PhaseDCE::PhaseDCE( PhaseNumber pnum ) : PhaseTransform(pnum) {
  // Global analysis goes here.  <empty for DCE>
}

//------------------------------do_transform-----------------------------------
// Top level driver for the recursive transformer
void PhaseDCE::do_transform() {
  // Swap out to old-space; emptying new-space 
  Arena *old = C->_arena.move_contents(&(C->_old_arena));
  C->_unique = 0;                // Reset unique Node counter
  // Correct leaves of new-space Nodes; they point to old-space.
  Node *tmp = C->_top_node;
  C->_top_node = NULL;   // Set to NULL so first TopNode gets constructed
  C->_top_node = transform( tmp );
  C->_root     = transform( C->_root     )->is_Root();
  assert( C->_top_node, "" );
  assert( C->_root, "" );
  // Now smoke old-space
  old->destruct_contents();
}

//------------------------------transform--------------------------------------
// Given a Node in old-space, clone him into new-space.
// Convert any of his old-space children into new-space children.
Node *PhaseDCE::transform( Node *n ) {
  //assert( !C->_arena.contains(n), "must be old-space" );
  Node *nn = n;
  if( !C->_arena.contains(n) ) {
    // Old space node
    nn = _nodes[n->_idx];       // Get new-space transformed Node
    if( nn ) return nn;         // Been there, done that, return old answer
    nn = transform_once(n);     // Tr
    // Put onto worklist for second igvnansform non-recursively
    C->get_for_igvn()->push(nn);
    _nodes.map( n->_idx, nn );  // Flag as having been cloned
  // }

    uint cnt = nn->req();
    for( uint i=0; i<cnt; i++ ) { // For all inputs do
      Node *m = nn->in(i);        // Get input
      if( !m ) continue;          // Ignore NULLs
      nn->set_req(i,transform(m));// Recursively xform inputs to new-space child
    }
  }
  return nn;
}

//------------------------------transform_once---------------------------------
// For PhaseDCE, transformation is always IDENTITY
Node *PhaseDCE::transform_once( Node *n ) {
  return n->clone();            // Clone into new-space
}

//=============================================================================

//=============================================================================
//------------------------------PhaseDefUse------------------------------------
PhaseDefUse::PhaseDefUse( uint unique, Node *root ) : Phase(DefUse),   _a(Thread::current()->resource_area()),
_unique(unique), _max_edges(0), _max_cnts(0), _cur_edges(0), _cur_cnts(unique) {
  compute_defuse_edges(1,&root);
}

PhaseDefUse::PhaseDefUse( uint unique, Node_List &root_set ) : Phase(DefUse),   _a(Thread::current()->resource_area()),
_unique(unique), _max_edges(0), _max_cnts(0), _cur_edges(0), _cur_cnts(unique) { 
  compute_defuse_edges(root_set.size(),root_set.adr()); 
}

PhaseDefUse::PhaseDefUse( Arena *a, uint unique, Node *root ) : Phase(DefUse), _a(a), _unique(unique), _max_edges(0), _max_cnts(0), _cur_edges(0), _cur_cnts(unique) {
  compute_defuse_edges(1,&root);
}

PhaseDefUse::PhaseDefUse( PhaseDefUse *p ) : Phase(DefUse),
  _a(p->_a),         // Arena used for allocation
  _unique(p->_unique),       // Max number of nodes
  _edges(p->_edges),         // Array of def-use edges
  _start(p->_start),         // starting point in _edges: at init, running sum of user counts
  _cnt(p->_cnt),             // _cnt[i] = Number of uses of node with index 'i'
  _max(p->_max),             // _max[i] = Maximum number of uses storable at _edges[start[i]]
  _max_edges(p->_max_edges), // Max space in _edges
  _max_cnts(p->_max_cnts),   // Max space in _start, _cnt, and _max
  _cur_edges(p->_cur_edges), // Currently used space in _edges
  _cur_cnts(p->_cur_cnts) {  // Currently used space in _start, _cnt and _max, max(idx) + 1
  // _nesting(p->_nesting) {    // assertion check for reallocations
}


//------------------------------compute_defuse_edges---------------------------
void PhaseDefUse::compute_defuse_edges( uint root_set_size, Node **root_set ) {
  _max_cnts = _unique + MIN_NODES_GROWTH * 10;
  _cur_cnts = _unique;     // _cur_cnts --> (_cur_cnts + 1)
  _start    = NEW_ARENA_ARRAY( _a , uint  , _max_cnts );
  memset( _start, 0, sizeof(uint) * (_max_cnts) );
  _cnt      = NEW_ARENA_ARRAY( _a , uint  , _max_cnts );
  memset( _cnt,   0, sizeof(uint  )*(_max_cnts) );
  _max      = NEW_ARENA_ARRAY( _a , uint  , _max_cnts );
  memset( _max,   0, sizeof(uint  )*(_max_cnts) );

  // Count users per Node.  Count is stored at the next position down in the
  // _start array, so that if Node 0 has use count "use0" and so on, we get:
  // _start[0] = 0;
  // _start[1] = use0;
  // _start[2] = use1;
  // _start[3] = use2; ...
  VectorSet visited(Thread::current()->resource_area());
  Node_List worklist;
  uint j;
  for( j=0; j<root_set_size; j++ )
    if( root_set[j] ) 
      worklist.push(root_set[j]);
  while( worklist.size() ) {
    Node *n = worklist.pop();
    if( visited.test_set(n->_idx) ) continue;
    uint cnt = n->len();
    for( uint i=0; i<cnt; i++ ) { // For all inputs do
      Node *m = n->in(i);         // Get input
      if( !m ) continue;          // Ignore NULLs
      _start[m->_idx+1]++;        // Bump count of users of 'n'
      worklist.push(m);
    }
  }
 

  // Compute a running sum of edge counts.  This allows a single table
  // of counts to encode both the starting point in the _edge array
  // and the number of outgoing edges.  The picture is now:
  // _start[0] = 0;
  // _start[1] = 0;
  // _start[2] = use0;
  // _start[3] = use0+use1; ...
  uint i,sum = 0;
  for( i=1; i<_unique+1; i++ ) {
    uint tmp = _start[i];
    _start[i] = sum;
    sum += tmp;
  }
  _start[i] = sum;

  // Edge array allocation
  _max_edges = sum + MIN_EDGES_GROWTH * 10;
  _cur_edges = sum;
  _edges = NEW_ARENA_ARRAY( _a , Node* , _max_edges );

  // Fill in edge array.  Bumps the _start to keep track of where to insert
  // in the edge array.  So now we get:
  // _start[0] = 0;
  // _start[1] = use0;
  // _start[2] = use0+use1;
  // _start[3] = use0+use1+use2; ...
  for( j=0; j<root_set_size; j++ )
    if( root_set[j] ) 
      worklist.push(root_set[j]);
  while( worklist.size() ) {
    Node *n = worklist.pop();
    if( !visited[n->_idx] ) continue;
    visited >>= n->_idx;          // Do not visit again
    uint cnt = n->len();
    for( uint i=0; i<cnt; i++ ) { // For all inputs do
      Node *m = n->in(i);         // Get input
      if( !m ) continue;          // Ignore NULLs
      _edges[_start[m->_idx+1]++] = n;
      worklist.push(m);
    }
  }

  // Fill in _cnt and _max array.  Captures the difference: _start[idx+1] - _start[idx]
  for( i=0; i<_unique+1; i++ ) {
    int delta = _start[i+1] - _start[i];
    _cnt[i] = delta;
    _max[i] = delta;
  }
}

//------------------------------def_use_exists--------------------------------
// Return 'true' if a def-use edge exists between 'def' and 'use'
bool PhaseDefUse::def_use_exists( const Node *def, const Node *use ) {
  uint   max  = cnt(def);    // Number of uses
  Node **outp = out(def);    // Start of uses
  for( uint i=0; i<max; i++ ) { // Check if any use matches 'use'
    if( *outp++ == use )  return true;
  }

  return false;
}


//------------------------------set_req_du-------------------------------------
// Change edge 'i' from its old def to the new def 'n'.  Update the DU edges
// as well.  Always remove an edge from 'this' to 'top', but never add any.
// Edges to 'top' are boring and are never inspected.
void Node::set_req_du( uint i, Node *n, PhaseDefUse *du ) {
  Node *old = in(i);
  if( old ) {
    bool found = du->delete_use_attempt( old, this );
    assert( found || old == du->C->_top_node, "missing du edge" );
  }
  _in[i] = n;                   // Range check on 'i' already done
  if( n && n != du->C->_top_node ) du->insert_use( n, this );
}

//------------------------------set_req_X--------------------------------------
void Node::set_req_X( uint i, Node *n, PhaseIterGVN *igvn ) {
  assert( igvn->hash_find(this) != this, "Need to remove from hash before changing edges" );
  Node *old = in(i);
  set_req_du( i, n, &igvn->_du );

  // old goes dead?
  if( old ) {
    switch( igvn->_du.cnt(old) ) {
    case 0:      // Kill all his inputs, and recursively kill other dead nodes.
      igvn->remove_dead_node( old );
      break;
    case 1:
      if( old->is_Store() )
        igvn->add_users_to_worklist( old );
      break;
    case 2:
      if( old->is_Store() )
        igvn->add_users_to_worklist( old );
      if( old->Opcode() == Op_Region )
        igvn->_worklist.push(old);
      break;
    case 3:
      if( old->Opcode() == Op_Region ) {
        igvn->_worklist.push(old);
        igvn->add_users_to_worklist( old );
      }
      break;
    default:
      break;
    }
  }

}

//------------------------------replace_node----------------------------------
// Using def-use info, replace one node for another.  Follow the def-use info
// to all users of the OLD node.  Then make all uses point to the NEW node.
void PhaseDefUse::replace_node( Node *old, Node *nn ) {
  uint max    = cnt(old);       // Number of uses
  Node **outp = out(old);       // Start of uses
  for( uint i=0; i<max; i++ ) { // For all uses
    Node *use = *outp++;
    for( uint j=0; j<use->len(); j++ )
      if( use->in(j) == old )
        use->set_req(j,nn);
  }
}

//------------------------------add_anti_dependence---------------------------
// Append def-use pointer from 'load' to 'store' to load's edges
// because 'load' may be anti-dependent on 'store'
// The 'store' will have a precedence edge to 'load'
void PhaseDefUse::add_anti_dependence( Node *store, Node *load ) {
  store->add_prec(load);
  insert_use( load, store );
}


//------------------------------update_an_edge-------------------------------
// Update at least one edge that went from 'def' to 'old'
// so that it points from 'def' to 'clone'
// If no candidate can be remapped, add a new def-use edge for 'def'
void PhaseDefUse::update_an_edge( Node *def, Node *old, Node *clone ) {
  uint   max  = cnt(def);
  Node **outp = out(def);
  for( uint i=0; i<max; i++ ) { // Find a 'def' to 'old' edge
    if( *outp == clone ) return;  // Already have an edge to 'clone'
    if( *outp == old ) {
      *outp = clone;              // Convert 'old' to 'clone'
      return;
    }
    ++outp;
  }
  // Did not find an edge that could be remapped

  // Shift edges and insert one from 'def' to 'clone'
  insert_use( def, clone );
  // shift_edges( def, clone, 1 );
}

//------------------------------grow_edges------------------------------------
// Grow the size of _edges
void PhaseDefUse::grow_edges( uint count ) {
  _nesting.check();
  if( _cur_edges + count >= _max_edges ) {
    uint new_size = _max_edges;
    new_size     += (_max_edges ) >= count && (_max_edges ) >= MIN_EDGES_GROWTH ?
      _max_edges   :  count + MIN_EDGES_GROWTH;
    _edges = REALLOC_ARENA_ARRAY(_a, Node*, _edges, _max_edges, new_size);
#ifdef ASSERT
    // if( _cur_edges > 40000 ) {
    //   tty->cr(); tty->print_cr("========================================");
    //   tty->print_cr("    PhaseDefUse request grew %d to %d", _max_edges, new_size );
    //   tty->cr(); tty->print_cr("========================================");
    // }
#endif 
    _max_edges = new_size;
  }
}

//------------------------------grow_nodes-------------------------------------
// If needed, add entry to _cnt, _max and _start area for node with index 'idx'
// Entry for 'idx' may have 0 edges according to PhaseDefUse::cnt(idx)
void PhaseDefUse::grow_nodes( uint idx ) {
  _nesting.check();
  if( idx > _cur_cnts ) {
    // assert( idx == (_cur_cnts +1), "Must be simple append to node indices");
    // If needed, add space to _start and _cnt for node 'idx'
    if( idx >= _max_cnts - 1 ) {
      // Grow _start, _max and _cnt array
      uint new_size = max(idx+1,_max_cnts);
      new_size += (new_size >> 4);

      _start = REALLOC_ARENA_ARRAY(_a, uint, _start, _max_cnts, new_size );
      _max   = REALLOC_ARENA_ARRAY(_a, uint, _max,   _max_cnts, new_size );
      _cnt   = REALLOC_ARENA_ARRAY(_a, uint, _cnt,   _max_cnts, new_size );
      // Clear new entries in _start, _max, _cnt.
      for( uint i = _max_cnts - 1; i < new_size; i++ ) {
        _start[i] = 0;
        _max[i]   = 0;
        _cnt[i]   = 0;
      }
      // Record new maximum size
      _max_cnts     = new_size;
    }
    _cur_cnts = idx;
    // --- Old code to clear entries when used
    // // Clear entries for nodes that have been created but not yet used.
    // for( uint i = _cur_cnts+1; i <= idx; i++ ) {
    //   // A new node, Clear the entries for _start, _max, _cnt
    //   _start[i] = 0;
    //   _max[i]   = 0;
    //   _cnt[i]   = 0;
    //   ++_cur_cnts;
    // }
  }
}


//------------------------------shift_edges-----------------------------------
// Shift the edges below those for 'def' to make room for 'count' new edges
// Update the entries in _start
// Fill in _edges with pointers to 'clone'
void PhaseDefUse::shift_edges( Node *def, Node *clone, uint count ) {
  grow_edges( count );
  // Move existing edges
  Node **begin = &(_edges[ start(def->_idx + 1) ]);
  size_t range = (start(_cur_cnts+1) - start(def->_idx + 1)) * sizeof(Node*);
  if( range != 0 )
    memmove( begin + count, begin, range);
  // Update _start after def, including trailing signal
  uint i;
  for( i = (def->_idx + 1); i <= (_cur_cnts+1); i++ ) {
    _start[i] += count;
  }

  // Add the edges from 'def' to 'clone'
  for( i = count; i != 0; --i ) {
    _edges[ start(def->_idx + 1) - i ] = clone;
  }
  _cur_edges += count;

  // Do not need to increment _cnt[def->_idx] by full 'count'
  // Add one new edge, remaining space can be used later.
  ++_cnt[def->_idx];
}


//------------------------------insert_use---------------------------------
// Add a Def-Use edge from 'def' to 'use'
// Update the entries in _start, _cnt, _max; 
void PhaseDefUse::insert_use( Node *def, Node *use ) {
  // If there isn't enough space available
  if( _cnt[def->_idx] >= _max[def->_idx] ) {
    uint current_size = _max[def->_idx];
    uint new_size = current_size + 
      ((current_size>>1 > MIN_USE_GROWTH) ? current_size>>1 : MIN_USE_GROWTH);  // add space for next edge now 
    // Get the space for new edges
    grow_edges(new_size);
    // !!!!! Can store information for reuse of too-be-moved space
    // Move current collection of edges to its new location in _edge[]
    Node **destination  = &(_edges[ _cur_edges ]);
    Node **begin = &(_edges[ start(def->_idx) ]);
    // size_t range = current_size * sizeof(Node*);
    // if( range != 0 ) memmove( destination, begin, range );
    for( Node **end = begin + current_size;  begin != end; ) {
      *destination++ = *begin++;
    }
    _start[def->_idx] = _cur_edges;
    // _cnt[def->_idx];  // update below after inserting pointer to use
    _max[def->_idx]   = new_size;
    _cur_edges       += new_size;
  }

  // add the edge and return
  _edges[_start[def->_idx]+_cnt[def->_idx]] = use;
  _cnt[def->_idx]++;
}

//------------------------------delete_use-------------------------------------
// Search uses of 'def' for first occurrence of'use'
// replace with last entry in list and Decrement the number of uses
void PhaseDefUse::delete_use( Node *def, Node *use ) {
  bool found_use = delete_use_attempt( def, use );
  assert( found_use, "Did not find use");
}

//------------------------------delete_use_attempt-----------------------------
// Search uses of 'def' for first occurrence of'use'
// replace with last entry in list and Decrement the number of uses
bool PhaseDefUse::delete_use_attempt( Node *def, Node *use ) {
  // Find index for first use of 'use' in 'def's info
  Node **outp = out(def->_idx);
  uint   max  = cnt(def->_idx);
  uint i = 0;
  for( ; i < max; i++ ) {
    if( outp[i] == use ) {
      // Replace with last use
      outp[i] = outp[max-1];
      // Decrement count of uses
      _cnt[def->_idx]--;
      return true;
    }
  }
  return false;
}

void PhaseDefUse::delete_all_uses( Node *def ) {
  // Set count of uses for 'def' to zero
  _cnt[def->_idx] = 0;
}


#ifndef PRODUCT
//------------------------------dump-----------------------------------------
// Dump the uses associated with each node
void PhaseDefUse::dump() {
  // For each node index
  for( uint i = 0; i < _cur_cnts; i++ ) {
    Node **outp = out(i);
    uint   max  = cnt(i);
    if( max > 0 ) {
      tty->print("def:%5d uses:", i);
      for( uint j = 0; j < max; j++ ) {
        // This does not remove duplicates
        tty->print(" %d", outp[j]->_idx);
      }
      tty->cr();
    }
  }
}
// Dump the uses for each node, and their inputs to depth 'depth'
void PhaseDefUse::dump(int depth) {
  // For each node index
  for( uint i = 0; i < _cur_cnts; i++ ) {
    Node **outp = out(i);
    uint   max  = cnt(i);
    if( max > 0 ) {
      tty->print_cr("def:%5d has %d uses:", i, max);
      for( uint j = 0; j < max; j++ ) {
        // This does not remove duplicates
        outp[j]->dump(depth);
      }
      tty->cr();
    }
  }
}
// Dump the uses of the node with index 'index'
void PhaseDefUse::dump_node(uint index) {
  Node **outp = out(index);
  uint   max  = cnt(index);
  tty->print_cr("def:%5d has %d uses:", index, max);
  if( max > 0 ) {
    for( uint j = 0; j < max; j++ ) {
      // This does not remove duplicates
      tty->print("   ");  outp[j]->dump(1);
    }
    tty->cr();
  }
}

// Verify that each 'use' of the def-node 
// has an input edge back to the def-node
void PhaseDefUse::verify_uses(const Node *def) {
  if( def == NULL )
    return;  // debug point
  if( def == C->_top_node )
    return;
  Node **outp = out(def);
  uint   max  = cnt(def);
  if( max > 0 ) {
    for( uint j = 0; j < max; j++ ) {
      Node *use = outp[j];
      // Check that 'def' is an input to 'use'
      bool verified = false;
      uint max_inputs = use->len();
      for( uint i = 0; i < max_inputs; i++ ) {
        if( use->in(i) == def )  { verified = true; break; }
      }
      guarantee( verified, "Def must appear as an input to a Use");
    }
  }
}
#endif




