// @(#)split_if.cpp	1.46 01/05/04 07:52:29

//  Copyright 1997-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2
#include "incls/_precompiled.incl"
#include "incls/_split_if.cpp.incl"


//------------------------------split_thru_region------------------------------
// Split Node 'n' through merge point.  
Node *PhaseIdealLoop::split_thru_region( Node *n, Node *region ) {
  uint wins = 0;
  assert( n->is_CFG(), "" );
  assert( region->is_Region(), "" );
  Node *r = new RegionNode( region->req() );
  IdealLoopTree *loop = get_loop( n );
  for( uint i = 1; i < region->req(); i++ ) {
    Node *x = n->clone();
    Node *in = n->in(0);
    if( in->in(0) == region ) x->set_req( 0, in->in(i) );
    for( uint j = 1; j < n->req(); j++ ) {
      Node *in = n->in(j);
      if( get_ctrl(in->_idx) == region ) 
        x->set_req( j, in->in(i) );
    }
    _igvn.register_new_node_with_optimizer(x);
    set_loop( x->_idx, loop );
    set_idom( x->_idx, x->in(0), dom_depth(x->in(0)->_idx)+1 );
    r->set_req( i, x );
  }
  
  // Record region
  r->set_req(0,region);         // Not a TRUE RegionNode
  _igvn.register_new_node_with_optimizer(r);
  set_loop( r->_idx, loop );
  if( !loop->_child )
    loop->_body.push(r);
  return r;
}

//------------------------------split_up---------------------------------------
// Split block-local op up through the phis to empty the current block
bool PhaseIdealLoop::split_up( Node *n, Node *blk1, Node *blk2 ) {
  PhaseDefUse &du = _igvn._du;
  if( n->is_CFG() ) {
    assert( n->in(0) != blk1, "Lousy candidate for split-if" );
    return false;
  }
  if( get_ctrl( n->_idx ) != blk1 && get_ctrl( n->_idx ) != blk2 ) 
    return false;		// Not block local
  if( n->is_Phi() ) return false; // Local PHIs are expected

  // Recursively split-up inputs
  uint i;
  for( i = 1; i < n->req(); i++ ) {
    if( split_up( n->in(i), blk1, blk2 ) ) {
      // Got split recursively and self went dead?
      if( du.cnt(n) == 0 ) 
	_igvn.remove_dead_node(n);
      return true;
    }
  }

  // Check for needing to clone-up a compare.  Can't do that, it forces
  // another (nested) split-if transform.  Instead, clone it "down".
  uint nop = n->Opcode();
  if( nop == Op_CmpI || nop == Op_CmpU || nop == Op_CmpP ) {
    assert( get_ctrl( n->_idx ) == blk2 ||
	    get_ctrl( n->_idx ) == blk1, "must be in block with IF" );
    // Check for simple Cmp/Bool/CMove which we can clone-up.  Cmp/Bool/CMove
    // sequence can have no other users and it must all reside in the split-if
    // block.  Non-simple Cmp/Bool/CMove sequences are 'cloned-down' below -
    // private, per-use versions of the Cmp and Bool are made.  These sink to
    // the CMove block.  If the CMove is in the split-if block, then in the
    // next iteration this will become a simple Cmp/Bool/CMove set to clone-up.
    Node *bol, *cmov;
    if( !(du.cnt(n) == 1 && 
	  (bol = du.out(n)[0]->is_Bool()) &&
	  (get_ctrl(bol->_idx) == blk1 ||
	   get_ctrl(bol->_idx) == blk2) &&
	  du.cnt(bol) == 1 &&
	  (cmov = du.out(bol)[0]->is_CMove()) &&
	  (get_ctrl(cmov->_idx) == blk1 ||
	   get_ctrl(cmov->_idx) == blk2) ) ) {
      
      // Must clone down
#ifndef PRODUCT
      if( PrintOpto && VerifyLoopOptimizations ) {
	tty->print("Cloning down: ");
	n->dump();
      }
#endif
      // Clone down any block-local BoolNode uses of this CmpNode
      for( uint i = 0; i < du.cnt(n); i++ ) {
	Node *bol = du.out(n)[i];
	assert( bol->is_Bool(), "" );
	if( du.cnt(bol) == 1 ) continue;
	if( get_ctrl( bol->_idx ) == blk1 ||
	    get_ctrl( bol->_idx ) == blk2 ) {
	  // Recursively sink any BoolNode
#ifndef PRODUCT
	  if( PrintOpto && VerifyLoopOptimizations ) {
	    tty->print("Cloning down: ");
	    bol->dump();
	  }
#endif	
	  while( du.cnt(bol) ) {
	    // Uses are either IfNodes or CMoves
	    Node *iff = du.out(bol)[0];
	    assert( iff->in(1) == bol, "" );
	    // Get control block of either the CMove or the If input
	    Node *iff_ctrl = iff->is_If() ? iff->in(0) : get_ctrl( iff->_idx );
	    Node *x = bol->clone();
	    register_new_node(x, iff_ctrl);
	    _igvn.hash_delete(iff);
	    iff->set_req( 1, x, &du );
	  }
	  _igvn.remove_dead_node( bol );
	  i--;			// Re-run iteration in the for loop
	}
      }
      // Clone down this CmpNode
      while( du.cnt(n) ) {
	Node *bol = du.out(n)[0];
	assert( bol->in(1) == n, "" );
	Node *x = n->clone();
	register_new_node(x, get_ctrl(bol->_idx) );
	_igvn.hash_delete(bol);
	bol->set_req( 1, x, &du );
      }
      _igvn.remove_dead_node( n );
      
      return true;
    }
  }

  // See if splitting-up a Store.  Any anti-dep loads must go up as
  // well.  An anti-dep load might be in the wrong block, because in
  // this particular layout/schedule we ignored anti-deps and allow
  // memory to be alive twice.  This only works if we do the same
  // operations on anti-dep loads as we do their killing stores.
  if( n->is_Store() && n->in(MemNode::Memory)->in(0) == n->in(0) ) {
    // Get store's memory slice
    uint alias_idx = C->get_alias_index(_igvn.type(n->in(MemNode::Address))->is_ptr());
   
    // Get memory-phi anti-dep loads will be using
    Node *memphi = n->in(MemNode::Memory);
    assert( memphi->is_Phi(), "" );
    // Hoist any anti-dep load to the splitting block;
    // it will then "split-up".
    for( uint i = 0; i < du.cnt(memphi); i++ ) {
      const LoadNode *load = du.out(memphi)[i]->is_Load();
      if( load && alias_idx == C->get_alias_index(_igvn.type(load->in(MemNode::Address))->is_ptr()) )
      set_ctrl(load->_idx,blk1);
    }
  }

  // Found some other Node; must clone it up    
#ifndef PRODUCT
  if( PrintOpto && VerifyLoopOptimizations ) {
    tty->print("Cloning up: ");
    n->dump();
  }
#endif

  // Now actually split-up this guy.  One copy per control path merging.
  Node *phi = new PhiNode( blk1, n->bottom_type() );
  for( uint j = 1; j < blk1->req(); j++ ) {
    Node *x = n->clone();
    if( n->in(0) && n->in(0) == blk1 ) 
      x->set_req( 0, blk1->in(j) );
    for( uint i = 1; i < n->req(); i++ ) {
      Node *m = n->in(i);
      if( get_ctrl( m->_idx ) == blk1 ) {
        assert( m->in(0) == blk1, "" );
        x->set_req( i, m->in(j) );
      }
    }
    register_new_node( x, blk1->in(j) );
    phi->set_req( j, x );
  }
  // Announce phi to optimizer
  register_new_node(phi, blk1);

  // Remove cloned-up value from optimizer; use phi instead
  _igvn.hash_delete(n);
  _igvn.subsume_node( n, phi );

  return true;
}

//------------------------------register_new_node------------------------------
void PhaseIdealLoop::register_new_node( Node *n, Node *blk ) {
  _igvn.register_new_node_with_optimizer(n);
  set_ctrl( n->_idx, blk );
  IdealLoopTree *loop = get_loop( blk );
  if( !loop->_child )
    loop->_body.push(n);
}

//------------------------------build_phi--------------------------------------
// Lazily build a Phi for a particular merge site
Node *PhaseIdealLoop::build_phi( Node *&cache, Node *def, Node *use_blk ) {
  if( cache ) return cache;
  cache = def->clone();
  cache->set_req(0, use_blk );
  register_new_node(cache, use_blk);
  return cache;
}

//------------------------------spinup-----------------------------------------
// "Spin up" the dominator tree, starting at the use site and stopping when we
// find the post-dominating point.

// We must be at the merge point which post-dominates 'new_false' and
// 'new_true'.  Figure out which edges into the RegionNode eventually lead up
// to false and which to true.  Put in a PhiNode to merge values; plug in 
// the appropriate false-arm or true-arm values.  If some path leads to the
// original IF, then insert a Phi recursively.
Node *PhaseIdealLoop::spinup( Node *iff_dom, Node *new_false, Node *new_true, Node *&phi_false, Node *&phi_true, Node *use_blk, Node *def ) {
  if( use_blk == C->_top_node ) // Handle dead uses
    return C->_top_node;
  Node *prior_n = (Node*)0xdeadbeef;
  Node *n = use_blk;	        // Get path input
  while( 1 ) {		        // Scan up till find true or false
    if( n == new_false ) return build_phi( phi_false, def, new_false );
    if( n == new_true  ) return build_phi( phi_true , def, new_true  );
    if( n == iff_dom ) {        // Found post-dominating point?

      // This method handles both control uses (looking for Regions) or data
      // uses (looking for Phis).  If looking for a control use, then we need
      // to insert a Region instead of a Phi; however Regions always exist
      // previously (the hash_find_insert below would always hit) so we can
      // return the existing Region.
      if( def->is_CFG() ) return prior_n; // If looking for CFG, return prior
      assert( def->is_Phi(), "" );
      // Need a Phi here
      assert( prior_n->is_Region(), "must be a post-dominating merge point" );
      Node *phi_post = new PhiNode( prior_n, ((PhiNode*)def)->_type );
      // Search for both true and false on all paths till find one.
      for( uint i = 1; i < phi_post->req(); i++ ) // For all paths
        phi_post->set_req( i, spinup( iff_dom, new_false, new_true, phi_false, phi_true, prior_n->in(i), def ) );
      Node *t = _igvn.hash_find_insert(phi_post);
      if( t ) {			// See if we already have this one
	phi_post->destruct();
        return t;
      }
      register_new_node( phi_post, prior_n );
      return phi_post;
    }
    prior_n = n;
    n = idom(n->_idx);          // Search higher
  } // End of while not gone high enough

}

//------------------------------small_cache------------------------------------
struct small_cache {
  Node *_use_blk1, *_use_blk2;
  Node *_new_def1, *_new_def2;

  small_cache() : _use_blk1(NULL), _use_blk2(NULL), _new_def1(NULL), _new_def2(NULL) {}
  Node *probe( Node *use_blk ) {
    if( use_blk == _use_blk1 ) return _new_def1;
    if( use_blk == _use_blk2 ) return _new_def2;
    return NULL;
  }
  void lru_insert( Node *use_blk, Node *new_def ) {
    _use_blk2 = _use_blk1;  _new_def2 = _new_def1;
    _use_blk1 =  use_blk ;  _new_def1 =  new_def ;
  }
};

//------------------------------find_use_block---------------------------------
// Find the block a USE is in.  Normally USE's are in the same block as the
// using instruction.  For Phi-USE's, the USE is in the predecessor block
// along the corresponding path.
Node *PhaseIdealLoop::find_use_block( Node *use, Node *def, Node *old_false, Node *new_false, Node *old_true, Node *new_true ) {
  // CFG uses are their own block
  if( use->is_CFG() ) 
    return use;

  if( use->is_Phi() ) {		// Phi uses in prior block
    // Grab the first Phi use; there may be many.
    // Each will be handled as a seperate iteration of
    // the "while( du->cnt(phi) )" loop.
    uint j;
    for( j = 1; j < use->req(); j++ ) 
      if( use->in(j) == def ) 
	break;
    return use->in(0)->in(j);
  } 
  // Normal (non-phi) use
  Node *use_blk = get_ctrl( use->_idx );
  // Some uses are directly attached to the old (and going away)
  // false and true branches.
  if( use_blk == old_false ) {
    use_blk = new_false;
    set_ctrl( use->_idx, new_false );
  }
  if( use_blk == old_true ) {
    use_blk = new_true;
    set_ctrl( use->_idx, new_true );
  }
  
  if( !use_blk ) {		// He's dead, Jim
    _igvn.hash_delete( use );
    _igvn.subsume_node( use, C->_top_node );
  }

  return use_blk;
}

//------------------------------handle_use-------------------------------------
// Handle uses of the merge point.  Basically, split-if makes the merge point
// go away so all uses of the merge point must go away as well.  Most block
// local uses have already been split-up, through the merge point.  Uses from
// far below the merge point can't always be split up (e.g., phi-uses are
// pinned) and it makes too much stuff live.  Instead we use a path-based
// solution to move uses down.
//
// If the use is along the pre-split-CFG true branch, then the new use will
// be from the post-split-CFG true merge point.  Vice-versa for the false
// path.  Some uses will be along both paths; then we sink the use to the
// post-dominating location; we may need to insert a Phi there.
void PhaseIdealLoop::handle_use( Node *use, Node *def, small_cache *cache, Node *region_dom, Node *new_false, Node *new_true, Node *phi_false, Node *phi_true, Node *old_false, Node *old_true ) {

  Node *use_blk = find_use_block(use,def,old_false,new_false,old_true,new_true);
  if( !use_blk ) return;	// He's dead, Jim
      
  // Probe cache: have we already computed where the USE goes for this block
  // from the same DEF?
  Node *new_def = cache->probe( use_blk );
  if( !new_def ) {
    // Walk up the dominator tree until I hit either the old IfFalse, the old
    // IfTrue or the old If.  Insert Phis where needed.
    new_def = spinup( region_dom, new_false, new_true, phi_false, phi_true, use_blk, def );
    // Fill in 2-element cache, LRU style
    cache->lru_insert( use_blk, new_def );
  }
  
  // Found where this USE goes.  Re-point him.
  uint i;
  for( i = 0; i < use->req(); i++ )
    if( use->in(i) == def ) 
      break;
  _igvn.hash_delete(use);
  use->set_req(i,new_def,&_igvn._du);
}

//------------------------------do_split_if------------------------------------
// Found an If getting its condition-code input from a Phi in the same block.
// Split thru the Region.
void PhaseIdealLoop::do_split_if( Node *iff ) {
#ifndef PRODUCT
  if( PrintOpto && VerifyLoopOptimizations ) 
    tty->print_cr("Split-if");
#endif
  C->_major_progress = true;

  Node *region = iff->in(0);
  Node *region_dom = idom(region->_idx);

  // We are going to clone this test (and the control flow with it) up through
  // the incoming merge point.  We need to empty the current basic block. 
  // Clone any instructions which must be in this block up through the merge
  // point.
  PhaseDefUse *du = &_igvn._du;
  bool progress = true;
  while( progress ) {
    progress = false;
    for( uint i = 0; i < du->cnt(region); i++ ) {
      Node *n = du->out(region)[i];
      if( n == region ) continue;
      // The IF to be split is OK.
      if( n == iff ) continue;
      if( !n->is_Phi() ) {	// Found pinned memory op or such
	if( split_up( n, region, iff ) )
	  progress = true;
	continue;
      }
      assert( n->in(0) == region, "" );
      
      // Recursively split up all users of a Phi
      for( uint j = 0; j < du->cnt(n); j++ ) {
	Node *m = du->out(n)[j];
	if( m != iff && split_up( m, region, iff ) )
	  progress = true;
      }
    }
  }

  // Now we have no instructions in the block containing the IF.
  // Split the IF.
  Node *new_iff = split_thru_region( iff, region );

  // Replace both uses of 'new_iff' with Regions merging True/False
  // paths.  This makes 'new_iff' go dead.
  Node *old_false, *old_true;
  Node *new_false, *new_true;
  while( du->cnt(iff) ) {
    Node *ifp = du->out(iff)[0];
    assert( ifp->Opcode() == Op_IfFalse || ifp->Opcode() == Op_IfTrue, "" );
    ifp->set_req(0,new_iff,du);
    Node *ifpx = split_thru_region( ifp, region );

    // Replace 'If' projection of a Region with a Region of
    // 'If' projections.
    ifpx->set_req(0,ifpx,du);   // A TRUE RegionNode

    // Setup dominator info
    set_idom( ifpx->_idx, region_dom, dom_depth(region_dom->_idx) );

    // Check for splitting loop tails
    if( get_loop(iff)->tail() == ifp )
      get_loop(iff)->_tail = ifpx;

    // Replace in the graph with lazy-update mechanism
    new_iff->set_req(0,new_iff,du); // hook self so it does not go dead
    lazy_replace_proj( ifp, ifpx );
    new_iff->set_req(0,region,du);

    // Record bits for later xforms
    if( ifp->Opcode() == Op_IfFalse ) {
      old_false = ifp;
      new_false = ifpx;
    } else {
      old_true = ifp;
      new_true = ifpx;
    }
  }
  _igvn.remove_dead_node(new_iff);
  // Lazy replace IDOM info with the region's dominator
  lazy_replace( iff, region_dom );

  // Now make the original merge point go dead, by handling all its uses.
  small_cache region_cache;
  while( du->cnt( region ) ) {
    Node *phi = du->out(region)[0];
    if( !phi->in(0) ) {         // Dead phi?  Remove it
      _igvn.remove_dead_node(phi);
      continue;
    }
    assert( phi->in(0) == region, "" );
    if( phi == region ) {	// Found the self-reference
      phi->set_req(0,NULL,du);
      continue;			// Break the self-cycle
    }
    // Expected common case: Phi hanging off of Region
    if( phi->is_Phi() ) {
      // Some uses of the Phi will be on the false branch, some on the true
      // branch and some at the post-dominating point.  Lazily construct
      // these as needed.
      Node *phi_false = NULL;
      Node *phi_true  = NULL;
      small_cache phi_cache;
      
      // Inspect all Phi uses to make the Phi go dead
      while( du->cnt( phi ) ) {
	Node *use = du->out(phi)[0];
	// Compute the new DEF for this USE.  New DEF depends on the path
	// taken from the original DEF to the USE.  The new DEF may be some
	// collection of PHI's merging values from different paths.  The Phis
	// inserted depend only on the location of the USE.  We use a
	// 2-element cache to handle multiple uses from the same block.
	handle_use( use, phi, &phi_cache, region_dom, new_false, new_true, phi_false, phi_true, old_false, old_true );
      } // End of while phi has uses
      
      // Remove the dead Phi
      _igvn.remove_dead_node( phi );

    } else {
      // Random memory op guarded by Region.  Compute new DEF for USE.
      handle_use( phi, region, &region_cache, region_dom, new_false, new_true, new_false, new_true, old_false, old_true );
    }

  } // End of while merge point has phis

  // Any leftover bits in the splitting block must not have depended on local 
  // Phi inputs (these have already been split-up).  Hence it's safe to hoist
  // these guys to the dominating point.
  lazy_replace( region, region_dom );
#ifndef PRODUCT
  if( VerifyLoopOptimizations ) verify();
#endif
}

