/*
 * @(#)jni.cpp	1.296 01/03/28 11:47:32
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_jni.cpp.incl"


// out-of-line helpers for class jfieldIDWorkaround:

unsigned int jfieldIDWorkaround::encode_klass_hash(klassOop k, int offset) {
  if (offset <= small_offset_mask) {
    klassOop field_klass = k;
    klassOop super_klass = Klass::cast(field_klass)->super();
    while (instanceKlass::cast(super_klass)->size_helper() > offset) {
      field_klass = super_klass;   // super contains the field also
      super_klass = Klass::cast(field_klass)->super();
    }
    unsigned int klass_hash = field_klass->identity_hash();
    return ((klass_hash & klass_mask) << klass_shift) | checked_mask_in_place;
  } else {
    #ifndef PRODUCT
    {
      ResourceMark rm;
      warning("VerifyJNIFields: long offset %d in %s", offset, Klass::cast(k)->external_name());
    }
    #endif
    return 0;
  }
}

bool jfieldIDWorkaround::klass_hash_ok(klassOop k, jfieldID id) {
  unsigned int as_uint = (unsigned int) id;
  int klass_hash = (as_uint >> klass_shift) & klass_mask;
  do {
    // Could use a non-blocking query for identity_hash here...
    if ((k->identity_hash() & klass_mask) == klass_hash)
      return true;
    k = Klass::cast(k)->super();
  } while (k != NULL);
  return false;
}

void jfieldIDWorkaround::verify_instance_jfieldID(klassOop k, jfieldID id) {
  guarantee(jfieldIDWorkaround::is_instance_jfieldID(k, id), "must be an instance field" ); 
  unsigned int as_uint = (unsigned int) id;
  int offset = raw_instance_offset(id);
  if (VerifyJNIFields) {
    if (is_checked_jfieldID(id)) {
      guarantee(klass_hash_ok(k, id),
		"Bug in native code: jfieldID class must match object");
    } else {
      #ifndef PRODUCT
      {
	ResourceMark rm;
	warning("VerifyJNIFields: unverified offset %d for %s", offset, Klass::cast(k)->external_name());
      }
      #endif
    }
  }
  guarantee(offset >= oopDesc::header_size(),
	    "Bug in native code: jfieldID offset must address interior of object");
  guarantee(offset < instanceKlass::cast(k)->size_helper(),
	    "Bug in native code: jfieldID offset must not reach beyond object");
}

// Wrapper to trace JNI functions

#ifdef ASSERT
  Histogram* JNIHistogram;

  class JNITraceWrapper : public StackObj {
   public:
    JNITraceWrapper(const char* format, ...) {      
      if (TraceJNICalls) {
        va_list ap;
        va_start(ap, format);
        tty->print("JNI ");
        tty->vprint_cr(format, ap);
        va_end(ap);
      }
    }
  };

  class JNIHistogramElement : public HistogramElement {
    public:
     JNIHistogramElement(const char* name);
  };

  JNIHistogramElement::JNIHistogramElement(const char* elementName) {
    _name = elementName;

    if(JNIHistogram == NULL)
      JNIHistogram = new Histogram("JNI Call Counts",100);

    JNIHistogram->add_element(this);
  }

  #define JNICountWrapper(arg)                                     \
     static JNIHistogramElement* e = new JNIHistogramElement(arg); \
      /* There is a MT-race condition in VC++. So we need to make sure that that e has been initialized */ \
     if (e != NULL) e->increment_count()
  #define JNIWrapper(arg) JNICountWrapper(arg); JNITraceWrapper(arg)
#else
  #define JNIWrapper(arg)
#endif


// Implementation of JNI entries

JNI_ENTRY(jclass, jni_DefineClass(JNIEnv *env, const char *name, jobject loaderRef, const jbyte *buf, jsize bufLen))
  JNIWrapper("DefineClass");
  
  // Since exceptions can be thrown, class initialization can take place  
  // if name is NULL no check for class name in .class stream has to be made.
  symbolOop class_name = NULL;
  if (name != NULL) {
    class_name = oopFactory::new_symbol((char*) name, CHECK_0);
  }

  ResourceMark rm;
  ClassFileStream st((u1*) buf, bufLen, NULL);
  oop class_loader = JNIHandles::resolve(loaderRef);
  klassOop k = SystemDictionary::resolve_from_stream(class_name, class_loader, NULL, &st, CHECK_0);

  return (jclass) JNIHandles::make_local(env, Klass::cast(k)->java_mirror());
JNI_END



static bool first_time_FindClass = true;


JNI_ENTRY(jclass, jni_FindClass(JNIEnv *env, const char *name))
  JNIWrapper("FindClass");

  // Remember if we are the first invocation of jni_FindClass
  bool first_time = first_time_FindClass;
  first_time_FindClass = false;

  //%note jni_3
  Handle loader;
  Handle protection_domain;
  // Find calling class
  instanceKlassHandle k= thread->security_get_caller_class(0);
  if (k.not_null()) {    
    loader = k->class_loader();
    // Special handling to make sure JNI_OnLoad and JNI_OnUnload are executed in the correct class context.
    if (loader == NULL && k->name() == vmSymbols::java_lang_ClassLoader_NativeLibrary()) {      
      JavaValue result(T_OBJECT);
      JavaCalls::call_static(&result, k(), 
                                      vmSymbolHandles::getFromClass_name(), 
                                      vmSymbolHandles::void_class_signature(),                                      
                                      thread);
      if (HAS_PENDING_EXCEPTION) {
        Handle ex(thread, thread->pending_exception());
        CLEAR_PENDING_EXCEPTION;
        THROW_HANDLE_0(ex);
      }
      oop mirror = (oop) result._value.h;
      loader = instanceKlass::cast(java_lang_Class::as_klassOop(mirror))->class_loader();
      protection_domain = instanceKlass::cast(java_lang_Class::as_klassOop(mirror))->protection_domain();
    }
  } else {
    // We call ClassLoader.getSystemClassLoader to obtain the system class loader.
    loader = SystemDictionary::java_system_loader();
  }
  symbolHandle sym = oopFactory::new_symbol((char*) name, CHECK_0);
  jclass result = find_class_from_class_loader(env, sym, true, loader, protection_domain, true, thread);

  // If we were the first invocation of jni_FindClass, we enable compilation again
  // rather than just allowing invocation counter to overflow and decay. Controlled 
  // by flag DelayCompilationDuringStartup.
  NOT_CORE(if (first_time && !CompileTheWorld) InvocationCounter::reinitialize(false);)

  return result;
JNI_END


JNI_ENTRY(jmethodID, jni_FromReflectedMethod(JNIEnv *env, jobject method))
  JNIWrapper("FromReflectedMethod");
  // method is a handle to a java.lang.reflect.Method object
  oop reflected  = JNIHandles::resolve_non_null(method);
  oop mirror     = NULL;
  int slot       = 0;

  if (reflected->klass() == SystemDictionary::reflect_constructor_klass()) {
    mirror = java_lang_reflect_Constructor::clazz(reflected);  
    slot   = java_lang_reflect_Constructor::slot(reflected);
  } else {
    assert(reflected->klass() == SystemDictionary::reflect_method_klass(), "wrong type");
    mirror = java_lang_reflect_Method::clazz(reflected);  
    slot   = java_lang_reflect_Method::slot(reflected);
  }
  klassOop k     = java_lang_Class::as_klassOop(mirror);
  // find corresponding JNIid object
  JNIid* id = instanceKlass::cast(k)->jni_id_for(slot);
  debug_only(id->set_is_method_id());
  return (jmethodID) id;
JNI_END


JNI_ENTRY(jfieldID, jni_FromReflectedField(JNIEnv *env, jobject field))
  JNIWrapper("FromReflectedField");

  // field is a handle to a java.lang.reflect.Field object
  oop reflected   = JNIHandles::resolve_non_null(field);
  oop mirror      = java_lang_reflect_Field::clazz(reflected);
  klassOop k      = java_lang_Class::as_klassOop(mirror);
  int slot        = java_lang_reflect_Field::slot(reflected);
  int modifiers   = java_lang_reflect_Field::modifiers(reflected);

  // First check if this is a static field
  if (modifiers & JVM_ACC_STATIC) {
    int offset = instanceKlass::cast(k)->fields()->ushort_at(slot + 4);
    JNIid* id = instanceKlass::cast(k)->jni_id_for(offset);
    assert(id != NULL, "corrupt Field object");
    debug_only(id->set_is_static_field_id();)
    // A jfieldID for a static field is a JNIid specifying the field holder and the offset within the klassOop
    return jfieldIDWorkaround::to_static_jfieldID(id);
  }

  // The slot is the index of the field description in the field-array (5-tupel)
  // The jfieldID is the offset of the field within the object
  // It may also have hash bits for k, if VerifyJNIFields is turned on.
  int offset = instanceKlass::cast(k)->fields()->ushort_at(slot + 4);

  assert(offset >= oopDesc::header_size(), "stay clear of header");
  assert(offset < instanceKlass::cast(k)->size_helper(), "stay within object");
  return jfieldIDWorkaround::to_instance_jfieldID(k, offset);
JNI_END


JNI_ENTRY(jobject, jni_ToReflectedMethod(JNIEnv *env, jclass cls, jmethodID methodID, jboolean isStatic))
  JNIWrapper("ToReflectedMethod");
  methodOop m = ((JNIid*) methodID)->method();
  assert(m->is_static() == isStatic, "jni_ToReflectedMethod access flags doesn't match");
  oop reflection_method;
  if (m->is_initializer()) {
    reflection_method = Reflection::new_constructor(m, CHECK_0);
  } else {
    reflection_method = Reflection::new_method(m, CHECK_0);
  }
  return JNIHandles::make_local(env, reflection_method);
JNI_END


JNI_ENTRY(jclass, jni_GetSuperclass(JNIEnv *env, jclass sub))
  JNIWrapper("GetSuperclass");
  oop mirror = JNIHandles::resolve_non_null(sub);
  // primitive classes return NULL
  if (java_lang_Class::is_primitive(mirror)) return NULL;

  // interfaces return NULL
  klassOop k = java_lang_Class::as_klassOop(mirror);
  if (Klass::cast(k)->is_interface()) return NULL;

  // return mirror for superclass
  klassOop super = Klass::cast(k)->super();
  return (super == NULL) ? NULL : (jclass) JNIHandles::make_local(Klass::cast(super)->java_mirror());
JNI_END


JNI_QUICK_ENTRY(jboolean, jni_IsAssignableFrom(JNIEnv *env, jclass sub, jclass super))
  JNIWrapper("IsSubclassOf");
  oop sub_mirror   = JNIHandles::resolve_non_null(sub);
  oop super_mirror = JNIHandles::resolve_non_null(super);
  if (java_lang_Class::is_primitive(sub_mirror) || java_lang_Class::is_primitive(super_mirror)) {
    return sub_mirror == super_mirror;
  }
  klassOop sub_klass   = java_lang_Class::as_klassOop(sub_mirror);
  klassOop super_klass = java_lang_Class::as_klassOop(super_mirror);
  assert(sub_klass != NULL && super_klass != NULL, "invalid arguments to jni_IsAssignableFrom");
  return Klass::cast(sub_klass)->is_subtype_of(super_klass) ? JNI_TRUE : JNI_FALSE;
JNI_END


JNI_ENTRY(jint, jni_Throw(JNIEnv *env, jthrowable obj))
  JNIWrapper("Throw");
  THROW_OOP_(JNIHandles::resolve(obj), JNI_OK);
JNI_END


JNI_ENTRY(jint, jni_ThrowNew(JNIEnv *env, jclass clazz, const char *message))  
  JNIWrapper("ThrowNew");  
  instanceKlass* k = instanceKlass::cast(java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz)));  
  symbolHandle name = symbolHandle(THREAD, k->name());
  Handle class_loader =  k->class_loader();
  Handle protection_domain = k->protection_domain();
  THROW_MSG_LOADER_(name, (char *)message, class_loader, protection_domain, JNI_OK);
JNI_END


JNI_ENTRY(jthrowable, jni_ExceptionOccurred(JNIEnv *env))
  JNIWrapper("ExceptionOccurred");
  oop exception = thread->pending_exception();
  return (jthrowable) JNIHandles::make_local(env, exception);
JNI_END


JNI_ENTRY(void, jni_ExceptionDescribe(JNIEnv *env))  
  JNIWrapper("ExceptionDescribe");
  if (thread->has_pending_exception()) {
    Handle ex(thread, thread->pending_exception());
    thread->clear_pending_exception();
    if (ex->is_a(SystemDictionary::threaddeath_klass())) {
      // Don't print anything if we are being killed.
    } else {
      jio_fprintf(stderr, "Exception ");
      if (thread != NULL && thread->threadObj() != NULL) {
        ResourceMark rm;
        jio_fprintf(stderr, "in thread \"%s\" ", thread->get_thread_name());
      }
      if (ex->is_a(SystemDictionary::throwable_klass())) {        
        JavaValue result(T_VOID);
        JavaCalls::call_virtual(&result, 
                                ex, 
                                SystemDictionary::throwable_klass(),
                                vmSymbolHandles::printStackTrace_name(), 
                                vmSymbolHandles::void_method_signature(), 
                                THREAD);
        // If an exception is thrown in the call it gets thrown away. Not much
        // we can do with it. The native code that calls this, does not check
        // for the exception - hence, it might still be in the thread when DestroyVM gets
        // called, potentially causing a few asserts to trigger - since no pending exception
        // is expected.
        CLEAR_PENDING_EXCEPTION;
      } else {
        ResourceMark rm;
        jio_fprintf(stderr, ". Uncaught exception of type %s.", Klass::cast(ex->klass())->external_name());        
      }
    }
  }
JNI_END


JNI_QUICK_ENTRY(void, jni_ExceptionClear(JNIEnv *env))
  JNIWrapper("ExceptionClear");
  thread->clear_pending_exception();
JNI_END


JNI_ENTRY(void, jni_FatalError(JNIEnv *env, const char *msg))
  JNIWrapper("FatalError");
  tty->print_cr("FATAL ERROR in native method: %s", msg);
  thread->print_stack();
  os::abort(false);	// Prevent core dump, causes a jck failure.
JNI_END


JNI_ENTRY(jint, jni_PushLocalFrame(JNIEnv *env, jint capacity))
  JNIWrapper("PushLocalFrame");
  //%note jni_11
  JNIHandleBlock* old_handles = thread->active_handles();
  JNIHandleBlock* new_handles = JNIHandleBlock::allocate_block(thread);
  assert(new_handles != NULL, "should not be NULL");
  new_handles->set_pop_frame_link(old_handles);
  thread->set_active_handles(new_handles);
  return JNI_OK;
JNI_END


JNI_ENTRY(jobject, jni_PopLocalFrame(JNIEnv *env, jobject result))
  JNIWrapper("PopLocalFrame");
  //%note jni_11
  Handle result_handle(thread, JNIHandles::resolve(result));
  JNIHandleBlock* old_handles = thread->active_handles();
  JNIHandleBlock* new_handles = old_handles->pop_frame_link();
  if (new_handles != NULL) {
    // As a sanity check we only release the handle blocks if the pop_frame_link is not NULL.
    // This way code will still work if PopLocalFrame is called without a corresponding
    // PushLocalFrame call. Note that we set the pop_frame_link to NULL explicitly, otherwise
    // the release_block call will release the blocks.
    thread->set_active_handles(new_handles);
    old_handles->set_pop_frame_link(NULL);              // clear link we won't release new_handles below
    JNIHandleBlock::release_block(old_handles, thread); // may block
    result = JNIHandles::make_local(thread, result_handle());
  }
  return result;
JNI_END


JNI_ENTRY(jobject, jni_NewGlobalRef(JNIEnv *env, jobject ref))
  JNIWrapper("NewGlobalRef");
  Handle ref_handle(thread, JNIHandles::resolve(ref));
  return JNIHandles::make_global(ref_handle);
JNI_END


JNI_QUICK_ENTRY(void, jni_DeleteGlobalRef(JNIEnv *env, jobject ref))
  JNIWrapper("DeleteGlobalRef");
  JNIHandles::destroy_global(ref);
JNI_END


JNI_QUICK_ENTRY(void, jni_DeleteLocalRef(JNIEnv *env, jobject obj))
  JNIWrapper("DeleteLocalRef");
  JNIHandles::destroy_local(obj);
JNI_END


JNI_QUICK_ENTRY(jboolean, jni_IsSameObject(JNIEnv *env, jobject r1, jobject r2))
  JNIWrapper("IsSameObject");
  oop a = JNIHandles::resolve(r1);
  oop b = JNIHandles::resolve(r2);
  return (a == b) ? JNI_TRUE : JNI_FALSE;
JNI_END


JNI_ENTRY(jobject, jni_NewLocalRef(JNIEnv *env, jobject ref))
  JNIWrapper("NewLocalRef");
  return JNIHandles::make_local(env, JNIHandles::resolve(ref));
JNI_END


JNI_LEAF(jint, jni_EnsureLocalCapacity(JNIEnv *env, jint capacity))
  JNIWrapper("EnsureLocalCapacity");
  return JNI_OK;
JNI_END


class JNI_ArgumentPusher : public SignatureIterator {
 protected:
  BasicType           return_type;
  JavaCallArguments*  _arguments; 

  virtual void get_bool   () = 0;
  virtual void get_char   () = 0;
  virtual void get_short  () = 0;
  virtual void get_byte   () = 0;
  virtual void get_int    () = 0;
  virtual void get_long   () = 0;
  virtual void get_float  () = 0;
  virtual void get_double () = 0;
  virtual void get_object () = 0;

  JNI_ArgumentPusher(symbolOop signature)
       : SignatureIterator(signature) {    
    this->return_type = T_ILLEGAL;
    _arguments = NULL;
  }

 public:
  void set_java_argument_object(JavaCallArguments *arguments) { _arguments = arguments; }

  void do_bool()                     { if (is_return_type()) return_type = T_BOOLEAN; else get_bool();   }
  void do_char()                     { if (is_return_type()) return_type = T_CHAR;    else get_char();   }
  void do_short()                    { if (is_return_type()) return_type = T_SHORT;   else get_short();  }
  void do_byte()                     { if (is_return_type()) return_type = T_BYTE;    else get_byte();   }
  void do_int()                      { if (is_return_type()) return_type = T_INT;     else get_int();    }
  void do_long()                     { if (is_return_type()) return_type = T_LONG;    else get_long();   }
  void do_float()                    { if (is_return_type()) return_type = T_FLOAT;   else get_float();  }
  void do_double()                   { if (is_return_type()) return_type = T_DOUBLE;  else get_double(); }
  void do_object(int begin, int end) { if (is_return_type()) return_type = T_OBJECT;  else get_object(); }
  void do_array(int begin, int end)  { if (is_return_type()) return_type = T_ARRAY;   else get_object(); } // do_array uses get_object -- there is no get_array
  void do_void()                     { if (is_return_type()) return_type = T_VOID; }

  BasicType get_ret_type()           { return return_type; }
  JavaCallArguments* arguments()     { return _arguments; }
  void push_receiver(Handle h)       { _arguments->push_oop(h); }
};


class JNI_ArgumentPusherVaArg : public JNI_ArgumentPusher {
 protected:
  va_list ap;

  void get_bool()   { _arguments->push_int(va_arg(ap, jint)); } // bool is coerced to int when using va_arg
  void get_char()   { _arguments->push_int(va_arg(ap, jint)); } // char is coerced to int when using va_arg
  void get_short()  { _arguments->push_int(va_arg(ap, jint)); } // short is coerced to int when using va_arg
  void get_byte()   { _arguments->push_int(va_arg(ap, jint)); } // byte is coerced to int when using va_arg
  void get_int()    { _arguments->push_int(va_arg(ap, jint)); }

  // each of these paths is exercized by the various jck Call[Static,Nonvirtual,][Void,Int,..]Method[A,V,] tests

  void get_long()   { _arguments->push_long(va_arg(ap, jlong)); }
  void get_float()  { _arguments->push_float((jfloat)va_arg(ap, jdouble)); } // float is coerced to double w/ va_arg
  void get_double() { _arguments->push_double(va_arg(ap, jdouble)); }
  void get_object() { jobject l =  va_arg(ap, jobject); 
                                 _arguments->push_oop(Handle((oop *)l, false)); }
      
 public:
  JNI_ArgumentPusherVaArg(symbolOop signature, va_list rap)
       : JNI_ArgumentPusher(signature) {
    ap = rap;
  }
  JNI_ArgumentPusherVaArg(jmethodID methodID, va_list rap)
      : JNI_ArgumentPusher(((JNIid*)methodID)->method()->signature()) {
    ap = rap;
  }
};


class JNI_ArgumentPusherArray : public JNI_ArgumentPusher {
 protected:
  jvalue *ap;

  void get_bool()   { _arguments->push_int((jint)(ap++)->z); }
  void get_char()   { _arguments->push_int((jint)(ap++)->c);  }
  void get_short()  { _arguments->push_int((jint)(ap++)->s); }
  void get_byte()   { _arguments->push_int((jint)(ap++)->b); }
  void get_int()    { _arguments->push_int((ap++)->i); }

  void get_long()   { _arguments->push_long((ap++)->j);  }
  void get_float()  { _arguments->push_float((ap++)->f); }
  void get_double() { _arguments->push_double((ap++)->d);}
  void get_object() { _arguments->push_oop(Handle((oop *)(ap++)->l, false)); }
 public:
  JNI_ArgumentPusherArray(symbolOop signature, jvalue *rap)
       : JNI_ArgumentPusher(signature) {
    ap = rap;
  }
  JNI_ArgumentPusherArray(jmethodID methodID, jvalue *rap) 
      : JNI_ArgumentPusher(((JNIid*)methodID)->method()->signature()) {
    ap = rap;
  }
};


enum JNICallType {
  JNI_STATIC,
  JNI_VIRTUAL,
  JNI_NONVIRTUAL
};

static methodHandle jni_resolve_interface_call(Handle recv, methodHandle method, TRAPS) {
  assert(!method.is_null() , "method should not be null");

  KlassHandle recv_klass; // Default to NULL (use of ?: can confuse gcc)
  if (recv.not_null()) recv_klass = KlassHandle(recv->klass());
  KlassHandle spec_klass = method->method_holder();
  symbolHandle name = method->name();
  symbolHandle signature = method->signature();
  CallInfo info;
  LinkResolver::resolve_interface_call(info, recv, recv_klass,  spec_klass, name, signature, KlassHandle((oop)NULL), false, true, CHECK_(methodHandle()));
  return info.selected_method();
}

static methodHandle jni_resolve_virtual_call(Handle recv, methodHandle method, TRAPS) {
  assert(!method.is_null() , "method should not be null");

  KlassHandle recv_klass; // Default to NULL (use of ?: can confuse gcc)
  if (recv.not_null()) recv_klass = KlassHandle(recv->klass());
  KlassHandle spec_klass = method->method_holder();
  symbolHandle name = method->name();
  symbolHandle signature = method->signature();
  CallInfo info;
  LinkResolver::resolve_virtual_call(info, recv, recv_klass,  spec_klass, name, signature, KlassHandle((oop)NULL), false, true, CHECK_(methodHandle()));
  return info.selected_method();
}



// We could instead have three separate functions and avoid the switch statement (Sheng)
static void jni_invoke(JNIEnv *env, JavaValue* result, jobject receiver, JNICallType call_type, jmethodID methodID, JNI_ArgumentPusher *args, TRAPS) {
  methodHandle method(THREAD, ((JNIid*)methodID)->method());  
  Handle recv(THREAD, JNIHandles::resolve(receiver));
  
  // Create object to hold arguments for the JavaCall, and associate it with
  // the jni parser  
  ResourceMark rm;  
  int number_of_parameters = method->size_of_parameters();
  JavaCallArguments java_args(number_of_parameters);
  args->set_java_argument_object(&java_args);
  
  // handle arguments
  if (call_type == JNI_STATIC) {
    assert(method->is_static(), "method should be static");
  } else {
    assert(!method->is_static(), "method should not be static");
    args->push_receiver(recv); // Push jobject handle

    if ((Klass::cast(method->method_holder()))->is_interface()) { // interface call
      method = jni_resolve_interface_call(recv, method, CHECK); 
    } else if ((call_type == JNI_VIRTUAL) &&  // virtual call and
	       (!method->is_private()) &&        // method is not private and 
	       (!(method->name() == vmSymbols::object_initializer_name())) // name is not <init>
	       ) {
      method = jni_resolve_virtual_call(recv, method, CHECK);
    }
  }
 
  // Fill out JavaCallArguments object
  args->iterate();
  // Initialize result type
  result->_type = args->get_ret_type(); 

  // Invoke the method. Result is returned as oop.
  JavaCalls::call(result, method, &java_args, CHECK);

  // Convert result
  if (result->_type == T_OBJECT || result->_type == T_ARRAY) {
    result->_value.h = JNIHandles::make_local(env, (oop) result->_value.h);
  }
}


static instanceOop alloc_object(jclass clazz, TRAPS) {
  KlassHandle k(THREAD, java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz)));
  Klass::cast(k())->check_valid_for_instantiation(false, CHECK_0);
  instanceKlass::cast(k())->initialize(CHECK_0);
  return instanceKlass::cast(k())->allocate_instance(THREAD);
}


JNI_ENTRY(jobject, jni_AllocObject(JNIEnv *env, jclass clazz))
  JNIWrapper("AllocObject");
  instanceOop i = alloc_object(clazz, CHECK_0);
  return JNIHandles::make_local(env, i);
JNI_END


JNI_ENTRY(jobject, jni_NewObjectA(JNIEnv *env, jclass clazz, jmethodID methodID, jvalue *args))  
  JNIWrapper("NewObjectA");
  instanceOop i = alloc_object(clazz, CHECK_0);
  jobject obj = JNIHandles::make_local(env, i);
  JavaValue jvalue(T_VOID);  
  JNI_ArgumentPusherArray ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK_0);
  return obj;
JNI_END


JNI_ENTRY(jobject, jni_NewObjectV(JNIEnv *env, jclass clazz, jmethodID methodID, va_list args))  
  JNIWrapper("NewObjectV");
  instanceOop i = alloc_object(clazz, CHECK_0);
  jobject obj = JNIHandles::make_local(env, i);
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK_0);
  return obj;
JNI_END


JNI_ENTRY(jobject, jni_NewObject(JNIEnv *env, jclass clazz, jmethodID methodID, ...))  
  JNIWrapper("NewObject");
  instanceOop i = alloc_object(clazz, CHECK_0);
  jobject obj = JNIHandles::make_local(env, i);
  va_list args;
  va_start(args, methodID);
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK_0);
  va_end(args);
  return obj;
JNI_END


JNI_ENTRY(jclass, jni_GetObjectClass(JNIEnv *env, jobject obj))
  JNIWrapper("GetObjectClass");
  klassOop k = JNIHandles::resolve_non_null(obj)->klass();
  return (jclass) JNIHandles::make_local(env, Klass::cast(k)->java_mirror());
JNI_END


JNI_QUICK_ENTRY(jboolean, jni_IsInstanceOf(JNIEnv *env, jobject obj, jclass clazz))
  JNIWrapper("IsInstanceOf");
  if (obj == NULL) return JNI_TRUE;
  klassOop k = java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz));
  if (k == NULL) return JNI_FALSE;
  return JNIHandles::resolve_non_null(obj)->is_a(k) ? JNI_TRUE : JNI_FALSE;
JNI_END


static jmethodID get_method_id(JNIEnv *env, jclass clazz, char *name_str, char *sig, bool is_static, TRAPS) {
  // %%%% This code should probably just call into a method in the LinkResolver
  symbolHandle signature = oopFactory::new_symbol(sig, CHECK_0);
  symbolHandle name;
  if (name_str == NULL) {
    name = vmSymbolHandles::object_initializer_name();
  } else {
    name = oopFactory::new_symbol(name_str, CHECK_0);
  }

  // Throw a NoSuchMethodError exception if we have an instance of a primitive java.lang.Class
  if (java_lang_Class::is_primitive(JNIHandles::resolve_non_null(clazz))) {
    THROW_MSG_0(vmSymbols::java_lang_NoSuchMethodError(), name_str);
  }

  KlassHandle klass(THREAD, java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz)));

  // Make sure class is linked and initialized before handing id's out to methodOops. 
  Klass::cast(klass())->initialize(CHECK_0);  

  methodOop m;
  if (name() == vmSymbols::object_initializer_name() || name() == vmSymbols::class_initializer_name()) {
    // Never search superclasses for constructors
    if (klass->oop_is_instance()) {
      m = instanceKlass::cast(klass())->find_method(name(), signature());
    } else {
      m = NULL;
    }
  } else {
    m = klass->lookup_method(name(), signature());
    // Look up interfaces
    if (m == NULL && klass->oop_is_instance()) {
      m = instanceKlass::cast(klass())->lookup_method_in_all_interfaces(name(), signature());      
    }
  }
  if (m == NULL || (m->is_static() != is_static)) {
    THROW_MSG_0(vmSymbols::java_lang_NoSuchMethodError(), name_str);
  }
  return (jmethodID) m->jni_id();
}


JNI_ENTRY(jmethodID, jni_GetMethodID(JNIEnv *env, jclass clazz, const char *name, const char *sig))
  JNIWrapper("GetMethodID");
  return get_method_id(env, clazz, (char*) name, (char*) sig, false, thread);
JNI_END


JNI_ENTRY(jmethodID, jni_GetStaticMethodID(JNIEnv *env, jclass clazz, const char *name, const char *sig))
  JNIWrapper("GetStaticMethodID");
  return get_method_id(env, clazz, (char*) name, (char*) sig, true, thread);
JNI_END


//
// Calling Methods
//


#define DEFINE_CALLMETHOD(ResultType, Result, Tag, unionType) \
JNI_ENTRY(ResultType, \
          jni_Call##Result##Method(JNIEnv *env, jobject obj, jmethodID methodID, ...)) \
  JNIWrapper("Call" XSTR(Result) "Method"); \
  va_list args; \
  va_start(args, methodID); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherVaArg ap(methodID, args); \
  jni_invoke(env, &jvalue, obj, JNI_VIRTUAL, methodID, &ap, CHECK_0); \
  va_end(args); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END \
\
\
JNI_ENTRY(ResultType, \
          jni_Call##Result##MethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args)) \
  JNIWrapper("Call" XSTR(Result) "MethodV"); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherVaArg ap(methodID, args); \
  jni_invoke(env, &jvalue, obj, JNI_VIRTUAL, methodID, &ap, CHECK_0); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END \
\
\
JNI_ENTRY(ResultType, \
          jni_Call##Result##MethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args)) \
  JNIWrapper("Call" XSTR(Result) "MethodA"); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherArray ap(methodID, args); \
  jni_invoke(env, &jvalue, obj, JNI_VIRTUAL, methodID, &ap, CHECK_0); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END

// the runtime type of subword integral basic types is integer
DEFINE_CALLMETHOD(jboolean, Boolean, T_BOOLEAN, i)
DEFINE_CALLMETHOD(jbyte,    Byte,    T_BYTE,    i)
DEFINE_CALLMETHOD(jchar,    Char,    T_CHAR,    i)
DEFINE_CALLMETHOD(jshort,   Short,   T_SHORT,   i)

DEFINE_CALLMETHOD(jobject,  Object,  T_OBJECT,  h)
DEFINE_CALLMETHOD(jint,     Int,     T_INT,     i)
DEFINE_CALLMETHOD(jlong,    Long,    T_LONG,    l)
DEFINE_CALLMETHOD(jfloat,   Float,   T_FLOAT,   f)
DEFINE_CALLMETHOD(jdouble,  Double,  T_DOUBLE,  d)


JNI_ENTRY(void, jni_CallVoidMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...))  
  JNIWrapper("CallVoidMethod");
  va_list args;
  va_start(args, methodID);
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_VIRTUAL, methodID, &ap, CHECK);
  va_end(args);
JNI_END
                                                
                                                   
JNI_ENTRY(void, jni_CallVoidMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args))  
  JNIWrapper("CallVoidMethodV");
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_VIRTUAL, methodID, &ap, CHECK);
JNI_END
                                  
                                     
JNI_ENTRY(void, jni_CallVoidMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args))  
  JNIWrapper("CallVoidMethodA");
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherArray ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_VIRTUAL, methodID, &ap, CHECK);
JNI_END


#define DEFINE_CALLNONVIRTUALMETHOD(ResultType, Result, Tag, unionType) \
\
\
JNI_ENTRY(ResultType, \
          jni_CallNonvirtual##Result##Method(JNIEnv *env, jobject obj, jclass cls, jmethodID methodID, ...)) \
  JNIWrapper("CallNonvitual" XSTR(Result) "Method"); \
  va_list args; \
  va_start(args, methodID); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherVaArg ap(methodID, args); \
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK_0); \
  va_end(args); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END \
\
\
JNI_ENTRY(ResultType, \
          jni_CallNonvirtual##Result##MethodV(JNIEnv *env, jobject obj, jclass cls, jmethodID methodID, va_list args)) \
  JNIWrapper("CallNonvitual" XSTR(Result) "#MethodV"); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherVaArg ap(methodID, args); \
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK_0); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END \
\
\
JNI_ENTRY(ResultType, \
          jni_CallNonvirtual##Result##MethodA(JNIEnv *env, jobject obj, jclass cls, jmethodID methodID, jvalue *args)) \
  JNIWrapper("CallNonvitual" XSTR(Result) "MethodA"); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherArray ap(methodID, args); \
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK_0); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END

// the runtime type of subword integral basic types is integer
DEFINE_CALLNONVIRTUALMETHOD(jboolean, Boolean, T_BOOLEAN, i)
DEFINE_CALLNONVIRTUALMETHOD(jbyte,    Byte,    T_BYTE,    i)
DEFINE_CALLNONVIRTUALMETHOD(jchar,    Char,    T_CHAR,    i)
DEFINE_CALLNONVIRTUALMETHOD(jshort,   Short,   T_SHORT,   i)

DEFINE_CALLNONVIRTUALMETHOD(jobject,  Object,  T_OBJECT,  h)
DEFINE_CALLNONVIRTUALMETHOD(jint,     Int,     T_INT,     i)
DEFINE_CALLNONVIRTUALMETHOD(jlong,    Long,    T_LONG,    l)
DEFINE_CALLNONVIRTUALMETHOD(jfloat,   Float,   T_FLOAT,   f)
DEFINE_CALLNONVIRTUALMETHOD(jdouble,  Double,  T_DOUBLE,  d)


JNI_ENTRY(void, jni_CallNonvirtualVoidMethod(JNIEnv *env, jobject obj, jclass cls, jmethodID methodID, ...))  
  JNIWrapper("CallNonvirtualVoidMethod");
  va_list args;
  va_start(args, methodID);
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK);
  va_end(args);
JNI_END
                                                 
                                                   
JNI_ENTRY(void, jni_CallNonvirtualVoidMethodV(JNIEnv *env, jobject obj, jclass cls, jmethodID methodID, va_list args))  
  JNIWrapper("CallNonvirtualVoidMethodV");
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK);
JNI_END
                             
                                     
JNI_ENTRY(void, jni_CallNonvirtualVoidMethodA(JNIEnv *env, jobject obj, jclass cls, jmethodID methodID, jvalue *args))  
  JNIWrapper("CallNonvirtualVoidMethodA");
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherArray ap(methodID, args);
  jni_invoke(env, &jvalue, obj, JNI_NONVIRTUAL, methodID, &ap, CHECK);
JNI_END


#define DEFINE_CALLSTATICMETHOD(ResultType, Result, Tag, unionType) \
\
\
JNI_ENTRY(ResultType, \
          jni_CallStatic##Result##Method(JNIEnv *env, jclass cls, jmethodID methodID, ...)) \
  JNIWrapper("CallStatic" XSTR(Result) "Method"); \
  va_list args; \
  va_start(args, methodID); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherVaArg ap(methodID, args); \
  jni_invoke(env, &jvalue, NULL, JNI_STATIC, methodID, &ap, CHECK_0); \
  va_end(args); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END \
\
\
JNI_ENTRY(ResultType, \
          jni_CallStatic##Result##MethodV(JNIEnv *env, jclass cls, jmethodID methodID, va_list args)) \
  JNIWrapper("CallStatic" XSTR(Result) "MethodV"); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherVaArg ap(methodID, args); \
  jni_invoke(env, &jvalue, NULL, JNI_STATIC, methodID, &ap, CHECK_0); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END \
\
\
JNI_ENTRY(ResultType, \
          jni_CallStatic##Result##MethodA(JNIEnv *env, jclass cls, jmethodID methodID, jvalue *args)) \
  JNIWrapper("CallStatic" XSTR(Result) "MethodA"); \
  JavaValue jvalue(Tag); \
  JNI_ArgumentPusherArray ap(methodID, args); \
  jni_invoke(env, &jvalue, NULL, JNI_STATIC, methodID, &ap, CHECK_0); \
  return (ResultType)(jvalue._value.##unionType); \
JNI_END

// the runtime type of subword integral basic types is integer
DEFINE_CALLSTATICMETHOD(jboolean, Boolean, T_BOOLEAN, i)
DEFINE_CALLSTATICMETHOD(jbyte,    Byte,    T_BYTE,    i)
DEFINE_CALLSTATICMETHOD(jchar,    Char,    T_CHAR,    i)
DEFINE_CALLSTATICMETHOD(jshort,   Short,   T_SHORT,   i)

DEFINE_CALLSTATICMETHOD(jobject,  Object,  T_OBJECT,  h)
DEFINE_CALLSTATICMETHOD(jint,     Int,     T_INT,     i)
DEFINE_CALLSTATICMETHOD(jlong,    Long,    T_LONG,    l)
DEFINE_CALLSTATICMETHOD(jfloat,   Float,   T_FLOAT,   f)
DEFINE_CALLSTATICMETHOD(jdouble,  Double,  T_DOUBLE,  d)


JNI_ENTRY(void, jni_CallStaticVoidMethod(JNIEnv *env, jclass cls, jmethodID methodID, ...))  
  JNIWrapper("CallStaticVoidMethod");
  va_list args;
  va_start(args, methodID);
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, NULL, JNI_STATIC, methodID, &ap, CHECK);
  va_end(args);
JNI_END
                                                 
                                                   
JNI_ENTRY(void, jni_CallStaticVoidMethodV(JNIEnv *env, jclass cls, jmethodID methodID, va_list args))  
  JNIWrapper("CallStaticVoidMethodV");
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherVaArg ap(methodID, args);
  jni_invoke(env, &jvalue, NULL, JNI_STATIC, methodID, &ap, CHECK);
JNI_END
                                  
                                     
JNI_ENTRY(void, jni_CallStaticVoidMethodA(JNIEnv *env, jclass cls, jmethodID methodID, jvalue *args))  
  JNIWrapper("CallStaticVoidMethodA");
  JavaValue jvalue(T_VOID);
  JNI_ArgumentPusherArray ap(methodID, args);
  jni_invoke(env, &jvalue, NULL, JNI_STATIC, methodID, &ap, CHECK);
JNI_END


//
// Accessing Fields
//


JNI_ENTRY(jfieldID, jni_GetFieldID(JNIEnv *env, jclass clazz, const char *name, const char *sig))
  JNIWrapper("GetFieldID");
  symbolHandle fieldname = oopFactory::new_symbol((char*) name, CHECK_0);
  symbolOop signame = oopFactory::new_symbol((char*) sig, CHECK_0);
  KlassHandle k(THREAD, java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz)));
  // Make sure class is initialized before handing id's out to fields
  Klass::cast(k())->initialize(CHECK_0);

  fieldDescriptor fd;
  if (!Klass::cast(k())->oop_is_instance() || !instanceKlass::cast(k())->jni_find_field(fieldname(), signame, false, &fd)) {
    THROW_MSG_0(vmSymbols::java_lang_NoSuchFieldError(), (char*) name);
  }

  // A jfieldID for a non-static field is simply the offset of the field within the instanceOop
  // It may also have hash bits for k, if VerifyJNIFields is turned on.
  return jfieldIDWorkaround::to_instance_jfieldID(k(), fd.offset());
JNI_END


JNI_ENTRY(jobject, jni_GetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID))
  JNIWrapper("GetObjectField");
  oop o = JNIHandles::resolve_non_null(obj);
  klassOop k = o->klass();
  int offset = jfieldIDWorkaround::from_instance_jfieldID(k, fieldID);
  // Keep JVM/DI addition small and only check enabled flag here.
  // jni_GetField_probe() assumes that is okay to create handles.
  if (jvmdi::enabled()) {
    o = jvmdi::jni_GetField_probe(thread, obj, o, k, fieldID, false);
  }
  return JNIHandles::make_local(env, o->obj_field(offset));
JNI_END


#define DEFINE_GETFIELD(Return,Fieldname,Result) \
JNI_QUICK_ENTRY(Return, jni_Get##Result##Field(JNIEnv *env, jobject obj, jfieldID fieldID)) \
  JNIWrapper("Get" XSTR(Result) "Field"); \
  oop o = JNIHandles::resolve_non_null(obj); \
  klassOop k = o->klass(); \
  int offset = jfieldIDWorkaround::from_instance_jfieldID(k, fieldID); \
  /* Keep JVM/DI addition small and only check enabled flag here.       */ \
  /* jni_GetField_probe_nh() assumes that is not okay to create handles */ \
  /* and creates a ResetNoHandleMark.                                   */ \
  if (jvmdi::enabled()) { \
    o = jvmdi::jni_GetField_probe_nh(thread, obj, o, k, fieldID, false); \
  } \
  return o->Fieldname##_field(offset); \
JNI_END

DEFINE_GETFIELD(jboolean, bool,   Boolean)
DEFINE_GETFIELD(jbyte,    byte,   Byte)
DEFINE_GETFIELD(jchar,    char,   Char)
DEFINE_GETFIELD(jshort,   short,  Short)
DEFINE_GETFIELD(jint,     int,    Int)
DEFINE_GETFIELD(jlong,    long,   Long)
DEFINE_GETFIELD(jfloat,   float,  Float)
DEFINE_GETFIELD(jdouble,  double, Double)


JNI_QUICK_ENTRY(void, jni_SetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID, jobject value))
  JNIWrapper("SetObjectField"); 
  oop o = JNIHandles::resolve_non_null(obj); 
  klassOop k = o->klass();
  int offset = jfieldIDWorkaround::from_instance_jfieldID(k, fieldID);
  // Keep JVM/DI addition small and only check enabled flag here.
  // jni_SetField_probe_nh() assumes that is not okay to create handles
  // and creates a ResetNoHandleMark.
  if (jvmdi::enabled()) {
    o = jvmdi::jni_SetField_probe_nh(thread, obj, o, k, fieldID, false, 'L', (jvalue *)value);
  }
  o->obj_field_put(offset, JNIHandles::resolve(value));
JNI_END

#define DEFINE_SETFIELD(Argument,Fieldname,Result,SigType) \
JNI_QUICK_ENTRY(void, jni_Set##Result##Field(JNIEnv *env, jobject obj, jfieldID fieldID, Argument value)) \
  JNIWrapper("Set" XSTR(Result) "Field"); \
  oop o = JNIHandles::resolve_non_null(obj); \
  klassOop k = o->klass(); \
  int offset = jfieldIDWorkaround::from_instance_jfieldID(k, fieldID); \
  /* Keep JVM/DI addition small and only check enabled flag here.       */ \
  /* jni_SetField_probe_nh() assumes that is not okay to create handles */ \
  /* and creates a ResetNoHandleMark.                                   */ \
  if (jvmdi::enabled()) { \
    o = jvmdi::jni_SetField_probe_nh(thread, obj, o, k, fieldID, false, SigType, (jvalue *)&value); \
  } \
  o->Fieldname##_field_put(offset, value); \
JNI_END

DEFINE_SETFIELD(jboolean, bool,   Boolean, 'Z')
DEFINE_SETFIELD(jbyte,    byte,   Byte,    'B')
DEFINE_SETFIELD(jchar,    char,   Char,    'C')
DEFINE_SETFIELD(jshort,   short,  Short,   'S')
DEFINE_SETFIELD(jint,     int,    Int,     'I')
DEFINE_SETFIELD(jlong,    long,   Long,    'J')
DEFINE_SETFIELD(jfloat,   float,  Float,   'F')
DEFINE_SETFIELD(jdouble,  double, Double,  'D')


//
// Accessing Static Fields
//

JNI_ENTRY(jobject, jni_ToReflectedField(JNIEnv *env, jclass cls, jfieldID fieldID, jboolean isStatic))
  JNIWrapper("ToReflectedField");
  fieldDescriptor fd;
  bool found = false;
  klassOop k = java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(cls));

  assert(jfieldIDWorkaround::is_static_jfieldID(fieldID) == isStatic, "invalid fieldID");

  if (isStatic) {
    // Static field. The fieldID a JNIid specifying the field holder and the offset within the klassOop.
    JNIid* id = jfieldIDWorkaround::from_static_jfieldID(fieldID);
    assert(id->is_static_field_id(), "invalid static field id");
    found = instanceKlass::cast(id->holder())->jni_find_local_field_from_offset(id->offset(), true, &fd);
  } else {
    // Non-static field. The fieldID is really the offset of the field within the instanceOop.
    int offset = jfieldIDWorkaround::from_instance_jfieldID(k, fieldID);    
    found = instanceKlass::cast(k)->jni_find_field_from_offset(offset, false, &fd);
  } 
  assert(found, "bad fieldID passed into jni_ToReflectedField");     
  oop reflected = Reflection::new_field(&fd, CHECK_0);
  return JNIHandles::make_local(env, reflected);
JNI_END


JNI_ENTRY(jfieldID, jni_GetStaticFieldID(JNIEnv *env, jclass clazz, const char *name, const char *sig))
  JNIWrapper("GetStaticFieldID");
  symbolHandle fieldname = oopFactory::new_symbol((char*) name, CHECK_0);
  symbolHandle signame = oopFactory::new_symbol((char*) sig, CHECK_0);
  KlassHandle k(THREAD, java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz)));
  // Make sure class is initialized before handing id's out to static fields
  Klass::cast(k())->initialize(CHECK_0);

  fieldDescriptor fd;
  if (!Klass::cast(k())->oop_is_instance() || !instanceKlass::cast(k())->jni_find_field(fieldname(), signame(), true, &fd)) {
    THROW_MSG_0(vmSymbols::java_lang_NoSuchFieldError(), (char*) name);
  }

  // A jfieldID for a static field is a JNIid specifying the field holder and the offset within the klassOop
  JNIid* id = instanceKlass::cast(fd.field_holder())->jni_id_for(fd.offset());
  debug_only(id->set_is_static_field_id();)

  debug_only(int first_offset = instanceKlass::cast(fd.field_holder())->offset_of_static_fields();)
  debug_only(int end_offset = first_offset + instanceKlass::cast(fd.field_holder())->static_field_size();)
  assert(id->offset() >= first_offset && id->offset() < end_offset, "invalid static field offset");

  return jfieldIDWorkaround::to_static_jfieldID(id);
JNI_END


JNI_ENTRY(jobject, jni_GetStaticObjectField(JNIEnv *env, jclass clazz, jfieldID fieldID))
  JNIWrapper("GetStaticObjectField");
  JNIid* id = jfieldIDWorkaround::from_static_jfieldID(fieldID);
  assert(id->is_static_field_id(), "invalid static field id");
  // Keep JVM/DI addition small and only check enabled flag here.
  // jni_GetField_probe() assumes that is okay to create handles.
  if (jvmdi::enabled()) {
    jvmdi::jni_GetField_probe(thread, NULL, NULL, id->holder(), fieldID, true);
  }
  return JNIHandles::make_local(id->holder()->obj_field(id->offset()));
JNI_END


#define DEFINE_GETSTATICFIELD(Return,Fieldname,Result) \
JNI_ENTRY(Return, jni_GetStatic##Result##Field(JNIEnv *env, jclass clazz, jfieldID fieldID)) \
  JNIWrapper("GetStatic" XSTR(Result) "Field"); \
  JNIid* id = jfieldIDWorkaround::from_static_jfieldID(fieldID); \
  assert(id->is_static_field_id(), "invalid static field id"); \
  /* Keep JVM/DI addition small and only check enabled flag here. */ \
  /* jni_GetField_probe() assumes that is okay to create handles. */ \
  if (jvmdi::enabled()) { \
    jvmdi::jni_GetField_probe(thread, NULL, NULL, id->holder(), fieldID, true); \
  } \
  return id->holder()-> Fieldname##_field (id->offset()); \
JNI_END

DEFINE_GETSTATICFIELD(jboolean, bool,   Boolean)
DEFINE_GETSTATICFIELD(jbyte,    byte,   Byte)
DEFINE_GETSTATICFIELD(jchar,    char,   Char)
DEFINE_GETSTATICFIELD(jshort,   short,  Short)
DEFINE_GETSTATICFIELD(jint,     int,    Int)
DEFINE_GETSTATICFIELD(jlong,    long,   Long)
DEFINE_GETSTATICFIELD(jfloat,   float,  Float)
DEFINE_GETSTATICFIELD(jdouble,  double, Double)


JNI_ENTRY(void, jni_SetStaticObjectField(JNIEnv *env, jclass clazz, jfieldID fieldID, jobject value))
  JNIWrapper("SetStaticObjectField");
  JNIid* id = jfieldIDWorkaround::from_static_jfieldID(fieldID);
  assert(id->is_static_field_id(), "invalid static field id");
  // Keep JVM/DI addition small and only check enabled flag here.
  // jni_SetField_probe() assumes that is okay to create handles.
  if (jvmdi::enabled()) {
    jvmdi::jni_SetField_probe(thread, NULL, NULL, id->holder(), fieldID, true, 'L', (jvalue *)&value);
  }
  id->holder()->obj_field_put(id->offset(), JNIHandles::resolve(value));
JNI_END


#define DEFINE_SETSTATICFIELD(Argument,Fieldname,Result,SigType) \
JNI_ENTRY(void, jni_SetStatic##Result##Field(JNIEnv *env, jclass clazz, jfieldID fieldID, Argument value)) \
  JNIWrapper("SetStatic" XSTR(Result) "Field"); \
  JNIid* id = jfieldIDWorkaround::from_static_jfieldID(fieldID); \
  assert(id->is_static_field_id(), "invalid static field id"); \
  /* Keep JVM/DI addition small and only check enabled flag here. */ \
  /* jni_SetField_probe() assumes that is okay to create handles. */ \
  if (jvmdi::enabled()) { \
    jvmdi::jni_SetField_probe(thread, NULL, NULL, id->holder(), fieldID, true, SigType, (jvalue *)&value); \
  } \
  id->holder()-> Fieldname##_field_put (id->offset(), value); \
JNI_END

DEFINE_SETSTATICFIELD(jboolean, bool,   Boolean, 'Z')
DEFINE_SETSTATICFIELD(jbyte,    byte,   Byte,    'B')
DEFINE_SETSTATICFIELD(jchar,    char,   Char,    'C')
DEFINE_SETSTATICFIELD(jshort,   short,  Short,   'S')
DEFINE_SETSTATICFIELD(jint,     int,    Int,     'I')
DEFINE_SETSTATICFIELD(jlong,    long,   Long,    'J')
DEFINE_SETSTATICFIELD(jfloat,   float,  Float,   'F')
DEFINE_SETSTATICFIELD(jdouble,  double, Double,  'D')


//
// String Operations
//

// Unicode Interface

JNI_ENTRY(jstring, jni_NewString(JNIEnv *env, const jchar *unicodeChars, jsize len))
  JNIWrapper("NewString");
  oop string=java_lang_String::create_from_unicode((jchar*) unicodeChars, len, CHECK_0);
  return (jstring) JNIHandles::make_local(env, string);
JNI_END


JNI_QUICK_ENTRY(jsize, jni_GetStringLength(JNIEnv *env, jstring string))
  JNIWrapper("GetStringLength");
  return java_lang_String::length(JNIHandles::resolve_non_null(string));
JNI_END


JNI_QUICK_ENTRY(const jchar*, jni_GetStringChars(JNIEnv *env, jstring string, jboolean *isCopy))
  JNIWrapper("GetStringChars");
  //%note jni_5
  if (isCopy != NULL) {
    *isCopy = JNI_TRUE;
  }
  oop s = JNIHandles::resolve_non_null(string);
  int s_len = java_lang_String::length(s);
  typeArrayOop s_value = java_lang_String::value(s);
  int s_offset = java_lang_String::offset(s);
  jchar* buf = NEW_C_HEAP_ARRAY(jchar, s_len + 1);  // add one for zero termination
  if (s_len > 0) {
    memcpy(buf, s_value->char_at_addr(s_offset), sizeof(jchar)*s_len);
  }
  buf[s_len] = 0;
  return buf;
JNI_END


JNI_QUICK_ENTRY(void, jni_ReleaseStringChars(JNIEnv *env, jstring str, const jchar *chars))
  JNIWrapper("ReleaseStringChars");
  //%note jni_6
  if (chars != NULL) {
    oop s = JNIHandles::resolve_non_null(str);
    int s_len = java_lang_String::length(s);
    typeArrayOop s_value = java_lang_String::value(s);
    int s_offset = java_lang_String::offset(s);
    if (s_len > 0) {
      memcpy(s_value->char_at_addr(s_offset), chars, sizeof(jchar)*s_len);
    }
    FreeHeap((void*) chars);
  }
JNI_END


// UTF Interface

JNI_ENTRY(jstring, jni_NewStringUTF(JNIEnv *env, const char *bytes))
  JNIWrapper("NewStringUTF");
  oop result = java_lang_String::create_from_str((char*) bytes, CHECK_0);
  return (jstring) JNIHandles::make_local(env, result);
JNI_END


JNI_ENTRY(jsize, jni_GetStringUTFLength(JNIEnv *env, jstring string))
  JNIWrapper("GetStringUTFLength");
  return java_lang_String::utf8_length(JNIHandles::resolve_non_null(string));
JNI_END


JNI_ENTRY(const char*, jni_GetStringUTFChars(JNIEnv *env, jstring string, jboolean *isCopy))
  JNIWrapper("GetStringUTFChars");
  ResourceMark rm;
  char* str = java_lang_String::as_utf8_string(JNIHandles::resolve_non_null(string));
  int length = strlen(str);
  char* result = AllocateHeap(length+1, "GetStringUTFChars");
  strcpy(result, str);
  if (isCopy != NULL) *isCopy = JNI_TRUE;
  return result;
JNI_END


JNI_LEAF(void, jni_ReleaseStringUTFChars(JNIEnv *env, jstring str, const char *chars))
  JNIWrapper("ReleaseStringUTFChars");
  FreeHeap((char*) chars);
JNI_END


JNI_QUICK_ENTRY(jsize, jni_GetArrayLength(JNIEnv *env, jarray array))
  JNIWrapper("GetArrayLength");
  arrayOop a = arrayOop(JNIHandles::resolve_non_null(array));
  assert(a->is_array(), "must be array");
  return a->length();
JNI_END


//
// Object Array Operations
//

JNI_ENTRY(jobjectArray, jni_NewObjectArray(JNIEnv *env, jsize length, jclass elementClass, jobject initialElement))
  JNIWrapper("NewObjectArray");
  KlassHandle ek(THREAD, java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(elementClass)));
  klassOop ako = Klass::cast(ek())->array_klass(CHECK_0);
  KlassHandle ak = KlassHandle(THREAD, ako);
  objArrayKlass::cast(ak())->initialize(CHECK_0);
  objArrayOop result = objArrayKlass::cast(ak())->allocate(length, CHECK_0);
  oop initial_value = JNIHandles::resolve(initialElement);
  if (initial_value != NULL) {  // array already initialized with NULL
    for (int index = 0; index < length; index++) {
      result->obj_at_put(index, initial_value);
    }
  }
  return (jobjectArray) JNIHandles::make_local(env, result);
JNI_END


JNI_ENTRY(jobject, jni_GetObjectArrayElement(JNIEnv *env, jobjectArray array, jsize index))
  JNIWrapper("GetObjectArrayElement");
  objArrayOop a = objArrayOop(JNIHandles::resolve_non_null(array));
  if (a->is_within_bounds(index)) {
    return JNIHandles::make_local(env, a->obj_at(index));
  } else {
    THROW_0(vmSymbols::java_lang_ArrayIndexOutOfBoundsException());
  }
JNI_END


JNI_ENTRY(void, jni_SetObjectArrayElement(JNIEnv *env, jobjectArray array, jsize index, jobject value))
  JNIWrapper("SetObjectArrayElement");
  objArrayOop a = objArrayOop(JNIHandles::resolve_non_null(array));
  oop v = JNIHandles::resolve(value);
  if (a->is_within_bounds(index)) {
    if (v == NULL || v->is_a(objArrayKlass::cast(a->klass())->element_klass())) {
      a->obj_at_put(index, v);
    } else {
      THROW(vmSymbols::java_lang_ArrayStoreException());  
    }
  } else {
    THROW(vmSymbols::java_lang_ArrayIndexOutOfBoundsException());
  }
JNI_END


#define DEFINE_NEWSCALARARRAY(Return,Allocator,Result) \
JNI_ENTRY(Return, \
          jni_New##Result##Array(JNIEnv *env, jsize len)) \
  JNIWrapper("New" XSTR(Result) "Array"); \
  oop obj= oopFactory::Allocator(len, CHECK_0); \
  return (Return) JNIHandles::make_local(env, obj); \
JNI_END

DEFINE_NEWSCALARARRAY(jbooleanArray, new_boolArray,   Boolean)
DEFINE_NEWSCALARARRAY(jbyteArray,    new_byteArray,   Byte)
DEFINE_NEWSCALARARRAY(jshortArray,   new_shortArray,  Short)
DEFINE_NEWSCALARARRAY(jcharArray,    new_charArray,   Char)
DEFINE_NEWSCALARARRAY(jintArray,     new_intArray,    Int)
DEFINE_NEWSCALARARRAY(jlongArray,    new_longArray,   Long)
DEFINE_NEWSCALARARRAY(jfloatArray,   new_singleArray, Float)
DEFINE_NEWSCALARARRAY(jdoubleArray,  new_doubleArray, Double)


#define DEFINE_GETSCALARARRAYELEMENTS(ElementTag,ElementType,Result, Tag) \
JNI_QUICK_ENTRY(ElementType*, \
          jni_Get##Result##ArrayElements(JNIEnv *env, ElementType##Array array, jboolean *isCopy)) \
  JNIWrapper("Get" XSTR(Result) "ArrayElements"); \
  /* allocate an chunk of memory in c land */ \
  typeArrayOop a = typeArrayOop(JNIHandles::resolve_non_null(array)); \
  ElementType* result = NEW_C_HEAP_ARRAY(ElementType, a->length()); \
  /* copy the array to the c chunk */ \
  memcpy(result, a->Tag##_at_addr(0), sizeof(ElementType)*a->length()); \
  if (isCopy) *isCopy = JNI_TRUE; \
  return result; \
JNI_END

DEFINE_GETSCALARARRAYELEMENTS(T_BOOLEAN, jboolean, Boolean, bool)
DEFINE_GETSCALARARRAYELEMENTS(T_BYTE,    jbyte,    Byte,    byte)
DEFINE_GETSCALARARRAYELEMENTS(T_SHORT,   jshort,   Short,   short)
DEFINE_GETSCALARARRAYELEMENTS(T_CHAR,    jchar,    Char,    char)
DEFINE_GETSCALARARRAYELEMENTS(T_INT,     jint,     Int,     int)
DEFINE_GETSCALARARRAYELEMENTS(T_LONG,    jlong,    Long,    long)
DEFINE_GETSCALARARRAYELEMENTS(T_FLOAT,   jfloat,   Float,   float)
DEFINE_GETSCALARARRAYELEMENTS(T_DOUBLE,  jdouble,  Double,  double)


#define DEFINE_RELEASESCALARARRAYELEMENTS(ElementTag,ElementType,Result,Tag) \
JNI_QUICK_ENTRY(void, \
          jni_Release##Result##ArrayElements(JNIEnv *env, ElementType##Array array, \
                                             ElementType *buf, jint mode)) \
  JNIWrapper("Release" XSTR(Result) "ArrayElements"); \
  typeArrayOop a = typeArrayOop(JNIHandles::resolve_non_null(array)); \
  if ((mode == 0) || (mode == JNI_COMMIT)) { \
    memcpy(a->Tag##_at_addr(0), buf, sizeof(ElementType)*a->length()); \
  } \
  if ((mode == 0) || (mode == JNI_ABORT)) { \
    FreeHeap(buf); \
  } \
JNI_END

DEFINE_RELEASESCALARARRAYELEMENTS(T_BOOLEAN, jboolean, Boolean, bool)
DEFINE_RELEASESCALARARRAYELEMENTS(T_BYTE,    jbyte,    Byte,    byte)
DEFINE_RELEASESCALARARRAYELEMENTS(T_SHORT,   jshort,   Short,   short)
DEFINE_RELEASESCALARARRAYELEMENTS(T_CHAR,    jchar,    Char,    char)
DEFINE_RELEASESCALARARRAYELEMENTS(T_INT,     jint,     Int,     int)
DEFINE_RELEASESCALARARRAYELEMENTS(T_LONG,    jlong,    Long,    long)
DEFINE_RELEASESCALARARRAYELEMENTS(T_FLOAT,   jfloat,   Float,   float)
DEFINE_RELEASESCALARARRAYELEMENTS(T_DOUBLE,  jdouble,  Double,  double)

#define DEFINE_GETSCALARARRAYREGION(ElementTag,ElementType,Result, Tag) \
JNI_ENTRY(void, \
jni_Get##Result##ArrayRegion(JNIEnv *env, ElementType##Array array, jsize start, \
			       jsize len, ElementType *buf)) \
  JNIWrapper("Get" XSTR(Result) "ArrayRegion"); \
  typeArrayOop src = typeArrayOop(JNIHandles::resolve_non_null(array)); \
  if (start < 0 || len < 0 || ((unsigned int)start + (unsigned int)len > (unsigned int)src->length())) { \
    THROW(vmSymbols::java_lang_ArrayIndexOutOfBoundsException()); \
  } else { \
    if (len > 0) { \
      int sc = typeArrayKlass::cast(src->klass())->scale(); \
      memcpy((u_char*) buf, \
             (u_char*) src->Tag##_at_addr(start), \
             len * sc); \
    } \
  } \
JNI_END

DEFINE_GETSCALARARRAYREGION(T_BOOLEAN, jboolean,Boolean, bool)
DEFINE_GETSCALARARRAYREGION(T_BYTE,    jbyte,   Byte,    byte)
DEFINE_GETSCALARARRAYREGION(T_SHORT,   jshort,  Short,   short)
DEFINE_GETSCALARARRAYREGION(T_CHAR,    jchar,   Char,    char)
DEFINE_GETSCALARARRAYREGION(T_INT,     jint,    Int,     int)
DEFINE_GETSCALARARRAYREGION(T_LONG,    jlong,   Long,    long)
DEFINE_GETSCALARARRAYREGION(T_FLOAT,   jfloat,  Float,   float)
DEFINE_GETSCALARARRAYREGION(T_DOUBLE,  jdouble, Double,  double)

#define DEFINE_SETSCALARARRAYREGION(ElementTag,ElementType,Result, Tag) \
JNI_ENTRY(void, \
jni_Set##Result##ArrayRegion(JNIEnv *env, ElementType##Array array, jsize start, \
			       jsize len, ElementType *buf)) \
  JNIWrapper("Set" XSTR(Result) "ArrayRegion"); \
  typeArrayOop dst = typeArrayOop(JNIHandles::resolve_non_null(array)); \
  if (start < 0 || len < 0 || ((unsigned int)start + (unsigned int)len > (unsigned int)dst->length())) { \
    THROW(vmSymbols::java_lang_ArrayIndexOutOfBoundsException()); \
  } else { \
    if (len > 0) { \
      int sc = typeArrayKlass::cast(dst->klass())->scale(); \
      memcpy((u_char*) dst->Tag##_at_addr(start), \
             (u_char*) buf, \
             len * sc); \
    } \
  } \
JNI_END

DEFINE_SETSCALARARRAYREGION(T_BOOLEAN, jboolean, Boolean, bool)
DEFINE_SETSCALARARRAYREGION(T_BYTE,    jbyte,    Byte,    byte)
DEFINE_SETSCALARARRAYREGION(T_SHORT,   jshort,   Short,   short)
DEFINE_SETSCALARARRAYREGION(T_CHAR,    jchar,    Char,    char)
DEFINE_SETSCALARARRAYREGION(T_INT,     jint,     Int,     int)
DEFINE_SETSCALARARRAYREGION(T_LONG,    jlong,    Long,    long)
DEFINE_SETSCALARARRAYREGION(T_FLOAT,   jfloat,   Float,   float)
DEFINE_SETSCALARARRAYREGION(T_DOUBLE,  jdouble,  Double,  double)


//
// Interception of natives
//

static bool register_native(klassOop k, symbolOop name, symbolOop signature, address entry, TRAPS) {
  methodOop method = Klass::cast(k)->lookup_method(name, signature);
  if (method == NULL || !method->is_native()) return false;
  method->set_native_function(entry);
  if (PrintJNIResolving) {
    ResourceMark rm;
    tty->print_cr("[Registering JNI native method %s.%s]", 
      Klass::cast(method->method_holder())->external_name(), 
      method->name()->as_C_string());
  }
  return true;
}


JNI_ENTRY(jint, jni_RegisterNatives(JNIEnv *env, jclass clazz,
                                    const JNINativeMethod *methods, jint nMethods))  
  JNIWrapper("RegisterNatives");
  KlassHandle h_k(thread, java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz)));

  for (int index = 0; index < nMethods; index++) {
    symbolHandle h_name = oopFactory::new_symbol(methods[index].name, CHECK_0);    
    symbolOop signature_oop = oopFactory::new_symbol(methods[index].signature, CHECK_0);
    bool res = register_native(h_k(), h_name(), signature_oop, (address) methods[index].fnPtr, CHECK_0);
    if (!res) return -1;
  }
  return 0;
JNI_END


JNI_ENTRY(jint, jni_UnregisterNatives(JNIEnv *env, jclass clazz))  
  JNIWrapper("UnregisterNatives");
  klassOop k   = java_lang_Class::as_klassOop(JNIHandles::resolve_non_null(clazz));
  //%note jni_2
  if (Klass::cast(k)->oop_is_instance()) {
    for (int index = 0; index < instanceKlass::cast(k)->methods()->length(); index++) {
      methodOop m = methodOop(instanceKlass::cast(k)->methods()->obj_at(index));
      if (m->is_native()) {
        m->set_native_function(NULL);
        m->set_signature_handler(NULL);
      }
    }
  }
  return 0;
JNI_END

//
// Monitor functions
//

JNI_ENTRY(jint, jni_MonitorEnter(JNIEnv *env, jobject jobj))

  // If the object is null, we can't do anything with it
  if (jobj == NULL) {
    THROW_(vmSymbols::java_lang_NullPointerException(), JNI_ERR);
  }

  Handle obj(thread, JNIHandles::resolve_non_null(jobj));
  ObjectSynchronizer::jni_enter(obj, CHECK_(JNI_ERR));
  return JNI_OK;

JNI_END

JNI_ENTRY(jint, jni_MonitorExit(JNIEnv *env, jobject jobj))

  // Don't do anything with a null object
  if (jobj == NULL) {
    THROW_(vmSymbols::java_lang_NullPointerException(), JNI_ERR);
  }

  Handle obj(JNIHandles::resolve_non_null(jobj));
  ObjectSynchronizer::jni_exit(obj(), CHECK_(JNI_ERR));

  return JNI_OK;
JNI_END

//
// Extensions
//

JNI_ENTRY(void, jni_GetStringRegion(JNIEnv *env, jstring string, jsize start, jsize len, jchar *buf))
  JNIWrapper("GetStringRegion");
  oop s = JNIHandles::resolve_non_null(string);
  int s_len = java_lang_String::length(s);
  if (start < 0 || len < 0 || start + len > s_len) {
    THROW(vmSymbols::java_lang_StringIndexOutOfBoundsException());
  } else {
    if (len > 0) {
      int s_offset = java_lang_String::offset(s);
      typeArrayOop s_value = java_lang_String::value(s);
      memcpy(buf, s_value->char_at_addr(s_offset+start), sizeof(jchar)*len);
    }
  }
JNI_END


JNI_ENTRY(void, jni_GetStringUTFRegion(JNIEnv *env, jstring string, jsize start, jsize len, char *buf))
  JNIWrapper("GetStringUTFRegion");
  oop s = JNIHandles::resolve_non_null(string);
  int s_len = java_lang_String::length(s);
  if (start < 0 || len < 0 || start + len > s_len) {
    THROW(vmSymbols::java_lang_StringIndexOutOfBoundsException());
  } else {
    //%note jni_7
    if (len > 0) {
      ResourceMark rm;
      char *utf_region = java_lang_String::as_utf8_string(s, start, len);
      int utf_len = strlen(utf_region);
      memcpy(buf, utf_region, utf_len);
      buf[utf_len] = 0;
    } else {
      // JDK null-terminates the buffer even in len is zero
      if (buf != NULL) {
        buf[0] = 0; 
      }
    }
  }
JNI_END


JNI_QUICK_ENTRY(void*, jni_GetPrimitiveArrayCritical(JNIEnv *env, jarray array, jboolean *isCopy))
  JNIWrapper("GetPrimitiveArrayCritical");
  GC_locker::lock();
  if (isCopy != NULL) {
    *isCopy = JNI_FALSE;
  }
  oop a = JNIHandles::resolve_non_null(array);
  assert(a->is_array(), "just checking");
  BasicType type;
  if (a->is_objArray()) {
    type = T_OBJECT;
  } else {
    type = typeArrayKlass::cast(a->klass())->type();
  }
  return arrayOop(a)->base(type);
JNI_END


JNI_LEAF(void, jni_ReleasePrimitiveArrayCritical(JNIEnv *env, jarray array, void *carray, jint mode))
  JNIWrapper("ReleasePrimitiveArrayCritical");
  // The array, carray and mode arguments are ignored
  GC_locker::unlock();
JNI_END


JNI_QUICK_ENTRY(const jchar*, jni_GetStringCritical(JNIEnv *env, jstring string, jboolean *isCopy))
  JNIWrapper("GetStringCritical");
  GC_locker::lock();
  if (isCopy != NULL) {
    *isCopy = JNI_FALSE;
  }
  oop s = JNIHandles::resolve_non_null(string);
  int s_len = java_lang_String::length(s);
  typeArrayOop s_value = java_lang_String::value(s);
  int s_offset = java_lang_String::offset(s);
  if (s_len > 0) {
    return s_value->char_at_addr(s_offset);
  } else {
    return (jchar*) s_value->base(T_CHAR);
  }
JNI_END


JNI_LEAF(void, jni_ReleaseStringCritical(JNIEnv *env, jstring str, const jchar *chars))
  JNIWrapper("ReleaseStringCritical");
  // The str and chars arguments are ignored
  GC_locker::unlock();
JNI_END


JNI_ENTRY(jweak, jni_NewWeakGlobalRef(JNIEnv *env, jobject ref))
  JNIWrapper("jni_NewWeakGlobalRef");
  Handle ref_handle(thread, JNIHandles::resolve(ref));
  return JNIHandles::make_weak_global(ref_handle);
JNI_END


JNI_QUICK_ENTRY(void, jni_DeleteWeakGlobalRef(JNIEnv *env, jweak ref))
  JNIWrapper("jni_DeleteWeakGlobalRef");
  JNIHandles::destroy_weak_global(ref);
JNI_END


JNI_QUICK_ENTRY(jboolean, jni_ExceptionCheck(JNIEnv *env))
  JNIWrapper("jni_ExceptionCheck");
  return (thread->has_pending_exception()) ? JNI_TRUE : JNI_FALSE;
JNI_END


JNI_LEAF(jint, jni_GetVersion(JNIEnv *env))
  JNIWrapper("GetVersion");
  return JNI_VERSION_1_2;
JNI_END

extern struct JavaVM_ main_vm;

JNI_LEAF(jint, jni_GetJavaVM(JNIEnv *env, JavaVM **vm))
  JNIWrapper("jni_GetJavaVM");
  *vm  = (JavaVM *)(&main_vm);
  return JNI_OK;
JNI_END

// Structure containing all jni functions
const struct JNINativeInterface_ jni_NativeInterface = {
    NULL,
    NULL,
    NULL,

    NULL,

    jni_GetVersion,

    jni_DefineClass,
    jni_FindClass,

    jni_FromReflectedMethod,
    jni_FromReflectedField,

    jni_ToReflectedMethod,

    jni_GetSuperclass,
    jni_IsAssignableFrom,

    jni_ToReflectedField,

    jni_Throw,
    jni_ThrowNew,
    jni_ExceptionOccurred,
    jni_ExceptionDescribe,
    jni_ExceptionClear,
    jni_FatalError,

    jni_PushLocalFrame,
    jni_PopLocalFrame,

    jni_NewGlobalRef,
    jni_DeleteGlobalRef,
    jni_DeleteLocalRef,
    jni_IsSameObject,

    jni_NewLocalRef,
    jni_EnsureLocalCapacity,

    jni_AllocObject,
    jni_NewObject,
    jni_NewObjectV,
    jni_NewObjectA,

    jni_GetObjectClass,
    jni_IsInstanceOf,

    jni_GetMethodID,

    jni_CallObjectMethod,
    jni_CallObjectMethodV,
    jni_CallObjectMethodA,
    jni_CallBooleanMethod,
    jni_CallBooleanMethodV,
    jni_CallBooleanMethodA,
    jni_CallByteMethod,
    jni_CallByteMethodV,
    jni_CallByteMethodA,
    jni_CallCharMethod,
    jni_CallCharMethodV,
    jni_CallCharMethodA,
    jni_CallShortMethod,
    jni_CallShortMethodV,
    jni_CallShortMethodA,
    jni_CallIntMethod,
    jni_CallIntMethodV,
    jni_CallIntMethodA,
    jni_CallLongMethod,
    jni_CallLongMethodV,
    jni_CallLongMethodA,
    jni_CallFloatMethod,
    jni_CallFloatMethodV,
    jni_CallFloatMethodA,
    jni_CallDoubleMethod,
    jni_CallDoubleMethodV,
    jni_CallDoubleMethodA,
    jni_CallVoidMethod,
    jni_CallVoidMethodV,
    jni_CallVoidMethodA,

    jni_CallNonvirtualObjectMethod,
    jni_CallNonvirtualObjectMethodV,
    jni_CallNonvirtualObjectMethodA,
    jni_CallNonvirtualBooleanMethod,
    jni_CallNonvirtualBooleanMethodV,
    jni_CallNonvirtualBooleanMethodA,
    jni_CallNonvirtualByteMethod,
    jni_CallNonvirtualByteMethodV,
    jni_CallNonvirtualByteMethodA,
    jni_CallNonvirtualCharMethod,
    jni_CallNonvirtualCharMethodV,
    jni_CallNonvirtualCharMethodA,
    jni_CallNonvirtualShortMethod,
    jni_CallNonvirtualShortMethodV,
    jni_CallNonvirtualShortMethodA,
    jni_CallNonvirtualIntMethod,
    jni_CallNonvirtualIntMethodV,
    jni_CallNonvirtualIntMethodA,
    jni_CallNonvirtualLongMethod,
    jni_CallNonvirtualLongMethodV,
    jni_CallNonvirtualLongMethodA,
    jni_CallNonvirtualFloatMethod,
    jni_CallNonvirtualFloatMethodV,
    jni_CallNonvirtualFloatMethodA,
    jni_CallNonvirtualDoubleMethod,
    jni_CallNonvirtualDoubleMethodV,
    jni_CallNonvirtualDoubleMethodA,
    jni_CallNonvirtualVoidMethod,
    jni_CallNonvirtualVoidMethodV,
    jni_CallNonvirtualVoidMethodA,

    jni_GetFieldID,

    jni_GetObjectField,
    jni_GetBooleanField,
    jni_GetByteField,
    jni_GetCharField,
    jni_GetShortField,
    jni_GetIntField,
    jni_GetLongField,
    jni_GetFloatField,
    jni_GetDoubleField,

    jni_SetObjectField,
    jni_SetBooleanField,
    jni_SetByteField,
    jni_SetCharField,
    jni_SetShortField,
    jni_SetIntField,
    jni_SetLongField,
    jni_SetFloatField,
    jni_SetDoubleField,

    jni_GetStaticMethodID,

    jni_CallStaticObjectMethod,
    jni_CallStaticObjectMethodV,
    jni_CallStaticObjectMethodA,
    jni_CallStaticBooleanMethod,
    jni_CallStaticBooleanMethodV,
    jni_CallStaticBooleanMethodA,
    jni_CallStaticByteMethod,
    jni_CallStaticByteMethodV,
    jni_CallStaticByteMethodA,
    jni_CallStaticCharMethod,
    jni_CallStaticCharMethodV,
    jni_CallStaticCharMethodA,
    jni_CallStaticShortMethod,
    jni_CallStaticShortMethodV,
    jni_CallStaticShortMethodA,
    jni_CallStaticIntMethod,
    jni_CallStaticIntMethodV,
    jni_CallStaticIntMethodA,
    jni_CallStaticLongMethod,
    jni_CallStaticLongMethodV,
    jni_CallStaticLongMethodA,
    jni_CallStaticFloatMethod,
    jni_CallStaticFloatMethodV,
    jni_CallStaticFloatMethodA,
    jni_CallStaticDoubleMethod,
    jni_CallStaticDoubleMethodV,
    jni_CallStaticDoubleMethodA,
    jni_CallStaticVoidMethod,
    jni_CallStaticVoidMethodV,
    jni_CallStaticVoidMethodA,

    jni_GetStaticFieldID,

    jni_GetStaticObjectField,
    jni_GetStaticBooleanField,
    jni_GetStaticByteField,
    jni_GetStaticCharField,
    jni_GetStaticShortField,
    jni_GetStaticIntField,
    jni_GetStaticLongField,
    jni_GetStaticFloatField,
    jni_GetStaticDoubleField,

    jni_SetStaticObjectField,
    jni_SetStaticBooleanField,
    jni_SetStaticByteField,
    jni_SetStaticCharField,
    jni_SetStaticShortField,
    jni_SetStaticIntField,
    jni_SetStaticLongField,
    jni_SetStaticFloatField,
    jni_SetStaticDoubleField,

    jni_NewString,
    jni_GetStringLength,
    jni_GetStringChars,
    jni_ReleaseStringChars,

    jni_NewStringUTF,
    jni_GetStringUTFLength,
    jni_GetStringUTFChars,
    jni_ReleaseStringUTFChars,

    jni_GetArrayLength,
 
    jni_NewObjectArray,
    jni_GetObjectArrayElement,
    jni_SetObjectArrayElement,

    jni_NewBooleanArray,
    jni_NewByteArray,
    jni_NewCharArray,
    jni_NewShortArray,
    jni_NewIntArray,
    jni_NewLongArray,
    jni_NewFloatArray,
    jni_NewDoubleArray,

    jni_GetBooleanArrayElements,
    jni_GetByteArrayElements,
    jni_GetCharArrayElements,
    jni_GetShortArrayElements,
    jni_GetIntArrayElements,
    jni_GetLongArrayElements,
    jni_GetFloatArrayElements,
    jni_GetDoubleArrayElements,

    jni_ReleaseBooleanArrayElements,
    jni_ReleaseByteArrayElements,
    jni_ReleaseCharArrayElements,
    jni_ReleaseShortArrayElements,
    jni_ReleaseIntArrayElements,
    jni_ReleaseLongArrayElements,
    jni_ReleaseFloatArrayElements,
    jni_ReleaseDoubleArrayElements,

    jni_GetBooleanArrayRegion,
    jni_GetByteArrayRegion,
    jni_GetCharArrayRegion,
    jni_GetShortArrayRegion,
    jni_GetIntArrayRegion,
    jni_GetLongArrayRegion,
    jni_GetFloatArrayRegion,
    jni_GetDoubleArrayRegion,

    jni_SetBooleanArrayRegion,
    jni_SetByteArrayRegion,
    jni_SetCharArrayRegion,
    jni_SetShortArrayRegion,
    jni_SetIntArrayRegion,
    jni_SetLongArrayRegion,
    jni_SetFloatArrayRegion,
    jni_SetDoubleArrayRegion,

    jni_RegisterNatives,
    jni_UnregisterNatives,

    jni_MonitorEnter,
    jni_MonitorExit,

    jni_GetJavaVM,

    jni_GetStringRegion,
    jni_GetStringUTFRegion,

    jni_GetPrimitiveArrayCritical,
    jni_ReleasePrimitiveArrayCritical,

    jni_GetStringCritical,
    jni_ReleaseStringCritical,

    jni_NewWeakGlobalRef,
    jni_DeleteWeakGlobalRef,

    jni_ExceptionCheck,
};


// Returns the function structure
const struct JNINativeInterface_* jni_functions() {
  return &jni_NativeInterface;
}


// Invocation API


// Forward declaration
extern const struct JNIInvokeInterface_ jni_InvokeInterface;

// Global invocation API vars
jint vm_created = 0;
struct JavaVM_ main_vm = {&jni_InvokeInterface};


#define JAVASTACKSIZE (400 * 1024)     /* Default size of a thread java stack */
#define PROCSTACKSIZE 0               /* 0 means default size in HPI */
enum { VERIFY_NONE, VERIFY_REMOTE, VERIFY_ALL };



_JNI_IMPORT_OR_EXPORT_ jint JNICALL JNI_GetDefaultJavaVMInitArgs(void *args_) {
  JDK1_1InitArgs *args = (JDK1_1InitArgs *)args_;
  if (args->version == JNI_VERSION_1_2) {
    return JNI_OK;
  }
  args->version         = JNI_VERSION_1_1;
  args->properties      = 0;
  args->checkSource     = 0;
  args->javaStackSize   = 0;
  args->verifyMode      = 0;
  args->classpath       = NULL;
  args->vfprintf        = 0;
  args->exit            = 0;
  args->abort           = 0;
  args->enableClassGC   = 1;
  args->disableAsyncGC  = 0;
  args->enableVerboseGC = 0;
  args->verbose         = 0;
  args->debugging       = JNI_FALSE;
  args->debugPort       = 0;

  args->nativeStackSize = 0;
  args->minHeapSize     = 0;
  args->maxHeapSize     = 0;
  args->classpath       = os::get_java_1_1_system_classpath();
  if (args->classpath == NULL) {
    return JNI_ERR;
  }
  return JNI_OK;
}


_JNI_IMPORT_OR_EXPORT_ jint JNICALL JNI_CreateJavaVM(JavaVM **vm, void **penv, void *args) {
  // At the moment it's only possible to have one Java VM, 
  // since some of the runtime state is in global variables. 

  // We cannot use our mutex locks here, since they only work on
  // Threads. We do an atomic compare and exchange to ensure only
  // one thread can call this method at a time
  if (atomic::exchange(&vm_created, 1) == 1) {
    return JNI_ERR;
  }
  assert(vm_created == 1, "vm_created is true during the creation");

  jint result = Threads::create_vm((JavaVMInitArgs*) args);
  if (result == JNI_OK) {
    JavaThread *thread = JavaThread::current();
    *vm = (JavaVM *)(&main_vm);
    *(JNIEnv**)penv = thread->jni_environment();      
    // Notify jvmdi
    if (jvmdi::enabled()) {
       jvmdi::post_thread_start_event(thread);
    }
    // Check if we should compile all classes on bootclasspath
    NOT_PRODUCT(if (CompileTheWorld) ClassLoader::compile_the_world();)
    // Since this is not a JVM_ENTRY we have to set the thread state manually before leaving.
    ThreadStateTransition::transition(thread, _thread_in_vm, _thread_in_native); 
  } else {
    // Creation failed. We must reset vm_created
    vm_created = 0;
    *vm = 0;
    *(JNIEnv**)penv = 0;    
  }

  return result;
}


_JNI_IMPORT_OR_EXPORT_ jint JNICALL JNI_GetCreatedJavaVMs(JavaVM **vm_buf, jsize bufLen, jsize *numVMs) {
  JNIWrapper("GetCreatedJavaVMs");
  if (vm_created) {
    if (numVMs != NULL) *numVMs = 1;
    if (bufLen > 0)     *vm_buf = (JavaVM *)(&main_vm);
  } else {
    if (numVMs != NULL) *numVMs = 0;
  }
  return JNI_OK;
}

extern "C" {                 

jint JNICALL jni_DestroyJavaVM(JavaVM *vm) {
  JNIWrapper("DestroyJavaVM");
  JNIEnv *env;
  jint res = vm->AttachCurrentThread((void **)&env, NULL);
  if (res != JNI_OK) return res;

  // Since this is not a JVM_ENTRY we have to set the thread state manually before entering.
  JavaThread* thread = JavaThread::current();
  ThreadStateTransition::transition(thread, _thread_in_native, _thread_in_vm);
  if (Threads::destroy_vm()) {
    // Should not change thread state, VM is gone
    vm_created = false;
    return JNI_OK;
  } else {
    ThreadStateTransition::transition(thread, _thread_in_vm, _thread_in_native);
    return JNI_ERR;
  }
}



jint JNICALL jni_AttachCurrentThread(JavaVM *vm, void **penv, void *_args) {
  JNIWrapper("AttachCurrentThread");
  JavaVMAttachArgs *args = (JavaVMAttachArgs *) _args;

  // Verify that the VM has been initialized
  if (!vm_created) {
    return JNI_ERR;
  }

  // Check below commented out from JDK1.2fcs as well
  /*
  if (args && (args->version != JNI_VERSION_1_1 || args->version != JNI_VERSION_1_2)) {
    return JNI_EVERSION;
  }
  */

  Thread* t = ThreadLocalStorage::get_thread_slow();
  if (t != NULL) {
    // If the thread has been attacted this operation is a no-op
    *(JNIEnv**)penv = ((JavaThread*) t)->jni_environment();
    return JNI_OK;
  }

  // Create a thread
  JavaThread* thread = new JavaThread();

  // Set correct safepoint info. The thread is going to call into Java when
  // initializing the Java level thread object. Hence, the correct state must
  // be set in order for the Safepoint code to deal with it correctly.  
  thread->set_thread_state(_thread_in_vm);    
  // Must do this before initialize_thread_local_storage
  thread->record_stack_base_and_size();    
  thread->initialize_thread_local_storage();
  os::create_attached_thread(thread);

  // Crucial that we do not have a safepoint check for this thread, since it has
  // not been added to the Thread list yet.
  { Threads_lock->lock_without_safepoint_check();    
    // This must be inside this lock in order to get FullGCALot to work properly, i.e., to
    // avoid this thread trying to do a GC before it is added to the thread-list
    thread->set_active_handles(JNIHandleBlock::allocate_block());  
    Threads::add(thread);
    Threads_lock->unlock();
  }

  // Create thread group and name info from attach arguments
  oop group = NULL;
  char* thread_name = NULL;
  if (args != NULL && args->version == JNI_VERSION_1_2) {
    group = JNIHandles::resolve(args->group);
    thread_name = args->name; // may be NULL
  }
  if (group == NULL) group = Universe::main_thread_group();

  // Create Java level thread object and attach it to this thread
  EXCEPTION_MARK;
  HandleMark hm(THREAD);
  Handle thread_group(THREAD, group);
  thread->allocate_threadObj(thread_group, thread_name, THREAD);
  if (HAS_PENDING_EXCEPTION) {
    CLEAR_PENDING_EXCEPTION;
    // FIX THIS: Cleanup is missing
    return JNI_ERR;
  }

  // Enable stack overflow checks
  if (UseStackBanging) {
    thread->enable_stack_yellow_zone();
    thread->enable_stack_red_zone();
  }

  // Notify the debugger
  if (jvmdi::enabled()) {
    jvmdi::post_thread_start_event(thread);
  }
  // Notify jvmpi
  if (jvmpi::is_event_enabled(JVMPI_EVENT_THREAD_START)) {
    jvmpi::post_thread_start_event(thread);
  }
 
  *(JNIEnv**)penv = thread->jni_environment();

  // Now leaving the VM, so change thread_state. This is normally automatically taken care
  // of in the JVM_ENTRY. But in this situation we have to do it manually. Notice, that by
  // using ThreadStateTransition::transition, we do a callback to the safepoint code if 
  // needed.

  ThreadStateTransition::transition(thread, _thread_in_vm, _thread_in_native);

  return JNI_OK;
}


jint JNICALL jni_DetachCurrentThread(JavaVM *vm)  {
  JNIWrapper("DetachCurrentThread");  

  // If the thread has been deattacted the operations is a no-op
  if (ThreadLocalStorage::thread() == NULL) return JNI_OK;

  JavaThread* thread = JavaThread::current();

  // Safepoint support. Have to do call-back to safepoint code, if in the
  // middel of a safepoint operation
  ThreadStateTransition::transition(thread, _thread_in_native, _thread_in_vm);
  
  // XXX: Note that JavaThread::exit() call below removes the guards on the
  // stack pages set up via enable_stack_{red,yellow}_zone() calls
  // above in jni_AttachCurrentThread. Unfortunately, while the setting
  // of the guards is visible in jni_AttachCurrentThread above,
  // the removal of the guards is buried below in JavaThread::exit()
  // here. The abstraction should be more symmetrically either exposed
  // or hidden (e.g. it could probably be hidden in the same
  // (platform-dependent) methods where we do alternate stack
  // maintenance work?)
  thread->exit(false);
  thread->delete_thread_and_TLS_current();

  return JNI_OK;
}


jint JNICALL jni_GetEnv(JavaVM *vm, void **penv, jint version) {
  Thread* thread = ThreadLocalStorage::thread();
  if (thread != NULL && thread->is_Java_thread()) {
    if (version == JNI_VERSION_1_1 || version == JNI_VERSION_1_2) {
      *(JNIEnv**)penv = ((JavaThread*) thread)->jni_environment();
      return JNI_OK;
    } else if (version == JVMPI_VERSION_1 || version == JVMPI_VERSION_1_1) {
      *penv = (void* )jvmpi::GetInterface_1(version);	// version 1.X support
      return JNI_OK; 
    } else if (version == JVMDI_VERSION_1) {
      *penv = (void *)jvmdi::GetInterface_1(vm);
      return JNI_OK;
    } else {
      *penv = NULL;
      return JNI_EVERSION;
    }
  } else {
    *penv = NULL;
    return JNI_EDETACHED;
  }
}

} // End extern "C"

const struct JNIInvokeInterface_ jni_InvokeInterface = {
    NULL,
    NULL,
    NULL,

    jni_DestroyJavaVM,
    jni_AttachCurrentThread,
    jni_DetachCurrentThread,
    jni_GetEnv,
};

