/*
 * @(#)jvmdi.cpp	1.106 01/03/14 10:47:33
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_jvmdi.cpp.incl"

// Wrapper to trace JVMDI functions

#ifdef ASSERT
  
//
// JvmdiTraceWrapper
//

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* str) {
      ResourceMark rm;
      const char *thread_name = JvmdiInternal::safe_get_thread_name(JavaThread::current()->threadObj());
      tty->print_cr("JVMDI Call %30s [%s]", str, thread_name);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, JvmdiMonitor *monitor) {
      ResourceMark rm;
      tty->print("  %s = ", parm);
      monitor->print(*tty);
      tty->print_cr("");
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, jlong l) {
      ResourceMark rm;
      tty->print_cr("  %s = %d", parm, (int) l);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, const char *s) {
      ResourceMark rm;
      tty->print_cr("  %s = %s", parm, s);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, oop obj) {
    ResourceMark rm;
    const char *class_name = (obj != NULL) ? obj->blueprint()->external_name() : "NULL";
    tty->print_cr("   %s = %s",parm,class_name);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, JavaThread *thread) {
      ResourceMark rm;
      const char *thread_name = (thread == NULL) 
                              ? "<NULL>" 
                              : JvmdiInternal::safe_get_thread_name(thread->threadObj()); 

      tty->print_cr("  %s = %s", parm, thread_name);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, int event_type, bool enabled) {
    ResourceMark rm;
    tty->print("  %s = ", parm);
    JvmdiEvent::write_kind(*tty,event_type);
    tty->print_cr(" %s ", enabled ? "enabled" : "disabled" );
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char* parm, methodOop m, jint bci) {
    ResourceMark rm;
    tty->print("  %s = ", parm);    
    const char *klass_name  = (m == NULL) ? "NULL" : m->klass_name()->as_C_string();
    const char *method_name = (m == NULL) ? "NULL" : m->name()->as_C_string();
    tty->print_cr("[%s.%s:%d]", klass_name, method_name, bci);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char *parm, JvmdiEvent& event) {
      ResourceMark rm;
      const char *thread_name = JvmdiInternal::safe_get_thread_name(JavaThread::current()->threadObj());
      tty->print("%s = ", parm);
      event.write(*tty);
      tty->print_cr("[%s]", thread_name);
  }

  JvmdiTraceWrapper::JvmdiTraceWrapper(const char *parm, JvmdiBreakpoint& bp) {
      ResourceMark rm;
      tty->print("  %s = ",parm);
      bp.print();
      tty->print_cr("");
  }

#endif 

//
// class GrowableCache - private methods
//

void GrowableCache::recache() {
  int len = _elements->length();

  FREE_C_HEAP_ARRAY(address, _cache);
  _cache = NEW_C_HEAP_ARRAY(address,len+1);

  for (int i=0; i<len; i++) {
    _cache[i] = _elements->at(i)->getCacheValue();
    //
    // The cache entry has gone bad. Without a valid frame pointer
    // value, the entry is useless so we simply delete it in product
    // mode. The call to remove() will rebuild the cache again
    // without the bad entry.
    //
    if (_cache[i] == NULL) {
      assert(false, "cannot recache NULL elements");
      remove(i);
      return;
    }
  }
  _cache[len] = NULL;

  _listener_fun(_this_obj,_cache);
}

bool GrowableCache::equals(void* v, GrowableElement *e2) {
  GrowableElement *e1 = (GrowableElement *) v;
  assert(e1 != NULL, "e1 != NULL");
  assert(e2 != NULL, "e2 != NULL");

  return e1->equals(e2);
}

//
// class GrowableCache - public methods
//

GrowableCache::GrowableCache() {
  _this_obj       = NULL;
  _listener_fun   = NULL;    
  _elements       = NULL;
  _cache          = NULL;
}

GrowableCache::~GrowableCache() {
  clear();
  _elements->clear_and_deallocate();
  FreeHeap(_elements);
  FREE_C_HEAP_ARRAY(address, _cache);
}

void GrowableCache::initialize(void *this_obj, void listener_fun(void *, address*) ) {
  _this_obj       = this_obj;
  _listener_fun   = listener_fun;    
  _elements       = new (ResourceObj::C_HEAP) GrowableArray<GrowableElement*>(5,true);
  recache();
}

// number of elements in the collection
int GrowableCache::length() { 
  return _elements->length(); 
}

// get the value of the index element in the collection
GrowableElement* GrowableCache::at(int index) {
  GrowableElement *e = (GrowableElement *) _elements->at(index);
  assert(e != NULL, "e != NULL");
  return e;
}
 
int GrowableCache::find(GrowableElement* e) {
  return _elements->find(e, GrowableCache::equals);
}

// append a copy of the element to the end of the collection
void GrowableCache::append(GrowableElement* e) {
  GrowableElement *new_e = e->clone();
  _elements->append(new_e);
  recache();
}

// insert a copy of the element using lessthan()
void GrowableCache::insert(GrowableElement* e) {
  GrowableElement *new_e = e->clone();
  _elements->append(new_e);

  int n = length()-2;
  for (int i=n; i>=0; i--) {
    GrowableElement *e1 = _elements->at(i);
    GrowableElement *e2 = _elements->at(i+1);
    if (e2->lessThan(e1)) {
      _elements->at_put(i+1, e1);
      _elements->at_put(i,   e2);
    }
  }

  recache();
}

// remove the element at index
void GrowableCache::remove (int index) {
  GrowableElement *e = _elements->at(index);
  assert(e != NULL, "e != NULL");
  _elements->remove(e);
  delete e;
  recache();
}

// clear out all elements, release all heap space and
// let our listener know that things have changed.
void GrowableCache::clear() {
  int len = _elements->length();
  for (int i=0; i<len; i++) {
    delete _elements->at(i);
  }
  _elements->clear();
  recache();
}

void GrowableCache::oops_do(void f(oop *)) {
  int len = _elements->length();
  for (int i=0; i<len; i++) {
    GrowableElement *e = _elements->at(i);
    e->oops_do(f);
    _cache[i] = e->getCacheValue();
  }
}


//
// class JvmdiObj
//

void *JvmdiObj::operator new(size_t size)   { return JvmdiInternal::jvmdiMalloc(size); }
void  JvmdiObj::operator delete(void *p)    { JvmdiInternal::jvmdiFree((jbyte *) p); }


//
// class JvmdiMonitor
//

JvmdiMonitor::JvmdiMonitor(const char *name, const int magic) {
#ifdef ASSERT
  _name = strcpy(NEW_C_HEAP_ARRAY(char, strlen(name) + 1), name);
#else
  _name = NULL;
#endif
  _magic = magic;
}

JvmdiMonitor::~JvmdiMonitor() {
#ifdef ASSERT
  FreeHeap(_name);
#endif
  _magic = 0;
}


//
// class JvmdiBreakpoint
//

JvmdiBreakpoint::JvmdiBreakpoint() {
  _method = NULL;
  _bci    = 0;
}

JvmdiBreakpoint::JvmdiBreakpoint(jclass clazz, jmethodID method, jlocation location) {
  klassOop klass = JvmdiInternal::get_klassOop(clazz);
  assert(klass != NULL, "klass != NULL");
  _method        = JvmdiInternal::get_methodOop(method, klass);
  assert(_method != NULL, "_method != NULL");
  _bci           = (int) location;  

  assert(_method->method_holder() == klass, "method holder not the klass passed in" );
  assert(_bci >= 0, "_bci >= 0"); 
}

void JvmdiBreakpoint::copy(JvmdiBreakpoint& bp) {
    _method   = bp._method;
    _bci      = bp._bci;
}

bool JvmdiBreakpoint::lessThan(JvmdiBreakpoint& bp) {
 Unimplemented();
 return false;
}

bool JvmdiBreakpoint::equals(JvmdiBreakpoint& bp) {
    return _method   == bp._method
      &&   _bci      == bp._bci;
}

bool JvmdiBreakpoint::is_valid() {
  return _method != NULL &&
         _bci >= 0;
}

address JvmdiBreakpoint::getBcp() {
  return _method->bcp_from(_bci);
}

void JvmdiBreakpoint::set() {
  assert(UseFastBreakpoints, "unexpected fast breakpoint");
  _method->set_breakpoint(_bci);
}

void JvmdiBreakpoint::clear() {
  assert(UseFastBreakpoints, "unexpected fast breakpoint");
  _method->clear_breakpoint(_bci);
}

void JvmdiBreakpoint::print() {
#ifndef PRODUCT
  const char *class_name  = (_method == NULL) ? "NULL" : _method->klass_name()->as_C_string();
  const char *method_name = (_method == NULL) ? "NULL" : _method->name()->as_C_string();

  tty->print("Breakpoint(%s,%s,%d,%x)",class_name, method_name, _bci, getBcp());
#endif
}




//
// class  BreakpointStepping - helper class,
//  turns bytecode_stepping on  when number of breakpoints changes from zero to non-zero
//  turns bytecode_stepping off when number of breakpoints changes from non-zero to zero
// Only active if !UseFastBreakpoints
//

class BreakpointStepping {
  JvmdiBreakpointCache *_bps;
  int _length;

public:
  BreakpointStepping(JvmdiBreakpointCache *bps) {
    assert(bps != NULL, "BreakpointStepping called with empty bkp list");
    _bps    = bps;
    _length = _bps->length();
  }

  ~BreakpointStepping() {
    // transition from no breakpoints to some breakpoints
    if (!UseFastBreakpoints) {
      if ((_length == 0) && (_bps->length() > 0)) {
        jvmdi::set_bytecode_stepping(true);
      } else if ((_length > 0) && (_bps->length() == 0) &&
                 !JvmdiEventDispatcher::is_enabled(JVMDI_EVENT_SINGLE_STEP)) {
        // transition from some breakpoints to no breakpoints
        // unless we are also single stepping
        jvmdi::set_bytecode_stepping(false);
      }
    }
  }

};

//
// class VM_ChangeBreakpoints
//
// Modify the Breakpoints data structure at a safepoint
//

void VM_ChangeBreakpoints::doit() {
  switch (_operation) {
   case SET_BREAKPOINT:
     _breakpoints->set_at_safepoint(*_bp);
     break;
   case CLEAR_BREAKPOINT:
     _breakpoints->clear_at_safepoint(*_bp);
     break;
   case CLEAR_ALL_BREAKPOINT:
     _breakpoints->clearall_at_safepoint();
     break;
  default:
    assert(false, "Unknown operation");
  }
}

//
// class JvmdiBreakpoints 
//
// a JVMDI internal collection of JvmdiBreakpoint
//

JvmdiBreakpoints::JvmdiBreakpoints(void listener_fun(void *,address *)) {
  _bps.initialize(this,listener_fun);
}

JvmdiBreakpoints:: ~JvmdiBreakpoints() {}

  void  JvmdiBreakpoints::oops_do(void f(oop *)) {  
    _bps.oops_do(f);
  } 

  void  JvmdiBreakpoints::print() {
#ifndef PRODUCT
    ResourceMark rm;

    int n = _bps.length();
    for (int i=0; i<n; i++) {
      JvmdiBreakpoint& bp = _bps.at(i);
      tty->print("%d: ", i);
      bp.print();
      tty->print_cr("");
    }
#endif
  }


void JvmdiBreakpoints::set_at_safepoint(JvmdiBreakpoint& bp) {
  BreakpointStepping bs(&_bps);
  int i = _bps.find(bp);
  if (i == -1) { 
    _bps.append(bp);
    if (UseFastBreakpoints)  bp.set();
  }
}

void JvmdiBreakpoints::clear_at_safepoint(JvmdiBreakpoint& bp) {
  BreakpointStepping bs(&_bps);
  int i = _bps.find(bp);
  if (i != -1) {
    _bps.remove(i);
     if (UseFastBreakpoints)  bp.clear();
  }
}

void JvmdiBreakpoints::clearall_at_safepoint() {
  BreakpointStepping bs(&_bps);
  if (UseFastBreakpoints) {
    int len = _bps.length();
    for (int i=0; i<len; i++) {
      _bps.at(i).clear();
    }
  }
  _bps.clear();
}
 
int JvmdiBreakpoints::length() { return _bps.length(); }

int JvmdiBreakpoints::set(JvmdiBreakpoint& bp) {
  if ( _bps.find(bp) != -1) {
     return JVMDI_ERROR_DUPLICATE;
  }
  VM_ChangeBreakpoints set_breakpoint(this,VM_ChangeBreakpoints::SET_BREAKPOINT, &bp);
  VMThread::execute(&set_breakpoint);
  return JVMDI_ERROR_NONE;
}

int JvmdiBreakpoints::clear(JvmdiBreakpoint& bp) {
  if ( _bps.find(bp) == -1) {
     return JVMDI_ERROR_NOT_FOUND;
  }

  VM_ChangeBreakpoints clear_breakpoint(this,VM_ChangeBreakpoints::CLEAR_BREAKPOINT, &bp);
  VMThread::execute(&clear_breakpoint);
  return JVMDI_ERROR_NONE;
}

void JvmdiBreakpoints::clearall() {
  VM_ChangeBreakpoints clearall_breakpoint(this,VM_ChangeBreakpoints::CLEAR_ALL_BREAKPOINT);
  VMThread::execute(&clearall_breakpoint);
}

//
// class JvmdiCurrentBreakpoints
// 

JvmdiBreakpoints *JvmdiCurrentBreakpoints::_jvmdi_breakpoints  = NULL;
address *         JvmdiCurrentBreakpoints::_breakpoint_list    = NULL;

void JvmdiCurrentBreakpoints::initialize() {}

void JvmdiCurrentBreakpoints::destroy() {
  delete _jvmdi_breakpoints;
  _jvmdi_breakpoints = NULL;
}


JvmdiBreakpoints& JvmdiCurrentBreakpoints::get_jvmdi_breakpoints() {
    if (_jvmdi_breakpoints != NULL) return (*_jvmdi_breakpoints);
    _jvmdi_breakpoints = new JvmdiBreakpoints(listener_fun);
    assert(_jvmdi_breakpoints != NULL, "_jvmdi_breakpoints != NULL");
    return (*_jvmdi_breakpoints);
}

void  JvmdiCurrentBreakpoints::listener_fun(void *this_obj, address *cache) { 
   JvmdiBreakpoints *this_jvmdi = (JvmdiBreakpoints *) this_obj;
   assert(this_jvmdi != NULL, "this_jvmdi != NULL");

   debug_only(int n = this_jvmdi->length(););
   assert(cache[n] == NULL, "cache must be NULL terminated");

   set_breakpoint_list(cache);
}


void JvmdiCurrentBreakpoints::oops_do(void f(oop *)) { 
  if (_jvmdi_breakpoints != NULL) {
    _jvmdi_breakpoints->oops_do(f);
  }
}



//
// class JvmdiEventEnabled
//

JvmdiEventEnabled::JvmdiEventEnabled(bool init_val) {
  for (int i=0; i<(JVMDI_MAX_EVENT_TYPE_VAL+1); i++) {
    _event_enabled[i] = init_val;
  }
}

bool JvmdiEventEnabled::is_enabled(jint event_type) {
  assert(jvmdi::is_valid_event_type(event_type), "invalid event type");
  return _event_enabled[event_type];
}

void JvmdiEventEnabled::set_enabled(jint event_type, bool enabled) {  
  assert(jvmdi::is_valid_event_type(event_type), "invalid event type");
  _event_enabled[event_type] = enabled;
}

//
// class JvmdiEventDispatcher
//

// create with all global events enabled, turn some off in initialize()
JvmdiEventEnabled JvmdiEventDispatcher::_global_event_enabled(true);
debug_only(static bool jvmdi_event_dispatcher_initialized = false;)

void JvmdiEventDispatcher::initialize() {
  assert(!jvmdi_event_dispatcher_initialized, "Should only initialize JvmdiEventDispatcher once");
  // set some global events off by default (from JVMDI spec)
  _global_event_enabled.set_enabled(JVMDI_EVENT_SINGLE_STEP,false);
  _global_event_enabled.set_enabled(JVMDI_EVENT_EXCEPTION_CATCH,false);
  _global_event_enabled.set_enabled(JVMDI_EVENT_METHOD_ENTRY,false);
  _global_event_enabled.set_enabled(JVMDI_EVENT_METHOD_EXIT,false);
  debug_only(jvmdi_event_dispatcher_initialized = true;)
}

void JvmdiEventDispatcher::destroy() {
}

void JvmdiEventDispatcher::reset_thread_location(JavaThread *thread, jint event, bool enabled) {
  assert(event == JVMDI_EVENT_SINGLE_STEP || event == JVMDI_EVENT_BREAKPOINT, "must be single-step or breakpoint event");

  // Thread location is used to detect the following:
  // 1) a breakpoint event followed by single-stepping to the same bci
  // 2) single-step to a bytecode that will be transformed to a fast version
  // We skip to avoid posting the duplicate single-stepping event.

  // If single-stepping is disabled, reset thread location so that 
  // single-stepping to the same method and bcp at a later time will be
  // detected if single-stepping is enabled at that time (see 4388912).

  // If single-stepping is enabled, set the thread location to the
  // current method and bcp. This covers the following type of case,
  // e.g., the debugger stepi command:
  // - bytecode single stepped
  // - SINGLE_STEP event posted and SINGLE_STEP event disabled
  // - SINGLE_STEP event reenabled
  // - bytecode rewritten to fast version

  // If breakpoint event is disabled, reset thread location only if
  // single-stepping is not enabled.  Otherwise, keep the thread location
  // to detect any duplicate events.

  // If enabling breakpoint, no need to reset.
  if (event == JVMDI_EVENT_BREAKPOINT && enabled) {
    return;
  }

  // Reset thread-specific location info
  if (thread != NULL) {
    JvmdiThreadState *state = thread->jvmdi_thread_state();
    JNIid* method_id = NULL;
    int bci = 0;
    if (enabled && thread->has_last_Java_frame()) {
      assert(event == JVMDI_EVENT_SINGLE_STEP, "enable BREAKPOINT should be done already");

      frame f = thread->last_frame();
      method_id = f.interpreter_frame_method()->jni_id();
      bci = f.interpreter_frame_bci();
    }
    state->set_current_location(method_id, bci);
  } else {
    // reset location info for all threads when stepping or breakpoint is disabled globally.
    MutexLocker tl(Threads_lock);
    for(JavaThread *jthread = Threads::first(); jthread != NULL; jthread = jthread->next()) {
      JvmdiThreadState *state = jthread->jvmdi_thread_state();

      if (enabled && jthread->has_last_Java_frame()) {
        assert(event == JVMDI_EVENT_SINGLE_STEP, "enable BREAKPOINT should be done already");
        frame f = jthread->last_frame();
        JNIid* method_id = f.interpreter_frame_method()->jni_id();
        int bci = f.interpreter_frame_bci();
        state->set_current_location(method_id, bci);
      } else if (!enabled && !state->is_enabled(event)) {
        // The event is disabled globally but we need to check the event is also not enabled
        // in thread level.  If this is to disable breakpoint, also check if single-step
        // is not enabled in the thread level.
        if (event == JVMDI_EVENT_SINGLE_STEP || !JvmdiEventDispatcher::is_enabled(jthread, JVMDI_EVENT_SINGLE_STEP)) {
          state->set_current_location(NULL, 0);
        }
      }
 
    }
  }
}

void JvmdiEventDispatcher::set_enabled(JavaThread *thread, jint event_type, bool enabled) {
  if (thread == NULL) {
    _global_event_enabled.set_enabled(event_type, enabled);   
  } else if (!(is_global_event(event_type))) { // global events cannot be toggled on threads
    thread->jvmdi_thread_state()->set_enabled(event_type, enabled);   
  }

  // Both single stepping and breakpoints use bytecode stepping
  if (event_type == JVMDI_EVENT_SINGLE_STEP ||
      event_type == JVMDI_EVENT_BREAKPOINT) {
    bool should_step = is_enabled(JVMDI_EVENT_SINGLE_STEP) || (!UseFastBreakpoints && is_enabled(JVMDI_EVENT_BREAKPOINT));
    jvmdi::set_bytecode_stepping(should_step);
    // reset thread-specific location info when single-stepping or breakpoint event is enabled/disabled.
    reset_thread_location(thread, event_type, enabled);
  }
}

bool JvmdiEventDispatcher::is_enabled(jint event_type) {
  if (_global_event_enabled.is_enabled(event_type)) {
    return true;
  }

  if (is_global_event(event_type)) {
    return false; // no need to check on individual threads
  }

  { 
    bool need_threads_lock = !Thread::current()->is_VM_thread();
    assert(need_threads_lock || Threads_lock->owner()->is_VM_thread(), "VM thread must own Threads_lock");
    MutexLockerEx mu(need_threads_lock ? Threads_lock : NULL);
    JavaThread *java_thread = Threads::first();
    while (java_thread != NULL) {
      JvmdiThreadState *thread_state = java_thread->jvmdi_thread_state();
      if ((thread_state != NULL) && (thread_state->is_enabled(event_type))) {
        return true;
      }    
      java_thread = java_thread->next();
    }
  }

  return false;
}

bool JvmdiEventDispatcher::is_enabled(JavaThread *thread, jint event_type) {
  if (_global_event_enabled.is_enabled(event_type)) {
    return true;
  }
 
  if (is_global_event(event_type)) {
    return false;  // no need to check on the thread 
  }
  return thread->jvmdi_thread_state()->is_enabled(event_type);
}

bool JvmdiEventDispatcher::is_globally_enabled(jint event_type) {
  return (_global_event_enabled.is_enabled(event_type));
}

//
// class JvmdiInternal
//

bool              JvmdiInternal::_jvmdi_initialized = false;
JVMDI_Interface_1 JvmdiInternal::_jvmdi_interface;
JVMDI_EventHook   JvmdiInternal::_event_hook  = (JVMDI_EventHook)JvmdiInternal::default_event_hook;
JavaVM*           JvmdiInternal::_vm          = NULL;
JVMDI_AllocHook   JvmdiInternal::_ahook       = NULL;
JVMDI_DeallocHook JvmdiInternal::_dhook       = NULL;


//
// Useful for debugging jvmdi.cpp:
// default_event_hook is only called when SetEventHook is not used.
//
void JvmdiInternal::default_event_hook(JNIEnv *env, JVMDI_Event *event) {
  assert(jvmdi::enabled(), "jvmdi::enabled()");  
  Unimplemented();
}

void JvmdiInternal::jvmdi_init() {
  assert(jvmdi::enabled(), "jvmdi::enabled()" );
}

void JvmdiInternal::jvmdi_init_done() {
  _jvmdi_initialized = true;
}

void JvmdiInternal::jvmdi_death() {
  assert(jvmdi::enabled(), "jvmdi::enabled()");
  jvmdi::destroy();
}

void  JvmdiInternal::set_allocation_hooks(JVMDI_AllocHook ahook, JVMDI_DeallocHook dhook) {
  _ahook = ahook;
  _dhook = dhook;
}


//
// Utility functions that manage memory by calling into JVMDI.
// You should Never use os::malloc/free directly since the allocation
// hooks may have been redirected.
//

jbyte *JvmdiInternal::jvmdiMalloc(jlong size) {
  jbyte *mem;
  jvmdiError result = AllocateInternal(size,&mem);
  assert(result == JVMDI_ERROR_NONE, "Allocate failed");
  return mem;
}

void JvmdiInternal::jvmdiFree(jbyte *mem) {
  DeallocateInternal(mem);
}

///////////////////////////////////////////////////////////////


//
// Helper classes JvmdiFrameIterator & JvmdiJavaFrameIterator
//

class JvmdiFrameIterator : public ResourceObj {
private:
  JavaThread *_java_thread;
  frame       _frame;
  frame       _next;
  bool        _frame_valid; 
  bool        _next_valid;  

  void set_next(frame& f) {
    RegisterMap rm(_java_thread,true); // Todo: what does 'true' do?
    if (!f.is_first_frame()) {
      _next  = f.sender(&rm);
    } else {
      _next_valid = false;
    }
  }

public:
  JvmdiFrameIterator(JavaThread *java_thread, frame f) {
    assert(java_thread != NULL, "java_thread != NULL");

    _java_thread = java_thread;
    _frame_valid = true;
    _next_valid  = true;

    _frame  = f;
    set_next(_frame);
  }

  bool hasNext() { 
    return _frame_valid;
  }

  frame next() {
    assert(hasNext(), "Invalid next() call");
    frame ret = _frame;

    if (!_next_valid) {
      _frame_valid = false;
    } else {
      _frame   = _next;
      set_next(_frame);
    }
    return ret;
  }
};

class JvmdiJavaFrameIterator : public ResourceObj {
private:
  JvmdiFrameIterator _iter;
  frame              _java_frame;
  bool               _java_frame_valid;

  bool set_java_frame() {
    while (_iter.hasNext()) {
      _java_frame = _iter.next();
      if (_java_frame.is_java_frame()) {
	return true;
      }
    }
    return false;
  }

public:
  JvmdiJavaFrameIterator(JavaThread *thread, frame f) 
  : _iter(thread,f) {    
    _java_frame_valid = set_java_frame();
  }

  bool hasNext() {
    return _java_frame_valid;
  }

  frame next() {
    assert(hasNext(), "Invalid next() call");
    frame ret = _java_frame;
    _java_frame_valid = set_java_frame();
    return ret;
  }
};


//
// class JvmdiWalkableThread - private methods
//

static int JVMDI_FR_MAGIC = (int)(('D' << 24) | ('I' << 16) | ('F' << 8) | 'R');

bool JvmdiWalkableThread::verify_jvmdi_frame(JvmdiFrame *jvmdi_frame) {
  if (jvmdi_frame == NULL || jvmdi_frame->magic() != JVMDI_FR_MAGIC) {
    // quick sanity check of the JvmdiFrame
    return false;
  }
  JvmdiFrameIterator iter(get_thread(), get_thread()->last_frame());
  while (iter.hasNext()) {
    frame f = iter.next();
    if (jvmdi_frame->equals(&f)) {
      return true;
    }
  }
  if (TraceJVMDICalls) tty->print("Inconsistent JvmdiFrame: %x", jvmdi_frame);
  return false;
}

bool JvmdiWalkableThread::verify_jvmdi_frames() {
  int len = _frames->length();
  for (int i=0; i<len; i++) {
    if (!verify_jvmdi_frame(_frames->at(i))) {
      return false;
    }
  }
  return true;
}

bool JvmdiWalkableThread::equalFrame(void* v, JvmdiFrame *jvmdi_frame2) {
  frame *f1 = (frame *) v;
  frame *f2 = jvmdi_frame2->get_frame_ptr();

  return (f1->fp() == f2->fp()) && (f1->pc() == f2->pc());  
}

jframeID JvmdiWalkableThread::lookup(frame* f) {
  int index = _frames->find(f, JvmdiWalkableThread::equalFrame);
 
  if (index != -1) {
    JvmdiFrame *jvmdi_frame = _frames->at(index);
    return to_jframeID(jvmdi_frame);
  }

  JvmdiFrame *jvmdi_frame = new JvmdiFrame(get_thread(), f, JVMDI_FR_MAGIC);
  _frames->append(jvmdi_frame);
  return to_jframeID(jvmdi_frame);
}

// Find a Java frame starting at the specified frame. Returned frame
// could be a copy of the specified frame.
bool JvmdiWalkableThread::find_java_frame(frame f, frame *found) {
  assert(found != NULL, "found != NULL");
  assert(has_last_frame(), "must have a Java frame");

  RegisterMap rm(get_thread());
  while (!f.is_java_frame()) {
    if (f.is_first_frame()) return false;  // reached the top with no luck

    f = f.sender(&rm);  // switch to caller
  }

  *found = f;  // copy out the frame we found
  return true;
}


//
// class JvmdiWalkableThread - public methods
//

JvmdiWalkableThread::JvmdiWalkableThread(JavaThread *java_thread, JvmdiWalkableThread *next, bool is_suspend) {
  _next           = next;
  _thread         = java_thread;
  _active         = java_thread == JavaThread::current();
  _frames         = new (ResourceObj::C_HEAP) GrowableArray<JvmdiFrame *>(5,true);
  _suspend        = is_suspend;
  current_frame(); // force current frame to be loaded into _frames
}

JvmdiWalkableThread::~JvmdiWalkableThread() {
  int len = _frames->length();
  for (int i=0; i< len; i++) {
    delete _frames->at(i);
  }
  _frames->clear_and_deallocate();
  FreeHeap(_frames);
  _frames = NULL;
}

bool JvmdiWalkableThread::equalFp(void* fp, JvmdiFrame *jvmdi_frame) {
  jint * f1 = (jint *)fp;
  jint * f2 = jvmdi_frame->get_frame().fp();

  return (f1 == f2);
}

JvmdiFrame *JvmdiWalkableThread::from_jframeID(jframeID fid) {
  assert(fid != NULL, "jframeID must not be NULL");
  int index = _frames->find(fid, JvmdiWalkableThread::equalFp);
  if (index == -1) return NULL;

  JvmdiFrame *jvmdi_frame = _frames->at(index);
  verify_jvmdi_frame(jvmdi_frame);
  return jvmdi_frame;
}

jframeID JvmdiWalkableThread::to_jframeID(JvmdiFrame *jvmdi_frame) {
  verify_jvmdi_frame(jvmdi_frame);
  // use the frame's FP as a unique value
  return (jframeID)jvmdi_frame->get_frame().fp();
}

jframeID JvmdiWalkableThread::invalid_jframeID() {
  return (jframeID)NULL;
}

jframeID JvmdiWalkableThread::unknown_jframeID() {
  return (jframeID)-1;
}


int JvmdiWalkableThread::nFrames() {
  if (!has_last_frame()) return 0;  // no Java frames

  int n = 0;
  JvmdiJavaFrameIterator iter(get_thread(), get_thread()->last_frame());
  while (iter.hasNext()) {
    frame f = iter.next();
    assert(f.is_java_frame(), "Sanity check");
    n++;
  }
  assert(verify_jvmdi_frames(), "Inconsistent JvmdiFrames");
  return n;
}

jframeID JvmdiWalkableThread::current_frame() {
  // The thread being queried does not have any Java frames
  // which is not a system failure so no assertion here.
  if (!has_last_frame()) return unknown_jframeID();

  frame f = get_thread()->last_frame();

  frame found;
  if (find_java_frame(f, &found)) {
    return lookup(&found);
  }

  // could not find the Java frame
  // XXX - this should probably be an assert()
  return unknown_jframeID();
}

jframeID JvmdiWalkableThread::get_caller(jframeID fid) {
  // The thread being queried does not have any Java frames
  // which is not a system failure so no assertion here.
  if (!has_last_frame()) return unknown_jframeID();

  JvmdiFrame *jvmdi_frame = from_jframeID(fid);
  if (jvmdi_frame == NULL) {
    assert(false, "JvmdiFrame must not be NULL");
    return invalid_jframeID();
  }

  frame *f = jvmdi_frame->get_frame_ptr();
  assert(f != NULL, "f != NULL");

  if (f->is_first_frame()) return fid;  // no frames above us

  // c is the frame above fid
  RegisterMap rm(get_thread());
  frame c = f->sender(&rm);

  frame found;
  if (find_java_frame(c, &found)) {
    return lookup(&found);
  }

  // no Java frames above fid
  return fid;  
}

bool JvmdiWalkableThread::is_opaque(jframeID fid) {
  JvmdiFrame *jvmdi_frame = from_jframeID(fid);
  if (jvmdi_frame == NULL) {
    // Currently we are only called by JvmdiWalkableThreads::is_opaque().
    // The caller used from_jframeID() to find our JvmdiWalkableThread
    // object so the fact that from_jframeID() returns NULL now is bad.
    assert(false, "JvmdiFrame must not be NULL");
    return true;  // be pessimistic and don't crash in product mode
  }
  frame *f = jvmdi_frame->get_frame_ptr();
  assert(f != NULL, "f != NULL");
  
  return !(f->is_interpreted_frame());
}

bool JvmdiWalkableThread::get_frame_location(jframeID fid, methodHandle *method_handle, jint *bci) {
  // The frame being queried does not have any Java frames
  // which is not a system failure so no assertion here.
  if (!has_last_frame()) return false;

  JvmdiFrame *jvmdi_frame = from_jframeID(fid);
  if (jvmdi_frame == NULL) {
    assert(false, "JvmdiFrame must not be NULL");
    return false;  // be pessimistic in product mode
  }
  frame *f = jvmdi_frame->get_frame_ptr();
  assert(f != NULL, "f != NULL");

  // for now call interpreter_frame_method, eventually want platform independent
  // way to identify method & bci for ANY frame (not just interpreted ones).
  assert (f->is_interpreted_frame(), "f must be interpreted");
  *method_handle = f->interpreter_frame_method();
  *bci           = f->interpreter_frame_bci();
  return true;
}


/////////////////////////////////////////////////////////////////////////////////////////

//
// class JvmdiWalkableThreads - see comments in jvmdi_impl.hpp
//

JvmdiWalkableThread  *JvmdiWalkableThreads::_first = NULL;

//
// private methods (class JvmdiWalkableThreads)
//


JvmdiWalkableThread *JvmdiWalkableThreads::get_thread(JavaThread *java_thread) {
  for (JvmdiWalkableThread *st = first(); st != NULL; st = st->next()) {
    if (st->equals(java_thread)) return st;
  }
  return NULL;
}

JvmdiWalkableThread *JvmdiWalkableThreads::get_thread(jframeID fid) {  
  assert(fid != NULL, "jframeID must not be NULL");
  for (JvmdiWalkableThread *st = first(); st != NULL; st = st->next()) {
    if (st->from_jframeID(fid) != NULL) return st;
  }
  return NULL;
}

bool JvmdiWalkableThreads::add_thread(JavaThread *java_thread, bool is_suspend) {
  if (get_thread(java_thread) != NULL) {	// sanity check
    assert(false, "entry already present");
    return false;
  }

  set_first(new JvmdiWalkableThread(java_thread, first(), is_suspend));
  return true;
}

bool JvmdiWalkableThreads::remove_thread(JavaThread *java_thread) {
  JvmdiWalkableThread *st      = first();
  JvmdiWalkableThread *prev_st = NULL;

  while (st != NULL) {
    if (st->equals(java_thread)) {

      if (prev_st != NULL) {
	prev_st->set_next(st->next());
      } else {
	set_first(st->next());
      }
      
      delete st;
      return true;
    }
    prev_st = st;
    st = st->next();
  }
  assert(false, "did not find entry");
  return false;
}


//
// public (synchronized) methods (class JvmdiWalkableThreads)
//

void JvmdiWalkableThreads::add_current() {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *wt = get_thread(JavaThread::current());
  if (wt == NULL) {
    add_thread(JavaThread::current(), false);
  } else {
    if (wt->is_suspend()) return;     // added by suspend is okay
    assert(false, "current thread already walkable (not by suspend)");
  }
}

void JvmdiWalkableThreads::remove_current() {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *wt = get_thread(JavaThread::current());
  assert(wt != NULL, "wt must be non-NULL in remove_current");
  if (wt->is_suspend()) return;     // added by suspend, nothing left to do

  assert(!JavaThread::current()->is_being_ext_suspended(),
    "current thread shouldn't be suspended");
  assert(wt->is_active(), "current thread should be active");

  remove_thread(JavaThread::current());
}

bool JvmdiWalkableThreads::suspend(JavaThread *java_thread) {  
    		// hSuspendThread should have caught suspending a thread twice

  if (java_lang_Thread::thread(java_thread->threadObj()) == NULL) {
    // the thread has not yet run or is in process of exiting
    return false;
  }

  // Immediate suspension required for jdb so JVM/DI agent threads do
  // not deadlock due to later suspension on transitions while holding
  // raw monitors.
  java_thread->java_suspend(true);

  // It would be nice to have the following assertion in all the time,
  // but it is possible for a racing resume request to have resumed
  // this thread right after we suspended it. Temporarily enable this
  // assertion if you are chasing a different kind of bug.
  //
  // assert(java_lang_Thread::thread(java_thread->threadObj()) == NULL ||
  //   java_thread->is_being_ext_suspended(), "thread is not suspended");

  if (java_lang_Thread::thread(java_thread->threadObj()) == NULL) {
    // check again because we can get delayed in java_suspend():
    // the thread is in process of exiting.
    return false;
  }

  // add thread to list of JvmdiWalkableThreads
  MutexLocker mu(JVMDI_frames_lock);  // grab JVMDI_frames_lock
    
  JvmdiWalkableThread *st = get_thread(java_thread);
  if (st == NULL) {
    add_thread(java_thread, true);
  } else {
    assert(st->is_active(), "existing entry must be for active thread");
    st->set_suspend(true);	// mark entry so it doesn't get released
  }

  // unlock JVMDI_frames_lock    
  return true;
}

bool JvmdiWalkableThreads::resume(JavaThread *java_thread) {  
		// hResumeThread should have caught resuming a thread twice
  assert(java_thread->is_being_ext_suspended(), "thread should be suspended");

  { MutexLocker mu(JVMDI_frames_lock);  // grab JVMDI_frames_lock

    JvmdiWalkableThread *st = get_thread(java_thread); 
    assert(st != NULL, "no entry for suspended thread");
    if (st != NULL) {
      if (st->is_active()) {	// just mark active thread as not suspended
	st->set_suspend(false);
      } else {
	remove_thread(java_thread);
      }
    }
  } // unlock JVMDI_frames_lock

  // resume thread
  {
    // must always grab Threads_lock, see JVM_SuspendThread
    MutexLocker ml(Threads_lock);  
    java_thread->java_resume(); 
  }
 
  return true;
}

bool JvmdiWalkableThreads::suspended(JavaThread *java_thread) {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *st = get_thread(java_thread);
  return (st != NULL && java_thread->is_being_ext_suspended());
}

jframeID JvmdiWalkableThreads::current_frame(JavaThread *java_thread) {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *st = get_thread(java_thread);
  if (st == NULL) {
    return JvmdiWalkableThread::invalid_jframeID();
  }
  
  return st->current_frame();
}


// load fid into the current walkable thread's frame cache
bool JvmdiWalkableThreads::load_frame(jframeID fid) {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *st = get_thread(JavaThread::current());
  if (st == NULL) {
    // The current thread must be walkable for us to load this frame
    return false;
  }

  // Effectively perform the series of GetCallerFrame() calls that
  // a JVM/DI client would perform on a suspended thread.

  jframeID current = st->current_frame();
  // easy case - the current frame was loaded by the JvmdiWalkableThread constructor
  while (current != fid) {
    jframeID parent = st->get_caller(current);
    if (parent == current || parent == JvmdiWalkableThread::invalid_jframeID()) {
      // we reached the first frame without finding our target or fid is invalid
      return false;
    }
    current = parent;
  }
  return true;  // we loaded the target frame
}


int JvmdiWalkableThreads::nFrames(JavaThread *java_thread) {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *st = get_thread(java_thread);
  if (st == NULL) {
    return -1;
  }

  return st->nFrames();
}

bool JvmdiWalkableThreads::get_caller(jframeID fid, jframeID *parentID) {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *st = get_thread(fid);
  if (st == NULL) return false;  // can't find walkable thread

  // parent of first frame is itself
  *parentID = st->get_caller(fid);
  return (*parentID != JvmdiWalkableThread::invalid_jframeID());
}

bool JvmdiWalkableThreads::is_opaque(jframeID fid) {
  MutexLocker mu(JVMDI_frames_lock);

  JvmdiWalkableThread *st = get_thread(fid);
  if (st == NULL) {
    // fid is not related to a walkable thread so consider it opaque.
    // fid could refer to a non-Java frame or to an invalid frame.
    return true;
  }
  return st->is_opaque(fid);
}


bool JvmdiWalkableThreads::get_frame_location(jframeID frame, jclass *klass, jmethodID *method, jlocation *location) {
  jint         bci;
  HandleMark   hm;
  methodHandle hmethod;

  {
    MutexLocker mu(JVMDI_frames_lock);

    JvmdiWalkableThread *st = get_thread(frame);
    if (st == NULL) return false;  // can't find walkable thread


    if (!st->get_frame_location(frame, &hmethod, &bci)) {
      return false;
    }
  }
  JVMDI_TRACE_ARG3("returns location", hmethod(), bci);

  *location = bci;
  
  if (bci >= 0) {
    *klass  = JvmdiInternal::get_jni_class_non_null(hmethod->method_holder());
    *method = JvmdiInternal::get_jni_method_non_null(hmethod());
  } else {
    *klass  = NULL;
    *method = NULL;
  }

  return true;
}


JvmdiFrame *JvmdiWalkableThreads::get_frame(jframeID fid) {
  JvmdiWalkableThread *st = get_thread(fid);
  if (st == NULL) return NULL;  // can't find walkable thread

  JvmdiFrame *jvmdi_frame = st->from_jframeID(fid);
  assert(jvmdi_frame != NULL, "jvmdi_frame != NULL");
  return jvmdi_frame;
}


bool JvmdiWalkableThreads::notify_frame_pop(jframeID fid) {
  if (TraceJVMDICalls) {
    ResourceMark rm;
    print();
  }

  JvmdiThreadState *thread_state = NULL;
  JvmdiFrame *      jvmdi_frame  = NULL;
  frame *                     f  = NULL;
  JavaThread *       java_thread = NULL;

  { MutexLocker mu(JVMDI_frames_lock);

    jvmdi_frame = get_frame(fid);
    if (jvmdi_frame == NULL) { return false; }

    f = jvmdi_frame->get_frame_ptr();
    assert(f != NULL, "f != NULL");

    java_thread = jvmdi_frame->get_thread();
    assert(java_thread != NULL, "java_thread != NULL");

    thread_state = java_thread->jvmdi_thread_state();
    assert(thread_state != NULL, "thread_state != NULL");
  }

  // We enable the method_exit_on flag so that post_method_exit()
  // is invoked to clean up this frame pop record. Since this is
  // housekeeping, we do it independently of the setting for
  // JVMDI_EVENT_FRAME_POP.
  jvmdi::set_method_exit_on(true);

  { // set the frame pop in our threadstate
    JvmdiFramePop fp(f);
    JvmdiFramePops& fps = thread_state->get_jvmdi_frame_pops();
    fps.set(fp);
  }

#ifndef PRODUCT
  if (TraceJVMDICalls) {
    tty->print_cr("JvmdiWalkableThreads::notify_frame_pop:");
    print_frame_pops(fid);
  }
#endif

  return true;
}

bool JvmdiWalkableThreads::get_local(jframeID fid, jint n, Handle *h) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }

  oop obj;
  if (!jvmdi_frame->get_local(n, &obj)) { return false; }

  *h = obj;
  return true;
}

bool JvmdiWalkableThreads::get_local(jframeID fid, jint n, jint *i) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->get_local(n, i);
}

bool JvmdiWalkableThreads::get_local(jframeID fid, jint n, jlong *l) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->get_local(n, l);
}


bool JvmdiWalkableThreads::get_local(jframeID fid, jint n, jfloat *f) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->get_local(n, f);
}

bool JvmdiWalkableThreads::get_local(jframeID fid, jint n, jdouble *d) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->get_local(n, d);
}

bool JvmdiWalkableThreads::set_local(jframeID fid, jint n, Handle object_handle) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->set_local(n, object_handle());
}

bool JvmdiWalkableThreads::set_local(jframeID fid, jint n, jint i) {
  MutexLocker mu(JVMDI_frames_lock);
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->set_local(n, i);
}

bool JvmdiWalkableThreads::set_local(jframeID fid, jint n, jlong l) {
  MutexLocker mu(JVMDI_frames_lock); 
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->set_local(n, l);
}

bool JvmdiWalkableThreads::set_local(jframeID fid, jint n, jfloat f) {
  MutexLocker mu(JVMDI_frames_lock); 
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->set_local(n, f);
}

bool JvmdiWalkableThreads::set_local(jframeID fid, jint n, jdouble d) {
  MutexLocker mu(JVMDI_frames_lock); 
  JvmdiFrame * jvmdi_frame = get_frame(fid);
  if (jvmdi_frame == NULL) { return false; }
  return jvmdi_frame->set_local(n, d);
}


void JvmdiWalkableThreads::print_frame_pops(jframeID fid) {
#ifndef PRODUCT
  MutexLocker mu(JVMDI_frames_lock);
  ResourceMark rm;

  JvmdiWalkableThread *st = get_thread(fid);
  if (st != NULL) {
    JvmdiFramePops& fps = st->get_thread()->jvmdi_thread_state()->get_jvmdi_frame_pops();
    fps.print();
  }
#endif
}

// Todo: This could replace JavaThread::get_thread_name() if the extra test is acceptable
const char *JvmdiInternal::safe_get_thread_name(oop threadObj) {
    if (threadObj == NULL) {
      return "NULL";
    }
    typeArrayOop name = java_lang_Thread::name(threadObj);
    if (name == NULL) {
      return "<NOT FILLED IN>";
    }
    return UNICODE::as_utf8((jchar*) name->base(T_CHAR), name->length());
}

void JvmdiWalkableThreads::print() {
#ifndef PRODUCT
  MutexLocker mu(JVMDI_frames_lock);
  ResourceMark rm;

  tty->print("Suspended Threads: [");
  JvmdiWalkableThread *st      = first();
  while (st != NULL) {

    const char *name   = JvmdiInternal::safe_get_thread_name(st->get_thread()->threadObj());
    int nFrames    = st->nFrames();

    tty->print("%s(%c%c ",name, st->get_thread()->is_being_ext_suspended() ? 'S' : '_', st->is_active() ? 'A' : '_');
    if (!st->has_last_frame()) {
      tty->print("no stack");
    }
    tty->print(") ");
    st = st->next();
  }
  tty->print_cr("]");
  
#endif  
}


//
// class JvmdiWalkableMark 
//

class JvmdiWalkableMark : public ResourceObj {
private:
  StateSaver _state_saver;
  JavaThread *_thread;

public:
  JvmdiWalkableMark(JavaThread *thread)
  : _state_saver(thread)
  , _thread(thread) {
    assert(thread == JavaThread::current(), "thread must be current!");

    // WalkableMark's can nest inside each other. However, the constructor and destructor code
    // should only be executed for the outermost mark. To insure this, each time a mark is 
    // constructed/destroyed, the markdepth for the thread increases/decreases.    
    int markdepth = thread->jvmdi_thread_state()->get_walkable_markdepth();        
    assert(markdepth >= 0, "markdepth should be non-negative");
    thread->jvmdi_thread_state()->set_walkable_markdepth(markdepth+1);

    // for inner marks, skip construction code
    if (markdepth > 0) { return; }

#ifndef PRODUCT
    if (0 && TraceJVMDICalls) {
      ResourceMark rm;
      const char *name = JvmdiInternal::safe_get_thread_name(JavaThread::current()->threadObj());
      tty->print_cr("JvmdiWalkableMark: Before add_current() [%s]",name);
      JvmdiWalkableThreads::print();
    }
#endif
    JvmdiWalkableThreads::add_current(); 
#ifndef PRODUCT
    if (0 && TraceJVMDICalls) {
      ResourceMark rm;
      const char *name = JvmdiInternal::safe_get_thread_name(JavaThread::current()->threadObj());
      tty->print_cr("JvmdiWalkableMark: After add_current() [%s]",name);
      JvmdiWalkableThreads::print();
    }
#endif
  }

  ~JvmdiWalkableMark()     {  

    // see comment in constructor
    int markdepth = _thread->jvmdi_thread_state()->get_walkable_markdepth() - 1;        
    assert(markdepth >= 0, "markdepth should be non-negative");
    _thread->jvmdi_thread_state()->set_walkable_markdepth(markdepth);

    // for inner marks, skip destruction code
    if (markdepth > 0) { return; }

#ifndef PRODUCT
    if (0 && TraceJVMDICalls) {
      ResourceMark rm;
      const char *name = JvmdiInternal::safe_get_thread_name(JavaThread::current()->threadObj());
      tty->print_cr("JvmdiWalkableMark: Before remove_current() [%s]",name);
      JvmdiWalkableThreads::print();
    }
#endif
    JvmdiWalkableThreads::remove_current(); 
#ifndef PRODUCT
    if (0 && TraceJVMDICalls) {
      ResourceMark rm;
      const char *name = JvmdiInternal::safe_get_thread_name(JavaThread::current()->threadObj());
      tty->print_cr("JvmdiWalkableMark: After remove_current() [%s]",name);
      JvmdiWalkableThreads::print();
    }
#endif
  }

  jframeID current_frame()       {
    return JvmdiWalkableThreads::current_frame(_thread); 
  }
};


//
// class JvmdiFramePop 
//

// Todo: Make Platform independent version
bool JvmdiFramePop::lessThan(JvmdiFramePop &fp) {
  bool result = getFp() < fp.getFp();
  return result; 
}

bool JvmdiFramePop::equals(JvmdiFramePop &fp)   {
  return (getFp() == fp.getFp());
}

void JvmdiFramePop::print() {
#ifndef PRODUCT
  tty->print_cr("_fp=%x", _fp);
#endif  
}


//
// class VM_ChangeFramePops
//
// Alter a JvmdiFramePops object at a safepoint
//

void VM_ChangeFramePops::doit() {
  _frame_pops->set_at_safepoint(*_fp);
}

//
// class JvmdiFramePops - private methods
//

void JvmdiFramePops::set_at_safepoint(JvmdiFramePop& fp) {
  if (_fps.find(fp) == -1) {
    _fps.insert(fp);
  }
}

//
// class JvmdiFramePops - public methods
//

JvmdiFramePops::JvmdiFramePops(JvmdiThreadState *threadstate, void listener_fun(void *, address *)) 
{
  _fps.initialize(threadstate,listener_fun);
  _threadstate = threadstate;
}

JvmdiFramePops::~JvmdiFramePops() {
  _fps.clear();
}

void JvmdiFramePops::clear(address fp) {
  assert(_fps.length() > 0, "No more frame pops");
  assert(JavaThread::current() == _threadstate->get_thread(), "Only thread itself can clear frame pop");

  int len = _fps.length();
  for (int i=0; i<len; i++) {
    if (_fps.at(i).getCacheValue() == fp) {
#ifndef PRODUCT
      if (TraceJVMDICalls) {
        tty->print_cr("JvmdiFramePops::clear(0x%08lx):", fp);
        print();
      }
#endif
      _fps.remove(i);
      return;
    }
  }
  // Only clear known frame pops, they should always exist.
  ShouldNotReachHere();
}

void JvmdiFramePops::clearTo(JvmdiFramePop& fp) {
  assert(JavaThread::current() == _threadstate->get_thread(), "Only thread itself can clear frame pop");
  assert(fp.getCacheValue() != NULL, "FramePop cannot be NULL");

  // count the number of frame pops lessThan fp
  int n = 0;
  int len = _fps.length();
  for (int i=0; i<len; i++) {
    if (_fps.at(i).lessThan(fp)) {
      n++;
    } else {
      break;
    }
  }

  // remove the first n frame pops
  for (int j=0; j<n; j++) {
    _fps.remove(0);
  }
}
    
void JvmdiFramePops::set(JvmdiFramePop& fp) {
  VM_ChangeFramePops set_frame_pop(this, &fp);
  VMThread::execute(&set_frame_pop);

#ifndef PRODUCT
  if (TraceJVMDICalls) {
    tty->print_cr("JvmdiFramePops::set:");
    print();
  }
#endif
}

void JvmdiFramePops::print() {
  ResourceMark rm;

  int n = _fps.length();
  for (int i=0; i<n; i++) {
    JvmdiFramePop& fp = _fps.at(i);
    tty->print("%d: ", i);
    fp.print();
    tty->print_cr("");
  }
}


///////////////////////////////////////////////////////////////
//
// class JvmdiThreadState
//
// Instances of JvmdiThreadState hang off of each thread.
// Thread local storage for the debugger.
//

JvmdiThreadState::JvmdiThreadState(JavaThread *thread)   { 
  _local_event_enabled = NULL;
  _thread              = thread;
  _jvmdi_frame_pops    = NULL;
  _frame_pop_list      = NULL;
  _exception_detected  = false;
  _exception_caught    = false;
  _walkable_markdepth  = 0;
  _debuggable          = true;
  _current_bci         = 0;
  _current_method_id   = NULL;
  _breakpoint_posted   = false;
  _single_stepping_posted = false;
  _hide_single_stepping = false;
  _hide_level          = 0;
  
#ifndef PRODUCT
  _monitors            = NULL;
  _jvmdi_count         = 0;
#endif

}

JvmdiThreadState::~JvmdiThreadState()   { 
  delete _local_event_enabled;
  _local_event_enabled = NULL;
  delete _jvmdi_frame_pops;
  _jvmdi_frame_pops = NULL;
  _frame_pop_list   = NULL;
  // _frame_pop_list gets removed by delete _jvmdi_frame_pops

#ifndef PRODUCT
  delete _monitors;
  _monitors = NULL;
#endif
}

void JvmdiThreadState::listener_fun(void *this_obj, address *cache) {
  JvmdiThreadState *this_jvmdi = (JvmdiThreadState *) this_obj;
  assert(this_jvmdi != NULL, "this_jvmdi != NULL");

  this_jvmdi->set_frame_pop_list(cache);
}


// If no local events have been set, always return true;
bool JvmdiThreadState::is_enabled(jint event_type) const {
  if (_local_event_enabled == NULL) return false;
  return _local_event_enabled->is_enabled(event_type);
}

void JvmdiThreadState::set_enabled(jint event_type, bool enabled) {
  assert(!JvmdiEventDispatcher::is_global_event(event_type), "should not be a global event");

  if (_local_event_enabled == NULL) {
    // threads start out with all events disabled
    _local_event_enabled = new JvmdiEventEnabled(false);
  }

  _local_event_enabled->set_enabled(event_type, enabled);  
}

// Maintain the current JVM/DI location on a per-thread basis
// and use it to filter out duplicate events:
// - instruction rewrites
// - breakpoint followed by single step
// - single step at a breakpoint
void JvmdiThreadState::compare_and_set_current_location(methodOop new_method, 
  address new_location, jint event) {

  int new_bci = new_location - new_method->code_base();

  // The method is identified and stored as a JNI id which is safe in this
  // case because the class cannot be unloaded while a method is executing.
  JNIid *new_method_id = new_method->jni_id();

  // the last breakpoint or single step was at this same location
  if (_current_bci == new_bci && _current_method_id == new_method_id) {
    switch (event) {
    case JVMDI_EVENT_BREAKPOINT:
      // Repeat breakpoint is complicated. If we previously posted a breakpoint
      // event at this location and if we also single stepped at this location
      // then we skip the duplicate breakpoint.
      _breakpoint_posted = _breakpoint_posted && _single_stepping_posted;
      break;
    case JVMDI_EVENT_SINGLE_STEP:
      // Repeat single step is easy: just don't post it again.
      _single_stepping_posted = true;
      break;
    default:
      assert(false, "invalid event value passed");
      break;
    }
    return;
  }

  set_current_location(new_method_id, new_bci);
  _breakpoint_posted = false;
  _single_stepping_posted = false;
}


JvmdiFramePops& JvmdiThreadState::get_jvmdi_frame_pops() {
  if (_jvmdi_frame_pops != NULL) return (*_jvmdi_frame_pops);
  _jvmdi_frame_pops = new JvmdiFramePops(this,listener_fun);
  assert(_jvmdi_frame_pops != NULL, "_jvmdi_frame_pops != NULL");
  return (*_jvmdi_frame_pops);   
}


///////////////////////////////////////////////////////////////


//
// Wrapper function for calling the event hook
//

// Todo: make handle code efficient, HandleMarkCleaner, HandleMark, ???
void JvmdiInternal::post_event(JvmdiEvent& jvmdi_e) {
  Thread *thread  = Thread::current();
  assert(thread->is_Java_thread(), "must be called by JavaThread");

  JVMDI_EventHook hook = JvmdiInternal::get_event_hook();

  if (hook == NULL || hook == (JVMDI_EventHook)default_event_hook) {
      JVMDI_TRACE_ARG2("Dropping Event (hook not set)", jvmdi_e);
      return;
  }

  ResourceMark rm;

  JavaThread *jthread = (JavaThread *)thread;
  JNIEnv     *env     = jthread->jni_environment();

  JVMDI_Event event;
  // convert VM handles in jvmdi_e to JNIhandles in event
  jvmdi_e.to_event(event);

  ThreadToNativeFromVM transition(jthread);
  HandleMark hm(jthread);

  JVMDI_TRACE_ARG2("*** Starting Event Hook ***", jvmdi_e);
  hook(env, &event);
  JVMDI_TRACE_ARG2("*** Finished Event Hook ***", jvmdi_e);
}


// Todo: make handle code efficient, HandleMarkCleaner, HandleMark, ???
void JvmdiInternal::post_proxy_event(JvmdiEventProxy& jvmdi_e, JavaThread *jthread) {
  assert(Thread::current()->is_VM_thread(), "must be called by VMThread");

  JVMDI_EventHook hook = JvmdiInternal::get_event_hook();

  if (hook == NULL || hook == (JVMDI_EventHook)default_event_hook) {
      JVMDI_TRACE_ARG2("Dropping Proxy Event (hook not set)", jvmdi_e);
      return;
  }

  ResourceMark rm;
  JNIEnv       *env = jthread->jni_environment();

  JVMDI_Event event;
  //
  // Convert VM handles in jvmdi_e to JNIhandles in event. The VMThread
  // is proxying on behalf of a JavaThread and we are adding local JNI
  // handles to the JavaThread. This really isn't a problem since the
  // JavaThread is at a safepoint, but we have to release those local
  // JNI handles before we return to VMThread work.
  //
  jvmdi_e.to_event(event, jthread);

  // Before we call the JVM/DI agent, we have to set the state in the
  // thread for which we are proxying.
  JavaThreadState prev_state = jthread->thread_state();
  assert(prev_state == _thread_blocked, "JavaThread should be at safepoint");
  jthread->set_thread_state(_thread_in_native);

  HandleMark hm;
  JVMDI_TRACE_ARG2("*** Starting Proxy Event Hook ***", jvmdi_e);
  hook(env, &event);
  JVMDI_TRACE_ARG2("*** Finished Proxy Event Hook ***", jvmdi_e);

  // Since we are the VMThread, we need to clean up the JNIhandles that
  // were allocated above.
  jvmdi_e.destroy_local_JNI_handles(event);
  assert(jthread->thread_state() == _thread_in_native, "JavaThread should be in native");
  jthread->set_thread_state(prev_state);
}


jvmdiError JvmdiInternal::AllocateInternal(jlong size, jbyte** memPtr) {  
  if (_ahook == NULL) {
    *memPtr = (jbyte *)os::malloc((size_t)size); 
  } else {
    JavaThread* thread = JavaThread::current();
    ThreadToNativeFromVM transition(thread);
    HandleMark hm(thread);
    _ahook(size, memPtr);
  }

  if (*memPtr == NULL) {
    return JVMDI_ERROR_OUT_OF_MEMORY;
  }
  return JVMDI_ERROR_NONE;
}

jvmdiError JvmdiInternal::DeallocateInternal(jbyte* mem) {
  if (mem == NULL) {
    return JVMDI_ERROR_NULL_POINTER;
  }
  if (_dhook == NULL) {
    os::free(mem);
  } else {
    JavaThread* thread = JavaThread::current();
    ThreadToNativeFromVM transition(thread);
    HandleMark hm(thread);
    _dhook(mem);
  }
  
  return JVMDI_ERROR_NONE;
}


///////////////////////////////////////////////////////
//
// JVMDI functions that are called by the debugger
//


//
// Memory Management
//

JVMDI_ENTER(jvmdiError, SetAllocationHooks, (JVMDI_AllocHook ahook, JVMDI_DeallocHook dhook))
  JvmdiInternal::set_allocation_hooks(ahook, dhook);
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER_NO_TRACE(jvmdiError, Allocate, (jlong size, jbyte** memPtr))
  JVMDI_RETURN(JvmdiInternal::AllocateInternal(size,memPtr));
JVMDI_END

JVMDI_ENTER_NO_TRACE(jvmdiError, Deallocate, (jbyte* mem))
  JVMDI_RETURN(JvmdiInternal::DeallocateInternal(mem));
JVMDI_END

//
// Events
// 

JVMDI_ENTER(jvmdiError, SetEventHook, (JVMDI_EventHook hook))
  // setup the event_hook function
  JvmdiInternal::set_event_hook(hook);
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


static void set_method_event_flags(JavaThread *thread, jint event_type, bool enabled);

JVMDI_ENTER(jvmdiError, SetEventNotificationMode, (jint mode, jint event_type, jthread jni_thread, ...))
  
  va_list argPtr;
  va_start(argPtr, jni_thread);
  va_end(argPtr);

  if (!jvmdi::is_valid_event_type(event_type)) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_EVENT_TYPE);
  }
  
  JavaThread* java_thread = NULL;
  if (jni_thread != NULL) {
    // If a thread is specified, SetEventNotificationMode only works for threads that has run.
    java_thread = JvmdiInternal::get_JavaThread(jni_thread);
    NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);
  }

  JVMDI_TRACE_ARG3("event_type",event_type,mode == JVMDI_ENABLE);
  JVMDI_TRACE_ARG2("thread",java_thread);

  // Check if the event can be controlled at thread level.
  if (java_thread != NULL && ((event_type == JVMDI_EVENT_VM_INIT)      || 
                              (event_type == JVMDI_EVENT_VM_DEATH)     ||
			      (event_type == JVMDI_EVENT_THREAD_START) || 
			      (event_type == JVMDI_EVENT_CLASS_UNLOAD))) {
    JVMDI_RETURN(JVMDI_ERROR_ILLEGAL_ARGUMENT);
  }

  bool enabled = (mode == JVMDI_ENABLE);
  JvmdiEventDispatcher::set_enabled(java_thread, event_type, enabled);
  
  if ((event_type == JVMDI_EVENT_METHOD_EXIT)   ||
      (event_type == JVMDI_EVENT_FRAME_POP)     ||
      (event_type == JVMDI_EVENT_METHOD_ENTRY)) {
    set_method_event_flags(java_thread, event_type, enabled);
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JavaThread *JvmdiInternal::get_JavaThread(jthread jni_thread) {
  oop t = JNIHandles::resolve_external_guard(jni_thread);
  if (t == NULL || !t->is_a(SystemDictionary::thread_klass())) {
    return NULL;
  }
  // The following returns NULL if the thread has not yet run or is in
  // process of exiting
  return java_lang_Thread::thread(t);
}


jobject *JvmdiInternal::new_jobjectArray(int length, Handle *handles) {
  if (length == 0) { 
    return NULL;
  }

  jobject *objArray = (jobject *) jvmdiMalloc(sizeof(jobject) * length);
  for (int i=0; i<length; i++) {
    objArray[i] = JNIHandles::make_global(handles[i]);
  }  
  return objArray;
}

jthread *JvmdiInternal::new_jthreadArray(int length, Handle *handles) {
  return (jthread *) new_jobjectArray(length,handles);
}

jthreadGroup *JvmdiInternal::new_jthreadGroupArray(int length, Handle *handles) {
  return (jthreadGroup *) new_jobjectArray(length,handles);
}



//
// Thread Execution Functions
//


JVMDI_ENTER(jvmdiError, GetThreadStatus, (jthread jni_thread,
                                          jint *threadStatusPtr, 
                                          jint *suspendStatusPtr))
  // We don't use get_JavaThread() here because the target thread may not
  // have run yet. get_JavaThread() only works on threads that have run.
  NULL_CHECK(jni_thread, JVMDI_ERROR_INVALID_THREAD);
  oop thread_oop = JNIHandles::resolve_external_guard(jni_thread);
  if (thread_oop == NULL || !thread_oop->is_a(SystemDictionary::thread_klass())) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_THREAD);
  }
  // at this point, we have a valid Thread object
  JavaThread* thr = java_lang_Thread::thread(thread_oop);    
  if (thr == NULL) {
    // the thread hasn't run yet or is in the process of exiting
    *threadStatusPtr = JVMDI_THREAD_STATUS_UNKNOWN;
    *suspendStatusPtr = 0;
  } else if (thr->osthread() == NULL) {
    *threadStatusPtr = JVMDI_THREAD_STATUS_ZOMBIE;
    *suspendStatusPtr = 0;
  } else {

    if (thr->is_being_ext_suspended()) {
      // internal suspend doesn't count for this flag
      *suspendStatusPtr  = JVMDI_SUSPEND_STATUS_SUSPENDED; 
    } else if (thr->osthread()->get_state() == BREAKPOINTED) {
      *suspendStatusPtr  = JVMDI_SUSPEND_STATUS_BREAK;
    } else {
      *suspendStatusPtr  = 0;
    }
  
    switch (thr->osthread()->get_state()) {
      case SLEEPING:
	*threadStatusPtr = JVMDI_THREAD_STATUS_SLEEPING;
	break;
      case CONDVAR_WAIT:
	*threadStatusPtr = JVMDI_THREAD_STATUS_WAIT;
	break;
      case MONITOR_WAIT:
	*threadStatusPtr = JVMDI_THREAD_STATUS_MONITOR;
	break;
      case ZOMBIE:
	*threadStatusPtr = JVMDI_THREAD_STATUS_ZOMBIE;
	break;
      case RUNNABLE:
	*threadStatusPtr = JVMDI_THREAD_STATUS_RUNNING;
	break;
      default:
	*threadStatusPtr = JVMDI_THREAD_STATUS_UNKNOWN;
    }
  }
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, GetAllThreads, (jint *threadsCountPtr, jthread **threadsPtr))

  int nthreads        = 0;
  int ndebugthreads   = 0;
  Handle *thread_objs = NULL;

  ResourceMark rm;
  HandleMark hm;

  { MutexLocker mu(Threads_lock);

    nthreads = Threads::number_of_threads();
    if (nthreads == 0) {
      *threadsCountPtr = 0;
      *threadsPtr      = 0;
      JVMDI_RETURN(JVMDI_ERROR_NONE);
    }
    thread_objs = NEW_RESOURCE_ARRAY(Handle, nthreads);
    NULL_CHECK(thread_objs, JVMDI_ERROR_OUT_OF_MEMORY);

    for (JavaThread *tp = Threads::first(); tp != NULL; tp = tp->next()) {
      assert(tp->is_Java_thread(), "Must be JavaThread");
      assert(ndebugthreads < nthreads, "Number of threads doesn't match");
      if (ndebugthreads >= nthreads) break;  // robustness: no more room in the array

      JavaThreadState state = tp->thread_state();
      // skip threads that haven't run yet
      if (state == _thread_new || state == _thread_new_trans) continue;
      // skip threads that have died, but haven't finished going away
      if (!java_lang_Thread::is_alive(tp->threadObj())) continue;

      thread_objs[ndebugthreads++] = tp->threadObj();
    }
  }

  // have to make global handles outside of Threads_lock
  jthread *jthreads  = JvmdiInternal::new_jthreadArray(ndebugthreads,thread_objs);
  NULL_CHECK(jthreads, JVMDI_ERROR_OUT_OF_MEMORY);

  #ifndef PRODUCT
  if (TraceJVMDICalls) {
    for (int i=0; i<ndebugthreads; i++) {
      JVMDI_TRACE_ARG2("thread",java_lang_Thread::thread(thread_objs[i]()));
    }
  }
  #endif  

  *threadsCountPtr = ndebugthreads;
  *threadsPtr      = jthreads;

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, hSuspendThread, (jthread jni_thread))

 JavaThread *java_thread = JvmdiInternal::get_JavaThread(jni_thread);
 NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);
 JVMDI_TRACE_ARG2("thread", java_thread);

 if (java_thread->is_being_ext_suspended()) {
   JVMDI_RETURN(JVMDI_ERROR_THREAD_SUSPENDED);
 }
 
 if (!JvmdiWalkableThreads::suspend(java_thread)) {
   // the thread has not yet run or is in process of exiting
   JVMDI_RETURN(JVMDI_ERROR_INVALID_THREAD);
 }

 JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, hResumeThread, (jthread jni_thread))

  JavaThread* java_thread = JvmdiInternal::get_JavaThread(jni_thread);
  NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);
  JVMDI_TRACE_ARG2("thread", java_thread);

  if (!java_thread->is_being_ext_suspended()) {
    JVMDI_RETURN(JVMDI_ERROR_THREAD_NOT_SUSPENDED);
  }

  if (!JvmdiWalkableThreads::resume(java_thread)) {
    JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, StopThread, (jthread jni_thread, jobject exception))
  JavaThread* thr = JvmdiInternal::get_JavaThread(jni_thread);
  NULL_CHECK(thr, JVMDI_ERROR_INVALID_THREAD);
  oop e = JNIHandles::resolve_external_guard(exception);
  NULL_CHECK(e, JVMDI_ERROR_NULL_POINTER);
  JVMDI_TRACE_ARG2("thread", thr);

  JavaThread::send_async_exception(thr->threadObj(), e);
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


// Todo: this is a duplicate of JVM_Interrupt; share code in future
JVMDI_ENTER(jvmdiError, InterruptThread, (jthread jni_thread))
  // Ensure that the C++ Thread and OSThread structures aren't freed before we operate
  JavaThread* java_thread = JvmdiInternal::get_JavaThread(jni_thread);
  NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);

  MutexLockerEx ml(thread->threadObj() == java_thread->threadObj() ? NULL : Threads_lock);
  // We need to re-resolve the java_thread, since a GC might have happened during the
  // acquire of the lock
  java_thread = JvmdiInternal::get_JavaThread(jni_thread);
  NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);
  
  JVMDI_TRACE_ARG2("thread", java_thread);
  
  Thread::interrupt(java_thread);

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, GetThreadInfo, (jthread jni_thread, JVMDI_thread_info *infoPtr))
  // We don't use get_JavaThread() here because the target thread may not
  // have run yet. get_JavaThread() only works on threads that have run.
  NULL_CHECK(jni_thread, JVMDI_ERROR_INVALID_THREAD);
  oop thread_oop = JNIHandles::resolve_external_guard(jni_thread);
  if (thread_oop == NULL || !thread_oop->is_a(SystemDictionary::thread_klass())) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_THREAD);
  }

  ResourceMark rm;
  HandleMark hm;

  Handle thread_obj(THREAD, thread_oop);
  typeArrayHandle    name;
  ThreadPriority priority;
  Handle     thread_group;
  bool          is_daemon;

  JVMDI_TRACE_ARG2("thread", thread_oop);
  
  { MutexLocker mu(Threads_lock);

            name = java_lang_Thread::name(thread_obj());
        priority = java_lang_Thread::priority(thread_obj());
    thread_group = java_lang_Thread::threadGroup(thread_obj());
       is_daemon = java_lang_Thread::is_daemon(thread_obj());
  }

  { const char *n = UNICODE::as_utf8((jchar*) name->base(T_CHAR), name->length()); 
    infoPtr->name = (char *) JvmdiInternal::jvmdiMalloc(strlen(n)+1);
    NULL_CHECK(infoPtr->name, JVMDI_ERROR_OUT_OF_MEMORY);
    strcpy(infoPtr->name, n);
  }

  infoPtr->is_daemon = is_daemon;
  infoPtr->priority  = priority;
  infoPtr->context_class_loader = NULL;
  infoPtr->thread_group = JNIHandles::make_global(thread_group());

  JVMDI_TRACE_ARG2("returns name", infoPtr->name);
  JVMDI_TRACE_ARG2("returns is_daemon", infoPtr->is_daemon ? "true" : "false");
  JVMDI_TRACE_ARG2("returns priority", infoPtr->priority);

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetOwnedMonitorInfo, (jthread jni_thread, 
                                                      JVMDI_owned_monitor_info *infoPtr))
  JavaThread* java_thread = JvmdiInternal::get_JavaThread(jni_thread);
  NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);
  JVMDI_TRACE_ARG2("thread", java_thread);

  JVMDI_RETURN(JVMDI_ERROR_NOT_IMPLEMENTED);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetCurrentContendedMonitor, (jthread jni_thread, jobject *monitor))
  JavaThread* java_thread = JvmdiInternal::get_JavaThread(jni_thread);
  NULL_CHECK(java_thread, JVMDI_ERROR_INVALID_THREAD);
  JVMDI_TRACE_ARG2("thread", java_thread);

  JVMDI_RETURN(JVMDI_ERROR_NOT_IMPLEMENTED);
JVMDI_END

DebuggerThread::DebuggerThread(JVMDI_StartFunction start_fn, void *start_arg)
  : JavaThread(start_function_wrapper) {
  _start_fn = start_fn;
  _start_arg = start_arg;
}

void DebuggerThread::start_function_wrapper(JavaThread *thread, TRAPS)
{
  // It is expected that any debugger threads will be created as
  // Java Threads.  If this is the case, notification of the creation
  // of the thread is given in JavaThread::thread_main().
  assert(thread->is_Java_thread(), "debugger thread should be a Java Thread");

  DebuggerThread *dthread = (DebuggerThread *)thread;
  JavaThread *current_thread = JavaThread::current();
  HandleMark hm(thread);
  ThreadToNativeFromVM transition(current_thread);
  StateSaver ss(thread);
  dthread->_start_fn(dthread->_start_arg);
}


// XXX - This function is similar to create_system_thread(); share code
// in the future.
JVMDI_ENTER(jvmdiError, RunDebugThread, (jthread jni_thread, 
                                         JVMDI_StartFunction proc, 
                                         void *arg,
                                         int priority))

  // We don't use get_JavaThread() here because the debug thread has not
  // run yet. get_JavaThread() only works on threads that have run.
  oop t = JNIHandles::resolve_external_guard(jni_thread);
  if (t == NULL || !t->is_a(SystemDictionary::thread_klass())) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_THREAD);
  }
  
  Handle thread_oop(t);
  {
    MutexLocker mu(Threads_lock); // grab Threads_lock

    DebuggerThread *new_thread = new DebuggerThread(proc, arg);

    // At this point it may be possible that no osthread was created for the
    // JavaThread due to lack of memory. We would have to throw an exception
    // in that case. However, since this must work and we do not allow
    // exceptions anyway, just use an assertion here.
    guarantee(new_thread->osthread() != NULL, "thread creation failed");

    java_lang_Thread::set_thread(thread_oop(), new_thread);
    java_lang_Thread::set_priority(thread_oop(), (ThreadPriority)priority);
    java_lang_Thread::set_daemon(thread_oop());

    new_thread->set_threadObj(thread_oop());
    Threads::add(new_thread);
    JVMDI_TRACE_ARG2("new_thread", new_thread);
    Thread::start(new_thread);
  } // unlock Threads_lock

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

//
// Thread Groups
//

JVMDI_ENTER(jvmdiError, GetTopThreadGroups, (jint *groupCountPtr, 
                                             jthreadGroup **groupsPtr))
  // Only one top level thread group now.
  *groupCountPtr = 1;
  
  // Allocate memory to store global-refs to the thread groups.
  // Assume this area is freed by caller.
  *groupsPtr = (jthreadGroup *) JvmdiInternal::jvmdiMalloc((sizeof(jthreadGroup)) * (*groupCountPtr));
  
  NULL_CHECK(*groupsPtr, JVMDI_ERROR_OUT_OF_MEMORY);

  // Convert oop to Handle, then convert Handle to global-ref.
  {
    Handle system_thread_group(thread, Universe::system_thread_group()); 
    *groupsPtr[0] = JNIHandles::make_global(system_thread_group);
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, GetThreadGroupInfo, (jthreadGroup group, 
                                             JVMDI_thread_group_info *infoPtr))
  NULL_CHECK(infoPtr, JVMDI_ERROR_NULL_POINTER);

  ResourceMark rm;
  HandleMark hm;

  Handle group_obj = JNIHandles::resolve_external_guard(group);
  NULL_CHECK(group_obj, JVMDI_ERROR_INVALID_THREAD_GROUP);

  typeArrayHandle name;
  Handle parent_group;
  bool is_daemon;
  ThreadPriority max_priority;

  { MutexLocker mu(Threads_lock);

    name         = java_lang_ThreadGroup::name(group_obj());
    parent_group = java_lang_ThreadGroup::parent(group_obj());
    is_daemon    = java_lang_ThreadGroup::is_daemon(group_obj());
    max_priority = java_lang_ThreadGroup::maxPriority(group_obj());
  }

  infoPtr->is_daemon    = is_daemon;
  infoPtr->max_priority = max_priority;
  infoPtr->parent       = JNIHandles::make_global(parent_group());

  { const char *n = UNICODE::as_utf8((jchar*) name->base(T_CHAR), name->length()); 
  infoPtr->name = (char *) JvmdiInternal::jvmdiMalloc(strlen(n)+1);    

    NULL_CHECK(infoPtr->name, JVMDI_ERROR_OUT_OF_MEMORY);

    strcpy(infoPtr->name, n);
  }
   
  JVMDI_TRACE_ARG2("returns name",         infoPtr->name);
  JVMDI_TRACE_ARG2("returns max_priority", infoPtr->max_priority);
  JVMDI_TRACE_ARG2("returns isDaemon",     infoPtr->is_daemon);


  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, GetThreadGroupChildren, (jthreadGroup group, 
                                                 jint *threadCountPtr, 
                                                 jthread **threadsPtr,
                                                 jint *groupCountPtr, 
                                                 jthreadGroup **groupsPtr))

 oop group_obj = (oop) JNIHandles::resolve_external_guard(group);
 NULL_CHECK(group_obj, JVMDI_ERROR_INVALID_THREAD_GROUP);
 NULL_CHECK(threadCountPtr, JVMDI_ERROR_NULL_POINTER);
 NULL_CHECK(threadsPtr, JVMDI_ERROR_NULL_POINTER);
 NULL_CHECK(groupCountPtr, JVMDI_ERROR_NULL_POINTER);
 NULL_CHECK(groupsPtr, JVMDI_ERROR_NULL_POINTER);

 Handle *thread_objs = NULL;
 Handle *group_objs  = NULL;
 int nthreads = 0;
 int ngroups = 0;

 ResourceMark rm;
 HandleMark hm;

 { MutexLocker mu(Threads_lock);

   nthreads = java_lang_ThreadGroup::nthreads(group_obj);
   ngroups  = java_lang_ThreadGroup::ngroups(group_obj);

   if (nthreads > 0) {
     objArrayOop threads = java_lang_ThreadGroup::threads(group_obj);
     assert(nthreads <= threads->length(), "too many threads");
     thread_objs = NEW_RESOURCE_ARRAY(Handle,nthreads);
     for (int i=0; i<nthreads; i++) {
       oop thread_obj = threads->obj_at(i);
       assert(thread_obj != NULL, "thread_obj != NULL");
       thread_objs[i] = thread_obj;
     }
   }

   if (ngroups > 0) {
     objArrayOop groups = java_lang_ThreadGroup::groups(group_obj);
     assert(ngroups <= groups->length(), "too many threads");
     group_objs = NEW_RESOURCE_ARRAY(Handle,ngroups);
     for (int i=0; i<ngroups; i++) {
       oop group_obj = groups->obj_at(i);
       assert(group_obj != NULL, "group_obj != NULL");
       group_objs[i] = group_obj;
     }
   }
  }

  // have to make global handles outside of Threads_lock
  *groupCountPtr  = ngroups;
  *threadCountPtr = nthreads;
  *threadsPtr     = JvmdiInternal::new_jthreadArray(nthreads, thread_objs);
  *groupsPtr      = JvmdiInternal::new_jthreadGroupArray(ngroups, group_objs);

  if ((nthreads > 0) && (*threadsPtr == NULL)) {
     JVMDI_RETURN(JVMDI_ERROR_OUT_OF_MEMORY);
  }

  if ((ngroups > 0) && (*groupsPtr == NULL)) {
     JVMDI_RETURN(JVMDI_ERROR_OUT_OF_MEMORY);
  }

   JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

//
// Stack Frame Access
//

JVMDI_ENTER(jvmdiError, GetFrameCount, (jthread jni_thread, jint *countPtr))

  JavaThread* thr = JvmdiInternal::get_JavaThread(jni_thread); 
  NULL_CHECK(thr, JVMDI_ERROR_INVALID_THREAD);
  NULL_CHECK(countPtr, JVMDI_ERROR_NULL_POINTER);
  JVMDI_TRACE_ARG2("thread",thr);

  int nFrames;
  if (thr == JavaThread::current()) {
    JvmdiWalkableMark wm(thr);
    nFrames = JvmdiWalkableThreads::nFrames(thr);
  } else {
    nFrames = JvmdiWalkableThreads::nFrames(thr);
  }
  JVMDI_TRACE_ARG2("returns nFrames",nFrames);

  if (nFrames == -1) {
    JVMDI_RETURN(JVMDI_ERROR_THREAD_NOT_SUSPENDED);
  }
    
  *countPtr = nFrames;
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, GetCurrentFrame, (jthread jni_thread, jframeID *framePtr))

  JavaThread* thr = JvmdiInternal::get_JavaThread(jni_thread); 
  NULL_CHECK(thr, JVMDI_ERROR_INVALID_THREAD);
  NULL_CHECK(framePtr, JVMDI_ERROR_NULL_POINTER);
  JVMDI_TRACE_ARG2("thread",thr);

  jframeID current = NULL;
  if (thr == JavaThread::current()) {
    // If thr is the current thread, then we need to make it walkable
    // in order to get the jframeID. Since we aren't doing anything
    // with the jframeID in this method, we only need the walkable
    // mark for the current_frame() call. This assumes that jframeIDs
    // are the same for unchanged stack frames (implemented by 4335224).
    JvmdiWalkableMark wm(thr);
    current = JvmdiWalkableThreads::current_frame(thr);
  } else {
    current = JvmdiWalkableThreads::current_frame(thr);
  }

  if (current == JvmdiWalkableThread::unknown_jframeID()) {
    JVMDI_RETURN(JVMDI_ERROR_NO_MORE_FRAMES);
  }

  if (current == JvmdiWalkableThread::invalid_jframeID()) {
    JVMDI_RETURN(JVMDI_ERROR_THREAD_NOT_SUSPENDED);
  }

  *framePtr = current;
  JVMDI_TRACE_ARG2("returned framePtr", (int)current);

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetCallerFrame, (jframeID called, jframeID *framePtr))
  NULL_CHECK(called, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(framePtr, JVMDI_ERROR_NULL_POINTER);

  if (called == JvmdiWalkableThread::invalid_jframeID()
    || called == JvmdiWalkableThread::unknown_jframeID()) {
    JVMDI_RETURN(JVMDI_ERROR_NO_MORE_FRAMES);
  } 

  jframeID parent = NULL;
  if (!JvmdiWalkableThreads::get_caller(called, &parent)) {
    // Cannot find the parent because called is not associated with a
    // walkable thread. We make the current thread walkable and try
    // again. If that works, then we are querying ourselves. If that
    // doesn't work, then called is either an invalid frame or is
    // associated with a thread that is not suspended.
    //
    // Since we aren't doing anything with the parent's jframeID in
    // this method, we only need the walkable mark for the get_caller()
    // call. This assumes that jframeIDs are the same for unchanged
    // stack frames (implemented by 4335224).
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(called) ||
        !JvmdiWalkableThreads::get_caller(called, &parent)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  if (parent == called) {
      JVMDI_RETURN(JVMDI_ERROR_NO_MORE_FRAMES);
  }

  *framePtr = parent;
  JVMDI_TRACE_ARG2("returned framePtr", (int)parent);

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

// Todo: do these global handles have to get collected?!?
JVMDI_ENTER(jvmdiError, GetFrameLocation, (jframeID frame, 
                                           jclass *classPtr, 
                                           jmethodID *methodPtr,
                                           jlocation *locationPtr))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(classPtr, JVMDI_ERROR_NULL_POINTER);
  NULL_CHECK(methodPtr, JVMDI_ERROR_NULL_POINTER);
  NULL_CHECK(locationPtr, JVMDI_ERROR_NULL_POINTER);

  JVMDI_TRACE_ARG2("frame", (int)frame);

  if (JvmdiWalkableThreads::is_opaque(frame)) {
    // frame might be opaque because it is not associated with a
    // walkable thread. We make the current thread walkable and try
    // again. If the frame is no longer opaque, then we are querying
    // ourselves and we need the walkable mark through the
    // get_frame_location() call.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        JvmdiWalkableThreads::is_opaque(frame)) {
      JVMDI_RETURN(JVMDI_ERROR_OPAQUE_FRAME);
    }

    if (!JvmdiWalkableThreads::get_frame_location(frame, classPtr, methodPtr, locationPtr)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  } else if (!JvmdiWalkableThreads::get_frame_location(frame, classPtr, methodPtr, locationPtr)) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
  } 

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER(jvmdiError, NotifyFramePop, (jframeID frame))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  JVMDI_TRACE_ARG2("frame", (int)frame);

  if (JvmdiWalkableThreads::is_opaque(frame)) {
    // frame might be opaque because it is not associated with a
    // walkable thread. We make the current thread walkable and try
    // again. If the frame is no longer opaque, then we are querying
    // ourselves and we need the walkable mark through the
    // notify_frame_pop() call. This assumes that jframeIDs are the
    // same for unchanged stack frames (implemented by 4335224).
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        JvmdiWalkableThreads::is_opaque(frame)) {
      JVMDI_RETURN(JVMDI_ERROR_OPAQUE_FRAME);
    }

    if (!JvmdiWalkableThreads::notify_frame_pop(frame)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  } else if (!JvmdiWalkableThreads::notify_frame_pop(frame)) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);

JVMDI_END

//
// Local Variable Access
//
 
JVMDI_ENTER(jvmdiError, GetLocalObject, (jframeID frame, 
                                         jint slot, 
				         jobject *valuePtr))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(valuePtr, JVMDI_ERROR_NULL_POINTER);

  HandleMark hm;
  Handle h;

  if (!JvmdiWalkableThreads::get_local(frame, slot, &h)) {
    // get_local() might have failed because frame is not associated
    // with a walkable thread. We make the current thread walkable
    // and try again. If that works, then we are querying ourselves.
    // If that doesn't work, then frame is either invalid or is
    // associated with a thread that is not suspended. We only need
    // the walkable mark for the get_local() call since remaining
    // use is via a handle.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::get_local(frame, slot, &h)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  *valuePtr = JvmdiInternal::get_jni_object(h());
  
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetLocalInt, (jframeID frame, 
                                      jint slot, 
                                      jint *valuePtr))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(valuePtr, JVMDI_ERROR_NULL_POINTER);

  if (!JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
    // get_local() might have failed because frame is not associated
    // with a walkable thread. We make the current thread walkable
    // and try again. If that works, then we are querying ourselves.
    // If that doesn't work, then frame is either invalid or is
    // associated with a thread that is not suspended.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetLocalLong, (jframeID frame, 
                                       jint slot, 
                                       jlong *valuePtr))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(valuePtr, JVMDI_ERROR_NULL_POINTER);

  if (!JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
    // See GetLocalInt() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetLocalFloat, (jframeID frame, 
                                        jint slot, 
                                        jfloat *valuePtr))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(valuePtr, JVMDI_ERROR_NULL_POINTER);

  if (!JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
    // See GetLocalInt() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, GetLocalDouble, (jframeID frame, 
                                         jint slot, 
                                         jdouble *valuePtr))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);
  NULL_CHECK(valuePtr, JVMDI_ERROR_NULL_POINTER);

  if (!JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
    // See GetLocalInt() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::get_local(frame, slot, valuePtr)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, SetLocalObject, (jframeID frame, 
                                         jint slot, jobject value))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);

  HandleMark hm;
  Handle object_handle = Handle(THREAD, JvmdiInternal::get_Oop(value) );
  if (!JvmdiWalkableThreads::set_local(frame, slot, object_handle)) {
    // set_local() might have failed because frame is not associated
    // with a walkable thread. We make the current thread walkable
    // and try again. If that works, then we are modifying ourselves.
    // If that doesn't work, then frame is either invalid or is
    // associated with a thread that is not suspended.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::set_local(frame, slot, object_handle)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, SetLocalInt, (jframeID frame, 
                                      jint slot, 
                                      jint value))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);

  if (!JvmdiWalkableThreads::set_local(frame, slot, value)) {
    // See SetLocalObject() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::set_local(frame, slot, value)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, SetLocalLong, (jframeID frame, 
                                       jint slot, 
                                       jlong value))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);

  if (!JvmdiWalkableThreads::set_local(frame, slot, value)) {
    // See SetLocalObject() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::set_local(frame, slot, value)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, SetLocalFloat, (jframeID frame, 
                                        jint slot, 
                                        jfloat value))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);

  if (!JvmdiWalkableThreads::set_local(frame, slot, value)) {
    // See SetLocalObject() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::set_local(frame, slot, value)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, SetLocalDouble, (jframeID frame, 
                                         jint slot, 
                                         jdouble value))
  NULL_CHECK(frame, JVMDI_ERROR_INVALID_FRAMEID);

  if (!JvmdiWalkableThreads::set_local(frame, slot, value)) {
    // See SetLocalObject() comments above.
    JvmdiWalkableMark wm(JavaThread::current());
    if (!JvmdiWalkableThreads::load_frame(frame) ||
        !JvmdiWalkableThreads::set_local(frame, slot, value)) {
      JVMDI_RETURN(JVMDI_ERROR_INVALID_FRAMEID);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

// 
// Raw Monitor Support
//

bool JvmdiInternal::catch_exception(int raw_result, TRAPS) {
  bool result = false;
  assert(raw_result == ObjectMonitor::OM_OK, "raw_result == OM_OK");

  if (raw_result != ObjectMonitor::OM_OK) {
    result = true;
  }

  if (HAS_PENDING_EXCEPTION) {
    CLEAR_PENDING_EXCEPTION;
    result = true;
  }

  return result;
}

static int JVMDI_RM_MAGIC = (int)(('D' << 24) | ('I' << 16) | ('R' << 8) | 'M');
#define JVMDI_RM_CHECK(m)                                 \
    if ((m) == NULL || (m)->magic() != JVMDI_RM_MAGIC) {  \
	JVMDI_RETURN(JVMDI_ERROR_INVALID_MONITOR);        \
    }

JVMDI_ENTER(jvmdiError, CreateRawMonitor, (char *name, 
                                           JVMDI_RawMonitor *monitorPtr))
  NULL_CHECK(name, JVMDI_ERROR_NULL_POINTER);
  NULL_CHECK(monitorPtr, JVMDI_ERROR_NULL_POINTER);
  JVMDI_TRACE_ARG2("name", name);

  JvmdiMonitor* monitor = new JvmdiMonitor(name, JVMDI_RM_MAGIC);
  NULL_CHECK(monitor, JVMDI_ERROR_OUT_OF_MEMORY);
  
  *monitorPtr = monitor;

  JVMDI_TRACE_ARG2("returns", (JvmdiMonitor *) monitor);
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, DestroyRawMonitor, (JVMDI_RawMonitor _monitor))
  JvmdiMonitor *monitor = (JvmdiMonitor *)_monitor;
  JVMDI_RM_CHECK(monitor);

  JVMDI_TRACE_ARG2("monitor", monitor);

  if (monitor->monitor().is_enterred(THREAD)) {
    // The caller owns this monitor which we are about to destroy.
    // We exit the underlying synchronization object so that the
    // "delete monitor" call below can work without an assertion
    // failure on systems that don't like destroying synchronization
    // objects that are locked.
    int r;
    PRESERVE_EXCEPTION_MARK;
    r = monitor->monitor().raw_exit(THREAD, false);
    assert(r == ObjectMonitor::OM_OK, "raw_exit should have worked");
    if (r != ObjectMonitor::OM_OK) {  // robustness
      JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
    if (JvmdiInternal::catch_exception(r, THREAD)) {
      JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
  }
  if (monitor->monitor().owner() != NULL) {
    // The caller is trying to destroy a monitor that is locked by
    // someone else. While this is not forbidden by the JVM/DI 1.0
    // spec, it will cause an assertion failure on systems that don't
    // like destroying synchronization objects that are locked.
    // We indicate a problem with the error return (and leak the
    // monitor's memory).
    JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
  }

  delete monitor;
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER_IN_NATIVE_NO_TRACE(jvmdiError, RawMonitorEnter, (JVMDI_RawMonitor _monitor))
  JvmdiMonitor *monitor = (JvmdiMonitor *)_monitor;
  JVMDI_RM_CHECK(monitor);

//JVMDI_TRACE_ARG2("monitor", monitor);

  { int r;
    PRESERVE_EXCEPTION_MARK;
    r = monitor->monitor().raw_enter(THREAD, false);    
    if (JvmdiInternal::catch_exception(r, THREAD)) {
	JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END



JVMDI_ENTER_IN_NATIVE_NO_TRACE(jvmdiError, RawMonitorExit, (JVMDI_RawMonitor _monitor))
  JvmdiMonitor *monitor = (JvmdiMonitor *)_monitor;
  JVMDI_RM_CHECK(monitor);

//JVMDI_TRACE_ARG2("monitor", monitor);

  { int r;
    PRESERVE_EXCEPTION_MARK;
    r = monitor->monitor().raw_exit(THREAD, false);
    if (r == ObjectMonitor::OM_ILLEGAL_MONITOR_STATE) {
      JVMDI_RETURN(JVMDI_ERROR_NOT_MONITOR_OWNER);
    }
    if (JvmdiInternal::catch_exception(r, THREAD)) {
	JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER_IN_NATIVE_NO_TRACE(jvmdiError, RawMonitorWait, (JVMDI_RawMonitor _monitor, 
                                                 jlong millis))
  JvmdiMonitor *monitor = (JvmdiMonitor *)_monitor;
  JVMDI_RM_CHECK(monitor);
//JVMDI_TRACE_ARG2("monitor", monitor);

  { int r;
    PRESERVE_EXCEPTION_MARK;

    r = monitor->monitor().raw_wait(millis, true, THREAD);
    switch (r) {
    case ObjectMonitor::OM_INTERRUPTED:
      JVMDI_RETURN(JVMDI_ERROR_INTERRUPT);
    case ObjectMonitor::OM_ILLEGAL_MONITOR_STATE:
      JVMDI_RETURN(JVMDI_ERROR_NOT_MONITOR_OWNER);
    }
    if (JvmdiInternal::catch_exception(r, THREAD)) {
      JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
  }
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER_NO_TRACE(jvmdiError, RawMonitorNotify, (JVMDI_RawMonitor _monitor))
  JvmdiMonitor *monitor = (JvmdiMonitor *)_monitor;
  JVMDI_RM_CHECK(monitor);

//JVMDI_TRACE_ARG2("monitor", monitor);

  { int r; 
    PRESERVE_EXCEPTION_MARK;
    r = monitor->monitor().raw_notify(THREAD);
    if (r == ObjectMonitor::OM_ILLEGAL_MONITOR_STATE) {
      JVMDI_RETURN(JVMDI_ERROR_NOT_MONITOR_OWNER);
    }
    if (JvmdiInternal::catch_exception(r, THREAD)) {
	JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
  }
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


JVMDI_ENTER_NO_TRACE(jvmdiError, RawMonitorNotifyAll, (JVMDI_RawMonitor _monitor))
  JvmdiMonitor *monitor = (JvmdiMonitor *)_monitor;
  JVMDI_RM_CHECK(monitor);

//JVMDI_TRACE_ARG2("monitor", monitor);

  { int r;
    PRESERVE_EXCEPTION_MARK;
    r = monitor->monitor().raw_notifyAll(THREAD);
    if (r == ObjectMonitor::OM_ILLEGAL_MONITOR_STATE) {
      JVMDI_RETURN(JVMDI_ERROR_NOT_MONITOR_OWNER);
    }
    if (JvmdiInternal::catch_exception(r, THREAD)) {
	JVMDI_RETURN(JVMDI_ERROR_INTERNAL);
    }
  }
  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


//
// Breakpoints
//

JVMDI_ENTER(jvmdiError, SetBreakpoint, (jclass clazz, jmethodID method, jlocation location))
  klassOop k = JvmdiInternal::get_klassOop(clazz);
  NULL_CHECK(k, JVMDI_ERROR_INVALID_CLASS);
  methodOop m = JvmdiInternal::get_methodOop(method, k);
  NULL_CHECK(m, JVMDI_ERROR_INVALID_METHODID);
  if (location < 0) {	// simple invalid location check first
    JVMDI_RETURN(JVMDI_ERROR_INVALID_LOCATION);
  }

  // verify that the breakpoint is not past the end of the method
  if (location >= (jlocation) m->code_size()) {
    JVMDI_RETURN(JVMDI_ERROR_INVALID_LOCATION);
  }

  ResourceMark rm;
  JvmdiBreakpoint bp(clazz, method, location);
  JVMDI_TRACE_ARG2("breakpoint", bp);

  /////////////////////////////////////////////////////////////////////////////
  //
  // BEGIN HACK ALERT (4/8/99)
  //
  // Currently, our mechanism for seeing when we hit a breakpoint
  // requires a check at every bytecode. We would like to not have to
  // pay this cost unless we actually have user breakpoints set.
  // However, JDI sets its own internal breakpoint on
  // java.lang.Thread.resume so user level resume's can't disturb
  // debugger suspended threads. For now, don't give JDI its breakpoint
  // so we can run faster in the common case. When breakpoints no
  // longer exact a per/bytecode cost this code should get removed!
  //

  if (!UseFastBreakpoints) {
    const char *class_name  = (m == NULL) ? "NULL" : m->klass_name()->as_C_string();
    const char *method_name = (m == NULL) ? "NULL" : m->name()->as_C_string();
    
    if ((class_name != NULL) && (strcmp(class_name,"java/lang/Thread") == 0)) {
      if ((method_name != NULL) && (strcmp(method_name,"resume") == 0)) {
	warning("Ignoring breakpoint on java.lang.Thread.resume");
	JVMDI_RETURN(JVMDI_ERROR_NONE);
      }
    }
  }

  //
  // END HACK ALERT (4/8/99)
  //
  /////////////////////////////////////////////////////////////////////////////

  JvmdiBreakpoints& jvmdi_breakpoints = JvmdiCurrentBreakpoints::get_jvmdi_breakpoints();
  if (jvmdi_breakpoints.set(bp) == JVMDI_ERROR_DUPLICATE)
	JVMDI_RETURN(JVMDI_ERROR_DUPLICATE);

  if (TraceJVMDICalls) {
    jvmdi_breakpoints.print();
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, ClearBreakpoint, (jclass clazz, jmethodID method, 
                                          jlocation location))
  JvmdiBreakpoint bp(clazz, method, location);
  JVMDI_TRACE_ARG2("breakpoint", bp);

  JvmdiBreakpoints& jvmdi_breakpoints = JvmdiCurrentBreakpoints::get_jvmdi_breakpoints();
  if (jvmdi_breakpoints.clear(bp) == JVMDI_ERROR_NOT_FOUND)
	JVMDI_RETURN(JVMDI_ERROR_NOT_FOUND);

  if (TraceJVMDICalls) {
    jvmdi_breakpoints.print();
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END

JVMDI_ENTER(jvmdiError, ClearAllBreakpoints, ())

  JvmdiBreakpoints& jvmdi_breakpoints = JvmdiCurrentBreakpoints::get_jvmdi_breakpoints();
  jvmdi_breakpoints.clearall();
  if (TraceJVMDICalls) {
    jvmdi_breakpoints.print();
  }

  JVMDI_RETURN(JVMDI_ERROR_NONE);
JVMDI_END


//
// Watched Fields
//

static jvmdiError field_watch_common(jclass clazz, jfieldID field,
jint watch_type, bool value) {
  // Check arguments
  oop mirror = JNIHandles::resolve_external_guard(clazz);
  if (mirror == NULL) return JVMDI_ERROR_INVALID_CLASS;
  if (field == NULL) return JVMDI_ERROR_INVALID_FIELDID;
  if (mirror->klass() != SystemDictionary::class_klass() ||
      java_lang_Class::is_primitive(mirror)) {
    return JVMDI_ERROR_INVALID_CLASS;
  }
  klassOop k = java_lang_Class::as_klassOop(mirror);
  if (k == NULL) return JVMDI_ERROR_INVALID_CLASS;

  ResourceMark rm;
  fieldDescriptor fd;
  // use get_field_descriptor to validate field and get current access flags
  if (!JvmdiInternal::get_field_descriptor(k, field, &fd)) {
    return JVMDI_ERROR_INVALID_FIELDID;
  }
  switch (watch_type) {
  case JVMDI_EVENT_FIELD_ACCESS:
    if (value) {
      // make sure we haven't set this watch before
      if (fd.is_field_access_watched()) return JVMDI_ERROR_DUPLICATE;
    } else {
      // make sure we have a watch to clear
      if (!fd.is_field_access_watched()) return JVMDI_ERROR_NOT_FOUND;
    }
    fd.set_is_field_access_watched(value);
    break;

  case JVMDI_EVENT_FIELD_MODIFICATION:
    if (value) {
      // make sure we haven't set this watch before
      if (fd.is_field_modification_watched()) return JVMDI_ERROR_DUPLICATE;
    } else {
      // make sure we have a watch to clear
      if (!fd.is_field_modification_watched()) return JVMDI_ERROR_NOT_FOUND;
    }
    fd.set_is_field_modification_watched(value);
    break;

  default:
    assert(false, "invalid watch_type");
    return JVMDI_ERROR_INTERNAL;
  }

  // update the access_flags for the field in the klass
  instanceKlass* ik = instanceKlass::cast(k);
  typeArrayOop fields = ik->fields();
  fields->ushort_at_put(fd.index(), (jushort)fd.access_flags().as_short());

  return (JVMDI_ERROR_NONE);
}


JVMDI_ENTER(jvmdiError, SetFieldAccessWatch, (jclass clazz, 
                                              jfieldID field))
  jvmdiError result = field_watch_common(clazz, field,
    JVMDI_EVENT_FIELD_ACCESS, true);

  if (result == JVMDI_ERROR_NONE) jvmdi::inc_field_access_count();

  JVMDI_RETURN(result);
JVMDI_END

JVMDI_ENTER(jvmdiError, ClearFieldAccessWatch, (jclass clazz, 
                                                jfieldID field))
  jvmdiError result = field_watch_common(clazz, field,
    JVMDI_EVENT_FIELD_ACCESS, false);

  if (result == JVMDI_ERROR_NONE) jvmdi::dec_field_access_count();

  JVMDI_RETURN(result);
JVMDI_END

JVMDI_ENTER(jvmdiError, SetFieldModificationWatch, (jclass clazz, 
                                                    jfieldID field))
  jvmdiError result = field_watch_common(clazz, field,
    JVMDI_EVENT_FIELD_MODIFICATION, true);

  if (result == JVMDI_ERROR_NONE) jvmdi::inc_field_modification_count();

  JVMDI_RETURN(result);
JVMDI_END

JVMDI_ENTER(jvmdiError, ClearFieldModificationWatch, (jclass clazz, 
                                                      jfieldID field))
  jvmdiError result = field_watch_common(clazz, field,
    JVMDI_EVENT_FIELD_MODIFICATION, false);

  if (result == JVMDI_ERROR_NONE) jvmdi::dec_field_modification_count();

  JVMDI_RETURN(result);
JVMDI_END


//////////////////////////////////////////////////////////////////////////////

//
// External Utility functions.
//

bool jvmdi::is_valid_event_type(jint event_type) {
  if ((event_type < 0) || (event_type > JVMDI_MAX_EVENT_TYPE_VAL)) {
    return false;
  }
  return true;
}

// State so VM knows whether it should post events or not.
bool              jvmdi::_bytecode_stepping       = false;
bool              jvmdi::_enabled                 = false;
int               jvmdi::_field_access_count      = 0;
int               jvmdi::_field_modification_count= 0;
bool              jvmdi::_method_entry_on         = false;
//
// Since JVMDI_EVENT_FRAME_POP is enabled by default, the method_exit_on
// flag should also be on by default. However, we only care about frame
// pops if NotifyFramePop() is called.
//
bool              jvmdi::_method_exit_on          = false;
int               jvmdi::_notice_safepoints_depth = 0;

void jvmdi::initialize() {
 JvmdiCurrentBreakpoints::initialize();
 JvmdiEventDispatcher::initialize();
}

void jvmdi::destroy() {
  disable();
  JvmdiCurrentBreakpoints::destroy();
  JvmdiEventDispatcher::destroy();
}

void jvmdi::enable() {
  jvmdi::initialize();
  _enabled = true;
}

void jvmdi::disable() { 
// XXX - If we do this, then trigger an "invalid method ordering length"
// assertion. This needs to be fixed.
//
//  _enabled = false; 
}

void jvmdi::set_bytecode_stepping(bool on) {
  // nothing to do or JVM/DI is not enabled
  if (on == _bytecode_stepping || !enabled()) return;

  _bytecode_stepping = on;

  //
  // notice_safepoints() and ignore_safepoints() support the
  // concept of nested uses of the "notice safepoints" table.
  // The other user of these methods is the regular safepoint
  // logic. However, we should always be first notice_safepoints()
  // call and the last ignore_safepoints() call.
  // 
  if (on) {
    assert(notice_safepoints_depth() == 0, "should not be nested before call");
    AbstractInterpreter::notice_safepoints();    
  } else {
    AbstractInterpreter::ignore_safepoints();    
    assert(notice_safepoints_depth() == 0, "should not be nested after call");
  }
}


//
// Private function to interpret changes to JVMDI_EVENT_FRAME_POP,
// JVMDI_EVENT_METHOD_ENTRY, and/or JVMDI_EVENT_METHOD_EXIT handling.
// The method_entry_on flag is used to enable or disable handling for
// the JVMDI_EVENT_METHOD_ENTRY event. Similarly the method_exit_on
// flag is used for JVMDI_EVENT_FRAME_POP or JVMDI_EVENT_METHOD_EXIT
// events. If the appropriate event is enabled globally or for any
// thread, then the corresponding flag is set to true. If the event
// is disabled globally *and* for all the threads, then the flag is
// set to false.
//
static void set_method_event_flags(JavaThread *thread, jint event_type, bool enabled) {
  assert(event_type == JVMDI_EVENT_METHOD_ENTRY ||
	 event_type == JVMDI_EVENT_METHOD_EXIT ||
	 event_type == JVMDI_EVENT_FRAME_POP, "Expected METHOD_ENTRY, METHOD_EXIT or FRAME_POP event");

  bool result = false;

  if (enabled) {
    // we enabled an event so just get to setting the flags
    result = enabled;
  } else {
    // We have disabled the event either globally or for the target
    // thread. Disabling at the global level is really a misnomer
    // because it really means disabling the event for all threads
    // don't explicitly have the event enabled. If another thread
    // has the event enabled, then we can't turn the related flag
    // off.

    // lock Threads_lock
    MutexLocker mu(Threads_lock);
    JavaThread *java_thread = Threads::first();
    while (java_thread != NULL) {
      JvmdiThreadState *thread_state = java_thread->jvmdi_thread_state();
      if (thread_state != NULL) {
        if (event_type == JVMDI_EVENT_METHOD_ENTRY) {
          // we still keep method_entry on if METHOD_ENTRY event is enabled either globally or in any threads.
          result = JvmdiEventDispatcher::is_enabled(java_thread, JVMDI_EVENT_METHOD_ENTRY);
        } else {
          // we still keep method_exit on if either METHOD_EXIT or FRAME_POP event is enabled either globally or in any threads.
          result = JvmdiEventDispatcher::is_enabled(java_thread, JVMDI_EVENT_METHOD_EXIT) || 
                   JvmdiEventDispatcher::is_enabled(java_thread, JVMDI_EVENT_FRAME_POP);
        }
        if (result == true) break; // we just need to know if one thread has event enabled
      }    
      java_thread = java_thread->next();
    }
  } // unlock Threads_lock

  // we use no synchronization here, so it is possible that we don't notify events
  // for sometime even though they are enabled
  switch (event_type) {
  case JVMDI_EVENT_METHOD_ENTRY: 
    jvmdi::set_method_entry_on(result);
    break;
  case JVMDI_EVENT_METHOD_EXIT:
  case JVMDI_EVENT_FRAME_POP: 
    jvmdi::set_method_exit_on(result);
    break;
  }
}


// Private function to see if we can turn off the method_exit_on flag.
static void reset_method_exit_on() {
  assert(!JvmdiEventDispatcher::is_enabled(JVMDI_EVENT_METHOD_EXIT), "METHOD_EXIT events must be disabled to reset");

  bool result = false;	// let's be optimistic

  {
    // lock Threads_lock
    MutexLocker mu(Threads_lock);
    JavaThread *java_thread = Threads::first();
    while (java_thread != NULL) {
      JvmdiThreadState *thread_state = java_thread->jvmdi_thread_state();
      if (thread_state != NULL) {
	JvmdiFramePops& fps = thread_state->get_jvmdi_frame_pops();
	if (fps.length() > 0) {
	  result = true;
	  break; // we just need to know if one thread has pending frame pops
	}
      }    
      java_thread = java_thread->next();
    }
  } // unlock Threads_lock
  jvmdi::set_method_exit_on(result);
}


////////////////////////////////////////////////////////////////////////////////////////////////


//
// JVM/DI field access management
//

void jvmdi::dec_field_access_count() {
  // use a lock to keep the count accurate
  MutexLocker mu(JVMDI_field_access_lock);
  if (_field_access_count > 0) _field_access_count--;
}


// interpreter generator needs the address of the counter
int jvmdi::get_field_access_count_addr() {
  // We don't grab JVMDI_field_access_lock because we don't want to
  // serialize field access between all threads. This means that a
  // thread on another processor can see the wrong count value and
  // may either miss making a needed call into post_field_access()
  // or will make an unneeded call into post_field_access(). We pay
  // this price to avoid slowing down the VM when we aren't watching
  // field accesses.
  return (int)(&_field_access_count);
}


void jvmdi::inc_field_access_count() {
  // use a lock to keep the count accurate
  MutexLocker mu(JVMDI_field_access_lock);
  _field_access_count++;
}


//
// JVM/DI field modification management
//

void jvmdi::dec_field_modification_count() {
  // use a lock to keep the count accurate
  MutexLocker mu(JVMDI_field_modification_lock);
  if (_field_modification_count > 0) _field_modification_count--;
}


// interpreter generator needs the address of the counter
int jvmdi::get_field_modification_count_addr() {
  // We don't grab JVMDI_field_modification_lock because we don't
  // want to serialize field modification between all threads. This
  // means that a thread on another processor can see the wrong
  // count value and may either miss making a needed call into
  // post_field_modification() or will make an unneeded call into
  // post_field_modification(). We pay this price to avoid slowing
  // down the VM when we aren't watching field modifications.
  return (int)(&_field_modification_count);
}


void jvmdi::inc_field_modification_count() {
  // use a lock to keep the count accurate
  MutexLocker mu(JVMDI_field_modification_lock);
  _field_modification_count++;
}


//
// JVM/DI single step management
//
void jvmdi::at_single_stepping_point(JavaThread *thread, methodOop method, address location) {
  assert(jvmdi::is_bytecode_stepping(), "must be bytecode stepping");
  // update information about current location and post a step event
  JvmdiThreadState *state = thread->jvmdi_thread_state();
  if (!state->hide_single_stepping()) {
    state->compare_and_set_current_location(method, location, JVMDI_EVENT_SINGLE_STEP);
    if (!state->single_stepping_posted()) {
      jvmdi::post_single_step_event(thread, method, location);
    }
  }
  // If we are using slow breakpoints and we are at the right spot, then
  // post a breakpoint.
  if (!UseFastBreakpoints && !state->breakpoint_posted() &&
	JvmdiCurrentBreakpoints::is_breakpoint(location)) {
    jvmdi::post_breakpoint_event(thread, method, location);
  }
}


void jvmdi::expose_single_stepping(JavaThread *thread) {
  if (!JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_SINGLE_STEP)) { return; }
  JvmdiThreadState *state = thread->jvmdi_thread_state();
  assert(state, "missing JVM/DI thread state");
  state->clear_hide_single_stepping();
}


void jvmdi::hide_single_stepping(JavaThread *thread) {
  if (!JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_SINGLE_STEP)) { return; }
  JvmdiThreadState *state = thread->jvmdi_thread_state();
  assert(state, "missing JVM/DI thread state");
  state->set_hide_single_stepping();
}


//
// JVMDI events that the VM posts to the debugger
//

void jvmdi::post_vm_initialized_event() {
  JvmdiInternal::jvmdi_init();

  { ResourceMark rm;
    JvmdiEventVm e(JVMDI_EVENT_VM_INIT);
    JvmdiInternal::post_event(e);
  }

  JvmdiInternal::jvmdi_init_done();
}

void jvmdi::post_vm_death_event() {
  { ResourceMark rm;
    JvmdiEventVm e(JVMDI_EVENT_VM_DEATH);
    JvmdiInternal::post_event(e);
  }

  JvmdiInternal::jvmdi_death();
}

void jvmdi::post_class_load_event(JavaThread *thread, klassOop klass) {
  { 
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    KlassHandle  klass_handle(klass);
  
    if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_CLASS_LOAD)) { 
      JvmdiWalkableMark wm(thread);
      JvmdiEventClass e(JVMDI_EVENT_CLASS_LOAD,thread_handle,klass_handle);
      JvmdiInternal::post_event(e);
    }
  }
}

void jvmdi::post_class_prepare_event(JavaThread *thread, klassOop klass) {
  { 
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    KlassHandle  klass_handle(klass);
  
    if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_CLASS_PREPARE)) { 
      JvmdiWalkableMark wm(thread);
      JvmdiEventClass e(JVMDI_EVENT_CLASS_PREPARE,thread_handle,klass_handle);
      JvmdiInternal::post_event(e);
    }
  }
}

void jvmdi::post_class_unload_event(klassOop klass) {
  if (JvmdiEventDispatcher::is_enabled(JVMDI_EVENT_CLASS_UNLOAD)) {
    Thread *thread = Thread::current();
    assert(thread->is_VM_thread(), "wrong thread");

    // get JavaThread for whom we are proxy
    JavaThread *jthread =
        (JavaThread *)((VMThread *)thread)->vm_operation()->calling_thread();

    HandleMark  hm();
    Handle      thread_handle(jthread->threadObj());
    KlassHandle klass_handle(klass);

    JvmdiEventClassUnload e(thread_handle, klass_handle);
    JvmdiInternal::post_proxy_event(e, jthread);
  }
}


void jvmdi::post_thread_start_event(JavaThread *thread) {
  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_THREAD_START)) {

    HandleMark   hm;
    Handle       thread_handle(thread->threadObj());

    JvmdiEventThreadChange e(JVMDI_EVENT_THREAD_START, thread_handle);
    JvmdiInternal::post_event(e);
  }
}

void jvmdi::post_thread_end_event(JavaThread *thread) {
  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_THREAD_END)) {

    HandleMark   hm;
    Handle       thread_handle(thread->threadObj());

    JvmdiEventThreadChange e(JVMDI_EVENT_THREAD_END, thread_handle);
    JvmdiInternal::post_event(e);
  }
}

void jvmdi::post_method_entry_event(JavaThread *thread, methodOop method, frame current_frame) {
  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_METHOD_ENTRY)) {
    
    JvmdiWalkableMark wm(thread);
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    methodHandle method_handle(method);
    KlassHandle  klass_handle(method->method_holder());
 
    JvmdiEventFrame e(JVMDI_EVENT_METHOD_ENTRY, thread_handle, klass_handle, method_handle, wm.current_frame());
    JvmdiInternal::post_event(e);
  }
}

void jvmdi::post_method_exit_event(JavaThread *thread, methodOop method, frame current_frame) {
  HandleMark   hm(thread);
  Handle       thread_handle(thread->threadObj());
  methodHandle method_handle(method);
  KlassHandle  klass_handle(method->method_holder());

  bool method_exit_enabled = JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_METHOD_EXIT);
  JvmdiThreadState *state = thread->jvmdi_thread_state();

  if (method_exit_enabled) {
    // post method exit event only when a method terminates not by throwing an exception
    // i.e. if an exception is thrown, it's already caught by the current method
    if (!state->is_exception_detected() || state->is_exception_caught()) {
      JvmdiWalkableMark wm(thread);
      JvmdiEventFrame e(JVMDI_EVENT_METHOD_EXIT, thread_handle, klass_handle, method_handle, wm.current_frame());
      JvmdiInternal::post_event(e);
    }
  }

  if (thread->jvmdi_thread_state()->is_frame_pop((address)current_frame.fp())) {
    // we have a NotifyFramePop entry for this frame
    if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_FRAME_POP)) {
      // we also need to issue a frame pop event for this frame
      JvmdiWalkableMark wm(thread);
      JvmdiEventFrame e(JVMDI_EVENT_FRAME_POP, thread_handle, klass_handle, method_handle, wm.current_frame());
      JvmdiInternal::post_event(e);
    }
    // remove the frame's entry
    JvmdiFramePops& fps = thread->jvmdi_thread_state()->get_jvmdi_frame_pops();
    fps.clear((address) current_frame.fp());
    // reset method_exit_on flag only if METHOD_EXIT event is not enabled
    if (!method_exit_enabled && fps.length() == 0) {
      // We have run out of frame pops for this thread so see if we
      // can turn off the method_exit_on flag to make things quicker.
      reset_method_exit_on();
    }
  }
}

void jvmdi::post_breakpoint_event(JavaThread *thread, methodOop method, address location) {
  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_BREAKPOINT)) {

    JvmdiWalkableMark wm(thread);
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    methodHandle method_handle(method);
    KlassHandle  klass_handle(method->method_holder());

    JvmdiEventBreakpoint e(thread_handle,klass_handle,method_handle,location);
    JvmdiInternal::post_event(e);      

    JvmdiThreadState *state = thread->jvmdi_thread_state();
    state->set_breakpoint_posted();
  }
}

// Todo: inline this for optimization
void jvmdi::post_single_step_event(JavaThread *thread, methodOop method, address location) {
  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_SINGLE_STEP)) {
    JvmdiWalkableMark wm(thread);
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    methodHandle method_handle(method);
    KlassHandle  klass_handle(method->method_holder());

    JvmdiEventSingleStep e(thread_handle,klass_handle,method_handle,location);
    JvmdiInternal::post_event(e);

    JvmdiThreadState *state = thread->jvmdi_thread_state();
    state->set_single_stepping_posted();
  }
}

void jvmdi::post_exception_throw_event(JavaThread *thread, methodOop method, address location, oop exception) {
  JvmdiThreadState *state = thread->jvmdi_thread_state();
  
  if (!state->is_exception_detected()) {
    state->set_exception_detected();

    if ((JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_EXCEPTION)) &&
	(exception != NULL)) {

      ResourceMark rm;
      JvmdiWalkableMark wm(thread);
      HandleMark   hm(thread);
      Handle       thread_handle(thread->threadObj());
      methodHandle method_handle(method);
      KlassHandle  klass_handle(method->method_holder());
      Handle       exception_handle(exception);
      methodHandle catch_method_handle(methodOop(NULL));
      address      catch_location = NULL;
      KlassHandle  catch_klass_handle;
    
      // Fix for 4307310; see also interpreterRuntime.cpp. Got rid of
      // JavaThread::current_catcher_for_exception since we needed
      // more visibility into the algorithm.

      // It's okay to clear these exceptions here because we duplicate
      // this lookup in InterpreterRuntime::exception_handler_for_exception.
      EXCEPTION_MARK;

      bool should_repeat;
      vframeStream st(thread);
      assert(!st.at_end(), "cannot be at end");
      methodOop current_method = NULL;
      int current_bci = -1;
      do {
	current_method = st.method();
	current_bci = st.bci();
	do {
	  should_repeat = false;
	  current_bci = current_method->fast_exception_handler_bci_for(exception_handle()->klass(), current_bci, true, THREAD);
	  if (HAS_PENDING_EXCEPTION) {
	    exception_handle = Handle(PENDING_EXCEPTION);
	    CLEAR_PENDING_EXCEPTION;
	    should_repeat = true;
	  }
	} while (should_repeat && (current_bci != -1));
	st.next();
      } while ((current_bci < 0) && (!st.at_end()));

      if (current_bci < 0) {
	catch_method_handle = methodHandle(methodOop(NULL));
	catch_location = NULL;
      } else {
	catch_method_handle = methodHandle(current_method);
	catch_location = current_method->code_base() + current_bci;
      }
      
      catch_klass_handle = (catch_method_handle.is_null() ? 
			    KlassHandle(klassOop(NULL)) : KlassHandle(catch_method_handle->method_holder()));
        
      JvmdiEventException e(thread_handle, klass_handle, method_handle, location, exception_handle,
			    catch_klass_handle, catch_method_handle, catch_location);
      JvmdiInternal::post_event(e);
    }
  }
}

void jvmdi::post_exception_catch_event(JavaThread *thread, methodOop method, address location, oop exception) {
  JvmdiThreadState *state = thread->jvmdi_thread_state();
  
  if (state->is_exception_detected()) {
    JvmdiFramePops& fps = state->get_jvmdi_frame_pops();
    
    // location is NULL when an exception is forwarded to its caller; 
    // As specified in the JVMDI spec, no frame pop event or method exit event will be generated if 
    // a method terminates by throwing an exception.
    if (location == NULL) {
      address fp = (address) thread->last_frame().fp();
      assert(!state->is_exception_caught(), "exception must not be caught yet.");
      if (state->is_frame_pop(fp)) {
        // clear framepop of the current frame
        fps.clear(fp);
      }
      return;
    }
          
    state->set_exception_caught();

    // clear pending framepops upto this frame.
    frame lastframe = thread->last_frame();
    JvmdiFramePop lastframepop(&lastframe);
    fps.clearTo(lastframepop);
    
    if ((JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_EXCEPTION_CATCH)) &&
	(exception != NULL)) {

      JvmdiWalkableMark wm(thread);
      HandleMark   hm(thread);
      Handle       thread_handle(thread->threadObj());
      methodHandle method_handle(method);
      KlassHandle  klass_handle(method->method_holder());
      Handle       exception_handle(exception);
      
      JvmdiEventExceptionCatch e(thread_handle, klass_handle, method_handle, location, exception_handle);
      JvmdiInternal::post_event(e);      
    }
  }
}

oop jvmdi::jni_GetField_probe(
  JavaThread *thread,
  jobject jobj,
  oop obj,
  klassOop klass,
  jfieldID fieldID,
  bool is_static
) {
  if (*((int *)get_field_access_count_addr()) > 0 && thread->has_last_Java_frame()) {
    // At least one field access watch is set so we have more work
    // to do. This wrapper is used by entry points that allow us
    // to create handles in post_field_access_by_jni().
    post_field_access_by_jni(thread, obj, klass, fieldID, is_static);
    // post_event() can block so refetch oop if we were passed a jobj
    if (jobj != NULL) return JNIHandles::resolve_non_null(jobj);
  }
  return obj;
}

oop jvmdi::jni_GetField_probe_nh(
  JavaThread *thread,
  jobject jobj,
  oop obj,
  klassOop klass,
  jfieldID fieldID,
  bool is_static
) {
  if (*((int *)get_field_access_count_addr()) > 0 && thread->has_last_Java_frame()) {
    // At least one field access watch is set so we have more work
    // to do. This wrapper is used by "quick" entry points that don't
    // allow us to create handles in post_field_access_by_jni(). We
    // override that with a ResetNoHandleMark.
    ResetNoHandleMark rnhm;
    post_field_access_by_jni(thread, obj, klass, fieldID, is_static);
    // post_event() can block so refetch oop if we were passed a jobj
    if (jobj != NULL) return JNIHandles::resolve_non_null(jobj);
  }
  return obj;
}

void jvmdi::post_field_access_by_jni(
  JavaThread *thread,
  oop obj,
  klassOop klass,
  jfieldID fieldID,
  bool is_static
) {
  // We must be called with a Java context in order to provide reasonable
  // values for the klazz, method, and location fields. The callers of this
  // function don't make the call unless there is a Java context.
  assert(thread->has_last_Java_frame(), "must be called with a Java context");

  ResourceMark rm;
  fieldDescriptor fd;
  // if get_field_descriptor finds fieldID to be invalid, then we just bail
  if (!JvmdiInternal::get_field_descriptor(klass, fieldID, &fd)) return;
  // field accesses are not watched so bail
  if (!fd.is_field_access_watched()) return;

  HandleMark hm(thread);
  if (is_static) {
    // static field accessors have no object
    KlassHandle h_klass(klass);
    post_field_access_event(thread, thread->last_frame().interpreter_frame_method(),
      thread->last_frame().interpreter_frame_bcp(), h_klass, NULL, fieldID);
  } else {
    // non-static field accessors have an object, but we need a handle
    assert(obj != NULL, "non-static needs an object");
    KlassHandle h_klass(klass);
    Handle h_obj(thread, obj);
    post_field_access_event(thread, thread->last_frame().interpreter_frame_method(),
      thread->last_frame().interpreter_frame_bcp(), h_klass, h_obj, fieldID);
  }
}

void jvmdi::post_field_access_event(JavaThread *thread, methodOop method,
  address location, KlassHandle field_klass, Handle object, jfieldID field) {

  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_FIELD_ACCESS)) {
    ResourceMark rm;
    JvmdiWalkableMark wm(thread);
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    methodHandle method_handle(method);
    KlassHandle  klass_handle(method->method_holder());

    JvmdiEventFieldAccess e(thread_handle, klass_handle, method_handle,
      location, field_klass, object, field);
    JvmdiInternal::post_event(e);
  }
}

oop jvmdi::jni_SetField_probe(
  JavaThread *thread,
  jobject jobj,
  oop obj,
  klassOop klass,
  jfieldID fieldID,
  bool is_static,
  char sig_type,
  jvalue *value
) {
  if (*((int *)get_field_modification_count_addr()) > 0 && thread->has_last_Java_frame()) {
    // At least one field modification watch is set so we have more work
    // to do. This wrapper is used by entry points that allow us
    // to create handles in post_field_modification_by_jni().
    post_field_modification_by_jni(thread, obj, klass, fieldID, is_static, sig_type, value);
    // post_event() can block so refetch oop if we were passed a jobj
    if (jobj != NULL) return JNIHandles::resolve_non_null(jobj);
  }
  return obj;
}

oop jvmdi::jni_SetField_probe_nh(
  JavaThread *thread,
  jobject jobj,
  oop obj,
  klassOop klass,
  jfieldID fieldID,
  bool is_static,
  char sig_type,
  jvalue *value
) {
  if (*((int *)get_field_modification_count_addr()) > 0 && thread->has_last_Java_frame()) {
    // At least one field modification watch is set so we have more work
    // to do. This wrapper is used by "quick" entry points that don't
    // allow us to create handles in post_field_modification_by_jni(). We
    // override that with a ResetNoHandleMark.
    ResetNoHandleMark rnhm;
    post_field_modification_by_jni(thread, obj, klass, fieldID, is_static, sig_type, value);
    // post_event() can block so refetch oop if we were passed a jobj
    if (jobj != NULL) return JNIHandles::resolve_non_null(jobj);
  }
  return obj;
}

void jvmdi::post_field_modification_by_jni(
  JavaThread *thread,
  oop obj,
  klassOop klass,
  jfieldID fieldID,
  bool is_static,
  char sig_type,
  jvalue *value
) {
  // We must be called with a Java context in order to provide reasonable
  // values for the klazz, method, and location fields. The callers of this
  // function don't make the call unless there is a Java context.
  assert(thread->has_last_Java_frame(), "must be called with Java context");

  ResourceMark rm;
  fieldDescriptor fd;
  // if get_field_descriptor finds fieldID to be invalid, then we just bail
  if (!JvmdiInternal::get_field_descriptor(klass, fieldID, &fd)) return;
  // field modifications are not watched so bail
  if (!fd.is_field_modification_watched()) return;

  HandleMark hm(thread);
  if (is_static) {
    // static field accessors have no object
    KlassHandle h_klass(klass);
    post_field_modification_event(thread, thread->last_frame().interpreter_frame_method(),
      thread->last_frame().interpreter_frame_bcp(), h_klass, NULL, fieldID, sig_type, value);
  } else {
    // non-static field accessors have an object, but we need a handle
    assert(obj != NULL, "non-static needs an object");
    KlassHandle h_klass(klass);
    Handle h_obj(thread, obj);
    post_field_modification_event(thread, thread->last_frame().interpreter_frame_method(),
      thread->last_frame().interpreter_frame_bcp(), h_klass, h_obj, fieldID, sig_type, value);
  }
}

void jvmdi::post_field_modification_event(JavaThread *thread, methodOop method,
  address location, KlassHandle field_klass, Handle object, jfieldID field,
  char sig_type, jvalue *value) {

  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_FIELD_MODIFICATION)) {
    ResourceMark rm;
    JvmdiWalkableMark wm(thread);
    HandleMark   hm(thread);
    Handle       thread_handle(thread->threadObj());
    methodHandle method_handle(method);
    KlassHandle  klass_handle(method->method_holder());

    JvmdiEventFieldModification e(thread_handle, klass_handle, method_handle,
      location, field_klass, object, field, sig_type, value);
    JvmdiInternal::post_event(e);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////

void jvmdi::initialize_thread(JavaThread *thread) {
  thread->set_jvmdi_thread_state(new JvmdiThreadState(thread));
}


void jvmdi::cleanup_thread(JavaThread* thread) {
  assert(JavaThread::current() == thread, "thread is not current");

  if (JvmdiEventDispatcher::is_enabled(thread, JVMDI_EVENT_SINGLE_STEP)) {
    //
    // Turn off single stepping for this thread. If this is the last thread
    // that was single stepping, then bytecode stepping is turned off by
    // set_enabled().
    //
    JvmdiEventDispatcher::set_enabled(thread, JVMDI_EVENT_SINGLE_STEP, false);
  }

  JvmdiWalkableThread *wt = JvmdiWalkableThreads::get_thread(thread);
  if (wt != NULL) {
    bool was_suspend = wt->is_suspend();	// were we added by suspend?
    JvmdiWalkableThreads::remove_current();
    // if we weren't added by suspend, then there is a problem
    assert(was_suspend, "current thread should not be walkable when exiting");
  }

  if (thread->jvmdi_thread_state()) {
    delete thread->jvmdi_thread_state();
    thread->set_jvmdi_thread_state(NULL);
  }
}

void jvmdi::oops_do(void f(oop *)) {
  JvmdiCurrentBreakpoints::oops_do(f);
}

JVMDI_Interface_1* jvmdi::GetInterface_1(JavaVM *interfaces_vm) {
  JVMDI_Interface_1 *jvmdi_interface = JvmdiInternal::get_jvmdi_interface();

  if (jvmdi_interface->SetEventHook == NULL) {
    JvmdiInternal::set_vm(interfaces_vm);

    // Pick up interfaces local to other files
    JvmdiInternal::set_info_hooks(jvmdi_interface);
    JvmdiInternal::set_info2_hooks(jvmdi_interface);

    jvmdi_interface->SetEventHook = SetEventHook;
    jvmdi_interface->SetEventNotificationMode = SetEventNotificationMode;
    jvmdi_interface->GetThreadStatus = GetThreadStatus;
    jvmdi_interface->GetAllThreads = GetAllThreads;
    jvmdi_interface->SuspendThread = hSuspendThread;
    jvmdi_interface->ResumeThread = hResumeThread;
    jvmdi_interface->StopThread = StopThread;
    jvmdi_interface->InterruptThread = InterruptThread;
    jvmdi_interface->GetThreadInfo = GetThreadInfo;
    jvmdi_interface->GetOwnedMonitorInfo = GetOwnedMonitorInfo;
    jvmdi_interface->GetCurrentContendedMonitor = GetCurrentContendedMonitor;
    jvmdi_interface->RunDebugThread = RunDebugThread;
    jvmdi_interface->CreateRawMonitor = CreateRawMonitor;
    jvmdi_interface->DestroyRawMonitor = DestroyRawMonitor;
    jvmdi_interface->RawMonitorEnter = RawMonitorEnter;
    jvmdi_interface->RawMonitorExit = RawMonitorExit;
    jvmdi_interface->RawMonitorWait = RawMonitorWait;
    jvmdi_interface->RawMonitorNotify = RawMonitorNotify;
    jvmdi_interface->RawMonitorNotifyAll = RawMonitorNotifyAll;
    jvmdi_interface->GetTopThreadGroups = GetTopThreadGroups;
    jvmdi_interface->GetThreadGroupInfo = GetThreadGroupInfo;
    jvmdi_interface->GetThreadGroupChildren = GetThreadGroupChildren;
    jvmdi_interface->GetFrameCount = GetFrameCount;
    jvmdi_interface->GetCurrentFrame = GetCurrentFrame;
    jvmdi_interface->GetCallerFrame = GetCallerFrame;
    jvmdi_interface->GetFrameLocation = GetFrameLocation;
    jvmdi_interface->NotifyFramePop = NotifyFramePop;
    jvmdi_interface->GetLocalObject = GetLocalObject;
    jvmdi_interface->GetLocalInt = GetLocalInt;
    jvmdi_interface->GetLocalLong = GetLocalLong;
    jvmdi_interface->GetLocalFloat = GetLocalFloat;
    jvmdi_interface->GetLocalDouble = GetLocalDouble;
    jvmdi_interface->SetLocalObject = SetLocalObject;
    jvmdi_interface->SetLocalInt = SetLocalInt;
    jvmdi_interface->SetLocalLong = SetLocalLong;
    jvmdi_interface->SetLocalFloat = SetLocalFloat;
    jvmdi_interface->SetLocalDouble = SetLocalDouble;
    jvmdi_interface->SetBreakpoint = SetBreakpoint;
    jvmdi_interface->ClearBreakpoint = ClearBreakpoint;
    jvmdi_interface->ClearAllBreakpoints = ClearAllBreakpoints;
    jvmdi_interface->SetFieldAccessWatch = SetFieldAccessWatch;
    jvmdi_interface->ClearFieldAccessWatch = ClearFieldAccessWatch;
    jvmdi_interface->SetFieldModificationWatch = SetFieldModificationWatch;
    jvmdi_interface->ClearFieldModificationWatch = ClearFieldModificationWatch;
    jvmdi_interface->SetAllocationHooks = SetAllocationHooks;
    jvmdi_interface->Allocate = Allocate;
    jvmdi_interface->Deallocate = Deallocate;
  }

  return jvmdi_interface;
}
