// @(#)jvmpi.cpp	1.96 01/03/21 16:41:56
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_jvmpi.cpp.incl"


//-------------------------------------------------------

// Unsolved problems:
//
// CPU profiling
// - we need an exact mapping of pc/bci to lineNo; this is not
//   always possible as:
//    - interpreted has not set up the frame completely
//    - compiled code is not at a safepoint

//-------------------------------------------------------

unsigned int jvmpi::_event_flags = JVMPI_PROFILING_OFF;
unsigned int jvmpi::_event_flags_array[JVMPI_MAX_EVENT_TYPE_VAL + 1];
JVMPI_Interface jvmpi::jvmpi_interface;
bool jvmpi::slow_allocation = false;


void jvmpi::initialize(int version) {
  // Enable JVMPI
  _event_flags |= JVMPI_PROFILING_ON;

  // First, initialize all JVMPI defined event notifications 
  // to be not available
  for (int i= 0; i <= JVMPI_MAX_EVENT_TYPE_VAL; i++) {
    _event_flags_array[i] = JVMPI_EVENT_NOT_SUPPORTED;
  }

  // Then, initialize events supported by the HotSpot VM
  // to be initially disabled.
  disable_event(JVMPI_EVENT_CLASS_LOAD);
  disable_event(JVMPI_EVENT_CLASS_UNLOAD);
  disable_event(JVMPI_EVENT_CLASS_LOAD_HOOK);
  disable_event(JVMPI_EVENT_OBJECT_ALLOC);
  disable_event(JVMPI_EVENT_OBJECT_FREE);
                                // supported in JVMPI_VERSION_1_1 and beyond
  if (version != JVMPI_VERSION_1) {
    disable_event(JVMPI_EVENT_INSTRUCTION_START);
  }
  disable_event(JVMPI_EVENT_THREAD_START);
  disable_event(JVMPI_EVENT_THREAD_END);
  disable_event(JVMPI_EVENT_JNI_GLOBALREF_ALLOC);
  disable_event(JVMPI_EVENT_JNI_GLOBALREF_FREE);
  disable_event(JVMPI_EVENT_JNI_WEAK_GLOBALREF_ALLOC);
  disable_event(JVMPI_EVENT_JNI_WEAK_GLOBALREF_FREE);
  disable_event(JVMPI_EVENT_METHOD_ENTRY);
  disable_event(JVMPI_EVENT_METHOD_ENTRY2);
  disable_event(JVMPI_EVENT_METHOD_EXIT);
  disable_event(JVMPI_EVENT_LOAD_COMPILED_METHOD);
  disable_event(JVMPI_EVENT_UNLOAD_COMPILED_METHOD);
  disable_event(JVMPI_EVENT_JVM_INIT_DONE);
  disable_event(JVMPI_EVENT_JVM_SHUT_DOWN);
  disable_event(JVMPI_EVENT_DUMP_DATA_REQUEST);
  disable_event(JVMPI_EVENT_RESET_DATA_REQUEST);
  disable_event(JVMPI_EVENT_OBJECT_MOVE);
  disable_event(JVMPI_EVENT_ARENA_NEW);
  disable_event(JVMPI_EVENT_DELETE_ARENA);
  disable_event(JVMPI_EVENT_RAW_MONITOR_CONTENDED_ENTER);
  disable_event(JVMPI_EVENT_RAW_MONITOR_CONTENDED_ENTERED);
  disable_event(JVMPI_EVENT_RAW_MONITOR_CONTENDED_EXIT);
  disable_event(JVMPI_EVENT_MONITOR_CONTENDED_ENTER);
  disable_event(JVMPI_EVENT_MONITOR_CONTENDED_ENTERED);
  disable_event(JVMPI_EVENT_MONITOR_CONTENDED_EXIT);
  disable_event(JVMPI_EVENT_MONITOR_WAIT);
  disable_event(JVMPI_EVENT_MONITOR_WAITED);
  disable_event(JVMPI_EVENT_GC_START);
  disable_event(JVMPI_EVENT_GC_FINISH);

  // set up the jvmpi_interface version the caller requested
  jvmpi_interface.version                 = version;
  
  // initialize the jvmpi_interface functions
  jvmpi_interface.EnableEvent             = &enable_event;
  jvmpi_interface.DisableEvent            = &disable_event;
  
  jvmpi_interface.RequestEvent            = &request_event;
  jvmpi_interface.GetCallTrace            = &get_call_trace;
  jvmpi_interface.GetCurrentThreadCpuTime = &get_current_thread_cpu_time;
  jvmpi_interface.ProfilerExit            = &profiler_exit;
  jvmpi_interface.RawMonitorCreate        = &raw_monitor_create;
  jvmpi_interface.RawMonitorEnter         = &raw_monitor_enter;
  jvmpi_interface.RawMonitorExit          = &raw_monitor_exit;
  jvmpi_interface.RawMonitorWait          = &raw_monitor_wait;
  jvmpi_interface.RawMonitorNotifyAll     = &raw_monitor_notify_all;
  jvmpi_interface.RawMonitorDestroy       = &raw_monitor_destroy;
  jvmpi_interface.SuspendThread           = &suspend_thread;
  jvmpi_interface.ResumeThread            = &resume_thread;
  jvmpi_interface.GetThreadStatus         = &get_thread_status;
  jvmpi_interface.ThreadHasRun            = &thread_has_run;
  jvmpi_interface.CreateSystemThread      = &create_system_thread;
  jvmpi_interface.SetThreadLocalStorage   = &set_thread_local_storage;
  jvmpi_interface.GetThreadLocalStorage   = &get_thread_local_storage;
  
  jvmpi_interface.DisableGC               = &disable_gc;
  jvmpi_interface.EnableGC                = &enable_gc;
  
  jvmpi_interface.RunGC                   = &run_gc;
  jvmpi_interface.GetThreadObject         = &get_thread_object;
  jvmpi_interface.GetMethodClass          = &get_method_class;

  if (version == JVMPI_VERSION_1) {
    jvmpi_interface.jobjectID2jobject     = NULL;
    jvmpi_interface.jobject2jobjectID     = NULL;
  } else {                                // JVMPI_VERSION_1_1 and beyond
    jvmpi_interface.jobjectID2jobject     = &jobjectID_2_jobject;
    jvmpi_interface.jobject2jobjectID     = &jobject_2_jobjectID;
  }
}


JVMPI_Interface* jvmpi::GetInterface_1(int version) {
  initialize(version);
  return &jvmpi_interface;
}


inline void jvmpi::post_event(JVMPI_Event* event) {
  Thread* thread = Thread::current();
  assert(thread->is_Java_thread(), "expecting a Java thread");

  JavaThread* jthread = (JavaThread*)thread;
  event->env_id = jthread->jni_environment();
  // prepare to call out across JVMPI
  ThreadToNativeFromVM transition(jthread);
  HandleMark  hm(thread);  
  // notify profiler agent
  jvmpi_interface.NotifyEvent(event);
}

// JVMPI 2.0: should cleanup race condition where calling_thread
// exits before being notified.
inline void jvmpi::post_event_vm_mode(JVMPI_Event* event, JavaThread* calling_thread) {
  Thread* thread = Thread::current();
  if (thread->is_Java_thread()) {
    JavaThread* jthread = (JavaThread*)thread;
    event->env_id = jthread->jni_environment();
    // prepare to call out across JVMPI
    JavaThreadState saved_state = jthread->thread_state();
    if (saved_state == _thread_in_Java) {
        ThreadStateTransition::transition_from_java(jthread, _thread_in_native);
    } else {
        ThreadStateTransition::transition(jthread, saved_state, _thread_in_native);
    }
    HandleMark  hm(thread);  
    // notify profiler agent
    jvmpi_interface.NotifyEvent(event);
    // restore state prior to posting event
        ThreadStateTransition::transition(jthread, _thread_in_native, saved_state); 
  } else {
    if (thread->is_VM_thread()) {
      // calling from VM thread

      if (calling_thread == NULL) {
	  calling_thread = JavaThread::active();
      }	  
	    
      assert(calling_thread != NULL && calling_thread->is_Java_thread(),
	     "wrong thread, expecting Java thread");

      event->env_id = (calling_thread != NULL && 
		       calling_thread->is_Java_thread()) ?
		       calling_thread->jni_environment() : NULL;
    } else {
      event->env_id = calling_thread->jni_environment();
    }
  // notify profiler agent
  jvmpi_interface.NotifyEvent(event);
  }
  
}


// ----------------------------------------------------------
// Functions called by other parts of the VM to notify events
// ----------------------------------------------------------

void issue_jvmpi_class_load_event(klassOop k) {
  jvmpi::post_class_load_event(Klass::cast(k)->java_mirror());
}


class IssueJVMPIobjAllocEvent: public ObjectClosure {
 public:
  void do_object(oop obj) {
    Universe::jvmpi_object_alloc(obj, obj->size() * wordSize);
  };
};


class VM_JVMPIPostObjAlloc: public VM_Operation {
 private:
  int   _flag;
 public:
  VM_JVMPIPostObjAlloc(int flag) { _flag = flag; }
  void doit() {
    // Issue object allocation events for all allocated objects
    IssueJVMPIobjAllocEvent blk;

    // Ensure Eden top is correct before iteration
    Universe::new_gen()->eden()->ensure_proper_top();
    if (UseTLE) {
      Universe::fill_all_tles();	 
    }
    Universe::object_iterate(&blk);
  }
  const char* name() const { return "post JVMPI object allocation"; }
};


void jvmpi::post_vm_initialization_events() {
  if (Universe::jvmpi_alloc_event_enabled()) {
    // Issue the object allocation events thru a VM operation since
    // it needs to be done at a safepoint
    VM_JVMPIPostObjAlloc op(0);
    VMThread::execute(&op);
  }

  if (jvmpi::is_event_enabled(JVMPI_EVENT_CLASS_LOAD)) {
    // Issue class load events for all loaded classes
    // Note: This must happen _after_ the allocation events, otherwise hprof has problems!
    SystemDictionary::classes_do(&issue_jvmpi_class_load_event);
  }

  if (jvmpi::is_event_enabled(JVMPI_EVENT_THREAD_START)) {
    // Issue thread creation events for all started threads
    int k = 0;
    int threadcount;
    JavaThread** ThreadSnapShot;
    { MutexLocker mu(Threads_lock);
      threadcount = Threads::number_of_threads();
      ThreadSnapShot = NEW_C_HEAP_ARRAY(JavaThread*, threadcount);
      for (JavaThread* tp = Threads::first() ; (tp != NULL) && ( k < threadcount); tp = tp->next(), k++) {
        ThreadSnapShot[k] = tp;
      }
    } // Release Threads_lock before calling up to agent code
    for (k = 0; k<threadcount; k++) {
      jvmpi::post_thread_start_event(ThreadSnapShot[k]);
    }
    FREE_C_HEAP_ARRAY(JavaThread*, ThreadSnapShot);
  }
}


void jvmpi::post_vm_initialized_event() {
  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_JVM_INIT_DONE;
  post_event(&event);
}

void jvmpi::post_vm_death_event() {
  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_JVM_SHUT_DOWN;
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_instruction_start_event(const frame& f) {
  ResourceMark rm;
  JVMPI_Event event;

  methodOop method = f.interpreter_frame_method();
  address   bcp    = f.interpreter_frame_bcp();

  // fill in generic information
  event.event_type = JVMPI_EVENT_INSTRUCTION_START;
  event.u.instruction.method_id = (jmethodID)method->jni_id();
  event.u.instruction.offset    = method->bci_from(bcp);

  // debugging
#ifdef ASSERT
  switch (Bytecodes::java_code(Bytecodes::cast(*bcp))) {
    case Bytecodes::_tableswitch : // fall through
    case Bytecodes::_lookupswitch: // fall through
    case Bytecodes::_ifnull      : // fall through
    case Bytecodes::_ifeq        : // fall through
    case Bytecodes::_ifnonnull   : // fall through
    case Bytecodes::_ifne        : // fall through
    case Bytecodes::_iflt        : // fall through
    case Bytecodes::_ifge        : // fall through
    case Bytecodes::_ifgt        : // fall through
    case Bytecodes::_ifle        : assert(f.interpreter_frame_expression_stack_size() >= 1, "stack size must be >= 1"); break;
    case Bytecodes::_if_acmpeq   : // fall through
    case Bytecodes::_if_icmpeq   : // fall through
    case Bytecodes::_if_acmpne   : // fall through
    case Bytecodes::_if_icmpne   : // fall through
    case Bytecodes::_if_icmplt   : // fall through
    case Bytecodes::_if_icmpge   : // fall through
    case Bytecodes::_if_icmpgt   : // fall through
    case Bytecodes::_if_icmple   : assert(f.interpreter_frame_expression_stack_size() >= 2, "stack size must be >= 2"); break;
  }
#endif

  // fill in bytecode-specific information
  //
  // Note: This code is necessary to satisfy the current interface for the jcov
  //       code coverage tool. The interface should be simplified and generalized
  //       to provide expression stack access instead of specific information for
  //       a few bytecodes only. Given expression stack access, the code below
  //       can move into jcov, the interface becomes simpler, more general, and
  //       also more powerful. With the next version/revision of JVMPI this clean
  //       up should be seriously considered (gri 11/18/99).

  int  size  = f.interpreter_frame_expression_stack_size();
  jint tos_0 = size > 0 ? f.interpreter_frame_expression_stack_at(size - 1) : 0;
  jint tos_1 = size > 1 ? f.interpreter_frame_expression_stack_at(size - 2) : 0;

  switch (Bytecodes::java_code(Bytecodes::cast(*bcp))) {
    case Bytecodes::_tableswitch :
      { const Bytecode_tableswitch* s = Bytecode_tableswitch_at(bcp);
        event.u.instruction.u.tableswitch_info.key = tos_0;
        event.u.instruction.u.tableswitch_info.low = s->low_key();
        event.u.instruction.u.tableswitch_info.hi  = s->high_key();
      }
      break;
    case Bytecodes::_lookupswitch:
      { Bytecode_lookupswitch* s = Bytecode_lookupswitch_at(bcp);
        int i;
        for (i = 0; i < s->number_of_pairs() && tos_0 != s->pair_at(i)->match(); i++);
        event.u.instruction.u.lookupswitch_info.chosen_pair_index = i;
        event.u.instruction.u.lookupswitch_info.pairs_total       = s->number_of_pairs();
      }
      break;
    case Bytecodes::_ifnull      : // fall through
    case Bytecodes::_ifeq        : event.u.instruction.u.if_info.is_true = tos_0 == 0; break;
    case Bytecodes::_ifnonnull   : // fall through
    case Bytecodes::_ifne        : event.u.instruction.u.if_info.is_true = tos_0 != 0; break;
    case Bytecodes::_iflt        : event.u.instruction.u.if_info.is_true = tos_0 <  0; break;
    case Bytecodes::_ifge        : event.u.instruction.u.if_info.is_true = tos_0 >= 0; break;
    case Bytecodes::_ifgt        : event.u.instruction.u.if_info.is_true = tos_0 >  0; break;
    case Bytecodes::_ifle        : event.u.instruction.u.if_info.is_true = tos_0 <= 0; break;
    case Bytecodes::_if_acmpeq   : // fall through
    case Bytecodes::_if_icmpeq   : event.u.instruction.u.if_info.is_true = tos_1 == tos_0; break;
    case Bytecodes::_if_acmpne   : // fall through
    case Bytecodes::_if_icmpne   : event.u.instruction.u.if_info.is_true = tos_1 != tos_0; break;
    case Bytecodes::_if_icmplt   : event.u.instruction.u.if_info.is_true = tos_1 <  tos_0; break;
    case Bytecodes::_if_icmpge   : event.u.instruction.u.if_info.is_true = tos_1 >= tos_0; break;
    case Bytecodes::_if_icmpgt   : event.u.instruction.u.if_info.is_true = tos_1 >  tos_0; break;
    case Bytecodes::_if_icmple   : event.u.instruction.u.if_info.is_true = tos_1 <= tos_0; break;
  }
  
  post_event(&event);
}

void jvmpi::post_thread_start_event(JavaThread* thread, jint flag)
{
  ResourceMark rm;
  JVMPI_Event event;
  
  assert(!Threads_lock->owned_by_self(), "must not own threads_lock for notify");

  { MutexLocker mu(Threads_lock);
    event.event_type = JVMPI_EVENT_THREAD_START | flag;
  
    event.u.thread_start.thread_name   = (char*)thread->get_thread_name();
    event.u.thread_start.group_name    = (char*)thread->get_threadgroup_name();
    event.u.thread_start.parent_name   = (char*)thread->get_parent_name();
    event.u.thread_start.thread_id     = (jobjectID)thread->threadObj();
    event.u.thread_start.thread_env_id = thread->jni_environment();
  } // Release Threads_lock
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: post_thread_start_event for thread id 0x%x [thread 0x%x <%s>] ", event.u.thread_start.thread_id, thread, event.u.thread_start.thread_name);
  }
  
  GC_locker::lock();
  post_event_vm_mode(&event, NULL);
  GC_locker::unlock();
}

void jvmpi::post_thread_start_event(JavaThread* thread) {
  post_thread_start_event(thread, 0);
}

void jvmpi::post_thread_end_event(JavaThread* thread) {
  ResourceMark rm;
  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_THREAD_END;

  { MutexLocker mu(Threads_lock);
    event.u.thread_start.thread_name   = (char* )thread->get_thread_name();
    event.u.thread_start.group_name    = (char*)thread->get_threadgroup_name();
    event.u.thread_start.parent_name   = (char* )thread->get_parent_name();
    event.u.thread_start.thread_id     = (jobjectID)thread->threadObj();
    event.u.thread_start.thread_env_id = thread->jni_environment();
  } // Release Threads_lock

  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: post_thread_end_event for thread id 0x%x [thread 0x%x <%s>] ", event.u.thread_start.thread_id, thread, event.u.thread_start.thread_name);
  }
  post_event(&event);
}


// Note: kOop must be mirror
void jvmpi::fillin_class_load_event(oop kOop, JVMPI_Event *eventp,
bool fillin_jni_ids) {
  eventp->event_type = JVMPI_EVENT_CLASS_LOAD;
  klassOop k = java_lang_Class::as_klassOop(kOop);
  instanceKlass* ik = instanceKlass::cast(k);
  // get field info
  int num_statics = 0;
  int num_instances = 0;
  for (FieldStream count_field_st(k, true, true); !count_field_st.eos(); count_field_st.next()) {
    if (count_field_st.access_flags().is_static()) {
      num_statics++;
    } else {
      num_instances++;
    }
  }
  JVMPI_Field* statics = NEW_RESOURCE_ARRAY(JVMPI_Field, num_statics);
  JVMPI_Field* instances = NEW_RESOURCE_ARRAY(JVMPI_Field, num_instances);
  int i_stat = 0;
  int i_inst = 0;
  for (FieldStream field_st(k, true, true); !field_st.eos(); field_st.next()) {
    char* f_name = field_st.name     ()->as_C_string();
    char* f_sig  = field_st.signature()->as_C_string();
    if (field_st.access_flags().is_static()) {
      statics[i_stat].field_name      = f_name;
      statics[i_stat].field_signature = f_sig;
      i_stat++;
    } else {
      instances[i_inst].field_name      = f_name;
      instances[i_inst].field_signature = f_sig;
      i_inst++;
    }
  }
  assert(i_inst == num_instances, "sanity check");
  assert(i_stat == num_statics, "sanity check");
  // get method info
  int num_methods = ik->methods()->length();
  JVMPI_Method* methods = NEW_RESOURCE_ARRAY(JVMPI_Method, num_methods);
  int i_meth = 0;
  for (MethodStream meth_st(k, true, true); !meth_st.eos(); meth_st.next()) {
    methodOop m = meth_st.method();
    methods[i_meth].method_name      = m->name()->as_C_string();
    methods[i_meth].method_signature = m->signature()->as_C_string();
    if (fillin_jni_ids) {
      methods[i_meth].method_id      = (jmethodID) m->jni_id();
    } else {
      // method_id doesn't mean much after class is unloaded
      methods[i_meth].method_id      = NULL;
    }
    methods[i_meth].start_lineno     = m->line_number_from_bci(0);
    if (m->code_size() > 0) {
      methods[i_meth].end_lineno     = m->line_number_from_bci(m->code_size() - 1);
    } else {
      methods[i_meth].end_lineno     = m->line_number_from_bci(0);
    }
    i_meth++;
  }

  eventp->u.class_load.class_name          = ik->external_name();
  if (ik->source_file_name() == NULL)
    eventp->u.class_load.source_name       = NULL;
  else
    eventp->u.class_load.source_name       = ik->source_file_name()->as_C_string();
  eventp->u.class_load.num_interfaces      = ik->local_interfaces()->length();
  eventp->u.class_load.num_methods         = num_methods;
  eventp->u.class_load.methods             = methods;
  eventp->u.class_load.num_static_fields   = num_statics;
  eventp->u.class_load.statics             = statics;
  eventp->u.class_load.num_instance_fields = num_instances;
  eventp->u.class_load.instances           = instances;
  eventp->u.class_load.class_id            = (jobjectID)ik->java_mirror();
}


// List of classes unloaded for the duration of the CLASS_UNLOAD event
// handler. Populated by save_class_unload_event_info(), queried by both
// post_class_load_event() and post_class_unload_events(), and cleaned
// up by post_class_unload_events().
static GrowableArray<JVMPI_Event*>* unloaded_classes = NULL;

// Note: kOop must be mirror
void jvmpi::post_class_load_event(oop kOop, jint flag) {

  if (flag == JVMPI_REQUESTED_EVENT && unloaded_classes != NULL) {
    // This is a synthesized event and we are in the middle of unloading
    // classes so see if the requested class is one that we unloaded.

    // walk the list of unloaded class event information
    for (int i = 0; i < unloaded_classes->length(); i++) {
      JVMPI_Event *ev = unloaded_classes->at(i);
      if ((oop)(ev->u.class_load.class_id) == kOop) {
        // We are in the event handler for CLASS_UNLOAD event so
        // we don't have to lock out GC. Post the saved event
        // information for the unloaded class to the agent.
        assert(GC_locker::is_active(), "GC must be locked when in event handler");
        post_event_vm_mode(ev, NULL);
        return;
      }
    }
  }

  ResourceMark rm;
  JVMPI_Event event;
  fillin_class_load_event(kOop, &event, true /* fillin JNI ids */);
  event.event_type |= flag;
  if (TraceJVMPI) {
    tty->print("JVMPI: post_class_load_event for klass mirror 0x%x ", kOop);
    java_lang_Class::as_klassOop(kOop)->print_value();
    tty->print(" ");
    kOop->print_value();
    tty->cr();
  }

  GC_locker::lock();
  post_event_vm_mode(&event, NULL);
  GC_locker::unlock();
}


void jvmpi::post_class_load_event(oop k) {
  post_class_load_event(k, 0);
}


void jvmpi::post_class_load_hook_event(unsigned char **ptrP, unsigned char **end_ptrP,
			                 void * (* malloc_f)(unsigned int))
{
  JVMPI_Event event;

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_CLASS_LOAD_HOOK;

  event.u.class_load_hook.class_data = *ptrP;
  event.u.class_load_hook.class_data_len = *end_ptrP - *ptrP;
  event.u.class_load_hook.malloc_f = malloc_f;

  post_event(&event);
    
  *ptrP = event.u.class_load_hook.new_class_data;
  *end_ptrP = *ptrP + event.u.class_load_hook.new_class_data_len;
}


// Post CLASS_UNLOAD events and/or release saved memory.
void jvmpi::post_class_unload_events() {
  if (unloaded_classes != NULL) {  // we unloaded some classes
    // walk the list of unloaded class event information
    for (int i = 0; i < unloaded_classes->length(); i++) {
      JVMPI_Event *ev = unloaded_classes->at(i);  // shorthand for saved info

      if (jvmpi::is_event_enabled(JVMPI_EVENT_CLASS_UNLOAD)) {
        // The caller is still interested in the events so post them.
        // Note: by the time we get called, the caller may no longer
        // be interested in the events, but we have to always free
        // the memory below.
        JVMPI_Event event;

        GC_locker::lock();

        // construct a CLASS_UNLOAD event from the saved into
        event.event_type = JVMPI_EVENT_CLASS_UNLOAD;
        event.u.class_unload.class_id = ev->u.class_load.class_id;

        post_event_vm_mode(&event, NULL);

        GC_locker::unlock();
      }
      delete ev;  // done with the saved info
    }

    delete unloaded_classes;
    unloaded_classes = NULL;
  }
}


// GC has caused a class to be unloaded so save CLASS_LOAD information
// just in case there is a RequestEvent(CLASS_LOAD) call from the
// CLASS_UNLOAD event handler.
void jvmpi::save_class_unload_event_info(oop k) {
  JVMPI_Event *ev = new JVMPI_Event();
  fillin_class_load_event(k, ev, false /* don't fillin JNI id values */);
  ev->event_type |= JVMPI_REQUESTED_EVENT;

  if (unloaded_classes == NULL) {
    // first unloaded class so setup initial space for the events
    unloaded_classes = new GrowableArray<JVMPI_Event*>(5);
  }
  unloaded_classes->append(ev);
}


void jvmpi::post_dump_event() {
  if (is_event_enabled(JVMPI_EVENT_DUMP_DATA_REQUEST)) {
    JVMPI_Event event;
    event.event_type = JVMPI_EVENT_DUMP_DATA_REQUEST;
    post_event(&event);
  }
  if (is_event_enabled(JVMPI_EVENT_RESET_DATA_REQUEST)) {
    JVMPI_Event event;
    event.event_type = JVMPI_EVENT_RESET_DATA_REQUEST;
    post_event(&event);
  }
}


void jvmpi::post_new_globalref_event(jobject ref, oop obj) {
  JVMPI_Event event;
    
  GC_locker::lock();

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_JNI_GLOBALREF_ALLOC;

  event.u.jni_globalref_alloc.obj_id = (jobjectID)(obj);
  event.u.jni_globalref_alloc.ref_id = ref;

  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_delete_globalref_event(jobject ref) {
  JVMPI_Event event;
    
  GC_locker::lock();

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_JNI_GLOBALREF_FREE;

  event.u.jni_globalref_free.ref_id = ref;

  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_new_weakref_event(jobject ref, oop obj) {
  JVMPI_Event event;
    
  GC_locker::lock();

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_JNI_WEAK_GLOBALREF_ALLOC;

  event.u.jni_globalref_alloc.obj_id = (jobjectID)(obj);
  event.u.jni_globalref_alloc.ref_id = ref;

  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_delete_weakref_event(jobject ref) {
  JVMPI_Event event;
    
  GC_locker::lock();

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_JNI_WEAK_GLOBALREF_FREE;

  event.u.jni_globalref_free.ref_id = ref;

  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_arena_new_event(int arena_id, const char* arena_name) {
  if (!is_event_enabled(JVMPI_EVENT_ARENA_NEW)) return;
  JVMPI_Event event;

  event.event_type = JVMPI_EVENT_ARENA_NEW;
  event.u.new_arena.arena_id = arena_id;
  event.u.new_arena.arena_name = arena_name;
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_arena_delete_event(int arena_id) {
  JVMPI_Event event;

  event.event_type = JVMPI_EVENT_DELETE_ARENA;
  event.u.delete_arena.arena_id = arena_id;
  post_event_vm_mode(&event, NULL);
}

/* post_object_alloc_event requires size to be in bytes */
void jvmpi::post_object_alloc_event(oop obj, int bytesize, jint arena_id, jint flag) {
  // do not emit the event if the allocation event is not enabled, except if it is
  // requested
  if (!is_event_enabled(JVMPI_EVENT_OBJECT_ALLOC) && flag != JVMPI_REQUESTED_EVENT) return;
  // bailout if obj is undefined
  if (obj == NULL) return;
  // bootstraping problem: Issue object allocation event for the java/lang/Class 
  // mirror with class set to NULL (to avoid infinite recursion).
  bool bootstrap = (obj == Klass::cast(SystemDictionary::class_klass())->java_mirror());
  // determine klass & is_array info
  oop klass;
  int is_array;
  if (bootstrap) {
    klass    = NULL;
    is_array = JVMPI_NORMAL_OBJECT;
  } else if (obj->is_instance()) {
    klass    = Klass::cast(obj->klass())->java_mirror();
    is_array = JVMPI_NORMAL_OBJECT;
  } else if (obj->is_objArray()) {
    klass    = Klass::cast(objArrayKlass::cast(obj->klass())->element_klass())->java_mirror();
    is_array = JVMPI_CLASS;
  } else if (obj->is_typeArray()) {
    klass    = NULL;
    is_array = typeArrayKlass::cast(obj->klass())->type();
  } else {
    klass    = JVMPI_INVALID_CLASS;
    is_array = JVMPI_NORMAL_OBJECT;
  }    
  // post event if ok
  if (klass != JVMPI_INVALID_CLASS) {
    if (!flag) GC_locker::lock();
    /* fill event info and notify the profiler */
    { JVMPI_Event event;
      event.event_type           = JVMPI_EVENT_OBJECT_ALLOC | flag;
      event.u.obj_alloc.arena_id = arena_id;
      event.u.obj_alloc.class_id = (jobjectID)klass;
      event.u.obj_alloc.is_array = is_array;
      event.u.obj_alloc.size     = bytesize;
      event.u.obj_alloc.obj_id   = (jobjectID)obj;
      if (TraceJVMPI) {
        tty->print_cr("JVMPI: post_object_alloc_event for object 0x%x ", obj);
      }
      post_event_vm_mode(&event, NULL);
    }
    if (!flag) GC_locker::unlock();
  }
}


void jvmpi::post_object_free_event(oop obj) {
  JVMPI_Event event;
    
  assert(MarkSweep::is_active(), "Only mark-sweep collections issue free events");

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_OBJECT_FREE;

  event.u.obj_free.obj_id = (jobjectID)obj;
    
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_object_move_event(oop oldobj, int old_arena_id, oop newobj, int new_arena_id) {
  JVMPI_Event event;
    
  assert(Scavenge::is_active() || MarkSweep::is_active(), "Should only move objects during GC");

  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_OBJECT_MOVE;

  event.u.obj_move.obj_id       = (jobjectID)oldobj;
  event.u.obj_move.arena_id     = old_arena_id;
  event.u.obj_move.new_obj_id   = (jobjectID)newobj;
  event.u.obj_move.new_arena_id = new_arena_id;

  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_method_entry2_event(methodOop m, oop receiver) {
  JVMPI_Event event;
  
  event.event_type = JVMPI_EVENT_METHOD_ENTRY2;
  event.u.method_entry2.method_id = (jmethodID)m->jni_id();
  event.u.method_entry2.obj_id = (jobjectID)receiver;
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_method_entry_event(methodOop m) {
  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_METHOD_ENTRY;
  event.u.method.method_id = (jmethodID)m->jni_id();
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_method_exit_event(methodOop m) {
  JVMPI_Event event;
  
  event.event_type = JVMPI_EVENT_METHOD_EXIT;
  event.u.method.method_id = (jmethodID)m->jni_id();
  post_event_vm_mode(&event, NULL);
}


// use  compiled_method_t so that the line number table can be constructed only
// temporarily and then released after post_compiled_method_load_event terminates
void jvmpi::post_compiled_method_load_event(compiled_method_t *compiled_method_info) {
  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_COMPILED_METHOD_LOAD;
  event.u.compiled_method_load.method_id         = (jmethodID)compiled_method_info->method->jni_id();
  event.u.compiled_method_load.code_addr         = compiled_method_info->code_addr;
  event.u.compiled_method_load.code_size         = compiled_method_info->code_size;
  event.u.compiled_method_load.lineno_table_size = compiled_method_info->lineno_table_len;
  event.u.compiled_method_load.lineno_table      = compiled_method_info->lineno_table;
}


void jvmpi::post_compiled_method_unload_event(methodOop method) {
  Unimplemented();
}


void jvmpi::post_monitor_contended_enter_event(void* object) {
  GC_locker::lock();

  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_MONITOR_CONTENDED_ENTER;
  event.u.monitor.object = (jobjectID)object;
  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_monitor_contended_entered_event(void* object) {
  GC_locker::lock();

  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_MONITOR_CONTENDED_ENTERED;
  event.u.monitor.object = (jobjectID)object;
  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_monitor_contended_exit_event(void* object) {
  GC_locker::lock();

  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_MONITOR_CONTENDED_EXIT;
  event.u.monitor.object = (jobjectID)object;
  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_monitor_wait_event(oop obj, jlong millis) {
  GC_locker::lock();

  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_MONITOR_WAIT;
  event.u.monitor_wait.object  = (jobjectID)obj;
  event.u.monitor_wait.timeout = millis;
  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_monitor_waited_event(oop obj, jlong millis) {
  GC_locker::lock();

  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_MONITOR_WAITED;
  event.u.monitor_wait.object  = (jobjectID)obj;
  event.u.monitor_wait.timeout = millis;
  post_event_vm_mode(&event, NULL);

  GC_locker::unlock();
}


void jvmpi::post_raw_monitor_contended_enter_event(RawMonitor* rmon) {
  Thread* tp = Thread::current();
  if (Thread::current()->is_VM_thread()) return;
  JVMPI_Event event;

  event.event_type = JVMPI_EVENT_RAW_MONITOR_CONTENDED_ENTER;
  event.u.raw_monitor.name = rmon->name();
  event.u.raw_monitor.id = (JVMPI_RawMonitor)rmon;
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_raw_monitor_contended_entered_event(RawMonitor* rmon) {
  if (Thread::current()->is_VM_thread()) return;
  JVMPI_Event event;

  event.event_type = JVMPI_EVENT_RAW_MONITOR_CONTENDED_ENTERED;
  event.u.raw_monitor.name = rmon->name();
  event.u.raw_monitor.id = (JVMPI_RawMonitor)rmon;
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_raw_monitor_contended_exit_event(RawMonitor* rmon) {
  if (Thread::current()->is_VM_thread()) return;
  JVMPI_Event event;

  event.event_type = JVMPI_EVENT_RAW_MONITOR_CONTENDED_EXIT;
  event.u.raw_monitor.name = rmon->name();
  event.u.raw_monitor.id = (JVMPI_RawMonitor)rmon;
  post_event_vm_mode(&event, NULL);
}


void jvmpi::post_gc_start_event() {
  JVMPI_Event event;
  assert(Thread::current()->is_VM_thread(), "wrong thread");

  Thread* calling_thread = JavaThread::active();
  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_GC_START;

  assert(calling_thread->is_Java_thread(), "wrong thread");
  post_event_vm_mode(&event, (JavaThread*)calling_thread);
}


class CountObjects: public ObjectClosure {
 private:
  int _nof_objects;
 public:
  CountObjects(): _nof_objects(0) {}

  void do_object(oop obj) { _nof_objects++;  };

  int nof_objects() const { return _nof_objects; }
};


void jvmpi::post_gc_finish_event(long used_obj_space, long total_obj_space) {
  JVMPI_Event event;
  assert(Thread::current()->is_VM_thread(), "wrong thread");
  long used_objs = 0;
  // compute number of used objects
  { // Note: this is slow and cumbersome
    CountObjects blk;

    // Although the calls to ensure_proper_top and fill_all_tles are redundant
    // due to this code running at the end of a GC, it has been placed here
    // for completeness and because this code has been moved around recently
    // and might get moved again to where it might not follow a prior
    // correction.
    Universe::new_gen()->eden()->ensure_proper_top();
    if (UseTLE) {
      Universe::fill_all_tles();	 
    }
    Universe::object_iterate(&blk);
    used_objs = blk.nof_objects();
  }
  Thread* calling_thread = JavaThread::active();
  /* fill event info and notify the profiler */
  event.event_type = JVMPI_EVENT_GC_FINISH;
  event.u.gc_info.used_objects       = used_objs;
  event.u.gc_info.used_object_space  = used_obj_space;
  event.u.gc_info.total_object_space = total_obj_space;

  assert(calling_thread->is_Java_thread(), "wrong thread");
  post_event_vm_mode(&event, (JavaThread*)calling_thread);
}


void jvmpi::post_trace_instr_event(unsigned char *pc, unsigned char opcode) {
  Unimplemented();
}


void jvmpi::post_trace_if_event(unsigned char *pc, int is_true) {
  Unimplemented();
}


void jvmpi::post_trace_tableswitch_event(unsigned char *pc, int key, int low, int hi) {
  Unimplemented();
}


void jvmpi::post_trace_lookupswitch_event(unsigned char *pc, int chosen_pair_index, int pairs_total) {
  Unimplemented();
}


// heap dumps

// Dump is a helper class for all kinds of dumps that require
// a buffer to hold the dump. 

class Dump: public StackObj {
 private:
  address _begin;                      // the beginning of the dump space, NULL if no space was allocated
  address _end;                        // the current dump position
  address _limit;                      // the limit of the dump space (debugging only)

  void init(int dump_size) {
    assert(dump_size <= 0 || _begin == NULL, "dump buffer already allocated");
    _begin = dump_size > 0 ? NEW_C_HEAP_ARRAY(unsigned char, dump_size) : NULL;
    _end   = _begin;
    _limit = _begin + dump_size;
  }
    
  bool write() const                   { return begin() != NULL; }
  address limit() const                { return _limit; }
  void range_check(int size)           { assert(end() + size <= limit(), "end of dump reached"); }

 public:
  // creation
  Dump()                               { init(0); }
  void enable_write(int dump_size)     { init(dump_size); }
  ~Dump()                              { if (write()) FREE_C_HEAP_ARRAY(unsigned char, begin()); }

  // accessors
  address begin() const                { return _begin; }
  address end() const                  { return _end; }
  int size() const                     { return end() - begin(); }

  // primitive dumpers
  void dump_u1(u1 x)                   { if (write()) { range_check(1); *_end = x;                   } _end += 1; }
  void dump_u2(u2 x)                   { if (write()) { range_check(2); Bytes::put_Java_u2(_end, x); } _end += 2; }
  void dump_u4(u4 x)                   { if (write()) { range_check(4); Bytes::put_Java_u4(_end, x); } _end += 4; }
  void dump_u8(u8 x)                   { if (write()) { range_check(8); Bytes::put_Java_u8(_end, x); } _end += 8; }

  // basic type dumpers
  void dump_bool  (jboolean* x)        { dump_u1(*(u1*)x); }
  void dump_char  (jchar*    x)        { dump_u2(*(u2*)x); }
  void dump_float (jfloat*   x)        { dump_u4(*(u4*)x); }
  void dump_double(jdouble*  x)        { dump_u8(*(u8*)x); }
  void dump_byte  (jbyte*    x)        { dump_u1(*(u1*)x); }
  void dump_short (jshort*   x)        { dump_u2(*(u2*)x); }
  void dump_int   (jint*     x)        { dump_u4(*(u4*)x); }
  void dump_long  (jlong*    x)        { dump_u8(*(u8*)x); }
  void dump_obj   (oop*      x)        { dump_oop(*x); }

  // other dumpers
  //
  // Note: jobjectID (oops) and JNIEnv* are not dumped in Java byte ordering
  //       like all other data types - which is an inconsistency. It should
  //       really be handled like all other data (and mapped to u4 for the
  //       ia32 architecture).
  void dump_oop(oop obj) {
    assert(obj == NULL || (obj->is_oop() && !obj->is_klass()), "not an opp or a klass");
    if (write()) {
      range_check(4);
      Bytes::put_native_u4(_end, (u4)obj);
    }
    _end += 4;
  }

  void dump_thread(JNIEnv* env)        { if (write()) { range_check(4); Bytes::put_native_u4(_end, (u4)env); } _end += 4; }
  void dump_rawmonitor(JVMPI_RawMonitor mon) { if (write()) { range_check(4); Bytes::put_native_u4(_end, (u4)mon); } _end += 4; }
  void dump_char_array(const char* s)        { if (write()) { range_check(4); Bytes::put_native_u4(_end, (u4)s); } _end += 4; }
  void dump_type (int type)            { dump_u1((u1)type); }
  void dump_voids(void* x)             { dump_u4((u4)x); }

  // patching
  void patch_u2(address at, u2 x) {
    if (write()) {
      assert(begin() <= at && at + 2 <= limit(), "patching outside dump space");
      Bytes::put_Java_u2(at, x);
    }
  }

  void patch_u4(address at, u4 x) {
    if (write()) {
      assert(begin() <= at && at + 4 <= limit(), "patching outside dump space");
      Bytes::put_Java_u4(at, x);
    }
  }
};


class FieldDumper: public SignatureIterator {
 private:
  Dump*   _dump;
  address _addr;
  bool    _dump_basic_types;

 public:
  FieldDumper(Dump* dump, int level, symbolHandle signature, address addr)
  : SignatureIterator(signature)
  , _dump(dump)
  , _addr(addr)
  { 
    _dump_basic_types = (level == JVMPI_DUMP_LEVEL_2);
    dispatch_field();
  }

  void do_bool  ()                     { if (_dump_basic_types) _dump->dump_bool  ((jboolean*)_addr); }
  void do_char  ()                     { if (_dump_basic_types) _dump->dump_char  ((jchar*   )_addr); }
  void do_float ()                     { if (_dump_basic_types) _dump->dump_float ((jfloat*  )_addr); }
  void do_double()                     { if (_dump_basic_types) _dump->dump_double((jdouble* )_addr); }
  void do_byte  ()                     { if (_dump_basic_types) _dump->dump_byte  ((jbyte*   )_addr); }
  void do_short ()                     { if (_dump_basic_types) _dump->dump_short ((jshort*  )_addr); }
  void do_int   ()                     { if (_dump_basic_types) _dump->dump_int   ((jint*    )_addr); }
  void do_long  ()                     { if (_dump_basic_types) _dump->dump_long  ((jlong*   )_addr); }
  void do_void  ()                     { ShouldNotReachHere();                                        }
  void do_object(int begin, int end)   {                        _dump->dump_obj   ((oop*     )_addr); }
  void do_array (int begin, int end)   {                        _dump->dump_obj   ((oop*     )_addr); }
};


// The ObjectDumper takes care of any heap object to be dumped.
// Note that non java-level objects are filtered out (such as
// klasses, methodOops, etc.) and that mirrors are converted
// into klasses for the dump.

class ObjectDumper: public StackObj {
 private:
  Dump* _dump;
  int   _level;

  void dump_instance(instanceOop instance) {
    if (_level == JVMPI_DUMP_LEVEL_0) {
      // dump type and id only
      _dump->dump_type(JVMPI_NORMAL_OBJECT);
      _dump->dump_oop(instance);
      return;
    }
    // dump header
    _dump->dump_type(JVMPI_GC_INSTANCE_DUMP);
    _dump->dump_oop(instance);
    _dump->dump_oop(Klass::cast(instance->klass())->java_mirror());
    _dump->dump_u4((u4)0);              // reserve space for no. of bytes - patched at the end
    address field_start = _dump->end(); // remember start of field dump
    // dump instance fields
    // (note: dumping in reverse order since the klass load event dumps
    //        the instance field description in reverse order as well.)
    { for (FieldStream s(instanceKlassHandle(instance->klass()), false, true); !s.eos(); s.next()) {
        // ignore static fields as they are not in the instance
        if (!s.access_flags().is_static()) {
          FieldDumper(_dump, _level, s.signature(), (address)instance + s.offset() * BytesPerWord);
        }
      }
    }
    // patch no. of bytes
    _dump->patch_u4(field_start - 4, _dump->end() - field_start);
  }

  void dump_obj_array(objArrayOop array) {
    // Note: Do not dump system object arrays as they are meaningless
    //       for hprof. Furthermore, they contain klasses which should
    //       never go out w/o extra treatment.
    if (array->klass() != Universe::systemObjArrayKlassObj()) {
      if (_level == JVMPI_DUMP_LEVEL_0) {
        // dump type and id only
        _dump->dump_type(JVMPI_CLASS);
        _dump->dump_oop(array);
        return;
      }
      oop klass = Klass::cast(objArrayKlass::cast(array->klass())->element_klass())->java_mirror();
      const int length = array->length();
      // dump header
      _dump->dump_type(JVMPI_GC_OBJ_ARRAY_DUMP);
      _dump->dump_oop(array);
      _dump->dump_u4(length);
      _dump->dump_oop(klass);
      // dump elements
      for (int i = 0; i < length; i++) _dump->dump_oop(array->obj_at(i));
      // debugging
      if (TraceJVMPI) {
        tty->print("JVMPI: dump @ 0x%x obj array [%d] (klass = 0x%x)", array, length, klass);
        if (Verbose) {
          tty->print(" {");
          for (int i = 0; i < length; i++) {
            if (i > 0) tty->print(", ");
            tty->print("0x%x", array->obj_at(i));
          }
          tty->print("}");
        }
        tty->cr();
      }
    }
  }

  void dump_type_array(typeArrayOop array) {
    const int length = array->length();
    const BasicType type = typeArrayKlass::cast(array->klass())->type();
    int jvmpi_type = -1;
    switch (type) {
      case T_BOOLEAN: jvmpi_type = JVMPI_BOOLEAN; break;
      case T_CHAR   : jvmpi_type = JVMPI_CHAR   ; break;
      case T_FLOAT  : jvmpi_type = JVMPI_FLOAT  ; break;
      case T_DOUBLE : jvmpi_type = JVMPI_DOUBLE ; break;
      case T_BYTE   : jvmpi_type = JVMPI_BYTE   ; break;
      case T_SHORT  : jvmpi_type = JVMPI_SHORT  ; break;
      case T_INT    : jvmpi_type = JVMPI_INT    ; break;
      case T_LONG   : jvmpi_type = JVMPI_LONG   ; break;
      default       : ShouldNotReachHere();
    }
    if (_level == JVMPI_DUMP_LEVEL_0) {
      // dump type and id only
      _dump->dump_type(jvmpi_type);
      _dump->dump_oop(array);
      return;
    }
    // dump header
    _dump->dump_type(JVMPI_GC_PRIM_ARRAY_DUMP);
    _dump->dump_oop(array);
    _dump->dump_u4(length);
    _dump->dump_type(jvmpi_type);
    // dump elements
    if (_level == JVMPI_DUMP_LEVEL_2) {
      switch (type) {
        case T_BOOLEAN: { for (int i = 0; i < length; i++) _dump->dump_bool  (array->bool_at_addr  (i)); } break;
        case T_CHAR   : { for (int i = 0; i < length; i++) _dump->dump_char  (array->char_at_addr  (i)); } break;
        case T_FLOAT  : { for (int i = 0; i < length; i++) _dump->dump_float (array->float_at_addr (i)); } break;
        case T_DOUBLE : { for (int i = 0; i < length; i++) _dump->dump_double(array->double_at_addr(i)); } break;
        case T_BYTE   : { for (int i = 0; i < length; i++) _dump->dump_byte  (array->byte_at_addr  (i)); } break;
        case T_SHORT  : { for (int i = 0; i < length; i++) _dump->dump_short (array->short_at_addr (i)); } break;
        case T_INT    : { for (int i = 0; i < length; i++) _dump->dump_int   (array->int_at_addr   (i)); } break;
        case T_LONG   : { for (int i = 0; i < length; i++) _dump->dump_long  (array->long_at_addr  (i)); } break;
        default       : ShouldNotReachHere();
      }
    }
    // debugging
    if (TraceJVMPI) {
      tty->print_cr("JVMPI: dump @ 0x%x prim array [%d] (type = %d)", array, length, type);
    }
  }

  void dump_klass(klassOop klass) {
    if (Klass::cast(klass)->oop_is_instance()) {
      instanceKlass* k = instanceKlass::cast(klass);
      // Check for level 0 dump
      if (_level == JVMPI_DUMP_LEVEL_0) {
        // dump type and id only
        _dump->dump_type(JVMPI_NORMAL_OBJECT);    // Is this right?
        _dump->dump_oop(k->java_mirror());
        return;
      }
      // dump header
      _dump->dump_type(JVMPI_GC_CLASS_DUMP);
      _dump->dump_oop(k->java_mirror());
      _dump->dump_oop(k->super() == NULL ? NULL : Klass::cast(k->super())->java_mirror());
      _dump->dump_oop(k->class_loader());
      _dump->dump_oop(k->signers());
      _dump->dump_oop(k->protection_domain());
      _dump->dump_oop(StringTable::lookup(k->name())); // NULL if not interned string
      _dump->dump_voids(NULL); // reserved
      _dump->dump_u4(k->size_helper() * BytesPerWord);
      // dump interfaces
      { objArrayOop interfaces = k->local_interfaces();
        for (int i = 0; i < interfaces->length(); i++) {
          oop interf = Klass::cast((klassOop)interfaces->obj_at(i))->java_mirror();
          _dump->dump_oop(interf);
        }
      }
      // dump constant pool
      { address size_loc = _dump->end();    // remember constant pool size location for later patching  
        _dump->dump_u2((u2)0);              // reserve space for constant pool size - patched at the end
        int size = 0;
        const constantPoolOop pool = k->constants();
        for (int i = 1; i < pool->length(); i++) { // index i = 0 is unused!
          address end = _dump->end();
          // for now we ignore all entries
          // eventually we should probably
          // dump at least the oop entries
          /*
          switch (pool->tag_at(i).value()) {
            case JVM_CONSTANT_Class:
            case JVM_CONSTANT_Fieldref:
            ...
          }
          */
          if (end != _dump->end()) size++; // count individual entries
        }
        // patch number of entries
        _dump->patch_u2(size_loc, size);
      }
      // dump static fields
      // (note: dumping in reverse order since the klass load event dumps
      //        the static field description in reverse order as well.)
      {
         instanceKlassHandle kh(klass);
         FieldStream s(kh, true, true);
         for (; !s.eos(); s.next()) { 
           // ignore instance fields as they are not in the klass
           if (s.access_flags().is_static()) {
             FieldDumper(_dump, _level, s.signature(), (address)klass + s.offset() * BytesPerWord);
           }
         }
      }
    } else if (Klass::cast(klass)->oop_is_objArray()) {
      objArrayKlass* k = objArrayKlass::cast(klass);
      // Check for level 0 dump
      if (_level == JVMPI_DUMP_LEVEL_0) {
        // dump type and id only
        _dump->dump_type(JVMPI_NORMAL_OBJECT);    // Is this right?
        _dump->dump_oop(k->java_mirror());
        return;
      }
      // still missing
    }
  }

 public:
  ObjectDumper(Dump* dump, int level, oop obj) : _dump(dump), _level(level) {
    // filter out all klasses
    if (obj->is_klass()) return;
    // convert mirrors
    if (obj->klass() == SystemDictionary::class_klass()) {
      // obj is a mirror - convert into corresponding class if possible
      if (!java_lang_Class::is_primitive(obj)) {
        // obj is not a mirror for a primitive class (basic type)
        // get the corresponding class for dumping
        obj = java_lang_Class::as_klassOop(obj);
        assert(obj != NULL, "class for non-primitive mirror must exist");
      } else {
        // obj is a mirror for a primitice class (basic type)
        // for which we don't have a (VM-level) class => dump
        // mirror as it is.
      }
    }
    // dump object
           if (obj->is_instance ()) { dump_instance  ((instanceOop )obj);
    } else if (obj->is_objArray ()) { dump_obj_array ((objArrayOop )obj);
    } else if (obj->is_typeArray()) { dump_type_array((typeArrayOop)obj);
    } else if (obj->is_klass    ()) { dump_klass     ((klassOop    )obj);
    }
  }
};


class HeapDumper: public ObjectClosure {
 private:
  Dump* _dump;
  int   _level;

 public:
  HeapDumper(Dump* dump, int level) : _dump(dump), _level(level) {
    assert(SafepointSynchronize::is_at_safepoint(), "must be at safepoint");

    // Ensure Eden top is correct before iteration
    Universe::new_gen()->eden()->ensure_proper_top();
    if(UseTLE) {
      Universe::fill_all_tles();
    }
    Universe::object_iterate(this);
  }

  void do_object(oop obj)              { ObjectDumper(_dump, _level, obj); }
};


// Move this in machine specific part !
class MonitorDumper: public StackObj {
 private:
  Dump* _dump;

  void dump_for_thread(ObjectMonitor* mid, JavaThread* thread) {
    assert(SafepointSynchronize::is_at_safepoint(), "must be at safepoint");
    ResourceMark rm;
    // klassOops may be locked (e.g. due to class initialization). Make sure to skip them.
    if (((oop)mid->object())->is_klass()) return;
    //
    // Solaris implements mid->count() differently than Win32 or Linux so
    // we had to create and use the OS specific contentions() function.
    //
    int n_want_lock = mid->contentions();     // number of threads contending for the monitor
    int n_waiters = mid->waiters();
    // this is an unused monitor so skip it
    if (thread == NULL && n_want_lock == 0 && n_waiters == 0) return;
    // dump header
    _dump->dump_type(JVMPI_MONITOR_JAVA);
    _dump->dump_oop((oop)mid->object());
    _dump->dump_thread(thread == NULL ? NULL : thread->jni_environment());
    _dump->dump_u4(n_want_lock + n_waiters); // entry count
    _dump->dump_u4(n_want_lock); // number of threads waiting to enter
    if (n_want_lock > 0) {
      GrowableArray<JavaThread*>* want_list = Threads::get_pending_threads(n_want_lock, (address)mid);
      for (int i = 0; i < n_want_lock; i++) {
        if (i < want_list->length()) {
          JavaThread* jt = want_list->at(i);
          _dump->dump_thread(jt->jni_environment());
        } else {
          _dump->dump_thread(NULL);
        }
      }
    }
    _dump->dump_u4(n_waiters); // number of threads waiting to be notified
    if (n_waiters > 0) {
      ObjectWaiter* waiter = mid->first_waiter();
      for (int i = 0; i < n_waiters; i++) {
//        assert(waiter != NULL, "wrong number of waiters");
// No guarantee this value doesn't change while dumping
	if (waiter != NULL) {
          Thread* thd = mid->thread_of_waiter(waiter);
          if (thd->is_Java_thread()) {
            _dump->dump_thread(((JavaThread*)thd)->jni_environment());
          } else {
            _dump->dump_thread(NULL);
          }
          waiter = mid->next_waiter(waiter);
	} else {
	  _dump->dump_thread(NULL);
	}
      }
    }
  }

 public:
  MonitorDumper(Dump* dump, ObjectMonitor* mid): _dump(dump) {
    // dump Java lock
    dump_for_thread(mid, Threads::owning_thread_from_monitor(mid));
  }
};


class JavaMonitorDumper: public MonitorClosure {
 private:
  Dump* _dump;

 public:
  JavaMonitorDumper(Dump* dump) : _dump(dump) {
    assert(SafepointSynchronize::is_at_safepoint(), "must be at safepoint");
    ObjectSynchronizer::monitors_iterate(this);   // first dump the monitor cache
    if (!UseHeavyMonitors) {	// now dump any lightweight monitors
      ResourceMark rm;
      GrowableArray<ObjectMonitor*>* fab_list = Threads::jvmpi_fab_heavy_monitors();
      for (int i = 0; i < fab_list->length(); i++) {
        ObjectMonitor* fab = fab_list->at(i);
        assert(fab != NULL, "Expected fabricated heavyweight monitor");
        MonitorDumper(_dump, fab);
        // ObjectMonitor is a CHeap object, so remember to free it
        delete fab;
      }
    }
  }
  void do_monitor(ObjectMonitor* mid)  { MonitorDumper(_dump, mid); }
};


class RawMonitorDumper: public StackObj {
 private:
  Dump* _dump;
 public:
  RawMonitorDumper(Dump* dump) : _dump(dump) {
    for(JavaThread *thread = Threads::first(); thread; thread = thread->next()) {
      dump_rawmonitors_for(thread);
    }
  }

  void dump_rawmonitors_for(JavaThread* thread) {
    char* no_name = NULL;
    for (RawMonitor* mon = thread->rawmonitor_list(); mon; mon = mon->next_raw()) {
      assert((PROF_RM_CHECK(mon)), "invalid raw monitor");
      _dump->dump_type(JVMPI_MONITOR_RAW);
      _dump->dump_char_array(mon->name());
      _dump->dump_rawmonitor((JVMPI_RawMonitor) mon);
      _dump->dump_thread(thread->jni_environment());
      dump_monitor_info(mon, thread);
    }
  }

  void dump_monitor_info(RawMonitor* mid, JavaThread* thread)
{
    //
    // Solaris implements mid->count() differently than Win32 or Linux so
    // we had to create and use the OS specific contentions() function.
    //
    int n_want_lock = mid->contentions();     // number of threads contending for the monitor
    int n_waiters = mid->waiters();
    // this is an unused monitor so skip it
    if (thread == NULL && n_want_lock == 0 && n_waiters == 0) return;
    // dump header
    _dump->dump_u4(n_want_lock + n_waiters); // entry count
    _dump->dump_u4(n_want_lock); // number of threads waiting to enter
    if (n_want_lock > 0) {
      GrowableArray<JavaThread*>* want_list = Threads::get_pending_threads(n_want_lock, (address)mid);
      for (int i = 0; i < n_want_lock; i++) {
        if (i < want_list->length()) {
          JavaThread* jt = want_list->at(i);
          _dump->dump_thread(jt->jni_environment());
        } else {
          _dump->dump_thread(NULL);
        }
      }
    }
    _dump->dump_u4(n_waiters); // number of threads waiting to be notified
    if (n_waiters > 0) {
      ObjectWaiter* waiter = mid->first_waiter();
      for (int i = 0; i < n_waiters; i++) {
//        assert(waiter != NULL, "wrong number of waiters");
// no guarantee that this is not changing dynamically
          if (waiter != NULL) {
            Thread* thd = mid->thread_of_waiter(waiter);
            if (thd->is_Java_thread()) {
              _dump->dump_thread(((JavaThread*)thd)->jni_environment());
            } else {
              _dump->dump_thread(NULL);
            }
            waiter = mid->next_waiter(waiter);
          } else {
            _dump->dump_thread(NULL);
          }
      }
    }
  }
};


// The RootCollector collects GC roots in the system.
// Eventually, JVMPI requires more information for stack
// roots, but since the interface may change, we provide
// only a minimal implementation for now.
//
// Note: We cannot use the same mechanism twice to measure
//       the required dump size (as for other dumps) since
//       oops_do introduces new handles which in turn changes
//       the dump size.

class RootCollector: public StackObj {
 private:
  static GrowableArray<oop>* _roots;

  static void f(oop* obj_ptr) {
    oop obj = *obj_ptr;
    if (obj != NULL && (obj->is_instance() || obj->is_array() || obj->is_klass())) {
      if (obj->is_klass()) obj = Klass::cast((klassOop)obj)->java_mirror();
      _roots->append(obj);
    }
  }

  static int cmp(oop* x, oop* y)       { return *x - *y; }

 public:
  RootCollector() {
    // allocate space for collection
    assert(_roots == NULL, "recursive or multithreaded root collection not supported");
    _roots = new GrowableArray<oop>(256,true);
    // collect roots
    { HandleMark hm;
      Threads::oops_do(f);
      ObjectSynchronizer::oops_do(f);
      vmSymbols::oops_do(f);
      JNIHandles::oops_do(f); // global (strong) JNI handles
    }
    // sort roots
    _roots->sort(cmp);
#ifdef ASSERT
    { for (int i = 0; i < _roots->length() - 1; i++) {
        assert(_roots->at(i) <= _roots->at(i+1), "array not sorted correctly");
      }
    }
#endif
    // remove duplicates by compacting array
    { const int len = _roots->length();
      oop obj = NULL; // we don't need NULL roots
      int j = 0;
      for (int i = 0; i < len; i++) {
        assert(i >= j, "algorithmic error");
        if (_roots->at(i) != obj) {
          obj = _roots->at(i);
          _roots->at_put(j++, obj);
        }
      }
      _roots->trunc_to(j);
      assert(number_of_roots() == j, "just checking");
    }
#ifdef ASSERT
    { for (int i = 0; i < _roots->length() - 1; i++) {
        assert(_roots->at(i) != NULL && _roots->at(i) < _roots->at(i+1), "array contents incorrect");
      }
    }
#endif
  }

  ~RootCollector()                     { _roots = NULL; }
  int number_of_roots() const          { return _roots->length(); }
  int root_dump_size() const           { return number_of_roots() * (BytesPerWord + 1); }

  void dump_roots(Dump* dump) const {
    int i = number_of_roots();
    while (i-- > 0) {
      dump->dump_type(JVMPI_GC_ROOT_UNKNOWN);
      dump->dump_oop(_roots->at(i));
    }
  }
};

GrowableArray<oop>* RootCollector::_roots = NULL;


void jvmpi::post_object_dump_event(oop obj, int flag) {
  No_GC_Verifier nogc;
  Dump dump;
  // 1st dump to measure dump size
  { ObjectDumper od(&dump, JVMPI_DUMP_LEVEL_2, obj); }
  // 2nd dump to actually write dump
  dump.enable_write(dump.size());
  { ObjectDumper od(&dump, JVMPI_DUMP_LEVEL_2, obj); }
  // create event
  JVMPI_Event event;
  event.event_type             = JVMPI_EVENT_OBJECT_DUMP | flag;
  event.u.heap_dump.begin      = (char*)dump.begin();
  event.u.heap_dump.end        = (char*)dump.end  ();
  event.u.heap_dump.num_traces = 0;
  event.u.heap_dump.traces     = NULL;
  // post event
  post_event_vm_mode(&event, NULL);
}

class VM_JVMPIPostHeapDump: public VM_Operation {
 private:
  Dump* _dump;
  int   _level;
  int   _flag;
 public:
  VM_JVMPIPostHeapDump(Dump* dump, int level, int flag) { _dump = dump; _level = level; _flag = flag; }
  void doit() {
    // 1st heap dump to measure dump size for heap objects
    { HeapDumper hd(_dump, _level); }
    // collect VM roots and dump them
    if (_level == JVMPI_DUMP_LEVEL_0) {
      // dump level 0 => no roots
      _dump->enable_write(_dump->size());
    } else {
      // dump level 1 & 2 => include roots
      RootCollector rd;
      _dump->enable_write(rd.root_dump_size() + _dump->size());
      rd.dump_roots(_dump);
      assert(rd.root_dump_size() == _dump->size(), "dump size inconsistent");
    }
    // 2nd heap dump to actually write heap objects
    { HeapDumper hd(_dump, _level); }
  }
  const char* name() const { return "post JVMPI heap dump"; }
};


void jvmpi::post_heap_dump_event_in_safepoint(int level, int flag) {
  Dump dump;
  // We count and collect the heap information at a safepoint
  VM_JVMPIPostHeapDump op(&dump, level, flag);
  VMThread::execute(&op);
  // Create and post the event in the JavaThread
  // We don't put this in a doit_epilogue to avoid exposing the Dump class
//  assert(Thread::current()->is_Java_thread(), "must be in JavaThread");
  JVMPI_Event event;
  event.event_type             = JVMPI_EVENT_HEAP_DUMP | flag;
  event.u.heap_dump.dump_level = level;
  event.u.heap_dump.begin      = (char*)dump.begin();
  event.u.heap_dump.end        = (char*)dump.end  ();
  event.u.heap_dump.num_traces = 0;
  event.u.heap_dump.traces     = NULL;
  // post event
  post_event_vm_mode(&event, NULL);
}


class VM_JVMPIPostMonitorDump: public VM_Operation {
 private:
  Dump* _dump;
  int   _flag;
 public:
  VM_JVMPIPostMonitorDump(Dump* dump, int flag) { _dump = dump; _flag = flag; }
  void doit() {
    // 1st dump to measure dump size
    { JavaMonitorDumper md(_dump); 
      RawMonitorDumper rmd(_dump);
    }
    // 2nd dump to actually write dump
    _dump->enable_write(_dump->size());
    { JavaMonitorDumper md(_dump); 
      RawMonitorDumper rmd(_dump);
    }
  }
  const char* name() const { return "post JVMPI monitor dump"; }
};


void jvmpi::post_monitor_dump_event_in_safepoint(int flag) {
  Dump dump;
  // We count and collect the monitor information at a safepoint
  VM_JVMPIPostMonitorDump op(&dump, flag);
  VMThread::execute(&op);
  // Create and post the event in the JavaThread
  // We don't put this in a doit_epilogue to avoid exposing the Dump class
//  assert(Thread::current()->is_Java_thread(), "must be in JavaThread");
  JVMPI_Event event;
  event.event_type = JVMPI_EVENT_MONITOR_DUMP | flag;
  event.u.monitor_dump.begin          = (char*)dump.begin();
  event.u.monitor_dump.end            = (char*)dump.end  ();
  event.u.monitor_dump.num_traces     = 0;
  event.u.monitor_dump.threads_status = 0;
  // post event
  post_event_vm_mode(&event, NULL);
}


bool should_invalidate_nmethods(jint event_type) {
  switch (event_type) {
    case JVMPI_EVENT_METHOD_ENTRY : // fall through
    case JVMPI_EVENT_METHOD_ENTRY2: // fall through
    case JVMPI_EVENT_METHOD_EXIT  : return true;
  }
  return false;
}


void invalidate_nmethods() {
  // need do deoptimize all frames; for the moment we just make all methods
  // non-entrant
}


bool needs_slow_allocation(jint event_type) {
  switch(event_type) {
    case JVMPI_EVENT_OBJECT_ALLOC       : // fall through
    case JVMPI_EVENT_OBJECT_MOVE        : // fall through
    case JVMPI_EVENT_OBJECT_FREE        : // fall through
    case JVMPI_EVENT_ARENA_NEW          : // fall through
    case JVMPI_EVENT_DELETE_ARENA       : // fall through
    case JVMPI_EVENT_JNI_GLOBALREF_ALLOC: // fall through
    case JVMPI_EVENT_JNI_GLOBALREF_FREE : return true;
  }
  return false;
}


// ----------------------------------------------
// Functions exported through the JVMPI interface
// ----------------------------------------------

JVMPI_ENTRY(jint, jvmpi::enable_event(jint event_type, void *arg))
  if (!is_event_supported(event_type)) {
    return JVMPI_NOT_AVAILABLE;
  }

  enable_event(event_type);
  if (should_invalidate_nmethods(event_type)) {
    invalidate_nmethods();
  }
  if (event_type == JVMPI_EVENT_OBJECT_MOVE) {
    Universe::set_jvmpi_move_event_enabled(true);
  } else if (event_type == JVMPI_EVENT_METHOD_ENTRY || event_type == JVMPI_EVENT_METHOD_ENTRY2) {
  // Missing disabling of inlining
    // Inline flag is a constant in product mode
    // Inline = false;
  } else if (event_type == JVMPI_EVENT_JNI_GLOBALREF_ALLOC) {
    Universe::set_jvmpi_jni_global_alloc_event_enabled(true);
  } else if (event_type == JVMPI_EVENT_JNI_GLOBALREF_FREE) {
    Universe::set_jvmpi_jni_global_free_event_enabled(true);
  } else if (event_type == JVMPI_EVENT_JNI_WEAK_GLOBALREF_ALLOC) {
    Universe::set_jvmpi_jni_weak_global_alloc_event_enabled(true);
  } else if (event_type == JVMPI_EVENT_JNI_WEAK_GLOBALREF_FREE) {
    Universe::set_jvmpi_jni_weak_global_free_event_enabled(true);
  }

  if (!slow_allocation && needs_slow_allocation(event_type)) {
    slow_allocation = true;
    Universe::set_jvmpi_alloc_event_enabled(true);
    // it is too early to report GC-events
    bool old_gc_start = is_event_enabled(JVMPI_EVENT_GC_START);
    bool old_gc_finish = is_event_enabled(JVMPI_EVENT_GC_FINISH);
    disable_event(JVMPI_EVENT_GC_START);
    disable_event(JVMPI_EVENT_GC_FINISH);
    Scavenge::invoke(0, CHECK_0);
    if (old_gc_start) enable_event(JVMPI_EVENT_GC_START);
    if (old_gc_finish) enable_event(JVMPI_EVENT_GC_FINISH);
  }
  return JVMPI_SUCCESS;
JVMPI_END


JVMPI_ENTRY(jint, jvmpi::disable_event(jint event_type, void *arg))
  if (!is_event_supported(event_type)) {
    return JVMPI_NOT_AVAILABLE;
  }

  if (should_invalidate_nmethods(event_type)) {
    invalidate_nmethods();
  }
  disable_event(event_type);
  if (event_type == JVMPI_EVENT_OBJECT_MOVE) {
    Universe::set_jvmpi_move_event_enabled(false);
  } else if (event_type == JVMPI_EVENT_OBJECT_ALLOC) {
    Universe::set_jvmpi_alloc_event_enabled(false);
  } else if (event_type == JVMPI_EVENT_JNI_GLOBALREF_ALLOC) {
    Universe::set_jvmpi_jni_global_alloc_event_enabled(false);
  } else if (event_type == JVMPI_EVENT_JNI_GLOBALREF_FREE) {
    Universe::set_jvmpi_jni_global_free_event_enabled(false);
  } else if (event_type == JVMPI_EVENT_JNI_WEAK_GLOBALREF_ALLOC) {
    Universe::set_jvmpi_jni_weak_global_alloc_event_enabled(false);
  } else if (event_type == JVMPI_EVENT_JNI_WEAK_GLOBALREF_FREE) {
    Universe::set_jvmpi_jni_weak_global_free_event_enabled(false);
  }
  return JVMPI_SUCCESS;
JVMPI_END


JVMPI_ENTRY(void, jvmpi::disable_gc())
  GC_locker::lock();
JVMPI_END


JVMPI_ENTRY(void, jvmpi::enable_gc())
  GC_locker::unlock();
JVMPI_END


// The thread we are examining must be suspended
void fill_call_trace_at_safepoint(JavaThread* thd, JVMPI_CallTrace* trace, int depth) {
      vframeStream st(thd); 

      int count = 0;
      // collect the rest
      for (;!st.at_end() && count < depth; st.next(), count++) {    
        methodOop m = st.method(); // The method is not stored GC safe
        int bci     = st.bci();
        int lineno  = m->is_native() ? (-3) : m->line_number_from_bci(bci);
        trace->frames[count].method_id = (jmethodID)m->jni_id();
        trace->frames[count].lineno = lineno;
      }

      trace->num_frames = count;
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: fill_call_trace_at_safepoint return, thread: 0x%x, trace->num_frames = %d\n", thd, trace->num_frames);
  }
  return; 
}

void fill_call_trace_given_top(JavaThread* thd, JVMPI_CallTrace* trace, int depth, frame top_frame, int count) {
    vframeStream st(thd, top_frame);
  
    for (; !st.at_end() && count < depth; st.next(), count++) {
      methodOop m = st.method(); // The method is not stored GC safe
      int bci     = st.bci();
      int lineno  = m->is_native() ? (-3) : m->line_number_from_bci(bci);
      trace->frames[count].method_id = (jmethodID)m->jni_id();
      trace->frames[count].lineno = lineno;
    }
    trace->num_frames = count;
  return;
}

inline bool is_valid_method(methodOop method) {
  if (method == NULL || 
      !method->is_perm() || 
      oop(method)->klass() != Universe::methodKlassObj() ||
      !method->is_method()) {
    return false;   // doesn't look good
  }
  return true;      // hopefully this is a method indeed
}

static bool wait_for_suspend_completion(JavaThread* thd, int count) {
  bool pending;
  bool is_suspended;


  { MutexLocker ml(Interrupt_lock);
    pending = thd->is_external_suspend();
    is_suspended = thd->is_suspended();
  }
  // must release Interrupt_lock to allow suspension to complete
  for (int retries = 0; ((pending == true) && (is_suspended == false) && retries < count); retries++) {

    if (thd->osthread()->get_state() == CONDVAR_WAIT) {
      // If we have reached CONDVAR_WAIT state, i.e., wait for Java
      // ObjectMonitor, then that is the same as externally suspended
      // since no additional bytecodes will be run.
      return true;
    }

    // If the VMThread is trying bring everyone to a safepoint,
    // then we block ourself on the Threads_lock. Our grab of the
    // Interrupt_lock will block us only if it is contended.
    if (SafepointSynchronize::is_synchronizing()) {
      Threads_lock->lock();
      Threads_lock->unlock();
    }
    os::yield_all(retries);
    { MutexLocker ml(Interrupt_lock);
      is_suspended = thd->is_suspended();
    }
  }
  return is_suspended;
}

// experimental number, can not allow to run forever
// or we could hang
#define SUSPENDRETRYCOUNT 100

JVMPI_ENTRY(void, jvmpi::get_call_trace(JVMPI_CallTrace *trace, jint depth))
  JavaThread* thd;
  ResourceMark rm;

  trace->num_frames = 0;
  if (!((trace->env_id) && (thd = JavaThread::thread_from_jni_environment(trace->env_id))))  {
    return;
  }

  // ensure thread suspension completed for other threads
  // Note: need to ensure hprof agent actually suspends threads
  // May need to temporarily suspend thread for the caller
  if (thd != Thread::current()) {
    if (!wait_for_suspend_completion(thd, SUSPENDRETRYCOUNT)) return;
  }

  switch (thd->thread_state()) {
    // The thread is either in the VM or in native code so use information
    // from the last Java frame.
    case _thread_blocked:
    case _thread_in_native:
    case _thread_in_native_blocked:
    case _thread_in_vm:    
      if (thd->has_last_Java_frame()) {
        fill_call_trace_at_safepoint(thd, trace, depth);
      }
      break;
    case _thread_in_Java:  
      { frame fr;
        bool walkframe = false;
        trace->num_frames = 0;
        // profile_top_frame sets up the frame 'fr' and returns true;
        if (thd->profile_top_frame(&fr)) {
          methodOop method = NULL;
          int lineno = -1;
          if (fr.is_interpreted_frame()) {
            if (fr.is_interpreted_frame_valid()) {
           
            if (fr.fp() != NULL) {
                // access address in order not to trigger asserts that
                // are built in interpreter_frame_method function
                method = *fr.interpreter_frame_method_addr();
                if (is_valid_method(method)) {
                  int bci = fr.interpreter_frame_bci();
                  lineno  = method->is_native() ? (-1) : method->line_number_from_bci(bci);
                  walkframe = true;
                } else {
                  method = NULL;
                }
              }
            }
#ifndef CORE
          } else if (CodeCache::contains(fr.pc())) {
            CodeBlob* cb = CodeCache::find_blob(fr.pc());
          //  method = (cb->is_nmethod()) ? ((nmethod *)cb)->method() : NULL;  
            lineno = -1;
            if (cb->is_nmethod()) {
                                method = ((nmethod *)cb)->method();
                                walkframe = true;
            // fprofiler finds it safe to examine the last three stub types
            // With clean suspension, we frequently see is_safepoint_stub
            } else if (cb->is_safepoint_stub() || cb->is_runtime_stub()
                      || cb->is_i2c_adapter() || cb->is_c2i_adapter()) {
                                method = NULL;
                                walkframe = true;
            }
#endif
          } else {
            // pc is returned from -1 off of sp. If this is not valid,
            // follow the valid frame pointer returned
            method = NULL;
            walkframe = true;
          }
          if (method != NULL) {
            trace->frames[0].method_id = (jmethodID)method->jni_id();
            trace->frames[0].lineno = lineno;
            trace->num_frames = 1;
          }
          if (walkframe == true) {
            // frame is set up  
            fill_call_trace_given_top(thd, trace, depth, fr, trace->num_frames);
          }
        }
      }
      break;
    default: break;
  }
JVMPI_END


JVMPI_ENTRY(jlong, jvmpi::get_current_thread_cpu_time())
  return os::thread_cpu_time();
JVMPI_END


JVMPI_RAW_ENTRY(JVMPI_RawMonitor, jvmpi::raw_monitor_create(char *lock_name))
  RawMonitor * monitor = new RawMonitor(lock_name, PROF_RM_MAGIC);
  return (JVMPI_RawMonitor)monitor;
JVMPI_RAW_END


JVMPI_RAW_ENTRY(void, jvmpi::raw_monitor_enter(JVMPI_RawMonitor lock_id))
  RawMonitor *monitor = (RawMonitor *)lock_id;
  if (!(PROF_RM_CHECK(monitor))) {
      return;
  }
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: raw_monitor_enter for thread id 0x%x lock_id = 0x%x ", THREAD, lock_id);
  }
  monitor->raw_enter(THREAD, true);
JVMPI_RAW_END


JVMPI_RAW_ENTRY(void, jvmpi::raw_monitor_exit(JVMPI_RawMonitor lock_id))
  RawMonitor *monitor = (RawMonitor *)lock_id;
  if (!(PROF_RM_CHECK(monitor))) {
      return;
  }
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: raw_monitor_exit for thread id 0x%x lock_id = 0x%x ", THREAD, lock_id);
  }
  monitor->raw_exit(THREAD, true);
JVMPI_RAW_END


JVMPI_RAW_ENTRY(void, jvmpi::raw_monitor_destroy(JVMPI_RawMonitor lock_id))
  RawMonitor *monitor = (RawMonitor *)lock_id;
  if (!(PROF_RM_CHECK(monitor))) {
      return;
  }
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: raw_monitor_destroy for thread id 0x%x lock_id = 0x%x ", THREAD, lock_id);
  }
  monitor->raw_exit(THREAD, true);
  monitor->raw_destroy();
JVMPI_RAW_END


JVMPI_RAW_ENTRY(void, jvmpi::raw_monitor_wait(JVMPI_RawMonitor lock_id, jlong ms))
  RawMonitor *monitor = (RawMonitor *)lock_id;
  if (!(PROF_RM_CHECK(monitor))) {
      return;
  }
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: raw_monitor_wait for thread id 0x%x lock_id = 0x%x ", THREAD, lock_id);
  }
  monitor->raw_wait(ms, true, THREAD);
JVMPI_RAW_END


JVMPI_RAW_ENTRY(void, jvmpi::raw_monitor_notify_all(JVMPI_RawMonitor lock_id))
  RawMonitor *monitor = (RawMonitor *)lock_id;
  if (!(PROF_RM_CHECK(monitor))) {
      return;
  }
  if (TraceJVMPI) {
    tty->print_cr("\nJVMPI: raw_monitor_notify_all for thread id 0x%x lock_id = 0x%x ", THREAD, lock_id);
  }
  monitor->raw_notifyAll(THREAD);
JVMPI_RAW_END

// Use shared java_suspend.
JVMPI_ENTRY(void, jvmpi::suspend_thread(JNIEnv *env))
  if (env == NULL) return;
  JavaThread *java_thread = JavaThread::thread_from_jni_environment(env);
  if (java_thread == NULL) return;
  // the thread has not yet run or is in process of exiting
  if (java_lang_Thread::thread(java_thread->threadObj()) == NULL) return;

  // Don't allow self-suspension, hprof agent expects to keep
  // running so as to process resumes of all threads.
  // Also don't allow repeated suspensions.
  if (Thread::current() == (Thread *)java_thread ||
      java_thread->is_being_ext_suspended()) {
    return;
  }

  //
  // If a thread in state _thread_in_native is not immediately
  // suspended, then a blocked RawMonitorEnter() call may enter
  // the RawMonitor even if RawMonitorExit() is called after
  // SuspendThread() returns.
  //
  java_thread->java_suspend(true);

  // It would be nice to have the following assertion in all the time,
  // but it is possible for a racing resume request to have resumed
  // this thread right after we suspended it. Temporarily enable this
  // assertion if you are chasing a different kind of bug.
  //
  // assert(java_lang_Thread::thread(java_thread->threadObj()) == NULL ||
  //   java_thread->is_being_ext_suspended(), "thread is not suspended");
JVMPI_END

// Use shared java_resume. Requires owning the Threads lock.
JVMPI_ENTRY(void, jvmpi::resume_thread(JNIEnv *env))
  JavaThread *java_thread;
  if ((env) && (java_thread = JavaThread::thread_from_jni_environment(env))) {
    MutexLocker ml(Threads_lock);
    java_thread->java_resume();
  }
JVMPI_END

// 2.0: redesign to match jvmdi. handle errors and more states
JVMPI_ENTRY(jint, jvmpi::get_thread_status(JNIEnv *env))
  jint res = JVMPI_THREAD_RUNNABLE;
  JavaThread *tp;
  { MutexLocker mu(Threads_lock);
    if ((env) && (tp = JavaThread::thread_from_jni_environment(env))) {
      JavaThreadState state;
      ThreadState t_state;
      if ((state = tp->thread_state()) && (tp->osthread()) && (t_state = tp->osthread()->get_state())) {

          if (state == _thread_blocked|| state == _thread_blocked_trans) {
              switch (t_state) {
                  case CONDVAR_WAIT: 
                      res = JVMPI_THREAD_CONDVAR_WAIT; 
                      break;
                  case MONITOR_WAIT: 
                      res = JVMPI_THREAD_MONITOR_WAIT; 
                      break;
                  case SUSPENDED:
                  case SLEEPING:
                  case ZOMBIE:
                  case RUNNABLE    : // fall through
                      res = JVMPI_THREAD_RUNNABLE;
                      break;
                  default:
                      break;
              }
           }
          if (tp->is_being_ext_suspended()) {
              // internal suspend doesn't count for this flag
              res = res | JVMPI_THREAD_SUSPENDED;
          }
          if (tp->osthread()->interrupted()) {
              res = res | JVMPI_THREAD_INTERRUPTED;
          }
      }
    }
  } // release Threads_lock
  return res;
JVMPI_END


// There is no provision in VM to check that; assume yes
// Do NOT call thread_is_running - this calls thr_getstate
// which only works if you have called thr_suspend.
JVMPI_ENTRY(jboolean, jvmpi::thread_has_run(JNIEnv *env))
  JavaThread* java_thread;
  if ((env) && (java_thread = JavaThread::thread_from_jni_environment(env)))  {
    return JNI_TRUE;
  } else {
    return JNI_FALSE;
  }
JVMPI_END


JVMPI_ENTRY(void, jvmpi::run_gc())
  MarkSweep::invoke(0, _java_lang_system_gc, CHECK);
JVMPI_END


JVMPI_ENTRY(void, jvmpi::profiler_exit(jint exit_code))
  os::exit(exit_code /*user_exit == true*/); 
  ShouldNotReachHere();
JVMPI_END


static void jvmpi_daemon_thread_entry(JavaThread* thread, TRAPS) {
  assert(thread->is_jvmpi_daemon_thread(), "wrong thread");
  JVMPIDaemonThread* daemon_thread = (JVMPIDaemonThread*)thread;

  // ThreadToNativeFromVM takes care of changing thread_state, so safepoint code knows that
  // we have left the VM
  { JavaThread* thread = (JavaThread*) THREAD;
    ThreadToNativeFromVM ttn(thread);
    HandleMark hm(thread);

    daemon_thread->function()(NULL);
  }
}


JVMPI_ENTRY(jint, jvmpi::create_system_thread(char *name, jint priority, JVMPIDaemonFunction f))
  const int invalid_res = JNI_ERR;
  instanceKlassHandle klass = SystemDictionary::resolve_or_fail(vmSymbolHandles::java_lang_Thread(), NULL, NULL, true, CHECK_(invalid_res));
  instanceHandle thread_oop = klass->allocate_instance(CHECK_(invalid_res));
  Handle string = java_lang_String::create_from_str(name, CHECK_(invalid_res));    

  // Initialize thread_oop to put it into the system threadGroup    
  Handle thread_group = Universe::system_thread_group();
  JavaValue result(T_VOID);
  JavaCalls::call_special(&result, thread_oop, 
                         klass, 
                         vmSymbolHandles::object_initializer_name(), 
                         vmSymbolHandles::threadgroup_string_void_signature(), 
                         thread_group, 
                         string, 
                         CHECK_(invalid_res));  
  
  { MutexLocker mu(Threads_lock);
    JVMPIDaemonThread* daemon_thread = new JVMPIDaemonThread(&jvmpi_daemon_thread_entry, f);

    // At this point it may be possible that no osthread was created for the
    // JavaThread due to lack of memory. We would have to throw an exception
    // in that case. However, since this must work and we do not allow
    // exceptions anyway, just use an assertion here.
    guarantee(daemon_thread->osthread() != NULL, "thread creation failed");
   
    ThreadPriority thread_priority = NoPriority;
    switch (priority) {
      case JVMPI_MINIMUM_PRIORITY: thread_priority = MinPriority ; break;
      case JVMPI_MAXIMUM_PRIORITY: thread_priority = MaxPriority ; break;
      case JVMPI_NORMAL_PRIORITY : thread_priority = NormPriority; break;
      default: ShouldNotReachHere();
    }

    java_lang_Thread::set_thread(thread_oop(), daemon_thread);      
    java_lang_Thread::set_priority(thread_oop(), thread_priority);
    java_lang_Thread::set_daemon(thread_oop());

    daemon_thread->set_threadObj(thread_oop());
    Threads::add(daemon_thread);  
    Thread::start(daemon_thread);

  } // Release Threads_lock before calling up to agent code
  // post_thread_start_event called from "run"
  
  return JNI_OK;
JVMPI_END


JVMPI_ENTRY(jint, jvmpi::request_event(jint event_type, void *arg))
  switch (event_type) {
    case JVMPI_EVENT_OBJECT_ALLOC:
      post_object_alloc_event((oop)arg, ((oop)arg)->size() * wordSize, Universe::addr_to_arena_id(arg), JVMPI_REQUESTED_EVENT);
      return JVMPI_SUCCESS;
    case JVMPI_EVENT_THREAD_START:
      post_thread_start_event(java_lang_Thread::thread((oop)arg), JVMPI_REQUESTED_EVENT);
      return JVMPI_SUCCESS;
    case JVMPI_EVENT_CLASS_LOAD:
      post_class_load_event((oop)arg, JVMPI_REQUESTED_EVENT);
      return JVMPI_SUCCESS;
    case JVMPI_EVENT_OBJECT_DUMP:
      post_object_dump_event((oop)arg, JVMPI_REQUESTED_EVENT);
      return JVMPI_SUCCESS;
    case JVMPI_EVENT_HEAP_DUMP: {
      int heap_dump_level;

      if (arg == NULL) {
        heap_dump_level = JVMPI_DUMP_LEVEL_2;
      } else {
        heap_dump_level = ((JVMPI_HeapDumpArg*)arg)->heap_dump_level;
      }

      post_heap_dump_event_in_safepoint(heap_dump_level, JVMPI_REQUESTED_EVENT);
      return JVMPI_SUCCESS;
    }
    case JVMPI_EVENT_MONITOR_DUMP:
      post_monitor_dump_event_in_safepoint(JVMPI_REQUESTED_EVENT);
      return JVMPI_SUCCESS;
    default:
      return JVMPI_NOT_AVAILABLE;
  }
JVMPI_END


JVMPI_ENTRY(void, jvmpi::set_thread_local_storage(JNIEnv *env, void *ptr))
  if (env != NULL)
    JavaThread::thread_from_jni_environment(env)->set_jvmpi_data(ptr);
JVMPI_END


JVMPI_ENTRY(void*, jvmpi::get_thread_local_storage(JNIEnv *env))
  if (env == NULL) return NULL;
  return JavaThread::thread_from_jni_environment(env)->jvmpi_data();
JVMPI_END


JVMPI_ENTRY(jobjectID, jvmpi::get_thread_object(JNIEnv *env))
  if (env == NULL) return NULL;
  return (jobjectID) JavaThread::thread_from_jni_environment(env)->threadObj();
JVMPI_END


JVMPI_ENTRY(jobjectID, jvmpi::get_method_class(jmethodID mid))
  return (jobjectID) Klass::cast(((JNIid*)mid)->holder())->java_mirror();
JVMPI_END


JVMPI_ENTRY(jobject, jvmpi::jobjectID_2_jobject(jobjectID jid))
  assert(GC_locker::is_active(), "jobjectID_2_jobject may be called only with disabled GC");
  Thread* thd = Thread::current();
  assert(thd->is_Java_thread(), "call to jobjectID_2_jobject can only happen in a Java thread");

  JavaThread* jthread = (JavaThread*)thd;

  JNIEnv* env = jthread->jni_environment();
  return JNIHandles::make_local(env, (oop)jid);
JVMPI_END


JVMPI_ENTRY(jobjectID, jvmpi::jobject_2_jobjectID(jobject jobj))
  assert(GC_locker::is_active(), "jobject_2_jobjectID may be called only with disabled GC");
  return (jobjectID)JNIHandles::resolve(jobj);
JVMPI_END

