/*
 * @(#)aprofiler.hpp	1.21 01/02/09 14:47:28
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// A simple allocation profiler for Java. The profiler collects and prints
// the number and total size of instances allocated per class, including
// array classes.
//
// The profiler is currently global for all threads. It can be changed to a 
// per threads profiler by keeping a more elaborate data structure and calling
// iterate_since_last_scavenge at thread switches.


class AllocationProfiler: AllStatic {
 private:
  static bool _active;                          // tells whether profiler is active
  static GrowableArray<klassOop>* _print_array; // temporary array for printing

  // Utility printing functions
  static void add_class_to_array(klassOop k);
  static void add_classes_to_array(klassOop k);
  static int  compare_classes(klassOop* k1, klassOop* k2);
  static int  average(int alloc_size, int alloc_count);
  static void sort_and_print_array(int cutoff);

  // Call for collecting allocation information. Called at scavenge, mark-sweep and disengage.
  static void iterate_since_last_gc();
  friend void Scavenge::invoke_at_safepoint(int, bool, bool&);
  friend void MarkSweep::invoke_at_safepoint(int, bool, bool, bool&, MarkSweepInvokeCause);

 public:
  // Start profiler
  static void engage();
  // Stop profiler
  static void disengage();
  // Tells whether profiler is active
  static bool is_active()                   { return _active; }
  // Print profile
  static void print(int cutoff);   // Cutoff in total allocation size (in words)
};
