/*
 * @(#)arguments.cpp	1.138 00/12/20 15:48:02
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


#include "incls/_precompiled.incl"
#include "incls/_arguments.cpp.incl"


UserProperty* Arguments::_user_properties       = NULL;
char*  Arguments::_sysclasspath                 = NULL;
bool   Arguments::_oldjava                      = false;
bool   Arguments::_has_profile                  = false;
bool   Arguments::_has_alloc_profile            = false;
size_t    Arguments::_initial_heap_size            = 0;
size_t    Arguments::_max_heap_size                = 0;
Arguments::Mode Arguments::_mode                = _mixed;

short  Arguments::CompileOnlyClassesNum	        = 0;
short  Arguments::CompileOnlyClassesMax	        = 0;
char** Arguments::CompileOnlyClasses		= NULL;
bool*  Arguments::CompileOnlyAllMethods	        = NULL;

short  Arguments::CompileOnlyMethodsNum	        = 0;
short  Arguments::CompileOnlyMethodsMax	        = 0;
char** Arguments::CompileOnlyMethods		= NULL;
bool*  Arguments::CompileOnlyAllClasses	        = NULL;

bool   Arguments::CheckCompileOnly              = false;

UserProperty* Arguments::_libraries             = NULL;
bool   Arguments::_init_libraries_at_startup    = false;

abort_hook_t     Arguments::_abort_hook         = NULL;
exit_hook_t      Arguments::_exit_hook          = NULL;
vfprintf_hook_t  Arguments::_vfprintf_hook      = NULL;


// Parses a memory size specification string.


static bool atomll(char *s, jlong* result) {
  jlong n = 0;
  int args_read = sscanf(s, os::jlong_format_specifier(), &n);
  if (args_read != 1) {
    return false;
  }
  while (*s != '\0' && isdigit(*s)) {
    s++;
  }
  switch (*s) {
    case 'G': case 'g':
      *result = n * M * K;
      return true;
    case 'M': case 'm':
      *result = n * M;
      return true;
    case 'K': case 'k':
      *result = n * K;
      return true;
    case '\0':
      *result = n;
      return true;
    default:
      return false;
  }
}


bool check_memory_size(jlong size, jlong min_size) {
  if (size < min_size) return false;
  // Check that size will fit in a size_t (only relevant on 32-bit)
  if ((julong) size > ULONG_MAX) return false;
  return true;
}


static bool set_bool_flag(char* name, bool value) {
  return DebugFlags::boolAtPut(name, &value);
}


static bool set_numeric_flag(char* name, char* value) {
  jlong v;
  if (!atomll(value, &v)) {
    return false;
  }
  int int_v = (int) v;
  if (DebugFlags::intAtPut(name, &int_v)) {
    return true;
  }
  size_t size_t_v = (size_t) v;
  if (DebugFlags::size_tAtPut(name, &size_t_v)) {
    return true;
  }
  return false;
}


void Arguments::parseOnlyLine (char* line, 
  short* classesNum, short* classesMax, char*** classes, bool** allMethods,
  short* methodsNum, short* methodsMax, char*** methods, bool** allClasses) {

  int i;
  char name[1024];
  bool className = true;	// current string is class name.
  bool addedClass = false;	
  bool addedMethod = false;

  while (*line != '\0') {
    for (i = 0 ; i < 1024 && *line != '\0' && *line != '.' && *line != ','; line++, i++)
      name[i] = *line;

    if (i > 0) {
      char* newName = NEW_C_HEAP_ARRAY( char, i + 1);
      if (newName == NULL)
        return;
      strncpy( newName, name, i);
      newName[i] = '\0';

      if (className) {
        addedClass = true;
        if (*classesNum == *classesMax) {
          *classesMax += 5;
          *classes = REALLOC_C_HEAP_ARRAY( char*, *classes, *classesMax);
          *allMethods = REALLOC_C_HEAP_ARRAY( bool, *allMethods, *classesMax);
          if (*classes == NULL || *allMethods == NULL)
            return;
        }
        (*classes)[*classesNum] = newName;
        (*allMethods)[*classesNum] = false;
        (*classesNum)++;
      }
      else {
        addedMethod = true;
        if (*methodsNum == *methodsMax) {
          *methodsMax += 5;

          *methods = REALLOC_C_HEAP_ARRAY( char*, *methods, *methodsMax);
          *allClasses = REALLOC_C_HEAP_ARRAY( bool, *allClasses, *methodsMax);
          if (*methods == NULL || *allClasses == NULL)
            return;
        }
        (*methods)[*methodsNum] = newName;
        (*allClasses)[*methodsNum] = false;
        (*methodsNum)++;
      }
    }

    if (*line == '.')
      className = false;

    if (*line == ',' || *line == '\0') {
      if (addedClass && !addedMethod) {
        (*allMethods)[*classesNum - 1] = true;
      }
      if (addedMethod && !addedClass) {
        (*allClasses)[*methodsNum - 1] = true;
      }
      className = true;
      addedClass = false;
      addedMethod = false;
    }

    line = *line == '\0' ? line : line + 1;
  }
}


bool Arguments::parse_argument(char* arg) {
  // range of acceptable characters spelled out for portability reasons
  char name[256];
  #define NAME_RANGE  "[abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789]"
  if (sscanf(arg, "-%" NAME_RANGE, name) == 1) {
    return set_bool_flag(name, false);
  }
  if (sscanf(arg, "+%" NAME_RANGE, name) == 1) {
    return set_bool_flag(name, true);
  }
  char value[256];
  #define VALUE_RANGE "[-kmKM0123456789]"
  if (sscanf(arg, "%" NAME_RANGE "=" "%" VALUE_RANGE, name, value) == 2) 
    return set_numeric_flag(name, value);

#ifndef PRODUCT
  // Parse the CompileOnly= option, if present, and record the classes and methods that
  // should always be compiled.  
  #define COMPILE_ONLY	"CompileOnly="
  if (strncmp( arg, COMPILE_ONLY, strlen( COMPILE_ONLY)) == 0) {
    parseOnlyLine( arg + strlen( COMPILE_ONLY),
      &CompileOnlyClassesNum, &CompileOnlyClassesMax, &CompileOnlyClasses, &CompileOnlyAllMethods,
      &CompileOnlyMethodsNum, &CompileOnlyMethodsMax, &CompileOnlyMethods, &CompileOnlyAllClasses);
    if (CompileOnlyClassesNum > 0 || CompileOnlyMethodsNum > 0)
      CheckCompileOnly = true;  
    return true;
  }
#endif

  return false;
}


bool Arguments::process_argument(char* arg, bool ignore_unrecognized) {
  if (parse_argument(arg)) {
    if (PrintVMOptions) {
      jio_fprintf(stdout, "VM option '%s'\n", arg);
    }
  } else {
    if (!ignore_unrecognized) {
      jio_fprintf(stderr, "Unrecognized VM option '%s'\n", arg);
      // allow for commandline "commenting out" options like -XX:#+Verbose
      if (strlen(arg) == 0 || arg[0] != '#') {
        return false;
      }
    }
  }
  return true;
}


bool Arguments::process_settings_file(const char* file_name, bool should_exist, bool ignore_unrecognized) {
  FILE* stream = fopen(file_name, "rb");
  if (stream == NULL) {
    if (should_exist) {
      jio_fprintf(stderr, "Could not open settings file %s\n", file_name);
      return false;
    } else {
      return true;
    }
  }

  char token[1024];
  int  pos = 0;

  bool in_white_space = true;
  bool in_comment     = false;
  bool result         = true;

  int c = getc(stream);
  while(c != EOF) {
    if (in_white_space) {
      if (in_comment) {
	if (c == '\n') in_comment = false;
      } else {
        if (c == '#') in_comment = true;
        else if (!isspace(c)) {
          in_white_space = false;
	  token[pos++] = c;
        }
      }
    } else {
      if (isspace(c)) {
        token[pos] = '\0';
        result &= process_argument(token, ignore_unrecognized);
	pos = 0;
	in_white_space = true;
      } else {
        token[pos++] = c;
      }
    }
    c = getc(stream);
  }
  if (pos > 0) {
    token[pos] = '\0';
    result &= process_argument(token, ignore_unrecognized);
  }
  fclose(stream);
  return result;
}


//=============================================================================================================
// Parsing of properties (-D) 

bool Arguments::add_property(char* prop) {
  char* eq = strchr(prop, '=');
  char* key;
  // ns must be static--its address may be stored in a UserProperty object.
  static char ns[1] = {0};
  char* value = ns;
  if (eq == NULL) {
    key = prop;
  } else {
    int key_len = eq - prop;
    key = AllocateHeap(key_len + 1, "add_property");
    strncpy(key, prop, key_len);
    key[key_len] = '\0';

    int value_len = strlen(prop) - key_len - 1;
    value = AllocateHeap(value_len + 1, "add_property");
    strncpy(value, &prop[key_len + 1], value_len + 1);    
  }

  if (strcmp(key, "java.compiler") == 0) {
	process_java_compiler_argument(value);
    return true;
  }

  // Create new property and add at the end of the list
  UserProperty* new_p = new UserProperty(key, value);
  UserProperty* p = _user_properties;
  if (p == NULL) {
    _user_properties = new_p;
  } else {
    while (p->next() != NULL) {
      p = p->next();
    }
    p->set_next(new_p);
  }
  return true;
}


//===========================================================================================================
// Setting int/mixed/comp mode flags 

void Arguments::set_int_mode_flags() {
  _mode                      = _int;
  UseInterpreter             = true;
  UseRecompilation           = false;
  UseLoopCounter             = false;
  AlwaysCompileLoopMethods   = false;
  UseOnStackReplacement      = false;
  JavaMonitorsInStackTrace   = true;
}

void Arguments::set_mixed_mode_flags() {
  _mode                      = _mixed;
  UseInterpreter             = true;
  UseRecompilation           = true;
}

void Arguments::set_comp_mode_flags() {
  _mode                      = _comp;
  UseInterpreter             = false;
  UseRecompilation           = true;
  // DontCompileHugeMethods  = false;  // flag is deprecated
  BackgroundCompilation      = false;
  ClipInlining               = false;
  CIDynamicCompilePriority   = false;
}


//===========================================================================================================
// Parsing of java.compiler property and JAVA_COMPILER environment variable

void Arguments::process_java_compiler_argument(char* arg) {
  // Ignore java.compiler property and JAVA_COMPILER environment variable. 
  // It will cause the java.lang.Compiler static initializer to try to dynamically 
  // link the dll, which is not going to work since HotSpot does not support the 
  // old JIT interface. 
  // For backwards compatibility, we switch to interpreted mode if "NONE" or "" is 
  // specified.
  if (strlen(arg) == 0 || strcasecmp(arg, "NONE") == 0) {
	set_int_mode_flags();
  }
}

void Arguments::parse_java_compiler_environment_variable() {
  char buffer[64];
  if (os::getenv("JAVA_COMPILER", buffer, sizeof(buffer))) {
	process_java_compiler_argument(buffer);
  }
}


//===========================================================================================================
// Parsing of main arguments

bool Arguments::methodExists (char* className, char* methodName, 
  int classesNum, char** classes, bool* allMethods,
  int methodsNum, char** methods, bool* allClasses) {

  int i;
  bool classExists = false, methodExists = false;

  for (i = 0; i < classesNum; i++)
    if (strstr( className, classes[i]) != NULL) {
      if (allMethods[i])
        return true;
      classExists = true;
    }

  for (i = 0; i < methodsNum; i++)
    if (strcmp( methods[i], methodName) == 0) {
      if (allClasses[i])
        return true;
      methodExists = true;
    }

  return classExists && methodExists;
}


// Check if head of 'option' matches 'name', and sets 'tail' remaining part of option string

static bool match_option(JavaVMOption *option, const char* name, char** tail) {  
  int len = strlen(name);
  if (!strncmp(option->optionString, name, len)) {
    *tail = option->optionString + len;
    return true;
  } else {
    return false;
  }
}

// Check the consistency of vm_init_args
bool Arguments::check_vm_args_consistency() {
  // Method for adding checks for flag consistency.
  // The intent is to warn the user of all possible conflicts,
  // before returning an error.
  // Note: Needs platform-dependent factoring.
  bool status = true;
  
  #if ( (defined(WIN32) || defined(LINUX)))
  if(UseISM || UsePermISM) {
    jio_fprintf(stderr, "UseISM and UsePermISM not supported on this OS.\n");
    
    status = false;
  }
  #endif 

  #if (! (defined(WIN32) || defined(LINUX)))
  if (!UseBoundThreads && !UseStackBanging) {
    jio_fprintf(stderr, "-UseStackBanging conflicts with -UseBoundThreads\n");
     
     status = false;
  }
  #endif

  #ifdef COMPILER1
    #ifdef SPARC
      if (UseTLE) {
        jio_fprintf(stderr, "+UseTLE conflicts with -client (on SPARC)");
        status = false;
      }
      if (CacheLocalsInLoops && CacheLocalsGlobally) {
        jio_fprintf(stderr, "(+CachedLocalsInLoops && +CachedLocalsGlobally)"
                            " conflicts with -client (on SPARC)");
        status = false;
      }
    #endif
  #endif

  return status;
}

// Parse JavaVMInitArgs structure

jint Arguments::parse_vm_init_args(JavaVMInitArgs* args) {

#ifdef FASTDEBUG
  // fastdebug build should not print or ask anything extra unless requested
  VerifyBeforeExit = false;
  PromptForDebug = false;
  ShowMessageBoxOnError = false; 
#endif

  // Remaining part of option string
  char* tail;
  

  // iterate over arguments  
  for (int index = 0; index < args->nOptions; index++) {
    JavaVMOption* option = args->options + index;    
    // -verbose:[class/gc/jni]
    if (match_option(option, "-verbose", &tail)) {
      if (!strcmp(tail, ":class") || !strcmp(tail, "")) {
        TraceClassLoading = true;
        TraceClassUnloading = true;
      } else if (!strcmp(tail, ":gc")) {
        PrintGC = true;
        PrintScavenge = true;
        TraceClassUnloading = true;
      } else if (!strcmp(tail, ":jni")) {
        PrintJNIResolving = true;
      }    
    // -bootclasspath:
    } else if (match_option(option, "-Xbootclasspath:", &tail)) {
      _sysclasspath = tail;
    // -bootclasspath/a:
    } else if (match_option(option, "-Xbootclasspath/a:", &tail)) {
        char *np = NEW_C_HEAP_ARRAY(char, strlen(tail) + strlen(_sysclasspath) + strlen(os::path_separator()) + 1);
	if (np == NULL) return JNI_ENOMEM;
        sprintf(np, "%s%s%s", _sysclasspath, os::path_separator(), tail);
        _sysclasspath = np;
    // -bootclasspath/p:
    } else if (match_option(option, "-Xbootclasspath/p:", &tail)) {
        char *np = NEW_C_HEAP_ARRAY(char, strlen(tail) + strlen(_sysclasspath) + strlen(os::path_separator()) + 1);
	if (np == NULL) return JNI_ENOMEM;
        sprintf(np, "%s%s%s", tail, os::path_separator(), _sysclasspath);
        _sysclasspath = np;
    } else if (match_option(option, "-Xrun", &tail)) {
      char* name = tail;
      char* options = strchr(name,':');
      if (options != NULL) {
        *options = 0;
        options++;
      }

      if(name != NULL) {
        name = strcpy(NEW_C_HEAP_ARRAY(char,strlen(name) + 1),name);
        if(options != NULL) {
          options = strcpy(NEW_C_HEAP_ARRAY(char,strlen(options) + 1),options);
        }
        add_init_library(name,options);
      }
    } else if (match_option(option, "-Xoldjava", &tail)) {
      // -Xoldjava
      _oldjava = true;
    // -Xnoclassgc
    } else if (match_option(option, "-Xnoclassgc", &tail)) {
      ClassUnloading = false;
    // -Xincgc
    } else if (match_option(option, "-Xincgc", &tail)) {
      UseTrainGC = true;
    // -Xnoincgc
    } else if (match_option(option, "-Xnoincgc", &tail)) {
      UseTrainGC = false;
      // -Xbatch
    } else if (match_option(option, "-Xbatch", &tail)) {
      BackgroundCompilation = false;
      CIDynamicCompilePriority = false;
    // -Xms
    } else if (match_option(option, "-Xms", &tail)) {
      jlong long_initial_heap_size = 0;
      if (!atomll(tail, &long_initial_heap_size) || !check_memory_size(long_initial_heap_size, 1)) {
        jio_fprintf(stderr, "Invalid initial heap size: %s\n", option->optionString);
        return JNI_EINVAL;
      }
      _initial_heap_size = (size_t) long_initial_heap_size;
    // -Xmx
    } else if (match_option(option, "-Xmx", &tail)) {
      jlong long_max_heap_size = 0;
      if (!atomll(tail, &long_max_heap_size) || !check_memory_size(long_max_heap_size, 1)) {
        jio_fprintf(stderr, "Invalid maximum heap size: %s\n", option->optionString);
        return JNI_EINVAL;
      }
      _max_heap_size = (size_t) long_max_heap_size;
    // Xmaxf
    } else if (match_option(option, "-Xmaxf", &tail)) {
      int maxf = atof(tail) * 100;
      if (maxf < 0 || maxf > 100) {
        jio_fprintf(stderr, "Bad max heap free percentage size: %s\n", option->optionString);
        return JNI_EINVAL;
      } else {
        MaxHeapFreeRatio = maxf;
      }
    // Xminf
    } else if (match_option(option, "-Xminf", &tail)) {
      int minf = atof(tail) * 100;
      if (minf < 0 || minf > 100) {
        jio_fprintf(stderr, "Bad min heap free percentage size: %s\n", option->optionString);
        return JNI_EINVAL;
      } else {
        MinHeapFreeRatio = minf;
      }
    // -Xss
    } else if (match_option(option, "-Xss", &tail)) {
      jlong long_ThreadStackSize = 0;
      if (!atomll(tail, &long_ThreadStackSize) || !check_memory_size(long_ThreadStackSize, 1000)) { 
        jio_fprintf(stderr, "Invalid thread stack size: %s\n", option->optionString);
        return JNI_EINVAL;
      }
      ThreadStackSize = (int) long_ThreadStackSize;
      // Internally track ThreadStackSize in units of 1024 bytes, 2^10.
      ThreadStackSize = round_to(ThreadStackSize, K) >> 10;
    // -Xoss
    } else if (match_option(option, "-Xoss", &tail)) {
	  // HotSpot does not have separate native and Java stacks, ignore silently for compatibility
    // -Xmaxjitcodesize
    } else if (match_option(option, "-Xmaxjitcodesize", &tail)) {
      jlong long_ReservedCodeCacheSize = 0;
      if (!atomll(tail, &long_ReservedCodeCacheSize) || !check_memory_size(long_ReservedCodeCacheSize, InitialCodeCacheSize)) {
        jio_fprintf(stderr, "Invalid maximum code cache size: %s\n", option->optionString);
        return JNI_EINVAL;
      }
      ReservedCodeCacheSize = (int) long_ReservedCodeCacheSize;
    // -green
    } else if (match_option(option, "-green", &tail)) {
      jio_fprintf(stderr, "Green threads support not available\n");
	  return JNI_EINVAL;
    // -native
    } else if (match_option(option, "-native", &tail)) {
	  // HotSpot always uses native threads, ignore silently for compatibility
    // -Xsqnopause
    } else if (match_option(option, "-Xsqnopause", &tail)) {
	  // EVM option, ignore silently for compatibility
    // -Xrs
    } else if (match_option(option, "-Xrs", &tail)) {
	  // Classic/EVM option, new functionality
      ReduceSignalUsage = true;
    // -Xoptimize
    } else if (match_option(option, "-Xoptimize", &tail)) {
	  // EVM option, ignore silently for compatibility
    // -Xprof
    } else if (match_option(option, "-Xprof", &tail)) {
      _has_profile = true;
    // -Xaprof
    } else if (match_option(option, "-Xaprof", &tail)) {
      _has_alloc_profile = true;
    // -Xconcurrentio
    } else if (match_option(option, "-Xconcurrentio", &tail)) {
      UseLWPSynchronization = true;
      BackgroundCompilation = false;
      CIDynamicCompilePriority = false;
      DeferThrSuspendLoopCount = 1;
      UseTLE = false;
      NewSizeThreadIncrease  = 16 * K;  // 20Kb per thread added to new generation

      // -Xinternalversion
    } else if (match_option(option, "-Xinternalversion", &tail)) {
      jio_fprintf(stdout, "%s\n", VM_Version::internal_vm_info_string());
      vm_exit(0);
#ifndef PRODUCT
    // -Xprintflags
    } else if (match_option(option, "-Xprintflags", &tail)) {
      DebugFlags::printFlags();
      vm_exit(0);
#endif
    // -D
    } else if (match_option(option, "-D", &tail)) {      
      if (!add_property(tail)) {
        return JNI_ENOMEM;
      }
    // -Xint
    } else if (match_option(option, "-Xint", &tail)) {
	  set_int_mode_flags();
    // -Xmixed
    } else if (match_option(option, "-Xmixed", &tail)) {
	  set_mixed_mode_flags();
    // -Xcomp
    } else if (match_option(option, "-Xcomp", &tail)) {
      // for testing the compiler; turn off all flags that inhibit compilation
	  set_comp_mode_flags();
    // -Xverify
    } else if (match_option(option, "-Xverify", &tail)) {      
      if (strcmp(tail, ":all") == 0 || strcmp(tail, "") == 0) {
        BytecodeVerificationLocal  = true;
        BytecodeVerificationRemote = true;
      } else if (strcmp(tail, ":remote") == 0) {
        BytecodeVerificationLocal  = false;
        BytecodeVerificationRemote = true;
      } else if (strcmp(tail, ":none") == 0) {
        BytecodeVerificationLocal  = false;
        BytecodeVerificationRemote = false;
      } else {
	if (os::obsolete_option(option)) {
	  if (!args->ignoreUnrecognized)
	    jio_fprintf(stderr, "Obsolete verification option: %s\n", option->optionString);
	} else {
	  jio_fprintf(stderr, "Illegal verification option: %s\n", option->optionString);
	  return JNI_EINVAL;
	}
      }
    // -Xdebug
    } else if (match_option(option, "-Xdebug", &tail)) {
      jvmdi::enable();
      set_int_mode_flags();
      UseFastEmptyMethods = false;
    // -Xnoagent 
    } else if (match_option(option, "-Xnoagent", &tail)) {    
      // For compatibility with classic. HotSpot refuses to load the old style agent.dll.
    } else if (match_option(option, "-Xboundthreads", &tail)) {    
      // Bind user level threads to kernel threads (Solaris only)
      UseBoundThreads = true;
    // JNI hooks
    } else if (match_option(option, "vfprintf", &tail)) {    
      _vfprintf_hook = CAST_TO_FN_PTR(vfprintf_hook_t, option->extraInfo);
    } else if (match_option(option, "exit", &tail)) {    
      _exit_hook = CAST_TO_FN_PTR(exit_hook_t, option->extraInfo);
    } else if (match_option(option, "abort", &tail)) {    
      _abort_hook = CAST_TO_FN_PTR(abort_hook_t, option->extraInfo);
    // -XX:+AggressiveHeap
    } else if (match_option(option, "-XX:+AggressiveHeap", &tail)) {
      _initial_heap_size = 3900u * M;
      _max_heap_size = 3900u * M;
      NewSize = 3072u * M;
      MaxNewSize = 3072u * M;
      ResizeTLE = false;
      TLESize = 256 * K;
      MaxLiveObjectEvacuationRatio = 0;
    // -XX:xxxx
    } else if (match_option(option, "-XX:", &tail)) {
      if (strncmp(tail, "Flags=", strlen("Flags=")) != 0) {
        if (!process_argument(tail, args->ignoreUnrecognized)) {
          return JNI_EINVAL;
        }
      }
    // Unknown option
    } else if (os::obsolete_option(option)) {
      if (!args->ignoreUnrecognized)
         jio_fprintf(stderr, "Obsolete option: %s\n", option->optionString);
    } else if (!args->ignoreUnrecognized) {
      jio_fprintf(stderr, "Unrecognized option: %s\n", option->optionString);
      return JNI_ERR;
    }
  }
  if (!check_vm_args_consistency())
     return JNI_ERR;
  return JNI_OK;
}


jint Arguments::parse_java_options_environment_variable() {
  char buffer[1024];
  if (os::getenv("_JAVA_OPTIONS", buffer, sizeof(buffer))) {
    const int N_MAX_OPTIONS = 32;
    // Construct JavaVMOption array
    JavaVMOption options[N_MAX_OPTIONS];
    jio_fprintf(stderr, "Picked up _JAVA_OPTIONS: %s\n", buffer);
    char* c = buffer;
    int i;
    for (i = 0; i < N_MAX_OPTIONS;) {
      // Skip whitespace
      while (isspace(*c)) c++;
      if (*c == 0) break;
      // Fill in option
      options[i++].optionString = c;
      while (*c != 0 && !isspace(*c)) c++;
      // Check for end
      if (*c == 0) break;
      // Zero terminate option
      *c++ = 0;
    }
    // Construct JavaVMInitArgs structure
    JavaVMInitArgs vm_args;
    vm_args.version = JNI_VERSION_1_2;
    vm_args.options = options;
    vm_args.nOptions = i;
    vm_args.ignoreUnrecognized = false;
    return(parse_vm_init_args(&vm_args));
  }
  return JNI_OK;
}


// Parse entry point called from JNI_CreateJavaVM

jint Arguments::parse(JavaVMInitArgs* args) {
  // Set default sysclasspath
  assert(_sysclasspath == NULL, "Should only parse arguments once");
  _sysclasspath = os::get_system_properties()->sysclasspath;

  // Remaining part of option string
  char* tail;
  
  // If flag "-XX:Flags=flags-file" is used it will be the first option to be processed.
  bool settings_file_specified = false;
  for (int index = 0; index < args->nOptions; index++) {
    JavaVMOption *option = args->options + index;
    if (match_option(option, "-XX:Flags=", &tail)) {
      if (!process_settings_file(tail, true, args->ignoreUnrecognized)) {
        return JNI_EINVAL;
      }
      settings_file_specified = true;
    } 
  }

  // Parse default .hotspotrc settings file
  if (!settings_file_specified) {
    if (!process_settings_file(".hotspotrc", false, args->ignoreUnrecognized)) {
      return JNI_EINVAL;
    }
  }

  // Parse JavaVMInitArgs structure passed in
  jint result = parse_vm_init_args(args);
  if (result != JNI_OK) {
    return result;
  }

  // Parse _JAVA_OPTIONS environment variable (if present) (mimics classic VM)
  result = parse_java_options_environment_variable();
  if (result != JNI_OK) {
    return result;
  }

  // Parse JAVA_COMPILER environment variable (if present) (mimics classic VM)
  parse_java_compiler_environment_variable();

#ifndef PRODUCT
  if (TraceBytecodesAt != 0) {
    TraceBytecodes = true;
  }
  if (CountCompiledCalls) {
#ifdef CORE
    warning("CountCalls is set but may not work (no invocation counters in CORE build)");
#else
    if (UseCounterDecay) {
      warning("UseCounterDecay disabled because CountCalls is set");
      UseCounterDecay = false;
    }
#endif // CORE
  }
  
  if (PrintBCIHistogram) {
    if (_mode != _int) 
      warning("PrintBCIHistogram is set, but mode is not interpreter-only.\n"
              "Compiled bci's will not be recorded (use -Xint to fix mode)");
    if (ClassUnloading) {
      warning("Disabling ClassUnloading in order to print bci histogram");
      ClassUnloading = false;
    }
  }
#endif // PRODUCT

  // Interactions between breakpoints and bytecode rewriting.
  if (jvmdi::enabled() && UseFastBreakpoints) {
    RewriteFrequentPairs = false;
  }

  // For extra robustness
  CORE_ONLY(set_int_mode_flags());
  return JNI_OK;
}

// %%% Should be Arguments::GetCheckCompileOnly(), Arguments::CompileMethod().

bool CheckCompileOnly () { return Arguments::GetCheckCompileOnly();}

bool CompileMethod (char* className, char* methodName) {
  return Arguments::CompileMethod( className, methodName);
}

int Arguments::library_count() {
  UserProperty* iter = libraries();
  int count = 0;
  while(iter != NULL) {
    count++;
    iter = iter->next();
  }

  return count;
}

char* Arguments::library_name(int index) {
  UserProperty* iter = libraries();
  int count = 0;
  char* ret_val = NULL;

  while(iter != NULL) {
    if(count >= index) {
      ret_val = iter->key();
      break;
    }
    count++;
    iter = iter->next();
  }

  return ret_val;
}

char* Arguments::library_options(int index) {
  UserProperty* iter = libraries();
  int count = 0;
  char* ret_val = NULL;

  while(iter != NULL) {
    if(count >= index) {
      ret_val = iter->value();
      break;
    }
    count++;
    iter = iter->next();
  }

  return ret_val;
}

void Arguments::add_init_library(char* name,char* options) {
  UserProperty* new_library = new UserProperty(name, options);
  UserProperty* iter = libraries();

  set_init_libraries_at_startup(true);

  if (iter == NULL) {
    set_libraries(new_library);
  } else {
    while (iter->next() != NULL) {
      iter = iter->next();
    }
    iter->set_next(new_library);
  }
}

