/*
 * @(#)arguments.hpp	1.40 01/02/09 14:47:30
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Arguments parses the command line and recognizes options

// Invocation API hook typedefs (these should really be defined in jni.hpp)
extern "C" {
  typedef void (JNICALL *abort_hook_t)(void);
  typedef void (JNICALL *exit_hook_t)(jint code);
  typedef jint (JNICALL *vfprintf_hook_t)(FILE *fp, const char *format, va_list args);
}

// Element describing user defined property using -Dkey=value flags

class UserProperty: public CHeapObj {
 private:
  char*         _key;
  char*         _value;
  UserProperty* _next;

 public:
  // Accessors
  char* key() const                         { return _key; }
  char* value() const                       { return _value; }
  UserProperty* next() const                { return _next; }
  void set_next(UserProperty* next)         { _next = next; }

  // Constructor
  UserProperty(char* key, char* value)      { _key = key; _value = value; _next = NULL; }
};

class Arguments : AllStatic {
 public:
  // Operation modi
  enum Mode {
    _int,       // corresponds to -Xint
    _mixed,     // corresponds to -Xmixed
    _comp       // corresponds to -Xcomp
  };

 private:  
  // Property list 
  static UserProperty* _user_properties;

  // Option flags       
  static bool   _has_profile;  
  static bool   _has_alloc_profile;
  static char*  _sysclasspath;
  static bool   _oldjava;
  static size_t    _initial_heap_size;
  static size_t    _max_heap_size;

  // -Xrun arguments
  static bool _init_libraries_at_startup;
  static UserProperty* _libraries;
  static UserProperty* libraries()                    { return _libraries; }
  static void set_libraries(UserProperty* library)    { _libraries = library; }
  static void set_init_libraries_at_startup(bool arg) { _init_libraries_at_startup = arg; }
  static void add_init_library(char* name,char* options);

  // Operation modi
  static Mode   _mode;
  static void set_int_mode_flags();
  static void set_mixed_mode_flags();
  static void set_comp_mode_flags();

  // Invocation API hooks
  static abort_hook_t     _abort_hook;
  static exit_hook_t      _exit_hook;
  static vfprintf_hook_t  _vfprintf_hook;

  // User propertires
  static bool add_property(char* prop);

  // Argument parsing
  static void do_pd_flag_adjustments();
  static bool parse_argument(char* arg);
  static bool process_argument(char* arg, bool ignore_unrecognized);
  static void process_java_compiler_argument(char* arg);
  static jint parse_java_options_environment_variable();
  static void parse_java_compiler_environment_variable();
  static jint parse_vm_init_args(JavaVMInitArgs* args);

  static bool methodExists(
    char* className, char* methodName, 
    int classesNum, char** classes, bool* allMethods,
    int methodsNum, char** methods, bool* allClasses
  );

  static void parseOnlyLine(
    char* line,
    short* classesNum, short* classesMax, char*** classes, bool** allMethods,
    short* methodsNum, short* methodsMax, char*** methods, bool** allClasses
  ); 

  static short	CompileOnlyClassesNum;
  static short	CompileOnlyClassesMax;
  static char**	CompileOnlyClasses;
  static bool*	CompileOnlyAllMethods;

  static short	CompileOnlyMethodsNum;
  static short	CompileOnlyMethodsMax;
  static char**	CompileOnlyMethods;
  static bool*	CompileOnlyAllClasses;
  
  static short	InterpretOnlyClassesNum;
  static short	InterpretOnlyClassesMax;
  static char**	InterpretOnlyClasses;
  static bool*	InterpretOnlyAllMethods;

  static bool	CheckCompileOnly;

 public:
  // Parses the arguments
  static jint parse(JavaVMInitArgs* args);
  // Check consistecy or otherwise of VM argument settings
  static bool check_vm_args_consistency();
  // Used by os_solaris
  static bool process_settings_file(const char* file_name, bool should_exist, bool ignore_unrecognized);
    
  // -Dkey=value flags
  static UserProperty*  user_properties()   { return _user_properties; }

  // -Xsysclasspath, if not speficied default sysclasspath
  static char* sysclasspath()               { return _sysclasspath; }

  // -Xoldjava flag
  static bool oldjava()                     { return _oldjava; }

  // -Xprof/-Xaprof
  static bool has_profile()                 { return _has_profile; }
  static bool has_alloc_profile()           { return _has_alloc_profile; }

  // -Xms , -Xmx
  static size_t  initial_heap_size()           { return _initial_heap_size; }
  static size_t  max_heap_size()               { return _max_heap_size; }

  // -Xrun
  static int library_count();
  static char* library_name(int index);
  static char* library_options(int index);
  static bool init_libraries_at_startup()   { return _init_libraries_at_startup; }

  // abort, exit, vfprintf hooks
  static abort_hook_t    abort_hook()          { return _abort_hook; }
  static exit_hook_t     exit_hook()           { return _exit_hook; }
  static vfprintf_hook_t vfprintf_hook()       { return _vfprintf_hook; }

  static bool GetCheckCompileOnly ()	    { return CheckCompileOnly; }

  static bool CompileMethod(char* className, char* methodName) {
    return
      methodExists(
        className, methodName, 
        CompileOnlyClassesNum, CompileOnlyClasses, CompileOnlyAllMethods,
        CompileOnlyMethodsNum, CompileOnlyMethods, CompileOnlyAllClasses
      );
  }

  // Operation modi
  static Mode mode()                        { return _mode; }
};

