// @(#)atomic.hpp	1.5 01/04/23
// 
// Copyright 1993-2001 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

class atomic : AllStatic {
 public:
  // Support for atomic 32-bit word operations

  // Atomically add to a 32-bit location, return updated value
  static inline jint add(jint* dest, jint add_value);
  // Atomically increment a 32-bit location
  static inline void increment(jint* dest);
  // Atomically decrement a 32-bit location
  static inline void decrement(jint* dest);
  // Performs atomic exchange of *dest with exchange_value. Returns old prior value of *dest.
  static inline jint exchange(jint* dest, jint exchange_value);
  // Atomically store to a 32-bit location
  static inline void store(jint* dest, jint store_value);
  // Performs atomic compare of *dest and compare_value, and exchanges *dest with exchange_value
  // if the comparison succeded. Returns prior value of *dest.
  static inline jint compare_and_exchange(jint* dest, jint exchange_value, jint compare_value);

  // Support for atomic machine word operations

  // Atomically add to a machine location, return updated value
  static inline intptr_t add_ptr(intptr_t* dest, intptr_t add_value);
  // Atomically increment a machine location
  static inline void increment_ptr(intptr_t* dest);
  // Atomically decrement a machine location
  static inline void decrement_ptr(intptr_t* dest);
  // Performs atomic exchange of *dest with exchange_value. Returns prior value of *dest.
  static inline intptr_t exchange_ptr(intptr_t* dest, intptr_t exchange_value);
  // Atomically store to a machine location
  static inline void store_ptr(intptr_t* dest, intptr_t store_value);
  // Performs atomic compare of *dest and compare_value, and exchanges *dest with exchange_value
  // if the comparison succeded. Returns prior value of *dest.
  static inline intptr_t compare_and_exchange_ptr(intptr_t* dest, intptr_t exchange_value, intptr_t compare_value);
  // Performs both read barrier and write barrier
  static inline void membar();

  // The next two methods must not be declared inline - for details see
  // src/os_cpu/solaris_sparc/vm/prefetch_solaris_sparc.il

  // Prefetch anticipating read; must not fault, semantically a no-op
  static void prefetch_read(void* address);

  // Prefetch anticipating write; must not fault, sematically a no-op
  static void prefetch_write(void* address);
};

class AtomicCounter VALUE_OBJ_CLASS_SPEC {
public:
  AtomicCounter(jint* dest) : _dest(dest) {
    atomic::increment(_dest);
  }
  ~AtomicCounter() {
    atomic::decrement(_dest);
  }
private:
  jint* _dest;
};

class AtomicPtrCounter VALUE_OBJ_CLASS_SPEC {
public:
  AtomicPtrCounter(intptr_t* dest) : _dest(dest) {
    atomic::increment_ptr(_dest);
  }
  ~AtomicPtrCounter() {
    atomic::decrement_ptr(_dest);
  }
private:
  intptr_t* _dest;
};

