/*
 * @(#)classFileParser.hpp	1.49 00/04/26 20:56:57
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Parser for for .class files
//
// The bytes describing the class file structure is read from a Stream object


class ClassFileParser VALUE_OBJ_CLASS_SPEC {
 private:
  ClassFileStream* _stream;              // Actual input stream

  // Accessors
  ClassFileStream* stream()                        { return _stream; }
  void set_stream(ClassFileStream* st)             { _stream = st; }

  // Constant pool parsing
  void parse_constant_pool_utf8_entry              (constantPoolHandle cp, int index, TRAPS);
  void parse_constant_pool_integer_entry           (constantPoolHandle cp, int index);
  void parse_constant_pool_float_entry             (constantPoolHandle cp, int index);
  void parse_constant_pool_long_entry              (constantPoolHandle cp, int index);
  void parse_constant_pool_double_entry            (constantPoolHandle cp, int index);
  void parse_constant_pool_class_entry             (constantPoolHandle cp, int index);
  void parse_constant_pool_string_entry            (constantPoolHandle cp, int index);
  void parse_constant_pool_fieldref_entry          (constantPoolHandle cp, int index);
  void parse_constant_pool_methodref_entry         (constantPoolHandle cp, int index);
  void parse_constant_pool_interfacemethodref_entry(constantPoolHandle cp, int index);
  void parse_constant_pool_nameandtype_entry       (constantPoolHandle cp, int index);
  void parse_constant_pool_entry                   (constantPoolHandle cp, int* index, TRAPS);
  constantPoolHandle parse_constant_pool           (TRAPS);

  // Interface parsing
  objArrayHandle parse_interfaces(constantPoolHandle cp, Handle class_loader, Handle protection_domain, TRAPS);

  // Field parsing
  void parse_field_attributes(constantPoolHandle cp, u2* constantvalue_index_addr, bool* is_synthetic_addr, TRAPS);
  typeArrayHandle parse_fields(constantPoolHandle cp, int* static_oop_count, int* nonstatic_oop_count, TRAPS);

  // Method parsing
  methodHandle parse_method                             (constantPoolHandle cp, AccessFlags* promoted_flags, TRAPS);
  objArrayHandle parse_methods                          (constantPoolHandle cp, AccessFlags* promoted_flags, TRAPS);
  typeArrayHandle sort_methods                          (objArrayHandle methods, TRAPS);

  // Classfile attribute parsing
  void parse_classfile_sourcefile_attribute   (constantPoolHandle cp, instanceKlassHandle k, TRAPS);
  void parse_classfile_inner_classes_attribute(constantPoolHandle cp, instanceKlassHandle k, TRAPS);
  void parse_classfile_attributes             (constantPoolHandle cp, instanceKlassHandle k, TRAPS);
  void parse_classfile_synthetic_attribute    (constantPoolHandle cp, instanceKlassHandle k, TRAPS);
  
  // Final setup
  int  compute_oop_map_size(instanceKlassHandle super, int nonstatic_oop_count, int first_nonstatic_oop_offset);
  void fill_oop_maps(instanceKlassHandle k, int nonstatic_oop_count, int first_nonstatic_oop_offset);
  void set_precomputed_flags(instanceKlassHandle k);
  objArrayHandle compute_transitive_interfaces(instanceKlassHandle super, objArrayHandle local_ifs, TRAPS);

 public:
  // Constructor
  ClassFileParser(ClassFileStream* st) { set_stream(st); }

  // Parse .class file and return new klassOop. The klassOop is not hooked up to the
  // system dictionary or any other structures, so a .class file can be loaded several
  // times if desired. The system dictionary hookup is done by ClassLoader.
  instanceKlassHandle parseClassFile(symbolHandle name, Handle class_loader, Handle protection_domain, TRAPS);

  // Verifier checks
  static void check_super_class_access(instanceKlassHandle this_klass, TRAPS);
  static void check_super_interface_access(instanceKlassHandle this_klass, TRAPS);
  static void check_final_method_override(instanceKlassHandle this_klass, TRAPS);
  static void check_illegal_static_method(instanceKlassHandle this_klass, TRAPS);
};
