/*
 * @(#)fieldDescriptor.cpp	1.37 00/06/28 11:11:22
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
#include "incls/_fieldDescriptor.cpp.incl"


oop fieldDescriptor::loader() const {
  return instanceKlass::cast(_cp->pool_holder())->class_loader(); 
}
                   
constantTag fieldDescriptor::initial_value_tag() const {
  return constants()->tag_at(_initial_value_index);
}

jint fieldDescriptor::int_initial_value() const {
  return constants()->int_at(_initial_value_index);
}

jlong fieldDescriptor::long_initial_value() const {
  return constants()->long_at(_initial_value_index);
}

jfloat fieldDescriptor::float_initial_value() const {
  return constants()->float_at(_initial_value_index);
}

jdouble fieldDescriptor::double_initial_value() const {
  return constants()->double_at(_initial_value_index);
}

oop fieldDescriptor::string_initial_value(TRAPS) const {
  return constants()->string_at(_initial_value_index, CHECK_0);
}

void fieldDescriptor::initialize(klassOop k, int index) {    
  instanceKlass* ik = instanceKlass::cast(k);
  _cp = ik->constants();
  typeArrayOop fields = ik->fields();

  assert(fields->length() % 5 == 0, "Illegal size of field array");
  assert(fields->length() >= index + 5, "Illegal size of field array");

  _access_flags.set_field_flags(fields->ushort_at(index));
  _name_index = fields->ushort_at(index + 1);
  _signature_index = fields->ushort_at(index + 2);
  _initial_value_index = fields->ushort_at(index + 3);
  guarantee(_name_index != 0 && _signature_index != 0, "bad constant pool index for fieldDescriptor");
  _offset = fields->ushort_at(index + 4);
  _index = index;
}

#ifndef PRODUCT

void fieldDescriptor::print_on(outputStream* st) const {
  _access_flags.print_on(st);
  constants()->symbol_at(_name_index)->print_value_on(st);
  st->print(" ");
  constants()->symbol_at(_signature_index)->print_value_on(st);
  st->print(" @%d ", offset());
  if (WizardMode && has_initial_value()) {
    st->print("(initval ");
    constantTag t = initial_value_tag();
    if (t.is_int()) {
      st->print("int %d)", int_initial_value());
    } else if (t.is_long()){
      st->print_jlong(long_initial_value());
    } else if (t.is_float()){
      st->print("float %f)", float_initial_value());
    } else if (t.is_double()){
      st->print("double %lf)", double_initial_value());
    }
  }
}

void fieldDescriptor::print_on_for(outputStream* st, oop obj) {
  print_on(st);
  BasicType ft = field_type();
  switch (ft) {
    case T_BYTE:
      st->print(" %d", obj->byte_field(offset()));
      break;
    case T_CHAR:
      {
        jchar c = obj->char_field(offset());
        st->print(" %c %d", isprint(c) ? c : ' ', c);
      }
      break;
    case T_DOUBLE:
      st->print(" %lf", obj->double_field(offset()));
      break;
    case T_FLOAT:
      st->print(" %f", obj->float_field(offset()));
      break;
    case T_INT:
      st->print(" %d", obj->int_field(offset()));
      break;
    case T_LONG:
      st->print(" ");
      st->print_jlong(obj->long_field(offset()));
      break;
    case T_SHORT:
      st->print(" %d", obj->short_field(offset()));
      break;
    case T_BOOLEAN:
      st->print(" %s", obj->int_field(offset()) ? "true" : "false");
      break;
    case T_ARRAY:
      st->print(" ");
      obj->obj_field(offset())->print_value_on(st);
      break;
    case T_OBJECT:
      st->print(" ");
      obj->obj_field(offset())->print_value_on(st);
      break;
    default:
      ShouldNotReachHere();
      break;
  }
  if (ft == T_LONG || ft == T_DOUBLE) {
    st->print(" (%x %x)", obj->int_field(offset()), obj->int_field(offset()+1));
  } else {
    st->print(" (%x)", obj->int_field(offset()));
  }
}

#endif /* PRODUCT */
