/*
 * @(#)globals.cpp	1.23 01/02/09 14:47:35
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_globals.cpp.incl"


RUNTIME_FLAGS(MATERIALIZE_DEVELOPER_FLAG, MATERIALIZE_PD_DEVELOPER_FLAG, MATERIALIZE_PRODUCT_FLAG, MATERIALIZE_PD_PRODUCT_FLAG)


struct Flag {
  const char *type;
  const char *name;
  void*       addr;
  const char *kind;

  bool is_bool() const { return strcmp(type, "bool") == 0; }
  int get_bool() const { return *((bool*) addr); }
  void set_bool(bool value) { *((bool*) addr) = value; }
  
  bool is_int()  const { return strcmp(type, "int")  == 0; }
  int get_int() const { return *((int*) addr); }
  void set_int(int value) { *((int*) addr) = value; }

  bool is_size_t() const              { return strcmp(type, "size_t") == 0; }
  size_t get_size_t() const           { return *((size_t*) addr); }
  void set_size_t(size_t value) const { *((size_t*) addr) = value; }

  void print_on(outputStream* st) {
    st->print("%5s %-35s = ", type, name);
    if (is_bool()) st->print("%-8s", get_bool() ? "true" : "false");
    if (is_int())  st->print("%-8d", get_int());
    if (is_size_t()) st->print("%-16ul", get_size_t());
    st->print(" %s", kind);
    st->cr();
  }
};

#define RUNTIME_PRODUCT_FLAG_STRUCT(type, name, value, doc) { #type, XSTR(name), &name, "{product}" },
#define RUNTIME_PD_PRODUCT_FLAG_STRUCT(type, name, doc)     { #type, XSTR(name), &name, "{pd product}" },
#ifdef PRODUCT
  #define RUNTIME_DEVELOP_FLAG_STRUCT(type, name, value, doc) /* flag is constant */ 
  #define RUNTIME_PD_DEVELOP_FLAG_STRUCT(type, name, doc)     /* flag is constant */ 
#else
  #define RUNTIME_DEVELOP_FLAG_STRUCT(type, name, value, doc) { #type, XSTR(name), &name, "" },
  #define RUNTIME_PD_DEVELOP_FLAG_STRUCT(type, name, doc)     { #type, XSTR(name), &name, "{pd}" }, 
#endif

#define C1_PRODUCT_FLAG_STRUCT(type, name, value, doc) { #type, XSTR(name), &name, "{C1 product}" },
#define C1_PD_PRODUCT_FLAG_STRUCT(type, name, doc)     { #type, XSTR(name), &name, "{C1 pd product}" },
#ifdef PRODUCT
  #define C1_DEVELOP_FLAG_STRUCT(type, name, value, doc) /* flag is constant */ 
  #define C1_PD_DEVELOP_FLAG_STRUCT(type, name, doc)     /* flag is constant */ 
#else
  #define C1_DEVELOP_FLAG_STRUCT(type, name, value, doc) { #type, XSTR(name), &name, "{C1}" },
  #define C1_PD_DEVELOP_FLAG_STRUCT(type, name, doc)     { #type, XSTR(name), &name, "{C1 pd}" }, 
#endif


#define C2_PRODUCT_FLAG_STRUCT(type, name, value, doc) { #type, XSTR(name), &name, "{C2 product}" },
#define C2_PD_PRODUCT_FLAG_STRUCT(type, name, doc)     { #type, XSTR(name), &name, "{C2 pd product}" },
#ifdef PRODUCT
  #define C2_DEVELOP_FLAG_STRUCT(type, name, value, doc) /* flag is constant */ 
  #define C2_PD_DEVELOP_FLAG_STRUCT(type, name, doc)     /* flag is constant */ 
#else
  #define C2_DEVELOP_FLAG_STRUCT(type, name, value, doc) { #type, XSTR(name), &name, "{C2}" },
  #define C2_PD_DEVELOP_FLAG_STRUCT(type, name, doc)     { #type, XSTR(name), &name, "{C2 pd}" }, 
#endif


static Flag flagTable[] = {
 RUNTIME_FLAGS(RUNTIME_DEVELOP_FLAG_STRUCT, RUNTIME_PD_DEVELOP_FLAG_STRUCT, RUNTIME_PRODUCT_FLAG_STRUCT, RUNTIME_PD_PRODUCT_FLAG_STRUCT)
#ifdef COMPILER1
 C1_FLAGS(C1_DEVELOP_FLAG_STRUCT, C1_PD_DEVELOP_FLAG_STRUCT, C1_PRODUCT_FLAG_STRUCT, C1_PD_PRODUCT_FLAG_STRUCT)
#endif
#ifdef COMPILER2
 C2_FLAGS(C2_DEVELOP_FLAG_STRUCT, C2_PD_DEVELOP_FLAG_STRUCT, C2_PRODUCT_FLAG_STRUCT, C2_PD_PRODUCT_FLAG_STRUCT)
#endif
 {0, NULL, NULL}
};

inline bool str_equal(const char* s, char* q, int len) {
  // s is null terminated, q is not!
  if (strlen(s) != (unsigned int) len) return false;
  return strncmp(s, q, len) == 0;
}

static Flag* find_flag(char* name, int length) {
  for (Flag* current = &flagTable[0]; current->name; current++) {
    if (str_equal(current->name, name, length)) {
      return current;
    }
  }
  return NULL;
}



bool DebugFlags::boolAt(char* name, int len, bool* value) {
  Flag* result = find_flag(name, len);
  if (result == NULL) return NULL;
  if (!result->is_bool()) return NULL;   
  *value = result->get_bool();
  return true;
}

bool DebugFlags::boolAtPut(char* name, int len, bool* value) {
  Flag* result = find_flag(name, len);
  if (result == NULL) return NULL;
  if (!result->is_bool()) return NULL;   
  bool old_value = result->get_bool();
  result->set_bool(*value); 
  *value = old_value;
  return true;
}


bool DebugFlags::intAt(char* name, int len, int* value) {
  Flag* result = find_flag(name, len);
  if (result == NULL) return NULL;
  if (!result->is_int()) return NULL;   
  *value = result->get_int();
  return true;
}

bool DebugFlags::intAtPut(char* name, int len, int* value) {
  Flag* result = find_flag(name, len);
  if (result == NULL) return NULL;
  if (!result->is_int()) return NULL;   
  int old_value = result->get_int();
  result->set_int(*value); 
  *value = old_value;
  return true;
}

bool DebugFlags::size_tAt(char* name, int len, size_t* value) {
  Flag* result = find_flag(name, len);
  if (result == NULL) return false;
  if (!result->is_size_t()) return false;
  *value = result->get_size_t();
  return true;
}

bool DebugFlags::size_tAtPut(char* name, int len, size_t* value) {
  Flag* result = find_flag(name, len);
  if (result == NULL) return false;
  if (!result->is_size_t()) return false;
  size_t old_value = result->get_size_t();
  result->set_size_t(*value);
  *value = old_value;
  return true;
}


#ifndef PRODUCT


void DebugFlags::verify() {
  assert(Arguments::check_vm_args_consistency(), "Some flag settings conflict");
}

extern "C" int compare_flags(const void* void_a, const void* void_b) {
  return strcmp((*((Flag**) void_a))->name, (*((Flag**) void_b))->name);
}

void DebugFlags::printFlags() {
  // Print the flags sorted by name
  // note: this method is called before the thread structure is in place
  //       which means resource allocation cannot be used.

  // Compute size
  int length= 0;
  while (flagTable[length].name != NULL) length++;

  // Sort
  Flag** array = NEW_C_HEAP_ARRAY(Flag*, length);
  for (int index = 0; index < length; index++) {
    array[index] = &flagTable[index];
  }
  qsort(array, length, sizeof(Flag*), compare_flags);

  // Print
  tty->print_cr("[Global flags]");
  for (int i = 0; i < length; i++) {
    array[i]->print_on(tty);
  }
  FREE_C_HEAP_ARRAY(Flag*, array);
}

#endif

