/*
 * @(#)globals.hpp	1.620 01/01/12 16:17:54
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// debug flags control various aspects of the VM and are global accessible

// use FlagSetting to temporarily change some debug flag
// e.g. FlagSetting fs(DebugThisAndThat, true);   
// restored to previous value upon leaving scope
class FlagSetting {
  bool val;
  bool* flag;
 public:
  FlagSetting(bool& fl, bool newValue) { flag = &fl; val = fl; fl = newValue; }
  ~FlagSetting()                       { *flag = val; }
};


class CounterSetting {
  int* counter;
 public:
  CounterSetting(int* cnt) { counter = cnt; (*counter)++; }
  ~CounterSetting()        { (*counter)--; }
};


class IntFlagSetting {
  int val;
  int* flag;
 public:
  IntFlagSetting(int& fl, int newValue)   { flag = &fl; val = fl; fl = newValue; }
  ~IntFlagSetting()                       { *flag = val; }
};


class DebugFlags {
 public:
  static bool boolAt(char* name, int len, bool* value);
  static bool boolAt(char* name, bool* value)     { return boolAt(name, strlen(name), value); }

  static bool boolAtPut(char* name, int len, bool* value);
  static bool boolAtPut(char* name, bool* value)  { return boolAtPut(name, strlen(name), value); }

  static bool intAt(char* name, int len, int* value);
  static bool intAt(char* name, int* value)       { return intAt(name, strlen(name), value); }

  static bool intAtPut(char* name, int len, int* value);
  static bool intAtPut(char* name, int* value)    { return intAtPut(name, strlen(name), value); }

  static bool size_tAt(char* name, int len, size_t* value);
  static bool size_tAt(char* name, size_t* value)    { return size_tAt(name, strlen(name), value); }
  static bool size_tAtPut(char* name, int len, size_t* value);
  static bool size_tAtPut(char* name, size_t* value) { return size_tAtPut(name, strlen(name), value); }

  static void printFlags() PRODUCT_RETURN;
  
  static void verify() PRODUCT_RETURN;
};

// use this for flags that are true per default in the debug version but false in the 
// optimized version, and vice versa
#ifdef ASSERT
#define trueInDebug  true
#define falseInDebug false
#else
#define trueInDebug  false
#define falseInDebug true
#endif

// use this for flags that are true per default in the product build
// but false in development builds, and vice versa
#ifdef PRODUCT
#define trueInProduct  true
#define falseInProduct false
#else
#define trueInProduct  false
#define falseInProduct true
#endif

// develop flags are settable / visible only during development and are constant in the PRODUCT version
// product flags are always settable / visible

#define RUNTIME_FLAGS(develop, develop_pd, product, product_pd)             \
                                                                            \
  product(bool, JavaMonitorsInStackTrace, false,                            \
          "Print info. about Java monitor locks when the stacks are dumped")\
									    \
  product(bool, UseISM, false,			                            \
          "Use Intimate Shared Memory (Solaris Only)") 			    \
									    \
  product(bool, UsePermISM, false,		                            \
          "Use ISM that stays in the system (Solaris Only)") 		    \
                                                                            \
  develop(bool, UseRevisedLookupSemantics, true,                            \
          "New revised lookup semantics for kestrel beta")                  \
                                                                            \
  develop(bool, UseNewOSExceptions, true,                                   \
          "Temp. flag for new OS exception tracking on Win32")              \
                                                                            \
  develop(bool, TraceItables, false,                                        \
          "Trace initialization and use of itables")                        \
                                                                            \
  develop(bool, OnStackReplaceALot, false,                                  \
          "Force a lot of OnStack Replacements")                            \
                                                                            \
  develop(bool, TraceRelocator, false,                                      \
          "Trace the bytecode relocator")                                   \
                                                                            \
  develop(bool, TraceLongCompiles, false,                                   \
          "Print out every time compilation is longer than "                \
          "a given threashold")                                             \
                                                                            \
  develop(bool, SafepointALot, false,                                       \
          "Generates a lot of safepoints. Works with "                      \
          "GuaranteedSafepointInterval")                                    \
                                                                            \
  product_pd(bool, BackgroundCompilation,                                   \
          "A thread requesting compilation is not blocked during "          \
          "compilation")                                                    \
                                                                            \
  develop(bool, TraceBackgroundCompilation, false,                          \
          "Trace background compilation")                                   \
                                                                            \
  develop(bool, TraceCompilationStalls, false,                              \
          "Prints out the length of a stall in ms. a thread is stalled "    \
          "due to compilation")                                             \
                                                                            \
  develop(bool, BailoutToInterpreterForThrows, false,                       \
          "Compiled methods which throws/catches exceptions will be "       \
          "deopt and intp.")                                                \
                                                                            \
  develop(bool, NoYieldsInMicrolock, false,                                 \
          "Disable yields in microlock")                                    \
                                                                            \
  develop(bool, TraceOopMapGeneration, false,                               \
          "Shows oopmap generation")                                        \
                                                                            \
  develop(bool, MethodFlushing, true,                                       \
          "Reclamation of zombie and not-entrant methods")                  \
                                                                            \
  develop(bool, VerifyStack, false,                                         \
          "Verify stack of each thread when it is entering a runtime call") \
                                                                            \
  develop(bool, StressDerivedPointers, false,                               \
          "Force scavenge when a derived pointers is detected on stack "    \
          "after rtm call")                                                 \
                                                                            \
  develop(bool, TraceDerivedPointers, false,                                \
          "Trace traversal of derived pointers on stack")                   \
                                                                            \
  develop(bool, TraceCodeBlobStacks, false,                                 \
          "Trace stack-walk of codeblobs")                                  \
                                                                            \
  develop(bool, TraceAdapterGeneration, false,                              \
          "Prints out whih adapters gets generated")                        \
                                                                            \
  product(bool, PrintJNIResolving, false,                                   \
          "Used to implement -v:jni")                                       \
                                                                            \
  develop(bool, PrintRewrites, false,                                       \
          "Print methods that are being rewritten")                         \
                                                                            \
  develop(bool, InlineArrayCopy, true,                                      \
          "inline arraycopy native that is known to be part of "            \
          "base library DLL")                                               \
                                                                            \
  develop(bool, InlineNewInstance, true,                                    \
          "inline Class::newInstance0() native that is known to be part "   \
          "of base library DLL")                                            \
                                                                            \
  develop(bool, InlineObjectHash, true,                                     \
          "inline Object::hashCode() native that is known to be part "      \
          "of base library DLL")                                            \
                                                                            \
  develop(bool, InlineNatives, true,                                        \
          "inline natives that are known to be part of base library DLL")   \
                                                                            \
  develop(bool, InlineMathNatives, true,                                    \
          "inline SinD, CosD, etc.")                                        \
                                                                            \
  develop(bool, ConvertCmpD2CmpF, true,                                     \
          "Convert cmpD to cmpF when one input is constant in float range") \
                                                                            \
  develop(bool, ConvertFloat2IntClipping, true,                             \
          "Convert float2int clipping idiom to integer clipping")           \
                                                                            \
  develop(bool, SpecialStringCompareTo, true,                               \
          "special version of string compareTo")                            \
                                                                            \
  develop(bool, TraceCallFixup, false,                                      \
          "traces all call fixups")                                         \
                                                                            \
  develop(bool, DeoptimizeALot, false,                                      \
          "deoptimize at every exit from the runtime system")               \
                                                                            \
  develop(bool, ZombieALot, false,                                          \
          "creates zombies (non-entrant) at exit from the runt. system")    \
                                                                            \
  develop(bool, WalkStackALot, false,                                       \
          "trace stack (no print) at every exit from the runtime system")   \
                                                                            \
  develop(bool, Debugging, false,                                           \
          "set when executing debug methods in debug.ccp "                  \
          "(to prevent triggering assertions)")                             \
                                                                            \
  develop(bool, StrictSafepointChecks, trueInDebug,                         \
          "Enable strict checks that safepoints cannot happen for threads " \
          "that used No_Safepoint_Verifier")                                \
                                                                            \
  develop(bool, VerifyLastFrame, false,                                     \
          "Verify oops on last frame on entry to VM")                       \
                                                                            \
  develop(bool, TraceHandleAllocation, false,                               \
          "Prints out warnings when suspicious many handles are allocated") \
                                                                            \
  develop(bool, PrintHandleAllocation, false,                               \
          "Prints out a line everytime a new handle is allocated")          \
                                                                            \
  product(bool, UseCompilerSafepoints, true,                                \
          "use safepoints in compiled code")                                \
                                                                            \
  develop(bool, UseStrictCompilerSafepoints, true,                          \
          "force safepoints in all casses in compiled code (SPARC only)")   \
                                                                            \
  develop_pd(bool, UseCallBackInInterpreter,                                \
          "Do call back to reach safepoint when a thread is in "            \
          "the interpreter")                                                \
                                                                            \
  develop(bool, ShowSafepointMsgs, false,                                   \
          "Show msg. about safepoint synch.")                               \
                                                                            \
  develop(bool, SafepointTimeout, falseInProduct,                           \
          "Time out and warn or fail after SafepointTimeoutDelay milliseconds if failed to reach safepoint")\
                                                                            \
  develop(bool, DieOnSafepointTimeout, false,                               \
          "Die upon failure to reach safepoint (see SafepointTimeout)")     \
                                                                            \
  product(bool, MaxFDLimit, true,                                           \
          "Bump the number of file descriptors to max in solaris.")         \
                                                                            \
  develop(bool, LogEvents, trueInDebug,                                     \
          "Enable Event log")                                               \
                                                                            \
  product(bool, BytecodeVerificationRemote, true,                           \
          "Enables the Java bytecode verifier for remote classes")          \
                                                                            \
  product(bool, BytecodeVerificationLocal, false,                           \
          "Enables the Java bytecode verifier for local classes")           \
                                                                            \
  develop(bool, ForceFloatExceptions, trueInDebug,                          \
          "Force exceptions on FP stack under/overflow")                    \
                                                                            \
  develop(bool, SoftMatchFailure, falseInDebug,                             \
          "If the DFA fails to match a node, print a message and bail out") \
                                                                            \
  develop(bool, VerifyStackAtCalls, false,                                  \
          "Verify that the stack pointer is unchanged after calls")         \
                                                                            \
  develop(bool, PrintHeapAllocation, false,                                 \
          "Print heap allocation")                                          \
                                                                            \
  develop(bool, PrintOopAddress, false,                                     \
          "Always print the location of the oop")                           \
                                                                            \
  develop(bool, VerifyScopes, false,                                        \
          "Verify scope descs when nmethod is created")                     \
                                                                            \
  develop(bool, VerifyCodeCacheOften, trueInDebug,                          \
          "Verify compiled-code cache often")                               \
                                                                            \
  develop(bool, ZapDeadCompiledLocals, false,                               \
          "Zap dead locals in compiler frames")                             \
                                                                            \
  develop(bool, ZapDeadLocalsOld, false,                                    \
          "Zap dead locals (old version, zaps all frames when "             \
          "entering the VM")                                                \
                                                                            \
  develop(bool, CheckOopishValues, false,                                   \
          "Warn if value contains oop ( requires ZapDeadLocals)")           \
                                                                            \
  develop(bool, UseMallocOnly, false,                                       \
          "use only malloc/free for allocation (no resource area/arena)")   \
                                                                            \
  develop(bool, PrintMalloc, false,                                         \
          "print all malloc/free calls")                                    \
                                                                            \
  develop(bool, ZapResourceArea, trueInDebug,                               \
          "Zap freed resource/arena space with 0xABABABAB")                 \
                                                                            \
  develop(bool, ZapVMHandleArea, trueInDebug,                               \
          "Zap freed VM handle space with 0xBCBCBCBC")                      \
                                                                            \
  develop(bool, ZapJNIHandleArea, trueInDebug,                              \
          "Zap freed JNI handle space with 0xFEFEFEFE")                     \
                                                                            \
  develop(bool, ZapUnusedHeapArea, trueInDebug,                             \
          "Zap unused heap space with 0xBAADBABE")                          \
                                                                            \
  develop(bool, PrintVMMessages, true,                                      \
          "Print vm messages on console")                                   \
                                                                            \
  develop(bool, PromptForDebug, trueInDebug,                                \
          "Request input at debug macros such as untested "                 \
          "(currently SPARC only)")                                         \
                                                                            \
  develop(bool, Verbose, false,                                             \
          "Prints additional debugging information")                        \
                                                                            \
  develop(bool, WizardMode, false,                                          \
          "Prints much more debugging information")                         \
                                                                            \
  product(bool, ShowMessageBoxOnError, trueInDebug,                         \
          "Show Windows error box on VM fatal")                             \
                                                                            \
  develop(bool, BreakAtWarning, false,                                      \
          "Execute breakpoint upon encountering VM warning")                \
                                                                            \
  develop(bool, UseTopLevelExceptionFilter, true,                           \
          "Filter top level exceptions in OS - turn off for debugging")     \
                                                                            \
  develop(bool, TraceVMOperation, false,                                    \
          "Trace vm operations")                                            \
                                                                            \
  develop(bool, UseFakeTimers, false,                                       \
          "Tells whether the VM should use system time or a fake timer")    \
                                                                            \
  product(bool, PrintCompilation, false,                                    \
          "Print compilations")                                             \
                                                                            \
  develop(bool, PrintAdapterCompilation, false,                             \
          "Print adapter compilations")                                     \
                                                                            \
  develop(bool, PrintCompilation2, false,                                   \
          "Print additional statistics per compilation")                    \
                                                                            \
  develop(bool, PrintAssembly, false,                                       \
          "Print assembly code")                                            \
                                                                            \
  develop(bool, PrintNMethods, false,                                       \
          "Print assembly code for nmethods when generated")                \
                                                                            \
  develop(bool, PrintNativeNMethods, false,                                 \
          "Print assembly code for native nmethods when generated")         \
                                                                            \
  develop(bool, InterceptOSException, false,                                \
          "Starts debugger when an implicit OS (e.g., NULL) "               \
          "exception happens")                                              \
                                                                            \
  develop(bool, PrintCodeCache, false,                                      \
          "Print the compiled_code cache when exiting")                     \
                                                                            \
  develop(bool, PrintCodeCache2, false,                                     \
          "Print detailed info on the compiled_code cache when exiting")    \
                                                                            \
  develop(bool, DumpEventsAtError, false,                                   \
          "Prnt part of event log when crashing")                           \
                                                                            \
  develop(bool, PrintStubCode, false,                                       \
          "Print generated stub code")                                      \
                                                                            \
  product(bool, StackTraceInThrowable, true,                                \
          "Collect backtrace in throwable when exception happens")          \
                                                                            \
  product(bool, ProfilerPrintByteCodeStatistics, false,                     \
          "Prints byte code statictics when dumping profiler output")       \
                                                                            \
  product(bool, ProfilerRecordPC, false,                                    \
          "Collects tick for each 16 byte interval of compiled code")       \
                                                                            \
  product(bool, ProfileVM,  false,                                          \
          "Profiles ticks that fall within VM (either in the VM Thread "    \
          "or VM code called through stubs)")                               \
                                                                            \
  product(bool, ProfileIntervals, false,                                    \
          "Prints profiles for each interval (see ProfileIntervalsTicks)")  \
                                                                            \
  product(bool, ProfilerLight, false,                                       \
          "Tries to avoid Solaris libthread Bug Id: 4207397 at a possible " \
          "precision cost. Needed to profile JDK under Solaris, etc.")      \
                                                                            \
  develop(bool, ProfilerCheckIntervals, false,                              \
          "Collect and print info on spacing of profiler ticks")            \
                                                                            \
  develop(bool, PrintJVMWarnings, false,                                    \
          "Prints warnings for unimplemented JVM functions")                \
                                                                            \
  develop(bool, InitializeJavaLangSystem, true,                             \
          "Initialize java.lang.System - turn off for individual "          \
          "method debugging")                                               \
                                                                            \
  develop(bool, InitializeJavaLangString, true,                             \
          "Initialize java.lang.String - turn off for individual "          \
          "method debugging")                                               \
                                                                            \
  develop(bool, InitializeJavaLangExceptionsErrors, true,                   \
          "Initialize various error and exception classes - turn off for "  \
          "individual method debugging")                                    \
                                                                            \
  develop(bool, RegisterFinalizers, true,                                   \
          "Tells whether the VM should register finalizable objects")       \
                                                                            \
  develop(bool, RegisterReferences, true,                                   \
          "Tells whether the VM should register soft/weak/final/phantom "   \
          "references")                                                     \
                                                                            \
  develop(bool, IgnoreRewrites, false,                                      \
          "Supress rewrites of bytecodes in the oopmap generator. "         \
          "This is unsafe!")                                                \
                                                                            \
  develop(bool, PrintCodeCacheExtension, false,                             \
          "Print extension of code cache")                                  \
                                                                            \
  product(bool, UseThreadPriorities, true,                                  \
          "Use native thread priorities (temporary)")                       \
                                                                            \
  develop(bool, UsePrivilegedStack, true,                                   \
          "Enable the security JVM functions")                              \
                                                                            \
  develop(bool, IEEEPrecision, true,                                        \
          "Enables IEEE precision (for INTEL only)")                        \
                                                                            \
  develop(bool, EagerInitialization, false,                                 \
          "Eagerly initialize classes if possible")                         \
                                                                            \
  develop(bool, ProtectionDomainVerification, true,                         \
          "Verifies protection domain before resolution in system "         \
          "dictionary")                                                     \
                                                                            \
  product(bool, ClassUnloading, true,                                       \
          "Do unloading of classes")                                        \
                                                                            \
  develop(bool, AlwaysUseXchg, false,                                       \
          "Use xchg-based locking even on uniprocessors")                   \
                                                                            \
  develop(bool, DisableStartThread, false,                                  \
          "Disable starting of additional Java threads "                    \
          "(for debugging only)")                                           \
                                                                            \
  develop(bool, MemProfiling, false,                                        \
          "Write memory usage profiling to log file")                       \
                                                                            \
  develop(bool, PrintSystemDictionaryAtExit, false,                         \
          "Prints the system dictionary at exit")                           \
                                                                            \
  develop(bool, UseInterruptibleIO, true,                                   \
          "Interrupt threads using signals (for SOLARIS only)")             \
                                                                            \
  develop(bool, GetThreadState, true,                                       \
          "Compiled code safepoints: get thread state via new libthread "   \
          "interface (if present). Otherwise use interrupts. "              \
          "(SOLARIS only)")                                                 \
                                                                            \
  develop(bool, UseThrGetStateWorkaround, true,                             \
          "Use sanity checks to work around thr_getstate() bug 4335248."    \
          "(SOLARIS only)")                                                 \
                                                                            \
  product_pd(bool, DontYieldALot,                                           \
          "Throw away obvious excess yield calls (for SOLARIS only)")       \
                                                                            \
  product_pd(bool, ConvertSleepToYield,                                     \
          "Converts sleep(0) to thread yield "                              \
          "(may be off for SOLARIS to improve GUI)")                        \
                                                                            \
  product(bool, ConvertYieldToSleep, false,                                 \
          "Converts yield to a sleep of MinSleepInterval to simulate Win32 "\
          "behavior (SOLARIS only)")                                        \
                                                                            \
  product(bool, UseBoundThreads, false,                                     \
          "Bind user level threads to kernel threads (for SOLARIS only)")   \
                                                                            \
  develop(bool, UseDetachedThreads, true,                                   \
          "Use detached threads that are recycled upon termination "        \
          "(for SOLARIS only)")                                             \
                                                                            \
  product(bool, UseLWPSynchronization, false,                               \
          "Use LWP-based instead of libthread-based synchronization "       \
          "(SPARC only)")                                                   \
                                                                            \
  develop(bool, UsePthreads, false,                                         \
          "Use pthread-based instead of libthread-based synchronization "   \
          "(SPARC only)")                                                   \
                                                                            \
  product(bool, AdjustConcurrency, false,                                   \
          "call thr_setconcurrency at thread create time to avoid "         \
          "LWP starvation on MP systems (For Solaris Only)")                \
                                                                            \
  develop(bool, UseNewStylePrintCodes, false,                               \
          "Use new-style method->print_codes")                              \
                                                                            \
  develop(bool, UpdateHotSpotCompilerFileOnError, true,                     \
          "Should the system attempt to update the compiler file when "     \
          "an error occurs?")                                               \
                                                                            \
  product(bool, ReduceSignalUsage, false,                                   \
          "Reduce the use of OS signals in Java and/or the VM")             \
                                                                            \
  develop(bool, ValidateMarkSweep, false,                                   \
          "Do extra validation during MarkSweep collection")                \
                                                                            \
  develop_pd(bool, ShareVtableStubs,                                        \
          "Share vtable stubs (smaller code but worse branch prediction")   \
                                                                            \
  develop(bool, SwitchDecisionTrees, true,                                  \
          "Use decision trees instead of linear compare/branch when "       \
          "compiling switch")                                               \
                                                                            \
  develop(bool, LoadLineNumberTables, true,                                 \
          "Tells whether the class file parser loads line number tables")   \
                                                                            \
  develop(bool, LoadLocalVariableTables, true,                              \
          "Tells whether the class file parser loads local variable tables")\
                                                                            \
  product(bool, ExitOnBadLibThread, true,                                   \
          "Do not continue VM startup if libthread is unpatched "           \
          "(Solaris only)")                                                 \
                                                                            \
  product(bool, AllowUserSignalHandlers, false,                             \
          "Do not complain if the application installs signal handlers "    \
          "(Solaris only)")                                                 \
                                                                            \
  product(bool, UseSignalChaining, true,                                    \
          "Use signal-chaining to invoke signal handlers installed "        \
          "by the application (Solaris & Linux only)")                      \
                                                                            \
  develop(bool, JavaUtilHashTableFix, true,                                 \
          "Performance fix to hashtable")                                   \
                                                                            \
  develop(bool, UseInlineImage, false,                                      \
          "Read inline decisions from a file, reusing these decisions in "  \
          "current run.")                                                   \
                                                                            \
  develop(bool, DumpInlineImage, false,                                     \
          "Write inline decisions to a file, sorted for easy comparison")   \
                                                                            \
  product(bool, UseSpinning, false,                                         \
          "Use spinning in monitor inflation and before entry")             \
                                                                            \
  product(bool, PreSpinYield, false,                                        \
          "Yield before inner spinning loop")                               \
                                                                            \
  product(bool, PostSpinYield, true,                                        \
          "Yield after inner spinning loop")                                \
                                                                            \
  /* tracing */                                                             \
                                                                            \
  develop(bool, TraceLookup, false,                                         \
          "Trace lookups")                                                  \
                                                                            \
  develop(bool, TraceRuntimeLookup, false,                                  \
          "Trace run-time lookups (may cause compilation)")                 \
                                                                            \
  develop(bool, TraceRuntimeCalls, false,                                   \
          "Trace run-time calls")                                           \
                                                                            \
  develop(bool, TraceJVMPI, false,                                          \
          "Trace JVMPI")                                                    \
                                                                            \
  develop(bool, TraceJNICalls, false,                                       \
          "Trace JNI calls")                                                \
                                                                            \
  develop(bool, TraceJVMCalls, false,                                       \
          "Trace JVM calls")                                                \
                                                                            \
  develop(bool, TraceJVMDICalls, false,                                     \
          "Trace JVMDI calls")                                              \
                                                                            \
  develop(bool, StopOnJVMDIErrorReturn, false,                              \
          "Stop if JVMDI call returns with error")                          \
                                                                            \
  develop(bool, TraceJNIHandleAllocation, false,                            \
          "Trace allocation/deallocation of JNI handle blocks")             \
                                                                            \
  develop(bool, TraceThreadEvents, false,                                   \
          "Trace all thread events")                                        \
                                                                            \
  develop(bool, TraceBytecodes, false,                                      \
          "Trace bytecode execution")                                       \
                                                                            \
  develop(bool, TraceMethods, false,                                        \
          "Trace (compiled and interpreted) method entries and exits")      \
                                                                            \
  develop(bool, TraceClassInitialization, false,                            \
          "Trace class initialization")                                     \
                                                                            \
  develop(bool, TraceExceptions, false,                                     \
          "Trace exceptions")                                               \
                                                                            \
  develop(bool, VMStackTraceAtException, false,                             \
          "Print VM stack trace at exceptions")                             \
                                                                            \
  develop(bool, VMStackTraceAtNullException, false,                         \
          "Print VM stack trace at NullPointerExceptions")                  \
                                                                            \
  develop(bool, TraceICs, false,                                            \
          "Trace inline cache changes")                                     \
                                                                            \
  develop(bool, TraceICmisses, false,                                       \
          "Trace inline cache misses")                                      \
                                                                            \
  develop(bool, TraceInvocationCounterOverflow, false,                      \
          "Trace method invocation counter overflow")                       \
                                                                            \
  develop(bool, TraceInlineCacheClearing, false,                            \
          "Trace clearing of inline caches in nmethods")                    \
                                                                            \
  develop(bool, TraceDependencies, false,                                   \
          "Trace dependencies")                                             \
                                                                            \
  develop(bool, TraceNewOopMapGeneration, false,                            \
          "Trace OopMapGeneration")                                         \
                                                                            \
  develop(bool, TraceNewOopMapGenerationDetailed, false,                    \
          "Trace OopMapGeneration: print detailed cell states")             \
                                                                            \
  develop(bool, TimeOopMap, false,                                          \
          "Time calls to GenerateOopMap::compute_map() in sum")             \
                                                                            \
  develop(bool, TimeOopMap2, false,                                         \
          "Time calls to GenerateOopMap::compute_map() individually")       \
                                                                            \
  develop(bool, TraceMonitorMismatch, false,                                \
          "Trace monitor matching failures during OopMapGeneration")        \
                                                                            \
  develop(bool, TraceOopMapRewrites, false,                                 \
          "Trace rewritting of method oops during oop map generation")      \
                                                                            \
  develop(bool, TraceSafepoint, false,                                      \
          "Trace safepoint operations")                                     \
                                                                            \
  develop(bool, TraceMethodIterator, false,                                 \
          "Trace execution of MethodIterator")                              \
                                                                            \
  develop(bool, TraceICBuffer, false,                                       \
          "Trace usage of IC buffer")                                       \
                                                                            \
  develop(bool, TraceCompiledIC, false,                                     \
          "Trace changes of compiled IC")                                   \
                                                                            \
  develop(bool, TraceZapDeadLocals, false,                                  \
          "Trace zapping dead locals")                                      \
                                                                            \
  develop(bool, TraceStartupTime, false,                                    \
          "Trace setup time")                                               \
                                                                            \
  develop(bool, TraceHPI, false,                                            \
          "Trace Host Porting Interface (HPI)")                             \
                                                                            \
  product(bool, FasterSync, true,                                           \
          "Temp flag for testing faster synch")                             \
                                                                            \
  product(bool, DefaultUseSpecLock, false,                                  \
          "Turns on speculative locking for selected classes on multiproc") \
                                                                            \
  product(bool, ForceUseSpecLock, false,                                    \
          "Turns on speculative locking on the uniproc and multiproc")      \
                                                                            \
  product(bool, UseSpecLock, false,                                         \
          "Must not be set by user. Set automatically from ForceUse, "      \
          "DefaultUse")                                                     \
                                                                            \
  develop(bool, PrintSpecLock, false,                                       \
          "Turn on printing of speculative locking")                        \
                                                                            \
  develop(bool, TraceProtectionDomainVerification, false,                   \
          "Trace protection domain verifcation")                            \
                                                                            \
  develop(bool, TraceClearedExceptions, false,                              \
          "Prints when an exception is forcibly cleared")                   \
                                                                            \
  /* assembler */                                                           \
  product(bool, CodeForP6, false,                                           \
          "generate faster code for P6 (flag set automatically at startup)")\
                                                                            \
  /* gc */                                                                  \
                                                                            \
  product(bool, UseTrainGC, false,                                          \
          "Tells whether the VM should use train algorithm GC for "         \
          "tenured objects")                                                \
                                                                            \
  develop(bool, ScavengeALot, false,                                        \
          "Force scavenge at every Nth exit from the runtime system "       \
          "(N=ScavengeALotInterval)")                                       \
                                                                            \
  develop(bool, FullGCALot, false,                                          \
          "Force full gc at every Nth exit from the runtime system "        \
          "(N=FullGCALotInterval)")                                         \
                                                                            \
  develop(bool, GCALotAtAllSafepoints, false,                               \
          "Enforce ScavengeALot/GCALot at all potential safepoints")        \
                                                                            \
  product_pd(bool, UseTLE,                                                  \
          "Use thread-local object allocation")                             \
                                                                            \
  product_pd(bool, ResizeTLE,                                               \
          "Dynamically resize tle size for threads")                        \
                                                                            \
  product(bool, ZeroTLE, false,                                             \
          "Zero out the newly created TLE")                                 \
                                                                            \
  product(bool, ParkTLE, false,                                             \
          "Enables sharing of TLEs across threads that do not execute "     \
          "concurrently")                                                   \
                                                                            \
  product(bool, PrintTLE, false,                                            \
          "Print various TLE related information")                          \
                                                                            \
  product(int, PrefetchCopyIntervalInBytes, 0,                              \
          "How far ahead to prefetch destination area (zero means off)")    \
                                                                            \
  product(int, PrefetchScavengeFieldsAhead, 0,                              \
          "How many fields ahead to prefetch in scavenge (zero means off)") \
                                                                            \
  develop(bool, VerifyTLE, false,                                           \
          "Enables verification of TLE-related data structures")            \
                                                                            \
  develop(bool, VerifyBeforeExit, trueInDebug,                              \
          "Verify system before exiting")                                   \
                                                                            \
  develop(bool, VerifyBeforeScavenge, false,                                \
          "Verify system before scavenge")                                  \
                                                                            \
  develop(bool, VerifyAfterScavenge, false,                                 \
          "Verify system after scavenge")                                   \
                                                                            \
  develop(bool, VerifyBeforeGC, false,                                      \
          "Verify system before mark-sweep")                                \
                                                                            \
  develop(bool, VerifyAfterGC, false,                                       \
          "Verify system after mark-sweep")                                 \
                                                                            \
  develop(bool, UseSpecialLargeObjectHandling, true,                        \
          "Tells whether the VM should handle large nonoop arrays "         \
          "specially for train GC")                                         \
                                                                            \
  product(bool, DisableExplicitGC, false,                                   \
          "Tells whether calling System.gc() does a full GC")               \
                                                                            \
  /* gc tracing */                                                          \
  product(bool, PrintScavenge, false,                                       \
          "Print message at scavenge")                                      \
                                                                            \
  product(bool, PrintGC, false,                                             \
          "Print message at garbage collect")                               \
                                                                            \
  develop(bool, PrintRemsetAtScavenge, false,                               \
          "Print remembered set at scavenge")                               \
                                                                            \
  develop(bool, PrintRemsetStatistics, false,                               \
          "Print remembered set statictics at scavenge")                    \
                                                                            \
  develop(bool, TraceMarkSweep, false,                                      \
          "Trace mark sweep")                                               \
                                                                            \
  develop(bool, TraceReferenceGC, false,                                    \
          "Trace handling of soft/weak/final/phantom references")           \
                                                                            \
  develop(bool, TraceFinalizerRegistration, false,                          \
         "Trace registration of final references")                          \
                                                                            \
  develop(bool, TraceScavenge, false,                                       \
          "Trace scavenge")                                                 \
                                                                            \
  develop(bool, TraceSpecialLargeObjectHandling, false,                     \
          "Trace special handling of large objects")                        \
                                                                            \
  product(bool, TraceClassLoading, false,                                   \
          "Trace all classes loaded")                                       \
                                                                            \
  product(bool, TraceClassUnloading, false,                                 \
          "Trace unloading of classes")                                     \
                                                                            \
  develop(bool, TraceClassLoadingTime, false,                               \
          "Trace accumulated time for loading classes")                     \
                                                                            \
  develop(bool, TraceScavengeTime, false,                                   \
          "Trace accumulated time for scavenge")                            \
                                                                            \
  develop(bool, TraceMarkSweepTime, false,                                  \
          "Trace accumulated time for mark sweep")                          \
                                                                            \
  develop(bool, TraceCarAllocation, false,                                  \
          "Trace allocation of cars in train generation")                   \
                                                                            \
  develop(bool, TraceMaxHeapCapacity, false,                                \
          "Trace maximum heap capacity over lifetime of VM")                \
                                                                            \
  product(bool, PrintTenuringDistribution, false,                           \
          "Print tenuring age information")                                 \
                                                                            \
  develop(bool, PrintHeapUsageOverTime, false,                              \
          "Print heap usage and capacity with timestamps")                  \
                                                                            \
  /* JVMDI */ 								    \
                                                                            \
  develop(bool, UsingBreakpoints,false,                                     \
          "JVMDI Degugging is off.. (1 disables)")                          \
                                                                            \
  develop(bool, TraceBreakpoints, false,                                    \
          "Trace installation and execution of breakpoints")                \
                                                                            \
  develop(bool, UseFastBreakpoints, true,                                   \
          "Use new breakpoint mechanism (_breakpoint bytecode)")            \
								            \
  /* compiler interface */                                                  \
                                                                            \
  develop(bool, CIPrintCompilerName, false,                                 \
          "when CIPrint is active, print the name of the active compiler")  \
                                                                            \
  develop(bool, CIPrintThreadName, false,                                   \
          "when CIPrint is active, print the name of the active compiler "  \
          "thread")                                                         \
                                                                            \
  develop(bool, CIPrintCompileQueue, false,                                 \
          "display the contents of the compile queue whenever a "           \
          "compilation is enqueued")                                        \
                                                                            \
  develop(bool, CIPrintRequests, false,                                     \
          "display every request for compilation")                          \
                                                                            \
  develop(bool, CITime, false,                                              \
          "collect timing information for compilation")                     \
                                                                            \
  develop(bool, CITimeEach, false,                                          \
          "display timing information after each successful compilation")   \
                                                                            \
  develop(bool, CICountOSR, true,                                           \
          "use a separate counter when assigning ids to osr compilations")  \
                                                                            \
  develop(bool, CICountNative, false,                                       \
          "use a separate counter when assigning ids to native "            \
          "compilations")                                                   \
                                                                            \
  develop(bool, CICompileNatives, true,                                     \
          "compile native methods if supported by the compiler")            \
                                                                            \
  develop_pd(bool, CICompileOSR,                                            \
          "compile on stack replacement methods if supported by the "       \
          "compiler")                                                       \
                                                                            \
  develop(bool, CIEagerAdapters, true,                                      \
          "compile adapters eagerly if the compiler needs adapters")        \
                                                                            \
  develop(bool, CIMaxPriority, false,                                       \
          "start compilation threads with maximum thread priority")         \
                                                                            \
  product(bool, CIDynamicCompilePriority, false,                            \
          "dynamically adjust compilation priority to aid progress and "    \
          "responsiveness")                                                 \
                                                                            \
  develop(bool, CICompileAdaptersInThread, false,                           \
          "compile adapters in the requesting thread")                      \
                                                                            \
  develop(bool, CIPrintMethodCodes, false,                                  \
          "print method bytecodes of the compiled code")                    \
                                                                            \
  /* compiler */                                                            \
                                                                            \
  develop_pd(bool, UseStackBanging,                                         \
          "use stack banging for stack overflow check (synchronized "       \
          "methods only)")                                                  \
                                                                            \
  develop(bool, Use24BitFPMode, true,                                       \
          "Set 24-bit FPU mode on a per-compile basis ")                    \
                                                                            \
  develop(bool, Use24BitFP, true,                                           \
          "use FP instructions that produce 24-bit precise results")        \
                                                                            \
  develop(bool, UseStrictFP, true,                                          \
          "use strict fp if modifier strictfp is set")                      \
                                                                            \
  develop_pd(bool, EnforceStrictFP,                                         \
          "use strictfp for all floats")                                    \
                                                                            \
  develop(bool, GenerateSynchronizationCode, true,                          \
          "generate locking/unlocking code for synchronized methods and "   \
          "monitors")                                                       \
                                                                            \
  develop(bool, GenerateCompilerNullChecks, true,                           \
          "Generate explicit null checks for loads/stores/calls")           \
                                                                            \
  develop(bool, GenerateRangeChecks, true,                                  \
          "Generate range checks for array accesses")                       \
                                                                            \
  develop(bool, ImplicitNullChecks, true,                                   \
          "generate code for implicit null checks")                         \
                                                                            \
  develop(bool, PrintSafepointStatistics, false,                            \
          "print statistics about safepoint synchronization")               \
                                                                            \
  develop(bool, PrintCHA, false,                                            \
          "print CHA information (clases and its subclasses")               \
                                                                            \
  develop(bool, InlineAccessors, true,                                      \
          "inline accessor methods (get/set)")                              \
                                                                            \
  develop(bool, Inline, true,                                               \
          "enable inlining")                                                \
                                                                            \
  product(bool, ClipInlining, true,                                         \
          "clip inlining if aggregate method exceeds DesiredMethodLimit")   \
                                                                            \
  develop(bool, UseCHA, true,                                               \
          "enable CHA")                                                     \
                                                                            \
  product_pd(bool, UseTypeProfile,                                          \
          "enable type profiling in interpreter")                           \
                                                                            \
  develop(bool, DeutschShiffmanALot, false,                                 \
          "do Deutsch-Shiffman type checks wherever UseTypeProfile might "  \
          "insert them")                                                    \
                                                                            \
  develop(bool, UseCallProfile, true,                                       \
          "enable hot call counting in interpreter")                        \
                                                                            \
  product(bool, InlineUnreachedCalls, true,                                 \
          "inline even those call sites which the interpreter has not yet " \
          "executed")                                                       \
                                                                            \
  develop(bool, TimeCompiler, false,                                        \
          "time the compiler")                                              \
                                                                            \
  develop(bool, TimeCompiler2, false,                                       \
          "detailed time the compiler (requires +TimeCompiler)")            \
                                                                            \
  develop(bool, PrintInlining, false,                                       \
          "prints inlining optimizations")                                  \
                                                                            \
  develop(bool, PrintMethodFlushing, false,                                 \
          "print the nmethods beeing flushed")                              \
                                                                            \
  develop(bool, LogMultipleMutexLocking, false,                             \
          "log locking and unlocking of mutexes (only if multiple locks "   \
          "are held)")                                                      \
                                                                            \
  develop(bool, UseRelocIndex, true,                                        \
         "use an index to speed random access to relocations")              \
                                                                            \
  product(bool, PrintVMOptions, trueInDebug,                                \
         "print VM flag settings")                                          \
                                                                            \
  develop(bool, LogVMOutput, trueInDebug,                                   \
         "Save vm output in hotspot.log")                                   \
                                                                            \
  product(bool, UseHeavyMonitors, false,                                    \
          "use heavyweight instead of lightweight Java monitors")           \
                                                                            \
  develop(bool, PrintSymbolTableSizeHistogram, false,                       \
          "print histogram of the symbol table")                            \
                                                                            \
  develop(bool, PrintScopeTree, false,                                      \
          "print scope tree")                                               \
                                                                            \
  develop(bool, ExitVMOnVerifyError, false,                                 \
          "standard exit from VM if bytecode verify error "                 \
          "(only in debug mode)")                                           \
                                                                            \
  develop(bool, RespectFinal, true,                                         \
          "respect 'final' declarations to avoid dispatch")                 \
                                                                            \
  develop(bool, DebugVtables, false,                                        \
          "add debugging code to vtable dispatch")                          \
                                                                            \
  develop(bool, PrintVtables, false,                                        \
          "print vtables when printing klass")                              \
                                                                            \
  develop(bool, PrintVtableStats, false,                                    \
          "print vtables stats at end of run")                              \
                                                                            \
  develop(bool, TraceCreateZombies, false,                                  \
          "trace creation of zombie nmethods")                              \
                                                                            \
  develop(bool, IgnoreLockingAssertions, false,                             \
          "disable locking assertions (for speed)")                         \
                                                                            \
  develop(bool, LoopOptimizations, true,                                    \
          "turn on major loop optimizations")                               \
                                                                            \
  develop(bool, VerifyLoopOptimizations, false,                             \
          "verify major loop optimizations")                                \
                                                                            \
  develop(bool, RangeCheckElimination, true,                                \
          "Split loop iterations to eliminate range checks")                \
                                                                            \
  develop(bool, UncommonNullCast, true,                                     \
          "Uncommon-trap NULLs past to check cast")                         \
                                                                            \
  develop(bool, DelayCompilationDuringStartup, true,                        \
          "Delay invoking the compiler until main application class is "    \
          "loaded")                                                         \
                                                                            \
  develop(bool, CompileTheWorld, false,                                     \
          "Compile all methods in all classes in bootstrap class path "     \
          "(stress test)")                                                  \
                                                                            \
  develop(bool, CompileTheWorldPreloadClasses, true,                        \
          "Preload all classes used by a class before start loading")       \
                                                                            \
  develop(bool, CompileTheWorldIgnoreInitErrors, false,                     \
          "Compile all methods although class initializer failed")          \
                                                                            \
  develop(bool, TraceDeadPhis, false,                                       \
          "Print progress of Dead Phi removal using Gosling Property")      \
                                                                            \
  develop(bool, TraceIterativeGVN, false,                                   \
          "Print progress during Iterative Global Value Numbering")         \
                                                                            \
  develop(bool, FillDelaySlots, true,                                       \
          "Fill delay slots (on SPARC only)")                               \
                                                                            \
  develop(bool, ScheduleInterpreter, false,                                 \
          "Schedule Interpreter (on SPARC only)")                           \
                                                                            \
  develop(bool, VerifyIterativeGVN, false,                                  \
          "Verify Def-Use modifications during sparse Iterative Global "    \
          "Value Numbering")                                                \
                                                                            \
  develop(bool, TracePhaseCCP, false,                                       \
          "Print progress during Conditional Constant Propagation")         \
                                                                            \
  develop(bool, TimeLivenessAnalysis, false,                                \
          "Time computation of bytecode liveness analysis")                 \
                                                                            \
  develop(bool, TraceLivenessGen, false,                                    \
          "Trace the generation of liveness analysis information")          \
                                                                            \
  develop(bool, TraceLivenessQuery, false,                                  \
          "Trace queries of liveness analysis information")                 \
                                                                            \
  develop(bool, CollectIndexSetStatistics, false,                           \
          "Collect information about IndexSets")                            \
                                                                            \
  develop(bool, PrintDominators, false,                                     \
          "Print out dominator trees for GVN")                              \
                                                                            \
  develop(bool, UseLoopSafepoints, true,                                    \
          "Generate Safepoint nodes in every loop")                         \
                                                                            \
  develop(bool, UseCompLeafSafepoints, true,                                \
           "Generate Safepoint nodes at comp leafs")                        \
                                                                            \
  develop(bool, CompSafepointALot, false,                                   \
           "Try to catch more safepoints in compiled code")                 \
                                                                            \
  develop(bool, TraceCISCSpill, false,                                      \
          "Trace allocators use of cisc spillable instructions")            \
                                                                            \
  develop(bool, TraceSpilling, false,                                       \
          "Trace spilling")                                                 \
                                                                            \
  develop(bool, UseNewAntiDependences, true,                                \
          "Use new version of identifying schedule restrictions from "      \
          "anti-dependences")                                               \
                                                                            \
  develop(bool, CloneThroughPhis, true,                                     \
          "Clone operations through Phis if some optimizations occur")      \
                                                                            \
  develop(bool, DeutschShiffmanExceptions, true,                            \
          "Fast check to find exception handler for precisely typed "       \
          "exceptions")                                                     \
                                                                            \
  develop(bool, SplitIfBlocks, true,                                        \
          "Clone compares and control flow through merge points to fold "   \
          "some branches")                                                  \
                                                                            \
  develop(bool, UseNewVersion, true,                                        \
          "Testing Only: Use the new version while testing")                \
                                                                            \
  /* statistics */                                                          \
  develop(bool, UseVTune, false,                                            \
          "enable support for Intel's VTune profiler")                      \
                                                                            \
  develop(bool, CountCompiledCalls, false,                                  \
          "counts method invocations")                                      \
                                                                            \
  develop(bool, CountJNICalls, false,                                       \
          "counts jni method invocations")                                  \
                                                                            \
  develop(bool, CountJVMCalls, false,                                       \
          "counts jvm method invocations")                                  \
                                                                            \
  develop(bool, CountICMisses, false,                                       \
          "count IC misses")                                                \
                                                                            \
  develop(bool, CountRemovableExceptions, false,                            \
          "count exceptions that could be replaced by branches due to "     \
          "inlining")                                                       \
                                                                            \
  develop(bool, ICMissHistogram, false,                                     \
          "produce histogram of IC misses")                                 \
                                                                            \
  develop(bool, PrintClassStatistics, false,                                \
          "prints class statistics at end of run")                          \
                                                                            \
  develop(bool, PrintMethodStatistics, false,                               \
          "prints method statistics at end of run")                         \
                                                                            \
  develop(bool, PrintTrainGCProcessingStats, false,                         \
          "print stats on train gc processing")                             \
                                                                            \
  /* interpreter */                                                         \
  develop(bool, ClearInterpreterLocals, false,                              \
          "Always clear local variables of interpreter activations upon "   \
          "entry")                                                          \
                                                                            \
  product(bool, RewriteFrequentPairs, true,                                 \
          "Rewrite frequently used bytecode pairs into a single bytecode")  \
                                                                            \
  develop(bool, PrintInterpreter, false,                                    \
          "Prints the generated interpreter code")                          \
                                                                            \
  product(bool, UseInterpreter, true,                                       \
          "Use interpreter for non-compiled methods")                       \
                                                                            \
  develop(bool, UseFastInterpreterAllocation, true,                         \
          "Use fast allocation for new bytecode")                           \
                                                                            \
  develop(bool, UseFastSignatureHandlers, true,                             \
          "Use fast signature handlers for native calls")                   \
                                                                            \
  develop(bool, UseV8InstrsOnly, false,                                     \
          "Use SPARC-V8 Compliant instruction subset")                      \
                                                                            \
  develop(bool, UseCASForSwap, false,                                       \
          "Do not use swap instructions, but only CAS (in a loop) on SPARC")\
                                                                            \
  product(bool, UseLoopCounter, true,                                       \
          "Increment invocation counter on backward branch")                \
                                                                            \
  product(bool, UseFastEmptyMethods, true,                                  \
          "Use fast method entry code for empty methods")                   \
                                                                            \
  product(bool, UseFastAccessorMethods, true,                               \
          "Use fast method entry code for accessor methods")                \
                                                                            \
  product_pd(bool, UseC2CallingConventions,                                 \
          "Use C2 calling conventions")                                     \
                                                                            \
  product_pd(bool, UseOnStackReplacement,                                   \
           "Use on stack replacement, calls runtime if invoc. counter "     \
           "overflows in loop")                                             \
                                                                            \
  develop(bool, TraceOnStackReplacement, false,                             \
          "Trace on stack replacement")                                     \
                                                                            \
  develop(bool, PoisonOSREntry, true,                                       \
           "Detect abnormal calls to OSR code")                             \
                                                                            \
  product(bool, PreferInterpreterNativeStubs, true,                         \
          "Use always interpreter stubs for native methods invoked via "    \
          "interpreter")                                                    \
                                                                            \
  develop(bool, CountBytecodes, false,                                      \
          "Count number of bytecodes executed")                             \
                                                                            \
  develop(bool, PrintBytecodeHistogram, false,                              \
          "Print histogram of the executed bytecodes")                      \
                                                                            \
  develop(bool, PrintBytecodePairHistogram, false,                          \
          "Print histogram of the executed bytecode pairs")                 \
                                                                            \
  develop(bool, PrintBCIHistogram, false,                                   \
          "Print histogram of bci execution")                               \
                                                                            \
  develop(bool, PrintSignatureHandlers, false,                              \
          "Print code generated for native method signature handlers")      \
                                                                            \
  develop(bool, GenerateInterpreterDiv0Checks, false,                       \
          "Generate explicit division by 0 checks in interpreter code")     \
                                                                            \
  develop(bool, GenerateInterpreterNullChecks, false,                       \
          "Generate explicit NULL checks in interpreter code")              \
                                                                            \
  develop(bool, VerifyOops, false,                                          \
          "Do plausibility checks for oops")                                \
                                                                            \
  develop(bool, VerifyJNIFields, trueInDebug,                               \
          "Verify jfieldIDs for instance fields")                           \
                                                                            \
  develop(bool, VerifyJNIEnvThread, false,                                  \
          "Verify JNIEnv.thread == Thread::current() when entering VM "     \
          "from JNI")                                                       \
                                                                            \
  develop(bool, VerifyFPU, false,                                           \
          "Verify FPU state (check for NaN's, etc.)")                       \
                                                                            \
  develop(bool, VerifyThread, false,                                        \
          "Watch the thread register for corruption (SPARC only)")          \
                                                                            \
  develop(bool, VerifyActivationFrameSize, false,                           \
          "Verify that activation frame didn't become smaller than its "    \
          "minimal size")                                                   \
                                                                            \
  develop(bool, VerifyReceiver, false,                                      \
          "Verify that receiver is a subclass of method holder")            \
                                                                            \
  product(bool, EnableJVMPIInstructionStartEvent, false,                    \
          "Enable JVMPI_EVENT_INSTRUCTION_START events - slows down "       \
          "interpretation")                                                 \
                                                                            \
  develop(bool, CountInterpBranches, false,                                 \
          "Collect branch frequency information for use by Compiler")       \
                                                                            \
  develop_pd(bool, CountInterpCalls,                                        \
          "Collect call frequency information for use by Compiler")         \
                                                                            \
  develop(bool, CountInterpInvocations, true,                               \
          "Collect invokation frequency information for use by Compiler")   \
                                                                            \
  develop(bool, TraceFrequencyInlining, false,                              \
          "Trace frequency based inlining")                                 \
                                                                            \
  develop(bool, TraceTypeProfile, false,                                    \
          "Trace type profile")                                             \
                                                                            \
  develop_pd(bool, InlineIntrinsics,                                        \
           "Inline intrinsics that can be statically resolved")             \
                                                                            \
                                                                            \
                                                                            \
  /* recompilation */                                                       \
  product(bool, UseRecompilation, true,                                     \
          "use recompilation")                                              \
                                                                            \
  develop(bool, UseCounterDecay, true,                                      \
           "adjust recompilation counters")                                 \
                                                                            \
  develop(bool, UseAdaptiveThresholds, false,                               \
          "use adaptive recompilation thresholds")                          \
                                                                            \
  product(bool, AlwaysCompileLoopMethods, false,                            \
          "when using recompilation, never interpret methods "              \
          "containing loops")                                               \
                                                                            \
  develop(bool, PrintRecompilation, false,                                  \
          "print one-line info for each recompilation")                     \
                                                                            \
  develop(bool, PrintRecompilation2, false,                                 \
          "print extensive info for each recompilation")                    \
                                                                            \
  develop(bool, PrintRecompilationMonitoring, false,                        \
          "print info about recompilation threshold adjusting")             \
                                                                            \
  product(bool, DontCompileHugeMethods, true,                               \
          "don't compile methods > HugeMethodLimit")                        \
                                                                            \
  /* deoptimization */                                                      \
  develop(bool, TraceDeoptimization, false,                                 \
          "Trace deoptimization")                                           \
                                                                            \
  develop(bool, DebugDeoptimization, false,                                 \
          "Tracing various information while debugging deoptimization")     \
                                                                            \
  product(int, SelfDestructTimer, 0,                                        \
          "Will cause VM to terminate after a given time (in minutes) "     \
          "(0 means off)")                                                  \
                                                                            \
  product(int, MaxJavaStackTraceDepth, 1024,                                \
          "Max. no. of lines in the stack trace for Java exceptions "       \
          "(0 means all)")                                                  \
                                                                            \
  develop(int, GuaranteedSafepointInterval, 1000,                           \
          "Guarantee a safepoint (at least) every so many milliseconds "    \
          "(0 means none)")                                                 \
                                                                            \
  develop(int, SafepointTimeoutDelay, 10000,                                \
          "Delay in milliseconds for option SafepointTimeout")              \
                                                                            \
  develop(int, MaxICLEStubsBeforeSafepoint, 1000,                           \
          "Force a safepoint after allocating this many "                   \
          "InterfaceLookupCache entries ")                                  \
                                                                            \
  develop(int, PrimaryCacheMask, -1,                                        \
          "Only use these bits when addressing the primary lookup cache")   \
                                                                            \
  develop(int, SecondaryCacheMask, -1,                                      \
          "Only use these bits when addressing the secondary lookup cache") \
                                                                            \
  product(int, NmethodSweepFraction, 4,                                     \
          "Number of invocations of sweeper to cover all nmethods")         \
                                                                            \
  product(int, NMethodSweepInterval, 1000,                                  \
          "Time between each invocation of the NMethodSweeper")             \
                                                                            \
  develop(int, MemProfilingInterval, 500,                                   \
          "Time between each invocation of the MemProfiler")                \
                                                                            \
  develop(int, MallocCatchPtr, -1,                                          \
          "Hit breakpoint when mallocing/freeing this pointer")             \
                                                                            \
  develop(int, PostMortemDump,falseInDebug,                                 \
          "Dump info upon unhandled trap; "                                 \
          "1 = dump some, 2 = dump more (unsafer)")                         \
                                                                            \
  develop(int, AssertRepeat, 1,                                             \
          "number of times to evaluate expression in assert "               \
          "(to estimate overhead)")                                         \
                                                                            \
  develop(int, HandleAllocationLimit, 1024,                                 \
          "Threshold for HandleMark allocation when +TraceHandleAllocation "\
          "is used")                                                        \
                                                                            \
  develop(int, TotalHandleAllocationLimit, 1024,                            \
          "Threshold for total handle allocation when "                     \
          "+TraceHandleAllocation is used")                                 \
                                                                            \
  develop(int, StackPrintLimit, 100,                                        \
          "number of stack frames to print in VM-level stack dump")         \
                                                                            \
  develop(int, MaxElementPrintSize, 256,                                    \
          "maximum number of elements to print")                            \
                                                                            \
  develop(int, MaxSubklassPrintSize, 4,                                     \
          "maximum number of subklasses to print when printing klass")      \
                                                                            \
  develop(int, MaxInlineLevel, 9,                                           \
          "maximum number of nested calls that are inlined")                \
                                                                            \
  develop(int, MaxRecursiveInlineLevel, 1,                                  \
          "maximum number of nested recursive calls that are inlined")      \
                                                                            \
  develop(int, InlineSmallCode, 1000,                                       \
          "Only inline already compiled methods if their code size is "     \
          "less than this")                                                 \
                                                                            \
  product(int, MaxInlineSize, 35,                                           \
          "maximum bytecode size of a method to be inlined")                \
                                                                            \
  product_pd(int, FreqInlineSize,                                           \
          "maximum bytecode size of a frequent method to be inlined")       \
                                                                            \
  develop(int, HotCallFrequency, 1000,                                      \
          "frequency threshold at which a call site becomes ``hot'' "       \
          "(as a percent)")                                                 \
                                                                            \
  develop(int, MinRecompRootSize,  20,                                      \
          "minimum bytecode size of a method to be compiled as a root")     \
                                                                            \
  develop(int, MaxTrivialSize, 6,                                           \
          "maximum bytecode size of a trivial method to be inlined")        \
                                                                            \
  develop(int, MinInliningThreshold, 250,                                   \
          "min. invocation count a method needs to have to be inlined")     \
                                                                            \
  develop(int, MaxBackPatches, 3,                                           \
          "maximum nof backpatches before switching it off")                \
                                                                            \
  develop(int, AlignEntryCode, 4,                                           \
          "aligns entry code to specified value (in bytes)")                \
                                                                            \
  develop(int, CodeBufferSize, 64*K,                                        \
          "default code buffer size (currently = max. nmethod size)")       \
                                                                            \
  develop(int, MaxNMethodSize, 8*K,                                         \
          "desired max. nmethod size")                                      \
                                                                            \
  develop(int, MethodHistogramCutoff, 100,                                  \
          "cutoff value for method invoc. histogram (+CountCalls)")         \
                                                                            \
  develop(int, ProfilerNumberOfInterpretedMethods, 25,                      \
          "# of interpreted methods to show in profile")                    \
                                                                            \
  develop(int, ProfilerNumberOfCompiledMethods, 25,                         \
          "# of compiled methods to show in profile")                       \
                                                                            \
  develop(int, ProfilerNumberOfStubMethods, 25,                             \
          "# of stub methods to show in profile")                           \
                                                                            \
  develop(int, ProfilerNumberOfRuntimeStubNodes, 25,                        \
          "# of runtime stub nodes to show in profile")                     \
                                                                            \
  product(int, ProfileIntervalsTicks, 100,                                  \
          "# of ticks between printing of interval profile "                \
          "(+ProfileIntervals)")                                            \
                                                                            \
  develop(int, ICacheAlignmentMode,     0,                                  \
          "controls the insertion of nops to achieve I$ alignment")         \
                                                                            \
  develop(int, ScavengeALotInterval,     1,                                 \
          "Interval between which scavenge will occur with +ScavengeALot")  \
                                                                            \
  develop(int, FullGCALotInterval,     1,                                   \
          "Interval between which full gc will occur with +FullGCALot")     \
                                                                            \
  develop(int, FullGCALotStart,     0,                                      \
          "For which invocation to start FullGCAlot")                       \
                                                                            \
  develop(int, FullGCALotDummies,  32*K,                                    \
          "Dummy object allocated with +FullGCALot, forcing all objects "   \
          "to move")                                                        \
                                                                            \
  develop(int, DontYieldALotInterval,    10,                                \
          "Interval between which yields will be dropped (milliseconds)")   \
                                                                            \
  develop(int, MinSleepInterval,     1,                                     \
          "Minimum sleep() interval (milliseconds) when "                   \
          "ConvertSleepToYield is off (used for SOLARIS)")                  \
                                                                            \
  product(int, EventLogLength,  2000,                                       \
          "maximum nof events in event log")                                \
                                                                            \
  develop(int, ProfilerPCTickThreshold,    15,                              \
          "Number of ticks in a PC buckets to be a hotspot")                \
                                                                            \
  develop(int, DeoptimizeALotInterval,     5,                               \
          "Number of exits until DeoptimizeALot kicks in")                  \
                                                                            \
  develop(int, ZombieALotInterval,     5,                                   \
          "Number of exits until ZombieALot kicks in")                      \
                                                                            \
  develop(int, MallocVerifyInterval,     0,                                 \
          "if non-zero, verify C heap after every N calls to "              \
          "malloc/realloc/free")                                            \
                                                                            \
  develop(int, MallocVerifyStart,     0,                                    \
          "if non-zero, start verifying C heap after Nth call to "          \
          "malloc/realloc/free")                                            \
                                                                            \
  develop(int, AllocateLargeLiveRangePolicy,    10,                         \
          "Do not color (share stack slots) live ranges that are bigger "   \
          "than this")                                                      \
                                                                            \
  develop(int, NodeCountCutoff, 37000,                                      \
          "If parser node generation exceeds limit resort to interpreter")  \
                                                                            \
  develop(int, NodeCountInliningCutoff, 18000,                              \
          "If parser node generation exceeds limit stop inlining")          \
                                                                            \
  develop(int, BranchFreqTableSize,    10,                                  \
          "Size of branch frequency counter table is 2 to the n power")     \
                                                                            \
  develop(int, CallFreqTableSize,    10,                                    \
          "Size of call frequency counter table is 2 to the n power")       \
                                                                            \
  develop(int, CallProfileSize,  1024,                                      \
          "Size of call profile table (must be a power of 2)")              \
                                                                            \
  develop(int, CallProfileExclusionRatio,    10,                            \
          "FIXIT: What does this really do?")                               \
                                                                            \
  develop(int, FreqCountInvocations,  1500,                                 \
          "Number of method invocations that will collect branch frequency "\
          "counts")                                                         \
                                                                            \
  develop(int, InlineFrequencyRatio,     5,                                 \
          "Ratio of call site execution to caller method invocation")       \
                                                                            \
  develop_pd(int, InlineFrequencyCount,                                     \
          "Count of call site execution necessary to trigger frequent "     \
          "inlining")                                                       \
                                                                            \
  develop(int, InlineThrowCount,    50,                                     \
          "Force inlining of interpreted methods that throw this often")    \
                                                                            \
  develop(int, InlineThrowMaxSize,   200,                                   \
          "Force inlining of throwing methods smaller than this")           \
                                                                            \
  product(int, AliasLevel,     2,                                           \
          "0 for no aliasing, 1 for oop/field/static/array split, "         \
          "2 for best")                                                     \
                                                                            \
  develop(int, WatcherThreadInterval,    10,                                \
          "Delay interval in milli secs for the watcher thread to check "   \
          "for tasks")                                                      \
                                                                            \
  develop(int, ProfilerNodeSize,  1024,                                     \
          "Size in K to allocate for the Profile Nodes of each thread")     \
                                                                            \
  develop(int, V8AtomicOperationUnderLockSpinCount,    50,                  \
          "Number of times to spin wait on a v8 atomic operation lock")     \
                                                                            \
  product(int, ReadSpinIterations,   100,                                   \
          "Number of read attempts before a yield (spin inner loop)")       \
                                                                            \
  product_pd(int, PreInflateSpin,                                           \
          "Number of times to spin wait before inflation")                  \
                                                                            \
  product(int, PreBlockSpin,    10,                                         \
          "Number of times to spin in an inflated lock before going to "    \
          "an OS lock")                                                     \
                                                                            \
  /* gc parameters */                                                       \
  product(size_t, MaxHeapSize,  64*M,                                          \
          "Default maximum size for object heap (in bytes)")                \
                                                                            \
  product_pd(size_t, NewSize,                                                  \
          "Default size of new generation (in bytes)")                      \
                                                                            \
  product_pd(size_t, MaxNewSize,                                               \
          "Maximum size of new generation (in bytes)")                      \
                                                                            \
  product_pd(size_t, TLESize,                                                  \
          "Default (or starting) size of TLE (in Kbytes)")                  \
                                                                            \
  product(int, MaxTLERatio,   128,                                          \
          "The max allowed fraction of tle size relative to the entire "    \
          "Eden (used with ResizeTLE)")                                     \
                                                                            \
  product(int, TLEThreadRatio,    10,                                       \
          "At least this fraction of threads must be able to have max "     \
          "size tles (used with ResizeTLE)")                                \
                                                                            \
  product(int, TLEFragmentationRatio,    64,                                \
          "Max. allowed fragmentation (power of 2)")                        \
                                                                            \
  product_pd(int, SurvivorRatio,                                            \
          "Ratio of eden/survivor space size")                              \
                                                                            \
  product_pd(int, NewRatio,                                                 \
          "Ratio of new/old generation sizes")                              \
                                                                            \
  product(int, MaxLiveObjectEvacuationRatio, 100,                           \
          "Max percent of eden objects that will be live at scavenge")      \
                                                                            \
  product_pd(size_t, NewSizeThreadIncrease,                                 \
          "Additional size added to desired new generation size per "       \
          "non-daemon thread (in bytes)")                                   \
                                                                            \
  product(size_t, OldSize,1408*K,                                              \
          "Default size of tenured generation (in bytes)")                  \
                                                                            \
  product(size_t, PermSize,   1*M,                                             \
          "Default size of permanent generation (in bytes)")                \
                                                                            \
  product_pd(size_t, MaxPermSize,					    \
          "Maximum size of permanent generation (in bytes)")                \
                                                                            \
  product(int, MinHeapFreeRatio,    40,                                     \
          "Min percentage of heap free after GC to avoid expansion")        \
                                                                            \
  product(int, MaxHeapFreeRatio,    70,                                     \
          "Max percentage of heap free after GC to avoid shrinking")        \
                                                                            \
  product(int, SoftRefLRUPolicyMSPerMB, 1000,                               \
          "Number of milliseconds per MB of free space in the heap")        \
                                                                            \
  product(size_t, MinOldHeapExpansion, 128*K,                               \
          "Min expansion of old generation (during scavenge) (in bytes)")   \
                                                                            \
  product(size_t, MinPermHeapExpansion, 256*K,                                 \
          "Min expansion of permanent heap (in bytes)")                     \
                                                                            \
  product(size_t, MaxPermHeapExpansion,   4*M,                                 \
          "Max expansion of permanent heap without full GC (in bytes)")     \
                                                                            \
  develop(int, LogOfCarSpaceSize,    16,                                    \
          "Size of car spaces in trains (2^n bytes)")                       \
                                                                            \
  product(int, MinTickInterval,    -2,                                      \
          "Minimum train tick interval")                                    \
                                                                            \
  product(int, DefaultTickInterval,     1,                                  \
          "Default train tick interval")                                    \
                                                                            \
  product(int, MaxTickInterval,     3,                                      \
          "Maximum train tick interval")                                    \
                                                                            \
  product(int, DelayTickAdjustment,    10,                                  \
          "Delay before adaptively adjusting tick rate after startup/full " \
          "gc")                                                             \
                                                                            \
  product(int, ProcessingToTenuringRatio,     6,                            \
          "Adaptive ratio of processed objects to tenured objects")         \
                                                                            \
  develop(int, MinTrainLength,     4,                                       \
          "Minimum length of (last) train before creating new train")       \
                                                                            \
  develop(int, VerifyScavengeStartAt,     1,                                \
          "Scavenge invoke count where +VerifyBefore/AfterScavenge "        \
          "kicks in")                                                       \
                                                                            \
  develop(int, VerifyGCStartAt,     1,                                      \
          "Full gc invoke count where +VerifyBefore/AfterGC kicks in")      \
                                                                            \
  develop(int, ScavengeBreakAt,    -1,                                      \
          "Enter breakpoint at this scavenge invoke count")                 \
                                                                            \
  develop(int, GCBreakAt,    -1,                                            \
          "Enter breakpoint at this full gc invoke count")                  \
                                                                            \
  product(int, MaxTenuringThreshold,    32,                                 \
          "Maximum value for tenuring threshold")                           \
                                                                            \
  product(int, TargetSurvivorRatio,    50,                                  \
          "Desired percentage of survivor space used after scavenge")       \
                                                                            \
  product(int, MarkSweepDeadRatio,     5,                                   \
          "Percentage dead space in the tenured space before a compaction " \
          "kicks in")                                                       \
                                                                            \
  product(int, PermMarkSweepDeadRatio,    20,                               \
          "Percentage dead space in the perm space before a compaction "    \
          "kicks in")                                                       \
                                                                            \
  product(int, MarkSweepAlwaysCompactCount,     4,                          \
          "How often should we fully compact the heap (ignoring the dead "  \
          "space parameters)")                                              \
                                                                            \
  product(int, DeferThrSuspendLoopCount,     0,                             \
          "Number of times to loop safepoint loop before suspending "       \
          "threads")                                                        \
                                                                            \
  /* stack parameters */                                                    \
  product(int, StackYellowPages, 2,                                         \
          "Number of yellow zone (recoverable overflows) pages")            \
                                                                            \
  product(int, StackRedPages, 1,                                            \
          "Number of red zone (unrecoverable overflows) pages")             \
                                                                            \
  product(int, AltStackSize, 16*K,                                          \
          "Solaris alternate signal stack size (in Kbytes)")                \
                                                                            \
  product_pd(int, ThreadStackSize,                                          \
          "Thread Stack Size (in Kbytes)")                                  \
                                                                            \
  product(int, VMThreadStackSize, 0,                                        \
          "Non-Java Thread Stack Size (in Kbytes)")                         \
                                                                            \
  product(int, CompilerThreadStackSize, 0,                                  \
          "Compiler Thread Stack Size (in Kbytes)")                         \
                                                                            \
  /* code cache parameters */                                               \
  develop(int, CodeCacheSegmentSize, 64,                                    \
          "Code cache segment size (in bytes) - smallest unit of "          \
          "allocation")                                                     \
                                                                            \
  develop_pd(int, CodeEntryAlignment,                                       \
          "Code entry alignment for generated code (in bytes)")             \
                                                                            \
  product(int, InitialCodeCacheSize, 160*K,                                 \
          "Initial code cache size (in bytes)")                             \
                                                                            \
  product(int, ReservedCodeCacheSize, 32*M,                                 \
          "Reserved code cache size (in bytes) - maximum code cache size")  \
                                                                            \
  product(int, CodeCacheMinimumFreeSpace, 500*K,                            \
          "When less than X space left, we stop compiling.")                \
                                                                            \
  product(int, CodeCacheExpansionSize,  32*K,                               \
          "Code cache expansion size (in bytes)")                           \
                                                                            \
  /* interpreter debugging */                                               \
  develop(int, BinarySwitchThreshold, 5,                                    \
          "Minimal number of lookupswitch entries for rewriting to binary " \
          "switch")                                                         \
                                                                            \
  develop(int, StopInterpreterAt, 0,                                        \
          "Stops interpreter execution at specified bytecode number")       \
                                                                            \
  develop(int, TraceBytecodesAt, 0,                                         \
          "Traces bytecodes starting with specified bytecode number")       \
                                                                            \
  /* compiler interface */                                                  \
  develop(int, CIStart, 0,                                                  \
          "the id of the first compilation to permit")                      \
                                                                            \
  develop(int, CIStop,    -1,                                               \
          "the id of the last compilation to permit")                       \
                                                                            \
  develop(int, CIStartOSR,     0,                                           \
          "the id of the first osr compilation to permit "                  \
          "(CICountOSR must be on)")                                        \
                                                                            \
  develop(int, CIStopOSR,    -1,                                            \
          "the id of the last osr compilation to permit "                   \
          "(CICountOSR must be on)")                                        \
                                                                            \
  develop(int, CIStartNative,     0,                                        \
          "the id of the first native compilation to permit "               \
          "(CICountNative must be on")                                      \
                                                                            \
  develop(int, CIStopNative,    -1,                                         \
          "the id of the last native compilation to permit "                \
          "(CICountNative must be on")                                      \
                                                                            \
  develop(int, CIBreakAtOSR,    -1,                                         \
          "id of osr compilation to break at")                              \
                                                                            \
  develop(int, CIBreakAtNative,    -1,                                      \
          "id of native compilation to break at")                           \
                                                                            \
  develop(int, CIBreakAt,    -1,                                            \
          "id of compilation to break at")                                  \
                                                                            \
  product(int, CICompilerCount,     1,                                      \
          "how many compiler threads should run")                           \
                                                                            \
  develop(int, CICompileLowRatio,     1,                                    \
          "hard-coded ratio of (low priority time / high priority time) "   \
          "for compiler")                                                   \
                                                                            \
  /* compiler debugging */                                                  \
  develop(int, CompileTheWorldStartAt,     1,                               \
          "First class to consider when using +CompileTheWorld")            \
                                                                            \
  develop(int, CompileTheWorldStopAt, max_jint,                             \
          "Last class to consider when using +CompileTheWorld")             \
                                                                            \
  develop(int, NewVersionAt,     0,                                         \
          "Testing Only: Create a dedicated integer parameter before "      \
          "putback")                                                        \
                                                                            \
  /* new oopmap storage allocation */                                       \
  develop(int, MinOopMapAllocation,     8,                                  \
          "Minimum number of OopMap entries in an OopMapSet")               \
                                                                            \
  /* Background Compilation */                                              \
  develop(int, BackgroundCompilationWaitThreshold ,      0,                 \
          "Max. time a thread wait for a compilation to complete")          \
                                                                            \
  develop(int, BackgroundCompilationMaxQueue,     20,                       \
          "Max. nof. compilations that can be in queue")                    \
                                                                            \
  develop(int, BackgroundCompilationExpiration,     10,                     \
          "Experiation time in secs. of a compilation request")             \
                                                                            \
  develop(int, LongCompileThreshold,     50,                                \
          "Used with +TraceLongCompiles")                                   \
                                                                            \
  product(int, StarvationMonitorInterval,    200,                           \
          "Pause between each check in ms")                                 \
                                                                            \
  develop(int, StarvationDeflateThreshold,      3,                          \
          "No. of methods to compile during a tick to deflate priority")    \
                                                                            \
  /* recompilation */                                                       \
  product_pd(int, CompileThreshold,                                         \
          "number of method invocations/branches before (re-)compiling")    \
                                                                            \
  product(int, OnStackReplaceThreshold, 14000,                              \
          "number of method invocations/branches before (re-)compiling OSR "\
          "code")                                                           \
                                                                            \
  develop(int, MaxRecompilationSearchLength,    10,                         \
          "max. # frames to inspect searching for recompilee")              \
                                                                            \
  develop(int, MaxInterpretedSearchLength,     3,                           \
          "max. # interp. frames to skip when searching for recompilee")    \
                                                                            \
  develop(int, DesiredMethodLimit,  8000,                                   \
          "desired max. method size (in bytecodes) after inlining")         \
                                                                            \
  develop(int, HugeMethodLimit,  8000,                                      \
          "don't compile methods larger than this if "                      \
          "+DontCompileHugeMethods")                                        \
                                                                            \
  /* recompilation monitoring / throttling */                               \
  /* (most values read only at startup)    */                               \
  develop(int, RecompilationMonitorIntervalLength,   100,                   \
          "length (in ms) of recompilation monitor interval")               \
                                                                            \
  develop(int, RecompilationMonitorIntervals,    10,                        \
          "number of recomp. monitor intervals for sliding avg.")           \
                                                                            \
  develop(int, CompilationPercentageThresholdHigh,    20,                   \
          "throttle compilation if %time above this threshold")             \
                                                                            \
  develop(int, CompilationPercentageThresholdLow,     5,                    \
          "ok to do more compilation if %time below this threshold")        \
                                                                            \
  develop(int, InterpreterOverheadThresholdHigh,15,                         \
          "max. desired interpreter overhead")                              \
                                                                            \
  develop(int, InterpreterOverheadThresholdLow,     5,                      \
          "min. desired interpreter overhead")                              \
                                                                            \
  develop_pd(int, CompileThresholdMin,                                      \
          "minimal compile threshold")                                      \
                                                                            \
  develop_pd(int, CompileThresholdMax,                                      \
          "maximal compile threshold")                                      \
                                                                            \
  develop(int, CompileThresholdAdjustFactor,   150,                         \
          "adjustment factor (scaled by 100)")                              \
                                                                            \
  develop(int, CounterHalfLifeTime,    30,                                  \
          "half-life time of invocation counters (in secs)")                \
                                                                            \
  develop(int, CounterDecayMinIntervalLength,   500,                        \
          "Min. ms. between invocation of CounterDecay")                    \
                                                                            \

/*
 *  Macros for factoring of globals
 */

// Interface macros
#define DECLARE_PRODUCT_FLAG(type, name, value, doc)    extern "C" type name;
#define DECLARE_PD_PRODUCT_FLAG(type, name, doc)        extern "C" type name;
#ifdef PRODUCT
#define DECLARE_DEVELOPER_FLAG(type, name, value, doc)  const type name = value; 
#define DECLARE_PD_DEVELOPER_FLAG(type, name, doc)      const type name = pd_##name; 
#else
#define DECLARE_DEVELOPER_FLAG(type, name, value, doc)  extern "C" type name;
#define DECLARE_PD_DEVELOPER_FLAG(type, name, doc)      extern "C" type name;
#endif

// Implementation macros
#define MATERIALIZE_PRODUCT_FLAG(type, name, value, doc)   type name = value;
#define MATERIALIZE_PD_PRODUCT_FLAG(type, name, doc)       type name = pd_##name; 
#ifdef PRODUCT
#define MATERIALIZE_DEVELOPER_FLAG(type, name, value, doc) /* flag name is constant */ 
#define MATERIALIZE_PD_DEVELOPER_FLAG(type, name, doc)     /* flag name is constant */ 
#else
#define MATERIALIZE_DEVELOPER_FLAG(type, name, value, doc) type name = value; 
#define MATERIALIZE_PD_DEVELOPER_FLAG(type, name, doc)     type name = pd_##name; 
#endif

RUNTIME_FLAGS(DECLARE_DEVELOPER_FLAG, DECLARE_PD_DEVELOPER_FLAG, DECLARE_PRODUCT_FLAG, DECLARE_PD_PRODUCT_FLAG)

