/*
 * @(#)init.cpp	1.80 00/02/04 12:52:26
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_init.cpp.incl"


// Add new initialization/finalization methods to the macro below; your
// functions must be parameterless and must be named after the .c file
// they belong to (e.g. "void foo_init(); void foo_exit()" for foo.c).
// The INIT_DO macro defines the initialization sequence; you need not
// add a corresponding entry to EXIT_DO if you don't need finalization.


// Initialization done by VM thread
# define VM_INIT_DO(template)         \
  template(speclock_init)             \
  template(check_ThreadShadow)        \
  template(check_basic_types)         \
  template(eventlog_init)             \
  template(mutex_init)                \
  template(chunkpool_init)


// Initialization done by Java thread
# define INIT_DO(template)            \
  template(vtune_init)                \
  template(bytecodes_init)            \
  template(classLoader_init)          \
  template(codeCache_init)            \
  template(icache_init)               \
  template(VM_Version_init)           \
  template(stubRoutines_init1)        \
  template(carSpace_init)             \
  template(universe_init)	      /* dependent on codeCache_init and stubRoutines_init */ \
  template(interpreter_init)          /* before any methods loaded */ \
  template(invocationCounter_init)    /* before any methods loaded */ \
  template(scavenge_init)	      \
  template(marksweep_init)	      \
  template(accessFlags_init)          \
  template(templateTable_init)        \
  template(InterfaceSupport_init)     \
  template(universe2_init)            /* dependent on codeCache_init and stubRoutines_init */ \
  template(instanceRefKlass_init)     \
  template(jni_handles_init)


/* compiler related */
#ifdef CORE
# define COMPILER_INIT_DO(template)
#endif


#ifdef COMPILER1
# define COMPILER_INIT_DO(template)   \
  template(compiler1_init)            \
  template(vtableStubs_init)          \
  template(InlineCacheBuffer_init)    \
  template(compilerOracle_init)       \
  template(onStackReplacement_init) 
#endif


#ifdef COMPILER2
# define COMPILER_INIT_DO(template)   \
  template(adapter_init)              \
  template(vtableStubs_init)          \
  template(InlineCacheBuffer_init)    \
  template(compilerOracle_init)       \
  template(compiler2_init)            /* needs ObjectArrayKlass vtables */ \
  template(onStackReplacement_init)   
#endif


// Initialization after compiler initialization
# define INIT_AFTER_COMP_DO(template) \
  template(universe_post_init)        /* must happen after compiler_init */ \
  template(javaClasses_init)          /* must happen after vtable initialization */ \
  template(stubRoutines_init2)        /* note: StubRoutines need 2-phase init */


# define EXIT_DO(template)            \
  template(threadLocalStorage_exit)   \
  template(ostream_exit)


# define DEFINE_TEMPLATE(name)        \
  void name();


# define CALL_TEMPLATE(name)          \
  name();


VM_INIT_DO(DEFINE_TEMPLATE)
INIT_DO(DEFINE_TEMPLATE)
COMPILER_INIT_DO(DEFINE_TEMPLATE)
INIT_AFTER_COMP_DO(DEFINE_TEMPLATE)
EXIT_DO(DEFINE_TEMPLATE)


#ifdef CORE
void adapter_init()           {}
void invocationCounter_init() {}
void codeCache_init()         {}
void sweeper_init()           {}
#endif


void vm_init_globals() {
  VM_INIT_DO(CALL_TEMPLATE)
}


void init_globals() {  
  INIT_DO(CALL_TEMPLATE)
  COMPILER_INIT_DO(CALL_TEMPLATE)  
  INIT_AFTER_COMP_DO(CALL_TEMPLATE)
  if (VerifyBeforeScavenge) {
    Universe::update_offset_tables();
    Universe::verify();   // make sure we're starting with a clean slate
  }  
}


void exit_globals() {
  static bool destructorsCalled = false;
  if (!destructorsCalled) {
    destructorsCalled = true;
    EXIT_DO(CALL_TEMPLATE);
  }
}


static bool _init_completed = false;

bool is_init_completed() {
  return _init_completed;
}


void set_init_completed() {
  _init_completed = true;
}
