/*
 * @(#)java.cpp	1.146 01/02/09 14:47:41
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_java.cpp.incl"

#ifndef PRODUCT

#ifndef CORE

// Statistics printing (method invocation histogram)

GrowableArray<methodOop>* collected_methods;

void collect_methods(methodOop m) {
  if (m->invocation_count() + m->compiled_invocation_count() >= 1 ) {
    collected_methods->push(m);
  }
}


int compare_methods(methodOop* a, methodOop* b) {
  return ((*b)->invocation_count() + (*b)->compiled_invocation_count())
       - ((*a)->invocation_count() + (*a)->compiled_invocation_count());
}


void print_method_invocation_histogram() {
  ResourceMark rm;
  HandleMark hm;
  collected_methods = new GrowableArray<methodOop>(1024);
  SystemDictionary::methods_do(collect_methods);
  collected_methods->sort(&compare_methods);
  //
  tty->cr();
  tty->print_cr("Histogram Over MethodOop Invocation Counters (cutoff = %d):", MethodHistogramCutoff);
  tty->cr();
  tty->print_cr("____Count_(I/C)____Method________________________Module_________________");
  int total = 0, int_total = 0, comp_total = 0, static_total = 0, final_total = 0, 
      synch_total = 0, nativ_total = 0, acces_total = 0;
  for (int index = 0; index < collected_methods->length(); index++) {
    methodOop m = collected_methods->at(index);
    int c = m->invocation_count() + m->compiled_invocation_count();
    if (c >= MethodHistogramCutoff) m->print_invocation_count();
    int_total  += m->invocation_count();
    comp_total += m->compiled_invocation_count();
    if (m->is_final())        final_total  += c;
    if (m->is_static())       static_total += c;
    if (m->is_synchronized()) synch_total  += c;
    if (m->is_native())       nativ_total  += c;
    if (m->is_accessor())     acces_total  += c;
  }
  tty->cr();
  total = int_total + comp_total;
  tty->print_cr("Invocations summary:");
  tty->print_cr("\t%9d (%4.1f%%) interpreted",  int_total,    100.0 * int_total    / total);
  tty->print_cr("\t%9d (%4.1f%%) compiled",     comp_total,   100.0 * comp_total   / total);
  tty->print_cr("\t%9d (100%%)  total",         total);
  tty->print_cr("\t%9d (%4.1f%%) synchronized", synch_total,  100.0 * synch_total  / total);
  tty->print_cr("\t%9d (%4.1f%%) final",        final_total,  100.0 * final_total  / total);
  tty->print_cr("\t%9d (%4.1f%%) static",       static_total, 100.0 * static_total / total);
  tty->print_cr("\t%9d (%4.1f%%) native",       nativ_total,  100.0 * nativ_total  / total);
  tty->print_cr("\t%9d (%4.1f%%) accessor",     acces_total,  100.0 * acces_total  / total);
  tty->cr();
  COMPILER2_ONLY(OptoRuntime::print_call_statistics(comp_total);)
}
# endif // CORE

void print_bytecode_count() {
  if (CountBytecodes || TraceBytecodes || StopInterpreterAt) {
    tty->print_cr("[BytecodeCounter::counter_value = %d]", BytecodeCounter::counter_value());
  }
}

AllocStats alloc_stats;



// General statistics printing (profiling ...)

void print_statistics() {
  
#ifdef ASSERT

  if (CountJNICalls) {
    extern Histogram *JNIHistogram;
    JNIHistogram->print();
  }

  if (CountJVMCalls) {
    extern Histogram *JVMHistogram;
    JVMHistogram->print();
  }

#endif

  if (MemProfiling) {
    MemProfiler::disengage();
  }

#ifndef CORE

#ifdef COMPILER1
  if (PrintC1Statistics && UseRecompilation) {
    Runtime1::print_statistics();
    Deoptimization::print_statistics();
  }
#endif

#ifdef COMPILER2
  if (PrintOptoStatistics && UseRecompilation) {
    Parse::print_statistics();
    PhaseCCP::print_statistics();
    PhaseChaitin::print_statistics();
    Scheduling::print_statistics();
    Deoptimization::print_statistics();
    OptoRuntime::print_statistics();
    PhasePeephole::print_statistics();

    os::print_statistics();
  }
  if (TimeLivenessAnalysis) {
    MethodLiveness::print_times();
  }
#ifdef ASSERT
  if (CollectIndexSetStatistics) {
    IndexSet::print_statistics();
  }
#endif // ASSERT
#endif // COMPILER2
  if (CountCompiledCalls) {
    print_method_invocation_histogram();
  }
  if (TimeCompiler) {
    COMPILER2_ONLY(Compile::print_timers();)
  }
#endif // CORE
  if (TimeOopMap) {
    GenerateOopMap::print_time();
  }  
  if (ProfilerCheckIntervals) {
    PeriodicTask::print_intervals();
  }
  if (PrintSymbolTableSizeHistogram) {
    SymbolTable::print_histogram();
  }
  if (CountBytecodes || TraceBytecodes || StopInterpreterAt) {
    BytecodeCounter::print();
  }  
  if (PrintBytecodePairHistogram) {
    BytecodePairHistogram::print();
  }

#ifndef CORE
  if (PrintCodeCache) {
    MutexLocker mu(CodeCache_lock);
    CodeCache::print();
  }

  if (PrintCodeCache2) {
    MutexLocker mu(CodeCache_lock);
    CodeCache::print_internals();
  }
#endif // CORE

  if (PrintClassStatistics) {
    SystemDictionary::print_class_statistics();
  }
  if (PrintMethodStatistics) {
    SystemDictionary::print_method_statistics();
  }

  if (PrintVtableStats) {
    klassVtable::print_statistics();
    klassItable::print_statistics();
  }
  if (TraceClassLoadingTime) {
    double time = ClassLoader::accumulated_time()->seconds();
    tty->print_cr("[Accumulated class loading time %3.7f secs]", time);
  }
  if (TraceScavengeTime) {
    double time = Scavenge::accumulated_time()->seconds();
    tty->print_cr("[Accumulated scavenge time %3.7f secs]", time);
  }
  if (TraceMarkSweepTime) {
    double time = MarkSweep::accumulated_time()->seconds();
    tty->print_cr("[Accumulated mark sweep time %3.7f secs]", time);
  }
  if (TraceMaxHeapCapacity) {
    Universe::print_max_heap_capacity();
  }
  if (VerifyOops) {
    tty->print_cr("+VerifyOops count: %d", StubRoutines::verify_oop_count());
  }

  print_bytecode_count();
  if (WizardMode) {
    tty->print("allocation stats: ");
    alloc_stats.print();
    tty->cr();
  }

  if (PrintBCIHistogram) {
    Universe::print_bci_histogram();
  }
  if (PrintSystemDictionaryAtExit) {
    SystemDictionary::print();
  }

#ifdef ENABLE_ZAP_DEAD_LOCALS
#ifdef COMPILER2
  if (ZapDeadCompiledLocals) {
    tty->print_cr("Compile::CompiledZap_count = %d", Compile::CompiledZap_count);
    tty->print_cr("OptoRuntime::ZapDeadCompiledLocals_count = %d", OptoRuntime::ZapDeadCompiledLocals_count);
  }
#endif // COMPILER2
#endif // ENABLE_ZAP_DEAD_LOCALS
}

#endif


// Helper class for registering on_exit calls through JVM_OnExit

extern "C" {
    typedef void (*__exit_proc)(void);
}

class ExitProc : public CHeapObj {
 private:
  __exit_proc _proc;
  // void (*_proc)(void);
  ExitProc* _next;
 public:
  // ExitProc(void (*proc)(void)) {
  ExitProc(__exit_proc proc) {
    _proc = proc;
    _next = NULL;
  }
  void evaluate()               { _proc(); }
  ExitProc* next() const        { return _next; }
  void set_next(ExitProc* next) { _next = next; }
};


// Linked list of registered on_exit procedures

static ExitProc* exit_procs = NULL;


extern "C" {
  void register_on_exit_function(void (*func)(void)) {
    ExitProc *entry = new ExitProc(func);
    // Classic vm does not throw an exception in case the allocation failed, 
    if (entry != NULL) {
      entry->set_next(exit_procs);
      exit_procs = entry;
    }
  }
}


void before_exit() {
  // The only difference between this and Win32's _onexit procs is that 
  // this version is invoked before any threads get killed.
  ExitProc* current = exit_procs;
  while (current != NULL) {
    ExitProc* next = current->next();
    current->evaluate();
    delete current;
    current = next;
  }
  // Print statistics gathered (profiling ...)
  if (Arguments::has_profile()) {    
    FlatProfiler::disengage();
    FlatProfiler::print(10);
  }
  if (Arguments::has_alloc_profile()) {
    // Will perform an object_iteration
    // This should be done at a safepoint, or after suspending
    // all threads, but it is not.
    // Ensure Eden top is correct before doing object_iteration
    Universe::new_gen()->eden()->ensure_proper_top();
    AllocationProfiler::disengage();
    AllocationProfiler::print(0);
  }
  if (PrintBytecodeHistogram) {
    BytecodeHistogram::print();
  }
#ifndef PRODUCT
#ifdef COMPILER2
  if (DumpInlineImage) {
    // For easy diffing, re-read the image file
    // and write it back in sorted form
    ResourceMark rm;
    HandleMark hm;
    InlineImage::parse_from_file();
    InlineImage::sort_by_name();
    InlineImage::dump_to_file();
  }
#endif
#endif
  if (jvmdi::enabled()) {
    jvmdi::post_vm_death_event();
  }
  if (jvmpi::is_event_enabled(JVMPI_EVENT_JVM_SHUT_DOWN)) {
    jvmpi::post_vm_death_event();
  }
  NOT_PRODUCT(print_statistics();)
  VTune::exit();
}


void vm_exit(int code) {  
  // Warning: do not call 'exit_globals()' here. All threads are still running. 
  // Calling 'exit_globals()' will disable thread-local-storage and cause all kinds 
  // of assertions to trigger in debug mode.
  if (is_init_completed()) {
    Thread* thread = Thread::current();
    if (thread->is_Java_thread()) {
      // We are leaving the VM, set state to native (in case any OS exit handlers call back to the VM)
      ((JavaThread*)thread)->set_thread_state(_thread_in_native);
    }
  }
  os::exit(code); 
  ShouldNotReachHere();
}


void vm_abort() {
// Warning: do not call 'exit_globals()' here. All threads are still running. 
  // Calling 'exit_globals()' will disable thread-local-storage and cause all kinds 
  // of assertions to trigger in debug mode.
  if (is_init_completed()) {
    Thread* thread = Thread::current();
    if (thread->is_Java_thread()) {
      // We are leaving the VM, set state to native (in case any OS exit handlers call back to the VM)
      ((JavaThread*)thread)->set_thread_state(_thread_in_native);
    }
  }
  os::abort(PRODUCT_ONLY(false)); // No core dump in product build.
  ShouldNotReachHere();
}

void vm_exit_during_initialization(Handle exception) {
  tty->print_cr("Error occurred during initialization of VM");
  java_lang_Throwable::print(exception, tty);
  tty->cr();
  if (ShowMessageBoxOnError && WizardMode) fatal("Error occurred during initialization of VM");
  vm_abort();
}

void vm_exit_during_initialization(symbolHandle exception_name, const char* message) {
  tty->print_cr("Error occurred during initialization of VM");
  ResourceMark rm;
  tty->print_cr("%s: %s", exception_name->as_C_string(), message == NULL ? "" : message);
  if (ShowMessageBoxOnError && WizardMode) fatal("Error occurred during initialization of VM");
  vm_abort();
}


void vm_exit_during_initialization(const char* error, const char* message) {
  tty->print_cr("Error occurred during initialization of VM");
  tty->print("%s", error);
  if (message != NULL) tty->print(": %s", message);
  tty->cr();
  if (ShowMessageBoxOnError && WizardMode) fatal("Error occurred during initialization of VM");
  vm_abort();
}


void vm_exit_out_of_memory(int size, const char* name) {
  tty->print("Exception ");
  Thread* thread = ThreadLocalStorage::get_thread_slow();
  if (thread != NULL && thread->is_Java_thread() && ((JavaThread*) thread)->threadObj() != NULL) {
    ResourceMark rm;
    tty->print("in thread \"%s\" ", ((JavaThread*) thread)->get_thread_name());
  }
  tty->print("java.lang.OutOfMemoryError: requested %d bytes", size);
  NOT_PRODUCT(if (name != NULL) tty->print(" for %s",name);)
  tty->cr();
  vm_abort();
}
