/*
 * @(#)memprofiler.cpp	1.10 99/06/22 16:43:48
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_memprofiler.cpp.incl"

#ifndef PRODUCT

// --------------------------------------------------------
// MemProfilerTask

class MemProfilerTask : public PeriodicTask {
 public:
  MemProfilerTask(int interval_time) : PeriodicTask(interval_time) {}
  void task();
};


void MemProfilerTask::task() {
  // Get thread lock to provide mutual exclusion, and so we can iterate safely over the thread list.
  MutexLocker mu(Threads_lock);
  MemProfiler::do_trace();
}


//----------------------------------------------------------
// Implementation of MemProfiler

MemProfilerTask* MemProfiler::_task   = NULL;
FILE*            MemProfiler::_log_fp = NULL;


bool MemProfiler::is_active() {
  return _task != NULL;
}


void MemProfiler::engage() {
  const char *log_name = "mprofile.log";
  if (!is_active()) {
    // Create log file
    _log_fp = fopen(log_name , "w+");
    if (_log_fp == NULL) {
      fatal1("MemProfiler: Cannot create log file: %s", log_name);
    }
    fprintf(_log_fp, "MemProfiler: sizes are in Kb, time is in seconds since startup\n\n");
    fprintf(_log_fp, "  time, #thr, #cls,  heap,  heap,  perm,  perm,  code, hndls, rescs, oopmp\n");
    fprintf(_log_fp, "                     used, total,  used, total, total, total, total, total\n"); 
    fprintf(_log_fp, "--------------------------------------------------------------------------\n");     

    _task = new MemProfilerTask(MemProfilingInterval);
    _task->enroll();    
  }
}


void MemProfiler::disengage() {
  if (!is_active()) return;
  // Do one last trace at disengage time
  do_trace();

  // Close logfile
  fprintf(_log_fp, "MemProfiler detached\n");
  fclose(_log_fp);

  // remove MemProfilerTask
  assert(_task != NULL, "sanity check");
  _task->deroll();
  delete _task;
  _task = NULL;
}


void MemProfiler::do_trace() {  
  // Calculate thread local sizes
  long handles_memory_usage    = VMThread::vm_thread()->handle_area()->size_in_bytes();
  long resource_memory_usage   = VMThread::vm_thread()->resource_area()->size_in_bytes();
  JavaThread *cur = Threads::first();
  while (cur != NULL) {    
    handles_memory_usage  += cur->handle_area()->size_in_bytes();
    resource_memory_usage += cur->resource_area()->size_in_bytes();
    cur = cur->next();
  }
  
  // Print trace line in log
  fprintf(_log_fp, "%6.1f,%5d,%5d,%6d,%6d,%6d,%6d,", 
      os::elapsedTime(),
      Threads::number_of_threads(),
      SystemDictionary::number_of_classes(),
      Universe::mem_used() / K,
      Universe::mem_capacity() / K,
      Universe::perm_gen()->used() / K,
      Universe::perm_gen()->capacity() / K);

  NOT_CORE(fprintf(_log_fp, "%6d,", CodeCache::capacity() / K);)

  fprintf(_log_fp, "%6d,%6d,%6d\n",
      handles_memory_usage / K,
      resource_memory_usage / K,
      OopMapCache::memory_usage() / K);
  fflush(_log_fp);
}

#endif 
