/*
 * @(#)methodTracer.cpp	1.10 00/03/21 16:17:56
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_sharedRuntime.cpp.incl"


// Usage Instructions for -XX:+TraceMethods
//
// Either,
//   a. Get debugging Macros (from Stephen Glim) to set breakpoints in breakForMethod &
//      set the value of _bDisplay or
//
//   b. Modify the method "breakForMethod" to set your own breakpoints for
//      a particular method.  
//
// The latter is easier to understand, but less convienient. The former requires
// you have a debugger that understands macros.
//
//
// sType  =  [ "entry" | "exit" ]
// sClass =  "<classnmame>", for example "java/lang/Object"
// sMethod=  "<methodname>", for example "write"


#ifndef PRODUCT

bool MethodTracer::_bUsage = false;

void MethodTracer::breakForMethod(const char *sType, char *sClass, 
  char *sMethod) {
  if (!_bUsage) {
    if (!_bDisplay) {
      tty->print("Trace Methods Enabled... ");
      tty->print("See runtime/MethodTracer::breakForMethod() for Usage instructions");
      tty->cr();
    }
    _bUsage = true;
  }

#if 0
  char *myType  = "entry";
  char *myClass = "java/lang/Object";
  char *myMethod= "<init>";

  if (   (strcmp(sType, myType) == 0)
      && (strcmp(sClass, myClass) == 0)
      && (strcmp(sMethod, myMethod) == 0) ) {

    breakpoint();
    _bDisplay = true;

  }
#endif // if 0
}


MethodTracer::MethodTracer() : _MAX_INDENT(40), _methodStack(NULL), _bDisplay(false) {
  // Use a _methodStack to verify that entries and exits match up
  //
  //_methodStack = new GrowableArray<methodOop>(10,true);
  _indent = 0;   
}


// Push/Pop methodOops to verify that # of entries matches # of exits

void MethodTracer::pushMethod(methodOop m) {
  if (_methodStack == NULL) return;
  _methodStack->push(m); 
}

methodOop MethodTracer::popMethod(methodOop m) {
  if (_methodStack == NULL) return NULL;
  methodOop mstack = _methodStack->pop();
  return mstack;
} 


// output spaces so printed Entries & Exits indent nicely

void MethodTracer::doIndent(outputStream *out, bool addToIndent, int dindent) {
  assert((dindent == 1) || (dindent == -1), "dindent must be +/-1");

  if (addToIndent && (dindent == -1)) _indent--;

  out->print("[%4d]",_indent);
  for (int i=0; i< (_indent % _MAX_INDENT); i++)  out->print(" ");      
  out->print((dindent > 0) ? "+" : "-");

  if (addToIndent && (dindent == 1)) _indent++;
}


// Dump a description of method to outputStream *out

void MethodTracer::describe(outputStream *out, int dindent, address pc, methodOop method) {
  // 'C' = Calling from compiled fragment
  // 'I' = Calling from the interpreter
  char entry   = (method == NULL) ? 'C' : 'I'; 
  CodeBlob* cb;

#ifndef CORE
  if (method == NULL) {
    cb = CodeCache::find_blob(pc);	      
    if (cb != NULL && cb->is_nmethod()) method = ((nmethod*)cb)->method();
  }	
#endif

  methodOop poppedMethod = NULL;
  char *methodName = NULL;

  if (method != NULL) {
    if (dindent > 0)  pushMethod(method);	  
    else              poppedMethod = popMethod(method);
  }

  if (method != NULL) {
    const char *sType = (dindent == 1) ? "entry" : "exit";
    char *sClass  = method->method_holder()->klass_part()->name()->as_C_string();
    char *sMethod = method->name()->as_C_string();
    breakForMethod(sType,sClass,sMethod);
  }

  if (_bDisplay) write(out,pc,cb,method,poppedMethod,entry,dindent);
}


void MethodTracer::write(
  outputStream *out, address pc, CodeBlob *cb,
  methodOop method, methodOop poppedMethod, 
  char entry, int dindent
) {
  ResourceMark rm;

  if (poppedMethod && (dindent < 0) && (poppedMethod != method)) {
    out->print("=========================");
    out->cr();
    out->print("Unmatched method popped from stack: ");
    poppedMethod->print_name(out);
    out->cr();
    out->print("method being exited: ");
    method->print_name(out);
    out->cr();
    out->print("=========================");
    out->cr();
  }

  if (cb != NULL) {
    out->print(" [0x%8x-0x%8x] ",cb->instructions_begin(),cb->instructions_end());
  } else if (entry == 'i') {
    out->print(" [0x%8x-0x%8x] ",BytecodeCounter::counter_value(),0);
  } else {
    out->print(" [0x%8x-0x%8x] ",0,0);
	}

  doIndent(out,(method != NULL), dindent);
  out->print("(%c%c) ",entry, (method != NULL) ? 'm' : '*');	  

  if (method != NULL) {
    method->print_name(out);
  } else {
    if (cb != NULL) {
      cb->print_value_on(out);
    } else {
      StubCodeDesc* d = StubCodeDesc::desc_for(pc);
      if (d != NULL) { 
        out->print(d->group());
        out->print("::");
        out->print(d->name());
        out->print(" [0x%x, 0x%x[ (%d bytes)", d->begin(), d->end(), d->size_in_bytes());
      } else {	    
	      InterpreterCodelet* i = Interpreter::codelet_containing(pc);
	      if (i != NULL) { 
	        if (i->description() != NULL) out->print("%s  ", i->description());
	        if (i->bytecode()    >= 0   ) out->print("%s  ", Bytecodes::name(i->bytecode()));
	      }	else if (StubRoutines::contains(pc)) {
	        out->print_cr("An (unnamed) stub routine");	 
	      } else if (Interpreter::contains(pc)) {
	        out->print_cr("Interpreter code (not bytecode specific)");
	      } else {
	        out->print("%8x: ??? ", pc);
        }
      }
    }
  }
  out->cr();
}


// static method, creates a new MethodTracer for a thread if one
// doesn't yet exist.

MethodTracer *MethodTracer::attachThread(JavaThread *thread) {
  MethodTracer* mt = thread->get_method_tracer();
  if (mt == NULL) {
    mt = new MethodTracer();
    thread->set_method_tracer(mt);
  }
  return mt;
}


void MethodTracer::entry(JavaThread *thread, methodOop method, address pc) {
  Unimplemented();
  /*
  MethodTracer *mt(MethodTracer::attachThread(thread));
  mt->describe(tty,+1,pc,method);
  */
}


void MethodTracer::exit(JavaThread *thread, methodOop method, address pc) {
  MethodTracer* mt = MethodTracer::attachThread(thread);
  mt->describe(tty,-1,pc, method);
}


#endif // !PRODUCT
