/*
 * @(#)mutexLocker.cpp	1.102 01/04/21 22:47:58
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_mutexLocker.cpp.incl"

// Mutexes used in the VM (see comment in mutexLocker.hpp)

Mutex*   Lookup_lock                  = NULL;
Mutex*   InterfaceLookup_lock         = NULL;
Mutex*   Event_lock                   = NULL;
Mutex*   SystemDictionary_lock        = NULL;
Mutex*   PackageTable_lock            = NULL;
Mutex*   PointerMap_lock              = NULL;
Mutex*   Patching_lock                = NULL;
Mutex*   CompiledIC_lock              = NULL;
Mutex*   VMStatistic_lock             = NULL;
Mutex*   CVWaitTable_lock             = NULL;
Mutex*   JNIGlobalHandle_lock         = NULL;
Mutex*   JNIHandleBlockFreeList_lock  = NULL;
Mutex*   JNIIdentifier_lock           = NULL;
Mutex*   ExpandHeap_lock              = NULL;
Mutex*   RecompilationMonitor_lock    = NULL;
Mutex*   SignatureHandlerLibrary_lock = NULL;
Mutex*   SymbolTable_lock             = NULL;
Mutex*   StringTable_lock             = NULL;
Mutex*   AdapterCache_lock            = NULL;
Mutex*   VtableStubs_lock             = NULL;
Mutex*   ThreadMap_lock               = NULL;
Monitor* VMOperationQueue_lock        = NULL;
Monitor* VMOperationRequest_lock      = NULL;
Monitor* Safepoint_lock               = NULL;
Monitor* Threads_lock                 = NULL;
Monitor* ThreadBlock_lock             = NULL;
Monitor* Compile_lock                 = NULL;
Monitor* AdapterCompileQueue_lock     = NULL;
Monitor* MethodCompileQueue_lock      = NULL;
Monitor* CompileTaskAlloc_lock        = NULL;
Monitor* CompileStatistics_lock       = NULL;
Mutex*   MultiArray_lock              = NULL;
Monitor* Terminator_lock              = NULL;
Mutex*   InlineCacheBuffer_lock       = NULL;
Mutex*   CodeCache_lock               = NULL;
Mutex*   VMThreadBoosting_lock        = NULL;
Monitor* Notify_lock                  = NULL;
Monitor* Interrupt_lock               = NULL;
Monitor* SuspendChecker_lock          = NULL;
Monitor* ProfileVM_lock               = NULL;
Monitor* Process_lock                 = NULL;
Mutex*   SocketInit_lock              = NULL;
Mutex*   ProfilePrint_lock            = NULL;
Mutex*   ExceptionCache_lock          = NULL;
Monitor* Signal_lock                  = NULL;
Mutex*   JVMDI_field_access_lock      = NULL;
Mutex*   JVMDI_field_modification_lock= NULL;
Mutex*   JVMDI_frames_lock 	      = NULL;
Mutex*   OsrList_lock                 = NULL;

Mutex*   Debug1_lock                  = NULL;
Mutex*   Debug2_lock                  = NULL;
Mutex*   Debug3_lock                  = NULL;

Mutex*   MonitorCache_lock            = NULL;

#ifdef ASSERT
void assert_lock(Mutex* lock) {
  // check if this thread owns the lock (common case)
  if (IgnoreLockingAssertions) return;
  if (lock->owned_by_self()) return;
  if (SafepointSynchronize::is_at_safepoint()) return;
  if (!Universe::is_fully_initialized()) return;
  // see if invoker of VM operation owns it
  VM_Operation* op = VMThread::vm_operation();
  if (op != NULL && op->calling_thread() == lock->owner()) return;
  fatal1("must own lock %s", lock->name());
}
#endif

#define def(var, type, pri, vm_block) \
  var = new type(Mutex::##pri, #var, vm_block)

void mutex_init() {  
  def(Patching_lock                , Mutex  , leaf,        true );  
  def(Threads_lock                 , Monitor, barrier,     true ); 
  def(Lookup_lock                  , Mutex  , safepoint,   true ); // used when doing a flush; may lock InterfaceLookup_lock
  def(InterfaceLookup_lock         , Mutex  , leaf,        true ); // used for allocating/recycling InterfaceLkupCache entries
  def(SystemDictionary_lock        , Mutex  , leaf,        true ); // lookups done by VM thread
  def(PackageTable_lock            , Mutex  , leaf,        false);
  def(AdapterCache_lock            , Mutex  , leaf,        true );
  def(PointerMap_lock              , Mutex  , leaf,        false);
  def(CompiledIC_lock              , Mutex  , nonleaf + 1, false); // locks VtableStubs_lock, InlineCacheBuffer_lock
  def(InlineCacheBuffer_lock       , Mutex  , leaf,        true );
  def(VMStatistic_lock             , Mutex  , leaf,        false);
  def(CVWaitTable_lock             , Mutex  , leaf,        false);
  def(ExpandHeap_lock              , Mutex  , leaf,        true ); // Used during compilation by VM thread
  def(JNIGlobalHandle_lock         , Mutex  , nonleaf,     true ); // locks JNIHandleBlockFreeList_lock
  def(JNIHandleBlockFreeList_lock  , Mutex  , leaf,        true ); // handles are used by VM thread
  def(JNIIdentifier_lock           , Mutex  , leaf,        false);
  def(RecompilationMonitor_lock    , Mutex  , leaf,        true );
  def(SignatureHandlerLibrary_lock , Mutex  , leaf,        false);
  def(SymbolTable_lock             , Mutex  , leaf,        true );
  def(StringTable_lock             , Mutex  , leaf,        true );
  def(VtableStubs_lock             , Mutex  , nonleaf,     true );
  def(ThreadMap_lock               , Mutex  , leaf,        true );
  def(Notify_lock                  , Monitor, nonleaf,     true ); 
  def(ThreadBlock_lock             , Monitor, leaf,        false);  
  def(VMThreadBoosting_lock        , Mutex  , leaf,        true);  

  def(Compile_lock                 , Monitor, nonleaf + 2, true ); 
  def(AdapterCompileQueue_lock     , Monitor, nonleaf + 3, true ); 
  def(MethodCompileQueue_lock      , Monitor, nonleaf + 3, true ); 
  def(CompileTaskAlloc_lock        , Monitor, nonleaf + 1, true ); 
  def(CompileStatistics_lock       , Monitor, nonleaf + 1, false); 
  def(MultiArray_lock              , Mutex  , nonleaf + 1, false); // locks SymbolTable_lock
  def(Terminator_lock              , Monitor, nonleaf,     true );
  def(CodeCache_lock               , Mutex  , special,     true );
  def(ProfilePrint_lock            , Mutex  , leaf,        false); // serial profile printing
  def(ExceptionCache_lock          , Mutex  , leaf,        false); // serial profile printing
  def(Signal_lock                  , Monitor, nonleaf,     true ); // sun.misc.Signal
  def(SocketInit_lock              , Mutex  , special,     false); // socket functions are inited
  def(OsrList_lock                 , Mutex  , leaf,        true );
  def(JVMDI_field_access_lock      , Mutex  , nonleaf + 3, true ); // lock for JVM/DI field access counter
  def(JVMDI_field_modification_lock, Mutex  , nonleaf + 3, true ); // lock for JVM/DI field modification counter
  def(JVMDI_frames_lock            , Mutex  , nonleaf + 3, true ); // lock for JVM/DI frame cache
    
  def(Debug1_lock                  , Mutex  , leaf       , true );
  def(Debug2_lock                  , Mutex  , nonleaf + 3, true );
  def(Debug3_lock                  , Mutex  , nonleaf + 3, true );

  def(VMOperationQueue_lock        , Monitor, nonleaf,     true ); // VM_thread allowed to block on these     
  def(VMOperationRequest_lock      , Monitor, nonleaf,     true ); 
  def(Safepoint_lock               , Monitor, safepoint,   true ); // locks SnippetCache_lock/Threads_lock
  def(Event_lock                   , Mutex  , event,       true ); // allow to lock in VM
  def(Interrupt_lock               , Monitor, special,     true ); // used for suspension/resumption
  def(SuspendChecker_lock          , Monitor, special,     true ); // used for VMThread deadlock detection
  def(ProfileVM_lock               , Monitor, nonleaf+3,   false); // used for profiling of the VMThread
  
  def(Process_lock                 , Monitor, nonleaf,     false); // used for fork/exec
  def(MonitorCache_lock            , Mutex,   special,     true ); // used for global monitor cache
}

#ifndef PRODUCT
static Mutex *TraceLocking_lock = NULL;

void trace_locking(Mutex* mutex, char *type, char *state) {
  if (TraceLocking_lock == NULL) {
    TraceLocking_lock = new Mutex(1, "TraceLocking_lock");
  }
  TraceLocking_lock->lock(); // Do not use MutexLocker here (infinite recursion)
  tty->print("%s thread=0x%x  lock=%s %s", type, ThreadLocalStorage::thread(), mutex->name(), state);  
  TraceLocking_lock->unlock();
}
#endif


