/*
 * @(#)mutexLocker.hpp	1.77 01/04/21 22:48:00
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Mutexes used in the VM.

extern Mutex*   Lookup_lock;                     // a lock for the method lookup table
extern Mutex*   InterfaceLookup_lock;            // a lock for interface dispatch lookup
extern Mutex*   Patching_lock;                   // a lock used to guard code patching of compiled code
extern Mutex*   SystemDictionary_lock;           // a lock on the system dictonary
extern Mutex*   PackageTable_lock;               // a lock on the class loader package table
extern Mutex*   PointerMap_lock;                 // a lock used to guard pointer map creation when using static  variables in iterator
extern Mutex*   CompiledIC_lock;                 // a lock used to guard compiled IC patching and access
extern Mutex*   InlineCacheBuffer_lock;          // a lock used to guard the InlineCacheBuffer
extern Mutex*   VMStatistic_lock;                // a lock used to guard statistics count increment
extern Mutex*   CVWaitTable_lock;                // a lock on the condition variable wait table
extern Mutex*   JNIGlobalHandle_lock;            // a lock on creating JNI global handles
extern Mutex*   JNIHandleBlockFreeList_lock;     // a lock on the JNI handle block free list
extern Mutex*   JNIIdentifier_lock;              // a lock on creating JNI method/static field identifiers
extern Mutex*   ExpandHeap_lock;                 // a lock on expanding the heap
extern Mutex*   RecompilationMonitor_lock;       // a lock to guard the RecompilationMonitor
extern Mutex*   SignatureHandlerLibrary_lock;    // a lock on the SignatureHandlerLibrary
extern Mutex*   VtableStubs_lock;                // a lock on the VtableStubs
extern Mutex*   SymbolTable_lock;                // a lock on the symbol table
extern Mutex*   StringTable_lock;                // a lock on the interned string table
extern Mutex*   AdapterCache_lock;               // a lock on the C2I/I2C Adapter cache's
extern Mutex*   ThreadMap_lock;                  // a lock on the ThreadMap table
extern Mutex*   CodeCache_lock;                  // a lock on the CodeCache
extern Mutex*   VMThreadBoosting_lock;           // a lock on boosting of the VMThread
extern Monitor* VMOperationQueue_lock;	         // a lock on queue of vm_operations waiting to execute
extern Monitor* VMOperationRequest_lock;         // a lock on Threads waiting for a vm_operation to terminate
extern Monitor* Safepoint_lock;                  // a lock used by the safepoint abstraction
extern Monitor* ThreadBlock_lock;                // a lock used to block threads on during a safepoint
extern Monitor* Threads_lock;                    // a lock on the Threads table of active Java threads 
                                                 // (also used by Safepoints too to block threads creation/destruction)
extern Monitor* Compile_lock;                    // a lock held when Compilation is updating code (used to block CodeCache traversal, CHA updates, etc)
extern Monitor* AdapterCompileQueue_lock;        // a lock held when adapter compilations are enqueued, dequeued
extern Monitor* MethodCompileQueue_lock;         // a lock held when method compilations are enqueued, dequeued
extern Monitor* CompileTaskAlloc_lock;           // a lock held when CompileTasks are allocated
extern Monitor* CompileStatistics_lock;          // a lock held when updating compilation statistics
extern Mutex*   MultiArray_lock;                 // a lock used to guard allocation of multi-dim arrays
extern Monitor* Terminator_lock;                 // a lock used to guard termination of the vm
extern Monitor* Notify_lock;                     // a lock used to synchronize the start-up of the vm
extern Monitor* Interrupt_lock;                  // a lock used for condition variable mediated suspension and interrupt processing
extern Monitor* SuspendChecker_lock;             // a lock used for VMThread deadlock detection during suspend operations
extern Monitor* ProfileVM_lock;                  // a lock used for profiling the VMThread
extern Monitor* Process_lock;                    // a lock used to synchronize fork and exec in unix
extern Mutex*   SocketInit_lock;                 // a lock used to synchronize the winsock lazy initialization
extern Mutex*   ProfilePrint_lock;               // a lock used to serialize the printing of profiles
extern Mutex*   ExceptionCache_lock;             // a lock used to synchronize exception cache updates
extern Monitor* Signal_lock;                     // used to implement sun.misc.Signal (optional)
extern Mutex*   OsrList_lock;                    // a lock used to serialize access to OSR queues
extern Mutex*   JVMDI_field_access_lock;         // a lock used to synchronize field access counter updates
extern Mutex*   JVMDI_field_modification_lock;   // a lock used to synchronize field modification counter updates
extern Mutex*   JVMDI_frames_lock;               // a lock used to synchronize updates to cached frame array
extern Mutex*   Debug1_lock;                     // A bunch of pre-allocated locks that can be used for tracing
extern Mutex*   Debug2_lock;                     // down synchronization related bugs!
extern Mutex*   Debug3_lock;

extern Mutex*   Event_lock;                      // a lock on the event queue

extern Mutex*   MonitorCache_lock;               // a lock on the global monitor cache

// A MutexLocker provides mutual exclusion with respect to a given mutex
// for the scope which contains the locker.  The lock is an OS lock, not
// an object lock, and the two do not interoperate.  Do not use Mutex-based
// locks to lock on Java objects, because they will not be respected if a
// that object is locked using the Java locking mechanism.

void trace_locking(Mutex* mutex, char *type, char *state);

char *lock_name(Mutex *mutex);

class MutexLocker: StackObj {
 private:
  Mutex* _mutex;
 public:
  MutexLocker(Mutex* mutex) {
    _mutex = mutex;
    _mutex->lock();
  }

  ~MutexLocker() {    
    _mutex->unlock();
  }
};

// for debugging: check that we're already owning this lock (or are at a safepoint)
#ifdef ASSERT
void assert_lock(Mutex* lock);
#else
#define assert_lock(lock)
#endif

// A MutexLockerEx behaves like a MutexLocker when its constructor is
// called with a Mutex.  Unlike a MutexLocker, its constructor can also be
// called with NULL, in which case the MutexLockerEx is a no-op.  There
// is currrently no corresponding MutexUnlockerEx.  We want to keep the
// basic MutexLocker as fast as possible.  MutexLockerEx can also lock
// without safepoint check.

class MutexLockerEx: StackObj {
 private:
  Mutex* _mutex;
 public:
  MutexLockerEx(Mutex* mutex, bool no_safepoint_check = !Mutex::_no_safepoint_check_flag) {
    _mutex = mutex;
    if (_mutex != NULL) {
      if (no_safepoint_check)
	_mutex->lock_without_safepoint_check();
      else
	_mutex->lock();
    }
  }

  ~MutexLockerEx() {
    if (_mutex != NULL) {
      _mutex->unlock();
    }
  }
};

// A MutexUnlocker temporarily exits a previously
// entered mutex for the scope which contains the unlocker.

class MutexUnlocker: StackObj {
 private:
  Mutex* _mutex;

 public:
  MutexUnlocker(Mutex* mutex) {
    _mutex = mutex;
    _mutex->unlock();
  }

  ~MutexUnlocker() {
    _mutex->lock();
  }
};

#ifndef PRODUCT
//
// A special MutexLocker that allows:
//   - reentrant locking
//   - locking out of order
//
// Only too be used for verify code, where we can relaxe out dead-lock
// dection code a bit (unsafe, but probably ok). This code is NEVER to
// be included in a product version.
//
class VerifyMutexLocker: StackObj {
 private:
  Mutex* _mutex;
  bool   _reentrant;
 public:
  VerifyMutexLocker(Mutex* mutex) {
    _mutex     = mutex;
    _reentrant = mutex->owned_by_self();
    if (!_reentrant) {
      // We temp. diable strict safepoint checking, while we require the lock
      FlagSetting fs(StrictSafepointChecks, false);    
      _mutex->lock();
    }
  }
  
  ~VerifyMutexLocker() {
    if (!_reentrant) {     
      _mutex->unlock();
    }
  }
};

#endif

