/*
 * @(#)objectMonitor.hpp	1.18 01/02/09 14:47:45
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// WARNING:
//   This is a very sensitive and fragile class. DO NOT make any
// change unless you are fully aware of the underlying semantics.

//   This class can not inherit from any other class, because I have
// to let the displaced header be the very first word. Otherwise I
// have to let markOop include this file, which would export the
// monitor data structure to everywhere.
//
// The ObjectMonitor class is used to implement JavaMonitors which have
// transformed from the lightweight structure of the thread stack to a
// heavy weight lock due to contention

// It is also used as RawMonitor by the JVMPI


class ObjectWaiter;

class ObjectMonitor {
 public:
  enum {
    OM_OK,                    // no error
    OM_SYSTEM_ERROR,          // operating system error
    OM_ILLEGAL_MONITOR_STATE, // IllegalMonitorStateException
    OM_INTERRUPTED,           // Thread.interrupt()
    OM_TIMED_OUT              // Object.wait() timed out
  };

 public:
  static int header_offset_in_bytes() { return (int)&((ObjectMonitor*)NULL)->_header; }
  static int object_offset_in_bytes() { return (int)&((ObjectMonitor*)NULL)->_object; }
  static int owner_offset_in_bytes()  { return (int)&((ObjectMonitor*)NULL)->_owner; }
  static int count_offset_in_bytes()  { return (int)&((ObjectMonitor*)NULL)->_count; }

 public:
  ObjectMonitor();
  ~ObjectMonitor();

  markOop   header() const;
  void      set_header(markOop hdr);

  intptr_t  is_busy() const;
  intptr_t  is_enterred(Thread* current) const;
  
  void*     owner() const;
  void      set_owner(void* owner);

  intptr_t    waiters() const;
  void*     queue() const;
  void      set_queue(void* queue);

  intptr_t  count() const;
  void      set_count(intptr_t count);

  void*     object() const;
  void*     object_addr();
  void      set_object(void* obj);

  bool      check(TRAPS);	// true if the thread owns the monitor.
  void      check_slow(TRAPS);
  void      clear();
#ifndef PRODUCT
  void      verify();
  void      print();
#endif
  
  void      enter(TRAPS);
  void      exit(TRAPS);
  void      wait(jlong millis, bool interruptable, TRAPS);
  void      notify(TRAPS);
  void      notifyAll(TRAPS);

  int       raw_enter(TRAPS, bool isRawMonitor);
  int       raw_exit(TRAPS, bool isRawMonitor);
  int       raw_wait(jlong millis, bool interruptable, TRAPS);
  int       raw_notify(TRAPS);
  int       raw_notifyAll(TRAPS);
  

 private:
  friend class ObjectSynchronizer;
  friend class ObjectWaiter;
  friend class RawMonitor;

  // WARNING: this must be the very first word of ObjectMonitor
  markOop   _header;      // displaced object mark

  // All the following fields must be machine word aligned
  void*     _object;      // backward object pointer
  void*     _owner;       // pointer to owning thread OR BasicLock
  void*     _queue;       // waiting queue structure
                          // (also used by ObjectSynchronizer)
  intptr_t  _count;       // number of entering threads
  intptr_t  _waiters;     // number of waiting threads
  intptr_t  _recursions;  // recursion count, 0 for first entry

  // the following contains processor/os specific definitions
  #include "incls/_objectMonitor_pd.hpp.incl"
};


