/*
 * @(#)os.hpp	1.128 01/03/05 12:11:06
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// os defines the interface to operating system; this includes traditional
// OS services (time, I/O) as well as other functionality with system-
// dependent code.

typedef void (*dll_func)(...);

class Thread;
class JavaThread;
class Event;
class DLL;
class FileHandle;

// %%%%% Moved ThreadState, START_FN, OSThread to new osThread.hpp. -- Rose

// Platform-independent error return values from OS functions
enum OSReturn {
  OS_OK         =  0,        // Operation was successful
  OS_ERR        = -1,        // Operation failed 
  OS_INTRPT     = -2,        // Operation was interrupted
  OS_TIMEOUT    = -3,        // Operation timed out
  OS_NOMEM      = -5,        // Operation failed for lack of memory
  OS_NORESOURCE = -6         // Operation failed for lack of nonmemory resource
};

extern void os_init();
extern jint os_init_2();

enum ThreadPriority {        // JLS 20.20.1-3
  NoPriority       = -1,     // Initial non-priority value
  MinPriority      =  1,     // Minimum priority
  NormPriority     =  5,     // Normal (non-daemon) priority
  NearMaxPriority  =  9,     // High priority, used for VMThread
  MaxPriority      = 10      // Highest priority, used for WatcherThread
                             // and SuspendCheckerThread, ensures that
                             // VMThread doesn't starve profiler
};


// HotSpot integration note:

// Struct containing system dependent properties. 
// Matches struct props_md_t in interpreter.h 1.213 99/05/19 from JDK1.3 beta H build

typedef struct {
    char *sysclasspath;
    char *library_path;
    char *dll_dir;
    char *java_home;
    char *ext_dirs;
} props_md_t;


// Typedef for structured exception handling support 
typedef void (*java_call_t)(JavaValue* value, methodHandle* method, JavaCallArguments* args, Thread* thread);

class os: AllStatic {
 private:
  static OSThread*          _starting_thread;
  static props_md_t*        _system_properties;
 public:
  // File names are case-insensitive on windows only
  // Override me as needed
  static inline int    file_name_strcmp(char* s1, char* s2);

  static bool getenv(const char* name, char* buffer, int len);
  

  static jlong  javaTimeMillis();
  
  // Returns the elapsed time in seconds since the vm started.
  static double elapsedTime();
  
  // Interface to the win32 performance counter
  static jlong elapsed_counter();
  static jlong elapsed_frequency();

  // Interface for detecting multiprocessor system
  static bool is_MP();

  // Interface for stack banging (predetect possible stack overflow for
  // exception processing)
  static int bang_stack_size_for_exceptions();

  // OS interface to Virtual Memory
  static int   vm_page_size();
  static char* reserve_memory(size_t bytes);
  static char* reserve_memory_special(size_t bytes);
  static void  release_memory_special();
  static bool  commit_memory(char* addr, size_t bytes);
  static bool  uncommit_memory(char* addr, size_t bytes);
  static bool  release_memory(char* addr, size_t bytes);
  static bool  guard_memory(char* addr, size_t bytes);
  static bool  unguard_memory(char* addr, size_t bytes);
  static char* non_memory_address_word();

  // threads

  enum ThreadType {
    vm_thread,
    java_thread,
    compiler_thread,
    watcher_thread,
    suspend_checker_thread
  };

  static bool create_thread(Thread* thread, ThreadType thr_type);
  static bool create_main_thread(Thread* thread);
  static bool create_attached_thread(Thread* thread);
  static void start_thread(Thread* thread);

  static void initialize_thread();
  static void free_thread(OSThread* osthread);
  static void exit_thread();

  static int current_thread_id();
  static int current_process_id();
  static int sleep(Thread* thread, jlong ms, bool interruptable);
  static void yield();        // Yields to all threads with same priority
  static void yield_all(int attempts = 0); // Yields to all other threads including lower priority
  static void loop_breaker(int attempts);  // called from within tight loops to possibly influence time-sharing
  static OSReturn set_priority(Thread* thread, ThreadPriority priority);
  static OSReturn get_priority(Thread* thread, ThreadPriority& priority);
  static bool boost_priority  (Thread* thread, int max_java_priority);
  static bool deflate_priority(Thread* thread, int min_java_priority);

  static void interrupt(Thread* thread);
  static bool is_interrupted(Thread* thread, bool clear_interrupted);

  static long pd_suspend_thread(Thread* thread, bool _fence);
  static long pd_resume_thread(Thread* thread);
  static bool pd_pre_self_suspend(OSThread* thread);
  static void pd_post_self_suspend(OSThread* thread);
  
  static ExtendedPC fetch_top_frame(Thread* thread, jint** sp, jint** fp);
  static bool set_thread_pc(Thread* thread, ExtendedPC addr);
  static ExtendedPC get_thread_pc(Thread *thread);
  static void breakpoint();

  static int max_framesize_without_stack_banging();
  static address current_stack_pointer();
  static address current_stack_base();
  static address current_stack_limit();
  static int current_stack_size();

  static int message_box(const char* title, const char* message);

  static void exit(int num);

  // Terminate with an error.  Default is to generate a core file on platforms
  // that support such things.
  static void abort(bool dump_core = true);

  // Fatal error handling

  // install/deinstall a simple signal handler during fatal error handling to print
  // out some basic information in case the fatal error handler crashes.
  static void set_exception_handler_during_fatal_error(int sig, int is_install);

  // generate fatal error report
  static void report_fatal_error(outputStream *st, address pc, int sig);

  // fatal error handler
  static void handle_unexpected_exception(Thread * thread, int sig, address pc, void * extra_info);

  static void handle_recursive_fatal_error(int sig);

  // Dynamic library extension
  static const char*    dll_file_extension();

  static const char*    get_temp_directory();

  // Map pc in jvm.dll/libjvm.so to symbol. The resulting string is not guaranteed to be
  // preserved between calls, so the caller will have to copy it. This call is currently
  // only guaranteed to work in non-product builds.
  static const char* dll_address_to_function_name(address addr);

  // Map pc to a dynamic library, returns the full path the library file
  static const char* dll_address_to_library_name(address addr);

  // Maps PC to source file name and line number. Returns -1 if failed.
  static int dll_address_to_line_number(address addr, char ** filename);

  // Find out whether the pc is in the static code for jvm.dll/libjvm.so.
  static bool dll_address_is_in_vm(address addr);

  // Dump information about all open dynamic libraries; currently it prints out a
  // list of library names and their base addresses. Output format is slightly
  // different on different platforms.
  static void dump_dll_info(outputStream *st);

  // returns a string to describe the exception/signal; returns NULL if no information
  // is available.
  static const char * os::exception_name(int exception_code);

  // Returns native Java library, loads if necessary
  static void*    native_java_library();

  // Fills in path to jvm.dll/libjvm.so (this info used to find hpi).
  static void     jvm_path(char *buf, jint buflen);

  // JNI names
  static void     print_jni_name_prefix_on(outputStream* st, int args_size);
  static void     print_jni_name_suffix_on(outputStream* st, int args_size);

  // File conventions
  static inline const char* file_separator();
  static inline const char* line_separator();
  static inline const char* path_separator();

  // Properties
  static props_md_t* get_system_properties();

  // Java 1.1 classpath for JNI_GetDefaultJavaVMInitArgs
  static char* get_java_1_1_system_classpath();

  // IO operations, non-JVM_ version.
  static int stat(const char* path, struct stat* sbuf);

  // Thread Local Storage
  static int   allocate_thread_local_storage();
  static void  thread_local_storage_at_put(int index, void* value);
  static void* thread_local_storage_at(int index);
  static void  free_thread_local_storage(int index);

  // General allocation (must be MT-safe)
  static void* malloc  (size_t size);
  static void* realloc (void *memblock, size_t size);
  static void  free    (void *memblock);
  static bool  check_heap(bool force = false);      // verify C heap integrity
  static char* strdup(const char *);  // Like strdup
#ifndef PRODUCT
  static int  num_mallocs;            // # of calls to malloc/realloc
  static int  alloc_bytes;            // # of bytes allocated
  static int  num_frees;              // # of calls to free
#endif

  // Printing 64 bit integers
  static inline const char* jlong_format_specifier();
  static inline const char* julong_format_specifier();

  // Support for signals (see JVM_RaiseSignal, JVM_RegisterSignal)
  static void  signal_init();
  static void  signal_init_pd();
  static void  signal_notify(int signal_number);
  static void* signal(int signal_number, void* handler);
  static void  signal_raise(int signal_number);
  static int   signal_wait();
  static int   signal_lookup();
  static void* user_handler();
  static void  terminate_signal_thread();

  // random number generation
  static long random();                    // return 32bit pseudorandom number
  static void init_random(long initval);   // initialize random sequence

  // Structured OS Exception support
  static void os_exception_wrapper(java_call_t f, JavaValue* value, methodHandle* method, JavaCallArguments* args, Thread* thread);

  // jvmpi
  static bool thread_is_running(JavaThread* tp);
  static jlong thread_cpu_time();

  // Hook for os specific jvm options that we don't want to abort on seeing
  static bool obsolete_option(JavaVMOption *option);

  // Platform dependent stuff
  #include "incls/_os_pd.hpp.incl"

  // debugging support (mostly used by debug.cpp)
  static bool find(address pc) PRODUCT_RETURN0; // OS specific function to make sense out of an address
  
  static bool os::dont_yield();                 // when true, JVM_Yield() is nop
  static void print_statistics();

 protected:
  // Thread priority helpers (implemented in OS-specific part)
  static OSReturn set_native_priority(Thread* thread, int native_prio);
  static OSReturn get_native_priority(Thread* thread, int* priority_ptr);
  static int java_to_os_priority[MaxPriority + 1];

  static long _rand_seed;                   // seed for random number generator
};


// A critial region for short, non-blocking sections of code.
class ThreadCritical {
 private:
  friend void os_init();
  friend void os_exit();
  static void initialize();
  static void release();

 public:
  ThreadCritical();
  ~ThreadCritical();
};


