/*
 * @(#)osThread.cpp	1.18 01/02/14 09:27:26
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_osThread.cpp.incl"


OSThread::OSThread(OSThreadStartFunc start_proc, void* start_parm) {
  pd_initialize();
  _suspend_flags = 0;
  _suspend_depth = 0;
  set_start_proc(start_proc);
  set_start_parm(start_parm);
}

OSThread::~OSThread() {
  pd_destroy();
}

// Printing
void OSThread::print() {
  tty->print("nid=0x%x ", thread_id());
  switch (state()) {
    case ALLOCATED:               tty->print("allocated ");                 break;
    case INITIALIZED:             tty->print("initialized ");               break;
    case RUNNABLE:                tty->print("runnable ");                  break;
    case SUSPENDED:               tty->print("suspended ");                 break;
    case MONITOR_WAIT:            tty->print("waiting for monitor entry "); break;
    case CONDVAR_WAIT:            tty->print("waiting on monitor ");        break;
    case BREAKPOINTED:            tty->print("at breakpoint");        	    break;
    case SLEEPING:            	  tty->print("sleeping");        	    break;
    case ZOMBIE:            	  tty->print("zombie");        	    	    break;
    default:                      tty->print("unknown state %d ", state()); break;
  }
}

// set_state_under_suspend sets the current state if we
// are not suspended, and sets the state_under_suspend
// if we are so we will restore that state upon resume.
// Note that at init time we need to set both.
void OSThread::set_state_under_suspend(ThreadState state) {
  if(suspend_depth() == 0) {
    set_state(state);
  }
  else {
    set_state_before_suspend(state);
  }
}

