/*
 * @(#)recompile.cpp	1.153 00/04/05 10:13:12
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_recompile.cpp.incl"

int Recompilation::_nofRecompilations = 0;

methodHandle Recompilation::recompilee() const {
  return _recompilee ? _recompilee->method() : methodHandle(methodOop(NULL)); 
}

void Recompilation::print() {
  tty->print("((Recompilation*)%#x) : ", this);
  _method->print_value_on(tty);
  tty->cr();
}


address Recompilation::methodOop_invocation_counter_overflow(methodHandle method,
                                                             TRAPS) {
  if (!method->invocation_counter()->has_overflowed()) {
    return NULL;
  }
  // TEMPORARY
  assert(!method->is_not_compilable(), "shouldn't be calling this");

  ResourceMark rm;  
  frame fr = JavaThread::current()->last_frame();
  assert(fr.is_interpreted_frame(), "must be interpreted");
  assert(fr.interpreter_frame_method() == method(), "bad method");

  // Make sure invocation counter doesn't overflow again right away
  // as would be the case for native methods.
  method->invocation_counter()->set_carry();
  // BUT also make sure the method doesn't look like it was never executed.
  int count = method->invocation_counter()->count();
  count = min(count >> 1, MinInliningThreshold + 1);
  method->invocation_counter()->set(method->invocation_counter()->state(), count);
  assert(!method->was_never_executed(), "don't reset to 0 -- could be mistaken for never-executed");

  if (method->code() == NULL) {      
    if (RecompilationPolicy::canBeCompiled(method) && UseRecompilation) {
      // If the method is native we should already have set the native_function field
      assert(!method->is_native() || method->native_function() != NULL, "checking native function");
      _nofRecompilations++;

      // Note: C1 doesn't do real inlining yet, so we always
      //       want to compile the callee and not the caller.
#ifdef COMPILER1
      CompileBroker::compile_method(method, InvocationEntryBci, CHECK_0);          
      // note: at this point, method->code() may still be NULL
      // (if the recompiler decided to compile a caller instead)
      return NULL;
#endif
            
      Recompilation recomp(method);
      if (recomp.recompilee().not_null()) {
        CompileBroker::compile_method(recomp.recompilee(), InvocationEntryBci, CHECK_0);          
        // note: at this point, method->code() may still be NULL
        // (if the recompiler decided to compile a caller instead)
      }
    }
  }
  
  // Note: Inline caches and static call sites are fixed by c2iadapters
  // which checks if the callee is compiled and updates the call site
  // => no inline cache update necessary  

  nmethod* nm = method->code();
  return nm ? nm->verified_entry_point() : NULL;
}


address Recompilation::update_inline_cache(methodHandle method) {  
  nmethod* nm = method->code();
  if (nm == NULL || nm->is_zombie()) {
    // nmethod has been removed/zombied by another thread => cannot happen w/o
    // MT/MP.  continue with interpreted method        
    return NULL;
  }  

  // nmethod exists (normal case) => update inline cache and continue with
  // nmethod first handle interpreted case
  JavaThread *thread = JavaThread::current();
  ResourceMark rm;
  RegisterMap map(thread, false);
  assert(!thread->last_frame().is_first_java_frame(), "VM should call correctly");
  frame caller = thread->last_frame().real_sender(&map);

  if (caller.is_interpreted_frame()) {
    // don't bother setting the inline caches right, just clear them
    methodOop m = caller.interpreter_frame_method();
    Events::log("Cleaning inline caches of interpreted method %#x", m);
    m->cleanup_inline_caches();    
  } else if(caller.is_compiled_frame()) {     
    reresolve_call_site(caller);    
  } 

  // no need to fix caller's inline cache
  return nm->verified_entry_point();
}

// Resets a call-site in compiled code so it will get resolved again. This routines handle 
// both virtual call sites, optimized virual call sites, and static call sites. Typically used
// to change a call sites destination from interpreter to compiled. The 'caller' frame's pc must
// point after call to update
void Recompilation::reresolve_call_site(frame caller) {  
  assert(caller.is_compiled_frame(), "must be compiled");     
  address pc = caller.pc();
  Events::log("update call-site at pc %#x", pc);
  
  // Location of call instruction 
  NativeCall *ncall = nativeCall_before(pc);
  address call_addr = ncall->instruction_address();   

  // Check for static or virtual call       
  bool is_static_call = false;
  { nmethod* caller_nm = CodeCache::find_nmethod(pc);     
    RelocIterator iter(caller_nm, call_addr, call_addr+1);
    int ret = iter.next(); // Get item
    assert(ret == 1, "relocation info. must exist for this address");
    assert(iter.addr() == call_addr, "sanity check");
#ifdef COMPILER1
    if (iter.type() == relocInfo::static_call_type || iter.type() == relocInfo::opt_virtual_call_type) {
      is_static_call = true; 
    } else {
      assert(iter.type() == relocInfo::virtual_call_type 
            , "unexpected relocInfo. type");
    }
#else
    if (iter.type() == relocInfo::static_call_type) {
      is_static_call = true; 
    } else {
      assert(iter.type() == relocInfo::virtual_call_type ||
             iter.type() == relocInfo::opt_virtual_call_type
            , "unexpected relocInfo. type");
    }
#endif
  }

  // Cleaning the inline cache will force a new resolve. This is more robust than 
  // directly setting it to the new destination, since resolving of calls is always
  // done through the same code path. (experience shows that it leads to very hard to
  // track down bugs, if an inline cache gets update to a wrong method). It should not be 
  // performance critical, since the resolve is only done once.
  
  Mutex *lock = (is_static_call) ? Patching_lock : CompiledIC_lock;
  MutexLocker ml(lock);    
  // Check if deoptimization happend while we were waiting for the lock.   
  if (!caller.is_deoptimized_frame()) {     
    if (is_static_call) {         
      CompiledStaticCall* ssc= compiledStaticCall_at(ncall->instruction_address());
      ssc->set_to_clean();
    } else {
      // compiled, dispatched call (which used to call an interpreted method)      
      CompiledIC* inline_cache = CompiledIC_at(ncall->instruction_address());
      inline_cache->set_to_clean();
    }
  }
}

void Recompilation::init() {
  _triggerVF = NULL;
  _recompilee = NULL;
}


void Recompilation::doit() {     
  assert(_method() != NULL, "no method is set");
  assert(RecompilationPolicy::canBeCompiled(_method), "compiling method that is not allowed to be compiled");

  if (PrintRecompilation) {
    tty->print("recompilation #%d trigger: %s ( %s %#x) ", _nofRecompilations,
               _method->name()->as_C_string(), 
               "interpreted",
               (char*)_method());
  }

  JavaThread* thread = JavaThread::current();
  RegisterMap reg_map(thread, false);
  _triggerVF = thread->last_java_vframe(&reg_map);
  // trigger is the frame that triggered its counter
  RFrame* first = new InterpretedRFrame(_triggerVF->fr(), thread, _method);

  RecompilationPolicy policy(first);
  if (handleStaleInlineCache(first)) {
    // called obsolete method/nmethod -- no need to recompile
    if (PrintRecompilation) tty->print(" --> %#x\n", first->top_method()->code());    
  } else {
    if (PrintRecompilation2) tty->cr();
    _recompilee = policy.findRecompilee();
    // NB: don't reset the trigger's counter so that it will trigger again at
    // the next call.  Thus, out-of-date inline caches will be updated on
    // demand.
  }

  // We could eagerly clean up the inline caches of the top few stacks, to
  // prevent frequent inline cache "misses" from ics pointing to interpreted
  // methods or obsolete compiled code.  However, this is tricky from a MT
  // standpoint: the VM thread can't clean methodOops because it might block
  // on one of them.  So, if at all, this should be done back in the thread
  // that initiated this compile.  For now, I disabled the eager cleaning; it
  // probably doesn't matter performance-wise.
  // policy.cleanupStaleInlineCaches();
}


bool Recompilation::handleStaleInlineCache(RFrame* first) {
  // check if the trigger was an interpreted method that has already been
  // compiled; return true if so
  return first->top_method()->has_compiled_code();
}


Recompilee* Recompilee::new_Recompilee(RFrame* rf) {
  if (rf->is_compiled()) {
    return new CompiledRecompilee(rf, ((CompiledRFrame*)rf)->nm());
  } else {
    InterpretedRFrame* irf = (InterpretedRFrame*)rf;
    return new InterpretedRecompilee(irf, irf->top_method());
  }
}


methodHandle  CompiledRecompilee::method() const { 
  return _nm->method(); 
}
