/*
 * @(#)recompile.hpp	1.33 99/06/22 16:43:59
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// The recompilation system determines which interpreted methods should be
// compiled by the native-code compiler, or which compiled methods need to be
// recompiled for further optimization. This file contains the interface to
// the run-time system.
//
// A Recompilation performs a recompilation from interpreted or compiled code
// to (hopefully better) compiled code. The global theRecompilation is set
// only during a recompilation.
 
class  Recompilee;
class  RFrame;

class Recompilation : public StackObj {
 private:  
  methodHandle  _method;             // trigger method
  javaVFrame*   _triggerVF;          // vframe of trigger method/nmethod
  Recompilee*   _recompilee;         // chosen recompilee
  static int    _nofRecompilations;  // number of times recompilation was triggered

 public:
  Recompilation(methodHandle method) {         // used if interpreted method triggers counter
    _method = method; init(); doit();
  }

  methodHandle recompilee() const;  
  void print();

  // entry points dealing with invocation counter overflow
  static address  methodOop_invocation_counter_overflow(methodHandle method, TRAPS); // called by interpreter runtime
  static address  update_inline_cache                  (methodHandle method);
  static void     reresolve_call_site                  (frame caller);

 protected:
  void init();
  void doit();
  bool handleStaleInlineCache(RFrame* first);
};


// A Recompilee represents something to be recompiled (either an interpreted
// method or a compiled method).

class Recompilee : public ResourceObj {
 protected:
  RFrame* _rf;
  Recompilee(RFrame* rf)                { _rf = rf; }
 public:
  virtual bool is_interpreted() const	{ return false; }
  virtual bool is_compiled() const      { return false; }
  virtual methodHandle method() const   = 0;
  virtual nmethod*   code() const       { ShouldNotCallThis(); return NULL; }	// only for compiled recompileed
  RFrame* rframe() const                { return _rf; }
  static Recompilee* new_Recompilee(RFrame* rf);
};

class InterpretedRecompilee : public Recompilee {
  methodHandle _method;
 public:
  InterpretedRecompilee(RFrame* rf, methodHandle m) : Recompilee(rf) { _method = m; }
  bool       is_interpreted() const  { return true; }
  methodHandle  method() const          { return _method; }
};

class CompiledRecompilee : public Recompilee {
  nmethod* _nm;
 public:
  CompiledRecompilee(RFrame* rf, nmethod* nm) : Recompilee(rf) { _nm = nm; }
  bool       is_compiled() const     { return true; }
  methodHandle  method() const;
  nmethod*   code() const            { return _nm; }
}; 

