/*
 * @(#)recompiler.cpp	1.30 00/03/09 11:21:04
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_recompiler.cpp.incl"

RecompilationPolicy::RecompilationPolicy(RFrame* first) {
  stack = new GrowableArray<RFrame*>(50);
  stack->push(first);
}


Recompilee* RecompilationPolicy::findRecompilee() {
  RFrame* rf = stack->at(0);
  if (PrintRecompilation2) {
    for (int i = 0; i < 10 && rf != NULL; i++, rf = senderOf(rf)) ;   // create 10 frames
    printStack();
  }
  RFrame* r = findTopInlinableFrame();
  if (r) {
    if (PrintRecompilation) r->print();
    return Recompilee::new_Recompilee(r);
  } else {
    return NULL;
  }
}

void RecompilationPolicy::cleanupStaleInlineCaches() {
  // clean inline caches of caller so interpreted method won't be called again
  fatal("disabled");    // see comment in recompile.cpp
  RFrame* caller = senderOf(stack->at(0));
  if (caller != NULL) caller->cleanupStaleInlineCaches();
  RFrame* rf = caller;
  const int len = stack->length();
  for (int i = 0; i < len && rf != NULL; i++, rf = senderOf(rf)) rf->cleanupStaleInlineCaches();
}

RFrame* RecompilationPolicy::findTopInlinableFrame() {
  // go up the stack until finding a frame that (probably) won't be inlined
  // into its caller
  RFrame* current = stack->at(0); // current choice for stopping
  assert( current && !current->is_compiled(), "" );
  msg = NULL; 

  while (1) {

    // before going up the stack further, check if doing so would get us into
    // compiled code
    RFrame* next = senderOf(current);
    if( !next )               // No next frame up the stack?
      break;                  // Then compile with current frame

    methodHandle m = current->top_method();
    methodHandle next_m = next->top_method();

    if( !Inline ) {           // Inlining turned off
      msg = "(Inlining turned off)";
      break;
    }
    if (next_m->is_not_compilable()) { // Did fail to compile this before/
      msg = "(caller not compilable)";
      break;
    }
    if (next->num() > MaxRecompilationSearchLength) {
      // don't go up too high when searching for recompilees
      msg = "(don't go up any further: > MaxRecompilationSearchLength)";
      break;
    }
    if (next->distance() > MaxInterpretedSearchLength) {
      // don't go up too high when searching for recompilees
      msg = "(don't go up any further: next > MaxInterpretedSearchLength)";
      break;
    }
    // Compiled frame above already decided not to inline;
    // do not recompile him.
    if (next->is_compiled()) {
      msg = "(not going up into optimized code)";
      break;
    }

    // Interpreted frame above us was already compiled.  Do not force
    // a recompile, although if the frame above us runs long enough an
    // OSR might stiff happen.
    if( current->is_interpreted() && next_m->has_compiled_code() ) {
      msg = "(not going up -- already compiled caller)";
      break;
    }

    // Compute how frequent this call site is.  We have current method 'm'.
    // We know next method 'next_m' is interpreted.  Find the call site and
    // check the various invocation counts.
    int invcnt = 0;             // Caller counts
#ifdef COMPILER2
    if( CountInterpInvocations ) {
      invcnt = next_m->interpreter_invocation_count();
    }
#endif
    int cnt = 0;                // Call site counts
#ifdef COMPILER2
    if( CountInterpCalls ) {
      int bci = next->top_vframe()->bci();
      int bcp = (int)(next_m->bcp_from(bci));
      bcp = bcp & right_n_bits(CallFreqTableSize);
      cnt = SharedInfo::callTable[bcp];
    }
#endif
    // Caller counts / call-site counts; i.e. is this call site
    // a hot call site for method next_m?
    int freq = (invcnt) ? cnt/invcnt : cnt;
    
    // Check size and frequency limits
    if ((msg = InliningPolicy::shouldInline(m, freq, cnt)) != NULL) {
      break;
    }
    // Check inlining negative tests
    if ((msg = InliningPolicy::shouldNotInline(m, NULL)) != NULL) {
      break;
    }


    // If the caller method is too big or something then we do not want to
    // compile it just to inline a method.
    if (!canBeCompiled(next_m)) {
      msg = "caller cannot be compiled";
      break;
    }

    if( next_m->name() == vmSymbols::class_initializer_name() ) {
      msg = "do not compile class initializer (OSR ok)";
      break;
    }

    current = next;
  }

  assert( !current || !current->is_compiled(), "" );

  if (PrintRecompilation && msg) tty->print("%s\n", msg);

  return current;
}

RFrame* RecompilationPolicy::senderOf(RFrame* rf) {
  RFrame* sender = rf->caller();
  if (sender && sender->num() == stack->length()) stack->push(sender);
  return sender;
}

void RecompilationPolicy::printStack() {	// for debugging
  for (int i = 0; i < stack->length(); i++ ) stack->at(i)->print();
}


// must method m be compiled before being executed?
// Note: this does not answer a "should" question; it is intended to force compiles for methods
// that cannot be interpreted for some reason.
bool RecompilationPolicy::mustBeCompiled(methodHandle m) {
  if (m->has_compiled_code()) return false;       // already compiled
  if (!canBeCompiled(m)) return false;

  return basic_mustBeCompiled(m);
}

