/*
 * @(#)recompiler.hpp	1.43 00/03/09 11:23:05
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// These protoptypes are declared here instead of arguments.hpp because
// otherwise arguments.hpp would always have to precede recompiler.hpp since
// these functions are used in inline functions defined in recompiler.hpp.
// They are defined in arguments.cpp.
extern bool CheckCompileOnly ();
extern bool CompileMethod (char* className, char* methodName);

// The RecompilationPolicy selects which method (if any) should be recompiled.
// It also decides which methods must always be compiled (i.e., are never
// interpreted).

class RecompilationPolicy : public ResourceObj {
 protected:
   GrowableArray<RFrame*>* stack;
   const char* msg;                              // for (performance) debugging: reason for not going up, etc.

   RFrame* senderOf(RFrame* rf);           // return rf->sender() and update stack if necessary
   RFrame* findTopInlinableFrame();
   void    printStack();
   static inline bool basic_mustBeCompiled(methodHandle m);   
public:
   RecompilationPolicy(RFrame* first);

   Recompilee* findRecompilee();           // determine what to recompile
   void cleanupStaleInlineCaches();        // clean up inline caches of top methods

   static bool mustBeCompiled(methodHandle m);      // returns true if m must be compiled before executing it
   static inline bool canBeCompiled(methodHandle m);       // returns true if m is allowed to be compiled   
};

// core mustBeCompiled, for debugging --- normally you should call mustBeCompiled
// Note: this is for *positive* checks only, i.e., don't test for "must not" conditions.
// Also, basic_mustBeCompiled should be invariant (i.e., always return the same result
// for the same method, modulo changes in debug variable settings).
inline bool RecompilationPolicy::basic_mustBeCompiled(methodHandle m) {  
  // compile synchronized methods for speed comparisons
  return !UseInterpreter ||                                                // must compile all methods
         (UseRecompilation && AlwaysCompileLoopMethods && m->has_loops()); // eagerly compile loop methods
}

// This function can be used to selectively prohibit certain methods from
// being compiled
inline bool RecompilationPolicy::canBeCompiled(methodHandle m) {
  if (m->is_abstract()) return false;
  if (DontCompileHugeMethods && m->code_size() > HugeMethodLimit) return false;

  // TEMPORARY: If we tried to compile and failed, do not try again
  // a method may have been marked as uncompilable
  if(m->is_not_compilable()) return false;
         
  // else
  return true;  
}




