/*
 * @(#)reflection.hpp	1.26 99/10/27 16:46:31
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// Class Reflection contains utility methods needed for implementing the
// reflection api. 
//
// Used by functions in the JVM interface.


class FieldStream;

class Reflection: public AllStatic {
 private:
  static bool match_parameter_types(methodHandle method, objArrayHandle types, int parameter_count, TRAPS);
  static objArrayHandle get_parameter_types(methodHandle method, int parameter_count, oop* return_type, TRAPS);
  static objArrayHandle get_exception_types(methodHandle method, TRAPS);

  // Creating new java.lang.reflect.xxx wrappers
  static oop new_type(symbolOop signature, klassOop k, TRAPS);
  static oop new_field(FieldStream* st, TRAPS);  

  // Access checking
  static bool reflect_check_access(klassOop field_class, AccessFlags acc, klassOop target_class, bool is_method_invoke, TRAPS);
  
  // method resolution for invoke
  static methodHandle resolve_interface_call(instanceKlassHandle klass, methodHandle method, KlassHandle recv_klass, Handle receiver, TRAPS);
  static methodHandle resolve_virtual_call(instanceKlassHandle klass, methodHandle method, KlassHandle recv_klass, Handle receiver, TRAPS);
  // Method call (shared by invoke_method and invoke_constructor)
  static oop  invoke(instanceKlassHandle klass, methodHandle method, Handle receiver, bool override, objArrayHandle ptypes, BasicType rtype, objArrayHandle args, bool is_method_invoke, TRAPS);

  // Narrowing of basic types. Used to create correct jvalues for 
  // boolean, byte, char and short return return values from interpreter
  // which are returned as ints. Throws IllegalArgumentException.
  static void narrow(jvalue* value, BasicType narrow_type, TRAPS);
  // Conversion
  static klassOop basic_type_mirror_to_arrayklass(oop basic_type_mirror, TRAPS);
  static oop      basic_type_arrayklass_to_mirror(klassOop basic_type_arrayklass, TRAPS);
  static BasicType basic_type_mirror_to_basic_type(oop basic_type_mirror, TRAPS);

 public:
  // Constants defined by java reflection api classes
  enum SomeConstants {
    PUBLIC            = 0,
    DECLARED          = 1,
    MEMBER_PUBLIC     = 0,
    MEMBER_DECLARED   = 1,
    MAX_DIM           = 255
  };

  // Create a java.lang.reflect.Method object based on a method
  static oop new_method(methodHandle method, TRAPS);
  // Create a java.lang.reflect.Constructor object based on a method
  static oop new_constructor(methodHandle method, TRAPS);  
  // Create a java.lang.reflect.Field object based on a field descriptor
  static oop new_field(fieldDescriptor* fd, TRAPS);

  // Boxing. Returns boxed value of appropriate type. Throws IllegalArgumentException.
  static oop box(jvalue* v, BasicType type, TRAPS);
  // Unboxing. Returns type code and sets value.
  static BasicType unbox_for_primitive(oop boxed_value, jvalue* value, TRAPS);
  static BasicType unbox_for_regular_object(oop boxed_value, jvalue* value);

  // Widening of basic types. Throws IllegalArgumentException.
  static void widen(jvalue* value, BasicType current_type, BasicType wide_type, TRAPS);

  // Reflective array access. Returns type code. Throws ArrayIndexOutOfBoundsException.
  static BasicType array_get(jvalue* value, arrayOop a, int index, TRAPS);
  static void      array_set(jvalue* value, arrayOop a, int index, BasicType value_type, TRAPS);
  // Returns mirror on array element type (NULL for basic type arrays and non-arrays).
  static oop       array_component_type(oop mirror, TRAPS);

  // Field lookup and verification.
  static bool      resolve_field(Handle field_mirror, Handle& receiver, fieldDescriptor* fd, bool check_final, TRAPS);
  // Reflective field access. Returns type code. Throws IllegalArgumentException. 
  static BasicType field_get(jvalue* value, fieldDescriptor* fd, Handle receiver);
  static void      field_set(jvalue* value, fieldDescriptor* fd, Handle receiver, BasicType value_type, TRAPS);

  // Reflective lookup of fields. Returns java.lang.reflect.Field instances.
  static oop         reflect_field(oop mirror, symbolOop field_name, jint which, TRAPS);
  static objArrayOop reflect_fields(oop mirror, jint which, TRAPS);

  // Reflective lookup of methods. Returns java.lang.reflect.Method instances.
  static oop         reflect_method(oop mirror, symbolOop method_name, objArrayOop types, jint which, TRAPS);
  static objArrayOop reflect_methods(oop mirror, jint which, TRAPS);

  // Reflective lookup of constructors. Returns java.lang.reflect.Constructor instances.
  static oop         reflect_constructor(oop mirror, objArrayOop types, jint which, TRAPS);
  static objArrayOop reflect_constructors(oop mirror, jint which, TRAPS);

  // Method invokation through java.lang.reflect.Method
  static oop      invoke_method(oop method_mirror, Handle receiver, objArrayHandle args, TRAPS);
  // Method invokation through java.lang.reflect.Constructor
  static oop      invoke_constructor(oop method_mirror, objArrayHandle args, TRAPS);

  // Object creation
  static arrayOop reflect_new_array(oop element_mirror, jint length, TRAPS);
  static arrayOop reflect_new_multi_array(oop element_mirror, typeArrayOop dimensions, TRAPS);

  // Verification
  static bool     verify_class_access(klassOop current_class, klassOop new_class, bool classloader_only);
  static bool     verify_field_access(klassOop current_class, klassOop field_class, AccessFlags access, 
                                      bool classloader_only, bool protected_restriction = false);
  static bool     is_same_class_package(klassOop class1, klassOop class2);

  // inner class reflection
  static void check_for_inner_class(instanceKlassHandle outer, instanceKlassHandle inner, TRAPS);
};

