/*
 * @(#)sharedRuntime.cpp	1.268 01/02/09 14:47:52
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_sharedRuntime.cpp.incl"
#include <math.h>


// Implementation of SharedRuntime

SharedRuntime::StrictMathFunction SharedRuntime::java_lang_strict_math_sin = NULL;
SharedRuntime::StrictMathFunction SharedRuntime::java_lang_strict_math_cos = NULL;

address SharedRuntime::_implicit_null_exception_filter = NULL;
address SharedRuntime::_implicit_div0_exception_filter = NULL;
address SharedRuntime::_stack_ovrflow_exception_filter = NULL;


void SharedRuntime::lookup_function(StrictMathFunction& f, const char* fname) {
  if (f == NULL) f = CAST_TO_FN_PTR(StrictMathFunction, NativeLookup::base_library_lookup("java/lang/StrictMath", fname, "(D)D"));
  assert(f != NULL, "lookup failed");
}


JRT_LEAF(jdouble, SharedRuntime::dsin(jdouble x))
  assert(java_lang_strict_math_sin != NULL, "sin entry point must exist");
  return (*java_lang_strict_math_sin)(NULL, NULL, x);
JRT_END


JRT_LEAF(jdouble, SharedRuntime::dcos(jdouble x))
  assert(java_lang_strict_math_cos != NULL, "cos entry point must exist");
  return (*java_lang_strict_math_cos)(NULL, NULL, x);
JRT_END


JRT_LEAF(jlong, SharedRuntime::lmul(jlong y, jlong x))
  return x * y;
JRT_END


JRT_LEAF(jlong, SharedRuntime::ldiv(jlong y, jlong x))
  if (x == min_jlong && y == -1L) {
    return x;
  } else {
    return x / y;
  }
JRT_END


JRT_LEAF(jlong, SharedRuntime::lrem(jlong y, jlong x))
  if (x == min_jlong && y == -1L) {
    return 0;
  } else {
    return x % y;
  }
JRT_END


JRT_LEAF(jfloat, SharedRuntime::frem(jfloat  x, jfloat  y))
  return (jfloat)fmod(x,y);
JRT_END


JRT_LEAF(jdouble, SharedRuntime::drem(jdouble x, jdouble y))
  return (jdouble)fmod(x,y);
JRT_END


JRT_LEAF(jint, SharedRuntime::f2i(jfloat  x))
  if (g_isnan(x)) {return 0;}
  jlong lltmp = (jlong)x;
  jint ltmp   = (jint)lltmp;
  if (ltmp == lltmp) {
    return ltmp;
  } else {
    if (x < 0) {
      return min_jint;
    } else {
      return max_jint;
    }
  }
JRT_END


JRT_LEAF(jlong, SharedRuntime::f2l(jfloat  x))  
  if (g_isnan(x)) {return 0;}
  jlong lltmp = (jlong)x;
  if (lltmp != min_jlong) {
    return lltmp;
  } else {
    if (x < 0) {
      return min_jlong;
    } else {
      return max_jlong;
    }
  }
JRT_END


JRT_LEAF(jint, SharedRuntime::d2i(jdouble x))
  if (g_isnan(x)) {return 0;}
  jlong lltmp = (jlong)x;
  jint ltmp   = (jint)lltmp;
  if (ltmp == lltmp) {
    return ltmp;
  } else {
    if (x < 0) {
      return min_jint;
    } else {
      return max_jint;
    }
  }
JRT_END


JRT_LEAF(jlong, SharedRuntime::d2l(jdouble x))
  if (g_isnan(x)) {return 0;}
  jlong lltmp = (jlong)x;
  if (lltmp != min_jlong) {
    return lltmp;
  } else {
    if (x < 0) {
      return min_jlong;
    } else {
      return max_jlong;
    }
  }
JRT_END


JRT_LEAF(jfloat, SharedRuntime::d2f(jdouble x))
  return (jfloat)x;
JRT_END


JRT_LEAF(jfloat, SharedRuntime::l2f(jlong x))
  return (jfloat)x;
JRT_END


JRT_LEAF(jdouble, SharedRuntime::l2d(jlong x))
  return (jdouble)x;
JRT_END

// Exception handling accross interpreter/compiler boundaries
//
// exception_handler_for_return_address(...) returns the continuation address.
// The continuation address is the entry point of the exception handler of the
// previous frame depending on the return address.

address SharedRuntime::raw_exception_handler_for_return_address(address return_address) {


  assert(frame::verify_return_pc(return_address), "must be a return pc");

#ifdef COMPILER1
  // the fastest case first
  CodeBlob* blob = CodeCache::find_blob(return_address);
  if (blob != NULL && blob->is_nmethod()) {
    nmethod* code = (nmethod*)blob;
    assert(code != NULL, "nmethod must be present");
    assert(code->header_begin() != code->exception_begin(), "no exception handler");
    return code->exception_begin();
  } 
#endif // COMPILER1

  
  // Entry code
  if (StubRoutines::returns_to_call_stub(return_address)) {
    return StubRoutines::catch_exception_entry();
  }
  // Interpreted code
  if (Interpreter::contains(return_address)) {
    return Interpreter::rethrow_exception_entry();
  }
#ifdef COMPILER1
  // OSR adapter code
  // Note: this code is the only runtime stub code blob
  //       that needs to be handled here - make sure this
  //       test happens before the blob->is_runtime_stub()
  //       check below 
  if (Runtime1::returns_to_osr_adapter(return_address)) {
    return Runtime1::entry_for(Runtime1::osr_unwind_exception_id);
  }
#endif // COMPILER1
#ifdef COMPILER2
  // Unpacking code for deoptimized frames
  if (OptoRuntime::deoptimization_blob() != NULL) {
    if (OptoRuntime::deoptimization_blob()->exception_address_is_unpack_entry(return_address)) {
      return OptoRuntime::deoptimization_blob()->unpack_with_exception();
    }
  }
#endif // COMPILER2
#ifndef CORE
  // Compiled code
  if (CodeCache::contains(return_address)) {
    CodeBlob* blob = CodeCache::find_blob(return_address);
    if (blob->is_nmethod()) {
      nmethod* code = (nmethod*)blob;
      assert(code != NULL, "nmethod must be present");
      assert(code->header_begin() != code->exception_begin(), "no exception handler");
      return code->exception_begin();
    } 
    if (blob->is_runtime_stub()) {
      ShouldNotReachHere();   // callers are responsible for skipping runtime stub frames
    }
#ifdef COMPILER2
    if (blob->is_c2i_adapter()) {
      return OptoRuntime::exception_blob()->instructions_begin();
    } 
    if (blob->is_i2c_adapter()) {
      return OptoRuntime::exception_blob()->instructions_begin();
    } 
    if (blob->is_osr_adapter()) {
      return OptoRuntime::exception_blob()->instructions_begin();
    }
#endif // COMPILER2
  }
  guarantee(!VtableStubs::contains(return_address), "NULL exceptions in vtables should have been handled already!");
#endif // CORE
#ifndef PRODUCT
  { ResourceMark rm;
    tty->print_cr("No exception handler found for exception at 0x%x - potential problems:", return_address);
    tty->print_cr("a) exception happened in (new?) code stubs/buffers that is not handled here");
    tty->print_cr("b) should have called exception_handler_for_address instead");
    tty->print_cr("c) other problem");
  }
#endif // PRODUCT
  ShouldNotReachHere();
  return NULL;
}


JRT_LEAF(address, SharedRuntime::exception_handler_for_return_address(address return_address))
  JavaThread* thread = JavaThread::current();
#ifndef CORE
  address ra = thread->safepoint_state()->compute_adjusted_pc(return_address);  
#else
  address ra = return_address;  
#endif
  return raw_exception_handler_for_return_address(ra);
JRT_END


oop SharedRuntime::retrieve_receiver( symbolOop sig, frame caller ) {
  assert(caller.is_interpreted_frame(), "");
  int args_size = ArgumentSizeComputer(sig).size() + 1;
  assert(args_size <= caller.interpreter_frame_expression_stack_size(), "receiver must be on interpreter stack");
  oop result = (oop) caller.interpreter_frame_tos_at(args_size - 1);
  assert(Universe::is_heap(result) && result->is_oop(), "receiver must be an oop");
  return result;
}

JRT_ENTRY(address, SharedRuntime::create_i2c_adapter(JavaThread* thread, methodOop method))
  COMPILER1_ONLY(ShouldNotReachHere();) // no adapters with C1
  if (TraceAdapterGeneration && Verbose) tty->print_cr("create_i2c_adapter");
#ifndef CORE
  methodHandle calleeMethod = method;
  // Verify that frame and arguments are correct
  debug_only(verify_caller_frame(thread->last_frame(), calleeMethod));  

  address i2c_entry = calleeMethod->code()->interpreter_entry_point();
  thread->set_vm_result(calleeMethod());
  return i2c_entry;
#else
  ShouldNotReachHere();
  return NULL;
#endif
JRT_END



JRT_LEAF(int, SharedRuntime::is_interpreter_exception())
  // Caution: This routine must be a leaf routine (no GCs. no locking!)
  JavaThread* thread = JavaThread::current();
  address pc = thread->saved_exception_pc();
  return (int)Interpreter::contains(pc);
JRT_END

// Method needs to be cleaned up !!
// ifdef CORE does not belong to sharedRuntime.cpp; the method has been
// designed to handle only implicit null exceptions, but there are two more
// implicit exception types (arithmetic and stack overflow)
// The method contains a cumbersome fix by using the flag is_implicit_null
JRT_LEAF(address, SharedRuntime::compute_exception_return_address(jint is_implicit_null))
#ifndef COMPILER2
  ShouldNotReachHere();
  return NULL;
#else   
  // Compute the (corrected) issuing pc for an exception pc.
  // Caution: This routine must be a leaf routine (no GCs. no locking!)
  //          because we are in the middle of exception resolving.
  //          before GC can happen we have to set the exception pc 
  //          correctly.
  //
  // Note: The pc is __not__ a conventional next_pc value (as from
  // frame::pc), but the real address of the trapping instruction.
  
  // compute the corrected pc (if the exception happened in a temporary code
  // buffer used for safepoint synchronization, we need to adjust it first)
  assert(!(bool)is_interpreter_exception(), "interpreter exceptions must be handled already");
  JavaThread* thread = JavaThread::current();
  address pc = thread->safepoint_state()->compute_adjusted_pc(thread->saved_exception_pc());
 
  if (VtableStubs::contains(pc)) {
    // exception happened in vtable stub
    // => must be a NULL receiver => use return address
    // pushed by caller => don't push another return address
    return NULL;
  }

  CodeBlob* cb = CodeCache::find_blob(pc);
  guarantee(cb != NULL, "exception happened outside interpreter, nmethods and vtable stubs (1)");
  
  // Exception happened in CodeCache. Must be either:
  // 1. Inline-cache check in C2I adapter,
  // 2. Inline-cache check in nmethod, or
  // 3. Implict null exception in nmethod, or
  // 4. Divide-by-zero in nmethod, or
  // 5. Stack overflow just about anywhere.

  if (!cb->is_nmethod()) {
    bool is_stack_overflow = !is_implicit_null;

    // (Note:  This path would also be taken for divide-by-zero
    // exceptions, if they could possibly arise from adapters.
    // But they cannot.)

    guarantee(is_stack_overflow ||
	      (cb->is_c2i_adapter() && ((C2IAdapter*)cb)->inlinecache_check_contains(pc)),
	      "exception happened outside interpreter, nmethods and vtable stubs (2)");

    // There is no handler here, so we will simply unwind.
    return NULL;
  }
  // Otherwise, it's an nmethod.  Consult its exception handlers.
    
  // compute return address if needed    
  nmethod* nm = (nmethod*)cb;

  // exception happened inside an nmethod
  if (nm->inlinecache_check_contains(pc)) {
    // exception happened inside inline-cache check code
    // => the nmethod is not yet active (i.e., the frame
    // is not set up yet) => use return address pushed by
    // caller => don't push another return address
    return NULL;
  }
  
  if (is_implicit_null == 1) {
#ifndef PRODUCT
    extern int implicit_null_throws;
    implicit_null_throws++;
#endif
    // Exception happened outside inline-cache check code => we are inside
    // an active nmethod => use cpc to determine a return address
    int exception_offset = pc - nm->instructions_begin();
    int cont_offset = ImplicitNullCheckTable(nm).at( exception_offset );
#ifdef ASSERT
    if (cont_offset == 0) {
      ResetNoHandleMark rnm; // Might be called from LEAF/QUICK ENTRY
      HandleMark hm;
      ResourceMark rm;
      CodeBlob* cb = CodeCache::find_blob(pc);
      assert(cb != NULL, "");
      tty->print_cr("implicit exception happened at 0x%x", pc);
      if (cb->is_nmethod()) {
        ((nmethod*)cb)->method()->print_codes();
        ((nmethod*)cb)->print_code();
        ((nmethod*)cb)->print_pcs();
      }
    }
#endif
    guarantee(cont_offset != 0, "null exception in compiled code");
    int offset_from_exception_pc = cont_offset - exception_offset;
    address target_pc  = thread->saved_exception_pc() + offset_from_exception_pc;
  
    // A codebuffer for this exception might have been created while inside the exception handler.
    // since the OS might lye about the exact pc for a thread. In this case, we make sure to redirect
    // the target pc into this codebuffer.
    if (thread->safepoint_state()->code_buffer() != NULL && CodeCache::contains(target_pc)) {
      target_pc = thread->safepoint_state()->code_buffer()->capture_pc(target_pc);
    }
    assert(thread->safepoint_state()->code_buffer() != NULL || CodeCache::contains(target_pc), "bad target_pc (1)");
    assert(thread->safepoint_state()->code_buffer() == NULL || thread->safepoint_state()->code_buffer()->contains(target_pc), "bad target_pc (2)");

    Events::log("Implicit null exception at 0x%x to 0x%x", thread->saved_exception_pc(), target_pc);
    return target_pc;       
  } else {
    return NULL;
  }
#endif // COMPILER2
JRT_END


JRT_ENTRY(address, SharedRuntime::handle_array_index_check(JavaThread* thread, jint index))
  // Note: method returns an oop through the thread local storage  
  address pc = thread->last_frame().pc();
  address handler_address = raw_exception_handler_for_return_address(pc);

  char buf[40];
  sprintf(buf, "%d", index);
  Handle exception = Exceptions::new_exception(thread, vmSymbols::java_lang_ArrayIndexOutOfBoundsException(), buf, NULL, NULL);  
  thread->set_vm_result(exception()); 
  return handler_address;
JRT_END


JRT_ENTRY(int, SharedRuntime::trace_bytecode(JavaThread* thread, int preserve_this_value))
  const frame f = thread->last_frame();
  assert(f.is_interpreted_frame(), "must be an interpreted frame");
  BytecodeTracer::trace(f.interpreter_frame_method(), f.interpreter_frame_bcp());
  if (EnableJVMPIInstructionStartEvent && jvmpi::is_event_enabled(JVMPI_EVENT_INSTRUCTION_START)) {
    jvmpi::post_instruction_start_event(f);
  }
  return preserve_this_value;
JRT_END


#ifndef CORE
#ifndef PRODUCT

JRT_ENTRY(void, SharedRuntime::trace_method_entry(JavaThread* thread, methodOop method, address pc))
  MethodTracer::entry(thread,method,pc);
JRT_END

JRT_ENTRY(void, SharedRuntime::trace_method_exit(JavaThread* thread, methodOop method, address pc))
  MethodTracer::exit(thread,method,pc);
JRT_END

#endif // !PRODUCT
#endif // !CORE

JRT_ENTRY(void, SharedRuntime::yield_all(JavaThread* thread, int attempts))
  os::yield_all(attempts);
JRT_END

// returns NULL exception oop and sets the correct handler_entry for given return address
JRT_ENTRY(address, SharedRuntime::get_null_exception_entry_and_oop (JavaThread *thread))
  address exc_start = raw_exception_handler_for_return_address(thread->last_frame().pc());
  Handle exception = Exceptions::new_exception(thread, vmSymbols::java_lang_NullPointerException(), NULL, NULL, NULL);    
  thread->set_vm_result(exception());
  return exc_start;
JRT_END

//------ simple_array_copy helpers--------

inline bool check_length_of_type_array(oop arr, int start, int copy_lng) {
  assert (arr->is_array(), "");
  int array_length = arrayOop(arr)->length();
  return (unsigned int) array_length >= (unsigned int)start + (unsigned int)copy_lng;
}


// returns T_VOID if cannot handle the array
inline BasicType get_array_klass_type(klassOop klass) {
  Klass* k = klass->klass_part();
  if (k->oop_is_objArray()) {
    return T_OBJECT; // disable fast object copying (need to set RememberedSet)
  } else {
    if (k->oop_is_typeArray()) {
      return ((typeArrayKlass*)k)->type();
    } else if (k->oop_is_symbol()) {
      return T_CHAR;
    }
  }
  return T_VOID;
}


// we assume that all necessary checks have been done in compiled code already
JRT_LEAF(int, SharedRuntime::simple_array_copy_no_checks(int length, int dst_pos, oop dst, int src_pos, oop src))
#ifdef CORE
  ShouldNotReachHere();
  return 0; 
#else
#ifdef COMPILER2
  ShouldNotReachHere();
  return 0; 
#else
  if (length == 0) return 0; // done
  assert(src != NULL && dst != NULL, "illegal arrays");
  assert(check_length_of_type_array(dst, dst_pos, length) && check_length_of_type_array(src, src_pos, length), "wrong parameters");
  assert(dst->is_array() && src->is_array(), "");
  BasicType type = get_array_klass_type(src->klass()); // this checks also if the oops are arrays
  if (type == T_OBJECT) {
    klassOop bound = objArrayKlass::cast(dst->klass())->element_klass();
    klassOop stype = objArrayKlass::cast(src->klass())->element_klass();
    if (stype == bound || Klass::cast(stype)->is_subtype_of(bound)) {
      oop* src_addr = objArrayOop(src)->obj_at_addr(src_pos);
      oop* dst_addr = objArrayOop(dst)->obj_at_addr(dst_pos);
      // elements are guaranteed to be subtypes, so no check necessary
      assert(length > 0, "sanity check")
      Memory::copy_oops_overlapping_atomic(src_addr, dst_addr, length);
      RememberedSet::record_array_store(dst_addr, dst_addr + length);
      return 0;
    } 
  } else if (type != T_VOID) {
    if (src->klass() != dst->klass()) {
      // this can happen for symbol vs char array...
      BasicType type_2 = get_array_klass_type(dst->klass());
      if (type_2 != type) return -1;
    }
    // do the copy
    unsigned int bytes_per_elem = type2aelembytes[type];
    char* src_a = (char*) arrayOop(src)->base(type) + (src_pos * bytes_per_elem);
    char* dst_a = (char*) arrayOop(dst)->base(type) + (dst_pos * bytes_per_elem);
    unsigned int nof_bytes = length * bytes_per_elem;
    // Potential problem: memmove is not guaranteed to be word atomic
    // Revisit in Merlin
    memmove(dst_a, src_a, nof_bytes); // use memmove as the areas may overlap
    return 0;      
  }
  return -1; // array was not copied
#endif
#endif
JRT_END


// Called from assembly code - not a real entry point

void SharedRuntime::debug(int edi, int esi, int ebp, int esp, int ebx, int edx, int ecx, int eax, int eip, char* msg) {
  // In order to get locks to work, we need to fake a in_VM state
  char buf[200];
  JavaThread* thread = JavaThread::current();
  thread->set_thread_state(_thread_in_vm);
  MutexLocker m(Event_lock);
  // Obfuscate error msg in product mode. 
  obfuscate_location(msg, 0, buf, sizeof(buf));  
  tty->print_cr("EXECUTION STOPPED: %s", buf);
  if (CountBytecodes || TraceBytecodes || StopInterpreterAt) {
    BytecodeCounter::print();
  }
  tty->print_cr("eip = 0x%08x", eip);
  tty->print_cr("eax = 0x%08x", eax);
  tty->print_cr("ebx = 0x%08x", ebx);
  tty->print_cr("ecx = 0x%08x", ecx);
  tty->print_cr("edx = 0x%08x", edx);
  tty->print_cr("edi = 0x%08x", edi);
  tty->print_cr("esi = 0x%08x", esi);
  tty->print_cr("ebp = 0x%08x", ebp);
  tty->print_cr("esp = 0x%08x", esp);
  #ifndef PRODUCT
  BREAKPOINT;
  #else
  os::abort();
  #endif
}


// ---------------------------------------------------------------------------------------------------------
// Non-product code
#ifndef PRODUCT

void SharedRuntime::verify_caller_frame(frame caller_frame, methodHandle callee_method) {
  ResourceMark rm;  
  assert (caller_frame.is_interpreted_frame(), "sanity check");
  assert (callee_method->has_compiled_code(), "callee must be compiled");  
  methodHandle caller_method = caller_frame.interpreter_frame_method();
  jint bci = caller_frame.interpreter_frame_bci();
  methodHandle method = find_callee_method_inside_interpreter(caller_frame, caller_method, bci);
  assert (callee_method == method, "incorrect method");
}

methodHandle SharedRuntime::find_callee_method_inside_interpreter(frame caller_frame, methodHandle caller_method, int bci) {
  EXCEPTION_MARK;
  Bytecode_invoke* bytecode = Bytecode_invoke_at(caller_method, bci);
  methodHandle staticCallee = bytecode->static_target(CATCH); // Non-product code
  
  bytecode = Bytecode_invoke_at(caller_method, bci);
  int bytecode_index = bytecode->index();
  Bytecodes::Code bc = bytecode->adjusted_invoke_code();      

  Handle receiver;
  if (bc == Bytecodes::_invokeinterface ||
      bc == Bytecodes::_invokevirtual ||
      bc == Bytecodes::_invokespecial) {
        receiver = retrieve_receiver(staticCallee->signature(), caller_frame);
  } else {
    receiver = NULL;
  }
  CallInfo result;
  constantPoolHandle constants = caller_method->constants();
  LinkResolver::resolve_invoke(result, receiver, constants, bytecode_index, bc, CATCH); // Non-product code
  methodHandle calleeMethod =  result.selected_method();
  return calleeMethod;
}

#endif 


// Must be entry as it may lock when acquring the jni_id of the method
JRT_ENTRY (void,  SharedRuntime::jvmpi_method_entry(JavaThread* thread, methodOop method, oop receiver))
  GC_locker::lock();
  if (jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY2)) {
    jvmpi::post_method_entry2_event(method, receiver);
  } 
  if (jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_ENTRY)) {
    jvmpi::post_method_entry_event(method);
  } 
  GC_locker::unlock();
JRT_END


// Must be entry as it may lock when acquring the jni_id of the method
JRT_ENTRY (void, SharedRuntime::jvmpi_method_exit(JavaThread* thread, methodOop method))
  GC_locker::lock();
  if (jvmpi::is_event_enabled(JVMPI_EVENT_METHOD_EXIT)) {
    jvmpi::post_method_exit_event(method);
  }
  GC_locker::unlock();
JRT_END


void SharedRuntime::initialize_StrictMath_entry_points() {
  assert(!Universe::is_jdk12x_version(), "entry points do not exist for JDK 1.2");
  lookup_function(java_lang_strict_math_sin, "sin");
  lookup_function(java_lang_strict_math_cos, "cos");
}
