/*
 * @(#)sharedRuntime.hpp	1.100 01/02/09 14:47:53
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


// Runtime is the base class for various runtime interfaces
// (InterpreterRuntime, CompilerRuntime, etc.). It provides
// shared functionality such as exception forwarding (C++ to
// Java exceptions), locking/unlocking mechanisms, statistical
// information, etc.

class SharedRuntime: AllStatic {
 private:
  // The following function pointers provide access to the
  // corresponding StrictMath functions - used for intrinsics
  // inlined by the compiler and special-cased by the interpreter
  // (must match compiled implementation).
  typedef jdouble (JNICALL *StrictMathFunction)(JNIEnv* env_unused, jclass class_unused, jdouble x);
  static inline void lookup_function(StrictMathFunction& f, const char* fname);
  static StrictMathFunction java_lang_strict_math_sin;
  static StrictMathFunction java_lang_strict_math_cos;

 public:
  // The following entry points are used to handle implicit
  // exceptions outside the interpreter - currently they are
  // setup via the JVMCI and only used by C1.
  static address _implicit_null_exception_filter;
  static address _implicit_div0_exception_filter;
  static address _stack_ovrflow_exception_filter;

  // The following arithmetic routines are used on platforms that do
  // not have machine instructions to implement their functionality.
  // Do not remove these.

  // long arithmetics
  static jlong   lmul(jlong y, jlong x);
  static jlong   ldiv(jlong y, jlong x);
  static jlong   lrem(jlong y, jlong x);

  // float and double remainder
  static jfloat  frem(jfloat  x, jfloat  y);
  static jdouble drem(jdouble x, jdouble y);

  // float conversion (needs to set appropriate rounding mode)
  static jint    f2i (jfloat  x);
  static jlong   f2l (jfloat  x);
  static jint    d2i (jdouble x);
  static jlong   d2l (jdouble x);
  static jfloat  d2f (jdouble x);
  static jfloat  l2f (jlong   x);
  static jdouble l2d (jlong   x);

  // double trigonometrics
  static jdouble dsin(jdouble x);
  static jdouble dcos(jdouble x);
  
  // exception handling accross interpreter/compiler boundaries
  static address raw_exception_handler_for_return_address(address return_address);
  static address exception_handler_for_return_address(address return_address);

  // exception handling
  static int     is_interpreter_exception();
  static address compute_exception_return_address(jint is_implicit_null);
  static address handle_array_index_check(JavaThread* thread, jint index);

  // bytecode tracing
  static int trace_bytecode(JavaThread* thread, int preserve_this_value);

#ifndef CORE
#ifndef PRODUCT
  // method tracing
  static void trace_method_entry(JavaThread *thread, methodOop method, address pc);
  static void trace_method_exit(JavaThread *thread, methodOop method, address pc);
#endif
#endif
  
  // Used to back off a spin lock that is under heavy contention
  static void yield_all(JavaThread* thread, int attempts = 0);

  // Returns null-exception object and sets the correct handler_entry for given return address.
  // Note: oop is returned through the thread object and handler_entry as return-parameter
  static address get_null_exception_entry_and_oop (JavaThread *thread);

  static oop retrieve_receiver( symbolOop sig, frame caller );  

  // called when interpreter vtable call needs an adapter before it continues
  static address create_i2c_adapter(JavaThread* thread, methodOop method);

  // return -1 if array was not copied; returns 0 if it was copied
  static int  simple_array_copy_no_checks (int length, int dst_pos, oop dst, int src_pos, oop src);

  // debugging
  static void debug(int edi, int esi, int ebp, int esp, int ebx, int edx, int ecx, int eax, int eip, char* msg);

  static void verify_caller_frame(frame caller_frame, methodHandle callee_method) PRODUCT_RETURN;
  static methodHandle find_callee_method_inside_interpreter(frame caller_frame, methodHandle caller_method, int bci) PRODUCT_RETURN_(return methodHandle((methodOop)NULL););

  // jvmpi
  static void jvmpi_method_entry(JavaThread* thread, methodOop method, oop receiver);
  static void jvmpi_method_exit(JavaThread* thread, methodOop method);

  // intialization
  static void initialize_StrictMath_entry_points();
};


