// @(#)speclock.cpp	1.10 99/11/17 15:26:11
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

#include "incls/_precompiled.incl"
#include "incls/_speclock.cpp.incl"

// +DefaultUseSpecLock => MP yes, Uniproc no
// -DefaultUseSpecLock => MP no,  Uniproc no
// +ForceUseSpecLock => MP yes, Uniproc yes
void speclock_init() {
  
#ifdef COMPILER1
  UseSpecLock = false;
  return;
#endif
  bool on_mp = os::is_MP();
  if (on_mp) {
    UseSpecLock = DefaultUseSpecLock || ForceUseSpecLock;
  } else {
    UseSpecLock = ForceUseSpecLock; 
  }
  if (Verbose || PrintSpecLock) {
    if (UseSpecLock) {
      tty->print_cr("Turning on speclocking");
    } else {
      tty->print_cr("Turning off speclocking");
    }
  }
    
  
}

bool SpecLock::has_speclock_field(symbolOop name) {
   return ((name == vmSymbols::java_util_AbstractList())
     || (name == vmSymbols::java_util_Hashtable())
     || (name == vmSymbols::java_lang_StringBuffer())
     );
}

bool SpecLock::is_not_speclock_class(klassOop lock_klass) {
  return false;
}

bool SpecLock::is_subtype_of(KlassHandle sub_class, klassOop super_class_oop) {
  bool res =  (super_class_oop != NULL) && (sub_class->is_subclass_of(super_class_oop));
  return res;
}

bool SpecLock::is_speclock_class(klassOop lock_klass_oop) {
  assert(lock_klass_oop != NULL, "must have lock klass");
  KlassHandle lock_klass(lock_klass_oop);
  
  ResourceMark rm;
  bool is_vector = is_subtype_of(lock_klass, SystemDictionary::vector_klass());
  bool is_hashtable = is_subtype_of(lock_klass, SystemDictionary::hashtable_klass());
  bool is_stringbuffer = is_subtype_of(lock_klass, SystemDictionary::stringBuffer_klass()); 
  
  bool res_value = is_vector 
    || is_hashtable 
    || is_stringbuffer
    ;
  
  return res_value;

}

bool SpecLock::is_owner_thread(Thread* th, Handle obj) {
  int speclock_owner = owner_from_oop(obj);
  int cur_thread_id = th->get_fast_thread_id();
  return (cur_thread_id == speclock_owner);
}

int SpecLock::owner_from_oop(Handle obj) {
// Normally assume that this is a speclocked oop
#ifdef ASSERT
  markOop mark = obj->mark();
  assert(mark->is_speclock(), "Checking for ownership is meaningless if not speclocked");
#endif
  address_word objaddr = (address_word) (obj());
  int speclock_owner = *((int *)(objaddr + speclock_field_offset()));
  return speclock_owner;
}

void SpecLock::trigger_fixup(Handle h_object, TRAPS) {
  VM_FixupSpecLock fixup_speclock(h_object);
  VMThread::execute(&fixup_speclock);
}

void SpecLock::trigger_fixup_safepoint(Handle h_object) {
  VM_FixupSpecLock fixup_speclock(h_object);
  VMThread::execute(&fixup_speclock);
}

void SpecLock::fixup_object(Handle h_obj) {
  markOop mark = h_obj->mark();
  Thread* in_thread = Thread::current();
 
  // If the object is no longer speclocked, just return
  if (!mark->is_speclock()) {
    if (PrintSpecLock) { tty->print_cr("fixup_object: No longer needed"); }
    return;
  }
  
  int owner_thread_id = owner_from_oop(h_obj);

  // May either be at a safepoint, or in the owner thread of h_obj
  assert(in_thread->is_VM_thread() || (in_thread->get_fast_thread_id() == owner_thread_id),
    "Speclock Fixup must either happen at a safepoint or in the owner thread");
 
  
  if (PrintSpecLock) {
    tty->print_cr("fixup_object is at safepoint? %d oop=%0x owner=%0x current thread=%0x class=%s", 
      SafepointSynchronize::is_at_safepoint(), h_obj(), owner_thread_id, in_thread->get_fast_thread_id(),
      Klass::cast(h_obj()->klass())->name()->as_C_string());
  }
  
  // Find the owner thread, and fix up its stack
  BasicLock* oldest_lock_found = NULL;
  for(JavaThread* thr = Threads::first(); thr; thr = thr->next()) {
    int thr_id = thr->get_fast_thread_id();
    
    if (thr_id == owner_thread_id) {
      oldest_lock_found = fixup_lock_records_on_stack(thr, h_obj);
      break;
    } 
  }
    
  // Now fix up the object header. If the object is recursively locked,
  // its header must point to the stack location of the oldest_lock_found
  // The only possible contention is with someone trying to set the hash value.
  // However: if an object is speclocked, the only thread that can set the
  // hash value is this thread - any other thread will call a fixup speclock
  oop obj = h_obj(); 
  markOop clearMark = mark->clear_speclock_bits();
  
  if (oldest_lock_found != NULL) {
    // Object is locked. Fixup header to point to stack location
    oldest_lock_found->set_displaced_header(clearMark);
    obj->set_mark((markOop)oldest_lock_found);
    if (PrintSpecLock) {
      tty->print_cr("fixup_object: locked header value=%0x", (int) oldest_lock_found);
    }
  } else {
      obj->set_mark(clearMark);
  }

  // Set the object owner field in the object to NULL
  int *thread_id_address = (int *)(speclock_field_offset() + (char *)obj);
  *thread_id_address = 0;

}

// Platform specific - stacks grow downward on Intel
bool SpecLock::is_older_on_stack(int is_expected_older, int is_expected_newer) {
    return (is_expected_older > is_expected_newer);
}

// Fixup all the objects present on this stack
void SpecLock::fixup_all_objects_on_stack(JavaThread* thr) {
  // Not the most efficient way to do the fixup - requires a stack
  // traversal per object. Using a hash table would be better, but
  // this should not happen often
  ResourceMark rm;
  // Find the speclocked objects on the stack
  GrowableArray<oop>* oopArray = new GrowableArray<oop>(10, true);
  if (PrintSpecLock) { tty->print_cr("Fixing up all objects on the stack"); }
  RegisterMap reg_map(thr);
  if (thr->has_last_Java_frame()) {
    frame last_f = thr->last_frame();
    vframe* vf = vframe::new_vframe(&last_f, &reg_map, thr);
    int vfnum = 0;
    for(;vf ; vfnum++, vf = vf->sender()) {
      if (vf->is_java_frame()) {
        javaVFrame* jvf = (javaVFrame *) vf;
        GrowableArray<MonitorInfo*>* mons = jvf->monitors();
        int i;
        for(i= 0; i < mons->length(); i++) {
          MonitorInfo* moninfo = mons->at(i);
          BasicLock *cur_lock = moninfo->lock();
          oop owner = moninfo->owner();
          if ((owner != NULL) && (owner->mark()->is_speclock())) {
            oopArray->push(owner);
          }
        } 
      }
    } 
  } 
  // Fixup all the objects we found
  int i;  for(i = 0; i < oopArray->length(); i++) {
    oop nextVal = oopArray->pop();
    Handle obj(nextVal);
    fixup_object(obj); 
  }

}

BasicLock* SpecLock::fixup_lock_records_on_stack(JavaThread* thr, Handle h_obj) {
  if (PrintSpecLock) { 
    tty->print_cr("SpecLock::fixup_locks_on_stack of %d", thr->get_fast_thread_id());     
    thr->trace_stack();
  } 
  
  // Traverse vframes, fixing up any records referring to h_obj
  BasicLock* oldest_lock_found = NULL;
  RegisterMap reg_map(thr);
  if (thr->has_last_Java_frame()) {
    frame last_f = thr->last_frame();
    if (last_f.is_native_frame()) { 
        last_f = last_f.sender(&reg_map, NULL);
    }
    vframe* vf = vframe::new_vframe(&last_f, &reg_map, thr);
    int vfnum = 0;
    for(;vf ; vfnum++, vf = vf->sender()) {
      if (vf->is_java_frame()) {
        javaVFrame* jvf = (javaVFrame *) vf;
        GrowableArray<MonitorInfo*>* mons = jvf->monitors();

        for(int i= 0; i < mons->length(); i++) {
          MonitorInfo* moninfo = mons->at(i);
          BasicLock *cur_lock = moninfo->lock();
          oop owner = moninfo->owner();
        
          if (owner == h_obj()) {
            cur_lock->set_displaced_header(0);
            if (PrintSpecLock) {
              tty->print_cr("fixup_locks_on_stack. Found lock at=%0x. Oldest=%0x", 
                (int) cur_lock, (int) oldest_lock_found);
            }
            if (oldest_lock_found == NULL) {
              oldest_lock_found = cur_lock;
            } else if (is_older_on_stack((int) cur_lock, (int) oldest_lock_found)) {
              oldest_lock_found = cur_lock;
            }       
          }  

        } 
      }
    } 
  } 
  return oldest_lock_found;
}
