// @(#)speclock.hpp	1.5 00/03/21 16:18:02
// 
// Copyright 1993-1998 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2

class SpecLock : public CHeapObj {
 public:
   
  // Offset of the speclock field with respect to the oop
  static inline int SpecLock::speclock_field_offset() {
         return oopDesc::header_size()*wordSize;
  }

  // Should this class be created speculatively locked
  // and checked for speculative locking?
  static bool is_speclock_class(klassOop k);

  // This class and any of its descendants are known to
  // not be speclocked 
  static bool is_not_speclock_class(klassOop k);

  // Returns true if this class should have the dummy field.
  // This is during class loading, so we can't use the klassOop
  // The caller of this method must check the class loader
  // Note that this is only true for the highest class in an
  // inheritance chain - all the subclasses get it automagically
  static bool has_speclock_field(symbolOop k);

  // Return true if the owner thread is 'th'
  // precondition (obj is speclocked)
  static bool is_owner_thread(Thread* th, Handle obj);

  // Trigger a speclock safepoint to be enqueued to
  // fixup locking on contendedObject
  static void trigger_fixup(Handle contended_object, TRAPS);

  // Avoid using this version! For use in identity_hash_code
  // since the the thread is not available.
  static void trigger_fixup_safepoint(Handle contended_object);

  // Fixup a single object. Must be called at a safepoint
  // or by the owner thread
  static void fixup_object(Handle oop);

  // Fixup all the objects that have monitor records on 
  // th's stack
  static void fixup_all_objects_on_stack(JavaThread* th);


 private:
  // Return the thread owner of this object. 
  // precondition (oop is speclocked)
  static int owner_from_oop(Handle oop);

  // Return true if the first argument address is older than the second addr
  static bool is_older_on_stack(int expected_older_adr, int expected_newer_adr);

  // Fixup all the stack locking records associated with h_obj.
  // Returns the oldest lock on the stack
  static BasicLock* fixup_lock_records_on_stack(JavaThread *thr, Handle obj);

  // Return true if subClass is a subtype of the superClass in the NULL
  // classloader
  static bool is_subtype_of(KlassHandle sub_class, klassOop super_class);


};

// Safepoint vm operation to turn a speculatively locked object into a normally
// locked object
class VM_FixupSpecLock: public VM_Operation {
 private:
   Handle contended_oop;

 public:
  VM_FixupSpecLock(Handle contended_oop_arg)     { contended_oop = contended_oop_arg; }
  void doit()                                    { SpecLock::fixup_object(contended_oop); }
  const char* name() const                       { return "speclockfixup"; }
};

// Initialization of global flags

void speclock_init();

