/*
 * @(#)stubRoutines.hpp	1.82 01/02/09 14:47:55
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// StubRoutines provides entry points to assembly routines used by
// compiled code and the run-time system. Platform-specific entry
// points are defined in the platform-specific inner class.
//
// Class scheme:
//
//    platform-independent               platform-dependent
//
//    stubRoutines.hpp  <-- included --  stubRoutines_<arch>.hpp
//           ^                                  ^
//           |                                  |
//       implements                         implements
//           |                                  |
//           |                                  |
//    stubRoutines.cpp                   stubRoutines_<arch>.cpp
//                                       stubGenerator_<arch>.cpp
//
// Note 1: The important thing is a clean decoupling between stub
//         entry points (interfacing to the whole vm; i.e., 1-to-n
//         relationship) and stub generators (interfacing only to
//         the entry points implementation; i.e., 1-to-1 relationship).
//         This significantly simplifies changes in the generator
//         structure since the rest of the vm is not affected.
//
// Note 2: stubGenerator_<arch>.cpp contains a minimal portion of
//         machine-independent code; namely the generator calls of
//         the generator functions that are used platform-independently.
//         However, it comes with the advantage of having a 1-file
//         implementation of the generator. It should be fairly easy
//         to change, should it become a problem later.
//
// Scheme for adding a new entry point:
//
// 1. determine if it's a platform-dependent or independent entry point
//    a) if platform independent: make subsequent changes in the independent files
//    b) if platform   dependent: make subsequent changes in the   dependent files
// 2. add a private instance variable holding the entry point address
// 3. add a public accessor function to the instance variable
// 4. implement the corresponding generator function in the platform-dependent
//    stubGenerator_<arch>.cpp file and call the function in generate_all() of that file


class StubRoutines: AllStatic {

 public:
  enum platform_independent_constants {
    max_size_of_parameters = 256                           // max. parameter size supported by megamorphic lookups
  };

  // Dependencies
  friend class StubGenerator;
  #include "incls/_stubRoutines_pd.hpp.incl"               // machine-specific parts

  NOT_CORE(static address _lazy_i2c_adapter_entry;)
  static address _verify_oop_entry;
  static jint    _verify_oop_count;
  static address _call_stub_return_address;                // the return PC, when returning to a call stub
  static address _call_stub_entry;  
  static address _forward_exception_entry;
  static address _catch_exception_entry;    
  static address _throw_null_exception_entry;  
  static address _atomic_exchange_entry;
  static address _atomic_store_entry;
  static address _atomic_compare_and_exchange_entry;
  static address _atomic_increment_entry;
  static bool    _is_initialized_1;
  static bool    _is_initialized_2;
  static u_char  _code1[code_size1];                       // code buffer for initial routines
  static u_char  _code2[code_size2];                       // code buffer for all other routines

  static address _d2i_wrapper;
  static address _d2l_wrapper;
  static jint    _fpu_cntrl_wrd_std;
  static jint    _fpu_cntrl_wrd_24;
  static jint    _fpu_cntrl_wrd_trunc;

 public:
  // Initialization/Testing
  static void    initialize1();                            // must happen before universe::genesis
  static void    initialize2();                            // must happen after  universe::genesis

  static bool    contains(address addr)                    { return (_code1 <= addr) && (addr < &_code1[code_size1]) ||
                                                                    (_code2 <= addr) && (addr < &_code2[code_size2]); }  

  // Debugging
  static address verify_oop_entry()                        { return _verify_oop_entry; }  
  static jint    verify_oop_count()                        { return _verify_oop_count; }  
  static jint*   verify_oop_count_addr()                   { return &_verify_oop_count; }  
  static address catch_exception_entry()                   { return _catch_exception_entry; }

  // Calls to Java
  typedef void (*CallStub)(
    address   link,
    jint*     result,
    BasicType result_type,
    methodOop method,
    address   entry_point,
    jint*     parameters,
    int       size_of_parameters,
    TRAPS
  );

  static CallStub call_stub()                              { return CAST_TO_FN_PTR(CallStub, _call_stub_entry); }
  static bool    returns_to_call_stub(address return_pc)   { return return_pc == _call_stub_return_address; }

  static address forward_exception_entry()                 { return _forward_exception_entry; }
  NOT_CORE(static address lazy_i2c_adapter_entry()         { return _lazy_i2c_adapter_entry; })
  
  // Exceptions
  static address throw_null_exception_entry()              { return _throw_null_exception_entry; }

  static address atomic_exchange_entry()                   { return _atomic_exchange_entry; }
  static address atomic_store_entry()                      { return _atomic_store_entry; }
  static address atomic_compare_and_exchange_entry()       { return _atomic_compare_and_exchange_entry; }
  static address atomic_increment_entry()                  { return _atomic_increment_entry; }

  static address d2i_wrapper()                             { return _d2i_wrapper; }
  static address d2l_wrapper()                             { return _d2l_wrapper; }
  static address addr_fpu_cntrl_wrd_std()                  { return (address)&_fpu_cntrl_wrd_std;   }
  static address addr_fpu_cntrl_wrd_24()                   { return (address)&_fpu_cntrl_wrd_24;   }
  static address addr_fpu_cntrl_wrd_trunc()                { return (address)&_fpu_cntrl_wrd_trunc; }
};

