/*
 * @(#)task.cpp	1.14 01/04/21 22:48:01
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_task.cpp.incl"

int PeriodicTask::num_tasks = 0;
PeriodicTask* PeriodicTask::tasks[PeriodicTask::max_tasks];
#ifndef PRODUCT
elapsedTimer PeriodicTask::_timer;
int PeriodicTask::_intervalHistogram[PeriodicTask::max_interval];
int PeriodicTask::_ticks;

void PeriodicTask::print_intervals() {
  if (ProfilerCheckIntervals) {
    for (int i = 0; i < PeriodicTask::max_interval; i++) {
      int n = _intervalHistogram[i];
      if (n > 0) tty->print_cr("%3d: %5d (%4.1f%%)", i, n, 100.0 * n / _ticks);
    }
  }
}
#endif

void PeriodicTask::real_time_tick(int delay_time) {
#ifndef PRODUCT
  if (ProfilerCheckIntervals) {
    _ticks++;
    _timer.stop();
    int ms = (int)(_timer.seconds() * 1000.0);
    _timer.reset();
    _timer.start();
    if (ms >= PeriodicTask::max_interval) ms = PeriodicTask::max_interval - 1;
    _intervalHistogram[ms]++;
  }
#endif
  for(int index = 0; index < num_tasks; index++) {
    tasks[index]->execute_if_pending(delay_time);
  }
}


PeriodicTask::PeriodicTask(int interval_time) {
  assert(is_init_completed(), "Periodic tasks should not start during VM initialization");
  counter  = 0;
  interval = interval_time;
}

PeriodicTask::~PeriodicTask() {
  if (is_enrolled())
    deroll();
}

bool PeriodicTask::is_enrolled() const {
  for(int index = 0; index < num_tasks; index++) 
    if (tasks[index] == this) return true;
  return false;
}

void PeriodicTask::enroll() {
  if (num_tasks == PeriodicTask::max_tasks)
    fatal("Overflow in PeriodicTask table");
  tasks[num_tasks++] = this;
  WatcherThread::start();
  SuspendCheckerThread::start();
}

void PeriodicTask::deroll() {
  int index;
  for(index = 0; index < num_tasks && tasks[index] != this; index++);
  if (index == PeriodicTask::max_tasks) return;
  num_tasks--;
  for (; index < num_tasks; index++) {
    tasks[index] = tasks[index+1];
  }
}

