/*
 * @(#)task.hpp	1.11 01/04/21 22:48:02
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// A PeriodicTask has the sole purpose of executing its task
// function with regular intervals.
// Usage:
//   PeriodicTask pf(10);
//   pf.enroll();
//   ...
//   pf.deroll();

class PeriodicTask: public CHeapObj {
 public:
  enum { max_tasks = 10, max_interval = 1000 };
 private:
  int counter;
  int interval;

  static int num_tasks;
  static PeriodicTask* tasks[PeriodicTask::max_tasks];
  static void real_time_tick(int delay_time);

#ifndef PRODUCT
  static elapsedTimer _timer;                      // measures time between ticks
  static int _ticks;                               // total number of ticks
  static int _intervalHistogram[max_interval];     // to check spacing of timer interrupts
 public:
  static void print_intervals();
#endif
  friend class WatcherThread;
  friend class SuspendCheckerThread;
 public:
  PeriodicTask(int interval_time); // interval is in milliseconds of elapsed time
  ~PeriodicTask();

  // Tells whether is enrolled
  bool is_enrolled() const;

  // Make the task active
  void enroll();

  // Make the task deactive
  void deroll();

  void execute_if_pending(int delay_time) {
    counter += delay_time;
    if (counter >= interval) {
      task();
      counter = 0;
    }
  }

  // The task to perform at each period
  virtual void task() = 0;
};

