/*
 * @(#)threadCodeBuffer.cpp	1.20 00/07/19 16:16:56
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
//
# include "incls/_precompiled.incl"
# include "incls/_threadCodeBuffer.cpp.incl"

// -------------------------------------------------------------------------------------------------------
// Implementation of ThreadCodeBuffer

ThreadCodeBuffer::ThreadCodeBuffer(int size_in_bytes, nmethod *nm, address real_pc) {
   _code = NEW_C_HEAP_ARRAY(u_char, size_in_bytes);
  assert(_code != NULL, "out of memory");
  _size = size_in_bytes;
  _method = nm;
  _real_pc = real_pc;

  debug_only(
    // Initialize area 
    for(int i=0; i<size_in_bytes; i++) {
      _code[i] = 0xEE;  
    }
  )
}

ThreadCodeBuffer::~ThreadCodeBuffer() {
  assert(_code != NULL, "buffer must be allocated");
  FREE_C_HEAP_ARRAY(char, _code);
  debug_only(
    _code = NULL;
    _size = 0;  
  )
}

// returns true if successful, false otherwise
bool ThreadCodeBuffer::reposition_thread(JavaThread* thread, ExtendedPC addr) {
  return os::set_thread_pc(thread, addr.adjust(method()->instructions_begin(), method()->instructions_end(), code_begin()));
}

void ThreadCodeBuffer::copy(int offset, address start, int length) {
  assert(length>=0, "invalid parameters to copy_code");
  assert(offset+length<=size(), "code buffer to small");
  assert(_method->constants_begin()<=start, "start not right nmethod");
  assert(start+length <= _method->instructions_end(), "end not right nmethod");
  
  // copy raw bytes
  memcpy(_code+offset, start, length);
}

void ThreadCodeBuffer::copy_code(int offset, address start, int length) {
  // Initialize buffer    
  assert(length>=0, "invalid parameters to copy_code");
  assert(offset+length<=size(), "code buffer to small");
  assert(_method->instructions_begin() <= start, "start not right nmethod");
  assert(start+length <= _method->instructions_end(), "end not right nmethod");

  address copy_start = _code + offset;

  // copy raw bytes
  memcpy(copy_start, start, length);

  // reallocate all position dependent code in range  
  int delta = copy_start - start;
  
  RelocIterator iter(delta, _method, copy_start, copy_start + length);
  while(iter.next()) {
    if (iter.type() != relocInfo::internal_word_type) {
      iter.reloc()->fix_relocation_at_move(delta);
    }
  }
}

