/*
 * @(#)threadCodeBuffer.hpp	1.20 99/08/26 15:48:21
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
//
  
//
// On-demand allocated code-buffer for roll-forward at safepoint
// 
// Contains all information need for roll-forward when patching code.
//

class ThreadCodeBuffer : public CHeapObj {
 private:  
  address   _code;
  int       _size;
  nmethod*  _method;
  address   _real_pc;         // value of pc that correspond to code_begin()

 public:
  ThreadCodeBuffer(int size_in_bytes, nmethod *nm, address real_pc);
  ~ThreadCodeBuffer();
  
  // Accessors   
  address   code_begin() const          { return _code; }
  address   code_end() const            { return _code + _size; }
  bool	    contains(address pc) const  { return (code_begin() <= pc && pc < code_end()); }
  bool      captures(address pc) const  { return (real_pc() <= pc && pc < real_pc() + size()); }
  int       size() const                { return _size; }
  address   real_pc() const             { return _real_pc; }
  nmethod * method() const              { return _method; }
  
  // calculates real_pc (pointing into nmethod) from pc pointing into code_buffer
  address compute_adjusted_pc(address pc);

  // the inverse:  pc was a "real_pc" and must be redirected into the code_buffer
  address capture_pc(address pc);

  // Execute code with thread
  void	  copy     (int offset, address start, int length);  
  void	  copy_code(int offset, address start, int length);  // do relocation
  bool    reposition_thread(JavaThread *thread, ExtendedPC addr);
};

inline address ThreadCodeBuffer::compute_adjusted_pc(address pc) {
  assert(contains(pc), "pc must point into codebuffer") 
  pc = real_pc() + (pc - code_begin());
  assert(method()->contains(pc), "result must be in nmethod");      
  return pc;
}
 
inline address ThreadCodeBuffer::capture_pc(address pc) {
  assert(captures(pc), "pc must point into original code for codebuffer") 
  pc = code_begin() + (pc - real_pc());
  assert(contains(pc), "result must be in codebuffer");
  return pc;
}
 
