/*
 * @(#)threadLocalStorage.cpp	1.24 99/07/10 01:13:18
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_threadLocalStorage.cpp.incl"

// static member initialization
int ThreadLocalStorage::_thread_index = 0;
Thread* ThreadLocalStorage::_get_thread_cache[ThreadLocalStorage::_pd_cache_size] = {NULL};

Thread* ThreadLocalStorage::get_thread_slow() {
  return (Thread*) os::thread_local_storage_at(ThreadLocalStorage::thread_index());
}

Thread* ThreadLocalStorage::get_thread_via_cache() {
  address sp        = pd_sp_address();
  unsigned sp_page  = pd_sp_page(sp);
  unsigned sp_index = pd_sp_hash(sp_page) % _pd_cache_size;
  Thread* thread = _get_thread_cache[ sp_index ];
  if (thread->_last_sp_hash == sp_page) {
    // This works because pd_sp_page does not throw away bits of the page frame.
    // Thus, a sp_page cannot be shared between two threads, since two addresses
    // in one page of memory cannot be found in two thread stacks.
    assert(thread->_stack_base == NULL || (sp <= thread->_stack_base && sp >= thread->_stack_base - thread->_stack_size) || is_error_reported(), "sp must be inside of selected thread stack");
    if ( VerifyThread )  guarantee(thread == get_thread_slow(), "TLS cache validity");
    return thread;
  } else {
    thread = get_thread_slow();
    if (thread != NULL) {
      guarantee(thread->_stack_base == NULL || (sp <= thread->_stack_base && sp >= thread->_stack_base - thread->_stack_size) || is_error_reported(), "sp must be inside of selected thread stack");
      thread->_last_sp_hash = sp_page;  // mark for quick retrieval
      _get_thread_cache[ sp_index ] = thread;
    }
    return thread;
  }
}

static const double all_zero[ sizeof(Thread) / sizeof(double) + 1 ] = {0};
#define NO_CACHED_THREAD ((Thread*)all_zero)

void ThreadLocalStorage::set_thread(Thread* thread) {
  Thread* old_thread = get_thread_slow();
  if (old_thread != NULL) {
    // Remove it from the cache:
    for (int i = 0; i < _pd_cache_size; i++) {
      if (_get_thread_cache[i] == old_thread)
	_get_thread_cache[i] = NO_CACHED_THREAD;
    }
  }
  os::thread_local_storage_at_put(ThreadLocalStorage::thread_index(), thread);
  // The following ensure that any optimization tricks we have tried
  // did not backfire on us:
  guarantee(get_thread()      == thread, "must be the same thread, slowly");
  guarantee(get_thread_slow() == thread, "must be the same thread, quickly");
}

void ThreadLocalStorage::init() {
  assert(ThreadLocalStorage::thread_index() == 0, "More than one attempt to initialize threadLocalStorage");
  for (int i = 0; i < _pd_cache_size; i++) {
    _get_thread_cache[i] = NO_CACHED_THREAD;
  }
  set_thread_index(os::allocate_thread_local_storage());
  generate_code_for_get_thread();
}

void threadLocalStorage_exit() {
  os::free_thread_local_storage(ThreadLocalStorage::thread_index());
}

