/*
 * @(#)timer.cpp	1.14 00/03/21 16:18:07
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_timer.cpp.incl"


void elapsedTimer::add(elapsedTimer t) {
  _counter += t._counter;
}

void elapsedTimer::start() {
  if (!_active) {
    _active = true;
    _counter = os::elapsed_counter() - _counter; // value of _counter is initially zero
  }
}

void elapsedTimer::stop() {
  if (_active) {
    _counter = os::elapsed_counter() - _counter;
    _active = false;
  }
}

double elapsedTimer::seconds() {
  double count = (double) _counter;
  double freq  = (double) os::elapsed_frequency();
  return count/freq;
}

void TimeStamp::update() {
  _counter = os::elapsed_counter();
  if (_counter == 0)  _counter = 1;
  assert(is_updated(), "must not look clear");
}

double TimeStamp::seconds() {
  assert(is_updated(), "must not be clear");
  jlong new_count = os::elapsed_counter();
  double count = (double) new_count - _counter;
  double freq  = (double) os::elapsed_frequency();
  return count/freq;
}

TraceTime::TraceTime(const char* title, bool doit) {
  _active = doit;
  _verbose = true;
  if (_active) {
    _accum = NULL;
    tty->print("[%s", title);
    _t.start();
  }
}

TraceTime::TraceTime(const char* title, elapsedTimer* accumulator, bool doit, bool verbose) {
  _active = doit;
  _verbose = verbose;
  if (_active) {
    if (_verbose)
      tty->print("[%s", title);
    _accum = accumulator;
    _t.start();
  }
}

TraceTime::~TraceTime() {
  if (_active) {
    _t.stop();
    if (_accum!=NULL) _accum->add(_t);
    if (_verbose)
      tty->print_cr(", %3.7f secs]", _t.seconds());
  }
}
