/*
 * @(#)timer.hpp	1.16 00/03/21 16:18:08
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Timers for simple measurement.

class elapsedTimer VALUE_OBJ_CLASS_SPEC {
 private:
  jlong _counter;
  bool  _active;
 public:
  elapsedTimer()    { _active = false; reset(); }
  void add(elapsedTimer t);
  void start();
  void stop();
  void reset()      { _counter = 0; }
  double seconds();
};

// TimeStamp is used for recording when an event took place.
class TimeStamp VALUE_OBJ_CLASS_SPEC {
 private:
  jlong _counter;
 public:
  TimeStamp()  { _counter = 0; }
  void clear() { _counter = 0; }
  // has the timestamp been updated since being created or cleared?
  bool is_updated() { return _counter != 0; }
  // update to current elapsed time
  void update();
  // returns seconds since updated
  // (must not be in a cleared state:  must have been previously updated)
  double seconds();
};

// TraceTime is used for tracing the execution time of a block
// Usage:
//  { TraceTime t("block time")
//    some_code();
//  }
//

class TraceTime: public StackObj {
 private:
  bool          _active;    // do timing
  bool          _verbose;   // report every timing
  elapsedTimer  _t;         // timer
  elapsedTimer* _accum;     // accumulator
 public:
  // Constuctors
  TraceTime(const char* title, bool doit = true);
  TraceTime(const char* title, elapsedTimer* accumulator, bool doit = true, bool verbose = false);
  ~TraceTime();

  // Accessors
  void set_verbose(bool verbose)  { _verbose = verbose; }
  bool verbose() const            { return _verbose;    }

  // Activation
  void suspend()  { if (_active) _t.stop();  }
  void resume()   { if (_active) _t.start(); }
};
