/*
 * @(#)vframeArray.cpp	1.94 01/04/21 22:48:05
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_vframeArray.cpp.incl"


vframeArray* vframeArray::allocate(JavaThread* thread, int locals_to_remove, GrowableArray<compiledVFrame*>* chunk, RegisterMap *reg_map, frame last_i2c_frame) {

  // Compute the size of the storage we need
  int size = vframeArray::array_length_for(chunk);

  // Allocate the vframeArray
  vframeArray * result = (vframeArray*) AllocateHeap(size * sizeof(int) + sizeof(vframeArray),
						     "vframeArray::allocate");
  result->_length = size;
  result->_adapter = NULL;      // Initialize some fields  
  result->_last_i2c_frame = last_i2c_frame;
  result->fill_in(thread, locals_to_remove, chunk, reg_map, last_i2c_frame.sp() == NULL);
  return result;
}



javaVFrame* vframeArray::vframe_at(int index) {
  return new deoptimizedVFrame(this, index);
}


int vframeArray::fill_in_locals(int index, compiledVFrame* vf) {
  GrowableArray<StackValue *> *locs = vf->locals();
  for(int i = 0; i < locs->length(); i++) {
    StackValue *value = locs->at(i);
    switch(value->type()) {      
      case T_OBJECT:
        obj_at_put(index + (i*2), value->get_obj()()); 
        int_at_put(index + (i*2) + 1, 1);
        break;
      case T_INT:
        int_at_put(index + (i*2), value->get_int()); 
        int_at_put(index + (i*2) + 1, 0);
        break;
      case T_CONFLICT:
        int_at_put(index + (i*2), 0);
        int_at_put(index + (i*2) + 1, 0);
        break;
      default:
        ShouldNotReachHere();
    }
  }
  return locs->length() * 2;
}


int vframeArray::fill_in_expression_stack(int index, compiledVFrame* vf) {
  GrowableArray<StackValue *> *exprs = vf->expressions();  
  for(int i = 0; i < exprs->length(); i++) {
    StackValue *value = exprs->at(i);
    switch(value->type()) {
      case T_OBJECT: 
        obj_at_put(index + (i*2), value->get_obj()());
        int_at_put(index + (i*2) + 1, 1);
        break;
      case T_INT:
        int_at_put(index + (i*2), value->get_int());
        int_at_put(index + (i*2) + 1, 0);
        break;
      default:
        ShouldNotReachHere();
    }
  }
  return exprs->length() * 2;
}


void vframeArray::migrate_monitors_off_stack() {
  for (int index = first_index(); index < length(); index = sender_index_for(index)) {
     MonitorArray* array = monitor_array_for(index);
     if (array != NULL) {
       array->migrate_locks_off_stack(owner_thread());
     }
  }
}

class MonitorArrayElement {
 public:
  oop        owner;
  BasicLock* lock;
};

MonitorArray::MonitorArray(int length, JavaThread* owner) {
  _length        = length;
  _monitors      = NEW_C_HEAP_ARRAY(MonitorArrayElement, _length);
  _owner         = owner;
  _monitor_chunk = NULL;
  _next          = NULL;
  owner->add_monitor_array(this);
}

MonitorArray::~MonitorArray() {
  assert(_monitors == NULL, "must be deallocated by now");;
  assert(!has_locks_on_stack(), "must have locks off stack");
  delete monitor_chunk();
}

MonitorArrayElement* MonitorArray::at(int index) {
  assert(has_locks_on_stack(), "must have locks on stack");
  assert(index >= 0 &&  index < _length, "out of bounds check");
  return &_monitors[index];
}

void MonitorArray::at_put(int index, oop owner, BasicLock* lock) {
  assert(has_locks_on_stack(), "must have locks on stack");
  at(index)->owner = owner;
  at(index)->lock  = lock;
}

void MonitorArray::oops_do(void f(oop*)) {
  if (!has_locks_on_stack()) return; // handled by MonitorChunk registerd in thread
  for (int index = 0; index < length(); index++) {
    f(&at(index)->owner);
  }
}

void MonitorArray::monitors_do(void f(oop, BasicLock*)) {
  for (int index = 0; index < length(); index++) {
    f(at(index)->owner, at(index)->lock);
  }
}

void MonitorArray::migrate_locks_off_stack(JavaThread* owner) {
  // Allocate monitor chunk
  MonitorChunk* chunk = new MonitorChunk(length());
  owner->add_monitor_chunk(chunk);

  // Migrate the BasicLocks from the stack to the monitor chunk
  for (int index = 0; index < length(); index++) {
    MonitorArrayElement* monitor = at(index);
    assert(monitor->owner == NULL || !monitor->owner->is_unlocked(), "object must be null or locked");
    BasicObjectLock* dest = chunk->at(index);
    dest->set_obj(monitor->owner);
    monitor->lock->move_to(monitor->owner, dest->lock());
    monitor->lock = dest->lock();
  }

  owner->remove_monitor_array(this);
  FREE_C_HEAP_ARRAY(MonitorArrayElement, _monitors);
  _monitors = NULL;

  _monitor_chunk = chunk;
}

GrowableArray<MonitorInfo*>* MonitorArray::as_monitor_list() {
  GrowableArray<MonitorInfo*>* result = new GrowableArray<MonitorInfo*>(length());
  if (has_locks_on_stack()) {
    for (int index = 0; index < length(); index++) {
      result->push(new MonitorInfo(at(index)->owner, at(index)->lock));
    }
  } else {
    for (int index = 0; index < length(); index++) {
      BasicObjectLock* ol = _monitor_chunk->at(index);
      result->push(new MonitorInfo(ol->obj(), ol->lock()));
    }
  }
  return result;
}

MonitorArray* vframeArray::register_monitors(compiledVFrame* vf) {
  GrowableArray<MonitorInfo*>* list = vf->monitors();
  if (list->is_empty()) return NULL;

  MonitorArray* result = new MonitorArray(list->length(), owner_thread());
  for (int index = 0; index < list->length(); index++) {
    MonitorInfo* monitor = list->at(index);
    // when the thread is blocked in wait, some objects may be unlocked temporarily,
    // therefore we must loosen the assert
    assert(vf->thread()->thread_state() == _thread_blocked || monitor->owner() == NULL || !monitor->owner()->is_unlocked(), "object must be null or locked or blocked in wait");
    result->at(index)->owner = monitor->owner();
    result->at(index)->lock  = monitor->lock();
  }
  return result;
}

int vframeArray::fill_in_vframe(int index, compiledVFrame* vf) {
  // Static part of activation
  obj_at_put(index + method_offset,       vf->method());
  int_at_put(index + bci_offset,          vf->raw_bci());
  int_at_put(index + monitor_list_offset, (int) register_monitors(vf));

  // Locals
  int locals_size = fill_in_locals(index + vframe_fixed_size, vf);
  int_at_put(index + locals_size_offset, locals_size);

  // Expression stack
  int expression_stack_size = fill_in_expression_stack(index + vframe_fixed_size + locals_size, vf);
  int_at_put(index + expression_stack_size_offset, expression_stack_size);

  // Sender index
  int sender_index = index + vframe_fixed_size + locals_size + expression_stack_size;
  int_at_put(index + sender_index_offset, sender_index);
  return sender_index;
}

void vframeArray::fill_in(JavaThread* thread, int frame_size, GrowableArray<compiledVFrame*>* chunk, const RegisterMap *reg_map, bool needs_adapter) {
#ifndef COMPILER2
  ShouldNotReachHere();
#else
  // Set owner first, it is used when adding monitor chunks
  _owner_thread = thread;
  _frame_size = frame_size;
  int index = first_index();
  for(int i = 0; i < chunk->length(); i++) {
    index = fill_in_vframe(index, chunk->at(i));
  }
  assert(index == length(), "checking vframe array length");

  // Copy registers for callee-saved registers
  if (reg_map != NULL) {
    for(int i = 0; i < RegisterMap::reg_count; i++) {
      jint* src = (jint*) reg_map->location(VMReg::Name(i));
      jint* dst = (jint*) register_location(i);      
      *dst = src != NULL ? *src : NULL;
    }
  }

  // Find C2IAdapter for last vframe, but only if we do not have an
  // I2C adapter just above us on the stack (avoids paired I2C/C2I
  // adapters).
  if( needs_adapter ) {
    javaVFrame *jvf = chunk->at(chunk->length() - 1);
    methodOop callee = jvf->method();
    set_adapter(C2IAdapterGenerator::adapter_for(callee));
  }
#endif
}

void vframeArray::unpack_to_stack(frame unpack_frame, frame caller_frame, int exec_mode) {
  // stack picture
  //   unpack_frame
  //   [space for unpacking]
  //   caller_frame

  // Collect frames to unpack
  GrowableArray<deoptimizedVFrame*>* chunk = new GrowableArray<deoptimizedVFrame*>(10);
  for (javaVFrame* vf = new deoptimizedVFrame(this, vframeArray::first_index()); vf; vf = vf->java_sender()) {
    chunk->push(deoptimizedVFrame::cast(vf));
  }

  // Do the unpacking
  frame current_caller = caller_frame;

  for (int index = chunk->length() - 1; index >= 0; index--) {
    int callee_parameters = index == 0
                          ? 0
                          : chunk->at(index-1)->method()->size_of_parameters();
    int callee_locals     = index == 0
                          ? 0
                          : chunk->at(index-1)->method()->max_locals();
    caller_frame = chunk->at(index)->unpack_vframe_on_stack(caller_frame, callee_parameters, callee_locals, index == 0, exec_mode);
  }

  // Compute the pc of the youngest unpacked frame
  address pc;
  switch (exec_mode) {
  case Deoptimization::Unpack_deopt:
    // normal deoptimization, use pc computed in unpack_vframe_on_stack
    pc = caller_frame.pc();
    break;
  case Deoptimization::Unpack_exception:
    // exception is pending
    pc = SharedRuntime::raw_exception_handler_for_return_address(caller_frame.pc());
    break;
  case Deoptimization::Unpack_uncommon_trap:
    // redo last byte code
    pc  = Interpreter::deopt_entry(vtos, 0);
    break;
  default:
    ShouldNotReachHere();
  }
  caller_frame.set_pc(pc);

  // Update return address and other bits in youngest frame
  unpack_frame.set_sender_pc_for_unpack(caller_frame, exec_mode);

  deallocate_monitor_chunks();
}

void vframeArray::validate_index(int index) const {
  // Validate the index as start of deoptimized vframe
}

int vframeArray::sender_index_for(int index) const {
  validate_index(index);
  return int_at(index + sender_index_offset);
}

methodOop vframeArray::method_for(int index) const {
  validate_index(index);
  methodOop result = methodOop(obj_at(index + method_offset));
  assert(result->is_method(), "just ckecking");
  return result;
}

int vframeArray::bci_for(int index) const {
  validate_index(index);
  return int_at(index + bci_offset);
}

void vframeArray::set_bci_for(int index, int rawbci) {
  validate_index(index);
  int_at_put(index + bci_offset, rawbci);
}

MonitorArray* vframeArray::monitor_array_for(int index) const {
  validate_index(index);
  return (MonitorArray*) int_at(index + monitor_list_offset);
}

int vframeArray::locals_size_for(int index) const {
  return int_at(index + locals_size_offset) / 2;
}

int vframeArray::expression_stack_size_for(int index) const  {
  return int_at(index + expression_stack_size_offset) / 2 ;
}

int vframeArray::locals_index_for(int index, int offset) const {
  return index + vframe_fixed_size + (offset*2);
}

int vframeArray::expression_stack_index_for(int index, int offset) const {
  return index + vframe_fixed_size + (locals_size_for(index) * 2) + (offset * 2);
}


bool vframeArray::locals_is_obj_for(int index, int offset) const {
  return int_at(locals_index_for(index, offset) + 1) == 1;
}

oop vframeArray::locals_obj_for(int index, int offset) const {
  return obj_at(locals_index_for(index, offset));
}

oop* vframeArray::locals_obj_addr_for(int index, int offset) const {
  return obj_at_addr(locals_index_for(index, offset));
}

jint vframeArray::locals_int_for(int index, int offset) const {
  return int_at(locals_index_for(index, offset));
}

bool vframeArray::expression_stack_is_obj_for(int index, int offset) const {
  return int_at(expression_stack_index_for(index, offset) + 1) == 1;
}

oop vframeArray::expression_stack_obj_for(int index, int offset) const {
  return obj_at(expression_stack_index_for(index, offset));
}

oop* vframeArray::expression_stack_obj_addr_for(int index, int offset) const {
  return obj_at_addr(expression_stack_index_for(index, offset));
}


jint vframeArray::expression_stack_int_for(int index, int offset) const {
  return int_at(expression_stack_index_for(index, offset));
}


int vframeArray::vframe_store_size(compiledVFrame* vf) {
  int result = 0;
  result += vframe_fixed_size;
  result += vf->method()->max_locals()  * 2;
  result += vf->expressions()->length() * 2;
  return result;
}

int vframeArray::array_length_for(GrowableArray<compiledVFrame*>* chunk) {
  int result = prologue_size;
  for(int index = 0; index < chunk->length(); index++) {
    result += vframe_store_size(chunk->at(index));
  }
  return result;
}

void vframeArray::deallocate_monitor_chunks() {
  for (int index = first_index(); index < length(); index = sender_index_for(index)) {
     MonitorArray* array = monitor_array_for(index);
     if (array != NULL) {
       assert(!array->has_locks_on_stack(), "must have locks off stack");
       owner_thread()->remove_monitor_chunk(array->monitor_chunk());
       delete array;
     }
  }
}

#ifndef PRODUCT

bool vframeArray::structual_compare(JavaThread* thread, GrowableArray<compiledVFrame*>* chunk) {
  if (owner_thread() != thread) return false;
  int index = 0;
  for (javaVFrame* vf = vframe_at(first_index()); vf; vf = vf->java_sender()) {
    if (index >= chunk->length() || !vf->structual_compare(chunk->at(index))) return false;
    index++;
  }
  if (index != chunk->length()) return false;
  return true;
}

#endif

address vframeArray::register_location(int i) const {
  assert(0 <= i && i < RegisterMap::reg_count, "index out of bounds");
  return (address) int_at_addr(first_register_entry_offset + (i*2));
}

void vframeArray::update_register_map(RegisterMap *map) {
  for(int i = 0; i < RegisterMap::reg_count; i++) {
    map->set_location(VMReg::Name(i), register_location(i));
  }
  map->set_include_argument_oops(false);
}

void vframeArray::oops_do(void f(oop*)) {
  // Handle the frames
  for(int index = first_index(); index < length(); index = sender_index_for(index)) {
    int locals_size =  locals_size_for(index);
    int stack_size  =  expression_stack_size_for(index);
    int i;
    
    // Locals
    for(i = 0; i < locals_size; i++) {
      if (locals_is_obj_for(index, i)) {
        f(locals_obj_addr_for(index, i));
      }
    }
    
    // Expression stack
    for(i = 0; i < stack_size; i++) {
      if (expression_stack_is_obj_for(index, i)) {
	f(expression_stack_obj_addr_for(index, i));
      }
    }
    
    // Static part of frame
    f(obj_at_addr(index + vframeArray::method_offset));
  }
}

#ifndef PRODUCT

// Printing

// Note: we cannot have print_on as const, as we allocate inside the method
void vframeArray::print_on_2(outputStream* st)  {
  st->print_cr(" - sp: 0x%lx", sp());
  st->print(" - thread: ");
  owner_thread()->print();
  st->print_cr(" - frame size: %d", frame_size());
  for (javaVFrame* vf = new deoptimizedVFrame(this, vframeArray::first_index()); vf; vf = vf->java_sender()) {
    vf->print();
  }
}


void vframeArray::print_value_on(outputStream* st) const {
  st->print_cr("vframeArray [%d] ", length());
}


#endif



