/*
 * @(#)vframe_hp.hpp	1.32 99/06/29 18:31:50
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// A deoptimizedVFrame is represented by a frame and an offset
// into the packed array of frames.

class deoptimizedVFrame: public javaVFrame {
 public:
  // JVM state
  methodOop                    method()      const;
  int                          bci()         const;
  GrowableArray<StackValue*>*  locals()      const;
  GrowableArray<StackValue*>*  expressions() const;
  GrowableArray<MonitorInfo*>* monitors()    const;

  // Virtuals defined in vframe
  bool is_deoptimized_frame() const { return true; }
  vframe* sender() const;
  bool is_top() const;

  // Casting
  static deoptimizedVFrame* cast(vframe* vf) {
    assert(vf == NULL || vf->is_deoptimized_frame(), "must be deoptimized frame");
    return (deoptimizedVFrame*) vf;
  }

 public:
  // Constructor
  deoptimizedVFrame(const frame* fr, const RegisterMap* reg_map, JavaThread* thread);
  deoptimizedVFrame(const frame* fr, const RegisterMap* reg_map, JavaThread* thread, int index);
  deoptimizedVFrame(vframeArray* array);
  deoptimizedVFrame(vframeArray* array, int index);
 private:
  int            _index;
  vframeArray* _array;

  int            index() const { return _index; }
  vframeArray* array() const { return _array; }  

 public:
  // Returns the the on stack word size for this frame
  // callee_parameters is the number of callee locals residing inside this frame
  #ifdef ASSERT
  int compute_old_frame_size() const;
  #endif
  jint *compute_caller_fp(frame caller_frame) const;
  int on_stack_size(int callee_parameters, int callee_locals) const;

  int raw_bci() const;  

  // Unpacks the vframe to to stack starting at sp, returns the resulting frame.
  frame unpack_vframe_on_stack(frame caller_frame, int callee_parameters, int callee_locals, bool is_top_frame, int exec_mode);
};

class compiledVFrame: public javaVFrame {
 public:
  // JVM state
  methodOop                    method()      const;
  int                          bci()         const;
  GrowableArray<StackValue*>*  locals()      const;
  GrowableArray<StackValue*>*  expressions() const;
  GrowableArray<MonitorInfo*>* monitors()    const;

  // Virtuals defined in vframe
  bool is_compiled_frame() const { return true; }
  vframe* sender() const;
  bool is_top() const;

  // Casting
  static compiledVFrame* cast(vframe* vf) {
    assert(vf == NULL || vf->is_compiled_frame(), "must be compiled frame");
    return (compiledVFrame*) vf;
  }

 public:
  // Constructors  
  compiledVFrame(const frame* fr, const RegisterMap* reg_map, JavaThread* thread, ScopeDesc* scope);

  // Returns the active nmethod
  nmethod*  code() const;

  // Returns the scopeDesc
  ScopeDesc* scope() const { return _scope; }

  // Returns SynchronizationEntryBCI or bci() (used for synchronization)
  int raw_bci() const;

 protected:
  ScopeDesc* _scope;

  //StackValue resolve(ScopeValue* sv) const;
  BasicLock* resolve_monitor_lock(Location location) const;
  StackValue *create_stack_value(ScopeValue *sv) const;

#ifndef PRODUCT
 public:
  void verify() const;
#endif
};

