/*
 * @(#)virtualspace.hpp	1.17 01/02/09 14:48:04
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// ReservedSpace is a data structure for reserving a contiguous address range.

class ReservedSpace VALUE_OBJ_CLASS_SPEC {
  friend class VMStructs;
 private:
  char* _base;
  size_t   _size;

  ReservedSpace(char* base, size_t size);

 public:
  // Constructor
  ReservedSpace(size_t size, size_t forced_base_alignment = 0, bool ISM = false);

  // Accessors
  char* base() { return _base; }
  size_t   size() { return _size; }

  bool is_reserved() { return _base != NULL; }

  // Splitting
  ReservedSpace first_part(size_t partition_size);
  ReservedSpace last_part (size_t partition_size);

  // Alignment
  static size_t page_align_size_up(size_t size);
  static size_t page_align_size_down(size_t size);
};


// VirtualSpace is data structure for committing a previously reserved address range in smaller chunks.

class VirtualSpace VALUE_OBJ_CLASS_SPEC {
  friend class VMStructs;
 private:
  // Reserved area
  char* _low_boundary;
  char* _high_boundary;

  // Committed area
  char* _low;
  char* _high;

  // Grow direction
  bool  _low_to_high;

 public:
  // Committed area
  char* low()  const { return _low; }
  char* high() const { return _high; }

  // Reserved area
  char* low_boundary()  const { return _low_boundary; }
  char* high_boundary() const { return _high_boundary; }

 public:
  // Initialization
  VirtualSpace();
  bool initialize(ReservedSpace rs, size_t committed_size, bool low_to_high = true);
  
  // Destruction
  ~VirtualSpace();

  // Testers (all sizes are byte sizes)
  size_t committed_size()   const;
  size_t reserved_size()    const;
  size_t uncommitted_size() const;
  bool contains(void* p) const;
  bool low_to_high()     const;

  // Operations
  bool expand_by(size_t bytes);    // returns true on success, false otherwise
  void shrink_by(size_t bytes);
  void release();

  // Debugging
  void print() PRODUCT_RETURN;
};
