/*
 * @(#)vmStructs.cpp	1.26 00/10/28 17:31:08
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2
# include "incls/_precompiled.incl"
# include "incls/_vmStructs.cpp.incl"

// Note: the cross-product of (c1, c2, product, nonproduct, ...),
// (nonstatic, static), and (unchecked, checked) has not been taken.
// Only the macros currently needed have been defined.

// A field whose type is not checked is given a null string as the
// type name, indicating an "opaque" type to the serviceability agent.

// NOTE: there is an interdependency between this file and
// HotSpotCTypeDataBase.java, which parses the type strings.


//--------------------------------------------------------------------------------
// VM_STRUCTS
//
// This list enumerates all of the fields the serviceability agent
// needs to know about. Be sure to see also the type table below this one.
// NOTE that there are platform-specific additions to this table in
// vmStructs_<os>_<cpu>.hpp.

#define VM_STRUCTS(nonstatic_field, static_field, unchecked_nonstatic_field, volatile_nonstatic_field, nonproduct_nonstatic_field, nonproduct_noncore_nonstatic_field, c2_nonstatic_field, noncore_nonstatic_field, noncore_static_field, unchecked_c1_static_field, last_entry) \
                                                                                                                                     \
  /*********************************/                                                                                                \
  /* OopDesc and Klass hierarchies */                                                                                                \
  /*********************************/                                                                                                \
                                                                                                                                     \
  nonstatic_field(oopDesc,                     _mark,                                         markOop)                               \
  nonstatic_field(oopDesc,                     _klass,                                        klassOop)                              \
  nonstatic_field(arrayKlass,                  _dimension,                                    int)                                   \
  nonstatic_field(arrayKlass,                  _higher_dimension,                             klassOop)                              \
  nonstatic_field(arrayKlass,                  _lower_dimension,                              klassOop)                              \
  nonstatic_field(arrayKlass,                  _vtable_len,                                   int)                                   \
  nonstatic_field(arrayKlass,                  _alloc_size,                                   juint)                                 \
  nonstatic_field(arrayKlass,                  _array_header_in_bytes,                        int)                                   \
  nonstatic_field(arrayOopDesc,                _length,                                       int)                                   \
  noncore_nonstatic_field(compiledICHolderKlass,   _alloc_size,                               juint)                                 \
  noncore_nonstatic_field(compiledICHolderOopDesc, _holder_method,                            methodOop)                             \
  noncore_nonstatic_field(compiledICHolderOopDesc, _holder_klass,                             klassOop)                              \
  nonstatic_field(constantPoolOopDesc,         _tags,                                         typeArrayOop)                          \
  nonstatic_field(constantPoolOopDesc,         _cache,                                        constantPoolCacheOop)                  \
  nonstatic_field(constantPoolOopDesc,         _pool_holder,                                  klassOop)                              \
  nonstatic_field(constantPoolCacheOopDesc,    _constant_pool,                                constantPoolOop)                       \
  nonstatic_field(instanceKlass,               _array_klasses,                                klassOop)                              \
  nonstatic_field(instanceKlass,               _methods,                                      objArrayOop)                           \
  nonstatic_field(instanceKlass,               _method_ordering,                              typeArrayOop)                          \
  nonstatic_field(instanceKlass,               _local_interfaces,                             objArrayOop)                           \
  nonstatic_field(instanceKlass,               _transitive_interfaces,                        objArrayOop)                           \
  nonstatic_field(instanceKlass,               _nof_implementors,                             int)                                   \
  nonstatic_field(instanceKlass,               _implementor,                                  klassOop)                              \
  nonstatic_field(instanceKlass,               _fields,                                       typeArrayOop)                          \
  nonstatic_field(instanceKlass,               _constants,                                    constantPoolOop)                       \
  nonstatic_field(instanceKlass,               _class_loader,                                 oop)                                   \
  nonstatic_field(instanceKlass,               _protection_domain,                            oop)                                   \
  nonstatic_field(instanceKlass,               _signers,                                      objArrayOop)                           \
  nonstatic_field(instanceKlass,               _source_file_name,                             symbolOop)                             \
  nonstatic_field(instanceKlass,               _inner_classes,                                typeArrayOop)                          \
  nonstatic_field(instanceKlass,               _nonstatic_field_size,                         int)                                   \
  nonstatic_field(instanceKlass,               _static_field_size,                            int)                                   \
  nonstatic_field(instanceKlass,               _static_oop_field_size,                        int)                                   \
  nonstatic_field(instanceKlass,               _nonstatic_oop_map_size,                       int)                                   \
  nonstatic_field(instanceKlass,               _is_marked_dependent,                          bool)                                  \
  nonstatic_field(instanceKlass,               _init_state,                                   instanceKlass::ClassState)             \
  nonstatic_field(instanceKlass,               _init_thread,                                  Thread*)                               \
  nonstatic_field(instanceKlass,               _vtable_len,                                   int)                                   \
  nonstatic_field(instanceKlass,               _itable_len,                                   int)                                   \
  nonstatic_field(instanceKlass,               _reference_type,                               ReferenceType)                         \
  nonstatic_field(instanceKlass,               _oop_map_cache,                                OopMapCache*)                          \
  nonstatic_field(instanceKlass,               _jni_ids,                                      JNIid*)                                \
  nonstatic_field(instanceKlass,               _osr_nmethods_head,                            nmethod*)                              \
  nonstatic_field(instanceKlass,               _breakpoints,                                  BreakpointInfo*)                       \
     static_field(instanceRefKlass,            _discoveredSoftRefs,                           oop)                                   \
     static_field(instanceRefKlass,            _discoveredWeakRefs,                           oop)                                   \
     static_field(instanceRefKlass,            _discoveredFinalRefs,                          oop)                                   \
     static_field(instanceRefKlass,            _discoveredPhantomRefs,                        oop)                                   \
     static_field(instanceRefKlass,            _processing_refs,                              bool)                                  \
     static_field(instanceRefKlass,            _sentinel_ref,                                 oop)                                   \
  unchecked_nonstatic_field(Klass,             _subtype_cache,                                2 * sizeof(klassOop)) /* NOTE: no type */ \
  nonstatic_field(Klass,                       _java_mirror,                                  oop)                                   \
  nonstatic_field(Klass,                       _super,                                        klassOop)                              \
  nonstatic_field(Klass,                       _size_helper,                                  int)                                   \
  nonstatic_field(Klass,                       _name,                                         symbolOop)                             \
  nonstatic_field(Klass,                       _access_flags,                                 AccessFlags)                           \
  nonstatic_field(Klass,                       _subklass,                                     klassOop)                              \
  nonstatic_field(Klass,                       _next_sibling,                                 klassOop)                              \
  nonproduct_nonstatic_field(Klass,            _verify_count,                                 int)                                   \
  nonstatic_field(Klass,                       _alloc_count,                                  juint)                                 \
  nonstatic_field(klassKlass,                  _alloc_size,                                   juint)                                 \
  nonstatic_field(methodKlass,                 _alloc_size,                                   juint)                                 \
  nonstatic_field(methodOopDesc,               _constants,                                    constantPoolOop)                       \
  nonstatic_field(methodOopDesc,               _exception_table,                              typeArrayOop)                          \
  nonstatic_field(methodOopDesc,               _method_size,                                  u2)                                    \
  nonstatic_field(methodOopDesc,               _max_stack,                                    u2)                                    \
  nonstatic_field(methodOopDesc,               _max_locals,                                   u2)                                    \
  nonstatic_field(methodOopDesc,               _size_of_parameters,                           u2)                                    \
  nonstatic_field(methodOopDesc,               _name_index,                                   u2)                                    \
  nonstatic_field(methodOopDesc,               _signature_index,                              u2)                                    \
  c2_nonstatic_field(methodOopDesc,            _null_cast_cache,                              u2)                                    \
  c2_nonstatic_field(methodOopDesc,            _null_cast_seen,                               unsigned char)                         \
  c2_nonstatic_field(methodOopDesc,            _interpreter_throwout_count,                   u2)                                    \
  nonstatic_field(methodOopDesc,               _access_flags,                                 AccessFlags)                           \
  nonstatic_field(methodOopDesc,               _code_size,                                    u2)                                    \
  nonstatic_field(methodOopDesc,               _parameter_info,                               u2)                                    \
  nonstatic_field(methodOopDesc,               _vtable_index,                                 int)                                   \
  noncore_nonstatic_field(methodOopDesc,       _invocation_counter,                           InvocationCounter)                     \
  c2_nonstatic_field(methodOopDesc,            _interpreter_invocation_count,                 int)                                   \
  nonproduct_noncore_nonstatic_field(methodOopDesc, _compiled_invocation_count,               int)                                   \
  nonstatic_field(methodOopDesc,               _code,                                         nmethod*)                              \
  nonstatic_field(methodOopDesc,               _interpreter_entry,                            address)                               \
  noncore_nonstatic_field(methodOopDesc,       _from_compiled_code_entry_point,               address)                               \
  nonproduct_nonstatic_field(methodOopDesc,    _bci_histogram,                                int*)                                  \
  nonstatic_field(objArrayKlass,               _element_klass,                                klassOop)                              \
  nonstatic_field(objArrayKlass,               _bottom_klass,                                 klassOop)                              \
  nonstatic_field(symbolKlass,                 _alloc_size,                                   juint)                                 \
  nonstatic_field(symbolOopDesc,               _next,                                         symbolOop)                             \
  nonstatic_field(symbolOopDesc,               _length,                                       unsigned short)                        \
  unchecked_nonstatic_field(symbolOopDesc,     _body,                                         sizeof(jbyte)) /* NOTE: no type */     \
  nonstatic_field(typeArrayKlass,              _type,                                         int)                                   \
  nonstatic_field(typeArrayKlass,              _scale,                                        int)                                   \
  nonstatic_field(typeArrayKlass,              _max_length,                                   int)                                   \
                                                                                                                                     \
  /************/                                                                                                                     \
  /* Universe */                                                                                                                     \
  /************/                                                                                                                     \
                                                                                                                                     \
     static_field(Universe,                    _boolArrayKlassObj,                            klassOop)                              \
     static_field(Universe,                    _byteArrayKlassObj,                            klassOop)                              \
     static_field(Universe,                    _charArrayKlassObj,                            klassOop)                              \
     static_field(Universe,                    _intArrayKlassObj,                             klassOop)                              \
     static_field(Universe,                    _shortArrayKlassObj,                           klassOop)                              \
     static_field(Universe,                    _longArrayKlassObj,                            klassOop)                              \
     static_field(Universe,                    _singleArrayKlassObj,                          klassOop)                              \
     static_field(Universe,                    _doubleArrayKlassObj,                          klassOop)                              \
     static_field(Universe,                    _symbolKlassObj,                               klassOop)                              \
     static_field(Universe,                    _methodKlassObj,                               klassOop)                              \
     static_field(Universe,                    _klassKlassObj,                                klassOop)                              \
     static_field(Universe,                    _arrayKlassKlassObj,                           klassOop)                              \
     static_field(Universe,                    _objArrayKlassKlassObj,                        klassOop)                              \
     static_field(Universe,                    _typeArrayKlassKlassObj,                       klassOop)                              \
     static_field(Universe,                    _instanceKlassKlassObj,                        klassOop)                              \
     static_field(Universe,                    _constantPoolKlassObj,                         klassOop)                              \
     static_field(Universe,                    _constantPoolCacheKlassObj,                    klassOop)                              \
  noncore_static_field(Universe,               _compiledICHolderKlassObj,                     klassOop)                              \
     static_field(Universe,                    _systemObjArrayKlassObj,                       klassOop)                              \
     static_field(Universe,                    _main_thread_group,                            oop)                                   \
     static_field(Universe,                    _system_thread_group,                          oop)                                   \
     static_field(Universe,                    _the_empty_short_array,                        typeArrayOop)                          \
     static_field(Universe,                    _the_empty_int_array,                          typeArrayOop)                          \
     static_field(Universe,                    _the_empty_system_obj_array,                   objArrayOop)                           \
     static_field(Universe,                    _the_empty_class_klass_array,                  objArrayOop)                           \
     static_field(Universe,                    _finalizer_register_method,                    methodOop)                             \
     static_field(Universe,                    _reflect_invoke_method,                        methodOop)                             \
     static_field(Universe,                    _loader_addClass_method,                       methodOop)                             \
     static_field(Universe,                    _out_of_memory_error_instance,                 oop)                                   \
     static_field(Universe,                    _null_ptr_exception_instance,                  oop)                                   \
     static_field(Universe,                    _arithmetic_exception_instance,                oop)                                   \
     static_field(Universe,                    _vm_exception,                                 oop)                                   \
     static_field(Universe,                    _new_size,                                     size_t)                                \
     static_field(Universe,                    _reserved_new_size,                            size_t)                                \
     static_field(Universe,                    _old_size,                                     size_t)                                \
     static_field(Universe,                    _reserved_old_size,                            size_t)                                \
     static_field(Universe,                    _perm_size,                                    size_t)                                \
     static_field(Universe,                    _reserved_perm_size,                           size_t)                                \
     static_field(Universe,                    _large_typearray_limit,                        int)                                   \
     static_field(Universe,                    _new_gen,                                      NewGeneration*)                        \
     static_field(Universe,                    _old_gen,                                      OldGeneration*)                        \
     static_field(Universe,                    _perm_gen,                                     PermGeneration*)                       \
     static_field(Universe,                    _base_vtable_size,                             int)                                   \
     static_field(Universe,                    _is_jdk12x_version,                            bool)                                  \
     static_field(Universe,                    _bootstrapping,                                bool)                                  \
     static_field(Universe,                    _fully_initialized,                            bool)                                  \
     static_field(Universe,                    _jvmpi_alloc_event_enabled,                    bool)                                  \
     static_field(Universe,                    _jvmpi_move_event_enabled,                     bool)                                  \
     static_field(Universe,                    _jvmpi_jni_global_alloc_event_enabled,         bool)                                  \
     static_field(Universe,                    _jvmpi_jni_global_free_event_enabled,          bool)                                  \
     static_field(Universe,                    _jvmpi_jni_weak_global_alloc_event_enabled,    bool)                                  \
     static_field(Universe,                    _jvmpi_jni_weak_global_free_event_enabled,     bool)                                  \
     static_field(Universe,                    _verify_count,                                 int)                                   \
                                                                                                                                     \
  /**********************************************************************************/                                               \
  /* Generation and Space hierarchies (NOTE: incomplete, but only missing train GC) */                                               \
  /**********************************************************************************/                                               \
                                                                                                                                     \
  nonstatic_field(Generation,                  virtual_space,                                 VirtualSpace)                          \
  nonstatic_field(Generation,                  low_boundary,                                  oop*)                                  \
  nonstatic_field(Generation,                  high_boundary,                                 oop*)                                  \
  nonstatic_field(NewGeneration,               eden_space,                                    EdenSpace)                             \
  nonstatic_field(NewGeneration,               from_space,                                    SurvivorSpace*)                        \
  nonstatic_field(NewGeneration,               to_space,                                      SurvivorSpace*)                        \
  nonstatic_field(NewGeneration,               _saved_mark,                                   WaterMark)                             \
  nonstatic_field(OldGeneration,               _card_marking_space,                           VirtualSpace)                          \
  nonstatic_field(OldGeneration,               _offset_space,                                 VirtualSpace)                          \
  nonstatic_field(OldGeneration,               _offset_array,                                 u_char*)                               \
  nonstatic_field(OneSpaceOldGeneration,       _the_space,                                    OldSpace*)                             \
  nonstatic_field(OneSpaceOldGeneration,       _saved_mark,                                   WaterMark)                             \
  nonstatic_field(OneSpaceOldGeneration,       _expansion_at_safepoint,                       size_t)                                \
  nonstatic_field(TenuredGeneration,           _shrink_factor,                                int)                                   \
  nonstatic_field(PermGeneration,              _capacity_expansion_limit,                     size_t)                                \
                                                                                                                                     \
  nonstatic_field(Space,                       _compaction_top,                               oop*)                                  \
  nonstatic_field(Space,                       _bottom,                                       oop*)                                  \
  nonstatic_field(Space,                       _top,                                          oop*)                                  \
  nonstatic_field(Space,                       _end,                                          oop*)                                  \
  nonstatic_field(Space,                       _first_dead,                                   oop*)                                  \
  nonstatic_field(EdenSpace,                   _proper_top,                                   oop*)                                  \
  nonstatic_field(OldSpace,                    _offset_array,                                 u_char*)                               \
  nonstatic_field(OldSpace,                    _next_offset_threshold,                        oop*)                                  \
  nonstatic_field(OldSpace,                    _next_offset_index,                            int)                                   \
  nonstatic_field(OldSpace,                    _last_offset_top,                              oop*)                                  \
                                                                                                                                     \
  nonstatic_field(ThreadLocalEden,             _start,                                        oop*)                                  \
  nonstatic_field(ThreadLocalEden,             _top,                                          oop*)                                  \
  nonstatic_field(ThreadLocalEden,             _end,                                          oop*)                                  \
  nonstatic_field(ThreadLocalEden,             _size,                                         int)                                   \
  nonstatic_field(ThreadLocalEden,             _has_terminated,                               bool)                                  \
  nonstatic_field(ThreadLocalEden,             _next,                                         ThreadLocalEden*)                      \
  nonstatic_field(ThreadLocalEden,             _prev,                                         ThreadLocalEden*)                      \
     static_field(ThreadLocalEden,             _log2_fragmentation_ratio,                     int)                                   \
     static_field(ThreadLocalEden,             _max_size,                                     int)                                   \
                                                                                                                                     \
  nonstatic_field(VirtualSpace,                _low_boundary,                                 char*)                                 \
  nonstatic_field(VirtualSpace,                _high_boundary,                                char*)                                 \
  nonstatic_field(VirtualSpace,                _low,                                          char*)                                 \
  nonstatic_field(VirtualSpace,                _high,                                         char*)                                 \
  nonstatic_field(VirtualSpace,                _low_to_high,                                  bool)                                  \
                                                                                                                                     \
  nonstatic_field(WaterMark,                   _point,                                        oop*)                                  \
  nonstatic_field(WaterMark,                   _space,                                        Space*)                                \
                                                                                                                                     \
  /*****************/                                                                                                                \
  /* RememberedSet */                                                                                                                \
  /*****************/                                                                                                                \
                                                                                                                                     \
     static_field(RememberedSet,               _byte_map,                                     jbyte*)                                \
     static_field(RememberedSet,               byte_map_base,                                 jbyte*)                                \
                                                                                                                                     \
  /********************************/                                                                                                 \
  /* CodeCache (NOTE: incomplete) */                                                                                                 \
  /********************************/                                                                                                 \
                                                                                                                                     \
     noncore_static_field(CodeCache,           _heap,                                         CodeHeap)                              \
                                                                                                                                     \
  /*******************************/                                                                                                  \
  /* CodeHeap (NOTE: incomplete) */                                                                                                  \
  /*******************************/                                                                                                  \
                                                                                                                                     \
  nonstatic_field(CodeHeap,                    _memory,                                       VirtualSpace)                          \
  nonstatic_field(CodeHeap,                    _segmap,                                       VirtualSpace)                          \
  nonstatic_field(CodeHeap,                    _log2_segment_size,                            int)                                   \
  nonstatic_field(HeapBlock,                   _header,                                       HeapBlock::Header)                     \
  nonstatic_field(HeapBlock::Header,           _length,                                       int)                                   \
  nonstatic_field(HeapBlock::Header,           _used,                                         int)                                   \
                                                                                                                                     \
  /**********************************/                                                                                               \
  /* Interpreter (NOTE: incomplete) */                                                                                               \
  /**********************************/                                                                                               \
                                                                                                                                     \
     static_field(AbstractInterpreter,         _code,                                         StubQueue*)                            \
                                                                                                                                     \
  /****************************/                                                                                                     \
  /* Stubs (NOTE: incomplete) */                                                                                                     \
  /****************************/                                                                                                     \
                                                                                                                                     \
  nonstatic_field(StubQueue,                   _stub_buffer,                                  address)                               \
  nonstatic_field(StubQueue,                   _buffer_limit,                                 int)                                   \
  nonstatic_field(StubQueue,                   _queue_begin,                                  int)                                   \
  nonstatic_field(StubQueue,                   _queue_end,                                    int)                                   \
  nonstatic_field(StubQueue,                   _number_of_stubs,                              int)                                   \
  nonstatic_field(InterpreterCodelet,          _size,                                         int)                                   \
  nonstatic_field(InterpreterCodelet,          _description,                                  const char*)                           \
  nonstatic_field(InterpreterCodelet,          _is_safepoint_safe,                            bool)                                  \
  nonstatic_field(InterpreterCodelet,          _bytecode,                                     Bytecodes::Code)                       \
                                                                                                                                     \
  /***********************************/                                                                                              \
  /* StubRoutines (NOTE: incomplete) */                                                                                              \
  /***********************************/                                                                                              \
                                                                                                                                     \
     static_field(StubRoutines,                _call_stub_return_address,                     address)                               \
                                                                                                                                     \
  /***************************************/                                                                                          \
  /* PcDesc and other compiled code info */                                                                                          \
  /***************************************/                                                                                          \
                                                                                                                                     \
  noncore_nonstatic_field(PcDesc,              _pc_offset,                                    int)                                   \
  noncore_nonstatic_field(PcDesc,              _scope_decode_offset,                          int)                                   \
                                                                                                                                     \
  /***************************************************/                                                                              \
  /* CodeBlobs (NOTE: incomplete, but only a little) */                                                                              \
  /***************************************************/                                                                              \
                                                                                                                                     \
  noncore_nonstatic_field(CodeBlob,            _size,                                         int)                                   \
  noncore_nonstatic_field(CodeBlob,            _header_size,                                  int)                                   \
  noncore_nonstatic_field(CodeBlob,            _relocation_size,                              int)                                   \
  noncore_nonstatic_field(CodeBlob,            _instructions_offset,                          int)                                   \
  noncore_nonstatic_field(CodeBlob,            _data_offset,                                  int)                                   \
  noncore_nonstatic_field(CodeBlob,            _oops_offset,                                  int)                                   \
  noncore_nonstatic_field(CodeBlob,            _oops_length,                                  int)                                   \
  noncore_nonstatic_field(CodeBlob,            _frame_size,                                   int)                                   \
  noncore_nonstatic_field(CodeBlob,            _return_address_offset,                        int)                                   \
  c2_nonstatic_field(CodeBlob,                 _link_offset,                                  int)                                   \
  noncore_nonstatic_field(CodeBlob,            _oop_maps,                                     OopMapSet*)                            \
                                                                                                                                     \
  /**************************************************/                                                                               \
  /* NMethods (NOTE: incomplete, but only a little) */                                                                               \
  /**************************************************/                                                                               \
                                                                                                                                     \
     noncore_static_field(nmethod,             _zombie_instruction_size,                      int)                                   \
  noncore_nonstatic_field(nmethod,             _method,                                       methodOop)                             \
  noncore_nonstatic_field(nmethod,             _invocation_count,                             int)                                   \
  noncore_nonstatic_field(nmethod,             _entry_bci,                                    int)                                   \
  noncore_nonstatic_field(nmethod,             _link,                                         nmethod*)                              \
  noncore_nonstatic_field(nmethod,             _exception_offset,                             int)                                   \
  noncore_nonstatic_field(nmethod,             _stub_offset,                                  int)                                   \
  noncore_nonstatic_field(nmethod,             _scopes_data_offset,                           int)                                   \
  noncore_nonstatic_field(nmethod,             _scopes_pcs_offset,                            int)                                   \
  noncore_nonstatic_field(nmethod,             _handler_table_offset,                         int)                                   \
  noncore_nonstatic_field(nmethod,             _nul_chk_table_offset,                         int)                                   \
  noncore_nonstatic_field(nmethod,             _nmethod_end_offset,                           int)                                   \
  noncore_nonstatic_field(nmethod,             _first_dependent,                              int)                                   \
  noncore_nonstatic_field(nmethod,             _number_of_dependents,                         int)                                   \
  noncore_nonstatic_field(nmethod,             _max_nof_monitors,                             int)                                   \
  noncore_nonstatic_field(nmethod,             _max_nof_locals,                               int)                                   \
  noncore_nonstatic_field(nmethod,             _entry_point,                                  address)                               \
  noncore_nonstatic_field(nmethod,             _verified_entry_point,                         address)                               \
  noncore_nonstatic_field(nmethod,             _interpreter_entry_point,                      address)                               \
  noncore_nonstatic_field(nmethod,             _osr_entry_point,                              address)                               \
  noncore_nonstatic_field(nmethod,             _is_osr_only,                                  bool)                                  \
  noncore_nonstatic_field(nmethod,             _frame_start_offset,                           int)                                   \
  noncore_nonstatic_field(nmethod,             _lock_count,                                   long)                                  \
  noncore_nonstatic_field(nmethod,             _last_seen_on_stack_for_scavenge,              long)                                  \
  noncore_nonstatic_field(nmethod,             _last_seen_on_stack_for_marksweep,             long)                                  \
                                                                                                                                     \
  /********************************/                                                                                                 \
  /* JavaCalls (NOTE: incomplete) */                                                                                                 \
  /********************************/                                                                                                 \
                                                                                                                                     \
  nonstatic_field(JavaCallWrapper,             _last_Java_sp,                                 jint*)                                 \
  nonstatic_field(JavaCallWrapper,             _not_at_call_sp,                               jint*)                                 \
                                                                                                                                     \
  /******************************/                                                                                                   \
  /* Threads (NOTE: incomplete) */                                                                                                   \
  /******************************/                                                                                                   \
                                                                                                                                     \
     static_field(Threads,                     _thread_list,                                  JavaThread*)                           \
                                                                                                                                     \
  nonstatic_field(Thread,                      _tle,                                          ThreadLocalEden)                       \
  nonstatic_field(JavaThread,                  _next,                                         JavaThread*)                           \
  nonstatic_field(JavaThread,                  _threadObj,                                    oop)                                   \
   volatile_nonstatic_field(JavaThread,        _last_Java_sp,                                 jint*)                                 \
   volatile_nonstatic_field(JavaThread,        _not_at_call_sp,                               jint*)                                 \
   volatile_nonstatic_field(JavaThread,        _thread_state,                                 JavaThreadState)                       \
   volatile_nonstatic_field(JavaThread,        _last_native_pc,                               address)                               \
  nonstatic_field(JavaThread,                  _osthread,                                     OSThread*)                             \
                                                                                                                                     \
  /************************/                                                                                                         \
  /* OopMap and OopMapSet */                                                                                                         \
  /************************/                                                                                                         \
                                                                                                                                     \
  noncore_nonstatic_field(OopMap,              _pc_offset,                                    int)                                   \
  noncore_nonstatic_field(OopMap,              _at_call,                                      bool)                                  \
  noncore_nonstatic_field(OopMap,              _omv_count,                                    int)                                   \
  noncore_nonstatic_field(OopMap,              _omv_data_size,                                int)                                   \
  noncore_nonstatic_field(OopMap,              _omv_data,                                     unsigned char*)                        \
  noncore_nonstatic_field(OopMap,              _write_stream,                                 CompressedWriteStream*)                \
  noncore_nonstatic_field(OopMapSet,           _om_count,                                     int)                                   \
  noncore_nonstatic_field(OopMapSet,           _om_size,                                      int)                                   \
  noncore_nonstatic_field(OopMapSet,           _om_data,                                      OopMap**)                              \
                                                                                                                                     \
  /********************/                                                                                                             \
  /* CompressedStream */                                                                                                             \
  /********************/                                                                                                             \
                                                                                                                                     \
  nonstatic_field(CompressedStream,            _buffer,                                       u_char*)                               \
  nonstatic_field(CompressedStream,            _position,                                     int)                                   \
                                                                                                                                     \
  /*********************************/                                                                                                \
  /* SharedInfo (NOTE: incomplete) */                                                                                                \
  /*********************************/                                                                                                \
                                                                                                                                     \
     noncore_static_field(SharedInfo,          stack0,                                        OptoReg::Name)                         \
                                                                                                                                     \
  /*******************************/                                                                                                  \
  /* Runtime1 (NOTE: incomplete) */                                                                                                  \
  /*******************************/                                                                                                  \
                                                                                                                                     \
  unchecked_c1_static_field(Runtime1,          _blobs,                                        sizeof(Runtime1::_blobs)) /* NOTE: no type */ \
                                                                                                                                     \
  /************************/                                                                                                         \
  /* Miscellaneous fields */                                                                                                         \
  /************************/                                                                                                         \
                                                                                                                                     \
  nonstatic_field(AccessFlags,                 _flags,                                        jint)                                  \
  noncore_nonstatic_field(InvocationCounter,   _counter,                                      unsigned int)

  /* NOTE that we do not use the last_entry() macro here; it is used  */
  /* in vmStructs_<os>_<cpu>.hpp's VM_STRUCTS_OS_CPU macro (and must  */
  /* be present there)                                                */

//--------------------------------------------------------------------------------
// VM_TYPES
//
// This list must enumerate at least all of the types in the above
// list. For the types in the above list, the entry below must have
// exactly the same spacing since string comparisons are done in the
// code which verifies the consistency of these tables (in the debug
// build).
//
// In addition to the above types, this list is required to enumerate
// the JNI's java types, which are used to indicate the size of Java
// fields in this VM to the SA. Further, oop types are currently
// distinguished by name (i.e., ends with "oop") over in the SA.
//
// The declare_toplevel_type macro should be used to declare types
// which do not have a superclass.
//
// The declare_integer_type and declare_unsigned_integer_type macros
// are required in order to properly identify C integer types over in
// the SA. They should be used for any type which is otherwise opaque
// and which it is necessary to coerce into an integer value. This
// includes, for example, the type uintptr_t. Note that while they
// will properly identify the type's size regardless of the platform,
// since it is does not seem possible to deduce or check signedness at
// compile time using the pointer comparison tricks, it is currently
// required that the given types have the same signedness across all
// platforms.
//
// NOTE that there are platform-specific additions to this table in
// vmStructs_<os>_<cpu>.hpp.

#define VM_TYPES(declare_type, declare_toplevel_type, declare_integer_type, declare_unsigned_integer_type, declare_noncore_type, declare_noncore_toplevel_type, declare_noncore_integer_type, declare_noncore_unsigned_integer_type, declare_c1_toplevel_type, declare_c2_type, last_entry) \
                                                                          \
  /*************************************************************/         \
  /* Java primitive types -- required by the SA implementation */         \
  /* in order to determine the size of Java fields in this VM  */         \
  /* (the implementation looks up these names specifically)    */         \
  /* NOTE: since we fetch these sizes from the remote VM, we   */         \
  /* have a bootstrapping sequence during which it is not      */         \
  /* valid to fetch Java values from the remote process, only  */         \
  /* C integer values (of known size).                         */         \
  /*************************************************************/         \
                                                                          \
  declare_toplevel_type(jboolean)                                         \
  declare_toplevel_type(jbyte)                                            \
  declare_toplevel_type(jchar)                                            \
  declare_toplevel_type(jdouble)                                          \
  declare_toplevel_type(jfloat)                                           \
  declare_toplevel_type(jint)                                             \
  declare_toplevel_type(jlong)                                            \
  declare_toplevel_type(jshort)                                           \
                                                                          \
  /*********************************************************************/ \
  /* C integer types. User-defined typedefs (like "size_t" or          */ \
  /* "intptr_t") are guaranteed to be present with the same names over */ \
  /* in the SA's type database. Names like "unsigned short" are not    */ \
  /* guaranteed to be visible through the SA's type database lookup    */ \
  /* mechanism, though they will have a Type object created for them   */ \
  /* and are valid types for Fields.                                   */ \
  /*********************************************************************/ \
                                                                          \
  /* FIXME: "bool" is #defined in globalDefinitions.hpp to be "jint". */  \
  /* This has required unwanted specialization of some of the SA's code. */ \
  /* declare_integer_type(bool) */                                        \
  declare_integer_type(int)                                               \
  declare_integer_type(long)                                              \
  declare_unsigned_integer_type(unsigned char)                            \
  declare_unsigned_integer_type(unsigned int)                             \
  declare_unsigned_integer_type(unsigned short)                           \
  /* The compiler thinks this is a different type than */                 \
  /* unsigned short on Win32 */                                           \
  declare_unsigned_integer_type(u2)                                       \
                                                                          \
  /*****************************/                                         \
  /* C primitive pointer types */                                         \
  /*****************************/                                         \
                                                                          \
  declare_toplevel_type(int*)                                             \
  declare_toplevel_type(char*)                                            \
  declare_toplevel_type(const char*)                                      \
  declare_toplevel_type(u_char*)                                          \
  declare_toplevel_type(unsigned char*)                                   \
                                                                          \
  /*******************************************************************/   \
  /* Types which it will be handy to have available over in the SA   */   \
  /* in order to do platform-independent address -> integer coercion */   \
  /* (note: these will be looked up by name)                         */   \
  /*******************************************************************/   \
                                                                          \
  declare_unsigned_integer_type(size_t)                                   \
  declare_integer_type(intptr_t)                                          \
  declare_unsigned_integer_type(uintptr_t)                                \
                                                                          \
  /*********************************/                                     \
  /* OopDesc and Klass hierarchies */                                     \
  /*********************************/                                     \
                                                                          \
  declare_toplevel_type(oopDesc)                                          \
  declare_toplevel_type(Klass_vtbl)                                       \
           declare_type(Klass, Klass_vtbl)                                \
           declare_type(arrayKlass, Klass)                                \
           declare_type(arrayKlassKlass, klassKlass)                      \
           declare_type(arrayOopDesc, oopDesc)                            \
           declare_type(compiledICHolderKlass, Klass)                     \
           declare_type(compiledICHolderOopDesc, oopDesc)                 \
           declare_type(constantPoolKlass, arrayKlass)                    \
           declare_type(constantPoolOopDesc, arrayOopDesc)                \
           declare_type(constantPoolCacheKlass, arrayKlass)               \
           declare_type(constantPoolCacheOopDesc, arrayOopDesc)           \
           declare_type(instanceKlass, Klass)                             \
           declare_type(instanceKlassKlass, klassKlass)                   \
           declare_type(instanceOopDesc, oopDesc)                         \
           declare_type(instanceRefKlass, instanceKlass)                  \
           declare_type(klassKlass, Klass)                                \
           declare_type(klassOopDesc, oopDesc)                            \
           declare_type(markOopDesc, oopDesc)                             \
           declare_type(methodKlass, Klass)                               \
           declare_type(methodOopDesc, oopDesc)                           \
           declare_type(objArrayKlass, arrayKlass)                        \
           declare_type(objArrayKlassKlass, arrayKlassKlass)              \
           declare_type(objArrayOopDesc, arrayOopDesc)                    \
           declare_type(symbolKlass, Klass)                               \
           declare_type(symbolOopDesc, oopDesc)                           \
           declare_type(typeArrayKlass, arrayKlass)                       \
           declare_type(typeArrayKlassKlass, arrayKlassKlass)             \
           declare_type(typeArrayOopDesc, arrayOopDesc)                   \
                                                                          \
  /******************************************************************/    \
  /* Oops (note that these are recognized as oops over in the SA by */    \
  /* checks on the names of the types, so replacing these with      */    \
  /* "oopDesc*" will currently break things)                        */    \
  /******************************************************************/    \
                                                                          \
  declare_toplevel_type(constantPoolOop)                                  \
  declare_toplevel_type(constantPoolCacheOop)                             \
  declare_toplevel_type(klassOop)                                         \
  declare_toplevel_type(markOop)                                          \
  declare_toplevel_type(methodOop)                                        \
  declare_toplevel_type(objArrayOop)                                      \
  declare_toplevel_type(oop)                                              \
  declare_toplevel_type(symbolOop)                                        \
  declare_toplevel_type(typeArrayOop)                                     \
                                                                          \
  /******************************************/                            \
  /* Generation and space hierarchies       */                            \
  /* (needed for run-time type information) */                            \
  /******************************************/                            \
                                                                          \
  declare_toplevel_type(Generation)                                       \
           declare_type(NewGeneration,         Generation)                \
           declare_type(OldGeneration,         Generation)                \
           declare_type(OneSpaceOldGeneration, OldGeneration)             \
           declare_type(TenuredGeneration,     OneSpaceOldGeneration)     \
           declare_type(PermGeneration,        OneSpaceOldGeneration)     \
           declare_type(TrainGeneration,       OldGeneration)             \
  declare_toplevel_type(Space)                                            \
           declare_type(NewSpace,              Space)                     \
           declare_type(EdenSpace,             NewSpace)                  \
           declare_type(SurvivorSpace,         NewSpace)                  \
           declare_type(OldSpace,              Space)                     \
           declare_type(TenuredSpace,          OldSpace)                  \
           declare_type(PermSpace,             OldSpace)                  \
           declare_type(CarSpace,              OldSpace)                  \
  declare_toplevel_type(RememberedSet)                                    \
  declare_toplevel_type(ThreadLocalEden)                                  \
  declare_toplevel_type(VirtualSpace)                                     \
  declare_toplevel_type(WaterMark)                                        \
                                                                          \
  /***********************************************************/           \
  /* Thread hierarchy (needed for run-time type information) */           \
  /***********************************************************/           \
                                                                          \
  declare_toplevel_type(Threads)                                          \
  declare_toplevel_type(ThreadShadow)                                     \
           declare_type(Thread, ThreadShadow)                             \
           declare_type(WatcherThread, Thread)                            \
           declare_type(SuspendCheckerThread, Thread)                     \
           declare_type(JavaThread, Thread)                               \
           declare_type(JVMPIDaemonThread, JavaThread)                    \
           declare_type(DebuggerThread, JavaThread)                       \
   declare_noncore_type(CompilerThread, JavaThread)                       \
  declare_toplevel_type(OSThread)                                         \
                                                                          \
  /***************/                                                       \
  /* Interpreter */                                                       \
  /***************/                                                       \
                                                                          \
  declare_toplevel_type(AbstractInterpreter)                              \
                                                                          \
  /*********/                                                             \
  /* Stubs */                                                             \
  /*********/                                                             \
                                                                          \
  declare_toplevel_type(StubQueue)                                        \
  declare_toplevel_type(StubRoutines)                                     \
  declare_toplevel_type(Stub)                                             \
           declare_type(InterpreterCodelet, Stub)                         \
                                                                          \
  /*************/                                                         \
  /* JavaCalls */                                                         \
  /*************/                                                         \
                                                                          \
  declare_toplevel_type(JavaCallWrapper)                                  \
                                                                          \
  /*************/                                                         \
  /* CodeCache */                                                         \
  /*************/                                                         \
                                                                          \
  declare_noncore_toplevel_type(CodeCache)                                \
                                                                          \
  /************/                                                          \
  /* CodeHeap */                                                          \
  /************/                                                          \
                                                                          \
  declare_toplevel_type(CodeHeap)                                         \
  declare_toplevel_type(HeapBlock)                                        \
  declare_toplevel_type(HeapBlock::Header)                                \
           declare_type(FreeBlock, HeapBlock)                             \
                                                                          \
  /*************************************************************/         \
  /* CodeBlob hierarchy (needed for run-time type information) */         \
  /*************************************************************/         \
                                                                          \
  declare_noncore_toplevel_type(CodeBlob)                                 \
  declare_noncore_type(nmethod,       CodeBlob)                           \
  declare_noncore_type(RuntimeStub,           CodeBlob)                   \
  declare_noncore_type(SafepointBlob,         CodeBlob)                   \
  declare_c2_type(C2IAdapter,         CodeBlob)                           \
  declare_c2_type(DeoptimizationBlob, CodeBlob)                           \
  declare_c2_type(ExceptionBlob,      CodeBlob)                           \
  declare_c2_type(I2CAdapter,         CodeBlob)                           \
  declare_c2_type(OSRAdapter,         CodeBlob)                           \
  declare_c2_type(UncommonTrapBlob,   CodeBlob)                           \
                                                                          \
  /***************************************/                               \
  /* PcDesc and other compiled code info */                               \
  /***************************************/                               \
                                                                          \
  declare_noncore_toplevel_type(PcDesc)                                   \
                                                                          \
  /************************/                                              \
  /* OopMap and OopMapSet */                                              \
  /************************/                                              \
                                                                          \
  declare_noncore_toplevel_type(OopMap)                                   \
  declare_noncore_toplevel_type(OopMapSet)                                \
                                                                          \
  /********************/                                                  \
  /* CompressedStream */                                                  \
  /********************/                                                  \
                                                                          \
  declare_toplevel_type(CompressedStream)                                 \
                                                                          \
  /**************/                                                        \
  /* SharedInfo */                                                        \
  /**************/                                                        \
                                                                          \
  declare_noncore_toplevel_type(SharedInfo)                               \
                                                                          \
  /**********************/                                                \
  /* Runtime1 (C1 only) */                                                \
  /**********************/                                                \
                                                                          \
  declare_c1_toplevel_type(Runtime1)                                      \
                                                                          \
  /***************/                                                       \
  /* Other types */                                                       \
  /***************/                                                       \
                                                                          \
   declare_integer_type(AccessFlags)  /* FIXME: wrong type (not integer) */\
   declare_unsigned_integer_type(juint)                                    \
  declare_toplevel_type(address)      /* FIXME: should this be an integer type? */\
  declare_toplevel_type(BreakpointInfo*)                                  \
   declare_integer_type(Bytecodes::Code)                                  \
  declare_noncore_toplevel_type(CodeBlob*)                                \
  declare_toplevel_type(CompressedWriteStream*)                           \
  declare_toplevel_type(ConstantPoolCacheEntry)                           \
   declare_integer_type(instanceKlass::ClassState)                        \
   declare_noncore_unsigned_integer_type(InvocationCounter) /* FIXME: wrong type (not integer) */ \
  declare_toplevel_type(JavaThread*)                                      \
   declare_integer_type(JavaThreadState)                                  \
  declare_toplevel_type(jbyte*)                                           \
  declare_toplevel_type(jint*)                                            \
  declare_toplevel_type(JNIid*)                                           \
  declare_toplevel_type(NewGeneration*)                                   \
  declare_toplevel_type(nmethod*)                                         \
  declare_toplevel_type(OldGeneration*)                                   \
  declare_toplevel_type(OldSpace*)                                        \
  declare_toplevel_type(oop*)                                             \
  declare_noncore_toplevel_type(OopMap**)                                 \
  declare_toplevel_type(OopMapCache*)                                     \
  declare_noncore_toplevel_type(OopMapSet*)                               \
   declare_noncore_integer_type(OptoReg::Name)                            \
  declare_toplevel_type(OSThread*)                                        \
  declare_toplevel_type(PermGeneration*)                                  \
   declare_integer_type(ReferenceType)                                    \
  declare_toplevel_type(Space*)                                           \
  declare_toplevel_type(StubQueue*)                                       \
  declare_toplevel_type(SurvivorSpace*)                                   \
  declare_toplevel_type(Thread*)                                          \
  declare_toplevel_type(ThreadLocalEden*)                                 \
  declare_toplevel_type(Universe)

  /* NOTE that we do not use the last_entry() macro here; it is used  */
  /* in vmStructs_<os>_<cpu>.hpp's VM_TYPES_OS_CPU macro (and must be */
  /* present there)                                                   */

//--------------------------------------------------------------------------------
// VM_INT_CONSTANTS
//
// This table contains integer constants required over in the
// serviceability agent. The "declare_constant" macro is used for all
// enums, etc., while "declare_preprocessor_constant" must be used for
// all #defined constants.

#define VM_INT_CONSTANTS(declare_constant, declare_preprocessor_constant, declare_c1_constant, declare_c2_constant, declare_c2_preprocessor_constant, declare_noncore_constant, last_entry) \
                                                                          \
  /******************/                                                    \
  /* Useful globals */                                                    \
  /******************/                                                    \
                                                                          \
  declare_preprocessor_constant("ALIGN_ALL_OBJECTS", ALIGN_ALL_OBJECTS)   \
  declare_constant(UseTLE)                                                \
                                                                          \
  /************************************************/                      \
  /* Shared JavaThread constants (see thread.hpp) */                      \
  /************************************************/                      \
                                                                          \
  declare_constant(JavaThread::not_at_call_bit)                           \
                                                                          \
  /*************/                                                         \
  /* BasicType */                                                         \
  /*************/                                                         \
                                                                          \
  declare_constant(T_BOOLEAN)                                             \
  declare_constant(T_CHAR)                                                \
  declare_constant(T_FLOAT)                                               \
  declare_constant(T_DOUBLE)                                              \
  declare_constant(T_BYTE)                                                \
  declare_constant(T_SHORT)                                               \
  declare_constant(T_INT)                                                 \
  declare_constant(T_LONG)                                                \
  declare_constant(T_OBJECT)                                              \
  declare_constant(T_ARRAY)                                               \
  declare_constant(T_VOID)                                                \
  declare_constant(T_ADDRESS)                                             \
  declare_constant(T_CONFLICT)                                            \
  declare_constant(T_ILLEGAL)                                             \
                                                                          \
  /*****************/                                                     \
  /* RememberedSet */                                                     \
  /*****************/                                                     \
                                                                          \
  declare_constant(RememberedSet::card_shift)                             \
  declare_constant(RememberedSet::card_size)                              \
  declare_constant(RememberedSet::card_size_in_oops)                      \
  declare_constant(RememberedSet::clean_card)                             \
  declare_constant(RememberedSet::newgen_card)                            \
  declare_constant(RememberedSet::dirty_card)                             \
                                                                          \
  /*******************/                                                   \
  /* JavaThreadState */                                                   \
  /*******************/                                                   \
                                                                          \
  declare_constant(_thread_uninitialized)                                 \
  declare_constant(_thread_new)                                           \
  declare_constant(_thread_new_trans)                                     \
  declare_constant(_thread_in_native)                                     \
  declare_constant(_thread_in_native_trans)                               \
  declare_constant(_thread_in_vm)                                         \
  declare_constant(_thread_in_vm_trans)                                   \
  declare_constant(_thread_in_Java)                                       \
  declare_constant(_thread_in_Java_trans)                                 \
  declare_constant(_thread_blocked)                                       \
  declare_constant(_thread_blocked_trans)                                 \
  declare_constant(_thread_in_comp_leaf)                                  \
  declare_constant(_thread_in_native_blocked)                             \
                                                                          \
  /**********************/                                                \
  /* Runtime1 (C1 only) */                                                \
  /**********************/                                                \
                                                                          \
  declare_c1_constant(Runtime1::osr_frame_return_id)                      \
                                                                          \
  /*********************************************/                         \
  /* MethodCompilation (globalDefinitions.hpp) */                         \
  /*********************************************/                         \
                                                                          \
  declare_constant(InvocationEntryBci)                                    \
  declare_constant(InvalidOSREntryBci)                                    \
                                                                          \
  /***************/                                                       \
  /* OopMapValue */                                                       \
  /***************/                                                       \
                                                                          \
  declare_noncore_constant(OopMapValue::type_bits)                        \
  declare_noncore_constant(OopMapValue::register_bits)                    \
  declare_noncore_constant(OopMapValue::type_shift)                       \
  declare_noncore_constant(OopMapValue::register_shift)                   \
  declare_noncore_constant(OopMapValue::type_mask)                        \
  declare_noncore_constant(OopMapValue::type_mask_in_place)               \
  declare_noncore_constant(OopMapValue::register_mask)                    \
  declare_noncore_constant(OopMapValue::register_mask_in_place)           \
  declare_noncore_constant(OopMapValue::unused_value)                     \
  declare_noncore_constant(OopMapValue::oop_value)                        \
  declare_noncore_constant(OopMapValue::value_value)                      \
  declare_noncore_constant(OopMapValue::dead_value)                       \
  declare_noncore_constant(OopMapValue::callee_saved_value)               \
  declare_noncore_constant(OopMapValue::derived_oop_value)                \
                                                                          \
  /********************************/                                      \
  /* Calling convention constants */                                      \
  /********************************/                                      \
                                                                          \
  declare_constant(UseC2CallingConventions)                               \
  declare_c1_constant(c1_MAX_SAVED_ON_ENTRY_REG_COUNT)                    \
  declare_c2_preprocessor_constant("REG_COUNT", REG_COUNT)                \
  declare_c2_preprocessor_constant("SAVED_ON_ENTRY_REG_COUNT", SAVED_ON_ENTRY_REG_COUNT) \
  declare_c2_preprocessor_constant("C_SAVED_ON_ENTRY_REG_COUNT", C_SAVED_ON_ENTRY_REG_COUNT)

  /* NOTE that we do not use the last_entry() macro here; it is used  */
  /* in vmStructs_<os>_<cpu>.hpp's VM_INT_CONSTANTS_OS_CPU macro (and */
  /* must be present there)                                           */


//--------------------------------------------------------------------------------
// Macros operating on the above lists
//--------------------------------------------------------------------------------

// This utility macro quotes the passed string
#define QUOTE(x) #x

//--------------------------------------------------------------------------------
// VMStructEntry macros
//

// This macro generates a VMStructEntry line for a nonstatic field
#define GENERATE_NONSTATIC_VM_STRUCT_ENTRY(typeName, fieldName, type)              \
 { QUOTE(typeName), QUOTE(fieldName), QUOTE(type), 0, offsetof(typeName, fieldName), NULL },

// This macro generates a VMStructEntry line for a static field
#define GENERATE_STATIC_VM_STRUCT_ENTRY(typeName, fieldName, type)                 \
 { QUOTE(typeName), QUOTE(fieldName), QUOTE(type), 1, 0, &typeName::fieldName },

// This macro generates a VMStructEntry line for an unchecked
// nonstatic field, in which the size of the type is also specified.
// The type string is given as NULL, indicating an "opaque" type.
#define GENERATE_UNCHECKED_NONSTATIC_VM_STRUCT_ENTRY(typeName, fieldName, size)    \
 { QUOTE(typeName), QUOTE(fieldName), NULL, 0, offsetof(typeName, fieldName), NULL },

// This macro generates a VMStructEntry line for an unchecked
// static field, in which the size of the type is also specified.
// The type string is given as NULL, indicating an "opaque" type.
#define GENERATE_UNCHECKED_STATIC_VM_STRUCT_ENTRY(typeName, fieldName, size)       \
 { QUOTE(typeName), QUOTE(fieldName), NULL, 1, 0, &typeName::fieldName },

// This macro generates the sentinel value indicating the end of the list
#define GENERATE_VM_STRUCT_LAST_ENTRY() \
 { NULL, NULL, NULL, 0, 0, NULL }

// This macro checks the type of a VMStructEntry by comparing pointer types
#define CHECK_NONSTATIC_VM_STRUCT_ENTRY(typeName, fieldName, type)                 \
 {typeName *dummyObj = NULL; type* dummy = &dummyObj->fieldName; }
 
// This macro checks the type of a volatile VMStructEntry by comparing pointer types
#define CHECK_VOLATILE_NONSTATIC_VM_STRUCT_ENTRY(typeName, fieldName, type)        \
 {typeName *dummyObj = NULL; volatile type* dummy = &dummyObj->fieldName; }
 
// This macro checks the type of a VMStructEntry by comparing pointer types
#define CHECK_STATIC_VM_STRUCT_ENTRY(typeName, fieldName, type)                    \
 {type* dummy = &typeName::fieldName; }
 
// This macro ensures the type of a field and its containing type are
// present in the type table. The assertion string is shorter than
// preferable because (incredibly) of a bug in Solstice NFS client
// which seems to prevent very long lines from compiling. This assertion
// means that an entry in VMStructs::localHotSpotVMStructs[] was not
// found in VMStructs::localHotSpotVMTypes[].
#define ENSURE_FIELD_TYPE_PRESENT(typeName, fieldName, type)                       \
 { assert(findType(QUOTE(typeName)) != 0, "type \"" QUOTE(typeName) "\" not found in type table"); \
   assert(findType(QUOTE(type)) != 0, "type \"" QUOTE(type) "\" not found in type table"); }

// This is a no-op macro for unchecked fields
#define CHECK_NO_OP(a, b, c)

// This is a no-op macro for the sentinel value
#define CHECK_SENTINEL()

//
// Build-specific macros:
//

// Generate and check a nonstatic field in non-product builds
#ifndef PRODUCT
# define GENERATE_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY(a, b, c) GENERATE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)    CHECK_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_NONPRODUCT_FIELD_TYPE_PRESENT(a, b, c)          ENSURE_FIELD_TYPE_PRESENT(a, b, c)
#  ifndef CORE
#   define GENERATE_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c) GENERATE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
#   define CHECK_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)    CHECK_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
#   define ENSURE_NONPRODUCT_NONCORE_FIELD_TYPE_PRESENT(a, b, c)          ENSURE_FIELD_TYPE_PRESENT(a, b, c)
#  else
#   define GENERATE_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
#   define CHECK_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
#   define ENSURE_NONPRODUCT_NONCORE_FIELD_TYPE_PRESENT(a, b, c)
#  endif
#else
# define GENERATE_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_NONPRODUCT_FIELD_TYPE_PRESENT(a, b, c)
# define GENERATE_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_NONPRODUCT_NONCORE_FIELD_TYPE_PRESENT(a, b, c)
#endif /* PRODUCT */

// Generate and check a nonstatic field in C2 builds
#ifdef COMPILER2
# define GENERATE_C2_NONSTATIC_VM_STRUCT_ENTRY(a, b, c) GENERATE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_C2_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)    CHECK_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_C2_FIELD_TYPE_PRESENT(a, b, c)          ENSURE_FIELD_TYPE_PRESENT(a, b, c)
#else
# define GENERATE_C2_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_C2_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_C2_FIELD_TYPE_PRESENT(a, b, c)
#endif /* COMPILER2 */

// Generate and check nonstatic and static fields in non-core builds
#ifndef CORE
# define GENERATE_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c) GENERATE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define GENERATE_NONCORE_STATIC_VM_STRUCT_ENTRY(a, b, c)    GENERATE_STATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)    CHECK_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONCORE_STATIC_VM_STRUCT_ENTRY(a, b, c)       CHECK_STATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_NONCORE_FIELD_TYPE_PRESENT(a, b, c)          ENSURE_FIELD_TYPE_PRESENT(a, b, c)
#else
# define GENERATE_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define GENERATE_NONCORE_STATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONCORE_NONSTATIC_VM_STRUCT_ENTRY(a, b, c)
# define CHECK_NONCORE_STATIC_VM_STRUCT_ENTRY(a, b, c)
# define ENSURE_NONCORE_FIELD_TYPE_PRESENT(a, b, c)
#endif /* CORE */


#ifdef COMPILER1
# define GENERATE_C1_UNCHECKED_STATIC_VM_STRUCT_ENTRY(a, b, c) GENERATE_UNCHECKED_STATIC_VM_STRUCT_ENTRY(a, b, c)
#else
# define GENERATE_C1_UNCHECKED_STATIC_VM_STRUCT_ENTRY(a, b, c)
#endif /* COMPILER1 */

//--------------------------------------------------------------------------------
// VMTypeEntry macros
//

#define GENERATE_VM_TYPE_ENTRY(type, superclass) \
 { QUOTE(type), QUOTE(superclass), 0, 0, sizeof(type) },

#define GENERATE_TOPLEVEL_VM_TYPE_ENTRY(type) \
 { QUOTE(type), NULL,              0, 0, sizeof(type) },

#define GENERATE_INTEGER_VM_TYPE_ENTRY(type) \
 { QUOTE(type), NULL,              1, 0, sizeof(type) },

#define GENERATE_UNSIGNED_INTEGER_VM_TYPE_ENTRY(type) \
 { QUOTE(type), NULL,              1, 1, sizeof(type) },

#define GENERATE_VM_TYPE_LAST_ENTRY() \
 { NULL, NULL, 0 }

#define CHECK_VM_TYPE_ENTRY(type, superclass) \
 { type* dummyObj = NULL; superclass* dummySuperObj = dummyObj; }

#define CHECK_VM_TYPE_NO_OP(a)
#define CHECK_SINGLE_ARG_VM_TYPE_NO_OP(a)

//
// Build-specific macros:
//

#ifndef CORE
# define GENERATE_NONCORE_VM_TYPE_ENTRY(a, b)                GENERATE_VM_TYPE_ENTRY(a, b)
# define GENERATE_NONCORE_TOPLEVEL_VM_TYPE_ENTRY(a)          GENERATE_TOPLEVEL_VM_TYPE_ENTRY(a)
# define GENERATE_NONCORE_INTEGER_VM_TYPE_ENTRY(a)           GENERATE_INTEGER_VM_TYPE_ENTRY(a)
# define GENERATE_NONCORE_UNSIGNED_INTEGER_VM_TYPE_ENTRY(a)  GENERATE_UNSIGNED_INTEGER_VM_TYPE_ENTRY(a)
# define CHECK_NONCORE_VM_TYPE_ENTRY(a, b)                   CHECK_VM_TYPE_ENTRY(a, b)
# define CHECK_NONCORE_TOPLEVEL_VM_TYPE_ENTRY(a)
# define CHECK_NONCORE_INTEGER_VM_TYPE_ENTRY(a)
# define CHECK_NONCORE_UNSIGNED_INTEGER_VM_TYPE_ENTRY(a)
#else
# define GENERATE_NONCORE_VM_TYPE_ENTRY(a, b)
# define GENERATE_NONCORE_TOPLEVEL_VM_TYPE_ENTRY(a)
# define GENERATE_NONCORE_INTEGER_VM_TYPE_ENTRY(a)
# define GENERATE_NONCORE_UNSIGNED_INTEGER_VM_TYPE_ENTRY(a)
# define CHECK_NONCORE_VM_TYPE_ENTRY(a, b)
# define CHECK_NONCORE_TOPLEVEL_VM_TYPE_ENTRY(a)
# define CHECK_NONCORE_UNSIGNED_INTEGER_VM_TYPE_ENTRY(a)
#endif /* CORE */

#ifdef COMPILER1
# define GENERATE_C1_TOPLEVEL_VM_TYPE_ENTRY(a)               GENERATE_TOPLEVEL_VM_TYPE_ENTRY(a)
# define CHECK_C1_TOPLEVEL_VM_TYPE_ENTRY(a)
#else
# define GENERATE_C1_TOPLEVEL_VM_TYPE_ENTRY(a)
# define CHECK_C1_TOPLEVEL_VM_TYPE_ENTRY(a)
#endif /* COMPILER1 */

#ifdef COMPILER2
# define GENERATE_C2_VM_TYPE_ENTRY(a, b)                     GENERATE_VM_TYPE_ENTRY(a, b)
# define CHECK_C2_VM_TYPE_ENTRY(a, b)                        CHECK_VM_TYPE_ENTRY(a, b)
#else
# define GENERATE_C2_VM_TYPE_ENTRY(a, b)
# define CHECK_C2_VM_TYPE_ENTRY(a, b)
#endif /* COMPILER2 */


//--------------------------------------------------------------------------------
// VMIntConstantEntry macros
//

#define GENERATE_VM_INT_CONSTANT_ENTRY(name) \
 { QUOTE(name), (int32_t) name },

#define GENERATE_PREPROCESSOR_VM_INT_CONSTANT_ENTRY(name, value) \
 { name, (int32_t) value },

// This macro generates the sentinel value indicating the end of the list
#define GENERATE_VM_INT_CONSTANT_LAST_ENTRY() \
 { NULL, 0 }

// Generate an int constant for a C1 build
#ifdef COMPILER1
# define GENERATE_C1_VM_INT_CONSTANT_ENTRY(name)  GENERATE_VM_INT_CONSTANT_ENTRY(name)
#else
# define GENERATE_C1_VM_INT_CONSTANT_ENTRY(name)
#endif /* COMPILER1 */

// Generate an int constant for a C2 build
#ifdef COMPILER2
# define GENERATE_C2_VM_INT_CONSTANT_ENTRY(name)                      GENERATE_VM_INT_CONSTANT_ENTRY(name)
# define GENERATE_C2_PREPROCESSOR_VM_INT_CONSTANT_ENTRY(name, value)  GENERATE_PREPROCESSOR_VM_INT_CONSTANT_ENTRY(name, value)
#else
# define GENERATE_C2_VM_INT_CONSTANT_ENTRY(name)
# define GENERATE_C2_PREPROCESSOR_VM_INT_CONSTANT_ENTRY(name, value)
#endif /* COMPILER1 */

// Generate an int constant for a non-core build
#ifndef CORE
# define GENERATE_NONCORE_VM_INT_CONSTANT_ENTRY(name)                      GENERATE_VM_INT_CONSTANT_ENTRY(name)
#else
# define GENERATE_NONCORE_VM_INT_CONSTANT_ENTRY(name)
#endif /*CORE*/

//
// Instantiation of VMStructEntries, VMTypeEntries and VMIntConstantEntries
// 

// These initializers are allowed to access private fields in classes
// as long as class VMStructs is a friend
VMStructEntry VMStructs::localHotSpotVMStructs[] = {
  VM_STRUCTS(GENERATE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_STATIC_VM_STRUCT_ENTRY, GENERATE_UNCHECKED_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_C2_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONCORE_STATIC_VM_STRUCT_ENTRY, GENERATE_C1_UNCHECKED_STATIC_VM_STRUCT_ENTRY, GENERATE_VM_STRUCT_LAST_ENTRY)
  VM_STRUCTS_OS_CPU(GENERATE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_STATIC_VM_STRUCT_ENTRY, GENERATE_UNCHECKED_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_C2_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, GENERATE_NONCORE_STATIC_VM_STRUCT_ENTRY, GENERATE_C1_UNCHECKED_STATIC_VM_STRUCT_ENTRY, GENERATE_VM_STRUCT_LAST_ENTRY)
};

VMTypeEntry VMStructs::localHotSpotVMTypes[] = {
  VM_TYPES(GENERATE_VM_TYPE_ENTRY, GENERATE_TOPLEVEL_VM_TYPE_ENTRY, GENERATE_INTEGER_VM_TYPE_ENTRY, GENERATE_UNSIGNED_INTEGER_VM_TYPE_ENTRY, GENERATE_NONCORE_VM_TYPE_ENTRY, GENERATE_NONCORE_TOPLEVEL_VM_TYPE_ENTRY, GENERATE_NONCORE_INTEGER_VM_TYPE_ENTRY, GENERATE_NONCORE_UNSIGNED_INTEGER_VM_TYPE_ENTRY, GENERATE_C1_TOPLEVEL_VM_TYPE_ENTRY, GENERATE_C2_VM_TYPE_ENTRY, GENERATE_VM_TYPE_LAST_ENTRY)
  VM_TYPES_OS_CPU(GENERATE_VM_TYPE_ENTRY, GENERATE_TOPLEVEL_VM_TYPE_ENTRY, GENERATE_INTEGER_VM_TYPE_ENTRY, GENERATE_UNSIGNED_INTEGER_VM_TYPE_ENTRY, GENERATE_NONCORE_VM_TYPE_ENTRY, GENERATE_NONCORE_TOPLEVEL_VM_TYPE_ENTRY, GENERATE_NONCORE_UNSIGNED_INTEGER_VM_TYPE_ENTRY, GENERATE_C1_TOPLEVEL_VM_TYPE_ENTRY, GENERATE_C2_VM_TYPE_ENTRY, GENERATE_VM_TYPE_LAST_ENTRY)
};

VMIntConstantEntry VMStructs::localHotSpotVMIntConstants[] = {
  VM_INT_CONSTANTS       (GENERATE_VM_INT_CONSTANT_ENTRY, GENERATE_PREPROCESSOR_VM_INT_CONSTANT_ENTRY, GENERATE_C1_VM_INT_CONSTANT_ENTRY, GENERATE_C2_VM_INT_CONSTANT_ENTRY, GENERATE_C2_PREPROCESSOR_VM_INT_CONSTANT_ENTRY, GENERATE_NONCORE_VM_INT_CONSTANT_ENTRY, GENERATE_VM_INT_CONSTANT_LAST_ENTRY)
  VM_INT_CONSTANTS_OS_CPU(GENERATE_VM_INT_CONSTANT_ENTRY, GENERATE_PREPROCESSOR_VM_INT_CONSTANT_ENTRY, GENERATE_C1_VM_INT_CONSTANT_ENTRY, GENERATE_C2_VM_INT_CONSTANT_ENTRY, GENERATE_C2_PREPROCESSOR_VM_INT_CONSTANT_ENTRY, GENERATE_VM_INT_CONSTANT_LAST_ENTRY)
};

// This is used both to check the types of referenced fields and, in
// debug builds, to ensure that all of the field types are present.
void
VMStructs::init() {
  VM_STRUCTS(CHECK_NONSTATIC_VM_STRUCT_ENTRY, CHECK_STATIC_VM_STRUCT_ENTRY, CHECK_NO_OP, CHECK_VOLATILE_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, CHECK_C2_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONCORE_STATIC_VM_STRUCT_ENTRY, CHECK_NO_OP, CHECK_SENTINEL);
  VM_STRUCTS_OS_CPU(CHECK_NONSTATIC_VM_STRUCT_ENTRY, CHECK_STATIC_VM_STRUCT_ENTRY, CHECK_NO_OP, CHECK_VOLATILE_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONPRODUCT_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONPRODUCT_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, CHECK_C2_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONCORE_NONSTATIC_VM_STRUCT_ENTRY, CHECK_NONCORE_STATIC_VM_STRUCT_ENTRY, CHECK_NO_OP, CHECK_SENTINEL);

  VM_TYPES(CHECK_VM_TYPE_ENTRY, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_NONCORE_VM_TYPE_ENTRY, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_C1_TOPLEVEL_VM_TYPE_ENTRY, CHECK_C2_VM_TYPE_ENTRY, CHECK_SENTINEL);
  VM_TYPES_OS_CPU(CHECK_VM_TYPE_ENTRY, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_NONCORE_VM_TYPE_ENTRY, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_SINGLE_ARG_VM_TYPE_NO_OP, CHECK_C1_TOPLEVEL_VM_TYPE_ENTRY, CHECK_C2_VM_TYPE_ENTRY, CHECK_SENTINEL);

  //
  // Split VM_STRUCTS() invocation into two parts to allow MS VC++ 6.0
  // to build with the source mounted over SNC3.2. Symptom was that
  // debug build failed with an internal compiler error. Has been seen
  // mounting sources from Solaris 2.6 and 2.7 hosts, but so far not
  // 2.8 hosts. Appears to occur because line is too long.
  //
  // If an assertion failure is triggered here it means that an entry
  // in VMStructs::localHotSpotVMStructs[] was not found in
  // VMStructs::localHotSpotVMTypes[]. (The assertion itself had to be
  // made less descriptive because of this above bug -- see the
  // definition of ENSURE_FIELD_TYPE_PRESENT.)
  //
  // NOTE: taken out because this was just not working on everyone's
  // Solstice NFS setup. If everyone switches to local workspaces on
  // Win32, we can put this back in.
#ifndef _WINDOWS
  debug_only(VM_STRUCTS(ENSURE_FIELD_TYPE_PRESENT, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_NO_OP, CHECK_SENTINEL));
  debug_only(VM_STRUCTS(CHECK_NO_OP, ENSURE_FIELD_TYPE_PRESENT, CHECK_NO_OP, ENSURE_FIELD_TYPE_PRESENT, ENSURE_NONPRODUCT_FIELD_TYPE_PRESENT, ENSURE_NONPRODUCT_NONCORE_FIELD_TYPE_PRESENT, ENSURE_C2_FIELD_TYPE_PRESENT, ENSURE_NONCORE_FIELD_TYPE_PRESENT, ENSURE_NONCORE_FIELD_TYPE_PRESENT, CHECK_NO_OP, CHECK_SENTINEL));
  debug_only(VM_STRUCTS_OS_CPU(ENSURE_FIELD_TYPE_PRESENT, ENSURE_FIELD_TYPE_PRESENT, CHECK_NO_OP, ENSURE_FIELD_TYPE_PRESENT, ENSURE_NONPRODUCT_FIELD_TYPE_PRESENT, ENSURE_NONPRODUCT_NONCORE_FIELD_TYPE_PRESENT, ENSURE_C2_FIELD_TYPE_PRESENT, ENSURE_NONCORE_FIELD_TYPE_PRESENT, ENSURE_NONCORE_FIELD_TYPE_PRESENT, CHECK_NO_OP, CHECK_SENTINEL));
#endif
}

#ifdef ASSERT
int
VMStructs::findType(const char* typeName) {
  VMTypeEntry* types = gHotSpotVMTypes;

  while (types->typeName != NULL) {
    if (!strcmp(typeName, types->typeName)) {
      return 1;
    }
    ++types;
  }
  return 0;
}
#endif

void vmStructs_init() {
  debug_only(VMStructs::init());
}

extern "C" VMStructEntry* gHotSpotVMStructs                 = VMStructs::localHotSpotVMStructs;
extern "C" uint64_t gHotSpotVMStructEntryTypeNameOffset     = offsetof(VMStructEntry, typeName);
extern "C" uint64_t gHotSpotVMStructEntryFieldNameOffset    = offsetof(VMStructEntry, fieldName);
extern "C" uint64_t gHotSpotVMStructEntryTypeStringOffset   = offsetof(VMStructEntry, typeString);
extern "C" uint64_t gHotSpotVMStructEntryIsStaticOffset     = offsetof(VMStructEntry, isStatic);
extern "C" uint64_t gHotSpotVMStructEntryOffsetOffset       = offsetof(VMStructEntry, offset);
extern "C" uint64_t gHotSpotVMStructEntryAddressOffset      = offsetof(VMStructEntry, address);
extern "C" uint64_t gHotSpotVMStructEntryArrayStride        = ((uint64_t) &gHotSpotVMStructs[1]) - ((uint64_t) &gHotSpotVMStructs[0]);
extern "C" VMTypeEntry*   gHotSpotVMTypes                   = VMStructs::localHotSpotVMTypes;
extern "C" uint64_t gHotSpotVMTypeEntryTypeNameOffset       = offsetof(VMTypeEntry, typeName);
extern "C" uint64_t gHotSpotVMTypeEntrySuperclassNameOffset = offsetof(VMTypeEntry, superclassName);
extern "C" uint64_t gHotSpotVMTypeEntryIsIntegerTypeOffset  = offsetof(VMTypeEntry, isIntegerType);
extern "C" uint64_t gHotSpotVMTypeEntryIsUnsignedOffset     = offsetof(VMTypeEntry, isUnsigned);
extern "C" uint64_t gHotSpotVMTypeEntrySizeOffset           = offsetof(VMTypeEntry, size);
extern "C" uint64_t gHotSpotVMTypeEntryArrayStride          = ((uint64_t) &gHotSpotVMTypes[1]) - ((uint64_t) &gHotSpotVMTypes[0]);
extern "C" VMIntConstantEntry* gHotSpotVMIntConstants       = VMStructs::localHotSpotVMIntConstants;
extern "C" uint64_t gHotSpotVMIntConstantEntryNameOffset    = offsetof(VMIntConstantEntry, name);
extern "C" uint64_t gHotSpotVMIntConstantEntryValueOffset   = offsetof(VMIntConstantEntry, value);
extern "C" uint64_t gHotSpotVMIntConstantEntryArrayStride   = ((uint64_t) &gHotSpotVMIntConstants[1]) - ((uint64_t) &gHotSpotVMIntConstants[0]);
