/*
 * @(#)vmThread.hpp	1.18 00/05/15 20:06:27
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2


//
// Prioritized queue of VM operations.
//
// Encapsulates both queue management and
// and priority policy
//
class VMOperationQueue : public CHeapObj {
 private:
  enum Priorities {
     SafepointPriority,     // Highest priority (operation executed at a safepoint)
     MediumPriority,        // Medium priority
     CompilationPriority,   // Concurrent compilations have lowest priority
     nof_priorities 
  };

  // We maintain a doubled linked list, with explicit count.
  int           _queue_length[nof_priorities];
  VM_Operation* _queue       [nof_priorities];  
  
  // Double-linked non-empty list insert.
  void insert(VM_Operation* q,VM_Operation* n);
  void unlink(VM_Operation* q);

  // Basic queue manipulation
  bool queue_empty                (int prio);
  void queue_add_front            (int prio, VM_Operation *op);
  void queue_add_back             (int prio, VM_Operation *op);
  VM_Operation* queue_remove_front(int prio);  
  void queue_oops_do(int queue, void f(oop *));
    
 public:
  VMOperationQueue();

  // Highlevel operations. Encapsulates policy
  bool add(VM_Operation *op);
  VM_Operation* remove_next();                        // Returns next or null
  VM_Operation* remove_next_at_safepoint_priority()   { return queue_remove_front(SafepointPriority); }

  // GC support
  void oops_do(void f(oop *));

  void verify_queue(int prio) PRODUCT_RETURN;
};


//
// A single VMThread (the primordial thread) spawns all other threads
// and is itself used by other threads to offload heavy vm operations
// like scavenge, garbage_collect etc.
//

class VMThread: public Thread {
 private:
  static ThreadPriority _current_priority;

  static bool _should_terminate;
  static bool _terminated;

  static bool should_terminate()                 { return _should_terminate; }

  void evaluate_operation(VM_Operation* op);
 public:
  // Constructor
  VMThread();

  // Tester
  bool is_VM_thread() const                      { return true; }
  
  // The ever running loop for the VMThread
  void loop();

  // Called to stop the VM thread
  static void wait_for_vm_thread_exit();
  static bool is_terminated()                     { return _terminated == true; }

  // Execution of vm operation
  static void execute(VM_Operation* op);

  // Returns the current vm operation if any.
  static VM_Operation* vm_operation()             { return _cur_vm_operation;   }  

  // Returns the single instance of VMThread.
  static VMThread* vm_thread()                    { return _vm_thread; }

  // GC support
  void oops_do(void f(oop*));

  // Debugging
  void print();
  void verify();

  // Entry for starting vm thread
  virtual void run();

  // Creations
  static void create();
 private:   
  // VM_Operation support  
  static VM_Operation*     _cur_vm_operation;   // Current VM operation
  static VMOperationQueue* _vm_queue;           // Queue (w/ policy) of VM operations
  
  // Pointer to single-instance of VM thread
  static VMThread*     _vm_thread;
};


