/*
 * @(#)bitMap.cpp	1.12 99/11/17 15:26:48
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_bitMap.cpp.incl"


BitMap::BitMap(unsigned int* map, int size_in_bits) {
  assert(size_in_bits >= 0, "just checking");
  _map = map;
  _size = size_in_bits;
}


BitMap::BitMap(int size_in_bits) {
  assert(size_in_bits >= 0, "just checking");
  _map = NULL;
  _size = size_in_bits;
  int nof_words = size_in_words();
  _map = NEW_RESOURCE_ARRAY(unsigned int, nof_words);
}


void BitMap::at_put(int offset, bool value) {
  assert(offset>=0 && offset < size(), "BitMap index out of bounds");
  int* w = (int*)word_for(offset);
  int pos = offset%BitsPerWord;
  if (value) {
    set_nth_bit(*w, pos);
  } else {
    clear_nth_bit(*w, pos);
  }  
}


bool BitMap::set_union(BitMap other) {
  assert(size() == other.size(), "must have same size");
  bool changed = false;
  for (int index = 0; index < size_in_words(); index++) {
    unsigned int temp = map()[index] | other.map()[index];
    changed = changed || (temp != map()[index]);
    map()[index] = temp;
  }
  return changed;
}


bool BitMap::set_difference(BitMap other) {
  assert(size() == other.size(), "must have same size");
  bool changed = false;
  for (int index = 0; index < size_in_words(); index++) {
    unsigned int temp = map()[index] & ~(other.map()[index]);
    changed = changed || (temp != map()[index]);
    map()[index] = temp;
  }
  return changed;
}


void BitMap::set_intersection(BitMap other) {
  assert(size() == other.size(), "must have same size");
  for (int index = 0; index < size_in_words(); index++) {
    map()[index]  = map()[index] & (other.map()[index]);
  }
}


void BitMap::set_from(BitMap other) {
  assert(size() == other.size(), "must have same size");
  for (int index = 0; index < size_in_words(); index++) {
    map()[index] = (other.map()[index]);
  }
}


bool BitMap::is_same(BitMap other) {
  assert(size() == other.size(), "must have same size");
  for (int index = 0; index < size_in_words(); index++) {
    if (map()[index] != (other.map()[index])) return false;
  }
  return true;
}


void BitMap::clear() {
  for (int index = 0; index < size_in_words(); index++) {
    map()[index] = NoBits;
  }
}


void BitMap::iterate(BitMapClosure* blk) {
  for (int index = 0; index < size_in_words(); index++) {
    unsigned int rest = map()[index];
    for (int offset = index * BitsPerWord; rest != NoBits; offset++) {
      if (rest % 2 == 1) {
        if (offset < size()) {
          blk->do_bit(offset);
        } else {
          return; // Passed end of map
        }
      }
      rest = rest >> 1;
    }
  }
}


#ifndef PRODUCT

void BitMap::print_on(outputStream* st) const {
  tty->print("Bitmap(%d):", size());
  for (int index = 0; index < size(); index++) {
    tty->print("%c", at(index) ? '1' : '0');
  }
  tty->cr();
}

#endif


