/*
 * @(#)bitMap.hpp	1.12 99/06/28 11:15:40
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Closure for iterating over BitMaps

class BitMapClosure VALUE_OBJ_CLASS_SPEC {
 public:
  // Callback when bit in map is set
  virtual void do_bit(int offset) = 0;
};


// Operations for bitmaps represented as arrays of unsigned 32-bit ints
//
// Bit offsets are numbered from 0 to size-1

class BitMap VALUE_OBJ_CLASS_SPEC {
 private:
  unsigned int* _map;     // First word in bitmap
  int           _size;    // Size of bitmap (in bits)

  unsigned int* map() const                { return _map; }
  unsigned int* word_for(int offset) const { return map() + (offset/BitsPerWord); }

 public:
  // Construction
  BitMap(unsigned int* map, int size_in_bits);

  // Allocates necessary data structure in resource area
  BitMap(int size_in_bits);

  void set_map(unsigned int* map) { _map = map; }

  // Accessing
  int size() const                    { return _size; }
  int size_in_words() const           { return (size() + BitsPerWord - 1) / BitsPerWord; }

  bool at(int offset) const {
    assert(offset>=0 && offset < size(), "BitMap index out of bounds");
    return is_set_nth_bit(*word_for(offset), offset%BitsPerWord); 
  }

  void at_put(int offset, bool value);

  // Clearing
  void clear();
  
  // Closure iteration
  void iterate(BitMapClosure* blk);

  // Set union
  bool set_union(BitMap bits);

  // Set difference
  bool set_difference(BitMap bits);

  // Set intersection
  void set_intersection(BitMap bits);

  void set_from(BitMap bits);

  bool is_same(BitMap bits);

#ifndef PRODUCT
 public:
  // Printing
  void print_on(outputStream* st) const;
#endif
};
