/*
 * @(#)globalDefinitions.cpp	1.19 00/03/21 16:18:17
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_globalDefinitions.cpp.incl"


// Basic error support

void basic_fatal(const char* msg) {
  fatal(msg);
}


// Something to help porters sleep at night

void check_basic_types() {
#ifdef ASSERT
  assert( 4 == sizeof( jobject),   "wrong size for basic type");
  assert( 1 == sizeof( jbyte),     "wrong size for basic type");
  assert( 2 == sizeof( jchar),     "wrong size for basic type");
  assert( 2 == sizeof( jshort),    "wrong size for basic type");
  assert( 4 == sizeof( jint),      "wrong size for basic type");
  assert( 1 == sizeof( jboolean),  "wrong size for basic type");
  assert( 8 == sizeof( jlong),     "wrong size for basic type");
  assert( 4 == sizeof( jfloat),    "wrong size for basic type");
  assert( 8 == sizeof( jdouble),   "wrong size for basic type");
  assert( 1 == sizeof( u1),        "wrong size for basic type");
  assert( 2 == sizeof( u2),        "wrong size for basic type");
  assert( 4 == sizeof( u4),        "wrong size for basic type");
#endif
}


// Convert a jchar from a classfile signature to a BasicType
enum BasicType char2type(char c) {
  switch( c ) {
  case 'B': return T_BYTE;    
  case 'C': return T_CHAR;    
  case 'D': return T_DOUBLE;  
  case 'F': return T_FLOAT;   
  case 'I': return T_INT;     
  case 'J': return T_LONG;    
  case 'S': return T_SHORT;   
  case 'Z': return T_BOOLEAN; 
  case 'V': return T_VOID;    
  case 'L': return T_OBJECT;  
  case '[': return T_ARRAY;   
  }
  ShouldNotReachHere();
  return T_ILLEGAL;
}


// Map BasicType to size in words
int type2size[17]={ -1, 0, 0, 0, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 0, 1, -1};

int type2aelembytes[T_CONFLICT+1] = {
  0,                      // 0
  0,                      // 1
  0,                      // 2
  0,                      // 3
  T_BOOLEAN_aelem_bytes,  // T_BOOLEAN  =  4,
  T_CHAR_aelem_bytes,     // T_CHAR     =  5,
  T_FLOAT_aelem_bytes,    // T_FLOAT    =  6,
  T_DOUBLE_aelem_bytes,   // T_DOUBLE   =  7,
  T_BYTE_aelem_bytes,     // T_BYTE     =  8,
  T_SHORT_aelem_bytes,    // T_SHORT    =  9,
  T_INT_aelem_bytes,      // T_INT      = 10,
  T_LONG_aelem_bytes,     // T_LONG     = 11,
  T_OBJECT_aelem_bytes,   // T_OBJECT   = 12,
  T_ARRAY_aelem_bytes,    // T_ARRAY    = 13,
  0,                      // T_VOID     = 14,
  T_INT_aelem_bytes,      // T_ADDRESS  = 15,
  0                       // T_CONFLICT = 16,
};


// Support for 64-bit integer arithmetic

// The following code is mostly taken from JVM typedefs_md.h and system_md.c


static const jlong  high_bit  = (jlong)1 << (jlong)63;
static const jlong other_bits = ~high_bit;

jlong float2long(jfloat f) {
  jlong tmp = (jlong) f;
  if (tmp != high_bit) {
    return tmp;
  } else {
    if (g_isnan((jdouble)f)) {
      return 0;
    }
    if (f < 0) {
      return high_bit;
    } else {
      return other_bits;
    }
  }
}


jlong double2long(jdouble f) {
  jlong tmp = (jlong) f;
  if (tmp != high_bit) {
    return tmp;
  } else {
    if (g_isnan(f)) {
      return 0;
    }
    if (f < 0) {
      return high_bit;
    } else {
      return other_bits;
    }
  }
}

