/*
 * @(#)globalDefinitions.hpp	1.133 00/09/29 16:48:52
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// This file holds all globally used constants & types, class (forward)
// declarations and a few frequently used utility functions.

#ifdef SPARC
#include <strings.h>
#endif


//----------------------------------------------------------------------------------------------------
// Constants

const int LogBytesPerShort   = 1;
const int LogBytesPerWord    = 2;
const int LogBytesPerLong    = 3;

const int BytesPerShort      = 1 << LogBytesPerShort;
const int BytesPerWord       = 1 << LogBytesPerWord;
const int BytesPerLong       = 1 << LogBytesPerLong;

const int LogBitsPerByte     = 3;
const int LogBitsPerShort    = LogBitsPerByte + LogBytesPerShort;
const int LogBitsPerWord     = LogBitsPerByte + LogBytesPerWord;
const int LogBitsPerLong     = LogBitsPerByte + LogBytesPerLong;

const int BitsPerByte        = 1 << LogBitsPerByte;
const int BitsPerShort       = 1 << LogBitsPerShort;
const int BitsPerWord        = 1 << LogBitsPerWord;
const int BitsPerLong        = 1 << LogBitsPerLong;

const int WordAlignmentMask  = (1 << LogBytesPerWord) - 1;
const int LongAlignmentMask  = (1 << LogBytesPerLong) - 1;

const int WordsPerLong       = 2;

const int oopSize            = sizeof(char*);
const int wordSize           = sizeof(jint );
const int longSize           = sizeof(jlong);

const int BitsPerJavaInteger = 32;

const int K                  = 1024;
const int M                  = K*K;

const jint min_jint          = 0x80000000;    // smallest jint
const jint max_jint          = 0x7FFFFFFF;    // largest jint

//----------------------------------------------------------------------------------------------------
// VM type definitions

NEEDS_CLEANUP
typedef unsigned int uint;


//----------------------------------------------------------------------------------------------------
// Java type definitions

#ifndef NULL
#define NULL 0
#endif

// All kinds of 'plain' byte addresses
typedef   signed char s_char;
typedef unsigned char u_char;
typedef u_char*       address;
typedef uintptr_t     address_word; // unsigned integer which will hold a pointer
				    // except for some implementations of a C++
				    // linkage pointer to function. Should never
				    // need one of those to be placed in this
				    // type anyway.

//  Utility functions to "portably" (?) bit twiddle pointers
//  Where portable means keep ANSI C++ compilers quiet

inline address       set_address_bits(address x, int m)       { return address(intptr_t(x) | m); }
inline address       clear_address_bits(address x, int m)     { return address(intptr_t(x) & ~m); }

//  Utility functions to "portably" make cast to/from function pointers.

inline address_word  mask_address_bits(address x, int m)      { return address_word(x) & m; }
inline address_word  castable_address(address x)              { return address_word(x) ; }
inline address_word  castable_address(void* x)                { return address_word(x) ; }
//
// ANSI C++ does not allow casting from one pointer type to a function pointer
// directly without at best a warning. This macro accomplishes it silently
// In every case that is present at this point the value be cast is a pointer
// to a C linkage function. In somecase the type used for the cast reflects
// that linkage and a picky compiler would not complain. In other cases because
// there is no convenient place to place a typedef with extern C linkage (i.e
// a platform dependent header file) it doesn't. At this point no compiler seems
// picky enough to catch these instances (which are few). It is possible that
// using templates could fix these for all cases. This use of templates is likely
// so far from the middle of the road that it is likely to be problematic in
// many C++ compilers.
//
#define CAST_TO_FN_PTR(func_type, value) ((func_type)(castable_address(value)))
#define CAST_FROM_FN_PTR(new_type, func_ptr) ((new_type)((address_word) func_ptr))

// Unsigned byte types for os and stream.hpp

// Unsigned one, two, four and eigth byte quantities used for describing
// the .class file format. See JVM book chapter 4.

typedef jubyte  u1;
typedef jushort u2;
typedef juint   u4;
typedef julong  u8;


// Booleans
#define bool    jint

#define true    v_true
#define false   v_false

const bool true        = 1;
const bool false       = 0;

const jboolean g_true  = 1;
const jboolean g_false = 0;


//----------------------------------------------------------------------------------------------------
// JVM spec restrictions

const int max_method_code_size = 64*K - 1;  // JVM spec, 2nd ed. section 4.8.1 (p.134)


//----------------------------------------------------------------------------------------------------
// Object alignment
//
// Determines whether all objects should be aligned on double word boundaries.

#define ALIGN_ALL_OBJECTS 1

// i486:  Can be set to 0, which will save some space (approximated to less 
//        than 2%) but cause double/long array intensive code to run slower.
// Sparc: Must be set to true for double/long access to work correctly.

// If ALIGN_ALL_OBJECTS is true, then any of ALIGN_{LONG,DOUBLE}_{FIELDS,ELEMENTS}
// may also be true.  Those four flags control the allocation of two-word instance
// fields and array elements.
// The ALIGN_XXX macros must take values of either 1 or 0,
// because they are sometimes used in "#if" preprocessor
// directives.  (The value "true" looks false to the cpp.)
// See Universe::element_type_should_be_aligned.

  // Machine dependent stuff
 #include "incls/_globalDefinitions_pd.hpp.incl"


// the alignment to be used by Arena::Amalloc.  See bugid 4169348.
// Note: this value must be a power of 2
#ifndef ARENA_AMALLOC_ALIGNMENT
#define ARENA_AMALLOC_ALIGNMENT 4
#endif

inline size_t align_size_up(size_t size, size_t alignment) { 
  return (size+(alignment-1)) & ~(alignment-1); 
}

inline size_t align_size_down(size_t size, size_t alignment) { 
  return (size) & ~(alignment-1); 
}

// If the target machine wants, align objects (i.e., base pointers).
inline int align_object_size(int size) { 
  return ALIGN_ALL_OBJECTS ? align_size_up(size, 2) : size;
}

// If the target machine aligns longs, then pad out certain offsets too.
inline int align_object_offset(int offset) {
  return ((ALIGN_DOUBLE_FIELDS|ALIGN_LONG_FIELDS)
	  && ALIGN_ALL_OBJECTS) ? align_size_up(offset, 2) : offset;
}


//----------------------------------------------------------------------------------------------------
// Utility macros for compilers
// used to silence compiler warnings

#define Unused_Variable(var) var


//----------------------------------------------------------------------------------------------------
// Miscellaneous

inline jint low (jlong value)                    { return jint(value); }
inline jint high(jlong value)                    { return jint(value >> 32); }

// the fancy casts are a hopefully portable way 
// to do unsigned 32 to 64 bit type conversion
inline void set_low (jlong* value, jint low )    { *value &= (jlong)0xffffffff << 32;
                                                   *value |= (jlong)(julong)(juint)low; }

inline void set_high(jlong* value, jint high)    { *value &= (jlong)(julong)(juint)0xffffffff;
                                                   *value |= (jlong)high       << 32; }

inline jlong jlong_from(jint h, jint l) {
  jlong result = 0; // initialization to avoid warning
  set_high(&result, h);
  set_low(&result,  l);
  return result;
}

void check_basic_types(); // cannot define here; uses assert


enum BasicType {
  T_BOOLEAN  =  4,
  T_CHAR     =  5,
  T_FLOAT    =  6,
  T_DOUBLE   =  7,
  T_BYTE     =  8,
  T_SHORT    =  9,
  T_INT      = 10,
  T_LONG     = 11,
  T_OBJECT   = 12,
  T_ARRAY    = 13,
  T_VOID     = 14,
  T_ADDRESS  = 15,
  T_CONFLICT = 16, // for stack value type with conflicting contents
  T_ILLEGAL  = 0xcafebabe
};
enum BasicType char2type(char c);  // Map a jchar to a BasicType
extern int type2size[17];	    // Map BasicType to result stack elements


enum BasicTypeSize {
  T_BOOLEAN_size = 1,
  T_CHAR_size    = 1,
  T_FLOAT_size   = 1,
  T_DOUBLE_size  = 2,
  T_BYTE_size    = 1,
  T_SHORT_size   = 1,
  T_INT_size     = 1,
  T_LONG_size    = 2,
  T_OBJECT_size  = 1,
  T_ARRAY_size   = 1,
  T_VOID_size    = 0
};

// size in bytes
enum ArrayElementSize {
  T_BOOLEAN_aelem_bytes = 1,
  T_CHAR_aelem_bytes    = 2,
  T_FLOAT_aelem_bytes   = 4,
  T_DOUBLE_aelem_bytes  = 8,
  T_BYTE_aelem_bytes    = 1,
  T_SHORT_aelem_bytes   = 2,
  T_INT_aelem_bytes     = 4,
  T_LONG_aelem_bytes    = 8,
  T_OBJECT_aelem_bytes  = 4,
  T_ARRAY_aelem_bytes   = 4,
  T_VOID_aelem_bytes    = 0
};

extern int type2aelembytes[T_CONFLICT+1]; // maps a BasicType to nof bytes used by its array element

// JavaValue serves as a container for arbitrary Java values.

class JavaValue {
 public:
  BasicType _type;
  union {
    jboolean b;
    jbyte    x;
    jchar    c;
    jfloat   f;
    jdouble  d;
    jshort   s;
    jint     i;
    jlong    l;
    jobject  h;
  } _value;
  JavaValue(BasicType t = T_ILLEGAL) { _type = t; }

  JavaValue(jfloat value) {
    _type    = T_FLOAT;
    _value.f = value;
  }

  JavaValue(jdouble value) {
    _type    = T_DOUBLE;
    _value.d = value;
  }
};


// TosState describes the top-of-stack state before and after the execution of
// a bytecode or method. The top-of-stack value may be cached in one or more CPU
// registers. The TosState corresponds to the 'machine represention' of this cached
// value. There's 4 states corresponding to the JAVA types int, long, float & double
// as well as a 5th state in case the top-of-stack value is actually on the top
// of stack (in memory) and thus not cached. The atos state corresponds to the itos
// state when it comes to machine representation but is used separately for (oop)
// type specific operations (e.g. verification code).

enum TosState {         // describes the tos cache contents
  itos = 0,             // int tos cached
  ltos = 1,             // long tos cached
  ftos = 2,             // float tos cached
  dtos = 3,             // double tos cached
  vtos = 4,             // tos not cached

  number_of_states,

  atos,                 // pointer tos cached (treated as itos)
  ilgl                  // illegal state: should not occur
};


// Helper function to convert BasicType info into TosState
inline TosState as_TosState(BasicType type) {
  switch (type) {
    case T_BYTE   : // fall through
    case T_BOOLEAN: // fall through
    case T_CHAR   : // fall through
    case T_SHORT  : // fall through
    case T_INT    : return itos;
    case T_LONG   : return ltos;
    case T_FLOAT  : return ftos;
    case T_DOUBLE : return dtos;
    case T_VOID   : return vtos;
    case T_ARRAY  : // fall through
    case T_OBJECT : return atos;
    default       : return ilgl;
  }
}


// ReferenceType is used to distinguish between java/lang/ref/Reference subclasses

enum ReferenceType {
 REF_NONE,      // Regular class
 REF_OTHER,     // Subclass of java/lang/ref/Reference, but not subclass of one of the classes below
 REF_SOFT,      // Subclass of java/lang/ref/SoftReference
 REF_WEAK,      // Subclass of java/lang/ref/WeakReference
 REF_FINAL,     // Subclass of java/lang/ref/FinalReference
 REF_PHANTOM    // Subclass of java/lang/ref/PhantomReference
};


// JavaThreadState keeps track of which part of the code a thread is executing in. This
// information is needed by the safepoint code.
//
// There are 4 essential states:
//
//  _thread_new         : Just started, but not executed init. code yet (most likely still in OS init code)
//  _thread_in_native   : In native code. This is a safepoint region, since all oops will be in jobject handles
//  _thread_in_vm       : Executing in the vm
//  _thread_in_Java     : Executing either interpreted or compiled Java code (or could be in a stub)
//
// Each state has an associated xxxx_trans state, which is an intermediate state used when a thread is in
// a transition from one state to another. These extra states makes it possible for the safepoint code to
// handle certain thread_states without having to suspend the thread - making the safepoint code faster.
//
// Given a state, the xxx_trans state can always be found by adding 1.
//
enum JavaThreadState {
  _thread_uninitialized     =  0, // should never happen (missing initialization)
  _thread_new               =  2, // just starting up, i.e., in process of being initialized
  _thread_new_trans         =  3, // corresponding transition state (not used, included for completness)
  _thread_in_native         =  4, // running in native code
  _thread_in_native_trans   =  5, // corresponding transition state
  _thread_in_vm             =  6, // running in VM
  _thread_in_vm_trans       =  7, // corresponding transition state
  _thread_in_Java           =  8, // running in Java or in stub code
  _thread_in_Java_trans     =  9, // corresponding transition state (not used, included for completness)
  _thread_blocked           = 10, // blocked in vm
  _thread_blocked_trans     = 11, // corresponding transition state  
  _thread_in_comp_leaf      = 12, // running in a leaf runtine called from compiled code
  _thread_in_native_blocked = 50  // Special state needed, since we cannot suspend a thread when it is in native_trans
};


// Handy constants for deciding which compiler mode to use.
enum MethodCompilation {
  InvocationEntryBci = -1,     // i.e., not a on-stack replacement compilation
  InvalidOSREntryBci = -2
};


// MarkSweepInvokeCause described why a full gc was invoked
enum MarkSweepInvokeCause {
  _java_lang_system_gc,
  _tenured_generation_full,
  _permanent_generation_full,
  _train_generation_full,
  _old_generation_expanded_on_last_scavenge,
  _old_generation_too_full_to_scavenge,
  _full_gc_alot
};


//----------------------------------------------------------------------------------------------------
// 'Forward' declarations of frequently used classes
// (in order to reduce interface dependencies & reduce
// number of unnecessary compilations after changes)

class symbolTable;
class ClassFileStream;

class Event;

class Thread;
class  VMThread;
class  JavaThread;
class Threads;

class VM_Operation;
class VMOperationQueue;

class CodeBlob;
class  nmethod;
class  OSRAdapter;
class  I2CAdapter;
class  C2IAdapter;
class CompiledIC;
class relocInfo;
class ScopeDesc;
class PcDesc;

class Recompiler;
class Recompilee;
class RecompilationPolicy;
class RFrame;
class  CompiledRFrame;
class  InterpretedRFrame;

class frame;

class vframe;
class   javaVFrame;
class     interpretedVFrame;
class     compiledVFrame;
class     deoptimizedVFrame;
class   externalVFrame;
class     entryVFrame;

class RegisterMap;

class Mutex;
class Monitor;
class BasicLock;
class BasicObjectLock;

class PeriodicTask;

class JavaCallWrapper;

class   oopDesc;
typedef class oopDesc* oop;

class NativeCall;

class zone;

class StubQueue;

class outputStream;

class ResourceArea;

class DebugInformationRecorder;
class ScopeValue;
class CompressedStream;
class   DebugInfoReadStream;
class   DebugInfoWriteStream;
class LocationValue;
class ConstantValue;
class IllegalValue;

class PrivilegedElement;
class MonitorArray;

class MonitorInfo;

class OffsetClosure;
class OopMapCache;
class OopMapCacheEntry;
class OSThread;

typedef int (*OSThreadStartFunc)(void*);

class Space;
class CarSpace;
class Train;

class JavaValue;
class methodHandle;
class JavaCallArguments;

// Basic support for errors (general debug facilities not defined at this point fo the include phase)

extern void basic_fatal(const char* msg);


//----------------------------------------------------------------------------------------------------
// Special constants for debugging

const jint    badInt           = -3;              // generic "bad int" value
const int     badAddressVal    = -2;              // generic "bad address" value
const int     badOopVal        = -1;              // generic "bad oop" value
const int     badHeapOopVal    = 0xBAADBABE;      // value used to zap heap after GC
const int     badHandleValue   = 0xBC;            // value used to zap vm handle area
const int     badResourceValue = 0xAB;            // value used to zap resource area
const int     badJNIHandleVal  = 0xFEFEFEFE;      // value used to zap jni handle area

// (These must be implemented as #defines because C++ compilers are
// not obligated to inline non-integral constants!)
#define       badAddress        ((address)::badAddressVal)
#define       badOop            ((oop)::badOopVal)
#define       badHeapOop        ((oop)::badHeapOopVal)
#define       badJNIHandle      ((oop)::badJNIHandleVal)


//----------------------------------------------------------------------------------------------------
// Utility functions for bitfield manipulations

const int AllBits = ~0;   // all bits set in a word
const int NoBits  =  0;   // no bits set in a word
const int OneBit  =  1;   // only right_most bit set in a word

// get a word with the n.th or the right-most n bits set
// (note: #define used only so that they can be used in enum constant definitions)
#define nth_bit(n)        (n >= BitsPerWord ? 0 : OneBit << (n))
#define right_n_bits(n)   (nth_bit(n) - 1)

// bit-operations using a mask m
inline void   set_bits    (int& x, int m) { x |= m; }
inline void clear_bits    (int& x, int m) { x &= ~m; }
inline int   mask_bits    (int  x, int m) { return x & m; }

// bit-operations using the n.th bit
inline void    set_nth_bit(int& x, int n) { set_bits  (x, nth_bit(n)); }
inline void  clear_nth_bit(int& x, int n) { clear_bits(x, nth_bit(n)); }
inline bool is_set_nth_bit(int  x, int n) { return mask_bits (x, nth_bit(n)) != NoBits; };

// returns the bitfield of x starting at start_bit_no with length field_length (no sign-extension!)
inline int bitfield(int x, int start_bit_no, int field_length) {
  return mask_bits(x >> start_bit_no, right_n_bits(field_length));
}


//----------------------------------------------------------------------------------------------------
// Utility functions for integers

// min/max
#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif

#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif


// true if x is a power of 2, false otherwise
inline bool is_power_of_2(int x) {
  return x != NoBits && mask_bits(x, x - 1) == NoBits;
}


//* largest i such that 2^i <= x
//  A negative value of 'x' will return '31'
inline int log2(int x) {
  int i = -1;
  unsigned int p =  1;
  while (p != 0 && p <= (unsigned int)x) {
    // p = 2^(i+1) && p <= x (i.e., 2^(i+1) <= x)
    i++; p *= 2;
  }
  // p = 2^(i+1) && x < p (i.e., 2^i <= x < 2^(i+1))
  // (if p = 0 then overflow occured and i = 31)
  return i;
}

//* largest i such that 2^i <= x
//  A negative value of 'x' will return '63'
inline int log2_long(jlong x) {
  int i = -1;
  julong p =  1;
  while (p != 0 && p <= (julong)x) {
    // p = 2^(i+1) && p <= x (i.e., 2^(i+1) <= x)
    i++; p *= 2;
  }
  // p = 2^(i+1) && x < p (i.e., 2^i <= x < 2^(i+1))
  // (if p = 0 then overflow occured and i = 31)
  return i;
}

//* the argument must be exactly a power of 2
inline int exact_log2(int x) {
  #ifdef ASSERT
    if (!is_power_of_2(x)) basic_fatal("x must be a power of 2");
  #endif
  return log2(x);
}


// returns integer round-up to the nearest multiple of s (s must be a power of two)
inline int round_to(int x, int s) {
  #ifdef ASSERT
    if (!is_power_of_2(s)) basic_fatal("s must be a power of 2");
  #endif
  const int m = s - 1;
  return mask_bits(x + m, ~m);
}


inline bool is_odd (int x) { return x & 1;      }
inline bool is_even(int x) { return !is_odd(x); }


inline int byte_size(void* from, void* to) {
  return (address)to - (address)from;
}

//----------------------------------------------------------------------------------------------------
// Avoid non-portable casts with these routines (DEPRECATED)

// NOTE: USE Bytes class INSTEAD WHERE POSSIBLE
//       Bytes is optimized machine-specifically and may be much faster then the portable routines below.

// Given sequence of four bytes, build into a 32-bit word
// following the conventions used in class files.
// On the 386, this could be realized with a simple address cast.
//

// This routine takes eight bytes:
inline u8 build_u8_from( u1 c1, u1 c2, u1 c3, u1 c4, u1 c5, u1 c6, u1 c7, u1 c8 ) {
  return  ( u8(c1) << 56 )  &  ( u8(0xff) << 56 )
       |  ( u8(c2) << 48 )  &  ( u8(0xff) << 48 )
       |  ( u8(c3) << 40 )  &  ( u8(0xff) << 40 )
       |  ( u8(c4) << 32 )  &  ( u8(0xff) << 32 )
       |  ( u8(c5) << 24 )  &  ( u8(0xff) << 24 )
       |  ( u8(c6) << 16 )  &  ( u8(0xff) << 16 )
       |  ( u8(c7) <<  8 )  &  ( u8(0xff) <<  8 )
       |  ( u8(c8) <<  0 )  &  ( u8(0xff) <<  0 );
}

// This routine takes four bytes:
inline u4 build_u4_from( u1 c1, u1 c2, u1 c3, u1 c4 ) {
  return  ( u4(c1) << 24 )  &  0xff000000
       |  ( u4(c2) << 16 )  &  0x00ff0000
       |  ( u4(c3) <<  8 )  &  0x0000ff00
       |  ( u4(c4) <<  0 )  &  0x000000ff;
}

// And this one works if the four bytes are contiguous in memory:
inline u4 build_u4_from( u1* p ) {
  return  build_u4_from( p[0], p[1], p[2], p[3] );
}

// Ditto for two-byte ints:
inline u2 build_u2_from( u1 c1, u1 c2 ) {
  return  u2(( u2(c1) <<  8 )  &  0xff00
          |  ( u2(c2) <<  0 )  &  0x00ff);
}

// And this one works if the two bytes are contiguous in memory:
inline u2 build_u2_from( u1* p ) {
  return  build_u2_from( p[0], p[1] );
}

// Ditto for floats:
inline jfloat build_float_from( u1 c1, u1 c2, u1 c3, u1 c4 ) {
  u4 u = build_u4_from( c1, c2, c3, c4 );
  return  *(jfloat*)&u;
}

inline jfloat build_float_from( u1* p ) {
  u4 u = build_u4_from( p );
  return  *(jfloat*)&u;
}


// now (64-bit) longs

inline jlong build_long_from( u1 c1, u1 c2, u1 c3, u1 c4, u1 c5, u1 c6, u1 c7, u1 c8 ) {
  return  ( jlong(c1) << 56 )  &  ( jlong(0xff) << 56 )
       |  ( jlong(c2) << 48 )  &  ( jlong(0xff) << 48 )
       |  ( jlong(c3) << 40 )  &  ( jlong(0xff) << 40 )
       |  ( jlong(c4) << 32 )  &  ( jlong(0xff) << 32 )
       |  ( jlong(c5) << 24 )  &  ( jlong(0xff) << 24 )
       |  ( jlong(c6) << 16 )  &  ( jlong(0xff) << 16 )
       |  ( jlong(c7) <<  8 )  &  ( jlong(0xff) <<  8 )
       |  ( jlong(c8) <<  0 )  &  ( jlong(0xff) <<  0 );
}

inline jlong build_long_from( u1* p ) {
  return  build_long_from( p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7] );
}


// Doubles, too!
inline jdouble build_double_from( u1 c1, u1 c2, u1 c3, u1 c4, u1 c5, u1 c6, u1 c7, u1 c8 ) {
  jlong u = build_long_from( c1, c2, c3, c4, c5, c6, c7, c8 );
  return  *(jdouble*)&u;
}

inline jdouble build_double_from( u1* p ) {
  jlong u = build_long_from( p );
  return  *(jdouble*)&u;
}


// Portable routines to go the other way:

inline void explode_short_to( u2 x, u1& c1, u1& c2 ) {
  c1 = u1(x >> 8);
  c2 = u1(x);
}

inline void explode_short_to( u2 x, u1* p ) {
  explode_short_to( x, p[0], p[1]);
}

inline void explode_int_to( u4 x, u1& c1, u1& c2, u1& c3, u1& c4 ) {
  c1 = u1(x >> 24);
  c2 = u1(x >> 16);
  c3 = u1(x >>  8);
  c4 = u1(x);
}

inline void explode_int_to( u4 x, u1* p ) {
  explode_int_to( x, p[0], p[1], p[2], p[3]);
}


// And to extract shorts from ints:

inline int extract_low_short_from_int(jint x) {
  return x & 0xffff;
}

inline int extract_high_short_from_int(jint x) {
  return (x >> 16) & 0xffff;
}


// Enable zap-a-lot if in debug version.

# ifdef ASSERT
#   define ENABLE_ZAP_DEAD_LOCALS
# endif




