// @(#)memory.hpp	1.4 00/08/01 21:48:58
// 
// Copyright 1993-1999 Sun Microsystems, Inc. 901 San Antonio Road,
// Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
// 
// This software is the confidential and proprietary information of Sun
// Microsystems, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Sun.
// 
// CopyrightVersion 1.2


class Memory: AllStatic {
 public:
  // Efficient block copy, word-aligned, non-overlapping, not guaranteed word atomic
  static void copy_oops(oop* from, oop* to, int count) {
    #ifdef ASSERT
      assert_params_ok      (from, to, count);
      assert_non_overlapping(from, to, count);
    #endif
    pd_copy_oops(from, to, count);
  }

  // Same as copy_oops, but caller guarantees alignment according to ALIGN_ALL_OBJECTS
  static void copy_oops_aligned(oop* from, oop* to, int count) {
    #ifdef ASSERT
      assert_params_ok      (from, to, count);
      assert_non_overlapping(from, to, count);
      assert_params_aligned (from, to, count);
    #endif
    pd_copy_oops_aligned(from, to, count);
  }

  // Same as copy_oops, but allows overlap
  static void copy_oops_overlapping(oop* from, oop* to, int count) {
    #ifdef ASSERT
      assert_params_ok      (from, to, count);
    #endif
    pd_copy_oops_overlapping(from, to, count);
  }

  // Block copy, word-aligned, overlapping, guaranteed word atomic
  static void copy_oops_overlapping_atomic(oop* from, oop* to, int count) {
    #ifdef ASSERT
      assert_params_ok      (from, to, count);
    #endif
    pd_copy_oops_overlapping_atomic(from, to, count);
  }

  // Same as copy_oops_aligned, but allows overlap
  static void copy_oops_aligned_overlapping(oop* from, oop* to, int count) {
    #ifdef ASSERT
      assert_params_ok      (from, to, count);
      assert_params_aligned (from, to, count);
    #endif
    pd_copy_oops_aligned_overlapping(from, to, count);
  }

  // Initialize oops to the given value, guaranteed word atomic
  static void set_oops(oop* to, int count, oop value = NULL) {
    #ifdef ASSERT
      if (count < 0) basic_fatal("negative count");
    #endif
    pd_set_oops(to, count, value);
  }

  // Move some bytes, not guaranteed word atomic
  static void copy_bytes_overlapping(void* from, void* to, int count) {
    #ifdef ASSERT
      if (count < 0) basic_fatal("negative count");
    #endif
    pd_copy_bytes_overlapping(from, to, count);
  }

  // Initialize words to the given value, guaranteed word atomic
  static void set_words(jint* to, int count, int value = 0) {
    #ifdef ASSERT
      if (count < 0) basic_fatal("negative count");
    #endif
    pd_set_words(to, count, value);
  }

 private:
  static bool params_non_overlapping(oop* from, oop* to, int count) {
    return (from + count <= to) || (to + count <= from);
  }

#ifdef ASSERT
  // These routines raise a fatal if they detect a problem.

  static void assert_params_ok(oop* from, oop* to, int count) {
    if (mask_bits((int)from, right_n_bits(LogBytesPerWord)) != 0) basic_fatal("not word aligned");
    if (mask_bits((int)to  , right_n_bits(LogBytesPerWord)) != 0) basic_fatal("not word aligned");
    if (count < 0                                               ) basic_fatal("negative count");
  }

  static void assert_params_aligned(oop* from, oop* to, int count) {
    if (!ALIGN_ALL_OBJECTS)  return;
    if (mask_bits((int)from, right_n_bits(LogBytesPerLong)) != 0) basic_fatal("not doubleword aligned");
    if (mask_bits((int)to  , right_n_bits(LogBytesPerLong)) != 0) basic_fatal("not doubleword aligned");
    // Note:  count might be odd, but it is OK to copy the trailing extra word.
  }

  static void assert_non_overlapping(oop* from, oop* to, int count) {
    if (!params_non_overlapping(from, to, count))                 basic_fatal("source and dest overlap");
  }

#endif

  // The following header contains implementations of the above functions:
  #include "incls/_memory_pd.hpp.incl"
};
