/*
 * @(#)ostream.hpp	1.20 00/04/20 01:11:11
 *
 * Copyright 1993-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

// Output streams for printing
class outputStream : public ResourceObj {
 protected:
   int _indentation; // current indentation
   int _width;       // width of the page
   int _position;    // position on the current line
   TimeStamp _stamp; // for time stamps

 public:
   // creation
   outputStream(int width = 80);
   outputStream(int width, bool has_time_stamps);

   // indentation
   void indent();
   void inc() { _indentation++; };
   void dec() { _indentation--; };
   int  indentation() const    { return _indentation; }
   void set_indentation(int i) { _indentation = i;    }
   void fill_to(int col);

   // sizing
   int width()    const { return _width;    }
   int position() const { return _position; }

   // printing
   void print(const char* format, ...);
   void print_cr(const char* format, ...);
   void vprint(const char *format, va_list argptr);
   void vprint_cr(const char* format, va_list argptr);
   virtual void print_raw(const char* str) = 0;
   void put(char ch);
   virtual void sp();
   virtual void cr();

   // Time stamp
   void stamp();

   // portable printing of 64 bit integers
   void print_jlong(jlong value);
   void print_julong(julong value);

   // flushing
   virtual void flush() {}

   void dec_cr() { dec(); cr(); }
   void inc_cr() { inc(); cr(); }
};

// standard output
				// ANSI C++ name collision
extern outputStream* tty;	    // tty output

// for writing to strings; buffer will expand automatically
class stringStream : public outputStream {
 protected:
  char* buffer;
  int   buffer_pos;
  int   buffer_length;
 public:
  stringStream(int initial_bufsize = 256);
  ~stringStream();
  virtual void print_raw(const char* c);
  char* as_string();
};

class fileStream : public outputStream {
 protected:
  FILE* _file;
 public:
  fileStream(char* file_name);
  fileStream(FILE* file) { _file = file; }
  ~fileStream();
  int is_open() const { return _file != NULL; }
  virtual void print_raw(const char* c);
  void flush();
};

void ostream_init();
void ostream_init_log();
void ostream_exit();

