/*
 * @(#)utf8.cpp	1.16 01/02/09 14:48:13
 *
 * Copyright 1993-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// CopyrightVersion 1.2

# include "incls/_precompiled.incl"
# include "incls/_utf8.cpp.incl"

char* UTF8::next(const char* str, jchar* value) {
  unsigned const char *ptr = (const unsigned char *)str;
  unsigned char ch, ch2, ch3;
  int length = -1;		/* bad length */
  jchar result;
  switch ((ch = ptr[0]) >> 4) {
      default:
	    result = ch;
	    length = 1;
	    break;

	case 0x8: case 0x9: case 0xA: case 0xB: case 0xF:
	    /* Shouldn't happen. */
	    break;

	case 0xC: case 0xD:	
	    /* 110xxxxx  10xxxxxx */
	    if (((ch2 = ptr[1]) & 0xC0) == 0x80) {
		unsigned char high_five = ch & 0x1F;
		unsigned char low_six = ch2 & 0x3F;
		result = (high_five << 6) + low_six;
		length = 2;
		break;
	    } 
	    break;

	case 0xE:
	    /* 1110xxxx 10xxxxxx 10xxxxxx */
	    if (((ch2 = ptr[1]) & 0xC0) == 0x80) {
		if (((ch3 = ptr[2]) & 0xC0) == 0x80) {
		    unsigned char high_four = ch & 0x0f;
		    unsigned char mid_six = ch2 & 0x3f;
		    unsigned char low_six = ch3 & 0x3f;
		    result = (((high_four << 6) + mid_six) << 6) + low_six;
		    length = 3;
		} else {
		    unsigned char high_four = ch & 0x0f;
		    unsigned char low_six = ch2 & 0x3f;
		    result = (high_four << 6) + low_six;
		    length = 2;
		}
	    }
	    break;
	} /* end of switch */

  if (length <= 0) {
    *value = ptr[0];	/* default bad result; */
    return (char*)(ptr + 1); // make progress somehow
  }

  *value = result;

  // The assert is correct but the .class file is wrong
  // assert(UNICODE::utf8_size(result) == length, "checking reverse computation");
  return (char *)(ptr + length);
}

int UTF8::unicode_length(const char* str, int len) {
  int result = 0;
  jchar value;
  for (const char* p = str; p - str < len; p = next(p, &value)) result++;
  return result;
}

int UTF8::unicode_length(const char* str) {
  int result = 0;
  jchar value;
  for (const char* p = str; *p; p = next(p, &value)) result++;
  return result;
}

// Writes a jchar a utf8 and returns the end
static u_char* utf8_write(u_char* base, jchar ch) {
  if ((ch != 0) && (ch <=0x7f)) {
    base[0] = (u_char) ch;
    return base + 1;
  }

  if (ch <= 0x7FF) { 
    /* 11 bits or less. */
    unsigned char high_five = ch >> 6;
    unsigned char low_six = ch & 0x3F;
    base[0] = high_five | 0xC0; /* 110xxxxx */
    base[1] = low_six | 0x80;	/* 10xxxxxx */
    return base + 2;
  }
  /* possibly full 16 bits. */
  char high_four = ch >> 12;
  char mid_six = (ch >> 6) & 0x3F;
  char low_six = ch & 0x3f;
  base[0] = high_four | 0xE0; /* 1110xxxx */
  base[1] = mid_six | 0x80;   /* 10xxxxxx */
  base[2] = low_six | 0x80;   /* 10xxxxxx */
  return base + 3;
}

void UTF8::convert_to_unicode(const char* utf8_str, jchar* unicode_str, int unicode_length) {
  const char* p = utf8_str;
  for (int index = 0; index < unicode_length; index++) {
    p = UTF8::next(p, &unicode_str[index]);
  }
}


// Returns NULL if 'c' it not found. This only works as long
// as 'c' is an ASCII character
jbyte* UTF8::strrchr(jbyte* base, int length, jbyte c) {
  assert(length >= 0, "sanity check");    
  assert(0 <= c && c < 128, "does not work for non-ASCII characters");
  // Skip backwards in string until 'c' is found or end is reached
  while(--length >= 0 && base[length] != c);
  return (length < 0) ? NULL : &base[length];
}

bool UTF8::equal(jbyte* base1, int length1, jbyte* base2, int length2) {
  // Length must be the same
  if (length1 != length2) return false;
  for (int i = 0; i < length1; i++) {  
    if (base1[i] != base2[i]) return false;
  } 
  return true;  
}


//-------------------------------------------------------------------------------------


int UNICODE::utf8_size(jchar c) {
  if ((0x0001 <= c) && (c <= 0x007F)) return 1;
  if (c <= 0x07FF) return 2;
  return 3;
}

int UNICODE::utf8_length(jchar* base, int length) {
  int result = 0;
  for (int index = 0; index < length; index++) {
    result += utf8_size(base[index]);
  }
  return result;
}

char* UNICODE::as_utf8(jchar* base, int length) {
  int utf8_len = utf8_length(base, length);
  u_char* result = NEW_RESOURCE_ARRAY(u_char, utf8_len + 1);
  u_char* p = result;
  for (int index = 0; index < length; index++) {
    p = utf8_write(p, base[index]);
  }
  *p = '\0';
  assert(p == &result[utf8_len], "length prediction must be correct");
  return (char*) result;

}


void UNICODE::convert_to_utf8(const jchar* base, int length, char* utf8_buffer) {
  for(int index = 0; index < length; index++) {
    utf8_buffer = (char*)utf8_write((u_char*)utf8_buffer, base[index]);
  }
  *utf8_buffer = '\0';
}



