/*
 * @(#)eventHandler.h	1.30 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
#include <jni.h>
#include <jvmdi.h>
#include "bag.h"

typedef struct HandlerNode_ HandlerNode;

typedef void (*EventHandler)(JNIEnv *env, JVMDI_Event *event, 
                                 HandlerNode *node, 
                                 struct bag *eventBag);

/* HandlerNode manipulation */
HandlerNode *eventHandler_alloc(jint selectorCount, jbyte kind, jbyte suspendPolicy);
jint eventHandler_setConditionalSelector(HandlerNode *node, jint index, jint exprID);
jint eventHandler_setCountSelector(HandlerNode *node, jint index, jint count);
jint eventHandler_setThreadOnlySelector(HandlerNode *node, jint index, jthread thread);
jint eventHandler_setLocationOnlySelector(HandlerNode *node, jint index, 
                     jclass clazz, jmethodID method, jlocation location);
jint eventHandler_setFieldOnlySelector(HandlerNode *node, jint index, 
                                  jclass clazz, jfieldID field);
jint eventHandler_setClassOnlySelector(HandlerNode *node, jint index, 
                                  jclass clazz);
jint eventHandler_setExceptionOnlySelector(HandlerNode *node, jint index, 
                                  jclass exceptionClass, jboolean caught, jboolean uncaught);
jint eventHandler_setClassMatchSelector(HandlerNode *node, jint index, 
                                   char *classPattern);
jint eventHandler_setClassExcludeSelector(HandlerNode *node, jint index, 
                                     char *classPattern);
jint eventHandler_setStepSelector(HandlerNode *node, jint index, 
                             jthread thread, jint size, jint depth);
jboolean eventHandler_getStepSelector(jbyte kind, jint handlerID, 
                             jthread *thread, jint *size, jint *depth);
jint eventHandler_insert(HandlerNode *node);
jint eventHandler_id(HandlerNode *node);
jint eventHandler_freeAll(jint kind);
jint eventHandler_free(jint kind, jint handlerID);
jint eventHandler_freeInternal(HandlerNode *node);

/* programatic handler insertion */
HandlerNode *eventHandler_insertInternal(jbyte kind, EventHandler handler, 
                                     jthread thread);
HandlerNode *eventHandler_insertInternalBreakpoint(EventHandler handler, 
                                     jthread thread, jclass clazz, 
                                     jmethodID method, jlocation location);

jboolean eventHandler_predictFiltering(HandlerNode *node, jframeID frame);

void eventHandler_initialize(jbyte sessionID);
void eventHandler_reset(jbyte sessionID);
void eventHandler_onConnect(void);
void eventHandler_onDisconnect(void);


void eventHandler_lock(void);
void eventHandler_unlock(void);



