/*
 * @(#)ArrayType.java	1.15 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jdi;

import java.util.List;

/**
 * Provides access to the class of an array and the type of 
 * its components in the target VM. 
 *
 * @see ArrayReference
 *
 * @author Robert Field
 * @author Gordon Hirsch
 * @author James McIlree
 * @since  1.3
 */
public interface ArrayType extends ReferenceType {

    /**
     * Creates a new instance of this array class in the target VM.
     * The array is created with the given length and each component 
     * is initialized to is standard default value.
     *
     * @param length the number of components in the new array
     * @return the newly created {@link ArrayReference} mirroring
     * the new object in the target VM.
     * @throws ObjectCollectedException if this array type has been 
     * unloaded and garbage collected.
     */
    ArrayReference newInstance(int length);

    /**
     * Gets the JNI signature of the components of this
     * array class. The signature 
     * describes the declared type of the components. If the components
     * are objects, their actual type in a particular run-time context
     * may be a subclass of the declared class.
     *
     * @return a string containing the JNI signature of array components.
     * @throws ObjectCollectedException if this array type has been 
     * unloaded and garbage collected.
     */
    String componentSignature();

    /**
     * @return a text representation of the component type.
     * @throws ObjectCollectedException if this array type has been 
     * unloaded and garbage collected.
     */
    String componentTypeName();

    /**
     * Returns the type of the array components. For primitive arrays,
     * an appropriate {@link PrimitiveType} is always returned. For object 
     * arrays, the appropriate {@link ReferenceType} is returned if it has 
     * been loaded through the enclosing type's class loader.
     *
     * @return the {@link Type} of the array's components.
     * @throws ClassNotLoadedException if the type has not yet been loaded
     * through the appropriate class loader. 
     * @throws ObjectCollectedException if this array type has been 
     * unloaded and garbage collected.
     */
    Type componentType() throws ClassNotLoadedException;
}

