/*
 * @(#)ClassLoaderReference.java	1.9 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jdi;

import java.util.List;

/**
 * A class loader object from the target VM.
 * A ClassLoaderReference is an {@link ObjectReference} with additional
 * access to classloader-specific information from the target VM. Instances
 * ClassLoaderReference are obtained through calls to 
 * {@link ReferenceType#classLoader}
 *
 * @see ObjectReference
 *
 * @author Gordon Hirsch
 * @since  1.3
 */
public interface ClassLoaderReference extends ObjectReference {

    /**
     * Returns a list of all loaded classes that were defined by this
     * class loader. No ordering of this list guaranteed.
     *
     * @return a List of {@link ReferenceType} objects mirroring types
     * loaded by this class loader. The list has length 0 if no types
     * have been defined by this classloader.
     * @throws ObjectCollectedException if this object has been 
     * garbage collected.
     */
    List definedClasses();

    /**
     * Returns a list of all classes for which this class loader has
     * been recorded as the initiating loader in the target VM. 
     * The list contains ReferenceTypes defined directly by this loader
     * (as returned by {@link #definedTypes}) and any types for which 
     * loading was delegated by this class loader to another class loader. 
     * <p>
     * The visible class list has useful properties with respect to 
     * the type namespace. A particular type name will occur at most 
     * once in the list. Each field or variable declared with that
     * type name in a class defined by
     * this class loader must be resolved to that single type.
     * <p>
     * No ordering of the returned list is guaranteed.
     * <p>
     * See section 5.3 of the revised JVM specification for 
     * more information on initiating classloader.
     *
     * @return a List of {@link ReferenceType} objects mirroring classes
     * initiated by this class loader. The list has length 0 if no classes
     * are visible to this classloader.
     * @throws ObjectCollectedException if this object has been 
     * garbage collected.
     */
    List visibleClasses();
}

