/*
 * @(#)ListeningConnector.java	1.7 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jdi.connect;

import java.util.Map;
import java.io.IOException;
import com.sun.jdi.VirtualMachine;

/**
 * A connector which listens for a connection initiated by a target VM.
 *
 * @author Gordon Hirsch
 * @since  1.3
 */
public interface ListeningConnector extends Connector {
    /**
     * Indicates whether this listening connector supports multiple 
     * connections for a single argument map. If so, a call to 
     * {@link #startListening} may allow
     * multiple target VM to become connected.
     *
     * @return <code>true</code> if multiple connections are supported;
     * <code>false</code> otherwise.
     */
    boolean supportsMultipleConnections();

    /**
     * Listens for one or more connections initiated by target VMs.
     * The connector uses the given argument map 
     * in determining the address at which to listen or else it generates
     * an appropriate listen address. In either case, an address string
     * is returned from this method which can be used in starting target VMs
     * to identify this this connector. The format of the address string
     * is connector, transport, and, possibly, platform dependent.
     * <p>
     * The argument map associates argument name strings to instances
     * of {@link Connector.Argument}. The default argument map for a 
     * connector can be obtained through {@link Connector#defaultArguments}.
     * Argument map values can be changed, but map entries should not be
     * added or deleted.
     * <p>
     * This method does not return a {@link VirtualMachine}, and, normally,
     * returns before any target VM initiates 
     * a connection. The connected target is obtained through 
     * {@link #accept} (using the same argument map as is passed to this
     * method).
     *
     * @return the address at which the connector is listening
     * for a connection. 
     * @throws java.io.IOException when unable to start listening. 
     * Specific exceptions are dependent on the Connector implementation
     * in use. 
     * @throws IllegalConnectorArgumentsException when one of the  
     * connector arguments is invalid.
     */
    String startListening(Map arguments) throws
                              IOException,
                              IllegalConnectorArgumentsException;

    /**
     * Cancels listening for connections. The given argument map should match
     * the argument map given for a previous {@link #startListening} invocation.
     *
     * @throws java.io.IOException when unable to stop listening. 
     * Specific exceptions are dependent on the Connector implementation
     * in use. 
     * @throws IllegalConnectorArgumentsException when one of the  
     * connector arguments is invalid.
     */
    void stopListening(Map arguments) throws
                           IOException,
                           IllegalConnectorArgumentsException;


    /**
     * Waits for a target VM to attach to this connector. 
     *
     * @throws java.io.IOException when unable to accept. 
     * Specific exceptions are dependent on the Connector implementation
     * in use. 
     * @throws IllegalConnectorArgumentsException when one of the  
     * connector arguments is invalid.
     */
    VirtualMachine accept(Map arguments) throws 
                              IOException,
                              IllegalConnectorArgumentsException;

}
