/*
 * @(#)BreakpointRequest.java	1.24 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jdi.request;

import com.sun.jdi.*;

/**
 * Identifies a {@link Location} in the target VM at which 
 * execution should be stopped. When an enabled BreakpointRequest is 
 * satisfied, an
 * {@link com.sun.jdi.event.EventSet event set} containing an
 * {@link com.sun.jdi.event.BreakpointEvent BreakpointEvent}
 * will be placed on the 
 * {@link com.sun.jdi.event.EventQueue EventQueue} and
 * the application is interrupted. The collection of existing breakpoints is 
 * managed by the {@link EventRequestManager}
 *
 * @see Location
 * @see com.sun.jdi.event.BreakpointEvent
 * @see com.sun.jdi.event.EventQueue
 * @see EventRequestManager
 *
 * @author Robert Field
 * @since  1.3
 */
public interface BreakpointRequest extends EventRequest, Locatable {

    /**
     * Returns the location of the requested breakpoint.
     * 
     * @return the {@link Location} where this breakpoint has been set.
     */
    Location location();

    /**
     * Restricts the events generated by this request to those in
     * the given thread.
     * @param thread the thread to filter on.
     * @throws ObjectCollectedException if the thread object has been 
     * garbage collected.
     * @throws VMMismatchException if a {@link Mirror} argument and this mirror
     * do not belong to the same {@link VirtualMachine}.
     * @throws InvalidRequestStateException if this request is currently
     * enabled. Filters may be added only to disabled requests.
     */
    void addThreadFilter(ThreadReference thread);
}


			
