/*
 * @(#)MethodEntryRequest.java	1.11 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jdi.request;

import com.sun.jdi.*;

/**
 * Request for notification when a method is invoked in the target VM.
 * When an enabled MethodEntryRequest is satisfied, an
 * {@link com.sun.jdi.event.EventSet event set} containing a
 * {@link com.sun.jdi.event.MethodEntryEvent MethodEntryEvent}
 * will be placed on the 
 * {@link com.sun.jdi.event.EventQueue EventQueue}.
 * The collection of existing MethodEntryRequests is 
 * managed by the {@link EventRequestManager}
 *
 * @see com.sun.jdi.event.MethodEntryEvent
 * @see com.sun.jdi.event.EventQueue
 * @see EventRequestManager
 *
 * @author Robert Field
 * @since  1.3
 */
public interface MethodEntryRequest extends EventRequest {

    /**
     * Restricts the events generated by this request to those in
     * this thread.
     * @param thread the thread to filter on.
     * @throws VMMismatchException if a {@link Mirror} argument and this mirror
     * do not belong to the same {@link VirtualMachine}.
     * @throws InvalidRequestStateException if this request is currently
     * enabled. Filters may be added only to disabled requests.
     */
    void addThreadFilter(ThreadReference thread);

    /**
     * Restricts the events generated by this request to those 
     * whose method is in the given reference type or any of its subtypes.
     * An event will be generated for any location in a reference type that can
     * be safely cast to the given reference type. 
     *
     * @param refType the reference type to filter on.
     * @throws VMMismatchException if a {@link Mirror} argument and this mirror
     * do not belong to the same {@link VirtualMachine}.
     * @throws InvalidRequestStateException if this request is currently
     * enabled. Filters may be added only to disabled requests.
     */
    void addClassFilter(ReferenceType refType);

    /**
     * Restricts the events generated by this request to those 
     * whose method is in a class whose name matches this restricted
     * regular expression. Regular expressions are limited 
     * to exact matches and patterns that begin with '*' or end with '*'; 
     * for example, "*.Foo" or "java.*".
     *
     * @param classPattern the pattern String to filter for.
     * @throws InvalidRequestStateException if this request is currently
     * enabled. Filters may be added only to disabled requests.
     */
    void addClassFilter(String classPattern);

    /**
     * Restricts the events generated by this request to those 
     * whose method is in a class whose name does <b>not</b> match this restricted
     * regular expression. e.g. "java.*" or "*.Foo".
     * @param classPattern the pattern String to filter against.
     * @throws InvalidRequestStateException if this request is currently
     * enabled. Filters may be added only to disabled requests.
     */
    void addClassExclusionFilter(String classPattern);
}

