/*
 * @(#)LocationImpl.java	1.21 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.tools.jdi;

import com.sun.jdi.*;

public class LocationImpl extends MirrorImpl
    implements Location
{
    private ReferenceType declaringType;
    private Method method;
    private long methodRef;
    private long codeIndex;
    private int lineNumber;
    private boolean triedLineNumber = false;

    LocationImpl(VirtualMachine vm, 
                 Method method, long codeIndex) {
        this(vm, method, codeIndex, -1);
    }

    /*
     * Use this constructor only when the line number is in hand.
     * If not, use the other constructor, and line number (if 
     * available will be retrieved lazily
     */
    LocationImpl(VirtualMachine vm, 
                 Method method, long codeIndex, int lineNumber) {
        super(vm);

        this.method = method;
        this.codeIndex = codeIndex;
        this.lineNumber = lineNumber;
        this.declaringType = method.declaringType();
    }

    /*
     * This constructor allows lazy creation of the method mirror. This
     * can be a performance savings if the method mirror does not yet 
     * exist. 
     */
    LocationImpl(VirtualMachine vm, ReferenceType declaringType,
                 long methodRef, long codeIndex) {
        super(vm);

        this.method = null;
        this.codeIndex = codeIndex;
        this.lineNumber = -1;
        this.declaringType = declaringType;
        this.methodRef = methodRef;
    }

    public boolean equals(Object obj) {
        if ((obj != null) && (obj instanceof Location)) {
            Location other = (Location)obj;
            return (method().equals(other.method())) &&
                   (codeIndex() == other.codeIndex()) &&
                   (declaringType().equals(other.declaringType())) &&
                   (lineNumber() == other.lineNumber()) &&
                   super.equals(obj);
        } else {
            return false;
        }
    }

    public int hashCode() {
        /*
         * TO DO: better hash code?
         */
        return method().hashCode() + (int)codeIndex();
    }

    public int compareTo(Object object) {
        LocationImpl other = (LocationImpl)object;
        int rc = method().compareTo(other.method());
        if (rc == 0) {
            long diff = codeIndex() - other.codeIndex();
            if (diff < 0)
                return -1;
            else if (diff > 0) 
                return 1;
            else 
                return 0;
        }
        return rc;
    }

    public ReferenceType declaringType() {
        return declaringType;
    }

    public Method method() {
        if (method == null) {
            ReferenceTypeImpl container = (ReferenceTypeImpl)declaringType();
            method = container.getMethodMirror(methodRef);
        }
        return method;
    }

    public long codeIndex() {
        return codeIndex;
    }

    public String sourceName() throws AbsentInformationException {
        return ((ReferenceTypeImpl)declaringType()).sourceName();
    }

    public int lineNumber() {
        if ((lineNumber == -1) && !triedLineNumber) {
            MethodImpl methodImpl = (MethodImpl)method();
            lineNumber = methodImpl.codeIndexToLineNumber(codeIndex());
            triedLineNumber = true;
        }
        return lineNumber;
    }

    public String toString() {
        if (lineNumber == -1) {
            return method().toString() + "+" + codeIndex();
        } else {
            return declaringType().name() + ":" + lineNumber();
        }
    }
}
