/*
 * @(#)SharedMemoryConnection.java	1.7 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.tools.jdi;

import java.io.IOException;
import java.io.EOFException;

class SharedMemoryConnection extends Object implements ConnectionService {
    private long id;
    private Object receiveLock = new Object();
    private Object sendLock = new Object();
    private boolean closed = false;

    SharedMemoryConnection(long id) throws IOException {
        this.id = id;
    }

    private native void close0(long id);
    private native byte receiveByte0(long id) throws IOException;
    private native void sendByte0(long id, byte b) throws IOException;
    private native void receivePacket0(long id, Packet packet) throws IOException;
    private native void sendPacket0(long id, Packet packet) throws IOException;

    public void close() {
        closed = true;
        close0(id);
    }

    public byte receiveByte() throws IOException {
        if (closed) {
            throw new IOException("Connection closed");
        }
        synchronized(receiveLock) {
            return receiveByte0(id);
        }
    }

    public void sendByte(byte b) throws IOException {
        if (closed) {
            throw new IOException("Connection closed");
        }
        synchronized(sendLock) {
            sendByte0(id, b);
        }
    }

    public Packet receivePacket() throws IOException {
        if (closed) {
            throw new IOException("Connection closed");
        }
        synchronized(receiveLock) {
            Packet packet = new Packet();
            receivePacket0(id, packet);
            return packet;
        }
    }

    public void sendPacket(Packet packet) throws IOException {
        if (closed) {
            throw new IOException("Connection closed");
        }
        synchronized(sendLock) {
            sendPacket0(id, packet);
        }
    }
}

