/*
 * @(#)ParameterizedTarget.java	1.1 00/05/30
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 1.3
 *
 * @author Stanley Man-Kit Ho
 */

package netscape.security;

/**
 * This class is a subclass of UserTarget. It's meant to use the optional 
 * argument to checkScopePermission() which allows more specific information 
 * to be passed to the target (i.e., file names or network addresses). Once 
 * you register a Principal with the ParameterizedTarget, you can 
 * enablePrivilege() to build up an access control list (mapping any objects 
 * to Privileges), and checkScopePermission() allows you to query the access 
 * control list. 
 *
 * This class acts as a stub to provide backward compatibility for Netscape 
 * 4.x VM.
 */
public class ParameterizedTarget extends netscape.security.Target
{
    /**
     * This constructor is only used by the Persistent store. 
     * Not guaranteed to be supported in future versions. 
     */
    protected ParameterizedTarget()
    {
	super();
    }


    /**
     * This constructor is only used by the Persistent store. 
     * Not guaranteed to be supported in future versions. 
     *
     * @param name The name of the target (by convention, target 
     *	      names use a "BouncyCaps" style, with the first word 
     *	      capitalized) 
     * @param prin The principal in which the name is defined 
     *	      (basically, principals define namespaces for targets) 
     * @param risk A word or short phrase describing how dangerous 
     *	      this target could make an applet (usual strings are 
     *	      "low", "medium", and "high") 
     * @param riskColor - A background color for the user dialog. 
     *	      Assume the text will be black and pick a light color, 
     *	      e.g., a pastel color. The format of the color is 
     *	      "#rrggbb". 
     * @param description A short sentence describing what this 
     *	      target permits for the user (i.e.: "Read-only access 
     *	      to the local filesystem") 
     * @param url A pointer to a Web page describing this target 
     *	      in as much detail as you want. 
     */
    public ParameterizedTarget(String name,
			       Principal prin,
			       String risk,
			       String riskColor,
			       String description,
			       String url)
    {
	super(name, prin, risk, riskColor, description, url);
    }


    /**
     * @return the string representation of the data object that is stored 
     *	       with the target 
     */
    public String getDetailedInfo(Object data)
    {
	return null;
    }

    /** 
     * An intelligent target will override this method to implement its 
     * own access control policies. This method is called indirectly when 
     * other code calls: PrivilegeManager.enablePrivilege(). 
     * 
     * @param prin The principal on the requesting class 
     * @param data Arbitrary data, as passed to enablePrivilege() strictly 
     *	      for putting up a dialog box. sigh 
     */
    public Privilege enablePrivilege(Principal prin,
                                     Object data)
    {
	PrivilegeManager privMgr = PrivilegeManager.getPrivilegeManager();
	privMgr.enablePrivilege(this, prin, data);

	return null;
    }


    /** 
     * An intelligent target will override this method to implement its 
     * own access control policies. 
     *
     * @param prin The principal on the requesting class 
     * @param data Arbitrary data, as passed to checkPrivilegeEnabled() 
     */
    public Privilege checkPrivilegeEnabled(Principal prinAry[],
	                                   Object data)
    {
	PrivilegeManager privMgr = PrivilegeManager.getPrivilegeManager();
	privMgr.checkPrivilegeEnabled(this, data);

	return null;
    }
}

