/*
 * @(#)PluginAppletPanel.java	1.2 PluginAppletPanel.java 98/12/16
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.applet;

import java.net.URL;

/**
 * Applet panel class. This class is a subclass of the vanilla JDK
 * AppletPanel, it fixes a race-condition bug in the class loader
 * resolution
 *
 * @version 	1.2.2, 11/17/98
 * @author 	Jerome Dochez
 */
public
abstract class PluginAppletPanel extends AppletPanel {


    /**
     * Get a class loader. Create in a restricted context
     */
    // NOTE: The whole reason for having this class in the plugin is
    // to fix a bug in the following method. There is a race condition
    // in class loading between looking up the class loader and
    // creating a new one. Hence, a synchronized(getClass()) needs to
    // be used. Making the method static does not work since it accesses
    // object state.
    synchronized AppletClassLoader getClassLoader(final URL codebase) {
	synchronized (getClass()) {
	    return super.getClassLoader(codebase);
	}
    }
}
