/*
 * @(#)BeanLoader.java	1.0 97/5/8
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;

/*
 * The interface for objects that are responsible for initializing
 * and loading beans from an underlying storage mechanism
 *
 * @version 1.0
 * @date 5/8/97
 * @author Jerome Dochez
 */

public interface BeanLoader {

  /*
   * Load a bean from the underlying storage mechanism
   * 
   * @param beanName identifies uniquely the bean within the storage
   *
   * @throw java.io.Exception
   * @throw ClassNotFoundException
   */
  public Object initBean(String beanName)
    throws java.io.IOException, ClassNotFoundException;

  /*
   * Load a bean from the underlying storage and initializes it with the 
   * imput stream that should be its persisted state
   *
   * @param beanIdentifier identifies uniquely the bean within the storage
   * @param input InputStream on a persisted state of the bean
   *
   * @throw java.io.Exception
   * @throw ClassNotFoundException
   */
  public Object loadBean(String beanIdentifier, java.io.InputStream input) 
    throws java.io.IOException, ClassNotFoundException;

  /*
   * Load a class from the underlying storage
   * 
   * @param class name to be loaded
   *
   * @throw ClassNotFoundException
   */
  public Class loadClass(String className) throws ClassNotFoundException;

}
