/*
 * @(#)OleInputStream.java	1.0 5/5/97
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0 
 *
 */


package sun.beans.ole;

/**
 * Class interfacing the JavaBeans bridge for ActiveX native code
 * with the Customizer object for a particular JavaBeans.
 * @version 1.0, 5/5/96
 * @author	Jerome Dochez
 */

import java.beans.*;
import java.awt.*;
import java.awt.event.*;

public class CustomizerInterface implements PropertyChangeListener {

    /*
     * Constructor
     * Build a new interface object for a customizer
     * @param customizerClass JavaBean customizer class name
     * @param handle Native code implementation reference
     */
    private CustomizerInterface(String customizerClass, int handle) {
	this.customizerClass = customizerClass;
	this.handle = handle;
    }

    /*
     * return the customizer component, used by the bridge to 
     * get the preferred dimension of the component.
     * @return Customizer AWT component
     */
    public java.awt.Component getCustomizerComponent() {

	// Get the customizer class reference 
	       Class clazz;
	try {
	    sun.beanbox.SimpleClassLoader ourLoader;
	    ourLoader= sun.beanbox.SimpleClassLoader.ourLoader;
	    clazz= ourLoader.loadClass(customizerClass);
	} catch (java.lang.ClassNotFoundException e) {
	    System.err.println("No class " + customizerClass);
	    return null;
	}

	// Customizer are supposed to be subclasses of component
	       // and have a null constructor
	       Object c;
	try {
	    c = clazz.newInstance();
	} catch(Throwable e) {
	    System.err.println("no instanciation");
	    return null;
	}

	// Return the desired type
	       if (c instanceof Component) {
		   return (Component) c;
	       }
   
	return null;
    }

    /*
     * Open the customizer inside the Embedded Frame specified 
     * @param frame Frame object hosting the customizer component
     */
    public boolean openCustomizer(sun.awt.EmbeddedFrame frame, Color bkColor) {

      // setObjects should have been called by this time
	   if (cust==null) 
	     return false;
       
      java.awt.Component c;
      if (cust instanceof Component) 
	c = (Component) cust;
      else
	return false;

      // Add the component to the hosting embedded frame
	   c.setSize(c.getPreferredSize());
      frame.setBackground(bkColor);
      c.setBackground(bkColor);
      frame.add(c);
      frame.validate();

      // Register ourselves to get notified when properties
	   // are changed
	   cust.addPropertyChangeListener(this);
      return true;
    }

    /*
     * closeCustomizer, close the customizer and release all
     * resources.
     */
    public void closeCustomizer() {
      cust.removePropertyChangeListener(this);
    }


    /*
     * setObjects specifies which JavaBeans are going to be 
     * attached to the customizer, there should be at least 
     * one but there can be several, rigth now we support 
     * only one.
     * @param beans array of JavaBeans to be customized
     */
    public void setObjects(Object[] beans) {

      if (beans.length==0)
	return;
      System.err.println("Nb of objects = " + beans.length);
      java.awt.Component c = getCustomizerComponent();
      if (c instanceof Customizer)
	cust = (Customizer) c;

      // Get the bean behind it is interface object
	   ObjectProxy proxy;
      if (beans[0] instanceof ObjectProxy)
	proxy = (ObjectProxy) beans[0];
      else
	return;
    
      cust.setObject(proxy.getInterfacedObject());
      targetBeans = beans;
    
    }
  
    /* 
     * apply all changes to the customized bean, since customizer
     * apply their changes synchronously, nothing to do here now.
     */
    public synchronized void apply() {
      dirty=false;
    }

    /*
     * Property Change Listener object that we are going to
     * register to the customizer to get notification of 
     * any property change
     */
    public void propertyChange(PropertyChangeEvent evt) {
      synchronized(this) {
	System.err.println("Property Changed");
	dirty = true;
      }
    }	

    private String customizerClass; // Customizer class name
    private Customizer cust;        // Customizer object
    private int    handle;          // Native code implementation ref
    private Object[] targetBeans;   // Beans array to be customized
    private boolean     dirty;      // Did any property got touched   
}

