/*
 * @(#)Dispatch.java	0.1 97/3/27
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */


package sun.beans.ole;

/* 
 * An Object who whishes to provide an IDispatch implementation should
 * implement this interface. The interface implementor is not necessarly 
 * the automated object, thus the target object for remote invocation 
 * should be returned by the getInterfacedObject method
 */
public interface Dispatch {

    /*
     * Completion code for a property or method not found error
     */
    public static final int DISP_E_MEMBERNOTFOUND = 0x80020003;    

    /*
     * Completion code for a parameter type mismatch
     */
    public static final int DISP_E_TYPEMISMATCH   = 0x80020005;

    /*
     * Completion code for a generic failure
     */
    public static final int E_FAIL                = 0x80004005;

    /*
     * Completion code for a wrong number of arguments
     */
    public static final int DISP_E_BADPARAMCOUNT  = 0x8002000E;

    /*
     * Completion code for success
     */
    public static final int S_OK                  = 0;

    /*
     * Invoke a method on the interfaced object
     * @param dispid index of the method to invoke
     * @param dispparms array of DISPPARAM structure containing the parameters
     * @param dispresult DISPPARAM structure to package the return value of the
     * method
     *
     * @return completion code
     */
    public abstract int invoke(int dispid, int dispparams, int dispresult) 
	throws Throwable;

    /*
     * Set a Property of the interfaced object
     * @param dispid of the property to set
     * @param dispparms array of DISPPARAM structure containing the parameters
     * @return completion code
     */
    public abstract int setProperty(int dispid, int dispparams) 
	throws Throwable;

    /*
     * Get the property of the interfaced object
     * @param dispid index of the property to get
     * @param dispresult DISPPARAM structure to package the return value of the
     * method
     *
     * @return completion code
     */
    public abstract int getProperty(int dispid, int dipresult) 
	throws Throwable;


    /*
     * return an unique integer identifier of the requested member that can
     * be used in the subsequent invoke call
     * @param name Name of the member to be mapped
     * @return 0xFFFFFFFF if not found
     *         or unique identifier of the member
     */
    public abstract int getIDsOfNames(String name);
}
