/*
 * @(#)DispatchImpl.java	1.01 97/09/3
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion bdk_beta
 *
 */

package sun.beans.ole;

import java.lang.reflect.*;


class DispatchImpl extends OleDispatchInterface
		   implements ObjectProxy 
{

    /*
     * Construct a new interface object between the native implementation
     * and the JavaBeans component
     *
     * @param interfacedObject the JavaBeans component
     */
    public DispatchImpl(Object interfacedObject) {
	this.interfacedObject = interfacedObject;
    }

    /*
     * return the interfaced object
     *
     */
    public Object getInterfacedObject() {
	return interfacedObject;
    } 

    public Object extractObjectParameter(String parmType, boolean isArray, int dispparams, int index) {

	String extractMethodName;
	String methodSufix;
	try {
	if (parmType.charAt(0)=='L') {
	    String className = parmType.substring(1);
	    methodSufix=(isArray?"ObjectArrayParm":"ObjectParm");
	    extractMethodName = "get" +
		className.substring(className.lastIndexOf('/')+1) +
		methodSufix;
	}
	else {
	    methodSufix=(isArray?"ArrayParm":"Parm");
	    String type="int";
	    switch(parmType.charAt(0)) {
		case 'C' : 
		    type = "char";
		    break;
		case 'Z' :
		    type = "boolean";
		    break;
		case 'F' : 
		    type = "float";
		    break;
		case 'D' :
		    type = "double";
		    break;
		case 'B' : 
		    type = "byte";
		    break;
		case 'I' :
		    type = "int";
		    break;
		case 'J' :
		    type = "long";
		    break;
		case 'S' :
		    type = "short";
		    break;
	    }
	    extractMethodName = type + methodSufix;
	}

	Method extract;
	try {
	    extract = getClass().getMethod(extractMethodName, extractTypes);
	} catch(NoSuchMethodException e) {	    
	    try {
		if (isArray)
    		    extract = getClass().getMethod("getObjectArrayParm", extractTypes);
		else
		    extract = getClass().getMethod("getObjectParm", extractTypes);
	    } catch(NoSuchMethodException ex) {
		ex.printStackTrace();
		return null;
	    }
	}
	if (extract!=null) {
	    Object[] extractParms = new Object[2];
	    extractParms[0] = new Integer(dispparams);
	    extractParms[1] = new Integer(index);
	    try {
		Object o = extract.invoke(this, extractParms);	   
		return o;	    
	    } catch (Throwable e) {
		e.printStackTrace();
		return null;
	    }
	}
	else 
	    return null;
    } catch (Throwable e) {
	e.printStackTrace();
	return null;
    }
    }

    public void storeObjectParameter(Object result, int dispresult)
    {
	if (result!=null) 
	    storeParms[2] = result.getClass();
	else
	    storeParms[2] = java.lang.Object.class;

	Method store;
	try {
	    store = getClass().getMethod("storeParm", storeParms);
	} catch (NoSuchMethodException e) {
	    storeParms[2] = java.lang.Object.class;
	    try {
		store = getClass().getMethod("storeParm", storeParms);
	    } catch (NoSuchMethodException ex) {
		System.out.println("Cannot find store method for " + result.getClass());
		return;
	    }
	}

	if (store!=null) {// Found the store method
	    Object[] parmsValues = new Object[4];
	    parmsValues[0] = new Integer(dispresult);
	    parmsValues[1] = new Integer(0);
	    parmsValues[2] = result;
	    parmsValues[3] = new Integer(READ_ONLY);
	    try {
		store.invoke(this, parmsValues);
	    } catch (Throwable e) {
		System.out.println("Error while packaging " + result);
		return;
	    }
	}
    }

    private Object interfacedObject;
    private Class[] storeParms = {java.lang.Integer.TYPE, java.lang.Integer.TYPE, null, java.lang.Integer.TYPE};
    private Class[] extractTypes = {java.lang.Integer.TYPE, java.lang.Integer.TYPE};

}

