/*
 * @(#)ExceptionReporter.java	1.0 17/1/97
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0 
 *
 */
package sun.beans.ole;

import java.awt.*;
import java.util.Vector;

/*
 * InformationDialog
 *
 * This Dialog class display an information message and stay modal until
 * the user accept it
 *
 *   @author Jerome Dochez
 *   @version 1.0         
 */

class InformationDialog extends Dialog implements java.awt.event.ActionListener {

   /**
    * Construct an initially invisible InformationDialog and set the title
    * and the message string to the parameters
    *
    * Parameters :
    *   @param parent - the owner of the dialog
    *   @param title - the title of the dialog
    *   @param messages - array of line messages to display
    */

    public InformationDialog(Frame parent, String title, String[] messages) {

        super(parent, title, true);
        init(parent, title, messages);
    }

   /**
    * Construct an initially invisible InformationDialog and set the title
    * and the message string to the parameters
    *
    * Parameters :
    *   @param parent - the owner of the dialog
    *   @param title - the title of the dialog
    *   @param messages - vector of messages to display
    */
    public InformationDialog(Frame parent, String title, Vector messages) {

        super(parent, title, true);
      
        String messageStr[] = new String[messages.size()];

        messages.copyInto(messageStr);
        init(parent, title, messageStr);

    }
   /**
    * Construct an initially invisible InformationDialog and set the title
    * and the message string to the parameters
    *
    * Parameters :
    *   @param parent - the owner of the dialog
    *   @param title - the title of the dialog
    *   @param messages - message to display
    */
    public InformationDialog(Frame parent, String title, String message) {

        super(parent, title, true);

        String messageStr[] = new String[1];
        messageStr[0] = message;
        init(parent, title, messageStr);
    }

    private void init(Frame parent, String title, String [] messages)
    {
        Rectangle parentBounds = parent.getBounds();
	
        setLayout(new GridLayout(messages.length+1,1,10,10));

        // Add the information messages
        for( int i=0 ; i < messages.length ; i++)  {

            add(new Label(messages[i], Label.CENTER));
        }

        // Add a sub Container
        // add the Ok Button
        button = new Button("OK");
        add(button);
	button.addActionListener(this);

	Rectangle location = 
	    new Rectangle(parentBounds.x + parentBounds.width/3,
			  parentBounds.y + parentBounds.height/3,
			  500,100 + (50*messages.length));
	setBounds(location);
        pack();
        show();

    }

    /*
     * ActionListener interface implementation
     * @param e button event
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
	dispose();
    }

    private Button            button;
}
