/*
 * @(#)JarBeanLoader.java	1.0 97/5/8
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;

/*
 * Implementation of the BeanLoader interface to load beans from a Jar
 * file.
 * @see sun.beans.ole.BeanLoader
 *
 * @version 1.0 
 * @date 5/8/97
 * @author Jerome Dochez
 */

import sun.beanbox.*;
import java.io.*;
import java.beans.Beans;

public class JarBeanLoader implements BeanLoader {

    /* 
     * Construct a new JarBeanLoader with the jar file name specified
     *
     * @param jar file name
     */
    public JarBeanLoader(String jarFileName) {
	this.jarFileName = jarFileName;
    }

    /*
     * Load a bean from the underlying storage mechanism
     * 
     * @param beanName identifies uniquely the bean within the storage
     */
    public Object initBean(String beanName) 
	throws java.io.IOException, ClassNotFoundException {

	try {
	    JarLoader jl = loadJar(jarFileName);
	    JarInfo ji = jl.loadJar();
	    return  ji.getInstance(beanName);
	} catch (Throwable e) {
	    System.err.println("Bean Class "+ beanName + " not found in the jar");
	    return null;
	}
    }

    /*
     * Load a bean from the underlying storage and initializes it with the 
     * imput stream that should be its persisted state
     *
     * @param beanName identifies uniquely the bean within the storage
     * @param input InputStream on a persisted state of the bean
     */
    public Object loadBean(String beanName, java.io.InputStream input) 
	throws java.io.IOException, ClassNotFoundException {

	Object bean=null;
	JarLoader jl = loadJar(jarFileName);
	JarInfo ji = jl.loadJar();

	java.io.BufferedInputStream bs = new  java.io.BufferedInputStream(input);
	bs.mark(Integer.MAX_VALUE);

	// Tell the class loader about it
	SimpleClassLoader scl = (SimpleClassLoader) jl.getLoader();       
	scl.setLocalResourceSource("NULL.ser", new MyJarProducer(bs));
		
    // Instantiate the object back with serialization...
    try {
		bean = java.beans.Beans.instantiate(scl, "NULL");
		if(bean == null)
		    throw new ClassNotFoundException(beanName);
    } catch (ClassNotFoundException e) {
		/*
		 * Ok the class loader has failed loading it, inform the user
		 * return a new instance.
		 */
		e.printStackTrace();
		return ji.getInstance(beanName);
	    }
	return bean;

    }

    /*
     * Load a class from the jar file
     *
     * @param class name
     *
     * @throw ClassNotFoundException
     */
    public Class loadClass(String className) throws ClassNotFoundException {

	// Ensure the jar has been loaded
	       loadJar(jarFileName);
	return SimpleClassLoader.ourLoader.loadClass(className);
    }

    /*
     * Try to load a jar file from the file system
     *
     * @param Jar File Name
     * @return JarLoader for this jar
     */
    private JarLoader loadJar(String jarFileName) {

	if(jarLoader!=null)
	    return jarLoader;

	if (jarFileName!=null) {
	    // A Jar file name has been specified, try to load the jar
		   JarLoader jl = null;
	    try {
		jl = new JarLoader(jarFileName);
		if (jl == null) {
		    System.err.println("Jar File " + jarFileName + " not found");
		    return null;
		}
	    } catch (Exception ex) {
		System.err.println("Jar File " + jarFileName + " not found");	    
		return null;
	    }
	    jarLoader = jl;
	    return jl;
	} else 
	    return null;
    }

    private String  jarFileName;       // Jar File Name
    private JarLoader jarLoader;       // Jar Loader object

}

/**
 * Auxiliary class to deliver the InputStream
 */

class MyJarProducer implements InputStreamProducer {
    private java.io.InputStream stream;

    public MyJarProducer(java.io.InputStream stream) {
	this.stream = stream;
    }

    public synchronized InputStream getInputStream() {
	return stream;
    }
}
