/*
 * @(#)JavaObjectInterface.java	0.01 96/12/17
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

 package sun.beans.ole;

 import java.lang.reflect.*;
 import java.util.Hashtable;
 import sun.beanbox.SimpleClassLoader;

 /*
  * Java Implementation of the IDispatch interface, lifetime of this object
  * is governed by its C++ peer. The C++ peer assumes that this class will
  * be used for all exported Java Object.
  */
 class JavaObjectInterface  extends OleDispatchInterface
                                   implements Dispatch, ObjectProxy {

    /*
     * <p>
     * Clear the caches since the objects are being closed
     * </p>
     */
     public static void clearAllCaches() {
	 synchronized(JavaObjectInterface.class) {
	    dispids=null;
	 }
     }

    /*
     * Construct a new JavaObjectInterface which is implementing the OLE
     * dispatch interface for the object passed.
     * @param interfacedObject    Java Object for which this object implements
     *                            the IDispatch interface
     */
     public JavaObjectInterface(Object interfacedObject) {

	 this.interfacedObject = interfacedObject;
     }

     /*
      * return an unique integer identifier of the requested member that can
      * be used in the subsequent invoke call. 
      * @param name Name of the member to be mapped
      */
     public int getIDsOfNames(String name) {
	    
         Hashtable h;
	 synchronized (JavaObjectInterface.class) {
	    // try to cache per class
	    if (dispids==null) 
		dispids=new Hashtable();	 

	    Object o = dispids.get(interfacedObject.getClass());

	    if (o==null) {
		// initialize it
		h = new Hashtable();
		// Save the hashtable for the dispids, and cache the array of methods
		// and fields.
		dispids.put(interfacedObject.getClass(), h);
		h.put("methods_automation", interfacedObject.getClass().getMethods());
		h.put("fields_automation", interfacedObject.getClass().getFields());
	    } else 
		h = (Hashtable) o;   
	 }

	 // Get the array of methods
	 methods = (Method[]) h.get("methods_automation");
	 fields = (Field[]) h.get("fields_automation");

	 Object dispid = h.get(name);
	 if (dispid!=null) {
	     if (dispid instanceof Integer)
		 return ((Integer) dispid).intValue();
	 }
	 int disp = getDispidOfName(name);
	 h.put(name, new Integer(disp));
	 return disp;
   
     }

     /*
      * return an unique integer identifier of the requested member that can
      * be used in the subsequent invoke call. 
      * The ID returned is computed as follow:
      * If the name is a property , ID is the index of the java.lang.reflect.Fied
      * object in the array returned by Class.getFields() + 0x8000000
      * @see java.lang.reflect.Field
      * if the name is a method, ID is the index of the java.lang.reflect.Method
      * object in the array returned by Class.getMethods() + 1 to avoid the 
      * the DISPID_VALUE(0) 
      * @see java.lang.reflect.Method
      * if the name is getNew, ID is 0x7FFFFFE
      * if the name is getConstructor, ID is 0x7FFFFFD
      * @param name Name of the member to be mapped
      * @return 0xFFFFFFFF if not found
      *         Unique identifier of the member
      */
     private int getDispidOfName(String name) {

        // Lazy initialization of the the cached information
        if (methods==null) {
            Class classObject = interfacedObject.getClass();

            methods = classObject.getMethods();
            fields = classObject.getFields();
        }

        // look fist in the list of properties
       if (fields!=null) {
            for (int i=0; i<fields.length;i++) {
                Field aField = fields[i];
                if (name.equalsIgnoreCase(aField.getName()))
                    return i + 0x8000000;
            }
        }

       if (methods!=null) {
            for (int i=0; i<methods.length; i++) {
                Method aMethod = methods[i];
                if (name.equalsIgnoreCase(aMethod.getName()))
                    return i+1;
            }
        }
	if (name.equalsIgnoreCase("getNew"))
	    return 0x7FFFFFE;
	if (name.equalsIgnoreCase("getConstructor"))
	    return 0x7FFFFFD;
        return 0xFFFFFFFF;
     }

     /*
      * Get the property of the interfaced object
      * @param dispid index of the property to get returned by getIDsOfNames
      * @param dispresult DISPPARAM structure to package the return value of the 
      * method
      * @return completion code
      */
     public int getProperty(int dispid, int dispresult) throws Throwable {

	 Field field;

	 try {
	     field = fields[dispid];
	 } catch(IndexOutOfBoundsException e) {
	     return DISP_E_MEMBERNOTFOUND;
	 }

	 try {
	     storeParm(dispresult, field.get(interfacedObject), true);
	 } catch (NullPointerException e) {
	     return E_FAIL;
	 } catch (IllegalArgumentException e) {
	     return DISP_E_TYPEMISMATCH;
	 } catch (IllegalAccessException e) {
	     return DISP_E_MEMBERNOTFOUND;
	 }
	 return 0;
     }

     /*
      * Set a Property of the interfaced object
      * @param dispid of the property to set returned by getIDsOfNames()
      * @param dispparms array of DISPPARAM structure containing the parameters
      * @return completion code
      */
     public int setProperty(int dispid, int dispparams) throws Throwable {

	 Field field;
	 try {
	     field  = fields[dispid];
	 } catch(IndexOutOfBoundsException e) {
	     return DISP_E_MEMBERNOTFOUND;
	 }

	 if (parmsNumber(dispparams)!=1)
	     return DISP_E_BADPARAMCOUNT;

	 Object newValue = extractParameter(field.getType(), dispparams, 0);
	 if (newValue==null)
             return DISP_E_TYPEMISMATCH;
	 try {
             field.set(interfacedObject, newValue);
	 } catch (NullPointerException e) {
             return E_FAIL;
	 } catch (IllegalArgumentException e) {
             return DISP_E_TYPEMISMATCH;
	 } catch (IllegalAccessException e) {
             return DISP_E_MEMBERNOTFOUND;
	 }
	 return 0;
     }
     /*
      * Invoke a method on the interfaced object
      * @param dispid index of the method to invoke returned by getIDsOfNames
      * @param dispparms array of DISPPARAM structure containing the parameters
      * @param dispresult DISPPARAM structure to package the return value of the 
      * method
      * @return completion code
      */
     public int invoke(int dispid, int dispparams, int dispresult)
	 throws Throwable {

	 Method method;
	 Object result=null;
	 boolean needUnwrap = true;

	 if (methods==null) {
            methods = interfacedObject.getClass().getMethods();
            fields = interfacedObject.getClass().getFields();
	 }

	 /* Look if these are not our special methods */
	 if (dispid==0x07FFFFFE) {
	     if (parmsNumber(dispparams)!=1)
		 return DISP_E_BADPARAMCOUNT;
	     String  className = getStringObjectParm(dispparams, 0);
	     Class clazz = null;
	     try {
		 clazz = Class.forName(className);
	     } catch (ClassNotFoundException notFound) {
		 clazz = SimpleClassLoader.ourLoader.loadClass(className);
	     }
	     if (clazz==null)
		 throw new NoClassDefFoundError("Class " + className + 
						" Not found");	 
	     result = clazz.newInstance();
	 } else
	     if (dispid==0x07FFFFFD) {
		 int n = parmsNumber(dispparams);
		 /*
		  * the parameters are orderd in reversed sequence
		  */	
		 String  className = getStringObjectParm(dispparams, n - 1);
		 Class clazz = null;
		 try {
			clazz = Class.forName(className);
		 } catch (ClassNotFoundException notFound) {
			clazz = SimpleClassLoader.ourLoader.loadClass(className);
		 }
		 String signature = "public " + className + "(" +
		     getStringObjectParm(dispparams, 0) + ")";
		 Constructor constructors[] = clazz.getConstructors();
		 for (int i=0;i<constructors.length;i++) {
		     if (Modifier.isPublic(constructors[i].getModifiers()) &&
                       constructors[i].getParameterTypes().length == n - 1) {
                        Class [] parameterTypes = constructors[i].
                               getParameterTypes();
                        int j;
                        for (j = 0; j < parameterTypes.length; j++) {
                               if (!parameterTypes[j].getName().equalsIgnoreCase(
                                   getStringObjectParm(dispparams, n - 2 - j)))
                                       break;
                        }
                        if (j == parameterTypes.length) {
			     result = constructors[i];
			     break;
			}
		     }
		 }
		 if (result==null)
		     return DISP_E_MEMBERNOTFOUND;  
	     }
	     else {
		 // This is a member dispatch
		 try {
		     // Remove one from the DISPID that has been added by 
		     // the GetIDsOfNames to avoid the DISPID_VALUE(0)
		     method = methods[dispid-1];
		 } catch (IndexOutOfBoundsException e){
		     return DISP_E_MEMBERNOTFOUND;
		 }

                String methodName = method.getName();
                /*
                 * look up the method again, given both the name
                 * and the number of parameters
                 */ 
               Method method_i = null;
               for (int i=0; i < methods.length; i++) {
                       if (methods[i].getName().equals(methodName) &&
                           methods[i].getParameterTypes().length ==
                               parmsNumber(dispparams)) {
                               method_i = methods[i];
                               break;
                       }
               }
               if (method_i == null)
                       return DISP_E_BADPARAMCOUNT;
               
               /*
                * method_i is now a method with same name and correct
                * parameter number
                */
               method = method_i; 


		 // Extract the parameters
		 Object parmValues[]=null;
		 Class[] parameterTypes = method.getParameterTypes();
	  
		 if (parameterTypes!=null) {
	      
		     if (parmsNumber(dispparams)!=parameterTypes.length)
			 return DISP_E_BADPARAMCOUNT;
	    
		     parmValues = new Object[parameterTypes.length];
		     for (int i=0; i<parameterTypes.length;i++) {
			 parmValues[i] = extractParameter(parameterTypes[i],
							  dispparams, 
							  parameterTypes.length-i-1);
		     }
		 }
		
                 Class returnType = method.getReturnType();
                 if (returnType.isPrimitive())
                       needUnwrap = true; 
                 else 
                       needUnwrap = false;

		 try {
		     result = method.invoke(interfacedObject, parmValues);
		 } catch (NullPointerException e) {
		     return E_FAIL;
		 } catch (IllegalArgumentException e) {
		     return DISP_E_TYPEMISMATCH;
		 } catch (InvocationTargetException e) {
		     throw e.getTargetException();
		 }
	     }
     
	 // Package the result type;
	 if (dispresult!=0 && result!=null)
	     storeParm(dispresult, result, needUnwrap);

	 return 0;
     }

     /*
      * @return the instance of the automated object
      */			     
     public Object getInterfacedObject() {
	 return interfacedObject;
     }

     /*
      * native implementation dispatching for the parameters storing. 
      *
      */
     private void storeParm(int dispresult, Object value, boolean needUnwrap) {
	 nativeStoreParm(dispresult, value, 
			 value.getClass().toString().replace('.', '/'),
			 value.getClass().isArray(),
			 needUnwrap);
     }

     private Object extractParameter(Class parameterType, int dispparams, int index) {

	 String className = parameterType.getName();

	 if (className.compareTo("java.lang.Integer")==0 
	     || className.compareTo("int")==0)
	     return getIntegerObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Short")==0
	     || className.compareTo("short")==0)
	     return getIntegerObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Double")==0
	     || className.compareTo("double")==0)
	     return getDoubleObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Float")==0
	     || className.compareTo("float")==0)
	     return getFloatObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Boolean")==0
	     || className.compareTo("boolean")==0)
	     return getIntegerObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Character")==0
	     || className.compareTo("char")==0)
	     return getCharacterObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Long")==0
	     || className.compareTo("long")==0)
	     return getLongObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.Byte")==0
	     || className.compareTo("byte")==0)
	     return getByteObjectParm(dispparams, index);
	 if (className.compareTo("java.lang.String")==0 )
	     return getStringObjectParm(dispparams, index);
	 if (className.compareTo("java.awt.Font")==0)
	     return getFontObjectParm(dispparams,index);
	 if (className.compareTo("java.awt.Color")==0) 
	     return getColorObjectParm(dispparams, index);	  
	 if (className.compareTo("[I")==0) 
	     return intArrayParm(dispparams, index);	  
	 if (className.compareTo("[S")==0) 
	     return shortArrayParm(dispparams, index);	  
	 if (className.compareTo("[F")==0) 
	     return floatArrayParm(dispparams, index);	  
	 if (className.compareTo("[D")==0) 
	     return doubleArrayParm(dispparams, index);	  
	 if (className.compareTo("[B")==0) 
	     return byteArrayParm(dispparams, index);	  
	 if (className.compareTo("[Z")==0) 
	     return booleanArrayParm(dispparams, index);	  
	 if (className.compareTo("[J")==0) 
	     return longArrayParm(dispparams, index);	  
	 if (className.compareTo("[C")==0) 
	     return charArrayParm(dispparams, index);	  
	 if (className.compareTo("[Ljava.lang.String;")==0) 
	     return getStringObjectArrayParm(dispparams, index);	  
	 if (className.compareTo("[Ljava.awt.Color;")==0) 
	     return getColorObjectArrayParm(dispparams, index);	  
	 if (className.compareTo("[Ljava.awt.Font;")==0) 
	     return getFontObjectArrayParm(dispparams, index);	  
	 if (parameterType.isArray())
	     return getObjectArrayParm(dispparams, index);
	 else
	     return getObjectParm(dispparams, index);
     }

     private Object interfacedObject;

     // Cached list of methods and fields implemented by the interfaced object
     private java.lang.reflect.Method[]      methods;
     private java.lang.reflect.Field[]       fields;

     private static java.util.Hashtable            dispids;
 }

