/*
 * @(#)OleBeanInfo.java	0.01 96/12/08
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

 package sun.beans.ole;

 import java.beans.*;
 import java.lang.reflect.*;
 import java.util.Vector;

 /*
  * Contains explicit information about a bean organized so the OLE
  * packager can extract the right information about the bean. Note that
  * this class is used at packaging time only.
  */

 public class OleBeanInfo implements BeanInfo {

     public boolean uncrackEvent;

    /*
     * This constructor takes a Bean object. This bean is the target of
     * the packaging step.
     */
    public OleBeanInfo(Class bean) throws IntrospectionException {

        Vector      defaultEventsV, extendedEventsV, propertyEventsV;
        EventSetDescriptor[]    events;

        beanInfo = Introspector.getBeanInfo(bean);
        beanClass = bean;

        // We need to separate the reals event interface from
        // the property change events since they are handled
        // differently in OLE
        defaultEventsV = new Vector();
        extendedEventsV = new Vector();
        propertyEventsV = new Vector();

        // Get the array of events this bean is exporting
        events = getEventSetDescriptors();
        for (int i=0 ; i< events.length; i++) {

            // if the event is a PropertyChange/PropertyVetoable event
            // this event will be handled differently in the OLE world
            // through the Property Change notification mechanism, so
            // store it in a specific array
            if (isImplementorOf(events[i].getListenerType(), 
				PropertyChangeListener.class)) 
                propChange = events[i];	   
	    else 
	      if (isImplementorOf(events[i].getListenerType(),
				  VetoableChangeListener.class)) {
		/*String methods[] = { "vetoableChange" };
		EventSetDescriptor newVeto = 
		  new EventSetDescriptor(OleVetoableListenerTag.class,
					 "oleVetoableListener",
					 OleVetoableListener.class,
					 methods,
					 "addOleVetoableListener",
					 "removeOleVetoableListener");
			      
		vetoChange = newVeto;    */
		vetoChange = events[i];
	      }
	      else {
                // Ok this is a real event (ala OLE) so check if it's
                // part of the Default outgoing interface so it get
                // handled in the default OLE outgoing inteface.
                if (events[i].isInDefaultEventSet())
                    defaultEventsV.addElement(events[i]);
                else
                    extendedEventsV.addElement(events[i]);
            }

        }

        // Copy into the array of default event descriptors
        if (defaultEventsV.size()>0) {
            defaultEvents = new EventSetDescriptor[defaultEventsV.size()];
            defaultEventsV.copyInto(defaultEvents);
        }

        // Copy into the array of extended event descriptors
        if (extendedEventsV.size()>0) {
            extendedEvents = new EventSetDescriptor[extendedEventsV.size()];
            extendedEventsV.copyInto(extendedEvents);
        }

	// Try to be smart about bound properties
        noBoundFix();
    }

    /*
     * @return An array of additional BeanInfo, never used by the packager
     * since the one specified by the Bean BeanInfo object will be merged
     * by the Introspector.
     */
    public BeanInfo[] getAdditionalBeanInfo() {
        return null;
    }

    /*
     * @return A bean descriptor returns overall information about the bean
     * such as its display name, its customizer.
     */
    public BeanDescriptor getBeanDescriptor() {
        return beanInfo.getBeanDescriptor();
    }

    /*
     * @return Index of the default event in the array of EventSetDescriptor
     */
    public int getDefaultEventIndex() {
        return beanInfo.getDefaultEventIndex();
    }

    /*
     * @return Index of the default property in the array of PropertySetDescriptor
     */
    public int getDefaultPropertyIndex() {
        return beanInfo.getDefaultPropertyIndex();
    }

    /*
     * @return An array of EventSetDescriptor describing the kind of events fired
     * by this bean.
     */
    public EventSetDescriptor[] getEventSetDescriptors() {
        return beanInfo.getEventSetDescriptors();
    }

    /*
     * @return An image object that could be used in toolboxes, toolbars, etc...
     */
    public java.awt.Image getIcon(int iconKind) {
        return beanInfo.getIcon(iconKind);
    }

    /*
     * This method will return an array of all non
     * overloaded methods that can be accessed from OLE
     * @return An array of methods supported by the Bean.
     */
    public MethodDescriptor[] getMethodDescriptors() {
	return beanInfo.getMethodDescriptors();
    }

    /*
     * @return An array of properties supported by the bean.
     */
    public PropertyDescriptor[] getPropertyDescriptors() {
        return beanInfo.getPropertyDescriptors();
    }

    /*
     * @return An array of EventSetDescriptors which are tagged
     * as being part of the default Event Set Interface through
     * setInDefaultEventSet
     * This events will be generated in an unique OLE default
     * outgoing interface.
     */
    public EventSetDescriptor[] getDefaultEventSetDescriptors() {
        return defaultEvents;
    }

    /*
     * @return An array of EventSetDescriptors which are tagged
     * as not being part of the default Event Set Interface through
     * setInDefaultEventSet
     * Each event interface will be generated in its own OLE
     * outgoing interface.
     */
    public EventSetDescriptor[] getExtendedEventSetDescriptors() {
        return extendedEvents;
    }

    /*
     * @return true if the JavaBean component implements at least one
     * bound property
     */
    public boolean supportBoundProperties() {
     return propChange!=null;
    }

    /*
     * @return true if the JavaBean component implements at least one
     * constrained property
     */
    public boolean supportConstrainedProperties() {
     return vetoChange!=null;
    }

    /*
     * @return Class object of the introspected bean
     */
    public Class getBeanClass() {
        return beanClass;
    }

    /*
     * @return Customizer class if the bean implements a Customizer
     */
    public Class getCustomizerClass() {

      BeanDescriptor bd = getBeanDescriptor();
      return bd.getCustomizerClass();
    }

    /*
     * This is a convenience method called from C.
     * @parm   index   the index of the property to study
     * @return True if the i'th property is indexed
     */
    public boolean isPropertyIndexed(PropertyDescriptor pd) {

        return pd instanceof IndexedPropertyDescriptor;
    }

   /*
    * @return true if the Bean component is visible, that is
    * if it is a subclass of java.awt.Component
    */
   public boolean isVisible() {
     Class currentClass = beanClass;
     do {
       if (currentClass == java.awt.Component.class)
	 return true;
       currentClass = currentClass.getSuperclass();
     } while (currentClass!=null);
     return false;
   }

    /*
     * This is a convenient method classed from C
     * @parm   index   the index of the propery in the PropertySetDescriptor
     * @return True if the i'th property is bound
     */
     private void noBoundFix() {

        if (propChange==null)
            return;

         PropertyDescriptor[] properties = getPropertyDescriptors();
	
         // Not terribly efficient but this is package time so we don't need
         // speed, to be refined anyway to cache this info

	 // OK this bean is firing property change notification
	 // look if it specify which property is actually bound
	 boolean anyBound = false;
	 for (int j=0; j < properties.length; j++)
	     anyBound = properties[j].isBound();
	 if (anyBound)
	     return;
	 else {
	     // Ok no property is marked as bound
	     // Change all the PropertyDescriptors to set the bound flag
	     for (int j=0;j<properties.length;j++)
		 properties[j].setBound(true);
	 }

            
       
     }

    /*
     * This is a fix for a Introspector bug that doesn't not look for
     * the thrown VetoableException to infere the list of constrained
     * property. Today I asssume they are all constrained
     */
/*     private void noConstrainedFix() {

        if (propertyEvents==null)
            return;

        PropertyDescriptor[] properties = getPropertyDescriptors();

         // look first if the bean is firing the property change event.
        for (int i=0; i<propertyEvents.length;i++) {
            Class vetoableChange;
            try {
                vetoableChange = Class.forName("java.beans.VetoableChangeListener");
            } catch (Throwable e) { return ; }
            if (isImplementorOf(propertyEvents[i].getListenerType(), vetoableChange)) {
                // OK this bean is firing property veto notification
                // look if it specify which property is actually constrained
                boolean anyConstrained = false;
                for (int j=0; j < properties.length; j++)
                    anyConstrained = properties[j].isConstrained();
                if (anyConstrained)
                    return;
                else {
                    // Ok no property is marked as constrained
                    // Change all the PropertyDescriptors to set the constrained flag
                    for (int j=0;j<properties.length;j++)
                        properties[j].setConstrained(true);
                }
            }
        }
     }

*/
    /*
     * @return True if the EventSetDescriptor describe a property
     * change notification interface
     */
    private boolean isPropertyNotification(EventSetDescriptor event) {
        Class propertyChange;
        Class proposedChange;
        try {
            propertyChange = Class.forName("java.beans.PropertyChangeListener");
            proposedChange = Class.forName("java.beans.VetoableChangeListener");
        } catch (Exception e) {
            new InformationDialog(null, "Fatal Error", 
				  "Cannot find listener classes");
            return false;
        }
        return (isImplementorOf(event.getListenerType(), propertyChange)
            || isImplementorOf(event.getListenerType(), proposedChange));
    }

    /*
     * This method looks if a class is not a sublclass or an implementation
     * the interface described by the class object superClass
     * @param subClass the class to study
     * @param superClass the potential superClass
     * @return True if the subClass i
     */
    private boolean isImplementorOf(Class subClass, Class superClass) {

        Class studyClass = subClass;

        while (studyClass!=null) {

            // Look first if the listener type is not the interface itself
            if (studyClass==superClass)
               return  true;

            // get the interfaces directly implemented by this object
            Class interfaces[] = studyClass.getInterfaces();
            for (int j=0;j<interfaces.length;j++) {
                if (interfaces[j] == superClass)
                   return true;
            }
            studyClass = studyClass.getSuperclass();
        }
        return false;
    }

    /*
     * Looks if the class object is suitable for being uncracked for event
     * delivery byt the OLE layer
     * @param clazz Class object to study
     * @return True if the class should be uncracked
     */
    public boolean isSuitableForUncracking(Class clazz) {
        try {
            return isImplementorOf(clazz, Class.forName("java.util.EventObject"));
        } catch(ClassNotFoundException e) {
            return false;
        }
    }

    /*
     * Construct and return the information string used by the bridge at run-time
     * to avoid relying on the introspection. Currently, just indicate the 
     * persistence preference of the JavaBean component.
     * @return infomation string
     */
    public String getInformationString() {
        Object b =  beanInfo.getBeanDescriptor().getValue("hidden-state");
	if (b == Boolean.TRUE)
	    return "{{CDG}} P"; // Persistence
	else
	    return "{{CDG}} C"; // Code Generation
		
    }

    private BeanInfo                beanInfo;
    private EventSetDescriptor[]    defaultEvents;
    private EventSetDescriptor[]    extendedEvents;
    private EventSetDescriptor      propChange;
    private EventSetDescriptor      vetoChange;
    private Class                   beanClass;

 }
