/*
 * @(#)OleBeanInfo.java	0.01 96/12/08
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.2
 *
 */

package sun.beans.ole;

import java.lang.Throwable;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.URL;
import java.io.*;
import sun.beanbox.*;
import java.beans.*;
import java.util.*;
import java.lang.reflect.Method;
import sun.beans.ole.event.EventManager;
import sun.beans.ole.event.EncapsulatedEventInformation;

/*
 * OleBeanInterface
 *
 * This interface class represent the bridge on the Java Side
 * between an ActiveX and the Bean object.
 * Most of the method implementation is a direct forwarding of
 * the message to the Bean component. This is for keeping only
 * one object as the responsible for this interface. By modifying
 * the registry file, one can subclass/rewrite this class to
 * add/change behaviours. The line to change in the registry file is
 * HKEY_CLASSES_ROOT\{UID}\InterfaceClass = sun/beans/ole/OleBeanInterface
 *
 *   @author Jerome Dochez
 *   @version 1.0
 */

public class OleBeanInterface implements ObjectProxy {

    /*
     * Static Method for adding a new bean running within this instance of the
     * ActiveX bridge. The Bean can be stored in a jar file and accessible through
     * a .class file or a .ser file (serialized format). The jar file can be null
     * if the bean is to be loaded using the system loader (accessible though the 
     * CLASSPATH) and the beanName must be the class name.
     * @param jarFileName  jar file name 
     * @param beanName    .class or .ser file name.
     * @parm handle       COMObject instance reference
     */
    private static Object addBean(String jarFileName, String beanName,int handle)
	throws Exception {

        OleBeanInterface newOB;

        newOB = new OleBeanInterface(jarFileName, beanName, handle);
        if (newOB.isValid())
            return newOB;
        return null;
    }

    /*
     * Static Method for loading a new bean from its persisted storage
     * @param jarFileName  jar file name 
     * @param input        input stream containing the persisted bean
     * @parm handle        COMObject instance reference
     */
    private static Object loadBean(String jarFileName, String beanName, 
				  java.io.InputStream input, int handle) 
	throws Exception {

        OleBeanInterface newOB = 
	    new OleBeanInterface(jarFileName, beanName, input, handle);
        if (newOB.isValid())
            return newOB;
        return null;
    }

     private void setBackground(java.awt.Color c) {
	 Object bean = getInterfacedObject();
	 if (bean instanceof java.awt.Component)
	     ((java.awt.Component) bean).setBackground(c);
     }

    /**
     * Constructor
     *
     * This method is called from the ActiveX upon creation of a new
     * ActiveX object.
     * @param  jarFileName Jar File Name
     * @param  beanName    Name of the class implementing the Bean
     * @param  handle      Take the OLE Object handle as parameter
     */
    protected OleBeanInterface(String jarFileName, String beanName, int handle) {

        Class beanClass;

        this.beanName = beanName;
	Object bean;	
	  
	if ((jarFileName!=null) && (!jarFileName.equals("")))
	    beanLoader = new JarBeanLoader(jarFileName);
	else
	    beanLoader = new SystemBeanLoader();
	  
	// Load from the BeanLoader instance
	try {
	    bean = beanLoader.initBean(beanName);
	} catch (Throwable e) {
	    notifyException(getMessage("error.beanclass") + beanName);
	    return;
	}
	setBean(bean, handle);
    }

    /**
     * Constructor
     *
     * This method is called from the ActiveX upon creation of a new
     * ActiveX object.
     *
     *   @param jar file name
     *   @param bean name in the jar file or from the CLASSPATH
     *   @param The persisted Bean
     *   @param COM Object reference
     */
    protected OleBeanInterface(String jarFileName, String beanName, 
			    java.io.InputStream input, int handle) {

	Object bean = null;

	if (jarFileName!=null)
	    beanLoader = new JarBeanLoader(jarFileName);
	else
	    beanLoader = new SystemBeanLoader();
	         
	try {
	    bean = beanLoader.loadBean(beanName, input);
	} catch(Throwable e) {	 
	    new ExceptionReporter(e, getMessage("error.loading") + beanName);
	    return;
	}
	
	if (bean!=null) {
	    this.beanName= beanName;
	    setBean(bean,handle);
	}
    }

    protected OleBeanInterface(Object bean, int handle) {
	setBean(bean, handle);
    }

    /**
     * @return True if this is a valid OleBeanInterface
     */
    public boolean isValid() {
        return (bean!=null);
    }

    /**
     * Called by the OLE environment when he is finished with this
     * Object. Remove it from the list of external objects and wait
     * peacefully until the GC does the job
     */
    public void release() {
	JavaObjectInterface.clearAllCaches();
        bean=null;
        handle=0;
    }

    /**
     * Finalize is being called, we are GCted. At this time, the
     * reference counting on us should be down to zero, which mean
     * that handle should have been reset.
     * Check if handle is null, if not complain
     */
    public void finalize() throws Throwable {
        if (handle!=0) 
            nativeRelease();        
    }

    /*
     * <p>
     * Register all source event interfaces with the bridge run-time
     * handlers
     * </p>
     *
     * @param interfaceDesc the list of interfaces and registration/
     * deregistration methods
     *
     * @param connectionPoint OLE interface pointer
     *
     * @param propertyNotify OLE property notification mechanism
     *
     * @return success or failure
     */
    public boolean addCustomEventListeners(String interfacesDesc,
					   int connectionPoint,
					   int propertyNotify) {

	Object bean = getInterfacedObject();
	if (bean==null)
	    return true;

	Vector v =  new Vector();

	Vector classes = new Vector();
	Vector registrationV = new Vector();
	Vector unregistrationV = new Vector();

	try {
	String s = interfacesDesc;
	int i;
	do {
	    i = s.indexOf(' ');
	    String className = s.substring(0,i);
	    s = s.substring(i+1);
	    i = s.indexOf(' ');
	    String registrationMethod = s.substring(0,i);
	    s = s.substring(i+1);
	    i = s.indexOf(' ');
	    String deregistrationMethod;
	    if (i!=-1) {
		deregistrationMethod = s.substring(0, i);
		s = s.substring(i+1);
	    }
	    else 
		deregistrationMethod = s;
 
	    Class c;
	    try {
		if (beanLoader==null) 
		    if (bean.getClass().getClassLoader()==null)
			// loading trough the classpath
			c=Class.forName(className);
		    else
			c = bean.getClass().getClassLoader().loadClass(className);		
		else
		    c = beanLoader.loadClass(className);
	    } catch (ClassNotFoundException e) {
		System.err.println(getMessage("error.beanclass") + className);
		return false;
	    }

	    Class[] parms = { c };
	
	    EncapsulatedEventInformation eei = new EncapsulatedEventInformation(
		    c,
		    bean.getClass().getMethod(registrationMethod, parms),
		    bean.getClass().getMethod(deregistrationMethod, parms));

	    v.addElement(eei);
	} while (i!=-1);

	EncapsulatedEventInformation[] interfacesInfo = new EncapsulatedEventInformation[v.size()];
	v.copyInto(interfacesInfo);

	myEventManager = new EventManager(interfacesInfo,getInterfacedObject() );
	customListener = new OleEncapsulatedEventListener(connectionPoint);
	setEventListenersActive(true);


	} catch (Exception e) {
	    e.printStackTrace();
	    return false;
	}
	return true;

    }
    /*
     * <p>
     * Register the bridge to the source interfaces produced by the
     * targeted JavaBean component. 
     * </p>
     *
     * @parm interfacesDesc      String describing the source interfaces. 
     * Each source interface is defined by the interface name and the 
     * registration/unregistration method.
     * 
     * @param connectionPoint	Ole interface pointer for custom event
     *
     * @param propertyNotify	Ole Interface pointer for property events firing
     *
     * @return true if success
     */
    public boolean addPropertyEventListener(String interfacesDesc, 
					    int connectionPoint,
					    int propertyNotify) 
    {

	Object bean = getInterfacedObject();

	String s = interfacesDesc;
	int i = s.indexOf(' ');
	String className = s.substring(0,i);
	s = s.substring(i+1);
	i = s.indexOf(' ');
	String registrationMethod = s.substring(0,i);
	s = s.substring(i+1);
	i = s.indexOf(' ');
	String unregistrationMethod = s;

 
	Class c;
	try {
	    c = Class.forName(className);
	} catch (ClassNotFoundException e) {
	    System.err.println(getMessage("error.loadclass") + className);
	    return false;
	}

	Class[] parms = { c };

	OlePropertyEventsListener l;
	if (c == java.beans.PropertyChangeListener.class) {
	    changeListener = 
		    new OlePropertyChangeListener( propertyNotify, 
						   connectionPoint);
	    l = changeListener;
	}
	else {
	    vetoableListener =
		    new OleVetoableChangeListener( propertyNotify,
						   connectionPoint);
	    l = vetoableListener;
	}

	Method r,u;
	try {
	    r = bean.getClass().getMethod(registrationMethod, parms);
	    u = bean.getClass().getMethod(unregistrationMethod, parms);
	} catch(Exception e) {
	    e.printStackTrace();
	    return false;
	}

	l.setRegistrationMethodsPair(r,u);
	l.setListenerActive(bean, true);
	return true;
    }

    /*
     * Open the server application in a server window. OutOfPlaceActivation
     * @return True if success
     */
    public boolean openOutOfPlaceFrame() {
	outFrame = new OutOfPlaceFrame(this);
	outFrame.add(getBeanUI());
	outFrame.pack();
	outFrame.show();
	return true;
    }

    /*
     * Set the Window Title and Document Name, Used in OutOfPlace activation
     * @param title Application Title
     * @param docName Document Title
     */
    public void setHostNames(String title, String docName) 
	throws IllegalAccessException {

	if (outFrame != null)
	    outFrame.setHostNames( title, docName);
    }

    /*
     * Open the customizer associated with this bean.
     * @return True if the customizer could be opened
     */
    public boolean openCustomizer(String customizerClass) {

        if (customizerClass==null)
	    return false;

        try {
	    java.beans.Customizer customizer = (java.beans.Customizer) 
		bean.getClass().getClassLoader().loadClass(customizerClass).newInstance();
            customizer.setObject(bean);
	    sun.beans.ole.CustomizerDialog dialog;
            dialog = new sun.beans.ole.CustomizerDialog(new Frame(), 
						      customizer, bean);
	    dialog.addWindowListener(getWindowListener());
        } catch (Throwable e) {
            notifyException(e, getMessage("error.custmoizer"));
            return false;
        }

	return true;
    }

    /*
     * Return a adaptor class that implements the java.awt.WindowListener
     * interface to spy the lifetime of the customizer dialog. When the 
     * customizer is closed, the native ActiveX bridge must be notified 
     * so it can notify the OLE container of the changes
     */
    private WindowListener getWindowListener() {

	return new java.awt.event.WindowListener() {

	    public void windowClosed(WindowEvent e) 
		{
		    viewChanged();
		}
	
	    public void windowClosing(WindowEvent e) {
		e.getWindow().dispose();
	    }
	    public void windowDeiconified(WindowEvent e) {}
	    public void windowIconified(WindowEvent e) {}
	    public void windowOpened(WindowEvent E) {}
	    public void windowActivated(WindowEvent e) {}
	    public void windowDeactivated(WindowEvent e) {}
	};
    }

    /*
     * Basic Serialization/Externalization implementation for the Bean object
     * @parm os Stream to use for persisting
     */
    public synchronized boolean saveBean(java.io.OutputStream os) {

        try {

	    // Notify we are being peristed
	    beforeSave();

	    Object bean = getInterfacedObject();

            // Create the serialization medium
            java.io.ObjectOutputStream s = new java.io.ObjectOutputStream(os);

            // Remove the OLE Listeners from the list of listeners since it
		   // makes no sense to serialize the OLE Listeners
	    boolean oldEventListenerFlag = eventListeners;
	    if (eventListeners) 
		setEventListenersActive(false);
	    	    
		s.writeObject(bean);
        s.close();
	    
	    if (oldEventListenerFlag) 
		setEventListenersActive(true);
	    	   
        } catch (Throwable e) {
            new ExceptionReporter(e,getMessage("error.persisting") + beanName);
            return false;
        }
	return true;
    }

    /*
     * <p>
     * Called by the save framework to prepare the object for saving
     * </p>
     */
    protected void beforeSave() {
    }
     /* 
      * Register or Unregister the bridge Events listener interfaces to the
      *  bean
      * @param flag true to register, false to unregister
      */
    public synchronized boolean setEventListenersActive(boolean flag) {

	Object bean = getInterfacedObject();

	eventListeners = flag;
	if (myEventManager!=null) {
	    if (flag)
		myEventManager.addEncapsulatedEventListener(bean, customListener);
	    else
		myEventManager.removeEncapsulatedEventListener(bean, customListener);
	}

	if (changeListener!=null)
	    changeListener.setListenerActive(bean, flag);

	if (vetoableListener != null)
	    vetoableListener.setListenerActive(bean, flag);

	return true;
    }

    /*
     * @return the instance of the automated object
     */			     
    public Object getInterfacedObject() {
	return bean;
    }


    /*
     * return the Dispatch implementation responsible for provideing access
     * to the Bean methods and properties
     *
     * @return Dispatch implementation
     */
    public Object getDispatchImpl() {
        return new DispatchImpl(getInterfacedObject());
    }

    /*
     * Return a string representation of the persistence information
     * of a particular property of the JavaBean component. This string
     * is obtained by first trying to get a property editor for this 
     * property and use the getAsText method. If no property editor exists
     * or the getAsText returns NULL, null is returned
     *
     * @param property reference
     * @return textual persistence for that property
     */
    public static String getPropertyAsText(Object property) {
	
	// If property is null, just return
	if (property==null)
	    return null;
	
	// Get the property editor for this property
	PropertyEditor pe;
	pe = PropertyEditorManager.findEditor(property.getClass());
	
	// A property editor exists for this property, try to use
	// the getAsText method.
	if (pe!=null) {
	    pe.setValue(property);
	    String result = pe.getAsText();
	    if (result!=null)
		return result;
	}
	return null;
    }

    /*
     * Return a byte array representation of the persistence information
     * of a particular property of the JavaBean component. The byte array
     * is the serialized form of the property. 
     *
     * @param property reference
     * @return textual persistence for that property
     */
    public static byte[] getPersistedProperty(Object property) {

	// Either there is no property editor for that property or 
	// the getAsText / setAsText is not implemented by this property
	// editor, use serialization
	ByteArrayOutputStream bos = new ByteArrayOutputStream();
	
	// Create the serialization medium
	if (property instanceof ObjectProxy) {
	    ObjectProxy p = (ObjectProxy) property;
	    try {
		java.io.ObjectOutputStream s;
		s = new java.io.ObjectOutputStream(bos);	
		s.writeObject(p.getInterfacedObject());
		s.flush();
	    } catch (java.io.IOException e) {
		System.err.println(getMessage("error.propertypersistence")  
                                   + p.getInterfacedObject());
		return null;
	    }
	    return bos.toByteArray();   
	} else
	    return null;
    }

    /*
     * Set a property value from its textual representation. Use the 
     * property editor to do the string to property value translation
     *
     * @param property reference
     * @param value is the textual representation of the string
     * @return new property value
     */
    public static Object setPropertyAsText(Object property, String value) {
	
	// If property is null, just return
	if (property==null)
	    return null;
	
	// Get the property editor for this property
	PropertyEditor pe;
	pe = PropertyEditorManager.findEditor(property.getClass());
	
	// A property editor exists for this property, try to use
	// the getAsText method.
	if (pe!=null) {
	    try {
		pe.setAsText(value);
	    } catch (IllegalArgumentException e) {
		System.err.println(getMessage("error.textproperty") 
				   + value);
		return null;
	    }
	    return pe.getValue();
	}
	return null;
    }

    /*
     * @return an object that was persisted using persistence APIs
     *
     */
    public static Object getPersistedValue(byte[] bytes) {
	
	// If persisted bytes are sull, just return
	if (bytes==null)
	    return null;
	
	ByteArrayInputStream bis = new ByteArrayInputStream(bytes);

	try {
	    ObjectInputStream ois = new ObjectInputStream(bis);
	    Object b = ois.readObject();
	    JavaObjectInterface c = new JavaObjectInterface(b);
	    return c;
	} catch (Throwable e) {
	    System.err.println(e);
	    return null;
	}
    }

    /* 
     * Set the bean we are interfacing
     *
     * @param bean JavaBean component
     * @param handle Native ActiveX COM interface handle
     */
    private void setBean(Object bean, int handle) {
	this.bean = bean;
	this.handle = handle;
    }


    /**
     * notify of exception in the Ole Bean Bridge
     *
     * parameters
     *  @param str the exception description
     */
    public static void notifyException(String str) {
    
	String messages[] = new String[1];
    
	messages[0] = str;

	// Create a dummy frame to display the dialog box
	       Frame frame = new Frame();
	frame.pack();
    
	InformationDialog warn = 
	    new InformationDialog(frame, 
				  getMessage("fatal.error"), 
				  messages);
    
    }

    /**
     * notify of exception in the Ole Bean Bridge and wait for user
     * aknowledgement
     *
     * parameters
     *  @parm  e   the exception
     *  @param str the exception description
     */
    public void notifyException(Throwable e, String msg) {

	ExceptionReporter reporter = new ExceptionReporter(e, msg);
	return;
    
    }

    /*
     * <p>
     * Get a internationalized string form the resource file
     * </p>
     * @param key of the resource
     * @return the localized resource
     */
    public static String getMessage(String key) {
	try {
	    return rb.getString(key);
	} catch (java.util.MissingResourceException ex) {
	    return key;
	}
    }
  
    private java.awt.Component getBeanUI() {
	if (Beans.isInstanceOf(bean, Component.class))
	    return (java.awt.Component) Beans.getInstanceOf(bean, Component.class);
        else return null;
    }

    /**
     * ActiveX gateways functions
     */
    public native void oleShowWindow(boolean flag);
    protected native void nativeRelease();
    public native void viewChanged();

    // The BeanLoader is used to load the bean from its storage (jar file...)
    private BeanLoader    beanLoader; 
 
    // Handle to the bean object itself
    private Object       bean; 

    // Handle to the Ole Object responsible for the interface
    protected int       handle=0; 

    // Class name of the bean we interface
    public String       beanName;  

    // Out Of Place frame if we are out of place active
    public OutOfPlaceFrame             outFrame;   
   
    // State of event listeners registration
    private boolean eventListeners = false;      

    private OleEncapsulatedEventListener customListener;
    private EventManager myEventManager;
    private OlePropertyChangeListener changeListener;
    private OleVetoableChangeListener vetoableListener;
    private static java.util.ResourceBundle rb = 
       java.util.ResourceBundle.getBundle("sun.beans.ole.resources.ActiveX");
}

