/*
 * @(#)OleBeanInfo.java	1.2 98/06/02
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;
import java.lang.*;

/*
 * OleDispatchInterface
 *
 * This class is responsible for allocating, storing and retrieal of java
 * values into OLE automation DISPARAMS structure and translating between
 * java values and C values. Mainly implemented in C++, in the ActiveX
 * implementation
 *
 * @author Jerome Dochez
 * @version 1.2
 */

public class OleDispatchInterface {

    public final static int READ_ONLY   = 1;
    public final static int READ_WRITE  = 2;
    public final static int UNWRAP_RETURN_VALUE = 4;

   /* 
    * Calculate the number of parameters allocated in the structure 
    * @param disparams pointer reference to an OLE DISPPARAMS struct
    * @return the number of parameters stored in the structure
    */
    public native int parmsNumber(int disparams);

    /*
     * store a java type inside an OLE DISPPARAMS struture used by 
     * automation methods. The Disppparams strucure act as an array of
     * elements stored and retrived with an index.
     * @param diparams pointer reference to the OLE structure
     * @param index index within the structure to store the element
     * @param value the java object or native type to store
     * @param flag indicate if the object is passed by value or reference
     */
    public void storeParm(int disparams, int index, Float value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.floatValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Integer value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.intValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Double value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.doubleValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Character value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.charValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Boolean value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.booleanValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Long value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.intValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, long value, int flag) {
        storeParm(disparams, index, (int) value, flag);
    }
    public void storeParm(int disparams, int index, Byte value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.floatValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Short value, int flag) {
	if ((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)
        	storeParm(disparams, index, value.shortValue(), flag);
	else
		storeParm(disparams, index, (Object)value, flag);
    }
    public void storeParm(int disparams, int index, Dispatch value, int flag) {
        nativeStoreParm(disparams, index, value, flag);
    }
    public void storeParm(int disparams, int index, Object value, int flag) {
        if (value==null) 
            nativeStoreParm(disparams, index, null, flag);
	if (value.getClass().isArray()) {
	    Object[] arrayValue = (Object[]) value;
	    storeParm(disparams, index, arrayValue, flag);
	    return;
	}
	if (value instanceof java.lang.String) {
	    storeParm(disparams, index, (java.lang.String) value, flag);
	    return;
	}
	if (value instanceof java.awt.Font) {
	    storeParm(disparams, index, (java.awt.Font) value, flag);
	    return;
	}
	if (value instanceof java.lang.Integer &&
		((flag & UNWRAP_RETURN_VALUE) == UNWRAP_RETURN_VALUE)) {
	    storeParm(disparams, index, ((java.lang.Integer) value).intValue(), flag);
	    return;
	}
        JavaObjectInterface dispatch = new JavaObjectInterface(value);
        nativeStoreParm(disparams, index, dispatch, flag);
        return;	
    }


    public void storeParm(int disparams, int index, Float[] value, int flag) {
	    float[] intrinsicArray = new float[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].floatValue();
	    storeParm(disparams, index, value, flag);
    }

    public void storeParm(int disparams, int index, Double[] value, int flag) {
	    double[] intrinsicArray = new double[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].doubleValue();
	    storeParm(disparams, index, value, flag);
    }

    public void storeParm(int disparams, int index, Character[] value, int flag) {
	    char[] intrinsicArray = new char[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].charValue();
	    storeParm(disparams, index, value, flag);
    }

    public void storeParm(int disparams, int index, Boolean[] value, int flag) {
	    boolean[] intrinsicArray = new boolean[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].booleanValue();
	    storeParm(disparams, index, value, flag);
    }
    public void storeParm(int disparams, int index, Byte[] value, int flag) {
	    byte[] intrinsicArray = new byte[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].byteValue();
	    storeParm(disparams, index, value, flag);
    }
    public void storeParm(int disparams, int index, Long[] value, int flag) {
	    int[] intrinsicArray = new int[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].intValue();
	    storeParm(disparams, index, value, flag);
    }
    public void storeParm(int disparams, int index, Integer[] value, int flag) {
	    int[] intrinsicArray = new int[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].intValue();
	    storeParm(disparams, index, value, flag);
    }
    public void storeParm(int disparams, int index, Short[] value, int flag) {
	    short[] intrinsicArray = new short[value.length];
	    for (int i=0;i<value.length;i++)
	        intrinsicArray[i] = value[i].shortValue();
	    storeParm(disparams, index, value, flag);
    }

    public void storeParm(int disparams, int index, 
			  Dispatch[] value, int flag) {
      nativeStoreArrayParm(disparams, index, value, flag);
    }

    public void storeParm(int disparams, int index, Object[] value, int flag) {
	JavaObjectInterface[] interfacesArray=
	    new JavaObjectInterface[value.length];
	for (int i=0;i<value.length;i++) {
	    Object dispatchable = value[i];
            if (dispatchable==null)
                interfacesArray[i]=null;
            else {
		JavaObjectInterface dispatch =
		    new JavaObjectInterface(dispatchable);
		interfacesArray[i] = dispatch;
	    }
        }
        nativeStoreArrayParm(disparams, index, interfacesArray, flag);
    }


   /*
    * return the java reference or 
    */
    public Object getObjectParm(int disparams, int index) {

       Object proxy = nativeGetObjectParm(disparams, index);
       if (proxy instanceof ObjectProxy)
	  return ((ObjectProxy) proxy).getInterfacedObject();
        else 	     
          return proxy;	  
    }

    public Double getDoubleObjectParm(int disparams, int index) {
        return new Double(doubleParm(disparams, index));
    }
    public Float getFloatObjectParm(int disparams, int index) {
        return new Float(floatParm(disparams, index));
    }
    public Character getCharacterObjectParm(int disparams, int index) {
        return new Character(charParm(disparams, index));
    }
    public Integer getIntegerObjectParm(int disparams, int index) {
        return new Integer(intParm(disparams, index));
    }
    public Long getLongObjectParm(int disparams, int index) {
        return new Long(longParm(disparams, index));
    }
    public Boolean getBooleanObjectParm(int disparams, int index) {
        return new Boolean(booleanParm(disparams, index));
    }
    public Short getShortObjectParm(int disparams, int index) {
        return new Short(shortParm(disparams, index));
    }
    public Byte getByteObjectParm(int disparams, int index) {
        return new Byte(byteParm(disparams, index));
    }
  
    public native void storeParm(int disparams, int index, float value, int flag);
    public native void storeParm(int disparams, int index, double value, int flag);
    public native void storeParm(int disparams, int index, char value, int flag);
    public native void storeParm(int disparams, int index, byte value, int flag);
    public native void storeParm(int disparams, int index, boolean value, int flag);
    public native void storeParm(int disparams, int index, short value, int flag);
    public native void storeParm(int disparams, int index, int value, int flag);
    public native void storeParm(int disparams, int index, String value, int flag);
    public native void storeParm(int disparams, int index,
				 java.awt.Font value, int flag);
    public native void storeParm(int disparams, int index, 
				 java.awt.Color value, int flag);
    public native void storeParm(int disparams, int index, 
				 sun.plugin.javascript.ocx.JSObject value, int flag);
  
    // Array Storage
    public native void storeParm(int disparams, int index, float[] value, int flag);
    public native void storeParm(int disparams, int index,
				 double[] value, int flag);
    public native void storeParm(int disparams, int index, char[] value, int flag);
    public native void storeParm(int disparams, int index, byte[] value, int flag);
    public native void storeParm(int disparams, int index,
				 boolean[] value,int flag);
    public native void storeParm(int disparams, int index, short[] value, int flag);
    public native void storeParm(int disparams, int index, int[] value, int flag);
    public native void storeParm(int disparams, int index, 
				 String[] value, int flag);
    public native void storeParm(int disparams, int index, 
				 java.awt.Font[] value, int flag);
    public native void storeParm(int disparams, int index, 
				 java.awt.Color[] value, int flag);

    protected native void nativeStoreArrayParm(int disparams, int index, 
					       Object[] value, int flag);
    protected native void nativeStoreParm(int disparams, int index,
					  Object value, int flag);
    public native void nativeStoreParm(int dispresult, Object value,
				       String className, boolean isArray, boolean needUnwrap);

    public native double doubleParm(int disparams, int index);
    public native float floatParm(int disparams, int index);
    public native char charParm(int disparams, int index);
    public native byte byteParm(int disparams, int index);
    public native boolean booleanParm(int disparams, int index);
    public native short shortParm(int disparams, int index);
    public native int intParm(int disparams, int index);
    public long longParm(int disparams, int index) {
	return (long) intParm(disparams, index);
    }
  
    public native String getStringObjectParm(int disparans, int index);
    protected native Object nativeGetObjectParm(int disparans, int index);
    public native java.awt.Font getFontObjectParm(int disparans, int index);
    public native java.awt.Color getColorObjectParm(int disparans, int index);
  
    // Array Extraction
    public native double[] doubleArrayParm(int disparams, int index);
    public native float[] floatArrayParm(int disparams, int index);
    public native char[] charArrayParm(int disparams, int index);
    public native byte[] byteArrayParm(int disparams, int index);
    public native boolean[] booleanArrayParm(int disparams, int index);
    public native short[] shortArrayParm(int disparams, int index);
    public native int[] intArrayParm(int disparams, int index);
    public native long[] longArrayParm(int disparams, int index);

    public Object[] getObjectArrayParm(int disparams, int index) {
	Object[] proxies= nativeGetObjectArrayParm(disparams, index);
	Object[] array= new Object[proxies.length];
        // Fill in
	for (int i=0;i<proxies.length;i++)
	    if (proxies[i] instanceof ObjectProxy)
		array[i]=((ObjectProxy) proxies[i]).getInterfacedObject();
	    else if (proxies[i] instanceof java.lang.String ||
		     proxies[i] instanceof java.lang.Integer)
		array[i] = proxies[i];
	    else 
		array[i] = null;
	return array;
    }


    public native Object[] nativeGetObjectArrayParm(int disparams, int index);

    public String[] getStringObjectArrayParm(int disparams, int index) {
	Object[] result = nativeGetObjectArrayParm(disparams, index);
	if (result==null)
	    return null;
	if (result instanceof String[])
	    return (String[]) result;
	else return null;
    }
    public java.awt.Font[] getFontObjectArrayParm(int disparams, int index) {
	Object[] result = nativeGetObjectArrayParm(disparams, index);
	if (result==null)
	    return null;
	if (result instanceof java.awt.Font[])
	    return (java.awt.Font[]) result;
	else return null;
    }
    public java.awt.Color[] getColorObjectArrayParm(int disparams, int index) {
	Object[] result = nativeGetObjectArrayParm(disparams, index);
	if (result==null)
	    return null;
	if (result instanceof java.awt.Color[])
	    return (java.awt.Color[]) result;
	else return null;
    }


    public Double[] getDoubleObjectArrayParm(int disparams, int index) {
        // get the array of intrinsic types
        double[] intrinsicArray = doubleArrayParm(disparams, index);
	// Allocate
	if (intrinsicArray==null) return null;
	Double[] array= new Double[intrinsicArray.length];
	// Fill in
        for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Double(intrinsicArray[i]);
	return array;
    }
    public Float[] getFloatObjectArrayParm(int disparams, int index) {
	// get the array of intrinsic types
	float[] intrinsicArray = floatArrayParm(disparams, index);
	// Allocate
	if (intrinsicArray==null) return null;
	Float[] array= new Float[intrinsicArray.length];
	// Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	     array[i]=new Float(intrinsicArray[i]);
	return array;
    }
    public Character[] getCharacterObjectArrayParm(int disparams, int index) {
	// get the array of intrinsic types
	char[] intrinsicArray = charArrayParm(disparams, index);
	// Allocate
	if (intrinsicArray==null) return null;
	Character[] array= new Character[intrinsicArray.length];
	// Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Character(intrinsicArray[i]);
	return array;
    }
    public Integer[] getIntegerObjectArrayParm(int disparams, int index) {
	// get the array of intrinsic types
	int[] intrinsicArray = intArrayParm(disparams, index);
	// Allocate
	if (intrinsicArray==null) return null;
	    Integer[] array= new Integer[intrinsicArray.length];
	    // Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Integer(intrinsicArray[i]);
	return array;
    }

    public Byte[] getByteObjectArrayParm(int disparams, int index) {
	// get the array of intrinsic types
	byte[] intrinsicArray = byteArrayParm(disparams, index);
	// Allocate
	if (intrinsicArray==null) return null;
	Byte[] array= new Byte[intrinsicArray.length];
	// Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Byte(intrinsicArray[i]);
	return array;
    }

    public Boolean[] getBooleanObjectArrayParm(int disparams, int index) {
	// get the array of intrinsic types
	boolean[] intrinsicArray = booleanArrayParm(disparams, index);
	// Allocate
        if (intrinsicArray==null) return null;
	Boolean[] array= new Boolean[intrinsicArray.length];
        // Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Boolean(intrinsicArray[i]);
	return array;
    }

    public Short[] getShortObjectArrayParm(int disparams, int index) {
        // get the array of intrinsic types
        short[] intrinsicArray = shortArrayParm(disparams, index);
        // Allocate
	if (intrinsicArray==null) return null;
	Short[] array= new Short[intrinsicArray.length];
	// Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Short(intrinsicArray[i]);
	return array;
    }

    public Long[] getLongObjectArrayParm(int disparams, int index) {
        // get the array of intrinsic types
        long[] intrinsicArray = longArrayParm(disparams, index);
	// Allocate
	if (intrinsicArray==null) return null;
	Long[] array= new Long[intrinsicArray.length];
	// Fill in
	for (int i=0;i<intrinsicArray.length;i++)
	    array[i]=new Long(intrinsicArray[i]);
	return array;
    }

    /* 
     * Allocate OLE structures for storing a number of Java parameters
     * @parm size the number of parameters to allocate for
     */
    public native int allocateParmsFor(int number);

    /*
     * Free an OLE structure that was allocated by allocateParmsFor
     * @parm dispparams OLE structure pointer
     */
    public native void freeParms(int dispparams);

}

