/*
 * @(#)OleEmbeddedFrame.java	1.43 01/02/09
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */
package sun.beans.ole;

import sun.awt.windows.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.Beans;
import java.applet.Applet;
import sun.plugin.SystemEventQueueUtilities;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;


/*
 * This is the java window used to inplace edit a JavaBeans component
 * An OleEmbeddedFrame does not have decorations, always live within a 
 * container window (specified by the container application).
 * The lifetime of an OleEmbeddedFrame is decided by the OLE container
 * Each OleEmbeddedWindow will itself embed a unique JavaBeans component
 *
 * @version 1.2.2
 * @date 12/15/98
 * @author Jerome Dochez
 */
public class OleEmbeddedFrame extends sun.awt.windows.WEmbeddedFrame
                              implements WindowListener
{
    private int waitingEvent = 0;

    /**
     * Create OleEmbeddedFrame instance on AWT event queue thread.
     */
    public static OleEmbeddedFrame createFrame( final int hWnd ) {
        MyRunnable myRun = new MyRunnable( hWnd );

        try {
            SystemEventQueueUtilities.invokeAndWait( myRun );
        } catch( Exception e ) {
            System.out.println( "Failed to create OCX frame: " );
            e.printStackTrace();
            return null;
        }

        return myRun.frame;
    }

    private static class MyRunnable implements Runnable {
        int hWnd;
        OleEmbeddedFrame frame;

        private MyRunnable( int hWnd ) {
            this.hWnd = hWnd;
        }

        public void run() {
            frame = new OleEmbeddedFrame( hWnd );
        }
    }


    /*
     * Build a new in place window and set its parent to the window 
     * handle specified. 
     *
     * @param nativeHandler native window implementation
     * @param hWnd Native Window handle of the parent window
     */
    protected OleEmbeddedFrame(int hWnd) 
    {
        super(hWnd);

        addWindowListener(this);
        try {
            Class c = Class.forName("sun.awt.windows.ModalityListener");
            // if we reach this point, we should be running in a jdk1.1.8 or 
            // jre 1.2.2 or superior.       
            c = Class.forName("sun.beans.ole.ModalityListenerImpl");
            Object o = c.newInstance();
            mlr = (ModalityListenerRegistration) o;
            mlr.register(OleEmbeddedFrame.this);

	    Trace.println("Registered modality listener", TraceFilter.JAVA_CONSOLE_ONLY);

        } catch (ClassNotFoundException e) {
            return;
        } catch (InstantiationException e) {
            return;
        } catch (IllegalAccessException e) {
            return;
        } 
    }

    /*
     * Specify which JavaBean component should be painted inside this
     * embedded frame window. 
     * 
     * @param bean the JavaBeans reference
     */
    public void setComponent(Component c) 
    {
	if (c!=null) {
	    // get the component original size
	    Dimension d = c.getSize();

	    // Use borderlayout, so the panel/applet/beans will always 
	    // be the same size as the frame.
	    setLayout(new BorderLayout());
      
	    // Add a panel since lightweight components seems require it
	    p = new Panel(); 
	    p.setLayout(new BorderLayout());
	    p.setBackground(c.getBackground());
	    p.add(c);
      
	    add(p);

	    // Set the size to the original value
	    if (d.width!=0 && d.height!=0)
		setSize(d);
    
	    // Now that we have a window, start if this is a applet !
	    if (Beans.isInstanceOf(c, Applet.class)) {
		Applet apl = (Applet) Beans.getInstanceOf(c, Applet.class);
		apl.start();
	    }
	    c.requestFocus();

	}
    }

    /*
     * Set the JavaBeans component as the embedded object.If the JavaBeans is
     * a visible bean, it supports java.awt.Component by direct inheritance or
     * by aggregation. If the bean is an invisible bean, then this method 
     * does nothing.
     * 
     * @param c Component
     */
    public void setBean(final Object component) {
        try {
            
            SystemEventQueueUtilities.invokeAndWait( new Runnable() {
                public void run() {
                    try {
                        c =(java.awt.Component) Beans.getInstanceOf(component, 
							Component.class);
                    } catch (ClassCastException e) {
                        return;
                    }
                    if (c!=null)
                        setComponent(c);
                 }
            } );

        } catch( Exception e ) {
            System.out.println( "Failed to setBean() : " );
            e.printStackTrace();
        }
    }

    /*
     * print all components in this embedded frame
     * 
     * @param g the Graphcis object to use for printing
     */
    public void printComponents(Graphics g) {
	/*
	  Dimension d = getSize();
	  java.awt.Image offScreen = super.createImage(d.width, d.height);
	  c.paint(offScreen.getGraphics());
	  Graphics gr = g.create(0,0,d.width, d.height);
	  gr.drawImage(offScreen, 0, 0, null, null);
	  gr.dispose();
	  */
	super.printComponents(g);
    }

    /*
     * <p>
     * Called by the native layers, when the container is requesting for 
     * closing the ActiveX component. 
     * We post the WINDOW_CLOSING event on the AWT window thread
     * </p>
     */
    public void destroy(int event)   
    {
	// Hide the frame first. This is very important
	// to hide the frame before calling dispose().
	// Otherwise, some race condition with repaint()
	// may be resulted.
	//
	setVisible(false);

	// Close and remove frame.
        setEnabled(false);

	// Set the waiting event for synchronization
	setWaitingEvent(event);

        WindowEvent we = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
        EventQueue eq = getToolkit().getSystemEventQueue();
        eq.postEvent(we);

	waitEvent(event);

	// Reset waiting event
	setWaitingEvent(0);
    }


    /*
     * Set the size of the embedded window, if the embedded window 
     * currently embed a component, resets its size as well.
     * 
     * @param width of the embedded frame
     * @param height of the embedded frame
     */
    public void setFrameSize(final int width, final int height) 
    {
	try
	{
	    SystemEventQueueUtilities.invokeAndWait(new Runnable()
		{
		    public void run()
		    {
			if (c instanceof sun.plugin.AppletViewer) {
			    sun.plugin.AppletViewer panel = (sun.plugin.AppletViewer) c;
			    panel.setParameter("width", Integer.toString(width));
			    panel.setParameter("height", Integer.toString(height));
			}

			// Resize embedded frame
			setSize(width, height);

			// Resize panel
			if (p!= null)
			    p.setSize(width, height);

			// Resize AppletViewer/applet/beans
			if (c!=null) 
			{
			    c.setSize(width, height);

			    if (c instanceof sun.plugin.AppletViewer)
			    {
				sun.plugin.AppletViewer panel = (sun.plugin.AppletViewer) c;
				panel.appletResize(width, height);

				Applet applet = panel.getApplet();

				if (applet != null)
				    applet.resize(width, height);
			    }
			}
		    }
		});
	    }
	    catch (Throwable e)
	    {

	    }
    }


    /*
     * @return the preferred size of the JavaBeans component
     */
    public Dimension getPreferredSize() {
	if (c!=null)
	    return c.getPreferredSize();
	else
	    return null;
    }

    /*
     * <p>
     * Override the requestFocus to manually notify the applet to
     * grab the focus
     * </p>
     */
    public void requestFocus() {
	WindowEvent we = new WindowEvent(this, WindowEvent.WINDOW_ACTIVATED);
	EventQueue eq = getToolkit().getSystemEventQueue();
	eq.postEvent(we);

        if (c instanceof sun.plugin.AppletViewer) {
            Applet a = ((sun.plugin.AppletViewer) c).getApplet();
            if (a!=null) 
		a.requestFocus();
        }
    }

    /*
     * <p>
     * Dispose the window, notify the native component to release
     * </p>
     *
     */
    public void dispose() {
        int handler = nativeHandler;
        nativeHandler = 0;	
        super.dispose();
//        nativeDispose(handler);
    }


    /*
     * <p>
     * Window Listener interface implementation.
     * Do nothing but register itself for assistive technology
     * </p>
     */
    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) { }
    public void windowClosing(WindowEvent e) {
	try {
	    removeWindowListener(this);
	    if (mlr!=null)
    		mlr.unregister(this);

	    Trace.println("Unregistered modality listener", TraceFilter.JAVA_CONSOLE_ONLY);


	    // Remove all components
    	    removeAll();

            dispose();

	    if (waitingEvent != 0)
		setEvent(waitingEvent);

	} catch(Throwable ex) {
	    ex.printStackTrace();
	}
    }
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}

    /*
     * Called by AWT when it enters a new level of modality
     */
    public void modalityPushed() {

	Trace.println("Modality pushed", TraceFilter.JAVA_CONSOLE_ONLY);
	enableModeless(nativeHandler, false);
    }

    /*
     * Called by AWT when it exits a level of modality
     */
    public void modalityPopped() {

	Trace.println("Modality popped", TraceFilter.JAVA_CONSOLE_ONLY);
	enableModeless(nativeHandler, true);
    }

    /*
     * <p>
     * Native methods implementations
     * /p>
     */
    private native void enableModeless(int nativeHandler, boolean fEnable);
    private native void nativeDispose(int handler);

    private native void setEvent(int event);
    private native void waitEvent(int event);

    public void setWaitingEvent(int event)
    {
	waitingEvent = event;
    }

    /* 
     * reference to the JavaBeans component
     */
    private Component c;
    private Panel p;
    private boolean hasFocus=false;
    private int nativeHandler=0;
    private ModalityListenerRegistration mlr;

}


