/*
 * @(#)OleEncapsulatedEventListener.java	 97/10/02
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1
 *
 */

/*
 * <p>
 * sun.beans.ole.OleEncpasulatedEventListener
 * <p>
 * @version 1.1
 * @author Jerome Dochez
 *
 */
package sun.beans.ole;

import sun.beans.ole.event.EncapsulatedEventListener;
import sun.beans.ole.event.EncapsulatedEvent;
import sun.beans.ole.event.EncapsulatedEventException;

/* 
 * <p>
 * The OleEncapsulatedEventListener is responsible for receiving all
 * encapsulated event from a target JavaBean component and forward this
 * event to the native implementation of the bridge itself responsible
 * for forwarding the event to the container through the ConnectionPoint
 * mechanism
 * </p>
 */
public class OleEncapsulatedEventListener extends OleEventListener
		    implements EncapsulatedEventListener {

    /*
     * <p>
     * Construct a new EncapsulatedEventListener object with the 
     * associated native ConnectionPoint pointer.
     * </p>
     * 
     * @param connectionPoint native COM object pointer
     */
    public OleEncapsulatedEventListener(int connectionPoint) {

	super(connectionPoint);
    }

    /*
     * <p>
     * Method invoked by the encapsulated event mechanism when 
     * one of the source interface event is fired by the JavaBean
     * component on which we are registered
     * </p>
     * 
     * @param ee the encapsulated event from the dynamic adaptor.
     */
    public void encapsulatedEvent(EncapsulatedEvent ee) 
	throws EncapsulatedEventException {

	Object[] args = ee.getEventArguments();
	int dispParams = allocateParmsFor(args.length);
	for (int i=0;i<args.length;i++)
	    storeParm(dispParams, i, args[i], READ_ONLY);
	fireEncapsulatedEvent(m_connectionPoint, 
	    ee.getListenerMethodName(), dispParams);
	//freeParms(dispParams);
    }

    /*
     * <p>
     * fire an event to the native bridge implementation, the event
     * is described by its event name and the event parameters. 
     * The parameters are already packaged as OLE types
     * </p>
     * 
     * @param COM ConnectionPoint object pointer
     * @param eventName is the event name
     * @param dispParams is the event parameters as OLE types
     *
     */
    protected native void fireEncapsulatedEvent(int connectionPoint,
						String eventName,
						int dispParams);

}
