/*
 * @(#)OleInputStream.java	1.0 17/1/97
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0 
 *
 */

package sun.beans.ole;

/*
 * Class representing an input stream of bytes mapped to an Ole IStream
 * object. This class can be instanciated from native code only.
 * This class is thread sensitive, several threads might be tempted to
 * write to the ole IStream interface which is illegal. This class will
 * do the necessary underlying marshalling if several threads are trying
 * to use it.
 * @see		InputStream
 * @version	1.0
 * @author	Jerome Dochez
 */
import java.io.*;

class OleInputStream extends ByteArrayInputStream {

    /**
     * Constructor accessible only from native code
     * @param outputStream Native interface object
     * @param oletream Ole interface IStream reference
     */
    private OleInputStream(int extInterface,int oleStream ) {
        super(new byte[1]);
        count = available(extInterface,oleStream);
        buf = new byte[count];

        this.extInterface=extInterface;

	// Initialize the list of threads
	threadsInfo = new java.util.Hashtable();
	threadsInfo.put(Thread.currentThread().toString(), 
			new Integer(oleStream));

	read(extInterface, oleStream, buf,0, count);
    }

    /*
     * close the stream and all OLE ressources
     */
    public void close() throws IOException {
        for (java.util.Enumeration e = threadsInfo.elements() ;
	     e.hasMoreElements() ;) {
            Integer os;
            try {
                os = (Integer) e.nextElement();
            } catch (ClassCastException ex) {
                throw new Error(ex.toString());
            }
            detachThread(os.intValue());
        }
        super.close();
    }

    /**
     * private method for ensuring that any thread that might use this 
     * instance is known by the bridge and more specifically by the COM
     *  layers through marshalled intefaces
     */
    private synchronized int attachThread() {

	// Look if the calling thread is in the list
	String currentThread = Thread.currentThread().toString();
	Integer os;
	try {
            os = (Integer) threadsInfo.get(currentThread);
	} catch (ClassCastException e) {
            throw new Error(e.toString());
	}
	if (os==null) {
            os = new Integer(attachThread(extInterface));
            threadsInfo.put(currentThread, os);
	}
	return os.intValue();
    }

    private native int read(int cOleIS, int oleStream, byte b[],
			    int off, int len);
    private native int available(int cOleIS, int oleStream);
    private native int attachThread(int outputStream);
    private native void detachThread(int oleStream);

    // Reference to the OleInputStrean native interface
    private int         extInterface;  

    // Reference the threads allowed to use this interface
    private java.util.Hashtable    threadsInfo;
}


