/*
 * @(#)OleOutputStream.java	1.00 97/1/16
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0 
 *
 */

package sun.beans.ole;

import java.io.*;

/**
 * Class representing an output stream of bytes mapped to an Ole IStream
 * object. This class can be instanciated from native code only.
 * This class is thread sensitive, several threads might be tempted to
 * write to the ole IStream interface which is illegal. This class will
 * do the necessary underlying marshalling if several threads are trying
 * to use it.
 * @see		OutputStream
 * @version 	1.0 
 * @author	Jerome Dochez
 */
public class OleOutputStream extends ByteArrayOutputStream {

    /**
     * Constructor accessible only from native code
     * @param outputStream Native interface object
     * @param oletream Ole interface IStream reference
     */
    private OleOutputStream(int extInterface,int oleStream ) {
        super(2048);
        this.extInterface=extInterface;

       // Initialize the list of threads
       threadsInfo = new java.util.Hashtable();
       threadsInfo.put(Thread.currentThread().toString(), 
		       new Integer(oleStream));
    }


    /**
     * Writes a sub array of bytes.
     * @param b	the data to be written
     * @param off	the start offset in the data
     * @param len	the number of bytes that are written
     * @exception IOException If an I/O error has occurred.
     */
    public void flush() throws IOException {

        // Check for threading
        int oleStream = attachThread();

        // This method is overloaded for performance reason
        int hResult = write(extInterface, oleStream, buf, 0, count);
        if (hResult!=0)
            throw new IOException("IStream::write reported " + hResult);

    }

    /*
     * close the stream and all OLE ressources
     */
     public void close() throws IOException {
        for (java.util.Enumeration e = threadsInfo.elements() ;
	     e.hasMoreElements() ;) {
            Integer os;
            try {
                os = (Integer) e.nextElement();
            } catch (ClassCastException ex) {
                throw new Error(ex.toString());
            }
            detachThread(os.intValue());
        }
        super.close();
     }

    /**
     * private method for ensuring that any thread that might use this 
     * instance is known by the bridge and more specifically by the COM 
     * layers through marshalled intefaces
     */
    private int attachThread() {

       // Look if the calling thread is in the list
       String currentThread = Thread.currentThread().toString();
       Integer os;
       try {
            os = (Integer) threadsInfo.get(currentThread);
       } catch (ClassCastException e) {
            throw new Error(e.toString());
       }
       if (os==null) {
            os = new Integer(attachThread(extInterface));
            threadsInfo.put(currentThread, os);
       }
       return os.intValue();
    }

    private native int write(int cOleOS, int oleStream, byte[] b, 
			     int off, int len);
    private native int attachThread(int outputStream);
    private native void detachThread(int oleStream);

    // Reference to the OleOutputStrean native interface
    private int         extInterface;

    // Reference the threads allowed to use this interface
    private java.util.Hashtable    threadsInfo;
}
