/*
 * @(#)OlePropertyChangeListener.java	0.1 97/3/27
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;

/*
 * OlePropertyEventsListener is responsible for forwarding all property
 * events notification to the native implementation of the bridge
 * 
 * @see java.beans.PropertyChangeListener
 * @version 1.0  
 * @author Jerome Dochez
 */

import java.lang.reflect.Method;

abstract public class OlePropertyEventsListener 
{

    /*
     * Construct a new property event listener
     * 
     * @param IPropertyNotifySink responsible for forwarding the property
     * change notifications
     * @param IConnectionPoint responsible for forwarding the events
     * of the bean
     */
    public OlePropertyEventsListener(int IPropertyNotifySink, 
				     int IConnectionPoint) {
	m_propertyNotifySink = IPropertyNotifySink;
	m_connectionPoint = IConnectionPoint;
    }

    /*
     * propertyChange is invoked by the bean when one of its bound property 
     * has been changed, forward it immediately to the bridge
     *  
     * @param boolean true if this is a VetoableChange happening
     * @param propertyNotify the property notification
     * @param connectionPoint normal event delivery sink
     * @param propertyName name of the property 
     * @param evt Automatable event object
     *
     * @return true if the vetoableChange is denied
     */
    protected native boolean nativePropertyEvent(boolean veto , int propertyNotify, 
			    int connectionPoint, String propertyName,
			    Dispatch evt);

    /*
     * <p>
     * Saves the registration/unregistration methods used by the target
     * JavaBean component. This methods will be used to register and
     * unregister ourselves from the source interface
     * </p>
     * 
     * @param registrationMethod method used to register a source 
     * interface listener
     *
     * @param unregistrationMethod method used to unregister a source
     * interface listener
     */
    public void setRegistrationMethodsPair( Method registrationMethod,
					    Method unregistrationMethod) {

	this.registrationMethod = registrationMethod;
	this.unregistrationMethod = unregistrationMethod;
    }

    /*
     * <p>
     * Register or unregister this listener interface to the target
     * JavaBean component
     * </p>
     *
     * @param target the source component for this listener interface
     * 
     * @param register true to register, false to unregister
     *
     */
    public void setListenerActive(Object target, boolean register) {

	if (target==null) throw new IllegalArgumentException("Target");

	// if the state of the listener does not change, return
	if (active==register) 
	    return;
	active=register;
    
	try {
	    Object[] parms = { this };
	    if (register) 
		registrationMethod.invoke(target, parms );
	    else
		unregistrationMethod.invoke(target, parms);
	} catch (Exception e) {
	    System.err.println("Cannot " +
				(register?"enable":"disable") + 
				"property listener");
	    return;
	}

    }

    protected int m_propertyNotifySink;
    protected int m_connectionPoint;

    private Method registrationMethod;
    private Method unregistrationMethod;
    private boolean active;
}

  
