/*
 * @(#)OleVetoableChangeListener.java	0.1 97/3/27
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;

/*
 * OleVetoablehangeListener implements the VetoableChangeListener interface
 * to listen to all contrained property change events. The implementation  
 * just forward any incoming propertyChange to the native implementation of  
 * the bridge that forward to the OLE listeners.
 * 
 * @see java.beans.VetoableChangeListener
 * @version 1.0 
 * @author Jerome Dochez
 */
import java.beans.*;

public class OleVetoableChangeListener extends OlePropertyEventsListener 
implements VetoableChangeListener {

    /*
     * Construct a new property vetoable change listener
     * 
     * @param IPropertyNotifySink responsible for forwarding the property
     * change notifications
     * @param IConnectionPoint responsible for forwarding the events
     * of the bean
     */
    public OleVetoableChangeListener(int IPropertyNotifySink, 
				     int IConnectionPoint) {
	super(IPropertyNotifySink, IConnectionPoint);
    }

    /* 
     * vetoableChange is invoked by the bean when it is about to change
     * one of its bound property and notify all listeners of it. The 
     * notification is fowarded to the bridge and if the bridge return
     * true, the OLE Listeners have denied the change, so a
     * PropertyVetoException is thrown
     */
    public void vetoableChange(PropertyChangeEvent evt)
	throws PropertyVetoException {    
	    if (!nativePropertyEvent(true, m_propertyNotifySink, 
				    m_connectionPoint,
				    evt.getPropertyName(),
				    new JavaObjectInterface(evt)))
		throw new PropertyVetoException("OLE listeners denied", evt);
    }

}

  
