/*
 * @(#)OleBeanInfo.java	0.01 96/12/08
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */
package sun.beans.ole;

import java.awt.*;
import java.awt.event.*;

/**
 * OutOfPlaceFrame
 *
 * This Frame class define a editing Ole OutOfPlace Activated frame window
 *
 * Author :
 *   @author Jerome Dochez
 *   @version version 0.2        2/14/96
 */

class OutOfPlaceFrame extends Frame implements WindowListener {

    public OutOfPlaceFrame(OleBeanInterface interfacer) {


        super(OleBeanInterface.getMessage("outofplace.title"));
        this.interfacer = interfacer;
        this.setLayout(new GridLayout(1,1));

        // Let's create the Menu bar now.

        fileMenu = new Menu(OleBeanInterface.getMessage("outofplace.file"));

	MenuItem mItem;
	mItem = new MenuItem(OleBeanInterface.getMessage("outofplace.save"));
	mItem.addActionListener(this.myOtherMenuListener());
        fileMenu.add(mItem);

        fileMenu.addSeparator();
	mItem = new MenuItem(OleBeanInterface.getMessage("outofplace.import"));
	mItem.addActionListener(this.myOtherMenuListener());
        fileMenu.add(mItem);
        fileMenu.addSeparator();
        exitMenuItem = new MenuItem(
			  OleBeanInterface.getMessage("outofplace.exit"));
	exitMenuItem.addActionListener(this.myExitMenuListener());
        fileMenu.add(exitMenuItem);
        MenuBar menuBar=new MenuBar();
        menuBar.add(fileMenu);
        helpMenu = new Menu(OleBeanInterface.getMessage("outofplace.help"));

	mItem = new MenuItem(OleBeanInterface.getMessage("outofplace.about")+
			     interfacer.beanName);
	mItem.addActionListener(this.myAboutMenuListener());

        helpMenu.add(mItem);
        menuBar.add(helpMenu);
        setMenuBar(menuBar);
	addWindowListener(this);
    }

   /**
    * Called to display an About.. dialog box
    *
    */
    public void about() {

        String messages[] = new String[2];
        InformationDialog dialog;

        messages[0] = interfacer.beanName ;
        messages[1] = sun.plugin.AppletViewer.getMessage("product_name");


        dialog = new InformationDialog(this, 
			 OleBeanInterface.getMessage("outofplace.about"),
				       messages);
   }

   /**
    * Add the bean as the edited entity
    *
    * Parameters :
    *   @param comp - the bean
    */
    public Component add(Component bean) {

        this.bean = bean;
        Component res = super.add(bean);
        pack();
	setLocation(200,200);
        return res;
    }

   /**
    * the container application is notifying us of new host names to use
    *
    * Parameters :
    *   @param application - application name we are embbeded in
    *   @param objectName - compound document name
    */
    public void setHostNames(String application, String objectName) {
        this.setTitle(application + " - " + objectName);
        this.objectName = objectName;
        exitMenuItem.setLabel(OleBeanInterface.getMessage("outofplace.hostexit") +
			      objectName);
    }


    public void windowClosed(WindowEvent e) {

    }
    public void windowClosing(WindowEvent e) {
	dispose();
	interfacer.oleShowWindow(false);
    }
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e){
    }
    public void windowActivated(WindowEvent e) {} 
    public void windowDeactivated(WindowEvent e) {}
    
    ActionListener myAboutMenuListener() {
	return new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		about();
	    }
	};
    }
    
    void warnUser() {
	String messages[] = new String[3];
	messages[0] = "This is just a fake menu that you have to extend";
	messages[1] = "Look in sun.beans.ole.OutOfPlaceFrame for example";
	messages[2] = "Subclass OleBeanInterface to point to your version";
	
	new InformationDialog(this, "Not quite yet", messages);
    }

    ActionListener myOtherMenuListener() {
	return new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		warnUser();
	    }
	};
    }

    ActionListener myExitMenuListener() {
	return new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		closeWindow();
	    }
	};
    }

    /*
     * <p>
     * Close this window
     * We post the WINDOW_CLOSING event on the AWT window thread
     * </p>
     */
    public void closeWindow() {
	WindowEvent we = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
	EventQueue eq = getToolkit().getSystemEventQueue();
	eq.postEvent(we);
    }

    private Component   bean;
    private String      objectName;     // name of the containing document
    // Object responsible for interfacing me and the ActiveX
    private OleBeanInterface interfacer; 
    private MenuBar          menuBar;      // Menu bar for this window
    private Menu             helpMenu,fileMenu;
    //MenuItem to exit, change upon the name of the container app
    private MenuItem         exitMenuItem; 

}
