/*
 * @(#)OleBeanInfo.java	0.01 96/12/08
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;

/* 
 * This class hold all information necessary for generating  and compiling
 * the OLE stubs for a particular Java bean component
 */
public class StubInformation {

    /* 
     * <p>
     * Create a new Stub Information for generating a type library
     * for a particular JavaBeans component or Applet. 
     * </p>
     * 
     * Parameters:
     * @param Class if the class of the object to generate the typelib for
     * @param targetDir is the desired directory for the generation
     */
   public static StubInformation getStub(Class appletClass, String targetDir) {
	try {
	    StubInformation stub = new StubInformation();
	    stub.setOleBeanInfo(new OleBeanInfo(appletClass));
	    stub.setTargetDir(targetDir);
	    stub.setEventGeneration(false);
	    stub.setGenerationType(true);
	    stub.setActiveXName(appletClass.getName());
	    return stub;
	} catch(Exception e) {
	    e.printStackTrace();
	    return null;
	}

    }
    /*
     * Set the bean name in the jar file
     * @param String bean name
     */
    public void setSourceBeanName(String sourceBeanName) {
	this.sourceBeanName = sourceBeanName;
    }

    /*
     * @return The bean name used in the jar file
     */
    public String getSourceBeanName() {
	return sourceBeanName;
    }

    /*
     * Set the target directory for the stub generation
     * @param String Target Directory
     */
    public void setTargetDir(String targetDir) {
	this.targetDir = targetDir;
    }

    /*
     * @return The target directory for the stub generation
     */
    public String getTargetDir() {
	return targetDir;
    }

    /*
     * Set the Jar file in which the Bean is packaged
     * @param 
     */
    public void setJarFileName(String jarName) {
	// Let's make sure we get the fullpath
	if (jarName.length()>2) {
	    if (jarName.charAt(1)==':') // already fullpath
		this.jarName = jarName;
	    else {
		java.util.Properties prop = System.getProperties();
		String currentDir = prop.getProperty("user.dir", null);
		this.jarName = currentDir + "\\" + jarName;
	    }
	} else
	    this.jarName = jarName;
    }

    /*
     * Return the jar file name where the bean is packaged, 
     * null if the bean is to be loaded from the system loader
     * @return the jar file name of the bean
     */
    public String getJarFileName() {
	return jarName;
    }

    /*
     * Set the OleBeanInfo instance describing the packaged bean
     * @param OleBeanInfo describing the packaged bean
     */
    public void setOleBeanInfo(OleBeanInfo beanInfo) {
	this.beanInfo = beanInfo;
    }

    /* 
     * @return OleBeanInfo describing the bean.
     */
    public OleBeanInfo getOleBeanInfo() {
	return beanInfo;
    }

    /*
     * set the generation process to generate events signature with
     * first class object (cracked event) or with list of intrinsic
     * types from the event object (uncracked event)
     * @param state true if to generate uncracked event
     */
    public void setEventGeneration(boolean state) {
	this.uncrackEvent = state;
    }

    /*
     * @return true if the generation process should uncrack the event
     */
    public boolean getEventGeneration() {
	return uncrackEvent;
    }

    /*
     * Set the ActiveX name for the packaged Bean
     * @param the name
     */
    public void setActiveXName(String activeXName) {
	// Replace all spaces with underscores
	StringBuffer sb = new StringBuffer(activeXName);
	int i;
	for (i=0;i<sb.length();i++) 
	    if (sb.charAt(i)==' ')
		sb.setCharAt(i, '_');
	this.activeXName = sb.toString();
    }

    /*
     * @return the ActiveX name for the bean
     */
    public String getActiveXName() {
	return activeXName;
    }

    /*
     * Return the package name for the bean as a Win32 path
     * @return package name
     */
    public String getPackageName() {
	if (beanInfo!=null) {
	    String fullyQualifiedName = beanInfo.getBeanClass().getName();
	    int packageSep = fullyQualifiedName.lastIndexOf('.');
	    String packageName = fullyQualifiedName.substring(0, packageSep);
	    return packageName.replace('.','\\');
	}
	return null;
    }

    /*
     * <p>
     * Set the register flag.
     * </p>
     * @param b True if the user wants the register this ActiveX control
     */
    public void setRegister(boolean b) {
	register = b;
    }

    /*
     * <p>
     * @return true if the use wants to register the control
     */
    public boolean doRegister() {
	return register;
    }

    /*
     * <p>
     * Are we in a Type library dynamic generation scenario
     * </p>
     *
     * @return true if the generation is done on the fly while loaded in IE
     */
    public boolean isDynamic() {
	return dynamic;
    }

    /* 
     * <p>
     * Set the type lib generation scenario, static or dynamic
     * </p>
     *
     * @param dynamic True if the type library is generated on the fly in IE
     */
    public void setGenerationType(boolean dynamic) {
	this.dynamic = dynamic;
    }

    /*
     * <p>
     * Set the type library path and named used to generate the typelib 
     * </p> 
     *
     * @param libName type library complete path
     */
    public void setTypeLibName(String libName) {
	this.libName = libName;
    }

    /* 
     * <p>
     * Return the type library path, valid only once the generaration process 
     * has been completed
     * </p>
     *
     * @return the type library name
     */
    public String getTypeLibName() {
	return libName;
    }

    public String toString() {
	return 
	    "Stub information for " +
	    sourceBeanName + " generation in " +
	    targetDir;
    }
    public native boolean generateTypeLib(boolean registry);

    private String sourceBeanName;
    private String targetDir;
    private String jarName;
    private OleBeanInfo beanInfo;
    private boolean uncrackEvent;
    private String  activeXName;
    private String  libName;
    private boolean register;
    private boolean dynamic=false;
    private static boolean bInitialize = false;

}

       
