/*
 * @(#)JarBeanLoader.java	1.0 97/5/8
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.beans.ole;

/*
 * Implementation of the BeanLoader interface to load beans from the 
 * system class loader, aka the CLASSPATH
 * @see sun.beans.ole.BeanLoader
 *
 * @version 1.0
 * @author Jerome Dochez
 */

import sun.beanbox.*;
import java.beans.Beans;
import java.applet.*;
import java.net.URL;
import java.awt.Image;
import java.io.InputStream;

public class SystemBeanLoader implements BeanLoader {

    /*
     * Load a bean from the underlying storage mechanism
     * 
     * @param beanName identifies uniquely the bean within the storage
     */
    public Object initBean(String beanName)  
	throws ClassNotFoundException 
	{
	    Class beanClass;
	    Object bean;

	    // Load the class first
	    try {
		beanClass = Class.forName(beanName);
	    } catch (ClassNotFoundException e) {
		System.err.println("Class " + beanName + 
				   " not found in the CLASSPATH");
		return null;
	    }
        
	    // instanciate it
	    try {
		bean = beanClass.newInstance();
	    } catch (InstantiationException e) {
		System.err.println("The instantiation of " + beanName + " failed");
		return null;
	    } catch (IllegalAccessException e) {
		System.err.println("Class or initializer for " + beanName 
				   + " in not accessible");
		return null;
	    }
	
	    // Now instanciate the bean
	    try {
		instantiate(beanName, bean, false);
	    } catch (Throwable e) {
		System.err.println("Beans.Instanciate failed for " + beanName);
		return null;
	    }
	    return bean;
	}


    /*
     * Load a bean from the underlying storage and initializes it with the 
     * imput stream that should be its persisted state
     *
     * @param beanName identifies uniquely the bean within the storage
     * @param input InputStream on a persisted state of the bean
     */
    public Object loadBean(String beanName, java.io.InputStream input) 
	throws java.io.IOException, ClassNotFoundException
	{
	    // TO DO
	    return null;
	}

    /*
     * Load a class from the jar file
     *
     * @param class name
     *
     * @throw ClassNotFoundException
     */
    public Class loadClass(String className) throws ClassNotFoundException {

	// Delegate to class
	return Class.forName(className);
    }
  
    // Beans.Instanciate fake
    private Object instantiate
    (
     String beanName, 
     Object result, 
     boolean serialized
	) throws java.io.IOException, ClassNotFoundException {

	    ClassLoader cls=null;
	    // Ok, if the result is an applet initialize it.
	    if (result != null && result instanceof Applet) {
		Applet applet = (Applet) result;

		// Figure our the codebase and docbase URLs.  We do this
		// by locating the URL for a known resource, and then
		// massaging the URL.

		// First find the "resource name" corresponding to the bean
		// itself.  So a serialzied bean "a.b.c" would imply a resource
		// name of "a/b/c.ser" and a classname of "x.y" would imply
		// a resource name of "x/y.class".

		String resourceName= beanName.replace('.','/').concat(".class");

		URL objectUrl = null;
		URL codeBase = null;
		URL docBase = null;

		// Now get the URL correponding to the resource name.
		if (cls == null) 
		    objectUrl = ClassLoader.getSystemResource(resourceName);
		else 
		    objectUrl = cls.getResource(resourceName);
	    

		// If we found a URL, we try to locate the docbase by taking
		// of the final path name component, and the code base by taking
		// of the complete resourceName.
		// So if we had a resourceName of "a/b/c.class" and we got an
		// objectURL of "file://bert/classes/a/b/c.class" then we would
		// want to set the codebase to "file://bert/classes/" and the
		// docbase to "file://bert/classes/a/b/"

		if (objectUrl != null) {
		    String s = objectUrl.toExternalForm();
		    if (s.endsWith(resourceName)) {
			int ix = s.length() - resourceName.length();
			codeBase = new URL(s.substring(0,ix));
			docBase = codeBase;
			ix = s.lastIndexOf('/');
			if (ix >= 0) {
			    docBase = new URL(s.substring(0,ix+1));
			}
		    }
		}

		// Setup a default context and stub.
		BeansAppletContext context = new BeansAppletContext(applet);
		BeansAppletStub stub;
		stub = new BeansAppletStub(applet, context, codeBase, docBase);
		applet.setStub(stub);

		// If it was deserialized then it was already init-ed.  Otherwise
		// we need to initialize it.
		if (!serialized) {
		    // We need to set a reasonable initial size, as many
		    // applets are unhappy if they are started without
		    // having been explicitly sized.
		    applet.setBounds(0,0,100,100);
		    applet.init();
		}
		stub.active = true;
	    }
	    return result;
    }
}

/**
 * Package private support class.  This provides a default AppletContext
 * for beans which are applets.
 */

class BeansAppletContext implements AppletContext {
    Applet target;
    java.util.Hashtable imageCache = new java.util.Hashtable();

    BeansAppletContext(Applet target) {
        this.target = target;
    }

    public AudioClip getAudioClip(URL url) {
	// We don't currently support audio clips in the Beans.instantiate
	// applet context, unless by some luck there exists a URL content
	// class that can generate an AudioClip from the audio URL.
	try {
	    return (AudioClip) url.getContent();
  	} catch (Exception ex) {
	    return null;
	}
    }

    public synchronized Image getImage(URL url) {
	Object o = imageCache.get(url);
	if (o != null) {
	    return (Image)o;
	}
	try {
	    o = url.getContent();
	    if (o == null) {
		return null;
	    }
	    if (o instanceof Image) {
		imageCache.put(url, o);
		return (Image) o;
	    }
	    // Otherwise it must be an ImageProducer.
	    Image img = target.createImage((java.awt.image.ImageProducer)o);
	    imageCache.put(url, img);
	    return img;

  	} catch (Exception ex) {
	    return null;
	}
    }

    public Applet getApplet(String name) {
	return null;
    }

    public java.util.Enumeration getApplets() {
	java.util.Vector applets = new java.util.Vector();
	applets.addElement(target);
	return applets.elements();
    }

    public void showDocument(URL url) {
	// We do nothing.
    }

    public void showDocument(URL url, String target) {
	// We do nothing.
    }

    public void showStatus(String status) {
	// We do nothing.
    }
}

/**
 * Package private support class.  This provides an AppletStub
 * for beans which are applets.
 */
class BeansAppletStub implements AppletStub {
    transient boolean active;
    transient Applet target;
    transient AppletContext context;
    transient URL codeBase;
    transient URL docBase;

    BeansAppletStub(Applet target,
		    AppletContext context, URL codeBase,
		    URL docBase) {
        this.target = target;
	this.context = context;
	this.codeBase = codeBase;
	this.docBase = docBase;
    }

    public boolean isActive() {
	return active;
    }

    public URL getDocumentBase() {
	// use the root directory of the applet's class-loader
	return docBase;
    }

    public URL getCodeBase() {
	// use the directory where we found the class or serialized object.
	return codeBase;
    }

    public String getParameter(String name) {
	return null;
    }

    public AppletContext getAppletContext() {
	return context;
    }

    public void appletResize(int width, int height) {
	// we do nothing.
    }
}

class MyProducer implements sun.beanbox.InputStreamProducer {
    private InputStream is = null;

    public MyProducer(InputStream is) {
	this.is = is;
    }

    public synchronized InputStream getInputStream() {
	InputStream back = is;
	is = null;
	return back;
    }
}

