/*
 * @(#)VisualFeedback.java	1.0 97/8/7
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */


/**
 * <p>
 * sun.beans.ole.TextFeeback
 * </p>
 *
 * @version 1.0
 * @author Jerome Dochez
 */

package sun.beans.ole;

/**
 * <p>
 * The VisualFeedback class implement the UserFeeback interface for
 * notifying visually the user of the packaging step events. It uses
 * a PrintStream to print all messages
 * </p>
 * @see java.io.PrintStream
 * @see sun.beans.ole.Feeback
 */

public class TextFeedback implements UserFeedback {


    /**
     * <p>
     * Construct a new VisualFeeback
     * </p>
     * @param java.io.PrintStream out the steam used to output all 
     * messages
     */
    public TextFeedback(java.io.PrintStream out) {
	this.out = out;
    }

    /**
     * <p>
     * print the text to the user
     * </p>
     * @param s the string to be printed
     */
    public void printText(String s) {
	out.println(s);
    }

    /**
     * <p>
     * warn the user with a specific title and message
     * @param java.lang.String title warning title
     * @param java.lang.String message warning message
     */
    public void warn(String title, String message) {
	printText(title + " : " + message);
    }

     /**
     * <p>
     * warn the user with a specific title and message
     * @param java.lang.String title warning title
     * @param java.lang.String message[] warning messages
     */
    public void warn(String title, String messages[]) {
	printText("---" + title + "---");
	for (int i=0;i<messages.length;i++)
	    printText(messages[i]);
    }

    private java.io.PrintStream out;
}
