/*
 *
 * @(#) EncapsulatedEventInformation.java 1.0@(#)
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

/*
 * <p>
 * sun.beans.ole.event.EncapsulatedEventInformation
 * </p>
 *
 * @version 1.0
 * @author Jerome Dochez
 */

package sun.beans.ole.event;

import java.lang.reflect.Method;

/*
 *
 * <p>
 * The EncapsulatedEventInformation  class contains all pertinent info
 * necessary about a source interface to dynamically generate stubs 
 * </p>
 * @see sun.beans.ole.EncapsulatedEventAdaptor
 */

public class EncapsulatedEventInformation {


    /*
     * <p>
     * This constructor creates an EncapsulatedEventInformation from
     * scratch using the java.lang.Class and java.lang.reflect.Method
     * </p>
     * 
     * @param listenerType The class for the listener type
     * @param addListenerMethod The method on the event source that can
     * be used to register an event listener object
     * @param removeListenerMethod The method on the event source that
     * can be used to unregister an event listener object.
     *
     */
    public EncapsulatedEventInformation(Class	listenerType,
					Method	addListenerMethod,
					Method	removeListenerMethod) {

	this.listenerType = listenerType;
	this.addListenerMethod = addListenerMethod;
	this.removeListenerMethod = removeListenerMethod;
    }
     
    /*
     * @return the class object for the target interface that will get
     * invoked when the event is fired
     */
    public Class getListenerType() {
	return listenerType;
    }

    /*
     * @return the method used to register listener interfaces on the 
     * target object
     */
    public Method getAddListenerMethod() {
	return addListenerMethod;
    }

    /*
     * @return the method used to unregister a listener to the source
     */
    public Method getRemoveListenerMethod() {
	return removeListenerMethod;
    }

    protected Method	addListenerMethod;
    protected Method	removeListenerMethod;
    protected Class	listenerType;
}
