/*
 * @(#)OleEventManager.java	0.01 97/09/29
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1
 *
 */

/*
 * <p>
 * sun.beans.ole.OleEventManager
 * <p>
 * @version 1.1
 * @author Jerome Dochez
 *
 */

package sun.beans.ole.event;

import java.util.Hashtable;
/*
 *
 *<p>
 * Manages the encapsulated events registration and deregistration of
 * the source interface.
 * </p>
 *
 */

class OleEventManager implements EncapsulatedEventManager, 
				 EncapsulatedEventListener {

    private Class[] sourceInterface;
    private transient Hashtable	sources = new Hashtable();

    public OleEventManager(Class[] sourceInterface) {
	this.sourceInterface = sourceInterface;
    }

    /*
     * <p>
     * Called to determine the events that a particular source emits. Will throw
     * IllegalArgumentException if s is not managed by this
     * EncapsulatedEventManager.
     * </p>
     *
     * @param s the event source
     *
     * @return list of java.util.EventListener sub-interfaces that s sources or null.
     *
     * @throws IllegalArgumentException
     * @throws NullPointerException
     */
    public Class[] getSourceEventListenerInterfaces(Object s) {	
	return sourceInterface;
    }
   /**
     * <p>
     * Adds the listener eel to the source s to receive ALL events s emits
     * as encpasulated events. Will throw IllegalArgumentException if s is
     * not managed by this EncapsulatedEventManager.
     * </p>
     *
     * @param s		the event source
     * @param eel	the listener
     *
     * @throws IllegalArgumentException
     * @throws NullPointerException
     */

    public void addEncapsulatedEventListener(Object s, EncapsulatedEventListener eel) {
	addEncapsulatedEventListener(s, eel, getSourceEventListenerInterfaces(s));
    }

    /**
     * <p>
     * Removes the listener eel from the source s, thus unregistering eel from
     * receiving encapsulated events for all the events that s emits.
     * Will throw IllegalArgumentException if s is not managed by this
     * EncapsulatedEventManager.
     * </p>
     *
     * @param s		the event source
     * @param eel	the listener
     *
     * @throws IllegalArgumentException
     * @throws NullPointerException
     */

    public void removeEncapsulatedEventListener(Object s, EncapsulatedEventListener eel) {
	removeEncapsulatedEventListener(s, eel, getSourceEventListenerInterfaces(s));
    }

    /**
     * <p>
     * Adds the listener eel to the source s to receive the events enumerated
     * by lc that s emits. Will throw IllegalArgumentException if s is not
     * managed by this EncapsulatedEventManager or if lc contains a reference
     * to a Class that s does not emit events on.
     * </p>
     *
     * @param s		the event source
     * @param eel	the listener
     * @param lc	the list of events to register the eel to receive.
     *
     * @throws IllegalArgumentException
     * @throws NullPointerException
     */

    public synchronized void addEncapsulatedEventListener(
				Object       		  s,
				EncapsulatedEventListener eel,
				Class[]		          lc
    ) {
	if (s   == null) throw new NullPointerException("Source");
	if (eel == null) throw new NullPointerException("Listener");
	if (lc  == null) throw new NullPointerException("Interfaces");

	Hashtable interfaces = (Hashtable)sources.get(s);

	if (interfaces == null) {
	    interfaces = new Hashtable();
	    sources.put(s, interfaces);
	}

	for (int i = 0; i < lc.length; i++) {
	    EncapsulatedEventAdaptor eea = (EncapsulatedEventAdaptor)interfaces.get(lc[i]);

	    if (eea == null) { // its new
		try {
		    eea = EncapsulatedEventAdaptor.getEncapsulatedEventAdaptor(lc[i], s);
		} catch (ClassNotFoundException  cnfe)  {
		    System.err.println("Cant create adaptor class");
		} catch (InstantiationException  inste) {
		    System.err.println("Cant instantiate adaptor");
		} catch (IllegalAccessException iae)   {
		    System.err.println("illegal access");
		} catch (IntrospectionException  intre) {
		    System.err.println("introspection error");
		}

		if (eea != null) {
		    synchronized(eea) {
			eea.addEncapsulatedEventListener(eel);
		    }
    		    interfaces.put(lc[i], eea);

		    em.append("Listening to: " + lc[i].getName() + " on: " + s + "\n");
		} else {
    		    interfaces.put(lc[i], null);
		}
	    }
	}
    }

    /**
     * <p>
     * Removes the listener eel from the source s, thus unregistering for 
     * encapsulated events from s enumerated by lc. Will throw
     * IllegalArgumentException if s is not managed by this
     * EncapsulatedEventManager or if lc contains a reference
     * to a Class that s does not emit events on. 
     * </p>
     *
     * @param s		the event source
     * @param eel	the listener
     * @param lc	the list of events to unregister the eel from receiving.
     *
     * @throws IllegalArgumentException
     * @throws NullPointerException
     */

    public synchronized void removeEncapsulatedEventListener(
				Object	     	  	  s,
				EncapsulatedEventListener eel,
    			        Class[]			  lc
    ) {

	if (s   == null) throw new NullPointerException("Source");
	if (eel == null) throw new NullPointerException("Listener");
	if (lc  == null) throw new NullPointerException("Interfaces");

	Hashtable interfaces = (Hashtable)sources.get(s);

	
	if (interfaces == null) throw new IllegalArgumentException("Source");

	for (int i = 0; i < lc.length; i++) {
	    EncapsulatedEventAdaptor eea = (EncapsulatedEventAdaptor)interfaces.get(lc[i]);

	    if (eea != null) {
		eea.removeEncapsulatedEventListener(eel);

		synchronized(eea) {
		    if (eea.getEncapsulatedListenerCount() == 0) try {
			eea.setSource(null);
		    } catch (Exception e) {
			System.err.println("failed to unregister source");
		    }

		    em.append("No longer listening to: " + lc[i].getName() + " on: " + s + "\n");
		    interfaces.put(lc[i], null); 
		} 
	    }
	}
    }

    /**
     * <p>
     * The event handler logs the encapsulated Event to the text area.
     * </p>
     *
     * @param ee The Encapsulated Event
     */

    public void encapsulatedEvent(EncapsulatedEvent ee) {
	Object	    s  = ee.getEventSource();
	EventObject eo = ee.getEvent();

	// log the event

	System.out.println(eventCnt++  + " Source: [" + s + "] Event: [" + eo + "]" + "\n");
    }
}
