/*
 * @(#)ActivatorAppletContext.java	1.8 01/02/09
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1
 *
 */

package sun.plugin;

import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;

/*
 * Helper class that implement most of the AppletContext
 *
 * @version 	1.2.1
 * @author	Jerome Dochez
 */

import java.applet.*;
import java.util.*;
import sun.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.lang.reflect.*;
import java.lang.ref.SoftReference;
import java.security.*;

public abstract class ActivatorAppletContext implements AppletContext {
     
    /*
     * Get an audio clip.
     *
     * @param url url of the desired audio clip
     */
    public AudioClip getAudioClip(final URL url) 
    {
        System.getSecurityManager().checkConnect(url.getHost(), url.getPort());

	AudioClip clip = null;

	synchronized (audioClipStore)
	{
	    // Obtain audio list according to applet context
	    HashMap audioClips = (HashMap) audioClipStore.get(appletPanel.getCodeBase());
	    
	    if (audioClips == null)
	    {
		audioClips = new HashMap();
		audioClipStore.put(appletPanel.getCodeBase(), audioClips);
	    }

	    ActivatorAppletAudioClipRef ref = (ActivatorAppletAudioClipRef) audioClips.get(url);

	    if (ref == null)
		ref = new ActivatorAppletAudioClipRef(url);

	    audioClips.put(url, ref);

	    clip = (AudioClip) ref.get();
	}

	Trace.println("Loaded audio clip: " + url, TraceFilter.JAVA_CONSOLE_ONLY);

	return clip;
    }


    /*
     * Get an image.
     * 
     * @param url of the desired image
     */
    public Image getImage(URL url) 
    {
        System.getSecurityManager().checkConnect(url.getHost(), url.getPort());

	Image image = null;

	synchronized (imageRefs) 
	{
	    ActivatorAppletImageRef ref = (ActivatorAppletImageRef) imageRefs.get(url);

	    if (ref == null) 
	    {
		ref = new ActivatorAppletImageRef(url);
		
		imageRefs.put(url, ref);
	    }

	    image = (Image) ref.get();
	}

	Trace.println("Loaded image: " + url, TraceFilter.JAVA_CONSOLE_ONLY);

	return image;
    }


    /**
     * Get an applet by name.
     */
    public Applet getApplet(String name) 
    {
	name = name.toLowerCase();

	Collection collection = appletPanels.values();

	for (Iterator iter = collection.iterator() ; iter.hasNext() ;) 
	{
	    SoftReference ref = (SoftReference) iter.next();

	    if (ref != null)
	    {
		AppletPanel p = (AppletPanel) ref.get();

		if (p != null)
		{
		    String param = p.getParameter("name");
		    if (param != null) {
			param = param.toLowerCase();
		    }
		    if (name.equals(param) && 
			p.getDocumentBase().equals(appletPanel.getDocumentBase())) {
			try {
			    if (checkConnect(appletPanel.getCodeBase().getHost(),
					 p.getCodeBase().getHost())==false)
				return null;
			} catch (InvocationTargetException ee) {
			    showStatus(ee.getTargetException().getMessage());
			    return null;
			} catch (Exception ee) {
			    showStatus(ee.getMessage());
			    return null;
			}
			return p.getApplet();
		    }
		}
	    }
	}
	return null;
    }

    /**
     * Return an enumeration of all the accessible
     * applets on this page.
     */
    public Enumeration getApplets() 
    {
	Vector v = new Vector();
	Collection collection = appletPanels.values();

	for (Iterator iter = collection.iterator() ; iter.hasNext() ;) 
	{
	    SoftReference ref = (SoftReference) iter.next();

	    if (ref != null)
	    {
		AppletPanel p = (AppletPanel) ref.get();

		if (p != null && p.getDocumentBase().equals(appletPanel.getDocumentBase())) 
		{
		    try {
			checkConnect(appletPanel.getCodeBase().getHost(), 
				     p.getCodeBase().getHost());
			v.addElement(p.getApplet());
		    } catch (InvocationTargetException ee) {
			showStatus(ee.getTargetException().getMessage());
		    } catch (Exception ee) {
			showStatus(ee.getMessage());
		    }
		}
	    }
	}
	return v.elements();
    }

    /*
     * <p>
     * Check that a particular applet is authorized to connect to 
     * the target applet.
     * This code is JDK 1.1 and 1.2 dependent
     * </p>
     *
     * @param source Source applet host name requesting the connect
     * @param target Target applet host name to connect to 
     * 
     * @return true if the connection is granted
     */
    private boolean checkConnect(String sourceHostName, String targetHostName)
	throws SecurityException, InvocationTargetException, Exception
    {
	// Attention JDK1.1 dependent code
	// We need to use jdk dependent code since the checkConnect method we used
	// in JDK1.1 is gone in JDK1.2. If this method does not exist, we 
	// are running in a JDK1.2 VM and we should use the new security
	// APIs to ensure proper checking.
	AppletCheckConnect acc = new sun.plugin.AppletCheckConnectForJDK12();
	return acc.checkConnect(sourceHostName, targetHostName);
    }
    /*
     * Replaces the Web page currently being viewed with the given URL
     *
     * @param url the address to transfer to
     */
    abstract public void showDocument(URL url);

    /*
     * Requests that the browser or applet viewer show the Web page
     * indicated by the url argument.
     *
     * @param url the address to transfer to
     * @param target One of the value
     *	"_self"  show in the current frame
     *  "_parent"show in the parent frame
     *  "_top"   show in the topmost frame
     *  "_blank" show in a new unnamed top-level windownameshow in a 
     *           new top-level window named name
     */
    abstract public void showDocument(URL url, String target);

    /*
     * Show status.
     * 
     * @param status status message
     */
    abstract public void showStatus(String status);

    /* 
     * Add a applet in this applet context
     * 
     * @param applet the applet to add
     */
    void addAppletInContext(AppletPanel appletPanel) {
	this.appletPanel = appletPanel;
	appletPanels.put(appletPanel.toString(), new SoftReference(appletPanel));
    }

    /* 
     * Remove applet from this applet context
     * 
     * @param applet applet to remove
     */
    void removeAppletFromContext(AppletPanel applet) {
	appletPanels.remove(applet.toString());

	stopAudioClips();
    }

    /*
     * Stop all audio clips
     *
     * @param url
     */
    private void stopAudioClips()
    {
	try
	{
	    synchronized(audioClipStore)
	    {
		// Obtain audioClips for a given URL
		HashMap audioClips = (HashMap) audioClipStore.get(appletPanel.getCodeBase());

		if (audioClips != null)
		{
		    Collection collection = audioClips.values();
		    Iterator iter = collection.iterator();
		    
		    while (iter.hasNext())
		    {
			ActivatorAppletAudioClipRef ref = (ActivatorAppletAudioClipRef) iter.next();
			ref.setDoDownload(false);
			AudioClip ac = (AudioClip) ref.get();
			ref.setDoDownload(true);
			if (ac != null)
			    ac.stop();	    
		    }	    
		}
	    }
	}
	catch (Throwable e)
	{

	}
    }
    

    private AppletPanel appletPanel;
    
    private static HashMap appletPanels = new HashMap();
    private static HashMap imageRefs = new HashMap();
    private static HashMap audioClipStore = new HashMap();
}   

