/*
 * @(#)ActivatorCookieHandler.java	1.16 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1
 *
 * Java Plug-in
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin;

import java.net.*;
import java.util.*;
import sun.plugin.protocol.*;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;
import sun.plugin.platform.*;


/**
 * <p> ActivatorProxyHandler is a class that encapsulates the mechanism for
 * obtaining the cookie value of a particular URL.
 * </p>
 */
public class ActivatorCookieHandler implements CookieHandler {

    /**
     * <p> Cookie cache. </p>
     */
    private HashMap cookieTable = null;

    /**
     * <p> use cookie cache </p>
     */
    private boolean useCache = false;

    /** 
     * <p> Construct an ActivatorCookieHandler object.
     * </p>
     */
    protected ActivatorCookieHandler()
    {
        cookieTable = new HashMap();

	String cache = (String) java.security.AccessController.doPrivileged(
			new sun.security.action.GetPropertyAction("javaplugin.cachecookie", "false"));

	Trace.netPrintln("Cookie Cache: " + cache, TraceFilter.JAVA_CONSOLE_ONLY);

	if (cache != null && cache.equalsIgnoreCase("true"))
	    useCache = true;
    }


    /**
     * <p> Sets the corresponding cookie value with respect to the given URL.
     * </p>
     *
     * @param u URL
     * @param String contains the corresponding cookie value.
     */
    public synchronized void setCookieInfo(URL u, String value)
    {
	PlatformHandler handler = ServiceProvider.getProvider();

	if (handler != null)
	{   
	    Trace.netPrintln("Server " + u + " requesting to set-cookie with \"" + value + "\"", TraceFilter.JAVA_CONSOLE_ONLY);
	    handler.SetCookie(u, value);
	}
    }

    /**
     * <p> Returns the corresponding cookie value with respect to the given URL.
     * </p>
     *
     * @param u URL
     * @returns String contains the corresponding cookie value.
     */
    public synchronized String getCookieInfo(URL u)
    {
	String cookie = null;

	if (useCache)
	{
	    // Find key to lookup cookie table
	    String key = u.getHost() + u.getFile();

	    // To lookup the cookie, just the URL hostname and path without filename
	    int index = key.lastIndexOf('/');

	    if (index < 0)
		return null;

	    key = key.substring(0, index);

	    cookie = (String) cookieTable.get(key);

	    if (cookie == null)
	    {
		cookie = findCookieForURL(u.toString());

		if (cookie != null && !cookie.equals("") && !cookie.equals("\n") && !cookie.equals("\r\n"))
		    cookieTable.put(key, cookie);
		else  {
		    cookieTable.put(key, "");
		    cookie = null;
		}
	    }
	}
	else
	{
	    // Obtain cookie from browser

	    cookie = findCookieForURL(u.toString());
	}

	if (cookie != null && cookie.equals(""))
	    cookie = null;

        if (cookie != null)
	    Trace.netPrintln("Connecting " + u + " with cookie \"" + cookie + "\"", TraceFilter.JAVA_CONSOLE_ONLY);

        return cookie;
    }

    /**
     * <p> Native method to obtain the cookie from the browser.
     * </p>
     * 
     * @param url URL
     * @returns String contains the corresponding cookie value.
     */
    private native String findCookieForURL(String url);
}


