/*
 * @(#)AppletViewer.java	1.19 01/03/05
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin;

/*
 * A recipient to show a Java Applet or a JavaBeans component in.
 * This class implements all platform independent behaviour of showing 
 * an applet in one version of the Java Activator.
 *
 * @version 	1.3
 * @author	Jerome Dochez
 */

import sun.applet.*;
import java.util.*;
import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.io.*;
import java.security.*;
import java.lang.reflect.*;
import java.util.Vector;
import sun.plugin.protocol.*;
import sun.plugin.security.PluginClassLoader;
import sun.plugin.cachescheme.*;
import sun.plugin.resources.ResourceHandler;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;
import sun.plugin.usability.TraceListener;
import sun.plugin.usability.DialogFactory;
import sun.awt.AppContext;


public abstract class AppletViewer extends AppletPanel 
				   implements ProxyType, TraceListener
{

    // Network Access Level
    public static final int NETWORK_ACCESS_NONE = 0;
    public static final int NETWORK_ACCESS_HOST = 1;
    public static final int NETWORK_ACCESS_UNRESTRICTED = 2;

    /*
     * are we initialized
     */
    private static boolean initialized = false;

    /*
     * are we initialized
     */
    public static boolean javaEnabled = true;

    /**
     * Some constants...
     */
    public static String theVersion = "1.1";
    private static String defaultSaveFile = "Applet.ser";
    private static String propertiesFileName = "properties" + 
						System.getProperty("javaplugin.nodotversion");;

    /**
     * Look here for the properties file
     */
    public static File theUserPropertiesFile;
    public static File theAppletViewerPropsFile;

    URL documentURL = null;
    URL baseURL = null;
    private static AppletMessageHandler amh = new AppletMessageHandler("appletpanel");

    // Parameters handler
    protected java.util.Hashtable atts = new java.util.Hashtable();

    // JAR files that have not yet been loaded.
    private String cookedJars;

    // Exception dialogboxes enable/disable status
    private static boolean fShowException = false;

    public static void loadPropertiesFiles() {

	String sep = File.separator;
	theUserPropertiesFile = new File(System.getProperty("user.home") +
					 sep + ".java" +
					 sep + propertiesFileName);
	// ensure the props folder can be made
	new File(theUserPropertiesFile.getParent()).mkdirs();
	theAppletViewerPropsFile = new File(System.getProperty("java.home") +
					    sep + "lib" +
					    sep + "appletviewer.properties");
    }

    /**
     * Prepare the enviroment for executing applets.
     */
    public static void initEnvironment(int proxyType, String proxyList, String proxyOverride)
    {
	// if we are already initialized, just return
	if (initialized)
	    return;
	initialized=true;

	Properties props = new Properties(System.getProperties());

	// Define a number of standard properties 
	props.put("acl.read", "+");
	props.put("acl.read.default", "");
	props.put("acl.write", "+");
	props.put("acl.write.default", "");

	// Standard browser properties
	props.put("browser", "sun.plugin");
	props.put("browser.version", theVersion);
	props.put("browser.vendor", "Sun Microsystems, Inc.");

	// Set HTTP User-Agent
	{
	    if (proxyType == PROXY_TYPE_BROWSERCONFIG)
    		props.put("http.agent", "Mozilla/5.0 (" + System.getProperty("os.name") + ")");
	    else
		props.put("http.agent", "Activator/" + System.getProperty("javaplugin.version"));
	}

	// Define which packages can NOT be accessed by applets
	props.put("package.restrict.access.sun", "true");
	// 
	// This is important to set the netscape package access to "false". 
	// Some applets running in IE and NS will access 
	// netscape.javascript.JSObject sometimes. If we set this 
	// restriction to "true", these applets will not run at all. 
	// However, if we set it to "false", the applet may continue 
	// to run by catching an exception.
	props.put("package.restrict.access.netscape", "false");

	// Define which packages can NOT be extended by applets
	props.put("package.restrict.definition.java", "true");
	props.put("package.restrict.definition.sun", "true");
	props.put("package.restrict.definition.netscape", "true");

	// Define which properties can be read by applets.
	// A property named by "key" can be read only when its twin
	// property "key.applet" is true.  The following ten properties
	// are open by default.  Any other property can be explicitly
	// opened up by the browser user setting key.applet=true in
	// ~/.java/properties.   Or vice versa, any of the following can
	// be overridden by the user's properties.
	props.put("java.version.applet", "true");
	props.put("java.vendor.applet", "true");
	props.put("java.vendor.url.applet", "true");
	props.put("java.class.version.applet", "true");
	props.put("os.name.applet", "true");
	props.put("os.version.applet", "true");
	props.put("os.arch.applet", "true");
	props.put("file.separator.applet", "true");
	props.put("path.separator.applet", "true");
	props.put("line.separator.applet", "true");

        // Install new protocol handler
	String pkgs = (String) props.get("java.protocol.handler.pkgs");
	if (pkgs != null) 
	    props.put("java.protocol.handler.pkgs", pkgs + "|sun.plugin.protocol.jdk12");
	else
	    props.put("java.protocol.handler.pkgs", "sun.plugin.protocol.jdk12");
	CookieHandler handler2 = new ActivatorCookieHandler();
	sun.plugin.protocol.jdk12.http.HttpURLConnection.setCookieHandler(handler2);


	// Set allow default user interaction in HTTP/HTTPS
	java.net.URLConnection.setDefaultAllowUserInteraction(true);


	// Try loading the saved user properties file to override some
        // of the above defaults.
	try {
	    FileInputStream in = new FileInputStream(theUserPropertiesFile);
	    props.load(new BufferedInputStream(in));
	    in.close();
	} catch (Exception e) {
	    // We silently ignore a missing property file.
	}
	
	if ("false".equalsIgnoreCase(props.getProperty("javaplugin.enabled", "true"))) {
	  javaEnabled = false;
	  System.err.println(getMessage("java_not_enabled"));
	  
	  System.setProperties(props);
	  return;
	}
	
	// Look in the user choices if we should open the Java Console
	if ("true".equalsIgnoreCase(props.getProperty("javaplugin.console", "false"))) 
	{
	    sun.plugin.JavaRunTime.showJavaConsole(true);
	}
	else
	{
	    // If user select not to show the console, we should display the console
	    // if we are on Windows, or if we are in Netscape 4.x on Unix. [stanleyh]
	    //
            String osname = (String) java.security.AccessController.doPrivileged(
                                     new sun.security.action.GetPropertyAction("os.name"));

            String workaround = (String) java.security.AccessController.doPrivileged(
                                         new sun.security.action.GetPropertyAction("mozilla.workaround", "false"));

	    if ((osname.indexOf("Windows") != -1)
		|| workaround.equalsIgnoreCase("false"))
	    {
		sun.plugin.JavaRunTime.showJavaConsole(false);    
	    }
	}
	
	//Check whether the exception dialogboxes are allowed
	if ("true".equalsIgnoreCase(props.getProperty("javaplugin.exception", "false"))) {
	    fShowException = true;
	}

        // Override proxy settings if use browser set false in the control panel.
        if ("false".equalsIgnoreCase(props.getProperty("javaplugin.proxy.usebrowsersettings", "true"))) {
           System.out.println(getMessage("user_overriden_browser"));
           proxyType = PROXY_TYPE_MANUAL;
           proxyList = props.getProperty("javaplugin.proxy.settings", "");
	   if (proxyList.length() == 0) {
	       proxyType = PROXY_TYPE_NO_PROXY;
	   }
           proxyOverride = "";
        }

	// Print out proxy setting -- this is very important for debugging
        System.out.print(getMessage("proxy_configuration"));
        switch (proxyType)  {
        case PROXY_TYPE_BROWSERCONFIG:
             System.out.println(getMessage("browser_config"));
             // Set system property "mozilla.workaround" to workaround Mozilla bugs
             System.setProperty("mozilla.workaround", "true");
             break;
        case PROXY_TYPE_AUTOCONFIG:
             System.out.println(getMessage("auto_config"));
             break;
        case PROXY_TYPE_MANUAL:
	     System.out.println(getMessage("manual_config"));
             System.out.println("     " + getMessage("proxy_is") + proxyList);
             System.out.println("     " + getMessage("proxy_override_is") + proxyOverride);
             break;
        default:
             System.out.println(getMessage("no_proxy"));	     
	     proxyType = PROXY_TYPE_NO_PROXY;
             break;
        }

        System.out.println("");

	try {
	    Class c = Class.forName("sun.misc.ExtensionDependency");
	    if (c != null) {
		Class[] parms = new Class[1];
		parms[0] = Class.forName("sun.misc.ExtensionInstallationProvider");
		java.lang.reflect.Method m = c.getMethod("addExtensionInstallationProvider", parms);
		if (m != null) {
		    Object[] args = new Object[1];
		    args[0] = new sun.plugin.extension.ExtensionInstallationImpl();
		    m.invoke(null, args);
		} else {
		    Trace.println("Dependent Extension Provider not " 
			       + "installed : Cannot get the "
			       + " addExtensionInstallationProvider"
			       +" method");
		}
	    } else {
		Trace.println("Dependent Extension Provider not " 
			   + "installed : Cannot get the "
			   + "Sun.misc.ExtensionDependency class");
	    }		    
	} catch(Throwable e) {
	    Trace.printException(e);
	}


	// Workaround JavaSound bug #4304996
	try {

	    // Loading these two classes will cause the event dispatch
	    // threads to be created in the main thread group.
	    //
	    Class c0 = Class.forName("com.sun.media.sound.AbstractLine");
	    Class c1 = Class.forName("com.sun.media.sound.AbstractMidiDevice");
	} catch(Throwable e) {
	    e.printStackTrace();
	}

	// Dump properties - if in trace mode
	if (Trace.isEnabled()) 
	{
	    sun.plugin.JavaRunTime.showSystemProperties();
	}

        // Remove Proxy Host & Port
	props.remove("proxyHost");
	props.remove("proxyPort");
        props.remove("http.proxyHost");
        props.remove("http.proxyPort");
        props.remove("https.proxyHost");
        props.remove("https.proxyPort");
        props.remove("ftpProxyHost");
        props.remove("ftpProxyPort");
        props.remove("ftpProxySet");
        props.remove("gopherProxyHost");
        props.remove("gopherProxyPort");
        props.remove("gopherProxySet");
	props.remove("socksProxyHost");
	props.remove("socksProxyPort");
	props.put("javaplugin.proxy.config.type", "direct");

	// Enable proxy in the protocol handler
        if (proxyType != PROXY_TYPE_NO_PROXY)
        {
	    // Make sure proxy is detected on the fly in http, ftp and gopher.
	    ProxyHandler handler = getDefaultProxyHandler(proxyType,
							  proxyList,
                                                          proxyOverride);

	    try 
	    {
		// Set default proxy handler
		sun.plugin.protocol.PluginProxyHandler.setDefaultProxyHandler(handler); 	       
	    
		// Let the handler to update the system properties if necessary, 
		// e.g. SOCKS.
		handler.setProperties(props);
	    } 
	    catch(Throwable e) 
	    {
		Trace.netPrintException(e);
		System.out.println(getMessage("protocol_handler_error"));
		DialogFactory.showMessageDialog(getMessage("protocol_handler_error"));
	    }

	    // Set RMI socket factory for proxy.
	    try {
		java.rmi.server.RMISocketFactory.setSocketFactory(new RMIPluginSocketFactory());
	    }
	    catch (IOException e)  {
	    }
        }

	// Enable proxy/web server authentication
	if ("true".equalsIgnoreCase(
	    props.getProperty("javaplugin.proxy.authentication", "true"))) {

	    java.net.Authenticator.setDefault(new PluginAuthenticator());
	}


	// Install a property list.
	System.setProperties(props);

	/* we reference this class to get it loaded
	 * from a class w/o a classloader.  If not,
	 * it first gets loaded from a thread w/ an
	 * applet on the stack, and then it can't read
	 * it's own content-types.properties file.
	 */
	try {
	    sun.net.www.MimeTable mt =
		sun.net.www.MimeTable.getDefaultTable();
	} catch (Throwable e) {
	    Trace.printException(e);
	}

	// Install security manager, so everyone feels
        // warm and fuzzy.  Hold me, the way I need to be held.

	ActivatorSecurityManager security = new ActivatorSecurityManager();
	System.setSecurityManager(security);

	// REMIND: Create and install a socket factory!

	
	// All initialization is done, show help information in Java Console.
	sun.plugin.JavaRunTime.showJavaConsoleHelp();
    }

    /**
     * Construct a new applet viewer.
     * Restricted to subclasses for security reasons,
     * 
     * @param appletContext the AppletContext to use
     */
    protected AppletViewer(ActivatorAppletContext appletContext) {
	if (appletContext==null) 
	    throw new IllegalArgumentException("AppletContext");
        setBackground(Color.lightGray);
	this.appletContext = appletContext;
        appletContext.addAppletInContext(this);

	class AppletEventListener implements AppletListener  {

	    public void appletStateChanged(AppletEvent evt) {
		switch (evt.getID()) {
                    case AppletPanel.APPLET_RESIZE: {
			AppletPanel src = (AppletPanel)evt.getSource();
			if(src != null) 
			{
			    Object obj = getViewedObject();

			    if (obj instanceof Component)
			    {
				resize(getSize());
				((Component)obj).setSize(getSize());
				validate();
			    }
                        }
		    }
		}
	    }
	};

	addAppletListener(new AppletEventListener());
    }


    /**
     * Init the applet, called once to start the download of the applet 
     */
    public void init() {

	ClassLoaderInfo cli = ClassLoaderInfo.find(this);
	cli.addReference();
	synchronized(AppletViewer.class) {
	    super.init();
	}

	// After super.init() is called, applet handler thread is 
	// created. Added applet thread group into tracing.
	//
	Thread t = getAppletHandlerThread();
	Trace.addTraceListener(t.getThreadGroup(), this);


    	// Just make sure we cache the HTML page from another thread if necessary
	Runnable work = new Runnable() {
	    public void run() {

		// Once documentURL and codebase are determined, try to see if 
		// the HTML page should be cached.
		HTMLInstaller installer = new HTMLInstaller();
		Hashtable params = (Hashtable) atts.clone();
		boolean success = false;

		try
		{
		    installer.install(params, getCodeBase());

		    success = true;
		}
		catch (sun.plugin.cachescheme.AppletInstallationException e1)
		{
		    Trace.printException(e1, ResourceHandler.getMessage("html.wrong_params"),
					 ResourceHandler.getMessage("html.caption"));
		}
		catch (sun.plugin.cachescheme.EntryNotFoundException e2)
		{
		    Trace.printException(e2, ResourceHandler.getMessage("html.cache_error"),
					 ResourceHandler.getMessage("html.caption"));
		}
		catch (java.io.IOException e3)
		{
		    Trace.printException(e3, ResourceHandler.getMessage("html.general_error"),
					 ResourceHandler.getMessage("html.caption"));
		}
		finally
		{
		    if (success)
			Trace.println("HTML Installation finished.", TraceFilter.JAVA_CONSOLE_ONLY);
		    else
			Trace.println("HTML Installation failed.", TraceFilter.JAVA_CONSOLE_ONLY);
		}
	    }
	};
	Thread htmlThread = new Thread(work);
	htmlThread.start();

	Trace.println("Sending events to applet. LOAD", this);
	sendEvent(sun.applet.AppletPanel.APPLET_LOAD);
	Trace.println("Sending events to applet. INIT", this);
	sendEvent(sun.applet.AppletPanel.APPLET_INIT);
    }

    /**
     * Start the applet.
     */
    public void appletStart() {
	Trace.println("Sending events to applet. START", this);
	sendEvent(sun.applet.AppletPanel.APPLET_START);
    }

    /**
     * Stop the applet.
     */
    public void appletStop() {
	Trace.println("Sending events to applet. STOP", this);
	sendEvent(sun.applet.AppletPanel.APPLET_STOP);
    }


    /**
     * Notification that the applet is being closed
     *
     * @param timeOut max time we are waiting for the applet to die
     *		in milliseconds.
     */ 
    public void onClose(final int timeOut) {

    	// Just make sure we are destroying the thread from a utility
	// thread to not block the main thread for that.

	Runnable work = new Runnable() {
	    public void run() {
		onPrivateClose(timeOut);	
	    }
	};
	Thread closingThread = new Thread(work);
	closingThread.start();
    }

    /**
     * Notification that the applet is being closed
     *
     * @param timeOut max time we are waiting for the applet to die
     *		in milliseconds.
     */
    protected void onPrivateClose(int timeOut) {
	appletContext.removeAppletFromContext(this);

	// Remove applet's thread group from tracing. Applet may 
	// still be running, so obtain applet handler thread
	// is legal.
	Thread t = getAppletHandlerThread();

	if (t != null)
	    Trace.removeTraceListener(t.getThreadGroup(), this);


	if (status==APPLET_LOAD) {
	    Trace.println("Applet loading in progress, stop loading.", this);
	    stopLoading();
	}
	Trace.println("Sending events to applet. STOP", this);
	sendEvent(APPLET_STOP);
	Trace.println("Sending events to applet. DESTROY", this);
        sendEvent(APPLET_DESTROY);
        Trace.println("Sending events to applet. DISPOSE", this);
        sendEvent(APPLET_DISPOSE);
        Trace.println("Sending events to applet. QUIT", this);
        sendEvent(APPLET_QUIT); 
	Trace.println("Finding information...", this, TraceFilter.JAVA_CONSOLE_ONLY);
	ClassLoaderInfo cli = ClassLoaderInfo.find(this);
	cli.removeReference();
	Trace.println("Done...", this, TraceFilter.JAVA_CONSOLE_ONLY);
    }

    // This version added by ben as a temporoary hack to get things
    // going under 1.2fcs. It is the same as onClose, but does not
    // send the QUIT event
    public void onCloseTemp(int timeOut) {
	appletContext.removeAppletFromContext(this);
	if (status==APPLET_LOAD) {
	    Trace.println("Loading applet..", this);
	    status=APPLET_ERROR;
	    stopLoading();
	    Trace.println("Stopped loading...", this);
	    getAppletHandlerThread().interrupt();
	    Trace.println("interrrupted thread...", this);
	    return;
	} else {
	    Trace.println("Sending events to applet. STOP", this);
	    sendEvent(APPLET_STOP);
	    Trace.println("Sending events to applet. DESTROY", this);
	    sendEvent(APPLET_DESTROY);
	    Trace.println("Sending events to applet. DISPOSE", this);
	    sendEvent(APPLET_DISPOSE);
	    Trace.println("Sending events to applet. QUIT", this);
	    /*	    sendEvent(APPLET_QUIT); */
	}
	try {
	    // Let's wait no more than 5 secs that the handler thread
	    // die of natural causes.
	    Trace.println("Waiting for timeout...", this);
	    if (timeOut>=0) {
		Trace.println("Doing a join...", this);
		getAppletHandlerThread().join(timeOut);
		Trace.println("Done with join ...", this);
	    }
	} catch (InterruptedException e) {
	    // Ignore
	}
	Trace.println("Finding information...", this, TraceFilter.JAVA_CONSOLE_ONLY);
	ClassLoaderInfo cli = ClassLoaderInfo.find(this);
	cli.removeReference();
	Trace.println("Done...", this, TraceFilter.JAVA_CONSOLE_ONLY);
    }

    /**
     * Initialized the properties for this AppletViewer like the Applet
     * codebase and document base.
     */
    protected void initProperties() {
	String att = getParameter("java_codebase");
	if (att==null)
	    att = getParameter("codebase");
	if (att != null) {
	    if (!att.endsWith("/")) {
		att += "/";
	    }
	    try {
		baseURL = new URL(documentURL, att);
	    } catch (MalformedURLException e) {
		// ignore exception
	    }
	}
	if (baseURL == null) {
	    String file = documentURL.getFile();
	    int i = file.lastIndexOf('/');
	    if (i > 0 && i < file.length() - 1) {
		try {
		    baseURL = new URL(documentURL, file.substring(0, i + 1));
		} catch (MalformedURLException e) {
		    // ignore exception
		}
	    }
	}
	
	// when all is said & done, baseURL shouldn't be null
	if (baseURL == null)
	    baseURL = documentURL;
    }

    /**
     * Get an applet parameter.
     */
    public String getParameter(String name) {
	name = name.toLowerCase(java.util.Locale.ENGLISH);
	if (atts.get(name) != null) {
	    return (String) atts.get(name);
	} else {
	    String value=null;
	    try {
    		value = getParameterFromHTML(name);
	    } catch (Throwable e) {
		Trace.printException(e);
	    }
	    if (value!=null)
    	        atts.put(name.toLowerCase(java.util.Locale.ENGLISH), value);
	    return value;
	}	   
    }


    /**
     * Get applet's parameters.
     */
    public Hashtable getParameters()
    {
	return (Hashtable) atts.clone();
    }


    /**
     * Abstract method to retrieve the parameter from the HTML tags
     * 
     * @param name the parameter name
     *
     * @return the parameter value, null if undefined
     */
    protected abstract String getParameterFromHTML(String name);

    /**
     * Set an applet parameter.
     */
    public void setParameter(String name, Object value) {
	name = name.toLowerCase(java.util.Locale.ENGLISH);
	atts.put(name, value.toString());
    }

    /**
     * Get the document url.
     */
    public URL getDocumentBase() {
	return documentURL;

    }

    /**
     * Set the document url.
     * This must be done early, before initProperties is called.
     */
    public void setDocumentBase(URL url) {
	documentURL = url;
    }

    /**
     * Get the base url.
     */
    public URL getCodeBase() {
	return baseURL;
    }
    /**
     * Get the width.
     */
    public int getWidth() {
	String w = getParameter("width");
	if (w != null) {
	    return Integer.valueOf(w).intValue();
	}
	return 0;
    }

    /**
     * Get the height.
     */
    public int getHeight() {
	String h = getParameter("height");
	if (h != null) {
	    return Integer.valueOf(h).intValue();
	} 
	return 0;
    }

    /**
     * Get the code parameter
     */
    public String getCode() {

	// Support HTML 4.0 style of OBJECT tag.
	//
	// <OBJECT classid=java:sun.plugin.MyClass .....>
	// <PARAM ....>
	// </OBJECT>
	// 
	// In this case, the CODE will be inside the classid
	// attribute.
	//
	String moniker = getParameter("classid");
	String code = null;

	if (moniker != null)
	{
	    int index = moniker.indexOf("java:");

	    if (index > -1)
	    {
		code = moniker.substring(5 + index);

		if (code != null || !code.equals(""))
		    return code;
	    }
	}

	code = getParameter("java_code");
	if (code==null)
	    code=getParameter("code");
	return code;
    }

    /**
     * Return the list of jar files if specified.
     * Otherwise return null.
     */
    public String getJarFiles() {
	return cookedJars;
    }


   /*
    *  Allow pre-loading of local .jar files in plug-in lib/app directory
    *  These .jar files are loaded with the PluginClassLoader so they
    *  run in the applet's sandbox thereby saving developers the trouble
    *  of writing trusted support classes.
    *  The ClassLoaderInfo cli should be locked.
    */
    protected Vector getLocalJarFiles()  {
    
        String fSep = File.separator;
        String libJarPath = 
	    System.getProperty("java.home") + fSep + "lib";
	    			
        String appJarPath = libJarPath + fSep + "applet";
        return(getJarFilesFromPath(appJarPath));
	
    }

        
    private Vector getJarFilesFromPath (String basePath)  {
	File dir = new File(basePath);
	if (!dir.exists())  {
	    return(null);
	}
    
	String[] jarList = dir.list(new FilenameFilter()  {
	    public boolean accept(File f, String s)  {
	        return(s.endsWith(".jar"));
	    }
	});
	
	Vector jars = new Vector();
	
        String fSep = File.separator;
        for (int i = 0; i < jarList.length; i++)  {
	    String fullJarPath = basePath + fSep + jarList[i];
	    jars.add(new File(fullJarPath));
	}
	
	return(jars);
    }    

  
    /*
     * We overload our parent loadJarFiles so tht we can avoid
     * reloading JAR files that are already loaded.
     *						KGH Mar 98
     */
    protected void loadJarFiles(AppletClassLoader loader)
                                throws IOException, InterruptedException 
    {
        // Figure out the list of all required JARs.
        String archive = getParameter("java_archive");
        if (archive == null) 
        {
	    archive = getParameter("archive");
        }

	Trace.println("Determine if the applet requests to install any JAR", TraceFilter.JAVA_CONSOLE_ONLY);

	boolean success = false;

	String cacheOption = null;
	String cacheArchive = null;

	try
	{
	    // Read the Cache Option from the cache_option tag
	    cacheOption = getParameter("cache_option");

	    // Read the cache Archive value from the cache_archive tag
	    cacheArchive = getParameter("cache_archive");

	    // Read the cache Version value from the cache_version tag
	    String cacheArchiveVer = getParameter("cache_version");

	    // Display jar cache option
	    StringBuffer buffer = new StringBuffer();
	    buffer.append("Jar cache option: " + cacheOption + "\n");
	    buffer.append("Jar archive(s): " + cacheArchive + "\n");
	    buffer.append("Jar cache version(s): " + cacheArchiveVer);

	    Trace.println(buffer.toString(), TraceFilter.JAVA_CONSOLE_ONLY);

	    try
	    {
		if (cacheOption == null)
		{
		    success = true;
		}
		else
		{
		    JarCacheManager cacheManager = new JarCacheManager();

		    JarCacheInterface handler = cacheManager.selectHandler(cacheOption);

		    Vector cachedJarFiles = handler.getJarFilesPath(cacheArchive, cacheArchiveVer, getCodeBase()); 

		    // Deposit the contents of the Vector in the PluginClassLoader
		    // Object only if the size of the vector is > zero
		    if( (cachedJarFiles != null) && (cachedJarFiles.size() > 0) ) 
		    {
			((PluginClassLoader)(loader)).addCacheJar(cachedJarFiles);
		    }

		    success = true;
		}
	    }
	    catch (InvalidCacheParameterException ex0)
	    {
		Trace.printException(ex0, ResourceHandler.getMessage("applet_install.wrong_param"),
				     ResourceHandler.getMessage("applet_install.caption"));
	    }
	    catch (PluginMutexException ex1)
	    {
		Trace.printException(ex1, ResourceHandler.getMessage("applet_install.mutex_error"),
				     ResourceHandler.getMessage("applet_install.caption"));
	    }
	    catch (IOException ex2)
	    {
		Trace.printException(ex2, ResourceHandler.getMessage("applet_install.io_error"),
				     ResourceHandler.getMessage("applet_install.caption"));
	    }
	    catch (JarCacheException ex3)
	    {
		Trace.printException(ex3, ResourceHandler.getMessage("applet_install.jar_cache_error"),
				     ResourceHandler.getMessage("applet_install.caption"));
	    }
	    catch (Throwable ex4)
	    {
		Trace.printException(ex4, ResourceHandler.getMessage("applet_install.general_error"),
				     ResourceHandler.getMessage("applet_install.caption"));
	    }
	}
	finally
	{
	    if (success)
		Trace.println("Applet Installation finished.", TraceFilter.JAVA_CONSOLE_ONLY);
	    else
		Trace.println("Applet Installation failed.", TraceFilter.JAVA_CONSOLE_ONLY);
	}


	// Resolve the archive jar file list. If the cache option is of the type Browser
	// the complete jar file list is the ones in archive and cache_archive tags
	if((cacheOption == null) || (cacheOption.equalsIgnoreCase("browser")) || (success == false)) 
        {
	    if(archive == null)
		archive = cacheArchive;
	    else
		archive = archive + "," + cacheArchive;
	}
      
        ClassLoaderInfo cli = ClassLoaderInfo.find(this);
      
        try 
        {
	    // Prevent two applets trying to load JARS from the same 
	    // classloader at the same time.
	    cli.lock();
	
	    if (!cli.getLocalJarsLoaded() && 
	        (loader instanceof PluginClassLoader) )  
            {
	        ((PluginClassLoader) loader).addCacheJar(getLocalJarFiles());
	        cli.setLocalJarsLoaded(true);
	    }
	
	    // If there are no JARs, this is easy.
	    if (archive == null) 
            {
	        return;
	    } 
      
	    // Figure out which JAR files still need to be loaded.
	    String cooked = "";
	    StringTokenizer st = new StringTokenizer(archive, ",", false);
	    while(st.hasMoreTokens()) 
            {
	        String tok = st.nextToken().trim();
	        if (cli.hasJar(tok)) 
                {
	            continue;
	        }
	        
                cli.addJar(tok);
	        
                if (cooked.equals("")) 
                {
	            cooked = tok;
	        } 
                else 
                {
	            cooked = cooked + "," + tok;
	        }
	    }
	                
            cookedJars = cooked;
	
	    // Now call into our superlcass to do the actual JAR loading.
	    // It will call back to our getJarFiles method to find which
	    // JARs need to be loaded, and we will give it the cooked list.
	    super.loadJarFiles(loader);
        } 
        finally 
        {
	    // Other people can load JARs now.
	    cli.unlock();
        }
    }	  
  
  
  /**
   * Return the value of the object param
   */
  public String getSerializedObject() {
    String object = getParameter("java_object");
    if (object==null) 
      object=getParameter("object");// another name?
    return object;
  }
  
    public Applet getApplet() {
	Applet applet = super.getApplet();
	if (applet!=null) {
	    if (applet instanceof BeansApplet)
		return null;
	    else
		return applet;
	}
	return null;
    }


    /*
     * return the viewed object
     */
    public Object getViewedObject() {
	Applet applet = super.getApplet();
	if (applet instanceof BeansApplet) 
	    return ((BeansApplet) applet).bean;
	else
	    return applet;
    }


    /**
     * Get the applet context. For now this is
     * also implemented by the AppletPanel class.
     */
    public AppletContext getAppletContext() {
	return appletContext; 
    }

    /**
     * Paint this panel while visible and loading an applet to entertain
     * the user.
     *
     * @param g the graphics context
     */
    public void paint(Graphics g)
    {
    	if (getViewedObject() == null && g != null) 
	{
	    String s = getParameter("graybox_message");

	    if (s != null && s.equalsIgnoreCase("true"))
	    {
		Graphics2D g2 = (Graphics2D) g;

		// Use antialiasing
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
				    RenderingHints.VALUE_ANTIALIAS_ON);

		// Set fore ground and background colors
		setBackground(Color.lightGray);
      		g2.setColor(Color.black);

		// Set font
		Font font = g2.getFont();
		font = font.deriveFont(Font.PLAIN, 10);
		g2.setFont(font);

		FontMetrics fm = g2.getFontMetrics(font);	
		Dimension d = getSize();		    

		// Display message on the screen if the applet is not started yet.
		if (d != null && fm != null)
		{
		    String message = null;
		    
		    synchronized(tracingBuffer)
		    { 
			message = tracingBuffer.toString();
		    }

		    // Obtain the number of line in the messages
		    StringTokenizer st = new StringTokenizer(message, "\n", false);
		    int count = 0;

		    while (st.hasMoreTokens())
		    {
			String str = st.nextToken();
			count = count + 1;
		    }

		    // Calculate transformation origin
		    int fmHeight = fm.getHeight();

		    if (fmHeight * count <= d.getHeight())
		    {
			// if height of all displayed text are less than the height
			// of current window, no need to transform
		    }
		    else
		    {
			// if height of all displayed text are more than the height
			// of current window, mimic the scroll operation by transform

			double x = 0;    // No need to transform x-coordinate
			double y = d.getHeight() - fmHeight * count;

    			// Move the origin to x, y
			AffineTransform at = AffineTransform.getTranslateInstance(x, y);
			g2.transform(at);
		    }

	    
		    // Display message one-by-one
		    //
		    st = new StringTokenizer(message, "\n", false);
		    int loc = fmHeight;

		    while(st.hasMoreTokens()) 
		    {
			String str = st.nextToken().trim();
		      
			g.drawString(str, 5, loc);
			loc = loc + fmHeight;
		    }
		}
    	    }
	    else
	    {
		setBackground(Color.lightGray);
      		g.setColor(Color.black);
		Dimension d = getSize();
		Font fontOld = g.getFont();
		FontMetrics fm = null;
		if (fontOld != null)
		    fm = g.getFontMetrics(fontOld);
		String str = getWaitingMessage();

		// Display message on the screen if the applet is not started yet.
		if (d != null && fm != null)
		    g.drawString(str, (d.width - fm.stringWidth(str))/ 2,
				 (d.height + fm.getAscent())/ 2);
    	    }
	}
    }

    public String getWaitingMessage() {
	return getMessage("loading") + getHandledType() + " ...";
    }

    /*
     * <p>
     * Load an applet from a serialized stream. This is likely to happen
     * when the user uses the Back/Forward buttons
     * </p>
     * 
     * @param is Input stream of the serialized applet
     */
    protected void load(java.io.InputStream is) {
	this.is = is;
    }

    protected Applet createApplet(AppletClassLoader loader) throws ClassNotFoundException,
    IllegalAccessException, IOException, InstantiationException, InterruptedException {

	// If we are not in a reload...
	if (is==null)
	    return super.createApplet(loader);

	ObjectInputStream ois =
	    new AppletObjectInputStream(is, loader);
	Object serObject = ois.readObject();
	Applet applet = (Applet) serObject;
	doInit = false; // skip over the first init
	if (Thread.interrupted()) {
	    try {
		status = APPLET_DISPOSE; // APPLET_ERROR?
		applet = null;
		// REMIND: This may not be exactly the right thing: the
	// status is set by the stop button and not necessarily
		// here.
		showAppletStatus("death");
	    } finally {
		Thread.currentThread().interrupt(); // resignal interrupt
		return null;
	    }
	}
	is = null;
	return applet;
    }
    /*
     * @return the applet name
     */
    public String getName()  {

	String name = getParameter("name");
	if (name!=null)
	    return name;

        // Remove .class extension
        name = getCode();
        if (name != null){
            int index = name.lastIndexOf(".class");
            if (index != -1)
                name = name.substring(0, index);
        } else{
            // Remove .object extension
            name = getSerializedObject();

            if (name != null) {
                int index = name.lastIndexOf(".ser");
                if (index != -1)
                    name = name.substring(0, index);
            }
        }

        return name;
    }


    /**
     * @return the java component displayed by this viewer class
     */
    protected String getHandledType() {
	return getMessage("java_applet");
    }

    public void setStatus(int status) {
	this.status = status;
    }

    public void showActivatorAppletStatus(String msg) {
	showAppletStatus(msg);
    }

    public void showActivatorAppletLog(String msg) {
	showAppletLog(msg);
    }

    public void setDoInit(boolean doInit) {
	this.doInit = doInit;
    }

    private ActivatorAppletContext appletContext;


    /**
     * Method to get an internationalized string from the Activator resource.
     */
    public static String getMessage(String key) 
    {
	return sun.plugin.resources.ResourceHandler.getMessage(key);
    }


    /**
     * Method to get an internationalized string from the Activator resource.
     */
    public static String[] getMessageArray(String key) 
    {
	return sun.plugin.resources.ResourceHandler.getMessageArray(key);
    }

    private java.io.InputStream is;

    /**
     * This method actually creates an AppletClassLoader.
     *
     * It can be override by subclasses (such as the Plug-in)
     * to provide different classloaders. This method should be
     * called only when running inside JDK 1.2.
     */    
    protected AppletClassLoader createClassLoader(final URL codebase) {
        return new PluginClassLoader(codebase);
    }

  
  /*
   * <p>
   * @return the default proxy handler
   *</p>
   */
  protected static ProxyHandler getDefaultProxyHandler(int proxyType, 
						       String proxyList, 
						       String proxyOverride) {
    return new ActivatorProxyHandler(proxyType,
				     proxyList,
				     proxyOverride);
  }
  
    /**
     * Called by the AppletPanel to provide
     * feedback when an exception has happened.
     */
    protected void showAppletException(Throwable exc) 
    {
	super.showAppletException(exc);
	
	if (fShowException) 
	{
    	    Trace.printException(exc);
	}
    }

    public void println(String msg, int filter)
    {
	if ((filter & TraceFilter.GRAY_BOX_ONLY) == TraceFilter.GRAY_BOX_ONLY)
	{
	    // Append buffer only if the object is not visible
       	    if (getViewedObject() == null) 
	    {
		synchronized (tracingBuffer)
		{
		    // Message should be displayed in gray box
		    //
		    initTracingBuffer();
		    tracingBuffer.append(msg);
		    tracingBuffer.append("\n");
		    repaint();
		}
	    }
	}
	else if ((filter & TraceFilter.STATUS_BAR_ONLY) == TraceFilter.STATUS_BAR_ONLY)
	{
	    // Message should be displayed in the status bar
	    //
	    showAppletStatus(msg);	    
	}
    }

    private void initTracingBuffer()
    {
	if (tracingBufferInit == false)
	{
	    tracingBufferInit = true;
	    tracingBuffer.append(getWaitingMessage());
	    tracingBuffer.append("\n");
	}
    }

    private StringBuffer tracingBuffer = new StringBuffer();
    private boolean tracingBufferInit = false;
}




