/*
 * @(#)ConsoleWindow.java	1.40 01/02/09
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.plugin;

/*
 * Open a new Java Console used to display the Syste.out and System.err
 * error messages sent by the running Applets or JavaBeans components
 *
 * @version 1.0 
 * @Author Jerome Dochez
 * @author Stanley Man-Kit Ho
 * @Author Devananda Jayaraman
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.TreeSet;
import java.util.Collection;
import java.util.Iterator;
import sun.awt.AppContext;
import sun.plugin.resources.ResourceHandler;
import sun.plugin.usability.Trace;


class ConsoleWindow extends JFrame
{
    /* 
     * Create the new console window
     */
    ConsoleWindow() 
    {
    	super("Java Console");

	try   
	{   
	    // Determine the platform that we are on
	    //
            String osName = (String) java.security.AccessController.doPrivileged(
                                    new sun.security.action.GetPropertyAction("os.name"));

	    if (osName.indexOf("Windows") != -1)
		onWindows = true;

            String workaround = (String) java.security.AccessController.doPrivileged(
                                         new sun.security.action.GetPropertyAction("mozilla.workaround", "false"));

	    if (workaround != null && workaround.equalsIgnoreCase("true"))
		isMozilla = true;
	}
	catch (Exception e)
	{
	    e.printStackTrace();
	}


	setSize(450,400);
	setResizable(true);


	// Make sure we don't do anything exception in the listener.
	//

	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);  

	/* WORKAROUND>>>
	 * There seems to be a major problem with double buffering in swing and Netscape 6.
	 * These two lines are here to turn off buffering until the problem with Netscape 6 
	 * or swing can be resolved	
	 */
	if (onWindows == false && isMozilla == true)
	{
	    getRootPane().setDoubleBuffered(false);
	    ((JPanel) getContentPane()).setDoubleBuffered(false);
	}

	getContentPane().setLayout(new BorderLayout());
     
	textArea = new JTextArea();
	textArea.setEditable(false);
	textArea.setMargin(new Insets(0, 5, 0, 0));
	scroller = new JScrollPane(textArea);
	JViewport viewport = scroller.getViewport();
	viewport.setScrollMode(JViewport.BLIT_SCROLL_MODE);

	sbVer = scroller.getVerticalScrollBar();
	sbHor = scroller.getHorizontalScrollBar();
	getContentPane().add(scroller, BorderLayout.CENTER);

	ActionListener showThreads = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		StringBuffer buffer = new StringBuffer();

		buffer.append("Dump thread list ...\n");

		Thread t = Thread.currentThread();
		ThreadGroup tg = t.getThreadGroup();
		dumpThreadGroup(tg, buffer);

		buffer.append("Done.");

		System.out.println(buffer.toString());
	    }
	};

	ActionListener showSystemProperties = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		displaySystemProperties();
	    }
	};

	ActionListener showHelp = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		displayHelp();
	    }
	};

	ActionListener showClassLoaderCache = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		ClassLoaderInfo.dumpClassLoaderCache(System.out);
	    }
	};

	ActionListener clearClassLoaderCache = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		ClassLoaderInfo.clearClassLoaderCache();

    		System.out.println("Clear classloader cache .... completed.");
	    }
	};

	ActionListener clearConsole = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
 		textArea.setText("");
		textArea.revalidate();

		// Display version info
		displayVersion();

		// Display help information after the screen is cleared.
		displayHelp();
	    }
	};

	ActionListener copyConsole = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		// Obtain current text selections
		int selectionStart = textArea.getSelectionStart();
		int selectionEnd = textArea.getSelectionEnd();

		// If no selection, copy all
		if (selectionEnd - selectionStart <= 0)
		    textArea.selectAll();

		// Copy text to clipboard
		textArea.copy();
	    }
	};

	ActionListener closeConsole = new ActionListener() 
	{ 
	    public void actionPerformed(ActionEvent e)  
	    {
		if (onWindows || isMozilla)
		{
		    setVisible(false);
		}
		else
		{
		    // On other platform with Netscape 4.x, closing the window
		    // will only iconified the console.
		    //
		    setState(Frame.ICONIFIED);
		}

		JavaRunTime.ConsoleStatus(false);
	    }
	};

	final ActionListener showMemory = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		long freeMemory = Runtime.getRuntime().freeMemory() / 1024;
		long totalMemory = Runtime.getRuntime().totalMemory() / 1024;
		long percentFree = (long) (100.0 / (((double)totalMemory) / freeMemory));

    		System.out.println("Memory: " + totalMemory + "K  Free: " + freeMemory + "K  (" + percentFree + "%) ... completed.");
	    }
	};


	ActionListener runFinalize = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		System.out.print("Finalize objects on finalization queue ");
		System.runFinalization();
		System.out.println("... completed.");
    
		// Display memory after finalization
		showMemory.actionPerformed(e);
	    }
	};


	ActionListener runGC = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
    		System.out.print("Garbage collect ");
		System.gc();
		System.out.println("... completed.");
	    
		// Display memory after garbage collection
		showMemory.actionPerformed(e);
	    }
	};

	ActionListener traceLevel0 = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		Trace.setBasicTrace(false);
		Trace.setNetTrace(false);
		Trace.setSecurityTrace(false);
		Trace.setExtTrace(false);
		Trace.setLiveConnectTrace(false);
    		System.out.println("Trace level set to 0: none ... completed.");
	    }
	};

	ActionListener traceLevel1 = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		Trace.setBasicTrace(true);
		Trace.setNetTrace(false);
		Trace.setSecurityTrace(false);
		Trace.setExtTrace(false);
		Trace.setLiveConnectTrace(false);
    		System.out.println("Trace level set to 1: basic ... completed.");
	    }
	};

	ActionListener traceLevel2 = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		Trace.setBasicTrace(true);
		Trace.setNetTrace(true);
		Trace.setSecurityTrace(false);
		Trace.setExtTrace(false);
		Trace.setLiveConnectTrace(false);
    		System.out.println("Trace level set to 2: basic, net ... completed.");
	    }
	};

	ActionListener traceLevel3 = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		Trace.setBasicTrace(true);
		Trace.setNetTrace(true);
		Trace.setSecurityTrace(true);
		Trace.setExtTrace(false);
		Trace.setLiveConnectTrace(false);
    		System.out.println("Trace level set to 3: basic, net, security ... completed.");
	    }
	};

	ActionListener traceLevel4 = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		Trace.setBasicTrace(true);
		Trace.setNetTrace(true);
		Trace.setSecurityTrace(true);
		Trace.setExtTrace(true);
		Trace.setLiveConnectTrace(false);
    		System.out.println("Trace level set to 4: basic, net, security, ext ... completed.");
	    }
	};

	ActionListener traceLevel5 = new ActionListener() 
	{
	    public void actionPerformed(ActionEvent e)  
	    {
		Trace.setBasicTrace(true);
		Trace.setNetTrace(true);
		Trace.setSecurityTrace(true);
		Trace.setExtTrace(true);
		Trace.setLiveConnectTrace(true);
    		System.out.println("Trace level set to 5: basic, net, security, ext, liveconnect ... completed.");
	    }
	};

	textArea.registerKeyboardAction(    showThreads, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_T, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    showSystemProperties, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_S, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    showHelp, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_H, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    showMemory, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_M, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    clearConsole, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_C, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    runGC, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_G, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    runFinalize, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_F, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    closeConsole, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_Q, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    traceLevel0, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_0, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    traceLevel1, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_1, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    traceLevel2, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_2, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    traceLevel3, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_3, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    traceLevel4, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_4, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    traceLevel5, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_5, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    clearClassLoaderCache, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_X, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);
	textArea.registerKeyboardAction(    showClassLoaderCache, 
					    KeyStroke.getKeyStroke(KeyEvent.VK_L, 0), 
					    JComponent.WHEN_IN_FOCUSED_WINDOW	);

	/* Get our resource strings.  We are guaranteed that ResourceHandler
         * has already loaded its resource because the same method that
         * did the resource loading subsequently calls showConsoleWindow.
         */
	JButton clear = new JButton(ResourceHandler.getMessage("console.clear"));
	JButton copy = new JButton(ResourceHandler.getMessage("console.copy"));
	JButton close = new JButton(ResourceHandler.getMessage("console.close"));

	JPanel panel = new JPanel();
	panel.setLayout(new FlowLayout(FlowLayout.CENTER));
	panel.add(clear);
	panel.add(new Label("    "));
	panel.add(copy);
	panel.add(new Label("    "));
	panel.add(close);

	// We use the background RGB from the SystemColor.control
	int rgb = SystemColor.control.getRGB();
	panel.setBackground(new Color(rgb));
	getContentPane().add(panel, BorderLayout.SOUTH);

        addWindowListener(  new WindowAdapter(){
 
		public void windowDeiconified(WindowEvent e) 
		{
		    // If windows is deiconified, notified
		    if (onWindows == false && isMozilla == false)
			JavaRunTime.ConsoleStatus(true);		
		}
		public void windowIconified(WindowEvent e) 
		{
		    if (onWindows == false && isMozilla == false)
    			JavaRunTime.ConsoleStatus(false);
		}
		public void windowClosing(WindowEvent e)
		{
		    if (onWindows || isMozilla)
		    {
			setVisible(false);
		    }
		    else
		    {
			// On other platform, closing the window
			// will only iconified the console.
			//
			setState(Frame.ICONIFIED);
		    }

		    JavaRunTime.ConsoleStatus(false);
		}
	});

	clear.addActionListener(clearConsole);
	copy.addActionListener(copyConsole);
	close.addActionListener(closeConsole);

	//Remember the queue to post the console events
	consoleEventQueue = (java.awt.EventQueue)
			    AppContext.getAppContext().get(AppContext.EVENT_QUEUE_KEY);


	// Remember the default toolkit
	toolkit = Toolkit.getDefaultToolkit();

/*   
        Container top = clear.getParent();
        while(top != null) 
	{
	    System.out.println(top.isDoubleBuffered()+"  " + top.toString());
	    System.out.println(" ");
	    top = top.getParent();
        }
*/
    }

    /**
     * <p>
     * Prints an output message to the console window
     * </p>
     *
     * @param text text to be printed
     */
    public void append(final String text) 
    {
	final Toolkit defaultToolkit = toolkit;

	// We need to post the event to the swing component's event queue   
	consoleEventQueue.postEvent(
		new InvocationEvent(defaultToolkit,
		    new Runnable(){   
			public void run() 
			{
			    textArea.append(text);
			    textArea.revalidate();
			    setScrollPosition();
			}
		    }
		));
    }

    /**
     * Dump threads in threadgroup recursively.
     *
     * @param tg Root of thread group tree.
     * @param buffer String buffer to append output
     */
    private void dumpThreadGroup(ThreadGroup tg, StringBuffer sb)
    {
	if (tg != null)
	{
	    // Clean up thread group if necessary.
	    try
	    {
		if (tg.activeCount() == 0 && tg.activeGroupCount() == 0 && tg.isDestroyed() == false)
		    tg.destroy();
	    }
	    catch (Throwable e)
	    {
	    }

	    // Dump thread group
	    sb.append("Group " + tg.getName());
	    sb.append(",ac=" + tg.activeCount());
	    sb.append(",agc=" + tg.activeGroupCount());
	    sb.append(",pri=" + tg.getMaxPriority());
	    if (tg.isDestroyed())
		sb.append(",destoyed");
	    if (tg.isDaemon())
		sb.append(",daemon");
	    sb.append("\n");

	    Thread[] tt = new Thread[1000];
	    tg.enumerate(tt, false);

	    // Dump threads within thread group
	    for (int i=0; i < tt.length; i++)
	    {
		if (tt[i] != null)
		{
		    sb.append("    ");
		    sb.append(tt[i].getName());
		    sb.append(",");
		    sb.append(tt[i].getPriority());

		    if (tt[i].isAlive())
			sb.append(",alive");
		    else
			sb.append(",not alive");

		    if (tt[i].isDaemon())
			sb.append(",dameon");

		    if (tt[i].isInterrupted())
			sb.append(",interrupted");

		    sb.append("\n");
		}
	    }

	    // Obtain sub-thread group
	    ThreadGroup[] tgArray = new ThreadGroup[1000];
	    tg.enumerate(tgArray, false);

	    // Dump sub-thread group
	    for (int i=0; i < tgArray.length; i++)
	    {
		if (tgArray[i] != null)
		    dumpThreadGroup(tgArray[i], sb);
	    }
	}
    }


    /**
     * Display help information.
     */
    public void displayHelp()
    {
	StringBuffer buffer = new StringBuffer();

	buffer.append("----------------------------------------------------\n");
	buffer.append("c:   clear console window\n");
	buffer.append("f:   finalize objects on finalization queue\n");
	buffer.append("g:   garbage collect\n");
	buffer.append("h:   display this help message\n");
	buffer.append("l:   dump classloader list\n");
	buffer.append("m:   print memory usage\n");
	buffer.append("q:   hide console\n");
	buffer.append("s:   dump system properties\n");
	buffer.append("t:   dump thread list\n");
	buffer.append("x:   clear classloader cache\n");
	buffer.append("0-5: set trace level to <n>\n");
	buffer.append("----------------------------------------------------");

	System.out.println(buffer.toString());
    }


    public void displayVersion()
    {
	// Tell people a little about ourselves.  This is partly so we can
	// get better information when people report problems.

	StringBuffer buffer = new StringBuffer();

	buffer.append(ResourceHandler.getMessage("product_name"));

	String javapluginVersion = System.getProperty("javaplugin.version");

	if (javapluginVersion != null) 
	{
	    buffer.append(": " + ResourceHandler.getMessage("version") + " " 
			  + System.getProperty("javaplugin.version"));
	}
	buffer.append("\n");
	buffer.append(ResourceHandler.getMessage("using_jre_version"));
	buffer.append(" ");
	buffer.append(System.getProperty("java.version"));
	buffer.append(" ");
	buffer.append(System.getProperty("java.vm.name"));
	buffer.append("\n");
	buffer.append(ResourceHandler.getMessage("user_home_dir"));
	buffer.append(" = ");
	buffer.append(System.getProperty("user.home"));

	System.out.println(buffer.toString());
    }


    public void displaySystemProperties()
    {
	// Obtain all properties name and sort them first
	//
	TreeSet treeSet = new TreeSet();

	for (java.util.Enumeration e = System.getProperties().propertyNames(); e.hasMoreElements();) 
	{
	    treeSet.add(e.nextElement());
	} 


	StringBuffer buffer = new StringBuffer();


	buffer.append("Dump system properties ...\n");
	buffer.append("---------------------------------------------------\n");

	// Iterate the sorted properties
	//	
	for (Iterator iter = treeSet.iterator(); iter.hasNext();) 
	{
	    String key = (String) iter.next();
	    String value = System.getProperty(key);

	    if (value != null)
    	    {
		if (value.equals("\n"))
		    value = "\\n";
		else if (value.equals("\r"))
		    value = "\\r";
		else if (value.equals("\r\n"))
		    value = "\\r\\n";
		else if (value.equals("\n\r"))
		    value = "\\n\\r";
		else if (value.equals("\n\n"))
		    value = "\\n\\n";
		else if (value.equals("\r\r"))
		    value = "\\r\\r";
	    }

	    buffer.append(key + " = " + value + "\n");
	}   

	buffer.append("---------------------------------------------------\n");
	buffer.append("Done.");

	System.out.println(buffer.toString());
    }    
    
    /**
     * Set the horizontal and vertical scroll bar positions  
     */
    public void setScrollPosition()
    {
 	scroller.validate();
	sbVer.setValue(sbVer.getMaximum());
	sbHor.setValue(sbHor.getMinimum());
    }

    private static EventQueue consoleEventQueue;
    private JTextArea textArea;
    private JScrollPane scroller;    
    private JScrollBar sbHor, sbVer;
    private boolean onWindows = false;
    private boolean isMozilla = false;
    private Toolkit toolkit = null;
}    
