/*
 * @(#)PluginAuthenticator.java	1.4 01/02/11
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *  @version 1.3
 *
 *  @author Jerome Dochez
 *  @author Stanley Man-Kit Ho
 *
 */

package sun.plugin;

import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.util.HashMap;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Box;
import sun.plugin.resources.ResourceHandler;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;
import sun.plugin.usability.DialogFactory;



/** 
 * Assist proxy authentication and web server authentication.
 * A unique instance of this class is registered at startup by the plugin
 * @see AppletViewer#initEnvironment
 * 
 * @see java.net.Authenticator
 * @see java.net.PasswordAuthentication
 */
public class PluginAuthenticator extends Authenticator {

    /**
     * <p>
     * Called when password authorization is needed by the superclass. The
     * instance is registered as being the default http proxy authentication
     * facility and will be called by the HTTP framework classes.
     * </p>
     * @return The PasswordAuthentication collected from the
     *		user, or null if none is provided.
     */
    protected synchronized PasswordAuthentication getPasswordAuthentication()
    {
	PasswordAuthentication pa = null; 	
	
	try
	{
	    // We have never seen this site before.

	    // Print out tracing 
	    StringBuffer buffer = new StringBuffer();
	    buffer.append("Firewall authentication: site=");
	    buffer.append(getRequestingSite());
	    buffer.append(":" + getRequestingPort());
	    buffer.append(", protocol=");
	    buffer.append(getRequestingProtocol());
	    buffer.append(", prompt=");
	    buffer.append(getRequestingPrompt());
	    buffer.append(", scheme=");
	    buffer.append(getRequestingScheme());

	    Trace.netPrintln(buffer.toString());
		
	    // Request the username/password from the user
	    pa = openDialog(getRequestingSite().toString(), getRequestingPrompt(), getRequestingScheme());
	}
	catch (Exception e)
	{
	    // We should catch all exception so the connection may continue
	    Trace.netPrintException(e);
	}

	return pa; 
    }

    /*
     * <p>
     * Open the dialog to request the user/password information 
     * from the user. The dialog box is based on Internet Explorer 4
     * dialog minus the icon
     * </p>
     *
     * @param site the HTTP site we are trying to connect
     * @param prompt the HTTP prompt from the server
     * @param scheme the HTTP scheme
     * @return the PasswordAuthentication object encapsulating the 
     * username/password entered by the user
     */    
    private PasswordAuthentication openDialog(String site, String prompt, String scheme) 
    {
	if (site == null)
	    site = "";

	if (prompt == null)
	    prompt = "";

	if (scheme == null)
	    scheme = "";

	
	JTextField userName = new JTextField();
	userName.setColumns(15);
	JPasswordField password = new JPasswordField();
	password.setColumns(15);

	JPanel optionPane = new JPanel();
	optionPane.setLayout(new BoxLayout(optionPane, BoxLayout.Y_AXIS));
	

	optionPane.add(new JLabel(getMessage("net.authenticate.firewall")));
	optionPane.add(Box.createVerticalStrut(15));
	optionPane.add(new JLabel(getMessage("net.authenticate.realm")));
	optionPane.add(Box.createVerticalStrut(15));
	optionPane.add(new JLabel(getMessage("net.authenticate.scheme")));
	optionPane.add(Box.createVerticalStrut(15));
	optionPane.add(new JLabel(getMessage("net.authenticate.username")));
	optionPane.add(Box.createVerticalStrut(20));
	optionPane.add(new JLabel(getMessage("net.authenticate.password")));
	optionPane.add(Box.createVerticalStrut(5));
	JPanel optionPane2 = new JPanel();
	optionPane2.setLayout(new BoxLayout(optionPane2, BoxLayout.Y_AXIS));
	optionPane2.add(new JLabel(site));
	optionPane2.add(Box.createVerticalStrut(15));
	optionPane2.add(new JLabel(prompt));
	optionPane2.add(Box.createVerticalStrut(15));
	optionPane2.add(new JLabel(scheme));
	optionPane2.add(Box.createVerticalStrut(15));
	optionPane2.add(userName);
	optionPane2.add(Box.createVerticalStrut(15));
	optionPane2.add(password);
	JPanel finalPane = new JPanel();
	finalPane.setLayout(new BoxLayout(finalPane, BoxLayout.X_AXIS));
	finalPane.add(optionPane);
	finalPane.add(Box.createHorizontalStrut(15));
	finalPane.add(optionPane2);
	
	Object[] objects = new Object[] {
//	    new JLabel(getMessage("net.authenticate.label")),
	    finalPane};

	int result = DialogFactory.showConfirmDialog(objects, getMessage("net.authenticate.title"));	    
	
	if (result == JOptionPane.OK_OPTION) 
	{
	    PasswordAuthentication pa;
	    pa = new PasswordAuthentication(userName.getText(), 
					    password.getPassword());

	    return pa;
	    
	} else {
	    return null;
	}
	
    }

    /*
     * <p>
     * Helper method to load resources for i18n
     * </p>
     */
    private String getMessage(String key) {
	return ResourceHandler.getMessage(key);
    }
	
}



