/*
 * @(#)HTMLGenerator.java	1.6 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.cachescheme;

import java.util.*;
import java.io.*;
import java.security.*;


/**
 * HTMLGenerator is used for generating installed HTML page for applet 
 * installation.
 */ 
public class HTMLGenerator 
{
    /**
     * Generate HTML page for applet installation.
     *
     * @param params OBJECT/EMBED tag's PARAMs
     * @param f Generated HTML file
     */
    public void generate(Hashtable params,
			 File f)
		throws IOException
    {
	final String html = generateHTML(params);

	final File file = f;

	// Open file and generate the HTML page		
	try
	{
	    AccessController.doPrivileged(new PrivilegedExceptionAction() {
			    
    		public Object run() throws IOException
		{
		    File parent = file.getParentFile();
		    parent.mkdirs();

		    FileOutputStream fos = new FileOutputStream(file);
		    BufferedOutputStream bos = new BufferedOutputStream(fos);
		    PrintWriter pw = new PrintWriter(bos, true);
		    pw.write(html);
		    pw.close();
		    bos.close();
		    fos.close();

		    return null;
		}
	    });
	}
	catch (PrivilegedActionException e)
	{
	    IOException ioe = (IOException) e.getException();
	    throw ioe;
	}
    }


    /**
     * Generate HTML text for applet installation
     *
     * @param inparams OBJECT/EMBED tag's PARAMs
     * @return generated HTML text
     */
    private String generateHTML(Hashtable inparams)
    {
	Hashtable params = (Hashtable) inparams.clone();

	String applet_html_title = (String) params.get("applet_html_title");
	String applet_object_clsid = (String) params.get("applet_object_clsid");
	String applet_object_codebase = (String) params.get("applet_object_codebase");
	String applet_embed_pluginpage = (String) params.get("applet_embed_pluginpage");
	String applet_embed_pluginsurl = (String) params.get("applet_embed_pluginsurl");

	StringBuffer buffer = new StringBuffer();
	
	// Emit HTML title
	buffer.append("<HTML>");					
	buffer.append('\n');
	buffer.append("<TITLE>");					
	buffer.append(applet_html_title);					
	buffer.append("</TITLE>");					
	buffer.append('\n');
	buffer.append("<BODY>");					
	buffer.append('\n');

	// Emit top portion of OBJECT tag
	buffer.append("<OBJECT classid=\"clsid:");
	buffer.append(applet_object_clsid);
	buffer.append("\" width=");
	buffer.append(params.get("width"));
	buffer.append(" height=");
	buffer.append(params.get("height"));

	if (applet_object_codebase != null)
	{
	    buffer.append(" codebase=");
	    appendWithQuote(buffer, applet_object_codebase);
	}

	buffer.append(">");
	buffer.append('\n');

	// Emit PARAMS of OBJECT tag
	Enumeration keyEnum = params.keys();

	while (keyEnum.hasMoreElements())
	{
	    String name = (String) keyEnum.nextElement();
	    String value = (String) params.get(name);

	    if (name != null
	        && value != null
		&& name.equalsIgnoreCase("width") == false 
	        && name.equalsIgnoreCase("height") == false)
	    {
		buffer.append("<PARAM NAME=");
		buffer.append(name);
		buffer.append(" VALUE=");
		appendWithQuote(buffer, value);
		buffer.append(">");
    		buffer.append('\n');
	    }
	}

	buffer.append("<COMMENT>");
	buffer.append('\n');

	// Emit EMBED tag
	buffer.append("<EMBED type=");
	
	// Specify "type" first
	appendWithQuote(buffer, (String) params.get("type"));
	params.remove("type");

	// Specify "width"
	buffer.append(" width=");
	buffer.append(params.get("width"));
	params.remove("width");

	// Specify "height"
	buffer.append(" height=");
	buffer.append(params.get("height"));
	params.remove("height");

	// Specify "pluginpage"
	if (applet_embed_pluginpage != null) 
	{
	    buffer.append(" pluginpage=");
	    appendWithQuote(buffer, applet_embed_pluginpage);
	}
	params.remove("pluginpage");

	// Specify "pluginsurl"
	if (applet_embed_pluginsurl != null) 
	{
	    buffer.append(" pluginsurl=");
	    appendWithQuote(buffer, applet_embed_pluginsurl);
	}
	params.remove("pluginsurl");

	// Emit the rest of the parameters in EMBED tag
	// Emit PARAMS of OBJECT tag
	keyEnum = params.keys();

	while (keyEnum.hasMoreElements())
	{
	    String name = (String) keyEnum.nextElement();
	    String value = (String) params.get(name);

	    if (name != null && value != null)
	    {
		buffer.append(" ");
		buffer.append(name);
		buffer.append("=");
		appendWithQuote(buffer, value);
	    }
	}

	buffer.append(">");
	buffer.append('\n');
	buffer.append("</COMMENT>");
	buffer.append('\n');
	buffer.append("</OBJECT>");
	buffer.append('\n');

	buffer.append("</BODY>");					
	buffer.append('\n');
	buffer.append("</HTML>");					
	buffer.append('\n');


	return buffer.toString();
    }

    /**
     * Append string to string buffer with quote.
     *
     * @param buffer String buffer to be appended
     * @param value String to append
     */
    public void appendWithQuote(StringBuffer buffer, String value)
    {
	if (value != null && value.indexOf('\"') == -1)
	{
	    buffer.append('\"');
    	    buffer.append(value);
	    buffer.append('\"');
	}
	else
	{
    	    buffer.append(value);
	}
    }
}


