/*
 * @(#)HTMLInstaller.java	1.15 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.cachescheme;

import java.util.*;
import java.io.*;
import java.lang.String;
import java.net.*;
import javax.swing.*;
import sun.plugin.resources.ResourceHandler;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;
import sun.plugin.usability.DialogFactory;


public class HTMLInstaller 
{
    public boolean install(Hashtable atts, URL codeBase)
		   throws AppletInstallationException, EntryNotFoundException, IOException
    {
	Trace.println("Determine if the applet requests to install any HTML page", TraceFilter.JAVA_CONSOLE_ONLY);

	Hashtable params = (Hashtable) atts.clone();
	
	// Remove codebase and java_codebase from params
	params.remove("codebase");
	params.remove("java_codebase");

	// Reset codebase in params
	params.put("codebase", codeBase.toString());

	String applet_html_title = (String) params.get("applet_html_title");
	String applet_html_version = (String) params.get("applet_html_version");
	String applet_object_clsid = (String) params.get("applet_object_clsid");
	String applet_object_codebase = (String) params.get("applet_object_codebase");
	String applet_embed_pluginpage = (String) params.get("applet_embed_pluginpage");
	String applet_embed_pluginsurl = (String) params.get("applet_embed_pluginsurl");
	String applet_type = (String) params.get("type");
	String applet_code = (String) params.get("code");
	String applet_codebase = (String) params.get("codebase");
    
	// Applet installation is not specified
	if (applet_html_title == null)
	    return false;

	// Error checking
	if (applet_html_version == null)
	    throw new AppletInstallationException("\"applet_html_version\" is missing from HTML parameter");

	if (applet_object_clsid == null)
	    throw new AppletInstallationException("\"applet_object_clsid\" is missing from HTML parameter");

	if (applet_type == null)
	    throw new AppletInstallationException("\"type\" is missing from HTML parameter");

	if (applet_code == null)
	    throw new AppletInstallationException("\"code\" is missing from HTML parameter");

	// Mangle HTML title
	String mangledHTMLTitle = mangleName(applet_html_title);

	// Check if the new version is more updated than the installed one
	try
	{
	    Trace.println("Checking local version number of the HTML page", TraceFilter.JAVA_CONSOLE_ONLY);

	    // First, check to see if the remote version is valid
	    long remoteVersion = calculateVersion(applet_html_version);

	    // Load local HTML version table to determine local version
	    HTMLVersionTable versionTable = new HTMLVersionTable();
	    versionTable.load();

	    String installedVersion = versionTable.getVersion(applet_html_title);

	    Trace.println("Remote HTML page version=" + applet_html_version 
			+ ", local HTML page version=" + installedVersion, TraceFilter.JAVA_CONSOLE_ONLY);

	    if (calculateVersion(installedVersion) >= remoteVersion)
	    {
		String path = versionTable.getPath(applet_html_title);
		String filename = versionTable.getFilename(applet_html_title);

		File file = new File(path, filename);

		if (file.exists() == true)
		{
		    Trace.println("Local HTML page is more up-to-dated - no need to install", TraceFilter.JAVA_CONSOLE_ONLY);
		    return true;
		}
		else
		{
		    Trace.println("Local HTML page doesn't exist anymore - regenerate", TraceFilter.JAVA_CONSOLE_ONLY);
		}
	    }
	    else
	    {
		Trace.println("Remote HTML page is more up-to-dated - ask user to install", TraceFilter.JAVA_CONSOLE_ONLY);
	    }
	}
	catch (EntryNotFoundException e)
	{
	    Trace.println("Info about local HTML page in the version table is missing - install again", TraceFilter.JAVA_CONSOLE_ONLY);
	}		

	// Prompt user
	if (promptUserForHTMLInstallation() == false)
	{
	    Trace.println("User has refused to install HTML page - abort HTML installation", TraceFilter.JAVA_CONSOLE_ONLY);
	    return true;
	}


	// Determine local HTML page filename
	//
	String userHome = (String) java.security.AccessController.doPrivileged(
	   new sun.security.action.GetPropertyAction("user.home"));

	// Determine filename of the version table
	String path = userHome 
		+ File.separator + relativeCacheDir
		+ File.separator + mangledHTMLTitle;

	String filename = "index.html";

	// Install HTML page
	installHTML(applet_html_title,
		    applet_html_version,
		    path,
		    filename,
		    params);
    
	return true;
    }  


    /**
     * Calculate the actual version from a given applet HTML version
     * in X.X.X.X form.
     *
     * @param applet_html_version Applet HTML version
     * @return Actual version
     */    
    public long calculateVersion(String applet_html_version)
		throws AppletInstallationException
    {
	long ver = 0;

	// Parse the version string
        StringTokenizer st = new StringTokenizer(applet_html_version, ".", false);

	if (st.countTokens() != 4)
	{
	    throw new AppletInstallationException("\"applet_html_version\" " + applet_html_version
						  + " must be in X.X.X.X form where X is 0 to F");
	}

	try
	{
	    while(st.hasMoreTokens()) 
	    {
		String tok = st.nextToken().trim();
	      
		ver = ver * 16;
		ver = ver + Integer.parseInt(tok, 16);
	    }
	}
	catch(Throwable e)
	{
	    throw new AppletInstallationException("\"applet_html_version\" " + applet_html_version
						  + " must be in X.X.X.X form where X is 0 to F");
	}

	return ver;
    }


    /**
     * Mangle HTML title name.
     * 
     * @param str HTML title
     * @return mangled name.
     */
    public String mangleName(String str)
		  throws AppletInstallationException
    {
	if (str.indexOf("/") != -1 
	    || str.indexOf("\\") != -1
	    || str.indexOf(":") != -1
	    || str.indexOf("*") != -1
	    || str.indexOf("?") != -1
	    || str.indexOf("\"") != -1
	    || str.indexOf("<") != -1
	    || str.indexOf(">") != -1
	    || str.indexOf("|") != -1)
	    throw new AppletInstallationException("\"applet_html_title\" can not contain the following characters: / \\ : * ? \" < > |");

	// Mangle all spaces into _
	StringBuffer buffer = new StringBuffer();

	for (int i=0; i < str.length(); i++)
	{
	    char c = str.charAt(i);

	    if (c == ' ')
		buffer.append('_');
	    else 
		buffer.append(c);
	}

	return buffer.toString();
    }


    /**
     * <P> Install HTML page in the local machine.
     * </P>
     *
     * @param applet_html_title Applet HTML title
     * @param applet_html_version Applet HTML version
     * @param path Path to local HTML page
     * @param filename Filename of local HTML page
     * @param params Parameters to be used to generate local HTML page
     */
    private void installHTML(String applet_html_title,
		 	     String applet_html_version,
			     String path,
			     String filename,
			     Hashtable params)
		 throws IOException
    {
	// Generate HTML page

	Trace.println("Generating local HTML page ...", TraceFilter.JAVA_CONSOLE_ONLY);
	HTMLGenerator generator = new HTMLGenerator();
	generator.generate(params, new File(path, filename));

	// Update version table
	Trace.println("Updating version info about local HTML page ...", TraceFilter.JAVA_CONSOLE_ONLY);

	HTMLVersionTable versionTable = new HTMLVersionTable();
	versionTable.load();
	versionTable.add(applet_html_title, applet_html_version, path, filename);
	versionTable.save();

	// Install bookmark
	Trace.println("Adding bookmark to the desktop ...", TraceFilter.JAVA_CONSOLE_ONLY);

	BookmarkManager bookmarkManager = new BookmarkManager();
	bookmarkManager.addBookmark(applet_html_title, path + File.separator + filename);
    }


    /**
     * Prompt user for HTML installation.
     */
    private boolean promptUserForHTMLInstallation() 
    {
	String title = ResourceHandler.getMessage("html.caption");
	String message = ResourceHandler.getMessage("html.prompt_user");

	int result = 0;
	
	if (Trace.isAutomationEnabled() == false)
	{
	    result = DialogFactory.showConfirmDialog(message, title);
	}
	else
	{
	    Trace.println("Automation: Accept HTML Installation", TraceFilter.JAVA_CONSOLE_ONLY);
	    result = 0;
	}

	return (result == 0);
    }

    public static String relativeCacheDir = PluginCacheTable.cacheHome
					    + File.separator + System.getProperty("javaplugin.version")
					    + File.separator + "html";
}


