/*
 * @(#)PluginCacheTable.java	1.3 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 
 *
 * @author Devananda J. based on stanley's HTMLVersionTable.java
 */

package sun.plugin.cachescheme;

import java.util.*;
import java.io.*;
import java.net.*;
import sun.plugin.usability.Trace;

/* 
 * Version table for storing installed HTML page information.
 */
public class PluginCacheTable {

    // Version table m_fileName
    private String m_fileName = null;

    static String userHome = (String) java.security.AccessController.doPrivileged(
					new sun.security.action.GetPropertyAction("user.home"));

    public static String cacheHome =  userHome + File.separator + "java_plugin_AppletStore" + 
	  			      File.separator + System.getProperty("javaplugin.version");
				      

    private String fileText = "This File is Machine Generated." +
		 	      "Please DO NOT CHANGE IT !! " +
			      "Changing it will BREAK the Applet Installation Cache. " +
			      "This will inturn cause the PLUGIN to break";

    // Properties to store version information
    Properties m_cacheTable = null;

    public PluginCacheTable(String subPath, String tableFileName)
    {
	m_cacheTable = new Properties();
	m_fileName = cacheHome + File.separator + subPath +  File.separator + tableFileName;

	File cacheHomeDir = new File(cacheHome + File.separator + subPath);
	
	if(!cacheHomeDir.exists())
	{
	    if(!cacheHomeDir.isDirectory()) 
	    {
		cacheHomeDir.mkdirs();
	    }
	}
    }
    
    /**
     * Load version information from version table file.
     */    
    public void load()throws IOException, PluginMutexException
    {	
	// Open version table file
	File file = new File(m_fileName);

	PluginMutex tableMutex = null;

	// Load version info only if file exists	
	if (file.exists())
	{	   
	    try
	    {
		//acquire a lock so that cachetable is not inconsistent
		tableMutex = new PluginMutex("CacheTable");
		tableMutex.waitForLock();

		FileInputStream fin = new FileInputStream(m_fileName);
		m_cacheTable.load(new BufferedInputStream(fin));
		fin.close();
	    }
	    finally
	    {
		//allow others to save the table
		if(tableMutex != null)
		    tableMutex.release();
	    }
	}
	else
	{
	    // If the cache table doesn't exists, remove the jar files 
	    // since they are of no use
      	    File[] fList = file.getParentFile().listFiles();
	    if(fList != null)
	    {
		for(int i=0; i<fList.length; i++)
		{ 
		    if(fList[i].isFile() && fList[i].getName().endsWith(".jar"))
			fList[i].delete();
		}
	    }
	}
    }

    /**
     * Save version information into version table file.
     */    
    public void save()throws IOException, PluginMutexException
    {
	PluginMutex tableMutex = null;

	try{
	    //acquire a lock so that cachetable is not inconsistent
	    tableMutex = new PluginMutex("CacheTable");
	    tableMutex.waitForLock();

	    //make sure you have the latest cache table by reloading it
	    File file = new File(m_fileName);

	    Properties props = new Properties();

	    // Load version info only if file exists	
	    if (file.exists())
	    {	   
		FileInputStream fin = new FileInputStream(m_fileName);
		props.load(new BufferedInputStream(fin));
		fin.close();
	    }

	    Enumeration keyElems = m_cacheTable.propertyNames();
	    for(;keyElems.hasMoreElements();)
	    {
		String key = keyElems.nextElement().toString();
		props.put(key, m_cacheTable.get(key));		
	    }

	    // Open version table file and store the table
	    BufferedOutputStream bos = 
		    new BufferedOutputStream(new FileOutputStream(m_fileName));
	    props.store(bos, fileText);
	    bos.close();
	}
	finally{
	    //allow others to save the table
	    if(tableMutex != null)
		tableMutex.release();
	}
    }

    
    /**
     * Add version information into version table file.
     * @param key -key
     * @param entry  -value
     */    
    public void add(String key, String entry)
    {
	m_cacheTable.put(key, entry);
    }


    /**
     * Add version information into version table file.
     * @param key -key
     */    
    public void remove(String key)
    {
	m_cacheTable.remove(key);
    }

    /**
     * Get version entry from version table.
     * @param key -key
     * @return entry for the key
     */    
    protected String getTableEntry(String key)
    {
	String entry = (String)m_cacheTable.get(key);

	return entry;
    }

    /**
     * Get version entry from version table.
     * @param key -key
     * @return entry for the key
     */    
    public void reset()
    {
	m_cacheTable.clear();
    }
}




