/*
 * @(#)PluginJarCacheTableEntry.java	1.8 00/09/16
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 
 *
 * @author Devananda J.
 */

package sun.plugin.cachescheme;

import java.io.*;
import java.util.*;
import sun.plugin.usability.Trace;

/* 
 * Entry in the plug-in jar cache table
 */
public class PluginJarCacheTableEntry {

    // delimiting character for version entry
    private static String delimitor = "#";

    //parses the given entry into actual values
    PluginJarCacheTableEntry(String entry) throws CacheTableEntryException
    {
	StringTokenizer st = new StringTokenizer(entry, delimitor, false);

	if (st.countTokens() != 4)
	    throw new CacheTableEntryException("Entry is found invalid");

	// Skip first token
	m_entry = entry;
	m_versionString = st.nextToken().trim();
	m_fileName  = st.nextToken().trim();

	try
	{
	    m_timeStamp = Long.parseLong(st.nextToken().trim(), 10);
	    m_fileSize  = Integer.parseInt(st.nextToken().trim(), 10);
	}
	catch(Exception exc)
	{
	    throw new CacheTableEntryException("Entry is found invalid");
	}
    }

    //Creates a cache table entry given the elements
    public static PluginJarCacheTableEntry createEntry(String version, String fileName, 
		    long timeStamp, int fileSize)throws CacheTableEntryException
    {
	String entry = version + delimitor + fileName + delimitor + String.valueOf(timeStamp)
		       + delimitor + String.valueOf(fileSize);
	PluginJarCacheTableEntry cacheEntry = new PluginJarCacheTableEntry(entry);
	return cacheEntry;
    }

    public String toString()
    {
	return m_entry;
    }

    //Verifies cached jar file version recorded in the table
    public boolean verifyCacheVersion(String version)throws JarFileInvalidException, InvalidCacheParameterException
    {
	if(!m_versionString.equalsIgnoreCase("null"))
	{
	    if(extractVersion(version) <= extractVersion(m_versionString))
		return true;
	}
	throw new JarFileInvalidException("Jarfile version doesn't match");
    }

    public Date getCacheTimeStamp()
    {
	return new Date(m_timeStamp);
    }

    public int getCacheFileSize()
    {
	return m_fileSize;
    }

    //Verifies cached jar file time stamp recorded in the table
    public boolean verifyCacheTimeStamp(Date timeStamp)throws JarFileInvalidException
    {
	if(m_timeStamp > 0 && timeStamp != null)
	{
	   Date cacheTimeStamp = getCacheTimeStamp();
	   
	   if (timeStamp.before(cacheTimeStamp) || timeStamp.equals(cacheTimeStamp))
		return true;
	}
	throw new JarFileInvalidException("File time stamp doesn't match");
    }

    //Verifies cached jar file file size recorded in the table
    public boolean verifyCacheFileSize(int fileSize)throws JarFileInvalidException
    {
	if(m_fileSize > 0 && fileSize > 0)
	{
	    if(fileSize == m_fileSize)
		return true;
	}
	throw new JarFileInvalidException("File size doesn't match");
    }

    //Return the file in cache, if no such file return null
    public File getFileInCache() throws IOException
    {
	File file = new File(PluginJarCacheTable.getCacheHomeDir() + File.separator + m_fileName);
	
	return file;
    }


    //Decodes the version which is available in the form of string
    public static long extractVersion(String versionString)throws InvalidCacheParameterException
    {
	long ver = 0;
	// Parse the version string
        StringTokenizer st = new StringTokenizer(versionString, ".", false);

	if (st.countTokens() != 4)
	{
	    throw new InvalidCacheParameterException("Requires attribute 'cache_version' in X.X.X.X form, where X is 0 to F");
	}

	while(st.hasMoreTokens()) 
	{
	    String tok = st.nextToken().trim();
	  
	    ver = ver * 16;
	    ver = ver + Integer.parseInt(tok, 16);
	}

	return ver;
    }

    long m_timeStamp;
    int m_fileSize;
    String m_fileName;
    String m_versionString;
    String m_entry;
}
