/*
 * @(#)PluginMutex.java	1.3 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 
 *
 * @author Devananda J.
 */

package sun.plugin.cachescheme;

import sun.plugin.usability.Trace;
import java.util.HashMap;
import java.util.Enumeration;

/*
public class PluginMutex{
    int mutexHandle;
    String m_mutexName;

    public PluginMutex(String name)
    {	
	if(name != null)
	{
	    m_mutexName = name;
	    mutexHandle = nativeCreate(name);
	}	    
    }

    public boolean release()throws PluginMutexException
    {
	boolean status = false;
	String message = null;

	int result = nativeRelease(mutexHandle);
	switch(result)
	{
	    case SUCCESS:
		status = true;
	    case FAILED:
		message = "Native call to release mutex failed";
		break;
	    case INVALID_HANDLE:
		message = "Invalid mutex handle";
		break;    
	}

	if(!status)
	    throw new PluginMutexException(message);

	return status;
    }

    public boolean waitForLock()throws PluginMutexException
    {

	boolean status = false;
	String message = null;
	Trace.println("Check if any other browser has a lock on " + m_mutexName, TraceFilter.JAVA_CONSOLE_ONLY);

	int result = nativeWaitForLock(mutexHandle);
	switch(result)
	{
	    case SUCCESS:
		status = true;
		break;
	    case FAILED:
		message = "Native call to acquire mutex failed";
		break;
	    case INVALID_HANDLE:
		message = "Invalid mutex handle";
		break;    
	}

	if(!status)
	    throw new PluginMutexException(message);

	Trace.println("No other browser has any lock on " + m_mutexName, TraceFilter.JAVA_CONSOLE_ONLY);
	return status;
    }

    public void finalize()throws PluginMutexException
    {
	release();	
    }

    native int nativeCreate(String name);
    native int nativeRelease(int handle);
    native int nativeWaitForLock(int handle);

    final int SUCCESS = 1;
    final int FAILED  = 0;
    final int INVALID_HANDLE = -1;
}

*/

class SyncObject
{
    static HashMap objTable = new HashMap();

    static synchronized Object create(String name)
    {
	Object obj = objTable.get(name);
	if(obj == null)
	{
	    obj = new Object();
	    objTable.put(name, obj);
	}
	return obj;
    }
}

public class PluginMutex
{
    String m_name = null;
    Object m_obj;
    boolean cv = false;

    public PluginMutex(String name)
    {	
	m_name = name;
	m_obj = SyncObject.create(name);
	cv = true;
    }

    public boolean release()throws PluginMutexException
    {
	boolean status = false;

	if(m_obj != null)
	{
	    synchronized(m_obj)
	    {
		cv = true;
		m_obj.notify();
	    }
	    status = true;
	}

	return status;
    }

    public boolean waitForLock()throws PluginMutexException
    {
	boolean status = false;
	//Trace.println("Check if anybody has a lock on " + m_obj.toString(), TraceFilter.JAVA_CONSOLE_ONLY);

	if (m_obj != null)
	{
	    try
	    {
		synchronized(m_obj)
		{
		    while (cv == false)
		    {
			m_obj.wait(0);

			Thread.currentThread().yield();
		    }

		    cv = false;
		}
		status = true;
		//Trace.println("Nobody has lock on " + m_obj.toString(), TraceFilter.JAVA_CONSOLE_ONLY);
	    }
	    catch (Exception e)
	    {
		throw new PluginMutexException("Unable to wait for mutex \"" + m_name + "\"");
	    }
	}

	return status;
    }

    public void finalize()throws PluginMutexException
    {
	release();	
    }
}





