/*
 * @(#)PluginURLConnection.java	1.6 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 * Java Plug-in 
 *
 * @author Devananda J. 
 */

package sun.plugin.cachescheme;

import java.io.*;
import java.net.*;
import sun.plugin.protocol.JarDownloadFeedback;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;


public class PluginURLConnection{

    URL m_url=null;
    int noOfBytes = 8192;
    boolean isHeaderAvailable = false;
    long m_lastModified = 0;
    int m_fileSize = 0;
    URLConnection m_conn = null;

    public PluginURLConnection(URL urlIn, String fileName)
	throws IOException
    {
	m_url = new URL(urlIn, fileName);
    }

    public synchronized long getLastModified()throws IOException
    {
	// Fetch HTTP headers
	fetchHTTPHeaders();

	return m_lastModified;
    }

    public synchronized int getContentLength()throws IOException
    {
	// Fetch HTTP headers
	fetchHTTPHeaders();

	return m_fileSize;
    }
    

    /**
     * Fetch HTTP headers if necessary.
     */
    private void fetchHTTPHeaders() throws IOException
    {
	if (isHeaderAvailable == true)
	    return;

	Trace.println("Determining time-stamp by connecting to " + m_url, TraceFilter.JAVA_CONSOLE_ONLY);

	// Open new URL connection
	m_conn = m_url.openConnection();
	
	m_conn.setUseCaches(false);

	if (m_conn instanceof HttpURLConnection)
	{
	    // If it is HTTP type, obtain headers through HEAD
	    //
	    HttpURLConnection httpConn = (HttpURLConnection) m_conn;

	    httpConn.setRequestMethod("HEAD");
	    httpConn.connect();

	    // Obtain the filesize and timestamp
	    m_fileSize = httpConn.getContentLength();
	    m_lastModified = httpConn.getLastModified();

	    httpConn.disconnect();

	    //we cannot re-use the connection, hence set m_conn to null
	    m_conn = null;
	}
	else
	{
	    m_conn.connect();

	    // Obtain the filesize and timestamp
	    m_fileSize = m_conn.getContentLength();
	    m_lastModified = m_conn.getLastModified();
	}
	
	isHeaderAvailable = true;
    }

    /**
     * Download JAR file from URL.
     */
    public void downLoadFile(File file) throws IOException
    {
	BufferedInputStream bis = null; 
	BufferedOutputStream bos = null;

	JarDownloadFeedback feedback = null;

	try
	{
	    //fetch the headers, so that there is no inconsistency in the
	    //values while comparing the filesize and time stamp later.
	    fetchHTTPHeaders();

	    if(m_conn == null)
	    {
		m_conn = m_url.openConnection();
		m_conn.setUseCaches(false);
	    }

	    // Make the connection
	    m_conn.connect();

	    byte[] buf = new byte[noOfBytes];

	    // Input Stream
	    bis = new BufferedInputStream(m_conn.getInputStream());

	    // Output Stream
	    bos = new BufferedOutputStream(new FileOutputStream(file));

	    int totalLen = getContentLength();
	    int len = 0;
	    int dataLen = 0;
	    
	    // Create binding feedback dialog
	    feedback = new JarDownloadFeedback(m_url.toString());
	    feedback.onStartBinding(dataLen, totalLen);

	    // Read/Write until EOF
	    while((len = bis.read(buf, 0, buf.length)) != -1) 
	    {
		dataLen = dataLen + len;

    		bos.write(buf, 0, len);

		// Update progress bar
		feedback.onProgress(dataLen, totalLen);
	    }
	}
	finally
	{
	    // Stop binding feedback dialog
	    if (feedback != null)
		feedback.onStopBinding();

	    //Close the output stream first, so that the cache file can be 
	    //deleted even when there is exception
	    if(bos != null)
		bos.close();

	    if(bis != null)
	        bis.close();	    

	    m_conn = null;
	}
    }
}
