/*
 * @(#)JavaExtensionInstaller.java	1.9 01/04/21
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.extension;

import java.net.URL;
import java.io.File;
import java.io.IOException;
import sun.plugin.resources.ResourceHandler;
import sun.plugin.usability.Trace;
import sun.plugin.usability.DialogFactory;



/**
 * Class that represents the Java standard extension installer.
 *
 * @author Stanley Man-Kit Ho
 */
public class JavaExtensionInstaller implements ExtensionInstaller
{
    /**
     * Install extension to the destination directory by
     * launching the Java Extension Installer.
     *
     * @param url URL of where the extension was downloaded from.
     * @param jarFileName Jar file that might contains the extensions.
     * @param destDir Destination directory to install the extension.
     * @return true if extension is installed successfully.
     */
    public boolean install(String url, String jarFileName, String destDir)
		   throws IOException
    {
	// The jar file itself contains the actual native extension 
	// installer. We launch the Java installer through the
	// "java -jar" command.
	//
	// There is no easy way to determine if the installation has
	// completed, so we popup a modal dialog and assume the user 
	// will dismiss the dialog when the installation has 
	// completed.
	//

	Trace.extPrintln("Launching Java Installer for Optional Package Installation");
	
	String javaHome = System.getProperty("java.home");
	String command = javaHome + File.separator + "bin" + File.separator 
			 + "java -jar \"" + jarFileName + "\"";

	Trace.extPrintln("Launching Java Installer through \'" + command + "\'");

	// Create a new process to launch the Java installer
	Process p = Runtime.getRuntime().exec(command);

	// Popup a modal dialog and hope the user will dismiss 
	// the dialog when the installation has completed.
	//
	DialogFactory.showMessageDialog(ResourceHandler.getMessage("optpkg.wait_for_installer"),
					ResourceHandler.getMessage("optpkg.launch_installer"));

	return true;
    }
}
