/*
 * @(#)RawExtensionInstaller.java	1.5 01/02/09
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.extension;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.jar.JarFile;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.security.PrivilegedActionException;
import sun.plugin.usability.Trace;


/**
 * Class that represents the Java standard extension installer.
 *
 * @author Stanley Man-Kit Ho
 */
public class RawExtensionInstaller implements ExtensionInstaller
{
    /**
     * Install raw extension to the destination directory.
     *
     * @param url URL of where the extension was downloaded from.
     * @param jarFileName Jar file that might contains the extensions.
     * @param destDir Destination directory to install the extension.
     * @return true if extension is installed successfully.
     */
    public boolean install(String url, String jarFileName, String destDir)
		   throws IOException
    {
	// The jar file itself is the extension that we should copy
	// into the destination directory. To minimal overhand, the
	// file is "moved" to the extension directory.
	//

	Trace.extPrintln("Installing Raw Optional Package from " + url);

	// Determine source and destination file name for the extension

	File sourceFile = new File(jarFileName);
	File destFile = new File(destDir + File.separatorChar + sourceFile.getName());

	Trace.extPrintln("Copying Raw Optional Package from " + sourceFile + " to " + destFile);	    

	FileInputStream fis = new FileInputStream(sourceFile);
	BufferedInputStream bis = new BufferedInputStream(fis);
	FileOutputStream fos = new FileOutputStream(destFile);
	BufferedOutputStream bos = new BufferedOutputStream(fos);

	ExtensionUtils.copy(bis, bos);

	bis.close();
	fis.close();
	bos.close();
	fos.close();

	return true;
    }
}
