/*
 * @(#)Document.java	1.11 98/05/18
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1_beta
 *
 * Java Plug-in 1.2
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.javascript.navig;

import java.util.Hashtable;
import netscape.javascript.JSObject;
import netscape.javascript.JSException;



/** 
 * <p> Emulate the Document object in the JavaScript Document Object Model
 * in Navigator 3.x.
 * </p>
 */
public class Document extends sun.plugin.javascript.navig.JSObject {


   /**
    * <p> Method table contains all method info in the Document object. </p>
    */
   private static Hashtable methodTable = new Hashtable();

   /**
    * <p> Field table contains all properties info in the Document object. </p>
    */
   private static Hashtable fieldTable = new Hashtable();

   static {

	// Initialize all method and field info in the Document object.
	//
	methodTable.put("clear",   new Boolean(false));
	methodTable.put("close",   new Boolean(false));
	methodTable.put("open",	   new Boolean(false));
	methodTable.put("write",   new Boolean(false));
	methodTable.put("writeln", new Boolean(false));

	fieldTable.put("alinkColor",	new Boolean(true));
	fieldTable.put("anchors",	new Boolean(false));
	fieldTable.put("applets",	new Boolean(false));
	fieldTable.put("bgColor",	new Boolean(true));
	fieldTable.put("cookie",	new Boolean(true));
	fieldTable.put("domain",	new Boolean(true));
	fieldTable.put("embeds",	new Boolean(false));
	fieldTable.put("fgColor",	new Boolean(true));
	fieldTable.put("forms",		new Boolean(false));
	fieldTable.put("images",	new Boolean(false));
	fieldTable.put("lastModified",	new Boolean(false));
	fieldTable.put("linkColor",	new Boolean(true));
	fieldTable.put("links",		new Boolean(false));
	fieldTable.put("location",	new Boolean(true));
	fieldTable.put("plugins",	new Boolean(false));
	fieldTable.put("referrer",	new Boolean(false));
	fieldTable.put("title",		new Boolean(false));
	fieldTable.put("URL",		new Boolean(false));
	fieldTable.put("vlinkColor",	new Boolean(true));
    }


    /**
     * <p> Construct a new Document object. 
     * </p>
     * 
     * @param instance Native plugin instance.
     * @param context Evaluation context.
     */
    protected Document(int instance, String context)  {
	super(instance, context);

	// Setup object property and method table.
	//
	addObjectTable(fieldTable, methodTable);
    }

 
    /**
     * <p> Retrieves a named member of the Document object. Equivalent to 
     * "self.document.name" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @return The value of the propery.
     */
    public Object getMember(String name) throws JSException {

	if (name.equals("links"))  {
	    return resolveObject(JSType.LinkArray, context + ".links");
	}
	else if (name.equals("anchors")) {
	    return resolveObject(JSType.AnchorArray, context + ".anchors");
	}
	else if (name.equals("forms")) {
	    return resolveObject(JSType.FormArray, context + ".forms");
	}
	else if (name.equals("images")) {
	    return resolveObject(JSType.ImageArray, context + ".images");
	}
	else if (name.equals("location")) {
	    return resolveObject(JSType.Location, context + ".location");
	}

	// Try to match the name of the property with the field table.
	//
        return super.getMember(name);
    }
}
