/*
 * @(#)Element.java	1.11 98/05/20
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1_beta
 *
 * Java Plug-in 1.2
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.javascript.navig;

import java.util.Hashtable;
import netscape.javascript.JSObject;
import netscape.javascript.JSException;


/** 
 * <p> Emulate the Element object in the JavaScript Document Object Model.
 * </p>
 */
class Element extends sun.plugin.javascript.navig.JSObject {

    /**
     * <p> Method table contains all method info in the Element object. </p>
     */
    private static Hashtable methodTable = new Hashtable();

    /**
     * <p> Field table contains all properties info in the Element object. </p>
     */
    private static Hashtable fieldTable = new Hashtable();

    static {

	// Initialize all method and field info in the Element object.
	//
	methodTable.put("blur",    new Boolean(false));
	methodTable.put("click",   new Boolean(false));
	methodTable.put("focus",   new Boolean(false));
	methodTable.put("select",  new Boolean(false));

	fieldTable.put("checked",	 new Boolean(true));
	fieldTable.put("defaultChecked", new Boolean(false));
	fieldTable.put("defaultValue",   new Boolean(false));
	fieldTable.put("forms",		 new Boolean(false));
	fieldTable.put("length",	 new Boolean(false));
	fieldTable.put("name",		 new Boolean(false));
	fieldTable.put("options",	 new Boolean(false));
	fieldTable.put("selectedIndex",	 new Boolean(false));
	fieldTable.put("type",		 new Boolean(false));
	fieldTable.put("value",		 new Boolean(true));
    }

    /**
     * <p> Form object that the element is contained in. </p>
     */
    private Form form;


    /**
     * <p> Construct a new Element object. 
     * </p>
     * 
     * @param instance Native plugin instance.
     * @param context Evaluation context.
     * @param form Form object that the element is contained in.
     */
    Element(int instance, String context, Form form) {
	super(instance, context);
	this.form = form;

	// Setup object property and method table.
	//
	addObjectTable(fieldTable, methodTable);
    }


    /**
     * <p> Retrieves a named member of the Element object. Equivalent to 
     * "self.document.forms[].elements[].name" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @return The value of the propery.
     */
    public Object getMember(String name) throws JSException {

	// Deal with special properties first.
	//
	if (name.equals("form"))
	    return form;
	else if (name.equals("options"))  {
	    return resolveObject(JSType.OptionArray, context + ".options");
	}

	// Try to match the name of the property with the field table.
	//
        return super.getMember(name);
    }
}
