/*
 * @(#)Form.java	1.11 98/05/20
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1_beta
 *
 * Java Plug-in 1.2
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.javascript.navig;

import java.util.Hashtable;
import netscape.javascript.JSObject;
import netscape.javascript.JSException;



/** 
 * <p> Emulate the Form object in the JavaScript Document Object Model.
 * </p>
 */
class Form extends sun.plugin.javascript.navig.JSObject {

    /**
     * <p> Method table contains all method info in the Element object. </p>
     */
    private static Hashtable methodTable = new Hashtable();

    /**
     * <p> Field table contains all properties info in the Form object. </p>
     */
    private static Hashtable fieldTable = new Hashtable();

    static {

	// Initialize all field info in the Window object.
	//
	methodTable.put("reset",    new Boolean(false));
	methodTable.put("submit",   new Boolean(false));

	fieldTable.put("action",   new Boolean(true));
	fieldTable.put("elements", new Boolean(false));
	fieldTable.put("encoding", new Boolean(true));
	fieldTable.put("method",   new Boolean(true));
	fieldTable.put("target",   new Boolean(true));
    }

    /**
     * <p> Construct a new Form object. 
     * </p>
     * 
     * @param instance Native plugin instance.
     * @param context Evaluation context.
     */
    Form(int instance, String context)  {
	super(instance, context);

	// Setup object property and method table.
	//
	addObjectTable(fieldTable, methodTable);
    }


    /**
     * <p> Retrieves a named member of the Form object. Equivalent to 
     * "self.document.forms[].name" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @return The value of the propery.
     */
    public Object getMember(String name) throws JSException {

	// Deal with special properties first.
	//
	
	if (name.equals("elements"))  {
	    return resolveObject(JSType.ElementArray, context + ".elements", this);
	}

	// Try to match the name of the property with the field table.
	//
        return super.getMember(name);
    }
}
