/*
 * @(#)Layer.java	1.11 98/05/19
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1_beta
 *
 * Java Plug-in 1.2
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.javascript.navig4;

import java.util.Hashtable;
import netscape.javascript.JSObject;
import netscape.javascript.JSException;
import sun.plugin.javascript.navig.JSType;



/** 
 * <p> Emulate the Layer object in the JavaScript Document Object Model
 * in Navigator 4.
 * </p>
 */
public class Layer extends sun.plugin.javascript.navig.JSObject {

    /**
     * <p> Field table contains all properties info in the Layer object. </p>
     */
    private static Hashtable fieldTable = new Hashtable();

    /**
     * <p> Method table contains all methods info in the Layer object. </p>
     */
    private static Hashtable methodTable = new Hashtable();

    static {

	// Initialize all method and field info in the Layer object.
	//
	fieldTable.put("document",     new Boolean(false));
	fieldTable.put("name",	       new Boolean(false));
	fieldTable.put("left",	       new Boolean(true));
	fieldTable.put("top",	       new Boolean(true));
	fieldTable.put("pageX",	       new Boolean(true));
	fieldTable.put("pageY",        new Boolean(true));
	fieldTable.put("zIndex",       new Boolean(true));
	fieldTable.put("visibility",   new Boolean(true));
	fieldTable.put("clip.top",     new Boolean(true));
	fieldTable.put("clip.left",    new Boolean(true));
	fieldTable.put("clip.right",   new Boolean(true));
	fieldTable.put("clip.bottom",  new Boolean(true));
	fieldTable.put("clip.width",   new Boolean(true));
	fieldTable.put("clip.height",  new Boolean(true));
	fieldTable.put("clip.top",     new Boolean(true));
	fieldTable.put("background",   new Boolean(true));
	fieldTable.put("bgColor",      new Boolean(true));
	fieldTable.put("siblingAbove", new Boolean(false));
	fieldTable.put("siblingBelow", new Boolean(false));
	fieldTable.put("above",        new Boolean(false));
	fieldTable.put("below",	       new Boolean(false));
	fieldTable.put("parentLayer",  new Boolean(false));
	fieldTable.put("src",	       new Boolean(true));

	methodTable.put("moveBy",	  new Boolean(false));
	methodTable.put("moveTo",	  new Boolean(false));
	methodTable.put("moveToAbsolute", new Boolean(false));
	methodTable.put("resizeBy",	  new Boolean(false));
	methodTable.put("resizeTo",	  new Boolean(false));
	methodTable.put("moveAbove",	  new Boolean(false));
	methodTable.put("moveBelow",	  new Boolean(false));
	methodTable.put("load",		  new Boolean(false));
    }

    
    /**
     * <p> Construct a new Layer object. 
     * </p>
     * 
     * @param instance Native plugin instance.
     * @param context Evaluation context
     */
    protected Layer(int instance, String context) {
	super(instance, context);

	// Setup object property and method table.
	//
	addObjectTable(fieldTable, methodTable);
    }


    /**
     * <p> Retrieves a named member of the Layer object. Equivalent to 
     * "self.document.layers[].name" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @return The value of the propery.
     */
    public Object getMember(String name) throws JSException {

	if (name.equals("document"))  {
	    Object val = evaluate(context + ".document");
	    if (val == null)
		return null;
	    else
		return resolveObject(JSType.Document, context + ".document");
	}
	else if (name.equals("siblingAbove"))  {
	    Object val = evaluate(context + ".siblingAbove");
	    if (val == null)
		return null;
	    else
		return resolveObject(JSType.Layer, context + ".siblingAbove");
	}
	else if (name.equals("siblingBelow"))	{
	    Object val = evaluate(context + ".siblingBelow");
	    if (val == null)
		return null;
	    else
		return resolveObject(JSType.Layer, context + ".siblingBelow");
	}
	else if (name.equals("above"))  {
	    Object val = evaluate(context + ".above");
	    if (val == null)
		return null;
	    else
    		return resolveObject(JSType.Layer, context + ".above");
	}
	else if (name.equals("below"))	{
	    Object val = evaluate(context + ".below");
	    if (val == null)
		return null;
	    else
		return resolveObject(JSType.Layer, context + ".below");
	}
	else if (name.equals("parentLayer"))  {
	    Object val = evaluate(context + ".parentLayer");
	    if (val == null)
		return null;
	    else
		return resolveObject(JSType.Layer, context + ".parentLayer");
	}

        return super.getMember(name);
    }
}
