/*
 * @(#)Window.java	1.11 98/05/19
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1_beta
 *
 * Java Plug-in 1.2
 *
 * @author Stanley Man-Kit Ho
 */


package sun.plugin.javascript.navig4;

import java.util.Hashtable;
import netscape.javascript.JSObject;
import netscape.javascript.JSException;
import sun.plugin.javascript.navig4.JSObjectFactory;
import sun.plugin.javascript.navig.Array;
import sun.plugin.javascript.navig.JSType;


/** 
 * <p> Emulate the Window object in the JavaScript Document Object Model
 * in Navigator 4.x.
 * </p>
 */
public class Window extends sun.plugin.javascript.navig.Window {

   /**
    * <p> Method table contains all method info in the Window object. </p>
    */
   private static Hashtable methodTable = new Hashtable();

   /**
    * <p> Field table contains all properties info in the Window object. </p>
    */
   private static Hashtable fieldTable = new Hashtable();

   static {

	// Initialize all method and field info in the Window object.
	//
	methodTable.put("back",			  new Boolean(false));
	methodTable.put("clearInterval",	  new Boolean(false));
	methodTable.put("disableExternalCapture", new Boolean(false));
	methodTable.put("enableExternalCapture",  new Boolean(false));
	methodTable.put("find",			  new Boolean(true));
	methodTable.put("forward",		  new Boolean(false));
	methodTable.put("home",			  new Boolean(false));
	methodTable.put("moveBy",		  new Boolean(false));
	methodTable.put("moveTo",	          new Boolean(false));
	methodTable.put("print",		  new Boolean(false));
	methodTable.put("resizeBy",		  new Boolean(false));
	methodTable.put("resizeTo",		  new Boolean(false));
	methodTable.put("scrollBy",		  new Boolean(false));
	methodTable.put("scrollTo",		  new Boolean(false));
	methodTable.put("setInterval",		  new Boolean(false));
	methodTable.put("stop",			  new Boolean(false));

	fieldTable.put("innerHeight",	new Boolean(true));
	fieldTable.put("innerWidth",	new Boolean(true));
	fieldTable.put("height",	new Boolean(true));
	fieldTable.put("width",		new Boolean(true));
	fieldTable.put("locationbar",	new Boolean(false));
	fieldTable.put("menubar",	new Boolean(false));
	fieldTable.put("outerHeight",	new Boolean(true));
	fieldTable.put("outerWidth",	new Boolean(true));
	fieldTable.put("pageXOffset",	new Boolean(true));
	fieldTable.put("pageYOffset",	new Boolean(true));
	fieldTable.put("personalbar",	new Boolean(false));
	fieldTable.put("scrollbars",	new Boolean(false));
	fieldTable.put("statusbar",	new Boolean(false));
	fieldTable.put("toolbar",	new Boolean(false));

    }


    /**
     * <p> Test if the JSObject factory has been set.
     * </p>
     */
    private static boolean factorySet = false;


    /**
     * <p> Construct a new Window object. 
     * </p>
     * 
     * @param instance Native plugin instance.
     * @param context Evaluation context.
     */
    public Window(int instance, String context)  {
	super(instance, context);

	// Set JSObject resolver.
	try  {
	    if (factorySet == false)   {
		setResolver(new JSObjectFactory());
		factorySet = true;
	    }
	} catch (JSException e)  {
	}

	// Setup object property and method table.
	//
	addObjectTable(fieldTable, methodTable);
    }


    /**
     * <p> Construct a new Window object. 
     * </p>
     * 
     * @param instance Native plugin instance.
     */
    public Window(int instance) {
	this(instance, "self");
    }


    /**
     * <p> Retrieves a named member of the Window object. Equivalent to 
     * "self.name" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @return The value of the propery.
     */
     public Object getMember(String name) throws JSException {

	// Deal with special properties first.
	//
	
	if (name.equals("locationbar"))
	    return resolveObject(JSType.UIBar, context + ".locationbar");
	else if (name.equals("menubar"))
	    return resolveObject(JSType.UIBar, context + ".menubar");
	else if (name.equals("personalbar"))
	    return resolveObject(JSType.UIBar, context + ".personalbar");
	else if (name.equals("scrollbars"))
	    return resolveObject(JSType.UIBar, context + ".scrollbars");
	else if (name.equals("statusbar"))
	    return resolveObject(JSType.UIBar, context + ".statusbar");
	else if (name.equals("toolbar"))
	    return resolveObject(JSType.UIBar, context + ".toolbar");


	// Try to match the name of the property with the field table.
	//
        return super.getMember(name);
    }
}
