/*
 * @(#)OleBeanInfo.java	1.2 98/06/02
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.0
 *
 */

package sun.plugin.javascript.ocx;

import netscape.javascript.*;
import sun.plugin.*;
import sun.beans.ole.OleDispatchInterface;

/*
 * JSObject
 *
 * This class is responsible for providing an implementation of the 
 * JSObject inside Internet Explorer browsers family. 
 *
 * @author Jerome Dochez
 * @version 1.2
 */ 

public class JSObject extends  sun.plugin.javascript.JSObject {

    static final int DISPATCH_METHOD = 0x1;
    static final int DISPATCH_PROPERTYGET = 0x2;
    static final int DISPATCH_PROPERTYPUT = 0x3;

    /**
     * <p>
     * Create a new Instance of the JSObject interface.
     * JSObject can only be created when acting as a proxy to an 
     * IDispatch implementation
     * </p>
     *
     * @param int native implementation instance of the JSObject
     */
    public JSObject(int pointer) {
	this.pointer = pointer;
    }

    /*
     * <p>
     * Set the applet context for this JSObject, this is used 
     * when this JSObject creates new JSObject as method invocation returns
     * </p>
     *
     * @param aac Applet Context which identify which applet "owns" this JSObject
     */
    private void setActiveXAppletContext(sun.plugin.ocx.ActiveXAppletContext aac, int hWnd) {
	this.aac = aac;
	handle = hWnd;
	aac.addJSObjectToExportedList(this);
    }

    /**
     * <p>
     * finalization
     * </p>
     */
    public void finalize() {
	cleanup();
    }

    /**
     * <p>
     * Release the native implementation object
     * </p>
     */
    public void cleanup() {
	if(!released)
	{
	    nativeRelease(pointer);
	    released = true;
	}
    }

    /**
     * <p> 
     * Calls a JavaScript method. Equivalent to 
     * "this.methodName(args[0], args[1], ...)" in JavaScript.
     * </p>
     *
     * @param methodName The name of the JavaScript method to be invoked.
     * @param args An array of Java object to be passed as arguments to the method.
     * @return Result of the method.
     */
    public synchronized Object invoke(String methodName, Object args[], int flag) throws JSException {
	// Get the ID of the method
	int id = getIDsOfNames(methodName, handle);
	if (id==0xFFFFFFFF) 
	   throw new JSException(AppletViewer.getMessage("jsobject_method") + 
				  methodName + AppletViewer.getMessage("not_found"));
	
	invoke(id, flag, args, handle);

	if (result instanceof sun.plugin.javascript.ocx.JSObject)
	   ((sun.plugin.javascript.ocx.JSObject) result).setActiveXAppletContext(aac, handle);

	return result;
    }

    /**
     * <p> Calls a JavaScript method. Equivalent to 
     * "this.methodName(args[0], args[1], ...)" in JavaScript.
     * </p>
     *
     * @param methodName The name of the JavaScript method to be invoked.
     * @param args An array of Java object to be passed as arguments to the method.
     * @return Result of the method.
     */
    public Object call(String methodName, Object args[]) throws JSException {
	return invoke(methodName, args, DISPATCH_METHOD);
    }

    private int packageParms(Object args[]) {
	int parmsNb = 0;
	if (args!=null)
	    parmsNb = args.length;
	int disparams = service.allocateParmsFor(parmsNb);
	for (int i=0;i<parmsNb; i++) 
	    service.storeParm(disparams, parmsNb-i-1, args[i], service.READ_ONLY);
	return disparams;
    }

    private void freeParms(int disparams) {
	service.freeParms(disparams);
    }

    /** 
     * <p> Evaluates a JavaScript expression. The expression is a string of 
     * JavaScript source code which will be evaluated in the context given by 
     * "this".
     * </p>
     *
     * @param s The JavaScript expression.
     * @return Result of the JavaScript evaluation.
     */
    public Object eval(String s) throws JSException 
    {
	Object args[] = new Object[1];

	// Escape all the '\'' and '\\'s in the string s
	StringBuffer buffer = new StringBuffer();

	for (int i=0; i < s.length(); i++)
	{
	    char c = s.charAt(i);

	    if (c == '\'' || c == '\"' || c == '\\')
		buffer.append('\\');

	    buffer.append(c);
	}


	// Fixed #4324382   
	args[0]="evalIntermediateValueToReturn=0;evalIntermediateValueToReturn=eval('" + buffer.toString() + "');";

	netscape.javascript.JSObject jsObject = aac.getJSObject();

	try
	{
	    jsObject.call("execScript", args);
	}
	catch (JSException ex0)
	{
	    throw new JSException("Failure to evaluate " + s);
	}

	try
	{
	    // Exception may be thrown if there is no return value.
	    return jsObject.getMember("evalIntermediateValueToReturn");    
	}
	catch (JSException ex1)
	{
	    return null;
	}
    }

    /**
     * <p> Retrieves a named member of a JavaScript object. Equivalent to 
     * "this.name" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @return The value of the propery.
     */
    public Object getMember(String name) throws JSException {
	return invoke(name, null, DISPATCH_PROPERTYGET);
    }

    /** 
     * <p> Sets a named member of a JavaScript object. Equivalent to 
     * "this.name = value" in JavaScript.
     * </p>
     *
     * @param name The name of the JavaScript property to be accessed.
     * @param value The value of the propery.
     */
    public void setMember(String name, Object value) throws JSException {
	Object args[] = { value };
	invoke(name, args, DISPATCH_PROPERTYPUT);
    }

    /**
     * <p> Removes a named member of a JavaScript object.
     * </p>
     *
     * @param name The name of the JavaScript property to be removed.
     */
    public void removeMember(String name) throws JSException {
	throw new JSException("removeMember does not support " + toString() + "." + name);
    }

    /**
     * <p> Retrieves an indexed member of a JavaScript object. Equivalent to 
     * "this[index]" in JavaScript.
     * </p>
     *
     * @param int The index of the array to be accessed.
     * @return The value of the indexed member.
     */
    public Object getSlot(int index) throws JSException {
	int id = getIDsOfNames("item", handle);
	if (id==0xFFFFFFFF)
	    throw new JSException(AppletViewer.getMessage("jsobject_getslot"));

	Object args[] = new Object[1];
	args[0] = new Integer(index);
	
	invoke(id, DISPATCH_METHOD, args, handle);
	return result;
    }

    /**
     * <p> Sets an indexed member of a JavaScript object. Equivalent to 
     * "this[index] = value" in JavaScript.
     * </p>
     *
     * @param int The index of the array to be accessed.
     */
    public void setSlot(int index, Object value) throws JSException {

	int id = getIDsOfNames("add", handle);
	if (id==0xFFFFFFFF)
	    throw new JSException(AppletViewer.getMessage("jsobject_setslot"));

	Object args[] = new Object[2];
	args[0] = new Integer(index);
	args[1] = value;

        invoke(id, DISPATCH_METHOD, args, handle);
    }


    private native void nativeRelease(int pointer);
    private native int getIDsOfNames(String methodName,int handle);
    private native Object invoke(int id, int flag, Object[] args, int handle);

    private int pointer;
    private OleDispatchInterface service = new OleDispatchInterface();
    private sun.plugin.ocx.ActiveXAppletContext aac;
    private Object result;
    private static Thread jsThread;
    private boolean released = false;

    private int handle = 0;
}

