/*
 * @(#)JavaScriptProtectionDomain.java	1.8 01/02/11
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.liveconnect;

import java.security.Permission;
import java.security.PermissionCollection;
import java.security.ProtectionDomain;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;

/** 
 *
 * <p> This JavaScriptProtectionDomain class encapulates the characteristics of
 * a virtual JavaScript domain, which represents the set of permissions when
 * JavaScript calls Java through LiveConnect.
 * </p>
 * 
 * @author Stanley Man-Kit Ho
 */

public class JavaScriptProtectionDomain extends ProtectionDomain
{
    
    // Permission collection
    PermissionCollection perms = null;

    /**
     * <P> Creates a new JavaScriptProtectionDomain with a given 
     * permission collections.
     * </P>
     *
     * @param securityContext the nsISecurityContext
     */
    public JavaScriptProtectionDomain(PermissionCollection perms) 
    {
	super(null, null);
	this.perms = perms;
    }

    /**
     * <P> Check and see if this JavaScriptProtectionDomain implies the 
     * permissions expressed in the Permission object.
     * </P>
     *
     * @param permission the Permission object to check.
     * @return true if "permission" is a proper subset of a permission in 
     *		    this ProtectionDomain, false if not.
     */
    public boolean implies(Permission permission) 
    {
//	Trace.liveConnectPrintln("Check for " + permission + " in JavaScriptProtectionDomain: " + perms,
//				 TraceFilter.JAVA_CONSOLE_ONLY);

	return perms.implies(permission);
    }


    /**
     * <P> Convert a JavaScriptProtectionDomain to a String.
     * </P>
     */
    public String toString() {
	return "JavaScriptProtectionDomain "+getCodeSource()+"\n"+getPermissions()+"\n";
    }
}
