/*
 * @(#)SecureInvocation.java	1.00 98/09/22
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.liveconnect;

import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.AccessControlException;
import java.security.ProtectionDomain;
import java.security.PrivilegedExceptionAction;
import java.security.PrivilegedActionException;
import java.security.AllPermission;

/**
 * <P> SecurityContextHelper is for evaluating whether an AccessControlContext
 * implies a particular LiveConnect permission.
 * </P>
 *
 * @version 	1.1 
 * @author	Stanley Man-Kit Ho
 */


public class SecurityContextHelper {

    /**
     * <P> Check if permission can be implied with security context.
     * </P>
     *
     * @param context Access Control Context
     * @param target Security target
     * @param action Security action
     * @return true if the operation is successful
     */
    public static boolean Implies(AccessControlContext context, String target, String action)
    {
	if (context == null || target == null)
	    return false;

	try 
	{
	    if (target.equals(LiveConnect.AllJavaPermission))
		context.checkPermission(new AllPermission());
/*	    else if (target.equals(LiveConnect.AllJavaScriptPermission))
		context.checkPermission(new JavaScriptPermission("Binary JavaScript Permission"));
	    else 
		return false;
*/	    else 
		context.checkPermission(new JavaScriptPermission(target));
	}
	catch (AccessControlException e)
	{
	    return false;
	}

	return true;
    }
}
