/*
 * @(#)URLDecoder.java	1.11 97/06/13
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1_beta
 *
 */

package sun.plugin.navig;

import java.io.ByteArrayOutputStream;

/**
 * The class contains a utility method for converting a
 * <code>String</code> from a MIME format called
 * "<code>x-www-form-urlencoded</code>" format.
 * <p>
 * To convert a <code>String</code>, each character is examined in turn:
 * <ul>
 * <li>The ASCII characters '<code>a</code>' through '<code>z</code>',
 *     '<code>A</code>' through '<code>Z</code>', and '<code>0</code>'
 *     through '<code>9</code>' remain the same.
 * <li>The space character '<code>&nbsp;</code>' is converted into a
 *     plus sign '<code>+</code>'.
 * <li>All other characters are converted into the 3-character string
 *     "<code>%<i>xy</i></code>", where <i>xy</i> is the two-digit
 *     hexadecimal representation of the lower 8-bits of the character.
 * </ul>
 *
 * @author  Stanley Man-Kit Ho
 * @version 1.11, 01/15/98
 * @since   JDK1.0
 */
public class URLDecoder {
    /**
     * You can't call the constructor.
     */
    private URLDecoder() { }

    /**
     * Translates a string from <code>x-www-form-urlencoded</code> format.
     *
     * @param   s   the translated <code>String</code>.
     * @return  the string translated to <code>String</code>.
     * @since   JDK1.0
     */
    public static String decode(String s) {
	int maxBytesPerChar = 10;
	ByteArrayOutputStream out = new ByteArrayOutputStream(s.length());

	for (int i = 0; i < s.length(); i++) {
	    int c = (int)s.charAt(i);
	    if (c == '%')
            {
		// convert to external decoding
         	c = (Character.digit(s.charAt(i+1), 0xF) << 4) +
		     Character.digit(s.charAt(i+2), 0xF);
                i = i + 2;
            }

            out.write(c);
        }

	return out.toString();
    }
}
