/*
 * @(#)MotifAppletViewer.java	1.44 01/02/09
 * 
 *  Copyright 1998-2001 Sun Microsystems, Inc. 901 San Antonio Road, 
 * Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * CopyrightVersion 1.2
 * 
 * @author Graham Hamilton, Robert Szcewczyk, Benedict Gomes, Stanley Ho
 */


package sun.plugin.navig.motif;

import java.awt.*;
import java.awt.peer.ComponentPeer;
import sun.awt.motif.MEmbeddedFrame;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.applet.*;
import java.io.PrintStream;
import java.io.StringWriter;
import java.io.PrintWriter;
import sun.applet.*;
import java.net.URL;
import java.util.Hashtable;
import java.lang.reflect.*;


/*
 *
 * Panel for each applet running in the Java Plugin VM.  This class
 * encapsulates the viewing of the Applet. It extends AppletViewer,
 * which extends AppletPanel, which provides the basic state machine
 * of an applet.  There are 5 threads associated with this class, in
 * addition to any associated with the applet:
 *
 *   The event processing thread (of type MotifAppletViewer)
 *   The class loading thread (also of type MotifAppletViewer,
 *         responsible for creating the class loader etc.)
 *   The init thread (Initer thread) , used for calling init on the applet
 *   The destroy thread (DestroyerThread), used for calling destroy
 *   The Plugin's main thread which runs the applet viewer by calling
 *                setWindow etc.
 *
 * Most of the hard work is done in our baseclasses, sun.plugin.AppletViewer
 * and sun.plugin.AppletPanel.
 *
 * The life-cycle for a MotifAppletpanel is slightly strange and is
 * driven by the Netscape plugin API.
 *
 *    (1) First, netscape creates a new plugin instance using CreateApplet

 *    (2) Typically, the next thing that happens is that Netscape
 *	  sets the X11 window information using JAVA_PLUGIN_WINDOW
 *	  At this point we can create our AWT EmbeddedFrame using
 *	  the parent X11 window ID privided by netscape.
 *
 *    (3) Typically, the next thing that happens will be that the
 *	  plugin figures out our DOCBASE and sends it to us.
 *	  This step may sometimes ocur before (2). JAVA_PLUGIN_DOCBASE
 *	  This allows us to set the DOCBASE in our parent.
 *
 *    (4) When both (2) and (3) have occurred we have both a DOCBASE
 *	  value and an EmbeddedFrame, so we can call init() to 
 *	  actually do all the hard work of starting the applet.
 *
 *    (5) We live a fruitful and industrious life, with our applet
 *	  happily running and painting.
 *
 *    (6) When netscape moves to a different page (or whatever) the
 *	  plugin JAVA_PLUGIN_DESTROY method will be called.
 *
 *	  At this point we are called to destroy the applet.
 * */

public class MotifAppletViewer 
    extends sun.plugin.AppletViewer 
    implements WindowListener
{

    // To satisfy the WindowEventListener interface. Required for 
    // accessibility to work. See 
    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}

    // For debugging purposes - to check the name of the viewer before
    // actually entering it.
    int getCurWinID() {
	return winid;
    }

    MotifAppletViewer(int pluginIndex) {
	super (new MotifAppletContext(pluginIndex));
	tracing = Plugin.getTracing();
    }

    /**
     * Used by our MotifBeanViewer subclass.
     */
    protected MotifAppletViewer(sun.plugin.ActivatorAppletContext 
				appletContext) {
	super(appletContext);
    }

  
    /* 
     * Over-ride the run method to put the maf_run method on the
     * stack to clearly identify this thread from its stack trace
     */
    public void run() {
	maf_run();
    }

    public void maf_run() {
	super.run();
    }

    /* Print the applet to the output stream, 'printOut'. Note - this
     * only works in Nav 4.5, and print is called 3 times. We only
     * use the last call. 
     */
    public void doPrint(int x, int y, int w, int h, PrintStream printOut) {
	Applet applet = getApplet();
	Dimension size = getSize();
	if (printCounter == 2) {
	    PluginPSPrintStream ppsps =
		new PluginPSPrintStream(null, printOut,
					applet.getClass().toString(), 
					size, false);
	    ppsps.sendBeginEPSF();
	    float scale1 =
		(float)(Math.rint(width*10.0/size.width)/10.0);
	    float scale2 =
		(float)(Math.rint(height*10.0/size.height)/10.0);
	    ppsps.sendPosition(x, 720-y%648, width, height,
                               scale1, scale2);
	    ppsps.sendBeginDoc();
	    ppsps.sendProlog();
	    ppsps.startPage(0);
	    Graphics ppsg = new PluginPSGraphics(ppsps, 0);
	    ppsg.clipRect(0, 0, size.width, size.height);

	    // Ignore printing exceptions.

	    try {
		getApplet().printAll(ppsg);
	    }
	    catch(Exception e) {
		StringWriter sw = new StringWriter();
		e.printStackTrace(new PrintWriter(sw));
		Plugin.trace("Printing exception!");
		Plugin.trace(sw.toString());
	    }

	    ppsg.dispose();
	    ppsps.endPage();
	    ppsps.close(0);
	    ppsps.sendEndDoc();
	    ppsps.sendEndEPSF();
	    printOut.flush();
	} 
	printCounter++;
	if (printCounter == 3)
	    printCounter = 0;
    }

     /**
     * Nested class to call 'init' on our applet. 
     */
    private class Initer extends Thread {
	MotifAppletViewer that;

	Initer(MotifAppletViewer that) {
	    super(Thread.currentThread().getThreadGroup(),"IniterThread");
	    this.that = that;
  	}

	/**
         * Main body of thread.  Call init on our AppletViewer.
	 */
	public void run() {
	    init_run_wrapper();	// To identify this thread's stack
	}
	
	private void init_run_wrapper() {
	    trace("Bounds before init:" + that.frame.getBounds());
	    that.init();
	    that.preInitStartCounter.eventTrigger();
	    int count = that.preInitStartCounter.getCounter();

	    Plugin.trace("Start counter is :" + count);

	    if (count == -1) {
		/* No start as yet. Do nothing */
		trace("Not starting after init");
	    } else if (count == 0) {
		trace("Do a start/stop");
		that.sendEvent(sun.applet.AppletPanel.APPLET_START);
		that.sendEvent(sun.applet.AppletPanel.APPLET_STOP);
	    } else if (count == 1) {
		trace("Do only a start");
		that.sendEvent(sun.applet.AppletPanel.APPLET_START);
	    } else {
		trace("Some problem? Start should not be > 1");
		// Make the best of it?
		that.sendEvent(sun.applet.AppletPanel.APPLET_START);
	    }
	}
    }
    /**
     * If we have both a docbase and a frame, and we have not
     * yet init'ed our parent, do it. 
     */
    private void maybeInit() {
	trace("Init ?");
	if (!inited && frame != null && getDocumentBase() != null) {
	    inited = true;
	    trace("Yes, init");
	    // We need to call init iun a differnet thread, as if we
	    // call it in the main Plugin worked thread we may deadlock.
	    if (javaEnabled) {
		initer = new Initer(this);
		initer.start();
	    }
	}
    }


    /**
     * Called when the plugin eventually figures out what the
     * document base URL is for the page we're in.
     */
    public void setDocumentBase(URL url) {
	super.setDocumentBase(url);
	initProperties();
	if (tracing){ 
	    trace(" setting doc base " + url.toString());
	    trace("docbaseURL = " + getDocumentBase());
	    trace("codebaseURL = " + getCodeBase());
	}

	// If the window has already been set, we can init the applet.
	maybeInit();
    }

    /**
     * Create our EmbeddedFrame.  
     * Embedded frame creation changed between 1.2 beta 2 and 1.2 beta3.  
     * to take a long instead of an int as the handle.
     */
    private Frame createFrame(int widget) {
 	// Class clz = sun.awt.motif.MEmbeddedFrame.class;
	Class clz = sun.plugin.navig.motif.MPluginEmbeddedFrame.class;
	// I.1 takes an integer argument to MEmbeddedFrame(int handle)
 	try {
 	    Class params[] = { int.class };
 	    Constructor cons = clz.getConstructor(params);
 	    Object args[] = { new Integer(widget) };
 	    return (Frame) cons.newInstance(args);
 	} catch (NoSuchMethodException ex) {
 	    // Just drop through.
 	} catch (InvocationTargetException ex) {
 	    // Caught an exception while imvoking the constructor.
 	    ex.getTargetException().printStackTrace();
 	    throw new RuntimeException("createFrame caught:" + ex);
 	} catch (Exception ex) {
 	    throw new RuntimeException("" + ex);
 	}
	try { 
	    // 1.2 takes a long argument to MEmbeddedFrame(int handle)
	    trace("Creating a frame  the 1.2er way");
 	    Class params[] = { long.class };
 	    Constructor cons = clz.getConstructor(params);
 	    Object args[] = { new Long(widget) };
 	    return (Frame) cons.newInstance(args);
 	} catch (NoSuchMethodException ex) {
 	    // Just drop through.
 	} catch (InvocationTargetException ex) {
 	    // Caught an exception while invoking the constructor.
 	    System.err.println("createFrame caught: " + 
			       ex.getTargetException());
 	    ex.getTargetException().printStackTrace();
 	    throw new RuntimeException("" + ex);
 	} catch (Exception ex) {
 	    throw new RuntimeException("Error in creating frame " + ex);
 	}
 	throw new RuntimeException("Couldn't find suitable " + 
				   " MEmbeddedFrame constructor");
    }
    
    /* Send a start event to the applet. 
     * NOTE: Take care to not call the actual sendEvent if the init
     * has not completed.  Init can block indefintely, and if we wait
     * for it there is a potential deadlock, since init may lock up
     * the AppletViewer while loading jars, potentially requesting
     * cookies, which require processing of the command pipe for the
     * init to terminate.
     * So, if init has not finished, we just keep track of starts and
     * stops. 
     */
    public void sendStartEvent() {
	// The startCounter will be incremented  only if the init has
	// not yet finished 
	int count = preInitStartCounter.getCounter();
	int increment = 1;
	// Increment by 2 the first time
	if (count < 0) 
	    increment = 2;
	if (!preInitStartCounter.changeCounter(increment)) {
	    Plugin.trace("Sending direct start");
	    sendEvent(sun.applet.AppletPanel.APPLET_START);
	} else {
	    Plugin.trace("Incremented counter by " + increment);
	}
    }

    /*
     * Similar to sendStartEvent
     */
    public void sendStopEvent() {
	if (!preInitStartCounter.changeCounter(-1))
	    sendEvent(sun.applet.AppletPanel.APPLET_STOP);
    }


    /**
     * Our X11 window information has changed.
     * Newid is the new X11 window ID of our plugin window in Navigator.
     * Width, height, x, and y provide more info on that window.
     * Set window is only called by the plugin, and there is only one thread
     * that can do this, so this need not be synchronized
     */
    public void setWindow(int newid, int width, 
			  int height, int x, int y) {
	int oldid = winid;
	if (newid == 0) {
	    /*
	     * netscape notify us of old window destory before SetWindow
	     * to a new window 
	     */
	    if (winid != 0) {
		this.removeNotify();
		winid = 0;
	    }
	    return;
	}
 
	// If the window id is the same, it means the dimensions are
	// also the same, so just return. This is because setwindow
	// gets called an indeterminate number of times from netscape
	// (3-4, it seems)
	if (newid == winid) {
	    trace("setWindow: Same winid.");
	    return;
	}

	winid = newid;
	Frame oldFrame = frame;
	// Create a Frame for the applet.
	// We do this by taking the X window handle provided by Netscape 
	// and creating an Xt widget inside of it.  Then we create a Motif
	// EmbeddedFrame inside that widget.
	int widget = getWidget(winid, width, height, x, y);
	frame = createFrame(widget);

	if (frame == null) {
	    System.err.println("Creation of frame failed");
	    return;
	}

	// We have to work around a JDK 1.1.4 v 1.1.5 inconsistency by
	// explicitly mapping the Widget at this point.
	mapWidget(widget, width, height, x, y);

	frame.setLayout(null);
	// Needed for accessibility stuff. There is some optimization
	// in the accessibility mechanism so that it only pays attention
	// to frames with window listeners. Ref: Peter Korn for details.
	// frame.addWindowListener(this);
	// frame.setVisible(true);
	    

	Object o = getViewedObject();
	Applet applet;
	// If the applet has not been set, attempt to set its dimensions.
	// Try to read the parameters as specified in the html tags
	// and stored in width and height. If those are not integers,
	// then just use the width, height provided by the browser
	// in this setwindow call.
	// The applet is normally set in the createApplet call
	if (o == null) {
	    trace("setWindow: call before applet exists:" + winid);
	    try {
		int wd = Integer.parseInt(getParameter("width"));
		int ht = Integer.parseInt(getParameter("height"));
		this.width = wd;
		this.height = ht;
	    } catch (NumberFormatException e) {
		// Try and maintain consistency between the parameters and
		// our width/height. If the parameters are not properly
		// set to be integers, then reset them to our w, h
		setParameter("width", new Integer(width));
		setParameter("height", new Integer(height));
		this.width = width;
		this.height = height;
	    }
	} else {
	    // The applet exists, so resize it and its frame, and
	    // have the width/height parameters reflect these values
	    setParameter("width", new Integer(width));
	    setParameter("height", new Integer(height));
	    this.width = width;
	    this.height = height;

	    setSize(width, height);
	    if (o instanceof Applet) {
		applet = (Applet) o;
	    } else {
		trace("setWindow: Viewing a bean component");
		Component c = (Component) o;
		c.setSize(width, height);
		applet = (Applet) c.getParent();
	    }
	    applet.resize(width, height);
	    applet.setVisible(true);
	}

	// Remove the appletpanel (self) from the old frame, if one existed
	if (oldFrame != null) {
	    oldFrame.removeAll();
	}

	/* Note: 2/3/99 - Ben
	   Be very careful with the order in which the window events are called.
	   frame.setbounds changes the size of the window, resulting in
	   xevents being generated that trigger the innerCanvasEH in awt_Frame.c
	   If the XtWidget has not yet been realized, this may result in the event
	   being dropped, and the widget does not get sized properly.
	   The XtWidgets are only realized when the embedded frame is shown.
	   It does not help to have the setbounds be done twice - if the bounds are the
	   same in both calls, the AWT widget will not do anything.
	   
	   The Xt Widget hierarchy for the trivial applet that does nothing is:
	   
	   widget=VendorShell level=5 name=AWTapp wd=100 ht=100
	   widget=XmForm level=4 name=main wd=100 ht=100
	   widget=XmDrawingArea level=3 name=frame_wrap wd=100 ht=100
	   widget=XmDrawingArea level=2 name=frame_canvas wd=100 ht=100
	   widget=XmDrawingArea level=1 name=canvas wd=100 ht=100
	   widget=XmDrawingArea level=0 name=canvas wd=100 ht=100

	   The width's and heights are, of course, from a particular run, but the point
	   is that they are all the same.

	   Something else that might be causing problems is the fact that the
	   EmbeddedFrame is reparented to itself (!) in awt_Frame_MEmbeddedFrame_pShow
	   which overrides the standard pShow (both args to the XReparent are 
	   the same widget).  
	   */
	setBounds(0, 0, width, height);
	// Add this appletpanel (self) to the frame we just created
	// so that it will be viewed in the new frame
	frame.add(this);
	/* Realizes the top level widget, which automatically creates the windows of
	   all its children. */
	frame.setVisible(true);
	frame.setBounds(0, 0, width, height);

	// If the docbase has already been set, we can init the applet.
	maybeInit();

	// If we had a previous frame, clean it up.
	if (oldFrame != null) {
	    oldFrame.dispose();
	}

	// To trigger the accessibility mechanisms we initiate the event
	// queue with a synthesized event
	WindowEvent actev = new WindowEvent(frame, 
					    WindowEvent.WINDOW_ACTIVATED);
	Toolkit tk = Toolkit.getDefaultToolkit();
	EventQueue eq = tk.getSystemEventQueue();
	eq.postEvent(actev);
    }

    
    /**
     * Called when Netscape wants to destroy a plugin instance.
     */
    public void destroy() {
	trace("Motif AppletViewer: destroy");
	if (!inited) {
	    trace("Not initialized. Not destroying...");
	    return;
	} 
	try {
	    initer.join(2000);
	} catch (InterruptedException ex) {
	    System.err.println("Interrupted destroy's join");
	}

	// Give the applet a chance to clean itself up.
	DestroyerThread destroy = new DestroyerThread(this);
	destroy.start();
	try {
	    destroy.join(4000);
	} catch (Exception ex) {
	    System.err.println("Destruction failed");
	    ex.printStackTrace();
	}
	trace("Calling frame.removeAll()");
	frame.removeAll();
	trace("Calling frame.dispose()");
	frame.dispose();
	trace("Done with destroy");
	frame = null;
    }

  /**
     * Get an applet parameter from the HTML.
     */
    public String getParameterFromHTML(String name) {
	trace("Trying (and failing) to get a parameter from the html:"+ name);
	return null;
    }

    private boolean evaluatingExp = false;
    private boolean returnedExp = false;
    private String returnJS = null;

    synchronized String evalString(int instance, String jsexp) {
	try {
	    while(evaluatingExp)
		wait();
	    evaluatingExp = true;
	    returnedExp = false;
	    Worker.sendJSRequest(instance, jsexp);
	    while (!returnedExp)
		wait();
	    evaluatingExp = false;
	    String ret = returnJS;
	    returnJS = null;
	    notifyAll();
	    return ret;
	} catch (Exception e) {
	    evaluatingExp = false;
	    returnJS =  null;
	    notifyAll();
	    return null;
	}
    }
    
    synchronized void setJSReply(String ret) {
	if (returnJS == null)
	    returnJS= ret;
	else 
	    returnJS += ret;
    }

    synchronized void finishJSReply() {
	returnedExp = true;
	notifyAll();
    }

    private void trace(String str) {
	Plugin.trace("MAV(" + winid + ") " + str);
    }

    private int winid;
    private Frame frame = null;

    /* Holds the initialization thread. Attempt to join before a destroy*/
    private Thread initer;
    /* True if init has been called */
    private boolean inited = false;

    private int width;
    private int height;

    /* Hack to get around problem with netscape printing - print gets
     * called 3 times. We ignore the first two calls */
    private int printCounter = 0;

    /* Counts the starts and stops before the init finishes */
    private PreEventCounter preInitStartCounter = new PreEventCounter(-1);

    static private boolean tracing;

    static private native int getWidget(int winid, int width, 
					int height, int x, int y);
    static private native int mapWidget(int widget, int w, int h, int x, int y);

}

// Make an asynchronous call to destroy. Destory sends a stop to each
// of the threads in the applet but the applet may be badly behaved
// and never properly terminate. So we do the QUIT in a separate thread
// and if that fails, we just return
class DestroyerThread  extends Thread {

    private MotifAppletViewer target;

    DestroyerThread(MotifAppletViewer target) {
	super(Thread.currentThread().getThreadGroup(), "AppletDestroyerThread");
	this.target = target;
    }

    public void run() {
	target.onClose(1000);
    }
}

/* A synchronized counter that stops working after an event happens */
class PreEventCounter {

    PreEventCounter(int initialValue) {
	eventHappened = false;
	counter = initialValue;
    }

    synchronized int getCounter() {
	return counter;
    }

    /*
     * If the event has not yet occured, increment the 
     * counter and return true. Else return false.
     */
    synchronized boolean changeCounter(int byValue) {
	if (!eventHappened) {
	    counter = counter + byValue;
	    return true;
	} else {
	    return false;
	}
    }

    /* Register that the event happened. No more counting will
     * take place 
     */
    synchronized void eventTrigger() {
	eventHappened = true;
    }

    synchronized boolean hasEventHappened() {
	return eventHappened;
    }
    
    private int counter;
    private boolean eventHappened;

}


