/*
 * @(#)Plugin.java      1.66 98/12/03
 * 
 *  Copyright 1998-2001 Sun Microsystems, Inc. 901 San Antonio Road, 
 * Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * CopyrightVersion 1.2
 * 
 */

/**
 * Main body of the Java Plug-in process on Un*x systems.
 *
 * This class is launched in a separate process from Navigator.
 * A small stub plugin runs in the Navigator process and sends
 * us requests down a pipe.
 *
 * @author Graham Hamilton
 */

package sun.plugin.navig.motif;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.net.URL;
import sun.plugin.*;

public class Plugin extends sun.plugin.JavaRunTime {

    /**
     * If tracing is enabled, print the given message.
     */
    public static void trace(String mess) {
        if (tracing) {
            try {
                System.err.println("Child: " + mess);
                tracefile.write(mess +"\n");
                tracefile.flush();
            } catch (IOException ex) {
                System.err.println("Could not write to trace file");
            }
        }
    }

    /* Return the tracing flag */
    public static boolean getTracingFlag() {
        return tracing;
    }

    public static int getNavigVersion() {
        return navig_version;
    }

    public static void error(String mess) {
        System.err.println("                      PLUGIN ERROR ");
        System.err.println("                      ************ ");
        System.err.println(mess);
        System.err.println("\n");
        trace(mess);
    }

        
    public static boolean isOldNavigator() {
        return (navig_version < 5);
    }

    /**
     * Start up the process.  This is called from our "C" main.
     */
    public static void start(boolean t) {
        tracing = t; 
        loadLibrary();	
	String version=System.getProperty("javaplugin.nodotversion");

        if (tracing) {
            try {
                tracefile = new FileWriter("/tmp/plugin_java"+version+".trace");
            } catch (IOException ex) {
                error("Could not create plugin_java"+version+".trace");
            }
        }
        String agent  = getenv("JAVA_PLUGIN_AGENT");
        if (agent.indexOf("Mozilla/3") >= 0) {
            trace("Setting Mozilla version to 3");
            navig_version = 3;
        } else if (agent.indexOf("Mozilla/4") >= 0) {
            trace("Setting Mozilla version to 4");
            navig_version = 4;
        } else {
            trace("Setting Mozilla version to 5");
            navig_version = 5;
        }
        Plugin a = new Plugin();
        a.doit();
    }

    /* Return the number of currently instantiated instances */
    public static int getNPlugins() {
        return panels.size();
    }
    

    private static String evalString(int instance, String jsexp)
    {
        MotifAppletViewer applet =(MotifAppletViewer) panels
            .get(new Integer(instance));
        return applet.evalString(instance, jsexp);
    }
   
    /*
     * This method is the main loop of the process.
     * We sit here reading and executing commands from our parent plugin.
     */
    private void doit() {
        try {
            trace("Plugin class started");

            // Load the appropriate version of the javaplugin library.
	    
            String javaHome = getenv("JAVA_HOME");
            String userHome = getenv("HOME");

            // Read the netscape proxy configuration.
            readProxyInfo();

            sun.plugin.JavaRunTime.initEnvironment(javaHome, "", userHome);

            String javapluginVersion = getenv("JAVA_PLUGIN_VERSION");
            if (javapluginVersion != null) {
                System.getProperties().put("javaplugin.version", 
                                           javapluginVersion);
            }

            if (!tracing) {
                // Assign /dev/null to be our default output. If the 
                //Java Console is enabled, our call on AppletViewer.
                //initEnvironment (below) will redirect output to the console.
                try {
                    FileOutputStream fout = new FileOutputStream("/dev/null");
                    BufferedOutputStream bout = 
                        new BufferedOutputStream(fout);
                    PrintStream pout = new PrintStream(bout);
                    System.setOut(pout);
                    System.setErr(pout);
                } catch (IOException ix) {
                    error("Java process: " + 
			  "Couldn't reassign stdout/stderr: " +
			  ix);
                }
            } 

            sun.plugin.AppletViewer.initEnvironment(proxyType, proxyInfo,
                                                    proxyOverrides);
            
            // Initializing AppletViewer will have brought up the
            // Java Console (if it is enabled).  So we can print any
            // messages we created earlier:
            trace("Initialized environment. Printing  messages.\n");
            System.out.print(messages);
            
	    Thread watcher = new Watcher();
	    watcher.start();

            initializeCommunication();

            // Now loop, diligently obeying our parent's commands.
            for (;;) {

                // If our parent has died, give up.
                if (!parentAlive()) {
                    trace("parent is dead.  Exiting.");
                    System.exit(4);
                }
                int code = -1;
                trace("Plugin: Reading next  code...");
                try {
                    code = cmdIn.readInt();
                } catch (IOException ex) {
                    trace("Could not read next command code!");
                }
                if (tracing)  
		    trace("VM Received Command >>>" +
			  protocol_to_str(code));
                if (code == JAVA_PLUGIN_NEW) {
                    // Navigator has created a new plugin instance.
                    // Create a corresponding applet viewer.
                    alive = true;
                    int id = cmdIn.readInt();
                    int bean = cmdIn.readInt();
                    int eventWindowId = cmdIn.readInt();
                    /* No event window in 5.0. */
		    // Mozilla changes
                    //if (isOldNavigator()) {
                        worker.setEventWindowID(eventWindowId);
                    //}
                    MotifAppletViewer panel;
                    if (bean == 0) {
                        // This is a regular applet.
                        panel = new MotifAppletViewer(id);
                    } else {
                        // This is a bean.
                        panel = new MotifBeansViewer(id);
                    }

                    // Set the applet parameters.
                    int argc = cmdIn.readInt();
                    for (int i = 0; i < argc; i++) {
                        String name = readString();
                        String value = readString();
                        if (tracing) 
                            trace("    >" + name + "=>" + value);
                        panel.setParameter(name, value);
                    }
                    boolean ms = false;
                    try {
                        String msString = (String) panel
                            .getParameter("MAYSCRIPT");
                        ms = Boolean.valueOf(msString).booleanValue();
                        trace("Setting mayscript to:" + ms);
                    } catch (Throwable error) {
                        error("Exception in setting mayscript");
                    }
                    if (ms) {
                        ((MotifAppletContext)panel.getAppletContext())
                            .setMayScript(ms);
                    }

                    panels.put(new Integer(id), panel);
                    trace("Registering panel:" + id);
                    replyOK(); 
                } else if (code == JAVA_PLUGIN_START) {
                    int id = cmdIn.readInt();
                    MotifAppletViewer panel = (MotifAppletViewer)panels
                        .get(new Integer(id));
                    if (panel != null) 
                        panel.sendStartEvent();
                } else if (code == JAVA_PLUGIN_STOP) {
                    int id = cmdIn.readInt();
                    MotifAppletViewer panel = (MotifAppletViewer)panels
                        .get(new Integer(id));
                    if (panel != null) 
                        panel.sendStopEvent();
                } else if (code == JAVA_PLUGIN_DESTROY) {
                    // Navigator is destroying a plugin instance.
                    // Shut down the corresponding applet panel.
                    int id = cmdIn.readInt();
                    MotifAppletViewer panel = (MotifAppletViewer)panels
                        .get(new Integer(id));
                    if (panel != null) {
                        Plugin.trace("Removing panel:" + id);
                        panels.remove(new Integer(id));
                        //This try-catch block added to fix: Bugtraq:438855
                        try {
                            panel.destroy();
                        }
                        catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                    replyOK();

                } else if (code == JAVA_PLUGIN_WINDOW) {
                    // Navigator is sending X11 window information
                    // about a plugin instance.  This incldues its X11
                    // window ID.  pass this through to the
                    // corresponding MotifAppletViewer
                    int id = cmdIn.readInt();
                    int winid = cmdIn.readInt();
                    int width = cmdIn.readInt();
                    int height = cmdIn.readInt();
                    int x = cmdIn.readInt();
                    int y = cmdIn.readInt();

                    MotifAppletViewer panel = (MotifAppletViewer)panels
                        .get(new Integer(id));
                    trace("Window " + winid + " " + width + "x" + height 
                          + " " + x + "x" + y );
		    int appletWinID = 0;
                    if (panel != null) 
                        panel.setWindow(winid, width, height, x, y);
                    replyOK();
                } else if (code == JAVA_PLUGIN_SHUTDOWN) {
                    // Navigator is about to unload our parent plugin.
                    // This will cause it do develop amnesia.  However
                    // we will stick around as a loving and attentive
                    // child, so that if the plugin gets reloaded, it
                    // can start talking to us again.
                    alive = false;

                } else if (code == JAVA_PLUGIN_DOCBASE) {
                    // When a plugin is first created, navigator doesn't tell
                    // us the docbase.   Our parental plugin plays some tricks
                    // to figure out the docbase value and sends that to us
                    // later.
                    int id = cmdIn.readInt();
                    String docbase = readString();
                    trace("DOCBASE := " + docbase);
                    if (!docbase.endsWith("/")) {
                        docbase += "/";
                    }
                    // Set the docbase for the given panel.
                    MotifAppletViewer panel = (MotifAppletViewer)panels
                        .get(new Integer(id));
                    trace("Setting docbase for " + id + " to " + docbase);
                    if (panel != null) {
                        panel.setDocumentBase(new URL(docbase));
                    }
                    // We don't need to send a reply.
                } else if (code == JAVA_PLUGIN_PROXMAP_READY) {
                    Worker.proxmapReady();
                    // We don't need to send a reply.

                } else if (code == JAVA_PLUGIN_PROXY_MAPPING) {
                    String url = readString();
                    String proxy = readString();
                    trace("Plugin.java: PROXY MAPPING: \"" + 
                          url + "\" => \"" +
                          proxy + "\"");
                    Worker.addProxyMapping(url, proxy);

                    // We don't need to send a reply.
                } else if (code == JAVA_PLUGIN_PRINT) {
		    int id = cmdIn.readInt();
		    int x = cmdIn.readInt()/10;
		    int y = cmdIn.readInt()/10;
		    int width = cmdIn.readInt()/10;
		    int height = cmdIn.readInt()/10;
		    trace("PRINT "+ id + " x=" + x + " y=" + y 
			  + " w=" + width + " h=" + height);

		    MotifAppletViewer panel =
			(MotifAppletViewer)panels.get(new Integer(id));
		    if (panel != null) {
			panel.doPrint(x, y, width, height, printOut);
		    }
		    replyOK();
                } else if (code == JAVA_PLUGIN_COOKIE) {
                    int id = cmdIn.readInt();
                    String cookie = readString();
                    MotifAppletViewer panel = (MotifAppletViewer)panels
                        .get(new Integer(id));
                    Worker.setCookieString(cookie);
                } else if (code == JAVA_PLUGIN_CACHED_FILE) {
                    String fileName = readString();
		    trace("Reading cached file:" + fileName);
                    Worker.setCacheFile(fileName);
                } else if (code == JAVA_PLUGIN_HTTPS_GET_REPLY) {
                    byte[] result = readByteArray();
                    Worker.setHttpsReply(result);
                } else if (code == JAVA_PLUGIN_HTTPS_POST_REPLY) {
                    byte[] result = readByteArray();
                    Worker.setHttpsPostReply(result);
                } else if (code == JAVA_PLUGIN_JAVASCRIPT_REPLY) { 
                    int id = cmdIn.readInt();
                    String retval = readString();
                    MotifAppletViewer panel = (MotifAppletViewer)
                        panels.get(new Integer(id));
                    if (panel != null) 
                        panel.setJSReply(retval);
                } else if (code == JAVA_PLUGIN_JAVASCRIPT_END) { 
                    int id = cmdIn.readInt();
                    MotifAppletViewer panel = (MotifAppletViewer)
                        panels.get(new Integer(id));
                    if (panel != null) 
                        panel.finishJSReply();
                } else if (code == JAVA_PLUGIN_REQUEST_ABRUPTLY_TERMINATED) {
                    Worker.terminateRequestAbruptly();
                } else if (code == JAVA_PLUGIN_ATTACH_THREAD) {
                    trace("Attach Thread ");
                    attachThread();
                } else if (code == JAVA_PLUGIN_GET_INSTANCE_JAVA_OBJECT) {
                    trace("Getting java object");
                    int ind = cmdIn.readInt();
                    int res = OJIPlugin.globalRefToJavaObjectForInstance(ind);
                    cmdOut.writeInt(res);
                    cmdOut.flush();
		} else if (code == JAVA_PLUGIN_CONSOLE_SHOW) {
		    trace("Showing Java Console");
		    JavaRunTime.showJavaConsole(true);
		} else if (code == JAVA_PLUGIN_CONSOLE_HIDE) {
		    trace("Hiding Java Console");
		    JavaRunTime.showJavaConsole(false);
		} else if (code == JAVA_PLUGIN_CONSOLE_PRINT) {
		    trace("Printing Java Console <not implemented>");
                } else {
                    // We got an unexpected command code.
                    error("Java process: unexpected request " +
                          Integer.toHexString(code));
		    System.exit(6);
                }

            }

        } catch (Throwable th) {
            // Something has gone wrong.
            // If our parent is still alive, whine about our ills.
            if (parentAlive()) {
                error("Java process caught exception: " + th);
                th.printStackTrace();
            }
            try {
                FileOutputStream fs = 
                    new FileOutputStream("plugin_stack.trace");
                PrintWriter ps = new PrintWriter(fs);
                trace("Java process caught exception: " + th.toString());
                th.printStackTrace(ps);
                ps.flush();
                ps.close();
            } catch (IOException ex) {
                error("Could not print the stack trace\n");
            }
            // In any event, exit.
            trace("Exiting. Navigator may also be dead.");
            System.exit(6);
        }
    }

    void initializeCommunication() {
        trace("Opening pipes at this end\n");

        // Fork a thread to keep an eye on our parent process.
        cmdIn = newInput("Command Input", CmdFD);
        cmdOut = newOutput("Command Output", CmdFD);
        DataInputStream workIn = newInput("Work Input", WorkFD);
        DataOutputStream workOut = newOutput("Work Output", WorkFD);
        
        FileDescriptor fd = getPipe(PrintFD);
        FileOutputStream fout = new FileOutputStream(fd);
        printOut = new PrintStream(fout);

        // Crank the Motif toolkit machinery to life.  This is
        // because we will need to access toolkit resources from
        // natiuve code before we all any AWT methods.
        java.awt.Toolkit tk = Toolkit.getDefaultToolkit();
        trace("Toolkit = " + tk.getClass().getName());
        
        worker = new Worker(workIn, workOut);
        trace("Initialized worker");
        
	// Send a special value on the out pipe to make sure that
	// that is working properly.
        try {
            cmdOut.write(17);
            cmdOut.flush();
            trace("Wrote initial ack on command pipe");
        } catch (IOException ex) {
            error("Error in writing back to the parent");
        }
	trace("Wrote the initial ack\n");

	// Get the event window ID
        int wid = -1;
        try {
            wid = cmdIn.readInt();
        } catch (IOException ex) {
            trace("Child could not read window id!");
        }
	trace("Read the event window id:" + wid);
        worker.setEventWindowID(wid);
        trace("Set window ID");
    }

    public static MotifAppletViewer getViewer(int plugin_number) {
	Integer i = new Integer(plugin_number);
	MotifAppletViewer maf =  (MotifAppletViewer) panels.get(i);
	if (maf == null) {
	    Plugin.error("Could not find a Viewer for " + plugin_number);
	} else {
	    Plugin.trace("Found a viewer for:" + plugin_number);
	}
	return maf;
    }

    // Create a data input stream for the raw pipe 'fd'
    static DataInputStream newInput(String msg, int fd) {
        Plugin.trace("Creating input pipe:" + msg + " fd = " + fd);
        FileDescriptor fdin = getPipe(fd);
        FileInputStream fin = new FileInputStream(fdin);
        DataInputStream ds =  new DataInputStream(fin);
        return ds;
    }


    // Create a data input stream for the raw pipe 'fd'
    static DataOutputStream newOutput(String msg, int fd) {
        Plugin.trace("Creating output pipe:" + msg + " fd = " + fd);
        FileDescriptor fdout = getPipe(fd);
        FileOutputStream fout = new FileOutputStream(fdout);
        BufferedOutputStream bout = new BufferedOutputStream(fout);
        DataOutputStream dout =  new DataOutputStream(bout);
        return dout;
    }

   
    private class Watcher extends Thread {
        public void run() {
            Plugin.trace(" Starting watcher\n");
            // We want to exit when our parent browser process exits.
            // 
            // We'd like to do this automagically by catching the EOF on
            // the control pipe.
            //
            // Unfortunately because of the way green threads interact with
            // pipe IO, we can't rely on getting woken out of our IO read
            // when the pipe gets closed.
            //
            // So we have to resort to checking on our parent at periodic
            // intervals.  Sigh.

            for (;;) {
                try {
                    Thread.sleep(30 * 1000);
                } catch (InterruptedException ix) {
                    // The Interrupted state is auto-clearing.
                }

                if (!parentAlive()) {
                    trace(" exiting due to parent death");
                    System.exit(2);
                }
            }
        }
    }


    /**
     * Read an ASCII string from out input pipe.
     */
    private static String readString() throws IOException {
        int len = cmdIn.readShort();
        char buff[] = new char[len];
        for (int i = 0; i < len; i++) {
            buff[i] = (char)cmdIn.readByte();
        }
        String res  = new String(buff);
        trace("readString:" + res);
        return res;
    }

    /* Read an array of bytes from the command pipe */
    private static byte[] readByteArray() throws IOException {
        int len = cmdIn.readInt(); // Length;
        byte buff[] = new byte[len];
        for(int i = 0; i < len; i++) {
            buff[i] = (byte) cmdIn.readByte();
        }
        return buff;
    }

    /**
     * Send an acknowledgement reply to our parent.
     */
    private static void replyOK() throws IOException {
        trace("Sending OK reply");
        cmdOut.writeInt(JAVA_PLUGIN_OK);
        cmdOut.flush();
    }

    /**
     * Load the appropriate version of our native library.
     */
    private static void loadLibrary() {
	String libname = System.getProperty("javaplugin.lib");

        try {
            System.load(libname);
        } catch (UnsatisfiedLinkError ex) {
            System.err.println("Plugin could not load:" + libname);
            // Note the following error message could be deceptive -
            // if the file is not found, it could mean that it had
            // unresolved symbols
            System.err.println("Path is:" + 
                               System.getProperty("java.library.path"));
            System.err.println(ex.toString());
        } 
    }


    void readProxyInfo() {
        proxyType = sun.plugin.AppletViewer.PROXY_TYPE_NO_PROXY;
        proxyInfo = "";
        proxyOverrides = "";
        String noProxy = null;

        String agent = getenv("JAVA_PLUGIN_AGENT");
        try {
	    //            if (agent.indexOf("Mozilla/3") >= 0) {
	    if(navig_version == 3 )
		{
		    // Parse a Netscape 3 style property file.
                String home = getenv("HOME");
                Hashtable props = new Hashtable();
                FileInputStream fin = new 
                    FileInputStream(home + "/.netscape/preferences");
                BufferedReader bin = new 
                    BufferedReader(new InputStreamReader(fin));
                for (;;) {
                    String line = bin.readLine();
                    if (line == null) {
                        break;
                    }
                    int ix = line.indexOf(':');
                    if (ix < 0) {
                        continue;
                    }
                    String name = line.substring(0,ix);
                    String value = line.substring(ix+1);
                    name = name.trim();
                    value = value.trim();
                    if (value.length() > 0) {
                        props.put(name, value);
                    }
                }

                // OK, now crunch over the data we've read.
                String mode = (String)props.get("PROXY_MODE");
                if (mode == null || mode.equals("0")) {
                    // No proxy configured.  We're done.
                    return;
                }
                if (mode.equals("2")) {
                    // Automatic proxy configuration isn't allowed with NS3.
                    messages += getMessage("no_autoconfig_on_ns3") + "\n";
                    return;
                }
                // Otherwise, set up manual poxy configuration.
                proxyType = sun.plugin.AppletViewer.PROXY_TYPE_MANUAL;
                String kinds[] = { "HTTP", "FTP", "GOPHER", "HTTPS", "WAIS" };
                for (int i = 0; i < kinds.length; i++) {
                    String kind = kinds[i];
                    String proxy = (String)props.get(kind + "_PROXY");
                    if (proxy != null) {
                        if (proxyInfo.length() > 0) {
                            proxyInfo += ";";
                        }
                        proxyInfo += kind.toLowerCase(java.util.Locale.ENGLISH) + "=" + proxy;
                    }
                }
                String sockshost = (String)props.get("SOCKS_HOST");
                if (sockshost != null) {
                    if (proxyInfo != null)
                        proxyInfo += ";";
                    proxyInfo += sockshost;
                }
                noProxy = (String)props.get("NO_PROXY");

            } else if (navig_version == 4) {
                // Parse a Netscape 4 style property file
                String home = getenv("HOME");
                Hashtable props = new Hashtable();
                FileInputStream fin = new 
                    FileInputStream(home + "/.netscape/preferences.js");
                BufferedReader bin = new 
                    BufferedReader(new InputStreamReader(fin));
                for (;;) {
                    String line = bin.readLine();
                    if (line == null) {
                        break;
                    }
                    // Is this a real preference line?
                    if (line.indexOf("user_pref(\"") < 0) {
                        continue;
                    }
                    line = line.substring(line.indexOf('"')+1);
                    int ix = line.indexOf('"');
                    String name = line.substring(0,ix);
                    line = line.substring(ix+2);
                    line = line.trim();
                    // Quoted strings need to be handled separately.
                    String value;
                    if (line.charAt(0) == '"') {
                        ix = line.indexOf('"', 1);
                        value = line.substring(1, ix);
                    } else {
                        ix = line.indexOf(')');
                        value = line.substring(0,ix);
                    }
                    props.put(name, value);
                }

                // OK, now crunch over the data we've read.
                String mode = (String)props.get("network.proxy.type");
                if (mode == null || mode.equals("0")) {
                    // No proxy configured.  We're done.
                    return;
                }
                if (mode.equals("2")) {
                    // Automatic proxy configuration.
                    // This will get handled by back-calls to our parent plugin
                    proxyType = sun.plugin.AppletViewer.PROXY_TYPE_AUTOCONFIG;
                    return;
                }
                // Otherwise, set up manual poxy configuration.
                proxyType = sun.plugin.AppletViewer.PROXY_TYPE_MANUAL;
                String kinds[] = { "http", "ftp", "gopher", "https", "wais" };
                for (int i = 0; i < kinds.length; i++) {
                    String kind = kinds[i];
                    String proxy = (String)props.get("network.proxy." + kind);
                    if (proxy != null) {
                        if (proxyInfo.length() > 0) {
                            proxyInfo += ";";
                        }
                        proxyInfo += kind.toLowerCase() + "=" + proxy;
                        String port = (String)props.get("network.proxy." +
                                                        kind + "_port");
                        if (port != null) {
                            proxyInfo += ":" + port;
                        }
                    }
                }
                String sockshost = 
                    (String) props.get("network.hosts.socks_server");
                if (sockshost != null) {
                    if (proxyInfo.length() > 0)
                        proxyInfo += ";";
                    proxyInfo += "socks="+sockshost;
                    String port = 
                        (String) props.get("network.hosts.socks_serverport");
                    if (port != null)
                        proxyInfo += ":" + port;
                }
                noProxy = (String)props.get("network.proxy.no_proxies_on");
		}
	    else if ( navig_version == 5){
		proxyType = sun.plugin.AppletViewer.PROXY_TYPE_BROWSERCONFIG;
		return;
	    }
	    
	} catch (Exception ex) {
            messages += getMessage("bad_read_ns_props") + ex + "\n";
        }
	
        // Translate any Netscape proxy overrides to standard form.
        // For example ".eng, .eng.sun.com" becomes "*.eng,*.eng.sun.com".
        while (noProxy != null) {
            String current;
            int ix = noProxy.indexOf(',');
            if (ix < 0) {
                current = noProxy;
                noProxy = null;
            } else {
                current = noProxy.substring(0, ix);
                noProxy = noProxy.substring(ix+1);
            }
            current = current.trim();
            if (current.length() == 0) {
                continue;
            }
            if (proxyOverrides.length() > 0) {
                proxyOverrides += ",";
            }
            proxyOverrides += "*" + current;
        }
    }

    /**
     * Return true iff the plugin is currently alive.
     */
    static boolean isPluginAlive() {
        return alive;
    }

    /**
     * Method to get an internationalized string from the Java Plugin resource.
     */
    static String getMessage(String key) {
        try {
            return rb.getString(key);
        } catch (java.util.MissingResourceException ex) {
            return key;
        }
    }

    static boolean getTracing() { return tracing; }

    private static Hashtable panels = new Hashtable();
    private static int nextId = 0;
    private static boolean tracing;
    private static PrintStream printOut;
    private static DataOutputStream cmdOut;
    private static DataInputStream cmdIn;
    private static Worker worker;
    private static boolean alive;
    private static String proxyInfo;
    private static int proxyType;
    private static String proxyOverrides;
    private static int navig_version;
    private static ResourceBundle rb = 
        ResourceBundle.getBundle("sun.plugin.resources.Motif");
    private static FileWriter tracefile;

    // Queued messages during initialization:
    private static String messages = "";

    // These should be kept consistent with the defs in plugin_defs.h
    public final static int CmdFD = 11;
    public final static int WorkFD = 12;
    public final static int PrintFD = 13;

    // Pointer to an encapsulated version of the OJI specific stuff
    // in the plugin.
    public static OJIPlugin oji;

    // Codes for messages from plugin to java_vm process.
    // Keep consistent with the values in the protocol.h files
    public final static int JAVA_PLUGIN_NEW             	    = 0xFA0001;
    public final static int JAVA_PLUGIN_DESTROY         	    = 0xFA0002;
    public final static int JAVA_PLUGIN_WINDOW          	    = 0xFA0003;
    public final static int JAVA_PLUGIN_SHUTDOWN        	    = 0xFA0004;
    public final static int JAVA_PLUGIN_DOCBASE         	    = 0xFA0005;
    public final static int JAVA_PLUGIN_PROXMAP_READY   	    = 0xFA0006;
    public final static int JAVA_PLUGIN_PROXY_MAPPING   	    = 0xFA0007;
    public final static int JAVA_PLUGIN_COOKIE          	    = 0xFA0008;
    public final static int JAVA_PLUGIN_CACHED_FILE     	    = 0xFA0009;
    public final static int JAVA_PLUGIN_JAVASCRIPT_REPLY	    = 0xFA000A;
    public final static int JAVA_PLUGIN_JAVASCRIPT_END  	    = 0xFA000B;
    public final static int JAVA_PLUGIN_START           	    = 0xFA0011;
    public final static int JAVA_PLUGIN_STOP            	    = 0xFA0012;
    public final static int JAVA_PLUGIN_ATTACH_THREAD   	    = 0xFA0013;
    public final static int JAVA_PLUGIN_REQUEST_ABRUPTLY_TERMINATED = 0xFA0014;
    public final static int JAVA_PLUGIN_GET_INSTANCE_JAVA_OBJECT    = 0xFA0015;
    public final static int JAVA_PLUGIN_PRINT                       = 0xFA0016;
    public final static int JAVA_PLUGIN_HTTPS_GET_REPLY             = 0xFA0017;
    public final static int JAVA_PLUGIN_HTTPS_POST_REPLY            = 0xFA0018;
    public final static int JAVA_PLUGIN_CONSOLE_SHOW                = 0xFA0019;
    public final static int JAVA_PLUGIN_CONSOLE_HIDE                = 0xFA001A;
    public final static int JAVA_PLUGIN_CONSOLE_PRINT               = 0xFA001B;

    // Replies from java_vm to plugin.
    public final static int JAVA_PLUGIN_OK              = 0xFB0001;

    static native FileDescriptor getPipe(int fd);
    static native boolean parentAlive();
    
    
    private static native String getenv(String name);
    private static native void attachThread();

    /* Convert the protocol code to a String for debugging purposes */
    public static String protocol_to_str(int code) {
        switch (code) {
        case JAVA_PLUGIN_NEW: return "JAVA_PLUGIN_NEW";
        case JAVA_PLUGIN_DESTROY: return "JAVA_PLUGIN_DESTROY";
        case JAVA_PLUGIN_WINDOW: return "JAVA_PLUGIN_WINDOW";
        case JAVA_PLUGIN_SHUTDOWN: return "JAVA_PLUGIN_SHUTDOWN" ;
        case JAVA_PLUGIN_DOCBASE: return "JAVA_PLUGIN_DOCBASE" ;
        case JAVA_PLUGIN_PROXMAP_READY: return "JAVA_PLUGIN_PROXMAP_READY";
        case JAVA_PLUGIN_PROXY_MAPPING: return "JAVA_PLUGIN_PROXY_MAPPING";
        case JAVA_PLUGIN_COOKIE: return "JAVA_PLUGIN_COOKIE     ";
        case JAVA_PLUGIN_CACHED_FILE: return "JAVA_PLUGIN_CACHED_FILE";
        case JAVA_PLUGIN_JAVASCRIPT_REPLY: 
            return "JAVA_PLUGIN_JAVASCRIPT_REPLY";
        case JAVA_PLUGIN_JAVASCRIPT_END: return "JAVA_PLUGIN_JAVASCRIPT_END";
        case JAVA_PLUGIN_START: return "JAVA_PLUGIN_START";
        case JAVA_PLUGIN_STOP: return "JAVA_PLUGIN_STOP";
        case JAVA_PLUGIN_ATTACH_THREAD: return "JAVA_PLUGIN_ATTACH_THRE";
        case JAVA_PLUGIN_OK: return "JAVA_PLUGIN_OK";
	case JAVA_PLUGIN_PRINT: return "JAVA_PLUGIN_PRINT";
        case JAVA_PLUGIN_HTTPS_GET_REPLY: return "JAVA_PLUGIN_HTTPS_GET_REPLY";
        case JAVA_PLUGIN_HTTPS_POST_REPLY: return "JAVA_PLUGIN_HTTPS_POST_REPLY";
        case JAVA_PLUGIN_REQUEST_ABRUPTLY_TERMINATED            : 
            return "JAVA_PLUGIN_REQUEST_ABRUPTLY_TERMINATED";
	case JAVA_PLUGIN_CONSOLE_SHOW: return "JAVA_PLUGIN_CONSOLE_SHOW";
	case JAVA_PLUGIN_CONSOLE_HIDE: return "JAVA_PLUGIN_CONSOLE_HIDE";
	case JAVA_PLUGIN_CONSOLE_PRINT: return "JAVA_PLUGIN_CONSOLE_PRINT";
        default:
            return "Unknown code:" + code;
        }

    }

}


