/*
 * @(#)Worker.java	1.31 00/05/04
 * 
 *  Copyright 1998-2001 Sun Microsystems, Inc. 901 San Antonio Road, 
 * Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * CopyrightVersion 1.2
 * 
 */

package sun.plugin.navig.motif;

import java.io.*;
import java.net.URL;
import java.util.Hashtable;
import sun.plugin.usability.Trace;

/* 
 * This class is used to manage the "worker pipe" that allow the java
 * child process to send supplications back to the parental plugin.
 *
 * The Worker class establishes the connection from the plugin to the
 * browser. Immediately after creation, it sends an acknowledgement
 * back to the browser side (the browser waits for this
 * acknowledgement before proceeding after creating the plugin). See
 * JavaVM::StartJavaVM 
 */
public class Worker {


    Worker(DataInputStream data_in, DataOutputStream data_out) {
	this.data_out = data_out;
	this.data_in = data_in;
	this.table = new Hashtable();
    }

    /*
     * Set the ID of the event window used to wake up the main thread
     * in the browser 
     */
    void setEventWindowID(int winid) {
	eventWindowID = winid;
    }

    /* 
     * Indicate that the request currently in process was
     * abruptly terminated due to some bad condition
     * (usually the plugin instance going away before the
     * request is done).
     */
    static synchronized void terminateRequestAbruptly() {
	terminateAbruptly();
    }

    private static synchronized void pushRequest(String mesg) 
	throws IOException {
        data_out.flush();
	count++;
	Plugin.trace("Worker pushRequest:" + mesg);

	// Notify the event widget on the browser side to process 
	// the request.
    	setX11Property(eventWindowID, "java_plugin_work", "" + count);
    }

    private static synchronized void writeString(String mess) 	
	throws IOException {
        data_out.writeInt(mess.length());
	for (int i = 0; i < mess.length(); i++) {
	    char ch = mess.charAt(i);
	    data_out.writeByte((byte)ch);
	}
    }


    private static synchronized void writeByteArr(byte[] mess)
	throws IOException {
        data_out.writeInt(mess.length);
	/* This will be horribly slow. Add a buffer or redo in native code? */
	for (int i = 0; i < mess.length; i++) 
	    data_out.writeByte(mess[i]);
    }

    /**
     * Request the plugin to move navigator to a different page.
     * Called by the MotifAppletContext to show a document at the head
     * 
     */
    static synchronized void showDocument(int pluginIndex, java.net.URL url,
					  String target) {
	try {
	    data_out.writeInt(JAVA_PLUGIN_SHOW_DOCUMENT);
 	    data_out.writeShort(pluginIndex);
	    writeString("" + url);
	    writeString(target);
 	    pushRequest("showDocument");
	} catch (IOException ix) {
	    Plugin.error("java process showDocument: write to parent failed"
			 + "\nException: " + ix.toString());
	}
    }

    /*
     * Show a status messgae for a given plugin instance.
     */
    static synchronized void showStatus(int pluginIndex, String mess) {
      //	if (!Plugin.isOldNavigator()) {
      //    Plugin.error("Show status not working with Mozilla 5");
      //    return;
      //}
	try {
	    // We often get a flurry of status messages after the plugin
	    // has been deleted.  Avoid queueing these.
	    if (! Plugin.isPluginAlive()) {
		return;
	    }

            // AppletContext.showStatus states that a null parameter
            // clears the status bar.
            if (mess == null) {
                mess = "";
            }

	    // Send all the status messages to the console as well.
	    data_out.writeInt(JAVA_PLUGIN_SHOW_STATUS);
 	    data_out.writeShort(pluginIndex);
	    writeString(mess);
 	    pushRequest("showstatus:" + mess);
	} catch (IOException ix) {
	    Plugin.error("java process show status: write to parent failed"
			 + "\nException: " + ix.toString());
	}
    }

    /*
     * Find out which proxy to use for a given URL. This call
     * must wait until the proxy mapping table has been initialized
     * by the browser. It then calls into the browser and the browser
     * is queried for the proxy mapping. This querying works differently
     * in Netscape 4.0 (and earlier), where a complex Javascript is
     * evaluated to find the proxy, and in Mozilla, where there
     * is a direct interface to find the proxy.
     */
    static synchronized String getProxySettings(String rawURL) {
	try {
	    URL url = new URL(rawURL);
	    // Navig 5 doesn't need to wait
	    if (proxmap == null && Plugin.getNavigVersion() == 5 )
		proxmap = new Hashtable();

	    // Wait until the plugin is ready to map proxies.
	    while (proxmap == null) {
		Plugin.trace("getProxySettings. Looking for proxmap.");
		Worker.class.wait();
	    }
	    String key = url.getProtocol() + ":" + 
		url.getHost() + ":" + url.getPort();

	    Plugin.trace("getProxySettings. Using key:" + key);
	    // If the result is already cached, don't ask again.
	    String result = (String) proxmap.get(key);
	    if (result != null) {
		Plugin.trace("Retrieving cached proxy:" + result);
		return result;
	    } 
	    // Wait to be alone to make a request
	    enterRequest("Proxy");

	    // Send a request to our parent plugin.
	    data_out.writeInt(JAVA_PLUGIN_FIND_PROXY);
 	    data_out.writeShort(-1);
	    writeString(rawURL);
	    writeString(url.getHost());
	    
 	    pushRequest("FindProxy");

	    // Wait for the result to be put in our proxmap table.
	    waitForResponse("Proxy");
	    
	    result = (String) proxmap.get(key);
	    clearRequest();
	    
	    return result;
	} catch (java.net.MalformedURLException ex) {
	    System.err.println("Bad URL in getProxySettings: " + rawURL);
	} catch (IOException ex) {
	    System.err.println("getProxySettings: IO error on plugin");
	} catch (InterruptedException ex) {
	    System.err.println("getProxySettings interrupted");
	}
	// We only get here if we failed.
	clearRequest();
	return null;
    }

    static synchronized void proxmapReady() {
	proxmap = new Hashtable();
	Worker.class.notifyAll();
    }

   
     /*
     * Upon a request from the plugin, add a URL to the proxy mapping
     * This sets up a mapping betweeen rawURL and the name by which it
     * can be obtained, "proxy"
     */
    static synchronized void addProxyMapping(String rawURL, String proxy) {
	try {
	    URL url = new URL(rawURL);
	    String key = url.getProtocol() + ":" + url.getHost() 
		+ ":" + url.getPort();
	    proxmap.put(key, proxy);
	    responseIsReady("Proxy");
	} catch (java.net.MalformedURLException ex) {
	    System.err.println("Bad URL in getting proxy: " + rawURL);
	}
    }

     /*
     *  Send a request to the plugin to eval a javascript string
     *  'expr' in the context of the plugin instance 'id'.
     *  May not execute if the plugin is already shut down.
     */
    static synchronized void sendJSRequest(int id, String expr) {
	try {
	    data_out.writeInt(JAVA_PLUGIN_JAVASCRIPT_REQUEST);
	    data_out.writeShort((short) id);
	    writeString(expr);
	    pushRequest("JS Request");
	} catch (IOException e) {
	    System.err.println("sendJSRequest: io error in Plugin " + e);
	}
    }
		
    /* Get a cached file, if one exists, for the url 'url' from
     * the browser
     */
    static synchronized String getCacheFileForURL(String url) {
	Plugin.trace("Worker.getCacheFileForURL:" + url);
	// Mozilla changes
	if (!Plugin.isOldNavigator()) {
		return null;
	}
	try {
	    // Start a request
	    enterRequest("Cache");
	    cacheFile = null;

	    data_out.writeInt(JAVA_PLUGIN_CACHE_URL);
	    data_out.writeShort(-1);
	    writeString(url);

	    pushRequest("getCache");

	    // Wait for a reply
	    waitForResponse("Cache");

	    String result = cacheFile;

	    clearRequest();
	    
	    if (result == null) 
		Plugin.trace(" No cache file");
	    else
		Plugin.trace(" Got cache file:" + cacheFile);

	    return result;
	    
	} catch (IOException ex) {
	    Plugin.trace("sendCacheRequest: io error in plugin "+ex);
	}
	// Even if there was some other exception limp along
	System.err.println("Error in getting cache file");
	clearRequest();
	return null;
    }

    /*
     *  Inform the worker that the response to the current 'url'
     *  cache request is 'c'
     */
    static synchronized void setCacheFile(String c) {
	cacheFile = c;
	responseIsReady("Cache");
    }

    /**
     * Find out which cookie to use for a given URL. Called
     * from native code from the ActivatorCookieHandler
     * Shared code keeps a cache and only calls us on cache misses.
     * This is done by querying for all cookies, since there is no direct
     * browser interface for finding the cookies for a URL.
     */
    static synchronized String findCookieForURL(String rawURL) {
	Plugin.trace("Worker.findCookieForURL: " + rawURL);
	try {
	    // Wait till all other requests are done
	    enterRequest("Cookie");
	    
	    cookieString = null;

	    // Make and send the cookie message
	    data_out.writeInt(JAVA_PLUGIN_FIND_COOKIE);
	    data_out.writeShort(-1);
	    writeString(rawURL);
	    pushRequest("FindCookie");

	    // Wait for a reply
	    waitForResponse("Cookie");

	    String returnString = cookieString; 
	    clearRequest();

	    Plugin.trace(" Got cookie string:" + returnString);
	    return returnString;
	} catch (IOException e) {
	    System.err.println("IOException in findCookieURL");
	} 
	// Even if there is a runtime exception, clear all flags. We
	// may be in a bad state, but limp along if possible
	System.err.println("Bad termination of cookie request!");
	clearRequest();
	return null;
    }


    /**
     * Set cookie for a given URL. Called from ActivatorCookieHandler
     * when the server sends us a "Set-Cookie" HTTP header.
     */
    public static synchronized String setCookieForURL(String rawURL, String cookie) 
    {
	Plugin.trace("Worker.setCookieForURL: " + rawURL + "=" + cookie);

	try {
	    data_out.writeInt(JAVA_PLUGIN_SET_COOKIE);
 	    data_out.writeShort(-1);
	    writeString("" + rawURL);
	    writeString("" + cookie);
 	    pushRequest("setCookie");
	} catch (IOException ix) {
	    Plugin.error("java process setCookie: write to parent failed"
			 + "\nException: " + ix.toString());
	}
	return null;
    }


    /**
     * Receive a response for a cooking request. Pairs up with
     * findCookieForURL and called by Plugin when a cookie appears.
     */
    static synchronized void setCookieString(String cookie) {
	cookieString = cookie;
	responseIsReady("Cookie");
    }

    /**
     * Post the HTTPS header and data to a web server using the 
     * browser's protocol handler 
     *
     * @param connection URLConnection
     * @param url URL
     * @param buf Buffer to post
     * @param length Length of buffer
     * @return HTTPS respond data
     */
    static synchronized byte[] postHttps(Object connection, String url,
					  byte[] buf, int length) {

	Plugin.trace("Worker.postHttps: " + url);

        byte[] result = null;

	try {
	    if (!Plugin.isPluginAlive()) {
		return null;
	    }

	    enterRequest("postHttps");
	    data_out.writeInt(JAVA_PLUGIN_POST_HTTPS);
	    data_out.writeShort(-1); /* Any plugin instance will do */
	    writeString(url);
	    writeByteArr(buf);
	    data_out.writeInt(length);
	    pushRequest("postHttps");

	    // Wait for a response
            waitForResponse("postHttps");

            result = httpsPostReply;
	    httpsPostReply = null;

	} catch (IOException ix) {
	    Plugin.error("IOException in postHttpsURL" + ix.toString());
	} finally {
	    clearRequest();
	}

	return result;
    }


    /**
     * Get the HTTPS data from a web server using the 
     * browser's protocol handler 
     *
     * @param connection URLConnection
     * @param url URL
     */
    static synchronized void getHttps(Object connection, String url) {
	/* Implementation:
	   - Send a request of JAVA_PLUGIN_HTTPS_GET on the Worker pipe.
	   - Process the request in JavaVM.
	   - Post a GetURL of JAVA_PLUGIN_GET_HTTPS
	   - Get a reply creating a new stream in JavaPluginInstance
	   - After the stream is complete, send back a response on the
	     command pipe of JAVA_PLUGIN_HTTPS_GET_REPLY
	 */

	Plugin.trace("Worker.getHttps: " + url);

	try {
	    if (!Plugin.isPluginAlive()) {
		return;
	    }

	    enterRequest("getHttps");
	    data_out.writeInt(JAVA_PLUGIN_GET_HTTPS);
	    data_out.writeShort(-1);
	    writeString(url);
	    pushRequest("getHttps:" + url);

	    // Wait for the response
	    waitForResponse("getHttps");

	    table.put(connection, new hashnode(httpsGetReply, 0));

	} catch (IOException ex) {
	    System.err.println("getHttps: IO Error on plugin");
	    ex.printStackTrace();
	} finally {
	    clearRequest();
	}
    } 


    /**
     * Read the HTTPS data that was obtained by HTTPS GET through
     * a web server using the browser's protocol handler 
     *
     * @param connection URLConnection
     * @param buf Buffer to store data
     * @param offset Offset to buffer
     * @param len Length of data
     * @return number of byte have been read
     */
    static synchronized int readHttpsURL(Object connection, byte[] buf, 
						int offset, int len) {
	Plugin.trace("Worker.readHttpsURL: offset=" + offset + " len=" + len);

	// Obtain HTTPS reply node
	hashnode node = (hashnode)table.get(connection);

	// Obtain HTTPS reply data and seek position
	byte[] httpsResult = node.get_httpsReply();
	int seek_pos = node.get_seek_pos();

	// Check if we have reached the end of the stream
	if (httpsResult.length == seek_pos)
	    return -1;

	// Otherwise, calculate the number of byte to be returned.
        int length = 0;

        if (len > httpsResult.length - seek_pos) 
            length = httpsResult.length - seek_pos;
        else
            length = len;

        for(int i = 0; i < length; i++) {
            buf[offset + i] = httpsResult[seek_pos + i];
        }

	// Advance seek position
	node.increment_seek_pos(length);

	Plugin.trace("Worker.readHttpsURL: returned length=" + length);

        return length;
    }


    /**
     * Determine number of byte available to be read from
     * the HTTPS data that was obtained by HTTPS GET through
     * a web server using the browser's protocol handler 
     *
     * @param connection URLConnection
     * @return number of byte could be read
     */
    static synchronized int bytesAvailable(Object connection) {
	Plugin.trace("Worker.bytesAvailable");

        int length = 0;
	hashnode node = (hashnode)table.get(connection);
	byte[] httpsResult = node.get_httpsReply();
	int seek_pos = node.get_seek_pos();

        length = httpsResult.length - seek_pos;

        if (length < 0) 
	    length = 0;

	Plugin.trace("Worker.bytesAvailable: return " + length);

        return length;
    }
 

    /**
     * Close the InputStream that represents the HTTPS data 
     * that was obtained by HTTPS GET through a web server 
     * using the browser's protocol handler 
     *
     * @param connection URLConnection
     */
    static synchronized void closeStream(Object connection) {
	Plugin.trace("Worker.closeStream");

	table.remove(connection);
    }
 

    /**
     * Set the HTTPS data that was obtained by HTTPS GET 
     * through a web server using the browser's protocol 
     * handler 
     *
     * @param reply HTTPS data byte array
     */
    public static synchronized void setHttpsReply(byte[] reply) {
	Plugin.trace("Worker.setHttpsReply");

	httpsGetReply = reply;
      	responseIsReady("getHttps");
    }


    /**
     * Set the HTTPS data that was obtained by HTTPS POST
     * through a web server using the browser's protocol 
     * handler 
     *
     * @param reply HTTPS data byte array
     */
    public static synchronized void setHttpsPostReply(byte[] reply) {
	Plugin.trace("Worker.setHttpsPostReply");

	httpsPostReply = reply;
	responseIsReady("postHttps");
    }

    private static DataInputStream data_in;
    private static DataOutputStream data_out;
    private static int eventWindowID;
    private static int count;

     // Cache and state for proxy configuration.
    private static Hashtable proxmap;

    // Cookie response is stored here
    private static String cookieString = null;

    // Cache response is stored here
    private static String cacheFile = null;

    // Https get response stored here
    private static byte[] httpsGetReply = null;

    // Https result stored here
    private static Hashtable table = null;

    // Https Postreply response stored here
    private static byte[] httpsPostReply = null;

    // Codes for messages from java process to plugin,
    public final static int JAVA_PLUGIN_SHOW_STATUS	   = 0xF60001;
    public final static int JAVA_PLUGIN_SHOW_DOCUMENT	   = 0xF60002;
    public final static int JAVA_PLUGIN_FIND_PROXY	   = 0xF60003;
    public final static int JAVA_PLUGIN_FIND_COOKIE        = 0xF60004;
    public final static int JAVA_PLUGIN_CACHE_URL          = 0xF60005;
    public final static int JAVA_PLUGIN_JAVASCRIPT_REQUEST = 0xF60006;
    public final static int JAVA_PLUGIN_GET_HTTPS          = 0xF60007;
    public final static int JAVA_PLUGIN_POST_HTTPS         = 0xF60008;
    public final static int JAVA_PLUGIN_SET_COOKIE         = 0xF60009;
    public final static int JAVA_PLUGIN_WORKER_TERMINATE   = 0x000001;

    private static native void setX11Property(int winid, String property, 
					      String mess);


    /* Handler to serialize requests from the plugin and to handle
     * abrupt termination. Multiple threads can make requests on
     * a request handler and only one will be let through at a time
     * The sequence of calls is:
     *   enterRequest -> waitForResponse -> clearRequest
     *                -> responseIsReady
     *                -> terminateAbruptly
     */
    /*
     * Wait until we can enter a request
     */
    synchronized static void enterRequest(String reqName) {
	try {
	    while(requestStatus != REQUEST_IDLE) 
		Worker.class.wait();
	    requestName = reqName;
	    requestStatus = REQUEST_IN_PROGRESS;
	    Plugin.trace("Entering request for:" + reqName);
	} catch (InterruptedException e) {
	    Plugin.trace("Request was interrupted when entering");
	}
    }

    /*
     * Wait until a response to the cookie, proxie of cache response
     * arrives. 
     * @arg reqName   Informational description of the type of request 
     * @return true if a response was received, false if abruptly
     *             terminated.
     */
    synchronized static boolean waitForResponse(String reqName) {
	Plugin.trace("Waiting for response: " + reqName);
	try {
	    while (requestStatus != RESPONSE_IS_READY) {
		Worker.class.wait();
		Plugin.trace("Woke up in request for:" + reqName);
		if (requestStatus == REQUEST_ABRUPTLY_TERMINATED) {
		    clearRequest();
		    return false;
		}
	    }
	    Plugin.trace("Got response for request:" + reqName);
	    return true;
	} catch(InterruptedException e) {
 	    Plugin.trace("Request was interrupted before response");
	}
	return false;
    }

    /* 
     * Change the request status to indicate that the response
     * is available and wake up all waiting threads
     */
    synchronized static void responseIsReady(String reqName) {
	Plugin.trace("Response is ready:" + reqName);
	requestStatus = RESPONSE_IS_READY;
	Worker.class.notifyAll();
    }

    /*
     * An abrupt termination request with the request incomplete.
     * This does not clear the request.
     */
    synchronized static void terminateAbruptly() {
	Plugin.trace("Request was abruptly terminated");
	if (requestStatus != REQUEST_IDLE) {
	    requestStatus = REQUEST_ABRUPTLY_TERMINATED;
	}
	Worker.class.notifyAll();
    }


    /* 
     * End the current request to permit a new request
     */
    synchronized static void clearRequest() {
	Plugin.trace("Request was cleared");
	requestStatus = REQUEST_IDLE;
        Worker.class.notifyAll();
    }

    /* Name of the current request */
    private static String requestName;

    // Codes for the status of the request. 
    private final static int REQUEST_IDLE = 1;
    private final static int REQUEST_IN_PROGRESS = 2;
    private final static int RESPONSE_IS_READY = 3;
    private final static int REQUEST_ABRUPTLY_TERMINATED = 4;

    // Current request status
    private static int requestStatus = REQUEST_IDLE;

}

class hashnode {
    
    // https Reply
    private byte[] httpsReply;
    private int seek_pos;

    hashnode(byte[] reply, int pos) {
	this.httpsReply = new byte[reply.length];
	for (int i = 0; i < httpsReply.length; i++) {
		this.httpsReply[i] = reply[i];
	}
	this.seek_pos = pos;
    }

    public int get_seek_pos() {
	Plugin.trace("hashnode.seekpos");
	return seek_pos;
    }

    public byte[] get_httpsReply() {
	Plugin.trace("hashnode.get_httpsReply");
	return httpsReply;
    }

    public int increment_seek_pos(int index ) {
	Plugin.trace("hashnode.increment_seek_pos");
	this.seek_pos += index;
	return this.seek_pos;
    }
}





