/*
 * @(#)ActiveXAppletViewer.java	1.3 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.ocx;

/*
 * A recipient to show a Java Applet in.
 * This class implements all platform dependent code to show an Applet
 * running in the Activator for Internet Explorer 3 and 4.
 *
 * @version 	1.1 
 * @author	Jerome Dochez
 */

import sun.plugin.*;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.Hashtable;
import sun.plugin.usability.Trace;


public class ActiveXAppletViewer extends AppletViewer {

    /*
     * <p>
     * Creates a new AppletViewer to show an applet in the Activator
     * in Internet Explorer
     * </p>
     *
     * @param handle native reference to the ActiveX object
     */
    private ActiveXAppletViewer(Hashtable attr, int handle) {
	super(new ActiveXAppletContext(handle));
	this.handle = handle;

	// Store attribute
	Enumeration keyEnum = attr.keys();

	while (keyEnum.hasMoreElements())
	{
	    String key = (String) keyEnum.nextElement();
	    String value = (String) attr.get(key);

	    setParameter(key, value);
	}

	setScriptFlag();
    }

    /*
     * <p>
     * Creates a new AppletViewer to show an applet in the Activator
     * in Internet Explorer
     * </p>
     *
     * @param appletContext applet context to use for this applet panel
     * @param handle native reference to the ActiveX object
     */
    protected ActiveXAppletViewer(Hashtable attr, ActivatorAppletContext appletContext, int handle) {
	super(appletContext);
	this.handle = handle;

	// Store attribute
	Enumeration keyEnum = attr.keys();

	while (keyEnum.hasMoreElements())
	{
	    String key = (String) keyEnum.nextElement();
	    String value = (String) attr.get(key);

	    setParameter(key, value);
	}

	setScriptFlag();
    }

    /*
     * <p>
     * Notification of a status change, notify the bridge
     * </p>
     *
     * @param stringStatus the new status 
     */
    public void showAppletStatus(String stringStatus) {
	if (handle!=0) {
	    super.showAppletStatus(stringStatus);
	    notifyStatus(status);
	}
    }

    /*
     * <p>
     * Notification of a status change, notify the bridge
     * </p>
     *
     * @param stringStatus the new status label
     * @param arg
     */
    public void showAppletStatus(String stringStatus, Object arg) {
        super.showAppletStatus(stringStatus, arg);
	notifyStatus(status);
    }

    /*
     * <p>
     * Method to retrieve the parameter from the HTML tags
     * </p>
     * 
     * @param name the parameter name
     *
     * @return the parameter value, null if undefined
     */
    public String getParameterFromHTML(String name) {

	if (handle!=0) 
	    return nativeGetParameter(name.toLowerCase(), handle);	
	else return null;
    }

    /*
     * <p>
     * The JavaBeans component or applet is being closed
     * </p>
     *
     * @param timeOut max time to wait for the applet to die
     */
    public void onClose(final int timeOut) {
	handle=0;
	((ActiveXAppletContext) getAppletContext()).onClose();
	super.onClose(timeOut);
    }

    /* 
     * <p>
     * Initialize a new applet in this applet viewer
     * </p>
     */
    public void init() {
	initProperties();
	super.init();
	super.appletStart();
    }

    /* 
     * <p>
     * @return true if the bridge can safely bridge events to the 
     * ActiveX world
     * </p>
     */
    public boolean bridgeEvents() {
	String forwardEvent = getParameter("bridgeevents");
	if (forwardEvent==null)
	    return false;
	if (forwardEvent.equalsIgnoreCase("yes")) 
	    return true;
	else return false;
    }

    /*
     * <p>
     * Set the document url.
     * This must be done early, before initProperties is called.
     * Remove the document name to be consistent with NS plugins
     * </p>
     */
    public void setDocumentBase(String url) {
	int fromIndex=-1,lastIndex;

	int fragmentIndex = url.indexOf('#');
	int queryIndex = url.indexOf('?');

	if(queryIndex != -1 && fragmentIndex != -1)
	{
		fromIndex = Math.min(fragmentIndex, queryIndex);
	}
	else if(fragmentIndex != -1)
	{
		fromIndex = fragmentIndex;
	}
	else if(queryIndex != -1)
	{
		fromIndex = queryIndex;
	}

	if (fromIndex==-1) {
	    lastIndex=url.lastIndexOf('/');
	    if (lastIndex<url.lastIndexOf('\\'))
			lastIndex=url.lastIndexOf('\\');
	} else {
	    lastIndex=url.lastIndexOf('/', fromIndex);
	    if (lastIndex<url.lastIndexOf('\\', fromIndex))
			lastIndex = url.lastIndexOf('\\', fromIndex);
	}

	String toDecodeURL;
	if (lastIndex==-1)
	    toDecodeURL=url;
	else 
	    toDecodeURL= url.substring(0,lastIndex+1);

	String decodedURL;
	try {
	    decodedURL=URLDecoder.decode(toDecodeURL);
	} catch(Exception e) {
	    decodedURL=toDecodeURL;
	}

	java.net.URL docBase;
	try {
	    docBase = new java.net.URL(decodedURL);
	} catch (java.net.MalformedURLException e) {
	    Trace.printException(e);
	    return;
	}
	super.setDocumentBase(docBase);
    }

    /*
     * <p>
     * Force the codebase for Internet Explorer 3.0 mounted drive
     * applet display. In IE 3.0, when displaying html pages from
     * a mounted the drive, the document base will not be 
     *	    file://<drive>:\...
     * but 
     *	    file://\\<server>\<mount_name>\...
     * which causes the failure to load ressources or classes from
     * a jar file. We add the . codebase to force the correct setting 
     */
    protected void initProperties() {
	String att = getParameter("java_codebase");
	if (att==null)
	    att = getParameter("codebase");
	if (att == null) {
		if (getDocumentBase().toString().indexOf('\\')>0) 
			setParameter("codebase", ".");
	}
	super.initProperties();
	}

    private void notifyStatus(final int status) {
	/*if (status==APPLET_INIT) {
	    Runnable work = new Runnable() {
		public void run() {
		    statusNotification(status);
		}
	    };
	    Thread workerThread = new Thread(work);
	    workerThread.start();
	} else */
	statusNotification(status);
    }

    /*
     * <p>
     * Allow/diallow the document object model scripting
     * </p>
     */
    private void setScriptFlag() {
	String htmlFlag = getParameter("mayscript");
        if (htmlFlag!=null) {
	    if (htmlFlag.equalsIgnoreCase("true"))
		((ActiveXAppletContext) getAppletContext()).setMayScriptFlag(true);
	}
    }
	    
    /*
     * <p>
     * Notify the bridge of the applet status
     * </p>
     */
   private synchronized native void statusNotification(int status);

    /* 
     * <p>
     * Native method responsible for retrieving parameter values
     * from the HTML tags.
     * </p>
     *
     * @param name the parameter name
     * @param handle the native object reference
     * 
     * @return the parameter value, null if undefined
     */
    private synchronized native String nativeGetParameter(String name, int handle);

    /*
     * <p>
     * Reference to the native implementation of the Applet Viewer
     * </p>
     */
    private int handle;	
}


