/*
 * @(#)TypeLibManager.java	1.3 01/02/09
 *
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1
 *
 */

package sun.plugin.ocx;

/*
 * This class is responsible for managing the generated type libraries
 * for Applets and JavaBeans components running inside Microsoft Web
 * Browsers.
 *
 * @version 	1.1
 * @author	Jerome Dochez
 */

import java.util.Hashtable;
import java.util.Enumeration;
import java.net.URL;

import sun.plugin.AppletViewer;
import sun.beans.ole.StubInformation;

import java.security.AccessController;
import java.security.PrivilegedAction;

import sun.plugin.usability.Trace;


public class TypeLibManager {

    static {
	// Force initialization
	getDefaultTypeLibManager();
    }

    /* 
     * <p>
     * Return the default typelib manager for the running javaplugin
     * </p>
     *
     * @return the default TypeLibManager
     */
    public static TypeLibManager getDefaultTypeLibManager() {
	if (tlm==null) {
	    tlm= new TypeLibManager();
	}
	return tlm;
    }

    /*
     * <p>
     * private constructor, users should use the getDefaultTypeLibManager()
     * </p>
     */
    private TypeLibManager() {
    }

    /*
     * <p>
     * Set a new typelib previously generated for a particular applet. If another
     * instance of the same applet class runs, the same type lib will be used.
     * </p>
     * 
     * see #getTypeLib
     * @param codebase codebase of the applet
     * @param appletClass the applet class
     * @param the path for the generated type lib if it exists
     */
    private synchronized static void setTypeLib(
	URL codebase, 
	Class appletClass, 
	String typeLibPath)
    {
	if (typeLibs==null)
	    typeLibs = new Hashtable();
	typeLibs.put(codebase.toString() + appletClass.getName(), typeLibPath);
    }

    /*
     * <p> 
     * Create a new type library for a particular applet downloaded from 
     * a specific codebase.
     * <p>
     * 
     * @param appletClass class 
     * @param url codebase
     * @param path where to generate
     */
    private String getTypeLib(final URL codebase, final Class appletClass) {

	if (typeLibs==null)
	    typeLibs = new Hashtable();

	// Construct the key
	String key = codebase.toString() + appletClass.getName();
	String libPath = (String) typeLibs.get(key);
	if (null == libPath) {
	    libPath = (String) 
 		AccessController.doPrivileged(new PrivilegedAction() {
		    public Object run() {
			try {
			// get the temp directory
			//
			String tmpPath = System.getProperty("java.io.tmpdir");
			
			// load the DLL to create the typelib
			//
			System.loadLibrary("packager");
			
			StubInformation stub = StubInformation.getStub(appletClass,
								       tmpPath);
			if (stub.generateTypeLib(false)) 
			    return stub.getTypeLibName();
			else 
			    return null; 
			} catch(Throwable e) {
			    Trace.printException(e);
			    return null;
			}
		    }
		});
	    if (null != libPath) 
		setTypeLib(codebase, appletClass, libPath);	    
	}
	return libPath;
	
    }

    private static String getTypeLib(AppletViewer appletViewer) {
	Object applet = appletViewer.getViewedObject();
	if (null != applet) {

	    return getDefaultTypeLibManager().getTypeLib(
		          appletViewer.getCodeBase(), 
			  applet.getClass());
	} else
	    return null;

    }

    /* 
     * <p>
     * Vector of the generated typelibs, the key is the codebase and the 
     * qualified class name of the applet, the value is the path to the
     * generated typelib
     * </p>
     */
    private static Hashtable typeLibs;

    /*
     * <p>
     * Unique default instance of the TypeLibManager
     * </p>
     */
    private static TypeLibManager tlm;

}


