/*
 * @(#)ActivatorPanel.java	1.3 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 */

package sun.plugin.panel;

/**
 * This is the main Activator Panel panel. It contains the tabbed panes
 * the user can use to set its preferences for running the java plugin
 *
 * @version 	1.3
 * @author	Jerome Dochez
 */

import sun.plugin.*;
import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import sun.plugin.usability.Trace;


public class ActivatorPanel extends ActivatorSubPanel 
		    implements ActionListener, PropertyChangeListener {

    /**
     * Construct the new panel
     * Add all the widgets
     */
    ActivatorPanel(ConfigurationInfo model) {
	super(model);

	PlatformDependentInterface pdi = getPlatformObject();
	if (pdi!=null) 
	    pdi.onLoad(model);

	model.addPropertyChangeListener(this);

	JTabbedPane tabbedPanel = new JTabbedPane();       
	
	basicPanel = new BasicPanel(model);
	tabbedPanel.addTab(mh.getMessage("basic"), basicPanel);

        advancedPanel = new AdvancedPanel(model);
	tabbedPanel.addTab(mh.getMessage("advanced"), advancedPanel);

	proxyPanel = new ProxyPanel(model);
	tabbedPanel.addTab(mh.getMessage("proxies"), proxyPanel);

	cachePanel = new CachePanel(model);
	tabbedPanel.addTab(mh.getMessage("cache"), cachePanel);

	certPanel = new CertificatePanel(model);
	tabbedPanel.addTab(mh.getMessage("cert"), certPanel);

	defaultPanel = new DefaultPanel(model);
	tabbedPanel.addTab(mh.getMessage("about"), defaultPanel);

	setLayout(new BorderLayout());
	add("Center", tabbedPanel);

	JPanel buttons = new JPanel();
	apply = addButton(buttons, mh.getMessage("apply"));
	cancel = addButton(buttons, mh.getMessage("cancel"));
	apply.setEnabled(false);
	add("South", buttons);

    }

    /**
     * The model has changed, enable the apply button to persist
     * the new user settings
     */
    public void propertyChange(PropertyChangeEvent evt) {
	if (apply != null) {
	    apply.setEnabled(true);
	}
    }

    /** 
     * @return the PlatformDependentInterface object responsible 
     * for implementing all platform dependent code for the 
     * Activator Control Panel.
     * The name of the class implementing this platform specific
     * behaviour is sun.plugin.panel.<os.name>Platform
     * <p>
     * The platform dependent object will be called with the
     * onLoad, onSave method
     */
    private PlatformDependentInterface getPlatformObject() {

    	String osName = System.getProperty("os.name");
	try {
	    String n = osName;
	    if (osName.indexOf(' ')!=-1)
		n = osName.substring(0, osName.indexOf(' '));		
	    String platformClassName = "sun.plugin.panel." +  n + "Platform";
	    java.lang.Class platformClass = Class.forName(platformClassName);
	    if (platformClass != null) {
		Object pc = platformClass.newInstance();
		if (pc instanceof PlatformDependentInterface) {
		    PlatformDependentInterface pdi = (PlatformDependentInterface) pc;
		    return pdi;
		}
	    }
	} catch(Exception e) 
	{
	    Trace.printException(e);
	}
	return null;
    }

    /**
     * ActionListener implementation
     * If the user presses the Cancel button, the model is released from
     * its persisted state and the panels are reset.
     * If the user presses the Apply button, the model is persisted
     */
    public void actionPerformed(ActionEvent e) 
    {
	if (e.getSource()==cancel) {
	    try
	    {
		model.reset();
		PlatformDependentInterface pdi = getPlatformObject();
		if (pdi!=null) 
		    pdi.onLoad(model);
		basicPanel.reset();
		advancedPanel.reset();
		proxyPanel.reset();
		certPanel.reset();
		apply.setEnabled(false);
	    }
	    catch (Exception ex1)
	    {
		Trace.printException(ex1);
	    }
	}

	if (e.getSource()==apply) {
	    try {
	        model.applyChanges();
	        apply.setEnabled(false);
	        PlatformDependentInterface pdi = getPlatformObject();
	        if (pdi!=null) {
	   	    pdi.onSave(model);
	        }

	    } 
	    catch (Exception ex2) 
	    {
		Trace.printException(ex2, mh.getMessage("apply_failed") + ": " + ex2.getMessage(), 
				     mh.getMessage("apply_failed_title"));
	    }
	}
    }

    /**
     * @return the model for this application
     */
    public ConfigurationInfo getModel() {
	return model;
    }

    /**
     * Utility method to add buttons to a panel
     */
    private JButton addButton(JPanel panel, String label) {
	JButton button = new JButton(label);
	button.addActionListener(this);
	panel.add(button);
	return button;
    }

    private DefaultPanel defaultPanel;
    private BasicPanel basicPanel;
    private AdvancedPanel advancedPanel;
    private ProxyPanel proxyPanel;
    private CertificatePanel certPanel;
    private CachePanel	cachePanel;
    JButton cancel, apply;

    private MessageHandler mh = new MessageHandler("panel");
}


