/*
 * @(#)AdvancedPanel.java	1.4 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 */

package sun.plugin.panel;

/**
 * The advanced panel contains settings for the JDK to run the javaplugin with
 * as well as debug information
 *
 * @version 	1.3
 * @author	Jerome Dochez
 */

import javax.swing.*;
import javax.swing.text.*;
import java.awt.event.*;
import java.awt.*;
import java.text.MessageFormat;

public class AdvancedPanel extends ActivatorSubPanel 
			   implements   ActionListener,
                                        ItemListener, 
					KeyListener
{

    /**
     * Construct the panel
     * Add the widgets
     */
    AdvancedPanel(ConfigurationInfo model) {

	super(model);

	setBorder(BorderFactory.createEmptyBorder(4,4,4,4));
	JPanel jdkPanel = new JPanel();
	jdkPanel.setBorder(BorderFactory.createTitledBorder(mh.getMessage("jre_name")));
				
	JPanel jdkVersion = new JPanel();
	jdkVersion.setLayout(new BoxLayout(jdkVersion, BoxLayout.X_AXIS));
	jdkPanel.setLayout(new BoxLayout(jdkPanel, BoxLayout.X_AXIS));
	jdkVersion.setBorder(BorderFactory.createEmptyBorder(4,4,4,4));
	jdkPanel.add(jdkVersion);
	jdkInstalled = new JComboBox();
	jdkInstalled.setMaximumRowCount(3);

	if (setInstalledJavaList()) {
	    jdkInstalled.addItemListener(this);
	    jdkVersion.add(jdkInstalled);
	}
	

	jdkPanel.add(Box.createRigidArea(new java.awt.Dimension(1,5)));
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	add(Box.createRigidArea(new java.awt.Dimension(1,15)));
	add(jdkPanel);
	add(Box.createRigidArea(new java.awt.Dimension(1,10)));
	JPanel pathPanel = new JPanel();
	pathPanel.setLayout(new GridLayout(1, 0));
	pathPanel.setBorder(BorderFactory.createEmptyBorder(15,8,4,4));
	
	//for spacial layout purposes
//	JPanel blankPanel = new JPanel();
//	pathPanel.add(blankPanel);
//	pathPanel.add(blankPanel);
//	pathPanel.add(blankPanel);


	
	JPanel smallPanel = new JPanel();
	smallPanel.setLayout(new BoxLayout(smallPanel, BoxLayout.X_AXIS));
	pathLabel = new JLabel(mh.getMessage("path") + " ");
	smallPanel.add(pathLabel);
	jdkPath = new JTextField();
	jdkPath.addKeyListener(this);
	smallPanel.add(jdkPath);
	pathPanel.add(smallPanel);

	

	jdkPanel.setLayout(new BoxLayout(jdkPanel, BoxLayout.Y_AXIS));
	jdkPanel.add(pathPanel);

	JPanel jitPanel = new JPanel();
	jitPanel.setBorder(BorderFactory.createEmptyBorder(4,4,4,4));
	jitPanel.setLayout(new BoxLayout(jitPanel, BoxLayout.X_AXIS));
	jitButton = new JCheckBox(mh.getMessage("enable_jit"));
	jitButton.addActionListener(this);
	jitPanel.add(jitButton);
	/*jitPanel.add(Box.createRigidArea(new java.awt.Dimension(5,1)));
	jitPathLabel = new JLabel(mh.getMessage("jit_path"));
	jitPanel.add(jitPathLabel);
	jitPath = new JTextField(model.getJITPath(), 8);
	jitPath.setAlignmentX(LEFT_ALIGNMENT);
	jitPath.setMaximumSize(jitPath.getPreferredSize());
	jitPath.addKeyListener(this);
	jitPanel.add(jitPath);
	*/
	if (!hideJITBox) {
	    jdkPanel.add(Box.createRigidArea(new java.awt.Dimension(1,5)));
	    jdkPanel.add(jitPanel);
	}
    
	debugButton = new JCheckBox(mh.getMessage("enable_debug"));
	debugButton.addActionListener(this);

	JPanel debugPanel = new JPanel();
	debugPanel.setBorder(BorderFactory.createEmptyBorder(4,4,4,4));
	debugPanel.setBorder(BorderFactory.createTitledBorder(
					mh.getMessage("debug_settings")));
	debugPanel.add(debugButton);

	debugPortLabel = new JLabel(mh.getMessage("debug_port"));
	debugPanel.add(debugPortLabel);
	debugPanel.add(Box.createRigidArea(new java.awt.Dimension(5,1)));
	debugPort = new JTextField("2502", 6);
	debugPort.setAlignmentX(LEFT_ALIGNMENT);
	debugPort.setMaximumSize(debugPort.getPreferredSize());
	debugPort.addKeyListener(this);
	debugPanel.add(debugPort);
	if (!hideDebugBox) {
	    add(Box.createRigidArea(new java.awt.Dimension(1,5)));
	    add(debugPanel);
	}

	reset();
    }

    /**
     * Initialize the list of JDK/JRE available for a particular JDK version
     * 
     * @param version the JDK version the user wants to use
     */
    private boolean setInstalledJavaList() {

	// Clean existing keys
	if (jdkInstalled.getItemCount()>0)
	    jdkInstalled.removeAllItems();

        jdkInstalled.addItem(defaultJRE);
	String[][] jreItems = model.getInstalledJREList();
	if (jreItems == null) {
	    jreNb = 0;
	}
	else {
	    jreNb = jreItems.length;
	}
	for (int i=0;i<jreNb;i++) {
	    String version = jreItems[i][0];
	    String path = jreItems[i][1];
	    Object []values = { version, path };
	    jdkInstalled.addItem(MessageFormat.format(mh.getMessage("jre_format"),
						      values));
	}

	String[][] jdkItems = model.getInstalledJDKList();
	if (jdkItems == null) {
	    jdkNb = 0;
	} else {
	    jdkNb = jdkItems.length;
	}
	for (int i=0;i<jdkNb;i++) {
	    String version = jdkItems[i][0];
	    String path = jdkItems[i][1];
	    Object []values = { version, path };
	    jdkInstalled.addItem(MessageFormat.format(mh.getMessage("jdk_format"),
				 values));
	}
	jdkInstalled.addItem(otherJDK);

	return true;
    }

    /**
     * ActionListener interface implementation. 
     * Responds to user selection/deselection of the debug checkbox
     * 
     * @param actionEvent
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getSource()==debugButton) {
	    debugPort.setEnabled(debugButton.isSelected());
	    debugPortLabel.setEnabled(debugButton.isSelected());
	    debugPort.repaint();
	    debugPortLabel.repaint();
	    model.setDebugEnabled(debugButton.isSelected());
	}
	else if (e.getSource()==jitButton) {
	    boolean jitEnabled = jitButton.isSelected();
	    //jitPathLabel.setEnabled(jitEnabled);
	    //jitPath.setEnabled(jitEnabled);
	    //jitPath.repaint();
	    //jitPathLabel.repaint();
	    model.setJITEnabled(jitEnabled);
	}
    } 

    /**
     * ItemListener interface implementation. 
     * Responds to user changes to the jdk selection
     * 
     * @param actionEvent
     */
    public void itemStateChanged(ItemEvent e) {

	if (e.getSource()==jdkInstalled) {
	    int index = jdkInstalled.getSelectedIndex();
	    if (index==0) {
		enableDebug(false);
		model.setJavaRuntimeType("Default");
		model.setJavaRuntimeVersion(null);
	    }
	    if (index>0 && index<=jreNb) {
		enableDebug(false);
		model.setJavaRuntimeType("JRE");
		String[][] jres = model.getInstalledJREList();	       
		model.setJavaRuntimeVersion(jres[index-1][0]);
	    }
	    if (index>jreNb && index<=(jreNb+jdkNb)) {
		enableDebug(true);
		model.setJavaRuntimeType("JDK");
		String[][] jdks = model.getInstalledJDKList();	       
		model.setJavaRuntimeVersion(jdks[index-jreNb-1][0]);
	    }
	    if (index==(jdkInstalled.getItemCount()-1)) {
		enableDebug(true);
		model.setJavaRuntimeType(jdkPath.getText());
		model.setJavaRuntimeVersion(null);
	    }
	    reset();
	}
    }

    /**
     * Key Listener implementation
     * Sets the custom JDK settings
     */
    public void keyTyped(KeyEvent e) {
	if (e.getSource() == debugPort) {
	    model.setDebugPort(debugPort.getText());
	} else if (e.getSource() == jdkPath) {
	    model.setJavaRuntimeType(jdkPath.getText());
	}/* else if (e.getSource() == jitPath) {
	    if (jitPath.getText().length()!=0)
		model.setJITPath(jitPath.getText());
	    else 
		model.setJITPath(null);
	}*/
    }

    public void keyReleased(KeyEvent e) {
	// This shouldn't be necessary.  Unfortunately it seems to be.
	//					KGH 2/9/98
	keyTyped(e);
    }

    public void keyPressed(KeyEvent e) {}

    /**
     * Reset all the settings from the model
     */
    public void reset() {

	boolean jitEnabled = model.isJITEnabled();
	jitButton.setSelected(jitEnabled);
	//jitPath.setText(model.getJITPath());
	//jitPathLabel.setEnabled(jitEnabled);
	//jitPath.setEnabled(jitEnabled);

	boolean debugEnabled = model.isDebugEnabled();
	debugButton.setSelected(debugEnabled);
	debugPort.setText(model.getDebugPort());
	debugPort.setEnabled(debugEnabled);
	debugPortLabel.setEnabled(debugEnabled);

	boolean other = true;
	String jdk = model.getJavaRuntimeType();
	pathLabel.setEnabled(false);
	jdkPath.setEnabled(false);
	try {
	    if (jdk.equalsIgnoreCase("Default")) {
		enableDebug(false);
		jdkInstalled.setSelectedIndex(0);
	    } else {
		if (jdk.equalsIgnoreCase("JDK")) {
		    enableDebug(true);
		    String[][] jdks = model.getInstalledJDKList();
		    String version = model.getJavaRuntimeVersion();
		    int jdkIndex = 0;
		    while (!jdks[jdkIndex][0].equalsIgnoreCase(version))
			jdkIndex++;
		    jdkInstalled.setSelectedIndex(jreNb+jdkIndex+1);
		} else { 
		    if (jdk.equalsIgnoreCase("JRE")) {
			enableDebug(false);
			String[][] jres = model.getInstalledJREList();
			String version = model.getJavaRuntimeVersion();
			int jreIndex = 0;
			while (!jres[jreIndex][0].equalsIgnoreCase(version))
			    jreIndex++;
			jdkInstalled.setSelectedIndex(jreIndex+1);
		    } else {
			jdkInstalled.setSelectedIndex(jdkInstalled.getItemCount()-1);
			jdkPath.setText(jdk);
			jdkPath.setEnabled(true);
			pathLabel.setEnabled(true);
			enableDebug(true);
		    }
		}
	    } 
	} catch(Exception e) {
	    // This is nasty but any kind of exception should trigger the
	    // reset of the property.
	    jdkInstalled.setSelectedIndex(0);
	    model.firePropertyChange();
	} 
	
    }

    public static void setHideJITBox(boolean x) {
	hideJITBox = x;
    }

    public static void setHideDebugBox(boolean x) {
	hideDebugBox = x;
    }
    
    /*
     * <p>
     * enable or disable the Debug fields depending if the JRE/JDK
     * selected is supporting the debug mode
     * </P>
     * @param b true to enable the user to enter debugging information
     */
    private void enableDebug(boolean b) {
	debugButton.setEnabled(b);
	debugPort.setEnabled(b);

	if (!b) {
	    model.setDebugEnabled(false);
	    debugButton.setSelected(false);
	}

    }

    private static boolean hideJITBox;
    private static boolean hideDebugBox;

    private MessageHandler mh = new MessageHandler("advanced");
    private JCheckBox debugButton, jitButton;
    private JTextField debugPort;
    private JLabel debugPortLabel;//, jitPathLabel;
    private JTextField jdkPath;//, jitPath;
    private JComboBox jdkInstalled;
    private final String otherJDK = mh.getMessage("other_jdk");
    private final String defaultJRE = mh.getMessage("default_jdk");
    private JLabel pathLabel;
    private int    jreNb, jdkNb;
}

