/*
 * @(#)BasicPanel.java	1.3 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3
 *
 */

package sun.plugin.panel;

/**
 * The Basic Panel shows the basic configuration information for the javaplugin
 * like console state, security attributes...
 *
 * @version 	1.3
 * @author	Jerome Dochez
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

public class BasicPanel extends ActivatorSubPanel 
			implements ActionListener, KeyListener {

    /**
     * Construct the panel, add widgets
     */ 
    BasicPanel(ConfigurationInfo model) {
	super(model);

	setBorder(BorderFactory.createEmptyBorder(4,30,4,30));	

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	// Add vertical pad
	add(Box.createGlue());
	Dimension vpad5 = new Dimension(1,5);
	add(Box.createRigidArea(vpad5));

	enableJava = new JCheckBox(mh.getMessage("enable_java"));
	enableJava.addActionListener(this);
	add(enableJava);
    
	// Add vertical pad
	add(Box.createRigidArea(vpad5));

	showConsole = new JCheckBox(mh.getMessage("show_console"));
	showConsole.addActionListener(this);
	add(showConsole);

	// Add vertical pad
	add(Box.createRigidArea(vpad5));

	cacheJars = new JCheckBox(mh.getMessage("recycle_classloader"));
	cacheJars.addActionListener(this);
	add(cacheJars);

	// Add vertical pad
	add(Box.createRigidArea(vpad5));

	showException = new JCheckBox(mh.getMessage("show_exception"));
	showException.addActionListener(this);
	add(showException);


	// Add vertical pad
	add(Box.createRigidArea(new Dimension(1,10)));

	// Create the combobox panel
	add(Box.createGlue());
	JPanel options = new JPanel();
	GridLayout gridLayout = new GridLayout(2,1);
	options.setLayout(gridLayout);

	options.add(new JLabel(mh.getMessage("java_parms"),
			       JLabel.CENTER));
	javaParms = new JTextField(model.getJavaParms());
	options.add(javaParms);
	javaParms.addKeyListener(this);

	add(options);
	// Add vertical pad
	add(Box.createRigidArea(new Dimension(1,10)));
	add(Box.createGlue());
	add(Box.createGlue());
	reset();
    }

    /**
     * Add a Combo box in the specified value, the items to be added 
     * to the ComboBox are also specified
     * 
     * @param panel where to place the ComboBox
     * @param title for the ComboBox
     * @param items items to add to the ComboBox
     * @param initialIndex 
     */
    private JComboBox addComboBoxFor(JPanel panel, JLabel title, String[] items, int initialIndex) {

	panel.add(title);
	JComboBox combo = new JComboBox();

	for (int i=0;i<items.length;i++) 
	    combo.addItem(items[i]);

	combo.setSelectedIndex(initialIndex);
	combo.addActionListener(this);
	panel.add(combo);
	return combo;
    }
    

    /**
     * ActionListener interface implementation. All fields in this 
     * panel will generate this message when changed. We use the 
     * internal UI state to save the user choices so we just reset
     * the hasChanged field of our model
     * 
     * @param ActionEvent info about the event
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getSource()==enableJava) 
	    model.setJavaEnabled(enableJava.isSelected());
	if (e.getSource()==showConsole) 
	    model.setConsoleEnabled(showConsole.isSelected());
	if (e.getSource()==cacheJars) 
	    model.setJarCacheEnabled(cacheJars.isSelected());
	if (e.getSource()==showException) 
	    model.setShowException(showException.isSelected());
    }

    /**
     * Key Listener implementation
     * Sets the custom JDK settings
     */
    public void keyTyped(KeyEvent e) {
	if (e.getSource() == javaParms) {
	    model.setJavaParms(javaParms.getText());
	}
    }

    public void keyReleased(KeyEvent e) {
	// This shouldn't be necessary.  Unfortunately it seems to be.
	//					KGH 2/9/98
	keyTyped(e);
    }

    public void keyPressed(KeyEvent e) {}

    /**
     * Reset the UI state from the Model
     */
    public void reset() {
	showConsole.setSelected(model.isConsoleEnabled());
	enableJava.setSelected(model.isJavaEnabled());
	cacheJars.setSelected(model.isJarCacheEnabled());
	showException.setSelected(model.isShowExceptionEnabled());
    }

    private JCheckBox enableJava, showConsole, cacheJars, showException;
    private JTextField javaParms;
    private MessageHandler mh = new MessageHandler("basic");
}


