/*
 * @(#)CachePanel.java	1.00
 *
 *  Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.panel;

/**
 * Panel to display cache options
 *
 * @version 	1.0 
 * @author	Devananda Jayaraman
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Dimension;
import java.io.File;
import sun.plugin.usability.*;

public class CachePanel extends ActivatorSubPanel 
			implements  ActionListener{

    /**
     * Contruct a new proxy panel
     */
    public CachePanel(ConfigurationInfo model) {
	super(model);
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	add(Box.createRigidArea(new java.awt.Dimension(1,5)));

	//HTML Cache entry
	JPanel htmlCachePanel = new JPanel();
	htmlCachePanel.setLayout(new BoxLayout(htmlCachePanel, BoxLayout.Y_AXIS));
	htmlCachePanel.setBorder(BorderFactory.createTitledBorder(mh.getMessage("html_cache")));
	JPanel buttonPanel1 = new JPanel();
	clearHTMLCache = new JButton(mh.getMessage("clear_html_cache"));
	clearHTMLCache.setAlignmentX(LEFT_ALIGNMENT);
	clearHTMLCache.addActionListener(this);
	buttonPanel1.add(clearHTMLCache);
	htmlCachePanel.add(buttonPanel1);

	//JAR Cache entry
	JPanel jarCachePanel = new JPanel();
	jarCachePanel.setLayout(new BoxLayout(jarCachePanel, BoxLayout.Y_AXIS));
	jarCachePanel.setBorder(BorderFactory.createTitledBorder(mh.getMessage("jar_cache")));
	JPanel buttonPanel2 = new JPanel();
	clearJARCache = new JButton(mh.getMessage("clear_jar_cache"));
	clearJARCache.setAlignmentX(LEFT_ALIGNMENT);
	clearJARCache.addActionListener(this);
	buttonPanel2.add(clearJARCache);
	jarCachePanel.add(buttonPanel2);

	add(jarCachePanel);
	add(Box.createRigidArea(new java.awt.Dimension(10,10)));
	//add(htmlCachePanel);
	//add(Box.createRigidArea(new java.awt.Dimension(70,70)));
    }


    /**
     * ActionListener interface implementation. 
     * Responds to user selection/deselection of the debug checkbox
     * 
     * @param actionEvent
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getSource()==clearJARCache) 
	{
        String jarCachePath = cacheDir + seperator + "jar";
	    
	    deleteFiles(new File(jarCachePath));

	    String message = mh.getMessage("clear_jar_cache_from");
	    String caption = mh.getMessage("caption");

		DialogFactory.showMessageDialog(message + jarCachePath, caption, false);
	}
	if (e.getSource()==clearHTMLCache) 
	{
	    String HTMLCachePath =  cacheDir + seperator + "html";
	    
		deleteFiles(new File(HTMLCachePath));

	    sun.plugin.cachescheme.BookmarkManager.removeAllBookmark();

	    String message = mh.getMessage("clear_html_cache_from");
	    String caption = mh.getMessage("caption");

	    DialogFactory.showMessageDialog(message + HTMLCachePath, caption, false );
	}
    }
    
    private void deleteFiles(File inPath){
	if(inPath.isDirectory())
	{
	    File[] list = inPath.listFiles();
	    for(int i=0; i < list.length; i++)
	    {
		deleteFiles(list[i]);
	    }
	}

	inPath.delete();
    }

    private MessageHandler mh = new MessageHandler("cache");
    JButton clearJARCache = null;
    JButton clearHTMLCache = null;
    String cacheDir = sun.plugin.cachescheme.PluginCacheTable.cacheHome;
    String pluginVersion = System.getProperty("javaplugin.version");
    String seperator = System.getProperty("file.separator");
}
