/*
 * @(#)CertificatePanel.java	1.6 00/04/13
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.panel;

/**
 * Panel to display information about the certificates to be used by 
 * the javaplugin
 *
 * @version 	1.0 
 * @author	Stanley Man-Kit Ho
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Dimension;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;



public class CertificatePanel extends ActivatorSubPanel 
			      implements ActionListener {

    /**
     * Contruct a new certificate panel
     */
    public CertificatePanel(ConfigurationInfo model) {
	super(model);

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	add(Box.createRigidArea(new java.awt.Dimension(1,5)));

	JPanel defaultPanel = new JPanel();
	defaultPanel.setLayout(new BoxLayout(defaultPanel, BoxLayout.X_AXIS));
	add(defaultPanel);

	add(Box.createRigidArea(new java.awt.Dimension(1,5)));

	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	panel.setBorder(BorderFactory.createTitledBorder(mh.getMessage("netscape_object_signing")));

	JPanel certPanel = new JPanel();
	certPanel.setLayout(new BorderLayout());

	certText = new JLabel(mh.getMessage("description"));

	JPanel textPanel = new JPanel();
	textPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
	textPanel.add(certText);
	certPanel.add(textPanel, BorderLayout.NORTH);
	
	certList = new JList();
	certList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	certPanel.add(certList, BorderLayout.CENTER);

	removeButton = new JButton(mh.getMessage("remove_button"));
	removeButton.addActionListener(this);

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
	buttonPanel.add(removeButton);
	certPanel.add(buttonPanel, BorderLayout.SOUTH);

	panel.add(certPanel);
	panel.add(Box.createRigidArea(new java.awt.Dimension(1,5)));
	reset();
	add(panel);

    }

    /**
     * Enable / Disable the UI depending on the number of outstanding 
     * certificates. Changed certificate from active set into inactive
     * set if "Remove" button is clicked.
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getSource()==removeButton) {
	    int i = certList.getSelectedIndex();
	    
	    if (i < 0)
		return;
	    		
	    // Changed the certificate from the active set into the 
	    // inactive set. This is for removing the certificate from
	    // the store when the user clicks on Apply.
	    //
	    String alias = (String) certList.getSelectedValue();
    	    model.deactivateCertificate(alias);

	    reset();
	}
    }

    /**
     * Refill the list box with the active set of certificates. Disable
     * UI if necessary.
     */
    public void reset() {

	Collection certs = model.getCertificates();

	if (certs == null || certs.size() == 0)
	    certList.setListData(new String[0]);
	else
	{
	    // Fill up list box with the certificates
	    //	     
	    certList.setListData(certs.toArray());
	}

	boolean enable = (certs != null && certs.size() > 0);

	setEnabled(certText, enable);
	setEnabled(certList, enable);
	setEnabled(removeButton, enable);

	if (enable)
	    certList.setSelectedIndex(0);
    }


    /**
     * Enable/Disabled a component
     *
     * @param field the component to enable/disable
     * @param b true to enable to text field, false to disable it
     */
    private void setEnabled(JComponent field, boolean b) {
	field.setEnabled(b);
	field.repaint();
    }

    private JLabel certText;
    private JList certList;
    private JButton removeButton;

    private MessageHandler mh = new MessageHandler("cert");
}

