/*
 * @(#)ConfigurationInfo.java	1.3 01/02/09
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.plugin.panel;

 /**
  * Model class for all configuration information pertinent to the Java
  * Activator. 
  *
  * Note that we want the contents of the poeprties file to be locale
  * independent.  We therefore do *NOT* internationalize the tag strings
  * or the other contents of the property file.
  *
  * End users should never be editing the properties file directly.  They
  * should be using the (carefully internatiuonalzied) ControlPanel.
  *
  * @author Jerome Dochez
  * @version 1.3, 02/09/01
  */

import java.beans.*;
import java.util.*;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.security.cert.Certificate;
import sun.applet.*;
import sun.plugin.AppletMessageHandler;
import sun.plugin.security.CertificateStore;
import sun.plugin.security.PluginCertificateStore;
import sun.plugin.usability.Trace;

public class ConfigurationInfo {

    /**
     * Contruct a new configuration information
     * 
     */
    public ConfigurationInfo() {
	if (theUserPropertiesFile == null) {
	    String sep = File.separator;
	    theUserPropertiesFile = new File(System.getProperty("user.home") +
					 sep + ".java" +
					 sep + "properties" + 
					 System.getProperty("javaplugin.nodotversion"));
	    new File(theUserPropertiesFile.getParent()).mkdirs();
	}
        reset();
    }

    /**
     * enable/disable the Java Activator
     */
    public void setJavaEnabled(boolean value) {
	javapluginEnabled = value;
	firePropertyChange();
    }

    /** 
     * @return true if the java javaplugin is enabled
     */
    public boolean isJavaEnabled() {
	return javapluginEnabled;
    }

    /**
     * enable/disable the Java Activator Console Window
     * 
     * @param true to enable to console
     */
    public void setConsoleEnabled(boolean value) {
	showConsole = value;
	firePropertyChange();
    }

    /**
     * @return true if the console is enabled
     */
    public boolean isConsoleEnabled() {
	return showConsole;
    }

    /**
     * enable/disable showing exception dialogboxes
     * 
     * @param true to enable
     */
    public void setShowException(boolean value) {
	showException = value;
	firePropertyChange();
    }

    /**
     * @return true if the console is enabled
     */
    public boolean isShowExceptionEnabled() {
	return showException;
    }

    /**
     * enable/disable the caching of JAR files in memory.
     * 
     * @param true to enable JAR file caching
     */
    public void setJarCacheEnabled(boolean value) 
    {
	classLoaderCacheEnabled = value;

	firePropertyChange();
    }

    /**
     * @return true if JAR file caching is enabled
     */
    public boolean isJarCacheEnabled() 
    {
	return classLoaderCacheEnabled;
    }

    /**
     * @return true if the JIT is enabled
     */
    public boolean isJITEnabled() {
	return useJIT;
    }

    /**
     * Enable or disable the usage of the JIT 
     */
    public void setJITEnabled(boolean b) {
	useJIT = b;
	firePropertyChange();
    }

    /**
     * @return the path for the JIT
     */
    public String getJITPath() {
	return JITPath;
    }

    /**
     * Set the path for the JIT
     */
    public void setJITPath(String path) {
	JITPath = path;
	firePropertyChange();
    }

    /**
     * Set the debugging mode to true or false
     * 
     * @param true to run the javaplugin in debug mode
     */
    public void setDebugEnabled(boolean mode) {
	debugMode = mode;
	firePropertyChange();
    }

    /**
     * @return true if the Activator is to be run in debugging mode
     */
    public boolean isDebugEnabled() {
	return debugMode;
    }

    /**
     * Set the debugging port if the debug mode is enabled
     *
     * @param debug port
     */
    public void setDebugPort(String debugPort) {
	this.debugPort = debugPort;
	firePropertyChange();
    }

    /**
     * @return the debug port for remote debugging
     */
    public String getDebugPort() {
	return debugPort;
    }
    
    /**
     * Set the list of installed JRE on the local machine 
     * The vector contains <jre version, jre path>*nb of installed jres
     */
    public void setInstalledJREList(Vector vector) {
	installedJREs = new String[vector.size()/2][2];
	int i=0;
	for (Enumeration e = vector.elements() ; e.hasMoreElements(); i++) {
	    installedJREs[i][0] = (String) e.nextElement();
	    installedJREs[i][1] = (String) e.nextElement();
	}
    }

    /**
     * @return the list of installed jre
     * the returned arrray is a two dimension array.
     * First arrray dimension is the number of JREs installed in the machine
     * the second dimension is always of size 2. The first element is the 
     * jre version, the second dimension of the jre path
     */
    public String[][] getInstalledJREList() {
	return installedJREs;
    }

    /**
     * Set the list of installed JDK on the local machine 
     * The vector contains <jdk version, jdk path>*nb of installed jres
     */
    public void setInstalledJDKList(Vector vector) {
	installedJDKs = new String[vector.size()/2][2];
	int i=0;
	for (Enumeration e = vector.elements() ; e.hasMoreElements(); i++) {
	    installedJDKs[i][0] = (String) e.nextElement();
	    installedJDKs[i][1] = (String) e.nextElement();
	}
    }

   /**
     * @return the list of installed jdk
     * the returned arrray is a two dimension array.
     * First arrray dimension is the number of JDKs installed in the machine
     * the second dimension is always of size 2. The first element is the 
     * jdk version, the second dimension of the jdk path
     */
    public String[][] getInstalledJDKList() {
	return installedJDKs;
    }

    public void setJavaRuntimeVersion(String version) {
	if (this.javaVersion == version) 
	    return;
	this.javaVersion = version;
	firePropertyChange();
    }

    public String getJavaRuntimeVersion() {
	return javaVersion;
    }

    public void setJavaRuntimeType(String type) 
    {
	if (type != null)
	{
	    if (type.equalsIgnoreCase("Default"))
	    {
		javaRuntimeType = "Default";
		javaRuntimePath = null;
	    }
	    else if (type.equalsIgnoreCase("JDK"))
	    {
		javaRuntimeType = "JDK";
		javaRuntimePath = null;
	    }
	    else if (type.equalsIgnoreCase("JRE"))
	    {
		javaRuntimeType = "JRE";
		javaRuntimePath = null;
	    }
	    else
	    {
		javaRuntimeType = "Other";
		javaRuntimePath = type;
	    }

	    firePropertyChange();
	}
    }
    
    public String getJavaRuntimeType() 
    {
	if (javaRuntimeType != null && javaRuntimeType.equalsIgnoreCase("Other"))
	    return javaRuntimePath;
	else
	    return javaRuntimeType;
    }

    /**
     * @return true if the javaplugin should use the browser settings
     */
    public boolean useDefaultProxy() {
	return useDefaultProxy;
    }

    /**
     * Sets the flag to use the default proxy settings of the browser
     *
     * @param b true for using the browser's settings
     */
    public void setDefaultProxySettings(boolean b) {
	useDefaultProxy=b;
	firePropertyChange();
    }

    /**
     * Set to true to use the same proxy for all protocols
     */
    public void setUseSameProxy(boolean b) {
	useSameProxy = b;
	firePropertyChange();
    }

    /**
     * @return true if the user want to use the same proxy for all protocols
     */
    public boolean useSameProxy() {
	return useSameProxy;
    }

    /** 
     * Sets the proxy settings string
     * 
     * @param str encoded strings for proxy settings
     */
    public void setProxySettingsString(String str) {
	proxySettings = str;
	firePropertyChange();
    }

    /**
     * @return the encoded proxy settings string
     */
    public String getProxySettingsString() {
	return proxySettings;
    }

    /**
     * Set the additional java parameters to be set when invoking the Java
     * Virtual Machine
     *
     * @parm str the parameters
     */
    public void setJavaParms(String str) {
	javaParms = str;
	firePropertyChange();
    }

    /**
     @return the additional parameters for the Java Virtual machine
     */
    public String getJavaParms() {
	return javaParms;
    }

    /**
     * Two set of certificate vectors are used to support the 
     * "Certificate Remove" option for removing Netscape Object 
     * Signing certificate from the store.
     * 
     * When the program starts, all certificates are in the "Active"
     * set of certificate vector. As users remove certificate from
     * the control panel, these certificates will be moved to the
     * "Inactive" set of certificate vector. When users click the
     * "Apply" button, the certificates in the "Inactive" set will 
     * be removed from the store. When users click the "Reset"
     * button, the certificates in the "Inactive" set will be
     * moved back to the "Active" set.
     *
     * Certificates are only loaded and saved when OnLoad() and
     * OnSave() are called.
     */

    /**
     * Return a collection which represents the list of alias 
     * of all the active certificate in the certificate store.
     *
     * @return Collection of active certificate alias
     */
    public Collection getCertificates() {

	return activeCertsMap.keySet();
    }


    /**
     * Change a certificate from the active set to the inactive 
     * set of certificates for Netscape Object Signing. 
     *
     * @param index Index of certificate in active set
     */
    public void deactivateCertificate(String alias)  {

	Object cert = activeCertsMap.remove(alias);
	inactiveCertsMap.put(alias, cert);
	firePropertyChange();
    }


    /**
     * Extrace CN from DN in the certificate.
     *
     * @param cert X509 certificate
     * @return CN
     */
    private String extractAliasName(X509Certificate cert)
    {
	String name = "Unknown subject";

	// Extract CN from the DN for each certificate
	try 
	{
       	    Principal principal = cert.getSubjectDN();

	    // Extract subject name
	    String subjectDNName = principal.getName();

	    int x = subjectDNName.indexOf("CN=");
	    int y = 0;

	    if (x < 0)
		name = "Unknown subject";
	    else
	    {
		x = x + 3;

		// Search for quote
		if (subjectDNName.charAt(x) == '\"')
		{
		    // if quote is found, search another quote

		    // skip the first quote
		    x = x + 1;
		    
		    y = subjectDNName.indexOf('\"', x);
		}
		else
		{
		    // quote is not found, search for comma
		    y = subjectDNName.indexOf(',', x);
		}

		if (y < 0)
		    name = subjectDNName.substring(x);			
		else
		    name = subjectDNName.substring(x, y);			
	    }
	}
	catch (Exception e) 
	{
	    Trace.printException(e);
	}

	return name;
    }

    /**
     * Apply all the changes to the storage medium
     */
    public void applyChanges() throws java.io.IOException {
	// First, update the poperties we read earlier from disk.

	// Activator Console
	props.put("javaplugin.console", (showConsole?"true":"false"));

	// show Exception dialogboxes
	props.put("javaplugin.exception", (showException?"true":"false"));

	// JAR file cache.
	props.put("javaplugin.classloader.cache.enabled", (classLoaderCacheEnabled?"true":"false"));

	// Activator Enabled
	props.put("javaplugin.enabled", (javapluginEnabled?"true":"false"));

	// JIT Enabled
	if (useJIT) 
	    props.put("javaplugin.jit.enabled", "true");
	else
	    props.put("javaplugin.jit.enabled", "false");


	// Debug Mode and Port
	if (debugMode) {
	    props.put("javaplugin.debug.mode", "true");
	    props.put("javaplugin.debug.port", debugPort);
	} else
	    props.put("javaplugin.debug.mode", "false");

	// Proxy settings
	if (useDefaultProxy) {
	    props.put("javaplugin.proxy.usebrowsersettings", "true");
	} else {
	    props.put("javaplugin.proxy.usebrowsersettings", "false");
	}
	if (proxySettings!=null) 
    	    props.put("javaplugin.proxy.settings", proxySettings);
	 else
	    props.remove("javaplugin.proxy.settings");

	if (useSameProxy) 
	    props.put("javaplugin.proxy.forallprotocols", "true");
	else
	    props.put("javaplugin.proxy.forallprotocols", "false");


	//
	// Store the JRE type into "javaplugin.jre.type".
	//
	// There are four possible options:
	// a) "Default"
	// b) "JDK"
	// c) "JRE"
	// d) "Other"
	//
	if (javaRuntimeType != null) {
	    props.put("javaplugin.jre.type", javaRuntimeType);
	} else {
	    javaRuntimeType = "Default";
	    props.put("javaplugin.jre.type", "Default");
	}


	//////////////////////////////////////////////////////////
	//
	// Store the JRE path into "javaplugin.jre.path". It always
	// contains the actual path to the JRE, or "Default".
	//
	String path = null;

	// If Java version is set, it must be either JDK/JRE
	if (javaVersion != null && javaVersion.length() > 0) 
	{
	    String[][] list = null;
	    if (javaRuntimeType.equalsIgnoreCase("JRE")) 
	    {
		list = getInstalledJREList();
	    }
	    if (javaRuntimeType.equalsIgnoreCase("JDK")) 
	    {
		list = getInstalledJDKList();
	    }
	    if (list != null) {
		for (int i=0; i<list.length; i++) {
		    if (list[i][0].equalsIgnoreCase(javaVersion)) 
			path = list[i][1];
		}	    
	    }
	}

	// If the Java Runtime type is "Other", we should use the path.
	//
	if (javaRuntimeType.equalsIgnoreCase("Other")) 
	{
	    // Make sure the path is not empty
	    if (javaRuntimePath != null && javaRuntimePath.trim().equals("") == false)
	    {
		path = javaRuntimePath;
	    }
	}

	// If JRE path not found or malformed, fallback to "default"
	if (path != null) 
	{
	    props.put("javaplugin.jre.path", path);
	}
	else 
	{
	    props.put("javaplugin.jre.type", "Default");
	    props.put("javaplugin.jre.path", "Default");
	}

	///////////////////////////////////////////////////////


	if (javaVersion != null && javaVersion.length() > 0) {
	    props.put("javaplugin.jre.version", javaVersion);
	} else {
	    props.remove("javaplugin.jre.version");
	}

	if (javaParms != null && javaParms.length() > 0) {
	    props.put("javaplugin.jre.params", javaParms);
	} else {
	    props.remove("javaplugin.jre.params");
	}

	// Save properties
        FileOutputStream fos = new FileOutputStream(theUserPropertiesFile);
        BufferedOutputStream bos = new BufferedOutputStream(fos);
	PrintWriter pw = new PrintWriter(bos);
	pw.println(mh.getMessage("property_file_header"));
	pw.flush();
	props.store(bos, "");
	bos.close();
	fos.close();

	////////////////////////////////////////////////////////////////////
	// Apply change to certificate store
	//
	try
	{
	    CertificateStore certStore = new PluginCertificateStore();
	    
	    // Load certificates from store
	    certStore.load();

	    // Remove inactive certificate one-by-one from store
	    Collection certs = inactiveCertsMap.values();
	    Iterator iter = certs.iterator();

	    while (iter.hasNext())
	    {
		certStore.remove((Certificate) iter.next());
	    }
	    
	    // Save the changes
	    certStore.save();
	}
	catch (Exception e)
	{
	    Trace.printException(e);
	}

	// Clear inactive certificate lists
	inactiveCertsMap.clear();
    }

    /**
     * Read all the information from the configuration storage
     */
    public void reset() {
	firePropertyChange();
        props = new Properties();
	try {
	    FileInputStream fis = new FileInputStream(theUserPropertiesFile);
	    BufferedInputStream bis = new BufferedInputStream(fis);

	    props.load(bis);

	    bis.close();
	    fis.close();
	} catch (Exception e) {
	    // Its OK if the file is missing,
	}

	String propValue;
	propValue = props.getProperty("javaplugin.console");
	if (propValue != null && propValue.equals("true")) {
	    showConsole = true;
	} else {
	    showConsole = false;
	}

	propValue = props.getProperty("javaplugin.exception");
	if (propValue == null || propValue.equals("false")) {
	    showException = false;
	} else {
	    showException = true;
	}

	propValue = props.getProperty("javaplugin.classloader.cache.enabled");
	if (propValue == null || propValue.equals("true")) {
	    classLoaderCacheEnabled = true;
	} else {
	    classLoaderCacheEnabled = false;
	}

	propValue = props.getProperty("javaplugin.enabled");
	if (propValue == null || propValue.equals("true")) {
	    javapluginEnabled = true;
	} else {
	    javapluginEnabled = false;
	}

	propValue = props.getProperty("javaplugin.jit.enabled");
	if (propValue == null || propValue.equals("true")) {
	    useJIT = true;
	} else {
	    useJIT = false;
	}

	propValue = props.getProperty("javaplugin.debug.mode");
	if (propValue != null && propValue.equals("true")) {
	    debugMode = true;
	} else {
	    debugMode = false;
	}

	debugPort = props.getProperty("javaplugin.debug.port");
	if (debugPort == null) {
	    debugPort = "2502";
	}

	propValue = props.getProperty("javaplugin.proxy.usebrowsersettings");
	if (propValue == null || propValue.equals("true")) {
	    useDefaultProxy = true;
	} else {
	    useDefaultProxy = false;
	}
	
   	proxySettings = props.getProperty("javaplugin.proxy.settings");

	propValue = props.getProperty("javaplugin.proxy.forallprotocols");
	useSameProxy = "true".equals(propValue);

	// Check JRE path
	javaRuntimePath = props.getProperty("javaplugin.jre.path");

	// Check JRE type
	javaRuntimeType = props.getProperty("javaplugin.jre.type");

	if (javaRuntimeType == null) 
	{
	    // If nothing specify, if javaRuntimePath is specified, default to others
	    if (javaRuntimePath != null && javaRuntimePath.trim().equals("") == false)
		javaRuntimeType = "Other";
	    else
		javaRuntimeType = "Default";
	}
	else if (javaRuntimeType.equalsIgnoreCase("JRE"))
	{
	    javaRuntimeType = "JRE";
	}
	else if (javaRuntimeType.equalsIgnoreCase("JDK"))
	{
	    javaRuntimeType = "JDK";
	}
	else if (javaRuntimeType.equalsIgnoreCase("Other"))
	{
	    if (javaRuntimePath != null && javaRuntimePath.trim().equals("") == false) 
		javaRuntimeType = "Other";
	    else
		javaRuntimeType = "Default";
	}
	else
	{
	    // Unrecognized type, fall back to default.
	    javaRuntimeType = "Default";
	}

	if (javaRuntimeType == null)
	    javaRuntimeType = "Default";


	javaVersion = props.getProperty("javaplugin.jre.version");

	javaParms = props.getProperty("javaplugin.jre.params");


	//////////////////////////////////////////////////////////////////////////
    	// Load the certificates from the certificate store

	// Clear the certificate maps
	activeCertsMap.clear();
	inactiveCertsMap.clear();

	try
	{
	    // Open certificate store
	    CertificateStore certStore = new PluginCertificateStore();

	    // Load certificates from store
	    certStore.load();

	    // Obtain certificate iterator
	    Iterator iter = certStore.iterator();

	    while (iter.hasNext())
	    {
		X509Certificate cert = (X509Certificate) iter.next();

		String alias = extractAliasName(cert);
		
		activeCertsMap.put(alias, cert);
	    }
	}
	catch (Exception e)
	{
	    Trace.printException(e);
	}
    }

    /**
     * Notify our PropertyChangeListeners that we have changed.
     */
    protected void firePropertyChange() {
	pcs.firePropertyChange(null, null, null);
    }

    /**
     * Add a property change listener.
     */
    public void addPropertyChangeListener(PropertyChangeListener pcl) {
	pcs.addPropertyChangeListener(pcl);
    }

    /**
     * Remove a property change listener.
     */
    public void removePropertyChangeListener(PropertyChangeListener pcl) {
	pcs.removePropertyChangeListener(pcl);
    }

    static private File theUserPropertiesFile;

    // The properties we read from disk.
    private Properties props;

    private boolean	debugMode = false;
    private boolean	javapluginEnabled = true;
    private boolean	showConsole = false;
    private boolean	showException = false;
    private boolean     classLoaderCacheEnabled = true;
    private boolean	useJIT = false;
    private String	JITPath;
    private String	debugPort = "2502";
    private String[][]  installedJREs;
    private String[][]  installedJDKs;
    private boolean	useDefaultProxy = true;
    private boolean	useSameProxy = false;
    private String	proxySettings;
    private String	javaParms;
    private String      javaVersion;
    private String      javaRuntimePath = null;
    private String	javaRuntimeType = "Default";

    // For RSA signing support in Win32 and UNIX
    private HashMap	activeCertsMap = new HashMap();
    private HashMap	inactiveCertsMap = new HashMap();

    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);

    private MessageHandler mh = new MessageHandler("config");
    
}

