/*
 * "@(#)MessageHandler.java	1.6 98/03/31
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.panel;

/**
 * Class to internationalize messages in the Activator ControlPanel.
 *
 * Each of the main classes of the ControlPanel creates an instance
 * of this using its own baseKey, such as "proxy".
 *
 * It then uses calls on (for example) getMessage() with a subKey
 * to locate specific localized strings.  The baseKey and the subKey
 * are combined as "baseKey.subKey" and resolved in the locale
 * specific ResourceBundle.
 *
 * @author Graham Hamilton
 *
 */

import java.util.ResourceBundle;

class MessageHandler {

    private String baseKey;
    private static ResourceBundle rb;

    MessageHandler(String baseKey) {
	this.baseKey = baseKey;
        try {
	    if (rb == null) {
                rb = ResourceBundle.getBundle("sun.plugin.resources.ControlPanel");
	    }
        } catch (java.util.MissingResourceException ex) {
	    // This "should never happen"
	    String mess = "Activator ControlPanel no locale info: " + ex;
            System.err.println(mess);
	    throw new Error(mess);
        }
    }

    String getMessage(String subKey) {
	String key = baseKey + "." + subKey;
        try {
	    return rb.getString(key);
        } catch (java.util.MissingResourceException ex) {
	    return key;
	}
    }

}
