/*
 * @(#)ProxyPanel.java	1.13 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 */

package sun.plugin.panel;

/**
 * Panel to display information about the proxies to be used by 
 * the javaplugin
 *
 * @version 	1.0 
 * @author	Jerome Dochez
 */

import javax.swing.*;
import java.awt.*;   
import java.awt.event.*;
import java.awt.Dimension;
import sun.plugin.usability.Trace;

public class ProxyPanel extends ActivatorSubPanel 
			implements  ActionListener,
				    KeyListener {

    /**
     * Contruct a new proxy panel
     */
    public ProxyPanel(ConfigurationInfo model) {
	super(model);

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	add(Box.createRigidArea(new java.awt.Dimension(1,5)));

	JPanel defaultPanel = new JPanel();
	defaultPanel.setLayout(new BoxLayout(defaultPanel, BoxLayout.X_AXIS));
	useDefault = new JCheckBox(mh.getMessage("use_browser"));
	useDefault.addActionListener(this);
	defaultPanel.add(useDefault);
	add(defaultPanel);

	add(Box.createRigidArea(new java.awt.Dimension(1,5)));

	JPanel panel = new JPanel();

	JPanel proxies = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	panel.setBorder(BorderFactory.createTitledBorder(mh.getMessage("proxy_settings")));
	GridLayout gridLayout = new GridLayout(6,3);
	gridLayout.setVgap(5);
	gridLayout.setHgap(10);
	proxies.setLayout(gridLayout);
	addCenteredLabel(proxies, mh.getMessage("protocol_type"));
    	addCenteredLabel(proxies, mh.getMessage("proxy_address"));
	addCenteredLabel(proxies, mh.getMessage("proxy_port"));

	httpLabel = addCenteredLabel(proxies, mh.getMessage("http"));
	http = addProxyTextField(proxies);
	httpPort = addProxyPortField(proxies);

	secureLabel = addCenteredLabel(proxies, mh.getMessage("https"));
	secure = addProxyTextField(proxies);
	securePort = addProxyPortField(proxies);

	ftpLabel = addCenteredLabel(proxies, mh.getMessage("ftp"));
	ftp = addProxyTextField(proxies);
	ftpPort = addProxyPortField(proxies);

	gopherLabel = addCenteredLabel(proxies, mh.getMessage("gopher"));
	gopher = addProxyTextField(proxies);
	gopherPort = addProxyPortField(proxies);

	socksLabel = addCenteredLabel(proxies, mh.getMessage("socks"));
	socks = addProxyTextField(proxies);
	socksPort = addProxyPortField(proxies);

	panel.add(proxies);
	sameProxies = new JCheckBox(mh.getMessage("same_for_all_protocols"));
	sameProxies.addActionListener(this);
	panel.add(Box.createRigidArea(new java.awt.Dimension(1,5)));
	panel.add(sameProxies);
	panel.add(Box.createRigidArea(new java.awt.Dimension(1,5)));

	reset();
	add(panel);

    }

    /**
     * Construct a new proxy setting panel
     *
     * @param panel to add the components in
     * @return the textfield
     */
    private JTextField addProxyTextField(JPanel container) {
	JTextField tf = new JTextField();
	tf.addKeyListener(this);
	container.add(tf);
	return tf;
    }
    
    private JTextField addProxyPortField(JPanel container) {
	JTextField tf = new JTextField();
	tf.addKeyListener(this);
	container.add(tf);
	return tf;
    }

    private JLabel addCenteredLabel(JPanel container, String label) {
	JLabel l = new JLabel(label);
	l.setHorizontalAlignment(l.CENTER);
	container.add(l);
	return l;
    }

    /**
     * Enable / Disable the fields depending on the use choices
     * Forward the choices to the model
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getSource()==useDefault) {
	    boolean b = useDefault.isSelected();
	    model.setDefaultProxySettings(b);
	    reset();
	}

	if (e.getSource()==sameProxies) {
	    boolean b = sameProxies.isSelected();
	    model.setUseSameProxy(b);

	    setProxySettings();

	    reset();
	}
    }

    /** 
     * Key Listener implementation.
     */
    public void keyTyped(KeyEvent e) {
 	setProxySettings();  
    }

    public void keyReleased(KeyEvent e) {
	// This shouldn't be necessary.  Unfortunately it seems to be.
	//					KGH 2/9/98
 	setProxySettings();
    }

    public void keyPressed(KeyEvent e) {
    }

    /** 
     * Reset proxy panel setting. Proxy panel setting should
     * be reinitialized according to model.
     */
    public void reset() 
    {
	boolean usedef  = model.useDefaultProxy();
	useDefault.setSelected(usedef);
	boolean same = model.useSameProxy();
	sameProxies.setSelected(same);

	boolean enable = !usedef;
	setEnabled(sameProxies, enable);
	setEnabled(httpLabel, enable);
	setEnabled(http, enable);
	setEnabled(httpPort, enable);

	enable = (!usedef) && (!same);
	setEnabled(secureLabel, enable);
	setEnabled(secure, enable);
	setEnabled(securePort, enable);
	setEnabled(ftpLabel, enable);
	setEnabled(ftp, enable);
	setEnabled(ftpPort, enable);    
	setEnabled(gopherLabel, enable);
	setEnabled(gopher, enable);
	setEnabled(gopherPort, enable); 
	setEnabled(socksLabel, enable); 
	setEnabled(socks, enable);
	setEnabled(socksPort, enable);

	// Populate proxy settings
	initProxySettings();
    }

    /** 
     * Saves the proxy settings 
     * The string is will be of the following format
     *	    protocol=proxy:proxyPort
     * Each protocol info is separated with a semicolumn (;)
     */
    private void setProxySettings() 
    {
	// Determine proxy for HTTP
	StringBuffer sb = new StringBuffer();
	boolean notFirst = false;
	if (http.getText().length()!=0) {
	    notFirst = true;
	    sb.append(http.getText());
	    if (httpPort.getText().length()!=0) {
		sb.append(":");
		sb.append(httpPort.getText());
	    }
	}
	
        String str = sb.toString();
	    
	// If sameProxies is not selected, figure out proxy for each protocols.
	//
	if (sameProxies.isSelected() == false)
	{
	    sb = new StringBuffer();
	    if (sameProxies.isSelected() && str.length()!=0) {
		sb.append("http=");
		sb.append(str);
		sb.append(";https=");
		sb.append(str);
		sb.append(";ftp=");
		sb.append(str);
		sb.append(";gopher=");
		sb.append(str);

		// Same proxy should never apply to SOCKS.
		//		-- Stanley  5/18/98.
	    } else {  
		if (http.getText().length()!=0) {
		    sb.append("http=");
		    sb.append(str);
		}

		if (secure.getText().length()!=0) {
		    if (notFirst)
			sb.append(";");
		    else
			notFirst = true;
		    sb.append("https=");
		    sb.append(secure.getText());
		    if (securePort.getText().length()!=0) {
			sb.append(":");
			sb.append(securePort.getText());
		    }
		}

		if (ftp.getText().length()!=0) {
		    if (notFirst)
			sb.append(";");
		    else
			notFirst = true;
		    sb.append("ftp=");
		    sb.append(ftp.getText());
		    if (ftpPort.getText().length()!=0) {
			sb.append(":");
			sb.append(ftpPort.getText());
		    }
		}

		if (gopher.getText().length()!=0) {
		    if (notFirst)
			sb.append(";");
		    else
			notFirst = true;
		    sb.append("gopher=");
		    sb.append(gopher.getText());
		    if (gopherPort.getText().length()!=0) {
			sb.append(":");
			sb.append(gopherPort.getText());
		    }
		}

		if (socks.getText().length()!=0) {
		    if (notFirst)
			sb.append(";");
		    sb.append("socks=");
		    sb.append(socks.getText());
		    if (socksPort.getText().length()!=0) {
			sb.append(":");
			sb.append(socksPort.getText());
		    }
		}
	    }

	    str = sb.toString();
	}


	if (str.length()!=0)
	    model.setProxySettingsString(str);
	else
	    model.setProxySettingsString(null);
    }

    /**
     * Load the proxy settings string in the proxy fields.
     * This method should only be called when reset() is
     * trigged, so we will populate all the fields 
     * according to proxySettings from the model.
     */
    private void initProxySettings() 
    {
	String proxySettings = model.getProxySettingsString();

	if (proxySettings==null)
	    return;

	if (proxySettings.length()==0)
	    return;

	try
	{
	    if (model.useSameProxy())
	    {
		// Only need to fill up HTTP
		fillProxySettings("http", proxySettings, http, httpPort);
		fillProxySettings("https", "", secure, securePort);
		fillProxySettings("ftp", "", ftp, ftpPort);
		fillProxySettings("gopher", "", gopher, gopherPort);
		fillProxySettings("socks", "", socks, socksPort);
	    }
	    else
	    {
		// Not using same proxy, then proxy should be read for
		// each protocol.
		//
		fillProxySettings("http", proxySettings, http, httpPort);
		fillProxySettings("https", proxySettings, secure, securePort);
		fillProxySettings("ftp", proxySettings, ftp, ftpPort);
		fillProxySettings("gopher", proxySettings, gopher, gopherPort);

		// If useSame proxy is specified in the field, skip SOCKS.
		//
		if (proxySettings.indexOf("=") != -1)
		    fillProxySettings("socks", proxySettings, socks, socksPort);
		else
		    fillProxySettings("socks", "", socks, socksPort);
	    }
	}
	catch (Exception e)
	{
	    Trace.printException(e);
	}	
    }


    /** 
     * Read the protocol configuration string for a particular protocol.
     * The settings are then set into two text fields for user manipulation
     * Each protocol can have a proxy server and a proxy port but not 
     * neccessarly two.
     * The protocol should be of the form :
     *	    protocol=proxy:port; 
     *	    protocol=proxy;
     *
     * @param settings the protocols string from the configuration file
     * @param proxyTf the text field to display the proxy server in
     * @param portTf the text field to display the proxy port in
     * 
     * @return the remaining protocol settings
     */
    private void fillProxySettings(String protocol, String proxyList, 
				   JTextField proxyTF, JTextField portTF)
    {
	String proxyInfo = readProxySetting(protocol, proxyList);

	int i = proxyInfo.indexOf(":");

	if (i == -1)
	{
	    proxyTF.setText(proxyInfo);
	    portTF.setText("");	    
	}		
	else
	{
	    proxyTF.setText(proxyInfo.substring(0, i));
	    portTF.setText(proxyInfo.substring(i + 1));
	}
    }


    private String readProxySetting(String protocol, String proxyList)
    {
	if (proxyList == null || proxyList.equals(""))
	    return "";

	int i = proxyList.indexOf("=");

	// Cannot find the protocol proxy, this is for all proxies
        if (i == -1)
	    return proxyList;
        
	i = proxyList.indexOf(protocol + "=");

	// Cannot find the proxy for this particular protocol
        if (i == -1)
	    return "";
	else
	{
	    int j = proxyList.indexOf(";", i);

	    if (j == -1) 
	    {
		return proxyList.substring(i + protocol.length() + 1);
	    }
	    else 
	    {
		return proxyList.substring(i + protocol.length() + 1, j);
	    }
	}
    }


    /**
     * Enable/Disabled a component
     *
     * @param field the component to enable/disable
     * @param b true to enable to text field, false to disable it
     */
    private void setEnabled(JComponent field, boolean b) {
	field.setEnabled(b);
	field.repaint();
    }

    private JLabel httpLabel;
    private JLabel ftpLabel;
    private JLabel gopherLabel;
    private JLabel secureLabel;
    private JLabel socksLabel;
    private JTextField http;
    private JTextField ftp;
    private JTextField gopher;
    private JTextField secure;
    private JTextField socks;
    private JTextField httpPort;
    private JTextField ftpPort;
    private JTextField gopherPort;
    private JTextField securePort;
    private JTextField socksPort;
    private JCheckBox  useDefault;
    private JCheckBox  sameProxies;

    private MessageHandler mh = new MessageHandler("proxy");
}

