/*
 * @(#)ServiceProvider.java	1.5 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.1 
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.platform;

import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;


/**
 * ServiceProvider is a factory class to obtain different native 
 * code implementation in each platform.
 */
public class ServiceProvider
{
    // Are we on Windows?
    private static boolean onWindows = false;

    // Platform handler for this platform
    private static PlatformHandler handler = null;

    static 
    {
	initialize();
    }

    /**
     * Returns service provider for a particular platform.
     *
     * @return PlatformHandler.
     */
    public synchronized static PlatformHandler getProvider()
    {
	return handler;
    }

    /** 
     * Setup platform handler.
     */
    private synchronized static void initialize()
    {
	try
	{
	    String osName = (String) java.security.AccessController.doPrivileged(
				    new sun.security.action.GetPropertyAction("os.name"));

	    Class c = sun.plugin.platform.DefaultPlatformHandler.class;

	    if (osName.indexOf("Windows") != -1)
		c = Class.forName("sun.plugin.platform.WindowsPlatformHandler");
	    else	    
		c = Class.forName("sun.plugin.platform.MotifPlatformHandler");

	    handler = (PlatformHandler) c.newInstance();
	}
	catch (Throwable e)
	{
	    Trace.printException(e);

	    handler = new sun.plugin.platform.DefaultPlatformHandler();
	}
    }
}


