/*
 * @(#)JarDownloadFeedback.java	1.1 99/05/21
 *
 * Copyright 1997-2000 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.plugin.protocol;

import javax.swing.JProgressBar;
import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.WindowEvent;


// This doesn't use javax.swing.ProgressMonitor because PM has a hardcoded title
// of "Progress..." and a Cancel button.
//
// However, the progress dialog is created lazily as per ProgressMonitor.

public class JarDownloadFeedback {

    // Milliseconds that must elapse before deciding whether or not to show dialog.

    private final static long   DECIDE_TIME     = 200;

    // This many milliseconds must remain for dialog to be shown (predicted value).

    private final static long   POPUP_TIME      = 1500;

    // NOTE: Hardcoded strings need to be localized...

    private final static String TITLE           = "Java Extension Installation";
    private final static String MSG             = "Downloading from URL :";


    // Time when dialog constructed.

    private long                T0;

    private String              url;

    private JProgressBar        progressBar;
    private JDialog             dialog;


    public JarDownloadFeedback( String url ) {
        this.url = url;
        T0       = System.currentTimeMillis();
    }


    // Initialize progress dialog. 

    public void onStartBinding(final int progress, final int progressMax) { 
        onProgress(progress, progressMax);
    }


    // Update progress dialog

    public void onProgress(final int progress, final int progressMax) {

        // Use an algorithm like ProgressMonitor to determine when to show dialog.

        long dT = System.currentTimeMillis() - T0;

        if ( dT >= DECIDE_TIME )
        {
            long predictedTime = ( progress > 0 ? ( dT * progressMax / progress ) : POPUP_TIME );
            
            if ( predictedTime >= POPUP_TIME )
            {
                SwingUtilities.invokeLater( new Runnable() {
                    public void run() {

                        if ( null == dialog )
                            createDialog();

                        progressBar.setMaximum(progressMax);
                        progressBar.setValue(progress);
                    }
                } );
            }   // IF it looks like the download will take long enough to show dialog.
        }   // IF it's time to decide when to show dialog.
    }


    // Destroy progress dialog 

    public void onStopBinding() {
        SwingUtilities.invokeLater( new Runnable() {
            public void run() {
                if ( null != dialog )
                    dialog.dispatchEvent(new WindowEvent(dialog, WindowEvent.WINDOW_CLOSING));
            }
        } );
    }


    // Create progress dialog.

    private void createDialog()
    {
        Box box = new Box( BoxLayout.Y_AXIS );

        EmptyBorder border = new EmptyBorder( 0, 5, 0, 5 );

        JLabel label = new JLabel( MSG );
        label.setAlignmentX( Component.CENTER_ALIGNMENT );
        label.setBorder( border );
        box.add( label );

        label = new JLabel( url );
        label.setAlignmentX( Component.CENTER_ALIGNMENT );
        label.setBorder( border );
        box.add( label );

        box.add( Box.createVerticalStrut( 10 ) );

        progressBar = new JProgressBar( 0, 100 );
        label.setAlignmentX( Component.CENTER_ALIGNMENT );
        box.add( progressBar );

        box.add( Box.createVerticalStrut( 10 ) );

        dialog = new JDialog( (javax.swing.JFrame) null, TITLE, false );
        dialog.getContentPane().add( box );
        dialog.pack();

        Dimension d = dialog.getSize();

        if ( d.width < 300 )
            d.width = 300;

        dialog.setSize( d );

        dialog.setLocation( 200, 200 );
        dialog.setVisible( true );
    }
}
