/*
 * @(#)BrowserHttpsOutputStream.java	1.5 00/04/27
 * 
 * Copyright (c) 1996-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package sun.plugin.protocol.https;

import java.net.URLConnection;
import java.io.IOException;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;

/**
 * HTTPS URL POST support. This class buffers all the output stream
 * until the stream is closed, then it will do a POST on the web
 * server through the browser API.
 *
 * @author Stanley Man-Kit Ho
 */
public
class BrowserHttpsOutputStream extends java.io.ByteArrayOutputStream
{
    // URLConnection of the HTTPS connection.
    BrowserHttpsURLConnection urlConnection = null;

    // Determine if the HTTPS output stream has been closed.
    private boolean bConnected = false;

    /**
     * <P> Create BrowserHttpsOutputStream object.
     * </P>
     *
     * @param url URL of the HTTPS connection.
     */
    public BrowserHttpsOutputStream(BrowserHttpsURLConnection urlConnection)
    {
	super();
	this.urlConnection = urlConnection;
	this.bConnected = true;
    }


    /**
     * <P> Create BrowserHttpsOutputStream object.
     * </P>
     *
     * @param url URL of the HTTPS connection.
     * @param size Size of the output stream buffer.
     */
    public BrowserHttpsOutputStream(BrowserHttpsURLConnection urlConnection, int size)
    {
	super(size);
	this.urlConnection = urlConnection;
	this.bConnected = true;
    }

    /**
     * <P> Closes this output stream.
     * </P>
     */
    public synchronized void close() throws IOException
    {
	// Make sure the stream has not been closed multiple times
	if (bConnected)
	{
	    bConnected = false;
	    super.close();

	    byte[] postResponse;
	    byte[] buf = toByteArray();

	    if (buf == null)
		postResponse = postHttpsURL(urlConnection, urlConnection.getURL().toString(), null, 0);
	    else
		postResponse = postHttpsURL(urlConnection, urlConnection.getURL().toString(), buf, buf.length);

	    if (urlConnection.getDoInput()) {
		urlConnection.postResponse = postResponse;
		urlConnection.postResponseReady = true;
	    }
	}
    }

    /**
     * <P> Native code to call back into the browser to do a POST.
     * </P>
     *
     * @param url URL of the HTTPS connection
     * @param buf Data to post.
     */
    private native byte[] postHttpsURL(URLConnection urlConnection, String url, byte[] buf, int length);
}
