/*
 * @(#)Handler.java	1.31 96/11/23
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 * Java Plug-in
 *
 * @author Jerome Dochez
 * @author Stanley Man-Kit Ho
 */

/*-
 *	FTP stream opener
 */

package sun.plugin.protocol.jdk12.ftp;

import java.net.URL;
import java.io.IOException;
import sun.plugin.protocol.jdk12.http.HttpURLConnection;
import sun.plugin.protocol.*;
import java.util.Map;
import java.util.HashMap;
import sun.net.ftp.FtpClient;
import sun.net.www.protocol.ftp.*;

/**
 * Open an ftp connection given a URL 
 */
public class Handler extends java.net.URLStreamHandler {


    public java.net.URLConnection openConnection(URL u) {
	/* if set for proxy usage then go through the gopher code to get
	 * the url connection.
         */
	ProxyHandler handler = PluginProxyHandler.getDefaultProxyHandler();
        ProxyInfo pinfo = null;

        if (handler != null)
           pinfo = handler.getProxyInfo(u);

	try  {
	    if (pinfo != null && pinfo.isProxyUsed())
	    {
		return new sun.plugin.protocol.jdk12.http.HttpURLConnection(u, pinfo.getProxy(), pinfo.getPort());
	    }
	    else 
	    {
  		/* make a direct gopher connection */
		return superOpenConnection(u);
	    }
        } catch(IOException e)  {
	    return superOpenConnection(u);
	}
    }


    protected java.net.URLConnection superOpenConnection(URL u) {
	/* if set for proxy usage then go through the http code to get */
	/* the url connection. Bad things will happen if a user and
	 * password are specified in the ftp url */
	try  {
	    if (FtpClient.getUseFtpProxy()) {
		String host = FtpClient.getFtpProxyHost();
		if (host != null &&
		    host.length() > 0) {
		    return new sun.plugin.protocol.jdk12.http.HttpURLConnection(u, host,
					         FtpClient.getFtpProxyPort());
		}
	    }
	} catch(IOException e)  {
	}

	/* make a direct ftp connection */
	return new FtpURLConnection(u);
    }

    // From sun.net.www.protocol.ftp.Handler
    protected int getDefaultPort() {
        return 21;
    }

    // From sun.net.www.protocol.ftp.Handler
    protected boolean equals(URL u1, URL u2) {
	String userInfo1 = u1.getUserInfo();
	String userInfo2 = u2.getUserInfo();
	return super.equals(u1, u2) && 
	    (userInfo1 == null? userInfo2 == null: userInfo1.equals(userInfo2));
    }
}
