/*
 * @(#)HttpClient.java	1.00 97/04/16
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 */

package sun.plugin.protocol.jdk12.http;

import java.io.*;
import java.net.*;
import java.util.*;
import sun.net.NetworkClient;
import sun.net.ProgressEntry;
import sun.net.ProgressData;
import sun.net.www.MessageHeader;
import sun.net.www.HeaderParser;
import sun.net.www.MeteredStream;
import sun.misc.Regexp;
import sun.misc.RegexpPool;
import sun.plugin.protocol.http.SocksSocket;

import sun.plugin.protocol.ProxyInfo;
import sun.plugin.protocol.ProxyHandler;

/**
 * @version 1.00, 04/16/98
 * @author  Stanley Man-Kit Ho
 */
public class HttpClient extends sun.net.www.http.HttpClient {

    /* This package-only CTOR should only be used for FTP piggy-backed on HTTP 
     * HTTP URL's that use this won't take advantage of keep-alive.
     * Additionally, this constructor may be used as a last resort when the
     * first HttpClient gotten through New() failed (probably b/c of a 
     * Keep-Alive mismatch).
     *
     * XXX That documentation is wrong ... it's not package-private any more
     */
    public HttpClient(URL url, String proxy, int proxyPort)
			throws IOException {
	super (url, proxy, proxyPort);
    }

    /* This class has no public constructor for HTTP.  This method is used to
     * get an HttpClient to the specifed URL.  If there's currently an 
     * active HttpClient to that server/port, you'll get that one.
     */
    public static HttpClient New(URL url, String proxy, int proxyPort) 
    throws IOException {
	/* see if one's already around */
	HttpClient ret = (HttpClient) kac.get(url);
	if (ret == null) {
	    ret = new HttpClient (url, proxy, proxyPort);  // CTOR called openServer()
	} else {
	    ret.url = url;
	}
	// don't know if we're keeping alive until we parse the headers
	// for now, keepingAlive is false
	return ret;
    }

    /**
     * Return a socket connected to the server, with any
     * appropriate options pre-established
     */
    protected Socket doConnect (String server, int port)
    throws IOException, UnknownHostException {

	ProxyHandler handler = sun.plugin.protocol.PluginProxyHandler.getDefaultProxyHandler();
	if (handler != null)
	{
	    ProxyInfo pinfo = handler.getProxyInfo(url);

	    if (pinfo != null && pinfo.isSocksUsed())
	    {
		// Use SOCKS !!
	        return new SocksSocket(server, port, pinfo.getSocksProxy(), pinfo.getSocksPort());
	    }
	}

	return super.doConnect(server, port);
    }
}
