/*
 * @(#)HttpURLConnection.java	1.41 01/02/15
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.3 
 *
 */

package sun.plugin.protocol.jdk12.http;

import java.net.URL;
import java.net.ProtocolException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.ByteArrayOutputStream;
import java.io.*;
import java.util.*;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.security.PrivilegedActionException;
import sun.net.*;
import sun.net.www.*;
import sun.plugin.protocol.*;
import sun.plugin.protocol.jdk12.http.Handler;
import sun.plugin.protocol.jdk12.http.HttpClient;
import sun.plugin.AppletViewer;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;


/**
 * A class to represent an HTTP connection to a remote object.
 *
 * @author  Graham Hamilton
 *          Stanley Man-Kit Ho
 *          Robert Szewczyk
 */

public class HttpURLConnection extends
sun.net.www.protocol.http.HttpURLConnection {

    protected String proxy = null;
    protected int proxyPort = -1;

    // This is to keep trace of the failedOnce value in the super class. Since failedOnce
    // in the super one are declared private, we do some hack to get this value.
    boolean failedOnce = false;

    public HttpURLConnection(URL u, Handler handler)
    throws IOException {
	super(u, handler);
    }

    /** this constructor is used by other protocol handlers such as ftp
        that want to use http to fetch urls on their behalf. */
    public HttpURLConnection(URL u, String host, int port) 
    throws IOException {
	super(u, host, port);
        this.proxy = host;
        this.proxyPort = port;
    }

    static CookieHandler handler = null;

    /* setCookieHandler is used only when a particular CookieHandler is required
     * to determine the cookie value of a particular URL on the fly.
     */
    public static void setCookieHandler(CookieHandler h)  {
        handler = h;
    }

    protected boolean fromClassLoader() {
	Exception e = new Exception();
	ByteArrayOutputStream ba = new ByteArrayOutputStream();
	PrintStream pos = new PrintStream(ba);
	//e.printStackTrace(pos);
	String trace = ba.toString();
	StringTokenizer tok = new StringTokenizer(trace);
	String s = null;
	while(tok.hasMoreTokens()) {
	    s = tok.nextToken();
	    if ((s.startsWith("sun.applet.AppletClassLoader")) ||
		(s.startsWith("sun.applet.AppletResourceLoader")) || 
		(s.startsWith("sun.plugin.security.PluginClassLoader"))) {
		return true;
	    }
	}
	return false;
    }
    
    protected boolean rightExt() {
	String fname = url.getFile();
	return (fname.endsWith(".jar") | fname.endsWith(".class"));
    }
    
    void privBlock() throws Exception {
	try {
	    if ("http".equals(url.getProtocol()) && !failedOnce) {
		http = HttpClient.New(url, proxy, proxyPort);
	    } else {
		// make sure to construct new connection if first attempt failed
		http = getProxiedClient(url, proxy, proxyPort);
	    }
	    ps = (PrintStream)http.getOutputStream();
	} catch (IOException e) {
	    throw e;
	}
    }


    // true if the first step for connection has been made
    private boolean connectStep1 = false;

    // true if the second step for connection has been made
    private boolean connectStep2 = false;

    // true if the set-cookie header has been checked
    private boolean checkSetCookie = false;

    // overridden in HTTPS subclass

    public synchronized void connect() throws IOException {

	if (connected)
	    return;

	// Perform connection step 1
	connectStep1();

	// Perform connection step 2
	connectStep2();

	try {
	    AccessController.doPrivileged(new PrivilegedBlockAction(this));	
	} catch (PrivilegedActionException e) {
	    IOException ioe = (IOException)e.getException();
	    throw ioe;
	}

	connected = true;
    }


    /**
     * Perform step 1 to establish a connection: security check and setup   
     * the caching option.
     */     
    public synchronized void connectStep1() throws IOException {

	if (connectStep1)
	    return;

	// Output connection message to tracing facilities
	//
	String mess = getMessage("opening_url") + url;
	Trace.netPrintln(mess, TraceFilter.DEFAULT | TraceFilter.STATUS_BAR_ONLY);

	// Check whether the applet is allowed to connect to the host.
	// This is necessary because we will enable privilege to 
	// connect to the proxy
	SecurityManager m = System.getSecurityManager();
	if (m != null)	{
	    m.checkConnect(url.getHost(), url.getPort());
	}

        // Workaround for Mozilla bugs
        String workaround = (String) java.security.AccessController.doPrivileged(
                                        new sun.security.action.GetPropertyAction("mozilla.workaround"));

        if (workaround != null && workaround.equalsIgnoreCase("true"))
        {
            // Disable browser caching in Mozilla
            setUseCaches(false);
        }
        else
        {
	    // constructor to HTTP client calls openserver
	    setUseCaches(getUseCaches() && rightExt());
        }

	connectStep1 = true;
    }


    /**
     * Perform step 2 to establish a connection: determine proxy/cookie
     * setting, and setup a real connection.
     */     
    public synchronized void connectStep2() throws IOException {

	if (connectStep2)
	    return;

	// Determine proxy setting for the connection

        if (proxy == null) {
	   ProxyInfo pinfo = null;
	   ProxyHandler proxyHandler = PluginProxyHandler.getDefaultProxyHandler();

	   if (proxyHandler!=null) {
		pinfo = proxyHandler.getProxyInfo(url);

		if (pinfo != null)
		{
        	    proxy = pinfo.getProxy();
		    proxyPort = pinfo.getPort();
		}
	   }
	}


	// Output connection message to tracing facilities
	//
	String mess = "Connecting " + url + " with ";
	if (proxy == null) {
	    mess += getMessage("no_proxy");
	} else {
	    mess += getMessage("proxy_equals") + proxy;
	    if (proxyPort != -1) {
		mess += ":" + proxyPort;
	    }
	}
	Trace.netPrintln(mess);

        // Determine cookie value
        if (handler != null)
        {
	    // Use browser cookie only if the cookie has not 
	    // been set by the applet.
	    if (getRequestProperty("cookie") == null)
	    {
	        String cookie = handler.getCookieInfo(url);

		if (cookie != null)
		    setRequestProperty("cookie", cookie);
	    }
        }
   
	connectStep2 = true;
    }

    public synchronized void checkCookieHeader()
    {
	if (checkSetCookie)
	    return;

	checkSetCookie = true;
	
	// We have been connected already, so we need to check 
	// if "Set-Cookie" is in the respond headers
        if (handler != null)
        {
	    String cookieValue = getHeaderField("Set-Cookie");

	    if (cookieValue != null)
	    {
	        handler.setCookieInfo(url, cookieValue);
	    }
        }
    }


    private InputStream cacheStream = null;

    public synchronized InputStream getInputStream() throws IOException{

	// Perform security check, without making real connection
	connectStep1();

	// If cache is enabled, try the browser cache.
	if (useCaches) {
    	    try {
		// check if we are allowed to read files. If not, there is
		// no point to caching. Now, we have no idea where the
		// browser will put the file, so lets try to read
		// java.home property
		if (cacheStream != null)
		    return cacheStream;
	
		cacheStream = (InputStream)
		    AccessController.doPrivileged(new FileCreator(this));
		if (cacheStream==null) {
		    useCaches = false;
		} 
		else 
		{
		    // We sucessfully obtained the cache file, and HTTP_OK must be
		    // set.
		    responseCode = HTTP_OK;

		    return cacheStream;
		}
	    } catch (PrivilegedActionException e) { //IOException. fall through, and try
		//remote
		Trace.netPrintException(e);
		Trace.netPrintln("IO Exception, using remote copy", TraceFilter.JAVA_CONSOLE_ONLY);
		useCaches = false;
	    } catch (SecurityException e) {
		Trace.netPrintException(e);
		Trace.netPrintln("Security exception, using remote copy", TraceFilter.JAVA_CONSOLE_ONLY);
		useCaches = false;
	    }  
	}

	// At this point, the browser has failed to obtain the file from
	// the cache, so let's make a real connection.
	connect();

	// Check "Set-cookie" header
	checkCookieHeader();

	return super.getInputStream();
    }

    /**
     * Create a new HttpClient object, bypassing the cache of
     * HTTP client objects/connections.
     *
     * @param url	the URL being accessed
     */
    protected sun.net.www.http.HttpClient getNewClient (URL url)
    throws IOException {

	// This is a hack to get the failedOnce value. Since getNewClient is only called by the 
	// super class when failedOnce is true, we can obtain the failedOnce value this way. 
	failedOnce = true;

	return getProxiedClient(url, proxy, proxyPort);
    }


    /**
     * Create a new HttpClient object, set up so that it uses
     * per-instance proxying to the given HTTP proxy.  This
     * bypasses the cache of HTTP client objects/connections.
     *
     * @param url	the URL being accessed
     * @param proxyHost	the proxy host to use
     * @param proxyPort	the proxy port to use
     */
    protected sun.net.www.http.HttpClient getProxiedClient (URL url, String proxyHost, int proxyPort)
    throws IOException {
	return new HttpClient (url, proxyHost, proxyPort);
    }

    /**
     * Method to get an internationalized string from the Activator resource.
     */
    public static String getMessage(String key) {
        try {
	    if (rb == null) {
		rb = ResourceBundle.getBundle("sun.plugin.resources.Activator");
	    }
	    return rb.getString(key);
        } catch (java.util.MissingResourceException ex) {
	    return key;
	}
    }

    // Localization strings.
    private static ResourceBundle rb;

    class FileCreator implements PrivilegedExceptionAction {
	HttpURLConnection conn;

	FileCreator(HttpURLConnection c) {
	    conn = c;
	}

	public Object run() throws Exception {
	    
	    String fileName = sun.plugin.CacheHandler.getCacheFile(conn.getURL());
	    if (fileName!=null)
	       return new BufferedInputStream(new FileInputStream(fileName));
	    else 
	       return null;
	}
    }

    class PrivilegedBlockAction implements PrivilegedExceptionAction {
	HttpURLConnection conn;
	PrivilegedBlockAction(HttpURLConnection c)
	{
	    conn = c;
	}

	public Object run() throws Exception {
	    conn.privBlock();
	    return null;
	}
    }

}


