/*
 * @(#)CertificateDialog.java	1.3 01/02/09
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.2
 *
 * Java Plug-in 
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.security;

import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import java.util.StringTokenizer;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import sun.security.x509.SerialNumber;
import sun.security.util.BigInt;
import sun.misc.HexDumpEncoder;

class CertificateDialog extends JDialog implements ListSelectionListener, ActionListener {

    private java.security.cert.Certificate[] certs = null;
    private int index = 0;

    JTextArea textArea = new JTextArea();

    /**
     * <P> Constructor for CertificateDialog.
     * </P>
     *
     * @param dialog Parent window of the certificate dialog
     * @param certs Array of certificates
     * @param index Index of the certificate array to show
     */
    CertificateDialog(JDialog dialog, java.security.cert.Certificate[] certs, int index)
    {
	super((dialog != null ? dialog : new JDialog()), getMessage("cert_dialog.caption"), true);
	this.certs = certs;
	this.index = index;
    }


    /**
     * <P> Show CertificateDialog.
     * </P>
     */
    void DoModal()  
    {

	if (certs.length > index && certs[index] instanceof X509Certificate)
	{
	    setBackground(SystemColor.control);

	    // Create borders
	    Border etchBorder = BorderFactory.createEtchedBorder();
	    TitledBorder fieldBorder = BorderFactory.createTitledBorder(etchBorder, 
									getMessage("cert_dialog.field"));
	    TitledBorder valueBorder = BorderFactory.createTitledBorder(etchBorder, 
									getMessage("cert_dialog.value"));

	    // Create JList for lhs	    
	    //
	    Object[] fields = { getMessage("cert_dialog.field.Version"),
				getMessage("cert_dialog.field.SerialNumber"),
				getMessage("cert_dialog.field.SignatureAlg"),
				getMessage("cert_dialog.field.Issuer"),
				getMessage("cert_dialog.field.EffectiveDate"),
				getMessage("cert_dialog.field.ExpirationDate"),
				getMessage("cert_dialog.field.Subject"),
				getMessage("cert_dialog.field.Signature") };
	    JList list = new JList(fields);
	    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    list.addListSelectionListener(this);
	    list.setSelectedIndex(0);

	    JPanel panelLeft = new JPanel();
	    panelLeft.setLayout(new BorderLayout());
	    panelLeft.setBorder(fieldBorder);
	    panelLeft.add(new JScrollPane(list), BorderLayout.CENTER);

	    // Create JTextArea for rhs	    
	    //
	    //textArea.setLineWrap(true);
	    textArea.setEditable(false);
	    JPanel panelRight = new JPanel();
	    panelRight.setLayout(new BorderLayout());
	    panelRight.setBorder(valueBorder);
	    panelRight.add(new JScrollPane(textArea), BorderLayout.CENTER);

	    JPanel panelCenter = new JPanel();
	    panelCenter.setLayout(new GridLayout(1, 2));
	    panelCenter.add(panelLeft);
	    panelCenter.add(panelRight);

	    // Add buttons to the UI
	    //
	    JPanel panelBottom = new JPanel();
	    JButton buttonOkay = new JButton(getMessage("cert_dialog.okButton"));
	    panelBottom.add(buttonOkay);
	    buttonOkay.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    dispose();
		}
	    });

	    JButton buttonIssuer = new JButton(getMessage("cert_dialog.issuerButton"));
	    if (certs.length == index + 1)
		buttonIssuer.setEnabled(false);
	    else
		buttonIssuer.addActionListener(this);
	    panelBottom.add(buttonIssuer);

	    getContentPane().setLayout(new BorderLayout());
	    getContentPane().add(panelCenter, BorderLayout.CENTER);
	    getContentPane().add(panelBottom, BorderLayout.SOUTH);
	    setSize(500, 300);
	    setResizable(false);
	    setVisible(true);
	}
    }


    /**
     * <P> Launch Certificate dialog if the "Issuer" button is clicked.
     * </P>
     *
     * @param e ActionEvent object.
     */
    public void actionPerformed(ActionEvent e)
    {
	CertificateDialog dialog = new CertificateDialog(this, certs, index + 1);
	dialog.DoModal();
    }


    /**
     * <P> Updated UI if different item in JList is selected.
     * </P>
     *
     * @param e ListSelectionEvent object.
     */
    public void valueChanged(ListSelectionEvent e)
    {
	JList list = (JList) e.getSource();
	int i = list.getSelectedIndex();
	X509Certificate cert = (X509Certificate) certs[index];

	// Populate text area with different info
	switch(i)  {

	case 0:	textArea.setText(cert.getVersion() + "");
		break;
	case 1:	{
		    textArea.setLineWrap(false);

		    SerialNumber serial = new SerialNumber(cert.getSerialNumber());
		    textArea.setText(serial.getNumber().toString());
		    textArea.repaint();
		    break;
		}
	case 2:	textArea.setLineWrap(false);
		textArea.setText(cert.getSigAlgName());
		textArea.repaint();
		break;
	case 3:	textArea.setLineWrap(true);
		textArea.setText(cert.getIssuerDN().toString());
		textArea.repaint();
		break;
	case 4:	textArea.setLineWrap(false);
		textArea.setText(cert.getNotBefore().toString());
		textArea.repaint();
		break;
	case 5:	textArea.setLineWrap(false);
		textArea.setText(cert.getNotAfter().toString());
		textArea.repaint();
		break;

	case 6:	textArea.setLineWrap(true);
		textArea.setText(cert.getSubjectDN().toString());
		textArea.repaint();
		break;
	case 7:	{
		    textArea.setLineWrap(false);
		
		    // Dump hex
		    HexDumpEncoder encoder = new HexDumpEncoder();
		    textArea.setText(encoder.encodeBuffer(cert.getSignature()));
		    textArea.repaint();
		    break;
		}
	}
    }


    /**
     * Overrides java.awt.Container.setVisible() so that we are
     * sure that the Frame's size is set before displaying.
     */
    public void setVisible(boolean state)
    {
	if (state)
	{
            Dimension bounds = getToolkit().getScreenSize();
            setLocation((bounds.width - getSize().width)/2,
        	        (bounds.height - getSize().height)/2);
        }

	super.setVisible(state);
    }


    /**
     * Method to get an internationalized string from the Activator resource.
     */
    private static String getMessage(String key)
    {
	return sun.plugin.resources.ResourceHandler.getMessage(key);
    }
}


