/*
 * @(#)SessionCertificateStore.java	1.7 00/04/13
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in 
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * Java Plug-in 
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.security;

import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import sun.plugin.usability.Trace;


/**
 * SessionCertificateStore is a class that represents the session certificate 
 * store which contains all the certificates that Java Plug-in recognizes. The
 * certificates store in this certificate store is only valid in the current
 * browser session. It is used in the certification verification process 
 * when signed applet is encountered.
 */
public class SessionCertificateStore implements CertificateStore
{
    // Collection of JPI certificates
    private Collection _certs = new HashSet();

    /**
     * Load the certificate store into memory.
     */
    public void load() throws IOException, CertificateException,
			      KeyStoreException, NoSuchAlgorithmException
    {
	Trace.securityPrintln("Loading certificates from JPI session certificate store");

	// Do nothing

	Trace.securityPrintln("Loaded certificates from JPI session certificate store");
    }


    /**
     * Persist the certificate store.
     */
    public void save() throws IOException, CertificateException,
			      KeyStoreException, NoSuchAlgorithmException
    {
	Trace.securityPrintln("Saving certificates from JPI session certificate store");

	// Do nothing

	Trace.securityPrintln("Saved certificates from JPI session certificate store");
    }

 
    /**
     * Add a certificate into the certificate store.
     *
     * @param cert Certificate object.
     */
    public void add(Certificate cert) throws KeyStoreException
    {
	Trace.securityPrintln("Adding certificate in JPI session certificate store");

	_certs.add(cert);

	Trace.securityPrintln("Added certificate in JPI session certificate store");
    }

    
    /**
     * Remove a certificate from the certificate store.
     *
     * @param cert Certificate object.
     */
    public void remove(Certificate cert) throws KeyStoreException
    {
	Trace.securityPrintln("Removing certificate in JPI session certificate store");

	_certs.remove(cert);

	Trace.securityPrintln("Removed certificate in JPI session certificate store");
    }


    /**
     * Check if a certificate is stored within the certificate store.
     *
     * @param cert Certificate object.
     * @return true if certificate is in the store.
     */
    public boolean contains(Certificate cert) throws KeyStoreException
    {
	Trace.securityPrintln("Checking if certificate is in JPI session certificate store");

	return _certs.contains(cert);
    }


    /**
     * Verify if a certificate is issued by one of the certificate
     * in the certificate store.
     *
     * @param cert Certificate object.
     * @return true if certificate is issued by one in the store.
     */
    public boolean verify(Certificate cert) throws KeyStoreException 
    {
	Trace.securityPrintln("Check if certificate can be verified using certificates in JPI session certificate store");

	// Session Certificate store is not intended to be used for verification.
	return false;
    }

    /**
     * Obtain all the certificates that are stored in this 
     * certificate store.
     *
     * @return Iterator for iterating certificates
     */
    public Iterator iterator() throws KeyStoreException
    {
	Trace.securityPrintln("Obtain certificate iterator in JPI session certificate store");

	return _certs.iterator();
    }
}
