/*
 * @(#)TrustDeciderDialog.java	1.12 00/08/02
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * CopyrightVersion 1.2
 *
 * Java Plug-in 
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.security;

import java.awt.*;
import java.awt.event.*;
import java.security.*;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import java.util.StringTokenizer;
import javax.swing.*;
import sun.plugin.usability.DialogFactory;
import sun.plugin.usability.Trace;
import sun.plugin.usability.TraceFilter;


class TrustDeciderDialog implements ActionListener {

    public static final int TrustOption_GrantThisSession = 0;
    public static final int TrustOption_Deny = 1;
    public static final int TrustOption_GrantAlways = 2;


    private java.security.cert.Certificate[] certs = null;

    /**
     * <P> Constructor for TrustDeciderDialog.
     * </P>
     *
     * @param certs Array of certificates
     */
    TrustDeciderDialog(java.security.cert.Certificate[] certs)  {
	this.certs = certs;
    }

    /**
     * <P> Show TrustDeciderDialog.
     * </P>
     *
     * @return 0 if "Grant this session" button is clicked.
     * @return 1 or -1 if "Deny" is clicked.
     * @return 2 if "Grant Always" button is clicked.
     */
    int DoModal()  
    {
	int ret = -1;

	// Check if the certificate is a x.509 certificate
	//
	if (certs[0] instanceof X509Certificate 
	    && certs[certs.length-1] instanceof X509Certificate)
	{
	    X509Certificate cert = (X509Certificate) certs[0];
	    X509Certificate cert2 = (X509Certificate) certs[certs.length-1];
	    Principal prinSubject = cert.getSubjectDN();
	    Principal prinIssuer = cert2.getIssuerDN();

	    // Extract subject name
	    String subjectDNName = prinSubject.getName();

	    String subjectName = null;

	    int i = subjectDNName.indexOf("CN=");
	    int j = 0;

	    if (i < 0)
	    {
		subjectName = "\"Unknown subject\"";
	    }
	    else
	    {
		try
		{
		    // Shift to the beginning of the CN text
		    i = i + 3;

		    // Check if it begins with a quote
		    if (subjectDNName.charAt(i) == '\"')
		    {
			// Skip the quote
			i = i + 1;

			// Search for another quote
			j = subjectDNName.indexOf('\"', i);
		    }
		    else
		    {
			// No quote, so search for comma
			j = subjectDNName.indexOf(',', i);
		    }

		    if (j < 0)
			subjectName = subjectDNName.substring(i);
		    else
			subjectName = subjectDNName.substring(i, j);

		    // Added quote to subject name
		    subjectName = "\"" + subjectName + "\"";
		}
		catch (Throwable e)
		{
		    subjectName = "\"Unknown subject\"";
		}
	    }


	    // Extract issuer name
	    String issuerDNName = prinIssuer.getName();
	    String issuerName = null;

	    i = issuerDNName.indexOf("O=");
	    j = 0;	

	    if (i < 0)
	    {
		issuerName = "\"Unknown issuer\"";
	    }
	    else
	    {
		try
		{
		    // Shift to the beginning of the O text
		    i = i + 2;

		    // Check if it begins with a quote
		    if (issuerDNName.charAt(i) == '\"')
		    {
			// Skip the quote
			i = i + 1;

			// Search for another quote
			j = issuerDNName.indexOf('\"', i);
		    }
		    else
		    {
			// No quote, so search for comma
			j = issuerDNName.indexOf(',', i);
		    }

		    if (j < 0)
			issuerName = issuerDNName.substring(i);
		    else
			issuerName = issuerDNName.substring(i, j);

		    // Added quote to issuer name
		    issuerName = "\"" + issuerName + "\"";
		}
		catch (Throwable e)
		{
		    issuerName = "\"Unknown issuer\"";
		}
	    }

	    // Construct dialog text
	    String dialogText = getMessage("security_dialog.text0")
			      + subjectName  
			      + getMessage("security_dialog.text1")
			      + issuerName 
			      + getMessage("security_dialog.text2")
			      + subjectName 
			      + getMessage("security_dialog.text3")
			      + subjectName 
			      + getMessage("security_dialog.text4");

	    JButton buttonInfo = new JButton(getMessage("security_dialog.buttonInfo"));
	    buttonInfo.addActionListener(this);

	    Object[] options = { getMessage("security_dialog.buttonYes"),
				 getMessage("security_dialog.buttonNo"), 
				 getMessage("security_dialog.buttonAlways"),
				 buttonInfo };

	    // Show dialog
	    if (Trace.isAutomationEnabled() == false)
	    {
		ret = DialogFactory.showOptionDialog(dialogText, getMessage("security_dialog.caption"), options, options[0]);
	    }
	    else
	    {	// If automation is enabled
		Trace.securityPrintln("Automation: Trust RSA certificate for signing", TraceFilter.JAVA_CONSOLE_ONLY);
		ret = 0;
	    }
	}	    	

	return ret;	    	
    }

  
    /**
     * <P> Launch Certificate dialog if the "More Info" button is clicked.
     * </P>
     *
     * @param e ActionEvent object.
     */
    public void actionPerformed(ActionEvent e)  {
	CertificateDialog dialog = new CertificateDialog((JDialog) null, certs, 0);
	dialog.DoModal();			
    }

    /**
     * Method to get an internationalized string from the Activator resource.
     */
    private static String getMessage(String key)  {
	return sun.plugin.resources.ResourceHandler.getMessage(key);
    }
}


