/*
 * @(#)DialogFactory.java	1.21 01/02/09
 *
 * Copyright 1997-2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 *
 * Java Plug-in 
 *
 */

package sun.plugin.usability;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.lang.reflect.InvocationTargetException;
import java.util.ResourceBundle;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.PrivilegedExceptionAction;
import sun.plugin.resources.ResourceHandler;

/**
* DialogFactory provides confirmation, input, exception and message dialogboxes
*
* @version 1.0 
* @Author Devananda Jayaraman
**/

public class DialogFactory 
{
    // Icon to be used in the dialog
    private static ImageIcon icon = null;

    // Dialog title    
    private static String confirmDialogTitle;
    private static String inputDialogTitle;
    private static String messageDialogTitle;
    private static String exceptionDialogTitle;
    private static String optionDialogTitle;
    private static String aboutDialogTitle;


    static
    {
	try
	{
	    icon = (ImageIcon) AccessController.doPrivileged(
			new PrivilegedAction() 
			{
			    public Object run() 
			    {
				URL url = ClassLoader.getSystemResource("sun/plugin/usability/dukeWave.gif");

				return new ImageIcon(url);
			    }
			});
	}
	catch(Throwable e)
	{
	    e.printStackTrace();
	}
 
	confirmDialogTitle = ResourceHandler.getMessage("usability.confirmDialogTitle");
	inputDialogTitle = ResourceHandler.getMessage("usability.inputDialogTitle");
	messageDialogTitle = ResourceHandler.getMessage("usability.messageDialogTitle");
	exceptionDialogTitle = ResourceHandler.getMessage("usability.exceptionDialogTitle");
	optionDialogTitle = ResourceHandler.getMessage("usability.optionDialogTitle");
	aboutDialogTitle = ResourceHandler.getMessage("usability.aboutDialogTitle");
    }

    public static synchronized void showAboutPluginDialog()
    {  
	AccessController.doPrivileged(
	    new PrivilegedAction() 
	    {
		public Object run() 
		{
		    StringBuffer buffer = new StringBuffer();
		    buffer.append(System.getProperty("java.runtime.name") + " " + System.getProperty("java.version"));
		    buffer.append("\n");
		    buffer.append(ResourceHandler.getMessage("default_vm_version") + System.getProperty("java.vm.version"));
		    buffer.append("\n");
		    buffer.append(ResourceHandler.getMessage("product_name"));
		    buffer.append(" ");
		    buffer.append(System.getProperty("javaplugin.version"));

    		    JOptionPane pane = new JOptionPane();
		    pane.setIcon(icon);
		    pane.setOptionType(JOptionPane.DEFAULT_OPTION);
		    pane.setWantsInput(false);
		    pane.setMessage(buffer);
		    showDialog(pane, aboutDialogTitle, false);

		    return null;
		}
	    });
    }
    
    public static synchronized int showConfirmDialog(Object message)
    {
	return showConfirmDialog(message, confirmDialogTitle);
    }

    public static synchronized String showInputDialog(Object message)
    {
	return showInputDialog(message, inputDialogTitle);
    }

    public static synchronized void showMessageDialog(Object message)
    {
	showMessageDialog(message, messageDialogTitle);
    }

    public static synchronized void showExceptionDialog(Object message)
    {
	showExceptionDialog(message, exceptionDialogTitle);
    }

    public static synchronized int showConfirmDialog(Object message, String title)
    {
	Object yes = ResourceHandler.getMessage("usability.confirm.yes");
	Object no = ResourceHandler.getMessage("usability.confirm.no");

	Object[] options = { yes, no };

	if(title == null)
	    title = confirmDialogTitle;

	// Popup YES/NO dialog, 
	return showOptionDialog(message, title, options, options[0]);
    }

    public static synchronized int showOptionDialog(Object message, String title, Object[] options, Object initValue)
    {
	int selection = -1;

    	JOptionPane pane = new JOptionPane();
	pane.setIcon(icon);
	pane.setOptions(options);
	pane.setInitialValue(initValue);
	pane.setWantsInput(false);

	Object msg = extractMessage(pane, message);
	pane.setMessage(msg);

	if(title == null)
	    title = optionDialogTitle;

	if(showDialog(pane, title) == true)
	{
	    Object selectedValue = pane.getValue();
	    if(selectedValue != null)
	    {
		for (int i=0; i < options.length; i++)
		{
		    if (options[i].equals(selectedValue))
		    {
			selection = i;
			break;
		    }
		}
	    Trace.println("User selected: " + selection, TraceFilter.JAVA_CONSOLE_ONLY);
	    }
	}

	return selection;
    }

    public static synchronized String showInputDialog(Object message, String title)
    {
	String selection = null;
    	JOptionPane pane = new JOptionPane();
	pane.setIcon(icon);
	pane.setOptionType(JOptionPane.OK_CANCEL_OPTION);
	pane.setWantsInput(true);

	Object msg = extractMessage(pane, message);
	pane.setMessage(msg);

	if (title == null)
	    title = inputDialogTitle;

	if(showDialog(pane, title) == true)
	{
	    Object selectedValue = pane.getInputValue();
	    if(selectedValue != null)
	    {
    		if(selectedValue instanceof String)
		    selection = selectedValue.toString();
	    	Trace.println("User typed: " + selection, TraceFilter.JAVA_CONSOLE_ONLY);
	    }
	}

	return selection;
    }

    public static synchronized void showMessageDialog(Object message, String title, boolean fModal)    
    {
    	JOptionPane pane = new JOptionPane();
	pane.setIcon(icon);
	pane.setOptionType(JOptionPane.DEFAULT_OPTION);
	pane.setWantsInput(false);

	Object msg = extractMessage(pane, message);
	pane.setMessage(msg);

	if (title == null)
	    title = messageDialogTitle;

	showDialog(pane, title, fModal);
    }

    public static synchronized void showMessageDialog(Object message, String title)    
    {
		showMessageDialog(message, title, true);
    }

    public static synchronized void showExceptionDialog(Object message, String title)    
    {
	JOptionPane pane = new JOptionPane();
	pane.setIcon(icon);
	pane.setOptionType(JOptionPane.DEFAULT_OPTION);
	pane.setWantsInput(false);

	Object msg = extractMessage(pane, message);
	pane.setMessage(msg);

	if (title == null)
	    title = exceptionDialogTitle;

	showDialog(pane, title);
    }


    private static synchronized boolean showDialog(JOptionPane pane, String title)
    {
	return showDialog(pane, title, true);
    }

    private static synchronized boolean showDialog(final JOptionPane pane, final String title, final boolean fModal)
    {
	boolean dialogNoError = true;
	try
	{
	    AccessController.doPrivileged(
		new PrivilegedExceptionAction() 
		{
		    public Object run() throws InterruptedException, InvocationTargetException
		    {
			pane.setValue(null);
			Runnable work = new Runnable()
			{
			    public void run()
			    {
				final JDialog dialog = pane.createDialog(null, title);
				dialog.setResizable(false);
				dialog.setModal(fModal);

				// Add listener to usability dialog, so
				// resources will be freed.
				//
				dialog.addWindowListener(  new WindowAdapter(){
				    public void windowClosing(WindowEvent e) 
				    { 
					dialog.removeWindowListener(this);
					dialog.dispose(); 
				    }
				});

				dialog.show();
				dialog.toFront();
			    }
			};

			if (SwingUtilities.isEventDispatchThread() == false)
			{
			    if (fModal)
			    {
				SwingUtilities.invokeAndWait(work);
			    }
			    else
			    {
				SwingUtilities.invokeLater(work);
			    }
			}
			else
			{
			    // If we are already in event dispatch thread, just call it.
			    work.run();
			}

			return null;
		    }
		});
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    dialogNoError = false;
	}

	return dialogNoError;
    }


    /**
     * <P> Extract message from the object and formatted it.
     * </P>
     *
     * @param message Object
     * @param formatted object
     */
    private static synchronized Object extractMessage(JOptionPane pane, Throwable message)
    {
	return formatExceptionMessage(pane, message);
    }


    /**
     * <P> Extract message from the object and formatted it.
     * </P>
     *
     * @param message Object
     * @param formatted object
     */
    private static synchronized Object extractMessage(JOptionPane pane, String message)
    {
	return formatStringMessage(pane, message);
    }


    /**
     * <P> Extract message from the object and formatted it.
     * </P>
     *
     * @param message Object
     * @param formatted object
     */
    private static synchronized Object extractMessage(JOptionPane pane, Object[] messages)
    {
	Object[] result = null;

	if (messages != null)
	{
	    result = new Object[messages.length];

	    for (int i=0; i < messages.length; i++)
		result[i] = extractMessage(pane, messages[i]);
	}

	return result;
    }


    /**
     * <P> Extract message from the object and formatted it.
     * </P>
     *
     * @param message Object
     * @param formatted object
     */
    private static synchronized Object extractMessage(JOptionPane pane, Object message)
    {
	if (message instanceof Object[])
	    return extractMessage(pane, (Object[]) message);
	else if (message instanceof String)
	    return extractMessage(pane, (String) message);
	else if (message instanceof Exception)
	    return extractMessage(pane, (Exception) message);
	else
    	    return message;
    }

    private static synchronized Object formatStringMessage(JOptionPane pane, String in)
    {
	JTextArea text = new JTextArea("" + in);
	text.setColumns(40);
	text.setAutoscrolls(true);
	text.setEditable(false);
	text.setLineWrap(true);
	text.setWrapStyleWord(true);
	text.setBackground(pane.getBackground());

	return text;
    }

    private static synchronized Object formatExceptionMessage(JOptionPane pane, Throwable e)
    {
	StringWriter byt = new StringWriter();
	PrintWriter print = new PrintWriter(byt);
	e.printStackTrace(print);

	JTextArea text = new JTextArea("" + byt.toString());
	text.setColumns(40);
	text.setRows(10);
	text.setEditable(false);

	JScrollPane spane = new JScrollPane( text ); 
	text.setBackground(pane.getBackground());

	return spane;
    }
}
