/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-*/
/*
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/*
 * These functions create, initialize, and shutdown a plugin.
 */

#include "nsIPlugin.h"
#include "nsIJVMPlugin.h"

class JavaPluginFactory : public nsIJVMPlugin
{
public:
    // The Release method on nsIPlugin corresponds to NPP_Shutdown.

    // The old NPP_New call has been factored into two plugin instance methods:
    //
    // NewInstance -- called once, after the plugin instance is created. This 
    // method is used to initialize the new plugin instance (although the 
    // actual plugin instance object will be created by the plugin manager).
    //
    // NPIPluginInstance::Start -- called when the plugin instance is to be
    // started. This happens in two circumstances: 
    // (1) after the plugin instance is first initialized, and 
    // (2) after a plugin instance is returned to
    // (e.g. by going back in the window history) after previously 
    // being stopped by the Stop method. 

    NS_DECL_ISUPPORTS
    
    NS_IMETHOD CreateInstance(nsISupports *aOuter,
			      REFNSIID aIID,
			      void **result);
    NS_IMETHOD LockFactory(PRBool aLock);
    NS_IMETHOD Initialize(nsISupports* pluginMgr);
    NS_IMETHOD Shutdown(void);
    
    // (Corresponds to NPP_GetMIMEDescription.)
    NS_IMETHOD GetMIMEDescription(const char **resDesc);
    // (Corresponds to NPP_GetValue.)
    NS_IMETHOD GetValue(nsPluginVariable variable, void *value);

    // (Corresponds to NPP_SetValue.)
    NS_IMETHOD SetValue(nsPluginVariable variable, void *value);

    NS_IMETHOD  StartupJVM(nsJVMInitArgs *initargs);
    NS_IMETHOD  ShutdownJVM(PRBool fullShutdown = PR_FALSE);
    NS_IMETHOD  AddToClassPath(const char* dirPath);
    NS_IMETHOD  RemoveFromClassPath(const char* dirPath);
    NS_IMETHOD  GetClassPath(const char* *result);
    NS_IMETHOD  GetPluginInstance(jobject javaObject,
				  nsIPluginInstance* *result);
    NS_IMETHOD  GetPluginInstance(JNIEnv* jenv, nsIPluginInstance* *result);
    NS_IMETHOD  GetJavaWrapper(JNIEnv* jenv, jint jint, jobject* obj);
    NS_IMETHOD  GetJavaVM(JavaVM* *result);
    NS_IMETHOD_(nsrefcnt) GetJNIEnv(JNIEnv* *result);
    NS_IMETHOD_(nsrefcnt) ReleaseJNIEnv(JNIEnv* env);

    NS_IMETHOD_(void) CreateApplet(const char *appletType, int appletNumber, 
				   int argc, char **argn, char **argv);
    JavaPluginFactory(void);
    NS_IMETHOD_(int) GetNavigatorVersion();
    virtual ~JavaPluginFactory(void) {}

private:
    void* factory_delegate;


};


    
