/*
 * @(#)native.c	1.24 01/02/09
 * 
 *  Copyright 1998-2001 Sun Microsystems, Inc. 901 San Antonio Road, 
 * Palo Alto, California, 94303, U.S.A.  All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * CopyrightVersion 1.2
 * 
 */


/*
 * Native Methods to support Java Plug-in on Un*x systems.
 *
 * This gets compiled with -DJDK11 for 1.1 and -DJDK12 for 1.2
 *
 *						    KGH Dec 97
 */

#include "sun_plugin_navig_motif_Plugin.h"
#include "sun_plugin_navig_motif_Worker.h"
#include "sun_plugin_navig_motif_MotifAppletViewer.h"
#include "sun_plugin_ActivatorProxyHandler.h"
#include "sun_plugin_cachescheme_BookmarkManager.h"
#include "sun_plugin_JavaRunTime.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/param.h>
#include <fcntl.h>
#include <errno.h>
#if 0
#include <sys/socket.h>
#else
#include <stropts.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/VendorS.h>
#include "plugin_defs.h"
#include <dlfcn.h>


/* This is the one and only AWT header file allowed. */
/*#include "awt_Plugin.h"*/

/* AWT interface functions. */
static void (* LockIt)(JNIEnv *) = NULL;
static void (* UnLockIt)(JNIEnv *) = NULL;
static void (* NoFlushUnlockIt)(JNIEnv *) = NULL;
static void *res = NULL;
static initialized_lock = 0;
static Display *display;


static int trace_native = 1;
/*
 * Create a Java FileDescriptor object for a given Unix descriptor number
 *      fd is the file descriptor number
 */

#ifdef DO_TRACE
#define TRACE(i) fprintf(stderr, "%s\n", i);
#else
#define TRACE(i) 
#endif


static void *awtHandle = NULL;

static void initAwtHandle() {
    char awtPath[MAXPATHLEN];

    sprintf(awtPath, "%s/lib/" ARCH "/libawt%s.so",
	    getenv("JAVA_HOME"),
	    getenv("JAVA_PLUGIN_DEBUG") ? "_g" : "");

    awtHandle = dlopen(awtPath, RTLD_LAZY);
    if (awtHandle == NULL) {
	fprintf(stderr,"reflect - bad awtHandle.\n");
	exit(123);
    }
    return;
}


#define REFLECT_VOID_FUNCTION(name, arglist, paramlist)			\
typedef name##_type arglist;						\
static void name arglist						\
{									\
    static name##_type *name##_ptr = NULL;				\
    if (name##_ptr == NULL) {						\
        if (awtHandle == NULL) {					\
	    initAwtHandle();						\
	}								\
	name##_ptr = (name##_type *) dlsym(awtHandle, #name);		\
	if (name##_ptr == NULL) {					\
	    fprintf(stderr,"reflect failed to find " #name ".\n");	\
	    exit(123);							\
	    return;							\
	}								\
    }									\
    (*name##_ptr)paramlist;						\
}

#define REFLECT_FUNCTION(return_type, name, arglist, paramlist)		\
typedef return_type name##_type arglist;				\
static return_type name arglist						\
{									\
    static name##_type *name##_ptr = NULL;				\
    if (name##_ptr == NULL) {						\
        if (awtHandle == NULL) {					\
	    initAwtHandle();						\
	}								\
	name##_ptr = (name##_type *) dlsym(awtHandle, #name);		\
	if (name##_ptr == NULL) {					\
	    fprintf(stderr,"reflect failed to find " #name ".\n");	\
	    exit(123);							\
	    return;							\
	}								\
    }									\
    return (*name##_ptr)paramlist;					\
}


/*
 * These entry points must remain in libawt.so ***for Java Plugin ONLY***
 * Reflect this call over to the correct libmawt.so.
 */

REFLECT_VOID_FUNCTION(getAwtLockFunctions,
		      (void (**AwtLock)(JNIEnv *), void (**AwtUnlock)(JNIEnv *),
		       void (**AwtNoFlushUnlock)(JNIEnv *), void *reserved), 
		      (AwtLock, AwtUnlock, AwtNoFlushUnlock, reserved))

REFLECT_VOID_FUNCTION(getAwtData,
		      (int *awt_depth, Colormap *awt_cmap, Visual **awt_visual,
		       int *awt_num_colors, void *pReserved),
		      (awt_depth, awt_cmap, awt_visual,
		       awt_num_colors, pReserved))

REFLECT_FUNCTION(Display *, getAwtDisplay, (void), ())


/* Event Handler to correct for Shell position */
static void
checkPos(Widget w, XtPointer data, XEvent *event)
{
        /* this is heinous, but necessary as we need to update
        ** the X,Y position of the shell if netscape has moved.
        ** we have to do this so that XtTranslateCoords used by
        ** popups and the like get the proper screen positions
        ** Additionally we can use XtSet/ XtMove/ConfigureWidget
        ** As the widget code will think the the shell has moved
        ** and generate a XConfigure which WILL move the window
        ** We are only trying to correct for the reparent hack.
        ** sigh.
        */

        w->core.x = event->xcrossing.x_root - event->xcrossing.x;
        w->core.y = event->xcrossing.y_root - event->xcrossing.y;
}
/* Event Handler to correct for Shell position */
static void
propertyHandler(Widget w, XtPointer data, XEvent *event)
{
        /* this is heinous, but necessary as we need to update
        ** the X,Y position of the shell is changed to wrong value.
        ** we have to do this so that XtTranslateCoords used by
        ** popups and the like get the proper screen positions
        ** Additionally we can use XtSet/ XtMove/ConfigureWidget
        ** 
        */
  int px, py;
  Window *dummy;
  
  XTranslateCoordinates(display, XtWindow(w), DefaultRootWindow(display), 0,0, &px, &py, &dummy);
  
  w->core.x=px;
  w->core.y=py;
     
}
/*
 * Create a local managed widget inside a given X window.
 * We allocate a top-level shell and then reparent it into the
 * given window id.
 *
 * This is used to take the X11 window ID that has been passed
 * to us by our parent Navigator plugin and return a widget
 * that can be used as the base for our Java EmbeddeFrame.
 *
 * Note that the ordering of the various calls is tricky here as
 * we have to cope with the variations between 1.1.3, 1.1.6,
 * and 1.2.
 */
JNIEXPORT jint JNICALL
Java_sun_plugin_navig_motif_MotifAppletViewer_getWidget(
		JNIEnv *env, jclass clz,
		jint winid, jint width, jint height, jint x, jint y)
{
    Arg args[40];
    int argc;
    Widget w;
    Window child, parent;
    Visual *visual;
    Colormap cmap;
    int depth;
    int ncolors;
    Display **awt_display_ptr;

    /* Initialize access to AWT lock functions. */
    if (initialized_lock == 0) {
	getAwtLockFunctions(&LockIt, &UnLockIt, &NoFlushUnlockIt, NULL);
	initialized_lock = 1 ;
    }

    /*
     * Create a top-level shell.  Note that we need to use the
     * AWT's own awt_display to initialize the widget.  If we
     * try to create a second X11 display connection the Java
     * runtimes get very confused.
     */
    (*LockIt)(env);
    argc = 0;
    XtSetArg(args[argc], XmNsaveUnder, False); argc++;
    XtSetArg(args[argc], XmNallowShellResize, False); argc++;

    /* the awt initialization should be done by now (awt_GraphicsEnv.c) */

    getAwtData(&depth,&cmap,&visual,&ncolors,NULL);

    awt_display_ptr = (Display **) dlsym(awtHandle, "awt_display");
    if (awt_display_ptr == NULL)
	display = getAwtDisplay();
    else
	display = *awt_display_ptr;

    XtSetArg(args[argc], XmNvisual, visual); argc++;
    XtSetArg(args[argc], XmNdepth, depth); argc++;
    XtSetArg(args[argc], XmNcolormap, cmap); argc++;

    XtSetArg(args[argc], XmNwidth, width); argc++;
    XtSetArg(args[argc], XmNheight, height); argc++;
    /* The shell has to have relative coords of O,0? */
    XtSetArg(args[argc], XmNx, 0); argc++;
    XtSetArg(args[argc], XmNy, 0); argc++;

    /* The shell widget starts out as a top level widget.
     * Without intervention, it will be managed by the window
     * manager and will be its own widow. So, until it is reparented,
     *  we don't map it. 
     */
    XtSetArg(args[argc], XmNmappedWhenManaged, False); argc++;

    w = XtAppCreateShell("AWTapp","XApplication",
				    vendorShellWidgetClass,
				    display,
				    args,
				    argc);
    XtRealizeWidget(w);
    XtAddEventHandler(w, EnterWindowMask, FALSE,(XtEventHandler) checkPos, 0);
    XtAddEventHandler(w, PropertyChangeMask , FALSE,(XtEventHandler) propertyHandler, 0);
    /*
     * Now reparent our new Widget into our Navigator window
     */
    parent = (Window) winid;
    child = XtWindow(w);
    XReparentWindow(display, child, parent, 0, 0);
    XFlush(display);
    XSync(display, False); 
    XtVaSetValues(w, XmNx, 0, XmNy, 0, NULL);
    XFlush(display);
    XSync(display, False);

    (*UnLockIt)(env);	

    return (int)w;
}

/*
 * Make sure the given widget is mapped.
 *
 * This isn't necessary on JDK 1.1.5 but is needed on JDK 1.1.4
 */
JNIEXPORT jint JNICALL
Java_sun_plugin_navig_motif_MotifAppletViewer_mapWidget
(JNIEnv *env, jclass clz, jint widget, 
 jint wd, jint ht, jint x, jint y) 
{
    Widget w = (Widget)widget;
    /*
     * this is what JDK 1.1.5 does in MFramePeer.pShow.
     */
    (*LockIt)(env);
    XtSetMappedWhenManaged(w, True);
    XtPopup(w, XtGrabNone);
    (*UnLockIt)(env);
    return (jint) 1;
}


/**
 * Return true if our parent process is still alive.
 */
JNIEXPORT jboolean JNICALL
Java_sun_plugin_navig_motif_Plugin_parentAlive(JNIEnv *env, jclass clz)
{
    int ppid = getppid();

    if (ppid >= 0 && ppid < 4) {
	return JNI_FALSE;
    } else {
        return JNI_TRUE;
    }
}

/*
 * Retrieve an Environment variable.
 */
JNIEXPORT jstring JNICALL
Java_sun_plugin_navig_motif_Plugin_getenv(
		JNIEnv *env, jclass clz, jstring nameString)
{
    const char *name;  
    const char *res;
    jstring resString = 0;

    if (nameString == 0) {
	return NULL;
    }
    name = (*env)->GetStringUTFChars(env, nameString, 0);

    res = getenv(name);
    if (res != 0) {
	resString = (*env)->NewStringUTF(env, res);
    }

    (*env)->ReleaseStringUTFChars(env, nameString, name);

    return resString;
}


/*
 * Update an X11 server property for the given window.
 */
JNIEXPORT void JNICALL
Java_sun_plugin_navig_motif_Worker_setX11Property(JNIEnv *env, jclass clz,
		jint winid, jstring propertyString, jstring messString)
{
    const char *property = (*env)->GetStringUTFChars(env, propertyString, 0);
    const char *mess = (*env)->GetStringUTFChars(env, messString, 0);
    Atom atom;

    (*LockIt)(env);
    atom = XInternAtom(display, property, FALSE);

    XChangeProperty(display, winid, atom, XA_STRING, 8, PropModeReplace,
		    (const unsigned char *)mess, strlen(mess));
    XFlush(display);

    (*UnLockIt)(env);

    (*env)->ReleaseStringUTFChars(env, propertyString, property);
    (*env)->ReleaseStringUTFChars(env, messString, mess);
}



/*
 * to do the work.
 */
JNIEXPORT jstring JNICALL
Java_sun_plugin_ActivatorProxyHandler_findProxyForURL(JNIEnv *env,
					jobject obj, jstring url) 
{
    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth;
    meth = (*env)->GetStaticMethodID(env, clz, "getProxySettings",
			"(Ljava/lang/String;)Ljava/lang/String;");
    return (*env)->CallStaticObjectMethod(env, clz, meth, url);

}

/*
 * This is the native method for finding the cookie for a URL.
 * For the Solaris plugin we just trampoline back to java code
 * to do the work.
 */

JNIEXPORT jstring JNICALL
Java_sun_plugin_ActivatorCookieHandler_findCookieForURL(JNIEnv *env,
	jobject obj, jstring url) {
    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth;
    meth = (*env)->GetStaticMethodID(env, clz, "findCookieForURL",
			"(Ljava/lang/String;)Ljava/lang/String;");
    return (*env)->CallStaticObjectMethod(env, clz, meth, url);
}


JNIEXPORT jstring JNICALL
Java_sun_plugin_javascript_navig_JSObject_evalScript(JNIEnv *env, jobject obj,
						   jint instance, jstring s) {
    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Plugin");
    jmethodID meth;
    meth = (*env)->GetStaticMethodID(env, clz, "evalString",
				     "(ILjava/lang/String;)Ljava/lang/String;");
    return (*env)->CallStaticObjectMethod(env, clz, meth, instance, s);
}

JNIEXPORT jstring JNICALL
Java_sun_plugin_CacheHandler_getCacheFilename(JNIEnv *env, jobject obj,
					      jobject cr, jstring url) {
    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth;
    meth = (*env)->GetStaticMethodID(env, clz, "getCacheFileForURL",
				     "(Ljava/lang/String;)Ljava/lang/String;");
    return (*env)->CallStaticObjectMethod(env, clz, meth, url);
}


/*
 * Class:     sun_plugin_protocol_https_BrowserHttpsOutputStream
 * Method:    postHttpsURL
 * Signature: (Ljava/net/URLConnection;Ljava/lang/String;[BI)V
 */
JNIEXPORT jbyteArray JNICALL
Java_sun_plugin_protocol_https_BrowserHttpsOutputStream_postHttpsURL
(JNIEnv *env, jobject obj, jobject connection, 
 jstring url, jbyteArray buf, jint length)
{
    /* This method is called when the Https URL output stream is closed  */
    jbyteArray jPostBuffer = NULL;
    jbyteArray rawRequestHeaders        = NULL;
    unsigned char      *lpszRawRequestHeaders = NULL;
    unsigned char      *lpszRawPostData = NULL;
    unsigned char      *lpszHttpsRequest = NULL;
    char szHeader[256];
    jint headerLen = 0;
    jint totalLength = 0;
    jsize uRawHeadersLength=0;
    jmethodID meth, m;
    jclass clz;

    m = (*env)->GetMethodID(env, (*env)->GetObjectClass(env,connection),
                                            "getRequestHeaders", "()[B");
    if (m)
	rawRequestHeaders = (jbyteArray) (*env)->CallObjectMethod(env, connection, m);

    if (rawRequestHeaders) {
	uRawHeadersLength = (*env)->GetArrayLength(env, rawRequestHeaders);
	lpszRawRequestHeaders = (unsigned char *) (*env)->GetByteArrayElements(env, rawRequestHeaders, NULL);
    };


    /* Get Raw post data */
    lpszRawPostData = (unsigned char *) (*env)->GetByteArrayElements(env, buf, NULL);

    /* The following is a fix for Netscape Enterprise Server.
       Must prepend the Content-Length header in the POST request
       header in order to make Netscape Enterprise Server to work.
    */
    sprintf(szHeader,"Content-Length: %d\r\n", length);
    headerLen = (jint) strlen(szHeader);

    /* Allocate enough buffer to hold the HTTPS POST header and data */
    lpszHttpsRequest = (unsigned char *) malloc((headerLen + 
						 ( (uRawHeadersLength > 0) ? (uRawHeadersLength-1) : 1) +
						 length) * sizeof (unsigned char));

    memcpy(lpszHttpsRequest, szHeader, headerLen);
    totalLength = headerLen;

    if (uRawHeadersLength > 0)
      {
	memcpy(lpszHttpsRequest + headerLen, lpszRawRequestHeaders, uRawHeadersLength-2);
	totalLength += (uRawHeadersLength - 2);
      }


    /* Strip off trailing "\r\n" and append single "\n"  */
    *(lpszHttpsRequest + totalLength) = '\n';
    totalLength += 1;


    /* Copy the data into buffer */
    memcpy(lpszHttpsRequest + totalLength, lpszRawPostData, length);    
    totalLength += length;


    /* Allocate buffer so the data may be passed back to Java */
    jPostBuffer = (*env)->NewByteArray(env, totalLength);
    (*env)->SetByteArrayRegion(env, jPostBuffer, 0, totalLength, (jbyte*)lpszHttpsRequest);


    /* Release resources */
    (*env)->ReleaseByteArrayElements(env, rawRequestHeaders,
				     (jbyte*)lpszRawRequestHeaders, JNI_ABORT);
    (*env)->ReleaseByteArrayElements(env, buf,
				     (jbyte*)lpszRawPostData, JNI_ABORT);
    free(lpszHttpsRequest);
    lpszRawPostData = NULL;
    lpszRawRequestHeaders = NULL;
    lpszHttpsRequest = NULL;

    /* Pass back the processed POST request to the pipe and send it to
       the browser. */
    clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    meth = (*env)->GetStaticMethodID(env, clz, "postHttps",
			"(Ljava/lang/Object;Ljava/lang/String;[BI)[B");
    return (*env)->CallStaticObjectMethod(env, clz, meth, connection, url, 
				   jPostBuffer, totalLength);
}


JNIEXPORT jobject JNICALL
Java_sun_plugin_navig_motif_Plugin_getPipe(JNIEnv *env, jclass clz, jint fd)
{
    jobject result;
    jclass fd_clz = (*env)->FindClass(env, "java/io/FileDescriptor");
    jfieldID field = (*env)->GetFieldID(env, fd_clz, "fd", "I");

    result = (*env)->AllocObject(env, fd_clz);

    /* On JDK1.2 the fd field gets set to fd */
    (*env)->SetIntField(env, result, field, fd);

    return result;
}

/*
 * Class:     sun_plugin_protocol_https_BrowserHttpsInputStream
 * Method:    openStream
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_plugin_protocol_https_BrowserHttpsInputStream_openStream(JNIEnv *env, jobject is) {

    jclass isClass = (*env)->GetObjectClass(env, is);
    jfieldID urlConnID = (*env)->GetFieldID( env, isClass,
                       "urlConnection",
                       "Lsun/plugin/protocol/https/BrowserHttpsURLConnection;");
    jobject urlConn = (*env)->GetObjectField( env, is, urlConnID );

    jmethodID getURL = (*env)->GetMethodID(env,  (*env)->GetObjectClass( env, urlConn ),
                                          "getURL",
                                          "()Ljava/net/URL;");
    jobject url = (jobject) (*env)->CallObjectMethod( env, urlConn, getURL);
    jmethodID toString = (*env)->GetMethodID( env, (*env)->GetObjectClass( env, url ),
                                            "toString",
                                            "()Ljava/lang/String;");
    jstring urlStr = (jstring) (*env)->CallObjectMethod( env, url, toString);

    /* 
       It is extremely important to set the nativeConnection field in 
       sun/plugin/protocol/https/BrowserHttpsInputStream to represent a
       call is in progress. Otherwise, reading from the stream will fail.
     */

    jfieldID nativeConnID = (*env)->GetFieldID(env, isClass, "nativeConnection", "J");

    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth = (*env)->GetStaticMethodID(env, clz, "getHttps",
			"(Ljava/lang/Object;Ljava/lang/String;)V");

    (*env)->SetLongField(env, is, nativeConnID, (jlong)1);

    (*env)->CallStaticVoidMethod(env, clz, meth, is, urlStr);
}

/*
 * Class:     sun_plugin_protocol_https_BrowserHttpsInputStream
 * Method:    readStream
 * Signature: ([BII)I
 */
JNIEXPORT jint JNICALL 
Java_sun_plugin_protocol_https_BrowserHttpsInputStream_readStream
  (JNIEnv *env, jobject this, jbyteArray buf, jint offset, jint len) {

    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth = (*env)->GetStaticMethodID(env, clz, "readHttpsURL",
			"(Ljava/lang/Object;[BII)I");
    return (jint)(*env)->CallStaticObjectMethod(env, clz, meth, this, buf, offset, len);
}

/*
 * Class:     sun_plugin_protocol_https_BrowserHttpsInputStream
 * Method:    bytesAvailable
 * Signature: ()I
 */
JNIEXPORT jint JNICALL 
Java_sun_plugin_protocol_https_BrowserHttpsInputStream_bytesAvailable
  (JNIEnv *env, jobject this) {

    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth = (*env)->GetStaticMethodID(env, clz, "bytesAvailable",
			"(Ljava/lang/Object;)I");
    return (jint)(*env)->CallStaticObjectMethod(env, clz, meth, this);
}

/*
 * Class:     sun_plugin_protocol_https_BrowserHttpsInputStream
 * Method:    closeStream
 * Signature: ()V
 */
JNIEXPORT void JNICALL 
Java_sun_plugin_protocol_https_BrowserHttpsInputStream_closeStream
  (JNIEnv *env, jobject is) {

    jclass isClass = (*env)->GetObjectClass(env, is);
    jclass clz = (*env)->FindClass(env, "sun/plugin/navig/motif/Worker");
    jmethodID meth = (*env)->GetStaticMethodID(env, clz, "closeStream",
			"(Ljava/lang/Object;)V");
    jfieldID nativeConnID = (*env)->GetFieldID(env, isClass, "nativeConnection", "J");

    (*env)->CallStaticObjectMethod(env, clz, meth, is);

    /* 
       It is extremely important to unset the nativeConnection field in 
       sun/plugin/protocol/https/BrowserHttpsInputStream to represent a
       call is NOT in progress. So, reading from the stream after the stream
       is closed will fail.
    */
    (*env)->SetLongField(env, is, nativeConnID, 0);

    return;
}


/*
 * Class:     sun_plugin_cachescheme_BookmarkManager
 * Method:    nativeAddBookmark
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_sun_plugin_cachescheme_BookmarkManager_nativeAddBookmark
  (JNIEnv *env, jobject bookmark, jstring name, jstring file)
{
  /* Not implemented yet. */
}

/*
 * Class:     sun_plugin_JavaRunTime
 * Method:    ConsoleStatus
 * Signature: ()V
*/
JNIEXPORT void JNICALL Java_sun_plugin_JavaRunTime_ConsoleStatus(JNIEnv *a, jclass b, jboolean c)
{
  /* Not implemented yet. */
}






