/*
 * @(#)CJavaPluginInstance.cpp	1.13 01/02/09
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 * 
 * Author: Robert Sczwyczk, Benedict Gomes
 *   For the original version, see plugin.c in plugin1.1.1
 */

/* 
 * A Plugin instance is a single instantiation of the plugin, with its
 * own window and its own corresponding MotifAppletViewer. It will
 * share a classloader with other plugin instances that have the same
 * document base (there is a shared table of class loaders indexed by 
 * document base.
 */
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/VendorS.h>
#include "Navig4.h"
#include "CPluginPrint.h"


static const char *s_szDocument_locator_url = "javascript:document.location";

/*
 * Indicates whether this plugin instance can be used.
 */
int CJavaPluginInstance::Valid(void) {
    return (m_iPlugin_number >= 0);
}

NS_DEFINE_IID(kPluginInstanceIID, NS_IPLUGININSTANCE_IID);
NS_DEFINE_IID(kSupportsIID, NS_ISUPPORTS_IID);
NS_DEFINE_IID(kPluginTagInfoIID, NS_IPLUGINTAGINFO_IID);
NS_DEFINE_IID(kPluginTagInfo2IID, NS_IPLUGINTAGINFO2_IID);

/**
 * Standard QI. Supports EventHandler and PluginInstance.
 */
nsresult 
CJavaPluginInstance::QueryInterface(REFNSIID aIID, void** aInstancePtr)  {

    trace("CJavaPluginInstance::QueryInterface\n");

    if (NULL == aInstancePtr)                                         
	return NS_ERROR_NULL_POINTER;  

    if (aIID.Equals(kPluginInstanceIID) || 
	aIID.Equals(kSupportsIID)) {
	    *aInstancePtr = (void*) this;
	    AddRef();

	    return NS_OK;		
    } else {
	return NS_NOINTERFACE;
    }
}

NS_IMPL_ADDREF(CJavaPluginInstance);
NS_IMPL_RELEASE(CJavaPluginInstance);

/*
 * Create a new instance - should only be called by the factory 
 */
CJavaPluginInstance::CJavaPluginInstance(CJavaPluginFactory *plugin) 
{
    trace("CJavaPluginInstance::Create\n");
    
    NS_INIT_REFCNT();
    m_iPlugin_number = -1;
    m_PluginFactory = plugin;
    baseURL = NULL;
    current_request = NULL;
    m_bIs_instance_alive = 1;
    mPeer = NULL;
    m_szHttps_data = NULL;
    m_iHttps_data_len = 0;
} 

/* 
 * Start this instance. Just propagate to the JVM. The start
 * may actually have to wait until other information such as
 * the document base etc. are available. See MotifAppletViewer.java
 * maybeInit()
 */
NS_IMETHODIMP 
CJavaPluginInstance::Start(void) 
{
    trace("[%d] CJavaPluginInstance::Start JAVA_PLUGIN_START\n", m_iPlugin_number);
    char buff[8];
    put_int(buff, 0, JAVA_PLUGIN_START);
    put_int(buff, 4, m_iPlugin_number);
    m_PluginFactory->SendRequest(buff, 8, FALSE);
    return NS_OK;
}


/* 
 * Nothing is done here.
 */
NS_IMETHODIMP 
CJavaPluginInstance::HandleEvent(nsPluginEvent* event, PRBool
					       *handled) {
    *handled = PR_FALSE;
    trace("[%d]Handle event %d\n", m_iPlugin_number, event);
    return NS_OK;
}

/* 
 * Get the peer NPIPluginInstancePeer for this NPIPluginInstance
 */
NS_IMETHODIMP 
CJavaPluginInstance::GetPeer(nsIPluginInstancePeer**resultingPeer) 
{
    trace("[%d] CJavaPluginInstance::GetPeer\n", m_iPlugin_number);
    *resultingPeer = mPeer;

    if (mPeer == NULL) 
	trace("CJavaPluginInstance::Peer is null! %d\n", m_iPlugin_number);

    mPeer->AddRef();
    return NS_OK;
}

/*
 * Get the value of some plugin instance variable.
 */
nsresult
CJavaPluginInstance::GetValue(nsPluginInstanceVariable var, void* val) {
    trace("CJavaPluginInstance::GetValue not yet implemented %d %d!",
	  (int) (&(var)), (int) val);
    return NS_OK;
}


/*
 * Initialization essentially consists of determining the document base.
 * This is different for 3.0, 4.0 and 5.0.
 * In 5.0, query the tag info
 * In 4.0 send a DOCBASE_QUERY to the browser
 * In 3.0 the information may be available as an argument, or
 *        otherwise send a DOCBASE_READ, which is handled in URLNotify.
 */
NS_IMETHODIMP 
CJavaPluginInstance::Initialize(nsIPluginInstancePeer *peer) {
    int i;
    const char *pluginType;
    unsigned short argc;
    char **argn;
    char **argv;
    nsIPluginTagInfo *tagInfo;
    nsIPluginTagInfo2 *extTagInfo;
    nsresult res;

    trace("CJavaPluginInstance::Initialize  %d \n",  (int) peer);
    peer->AddRef(); 
    mPeer = peer;

    // Initialize the proxy support
    m_PluginFactory->GetProxySupport()->ProxmapInitialize(this);

    // Determine the mime type
    if (NS_OK != (res = peer->GetMIMEType(&pluginType))){
	trace("InstanceInitialize: Could not get MIME type!\n");
	return res; 
    }
    
    // Get information about the plugin's tags in an argc, argv form
    peer->QueryInterface(kPluginTagInfoIID, (void **)&tagInfo);
    if (tagInfo != NULL) {
        tagInfo->GetAttributes(argc, (const char * const *&)argn, 
			       (const char * const*&) argv);
        tagInfo->Release();
    }

    // Register this instance
    m_iPlugin_number = m_PluginFactory->RegisterInstance(this);
    if (m_iPlugin_number < 0) {
	trace("Out of memory in registering instance!\n");
	return NS_ERROR_OUT_OF_MEMORY;
    }

    // Create the actual applet for this instance
    m_PluginFactory->CreateApplet(pluginType, m_iPlugin_number, 
				  argc, argn, argv);

    //  We need to figure out our document base.

    // Get the plugin tag info2
    res = peer->QueryInterface(kPluginTagInfo2IID, (void **)&extTagInfo);

    // If this is a 5.0 plugin
    if ((res == NS_OK) && (extTagInfo != NULL)) {
        // Netscape 5.0: we get explicitly get the codebase from the browser
	trace("Instance Initialize(%d). 5.0 codebase \n", m_iPlugin_number); 

	// Get the document base from the tags
        const char *docbase = NULL;
        if (NS_OK != (res = extTagInfo->GetDocumentBase(&docbase))) {
	    trace("[%d] Initialize. No docbase? \n", m_iPlugin_number); 
	    return res;
	}

	trace("[%d] Initialize. Docbase=%s \n", m_iPlugin_number, docbase);

        extTagInfo->Release();

	// Determine the directory portion of the document base
        if (docbase != NULL) {
            char *slash = strrchr((char*)docbase, '/');
            if (slash != NULL) {
                *slash = 0;
            }
            SetDocbase(docbase);
            return NS_OK;
        }
    }


    // If this is a 3.0 or older browser, we get the document base
    // either from the creation arguments (docbase or java_docbase),
    // or by opening a empty relative url that should return the
    // proper URL.
    if (m_PluginFactory->GetNavigatorVersion() <= 3) {

	trace("Instance Initialize(%d). <= Navig 3.0 \n", m_iPlugin_number);

        /* Check if a java_docbase is explicitely defined in the tag's args */
	char *docbase = NULL;
        for (i = 0; i < argc; i++) {
       	    char *name = argn[i];
	    if (strcasecmp(name, "docbase") == 0 || 
		strcasecmp(name, "java_docbase") == 0) {
		docbase = argv[i];
	    }
	}
	if (docbase != NULL) {
	    /* We have an explicit docbase. */
	    SetDocbase(docbase);
	} else {
	    trace("[%d]Instance Initialize. no docbase. Get nm\n", 
		  m_iPlugin_number);
	}
    } else {
        // Netscape 4.x
        // In order to locate our DOCBASE invoke the JavaScript method
        //  "document.location" via a "javascript:" URL.
	// We'll get the result back in NPP_Write.
	trace("Get Docubase in 4.0");
	nsIPluginManager*pm = m_PluginFactory->GetPluginManager();
	pm->GetURL(this, s_szDocument_locator_url, 0, 
		   (void *)JAVA_PLUGIN_DOCBASE_QUERY);
    }
    return NS_OK;
}


/* 
 *  Set the window parameters of this plugin instance. Called at
 *  creation and after a resize. 'window' holds dimension and other 
 * information
 */
NS_IMETHODIMP 
CJavaPluginInstance::SetWindow(nsPluginWindow* window) {

    nsPluginSetWindowCallbackStruct *ws_info;
    char buff[40];

    trace("CJavaPluginInstance::SetWindow JAVA_PLUGIN_WINDOW %d\n", 
	  m_iPlugin_number);

    if (window == NULL || window->window == NULL) {
        trace("SetWindow NULL");
        put_int(buff, 0, JAVA_PLUGIN_WINDOW);
        put_int(buff, 4, m_iPlugin_number);
        put_int(buff, 8, 0);
        m_PluginFactory->SendRequest(buff, 28, TRUE);
        return NS_OK;
    }

    // Determine the window dimensions.
    ws_info = (nsPluginSetWindowCallbackStruct *) window->ws_info;
    trace("%d: SetWindow  win=0x%X, x=%d, y=%d, wd=%d, ht=%d, dp= %d,"
	  " ws_info=0x%x\n",
	  m_iPlugin_number,
	  window->window, window->x, window->y, window->width,
	  window->height, ws_info->depth, ws_info);

    // Flush any pending X requests.  This is necessary because window updates
    // affecting the target window may not yet have reached the server, and
    // the javaplugin child process may get out of step.
    XSync(ws_info->display, 0);

    // forward the request to the server 
    put_int(buff, 0, JAVA_PLUGIN_WINDOW);
    put_int(buff, 4, m_iPlugin_number);
    put_int(buff, 8, (int)window->window);
    put_int(buff, 12, window->width);
    put_int(buff, 16, window->height);
    put_int(buff, 20, window->x);
    put_int(buff, 24, window->y);
    m_PluginFactory->SendRequest(buff, 28, TRUE);
    return NS_OK;
}

/*
 * Create a new stream for the browser to write data into.
 */
NS_IMETHODIMP 
CJavaPluginInstance::NewStream(nsIPluginStreamPeer* peer,
					     nsIPluginStream* *result) {

    trace("CJavaPluginInstance::NewStream %d\n", m_iPlugin_number);

    *result = new CJavaStream(this, peer);
    return NS_OK;
}
/*
 * Provides printing support 
 */

NS_IMETHODIMP CJavaPluginInstance::Print(nsPluginPrint* printInfo) {
    CPluginPrint *printer;
    int result;
    nsPluginEmbedPrint ep;
    ep = printInfo->print.embedPrint;
    nsPluginWindow * win= (nsPluginWindow*) &(ep.window);
    if (win == NULL){
	delete printer;
	return NS_OK;
    }
    printer = new CPluginPrint(this, printInfo );
    result= printer->Print();
    delete printer;
    return result;
}

/*
 * Notification as the result of a query into the browser i.e.
 * the return of a GetURL query.
 * The sequence of events is:
 *     GetURLNotify
 *     NewStream, 
 *     A sequence of writes to the stream
 *     Notify the instance that the writes are done
 * The notification happens *after* all writes to the stream
 * have completed.
 */
NS_IMETHODIMP 
CJavaPluginInstance::URLNotify(const char* url, 
			       const char*target, 
			       nsPluginReason reason, 
			       void* notifyData) {
    int key = (int)notifyData;
    if (tracing) {
	char head[31];
	int clen = slen(url);
	if (clen > 30) {
	    clen = 30;
	}
	memcpy(head, url, clen);
	head[clen] = 0;
	UNUSED(target);
        trace("%d: NPP_URLNotify: key=0x%X %s => %d\n", m_iPlugin_number,
	      key, head, reason);
	    
    }
    if (key == JAVA_PLUGIN_DOCBASE_READ) {    
	// In 3.0 we get the docbase through a JS request 
	// No longer functioning
	SetDocbase(url);
  } else if (key == JAVA_PLUGIN_GET_HTTPS) {
        /* All the writes are complete. 
           Send back the data and free the space */
        trace("Finishing HTTPS. %d\n", m_iHttps_data_len);
        char* buff = (char *) malloc(m_iHttps_data_len + 8);
        put_int(buff, 0, JAVA_PLUGIN_HTTPS_GET_REPLY);
        put_int(buff, 4, m_iHttps_data_len);
        memcpy(buff + 8, m_szHttps_data, m_iHttps_data_len);
	if (m_iHttps_data_len > 0) 
	    m_PluginFactory->SendRequest(buff, m_iHttps_data_len + 8, FALSE);

	if (buff) 
	    free(buff);
	if (m_szHttps_data) 
	    free(m_szHttps_data);

        m_iHttps_data_len = 0;
        m_szHttps_data = NULL;
    } else if (key ==  JAVA_PLUGIN_POST_HTTPS) {
        /* All the writes are complete.
           Send back the data and free the space */
        trace("Finishing HTTPS. %d\n", m_iHttps_data_len);
        char* buff = (char *) malloc(m_iHttps_data_len + 8);
        put_int(buff, 0, JAVA_PLUGIN_HTTPS_POST_REPLY);
        put_int(buff, 4, m_iHttps_data_len);
        memcpy(buff + 8, m_szHttps_data, m_iHttps_data_len);
        if (m_iHttps_data_len > 0)
            m_PluginFactory->SendRequest(buff, m_iHttps_data_len + 8, FALSE);

        if (buff)
            free(buff);
        if (m_szHttps_data)
            free(m_szHttps_data);

        m_iHttps_data_len = 0;
        m_szHttps_data = NULL;
    } else if (key == JAVA_PLUGIN_PROXMAP_INPUT) {
	// After a proxy mapping has been obtained, we can write it back
	m_PluginFactory->GetProxySupport()->ProxmapInputDone(reason);
    } else if (key == JAVA_PLUGIN_JAVASCRIPT_REQUEST) {
	// A general 4.0 JS request has just finished giving us all its data
	char buff[10];
	put_int(buff, 0, JAVA_PLUGIN_JAVASCRIPT_END);
	put_int(buff, 4, m_iPlugin_number);
	m_PluginFactory->SendRequest(buff, 8, FALSE);
    } else if (key == JAVA_PLUGIN_DOCBASE_QUERY) {
	// In 4.0 we get the docbase through a JS request.
    } else {
	trace("[%d] Other URLNotify %X \n", m_iPlugin_number, key);
    }
    return NS_OK;
}


/*
 * Return the associated plugin instance factory.
 */
CJavaPluginFactory * CJavaPluginInstance::GetPluginFactory(void){
    trace("[%d] Getting plugin factory\n", m_iPlugin_number);
    return m_PluginFactory;
}

/* Receive the reply to a Https query */
void CJavaPluginInstance::HttpsReply(const char* buffer, int length) 
{
    trace("[%d] CJavaPluginInstance::HttpsReply\n", m_iPlugin_number);

    char *destptr = NULL;

    trace("Reply to https length=%d\n", length);

    if (m_szHttps_data == NULL) {
        m_szHttps_data = (char*) malloc(length);
        destptr = m_szHttps_data;
    } else {
        /* Copy the old data over */
        char* old_https_data = m_szHttps_data;
        m_szHttps_data = (char *) malloc(length + m_iHttps_data_len);
        memcpy(m_szHttps_data, old_https_data, m_iHttps_data_len);
        destptr = m_szHttps_data + m_iHttps_data_len;
        free(old_https_data);
    }
    /* Copy the new data into the buffer */
    memcpy(destptr, buffer, length);

    m_iHttps_data_len += length;
}

/*
 * Set the document base for this instance to the url 'url'.
 * This is called either during initialization, if the document
 * base is obtained directly, or later during a call to URLNotify,
 * as the result of one of the asynch calls into the browser.
 * Send the data to the jvm, which might now have all the information
 * required to start an applet.
 */
void 
CJavaPluginInstance::SetDocbase(const char *url) 
{
    trace("[%d] CJavaPluginInstance::SetDocbase JAVA_PLUGIN_DOCBASE %s\n", 
	  m_iPlugin_number, url);

    int len = slen(url);
    char *buff = (char *)malloc(len+20);

    if (url == NULL) 
	plugin_error("Null Docbase?");

    int baseLength = strlen(url);
    baseURL = (char *) malloc(baseLength + 1);
    baseURL = strcpy(baseURL, url);
    
    put_int(buff, 0, JAVA_PLUGIN_DOCBASE);
    put_int(buff, 4, m_iPlugin_number);
    put_short(buff, 8, len);
    memcpy(buff+10, url, len);
    m_PluginFactory->SendRequest(buff, len+10, FALSE);
    free(buff);
}

/*
 * Reply to a javascript query. No call for this as yet?
 */
void 
CJavaPluginInstance::JavascriptReply(const char *reply) 
{
    trace("[%d] CJavaPluginInstance::JavascriptReply JAVA_PLUGIN_JAVASCRIPT_REPLY\n", m_iPlugin_number);

    int len = slen(reply);    
    char *buff = (char *)malloc(len + 20);
    put_int(buff, 0, JAVA_PLUGIN_JAVASCRIPT_REPLY);
    put_int(buff, 4, m_iPlugin_number);
    put_short(buff, 8, len);
    memcpy(buff+10, reply, len);
    m_PluginFactory->SendRequest(buff, len+10, FALSE);
    free(buff);
}

/*
 * Reply to a cookie query coming back from the browser. The request
 * originates from the JavaVM for the cookies associated with a particular
 * page.
 */
void 
CJavaPluginInstance::CookieReply(const char *cookie) {

    trace("[%d] CJavaPluginInstance::CookieReply JAVA_PLUGIN_COOKIE = %s\n", 
	  m_iPlugin_number, cookie);

    int len = slen(cookie);
    char *buff = (char *)malloc(len+20);
    put_int(buff, 0, JAVA_PLUGIN_COOKIE);
    put_int(buff, 4, m_iPlugin_number);
    put_short(buff, 8, len);
    memcpy(buff+10, cookie, len);
    m_PluginFactory->SendRequest(buff, len+10, FALSE);
    free(buff);
}

char* 
CJavaPluginInstance::GetDocbaseURL() 
{
    trace("[%d] CJavaPluginInstance::GetDocbaseURL\n", 
	  m_iPlugin_number);

    if (baseURL == NULL) {
        return (char *)"BADURL";
    } else {
        trace("Returning docbase for %d is %s\n", m_iPlugin_number, baseURL);
        return baseURL;
    }
}

int 
CJavaPluginInstance::isInstanceAlive() 
{
    trace("CJavaPluginInstance::isInstanceAlive\n");
    return m_bIs_instance_alive;
}

/* 
 * Stop this instance. Just propagate to the JVM.
 */
NS_IMETHODIMP 
CJavaPluginInstance::Stop(void) 
{
    trace("[%d] CJavaPluginInstance::Stop JAVA_PLUGIN_STOP\n",
	  m_iPlugin_number);

    char buff[8];
    put_int(buff, 0, JAVA_PLUGIN_STOP);
    put_int(buff, 4, m_iPlugin_number);
    m_PluginFactory->SendRequest(buff, 8, FALSE);
    return NS_OK;
}

/* 
 * Destroy this instance. Just propagate to the JVM.
 */
NS_IMETHODIMP 
CJavaPluginInstance::Destroy(void) 
{
    trace("[%d] CJavaPluginInstance::Destroy JAVA_PLUGIN_DESTROY\n",
          m_iPlugin_number);

    // Unregister the instance
    if (m_iPlugin_number >= 0)
    {
        m_PluginFactory->UnregisterInstance(this);
    }

    m_bIs_instance_alive = 0;

    // Tell the VM end to go die

    char buff[8];
    put_int(buff, 0, JAVA_PLUGIN_DESTROY);
    put_int(buff, 4, m_iPlugin_number);
    m_PluginFactory->SendRequest(buff, 8, TRUE);

    if (current_request != NULL) {
        trace("Destroying instance, abruptly terminating request!");
        char msg[4];
        put_int(msg, 0, JAVA_PLUGIN_REQUEST_ABRUPTLY_TERMINATED);
        m_PluginFactory->SendRequest(msg, 4, FALSE);
    }

    m_iPlugin_number = -1;


    // The object destructor will be called when refs go to zero
    trace("CJavaPluginInstance::Destroy done %d \n", m_iPlugin_number);
    return NS_OK;
}

/* 
 * Destroy a plugin instance. Called after release?
 */
CJavaPluginInstance::~CJavaPluginInstance(void) 
{
    trace("[%d] CJavaPluginInstance::~destructor\n", m_iPlugin_number);

    if (m_iPlugin_number >= 0)
	m_PluginFactory->UnregisterInstance(this);
    if (mPeer != NULL)
	mPeer->Release();
    m_PluginFactory = NULL;
    if (baseURL != NULL) 
	free(baseURL);
    
}

void CJavaPluginInstance::EnterRequest(const char* requestName) 
{
    trace("[%d] CJavaPluginInstance::EnterRequest: %s\n", 
	  m_iPlugin_number, requestName);
    current_request = requestName;
}

void CJavaPluginInstance::ExitRequest(const char* requestName) 
{
    trace("[%d] CJavaPluginInstance::ExitRequest: %s\n",
	  m_iPlugin_number, requestName);
    current_request = NULL;
}
