/*
 * @(#)CJavaPluginInstance.h	1.4 00/03/30
 * 
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/*
 * Contains declaration of the plugin instance, modified from the 
 * Win32 version
 */


#ifndef CJAVAPLUGININSTANCE_H
#define CJAVAPLUGININSTANCE_H

#include "nsPlugin.h"

class CJavaPluginFactory;

class CJavaPluginInstance : public nsIPluginInstance {

public:
    // Support for QI
    NS_DECL_ISUPPORTS

    // See comment for nsIPlugin::NewInstance, above.
    NS_IMETHOD Initialize(nsIPluginInstancePeer *peer);

    NS_IMETHOD Start(void);

    NS_IMETHOD GetPeer(nsIPluginInstancePeer* *resultingPeer);

    // The old NPP_Destroy call has been factored into two plugin instance 
    // methods:
    //
    // Stop -- called when the plugin instance is to be stopped (e.g. by 
    // displaying another plugin manager window, causing the page containing 
    // the plugin to become removed from the display).
    //
    // Release -- called once, before the plugin instance peer is to be 
    // destroyed. This method is used to destroy the plugin instance.
    NS_IMETHOD Stop(void);

    NS_IMETHOD Destroy(void);

    // (Corresponds to NPP_SetWindow.)
    NS_IMETHOD SetWindow(nsPluginWindow* window);

    // (Corresponds to NPP_NewStream.)
    NS_IMETHOD NewStream(nsIPluginStreamPeer* peer, nsIPluginStream* *result);

    // (Corresponds to NPP_Print.)
    NS_IMETHOD Print(nsPluginPrint* platformPrint);

    NS_IMETHOD GetValue(nsPluginInstanceVariable var, void* val);

    // Handle events. We don't.
    NS_IMETHOD HandleEvent(nsPluginEvent* event, PRBool *handled);

    // (Corresponds to NPP_URLNotify.)
    NS_IMETHOD URLNotify(const char* url, const char* target,
				nsPluginReason reason, void* notifyData);


    CJavaPluginInstance(CJavaPluginFactory *plugin);

    NS_IMETHOD_(int) Valid(void);

    // The following methods handle the return codes from various
    // GetURL requests
    NS_IMETHOD_(void) SetDocbase(const char *url);

    NS_IMETHOD_(void) JavascriptReply(const char *reply);

    NS_IMETHOD_(void) HttpsReply(const char *reply, int length);

    NS_IMETHOD_(void) CookieReply(const char *cookie);

    // Return the plugin factory associated with this instance
    CJavaPluginFactory * GetPluginFactory(void);

    virtual	~CJavaPluginInstance(void);
    
    // Return the index of this plugin
    int GetPluginNumber(void) { return m_iPlugin_number; } ;

      // Indicate to this instance that a request is in progress
    // and abrupt termination must be sent if this instance dies
    void EnterRequest(const char *requestName);

    void ExitRequest(const char *requestName);

    // Return true if destroy has not been called on this instance
    int isInstanceAlive();

    // Return the codebase associated with this instance. Returns
    // the string 'BADURL' on error
    char* GetDocbaseURL();

protected:
    // Browser side object
    nsIPluginInstancePeer*	mPeer;

    // Pointer to the factory
    CJavaPluginFactory *m_PluginFactory;

    // Index of this plugin in the plugin table
    int m_iPlugin_number;

    // Docbase url
    char* baseURL;

    // If non-null indicate that a request is in process
    const char* current_request;

    // Set to false as soon as a destroy is called on it.
    // The destroy will also send an abrupt termination if
    // that is needed.
    int m_bIs_instance_alive;

    // Space to store https reply data
    char *m_szHttps_data;

    // Current amount of data received
    int m_iHttps_data_len;
    
};


#endif 
