/*
 * @(#)CJavaStream.cpp	1.12 00/05/31
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 * Modified from Stanley Ho's Windows version by RS, BG.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "Navig4.h"

/*
 * Construct and initialize a new stream
 */
CJavaStream::CJavaStream(CJavaPluginInstance* pPluginInstance,
			 nsIPluginStreamPeer* peer):	
    CNetscapeStream(peer), m_PluginInstance(pPluginInstance), m_iCount(0)
{
    trace("CJavaStream::CJavaStream Creating a new stream\n");
    const char* szURL;
    peer->GetURL(&szURL);

    if (szURL == NULL) {

	m_szURL[0] = '\0';  // Patch for Opera Browser

    } else {

	strncpy(m_szURL, szURL, MAX_PATH);
    }
}


/*
 * Destroy the stream
 */
CJavaStream::~CJavaStream()
{
    trace("CJavaStream::~CJavaStream\n");
}

/*
 * StreamAsFile is called to return the filename in the cache.
 */
NS_IMETHODIMP CJavaStream::AsFile(const char *fname) {
    const char *nstring;
    int key;
    nsresult res;

    if (NS_OK != (res = (int)mStreamPeer->GetNotifyData((void **) &key))) 
	return res;

    if (fname == NULL) {
	nstring= "";
    } else {
	nstring = fname;
    }

    trace("StreamAsFile: %08x %s\n", key, nstring); 
    if (key == JAVA_PLUGIN_CACHE_QUERY) { 
	int len = slen(nstring);
	char *buff = (char *) malloc(len + 16);
	put_int(buff, 0, JAVA_PLUGIN_CACHED_FILE);
	put_short(buff, 4, len);
	memcpy(buff + 6, fname, len);
	trace("JavaStream::AsFile cache response len = %d\n", len);
	if (m_PluginInstance != NULL && m_PluginInstance->isInstanceAlive()){
	    m_PluginInstance->GetPluginFactory()
		->SendRequest(buff, len + 6, FALSE);
	    m_PluginInstance->ExitRequest("cache");
	}
	free(buff);
    }
    return NS_OK;
}


/* 
 * Write is called to handle a piece of data in the plugin stream.
 */
NS_IMETHODIMP CJavaStream::Write(const char* buffer, PRInt32 offset, PRInt32
				 len, PRInt32 *bytesWritten)  
{
    /* 
       Notice that the offset only represents where the buffer is in 
       the entire stream, NOT where the data is in the buffer. Therefore,
       offset should never never be used, because the buffer already 
       contains all the data we need. 
    */

    (void) offset; // defeat compiler "not used" warning.
    nsresult errorResult = NS_OK;
    int key;
    *bytesWritten = 0;

    // Get the notify key, which we use to indicate the type of request
    // from the stream peer 
    if (NS_OK != (errorResult = mStreamPeer->GetNotifyData((void **)&key)))
	return errorResult;

    // Dump the write request data
    if (tracing) {
	char head[81];
	int clen = slen(m_szURL);
	if (clen > 30) {
	    clen = 30;
	}
	memcpy(head, m_szURL, clen);
	head[clen] = 0;
        trace("CJavaStream::NPP_Write \"%s\" %d\n", head, len);
	if (len > 70) {	
	    memcpy(head, buffer, 70);
	    head[70] = 0;
            fprintf(stderr, "%s...\n", head);
	} else {
	    memcpy(head, buffer, len);
	    head[len] = 0;
            fprintf(stderr, "%s\n", head);
	}	
    }

    if (key == JAVA_PLUGIN_DOCBASE_QUERY) {
	// We've just executed a javascript command to find the docbase.
	// Send the result to the java process.
 	char *slash = (char*) strrchr((char *)buffer, '/');
 	if (slash != NULL) {
 	    *slash = 0;
 	}
	m_PluginInstance->SetDocbase(buffer);	
	return errorResult;

    } else if (key == JAVA_PLUGIN_PROXMAP_INPUT) {

	// The proxymapping initialization is done. 
	// The proxy support is notified that the data is available.
	m_PluginInstance->GetPluginFactory()->GetProxySupport()
	    ->ProxmapInput((char *) (buffer),len);

    } else if (key == JAVA_PLUGIN_PROXMAP_QUERY) {

	// Finished obtaining the mapping for a particular URL
	m_PluginInstance->GetPluginFactory()->GetProxySupport()
	    ->ProxmapReply(m_szURL, len, (void *)(buffer));

    } else if (key == JAVA_PLUGIN_COOKIE_QUERY) {

	// The result of a cookie query
	m_PluginInstance->CookieReply(buffer);

    } else if (key == JAVA_PLUGIN_JAVASCRIPT_REQUEST) {

	// A general java script reply has arrived
	m_PluginInstance->JavascriptReply(buffer);

    } else if (key == JAVA_PLUGIN_GET_HTTPS) {
	m_PluginInstance->HttpsReply(buffer, len);
    } else if (key == JAVA_PLUGIN_POST_HTTPS) {
      //m_PluginInstance->HttpsReply(buffer, len);
        m_PluginInstance->HttpsReply(buffer, len);
    } else {
        trace(" Other Stream Write %X \n",  key);
    }
    *bytesWritten = len;

    // Number of bytes accepted
    return errorResult;        
}

/*
 * Corresponds to NPP_NewStream's stype return parameter.
 */
NS_IMETHODIMP CJavaStream::GetStreamType(nsPluginStreamType *result)
{

    int key;
    nsresult res;

    trace("CJavaStream::GetStreamType\n");

    // Get the notify data
    if ( NS_OK != (res = mStreamPeer->GetNotifyData((void **) &key)))
	return res;

    // If this is a cache query, request the stream to be returned as
    // a file name
    if (key == JAVA_PLUGIN_CACHE_QUERY) {
	trace("Cache query. Request a file\n");
	*result =  nsPluginStreamType_AsFile;
	return NS_OK;
    }

    return CNetscapeStream::GetStreamType(result);
}

