/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-*/
/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


enum nsJVMStatus {
    nsJVMStatus_Enabled,  // but not Running
    nsJVMStatus_Disabled, // explicitly disabled
    nsJVMStatus_Running,  // enabled and started
    nsJVMStatus_Failed    // enabled but failed to start
};

struct LongTermState {
    /* This pipe is used for sending commands to the Java process */
    int command_pipe;
    /* This pipe is used for receiving work from the Java process */
    int work_pipe;
    /* This pipe is used for receiving printing from the VM */
    int print_pipe;

    /* The directory the Java Plug-in JRE is installed in */
    char *java_dir;
    /* The netscape version.  Typically 3 or 4 */
    int navigator_version;
    /* URL used for automatic proxy configuration (null if not used) */
    char *proxy_config_url;
    /* The proxy configuration script, for auto proxy configuration. */
    char *proxy_script;
    /* Process ID of child process */
    int child_pid;
    /* handle to dymanic library we've opened in 5.0 */
    void* handle;
};

class CJavaVM {
public:
    CJavaVM(CJavaPluginFactory *plugin);
    /* Initialize is separate from the constructor The constructor is
       called from the plugin manager constructor At that point, the
       version of navigator is not known.  The navigator version can
       only be obtained after the plugin factory initialization, when
       the plugin manager is provided.  At this point, we can decide
       whether or not to load libjni.so.  Hence, we have a separate
       Initialize method which is called from the PluginFactory
       initialization.
       */
    NS_IMETHOD_(void) Initialize(int navigator_version);
    /* Fork and exec the java_vm process, after setting up 
       the child environment*/
    NS_IMETHOD StartJavaVM(const char * addClasspath);
    NS_IMETHOD_(void) ShutdownJavaVM(int RealShutdown = 0);
    NS_IMETHOD_(void) ProcessWorkQueue(void);
    NS_IMETHOD_(void) SendRequest(char *buff, int len, int wait_for_reply);
    NS_IMETHOD_(void) CreateApplet(const char *appletType, int appletNumber, 
				   int argc, char **argn, char **argv);
    virtual ~CJavaVM();
    struct LongTermState *state;
    CJavaPluginFactory *GetPlugin(void);
    NS_IMETHOD_(PRBool)GetJVMEnabled(void);
    NS_IMETHOD_(nsJVMStatus)GetJVMStatus(void);
    NS_IMETHOD_(void)SetJVMEnabled(PRBool enable);
    NS_IMETHOD_(const char *)GetClassPath(void);
    NS_IMETHOD_(void) AddToClassPath(const char *dirPath);
    NS_IMETHOD_(nsrefcnt) GetJNIEnv(JNIEnv **result);
    NS_IMETHOD_(nsrefcnt) ReleaseJNIEnv(JNIEnv *env);
    JavaVM * GetJavaVM(void);
    NS_IMETHOD_(void) ReceivePrinting(FILE *fp);
    NS_IMETHOD_(void) TerminateRequestAbruptly(const char* msg);

protected:
    NS_IMETHOD_(void) DoWork(void);
    NS_IMETHOD_(void) SetUpClasspath(const char *);
    NS_IMETHOD_(char *) ReadWorkString(void);
    NS_IMETHOD_(char *) ReadWorkByteArray(void);
    NS_IMETHOD_(void) WorkError(int x);
    NS_IMETHOD_(void) Cleanup(void);
    NS_IMETHOD_(void) SetupChildEnvironment(void);
    NS_IMETHOD_(char *) FindJRE(void);
    NS_IMETHOD_(int) FindJVMType(void);
    NS_IMETHOD_(void) FindJavaDir(void);
    NS_IMETHOD_(int) FindEnabled(void);
    NS_IMETHOD_(void) AllocateEventWidget(void);
    // Sets the variable

private:
/* This variable is similarly set if the child is to use green threads.
   It is set by examining the environment variable THREADS_FLAG to see 
   if it is set to native. Otherwise green threads are used.
   A putenv of the secret environment variable _JVM_THREADS_TYPE to 
   green_threads is done after the fork if this variable is true
   */
    int child_uses_green_threads; // Initialized to -1

    nsJVMStatus mStatus;
    nsJVMStatus mBaseState;
    char * mClasspath;
    int mBufLen;
    int mClasspathLen;
    Widget event_widget;
    int event_window_id;
    CJavaPluginFactory  *mPlugin;
    int m_java_process_ready;

    char stateENVName[100];
};
