/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

//
// CPluginServiceProvider.h  by Stanley Man-Kit Ho
//
//
// Contains declaration of the CPluginServiceProvider
//

#ifndef CPLUGINSERVICEPROVIDER_H
#include "nsIServiceManager.h"

#define CPLUGINSERVICEPROVIDER_H


class CPluginServiceProvider {
public:

    // from nsIPluginServiceProvider:

    /**
     * Obtain a plugin service. An nsIPluginServiceProvider will obtain the 
     * plugin service according to the current browser version.
     *
     * @param clsid - the CLSID of the requested service.
     * @param iid - the IID of the request service.
     * @param result - the interface pointer of the requested service
     * @return - NS_OK if this operation was successful.
     */
    NS_IMETHOD
    QueryService(/*[in]*/  const nsCID& clsid, 
		 /*[in]*/  const nsIID& iid,
                 /*[out]*/ nsISupports* *result);


    /**
     * Release a plugin service. An nsIPluginServiceProvider will release the 
     * plugin service according to the current browser version.
     *
     * @param clsid - the CLSID of the service.
     * @param result - the interface pointer of the service
     * @return - NS_OK if this operation was successful.
     */
    NS_IMETHOD
    ReleaseService(/*[in]*/ const nsCID& clsid, 
		   /*[in]*/ nsISupports* pService);


    // CPluginServiceProvider specific methods:

    CPluginServiceProvider(nsISupports* pProvoder);
    virtual ~CPluginServiceProvider(void);

protected:
    nsIServiceManager* mService;
    nsISupports* mMgr;

    // For caching purpose
    nsISupports* mCapsManager; 
    nsISupports* mPluginManager;
    nsISupports* mJVMManager;
    nsISupports* mLiveConnect;
    nsISupports* mCookieStorage;
};

#endif // CPLUGINSERVICEPROVIDER_H


