/*
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

///=--------------------------------------------------------------------------=
//
// CSecureJNITester.cpp  by Stanley Man-Kit Ho
//
///=--------------------------------------------------------------------------=
//
// These functions create object, invoke method, get/set field in JNI with
// security context.
//

#include "nsPlugin.h"
#include "nsISecurityContext.h"
#include "nsISecureJNIEnv.h"
#include "remotejni.h"
#include "nsAgg.h"
#include "jni.h"
#include "Debug.h"

#define false 0
#define FALSE 0
#define true 1
#define TRACE(m) trace("CsecureJNITester %s\n", m);

static jclass      g_jclass_Class = NULL;
static jclass      g_jclass_Boolean = NULL;
static jclass      g_jclass_Byte = NULL;
static jclass      g_jclass_Character = NULL;
static jclass      g_jclass_Short = NULL;
static jclass      g_jclass_Integer = NULL;
static jclass      g_jclass_Long = NULL;
static jclass      g_jclass_Float = NULL;
static jclass      g_jclass_Double = NULL;
static jclass      g_jclass_Void = NULL;
static jmethodID   g_jmethod_Boolean_booleanValue = NULL;
static jmethodID   g_jmethod_Byte_byteValue = NULL;
static jmethodID   g_jmethod_Character_charValue = NULL;
static jmethodID   g_jmethod_Short_shortValue = NULL;
static jmethodID   g_jmethod_Integer_intValue = NULL;
static jmethodID   g_jmethod_Long_longValue = NULL;
static jmethodID   g_jmethod_Float_floatValue = NULL;
static jmethodID   g_jmethod_Double_doubleValue = NULL;
static jmethodID   g_jmethod_Boolean_init = NULL;
static jmethodID   g_jmethod_Byte_init = NULL;
static jmethodID   g_jmethod_Character_init = NULL;
static jmethodID   g_jmethod_Short_init = NULL;
static jmethodID   g_jmethod_Integer_init = NULL;
static jmethodID   g_jmethod_Long_init = NULL;
static jmethodID   g_jmethod_Float_init = NULL;
static jmethodID   g_jmethod_Double_init = NULL;


void Initialize(nsISecureEnv* env, RemoteJNIEnv* m_env)
{
    fprintf(stderr, "Secure env %d\n", (int) env);
    g_jclass_Class      = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Class"));
    g_jclass_Boolean    = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Boolean"));
    g_jclass_Byte       = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Byte"));
    g_jclass_Character  = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Character"));
    g_jclass_Short      = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Short"));
    g_jclass_Integer    = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Integer"));
    g_jclass_Long       = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Long"));
    g_jclass_Float      = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Float"));
    g_jclass_Double     = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Double"));
    g_jclass_Void       = (jclass) m_env->NewGlobalRef(m_env->FindClass("java/lang/Void"));

    g_jmethod_Boolean_booleanValue  = m_env->GetMethodID(g_jclass_Boolean, "booleanValue", "()Z");
    g_jmethod_Byte_byteValue        = m_env->GetMethodID(g_jclass_Byte, "byteValue", "()B");
    g_jmethod_Character_charValue   = m_env->GetMethodID(g_jclass_Character, "charValue", "()C");
    g_jmethod_Short_shortValue      = m_env->GetMethodID(g_jclass_Short, "shortValue", "()S");
    g_jmethod_Integer_intValue      = m_env->GetMethodID(g_jclass_Integer, "intValue", "()I");
    g_jmethod_Long_longValue        = m_env->GetMethodID(g_jclass_Long, "longValue", "()J");
    g_jmethod_Float_floatValue      = m_env->GetMethodID(g_jclass_Float, "floatValue", "()F");
    g_jmethod_Double_doubleValue    = m_env->GetMethodID(g_jclass_Double, "doubleValue", "()D");
    fprintf(stderr, "Initializing g_jmethod_Boolean_init\n");
    g_jmethod_Boolean_init      = m_env->GetMethodID(g_jclass_Boolean, "<init>", "(Z)V");
    g_jmethod_Byte_init         = m_env->GetMethodID(g_jclass_Byte, "<init>", "(B)V");
    g_jmethod_Character_init    = m_env->GetMethodID(g_jclass_Character, "<init>", "(C)V");
    g_jmethod_Short_init        = m_env->GetMethodID(g_jclass_Short, "<init>", "(S)V");
    g_jmethod_Integer_init      = m_env->GetMethodID(g_jclass_Integer, "<init>", "(I)V");
    g_jmethod_Long_init         = m_env->GetMethodID(g_jclass_Long, "<init>", "(J)V");
    g_jmethod_Float_init        = m_env->GetMethodID(g_jclass_Float, "<init>", "(F)V");
    g_jmethod_Double_init       = m_env->GetMethodID(g_jclass_Double, "<init>", "(D)V");
}


void compare_print(jni_type type, jvalue v, const char* str, 
		   const char* expected)
{
    char temp[256];

    fprintf(stderr, "TEST RESULT>>>>\n");
    fprintf(stderr, "Printing field value. Expecting %s\n", expected);

    switch (type)
    {
    case jboolean_type: sprintf(temp, "%s = %s\n", str, (v.z ? "true" : "false")); break;
    case jbyte_type: sprintf(temp, "%s = %d\n", str, v.b); break;
    case jchar_type: sprintf(temp, "%s = %c\n", str, v.c); break;
    case jshort_type: sprintf(temp, "%s = %d\n", str, v.s); break;
    case jint_type: sprintf(temp, "%s = %d\n", str, v.i); break;
    case jlong_type: sprintf(temp, "%s = %d\n", str, (int) v.j); break;
    case jfloat_type: sprintf(temp, "%s = %f\n", str, v.f); break;
    case jdouble_type: sprintf(temp, "%s = %g\n", str, v.d); break;
    case jvoid_type: sprintf(temp, "%s = void\n", str); break;
    case jobject_type: sprintf(temp, "%s = Object\n", str); break;
    default:    sprintf(temp, "Error type\n"); 
    }

    TRACE(temp);
}

void print(jni_type type, jvalue v, const char* str)
{
    char temp[256];
    
    fprintf(stderr, "Printing field value\n");

    switch (type)
    {
    case jboolean_type: sprintf(temp, "%s = %s\n", str, (v.z ? "true" : "false")); break;
    case jbyte_type: sprintf(temp, "%s = %d\n", str, v.b); break;
    case jchar_type: sprintf(temp, "%s = %c\n", str, v.c); break;
    case jshort_type: sprintf(temp, "%s = %d\n", str, v.s); break;
    case jint_type: sprintf(temp, "%s = %d\n", str, v.i); break;
    case jlong_type: sprintf(temp, "%s = %d\n", str, (int) v.j); break;
    case jfloat_type: sprintf(temp, "%s = %f\n", str, v.f); break;
    case jdouble_type: sprintf(temp, "%s = %g\n", str, v.d); break;
    case jvoid_type: sprintf(temp, "%s = void\n", str); break;
    case jobject_type: sprintf(temp, "%s = Object\n", str); break;
    default:    sprintf(temp, "Error type\n"); 
    }

    TRACE(temp);
}

void PrintFieldMain(nsISecureEnv* env, jclass clazz, jobject tester)
{
    jmethodID m = NULL;

    env->GetMethodID(clazz, "i_print", "()V", &m);
    env->CallMethod(jvoid_type, tester, m, NULL, NULL);
}


void PrintStaticFieldMain(nsISecureEnv* env, jclass clazz)
{
    jmethodID m = NULL;

    env->GetStaticMethodID(clazz, "s_print", "()V", &m);
    env->CallStaticMethod(jvoid_type, clazz, m, NULL, NULL);
}

void GetInstanceField(nsISecureEnv* env, jclass clazz, jobject tester, 
		      jni_type type, 
                      const char* fieldName, const char* sig, const char* res)
{
    jfieldID f = NULL;
    jvalue result;

    env->GetFieldID(clazz, fieldName, sig, &f);
    fprintf(stderr, "GotFieldID for %s sig %s id = %d\n", fieldName, sig,
	    (int) f);
    env->GetField(type, tester, f, &result);
    compare_print(type, result, fieldName, res);
}


void SetInstanceField(nsISecureEnv* env, jclass clazz, jobject tester, 
		      jni_type type, 
                      const char* fieldName, const char* sig, jvalue val)
{
    jfieldID f = NULL;

    env->GetFieldID(clazz, fieldName, sig, &f);
    fprintf(stderr, "SetInstanceField Got field ID for %s %s: %d\n", 
	    fieldName, sig, (int) f);
    env->SetField(type, tester, f, val);
    fprintf(stderr, "SetInstanceField Set field for %s \n", fieldName);
}

/* Get object fields that happen to be of wrapped primitive types
   (Boolean etc.) to make sure there is no confusion in conversions */
void GetInstanceField2(nsISecureEnv* env, jclass clazz, jobject tester, 
		       jni_type type, 
                       const char* fieldName, const char* sig,
		       const char* expectedRes)
{
    jfieldID f = NULL;
    jvalue result, field_result;

    env->GetFieldID(clazz, fieldName, sig, &f);
    fprintf(stderr, "GetInstanceField2. Got the field id %s %s %d\n", 
	    fieldName, sig, (int) f);
    env->GetField(jobject_type, tester, f, &field_result);


    jobject field_obj = NULL;
    //  env->NewGlobalRef(result.l, &obj);
    field_obj = field_result.l;

    /* Now unwrap the result */
    switch (type)
    {
    case jboolean_type: {
	env->CallMethod(jboolean_type, field_obj, 
			g_jmethod_Boolean_booleanValue, NULL, &result);
	break;
    }
    case jbyte_type: 
	env->CallMethod(jbyte_type, field_obj, 
			g_jmethod_Byte_byteValue, NULL, &result);
	break;
    case jchar_type: env->CallMethod(jchar_type, field_obj,
				     g_jmethod_Character_charValue,
				     NULL, &result);
                     break;
    case jshort_type: env->CallMethod(jshort_type, field_obj, 
				      g_jmethod_Short_shortValue, 
				      NULL, &result);
                      break;
    case jint_type: env->CallMethod(jint_type, field_obj, 
				    g_jmethod_Integer_intValue, 
				    NULL, &result);
                    break;
    case jlong_type: env->CallMethod(jlong_type, field_obj, 
				     g_jmethod_Long_longValue, NULL, &result);
                     break;
    case jfloat_type: env->CallMethod(jfloat_type, field_obj, 
				      g_jmethod_Float_floatValue, 
				      NULL, &result);
                      break;
    case jdouble_type: env->CallMethod(jdouble_type, field_obj, 
				       g_jmethod_Double_doubleValue, 
				       NULL, &result);
                       break;
    case jvoid_type: return;
    case jobject_type: return;
    default:    {}
    }

    compare_print(type, result, fieldName, expectedRes);
}


void banner(const char* msg) {
    fprintf(stderr, "****************************************************\n");
    fprintf(stderr, "%s\n", msg);
    fprintf(stderr, "****************************************************\n");

}

void GetInstanceFieldMain(nsISecureEnv* env, jclass clazz, jobject tester)
{
    banner("GetInstanceFieldMain");

    GetInstanceField(env, clazz, tester, jint_type, "m_i_int", "I", "999");
    GetInstanceField(env, clazz, tester, jboolean_type, "m_i_boolean", "Z", 
		     "true");
    GetInstanceField(env, clazz, tester, jbyte_type, "m_i_byte", "B", "10");
    GetInstanceField(env, clazz, tester, jchar_type, "m_i_char", "C", "f");
    GetInstanceField(env, clazz, tester, jshort_type, "m_i_short", "S", "68");
    GetInstanceField(env, clazz, tester, jlong_type, "m_i_long", "J", 
		     "1000001");
    GetInstanceField(env, clazz, tester, jfloat_type, "m_i_float", "F",
		     "8.23423");
    GetInstanceField(env, clazz, tester, jdouble_type, "m_i_double", "D",
		     "3452.234234232");
    GetInstanceField2(env, clazz, tester, jint_type, "m_i_Integer", 
		      "Ljava/lang/Integer;", "1056");
    GetInstanceField2(env, clazz, tester, jbyte_type, "m_i_Byte", 
		      "Ljava/lang/Byte;", "20");

    GetInstanceField2(env, clazz, tester, jchar_type, "m_i_Character", 
		      "Ljava/lang/Character;","w");
    GetInstanceField2(env, clazz, tester, jshort_type, "m_i_Short",
		      "Ljava/lang/Short;","101");
    
    GetInstanceField2(env, clazz, tester, jlong_type, "m_i_Long", 
		      "Ljava/lang/Long;", "4000005");
    GetInstanceField2(env, clazz, tester, jfloat_type, "m_i_Float", 
		      "Ljava/lang/Float;", "56.29873");
    GetInstanceField2(env, clazz, tester, jdouble_type, "m_i_Double", 
		      "Ljava/lang/Double;", " 6432.34507897");
    GetInstanceField2(env, clazz, tester, jboolean_type, "m_i_Boolean", 
		      "Ljava/lang/Boolean;", "true");
    banner("Done with GetInstanceFieldMain");
}

void SetInstanceFieldMain(nsISecureEnv* env, jclass clazz, jobject tester)
{
    jvalue val;

     val.b = 51;
     SetInstanceField(env, clazz, tester, jbyte_type, "m_i_byte", "B", val);
    val.z = false;
    SetInstanceField(env, clazz, tester, jboolean_type, "m_i_boolean", "Z", 
 		     val);
     val.c = 'a';
     SetInstanceField(env, clazz, tester, jchar_type, "m_i_char", "C", val);
     val.s = 97;
     SetInstanceField(env, clazz, tester, jshort_type, "m_i_short", "S", val);
     val.i = 2061;
     SetInstanceField(env, clazz, tester, jint_type, "m_i_int", "I", val);
     val.j = 102345;
     SetInstanceField(env, clazz, tester, jlong_type, "m_i_long", "J", val);
     val.f = (float)123.454;
     SetInstanceField(env, clazz, tester, jfloat_type, "m_i_float", "F", val);
     val.d = 987.232;
     SetInstanceField(env, clazz, tester, jdouble_type, "m_i_double", "D", val);

    jvalue arg;

    arg.b = 73;
    env->NewObject(g_jclass_Byte, g_jmethod_Byte_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Byte", "Ljava/lang/Byte;", val);
    GetInstanceField2(env, clazz, tester, jbyte_type, "m_i_Byte", 
		      "Ljava/lang/Byte;", "73");

    arg.z = true;
    env->NewObject(g_jclass_Boolean, g_jmethod_Boolean_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Boolean", "Ljava/lang/Boolean;", val);


    arg.c = 'r';
    env->NewObject(g_jclass_Character, g_jmethod_Character_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Character", "Ljava/lang/Character;", val);
    arg.s = 43;
    env->NewObject(g_jclass_Short, g_jmethod_Short_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Short", "Ljava/lang/Short;", val);
    arg.i = 911;
    env->NewObject(g_jclass_Integer, g_jmethod_Integer_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Integer", "Ljava/lang/Integer;", val);
    arg.j = 34931;
    env->NewObject(g_jclass_Long, g_jmethod_Long_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Long", "Ljava/lang/Long;", val);
    arg.f = (float)632.3121;
    env->NewObject(g_jclass_Float, g_jmethod_Float_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Float", "Ljava/lang/Float;", val);
    arg.d = 13632.3121;
    env->NewObject(g_jclass_Double, g_jmethod_Double_init, &arg, &val.l);
    SetInstanceField(env, clazz, tester, jobject_type, "m_i_Double", "Ljava/lang/Double;", val);
    banner("Done with set instance field");
    PrintFieldMain(env, clazz, tester);
}

void GetStaticField(nsISecureEnv* env, jclass clazz, jni_type type, 
		    const char* fieldName, const char* sig,
		    const char* expected)
{
    jfieldID f = NULL;
    jvalue result;
    
    fprintf(stderr, "GetStaticField %s expect=%s\n", fieldName,
	    expected);
    env->GetStaticFieldID(clazz, fieldName, sig, &f);
    fprintf(stderr, "GetStaticField got ID\n");
    env->GetStaticField(type, clazz, f, &result);
    fprintf(stderr, "GetStaticField got field\n");
    compare_print(type, result, fieldName, expected);
}


void SetStaticField(nsISecureEnv* env, jclass clazz, jni_type type, 
                    const char* fieldName, const char* sig, jvalue val)
{
    jfieldID f = NULL;

    env->GetStaticFieldID(clazz, fieldName, sig, &f);
    env->SetStaticField(type, clazz, f, val);
}

void GetStaticField2(nsISecureEnv* env, jclass clazz, jni_type type, 
		     const char* fieldName, const char* sig,
		     const char* expected)
{
    jfieldID f = NULL;
    jvalue result;

    env->GetStaticFieldID(clazz, fieldName, sig, &f);
    env->GetStaticField(jobject_type, clazz, f, &result);

    jobject obj = NULL;
  //  env->NewGlobalRef(result.l, &obj);
    obj = result.l;

    switch (type)
    {
    case jboolean_type: 
	env->CallMethod(jboolean_type, obj, 
			g_jmethod_Boolean_booleanValue, NULL, &result);
                        break;
    case jbyte_type: env->CallMethod(jbyte_type, obj, g_jmethod_Byte_byteValue, NULL, &result);
                     break;
    case jchar_type: env->CallMethod(jchar_type, obj, g_jmethod_Character_charValue, NULL, &result);
                     break;
    case jshort_type: env->CallMethod(jshort_type, obj, g_jmethod_Short_shortValue, NULL, &result);
                      break;
    case jint_type: env->CallMethod(jint_type, obj, g_jmethod_Integer_intValue, NULL, &result);
                    break;
    case jlong_type: env->CallMethod(jlong_type, obj, g_jmethod_Long_longValue, NULL, &result);
                     break;
    case jfloat_type: env->CallMethod(jfloat_type, obj, g_jmethod_Float_floatValue, NULL, &result);
                      break;
    case jdouble_type: env->CallMethod(jdouble_type, obj, g_jmethod_Double_doubleValue, NULL, &result);
                       break;
    case jvoid_type: return;
    case jobject_type: return;
    default:    {}
    }

    compare_print(type, result, fieldName, expected);
}


void GetStaticFieldMain(nsISecureEnv* env, jclass clazz)
{
    GetStaticField(env, clazz, jboolean_type, "m_s_boolean", "Z","false");
    GetStaticField(env, clazz, jbyte_type, "m_s_byte", "B","-10");
    GetStaticField(env, clazz, jchar_type, "m_s_char", "C","t");
    GetStaticField(env, clazz, jshort_type, "m_s_short", "S","-68");
    GetStaticField(env, clazz, jint_type, "m_s_int", "I","-999");
    GetStaticField(env, clazz, jlong_type, "m_s_long", "J","-10000001");
    GetStaticField(env, clazz, jfloat_type, "m_s_float", "F","-8.23423");
    GetStaticField(env, clazz, jdouble_type, "m_s_double", "D","-3452.234234");
 
    GetStaticField2(env, clazz, jboolean_type, "m_s_Boolean", 
		    "Ljava/lang/Boolean;","true");
    GetStaticField2(env, clazz, jbyte_type, "m_s_Byte", "Ljava/lang/Byte;",
		    "-20");
    GetStaticField2(env, clazz, jchar_type, "m_s_Character", 
		    "Ljava/lang/Character;", "q");
    GetStaticField2(env, clazz, jshort_type, "m_s_Short", 
		    "Ljava/lang/Short;","-101");
    GetStaticField2(env, clazz, jint_type, "m_s_Integer", 
		    "Ljava/lang/Integer;","-1056");
    GetStaticField2(env, clazz, jlong_type, "m_s_Long", 
		    "Ljava/lang/Long;","-4000005");
    GetStaticField2(env, clazz, jfloat_type, "m_s_Float", 
		    "Ljava/lang/Float;", "-56.29873");
    GetStaticField2(env, clazz, jdouble_type, "m_s_Double", 
		    "Ljava/lang/Double;", "-6432.34507897");
}

void SetStaticFieldMain(nsISecureEnv* env, jclass clazz)
{
    jvalue val;

    val.b = 33;;
    SetStaticField(env, clazz, jbyte_type, "m_s_byte", "B", val);
    val.z = true;
    SetStaticField(env, clazz, jboolean_type, "m_s_boolean", "Z", val);
    val.c = 'x';
    SetStaticField(env, clazz, jchar_type, "m_s_char", "C", val);
    val.s = -3;
    SetStaticField(env, clazz, jshort_type, "m_s_short", "S", val);
    val.i = 652;
    SetStaticField(env, clazz, jint_type, "m_s_int", "I", val);
    val.j = 490211;
    SetStaticField(env, clazz, jlong_type, "m_s_long", "J", val);
    val.f = (float)102.934;
    SetStaticField(env, clazz, jfloat_type, "m_s_float", "F", val);
    val.d = 7123.214;
    SetStaticField(env, clazz, jdouble_type, "m_s_double", "D", val);

    jvalue arg;

    arg.z = false;
    env->NewObject(g_jclass_Boolean, g_jmethod_Boolean_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Boolean", "Ljava/lang/Boolean;", val);
    arg.b = -11;
    env->NewObject(g_jclass_Byte, g_jmethod_Byte_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Byte", "Ljava/lang/Byte;", val);
    arg.c = 'a';
    env->NewObject(g_jclass_Character, g_jmethod_Character_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Character", "Ljava/lang/Character;", val);
    arg.s = 53;
    env->NewObject(g_jclass_Short, g_jmethod_Short_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Short", "Ljava/lang/Short;", val);
    arg.i = 1021;
    env->NewObject(g_jclass_Integer, g_jmethod_Integer_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Integer", "Ljava/lang/Integer;", val);
    arg.j = 53452;
    env->NewObject(g_jclass_Long, g_jmethod_Long_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Long", "Ljava/lang/Long;", val);
    arg.f = (float)380.234;
    env->NewObject(g_jclass_Float, g_jmethod_Float_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Float", "Ljava/lang/Float;", val);
    arg.d = 9823.234;
    env->NewObject(g_jclass_Double, g_jmethod_Double_init, &arg, &val.l);
    SetStaticField(env, clazz, jobject_type, "m_s_Double", "Ljava/lang/Double;", val);
}


void CallInstanceParamMethod(nsISecureEnv* env, jclass clazz, jobject tester)
{
    jvalue val[9];

    val[0].z = true;
    val[1].b = 13;
    val[2].c = 'y';
    val[3].s = 71;
    val[4].i = 3056;
    val[5].j = 72342;
    val[6].f = (float)209.4254;
    val[7].d = 1131.32;

    jmethodID m = NULL;
    env->GetMethodID(clazz, "i_call_1", "(ZBCSIJFD)V", &m);
    env->CallMethod(jvoid_type, tester, m, val, NULL);

    jvalue arg;

    arg.z = false;
    env->NewObject(g_jclass_Boolean, g_jmethod_Boolean_init, &arg, &val[0].l);
    arg.b = 9;
    env->NewObject(g_jclass_Byte, g_jmethod_Byte_init, &arg, &val[1].l);
    arg.c = 'z';
    env->NewObject(g_jclass_Character, g_jmethod_Character_init, &arg, &val[2].l);
    arg.s = 88;
    env->NewObject(g_jclass_Short, g_jmethod_Short_init, &arg, &val[3].l);
    arg.i = 216;
    env->NewObject(g_jclass_Integer, g_jmethod_Integer_init, &arg, &val[4].l);
    arg.j = 98234;
    env->NewObject(g_jclass_Long, g_jmethod_Long_init, &arg, &val[5].l);
    arg.f = (float)444.4444;
    env->NewObject(g_jclass_Float, g_jmethod_Float_init, &arg, &val[6].l);
    arg.d = 590.23;
    env->NewObject(g_jclass_Double, g_jmethod_Double_init, &arg, &val[7].l);
    env->NewStringUTF("Yes!!", (jstring*)&val[8].l);

    env->GetMethodID(clazz, "i_call_2", 
		     "(Ljava/lang/Boolean;Ljava/lang/Byte;Ljava/lang/Character;Ljava/lang/Short;Ljava/lang/Integer;Ljava/lang/Long;Ljava/lang/Float;Ljava/lang/Double;Ljava/lang/String;)V", 
		     &m);
    env->CallMethod(jvoid_type, tester, m, val, NULL);
}


void CallStaticParamMethod(nsISecureEnv* env, jclass clazz)
{
    jvalue val[9];

    val[0].z = true;
    val[1].b = 3;
    val[2].c = 'd';
    val[3].s = 97;
    val[4].i = 789;
    val[5].j = 98121;
    val[6].f = (float)634.23;
    val[7].d = 777.77;

    jmethodID m = NULL;
    env->GetStaticMethodID(clazz, "s_call_1", "(ZBCSIJFD)V", &m);
    env->CallStaticMethod(jvoid_type, clazz, m, val, NULL);

    jvalue arg;

    arg.z = true;
    env->NewObject(g_jclass_Boolean, g_jmethod_Boolean_init, &arg, &val[0].l);
    arg.b = 89;
    env->NewObject(g_jclass_Byte, g_jmethod_Byte_init, &arg, &val[1].l);
    arg.c = 'u';
    env->NewObject(g_jclass_Character, g_jmethod_Character_init, &arg, &val[2].l);
    arg.s = 23;
    env->NewObject(g_jclass_Short, g_jmethod_Short_init, &arg, &val[3].l);
    arg.i = 9873;
    env->NewObject(g_jclass_Integer, g_jmethod_Integer_init, &arg, &val[4].l);
    arg.j = 89022;
    env->NewObject(g_jclass_Long, g_jmethod_Long_init, &arg, &val[5].l);
    arg.f = (float)555.5555;
    env->NewObject(g_jclass_Float, g_jmethod_Float_init, &arg, &val[6].l);
    arg.d = 6666.2323;
    env->NewObject(g_jclass_Double, g_jmethod_Double_init, &arg, &val[7].l);
    env->NewStringUTF("No!!", (jstring*) &val[8].l);
 
    env->GetStaticMethodID(clazz, "s_call_2", 
			   "(Ljava/lang/Boolean;Ljava/lang/Byte;Ljava/lang/Character;Ljava/lang/Short;Ljava/lang/Integer;Ljava/lang/Long;Ljava/lang/Float;Ljava/lang/Double;Ljava/lang/String;)V", 
			   &m);
    env->CallStaticMethod(jvoid_type, clazz, m, val, NULL);
}


void CallInstanceTypeMethod(nsISecureEnv* env, jclass clazz, jobject tester)
{
    jmethodID m = NULL;
    jvalue result;

    env->GetMethodID(clazz, "i_get_boolean", "()Z", &m);
    env->CallMethod(jboolean_type, tester, m, NULL, &result);
    print(jboolean_type, result, "m_i_boolean");

    env->GetMethodID(clazz, "i_get_byte", "()B", &m);
    env->CallMethod(jbyte_type, tester, m, NULL, &result);
    print(jbyte_type, result, "m_i_byte");

    env->GetMethodID(clazz, "i_get_char", "()C", &m);
    env->CallMethod(jchar_type, tester, m, NULL, &result);
    print(jchar_type, result, "m_i_char");

    env->GetMethodID(clazz, "i_get_short", "()S", &m);
    env->CallMethod(jshort_type, tester, m, NULL, &result);
    print(jshort_type, result, "m_i_short");

    env->GetMethodID(clazz, "i_get_int", "()I", &m);
    env->CallMethod(jint_type, tester, m, NULL, &result);
    print(jint_type, result, "m_i_int");

    env->GetMethodID(clazz, "i_get_long", "()J", &m);
    env->CallMethod(jlong_type, tester, m, NULL, &result);
    print(jlong_type, result, "m_i_long");

    env->GetMethodID(clazz, "i_get_float", "()F", &m);
    env->CallMethod(jfloat_type, tester, m, NULL, &result);
    print(jfloat_type, result, "m_i_float");

    env->GetMethodID(clazz, "i_get_double", "()D", &m);
    env->CallMethod(jdouble_type, tester, m, NULL, &result);
    print(jdouble_type, result, "m_i_double");

    env->GetMethodID(clazz, "i_get_String", "()Ljava/lang/String;", &m);
    env->CallMethod(jobject_type, tester, m, NULL, &result);
    
    //    char temp[256];
    const char* t = NULL;
    env->GetStringUTFChars((jstring)result.l, FALSE, &t);
    fprintf(stderr, "\nm_i_String = %s\n", t);
    env->ReleaseStringUTFChars((jstring)result.l, t);
}

 
void CallStaticTypeMethod(nsISecureEnv* env, jclass clazz)
{
    jmethodID m = NULL;
    jvalue result;

    env->GetStaticMethodID(clazz, "s_get_boolean", "()Z", &m);
    env->CallStaticMethod(jboolean_type, clazz, m, NULL, &result);
    print(jboolean_type, result, "m_s_boolean");

    env->GetStaticMethodID(clazz, "s_get_byte", "()B", &m);
    env->CallStaticMethod(jbyte_type, clazz, m, NULL, &result);
    print(jbyte_type, result, "m_s_byte");

    env->GetStaticMethodID(clazz, "s_get_char", "()C", &m);
    env->CallStaticMethod(jchar_type, clazz, m, NULL, &result);
    print(jchar_type, result, "m_s_char");

    env->GetStaticMethodID(clazz, "s_get_short", "()S", &m);
    env->CallStaticMethod(jshort_type, clazz, m, NULL, &result);
    print(jshort_type, result, "m_s_short");

    env->GetStaticMethodID(clazz, "s_get_int", "()I", &m);
    env->CallStaticMethod(jint_type, clazz, m, NULL, &result);
    print(jint_type, result, "m_s_int");

    env->GetStaticMethodID(clazz, "s_get_long", "()J", &m);
    env->CallStaticMethod(jlong_type, clazz, m, NULL, &result);
    print(jlong_type, result, "m_s_long");

    env->GetStaticMethodID(clazz, "s_get_float", "()F", &m);
    env->CallStaticMethod(jfloat_type, clazz, m, NULL, &result);
    print(jfloat_type, result, "m_s_float");

    env->GetStaticMethodID(clazz, "s_get_double", "()D", &m);
    env->CallStaticMethod(jdouble_type, clazz, m, NULL, &result);
    print(jdouble_type, result, "m_s_double");

    env->GetStaticMethodID(clazz, "s_get_String", "()Ljava/lang/String;", &m);
    env->CallStaticMethod(jobject_type, clazz, m, NULL, &result);
    
    // char temp[256];
    const char* t = NULL;
    env->GetStringUTFChars((jstring)result.l, FALSE, &t);
    fprintf(stderr, "m_s_String = %s\n", t);

    env->ReleaseStringUTFChars((jstring)result.l, t);
}



void TestSecureJNI(nsISecureEnv* env, RemoteJNIEnv* remoteev)
{
    jclass clazz = NULL;
    jmethodID m = NULL;
    jfieldID f = NULL;
    jobject tester = NULL;

    Initialize(env, remoteev);
    

    fprintf(stderr, "\n\nDone with initialization of test\n");
    env->FindClass("sun/plugin/navig/motif/SecureJNITester", &clazz);

    fprintf(stderr, "SecureJNITester Class found: %d \n", (int) clazz);
    env->GetMethodID(clazz, "<init>", "()V", &m);

    fprintf(stderr, "Method found: %d \n", (int) m);
    env->NewObject(clazz, m, NULL, &tester);

    banner("Done with get new object");
    if (!tester)
	fprintf(stderr, "Could not create tester\n");
    else
	fprintf(stderr,"Tester is %d\n", (int) tester);

    //    GetInstanceFieldMain(env, clazz, tester);

    //    SetInstanceFieldMain(env, clazz, tester);

    banner("Done with get/set instance");

    GetStaticFieldMain(env, clazz);

    banner("Done with get static");
    
    SetStaticFieldMain(env, clazz);

    banner("Done with set static");

    PrintFieldMain(env, clazz, tester);

    PrintStaticFieldMain(env, clazz);

    banner("Done with printing after statics");

    CallInstanceParamMethod(env, clazz, tester);

    banner("Called instance param method");

    CallStaticParamMethod(env, clazz);

    banner("Called static param method ");

    CallInstanceTypeMethod(env, clazz, tester);

    banner("Called instance type method ");

    CallStaticTypeMethod(env, clazz);

    banner("Called static type method");

    PrintFieldMain(env, clazz, tester);
}

















