/*
 * @(#)CookieSupport.cpp	1.6 01/02/09
 * 
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 * Author: Ali Baraty
 */
#include <stdio.h>
#include <stdlib.h>

#include "remotejni.h"
#include "nsIJVMPluginInstance.h"
#include "Navig5.h"
#include "nsICookieStorage.h"
#include "CookieSupport.h"

CookieSupport::CookieSupport(JavaVM5 *jvm) : mJvm(jvm)
{
}

/*
 * Gets the cookies from nsICookieStorage and sends it to the requester
 */
void CookieSupport::FindCookieForURL(JavaPluginInstance5 *inst, 
				     const char *url){

  PRUint32 cookieSize =  MAX_COOKIE;
  char cookieResult[MAX_COOKIE];
  nsICookieStorage *cookieStorage=mJvm->GetPluginFactory()->GetCookieStorage();
  if (cookieStorage->GetCookie(url, 
			       (void*) cookieResult, 
			       cookieSize ) == NS_OK)
    {
      ReplyCookie(cookieResult, cookieSize, inst->GetPluginNumber());
    }
    else
    {
      /* The applet is waiting on the pipe for a reply.  We must provide one!! */
      ReplyCookie(" ",1,inst->GetPluginNumber());
    }
  return;
}

/*
* Sets the cookies in nsICookieStorage from to the requester
*/
void CookieSupport::SetCookieForURL(const char *url, const char* cookie)
{
    nsICookieStorage *cookieStorage=mJvm->GetPluginFactory()->GetCookieStorage();

    // Set the cookie in the browser
    cookieStorage->SetCookie(url, (void*) cookie, strlen(cookie));

    return;
}

/*
 * Replies to cookie requestor message
 */
void CookieSupport::ReplyCookie(char *cookie, int len, int pnum){

    char *buff;
    char *cp;
    trace("CookieSupport reply for an original JAVA_PLUGIN_COOKIE_REQUEST\n");
    buff = (char *) malloc(len + 10);
    put_int(buff, 0, JAVA_PLUGIN_COOKIE);
    cp = buff + 4;
    put_int(cp, 0, pnum);
    cp += 4;
    put_short(cp, 0, len);
    cp += 2;
    memcpy(cp, cookie, len);
    mJvm->SendRequest(buff, len +10, FALSE);
    free(buff);

}
