/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/* 
 * JavaPluginFactory5.cpp  by Mark Lin (mark.lin@eng.sun.com)
 * These functions create, initialize, and shutdown a plugin.
 */


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <libintl.h>
#include <dlfcn.h>
#include "nsIComponentManager.h"
#include "nsCOMPtr.h"

// Added by Mark : Mozilla 5.x headers
#include "remotejni.h"
#include "Navig5.h"

typedef nsIJVMPlugin* (*CreateFactoryFnType)(nsISupports* );

extern "C" {
#include "Debug.h"
#include <assert.h>
}

#ifndef NO_MINIMAL_TRACE
#define TRACE(m) trace("JavaPluginFactory5(main):%s\n", m);
#endif

NS_DEFINE_IID(kPluginIID, NS_IPLUGIN_IID);
NS_DEFINE_IID(kJVMPluginIID, NS_IJVMPLUGIN_IID);
NS_DEFINE_IID(kPluginManagerIID, NS_IPLUGINMANAGER_IID);
NS_DEFINE_IID(kPluginInstanceIID, NS_IPLUGININSTANCE_IID);
NS_DEFINE_IID(kFactoryIID, NS_IFACTORY_IID);
NS_DEFINE_IID(kSupportsIID, NS_ISUPPORTS_IID);

/* Create an OJI factory and return it */
extern "C" {
void* CreateOJIFactory(nsISupports* pProvider);
}

// Added by Mark : Need to define PluginCID so that they get registered
static NS_DEFINE_CID(kPluginCID, NS_PLUGIN_CID);
static NS_DEFINE_CID(kComponentManagerCID, NS_COMPONENTMANAGER_CID);


extern "C" NS_EXPORT
nsresult JPI_NSGetFactory(nsISupports * pProvider,
			  const nsCID &aClass,
			  const char * aClassName,
			  const char * aProgID,
			  nsIFactory **aFactory) 
{
    trace("JPI_NSGetFactory\n");

    if (aFactory == NULL){
	plugin_error("Received a null pointer to pointer in NSGetFactory!\n");
	return NS_ERROR_UNEXPECTED;
    }
    // Changed by Mark : CID's are used instead of IID's. 
    if (aClass.Equals(kPluginCID)) {
	trace("create oji factory\n");
	*aFactory = (nsIFactory *) CreateOJIFactory(pProvider);

	init_utils();
	(*aFactory)->AddRef();

	return NS_OK;
    } else {
	return NS_ERROR_NO_INTERFACE;
    }
}

/* The very first function called.  It determines which version of
 * mozilla we are running in and initializes the appropriate function
 * table for the other factory functions. It then creates a plugin
 * instance factory, which can then be used for all subsequent calls.
 */
// Changed by Mark : XPCOM headers have changed
extern "C" NS_EXPORT nsresult NSGetFactory(nsISupports * pProvider,
					   const nsCID &aClass,
					   const char * aClassName,
					   const char * aProgID,
					   nsIFactory **aFactory) {
  trace("NSGetFactory in GetFactory5\n");

  // Delegate the call to the real method
  return JPI_NSGetFactory(pProvider, aClass, aClassName, aProgID, aFactory);
}


extern "C" NS_EXPORT PRBool NSCanUnload(nsISupports * aServMgr) {
    return PR_FALSE;
}

extern "C" NS_EXPORT nsresult NSRegisterSelf(nsISupports * aServMgr,
					     const char *path) {
  trace("Register self %s\n", path);

  nsresult rv;
  NS_WITH_SERVICE1(nsIComponentManager, compMgr,aServMgr, kComponentManagerCID, &rv);

  trace("ns with service called\n");
  if (NS_FAILED(rv)) {
    return rv;
  }

  rv = compMgr->RegisterComponent(kPluginCID,
				  "Java Plug-in (TM) Service",
				  "component://netscape/inline-plugin/application/x-java-vm",
				  path, 
				  PR_TRUE, PR_TRUE);

  trace("return from NSRegisterSelf\n");
  return rv;
}
 
// Changed by Mark :  This still needs to be implemented
extern "C" NS_EXPORT nsresult NSUnregisterSelf(nsISupports * aServMgr,
					       const char *path) {
    trace("UnRegister self %s\n", path);
    return NS_OK;
}

/*
NS_IMPL_ADDREF(JavaPluginFactory5)
NS_IMPL_RELEASE(JavaPluginFactory5)
*/





