/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-*/
/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/*
 * Contains declaration of the CNetscapePlugin. Modified version of
 *  Stanley's Win32 header.
 */


#ifndef JAVAPLUGININSTANCE5_H
#define JAVAPLUGININSTANCE5_H

#include "nsIJVMPluginInstance.h"

class JavaPluginFactory5;

class JavaPluginInstance5 : public nsIJVMPluginInstance, nsIPluginInstance {
public:

    NS_DECL_ISUPPORTS
		

    JavaPluginInstance5(JavaPluginFactory5 *plugin);
    virtual	~JavaPluginInstance5(void);

    JavaPluginFactory5* GetPluginFactory(void);
    // See comment for nsIPlugin::NewInstance, above.
    NS_IMETHOD Initialize(nsIPluginInstancePeer *peer);
    NS_IMETHOD Start(void);
    NS_IMETHOD GetPeer(nsIPluginInstancePeer* *resultingPeer);

    NS_IMETHOD GetJavaObject(jobject *result);

    NS_IMETHOD GetText(const char* *result);

    // The old NPP_Destroy call has been factored into two plugin instance 
    // methods:
    // Stop -- called when the plugin instance is to be stopped (e.g. by 
    // displaying another plugin manager window, causing the page containing 
    // the plugin to become removed from the display).
    //
    // Release -- called once, before the plugin instance peer is to be 
    // destroyed. This method is used to destroy the plugin instance.
    NS_IMETHOD Stop(void);

    NS_IMETHOD Destroy(void);

    // (Corresponds to NPP_SetWindow.)
    NS_IMETHOD SetWindow(nsPluginWindow* window);

    // (Corresponds to NPP_NewStream.)
    NS_IMETHOD NewStream(nsIPluginStreamListener** listener);

    // (Corresponds to NPP_Print.)
    NS_IMETHOD Print(nsPluginPrint* platformPrint);

    NS_IMETHOD GetValue(nsPluginInstanceVariable var, void* val);

    // (Corresponds to NPP_HandleEvent.)
    // Note that for Unix and Mac the NPPluginEvent structure is different
    // from the old NPEvent structure -- it's no longer the native event
    // record, but is instead a struct. Done for future extensibility,
    // and so that the Mac could receive the window argument too. For Windows
    // and OS2, it's always been a struct, so there's no change for them.
    NS_IMETHOD HandleEvent(nsPluginEvent* event, PRBool *handled);

    // (Corresponds to NPP_URLNotify.)
    NS_IMETHOD URLNotify(const char* url, const char* target,
				nsPluginReason reason, void* notifyData);

    NS_IMETHOD_(int) Valid(void);
    NS_IMETHOD_(void) SetDocbase(const char *url);
 
    NS_IMETHOD_(void) EnterRequest(char* msg);
    NS_IMETHOD_(void) ExitRequest(char* msg);

    // Return the index of this plugin
    int GetPluginNumber(void) { return plugin_number; } ;
    // Mozilla changes: This will embed the given AppletWindow ID into Navigator's
    // containing browser window
    void EmbedAppletWindow(int windowID);

    

protected:
    nsIPluginInstancePeer*	instance_peer;
    JavaPluginFactory5*         plugin_factory;
    int plugin_number;
    
    // Window
    nsPluginWindow* window;

    // Docbase url
    char* baseURL;

    // If non-null indicate that a request is in process
    char* current_request;

    // Set to false as soon as a destroy is called on it.
    // The destroy will also send an abrupt termination if
    // that is needed.
    int is_instance_alive;

    // Space to store https reply data
    char *https_data;

    // Current amount of data received
    int https_data_len;
};

#endif 
