/*
 * @(#)JavaStream5.cpp	1.9 00/03/31
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */
 
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <stdio.h>
#include <stdlib.h>
#include "remotejni.h"
#include "jni.h"
#include "nsIPluginStreamPeer.h"
#include "nsIJVMPluginInstance.h"
#include "Navig5.h"
#include "JavaStream5.h"

#define TRACE(m) trace("JavaStream5::%s\n", m);
#define TRACE_INT(m, i) trace("JavaStream5::%s %d\n", m, i);


/* 
 * Constructor
 */
JavaStream5::JavaStream5(JavaPluginInstance5* plugin_inst,
			 nsIPluginStreamPeer* peer)
    : NetscapeStream5(peer), plugin_instance(plugin_inst), n_write_calls(0)
{
    TRACE("constructor");
    const char* szURL;
    peer->GetURL(&szURL);
    if (szURL == NULL) {
	m_szURL[0] = '\0';  // Patch for Opera Browser
    } else {
	strncpy(m_szURL, szURL, MAX_PATH);
    }

}

/*
 * Stream destructor
 */
JavaStream5::~JavaStream5()
{
    TRACE("Destructor");
}

/*
 * StreamAsFile is called to return the filename in the cache.
 */
NS_IMETHODIMP JavaStream5::AsFile(const char *fname) {
    //XXXFIX caching support is not yet in
    plugin_error("No caching in Mozilla 5 ");
    UNUSED(fname);
    return NS_OK;
}


/* Write is called to handle a piece of data in the plugin stream. */
NS_IMETHODIMP JavaStream5::Write(const char* buffer, PRInt32 offset, PRInt32
				 len, PRInt32 *bytesWritten)  
{
    /* 
       Notice that the offset only represents where the buffer is in 
       the entire stream, NOT where the data is in the buffer. Therefore,
       offset should never never be used, because the buffer already 
       contains all the data we need. 
     */

    nsresult errorResult = NS_OK;
    int key;
    
    TRACE("Write");

    *bytesWritten = 0;
    if (NS_OK != (errorResult = stream_peer->GetNotifyData((void **)&key)))
	return errorResult;

    if (tracing) {
	char head[81];
	int clen = slen(m_szURL);
	if (clen > 30) {
	    clen = 30;
	}
	memcpy(head, m_szURL, clen);
	head[clen] = 0;
        trace("JavaStream5::NPP_Write \"%s\" %d\n", head, len);
	if (len > 70) {	
	    memcpy(head, buffer, 70);
	    head[70] = 0;
            fprintf(stderr, "%s...\n", head);
	} else {
	    memcpy(head, buffer, len);
	    head[len] = 0;
            fprintf(stderr, "%s\n", head);
	}	
    }
    if (key == JAVA_PLUGIN_PROXMAP_INPUT) {
	// The proxymapping initialization is done. The resulting data
	TRACE_INT("Write PROXMAP_INPUT", len);
	plugin_instance->GetPluginFactory()->GetProxySupport()
	    ->ProxmapInput((char *) (buffer),len);
    } else if (key == JAVA_PLUGIN_PROXMAP_QUERY) {
	TRACE_INT("Write PROXMAP_QUERY", len);
	plugin_instance->GetPluginFactory()->GetProxySupport()
	    ->ProxmapReply(m_szURL, len, (void *)(buffer));
    } else {
	plugin_error("Unknown key in creating stream? %X\n", key);
    }
    *bytesWritten = len;
    return errorResult;        /* The number of bytes accepted */
}


// (Corresponds to NPP_NewStream's stype return parameter.)
NS_IMETHODIMP JavaStream5::GetStreamType(nsPluginStreamType *result)
{
    TRACE("GetStreamType");
    int key;
    nsresult res;
    if ( NS_OK != (res =stream_peer->GetNotifyData((void **) &key)))
	return res;

    // If this is a cache query, return the stream as a file
    if (key == JAVA_PLUGIN_CACHE_QUERY) {
        *result =  nsPluginStreamType_AsFile;
        return NS_OK;
    }

    return NetscapeStream5::GetStreamType(result);
}






