/*
 * @(#)JavaVM5.cpp	1.54 01/03/01
 * @(#)JavaVM5.cpp	1.34 00/09/15
 * 
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 * Author: Benedict Gomes, Robert Szewczyk
 */

/*  
 * The JavaVM5 class represents our connect to the actual VM.  It is
 * created, intialized and then started, which spawns off the VM
 * process. The basic execution structure is the same as that in the
 * original plugin 1.1.  BG Sept 98
 *                                      
 */

#include <sys/sem.h>
#include <sys/ipc.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>
#include <stropts.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/VendorS.h>
#include <stdarg.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <stropts.h>
#include <dlfcn.h>
#include <limits.h>
#include "jni.h"
#include "remotejni.h"
#include "nsprPrivate.h"
#include "private/pprio.h"

/* For htonl and htons */
#include <arpa/inet.h>

#ifdef LINUX
#include <linux/limits.h>
#include <linux/types.h>
#include <linux/dirent.h>
#define _DIRENTRY_H
#else
#include <ulimit.h>
#endif
#include <dirent.h>
#include <sys/param.h>          // For MAXPATHLEN

//#include "nsICapsManager.h"
#include "gdk/gdk.h"
#include "nsIJVMManager.h"
#include "Navig5.h"
#include "JavaVM5.h"
#include "pluginversion.h"
#include "CNetscapeStreamListener.h"
#include "HttpsStreamListener.h"
#include "PostHttpsListener.h"
#include "CookieSupport.h"

#ifndef NO_MINIMAL_TRACE
#define TRACE(m) trace("JavaVM5 %s\n", m);
#define TRACE_INT(m, i) trace("JavaVM5 %s %X\n", m, i);
#define TRACE_STR(m, p) trace("JavaVM5 %s %s\n", m, p);
#else
#define TRACE(m) 
#define TRACE_INT(m, i) 
#define TRACE_STR(m, p) 
#endif

void socket_cleanup()
{
    char path[PATH_MAX];
    bzero(path,PATH_MAX);
    pid_t pid = getpid();
    sprintf(path,"%s.%s.%d",JAVA_PLUGIN_SOCKFILE,PLUGIN_NODOTVERSION,pid);
    unlink(path);
}

static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);

jboolean IsAllPermissionGranted(const char *codebaseURL,
                                const unsigned char **certChain,
                                unsigned int *certLengths,
                                int numOfCerts);

static 
void gdk_spont_event_handler(gpointer data, gint source, GdkInputCondition) {
	TRACE("In spont event handler");
	if(data != NULL)  {
        	JavaVM5 *vm = (JavaVM5*) data;
        	vm->ProcessSpontaneousQueue();
	}
}
static 
void gdk_worker_event_handler(gpointer data, gint source, GdkInputCondition) {
	TRACE("In worker event handler");
	if(data != NULL)  {
        	JavaVM5 *vm = (JavaVM5*) data;
        	vm->ProcessWorkQueue();
	}
}

/*
 * This is the callback associated with the event widget. 
 * When an event occurs this callback is invoked and more work
 * can be processed on the work queue, coming from Worker.java
 */
static Atom work_atom;

/*
 * Allocate the event widget which will be used by the Worker class
 * (see Worker.pushRequest) to send back information, by invoking
 * its attached handler (child_handle_event)
 */
void JavaVM5::AllocateEventWidget(void) {
    Arg args[40];
    int argc;
    // Mozilla changes : In Mozilla5, getting the display hasn't been implemented yet!
    Display *display = (Display *) NULL;

    TRACE("AllocateEventWidget\n");
    m_pPluginFactory->GetPluginManager()
	->GetValue(nsPluginManagerVariable_XDisplay, &display);

    if (display == NULL) plugin_error("Could not open display!");

    /* create a top-level shell */
    argc = 0;
    XtSetArg(args[argc], XmNallowShellResize, True); argc++;
    XtSetArg(args[argc], XmNwidth, 100); argc++;
    XtSetArg(args[argc], XmNheight, 100); argc++;
    XtSetArg(args[argc], XmNmappedWhenManaged, False); argc++;

    char ** argss = NULL;
    int argcc = 0;

    XtAppContext aContext;

    event_widget = XtOpenApplication(&aContext, "XApplication", NULL, 0,
                                   &argcc, argss, NULL,
                                   vendorShellWidgetClass,
                                   args, argc);

    // Realize widget
    XtRealizeWidget(event_widget);

    // Obtain display from the widget
    display = XtDisplay(event_widget);

//For some reason this call is dying so the rest of the code in this function is really 
//just to give us a dummy value to send to the java_vm process

/*
    event_widget  = XtAppCreateShell("AWTapp","XApplication",
                                     vendorShellWidgetClass,
                                     display,
                                     args,
                                     argc);
				     
    TRACE("Created event widget");
    if (event_widget == NULL) {
        fprintf(stderr, "Plugin: couldn't allocate event widget");
        return;
    }

    XtRealizeWidget(event_widget);
*/
    event_window_id = XtWindow(event_widget);
/*
    XtAddEventHandler(event_widget, PropertyChangeMask, FALSE,
                      child_event_handler, this);
*/
    work_atom = XInternAtom(display, "java_plugin_work", FALSE);

    TRACE_INT("Event widget", (int) event_widget);
    TRACE_INT("Event window id", (int) event_window_id);
}


/*
 * Process work from the worker queue, coming from the out-of-proc
 * VM. Basically wait in an infinite loop on a timed select.
 * If we time out, return, since this may be called in the main
 * thread, and it is important that other work also get done
 * in the browser!
 */
void JavaVM5::ProcessWorkQueue(void) {    
    TRACE("ProcessWorkQueue");
    int wp = PRFileDesc_To_FD(state->work_pipe);
    if (wp < 0) {
        TRACE("work pipe is dead");
        return;
    }
    for (;;) {
        int rc;
        fd_set fs;
        struct timeval tv;
        tv.tv_sec =0;
        tv.tv_usec = 0;
        TRACE("Doing work");
        FD_ZERO(&fs);
        FD_SET(wp, &fs);
        rc = select(wp + 1, &fs, NULL, NULL, &tv);
        if (rc < 1) {
            TRACE("No work on work pipe");
            break;
        }
        /* We have work in the pipe */
        DoWork();
    }
    TRACE("Done with processing work queue");
}


/*  
 * Create a new JavaVM, associated with the plugin factory 'plugin'
 * (an instance of NPIPlugin).  Also determine whether the VM should
 * use green or native threads based on the THREADS_FLAG.  Initialize
 * and then StartVM must be called.
 */
JavaVM5::JavaVM5(JavaPluginFactory5 *plugin) {
    TRACE("Creating JavaVM5");

    state = new LongTermState();
    sprintf(stateENVName,"%s%s","JAVA_PLUGIN_STATE",PLUGIN_NODOTVERSION);
    is_java_process_ready = 0;
    m_pPluginFactory = plugin;
    memset(state, 0, sizeof(struct LongTermState));
    jvm_classpath = NULL;
    jvm_classpath_buflen = 0;
    jvm_classpath_len = 0; 
    jvm_status = nsJVMStatus_Enabled;
    base_status = nsJVMStatus_Enabled;
    if(FindEnabled())
       base_status = nsJVMStatus_Enabled;
    else
       base_status = nsJVMStatus_Disabled;
 

    /* Check for whetehr we use use green threads */
    char *threads_flag = getenv("THREADS_FLAG");
    child_uses_green_threads = 0;
    if (threads_flag != NULL) { 
        if (strcmp(threads_flag, "green") == 0) {
            /* If the flag is green, then don't use native threads */
            child_uses_green_threads = 1;
        } 
    }
    /* Set the environment variable for the child process. Store the
       value and putenv it when creating the VM*/
    char *buff = (char *)checked_malloc(100);
    if (child_uses_green_threads == 1) {
        sprintf(buff, "THREADS_TYPE=green_threads");
    } else {
        sprintf(buff, "THREADS_TYPE=native_threads");
    }
    env_THREADS_TYPE = buff;

}

/*  
 * Initialize the VM to indicate the version of navigator This is
 * called from the PluginFactory initialization, so the peer NPIPlugin
 * is known, and hence the version.  If version >= 5.0 we load
 * libjni.so and also load the real lock functions from nspr21.so
 * Otherwise, we just use the dummy locks etc.
 */
void
JavaVM5::Initialize(int navigator_version) {
    TRACE_INT("Initializing", navigator_version);
    state->navigator_version = navigator_version;
    if (navigator_version < 5) plugin_error("Pre 5.0 browser!");
}

/*
 * Destroy the VM. In mozilla 5, since the factory is never deleted,
 * this should never be called
 */
JavaVM5::~JavaVM5() {
    TRACE("Destroying JavaVM");
}

/* Return the actual status of the JVM - whether running or not */
nsJVMStatus JavaVM5::GetJVMStatus(void) {
    TRACE("GetJVMStatus");
    return jvm_status;
}

/* Return whether the browser has enabled the  VM  (it may
 * be disabled even if it is running.
 */
PRBool JavaVM5::GetJVMEnabled(void) {
    TRACE("GetJVMEnabled");
    if (base_status == nsJVMStatus_Enabled)
        return PR_TRUE;
    else
        return PR_FALSE;
}

/* Enable the VM. */
void JavaVM5::SetJVMEnabled(PRBool enable) {
    TRACE("SetJVMEnabled");
    if (enable) {
        base_status = nsJVMStatus_Enabled;
    } else {
        base_status = nsJVMStatus_Disabled;
    }
}

/* Return the current class path */
const char* JavaVM5::GetClassPath(void) {
    TRACE("GetClassPath");
    return (const char *)jvm_classpath;
}

/*
 * Return the plugin factory associated with this VM
 */
JavaPluginFactory5* JavaVM5::GetPluginFactory(void) {
    TRACE("GetPluginFactory");
    return m_pPluginFactory;
}


/* Dup the fdarr to safety range */
void
dup2_to_safety(int nfds, int fdarr[], 
               int safe_range_start, int safe_range_end) {

    int i;

    /* Find the max fd to identify a fd range to store fds
       temporarily during duping */
    int tempfd = safe_range_end + 1;
    for(i = 0; i < nfds ; i++)
        tempfd = (fdarr[i] >= tempfd) ? fdarr[i] + 1 : tempfd;
    
    /* Dup the FDs out of the safe range */
    for(i = 0; i < nfds ; i++){
        int curfd = fdarr[i];
        /* If the FD is already in the safe range, dup it temporarily
           in the temporary range */
        if (curfd >= safe_range_start && curfd <= safe_range_end) {
            TRACE_INT("Conflict", i);
            TRACE_INT("fd", fdarr[i]);
            TRACE_INT("fd target", tempfd);
            fdarr[i] = wrap_dup2(curfd, tempfd);
            tempfd++;
        }
    }

    /* assert fdarr[i] not in safe_range */
    /* Dup the FDs into the safe range */
    int j;
    for(i = 0, j=safe_range_start; i < nfds; i++, j++) {
        fdarr[i] = wrap_dup2(fdarr[i], j);
    }


}


/*
 * Start the actual VM, using the class path 'addClasspath'
 * in addition to anything else determined to be in the class path.
 * Determine the location of the JRE, determine the child's
 * environment (set up the env_* strings) and then do the
 * fork1. Then putenv the env_* strings and do an exec of
 * java_vm or java_vm_native
 */
int JavaVM5::StartJavaVM(const char *addClasspath)
{
    int rc;
    char *buff;
    char *env_JAVA_PLUGIN_AGENT;
    char *progname;
    const char *agent = "No agent";
    nsresult res;

    TRACE("********************* StartJavaVM ***************************");

/* This is just wrong!! The VM needs to run whether we are enabled or not
    if (!FindEnabled()) {
        Cleanup();
        return NS_ERROR_FAILURE;
    }
*/

    /* Start the class path with the arg 'addClasspath' */
    if (addClasspath) {
        jvm_classpath_len = slen(addClasspath);
        jvm_classpath = (char *)checked_malloc(jvm_classpath_len);
        memcpy(jvm_classpath, addClasspath, jvm_classpath_len);
    }

   TRACE("Allocating event widget");
    /* Allocate an X widget as a communication channel with our child */
    AllocateEventWidget();

    /*
     * Check if we already have an Java child process.
     * We use the Unix environment to locate our saved state
     */
    LongTermState *pstateTmp;
    buff = getenv(stateENVName);
    if (buff != NULL) {
       rc = sscanf(buff, "%X", &pstateTmp);
       if (rc == 1 && pstateTmp != 0) {
           /* We already have an active connection. */
                 delete state;
	         state = pstateTmp;
                 TRACE("reusing child process");
                 jvm_status = nsJVMStatus_Running;
                 return NS_OK;
       }
    } 
        // We are first on the list
        /* Allocate and record a state struct.  This is used to retrieve
         * our state from the environment if the plugin is restarted 
         */
        buff = (char* )checked_malloc(140);
        sprintf(buff, "%s=%X", stateENVName,(int) state);
        putenv(buff);

    /* Get the browser's name (Mozilla-5) to store in the environment
     *  variable JAVA_PLUGIN_AGENT for the child to read to determine
     *  its version. 
     */
    res = m_pPluginFactory->GetPluginManager()->UserAgent(&agent);
    if (NS_OK != res) 
        return res;
    env_JAVA_PLUGIN_AGENT = (char *)checked_malloc(40 + slen(agent));
    sprintf(env_JAVA_PLUGIN_AGENT, "JAVA_PLUGIN_AGENT=%s", agent);

    /* Find directory where plugin JRE is installed */
    FindJavaDir();
    if (state->java_dir == NULL) {
        plugin_formal_error("Plugin: can't find plugins/../java directory");
        return NS_ERROR_NOT_AVAILABLE;
    }

    /* Set up the child environment variables. Wait till after the
     * fork to actually put them into the environment.
     */
    SetupChildEnvironment();


    progname = "java_vm";

    /* Initialize the server socket for later use with attach_thread */
#if defined(USE_TCP_SOCKET)
    PRFileDesc* server_socket = PR_NewTCPSocket();
#else
    PRFileDesc* server_socket = PR_Socket(PR_AF_LOCAL, PR_SOCK_STREAM, 0);
#endif
    PRNetAddr net_address;

    int try_port = INITIAL_ATHREAD_PORT;
#if defined(USE_TCP_SOCKET)
    net_address.inet.family = AF_INET;
    net_address.inet.port = htons(try_port);
    net_address.inet.ip = htonl(INADDR_ANY);
    while (PR_Bind(server_socket, &net_address) != PR_SUCCESS) {
        /* Try higher number addresses */
        TRACE_INT("Binding of server socket failed", try_port);
        try_port++;
        net_address.inet.port = htons(try_port);
    }
#else
    pid_t spid = getpid();
    net_address.local.family = PR_AF_LOCAL;
    bzero(net_address.local.path,sizeof(net_address.local.path));
    sprintf(net_address.local.path,"%s.%s.%d",JAVA_PLUGIN_SOCKFILE,PLUGIN_NODOTVERSION,spid);
    unlink(net_address.local.path);
    try_port = htons(spid);
    if(PR_Bind(server_socket, &net_address) != PR_SUCCESS) {
        TRACE_INT("Binding of server socket failed", try_port);
    }
    atexit(socket_cleanup);
#endif
    
    if (PR_Listen(server_socket, 3) != PR_SUCCESS)
        plugin_error("Listen on server socket failed");

    state->server_socket = server_socket;
    state->port = try_port;
    

    /* For the worker pipe communication */
    PRFileDesc *Work[2];
    wrap_PR_CreateSocketPair("work", Work);

    /* For the command pipe communication */
    PRFileDesc *Command[2];
    wrap_PR_CreateSocketPair("command", Command);

    /* For the other JNI/JS communication from JVM */
    PRFileDesc *Spontaneous[2];
    wrap_PR_CreateSocketPair("spontaneous", Spontaneous);

    /* For the Print communication */
    PRFileDesc* Printing[2];
    wrap_PR_CreateSocketPair("print", Printing);

    /* Determine the full executable path name */
    char *fullprogname = (char *) checked_malloc(100 + slen(state->java_dir));
    sprintf(fullprogname, "%s/bin/%s/native_threads/%s", state->java_dir, ARCH,progname);

    int JVMWorkFD = PRFileDesc_To_FD(Work[1]);
    int JVMCommandFD = PRFileDesc_To_FD(Command[1]);
    int JVMSpontaneousFD = PRFileDesc_To_FD(Spontaneous[1]);
    int JVMPrintFD = PRFileDesc_To_FD(Printing[1]);

    int nfds = 4;
    int fdarr[4] = {JVMSpontaneousFD, 
                    JVMCommandFD, JVMWorkFD, JVMPrintFD};


    /* If the vm uses native threads make these fds be blocking.
       The FDs that mozilla returns are non-blocking, at
       least when created with a "local threads" nspr.
       If the VM uses green threads it requires blocking FDs */
    if (!child_uses_green_threads) {
        for(int i=0; i < nfds; i++) {
            int origflags = fcntl(fdarr[i], F_GETFL);
            int newflags = origflags & (~O_NONBLOCK);
            fcntl(fdarr[i], F_SETFL, newflags);
            trace("native vm:%d Orig flags = %X New flags = %X \n", fdarr[i],
                  origflags, newflags);
        }
    }


    /* Currently not used */
    char env_JAVA_PLUGIN_ARGS[1024];
    sprintf(env_JAVA_PLUGIN_ARGS, "JAVA_PLUGIN_ARGS=nothing");

    char env_MOZILLA_WORKAROUND[1024];
    sprintf(env_MOZILLA_WORKAROUND, "MOZILLA_WORKAROUND=true");

    TRACE("Ready to fork");


    /* Create our child java process */
#if defined(__linux__) 
    rc = fork();
#else
    rc = fork1();
#endif

    if (rc == -1) {
        plugin_error("Fork failed: %d\n", errno);
        return NS_ERROR_OUT_OF_MEMORY;
    }

    if (rc == 0) {
        /* Child process.  Prepare to exec. */
        int i, max;

        if (child_uses_green_threads) {
            putenv("_JVM_THREADS_TYPE=green_threads");
        }
        /* Not used */
        putenv(env_JAVA_PLUGIN_ARGS);
        /* native_threads or green_threads */
        putenv(env_THREADS_TYPE);
        /* Must include javaplugin.jar */
        //putenv(env_CLASSPATH);
        putenv(env_LD_LIBRARY_PATH);
        /* Used in call to sun.plugin.JavaRunTime.initEnvironment */
        putenv(env_JAVA_HOME);
        /* To derive the version of mozilla calling the plugin */
        putenv(env_JAVA_PLUGIN_AGENT);
	/* Workaround Mozilla bug */
	putenv(env_MOZILLA_WORKAROUND);

	putenv(env_PLUGIN_HOME);

        if (tracing) {
            /* If PLUGIN_LD_DEBUG is set to anything pass it as LD_DEBUG*/
            char* ld_flags = getenv("JAVA_PLUGIN_LD_DEBUG");
            if (ld_flags != NULL) {
                char* ldbuf = (char *) checked_malloc(strlen(ld_flags) + 11);
                sprintf(ldbuf, "LD_DEBUG=%s", ld_flags);
                putenv(ldbuf);
            }
        }
        /* Clear the close-on-exec flags for stdio and our pipe. */
        for (i = 0; i < 3; i++) {
            fcntl(i, F_SETFD, 0);
        }

        /* We need to dup the FDs into a lower range since under green
           threads monitors for only the first 16 FDs are
           pre-allocated. 
           */
        int safe_range_start = MOZ5_STARTSAFEFD;
        int safe_range_end   = MOZ5_ENDSAFEFD;
        dup2_to_safety(nfds, fdarr, safe_range_start, safe_range_end);

        /* Don't close on exec for our pipes */
        for(i = 0; i < nfds; i++)
            fcntl(fdarr[i], F_SETFD, 0);
        

        /* Close all other file descriptors. */
#ifndef __linux__
        max = ulimit(UL_GDESLIM);
#else
        max = sysconf(_SC_OPEN_MAX);
#endif       

        for (i = 3; i <= max; i++) {
            int do_close = 1;
            for(int j = 0; j < nfds; j++)
                if (i == fdarr[j] ) do_close = 0;
            if (do_close) close(i);
        }

        if (tracing) {
            execlp(fullprogname, progname, "-t", 0);
        } else {
            execlp(fullprogname, progname, "", 0);
        }

        plugin_error("Exec of \"%s\" failed: %d\n<", progname, errno);
        exit(6);
    }

    TRACE("Continuing in parent process....");
    state->child_pid = rc;
    state->command_pipe = Command[0];
    state->work_pipe = Work[0];
    state->spont_pipe = Spontaneous[0];
    state->print_pipe = Printing[0];

    PR_Close(Command[1]);
    PR_Close(Work[1]);
    PR_Close(Spontaneous[1]);
    PR_Close(Printing[1]);

    int resack = PR_Read(Command[0], buff, 1);
    if (resack != 1) {
	plugin_error("Could not read ack from browser");
    }
    TRACE_INT("Parent OK, child pid\n", state->child_pid);

    jvm_status = nsJVMStatus_Running;

    /* Send the initial event window id */
    char msg[4];
    put_int(msg, 0, event_window_id);
    SendRequest(msg, 4, 0);

    //Add the worker fd to the event queue
    gdk_input_add(PRFileDesc_To_FD(state->work_pipe),GDK_INPUT_READ,gdk_worker_event_handler,this);

    spont_remotejni            = create_RemoteJNIEnv();

    CSecureJNIEnv* secjni        = new CSecureJNIEnv(NULL, spont_remotejni);

    JNIEnv* proxyJNI           = NULL;

    if (NS_SUCCEEDED((m_pPluginFactory->GetJVMManager())->CreateProxyJNI(secjni,&proxyJNI))){
        //fprintf(stderr, " Succeeding in creating proxy\n");
    }

    /* Register the mapping between proxy and remote jni */
    int spont_env_index =  m_pPluginFactory
        ->RegisterRemoteEnv(spont_remotejni, proxyJNI);

    /* Initialize the remote JNI Env with its pipe etc */
    init_RemoteJNIEnv(spont_remotejni, spont_env_index, state->spont_pipe);


    //Add the spontanious fd to the event queue
    gdk_input_add(PRFileDesc_To_FD(state->spont_pipe),GDK_INPUT_READ,gdk_spont_event_handler,this);

    return NS_OK;
}
/*
 * ReceivePrint polls the printpipe until it receives ok from
 * plugin
 */

void JavaVM5::ReceivePrinting(FILE *fp) {
    struct PRPollDesc pollDesc[2];
    PRIntervalTime timeout=PR_INTERVAL_NO_TIMEOUT;
    PRIntn npds= 2;
    int finished = 0;
    char *buf;
    int ok;
    int n;
    int oldsize=0;

    TRACE("Receiving printing ");
    pollDesc[0].fd = state->command_pipe;
    pollDesc[1].fd = state->print_pipe;
    pollDesc[0].in_flags = pollDesc[1].in_flags=PR_POLL_READ;
    
    for (;;)
      {
	if (PR_Poll(pollDesc, npds, timeout ) < 0 )
	    {
	       continue;
	    }
	if (pollDesc[1].out_flags & PR_POLL_READ)
	  {
	    n = PR_Available(pollDesc[1].fd);
	    if ( n < 0 ){
		fprintf(stderr, "Error reading print pipe %d\n",PR_GetError());
		return;
	    }  
	    if ( oldsize < n ){
		buf = (char *) realloc(buf, n);
		oldsize =n;
	    }
	    n = PR_Read(pollDesc[1].fd, buf, n);
	    if ( n > 0 ){
		fwrite(buf, n, 1, fp);
	    }
	    else if ( n < 0 ) {
	      fprintf(stderr, "Error reading print pipe %d\n", PR_GetError());
	      return;
	    }
	  }
	else if (finished)
	  return;
	if (pollDesc[0].out_flags & PR_POLL_READ)
	  {
	    n=PR_Read(pollDesc[0].fd, buf, 4);
	    if ( n > 0 )
	      {
		  ok = get_int(buf,0);
		  if ( ok == JAVA_PLUGIN_OK)
		      {
			  finished = 1;
			  timeout=0;
		      }
	      }
	  }
      }
    if (buf != NULL)
	free(buf);
}

jobject
JavaVM5::GetJavaObjectForInstance(int plugin_number) {
    char msg[8];
    int code = JAVA_PLUGIN_GET_INSTANCE_JAVA_OBJECT;
    int ci = 0;
    put_int(msg, ci, code); ci += 4;
    put_int(msg, ci, plugin_number);
    int res = SendBlockingRequest(msg, 8);
    return (jobject) res;
}

/*
 * This is called when Netscape is about to unload the library.
 * All our plugin instances should be closed down at this point.
 * We keep our Java child process alive for reuse later.
 */
void
JavaVM5::ShutdownJavaVM(int RealShutdown)
{
    char buff[4];
    TRACE("Shutdown");

    /* This is just advice, the java_vm process will actually stick around */
    put_int(buff, 0, JAVA_PLUGIN_SHUTDOWN);
    SendRequest(buff, 4, FALSE);

    // Clean off our work queue
    ProcessWorkQueue();

    if (event_widget != NULL) {
        XtDestroyWidget(event_widget);
    }

    if (RealShutdown) {
//        putenv((char *) "JAVA_PLUGIN_STATE=0x0");
//        close(state->command_pipe);
//        close(state->work_pipe);
//        free(state);
//        state = NULL;
    }
}

/* Handle requests on the spontaneous pipe. These requests are handled
   similarly to requests on */
void
JavaVM5::ProcessSpontaneousQueue() {

    int wp = PRFileDesc_To_FD(state->spont_pipe);
    if (wp < 0) {
        TRACE("spont pipe is dead");
        return;
    }
    for (;;) {
        int rc;
        fd_set fs;
        struct timeval tv;
        tv.tv_sec =0;
        tv.tv_usec = 0;
        TRACE("Doing spontainious work");
        FD_ZERO(&fs);
        FD_SET(wp, &fs);
        rc = select(wp + 1, &fs, NULL, NULL, &tv);
        if (rc < 1) {
            TRACE("No work on spont pipe");
            break;
        }
        /* We have work in the pipe */
        int request_code = 0;

        TRACE("Spontaneous thread waiting for next request...");

        read_PR_fully("Spont Req", state->spont_pipe, (char *)&request_code, 4);

        TRACE_INT("Received request code: ", request_code);

        switch (request_code) {
        case JAVA_PLUGIN_REQUEST: 
            {
                /* This is a JS call from a spontaneous thread */
                TRACE("SPONTANEOUS CALL!!! (JAVA_PLUGIN_REQUEST)");
                JSHandler(spont_remotejni);
                break;
            }
        case JAVA_PLUGIN_TRUST_ALLPERM:
            {
                int NumOfCerts;

                TRACE("SPONTANEOUS CALL!!! (TRUST_ALLPERM)");
        
                read_PR_fully("NCert",state->spont_pipe, (char *)&NumOfCerts, 4);

                int CertLengths_sz =  sizeof(jint) * NumOfCerts;

                unsigned char **pChain;

                unsigned int* pCertLengths = 
                    (unsigned int*) checked_malloc(CertLengths_sz);

                read_PR_fully("CertLengths", state->spont_pipe, 
                              (char*) pCertLengths, CertLengths_sz);

                int chain_sz;
                read_PR_fully("Chain Size", state->spont_pipe,
                              (char *) &chain_sz, 4);

                pChain =  (unsigned char**) checked_malloc(chain_sz);
                read_PR_fully("Chain", state->spont_pipe, (char *) pChain, chain_sz);

                int pCodebaseURL_len;
                read_PR_fully("URL_len",state->spont_pipe, 
                              (char*) &pCodebaseURL_len, 4);

                char* pCodebaseURL = (char *) malloc(pCodebaseURL_len);
                read_PR_fully("pCodeBaseURL", state->spont_pipe, 
                              pCodebaseURL, pCodebaseURL_len);
                jboolean res = 
                    IsAllPermissionGranted(pCodebaseURL,
                                           (const unsigned char **)pChain,
                                           pCertLengths, 
                                           NumOfCerts);
                write_PR_fully("Trustres", state->spont_pipe, 
                               (char *) &res, sizeof(jboolean));
                break;
            }
        default:
            plugin_error("Did not understand spontaneous code %X\n", 
                         request_code);
        }
    }
}



/* 
 * Inform plugin to terminate current request without a reply
 */
void JavaVM5::TerminateRequestAbruptly(char* infoMsg) {
    TRACE_STR("Abruptly terminating request", infoMsg);
   
    char msg[4];
    put_int(msg, 0, JAVA_PLUGIN_REQUEST_ABRUPTLY_TERMINATED);
    SendRequest(msg, 4, FALSE);

}


/*
 * This method is called when we detect that our child process has 
 * sent us a work request that we ought to process.
 */
void JavaVM5::DoWork()
{
    PRFileDesc* pipe;
    int code; 
    short ix;
    char buff[4];
    JavaPluginInstance5* pluginInstance;

    if (state->work_pipe < 0) {
        plugin_error("JavaVM5::work pipe is dead");
        m_pPluginFactory->ExitMonitor("ProcessWorkQueue-1");
        return;
    }

    PRFileDesc* work_pipe = state->work_pipe;
    pipe = work_pipe;
    
    // First read the request code. The special value of 0 is used
    // to indicate that the child is ready and is otherwise ignored.
    read_PR_fully("WorkCode", work_pipe, buff, 4);
    code = get_int(buff, 0);
    
    TRACE_INT("Obtained next work code code", code);

    read_PR_fully("Instance", work_pipe, buff, 2);

    ix = get_short(buff, 0);
    pluginInstance = m_pPluginFactory->GetInstance(ix);

    /* Now handle the normal cases */
    if (code == JAVA_PLUGIN_SHOW_STATUS) {
        char *mess = ReadWorkString(pipe);
        if (mess == NULL)
	  mess=strdup(" ");
        if (pluginInstance != NULL) {
            nsIPluginInstancePeer *peer = NULL;
            pluginInstance->GetPeer(&peer);
            if (peer != NULL) {
                peer->ShowStatus(mess);
                peer->Release();
            } else {
                plugin_error("No peer found for show status!");
            }
        }
        if (mess) free(mess);
    } else if (code == JAVA_PLUGIN_SHOW_DOCUMENT) {
        char *url = ReadWorkString(pipe);
        char *target = ReadWorkString(pipe);
        if (url == NULL || target == NULL) {
            WorkError(4);
            return;
        }
        TRACE_STR("Show document URL", url);
        TRACE_STR("Show document target", target);
        if (pluginInstance != NULL) {
            nsIPluginManager *mgr = m_pPluginFactory->GetPluginManager();
            nsresult rv  = mgr->GetURL((nsIJVMPluginInstance*)pluginInstance, url, 
							target, NULL);
            TRACE_INT("Return from GetURL",rv);
        }
        free(url);
        free(target);
    } else if (code == JAVA_PLUGIN_FIND_PROXY) {
        char *url = ReadWorkString(pipe);
        char *host = ReadWorkString(pipe);
	if (url == NULL || host == NULL) {
	  WorkError(5);
	  return;
	}

	if (pluginInstance != NULL) {
	  m_pPluginFactory->GetProxySupport()->ProxmapFindProxy((nsIPluginInstance*) pluginInstance, url, host);
	} else {
	  TerminateRequestAbruptly("FindProxy");
	}
        free(url);
        free(host);
    } else if (code == JAVA_PLUGIN_FIND_COOKIE) {
        char *url = ReadWorkString(pipe);
	if (pluginInstance != NULL) {
	    m_pPluginFactory->GetCookieSupport()->FindCookieForURL(pluginInstance,url);
	}
        free(url);
    } else if (code == JAVA_PLUGIN_SET_COOKIE) {
        char *url = ReadWorkString(pipe);
        char *cookie = ReadWorkString(pipe);
	if (pluginInstance != NULL) {
	    m_pPluginFactory->GetCookieSupport()->SetCookieForURL(url, cookie);
	}
        free(url);
        free(cookie);
    } else if (code == JAVA_PLUGIN_CACHE_URL) {
        /* Read an index ('src') which is used as a key by the VM side.
         * Add this to the query "code" and then subtract it when we
         * get the result, and send the key back along with the resulting
         *  filename. (See JavaStream5)
         */
	/* XXXFIX - the cache url must be made to work with the new
	   stream APIs*/
        // Mozilla changes: Can't return plugin_error here, or it'll die
        printf("sleep on it!\n");
        //PR_Sleep(PR_INTERVAL_NO_WAIT);

        char buff[6];
        printf("sending %i\n", JAVA_PLUGIN_CACHED_FILE);
        put_int(buff, 0, JAVA_PLUGIN_CACHED_FILE);
        put_short(buff, 4, 0);
        SendRequest(buff, 6, FALSE);
	//plugin_error("Cache url won't work with the old URL");
	/*
        char *url;
        int src = JAVA_PLUGIN_CACHE_QUERY;
        url = ReadWorkString();
        if (url == NULL) {
            WorkError(7);
            return;
        } 
        if (pluginInstance != NULL) {
            nsIPluginManager* mgr = mPlugin->GetPluginManager();
            pluginInstance->EnterRequest("cache");
            mgr->GetURL(pluginInstance, url, 
							0, (void *) src);
            free(url);
        } else {
            TerminateRequestAbruptly("CacheURL");
        }
        free(url);
	*/
    } else if (code == JAVA_PLUGIN_GET_HTTPS) {

	char *url = ReadWorkString(pipe);
	if (url == NULL) {
	  WorkError(4);
	  return;
	}
	
	TRACE_STR("Get https URL", url);
	if (pluginInstance != NULL) {
	  nsIPluginManager *mgr = m_pPluginFactory->GetPluginManager();
	  HttpsStreamListener *listener = 
	    new HttpsStreamListener(pluginInstance,url,this);
	  listener->AddRef();
	  nsresult rv=mgr->GetURL((nsIJVMPluginInstance*)pluginInstance,url,NULL ,listener);
	  listener->Release();
	  TRACE_INT("Result of get ", rv);
	}
	free(url);
    } else if (code == JAVA_PLUGIN_POST_HTTPS) {
	char *url = ReadWorkString(pipe);
	char *data = ReadWorkByte(pipe);
	char *len_buff=(char *)malloc(4);
        int rc=PR_Read(pipe, len_buff, 4);

	int length = get_int(len_buff, 0);
        if (url == NULL || data == NULL) {
            WorkError(4);
            return;
        }
        TRACE_STR("Post https URL", url);
	if (pluginInstance != NULL) {
	  nsIPluginManager *mgr = m_pPluginFactory->GetPluginManager();
	  PostHttpsListener *listener = new 
	    PostHttpsListener(pluginInstance, url, this);
	  listener->AddRef();
	  nsresult rv= mgr->PostURL((nsIJVMPluginInstance*)pluginInstance, 
				    url, 
				    length,
				    data,
				    PR_FALSE,
				    NULL,
				    listener);
	  
	  listener->Release();
	}
	free(url);
	free(data);
	free(len_buff);
    } else {
        plugin_formal_error("Plugin: unexpected work request from child");
        plugin_error("Code = %0x", code);
    }
}


/*
 * Define the class path for the child VM based on the threads
 * type and paying attention to directory structure differences between
 * JREs. Sets the variable env_CLASSPATH, which is later putenv'd after
 * the fork1 of the child
 * The JRE is either javaplugin.jre.path=... in the .java/properties file
 * or the standard ~/.netscape/java directory
 * The classpath consists of 
 *   $JAVA_PLUGIN_CLASSPATH (internal debugging use only)
 *   $CLASSPATH (any components with the word netscape in them are ignored)
 *   $JRE   lib/rt.jar, lib/classes.zip  lib/classes classes
 *   ~/.netscape/java/javaplugin.jar
 *   Any jar files found in $JRE/lib/ 
 *   ~/.netscape/java/lib/swingall.jar
 */
void JavaVM5::SetUpClasspath(const char * jre) {
    int buff_size, len;
    char *buff, *user_classpath, *cp;
    char *plugin_classpath;     
    DIR *dir;
    struct dirent *de;
    char libpath[400];

    /*
     * Figure out the user's CLASSPATH 
     * We need to skip over the netscape entries on the classpath.
     */
    TRACE("Setting up class path");
    user_classpath = getenv("CLASSPATH");
    if (user_classpath == NULL) {
        user_classpath = "";
    }
    /* If any components of the classpath contain the word netscape in
       them, we just ignore them */
    for (;;) {
        /* Look for netscape in the path */
        cp = strstr(user_classpath, "netscape");
        /* If it does not exist go on */
        if (cp == NULL) {
            break;
        }
        /* Go forward from netscape to the next portion of the path  */
        while (*cp != 0 && *cp != ':') {
            cp++;
        }
        if (*cp == 0) {
            user_classpath = cp;
            break;
        }
        user_classpath = cp + 1;
    }


    buff_size = slen(user_classpath) + 2000 + slen(jvm_classpath);
    buff = (char *)checked_malloc(buff_size);
    sprintf(buff, "CLASSPATH=");
    
    /* plugin_classpath is used *before* the standard class path and
       may be used for substituting the rt.jar with another
       installation for debugging purposes */
    plugin_classpath = getenv("JAVA_PLUGIN_CLASSPATH");
    if (plugin_classpath != NULL) {
        TRACE_STR("Plugin classpath", plugin_classpath);
        strcat(buff, plugin_classpath);
        strcat(buff, ":");
    }

    /*
     * Append jre/lib/rt.jar jre/lib/classes.zip and jre/lib/classes
     */
    strcat(buff, jre);
    strcat(buff, "/lib/rt.jar");
    strcat(buff, ":");
    strcat(buff, jre);
    strcat(buff, "/lib/classes.zip");
    strcat(buff, ":");
    strcat(buff, jre);
    strcat(buff, "/lib/classes");
    strcat(buff, ":");
    strcat(buff, jre);
    strcat(buff, "/classes");
    strcat(buff, ":");

    /*
     * Append the javaplugin JAR
     */
    strcat(buff, state->java_dir);
    strcat(buff, "/lib/javaplugin.jar");
    strcat(buff, ":");

    /* Scan the target JRE directory for .jar files. */
    sprintf(libpath, "%s/lib/", jre);
    dir = opendir(libpath);
    if (dir == NULL) {
        /*fprintf(stderr, "%s %s\n", 
                DGETTEXT(JAVA_PLUGIN_DOMAIN, 
                         "Plugin: couldn't read directory"),
                libpath);*/
        return;
    }
    for (;;) {
        de = readdir(dir);
        if (de == NULL) {
            break;
        }
        cp = strstr(de->d_name, ".jar");
        if (cp && cp[4] == 0 && strcmp(de->d_name, "javaplugin.jar") != 0) {
            /* It's a .jar file. */
            strcat(buff, libpath);
            strcat(buff, de->d_name);
            strcat(buff, ":");
        }
    }
    
    /* put in the class path gotten from mozilla BEFORE the user class path 
       I think it might be useful to adopt the following policy:
       1. Java Plug-in knows best
       2. Netscape Communicator might have a clue about the CLASSPATH, so use
       AddToClasspath
       */
    
    if (jvm_classpath_len > 0)
        strcat(buff, jvm_classpath);

    /*
     * Put Swing classes shipped with the plugin at the end of the classpath.
     */
    strcat(buff, state->java_dir);
    strcat(buff, "/lib/swingall.jar");
    strcat(buff, ":");

    strcat(buff, state->java_dir);
    strcat(buff, "/lib/jsj10.jar");
    strcat(buff, ":");

    len = strlen(buff);
    if (buff[len-1] == ':') {
        buff[len-1] = 0;
    }
    TRACE_STR("Classpath", buff);
    env_CLASSPATH = buff;//    putenv(buff);
}

/* 
 * Extend the class path of hte VM. Only makes sense before
 * starting the actual VM.
 */
void JavaVM5::AddToClassPath(const char *dirPath) {
    int dirpath_len = slen(dirPath);

    if (dirpath_len) { 

        /* are we passed a non-NULL dirPath? */
        if ((jvm_classpath_len + dirpath_len) > jvm_classpath_buflen) { 
            /* do we have enough space in the buffer ? If not, reallocate.*/
            char* buf = (char *) checked_malloc(jvm_classpath_buflen + dirpath_len 
                                                + 500);
            jvm_classpath_buflen += dirpath_len + 500;
            if (jvm_classpath != NULL) {
                strcpy(buf, jvm_classpath);
                free(jvm_classpath);
                jvm_classpath = buf;
            } else {
                jvm_classpath[0] = 0;
            }
        }
        /* Update length */
        jvm_classpath_len += dirpath_len; 
        // enforce protocol: The dirpath does not beginwit a semicolon but it
        // ends with one. 
        if (dirPath[0] == ':') 
            strcat(jvm_classpath, dirPath + 1);
        else
            strcat(jvm_classpath, dirPath);
        if (jvm_classpath[jvm_classpath_len - 1] != ':') {
            jvm_classpath[jvm_classpath_len++] = ':';
            jvm_classpath[jvm_classpath_len] = 0;
        }
    }
}

char* JavaVM5::ReadWorkString(PRFileDesc* pipe) {
    char buff[4];
    short len;
    char *result;
    /* Read the length of the string */
    int rc = PR_Read(pipe, buff, 4);
    if (rc != 4) {
        return NULL;
    }
    len = get_int(buff, 0);
    /* Allocate space and read the string itself */
    result = (char *) checked_malloc(len+1);
    rc = read_PR_fully("ReadWorkStinr",pipe, result, len);
    if (rc != 1) {
        free(result);
        return NULL;
    }
    result[len] = 0;
    return result;
}
char* JavaVM5::ReadWorkByte(PRFileDesc* pipe) {
    char buff[4];
    int len;
    char *result;
    /* Read the length of the string */
    int rc = PR_Read(pipe, buff, 4);
    len = get_int(buff, 0);
    
    /* Allocate space and read the string itself */
    result = (char *) checked_malloc(len+1);
    rc = read_PR_fully("getbyte",pipe, result, len);
    if (rc != 1) {
        free(result);
        return NULL;
    }
    result[len]=0;
    return result;
}


/*
 * Clean up the java process since an error has occurred 
 */
void JavaVM5::Cleanup(void) {

    plugin_formal_error("Plugin: Java VM process has died.");
        
    jvm_status = nsJVMStatus_Failed;
    /* Wipe out the JAVA_PLUGIN_STATE environment variable, close pipes */
    if (state->command_pipe >= 0) {
//        char *buff;
        PR_Close(state->command_pipe);
        PR_Close(state->work_pipe);
        state->command_pipe = 0;
        state->work_pipe = 0;
//        free(state);
//        state = NULL;
//       buff = (char *)checked_malloc(40);
//        sprintf(buff, "JAVA_PLUGIN_STATE=XXX");
//        putenv(buff);
        TRACE("Cleaned up child state");
    }
    TRACE("Waiting for child process to termiante ");
    /* Wait for the child process to terminate */
    if (state->child_pid != 0) {
        int stat = 0;
        int rc = waitpid(state->child_pid, &stat, WNOHANG);
        if (rc > 0) {
            if (WIFEXITED(stat)) {
                fprintf(stderr, "%s %d\n",
                        DGETTEXT(JAVA_PLUGIN_DOMAIN, 
                                 "plugin: java process exited with status"), 
                        WEXITSTATUS(stat));
            } else if (WIFSIGNALED(stat)) {
                fprintf(stderr,"%s %d\n", 
                        DGETTEXT(JAVA_PLUGIN_DOMAIN, 
                                 "plugin: java process died due to signal"), 
                        WTERMSIG(stat));
                if (WCOREDUMP(stat)) {
                    fprintf(stderr, "%s\n", 
                            DGETTEXT(JAVA_PLUGIN_DOMAIN, 
                                     "  a core file was generated"));
                }
            }
        }
        state->child_pid = 0; 
    }
}



/* Send the message to create a new applet */
void JavaVM5::CreateApplet(const char *appletType, int appletNumber, int argc,
                           char **argn, char **argv) {
    char *buff;
    char *bp;
    int i;
    int len = 20;
    
    TRACE_INT("JavaVM JAVA_PLUGIN_NEW creating applet", appletNumber);
    for (i = 0; i < argc; i++) {
        char *s;
        len += 2;
        s = argn[i];
        len += slen(s);
        len += 2;
        s = argv[i];
        len += slen(s);
	if(strcasecmp("mayscript",argn[i]) == 0) {
		TRACE("Found MAYSCRIPT argument");
		if(s == NULL || slen(s) < 1 || s[0] == ' ') {
			TRACE("No value found on MAYSCRIPT providing \"true\" as default");
			len = len - slen(s);
			len = len + slen("true");
		}
	}
        trace("JavaVM args to applet[%d] %d %s %s \n", 
              appletNumber, i, argn[i], argv[i]);
    }

    buff = (char* )checked_malloc(len);
    put_int(buff, 0, JAVA_PLUGIN_NEW);
    put_int(buff, 4, appletNumber);

    if (strstr(appletType, "bean")) {
        /* It's a bean */
        put_int(buff, 8, 1);
    } else {
        /* Its a regular applet */
        put_int(buff, 8, 0);
    }
    put_int(buff, 12, event_window_id);
    put_int(buff, 16, argc);
    bp = buff + 20;
    for (i = 0; i < argc; i++) {
        char *s;
        int l;
        s = argn[i];
        l = slen(s);
        put_short(bp, 0, l);
        bp += 2;
        memcpy(bp, s, l);
        bp += l;

        s = argv[i];
        l = slen(s);
	if(strcasecmp("mayscript",argn[i]) == 0) {
		if(s == NULL || slen(s) < 1 || s[0] == ' ') {
			l = slen("true");
			s = "true";
		}
	}
        put_short(bp, 0, l);
        bp += 2;
        memcpy(bp, s, l);
        bp += l;
    }
    SendRequest(buff, len, TRUE);
    free(buff);
}

/*
 * Send blocking request which waits for a reply
 */
int
JavaVM5::SendBlockingRequest(char *buff, int len)
{
    char rbuff[4];
    int reply;

    /* Pipe monitor must be held */
    m_pPluginFactory->EnterMonitor("SendRequest");

    /* Check that the command pipe is alive */
    if (state->command_pipe <= 0) {
        plugin_error("SendRequest: Bad pipe. Dead VM?");
        return -1;
    }

    if (!write_PR_fully("sendrequest", state->command_pipe, buff, len))
        {
            /* Write the message to the command pipe */
            plugin_error("SendRequest: write failed. Dead VM? %d\n", 
                         errno);
            m_pPluginFactory->ExitMonitor("SendRequest-failedread");
            Cleanup();
            return -1;
        }
    PR_Sync(state->command_pipe);

    /* Get the response */
    if (!read_PR_fully("GetResponse", state->command_pipe, rbuff, 4)) {
        plugin_error("SendRequest: Read of ack failed: %d\n", errno);
        m_pPluginFactory->ExitMonitor("SendRequest-replynotread");
        Cleanup();
        return -1;
    }

    reply = get_int(rbuff, 0);
    m_pPluginFactory->ExitMonitor("SendRequest-ok");

    return reply;
}

/*
     * Send a request to the VM on the command pipe. Depending on
     * wait_for_reply either do or do not wait for an acknowledgement.
     */
void JavaVM5::SendRequest(char *buff, int len, int wait_for_reply)
{
    int rc;
    char rbuff[4];
    int reply;
    static int request_id = 0;  // To identify requests

    if (base_status != nsJVMStatus_Enabled) {
        return;
    }

    /* Pipe monitor must be held */
    m_pPluginFactory->EnterMonitor("SendRequest");

    request_id++;
    TRACE("Sending command ");

    /* Check that the command pipe is alive */
    if (state->command_pipe <= 0) {
        plugin_error("SendRequest: Bad pipe. Dead VM?");
        m_pPluginFactory->ExitMonitor("SendRequest-badpipe");
        return;
    }

    if (!write_PR_fully("sendrequest", 
                        state->command_pipe, buff, len)) {
        /* Write the message to the command pipe */
        plugin_error("SendRequest: write failed. Dead VM? %d\n", 
                     errno);
        m_pPluginFactory->ExitMonitor("SendRequest-failedread");
        Cleanup();
        return;
    }
    rc = PR_Sync(state->command_pipe);

    /* See if we need an ACK from the VM. Return, if not */
    if (wait_for_reply == 0) {
        TRACE("SendRequest: Wrote request. No reply needed.");
        m_pPluginFactory->ExitMonitor("SendRequest-noreply");
        return;
    }
    
    /* Get the ACK */
    TRACE("SendRequest: Wrote request. Waiting for ack.");
    if (!read_PR_fully("GetAck", state->command_pipe, rbuff, 4)) {
        plugin_error("SendRequest: Read of ack failed: %d\n", errno);
        m_pPluginFactory->ExitMonitor("SendRequest-replynotread");
        Cleanup();
        return;
    }

    /* Verify that the ack is OK */
    reply = get_int(rbuff, 0);
    if (reply == JAVA_PLUGIN_OK) {
        TRACE_INT("SendRequest: Read OK acknowledgement", request_id);
    } else {
        Cleanup();
        plugin_formal_error("Java Plug-in ERROR");
        plugin_error("SendRequest: Got an erroneous ack. %d %d\n", request_id,
                     reply);
    }

    m_pPluginFactory->ExitMonitor("SendRequest-ok");
    return;
}



/*
 * Get some response data over the command pipe, other than an OK
 */
void JavaVM5::GetResponseData(char *buff, int len) 
{

    /* Check that the command pipe is alive */
    if (state->command_pipe <= 0) {
        plugin_error("SendRequest: Bad pipe. Dead VM?");
        return;
    }

    /* Pipe monitor must be held */
    m_pPluginFactory->EnterMonitor("GetResponseData");

    /* Read the message from the command pipe */
    read_PR_fully("GetResponse", state->command_pipe, buff, len);

    m_pPluginFactory->ExitMonitor("SendRequest-ok");
    return;
}



/* Create a new remote JNI Env for a given secure jni env  */
RemoteJNIEnv* JavaVM5::CreateRemoteJNIEnv(JNIEnv *proxyenv) {
    m_pPluginFactory->EnterMonitor("createenv");

    RemoteJNIEnv *env =  create_RemoteJNIEnv();
    
    int env_index = m_pPluginFactory->RegisterRemoteEnv(env, proxyenv);

    char msg[4];
    put_int(msg,0, JAVA_PLUGIN_ATTACH_THREAD);
    
    write_PR_fully("SendingAttach", state->command_pipe, (char*)&msg, 4);

    write_PR_fully("SendingAttachPort", state->command_pipe, 
                   (char*)&(state->port), 4);

    PRNetAddr result_addr;

    PRFileDesc* fd = PR_Accept(state->server_socket, &result_addr, 1000000);
    
    if (fd == NULL) {
        plugin_error("Did not accept a connection");
    }

    int junk_ack;
    PR_Read(fd, &junk_ack, 4);

    TRACE("Read the initial ack");

    if (junk_ack != 5050) 
        plugin_error("Could not read initial ack over the new fd");

    junk_ack++;

    PR_Write(fd, &junk_ack, 4);

    TRACE("Wrote the initial ack");

    int change_fd = PRFileDesc_To_FD(fd);

    {
        int flags = fcntl(change_fd, F_GETFL);
        /*      int newflags = flags  | O_NONBLOCK; */
        int newflags = flags & ~O_NONBLOCK;
        int res = fcntl(change_fd, F_SETFL, newflags);
        if (res == -1)
            fprintf(stderr, "\n***CHANGING FLAGS DID NOT WORK ON BROWSER\n");
        flags = fcntl(change_fd, F_GETFL);
        //fprintf(stderr, "\nbrowser flags = %X\n", flags);
    }

    m_pPluginFactory->ExitMonitor("createenv");

    init_RemoteJNIEnv(env, env_index, fd);

    {      
        for(int i = 0; i < 2; i++) {
            env->FindClass("java/lang/System");
            env->ExceptionClear();
            env->ExceptionOccurred();
        }
    }
    return env;

}

/*
 * Define the variables needed by the child process. This merely initializes
 * the env_ variables. They are later putenv'd after the fork.
 */
void JavaVM5::SetupChildEnvironment(void) {
  TRACE("JavaVM5::SetupChildEnvironment");

  // Figure out which JRE to use.
  char *jre = FindJRE();
  TRACE_STR("Using JRE from", jre);

  int jvm_type = FindJVMType();
  TRACE_INT("Using JVM from", jvm_type);

  // Set up the ClASSPATH
  SetUpClasspath(jre);

  char *ld_path = getenv("LD_LIBRARY_PATH");
  char *buff = (char *)malloc(300 + slen(state->java_dir) + 5*slen(jre)
                              + 5*slen(ARCH) + slen(ld_path));

  // The ARCH macro is defined in the makefile.  E.g. -DARCH="sparc"
  // Be careful of the LD_LIBRARY_PATH - beware the dummy libhpi
  // and libjvm.so
  //   In 1.2 the non-dummy versions are in
  //        <jre>/lib/sparc/<thread_type>/libhpi.so
  //  If jre is the same as state->javadir (typically ~/.netscape/java)
  //  then if jre/lib is too early it ends up pointing to
  // .netscape/java/lib
  //  and picks up the (wrong, dummy) version of libhpi etc which are
  //   only meant for 1.1. So put the thread specific directory first
  //   Also: The 1.2fcs build places libjvm in
  //          jre/lib/<arch>/classic/libjvm.so
  // (to permit hotspot later).  So this path is only needed for 1.2

      sprintf(buff, "LD_LIBRARY_PATH=%s/lib/%s/client:%s/lib/%s",
              jre, ARCH,                // libjvm
              jre, ARCH);               // libjava, libawt & friends

  if (slen(ld_path) > 0) {
    // Append the user's LD_LIBRARY_PATH
    strcat(buff, ":");
    strcat(buff, ld_path);
  }
  trace("Library path is %s\n", buff);
  env_LD_LIBRARY_PATH = buff;

  buff =(char *) malloc(100 + slen(jre));
  sprintf(buff, "JAVA_HOME=%s", jre);

  trace(" JAVA_HOME is %s\n", buff);
  env_JAVA_HOME = buff;

  buff =(char *) malloc(100 + slen(state->java_dir));
  sprintf(buff, "PLUGIN_HOME=%s", state->java_dir);

  trace(" PLUGIN_HOME is %s\n", state->java_dir);
  env_PLUGIN_HOME = buff;
}

char *JavaVM5::FindJRE(void) {
    FILE *fin;
    char *home = getenv("HOME");
    char path[1024];
    char line[200];
    char jre[200];
    int rc;

    /* Look in the properties file for javaplugin.jre.path */
    sprintf(path, "%s/.java/properties%s", home,PLUGIN_NODOTVERSION); 

    fin = fopen(path, "r");
    if (fin == NULL) {
        return state->java_dir;
    }

    jre[0] = 0;
    while (fgets(line, sizeof(line), fin)) {
        trace_verbose("%s:%s\n", path,line);
        sscanf(line, "javaplugin.jre.path=%s", (char*) &jre);
    }
    fclose(fin);

    /* If there is a javaplugin.jre.path defined, verify that it is ok */
    if (jre[0] != 0) {
	int len = slen(jre)+1;
        /* First check if the property is set to "Default" .
         */
        if (len > 4) {
            char* jrePrefix = "Default";
            int match = TRUE;
            for(int i = 0; i < 6; i++) {
                if (jre[i] != jrePrefix[i]) 
                    match = FALSE;
            }
            if (match) 
                return state->java_dir;
        }
	
	/* Now check if that directory is reasonable */
        struct stat sbuf;
        sprintf(path, "%s/lib", jre);
        rc = stat(path, &sbuf);
        if (rc == 0) {
            /* We've been given a plausible JRE string */
            int jrelen = slen(jre)+1;
            char *result =(char *) checked_malloc(jrelen);
            memcpy(result, jre, jrelen+1);
            return result;
        }
        plugin_formal_error("Java property javaplugin.jre.path defined as");
        plugin_raw_formal_error(jre);
        plugin_formal_error("But that directory does not exist."); 
        plugin_formal_error("Using JRE from");
        plugin_raw_formal_error(state->java_dir);
    }
    return state->java_dir;
}

/*
 */
bool JavaVM5::FindEnabled(void) {
    FILE *fin;
    char *home = getenv("HOME");
    char path[1024];
    char line[200];
    char jre[200];


    // Look in the properties file
    sprintf(path, "%s/.java/properties%s", home,PLUGIN_NODOTVERSION);	

    fin = fopen(path, "r");
    if (fin == NULL) {
	return true;
    }

    jre[0] = 0;
    while (fgets(line, sizeof(line), fin)) {
        trace("%s:%s\n",path, line);
	sscanf(line, "javaplugin.enabled=%s", &jre);
    }
    fclose(fin);

    if (strstr(jre, "true") == NULL) return false;
    return true;
}

/*
 */
int JavaVM5::FindJVMType(void) {
    FILE *fin;
    char *home = getenv("HOME");
    char path[1024];
    char line[200];
    char jre[200];


    // Look in the properties file
    sprintf(path, "%s/.java/properties%s", home,PLUGIN_NODOTVERSION);	

    fin = fopen(path, "r");
    if (fin == NULL) {
	return 1;
    }

    jre[0] = 0;
    while (fgets(line, sizeof(line), fin)) {
        trace("%s:%s\n", path,line);
	sscanf(line, "javaplugin.jre.params=%s", &jre);
    }
    fclose(fin);

    if (strstr(jre, "-classic") != NULL) return 0;
    return 1;
}

void JavaVM5::FindJavaDir(void) {
    Dl_info dlinfo;
    static int dummy;
    char buf[PATH_MAX];
    char real[PATH_MAX];

    // Now: Only use the JRE which came with the plugin.

    // Formerly:
    //   Use the NPX_PLUGIN_PATH to find the directories holding plugins
    //   Find the first of those that has a ../java directory.

    state->java_dir = NULL;
    dladdr((void *)&dummy, &dlinfo);
    strcpy(buf, dlinfo.dli_fname);
   
    *(strrchr(buf, '/')) = '\0';  /* executable file      */
    *(strrchr(buf, '/')) = '\0';  // NS600 subdirector used to isolate ongoing version
    *(strrchr(buf, '/')) = '\0';  /* $(ARCH)              */
    *(strrchr(buf, '/')) = '\0';  /* plugin               */

    if (realpath(buf, real) == NULL) {
        fprintf(stderr, "Error: realpath(`%s') failed.\n", buf);
        return;
    }
    state->java_dir = strdup(real);
}


/*
 * Report an error in performing work, and terminate the jvm
 */
void JavaVM5::WorkError(int x) {
    perror("Plugin worker error");
    fprintf(stderr, "%s (%d)\n",
            DGETTEXT(JAVA_PLUGIN_DOMAIN, 
                     "Plugin: trouble with work request from child"),
            x);
    Cleanup();
}



/* For testing the secure JNI Env */
extern void TestSecureJNI(nsISecureEnv* secenv, RemoteJNIEnv* env);

/* Test the created vm */
void JavaVM5::TestVM(void) {
    int i;
    RemoteJNIEnv* env = CreateRemoteJNIEnv(NULL);
    if (env == NULL) {
        plugin_error("Creation of remote JNI Env does not work !");
    }
    jclass clz_System;

    //fprintf(stderr, "env");
    for(i = 0; i < 1000; i++)
        clz_System = env->FindClass("java/lang/System");


    fprintf(stderr, "clz_System");
    jfieldID fid_System_out = 
        env->GetStaticFieldID(clz_System, "out", "Ljava/io/PrintStream;");
    fprintf(stderr, "fid %d\n", (int) fid_System_out);
    jobject obj_System_out = env->GetStaticObjectField(clz_System, 
                                                       fid_System_out);
    fprintf(stderr, "ojb out field %d\n", (int) obj_System_out);
    jclass clz_PrintStream = env->FindClass("java/io/PrintStream");
    fprintf(stderr, "printstream clz %d\n", (int) clz_PrintStream);         
    jmethodID print_method = env->GetMethodID(clz_PrintStream, "println",
                                              "(Ljava/lang/String;)V");
    fprintf(stderr, "mid");
    jstring msg = env->NewStringUTF("Tickling the VM!!!");
    fprintf(stderr, "new");
    env->CallVoidMethod(obj_System_out, print_method, msg);

    /* Create a new array */
    jbyteArray arr = env->NewByteArray(10);
    /* Get the elements of the array with no copying */
    signed char *elements = env->GetByteArrayElements(arr, 0);
    for (i = 0; i < 10; i++) {
        elements[i] = 'a' + i;
    }
    env->ReleaseByteArrayElements(arr, elements, 0);


    jclass clz_Plugin = env->FindClass("sun/plugin/navig/motif/Plugin");
    jmethodID mid_Plugin_printArray = 
        env->GetStaticMethodID(clz_Plugin, "printArray", 
                               "([B)V");
    fprintf(stderr, "mid_Plugin_printArray %d\n", (int) mid_Plugin_printArray);
    env->CallStaticVoidMethod(clz_Plugin, mid_Plugin_printArray, arr);
    fprintf(stderr, "call");
    
    CSecureJNIEnv* secjni = new CSecureJNIEnv(NULL, env);

    TestSecureJNI(secjni, env);
}

//NS_DEFINE_IID(kICapsManager_IID, NS_ICAPSMANAGER_IID);
//NS_DEFINE_IID(kCCapsManager_CID, NS_CCAPSMANAGER_CID);


//=--------------------------------------------------------------------------=
// For signed applet support. Should be moved into its own file
//=--------------------------------------------------------------------------=
// IsAllPermissionGranted calls back Mozilla to check if a principal has
// already been granted or should be granted AllPermission. 
//
// parameters :
//  cont char*            [in]    codebase
//  const unsigned char** [in]    certificate chain
//  int*                  [in]    certificate lengths
//  int                   [in]    chain length (number of certificates)
//
// out:
//  jboolean              [out]   true iff AllPermission is granted, false
//                                otherwise
//XXXFIX not yet tested at all.
jboolean 
IsAllPermissionGranted(const char *codebaseURL,
                       const unsigned char **certChain,
                       unsigned int *certLengths,
                       int numOfCerts)
{
  // Mozilla changes: PENDING(mark): I haven't ported this part yet
  // since the Mozilla security headers have changed!
  /*
    jboolean ret = JNI_FALSE;

    JavaPluginFactory5* m_pPluginFactory = get_global_factory();

  CPluginServiceProvider* service_provider = 
    m_pPluginFactory->GetServiceProvider();

  if (!service_provider)   
    return JNI_FALSE;

  nsICapsManager* pCapsMgr = NULL;

  if (NS_SUCCEEDED(service_provider
		   ->QueryService(kCCapsManager_CID,
				  kICapsManager_IID,
				  (nsISupports**) &pCapsMgr))) {
    nsIPrincipal* pPrin = NULL;
            
    if ((codebaseURL != NULL) && (certChain != NULL)) {
      pCapsMgr->CreateCodeSourcePrincipal(certChain,
					  certLengths,
					  numOfCerts,
					  codebaseURL,
					  &pPrin);
    } else if (certChain != NULL) {
      pCapsMgr->CreateCertPrincipal(certChain, 
				    certLengths, numOfCerts,
				    &pPrin);
    } else if (codebaseURL != NULL) {
      pCapsMgr->CreateCodebasePrincipal(codebaseURL, &pPrin);
    } else {
      service_provider->ReleaseService(kCCapsManager_CID,
				       pCapsMgr);
      return JNI_FALSE;
    }
    

    // check if principal is trusted
    PRBool trusted;
    pPrin->IsTrusted(NULL, &trusted);
	
    if (trusted == PR_TRUE)  {
      // Check if principal already is in CapsManager database
      nsPermission perm;
      pCapsMgr->GetPermission(pPrin, NS_ALL_PRIVILEGES, &perm);
      if ((perm == nsPermission_AllowedSession) ||
	  (perm == nsPermission_AllowedForever)) {
	// principal already has NS_ALL_PRIVILEGES
	service_provider->ReleaseService(kCCapsManager_CID,
					   pCapsMgr);
	ret = JNI_TRUE;
      } else if ((perm == nsPermission_DeniedSession) ||
		 (perm == nsPermission_DeniedForever)) {
	service_provider->ReleaseService(kCCapsManager_CID,
					   pCapsMgr);
	ret = JNI_FALSE;
      } else {
	pCapsMgr->AskPermission(pPrin, NS_ALL_PRIVILEGES, &perm);
		
	if ((perm == nsPermission_AllowedSession) ||
	    (perm == nsPermission_AllowedForever)) {
	  // grant code JDK_ALL_PERMISSION
	  service_provider->ReleaseService(kCCapsManager_CID,
					   pCapsMgr);
	  ret = JNI_TRUE;
	}
      }
    } else {
      service_provider->ReleaseService(kCCapsManager_CID,
				       pCapsMgr);

      ret = JNI_FALSE;
    }

  }
  service_provider->ReleaseService(kCCapsManager_CID,
				   pCapsMgr);

  return ret;
  */
  return JNI_TRUE;
}
