/* 
   /*
    *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

#include <stdio.h>
#include <stdlib.h>
#include "nsplugin.h"
#include "Debug.h"
#include "nsIBaseStream.h"
#include "nsIPluginStream.h"
#include "nsIPluginStreamPeer.h"
#include "NetscapeStream5.h"


#define TRACE(m) trace("NetscapeStream::%s\n", m);

#define UNUSED(x) x=x

const long NetscapeStream5::kMaxBufSize = 0X0FFFFFFF;

NetscapeStream5::NetscapeStream5(nsIPluginStreamPeer* stream)
    :	stream_peer( stream )
{
    TRACE("NetscapeStream5 construct");

    NS_INIT_REFCNT();

    stream_peer->AddRef();
}

NetscapeStream5::~NetscapeStream5()
{
    TRACE("NetscapeStream5::~NetscapeStream5\n");

    if (stream_peer != NULL)
	stream_peer->Release();
}


NS_DEFINE_IID(kStreamIID, NS_IBASESTREAM_IID);
NS_DEFINE_IID(kPluginStreamIID, NS_IPLUGINSTREAM_IID);
NS_DEFINE_IID(kOutputStreamIID, NS_IOUTPUTSTREAM_IID);
NS_DEFINE_IID(kSupportsIID, NS_ISUPPORTS_IID);
NS_IMPL_ADDREF(NetscapeStream5);
NS_IMPL_RELEASE(NetscapeStream5);

/**
 * A run time mechanism for interface discovery.
 * aIID [in] A requested interface IID
 * aInstancePtr [out] A pointer to an interface pointer to
 * receive the result.
 * returns NS_OK if the interface is supported by the associated
 * instance, NS_NOINTERFACE if it is not. 
 * NS_ERROR_INVALID_POINTER if aInstancePtr is NULL
 */
nsresult NetscapeStream5::QueryInterface(REFNSIID aIID, void** aInstancePtr)  {

    TRACE("QueryInterface\n");

    if (NULL == aInstancePtr)                                         
	return NS_ERROR_NULL_POINTER;  

    if (aIID.Equals(kStreamIID) || aIID.Equals(kPluginStreamIID) 
	|| aIID.Equals(kOutputStreamIID) || aIID.Equals(kSupportsIID))  {
	*aInstancePtr = (void*) this;
	AddRef();

	return NS_OK;		
    }
    else
	return NS_NOINTERFACE;
}


/* Write some data to this stream. Do nothing for now */
NS_IMETHODIMP
NetscapeStream5::Write(const char* unused_buffer, PRInt32 offset, PRInt32 len,
		       PRInt32 *bytesWritten)
{
    if (tracing) trace("Write %d %d \n", (int) offset, (int) len);
    *bytesWritten = len;
    UNUSED(unused_buffer);
    return NS_OK;
}

/* Close this stream */
NS_IMETHODIMP NetscapeStream5::Close(void) {
    TRACE("Close");
    return NS_OK;
}

// (Corresponds to NPP_NewStream's stype return parameter.)
NS_IMETHODIMP
NetscapeStream5::GetStreamType(nsPluginStreamType *result)
{
    TRACE("GetStreamType");

    *result =  nsPluginStreamType_Normal;
    return NS_OK;
}


/* Return the stream as a file */
NS_IMETHODIMP
NetscapeStream5::AsFile(const char* unused_fname)
{ 
    TRACE("AsFile");
    UNUSED(unused_fname);
    return NS_OK;
}
