/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-*/
/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/VendorS.h>
#include "remotejni.h"
#include "nsIPluginInstance.h"
#include "nsIJVMPluginInstance.h"
#include "nsIEventHandler.h"
#include "nsplugin.h"
#include "Navig5.h"
#include "nsplugindefs.h"
#include "PluginPrint.h"

PluginPrint::PluginPrint(JavaPluginInstance5 *inst, 
			 nsPluginPrint* printinfo)
  :mInst(inst), mPrintInfo(printinfo)
{
  mInst->AddRef();
}

/* Desctruction */
PluginPrint::~PluginPrint()
{
  if (mInst != NULL)
    mInst->Release();
  
}

/* Actual Printing method */
NS_IMETHODIMP PluginPrint::Print()
{
  if (mPrintInfo->mode == nsPluginMode_Full)
    {
      return FullPrint();
    }
  else 
    {
      return EmbedPrint();
    }
  return NS_ERROR_FAILURE;
}


/* If the request was a fullPrint */
NS_IMETHODIMP PluginPrint::FullPrint()
{
  mPrintInfo->print.fullPrint.pluginPrinted=PR_FALSE;
  return NS_OK;
}

/*
 * Embed Print does the following:
 * 1) Send message to Plugin to print requested area
 * 2) Request is send and received ack that print pipe is ready
 * 3) Poll the printpipe untile you receive ok message
 * 
 *
 */
NS_IMETHODIMP PluginPrint::EmbedPrint()
{
  FILE *fp;
  JavaVM5 *jvm ;
  sendRequest();
  nsPluginEmbedPrint ep;
  ep = mPrintInfo->print.embedPrint;
  void *platformPrint = (nsPluginPrintCallbackStruct*)(ep.platformPrint);
  
  fp = ((nsPluginPrintCallbackStruct *)(platformPrint))->fp;
  jvm =mInst->GetPluginFactory()->GetJavaVM();
  jvm->ReceivePrinting(fp);
  
  return NS_OK;
}

void PluginPrint::sendRequest()
{
  nsPluginEmbedPrint embedPrint= mPrintInfo->print.embedPrint;
  nsPluginWindow window=embedPrint.window;
  char buff[24];
  put_int(buff, 0 , JAVA_PLUGIN_PRINT);
  put_int(buff, 4,  mInst->GetPluginNumber());
  put_int(buff, 8, window.x);
  put_int(buff, 12, window.y);
  put_int(buff, 16, window.width);
  put_int(buff, 20, window.height);
  mInst->GetPluginFactory()->SendRequest(buff, 24, FALSE);
  free(buff);
} 
