/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/*
 * Contains implementation of the CPostHttpsListener
 */
#include "remotejni.h"
#include "nsIPluginInstance.h"
#include "nsIJVMPluginInstance.h"
#include "nsIEventHandler.h"
#include "nsplugin.h"
#include "Navig5.h"
#include "JavaPluginInstance5.h"
#include "nsIJVMPluginInstance.h"
#include "nsIPluginStreamListener.h"
#include "CNetscapeStreamListener.h"
#include "HttpsStreamListener.h"
#include "PostHttpsListener.h"
#include "Debug.h"

/*
 * PostHttpsListener Methods
 */

/*
 * PostHttpsListener::PostHttpsListener
 */
PostHttpsListener::PostHttpsListener(JavaPluginInstance5* inst,
				     const char* url,
				     JavaVM5* jvm):
    CNetscapeStreamListener(inst, url),
    mJvm(jvm)
{
    m_fSignalled=0;
    m_pszHttpsResult = NULL;
    m_iHttpsResultLength = 0;
}

/*
 * PostHttpsListener::OnDataAvailable
 */
NS_METHOD 
PostHttpsListener::OnDataAvailable(nsIPluginStreamInfo* pluginInfo, 
				     nsIInputStream* input,
				     PRUint32 length)
{

  if (pluginInfo == NULL || input == NULL)
    return NS_ERROR_NULL_POINTER;

  nsresult result = NS_OK;

  // Check if HTTPS buffer size is big enough
  if (m_pszHttpsResult == NULL)
    {
      // No buffer has been allocated yet, allocate new buffer.
      m_pszHttpsResult = (char*) malloc(length);
    }
  else
    {
	// Buffer has been allocated before, realloc buffer. This is necessary 
	// to deal with dynamic content because we could not determine the 
	//content length until the connection is closed.
      m_pszHttpsResult = (char*) realloc((void *)m_pszHttpsResult, 
					 m_iHttpsResultLength + length);
    }

  // Copy the data into the buffer
  PRUint32 amountRead = 0;
  result = input->Read((m_pszHttpsResult + m_iHttpsResultLength), length, &amountRead);

  // Increment buffer length that has been read
  m_iHttpsResultLength += amountRead;

  return result;
}


/*
 * PostHttpsListener::~PostHttpsListener
 */
PostHttpsListener::~PostHttpsListener(void)
{
     if ( !m_fSignalled )
    {
       HttpsPostDone(NULL, 0);
    }
}

/*
 * PostHttpsListener::OnStopBinding
 */
NS_METHOD 
PostHttpsListener::OnStopBinding(nsIPluginStreamInfo* pluginInfo, nsresult status)
{

    HttpsPostDone(m_pszHttpsResult, m_iHttpsResultLength);

    // from the base class
    m_fSignalled = 1;
    m_pszHttpsResult = NULL;
    m_iHttpsResultLength = 0;

    return NS_OK;

}

void PostHttpsListener::HttpsPostDone(char *msg, int len)
{
  char *buff = (char *)malloc(len +8);
  put_int(buff, 0, JAVA_PLUGIN_HTTPS_POST_REPLY);
  put_int(buff, 4, len);
  memcpy(buff+8, msg, len); 
  mJvm->SendRequest(buff, len+8, FALSE);
  free(buff);
}
