/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


/*
 * Contains declaration of the HttpsStreamListener
 */

#ifndef POSTHTTPSLISTENER_H
#define POSTHTTPSLISTENER_H


class JavaPluginInstance5;
class HttpsStreamListener;

class PostHttpsListener : public CNetscapeStreamListener {

 public:
  
  /**
   * Notify the observer that the URL has finished loading.  This method is 
   * called once when the networking library has finished processing the 
   * URL transaction initiatied via the nsINetService::Open(...) call.<BR><BR>
   * 
   * This method is called regardless of whether the URL loaded successfully.<BR><BR>
   * 
   * @param status    Status code for the URL load.
   * @param msg   A text string describing the error.
   * @return The return value is currently ignored.
   */
  
  NS_IMETHOD
    OnStopBinding(nsIPluginStreamInfo* info, nsresult status);
 
  /**
   * Notify the client that data is available in the input stream.  This
   * method is called whenver data is written into the input stream by the
   * networking library...<BR><BR>
   * 
   * @param aIStream  The input stream containing the data.  This stream can
   * be either a blocking or non-blocking stream.
   * @param length    The amount of data that was just pushed into the stream.
   * @return The return value is currently ignored.
   */
  NS_IMETHOD
    OnDataAvailable(nsIPluginStreamInfo* info, nsIInputStream* input,
		    PRUint32 length);

  /*
   * PostStreamListener specific methods:
   */
  // send https post reply back to the worker thread
  PostHttpsListener( JavaPluginInstance5* inst, const char *url, JavaVM5 *jvm);

  // send https post reply back to the worker thread
  void HttpsPostDone(char* msg, int len);

  virtual ~PostHttpsListener(void);

 protected:

  char* m_pszHttpsResult;
  int m_iHttpsResultLength;
  int m_fSignalled;
  JavaVM5* mJvm;
};

#endif // POSTHTTPSLISTENER_H
