/*
 *  Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 * 
 * Author: Robert Sczewyczk, Benedict Gomes
 *
 */

/* 
 * This module provides support for querying for proxy URLs from the browser.
 * In the intialization phase the auto proxy URL is first found from the
 * netscape preferences file. state->proxy_config_url.
 * A whole bunch of javascript functions along with any responses from
 * asking for the autoproxy url from the host are stored in 
 *  state->proxy_script, which is then used together with the javascript
 * command javascript:FindProxyForURL to get the proxy of a URL.
 * The response comes in via ProxmapReply.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <libintl.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include "remotejni.h"
#include "nsIJVMPluginInstance.h"
#include "Navig5.h"
#include "ProxyScript.h"
#include "ProxySupport5.h"
extern "C" {
#ifdef __linux__
extern int gethostname(char *, size_t);
#else
extern int gethostname(char *, int);
#endif
}
/*
NS_DEFINE_IID(kNetworkManagerIID, NS_INETWORKMANAGER_IID);
*/

ProxySupport5::ProxySupport5(JavaVM5* javaVM_arg) {
    javaVM = javaVM_arg;
    read_buff = NULL;
    read_buff_size = 0;
}

/* 
 * Initialize the proxy support by sending a query to the browser
 * of the autoproxy URL, which results in several calls back 
 * to ProxmapInput and finally ProxmapInputDone.
 */
void ProxySupport5::ProxmapInitialize(nsIPluginInstance *inst) {

    const char *cp;
    char *bp;
    char host[256];
    struct hostent *hp;
    struct in_addr in;

    trace("ProxmapInitialize: starting...\n");
    state = javaVM->state;
    if (state->proxy_config_url == NULL) {
	trace("ProxymapInitialize: FindingProxyConfigUrl...\n");
	FindProxyConfigUrl();	
    }

    if (state->proxy_config_url == (char *)-1) {
	trace("ProxymapInitialize: null config_url - proxies not supported\n");
	return;
    }
    if (state->proxy_script != NULL){
	trace("ProxymapInitialize: proxy_script not null \n");
	return;
    }
    if ( read_buff != NULL) {
	trace("ProxymapInitialize: readbuf not null %s\n", read_buff);
	return;
    }

    /*
     * We setup some standard JavaScript functions expected by
     * the user supplied configuration scripts.
     */
    trace("ProxymapInitialize: setting up buffer...\n");
    read_buff_size = 12000;
    read_buff = (char *) malloc(read_buff_size);
    read_buff[0] = 0;
    strcat(read_buff, "javascript: ");
    strcat(read_buff, g_jsGlobal);
    strcat(read_buff, g_jsDnsDomainIs);
    strcat(read_buff, g_jsIsInNet);
    strcat(read_buff, g_jsIsPlainHostName);
    strcat(read_buff, g_jsIsResolvable);
    strcat(read_buff, g_jsLocalHostOrDomainIs);
    strcat(read_buff, g_jsDnsDomainLevels);
    strcat(read_buff, g_jsDnsResolve);
    trace("ProxymapInitialize: Constructed buffer\n");

    /* We have to do a substitution for our IP address*/
    gethostname(host, sizeof(host));
    trace("ProxymapInitialize: Called gethostname\n");
    hp = gethostbyname(host);
    trace("ProxymapInitialize: Called gethostbyname\n");
    if (hp == 0 || hp->h_addr_list == 0 || hp->h_addr_list[0] == 0) {
	fprintf(stderr, "%s\n",
		dgettext(JAVA_PLUGIN_DOMAIN, 
			 "Plugin: can't find IP address for current host???"));
    } else {
	trace("Found host address\n");
        memcpy(&in.s_addr, hp->h_addr_list[0], sizeof (in.s_addr));
    }

    bp = read_buff + strlen(read_buff);
    cp = g_jsMyIpAddress;
    while (*cp) {
	if (cp[0] == '%' && cp[1] == 's') {
	    cp += 2;
	    sprintf(bp, "%s", inet_ntoa(in));
	    bp = bp+strlen(bp);
	    continue;
	}
	*bp++ = *cp++;
    }
    *bp++ = 0;

    strcat(read_buff, g_jsShExpMatch);
    strcat(read_buff, g_jsEnableDateRange);
    strcat(read_buff, g_jsEnableTimeRange);
    strcat(read_buff, g_jsEnableWeekdayRange);
    
    nsIPluginManager *mgr = javaVM->GetPluginFactory()->GetPluginManager();
    trace("GetURL JAVA_PLUGIN_PROXMAP_INPUT\n");
    UNUSED(inst);
    /*mgr->GetURL(inst, state->proxy_config_url, 0, 
		(void *)JAVA_PLUGIN_PROXMAP_INPUT); */
}

/*
 * As a result of proxmap initialization this method gets called, 
 * indirectly via a stream. The URL request is JAVA_PLUGIN_PROXMAP_INPUT
 * and the response comes back through a JavaStream5. The result
 * is appended to the read_buff. When a plugin instance gets 
 * JAVA_PLUGIN_PROXMAP_INPUT, it indicates that getting input is done and
 * ProxmapInputDone is called, then the state->proxy_script is set to 
 * the read_buff
 */
void ProxySupport5::ProxmapInput(char *buffer, int len) {
    int old_length = strlen(read_buff);
    int new_length = old_length + len;

    trace("Entering proxmapInput\n");
    // Store the result in the read_buff. 
    // This is efficient but confusing - BG
    if (new_length >= read_buff_size) {
	int new_size = new_length + 5000;
	char *new_buff = (char *)malloc(new_size);
	memcpy(new_buff, read_buff, old_length);
	free(read_buff);
	read_buff = new_buff;
	read_buff_size = new_size;
    }
    // Store the result in the read buffer
    memcpy(read_buff + old_length, buffer, len);
    read_buff[new_length] = 0;
    trace("Result of proxmap input in read_buf %s\n", read_buff);
}

/*
 * Called when a JAVA_PLUGIN_PROXMAP_INPUT returns with a write
 * on the JavaPluginInstance instead of creating a stream.
 * This indicates that all the initialization is over and 
 * we can store the read_buff in proxy_script
 * Called by some plugin instance.
 */
void ProxySupport5::ProxmapInputDone(nsPluginReason reason) {
    char buff[4];
    
    if (reason != nsPluginReason_Done) {
	char *errmsg = dgettext(JAVA_PLUGIN_DOMAIN, 
		      "Plugin: trouble reading Netscape proxy "
				" configuration URL from\n");
	fprintf(stderr, "%s\n", errmsg);
	fprintf(stderr, "           %s\n", state->proxy_config_url);
	trace("ProxmapInputDone... some badness %s %s \n", errmsg,
	      state->proxy_config_url);
	state->proxy_config_url = 0;
	return;
    }
    /* 
     * Finally, we can set the proxmap in the worker, so that 
     * proxy requests can go through
     */
    trace("Sending JAVA_PLUGIN_PROXMAP_READY to Worker with proxy\n");
    trace("ProxySupport5::ProxmapInputDone\n");
    state->proxy_script = read_buff;

}
 
/* Internal use - private method
 * Called by the proxmap initialization to find out the autoproxy URL
 * if the state->proxy_config_url is not set. 
 * If Navigator version < 3, just return. Otherwise look at the 
 * .netscape/preferences.js file for the
 *    user_pref("network.proxy.type", 2) and
 *    user_pref("network.proxy.autoconfig_url", <some url>) and
 * set state->proxy_config_url to this autoproxy URL.
 */
void ProxySupport5::FindProxyConfigUrl()
{
    char *home = getenv("HOME");
    FILE *fin;
    char path[1024];
    char line[200];
    char config_url[200];
    int type, rc;

    trace("In FindProxyConfigURL\n");
    state->proxy_config_url = (char *)-1;
    config_url[0] = 0;
    type = 0;

    sprintf(path, "%s/.netscape/preferences.js", home);	
    trace("preferences file: %s \n", path);
    fin = fopen(path, "r");
    if (fin == NULL) {
	trace("FindProxyConfigUrl: Could not open preferences %s\n", path);
	return;
    }

    /* Look for the either autoconfig_url and/or the proxy type */
    while (fgets(line, sizeof(line), fin)) {
	trace_verbose("Next line of preferences: %s\n", line);
	sscanf(line, "user_pref(\"network.proxy.type\", %d);", &type);
        rc = sscanf(line,
		    "user_pref(\"network.proxy.autoconfig_url\", \"%s\");", 
		    &config_url);
	if (rc == 1) {
	    trace("Found proxy config in preferences file\n");
	    /* Get rid of (put null before) trailing junk after ); */
	    char *tail = strstr(config_url, "\");");
	    if (tail) {
		tail[0] = 0;
	   }
	}
    }
    fclose(fin);
    trace("Closed preferences file\n");

    // See if http (type==2 ?) uses the autoproxy.
    if (type == 2 && config_url[0] != 0) {
	state->proxy_config_url = (char *)malloc(4+strlen(config_url));
	strcpy(state->proxy_config_url, config_url);
	trace("FindProxyConfigUrl: type==2 \n\t%s \n\t%s \n ", 
	      config_url, state->proxy_config_url);
    } else {
	trace("FindProxyConfigUrl:  -1 return \n ");
	state->proxy_config_url = (char *) -1;
    }
}


/*
 * This function will eventually provide the proxy for a given URL
 * This sends a JAVA_PLUGIN_PROXMAP_QUERY to the peer with a javascript
 * command 
 *     javascript:FindProxyForURL(<url>,<host>)
 * The GetURL will eventually return via a JavaStream5 which,  calls
 * ProxmapReply.
 * It is invoked by the Worker thread via the JavaVM5, through the 
 * JAVA_PLUGIN_FIND_PROXY command. 
 */
void ProxySupport5::ProxmapFindProxy(nsIPluginInstance *inst,
				    char * url,
				    char * host) {
  /* First issue (or reissue) the script definition.  Then issue the
   * specific mapping request * We'll get an answer later through a
   * stream write * redircted through to proxmap_reply
   */
  
  char *proxyResult;
  
  nsIPluginManager2 *mgr = javaVM->GetPluginFactory()->GetPluginManager();
  
  if (mgr->FindProxyForURL(url , &proxyResult) != NS_OK)
      return;

  ProxmapReply(url, strlen(proxyResult), proxyResult);

  free((void *)proxyResult);

  UNUSED(inst);

}

/* 
 * Called by the browser (via a stream) with the response of a proxy request
 * from a worker thread
 * The chain of events is:
 *   Worker thread -> JAVA_PLUGIN_FIND_PROXY to plugin
 *   JavaVM5::doWork  -> ProxySupport5->ProxmapFindProxy
 *   ProxmapFindProxy -> GetURL javascript:findproxyforurl...
 *                        -> Browser  JAVA_PLUGIN_PROXMAP_QUERY
 *.  Browser -> JavaPluginInstance new Stream
 *   Browser -> Stream write    JAVA_PLUGIN_PROXMAP_QUERY
 *   New stream write -> ProxySupport5.ProxmapReply
 */
void ProxySupport5::ProxmapReply(const char *stream_url, int len, 
				 void *buffer)
{
    char *url;
    char *cp;
    char *packet;
    int l;

    /*
     * Figure out the URL that was being resolved.
     */
    trace("Proxmap reply for an original JAVA_PLUGIN_PROXY_REQUEST");
    url = stream_url;
    trace("Mapping for proxy : %s ", url);
    packet = (char *) malloc(strlen(url) + len + 8);
    put_int(packet, 0, JAVA_PLUGIN_PROXY_MAPPING);
    cp = packet + 4;
    l = strlen(url);
    put_short(cp, 0, l);
    cp += 2;
    memcpy(cp, url, l);
    cp += l;
    put_short(cp, 0, len);
    cp += 2;
    memcpy(cp, buffer, len);
    cp += len;

    javaVM->SendRequest(packet, l + len + 8, FALSE);

    free(packet);
}

