/* Call a method on Java object in LiveConnect through reflection and JNI. */

NS_IMETHODIMP CSecureJNI2::CallJavaMethod(/*[in]*/  jobject obj, 
                                          /*[in]*/  jobject method, 
                                          /*[in]*/  jvalue* arg, 
                                          /*[in]*/  nsISecurityContext* ctx, 
                                          /*[out]*/ jvalue* result)
{
    trace("CSecureJNI2::CallJavaMethod\n");

    const char *cName = "sun/plugin/liveconnect/SecureInvocation";
    const char *mName = "CallMethod";
    const char *sig = "(Ljava/lang/Object;Ljava/lang/reflect/Method;III)Z";
    jboolean ret = 0;

    do 	// Simulate goto
    {
        if (ctx)
            ctx->AddRef();

		// Clear all exceptions
		m_env->ExceptionClear();

        jclass secure = m_env->FindClass(cName);
		if (!secure) 
			continue;

        jmethodID m = m_env->GetStaticMethodID(secure, mName, sig);
		if (!m) 
			continue;
    
		ret = m_env->CallStaticBooleanMethod(secure, m, obj, 
						     method, (jint)arg, 
						     (jint)ctx, (jint)result);
    }
    while (0);

    if (ret)
        return NS_OK;
    else
        return NS_ERROR_FAILURE;
}



/* Get a field on Java object in LiveConnect through reflection and JNI. */
NS_IMETHODIMP CSecureJNI2::GetJavaField(/*[in]*/  jobject obj, 
                                        /*[in]*/  jobject field, 
                                        /*[in]*/  nsISecurityContext* ctx, 
                                        /*[out]*/ jvalue* result)
{
    trace("CSecureJNI2::GetJavaField\n");

    const char *cName = "sun/plugin/liveconnect/SecureInvocation";
    const char *mName = "GetField";
    const char *sig = "(Ljava/lang/Object;Ljava/lang/reflect/Field;II)Z";
    jboolean ret = 0;

    do 	// Simulate goto
    {
        if (ctx)
            ctx->AddRef();
 
        // Clear all exceptions
		m_env->ExceptionClear();

        jclass secure = m_env->FindClass(cName);
		if (!secure) 
			continue;

        jmethodID m = m_env->GetStaticMethodID(secure, mName, sig);
		if (!m) 
			continue;
    
		ret = m_env->CallStaticBooleanMethod(secure, m, obj, 
						     field, (jint)ctx, 
						     (jint)result);
    }
    while (0);

    if (ret)
        return NS_OK;
    else
        return NS_ERROR_FAILURE;
}


/* Set a field on Java object in LiveConnect through reflection and JNI. */
NS_IMETHODIMP CSecureJNI2::SetJavaField(/*[in]*/ jobject obj, 
                                        /*[in]*/ jobject field, 
                                        /*[in]*/ jvalue val, 
                                        /*[in]*/ nsISecurityContext* ctx)
{
    trace("CSecureJNI2::SetJavaField\n");

    const char *cName = "sun/plugin/liveconnect/SecureInvocation";
    const char *mName = "SetField";
    const char *sig = "(Ljava/lang/Object;Ljava/lang/reflect/Field;II)Z";
    jboolean ret = 0;

    do 	// Simulate goto
    {
        if (ctx)
            ctx->AddRef();

		// Clear all exceptions
		m_env->ExceptionClear();

        jclass secure = m_env->FindClass(cName);
		if (!secure) 
			continue;

        jmethodID m = m_env->GetStaticMethodID(secure, mName, sig);
		if (!m) 
			continue;
    
		ret = m_env->CallStaticBooleanMethod(secure, m, 
						     obj, field, 
						     (jint)&val, (jint)ctx);
    }
    while (0);

    if (ret)
        return NS_OK;
    else
        return NS_ERROR_FAILURE;
}


