/*
 * Copyright (c) 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

//=--------------------------------------------------------------------------=
//
// NSReg.h  by Stanley Man-Kit Ho
//
//=--------------------------------------------------------------------------=

#ifndef _NSREG_H_
#define _NSREG_H_

/* --------------------------------------------------------------------
 * Miscellaneous Definitions
 * --------------------------------------------------------------------
 */
#define MAGIC_NUMBER    0x76644441L
#define MAJOR_VERSION   1          /* major version for incompatible changes */
#define MINOR_VERSION   2          /* minor ver for new (compatible) features */
#define PATHDEL         '/'
#define HDRRESERVE      128        /* number of bytes reserved for hdr */
#define INTSIZE         4
#define DOUBLESIZE      8

#define PACKBUFFERSIZE  2048


/* Node types */
#define REGTYPE_KEY                   (1)
#define REGTYPE_DELETED               (0x0080)

/* Private standard keys */
#define ROOTKEY                       (0x20)
#define ROOTKEY_VERSIONS              (0x21)

/* strings for standard keys */
#define ROOTKEY_STR             "/"
#define ROOTKEY_VERSIONS_STR    "Version Registry"
#define ROOTKEY_USERS_STR       "Users"
#define ROOTKEY_COMMON_STR      "Common"
#define ROOTKEY_PRIVATE_STR     "Private Arenas"

#define OLD_VERSIONS_STR        "ROOTKEY_VERSIONS"
#define OLD_USERS_STR           "ROOTKEY_USERS"
#define OLD_COMMON_STR          "ROOTKEY_COMMON"

/* needs to be kept in sync with PE. see ns/cmd/winfe/profile.h */
/* and ns/cmd/macfe/central/profile.cp */
#define ASW_MAGIC_PROFILE_NAME "User1"

/* macros */
#define COPYDESC(dest,src)  memcpy((dest),(src),sizeof(REGDESC))

#define VALID_FILEHANDLE(fh)    ((fh) != NULL)

#define INVALID_NAME_CHAR(p)    ( ((unsigned char)(p) < 0x20) )

#define TYPE_IS_ENTRY(type)       ( (type) & REGTYPE_ENTRY )
#define TYPE_IS_KEY(type)         ( !((type) & REGTYPE_ENTRY) )

#define VERIFY_HREG(h)\
    ( ((h) == NULL) ? REGERR_PARAM : \
    ( (((REGHANDLE*)(h))->magic == MAGIC_NUMBER) ? REGERR_OK : REGERR_BADMAGIC ) )



/* --------------------------------------------------------------------
 * Types and Objects
 * --------------------------------------------------------------------
 */
//#undef REGOFF
/*
#ifndef _UINT16
#define _UINT16
typedef unsigned short uint16;
#endif

#ifndef _UINT32
#define _UINT32
typedef unsigned int uint32;
#endif

#ifndef _INT16
#define _INT16
typedef short int16;
#endif
*/
#ifndef _UINT16
#define _UINT16
typedef unsigned short uint16;
#endif

#ifndef _INT32
#define _INT32
typedef int int32;
#endif


typedef int32 REGOFF;   /* offset into registry file */

typedef struct _desc
{
    REGOFF  location;   /* this object's offset (for verification) */
    REGOFF  name;       /* name string */
    uint16  namelen;    /* length of name string (including terminator) */
    uint16  type;       /* node type (key, or entry style) */
    REGOFF  left;       /* next object at this level (0 if none) */
    REGOFF  down;       /* KEY: first subkey        VALUE: 0 */
    REGOFF  value;      /* KEY: first entry object  VALUE: value string */
    uint32  valuelen;   /* KEY: 0  VALUE: length of value data */
    uint32  valuebuf;   /* KEY: 0  VALUE: length available */
    REGOFF  parent;     /* the node on the immediate level above */
} REGDESC;

/* offsets into structure on disk */
#define DESC_LOCATION   0
#define DESC_NAME       4
#define DESC_NAMELEN    8
#define DESC_TYPE       10
#define DESC_LEFT       12
#define DESC_DOWN       16
#define DESC_VALUE      20
#define DESC_VALUELEN   24
#define DESC_VALUEBUF   16    /* stored in place of "down" for entries */
#define DESC_PARENT     28

#define DESC_SIZE       32    /* size of desc on disk */

typedef struct _hdr
{
    uint32  magic;      /* must equal MAGIC_NUMBER */
    uint16  verMajor;   /* major version number */
    uint16  verMinor;   /* minor version number */
    REGOFF  avail;      /* next available offset */
    REGOFF  root;       /* root object */
} REGHDR;

/* offsets into structure on disk*/
#define HDR_MAGIC       0
#define HDR_VERMAJOR    4
#define HDR_VERMINOR    6
#define HDR_AVAIL       8
#define HDR_ROOT        12

typedef FILE FILEHANDLE; /* platform-specific file reference */

#endif  /* _NSREG_H_ */


