/*
 *  Copyright (C) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/*
 *--------------------------------------------------------------------------
 * ProxyScript.cpp  -- By Stanley Man-Kit Ho
 *--------------------------------------------------------------------------
 * implements all emulated JavaScript functions which are used by IE and 
 * Navigator for finding the proxy setting.
 */

#ifndef _PROXYSCRIPT_H_
#define _PROXYSCRIPT_H_

#ifdef _IE_SCRIPT_
#define JS_TYPE	LPCOLESTR
#define JS_TEXT(x)	L ## x

#else
#define JS_TYPE	const char*
#define JS_TEXT(x)	x    
#endif

JS_TYPE g_jsGlobal = 
JS_TEXT("var _mon = new Array('JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC');")
JS_TEXT("var _day = new Array('SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT');")
JS_TEXT("function _isGmt(i) {")
JS_TEXT(   " return typeof i == 'string' && i == 'GMT'; }");


JS_TYPE g_jsDnsDomainIs = 
JS_TEXT("function dnsDomainIs(host, domain) {")
JS_TEXT(    "if (domain != null && domain.charAt(0) != '.')")
JS_TEXT(	"return false;")
JS_TEXT(    "return shExpMatch(host, '*' + domain); }");


JS_TYPE g_jsIsInNet = 
JS_TEXT("function isInNet(host, pattern, mask){")
JS_TEXT(    "if (typeof host != 'string' || typeof pattern != 'string' || typeof mask != 'string') return false; ")
JS_TEXT(    "if (dnsDomainLevels(pattern) != 3 || dnsDomainLevels(mask) != 3)")
JS_TEXT(	"return false;")
JS_TEXT(    "var offset1 = 0, offset2 = 0, ofs1 = 0, ofs2 = 0;")
JS_TEXT(    "var m = '', result;")
JS_TEXT(    "for (var i=0; i < 4; i++){")
JS_TEXT(	"ofs1 = pattern.indexOf('.', offset1);")
JS_TEXT(	"ofs2 = mask.indexOf('.', offset2);")
JS_TEXT(	"if (i != 3)")
JS_TEXT(	    "result = (pattern.substring(offset1, ofs1) - 0) & (mask.substring(offset2, ofs2) - 0);")
JS_TEXT(	"else ")
JS_TEXT(	    "result = (pattern.substring(offset1) & mask.substring(offset2));")
JS_TEXT(	"if (result == 0)")
JS_TEXT(	    "result = '*';")
JS_TEXT(	"m = m + result;")
JS_TEXT(	"if (i != 3)")
JS_TEXT(	    "m = m + '.';")
JS_TEXT(	"offset1 = ofs1 + 1;")
JS_TEXT(	"offset2 = ofs2 + 1;")
JS_TEXT(    "}")
JS_TEXT(    "return shExpMatch(host, m); }");



JS_TYPE g_jsIsPlainHostName = 
JS_TEXT("function isPlainHostName(host){")
JS_TEXT(    "return (dnsDomainLevels(host) == 0); }");



JS_TYPE g_jsIsResolvable = 
JS_TEXT("function isResolvable(host){")
JS_TEXT(    "return (dnsResolve(host) != ''); }");



JS_TYPE g_jsLocalHostOrDomainIs = 
JS_TEXT("function localHostOrDomainIs(host, hostdom){")
JS_TEXT(    "return shExpMatch(hostdom, host + '*'); }");



JS_TYPE g_jsDnsDomainLevels = 
JS_TEXT("function dnsDomainLevels(host){")
JS_TEXT(    "var s = host + '';")
JS_TEXT(    "for (var i=0, j=0; i < s.length; i++)")
JS_TEXT(	"if (s.charAt(i) == '.')")
JS_TEXT(	    "j++;")
JS_TEXT(    "return j; }");



JS_TYPE g_jsDnsResolve = 
JS_TEXT("function dnsResolve(host){")
JS_TEXT(    "if (typeof host != 'string' || dnsDomainLevels(host) != 3) return ''; ")
JS_TEXT(    "for (var i=0; i < host.length; i++) ")
JS_TEXT(        "if ((host.charAt(i) < '0' || host.charAt(i) > '9') && host.charAt(i) != '.') return ''; ")
JS_TEXT(    "return host; }");



JS_TYPE g_jsMyIpAddress = 
JS_TEXT("function myIpAddress(){")
JS_TEXT(    "return '%s'; }");


JS_TYPE g_jsShExpMatch = 
JS_TEXT("function shExpMatch(str, shexp){ ")
JS_TEXT(    " if (typeof str != 'string' || typeof shexp != 'string') return false; ")
JS_TEXT(    " if (shexp == '*') return true; ")
JS_TEXT(    " if (str == '' && shexp == '') return true; ")
JS_TEXT(    " str = str.toLowerCase();")
JS_TEXT(    " shexp = shexp.toLowerCase();")
JS_TEXT(    " var index = shexp.indexOf('*');")
JS_TEXT(    " if (index == -1) { return (str == shexp); } ")
JS_TEXT(    " else if (index == 0) { ")
JS_TEXT(	" for (var i=0; i <= str.length; i++) { ")
JS_TEXT(	    " if (shExpMatch(str.substring(i), shexp.substring(1))) return true; ")
JS_TEXT(	" } return false; } ")
JS_TEXT(     "else { ")
JS_TEXT(	"var sub = null, sub2 = null; ")
JS_TEXT(	"sub = shexp.substring(0, index);")
JS_TEXT(        "if (index <= str.length) sub2 = str.substring(0, index); ")
JS_TEXT(	"if (sub != '' && sub2 != '' && sub == sub2) { ")
JS_TEXT(	    "return shExpMatch(str.substring(index), shexp.substring(index)); }")
JS_TEXT(	"else {	return false; }")
JS_TEXT(     "} }");


JS_TYPE g_jsEnableDateRange = 
JS_TEXT("function _dateRange(day1, month1, year1, day2, month2, year2, gmt){")
JS_TEXT(    "if (typeof day1 != 'number' || day1 <= 0 || typeof month1 != 'string' || typeof year1 != 'number' || year1 <= 0")
JS_TEXT(      " || typeof day2 != 'number' || day2 <= 0 || typeof month2 != 'string' || typeof year2 != 'number' || year2 <= 0")
JS_TEXT(      " || typeof gmt != 'boolean') return false; ")

JS_TEXT(    "var m1 = -1, m2 = -1;")

JS_TEXT(    "for (var i=0; i < _mon.length; i++){")
JS_TEXT(	"if (_mon[i] == month1)")
JS_TEXT(            "m1 = i;")
JS_TEXT(        "if (_mon[i] == month2)")
JS_TEXT(            "m2 = i;")
JS_TEXT(    "}")

JS_TEXT(    "var cur = new Date();")
JS_TEXT(    "var d1 = new Date(year1, m1, day1, 0, 0, 0);")
JS_TEXT(    "var d2 = new Date(year2, m2, day2, 23, 59, 59);")

JS_TEXT(    "if (gmt == true)")
JS_TEXT( 	"cur = new Date(cur.getTime() - cur.getTimezoneOffset() * 60 * 1000);")

JS_TEXT(    "return ((d1.getTime() <= cur.getTime()) && (cur.getTime() <= d2.getTime()));")
JS_TEXT("}")
JS_TEXT("function dateRange(p1, p2, p3, p4, p5, p6, p7){")
JS_TEXT(    "var cur = new Date();")

JS_TEXT(    "if (typeof p1 == 'undefined')")
JS_TEXT(        "return false;")
JS_TEXT(    "else if (typeof p2 == 'undefined' || _isGmt(p2))")
JS_TEXT(    "{")
JS_TEXT(        "if ((typeof p1) == 'string')")
JS_TEXT(             "return _dateRange(1, p1, cur.getYear(), 31, p1, cur.getYear(), _isGmt(p2));")
JS_TEXT(        "else if (typeof p1 == 'number' && p1 > 31)")
JS_TEXT(             "return _dateRange(1, 'JAN', p1, 31, 'DEC', p1, _isGmt(p2));")
JS_TEXT(	"else {")
JS_TEXT(             "for (var i=0; i < 11; i++)")
JS_TEXT(                  "if (_dateRange(p1, _mon[i], cur.getYear(), p1, _mon[i], cur.getYear(), _isGmt(p2)))")
JS_TEXT(	              " return true;")
JS_TEXT(	     "return false;")
JS_TEXT(	"}")
JS_TEXT(     "}")
JS_TEXT(     "else if (typeof p3 == 'undefined' || _isGmt(p3))")
JS_TEXT(     "{")
JS_TEXT(	"if ((typeof p1) == 'string')")
JS_TEXT(             "return _dateRange(1, p1, cur.getYear(), 31, p2, cur.getYear(), _isGmt(p3));")
JS_TEXT(        "else if (typeof p1 == 'number' && typeof p2 == 'number' && (p1 > 31 || p2 > 31))")
JS_TEXT(             "return _dateRange(1, 'JAN', p1, 31, 'DEC', p2, _isGmt(p3));")
JS_TEXT(        "else ")
JS_TEXT(        "{")
JS_TEXT(	     "if ((typeof p2) == 'string')")
JS_TEXT(             "{")
JS_TEXT(		  "return _dateRange(p1, p2, cur.getYear(), p1, p2, cur.getYear(), _isGmt(p3));")
JS_TEXT(             "}")
JS_TEXT(             "else ")
JS_TEXT(             "{")
JS_TEXT(                  "for (var i=0; i < 11; i++)")
JS_TEXT(                       "if (_dateRange(p1, _mon[i], cur.getYear(), p2, _mon[i], cur.getYear(), _isGmt(p3)))")
JS_TEXT(	                   "return true;")
	
JS_TEXT( 	      	  "return false;")
JS_TEXT(             "}")
JS_TEXT(         "}")
JS_TEXT(     "}")
JS_TEXT(     "else if (typeof p4 == 'undefined' || _isGmt(p4))")
JS_TEXT(         "return _dateRange(p1, p2, p3, p1, p2, p3, _isGmt(p4));")
JS_TEXT(     "else if (typeof p5 == 'undefined' || _isGmt(p5))")
JS_TEXT(     "{")
JS_TEXT(         "if (typeof p2 == 'number')")
JS_TEXT(              "return _dateRange(1, p1, p2, 31, p3, p4, _isGmt(p5));")
JS_TEXT(         "else ")
JS_TEXT(              "return _dateRange(p1, p2, cur.getYear(), p3, p4, cur.getYear(), _isGmt(p5))")
JS_TEXT(     "}")
JS_TEXT(     "else if (typeof p6 == 'undefined')")
JS_TEXT(         "return false;")
JS_TEXT(     "else ")			
JS_TEXT(         "return _dateRange(p1, p2, p3, p4, p5, p6, _isGmt(p7));")
JS_TEXT("}");


JS_TYPE g_jsEnableTimeRange = 
JS_TEXT("function timeRange(p1, p2, p3, p4, p5, p6, p7) {")
JS_TEXT(      "if (typeof p1 == 'undefined')")
JS_TEXT(           "return false;")
JS_TEXT(      "else if (typeof p2 == 'undefined' || _isGmt(p2))")
JS_TEXT(	   "return _timeRange(p1, 0, 0, p1, 59, 59, _isGmt(p2));")
JS_TEXT(      "else if (typeof p3 == 'undefined' || _isGmt(p3))")
JS_TEXT(           "return _timeRange(p1, 0, 0, p2, 0, 0, _isGmt(p3));")
JS_TEXT(      "else if (typeof p4 == 'undefined')")
JS_TEXT(           "return false;")
JS_TEXT(      "else if (typeof p5 == 'undefined' || _isGmt(p5))")
JS_TEXT(           "return _timeRange(p1, p2, 0, p3, p4, 0, _isGmt(p5));")
JS_TEXT(      "else if (typeof p6 == 'undefined')")
JS_TEXT(           "return false;")
JS_TEXT(      "else ")		
JS_TEXT(           "return _timeRange(p1, p2, p3, p4, p5, p6, _isGmt(p7));")
JS_TEXT("}")
JS_TEXT("function _timeRange(hour1, min1, sec1, hour2, min2, sec2, gmt) {")
JS_TEXT(      "if (typeof hour1 != 'number' || typeof min1 != 'number' || typeof sec1 != 'number' ")
JS_TEXT(          "|| hour1 < 0 || min1 < 0 || sec1 < 0 ")
JS_TEXT(	  "|| typeof hour2 != 'number' || typeof min2 != 'number' || typeof sec2 != 'number' ")
JS_TEXT(	  "|| hour2 < 0 || min2 < 0 || sec2 < 0 ")
JS_TEXT(	  "|| typeof gmt != 'boolean')  return false; ")
JS_TEXT(      "var cur = new Date();")
JS_TEXT(      "var d1 = new Date();")
JS_TEXT(      "var d2 = new Date();")
JS_TEXT(      "d1.setHours(hour1);")
JS_TEXT(      "d1.setMinutes(min1);")
JS_TEXT(      "d1.setSeconds(sec1);")
JS_TEXT(      "d2.setHours(hour2);")
JS_TEXT(      "d2.setMinutes(min2);")
JS_TEXT(      "d2.setSeconds(sec2);")
JS_TEXT(      "if (gmt == true)")
JS_TEXT(          "cur = new Date(cur.getTime() - cur.getTimezoneOffset() * 60 * 1000);")
JS_TEXT(      "return ((d1.getTime() <= cur.getTime()) && (cur.getTime() <= d2.getTime()));")
JS_TEXT("}");


JS_TYPE g_jsEnableWeekdayRange = 
JS_TEXT("function weekdayRange(wd1, wd2, gmt){")
JS_TEXT(      "if (typeof wd1 == 'undefined') ")
JS_TEXT(	  "return false;")
JS_TEXT(      "else if (typeof wd2 == 'undefined' || _isGmt(wd2)) ")
JS_TEXT(          "return _weekdayRange(wd1, wd1, _isGmt(wd2)); ")
JS_TEXT(      "else ")					
JS_TEXT(	  "return _weekdayRange(wd1, wd2, _isGmt(gmt)); }")

JS_TEXT("function _weekdayRange(wd1, wd2, gmt) {")
JS_TEXT(      "if (typeof wd1 != 'string' || typeof wd2 != 'string' || typeof gmt != 'boolean') return false; ")
JS_TEXT(      "var w1 = -1, w2 = -1;")
JS_TEXT(      "for (var i=0; i < _day.length; i++) {")
JS_TEXT(          "if (_day[i] == wd1)")  
JS_TEXT(              "w1 = i;")
JS_TEXT(          "if (_day[i] == wd2)")
JS_TEXT(              "w2 = i; }")
JS_TEXT(      "var cur = new Date();")
JS_TEXT(      "if (gmt == true)")
JS_TEXT(          "cur = new Date(cur.getTime() - cur.getTimezoneOffset() * 60 * 1000);")
JS_TEXT(      "var w3 = cur.getDay();")
JS_TEXT(      "if (w1 > w2)")
JS_TEXT(          "w2 = w2 + 7;")
JS_TEXT(      "if (w1 > w3)")
JS_TEXT(          "w3 = w3 + 7;")
JS_TEXT(      "return (w1 <= w3 && w3 <= w2); }");


/*
 ***********************************************************************
 ***********************************************************************
 *  THIS IS FOR TEMPORARY INTERNAL TESTING ONLY
 ***********************************************************************
 ***********************************************************************
 */
/*
JS_TYPE g_jsFindProxyForURL = 
JS_TEXT("function FindProxyForURL(url, host) {")
JS_TEXT(    "if (isPlainHostName(host) ||")
JS_TEXT(	"dnsDomainIs(host, '.corp') ||")
JS_TEXT(	"dnsDomainIs(host, '.corp.sun.com') ||")
JS_TEXT(	"dnsDomainIs(host, '.ebay') ||")
JS_TEXT(	"dnsDomainIs(host, '.ebay.sun.com') ||")
JS_TEXT(	"dnsDomainIs(host, '.eng') ||")
JS_TEXT(	"dnsDomainIs(host, '.eng.sun.com')) {")
JS_TEXT(    "return 'DIRECT' ;")
JS_TEXT(    "}")
JS_TEXT(    "else if (isInNet('129.144.81.0', myIpAddress(), '255.255.255.0')) {")
JS_TEXT(	"return 'PROXY webcache-lhd.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT(    "else if (isInNet('129.144.6.0', myIpAddress(), '255.255.255.0')) {")
JS_TEXT(	"return 'PROXY webcache-svl.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache-mtv-01.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT(    "else if (isInNet('129.144.75.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.85.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.89.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.123.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.124.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.125.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.170.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.171.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.172.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.173.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.174.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.175.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.176.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.177.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.250.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.251.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.252.0', myIpAddress(), '255.255.255.0')) ")
JS_TEXT(    "{")
JS_TEXT(	"return 'PROXY webcache-cup.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache-mpk-01.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT(    "else if (isInNet('129.144.25.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.26.0', myIpAddress(), '255.255.255.0')) {")
JS_TEXT(	"return 'PROXY netracache1.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache-mpk-01.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT(    "else if (isInNet('129.144.204.0', myIpAddress(), '255.255.255.0') ||")
JS_TEXT(	    "isInNet('129.144.205.0', myIpAddress(), '255.255.255.0')) {")
JS_TEXT(	"return 'PROXY netracache2.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache-mpk-01.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT(    "else if (isInNet('129.144.0.0', myIpAddress(), '255.255.0.0')) {")
JS_TEXT(	"return 'PROXY webcache-mtv-01.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache1.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT(    "else {")
JS_TEXT(	"return 'PROXY webcache-mpk-01.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache1.Eng.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.Corp.Sun.COM:8080;' +")
JS_TEXT(	"'PROXY webcache.EBay.Sun.COM:8080';")
JS_TEXT(    "}")
JS_TEXT("}");
*/

#endif //_PROXYSCRIPT_H_

