/*
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

//=--------------------------------------------------------------------------=
// ProxySupport.h	by Stanley Man-Kit Ho
//=--------------------------------------------------------------------------=
// Contains declaration of the ProxySupport functions.
//

#ifndef _PROXYSUPPORT_H_
#define _PROXYSUPPORT_H_

#ifdef __cplusplus
extern "C" {
#endif

#define PROXY_TYPE_NO_PROXY	0
#define PROXY_TYPE_MANUAL	1
#define PROXY_TYPE_AUTOCONFIG	2



//=--------------------------------------------------------------------------=
// Proxy_ExtractHostName
//=--------------------------------------------------------------------------=
// Proxy_ExtractHostName is a function which takes a URL and returns the 
// hostname in the URL.
//
// parameters :
//	lpszURL		[in]	URL for hostname extraction
//	lpszHostName	[out]	hostname to return
//
//
void Proxy_ExtractHostName(const char* lpszURL, char* lpszHostName);


//=--------------------------------------------------------------------------=
// Proxy_GetCurrentIpAddress
//=--------------------------------------------------------------------------=
// Proxy_GetCurrentIpAddress is a function which takes a JNIEnv pointer and 
// returns the IP address of the local machine.
//
// parameters :
//	JNIEnv*		[in]	JNI pointer
//	lpszIpAddress	[out]	IP address to return.
//
//
void Proxy_GetCurrentIpAddress(JNIEnv* env, char* szIpAddress);



//=--------------------------------------------------------------------------=
// Proxy_NSCanonicalizeProxyOverride
//=--------------------------------------------------------------------------=
// Proxy_NSCanonicalizeProxyOverride is a function which takes a proxy-bypass-list
// which is read directly from the Netscape Navigator preference, and returns 
// the canonicalized version of the proxy-bypass-list which can be used in
// Proxy_ExtractManualProxySetting function.
//
// parameters :
//      lpszProxyOverride [in,out]  a string which contains a list of domain which
//  				    proxy should not be used when a URL is requested.
//
// Notes: i) lpszProxyOverride contains all the domain which no proxy should be used
//        when a connection is made. For example, "a.eng,b.eng,c.eng". In Netscape
//        Navigator preference, no wildcard is allowed in the list. In order to use
//        the proxy-bypass-list in Proxy_ExtractManualProxySetting, the string must
//        be converted into "*.a.eng,*.b.eng,*.c.eng".
//
//
void Proxy_NSCanonicalizeProxyOverride(char* lpszProxyOverride);



//=--------------------------------------------------------------------------=
// Proxy_IECanonicalizeProxyOverride
//=--------------------------------------------------------------------------=
// Proxy_IECanonicalizeProxyOverride is a function which takes a proxy-bypass-list
// which is read directly from the Internet Explorer preference, and returns 
// the canonicalized version of the proxy-bypass-list which can be used in
// Proxy_ExtractManualProxySetting function.
//
// parameters :
//      lpszProxyOverride [in,out]  a string which contains a list of domain which
//  				    proxy should not be used when a URL is requested.
//
// Notes: i) lpszProxyOverride contains all the domain which no proxy should be used
//        when a connection is made. For example, "a.eng;b.*.eng;c.eng". In order 
//        to use the proxy-bypass-list in Proxy_ExtractManualProxySetting, all ';'
//        in the string must be converted into ','.
//
//
void Proxy_IECanonicalizeProxyOverride(char* lpszProxyOverride);


//=--------------------------------------------------------------------------=
// indexOf
//=--------------------------------------------------------------------------=
// indexOf is a function which takes a string and a substring, and returns the
// first occurance index of the substring in the original string. If the 
// substring is not found, -1 is returned.
//
// parameters :
//	lpszString	[in]	a string to be searched
//      lpszSubstring	[in]	substring to search for
//
// out:
//	int	        [out]   index of the substring inside the original 
//                              string. If substring is not found, -1 is returned.
//
//
int indexOf(const char* lpszString, const char* lpszSubstring);

#ifdef __cplusplus
}
#endif


#endif // _PROXYSUPPORT_H_
