/*
 * @(#)ConverterGUI.java	1.10 00/08/20
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *  @version 1
 *
 */

package sun.plugin.converter.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import sun.plugin.converter.ResourceHandler;
import sun.plugin.converter.util.*;
import sun.plugin.converter.engine.*;
import javax.swing.*;
import javax.swing.border.*;


/**
 * Frame which contains the major component of the GUI of <B><I>Cuckoo</I></B>.
 *
 */
public class ConverterGUI extends JFrame 
			  implements ActionListener, ItemListener
{
    private JButton dirBttn, backupBttn, runBttn;
    private JCheckBox recursiveCheckBox;
    private JTextField dirTF, matchingTF, backupTF;
    private TemplateFileChoice templateCh;

    JMenuItem exitMenuItem, optionMenuItem, helpMenuItem, aboutMenuItem;

    private static Image bgImage; 
    private static Image dukeImage;

    static
    {
	try
	{
	    // Load background gif
	    InputStream is = ConverterGUI.class.getClassLoader().getResourceAsStream("images/background.gif");
	    ByteArrayOutputStream bos = new ByteArrayOutputStream();

	    byte[] buf = new byte[2048];
	    int len = 0;

	    while ((len = is.read(buf, 0, 2048)) != -1)
		bos.write(buf, 0, len);

	    bgImage = new ImageIcon(bos.toByteArray()).getImage();

	    bos.close();
	    is.close();

	    // Load icon gif
	    is = ConverterGUI.class.getClassLoader().getResourceAsStream("images/duke.gif");
	    bos = new ByteArrayOutputStream();

	    len = 0;

	    while ((len = is.read(buf, 0, 2048)) != -1)
		bos.write(buf, 0, len);

	    dukeImage = new ImageIcon(bos.toByteArray()).getImage();

	    bos.close();
	    is.close();
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	}
    }


    private String[] descriptors;

    private String defaultDirBackupPath, defaultOneFileBackupPath;

    private boolean changingItem = false;

    private boolean firstTime = true;

    private static PluginConverter converter;
    private ConvertSet convertSet;

    /**
     * Creates new ConverterGUI Frame.
     */
    public ConverterGUI()
    {
	super(ResourceHandler.getMessage("product_name"));   // UniCode (tm) == \u2122

	try
	{
	    converter = new PluginConverter();
	    converter.setShowProgressStdOut(false);
	}
	catch (FileNotFoundException e)
	{
	    showNoTemplateDialog();
	}
	converter.setConvertSet(convertSet = new DefaultSet());

	setResizable(false);

	setIconImage(dukeImage);

	majorLayout();
	setup();
    }

    /**
     * Overrides java.awt.Container.setVisible() so that we are
     * sure that the Frame's size is set before displaying.
     */
    public void setVisible(boolean state)
    {
	if (state)
	{
            Dimension bounds = getToolkit().getScreenSize();
            setLocation((bounds.width - getSize().width)/2,
        	        (bounds.height - getSize().height)/2);
        }

	super.setVisible(state);
    }

    /**
     * Creates the GUI.
     */
    public void majorLayout()
    {
	//
	// Setup Menu 
	//
	JMenuBar menuBar = new JMenuBar();
	JMenu file = new JMenu(ResourceHandler.getMessage("menu.file"));
	file.add(exitMenuItem = new JMenuItem(ResourceHandler.getMessage("menu.exit")));
	exitMenuItem.addActionListener(this);
	menuBar.add(file);

	JMenu edit = new JMenu(ResourceHandler.getMessage("menu.edit"));
	edit.add(optionMenuItem = new JMenuItem(ResourceHandler.getMessage("menu.option")));
	optionMenuItem.addActionListener(this);
	menuBar.add(edit);

	JMenu help = new JMenu(ResourceHandler.getMessage("menu.help"));
	help.add(helpMenuItem = new JMenuItem(ResourceHandler.getMessage("menu.help")));
	help.add(new JSeparator());
	help.add(aboutMenuItem = new JMenuItem(ResourceHandler.getMessage("menu.about")));
	helpMenuItem.addActionListener(this);
	aboutMenuItem.addActionListener(this);
	menuBar.add(help);

	setJMenuBar(menuBar);


	//
	// Setup main GUI
	//
	JLabel dirLabel = new JLabel(ResourceHandler.getMessage("converter_gui.lablel0"));
	dirTF        = new JTextField();
	dirBttn      = new JButton(ResourceHandler.getMessage("button.browse"));
	JLabel matchingLabel  = new JLabel(ResourceHandler.getMessage("converter_gui.lablel1"));
	matchingTF   = new JTextField(ResourceHandler.getMessage("converter_gui.lablel2"));
	recursiveCheckBox  = new JCheckBox(ResourceHandler.getMessage("converter_gui.lablel3"));

	recursiveCheckBox.setOpaque(false);

	JLabel backupLabel = new JLabel(ResourceHandler.getMessage("converter_gui.lablel5"));
	backupTF     = new JTextField();
	backupBttn   = new JButton(ResourceHandler.getMessage("button.browse"));

	JLabel templateLabel = new JLabel(ResourceHandler.getMessage("converter_gui.lablel7"));
	templateCh   = new TemplateFileChoice();
	runBttn      = new JButton(ResourceHandler.getMessage("button.convert"));

	JSeparator sep = new JSeparator();

	addListeners();

	    final int buf=10,           // Buffer (between components and form)
		      sp=10,            // Space between components
		      vsp=5,            // Vertical space
		      indent=20;        // Indent between form (left edge) and component


	GridBagConstraints gbc = new GridBagConstraints();
	GridBagLayout gbl = new GridBagLayout();
	getContentPane().setLayout(gbl);

	//
	// Setup top panel
	//
	GridBagLayout topLayout = new GridBagLayout();
	JPanel topPanel = new JPanel();
	topPanel.setOpaque(false);
	topPanel.setLayout(topLayout);

	topLayout.setConstraints(dirLabel, new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.WEST, 
					GridBagConstraints.NONE, new Insets(10,0,0,0), 0, 0));
	topLayout.setConstraints(dirTF, new GridBagConstraints(0, 1, 1, 1, 1, 0, GridBagConstraints.WEST, 
					GridBagConstraints.HORIZONTAL, new Insets(vsp,0,0,0), 0, 0));
	topLayout.setConstraints(dirBttn, new GridBagConstraints(1, 1, 1, 1, 0, 0, GridBagConstraints.CENTER, 
					  GridBagConstraints.NONE, new Insets(10,sp,0,0), 0,0));
	topLayout.setConstraints(matchingLabel, new GridBagConstraints(0, 2, 1, 1, 0, 0, GridBagConstraints.WEST, 
					  GridBagConstraints.NONE, new Insets(10,0,0,0), 0,0));
	topLayout.setConstraints(matchingTF, new GridBagConstraints(0, 3, 1, 1, 1, 0, GridBagConstraints.WEST, 
					GridBagConstraints.HORIZONTAL, new Insets(vsp,0,0,0), 0, 0));
	topLayout.setConstraints(recursiveCheckBox, new GridBagConstraints(0, 4, GridBagConstraints.REMAINDER, 1, 1, 0, 
					      GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(vsp,0,0,0), 0,0));
	topLayout.setConstraints(sep, new GridBagConstraints(0, 5, GridBagConstraints.REMAINDER, 1, 1, 0, 
					GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(10,10,10,10), 0, 0));
	topLayout.setConstraints(backupLabel, new GridBagConstraints(0, 6, 1, 1, 0, 0, GridBagConstraints.WEST, 
					GridBagConstraints.NONE, new Insets(10,0,0,0), 0, 0));
	topLayout.setConstraints(backupTF, new GridBagConstraints(0, 7, 1, 1, 1, 0, GridBagConstraints.CENTER, 
					GridBagConstraints.HORIZONTAL, new Insets(vsp,0,0,0), 0, 0));
	topLayout.setConstraints(backupBttn, new GridBagConstraints(1, 7, 1, 1, 0, 0, GridBagConstraints.CENTER, 
					  GridBagConstraints.NONE, new Insets(10,sp,0,0), 0,0));
	topLayout.setConstraints(templateLabel, new GridBagConstraints(0, 8, 1, 1, 0, 0, GridBagConstraints.WEST, 
					GridBagConstraints.NONE, new Insets(10,0,0,0), 0, 0));
	topLayout.setConstraints(templateCh, new GridBagConstraints(0, 9, 1, 1, 0, 0, GridBagConstraints.WEST, 
					GridBagConstraints.HORIZONTAL, new Insets(vsp,0,0,0), 0, 0));

	JButton invisibleBttn = new JButton();
	invisibleBttn.setVisible(false);
	topLayout.setConstraints(invisibleBttn, new GridBagConstraints(2, 6, 1, 1, 0, 0, GridBagConstraints.CENTER, 
						    GridBagConstraints.CENTER, new Insets(indent,sp,0,0), 0,0));

	topPanel.add(dirLabel);
	topPanel.add(dirTF);
	topPanel.add(dirBttn);
	topPanel.add(matchingLabel);
	topPanel.add(matchingTF);
	topPanel.add(recursiveCheckBox);
	topPanel.add(sep);
	topPanel.add(backupLabel);
	topPanel.add(backupTF);
	topPanel.add(backupBttn);
	topPanel.add(templateLabel);
	topPanel.add(templateCh);
	topPanel.add(invisibleBttn);


	// 
	// Setup bottom panel
	//
	GridBagLayout buttomLayout = new GridBagLayout();
	JPanel buttomPanel = new JPanel();
	buttomPanel.setOpaque(false);
	buttomPanel.setLayout(buttomLayout);
	
	buttomLayout.setConstraints(runBttn, new GridBagConstraints(3, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, 
					GridBagConstraints.NONE, new Insets(sp,0,0,0), 0, 0));
	buttomPanel.add(runBttn);	



	//
	// Setup main panel
	//
	GridBagLayout mainLayout = new GridBagLayout();
	JPanel mainPanel = new JPanel() {
				public void paint(Graphics g)
				{
				    Dimension size = getSize();
				    g.drawImage(bgImage, 0, 0, size.width, size.height, 
							 0, 0, bgImage.getWidth(this), bgImage.getHeight(this), this);
				    super.paint(g);
				}
			    };
	mainPanel.setOpaque(false);
	mainPanel.setLayout(mainLayout);
	
	mainLayout.setConstraints(topPanel, new GridBagConstraints(0, 0, 1, 1, 1, 0, GridBagConstraints.CENTER, 
					GridBagConstraints.HORIZONTAL, new Insets(buf,buf,0,buf), 0, 0));
	mainLayout.setConstraints(buttomPanel, new GridBagConstraints(0, 1, 1, 1, 1, 1, GridBagConstraints.SOUTH, 
					GridBagConstraints.HORIZONTAL, new Insets(0,buf,buf,buf), 0, 0));
	mainPanel.add(topPanel);	
	mainPanel.add(buttomPanel);	


	Border border = BorderFactory.createEtchedBorder();
	mainPanel.setBorder(border);


	GridBagLayout layout = new GridBagLayout();
	getContentPane().setLayout(layout);
	    
	layout.setConstraints(mainPanel, 
			      new GridBagConstraints(0, 0, 1, 1, 
						     1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 
						     new Insets(0,0,0,0), 0, 0));

	getContentPane().add(mainPanel);

	setSize(500, 400);
    }


    /**
     * Used to initialize the componenets.  Also called after every event that
     * is required to update other fields.
     */
    public void setup()
    {
	if (firstTime)
	{
	    defaultDirBackupPath = convertSet.getBackupPath().getPath();
	    defaultOneFileBackupPath = System.getProperty("user.dir");
	    firstTime = false;
	}

	dirTF.setText(convertSet.getSourcePath().getPath());
	recursiveCheckBox.setSelected(converter.isRecurse());
	backupTF.setText(defaultDirBackupPath);
    }

    /**
     * Reset to defaults.
     */
    private void resetDefaults()
    {
	firstTime        = true;

	setup();
    }

    /**
     * Add listeners to components.
     */
    private void addListeners()
    {
	dirBttn.addActionListener(this);
	backupBttn.addActionListener(this);
	runBttn.addActionListener(this);

	recursiveCheckBox.addItemListener(this);
	templateCh.addItemListener(this);

	dirTF.addActionListener(this);
    
	addWindowListener(new WindowAdapter()
	{
	    public void windowClosing(WindowEvent e)
	    {
	        quit();
	    }
	});
    }

    /**
     * Handle ActionEvents.
     */
    public void actionPerformed(ActionEvent e)
    {
	Component target = (Component) e.getSource();

	if (target == dirBttn)
	{
	    JFileChooser chooser = new JFileChooser(); 
	    
	    //Gabe Boys: Changed form just dirs to files and dirs 
	    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); 

	    try
	    {
		chooser.setCurrentDirectory(new File(dirTF.getText()));
	    }
	    catch (Exception ex)
	    {
	    }

	    int returnVal = chooser.showOpenDialog(this); 
	    if(returnVal == JFileChooser.APPROVE_OPTION) 
	    { 
    		dirTF.setText(chooser.getSelectedFile().getAbsolutePath());
    		
		//Gabe Boys : If the source is a file set the backupPath to the directory instead of the file
		if(!chooser.getSelectedFile().isDirectory())
		{
		    backupTF.setText(chooser.getSelectedFile().getParentFile().getAbsolutePath() + "_BAK");
		}
		else 
		{
		    backupTF.setText(chooser.getSelectedFile().getAbsolutePath() + "_BAK");
		}
	    } 
	}
	else if (target == backupBttn)
	{
	    JFileChooser chooser = new JFileChooser(); 
	    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); 

	    try  {
		chooser.setCurrentDirectory(new File(backupTF.getText()));
	    } catch (Exception ex)  {
	    }

	    int returnVal = chooser.showOpenDialog(this); 
	    if(returnVal == JFileChooser.APPROVE_OPTION) 
	    { 
		backupTF.setText(chooser.getSelectedFile().getAbsolutePath());
	    } 
	}
	else if (target == optionMenuItem)
	{
	    showOptionDialog();
	}
	else if (target == helpMenuItem)
	{
	    showHelpDialog();
	}
	else if (target == runBttn)
	{
	    boolean noExeceptionThrown = true;
	    noExeceptionThrown = setDirectory();

	    if(noExeceptionThrown == true)
	    {
		if (!testSourceVsBackup())
		{
		    // Set backup 
		    convertSet.setBackupPath(new File(backupTF.getText()));

    		    noExeceptionThrown = (setTemplateFile((String)templateCh.getSelectedItem()) && noExeceptionThrown);

    		    if (noExeceptionThrown)
    			(new ProgressGUI(converter)).setVisible(true);
		}
	    }	
	}
	else 
	if (target == exitMenuItem)
	{
	    quit();
	}
	else 
	if (target == aboutMenuItem)
	{
	    showAboutDialog();
	}
    }

    /**
     * Handle ItemEvents.
     */
    public void itemStateChanged(ItemEvent e) {
	Component target = (Component) e.getSource();

	if (target == recursiveCheckBox) {
	    converter.setRecurse(recursiveCheckBox.isSelected());
	}
	else if (target == templateCh) {

	    // Get the current template selection
	    String choiceStr = (String)templateCh.getSelectedItem();


	    // If the user chooses 'other', display a file dialog to allow
	    // them to select a template file.
	    if (choiceStr.equals(TemplateFileChoice.OTHER_STR) && !changingItem) {
		String templatePath = null;
		changingItem = true;
		FileDialog fd = new FileDialog(this, "Select File", FileDialog.LOAD);
		fd.show();

		// Capture the path entered, if any.
		if (fd.getDirectory() != null && fd.getFile() != null) {
		    templatePath = fd.getDirectory() + fd.getFile();
		}

		// If the template file is valid add it and select it.
		if ( templatePath != null && setTemplateFile(templatePath) ) {
		    if (!templateCh.testIfInList(templatePath)) {
			templateCh.addItem(templatePath);
			changingItem = true;
		    }
		    templateCh.select(templatePath);
		}
		else {
		    templateCh.select(templateCh.getPreviousSelection());
		}    
		changingItem = false;
		fd.dispose();
	    }
	    else {
		templateCh.select(choiceStr);
	    }
	}
    }

    /**
     * Pop-up a Dialog if the source and backup fields are the same.
     * Returns true if the source and backup remain the same.
     */
    private boolean testSourceVsBackup()
    {
    if (dirTF.getText().equals(backupTF.getText()))
    {
	final String caption = ResourceHandler.getMessage("caption.warning");
	final String info = ResourceHandler.getMessage("warning_dialog.info") + backupTF.getText() + "_BAK";

	int n = JOptionPane.showConfirmDialog(this, info, caption, JOptionPane.WARNING_MESSAGE);

        if (n == JOptionPane.YES_OPTION)
        {
            backupTF.setText(backupTF.getText() + "_BAK");

            return false;
        }
        else if (n == JOptionPane.NO_OPTION)
            return true;
    }

    return false;
    }

    /**
     * Returns false if Exception is thrown.
     */
    private boolean setDirectory()
    {
	String pathStr = dirTF.getText().trim();
	if (pathStr.equals(""))
	    pathStr = System.getProperty("user.dir");
	try
	{
	    File dirPath = new File(pathStr);
	    if(!dirPath.isDirectory())
	    {
		if(!dirPath.exists())
		{
		    if(recursiveCheckBox.isSelected())
			throw new NotDirectoryException(dirPath.getAbsolutePath());
		    else
			throw new NotFileException(dirPath.getAbsolutePath());
		}
		else
		{
		    convertSet.setFile(dirPath);
		    convertSet.setDestinationPath(dirPath.getParentFile());
		}
	    }
	    else
	    {
		//Set the descriptors
		setMatchingFileNames();

		FlexFilter flexFilter = new FlexFilter();
		flexFilter.addDescriptors(descriptors);
		flexFilter.setFilesOnly(!recursiveCheckBox.isSelected());

		convertSet.setSourcePath(dirPath, flexFilter);
		convertSet.setDestinationPath(dirPath);
	    }
	}
	catch (NotDirectoryException e1)
	{
	    final String caption = ResourceHandler.getMessage("notdirectory_dialog.caption1");
	    final String info = ResourceHandler.getMessage("notdirectory_dialog.info0") 
				+ (pathStr.equals("") ? ResourceHandler.getMessage("notdirectory_dialog.info5") 
				: pathStr);

	    JOptionPane.showMessageDialog(this, info, caption, JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	catch (NotFileException e2)
	{
	    final String caption = ResourceHandler.getMessage("notdirectory_dialog.caption0");
	    final String info = ResourceHandler.getMessage("notdirectory_dialog.info1") + pathStr;

	    JOptionPane.showMessageDialog(this, info, caption, JOptionPane.ERROR_MESSAGE);
	    return false;
	}

	return true; // no exception thrown
    }

    /**
     * Set the template file path.
     */
    private boolean setTemplateFile(String pathStr)
    {
	File templateFile = new File(templateCh.getSelectedPath(pathStr));

	if (!templateFile.getName().toLowerCase().endsWith(".tpl"))
	{
	    final String templateCaption = ResourceHandler.getMessage("nottemplatefile_dialog.caption");
	    final String templateInfo0 = ResourceHandler.getMessage("nottemplatefile_dialog.info0");

	    String defaultTemplateName = PluginConverter.getDefaultTemplateFileName();
	    final String errorMessage = templateCaption + System.getProperty("line.separator") +
		templateFile.getName();
	    JOptionPane.showMessageDialog(this, errorMessage, null, JOptionPane.ERROR_MESSAGE);
	    templateCh.select(templateCh.getPreviousSelection());
	    return false;
	}

	try
	{
	    converter.setTemplateFilePath(templateCh.getSelectedPath(pathStr));
	}
	catch (FileNotFoundException e)
	{
		// TO-DO:  found it, but it's not a file.
		// TO-DO:  Throw up a Dialog -- "Not a valid file, resetting to default file"

	    pathStr = ((String)templateCh.getSelectedItem()).trim();
	    final String caption = ResourceHandler.getMessage("notdirectory_dialog.caption0");
	    final String info = ResourceHandler.getMessage("notdirectory_dialog.info1") 
				+ ResourceHandler.getMessage("notdirectory_dialog.info4") 
				+ (pathStr.equals("") ? ResourceHandler.getMessage("notdirectory_dialog.info5") 
				    : pathStr);

	    return false;
	}

	return true;
    }

    /**
     * Set the matching descriptors.
     */
    private void setMatchingFileNames()
    {
	StringTokenizer st = new StringTokenizer(matchingTF.getText(), ",");

	int numTokens = st.countTokens();

	if (numTokens == 0)
	{
	    descriptors = new String[1];
	    descriptors[0] = "*";
	    return;
	}

	descriptors = new String[numTokens];

	int i=0;
	while (st.hasMoreElements())
	{
	    descriptors[i++] = st.nextToken().trim();
	}
    }

    /**
     * Accessor to the converter.
     * @return PluginConverter
     */
    public static PluginConverter getPluginConverter()
    {
	return converter;
    }

    /**
     * Hide, reclaim resources, and exit.
     */
    protected void quit()
    {
	setVisible(false);
	dispose();
	System.exit(0);
    }

    /**
     * Displayed when the template file is not found in the classpath or
     * the working directory.
     */
    private void showNoTemplateDialog()
    {
	final String caption = ResourceHandler.getMessage("notemplate_dialog.caption");
	final String info = ResourceHandler.getMessage("notemplate_dialog.info0") 
			    + PluginConverter.getDefaultTemplateFileName()
			    + ResourceHandler.getMessage("notemplate_dialog.info1");

	JOptionPane.showMessageDialog(this, info, caption, JOptionPane.ERROR_MESSAGE);

	System.exit(0);
    }

    private void showAboutDialog()
    {
	final String aboutCaption = ResourceHandler.getMessage("about_dialog.caption");

	// Version string
	final String version = "1.3.1";

	final String aboutInfo = ResourceHandler.getMessage("about_dialog.info")
				 + version + ResourceHandler.getMessage("about_dialog.info2");


	JOptionPane.showMessageDialog(this, aboutInfo, aboutCaption, JOptionPane.INFORMATION_MESSAGE);
    }

    private void showHelpDialog()
    {
	HelpDialog dialog = new HelpDialog(this, false);
	dialog.setVisible(true);
    }

    private void showOptionDialog()
    {
	AdvancedDialog dialog = new AdvancedDialog(this, false, converter);
	dialog.setVisible(true);
    }   

    /**
     * TESTING...
     */
    public static void main(String[] argv)
    {
	(new ConverterGUI()).setVisible(true);
    }
}


// ---------------------------------------------------------------------------------------- //
class TemplateFileChoice extends JComboBox
{
    public static final int DEFAULT   = 0,
                            EXTEND    = 1,
                            IE        = 2,
                            NS        = 3,
                            SEPARATOR = 4,
                            OTHER     = 5;

    public static final String DEFAULT_STR   = ResourceHandler.getMessage("template.default"),
                               EXTEND_STR    = ResourceHandler.getMessage("template.extend"),
                               IE_STR        = ResourceHandler.getMessage("template.ieonly"),
                               NS_STR        = ResourceHandler.getMessage("template.nsonly"),
                               SEPARATOR_STR = "-----------------------------------------------------------------------",
                               OTHER_STR     = ResourceHandler.getMessage("template.other");

    public static final String DEFAULT_PATH = "templates/default.tpl",
                               EXTEND_PATH  = "templates/extend.tpl",
                               IE_PATH      = "templates/ieonly.tpl",
                               NS_PATH      = "templates/nsonly.tpl";

    private String previousSelection = DEFAULT_STR;

    private PluginConverter converter = ConverterGUI.getPluginConverter();

    public TemplateFileChoice() {
        super();

        // Define and set our JComboBox model
        String[] modelStrings = {
            DEFAULT_STR, EXTEND_STR, IE_STR, NS_STR, SEPARATOR_STR, OTHER_STR 
        };
        setModel(new DefaultComboBoxModel(modelStrings));
    }

    public void select(String item)
    {
        if (item.equals(SEPARATOR_STR))
            item = previousSelection;
        else
            previousSelection = item;

        setSelectedItem(item);
    }

    public String getPreviousSelection()
    {
        return previousSelection;
    }

    public String getSelectedPath(String item)
    {
        String path = "", fileSep = System.getProperty("file.separator");

/*
        try
        {
            path = converter.getDefaultTemplateFilePath().getParent() + fileSep;
        }
        catch (FileNotFoundException e)
        {
	    final String caption = ResourceHandler.getMessage("notemplate_dialog.caption");
	    final String info = ResourceHandler.getMessage("notemplate_dialog.info0") 
				+ PluginConverter.getDefaultTemplateFileName()
				+ ResourceHandler.getMessage("notemplate_dialog.info1");

	    JOptionPane.showMessageDialog(this, info, caption, JOptionPane.ERROR_MESSAGE);

            System.exit(0);
        }
*/
        if (item.equals(DEFAULT_STR))
            return path + DEFAULT_PATH;
        else if (item.equals(EXTEND_STR))
            return path + EXTEND_PATH;
        else if (item.equals(IE_STR))
            return path + IE_PATH;
    	else if (item.equals(NS_STR))
    	    return path + NS_PATH;
    	else
    	    return item;
    }

    public boolean testIfInList(String item)
    {
        for (int i=0; i < getItemCount(); i++)
        {
            if (getItemAt(i).equals(item))
                return true;
        }

        return false;
    }
}
