/*
 * @(#)HelpDialog.java	1.7 01/04/21
 *
 * Copyright (c) 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * @version 1.3
 *
 * @author Stanley Man-Kit Ho
 */

package sun.plugin.converter.util;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import sun.plugin.converter.ResourceHandler;
import sun.plugin.converter.util.BasicPrinter;
import javax.swing.*;


/**
 * Dialog to display the help.
 *
 */
public class HelpDialog extends JDialog implements ActionListener
{
    JTextArea textArea;
    JButton printButton, okayButton;

    /* HelpText Buffer */
    String buffer = "";

    private static final int WIDTH = 560, HEIGHT = 325;

    private static final String helpFileName = "readme.txt";

    public HelpDialog(Frame parent)
    {
	this(parent, ResourceHandler.getMessage("help_dialog.caption"), false);
    }

    public HelpDialog(Frame parent, boolean modal)
    {
	this(parent, ResourceHandler.getMessage("help_dialog.caption"), modal);
    }

    public HelpDialog(Frame parent, String title)
    {
	this(parent, title, false);
    }

    public HelpDialog(Frame parent, String title, boolean modal)
    {
        super(parent, title, modal);

        majorLayout();
    }


    /** 
     * <P> Layout all the elements of the help dialog.
     * </P>
     */
    public void majorLayout()
    {
        setSize(WIDTH, HEIGHT);
        setResizable(true);

	// Setup text area for help text
        textArea    = new JTextArea();
        textArea.setEditable(false);
        textArea.setBackground(Color.white);
	textArea.setText(getHelpText());

	// Setup scroll pane for text area
	JScrollPane pane = new JScrollPane();
	pane.getViewport().add(textArea);
	
	//Gabe Boys: Set the preffered size of the scroll pane
	//so that when the pack() happens the text are is not
	//to big
	Dimension screenSize = getToolkit().getScreenSize();
	pane.setPreferredSize(new Dimension((screenSize.width/2),(screenSize.height/2)));

	// Setup print button
        printButton = new JButton(ResourceHandler.getMessage("button.print"));
	printButton.addActionListener(this);

	// Setup okay button	
        okayButton    = new JButton(ResourceHandler.getMessage("button.okay"));
	okayButton.addActionListener(this);


	JPanel panel = new JPanel();
	panel.setLayout(new FlowLayout(FlowLayout.CENTER));
	panel.add(printButton);
	panel.add(okayButton);

	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(pane, BorderLayout.CENTER);
	getContentPane().add(panel, BorderLayout.SOUTH);
    }

    public String getHelpText()
    {
	// Read help text from file.
	try
	{
	    // Get help file directly from the converter JAR file.
	    InputStream in = getClass().getClassLoader().getResourceAsStream(helpFileName);
	    
	    byte[] text = new byte[2048];
	    StringBuffer sBuf = new StringBuffer();
	    int len = 0;

	    while ((len = in.read(text, 0, 2048)) != -1)
		sBuf.append(new String(text, 0, len));

    	    buffer = sBuf.toString();

    	    in.close();
	}
	catch (Exception e)  
	{
	    e.printStackTrace();
	}

	return buffer;
    }

    public void actionPerformed(ActionEvent e)
    {
        Component target = (Component) e.getSource();

        if (target == okayButton)
	{
	    setVisible(false);
	    dispose();
	}
        else if (target == printButton)
	{
	    BasicPrinter.printDocument(new Frame(), buffer, "Print Help Text");
	}
    }

    /**
     * Overrides java.awt.Container.setVisible() so that we are
     * sure that the Frame's size is set before displaying.
     */
    public void setVisible(boolean state)
    {
	if (state)
	{
            Dimension bounds = getToolkit().getScreenSize();
            setLocation((bounds.width - getSize().width)/2,
        	        (bounds.height - getSize().height)/2);
        }
	
	//Gabe Boys:  Added the pack so that localization
	// would appear correctly(i.e. Strings don't run off 
	//of the edge of the Dialog box anymore)
  	if( state == true ) {	
	   super.pack();
	}
	textArea.setCaretPosition(0);
        super.setVisible(state);
    }
}
