#
# @(#)compare.sh	1.3 99/02/07
#
# ksh script to compare two rt.jar files and check that they
# disassemble to the same instructions.
#
# Run it using
#           sh compare.sh [-v] <bootclasspath> <one.jar> <two.jar>
#
# The "-v" flag leads to a line of information on each file.
#
#
#							   KGH Jan 99
#
#######################################################################
#
# function to compare a given class
# The argument to this function is a class name.
function doclass {
    cname=$1
    if [ $verbose = 1 ]
    then
        print -n "Checking $1  "
    fi

    # Note that some of the jaws.jar classes cannot be disassembled because they
    # contain references to classes that aren't present in 1.2. 
    # In these cases an error message is generated.  We check that the error
    # messages match from before and after uncrunching.

    javap -bootclasspath $f1\;$bootclasspath -c -private -s -verbose $cname 2>errs.11 | sed -e 's/#[0-9]*//' -e 's/^No source//' -e '/^Compiled from/d' > $t1 2>errs.12
    javap -bootclasspath $f2\;$bootclasspath -c -private -s -verbose $cname 2>errs.21 | sed -e 's/#[0-9]*//' -e 's/^No source//' -e '/^Compiled from/d' > $t2 2>errs.22

    cat errs.11 errs.12 >> $t1
    cat errs.21 errs.22 >> $t2

    if cmp $t1 $t2
    then
        if [ $verbose = 1 ]
        then
            print OK
	fi
    else 
        if [ $verbose = 0 ]
        then
            print -n "$1  "
	fi
	print ERROR
	let error_count=error_count+1
	cp $t1 err1.tmp
        cp $t2 err2.tmp
    fi
}

# function to compare all the classes.
# Stdin for this function is a list of JAR entry names
function doclasses {
    while read ename; do
	cname=`echo $ename | sed s/.class//`
        doclass $cname
    done
}

verbose=0
if [ $1 = -v ]
then
    shift
    verbose=1
fi

bootclasspath=$1
f1=$2
f2=$3

cnames=cnames.tmp
t1=one.tmp
t2=two.tmp

jar tf $f1 2>errs | egrep .class > $cnames
jar tf $f2 2>errs | egrep .class > $t2

l1=`cat $cnames | wc -l`
l2=`cat $t2 | wc -l`

if [ $l1 != $l2 ]
then
    echo The two jars contain different numbers of entries:
    echo $f1 has $l1 classes
    echo $f2 has $l2 classes
    exit 2
fi

echo Comparing $l1 classes
error_count=0

# compare all the classes
doclasses < $cnames

if [ $error_count = 0 ]
then
    print All $l1 classes matched OK.
    exit 0
else
    print Some classes did not match!!
    print $error_count errors.
    exit 5
fi

