/*
 * @(#)TransientNameService.java	1.32 00/02/02
 *
 * Copyright 1993-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.CosNaming;

// Get CORBA type
import org.omg.CORBA.INITIALIZE;
import org.omg.CORBA.ORB;
import org.omg.CORBA.CompletionStatus;

// Get org.omg.CosNaming types
import org.omg.CosNaming.NamingContext;

// Import transient naming context
import com.sun.corba.se.internal.CosNaming.TransientNamingContext;

/**
 * Class TransientNameService implements a transient name service
 * using TransientNamingContexts and TransientBindingIterators, which
 * implement the org.omg.CosNaming::NamingContext and org.omg.CosNaming::BindingIterator
 * interfaces specfied by the OMG Common Object Services Specification.
 * <p>
 * The TransientNameService creates the initial NamingContext object.
 * @see NamingContextImpl
 * @see BindingIteratorImpl
 * @see TransientNamingContext
 * @see TransientBindingIterator
 */
public class TransientNameService
{
    /**
     * Constructs a new TransientNameService, and creates an initial
     * NamingContext, whose object
     * reference can be obtained by the initialNamingContext method.
     * @param orb The ORB object
     * @exception org.omg.CORBA.INITIALIZE Thrown if
     * the TransientNameService cannot initialize.
   */
    public TransientNameService(org.omg.CORBA.ORB orb)
	throws org.omg.CORBA.INITIALIZE
    {
	// Create impldefs for naming contexts and binding iterators
	try {
	    // Create an initial context
	    TransientNamingContext initialContext = new TransientNamingContext(orb,null);
	    // Get its object ref
	    theInitialNamingContext = (org.omg.CosNaming.NamingContext)initialContext;
	    // Patch in the object ref into the servant
	    initialContext.localRoot = theInitialNamingContext;
	} catch (org.omg.CORBA.SystemException e) {
	    NamingUtils.printException(e);
	    throw new org.omg.CORBA.INITIALIZE(MinorCodes.TRANS_NS_CANNOT_CREATE_INITIAL_NC_SYS,
					       CompletionStatus.COMPLETED_NO);
	} catch (Exception e) {
	    NamingUtils.printException(e);
	    throw new org.omg.CORBA.INITIALIZE(MinorCodes.TRANS_NS_CANNOT_CREATE_INITIAL_NC,
					       CompletionStatus.COMPLETED_NO);
	}
    }

    /**
   * Return the initial NamingContext.
   * @return the object reference for the initial NamingContext.
   */
    public org.omg.CosNaming.NamingContext initialNamingContext()
    {
	return theInitialNamingContext;
    }

    // The initial naming context for this name service
    private org.omg.CosNaming.NamingContext theInitialNamingContext;
}
