/*
 * @(#)AnyImpl.java	1.39 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.corba;

import org.omg.CORBA.*;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.Streamable;

import com.sun.corba.se.internal.core.*;
import com.sun.corba.se.internal.iiop.CDRInputStream;
import com.sun.corba.se.internal.iiop.CDROutputStream;
import com.sun.corba.se.internal.orbutil.RepositoryIdWrapper;

import java.io.Serializable;
import org.omg.CORBA.portable.ObjectImpl;

//import test.Debug;


final class AnyInputStream extends CDRInputStream {
    
    AnyInputStream(CDRInputStream theStream ) {
	super( theStream );
    }
}

final class AnyOutputStream extends CDROutputStream {

    public AnyOutputStream(org.omg.CORBA.ORB orb) {
        super(orb, false);
    }

    public AnyOutputStream(org.omg.CORBA.ORB orb, int size) {
        super(orb, false, size);
    }

    public InputStream create_input_stream() {
        return new AnyInputStream((CDRInputStream)super.create_input_stream());
    }
}


// subclasses must provide a matching helper class
public class AnyImpl extends Any
{

    //
    // Always valid.
    //
    private TypeCodeImpl typeCode;
    protected org.omg.CORBA.ORB orb;

    //
    // Validity depends upon typecode. The 'value' and 'object' instance
    // members are used to hold immutable types as defined by the
    // isStreamed[] table below. Otherwise, 'stream' is non-null and
    // holds the value in CDR marshaled format. As an optimization, the
    // stream type is an Any extension of CDR stream that is used to
    // detect an optimization in read_value().
    //

    private CDRInputStream stream;
    private long value;
    private java.lang.Object object;

    private static final int DEFAULT_BUFFER_SIZE = 32;

    /*
     * This boolean array tells us if a given typecode must be
     * streamed. Objects that are immutable don't have to be streamed.
     */

    static boolean isStreamed[] = {
	false,  // null
	false,  // void
	false,  // short
	false,  // long
	false,  // ushort
	false,  // ulong
	false,  // float
	false,  // double
	false,  // boolean
	false,  // char
	false,  // octet
	false,   // any
	false,  // TypeCode
	true,   // Principal
	false,  // objref
	true,   // struct
	true,   // union
	true,   // enum
    	false,  // string
	true,   // sequence
	true,   // array
	true,   // alias
	true,   // except
	false,  // longlong
	false,  // ulonglong
	false,  // longdouble
	false,  // wchar
	false,  // wstring
	false,  // fixed
	false,   // value
	false    // value_box (used to be true)
    };


    ///////////////////////////////////////////////////////////////////////////
    // Constructors

    /**
     * A constructor that sets the Any to contain a null. It also marks
     * the value as being invalid so that extractions throw an exception
     * until an insertion has been performed. 
     */

    //private Debug debug = new Debug ("AnyImpl ");

    public AnyImpl(org.omg.CORBA.ORB orb)
    {
	this.orb = orb;

	typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_null);
	stream = null;
	object = null;
	value = 0;
    }


    //
    // Create a new AnyImpl which is a copy of obj.
    //
    AnyImpl(org.omg.CORBA.ORB orb, Any obj) {
	this(orb);

	if ((obj instanceof AnyImpl)) {
	    AnyImpl objImpl = (AnyImpl)obj;
	    typeCode = objImpl.typeCode;
	    value = objImpl.value;
	    object = objImpl.object;

	    if (objImpl.stream != null)
		stream = objImpl.stream.dup();

	} else {
	    read_value(obj.create_input_stream(), obj.type());
	}
    }


    ///////////////////////////////////////////////////////////////////////////
    // basic accessors

    /**
     * returns the type of the element contained in the Any.
     *
     * @result		the TypeCode for the element in the Any
     */
   
    public TypeCode type() 
    {
	//debug.log ("type");
	return typeCode;
    }


/**
 * sets the type of the element to be contained in the Any.
 *
 * @param tc		the TypeCode for the element in the Any
 */
   
public void type(TypeCode tc)
{
    //debug.log ("type2");
    // set the typecode
    typeCode = TypeCodeImpl.convertToNative(orb, tc);

    stream = null;
    value = 0;
    object = null;
}


/**
 * checks for equality between Anys.
 *
 * @param otherAny	the Any to be compared with.
 * @result		true if the Anys are equal, false otherwise.
 */
   
public boolean equal(Any otherAny)
{
    //debug.log ("equal");
    // first check for typecode equality.
    // note that this will take aliases into account
    if (!typeCode.equal(otherAny.type()))
	return false;

    switch (typeCode.kind().value()) {
	// handle primitive types
    case TCKind._tk_null:
    case TCKind._tk_void:
	return true;
    case TCKind._tk_short:
	return (extract_short() == otherAny.extract_short());
    case TCKind._tk_long:
	return (extract_long() == otherAny.extract_long());
    case TCKind._tk_ushort:
	return (extract_ushort() == otherAny.extract_ushort());
    case TCKind._tk_ulong:
	return (extract_ulong() == otherAny.extract_ulong());
    case TCKind._tk_float:
	return (extract_float() == otherAny.extract_float());
    case TCKind._tk_double:
	return (extract_double() == otherAny.extract_double());
    case TCKind._tk_boolean:
	return (extract_boolean() == otherAny.extract_boolean());
    case TCKind._tk_char:
	return (extract_char() == otherAny.extract_char());
    case TCKind._tk_octet:
	return (extract_octet() == otherAny.extract_octet());
    case TCKind._tk_any:
	return extract_any().equal(otherAny.extract_any());
    case TCKind._tk_TypeCode:
	return extract_TypeCode().equal(otherAny.extract_TypeCode());
    case TCKind._tk_string:
	return extract_string().equals(otherAny.extract_string());
    case TCKind._tk_longlong:
	return (extract_longlong() == otherAny.extract_longlong());
    case TCKind._tk_ulonglong:
	return (extract_ulonglong() == otherAny.extract_ulonglong());
    case TCKind._tk_enum:
	return (create_input_stream().read_long() == 
		otherAny.create_input_stream().read_long());
     
	// _REVISIT_ not yet implemented for extended types

    case TCKind._tk_longdouble:
    case TCKind._tk_wchar:
    case TCKind._tk_wstring:
	throw new org.omg.CORBA.NO_IMPLEMENT();

	// _REVISIT_ not yet implemented for complex types

    case TCKind._tk_objref:
    case TCKind._tk_Principal:
    case TCKind._tk_struct:
    case TCKind._tk_union:
    case TCKind._tk_sequence:
    case TCKind._tk_array:
    case TCKind._tk_alias:
    case TCKind._tk_except:
	throw new org.omg.CORBA.NO_IMPLEMENT();
      
	// _REVISIT_ not yet implemented in spec. nor here
    case TCKind._tk_value:
    case TCKind._tk_value_box:
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }
    return false;
}

    ///////////////////////////////////////////////////////////////////////////
// accessors for marshaling/unmarshaling

/**
 * returns an output stream that an Any value can be marshaled into. 
 *
 * @result		the OutputStream to marshal value of Any into
 */
   
public OutputStream create_output_stream()
{
    //debug.log ("create_output_stream");
    return new AnyOutputStream(orb, DEFAULT_BUFFER_SIZE);
}

/**
 * returns an input stream that an Any value can be marshaled out of.
 *
 * @result		the InputStream to marshal value of Any out of.
 */

//
// We create a new InputStream so that multiple threads can call here
// and read the streams in parallel without thread safety problems.
//
public InputStream create_input_stream()
{
    //debug.log ("create_input_stream");
    if (stream != null) {
	return stream.dup();
    } else {
	org.omg.CORBA_2_3.portable.OutputStream os =
	    (org.omg.CORBA_2_3.portable.OutputStream)orb.create_output_stream();
	TCUtility.marshalIn(os, typeCode.kind().value(), value, object);

	return os.create_input_stream();
    }
}

///////////////////////////////////////////////////////////////////////////
// marshaling/unmarshaling routines

//
// If the InputStream is a CDRInputStream then we can copy the bytes
// since it is in our format and does not have alignment issues.
//
public void read_value(InputStream in, TypeCode tc)
{ 
    //debug.log ("read_value");
    //
    // Assume that someone isn't going to think they can keep reading
    // from this stream after calling us. That would be likely for
    // an IIOPInputStream but if it is an AnyInputStream then they
    // presumably obtained it via our create_output_stream() so they could
    // write the contents of an IDL data type to it and then call
    // create_input_stream() for us to read it. This is how Helper classes
    // typically implement the insert() method.
    // We should probably document this behavior in the 1.1 revision
    // task force.
    //

    typeCode = TypeCodeImpl.convertToNative(orb, tc);
    int type = typeCode.kind().value();

    if (AnyImpl.isStreamed[type]) {
	if ( in instanceof AnyInputStream ) {
	    // could only have been created here
	    stream = (CDRInputStream)in;
	} else {
	    org.omg.CORBA_2_3.portable.OutputStream out =
		(org.omg.CORBA_2_3.portable.OutputStream)orb.create_output_stream();
	    typeCode.copy((org.omg.CORBA_2_3.portable.InputStream)in, out);
	    stream = (CDRInputStream)out.create_input_stream();
	}
    } else {
	java.lang.Object[] objholder = new java.lang.Object[1];
	objholder[0] = object;
	long[] longholder = new long[1];
	TCUtility.unmarshalIn(in, type, longholder, objholder);
	value = longholder[0];
	object = objholder[0];
	stream = null;
    }
}


//
// We could optimize this by noticing whether the target stream
// has ever had anything marshaled on it that required an
// alignment of greater than 4 (was write_double() ever called on it).
// If not, then we can just do a byte array copy without having to
// drive the remarshaling through typecode interpretation.
//
public void write_value(OutputStream out)
{
    //debug.log ("write_value");
    if (stream != null) {
	typeCode.copy(stream.dup(),(org.omg.CORBA_2_3.portable.OutputStream)out);
    } else {
	TCUtility.marshalIn(out, typeCode.kind().value(), value, object);
    }
}

///////////////////////////////////////////////////////////////////////////
// insertion of streamables 
//
// We need to more clearly define the semantics of streamables
// passed in here. The current implementation is safe. A faster
// implementation would merely save the streamable reference and
// defer using it until needed. Doing this would eliminate the
// extra objects generated below in order to capture the state of the
// streamable immediately. However, we need to document this behavior
// since it has an impact on users of the API. This should be handled
// as part of the OMG Java IDL mapping 1.1 revision task force.
//

/**
 * takes a streamable and inserts it into the any
 *
 * @param s		the streamable to insert
 */
public void insert_Streamable(Streamable s)
{
    //debug.log ("insert_Streamable");
    typeCode = TypeCodeImpl.convertToNative(orb, s._type());
    object = s;

    /*
      AnyOutputStream os = new AnyOutputStream(orb);
      s._write(os);
      read_value(os.create_input_stream(), s._type());
    */
}
  
///////////////////////////////////////////////////////////////////////////
// insertion/extraction/replacement for all basic types

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_short(short s)
{
    //debug.log ("insert_short");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_short);
    value = s;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public short extract_short()
{
    //debug.log ("extract_short");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_short)))
	throw new BAD_OPERATION();
    return (short)value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_long(int l)
{
    //debug.log ("insert_long");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_long);
    value = l;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public int extract_long()
{
    //debug.log ("extract_long");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_long)))
	throw new BAD_OPERATION();
    return (int)value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_ushort(short s)
{
    //debug.log ("insert_ushort");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_ushort);
    value = s;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public short extract_ushort()
{
    //debug.log ("extract_ushort");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_ushort)))
	throw new BAD_OPERATION();
    return (short)value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_ulong(int l)
{
    //debug.log ("insert_ulong");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_ulong);
    value = l;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public int extract_ulong()
{
    //debug.log ("extract_ulong");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_ulong)))
	throw new BAD_OPERATION();
    return (int)value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_float(float f)
{
    //debug.log ("insert_float");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_float);
    value = Float.floatToIntBits(f);
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public float extract_float()
{
    //debug.log ("extract_float");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_float)))
	throw new BAD_OPERATION();
    return Float.intBitsToFloat((int)value);
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_double(double d)
{
    //debug.log ("insert_double");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_double);
    value = Double.doubleToLongBits(d);
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public double extract_double()
{
    //debug.log ("extract_double");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_double)))
	throw new BAD_OPERATION();
    return Double.longBitsToDouble(value);
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_longlong(long l)
{
    //debug.log ("insert_longlong");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_longlong);
    value = l;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public long extract_longlong()
{
    //debug.log ("extract_longlong");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_longlong)))
	throw new BAD_OPERATION();
    return value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_ulonglong(long l)
{
    //debug.log ("insert_ulonglong");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_ulonglong);
    value = l;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public long extract_ulonglong()
{
    //debug.log ("extract_ulonglong");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_ulonglong)))
	throw new BAD_OPERATION();
    return value;
}


/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_boolean(boolean b)
{
    //debug.log ("insert_boolean");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_boolean);
    value = (b)? 1:0;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public boolean extract_boolean()
{
    //debug.log ("extract_boolean");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_boolean)))
	throw new BAD_OPERATION();
    return (value == 0)? false: true;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_char(char c)
{
    //debug.log ("insert_char");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_char);
    value = c;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public char extract_char()
{
    //debug.log ("extract_char");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_char)))
	throw new BAD_OPERATION();
    return (char)value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_wchar(char c)
{
    //debug.log ("insert_wchar");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_wchar);
    value = c;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public char extract_wchar()
{
    //debug.log ("extract_wchar");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_wchar)))
	throw new BAD_OPERATION();
    return (char)value;
}


/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_octet(byte b)
{
    //debug.log ("insert_octet");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_octet);
    value = b;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public byte extract_octet()
{
    //debug.log ("extract_octet");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_octet)))
	throw new BAD_OPERATION();
    return (byte)value;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_string(String s)
{
    //debug.log ("insert_string");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_string);
    object = s;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public String extract_string()
{
    //debug.log ("extract_string");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_string)))
	throw new BAD_OPERATION();
    return (String)object;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_wstring(String s)
{
    //debug.log ("insert_wstring");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_wstring);
    object = s;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public String extract_wstring()
{
    //debug.log ("extract_wstring");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_wstring)))
	throw new BAD_OPERATION();
    return (String)object;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_any(Any a)
{
    //debug.log ("insert_any");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_any);
    object = a;
    stream = null;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public Any extract_any()
{
    //debug.log ("extract_any");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_any)))
	throw new BAD_OPERATION();
    return (Any)object;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_Object(org.omg.CORBA.Object o)
{
    //debug.log ("insert_Object");
    if ( o == null ) {
	typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_objref);
    } else {
	ObjectImpl objImpl = (ObjectImpl)o;
	String[] ids = objImpl._ids();
	typeCode = new TypeCodeImpl(orb, TCKind._tk_objref, ids[0], "");
    }
    
    object = o;
}

/**
 * A variant of the insertion operation that takes a typecode
 * argument as well.
 */

public void insert_Object(org.omg.CORBA.Object o, TypeCode tc)
{
    //debug.log ("insert_Object2");
    try {
	if ( tc.id().equals("IDL:omg.org/CORBA/Object:1.0") || o._is_a(tc.id()) )
	    {
		typeCode = TypeCodeImpl.convertToNative(orb, tc);
		object = o;
	    }
	else {
	    throw new BAD_OPERATION();
	}
    } catch ( Exception ex ) {
	throw new BAD_OPERATION();
    }
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public org.omg.CORBA.Object extract_Object()
{
    //debug.log ("extract_Object");
    // Check if the object contained here is of the type in typeCode
    org.omg.CORBA.Object obj = null;
    try {
	obj = (org.omg.CORBA.Object) object;
	if ( typeCode.id().equals("IDL:omg.org/CORBA/Object:1.0") 
	     || obj._is_a(typeCode.id()) ) {
	    return obj;
	}
	else {
	    throw new BAD_OPERATION();
	}
    } catch ( Exception ex ) {
	throw new BAD_OPERATION();
    }
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public void insert_TypeCode(TypeCode tc)
{
    //debug.log ("insert_TypeCode");
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_TypeCode);
    object = tc;
}

/**
 * See the description of the <a href="#anyOps">general Any operations.</a>
 */

public TypeCode extract_TypeCode()
{
    //debug.log ("extract_TypeCode");
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_TypeCode)))
	throw new BAD_OPERATION();
    return (TypeCode)object;
}

/**
 * @deprecated
 */

public void insert_Principal(Principal p)
{
    typeCode = TypeCodeImpl.get_primitive_tc(TCKind.tk_Principal);
    object = p;
}

/**
 * @deprecated
 */

public Principal extract_Principal()
{
    if (!typeCode.equal(TypeCodeImpl.get_primitive_tc(TCKind.tk_Principal)))
	throw new BAD_OPERATION();
    return (Principal)object;
}

public Serializable extract_Value() throws org.omg.CORBA.BAD_OPERATION
{
    //debug.log ("extract_Value");

    int kind = typeCode.kind().value();
    if ( kind != TCKind._tk_value && kind != TCKind._tk_value_box )
	throw new BAD_OPERATION();

    return (Serializable)object;
}

public void insert_Value(Serializable v)
{
    //debug.log ("insert_Value");
    object = v;

    TypeCode tc;

    if ( v == null ) {
	tc = orb.get_primitive_tc (TCKind.tk_value);
    } else {
	tc = createTypeCodeForClass (v.getClass());
    }

    typeCode = TypeCodeImpl.convertToNative(orb, tc);
}
  
public void insert_Value(Serializable v, org.omg.CORBA.TypeCode t)
    throws org.omg.CORBA.MARSHAL
{
    //debug.log ("insert_Value2");
    object = v;
    typeCode = TypeCodeImpl.convertToNative(orb, t);
}

  
private TypeCode createTypeCodeForClass (java.lang.Class c)
{
    //debug.log ("createTypeCodeForClass");
    if ( c.isArray() ) {
	// Arrays - may recurse for multi-dimensional arrays
	Class componentClass = c.getComponentType();
	TypeCode embeddedType;
	if ( componentClass.isPrimitive() ){
	    embeddedType = getPrimitiveTypeCodeForClass (componentClass);
	} else {
	    embeddedType = createTypeCodeForClass (componentClass);
	}
	TypeCode t = orb.create_sequence_tc (0, embeddedType);
	String id = RepositoryIdWrapper.createForJavaType(orb, c);
	return orb.create_value_box_tc (id, "Sequence", t);
    } else if ( c == java.lang.String.class ) {
	// Strings
	TypeCode t = orb.create_string_tc (0);
	String id = RepositoryIdWrapper.createForJavaType(orb, c);
	return orb.create_value_box_tc (id, "StringValue", t);
    }

    // Anything else
    return orb.get_primitive_tc (TCKind.tk_value);
}

private TypeCode getPrimitiveTypeCodeForClass (Class c)
{
    //debug.log ("getPrimitiveTypeCodeForClass");

    if (c == Integer.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_long);
    } else if (c == Byte.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_octet);
    } else if (c == Long.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_longlong);
    } else if (c == Float.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_float);
    } else if (c == Double.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_double);
    } else if (c == Short.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_short);
    } else if (c == Character.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_char);
    } else if (c == Boolean.TYPE) {
	return orb.get_primitive_tc (TCKind.tk_boolean);
    } else {
	// _REVISIT_ Not sure if this is right.
	return orb.get_primitive_tc (TCKind.tk_any);
    }
}

}

