/*
 * @(#)ClientDelegate.java	1.32 01/02/09
 *
 * Copyright 1993-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.sun.corba.se.internal.corba;

import org.omg.CORBA.ORB;
import org.omg.CORBA.portable.ObjectImpl;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.Delegate;
import org.omg.CORBA.portable.ServantObject;
import org.omg.CORBA.portable.ApplicationException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.WrongTransaction;
import org.omg.CORBA.Request;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NVList;
import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.DATA_CONVERSION;

import javax.rmi.CORBA.Util;
import javax.rmi.CORBA.Tie;

import java.rmi.RemoteException;

import com.sun.org.omg.SendingContext.CodeBase;

import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.Profile;
import com.sun.corba.se.internal.core.ClientGIOP;
import com.sun.corba.se.internal.core.ClientSubcontract;
import com.sun.corba.se.internal.core.ClientRequest;
import com.sun.corba.se.internal.core.ClientResponse;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.MarshalOutputStream;
import com.sun.corba.se.internal.core.MarshalInputStream;
import com.sun.corba.se.internal.core.CodeSetComponentInfo;
import com.sun.corba.se.internal.core.UEInfoServiceContext;
import com.sun.corba.se.internal.core.CodeSetServiceContext;
import com.sun.corba.se.internal.core.SendingContextServiceContext;
import com.sun.corba.se.internal.core.ORBVersionServiceContext;
import com.sun.corba.se.internal.core.NoSuchServiceContext;
import com.sun.corba.se.internal.core.DuplicateServiceContext;
import com.sun.corba.se.internal.core.GIOPVersion;


import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.util.JDKBridge;
import com.sun.corba.se.internal.core.ORBVersion;
import com.sun.corba.se.internal.core.ORBVersionFactory;
import com.sun.corba.se.internal.orbutil.ORBConstants;

import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.portable.UnknownException;
import com.sun.corba.se.internal.iiop.CDRInputStream;
import com.sun.corba.se.internal.iiop.IIOPConnection;
import com.sun.corba.se.internal.iiop.Connection;
import com.sun.corba.se.internal.iiop.ClientRequestImpl;
import com.sun.corba.se.internal.iiop.LocalClientRequestImpl;

class Debug{
	
    public Debug(String s){}
    public void log(String s){}
}


/**
 * ClientDelegate is the RMI client-side subcontract or representation 
 * It implements RMI delegate as well as our internal ClientSubcontract
 * interface.
 */
public class ClientDelegate
    extends org.omg.CORBA_2_3.portable.Delegate
    implements ClientSubcontract
{
    protected com.sun.corba.se.internal.corba.ORB orb;
    protected IOR ior;
    protected IOR locatedIOR;
    protected int scid; // my subcontract id
    protected ServantObject servant;

    public static Debug debug = new Debug("ClientDelegate:");

    public ClientDelegate() {
	super();
	servant = null;
    }

    public ClientDelegate(com.sun.corba.se.internal.corba.ORB orb, IOR ior, int id) {
        this(orb,ior,id,null);
    }
                            
    public ClientDelegate(com.sun.corba.se.internal.corba.ORB orb, IOR ior, int id,
			  java.lang.Object servant) {
	this();
	setOrb(orb);
	this.ior = ior;
	this.locatedIOR = ior;
	this.scid = id;
	if (servant != null && servant instanceof Tie) {
	    this.servant = new ServantObject();
	    this.servant.servant = ((Tie)servant).getTarget();
	} else {
	    this.servant = null;
	}
    }

    public void unexport() {
	ior = null;
	servant = null;
    }

    public IOR marshal() {
	return ior;
    }

    public IOR getIOR() {
	return ior;
    }

    public void unmarshal(IOR ior) {
	this.ior = ior;
	this.locatedIOR = ior;
    }

    public void setId(int id) {
	this.scid = id;
    }

    public void setOrb(com.sun.corba.se.internal.core.ORB orb) {
	this.orb = (com.sun.corba.se.internal.corba.ORB) orb;
    }

    // Invoked by DII layer (from RequestImpl.doInvocation()).
    public ClientResponse invoke(ClientRequest request) {

	ClientResponse resp = request.invoke();

	if (request.isOneWay())
	    return null;	

        // consume service contexts that were returned to the client
        Connection c = request.getConnection();
        if (resp != null)
            resp.setConnection(c);

        consumeServiceContexts(resp, (IIOPConnection)c);

	if (resp.isLocationForward()) {
	    IOR[] iorHolder = new IOR[1];
	    iorHolder[0] = resp.getForwardedIOR();
	    resp = request.reInvoke(iorHolder);
	    locatedIOR = iorHolder[0];
	} else if ( resp.isSystemException() ) {
	    SystemException ex = resp.getSystemException();

	    if ( ex instanceof org.omg.CORBA.COMM_FAILURE 
	         && ex.completed == CompletionStatus.COMPLETED_NO ) {

		// invoke again using the original IOR.
		if ( locatedIOR != ior ) {             
		    IOR[] iorHolder = new IOR[1];
		    iorHolder[0] = ior;
		    resp = request.reInvoke(iorHolder);
		    locatedIOR = iorHolder[0];
		}
	    }
	}

	return resp;
    }

    public InputStream invoke(org.omg.CORBA.Object self, OutputStream out)
	throws ApplicationException, org.omg.CORBA.portable.RemarshalException {
	
	ClientRequest request = (ClientRequest) out;
	ClientResponse response = this.invoke(request);

        if (request.isOneWay()) //oneway calls
            return null;

	if (response.isSystemException()) {
	    SystemException se = response.getSystemException();
	    ServiceContexts contexts =  response.getServiceContexts();
	    if (contexts != null) {
		try {
		    UEInfoServiceContext usc = 
			(UEInfoServiceContext)(contexts.get( 
							    UEInfoServiceContext.SERVICE_CONTEXT_ID )) ;
		    Throwable unknown = usc.getUE() ;
		    throw new UnknownException(unknown);
		} catch (NoSuchServiceContext exc) {
		    // NO-OP: handled by system exception below
		}
	    }

	    throw se;
	} else if (response.isUserException()) {
	    throw new ApplicationException(response.peekUserExceptionId(),
					   (org.omg.CORBA.portable.InputStream)response);
	} else if (response.isLocationForward()) {
	    throw new org.omg.CORBA.portable.RemarshalException();
	} else /* normal response */ {
	    return (InputStream) response;
	}
    }

    public ServiceContexts getServiceContexts( Connection c, 
					       int requestId, String opName, boolean isOneWay ) 
    {
	ServiceContexts contexts = new ServiceContexts( orb ) ;
			
	CodeSetServiceContext cssc = new CodeSetServiceContext(
							       CodeSetComponentInfo.myCodeSet ) ;

	try {
	    contexts.put( cssc ) ;
	} catch (DuplicateServiceContext dsc) {
	}

        // ORBVersion servicecontext needs to be sent

        ORBVersionServiceContext ovsc = new ORBVersionServiceContext(ORBVersionFactory.getORBVersion());

        try {
            contexts.put( ovsc ) ;
        } catch (DuplicateServiceContext dsc) {
        }

	// NOTE : We only want to send the runtime context the first time
	if ((c != null) && !c.isPostInitialContexts()) {
	    c.setPostInitialContexts();
	    SendingContextServiceContext scsc = 
		new SendingContextServiceContext( orb.getServantIOR() ) ; //d11638
	    try {
		contexts.put( scsc ) ;
	    } catch (DuplicateServiceContext dsc) {
	    }
	}

	return contexts ;
    }

    public void consumeServiceContexts(ClientResponse response,
                                       IIOPConnection connection) {
	ServiceContexts ctxts = response.getServiceContexts();
	ServiceContext sc ;

        if (ctxts == null) {
            // no service context available, return gracefully.
            return;
        }

	try {
	    sc = ctxts.get( CodeSetServiceContext.SERVICE_CONTEXT_ID ) ;
	
	    CodeSetServiceContext cssc = (CodeSetServiceContext)sc ;
	    CodeSetComponentInfo.CodeSetContext csctx =
		cssc.getCodeSetContext() ;
	    
	    if (csctx.validateWith( CodeSetComponentInfo.myCodeSet) == false)
		throw new DATA_CONVERSION( MinorCodes.CODESET_INCOMPATIBLE,
					   CompletionStatus.COMPLETED_NO);
	} catch (NoSuchServiceContext exc) {
	    // ignore: this type not present
	}

	try {
	    sc = ctxts.get( SendingContextServiceContext.SERVICE_CONTEXT_ID ) ;

	    SendingContextServiceContext scsc = 
		(SendingContextServiceContext)sc ;
	    IOR ior = scsc.getIOR() ;

	    try {

                if (connection != null)
                    connection.setCodeBaseRef(ior);

	    } catch (ThreadDeath td) {
		throw td ;
	    } catch (Throwable t) {
		throw new DATA_CONVERSION( MinorCodes.BAD_STRINGIFIED_IOR,
					   CompletionStatus.COMPLETED_NO);
	    }
	} catch (NoSuchServiceContext exc) {
	    // ignore: this type not present
	}

        // see if the version subcontract is present, if yes, then set
        // the ORBversion
	try {
	    sc = ctxts.get( ORBVersionServiceContext.SERVICE_CONTEXT_ID ) ;

	    ORBVersionServiceContext ovsc =
	       (ORBVersionServiceContext) sc;

	    ORBVersion version = ovsc.getVersion();
	    orb.setORBVersion( version ) ;
	} catch (NoSuchServiceContext exc) {
	    // ignore: this type not present
	}
    }

    protected ClientRequest createRequest( IOR ior, String opName, 
					   boolean isOneWay, int requestId )
    {
	Profile iop = ior.getProfile();
	byte[]  key = iop.getObjectKey();

	Connection conn = null ;
	ServiceContexts svc = null ;

	if (ior.isLocal()) {
	    svc = getServiceContexts( conn, requestId, opName, isOneWay ) ;

	    return new LocalClientRequestImpl( (com.sun.corba.se.internal.iiop.ORB)orb, key, 
					       opName, isOneWay, svc, requestId ) ;
	} else {
            String host = iop.getHost();
            int    port = iop.getPort();
	    //conn = orb.getClientGIOP().getConnection(host, port);
            conn = orb.getClientGIOP().getConnection(ior.getEndpoint());

	    svc = getServiceContexts( conn, requestId, opName, isOneWay ) ; 
	    
	    return new ClientRequestImpl( key, opName, isOneWay, svc, 
					  requestId, conn );
	}
    }

    // Invoked by DII layer (from RequestImpl.doInvocation()).
    // before marshaling arguments.
    public ClientRequest createRequest(String opName, boolean isOneWay) {
	// Create request object which is also the outputstream
	ClientRequest request = null;
	ClientGIOP giop = orb.getClientGIOP();
	int id = giop.allocateRequestId() ;
	    
	try {
	    request = createRequest(locatedIOR, opName, isOneWay, id );
	} catch (org.omg.CORBA.COMM_FAILURE ex) {
	    // The COMM_FAILURE may have happened because the server died.
	    // Try creating the request with the original IOR.
	    request = createRequest(ior, opName, isOneWay, id );
	}

	return request;
    }

	// Invoked by DII layer (from RequestImpl.doInvocation())
    // after unmarshaling reply and exceptions.
    public void releaseReply(ClientResponse resp, String method,
			     Exception exception)
	throws WrongTransaction, SystemException
    {
    }

    public void releaseReply(org.omg.CORBA.Object self,
                             InputStream input) {
        // NOOP: We believe in GC ;-)
    }

    public boolean is_equivalent(org.omg.CORBA.Object obj, 
				 org.omg.CORBA.Object ref)
    {
	if ( ref == null )
	    return false;
	ObjectImpl oi = (ObjectImpl)ref;

	ClientSubcontract del = (ClientSubcontract)oi._get_delegate();
	if (del == this)
	    return true;
	if (!(del instanceof ClientDelegate))
	    return false;
	return ior.isEquivalent(del.marshal());
    }

    public int hash(org.omg.CORBA.Object obj, int maximum) 
    {
	int h = this.hashCode();
	if ( h > maximum )
	    return 0;
	return h;
    }

    public org.omg.CORBA.Object duplicate(org.omg.CORBA.Object orig) {
	return orig;
    }

    public void release(org.omg.CORBA.Object obj) {
	// DONT clear out internal variables to release memory !!
	// This delegate may be pointed-to by other objrefs !
    }

    public org.omg.CORBA.Request create_request(org.omg.CORBA.Object obj, 
						org.omg.CORBA.Context ctx,
						String operation,
						org.omg.CORBA.NVList arg_list,
						org.omg.CORBA.NamedValue result)
    {
	return new RequestImpl(orb, obj, ctx, operation, arg_list,
			       result, null, null);
    }

    public org.omg.CORBA.Request request(org.omg.CORBA.Object obj,
					 String operation) 
    {
	return new RequestImpl(orb, obj, null, operation, null, null, null,
			       null);
    }

    public org.omg.CORBA.ORB orb(org.omg.CORBA.Object self) 
    {
	return this.orb;
    }

    public org.omg.CORBA.Request create_request(
						org.omg.CORBA.Object obj, 
						org.omg.CORBA.Context ctx, 
						String operation, 
						org.omg.CORBA.NVList arg_list,
						org.omg.CORBA.NamedValue result, 
						org.omg.CORBA.ExceptionList exclist, 
						org.omg.CORBA.ContextList ctxlist)
    {
	return new RequestImpl(orb, obj, ctx, operation, arg_list, result,
			       exclist, ctxlist);
    }
      
    public org.omg.CORBA.Object get_interface_def(org.omg.CORBA.Object obj) 
    {
	// Invoke on server
	ClientRequest os = createRequest("_interface", false);
	ClientResponse is = invoke(os);

	// Process reply
	if ( is.isSystemException() ) 
	    throw is.getSystemException();
	else if ( is.isLocationForward() ) {
	    locatedIOR = is.getForwardedIOR();
		    
	    // Invoke again.
	    return get_interface_def(obj);
	}
	else { // normal return
	    ObjectImpl objimpl = (ObjectImpl) is.read_Object();

	    // check if returned object is of correct type
	    if ( !objimpl._is_a("IDL:omg.org/CORBA/InterfaceDef:1.0") )
		throw new org.omg.CORBA.UNKNOWN(
		    "InterfaceDef object of wrong type returned by server", 0, 
		    CompletionStatus.COMPLETED_MAYBE);
	
	    // instantiate the stub
	    ObjectImpl stub;
	    try {
		stub = (ObjectImpl)
		    JDKBridge.loadClass("org.omg.CORBA._InterfaceDefStub").newInstance();
	    } catch (Exception ex) {
		throw new org.omg.CORBA.UNKNOWN(
		    "org.omg.CORBA._InterfaceDefStub class not available", 0, 
		    CompletionStatus.COMPLETED_NO);
	    }
	
	    // set the delegate in the stub
	    org.omg.CORBA.portable.Delegate del = objimpl._get_delegate();

	    stub._set_delegate(del);
	
	    return stub;
	}
    }

    public boolean is_a(org.omg.CORBA.Object obj, String dest) 
    {
	// dest is the typeId of the interface to compare against.
	// repositoryIds is the list of typeIds that the stub knows about.
	// First we look for an answer using local information.
	// But repositoryIds may not be complete, so it may be necessary to
	// go to server.
	String [] repositoryIds = ((ObjectImpl)obj)._ids();
	String myid = ior.getTypeId();
	if ( dest.equals(myid) )
	    return true;
	for ( int i=0; i<repositoryIds.length; i++ )
	    if ( dest.equals(repositoryIds[i]) )
		return true;

	// Invoke on server
	ClientRequest os = createRequest("_is_a", false);
	os.write_string(dest);
	ClientResponse is = invoke(os);

	// Process reply
	if ( is.isSystemException() ) 
	    throw is.getSystemException();
	else if ( is.isLocationForward() ) {
	    locatedIOR = is.getForwardedIOR();
	    // Invoke again.
	    return is_a(obj, dest);
	}
	else // normal return
	    return is.read_boolean();
    }

    public boolean non_existent(org.omg.CORBA.Object obj) 
    {
	try {
	    // Invoke on server. operation name *is* _non_existent!
	    ClientRequest os = createRequest("_non_existent", false);
	    ClientResponse is = invoke(os);

	    // Process reply
	    if ( is.isSystemException() ) 
		return true;
	    else if ( is.isLocationForward() ) {
		locatedIOR = is.getForwardedIOR();
		// Invoke again.
		return non_existent(obj);
	    }
	    else // normal return
		return is.read_boolean();
	} catch (SystemException ex) {
	    return true;
	}
    }

    public OutputStream request(org.omg.CORBA.Object self,
				String operation,
				boolean responseExpected) {
	return (OutputStream) createRequest(operation, !responseExpected);
    }

    /**
     * Returns true if this object is implemented by a local servant. 
     *
     * @param self The object reference which delegated to this delegate.
     * @return true only if the servant incarnating this object is located in 
     * this Java VM. Return false if the servant is not local or the ORB 
     * does not support local stubs for this particular servant. The default 
     * behavior of is_local() is to return false.
     */
    public boolean is_local(org.omg.CORBA.Object self) {
	return servant != null;
    }
	
    /**
     * Returns a Java reference to the servant which should be used for this 
     * request. servant_preinvoke() is invoked by a local stub.
     * If a ServantObject object is returned, then its servant field 
     * has been set to an object of the expected type (Note: the object may 
     * or may not be the actual servant instance). The local stub may cast 
     * the servant field to the expected type, and then invoke the operation 
     * directly. The ServantRequest object is valid for only one invocation, 
     * and cannot be used for more than one invocation.
     *
     * @param self The object reference which delegated to this delegate.
     *
     * @param operation a string containing the operation name.
     * The operation name corresponds to the operation name as it would be 
     * encoded in a GIOP request.
     *
     * @param a Class object representing the expected type of the servant.
     * The expected type is the Class object associated with the operations 
     * class of the stub's interface (e.g. A stub for an interface Foo, 
     * would pass the Class object for the FooOperations interface).
     *
     * @return a ServantObject object.
     * The method may return a null value if it does not wish to support 
     * this optimization (e.g. due to security, transactions, etc). 
     * The method must return null if the servant is not of the expected type.
     */
    public ServantObject servant_preinvoke(org.omg.CORBA.Object self,
					   String operation,
					   Class expectedType) {        
	if (servant != null &&
	    expectedType.isAssignableFrom(servant.servant.getClass())) {
		
	    // _REVISIT_ Security check? If we put a security check
	    // on the client side of a connection (IIOPConnection.send()?)
	    // to ensure that the client has permission, do so here as
	    // well. Since a permission check is done when the socket
	    // is initially opened, but not on each use, we probably
	    // need to do the check ourselves since the call stack can
	    // be completely different each time, and a new caller may
	    // not have permission.
		
	    return servant;
	}
	return null;
    }

    /* Returns the codebase for object reference provided.
     * @param self the object reference whose codebase needs to be returned.
     * @return the codebase as a space delimited list of url strings or
     * null if none.
     */
    public String get_codebase(org.omg.CORBA.Object self) {
	if (ior != null) {
	    return ior.getCodebase();
	}
	return null;
    }
	
    public String toString() {
	return ior.stringify();
    }	
}
