/*
 * @(#)ORB.java	1.201 01/02/09
 *
 * Copyright 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.corba;

// Import JDK stuff
import java.net.*;
import java.util.*;
import java.applet.Applet;
import java.lang.reflect.Field ;
import java.lang.reflect.Modifier ;
import java.io.File ;
import java.io.IOException ;
import java.io.FileInputStream ;
import java.io.FileNotFoundException ;
import java.rmi.server.RMISocketFactory;
import javax.rmi.CORBA.ValueHandler;

// Import our stuff
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.INITIALIZE;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.Environment;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CORBA.NVList;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.Object;
import org.omg.CORBA.Request;
import org.omg.CORBA.OBJ_ADAPTER;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.StructMember;
import org.omg.CORBA.UnionMember;
import org.omg.CORBA.ValueMember;
import org.omg.CORBA.BAD_PARAM;

import org.omg.CORBA.portable.*;
import org.omg.CORBA.portable.ValueFactory;

import com.sun.corba.se.internal.core.ClientSubcontract;
import com.sun.corba.se.internal.core.ServerSubcontract;
import com.sun.corba.se.internal.core.SubcontractRegistry;
import com.sun.corba.se.internal.core.ClientGIOP;
import com.sun.corba.se.internal.core.ServerGIOP;
import com.sun.corba.se.internal.core.MarshalInputStream;
import com.sun.corba.se.internal.core.MarshalOutputStream;
import com.sun.corba.se.internal.core.IOR;

import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.util.Utility;
import com.sun.corba.se.internal.orbutil.SubcontractList;

import com.sun.corba.se.internal.orbutil.ValueHandlerImpl_1_3;
import com.sun.corba.se.internal.orbutil.ORBConstants;
import com.sun.corba.se.internal.core.ORBVersion;
import com.sun.corba.se.internal.core.ORBVersionFactory;
import com.sun.corba.se.internal.orbutil.ORBUtility;

import com.sun.corba.se.internal.iiop.CDRInputStream;
import com.sun.corba.se.internal.iiop.CDROutputStream;
import com.sun.corba.se.internal.iiop.Connection;


import java.security.AccessController;
import sun.security.action.GetPropertyAction;

//d11638

import javax.rmi.CORBA.Util;
import com.sun.org.omg.SendingContext.CodeBase;
import com.sun.org.omg.SendingContext.CodeBaseHelper;

/** 
 * The JavaIDL ORB implementation.
 */
public abstract class ORB extends com.sun.corba.se.internal.core.ORB implements TypeCodeFactory
{

    //d11638; pure java orb, caching the servant IOR per ORB
    private static IOR ior = null;

    protected void dprint( String msg ) 
    {
	Utility.dprint( this, msg ) ;
    }

    // Flag set at compile time to debug flag processing: this can't 
    // be one of the xxxDebugFlags because it is used to debug the mechanism
    // that sets the xxxDebugFlags!
    public static boolean ORBInitDebug = false ;

    // Currently defined debug flags.  Any additions must be called xxxDebugFlag.
    // All debug flags must be public boolean types.
    // These are set by passing the flag -ORBDebug x,y,z in the ORB init args.
    // Note that x,y,z must not contain spaces.
    public boolean transportDebugFlag = false ;
    public boolean subcontractDebugFlag = false ;
    public boolean poaDebugFlag = false ;
    public boolean orbdDebugFlag = false ;
    public boolean namingDebugFlag = false ;
    public boolean serviceContextDebugFlag = false ;
    public boolean transientObjectManagerDebugFlag = false ;

    private static final String[] JavaIDLPropertyNames = {
        "com.sun.corba.se.internal.POA.ORBDebug",
	"org.omg.CORBA.ORBInitialHost",
	"org.omg.CORBA.ORBInitialPort",
	"org.omg.CORBA.ORBInitialServices",
	"org.omg.CORBA.ORBPort", // do we do something with this?
        "org.omg.CORBA.ORBSocketFactoryClass",
	"com.sun.CORBA.ORBServerPort",
	"com.sun.CORBA.ORBServerHost"
    };
    
    private static final String[] JavaIDLURLPropertyNames = {
	"org.omg.CORBA.ORBInitialServices"
    };

    //
    // The following fields form our special little collection of global state. 
    // We keep it bottled up here in the ORB class and attach a reference to
    // ourselves to every object reference that we create and handle.
    //

    // Vector holding deferred Requests
    private Vector    _dynamicRequests;
    protected SynchVariable     _svResponseReceived;

    // Applet/command-line parameters
    protected String ORBInitialHost = "";
    protected int ORBInitialPort;
    protected String ORBServerHost = "";
    protected int ORBServerPort = 0;
    protected String appletHost = "";
    protected URL appletCodeBase = null;
    protected RMISocketFactory socketFactory;
    protected ClassLoader classLoader;

    // Synchronization variable for shutdown 
    protected java.lang.Object shutdownObj = new java.lang.Object();

    InitialNamingClient initialNamingClient;

    // This map is needed for resolving recursive type code placeholders
    // based on the unique repository id.
    // _REVISIT_ No garbage collection. For Java2 use WeakHashMap.
    private Hashtable typeCodeMap = null;
    private String savedId = null;
    private TypeCodeImpl savedCode = null;

    // Cache to hold ValueFactories (Helper classes) keyed on repository ids
    protected Hashtable valueFactoryCache = new Hashtable();

    // thread local variable to store the current ORB version
    private ThreadLocal orbVersionThreadLocal;

    /** 
     * Create a new ORB. Should only be invoked from the
     * CORBA ORB superclass. Should be followed by the appropriate
     * set_parameters() call.
     */
    public ORB() 
    {
        _dynamicRequests = new Vector();
        _svResponseReceived = new SynchVariable();

	initialNamingClient = new InitialNamingClient(this);
   
	// default ORB version is the version of ORB with correct Rep-id
	// changes
        orbVersionThreadLocal  = new ThreadLocal () {
           protected java.lang.Object initialValue() {
               // set default to version of the ORB with correct Rep-ids
               return ORBVersionFactory.getORBVersion() ; 
           }
        };
    }

    public ORBVersion getORBVersion()
    {
	// return the thread local data
	return (ORBVersion)(orbVersionThreadLocal.get()) ;
    }

    public void setORBVersion(ORBVersion verObj)
    {
	// set the thread local data
	orbVersionThreadLocal.set(verObj);
    }

    /****************************************************************************
     * The following methods deal with parsing parameters and doing appropriate
     * initialization.
     ****************************************************************************/

    /**
     * Initialize any necessary ORB state; get attributes if possible.
     * Called from org.omg.CORBA.ORB.init().
     * @param app the applet
     * @param props the applet properties
     */
    protected void set_parameters(Applet app, Properties props) 
    {
        // Note: In some applet-development frameworks
        // the applet object may not be directly available and so it's
        // possible for the Applet instance to be null.

	// Get the full list of property names that we are interested in
        String[] propertyNames = getPropertyNames();

        if (app != null) {
            appletHost = app.getCodeBase().getHost( );
            appletCodeBase = app.getCodeBase();
            classLoader = app.getClass().getClassLoader();
        }

        // Build up the full list of configuration properties
	// from the applet-params and props-argument. 

        Properties propList = new Properties();

        // Until we decide it's ok for getSystemProperty() to
        // use AccessController.beginPrivileged(), I've commented
        // out the use of System properties for applets since
        // this will result in confusing SecurityExceptions in
        // most situations.
        //
        // findORBPropertiesFromSystem(propList, propertyNames);

        findPropertiesFromProperties(propList, props, propertyNames);
        findPropertiesFromApplet(propList, app, propertyNames);

        checkAppletPropertyDefaults(propList);

        // Use the full props list to set ORB state.
        parseProperties(propList);
    }

    /**
     * Initialize any necessary ORB state; get attributes if possible.
     * Called from org.omg.CORBA.ORB.init().
     * @param params An array of parameters in the form of alternating <br>
     * "-param-name" and "param-value" strings.
     * @param props the application properties
     */
    protected void set_parameters (String[] params, Properties props)
    {
	// Get the full list of property names that we are interested in
        String[] propertyNames = getPropertyNames();

        // Build up the full list of configuration properties
	// from the command-line-params, props-argument, System properties.

        Properties propList = new Properties();

        findORBPropertiesFromSystem(propList, propertyNames);
        findPropertiesFromProperties(propList, props, propertyNames);
	findPropertiesFromFile( propList, propertyNames ) ;
        findPropertiesFromArgs(propList, params, propertyNames);

        checkApplicationPropertyDefaults(propList);

        // Use the full props list to set ORB state.
        parseProperties(propList);
    }


    /** Return a list of property names that this ORB is interested in.
     *  This may be overridden by subclasses, but subclasses must call
     *  super.getPropertyNames() to get all names.
     */
    protected String[] getPropertyNames()
    {
	String[] names = new String[JavaIDLPropertyNames.length];
	for ( int i=0; i<JavaIDLPropertyNames.length; i++ ) 
	    names[i] = JavaIDLPropertyNames[i]; 

	if (ORBInitDebug)
	    dprint( "getPropertyNames returns " + Utility.objectToString( names ) ) ;

	return names;
    }


    protected boolean singleParam(String param)
    {
	// Return true if this param does not have a data field after it
	// (e.g. for on/off flags).

        return false;
    }

    protected String findMatchingPropertyName( String[] propertyNames,
					       String suffix ) 
    {
	for (int ctr=0; ctr<propertyNames.length; ctr++) 
	    if (propertyNames[ctr].endsWith( suffix ))
		return propertyNames[ctr] ;

	return null ;
    }

    //
    // Map command-line arguments to ORB properties.
    //
    protected void findPropertiesFromArgs(Properties props, String[] params,
					  String[] propertyNames) 
    {
	if (ORBInitDebug) 
	    dprint( "findPropertiesFromArgs called with params=" +
		    Utility.objectToString( params ) + " propertyNames = " +
		    Utility.objectToString( propertyNames ) ) ;

        if (params == null) 
            return;

	// All command-line args are of the form "-ORBkey value".
	// The key is mapped to org.omg.CORBA.key.

	String name ;
	String value ;

        for ( int i=0; i<params.length; i++ ) {
	    value = null ;
	    name = null ;

            if ( params[i] != null && params[i].startsWith("-ORB") ) {
		String argName = params[i].substring( 1 ) ;
		name = findMatchingPropertyName( propertyNames, argName ) ;

		if (name != null)
		    if (singleParam(params[i])) {
			value = params[i] ;
		    } else if ( i+1 < params.length && params[i+1] != null ) {
			value = params[++i];
		    } 
            }

	    if (value != null) {
		if (ORBInitDebug) 
		    dprint( "Found property " + name + "=" + value + 
			    " in args" ) ;

		props.put( name, value ) ;
	    }
        }  
    }

    //
    // Map applet parameters to ORB properties.
    //
    protected void findPropertiesFromApplet(Properties props, Applet app, 
					    String[] propertyNames) 
    {
        if (app == null)
            return;

        for (int i=0; i < propertyNames.length; i++) {

            String value = app.getParameter(propertyNames[i]);
            if (value == null)
	        continue;

            props.put(propertyNames[i], value);
        }

        //
        // Special Case:
        //
        // Convert any applet parameter relative URLs to an
        // absolute URL based on the Document Root. This is so HTML URLs can be
        // kept relative which is sometimes useful for managing the
        // Document Root layout.
        //
        for (int i=0; i < JavaIDLURLPropertyNames.length; i++) {
            String value;

            value = props.getProperty(JavaIDLURLPropertyNames[i]);
            if (value == null)
                continue;

            try {
	        URL url;
                url = new URL(app.getDocumentBase(), value);
	        props.put(JavaIDLURLPropertyNames[i], url.toExternalForm());
            } catch (java.net.MalformedURLException ex) {
	        //
	        // This will be caught again later if this property is used.
	        // Don't worry about reporting exceptions now.
	        //
            }
        }
    }

    private static Class thisClass = ORB.class;
    private static String getSystemProperty(final String name) {
        return (String)AccessController.doPrivileged(new GetPropertyAction(name));
    }

    Properties getFileProperties() 
    {
	String userDir = getSystemProperty( "user.dir" ) ;
	File file = new File( userDir, "orb.properties" ) ;
	FileInputStream in = null ;

	try {
	    in = new FileInputStream( file ) ;
	} catch (FileNotFoundException fnne) {
	    if (ORBInitDebug) 
		dprint( "ORB properties file not found: " + fnne) ;

	    return null ;
	}

	Properties result = new Properties() ;

	try {
	    result.load( in ) ;
	} catch (IOException ioexc) {
	    if (ORBInitDebug) 
		dprint( "Error in loading properties from file properties" ) ;

	    return null ;
	}

	return result ;
    }

    protected void findPropertiesFromFile(Properties props, 
					  String[] propertyNames) 
    {
	Properties fileProps = getFileProperties() ;
	if (fileProps==null)
	    return ;

        for (int i=0; i < propertyNames.length; i++) {
            String value;

            value = (String)fileProps.get(propertyNames[i]);
            if (value == null)
	        continue;

            props.put(propertyNames[i], value);
	    if (ORBInitDebug) 
		dprint( "Found property " + propertyNames[i] + "=" +
			value + " in file properties" ) ;
	}
    }

    //
    // Map System properties to ORB properties.
    // Security bug fix 4278205:
    // Allow only reading of system properties with ORB prefixes.
    // Previously a malicious subclass was able to read ANY system property.
    //
    private void findORBPropertiesFromSystem(Properties props, 
					    String[] propertyNames) 
    {
        for (int i=0; i < propertyNames.length; i++) {
            if (propertyNames[i].startsWith("org.omg.CORBA") ||
                propertyNames[i].startsWith("com.sun.corba") ||
                propertyNames[i].startsWith("com.sun.CORBA"))
            {
                String value = getSystemProperty(propertyNames[i]);
                if (value != null) {
                    props.put(propertyNames[i], value);
                    if (ORBInitDebug)
                        dprint( "Found property " + propertyNames[i] + "=" +
                                value + " in system properties" ) ;
                }
            }
        }
    }


    //
    // Map/copy the properties argument from set_properties() into our
    // ORB properties object.
    //
    protected void findPropertiesFromProperties(Properties props, 
						Properties arg, 
						String[] propertyNames)
    {
	if (ORBInitDebug)
	    dprint( "FindPropertiesFromProperties called with args = " + 
		    Utility.objectToString( arg ) + "propertytNames = " + 
		    Utility.objectToString( propertyNames ) ) ;

        if (arg == null)
            return;

        for (int i=0; i < propertyNames.length; i++) {
            String value;

            value = arg.getProperty(propertyNames[i]);
            if (value == null)
	        continue;

            props.put(propertyNames[i], value);
	    if (ORBInitDebug) 
		dprint( "Found property " + propertyNames[i] + "=" +
			value + " in properties argument" ) ;
        }
    }

    // Set appropriate defaults for an applet ORB.
    private void checkAppletPropertyDefaults(Properties props) 
    {
        String host =
	    props.getProperty("org.omg.CORBA.ORBInitialHost"); 
        if ((host == null) || (host.equals(""))) {
	    props.put("org.omg.CORBA.ORBInitialHost", appletHost);
        }
	
	String serverHost =
	    props.getProperty("com.sun.CORBA.ORBServerHost");
	if ((serverHost == null) || (serverHost.equals(""))) {
	    try {
		// Using IP address supports access
		// across domains
		String hostip = InetAddress.getLocalHost().getHostAddress();
		props.put("com.sun.CORBA.ORBServerHost",hostip);
	    }
	    catch (java.net.UnknownHostException ex) {
		// preserve original behavior
		// in case of exception
		props.put("com.sun.CORBA.ORBServerHost",
				Connection.getLocalHost());
	    }
	}
    }

    // Set appropriate defaults for an application ORB.
    private void checkApplicationPropertyDefaults(Properties props) 
    {
        String host = props.getProperty("org.omg.CORBA.ORBInitialHost");
        if ((host == null) || (host.equals(""))) {
	    try {
		// Using IP address supports access
		// across domains
		String hostip = InetAddress.getLocalHost().getHostAddress();
		props.put("org.omg.CORBA.ORBInitialHost",hostip);
	    }
	    catch (java.net.UnknownHostException ex) {
		// preserve original behavior
		// in case of exception
	        props.put("org.omg.CORBA.ORBInitialHost", 
		      com.sun.corba.se.internal.iiop.Connection.getLocalHost());
	    }
        }
	String serverHost =
	    props.getProperty("com.sun.CORBA.ORBServerHost");
	if ((serverHost == null) || (serverHost.equals(""))) {
	    try {
		// Using IP address supports access
		// across domains
		String hostip = InetAddress.getLocalHost().getHostAddress();
		props.put("com.sun.CORBA.ORBServerHost", hostip);
	    } 
	    catch (java.net.UnknownHostException ex) {
		// preserve original behavior
		// in case of exception
		props.put("com.sun.CORBA.ORBServerHost",
		      Connection.getLocalHost());
	    }
	}
    }

    protected void setDebugFlags( String args ) 
    {
	StringTokenizer st = new StringTokenizer( args, "," ) ;
	while (st.hasMoreTokens()) {
	    String token = st.nextToken() ;

	    // If there is a public boolean data member in this class
	    // named token + "DebugFlag", set it to true.
	    try {
		Field fld = this.getClass().getField( token + "DebugFlag" ) ; 
		int mod = fld.getModifiers() ;
		if (Modifier.isPublic( mod ) && !Modifier.isStatic( mod ))
		    if (fld.getType() == boolean.class)
			fld.setBoolean( this, true ) ;
	    } catch (Exception exc) {
		// ignore it
	    }
	}
    }

    /** Use the properties object to configure ORB state.
     *  This may be overridden by subclasses, but they must call
     *  super.parseProperties() to allow this class to set its state.
     */ 
    protected void parseProperties(Properties props)
    {
	// get server debug flags
        String debugFlags = props.getProperty("com.sun.corba.se.internal.POA.ORBDebug");
        if (debugFlags != null) {
	    if (ORBInitDebug)
		dprint( "Setting debug flags to " + debugFlags ) ;
            setDebugFlags(debugFlags) ;
        }
    
        String param = props.getProperty("org.omg.CORBA.ORBInitialHost");
        if (param != null) {
	    if (ORBInitDebug)
		dprint( "setting initial host to " + param ) ;
            ORBInitialHost = param;
        }

        param = props.getProperty("org.omg.CORBA.ORBInitialPort");
        if (param != null) {
            try {
		ORBInitialPort = Integer.parseInt(param);
		if (ORBInitDebug)
		    dprint( "setting initial services port to " + ORBInitialPort ) ;
	        initialNamingClient.setInitialServicesPort(ORBInitialPort);
            } catch (java.lang.NumberFormatException e) {}
        }

	param = props.getProperty("com.sun.CORBA.ORBServerHost");
	if (param != null) {
	    if (ORBInitDebug)
		dprint( "setting ORB server host to " + param ) ;
	    ORBServerHost = param;
	}

	param = props.getProperty("com.sun.CORBA.ORBServerPort");
	if (param != null) {
	    try {
		ORBServerPort = Integer.parseInt(param);
		if (ORBInitDebug)
		    dprint( "setting ORB server port to " + ORBServerPort ) ;
	    } catch (java.lang.NumberFormatException e) { }
	}

	/* _REVISIT_ Is this needed ???
	   param = props.getProperty("org.omg.CORBA.ORBPort");
	   if (param != null) {
	   serverPort = java.lang.Integer.parseInt(param);
	   }
	*/

        param = props.getProperty("org.omg.CORBA.ORBInitialServices");
        if (param != null) {
	    try {
		if (ORBInitDebug)
		    dprint( "setting initial services URL to " + param ) ;
	        initialNamingClient.setServicesURL(new URL(param));
	    } catch (java.io.IOException ex) {
	        // Fallthrough
            }
	}

        param = props.getProperty("org.omg.CORBA.ORBSocketFactoryClass");
        if (param != null) {
            try {
                Class socketFactoryClass;
                try {
                    socketFactoryClass = Class.forName(param);
                } catch (ClassNotFoundException cnfe) {
                    // try again...
                    socketFactoryClass = classLoader.loadClass(param);
                }
                // For security reasons avoid creating an instance if this socket factory class
                // is not one that would fail the class cast anyway.
                if (RMISocketFactory.class.isAssignableFrom(socketFactoryClass)) {
                    socketFactory = (RMISocketFactory)socketFactoryClass.newInstance();
                } else {
                    // throw some exception just to get into the outer catch clause
                    throw new ClassCastException();
                }
            } catch (Exception ex) {
                // ClassNotFoundException, IllegalAccessException, InstantiationException,
                // SecurityException or ClassCastException
                throw new INITIALIZE("can't instantiate custom socket factory" + param);
            }
        } else {
            socketFactory = new DefaultSocketFactory();
        }
    }




    /****************************************************************************
     * The following methods are getters and setters for ORB variables.
     ****************************************************************************/


    /** 
     * Get the name of the host running the initial services nameserver.
     * @return The name of the ORBInitialHost.
     */
    public String getORBInitialHost()
    {
        return ORBInitialHost;
    }

    /** 
     * Get the port of the initial services nameserver.
     */
    public int getORBInitialPort()
    {
        return ORBInitialPort;
    }


    public String getORBServerHost() {
        return ORBServerHost;
    }

    public int getORBServerPort() {
        return ORBServerPort;
    }


    /** 
     * Get the name of the host from which this applet was downloaded.
     * @return The name of the AppletHost.
     */
    public String getAppletHost()
    {
        return appletHost;
    }

    /** 
     * Get the codebase from which this applet was downloaded.
     */
    public URL getAppletCodeBase()
    {
        return appletCodeBase;
    }

    /**
     * Get the socket factory for this ORB.
     */
    public RMISocketFactory getSocketFactory()
    {
        return socketFactory;
    }

    public org.omg.CORBA.portable.OutputStream create_output_stream()
    {
        return new CDROutputStream(this);
    }

    


    /**
     * Get a Current pseudo-object.
     * The Current interface is used to manage thread-specific
     * information for use by the transactions, security and other
     * services. This method is deprecated, 
     * and replaced by ORB.resolve_initial_references("NameOfCurrentObject");
     *
     * @return          a Current pseudo-object.
     * @deprecated
     */
    public org.omg.CORBA.Current get_current()
    {
        /* _REVISIT_
           The implementation of get_current is not clear. How would
           ORB know whether the caller wants a Current for transactions
           or security ?? Or is it assumed that there is just one
           implementation for both ? If Current is thread-specific,
           then it should not be instantiated; so where does the
           ORB get a Current ? */
        throw new NO_IMPLEMENT();
    }




    /****************************************************************************
     * The following methods deal with creation of various types of lists.
     ****************************************************************************/

    /**
     * Create an NVList
     *
     * @param count	size of list to create
     * @result		NVList created
     *
     * @see NVList
     */
    public NVList create_list(int count)
    {
        return new NVListImpl(this, count);
    }

    /**
     * Create an NVList corresponding to an OperationDef
     *
     * @param oper	operation def to use to create list
     * @result		NVList created
     *
     * @see NVList
     */
    public NVList create_operation_list(org.omg.CORBA.Object oper)
    {
        throw new NO_IMPLEMENT();
    }

    /**
     * Create a NamedValue
     *
     * @result		NamedValue created
     */
    public NamedValue create_named_value(String s, Any any, int flags)
    {
	return new NamedValueImpl(this, s, any, flags);
    }

    /**
     * Create an ExceptionList
     *
     * @result		ExceptionList created
     */
    public org.omg.CORBA.ExceptionList create_exception_list()
    {
	return new ExceptionListImpl();
    }

    /**
     * Create a ContextList
     *
     * @result		ContextList created
     */
    public org.omg.CORBA.ContextList create_context_list()
    {
        return new ContextListImpl(this);
    }

    /**
     * Get the default Context object
     *
     * @result		the default Context object
     */
    public org.omg.CORBA.Context get_default_context()
    {
        throw new NO_IMPLEMENT();
    }

    /**
     * Create an Environment
     *
     * @result		Environment created
     */
    public org.omg.CORBA.Environment create_environment()
    {
        return new EnvironmentImpl();
    }




    /****************************************************************************
     * The following methods deal with multiple/deferred DII invocations.
     ****************************************************************************/

    public void send_multiple_requests_oneway(Request[] req)
    {
        // Invoke the send_oneway on each new Request
        for (int i = 0; i < req.length; i++) {
            req[i].send_oneway();
        }
    }
 
    /**
     * Send multiple dynamic requests asynchronously.
     *
     * @param req         an array of request objects.
     */
    public void send_multiple_requests_deferred(Request[] req)
    {
        // add the new Requests to pending dynamic Requests
        for (int i = 0; i < req.length; i++) {
            _dynamicRequests.addElement(req[i]);
        }
 
        // Invoke the send_deferred on each new Request
        for (int i = 0; i < req.length; i++) {
	    AsynchInvoke invokeObject = new AsynchInvoke(this,
							 (com.sun.corba.se.internal.corba.RequestImpl)req[i], true);
	    new Thread(invokeObject).start();
        }
    }


    /**
     * Find out if any of the deferred invocations have a response yet. 
     */
    public boolean poll_next_response()
    {
        Request currRequest;
 
        // poll on each pending request
        Enumeration ve = _dynamicRequests.elements();
        while (ve.hasMoreElements() == true) {
            currRequest = (Request)ve.nextElement();
            if (currRequest.poll_response() == true) {
                return true;
            }
        }
        return false;
    }
 
    /**
     * Get the next request that has gotten a response.
     *
     * @result            the next request ready with a response.
     */
 
    public org.omg.CORBA.Request get_next_response()
        throws org.omg.CORBA.WrongTransaction
    {

	while (true) {
	    // check if there already is a response
	    synchronized ( _dynamicRequests ) {
		Enumeration elems = _dynamicRequests.elements();
		while ( elems.hasMoreElements() ) {
		    Request currRequest = (Request)elems.nextElement();
		    if ( currRequest.poll_response() ) {
			// get the response for this successfully polled Request
			currRequest.get_response();
			_dynamicRequests.removeElement(currRequest);
			return currRequest;
		    }
		}
	    }

	    // wait for a response
	    synchronized(this._svResponseReceived) {
	        this._svResponseReceived.reset();
		while (this._svResponseReceived.value() == false) {
    		    try {
		        this._svResponseReceived.wait();
    		    } catch(java.lang.InterruptedException ex) {}
		}
		// reinitialize the response flag
		this._svResponseReceived.reset();
	    }
	}
    }

    /** 
     * Notify response to ORB for get_next_response
     */
    synchronized void notifyResponse() {
        this._svResponseReceived.set();
        this._svResponseReceived.notify();
    }



    /****************************************************************************
     * The following methods deal with stringifying/destringifying
     * and connecting/disconnecting object references.
     ****************************************************************************/

    /**
     * Convert an object ref to a string.
     * @param obj The object to stringify.
     * @return A stringified object reference.
     */
    public String object_to_string(org.omg.CORBA.Object obj) 
    {

	// Handle the null objref case
        if (obj == null) 
	    return IOR.NULL.stringify(this);

	// Try and get the objref's delegate if it exists
        ClientSubcontract rep=null;
        ObjectImpl oi = (ObjectImpl)obj;
        try {
	    // throws BAD_OPERATION if no delegate
            rep = (ClientSubcontract)oi._get_delegate();  
        } catch ( Exception ex ) {}

        if ( rep == null ) {
            // No delegate: servant was not connected to ORB
	    connect(obj);
	    rep = (ClientSubcontract)oi._get_delegate();
        }

        return rep.marshal().stringify();
    }


    static protected int hexOf(char x)
    {
        int val;

        val = x - '0';
        if (val >=0 && val <= 9)
            return val;

        val = (x - 'a') + 10;
        if (val >= 10 && val <= 15)
            return val;

        val = (x - 'A') + 10;
        if (val >= 10 && val <= 15)
            return val;

        throw new DATA_CONVERSION(MinorCodes.BAD_HEX_DIGIT, 
          			  CompletionStatus.COMPLETED_NO);
    }

    /**
     * Convert a stringified object reference to the object it represents.
     * @param str The stringified object reference.
     * @return The unstringified object reference.
     */
    public org.omg.CORBA.Object string_to_object(String str) 
    {
        if ( str == null )
            throw new org.omg.CORBA.BAD_PARAM(MinorCodes.NULL_PARAM, 
					      CompletionStatus.COMPLETED_NO);

        //
        // Must start with IOR and must be even length for dehexification.
        //
        if ( !str.startsWith("IOR:") ) 
            throw new org.omg.CORBA.DATA_CONVERSION( 
						    MinorCodes.BAD_STRINGIFIED_IOR,
						    CompletionStatus.COMPLETED_NO );

        if ( (str.length() & 0x1) != 0 ) {
            throw new org.omg.CORBA.DATA_CONVERSION( 
						    MinorCodes.BAD_STRINGIFIED_IOR_LEN,
						    CompletionStatus.COMPLETED_NO
						    );
	}
          
        //
        // Dehexify
        //
        byte[] buf = new byte[(str.length() - 4) / 2];
        for (int i=4, j=0; i < str.length(); i +=2, j++) {
            buf[j] = (byte)((hexOf(str.charAt(i)) << 4) & 0xF0);
            buf[j] |= (byte)((hexOf(str.charAt(i+1)) << 0) & 0x0F);
        }
          
        CDRInputStream s = new CDRInputStream(this, buf, buf.length);
        s.consumeEndian();   

        return s.read_Object();
    }


    /** This is the implementation of the public API used to connect
     *  a servant-skeleton to the ORB. 
     */
    public void connect(org.omg.CORBA.Object servant)
    {
        int subContractIndex = SubcontractList.defaultSubcontract;
        
        try {
            ServerSubcontract s
		= getSubcontractRegistry().getServerSubcontract(subContractIndex);

            s.createObjref((byte[]) null, servant);
        }
        catch ( Exception ex ) {
	    if (subcontractDebugFlag) {
		dprint( "Error in createObjref: " + ex ) ;
		ex.printStackTrace() ;
	    }

            throw new OBJ_ADAPTER(MinorCodes.ORB_CONNECT_ERROR, 
				  CompletionStatus.COMPLETED_NO);
	}
    }


    public void disconnect(org.omg.CORBA.Object obj)
    {
        try {
            // Get the delegate, then ior, then server-subcontract, 
	    // then tell it to delete the servant from its "servant manager".
            ObjectImpl oi = (ObjectImpl)obj;
            ClientSubcontract rep = (ClientSubcontract)oi._get_delegate();
            IOR ior = rep.marshal();
            byte[] objKey = ior.getProfile().getObjectKey();
            ServerSubcontract sc = getSubcontractRegistry().getServerSubcontract(objKey);
	    sc.destroyObjref(obj);
        }
        catch ( Exception ex ) {
            throw new OBJ_ADAPTER(MinorCodes.ORB_CONNECT_ERROR, CompletionStatus.COMPLETED_MAYBE);
        }
    }

    //d11638: pure java orb support, moved this method from FVDCodeBaseImpl
    // Note that we connect this if we have not already done so.
    public IOR getServantIOR(){

        if (ior != null) // i.e. We are already connected to it
            return ior;

        // backward compatability 4365188
        CodeBase cb;
        ValueHandler vh;

        if (ORBUtility.isLegacyORB(this)) {
            vh = new ValueHandlerImpl_1_3();
        } else {
            vh = Util.createValueHandler();
        }

        cb = (CodeBase)vh.getRunTimeCodeBase();

        connect(cb);

        ClientSubcontract rep=null;
        ObjectImpl oi = (ObjectImpl)cb;
        rep = (ClientSubcontract)oi._get_delegate();
        ior = rep.marshal();
        return ior;

    }



    /****************************************************************************
     * The following methods deal with creation of TypeCodes and Any, etc.
     ****************************************************************************/

    /**
     * Get the TypeCode for a primitive type.
     *
     * @param tcKind	the integer kind for the primitive type
     * @return		the requested TypeCode
     */

    public TypeCode get_primitive_tc(TCKind tcKind)
    {
        // _REVISIT_ if this returns a null, throw an exception perhaps?
        return TypeCodeImpl.get_primitive_tc(tcKind);
    }

    /**
     * Create a TypeCode for a structure.
     *
     * @param id		the logical id for the typecode.
     * @param name	the name for the typecode.
     * @param members	an array describing the members of the TypeCode.
     * @return		the requested TypeCode.
     */

    public TypeCode create_struct_tc(String id,
				     String name,
				     StructMember[] members)
    {
        return new TypeCodeImpl(this, TCKind._tk_struct, id, name, members);
    }
    
    
    /**
     * Create a TypeCode for a union.
     *
     * @param id		the logical id for the typecode.
     * @param name	the name for the typecode.
     * @param discriminator_type
     *			the type of the union discriminator.
     * @param members	an array describing the members of the TypeCode.
     * @return		the requested TypeCode.
     */

    public TypeCode create_union_tc(String id,
				    String name,
				    TypeCode discriminator_type,
				    UnionMember[] members)
    {
        return new TypeCodeImpl(this,
				TCKind._tk_union, 
				id, 
				name, 
				discriminator_type, 
				members);
    }
    
    /**
     * Create a TypeCode for an enum.
     *
     * @param id		the logical id for the typecode.
     * @param name	the name for the typecode.
     * @param members	an array describing the members of the TypeCode.
     * @return		the requested TypeCode.
     */

    public TypeCode create_enum_tc(String id,
				   String name,
				   String[] members)
    {
        return new TypeCodeImpl(this, TCKind._tk_enum, id, name, members);
    }
    
    /**
     * Create a TypeCode for an alias.
     *
     * @param id		the logical id for the typecode.
     * @param name	the name for the typecode.
     * @param original_type
     * 			the type this is an alias for.
     * @return		the requested TypeCode.
     */

    public TypeCode create_alias_tc(String id,
				    String name,
				    TypeCode original_type)
    {
        return new TypeCodeImpl(this, TCKind._tk_alias, id, name, original_type);
    }
    
    /**
     * Create a TypeCode for an exception.
     *
     * @param id		the logical id for the typecode.
     * @param name	the name for the typecode.
     * @param members	an array describing the members of the TypeCode.
     * @return		the requested TypeCode.
     */

    public TypeCode create_exception_tc(String id,
					String name,
					StructMember[] members)
    {
        return new TypeCodeImpl(this, TCKind._tk_except, id, name, members);
    }
    
    /**
     * Create a TypeCode for an interface.
     *
     * @param id		the logical id for the typecode.
     * @param name	the name for the typecode.
     * @return		the requested TypeCode.
     */

    public TypeCode create_interface_tc(String id,
					String name)
    {
        return new TypeCodeImpl(this, TCKind._tk_objref, id, name);
    }
    
    /**
     * Create a TypeCode for a string.
     *
     * @param bound	the bound for the string.
     * @return		the requested TypeCode.
     */

    public TypeCode create_string_tc(int bound)
    {
        return new TypeCodeImpl(this, TCKind._tk_string, bound);
    }
    
    /**
     * Create a TypeCode for a wide string.
     *
     * @param bound	the bound for the string.
     * @return		the requested TypeCode.
     */

    public TypeCode create_wstring_tc(int bound)
    {
        return new TypeCodeImpl(this, TCKind._tk_wstring, bound);
    }
    
    /**
     * Create a TypeCode for a sequence.
     *
     * @param bound	the bound for the sequence.
     * @param element_type
     *			the type of elements of the sequence.
     * @return		the requested TypeCode.
     */

    public TypeCode create_sequence_tc(int bound,
				       TypeCode element_type)
    {
        return new TypeCodeImpl(this, TCKind._tk_sequence, bound, element_type);
    }
    
    
    /**
     * Create a recursive TypeCode in a sequence.
     *
     * @param bound	the bound for the sequence.
     * @param offset	the index to the enclosing TypeCode that is
     *			being referenced.  
     * @return		the requested TypeCode.
     */

    public TypeCode create_recursive_sequence_tc(int bound,
						 int offset)
    {
        return new TypeCodeImpl(this, TCKind._tk_sequence, bound, offset);
    }
    
    
    /**
     * Create a TypeCode for an array.
     *
     * @param length	the length of the array.
     * @param element_type
     *			the type of elements of the array.
     * @return		the requested TypeCode.
     */

    public TypeCode create_array_tc(int length,
				    TypeCode element_type)
    {
        return new TypeCodeImpl(this, TCKind._tk_array, length, element_type);
    }


    public org.omg.CORBA.TypeCode create_native_tc(String id,
                                                   String name)
    {
        return new TypeCodeImpl(this, TCKind._tk_native, id, name);
    }

    public org.omg.CORBA.TypeCode create_abstract_interface_tc(
							       String id,
							       String name)
    {
        return new TypeCodeImpl(this, TCKind._tk_abstract_interface, id, name);
    }

    public org.omg.CORBA.TypeCode create_fixed_tc(short digits, short scale)
    {
        return new TypeCodeImpl(this, TCKind._tk_fixed, digits, scale);
    }

    // orbos 98-01-18: Objects By Value -- begin

    public org.omg.CORBA.TypeCode create_value_tc(String id,
                                                  String name,
                                                  short type_modifier,
                                                  TypeCode concrete_base,
                                                  ValueMember[] members)
    {
        return new TypeCodeImpl(this, TCKind._tk_value, id, name,
                                type_modifier, concrete_base, members);
    }

    public org.omg.CORBA.TypeCode create_recursive_tc(String id) {
        return new TypeCodeImpl(this, id);
    }

    public org.omg.CORBA.TypeCode create_value_box_tc(String id,
						      String name,
						      TypeCode boxed_type)
    {
        return new TypeCodeImpl(this, TCKind._tk_value_box, id, name, boxed_type);
    }

    // orbos 98-01-18: Objects By Value -- end
    
    /**
     * Create a new Any
     *
     * @return		the new Any created.
     */

    public Any create_any()
    {
        return new AnyImpl(this);
    }

    // TypeCodeFactory interface methods.
    // Keeping track of type codes by repository id.

    public void setTypeCode(String id, TypeCodeImpl code) {
        if (typeCodeMap == null) {
            if (savedId == null) {
                savedId = id;
                savedCode = code;
                return;
            }
            typeCodeMap = new Hashtable(64);
            typeCodeMap.put(savedId, savedCode);
            savedId = null;
            savedCode = null;
        }
        typeCodeMap.put(id, code);
    }

    public TypeCodeImpl getTypeCode(String id) {
        if (typeCodeMap == null) {
            if (savedId == null)
                return null;
            return id.equals(savedId) ? savedCode : null;
        }
        return (TypeCodeImpl)typeCodeMap.get(id);
    }



    /****************************************************************************
     * The following methods deal with listing and resolving the initial
     * (bootstrap) object references such as "NameService".
     ****************************************************************************/


    /**
     * Get a list of the initially available CORBA services. 
     * This does not work unless an ORBInitialHost is specified during initialization
     * (or unless there is an ORB running on the AppletHost) since the localhostname 
     * is inaccessible to applets. If a service properties URL was specified,
     * then it is used, otherwise the bootstrapping protocol is used.
     * @return A list of the initial services available.
     */
    public String[] list_initial_services() 
    {
        return initialNamingClient.list_initial_services();
    }


    /**
     * Resolve the stringified reference of one of the initially
     * available CORBA services.
     * @param identifier The stringified object reference of the
     * desired service.
     * @return An object reference for the desired service.
     * @exception InvalidName The supplied identifier is not associated
     * with a known service.
     * @exception SystemException One of a fixed set of Corba system exceptions.
     */
    public org.omg.CORBA.Object resolve_initial_references(String identifier)
        throws InvalidName
    {
	return initialNamingClient.resolve_initial_references(identifier);
    }






    /****************************************************************************
     * The following methods (introduced in POA / CORBA2.1) deal with
     * shutdown / single threading.
     ****************************************************************************/


    public void run()
    {
        synchronized ( shutdownObj ) {
            try {
                shutdownObj.wait();
            } catch ( InterruptedException ex ) {}
        }
    }

    public void shutdown(boolean wait_for_completion)
    {
        if ( wait_for_completion == true ) {
            throw new NO_IMPLEMENT("shutdown with wait_for_completion==true not implemented.");
        }
        synchronized ( shutdownObj ) {
            shutdownObj.notify();
        }
    }

    public boolean work_pending()
    {
	throw new NO_IMPLEMENT();
    }

    public void perform_work()
    {
	throw new NO_IMPLEMENT();
    }

    /**
     * Registers a value factory for a particular repository ID.
     *
     * @param repositoryID the repository ID.
     * @param factory the factory.
     * @return the previously registered factory for the given repository ID, or null
     * if no such factory was previously registered.
     * @exception org.omg.CORBA.BAD_PARAM if the registration fails.
     **/
    public ValueFactory register_value_factory(String repositoryID, ValueFactory factory) {
        if ((repositoryID == null) || (factory == null))
            throw new BAD_PARAM(MinorCodes.UNABLE_REGISTER_VALUE_FACTORY,
                                CompletionStatus.COMPLETED_NO);
        return (ValueFactory)valueFactoryCache.put(repositoryID, factory);
    }

    /**
     * Unregisters a value factory for a particular repository ID.
     *
     * @param repositoryID the repository ID.
     **/
    public void unregister_value_factory(String repositoryID) {
        if (valueFactoryCache.remove(repositoryID) == null)
            throw new BAD_PARAM(MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_NO);
    }
    
    /**
     * Finds and returns a value factory for the given repository ID.
     * The value factory returned was previously registered by a call to
     * {@link #register_value_factory} or is the default factory.
     *
     * @param repositoryID the repository ID.
     * @return the value factory.
     * @exception org.omg.CORBA.BAD_PARAM if unable to locate a factory.
     **/
    public ValueFactory lookup_value_factory(String repositoryID) {
        ValueFactory factory = (ValueFactory)valueFactoryCache.get(repositoryID);
	if (factory != null) {
	    return factory;
	} else {
	    try {
		return Utility.getFactory(null, null, null, repositoryID);
	    } catch(org.omg.CORBA.MARSHAL ex) {
		throw new org.omg.CORBA.BAD_PARAM(ex.getMessage(),
                                    MinorCodes. UNABLE_FIND_VALUE_FACTORY,
                                    CompletionStatus.COMPLETED_NO);
	    }
	}
    }


} // Class ORB

////////////////////////////////////////////////////////////////////////
/// Helper class for a Synchronization Variable
 
class SynchVariable {
 
    // Synchronization Variable
    public boolean _flag;
 
    // Constructor
    SynchVariable() {
        _flag = false;
    }
 
    // set Flag to true
    public synchronized void set() {
        _flag = true;
    }

        // get value
    public synchronized boolean value() {
        return _flag;
    }

    // reset Flag to true
    public synchronized void reset() {
        _flag = false;
    }
}


class DefaultSocketFactory extends RMISocketFactory {
    public Socket createSocket(String host, int port) throws IOException {
        return new Socket(host, port);
    }

    public ServerSocket createServerSocket(int port) throws IOException {
        return new ServerSocket(port);
    }
}
