/*
 * @(#)ServerDelegate.java	1.32 01/02/09
 *
 * Copyright 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


package com.sun.corba.se.internal.corba;

import javax.rmi.CORBA.Tie;

import org.omg.CORBA.portable.ApplicationException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.INTERNAL; 
import org.omg.CORBA.UNKNOWN; 
import org.omg.CORBA.OBJECT_NOT_EXIST;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.ObjectImpl;
import org.omg.CORBA.portable.InvokeHandler;
import org.omg.CORBA.DynamicImplementation;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.UnknownException;
import org.omg.CORBA.CompletionStatus;

import java.rmi.Remote;

import com.sun.corba.se.internal.corba.ORB;
import com.sun.corba.se.internal.core.ServerSubcontract;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.ServerRequest;
import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.DuplicateServiceContext;
import com.sun.corba.se.internal.core.NoSuchServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.UEInfoServiceContext;
import com.sun.corba.se.internal.core.CodeSetServiceContext;
import com.sun.corba.se.internal.core.SendingContextServiceContext;
import com.sun.corba.se.internal.core.ORBVersionServiceContext;
import com.sun.corba.se.internal.core.EndPoint;
import com.sun.corba.se.internal.core.ClientSubcontract;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.MarshalInputStream;
import com.sun.corba.se.internal.core.MarshalOutputStream;
import com.sun.corba.se.internal.core.CodeSetComponentInfo;
import com.sun.corba.se.internal.core.GIOPVersion;

import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.util.Utility;
import com.sun.org.omg.SendingContext.CodeBase;

import com.sun.corba.se.internal.orbutil.TransientObjectManager;
import com.sun.corba.se.internal.core.ORBVersion;
import com.sun.corba.se.internal.core.ORBVersionImpl;
import com.sun.corba.se.internal.core.ORBVersionFactory;
import com.sun.corba.se.internal.orbutil.ORBConstants;

import com.sun.corba.se.internal.iiop.Connection;
import com.sun.corba.se.internal.iiop.IIOPConnection;
import com.sun.corba.se.internal.iiop.CDROutputStream;


public class ServerDelegate implements ServerSubcontract {
    private void dprint( String msg )
    {
	Utility.dprint( this, msg ) ;
    }

    protected ORB orb; // my ORB instance
    protected int scid;  // my subcontract id
    protected TransientObjectManager servants;

    // Added from last version because it broke the build - RTW
    public static final int UNKNOWN_EXCEPTION_INFO_ID = 9;

    // These offsets are for the object key. Assume the first 8 bytes
    // contain the magic and the sc-id.
    public static final int SERVERID_OFFSET = 8;
    public static final int USERKEYLEN_OFFSET = 12;
    public static final int USERKEY_OFFSET = 16;

    public ServerDelegate() {
	servants = new TransientObjectManager();
    }

    public ServerDelegate(ORB orb) {
	this();
	setOrb( orb ) ;
    }

    public void setId(int id) {
	this.scid = id;
    }

    public void setOrb(com.sun.corba.se.internal.core.ORB orb) {
	this.orb = (com.sun.corba.se.internal.corba.ORB) orb;
	servants.setOrb( (com.sun.corba.se.internal.corba.ORB)orb ) ;
    }

    public Class getClientSubcontractClass() {
	return ClientDelegate.class;
    }

    // Need to signal one of OBJECT_HERE, OBJECT_FORWARD, OBJECT_NOT_EXIST.
    public IOR locate(byte[] key) {
        // Check if the serverid matches this server's transientServerId.
        int sId = Utility.bytesToInt(key, SERVERID_OFFSET);
        if ( sId != orb.getTransientServerId() )
            throw new OBJECT_NOT_EXIST(MinorCodes.BAD_SERVER_ID,
				       CompletionStatus.COMPLETED_NO);

	// Get the servant
	java.lang.Object servant = getServant(key);

	if (servant == null)
	    throw new OBJECT_NOT_EXIST(MinorCodes.SERVANT_NOT_FOUND,
				       CompletionStatus.COMPLETED_NO);

	// If we reached here, it means we got the servant,
	// so just return null to signal OBJECT_HERE.
	return null;
    }

    public Object getServant(byte[] key) {
	int serverId = Utility.bytesToInt(key, SERVERID_OFFSET);
	if (serverId != orb.getTransientServerId())
	    return null;

	// Get the userkey and lookup servant list
        int len = Utility.bytesToInt(key, USERKEYLEN_OFFSET);
        byte[] userKey = new byte[len];
        for (int i = 0; i < len; i++ )
            userKey[i] = key[i+USERKEY_OFFSET];
        return servants.lookupServant(userKey);
    }

    public boolean isServantSupported() {
        return true;
    }    

    public void consumeServiceContexts(ServerRequest request) {
	ServiceContexts ctxts = request.getServiceContexts();
	ServiceContext sc ;

        boolean codeSC = false; // codeset ServiceContext

        GIOPVersion giopVersion = request.getGIOPVersion();
	try {
	    sc = ctxts.get( CodeSetServiceContext.SERVICE_CONTEXT_ID ) ;
	
	    CodeSetServiceContext cssc = (CodeSetServiceContext)sc ;
	    CodeSetComponentInfo.CodeSetContext csctx =
		cssc.getCodeSetContext() ;

            codeSC = true;
	    
	    if (cssc.getCodeSetContext().validateWith( 
		CodeSetComponentInfo.myCodeSet) == false)
		throw new DATA_CONVERSION( MinorCodes.CODESET_INCOMPATIBLE,
					   CompletionStatus.COMPLETED_NO);
	} catch (NoSuchServiceContext exc) {
	    // ignore: this type not present
	}

	try {
	    sc = ctxts.get( 
		SendingContextServiceContext.SERVICE_CONTEXT_ID ) ;

	    SendingContextServiceContext scsc = 
		(SendingContextServiceContext)sc ;
	    IOR ior = scsc.getIOR() ;

	    try {
		IIOPConnection c = 
		    IIOPConnection.getCurrentReaderConnection();
		c.setCodeBaseRef(ior);
	    } catch (ThreadDeath td) {
		throw td ;
	    } catch (Throwable t) {
		throw new DATA_CONVERSION( MinorCodes.BAD_STRINGIFIED_IOR,
					   CompletionStatus.COMPLETED_NO);
	    }
	} catch (NoSuchServiceContext exc) {
	    // ignore: this type not present
	}

        // the RTSC is sent only once during session establishment.  We
        // need to find out if the CodeBaseRef is already set.  If yes,
        // then also the rtSC flag needs to be set to true
        // this is not possible for the LocalCase since there is no
        // IIOPConnection for the LocalCase

        // used for a case where we have JDK 1.3 supporting 1.0 protocol,
        // but sending 2 service contexts, that is not normal as per
        // GIOP rules, based on above information, we figure out that we
        // are talking to the legacy ORB and set the ORB Version Accordingly.

        // this special case tell us that it is legacy SUN orb
        // and not a foreign one
        // rtSC is not available for localcase due to which this generic
        // path would fail if relying on rtSC
        //if (giopVersion.equals(GIOPVersion.V1_0) && codeSC && rtSC)

	if (giopVersion.equals(GIOPVersion.V1_0) && codeSC) {
	    orb.setORBVersion(ORBVersionImpl.OLD) ;
	    // System.out.println("setting legacy ORB version");
	} else {
	    // XXX this is redundant
	    orb.setORBVersion(ORBVersionFactory.getORBVersion());
	}

	// try to get the ORBVersion sent as part of the ServiceContext
	// if any
        try {
            sc = ctxts.get( ORBVersionServiceContext.SERVICE_CONTEXT_ID ) ;

            ORBVersionServiceContext ovsc =
               (ORBVersionServiceContext) sc;

            ORBVersion version = ovsc.getVersion();
	    orb.setORBVersion( version ) ;
        } catch (NoSuchServiceContext exc) {
            // ignore: this type not present
        }
    }

    public ServerResponse dispatch(ServerRequest request) {
	if (orb.subcontractDebugFlag)
	    dprint( "Entering dispatch method" ) ;

	byte[] key = request.getObjectKey();
	Object servant = getServant(key);
	String operation = request.getOperationName();
	ServerResponse response = null;

	int sId = Utility.bytesToInt(key, SERVERID_OFFSET);

	if (sId != orb.getTransientServerId())
	    throw new OBJECT_NOT_EXIST(MinorCodes.BAD_SERVER_ID,
				       CompletionStatus.COMPLETED_NO);

	consumeServiceContexts(request);
	
	if (orb.subcontractDebugFlag)
	    dprint( "Dispatching to servant" ) ;

	try {
	    if (SpecialMethod.isSpecialMethod(operation)) {
		if (orb.subcontractDebugFlag)
		    dprint( "Handling special method" ) ;

		response = SpecialMethod.getSpecialMethod(
							  operation).invoke(servant, request);
	    } else if (servant instanceof InvokeHandler) {
		if (orb.subcontractDebugFlag)
		    dprint( "Handling invoke handler type servant" ) ;

		response = (ServerResponse)((InvokeHandler)servant)._invoke(
									    operation, (org.omg.CORBA.portable.InputStream) request,
									    request);
	    } else {
		if (orb.subcontractDebugFlag)
		    dprint( "Handling DSI type servant" ) ;

		// Invoke on servant using DSI
		ServerRequestImpl sreq = new ServerRequestImpl(request, orb);
		DynamicImplementation dServant = (DynamicImplementation) servant;
		OutputStream os;
		InputStream is = (InputStream) request;
		
		dServant.invoke(sreq);

		// Check if ServerRequestImpl.result() has been called
		Any excany = sreq.checkResultCalled();
    
		if ( excany == null ) { // normal return
		    if (orb.subcontractDebugFlag)
			dprint( "DSI type servant: normal response" ) ;

		    response = request.createResponse(null);

		    // Marshal out/inout/return parameters into the ReplyMessage
		    os = (OutputStream) response;
		    sreq.marshalReplyParams(os);
		} else { 
		    if (orb.subcontractDebugFlag)
			dprint( "DSI type servant: error response" ) ;

		    // Check if the servant set a SystemException or 
		    // UserException
		    String repId = excany.type().id();
		    if (Utility.isSystemException(repId)) {
			if (orb.subcontractDebugFlag)
			    dprint( "DSI type servant: system exception" ) ;

			// Get the exception object from the Any
			InputStream in = excany.create_input_stream();
			SystemException ex = Utility.readSystemException(in);

			// Marshal the exception back
			response = request.createSystemExceptionResponse(ex, null);
		    } else {
			if (orb.subcontractDebugFlag)
			    dprint( "DSI type servant: user exception" ) ;

			response = request.createUserExceptionResponse(null);
			os = (OutputStream) response;
			excany.write_value(os);
		    }
		}
	    }
	} catch (UnknownException ex) {
	    if (orb.subcontractDebugFlag)
		dprint( "After dispatch: Unknown exception " + ex ) ;

	    ServiceContexts contexts = new ServiceContexts( orb ) ;
	    UEInfoServiceContext usc = new UEInfoServiceContext( 
		ex.originalEx ) ;

	    try {
		contexts.put( usc ) ;
	    } catch (DuplicateServiceContext dsc) {
		// Can't happen, since we are adding usc to an empty contexts object
	    }

	    SystemException sys = new UNKNOWN( 0, 
		CompletionStatus.COMPLETED_MAYBE);
	    response = request.createSystemExceptionResponse(sys,contexts);
	} catch (ThreadDeath ex) {
	    if (orb.subcontractDebugFlag)
		dprint( "After dispatch: thread death" ) ;

	    throw ex;
	} catch (Throwable ex) {
	    if (orb.subcontractDebugFlag)
		dprint( "After dispatch: other exception " + ex ) ;

	    SystemException sex;
	    
	    if (ex instanceof SystemException) {
		if (orb.subcontractDebugFlag)
		    dprint( 
			"After dispatch: other exception is system exception" ) ;

		sex = (SystemException) ex;
	    } else {
		if (orb.subcontractDebugFlag)
		    dprint( 
			"After dispatch: other exception is user exception" ) ;

		sex = new UNKNOWN(
		    "Unknown Application exception on server: " +
		    ex.getClass().getName(), MinorCodes.RUNTIMEEXCEPTION,
			CompletionStatus.COMPLETED_NO);
	    }
	    
	    response = request.createSystemExceptionResponse(sex,null);
	}

	return response;
    }

    public void destroyObjref(Object objref) {
        // Get the delegate, then ior, then transientKey, then delete servant
        ObjectImpl oi = (ObjectImpl)objref;
        ClientSubcontract del = (ClientSubcontract)oi._get_delegate();
        IOR ior = del.marshal();
        byte[] objKey = ior.getProfile().getObjectKey();
        int len = Utility.bytesToInt(objKey, USERKEYLEN_OFFSET);
        byte[] transientKey = new byte[len];
        for ( int i=0; i<len; i++ )
            transientKey[i] = objKey[i+USERKEY_OFFSET];
        servants.deleteServant(transientKey);
	del.unexport(); 
    }

    public Object createObjref(IOR ior) {
	throw new INTERNAL( 
	    com.sun.corba.se.internal.util.MinorCodes.WRONG_CLIENTSC, 
	    CompletionStatus.COMPLETED_MAYBE);
    }

    public Object createObjref(byte[] key, Object servant) {
	// First, make sure this is an ObjectImpl.

	ObjectImpl objectImpl = (ObjectImpl) servant;

	// Store it and get a userkey allocated by the transient
	// object manager.
	key = servants.storeServant(objectImpl, null);

	// Make a system key out of it.
	byte[] orbKey = constructSystemKey(key);

	// Find out the repository ID for this servant.
	String id = getId(objectImpl);

	// Find out the port number to put in the IOR.
	EndPoint endpoint = orb.getServerGIOP().getDefaultEndpoint();
	if (endpoint == null) {
	    orb.getServerGIOP().initEndpoints();
	    endpoint = orb.getServerGIOP().getDefaultEndpoint();
	}

	int port = endpoint.getPort();
	String host = endpoint.getHostName();

	// Construct the IOR.
	IOR ior = new IOR(orb, id, host, port, orbKey, servant);

	// Create the delegate and set it in the tie.
	ClientDelegate delegate = new ClientDelegate(orb, ior, this.scid,
						     servant );
	objectImpl._set_delegate(delegate);

	return objectImpl;
    }

    protected byte[] constructSystemKey(byte[] key) {
	byte[] orbKey = new byte[USERKEY_OFFSET+key.length];
	Utility.intToBytes(JAVAMAGIC, orbKey, 0);
	Utility.intToBytes(scid, orbKey, 4);
	Utility.intToBytes(orb.getTransientServerId(), orbKey,
			   SERVERID_OFFSET);
	Utility.intToBytes(key.length, orbKey, USERKEYLEN_OFFSET);
	for(int i = 0; i < key.length; i++)
	    orbKey[i+USERKEY_OFFSET] = key[i];
	return orbKey;
    }

    public static String getId(ObjectImpl theTie) {
	return theTie._ids()[0];
    }

}
