/*
 * @(#)WrapperInputStream.java	1.2 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.corba;

import org.omg.CORBA.*;
import org.omg.CORBA.TypeCodePackage.*;

import org.omg.CORBA_2_3.portable.InputStream;
import org.omg.CORBA_2_3.portable.OutputStream;

import com.sun.corba.se.internal.core.*;
import com.sun.corba.se.internal.iiop.CDRInputStream;
import com.sun.corba.se.internal.iiop.CDROutputStream;

import java.util.Hashtable;
import java.io.IOException;

class WrapperInputStream extends org.omg.CORBA_2_3.portable.InputStream implements TypeCodeReader
{
    private CDRInputStream stream;
    private Hashtable typeMap = null;
    private InputStream enclosure = null;

    public WrapperInputStream(CDRInputStream s) {
        super();
        stream = s;
    }

    public int read() throws IOException { return stream.read(); }
    public int read(byte b[]) throws IOException { return stream.read(b); }
    public int read(byte b[], int off, int len) throws IOException {
	return stream.read(b, off, len);
    }
    public long skip(long n) throws IOException { return stream.skip(n); }
    public int available() throws IOException { return stream.available(); }
    public void close() throws IOException { stream.close(); }
    public void mark(int readlimit) { stream.mark(readlimit); }
    public void reset() throws IOException { stream.reset(); }
    public boolean markSupported() { return stream.markSupported(); }
    public void consumeEndian() { stream.consumeEndian(); }
    public int  mark() { return stream.mark(); }
    public void reset(int mark) { stream.reset(mark); }

    public boolean read_boolean() { return stream.read_boolean(); }
    public char read_char() { return stream.read_char(); }
    public char read_wchar() { return stream.read_wchar(); }
    public byte read_octet() { return stream.read_octet(); }
    public short read_short() { return stream.read_short(); }
    public short read_ushort() { return stream.read_ushort(); }
    public int read_long() { return stream.read_long(); }
    public int read_ulong() { return stream.read_ulong(); }
    public long read_longlong() { return stream.read_longlong(); }
    public long read_ulonglong() { return stream.read_ulonglong(); }
    public float read_float() { return stream.read_float(); }
    public double read_double() { return stream.read_double(); }
    public String read_string() { return stream.read_string(); }
    public String read_wstring() { return stream.read_wstring(); }

    public void read_boolean_array(boolean[] value, int offset, int length) {
	stream.read_boolean_array(value, offset, length);
    }
    public void read_char_array(char[] value, int offset, int length) {
	stream.read_char_array(value, offset, length);
    }
    public void read_wchar_array(char[] value, int offset, int length) {
	stream.read_wchar_array(value, offset, length);
    }
    public void read_octet_array(byte[] value, int offset, int length) {
	stream.read_octet_array(value, offset, length);
    }
    public void read_short_array(short[] value, int offset, int length) {
	stream.read_short_array(value, offset, length);
    }
    public void read_ushort_array(short[] value, int offset, int length) {
	stream.read_ushort_array(value, offset, length);
    }
    public void read_long_array(int[] value, int offset, int length) {
	stream.read_long_array(value, offset, length);
    }
    public void read_ulong_array(int[] value, int offset, int length) {
	stream.read_ulong_array(value, offset, length);
    }
    public void read_longlong_array(long[] value, int offset, int length) {
	stream.read_longlong_array(value, offset, length);
    }
    public void read_ulonglong_array(long[] value, int offset, int length) {
	stream.read_ulonglong_array(value, offset, length);
    }
    public void read_float_array(float[] value, int offset, int length) {
	stream.read_float_array(value, offset, length);
    }
    public void read_double_array(double[] value, int offset, int length) {
	stream.read_double_array(value, offset, length);
    }

    public org.omg.CORBA.Object read_Object() { return stream.read_Object(); }
    public java.io.Serializable read_value() {return stream.read_value();}
    public TypeCode read_TypeCode() { return stream.read_TypeCode(); }
    public Any read_any() { return stream.read_any(); }
    public Principal read_Principal() { return stream.read_Principal(); }
    public java.math.BigDecimal read_fixed() { return stream.read_fixed(); }
    public org.omg.CORBA.Context read_Context() { return stream.read_Context(); }

    public org.omg.CORBA.ORB orb() { return stream.orb(); }

    public void addTypeAtPosition(TypeCodeImpl typeCode, int position) {
        if (typeMap == null)
            typeMap = new Hashtable(16);
        typeMap.put(new Integer(position), typeCode);
    }

    public TypeCodeImpl getTypeAtPosition(int position) {
        return (TypeCodeImpl)typeMap.get(new Integer(position));
    }

    public void setEnclosingInputStream(InputStream enclosure) {
        this.enclosure = enclosure;
    }

    public TypeCodeReader getTopLevelStream() {
        if (enclosure == null)
            return this;
        if (enclosure instanceof TypeCodeReader)
            return ((TypeCodeReader)enclosure).getTopLevelStream();
        // Recursion ends at top level TypeCodeReader.
        //return enclosure;
        return this;
    }

    public int getTopLevelPosition() {
        if (enclosure == null)
            return mark();
        if (enclosure instanceof TypeCodeReader) {
            return ((TypeCodeReader)enclosure).getTopLevelPosition()
		- 4 + mark();
        }
        // Recursion ends at top level TypeCodeReader.
        //return enclosure.getSize();
        return mark();
    }

    public TypeCodeInputStream read_encapsulation(org.omg.CORBA.ORB _orb) {
	// _REVISIT_ Would be nice if we didn't have to copy the buffer!
	TypeCodeInputStream _encap;
	// pop off the encapsulation length
	int encapLength = this.read_long();

	// read off part of the buffer corresponding to the encapsulation
	byte[] encapBuffer = new byte[encapLength];
	this.read_octet_array(encapBuffer, 0, encapBuffer.length);

	// create an encapsulation using the marshal buffer
	// _REVISIT_ We have no information about the endianess!
	_encap = new TypeCodeInputStream(_orb, encapBuffer, encapBuffer.length);
	_encap.setEnclosingInputStream(this);

	// read and set the endianness of the encapsulation
	_encap.consumeEndian();
	return _encap;
    }
}
