/*
 * @(#)CodeSetComponentInfo.java	1.14 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.core;

import org.omg.CORBA.INV_OBJREF;
import org.omg.CORBA.CompletionStatus;

import com.sun.corba.se.internal.util.MinorCodes;


public final class CodeSetComponentInfo {

    // REVISIT:
    //		1. Check if these numbers are right (ask Ken or Simon)
    //		2. Should I add these numbers to the conversion code set?
    //                                    - Anil

    public static final int ASCII = 0x00010020;
    public static final int UNICODE = 0x00010100;
    
    public static final class CodeSetComponent {
	private int nativeCodeSet;
	private int[] conversionCodeSets;
	
	public CodeSetComponent(int nativeCodeSet, int[] conversionCodeSets) {
	    this.nativeCodeSet = nativeCodeSet;
	    if (conversionCodeSets == null)
		this.conversionCodeSets = new int[0];
	    else 
		this.conversionCodeSets = conversionCodeSets;
	}

	public void read(MarshalInputStream in) {
	    nativeCodeSet = in.read_ulong();
	    int len = in.read_long();
	    conversionCodeSets = new int[len];
	    in.read_ulong_array(conversionCodeSets, 0, len);

	}
	
	public void write(MarshalOutputStream out) {
	    out.write_ulong(nativeCodeSet);
	    out.write_long(conversionCodeSets.length);
	    out.write_ulong_array(conversionCodeSets, 0, conversionCodeSets.length);
	}

	boolean validateWith(int myCodeSet) 
	{
	    boolean canConvertToMine = false;
	    
	    if (nativeCodeSet != myCodeSet) {
		for(int i = 0; i < conversionCodeSets.length; i++)
		    if (conversionCodeSets[i] == myCodeSet) {
			canConvertToMine = true;
			break;
		    }
	    }
	    else
		canConvertToMine = true;
	    
	    return canConvertToMine;
	}
    }

    private CodeSetComponent forCharData;
    private CodeSetComponent forWCharData;

    public CodeSetComponentInfo() {
	this.forCharData = new CodeSetComponent(ASCII, null);
	this.forWCharData = new CodeSetComponent(UNICODE, null);
    }

    public void read(MarshalInputStream in) {
	forCharData.read(in);
	forWCharData.read(in);

	if (forCharData.validateWith(ASCII) == false || 
	    forWCharData.validateWith(UNICODE) == false)
	    throw new INV_OBJREF(MinorCodes.BAD_CODE_SET, CompletionStatus.COMPLETED_NO);
    }
    

    public void write(MarshalOutputStream out) {
	forCharData.write(out);
	forWCharData.write(out);
    }


    public static final class CodeSetContext {
	private int char_data;
	private int wchar_data;
	
	public CodeSetContext() {
	    char_data = ASCII;
	    wchar_data = UNICODE;
	}
	

	public void read(MarshalInputStream in) {
	    char_data = in.read_ulong();
	    wchar_data = in.read_ulong();
	}
	
	public void write(MarshalOutputStream out) {
	    out.write_ulong(char_data);
	    out.write_ulong(wchar_data);
	}
	
	public boolean validateWith(CodeSetContext cs) {
	    if (cs.char_data != this.char_data ||
		cs.wchar_data != this.wchar_data)
		return false;
	    return true;
	}
    }

    public static final CodeSetContext myCodeSet = new CodeSetContext();
}
