/*
 * @(#)IOR.java	1.65 00/02/11
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.core;

import java.io.StringWriter;
import java.io.IOException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.INV_OBJREF;
import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.orbutil.HexOutputStream;
import org.omg.CORBA.portable.*;
import com.sun.corba.se.internal.util.JDKBridge;
import javax.rmi.CORBA.Util;

public class IOR
{
    public static final IOR NULL = new IOR();
    
    
    protected String typeId;
    protected int[] profileTags;
    protected byte[][] profileData;

    protected ORB factory;
    protected String host;
    protected int port;
    protected byte[] key;
    protected Object servant;
    private EndPoint ep;

    // Cached lookups
    protected Profile iop;
    protected String codebase;
    protected boolean cachedCodebase = false;   
    private static Profile.TaggedComponent localCodeBaseTC = null;

    public static final int InternetIOPTag = 0;
    
    /*
     * Turns debugging on or off.
     * @param enable Toggle for debug state.
     public static void setDebug(boolean enable) {
     debug = enable;
     }
    */
    
    /*
     * Only for the NULL instance.
     */
    private IOR() {
	this.factory = null;
	this.typeId = "";
	profileTags = new int[0];
    }

    public IOR( IOR ior ) {
	this( ior.factory, ior.typeId, ior.host, ior.port, 
	      ior.key, ior.servant ) ;
    }

    public IOR(ORB factory) {
	this.factory = factory;
    }
   
    public IOR(ORB factory, String typeId, String host,
	       int port, byte[] key) throws SystemException {
	this(factory,typeId,host,port,key,null);
    }
 
    public IOR(ORB factory, String typeId, String host,
	       int port, byte[] key, Object servant) throws SystemException {
    	this(factory);
        this.typeId = typeId;

	this.host = host ;
	this.port = port ;
	this.key = key ;
	this.servant = servant ;
	
    	// Create the TAG_CODE_SETS component...
    	
    	Profile.TaggedComponent codeSetTC = createCodeSetComponent();

        // Create the TAG_JAVA_CODEBASE component if we can...
        
    	Profile.TaggedComponent codeBaseTC = createCodebaseComponent(servant);
        
        // Set the profile...
        
        setProfile(host,port,key,codeSetTC,codeBaseTC);
    }

    protected void setProfile(String host,
			      int port,
			      byte[] key,
			      Profile.TaggedComponent codeSetTC,
                              Profile.TaggedComponent codeBaseTC) {
                                
        // Create the components array...
        
    	Profile.TaggedComponent[] components;
    	if (codeBaseTC == null) {
    	    components = new Profile.TaggedComponent[] { codeSetTC };
    	} else {
    	    components = new Profile.TaggedComponent[] { codeSetTC,codeBaseTC };
        }
        
        // Create the profile...
        
        Profile p = new Profile(factory, host, port, key, components);

        putProfile(p);
    }
                              
    protected Profile.TaggedComponent createCodeSetComponent () {
    	MarshalOutputStream out = factory.newOutputStream();
    	CodeSetComponentInfo cs = factory.getCodeSetComponentInfo();
    	out.putEndian();
    	cs.write(out);
    	byte[] data = out.toByteArray();
    	Profile.TaggedComponent codeSetTC 
    	    = new Profile.TaggedComponent(Profile.TaggedComponent.TAG_CODE_SETS,
    					  data);
        return codeSetTC;
    }
    
    protected Profile.TaggedComponent createCodebaseComponent (Object servant) {
        if (servant != null) {
            codebase = Util.getCodebase(servant.getClass());
            cachedCodebase = true;
            return createCodebaseComponent(codebase);
        } else {
            return null;
        }
    }
  
    protected Profile.TaggedComponent createCodebaseComponent (String codebase) {
    	Profile.TaggedComponent codeBaseTC = null;
        try {
    	    MarshalOutputStream out = factory.newOutputStream();
    	    out.putEndian();
    	    out.write_string(codebase);
            byte[] data = out.toByteArray();
            if (data != null && data.length > 0) {
                codeBaseTC = new Profile.TaggedComponent(
							 Profile.TaggedComponent.TAG_JAVA_CODEBASE,data);
            }
        } catch (Exception e) {}
        
        return codeBaseTC;
    }

    public void read(InputStream s)
    {
        int len;

        typeId = s.read_string();
        len = s.read_long();
        profileTags = new int[len];
        profileData = new byte[len][];
        for (int i=0; i < len; i++) {
            profileTags[i] = s.read_long();
            profileData[i] = new byte[s.read_long()];
            s.read_octet_array(profileData[i], 0, profileData[i].length);
        }
        cachedCodebase = false;
    
        if (!is_nil()) {
            
            // If there is no codebase in this IOR and there IS a
            // java.rmi.server.codebase property set, we need to
            // update the IOR with the local codebase.  This is
            // really only required when the IOR is written, but
            // doing it here means that no synchronization is
            // required to keep IORs thread-safe.
            
            // First, do we have a local codebase?  If not, we're
            // done...
            
            String localCodebase = JDKBridge.getLocalCodebase();
            if (localCodebase != null) {
            
                // We have one. Does this IOR have a codebase
                // already?
      
    	        // _REVISIT_ The following code could be much more
    	        // efficient.  Rather than calling getCodebase(),
    	        // which requires creating the Profile at this point,
    	        // the profileData should be directly interrogated.
    	        // Then, if the codebase does need to be added, the
    	        // cached component (localCodeBaseTC) should be directly
    	        // appended to the profileData array.  I have not done
    	        // this here because it is tricky, and I'm fast running
    	        // out of time!
                
                if (getCodebase() == null) {

                    // Nope, so we must add it...
                    
                    if (localCodeBaseTC == null) {
                        localCodeBaseTC = createCodebaseComponent(localCodebase);
    	            }
    	            
    	            Profile.TaggedComponent codeSetTC = null;
                    Profile p = getProfile();
                    Profile.TaggedComponent[] components = p.getTaggedComponents();
		    for (int i=0; i < components.length; i++) {
                        if (components[i].tag == Profile.TaggedComponent.TAG_CODE_SETS) {
                            codeSetTC = components[i];
			}
		    }
                    setProfile(p.getHost(),p.getPort(),p.getObjectKey(),codeSetTC,localCodeBaseTC);
                    codebase = localCodebase;
                    cachedCodebase = true;
                }
            }
        }
    }

    public void write(MarshalOutputStream s) {	 
	s.write_string(typeId);
	s.write_long(profileTags.length);
	for (int i=0; i < profileTags.length; i++) {
	    s.write_long(profileTags[i]);
	    s.write_long(profileData[i].length);
	    s.write_octet_array(profileData[i], 0 , profileData[i].length);
	}
    }
    
    public String getCodebase() {
        if (!cachedCodebase && profileTags.length > 0) {
        
            // Extract codebase from profile if present...

            Profile.TaggedComponent[] components = getProfile().getTaggedComponents();
            
	    for (int i=0; i < components.length; i++) {
                if (components[i].tag == Profile.TaggedComponent.TAG_JAVA_CODEBASE) {
                    try {
			MarshalInputStream in = components[i].getComponentData(factory);
			codebase = in.read_string();
                    } catch (Exception e) {}
		}
	    }
            cachedCodebase = true;
        }

        return codebase;
    }

    public synchronized Profile getProfile()
    {
	if (iop != null) {
	    return iop;
	}

	for (int i=0; i < profileTags.length; i++) {
	    if (profileTags[i] == InternetIOPTag) {
		iop = new Profile(factory, profileData[i]);
		return iop;
	    }
	}

        // if we come to this point then no interesting Profile
        // is present.  Therefore, throw an INV_OBJREF exception.
        throw new INV_OBJREF( MinorCodes.NO_PROFILE_PRESENT,
			      CompletionStatus.COMPLETED_NO );
    }

    public void putProfile(Profile x)
    {
	int[] tags;
	byte[][] data;
	int len = 0;
	int i;

	if (profileTags != null) {
	    len = profileTags.length;
        }
        
	for (i=0; i < len; i++) {
            if (profileTags[i] == InternetIOPTag) {
		profileData[i] = x.getEncapsulation();
		iop = x;
		return;
	    }
	}

	tags = new int[len+1];
	data = new byte[len+1][];
	for (i=0; i < len; i++) {
	    tags[i] = profileTags[i];
	    data[i] = profileData[i];
	}
	tags[i] = InternetIOPTag;
	data[i] = x.getEncapsulation();

	profileTags = tags;
	profileData = data;
	iop = x;
    }

    public String getTypeId()
    {
	return typeId;
    }

    public String stringify()
    {
        return stringify(factory);
    }

    public String stringify(ORB orb)
    {
	StringWriter bs;

	MarshalOutputStream s = orb.newOutputStream();
	s.putEndian();
	write(s);
	bs = new StringWriter();
	try {
	    s.writeTo(new HexOutputStream(bs));
	} catch (IOException ex) {
	    throw new INTERNAL( MinorCodes.STRINGIFY_WRITE_ERROR,
				CompletionStatus.COMPLETED_NO );
	}

	return "IOR:" + bs;
    }

    public boolean is_nil()
    {
        //
        // The check for typeId length of 0 below is commented out
        // as a workaround for a bug in ORBs which send a
        // null objref with a non-empty typeId string.
        //
	return ((profileTags.length == 0) /* && (typeId.length() == 0) */);
    }

    public boolean isEquivalent(IOR ior)
    {
	return getProfile().isEquivalent(ior.getProfile());
    }

    public boolean isLocal()
    {
	if (factory == null)
	    return false ;

	Profile iop = getProfile() ;
	String host = iop.getHost() ;
	byte[] key = iop.getObjectKey() ;

	return factory.isLocalHost( host ) && factory.isLocalServerId( key ) ;
    }

    /*
     * Create a default client EndPoint. For now, reach down into
     * the iiop package for the default implementation.
     */
    public synchronized EndPoint getEndpoint() {
        if (ep == null)
            ep = factory.newEndPoint(this);
        return ep;
    }
}
