package com.sun.corba.se.internal.core ;

import com.sun.corba.se.internal.orbutil.ORBConstants;
import com.sun.corba.se.internal.core.ORBVersion ;
import com.sun.corba.se.internal.core.ORBVersionImpl ;
import org.omg.CORBA.portable.InputStream ;
import org.omg.CORBA.INTERNAL ;

public class ORBVersionFactory {
    private ORBVersionFactory() {} ;

    public static ORBVersion getORBVersion()
    {
	return ORBVersionImpl.NEW ;
    }

    public static ORBVersion create( byte[] key, int versionOffset )
    {
	byte value = key[versionOffset] ;
	return byteToVersion( value ) ;
    }

    public static ORBVersion create( InputStream is ) 
    {
	byte value = is.read_octet() ;
	return byteToVersion( value ) ;
    }

    public static ORBVersion createFromMagic(int magic)
    {
        if (magic == ORBConstants.JDK13Magic)
            return ORBVersionImpl.OLD;
        else
            return ORBVersionFactory.getORBVersion();
    }

    private static ORBVersion byteToVersion( byte value ) 
    {
	/* Throwing an exception here would cause Merlin to be incompatible
	* with future versions of the ORB, to the point that Merlin could
	* not even unmarshal objrefs from a newer version that uses 
	* extended versioning.  Therefore, we will simply treat all 
	* unknown versions as NEWER.
	if (value < 0)
	    throw new INTERNAL() ;
	*/

	switch (value) {
	    case ORBVersion.FOREIGN : return ORBVersionImpl.FOREIGN ;
	    case ORBVersion.OLD : return ORBVersionImpl.OLD ;
	    case ORBVersion.NEW : return ORBVersionImpl.NEW ;
	    case ORBVersion.NEWER : return ORBVersionImpl.NEWER ;
	    default : return ORBVersionImpl.NEWER ;
	}
    }
}
