/*
 * @(#)Profile.java	1.48 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.core;

import org.omg.CORBA.SystemException;

public class Profile
{
    public static class TaggedComponent {
	
	public static final int TAG_CODE_SETS = 1;
	public static final int TAG_JAVA_CODEBASE = 25;
	
	public int tag;
	public byte[] component_data;

	public TaggedComponent() 
	{
	    tag = -1;
	    component_data = null;
	}

	public TaggedComponent(int tag, byte[] component_data) 
	{
	    this.tag = tag;
	    this.component_data = component_data;
	}
	
	public void read(MarshalInputStream in) 
	{
	    tag = in.read_ulong();
	    int len = in.read_long();
	    component_data = new byte[len];
	    in.read_octet_array(component_data, 0, len);
	}
	
	public void write(MarshalOutputStream out)
	{
	    out.write_ulong(tag);
	    out.write_long(component_data.length);
	    out.write_octet_array(component_data, 0, component_data.length);
	}

	public MarshalInputStream getComponentData(ORB factory) {
	    
	    MarshalInputStream in = factory.newInputStream(component_data, 
							   component_data.length);
	    in.consumeEndian();
	    return in;
	}
    }
    
	
    protected String host;
    protected int port;
    protected byte major;
    protected byte minor;
    protected byte[] objectKey;
    private byte[] data;

    protected TaggedComponent[] components;
    
    private ORB factory;

    public Profile(ORB factory, byte[] data) 
	throws SystemException 
    {
	this.factory = factory;

	MarshalInputStream s = factory.newInputStream(data, data.length);
	s.consumeEndian();
	major = s.read_octet();
	minor = s.read_octet();

	host = s.read_string();
	port = s.read_short() & 0xFFFF;
	objectKey = new byte[s.read_long()];
	s.read_octet_array(objectKey, 0, objectKey.length);

	this.data = data;

        // IIOP 1.1: Unmarshal a tagged component sequence
	// here:
	
	if (minor > 0) {
	    int nComponents = s.read_long();
	    components = new TaggedComponent[nComponents];
	    for(int i = 0; i < nComponents; i++) {
		components[i] = new TaggedComponent();
		components[i].read(s);
	    }

	    for(int j = 0; j < nComponents; j++) {
		if (components[j].tag == TaggedComponent.TAG_CODE_SETS) {
		    MarshalInputStream in = components[j].getComponentData(factory);
		    CodeSetComponentInfo cs = factory.getCodeSetComponentInfo();
		    cs.read(in);
		}
	    }
	}

    }

    public Profile(ORB factory, String host, int port, byte[] key, TaggedComponent[] components) {
	this.factory = factory;
	this.host = host;
	this.port = port;
	objectKey = key;
	major = 1;
	minor = 1;
	if (components == null)
	    this.components = new TaggedComponent[0];
	else
	    this.components = components;
    }

    byte[] getEncapsulation() throws SystemException {

	if (data != null)
	    return data;

	MarshalOutputStream s = factory.newOutputStream();
	s.putEndian();
	s.write_octet(major);
	s.write_octet(minor);
	s.write_string(host);
	s.write_short((short)port);
	s.write_long(objectKey.length);
	s.write_octet_array(objectKey, 0, objectKey.length);

	// In IIOP 1.1: Marshal a sequence of
	// tagged components here.

	if (minor > 0) {
	    s.write_long(components.length);
	    for(int i = 0; i < components.length; i++)
		components[i].write(s);
	}
	
	data = s.toByteArray();
	return data;
    }

    public TaggedComponent[] getTaggedComponents() {
	return components;
    }

    public String getHost() {
	return host;
    }

    public int getPort() {
	return port;
    }

    public byte[] getObjectKey() {
	return objectKey;
    }

    public int getTaggedComponentsLength() {
	if (components != null) {
            return components.length;
	} else  {
	    return 0;
	}

    }

    public ORB getFactory() {
	return factory;
    }

    boolean isEquivalent(Profile p) {
	if (getPort() != p.getPort() ||
	    !getHost().equalsIgnoreCase(p.getHost()) ||
	    getObjectKey().length != p.getObjectKey().length)
	    return false;

	byte[] key = p.getObjectKey();
	for (int i=0; i < objectKey.length; i++)
	    if (objectKey[i] != key[i])
		return false;

	return true;
    }

}
