/*
 * @(#)ServerSubcontract.java	1.23 01/02/09
 *
 * Copyright 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.core;

/**
 * Server subcontract adds behavior on the server-side -- specifically
 * on the dispatch path. A single server subcontract instance serves
 * many server objects.
 */
public interface ServerSubcontract {

    /**
     * This should be the first four bytes of IORs created by
     * JavaIDL and all derivative ORBs.
     * The JAVAMAGIC is changed from 0xAFabCafe to 0xAFabCaff to handle
     * backward compatibility. This is according to the ORBVersioning 
     * changes.
     */
    public final int JAVAMAGIC = 0xAFabCaff;
    
    /**
     * Create an object reference given a servant and/or user-key.
     *
     * @param key the user key part of the ORB key
     * @param servant the server object (may be null)
     */
    public Object createObjref(byte[] key, Object servant);

    /**
     * Create an object reference given an IOR.
     *
     * @param ior the ior to create an objref for.
     */
    public Object createObjref(IOR ior) throws Exception;

    /**
     * Destroy the object reference.
     */
    public void destroyObjref(Object objref);

    /**
     * Dispatch to the server object and return its
     * response.
     *
     * dispatch() should catch all exceptions and convert
     * them to a ServerResponse.
     *
     */
    public ServerResponse dispatch(ServerRequest request);

    /**
     * Handle a locate request.
     */
    public IOR locate(byte[] key);

    /**
     * Lookup and return the servant corresponding to the object key
     */
    public Object getServant(byte[] key);

    /**
     * Return a boolean indicating whether or not a servant is supported
     */

    public boolean isServantSupported();

    /**
     * Get the parallel client subcontract class
     */
    public Class getClientSubcontractClass();

    /**
     * Set the subcontract ID assigned by the ORB for this
     * class.
     */
    void setId(int id);

    /**
     * Set the ORB instance that created this SC
     */
    void setOrb(ORB orb);
}
