/*
 * @(#)CDRInputStream.java	1.31 01/02/09
 *
 * Copyright 1993-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.iiop;

import java.io.IOException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.Object;
import org.omg.CORBA.Principal;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.Delegate;
import org.omg.CORBA.portable.ValueBase;
import org.omg.CORBA.portable.IndirectionException;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.*;
import org.omg.CORBA.TypeCodePackage.*;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.CustomMarshal;

import java.rmi.Remote;
import java.rmi.StubNotFoundException;
import java.io.Serializable;
import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.StreamCorruptedException;
import java.io.OptionalDataException;
import java.lang.ClassNotFoundException;
import java.net.MalformedURLException;
import org.omg.CORBA.portable.ObjectImpl;
import org.omg.CORBA.MARSHAL;
import javax.rmi.PortableRemoteObject;
import javax.rmi.CORBA.Tie;
import javax.rmi.CORBA.Util;
import com.sun.corba.se.internal.corba.ServerDelegate;

import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.Profile;
import com.sun.corba.se.internal.core.SubcontractRegistry;
import com.sun.corba.se.internal.core.ServerSubcontract;
import com.sun.corba.se.internal.core.ClientSubcontract;
import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.util.Utility;
import com.sun.corba.se.internal.util.RepositoryId;
import com.sun.corba.se.internal.orbutil.CacheTable;
import com.sun.corba.se.internal.orbutil.ORBUtility;
import com.sun.corba.se.internal.orbutil.RepositoryIdWrapper;
import com.sun.corba.se.internal.orbutil.RepositoryId_1_3;
import com.sun.corba.se.internal.orbutil.ValueHandlerImpl_1_3;

import com.sun.corba.se.internal.corba.PrincipalImpl;
import com.sun.corba.se.internal.corba.TypeCodeImpl;

import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Principal;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.BoxedValueHelper;
import org.omg.CORBA.portable.ValueFactory;
import org.omg.CORBA.portable.CustomValue;
import org.omg.CORBA.portable.StreamableValue;
import com.sun.org.omg.CORBA.portable.ValueHelper;

import java.io.IOException;
import java.util.Stack;
import java.net.URL;
import java.net.MalformedURLException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.sun.org.omg.SendingContext.CodeBase;

public class CDRInputStream
    extends org.omg.CORBA_2_3.portable.InputStream
    implements com.sun.corba.se.internal.core.MarshalInputStream, org.omg.CORBA.DataInputStream
{
    private static final String kReadMethod = "read";
    private static final int maxBlockLength = 0x7fffff00;
 
    protected boolean littleEndian;
    protected int index;
    protected int size, offset;
    protected byte[] buf;
    protected com.sun.corba.se.internal.core.ORB orb;

    // Value cache
    private CacheTable valueCache = null;
    
    // Repository ID cache
    private CacheTable repositoryIdCache = null;

    // codebase cache
    private CacheTable codebaseCache = null;

    // Current Class Stack (repository Ids of current class being read)
    // private Stack currentStack = null;

    // Length of current chunk, or a large positive number if not in a chunk
    private long blockLength = maxBlockLength;

    // Expected type (set by com.sun.corba.se.internal.io.IIOPInputStream) used when
    // no rep. id sent on wire because expected type is real type.
    private Class expectedType = null;

    // Read end flag (value nesting depth)
    private int end_flag = 0;

    // Flag used to determine whether blocksize was zero
    // private int checkForNullBlock = -1;

    // In block flag
    // private boolean inBlock = false;

    // Indicates whether we are inside a value
    // private boolean outerValueDone = true;

    // Int used by read_value(Serializable) that is set by this class
    // before calling ValueFactory.read_value
    private int valueIndirection = 0;

    // Int set by readStringOrIndirection to communicate the actual
    // offset of the string length field back to the caller
    private int stringIndirection = 0;

    // Flag indicating whether we are unmarshalling a chunked value
    private boolean isChunked = false;

    // ValueHandler
    private javax.rmi.CORBA.ValueHandler valueHandler = null;

    public CDRInputStream() {
    }

    public CDRInputStream(org.omg.CORBA.ORB orb, byte[] data, int size) {
	this(orb, data, size, false);
    }

    public CDRInputStream(org.omg.CORBA.ORB orb, byte[] data, int size, boolean littleEndian) {
    	this.orb = (com.sun.corba.se.internal.core.ORB)orb;
    	this.littleEndian = littleEndian;
    	this.size = size;
    	buf = data;
    	offset = 0;
    }

    public CDRInputStream(CDRInputStream is) {
    	this.littleEndian = is.littleEndian;
    	this.index = is.index;
    	this.size = is.size;
    	this.buf = is.buf;
    	this.orb = is.orb;
    	offset = 0;
    }

    // needed by AnyImpl
    public com.sun.corba.se.internal.core.ORB getORB() {
	return orb;
    }
    
    public CDRInputStream dup() {
        return new CDRInputStream(this);
    }

    private final void alignAndCheck(int align, int n) {

	// Since chunks can end at arbitrary points (though not within
	// primitive CDR types, arrays of primitives, strings, or wstrings),
	// we must check here for termination of the current chunk.
	// This also takes care of terminating an open chunk when a
	// value tag is encountered.
	if (blockLength == index) {
	    blockLength = maxBlockLength;
	    start_block();

	} else if (blockLength < index) {
	    // current chunk has overflowed
	    throw new MARSHAL("Chunk overflow at offset "+index,
			      998, CompletionStatus.COMPLETED_NO);
	}

	if (align > 1) {
            int incr = index & (align - 1);
            if (incr != 0) {
                incr = (align - incr);
                index += incr;
            }
        }
    	if (index + n > size) {
            grow(align, n);
    	}

    }

    //
    // This can be overridden....
    //
    protected void grow(int align, int n) {
	throw new MARSHAL(999,
			  CompletionStatus.COMPLETED_NO);

    }

    public void setOffset(int offset) {
        index = offset;
    }  


    //
    // Marshal primitives.
    //

    public final void consumeEndian() {
	littleEndian = read_boolean();
    }

    // No such type in java
    public final double read_longdouble() {
	throw new NO_IMPLEMENT(MinorCodes.SEND_DEFERRED_NOTIMPLEMENTED,
			       CompletionStatus.COMPLETED_MAYBE);
    }

    public final boolean read_boolean()
    {
	boolean result;

	result = (read_octet() != 0)? true: false;
	return result;
    }

    public final char read_char() {
        alignAndCheck(1, 1);

        char x = (char)(buf[index++] & 0xFF);
    	if ( x > '\u00ff' )
    	    throw new MARSHAL(MinorCodes.CHARACTER_OUTOFRANGE,
			      CompletionStatus.COMPLETED_NO);
	return x;
    }

    public final char read_wchar() {

	int b1, b2;

	alignAndCheck(2, 2);

	if (littleEndian) {
	    b2 = buf[index++] & 0x00FF;
	    b1 = buf[index++] & 0x00FF;
	} else {
	    b1 = buf[index++] & 0x00FF;
	    b2 = buf[index++] & 0x00FF;
	}

	return (char)((b1 << 8) + (b2 << 0));
    }

    public final byte read_octet() {
        if (index + 1 >= size)
            alignAndCheck(1, 1);
    	return buf[index++];
    }

    public final short read_short() {
    	int b1, b2;

    	alignAndCheck(2, 2);

    	if (littleEndian) {
    	    b2 = (buf[index++] << 0) & 0x000000FF;
    	    b1 = (buf[index++] << 8) & 0x0000FF00;
    	} else {
    	    b1 = (buf[index++] << 8) & 0x0000FF00;
    	    b2 = (buf[index++] << 0) & 0x000000FF;
    	}

    	return (short)(b1 | b2);
    }

    public final short read_ushort() {
	return read_short();
    }

    public final int read_long() {
    	int b1, b2, b3, b4;

        if ((index & 3) != 0 || index + 4 >= size)
            alignAndCheck(4, 4);

    	if (littleEndian) {
    	    b4 = buf[index++] & 0xFF;
    	    b3 = buf[index++] & 0xFF;
    	    b2 = buf[index++] & 0xFF;
    	    b1 = buf[index++] & 0xFF;
    	} else {
    	    b1 = buf[index++] & 0xFF;
    	    b2 = buf[index++] & 0xFF;
    	    b3 = buf[index++] & 0xFF;
    	    b4 = buf[index++] & 0xFF;
    	}

    	return (b1 << 24) | (b2 << 16) | (b3 << 8) | b4;
    }

    public final int read_ulong() {
	return read_long();
    }

    public final long read_longlong() {
    	long i1, i2;

    	alignAndCheck(8, 8);

    	if (littleEndian) {
    	    i2 = read_long() & 0xFFFFFFFFL;
    	    i1 = (long)read_long() << 32;
    	} else {
    	    i1 = (long)read_long() << 32;
    	    i2 = read_long() & 0xFFFFFFFFL;
    	}

    	return (i1 | i2);
    }

    public final long read_ulonglong() {
	return read_longlong();
    }

    public final float read_float() {
	return Float.intBitsToFloat(read_long());
    }

    public final double read_double() {
	return Double.longBitsToDouble(read_longlong());
    }

    protected final String readStringOrIndirection(boolean allowIndirection) {

    	int len = read_long();

        //
        // Check for indirection
        //
        if (allowIndirection) {
            if (len == 0xffffffff)
		return null;
	    else
		stringIndirection = get_offset() - 4;
	}	

    	//
    	// Workaround for ORBs which send string lengths of
    	// zero to mean empty string.
    	//
    	if (len == 0)
    	    return "";

        len--;
        char[] c = new char[len];

    	int n = 0;
    	while (n < len) {
    	    int avail;
    	    int bytes;
    	    int wanted;

    	    avail = size - index;
            if (avail <= 0) {
                grow(1, 1);
                avail = size - index;
            }
    	    wanted = len - n;
    	    bytes = (wanted < avail) ? wanted : avail;
            for (int i=0; i<bytes; i++) {
        	c[n+i] = (char) (buf[index+i] & 0xFF);
            }
    	    index += bytes;
    	    n += bytes;
    	}

        //
        // Skip past terminating null byte
        //
        if (index + 1 >= size)
            alignAndCheck(1, 1);
    	index++;

    	return new String(c);
    }

    public final String read_string() {
        return readStringOrIndirection(false);
    }

    // This is used ONLY while unmarshalling user exceptions, 
    // to peek the repositoryId. Called from ReplyMessage.unmarshal();
    // NOTE : We assume that the entire string can be obtained from
    // the current message buffer. This may have to be changed for
    // IIOP 1.1.
    public final String peek_string() {
	//        byte[] b;
	//        String result;
        int length;

    	// First peek a long for string length
    	int b1, b2, b3, b4;
    	// Its ok to align: the next read operation MUST be a read_long
    	// so the alignment here will be done anyway.
    	alignAndCheck(4, 4);
    	if (littleEndian) {
    	    b4 = (buf[index] << 0)  & 0x000000FF;
    	    b3 = (buf[index+1] << 8)  & 0x0000FF00;
    	    b2 = (buf[index+2] << 16) & 0x00FF0000;
    	    b1 = (buf[index+3] << 24) & 0xFF000000;
    	} else {
    	    b1 = (buf[index] << 24) & 0xFF000000;
    	    b2 = (buf[index+1] << 16) & 0x00FF0000;
    	    b3 = (buf[index+2] << 8)  & 0x0000FF00;
    	    b4 = (buf[index+3] << 0)  & 0x000000FF;
    	}
        length = (b1 | b2 | b3 | b4);

        //
        // Workaround for ORBs which send string lengths of
        // zero to mean empty string.
        //
        if (length == 0)
            return "";

	//        b = new byte[length];
    	int index4 = index + 4;  // the string starts after long

    	// peek the string
    	alignAndCheck(1, 1);
    	int avail = size - index4;
    	if ( length > avail ) {
    	    throw new INTERNAL(MinorCodes.PEEKSTRING_FAILED, 
			       CompletionStatus.COMPLETED_MAYBE);
    	}
	//    	System.arraycopy(buf, index4, b, 0, length);

	//            //
	//            // Strip out the embedded string null termination
	//            // byte.
	//            //
	//            result = new String (b, 0, b.length - 1);
	//
	//            // Do range checking
	//            for ( int i=0; i<result.length(); i++ ) {
	//                if ( result.charAt(i) > '\u00ff' )
	//                    throw new MARSHAL(MinorCodes.CHARACTER_OUTOFRANGE,
	//                                      CompletionStatus.COMPLETED_MAYBE);
        length--;
        char[] c = new char[length];
    	for ( int i=0; i<length; i++ ) {
    	    c[i] = (char) (buf[index4 + i] & 0xFF);
    	}

    	return new String(c);
    }

    public final String read_wstring() {

    	int len = read_long();

    	//
    	// Workaround for ORBs which send string lengths of
    	// zero to mean empty string.
    	//
    	if (len == 0)
    	    return "";

        if (index + (len * 2) >= size)
            alignAndCheck(2, len * 2);

        len--;
        char[] c = new char[len];

	if (littleEndian) {
	    for (int i = 0; i < len; i++) {
		int b1, b2;

        	b2 = buf[index++] & 0x00FF;
        	b1 = buf[index++] & 0x00FF;
		c[i] = (char)((b1 << 8) + (b2 << 0));
            }
        } else {
	    for (int i = 0; i < len; i++) {
		int b1, b2;

        	b1 = buf[index++] & 0x00FF;
        	b2 = buf[index++] & 0x00FF;
		c[i] = (char)((b1 << 8) + (b2 << 0));
	    }
	}

	index += 2;  //  skip over the two null terminator bytes.  More efficient than read_wchar'ing them.

    	return new String(c);
    }

    public final void read_octet_array(byte[] b, int offset, int length) {
    	if ( b == null )
    	    throw new BAD_PARAM();

    	int n = offset;
    	while (n < length+offset) {
    	    int avail;
    	    int bytes;
    	    int wanted;

    	    avail = size - index;
            if (avail <= 0) {
                grow(1, 1);
                avail = size - index;
            }
    	    wanted = (length + offset) - n;
    	    bytes = (wanted < avail) ? wanted : avail;
    	    System.arraycopy(buf, index, b, n, bytes);
    	    index += bytes;
    	    n += bytes;
    	}
    }

    public Principal read_Principal() {
    	int len = read_long();
    	byte[] pvalue = new byte[len];
    	read_octet_array(pvalue,0,len);

    	Principal p = new PrincipalImpl();
    	p.name(pvalue);	
    	return p;
    }

    public TypeCode read_TypeCode() {
        TypeCodeImpl tc = new TypeCodeImpl(orb);
        tc.read_value(this);
	return tc;
    }
  
    public Any read_any() {
        // create an Any
        Any any;
        any = orb.create_any();

        // read off the typecode
        TypeCodeImpl tc = new TypeCodeImpl(orb);
        tc.read_value(this);

        // read off the value of the any
        any.read_value(this, tc);

        return any;
    }

    public org.omg.CORBA.Object read_Object() { 
        return read_Object(null);
    }

    // ------------ RMI related methods --------------------------

    public org.omg.CORBA.Object read_Object(Class stubClass) {
        IOR ior = new IOR(orb);
        ior.read(this);

        if (ior.is_nil()) {
            return null;
        }
      
    	org.omg.CORBA.Object objref = null;
    	Profile profile = ior.getProfile();
    	byte[] objKey = profile.getObjectKey();
    	String host   = profile.getHost();
    	SubcontractRegistry registry = orb.getSubcontractRegistry();

        // Is this an IOR for a local servant?  First check to
        // see if the host from this IOR is exactly the same string
        // we put into our IORs...
            
    	if (host.equals(orb.getORBServerHost())) {

            // It's the same host. Now check to see if the transient
            // server id matches...
                
            int serverId = Utility.bytesToInt(objKey, ServerDelegate.SERVERID_OFFSET);
            if (serverId == orb.getTransientServerId()) {

            	// Ok so far. Can we get a valid servant?
            		
            	ServerSubcontract sc = registry.getServerSubcontract(objKey);
                // Ram Jayaraman's fix.
            	if (sc != null&&
                     (sc.isServantSupported())) {
		    java.lang.Object servant = sc.getServant(objKey);
		    if (servant != null ) {
            		        
			// Got a valid servant. Is it a Tie?
            		        
			if (servant instanceof Tie) {
            		        
            		    // Yes, so it is a local servant. Load a stub
            		    // for it using the codebase from the IOR and
            		    // the delegate from the tie.
            		        
			    String codebase = ior.getCodebase();
			    objref = (org.omg.CORBA.Object)
				Utility.loadStub((Tie)servant,stubClass,codebase,false);
            		        
            		    // If we managed to load a stub, return it, otherwise we
            		    // must fail...
            		        
			    if (objref != null) {
				return objref;   
			    } else {
				throw new MARSHAL(MinorCodes.READ_OBJECT_EXCEPTION, 
						  CompletionStatus.COMPLETED_NO);
			    }
			} else if (servant instanceof org.omg.CORBA.Object) {
            		            
            		    // No, so assume IDL style stub...
            		            
			    return (org.omg.CORBA.Object) servant;
			}
		    }
            	}
            }
    	}
    	        
    	// Create delegate...
        	    
    	ClientSubcontract rep = registry.getClientSubcontract(objKey);
    	rep.unmarshal(ior);
    	rep.setOrb(orb);

        // Load stub, set the delegate and return it...
            
        return loadStub(ior,stubClass,(Delegate)rep);
    }
    
    protected static ObjectImpl loadStub(IOR ior, Class stubClass,
                                         Delegate delegate) {	                                    

        // Use the stubClass, if we have it...

        
        if (stubClass != null) {
            try {
                return newStub(stubClass,delegate);
            } catch (Throwable e) {
                if (e instanceof ThreadDeath) {
                    throw (ThreadDeath) e;
                }
            }
	} else {
            
            // Try to load from the ior...

            try {         
		String repID = ior.getTypeId();
		
		// If the repID is "", fall thru to returning the default
		// stub, otherwise try to load the class...
				
		if (repID.length() > 0) {
		    String codebase = ior.getCodebase();
		    Class clz = Utility.loadStubClass(repID, codebase, null); //d11638
                    return newStub(clz,delegate);
                }
            } catch (Throwable e) {
                if (e instanceof ThreadDeath) {
                    throw (ThreadDeath) e;
                }
            }

            // Return the "default" stub...
            	    
            ObjectImpl objref = new ObjectImpl() {
                public String[] _ids() {
		    String[] typeids = new String[1];
		    typeids[0] = "IDL:omg.org/CORBA/Object:1.0";
		    return typeids;
                }
	    };
        	    
    	    objref._set_delegate(delegate);
    	    return objref;
	}
        
        // We failed...

	throw new MARSHAL(MinorCodes.READ_OBJECT_EXCEPTION, 
			  CompletionStatus.COMPLETED_NO);
    }
 
    protected static ObjectImpl newStub(Class stubClass,Delegate delegate) 
	throws InstantiationException,
	       IllegalAccessException,
	       NoSuchMethodException,
	       InvocationTargetException {
        // What kind of stub do we have?
        
	if (java.rmi.Remote.class.isAssignableFrom(stubClass)) {
	        
	    // RMI. Instantiate, set delegate and return...
	        
            ObjectImpl objref = (ObjectImpl) stubClass.newInstance();
    	    objref._set_delegate(delegate);
    	    return objref;
	}
	    
	// RMI Abstract OR IDL. Instantiate, set delegate and return...
	        
        try {
                    
            // Try creating the stub using the default constructor...

            ObjectImpl result = (ObjectImpl)stubClass.newInstance();
            result._set_delegate(delegate);
            return result;
        } catch (Throwable e) {
            if (e instanceof ThreadDeath) {
                throw (ThreadDeath) e;
            }
        }
        
        // Try creating the stub using the delegate constructor...

        Class[] intArgsClass = new Class[] {org.omg.CORBA.portable.Delegate.class};
        java.lang.Object[]intArgs = new java.lang.Object[] {delegate};
        java.lang.reflect.Constructor intArgsConstructor;
        intArgsConstructor = stubClass.getConstructor(intArgsClass);
        return (ObjectImpl)intArgsConstructor.newInstance(intArgs);
    }

    public java.lang.Object read_abstract_interface() {
        return read_abstract_interface(null);
    }

    public java.lang.Object read_abstract_interface(java.lang.Class clz) {
    	boolean object = read_boolean();
        if (object) {
            return read_Object(clz);
        } else {
	    return read_value();
	}
    }

    public Serializable read_value() {

        // Read value tag
        int vType = readValueTag();
		
        int indirection = get_offset();
		
        if (vType == 0)
            return null; // value is null
        else if (vType == 0xffffffff) { // Indirection tag
	    int bytesBack = read_long();
            indirection = indirection + bytesBack;
            if (valueCache != null && valueCache.containsVal(indirection))
		{
		    java.io.Serializable cachedValue = (java.io.Serializable)valueCache.getKey(indirection);
		    return cachedValue;
		}
            else {
		throw new IndirectionException(indirection);
	    }
	}
        else {
	    indirection = indirection - 4;

	    // end_block();

	    boolean saveIsChunked = isChunked;
	    isChunked = RepositoryId.isChunkedEncoding(vType);
			
	    java.lang.Object value = null;
			
	    String codebase_URL = null;			
	    if (RepositoryId.isCodeBasePresent(vType)){
		codebase_URL = read_codebase_URL();
	    }
			
            // Read repository id
            String repositoryIDString = null; 
			
	    switch(RepositoryId.getTypeInfo(vType)){
	    case RepositoryId.kNoTypeInfo :
		repositoryIDString = RepositoryIdWrapper.createForAnyType(orb, expectedType);
		break;
	    case RepositoryId.kSingleRepTypeInfo :
		repositoryIDString = read_repositoryId(); 
		break;
	    case RepositoryId.kPartialListTypeInfo :
		repositoryIDString = read_repositoryIds();
		break;
		 			
	    }

	    // indirection = get_offset();

	    start_block();
	    end_flag--;

            if (repositoryIDString.equals(
                RepositoryId.kWStringValueRepID))
	    {
	        value = read_wstring();
	    }       
            // Fix to Take Care of Bad Rep Id
            else if ((repositoryIDString.equals(
                     RepositoryId.kClassDescValueRepID)) 
                 || (repositoryIDString.equals(
                        RepositoryId_1_3.kClassDescValueRepID)))
	    {
                // Read the class either with the wrong or
                // right RepId for the classDesc
	        value = readClass();
	    }       
            else {
                Class valueClass = expectedType;
                // Fix to Take Care of Bad Rep Id
                if( ORBUtility.isLegacyORB(
                    (com.sun.corba.se.internal.corba.ORB)orb)) 
                {
                    RepositoryId_1_3 repositoryID = 
                       RepositoryId_1_3.cache.getId(repositoryIDString);
                    try {
                        valueClass = repositoryID.getClassFromType();
                    } catch(java.lang.ClassNotFoundException e) {
                        try{
                            valueClass = repositoryID.getClassFromType(
                                         codebase_URL);
                        }
                        catch(java.lang.ClassNotFoundException e2) {
                       
                            // Make sure valueClass is null
                            valueClass = null;
                       
                        }
                        catch(java.net.MalformedURLException e3) {
                            throw new org.omg.CORBA.MARSHAL(
                            "Unable to locate value class for rep. id :"
                            + repositoryID + " because of malformed URL"                            + " " + codebase_URL);
                        }
                    }

                }
                else {
                    RepositoryId repositoryID = 
                        RepositoryId.cache.getId(repositoryIDString);
				
                    try {
		        valueClass = repositoryID.getClassFromType();
                    }
                    catch(java.lang.ClassNotFoundException e) {
		        try{
			    valueClass = repositoryID.getClassFromType(
                                     codebase_URL);
		        }
		        catch(java.lang.ClassNotFoundException e2) {
			    // Make sure valueClass is null
			    valueClass = null;
		        }
		        catch(java.net.MalformedURLException e3) {
			    throw new org.omg.CORBA.MARSHAL(
                            "Unable to locate value class for rep. id :"
                            + repositoryID + " because of malformed URL"                            + " " + codebase_URL);
		        }
                    }
                }
                if ((valueClass != null) && 
		org.omg.CORBA.portable.IDLEntity.class.isAssignableFrom(                valueClass)) 
                {
		    value = readIDLValue(indirection,repositoryIDString,
                            valueClass, codebase_URL);

		} else {

                    try{
			if (valueHandler == null) {
                            if (ORBUtility.isLegacyORB(
                            (com.sun.corba.se.internal.corba.ORB)orb)) 
                            {
                                valueHandler = 
                                   new ValueHandlerImpl_1_3();//d4365188
                            }
                            else {
			        valueHandler = 
                                    Util.createValueHandler(); //d11638
                            }
                        }
			value = valueHandler.readValue(this,indirection,
                                valueClass, repositoryIDString, 
                                getCodeBase());
                    }
                    catch(Exception ex) {
			ex.printStackTrace();
                        throw new org.omg.CORBA.MARSHAL(
                        "Unable to read value from underlying bridge : "
                        + ex.getMessage());
                    }
                    catch(Error e) {
                        throw new org.omg.CORBA.MARSHAL(
                        "Unable to read value from underlying bridge : "                        + e.getMessage());
                    }

                }
            }
	    
	    end_block();
	    readEndTag();

	    // Put into valueCache
	    if (valueCache == null)
		valueCache = new CacheTable(false);
	    valueCache.put(value, indirection);
	
	    // allow for possible continuation chunk
	    isChunked = saveIsChunked;
	    start_block();

	    return (java.io.Serializable)value;
        }
    }

    public Serializable read_value(Class expectedType) {

	String expectedTypeRepId = RepositoryIdWrapper.createForAnyType(orb, expectedType);

        // Read value tag
        int vType = readValueTag();
		
        int indirection = get_offset();
		
        if (vType == 0)
            return null; // value is null
        else if (vType == 0xffffffff) { // Indirection tag
	    int bytesBack = read_long();
            indirection = indirection + bytesBack;
            if (valueCache != null && valueCache.containsVal(indirection))
		{
		    java.io.Serializable cachedValue = (java.io.Serializable)valueCache.getKey(indirection);
		    return cachedValue;
		}
            else {
		throw new IndirectionException(indirection);
	    }
	}
        else {
	    indirection = indirection - 4;

	    // end_block();

	    boolean saveIsChunked = isChunked;
	    isChunked = RepositoryId.isChunkedEncoding(vType);
			
	    java.lang.Object value = null;
			
	    String codebase_URL = null;			
	    if (RepositoryId.isCodeBasePresent(vType)){
		codebase_URL = read_codebase_URL();
	    }
			
            // Read repository id
            String repositoryIDString = null; 
			
	    switch(RepositoryId.getTypeInfo(vType)){
	    case RepositoryId.kNoTypeInfo :
		repositoryIDString = RepositoryIdWrapper.createForAnyType(orb, expectedType);
		break;
	    case RepositoryId.kSingleRepTypeInfo :
		repositoryIDString = read_repositoryId(); 
		break;
	    case RepositoryId.kPartialListTypeInfo :
		repositoryIDString = read_repositoryIds();
		break;
		 			
	    }

	    // indirection = get_offset();

	    start_block();
	    end_flag--;

            if (repositoryIDString.equals(RepositoryId.kWStringValueRepID))
	    {
	        value = read_wstring();
	    }       
            else if((repositoryIDString.equals(
                     RepositoryId.kClassDescValueRepID)) ||
                 (repositoryIDString.equals(
                     RepositoryId_1_3.kClassDescValueRepID)))
	    {
                // Read the class when the RepositoryId is right or 
                // wrong (Old ORB in JDK 1.3)    
	        value = readClass();
	    }       
            else {
                Class valueClass = expectedType;
                if (ORBUtility.isLegacyORB(
                    (com.sun.corba.se.internal.corba.ORB)orb)) 
                {
                    RepositoryId_1_3 repositoryID = 
                       RepositoryId_1_3.cache.getId(repositoryIDString);
                    if ((!expectedTypeRepId.equals(repositoryIDString))
                     || (valueClass == null)) 
                    {
                        try {
                            valueClass = repositoryID.getClassFromType(
                                         valueClass, codebase_URL);
                        } catch(java.lang.ClassNotFoundException e) {
                            // Make sure valueClass is null
                            valueClass = null;
                        }
                    }
                }
                else {
                    RepositoryId repositoryID = 
                        RepositoryId.cache.getId(repositoryIDString);
		    if ((!expectedTypeRepId.equals(repositoryIDString))
                     || (valueClass == null)) 
                    {
		        try {
			    valueClass = repositoryID.getClassFromType(
                                         valueClass, codebase_URL);
		        }
		        catch(java.lang.ClassNotFoundException e) {
			    // Make sure valueClass is null
			    valueClass = null;
		        }
		    }
                }

                if ((valueClass != null) && 
		org.omg.CORBA.portable.IDLEntity.class.isAssignableFrom(                    valueClass)) 
                {
		    value =  readIDLValue(indirection, 
                        repositoryIDString, valueClass, codebase_URL);

		} else {

                    try{
			if (valueHandler == null) {
                            if (ORBUtility.isLegacyORB(
                              (com.sun.corba.se.internal.corba.ORB)orb))             
                            {
                                valueHandler = 
                                   new ValueHandlerImpl_1_3();//d4365188
                            } else {
			        valueHandler = 
                                    Util.createValueHandler(); //d11638
                            }
                        }
			value = valueHandler.readValue(this, 
                             indirection, valueClass,repositoryIDString,
                             getCodeBase());
                    }
                    catch(Exception ex){
			ex.printStackTrace();
                        throw new org.omg.CORBA.MARSHAL(
                        "Unable to read value from underlying bridge : "
                        + ex.getMessage());
                    }
                    catch(Error e){
                        // Delete this line
                        e.printStackTrace();
                        throw new org.omg.CORBA.MARSHAL(
                        "Unable to read value from underlying bridge : "
                        + e.getMessage());
                    }

                }
            }
	    
	    end_block();
	    readEndTag();

	    // Put into valueCache
	    if (valueCache == null)
		valueCache = new CacheTable(false);
	    valueCache.put(value, indirection);
	
	    // allow for possible continuation chunk
	    isChunked = saveIsChunked;
	    start_block();

	    return (java.io.Serializable)value;
        }		
    }

    public Serializable read_value(BoxedValueHelper factory) {

        // Read value tag
        int vType = readValueTag();

        int indirection = get_offset();

        if (vType == 0)
            return null; // value is null
        else if (vType == 0xffffffff) { // Indirection tag
            indirection = indirection + read_long();
            if (valueCache != null && valueCache.containsVal(indirection))
		{
		    java.io.Serializable cachedValue = (java.io.Serializable)valueCache.getKey(indirection);
		    return cachedValue;
		}
            else {
		throw new IndirectionException(indirection);
	    }
	}
        else {
	    indirection = indirection - 4;

	    // end_block();

	    boolean saveIsChunked = isChunked;
	    isChunked = RepositoryId.isChunkedEncoding(vType);

	    java.lang.Object value = null;

	    String codebase_URL = null;			
	    if (RepositoryId.isCodeBasePresent(vType)){
		codebase_URL = read_codebase_URL();
	    }

            // Read repository id
            String repositoryIDString = null; 

	    switch(RepositoryId.getTypeInfo(vType)){
	    case RepositoryId.kNoTypeInfo :
		throw new org.omg.CORBA.MARSHAL("No class description available (value_tag indicates no type information present)");
	    case RepositoryId.kSingleRepTypeInfo :
		repositoryIDString = read_repositoryId(); 
		break;
	    case RepositoryId.kPartialListTypeInfo :
		repositoryIDString = read_repositoryIds(); 
		break;
					
	    }

            // Fetch id container
            if (ORBUtility.isLegacyORB(
                (com.sun.corba.se.internal.corba.ORB)orb)) {
                RepositoryId_1_3 repositoryID = 
                   RepositoryId_1_3.cache.getId(repositoryIDString);
            } else {
                RepositoryId repositoryID = 
                   RepositoryId.cache.getId(repositoryIDString);
            }

            // Compare rep. ids to see if we should use passed helper
            if (!repositoryIDString.equals(factory.get_id()))
		factory = Utility.getHelper(null, codebase_URL, repositoryIDString);

	    start_block();
	    end_flag--;
	    
	    if (factory instanceof ValueHelper) {
		value = readIDLValueWithHelper((ValueHelper)factory, indirection);
	    } else {
		valueIndirection = indirection;  // for callback
		value = factory.read_value(this);
	    }

	    end_block();
	    readEndTag();

	    // Put into valueCache
	    if (valueCache == null)
		valueCache = new CacheTable(false);
	    valueCache.put(value, indirection);
	
	    // allow for possible continuation chunk
	    isChunked = saveIsChunked;
	    start_block();

            return (java.io.Serializable)value;
        }
    }

    private boolean isCustomType(ValueHelper helper) {
	try{
	    TypeCode tc = helper.get_type();
	    int kind = tc.kind().value();
	    if (kind == TCKind._tk_value) {
		return (tc.type_modifier() == org.omg.CORBA.VM_CUSTOM.value);
	    }
	}
	catch(BadKind ex) {
	    throw new org.omg.CORBA.MARSHAL();
	}

	return false;
    }

    // This method is actually called indirectly by 
    // read_value(String repositoryId).
    // Therefore, it is not a truly independent read call that handles
    // header information itself.
    public java.io.Serializable read_value(java.io.Serializable value) {

	// Put into valueCache using valueIndirection
	if (valueCache == null)
	    valueCache = new CacheTable(false);
	valueCache.put(value, valueIndirection);

	if (value instanceof StreamableValue)
	    ((StreamableValue)value)._read(this);
	else if (value instanceof CustomValue)
	    ((CustomValue)value).unmarshal(this);
			
	return value;
    }

    public java.io.Serializable read_value(java.lang.String repositoryId) {

	// if (inBlock)
	//    end_block();

        // Read value tag
        int vType = readValueTag();

        int indirection = get_offset();

        if (vType == 0)
            return null; // value is null
        else if (vType == 0xffffffff) { // Indirection tag
            indirection = indirection + read_long();
            if (valueCache != null && valueCache.containsVal(indirection))
		{
		    java.io.Serializable cachedValue = (java.io.Serializable)valueCache.getKey(indirection);
		    return cachedValue;
		}
            else {
		throw new IndirectionException(indirection);
	    }
	}
        else {
	    indirection = indirection - 4;

	    // end_block();

	    boolean saveIsChunked = isChunked;
	    isChunked = RepositoryId.isChunkedEncoding(vType);

	    java.lang.Object value = null;

	    String codebase_URL = null;			
	    if (RepositoryId.isCodeBasePresent(vType)){
		codebase_URL = read_codebase_URL();
	    }

            // Read repository id
            String repositoryIDString = null; 

	    switch(RepositoryId.getTypeInfo(vType)){
	    case RepositoryId.kNoTypeInfo :
		repositoryIDString = repositoryId; 
		break;
	    case RepositoryId.kSingleRepTypeInfo :
		repositoryIDString = read_repositoryId(); 
		break;
	    case RepositoryId.kPartialListTypeInfo :
		repositoryIDString = read_repositoryIds(); 
		break;
					
	    }

	    ValueFactory factory = Utility.getFactory(null, codebase_URL, orb, repositoryIDString);

	    start_block();
	    end_flag--;

	    valueIndirection = indirection;  // for callback
	    value = factory.read_value(this);

	    end_block();
	    readEndTag();

	    // Put into valueCache
	    if (valueCache == null)
		valueCache = new CacheTable(false);
	    valueCache.put(value, indirection);
	
	    // allow for possible continuation chunk
	    isChunked = saveIsChunked;
	    start_block();

            return (java.io.Serializable)value;
        }		
    }

    private Class readClass() {

	String classRepId = (String)read_value();
	String codebases = (String)read_value();
		
        Class cl = null;
        if (ORBUtility.isLegacyORB(
            (com.sun.corba.se.internal.corba.ORB)orb)) 
        {
            RepositoryId_1_3 repositoryID = 
                RepositoryId_1_3.cache.getId(classRepId);

            try {
                cl = repositoryID.getClassFromType(codebases);
            } catch(ClassNotFoundException cnfe){
                throw new org.omg.CORBA.MARSHAL(
                "Unable to load Class " + repositoryID.getClassName() +
                " : " + cnfe.getMessage()); 
            } catch(MalformedURLException me){
                throw new org.omg.CORBA.MARSHAL(
                "Unable to load Class " + repositoryID.getClassName() +
                " : " + me.getMessage());
            }
        } else {
            RepositoryId repositoryID = 
                RepositoryId.cache.getId(classRepId);
            try {
                cl = repositoryID.getClassFromType(codebases);
            } catch(ClassNotFoundException cnfe){
                throw new org.omg.CORBA.MARSHAL(
                "Unable to load Class " + repositoryID.getClassName() +
                " : " + cnfe.getMessage());
            }
            catch(MalformedURLException me){
                throw new org.omg.CORBA.MARSHAL(
                "Unable to load Class " + repositoryID.getClassName() +
                " : " + me.getMessage());
            }
        }
        return cl;
    }

    private java.lang.Object readIDLValueWithHelper(ValueHelper helper, int indirection) {
	// look for two-argument static read method
	Method readMethod;
	try {
	    Class argTypes[] = {org.omg.CORBA.portable.InputStream.class, helper.get_class()};
	    readMethod = helper.getClass().getDeclaredMethod(kReadMethod, argTypes);
	}
	catch(NoSuchMethodException nsme) { // must be boxed value helper
	    java.lang.Object result = helper.read_value(this);
	    return result;
	}

	// found two-argument read method, so must be non-boxed value...
	// ...create a blank instance
	java.lang.Object val = null;
	try {
	    val = helper.get_class().newInstance();
	} 
	catch(java.lang.InstantiationException ie){
	    throw new org.omg.CORBA.MARSHAL(ie.getMessage());
	}	
	catch(IllegalAccessException iae){ 
	    // Value's constructor is protected or private
	    //
	    // So, use the helper to read the value.
	    //
	    // NOTE : This means that in this particular case a recursive ref.
	    // would fail.
	    return helper.read_value(this);
	}

	// add blank instance to cache table
        if (valueCache == null)
            valueCache = new CacheTable(false);
	valueCache.put(val, indirection);

	// if custom type, call unmarshal method
	if (val instanceof CustomMarshal && isCustomType(helper)) {
            ((CustomMarshal)val).unmarshal(this);
	    return val;
	}

	// call two-argument read method using reflection
	try {
	    java.lang.Object args[] = {this, val};
	    readMethod.invoke(helper, args);
            return val;
	}
	catch(IllegalAccessException iae2){
	    throw new org.omg.CORBA.MARSHAL(iae2.getMessage());
	}
	catch(InvocationTargetException ite){
	    throw new org.omg.CORBA.MARSHAL(ite.getMessage());
	}
    }

    private java.lang.Object readBoxedIDLEntity(Class clazz, String codebase)
    {
	try {
            ClassLoader clazzLoader = (clazz == null ? null : clazz.getClassLoader());
	    Class helperClass = Utility.loadClassForClass(clazz.getName()+"Helper", codebase,
                                                   clazzLoader, clazz, clazzLoader);
	    Class argTypes[] = {org.omg.CORBA.portable.InputStream.class};
	    Method readMethod = helperClass.getDeclaredMethod(kReadMethod, argTypes);
	    java.lang.Object args[] = {this};
	    return readMethod.invoke(null, args);

	} catch (ClassNotFoundException cnfe) {
	    throw new org.omg.CORBA.MARSHAL(cnfe.getMessage());
	} catch(NoSuchMethodException nsme) {
	    throw new org.omg.CORBA.MARSHAL(nsme.getMessage());
	} catch(IllegalAccessException iae) {
	    throw new org.omg.CORBA.MARSHAL(iae.getMessage());
	} catch(InvocationTargetException ite) {
	    throw new org.omg.CORBA.MARSHAL(ite.getMessage());
	}
    }

    private java.lang.Object readIDLValue(int indirection, String repId,
					  Class clazz, String codebase)
    {					
	if (StreamableValue.class.isAssignableFrom(clazz) ||
    	    CustomValue.class.isAssignableFrom(clazz)) {

	    // use new-style OBV support (factory object)
	    ValueFactory factory = Utility.getFactory(clazz, codebase, orb, repId);
	    valueIndirection = indirection;  // for callback
	    return factory.read_value(this);

	} else if (ValueBase.class.isAssignableFrom(clazz)) {

	    // use old-style OBV support (helper object)
	    BoxedValueHelper helper = Utility.getHelper(clazz, codebase, repId);
	    if (helper instanceof ValueHelper)
		return readIDLValueWithHelper((ValueHelper)helper, indirection);
	    else
		return helper.read_value(this);
	
	} else {

	    // must be a boxed IDLEntity, so make a reflective call to the
	    // helper's static read method...
	    return readBoxedIDLEntity(clazz, codebase);
	}

    }

    // This method handle end tag compaction.  Note that it is lazy in the sense
    // that it always assumes a tag has been compacted if the end tag is not what
    // it expected...
    private void readEndTag() {
	if (isChunked) {
	    int markIndex = mark();
	    int anEndTag = read_long();
	    if (anEndTag != end_flag) {
		reset(markIndex);
	    }
	}

	// if (end_flag == -1)
	//     outerValueDone = true;
	// else 
	//     outerValueDone = false;

	end_flag++;

	//if (anEndTag == -1)
    }

    private int get_offset() {
	return index;
    }

    private void start_block() {
		
	// if (outerValueDone)
	if (!isChunked)
	    return;
	
	// if called from alignAndCheck, need to reset blockLength
	// to avoid an infinite recursion loop on read_long() call
	blockLength = maxBlockLength;

	blockLength = read_long();
		
	if ((blockLength > 0) && (blockLength < maxBlockLength)) {
	    blockLength += index;  // _REVISIT_ unsafe, should use a Java long
	    // inBlock = true;
	} else {
	    // not a chunk length field
	    blockLength = maxBlockLength;
	    index -= 4;
	}
		
    }

    private void end_block() {

	// if in a chunk, check for underflow or overflow
	if (blockLength != maxBlockLength)
	    if (blockLength == index)
		blockLength = maxBlockLength;  // chunk has ended
	    else
	        throw new MARSHAL("Incorrect chunk length "+blockLength+" at offset "+index,
				  997, CompletionStatus.COMPLETED_NO);

	// blockLength = -1;

	// if (!inBlock)
	//    return true;

	// Check to see if block deleted
	// if (checkForNullBlock == index){
	//     checkForNullBlock = -1;
	//     index = index - 4;
	//     inBlock = false;
	//     return true;
	// }

	// if (outerValueDone){
	//     inBlock = false;
	//     return true;
	// }
		

	// if (isAtEnd() || ((index+4)>size))
	//     return false;

	// inBlock = false;

	// return true;
    }
    
    private int readValueTag(){
	// outerValueDone = false;
	return read_long();
    }

    public org.omg.CORBA.ORB orb() {
        return orb;    
    }

    // ------------ End RMI related methods --------------------------

    public final void read_boolean_array(boolean[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_boolean();
    	}
    }

    public final void read_char_array(char[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_char();
    	}
    }

    public final void read_wchar_array(char[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_wchar();
    	}
    }

    public final void read_short_array(short[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_short();
    	}
    }

    public final void read_ushort_array(short[] value, int offset, int length) {
    	read_short_array(value, offset, length);
    }

    public final void read_long_array(int[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_long();
    	}
    }

    public final void read_ulong_array(int[] value, int offset, int length) {
    	read_long_array(value, offset, length);
    }

    public final void read_longlong_array(long[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_longlong();
    	}
    }

    public final void read_ulonglong_array(long[] value, int offset, int length) {
    	read_longlong_array(value, offset, length);
    }

    public final void read_float_array(float[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_float();
    	}
    }

    public final void read_double_array(double[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_double();
    	}
    }

    public final void read_any_array(org.omg.CORBA.Any[] value, int offset, int length) {
    	for(int i=0; i < length; i++) {
    	    value[i+offset] = read_any();
    	}
    }

    //--------------------------------------------------------------------//
    // CDRInputStream state management.
    //

    public final void rewind() {
    	offset += index;
    	index = 0;
    }

    /** 
     * Are we at the end of the input stream?
     */
    public final boolean isAtEnd() {
    	return index == size;
    }

    public int available() throws IOException {
        return size - index;
    }

    // MarshalInputStream methods mark, reset
    public int mark() {
    	return index;
    }

    public void reset(int mark) {
    	index = mark;
    }

    
    public void setExpectedType(Class expectedType) {
	this.expectedType = expectedType;
    }

    private String read_repositoryIds() {
		
	// Read # of repository ids
	int numRepIds = read_long();
	if (numRepIds == 0xffffffff) {
            int indirection = get_offset() + read_long();
            if (repositoryIdCache != null && repositoryIdCache.containsOrderedVal(indirection))
		return (String)repositoryIdCache.getKey(indirection);
            else
		throw new org.omg.CORBA.MARSHAL("Unable to locate array of repository IDs from indirection " + indirection);			

	} else {

	    // read first array element and store it as an indirection to the whole array
	    int indirection = get_offset() - 4; 
	    String repID = read_repositoryId();
            if (repositoryIdCache == null)
        	repositoryIdCache = new CacheTable(false);
            repositoryIdCache.put(repID, indirection);

	    // read and ignore the subsequent array elements, but put them in the
	    // indirection table in case there are later indirections back to them
	    for (int i = 1; i < numRepIds; i++) {
		read_repositoryId();
	    }
		
	    return repID;
	}
    }

    private final String read_repositoryId() {

        String result = readStringOrIndirection(true);

        if (result == null) { // Indirection

            int indirection = get_offset() + read_long();

            if (repositoryIdCache != null && repositoryIdCache.containsOrderedVal(indirection))
                return (String)repositoryIdCache.getKey(indirection);
            else
		throw new org.omg.CORBA.MARSHAL("Repid indirection @ " + index, 0,
						CompletionStatus.COMPLETED_MAYBE);
        } else {

            if (repositoryIdCache == null)
                repositoryIdCache = new CacheTable(false);
            repositoryIdCache.put(result, stringIndirection);

            return result;
        }
    }

    private final String read_codebase_URL() {

        String result = readStringOrIndirection(true);

        if (result == null) { // Indirection

            int indirection = get_offset() + read_long();

            if (codebaseCache != null && codebaseCache.containsVal(indirection))
                return (String)codebaseCache.getKey(indirection);
            else
		throw new org.omg.CORBA.MARSHAL("Codebase indirection @ " + index, 0,
						CompletionStatus.COMPLETED_MAYBE);
	} else {

	    if (codebaseCache == null)
		codebaseCache = new CacheTable(false);
	    codebaseCache.put(result, stringIndirection);

            return result;
        }
    }

    /* DataInputStream methods */

    public java.lang.Object read_Abstract () {
        return read_abstract_interface();
    }

    public java.io.Serializable read_Value () {
        return read_value();
    }

    public void read_any_array (org.omg.CORBA.AnySeqHolder seq, int offset, int length) {
        read_any_array(seq.value, offset, length);
    }

    public void read_boolean_array (org.omg.CORBA.BooleanSeqHolder seq, int offset, int length) {
        read_boolean_array(seq.value, offset, length);
    }

    public void read_char_array (org.omg.CORBA.CharSeqHolder seq, int offset, int length) {
        read_char_array(seq.value, offset, length);
    }

    public void read_wchar_array (org.omg.CORBA.WCharSeqHolder seq, int offset, int length) {
        read_wchar_array(seq.value, offset, length);
    }

    public void read_octet_array (org.omg.CORBA.OctetSeqHolder seq, int offset, int length) {
        read_octet_array(seq.value, offset, length);
    }

    public void read_short_array (org.omg.CORBA.ShortSeqHolder seq, int offset, int length) {
        read_short_array(seq.value, offset, length);
    }

    public void read_ushort_array (org.omg.CORBA.UShortSeqHolder seq, int offset, int length) {
        read_ushort_array(seq.value, offset, length);
    }

    public void read_long_array (org.omg.CORBA.LongSeqHolder seq, int offset, int length) {
        read_long_array(seq.value, offset, length);
    }

    public void read_ulong_array (org.omg.CORBA.ULongSeqHolder seq, int offset, int length) {
        read_ulong_array(seq.value, offset, length);
    }

    public void read_ulonglong_array (org.omg.CORBA.ULongLongSeqHolder seq, int offset, int length) {
        read_ulonglong_array(seq.value, offset, length);
    }

    public void read_longlong_array (org.omg.CORBA.LongLongSeqHolder seq, int offset, int length) {
        read_longlong_array(seq.value, offset, length);
    }

    public void read_float_array (org.omg.CORBA.FloatSeqHolder seq, int offset, int length) {
        read_float_array(seq.value, offset, length);
    }

    public void read_double_array (org.omg.CORBA.DoubleSeqHolder seq, int offset, int length) {
        read_double_array(seq.value, offset, length);
    }

    private static String _id = "IDL:omg.org/CORBA/DataInputStream:1.0";
    private static String[] _ids = { _id };

    public String[] _truncatable_ids() {
        return _ids;
    }

    /* for debugging */

    public void printBuffer() {
	for (int i = 0; i < size; i += 16) {
	    for (int j = 0; j < 16; j++) {
	        int k = buf[i+j];
		if (k < 0)
		    k = 256+k;
                String hex = Integer.toHexString(k);
		if (hex.length() == 1)
		    hex = "0"+hex;
		System.out.print(hex+" ");
	    }
	    System.out.println(new String(buf, i, 16));
	}
    }

    // Fix for serialization bugs.  (Put bug ID)
    CodeBase getCodeBase() {
        return FVDCodeBaseDelegate.singleton;
    }
}
