/*
 * @(#)CDROutputStream.java	1.67 01/02/09
 *
 * Copyright 1993-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.iiop;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.Object;
import org.omg.CORBA.Principal;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.ObjectImpl;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.ValueBase;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.portable.IDLEntity;
import org.omg.CORBA.VM_CUSTOM;
import org.omg.CORBA.VM_TRUNCATABLE;
import org.omg.CORBA.VM_NONE;

import java.rmi.Remote;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.core.MarshalOutputStream;

import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.ClientSubcontract;
import com.sun.corba.se.internal.corba.TypeCodeImpl;
import com.sun.corba.se.internal.util.RepositoryId;
import com.sun.org.omg.CORBA.portable.ValueHelper;
import org.omg.CORBA.portable.CustomValue;
import org.omg.CORBA.portable.StreamableValue;
import org.omg.CORBA.CustomMarshal;
import org.omg.CORBA.DataOutputStream;
import org.omg.CORBA.portable.BoxedValueHelper;
import org.omg.CORBA.TypeCodePackage.BadKind;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Stack;
import com.sun.corba.se.internal.orbutil.CacheTable;
import com.sun.corba.se.internal.orbutil.RepositoryIdWrapper;
import com.sun.corba.se.internal.orbutil.RepositoryId_1_3;
import com.sun.corba.se.internal.orbutil.ValueHandlerImpl_1_3;
import javax.rmi.CORBA.ValueHandler;
import com.sun.corba.se.internal.orbutil.ORBUtility;
import com.sun.corba.se.internal.util.Utility;

import javax.rmi.CORBA.Util;

public class CDROutputStream
    extends org.omg.CORBA_2_3.portable.OutputStream
    implements com.sun.corba.se.internal.core.MarshalOutputStream, org.omg.CORBA.DataOutputStream
{
    protected boolean littleEndian;
    protected int size, buflen, offset;
    protected byte[] buf;
    protected org.omg.CORBA.ORB orb;
	
    protected int blockSizeIndex = -1;

    private static final int DEFAULT_BUFFER_SIZE = 1024;
    private static final String kWriteMethod = "write";

    // Codebase cache
    private CacheTable codebaseCache = null;

    // Value cache
    private CacheTable valueCache = null;

    // Repository ID cache
    private CacheTable repositoryIdCache = null;

    // Write end flag
    private int end_flag = 0;

    // Nesting custom marshaling - used to tell whether we can skip chunking
    private boolean mustChunk = false;

    // In block marker
    private boolean inBlock = false;

    // Last end tag position
    private int end_flag_index = 0;

    // ValueHandler
    private javax.rmi.CORBA.ValueHandler valueHandler = null;

    public CDROutputStream(org.omg.CORBA.ORB orb) {
	this(orb, false);
    }

    public CDROutputStream(org.omg.CORBA.ORB orb, boolean littleEndian) {
	this(orb, littleEndian, DEFAULT_BUFFER_SIZE);
    }

    public CDROutputStream(org.omg.CORBA.ORB orb, boolean littleEndian, int size) {
    	this.orb = (org.omg.CORBA.ORB)orb;
    	this.littleEndian = littleEndian;
    	this.buf = new byte[size];
    	this.size = 0;
    	this.buflen = size;
    	offset = 0;
    }

    public org.omg.CORBA.ORB getORB()
    {
	return orb ;
    }


    public CDROutputStream() {
    }

    public byte[] toByteArray() {
    	byte[] it;

    	it = new byte[size];
    	System.arraycopy(buf, 0, it, 0, size);

    	return it;
    }

    private final void alignAndReserve(int align, int n) {
        if (align > 1) {
            int incr = size & (align - 1);
            if (incr != 0) {
                incr = (align - incr);
		size += incr;
            }
        }
    	if (size + n >= buflen)
	    grow(align, n);
    }

    //
    // Default implementation of grow.  Subclassers may override this.
    // Always grow the single buffer. This needs to delegate
    // fragmentation policy for IIOP 1.1.
    //
    protected void grow(int align, int n) {

    	if (buf == null) {
    	    int newLength = DEFAULT_BUFFER_SIZE;
    	    while (size + n >= newLength)
    		newLength = newLength * 2;
    	    buf = new byte[newLength];
    	    buflen = buf.length;
    	    return;
        }

    	byte[] old = buf;
    	int newLength = old.length * 2;
    	while (size + n >= newLength)
    	    newLength = newLength * 2;
    	buf = new byte[newLength];
    	System.arraycopy(old, 0, buf, 0, old.length);
	buflen = buf.length;
    }

    public final void putEndian() throws SystemException {
    	write_boolean(littleEndian);
    }

    public final boolean littleEndian() {
    	return littleEndian;
    }

    public org.omg.CORBA.portable.InputStream create_input_stream()
    {
		
	if (codebaseCache != null)
	    codebaseCache.done();

	if (valueCache != null)
	    valueCache.done();
		
	if (repositoryIdCache != null)
	    repositoryIdCache.done();
		
    	return new CDRInputStream(orb, buf, size, littleEndian);
    }

    // No such type in java
    public final void write_longdouble(double x) 
    {
	throw new NO_IMPLEMENT(MinorCodes.SEND_DEFERRED_NOTIMPLEMENTED,
			       CompletionStatus.COMPLETED_MAYBE);
    }

    public final void write_octet(byte x) 
    {
        if (size + 1 >= buflen)
            alignAndReserve(1, 1);

    	buf[size++] = x;
    }

    public final void write_boolean(boolean x)
    {
	write_octet(x? (byte)1:(byte)0);
    }

    public final void write_char(char x) 
    {
    	if ( x > '\u00ff' )
    	    throw new DATA_CONVERSION(com.sun.corba.se.internal.util.MinorCodes.CHARACTER_OUTOFRANGE,
				      CompletionStatus.COMPLETED_NO);

	write_octet((byte)(x & 0xFF));
    }

    private final void writeLittleEndianWchar(char x) {
    	buf[size++] = (byte)(x & 0xFF);
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    }

    private final void writeBigEndianWchar(char x) {
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    	buf[size++] = (byte)(x & 0xFF);
    }

    private final void writeLittleEndianShort(short x) {
    	buf[size++] = (byte)(x & 0xFF);
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    }

    private final void writeBigEndianShort(short x) {
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    	buf[size++] = (byte)(x & 0xFF);
    }

    private final void writeLittleEndianLong(int x) {
    	buf[size++] = (byte)(x & 0xFF);
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    	buf[size++] = (byte)((x >>> 16) & 0xFF);
    	buf[size++] = (byte)((x >>> 24) & 0xFF);
    }

    private final void writeBigEndianLong(int x) {
    	buf[size++] = (byte)((x >>> 24) & 0xFF);
    	buf[size++] = (byte)((x >>> 16) & 0xFF);
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    	buf[size++] = (byte)(x & 0xFF);
    }

    private final void writeLittleEndianLongLong(long x) {
    	buf[size++] = (byte)(x & 0xFF);
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    	buf[size++] = (byte)((x >>> 16) & 0xFF);
    	buf[size++] = (byte)((x >>> 24) & 0xFF);
    	buf[size++] = (byte)((x >>> 32) & 0xFF);
    	buf[size++] = (byte)((x >>> 40) & 0xFF);
    	buf[size++] = (byte)((x >>> 48) & 0xFF);
    	buf[size++] = (byte)((x >>> 56) & 0xFF);
    }

    private final void writeBigEndianLongLong(long x) {
    	buf[size++] = (byte)((x >>> 56) & 0xFF);
    	buf[size++] = (byte)((x >>> 48) & 0xFF);
    	buf[size++] = (byte)((x >>> 40) & 0xFF);
    	buf[size++] = (byte)((x >>> 32) & 0xFF);
    	buf[size++] = (byte)((x >>> 24) & 0xFF);
    	buf[size++] = (byte)((x >>> 16) & 0xFF);
    	buf[size++] = (byte)((x >>> 8) & 0xFF);
    	buf[size++] = (byte)(x & 0xFF);
    }

    public final void write_wchar(char x)
    {
    	alignAndReserve(2, 2);
	
    	if (littleEndian) {
    	    writeLittleEndianWchar(x);
    	} else {
    	    writeBigEndianWchar(x);
    	}
    }

    public final void write_short(short x) 
    {
    	alignAndReserve(2, 2);
    	
    	if (littleEndian) {
    	    writeLittleEndianShort(x);
    	} else {
    	    writeBigEndianShort(x);
    	}
    }

    public final void write_ushort(short x)
    {
	write_short(x);
    }

    public final void write_long(int x) 
    {
        if ((size & 3) != 0 || size + 4 >= buflen)
            alignAndReserve(4, 4);

    	if (littleEndian) {
    	    writeLittleEndianLong(x);
    	} else {
    	    writeBigEndianLong(x);
    	}
    }

    public final void write_ulong(int x)
    {
	write_long(x);
    }

    public final void write_longlong(long x) 
    {
    	alignAndReserve(8, 8);

    	if (littleEndian) {
    	    writeLittleEndianLongLong(x);
    	} else {
    	    writeBigEndianLongLong(x);
    	}
    }

    public final void write_ulonglong(long x)
    {
	write_longlong(x);
    }

    public final void write_float(float x) 
    {
	write_long(Float.floatToIntBits(x));
    }

    public final void write_double(double x) 
    {
	write_longlong(Double.doubleToLongBits(x));
    }

    public final void write_string(String value)
    {
        if ( value == null ) {
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);
        }

	// align to the next 4 byte boundary
        int incr = size & 3;
        if (incr != 0) {
            incr = (4 - incr);
            size += incr;
        }

    	int len = value.length() + 1;
        if (size + 4 + len >= buflen)
            alignAndReserve(4, 4 + len);

	if (littleEndian)
	    writeLittleEndianLong(len);
	else
	    writeBigEndianLong(len);

	len--;
    	for (int i=0; i < len; i++) {
    	    char x = value.charAt(i);

            if ( x > '\u00ff' )
        	throw new DATA_CONVERSION(com.sun.corba.se.internal.util.MinorCodes.CHARACTER_OUTOFRANGE,
					  CompletionStatus.COMPLETED_NO);

            buf[size++] = (byte)(x & 0xFF);
	}
        buf[size++] = (byte)0;
    }

    public final void write_wstring(String value)
    {
        if ( value == null ) {
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);
        }

	// align to the next 4 byte boundary
        int incr = size & 3;
        if (incr != 0) {
            incr = (4 - incr);
            size += incr;
        }

    	int len = value.length() + 1;
        if (size + 4 + (len * 2) >= buflen)
            alignAndReserve(4, 4 + (len * 2));

	if (littleEndian) {
	    writeLittleEndianLong(len);
	    len--;
    	    for (int i=0; i < len; i++) {
    		char x = value.charAt(i);
    		buf[size++] = (byte)(x & 0xFF);
    		buf[size++] = (byte)((x >>> 8) & 0xFF);
	    }
	} else {
	    writeBigEndianLong(len);
	    len--;
    	    for (int i=0; i < len; i++) {
    		char x = value.charAt(i);
    		buf[size++] = (byte)((x >>> 8) & 0xFF);
    		buf[size++] = (byte)(x & 0xFF);
	    }
	}
    	buf[size++] = (byte)0;
    	buf[size++] = (byte)0;
    }

    public final void write_octet_array(byte b[], int offset, int length)
    {
        if ( b == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

    	int n = offset;

    	while (n < length+offset) {
    	    int avail;
    	    int bytes;
    	    int wanted;

            if (size + 1 >= buflen)
        	alignAndReserve(1, 1);
    	    avail = buf.length - size;
    	    wanted = (length + offset) - n;
    	    bytes = (wanted < avail) ? wanted : avail;
    	    System.arraycopy(b, n, buf, size, bytes);
    	    size += bytes;
    	    n += bytes;
    	}
    }

    public void write_Principal(Principal p)
    {
    	write_long(p.name().length);
    	write_octet_array(p.name(), 0, p.name().length);
    }

    public void write_any(Any any) 
    {
        if ( any == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

    	write_TypeCode(any.type());
    	any.write_value(this);
    }

    public void write_TypeCode(TypeCode tc)
    {
        if ( tc == null ) {

            throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);
	}
        TypeCodeImpl tci;
        if (tc instanceof TypeCodeImpl) {
	    tci = (TypeCodeImpl)tc;
	}
        else {
	    tci = new TypeCodeImpl(orb, tc);
	}
        tci.write_value(this);
    }
 
    public void write_Object(org.omg.CORBA.Object ref)
    {
        if (ref == null) {
            IOR.NULL.write(this);
            return;
        }
		
        ObjectImpl oi = (ObjectImpl)ref;
	ClientSubcontract rep = null;
        try {
	    // throws BAD_OPERATION if no delegate
	    rep = (ClientSubcontract) oi._get_delegate();  
        } catch ( Exception ex ) {}
        if ( rep == null ) {
	    // no delegate: servant was not connected to ORB
			
	    if (ref instanceof javax.rmi.CORBA.Stub) {
                try {
		    ((javax.rmi.CORBA.Stub)ref).connect(orb);
		} catch (java.rmi.RemoteException e) {
		    throw new MARSHAL(0, CompletionStatus.COMPLETED_MAYBE);
		}
	    } else {
		orb.connect(ref);
	    }
	    rep = (ClientSubcontract) oi._get_delegate();
        }
	
        IOR ior = rep.marshal();
	ior.write(this);
	return;
    }

    // ------------ RMI related methods --------------------------

    public void write_abstract_interface(java.lang.Object obj) {
	boolean corbaObject = false; // Assume value type.
	org.omg.CORBA.Object theObject = null;
	    
	// Is it a CORBA.Object?
	    
	if (obj != null && obj instanceof org.omg.CORBA.Object) {
	        
	    // Yes.
	        
	    theObject = (org.omg.CORBA.Object)obj;
	    corbaObject = true;	        
	}
	    
	// Write our flag...
	    
	write_boolean(corbaObject);
	    
	// Now write out the object...
	    
	if (corbaObject) {
	    write_Object(theObject);
	} else {
	    try {
		write_value((java.io.Serializable)obj);
	    }
	    catch(ClassCastException cce){
		if (obj instanceof java.io.Serializable)
		    throw cce;
		else {
		    Utility.throwNotSerializableForCorba();
				
		}
	    }
	}
    }

    public void write_value(Serializable object, Class clz) {

	write_value(object); 
    }

    public void write_value(Serializable object, String repository_id) {

	// Handle null references
	if (object == null) {
	    // Write null tag and return
	    write_long(0);
	    return;
	}

	// Handle shared references
	if ((valueCache != null) && valueCache.containsKey(object)) {
	    // write indirection tag
	    write_long(0xffffffff);
			
	    // write indirection
	    write_long(valueCache.getVal(object) - get_offset());
	    return;
	} 
		
	Class clazz = object.getClass();
	boolean oldMustChunk = mustChunk;

	// _REVISIT_ should this be done when mustChunk is false?
	if (inBlock)
	    end_block();

	// Handle arrays
	if (clazz.isArray())
	    {
		if (valueHandler == null)
		    valueHandler = createValueHandler();

		if (mustChunk) {
		    // Write value_tag
		    int indirection = writeValueTag(true, false, Util.getCodebase(clazz));
				
		    // Write repository ID
		    write_repositoryId(RepositoryIdWrapper.createSequenceRepID(orb, clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    start_block();
		    end_flag--;
		    valueHandler.writeValue(this, object);
		    end_block();
				
		    // Write end tag
		    writeEndTag(true);
		}
		else {
		    // Write value_tag
		    int indirection = writeValueTag(false, false, Util.getCodebase(clazz));
				
		    // Write repository ID
		    write_repositoryId(RepositoryIdWrapper.createSequenceRepID(orb, clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    end_flag--;
		    valueHandler.writeValue(this, object);
				
		    // Write end tag
		    writeEndTag(false);
		}
			
	    }
	// Handle IDL Value types
	else if (object instanceof org.omg.CORBA.portable.ValueBase)
	    {
		// _REVISIT_ could check to see whether chunking really needed 
		mustChunk = true;
			
		// Write value_tag
		int indirection = writeValueTag(true, false, Util.getCodebase(clazz));
			
		// Get rep id
		String repId = ((ValueBase)object)._truncatable_ids()[0];
			
		// Write rep id
		write_repositoryId(repId);
			
		// Add indirection for object to indirection table
		updateIndirectionTable(indirection, object, object);
			
		// Write Value chunk
		start_block();
		end_flag--;
		writeIDLValue(object, repId);
		end_block();
			
		// Write end tag
		writeEndTag(true);
	    }
	else if (shouldWriteAsIDLEntity(object))
	    {
		writeIDLEntity((IDLEntity)object);
	    }
	else if (object instanceof java.lang.String)
	    {
		if (mustChunk) {
		    // Write value_tag
		    int indirection = writeValueTag(true, false, null);
				
		    // Write WStringValue's repository ID
		    write_repositoryId(RepositoryId.kWStringValueRepID);
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    start_block();
		    end_flag--;
		    write_wstring((java.lang.String)object);
		    end_block();
				
		    // Write end tag
		    writeEndTag(true);
		}
		else {
		    int indirection = writeValueTag(false, false, null);
				
		    // Write WStringValue's repository ID
		    write_repositoryId(RepositoryId.kWStringValueRepID);
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    end_flag--;
		    write_wstring((java.lang.String)object);
				
		    // Write end tag
		    writeEndTag(false);
		}
			
	    }
	else if (object instanceof java.lang.Class)
	    {
                if (ORBUtility.isLegacyORB(
                    (com.sun.corba.se.internal.corba.ORB) orb)) 
                {
                    writeClass(RepositoryId_1_3.kClassDescValueRepID, 
                        (Class)object);
                } else {
                    writeClass(RepositoryId.kClassDescValueRepID, 
                        (Class)object);
                }
	    }
	else // Not a CORBA Value Type
	    {
		if (valueHandler == null)
		    valueHandler = createValueHandler();

		Serializable key = object;
		object = valueHandler.writeReplace(key);
		
		if (object == null) {
		    // Write null tag and return
		    write_long(0);
		    return;
		}
		
		if (object != key) {
		    if ((valueCache != null) && valueCache.containsKey(object)) {
			// write indirection tag
			write_long(0xffffffff);
			
			// write indirection
			write_long(valueCache.getVal(object) - get_offset());
			return;
		    }

		    clazz = object.getClass();
		}

		if (mustChunk || valueHandler.isCustomMarshaled(clazz)) {

		    mustChunk = true;
				
		    // Write value_tag
		    int indirection = writeValueTag(true, false, Util.getCodebase(clazz));
				
		    // Write rep. id
		    write_repositoryId(RepositoryIdWrapper.createForJavaType(orb,clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, key);
				
		    // Write Value chunk
		    end_flag--;
		    start_block();
		    valueHandler.writeValue(this, object);
		    end_block();
				
		    // Write end tag
		    writeEndTag(true);
		}
		else {
		    // Write value_tag
		    int indirection = writeValueTag(false, false, Util.getCodebase(clazz));
				
		    // Write rep. id
		    write_repositoryId(RepositoryIdWrapper.createForJavaType(orb,clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, key);
				
		    // Write Value chunk
		    end_flag--;
		    valueHandler.writeValue(this, object);
				
		    // Write end tag
		    writeEndTag(false);
		}
			
	    }
		
	mustChunk = oldMustChunk;

	// Check to see if we need to start another block for a
	// possible outer value
	if (mustChunk)
	    start_block();

    }

    public void write_value(Serializable object)
    {

	// Handle null references
	if (object == null) {
	    // Write null tag and return
	    write_long(0);
	    return;
	}

	// Handle shared references
	if ((valueCache != null) && valueCache.containsKey(object)) {
	    // write indirection tag
	    write_long(0xffffffff);

	    // write indirection
	    write_long(valueCache.getVal(object) - get_offset());
	    return;
	}
	
	Class clazz = object.getClass();
	boolean oldMustChunk = mustChunk;

	// _REVISIT_ should this be done when mustChunk is false?
	if (inBlock)
	    end_block();

	// Handle arrays
	if (clazz.isArray())
	    {
		if (valueHandler == null)
		    valueHandler = createValueHandler();

		if (mustChunk) {
		    // Write value_tag
		    int indirection = writeValueTag(true, false, Util.getCodebase(clazz));
				
		    // Write repository ID
		    write_repositoryId(RepositoryIdWrapper.createSequenceRepID(orb, clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    start_block();
		    end_flag--;
		    valueHandler.writeValue(this, object);
		    end_block();
				
		    // Write end tag
		    writeEndTag(true);
		}
		else {
		    // Write value_tag
		    int indirection = writeValueTag(false, false, Util.getCodebase(clazz));
				
		    // Write repository ID
		    write_repositoryId(RepositoryIdWrapper.createSequenceRepID(orb, clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    end_flag--;
		    valueHandler.writeValue(this, object);
				
		    // Write end tag
		    writeEndTag(false);
				
		}
			
	    }
	// Handle IDL Value types
	else if (object instanceof org.omg.CORBA.portable.ValueBase)
	    {
		// _REVISIT_ could check to see whether chunking really needed 
		mustChunk = true;
			
		// Write value_tag
		int indirection = writeValueTag(true, false, Util.getCodebase(clazz));
				
		// Get rep id
		String repId = ((ValueBase)object)._truncatable_ids()[0];
			
		// Write rep id
		write_repositoryId(repId);
			
		// Add indirection for object to indirection table
		updateIndirectionTable(indirection, object, object);
			
		// Write Value chunk
		start_block();
		end_flag--;
			
		writeIDLValue(object, repId);
			
		end_block();
			
		writeEndTag(true);
	    }
	else if (shouldWriteAsIDLEntity(object))
	    {
		writeIDLEntity((IDLEntity)object);
	    }
	else if (object instanceof java.lang.String)
	    {
			
		// Write value_tag
		//   Are we nested ?
		if (mustChunk) { 
		    int indirection = writeValueTag(true, false, null);
				
		    // Write WStringValue's repository ID
		    write_repositoryId(RepositoryId.kWStringValueRepID);
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    start_block();
		    end_flag--;
		    write_wstring((java.lang.String)object);
		    end_block();
				
		    // Write end tag
		    writeEndTag(true);
		}
		else {
		    int indirection = writeValueTag(false, false, null);
				
		    // Write WStringValue's repository ID
		    write_repositoryId(RepositoryId.kWStringValueRepID);
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, object);
				
		    // Write Value chunk
		    end_flag--;
		    write_wstring((java.lang.String)object);
				
		    // Write end tag
		    writeEndTag(false);
				
		}
	    }
	else if (object instanceof java.lang.Class)
	    {
                if (ORBUtility.isLegacyORB(
                    (com.sun.corba.se.internal.corba.ORB) orb)) {
                    writeClass(RepositoryId_1_3.kClassDescValueRepID, 
                        (Class)object);
                } else {
                    writeClass(RepositoryId.kClassDescValueRepID, 
                       (Class)object);
                }

	    }
	else // Not a CORBA Value Type
	    {
		if (valueHandler == null)
		    valueHandler = createValueHandler();

		Serializable key = object;
		object = valueHandler.writeReplace(key);
		
		if (object == null) {
		    // Write null tag and return
		    write_long(0);
		    return;
		}
		
		if (object != key) {
		    if ((valueCache != null) && valueCache.containsKey(object)) {
			// write indirection tag
			write_long(0xffffffff);
			
			// write indirection
			write_long(valueCache.getVal(object) - get_offset());
			return;
		    }

		    clazz = object.getClass();
		}

		if (mustChunk || valueHandler.isCustomMarshaled(clazz)) {

		    mustChunk = true;

		    // Write value_tag
		    int indirection = writeValueTag(true, false, Util.getCodebase(clazz));
				
		    // Write rep. id
		    write_repositoryId(RepositoryIdWrapper.createForJavaType(orb,clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, key);
				
		    // Write Value chunk
		    end_flag--;
		    start_block();
				
		    valueHandler.writeValue(this, object);
				
		    end_block();
				
		    // Write end tag
		    writeEndTag(true);
		}
		else {
		    // Write value_tag
		    int indirection = writeValueTag(false, false, Util.getCodebase(clazz));
				
		    // Write rep. id
		    write_repositoryId(RepositoryIdWrapper.createForJavaType(orb,clazz));
				
		    // Add indirection for object to indirection table
		    updateIndirectionTable(indirection, object, key);
				
		    // Write Value chunk
		    end_flag--;
				
		    valueHandler.writeValue(this, object);
				
		    // Write end tag
		    writeEndTag(false);
				
		}
	    }
		
	mustChunk = oldMustChunk;

	// Check to see if we need to start another block for a
	// possible outer value
	if (mustChunk)
	    start_block();
		
    }

    public void write_value(Serializable object, org.omg.CORBA.portable.BoxedValueHelper factory)
    {
        // Handle null references
        if (object == null) {
            // Write null tag and return
            write_long(0);
            return;
        }
        
        // Handle shared references
	if ((valueCache != null) && valueCache.containsKey(object)) {
	    // write indirection tag
	    write_long(0xffffffff);

	    // write indirection
	    write_long(valueCache.getVal(object) - get_offset());
	    return;
	} 

	boolean oldMustChunk = mustChunk;
	boolean isCustom = false;
	if (factory instanceof ValueHelper) {
	    short modifier;
	    try {
		modifier = ((ValueHelper)factory).get_type().type_modifier();
	    } catch(BadKind ex) {  // tk_value_box
		modifier = VM_NONE.value;
	    }  
	    if (object instanceof CustomMarshal &&
	        modifier == VM_CUSTOM.value) {
		isCustom = true;
		mustChunk = true;
	    }
	    if (modifier == VM_TRUNCATABLE.value)
		mustChunk = true;
	}

	if (mustChunk) {
			
	    if (inBlock)
		end_block();
			
	    // Write value_tag
	    int indirection = writeValueTag(true, false, Util.getCodebase(object.getClass()));
			
	    write_repositoryId(factory.get_id());
			
	    // Add indirection for object to indirection table
	    updateIndirectionTable(indirection, object, object);
			
	    // Write Value chunk
	    start_block();
	    end_flag--;
	    if (isCustom)
		((CustomMarshal)object).marshal(this);
	    else 
		factory.write_value(this, object);
	    end_block();
			
	    // Write end tag
	    writeEndTag(true);
	}
	else {
	    // Write value_tag
	    int indirection = writeValueTag(false, false, Util.getCodebase(object.getClass()));
			
	    write_repositoryId(factory.get_id());
			
	    // Add indirection for object to indirection table
	    updateIndirectionTable(indirection, object, object);
			
	    // Write Value chunk
	    end_flag--;
	    // no need to test for custom on the non-chunked path
	    factory.write_value(this, object);
			
	    // Write end tag
	    writeEndTag(false);
	}

	mustChunk = oldMustChunk;

	// Check to see if we need to start another block for a
	// possible outer value
	if (mustChunk)
	    start_block();

    }
	
    public int get_offset() {
	return offset + size;
    }

    public void start_block() {

	inBlock = true;

	// Save space in the buffer for block size
	write_long(0);

	// Remember where to put the size of the endblock less 4
	blockSizeIndex = size;

    }

    public void end_block() {

	if (!inBlock)
	    return;

	inBlock = false;

	// Test to see if the block was of zero length
	// If so, remove the block instead of ending it
	// (This can happen if the last field written 
	//  in a value was another value)
	if (size == blockSizeIndex){
	    size = size - 4;
	    blockSizeIndex = -1;
	    return;
	}

	int oldSize = size;
	size = blockSizeIndex - 4;
	write_long(oldSize - blockSizeIndex);
	size = oldSize;
	blockSizeIndex = -1;
    }
    
    public org.omg.CORBA.ORB orb()
    {
        return orb;    
    }

    // ------------ End RMI related methods --------------------------
    
    public final void write_boolean_array(boolean[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        if (size + length >= buflen)
            alignAndReserve(1, length);
    	for(int i=0; i < length; i++) {
	    buf[size++] = (value[i+offset]? (byte)1:(byte)0);
    	}
    }

    public final void write_char_array(char[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        if (size + length >= buflen)
            alignAndReserve(1, length);
    	for(int i=0; i < length; i++) {
    	    char x = value[i+offset];

    	    if ( x > '\u00ff' )
    		throw new DATA_CONVERSION(com.sun.corba.se.internal.util.MinorCodes.CHARACTER_OUTOFRANGE,
					  CompletionStatus.COMPLETED_NO);

	    buf[size++] = (byte)(x & 0xFF);
    	}
    }

    public final void write_wchar_array(char[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        alignAndReserve(2, length * 2);
        if (littleEndian) {
    	    for(int i=0; i < length; i++) {
    		writeLittleEndianWchar(value[i+offset]);
    	    }
    	} else {
    	    for(int i=0; i < length; i++) {
    		writeBigEndianWchar(value[i+offset]);
    	    }
    	}
    }

    public final void write_short_array(short[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        alignAndReserve(2, length * 2);
        if (littleEndian) {
    	    for(int i=0; i < length; i++) {
    		writeLittleEndianShort(value[i+offset]);
    	    }
    	} else {
    	    for(int i=0; i < length; i++) {
    		writeBigEndianShort(value[i+offset]);
    	    }
    	}
    }

    public final void write_ushort_array(short[]value, int offset, int length) {
    	write_short_array(value, offset, length);
    }

    public final void write_long_array(int[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        alignAndReserve(4, length * 4);
        if (littleEndian) {
    	    for(int i=0; i < length; i++) {
    		writeLittleEndianLong(value[i+offset]);
    	    }
    	} else {
    	    for(int i=0; i < length; i++) {
    		writeBigEndianLong(value[i+offset]);
    	    }
    	}
    }

    public final void write_ulong_array(int[]value, int offset, int length) {
    	write_long_array(value, offset, length);
    }

    public final void write_longlong_array(long[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        alignAndReserve(8, length * 8);
        if (littleEndian) {
    	    for(int i=0; i < length; i++) {
    		writeLittleEndianLongLong(value[i+offset]);
    	    }
    	} else {
    	    for(int i=0; i < length; i++) {
    		writeBigEndianLongLong(value[i+offset]);
    	    }
    	}
    }

    public final void write_ulonglong_array(long[]value, int offset, int length) {
    	write_longlong_array(value, offset, length);
    }

    public final void write_float_array(float[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        alignAndReserve(4, length * 4);
        if (littleEndian) {
    	    for(int i=0; i < length; i++) {
    		writeLittleEndianLong(Float.floatToIntBits(value[i+offset]));
    	    }
    	} else {
    	    for(int i=0; i < length; i++) {
    		writeBigEndianLong(Float.floatToIntBits(value[i+offset]));
    	    }
    	}
    }

    public final void write_double_array(double[]value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(com.sun.corba.se.internal.util.MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);

        alignAndReserve(8, length * 8);
        if (littleEndian) {
    	    for(int i=0; i < length; i++) {
    		writeLittleEndianLongLong(Double.doubleToLongBits(value[i+offset]));
    	    }
    	} else {
    	    for(int i=0; i < length; i++) {
    		writeBigEndianLongLong(Double.doubleToLongBits(value[i+offset]));
    	    }
    	}
    }

    public void write_string_array(String[] value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);
	    
    	for(int i=0; i < length; i++) {
    	    write_string(value[i+offset]);
    	}
    }
    
    public void write_wstring_array(String[] value, int offset, int length) {
        if ( value == null )
	    throw new BAD_PARAM(MinorCodes.NULL_PARAM, CompletionStatus.COMPLETED_MAYBE);
	    
    	for(int i=0; i < length; i++) {
    	    write_wstring(value[i+offset]);
    	}
    }

    public final void write_any_array(org.omg.CORBA.Any value[], int offset, int length)
    {
    	for(int i=0; i < length; i++) {
    	    write_any(value[i+offset]);
    	}

    }

    //--------------------------------------------------------------------//
    // CDROutputStream state management.
    //

    public final void reset()
    {
        offset += size;
    	size = 0;
	end_flag = 0;
	end_flag_index = 0;
    }

    /**
     * Write the contents of the CDROutputStream to the specified output stream.
     * @param s The output stream to write to.
     */
    public void writeTo(java.io.OutputStream s) throws java.io.IOException {
	s.write(buf, 0, size);
    }

    public void writeOctetSequenceTo(org.omg.CORBA.portable.OutputStream s) {
    	s.write_long(size);
    	s.write_octet_array(buf, 0, size);
    }

    public final int getSize() {
    	return size;
    }

    protected byte[] getBuffer() {
        return buf;
    }

    private final void updateIndirectionTable(int indirection, java.lang.Object object,
                                              java.lang.Object key) {
	// int indirection = get_offset();
	if (valueCache == null)
	    valueCache = new CacheTable(true);
	valueCache.put(object, indirection);
	if (key != object)
	    valueCache.put(key, indirection);
    }

    private final void write_repositoryId(String id) {
        if (repositoryIdCache != null && repositoryIdCache.containsKey(id)) {
            write_long(0xffffffff);
            write_long(repositoryIdCache.getVal(id) - get_offset());
	    return;
        }

	// align to the next 4 byte boundary
        int incr = size & 3;
        if (incr != 0) {
            incr = (4 - incr);
            size += incr;
        }
        byte[] idBytes = RepositoryId.getByteArray(id);
        if (idBytes == null) {
	    int len = id.length() + 1;
	    idBytes = new byte[4 + len];
	    int offset = 0;
    	    if (littleEndian) {
    		idBytes[offset++] = (byte)(len & 0xFF);
    		idBytes[offset++] = (byte)((len >>> 8) & 0xFF);
    		idBytes[offset++] = (byte)((len >>> 16) & 0xFF);
    		idBytes[offset++] = (byte)((len >>> 24) & 0xFF);
    	    } else {
    		idBytes[offset++] = (byte)((len >>> 24) & 0xFF);
    		idBytes[offset++] = (byte)((len >>> 16) & 0xFF);
    		idBytes[offset++] = (byte)((len >>> 8) & 0xFF);
    		idBytes[offset++] = (byte)(len & 0xFF);
    	    }
    	    len--;
	    for (int i=0; i < len; i++) {
        	char x = id.charAt(i);

        	if ( x > '\u00ff' )
        	    throw new DATA_CONVERSION(com.sun.corba.se.internal.util.MinorCodes.CHARACTER_OUTOFRANGE,
					      CompletionStatus.COMPLETED_NO);

        	idBytes[offset++] = (byte)((x >>> 0) & 0xFF);
	    }
    	    idBytes[offset++] = 0;
    	    RepositoryId.setByteArray(id, idBytes);
	}

	if (repositoryIdCache == null)
	    repositoryIdCache = new CacheTable(true);
	repositoryIdCache.put(id, get_offset());
	write_octet_array(idBytes, 0, idBytes.length);
    }

    private void write_codebase(String str, int pos) {
        if (codebaseCache != null && codebaseCache.containsKey(str)) {
            write_long(0xffffffff);
            write_long(codebaseCache.getVal(str) - get_offset());
        }
        else {
	    write_string(str);
            if (codebaseCache == null)
        	codebaseCache = new CacheTable(true);
            codebaseCache.put(str, pos);
        }
    }

    private final int writeValueTag(boolean chunkIt, boolean repNotWritten, 
				    String codebase) {
	int indirection = 0;

	if (chunkIt && repNotWritten){
	    if (codebase == null) {
		write_long(RepositoryId.kPreComputed_StandardRMIChunked_NoRep);
		indirection = get_offset() - 4;
	    } else {			
		write_long(RepositoryId.kPreComputed_CodeBaseRMIChunked_NoRep);
		indirection = get_offset() - 4;
		write_codebase(codebase, get_offset());
	    }
	} else if (chunkIt && !repNotWritten){
	    if (codebase == null) {
		write_long(RepositoryId.kPreComputed_StandardRMIChunked);
		indirection = get_offset() - 4;
	    } else {			
		write_long(RepositoryId.kPreComputed_CodeBaseRMIChunked);
		indirection = get_offset() - 4;
		write_codebase(codebase, get_offset());
	    }
	} else {
	    if (codebase == null) {
		write_long(RepositoryId.kPreComputed_StandardRMIUnchunked);
		indirection = get_offset() - 4;
	    } else {			
		write_long(RepositoryId.kPreComputed_CodeBaseRMIUnchunked);
		indirection = get_offset() - 4;
		write_codebase(codebase, get_offset());
	    }
	}

        return indirection;
    }

    private void writeIDLValue(Serializable object, String repID)
    {
    	if (object instanceof StreamableValue) {
	    ((StreamableValue)object)._write(this);

	} else if (object instanceof CustomValue) {
	    ((CustomValue)object).marshal(this);

	} else {
	    BoxedValueHelper helper = Utility.getHelper(object.getClass(), null, repID);
	    boolean isCustom = false;
	    if (helper instanceof ValueHelper && object instanceof CustomMarshal) {
		try {
		    if (((ValueHelper)helper).get_type().type_modifier() == VM_CUSTOM.value)
		        isCustom = true;
	        } catch(BadKind ex) {
		    throw new org.omg.CORBA.MARSHAL();
		}  
	    }
	    if (isCustom)
		((CustomMarshal)object).marshal(this);
	    else
		helper.write_value(this, object);
	}
    }

    // Handles end tag compaction...
    private void writeEndTag(boolean chunked){
		
	if (chunked) {
	    if (size == end_flag_index) {
		size -= 4; 
	    }
			
	    write_long(end_flag);
	}
	end_flag_index = size;
	end_flag++;

		
    }

    private void writeClass(String repository_id, Class clz) {
			
	if (mustChunk) {
	    // Write value_tag
	    int indirection = writeValueTag(true, false, null);
	    updateIndirectionTable(indirection, clz, clz);
            			
	    // Write repository ID
	    if (repository_id != null)
		write_repositoryId(repository_id);
	    else {
                if (ORBUtility.isLegacyORB(
                    (com.sun.corba.se.internal.corba.ORB) orb)) 
                {
                    write_repositoryId(
                        RepositoryId_1_3.kClassDescValueRepID);
                } else {
                    write_repositoryId(
                        RepositoryId.kClassDescValueRepID);
                }
            }

			
	    // Write Value chunk
	    start_block();
	    end_flag--;
	    write_value(RepositoryIdWrapper.createForAnyType(orb,clz));
	    write_value(Util.getCodebase(clz));
	    end_block();
			
	    // Write end tag
	    writeEndTag(true);
	}
	else {
	    // Write value_tag
	    int indirection = writeValueTag(false, false, null);
	    updateIndirectionTable(indirection, clz, clz);
			
	    // Write repository ID
	    if (repository_id != null)
		write_repositoryId(repository_id);
	    else {
                if (ORBUtility.isLegacyORB(
                    (com.sun.corba.se.internal.corba.ORB) orb)) 
                {
                    write_repositoryId(
                        RepositoryId_1_3.kClassDescValueRepID);
                } else {
                    write_repositoryId(
                        RepositoryId.kClassDescValueRepID);
                }
            }
	    // Write Value chunk
	    end_flag--;
	    write_value(RepositoryIdWrapper.createForAnyType(orb,clz));
	    write_value(Util.getCodebase(clz));
			
	    // Write end tag
	    writeEndTag(false);
	}
    }

    // Casts and returns an Object as a Serializable
    // This is required for JDK 1.1 only to avoid VerifyErrors when
    // passing arrays as Serializable
    // private java.io.Serializable make_serializable(java.lang.Object object)
    // {
    //	 return (java.io.Serializable)object;
    // }

    private boolean shouldWriteAsIDLEntity(Serializable object)
    {
	return ((object instanceof IDLEntity) && (!(object instanceof ValueBase)) &&
		(!(object instanceof org.omg.CORBA.Object)));
			
    }
	
    private void writeIDLEntity(IDLEntity object) {

	// _REVISIT_ could check to see whether chunking really needed 
	mustChunk = true;

	String repository_id = RepositoryIdWrapper.createForJavaType(orb,object);
	Class clazz = object.getClass();
	String codebase = Util.getCodebase(clazz); 
		
	// Write value_tag
	int indirection = writeValueTag(true, false, codebase);
	updateIndirectionTable(indirection, object, object);
		
	// Write rep. id
	write_repositoryId(repository_id);
		
	// Write Value chunk
	end_flag--;
	start_block();

	// Write the IDLEntity using reflection 
	try {
            ClassLoader clazzLoader = (clazz == null ? null : clazz.getClassLoader());
	    Class helperClass = Utility.loadClassForClass(clazz.getName()+"Helper", codebase,
                                                   clazzLoader, clazz, clazzLoader);
	    Class argTypes[] = {org.omg.CORBA.portable.OutputStream.class, clazz};
	    Method writeMethod = helperClass.getDeclaredMethod(kWriteMethod, argTypes);
	    java.lang.Object args[] = {this, object};
	    writeMethod.invoke(null, args);
	} catch (ClassNotFoundException cnfe) {
	    throw new org.omg.CORBA.MARSHAL(cnfe.getMessage());
	} catch(NoSuchMethodException nsme) {
	    throw new org.omg.CORBA.MARSHAL(nsme.getMessage());
	} catch(IllegalAccessException iae) {
	    throw new org.omg.CORBA.MARSHAL(iae.getMessage());
	} catch(InvocationTargetException ite) {
	    throw new org.omg.CORBA.MARSHAL(ite.getMessage());
	}
	end_block();
		
	// Write end tag
	writeEndTag(true);
    }
    
    /* DataOutputStream methods */

    public void write_Abstract (java.lang.Object value) {
        write_abstract_interface(value);
    }

    public void write_Value (java.io.Serializable value) {
        write_value(value);
    }

    private static String _id = "IDL:omg.org/CORBA/DataOutputStream:1.0";
    private static String[] _ids = { _id };

    public String[] _truncatable_ids() {
        return _ids;
    }

    /* for debugging */

    public void printBuffer() {
	for (int i = 0; i < size; i += 16) {
	    for (int j = 0; j < 16; j++) {
	        int k = buf[i+j];
		if (k < 0)
		    k = 256+k;
                String hex = Integer.toHexString(k);
		if (hex.length() == 1)
		    hex = "0"+hex;
		System.out.print(hex+" ");
	    }
	    System.out.println(new String(buf, i, 16));
	}
    }

    // Isolates the versioning of the value handler
    private ValueHandler createValueHandler()
    {
        if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
            return new ValueHandlerImpl_1_3(); // d4365188
        } else {
            return Util.createValueHandler(); //d11638
        }
    }
}
