/*
 * @(#)ClientRequestImpl.java	1.30 01/02/09
 *
 * Copyright 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import com.sun.corba.se.internal.core.Request;
import com.sun.corba.se.internal.core.ClientRequest;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.ClientResponse;
import com.sun.corba.se.internal.core.GIOPVersion;
import com.sun.corba.se.internal.iiop.Connection;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.ORBVersionFactory;
import com.sun.corba.se.internal.orbutil.ORBConstants;
import com.sun.corba.se.internal.orbutil.ORBUtility;

public class ClientRequestImpl extends IIOPOutputStream implements ClientRequest {

    public ClientRequestImpl( byte[] objectKey, String operationName, 
                              boolean oneway, ServiceContexts svc, 
                              int requestId, Connection conn) 
    {
	super(conn.getORB(), conn);

	this.isOneway = oneway;
	boolean responseExpected = !isOneway;
        
        // we need to set the ORB version we need to talk to, based on the
        // magic in the object key.  This is for interoperability purposes
        // with the ORB that was shipped with the wrong Repository Ids 
        // calcs.
        int magic = ORBUtility.bytesToInt(objectKey, 0);

        // set the ORB version associated with this request
        com.sun.corba.se.internal.corba.ORB orb = conn.getORB();

        orb.setORBVersion(ORBVersionFactory.createFromMagic(magic));

	this.request = new RequestMessage(svc, requestId, responseExpected,
					  objectKey, operationName, null, 
					  ((com.sun.corba.se.internal.corba.ORB)orb).transportDebugFlag);
	request.write(this);
	setMessage(request);

	// mark beginning of msg body for possible later use
	bodyBegin = size;
    }

    public int getRequestId() {
	return request.getRequestId();
    }
    
    public boolean isOneWay() {
	return isOneway;
    }

    public ServiceContexts getServiceContexts() {
	return request.getServiceContexts();
    }

    public String getOperationName() {
	return request.getOperation();
    }

    public byte[] getObjectKey() {
	return request.getObjectKey();
    }
    
    public byte[] getBody(){
    	RequestMessage req = (RequestMessage)msg;
	CDROutputStream newCDROS = new CDROutputStream(orb);
	req.write(newCDROS);
	byte[] headerBytes = newCDROS.toByteArray();
	int headerlen = headerBytes.length;
        if(size <= headerlen) return null;
        byte[] body     = new byte[size - headerlen];
        System.arraycopy(buf,headerlen,body,0,size-headerlen);
        return body;
    }

    public ClientResponse invoke() 
    {
        return (ClientResponse)super.invoke(isOneway);
    }

    /**
     * Resend the invocation to the new IOR, after a location forward.
     */
    public ClientResponse reInvoke(IOR[] iorHolder)
    {
	IOR ior = iorHolder[0];
	int[] bodyBeginHolder = new int[1];
	bodyBeginHolder[0] = bodyBegin;
	ClientResponse resp;
	while (true) { // Keep invoking if there are multiple location forwards
	    resp = (ClientResponse) super.reInvoke(ior, bodyBeginHolder);
	    if ( !resp.isLocationForward() )
		break;
	    ior = resp.getForwardedIOR();
	    bodyBegin = bodyBeginHolder[0] ;
	}
	iorHolder[0] = ior;
	return resp;
    }

    /**
     * Resend the invocation to the new IOR, 
     * only once after a location forward.
     */
    public ClientResponse reInvokeOnce(IOR[] iorHolder)
    {
	IOR ior = iorHolder[0];
	int[] bodyBeginHolder = new int[1];
	bodyBeginHolder[0] = bodyBegin;
	ClientResponse resp;
	resp = (ClientResponse) super.reInvoke(ior, bodyBeginHolder);
	
	if (resp.isLocationForward()){ 
	    ior = resp.getForwardedIOR();
	    iorHolder[0] = ior;
	}

	bodyBegin = bodyBeginHolder[0] ;

	return resp;
    }


    /**
     * Check to see if request is local.
     */
    public boolean isLocal(){
        return false;
    }

    /**
     * Get the GIOPVersion from the Request Message.
     */
    public GIOPVersion getGIOPVersion( ) {
        return this.request.getGIOPVersion();
    }

    private RequestMessage request;
    private int bodyBegin;
    protected boolean isOneway;
}
