/*
 * @(#)Connection.java	1.60 00/02/11
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.iiop;

import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.rmi.server.RMISocketFactory;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.OBJECT_NOT_EXIST;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.INTERNAL;

import org.omg.CORBA.portable.*;

import com.sun.corba.se.internal.core.ServerGIOP;
import com.sun.corba.se.internal.core.EndPoint;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.MarshalOutputStream;
import com.sun.corba.se.internal.util.MinorCodes; //d11638
import com.sun.corba.se.internal.util.Utility; //d11638

/** 
 * Common connection base class.
 */
abstract public class Connection {

    // Connection close states
    public static final int CONN_ABORT = 1;
    public static final int CONN_REBIND = 2;

    protected ORB orb;
    protected Socket socket;    // The socket used for this connection.
    protected long timeStamp = 0;
    protected boolean isServer = false;
    protected ConnectionTable connectionTable = null; 

    void dprint(String msg) {
	Utility.dprint(this, msg);
    }

    public ORB getORB() {
        return orb;
    }

    Socket getSocket() {
	return socket;
    }


    public static Socket newSocket(ORB orb, EndPoint ep)
        throws IOException
    {
        RMISocketFactory factory = ep.getSocketFactory();

        return factory.createSocket(ep.getHostName(), ep.getPort());
    }

    abstract public IIOPInputStream invoke(IIOPOutputStream s)
	throws SystemException;

    abstract public void delete();

    abstract public java.io.InputStream getInputStream();

    abstract public ServerGIOP getServerGIOP();

    abstract public IIOPInputStream send(IIOPOutputStream s, boolean oneWay);
    
    abstract public void sendReply(MarshalOutputStream s) throws Exception;

    abstract public IIOPInputStream createInputStream() throws Exception;


    // All of Following abstract required for Connection cleanup

    abstract public void cleanUp() throws java.lang.Exception;

    abstract public boolean isBusy();

    abstract public void requestBegins();
    abstract public void requestEnds();

    abstract public void print();

    abstract public void setConnection(Socket _socket, ConnectionTable ctab) 
	throws java.lang.Exception;
    abstract public void abortConnection();

    // Indicates whether or not ServiceContexts have even been exchange yet
    abstract public boolean isPostInitialContexts();

    // Sets to true the state that ServiceContexts have indeed been exchanged
    // once already
    abstract public void setPostInitialContexts();

    public IOR locate(int id, byte [] key)
    {
        LocateRequestMessage msg;
        IIOPOutputStream os;
        IIOPInputStream is;

        msg = new LocateRequestMessage(id, key, orb.transportDebugFlag);
        os = orb.newOutputStream(this);
        msg.write(os);
        os.setMessage(msg);

        is = send(os, false);

        LocateReplyMessage reply;
        reply = (LocateReplyMessage)is.getMessage();
        switch (reply.getRequestStatus()) {
	case Message.GIOPLocateReplyUnknownObject:
            throw new OBJECT_NOT_EXIST( MinorCodes.LOCATE_UNKNOWN_OBJECT,
					CompletionStatus.COMPLETED_NO );

	case Message.GIOPLocateReplyObjectHere:
            return null;

	case Message.GIOPLocateReplyObjectForward: {
            IOR ior;

            ior = new IOR(orb);
            ior.read(is);

            return ior;
	}
        }

        throw new INTERNAL( MinorCodes.BAD_LOCATE_REQUEST_STATUS,
            		    CompletionStatus.COMPLETED_NO );
    }

    // cannot cache this value and return it since it is a securiy hole
    // any unprivileged client can get access to the localHost information
    // therefore we are removing this variable from here and doing explicit
    // calls to getLocalHost each time
    // protected static String localHostString = null;
    
    public static String getLocalHost() {
        try {
            // _REVISIT_ We would much prefer to get the fully qualified
            // name using the following code, but this will hang on some
            // machine configurations when not connected to the net.
            //
            //    localHostString = (InetAddress.getByName((InetAddress.getLocalHost()).getHostAddress())).getHostName();
            //
            // Instead, we will use the IP address...
                
            return InetAddress.getLocalHost().getHostAddress();
            
        } catch (Exception ex) {
            throw new INTERNAL( MinorCodes.GET_LOCAL_HOST_FAILED,
				CompletionStatus.COMPLETED_NO );
        }
    }

    public void stampTime() {
	connectionTable.stampTime(this);
    }

}
