/*
 * @(#)EndPointImpl.java	1.20 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import java.lang.reflect.Constructor;
import java.rmi.server.RMISocketFactory;

import com.sun.corba.se.internal.core.EndPoint;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.Profile;

public class EndPointImpl implements EndPoint {
    public EndPointImpl(int type, int port, String hostname) {
	this.type = type;
	this.port = port;
	this.hostname = hostname;
    }

    EndPointImpl(ORB orb, IOR ior) {
        Profile p = ior.getProfile();
        hostname = p.getHost().toLowerCase();
        port = p.getPort();
        if (p.getTaggedComponentsLength() != 0) {
            try {
                Constructor cons;
                Class[] params;

                params = new Class[]{org.omg.CORBA.ORB.class, String.class};
                cons = orb.getSocketFactory().getClass().getConstructor(params);
                Object[] args = new Object[] {orb, ior.stringify()};
                factory = (RMISocketFactory)cons.newInstance(args);
                return;
            } catch (Exception e) {
                // fallback on default socket factory
            }
        }
        factory = (RMISocketFactory)orb.getSocketFactory();
    }

    public int getType() {
	return type;
    }

    public int getPort() {
	return port;
    }

    public String getHostName() {
	return hostname;
    }

    public RMISocketFactory getSocketFactory() {
        return factory;
    }

    public int hashCode() {
        int hash = 0;

        if (factory != null) {
            hash = factory.hashCode();
        }
        return hash ^ hostname.hashCode() ^ port;
    }

    public boolean equals(Object obj) {
        if (!(obj instanceof EndPointImpl))
            return false;
        EndPointImpl other = (EndPointImpl)obj;
        if (port != other.port)
            return false;
        if (!hostname.equals(other.hostname))
            return false;
        if ((factory == null) ^ (other.factory == null))
            return false;
        if (factory != null) {
            if (!factory.equals(other.factory))
                return false;
        }
        return true;
    }

    private int type;
    private int port;
    private String hostname;
    private RMISocketFactory factory;
}
