/*
 * @(#)GIOPImpl.java	1.36 00/02/11
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import java.net.*;
import java.util.*;

import com.sun.corba.se.internal.core.ClientGIOP;
import com.sun.corba.se.internal.core.ServerGIOP;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.RequestHandler;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.ClientRequest;
import com.sun.corba.se.internal.core.EndPoint;
import com.sun.corba.se.internal.core.Profile;
import com.sun.corba.se.internal.corba.ClientDelegate;

import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.CompletionStatus;

import com.sun.corba.se.internal.util.MinorCodes;  //d11638
import java.net.InetAddress;

import java.security.AccessController;
import java.security.PrivilegedAction;

public class GIOPImpl implements ClientGIOP, ServerGIOP {

    // Start at some value other than zero since this is a magic
    // value in some protocols.
    private int requestId = 5;

    protected synchronized int getNextRequestId() {
	return requestId++;
    }

    protected ConnectionTable table;
    protected ORB orb;
    
    public GIOPImpl(ORB orb, RequestHandler handler) {
	this.orb = orb;
	this.table = new ConnectionTable(orb, this);
	setRequestHandler(handler);
    }

    public Connection getConnection( EndPoint ep ) {
	return table.get( ep ) ;
    }

    public void deleteConnection( EndPoint ep ) {
        try
        {
                table.deleteConn( ep ) ;
        }
        catch( Exception e )
        {
                // Ignore the exception
        }
    }


    /**
     * Allocate a new request id.
     */
    public int allocateRequestId()
    {
	return getNextRequestId();
    }

    private RequestHandler handler;
    
    public void setRequestHandler(RequestHandler handler) {
	this.handler = handler;
    }

    public RequestHandler getRequestHandler() {
	return handler;
    }

    public IOR locate(IOR ior)
    {
	Profile iop = ior.getProfile();
	byte[]  objKey = iop.getObjectKey();

        Connection c = table.get(ior.getEndpoint());
	IOR newIOR = c.locate(getNextRequestId(), objKey);
	if ( newIOR == null ) // we got an OBJECT_HERE reply
	    newIOR = ior;
	return newIOR;
    }

    /****************************************************************************
    The following methods are implementations of the ServerGIOP's endpoint APIs.
    ****************************************************************************/

    protected int listenerPort;
    protected Vector endPoints = new Vector();
    protected ListenerThread listenerThread;
    private EndPoint bootstrapEndpoint;


    public synchronized void initEndpoints() {
	if (endPoints.size() == 0) // create a default endpoint
	    getEndpoint(EndPoint.CLEAR_TEXT, 0, null);
    }

    public synchronized EndPoint getDefaultEndpoint() {
	if (endPoints.size() == 0)
	    return null;
	return (EndPoint) endPoints.elementAt(0);
    }

    /**
     * Get an EndPoint for the specified type, listenPort and local address.
     * type must be one of CLEAR_TEXT or SSL.
     * If listenPort == 0, a listening port will be assigned.
     * If addr == null, InetAddress.getLocalHost() is used.
     * If an EndPoint at the specified port/address does not exist,
     * then it will be created.
     */
    public synchronized EndPoint getEndpoint(int type, int port,
				             InetAddress addr)
    {
	// Check if this endpoint was already created
        String host = null;
        if (addr != null)
            host = addr.getHostName().toLowerCase();
	Enumeration eps = endPoints.elements();
	while ( eps.hasMoreElements() ) {
	    EndPoint ep = (EndPoint)eps.nextElement();
	    if ( ep.getType() == type &&
                 ep.getPort() == port &&
                 ep.getHostName().equals(host) )
            {
		return ep;
            }
	}

	if (port == 0 && orb.getORBServerPort() != 0)
	    port = orb.getORBServerPort();

	// Create the endpoint
	listenerThread = createListener(port);
	ServerSocket listenSocket = listenerThread.getSocket();
	listenerPort = listenSocket.getLocalPort();
	EndPoint ep = new EndPointImpl(type, listenerPort,
				       orb.getORBServerHost());
	endPoints.addElement(ep);

	return ep;
    }

    /**
     * Get an EndPoint for the bootstrap naming service, at the specified port.
     * If listenPort == 0 and this is the first time getBootstrapEndpoint was
     * invoked, then an EndPoint at the default port of 900 will be created.
     * If listenPort == 0 and this is not the first time getBootstrapEndpoint
     * was called, then the existing bootstrap EndPoint will be returned.
     */
    public synchronized EndPoint getBootstrapEndpoint(int port)
    {
	if (bootstrapEndpoint == null)
	    bootstrapEndpoint = getEndpoint(EndPoint.CLEAR_TEXT, port, null);

	return bootstrapEndpoint;
    }

    private synchronized ListenerThread createListener(final int port) {
	final ServerSocket ss;
        ListenerThread lis;
        final ConnectionTable finalTable = table;
        try {
            ss = orb.getSocketFactory().createServerSocket(port);
            lis = (ListenerThread) AccessController.doPrivileged(new PrivilegedAction() {
                public java.lang.Object run() {
                    ListenerThread thread = new ListenerThread(finalTable, ss);
                    thread.setDaemon(true);
                    return thread;
                }
            });
            lis.start();
        }
        catch (java.lang.Exception e) {
            throw new INTERNAL(MinorCodes.CREATE_LISTENER_FAILED,
			       CompletionStatus.COMPLETED_NO);
        }
        return lis;
    }
}
