/*
 * @(#)IIOPInputStream.java	1.36 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import java.io.IOException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.Object;
import org.omg.CORBA.Principal;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.CompletionStatus;

import com.sun.corba.se.internal.util.MinorCodes;
import com.sun.corba.se.internal.util.Utility;

import com.sun.org.omg.SendingContext.CodeBase;

public class IIOPInputStream extends CDRInputStream
{
    private static final int DEFAULT_BUFFER_SIZE = 1024;
    protected Connection conn;
    protected Message msg;

    private void dprint( String msg ) {
	Utility.dprint( this, msg ) ;
    }

    public IIOPInputStream() {
    }

    public IIOPInputStream(ORB orb, Connection c) {
	super(orb, null, 0);
	this.conn = c;
    }

    public IIOPInputStream(ORB orb, byte[] msgbuf, int msgsize, boolean endian,
			   Message msg)
    {
	super(orb, msgbuf, msgsize, endian);
	this.msg = msg;
    }

    public IIOPInputStream(Connection c, byte[] msgbuf, Message header)
        throws java.io.IOException
    {
        this(c.getORB(), c);

	com.sun.corba.se.internal.corba.ORB theORB = (com.sun.corba.se.internal.corba.ORB)(c.getORB()) ;
	if (theORB.transportDebugFlag)
	    dprint( "Constructing IIOPInputStream object" ) ;
	
        this.buf = msgbuf;
        this.msg = header;

        this.index = Message.GIOPMessageHeaderLength;
        this.littleEndian = msg.isLittleEndian();

        size = msg.getSize();
        if (size > buf.length) {
            byte[] old;
 
            old = buf;
            buf = new byte[size];
            System.arraycopy(old, 0, buf, 0, old.length);
        }

	if (theORB.transportDebugFlag)
	    dprint( "Reading the fully message" ) ;
	
        Message.readFully(c.getInputStream(), buf, index, size - index);

	if (theORB.transportDebugFlag)
	    dprint( "Unmarshalling the GIOP message" ) ;
	
        // Unmarshal the extended GIOP message from the buffer.
        msg.read(this);

	if (theORB.transportDebugFlag)
	    dprint( "Setting the time stamp" ) ;
	
        c.stampTime();
    }


    public final Connection getConnection() {
        return conn;
    }

    public final void setConnection(Connection c) {
        this.conn = c;
        // to avoid null pointer exception for the local case
        if (c != null) {
	    orb = c.getORB();
        }
    }

    public final Message getMessage() {
        return msg;
    }

    //
    // We ran out of things to read. We pre-read everything
    // off of the java.io.InputStream in IIOPConnection.createInputStream()
    // so this can't happen normally.
    //
    protected void grow(int align, int n) {
	throw new MARSHAL(MinorCodes.IIOPINPUTSTREAM_GROW,
			  CompletionStatus.COMPLETED_MAYBE);
    }

    final CodeBase getCodeBase() {
        if (conn == null)
            return FVDCodeBaseDelegate.singleton;
        else
            return ((IIOPConnection)conn).getCodeBaseReference();
    }
}

