/*
 * @(#)IIOPOutputStream.java	1.41 00/02/11
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import java.io.IOException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.TRANSIENT;
import org.omg.CORBA.NO_RESOURCES;
import org.omg.CORBA.Principal;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.*;
import com.sun.corba.se.internal.util.Utility; //d11638
import com.sun.corba.se.internal.util.MinorCodes; //d11638
import com.sun.corba.se.internal.core.ClientResponse;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.Profile;


public class IIOPOutputStream extends CDROutputStream
{
    protected Connection conn;
    protected Message msg;
    private IIOPOutputStream fIIOPOutputStream;

    public IIOPOutputStream(ORB orb, Connection c) {
	super(orb);
	this.conn = c;
    }

    public IIOPOutputStream(IIOPInputStream s) {
	super(s.orb);
	conn = s.getConnection();
    }

    public final Connection getConnection() {
	return conn;
    }

    public final void setConnection(Connection c) {
	this.conn = c;
    }

    public final Message getMessage() {
	return msg;
    }

    public final void setMessage(Message msg) {
	this.msg = msg;
    }


    protected static final int MAX_RETRIES = 3;

    public IIOPInputStream invoke(boolean isOneway) 
    {
        SystemException exception = null;
        ClientResponse resp = null;
        int ntries = 0;
        while ( ntries < MAX_RETRIES ) {
            try {
                resp = (ClientResponse)conn.send(this, isOneway);
		if ( isOneway )
		    return null;
                if ( !resp.isSystemException() )
                    return (IIOPInputStream)resp;
                exception = resp.getSystemException();
            }
            catch ( SystemException ex ) {
                exception = ex;
            }
        
            if ( exception.completed == CompletionStatus.COMPLETED_NO &&
	         ( exception.getClass() == COMM_FAILURE.class ||
	           exception.getClass() == TRANSIENT.class ||
	           exception.getClass() == NO_RESOURCES.class ) ) {

		// _REVISIT_ should these be retried by the ORB?
		ntries++;

	    } else {    
		if ( resp != null )
		    // resp may contain svcContext etc from server
		    return (IIOPInputStream)resp; 
		else 
                    return new ClientResponseImpl(exception);
	    }
    
            if ( exception.minor == MinorCodes.CONN_CLOSE_REBIND ) {
                // create a new connection to server if the existing one died
                GIOPImpl g = (GIOPImpl)(((com.sun.corba.se.internal.core.ORB)orb).getServerGIOP());
                conn = g.getConnection(((IIOPConnection)conn).getEndpoint());
            }
        }
	// Fix for Bug ID: 4269610
        // If the connection is bad after doing MAX reties then it will
        // be cleaned up from the connection cache
        if ( (exception.completed == CompletionStatus.COMPLETED_NO) &&
           ( exception.getClass() == COMM_FAILURE.class ) )
        {
                GIOPImpl gc = (GIOPImpl)(((com.sun.corba.se.internal.core.ORB)orb).getClientGIOP());
                if( gc != null )
                {
                      gc.deleteConnection( 
			((IIOPConnection)conn).getEndpoint() );
                }
        }


	if ( isOneway )
	    return null;
            
        // too many retries: give up.
	if ( resp != null )
	    return (IIOPInputStream)resp;

	if ( exception == null )
            exception = new COMM_FAILURE(MinorCodes.INVOKE_ERROR,
                                         CompletionStatus.COMPLETED_NO);
        return new ClientResponseImpl(exception);
    }

    /**
     * Resend the invocation to the new IOR, after a location forward.
     * bodyBegin is the beginning of the request body.
     */
    public void realign(byte[] objKey, int[] bodyBeginHolder)
    {
	RequestMessage req = (RequestMessage)msg;

	// marshal the request header and get the resulting bytes
	CDROutputStream newos = new CDROutputStream(orb);

	// XXX figure out if alignment change is needed by checking if
	// a double / longlong / fixed was written to this outputstream.
	// This requires a simple change to CDROutputStream: just add
	// a is8ByteAligned() method which returns a flag that is init false
	// and set to true only when write_longlong() or write_double() is
	// called.  We do not currently support long double.
	boolean alignChangeNeeded = true;

	req.realignAndWrite(newos, objKey, alignChangeNeeded);
	byte[] headerBytes = newos.toByteArray();
	int headerlen = headerBytes.length;

	// copy the new header into our message buffer
	int bodyBegin = bodyBeginHolder[0];
	if ( headerlen == bodyBegin ) {
	    // header size didnt change
	    System.arraycopy(headerBytes, 0, buf, 0, headerlen);
	} else {
	    // create a new buffer and copy stuff
	    int newsize = size - bodyBegin + headerlen;
	    byte[] newbuf = new byte[newsize];
	    System.arraycopy(headerBytes, 0, newbuf, 0, headerlen);
	    System.arraycopy(buf, bodyBegin, newbuf, headerlen,
			     size - bodyBegin);
	    buf = newbuf;
	    size = newsize;
	    bodyBeginHolder[0] = headerlen;
	}
    }

    public IIOPInputStream reInvoke(IOR ior, int[] bodyBeginHolder)
    {
    	// Modify RequestHeader with objKey from
        // new IOR and resend the message to new IOR.
        // Note: we cannot remarshal method arguments, so
        // we may have to do correct alignment for the request body.

        Profile iop = ior.getProfile();
        byte[]  objKey = iop.getObjectKey();
	realign( objKey, bodyBeginHolder ) ;

	// Create a connection for the new endpoint
        GIOPImpl giop = (GIOPImpl)(((com.sun.corba.se.internal.core.ORB)orb).getServerGIOP());
        conn = giop.getConnection(ior.getEndpoint());

	// Invoke on the server. Note: location forwards cannot be oneway.
	return invoke(false);
    }    

    /**
     * Write the contents of the CDROutputStream to the specified 
     * output stream.  Has the side-effect of pushing any current 
     * Message onto the Message list.
     * @param s The output stream to write to.
     */
    public void writeTo(java.io.OutputStream s) throws java.io.IOException {
        //
        // Update the GIOP MessageHeader size field.
        //
        msg.setSize(getBuffer(), size);

        s.write(getBuffer(), 0, size);

        // TimeStamp connection to indicate it has been used
        // Note granularity of connection usage is assumed for
        // now to be that of a IIOP packet.
        conn.stampTime();

    }


    public void sendUserException(org.omg.CORBA.portable.Streamable exholder, int requestId)
        throws SystemException
    {
        reset();

	// First marshal the reply header
        ReplyMessage msg = new ReplyMessage(null, requestId,
					    ReplyMessage.USER_EXCEPTION, null, 
					    ((com.sun.corba.se.internal.corba.ORB)orb).transportDebugFlag );
        setMessage(msg);
        msg.write(this);

        exholder._write(this);
    }


    public void sendDSIException(Any any, IIOPInputStream in)
    {
        reset();

	// First marshal the reply header
	int requestId = in.getMessage().getRequestId();
        ReplyMessage msg = new ReplyMessage(null, requestId,
					    ReplyMessage.USER_EXCEPTION, null, 
					    ((com.sun.corba.se.internal.corba.ORB)orb).transportDebugFlag ) ;
        setMessage(msg);
        msg.write(this);

	// write only the value
	any.write_value(this);
    }


    /**
     * sendSystemException(Exception, user_exception), is used to
     * marshal appropriate header and marshal the exception in
     * the marshal buffer
     */
    public void sendSystemException(SystemException exc, int requestId)
        throws SystemException
    {
        reset();
        
	// First marshal the reply header
        ReplyMessage msg = new ReplyMessage(null, requestId,
					    ReplyMessage.SYSTEM_EXCEPTION, null,
					    ((com.sun.corba.se.internal.corba.ORB)orb).transportDebugFlag );
        setMessage(msg);
        msg.write(this);
 
        // Puts repositoryId too
        Utility.writeSystemException(exc, this);
    }


    /** overrides create_input_stream from CDROutputStream */
    public org.omg.CORBA.portable.InputStream create_input_stream()
    {
        return new IIOPInputStream((com.sun.corba.se.internal.iiop.ORB)orb, getBuffer(), size, littleEndian, msg);
    }

}

