/*
 * @(#)LocalClientRequestImpl.java	1.17 01/02/09
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import com.sun.corba.se.internal.core.Request;
import com.sun.corba.se.internal.core.ClientRequest;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.ClientResponse;
import com.sun.corba.se.internal.core.ServerRequest;
import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.Profile;
import com.sun.corba.se.internal.core.GIOPVersion;
import com.sun.corba.se.internal.orbutil.ORBConstants;
import com.sun.corba.se.internal.core.ORBVersionFactory;
import com.sun.corba.se.internal.orbutil.ORBUtility;

public class LocalClientRequestImpl extends IIOPOutputStream implements ClientRequest {

    public LocalClientRequestImpl(com.sun.corba.se.internal.iiop.ORB orb, byte[] objectKey, 
				  String operationName, boolean oneway, ServiceContexts svc, int requestId) 
    {
	super(orb, null);

	this.isOneway = oneway;
	boolean responseExpected = !isOneway;
        // we need to set the ORB version we need to talk to, based on the
        // magic in the object key.  This is for interoperability purposes
        // with the ORB that was shipped with the wrong Repository Ids calcs.
        int magic = ORBUtility.bytesToInt(objectKey, 0);

        orb.setORBVersion(ORBVersionFactory.createFromMagic(magic));

	this.request = new RequestMessage(svc, requestId, responseExpected,
					  objectKey, operationName,
					  null, orb.transportDebugFlag);
	request.write(this);
	setMessage(request);

	// mark beginning of msg body for possible later use
	bodyBegin = size;
    }
    
    public int getRequestId() {
	return request.getRequestId();
    }
    
    public boolean isOneWay() {
	return isOneway;
    }

    public ServiceContexts getServiceContexts() {
	return request.getServiceContexts();
    }

    public String getOperationName() {
	return request.getOperation();
    }

    public byte[] getObjectKey() {
	return request.getObjectKey();
    }

    public byte[] getBody(){
        if(size <= bodyBegin) return null;
        byte[] body     = new byte[size - bodyBegin];
        System.arraycopy(toByteArray(),bodyBegin,body,0,size-bodyBegin);
        return body;
    }

    public ServerRequest getServerRequest()
    {
	// Set the size of the marshalled data in the message header.
	getMessage().setSize( getBuffer(), size ) ;
	
	// Construct a new ServerRequest out of the buffer in this ClientRequest
	LocalServerRequestImpl serverRequest = new LocalServerRequestImpl( 
									  (com.sun.corba.se.internal.iiop.ORB)orb, toByteArray(), request ) ;
	    
	// Skip over all of the GIOP header information.  This positions
	// the offset in the buffer so that the skeleton can correctly read
	// the marshalled arguments.
	serverRequest.setOffset( bodyBegin ) ;
	
	return serverRequest ;
    }
    
    public ClientResponse invoke() 
    {	
	com.sun.corba.se.internal.iiop.ORB myORB = (com.sun.corba.se.internal.iiop.ORB)orb ;
	
	ServerResponse serverResponse = myORB.process( getServerRequest() ) ;
	
	LocalServerResponseImpl lsr = (LocalServerResponseImpl)serverResponse ;

	return lsr.getClientResponse() ;
    }


    /**
     * Resend the invocation to the new IOR, after a location forward.
     */
    public ClientResponse reInvoke(IOR[] iorHolder)
    {
	IOR ior = iorHolder[0];
	int[] bodyBeginHolder = new int[1];
	bodyBeginHolder[0] = bodyBegin;
	ClientResponse resp;
	while (true) { // Keep invoking if there are multiple location forwards
	    Profile iop = ior.getProfile() ;
	    byte[] objKey = iop.getObjectKey() ;
	    realign( objKey, bodyBeginHolder ) ;
	    resp = (ClientResponse) invoke();

	    if ( !resp.isLocationForward() )
		break;

	    ior = resp.getForwardedIOR();
	    bodyBegin = bodyBeginHolder[0] ;
	}
	iorHolder[0] = ior;
	return resp;
    }

    /**
     * Resend the invocation to the new IOR, after a location forward.
     */
    public ClientResponse reInvokeOnce(IOR[] iorHolder)
    {
	IOR ior = iorHolder[0];
	int[] bodyBeginHolder = new int[1];
	bodyBeginHolder[0] = bodyBegin;
	ClientResponse resp;

	Profile iop = ior.getProfile() ;
	byte[] objKey = iop.getObjectKey() ;
	realign( objKey, bodyBeginHolder ) ;
	resp = (ClientResponse) invoke();

	if (resp.isLocationForward()) {
	    ior = resp.getForwardedIOR();
	    iorHolder[0] = ior;
	}

	bodyBegin = bodyBeginHolder[0] ;

	return resp;
    }


    /**
     * Check to see if the request is local.
     */
    public boolean isLocal(){
        return true;
    }

    /** 
     * Get the GIOPVersion from the request.
     */
    public GIOPVersion getGIOPVersion( ) {
        return request.getGIOPVersion();
    }

    private RequestMessage request;
    private int bodyBegin;
    private boolean isOneway;
}
