/*
 * @(#)LocalClientResponseImpl.java	1.11 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import java.io.IOException;

import org.omg.CORBA.SystemException;

import com.sun.corba.se.internal.core.Response;
import com.sun.corba.se.internal.core.ClientResponse;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;

class LocalClientResponseImpl extends IIOPInputStream implements ClientResponse
{
    LocalClientResponseImpl(ORB orb, byte[] buf, ReplyMessage header)
    {
	super(orb, buf, header.getSize(), header.isLittleEndian(), header );

        this.reply = header;

        // BUGFIX(Ram Jeyaraman) This has been moved from
        // LocalServerResponse.getClientResponse()
        // Skip over all of the GIOP header information.  This positions
	// the offset in the buffer so that the skeleton can correctly read
	// the marshalled arguments.
	this.setOffset(Message.GIOPMessageHeaderLength);

        // BUGFIX(Ram Jeyaraman) For local invocations, the reply mesg fields
        // needs to be set, by reading the response buffer contents
        // to correctly set the exception type and other info.
        this.reply.read(this);
    }

    LocalClientResponseImpl(SystemException ex)
    {
	this.systemException = ex;
    }

    public boolean isSystemException() {
	if ( reply != null )
	    return reply.getReplyStatus() == ReplyMessage.SYSTEM_EXCEPTION;
	else
	    return (systemException != null);	
    }

    public boolean isUserException() {
	if ( reply != null )
	    return reply.getReplyStatus() == ReplyMessage.USER_EXCEPTION;
	else
	    return false;
    }

    public boolean isLocationForward() {
	if ( reply != null )
	    return reply.getReplyStatus() == ReplyMessage.LOCATION_FORWARD;
	else
	    return false;
    }

    public IOR getForwardedIOR() {
	if ( reply != null )
	    return reply.getIOR();
	else
	    return null;
    }

    public int getRequestId() {
	if ( reply != null )
	    return reply.getRequestId();
	else
	    throw new org.omg.CORBA.INTERNAL("Error in getRequestId");
    }

    public ServiceContexts getServiceContexts() {
	if ( reply != null )
	    return reply.getServiceContexts();
	else
	    return null;
    }

    public SystemException getSystemException() {
	if ( reply != null )
	    return reply.getSystemException();
	else
	    return systemException;
    }

    public java.lang.String peekUserExceptionId() {
	int pos = mark();
	String exid = read_string();
	reset(pos);
	return exid;
    }

    /**
     * Check to see if the response is local.
     */
    public boolean isLocal(){
        return true;
    }

    private ReplyMessage reply;
    private SystemException systemException;
}
