/*
 * @(#)LocalServerRequestImpl.java	1.15 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import org.omg.CORBA.SystemException;

import com.sun.corba.se.internal.core.ServerRequest;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.DuplicateServiceContext;
import com.sun.corba.se.internal.core.UEInfoServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.GIOPVersion;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.util.Utility;  //d11638
import org.omg.CORBA.portable.UnknownException;
import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.CompletionStatus;

class LocalServerRequestImpl extends IIOPInputStream implements ServerRequest
{
    org.omg.CORBA.portable.OutputStream replyStream;
    org.omg.CORBA.portable.OutputStream exceptionReplyStream;

    LocalServerRequestImpl(ORB orb, byte[] buf, RequestMessage header)
    {
	super(orb, buf, header.getSize(), header.isLittleEndian(), header );

	this.request = header;
    }

    public int getRequestId() {
	return request.getRequestId();
    }

    public boolean isOneWay() {
	return !request.isResponseExpected();
    }

    public ServiceContexts getServiceContexts() {
	return request.getServiceContexts();
    }
    
    public String getOperationName() {
	return request.getOperation();
    }
    
    public byte[] getObjectKey() {
	return request.getObjectKey();
    }

    public ServerResponse createResponse(ServiceContexts svc)
    {
	return new LocalServerResponseImpl(this, svc);
    }
    
    public org.omg.CORBA.portable.OutputStream createReply() {
        if ( replyStream == null ) {
            replyStream = (org.omg.CORBA.portable.OutputStream)createResponse(null);
        } 
        return replyStream;  
    }

    public org.omg.CORBA.portable.OutputStream createExceptionReply() {
        if ( exceptionReplyStream == null ) {
            exceptionReplyStream =  (org.omg.CORBA.portable.OutputStream)createUserExceptionResponse(null);
        }
        return exceptionReplyStream;
    }
    
    public ServerResponse createUserExceptionResponse(
	ServiceContexts svc) 
    {
	return new LocalServerResponseImpl(this, svc, true);
    }

    public ServerResponse createUnknownExceptionResponse(
	UnknownException ex
    )
    {
	ServiceContexts contexts = null;
	SystemException sys = new UNKNOWN( 0, 
	    CompletionStatus.COMPLETED_MAYBE);

        try {
	    contexts = new ServiceContexts( (com.sun.corba.se.internal.corba.ORB)orb );
	    UEInfoServiceContext uei = new UEInfoServiceContext(sys);
	    contexts.put( uei ) ;
        } catch (DuplicateServiceContext d) {
	    // can't happen
        }

	return createSystemExceptionResponse(sys,contexts);
    }

    public ServerResponse createSystemExceptionResponse(
	SystemException ex, ServiceContexts svc) 
    {
	LocalServerResponseImpl response =
	    new LocalServerResponseImpl(this, svc, false);
	Utility.writeSystemException(ex, response);
	return response;
    }

    public ServerResponse createLocationForward(
	IOR ior, ServiceContexts svc)
    {
	ReplyMessage reply = new ReplyMessage(svc, getRequestId(),
	    ReplyMessage.LOCATION_FORWARD, ior, 
	    ((com.sun.corba.se.internal.corba.ORB)orb).transportDebugFlag);

	LocalServerResponseImpl response = 
	    new LocalServerResponseImpl(this, reply, ior);

	return response;
    }

    private RequestMessage request;

    /**
     * Check to see if the request is local.
     */
    public boolean isLocal(){
        return true;
    }

    /**
     * Get the GIOPVersion from the request.
     */
    public GIOPVersion getGIOPVersion( ) {
        return request.getGIOPVersion();
    }
  

}
