/*
 * @(#)LocalServerResponseImpl.java	1.14 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import org.omg.CORBA.SystemException;

import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.ClientResponse;

class LocalServerResponseImpl
    extends     IIOPOutputStream
    implements  ServerResponse
{
    LocalServerResponseImpl(LocalServerRequestImpl request, ServiceContexts svc)
    {
	this(request,
	     new ReplyMessage(svc, request.getRequestId(),
			      ReplyMessage.NO_EXCEPTION, null, 
			      ((com.sun.corba.se.internal.corba.ORB)request.orb).transportDebugFlag),
	     null );
    }

    LocalServerResponseImpl(LocalServerRequestImpl request, ServiceContexts svc,
			    boolean user)
    {
	this(request,
	     new ReplyMessage(svc, request.getRequestId(),
			      user ? ReplyMessage.USER_EXCEPTION :
			      ReplyMessage.SYSTEM_EXCEPTION, null, 
			      ((com.sun.corba.se.internal.corba.ORB)request.orb).transportDebugFlag ),
	     null);
    }

    LocalServerResponseImpl( LocalServerRequestImpl request, ReplyMessage reply, 
			     IOR ior)
    {
	super((com.sun.corba.se.internal.iiop.ORB)(request.orb), null);
	reply.write(this);
	setMessage(reply);
	if (ior != null)
	    ior.write(this);

	this.reply = reply;
	this.ior = ior;

	bodyBegin = size ;
    }
    
    public boolean isSystemException() {
	if (reply != null)
	    return reply.getReplyStatus() == ReplyMessage.SYSTEM_EXCEPTION;
	return false;
    }

    public boolean isUserException() {
	if (reply != null)
	    return reply.getReplyStatus() == ReplyMessage.USER_EXCEPTION;
	return false;
    }

    public boolean isLocationForward() {
	if (ior != null)
	    return true;
	return false;
    }

    public IOR getForwardedIOR() {
	return ior;
    }

    public int getRequestId() {
	if (reply != null)
	    return reply.getRequestId();
	return -1;
    }

    public ServiceContexts getServiceContexts() {
	if (reply != null)
	    return reply.getServiceContexts();
	return null;
    }

    public SystemException getSystemException() {
	if (reply != null)
	    return reply.getSystemException();
	return null;
    }

    public ReplyMessage getReply()
    {
	return reply ;
    }

    public ClientResponse getClientResponse()
    {
	// set the size of the marshalled data in the message header
	getMessage().setSize( getBuffer(), size ) ;

	// Construct a new ClientResponse out of the buffer in this ClientRequest
	LocalClientResponseImpl result = new LocalClientResponseImpl( 
								     (com.sun.corba.se.internal.iiop.ORB)orb, toByteArray(), reply ) ;

	// BUGFIX(Ram Jeyaraman) result.setOffset is now done in
	// LocalClientResponseImpl constructor.
	/*
	  // Skip over all of the GIOP header information.  This positions
	  // the offset in the buffer so that the skeleton can correctly read
	  // the marshalled arguments.
	  result.setOffset( bodyBegin ) ;
	*/
    
	return result ;
    }

    /**
     * Check to see if the response is local.
     */
    public boolean isLocal(){
        return true;
    }

    private int bodyBegin ;
    private ReplyMessage reply;
    private IOR ior; // forwarded IOR
}
