/*
 * @(#)ORB.java	1.182 01/02/09
 *
 * Copyright 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

// Import JDK stuff
import java.net.*;
import java.util.*;


// Import our stuff

import org.omg.CORBA.SystemException;
import org.omg.CORBA.OBJ_ADAPTER;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.CompletionStatus;

import com.sun.corba.se.internal.core.ClientSubcontract;
import com.sun.corba.se.internal.core.ServerSubcontract;
import com.sun.corba.se.internal.core.SubcontractRegistry;
import com.sun.corba.se.internal.core.ClientGIOP;
import com.sun.corba.se.internal.core.ServerGIOP;
import com.sun.corba.se.internal.core.MarshalInputStream;
import com.sun.corba.se.internal.core.MarshalOutputStream;
import com.sun.corba.se.internal.core.RequestHandler;
import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.ServerRequest;
import com.sun.corba.se.internal.core.ServiceContextRegistry;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.DuplicateServiceContext;
import com.sun.corba.se.internal.core.NoSuchServiceContext;
import com.sun.corba.se.internal.core.SendingContextServiceContext;
import com.sun.corba.se.internal.core.CodeSetServiceContext;
import com.sun.corba.se.internal.core.UEInfoServiceContext;
import com.sun.corba.se.internal.core.CodeSetComponentInfo;
import com.sun.corba.se.internal.core.ORBVersionServiceContext;

import com.sun.corba.se.internal.core.EndPoint;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.util.MinorCodes; //d11638
import com.sun.corba.se.internal.orbutil.SubcontractList; 

/** 
 * The JavaIDL IIOP ORB implementation.
 */
public class ORB extends com.sun.corba.se.internal.corba.ORB implements RequestHandler {
    // The following fields form our special little collection of global state. 
    // We keep it bottled up here in the ORB class and attach a reference to
    // ourselves to every object reference that we create and handle.
    protected SubcontractRegistry subcontractRegistry;
    protected GIOPImpl giopTransport;

    // This is the unique id of this server (JVM). Multiple incarnations
    // of this server will get different ids.
    protected int transientServerId=0;

    // The thread group of the main thread (for applications) or applet.
    protected ThreadGroup threadGroup;

    protected ServiceContextRegistry scr ;

    /** 
     * Create a new ORB. Should only be invoked from the
     * CORBA ORB superclass. Should be followed by the appropriate
     * set_parameters() call.
     */
    public ORB() 
    {
        //
        // We attempt to create new threads in this thread group, if
        // possible. This avoids problems if the application/applet
        // creates a thread group, makes JavaIDL calls which create a new
        // connection and ReaderThread, and then destroys the thread
        // group. If our ReaderThread were part of this destroyed thread
        // group then it might get killed and cause other invoking threads
        // sharing the same connection to get a non-restartable
        // CommunicationFailure. We'd like to avoid that.
        //
        // Our solution is to create all of our threads in the same
        // thread group that we were initialized under.
        //
        threadGroup = Thread.currentThread().getThreadGroup();

        // Compute transientServerId = (milliseconds since Jan 1, 1970)/10.
	// Note: transientServerId will wrap in about 2^32 / 8640000 = 497 days.
        // Randomize time before currentTimeMillis() call to prevent multiple
        // processes on same (multiprocessor) machine from getting same serverID
        long rndVal = new java.util.Random().nextLong();
        int x = 0;
        while ( rndVal > 0 ) {
	    if ((rndVal & 1)!=0)
	        x++;
	    rndVal >>= 1;
        }
        x = x*100;
        while ( x > 0 )
            x--;
        transientServerId = (int)(System.currentTimeMillis()/10);

	if (ORBInitDebug)
	    dprint( "Creating service context registry" ) ;

	scr = new ServiceContextRegistry( this ) ;

	if (ORBInitDebug)
	    dprint( "Registering service context classes" ) ;

	// Register service contexts 
	try {
	    scr.register( UEInfoServiceContext.class ) ;
	    scr.register( CodeSetServiceContext.class ) ;
	    scr.register( SendingContextServiceContext.class ) ;
           
            // register the service context class to handle ORB versioning
            scr.register( ORBVersionServiceContext.class ) ;
	} catch (DuplicateServiceContext dsc) {
	    if (ORBInitDebug)
		dprint( "Duplicate service context registration error" ) ;

	    throw new INTERNAL( "Duplicate Service Context error" ) ;
	} catch (NoSuchServiceContext dsc) {
	    if (ORBInitDebug)
		dprint( "No such service context registration error" ) ;

	    throw new INTERNAL( 
			       "Error in service context class definition" ) ;
	} 
        

	// Register subcontracts in subcontractRegistry
        subcontractRegistry
	    = new SubcontractRegistry(SubcontractList.defaultSubcontract);

	// register the server subcontract *instance*
	
	for(int i = 0; i < SubcontractList.subcontracts.length; i++) {
	    try {
		Class serverClass =
		    SubcontractList.subcontracts[i].serverSC;
		ServerSubcontract sc = (ServerSubcontract)
		    serverClass.newInstance();
		sc.setOrb(this);
		subcontractRegistry.registerServer(sc,
						   SubcontractList.subcontracts[i].id);
                // register ServerDelegate as the default bootstrap SC
                // so that SC lookups with objkey = "INIT" will work
		// The BootstrapServer will override this with its own SC.
                // See bugid 4155455
                if ( serverClass == com.sun.corba.se.internal.corba.ServerDelegate.class )
		    subcontractRegistry.registerBootstrapServer(sc);

		Class clientClass =
		    SubcontractList.subcontracts[i].clientSC;
		if (clientClass != null)
		    subcontractRegistry.registerClient(clientClass,
						       SubcontractList.subcontracts[i].id);
	    } catch (Exception ex) { }
	}
	    
	// Initialize the GIOP transport
	giopTransport = new GIOPImpl(this, this);
    }

    public ClientGIOP getClientGIOP()
    {
	return giopTransport;
    }

    public ServerGIOP getServerGIOP()
    {
	return giopTransport;
    }

    public MarshalInputStream newInputStream() {
	return new CDRInputStream(this, new byte[1024], 1024);
    }

    public MarshalInputStream newInputStream(byte[] buffer, int size) {
	return new CDRInputStream(this, buffer, size);
    }

    public MarshalInputStream newInputStream(byte[] buffer, int size, boolean littleEndian) {
	return new CDRInputStream(this, buffer, size, littleEndian);
    }
    

    public MarshalOutputStream newOutputStream() {
	return new CDROutputStream(this);
    }

    public IIOPOutputStream newOutputStream(Connection c) {
	return new IIOPOutputStream(this, c);
    }

    public EndPoint newEndPoint(IOR ior) {
        return new EndPointImpl(this, ior);
    }

    

    public int getTransientServerId()
    {
        return transientServerId;
    }

    public SubcontractRegistry getSubcontractRegistry()
    {
        return subcontractRegistry;
    }

    public ServiceContextRegistry getServiceContextRegistry()
    {
	return scr ;
    } 

    public ServerResponse process(ServerRequest request) {
	ServerSubcontract sc =
	    subcontractRegistry.getServerSubcontract(request.getObjectKey());
	if (sc == null) {
	    SystemException ex =
		new OBJ_ADAPTER(MinorCodes.NO_SERVER_SC_IN_DISPATCH,
				CompletionStatus.COMPLETED_NO);
			    
	    return request.createSystemExceptionResponse(ex, null);
	}
	return sc.dispatch(request);
    }

    public IOR locate(byte[] key) {
	ServerSubcontract sc =
	    subcontractRegistry.getServerSubcontract(key);
	if (sc == null)
	    return null;
	return sc.locate(key);
    }

    public CodeSetComponentInfo getCodeSetComponentInfo() {
	return new CodeSetComponentInfo();
    } 
    
    public boolean isLocalHost( String hostName ) {
	return hostName.equals( getORBServerHost() ) ||
	    hostName.equals( getLocalHostName() ) ;
    }

    public boolean isLocalServerId( byte[] key )
    {
	int serverId = subcontractRegistry.getServerId( key ) ;

	return serverId == transientServerId ; 
    }


    /* keeping a copy of the getLocalHostName so that it can only be called 
     * internally and the unauthorized clients cannot have access to the
     * localHost information, originally, the above code was calling getLocalHostName
     * from Connection.java.  If the hostname is cached in Connection.java, then
     * it is a security hole, since any unauthorized client has access to
     * the host information.  With this change it is used internally so the
     * security problem is resolved.  Also in Connection.java, the getLocalHost()
     * implementation has changed to always call the 
     * InetAddress.getLocalHost().getHostAddress()
     */

    private String localHostString = null;

    private String getLocalHostName() {
        if (localHostString != null) {
            return localHostString;
        } else {
            try {
                localHostString = InetAddress.getLocalHost().getHostAddress();
                return localHostString;

            } catch (Exception ex) {
                throw new INTERNAL( MinorCodes.GET_LOCAL_HOST_FAILED,
                                CompletionStatus.COMPLETED_NO );
            }
	}
    }

 
} // Class ORB
