/*
 * @(#)ReplyMessage.java	1.47 00/02/11
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.portable.*;

import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.util.Utility; //d11638
import com.sun.corba.se.internal.corba.ORB ;

public class ReplyMessage extends Message {
    private void dprint( String msg )
    {
	Utility.dprint( this, msg ) ;
    }

    protected ServiceContexts ctx;
    protected int status;
    protected String exceptionId;
    protected String exClassName;
    protected int minorCode;
    protected CompletionStatus completionStatus;
    protected IOR ior;
    protected ORB orb;


    public static final int NO_EXCEPTION = 0;
    public static final int USER_EXCEPTION = 1;
    public static final int SYSTEM_EXCEPTION = 2;
    public static final int LOCATION_FORWARD = 3;

    public ReplyMessage( boolean debug ) {
	super( debug ) ;
    }

    public ReplyMessage( ServiceContexts ctx, int requestId, 
			 int status, IOR ior, boolean debug ) 
    {
	super( debug );
	setType(Message.GIOPReply);
	this.ctx = ctx;
	this.requestId = requestId;
	this.status = status;
	this.ior = ior;
    }

    public ServiceContexts getServiceContexts() {
	return ctx;
    }

    public int getReplyStatus() {
	return status;
    }

    public String getExceptionId() {
	return exceptionId;
    }

    public int getMinorCode() {
	return minorCode;
    }

    public IOR getIOR() {
	return ior;
    }

    public SystemException getSystemException()
    {
	if (debug) {
	    dprint( "getSystemException called" ) ;
	    dprint( "exClassName = " + exClassName ) ;	    
	    Exception ex = new Exception( "traceback" ) ;
	    ex.fillInStackTrace() ;
	    ex.printStackTrace() ;
	}

	SystemException sysEx;

	try {
	    sysEx = (SystemException)Class.forName(exClassName).newInstance();
        } catch ( Exception someEx ) {
	    if (debug)
		dprint( "Bad system exception" ) ;

            throw new INTERNAL("BAD SystemException: " + exClassName,
			       0, CompletionStatus.COMPLETED_MAYBE);
        }

	sysEx.minor = minorCode;
	sysEx.completed = completionStatus;

	if (debug)
	    dprint( "returning system exception " + sysEx ) ;

	return sysEx;
    }

    public void write(OutputStream s) throws SystemException {
	//
	// Handle nulls for Service Context and
	// Principal in case someone doesn't set these
	// to meaningful values yet 
	//
	super.write(s);

	if (ctx != null)
	    ctx.write( (org.omg.CORBA_2_3.portable.OutputStream)s ) ; 
	else
	    ServiceContexts.writeNullServiceContext( 
						    (org.omg.CORBA_2_3.portable.OutputStream)s ) ;

	s.write_long(requestId);
	s.write_long(status);
    }

    public void read(InputStream s) throws SystemException {
	if (debug)
	    dprint( "read from input stream called" ) ;

	ctx = new ServiceContexts( (org.omg.CORBA_2_3.portable.InputStream)s ) ;
	requestId = s.read_long();

	if (debug)
	    dprint( "requestId = " + requestId ) ;

	status = s.read_long();

	if (debug)
	    dprint( "status = " + status ) ;

	if (status == SYSTEM_EXCEPTION) {
	    if (debug)
		dprint( "messsage is system exception" ) ;

            String reposId = s.read_string();
	    if (debug)
		dprint( "reposId is " + reposId ) ;

            exClassName = Utility.classNameOf(reposId);
	    if (debug)
		dprint( "exClassName is " + exClassName ) ;

            minorCode = s.read_long();
	    if (debug)
		dprint( "minorCode is " + minorCode ) ;

            int status = s.read_long();
	    if (debug)
		dprint( "status is " + status ) ;

            switch ( status ) {
	    case CompletionStatus._COMPLETED_YES:
		completionStatus = CompletionStatus.COMPLETED_YES;
		break;
	    case CompletionStatus._COMPLETED_NO:
		completionStatus = CompletionStatus.COMPLETED_NO;
		break;
	    case CompletionStatus._COMPLETED_MAYBE:
		completionStatus = CompletionStatus.COMPLETED_MAYBE;
		break;
	    default:
		if (debug)
		    dprint( "Bad completion status error" ) ;

		throw new INTERNAL("BAD completion status: " + status,
				   0,
				   CompletionStatus.COMPLETED_MAYBE);
            }
	} else if (status == USER_EXCEPTION) {
	    if (debug)
		dprint( "message is user exception" ) ;

	    // NOTE: at this point we should only PEEK the exceptionID
            // since it is actually read by exceptionHolder.__read().
	    IIOPInputStream iis = (IIOPInputStream)s;
	    exceptionId = iis.peek_string();
	    if (debug)
		dprint( "exceptionId is " + exceptionId ) ;
	} else if (status == LOCATION_FORWARD) {
	    if (debug)
		dprint( "message is location forward" ) ;

	    CDRInputStream cdr = (CDRInputStream) s;
	    ior = new IOR(cdr.orb);
	    ior.read(s);
	}
    }
}
