/*
 * @(#)RequestMessage.java	1.48 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import org.omg.CORBA.Principal;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.portable.*;

import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.util.Utility ;

public class RequestMessage extends Message {
    private void dprint( String msg ) 
    {
	Utility.dprint( this, msg ) ;
    }

    protected ServiceContexts ctx;
    protected boolean responseExpected;
    protected byte[] objectKey;
    protected String operation;
    protected Principal principal;

    public RequestMessage( boolean debug ) {
	super( debug ) ;

	if (debug)
	    dprint( "RequestMessage constructor completed" ) ;
    }

    //
    // Create an instance with initialized instance variables
    // and a filled in request header in the buffer ready to be
    // appended to.
    //
    public RequestMessage(ServiceContexts ctx, int requestId,
			  boolean response, byte[] objectKey, String operation,
			  Principal principal, boolean debug )
    {
	super( debug ) ;
	setType(Message.GIOPRequest);
	this.ctx = ctx;
	this.requestId = requestId;
	responseExpected = response;
	this.objectKey = objectKey;
	this.operation = operation;
	this.principal = principal;

	if (debug)
	    System.out.println( "RequestMessage constructor completed" ) ;
    }

    //
    // The common case, with no ServiceContext or Principal.
    //
    public RequestMessage(int requestId, boolean response, byte[] objectKey,
			  String operation, boolean debug )
    {
	this((ServiceContexts)null, requestId, response, objectKey,
	     operation, (Principal)null, debug );
    }

    public byte[] getObjectKey() {
	return objectKey;
    }

    public ServiceContexts getServiceContexts() {
	return ctx;
    }

    public String getOperation() {
	return operation;
    }

    public Principal getPrincipal() {
	return principal;
    }

    public boolean isResponseExpected() {
	return responseExpected;
    }

    public void write(OutputStream s) throws SystemException {
	//
	// Handle nulls for Service Context and
	// Principal in case someone doesn't set these
	// to meaning values yet (we don't care about
	// them typically).
	//
	super.write(s);
	if (ctx != null) {
	    ctx.write( (org.omg.CORBA_2_3.portable.OutputStream)s ) ;
	} else
	    ServiceContexts.writeNullServiceContext( 
						    (org.omg.CORBA_2_3.portable.OutputStream)s ) ;

	s.write_long(requestId);
	s.write_boolean(responseExpected);
        s.write_long(objectKey.length);
	s.write_octet_array(objectKey,0,objectKey.length);
	s.write_string(operation);

	if (principal != null)
	    s.write_Principal(principal);
	else
	    s.write_long(0);
    }

    public void read(InputStream s) throws SystemException {
	if (debug)
	    dprint( "Reading service contexts from inputstream" ) ;

	org.omg.CORBA_2_3.portable.InputStream is = null ;

	try {
	    is = (org.omg.CORBA_2_3.portable.InputStream)s ;

	    if (debug)
		dprint( "Stream class cast successful" ) ;
	} catch (ClassCastException cce) {
	    if (debug)
		dprint( "Error: could not cast input stream: it is a " + s.getClass().getName() ) ;
	}

	try {
	    ctx = new ServiceContexts( is ) ;
	} catch (SystemException sex) {
	    if (debug) {
		dprint( "Error caught in constructing ServiceContexts: " + 
			sex ) ;
		sex.printStackTrace() ;
	    }

	    throw sex ;
	}

	if (debug)
	    dprint( "Reading rest of message" ) ;

	requestId = s.read_long();
	responseExpected = s.read_boolean();
	int len = s.read_long();
	objectKey = new byte[len];
	s.read_octet_array(objectKey,0,len);
	operation = s.read_string();
	principal = s.read_Principal();

	if (debug)
	    dprint( "Read of RequestMessage complete" ) ;
    }



    public void realignAndWrite(OutputStream os, byte[] newObjKey,
				boolean alignChangeNeeded)
    {
	// Note: Since the last component of the RequestMessage header
	// is the Principal which is always null, we will always end on
	// a 4-byte or 8-byte alignment.

	if ( newObjKey.length != objectKey.length && alignChangeNeeded ) {
	    // In the common case, the object key length remains the same.
	    // Also, if no 8 byte CDR types (i.e. double, long double, long long)
	    // are marshalled, there is no need to realign.

	    // Find if the new marshaled msg header will end on a
	    // different alignment.

	    // alignment = 8-byte if len = 8*x + 5, 6, 7, 8
	    // alignment = 4-byte if len = 8*x + 1, 2, 3, 4

	    boolean currentAlign8 = ((objectKey.length-1) % 8) >= 4 ;
	    boolean newAlign8 = ((newObjKey.length-1) % 8) >= 4 ;

	    if ( currentAlign8 != newAlign8 ) 
		ctx.addAlignmentPadding() ;
	}

	objectKey = newObjKey;
	write(os);
    }
}
