/*
 * @(#)ServerRequestImpl.java	1.63 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import org.omg.CORBA.SystemException;
import org.omg.CORBA.portable.UnknownException;
import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.CompletionStatus;

import com.sun.corba.se.internal.core.ServerRequest;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.DuplicateServiceContext;
import com.sun.corba.se.internal.core.SendingContextServiceContext;
import com.sun.corba.se.internal.core.UEInfoServiceContext;
import com.sun.corba.se.internal.core.ORBVersionServiceContext;
import com.sun.corba.se.internal.core.ORBVersionFactory;
import com.sun.corba.se.internal.core.ServiceContexts;
import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.GIOPVersion;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.io.FVDCodeBaseImpl;
import com.sun.corba.se.internal.core.MarshalOutputStream;
import com.sun.corba.se.internal.util.Utility;
import com.sun.corba.se.internal.orbutil.ORBConstants;


public class ServerRequestImpl extends IIOPInputStream implements ServerRequest
{
    private void dprint( String msg )
    {
	Utility.dprint( this, msg ) ;
    }

    org.omg.CORBA.portable.OutputStream replyStream;
    org.omg.CORBA.portable.OutputStream exceptionReplyStream;

    public static final int UNKNOWN_EXCEPTION_INFO_ID = 9;

    protected ServerRequestImpl(Connection c, byte[] buf, RequestMessage header)
	throws java.io.IOException
    {
	super(c, buf, header);
	com.sun.corba.se.internal.corba.ORB theOrb = (com.sun.corba.se.internal.corba.ORB)orb ;
	if (theOrb.subcontractDebugFlag)
	    dprint( "Constructing ServerRequestImpl object" ) ;

	this.request = header;
    }

    protected ServerRequestImpl()
	throws java.io.IOException
    {
    }

    public int getRequestId() {
	return request.getRequestId();
    }

    public boolean isOneWay() {
	return !request.isResponseExpected();
    }

    public ServiceContexts getServiceContexts() {
	return request.getServiceContexts();
    }
    
    public String getOperationName() {
	return request.getOperation();
    }
    
    public byte[] getObjectKey() {
	return request.getObjectKey();
    }

    public ServerResponse createResponse(ServiceContexts svc)
    {
	return new ServerResponseImpl(this,
                                      getServiceContextsForReply(getConnection(),
                                                                 svc));
    }
    
    public org.omg.CORBA.portable.OutputStream createReply() {
        if ( replyStream == null ) {
            replyStream = 
		(org.omg.CORBA.portable.OutputStream)
		createResponse(getServiceContextsForReply(getConnection(),
                                                          null));
        } 
        return replyStream;  
    }

    public org.omg.CORBA.portable.OutputStream createExceptionReply() {
        if ( exceptionReplyStream == null ) {
            exceptionReplyStream =  
		(org.omg.CORBA.portable.OutputStream)
		createUserExceptionResponse(getServiceContextsForReply(getConnection(),
                                                                       null));
        }
        return exceptionReplyStream;
    }
    
    public ServerResponse createUserExceptionResponse(ServiceContexts svc) {
	return new ServerResponseImpl((ServerRequestImpl)this,
                                      getServiceContextsForReply(getConnection(),
                                                                 svc),
                                      true);
    }

    /**
     * Create a response that represents an unknown exception.
     *
     */
    public ServerResponse createUnknownExceptionResponse(
	UnknownException ex
    )
    {
	ServiceContexts contexts = null;
	SystemException sys = new UNKNOWN( 0, 
	    CompletionStatus.COMPLETED_MAYBE);

        try {
	    contexts = new ServiceContexts( (com.sun.corba.se.internal.corba.ORB)orb );
	    UEInfoServiceContext uei = new UEInfoServiceContext(sys);
	    contexts.put( uei ) ;
        } catch (DuplicateServiceContext d) {
	    // can't happen
        }

	return createSystemExceptionResponse(sys,contexts);
    }

    public ServerResponse createSystemExceptionResponse(
	SystemException ex, ServiceContexts svc) 
    {
	ServerResponseImpl response =
	    new ServerResponseImpl((ServerRequestImpl)this, 
                                   getServiceContextsForReply(getConnection(),
                                                              svc),
                                   false);
	Utility.writeSystemException(ex, response);
	return response;
    }

    public ServerResponse createLocationForward(IOR ior, ServiceContexts svc)
    {
	ReplyMessage reply = 
            new ReplyMessage(getServiceContextsForReply(getConnection(), 
                                                        svc), 
                             getRequestId(),
                             ReplyMessage.LOCATION_FORWARD, 
                             ior, 
                             ((com.sun.corba.se.internal.corba.ORB)orb).transportDebugFlag );
	ServerResponseImpl response = new ServerResponseImpl(this, reply, ior);

	return response;
    }

    protected RequestMessage request;
    
    /**
     * Check to see if request is local.
     */
    public boolean isLocal(){
        return false;
    }

    /** 
     * Get the GIOPVersion from request.
     */
    public GIOPVersion getGIOPVersion( ) {
        return request.getGIOPVersion();
    }

    /**
     * Returns ServiceContexts for reply.  If the contexts parameter is null,
     * it creates a new ServiceContexts instance with the code base (first
     * time only) and ORB version contexts.  Otherwise, it tries to add these
     * to the instance given, and returns that instance.
     */
    private ServiceContexts getServiceContextsForReply(Connection c,
                                                       ServiceContexts contexts) 
    {
        com.sun.corba.se.internal.corba.ORB theOrb = 
            (com.sun.corba.se.internal.corba.ORB)orb() ;
        
        if (theOrb.subcontractDebugFlag)
            dprint( "getServiceContextsForReply called with connection " + c ) ;

        if (contexts == null)
            contexts = new ServiceContexts( theOrb ) ;

        // NOTE : We only want to send the runtime context the first time

        if (c != null && !c.isPostInitialContexts()) {
          
            c.setPostInitialContexts();
            SendingContextServiceContext scsc =
                new SendingContextServiceContext(theOrb.getServantIOR()) ; 
            //d11638
            try {
                contexts.put( scsc ) ;
                if (theOrb.subcontractDebugFlag)
                    dprint( "Added SendingContextServiceContext" ) ;
            } catch (DuplicateServiceContext dsc) {
                // Ignore
            }
        }

        // send ORBVersion servicecontext as part of the Reply

        ORBVersionServiceContext ovsc 
            = new ORBVersionServiceContext(ORBVersionFactory.getORBVersion());

        try {
            contexts.put( ovsc ) ;
            if (theOrb.subcontractDebugFlag)
                dprint("Added ORB version service context");
        } catch (DuplicateServiceContext dsc) {
            // Ignore
        }

        return contexts;
    }
}
