/*
 * @(#)ServerResponseImpl.java	1.24 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.corba.se.internal.iiop;

import org.omg.CORBA.SystemException;

import com.sun.corba.se.internal.core.ServerResponse;
import com.sun.corba.se.internal.core.IOR;
import com.sun.corba.se.internal.core.ServiceContext;
import com.sun.corba.se.internal.core.ServiceContexts;

public class ServerResponseImpl
    extends     IIOPOutputStream
    implements  ServerResponse
{
    protected ServerResponseImpl(ServerRequestImpl request, 
				 ServiceContexts svc)
    {
	this(request,
	     new ReplyMessage(svc, request.getRequestId(),
			      ReplyMessage.NO_EXCEPTION, null, 
			      ((com.sun.corba.se.internal.corba.ORB)request.orb).transportDebugFlag ),
	     null);
    }

    protected ServerResponseImpl(ServerRequestImpl request, 
				 ServiceContexts svc, boolean user)
    {
	this(request,
	     new ReplyMessage(svc, request.getRequestId(),
			      user ? ReplyMessage.USER_EXCEPTION :
			      ReplyMessage.SYSTEM_EXCEPTION, null, 
			      ((com.sun.corba.se.internal.corba.ORB)request.orb).transportDebugFlag ),
	     null);
    }

    protected ServerResponseImpl(ServerRequestImpl request, ReplyMessage reply, IOR ior)
    {
	super(request.getConnection().getORB(), request.getConnection());
	reply.write(this);
	setMessage(reply);
	if (ior != null)
	    ior.write(this);

	this.reply = reply;
	this.ior = ior;
    }
    
    public boolean isSystemException() {
	if (reply != null)
	    return reply.getReplyStatus() == ReplyMessage.SYSTEM_EXCEPTION;
	return false;
    }

    public boolean isUserException() {
	if (reply != null)
	    return reply.getReplyStatus() == ReplyMessage.USER_EXCEPTION;
	return false;
    }

    public boolean isLocationForward() {
	if (ior != null)
	    return true;
	return false;
    }

    public IOR getForwardedIOR() {
	return ior;
    }

    public int getRequestId() {
	if (reply != null)
	    return reply.getRequestId();
	return -1;
    }

    public ServiceContexts getServiceContexts() {
	if (reply != null)
	    return reply.getServiceContexts();
	return null;
    }

    public SystemException getSystemException() {
	if (reply != null)
	    return reply.getSystemException();
	return null;
    }

    /**
     * Check to see if response is local.
     */
    public boolean isLocal(){
        return false;
    }

    private ReplyMessage reply;
    private IOR ior; // forwarded IOR
}
