/*
 * @(#)ValueUtility.java	1.16 01/02/09
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.io;

import com.sun.org.omg.CORBA.ValueDefPackage.FullValueDescription;
import com.sun.org.omg.CORBA.OperationDescription;
import com.sun.org.omg.CORBA.AttributeDescription;
import org.omg.CORBA.ValueMember;
import com.sun.org.omg.CORBA.Initializer;
import org.omg.CORBA.IDLType;
import com.sun.org.omg.CORBA._IDLTypeStub;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCodePackage.*;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TCKind;
import java.lang.reflect.*;
import com.sun.corba.se.internal.util.RepositoryId;
import java.util.Hashtable;
import javax.rmi.CORBA.Util;
import javax.rmi.CORBA.ValueHandler;

/**
 * Holds utility methods for converting from ObjectStreamClass to
 * FullValueDescription.
 **/
public class ValueUtility {

    public static final short PRIVATE_MEMBER = 0;
    public static final short PUBLIC_MEMBER = 1;
	
    private static final String primitiveConstants[] = {
	null,       // tk_null         0  
	null,		// tk_void         1
	"S",		// tk_short        2
	"I",		// tk_long         3
	"S",		// tk_ushort       4
	"I",		// tk_ulong        5
	"F",		// tk_float        6
	"D",		// tk_double       7
	"Z",		// tk_boolean      8
	"C",		// tk_char         9
	"B",		// tk_octet        10
	null,		// tk_any          11
	null,		// tk_typecode     12
	null,		// tk_principal    13
	null,		// tk_objref       14
	null,		// tk_struct       15
	null,		// tk_union        16
	null,		// tk_enum         17
	null,		// tk_string       18
	null,		// tk_sequence     19
	null,		// tk_array        20
	null,		// tk_alias        21
	null,		// tk_except       22
	"J",		// tk_longlong     23
	"J",		// tk_ulonglong    24
	"D",		// tk_longdouble   25
	"C",		// tk_wchar        26
	null,		// tk_wstring      27
	null,       // tk_fixed        28
	null,       // tk_value        29 
	null,       // tk_value_box    30
	null,       // tk_native       31
        null,       // tk_abstract_interface 32
    };

    public static String getSignature(ValueMember member)
	throws ClassNotFoundException {
		
	if (member.type.kind().value() == TCKind._tk_value_box) {
	    Class c = RepositoryId.cache.getId(member.id).getClassFromType();
	    if (c.isArray())
		return ObjectStreamClass.getSignature(c);
	    else
		return ObjectStreamClass.getSignature(c);
	}
	else {
	    return primitiveConstants[member.type.kind().value()];
	}
		
    }

    public static FullValueDescription translate(ORB orb, ObjectStreamClass data, ValueHandler vh){
		
	// Create FullValueDescription
	FullValueDescription result = new FullValueDescription();
	Class className = data.forClass();

        ValueHandlerImpl vhandler = (com.sun.corba.se.internal.io.ValueHandlerImpl) vh;
        String repId = vhandler.createForAnyType(className);

	// Set FVD name
	result.name = vhandler.getUnqualifiedName(repId);
        if (result.name == null)
            result.name = "";

	// Set FVD id _REVISIT_ : Manglings
	result.id = vhandler.getRMIRepositoryID(className);
        if (result.id == null)
            result.id = "";

	// Set FVD is_abstract
	result.is_abstract = ObjectStreamClassCorbaExt.isAbstractInterface(className);
		
	// Set FVD is_custom
	result.is_custom = data.isCustomMarshaled();

	// Set FVD defined_in _REVISIT_ : Manglings
	result.defined_in = vhandler.getDefinedInId(repId);
        if (result.defined_in == null)
            result.defined_in = "";

	// Set FVD version 
	result.version = vhandler.getSerialVersionUID(repId);
        if (result.version == null)
            result.version = "";

	// Skip FVD operations - N/A
	result.operations = new OperationDescription[0];

	// Skip FVD attributed - N/A
	result.attributes = new AttributeDescription[0];

	// Set FVD members
	ObjectStreamField fields[] = data.getFields();
	int fieldsLength = fields.length;
	result.members = new ValueMember[fieldsLength];
	// Note : fields come out of ObjectStreamClass in correct order for
	// writing.  So, we will create the same order in the members array.
	for (int i = 0; i < fieldsLength; i++) {
	    String valRepId = vhandler.getRMIRepositoryID(fields[i].getClazz());
	    result.members[i] = new ValueMember();
	    result.members[i].name = fields[i].getName();
	    result.members[i].id = valRepId; // _REVISIT_ : Manglings
	    result.members[i].defined_in = vhandler.getDefinedInId(valRepId);// _REVISIT_ : Manglings
	    result.members[i].version = "1.0";
	    result.members[i].type_def = new _IDLTypeStub(); // _REVISIT_ : IDLType implementation missing 

	    int m = fields[i].getField().getModifiers();
	    if (Modifier.isPublic(m))
		result.members[i].access = PUBLIC_MEMBER;
	    else
		result.members[i].access = PRIVATE_MEMBER;

	    switch (fields[i].getTypeCode()) {
	    case 'B':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_octet); //11638
		break;
	    case 'C':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_char); // 11638
		break;
	    case 'F':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_float); //11638
		break;
	    case 'D' :
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_double); //11638
		break;
	    case 'I':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_long); //11638
		break;
	    case 'J':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_longlong); //11638
		break;
	    case 'S':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_short); //11638
		break;
	    case 'Z':
		result.members[i].type = orb.get_primitive_tc(TCKind.tk_boolean); //11638
		break;
				// case '[':
				//	result.members[i].type = orb.get_primitive_tc(TCKind.tk_value_box); //11638
				//	result.members[i].id = RepositoryId.createForAnyType(fields[i].getType());
				//	break;
	    default:
		result.members[i].type = createTypeCodeForClass(orb, fields[i].getClazz(), vhandler);
		result.members[i].id = vhandler.createForAnyType(fields[i].getType());
		break;

					
	    } // end switch

	} // end for loop
		
	// Skip FVD initializers - N/A
	result.initializers = new Initializer[0];
		
	Class interfaces[] = data.forClass().getInterfaces();
	int abstractCount = 0;

	// Skip FVD supported_interfaces
	result.supported_interfaces =  new String[interfaces.length];
	for (int interfaceIndex = 0; interfaceIndex < interfaces.length;
	     interfaceIndex++) {
	    result.supported_interfaces[interfaceIndex] =
		vhandler.createForAnyType(interfaces[interfaceIndex]);
			
	    if ((!(java.rmi.Remote.class.isAssignableFrom(interfaces[interfaceIndex]))) ||
		(!Modifier.isPublic(interfaces[interfaceIndex].getModifiers())))
		abstractCount++;
	}
		
	// Skip FVD abstract_base_values - N/A
	result.abstract_base_values = new String[abstractCount];
	for (int interfaceIndex = 0; interfaceIndex < interfaces.length;
	     interfaceIndex++) {
	    if ((!(java.rmi.Remote.class.isAssignableFrom(interfaces[interfaceIndex]))) ||
		(!Modifier.isPublic(interfaces[interfaceIndex].getModifiers())))
		result.abstract_base_values[interfaceIndex] =
		    vhandler.createForAnyType(interfaces[interfaceIndex]);
		
	}
		
	result.is_truncatable = false;
		
	// Set FVD base_value
	Class superClass = data.forClass().getSuperclass();
	if (java.io.Serializable.class.isAssignableFrom(superClass))
	    result.base_value = vhandler.getRMIRepositoryID(superClass);
	else 
	    result.base_value = "";
		
	// Set FVD type
	//result.type = createTypeCodeForClass(orb, data.forClass());
	result.type = orb.get_primitive_tc(TCKind.tk_value); //11638

	return result;
		
    }
	
    private static boolean exists(String str, String strs[]){
	for (int i = 0; i < strs.length; i++)
	    if (str.equals(strs[i]))
		return true;
		
	return false;
    }

    public static boolean isAssignableFrom(String clzRepositoryId, FullValueDescription type,
					   com.sun.org.omg.SendingContext.CodeBase sender){
		
	if (exists(clzRepositoryId, type.supported_interfaces))
	    return true;

	if (clzRepositoryId.equals(type.id))
	    return true;
		
	if ((type.base_value != null) &&
	    (!type.base_value.equals(""))) {
	    FullValueDescription parent = sender.meta(type.base_value);
			
	    return isAssignableFrom(clzRepositoryId, parent, sender);
	}

	return false;

    }

    public static TypeCode createTypeCodeForClass (ORB orb, java.lang.Class c, ValueHandler vh) {

	if ( c.isArray() ) {
	    // Arrays - may recurse for multi-dimensional arrays
	    Class componentClass = c.getComponentType();
	    TypeCode embeddedType;
	    if ( componentClass.isPrimitive() ){
		embeddedType = getPrimitiveTypeCodeForClass (orb, componentClass);
	    } else {
		embeddedType = createTypeCodeForClass (orb, componentClass, vh);
	    }
	    TypeCode t = orb.create_sequence_tc (0, embeddedType);
	    String id = vh.getRMIRepositoryID(c);
	    return orb.create_value_box_tc (id, "Sequence", t);
	} else if ( c == java.lang.String.class ) {
	    // Strings
	    TypeCode t = orb.create_string_tc (0);
	    String id = vh.getRMIRepositoryID(c);
	    return orb.create_value_box_tc (id, "StringValue", t);
	} else if (java.rmi.Remote.class.isAssignableFrom(c)) {
	    return orb.get_primitive_tc(TCKind.tk_objref);
	} else if (org.omg.CORBA.Object.class.isAssignableFrom(c)) {
	    return orb.get_primitive_tc(TCKind.tk_objref);
	} 
		
	// Anything else
	String id = vh.getRMIRepositoryID(c);
	return orb.create_value_box_tc (id, "ValueBox", orb.get_primitive_tc (TCKind.tk_value));
		
    }

    public static TypeCode getPrimitiveTypeCodeForClass (ORB orb, Class c) {
		
	if (c == Integer.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_long);
	} else if (c == Byte.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_octet);
	} else if (c == Long.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_longlong);
	} else if (c == Float.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_float);
	} else if (c == Double.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_double);
	} else if (c == Short.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_short);
	} else if (c == Character.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_char);
	} else if (c == Boolean.TYPE) {
	    return orb.get_primitive_tc (TCKind.tk_boolean);
	} else {
	    // _REVISIT_ Not sure if this is right.
	    return orb.get_primitive_tc (TCKind.tk_any);
	}
    }


    /*			
	public static boolean logEnabled = true;
	public static boolean printEnabled = true;
	public static void log(String who, String what){

	if (!logEnabled) {
	if (printEnabled)
	System.out.println(who + ":" + what);
	return;
	}
	try{
	Class args[] = {String.class, String.class};
	Class target = Class.forName("javax.rmi.fvd.LogImpl");
	Method logMethod =target.getDeclaredMethod("logMssg", args);
	Object params[] = {who, what};
	logMethod.invoke(null, params);
			
	}
	catch(Throwable t){

	}
	}
	
    */

}
