/*
 * @(#)RepositoryIdWrapper.java	1.4 01/02/09
 *
 * Copyright 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package com.sun.corba.se.internal.orbutil;

// This class is used for handling ORB versioning.  For Example, in
// JDK1.3, the wrong RepIds were being generated.  This code is a
// wrapper to delegate to wrong or correct RepIdHandlers based on
// ORB version.  The methods that are being used by the code and
// which make use of the code that affects RepId calculations has
// been factored out and will be delegated based on ORB version to
// the correct RepositoryId.java

import com.sun.corba.se.internal.util.RepositoryId;

public final class RepositoryIdWrapper {
    public static String createForAnyType(org.omg.CORBA.ORB orb, Class type) {

	String repid;
	
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    // create old style RepID
	    repid = RepositoryId_1_3.createForAnyType(type);
	} else {
	    // create new style RepID
	    repid = RepositoryId.createForAnyType(type);
	}
	return repid;
    }

    public static String createForJavaType(org.omg.CORBA.ORB orb, java.io.Serializable ser)
        throws com.sun.corba.se.internal.io.TypeMismatchException
    {
	String repid;
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    // create old style RepID
	    repid = RepositoryId_1_3.createForJavaType(ser);
	} else {
	    // create new style RepID
	    repid = RepositoryId.createForJavaType(ser);
	}
	return repid;
    }

    public static String createForJavaType(org.omg.CORBA.ORB orb, Class clz)
        throws com.sun.corba.se.internal.io.TypeMismatchException
    {
	String repid;
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    // create old style RepID
	    repid = RepositoryId_1_3.createForJavaType(clz);
	} else {
	    // create new style RepID
	    repid = RepositoryId.createForJavaType(clz);
	}
	return repid;
    }

    public static String createSequenceRepID(org.omg.CORBA.ORB orb, java.lang.Object ser)
    {
	String seqRepId;
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    seqRepId = RepositoryId_1_3.createSequenceRepID(ser);
	} else {
	    seqRepId = RepositoryId.createSequenceRepID(ser);
	}
	return seqRepId;
    }

    public static String createSequenceRepID(org.omg.CORBA.ORB orb, java.lang.Class clazz)
    {
	String seqRepId;
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    seqRepId = RepositoryId_1_3.createSequenceRepID(clazz);
	} else {
	    seqRepId = RepositoryId.createSequenceRepID(clazz);
	}
	return seqRepId;
    }

    public static byte[] getByteArray(org.omg.CORBA.ORB orb, String repStr)
    {
	byte[] barray;
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    barray = RepositoryId_1_3.getByteArray(repStr);
	} else {
	    barray = RepositoryId.getByteArray(repStr);
	}
	return barray;
    }

    public static void setByteArray(org.omg.CORBA.ORB orb, String repStr, byte[] repStrBytes)
    {
	if (ORBUtility.isLegacyORB((com.sun.corba.se.internal.corba.ORB)orb)) {
	    RepositoryId_1_3.setByteArray(repStr, repStrBytes);
	} else {
	    RepositoryId.setByteArray(repStr, repStrBytes);
	}
    }
}
