/*
 * @(#)Utility.java	1.23 01/02/13
 *
 * Copyright 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.sun.corba.se.internal.util;

import org.omg.CORBA.SystemException;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.UserException;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.Any;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Principal;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.BoxedValueHelper;
import org.omg.CORBA.portable.ValueFactory;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.portable.Streamable;

import java.lang.reflect.Field;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties ;
import java.util.NoSuchElementException;
import java.rmi.Remote;
import org.omg.CORBA.ORB;
import javax.rmi.PortableRemoteObject;
import javax.rmi.CORBA.Stub;
import javax.rmi.CORBA.Tie;
import javax.rmi.CORBA.Util;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.portable.Delegate;
import java.rmi.server.RemoteStub;
import org.omg.CORBA.portable.ObjectImpl;
import java.rmi.NoSuchObjectException;
import org.omg.CORBA.INV_OBJREF;
import com.sun.org.omg.SendingContext.CodeBase;
import java.rmi.RemoteException;
import java.io.Serializable;
import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
// import com.sun.corba.se.internal.util.RepositoryId; // d11638

/**
 *  Handy class full of static functions.
 */
public final class Utility {

    public static final String STUB_PREFIX = "_";
    public static final String RMI_STUB_SUFFIX = "_Stub";
    public static final String IDL_STUB_SUFFIX = "Stub";
    public static final String TIE_SUFIX = "_Tie";
    public static final String STUB_PACKAGE_PREFIX = "org.omg.stub.";
    private static IdentityHashtable tieCache = new IdentityHashtable();
    private static IdentityHashtable tieToStubCache = new IdentityHashtable();
    private static IdentityHashtable stubToTieCache = new IdentityHashtable();
    private static Object CACHE_MISS = new Object();
  
    /**
     * Ensure that stubs, ties, and implementation objects
     * are 'connected' to the runtime. Converts implementation
     * objects to a type suitable for sending on the wire.
     * @param obj the object to connect.
     * @param orb the ORB to connect to if obj is exported to IIOP.
     * @param convertToStub true if implementation types should be
     * converted to Stubs rather than just org.omg.CORBA.Object.
     * @return the connected object.
     * @exception NoSuchObjectException if obj is an implementation
     * which has not been exported.
     */
    public static Object autoConnect(Object obj, ORB orb, boolean convertToStub) {
      
        // Is it null?
        
        if (obj == null) {
            
            // Yes, so just return it.
            
            return obj;
        }
        
        // Is it an RMI Stub?
        
        if (obj instanceof Stub) {
            
            // Yes, is it connected?
            
            Stub it = (Stub)obj;
            try {
                it._get_delegate();
            } catch (BAD_OPERATION okay) {
                    
                // No, so do it...
                    
                try {
                    it.connect(orb);
                } catch (RemoteException e) {
               
                    // The stub could not be connected because it
                    // has an invalid IOR...
                    
                    throw new INV_OBJREF(obj.getClass().getName()+
					 " did not originate from connected object");
                }
            }                
            
            // Return the connected stub...
            
            return obj;
        }
        
        // Is it a Remote object?
        
        if (obj instanceof Remote) {
	        
	    // Yes. Is this an exported IIOP implemenation
	    // object?
        
	    Remote remoteObj = (Remote)obj;
            Tie theTie = Util.getTie(remoteObj);
            if (theTie != null) {
                
                // Yes, is it connected?
                
                try {
                    theTie.orb();
                } catch (BAD_OPERATION okay) {
                    
                    // No, so do it...
                    
                    theTie.orb(orb);               
                }                
               
                if (convertToStub) {
                    
                    // Return a stub for this object...
                    
                    Object result = loadStub(theTie,null,null,true);  // d10926
                    if (result != null) {
                        return result;
                    } else {
                        throw new INV_OBJREF("Could not load stub for "+
					     obj.getClass().getName());
                    }
                    
                } else {
                    
                // Return the result of calling thisObject()
                // on the connected tie...
            
                return theTie.thisObject();
                }
            } else {
                
                // This is an implementation object which has not been
                // exported to IIOP OR is a JRMP stub or implementation
                // object which cannot be marshalled into an ORB stream...
                
                throw new INV_OBJREF(obj.getClass().getName()+
				     " not exported or is a JRMP stub");
            }
        }
        
        // Didn't need to do anything, just return the input...
        
        return obj;
    }
                                            
    /*
     * Get a new instance of an RMI-IIOP Tie for the
     * given server object.
     */
    public static Tie loadTie(Remote obj) {
    	Tie result = null;
    	Class objClass = obj.getClass();
    	
    	// Have we tried to find this guy before?
    	
        synchronized (tieCache) {
            
    	    Object it = tieCache.get(obj);
        	
    	    if (it == null) {
        	    
    	        // No, so try it...
        	    
    	        try {

    	            // First try the classname...
            	    
    	            result = loadTie(objClass);
            	        
    	            // If we don't have a valid tie at this point,
    	            // walk up the parent chain until we either
    	            // load a tie or encounter PortableRemoteObject
    	            // or java.lang.Object...

                    while (result == null &&
                           (objClass = objClass.getSuperclass()) != null &&
                           objClass != PortableRemoteObject.class &&
                           objClass != Object.class) {
                                
                        result = loadTie(objClass);   
                    }
    	        } catch (Exception ex) {}
            
                // Did we get it?
                
                if (result == null) {
                    
                    // Nope, so cache that fact...
                    
                    tieCache.put(obj,CACHE_MISS);
                    
                } else {
                    
                    // Yes, so cache it...
                    
                    tieCache.put(obj,result);
                }
            } else {
                
                // Yes, return a new instance or fail again if
                // it was a miss last time...
                
                if (it != CACHE_MISS) {
                    try {
                        result = (Tie) it.getClass().newInstance();
                    } catch (Exception e) {
                    }
                }
            }
        }
        
        return result;    
    }
    
    /*
     * Load an RMI-IIOP Tie
     */
    private static Tie loadTie(Class theClass) {
	try {
    	    String className = tieName(theClass.getName());
	    // _REVISIT_ The spec does not specify a loadingContext parameter for
	    // the following call.  Would it be useful to pass one?  
    	    Class tieClass = loadClassForClass(className, Util.getCodebase(theClass),
                                            null, theClass, theClass.getClassLoader());
    	    return (Tie) tieClass.newInstance();
        } catch (Exception err) {
            return null;    
        }
    }
 
    /*
     * Clear the stub/tie caches. Intended for use by
     * test code.
     */
    public static void clearCaches() {
        synchronized (tieToStubCache) {
            tieToStubCache.clear();
        }
        synchronized (tieCache) {
            tieCache.clear();
        }
        synchronized (stubToTieCache) {
            stubToTieCache.clear();
        }
    }
    
    /*
     * Load a class and check that it is assignable to a given type.
     * @param className the class name.
     * @param remoteCodebase the codebase to use. May be null.
     * @param loader the class loader of last resort. May be null.
     * @param expectedType the expected type. May be null.
     * @return the loaded class.
     */
    static Class loadClassOfType (String className,
                                  String remoteCodebase,
                                  ClassLoader loader,
                                  Class expectedType,
                                  ClassLoader expectedTypeClassLoader)
	throws ClassNotFoundException {
	
	Class loadedClass = null;

	try {
            //Sequence finding of the stubs according to spec
            try{
                //If-else is put here for speed up of J2EE.
                //According to the OMG spec, the if clause is not dead code.
                //It can occur if some compiler has allowed generation
                //into org.omg.stub hierarchy for non-offending
                //classes. This will encourage people to
                //produce non-offending class stubs in their own hierarchy.
                if(!PackagePrefixChecker
                   .hasOffendingPrefix(PackagePrefixChecker
                                       .withoutPackagePrefix(className))){
                    loadedClass = Util.loadClass
                        (PackagePrefixChecker.withoutPackagePrefix(className),
                         remoteCodebase,
                         loader);
                } else {
                    loadedClass = Util.loadClass
                        (className,
                         remoteCodebase,
                         loader);
                }
            } catch (ClassNotFoundException cnfe) {
                loadedClass = Util.loadClass
                    (className, 
                     remoteCodebase, 
                     loader);
            }
            if (expectedType == null)
	        return loadedClass;
	} catch (ClassNotFoundException cnfe) {
	    if (expectedType == null)
	        throw cnfe;
	}
	
        // If no class was not loaded, or if the loaded class is not of the 
	// correct type, make a further attempt to load the correct class
	// using the classloader of the expected type.
	// _REVISIT_ Is this step necessary, or should the Util,loadClass
	// algorithm always produce a valid class if the setup is correct?
	// Does the OMG standard algorithm need to be changed to include
	// this step?
        if (loadedClass == null || !expectedType.isAssignableFrom(loadedClass)) {
            if (expectedType.getClassLoader() != expectedTypeClassLoader)
                throw new IllegalArgumentException("expectedTypeClassLoader not class loader of expectedType.");

            if (expectedTypeClassLoader != null)
		loadedClass = expectedTypeClassLoader.loadClass(className);
            else
                loadedClass = Class.forName(className);
        }

	return loadedClass;
    }

    /*
     * Load a class and check that it is compatible with a given type.
     * @param className the class name.
     * @param remoteCodebase the codebase to use. May be null.
     * @param loadingContext the loading context. May be null.
     * @param relatedType the related type. May be null.
     * @return the loaded class.
     */
    public static Class loadClassForClass (String className,
                                           String remoteCodebase,
                                           ClassLoader loader,
					   Class relatedType,
                                           ClassLoader relatedTypeClassLoader)
        throws ClassNotFoundException {

        if (relatedType == null)
	    return Util.loadClass(className, remoteCodebase, loader);

	Class loadedClass = null;
	try {
	    loadedClass = Util.loadClass(className, remoteCodebase, loader);
	} catch (ClassNotFoundException cnfe) {
	    if (relatedType.getClassLoader() == null)
        	throw cnfe;
	}
	
        // If no class was not loaded, or if the loaded class is not of the 
	// correct type, make a further attempt to load the correct class
	// using the classloader of the related type.
	// _REVISIT_ Is this step necessary, or should the Util,loadClass
	// algorithm always produce a valid class if the setup is correct?
	// Does the OMG standard algorithm need to be changed to include
	// this step?
        if (loadedClass == null || 
	    (loadedClass.getClassLoader() != null &&
	     loadedClass.getClassLoader().loadClass(relatedType.getName()) != relatedType))
        {
            if (relatedType.getClassLoader() != relatedTypeClassLoader)
                throw new IllegalArgumentException("relatedTypeClassLoader not class loader of relatedType.");

            if (relatedTypeClassLoader != null)
		loadedClass = relatedTypeClassLoader.loadClass(className);
        }
	
	return loadedClass;
    }

    /*
     * Load an RMI-IIOP Stub given a Tie.
     * @param tie the tie.
     * @param stubClass the stub class. May be null.
     * @param remoteCodebase the codebase to use. May be null.
     * @param onlyMostDerived if true, will fail if cannot load a stub for the
     * first repID in the tie. If false, will walk all repIDs.
     * @return the stub or null if not found.
     */
    public static Remote loadStub (Tie tie,
                                   Class stubClass,
                                   String remoteCodebase,
                                   boolean onlyMostDerived) {
 
        StubEntry entry = null;

        // Do we already have it cached?
        
        synchronized (tieToStubCache) {
            
            Object cached = tieToStubCache.get(tie);
       
            if (cached == null) {
                
                // No, so go try to load it...
                
                entry = loadStubAndUpdateCache(tie,stubClass,remoteCodebase,onlyMostDerived);

            } else {
            
                // Yes, is it a stub?  If not, it was a miss last
                // time, so return null again...
               
                if (cached != CACHE_MISS) {
                    
                    // It's a stub.
                    
                    entry = (StubEntry) cached;
                    
                    // Does the cached stub meet the requirements
                    // of the caller? If the caller does not require
                    // the most derived stub and does not require
                    // a specific stub type, we don't have to check
                    // any further because the cached type is good
                    // enough...
                    
                    if (!entry.mostDerived && onlyMostDerived) {
                                
                        // We must reload because we do not have
                        // the most derived cached already...
                            
                        entry = loadStubAndUpdateCache(tie,null,remoteCodebase,true);
                               
                    } else if (stubClass != null && entry.stub.getClass() != stubClass) {
                                
                        // We do not have exactly the right stub. First, try to
                        // upgrade the cached stub by forcing it to the most
                        // derived stub...
                            
                        entry = loadStubAndUpdateCache(tie,null,remoteCodebase,true);

                        // If that failed, try again with the exact type
                        // we need...
                        
                        if (entry == null) {
                            entry = loadStubAndUpdateCache(tie,stubClass,remoteCodebase,onlyMostDerived);
                        }
                    
                    } else {
                                            
                        // Use the cached stub. Is the delegate set?
                        
                        try {
                            Delegate stubDel = ((ObjectImpl)entry.stub)._get_delegate();
                        } catch (Exception e2) {
                            
                            // No, so set it if we can...
                            
                            try {            
                                Delegate del = ((ObjectImpl)tie)._get_delegate();
                                ((ObjectImpl)entry.stub)._set_delegate(del);
                            } catch (Exception e) {}
                        }
                    }
                }
            }
        }
        
        if (entry != null) {
            return (Remote)entry.stub;
        } else {
            return null;
        }
    }
    
    /*
     * Load an RMI-IIOP Stub given a Tie, but do not look in the cache.
     * This method must be called with the lock held for tieToStubCache.
     * @param tie the tie.
     * @param stubClass the stub class. May be null.
     * @param remoteCodebase the codebase to use. May be null.
     * @param onlyMostDerived if true, will fail if cannot load a stub for the
     * first repID in the tie. If false, will walk all repIDs.
     * @return the StubEntry or null if not found.
     */
    private static StubEntry loadStubAndUpdateCache (Tie tie,
						     Class stubClass,
						     String remoteCodebase,
						     boolean onlyMostDerived) {
 
        Stub stub = null;
        StubEntry entry = null;

                // Were we given a stub class?
                
                if (stubClass != null) {
                    
                    // Yes, so try to instantiate it...
                    
                    try {
                stub = (Stub) stubClass.newInstance();
                    } catch (Throwable e) {
                        if (e instanceof ThreadDeath) {
                            throw (ThreadDeath) e;
                        }
                    }
                } else {
                    
                    // Nope, so we must find it. First, get the list
                    // of ids from the tie...
                    
                    String[] ids = ((ObjectImpl)tie)._ids();
                                
                    // If we were not given a remoteCodebase,
                    // get one from the tie...
                    
                    if (remoteCodebase == null) {
                remoteCodebase = Util.getCodebase(tie.getClass());
                    }
                    
                    // Now walk all the RepIDs till we find it or fail...
                    
                    for (int i = 0; i < ids.length; i++) {
            	        
    	                // Check for the java.rmi.Remote special case...
            	        
    	                if (ids[i].length() == 0) {
    	            stub = new org.omg.stub.java.rmi._Remote_Stub();
    	                    break;
    	                }
            	        
    	                // Get the classname from the RepID...
            	        
    	                String className = stubNameFromRepID(ids[i]);

    	                // Now try to load it...
            	        
    	                try {
                            // _REVISIT_ The spec does not specify a loadingContext parameter for
                            // the following call...
                            Class resultClass = null;
                            try{
                                //If-else is put here for speed up of J2EE.
                                //According to the OMG spec, the if clause is 
                                //not dead code. It can occur if the compiler 
                                //has allowed generation into org.omg.stub 
                                //hierarchy for non-offending classes. This will
                                //encourage people to produce non-offending 
                                //class stubs in their own hierarchy.
                                if(!PackagePrefixChecker
                                   .hasOffendingPrefix(PackagePrefixChecker
                                   .withoutPackagePrefix(className)))
                                {
                                    resultClass = Util.loadClass
                                        (PackagePrefixChecker.withoutPackagePrefix(className),
                                         remoteCodebase,
                                         tie.getClass().getClassLoader());
                                } else {
                                    resultClass = Util.loadClass
                                        (className,
                                         remoteCodebase,
                                         tie.getClass().getClassLoader());
                                }   
                            } catch (ClassNotFoundException cnfe){
                                resultClass = Util.loadClass
                                    (className, 
                                     remoteCodebase, 
                                     tie.getClass().getClassLoader());
                            }
                            stub = (Stub) resultClass.newInstance();
                            break;
    	                } catch (Exception e){}
            	        
    	                // If we failed to load it, see if we should continue...
            	        
    	                if (onlyMostDerived) {
            	            
    	                    // Stop here.
            	                
    	                    break;
    	                }
                    }
                }
                
        // Did we get it?
                
        if (stub != null) {
                    
                    // If the tie has a delegate set, grab it
                    // and stuff it in the stub...
                        		        
                    try {            
                        Delegate del = ((ObjectImpl)tie)._get_delegate();
                ((ObjectImpl)stub)._set_delegate(del);
                    } catch (Exception e) {
                        
                        // The tie does not have a delegate set, so stash
                        // this tie away using the stub as a key so that
                        // later, when the stub is connected, we can find
                        // and connect the tie as well...
                        
                        synchronized (stubToTieCache) {
                    stubToTieCache.put(stub,tie);
                        }
                    }
                
                    // Update the cache...
                    
            entry = new StubEntry(stub,onlyMostDerived);
            tieToStubCache.put(tie,entry);
                    
                } else {
                
                    // No, so cache the miss...
                    
                    tieToStubCache.put(tie,CACHE_MISS);
                }
            
        return entry;
                    }
                                   
    /*
     * If we loadStub(Tie,...) stashed away a tie which was
     * not connected, remove it from the cache and return
     * it.
     */
    public static Tie getAndForgetTie (Stub stub) {
        synchronized (stubToTieCache) {
            return (Tie) stubToTieCache.remove(stub);
        }
    }
    
    /*
     * Remove any cached Stub for the given tie.
     */
    public static void purgeStubForTie (Tie tie) {
        StubEntry entry;
        synchronized (tieToStubCache) {
            entry = (StubEntry)tieToStubCache.remove(tie);
        }
	if (entry != null) {
            synchronized (stubToTieCache) {
                stubToTieCache.remove(entry.stub);
            }
	}
    }
    
    /*
     * Remove cached tie/servant pair.
     */
    public static void purgeTieAndServant (Tie tie) {
	synchronized (tieCache) {
	    Object target = tie.getTarget();
	    if (target != null)
		tieCache.remove(target);
	}
    }

    /*
     * Convert a RepId to a stubName...
     */
    public static String stubNameFromRepID (String repID) {
        
        // Convert the typeid to a RepositoryId instance, get
        // the className and mangle it as needed...

        RepositoryId id = RepositoryId.cache.getId(repID);
        String className = id.getClassName();
        
        if (id.isIDLType()) {
            className = idlStubName(className);
        } else {
            className = stubName(className);
        }
        return className;
    }
  
    
    /*
     * Load an RMI-IIOP Stub.
     */
    public static Remote loadStub (ObjectImpl narrowFrom,
                                   Class narrowTo) {
        Remote result = null;
            
	try {
            
            // Get the codebase from the delegate to use when loading
            // the new stub, if possible...
            
            String codebase = null;
            try {
                //We can't assume that narrowFrom is a CORBA_2_3 ObjectImpl, yet
                //it may have a 2_3 Delegate that provides a codebase.  Swallow
                //the ClassCastException otherwise.
                Delegate delegate = narrowFrom._get_delegate();
                codebase = 
                   ((org.omg.CORBA_2_3.portable.Delegate)delegate).get_codebase(
                   narrowFrom);
            } catch (ClassCastException e) {}
    	    String stubName = stubName(narrowTo.getName());
    	    
	    // _REVISIT_ Should the narrowFrom or narrowTo class be used as the 
	    // loadingContext in the following call?  The spec says narrowFrom,
	    // but this does not seem correct...
            Class resultClass = null;
            try {
    	        resultClass = loadClassOfType(stubName,
                                                codebase, 
						narrowFrom.getClass().getClassLoader(),
                                                narrowTo,
                                                narrowTo.getClassLoader());
            } catch(ClassNotFoundException cnfe) {
    	        resultClass = loadClassOfType(STUB_PACKAGE_PREFIX + stubName,
                                                codebase, 
						narrowFrom.getClass().getClassLoader(),
                                                narrowTo,
                                                narrowTo.getClassLoader());
	    }
    	    // Create a stub instance and set the delegate...
    	    result = (Remote) resultClass.newInstance();
            ((ObjectImpl)result)._set_delegate(narrowFrom._get_delegate());
    	    
        } catch (Exception err) {}
        
        return result;
    }
        
    /*
     * Load an RMI-IIOP Stub class.
     */
    //d11638 removed unused "CodeBase sender" parameter

    public static Class loadStubClass(String repID,
                                      String remoteCodebase,
				      Class expectedType) 
	throws ClassNotFoundException {	                                    

        // Get the repID and check for "" special case.
        // We should never be called with it (See CDRInputStream
        // and the loadStub() method)...
        
        if (repID.length() == 0) {
            throw new ClassNotFoundException();   
        }
        
        // Get the stubname from the repID and load
        // the class. If we have a valid 'sender', fall
        // back to using its codebase if we need to...

        String className = Utility.stubNameFromRepID(repID);
        ClassLoader expectedTypeClassLoader = (expectedType == null ? null : expectedType.getClassLoader());
        try {
              return loadClassOfType(className,
                                       remoteCodebase,
                                       expectedTypeClassLoader,
                                       expectedType,
                                       expectedTypeClassLoader);
        } catch (ClassNotFoundException e) {
            
	    return loadClassOfType(STUB_PACKAGE_PREFIX + className,
                                   remoteCodebase,
                                   expectedTypeClassLoader,
                                   expectedType,
                                   expectedTypeClassLoader);
                    
        }
    }

    /**
     * Create an RMI stub name.
     */
    public static String stubName (String className) {
        return
            PackagePrefixChecker.hasOffendingPrefix(stubNameForCompiler(className)) ?
            PackagePrefixChecker.packagePrefix() + stubNameForCompiler(className) :
            stubNameForCompiler(className);
    }
    public static String stubNameForCompiler (String className) {
        int index = className.indexOf('$');
        if (index < 0) {
            index = className.lastIndexOf('.');
        }
        if (index > 0) {
            return 
                className.substring(0,index+1) +
                STUB_PREFIX +
                className.substring(index+1) +
                RMI_STUB_SUFFIX;
        } else {
            return STUB_PREFIX +
                className +
                RMI_STUB_SUFFIX;
        }
    }

    /**
     * Create an RMI tie name.
     */
    public static String tieName (String className) {
        return
            PackagePrefixChecker.hasOffendingPrefix(tieNameForCompiler(className)) ?
            PackagePrefixChecker.packagePrefix() + tieNameForCompiler(className) :
            tieNameForCompiler(className);
    }

    public static String tieNameForCompiler (String className) {
        int index = className.indexOf('$');
        if (index < 0) {
            index = className.lastIndexOf('.');
        }
        if (index > 0) {
            return className.substring(0,index+1) +
		STUB_PREFIX +
		className.substring(index+1) +
		TIE_SUFIX;
        } else {
            return STUB_PREFIX +
		className +
		TIE_SUFIX;
        }
    }

    /**
     * Throws the CORBA equivalent of a java.io.NotSerializableException
     */
    public static void throwNotSerializableForCorba() {
        throw new BAD_PARAM(MinorCodes.NOT_SERIALIZABLE,
                            CompletionStatus.COMPLETED_MAYBE);
    }

    /**
     * Create an IDL stub name.
     */
    public static String idlStubName(String className) {
        String result = null;
        int index = className.lastIndexOf('.');
        if (index > 0) {
            result = className.substring(0,index+1) + 
		STUB_PREFIX +
		className.substring(index+1) + 
		IDL_STUB_SUFFIX;
        } else {
            result = STUB_PREFIX +
		className +
		IDL_STUB_SUFFIX;
        }
        return result;
    }
    
    //
    // Implements all dprint calls in this package.
    //
    public static void dprint(java.lang.Object obj, String msg) {
	System.out.println(obj.getClass().getName() + "("  +
			   Thread.currentThread() + "): " + msg);
    }

    public static void dprint(String className, String msg) {
	System.out.println(className + "("  +
			   Thread.currentThread() + "): " + msg);
    }

    public void dprint(String msg) {
	Utility.dprint(this, msg);
    }

    public static void printStackTrace() 
    {
	Throwable thr = new Throwable( "Printing stack trace:" ) ;
	thr.fillInStackTrace() ;
	thr.printStackTrace() ;
    }

    /** A smarter toString that uses the built-in toString.
     * If obj isA:
     *	Object that supports toString() directly: call toString
     *   array of objects: array[length]( toString results of non-null elements ) 
     *   other: print the contents of the object's public fields.
     * Note that this method is NOT SAFE FOR USE WITH CIRCULAR REFERENCES!
     * It will go into an infinite recursion if handed the top level
     * reference to a cyclic graph of objects.
     */
    public static String objectToString( Object obj )
    {
	if (obj==null)
	    return "null" ;

	StringBuffer result = new StringBuffer() ;
	Class cls = obj.getClass() ;
	Class compClass = cls.getComponentType() ;

	if (obj instanceof java.util.Properties) {
	    Properties props = (Properties)obj ;
	    result.append( cls.getName() ) ;
	    result.append( "(" ) ;
	    Enumeration keys = props.propertyNames() ;
	    while (keys.hasMoreElements()) {
		String key = (String)(keys.nextElement()) ;
		String value = props.getProperty( key ) ;
		result.append( " " ) ;
		result.append( key ) ;
		result.append( "=" ) ;
		result.append( value ) ;
	    }

	    result.append( " )" ) ;
	    return result.toString() ;
	}

	try {
	    // If class of obj directly declares toString(), use it.
	    cls.getDeclaredMethod( "toString", null ) ;
	    return obj.toString() ;
	} catch (Exception exc) {
	    if (compClass == null) {
		// Must be some reference type, so use reflection to print fields
		Field[] fields ;

		try {
		    fields = cls.getFields() ;
		} catch (SecurityException sexc) {
		    // Can't get fields, so use default toString()
		    return obj.toString() ;
		}

		result.append( cls.getName() ) ;
		result.append( "(" ) ;

		for (int ctr=0; ctr<fields.length; ctr++ ) {
		    Field fld = fields[ctr] ;
		    result.append( " " ) ;
		    result.append( fld.getName() ) ;
		    result.append( "=" ) ;

		    try {
			Object value = fld.get( obj ) ;
			result.append( objectToString( value ) ) ;
		    } catch (Exception exc2) {
			result.append( "???" ) ;
		    }
		}

		result.append( " )" ) ;
		return result.toString() ;
	    } else // we have an array
		result.append( compClass.getName() ) ;
	    result.append( "[" ) ;
	    if (compClass == boolean.class) {
		boolean[] arr = (boolean[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == byte.class) {
		byte[] arr = (byte[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == short.class) {
		short[] arr = (short[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == int.class) {
		int[] arr = (int[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == long.class) {
		long[] arr = (long[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == char.class) {
		char[] arr = (char[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == float.class) {
		float[] arr = (float[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else if (compClass == double.class) {
		double[] arr = (double[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( arr[ctr] ) ;
		}
	    } else { // array of object
		Object[] arr = (Object[])obj ;
		result.append( arr.length ) ;
		result.append( "](" ) ;
		for (int ctr=0; ctr<arr.length; ctr++) {
		    result.append( " " ) ;
		    result.append( objectToString( arr[ctr] ) ) ;
		}
	    }

	    result.append( " ]" ) ;
	    return result.toString() ;
	}
    }

    /** Unmarshal a byte array to an integer.
	Assume the bytes are in BIGENDIAN order.
	i.e. array[offset] is the most-significant-byte
	and  array[offset+3] is the least-significant-byte.
	@param array The array of bytes.
	@param offset The offset from which to start unmarshalling.
    */
    public static int bytesToInt(byte[] array, int offset)
    {
	int b1, b2, b3, b4;

        b1 = (array[offset++] << 24) & 0xFF000000;
        b2 = (array[offset++] << 16) & 0x00FF0000;
        b3 = (array[offset++] << 8)  & 0x0000FF00;
        b4 = (array[offset++] << 0)  & 0x000000FF;

	return (b1 | b2 | b3 | b4);
    }

    /** Marshal an integer to a byte array.
	The bytes are in BIGENDIAN order.
	i.e. array[offset] is the most-significant-byte
	and  array[offset+3] is the least-significant-byte.
	@param array The array of bytes.
	@param offset The offset from which to start marshalling.
    */
    public static void intToBytes(int value, byte[] array, int offset)
    {
        array[offset++] = (byte)((value >>> 24) & 0xFF);
        array[offset++] = (byte)((value >>> 16) & 0xFF);
        array[offset++] = (byte)((value >>> 8) & 0xFF);
        array[offset++] = (byte)((value >>> 0) & 0xFF);
    }


    /**
     * Static method for writing a CORBA standard exception to a stream.
     * @param strm The OutputStream to use for marshaling.
     */
    public static void writeSystemException(SystemException ex, OutputStream strm)
    {
	String s;

	s = repositoryIdOf(ex.getClass().getName());
        strm.write_string(s);
        strm.write_long(ex.minor);
        strm.write_long(ex.completed.value());
    }


    /**
     * Static method for reading a CORBA standard exception from a stream.
     * @param strm The InputStream to use for unmarshaling.
     */
    public static SystemException readSystemException(InputStream strm)
    {
	try {
	    String name = classNameOf(strm.read_string());
	    SystemException ex = (SystemException)Class.forName(name).newInstance();
	    ex.minor = strm.read_long();
	    ex.completed = CompletionStatus.from_int(strm.read_long());
	    return ex;
	} catch ( Exception ex ) {
	    return new org.omg.CORBA.UNKNOWN(MinorCodes.UNKNOWN_DSI_SYSEX,
					     CompletionStatus.COMPLETED_MAYBE);
	}
    }


    /**
     * Get the class name corresponding to a particular repository Id.
     * This is used by the system to unmarshal (instantiate) the
     * appropriate exception class for an marshaled as the value of
     * its repository Id.
     * @param repositoryId The repository Id for which we want a class name.
     */
    public static String classNameOf(String repositoryId)
    {
	String className=null;

	className = (String) exceptionClassNames.get(repositoryId);
	if (className == null)
	    className = "org.omg.CORBA.UNKNOWN";

	return className;
    }


    /**
     * Return true if this repositoryId is a SystemException.
     * @param repositoryId The repository Id to check.
     */
    public static boolean isSystemException(String repositoryId)
    {
	String className=null;

	className = (String) exceptionClassNames.get(repositoryId);
	if (className == null)
	    return false;
	else
	    return true;
    }


    /**
     * Get the repository id corresponding to a particular class.
     * This is used by the system to write the
     * appropriate repository id for a system exception.
     * @param name The class name of the system exception.
     */
    public static String repositoryIdOf(String name)
    {
	String id;

	id = (String) exceptionRepositoryIds.get(name);
	if (id == null)
	    id = "IDL:omg.org/CORBA/UNKNOWN:1.0";

	return id;
    }

    /**
     * Get the helper for an IDLValue
     *
     * Throws MARSHAL exception if no helper found.
     */
    public static BoxedValueHelper getHelper(Class clazz, String codebase, String repId)
    {
	String className = null;
        if (clazz != null) {
	    className = clazz.getName();
	    if (codebase == null)
	        codebase = Util.getCodebase(clazz);
	} else {
	    if (repId != null) 
                className = RepositoryId.cache.getId(repId).getClassName();
	    if (className == null) // no repId or unrecognized repId
		throw new org.omg.CORBA.MARSHAL(MinorCodes.UNABLE_LOCATE_VALUE_HELPER,
                                  CompletionStatus.COMPLETED_MAYBE);
	}

    	try {
            ClassLoader clazzLoader = (clazz == null ? null : clazz.getClassLoader());
            Class helperClass = loadClassForClass(className+"Helper", codebase, clazzLoader, clazz, clazzLoader);
	    return (BoxedValueHelper)helperClass.newInstance();

    	} catch (ClassNotFoundException cnfe) {
            throw new org.omg.CORBA.MARSHAL(cnfe.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_HELPER,
                                            CompletionStatus.COMPLETED_MAYBE);
        } catch (IllegalAccessException iae) {
            throw new org.omg.CORBA.MARSHAL(iae.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_HELPER,
                                            CompletionStatus.COMPLETED_MAYBE);
        } catch (InstantiationException ie) {
            throw new org.omg.CORBA.MARSHAL(ie.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_HELPER,
                                            CompletionStatus.COMPLETED_MAYBE);
        } catch (ClassCastException cce) {
            throw new org.omg.CORBA.MARSHAL(cce.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_HELPER,
                                            CompletionStatus.COMPLETED_MAYBE);
        }    
    }

    /**
     * Get the factory for an IDLValue
     *
     * Throws MARSHAL exception if no factory found.
     */
    public static ValueFactory getFactory(Class clazz, String codebase, ORB orb, String repId)
    {
	ValueFactory factory = null;
	if ((orb != null) && (repId != null)) {
	    try {
                factory = ((org.omg.CORBA_2_3.ORB)orb).lookup_value_factory(repId);
	    } catch (org.omg.CORBA.BAD_PARAM ex) {
	        // Try other way
	    }
	}

	String className = null;
        if (clazz != null) {
	    className = clazz.getName();
	    if (codebase == null)
	        codebase = Util.getCodebase(clazz);
	} else {
	    if (repId != null) 
                className = RepositoryId.cache.getId(repId).getClassName();
	    if (className == null) // no repId or unrecognized repId
		throw new org.omg.CORBA.MARSHAL(MinorCodes.UNABLE_LOCATE_VALUE_FACTORY,
                                  CompletionStatus.COMPLETED_MAYBE);
	}

	// if earlier search found a non-default factory, or the same default
	// factory that loadClassForClass would return, bale out now... 
	if (factory != null && 
	    (!factory.getClass().getName().equals(className+"DefaultFactory") ||
	     (clazz == null && codebase == null)))
	    return factory;

    	try {
            ClassLoader clazzLoader = (clazz == null ? null : clazz.getClassLoader());
	    Class factoryClass = loadClassForClass(className+"DefaultFactory", codebase,
                                            clazzLoader, clazz, clazzLoader);
	    return (ValueFactory)factoryClass.newInstance();

    	} catch (ClassNotFoundException cnfe) {
            throw new org.omg.CORBA.MARSHAL(cnfe.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_FACTORY,
                                            CompletionStatus.COMPLETED_MAYBE);
        } catch (IllegalAccessException iae) {
            throw new org.omg.CORBA.MARSHAL(iae.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_FACTORY,
                                            CompletionStatus.COMPLETED_MAYBE);
        } catch (InstantiationException ie) {
            throw new org.omg.CORBA.MARSHAL(ie.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_FACTORY,
                                            CompletionStatus.COMPLETED_MAYBE);
        } catch (ClassCastException cce) {
            throw new org.omg.CORBA.MARSHAL(cce.toString(),
                                            MinorCodes.UNABLE_LOCATE_VALUE_FACTORY,
                                            CompletionStatus.COMPLETED_MAYBE);
        }    
    }

    /**
     * Read an object reference from the input stream and narrow
     * it to the desired type.
     * @param in the stream to read from.
     * @throws ClassCastException if narrowFrom cannot be cast to narrowTo.
     */
    public static Object readObjectAndNarrow(InputStream in,
                                             Class narrowTo)
	throws ClassCastException {
        Object result = in.read_Object();
	if (result != null) 
            return PortableRemoteObject.narrow(result, narrowTo);
	else
	    return null;
    }

    /**
     * Read an abstract interface type from the input stream and narrow
     * it to the desired type.
     * @param in the stream to read from.
     * @throws ClassCastException if narrowFrom cannot be cast to narrowTo.
     */
    public static Object readAbstractAndNarrow(org.omg.CORBA_2_3.portable.InputStream in,
                                               Class narrowTo)
	throws ClassCastException {
        Object result = in.read_abstract_interface();
	if (result != null) 
            return PortableRemoteObject.narrow(result, narrowTo);
	else
	    return null;
    }

    private static final Hashtable exceptionClassNames = new Hashtable();
    private static final Hashtable exceptionRepositoryIds = new Hashtable();

    static {

	//
	// construct repositoryId -> className hashtable
	//
	exceptionClassNames.put("IDL:omg.org/CORBA/BAD_CONTEXT:1.0",
				"org.omg.CORBA.BAD_CONTEXT");
	exceptionClassNames.put("IDL:omg.org/CORBA/BAD_INV_ORDER:1.0",
				"org.omg.CORBA.BAD_INV_ORDER");
	exceptionClassNames.put("IDL:omg.org/CORBA/BAD_OPERATION:1.0",
				"org.omg.CORBA.BAD_OPERATION");
	exceptionClassNames.put("IDL:omg.org/CORBA/BAD_PARAM:1.0",
				"org.omg.CORBA.BAD_PARAM");
	exceptionClassNames.put("IDL:omg.org/CORBA/BAD_TYPECODE:1.0",
				"org.omg.CORBA.BAD_TYPECODE");
	exceptionClassNames.put("IDL:omg.org/CORBA/COMM_FAILURE:1.0",
				"org.omg.CORBA.COMM_FAILURE");
	exceptionClassNames.put("IDL:omg.org/CORBA/DATA_CONVERSION:1.0",
				"org.omg.CORBA.DATA_CONVERSION");
	exceptionClassNames.put("IDL:omg.org/CORBA/IMP_LIMIT:1.0",
				"org.omg.CORBA.IMP_LIMIT");
	exceptionClassNames.put("IDL:omg.org/CORBA/INTF_REPOS:1.0",
				"org.omg.CORBA.INTF_REPOS");
	exceptionClassNames.put("IDL:omg.org/CORBA/INTERNAL:1.0",
				"org.omg.CORBA.INTERNAL");
	exceptionClassNames.put("IDL:omg.org/CORBA/INV_FLAG:1.0",
				"org.omg.CORBA.INV_FLAG");
	exceptionClassNames.put("IDL:omg.org/CORBA/INV_IDENT:1.0",
				"org.omg.CORBA.INV_IDENT");
	exceptionClassNames.put("IDL:omg.org/CORBA/INV_OBJREF:1.0",
				"org.omg.CORBA.INV_OBJREF");
	exceptionClassNames.put("IDL:omg.org/CORBA/MARSHAL:1.0",
				"org.omg.CORBA.MARSHAL");
	exceptionClassNames.put("IDL:omg.org/CORBA/NO_MEMORY:1.0",
				"org.omg.CORBA.NO_MEMORY");
	exceptionClassNames.put("IDL:omg.org/CORBA/FREE_MEM:1.0",
				"org.omg.CORBA.FREE_MEM");
	exceptionClassNames.put("IDL:omg.org/CORBA/NO_IMPLEMENT:1.0",
				"org.omg.CORBA.NO_IMPLEMENT");
	exceptionClassNames.put("IDL:omg.org/CORBA/NO_PERMISSION:1.0",
				"org.omg.CORBA.NO_PERMISSION");
	exceptionClassNames.put("IDL:omg.org/CORBA/NO_RESOURCES:1.0",
				"org.omg.CORBA.NO_RESOURCES");
	exceptionClassNames.put("IDL:omg.org/CORBA/NO_RESPONSE:1.0",
				"org.omg.CORBA.NO_RESPONSE");
	exceptionClassNames.put("IDL:omg.org/CORBA/OBJ_ADAPTER:1.0",
				"org.omg.CORBA.OBJ_ADAPTER");
	exceptionClassNames.put("IDL:omg.org/CORBA/INITIALIZE:1.0",
				"org.omg.CORBA.INITIALIZE");
	exceptionClassNames.put("IDL:omg.org/CORBA/PERSIST_STORE:1.0",
				"org.omg.CORBA.PERSIST_STORE");
	exceptionClassNames.put("IDL:omg.org/CORBA/TRANSIENT:1.0",
				"org.omg.CORBA.TRANSIENT");
	exceptionClassNames.put("IDL:omg.org/CORBA/UNKNOWN:1.0",
				"org.omg.CORBA.UNKNOWN");
	exceptionClassNames.put("IDL:omg.org/CORBA/OBJECT_NOT_EXIST:1.0",
				"org.omg.CORBA.OBJECT_NOT_EXIST");

	// SystemExceptions from OMG Transactions Service Spec
	exceptionClassNames.put("IDL:omg.org/CORBA/INVALID_TRANSACTION:1.0",
				"org.omg.CORBA.INVALID_TRANSACTION");
	exceptionClassNames.put("IDL:omg.org/CORBA/TRANSACTION_REQUIRED:1.0",
				"org.omg.CORBA.TRANSACTION_REQUIRED");
	exceptionClassNames.put("IDL:omg.org/CORBA/TRANSACTION_ROLLEDBACK:1.0",
				"org.omg.CORBA.TRANSACTION_ROLLEDBACK");

	// from portability RTF 98-07-01.txt
	exceptionClassNames.put("IDL:omg.org/CORBA/INV_POLICY:1.0",
				"org.omg.CORBA.INV_POLICY");


	//
	// construct className -> repositoryId hashtable
	//
	Enumeration keys = exceptionClassNames.keys();
	java.lang.Object s;
	String rId;
	String cName;

	try{
            while (keys.hasMoreElements()) {
                s = keys.nextElement();
	        rId = (String) s;
	        cName = (String) exceptionClassNames.get(rId);
	        exceptionRepositoryIds.put (cName, rId);
	    }
	} catch (NoSuchElementException e) { }
    }

    /** Parse a version string such as "1.1.6" or "jdk1.2fcs" into
	a version array of integers {1, 1, 6} or {1, 2}.
	A string of "n." or "n..m" is equivalent to "n.0" or "n.0.m" respectively.
    */
    public static int[] parseVersion(String version) {
	if (version == null)
	    return new int[0];
	char[] s = version.toCharArray();
	//find the maximum span of the string "n.n.n..." where n is an integer
	int start = 0;
	for (; start < s.length  && (s[start] < '0' || s[start] > '9'); ++start)
	    if (start == s.length)	//no digit found
		return new int[0];
	int end = start + 1;
	int size = 1;
	for (; end < s.length; ++end)
	    if (s[end] == '.')
		++size;
	    else if (s[end] < '0' || s[end] > '9')
		break;
	int[] val = new int[size];
	for (int i = 0; i < size; ++i) {
	    int dot = version.indexOf('.', start);
	    if (dot == -1 || dot > end)
		dot = end;
	    if (start >= dot)	//cases like "n." or "n..m"
		val[i] = 0;	//convert equivalent to "n.0" or "n.0.m"
	    else
		val[i] = Integer.parseInt(version.substring(start, dot));
	    start = dot + 1;
	}
	return val;
    }

    /** Compare two version arrays.
	Return 1, 0 or -1 if v1 is greater than, equal to, or less than v2.
    */
    public static int compareVersion(int[] v1, int[] v2) {
	if (v1 == null)
	    v1 = new int[0];
	if (v2 == null)
	    v2 = new int[0];
	for (int i = 0; i < v1.length; ++i) {
	    if (i >= v2.length || v1[i] > v2[i])	//v1 is longer or greater than v2
		return 1;
	    if (v1[i] < v2[i])
		return -1;
	}
	return v1.length == v2.length ? 0 : -1;
    }

    /** Compare two version strings.
	Return 1, 0 or -1 if v1 is greater than, equal to, or less than v2.
    */
    public static int compareVersion(String v1, String v2) {
	return compareVersion(parseVersion(v1), parseVersion(v2));
    }

    /** Converts an Ascii Character into Hexadecimal digit
     */
    static int hexOf( char x )
    {
        int val;

        val = x - '0';
        if (val >=0 && val <= 9)
            return val;

        val = (x - 'a') + 10;
        if (val >= 10 && val <= 15)
            return val;

        val = (x - 'A') + 10;
        if (val >= 10 && val <= 15)
            return val;

        throw new DATA_CONVERSION(MinorCodes.BAD_HEX_DIGIT,
                                  CompletionStatus.COMPLETED_NO);
    }
}

class StubEntry {
    Stub stub;
    boolean mostDerived;
    
    StubEntry(Stub stub, boolean mostDerived) {
        this.stub = stub;
        this.mostDerived = mostDerived;
    }
}
