/*
 * @(#)MacInternalFrameUI.java	1.4 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.java.swing.plaf.mac;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;
import java.util.EventListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import javax.swing.plaf.*;

/**
 * Mac implementation of JInternalFrame.  
 * <p>
 *
 * @version 1.4, 02/02/00
 * @author Steve Wilson
 */
public class MacInternalFrameUI extends BasicInternalFrameUI {

    private MacInternalFrameTitlePane titlePane;

    private PropertyChangeListener contentPaneListener;

    private static final Border handyEmptyBorder = new EmptyBorder(0,0,0,0);

    protected static String IS_PALETTE = "JInternalFrame.isPalette";

    public MacInternalFrameUI(JInternalFrame b)   {
        super(b);
    }

    public static ComponentUI createUI(JComponent c)    {
        return new MacInternalFrameUI( (JInternalFrame) c);
    }

    public void installUI(JComponent c) { 
        frame = (JInternalFrame)c;

        contentPaneListener = new ContentPaneListener();
        c.addPropertyChangeListener(contentPaneListener);

        super.installUI(c);

        Container content = frame.getContentPane();
        stripContentBorder(content);    
        c.setOpaque(false);
    }


    public void uninstallUI(JComponent c) {                  
        c.removePropertyChangeListener(contentPaneListener);

        Container cont = ((JInternalFrame)(c)).getContentPane();
        if (cont instanceof JComponent) {
            JComponent content = (JComponent)cont;
            if ( content.getBorder() == handyEmptyBorder) {
                content.setBorder(null);
            }
        }
        super.uninstallUI(c);
    } 

    private void stripContentBorder(Object c) {
        if ( c instanceof JComponent ) {
            JComponent contentComp = (JComponent)c;
            Border contentBorder = contentComp.getBorder();
            if (contentBorder == null || contentBorder instanceof UIResource) {
                contentComp.setBorder( handyEmptyBorder );
            }
        }
    }


    protected JComponent createNorthPane(JInternalFrame w) {
        titlePane = new MacInternalFrameTitlePane(w);
        return titlePane;
    } 

    protected LayoutManager createLayoutManager(){
      return new MacInternalFrameLayout();
    }

    class ContentPaneListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            String name = e.getPropertyName();
            if ( name.equals( JInternalFrame.CONTENT_PANE_PROPERTY ) ) {
                stripContentBorder(e.getNewValue());
            }
        }
    } // end class ContentPaneListener

    class MacInternalFrameLayout extends InternalFrameLayout {
        public void layoutContainer(Container c) {
            super.layoutContainer(c);
            JComponent north = getNorthPane();
            north.setBounds(north.getX()-1, 
                            north.getY(), 
                            north.getWidth()+2, 
                            north.getHeight());
        }
    }
}

