/*
 * @(#)PackageDoc.java	1.4 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.javadoc;

/**
 * Represents a java package.  Provides access to information
 * about the package, the package's comment and tags, and the
 * classes in the package.
 *
 * @since JDK1.2
 * @author Kaiyang Liu (original)
 * @author Robert Field (rewrite)
 */
public interface PackageDoc extends Doc {

    /**
     * Get all classes (including Exceptions and Errors)
     * and interfaces.
     *
     * @return all included classes and interfaces in this package.
     */
    ClassDoc[] allClasses();

    /**
     * Get ordinary classes (excluding Exceptions and Errors)
     * in this package.
     *
     * @return included classes in this package.
     */
    ClassDoc[] ordinaryClasses();

    /**
     * Get Exception classes in this package.
     *
     * @return included Exceptions in this package.
     */
    ClassDoc[] exceptions();

    /**
     * Get Error classes in this package.
     *
     * @return included Errors in this package.
     */
    ClassDoc[] errors();

    /**
     * Get Interfaces in this package.
     *
     * @return included Errors in this package.
     */
    ClassDoc[] interfaces();

    /**
     * Lookup for a class within this package.
     *
     * @return ClassDoc of found class, or null if not found.
     */
    ClassDoc findClass(String className);
}
