/*
 * @(#)ProgramElementDoc.java	1.5 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.javadoc;

/**
 * Represents a java program element: class, interface, field,
 * constructor, or method.
 * This is an abstract class dealing with information common to
 * these elements.
 *
 * @see MemberDoc
 * @see ClassDoc
 *
 * @author Robert Field
 */
public interface ProgramElementDoc extends Doc {

    /**
     * Get the containing class of this program element.
     *
     * @return a ClassDoc for this element's containing class.
     * If this is a class with no outer class, return null.
     */
    ClassDoc containingClass();

    /**
     * Get the package that this program element is contained in.
     *
     * @return a PackageDoc for this element containing package.
     * If in the unnamed package, this PackageDoc will have the
     * name "".
     */
    PackageDoc containingPackage();

    /**
     * Get the fully qualified name.
     * <pre>
     * Example:
     *  for the class java.util.Hashtable, return:
     *       java.util.Hashtable
     *  for the method bar() in class Foo in the unnamed package, return:
     *       Foo.bar()
     * </pre>
     *
     * @return the qualified name of the program element as a String.
     */
    String qualifiedName();

    /**
     * Get the modifier specifier integer.
     *
     * @see java.lang.reflect.Modifier
     */
    int modifierSpecifier();

    /**
     * Get modifiers string.
     * <pre>
     * Example, for:
     *   public abstract int foo() { ... }
     * modifiers() would return:
     *   'public abstract'
     * </pre>
     */
    String modifiers();

    /**
     * Return true if this program element is public
     */
    boolean isPublic();

    /**
     * Return true if this program element is protected
     */
    boolean isProtected();

    /**
     * Return true if this program element is private
     */
    boolean isPrivate();

    /**
     * Return true if this program element is package private
     */
    boolean isPackagePrivate();
    /**
     * Return true if this program element is static
     */
    boolean isStatic();

    /**
     * Return true if this program element is final
     */
    boolean isFinal();
}
