/*
 * @(#)Tag.java	1.7 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.javadoc;

/**
 * Represents a documentation tag, e.g. @since, @author, @version.
 * Given a tag (e.g. "@since 1.2"), holds tag name (e.g. "@since")
 * and tag text (e.g. "1.2").  Tags with structure or which require
 * special processing are handled by subclasses (ParamTag, SeeTag,
 * and ThrowsTag.
 *
 * @author Robert Field
 * @author Atul M Dambalkar
 * @see SeeTag
 * @see ParamTag
 * @see ThrowsTag
 * @see Doc#tags()
 *
 */
public interface Tag extends java.io.Serializable {

    /**
     * Return the name of this tag.
     */
    String name();

    /**
     * Return the kind of this tag.
     */
    String kind();

    /**
     * Return the text of this tag, that is, portion beyond tag name.
     */
    String text();

    /**
     * convert this object to a string.
     */
    String toString();

    /**
     * For documentation comment with embedded @link tags, return the array of
     * Tags consisting of SeeTag(s) and text containing Tag(s).
     * Within a comment string "This is an example of inline tags for a
     * documentaion comment {@link Doc {@link Doc commentlabel}}",
     * where inside the inner braces, the first "Doc" carries exctly the same
     * syntax as a SeeTag and the second "commentlabel" is label for the Html
     * Link, will return an array of Tag(s) with first element as Tag with
     * comment text "This is an example of inline tags for a documentation
     * comment" and second element as SeeTag with referenced class as "Doc"
     * and the label for the Html Link as "commentlabel".
     *
     * @return Tag[] Array of tags with inline SeeTags.
     * @see ParamTag
     * @see ThrowsTag
     */
    Tag[] inlineTags();

    /**
     * Return the first sentence of the comment as tags. 
     * Includes inline tags 
     * (i.e. {&#64link <i>reference</i>} tags)  but not
     * regular tags.  
     * Each section of plain text is represented as a {@link Tag} 
     * of kind "Text".
     * Inline tags are represented as a {@link SeeTag} of kind "link".
     * If the locale is English language, the first sentence is 
     * determined by the rules described in the Java Language 
     * Specification (first version): &quot;This sentence ends
     * at the first period that is followed by a blank, tab, or
     * line terminator or at the first tagline.&quot;, in 
     * addition a line will be terminated by paragraph and
     * section terminating HTML tags: &lt;p&gt;  &lt;/p&gt;  &lt;h1&gt; 
     * &lt;h2&gt;  &lt;h3&gt; &lt;h4&gt;  &lt;h5&gt;  &lt;h6&gt; 
     * &lt;hr&gt;  &lt;pre&gt;  or &lt;/pre&gt;.
     * If the locale is not English, the sentence end will be
     * determined by
     * {@link java.text.BreakIterator#getSentenceInstance(Locale)
     * java.text.BreakIterator.getSentenceInstance(Locale)}.
     *
     * @return an array of {@link Tag}s representing the 
     * first sentence of the comment
     */
    Tag[] firstSentenceTags();
}
