/*
 * @(#)LdapRequest.java	1.5 01/02/09
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jndi.ldap;

import java.util.Vector;
import javax.naming.CommunicationException;

final class LdapRequest {

    LdapRequest next;   // Set/read in synchronized Connection methods
    int msgId;    	// read-only

    private int gotten = 0;
    private Vector replies = new Vector(3);
    private boolean cancelled = false;

    LdapRequest(int msgId) {
	this.msgId = msgId;
    }

    synchronized void cancel() {
	cancelled = true;

	// Unblock reader of pending request
	// Should only ever have atmost one waiter
	notify();   
    }

    synchronized void addReplyBer(BerDecoder ber) {
	if (cancelled) {
	    return;
	}
	replies.addElement(ber);
	notify(); // notify anyone waiting for reply
    }

    synchronized BerDecoder getReplyBer() throws CommunicationException {
	if (cancelled) {
	    throw new CommunicationException("Request: " + msgId +
		"cancelled");
	}

	if (gotten < replies.size()) {
	    BerDecoder answer = (BerDecoder)replies.elementAt(gotten);
	    replies.setElementAt(null, gotten); // remove reference
	    ++gotten; // skip to next
	    return answer;
	} else {
	    return null;
	}
    }
}
