/*
 * @(#)UrlUtil.java	1.3 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jndi.toolkit.url;

import java.net.MalformedURLException;

/**
 * Utilities for dealing with URLs.
 * @author Vincent Ryan
 * @version 1.3 00/02/02
 */

final public class UrlUtil {

    // To prevent creation of this static class
    private UrlUtil() {
    }

    /**
     * Decode an encoded URL string (RFC 1738).
     *
     * Three-character strings "%xy", where 'xy' is the two-digit
     * hexadecimal representation of the lower 8-bits of a character,
     * are decoded into the character itself.
     */
    public static final String decode(String enc) 
	throws MalformedURLException {

        int length = enc.length();
        StringBuffer dec = new StringBuffer(length);

        for (int i = 0; i < length; i++) {
            if (enc.charAt(i) == '%') {
		i++;
		try {
		    dec.append((char)Integer.parseInt(enc.substring(i, i + 2),
			16));

		} catch (Exception e) {
		    throw new MalformedURLException(
			"Invalid URL encoding: " + enc);
		}
		i++;
            }
            else {
                dec.append(enc.charAt(i));
            }
        }

        return dec.toString();
    }
}
