/*
 * @(#)ldapURLContextFactory.java	1.5 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.jndi.url.ldap;

import javax.naming.*;
import javax.naming.directory.DirContext;
import javax.naming.spi.*;

import java.util.Hashtable;

import com.sun.jndi.ldap.LdapCtx;
import com.sun.jndi.ldap.LdapURL;

/**
 * An LDAP URL context factory.
 * 
 * @author Rosanna Lee
 * @author Scott Seligman
 * @author Vincent Ryan
 * @version 1.5 00/02/02
 */

final public class ldapURLContextFactory implements ObjectFactory {

    public Object getObjectInstance(Object urlInfo, Name name, Context nameCtx,
				    Hashtable env) throws Exception {

//System.out.println("ldapURLContextFactory " + urlInfo);
	if (urlInfo == null) {
	    return new ldapURLContext(env);
	}

	if (urlInfo instanceof String) {
	    return getUsingURL((String)urlInfo, env);
	} else if (urlInfo instanceof String[]) {
	    return getUsingURLs((String[])urlInfo, env);
	} else {
	    throw (new IllegalArgumentException(
		    "ldapURLContextFactory.getObjectInstance: " +
		    "argument must be an LDAP URL String or array of them"));
	}
    }

    static ResolveResult getUsingURLIgnoreRootDN(String url, Hashtable env)
	    throws NamingException {
	LdapURL ldapUrl = new LdapURL(url);
	DirContext ctx = new LdapCtx("", ldapUrl.getHost(), ldapUrl.getPort(), env);
	String dn = (ldapUrl.getDN() != null? ldapUrl.getDN() : "");

	// Represent DN as empty or single-component composite name.
	CompositeName remaining = new CompositeName();
	if (!"".equals(dn)) {
	    // if nonempty, add component
	    remaining.add(dn);
	}
	
	return new ResolveResult(ctx, remaining);
    }

    private static DirContext getUsingURL(String url, Hashtable env)
	    throws NamingException {
	LdapURL ldapUrl = new LdapURL(url);
	String dn = (ldapUrl.getDN() != null? ldapUrl.getDN() : "");
	return new LdapCtx(dn, ldapUrl.getHost(), ldapUrl.getPort(), env);
    }

    /*
     * Try each URL until one of them succeeds.
     * If all URLs fail, throw one of the exceptions arbitrarily.
     * Not pretty, but potentially more informative than returning null.
     */
    private static DirContext getUsingURLs(String[] urls, Hashtable env)
	    throws NamingException
    {
	NamingException ne = null;
	for (int i = 0; i < urls.length; i++) {
	    try {
		return getUsingURL(urls[i], env);
	    } catch (NamingException e) {
		ne = e;
	    }
	}
	throw ne;
    }
}
