/*
 * @(#)MidiInDevice.java	1.11 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.media.sound;

import java.util.Vector;
import java.util.StringTokenizer;	// for parsing classpath

import java.net.URL;
import java.net.MalformedURLException;

import java.io.File;
import java.io.IOException;

import javax.sound.midi.*;
			 


/******************************************************************************************
IMPLEMENTATION TODO:


******************************************************************************************/



/**
 * MidiInDevice class representing functionality of MidiIn devices.
 *
 * @version 1.11, 00/02/02
 * @author David Rivas
 * @author Kara Kytle
 */
class MidiInDevice extends AbstractMidiDevice implements Runnable {


// STATIC VARIABLES
	
	private static Thread midiInThread = null;
	private static Vector devices = new Vector();

// INSTANCE VARIABLES

	/**
	 * This is the device handle returned from native code
	 */
	//private int id = 0;

	
	/**
	 * The current short event being processed.  I just use this
	 * even over and over to avoid event creation.
	 */ 
	ShortMessage shortMessage = new ShortMessage();

	// DEVICE STATE

	private Vector transmitters	= new Vector();



// CONSTRUCTOR
	
/*
	MidiInDevice(MidiInDeviceProvider.MidiInDeviceInfo info, int mode, int[] modes) {
		
		super(info, mode, modes);

		if(Printer.trace) Printer.trace(">> MidiInDevice CONSTRUCTOR");
		if(Printer.trace) Printer.trace("<< MidiInDevice CONSTRUCTOR completed");
	}
*/

	MidiInDevice(MidiInDeviceProvider.MidiInDeviceInfo info) {
		
		super(info);

		if(Printer.trace) Printer.trace(">> MidiInDevice CONSTRUCTOR");
		if(Printer.trace) Printer.trace("<< MidiInDevice CONSTRUCTOR completed");
	}


// IMPLEMENTATION OF ABSTRACT MIDI DEVICE METHODS

	// $$kk: 06.24.99: i have this both opening and starting the midi in device.
	// may want to separate these??
	synchronized void implOpen() throws MidiUnavailableException {

		// can throw MidiUnavailableException
		id = nOpen( ((MidiInDeviceProvider.MidiInDeviceInfo)getDeviceInfo()).getIndex() );

		if (id == 0) {
			throw new MidiUnavailableException("Unable to open native device");
		}

		// add this device to our list of open devices
		devices.addElement(this);
		
		// create / start a thread to get messages

		if (midiInThread == null) {

			midiInThread = new Thread(this);
			midiInThread.start();
		}

		// can throw MidiUnavailableException
		try {
			nStart(id);
		} catch (MidiUnavailableException e) {
			devices.removeElement(this);
			throw e;
		}	
	}


	// $$kk: 06.24.99: i have this both stopping and closing the midi in device.
	// may want to separate these??
	synchronized void implClose() {

		// close all transmitters
		for(int i = 0; i < transmitters.size(); i++) {
			((Transmitter)transmitters.elementAt(i)).close();
		}
		transmitters.removeAllElements();

		// close the device

		devices.removeElement(this);
		
		int oldId = id;
		id = 0;

		nStop(oldId);
		nClose(oldId);
	}


// OVERRIDES OF ABSTRACT MIDI DEVICE METHODS

					
	public int getMaxTransmitters() {

		// $$kk: 09.27.99: need a not-specified value here
		return -1;		
	}


	public synchronized Transmitter getTransmitter() throws MidiUnavailableException {
		
		Transmitter transmitter = new MidiInTransmitter();
		transmitters.addElement(transmitter);
		return transmitter;
	}


// INNER CLASSES

	class MidiInTransmitter implements Transmitter {

		private boolean open = true;
		private Receiver receiver = null;

		public void setReceiver(Receiver receiver) {
			this.receiver = receiver;
		}


		public Receiver getReceiver() {
			return receiver;
		}
			 
		public void close() {
			transmitters.removeElement(this);
			open = false;
		}

	} // class MidiInTransmitter


// RUNNABLE METHODS

	public void run() {

		// while any device is open, keep trying to get messages.
		// this thread returns from native code whenever the queue
		// is empty.
		while(!devices.isEmpty()) {
			
			// go into native code and retrieve messages
			nGetMessages();
		}

		// let the thread exit
		midiInThread = null;
	}


// CALLBACKS FROM JAVA

	/** 
	 * Callback from native code when a short MIDI event is received from hardware.
	 * $$kk: 06.22.99: need to handle sysex messages!
	 * @param status the status byte
	 * @param data1 the first data byte, if any (otherwise -1)
	 * @param data2 the second data bytes, if any (otherwise -1)
	 * @param timeStamp time-stamp in microseconds
	 */
	void callbackShortMessage(int status, int data1, int data2, long timeStamp) {

		if(Printer.verbose) Printer.verbose(">> MidiInDevice callbackShortMessage: status: " + status + " data1: " + data1 + " data2: " + data2 + " timeStamp: " + timeStamp);


		try {
			// convert timestamp from milliseconds to microseconds (hack!!)
			// shortEvent.setMessage(status, data1, data2, timeStamp*1000);
			shortMessage.setMessage(status, data1, data2);
		} catch (InvalidMidiDataException e) {
			/* this shouldn't happen!!! */
			return;
		}

		synchronized(transmitters) {

			if (Printer.verbose) Printer.verbose(">> MidiInDevice callbackShortMessage: sending message: " + shortMessage.toString() + " timeStamp: " + timeStamp);
						
			for (int i = 0; i < transmitters.size(); i++) {
				Receiver receiver = ((Transmitter)transmitters.elementAt(i)).getReceiver();
				if (receiver != null) {
					receiver.send(shortMessage, timeStamp);
				}
			}
		}
	}


// NATIVE METHODS

	private native int nOpen(int index) throws MidiUnavailableException;
	private native void nClose(int id);

	private native void nStart(int id) throws MidiUnavailableException;
	private native void nStop(int id);

	// go into native code and get messages.  global for all MidiInDevice
	// instances.  returns whenever the queue is empty.
	private native void nGetMessages();
}
