/*
 * @(#)MidiInDeviceProvider.java	1.7 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.media.sound;

import java.util.Vector;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.Transmitter;
import javax.sound.midi.spi.MidiDeviceProvider;


/**
 * MIDI input device provider.  
 *
 * @version 1.7, 00/02/02
 * @author Kara Kytle
 */
public class MidiInDeviceProvider extends MidiDeviceProvider {


// STATIC VARIABLES
			
	
	/**
	 * Set of info objects for all MIDI input devices on the system.
	 */
	private static MidiInDeviceInfo[] infos;
	
	/**
	 * Set of all MIDI input devices on the system.
	 */
	private static MidiInDevice[] devices;

	

// STATIC

	/**
	 * Create objects representing all MIDI input devices on the system.
	 */
	static {

		if (Printer.trace) Printer.trace("MidiInDeviceProvider: static");
		
		// initialize 
		Platform.initialize();
		
		// get the number of input devices
		int numDevices = nGetNumDevices();

		// initialize the arrays
		infos = new MidiInDeviceInfo[numDevices];
		devices = new MidiInDevice[numDevices];

		// fill in the info objects now.
		// we'll fill in the device objects as they're requested.

		String name;
		String vendor;
		String description; 
		String version;

		for (int i = 0; i < infos.length; i++) {
			
			name = nGetName(i);
			vendor = nGetVendor(i);
			description = nGetDescription(i);
			version = nGetVersion(i);
			
			infos[i] = new MidiInDeviceInfo(name, vendor, description, version, i, MidiInDeviceProvider.class);
		}

		if (Printer.trace) Printer.trace("MidiInDeviceProvider: static: found numDevices: " + numDevices);
	}


// CONSTRUCTOR


	/**
	 * Required public no-arg constructor.
	 */ 
	public MidiInDeviceProvider() {
		if (Printer.trace) Printer.trace("MidiInDeviceProvider: constructor");
	}


	public MidiDevice.Info[] getDeviceInfo() {

		MidiDevice.Info[] localArray = new MidiDevice.Info[infos.length];
		System.arraycopy(infos, 0, localArray, 0, infos.length);
		return localArray;
	}	


	public MidiDevice getDevice(MidiDevice.Info info) {

		if (! (info instanceof MidiInDeviceInfo) ) {
			throw new IllegalArgumentException("MidiDevice " + info.toString() + " not supported by this provider.");
		}

		MidiInDeviceInfo midiInInfo = (MidiInDeviceInfo)info;

		for (int i = 0; i < infos.length; i++) {

			if (midiInInfo == infos[i]) { 

				int index = midiInInfo.getIndex();

				if (devices[index] == null) {
					devices[index] = new MidiInDevice(midiInInfo);
				}

				return devices[index];
			}
		}

		throw new IllegalArgumentException("MidiDevice " + info.toString() + " not supported by this provider.");
	}



// INNER CLASSES


	/**
	 * Info class for MidiInDevices.  Adds an index value for
	 * making native references to a particular device.
	 */
	static class MidiInDeviceInfo extends MidiDevice.Info {

		private int index;
		private Class providerClass;

		private MidiInDeviceInfo(String name, String vendor, String description, String version, int index, Class providerClass) {

			super(name, vendor, description, version);
			this.index = index;
			this.providerClass = providerClass;
		}


		int getIndex() {			
			return index;
		}		

	} // class MidiInDeviceInfo


// NATIVE METHODS  

	private static native int nGetNumDevices();
	private static native String nGetName(int index);
	private static native String nGetVendor(int index);
	private static native String nGetDescription(int index);
	private static native String nGetVersion(int index);
}

