/*
 * @(#)WaveFileFormat.java	1.8 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.media.sound;	  	 

import java.util.Vector;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.lang.IllegalArgumentException;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.SequenceInputStream;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;


/**
 * WAVE file format class.
 *
 * @version 1.8 00/02/02
 * @author Jan Borgersen
 */

class WaveFileFormat extends AudioFileFormat {

	/**
	 * Wave format type.
	 */
	private int waveType;

	// magic numbers
    static  final int RIFF_MAGIC         = 1380533830; 
    static  final int WAVE_MAGIC         = 1463899717; 
	static  final int FMT_MAGIC			 = 0x666d7420; // "fmt "
	static  final int DATA_MAGIC		 = 0x64617461; // "data"

	// encodings
    static final int WAVE_FORMAT_UNKNOWN   = 0x0000;
    static final int WAVE_FORMAT_PCM       = 0x0001;
    static final int WAVE_FORMAT_ADPCM     = 0x0002;
    static final int WAVE_FORMAT_ALAW      = 0x0006;
    static final int WAVE_FORMAT_MULAW     = 0x0007;
    static final int WAVE_FORMAT_OKI_ADPCM = 0x0010;
    static final int WAVE_FORMAT_DIGISTD   = 0x0015;
    static final int WAVE_FORMAT_DIGIFIX   = 0x0016;
    static final int WAVE_IBM_FORMAT_MULAW = 0x0101;
    static final int WAVE_IBM_FORMAT_ALAW  = 0x0102;
    static final int WAVE_IBM_FORMAT_ADPCM = 0x0103;
    static final int WAVE_FORMAT_DVI_ADPCM = 0x0011;
    static final int WAVE_FORMAT_SX7383    = 0x1C07;


	protected WaveFileFormat( AudioFileFormat aff ) {

		this( aff.getType(), aff.getByteLength(), aff.getFormat(), aff.getFrameLength() );
	}

	protected WaveFileFormat(AudioFileFormat.Type type, int lengthInBytes, AudioFormat format, int lengthInFrames) {
		
		super(type,lengthInBytes,format,lengthInFrames);
	
		AudioFormat.Encoding encoding = format.getEncoding();

		if( encoding==AudioFormat.Encoding.ALAW ) {
			waveType = WAVE_FORMAT_ALAW;
		} else if( encoding==AudioFormat.Encoding.ULAW ) {
			waveType = WAVE_FORMAT_MULAW;
		} else if( (encoding==AudioFormat.Encoding.PCM_SIGNED) ||
				(encoding==AudioFormat.Encoding.PCM_UNSIGNED) ) {
			waveType = WAVE_FORMAT_PCM;
		} else {
			waveType = WAVE_FORMAT_UNKNOWN;
		}
	}

	protected int getWaveType() {

		return waveType;
	}
}


