/*
 * @(#)WaveFileWriter.java	1.10 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.media.sound;	  	 

import java.util.Vector;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.lang.IllegalArgumentException;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.SequenceInputStream;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;


/**
 * WAVE file writer.
 *
 * @version 1.10, 02/02/00
 * @author Jan Borgersen
 */
public class WaveFileWriter extends SunFileWriter {

	// magic numbers
    static  final int RIFF_MAGIC         = 1380533830; 
    static  final int WAVE_MAGIC         = 1463899717; 
	static  final int FMT_MAGIC			 = 0x666d7420; // "fmt "
	static  final int DATA_MAGIC		 = 0x64617461; // "data"

	// encodings
    static final int WAVE_FORMAT_UNKNOWN   = 0x0000;
    static final int WAVE_FORMAT_PCM       = 0x0001;
    static final int WAVE_FORMAT_ADPCM     = 0x0002;
    static final int WAVE_FORMAT_ALAW      = 0x0006;
    static final int WAVE_FORMAT_MULAW     = 0x0007;
    static final int WAVE_FORMAT_OKI_ADPCM = 0x0010;
    static final int WAVE_FORMAT_DIGISTD   = 0x0015;
    static final int WAVE_FORMAT_DIGIFIX   = 0x0016;
    static final int WAVE_IBM_FORMAT_MULAW = 0x0101;
    static final int WAVE_IBM_FORMAT_ALAW  = 0x0102;
    static final int WAVE_IBM_FORMAT_ADPCM = 0x0103;
    static final int WAVE_FORMAT_DVI_ADPCM = 0x0011;
    static final int WAVE_FORMAT_SX7383    = 0x1C07;

	// known values for the naive WAVE file we're writing
	// $$jb: 03.26.99: This is bad if we want to write formats
	//                 that have longer format chunks, but it
	//                 works for now...
	static final int WAVE_HEADERSIZE	   = 44;
	static final int WAVE_DATASIZE_OFFSET  = 40;

	/**
	 * WAVE type
	 */

	public static final AudioFileFormat.Type types[] = {
		AudioFileFormat.Type.WAVE
	};

	
	/**
	 * Constructs a new WaveFileWriter object.
	 */
	public WaveFileWriter() {
	}


// METHODS TO IMPLEMENT AudioFileWriter


/*	public boolean isFileTypeSupported(AudioFileFormat.Type fileType, AudioInputStream stream) {

		AudioFileFormat.Type fileTypes[] = getAudioFileTypes(stream);

		for( int i=0; i< fileTypes.length; i++ ) {
			if( fileTypes[i] == fileType ) {
				return true;
			}
		}

		// none matched.
		return false;
	}
*/

/*
	public boolean isFileFormatSupported(AudioFileFormat fileFormat, AudioInputStream stream) {

		AudioFormat format = fileFormat.getFormat();
		AudioFormat affFormat = null;


		for( int i=0; i< affs.length; i++ ) {

			if( affs[i].getType() == fileFormat.getType() ) {
				
				affFormat = affs[i].getFormat();

				if( format.getEncoding() == affFormat.getEncoding() ) {

					if( format.isBigEndian() == affFormat.isBigEndian() ) {
					
						if( (format.getChannels()==AudioSystem.NOT_SPECIFIED) ||
							(format.getChannels()==affFormat.getChannels()) ) {

							if( (format.getFrameRate()==AudioSystem.NOT_SPECIFIED) ||
								(format.getFrameRate()==affFormat.getFrameRate()) ) {

								if( (format.getFrameSize()==AudioSystem.NOT_SPECIFIED) ||
									(format.getFrameSize()==affFormat.getFrameSize()) ) {

									if( (format.getSampleRate()==AudioSystem.NOT_SPECIFIED) ||
										(format.getSampleRate()==affFormat.getSampleRate()) ) {

										if( (format.getSampleSizeInBits()==AudioSystem.NOT_SPECIFIED) ||
											(format.getSampleSizeInBits()==affFormat.getSampleSizeInBits()) ) {

											return true;
										}
									}
								}
							}
						}
					}
				}
			}
		}

		// missed a match, return false
		return false;
	}
*/


	public AudioFileFormat.Type[] getAudioFileTypes(AudioInputStream stream) {

		AudioFileFormat.Type[] filetypes = new AudioFileFormat.Type[types.length];
		System.arraycopy(types, 0, filetypes, 0, types.length);

		// make sure we can write this stream
		AudioFormat format = stream.getFormat();
		AudioFormat.Encoding encoding = format.getEncoding();

		if( (encoding==AudioFormat.Encoding.ALAW) ||
			(encoding==AudioFormat.Encoding.ULAW) ||
			(encoding==AudioFormat.Encoding.PCM_SIGNED) ||
			(encoding==AudioFormat.Encoding.PCM_UNSIGNED) ) {

			return filetypes;
		}
		
		return new AudioFileFormat.Type[0];
	}


	public int write(AudioInputStream stream, AudioFileFormat.Type fileType, OutputStream out) throws IOException {

		// we must know the total data length to calculate the file length
		if( stream.getFrameLength() == AudioSystem.NOT_SPECIFIED ) {
			throw new IOException("stream length not specified");
		}

		// throws IllegalArgumentException if not supported
		WaveFileFormat waveFileFormat = (WaveFileFormat)getAudioFileFormat(fileType, stream);

		int bytesWritten = writeWaveFile(stream, waveFileFormat, out);
		return bytesWritten;
	}


	public int write(AudioInputStream stream, AudioFileFormat.Type fileType, File out) throws IOException {

		// throws IllegalArgumentException if not supported
		WaveFileFormat waveFileFormat = (WaveFileFormat)getAudioFileFormat(fileType, stream);
		
		// first write the file without worrying about length fields
		FileOutputStream fos = new FileOutputStream( out );	// throws IOException
		BufferedOutputStream bos = new BufferedOutputStream( fos, bisBufferSize );
		int bytesWritten = writeWaveFile(stream, waveFileFormat, bos );
		bos.close();

		// now, if length fields were not specified, calculate them,
		// open as a random access file, write the appropriate fields, 
		// close again....
		if( waveFileFormat.getByteLength()== AudioSystem.NOT_SPECIFIED ) {
			
			// $$kk: 10.22.99: jan: please either implement this or throw an exception!
			// TODO
		}

		return bytesWritten;
	}

//--------------------------------------------------------------------

	/**
	 * Returns the AudioFileFormat describing the file that will be written from this AudioInputStream.
	 * Throws IllegalArgumentException if not supported.
	 */
	private AudioFileFormat getAudioFileFormat(AudioFileFormat.Type type, AudioInputStream stream) {

		AudioFormat format = null;
		WaveFileFormat fileFormat = null;
		AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;

		AudioFormat streamFormat = stream.getFormat();
		AudioFormat.Encoding streamEncoding = streamFormat.getEncoding();


		float sampleRate;
		int sampleSizeInBits;
		int channels;
		int frameSize;
		float frameRate;
		int fileSize;

		if( type != types[0] ) {
			throw new IllegalArgumentException("File type " + type + " not supported.");
		}

		if( (streamEncoding==AudioFormat.Encoding.ALAW) ||
			(streamEncoding==AudioFormat.Encoding.ULAW) ) {

			encoding = streamEncoding;
			sampleSizeInBits = streamFormat.getSampleSizeInBits();

		} else if ( streamFormat.getSampleSizeInBits()==8 ) {

			encoding = AudioFormat.Encoding.PCM_UNSIGNED;
			sampleSizeInBits=8;

		} else {

			encoding = AudioFormat.Encoding.PCM_SIGNED;
			sampleSizeInBits=streamFormat.getSampleSizeInBits();
		}
		

		format = new AudioFormat( encoding,	
						streamFormat.getSampleRate(),	
						sampleSizeInBits, 
						streamFormat.getChannels(),
						streamFormat.getFrameSize(), 
						streamFormat.getFrameRate(), 
						false);	// WAVE is little endian

		if( stream.getFrameLength()!=AudioSystem.NOT_SPECIFIED ) {
			fileSize = (int)stream.getFrameLength()*streamFormat.getFrameSize() + WAVE_HEADERSIZE;
		} else {
			fileSize = AudioSystem.NOT_SPECIFIED;
		}

		fileFormat = new WaveFileFormat( AudioFileFormat.Type.WAVE,
								fileSize,
								format,
								(int)stream.getFrameLength() );

		return fileFormat;
	}

	
	private int writeWaveFile(InputStream in, WaveFileFormat waveFileFormat, OutputStream out) throws IOException {

		int bytesRead = 0;
		int bytesWritten = 0;
		InputStream fileStream = getFileStream(waveFileFormat, in);
		byte buffer[] = new byte[bisBufferSize];
		int maxLength = waveFileFormat.getByteLength();

		while( (bytesRead = fileStream.read( buffer )) >= 0 ) {
			
			if (maxLength>0) {
				if( bytesRead < maxLength ) {
					out.write( buffer, 0, (int)bytesRead );
					bytesWritten += bytesRead;
					maxLength -= bytesRead;
				} else {
					out.write( buffer, 0, (int)maxLength );
					bytesWritten += maxLength;
					maxLength = 0;
					break;
				}
			} else {
				out.write( buffer, 0, (int)bytesRead );
				bytesWritten += bytesRead;
			}
		}

		return bytesWritten;
	}

	private InputStream getFileStream(WaveFileFormat waveFileFormat, InputStream audioStream) throws IOException {

		// private method ... assumes audioFileFormat is a supported file type

		// WAVE header fields
		AudioFormat audioFormat = waveFileFormat.getFormat();
		int headerLength       = 44;
		int riffMagic          = WaveFileFormat.RIFF_MAGIC;
		int waveMagic          = WaveFileFormat.WAVE_MAGIC;
		int fmtMagic           = WaveFileFormat.FMT_MAGIC;
		int fmtLength          = 16;
		short wav_type         = (short) waveFileFormat.getWaveType();
		short channels         = (short) audioFormat.getChannels();
		short sampleSizeInBits = (short) audioFormat.getSampleSizeInBits();
		int sampleRate         = (int) audioFormat.getSampleRate();
		int frameSizeInBytes   = (int) audioFormat.getFrameSize();
		int frameRate		   = (int) audioFormat.getFrameRate();
		int avgBytesPerSec     = channels * sampleSizeInBits * sampleRate / 8;;
		short blockAlign       = (short) ((sampleSizeInBits / 8) * channels);		
		int dataMagic		   = WaveFileFormat.DATA_MAGIC;
		int dataLength		   = waveFileFormat.getFrameLength() * frameSizeInBytes;
		int length			   = waveFileFormat.getByteLength();
		int riffLength = dataLength + headerLength - 8;

		byte header[] = null;
		ByteArrayInputStream headerStream = null;
		ByteArrayOutputStream baos = null;
		DataOutputStream dos = null;
		SequenceInputStream waveStream = null;

		AudioFormat audioStreamFormat = null;
		AudioFormat.Encoding encoding = null;
		InputStream codedAudioStream = audioStream;


		// if audioStream is an AudioInputStream and we need to convert, do it here...

		if(audioStream instanceof AudioInputStream) {
		
			audioStreamFormat = ((AudioInputStream)audioStream).getFormat();
			
			encoding = audioStreamFormat.getEncoding();

			if(encoding==AudioFormat.Encoding.PCM_SIGNED) {
				if( sampleSizeInBits==8 ) {
					wav_type = WaveFileFormat.WAVE_FORMAT_PCM;
					// plug in the transcoder to convert from PCM_SIGNED to PCM_UNSIGNED
					codedAudioStream = AudioSystem.getAudioInputStream( new AudioFormat (
																	AudioFormat.Encoding.PCM_UNSIGNED,
																	audioStreamFormat.getSampleRate(),
																	audioStreamFormat.getSampleSizeInBits(),
																	audioStreamFormat.getChannels(),
																	audioStreamFormat.getFrameSize(),
																	audioStreamFormat.getFrameRate(),
																	false),
																	(AudioInputStream)audioStream);	
				} 
			}
			if( ((encoding==AudioFormat.Encoding.PCM_SIGNED) && audioStreamFormat.isBigEndian()) ||
					   ((encoding==AudioFormat.Encoding.PCM_UNSIGNED) && !audioStreamFormat.isBigEndian()) ||
					   ((encoding==AudioFormat.Encoding.PCM_UNSIGNED) && audioStreamFormat.isBigEndian()) ) {
				if( sampleSizeInBits!=8) {
					wav_type = WaveFileFormat.WAVE_FORMAT_PCM;
					// plug in the transcoder to convert to PCM_SIGNED_LITTLE_ENDIAN
					codedAudioStream = AudioSystem.getAudioInputStream( new AudioFormat (
																	AudioFormat.Encoding.PCM_SIGNED,
																	audioStreamFormat.getSampleRate(),
																	audioStreamFormat.getSampleSizeInBits(),
																	audioStreamFormat.getChannels(),
																	audioStreamFormat.getFrameSize(),
																	audioStreamFormat.getFrameRate(),
																	false),
																	(AudioInputStream)audioStream);
				} 
			}
		}


		// Now push the header into a stream, concat, and return the new SequenceInputStream

		baos = new ByteArrayOutputStream();
		dos = new DataOutputStream(baos);

		// we write in littleendian...
		dos.writeInt(riffMagic);
		dos.writeInt(big2little( riffLength ));
		dos.writeInt(waveMagic);
		dos.writeInt(fmtMagic);
		dos.writeInt(big2little(fmtLength));
		dos.writeShort(big2littleShort(wav_type));
		dos.writeShort(big2littleShort(channels));
		dos.writeInt(big2little(sampleRate));
		dos.writeInt(big2little(avgBytesPerSec));
		dos.writeShort(big2littleShort(blockAlign));  
		dos.writeShort(big2littleShort(sampleSizeInBits));

		dos.writeInt(dataMagic);
		dos.writeInt(big2little(dataLength));			

		dos.close();
		header = baos.toByteArray();
		headerStream = new ByteArrayInputStream( header );


		waveStream = new SequenceInputStream(headerStream,codedAudioStream);

		return (InputStream)waveStream;

	}


}
