/*
 * @(#)Token.java	1.8 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * COMPONENT_NAME: idl.parser
 *
 * ORIGINS: 27
 *
 * Licensed Materials - Property of IBM
 * 5639-D57 (C) COPYRIGHT International Business Machines Corp. 1997, 1999
 * RMI-IIOP v1.0
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * @(#)Token.java	1.8 00/02/02
 */

package com.sun.tools.corba.se.idl;

// NOTES:
// -F46082.51<daz> Remove -stateful feature.
// -D59166<daz> Add support for keyword/identifier collision detection.  This
//  feature is implemented here, rather than class Scanner, to allow the Parser
//  to handle the problem.
// -F60858.1<daz> Support -corba option, level <= 2.2: identify 2.3 keywords.
// -D62023<daz> Support -corba option, level <= 2.3, identify 2.4 keywords.
//
// Should escaped Identifier should be a type rather than an attribute?
//

/**
 * Class Token represents a lexeme appearing within an IDL source.  Every
 * Token has a type.  Depending on its type and on the supported version
 * of IDL, a Token will have other associated attributes, such as a name
 * (identifier, e.g.), and whether it is escaped, deprecated, or is a type
 * that is known to be in a future version of IDL.
 **/
class Token
{
  ///////////////
  // Available types

  static final int                // Keywords
      Any                  =   0, // 2.2
      Attribute            =   1, // |
      Boolean              =   2, // .
      Case                 =   3, // .
      Char                 =   4, // .
      Const                =   5,
      Context              =   6,
      Default              =   7,
      Double               =   8,
      Enum                 =   9,
      Exception            =  10,
      FALSE                =  11,
      Float                =  12,
      In                   =  13,
      Inout                =  14,
      Interface            =  15,
      Long                 =  16,
      Module               =  17,
      Object               =  18,
      Octet                =  19,
      Oneway               =  20,
      Out                  =  21,
      Raises               =  22,
      Readonly             =  23,
      Sequence             =  24,
      Short                =  25,
      String               =  26,
      Struct               =  27,
      Switch               =  28,
      TRUE                 =  29,
      Typedef              =  30,
      Unsigned             =  31, // .
      Union                =  32, // .
      Void                 =  33, // .
      Wchar                =  34, // |
      Wstring              =  35, // 2.2
      // <f46082.40> New OBV keywords...
      // <d62023> In 2.4rtf, "factory" is synonymous to "init" in 2.3
      Init                 =  36, // 2.3 only
      Abstract             =  37, // 2.3        2.4rtf
      Custom               =  38, // |          |
      Private              =  39, // |          |
      Public               =  40, // |          |
      Supports             =  41, // |          |
      Truncatable          =  42, // |          |
      ValueBase            =  43, // |          |
      Valuetype            =  44, // 2.3        2.4rtf
      Factory              =  45, //            2.4rtf only
                                     // <f46082.51> Remove -stateful feature.
      //Protected            =  46,  // Deprecated "stateful" - remove ASAP
      //State                =  47,

      Identifier           =  60, // Identifier
      MacroIdentifier      =  61, // Macro Identifier

      Semicolon            = 100, // Symbols
      LeftBrace            = 101,
      RightBrace           = 102,
      Colon                = 103,
      Comma                = 104,
      Equal                = 105,
      Plus                 = 106,
      Minus                = 107,
      LeftParen            = 108,
      RightParen           = 109,
      LessThan             = 110,
      GreaterThan          = 111,
      LeftBracket          = 112,
      RightBracket         = 113,
      Apostrophe           = 114,
      Quote                = 115,
      Backslash            = 116,
      Bar                  = 117,
      Carat                = 118,
      Ampersand            = 119,
      Star                 = 120,
      Slash                = 121,
      Percent              = 122,
      Tilde                = 123,
      DoubleColon          = 124,
      ShiftLeft            = 125,
      ShiftRight           = 126,
      Period               = 127,
      Hash                 = 128,
      Exclamation          = 129,
      DoubleEqual          = 130,
      NotEqual             = 131,
      GreaterEqual         = 132,
      LessEqual            = 133,
      DoubleBar            = 134,
      DoubleAmpersand      = 135,

      BooleanLiteral       = 200, // Literals
      CharacterLiteral     = 201,
      IntegerLiteral       = 202,
      FloatingPointLiteral = 203,
      StringLiteral        = 204,
      Literal              = 205,

      Define               = 300, // Directives
      Undef                = 301,
      If                   = 302,
      Ifdef                = 303,
      Ifndef               = 304,
      Else                 = 305,
      Elif                 = 306,
      Include              = 307,
      Endif                = 308,
      Line                 = 309,
      Error                = 310,
      Pragma               = 311,
      Null                 = 312,
      Unknown              = 313,

      Defined              = 400,

      // <f46082.40> Keyword identifiers.
      //Abstract             = 500,
      //Custom               = 501,
      //Init                 = 502,
      //Private2             = 503,
      //Public2              = 504,
      //Supports             = 505,
      //Truncatable          = 506,
      //ValueBase            = 507,
      //Valuetype            = 508,

      EOF                  = 999; // End of Input

  // Available types
  ///////////////
  // Keywords

  static final String [] Keywords = {
      "any",         "attribute",    "boolean",
      "case",        "char",         "const",
      "context",     "default",      "double",
      "enum",        "exception",    "FALSE",
      "float",       "in",           "inout",
      "interface",   "long",         "module",
      "Object",      "octet",        "oneway",
      "out",         "raises",       "readonly",
      "sequence",    "short",        "string",
      "struct",      "switch",       "TRUE",
      "typedef",     "unsigned",     "union",
      "void",        "wchar",        "wstring",
      "init", // In 2.3 only
      "abstract",     "custom",      "private",      // 2.3 and 2.4rtf
      "public",       "supports",    "truncatable",
      "ValueBase",    "valuetype",
      "factory"}; // In 2.4rtf only
      //"protected",   "state"};  // <f46082.51> Remove -stateful feature.

  // <f46082.40> Remove keyword identifiers.
  //static final int
  //    FirstKeywordIdentifier = 500,
  //    LastKeywordIdentifier  = Valuetype;
  //
  //static final String[] KeywordIdentifiers = {
  //    "abstract",    "custom",    "init",
  //    "private",     "public",    "supports",
  //    "truncatable", "valueBase", "valuetype"};

  /**
   * Determine whether this token is a keyword.
   * @return true iff this token is a keyword.
   **/
  boolean isKeyword ()
  {
    return type >= FirstKeyword && type <= LastKeyword;
  } // isKeyword

  static final int
      FirstKeyword = Any, // 0
      LastKeyword  = Factory;
      //LastKeyword  = State;  // <f46082.51>  Remove -stateful feature.

  // <f60858.1> Keywords in CORBA 2.2 that we support.
  static final int
      First22Keyword = Any, // 0
      Last22Keyword  = Wstring;

  // <f60858.1> New keywords in CORBA 2.3 (preliminary) that we support.
  static final int
      First23Keyword = Init,
      Last23Keyword  = Valuetype;

  // <d62023> New keywords in CORBA 2.4rtf (accepted 2.3) that we support.
  // Note that "factory" replaces "init".  Scanner must account for this in
  // keyword scan.
  static final int
      First24rtfKeyword = Abstract,
      Last24rtfKeyword  = Factory;

  // Keywords
  ///////////////
  // Symbols

  static final int
      FirstSymbol = 100,
      LastSymbol  = 199;

  static final String [] Symbols = {
      ";",  "{",  "}",  ":", ",", "=", "+",  "-",
      "(",  ")",  "<",  ">", "[", "]", "'",  "\"",
      "\\", "|",  "^",  "&", "*", "/", "%",  "~",
      "::", "<<", ">>", ".", "#", "!", "==", "!=",
      ">=", "<=", "||", "&&"};

  // Symbols
  ///////////////
  // Literals

  static final int
      FirstLiteral = 200,
      LastLiteral  = 299;

  static final String [] Literals = {
      Util.getMessage ("Token.boolLit"),
      Util.getMessage ("Token.charLit"),
      Util.getMessage ("Token.intLit"),
      Util.getMessage ("Token.floatLit"),
      Util.getMessage ("Token.stringLit"),
      Util.getMessage ("Token.literal")};

  // Literals
  ///////////////
  // Directives

  /**
   * Determine whether this token is a preprocessor directive.
   * @return true iff this token is a preprocessor directive.
   **/
  boolean isDirective ()
  {
    return type >= FirstDirective && type <= LastDirective;
  } // isDirective

  static final int
      FirstDirective = 300,
      LastDirective  = 399;

  static final String [] Directives = {
      "define", "undef",  "if",
      "ifdef",  "ifndef", "else",
      "elif",   "include","endif",
      "line",   "error",  "pragma",
      ""};

  // Directives
  ///////////////
  // Specials

  static final int
      FirstSpecial = 400,
      LastSpecial  = 499;

  static final String [] Special = {
      "defined"};

  // Specials
  ///////////////

  /**
   * Constructor.
   * @return a Token of the supplied type.
   **/
  Token (int tokenType)
  {
    type = tokenType;
  } // ctor

  // <d62023>
  /**
   * Constructor.
   * @return a Token having the supplied attributes.
   **/
  Token (int tokenType, boolean deprecated)
  {
    this.type = tokenType;
    this.isDeprecated = deprecated;
  } // ctor

  /**
   * Constructor.
   * @return a Token having the supplied attributes.
   **/
  Token (int tokenType, String tokenName)
  {
    type = tokenType;
    name = tokenName;
  } // ctor

  /**
   * Constructor.
   * @return a Token having the supplied attribtues.
   *  having
   **/
  Token (int tokenType, String tokenName, boolean escaped)
  {
    this (tokenType, tokenName);
    this.isEscaped = escaped;
  } // ctor

  // <d62023>
  /**
   * Constructor.
   * @return a Token having the supplied attributes.
   **/
  Token (int tokenType, String tokenName, boolean escaped,
      boolean collidesWithKeyword, boolean deprecated)
  {
    this (tokenType, tokenName);
    this.isEscaped = escaped;
    this.collidesWithKeyword = collidesWithKeyword;
    this.isDeprecated = deprecated;
  } // ctor

  // <f46082.40> Remove keyword identifiers.
  ///**
  // * Constructor.
  // * @return a Token having the supplied attributes.
  // **/
  //Token (int tokenType, int tokenSubType, String tokenName)
  //{
  //  type    = tokenType;
  //  subType = tokenSubType;
  //  name    = tokenName;
  //} // ctor

  /**
   * Get the String representation of this Token.
   * @return a String containing representation of this Token.
   **/
  public String toString ()
  {
    if (type == Identifier)
      return name;
    if (type == MacroIdentifier)
      return name + '(';
    return Token.toString (type);
  } // toString

  /**
   * Get the String representation of a supplied Token type.
   * @return A String containing the name of the supplied Token type.
   **/
  static String toString (int type)
  {
    if (type <= LastKeyword)
      return Keywords[type];
    // <f46082.40> Remove keyword identifiers.
    //if ( (type >= FirstKeywordIdentifier) && (type <= LastKeywordIdentifier) )
    //  return KeywordIdentifiers[ type - FirstKeywordIdentifier ];
    if (type == Identifier || type == MacroIdentifier)
      return Util.getMessage ("Token.identifier");
    if (type <= LastSymbol)
      return Symbols[type - FirstSymbol];
    if (type <= LastLiteral)
      return Literals[type - FirstLiteral];
    if (type <= LastDirective)
      return Directives[type - FirstDirective];
    if (type <= LastSpecial)
      return Special[type - FirstSpecial];
    if (type == EOF)
      return Util.getMessage ("Token.endOfFile");
    return Util.getMessage ("Token.unknown");
  } // toString

  ///////////////
  // Accessors and Predicates

  /**
   * Determine whether this token equals a supplied token.
   * @return true iff the types and names of this and the supplied
   * Token are equal.
   **/
  boolean equals (Token that)
  {
    if (this.type == that.type)
      if (this.name == null)
        return that.name == null;
      else
        return this.name.equals (that.name);
    return false;
  } // equals

  /**
   * Determine whether the this token is of a supplied type.
   * @return true iff the type of this Token equals that supplied.
   **/
  boolean equals (int type)
  {
    return this.type == type;
  } // equals

  /**
   * Determine whether this identifier has the supplied name.
   * @return true iff this Token is an identifier having the supplied name.
   **/
  boolean equals (String name)
  {
    return (this.type == Identifier && this.name.equals (name));
  } // equals

  // Although isEscaped is an independent attribute, it may be true only
  // when type is Identifer.
  /**
   * Accessor.
   * @return true iff this token is an escaped identifier.
   **/
  public boolean isEscaped ()
  {
    return type == Identifier && isEscaped;
  } // isEscaped

  // <d62023>
  /**
   * Accessor.
   * @return true iff this token is an identifier having a name matching
   * a keyword in a version of CORBA greater than the specified CORBA level,
   * or iff it matches a keyword in letter, but note case.
   **/
  public boolean collidesWithKeyword ()
  {
    return collidesWithKeyword;
  } // collidesWithKeyword

  // <d62023> Storing deprecation information in a token seems a natural
  // means to notify the parser about deprecated types.
  /**
   * Accessor.
   * @return true iff this token is a deprecated lexeme or lexical type with
   * respect to the specified CORBA level.
   **/
  public boolean isDeprecated ()
  {
    return isDeprecated;
  }
  // isDeprecated

  // <d59166><d62023> It's more efficient if Scanner determines this attribute.
  /**
   * Determine whether this token collides with an IDL keyword.
   **/
  //public boolean collidesWithKeyword ()
  //{
  //  if (name != null && type == Identifier && !isEscaped)
  //  {
  //    String lcName = name.toLowerCase ();
  //    for (int i = FirstKeyword; i <= LastKeyword; ++i)
  //      if (lcName.equals (Token.Keywords [i].toLowerCase ()))
  //        return true;
  //  }
  //  return false;
  //} // collidesWithKeyword

  // Accessors and Predicates
  ///////////////

  /**
   * Code identifying the lexical class to which this token belongs, e.g.,
   * Keyword, Identifier, ...
   **/
  int type;
  /**
   * Lexeme extracted from the source for this token.
   **/
  String name = null;
  /**
   * Source comment associated with this token.
   **/
  Comment comment = null;
  /**
   * True iff this token is an escaped identifier.
   **/
  boolean isEscaped = false; // <d59165>
  /**
   * True iff this token is an identifier that is known to be a keyword
   * in another version of CORBA or matches a keyword in letter, but not case.
   **/
  boolean collidesWithKeyword = false;  // <d62023>
  /**
   * True iff this token is deprecated.
   **/
  boolean isDeprecated = false;  // <d62023>
  // <f46082.40> Remove keyword identifier implementation.
  ///**
  // * Non-zero only when type = [Macro]Identifier
  // **/
  //int subType = 0;
} // class Token

