/*
 * @(#)Skeleton.java	1.9 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * COMPONENT_NAME: idl.toJava
 *
 * ORIGINS: 27
 *
 * Licensed Materials - Property of IBM
 * 5639-D57 (C) COPYRIGHT International Business Machines Corp. 1997, 1999
 * RMI-IIOP v1.0
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * @(#)Skeleton.java	1.8 99/08/18
 */

package com.sun.tools.corba.se.idl.toJavaPortable;

// NOTES:
// -F46082.51<daz> Remove -stateful feature; javaStatefulName() obsolete.
// -D57147   <klr> Make _Tie implement org.omg.CORBA.portable.InvokeHandler
// -D58037   <klr> Make _Tie delegate to Operations interface
// -D62739   <klr> no TIE for values that support abstract interfaces, etc.

import java.io.File;
import java.io.PrintWriter;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.sun.tools.corba.se.idl.AttributeEntry;
import com.sun.tools.corba.se.idl.GenFileStream;

import com.sun.tools.corba.se.idl.InterfaceEntry;
import com.sun.tools.corba.se.idl.InterfaceState;
import com.sun.tools.corba.se.idl.MethodEntry;
import com.sun.tools.corba.se.idl.SymtabEntry;
import com.sun.tools.corba.se.idl.TypedefEntry;
import com.sun.tools.corba.se.idl.ValueEntry;

/**
 *
 **/
public class Skeleton implements AuxGen
{
  /**
   * Public zero-argument constructor.
   **/
  public Skeleton ()
  {
  } // ctor

  /**
   *
   **/
  public void generate (Hashtable symbolTable, SymtabEntry entry)
  {
    // <d62739-begin> 
    // Per Simon, 5-12-99, don't generate TIE or Skeleton for 
    //
    // 1) valuetypes supporting abstract interfaces 
    // 2) valuetypes with no supports.
    // 3) abstract interfaces
    //
    if (entry instanceof ValueEntry) 
    {
      ValueEntry v = (ValueEntry) entry;
      if ((v.supports ().size () == 0) ||
          ((InterfaceEntry) v.supports ().elementAt (0)).isAbstract ()) {
        return;
	}
    }
    if (((InterfaceEntry) entry).isAbstract ()) {
    	return;
    }
    // <d62739-end>

    this.symbolTable = symbolTable;

    this.i           = (InterfaceEntry)entry;
    init ();
    openStream ();
    if (stream == null)
      return;
    writeHeading ();
    writeBody ();
    writeClosing ();
    closeStream ();
  } // generate

  /**
   * Initialize members unique to this generator.
   **/
  protected void init ()
  {
    tie = ((Arguments)Compile.compiler.arguments).TIEServer;
    if (tie)
      className = i.name () + "_Tie";
    else
      className = i.name () + "ImplBase";
    // the interface name that the Skeleton or Tie class implements

    // -F46082.51<daz> Remove -stateful feature.
    //intfName = Util.javaStatefulName (i);
    intfName = Util.javaName (i);
    // for valuetype, get the name of the interface the valuetype supports
    if (i instanceof ValueEntry)
    {
      ValueEntry v = (ValueEntry) i;
      InterfaceEntry intf = (InterfaceEntry) v.supports ().elementAt (0);
      //intfName = Util.javaStatefulName (intf);
      intfName = Util.javaName (intf);
    }
  } // init

  /**
   *
   **/
  protected void openStream ()
  {
    String name;
    if (tie)
       name = i.name () + "_Tie";
    else
       name = '_' + i.name () + "ImplBase";
    String pkg = Util.containerFullName (i.container ());
    if (pkg != null && !pkg.equals (""))
    {
      Util.mkdir (pkg);
      name = pkg + '/' + name;
    }
    stream = Util.getStream (name.replace ('/', File.separatorChar) + ".java", i);
  } // openStream

  /**
   *
   **/
  protected void writeHeading ()
  {
    Util.writePackage (stream, i, Util.StubFile);
    Util.writeProlog (stream, ((GenFileStream)stream).name ());
    if (i.comment () != null)
      i.comment ().generate ("", stream);
    writeClassDeclaration ();
    stream.println ('{');
    stream.println ();
  } // writeHeading

  /**
   *
   **/
  protected void writeClassDeclaration ()
  {
    if (tie)
    {
      stream.println ("public class " + className + " extends org.omg.CORBA.portable.ObjectImpl");
      stream.print   ("                implements " + intfName + ", ");
      stream.println ("org.omg.CORBA.portable.InvokeHandler"); // d57147
      return;
    }
    stream.println ("public abstract class _" + className + " extends org.omg.CORBA.portable.ObjectImpl");
    stream.print   ("                implements " + intfName + ", ");
    stream.println ("org.omg.CORBA.portable.InvokeHandler");
  } // writeClassDeclaration

  /**
   *
   **/
  protected void writeBody ()
  {
    // <f46082.51> Remove -stateful feature.  ?????
    //if (i.state () != null)
    //  writeState ();
    writeCtors ();
    if (i instanceof ValueEntry)
    {
      // use the interface the valuetype supports to generate the
      // tie class instead of using the valuetype itself
      ValueEntry v = (ValueEntry) i;
      this.i = (InterfaceEntry) v.supports ().elementAt (0);
    }
    buildMethodList ();
    writeMethodTable ();
    writeDispatchMethod ();
    writeMethods ();
    writeCORBAOperations ();
    writeOperations ();
    if (tie)
      stream.println ("  private " + intfName + "Operations _impl;");
  } // writeBody

  /**
   * Close the skeleton class. The singleton ORB member is
   * necessary only for portable skeletons.
   **/
  protected void writeClosing ()
  {
    stream.println ();
    if (tie)
       stream.println ("} // class " + className);
    else
       stream.println ("} // class _" + className);
  } // writeClosing

  /**
   * Close the print stream, which flushes the stream to file.
   **/
  protected void closeStream ()
  {
    stream.close ();
  } // closeStream

  // <f46082.51> Remove -stateful feature.
  /*
  protected PrintWriter openStateStream ()
  {
    String name = '_' + i.name () + "State";
    String pkg = Util.containerFullName (i.container ());
    if (pkg != null && !pkg.equals (""))
    {
      Util.mkdir (pkg);
      name = pkg + '/' + name;
    }
    return Util.getStream (name.replace ('/', File.separatorChar) + ".java", i);
  } // openStateStream
  */
  
  // <f46082.51> Remove -stateful feature.  ????
  /**
   * Write the _XXXState interface file itself.
   **/
  /*
  private void writeStateClass ()
  {
    PrintWriter stream = openStateStream ();

    Util.writePackage (stream, i, Util.StateFile);
    Util.writeProlog (stream, ((GenFileStream)stream).name ());

    stream.print ("public interface _" + i.name () + "State extends " + Util.javaName (i));

    // Does it derive from a stateful interface?
    InterfaceEntry parent = (InterfaceEntry)i.derivedFrom ().firstElement ();
    if (parent.state () == null)
      stream.println ();
    else
      stream.println (", " + Util.javaStatefulName (parent));
    stream.println ('{');
    Enumeration e = i.state ().elements ();
    while (e.hasMoreElements ())
    {
      SymtabEntry member     = ((InterfaceState)e.nextElement ()).entry;
      String      memberName = member.name ();
      String      typeName   = Util.javaStatefulName (member);

      // write statedef comment
      if (member.comment () != null)
        member.comment ().generate ("    ", stream);

      // print the getter
      stream.println ("  public " + typeName + ' ' + memberName + " ();");
      stream.println ();

      // print the setter
      stream.println ("  public void " + memberName + " (" + typeName + " _" + memberName + ");");
      stream.println ();
    }
    stream.println ("} // interface _" + i.name () + "State");
    stream.close ();
  } // writeStateClass

  protected void writeState ()
  {
    if (!tie)
    {
      // Write the public state instance variables
      Enumeration e = i.state ().elements ();
      while (e.hasMoreElements ())
      {
        TypedefEntry member = ((InterfaceState)e.nextElement ()).entry;
        // write statedef comment
        if (member.comment () != null)
          member.comment ().generate ("    ", stream);

        Util.writeInitializer ("    public ", '_' + member.name (), "", member, stream);
      }
      stream.println ();
    }

    // Write the methods from the _XXXState interface
    Enumeration e = i.state ().elements ();
    while (e.hasMoreElements ())
    {
      SymtabEntry member     = ((InterfaceState)e.nextElement ()).entry;
      String      memberName = member.name ();
      String      typeName   = Util.javaStatefulName (member);

      // print the getter
      stream.println ("  public " + typeName + ' ' + memberName + " ()");
      stream.println ("  {");
      if (tie)
        stream.println ("    return _impl." + memberName + " ();");
      else
        stream.println ("    return _" + memberName + ';');
      stream.println ("  }");
      stream.println ();

      // print the setter
      stream.println ("  public void " + memberName + " (" + typeName + " new" + memberName + ")");
      stream.println ("  {");
      if (tie)
        stream.println ("    _impl." + memberName + " (_" + memberName + ");");
      else
        stream.println ("    _" + memberName + " = new" + memberName + ';');
      stream.println ("  }");
      stream.println ();
    }

    // Now write the _XXXState interface file itself.
    writeStateClass ();
  } // writeState
  */

  /**
   *
   **/
  protected void writeCtors ()
  {
    stream.println ("  // Constructors");
    if (tie)
      stream.println ("  public " + className + " ()");
    else
      stream.println ("  public _" + className + " ()");
    stream.println ("  {");
    stream.println ("  }");
    stream.println ();

    if (tie)
    {
      stream.println ("  public " + className + " (" + intfName + "Operations impl)");
      stream.println ("  {");

      // Does it derive from a interface having state, e.g., valuetype?
      if (((InterfaceEntry)i.derivedFrom ().firstElement ()).state () != null)
        stream.println ("    super (impl);");
      else
        stream.println ("    super ();");
      stream.println ("    _impl = impl;");
      stream.println ("  }");
      stream.println ();
    }
  } // writeCtors

  /**
   * Build a list of all of the methods, keeping out duplicates.
   **/
  protected void buildMethodList ()
  {
    // Start from scratch
    methodList = new Vector ();

    buildMethodList (i);
  } // buildMethodList

  /**
   *
   **/
  private void buildMethodList (InterfaceEntry entry)
  {
    // Add the local methods
    Enumeration locals = entry.methods ().elements ();
    while (locals.hasMoreElements ())
      addMethod ((MethodEntry)locals.nextElement ());

    // Add the inherited methods
    Enumeration parents = entry.derivedFrom ().elements ();
    while (parents.hasMoreElements ())
    {
      InterfaceEntry parent = (InterfaceEntry)parents.nextElement ();
      if (!parent.name ().equals ("Object"))
        buildMethodList (parent);
    }
  } // buildMethodList

  /**
   *
   **/
  private void addMethod (MethodEntry method)
  {
    if (!methodList.contains (method))
      methodList.addElement (method);
  } // addMethod

  /**
   *
   **/
  protected void writeDispatchMethod ()
  {
    String indent = "                                ";
    stream.println ("  public org.omg.CORBA.portable.OutputStream _invoke (String method,");
    stream.println (indent + "org.omg.CORBA.portable.InputStream in,");
    stream.println (indent + "org.omg.CORBA.portable.ResponseHandler rh)");
    stream.println ("  {");

    // Per Simon 8/26/98, create and return reply stream for all methods - KLR
    stream.println ("    org.omg.CORBA.portable.OutputStream out = null;");
    stream.println ("    java.lang.Integer __method = (java.lang.Integer)_methods.get (method);");
    stream.println ("    if (__method == null)");
    stream.println ("      throw new org.omg.CORBA.BAD_OPERATION (0, org.omg.CORBA.CompletionStatus.COMPLETED_MAYBE);");
    stream.println ();
    if (methodList.size () > 0)
    {
      stream.println ("    switch (__method.intValue ())");
      stream.println ("    {");

      // Write the method case statements
      int realI = 0;
      for (int i = 0; i < methodList.size (); ++i)
      {
        MethodEntry method = (MethodEntry)methodList.elementAt (i);
        ((MethodGen)method.generator ()).dispatchSkeleton (symbolTable, method, stream, realI);
        if (method instanceof AttributeEntry && !((AttributeEntry)method).readOnly ())
          realI += 2;
        else
          ++realI;
      }

      indent = "       ";
      stream.println (indent + "default:");
      stream.println (indent + "  throw new org.omg.CORBA.BAD_OPERATION (0, org.omg.CORBA.CompletionStatus.COMPLETED_MAYBE);");
      stream.println ("    }");
      stream.println ();
    }
    stream.println ("    return out;");
    stream.println ("  } // _invoke");
    stream.println ();
  } // writeDispatchMethod

  /**
   *
   **/
  protected void writeMethodTable ()
  {
    // Write the methods hashtable
    stream.println ("  private static java.util.Hashtable _methods = new java.util.Hashtable ();");
    stream.println ("  static");
    stream.println ("  {");

    int count = -1;
    Enumeration e = methodList.elements ();
    while (e.hasMoreElements ())
    {
      MethodEntry method = (MethodEntry)e.nextElement ();
      if (method instanceof AttributeEntry)
      {
        stream.println ("    _methods.put (\"_get_" + Util.stripLeadingUnderscores (method.name ()) + "\", new java.lang.Integer (" + (++count) + "));");
        if (!((AttributeEntry)method).readOnly ())
          stream.println ("    _methods.put (\"_set_" + Util.stripLeadingUnderscores (method.name ()) + "\", new java.lang.Integer (" + (++count) + "));");
      }
      else
        stream.println ("    _methods.put (\"" + Util.stripLeadingUnderscores (method.name ()) + "\", new java.lang.Integer (" + (++count) + "));");
    }
    stream.println ("  }");
    stream.println ();
  } // writeMethodTable

  /**
   *
   **/
  protected void writeMethods ()
  {
    if (tie)
    {
      // Write the methods
      int realI = 0;
      for (int i = 0; i < methodList.size (); ++i)
      {
        MethodEntry method = (MethodEntry)methodList.elementAt (i);
        ((MethodGen)method.generator ()).skeleton (symbolTable, method, stream, realI);
        if (method instanceof AttributeEntry && !((AttributeEntry)method).readOnly ())
          realI += 2;
        else
          ++realI;
        stream.println ();
      }
    }
  } // writeMethods

  /**
   *
   **/
  private void writeIDs ()
  {
    Vector list = new Vector ();
    buildIDList (i, list);
    Enumeration e = list.elements ();
    boolean first = true;
    while (e.hasMoreElements ())
    {
      if (first)
        first = false;
      else
        stream.println (", ");
      stream.print ("    \"" + (String)e.nextElement () + '"');
    }
  } // writeIDs

  /**
   *
   **/
  private void buildIDList (InterfaceEntry entry, Vector list)
  {
    if (!entry.fullName ().equals ("org/omg/CORBA/Object"))
    {
      String id = Util.stripLeadingUnderscoresFromID (entry.repositoryID ().ID ());
      if (!list.contains (id))
        list.addElement (id);
      Enumeration e = entry.derivedFrom ().elements ();
      while (e.hasMoreElements ())
        buildIDList ((InterfaceEntry)e.nextElement (), list);
    }
  } // buildIDList

  /**
   *
   **/
  protected void writeCORBAOperations ()
  {
    stream.println ("  // Type-specific CORBA::Object operations");

    stream.println ("  private static String[] __ids = {");
    writeIDs ();
    stream.println ("};");
    stream.println ();
    stream.println ("  public String[] _ids ()");
    stream.println ("  {");
    stream.println ("    return __ids;");
    stream.println ("  }");
    stream.println ();
  } // writeCORBAOperations

  /**
   *
   **/
  protected void writeOperations ()
  {
    // _get_ids removed at Simon's request 8/26/98 - KLR
  } // writeOperations

  protected Hashtable      symbolTable = null;
  protected InterfaceEntry i           = null;
  protected PrintWriter    stream      = null;

  // Unique to this generator
  protected String         className   = null;
  protected boolean        tie         = false;
  protected Vector         methodList  = null;
  protected String         intfName    = "";
} // class Skeleton

