/*
 * @(#)MemberDocImpl.java	1.30 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package com.sun.tools.javadoc;

import com.sun.javadoc.*;

import sun.tools.java.MemberDefinition;
import sun.tools.java.Identifier;

import java.text.CollationKey;
import java.text.Collator;

import java.lang.reflect.Modifier;

/**
 * Represents a member of a java class: field, constructor, or method.
 * This is an abstract class dealing with information common to
 * method, constructor and field members. Class members of a class
 * (innerclasses) are represented instead by ClassDocImpl.
 *
 * @see MethodDocImpl
 * @see FieldDocImpl
 * @see ClassDocImpl
 *
 * @author Robert Field
 */
abstract class MemberDocImpl extends ProgramElementDocImpl 
                                             implements MemberDoc {

    String name;

    /**
     * constructor.
     */
    MemberDocImpl(Env env, MemberDefinition f) {
        super(env, f.getDocumentation(), mods(f), f.getClassDefinition());
	this.name = f.getName().toString();
    }

    /**
     * constructor assist
     *
     * Return modifier bits.
     */
    private static int mods(MemberDefinition f) {
        int mods = f.getModifiers();
        if ((mods & f.M_ABSTRACT) != 0 
                      && f.getClassDefinition().isInterface()) {
            mods ^= f.M_ABSTRACT;
        }
        return mods;
    }

    ClassDocImpl thisClassDocImpl() {
        return containingClass;
    }    

    /**
     * Get the name.
     *
     * @return the qualified name of the member.
     */
    public String qualifiedName() {
        String qn = containingClass.qualifier;
        StringBuffer sb = new StringBuffer();
	if (qn.length() > 0) {
	    sb.append(qn);
	    sb.append('.');
	}
	sb.append(containingClass.name);
	sb.append('.');
        sb.append(name);
        return sb.toString();
    }

    /**
     * Get the name.
     *
     * @return the unqualified name of the member.
     */
    public String name() {
	return name;
    }

    /**
     * Return the package that this member is contained in.
     * Return "" if in empty package.
     */
    public PackageDoc containingPackage() {
        return containingClass.containingPackage;
    }

    /** 
     * return true if this member is include in the active set.
     */
    public boolean isIncluded() {
      return true;
    }


    /** 
     * Returns true if this field was synthesized by the compiler.
     */
    public boolean isSynthetic() {
	return (sun.tools.java.Constants.M_SYNTHETIC & modifiers) != 0;
    }

    /**
     * Returns a string representation of the member.
     *
     * @return a description of the member
     */
    public String toString() {
	return name();
    }

    /** lookup key */
    static String lookupKey(MemberDefinition f) { 
        Identifier cid = f.getClassDefinition().getName();
	String name = f.getName().toString();
	String tn = f.getType().typeString(name, false, false);
        StringBuffer sb = new StringBuffer(tn);
	sb.append(' ');
	sb.append(cid.getFlatName());
	sb.append('.');
	sb.append(cid.getQualifier());
	return sb.toString();
    }

    String toQualifiedString() {
        return qualifiedName();
    }
}
