/*
 * @(#)Receiver.java	1.13 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.sound.midi;		  	 


/**
 * A <code>Receiver</code> receives <code>{@link MidiEvent}</code> objects and
 * typically does something useful in response, such as interpreting them to
 * generate sound or raw MIDI output.  Common MIDI receivers include 
 * synthesizers and MIDI Out ports.  
 *
 * @see MidiDevice
 * @see Synthesizer
 * @see Transmitter
 *
 * @version 1.13, 00/02/02
 * @author Kara Kytle
 */
public interface Receiver {


	/**
	 * Sends a MIDI message and time-stamp to this receiver.
	 * If time-stamping is not supported by this receiver, the time-stamp 
	 * value should be -1.
	 * @param message the MIDI message to send
	 * @param time-stamp the time-stamp for the message, in microseconds.
	 * @throws IllegalStateException if the receiver is not open
	 */ 
	public void send(MidiMessage message, long timeStamp);													 
	
	/**
	 * Indicates that the application has finished using the receiver, and
	 * that limited resources it requires may be released or made available.
	 * Invoking methods on a receiver which has been closed may cause an 
	 * <code>IllegalArgumentException</code> or other exception to be thrown.
	 */
	public void close();
}
