/*
 * @(#)AudioData.java	1.7 00/02/02
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.audio;

import java.io.*;
import javax.sound.sampled.*;


/**
 * A clip of audio data. This data can be used to construct an
 * AudioDataStream, which can be played. <p>
 *
 * @author  Arthur van Hoff
 * @author  Kara Kytle
 * @version 1.7 00/02/02
 * @see     AudioDataStream
 * @see     AudioPlayer
 */

 /*
  * the idea here is that the AudioData object encapsulates the
  * data you need to play an audio clip based on a defined set
  * of data.  to do this, you require the audio data (a byte 
  * array rather than an arbitrary input stream) and a format
  * object.
  */


public class AudioData {

    AudioFormat format;   // carry forth the format array amusement
    byte buffer[];

    /**
     * Constructor
     */
    public AudioData(byte buffer[]) {

        // okay, we need to extract the format and the byte buffer of data

        try {

			AudioInputStream ais = AudioSystem.getAudioInputStream( new ByteArrayInputStream(buffer) );
			this.format = ais.getFormat();
			ais.close();
			this.buffer = buffer;


            /*AudioContainerInputStream acis = AudioContainerInputStream.getStream(new ByteArrayInputStream(buffer));
            acis.setData();
            Format[] format = acis.getFormat();
            this.format = (AudioFormat)format[0];
            this.buffer = acis.getBytes();
			*/

        } catch (IOException e) {

            // if we cannot extract valid format information, we resort to assuming the data will be 8k mono u-law
            // in order to provide maximal backwards compatibility....

			AudioFormat mono_8k_ulaw_format = new AudioFormat( AudioFormat.Encoding.ULAW,
												8000,		// sample rate
												8,			// sample size in bits
												1,			// channels
												1,			// frame size in bytes
												8000,		// frame rate
												true );		// bigendian (irrelevant for 8-bit data)
                                            
            this.format = mono_8k_ulaw_format;
            this.buffer = buffer; 
        } catch (UnsupportedAudioFileException e1 ) {

            // if we cannot extract valid format information, we resort to assuming the data will be 8k mono u-law
            // in order to provide maximal backwards compatibility....

			AudioFormat mono_8k_ulaw_format = new AudioFormat( AudioFormat.Encoding.ULAW,
												8000,		// sample rate
												8,			// sample size in bits
												1,			// channels
												1,			// frame size in bytes
												8000,		// frame rate
												true );		// bigendian (irrelevant for 8-bit data)
                                            
            this.format = mono_8k_ulaw_format;
            this.buffer = buffer; 

		}
    }


    /**
     * Non-public constructor; this is the one we use in ADS and CADS
     * constructors.
     */
    AudioData(AudioFormat format, byte[] buffer) {

        this.format = format;
        this.buffer = buffer;
    }
}
