/*
 * @(#)FontProperties.java	1.2 00/07/19
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import sun.io.CharacterEncoding;


public class FontProperties extends Properties {

    protected static String osVersion;

    public FontProperties(String xosVersion) {
	super(getDefaultFontProperties());
	osVersion = xosVersion;
	initializeProperties();
    }

    private void initializeProperties() {

	// Find property file
        // give preference to lang_region_encoding within (user, jhome)
	String uhome = System.getProperty("user.home");
	String jhome = System.getProperty("java.home");
	if (jhome == null){
	    throw new Error("java.home property not set");
	}
	String language = System.getProperty("user.language", "en");
	String region = System.getProperty("user.region");

        // Translate the raw encoding name returned by the VM to the canonical
        // name from the alias table in CharacterEncoding. Map unlisted raw
        // encoding names to themselves. - liu 8/10/98 bug 4163038
	String rawEncoding = System.getProperty("file.encoding");
        String encoding = CharacterEncoding.aliasName(rawEncoding);
        if (encoding == null) {
            encoding = rawEncoding;
        }

	try {
	    File f = null;

            if (region != null) {
                f = tryOpeningFontProp(
                        f, uhome, language, region + "_" + encoding);
                f = tryOpeningFontProp(
                        f, jhome, language, region + "_" + encoding);
                f = tryOpeningFontProp(f, uhome, language, region);
                f = tryOpeningFontProp(f, jhome, language, region);
            }

            f = tryOpeningFontProp(f, uhome, language, encoding);
            f = tryOpeningFontProp(f, jhome, language, encoding);
            f = tryOpeningFontProp(f, uhome, language, null);
            f = tryOpeningFontProp(f, jhome, language, null);
            f = tryOpeningFontProp(f, uhome, encoding, null);
            f = tryOpeningFontProp(f, jhome, encoding, null);
            f = tryOpeningFontProp(f, uhome, null, null);
            f = tryOpeningFontProp(f, jhome, null, null);

            if (f != null) {
	        // Load property file
	        FileInputStream in = new FileInputStream(f.getPath());
	        load(in);
	        in.close();
		verifyProperties(f);
            }
        } catch (Exception e){
	}
    }


    private File tryOpeningFontProp(File f, String homedir,
				    String language, String ext) {
        if (f != null) {
            return f;       // already validated
        }
        String filename = homedir + File.separator
                            + "lib" + File.separator
                            + "font.properties";
        if (language != null) {
            filename += "." + language;
            if (ext != null) {
                filename += "_" + ext;
            }
        }

        // Try the os-specific prop file first
        File propsFile = new File(filename+"."+osVersion);
        if ((propsFile != null) && propsFile.canRead()) {
            return propsFile;
        }

        propsFile = new File(filename);
        if ((propsFile != null) && propsFile.canRead()) {
            return propsFile;
        }
        
        return null;
    }


    protected void verifyProperties(File f) {
    }


    private static Properties getDefaultFontProperties() {
	// set default props to prevent crashing
	// with corrupted or missing font.properties
	Properties defaultProps = new Properties();
	defaultProps.put("serif.0", "unknown");
	defaultProps.put("sansserif.0", "unknown");
	defaultProps.put("monospaced.0", "unknown");
	defaultProps.put("dialog.0", "unknown");
	defaultProps.put("dialoginput.0", "unknown");

	defaultProps.put("default.0", "sansserif");
	defaultProps.put("timesroman.0", "serif");
	defaultProps.put("helvetica.0", "sansserif");
	defaultProps.put("courier.0", "monospaced");
	defaultProps.put("zapfdingbats.0", "unknown");
	return defaultProps;
    }
}
