/*
 * @(#)SunGraphicsCallback.java	1.3 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt;

import java.awt.*;


public abstract class SunGraphicsCallback {
    public static final int HEAVYWEIGHTS = 0x1;
    public static final int LIGHTWEIGHTS = 0x2;
    public static final int TWO_PASSES = 0x4;

    private static final DebugHelper dbg =
        DebugHelper.create(SunGraphicsCallback.class);

    public abstract void run(Component comp, Graphics cg);

    public final void runOneComponent(Component comp, Rectangle bounds,
				      Graphics g, Shape clip,
				      int weightFlags) {
        if (comp == null || comp.getPeer() == null || !comp.isVisible()) {
	    return;
	}
	boolean lightweight = comp.isLightweight();
	if ((lightweight && (weightFlags & LIGHTWEIGHTS) == 0) ||
	    (!lightweight && (weightFlags & HEAVYWEIGHTS) == 0)) {
	    return;
	}

	if (bounds == null) {
	    bounds = comp.getBounds();
	}

	if (clip == null || clip.intersects(bounds)) {
	    Graphics cg = g.create(bounds.x, bounds.y, bounds.width,
				   bounds.height);
	    try {
	        cg.setFont(comp.getFont());
		cg.setColor(comp.getForeground());
		if (cg instanceof Graphics2D) {
	            ((Graphics2D)cg).setBackground(comp.getBackground());
		} else if (cg instanceof Graphics2Delegate) {
		    ((Graphics2Delegate)cg).setBackground(
		        comp.getBackground());
		}
		run(comp, cg);
	    } finally {
	        cg.dispose();
	    }
	}
    }

    public final void runComponents(Component[] comps, Graphics g,
				    int weightFlags) {
        int ncomponents = comps.length;
	Shape clip = g.getClip();

	if (dbg.on) {
	    Rectangle newrect = clip.getBounds();
	    dbg.println("GraphicsCallback::runComponents : x = " +
			newrect.x + " y = " + newrect.y + " width = " +
			newrect.width + " height = " + newrect.height); 
	}

        // A seriously sad hack--
        // Lightweight components always paint behind peered components,
        // even if they are at the top of the Z order. We emulate this
        // behavior by making two printing passes: the first for lightweights;
        // the second for heavyweights.
	//
	// ToDo(dpm): Either build a list of heavyweights during the
	// lightweight pass, or redesign the components array to keep
	// lightweights and heavyweights separate.
	if ((weightFlags & TWO_PASSES) != 0) {
	    for (int i = ncomponents - 1; i >= 0; i--) {
	        runOneComponent(comps[i], null, g, clip, LIGHTWEIGHTS);
	    }
	    for (int i = ncomponents - 1; i >= 0; i--) {
	        runOneComponent(comps[i], null, g, clip, HEAVYWEIGHTS);
	    }
	} else {
	    for (int i = ncomponents - 1; i >= 0; i--) {
	        runOneComponent(comps[i], null, g, clip, weightFlags);
	    }
	}
    }

    public static final class PaintHeavyweightComponentsCallback
        extends SunGraphicsCallback
    {
        private static PaintHeavyweightComponentsCallback instance =
            new PaintHeavyweightComponentsCallback();

        private PaintHeavyweightComponentsCallback() {}
        public void run(Component comp, Graphics cg) {
            if (!comp.isLightweight()) {
                comp.paintAll(cg);
            } else if (comp instanceof Container) {
                runComponents(((Container)comp).getComponents(), cg,
                              LIGHTWEIGHTS | HEAVYWEIGHTS);
            }
        }
        public static PaintHeavyweightComponentsCallback getInstance() {
            return instance;
        }
    }
    public static final class PrintHeavyweightComponentsCallback
        extends SunGraphicsCallback
    {
        private static PrintHeavyweightComponentsCallback instance =
            new PrintHeavyweightComponentsCallback();

        private PrintHeavyweightComponentsCallback() {}
        public void run(Component comp, Graphics cg) {
            if (!comp.isLightweight()) {
                comp.printAll(cg);
            } else if (comp instanceof Container) {
                runComponents(((Container)comp).getComponents(), cg,
                              LIGHTWEIGHTS | HEAVYWEIGHTS);
            }
        }
        public static PrintHeavyweightComponentsCallback getInstance() {
            return instance;
        }
    }
}
