/*
 * @(#)ExtendedTextLabelComponent.java	1.8 00/01/19
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * (C) Copyright IBM Corp. 1998, All Rights Reserved
 *
 */

package sun.awt.font;

import java.awt.font.LineMetrics;
import java.awt.font.GlyphJustificationInfo;
import java.awt.font.GlyphVector;
import java.awt.font.FontRenderContext;
import java.awt.font.TextAttribute;

import java.awt.geom.Rectangle2D;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Shape;

import java.util.Map;

public final class ExtendedTextLabelComponent
                        extends TextLineComponent {

    private ExtendedTextLabel label;

    /**
     * Create an ExtendedTextLabelComponent.  If <tt>styles</tt>
     * is null it is considered empty.
     */
    public ExtendedTextLabelComponent(ExtendedTextLabel label,
                                      Map styles) {

        super(styles, label.getLineMetrics());

        this.label = label;
    }

    private ExtendedTextLabelComponent(ExtendedTextLabel label,
                                       TextLineComponent parent) {

        super(parent);
        this.label = label;
    }

    public float getItalicAngle() {

        return label.getItalicAngle();
    }

    protected Rectangle2D handleGetVisualBounds() {

        return label.getVisualBounds();
    }

    protected Shape handleGetOutline(float x, float y) {

        return label.getOutline(x, y);
    }

    protected void handleDraw(Graphics2D g2d, float x, float y) {

        label.draw(g2d, x, y);
    }

    public int getNumCharacters() {

        return label.getNumCharacters();
    }

    public float getCharX(int index) {

        return label.getCharX(index);
    }

    public float getCharY(int index) {

        return label.getCharY(index);
    }

    public float getCharAdvance(int index) {

        return label.getCharAdvance(index);
    }

    public boolean caretAtOffsetIsValid(int index) {

        return label.caretAtOffsetIsValid(index);
    }

    protected Rectangle2D handleGetCharVisualBounds(int index) {

        return label.getCharVisualBounds(index);
    }

    // measures characters in context, in logical order
    public int getLineBreakIndex(int start, float width) {

        return label.getLineBreakIndex(start, width);
    }

    // measures characters in context, in logical order
    public float getAdvanceBetween(int start, int limit) {

      return label.getCharAdvanceBetween(start, limit);
    }

    public Rectangle2D getLogicalBounds() {
      return label.getLogicalBounds();
    }

    public TextLineComponent getSubset(int start, int limit, int dir) {

        final int numChars = label.getNumCharacters();
        if (start < 0 || limit > numChars || start >= limit) {
            throw new IllegalArgumentException("Invalid range");
        }

        if (start == 0 && limit == numChars && dir == UNCHANGED) {
            return this;
        }

        return new ExtendedTextLabelComponent(label.getSubset(start, limit, dir), this);
    }


  public int getNumJustificationInfos() {
    return label.getNumJustificationInfos();
  }

  public void getJustificationInfos(GlyphJustificationInfo[] infos, int infoStart, int charStart, int charLimit) {
    label.getJustificationInfos(infos, infoStart, charStart, charLimit);
  }

  public TextLineComponent applyJustificationDeltas(float[] deltas, int deltaStart, boolean[] flags) {
    return new ExtendedTextLabelComponent(label.applyJustificationDeltas(deltas, deltaStart, flags), this);
  }

    public String toString() {

        return "[characters:" + label + "]";
    }
}
