/*
 * @(#)SimpleInputMethodWindow.java	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.im;

import java.awt.Frame;

/**
 * Implements a simple input method window that provides the minimal
 * functionality as specified in
 * {@link java.awt.im.spi.InputMethodContext#createInputMethodWindow}.
 *
 * @version 1.5 02/02/00
 */
public class SimpleInputMethodWindow
        extends Frame
        implements InputMethodWindow {

    InputContext inputContext = null;
    boolean needsToSuppressFocusChange;

    /**
     * Constructs a simple input method window.
     * If this window is attached to an input context and the platform
     * specifies that we need to suppress input method deactivation
     * during focus changes, the window will work with the input context
     * to suppress deactivation.
     */
    public SimpleInputMethodWindow(String title, InputContext context, boolean needsToSuppressFocusChange) {
        super(title);
        if (context != null) {
            this.inputContext = context;
        }
        this.needsToSuppressFocusChange = needsToSuppressFocusChange;
    }
    
    public void setVisible(boolean visible) {
        if (needsToSuppressFocusChange &&
                inputContext != null && visible && !isVisible()) {
            inputContext.suppressFocusChange();
        }
        super.setVisible(visible);
    }
    
    public void setInputContext(InputContext inputContext) {
        this.inputContext = inputContext;
    }

    public java.awt.im.InputContext getInputContext() {
        if (inputContext != null) {
            return inputContext;
        } else {
            return super.getInputContext();
        }
    }
}
