/*
 * @(#)OffScreenImage.java	1.18 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.image;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.image.ImageProducer;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import sun.awt.image.BufferedImageGraphics2D;

public class OffScreenImage extends BufferedImage {
    Component c;
    OffScreenImageSource osis;

    /**
     * Constructs an OffScreenImage with the default image type,
     * TYPE_4BYTE_ABGR, for offscreen rendering to be used with a
     * given component.
     * The component is used to obtain the foreground color, background
     * color and font.
     * REMIND:  At some point, we might want to look at the component
     * and figure out the best image type to construct.
     */
    public OffScreenImage(Component c, int width, int height) {
        // REMIND:  Should get image type from the component.getGC
        super(width, height, TYPE_4BYTE_ABGR);
        this.c = c;
	initBackground(width, height);
    }

    /**
     * Constructs an OffScreenImage with the given image type,
     * for offscreen rendering to be used with a given component.
     * The component is used to obtain the foreground color, background
     * color and font.
     */
    public OffScreenImage(Component c, int width, int height,
                             int imageType)
    {
        super (width, height, imageType);
        this.c = c;
	initBackground(width, height);
    }

    /**
     * Constructs an OffScreenImage given a color model and tile,
     * for offscreen rendering to be used with a given component.
     * The component is used to obtain the foreground color, background
     * color and font.
     */
    public OffScreenImage(Component c, ColorModel cm, WritableRaster raster,
                            boolean isRasterPremultiplied)
    {
        super(cm, raster, isRasterPremultiplied, null);
        this.c = c;
	initBackground(raster.getWidth(), raster.getHeight());
    }

    public Graphics getGraphics() {
        return new BufferedImageGraphics2D(this, c);
    }

    public Graphics2D createGraphics() {
        return new BufferedImageGraphics2D(this, c);
    }

    public void initBackground(int width, int height) {
	Graphics2D g2 = createGraphics();
	try {
	    g2.clearRect(0, 0, width, height);
	} finally {
	    g2.dispose();
	}
    }

    public ImageProducer getSource() {
	if (osis == null) {
	    osis = new OffScreenImageSource(this);
	}
        return osis;
    }

}
