/*
 * @(#)EventProbe.java	1.5 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.robot.probe;

import java.awt.*;
import java.awt.event.*;

/**
 * Probe source that listens to the AWT Event queue.
 * This class, when instantiated, connects with the AWTEventQueue,
 * attaches with the AWTEventListener interface, and asks to
 * receive all events.  It then dispatches, through ProbeMonitor,
 * all events it receives.<p>
 *
 * <b>Note:</b> <i>EventProbe</i> only works with JDK 1.2 or later.
 * Previous implementations of JDK do not have appropriate hooks
 * in the AWT event queue.
 *
 * @see ProbeMonitor
 *
 * @author David Herron <tt>&lt;David.Herron@eng.sun.com&gt;</tt>
 */
public class EventProbe implements java.awt.event.AWTEventListener {

  //EventQueue   eventQ;
  Toolkit      toolkit;
  ProbeMonitor monitor;

  /**
   * Flag indicating whether to distribute events.
   * Set this to <tt>true</tt> to indicate that events are
   * to be dispatched.
   */
  public boolean dispatch;

  /**
   * Initialize, initially connecting with the EventQueue and dispatching.
   *
   * @param monitor The <i>ProbeMonitor</i> to post events to.
   */
  public EventProbe(ProbeMonitor monitor) {
    this(monitor, true);
  }

  /**
   * Initialize allowing the caller to specify whether to
   * immediately connect & start dispatching.
   */
  public EventProbe(ProbeMonitor monitor, boolean startNow) {
    this.monitor = monitor;
    toolkit = Toolkit.getDefaultToolkit(); 
    //eventQ  = toolkit.getSystemEventQueue();
    if (startNow) {
      connect();
      dispatch = true;
    }
    else {
      dispatch = false;
    }
  }

  protected void finalize() {
    disconnect();
  }

  /**
   * Connect to the AWT EventQueue.  Depending on the state
   * of the <i>dispatch</i> variable, events may start being
   * delivered to the <i>ProbeMonitor</i>.
   */
  public void connect() {
    toolkit.addAWTEventListener(this, 0xFFFFFFFF);
  }

  /**
   * Disconnect from the AWT EventQueue.  Events will stop
   * being delivered to the <i>ProbeMonitor</i>.
   */
  public void disconnect() {
    toolkit.removeAWTEventListener(this);
  }

  /**
   * Receive AWT events, posting them into ProbeMonitor.
   *
   * @param e The event to dispatch.
   */
  public void eventDispatched(AWTEvent e) {
    if (dispatch)
      monitor.post(new ProbeAWTEvent(e));
  }
}
