/*
 * @(#)ProbePrinter.java	1.4 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.robot.probe;

import java.io.*;

/**
 * Listener for ProbeObjects that just prints everything it receives.
 *
 * @see ProbeMonitor
 * @see ProbeObject
 *
 * @author David Herron <tt>&lt;David.Herron@eng.sun.com&gt;</tt>
 */
public class ProbePrinter implements ProbeObjectListener {

  boolean bPrinting = false; // Recording mode
  ProbeMonitor monitor;       // The monitor being listened to

  PrintStream out = null;

  /**
   * Initialize the ProbePrinter.
   *
   * @param monitor Which ProbeMonitor to listen to
   */
  public ProbePrinter(ProbeMonitor monitor) {
    this(monitor, false);
  }

  /**
   * Initialize the ProbePrinter.
   *
   * @param monitor Which ProbeMonitor to listen to.
   * @param start   Indicate whether to start printing immediately.
   */
  public ProbePrinter(ProbeMonitor monitor, boolean start) {
    this.monitor = monitor;
    bPrinting = start;
    connect();
  }

  protected void finalize() {
    disconnect();
  }

  /**
   * Set the output stream for printing.
   */
  public void setOutput(PrintStream out) { this.out = out; }

  /**
   * Know what the output stream is.
   */
  public PrintStream getOutput() { return out; }

  /**
   * Connect to the ProbeMonitor.  Probes can be printed only
   * if the printer is connected to the monitor, and printing
   * has been enabled.
   */
  public void connect() {
    monitor.addProbeObjectListener(this);
  }

  /**
   * Disconnect from the ProbeMonitor.  Stops printing.
   */
  public void disconnect() {
    monitor.removeProbeObjectListener(this);
  }

  /**
   * Enable printing of probes.  Allows printing to happen
   * if the printer is connected to the monitor.
   */
  public void startPrinting() { connect(); bPrinting = true; }

  /**
   * Disable printing of probes.
   */
  public void stopPrinting() { disconnect(); bPrinting = false; }

  /**
   * Indicate whether we are printing probes.
   */
  public boolean isPrinting() { return bPrinting; }

  /**
   * Method to receive the ProbeObject's.
   *
   * @param evt The event to print.
   */
  public void probeTriggered(ProbeObject evt) {
    if (!bPrinting)
      return;

    // This may be useful.  It is left over from
    // the ProbeViewer implementation.  There this was
    // necessary so that events in the ProbeViewer pane
    // do not get recorded.
    //
    // XXX: There is a general need to filter events
    // in order to maybe record or not record them.
    //
    //    if (evt.type == ProbeObject.EVENT) {
    //      if (this.isAncestorOf((Component)evt.getSource()) == true
    //       || evt.getSource() == this)
    //	return;
    //    }

    if (out != null)
      out.println(evt.toString());
  }
}
