/*
 * @(#)Wrap.java	1.4 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.robot.probe;

/**
 * A superclass for Probe sources that <i>wrap</i> around other objects.
 * The principle example for a <i>wrapper</i> is the <i>WrapComponent</i>
 * class.
 *
 * <p>AWT Components have a number of <i>listener</i> interfaces that indicate
 * activity within the component.  For the purpose of verifying correct
 * behavior of the Component, it is useful to watch the listener interface
 * to see if the expected pattern of listener events are fired.
 *
 * <p>There are many kinds of objects which one potentially would want to
 * test behavior in this manner (send in events - watch the listeners
 * for proper behavior).  Hence this super class with subclasses for
 * wrapping specific types of objects.
 *
 * <p>Generally the concept is to have an object class which knows all
 * the listener interfaces in a particular area.  (i.e. for AWT
 * Components, the listener interfaces came by browsing the java.awt.*
 * docs) That class implements all the interfaces of that area.  In
 * each of the listener methods, the method then instantiates an
 * appropriate ProbeObject subclass and sends it to the monitor
 * for distribution.
 *
 * <p>Verifying behavior, then, is done by listening to the monitor
 * for expected ProbeObjects.  (Or using the ProbeBuffer to
 * record the activity, and look through the buffer afterwards).
 *
 * <p>Instantiating a Wrap object need only be done once, and this
 * one instantiation will serve for all objects this kind of
 * Wrap object wraps.  This is different from what one might expect
 * (the expectation is one Wrap object per object being wrapped).
 * At least for wrapping AWT Components, it is unnecessary to
 * have one wrapper object per wrapped object.
 *
 * @see ProbeMonitor
 * @see ProbeObject
 * @see WrapComponent
 * @see ProbeBuffer
 * @see ProbePrinter
 *
 * @author David Herron <tt>&lt;David.Herron@eng.sun.com&gt;</tt>
 */
public class Wrap {


  /**
   * The ProbeMonitor to be posted to.  Subclasses are expected
   * to use this to post events.
   * @see ProbeMonitor
   */
  public ProbeMonitor monitor;

  /**
   * Initialization.
   * @param monitor The ProbeMonitor to be posted to.
   */
  public Wrap(ProbeMonitor monitor) {
    this.monitor = monitor;
  }

}
