/*
 * @(#)WrapComponent.java	1.5 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.robot.probe;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.lang.*;
import java.lang.reflect.*;

/**
 * Wrapper around java.awt.Component or any subclass.  The "wrapper"
 * concept is to connect to all the listener interfaces of the Component
 * (using Reflection to determine which listener interfaces it has)
 * such that the listener method generates appropriate ProbeObject's
 * and other information.
 *
 * @see ProbeMonitor
 * @see ProbeObject
 * @see Wrap
 *
 * @author David Herron <tt>&lt;David.Herron@eng.sun.com&gt;</tt>
 */
public class WrapComponent extends Wrap implements
  ActionListener, AdjustmentListener/*, AWTEventListener*/, ComponentListener,
  ContainerListener, FocusListener, InputMethodListener, ItemListener,
  KeyListener, MouseListener, MouseMotionListener, PropertyChangeListener,
  TextListener, WindowListener/*, VetoableChangeListener*/ {

  public WrapComponent(ProbeMonitor monitor) {
    super(monitor);
  }

  /*
   * Argument lists for querying the class for listener methods.  In
   * the pursuit of connecting to all listeners, we need to ask the
   * object if it has any of the known listener methods.  This is
   * done with the Class.getMethod method and this looks for method
   * names matching the desired signature.  The argument list for
   * any listener method is designed, by convention, to be just one
   * item - the interface class.
   */
  private static Class[] ALActionListener         = new Class[1];
  private static Class[] ALAdjustmentListener     = new Class[1];
  private static Class[] ALAWTEventListener       = new Class[1];
  private static Class[] ALComponentListener      = new Class[1];
  private static Class[] ALContainerListener      = new Class[1];
  private static Class[] ALFocusListener          = new Class[1];
  private static Class[] ALInputMethodListener    = new Class[1];
  private static Class[] ALItemListener           = new Class[1];
  private static Class[] ALKeyListener            = new Class[1];
  private static Class[] ALMouseListener          = new Class[1];
  private static Class[] ALMouseMotionListener    = new Class[1];
  private static Class[] ALPropertyChangeListener = new Class[1];
  private static Class[] ALTextListener           = new Class[1];
  private static Class[] ALWindowListener         = new Class[1];
  //  private static Class[] ALVetoableChangeListener = new Class[1];
  //  private static Class[] AL = new Class[1];

  /*
   * Initializing the argument lists.
   */
  static {
    try {
      ALActionListener[0] = Class.forName("java.awt.event.ActionListener");
    } catch (Exception e) {}
    try {
      ALAdjustmentListener[0] = Class.forName("java.awt.event.AdjustmentListener");
    } catch (Exception e) {}
    try {
      ALAWTEventListener[0] = Class.forName("java.awt.event.AWTEventListener");
    } catch (Exception e) {}
    try {
      ALComponentListener[0] = Class.forName("java.awt.event.ComponentListener");
    } catch (Exception e) {}
    try {
      ALContainerListener[0] = Class.forName("java.awt.event.ContainerListener");
    } catch (Exception e) {}
    try {
      ALFocusListener[0] = Class.forName("java.awt.event.FocusListener");
    } catch (Exception e) {}
    try {
      ALInputMethodListener[0] = Class.forName("java.awt.event.InputMethodListener");
    } catch (Exception e) {}
    try {
      ALItemListener[0] = Class.forName("java.awt.event.ItemListener");
    } catch (Exception e) {}
    try {
      ALKeyListener[0] = Class.forName("java.awt.event.KeyListener");
    } catch (Exception e) {}
    try {
      ALMouseListener[0] = Class.forName("java.awt.event.MouseListener");
    } catch (Exception e) {}
    try {
      ALMouseMotionListener[0] = Class.forName("java.awt.event.MouseMotionListener");
    } catch (Exception e) {}
    try {
      ALPropertyChangeListener[0] = Class.forName("java.beans.PropertyChangeListener");
    } catch (Exception e) {}
    try {
      ALTextListener[0] = Class.forName("java.awt.event.TextListener");
    } catch (Exception e) {}
    try {
      ALWindowListener[0] = Class.forName("java.awt.event.WindowListener");
    } catch (Exception e) {}
    //    try {
    //      ALVetoableChangeListener[0] = Class.forName("java.beans.VetoableChangeListener");
    //    } catch (Exception e) {}
    //    try {
    //      AL[0] = Class.forName("java.awt.event.");
    //    } catch (Exception e) {}
  }

    /**
     * Connect to the listeners of this Component.  Using
     * the reflection features of Java we query the class
     * (and all super classes) of the component.  If there
     * is a method with this pattern:
     *
     *    add<interface>Listener(interface)
     *
     * The method is called, causing this object to begin
     * receiving listener calls from this component.
     */
  public void wrap(Component component) {

    Class cl = component.getClass();
    Method m;

    //    System.out.println("WrapComponent.wrap Component "
    //                       + component.toString()
    //		       + " size" + component.getSize().toString()
    //		       + " screenLoc " + component.getLocationOnScreen().toString());

    // For each listener method
    //    Get the method from the class
    //
    //    If the method does not have this method, an exception will
    //    be thrown, skipping the next step
    //
    //    Call the add<interface>Listener method to hook up with the object.
    //
    //    This is done inside a try{}catch{} on purpose because
    //    many exceptions will be thrown here.
    //
    try {
      m = cl.getMethod("addActionListener", WrapComponent.ALActionListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addAdjustmentListener", WrapComponent.ALAdjustmentListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addAWTEventListener", WrapComponent.ALAWTEventListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addComponentListener", WrapComponent.ALComponentListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addContainerListener", WrapComponent.ALContainerListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addFocusListener", WrapComponent.ALFocusListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addInputMethodListener", WrapComponent.ALInputMethodListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addItemListener", WrapComponent.ALItemListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addKeyListener", WrapComponent.ALKeyListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addMouseListener", WrapComponent.ALMouseListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addMouseMotionListener", WrapComponent.ALMouseMotionListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addPropertyChangeListener", WrapComponent.ALPropertyChangeListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addTextListener", WrapComponent.ALTextListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("addWindowListener", WrapComponent.ALWindowListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    //      try {
    //	m = cl.getMethod("addVetoableChangeListener", WrapComponent.ALVetoableChangeListener);
    //	m.invoke(component, new Object[] { this });
    //      } catch (Exception e) { }
    //      try {
    //	m = cl.getMethod("add", WrapComponent.AL);
    //	m.invoke(component, new Object[] { this });
    //      } catch (Exception e) { }
  }

  /**
   * Wrap a container by descending the tree of child
   * components and wrapping them all.
   */
  public void wrap(Container container) {
    // First wrap it as if it's a Component
    wrap((Component)container);

    // Next wrap each child, being careful to
    // treat Containers as such.
    Component[] components = container.getComponents();
    int nComponents = components.length;
    int i;

    for (i = 0; i < nComponents; i++) {
      Component component = components[i];
      if (component instanceof Container)
      	wrap((Container)component);
      else
        wrap(component);
    }
  }

  /**
   * Undo the wrapping on a container.  The special behavior
   * is to descend the tree of child components and unwrap
   * them all.
   */
  public void unwrap(Container container) {

    // First unwrap it as if it's a Component
    unwrap((Component)container);

    // Next unwrap each child component, being careful
    // to treat Containers as such.
    Component[] components = container.getComponents();
    int nComponents = components.length;
    int i;

    for (i = 0; i < nComponents; i++) {
      Component component = components[i];
      if (component instanceof Container)
        unwrap((Container)component);
      else
        unwrap(component);
    }
  }

    /**
     * Undo the wrapping on a component.  That is, remove
     * the listener hookup.
     */
  public void unwrap(Component component) {

    Class cl = component.getClass();
    Method m;

    // For each listener method
    //    Get the method from the class
    //
    //    If the method does not have this method, an exception will
    //    be thrown, skipping the next step
    //
    //    Call the remove<interface>Listener method to hook up with the object.
    //
    //    This is done inside a try{}catch{} on purpose because
    //    many exceptions will be thrown here.
    //
    try {
      m = cl.getMethod("removeActionListener", WrapComponent.ALActionListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeAdjustmentListener", WrapComponent.ALAdjustmentListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeAWTEventListener", WrapComponent.ALAWTEventListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeComponentListener", WrapComponent.ALComponentListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeContainerListener", WrapComponent.ALContainerListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeFocusListener", WrapComponent.ALFocusListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeInputMethodListener", WrapComponent.ALInputMethodListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeItemListener", WrapComponent.ALItemListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeKeyListener", WrapComponent.ALKeyListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeMouseListener", WrapComponent.ALMouseListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeMouseMotionListener", WrapComponent.ALMouseMotionListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removePropertyChangeListener", WrapComponent.ALPropertyChangeListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeTextListener", WrapComponent.ALTextListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    try {
      m = cl.getMethod("removeWindowListener", WrapComponent.ALWindowListener);
      m.invoke(component, new Object[] { this });
    } catch (Exception e) { }
    //      try {
    //	m = cl.getMethod("removeVetoableChangeListener", WrapComponent.ALVetoableChangeListener);
    //	m.invoke(component, new Object[] { this });
    //      } catch (Exception e) { }
    //      try {
    //	m = cl.getMethod("remove", WrapComponent.AL);
    //	m.invoke(component, new Object[] { this });
    //      } catch (Exception e) { }
  }

  ///////////////////////////
  //                       //
  // ActionLisener support //
  //                       //
  ///////////////////////////

  /**
   * Invoked when an action occurs.
   */
  public void actionPerformed(ActionEvent e) {
    monitor.post(new ProbeActionEvent(e, "", "actionPerformed"));
  }


  ////////////////////////////////
  //                            //
  // AdjustmentListener support //
  //                            //
  ////////////////////////////////

  /**
   * Invoked when the value of the adjustable has changed.
   */
  public void adjustmentValueChanged(AdjustmentEvent e) {
    monitor.post(new ProbeAdjustmentEvent(e, "", "adjustmentValueChanged"));
  }

  ///////////////////////////////
  //                           //
  // ComponentListener support //
  //                           //
  ///////////////////////////////

  /**
   * Invoked when the component's size changes.
   */
  public void componentResized(ComponentEvent e) {
    monitor.post(new ProbeComponentEvent(e, "", "componentResized"));
  }

  /**
   * Invoked when the component's position changes.
   */
  public void componentMoved(ComponentEvent e) {
    monitor.post(new ProbeComponentEvent(e, "", "componentMoved"));
  }

  /**
   * Invoked when the component has been made visible.
   */
  public void componentShown(ComponentEvent e) {
    monitor.post(new ProbeComponentEvent(e, "", "componentShown"));
  }

  /**
   * Invoked when the component has been made invisible.
   */
  public void componentHidden(ComponentEvent e) {
    monitor.post(new ProbeComponentEvent(e, "", "componentHidden"));
  }

  ///////////////////////////////
  //                           //
  // ContainerListener support //
  //                           //
  ///////////////////////////////

  /**
   * Invoked when a component has been added to the container.
   */
  public void componentAdded(ContainerEvent e) {
    monitor.post(new ProbeContainerEvent(e, "", "componentAdded"));
  }

  /**
   * Invoked when a component has been removed from the container.
   */
  public void componentRemoved(ContainerEvent e) {
    monitor.post(new ProbeContainerEvent(e, "", "componentRemoved"));
  }

  ///////////////////////////////
  //                           //
  // FocusListener support     //
  //                           //
  ///////////////////////////////

  /**
   * Invoked when a component gains the keyboard focus.
   */
  public void focusGained(FocusEvent e) {
    monitor.post(new ProbeFocusEvent(e, "", "focusGained"));
  }

  /**
   * Invoked when a component loses the keyboard focus.
   */
  public void focusLost(FocusEvent e) {
    monitor.post(new ProbeFocusEvent(e, "", "focusLost"));
  }

  /////////////////////////////////
  //                             //
  // InputMethodListener support //
  //                             //
  /////////////////////////////////

  /**
   * Invoked when the text entered through an input method has changed.
   */
  public void inputMethodTextChanged(InputMethodEvent e) {
    monitor.post(new ProbeInputMethodEvent(e, "", "inputMethodTextChanged"));
  }

  /**
   * Invoked when the caret within composed text has changed.
   */
  public void caretPositionChanged(InputMethodEvent e) {
    monitor.post(new ProbeInputMethodEvent(e, "", "caretPositionChanged"));
  }

  ///////////////////////////
  //                       //
  // ItemLisener support   //
  //                       //
  ///////////////////////////

  /**
   * Invoked when an item has been selected or deselected. The code
   * written for this method performs the operations that need to
   * occur when an item is selected (or deselected).
   */
  public void itemStateChanged(ItemEvent e) {
    monitor.post(new ProbeItemEvent(e, "", "itemStateChanged"));
  }

  /////////////////////////
  //                     //
  // KeyListener support //
  //                     //
  /////////////////////////

  /**
   * Invoked when a key has been typed. This event occurs when a key press is followed by a key
   * release.
   */
  public void keyTyped(KeyEvent e) {
    monitor.post(new ProbeKeyEvent(e, "", "keyTyped"));
  }

  /**
   * Invoked when a key has been pressed.
   */
  public void keyPressed(KeyEvent e) {
    monitor.post(new ProbeKeyEvent(e, "", "keyPressed"));
  }

  /**
   * Invoked when a key has been released.
   */
  public void keyReleased(KeyEvent e) {
    monitor.post(new ProbeKeyEvent(e, "", "keyReleased"));
  }

  ///////////////////////////
  //                       //
  // MouseListener support //
  //                       //
  ///////////////////////////

  /**
   * Invoked when the mouse has been clicked on a component.
   */
  public void mouseClicked(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mouseClicked"));
  }

  /**
   * Invoked when a mouse button has been pressed on a component.
   */
  public void mousePressed(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mousePressed"));
  }

  /**
   * Invoked when a mouse button has been released on a component.
   */
  public void mouseReleased(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mouseReleased"));
  }

  /**
   * Invoked when the mouse enters a component.
   */
  public void mouseEntered(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mouseEntered"));
  }

  /**
   * Invoked when the mouse exits a component.
   */
  public void mouseExited(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mouseExited"));
  }

  /////////////////////////////////
  //                             //
  // MouseMotionListener support //
  //                             //
  /////////////////////////////////

  /**
   * Invoked when a mouse button is pressed on a component and then
   * dragged. Mouse drag events will continue to be delivered to the
   * component where the first originated until the mouse button is
   * released (regardless of whether the mouse position is within the
   * bounds of the component).
   */
  public void mouseDragged(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mouseDragged"));
  }

  /**
   * Invoked when the mouse button has been moved on a component (with no
   * buttons no down).
   */
  public void mouseMoved(MouseEvent e) {
    monitor.post(new ProbeMouseEvent(e, "", "mouseMoved"));
  }

  ////////////////////////////////////
  //                                //
  // PropertyChangeListener support //
  //                                //
  ////////////////////////////////////

  /**
   * This method gets called when a bound property is changed.
   */
  public void propertyChange(PropertyChangeEvent e) {
    monitor.post(new ProbePropertyChangeEvent(e, "", "propertyChange"));
  }

  //////////////////////////
  //                      //
  // TextListener support //
  //                      //
  //////////////////////////

  /**
   * Invoked when the value of the text has changed. The code written
   * for this method performs the operations that need to occur when
   * text changes.
   */
  public void textValueChanged(TextEvent e) {
    monitor.post(new ProbeTextEvent(e, "", "textValueChanged"));
  }

  ////////////////////////////
  //                        //
  // WindowListener support //
  //                        //
  ////////////////////////////

  /**
   * Invoked when a window has been opened.
   */
  public void windowOpened(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowOpened"));
  }

  /**
   * Invoked when a window is in the process of being closed. The
   * close operation can be overridden at this point.
   */
  public void windowClosing(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowClosing"));
  }

  /**
   * Invoked when a window has been closed.
   */
  public void windowClosed(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowClosed"));
  }

  /**
   * Invoked when a window is iconified.
   */
  public void windowIconified(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowIconified"));
  }

  /**
   * Invoked when a window is de-iconified.
   */
  public void windowDeiconified(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowDeiconified"));
  }

  /**
   * Invoked when a window is activated.
   */
  public void windowActivated(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowActivated"));
  }

  /**
   * Invoked when a window is de-activated.
   */
  public void windowDeactivated(WindowEvent e) {
    monitor.post(new ProbeWindowEvent(e, "", "windowDeactivated"));
  }


}
