/*
 * @(#)BinaryCompositeBg.java	1.10 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */

package sun.java2d.loops;

import sun.java2d.loops.GraphicsPrimitive;
import java.awt.Color;

/**
*   BinaryCompositeBg
*   1)  copies rectangle of pixels from one surface to another
*   2)  blends src and dest based on alpha rule,
*       extraAlpha, srcAlpha, and dstAlpha
*   3) must accept [sx, sy, dx, dy]
*      from within the surface description data
*   4) applies an XOR color
*   5) fills transparent result pixels with a background color
*
*   precise behavior is undefined if the source surface
*   and the destination surface are the same surface
*   with overlapping regions of pixels
*/

public abstract class BinaryCompositeBg extends GraphicsPrimitive
{
    private final static String methodSignature =
        "BinaryCompositeBg(Lsun/java2d/loops/ImageData;Lsun/java2d/loops/ImageData;III[BIIFLjava/awt/Color;Ljava/awt/Color;)V";
                
    public static String getMethodSignature() {
        return methodSignature;
    }

    private final static int lockFlags[] = { LOCK_FOR_READ, LOCK_FOR_WRITE };

    protected BinaryCompositeBg(int[] surfaceTypes) {
	super(surfaceTypes, lockFlags, methodSignature);
    }

    /**
    *   All BinaryCompositeBg implementors must have this invoker method
    */
    public abstract void BinaryCompositeBg(
        ImageData srcImage,
        ImageData dstImage,
        int width, int height,

        int nRule,

        byte [] bAlphaArray, int nAlphaArrayOffset, int nAlphaArraySize,
        float fExtraAlpha,

        Color bgColor, Color XORColor
    );
}
