/*
 * @(#)DrawGlyphVector.java	1.12 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */

package sun.java2d.loops;

import sun.java2d.loops.GraphicsPrimitive;
import java.awt.Color;
import java.awt.Font;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import sun.java2d.loops.DrawGlyphVectorRasterContext;

/**
*   DrawGlyphVector
*   1) draw solid color text onto destination surface
*   2) must accept output area [x, y, dx, dy]
*      from within the surface description data for clip rect
*/

public abstract class DrawGlyphVector extends GraphicsPrimitive
{
    private final static String methodSignature =
        "DrawGlyphVector(Lsun/java2d/loops/ImageData;Ljava/awt/font/GlyphVector;FFLjava/awt/Color;Ljava/lang/Font;[FZZ)V";

    public static String getMethodSignature() {
        return methodSignature;
    }

    private final static int lockFlags[] = {LOCK_FOR_WRITE | LOCK_FOR_READ};

    protected DrawGlyphVector(int[] surfaceTypes) {
	super(surfaceTypes, lockFlags, methodSignature);
    }

    protected DrawGlyphVector(int[] surfaceTypes, int[] lockFlags) {
	super(surfaceTypes, lockFlags, methodSignature);
    }

    /**
    *   All DrawGlyphVector implementors must have this invoker method
    */
    public abstract void DrawGlyphVector(
        ImageData dest,
        GlyphVector gv,
        float fX,
        float fY,
        Color fgColor,
        Font fontObject,
        double[] txGlyph,
        double[] txDev,
        boolean doAntiAlias,
        boolean doFractEnable
    );
    /*
    *   create cached context for raster drawing
    */
    public DrawGlyphVectorRasterContext createCachedContext(
            Raster raster, ColorModel cm, Color color, Font font,
            AffineTransform txGlyph, AffineTransform txDev,
            boolean doAntiAlias,
            boolean doFractEnable,
            int clipX, int clipY, int clipW, int clipH) {
        return null;
    }
}
