/*
 * @(#)DrawPolygons.java	1.10 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */

package sun.java2d.loops;

import sun.java2d.loops.GraphicsPrimitive;
import java.awt.Color;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import sun.java2d.loops.DrawPolygonsRasterContext;

/**
*   DrawPolygons
*   1) draw single-width line polygons onto destination surface
*   2) must accept output area [x, y, dx, dy]
*      from within the surface description data for clip rect
*/

public abstract class DrawPolygons extends GraphicsPrimitive
{
    private final static String methodSignature =
        "DrawPolygons(Lsun/java2d/loops/ImageData;Ljava/awt/Color;[I[I[IIZ)V";
                
    public static String getMethodSignature() {
        return methodSignature;
    }

    private final static int lockFlags[] = {LOCK_FOR_WRITE};

    protected DrawPolygons(int[] surfaceTypes) {
	super(surfaceTypes, lockFlags, methodSignature);
    }

    /**
    *   All DrawPolygon implementors must have this invoker method
    */
    public abstract void DrawPolygons(
        ImageData dstImage,
        Color color,
        int xPoints[], int yPoints[], int nPoints[],
        int numPolys, boolean close
    );

    /*
    *   create cached context for raster drawing
    */
    public DrawPolygonsRasterContext createCachedContext(
            Raster raster, ColorModel cm, Color color,
            int clipX, int clipY, int clipW, int clipH) {
        return null;
    }
}
