/*
 * @(#)DrawString.java	1.12 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */

package sun.java2d.loops;

import sun.java2d.loops.GraphicsPrimitive;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.AffineTransform;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import sun.java2d.loops.DrawStringRasterContext;

/**
*   DrawString
*   1) draw solid color text onto destination surface
*   2) must accept output area [x, y, dx, dy]
*      from within the surface description data for clip rect
*/

public abstract class DrawString extends GraphicsPrimitive
{
    private final static String methodSignature =
        "DrawString(Lsun/java2d/loops/ImageData;Ljava/lang/String;FFLjava/awt/Color;Ljava/lang/Font;[FZZ)V";

    public static String getMethodSignature() {
        return methodSignature;
    }

    private final static int lockFlags[] = {LOCK_FOR_WRITE | LOCK_FOR_READ};

    protected DrawString(int[] surfaceTypes) {
	super(surfaceTypes, lockFlags, methodSignature);
    }

    protected DrawString(int[] surfaceTypes, int[] lockFlags) {
	super(surfaceTypes, lockFlags, methodSignature);
    }

    /**
    *   All DrawString implementors must have this invoker method
    */
    public abstract void DrawString(
        ImageData dest,
        String srcData,
        float fX,
        float fY,
        Color fgColor,
        Font fontObject,
        double[] matrix,
        boolean doAntiAlias,
        boolean doFractEnable
    );
    /*
    *   create cached context for raster drawing
    */
    public DrawStringRasterContext createCachedContext(
            Raster raster, ColorModel cm, Color color, Font font,
            AffineTransform tx,
            boolean doAntiAlias,
            boolean doFractEnable,
            int clipX, int clipY, int clipW, int clipH) {
        return null;
    }
}
