/*
 * @(#)DrawToFillConverter.java	1.12 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.java2d.pipe;

import java.awt.BasicStroke;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import sun.awt.SunHints;
import sun.java2d.SunGraphics2D;
import sun.dc.path.FastPathProducer;
import sun.dc.path.PathConsumer;
import sun.dc.path.PathException;

/**
 * This interface defines the set of calls that pipeline objects
 * can use to pass on responsibility for drawing generic Shape
 * objects.
 */
public class DrawToFillConverter
    extends DuctusRenderer
    implements ShapeDrawPipe
{
    SimpleRenderPipe outpipe;
    ShapeDrawPipe fillpipe;
    boolean doTx;

    public DrawToFillConverter(SimpleRenderPipe pipe,
			       ShapeDrawPipe fillpipe,
			       boolean doTransform) {
	outpipe = pipe;
	this.fillpipe = fillpipe;
	doTx = doTransform;
    }

    public void draw(SunGraphics2D sg, Shape s) {
	ShapeSpanIterator sr = new ShapeSpanIterator();
	try {
	    draw2(sg, sr, s);
	} finally {
	    sr.dispose();
	}
    }

    public void draw2(SunGraphics2D sg, ShapeSpanIterator sr, Shape s) {
	sr.setOutputArea(sg.getCompBounds());

	if (sg.stroke instanceof BasicStroke) {
	    sr.setRule(PathIterator.WIND_NON_ZERO);

	    BasicStroke bs = (BasicStroke) sg.stroke;
	    AffineTransform transform =
		(sg.transformState == sg.TRANSFORM_GENERIC
		 ? sg.transform : null);

	    PathConsumer consumer =
		DuctusRenderer.createStroker(sr, bs, transform);

	    transform =
		((doTx && sg.transformState != sg.TRANSFORM_ISIDENT)
		 ? sg.transform : null);
	    PathIterator pi = s.getPathIterator(transform);

	    try {
		boolean adjust =
		    (sg.strokeHint != SunHints.INTVAL_STROKE_PURE);
		DuctusRenderer.feedConsumer(pi, consumer, adjust, 0.25f);
	    } catch (PathException e) {
		throw new InternalError("Unable to Stroke shape ("+
					e.getMessage()+")");
	    }
	} else {
	    s = sg.stroke.createStrokedShape(s);
	    AffineTransform at =
		((doTx && sg.transformState != sg.TRANSFORM_ISIDENT)
		 ? sg.transform : null);
	    sr.appendPath(s.getPathIterator(at));
	}
	outpipe.fillSpans(sg, sr);
    }

    public void fill(SunGraphics2D sg, Shape s) {
	fillpipe.fill(sg, s);
    }
}
