/*
 * @(#)PixelToPolyConverter.java	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.java2d.pipe;

import sun.java2d.SunGraphics2D;

/**
 * This class converts calls to the basic pixel rendering
 * methods into calls to line drawing or rect or polygon filling.
 */
public class PixelToPolyConverter
    extends PixelToShapeConverter
    implements PixelDrawPipe
{
    SimpleRenderPipe simplepipe;

    public PixelToPolyConverter(ShapeDrawPipe sdp, SimpleRenderPipe srp) {
	super(sdp);
	simplepipe = srp;
    }

    public void drawLine(SunGraphics2D sg,
			 int x1, int y1, int x2, int y2) {
	simplepipe.drawLine(sg, x1, y1, x2, y2);
    }

    public void drawRect(SunGraphics2D sg,
			 int x1, int y1, int width, int height) {
	if (width <= 0) {
	    return;
	}
	int x2 = x1 + width;
	int y2 = y1 + height;
	simplepipe.drawLine(sg, x1, y1, x2-1, y1);
	simplepipe.drawLine(sg, x2, y1, x2, y2-1);
	simplepipe.drawLine(sg, x2, y2, x1+1, y2);
	simplepipe.drawLine(sg, x1, y2, x1, y1+1);
    }

    public void fillRect(SunGraphics2D sg,
			 int x, int y, int width, int height) {
	simplepipe.fillRect(sg, x, y, width, height);
    }

    public void drawPolyline(SunGraphics2D sg,
			     int xPoints[], int yPoints[],
			     int nPoints) {
	int pts[] = {nPoints};
	simplepipe.drawPolys(sg, xPoints, yPoints, pts, 1, false);
    }

    public void drawPolygon(SunGraphics2D sg,
			    int xPoints[], int yPoints[],
			    int nPoints) {
	int pts[] = {nPoints};
	simplepipe.drawPolys(sg, xPoints, yPoints, pts, 1, true);
    }

    public void fillPolygon(SunGraphics2D sg,
			    int xPoints[], int yPoints[],
			    int nPoints) {
	int pts[] = {nPoints};
	simplepipe.fillPolys(sg, xPoints, yPoints, pts, 1, true);
    }
}
