/*
 * @(#)JdbcOdbcLimits.java	1.24 00/02/02
 *
 * Copyright 1993-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 *
 */

//----------------------------------------------------------------------------
//
// Module:      JdbcOdbcCallableStatement.java
//
// Description: Impementation of the CallableStatement interface class
//
// Product:     JDBCODBC (Java DataBase Connectivity using
//              Open DataBase Connectivity)
//
// Author:      Karl Moss
//
// Date:        March, 1996
//
//----------------------------------------------------------------------------

package sun.jdbc.odbc;

import java.sql.*;
import java.util.Map;
import java.util.Calendar;
import java.math.BigDecimal;

public class JdbcOdbcCallableStatement
	extends		JdbcOdbcPreparedStatement
	implements	java.sql.CallableStatement {

	//====================================================================
	// We need to save registerOutParameter's scale parameters
	// for calls to getObject(BigDecimal);
        // We will use an array of bytes to save space (scale will never be 
	// more than 127.
           public byte scalez[]=new byte[200];
        //====================================================================
   
	//====================================================================
	// Public methods
	//====================================================================

	//--------------------------------------------------------------------
	// Constructor
	// Perform any necessary initialization.
	//--------------------------------------------------------------------

	public JdbcOdbcCallableStatement (
		JdbcOdbcConnectionInterface con)
	{
		super (con);
		lastParameterNull = false;
	}

	//--------------------------------------------------------------------
	// registerOutParameter
	// Before executing a stored procedure call you must explicitly
	// call registerOutParameter to register the java.sql.Type of each
	// out parameter.
	// Note that all parameters are bound to a SQL_C_CHAR data type,
	// so the resulting buffer is a string.  When retrieving the value
	// of the output parameter (with getXXXX), the buffer will be converted
	// to a String first, then the appropriate data type for the
	// getXXXX method.
	//--------------------------------------------------------------------

	public void registerOutParameter (
		int parameterIndex,
		int sqlType)
		throws SQLException
	{
		registerOutParameter (parameterIndex, sqlType, 0);
	}

	// You must also specify the scale for numeric/decimal types:	
	public void registerOutParameter (
		int parameterIndex,
		int sqlType,
		int scale)
		throws SQLException
	{
		// Set the sql type for this OUT parameter

		setSqlType (parameterIndex, sqlType);

		// Save the scale in scalez[]
                 if (parameterIndex <= 200)
                    scalez[parameterIndex]=(byte)scale;

		// Indicate that this is an output parameter
		setOutputParameter (parameterIndex, true);

		// First, allocate a buffer to bind the output parameter
		// to.

		int maxLen;

		switch (sqlType) {
		case Types.DATE:
			maxLen = 10;
			break;
		case Types.TIME:
			maxLen = 8;
			break;
		case Types.TIMESTAMP:
			maxLen = 19;
			if (scale > 0) {
				maxLen += (scale + 1);
			}
			break;
		case Types.BIT:
			maxLen = 1;
			break;
		case Types.TINYINT:
			maxLen = 4;
			break;
		case Types.SMALLINT:
			maxLen = 6;
			break;
		case Types.INTEGER:
			maxLen = 11;
			break;
		case Types.BIGINT:
			maxLen = 20;
			break;
		case Types.REAL:
			maxLen = 13;
			break;
		case Types.FLOAT:
		case Types.DOUBLE:
			maxLen = 22;
			break;
		case Types.DECIMAL:
		case Types.NUMERIC:
			maxLen = 38;
			break;
		default:
			// Get the precision for the SQL type.  

			maxLen = getPrecision (sqlType);
			if ((maxLen <= 0) ||
				(maxLen > JdbcOdbcLimits.DEFAULT_IN_PRECISION)) {
				maxLen = JdbcOdbcLimits.DEFAULT_IN_PRECISION;
			}
		}

		

		// Get the buffer to be used for the output length

		byte dataLen[] = getLengthBuf (parameterIndex);

		// Convert the given Java SQL type to an ODBC SQL type

		sqlType = OdbcDef.jdbcTypeToOdbc (sqlType);

                int buffers[]=new int[4];
                buffers[0]=0;
                buffers[1]=0;
                buffers[2]=0;
                buffers[3]=0;
		byte[] dataBuf = null;
		if (boundParams[parameterIndex - 1].isInOutParameter() &&
			boundParams[parameterIndex - 1].boundValue == null)
			dataBuf = null;
		else
			dataBuf = allocBindBuf (parameterIndex, maxLen + 1);;
		if (boundParams[parameterIndex - 1].isInOutParameter())
		{			
			if (boundParams[parameterIndex - 1].boundValue == null)
			{
				OdbcApi.SQLBindInOutParameterNull (hStmt, parameterIndex, 
				    sqlType, maxLen, boundParams[parameterIndex - 1].scale,
					dataLen, buffers);				
			}			
			else if (sqlType == Types.DOUBLE)
			{
				if (boundParams[parameterIndex - 1].boundType == Types.DOUBLE)
				{
					double value = ((Double)(boundParams[parameterIndex - 1].boundValue)).doubleValue();
					//OdbcApi.SQLBindInOutParameter (hStmt, parameterIndex,
				    //Types.DOUBLE, maxLen, boundParams[parameterIndex - 1].scale,
					//value, dataBuf, buffers);
					String sValue = Double.toString(value);
					try
					{						
						dataBuf = CharsToBytes(OdbcApi.charSet, sValue.toCharArray());
						boundParams[parameterIndex - 1].resetBindDataBuffer(dataBuf);
					}
					catch (Exception e)
					{
					}
					OdbcApi.SQLBindInOutParameterString (hStmt, parameterIndex,
					sqlType, maxLen, boundParams[parameterIndex - 1].scale,
					dataBuf, dataLen, buffers);
				}
			}
			else if (sqlType == Types.BINARY 
				|| (sqlType == Types.LONGVARBINARY &&
					((byte[])(boundParams[parameterIndex - 1].boundValue)).length 
						< JdbcOdbcLimits.DEFAULT_IN_PRECISION)
				|| (sqlType == Types.VARBINARY &&
					((byte[])(boundParams[parameterIndex - 1].boundValue)).length 
						< JdbcOdbcLimits.DEFAULT_IN_PRECISION))
			{
				if (boundParams[parameterIndex - 1].boundType == Types.BINARY ||
					boundParams[parameterIndex - 1].boundType == Types.VARBINARY ||
					boundParams[parameterIndex - 1].boundType == Types.LONGVARBINARY)
				{					
					OdbcApi.SQLBindInOutParameterBinary (hStmt, parameterIndex,
					sqlType, maxLen, boundParams[parameterIndex - 1].scale,
					dataBuf, dataLen, buffers);					
				}
			}
			else if (sqlType == OdbcDef.SQL_TIMESTAMP)
			{
				if (boundParams[parameterIndex - 1].boundType == Types.TIMESTAMP)
				{
					//OdbcApi.SQLBindInOutParameterTimeStamp (hStmt, parameterIndex,
					//sqlType, maxLen, boundParams[parameterIndex - 1].scale,
					//dataBuf, dataLen, buffers);
					Timestamp ts = (Timestamp)boundParams[parameterIndex - 1].boundValue;
					String sTS = ts.toString();
					try
					{						
						dataBuf = CharsToBytes(OdbcApi.charSet, sTS.toCharArray());
						boundParams[parameterIndex - 1].resetBindDataBuffer(dataBuf);
					}
					catch (Exception e)
					{
					}
					OdbcApi.SQLBindInOutParameterString (hStmt, parameterIndex,
					sqlType, maxLen, boundParams[parameterIndex - 1].scale,
					dataBuf, dataLen, buffers);					
				}
			}
			else
			{
				OdbcApi.SQLBindInOutParameterString (hStmt, parameterIndex,
			    sqlType, maxLen, boundParams[parameterIndex - 1].scale,
				dataBuf, dataLen, buffers);						
			}
		}//if InOutParameter
		else
		{
			// Bind the output parameter, always as string data			        
			OdbcApi.SQLBindOutParameterString (hStmt, parameterIndex,
				sqlType, scale, dataBuf, dataLen, buffers);							
		}//if OutParameter
		//save the native pointers from garbage collection
		boundParams[parameterIndex - 1].pA1=buffers[0];
		boundParams[parameterIndex - 1].pA2=buffers[1];
		boundParams[parameterIndex - 1].pB1=buffers[2];
		boundParams[parameterIndex - 1].pB2=buffers[3];
}

	//--------------------------------------------------------------------
	// wasNull
	// An OUT parameter may have the value of SQL NULL; wasNull reports 
	// whether the last value read has this special value.
	//--------------------------------------------------------------------

	public boolean wasNull ()
		throws SQLException
	{
		return lastParameterNull;
	}

	//--------------------------------------------------------------------
	// getString
	//--------------------------------------------------------------------

	public String getString (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return a null object
		if (isNull (parameterIndex)) {
			return null;
		}
		
		// Get the SQL type of the parameter

		int sqlType = getSqlType (parameterIndex); 

		// Convert the data buffer for the output parameter into
		// a String
		String s=null;
		try 
		{
			byte[] bindBuf = getDataBuf(parameterIndex);										
			if (bindBuf != null)
			{				
				s = BytesToChars (OdbcApi.charSet, bindBuf);
			}
		} 
		catch (java.io.UnsupportedEncodingException exx) {}
		// Remove trailing spaces

		//s = s.trim ();

		if (isTracing ()) {
			trace ("String value for OUT parameter " +
				   parameterIndex + "=" + s);
		}		
		return s;
	}

	//--------------------------------------------------------------------
	// Since the output parameter was bound as a specific C type 
	// (always a SQL_C_CHAR), we will have to get the output parameter
	// as a character, then convert it to the proper return type.
	//--------------------------------------------------------------------

	//--------------------------------------------------------------------
	// getBoolean
	//--------------------------------------------------------------------

	public boolean getBoolean (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return false
		if (isNull (parameterIndex)) {
			return false;
		}

		boolean rc = false;

		// Get the out parameter as a String, and look for a
		// '1' for a true value
		String s = getString (parameterIndex).trim();
		if (s.equals("1")) {
			rc = true;
		}
		return rc;
	}

	//--------------------------------------------------------------------
	// getByte
	//--------------------------------------------------------------------

	public byte getByte (
		int parameterIndex)
		throws SQLException
	{
		// Cast getInt to a byte value

		return (byte) getInt (parameterIndex);
	}

	//--------------------------------------------------------------------
	// getShort
	//--------------------------------------------------------------------

	public short getShort (
		int parameterIndex)
		throws SQLException
	{
		// Cast getInt to a short value

		return (short) getInt (parameterIndex);
	}

	//--------------------------------------------------------------------
	// getInt
	//--------------------------------------------------------------------

	public int getInt (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return zero
		if (isNull (parameterIndex)) {
			return 0;
		}

		// Create an Integer object from the output parameter string
		Integer i = new Integer (getString (parameterIndex).trim());

		return i.intValue ();
	}

	//--------------------------------------------------------------------
	// getLong
	//--------------------------------------------------------------------

	public long getLong (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return zero
		if (isNull (parameterIndex)) {
			return 0;
		}
		// Create a Long object from the output parameter string
		Long l = new Long (getString (parameterIndex).trim());

		return l.longValue ();
	}

	//--------------------------------------------------------------------
	// getFloat
	//--------------------------------------------------------------------

	public float getFloat (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return zero
		if (isNull (parameterIndex)) {
			return 0;
		}
		// Create a Float object from the output parameter string
		Float f = new Float (getString (parameterIndex).trim());

		return f.floatValue ();
	}

	//--------------------------------------------------------------------
	// getDouble
	//--------------------------------------------------------------------

	public double getDouble (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return zero
		if (isNull (parameterIndex)) {
			return 0;
		}
		// Create a Double object from the output parameter string
		Double d = new Double (getString (parameterIndex).trim());

		return d.doubleValue ();
/****
		double outValue = 0;
		getDataBuf(parameterIndex);
		if (boundParams[parameterIndex-1].pA1 != 0)
			outValue = getTheOutValue(boundParams[parameterIndex-1].pA1);
		return outValue;
****/
	}

	//--------------------------------------------------------------------
	// getBigDecimal
	//--------------------------------------------------------------------

	public BigDecimal getBigDecimal (
		int parameterIndex,
		int scale)
		throws SQLException
	{
		// If the value is null, return null
		if (isNull (parameterIndex)) {
			return null;
		}
		// Create a BigDecimal object from the output parameter string
		BigDecimal num = new BigDecimal(getString (parameterIndex).trim());
		/* NOTE: In the old-style Bignum there was the notion of 
		   a default rounding mode. There is no longer this 
		   notion, and so we default to something reasonable. */
		return num.setScale(scale, BigDecimal.ROUND_HALF_EVEN);
	}

	//--------------------------------------------------------------------
	// getBytes
	// Returns a byte array of the bound parameter data.
	//--------------------------------------------------------------------

	public byte[] getBytes (
		int parameterIndex)
		throws SQLException
	{
		// If the value is null, return null
		if (isNull (parameterIndex)) {
			return null;
		}
		if (boundParams[parameterIndex-1].isInOutParameter())
		{
		int paramLength = getParamLength(parameterIndex);
		
		byte[] bindBuf = getDataBuf(parameterIndex);
		if (paramLength < bindBuf.length)
		{
			byte[] outBuf = new byte[paramLength];
			for (int i=0; i<getParamLength(parameterIndex); i++)
				outBuf[i] = bindBuf[i];
			boundParams[parameterIndex-1].resetBindDataBuffer(outBuf);
			return outBuf;
		}
		return bindBuf;
		}
		// Convert the bound character string (in hexadecimal)
		// into a byte array
		return hexStringToByteArray (getString (parameterIndex).trim());
	}

	//--------------------------------------------------------------------
	// getDate
	//--------------------------------------------------------------------

	public java.sql.Date getDate (
		int parameterIndex)
		throws SQLException
	{
		// Get the string value and convert to Date object

		String s = getString (parameterIndex).trim();
		if (s == null) {
			return null;
		}
		return java.sql.Date.valueOf (s);
	}

	//--------------------------------------------------------------------
	// getTime
	//--------------------------------------------------------------------

	public java.sql.Time getTime (
		int parameterIndex)
		throws SQLException
	{
		// Get the string value and convert to Time object

		String s = getString (parameterIndex).trim();
		if (s == null) {
			return null;
		}
		return java.sql.Time.valueOf (s);
	}

	//--------------------------------------------------------------------
	// getTimestamp
	//--------------------------------------------------------------------

	public java.sql.Timestamp getTimestamp (
		int parameterIndex) 
		throws SQLException
	{
		// Get the string value and convert to Timestamp object

		String s = getString (parameterIndex).trim();
		if (s == null) {
			return null;
		}

		// If all we got was a date, put on default seconds

		if (s.length () == 10) {
			s += " 00:00:00";
		}		
		return java.sql.Timestamp.valueOf (s);
	}


	//--------------------------------------------------------------------
	// getObject
	// Returns a Java object for the parameter.
	// See the JDBC spec's "Dynamic Programming" chapter for details.
	//--------------------------------------------------------------------

	public Object getObject (
		int parameterIndex)
		throws SQLException
	{
		Object	value = null;

		// Get the SQL type of the parameter

		int sqlType = getSqlType (parameterIndex); 

		// If the column is null, always return a null object

		if (isNull (parameterIndex)) {
			return null;
		}

		// For each SQL type, call the appropriate routine and
		// convert to the proper object type

		switch (sqlType) {

		case Types.CHAR:
		case Types.VARCHAR:
		case Types.LONGVARCHAR:

//			// Get the data as a string, convert it to a byte
//			// array, and return an InputStream
//
//			String s = getLongVarChar (parameterIndex);
//			if (s != null) {
//				byte b[] = new byte[s.length ()];
//				s.getBytes (0, s.length (), b, 0);
//				value = (Object) new JdbcOdbcInputStream (
//					OdbcApi, hStmt, parameterIndex, b);
//			}

			value = (Object) getString (parameterIndex);
			break;

		case Types.NUMERIC:
		case Types.DECIMAL:

			// The getNumeric method requires us to provide
			// a scale.  The only way to get the scale is to
			// call DatabaseMetaData.getProcedureColumns
			// (which may not be supported for the data source)
			// We'll use the scale stored in scalez[]
			  if (parameterIndex<=200)
                             value = (Object) getBigDecimal(parameterIndex, scalez[parameterIndex]);
                          else
       			     value = (Object) getBigDecimal(parameterIndex, JdbcOdbcLimits.DEFAULT_OUT_SCALE);
			break;

		case Types.BIT:
			value = (Object) new Boolean (
					getBoolean (parameterIndex));
			break;

		case Types.TINYINT:
			value = (Object) new Integer (getByte (
					parameterIndex));
			break;

		case Types.SMALLINT:
			value = (Object) new Integer (getShort (
					parameterIndex));
			break;

		case Types.INTEGER:
			value = (Object) new Integer (getInt (
					parameterIndex));
			break;

		case Types.BIGINT:
			value = (Object) new Long (getLong (parameterIndex));
			break;

		case Types.REAL:
		case Types.FLOAT:
			value = (Object) new Float (getFloat (parameterIndex));
			break;

		case Types.DOUBLE:
			value = (Object) new Double (getDouble (
					parameterIndex));
			break;
			
		case Types.BINARY:
		case Types.VARBINARY:
		case Types.LONGVARBINARY:

//			// Get the data as a byte array and convert to an
//			// InputStream
//
//			byte b[] = getLongVarBinary (parameterIndex);
//			if (b != null) {
//				value = (Object) new JdbcOdbcInputStream (
//					OdbcApi, hStmt, parameterIndex, b);
//			}
			value = (Object) getBytes (parameterIndex);
			break;

		case Types.DATE:
			value = (Object) getDate (parameterIndex);
			break;

		case Types.TIME:
			value = (Object) getTime (parameterIndex);
			break;

		case Types.TIMESTAMP:
			value = (Object) getTimestamp (parameterIndex);
			break;

		}

		return value;
	}


	// New JDBC 2.0 API

	public BigDecimal getBigDecimal (
		int parameterIndex)
		throws SQLException
	{	  
		// If the value is null, return null
		if (isNull (parameterIndex)) 
		{
			return null;
		}
		// Create a BigDecimal object from the output parameter string
		BigDecimal num = new BigDecimal(getString (parameterIndex).trim());
		
		/* NOTE: for 2.0 version the decimal value return a full precision.
		   no scale is specified. */
		
		return num;
	}


	public Object getObject (
		int i,
		Map map)
		throws SQLException
	{ return null; }

	public Ref getRef (
		int i)
		throws SQLException
	{ return null; }

	public Blob getBlob (
		int i)
		throws SQLException
	{ return null; }

	public Clob getClob (
		int i)
		throws SQLException
	{ return null; }

	public Array getArray (
		int i)
		throws SQLException
	{ return null; }

	public Date getDate ( 
		int parameterIndex, 
		Calendar cal) 
		throws SQLException
	{ 
		long lValue = 0L;
		java.util.Date tmpDate = null;
		
		// Get the string value and convert to Date object

		String s = getString (parameterIndex).trim();
		
		if (s != null)
		{
			
			  tmpDate = Date.valueOf(s);			    
		
			  cal.setTime(tmpDate);
			
			  lValue = (cal.getTime()).getTime();
		
		}		 
		else if (s == null)
		{
			return null;
		}
		
		if (lValue == 0L)
		{
			return null;
		}
		else		    
			return new java.sql.Date(lValue);
	}

	public Time getTime ( 
		int parameterIndex, 
		Calendar cal) 
		throws SQLException
	{		  
		long lValue = 0L;
		java.util.Date tmpTime = null;
	
		// Get the string value and convert to Time object

		String s = getString (parameterIndex).trim();
		
		if (s != null)
		{
			try
			{
			  tmpTime = Time.valueOf(s);
			
			  cal.setTime(tmpTime);
			
			  lValue = (cal.getTime()).getTime();
			}
			catch (Exception te){}												      
		
		}
		else if (s == null) 
		{
			return null;
		}
		
		if (lValue == 0L)
		{
			return null;
		}
		else
			return new java.sql.Time(lValue);

	}

	public Timestamp getTimestamp ( 
		int parameterIndex, 
		Calendar cal) 
		throws SQLException
	{ 
		long lValue = 0L;
		java.util.Date tmpTS = null;
				
		// Get the string value and convert to Timestamp object

		String s = getString (parameterIndex).trim();
	
		if (s != null)
		{
		
			// If all we got was a date, put on default seconds

			if (s.length () == 10) 
			{
				s += " 00:00:00";
			}
		
			try
			{
			  tmpTS = Timestamp.valueOf(s);
			
			  cal.setTime(tmpTS);
			
			  lValue = (cal.getTime()).getTime();
			}
			catch (Exception te){}												      
		
		}					 
		else if (s == null)		    
		{
			return null;
		}
		
		if (lValue == 0L)
		{
			return null;
		}
		else
			return new java.sql.Timestamp(lValue);

	}

	public void registerOutParameter (
		int paramIndex,
		int sqlType,
		String typeName)
		throws SQLException
	{}

	//--------------------------------------------------------------------
	// isNull
	// Helper method to determine if an output parameter is null.  Also
	// sets the lastParameterNull attribute for use with wasNull.
	//--------------------------------------------------------------------

	protected boolean isNull (
		int parameterIndex)
		throws SQLException
	{
		// Verify that the given parameter is for OUTPUT

		if (!isOutputParameter (parameterIndex)) {
			throw new SQLException ("Parameter " + parameterIndex +
					" is not an OUTPUT parameter");
		}

		boolean rc = false;
		rc = (getParamLength(parameterIndex) == OdbcDef.SQL_NULL_DATA);
		if (isTracing ()) {
			trace ("Output Parameter " + parameterIndex +
					" null: " + rc);
		}
		lastParameterNull = rc;
		return rc;
	}

/*
	//THIS MAY NEED TO BE MOVE TO JdbcOdbcPreparedStatement class!!
	// now also used by executeBatchUpdate for PreparedStatement.
	//--------------------------------------------------------------------
	// setSqlType
	// Sets the Java sql type for when registering an OUT parameter
	//--------------------------------------------------------------------

	protected void setSqlType (
		int index,
		int type)
	{
		// Sanity check the parameter number

		if ((index >= 1) &&
		    (index <= numParams)) {
			boundParams[index - 1].setSqlType (type);
		}
	}


	//THIS MAY NEED TO BE MOVE TO JdbcOdbcPreparedStatement class!!
	// now also used by executeBatchUpdate for PreparedStatement.
	//--------------------------------------------------------------------
	// getSqlType
	// Gets the Java sql type for when registering an OUT parameter
	//--------------------------------------------------------------------

	protected int getSqlType (
		int index)
	{
		int type = Types.OTHER;

		// Sanity check the parameter number

		if ((index >= 1) &&
		    (index <= numParams)) {
			type = boundParams[index - 1].getSqlType ();
		}
		return type;
	}
*/

	//--------------------------------------------------------------------
	// setOutputParameter
	// Sets the output parameter flag
	//--------------------------------------------------------------------

	protected void setOutputParameter (
		int index,
		boolean output)
	{
		// Sanity check the parameter number

		if ((index >= 1) &&
		    (index <= numParams)) {
			boundParams[index - 1].setOutputParameter (output);
		}
	}

	//--------------------------------------------------------------------
	// isOutputParameter
	// Returns true if the given parameter is for OUTPUT
	//--------------------------------------------------------------------

	protected boolean isOutputParameter (
		int index)
	{
		boolean output = false;

		// Sanity check the parameter number

		if ((index >= 1) &&
		    (index <= numParams)) {
			output = boundParams[index - 1].isOutputParameter ();
		}
		return output;
	}



	public synchronized void close ()
		throws SQLException
	{
		if (isTracing ()) {
			trace ("*Statement.close");
		}

		// Close/clear our result set

		clearMyResultSet ();

		// Reset last warning message

		try {
			clearWarnings ();
			if (hStmt != OdbcDef.SQL_NULL_HSTMT) {
				OdbcApi.SQLFreeStmt (hStmt, OdbcDef.SQL_DROP);
				hStmt = OdbcDef.SQL_NULL_HSTMT;
				FreeParams();
				for (int pindex=1; boundParams != null && pindex <= boundParams.length; pindex++)
				{
					boundParams[pindex-1].binaryData = null;
					boundParams[pindex-1].initialize ();
					boundParams[pindex-1].paramInputStream = null;
					boundParams[pindex-1].inputParameter = false;
				}
			}
		}
		catch (SQLException ex) {
			// If we get an error, ignore
		}

		// Remove this Statement object from the Connection object's
		// list

		myConnection.deregisterStatement (this);
	}

public synchronized void FreeParams()
        throws NullPointerException
{
	try {
		for (int pindex=1; pindex <= boundParams.length; pindex++)
		{
			if (boundParams[pindex-1].pA1!=0)
			{
				OdbcApi.ReleaseStoredBytes (boundParams[pindex-1].pA1, boundParams[pindex-1].pA2);
				boundParams[pindex-1].pA1=0;
				boundParams[pindex-1].pA2=0;
			}
			if (boundParams[pindex-1].pB1!=0)
			{
				OdbcApi.ReleaseStoredBytes (boundParams[pindex-1].pB1, boundParams[pindex-1].pB2);
				boundParams[pindex-1].pB1=0;
				boundParams[pindex-1].pB2=0;
			}
			if (boundParams[pindex-1].pC1!=0)
			{
				OdbcApi.ReleaseStoredBytes (boundParams[pindex-1].pC1, boundParams[pindex-1].pC2);
				boundParams[pindex-1].pC1=0;
				boundParams[pindex-1].pC2=0;
			}
			if (boundParams[pindex-1].pS1!=0)
			{
				OdbcApi.ReleaseStoredChars (boundParams[pindex-1].pS1, boundParams[pindex-1].pS2);
				boundParams[pindex-1].pS1=0;
				boundParams[pindex-1].pS2=0;
			}
		}
	}
	catch (NullPointerException npx)
	{
		//Do nothing
	}
}//FreeParams


	//====================================================================
	// Data attributes
	//====================================================================

	private boolean lastParameterNull;	// true if the last parameter
						//  referenced by a getXXX
						//  function was null
	
}
