/*
 * @(#)NetworkClient.java	1.25 00/02/02
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.net;

import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;
import java.net.URL;

/**
 * This is the base class for network clients.
 *
 * @version	1.25, 02/02/00
 * @author	Jonathan Payne
 */
public class NetworkClient {
    /** Socket for communicating with server. */
    protected Socket	serverSocket = null;

    /** Stream for printing to the server. */
    public PrintStream	serverOutput;

    /** Buffered stream for reading replies from server. */
    public InputStream	serverInput;

    /** Open a connection to the server. */
    public void openServer(String server, int port)
	throws IOException, UnknownHostException {
	if (serverSocket != null)
	    closeServer();
	serverSocket = doConnect (server, port);
	serverOutput = new PrintStream(new BufferedOutputStream(serverSocket.getOutputStream()),
				       true);
	serverInput = new BufferedInputStream(serverSocket.getInputStream());
    }

    /**
     * Return a socket connected to the server, with any
     * appropriate options pre-established
     */
    protected Socket doConnect (String server, int port)
    throws IOException, UnknownHostException {
	return new Socket (server, port);
    }

    /** Close an open connection to the server. */
    public void closeServer() throws IOException {
	if (! serverIsOpen()) {
	    return;
	}
	serverSocket.close();
	serverSocket = null;
	serverInput = null;
	serverOutput = null;
    }

    /** Return server connection status */
    public boolean serverIsOpen() {
	return serverSocket != null;
    }

    /** Create connection with host <i>host</i> on port <i>port</i> */
    public NetworkClient(String host, int port) throws IOException {
	openServer(host, port);
    }

    public NetworkClient() {}
}
