/*
 * @(#)FtpInputStream.java	1.13 01/04/21
 *
 * Copyright 1995-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.net.ftp;

import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.IOException;
import sun.net.TelnetInputStream;

/*
 * The only purpose of this class is to hold onto an instance of
 * FtpClient to prevent the FtpClient from being garbage collected.
 * If the client gets collected it will close the command stream
 * which will cause the remote ftp daemon to close its outputstream
 * which will cause and EOF on this inputstream on our end.  
 *
 * The real answer is to re-architect all this stuff to not open an
 * ftp connection each time the user needs something from the remote
 * side.  The other correct thing it to totally ignore all the ftp
 * client stuff and just use the ftp proxy stuff in the CERN HTTP
 * server.
 *
 * @version	1.13, 04/21/01
 * @author	Chris Warth
 */

public class FtpInputStream extends TelnetInputStream {
    FtpClient ftp;

    FtpInputStream(FtpClient ftp, InputStream fd, boolean binary) {
	super(fd, binary);
	this.ftp = ftp;
    }

    public void close() {
	try {
	    super.close();
	    if (ftp != null) {
		ftp.closeServer();
	    }
	} catch (IOException e) {
	}
    }
}
