/*
 * @(#)KeepAliveStream.java	1.14 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.net.www.http;

import java.net.URL;
import java.net.HttpURLConnection;
import java.io.*;
import sun.net.www.MeteredStream;
import sun.net.ProgressData;
import sun.net.ProgressEntry;
import java.util.StringTokenizer;

/**
 * A stream that has the property of being able to be kept alive for
 * multiple downloads from the same server.
 *
 * @version 1.14, 02/02/00
 * @author Stephen R. Pietrowicz (NCSA)
 * @author Dave Brown
 */
public
class KeepAliveStream extends MeteredStream {

    // instance variables
    HttpClient hc;

    boolean hurried;

    /**
     * Constructor
     */
    public KeepAliveStream(InputStream is, ProgressEntry te, HttpClient hc) {
	super(is, te);
	this.hc = hc;
    }

    /**
     * Attempt to cache this connection
     */
    public void close() throws IOException {
	// Skip past the data that's left in the Inputstream because
	// some sort of error may have occurred.
	// REMIND:  Just doing a skip() is a very naive thing to do.
	// We could have died on the first two bytes of a huge
	// download, in which case it would probably be better to
	// just close down the KeepAliveStream, rather than spending
	// the time to skip all that data.  We need to determine
	// what the right cut-off time is.
	// NOTE: Don't close super class
	try {
	    if (expected > count) {
		long nskip = (long) (expected - count);
		long n = 0;
		while (n < nskip) {
		    nskip = nskip - n;
		    n = skip(nskip);		    
		}
	    }          
	    if (!closed && !hurried) {
		HttpClient.finished(hc);
	    }
	} finally {
	    ProgressData.pdata.unregister(te);
	    // nulling out the underlying inputstream as well as
	    // httpClient to let gc collect the memories faster
	    in = null;
	    hc = null;
	    closed = true;
	}
    }

    /* we explicitly do not support mark/reset */

    public boolean markSupported() {
	return false;
    }

    public void mark(int limit) {}

    public void reset() throws IOException {
	throw new IOException("mark/reset not supported");
    }

    public synchronized boolean hurry() {
	try {
	    /* CASE 0: we're actually already done */
	    if (closed || count >= expected) {
		return false;
	    } else if (in.available() < (expected - count)) {
		/* CASE I: can't meet the demand */
		return false;
	    } else {
		/* CASE II: fill our internal buffer
		 * Remind: possibly check memory here
		 */
		byte[] buf = new byte[expected - count];
		DataInputStream dis = new DataInputStream(in);
		dis.readFully(buf);
		in = new ByteArrayInputStream(buf);
		hurried = true;
		return true;
	    }
	} catch (IOException e) {
	    // e.printStackTrace();
	    return false;
	}
    }
}



