/*
 * @(#)AppletResourceConnection.java	1.13 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.net.www.protocol.appletresource;

import java.net.URL;
import java.io.IOException;
import java.io.InputStream;
import sun.net.www.URLConnection;
import sun.net.www.MessageHeader;
// import sun.applet.AppletResourceLoader;
import sun.net.*;
import java.net.*;

/**
 * AppletResource: represents a resource obtained through an AppletClassLoader
 * Resource URLs are of the form "appresource:<baseURL>/+/<resource name>"
 *
 * @see		Handler
 * @version 	1.13, 02/02/00
 * @author	Eduardo Pelegri-Llopart
 */

public class AppletResourceConnection extends URLConnection {
    private static boolean debug = false;// debugging?

    private Object resource;	// the resource we are looking for
    private String base;	// basename for resource
    private String member;	// member relative to base
    private URL cachedURL;	// the URL under which the resource is cached

    protected AppletResourceConnection (URL url)
		throws MalformedURLException, IOException
    {
	super(url);
	debug("AppletResourceConnection("+url+")");
	String file = url.getFile();
	if (file.startsWith("/")) {
	    file = file.substring(1);
	}
	base = file.substring(0, file.indexOf("/+/"));
	member = file.substring(file.indexOf("/+/")+3);
	cachedURL = new URL(base + member);

	debug(" base: "+base);
	debug(" member: "+member);
	debug(" cachedURL: "+cachedURL);
    }

    public void connect() throws IOException {
	debug("Looking for "+cachedURL+" in AppletResourceLoader");
	// Object o = AppletResourceLoader.getLocalResource(cachedURL);
	Object o = null;
	if (o == null) {
	    debug("Invalid resource name");
	    resource = null;
	    return;
	} else {
	    debug("Found resource");
	    resource = o;
	}
    }

    public Object getContent() throws IOException {
	if (!connected) {
	    connect();
	}
	return resource;
    }

    public InputStream getInputStream() throws IOException {
	if (!connected) {
	    connect();
	}

	if (resource instanceof InputStream) {
	    return (InputStream) resource;
	}
	// return AppletResourceLoader.getLocalResourceStream(cachedURL);
	return null;
    }

    private void debug(String msg) {
	if (debug) {
	    System.err.println(msg);
	}
    }
}
