/*
 * @(#)FtpURLConnection.java	1.49 00/02/02
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/**
 * FTP stream opener.
 */

package sun.net.www.protocol.ftp;

import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.PrintStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.FileNotFoundException;
import java.io.File;
import java.net.URL;
import java.net.URLStreamHandler;
import java.net.SocketPermission;
import java.net.UnknownHostException;
import java.net.MalformedURLException;
import java.security.Permission;
import sun.net.www.MessageHeader;
import sun.net.www.MeteredStream;
import sun.net.www.URLConnection;
import sun.net.ftp.FtpClient;
import sun.net.ProgressData;
import sun.net.ProgressEntry;
import sun.net.www.ParseUtil;
import sun.security.action.GetPropertyAction;


/** open an ftp input stream given a URL */
public class FtpURLConnection extends URLConnection {

    InputStream is = null;

    FtpClient ftp = null;
    Permission permission;

    String password;
    String user;

    String host;

    public FtpURLConnection(URL url) {
	super(url);
	host = url.getHost();
        String userInfo = url.getUserInfo();
        
	if (userInfo != null) { // get the user and password
	    int delimiter = userInfo.indexOf(':');
	    user = userInfo.substring(0, delimiter++);
	    password = userInfo.substring(delimiter);
	}
    }

    public synchronized void connect() throws IOException {
	if (connected) {
	    return;
	}
	try {
	    ftp = new FtpClient(host);
	} catch (UnknownHostException e) {
	    // Maybe do something smart here, like use a proxy like iftp.
	    // Just keep throwing for now.
	    throw e;
	}
	if (user == null || password == null) {
	    user = "anonymous";
	    String vers = (String) java.security.AccessController.doPrivileged(
                new GetPropertyAction("java.version"));
	    password = (String) java.security.AccessController.doPrivileged(
                new GetPropertyAction("ftp.protocol.user", 
				      "Java" + vers +"@"));
	}
	ftp.login(user, password);
    }

    public InputStream getInputStream() throws IOException {
	if (!connected) {
	    connect();
	}

	if (is != null) {
	    return is;
	}

	MessageHeader msgh = new MessageHeader();
        String decodedPath = null;

	try {
	    ftp.binary();
            decodedPath = ParseUtil.decode(url.getPath());
	    is = ftp.get(decodedPath);

	    /* Try to get the size of the file in bytes.  If that is
	       successful, then create a MeteredStream. */
	    try {
		String response = ftp.getResponseString();
		int offset;

		if ((offset = response.indexOf(" bytes)")) != -1) {
		    int i = offset;
		    int c;

		    while (--i >= 0 && ((c = response.charAt(i)) >= '0'
					&& c <= '9'))
			;
		    i = Integer.parseInt(response.substring(i + 1, offset));
		    msgh.add("content-length", ""+i);
		    if (i > 0) {
			ProgressEntry pe = new ProgressEntry(decodedPath, null);
			pe.update(0, i);
			ProgressData.pdata.register(pe);
			is = new MeteredStream(is, pe);
		    }
		}
	    } catch (Exception e) {
		e.printStackTrace();
		/* do nothing, since all we were doing was trying to
		   get the size in bytes of the file */
	    }

	    String type = guessContentTypeFromName(decodedPath);
	    if (type == null && is.markSupported()) {
		type = guessContentTypeFromStream(is);
	    }
	    if (type != null) {
		msgh.add("content-type", type);
	    }
	} catch (FileNotFoundException e) {
	    ftp.cd(url.getFile());
	    /* if that worked, then make a directory listing
	       and build an html stream with all the files in
	       the directory */
	    ftp.ascii();

	    PipedOutputStream   os = new PipedOutputStream();

	    is = new PipedInputStream(os);
	    new FtpDirectoryThread(this, ftp, url, host, ftp.list(), os).start();
	    msgh.add("content-type", "text/html");
	}
	setProperties(msgh);
	return is;
    }

    String guessContentTypeFromFilename(String fname) {
	return guessContentTypeFromName(fname);
    }

    public Permission getPermission() {
	if (permission == null) {
	    int port = url.getPort();
	    port = port < 0 ? FtpClient.FTP_PORT : port;
	    String host = this.host + ":" + port;
	    permission = new SocketPermission(host, "connect");
	}
	return permission;
    }
}

class FtpDirectoryThread extends Thread {

    String host;
    static Hashtable images = new Hashtable();

    static final int imageWidth = Integer.getInteger("hotjava.file.iconwidth", 24).intValue();
    static final int imageHeight = Integer.getInteger("hotjava.file.iconheight", 26).intValue();

    static String fullImagePath(String name) {

	return "doc:/lib/images/ftp/" + name + ".gif";
    }

    static {
	images.put("image/gif", fullImagePath("gif"));
	images.put("image/tiff", fullImagePath("tiff"));
	images.put("audio/basic", fullImagePath("audio"));
	images.put("application/octet-stream", fullImagePath("compress"));
	images.put("application/postscript", fullImagePath("ps"));
	images.put("text/plain", fullImagePath("text"));
    }

    FtpURLConnection	conn;
    FtpClient		ftp;
    PrintStream		os;
    InputStream		is;
    URL			url;

    FtpDirectoryThread(FtpURLConnection conn, FtpClient ftp, URL url,
		       String host, InputStream is, OutputStream os) {
	this.conn = conn;
	this.ftp = ftp;
	this.url = url;
	this.host = host;
	this.is = is;
	this.os = new PrintStream(new BufferedOutputStream(os));
    }

    /* drwxrwxr-x  30 jpayne   staff       1024 Jun 20  1994 src */

    public void run() {
	try {
	    is = new BufferedInputStream(is);

	    byte	    data[] = new byte[512];
	    String	    file;
	    DataInputStream dis = new DataInputStream(is);
	    String	    urlString = url.toExternalForm();
	    String	    title;

	    if (!urlString.endsWith("/")) {
		urlString += "/";
	    }

            String decodedPath = ParseUtil.decode(url.getPath());
	    title = "Directory: " + decodedPath + "@" + this.host;

	    os.println("<html>\n<head>\n<title>"
		       + title + "</title>\n</head>");
	    os.println("<body>");
	    os.println("<h2>" + title + "</h2>");
	    os.println("<pre>");
	    if (ftp.welcomeMsg != null) {
		os.println(ftp.welcomeMsg);
		ftp.welcomeMsg = null;
	    }

	    try {
		addFile('d', "&lt;Parent Directory&gt;", null,
			new URL(null, urlString + "../"));
	    } catch (MalformedURLException e) {
	    }
	    while (true) {
		try {
		    file = dis.readLine();
		} catch (IOException e) {
		    System.err.println("Premature end of FTP directory " + e +
				       " (host: " + host + ")");
		    break;
		}
		if (file == null) {
		    break;
		}
		StringTokenizer	t = new StringTokenizer(file, " ");
		String	mode;
		String	size;
		String	name;

		try {
		    mode = t.nextToken();
		    t.nextToken();
		    t.nextToken();
		    t.nextToken();
		    size = t.nextToken();
		    t.nextToken();
		    t.nextToken();
		    t.nextToken();
		    name = t.nextToken();
		    if (name.equals(".") || name.equals("..")) {
			continue;
		    } else {
			try {
			    addFile(mode.charAt(0) == 'd' ? 'd' : 'f',
				    name, size, new URL(null, urlString + name));
			} catch (MalformedURLException e) {
			}
		    }
		} catch (NoSuchElementException e) {
		}
	    }
	    os.println("</pre></body>\n</html>");
	} finally {
	    os.close();
	    try {
		is.close();
	    } catch (IOException e) {
	    }
	    try {
		ftp.closeServer();
	    } catch (IOException e) {
	    }
	}
    }

    void addImage(String name) {
	os.print("<img align=\"middle\" src=\""+name+"\" border=0 width="+
		 imageWidth+" height="+imageHeight+">");
    }

    String lookupImage(String contentType) {
	return (String)images.get(contentType);
    }

    void addFile(int type, String name, String size, URL url) {
	os.print("  ");
	os.print("<a href=\""+url+"\">");
	switch (type) {
	case 'f':
            String decodedPath = ParseUtil.decode(url.getPath());
	    String ctype = conn.guessContentTypeFromFilename(decodedPath);
	    String image;
	    if (ctype == null) {
		image = null;
	    } else {
		image = lookupImage(ctype);
	    }

	    if (image == null) {
		image = fullImagePath("file");
	    }

	    addImage(image);
	    break;

	case 'd':
	    addImage(fullImagePath("directory"));
	    break;
	}
	os.print("\t");
	os.print(name);
	if (type == 'd') {
	    os.print("/");
	} else if (size != null) {
	    os.print(" (" + size + " bytes)");
	}
	os.print("</a>");
	os.print("\n");
    }
}

