/*
 * @(#)BasicAuthentication.java	1.9 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.net.www.protocol.http;

import java.net.URL;
import java.net.PasswordAuthentication;


/**
 * BasicAuthentication: Encapsulate an http server authentication using
 * the "basic" scheme.
 *
 * @author Bill Foote
 * @version 1.9, 02/02/00 
 */


class BasicAuthentication extends AuthenticationInfo {


    /** The authentication string for this host, port, and realm.  This is
	a simple BASE64 encoding of "login:password".    */
    String auth;

    /**
     * Create a BasicAuthentication
     */
    public BasicAuthentication(boolean isProxy, String host, int port,
			       String realm, PasswordAuthentication pw) {
	super(isProxy ? PROXY_AUTHENTICATION : SERVER_AUTHENTICATION,
	      host, port, realm);
	String plain = pw.getUserName() + ":";
	byte[] nameBytes = plain.getBytes();

	// get password bytes
	char[] passwd = pw.getPassword();
	byte[] passwdBytes = new byte[passwd.length];
	for (int i=0; i<passwd.length; i++)
	    passwdBytes[i] = (byte)passwd[i];

	// concatenate user name and password bytes and encode them
	byte[] concat = new byte[nameBytes.length + passwdBytes.length];
	System.arraycopy(nameBytes, 0, concat, 0, nameBytes.length);
	System.arraycopy(passwdBytes, 0, concat, nameBytes.length,
			 passwdBytes.length);
	this.auth = "Basic " + (new sun.misc.BASE64Encoder()).encode(concat);

	// clear the password information
	java.util.Arrays.fill(passwd, ' ');
	java.util.Arrays.fill(passwdBytes, (byte)0x00);
	java.util.Arrays.fill(concat, (byte)0x00);
    }

    /**
     * Create a BasicAuthentication
     */
    public BasicAuthentication(boolean isProxy, String host, int port,
			       String realm, String auth) {
	super(isProxy ? PROXY_AUTHENTICATION : SERVER_AUTHENTICATION,
	      host, port, realm);
	this.auth = "Basic " + auth;
    }

    /**
     * Create a BasicAuthentication
     */
    public BasicAuthentication(boolean isProxy, URL url, String realm,
				   PasswordAuthentication pw) {
	super(isProxy ? PROXY_AUTHENTICATION : SERVER_AUTHENTICATION,
	      url, realm);
	String plain = pw.getUserName() + ":";
	byte[] nameBytes = plain.getBytes();

	// get password bytes
	char[] passwd = pw.getPassword();
	byte[] passwdBytes = new byte[passwd.length];
	for (int i=0; i<passwd.length; i++)
	    passwdBytes[i] = (byte)passwd[i];

	// concatenate user name and password bytes and encode them
	byte[] concat = new byte[nameBytes.length + passwdBytes.length];
	System.arraycopy(nameBytes, 0, concat, 0, nameBytes.length);
	System.arraycopy(passwdBytes, 0, concat, nameBytes.length,
			 passwdBytes.length);
	this.auth = "Basic " + (new sun.misc.BASE64Encoder()).encode(concat);

	// clear the password information
	java.util.Arrays.fill(passwd, ' ');
	java.util.Arrays.fill(passwdBytes, (byte)0x00);
	java.util.Arrays.fill(concat, (byte)0x00);
    }

    /**
     * Create a BasicAuthentication
     */
    public BasicAuthentication(boolean isProxy, URL url, String realm,
				   String auth) {
	super(isProxy ? PROXY_AUTHENTICATION : SERVER_AUTHENTICATION,
	      url, realm);
	this.auth = "Basic " + auth;
    }

    /**
     * @return true if this authentication supports preemptive authorization
     */
    boolean supportsPreemptiveAuthorization() {
	return true;
    }

    /**
     * @return the name of the HTTP header this authentication wants set
     */
    String getHeaderName() {
	if (type == SERVER_AUTHENTICATION) {
	    return "Authorization";
	} else {
	    return "Proxy-authorization";
	}
    }

    /**
     * @return the value of the HTTP header this authentication wants set
     */
    String getHeaderValue() {
	return auth;
    }


}
