/*
 * @(#)ConnectionOutputStream.java	1.10 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.rmi.transport;

import java.io.*;
import java.util.Vector;
import java.rmi.server.UID;
import sun.rmi.server.MarshalOutputStream;

/**
 * Special stream to keep track of refs being marshaled as return
 * results to determine whether a special ack needs to be sent
 * to the distributed collector.
 *
 * @author Ann Wollrath
 */
class ConnectionOutputStream extends MarshalOutputStream {

    /** indicates whether output stream is used to marshal results */
    private boolean resultStream = false;
    /** list to keep objects from potentially being gc'd during hand-off */
    private Vector objList = null;
    /** identifier for gc ack*/
    private UID ackID = new UID();
    /** connection associated with ConnectionOutputStream */
    private Connection conn = null;

    /**
     * Constructs a marshal output stream using the underlying
     * stream "out".
     */
    ConnectionOutputStream(OutputStream out)
	throws IOException
    {	      
	super(out);
    }


    /**
     * Constructs an marshal output stream using the underlying
     * stream associated with the connection, the parameter c.
     * @param c is the Connection object associated with the 
     * ConnectionOutputStream object being constructed
     * @param resultStream indicates whether this stream is used
     * to marshal return results
     */
    ConnectionOutputStream(Connection c, boolean resultStream)
	throws IOException
    {	      
	super(c.getOutputStream());
	this.conn         = c;
	this.resultStream = resultStream;
    }

    void writeID()  throws IOException
    {
	ackID.write(this);
    }
    
    /**
     * Returns true if this output stream is used to marshal return
     * results; otherwise returns false.
     */
    boolean isResultStream() {
	return resultStream;
    }
    
    /**
     * Add reference to object list (only needs to be done if this
     * stream is an output stream for marshaling return results).
     * This hook is used by the distributed collector.
     */
    void saveRef(LiveRef ref) {
	if (resultStream) {
	  if (objList == null) {
		objList = new Vector();
		new DGCAckHandler(conn.getChannel(), ackID, objList);
          }
	    objList.addElement(ref);
	}
    }

} //end of class ConnectionOutputStream
