/*
 * @(#)DGCAckHandler.java	1.9 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.rmi.transport;

import java.util.*;
import java.rmi.server.UID;

/**
 * The DGCAckHandler keeps track of and handles acknowledgments
 * from the distributed garbage collector.
 *
 * @author Ann Wollrath
 */
public class DGCAckHandler implements Notifiable {

    /** table mapping ack ID to remote object list*/
    private static Hashtable objLists = new Hashtable();

    private Channel ch;
    private UID id;
    private Vector list;

    /**
     * Construct (and register) a handler for the expected
     * acknowledgment from the distributed collector.  Remember the
     */
    public DGCAckHandler(Channel channel, UID ackID, Vector objList) 
    {
	ch = channel;
	id = ackID;
	list = objList;
	objLists.put(id, this);
	ch.addNotifiable(channel.getEndpoint(), this);
    }

    /**
     * Informs handler that an acknowledgment is received. Forget
     * about list of remote objects.
     */
    public static void received(UID id) 
    {
	DGCAckHandler entry = (DGCAckHandler)objLists.remove(id);
	if (entry != null) {
	    entry.removeNotifiable();
	}
    }

    /**
     * Private method to remove notification request
     */
    private void removeNotifiable()
    {
	ch.removeNotifiable(ch.getEndpoint(), this);
    }

    /**
     * VM sending ack is inaccessible. Forget about it.
     */
    public void notify(Endpoint ep, int state)
    {
	objLists.remove(id);
    }
}
