/*
 * @(#)GeneralSubtrees.java	1.8 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.security.x509;

import java.io.*;
import java.util.Vector;

import sun.security.util.*;

/**
 * Represent the GeneralSubtrees ASN.1 object.
 *
 * @version 1.8
 *
 * @author Amit Kapoor
 * @author Hemma Prafullchandra
 */
public class GeneralSubtrees {
    private Vector	trees;

    /**
     * The default constructor for the class.
     *
     * @params trees the sequence of GeneralSubtree.
     */
    public GeneralSubtrees(Vector trees) {
        this.trees = trees;
    }

    /**
     * Create the object from the passed DER encoded form.
     *
     * @param val the DER encoded form of the same.
     */
    public GeneralSubtrees(DerValue val) throws IOException {
        trees = new Vector(1, 1);
        if (val.tag != DerValue.tag_Sequence) {
            throw new IOException("Invalid encoding of GeneralSubtrees.");
        }
        while (val.data.available() != 0) {
            DerValue opt = val.data.getDerValue();
            GeneralSubtree tree = new GeneralSubtree(opt);
            trees.addElement(tree);
        }
    }

    /**
     * Return a printable string of the GeneralSubtree.
     */
    public String toString() {
        String s = "   GeneralSubtrees:\n" + trees.toString()
                   + "\n";

        return (s);
    }

    /**
     * Encode the GeneralSubtrees.
     *
     * @params out the DerOutputStrean to encode this object to.
     */
    public void encode(DerOutputStream out) throws IOException {
        DerOutputStream seq = new DerOutputStream();

        for (int i = 0; i < trees.size(); i++) {
            ((GeneralSubtree)trees.elementAt(i)).encode(seq);
        }
        out.write(DerValue.tag_Sequence,seq);
    }
}
