/*
 * @(#)IPAddressName.java	1.8 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.security.x509;

import java.io.*;
import sun.security.util.*;

/**
 * This class implements the IPAddressName as required by the GeneralNames
 * ASN.1 object.
 *
 * @see GeneralName
 * @see GeneralNameInterface
 * @see GeneralNames
 *
 * @version 1.8
 *
 * @author Amit Kapoor
 * @author Hemma Prafullchandra
 */
public class IPAddressName implements GeneralNameInterface {
    private byte[] address;

    /**
     * Create the IPAddressName object from the passed encoded Der value.
     *
     * @params derValue the encoded DER IPAddressName.
     * @exception IOException on error.
     */
    public IPAddressName(DerValue derValue) throws IOException {
        address = derValue.getOctetString();
    }

    /**
     * Create the IPAddressName object with the specified name.
     *
     * @params name the IPAddressName.
     */
    public IPAddressName(byte[] address) {
        this.address = address;
    }

    /**
     * Return the type of the GeneralName.
     */
    public int getType() {
        return (GeneralNameInterface.NAME_IP);
    }

    /**
     * Encode the IPAddress name into the DerOutputStream.
     *
     * @params out the DER stream to encode the IPAddressName to.
     * @exception IOException on encoding errors.
     */
    public void encode(DerOutputStream out) throws IOException {
        out.putOctetString(address);
    }

    /**
     * Return a printable string of IPaddress
     */
    public String toString() {
        return ("IPAddress: " + (address[0] & 0xff) + "."
                 + (address[1] & 0xff) + "."
                 + (address[2] & 0xff) + "."
                 + (address[3] & 0xff));
    }

    /**
     * Returns this IPAddress name.
     */
    public byte[] getBytes() {
        return (byte[])address.clone();
    }

    /**
     * Compares this name with another, for equality.
     *
     * @return true iff the names are identical.
     * 
     */
    public boolean equals(Object obj) {
	if (this == obj)
	    return true;

	if (!(obj instanceof IPAddressName))
	    return false;

	IPAddressName other = (IPAddressName)obj;

	return java.util.Arrays.equals(address, other.address);
    }

    /**
     * Returns the hash code value for this object.
     * 
     * @return a hash code value for this object.
     */
    public int hashCode() {
	int retval = 0;

	for (int i=0; i<address.length; i++)
	    retval += address[i] * i;

	return retval;
    }
}
