/*
 * @(#)CachedField.java	1.7 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.tools.agent;

/**
 *   The Cached* classes cache data retrieved from JVMDI.  
 *   This class provides access to field information.
 *   @author Robert Field
 */

class CachedField {
    final long fieldID;
    final CachedClass cachedClass;
    final String name;
    final String signature;

    static protected native long getFieldID(Class clazz, String name, String signature);
    static protected native long getStaticFieldID(Class clazz, String name, String signature);
    static protected native String getName(Class clazz, long fieldID);
    static protected native String getSignature(Class clazz, long fieldID);

    static protected native Class getDeclaringClass(Class clazz, long fieldID);
    static protected native int getModifiers(Class clazz, long fieldID);

    private CachedField(CachedClass cls, long fieldID, String name, String signature) {
	this.fieldID = fieldID;
        this.cachedClass = cls;
        this.name = name;
        this.signature = signature;
    }

    protected static long getStaticOrInstanceFieldID(CachedClass cls, 
                                         String name, String signature) {
        long fieldID;
        try {
            fieldID = getFieldID(cls.getRealClass(), name, signature);
        } catch(NoSuchMethodError exc) {
            fieldID = getStaticFieldID(cls.getRealClass(), name, signature);
        }
        return fieldID;
    }

    static CachedField getField(CachedClass cls, long fieldID) {
        CachedField field = cls.findField(fieldID);
        if (field == null) {
            String name = getName(cls.getRealClass(), fieldID);
            String signature = getSignature(cls.getRealClass(), fieldID);
            field = new CachedField(cls, fieldID, name, signature);
        }
        return field;
    }

    static CachedField getField(CachedClass cls, String name, 
                                        String signature) {
	long fieldID = getStaticOrInstanceFieldID(cls, name, signature);
        CachedField field = cls.findField(fieldID);
        if (field == null) {
            field = new CachedField(cls, fieldID, name, signature);
        }
        return field;
    }

    String getName() {
	return name;
    }

    public String getTypeSignature() {
	return signature;
    }

    CachedClass getDefiningClass() throws AbsentInformationException {
        Class clazz = getDeclaringClass(getRealClass(), fieldID);
        return CachedClass.getCachedClass(clazz);
    }

    int getModifiers() throws AbsentInformationException {
	return getModifiers(getRealClass(), fieldID);
    }

    Class getRealClass() {
        return cachedClass.getRealClass();
    }
}
