/*
 * @(#)FieldConstantData.java	1.15 00/02/02
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.tools.asm;

import sun.tools.java.*;
import java.io.IOException;
import java.io.DataOutputStream;

/**
 * This is a field constant pool data item
 */
final
class FieldConstantData extends ConstantPoolData {
    MemberDefinition field;
    NameAndTypeData nt;

    /**
     * Constructor
     */
    FieldConstantData(ConstantPool tab, MemberDefinition field) {
	this.field = field;
	nt = new NameAndTypeData(field);
	tab.put(field.getClassDeclaration());
	tab.put(nt);
    }

    /**
     * Write the constant to the output stream
     */
    void write(Environment env, DataOutputStream out, ConstantPool tab) throws IOException {
	if (field.isMethod()) {
	    if (field.getClassDefinition().isInterface()) {
		out.writeByte(CONSTANT_INTERFACEMETHOD);
	    } else {
		out.writeByte(CONSTANT_METHOD);
	    }
	} else {
	    out.writeByte(CONSTANT_FIELD);
	}
	out.writeShort(tab.index(field.getClassDeclaration()));
	out.writeShort(tab.index(nt));
    }

    /**
     * Return the order of the constant
     */
    int order() {
	return 2;
    }
}
