/*
 * @(#)DebuggerCallback.java	1.15 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.tools.debug;

/**
 * The DebuggerCallback interface is used to communicate asynchronous
 * information from the debugger to its client.  This may be the actual
 * client object, or a delegate of its choosing.
 */
public interface DebuggerCallback {

    /**
     * Print text to the debugger's console window.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    void printToConsole(String text) throws Exception;

    /**
     * A breakpoint has been hit in the specified thread.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    void breakpointEvent(RemoteThread t) throws Exception;

    /**
     * An exception has occurred.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    void exceptionEvent(RemoteThread t, String errorText) throws Exception;

    /**
     * A thread has died.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    void threadDeathEvent(RemoteThread t) throws Exception;

    /**
     * The client interpreter has exited, either by returning from its
     *  main thread, or by calling System.exit().
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    void quitEvent() throws Exception;
}
