/*
 * @(#)RemoteThreadGroup.java	1.14 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.tools.debug;

/**
 * The RemoteThreadGroup class allows access to a threadgroup in a
 * remote Java interpreter.  
 *
 * @see RemoteDebugger
 * @see RemoteThreadGroup
 * @author Thomas Ball
 */
public class RemoteThreadGroup extends RemoteObject {
    RemoteThreadGroup parent;
    String name;
    int maxPriority;
    boolean daemon;

    RemoteThreadGroup(RemoteAgent agent, int id, RemoteClass clazz) {
	super(agent, TP_THREADGROUP, id, clazz);
    }

    private void getThreadGroupInfo() throws Exception {
	if (name == null) {
	    agent.getThreadGroupInfo(this);
	}
    }

    /**
     * Return the threadgroup's name.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    public String getName() throws Exception {
	getThreadGroupInfo();
	return name;
    }

    /**
     * Stop the remote threadgroup.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    public void stop() throws Exception {
	agent.stopThreadGroup(id);
    }

    /**
     * List a threadgroup's threads.
     *
     * @exception java.lang.Exception if a general exception occurs.
     */
    public RemoteThread[] listThreads(boolean recurse) throws Exception {
	return agent.listThreads(this, recurse);
    }
}
