/*
 * @(#)Runner.java	1.11  99/11/07
 *
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.sound.midi.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Date;
import java.lang.reflect.*;
import java.io.File;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.PrintWriter;
import javax.sound.sampled.Clip;
import Juke;
import Groove;
import MidiSynth;
import CapturePlayback;


/**
 * Runner for automating the Java Sound demos.
 *
 * Assumptions :
 * 
 *    Juke - either a ../jfc/Java2D/media or a ./audio directory exists or
 *           a audio directory has been passed in on the command line.
 *
 *    CapturePlayback - the 22-new.aif file exists in either the 
 *           ../jfc/Java2D/media or ./audio directories, or else the directory
 *           in which the file resides must been passed in on the command 
 *           line.
 *
 *
 * @version @(#)Runner.java	1.11 99/11/07
 * @author Brian Lichtenwalter  
 */
public class Runner extends JPanel implements ActionListener, ChangeListener, Runnable {


    boolean STOP;
    boolean NOTIFY;
    int numRuns = 1;
    Hashtable modules = new Hashtable();
    Hashtable audio = new Hashtable();
    Thread thread;
    JTabbedPane tabPane;
    int index;
    JButton startStopB;
    JTextArea textarea;
    PrintWriter fileOut;


    public Runner(String args[]) {
        
        setLayout(new BorderLayout());

        loadModules();

        /**
         * Based on input args select appropriate tests.
         */
        boolean testsSelected = false;
        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("Run=")) { 
                String s = args[i].substring(args[i].indexOf('=')+1);
                numRuns = Integer.valueOf(s).intValue();
            } else if (args[i].startsWith("QuickLook")) { 
                testsSelected = true;
                for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
                    ModuleData moduleData = (ModuleData) e.nextElement();
                    moduleData.setSelected(true);
                    ((MethodData) moduleData.methods.get("Test_QuickLook")).setSelected(true);
                }
            } else if (args[i].indexOf(".") != -1) { 
                // Juke.Test_FooBar
                String s = args[i];
                testsSelected = true;
                String s1 = s.substring(0, s.indexOf('.'));
                ModuleData moduleData = (ModuleData) modules.get(s1);
                moduleData.setSelected(true);
                String s2 = s.substring(s.indexOf('.')+1, s.length());
                ((MethodData) moduleData.methods.get(s2)).setSelected(true);
            } else if (args[i].startsWith("Juke") || args[i].startsWith("Groove") || args[i].startsWith("MidiSynth") || args[i].startsWith("CapturePlayback")) {
                testsSelected = true;
                ((ModuleData) modules.get(args[i])).setMethodsSelected(true);
            } else if (args[i].startsWith("File")) {
                openOutputStream(args[i]);
            } else {
                File file = new File(args[i]);
                if (file != null && file.exists() && file.isDirectory()) {
                    JukeModule jm = (JukeModule) ((ModuleData) modules.get("Juke")).module;
                    loadAudio(args[i]);
                    jm.startB.setEnabled(true);
                    jm.loadJuke(args[i]); 
                    jm.jukeTable.tableChanged();
                }
            }
        }

        if (!testsSelected) {
            for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
                ((ModuleData) e.nextElement()).setMethodsSelected(true);
            }
        }

        if (fileOut == null) {
            openOutputStream("results.txt");
        }

       
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(new EmptyBorder(5,5,5,5));
        textarea = new JTextArea();
        textarea.setFont(new Font("TimesRoman", Font.PLAIN, 14));
        textarea.setForeground(Color.black);
        textarea.setBackground(Color.lightGray);
        textarea.setEditable(false);
        textarea.setAutoscrolls(true);
        JScrollPane scroller = new JScrollPane();
        scroller.getViewport().add(textarea);
        panel.add(scroller);

        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
                            new SelectionUI(), panel);
        splitPane.setContinuousLayout(true);
        add("Center", splitPane);

        panel = new JPanel(new BorderLayout());
        panel.add(startStopB = addButton("Start", panel));
        add("South", panel);

        String s = "\n\nJava Sound Runner\n" +
                   "java version: " +
                   System.getProperty("java.version") + "\n" +
                   System.getProperty("os.name") + " " +
                   System.getProperty("os.version") + "\n" +
                   new Date().toString() + "\n";
        print(s);
    }


    private JButton addButton(String name, JPanel p) {
        JButton b = new JButton(name);
        b.addActionListener(this);
        p.add(b);
        return b;
    }


    private void loadModules() {

        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = (JMenu) menuBar.add(new JMenu("File"));
        JMenuItem item = (JMenuItem) fileMenu.add(new JMenuItem("Exit"));
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { System.exit(0); }
        });

        tabPane = new JTabbedPane();
        tabPane.addChangeListener(this);

        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        BevelBorder bb = new BevelBorder(BevelBorder.LOWERED);
        CompoundBorder cb = new CompoundBorder(eb,bb);
        JPanel p = new JPanel(new BorderLayout());
        p.setBorder(new CompoundBorder(cb,new EmptyBorder(0,0,90,0)));
        JukeModule jm = new JukeModule(null);
        p.add(jm);
        modules.put("Juke", new ModuleData(jm, 0));
        tabPane.addTab("Juke Box", p);

        CapturePlaybackModule cpm = new CapturePlaybackModule();
        p = new JPanel(new BorderLayout());
        p.setBorder(new CompoundBorder(cb,new EmptyBorder(0,0,90,0)));
        modules.put("CapturePlayback", new ModuleData(cpm, 1));
        p.add(cpm);
        tabPane.addTab("Capture/Playback", p);

        MidiSynthModule msm = new MidiSynthModule();
        modules.put("MidiSynth", new ModuleData(msm, 2));
        tabPane.addTab("Midi Synthesizer", msm);

        p = new JPanel(new BorderLayout());
        p.setBorder(new CompoundBorder(cb,new EmptyBorder(0,0,5,20)));
        GrooveModule gm = new GrooveModule();
        p.add(gm);
        modules.put("Groove", new ModuleData(gm, 3));
        tabPane.addTab("Groove Box", p);


        JFrame f = new JFrame("Java Sound Demo");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
        f.getContentPane().add("North", menuBar);
        f.getContentPane().add("Center", tabPane);
        int width = 760, height = 500;
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        f.pack();
        f.setLocation(d.width/2 - width/2, d.height/2 - height/2);
        f.setSize(width, height);
        f.setVisible(true);
    }


    private void addSound(File file) {
        String s = file.getName();
        if (s.endsWith(".au") || s.endsWith(".rmf") ||
            s.endsWith(".mid") || s.endsWith(".wav") ||
            s.endsWith(".aif") || s.endsWith(".aiff"))
        {
            audio.put(s, file);
        }
    }


    public void loadAudio(String name) {
        File file = new File(name);
        if (file != null && file.isDirectory()) {
            String files[] = file.list();
            for (int i = 0; i < files.length; i++) {
                File leafFile = new File(file.getAbsolutePath(), files[i]);
                if (leafFile.isDirectory()) {
                    loadAudio(leafFile.getAbsolutePath());
                } else {
                    addSound(leafFile);
                }
            }
        } else if (file != null && file.exists()) {
            addSound(file);
        }
    }

 
    private void openOutputStream(String name) {
        try {
            File file = new File(name);
            if (file.canWrite()) {
                FileOutputStream fos = new FileOutputStream(file);
                fileOut = new PrintWriter(fos);
            }
        } catch (Exception ex) {
            System.out.println("Error creating results file " + ex.toString());
            return;
        }
    }


    public void print(String s) {
        System.out.println(s);
        textarea.append(s + "\n");
        int y = textarea.getPreferredSize().height;
        textarea.scrollRectToVisible(new Rectangle(0,y,1,1));
        if (fileOut != null) {
            fileOut.print(s);
            fileOut.flush();
        }
    }


    public void actionPerformed(ActionEvent e) {
        JButton button = (JButton) e.getSource();
        if (button.equals(startStopB)) {
            if (button.getText().equals("Start")) {
                STOP = false;
                start();
                button.setText("Stop");
            } else if (button.getText().equals("Stop")) {
                STOP = true;
                stop();
                button.setText("Start");
                print("Finished!\n");
            }
        }
    }


    public void stateChanged(ChangeEvent e) {
        close();
        System.gc();
        index = tabPane.getSelectedIndex();
        open();
        NOTIFY = true;
    }


    public void close() {
        for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
            ModuleData moduleData = (ModuleData) e.nextElement();
            if (moduleData.index == index) {
                ((ControlContext) moduleData.module).close();
                break;
            }
        }
    }


    public void open() {
        for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
            ModuleData moduleData = (ModuleData) e.nextElement();
            if (moduleData.index == index) {
                ((ControlContext) moduleData.module).open();
                break;
            }
        }
    }


    public void start() {
        thread = new Thread(this);
        thread.setName("Java Sound Runner");
        thread.start();
    }


    public void stop() {
        if (thread != null) {
            thread.interrupt();
        }
        thread = null;
    }


    public void run() {
        for (int i = 0; i < numRuns && !STOP; i++) {
            print("\nInitiating Run #" + i);
            for (Enumeration e = modules.elements(); e.hasMoreElements() && !STOP; ) {
                ModuleData moduleData = (ModuleData) e.nextElement();
                if (moduleData.isSelected()) {
                    print(moduleData.name);
                    if (moduleData.index != tabPane.getSelectedIndex()) {
                        NOTIFY = false;
                        tabPane.setSelectedIndex(moduleData.index);
                        while (!NOTIFY && !STOP) {
                           try { thread.sleep(99); } catch (Exception ex) { }
                        } 
                        try { thread.sleep(999); } catch (Exception ex) { }
                    } else {
                        ((ControlContext) moduleData.module).open();
                    }
                    for (Enumeration ee = moduleData.methods.elements(); ee.hasMoreElements() && !STOP; ) {
                        MethodData methodData = (MethodData) ee.nextElement();
                        if (methodData.isSelected()) {
                            String methodName = methodData.method.getName();
                            print("    " + methodName);
                            try {
                                methodData.method.invoke(moduleData.module, null);
                                // take a small break between tests 
                                thread.sleep(999);
                            } catch (Exception ex) { ex.printStackTrace(); }
                        }
                    }
                }
            }
        }
        if (thread != null) {
            startStopB.doClick();
        }
        if (fileOut != null) {
            fileOut.close();
        }
    }

 
    /**
     * Properties for our ModuleData and MethodData. 
     */
    public class DataProperties extends Object {

        String name;
        boolean isSelected;

        public DataProperties() {
        }

        public DataProperties(String name, boolean isSelected) {
            this.name = name;
            this.isSelected = isSelected;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public void setSelected(boolean isSelected) {
            this.isSelected = isSelected;
        }

        public boolean isSelected() {
           return isSelected;
        }
    }
    

    /**
     * A storage class for our modules.
     */
    class ModuleData extends DataProperties {

        Object module;
        Hashtable methods = new Hashtable();
        int index;

        public ModuleData(Object module, int index) {
            this.module = module;
            this.index = index;
            String s = module.getClass().getName();
            s = s.substring(s.indexOf('$')+1);
            setName(s.substring(0, s.indexOf("Module")));
            Method[] m = module.getClass().getDeclaredMethods();
            for (int i = 0; i < m.length; i++) {
                if (m[i].getName().startsWith("Test_")) {
                    methods.put(m[i].getName(), new MethodData(m[i]));
                }
            }
        }

        public void setMethodsSelected(boolean isSelected) {
            this.isSelected = isSelected;
            for (Enumeration e = methods.elements(); e.hasMoreElements(); ) {
                ((MethodData) e.nextElement()).setSelected(isSelected);
            }
        }
    }


    /**
     * A storage class for our module methods.
     */
    class MethodData extends DataProperties {

        Method method;

        public MethodData(Method method) {
            this.method = method;
            setName(method.getName());
        }
    }



    /**
     * UI for selecting tests to run.  
     */
    class SelectionUI extends JPanel {

        private JTree tree;

        public SelectionUI() {
    
            setLayout(new BorderLayout());
            setBorder(new EmptyBorder(5,5,5,5));
            setPreferredSize(new Dimension(180, 200));

            DataProperties dp = new DataProperties("tests", true);
            DefaultMutableTreeNode top = new DefaultMutableTreeNode(dp);

            for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
                ModuleData moduleData = (ModuleData) e.nextElement();
                DefaultMutableTreeNode node = new DefaultMutableTreeNode(moduleData);
                top.add(node);
                for (Enumeration ee = moduleData.methods.elements(); ee.hasMoreElements(); ) {
                    MethodData methodData = (MethodData) ee.nextElement();
                    node.add(new DefaultMutableTreeNode(methodData));
                }
            }

            tree = new JTree(top);
            tree.setBackground(Color.lightGray);
            tree.addMouseListener(ml);
            tree.setCellRenderer(new TreeRenderer());

            JScrollPane sp = new JScrollPane();
            sp.getViewport().add(tree);
            add(sp);
        }


        private void modifySelected(DefaultMutableTreeNode node, boolean b) {
            DataProperties dp = (DataProperties) node.getUserObject();
            dp.setSelected(b);
            if (!node.isLeaf()) {
                TreeModel tm = tree.getModel();
                int num = tm.getChildCount(node);
                for (int i = 0; i < num; i++) {
                    modifySelected((DefaultMutableTreeNode) tm.getChild(node, i), b);
                }
            }
        }


        MouseListener ml = new MouseAdapter(){
            public void mouseClicked(MouseEvent e){
                if (e.getClickCount() == 1) {
                    TreePath treepath = tree.getSelectionPath();
                    if (treepath == null) {
                        return;
                    }
                    Object[] os = treepath.getPath();
                    int x = e.getX();
                    int y = e.getY();
                    if ((os.length == 1 && y <= 20) ||
                           (os.length == 2 && x >= 20) ||
                              (os.length == 3 && x >= 40))
                    {
                        DefaultMutableTreeNode tn = 
                            (DefaultMutableTreeNode) treepath.getLastPathComponent();
                        DataProperties dp = (DataProperties) tn.getUserObject();
                        modifySelected(tn, !dp.isSelected());
                        tree.treeDidChange();
                    }
                }
            }
        };



        class TreeRenderer extends JCheckBox implements TreeCellRenderer {
    
            public Component getTreeCellRendererComponent(
                    JTree tree,
                    Object value,       
                    boolean selected,    
                    boolean expanded,  
                    boolean leaf,
                    int row,
                    boolean hasFocus)
            {
                setBackground(Color.lightGray);
                DefaultMutableTreeNode tn = (DefaultMutableTreeNode) value;
                DataProperties dp = (DataProperties) tn.getUserObject();
                setText(dp.getName());
                setSelected(dp.isSelected());
                return this;
            }
        }
    } // End class SelectionUI



    /**
     * Runner for Juke demo.
     */
    class JukeModule extends Juke {

        public JukeModule(String dirName) {
            super(dirName);
        }

        // play all listed files once for 4 second duration
        public void Test_QuickLook() {
            if (loopB.isSelected()) {
                loopB.doClick();  
            }
            if (sounds.size() == 0) {
                // look for audio files in two places
                loadJuke("../jfc/Java2D/media");
                loadJuke("audio");
                if (sounds.size() != 0) {
                    jukeTable.tableChanged();
                    startB.setEnabled(true);
                    loadAudio("../jfc/Java2D/media");
                    loadAudio("audio");
                    try { getThread().sleep(500); } catch (Exception e) { }
                }
            }
            if (sounds.size() != 0) {
                table.getSelectionModel().setSelectionInterval(0, 0);
                startB.doClick();
                do {
                    playForDuration(4);
                } while (getThread() != null && !STOP);
            } 
        }


        private void playForDuration(int duration) {
            // wait about 4 seconds for the sound to load
            for (int i = 0; i < 40 && currentSound == null && !STOP; i++) {
                try { getThread().sleep(99); } catch (Exception e) { break; }
            }
            try { getThread().sleep(99); } catch (Exception e) { }
            if (currentSound instanceof Sequence || currentSound instanceof BufferedInputStream && getThread() != null) {
                for (int i = 0; !midiEOM && getThread() != null && !bump && !STOP && i < duration; i++) {
                    for (int j = 0; !midiEOM && getThread() != null && !bump && !STOP && j < 10; j++) {
                        try { getThread().sleep(90); } catch (Exception e) {break;}
                    }
                }
            } else if (currentSound instanceof Clip && getThread() != null) {
                Clip clip = (Clip) currentSound;
                for (int i = 0; (paused || clip.isActive()) && getThread() != null && !STOP && !bump && i < duration; i++) {
                    for (int j = 0; (paused || clip.isActive()) && getThread() != null && !STOP && !bump && j < 10; j++) {
                        try { getThread().sleep(90); } catch (Exception e) {break;}
                    }
                }
            }
            bump = true;
            // wait for currentSound to equal null or wait for two seconds
            for (int i = 0; i < 20 && currentSound != null && !STOP; i++) {
                try { getThread().sleep(99); } catch (Exception e) { break; }
            }
            bump = false;
        }
    }


    /**
     * Runner for CapturePlayback demo.
     */
    class CapturePlaybackModule extends CapturePlayback {

        public CapturePlaybackModule() {
            super();
        }

        // capture & playback 5 seconds of audio at default format
        public void Test_QuickLook() {
            captB.doClick();
            try { thread.sleep(4000); } catch (Exception ex) { return; }
            captB.doClick();
            // wait for audioInputStream 
            for (int i = 0; i < 60 && audioInputStream == null && !STOP; i++) {
                try { thread.sleep(99); } catch (Exception e) { break; }
            }
            // wait for waveform to paint
            try { thread.sleep(2000); } catch (Exception e) { }
            playB.doClick();
            try { thread.sleep(5000); } catch (Exception ex) { }
        }

        // load a file & playback & save & load saved file & playback
        public void Test_LoadSaveFile() {
            if (audio.size() == 0) {
                loadAudio("../jfc/Java2D/media");
                loadAudio("audio");
            }
            File file = (File) audio.get("22-new.aif");
            if (file != null && file.exists()) {
                createAudioInputStream(file, true);
                try { thread.sleep(2000); } catch (Exception ex) { return; }
                playB.doClick();
                try { thread.sleep(6000); } catch (Exception ex) { return; }
		String path = System.getProperty("user.home");
		String sep  = String.valueOf(System.getProperty("file.separator").toCharArray()[0]);
		String fullpath = (path + sep + "untitled.au");
                textField.setText( fullpath );
                auB.doClick();
                try { thread.sleep(2000); } catch (Exception ex) { return; }
                file = new File( fullpath );
                if (file != null && file.exists()) {
                    createAudioInputStream(file, true);
                }
                playB.doClick();
                try { thread.sleep(6000); } catch (Exception ex) { }
            }
        }
    }



    /**
     * Runner for MidiSynth demo.
     */
    class MidiSynthModule extends MidiSynth {

        public MidiSynthModule() {
            super();
        }

        // Play all the white keys for one instrument
        public void Test_QuickLook() {
            playWhiteKeys();
        }


        // Record Some Midi & Save & then play thru Juke
        public void Test_RecordSave() {
            controls.recordB.doClick();
            try { thread.sleep(1000); } catch (Exception ex) { return; } 
            recordFrame.recordB.doClick();
            playWhiteKeys();
            recordFrame.recordB.doClick();
            try { thread.sleep(1000); } catch (Exception ex) { return; } 
            recordFrame.playB.doClick();
            try { thread.sleep(5000); } catch (Exception ex) { return; } 
	    String path = System.getProperty("user.home");
            File file = new File( path, "test.mid" );
            recordFrame.saveMidiFile(file);
            try { thread.sleep(1000); } catch (Exception ex) { return; } 
            
            ModuleData md = (ModuleData) modules.get("Juke");
            JukeModule jm = (JukeModule) md.module;
            tabPane.setSelectedIndex(md.index); 
            try { thread.sleep(2000); } catch (Exception ex) { return; } 
            if (jm.sounds.size() == 0) {
                jm.startB.setEnabled(true);
            }
            jm.loadJuke("test.mid"); 
            jm.jukeTable.tableChanged();
            int row = jm.sounds.size() - 1;
            jm.table.getSelectionModel().setSelectionInterval(row, row);
            if (jm.loopB.isSelected()) {
                jm.loopB.doClick();
            }
            jm.startB.doClick();
            try { thread.sleep(5000); } catch (Exception ex) { return; } 
            int i =  ((ModuleData) modules.get("MidiSynth")).index;
            tabPane.setSelectedIndex(i);     
            try { thread.sleep(2000); } catch (Exception ex) { } 
        }


        // Play all the white keys for all the instruments
        public void Test_AllInstruments() {
            ListSelectionModel r = table.getSelectionModel();
            ListSelectionModel c = table.getColumnModel().getSelectionModel();
            for (int x = 0; x < table.getColumnCount() && !STOP; x++) {
                c.setSelectionInterval(0, x);
                for (int y = 0; y < table.getRowCount() && !STOP; y++) {
                    r.setSelectionInterval(x, y);
                    playWhiteKeys();
                }
            }
        }


        private void playWhiteKeys() {
            for (int j = 0; j < whiteKeys.size() && !STOP; j++) {
                Key key = (Key) whiteKeys.get(j); 
                piano.mousePressed(new MouseEvent(piano, MouseEvent.MOUSE_PRESSED, 0, 0, key.x+1, key.y+70, 1, false));
                try {
                    thread.sleep(100);
                } catch (InterruptedException e) { STOP = true; break; }
                piano.mouseReleased(new MouseEvent(piano, MouseEvent.MOUSE_RELEASED, 0, 0, key.x+1, key.y+70, 1, false));
            }
        }
    }


    /**
     * Runner for Groove demo.
     */
    class GrooveModule extends Groove {

        public GrooveModule() {
            super();
        }

        // play one beat at the default tempo
        public void Test_QuickLook() {
            tempoDial.setTempo(120);
            startB.doClick();
            try { thread.sleep(3000); } catch (Exception ex) { }
        }

        // iterate through tempo values
        public void Test_Tempo() {
            if (!loopB.isSelected()) {
                loopB.doClick();
            }
            startB.doClick();
            for (int tempo = 40; tempo < 180 && !STOP; tempo+=10) {
                tempoDial.setTempo((float) tempo);
                try { thread.sleep(2000); } catch (Exception ex) { break; }
            }
            startB.doClick();
            loopB.doClick();
        }

        // iterate through rock beats
        public void Test_Beats() {
            if (!loopB.isSelected()) {
                loopB.doClick();
            }
            startB.doClick();
            for (int i = 0; i < combo.getItemCount() && !STOP; i++) {
                combo.setSelectedIndex(i);
                try { thread.sleep(9000); } catch (Exception ex) { break; }
            }
            startB.doClick();
            loopB.doClick();
        }
    }



    public static void main(String args[]) {

        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("-h") || args[i].startsWith("-help")) {
                String s = "Run through all the tests once : \n" +
                   "    java -cp JavaSound.jar Runner Run \n" +
                   "\n" +
                   "Run through all the tests twenty times : \n" +
                   "    java -cp JavaSound.jar Runner Run=20 \n" +
                   "\n" +
                   "Run through all the QuickLook tests : \n" +
                   "    java -cp JavaSound.jar Runner QuickLook \n" +
                   "\n" +
                   "Run through all the Juke tests : \n" +
                   "    java -cp JavaSound.jar Runner Juke \n" +
                   "\n" +
                   "Run through one of the Juke tests : \n" +
                   "    java -cp JavaSound.jar Runner Juke.Test_QuickLook \n" +
                   "\n" +
                   "Pass several directories to Runner : \n" +
                   "    java -cp JavaSound.jar Runner audio1 audio2 " +
                   "\n" +
                   "Run with a named file : \n" +
                   "    java -cp JavaSound.jar Runner File=JDK13-A.results \n";
                System.out.println("\n" + s);
                System.exit(0);
            }
        }

        final Runner runner = new Runner(args);
        JFrame f = new JFrame("Java Sound Runner");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
        f.getContentPane().add("Center", runner);
        f.pack();
        f.setLocation(0, 0);
        f.setSize(new Dimension(420, 300));
        f.setVisible(true);

        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("Run")) {
                runner.startStopB.doClick();
            }
        }
    }
} 
