package com.sun.tools.javac.v8.code;
import com.sun.tools.javac.v8.util.*;

import com.sun.tools.javac.v8.code.Type.*;

import com.sun.tools.javac.v8.code.Symbol.*;


/**
 * This class contains some minor modifications wrt ClassReader
 *  which read complete constant pool and code sections with the class that is read in.
 *  Usually, these sections are omitted when reading class files.
 */
public class CompleteClassReader extends ClassReader {

    public CompleteClassReader(Hashtable options) {
        super(options);
    }

    /**
      * As in ClassReader except that all of constant pool is immediately read in.
      */
    void indexPool() {
        super.indexPool();
        for (int i = 1; i < poolObj.length; i++)
            readPool(i);
    }

    /**
      * As in ClassReader except that constant pool is stored in class `c'.
      */
    void readClass(ClassSymbol c) {
        c.pool = new Pool(poolObj.length, poolObj);
        super.readClass(c);
    }

    /**
      * As in ClassReader except that code is read in and entered
      *  into method symbol.
      */
    void readMemberAttr(Symbol sym, Name attrName, int attrLen) {
        if (attrName == Names.Code) {
            ((MethodSymbol) sym).code = readCode(sym);
        } else {
            super.readMemberAttr(sym, attrName, attrLen);
        }
    }

    /**
      * Read code block.
      */
    Code readCode(Symbol owner) {
        Code code = new Code(false, true, true);
        code.max_stack = nextChar();
        code.max_locals = nextChar();
        code.cp = nextInt();
        code.code = new byte[code.cp];
        System.arraycopy(buf, bp, code.code, 0, code.cp);
        bp += code.cp;
        char ncatches = nextChar();
        for (int i = 0; i < ncatches; i++) {
            code.addCatch(nextChar(), nextChar(), nextChar(), nextChar());
        }
        char ac = nextChar();
        for (int i = 0; i < ac; i++) {
            Name attrName = readName(nextChar());
            int attrLen = nextInt();
            if (attrName == Names.LineNumberTable) {
                char nlines = nextChar();
                for (int j = 0; j < nlines; j++) {
                    code.addLineNumber(nextChar(), nextChar());
                }
            } else if (attrName == Names.LocalVariableTable) {
                code.nvars = nextChar();
                code.lvar_start_pc = new char[code.nvars];
                code.lvar_length = new char[code.nvars];
                code.lvar_reg = new char[code.nvars];
                code.lvar = new VarSymbol[code.nvars];
                for (int j = 0; j < code.nvars; j++) {
                    code.lvar_start_pc[j] = nextChar();
                    code.lvar_length[j] = nextChar();
                    code.lvar[j] = new VarSymbol(0, readName(nextChar()),
                            readType(nextChar()), owner);
                    code.lvar[j].adr = code.lvar_reg[j] = nextChar();
                }
            } else {
                unrecogized(attrName);
                bp = bp + attrLen;
            }
        }
        return code;
    }
}
