package com.sun.tools.javac.v8.code;

/**
 * Access flags and other modifiers for Java classes and members.
 */
public interface Flags {
    int PUBLIC = 1;
    int PRIVATE = 2;
    int PROTECTED = 4;
    int STATIC = 8;
    int FINAL = 16;
    int SYNCHRONIZED = 32;
    int VOLATILE = 64;
    int TRANSIENT = 128;
    int NATIVE = 256;
    int INTERFACE = 512;
    int ABSTRACT = 1024;
    int STRICTFP = 2048;
    int StandardFlags = 4095;
    int ACC_SUPER = SYNCHRONIZED;

    /**
     * Flag is set if symbol has a synthetic attribute.
     */
    int SYNTHETIC = 65536;

    /**
     * Flag is set if symbol is deprecated.
     */
    int DEPRECATED = 131072;

    /**
     * Flag is set for a variable symbol if the variable's definition
     *  has an initializer part.
     */
    int HASINIT = 262144;

    /**
     * Flag is set for a local variable symbol if the variable is accessed
     *  from an inner class.
     */
    int CAPTURED = 524288;

    /**
     * Flag is set for compiler-generated anonymous method symbols that `own' an
     *  initializer block.
     */
    int BLOCK = 1048576;

    /**
     * Flag is set for compiler-generated abstract methods that implement
     *  an interface method (Miranda methods).
     */
    int IPROXY = 2097152;

    /**
     * Flag is set for nested classes that do not access instance members or `this'
     *  of an outer class and therefore don't need to be passed a this$n reference.
     *  This flag is currently set only for anonymous classes in superclass
     *  constructor calls.
     *  todo: use this flag for optimizing away this$n parameters in other cases.
     */
    int NOOUTERTHIS = 4194304;

    /**
     * Flag is set for package symbols if a package has a directory
     *  and therefore exists.
     */
    int HASDIRECTORY = 8388608;

    /**
     * Flag is set for class symbols if a class file was found for this class.
     */
    int CLASS_SEEN = 33554432;

    /**
     * Flag is set for class symbols if a source file was found for this class.
     */
    int SOURCE_SEEN = 67108864;

    /**
     * Flag for class symbols is set and later re-set as a lock in Enter
     *  to detect cycles in the superclass/superinterface relations.
     */
    int LOCKED = 268435456;

    /**
     * Flag for class symbols is set and later re-set to indicate that a class
     *  has been entered but has not yet been attributed.
     */
    int UNATTRIBUTED = 536870912;

    /**
     * Modifier masks.
     */
    static final int AccessFlags = PUBLIC | PROTECTED | PRIVATE;

    /**
     * Modifier masks.
     */
    static final int LocalClassFlags = FINAL | ABSTRACT | STRICTFP;

    /**
     * Modifier masks.
     */
    static final int MemberClassFlags = LocalClassFlags | INTERFACE | AccessFlags;

    /**
     * Modifier masks.
     */
    static final int ClassFlags = LocalClassFlags | INTERFACE | PUBLIC;

    /**
     * Modifier masks.
     */
    static final int LocalVarFlags = FINAL;

    /**
     * Modifier masks.
     */
    static final int InterfaceVarFlags = FINAL | STATIC | PUBLIC;

    /**
     * Modifier masks.
     */
    static final int VarFlags = AccessFlags | FINAL | STATIC | VOLATILE | TRANSIENT;

    /**
     * Modifier masks.
     */
    static final int ConstructorFlags = AccessFlags;

    /**
     * Modifier masks.
     */
    static final int InterfaceMethodFlags = ABSTRACT | PUBLIC;

    /**
     * Modifier masks.
     */
    static final int MethodFlags =
            AccessFlags | ABSTRACT | STATIC | NATIVE | SYNCHRONIZED | FINAL |
            STRICTFP;
}
