/**
 * Everything that goes into a constant pool of a classfile.
 */
package com.sun.tools.javac.v8.code;
import com.sun.tools.javac.v8.util.*;


/**
 * An internal structure that corresponds to the constant pool of a classfile.
 */
public class Pool {

    /**
     * Index of next constant to be entered.
     */
    int pp;

    /**
     * The initial pool buffer.
     */
    Object[] pool;

    /**
     * A hashtable containing all constants in the pool.
     */
    Hashtable indices;

    /**
     * Construct a pool with given number of elements and element array.
     */
    public Pool(int pp, Object[] pool) {
        super();
        this.pp = pp;
        this.pool = pool;
        this.indices = new Hashtable(pool.length);
        for (int i = 1; i < pp; i++) {
            if (pool[i] != null)
                indices.put(pool[i], new Integer(i));
        }
    }

    /**
      * Construct an empty pool.
      */
    public Pool() {
        this(1, new Object[64]);
    }

    /**
      * Remove everything from this pool.
      */
    public void reset() {
        pp = 1;
        indices.reset();
    }

    /**
      * Double pool buffer in size.
      */
    private void doublePool() {
        Object[] newpool = new Object[pool.length * 2];
        System.arraycopy(pool, 0, newpool, 0, pool.length);
        pool = newpool;
    }

    /**
      * Place an object in the pool, unless it is already there.
      *  If object is a symbol also enter its owner unless the owner is a package.
      *  Return the object's index in the pool.
      */
    public int put(Object value) {
        Integer index = (Integer) indices.get(value);
        if (index == null) {
            index = new Integer(pp);
            indices.put(value, index);
            if (pp == pool.length)
                doublePool();
            pool[pp++] = value;
            if (value instanceof Long || value instanceof Double) {
                if (pp == pool.length)
                    doublePool();
                pool[pp++] = null;
            }
        }
        return index.intValue();
    }

    /**
      * Return the given object's index in the pool,
      *  or -1 if object is not in there.
      */
    public int get(Object o) {
        Integer n = (Integer) indices.get(o);
        return n == null ? -1 : n.intValue();
    }
}
