package com.sun.tools.javac.v8.util;

/**
 * A class for constructing lists by appending elements. Modelled after
 *  java.lang.StringBuffer.
 */
public class ListBuffer {

    /**
     * The list of elements of this buffer.
     */
    public List elems;

    /**
     * A pointer pointing to the last, sentinel element of `elems'.
     */
    public List last;

    /**
     * The number of element in this buffer.
     */
    public int count;

    /**
     * Has a list been created from this buffer yet?
     */
    public boolean shared;

    /**
     * Create a new initially empty list buffer.
     */
    public ListBuffer() {
        super();
        this.elems = new List();
        this.last = this.elems;
        count = 0;
        shared = false;
    }

    /**
      * Return the number of elements in this buffer.
      */
    public int length() {
        return count;
    }

    /**
      * Is buffer emepty?
      */
    public boolean isEmpty() {
        return count == 0;
    }

    /**
      * Is buffer not empty?
      */
    public boolean nonEmpty() {
        return count != 0;
    }

    /**
      * Copy list and sets last.
      */
    private void copy() {
        List xs = elems;
        elems = new List();
        last = elems;
        while (xs.nonEmpty()) {
            last.head = xs.head;
            last.tail = new List();
            last = last.tail;
            xs = xs.tail;
        }
        shared = false;
    }

    /**
      * Append an element to buffer.
      */
    public ListBuffer append(Object x) {
        if (shared)
            copy();
        last.head = x;
        last.tail = new List();
        last = last.tail;
        count++;
        return this;
    }

    /**
      * Append all elements in a list to buffer.
      */
    public ListBuffer appendList(List xs) {
        while (xs.nonEmpty()) {
            append(xs.head);
            xs = xs.tail;
        }
        return this;
    }

    /**
      * Append all elements in an array to buffer.
      */
    public ListBuffer appendArray(Object[] xs) {
        for (int i = 0; i < xs.length; i++) {
            append(xs[i]);
        }
        return this;
    }

    /**
      * Convert buffer to a list of all its elements.
      */
    public List toList() {
        shared = true;
        return elems;
    }

    /**
      * The first element in this buffer.
      */
    public Object first() {
        return elems.head;
    }

    /**
      * Remove the first element in this buffer.
      */
    public void remove() {
        if (elems != last) {
            elems = elems.tail;
            count--;
        }
    }

    /**
      * An enumeration of all elements in this buffer.
      */
    public Enumeration elements() {
        return new Enumerator(elems, last);
    }

    static class Enumerator implements Enumeration {
        List elems;
        List last;

        Enumerator(List elems, List last) {
            super();
            this.elems = elems;
            this.last = last;
        }

        public boolean hasMoreElements() {
            return elems != last;
        }

        public Object nextElement() {
            Object elem = elems.head;
            elems = elems.tail;
            return elem;
        }
    }
}
