package com.sun.tools.javac.v8.util;

/**
 * Class contains some flags and methods of global use.
 */
public class Util {

    public Util() {
        super();
    }

    /**
      * Global flags.
      */
    public static final boolean checks = true;
    public static final boolean debug = false;
    public static final boolean statistics = false;

    /**
     * Assertion for internal error checking.
     *  If given condition is false, throw an InternalError.
     */
    public static void assert(boolean c) {
        if (!c)
            throw new InternalError("assertion failed");
    }

    /**
      * Assertion for internal error checking.
      *  If given condition is false, throw an InternalError with
      *  given `info' object as message.
      */
    public static void assert(boolean c, Object info) {
        if (!c)
            throw new InternalError("assertion failed: " + info);
    }

    /**
      * Does list contain an occurence of word?
      *  @param list    A string consisting of a list of words separated by
      *                 the given separator character.
      *  @param word    The word to be found in the list.
      *  @param sep     The character separating words in the list.
      */
    public static boolean contains(String list, String word, char sep) {
        int i = 0;
        int j = pos(list, sep, i);
        while (j >= i && !list.substring(i, j).equals(word)) {
            i = j + 1;
            j = pos(list, sep, i);
        }
        return j >= i;
    }

    /**
      * Making String.indexOf behave rationally.
      */
    public static int pos(String s, char ch, int start) {
        int p = s.indexOf(ch, start);
        return p < 0 ? s.length() : p;
    }
}
