/*
 * @(#)ioserx.c	1.7 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/***************************** OLD NATIVE INTERFACE **********************************/

/* All C native method functions need to include StubPreamble.h,
 * located in java/include
 * see also interpreter.h, which has various utility functions and
 * macros (and is included by StubPreamble.h
 * see also javaString.h, which has Java-string manipulation functions.
 */
#include <StubPreamble.h>
#include "sys_api.h"
#include "tree.h"

#define OBJECT_PACKAGE "java/io/"
#define WRITEOBJECT "writeObject"
#define READOBJECT "readObject"
#define LOADCLASS "loadClass0"
#define WRITE_SIGNATURE "(L" OBJECT_PACKAGE "ObjectOutputStream;)V"
#define READ_SIGNATURE "(L" OBJECT_PACKAGE "ObjectInputStream;)V"
#define LOAD_SIGNATURE "(Ljava/lang/Object;Ljava/lang/Class;Ljava/lang/String;)Ljava/lang/Class;"

/* Native.h is our javah-generated header file for our Native object
 * native methods
 */
#include "com_sun_corba_se_internal_io_NativeHook.h"

static char *copyright[] = {
    "Licensed Materials - Property of IBM and Sun",
    "RMI-IIOP v1.0",
    "Copyright IBM Corp. 1998 1999  All Rights Reserved",
    "US Government Users Restricted Rights - Use, duplication or",
    "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.",
    "Copyright 1998-1999 Sun Microsystems, Inc. 901 San Antonio Road,",
    "Palo Alto, CA  94303, U.S.A.  All rights reserved."
};

/*
 * Find the method block for either the writeObject or readObject method
 * of the specified class.  If findwrite is non-zero the method block
 * for the write method is returned, otherwise the readObject method is
 * returned.  If neither is present it returns NULL.
 * The NameAndType values are cached for subsequent use.
 */
struct methodblock * findWriteReadObject(ExecEnv *ee, ClassClass *cb, int findwrite) {
    struct methodblock* methods = cbMethods(cb);
    int i;

    /* Search through the methods of this class */
    for (i = cbMethodsCount(cb) - 1; i >= 0; i--) {
	if (methods[i].fb.access & ACC_PRIVATE &&
	    !(methods[i].fb.access & ACC_STATIC))  {
	    if (((findwrite != 0) && 
			 (!strcmp("(Ljava/io/ObjectOutputStream;)V", methods[i].fb.signature)) &&
			 (!strcmp(WRITEOBJECT, methods[i].fb.name))
			 ) ||
		((findwrite == 0) && 
		 (!strcmp("(Ljava/io/ObjectInputStream;)V", methods[i].fb.signature)) &&
		 (!strcmp(READOBJECT, methods[i].fb.name)))
			) {
		return &methods[i];
	    }
	}
    }

    /* If neither is found, return NULL */
    return NULL;
}

extern void com_sun_corba_se_internal_io_NativeHook_readObject(
	struct Hcom_sun_corba_se_internal_io_NativeHook * this,
	struct Hjava_lang_Object * target,
	struct Hjava_lang_Class * targetType,
	struct Hjava_io_ObjectInputStream * stream)
{
    struct methodblock* method;
    ExecEnv *ee = EE();

    method = findWriteReadObject(ee, targetType, 0);
    if (method == NULL || exceptionOccurred(ee))
	return;

    do_execute_java_method(ee,
			   /* object */target,
			   /* method name */  READOBJECT,
			   /* method signature */  READ_SIGNATURE,
			   /* method block of  */ method,
			   /* isStaticCall */ 0,
			   /* argument */ stream, NULL);

}



extern void com_sun_corba_se_internal_io_NativeHook_writeObject(
	struct Hcom_sun_corba_se_internal_io_NativeHook * this,
	struct Hjava_lang_Object * target,
	struct Hjava_lang_Class * targetType,
	struct Hjava_io_ObjectOutputStream * stream)
{
    struct methodblock* method;
    ExecEnv *ee = EE();

    method = findWriteReadObject(ee, targetType, 1);
    if (method == NULL || exceptionOccurred(ee))
	return;

    do_execute_java_method(ee,
			   /* object */target,
			   /* method name */  WRITEOBJECT,
			   /* method signature */  WRITE_SIGNATURE,
			   /* method block of  */ method,
			   /* isStaticCall */ 0,
			   /* argument */ stream, NULL);

}
