/*
 * @(#)MidiInDevice.c	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


// STANDARD includes


// JNI includes
#include <jni.h>

// Platform MIDI includes
#include "PlatformMidi.h"

// UTILITY includes
#include "Utilities.h"

// SimpleInputDeviceProvider includes
#include "com_sun_media_sound_MidiInDevice.h"

//jclass globalMidiInDeviceClass				= NULL;
//jmethodID callbackShortMessageMethodID		= NULL;



// HELPER METHODS

/*
 * return the object's global reference for success, NULL for failure.
 */
/*
static jobject InitializeCallbacksToJava(JNIEnv* e, jobject obj)
{
	jobject globalObject = (*e)->NewGlobalRef(e, obj);

	TRACE1("InitializeCallbacksToJava: globalObject: %d\n", globalObject);

	if (globalObject == NULL)
	{
		ERROR0("MidiInDevice: InitializeCallbacksToJava: globalObject is NULL\n");
		return NULL;
	}

	if (globalMidiInDeviceClass == NULL)
	{
		jclass localClass = (*e)->GetObjectClass(e, globalObject);;
		if (localClass == NULL)
		{
			ERROR0("MidiInDevice: InitializeCallbacksToJava: localClass is NULL\n");
			return NULL;
		}

		globalMidiInDeviceClass = (*e)->NewGlobalRef(e, localClass);	
		if (globalMidiInDeviceClass == NULL)
		{
			ERROR0("MidiInDevice: InitializeCallbacksToJava: globalMidiInDeviceClass is NULL\n");
			return NULL;
		}
	}

	if (callbackShortMessageMethodID == NULL)
	{
		// save the callbackShortMessage callback method id.  
		// this is valid as long as the class is not unloaded.
		callbackShortMessageMethodID = (*e)->GetMethodID(e, globalMidiInDeviceClass, "callbackShortMessage", "(IIIJ)V");
		if (callbackShortMessageMethodID == 0)
		{
			ERROR0("MidiInDevice: InitializeCallbacksToJava: callbackShortMessageMethodID is 0\n");
			return NULL;
		}
	}

	return globalObject;
}
*/


/*
 * release the object's global ref. 
 * $$kk: 06.24.99: should also eventually release the global class and method id!
 */
static void ReleaseCallbacksToJava(JNIEnv* e, jobject obj)
{
	(*e)->DeleteGlobalRef(e, obj);
}


// CALLBACKS
/*
void MidiInputShortMessageCallbackProc(void *threadContext, void *deviceHandle, char status, char data1, char data2, unsigned long timestamp)
{

}
*/

/*
void CallToJavaShortMessage(JNIEnv* e, void * deviceHandle, void * objectHandle, char status, char data1, char data2, unsigned long timestamp)
{
	TRACE0("CallToJavaShortMessage\n");

	TRACE1("CallToJavaShortMessage: objectHandle: %d\n", objectHandle);

	(*e)->CallVoidMethod(e, (jobject)objectHandle, callbackShortMessageMethodID, (jint)status, (jint)data1, (jint)data2, (jlong)timestamp);
 
	TRACE0("CallToJavaShortMessage completed\n");
}
*/

// NATIVE METHODS



JNIEXPORT jint JNICALL
Java_com_sun_media_sound_MidiInDevice_nOpen(JNIEnv* e, jobject thisObj, jint index)
{
	long deviceHandle = 0;
	jobject globalObj = NULL;
		
	TRACE1("Java_com_sun_media_sound_MidiInDevice_nOpen: index: %d\n", index);

#if USE_PLATFORM_MIDI_IN == TRUE

	globalObj = (*e)->NewGlobalRef(e, thisObj);

	//fprintf(stderr, "NATIVE OPEN: globalObj: %d\n", globalObj);

	if (globalObj == NULL)
	{
		char *msg = "Failed to get global obj.\0";
		ERROR0("Java_com_sun_media_sound_MidiInDevice_nOpen: failed to get global obj\n");
		ThrowJavaMessageException(e, JAVA_MIDI_PACKAGE_NAME"/MidiUnavailableException", msg);
	}

	deviceHandle = MIDI_IN_OpenDevice((long)index, (long)globalObj);

#endif // USE_PLATFORM_MIDI_IN 

	// if we didn't get a valid handle, throw a MidiUnavailableException
	// $$kk: 06.24.99: should be getting more information here!
	if ( !deviceHandle )
	{
		char *msg = "Failed to open the device.\0";
		
		if (globalObj != NULL)
		{
			ReleaseCallbacksToJava(e, globalObj);
		}

		ERROR0("Java_com_sun_media_sound_MidiInDevice_nOpen: Failed to open the device\n");
		ThrowJavaMessageException(e, JAVA_MIDI_PACKAGE_NAME"/MidiUnavailableException", msg);
	}

	TRACE0("Java_com_sun_media_sound_MidiInDevice_nOpen succeeded\n");
	return (jint)deviceHandle;
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MidiInDevice_nClose(JNIEnv* e, jobject thisObj, jint deviceHandle)
{
    TRACE0("Java_com_sun_media_sound_MidiInDevice_nClose.\n");

#if USE_PLATFORM_MIDI_IN == TRUE

	MIDI_IN_CloseDevice((long)deviceHandle);
	//ReleaseCallbacksToJava(thisObj);

#endif // USE_PLATFORM_MIDI_IN 

	TRACE0("Java_com_sun_media_sound_MidiInDevice_nClose succeeded\n");
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MidiInDevice_nStart(JNIEnv* e, jobject thisObj, jint deviceHandle)
{
    jboolean ok = FALSE;

	TRACE0("Java_com_sun_media_sound_MidiInDevice_nStart.\n");

#if USE_PLATFORM_MIDI_IN == TRUE

	//ok = MIDI_IN_StartDevice((long)deviceHandle, (MIDI_IN_ShortMessageCallbackProc)&MidiInputShortMessageCallbackProc);
	ok = MIDI_IN_StartDevice((long)deviceHandle);

#endif // USE_PLATFORM_MIDI_IN 

	// if we didn't get a valid handle, throw a MidiUnavailableException
	// $$kk: 06.24.99: should be getting more information here!
	if ( !ok )
	{
		char *msg = "Failed to start the device.\0";
		ERROR0("Java_com_sun_media_sound_MidiInDevice_nStart: Failed to start the device\n");
		ThrowJavaMessageException(e, JAVA_MIDI_PACKAGE_NAME"/MidiUnavailableException", msg);
	}
	else
	{
		TRACE0("Java_com_sun_media_sound_MidiInDevice_nStart succeeded\n");
	}
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MidiInDevice_nStop(JNIEnv* e, jobject thisObj, jint deviceHandle)
{
    TRACE0("Java_com_sun_media_sound_MidiInDevice_nStop.\n");

#if USE_PLATFORM_MIDI_IN == TRUE

	// stop the device and remove all queued events for this device handle
	MIDI_IN_StopDevice((long)deviceHandle);

#endif // USE_PLATFORM_MIDI_IN 

	TRACE0("Java_com_sun_media_sound_MidiInDevice_nStop succeeded\n");
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MidiInDevice_nGetMessages(JNIEnv* e, jobject thisObj)
{
	MIDI_MESSAGE * pMessage;

	jclass globalMidiInDeviceClass				= NULL;
	jmethodID callbackShortMessageMethodID		= NULL;


	TRACE0("Java_com_sun_media_sound_MidiInDevice_nGetMessages\n");


#if USE_PLATFORM_MIDI_IN == TRUE

	while( (pMessage = MIDI_IN_GetMessage()) != NULL)
	{
		
		
		
		if ( (globalMidiInDeviceClass == NULL) || (callbackShortMessageMethodID == NULL) )
		{
			//jobject theObj = (jobject)pMessage->pObject;
			jobject theObj = (jobject)pMessage->objectHandle;
			//fprintf(stderr, "GETMESSSAGES: theObj: %d\n", theObj);

			if (!theObj)
			{
				ERROR0("MidiInDevice: Java_com_sun_media_sound_MidiInDevice_nGetMessages: theObj is NULL\n");
				return;
			}

			if (globalMidiInDeviceClass == NULL)
			{
				jclass localClass = (*e)->GetObjectClass(e, theObj);
				if (localClass == NULL)
				{
					ERROR0("MidiInDevice: Java_com_sun_media_sound_MidiInDevice_nGetMessages: localClass is NULL\n");
					return;
				}

				globalMidiInDeviceClass = (*e)->NewGlobalRef(e, localClass);	
				if (globalMidiInDeviceClass == NULL)
				{
					ERROR0("MidiInDevice: Java_com_sun_media_sound_MidiInDevice_nGetMessages: globalMidiInDeviceClass is NULL\n");
					return;
				}
			}

			if (callbackShortMessageMethodID == NULL)
			{
				// save the callbackShortMessage callback method id.  
				// this is valid as long as the class is not unloaded.
				callbackShortMessageMethodID = (*e)->GetMethodID(e, globalMidiInDeviceClass, "callbackShortMessage", "(IIIJ)V");
				if (callbackShortMessageMethodID == 0)
				{
					ERROR0("MidiInDevice: Java_com_sun_media_sound_MidiInDevice_nGetMessages: callbackShortMessageMethodID is 0\n");
					return;
				}
			}
		}
		

		
		switch((int)pMessage->type)
		{
		case SHORT_MESSAGE:
			TRACE0("Java_com_sun_media_sound_MidiInDevice_nGetMessages: got SHORT_MESSAGE\n");
			//fprintf(stderr, "\nJava_com_sun_media_sound_MidiInDevice_nGetMessages: got SHORT_MESSAGE\n");
			//fprintf(stderr, "status: %c, data1: %c, data2: %c, timestamp: %d\n", pMessage->pData[0], pMessage->pData[1], pMessage->pData[2], pMessage->timestamp);
			//fprintf(stderr, "status: %d, data1: %u, data2: %u, timestamp: %u\n\n", pMessage->pData[0], pMessage->pData[1], pMessage->pData[2], pMessage->timestamp);
			//CallToJavaShortMessage(e, (void *)pMessage->pDevice, (void *)pMessage->pObject, (char)*pMessage->pData, ((char)*(pMessage->pData + 1)), ((char)*(pMessage->pData + 2)), (unsigned long)pMessage->timestamp);
			(*e)->CallVoidMethod(e, (jobject)pMessage->objectHandle, callbackShortMessageMethodID, (jint)pMessage->pData[0], (jint)pMessage->pData[1], (jint)pMessage->pData[2], (jlong)pMessage->timestamp);

			break;

		default:
			TRACE1("Java_com_sun_media_sound_MidiInDevice_nGetMessages: got unsupported message, type %d\n", pMessage->type);
			break;
		} // switch
	}

#endif // USE_PLATFORM_MIDI_IN 

	TRACE0("Java_com_sun_media_sound_MidiInDevice_nGetMessages returning\n");
}
