/*
 * @(#)MixerSourceLine.c	1.23 00/04/21
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


// STANDARD includes


// JNI includes
#include <jni.h>

// ENGINE includes
#include "engine/GenSnd.h"
#include "engine/X_API.h"


// UTILITY includes
#include "Utilities.h"																											  

// MixerSourceLine includes
#include "com_sun_media_sound_MixerSourceLine.h"


// $$kk: 03.21.99: TEMP: need this for old un-merged code
//typedef long STREAM_REFERENCE;


// FORWARDS

// callbacks for stream create, get data, and destroy
static OPErr    MixerSourceLineCallbackProc(void* context, GM_StreamMessage message, GM_StreamData* pAS);


// GLOBALS

jclass g_mixerSourceLineClass = NULL;

jmethodID g_destroyMethodID = 0;
jmethodID g_getDataMethodID = 0;
jmethodID g_startMethodID = 0;
jmethodID g_stopMethodID = 0;
jmethodID g_eomMethodID = 0;
jmethodID g_activeMethodID = 0;
jmethodID g_inactiveMethodID = 0;

jfieldID g_dataArrayFieldID = 0; 


// HELPER METHODS

static XBOOL initializeJavaCallbackVars(JNIEnv* e, jobject thisObj) 
{	
	jclass objClass = (*e)->GetObjectClass(e, thisObj);
	if (objClass == NULL)
	{
		ERROR0("initializeJavaCallbackVars: (objClass == NULL)\n");
		return (XBOOL)FALSE;
	}

	g_mixerSourceLineClass = (*e)->NewGlobalRef(e, objClass);
	if (g_mixerSourceLineClass == NULL)
	{
		ERROR0("initializeJavaCallbackVars: (g_mixerSourceLineClass == NULL)\n");
		return (XBOOL)FALSE;
	}

	g_destroyMethodID =		(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamDestroy", "()V");
	g_getDataMethodID =		(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamGetData", "([BI)I");
	g_startMethodID =		(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamStart", "()V");
	g_stopMethodID =		(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamStop", "()V");
	g_eomMethodID =			(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamEOM", "()V");
	g_activeMethodID =		(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamActive", "()V");
	g_inactiveMethodID =	(*e)->GetMethodID(e, g_mixerSourceLineClass, "callbackStreamInactive", "()V");

	if ( (g_destroyMethodID == 0) || (g_getDataMethodID == 0) || (g_startMethodID == 0) ||
		(g_stopMethodID == 0) || (g_eomMethodID == 0) )
	{
		ERROR0("initializeJavaCallbackVars: (failed to get method IDs)\n");
		return (XBOOL)FALSE;
	}

	g_dataArrayFieldID =	(*e)->GetFieldID(e, g_mixerSourceLineClass, "dataBuffer", "[B");
	if (g_dataArrayFieldID == 0)
	{
		ERROR0("initializeJavaCallbackVars: (g_dataArrayFieldID == 0)\n");
		return (XBOOL)FALSE;
	}

	return (XBOOL)TRUE;
}


// $$kk: 03.22.99: need to call this on shutdown!!
static void releaseJavaCallbackVars(JNIEnv* e) 
{
	jclass localClass = g_mixerSourceLineClass;

	g_mixerSourceLineClass = NULL;
	g_destroyMethodID = 0;
	g_getDataMethodID = 0;
	g_startMethodID = 0;
	g_stopMethodID = 0;
	g_eomMethodID = 0;

	(*e)->DeleteGlobalRef(e, localClass);
}


// STREAMING CHANNEL MANIPULATIONS


JNIEXPORT jint JNICALL
Java_com_sun_media_sound_MixerSourceLine_nOpen(JNIEnv* e, jobject thisObj, jint sampleSizeInBits, jint channels, jfloat sampleRate, jint bufferSize)
{

	OPErr           opErr = NO_ERR;
	long			id = 0;
	jobject			globalSourceLineObj = NULL;

	// get all the class, method, and field id's ONCE.
	// store a global reference to the java source line class to make sure the id's remain valid.
	// $$kk: 03.22.99: need to make sure we release this reference eventually!

	if (g_mixerSourceLineClass == NULL) 
	{
		if (!initializeJavaCallbackVars(e, thisObj))
		{
			ERROR0("Failed to initalized Java callback vars!\n");
			return GENERAL_BAD;
		}
	}

	
	// $$kk: 03.15.98: GLOBAL REF!  this has definite memory leak potential; need to make sure
	// to release all global refs....
	globalSourceLineObj = (*e)->NewGlobalRef(e, thisObj);

	
	// possible error values: NO_ERR, STREAM_STOP_PLAY, PARAM_ERR, NO_FREE_VOICES, possibly others??, 

    TRACE0("Java_com_sun_media_sound_MixerSourceLine_nOpen.\n");
	TRACE2("e %d, thisObj %d\n", (int)e, (int)thisObj);
	TRACE2("bufferSize %d, sampleRate %d\n", (unsigned long)bufferSize, FLOAT_TO_FIXED(sampleRate));
	TRACE2("sampleSizeInBits %d, channels %d\n", (int)sampleSizeInBits, (int)channels);

	TRACE0("Calling GM_AudioStreamSetup\n");

	// we are not handling errors properly here
	id = GM_AudioStreamSetup((void*)e, (long)globalSourceLineObj, MixerSourceLineCallbackProc,
                                    (unsigned long)bufferSize,
                                    FLOAT_TO_FIXED(sampleRate),
                                    (char)sampleSizeInBits, (char)channels);

	TRACE0("Returned from GM_AudioStreamSetup\n");


	if (id != 0)
	{
		if (GM_AudioStreamError(id) != NO_ERR)
		{
			id = 0;
		}
	}

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nOpen completed -> new stream id: %d\n", id);
	
	return (jint)id;
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MixerSourceLine_nStart(JNIEnv* e, jobject thisObj, jint id)
{
	OPErr           opErr = NO_ERR;

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nStart %d.\n", id);
 	VTRACE1("-> stream id: %d\n", (STREAM_REFERENCE)id);

	// $$kk: 03.21.99: TEMP: get rid of this line for old un-merged code
	opErr = GM_AudioStreamPreroll((STREAM_REFERENCE)id);
	if (opErr != NO_ERR)
	{
		ERROR1("Returned an error from GM_AudioStreamPreroll: %d\n", opErr);
	}
	else 
	{
		opErr = GM_AudioStreamStart((STREAM_REFERENCE)id);

		if (opErr != NO_ERR) 
		{
			ERROR1("Returned an error from GM_AudioStreamStart: %d\n", opErr);
		}
	}

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nStart %d completed.\n", id);
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MixerSourceLine_nClose(JNIEnv* e, jobject thisObj, jint id)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nClose %d.\n", id);

    GM_AudioStreamStop((void *)e, (STREAM_REFERENCE)id);

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nClose %d completed.\n", id);
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MixerSourceLine_nPause(JNIEnv* e, jobject thisObj, jint id)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nPause %d.\n", id);

    GM_AudioStreamPause((STREAM_REFERENCE)id);

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nPause %d completed\n", id);
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MixerSourceLine_nResume(JNIEnv* e, jobject thisObj, jint id)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nResume( %d.\n", id);

    GM_AudioStreamResume((STREAM_REFERENCE)id);

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nResume( %d completed\n", id);
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MixerSourceLine_nDrain(JNIEnv* e, jobject thisObj, jint id)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nDrain %d.\n", id);

	GM_AudioStreamDrain((void *)e, (STREAM_REFERENCE)id);
    
	TRACE1("Java_com_sun_media_sound_MixerSourceLine_nDrain %d completed\n", id);
}


JNIEXPORT void JNICALL
Java_com_sun_media_sound_MixerSourceLine_nFlush(JNIEnv* e, jobject thisObj, jint id)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nFlush %d.\n", id);

    GM_AudioStreamFlush((STREAM_REFERENCE)id);

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nFlush %d completed\n", id);
}


JNIEXPORT jlong JNICALL
Java_com_sun_media_sound_MixerSourceLine_nGetPosition(JNIEnv* e, jobject thisObj, jint id)
{
    jlong samples;

	VTRACE1("Java_com_sun_media_sound_MixerSourceLine_nGetPosition %d.\n", id);
 	VTRACE1("-> stream id: %d\n", (STREAM_REFERENCE)id);

 	samples = (jlong)GM_AudioStreamGetSamplesPlayed((STREAM_REFERENCE)id);

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nGetPosition %d completed.\n", id);
	return samples;
}


JNIEXPORT jfloat JNICALL
Java_com_sun_media_sound_MixerSourceLine_nGetLevel(JNIEnv* e, jobject thisObj, jint id)
{
	ERROR0("Java_com_sun_media_sound_MixerSourceLine_nGetLevel: NOT IMPLEMENTED! \n");
	return (jfloat)0;
}


// CONTROLS

JNIEXPORT jfloat JNICALL
Java_com_sun_media_sound_MixerSourceLine_nSetLinearGain(JNIEnv* e, jobject thisObj, jint id, jfloat linearGain)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nSetLinearGain %d.\n", id);

	GM_AudioStreamSetVolume((long)id, FLOAT_TO_VOLUME(linearGain), FALSE);

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nSetLinearGain %d completed.\n", id);

	return VOLUME_TO_FLOAT(GM_AudioStreamGetVolume((long)id));
}


JNIEXPORT jfloat JNICALL
Java_com_sun_media_sound_MixerSourceLine_nSetPan(JNIEnv* e, jobject thisObj, jint id, jfloat pan)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nSetPan %d.\n", id);
 	VTRACE1("-> stream id: %d\n", (STREAM_REFERENCE)id);

	GM_AudioStreamSetStereoPosition((STREAM_REFERENCE)id, (short int)(FLOAT_TO_PAN(pan)));
    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nSetPan %d completed.\n", id);

    return (jfloat)(PAN_TO_FLOAT(GM_AudioStreamGetStereoPosition((STREAM_REFERENCE)id)));
}


JNIEXPORT jint JNICALL
Java_com_sun_media_sound_MixerSourceLine_nSetSampleRate(JNIEnv* e, jobject thisObj, jint id, jint rate)
{

    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nSetSampleRate %d.\n", id);
 	VTRACE1("-> stream id: %d\n", (STREAM_REFERENCE)id);
	
	// $$kk: 04.06.99: steve hales says i should be using the UNSIGNED_LONG macros here.  
	GM_AudioStreamSetRate((STREAM_REFERENCE)id, UNSIGNED_LONG_TO_XFIXED(rate));
    TRACE1("Java_com_sun_media_sound_MixerSourceLine_nSetSampleRate %d completed.\n", id);

	// $$kk: 04.06.99: steve hales says i should be using the UNSIGNED_LONG macros here.  
    return (jint)XFIXED_TO_UNSIGNED_LONG(GM_AudioStreamGetRate((STREAM_REFERENCE)id));
}


// CALLBACKS INTO JAVA

static OPErr CallToJavaStreamDestroy(JNIEnv* e, jobject sourceLine)
{
    TRACE1("CallToJavaStreamDestroy, sourceLine: %d.\n", (long)sourceLine);
	(*e)->CallVoidMethod(e, sourceLine, g_destroyMethodID);

	// $$kk: 04.13.98: moving this here from above because we can still get this callback *after* nDisposeSourceLine,
	// giving us a vm crash....
	
	(*e)->DeleteGlobalRef(e, sourceLine);

    TRACE1("CallToJavaStreamDestroy, sourceLine: %d completed.\n", (long)sourceLine);
	return NO_ERR;
}


static jint CallToJavaStreamGetData(JNIEnv* e, jobject sourceLine, jbyteArray array, jint dataLength) 
{
    jint length;    
	VTRACE2("CallToJavaStreamGetData, sourceLine: %d, dataLength: %d\n", (long)sourceLine, dataLength);
	length = (*e)->CallIntMethod(e, sourceLine, g_getDataMethodID, array, dataLength);
	VTRACE1("CallToJavaStreamGetData, sourceLine: %d completed.\n", (long)sourceLine);
	return length; 
}

static OPErr CallToJavaStreamStart(JNIEnv* e, jobject sourceLine)
{
    TRACE1("CallToJavaStreamStart, sourceLine: %d.\n", (long)sourceLine);
	(*e)->CallVoidMethod(e, sourceLine, g_startMethodID);
    TRACE1("CallToJavaStreamStart, sourceLine: %d completed.\n", (long)sourceLine);
	return NO_ERR;
}

static OPErr CallToJavaStreamStop(JNIEnv* e, jobject sourceLine)
{
    TRACE1("CallToJavaStreamStop, sourceLine: %d.\n", (long)sourceLine);
	(*e)->CallVoidMethod(e, sourceLine, g_stopMethodID);
    TRACE1("CallToJavaStreamStop, sourceLine: %d completed.\n", (long)sourceLine);
	return NO_ERR;
}

static OPErr CallToJavaStreamEOM(JNIEnv* e, jobject sourceLine)
{
    TRACE1("CallToJavaStreamEOM, sourceLine: %d.\n", (long)sourceLine);
	(*e)->CallVoidMethod(e, sourceLine, g_eomMethodID);
    TRACE1("CallToJavaStreamEOM, sourceLine: %d completed.\n", (long)sourceLine);
	return NO_ERR;
}

static OPErr CallToJavaStreamActive(JNIEnv* e, jobject sourceLine)
{
    TRACE1("CallToJavaStreamActive, sourceLine: %d.\n", (long)sourceLine);
	(*e)->CallVoidMethod(e, sourceLine, g_activeMethodID);
    TRACE1("CallToJavaStreamActive, sourceLine: %d completed.\n", (long)sourceLine);
	return NO_ERR;
}

static OPErr CallToJavaStreamInactive(JNIEnv* e, jobject sourceLine)
{
    TRACE1("CallToJavaStreamInactive, sourceLine: %d.\n", (long)sourceLine);
	(*e)->CallVoidMethod(e, sourceLine, g_inactiveMethodID);
    TRACE1("CallToJavaStreamInactive, sourceLine: %d completed.\n", (long)sourceLine);
	return NO_ERR;
}


// CALLBACK PROC

static OPErr MixerSourceLineCallbackProc(void* context, GM_StreamMessage message, GM_StreamData* pAS)
{
    JNIEnv* e = (JNIEnv*)context;														    
    VTRACE2("MixerSourceLineCallbackProc, context: %d, message: %d\n", context, message);

    // for STREAM_CREATE, we just need to allocate the buffer.
	// we need to convert from gmData->dataLength (number of frames) to number of bytes.
	// $$kk: 03.15.98: this is the same calculation done by PV_GetSampleSizeInBytes.  however, this
	// method is not available outside GenAudioStreams.c right now.

	if (message == STREAM_CREATE)
    {
		jint    const byteCount = pAS->dataLength * pAS->channelSize * (pAS->dataBitSize / 8);

		VTRACE1("STREAM_CREATE: %d\n", pAS->userReference);
		VTRACE3("STREAM_CREATE: byteCount = %d, dataLength = %d, frameSize = %d.\n", byteCount, pAS->dataLength, (pAS->channelSize * (pAS->dataBitSize / 8)) );

        pAS->pData = XNewPtr(byteCount);
        if (!pAS->pData)
        {
            ERROR1("MixerSourceLineCallbackProc: STREAM_CREATE: Failed to allocate %d bytes\n", byteCount);
            return MEMORY_ERR;
        }

		VTRACE1("STREAM_CREATE: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_DESTROY)
    {
		VTRACE1("STREAM_DESTROY: %d\n", pAS->userReference);

		CallToJavaStreamDestroy(e, (jobject)pAS->userReference);

		// $$kk: 12.21.98: for some reason, this causes the vm to crash some
		// random time later.  so we have a memory leak here.  i don't know
		// why we can't release the memory here....
		
		// $$kk: 03.22.00: i am putting this code back in.  the memory leak is
		// causing problems, and so far i have not been able to reproduce the
		// crash.  the memory leak is reported in bug #4319431: "Memory leaks
		// on opening and closing source DataLines."

        if (pAS->pData)
        {
           XDisposePtr(pAS->pData);
        }

		VTRACE1("STREAM_DESTROY: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_GET_DATA)
    {		
		int length;
		int frameLengthInBytes = pAS->channelSize * (pAS->dataBitSize / 8);
		jbyteArray array = (jbyteArray)(*e)->GetObjectField(e, (jobject)pAS->userReference, g_dataArrayFieldID);

		if (array == NULL) 
		{
			ERROR0("STREAM_GET_DATA: failed to get array\n");
			return GENERAL_BAD;
		}

		// make the callback.
		// pAS->dataLength is the length in FRAMES.
		// length returned is length in FRAMES.
		length = (int)CallToJavaStreamGetData(e, (jobject)pAS->userReference, array, (jint)pAS->dataLength);

		// length will be -1 when the stream finishes
		
		if (length < 0)
		{
			VTRACE0("STREAM_GET_DATA: got negative length; stopping stream\n");
			return STREAM_STOP_PLAY;
		}
													   
		// get the relevant array elements

		// $$kk: 03.15.98: can we avoid this copy??
		(*e)->GetByteArrayRegion(e, array, (jint)0, (jint)(length * frameLengthInBytes), (jbyte*)(pAS->pData));
		
		// $$kk: 04.21.00: fix for bug #4332327: "JNI leaks local references on 
		// keeping a SourceLine open for long."
		(*e)->DeleteLocalRef(e, array);		
		
		// set dataLength to the number of frames read
		pAS->dataLength = length;

		VTRACE2("STREAM_GET_DATA: length: %d, pAS->dataLength: %d\n", length, pAS->dataLength);
		VTRACE1("STREAM_GET_DATA: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_START)
    {
		TRACE2("STREAM_START: stream: %d,  framePosition: %d\n", pAS->userReference, pAS->startSample);
		CallToJavaStreamStart(e, (jobject)pAS->userReference);
		TRACE1("STREAM_START: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_STOP)
    {
		TRACE2("STREAM_STOP: %d,  framePosition: %d\n", pAS->userReference, pAS->startSample);
		CallToJavaStreamStop(e, (jobject)pAS->userReference);
		TRACE1("STREAM_STOP: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_EOM)
    {
		TRACE2("STREAM_EOM: %d,  framePosition: %d\n", pAS->userReference, pAS->startSample);
		CallToJavaStreamEOM(e, (jobject)pAS->userReference);
		TRACE1("STREAM_EOM: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_ACTIVE)
    {
		TRACE2("STREAM_ACTIVE: %d,  framePosition: %d\n", pAS->userReference, pAS->startSample);
		CallToJavaStreamActive(e, (jobject)pAS->userReference);
		TRACE1("STREAM_ACTIVE: %d completed\n", pAS->userReference);
    }
    else if (message == STREAM_INACTIVE)
    {
		TRACE2("STREAM_INACTIVE: %d,  framePosition: %d\n", pAS->userReference, pAS->startSample);
		CallToJavaStreamInactive(e, (jobject)pAS->userReference);
		TRACE1("STREAM_INACTIVE: %d completed\n", pAS->userReference);
    }

    else
    {
        ERROR1("Bad message code in MixerSourceLineCallbackProc() : %d\n", message);
        return GENERAL_BAD;
    }

	return NO_ERR;
}
