/*
 * @(#)PlatformMidi.h	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * $$kk: 06.23.99
 * This file contains header information for platform midi support.
 * This does not interface with the Beatnik engine at all.
 */

// JNI includes
#include <jni.h>

#include "Configure.h"				// put flags for debug msgs etc. here

/*
 * Platform MIDI IN support.
 * deviceId:		device-by-number
 * deviceHandle:	native device handle
 */

#if USE_PLATFORM_MIDI_IN == TRUE

// MIDI message types
typedef enum
{
	SHORT_MESSAGE = 0,
	LONG_MESSAGE = 1
} MidiMessageType;


// MIDI message object
struct MIDI_MESSAGE
{
	long deviceHandle;			// handle to the device
	long objectHandle;			// handle to the object
	MidiMessageType type;		// message type (SHORT_MESSAGE or LONG_MESSAGE)
	int length;					// data length of the message in bytes
	unsigned char *pData;		// message data
	int timestamp;				// timestamp in milliseconds
	struct MIDI_MESSAGE *pNext;	// next message
};

typedef struct MIDI_MESSAGE MIDI_MESSAGE; 


//typedef void (*MIDI_IN_ShortMessageCallbackProc)(void *objectContext, void *deviceHandle, char status, char data1, char data2, unsigned long timestamp);
/*
 * Process a short message received from the hardware.  Timestamp is in microseconds.
 */
//void	MIDI_IN_ProcessShortMessage(unsigned byte status, unsigned byte data1, unsigned byte data2, unsigned long timestamp);

	
/*
 * Get the number of MIDI IN devices on the system.
 */
long	MIDI_IN_GetNumDevices(void);
	
/*
 * Get the name of the device with this id
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_GetDeviceName(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Get the vendor of the device with this id
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_GetDeviceVendor(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Get the description of the device with this id
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_GetDeviceDescription(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Get the version of the device with this id
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_GetDeviceVersion(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Open the device with this id.
 * Returns a device handle or 0 for failure.
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_OpenDevice(long deviceID, long objectHandle);
	
/*
 * Close the device handle.
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_CloseDevice(long deviceHandle);
	
/*
 * Start the device with this handle.
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_StartDevice(long deviceHandle);
	
/*
 * Stop the device with this handle.
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_IN_StopDevice(long deviceHandle);

/*
 * Get the mext message from the queue.
 */
MIDI_MESSAGE * MIDI_IN_GetMessage(void);

#endif // USE_PLATFORM_MIDI_IN


/*
 * Platform MIDI OUT support.
 * deviceId:		device-by-number
 * deviceHandle:	native device handle
 */

#if USE_PLATFORM_MIDI_OUT == TRUE

/*
 * Get the number of MIDI OUT devices on the system.
 */
long	MIDI_OUT_GetNumDevices(void);
	
/*
 * Get the name of the device with this id
 */
long	MIDI_OUT_GetDeviceName(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Get the vendor of the device with this id
 */
long	MIDI_OUT_GetDeviceVendor(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Get the description of the device with this id
 */
long	MIDI_OUT_GetDeviceDescription(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Get the version of the device with this id
 */
long	MIDI_OUT_GetDeviceVersion(long deviceID, char *name, unsigned long nameLength);
	
/*
 * Open the device with this id.
 * Returns a device handle or 0 for failure.
 */
long	MIDI_OUT_OpenDevice(long deviceID, long objectHandle);
	
/*
 * Close the device handle.
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_OUT_CloseDevice(long deviceHandle);

/*
 * Process a short message sent to the hardware.  Timestamp is in microseconds.
 * Returns TRUE (1) or FALSE (0)
 */
long	MIDI_OUT_ProcessShortMessage(long deviceHandle, unsigned char status, unsigned char data1, unsigned char data2, unsigned long timestamp);

#endif // USE_PLATFORM_MIDI_OUT
