/*
 * @(#)alphaCompst_loop_uti.h	1.27 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <string.h>
/**
 * Macros used in different variants of the compositing loop.
 */

#define ReadDst_Default ReadDstPixel(dstAlpha, dstRed, dstGreen, dstBlue)
#define ReadDst ReadDst_Default

#ifdef DITHERED_INDEXED
#undef  DeclareDitherVar
#define DeclareDitherVar int skipDither=0;
#else
#define DeclareDitherVar
#endif

#ifdef DITHERED_INDEXED
#undef  DitherON
#define DitherON skipDither=0
#undef  DitherBgON
#define DitherBgON skipDither=0
#else
#define DitherON
#define DitherBgON
#endif

#ifdef DITHERED_INDEXED
#undef  DitherOFF
#define DitherOFF skipDither=1
#undef  DitherBgOFF
#define DitherBgOFF skipDither=1
#else
#define DitherOFF
#define DitherBgOFF
#endif

extern unsigned char blendlut[256][256];
extern int blendlutInitialized;
extern void initBlendLut();

#define ALPHAMULT(a,b) blendlut[a][b]

#define CLIP_BLEND(rA, cA, s1, s2)					\
    ((ALPHAMULT(rA,s1)) + (ALPHAMULT(cA,s2)))

#define CLIP_BLEND16(rA, cA, s1, s2)					\
    ((((rA) * (s1))/NORM_SCALE) + (((cA) * (s2)) / NORM_SCALE))

/* CLEAR Rule is the same for all composite flavors */
#define CLEAR_COMP                                                      \
do {                                                                       \
    if (pathData) {                                                     \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                ReadRastAlpha(rastAlpha);                               \
                if (rastAlpha > 0) {					\
                    int a = NORM_SCALE - rastAlpha;                     \
                    if (a) {                                            \
                         ReadDst;                                       \
                         dstAlpha   = ALPHAMULT(a,dstAlpha);            \
                         dstRed     = ALPHAMULT(a,dstRed);              \
                         dstGreen   = ALPHAMULT(a,dstGreen);            \
                         dstBlue    = ALPHAMULT(a,dstBlue);             \
                    } else {                                            \
                         dstAlpha = dstRed = dstGreen = dstBlue  = 0;   \
                    }                                                   \
                    WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                }                                                       \
                NextRastAlpha;                                          \
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
            NextRastAlphaRow;                                           \
        }                                                               \
    } else {                                                            \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                dstAlpha = dstRed = dstGreen = dstBlue  = 0;            \
                WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue);     \
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
        }                                                               \
    }                                                                   \
} while (0);

#define INT_FASTCOPYLOOP                                                \
    do {								\
        int bytes_per_row = width*sizeof(int);                          \
        for (row = 0 ; row < height ; row++) {                          \
            memcpy(dstPtr, srcPtr, bytes_per_row);                      \
            dstPtr += dstScanStr + width;                               \
            srcPtr += srcScanStr + width;                               \
        }								\
    } while (0);

#define INT_FASTXORLOOP                                                 \
do {                                                                    \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                *dstPtr++ ^= color;                                     \
            }                                                           \
            dstPtr += dstScanStr;                                       \
        }                                                               \
} while (0); 

#define COMPOSITE_TILE(AssignBG, Rule)                                  \
do {                                                                    \
    if (pathData) {                                                     \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                ReadRastAlpha(rastAlpha);                               \
                if (rastAlpha > 0) {					\
                    ReadSrcPixel(srcAlpha, srcRed, srcGreen, srcBlue);  \
                    ReadDst;                                            \
                    AssignBG;                                           \
                    skipDst = 0;                                        \
                    Rule;                                               \
                    if (!skipDst) {                                     \
                        WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                    }                                                   \
                }                                                       \
                NextRastAlpha;                                          \
                NextSrcPixel;                                           \
                NextDstPixel;                                           \
            }                                                           \
            NextSrcRow;                                                 \
            NextDstRow;                                                 \
            NextRastAlphaRow;                                           \
        }                                                               \
    } else {                                                            \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                rastAlpha = 255;                                        \
                ReadSrcPixel(srcAlpha, srcRed, srcGreen, srcBlue);      \
                ReadDst;                                                \
                AssignBG;                                               \
                skipDst = 0;                                            \
                Rule;                                                   \
                if (!skipDst) {                                         \
                    WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                }                                                       \
                NextSrcPixel;                                           \
                NextDstPixel;                                           \
            }                                                           \
            NextSrcRow;                                                 \
            NextDstRow;                                                 \
        }                                                               \
    }                                                                   \
} while (0);

#define RestoreSrcColor()						\
    do {								\
	srcAlpha = colorAlpha;						\
	srcRed   = colorRed;						\
	srcGreen = colorGreen;						\
	srcBlue  = colorBlue;						\
    } while (0)

#ifdef DST_XCCC

#define COMPOSITE_COLOR(Rule)                                           \
do {                                                                    \
    if (pathData) {                                                     \
      if (rule == SRC &&                                                \
	  extraAlpha == NORM_SCALE &&                                   \
	  colorAlpha == 255) {		                                \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                ReadRastAlpha(rastAlpha);                               \
                if (rastAlpha > 0) {					\
		    if (rastAlpha >= 255) {				\
			CopyColorToDst;					\
		    } else {						\
			ReadDst;                                        \
                        skipDst = 0;                                    \
			Rule;                                           \
                        if (!skipDst) {                                 \
			    WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                        }                                               \
			RestoreSrcColor();				\
		    }							\
                }                                                       \
                NextRastAlpha;                                          \
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
            NextRastAlphaRow;                                           \
        }                                                               \
      } else {								\
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                ReadRastAlpha(rastAlpha);                               \
                if (rastAlpha > 0) {					\
		    ReadDst;						\
                    skipDst = 0;                                        \
		    Rule;						\
                    if (!skipDst) {                                     \
		         WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                    }                                                   \
		    RestoreSrcColor();					\
                }                                                       \
                NextRastAlpha;                                          \
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
            NextRastAlphaRow;                                           \
        }                                                               \
      }									\
    } else {                                                            \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                rastAlpha = 255;                                        \
                ReadDst;                                                \
                skipDst = 0;                                            \
                Rule;                                                   \
                if (!skipDst) {                                         \
                     WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                }                                                       \
		RestoreSrcColor();					\
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
        }                                                               \
    }                                                                   \
} while (0);
#else

#define COMPOSITE_COLOR(Rule)                                           \
do {                                                                    \
    if (pathData) {                                                     \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                ReadRastAlpha(rastAlpha);                               \
                if (rastAlpha > 0) {					\
		    ReadDst;						\
                    skipDst = 0;                                        \
		    Rule;						\
                    if (!skipDst) {                                     \
		        WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                    }                                                   \
		    RestoreSrcColor();					\
                }                                                       \
                NextRastAlpha;                                          \
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
            NextRastAlphaRow;                                           \
        }                                                               \
    } else {                                                            \
        for (row = 0 ; row < height ; row++) {                          \
            for (column = 0 ; column < width ; column++) {              \
                rastAlpha = 255;                                        \
                ReadDst;                                                \
                skipDst = 0;                                            \
                Rule;                                                   \
                if (!skipDst) {                                         \
		    WriteDstPixel(dstAlpha, dstRed, dstGreen, dstBlue); \
                }                                                       \
		RestoreSrcColor();					\
                NextDstPixel;                                           \
            }                                                           \
            NextDstRow;                                                 \
        }                                                               \
    }                                                                   \
} while (0);
#endif

#define EMPTY_STATEMENT

#define ASSIGN_BG(sA, sR, sG, sB, bA, bR, bG, bB)                       \
        if (sA == 0){                                                   \
            sA = bA;                                                    \
            sR = bR;                                                    \
            sG = bG;                                                    \
            sB = bB;                                                    \
            if (sA >= NORM_SCALE) {                                     \
                DitherBgOFF;                                            \
            } else {                                                    \
                DitherBgON;                                             \
            }                                                           \
        }
