/*
 * @(#)alphaCompst_paintloop.h	1.27 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <alphaCompst_XOR_MODE.h>
#include <alphaCompst_SRC.h>
#include <alphaCompst_SRC_OVER.h>
#include <alphaCompst_DST_OVER.h>
#include <alphaCompst_SRC_IN.h>
#include <alphaCompst_DST_IN.h>
#include <alphaCompst_SRC_OUT.h>
#include <alphaCompst_DST_OUT.h>
#include <alphaCompst_loop_uti.h>

extern PaintLoopFunc    TILE_BLENDFUNC;

void
TILE_BLENDFUNC(JNIEnv *env,
	       void *dstBase, jint dstScanStr, jint dstPixStr,
               void *srcBase, jint srcScanStr, jint srcPixStr,
	       unsigned char *pathData, jint offset,
	       jint alphaTileScan, jint rule, jfloat fXtraAlpha,
               jint width, jint height, jboolean srcPre, jint xorColor,
	       ColorData *cData)
{
    int extraAlpha = (int)(fXtraAlpha*(float)NORM_SCALE);
    int row, column;
    int rastAlpha = 255;
    int compAlpha = 0;
    int srcAlpha;
    int srcRed;
    int srcGreen;
    int srcBlue;
    int dstAlpha = 0;
    int dstRed = 0;
    int dstGreen = 0;
    int dstBlue = 0;
    int pathDataInc = 1;
    int xorRed   = ((xorColor >> 16) & 0xff);
    int xorGreen = ((xorColor >>  8) & 0xff);
    int xorBlue  = ((xorColor >>  0) & 0xff);
    int skipDst;

    DeclareDitherVar

    DeclareSrc
    DeclareDst

    if (!blendlutInitialized) {
        initBlendLut();
    }
    
    InitSrc;
    InitRastAlpha;

    InitDst;

    switch (rule) {

    case XOR_MODE:

        DitherOFF;
        
#ifdef FastXORLoop
        FastXORLoop;
#else
        COMPOSITE_TILE(EMPTY_STATEMENT,RASTEROP_XOR(rastAlpha, compAlpha,
                                                    extraAlpha,
                                                    srcAlpha,
                                                    (srcRed ^ xorRed),
                                                    (srcGreen ^ xorGreen),
                                                    (srcBlue ^ xorBlue),
                                                    dstAlpha, dstRed,
                                                    dstGreen, dstBlue));
#endif
        DitherON;
        break;

    case CLEAR:
        CLEAR_COMP;
        break;
        
#ifdef DITHERED_INDEXED
#undef  DitherON
#define DitherON
#endif
#ifdef DITHERED_INDEXED
#undef  DitherOFF
#define DitherOFF
#endif
        
#undef ReadDst
#define ReadDst
    case SRC:
#ifdef FastCopyLoop
    if ((pathData  == 0) && (extraAlpha == 255)) {
        FastCopyLoop;
    } else {
        COMPOSITE_TILE(EMPTY_STATEMENT, SRC_RULE(rastAlpha, compAlpha,
                                                 extraAlpha,
                                                 srcAlpha, srcRed,
                                                 srcGreen, srcBlue,
                                                 dstAlpha, dstRed,
                                                 dstGreen, dstBlue));
    }
#else
    COMPOSITE_TILE(EMPTY_STATEMENT, SRC_RULE(rastAlpha, compAlpha, extraAlpha,
                                             srcAlpha, srcRed,
                                             srcGreen, srcBlue,
                                             dstAlpha, dstRed,
                                             dstGreen, dstBlue));
#endif
        break;

    case SRC_OVER:
        COMPOSITE_TILE(EMPTY_STATEMENT, SRC_OVER_RULE(rastAlpha, compAlpha,
                                                      extraAlpha,
                                                      srcAlpha, srcRed,
                                                      srcGreen, srcBlue,
                                                      dstAlpha, dstRed,
                                                      dstGreen, dstBlue));
        break;
#undef ReadDst
#define ReadDst ReadDst_Default

    case DST_OVER:
        COMPOSITE_TILE(EMPTY_STATEMENT, DST_OVER_RULE(rastAlpha, compAlpha,
                                                      extraAlpha,
                                                      srcAlpha, srcRed,
                                                      srcGreen, srcBlue,
                                                      dstAlpha, dstRed,
                                                      dstGreen, dstBlue));
        break;

    case SRC_IN:
        COMPOSITE_TILE(EMPTY_STATEMENT, SRC_IN_RULE(rastAlpha, compAlpha,
                                                    extraAlpha,
                                                    srcAlpha, srcRed,
                                                    srcGreen, srcBlue,
                                                    dstAlpha, dstRed,
                                                    dstGreen, dstBlue));
        break;

    case DST_IN:
        COMPOSITE_TILE(EMPTY_STATEMENT, DST_IN_RULE(rastAlpha, compAlpha,
                                                    extraAlpha,
                                                    srcAlpha, srcRed,
                                                    srcGreen, srcBlue,
                                                    dstAlpha, dstRed,
                                                    dstGreen, dstBlue));
        break;

    case SRC_OUT:
        COMPOSITE_TILE(EMPTY_STATEMENT, SRC_OUT_RULE(rastAlpha, compAlpha,
                                                     extraAlpha,
                                                     srcAlpha, srcRed,
                                                     srcGreen, srcBlue,
                                                     dstAlpha, dstRed,
                                                     dstGreen, dstBlue));
        break;

    case DST_OUT:
        COMPOSITE_TILE(EMPTY_STATEMENT, DST_OUT_RULE(rastAlpha, compAlpha,
                                                     extraAlpha,
                                                     srcAlpha, srcRed,
                                                     srcGreen, srcBlue,
                                                     dstAlpha, dstRed,
                                                     dstGreen, dstBlue));
        break;

    }
}
