/*
 * @(#)kcpcache.h	1.10 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 	@(#)kcpcache.h	1.70	99/03/04

	Contains:	interface to color processors' special evaluations

	COPYRIGHT (c) 1995-1999 Eastman Kodak Company
	As an unpublished work pursuant to Title 17 of the United
	States Code.  All rights reserved.
 */

#ifndef _KCPCACHE_H_
#define _KCPCACHE_H_ 1

#include "kcptmgr.h"
#include "attrib.h"
	

#if defined (KPALPHA)
#define KCP_SHIFT_RIGHT(data, result, bits)  \
		if (data < 0) { \
			result = -((-data) >> bits); \
		} \
		else { \
			result = (data >> bits); \
		}

#define KCP_SHIFT_RIGHT_ROUND(data, result, bits)  \
		if ((data + (((1 << bits) -1) >> 1)) < 0) { \
			result = -(((-data + ((1 << (bits -1)) -1) -1)) >> bits); \
		} \
		else { \
			result = ((data + ((1 << (bits -1)) -1)) >> bits); \
		}
#else
#define KCP_SHIFT_RIGHT(data, result, bits) result = data >> bits;

#define KCP_SHIFT_RIGHT_ROUND(data, result, bits) result = ((data + ((1 << (bits -1)) -1)) >> bits);

#endif

#define KCP_EXTRACT_COMPONENT(data, dataBits, position) \
	((data >> position) & ((1 << dataBits) -1))

#define KCP_CONVERT_DOWN(data, startBits, endBits) \
	((data + ((1 << (startBits - endBits -1)) - (data >> endBits))) >> (startBits - endBits))

/* the following macro will only work if (startBits - (endBits - startBits)) >= 0
 * which means endBits can be no more than twice startBits
 * Use the macro successively if this condition is not met */
#define KCP_CONVERT_UP(data, startBits, endBits) \
((data << (endBits-startBits)) + ((data >> (startBits - (endBits - startBits)))))

/* The overhead associated with optimizing the tables means that below some
	threshold evaluation will be faster using non-optimized evaluation */
#define TH1_MIN_EVALS (1500)
#define TH1_MIN_16BIT_EVALS (15000)

#define EVAL_FRACBITS	(19)						/* fractional bits for interpolations */
#define EVAL_FRACMASK	((1 << EVAL_FRACBITS) -1)	/* mask of fractional bits */

#define EVAL_OLUT_ENTRIESD8 (FUT_OUTTBL_ENT)	/* size of 8 bit evaluation output lut */
#define EVAL_OLUT_ENTRIESD16 (1 << 16)			/* size of 12 & 16 bit evaluation output lut */

#define ET_I8	(0)
#define ET_I12	(1)
#define ET_I16	(2)
#define ET_G12	(3)
#define ET_G16	(4)
#define ET_O8	(5)
#define ET_O12	(6)
#define ET_O16	(7)

/* image pointers definition */
typedef union imagePtr_u {
	PTImgAddr_t pI;
	KpUInt8_p	p8;
	KpUInt16_p	p16;
	KpUInt32_p	p32;
} imagePtr_t, FAR* imagePtr_p;

typedef struct evalILut_s {
	KpInt32_t	index;	/* index into grid table */
	KpInt32_t	frac;	/* the interpolant */
} evalILut_t, FAR* evalILut_p;

/* Evaluation output lut definition  */
typedef union evalOLutPtr_u {
	KpUInt8_p	p8;
	KpUInt16_p	p16;
	KpUInt32_p	p32;
} evalOLutPtr_t, FAR* evalOLutPtr_p; 

typedef void (*evalTh1Proc_t) (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
typedef evalTh1Proc_t FAR* evalTh1Proc_p;
typedef void (*formatFunc_t) (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

typedef struct evalControl_s {
	callBack_p		callBack;
	evalTh1Proc_t	evalFunc;
	KpInt32_t		nFuts;
	PTTable_p*		evalList;
	KpUInt32_t		ioMaskList[FUT_NOCHAN];
	KpUInt32_t		compatibleDataType;
	KpUInt32_t		optimizedEval;
	KpUInt32_t		evalDataTypeI, evalDataTypeO;
	KpInt32_t		imageLines, imagePels;
	imagePtr_t		inputData[FUT_NICHAN], outputData[FUT_NOCHAN];
	KpInt32_t		inPelStride[FUT_NICHAN], inLineStride[FUT_NICHAN];
	KpInt32_t		outPelStride[FUT_NOCHAN], outLineStride[FUT_NOCHAN];

	/* needed only if image formats do not match an optimized function */
	formatFunc_t	formatFuncI, formatFuncO;
	KpInt32_t		tempPelStride[FUT_NOCHAN];
	
	/* needed for returning MP errors */
	PTErr_t			PTErr;
} evalControl_t, FAR* evalControl_p, FAR* FAR* evalControl_h;


PTErr_t		allocEvalState (PTTable_p);
void		unlockEvalTables (PTTable_p);
PTErr_t		getEvalFunc (KpInt32_t, KpUInt32_t, KpUInt32_t, evalControl_p);
PTErr_t		initEvalTables (evalControl_p);
PTErr_t		evalImageMP (evalControl_p);
PTErr_t		evalImage (evalControl_p);

void evalTh1gen (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o1d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o1d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o2d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o2d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o3d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o3d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1iB24oB24 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1iL24oL24 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1iQDoQD (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1iQDo3 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3oQD (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o4d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o4d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o5d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o5d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o6d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o6d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o7d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o7d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o8d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i3o8d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);

/* Tetrahedral Interpolation evaluation of 3-channel 8-bit to 3-channel 12/16-bit data */
void evalTh1i3o3d8to16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);

/* Tetrahedral Interpolation evaluation of 3-channel 12/16-bit to 3-channel 8-bit data */
void evalTh1i3o3d16to8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);

void evalTh1i4o1d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o1d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o2d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o2d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o3d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o3d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o3QD (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1iB32oB32 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1iL32oL32 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o4d8 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);
void evalTh1i4o4d16 (imagePtr_p, KpInt32_p, KpUInt32_t, imagePtr_p, KpInt32_p, KpUInt32_t, KpInt32_t, PTTable_p);

void format555to8 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format565to8 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void format555to12 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format565to12 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format8to12 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format10to12 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void format555to16 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format565to16 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format8to16 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format10to16 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format12to16 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void pass8in (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void pass16in (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void format8to555 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format8to565 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void format12to555 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format12to565 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format12to8 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format12to10 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void format16to555 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format16to565 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format16to8 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format16to10 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void format16to12 (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

void pass8out (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);
void pass16out (KpInt32_t, imagePtr_p, KpInt32_p, imagePtr_p);

#endif	/* _KCPCACHE_H_ */
