/*
 * @(#)makefuts.h	1.8 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 	@(#)makefuts.h	1.20 98/12/21

	Contains:	header file for making ICM specific futs

	Written by:	Poe

	COPYRIGHT (c) 1991-1998 Eastman Kodak Company
	As an unpublished work pursuant to Title 17 of the United
	States Code.  All rights reserved.
 */


#ifndef _MAKEFUTS_H_
#define _MAKEFUTS_H_ 1

#include <math.h>
#include "fut.h"
#include "fut_util.h"
#include "kcmptlib.h"

/* These constants are used in logrgb.c and loguvl.c to shape shadows by adding `flare' */

#define	FLARE_YMIN		0.00392156862745	/* Y_min = 1/255, a la DM8000 */
#define FLARE_YFLARE	0.00393700787402	/* Y_min/(1 + Y_min) */
#define	FLARE_DMAX		2.40654018043395	/* D_max = -(log Y_min) */
#define	FLARE_TANPT		0.01065992873906	/* eY_min */
#define	FLARE_MAPPT		0.18046425546277	/* (log e)/D_max */
#define	FLARE_SLOPE		16.9292178100213	/* MAPPT/TANPT */

/* This constant was define in the non-ansi c compilers math.h.  It does not
 * seem to be defined by any of the compilers that we are using */

#define M_LN10				2.30258509299404568402

	/*
		These scale factors define the neutral point for the Lab8 and Lab16 color spaces.
	*/
#define KCP_LAB8_NEUTRAL (0.50196078431)	/* = 128.0/255.0 */
#define KCP_LAB16_NEUTRAL (0.50000762951)	/* = 32768.0/65535.0 */
#define KCP_LINLAB_GRID_SIZE (16)

#define KCP_LAB8_L_NEUTRAL (0xffff)
#define KCP_LAB8_AB_NEUTRAL (0x8080)
#define KCP_LAB16_L_NEUTRAL (0xff00)
#define KCP_LAB16_AB_NEUTRAL (0x8000)

/* inverse gamma and offset for Rec 709 transfer function */
#define KCP_709_INV_GAMMA (0.45)	/* inverse gamma */
#define KCP_709_OFFSET (0.099)		/* offset */

/* D50 white point */
#define	KCP_D50_X	(0.9642)
#define	KCP_D50_Y	(1.0000)
#define	KCP_D50_Z	(0.8249)

	/* restricts t to interval [low, high] */
#define RESTRICT(t, low, high) (MAX ((low), MIN ((high), (t))))

	/* quantizes t to nearest (short) integer in given scale */
#define QUANT(t, scale)	((short)((double)(scale) * RESTRICT((t), 0.0, 1.0) + 0.5))

	/* quantizes t to nearest integer in given scale */
	/* seperate the lines to fix floating point precision error on 68k */
#define QUANT1(t, scale)	\
			t = (double)RESTRICT((t), 0.0, 1.0); \
			t *= (double)scale; \
			t += (double)0.5

	/* (approximate) inverse of QUANT() */
#define DEQUANT(t, scale) ((double)(t)/(double)(scale))

#define	POW(x, power)	( ((x) > 0.0) ? exp ((power) * log ((x))) : pow ((x), (power)) )
#define	ANTILOG(x)	( exp ((x) * M_LN10) )

	/* CIE visual-response function */
#define H(t) ( ((t) <= 0.008856) ? 9.033 * (t) : 1.16 * POW((t), 1.0/3.0) - 0.16)

	/* inverse of H() */
#define H_inverse(t) ( ((t) <= 0.08) ? (t)/9.033 : POW(((t) + 0.16)/1.16, 3.0) )

#define NUMFINE 25

typedef struct xfer_s {			/* transfer-table object */
	double		nonlinear[NUMFINE];	/* nonlinear (e.g., device) coordinates */
	double		linear[NUMFINE];	/* linear (e.g., radiant flux) coordinates */
	double_p	from;				/* source selector */
	double_p	to;					/* destination selector */
} xfer_t, FAR* xfer_p;

#define ALLOC(number,size)	(allocBufferPtr((KpInt32_t)((number)*(size))))
#define	DALLOC(ptr)	(freeBufferPtr((KpChar_p)(ptr)), (ptr) = NULL)	/* deallocate memory */

#define	SLOPE_COUNT 128		/* limit the slope if less than SLOPE_COUNT */
#define	SLOPE_LIMIT 16		/* limit the slope to SLOPE_LIMIT */

#define	SCALEFIXED	65536.0			/* 2^16 */
#define	SCALEDOT16	65536.0			/* 2^16 */
#define	SCALEDOT15	32768.0			/* 2^15 */
#define	SCALEDOT8	  256.0			/* 2^8  */

	/*	This scale factor is applied to the X, Y, and Z values prior to
		the 12-bit quantization so that, when shifted left 4 bits,
		it will produce an unsigned 16-bit number with one integer bit,
		as per the ICC profile format.  It is *approximately* 
		1/2:  the exact number is 2^15/(4080 << 4), since 1.0 is represented 
		as 2^15 and FUT_MAX_PEL12 (= 4080) is the quantization scale factor.
	*/
#define	XYZSCALE_V0	0.50196078431373	/* = 2^15 / (FUT_MAX_PEL12 << 4) */

/* but now that's gone */
#define	XYZSCALE	0.5	/* = 2^15 / 2^16 */

typedef struct tagMATRIXDATA {
	KpUInt16_t			dim;		/* dimension (= 3) */
	double_h			matrix;		/* i.e., matrix[dim][dim] */
	ResponseRecord_h	response;	/* i.e., response[dim] */
} MATRIXDATA, FAR* LPMATRIXDATA;

#define RRECORD_DATA_SIZE 65536


PTErr_t calcOtblLSN (fut_otbldat_p, ResponseRecord_p);
PTErr_t calcOtblLS1 (fut_otbldat_p, double);
PTErr_t calcOtblLN (fut_otbldat_p, ResponseRecord_p);
PTErr_t calcOtblL1 (fut_otbldat_p, double);
PTErr_t makeForwardXformMono (ResponseRecord_p, fut_p);
PTErr_t makeInverseXformMono (ResponseRecord_p, fut_p);
fut_otbldat_t otblFunc (fut_gtbldat_t, fut_calcData_p);
PTErr_t makeOutputMatrixXform (Fixed_p matrix, KpUInt32_t gridsize, fut_p *theFut);
PTErr_t makeForwardXformFromMatrix (LPMATRIXDATA, KpUInt32_t, KpInt32_p, fut_p);
PTErr_t makeInverseXformFromMatrix (LPMATRIXDATA, KpUInt32_t, KpInt32_p, fut_p);
void makeMonotonic (KpUInt32_t count, KpUInt16_p table);
void makeInverseMonotonic (KpUInt32_t count, KpUInt16_p table);
double f4l (double x, double xtab[], double ytab[], KpInt32_t n, KpInt32_p hint);
void calcGtbl3 (fut_gtbldat_p*, KpInt32_p, double_h, double);
void calcItbl1 (fut_itbldat_p, KpInt32_t, double);
void calcItbl256 (fut_itbldat_p, KpInt32_t, KpUInt16_p);
PTErr_t calcItblN (fut_itbldat_p table, KpInt32_t gridSize, ResponseRecord_p rrp, KpUInt32_t interpMode);
void calcOtbl0 (fut_otbldat_p);
void calcOtbl1 (fut_otbldat_p, double);
PTErr_t calcOtblN (fut_otbldat_p table, ResponseRecord_p rrp, KpUInt32_t interpMode);
double calcInvertTRC (double p, KpUInt16_p data, KpUInt32_t length);
KpInt32_t solvemat (KpInt32_t, double_h, double_p);
PTErr_t init_xfer (xfer_p, ResponseRecord_p);
PTErr_t set_xfer (xfer_p, KpInt32_t, KpInt32_t);
double xfer (xfer_p, double, KpInt32_p);
fut_p get_lab2xyz (KpInt32_t);
fut_p get_xyz2lab (KpInt32_t);
fut_p get_linlab_fut (KpInt32_t, double, PTDataClass_t, PTDataClass_t);
fut_p get_idenMonCurv_fut (KpInt32_t, double, double);

#endif	/* _MAKEFUTS_H_ */

