/*
 * @(#)matdata.h	1.8 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 	@(#)matdata.h	1.16 98/07/07

	Contains:	interface definitions for
			make{Forward|Inverse}XformFromMatrix

	Written by:	The Boston White Sox

	COPYRIGHT (c) 1993-1998 Eastman Kodak Company
	As an unpublished work pursuant to Title 17 of the United
	States Code.  All rights reserved.
 */


#ifndef MATRIXDATA_H
#define MATRIXDATA_H

#include "kcmptlib.h"
#include "fut.h"
#include "fut_util.h"

/*-----------------------------------------------------------
 *  MATRIXDATA -- internal data format 
 *-----------------------------------------------------------
 */

typedef struct tagMATRIXDATA {
	KpUInt16_t	dim;				/* dimension (= 3) */
	double	FAR* FAR* matrix;		/* i.e., matrix[dim][dim] */
	ResponseRecord_h response;	/* i.e., response[dim] */
} MATRIXDATA, FAR* LPMATRIXDATA;


/*-----------------------------------------------------------
 *  function prototypes
 *-----------------------------------------------------------
 */

PTErr_t makeProfileXform (
			FixedXYZColor_p rXYZ, FixedXYZColor_p gXYZ, FixedXYZColor_p bXYZ, 
			ResponseRecord_p rTRC, ResponseRecord_p gTRC, ResponseRecord_p bTRC, 
			KpUInt32_t gridsize, KpBool_t invert, newMGmode_p newMGmodeP, fut_p FAR* theFut);


PTErr_t makeOutputMatrixXform 
			(Fixed_p matrix, KpUInt32_t gridsize, fut_p *theFut);

PTErr_t makeForwardXformFromMatrix (
			LPMATRIXDATA mdata, KpUInt32_t gridsize, KpUInt32_t interpMode, fut_p * theFut);

PTErr_t makeInverseXformFromMatrix (
			LPMATRIXDATA mdata, KpUInt32_t gridsize, KpUInt32_t interpMode, fut_p * theFut);

void makeMonotonic (KpUInt32_t count, KpUInt16_p table);

void makeInverseMonotonic (KpUInt32_t count, KpUInt16_p table);

KpInt32_t
	kcpIsFPUpresent (void);

/*-----------------------------------------------------------
 *  useful macros
 *-----------------------------------------------------------
 */

#define	SCALEFIXED	65536.0			/* 2^16 */
#define	SCALEDOT16	65536.0			/* 2^16 */
#define	SCALEDOT15	32768.0			/* 2^15 */
#define	SCALEDOT8	  256.0			/* 2^8  */
	/*
		This scale factor is applied to the X, Y, and Z values prior to
		the 12-bit quantization so that, when shifted left 4 bits,
		it will produce an unsigned 16-bit number with one integer bit,
		as per the ColorSync open profile format.  It is *approximately* 
		1/2:  the exact number is 2^15/(4080 << 4), since 1.0 is represented 
		as 2^15 and FUT_MAX_PEL12 (= 4080) is the quantization scale factor.
	*/


#endif	/* MATRIXDATA_H */
