/*
 * @(#)CompositeFont.h	1.10 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */

#ifndef CompositeFont_header
#define CompositeFont_header

#include "FontGlue.h"

class CompositeFont : public fontObject {
private:
    virtual const void* ReadChunk(UInt32 offset,
                                  UInt32 length, void* chunk = nil);
    virtual void readBlock(size_t offs, size_t size, char *dest);
    virtual void ReleaseChunk(const void* chunk);
    virtual Boolean MatchName(int nameID, const UInt16 *name, int nameLen);
    virtual int GetName(UInt16& platformID, UInt16& scriptID,
                        UInt16& languageID, UInt16& nameID, UInt16 *name);
    virtual const Unicode *GetFontName(int& nameLen);
    virtual UInt32 GetDataLength();

    CompositeFont();                // no implementation
    CompositeFont(CompositeFont&);  // no implementation
    CompositeFont& operator =(const CompositeFont&);

private:
    jchar *fCompositeName;
    int   fNameLen;
    jint  *fRanges;
    jint  *fMaxIndices;
    jchar **fComponentNames;
    int   *fComponentNameLengths;
    fontObject **fComponentFonts;
    const void **fComponentChunks;  // Used to keep open the component fonts
    jint  fNumComponents;
    int   fInitialized;

public:
    virtual ~CompositeFont();
    CompositeFont(JNIEnv *env, jstring compositeName);
    virtual void addRanges(JNIEnv *env, jintArray exclusionRanges);
    virtual void addMaxIndices(JNIEnv *env, jintArray maxIndices);
    virtual void registerComponentName(int slot, JStringBuffer& componentName);
    virtual int countSlots(void);
    virtual fontObject *getSlotFont(int slot);
    virtual int isValid(void);
    virtual int isExcludedChar(int slot, Unicode unicode);
};


#endif // CompositeFont_header
