/*
 * @(#)Strike.cpp	1.41 01/02/09
 *
 * Copyright 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */

#include "Strike.h"
#include "CompositeFont.h"
#include <string.h>
#include <java_awt_Font.h>

//////////////////////////////////////////////////////////////////////////////

const unsigned int SLOTMASK     = 0xff000000;
const unsigned int GLYPHMASK    = 0x00ffffff;
#define GLYPHSLOT(g)            ((g >> 24) & 0xff)
#define GLYPHCODE(s, g) \
        (( (unsigned int)s << 24) | ((unsigned int)g & GLYPHMASK))

class CompositeGlyphMapper : public CharToGlyphMapper {
private:
    class GlyphMapEntry {
    public:
        int     fSlot;
        UInt32  fGlyphCode;
    };

private:
    CompositeFont&  fCompositeFont;
    const Strike&   fStrike;
    UInt32          fMissingGlyph;
    int             fNumSlots;
    GlyphMapEntry   *fGlyphMaps[256];

private:
    CompositeGlyphMapper();                         // no implementation
    CompositeGlyphMapper(CompositeGlyphMapper&);    // no implementation
    CompositeGlyphMapper& operator =(const CompositeGlyphMapper&);
    UInt32 getCachedGlyphCode(const Unicode unicode) const;
    void setCachedGlyphCode(const Unicode unicode,
                            const UInt32 glyphCode,
                            const int slot) const;

public:
    virtual ~CompositeGlyphMapper() {
        for (int i=0; i < 256; i++) {
            if (fGlyphMaps[i]) {
                delete [] fGlyphMaps[i];
                fGlyphMaps[i] = NULL;
            }
        }
    };

    CompositeGlyphMapper(CompositeFont& cf, const Strike& strike)
            : fCompositeFont(cf), fStrike(strike) {
        fMissingGlyph = (UInt32)-1;
        fNumSlots = fCompositeFont.countSlots();
        memset(fGlyphMaps, 0, sizeof(fGlyphMaps));
    };

    virtual void CharsToGlyphs(
        int count, const Unicode unicodes[], UInt32 glyphs[]) const;
    virtual Boolean canDisplay(Unicode ch) const;
    virtual int getMissingGlyphCode() const;
};
//////////////////////////////////////////////////////////////////////////////

Strike::Strike( fontObject& fo, FontTransform& tx,
        jboolean isAntiAliased, jboolean usesFractionalMetrics)
    :   fFont(fo),
	strike(NULL),
	compositeStrike(NULL),
	isComposite(JNI_FALSE),
	fMapper(NULL), 
	fCachedLineMetrics(JNI_FALSE),
        fStrikeDescs(NULL),
        totalCompositeGlyphs(0),
	baseGlyphs(0)
{
    enum FontFormats format = fFont.GetFormat();
    enum FontFormats scalerID = ((fontObject&)fFont).getScalerID(
                                            tx,
                                            isAntiAliased,
                                            usesFractionalMetrics);
    if (format == kCompositeFontFormat) {
        isComposite = JNI_TRUE;
        templateDesc.fFontObj = NULL;
        tx.getMatrixInto(templateDesc.fMatrix, 4);
        templateDesc.fDoAntiAlias = isAntiAliased;
        templateDesc.fDoFracEnable = usesFractionalMetrics;
        templateDesc.fStyle = fFont.m_currentStyle;
        return;
    }

    UInt32  sizes[2] = { sizeof(UInt32), sizeof(FontStrikeDesc) };

    hsDescriptor desc = hsDescriptor_New(2, sizes);
    FontStrikeDesc* javaDesc;

    hsDescriptor_Add32(desc, kScalerID_FontScalerDesc, scalerID);
    javaDesc = (FontStrikeDesc *)hsDescriptor_Add(desc,
                                                kFontStrikeDescTag,
                                                sizeof(FontStrikeDesc));
    javaDesc->fFontObj = (fontObject *)&fFont;
    tx.getMatrixInto(javaDesc->fMatrix, 4);
    javaDesc->fDoAntiAlias = isAntiAliased;
    javaDesc->fDoFracEnable = usesFractionalMetrics;
    javaDesc->fStyle = fFont.m_currentStyle;

    hsDescriptor_UpdateCheckSum(desc);

    strike = hsGGlyphStrike::RefStrike(desc);

    hsDescriptor_Delete(desc);
}

Strike::Strike( fontObject& fo, enum FontFormats scalerID, FontTransform& tx,
        jboolean isAntiAliased, jboolean usesFractionalMetrics)
    :   fFont(fo), strike(NULL), compositeStrike(NULL), isComposite(JNI_FALSE),
	fMapper(NULL), fCachedLineMetrics(JNI_FALSE),
	fStrikeDescs(NULL), totalCompositeGlyphs(0), baseGlyphs(0)
{
    enum FontFormats format = fFont.GetFormat();
    if (format == kCompositeFontFormat) {
        isComposite = JNI_TRUE;
        templateDesc.fFontObj = NULL;
        tx.getMatrixInto(templateDesc.fMatrix, 4);
        templateDesc.fDoAntiAlias = isAntiAliased;
        templateDesc.fDoFracEnable = usesFractionalMetrics;
        templateDesc.fStyle = fFont.m_currentStyle;
        return;
    }

    UInt32  sizes[2] = { sizeof(UInt32), sizeof(FontStrikeDesc) };

    hsDescriptor desc = hsDescriptor_New(2, sizes);
    FontStrikeDesc* javaDesc;

    hsDescriptor_Add32(desc, kScalerID_FontScalerDesc, scalerID);
    javaDesc = (FontStrikeDesc *)hsDescriptor_Add(desc,
                                                kFontStrikeDescTag,
                                                sizeof(FontStrikeDesc));
    javaDesc->fFontObj = (fontObject *)&fFont;
    tx.getMatrixInto(javaDesc->fMatrix, 4);
    javaDesc->fDoAntiAlias = isAntiAliased;
    javaDesc->fDoFracEnable = usesFractionalMetrics;
    javaDesc->fStyle = fFont.m_currentStyle;

    hsDescriptor_UpdateCheckSum(desc);

    strike = hsGGlyphStrike::RefStrike(desc);

    hsDescriptor_Delete(desc);
}

Strike::~Strike() {
    if (strike) {
        hsRefCnt_SafeUnRef(strike);
        strike = NULL;
    } else {
        if (fMapper) {
            delete fMapper;
        }
    }
    if (compositeStrike) {
        CompositeFont *cf = (CompositeFont *)&fFont;
        int numSlots = cf->countSlots();
        for (int i=0; i < numSlots; i++) {
            if (compositeStrike[i]) {
                delete compositeStrike[i];
                compositeStrike[i] = NULL;
            }
        }
        if (compositeStrike != defaultStrikes) {
            delete [] compositeStrike;
        }
        compositeStrike = NULL;
    }
    if (fStrikeDescs) {
        if (fStrikeDescs != defaultStrikeDescs) {
            delete [] fStrikeDescs;
        }
        fStrikeDescs = NULL;
    }
}

jfloat Strike::getCharAdvanceX(Unicode ch) {
    jfloat retval = 0;
    // REMIND: removing const --- fix method declaration and implementation
    return getGlyphAdvanceX(CharToGlyph(ch));
}

jfloat Strike::getGlyphAdvanceX(int glyphCode) {
    jfloat retval = 0;
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        hsFixedPoint2 glyphAdvance;
        theStrike->GetMetrics(glyphCode, nil, &glyphAdvance);
        retval = hsFixedToFloat(glyphAdvance.fX);
    }
    return retval;
}

Boolean
Strike::getMetrics(int glyphCode, hsGGlyph& glyphRef, hsFixedPoint2& advXY)
{
    Boolean retval = false;
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        theStrike->GetMetrics(glyphCode, &glyphRef, &advXY);
        retval = true;
    }
    return retval;
}

Boolean
Strike::getMetricsWithImage(int glyphCode, hsGGlyph& glyphRef, hsFixedPoint2& advXY, Boolean* refcheck)
{
    Boolean retval = false;
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        theStrike->GetMetricsWithImage(glyphCode, &glyphRef, &advXY, refcheck);
        retval = true;
    }
    return retval;
}

const void *
Strike::getImage(int glyphCode, Boolean* refcheck)
{
    const void *retval = NULL;
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        retval = theStrike->GetImage(glyphCode, refcheck);
    }
    return retval;
}

const void *
Strike::refImage(int glyphCode)
{
    const void *retval = NULL;
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        retval = theStrike->RefImage(glyphCode);
    }
    return retval;
}

void
Strike::prepareCache(int numGlyphs, Boolean* purged) {
  hsGGlyphStrike *theStrike = strike;
  if (!theStrike) {
    // assume strike 0 will do
    theStrike = compositeStrikeForSlot(0);
  }
  if (theStrike) {
    theStrike->PrepareCache(numGlyphs, purged);
  }
}

static void useMax(hsFixedPoint2& target, hsFixedPoint2& src) {
    if (src.fX > target.fX) {
        target.fX = src.fX;
    }
    if (src.fY > target.fY) {
        target.fY = src.fY;
    }
}

static void useMin(hsFixedPoint2& target, hsFixedPoint2& src) {
    if (src.fX < target.fX) {
        target.fX = src.fX;
    }
    if (src.fY < target.fY) {
        target.fY = src.fY;
    }
}

void
Strike::GetLineHeight(hsFixedPoint2& ascent,
                      hsFixedPoint2& descent,
                      hsFixedPoint2& baseline,
                      hsFixedPoint2& leading,
                      hsFixedPoint2& maxAdvance)
{
    if (fCachedLineMetrics) {
        ascent      = fAscent;
        descent     = fDescent;
        baseline    = fBaseline;
        leading     = fLeading;
        maxAdvance  = fMaxAdvance;
        return;
    }

    if (strike) {
        strike->GetLineHeight(&ascent, &descent,
                              &baseline, &leading, &maxAdvance);
    } else {
        ascent.fX       = 0;
        ascent.fY       = 0;
        descent.fX      = 0;
        descent.fY      = 0;
        baseline.fX     = 0;
        baseline.fY     = 0;
        leading.fX      = 0;
        leading.fY      = 0;
        maxAdvance.fX   = 0;
        maxAdvance.fY   = 0;
        if (isComposite) {
            CompositeFont *cf = (CompositeFont *)&fFont;
            int numSlots = cf->countSlots();
            for (int i=0; i < numSlots; i++) {
                hsFixedPoint2 lAscent = {0, 0};
                hsFixedPoint2 lDescent = {0, 0};
                hsFixedPoint2 lBaseline = {0, 0};
                hsFixedPoint2 lLeading = {0, 0};
                hsFixedPoint2 lMaxAdvance = {0, 0};

                hsGGlyphStrike *currentStrike = compositeStrikeForSlot(i);
                if (currentStrike) {
                    currentStrike->GetLineHeight(&lAscent, &lDescent,
                                                &lBaseline,
                                                &lLeading, &lMaxAdvance);
                }
                useMin(ascent,      lAscent);
                useMax(descent,     lDescent);
                useMax(baseline,    lBaseline);
                useMin(leading,     lLeading);
                useMax(maxAdvance,  lMaxAdvance);
            }
        }
    }
    fCachedLineMetrics = JNI_TRUE;
    fAscent      = ascent;
    fDescent     = descent;
    fBaseline    = baseline;
    fLeading     = leading;
    fMaxAdvance  = maxAdvance;
}

void
Strike::GetItalicAngle(hsFixedPoint2& italicAngle)
{
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
		int glyphCode = 0;
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        theStrike->GetItalicAngle(&italicAngle);
    } else {
        italicAngle.fX = 0;
        italicAngle.fY = 0;
    }
}

int
Strike::canRotate () 
{
    int retval = sun_awt_font_NativeFontWrapper_ROTATE_FALSE;

    if (!strike) {
	retval = compositeCanRotate ();
    } else {
	retval = strike->canRotate ();
    }

    return retval;
}

Boolean
Strike::canRotateString (int fNumGlyphs, UInt32 *glyphs) 
{
    Boolean retval = true;

    for (int i = 0; (i < fNumGlyphs) && (retval); i++) {
	int glyphCode = glyphs [i];
        hsGGlyphStrike *theStrike = strike;
        if (!theStrike) {
	    theStrike = compositeStrikeForGlyph (glyphCode);
	}
	if (theStrike) {
	    retval = theStrike->canRotate ();
	} else {
	    retval = false;
	}
    }
    return retval;
}
	
void
Strike::GetGlyphOutline(int glyphCode, hsPathSpline& path)
{
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
        theStrike->GetPath(glyphCode, &path, nil);
    }
}

int Strike::GetNumGlyphs()
{
    int retval = 0;
    if (strike) {
        retval = strike->CountGlyphs();
    } else if (isComposite) {
        if (!totalCompositeGlyphs) {
            CompositeFont *cf = (CompositeFont *)&fFont;
            int numSlots = cf->countSlots();
            for (int i=0; i < numSlots; i++) {
                hsGGlyphStrike *slotStrike = compositeStrikeForSlot(i);
                if (slotStrike) {
                    totalCompositeGlyphs += slotStrike->CountGlyphs();
                }
            }
        }
        retval = totalCompositeGlyphs;
    }
    return retval;
}

void
Strike::allocateStrikes()
{
    if (!compositeStrike) {
        CompositeFont *cf = (CompositeFont *)&fFont;
        int numSlots = cf->countSlots();
        if (numSlots <= DEFAULT_STRIKE_SLOTS) {
            compositeStrike = defaultStrikes;
        } else {
            compositeStrike = new Strike *[numSlots];
        }
        if (compositeStrike) {
            for (int i=0; i < numSlots; i++) {
                compositeStrike[i] = NULL;
            }
        }
    }
}

void
Strike::allocateDescs()
{
    if (!fStrikeDescs) {
        CompositeFont *cf = (CompositeFont *)&fFont;
        int numSlots = cf->countSlots();
        if (numSlots <= DEFAULT_STRIKE_SLOTS) {
            fStrikeDescs = defaultStrikeDescs;
        } else {
            fStrikeDescs = new FontStrikeDesc[numSlots];
        }
        if (fStrikeDescs) {
            memset(fStrikeDescs, 0, numSlots * sizeof(FontStrikeDesc));
        }
    }
}

static int
CompAsciiUnicode(const char *aStr, const Unicode *uStr, int len) {
    if (!len || !aStr || !uStr) {
        return 0;
    }
    while (*aStr && len--) {
        if (*aStr++ != *uStr++) {
            return 0;
        }
    }
    return (*aStr == 0);
}


static Boolean EqualUnicodeStrings ( const Unicode *str1, int str1Len, 
				     const Unicode *str2, int str2Len )
{
  if ( str1Len == str2Len ) {
    for ( int i = 0; i < str1Len; i++) {
      if ( str1[i] != str2[i] )
	return false;
    }
    return true;
  }
  return false;
}

static void printUnicode(const UInt16 *name, int nameLen) {
    for (int i=0; i < nameLen; i++) {
        printf( "%c", (char)name[i]);
    }
    printf ("\n" );
}

int
Strike::algorithmicStyle(
        const CompositeFont& cf, const fontObject& fo, int slot ) {
    int cfNameLen;
    const Unicode *cfName = ((fontObject&)cf).GetFontName(cfNameLen);
    if (!cfName) {
        return java_awt_Font_PLAIN;
    }
    for (int index=0; index < cfNameLen; index++) {
        if (cfName[index] == '.') {
            // it is a styled composite font
            fontObject *foPlain =
                ::FindFontObject(cfName, index, java_awt_Font_PLAIN);
            if (foPlain && (foPlain->GetFormat() == kCompositeFontFormat)) {
                // and root is also composite font
                CompositeFont *cfPlain = (CompositeFont *)foPlain;
                fontObject *foPlainSlot = cfPlain->getSlotFont(slot);
                if (foPlainSlot == &fo) {
                    // and root slot == styled slot
                    if (CompAsciiUnicode(".bolditalic",
                                    &cfName[index], cfNameLen-index)) {
                        return java_awt_Font_BOLD | java_awt_Font_ITALIC;
                    }
                    if (CompAsciiUnicode(".bold",
                                    &cfName[index], cfNameLen-index)) {
                        return java_awt_Font_BOLD;
                    }
                    if (CompAsciiUnicode(".italic",
                                    &cfName[index], cfNameLen-index)) {
                        return java_awt_Font_ITALIC;
                    }
                }
            }
            return java_awt_Font_PLAIN;
        }
    }
    return java_awt_Font_PLAIN;
}



void
Strike::needsAlgorithmicStyle(
        const CompositeFont& cf, int slot, Boolean *algorithmic) {
    int cfNameLen;
    const Unicode *cfName = ((fontObject&)cf).GetFontName(cfNameLen);
    if (!cfName) {
        return;
    }
    *algorithmic = false;
    //    printf ( " The slot being compared is = %d\n", slot );
    //    printf ( " The font name = " );
    //    printUnicode ( cfName, cfNameLen );
    fontObject *foSlot = ((CompositeFont& ) cf).getSlotFont(slot);
    int styleLength =0;
    const Unicode *styleName = foSlot->GetFontName( styleLength );
    //    printf ( "The slot name is = " );
    //    printUnicode ( styleName, styleLength );


    for (int index=0; index < cfNameLen; index++) {
        if (cfName[index] == '.') {
            // it is a styled composite font
	    // printf( " The font name being passed to FFO = " );
	    //   printUnicode ( cfName, index );
	    fontObject *foPlain = ::FindFontObject(cfName, index, java_awt_Font_PLAIN);
            if ( !foPlain) return;
	    if ( foPlain->GetFormat() != kCompositeFontFormat ) {
	      // The japanese font.properties files names plain styles
	      // differently as dialog.plain.
	      // thus if it not a composite font - then there is a problem
	      // try appending the .plain to the name and get the font
	      // it needs to be a composite font
	        UInt16  tryPlainStr[256];
		int tryPlainStrLen = index;
		const char *plainStr = ".plain";

                int j;
		for (j = 0; j < index; j++) {
		    tryPlainStr[j] = cfName[j];
		}
		for ( j = 0; j < strlen(plainStr) ; j++ ) {
		    tryPlainStr[j + index] = plainStr[j];
		}
		tryPlainStrLen += strlen(plainStr);
		// printf ( "The font plain object was not composite\n" );
		// printf ( "Try with the string " );
		// printUnicode ( tryPlainStr, tryPlainStrLen );
		foPlain = FindFontObject(tryPlainStr, tryPlainStrLen, java_awt_Font_PLAIN);
		// printf ( " Did another try with .plain and name = " );
		// int l = 0;
		// const Unicode *nm;
		// nm = foPlain->GetFontName(l);
		// printUnicode ( nm, l );
	    }
	      

            if (foPlain) {
	        fontObject *foMatchingSlot = foPlain;
	        if ( foPlain->GetFormat() == kCompositeFontFormat ) {
	            foMatchingSlot = ((CompositeFont *) foPlain)->getSlotFont(slot);
	        }
		int plainLength = 0;
		const Unicode *plainName = foMatchingSlot->GetFontName(plainLength );
		// printf ( "The matching slot name is = " );
		// printUnicode ( plainName, plainLength );

		if (EqualUnicodeStrings (plainName, plainLength, styleName, styleLength )) {
                    // and root slot == styled slot
                    if (CompAsciiUnicode(".bolditalic",
                                    &cfName[index], cfNameLen-index)) {
		        *algorithmic = true;
                    }
                    else if (CompAsciiUnicode(".bold",
                                    &cfName[index], cfNameLen-index)) {
		        // printf ( " we are setting the flag here in bold\n" );
		        *algorithmic = true;
                    }
                    else if (CompAsciiUnicode(".italic",
                                    &cfName[index], cfNameLen-index)) {
		        *algorithmic = true;
                    }
                }
	    }
	    break;
        }
    }
}

hsGGlyphStrike *
Strike::compositeStrikeForSlot(int slot)
{
    hsGGlyphStrike *retval = NULL;
    CompositeFont *cf = (CompositeFont *)&fFont;
    int numSlots = cf->countSlots();
    if ((slot < 0) || (slot >= numSlots)) {
        return retval;
    }
    if (!compositeStrike) {
        allocateStrikes();
        if (!compositeStrike) {
            return retval;
        }
    }
    Strike *slotStrike = compositeStrike[slot];
    if (!slotStrike) {
        if (!fStrikeDescs) {
            allocateDescs();
        }
        if (!fStrikeDescs) {
            return retval;
        }
        FontStrikeDesc& desc = fStrikeDescs[slot];
        if (!desc.fFontObj) {
            desc = templateDesc;
            desc.fFontObj = cf->getSlotFont(slot);
            if (desc.fFontObj) {
                // if this is a styled composite font and this
                // slot's fontObject is the same as the "plain"
                // equivalent composite font, apply algorithmic
                // styles with a transform.
                FontTransform tx(desc.fMatrix);
		
                desc.fFontObj->m_currentStyle = cf->m_currentStyle;
                if (desc.fFontObj->m_currentStyle == java_awt_Font_PLAIN) {
                    desc.fFontObj->m_currentStyle = 
                                algorithmicStyle(*cf, *desc.fFontObj, slot );
                } else {
		    Boolean algorithmic = false;
		    needsAlgorithmicStyle (*cf, slot, &algorithmic);
	            if ( algorithmic == true ) {
		      // NOTE :: HACK HACK HACK ---
		      // by turning on this flag we are essentially turning on t2k
		      // rasterizing which can algorithmically bold stuff
		      // Windows will not algorithmicize stuff by the API's we
		      // are using - it always returns you the plain version of the
		      // font - so we insert this flag and force this to go thru t2k.
		      // printf ( "Need to algorithmicize this style \n" );
		      desc.fDoFracEnable = true;
		    }
		}
                compositeStrike[slot] = new Strike(*desc.fFontObj, tx,
                                                      desc.fDoAntiAlias,
                                                      desc.fDoFracEnable);
		if (slot == 0) {
		  baseGlyphs = compositeStrike[0]->strike->CountGlyphs();
		}
            }
        }
        slotStrike = compositeStrike[slot];
    }
    if (slotStrike) {
        retval = slotStrike->strike;
    }
    return retval;
}

int
Strike::compositeCanRotate () 
{
    CompositeFont *cf = (CompositeFont *)&fFont;
    int numSlots = cf->countSlots();
    int retval = sun_awt_font_NativeFontWrapper_ROTATE_FALSE;
    if (numSlots > 0) {
	hsGGlyphStrike *tmpStrike = compositeStrikeForSlot (0);
	if (!tmpStrike) {
	    return retval;
   	}

	retval = tmpStrike->canRotate ();

        for (int i = 1; i < numSlots; i++) {
	    tmpStrike = compositeStrikeForSlot (i);
	    if (!tmpStrike) {
		return sun_awt_font_NativeFontWrapper_ROTATE_FALSE;
	    } else {
		int val = tmpStrike->canRotate ();
		if (val != retval) {
	    	    return sun_awt_font_NativeFontWrapper_ROTATE_CHECK_STRING;
		}
	    }
	}
    }

    return retval;
}

	
	

hsGGlyphStrike *
Strike::compositeStrikeForGlyph(int& glyphCode)
{
    hsGGlyphStrike *retval = NULL;
    if (glyphCode >= 0) {
        if (glyphCode < baseGlyphs) {
            // performance optimization
            retval = compositeStrike[0]->strike;
        } else if (glyphCode & SLOTMASK) {
            // assumes glyph code contains slot number
            retval = compositeStrikeForSlot(GLYPHSLOT(glyphCode));
            glyphCode &= GLYPHMASK;
        } else {
            // assumes glyph ranges for component fonts have been concatenated
            CompositeFont *cf = (CompositeFont *)&fFont;
            int numSlots = cf->countSlots();
            int slot = 0;
            for (; slot < numSlots; slot++) {
                hsGGlyphStrike *slotStrike = compositeStrikeForSlot(slot);
                int slotGlyphs = slotStrike->CountGlyphs();
                if (glyphCode < slotGlyphs) {
                    break;
                }
                glyphCode -= slotGlyphs;
            }
            retval = compositeStrikeForSlot(slot);
        }
    }
    return retval;
}

Boolean Strike::isStyleSupported(int aStyle)
{
    Boolean retCode = false;
    if ( NULL != strike ) {
        retCode = strike->isStyleSupported(aStyle);
    }
    return retCode;
}

hsPathSpline::Type Strike::GetPathType (int glyphCode) 
{
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        theStrike = compositeStrikeForGlyph(glyphCode);
    }
    if (theStrike) {
	return theStrike->GetPathType ();
    }

    return hsPathSpline::kQuadType;
}

UInt32 Strike::CharToGlyph(Unicode unicode)
{
    if (getMapper()) {
        UInt32 glyph;
        fMapper->CharsToGlyphs(1, &unicode, &glyph);
        return glyph;
    }
    return getMissingGlyphCode();
}

int Strike::getMissingGlyphCode() const
{
    if (getMapper()) {
        return fMapper->getMissingGlyphCode();
    }
    return 0;
}

Boolean Strike::canDisplay(Unicode ch)
{
    if (getMapper()) {
        return fMapper->canDisplay(ch);
    }
    return 0;
}

CharToGlyphMapper *Strike::getMapper() const
{
    if (fMapper) {
        return fMapper;
    }
    // override const for private cached readonly value
    Strike *myPtr = (Strike *)this;
    hsGGlyphStrike *theStrike = strike;
    if (!theStrike) {
        myPtr->fMapper = new CompositeGlyphMapper(
                                    (CompositeFont&)fFont, *this);
    } else {
        myPtr->fMapper = theStrike->getMapper();
    }
    return fMapper;
}

UInt32
CompositeGlyphMapper::getCachedGlyphCode(const Unicode unicode) const
{
    UInt32 hiByte = (unicode >> 8) & 0xff;
    if (fGlyphMaps[hiByte]) {
        UInt32 loByte = unicode & 0xff;
        if (fGlyphMaps[hiByte][loByte].fGlyphCode) {
            return fGlyphMaps[hiByte][loByte].fGlyphCode;
        }
    }
    return getMissingGlyphCode();
}

void
CompositeGlyphMapper::setCachedGlyphCode(const Unicode unicode,
                                         const UInt32 glyphCode,
                                         const int slot) const
{
    // not really const ... but changes no external behavior
    // except to boost performance...we override const to maintain
    // read-only cached state
    CompositeGlyphMapper *myPtr = (CompositeGlyphMapper *)this;
    UInt32 hiByte = (unicode >> 8) & 0xff;
    if (!fGlyphMaps[hiByte]) {
        myPtr->fGlyphMaps[hiByte] = new GlyphMapEntry[256];
        if (fGlyphMaps[hiByte]) {
            memset(myPtr->fGlyphMaps[hiByte], 0, 256 * sizeof(GlyphMapEntry));
        }
    }
    if (fGlyphMaps[hiByte]) {
        UInt32 loByte = unicode & 0xff;
        myPtr->fGlyphMaps[hiByte][loByte].fSlot      = slot;
        myPtr->fGlyphMaps[hiByte][loByte].fGlyphCode =
                                                GLYPHCODE(slot, glyphCode);
    }
}

void
CompositeGlyphMapper::CharsToGlyphs(
        int count, const Unicode unicodes[],
        UInt32 glyphs[]) const
{
    getMissingGlyphCode();
    if (count) {
        Strike& myStrike = (Strike&)fStrike;
        for (int i=0; i<count; i++) {

            glyphs[i] = getCachedGlyphCode(unicodes[i]);
            if (glyphs[i] != fMissingGlyph) {
                continue;
            }

            for (int slot = 0; slot < fNumSlots; slot++) {
                hsGGlyphStrike *slotStrike =
                                    myStrike.compositeStrikeForSlot(slot);
                if (    slotStrike
                        && !fCompositeFont.isExcludedChar(slot, unicodes[i]))
                {
                    int glyphCode = fMissingGlyph;
                    CharToGlyphMapper *strikeMapper =
                                            slotStrike->getMapper();
                    if (strikeMapper) {
                        glyphCode = strikeMapper->CharToGlyph(unicodes[i]);
                    }
                    if ((UInt32)glyphCode != fMissingGlyph) {
		        glyphs[i] = GLYPHCODE(slot, glyphCode);
                        setCachedGlyphCode(unicodes[i], glyphCode, slot);
                        break;
                    }
                }
            }
        }
    }
}

Boolean
CompositeGlyphMapper::canDisplay(Unicode ch) const
{
    UInt32 glyph = getMissingGlyphCode();
    CharsToGlyphs(1, &ch, &glyph);
    return ((int)glyph != getMissingGlyphCode());
}

int
CompositeGlyphMapper::getMissingGlyphCode() const
{
    if (((UInt32)-1) == fMissingGlyph) {
        // remove const for cached read only init
        CompositeGlyphMapper *myPtr = (CompositeGlyphMapper *)this;
        myPtr->fMissingGlyph = 0;
        Strike& myStrike = (Strike&)fStrike;
        hsGGlyphStrike *strike = myStrike.compositeStrikeForSlot(0);
        if (strike) {
            CharToGlyphMapper *strikeMapper = strike->getMapper();
            if (strikeMapper) {
                myPtr->fMissingGlyph = strikeMapper->getMissingGlyphCode();
            }
        }
    }
    return fMissingGlyph;
}
