/*
 * @(#)Strike.h	1.20 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * @author Charlton Innovations, Inc.
 */
#ifndef strike_header
#define strike_header

#include "FontGlue.h"
#include "hsGGlyphCache.h"
#include "hsGFontScaler.h"
#include "hsFixedTypes.h"

class fontObject;           // forward reference
class CompositeFont;        // forward reference
class CharToGlyphMapper;    // forward reference

// Strike --- encapsulate "strike" instance for font as determined
// by transform and rendering hints.
class Strike {
private:
    const fontObject& fFont;
    hsGGlyphStrike  *strike;
    Strike          **compositeStrike;
    int             isComposite;
private:
    // cached state
    CharToGlyphMapper *fMapper;

    // cached state to speed up line metrics repeat calls
    int           fCachedLineMetrics;
    hsFixedPoint2 fAscent;
    hsFixedPoint2 fDescent;
    hsFixedPoint2 fBaseline;
    hsFixedPoint2 fLeading;
    hsFixedPoint2 fMaxAdvance;

    // cached state needed for efficient implementation of compositeStrike
    FontStrikeDesc *fStrikeDescs;
    int totalCompositeGlyphs;
    int baseGlyphs;

    enum { DEFAULT_STRIKE_SLOTS = 6};

    // default buffers to avoid mallocs in the common cases
    FontStrikeDesc templateDesc;
    FontStrikeDesc defaultStrikeDescs[DEFAULT_STRIKE_SLOTS];
    Strike *defaultStrikes[DEFAULT_STRIKE_SLOTS];

private:
    // methods for compositeStrike
    void allocateStrikes();
    void allocateDescs();
    inline hsGGlyphStrike *compositeStrikeForGlyph(int& glyphCode);
    int compositeCanRotate ();

private:
    // methods for construction phases
    CharToGlyphMapper *getMapper() const;
private:
// make sure that caller must place these objects on the stack
// since they have internal state which is stack dependent.
// (actually this is a gentle hint to other programmers, since
// they can get around this if they want to)
    Strike();           // no implementation
    Strike(Strike &);   // no implementation
    Strike& operator =(const Strike&);
//    static void *operator new(size_t size);
//    static void operator delete(void *ptr);
public:
    Strike( fontObject& fo, FontTransform& tx,
            jboolean isAntiAliased, jboolean usesFractionalMetrics);
    Strike( fontObject& fo, enum FontFormats scalerID, FontTransform& tx,
            jboolean isAntiAliased, jboolean usesFractionalMetrics);
    ~Strike();
    jfloat getCharAdvanceX(Unicode ch);
    jfloat getGlyphAdvanceX(int glyphCode);
    Boolean getMetrics(int glyphCode, hsGGlyph& glyphRef,
                                                hsFixedPoint2& advXY);
    Boolean getMetricsWithImage(int glyphCode, hsGGlyph& glyphRef,
				hsFixedPoint2& advXY, Boolean* refcheck = 0);
    const void *getImage(int glyphCode, Boolean* refcheck = 0);
    const void *refImage(int glyphCode);
    void prepareCache(int numGlyphs, Boolean* refcheck = 0);
    void GetLineHeight(hsFixedPoint2& ascent,
                      hsFixedPoint2& descent,
                      hsFixedPoint2& baseline,
                      hsFixedPoint2& leading,
                      hsFixedPoint2& maxAdvance);
    void GetItalicAngle(hsFixedPoint2& italicAngle);
    void GetGlyphOutline(int glyphCode, hsPathSpline& path);
    int  GetNumGlyphs();

    int  algorithmicStyle(  const CompositeFont& cf,
                            const fontObject& fo,
                            int slot );
    void needsAlgorithmicStyle ( const CompositeFont& cf, 
				 int slot, Boolean *algorithmic );
				 
    Boolean isStyleSupported(int);
    Boolean canDisplay(Unicode ch);
    int canRotate ();
    Boolean canRotateString (int fNumGlyphs, UInt32 *glyphs);
    UInt32 CharToGlyph(Unicode unicode);
    int getMissingGlyphCode() const;
    Boolean doesGlyphShaping(void) const { return false; }

    hsGGlyphStrike *compositeStrikeForSlot(int slot);
    hsPathSpline::Type GetPathType (int glyphCode);
};

#endif // strike_header
