/*
 * @(#)truetype.h	1.11 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * TRUETYPE.H
 * Copyright (C) 1989-1998 all rights reserved by Type Solutions, Inc. Plaistow, NH, USA.
 * Author: Sampo Kaasila
 *
 * This software is the property of Type Solutions, Inc. and it is furnished
 * under a license and may be used and copied only in accordance with the
 * terms of such license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or otherwise
 * made available to any other person or entity except as allowed under license.
 * No title to and ownership of the software or intellectual property
 * therewithin is hereby transferred.
 *
 * This information in this software is subject to change without notice
 */
#ifndef __T2K_TRUETYPE__
#define __T2K_TRUETYPE__
#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif  /* __cplusplus */

/*
 * T2K internal font wide metric data type. used here and in T2KSBIT.H
 */
typedef struct {
	int			isValid;			
	int16		Ascender;			
	int16		Descender;			
	int16		LineGap;			
	uint16		maxAW;	
	F16Dot16	caretDx, caretDy;
} T2K_FontWideMetrics;

#include "tt_prvt.h"
#include "t2ksbit.h"
#include "glyph.h"
#ifdef ALGORITHMIC_STYLES
#include "shapet.h"
#endif
#include "t1.h"
#ifdef ENABLE_T2KE
#include "T2KEXEC.H"
#endif

typedef void (*StyleFuncPtr)( GlyphClass *glyph, tsiMemObject *mem, short UPEM, F16Dot16 params[] );
typedef void (*StyleMetricsFuncPtr)( hmtxClass *hmtx, tsiMemObject *mem, short UPEM, F16Dot16 params[] );
#define MAX_STYLE_PARAMS 4

typedef struct {
	/* private */
	sfnt_OffsetTable *offsetTable0;

#ifdef ENABLE_T1
	T1Class *T1;
#endif
#ifdef ENABLE_CFF
	CFFClass *T2;
#endif
#ifdef ENABLE_T2KE
	T2KEClass *T2KE;
#endif

#ifdef ENABLE_SBIT
	blocClass *bloc;
	ebscClass *ebsc;
	uint32 bdatOffset;
#endif

	ttcfClass *ttcf;
	
	headClass *head;
	maxpClass *maxp;
	locaClass *loca;
	hheaClass *hhea;
	hheaClass *vhea;
	hmtxClass *hmtx;
	cmapClass *cmap;
	kernClass *kern;
	uint16 preferedPlatformID, preferedPlatformSpecificID;
	
	void *globalHintsCache;
	StyleFuncPtr StyleFunc;
	StyleMetricsFuncPtr StyleMetricsFunc;
	F16Dot16 params[MAX_STYLE_PARAMS];
	
	
	InputStream *in;
	OutputStream *out;
	tsiMemObject *mem;
	
	/* OrionModelClass *model; */
	void *model;
	/* public */
	
} sfntClass;

hmtxClass *New_hmtxEmptyClass( tsiMemObject *mem, int32 numGlyphs, int32 numberOfHMetrics );
void Delete_hmtxClass( hmtxClass *t );

/* Some useful getter methods */
short GetUPEM( sfntClass *t);
short GetMaxPoints( sfntClass *t);

void GetFontWideOutlineMetrics( sfntClass *font, T2K_FontWideMetrics *hori, T2K_FontWideMetrics *vert );

typedef struct {
	StyleFuncPtr StyleFunc;
	StyleMetricsFuncPtr StyleMetricsFunc;
	F16Dot16 params[MAX_STYLE_PARAMS];
} T2K_AlgStyleDescriptor;

/* Caller does something like in = New_InputStream3( t->mem, data, length ); */
#define FONT_TYPE_1 1
#define FONT_TYPE_2 22
#define FONT_TYPE_TT_OR_T2K 2

#define New_sfntClass( mem, fontType, in, styling, errCode ) New_sfntClassLogical( mem, fontType, 0, in, styling, errCode )

sfntClass *New_sfntClassLogical( tsiMemObject *mem, short fontType, long fontNum, InputStream *in, T2K_AlgStyleDescriptor *styling, int *errCode );
#define CMD_GRID 2
#define CMD_TT_TO_T2K 3
#define CMD_T2K_TO_TT 4
#define CMD_HINT_ROMAN 5
#define CMD_HINT_OTHER 6
#define CMD_TT_TO_T2KE 7

sfntClass *New_sfntClass2( sfntClass *sfnt0, int cmd, int param );

void WriteToFile_sfntClass( sfntClass *t, const char *fname );
void Purge_cmapMemory( sfntClass *t );

#define tag_T2KG        		0x54324B47        /* 'T2KG' */
#define tag_T2KC        		0x54324B43        /* 'T2KC' */
sfnt_DirectoryEntry *GetTableDirEntry_sfntClass( sfntClass *t, long tag );
/* caller need to do Delete_InputStream on the stream */
InputStream *GetStreamForTable( sfntClass *t, long tag  );

void Delete_sfntClass( sfntClass *t, int *errCode );

/* 	mem									= tsi_NewMemhandler(); */
/* 	tsi_DeleteMemhandler( mem ); */

GlyphClass *GetGlyphByIndex( sfntClass *t, long index, char readHints, uint16 *aw );
GlyphClass *GetGlyphByCharCode( sfntClass *t, long charCode, char readHints, uint16 *aw );

int IsFigure( sfntClass *t, uint16 gIndex );

#ifdef ENABLE_KERNING
void GetSfntClassKernValue( sfntClass *t, uint16 leftGIndex, uint16 rightGIndex, int16 *xKern, int16 *yKern );
#endif /* ENABLE_KERNING */

uint16 GetSfntClassGlyphIndex( sfntClass *t, uint16 charCode );


long GetNumGlyphs_sfntClass( sfntClass *t );


#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif /* __T2K_TRUETYPE__ */

