/*
 * @(#)mlib_ImageAffineEdge.c	1.10 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifdef _MSC_VER
#pragma warning(disable:4068 4244 4101)
#endif /*_MSC_VER*/

#ifdef __SUNPRO_C
#pragma ident  "@(#)mlib_ImageAffineEdge.c	1.18  99/04/09 SMI"
#endif /* __SUNPRO_C */

/*
 * FUNCTION
 *      mlib_ImageAffineZeroEdge - implementation of MLIB_EDGE_DST_FILL_ZERO
 *                                 edge condition
 *
 * SYNOPSIS
 *      void mlib_ImageAffineEdgeZero(mlib_image *dst,
 *                                    mlib_s32 *leftEdges,
 *                                    mlib_s32 *rightEdges,
 *                                    mlib_s32 *sides,
 *                                    mlib_s32 *leftEdgesE,
 *                                    mlib_s32 *rightEdgesE,
 *                                    mlib_s32 *sidesE)
 *
 * ARGUMENTS
 *      dst         - pointer to destination image
 *      leftEdges   - array[dstHeight] of xLeft coordinates for the
 *                  corresponding interpolation (bilinear or bicubic)
 *      rightEdges  - array[dstHeight] of xRight coordinates for the
 *                  corresponding interpolation (bilinear or bicubic)
 *      sides       - array[4]. sides[0] is yStart, sides[1] is yFinish,
 *                  sides[2] is dx * 65536, sides[3] is dy * 65536 for
 *                  the corresponding interpolation (bilinear or bicubic)
 *      leftEdgesE  - array[dstHeight] of xLeft coordinates for the
 *                  nearest neighbour interpolation
 *      rightEdgesE - array[dstHeight] of xRight coordinates for the
 *                  nearest neighbour interpolation
 *      sidesE      - array[4]. sides[0] is yStart, sides[1] is yFinish,
 *                  sides[2] is dx * 65536, sides[3] is dy * 65536 for
 *                  nearest neighbour interpolation
 *
 * DESCRIPTION
 *      This function fills the edge pixels (i.e. thouse one which can not
 *      be interpolated with given resampling filter because their prototypes
 *      in the source image lie too close to the border) in the destination
 *      image with zeroes.
 *
 * FUNCTION
 *      mlib_ImageAffineEdgeNearest - implementation of
 *                                 edge condition MLIB_EDGE_OP_NEAREST
 *
 * SYNOPSIS
 *      void mlib_ImageAffineEdgeNearest(mlib_image *dst,
 *                                      mlib_s32 *leftEdges,
 *                                      mlib_s32 *rightEdges,
 *                                      mlib_s32 *sides,
 *                                      mlib_s32 *leftEdgesE,
 *                                      mlib_s32 *rightEdgesE,
 *                                      mlib_s32 *xStartsE,
 *                                      mlib_s32 *yStartsE,
 *                                      mlib_s32 *sidesE
 *                                      mlib_u8  **lineAddr)
 *
 * ARGUMENTS
 *      dst         - pointer to destination image
 *      leftEdges   - array[dstHeight] of xLeft coordinates for the
 *                  corresponding interpolation (bilinear or bicubic)
 *      rightEdges  - array[dstHeight] of xRight coordinates for the
 *                  corresponding interpolation (bilinear or bicubic)
 *      sides       - array[4]. sides[0] is yStart, sides[1] is yFinish,
 *                  sides[2] is dx * 65536, sides[3] is dy * 65536 for
 *                  the corresponding interpolation (bilinear or bicubic)
 *      leftEdgesE  - array[dstHeight] of xLeft coordinates for the
 *                  nearest neighbour interpolation
 *      rightEdgesE - array[dstHeight] of xRight coordinates for the
 *                  nearest neighbour interpolation
 *      xStartsE    - array[dstHeight] of xStart * 65536 coordinates
 *                  for the nearest neighbour interpolation
 *      yStartsE    - array[dstHeight] of yStart * 65536 coordinates
 *                  for the nearest neighbour interpolation
 *      sidesE      - array[4]. sides[0] is yStart, sides[1] is yFinish,
 *                  sides[2] is dx * 65536, sides[3] is dy * 65536 for
 *                  nearest neighbour interpolation
 *      lineAddr    - array[srcHeight] of pointers to the first pixel on
 *                  the corresponding lines in source image
 *
 * DESCRIPTION
 *      This function fills the edge pixels (i.e. thouse one which can not
 *      be interpolated with given resampling filter because their prototypes
 *      in the source image lie too close to the border) in the destination
 *      image according to the nearest neighbour interpolation.
 *
 */

#include <stdlib.h>
#include "mlib_image.h"
#include "mlib_ImageFilters.h"

extern mlib_d64 mlib_U82D64[];

void mlib_ImageAffineEdgeZero(mlib_image *dst,
                              mlib_s32 *leftEdges,
                              mlib_s32 *rightEdges,
                              mlib_s32 *sides,
                              mlib_s32 *leftEdgesE,
                              mlib_s32 *rightEdgesE,
                              mlib_s32 *sidesE)
{
  mlib_type type      = mlib_ImageGetType(dst);
  mlib_s32  stride    = mlib_ImageGetStride(dst);
  mlib_s32  channels  = mlib_ImageGetChannels(dst);
  mlib_u8   *data     = (mlib_u8 *)mlib_ImageGetData(dst);
  mlib_s32  yStart    = sides[0];
  mlib_s32  yFinish   = sides[1];
  mlib_s32  yStartE   = sidesE[0];
  mlib_s32  yFinishE  = sidesE[1];
  mlib_s32  xLeft, xRight, xLeftE, xRightE;
  mlib_s32  j;

  data += (yStartE - 1) * stride;

  switch (type)
  {
    case MLIB_BYTE:
    {
      mlib_u8 *dstPixelPtr;
      mlib_u8 *dstLineEnd;

      for (j = yStartE; j < yStart; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = data + channels * xLeftE;
        dstLineEnd   = data + channels * (xRightE + 1);

        for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
          *dstPixelPtr = 0;
      }

      for (; j <= yFinish; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        xLeft   = leftEdges[j];
        xRight  = rightEdges[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = data + channels * xLeftE;
          dstLineEnd   = data + channels * (xRightE + 1);

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;
        }
        else
        {
          dstPixelPtr = data + channels * xLeftE;
          dstLineEnd  = data + channels * xLeft;

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;

          dstPixelPtr = data + channels * (xRight + 1);
          dstLineEnd  = data + channels * (xRightE + 1);

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;
        }
      }

      for (; j <= yFinishE; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = data + channels * xLeftE;
        dstLineEnd   = data + channels * (xRightE + 1);

        for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
          *dstPixelPtr = 0;
      }
    }
    break;

    case MLIB_SHORT:
    {
      mlib_s16 *dstPixelPtr;
      mlib_s16 *dstLineEnd;

      channels <<= 1;

      for (j = yStartE; j < yStart; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + channels * (xRightE + 1));

        for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
          *dstPixelPtr = 0;
      }

      for (; j <= yFinish; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        xLeft   = leftEdges[j];
        xRight  = rightEdges[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s16 *)(data + channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + channels * (xRightE + 1));

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;
        }
        else
        {
          dstPixelPtr  = (mlib_s16 *)(data + channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + channels * xLeft);

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;

          dstPixelPtr = (mlib_s16 *)(data + channels * (xRight + 1));
          dstLineEnd  = (mlib_s16 *)(data + channels * (xRightE + 1));

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;
        }
      }

      for (; j <= yFinishE; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + channels * (xRightE + 1));

        for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
          *dstPixelPtr = 0;
      }
    }
    break;

    case MLIB_INT:
    {
      mlib_s32 *dstPixelPtr;
      mlib_s32 *dstLineEnd;

      channels <<= 2;

      for (j = yStartE; j < yStart; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + channels * (xRightE + 1));

        for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
          *dstPixelPtr = 0;
      }

      for (; j <= yFinish; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        xLeft   = leftEdges[j];
        xRight  = rightEdges[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s32 *)(data + channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + channels * (xRightE + 1));

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;
        }
        else
        {
          dstPixelPtr  = (mlib_s32 *)(data + channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + channels * xLeft);

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;

          dstPixelPtr = (mlib_s32 *)(data + channels * (xRight + 1));
          dstLineEnd  = (mlib_s32 *)(data + channels * (xRightE + 1));

          for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
            *dstPixelPtr = 0;
        }
      }

      for (; j <= yFinishE; j++)
      {
        xLeftE  = leftEdgesE[j];
        xRightE = rightEdgesE[j];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + channels * (xRightE + 1));

        for (; dstPixelPtr < dstLineEnd; dstPixelPtr++)
          *dstPixelPtr = 0;
      }
    }
    break;
  }
}

/***************************************************************/

#define MLIB_SHIFT   16

#define MLIB_EDGE_NEAREST_U8                                                        \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                   \
        {                                                                           \
          ySrc = (Y>>(MLIB_SHIFT));                                                 \
          xSrc = X>>MLIB_SHIFT;                                                     \
          srcPixelPtr = ((mlib_u8 **)lineAddr)[ySrc] + xSrc * channels;             \
                                                                                    \
          for (j = 0; j < channels; j++)                                            \
            dstPixelPtr[j] = srcPixelPtr[j];                                        \
                                                                                    \
          Y += dY;                                                                  \
          X += dX;                                                                  \
        }

#define MLIB_EDGE_NEAREST_S16                                                                 \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                             \
        {                                                                                     \
          ySrc = (Y>>(MLIB_SHIFT));                                                           \
          xSrc = X>>MLIB_SHIFT;                                                               \
          srcPixelPtr = ((mlib_s16 **)lineAddr)[ySrc] + xSrc * channels;                      \
                                                                                              \
          for (j = 0; j < channels; j++)                                                      \
            dstPixelPtr[j] = srcPixelPtr[j];                                                  \
                                                                                              \
          Y += dY;                                                                            \
          X += dX;                                                                            \
        }

#define MLIB_EDGE_NEAREST_S32                                                                 \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                             \
        {                                                                                     \
          ySrc = (Y>>(MLIB_SHIFT));                                                           \
          xSrc = X>>MLIB_SHIFT;                                                               \
          srcPixelPtr = ((mlib_s32 **)lineAddr)[ySrc] + xSrc * channels;                      \
                                                                                              \
          for (j = 0; j < channels; j++)                                                      \
            dstPixelPtr[j] = srcPixelPtr[j];                                                  \
                                                                                              \
          Y += dY;                                                                            \
          X += dX;                                                                            \
        }

/***************************************************************/

void mlib_ImageAffineEdgeNearest(mlib_image *dst,
                                mlib_s32 *leftEdges,
                                mlib_s32 *rightEdges,
                                mlib_s32 *sides,
                                mlib_s32 *leftEdgesE,
                                mlib_s32 *rightEdgesE,
                                mlib_s32 *xStartsE,
                                mlib_s32 *yStartsE,
                                mlib_s32 *sidesE,
                                mlib_u8  **lineAddr)
{
  mlib_type type      = mlib_ImageGetType(dst);
  mlib_s32  channels  = mlib_ImageGetChannels(dst);
  mlib_s32  stride    = mlib_ImageGetStride(dst);
  mlib_u8   *data     = (mlib_u8 *)mlib_ImageGetData(dst);
  mlib_s32  yStart    = sides[0];
  mlib_s32  yFinish   = sides[1];
  mlib_s32  yStartE   = sidesE[0];
  mlib_s32  yFinishE  = sidesE[1];
  mlib_s32  dX        = sidesE[2];
  mlib_s32  dY        = sidesE[3];
  mlib_s32  xSrc, ySrc, X, Y;
  mlib_s32  xLeft, xRight, xLeftE, xRightE;
  mlib_s32  i, j;

  data += (yStartE - 1) * stride;

  switch (type)
  {
    case MLIB_BYTE:
    {
      mlib_u8 *srcPixelPtr;
      mlib_u8 *dstPixelPtr;
      mlib_u8 *dstLineEnd;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (data + channels * xLeftE);
        dstLineEnd   = (data + channels * (xRightE + 1));

        MLIB_EDGE_NEAREST_U8
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (data + channels * xLeftE);
          dstLineEnd   = (data + channels * (xRightE + 1));

          MLIB_EDGE_NEAREST_U8
        }
        else
        {
          dstPixelPtr  = (data + channels * xLeftE);
          dstLineEnd   = (data + channels * xLeft);

          MLIB_EDGE_NEAREST_U8

          dstPixelPtr = (data + channels * (xRight + 1));
          dstLineEnd  = (data + channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_NEAREST_U8
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (data + channels * xLeftE);
        dstLineEnd   = (data + channels * (xRightE + 1));

        MLIB_EDGE_NEAREST_U8
      }
    }
    break;

    case MLIB_SHORT:
    {
      mlib_s16 *srcPixelPtr;
      mlib_s16 *dstPixelPtr;
      mlib_s16 *dstLineEnd;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

        MLIB_EDGE_NEAREST_S16
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

          MLIB_EDGE_NEAREST_S16
        }
        else
        {
          dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + 2 * channels * xLeft);

          MLIB_EDGE_NEAREST_S16

          dstPixelPtr = (mlib_s16 *)(data + 2 * channels * (xRight + 1));
          dstLineEnd  = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_NEAREST_S16
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

        MLIB_EDGE_NEAREST_S16
      }
    }
    break;

    case MLIB_INT:
    {
      mlib_s32 *srcPixelPtr;
      mlib_s32 *dstPixelPtr;
      mlib_s32 *dstLineEnd;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

        MLIB_EDGE_NEAREST_S32
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

          MLIB_EDGE_NEAREST_S32
        }
        else
        {
          dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + 4 * channels * xLeft);

          MLIB_EDGE_NEAREST_S32

          dstPixelPtr = (mlib_s32 *)(data + 4 * channels * (xRight + 1));
          dstLineEnd  = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_NEAREST_S32
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += stride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

        MLIB_EDGE_NEAREST_S32
      }
    }
    break;
  }
}

/***************************************************************/

#define MLIB_SIGN_SHIFT 31
#define MLIB_PREC       (1 << MLIB_SHIFT)
#define MLIB_MASK       (MLIB_PREC - 1)

#define MLIB_EDGE_EXTEND_BL_U8                                                      \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                   \
        {                                                                           \
          ySrc = ((Y - 32768) >> MLIB_SHIFT);                                       \
          xSrc = ((X - 32768) >> MLIB_SHIFT);                                       \
                                                                                    \
          t =  ((X - 32768) & MLIB_MASK) * scale;                                   \
          u = ((Y - 32768) & MLIB_MASK) * scale;                                    \
          k = t * u;                                                                \
                                                                                    \
          xDelta = (((xSrc + 1 - srcWidth)) >> MLIB_SIGN_SHIFT) & channels;         \
          yDelta = (((ySrc + 1 - srcHeight)) >> MLIB_SIGN_SHIFT) & srcStride;       \
                                                                                    \
          xFlag = (xSrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                         \
          xSrc = xSrc + (1 & xFlag);                                                \
          xDelta = xDelta &~ xFlag;                                                 \
                                                                                    \
          yFlag = (ySrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                         \
          ySrc = ySrc + (1 & yFlag);                                                \
          yDelta = yDelta &~ yFlag;                                                 \
                                                                                    \
          srcPixelPtr = ((mlib_u8 **)lineAddr)[ySrc] + xSrc * channels;             \
                                                                                    \
          for (j = 0; j < channels; j++)                                            \
          {                                                                         \
            a00   = *srcPixelPtr;                                                   \
            a01   = *(srcPixelPtr + xDelta);                                        \
            k0    = (a01 - a00) * t;                                                \
            a10   = *(srcPixelPtr + yDelta);                                        \
            k0   += (a10 - a00) * u;                                                \
            a11   = *(srcPixelPtr + yDelta + xDelta);                               \
            pix0  = k0 + (a00 - a01 + a11 - a10) * k;                               \
                                                                                    \
            dstPixelPtr[j] = a00 + (mlib_s32)pix0;                                  \
            srcPixelPtr++;                                                          \
          }                                                                         \
                                                                                    \
          X += dX;                                                                  \
          Y += dY;                                                                  \
        }

#define MLIB_EDGE_EXTEND_BL_S16                                                             \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                           \
        {                                                                                   \
          ySrc = ((Y - 32768) >> MLIB_SHIFT);                                               \
          xSrc = ((X - 32768) >> MLIB_SHIFT);                                               \
                                                                                            \
          t =  ((X - 32768) & MLIB_MASK) * scale;                                           \
          u = ((Y - 32768) & MLIB_MASK) * scale;                                            \
          k = t * u;                                                                        \
                                                                                            \
          xDelta = (((xSrc + 1 - srcWidth)) >> MLIB_SIGN_SHIFT) & channels;                 \
          yDelta = (((ySrc + 1 - srcHeight)) >> MLIB_SIGN_SHIFT) & (srcStride >> 1);        \
                                                                                            \
          xFlag = (xSrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                 \
          xSrc = xSrc + (1 & xFlag);                                                        \
          xDelta = xDelta &~ xFlag;                                                         \
                                                                                            \
          yFlag = (ySrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                 \
          ySrc = ySrc + (1 & yFlag);                                                        \
          yDelta = yDelta &~ yFlag;                                                         \
                                                                                            \
          srcPixelPtr = ((mlib_s16 **)lineAddr)[ySrc] + xSrc * channels;                    \
                                                                                            \
          for (j = 0; j < channels; j++)                                                    \
          {                                                                                 \
            a00   = *srcPixelPtr;                                                           \
            a01   = *(srcPixelPtr + xDelta);                                                \
            k0    = (a01 - a00) * t;                                                        \
            a10   = *(srcPixelPtr + yDelta);                                                \
            k0   += (a10 - a00) * u;                                                        \
            a11   = *(srcPixelPtr + yDelta + xDelta);                                       \
            pix0  = k0 + (a00 - a01 + a11 - a10) * k;                                       \
                                                                                            \
            dstPixelPtr[j] = a00 + (mlib_s32)pix0;                                          \
            srcPixelPtr++;                                                                  \
          }                                                                                 \
                                                                                            \
          X += dX;                                                                          \
          Y += dY;                                                                          \
        }

#define MLIB_EDGE_EXTEND_BL_S32                                                             \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                           \
        {                                                                                   \
          ySrc = ((Y - 32768) >> MLIB_SHIFT);                                               \
          xSrc = ((X - 32768) >> MLIB_SHIFT);                                               \
                                                                                            \
          t =  ((X - 32768) & MLIB_MASK) * scale;                                           \
          u = ((Y - 32768) & MLIB_MASK) * scale;                                            \
          k3 = t * u; k2 = (1.0 - t) * u;                                                   \
          k1 = t * (1.0 - u); k0 = (1.0 - t) * (1.0 - u);                                   \
                                                                                            \
          xDelta = (((xSrc + 1 - srcWidth)) >> MLIB_SIGN_SHIFT) & channels;                 \
          yDelta = (((ySrc + 1 - srcHeight)) >> MLIB_SIGN_SHIFT) & (srcStride >> 2);        \
                                                                                            \
          xFlag = (xSrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                 \
          xSrc = xSrc + (1 & xFlag);                                                        \
          xDelta = xDelta &~ xFlag;                                                         \
                                                                                            \
          yFlag = (ySrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                 \
          ySrc = ySrc + (1 & yFlag);                                                        \
          yDelta = yDelta &~ yFlag;                                                         \
                                                                                            \
          srcPixelPtr = ((mlib_s32 **)lineAddr)[ySrc] + xSrc * channels;                    \
                                                                                            \
          for (j = 0; j < channels; j++)                                                    \
          {                                                                                 \
            a00   = *srcPixelPtr;                                                           \
            a01   = *(srcPixelPtr + xDelta);                                                \
            a10   = *(srcPixelPtr + yDelta);                                                \
            a11   = *(srcPixelPtr + yDelta + xDelta);                                       \
                                                                                            \
            pix0 = k0 * a00  + k1 * a01 + k2 * a10 + k3 * a11;                              \
                                                                                            \
            dstPixelPtr[j] = (mlib_s32)pix0;                                                \
            srcPixelPtr++;                                                                  \
          }                                                                                 \
                                                                                            \
          X += dX;                                                                          \
          Y += dY;                                                                          \
        }

/***************************************************************/

void mlib_ImageAffineEdgeExtend_BL(mlib_image *dst,
                                  mlib_image *src,
                                  mlib_s32 *leftEdges,
                                  mlib_s32 *rightEdges,
                                  mlib_s32 *sides,
                                  mlib_s32 *leftEdgesE,
                                  mlib_s32 *rightEdgesE,
                                  mlib_s32 *xStartsE,
                                  mlib_s32 *yStartsE,
                                  mlib_s32 *sidesE,
                                  mlib_u8  **lineAddr)
{
  mlib_type type      = mlib_ImageGetType(dst);
  mlib_s32  channels  = mlib_ImageGetChannels(dst);
  mlib_s32  dstStride = mlib_ImageGetStride(dst);
  mlib_s32  srcStride = mlib_ImageGetStride(src);
  mlib_s32  srcWidth  = mlib_ImageGetWidth(src);
  mlib_s32  srcHeight = mlib_ImageGetHeight(src);
  mlib_u8   *data     = (mlib_u8 *)mlib_ImageGetData(dst);
  mlib_s32  yStart    = sides[0];
  mlib_s32  yFinish   = sides[1];
  mlib_s32  yStartE   = sidesE[0];
  mlib_s32  yFinishE  = sidesE[1];
  mlib_s32  dX        = sidesE[2];
  mlib_s32  dY        = sidesE[3];
  mlib_d64  scale     = 1.0 / 65536.0;
  mlib_s32  xSrc, ySrc, X, Y, xDelta, yDelta, xFlag, yFlag;
  mlib_s32  xLeft, xRight, xLeftE, xRightE;
  mlib_d64  t, u, k, k0, pix0;
  mlib_s32  a00, a01, a10, a11;
  mlib_s32  i, j;

  data += (yStartE - 1) * dstStride;

  switch (type)
  {
    case MLIB_BYTE:
    {
      mlib_u8 *srcPixelPtr;
      mlib_u8 *dstPixelPtr;
      mlib_u8 *dstLineEnd;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_u8 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_u8 *)(data + channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BL_U8
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_u8 *)(data + channels * xLeftE);
          dstLineEnd   = (mlib_u8 *)(data + channels * (xRightE + 1));

          MLIB_EDGE_EXTEND_BL_U8
        }
        else
        {
          dstPixelPtr  = (data + channels * xLeftE);
          dstLineEnd   = (data + channels * xLeft);

          MLIB_EDGE_EXTEND_BL_U8

          dstPixelPtr = (data + channels * (xRight + 1));
          dstLineEnd  = (data + channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_EXTEND_BL_U8
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_u8 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_u8 *)(data + channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BL_U8
      }
    }
    break;

    case MLIB_SHORT:
    {
      mlib_s16 *srcPixelPtr;
      mlib_s16 *dstPixelPtr;
      mlib_s16 *dstLineEnd;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BL_S16
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

          MLIB_EDGE_EXTEND_BL_S16
        }
        else
        {
          dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + 2 * channels * xLeft);

          MLIB_EDGE_EXTEND_BL_S16

          dstPixelPtr = (mlib_s16 *)(data + 2 * channels * (xRight + 1));
          dstLineEnd  = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_EXTEND_BL_S16
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BL_S16
      }
    }
    break;

    case MLIB_INT:
    {
      mlib_s32 *srcPixelPtr;
      mlib_s32 *dstPixelPtr;
      mlib_s32 *dstLineEnd;
      mlib_d64 k1, k2, k3;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BL_S32
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

          MLIB_EDGE_EXTEND_BL_S32
        }
        else
        {
          dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + 4 * channels * xLeft);

          MLIB_EDGE_EXTEND_BL_S32

          dstPixelPtr = (mlib_s32 *)(data + 4 * channels * (xRight + 1));
          dstLineEnd  = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_EXTEND_BL_S32
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BL_S32
      }
    }
    break;
  }
}

/***************************************************************/
#define FILTER_SHIFT_U8  4
#define FILTER_MASK_U8   (((1 << 8) - 1) << 4)
#define FILTER_SHIFT_S16 3
#define FILTER_MASK_S16  (((1 << 9) - 1) << 4)
#define sPtr srcPixelPtr

#ifdef MLIB_USE_FTOI_CLAMPING

#define SAT8(DST) \
  DST = ((mlib_s32)(val0 - sat) >> 24) ^ 0x80;

#define SAT16(DST) \
  DST = ((mlib_s32)val0) >> 16;

#define SAT32(DST) \
  DST = (mlib_s32)val0;

#else

#define SAT8(DST)                         \
  val0 -= sat;                            \
  if (val0 >= MLIB_S32_MAX)               \
    DST = MLIB_U8_MAX;                    \
  else if (val0 <= MLIB_S32_MIN)          \
    DST = MLIB_U8_MIN;                    \
  else                                    \
    DST = ((mlib_s32) val0 >> 24) ^ 0x80;

#define SAT16(DST)               \
  if (val0 >= MLIB_S32_MAX)      \
    DST = MLIB_S16_MAX;          \
  else if (val0 <= MLIB_S32_MIN) \
    DST = MLIB_S16_MIN;          \
  else                           \
    DST = (mlib_s32)val0 >> 16;

#define SAT32(DST)               \
  if (val0 >= MLIB_S32_MAX)      \
    DST = MLIB_S32_MAX;          \
  else if (val0 <= MLIB_S32_MIN) \
    DST = MLIB_S32_MIN;          \
  else                           \
    DST = (mlib_s32)val0;

#endif

#define MLIB_EDGE_EXTEND_BC_U8                                                              \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                           \
        {                                                                                   \
          filterpos = ((X - 32768) >> FILTER_SHIFT_U8) & FILTER_MASK_U8;                    \
          fptr = (mlib_f32 *) ((mlib_u8 *)mlib_filters_u8f_bc + filterpos);                 \
                                                                                            \
          xf0 = fptr[0];                                                                    \
          xf1 = fptr[1];                                                                    \
          xf2 = fptr[2];                                                                    \
          xf3 = fptr[3];                                                                    \
                                                                                            \
          filterpos = ((Y - 32768) >> FILTER_SHIFT_U8) & FILTER_MASK_U8;                    \
          fptr = (mlib_f32 *) ((mlib_u8 *)mlib_filters_u8f_bc + filterpos);                 \
                                                                                            \
          yf0 = fptr[0];                                                                    \
          yf1 = fptr[1];                                                                    \
          yf2 = fptr[2];                                                                    \
          yf3 = fptr[3];                                                                    \
                                                                                            \
          xSrc = ((X - 32768) >> MLIB_SHIFT);                                               \
          ySrc = ((Y - 32768) >> MLIB_SHIFT);                                               \
                                                                                            \
          xDelta0 = ((~((xSrc - 1) >> MLIB_SIGN_SHIFT)) & (-channels));                     \
          yDelta0 = ((~((ySrc - 1) >> MLIB_SIGN_SHIFT)) & (-srcStride));                    \
          xDelta1 = ((xSrc + 1 - srcWidth) >> MLIB_SIGN_SHIFT) & (channels);                \
          yDelta1 = ((ySrc + 1 - srcHeight) >> MLIB_SIGN_SHIFT) & (srcStride);              \
          xDelta2 = xDelta1 + (((xSrc + 2 - srcWidth) >> MLIB_SIGN_SHIFT) & (channels));    \
          yDelta2 = yDelta1 + (((ySrc + 2 - srcHeight) >> MLIB_SIGN_SHIFT) & (srcStride));  \
                                                                                            \
          xFlag = (xSrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                 \
          xSrc = xSrc + (1 & xFlag);                                                        \
          xDelta2 -= (xDelta1 & xFlag);                                                     \
          xDelta1 = (xDelta1 &~ xFlag);                                                     \
                                                                                            \
          yFlag = (ySrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                 \
          ySrc = ySrc + (1 & yFlag);                                                        \
          yDelta2  -= (yDelta1 & yFlag);                                                    \
          yDelta1 = yDelta1 &~ yFlag;                                                       \
                                                                                            \
          sPtr = ((mlib_u8 **)lineAddr)[ySrc] + xSrc * channels;                            \
                                                                                            \
          for (j = 0; j < channels; j++)                                                    \
          {                                                                                 \
            c0 = mlib_U82D64[(*(sPtr + yDelta0 + xDelta0))] * xf0 +                         \
                 mlib_U82D64[(*(sPtr + yDelta0))]           * xf1 +                         \
                 mlib_U82D64[(*(sPtr + yDelta0 + xDelta1))] * xf2 +                         \
                 mlib_U82D64[(*(sPtr + yDelta0 + xDelta2))] * xf3;                          \
                                                                                            \
            c1 = mlib_U82D64[(*(sPtr + xDelta0))] * xf0 +                                   \
                 mlib_U82D64[(*sPtr)]             * xf1 +                                   \
                 mlib_U82D64[(*(sPtr + xDelta1))] * xf2 +                                   \
                 mlib_U82D64[(*(sPtr + xDelta2))] * xf3;                                    \
                                                                                            \
            c2 = mlib_U82D64[(*(sPtr + yDelta1 + xDelta0))] * xf0 +                         \
                 mlib_U82D64[(*(sPtr + yDelta1))]           * xf1 +                         \
                 mlib_U82D64[(*(sPtr + yDelta1 + xDelta1))] * xf2 +                         \
                 mlib_U82D64[(*(sPtr + yDelta1 + xDelta2))] * xf3;                          \
                                                                                            \
            c3 = mlib_U82D64[(*(sPtr + yDelta2 + xDelta0))] * xf0 +                         \
                 mlib_U82D64[(*(sPtr + yDelta2))]           * xf1 +                         \
                 mlib_U82D64[(*(sPtr + yDelta2 + xDelta1))] * xf2 +                         \
                 mlib_U82D64[(*(sPtr + yDelta2 + xDelta2))] * xf3;                          \
                                                                                            \
            val0 = (c0*yf0 + c1*yf1 + c2*yf2 + c3*yf3);                                     \
                                                                                            \
            SAT8(dstPixelPtr[j]);                                                           \
                                                                                            \
            sPtr++;                                                                         \
          }                                                                                 \
                                                                                            \
          X += dX;                                                                          \
          Y += dY;                                                                          \
        }

#define MLIB_EDGE_EXTEND_BC_S16                                                                 \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                               \
        {                                                                                       \
          filterpos = ((X - 32768) >> FILTER_SHIFT_S16) & FILTER_MASK_S16;                      \
          fptr = (mlib_f32 *) ((mlib_u8 *)mlib_filters_s16f_bc + filterpos);                    \
                                                                                                \
          xf0 = fptr[0];                                                                        \
          xf1 = fptr[1];                                                                        \
          xf2 = fptr[2];                                                                        \
          xf3 = fptr[3];                                                                        \
                                                                                                \
          filterpos = ((Y - 32768) >> FILTER_SHIFT_S16) & FILTER_MASK_S16;                      \
          fptr = (mlib_f32 *) ((mlib_u8 *)mlib_filters_s16f_bc + filterpos);                    \
                                                                                                \
          yf0 = fptr[0];                                                                        \
          yf1 = fptr[1];                                                                        \
          yf2 = fptr[2];                                                                        \
          yf3 = fptr[3];                                                                        \
                                                                                                \
          xSrc = ((X - 32768) >> MLIB_SHIFT);                                                   \
          ySrc = ((Y - 32768) >> MLIB_SHIFT);                                                   \
                                                                                                \
          xDelta0 = ((~((xSrc - 1) >> MLIB_SIGN_SHIFT)) & (-channels));                         \
          yDelta0 = ((~((ySrc - 1) >> MLIB_SIGN_SHIFT)) & (-(srcStride >> 1)));                 \
          xDelta1 = ((xSrc + 1 - srcWidth) >> MLIB_SIGN_SHIFT) & (channels);                    \
          yDelta1 = ((ySrc + 1 - srcHeight) >> MLIB_SIGN_SHIFT) & (srcStride >> 1);             \
          xDelta2 = xDelta1 + (((xSrc + 2 - srcWidth) >> MLIB_SIGN_SHIFT) & (channels));        \
          yDelta2 = yDelta1 + (((ySrc + 2 - srcHeight) >> MLIB_SIGN_SHIFT) & (srcStride >> 1)); \
                                                                                                \
          xFlag = (xSrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                     \
          xSrc = xSrc + (1 & xFlag);                                                            \
          xDelta2 -= (xDelta1 & xFlag);                                                         \
          xDelta1 = (xDelta1 &~ xFlag);                                                         \
                                                                                                \
          yFlag = (ySrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                     \
          ySrc = ySrc + (1 & yFlag);                                                            \
          yDelta2  -= (yDelta1 & yFlag);                                                        \
          yDelta1 = yDelta1 &~ yFlag;                                                           \
                                                                                                \
          sPtr = ((mlib_s16 **)lineAddr)[ySrc] + xSrc * channels;                               \
                                                                                                \
          for (j = 0; j < channels; j++)                                                        \
          {                                                                                     \
            c0 = (*(sPtr + yDelta0 + xDelta0)) * xf0 +                                          \
                 (*(sPtr + yDelta0)) * xf1 +                                                    \
                 (*(sPtr + yDelta0 + xDelta1)) * xf2 +                                          \
                 (*(sPtr + yDelta0 + xDelta2)) * xf3;                                           \
                                                                                                \
            c1 = (*(sPtr + xDelta0)) * xf0 +                                                    \
                 (*sPtr) * xf1 +                                                                \
                 (*(sPtr + xDelta1)) * xf2 +                                                    \
                 (*(sPtr + xDelta2)) * xf3;                                                     \
                                                                                                \
            c2 = (*(sPtr + yDelta1 + xDelta0)) * xf0 +                                          \
                 (*(sPtr + yDelta1)) * xf1 +                                                    \
                 (*(sPtr + yDelta1 + xDelta1)) * xf2 +                                          \
                 (*(sPtr + yDelta1 + xDelta2)) * xf3;                                           \
                                                                                                \
            c3 = (*(sPtr + yDelta2 + xDelta0)) * xf0 +                                          \
                 (*(sPtr + yDelta2)) * xf1 +                                                    \
                 (*(sPtr + yDelta2 + xDelta1)) * xf2 +                                          \
                 (*(sPtr + yDelta2 + xDelta2)) * xf3;                                           \
                                                                                                \
            val0 = (c0*yf0 + c1*yf1 + c2*yf2 + c3*yf3);                                         \
                                                                                                \
            SAT16(dstPixelPtr[j]);                                                              \
                                                                                                \
            sPtr++;                                                                             \
          }                                                                                     \
                                                                                                \
          X += dX;                                                                              \
          Y += dY;                                                                              \
        }

#define MLIB_EDGE_EXTEND_BC_S32                                                                 \
        for (; dstPixelPtr < dstLineEnd; dstPixelPtr += channels)                               \
        {                                                                                       \
          dx =  ((X - 32768) & MLIB_MASK) * scale;  dy = ((Y - 32768) & MLIB_MASK) * scale;     \
          dx_2 = 0.5 * dx;    dy_2 = 0.5 * dy;                                                  \
          dx2 = dx * dx;      dy2 = dy * dy;                                                    \
          dx3_2 = dx_2 * dx2; dy3_2 = dy_2 * dy2;                                               \
          dx3_3 = 3.0 * dx3_2;  dy3_3 = 3.0 * dy3_2;                                            \
                                                                                                \
          xf0 = dx2 - dx3_2 - dx_2;                                                             \
          xf1 = dx3_3 - 2.5 * dx2 + 1.0;                                                        \
          xf2 = 2.0 * dx2 - dx3_3 + dx_2;                                                       \
          xf3 = dx3_2 - 0.5 * dx2;                                                              \
                                                                                                \
          yf0 = dy2 - dy3_2 - dy_2;                                                             \
          yf1 = dy3_3 - 2.5 * dy2 + 1.0;                                                        \
          yf2 = 2.0 * dy2 - dy3_3 + dy_2;                                                       \
          yf3 = dy3_2 - 0.5 * dy2;                                                              \
                                                                                                \
          xSrc = ((X - 32768) >> MLIB_SHIFT);                                                   \
          ySrc = ((Y - 32768) >> MLIB_SHIFT);                                                   \
                                                                                                \
          xDelta0 = ((~((xSrc - 1) >> MLIB_SIGN_SHIFT)) & (-channels));                         \
          yDelta0 = ((~((ySrc - 1) >> MLIB_SIGN_SHIFT)) & (-(srcStride >> 2)));                 \
          xDelta1 = ((xSrc + 1 - srcWidth) >> MLIB_SIGN_SHIFT) & (channels);                    \
          yDelta1 = ((ySrc + 1 - srcHeight) >> MLIB_SIGN_SHIFT) & (srcStride >> 2);             \
          xDelta2 = xDelta1 + (((xSrc + 2 - srcWidth) >> MLIB_SIGN_SHIFT) & (channels));        \
          yDelta2 = yDelta1 + (((ySrc + 2 - srcHeight) >> MLIB_SIGN_SHIFT) & (srcStride >> 2)); \
                                                                                                \
          xFlag = (xSrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                     \
          xSrc = xSrc + (1 & xFlag);                                                            \
          xDelta2 -= (xDelta1 & xFlag);                                                         \
          xDelta1 = (xDelta1 &~ xFlag);                                                         \
                                                                                                \
          yFlag = (ySrc >> (MLIB_SIGN_SHIFT - MLIB_SHIFT));                                     \
          ySrc = ySrc + (1 & yFlag);                                                            \
          yDelta2  -= (yDelta1 & yFlag);                                                        \
          yDelta1 = yDelta1 &~ yFlag;                                                           \
                                                                                                \
          sPtr = ((mlib_s32 **)lineAddr)[ySrc] + xSrc * channels;                               \
                                                                                                \
          for (j = 0; j < channels; j++)                                                        \
          {                                                                                     \
            c0 = (*(sPtr + yDelta0 + xDelta0)) * xf0 +                                          \
                 (*(sPtr + yDelta0)) * xf1 +                                                    \
                 (*(sPtr + yDelta0 + xDelta1)) * xf2 +                                          \
                 (*(sPtr + yDelta0 + xDelta2)) * xf3;                                           \
                                                                                                \
            c1 = (*(sPtr + xDelta0)) * xf0 +                                                    \
                 (*sPtr) * xf1 +                                                                \
                 (*(sPtr + xDelta1)) * xf2 +                                                    \
                 (*(sPtr + xDelta2)) * xf3;                                                     \
                                                                                                \
            c2 = (*(sPtr + yDelta1 + xDelta0)) * xf0 +                                          \
                 (*(sPtr + yDelta1)) * xf1 +                                                    \
                 (*(sPtr + yDelta1 + xDelta1)) * xf2 +                                          \
                 (*(sPtr + yDelta1 + xDelta2)) * xf3;                                           \
                                                                                                \
            c3 = (*(sPtr + yDelta2 + xDelta0)) * xf0 +                                          \
                 (*(sPtr + yDelta2)) * xf1 +                                                    \
                 (*(sPtr + yDelta2 + xDelta1)) * xf2 +                                          \
                 (*(sPtr + yDelta2 + xDelta2)) * xf3;                                           \
                                                                                                \
            val0 = (c0*yf0 + c1*yf1 + c2*yf2 + c3*yf3);                                         \
                                                                                                \
            SAT32(dstPixelPtr[j])                                                               \
                                                                                                \
            sPtr++;                                                                             \
          }                                                                                     \
                                                                                                \
          X += dX;                                                                              \
          Y += dY;                                                                              \
        }

/***************************************************************/

void mlib_ImageAffineEdgeExtend_BC(mlib_image *dst,
                                  mlib_image *src,
                                  mlib_s32 *leftEdges,
                                  mlib_s32 *rightEdges,
                                  mlib_s32 *sides,
                                  mlib_s32 *leftEdgesE,
                                  mlib_s32 *rightEdgesE,
                                  mlib_s32 *xStartsE,
                                  mlib_s32 *yStartsE,
                                  mlib_s32 *sidesE,
                                  mlib_u8  **lineAddr)
{
  mlib_type type        = mlib_ImageGetType(dst);
  mlib_s32  channels    = mlib_ImageGetChannels(dst);
  mlib_s32  dstStride   = mlib_ImageGetStride(dst);
  mlib_s32  srcStride   = mlib_ImageGetStride(src);
  mlib_s32  srcWidth    = mlib_ImageGetWidth(src);
  mlib_s32  srcHeight   = mlib_ImageGetHeight(src);
  mlib_u8   *data       = (mlib_u8 *)mlib_ImageGetData(dst);
  mlib_s32  yStart      = sides[0];
  mlib_s32  yFinish     = sides[1];
  mlib_s32  yStartE     = sidesE[0];
  mlib_s32  yFinishE    = sidesE[1];
  mlib_s32  dX          = sidesE[2];
  mlib_s32  dY          = sidesE[3];
  mlib_s32  xSrc, ySrc, X, Y, xFlag, yFlag;
  mlib_s32  xLeft, xRight, xLeftE, xRightE;
  mlib_s32  i, j;

  mlib_d64  xf0, xf1, xf2, xf3;
  mlib_d64  yf0, yf1, yf2, yf3;
  mlib_d64  c0, c1, c2, c3, val0;
  mlib_s32  filterpos;
  mlib_f32  *fptr;

  mlib_s32  xDelta0, xDelta1, xDelta2;
  mlib_s32  yDelta0, yDelta1, yDelta2;

  data += (yStartE - 1) * dstStride;

  switch (type)
  {
    case MLIB_BYTE:
    {
      mlib_u8 *srcPixelPtr;
      mlib_u8 *dstPixelPtr;
      mlib_u8 *dstLineEnd;
      mlib_d64 sat = (mlib_d64)0x7F800000;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_u8 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_u8 *)(data + channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BC_U8
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_u8 *)(data + channels * xLeftE);
          dstLineEnd   = (mlib_u8 *)(data + channels * (xRightE + 1));

          MLIB_EDGE_EXTEND_BC_U8
        }
        else
        {
          dstPixelPtr  = (data + channels * xLeftE);
          dstLineEnd   = (data + channels * xLeft);

          MLIB_EDGE_EXTEND_BC_U8

          dstPixelPtr = (data + channels * (xRight + 1));
          dstLineEnd  = (data + channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_EXTEND_BC_U8
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_u8 *)(data + channels * xLeftE);
        dstLineEnd   = (mlib_u8 *)(data + channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BC_U8
      }
    }
    break;

    case MLIB_SHORT:
    {
      mlib_s16 *srcPixelPtr;
      mlib_s16 *dstPixelPtr;
      mlib_s16 *dstLineEnd;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BC_S16
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

          MLIB_EDGE_EXTEND_BC_S16
        }
        else
        {
          dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
          dstLineEnd   = (mlib_s16 *)(data + 2 * channels * xLeft);

          MLIB_EDGE_EXTEND_BC_S16

          dstPixelPtr = (mlib_s16 *)(data + 2 * channels * (xRight + 1));
          dstLineEnd  = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_EXTEND_BC_S16
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s16 *)(data + 2 * channels * xLeftE);
        dstLineEnd   = (mlib_s16 *)(data + 2 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BC_S16
      }
    }
    break;

    case MLIB_INT:
    {
      mlib_s32 *srcPixelPtr;
      mlib_s32 *dstPixelPtr;
      mlib_s32 *dstLineEnd;
      mlib_d64  dx, dx_2, dx2, dx3_2, dx3_3;
      mlib_d64  dy, dy_2, dy2, dy3_2, dy3_3;
      mlib_d64  scale = 1. / 65536.;

      for (i = yStartE; i < yStart; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BC_S32
      }

      for (; i <= yFinish; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        xLeft   = leftEdges[i];
        xRight  = rightEdges[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        if (xLeft > xRight)
        {
          dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

          MLIB_EDGE_EXTEND_BC_S32
        }
        else
        {
          dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
          dstLineEnd   = (mlib_s32 *)(data + 4 * channels * xLeft);

          MLIB_EDGE_EXTEND_BC_S32

          dstPixelPtr = (mlib_s32 *)(data + 4 * channels * (xRight + 1));
          dstLineEnd  = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

          X += dX * (xRight - xLeft + 1);
          Y += dY * (xRight - xLeft + 1);

          MLIB_EDGE_EXTEND_BC_S32
        }
      }

      for (; i <= yFinishE; i++)
      {
        xLeftE  = leftEdgesE[i];
        xRightE = rightEdgesE[i];
        X       = xStartsE[i];
        Y       = yStartsE[i];
        data   += dstStride;

        if (xLeftE > xRightE)
          continue;

        dstPixelPtr  = (mlib_s32 *)(data + 4 * channels * xLeftE);
        dstLineEnd   = (mlib_s32 *)(data + 4 * channels * (xRightE + 1));

        MLIB_EDGE_EXTEND_BC_S32
      }
    }
    break;
  }
}

/***************************************************************/

#undef sPtr

/***************************************************************/
