/*
 * @(#)mlib_c_ImageConv5x5.c	1.8 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifdef _MSC_VER
#pragma warning(disable:4068 4244 4101)
#endif /*_MSC_VER*/

#ifdef __SUNPRO_C
#pragma ident  "@(#)mlib_c_ImageConv5x5.c	1.19  99/04/23 SMI"
#endif /* __SUNPRO_C */

/*
 * FUNCTION
 *      mlib_ImageConv5x5 - image convolution with edge condition
 *
 * SYNOPSIS
 *      mlib_status mlib_ImageConv5x5(mlib_image *dst,
 *                                    mlib_image *src,
 *                                    mlib_s32   *kernel,
 *                                    mlib_s32   scale,
 *                                    mlib_s32   cmask,
 *                                    mlib_edge  edge)
 *
 * ARGUMENTS
 *      dst       Pointer to destination image.
 *      src       Pointer to source image.
 *      kernel    Pointer to convolution kernel.
 *      scale     The scaling factor to convert the input integer
 *                coefficients into floating-point coefficients:
 *                floating-point coefficient = integer coefficient * 2^(-scale)
 *      cmask     Channel mask to indicate the channels to be convolved.
 *                Each bit of which represents a channel in the image. The
 *                channels corresponded to 1 bits are those to be processed.
 *      edge      Type of edge condition.
 *
 * DESCRIPTION
 *      2-D convolution, 5x5 kernel.
 *
 *      The center of the source image is mapped to the center of the
 *      destination image.
 *      The unselected channels are not overwritten. If both src and dst have
 *      just one channel, cmask is ignored.
 *
 *      The edge condition can be one of the following:
 *              MLIB_EDGE_DST_NO_WRITE  (default)
 *              MLIB_EDGE_DST_FILL_ZERO
 *              MLIB_EDGE_DST_COPY_SRC
 *              MLIB_EDGE_SRC_EXTEND
 *
 * RESTRICTION
 *      The src and the dst must be the same type and have same number
 *      of channels (1, 2, 3, or 4).
 *      For data type MLIB_BYTE - 16 <= scale <= 31 (to be compatible with VIS
 *      version).
 *      For data type MLIB_SHORT - 17 <= scale <= 32 (to be compatible with VIS
 *      version).
 *      For data type MLIB_INT - scale >= 0.
 */

/***************************************************************************/

#include <stdlib.h>
#include <mlib_image.h>
#include <mlib_ImageCheck.h>

mlib_status mlib_c_conv5x5_32nw( mlib_image *dst, mlib_image *src,
                                 mlib_s32 *kern, mlib_s32 scalef_expon,
                                 mlib_s32 cmask );

mlib_status mlib_c_conv5x5_32ext( mlib_image *dst, mlib_image *src,
                                  mlib_s32  dx_l, mlib_s32 dx_r,
                                  mlib_s32  dy_t, mlib_s32 dy_b,
                                  mlib_s32 *kernel, mlib_s32 scalef_expon,
                                  mlib_s32 cmask );

mlib_status mlib_c_conv5x5_16nw( mlib_image *dst, mlib_image *src,
                                 mlib_s32 *kern, mlib_s32 scalef_expon,
                                 mlib_s32 cmask );

mlib_status mlib_c_conv5x5_16ext( mlib_image *dst, mlib_image *src,
                                  mlib_s32  dx_l, mlib_s32 dx_r,
                                  mlib_s32  dy_t, mlib_s32 dy_b,
                                  mlib_s32 *kernel, mlib_s32 scalef_expon,
                                  mlib_s32 cmask );

mlib_status mlib_c_conv5x5_8nw( mlib_image *dst, mlib_image *src,
                                mlib_s32 *kernel, mlib_s32 scalef_expon,
                                mlib_s32 cmask );

mlib_status mlib_c_conv5x5_8ext( mlib_image *dst, mlib_image *src,
                                 mlib_s32  dx_l, mlib_s32 dx_r,
                                 mlib_s32  dy_t, mlib_s32 dy_b,
                                 mlib_s32 *kernel, mlib_s32 scalef_expon,
                                 mlib_s32 cmask );

mlib_status mlib_c_ImageConvCopyEdge(mlib_image *dst, mlib_image *src, mlib_s32 dx_l,
                                     mlib_s32 dx_r, mlib_s32 dy_t, mlib_s32 dy_b, mlib_s32 cmask);
mlib_status mlib_c_ImageConvClearEdge(mlib_image *img, mlib_s32 dx_l, mlib_s32 dx_r,
                                      mlib_s32 dy_t, mlib_s32 dy_b, mlib_s32 *color, mlib_s32 cmask);

mlib_image *mlib_ImageSet(mlib_image *image, 
                          mlib_type  type, 
                          mlib_s32   channels,
                          mlib_s32   width, 
                          mlib_s32   height, 
                          mlib_s32   stride, 
                          void       *data);

/*****************************************************************************************************/
#define LEFT   2
#define RIGHT  2
#define TOP    2
#define BOTTOM 2
/*****************************************************************************************************/
#ifndef MLIB_USE_FTOI_CLAMPING
#define SAT_U8(x) (mlib_s32)(((x) < MLIB_U8_MIN) ? MLIB_U8_MIN : (((x) > MLIB_U8_MAX) ? MLIB_U8_MAX : (x)))
#define SAT_S16(x) (mlib_s32)(((x) < MLIB_S16_MIN) ? MLIB_S16_MIN : (((x) > MLIB_S16_MAX) ? MLIB_S16_MAX : (x)))
#define SAT_S32(x) (mlib_s32)(((x) < MLIB_S32_MIN) ? MLIB_S32_MIN : (((x) > MLIB_S32_MAX) ? MLIB_S32_MAX : (x)))
#else
#define SAT_U8(x) (mlib_s32)((x) - sat)
#define SAT_S16(x) (mlib_s32)(x)
#define SAT_S32(x) (mlib_s32)(x)
#endif
/*****************************************************************************************************/
#define DEFINE_IDENTIFICATORS(type)                                                                   \
  type     *sl, *adr_src;                       /* pointers for pixel and line of source */           \
  type     *da, *dl, *adr_dst;                  /* pointers for pixel and line of destination */      \
  mlib_s32 row, col, block;                     /* indices */                                         \
  mlib_s32 *row0, *row1, *row2, *row3, *row4;                                                         \
  mlib_d64 scalef;                              /* divide coeffs by scalefactor=2**scalef_expon */    \
  mlib_d64 sum1, sum2, sum3;                    /* accumulators */                                    \
  mlib_d64 k0, k1, k2, k3, k4, k5, k6, k7, k8;   /* coeff's */                                        \
  mlib_d64 k9, k10, k11, k12, k13, k14, k15;     /* coeff's */                                        \
  mlib_d64 k16, k17, k18, k19, k20, k21, k22;    /* coeff's */                                        \
  mlib_d64 k23, k24;                             /* coeff's */                                        \
  mlib_d64 p00, p01, p02, p03, p04, p05, p06,    /* row 0 pixel values */                             \
           p10, p11, p12, p13, p14, p15, p16,    /* row 1 pixel values */                             \
           p20, p21, p22, p23, p24, p25, p26;    /* row 2 pixel values */                             \
  mlib_s32 sll,                                 /* line length (in pixels) of source image */         \
           dll,                                 /* line length (in pixels) of destination image */    \
           wid,                                 /* src width in pixels */                             \
           hgt;                                 /* src height in pixels */                            \
  mlib_s32 num_blocks;                          /* do output pts 3 at a time; loop count for this */  \
  mlib_s32 leftover_cols;                       /* what can't be done in blocks */                    \
  mlib_s32 chan, testchan, nchannel;                                                                  \
  mlib_d64 *doutbuf, *d_outbuf;
/*****************************************************************************************************/
#define DEFINE_DOP_IDENTIFICATORS(type)                                                               \
  type     *sa;                                 /* pointers for pixel and line of source */           \
                                                                                                      \
   /* Keep 5 rows of data in 32-bit int buffers to save cost of repeatedly            */              \
   /* casting byte (short) data to 32-bit float. (To caste byte (short) to float,     */              \
   /* it reads byte (short) from mem into int reg, converts to 32-bit int, writes it  */              \
   /* to mem, then reads it into float reg, converting to float once inside           */              \
   /* the float reg. Thus the cost is in converting from byte (short) to int, not     */              \
   /* int to float.)                                                                  */              \
                                                                                                      \
  mlib_s32 *rowbuf[5];                          /* int row buf ptrs */                                \
  mlib_s32 *prow;                               /* general-purpose ptr into a row */                  \
  /* Store results of each row to an integer type intermediate output buf;            */              \
  /* then convert row buf data to byte (short) and store in final destination.        */              \
  /* This is actually faster than casting each float result to byte (short) as        */              \
  /* it is computed                                                                   */              \
                                                                                                      \
  mlib_s32 *outbuf, *pout;                       /* int output buf & ptr */                           \
  mlib_s32 isum;
/*****************************************************************************************************/
#define LOAD_KERNEL_INTO_DOUBLE()                                                                     \
  while (scalef_expon > 30) {                                                                         \
    scalef /= (1 << 30);                                                                              \
    scalef_expon -= 30;                                                                               \
  }                                                                                                   \
                                                                                                      \
  scalef /= (1 << scalef_expon);                                                                      \
                                                                                                      \
  /* keep kernel in regs */                                                                           \
  k0 = scalef * kern[0];  k1 = scalef * kern[1];  k2 = scalef * kern[2];                              \
  k3 = scalef * kern[3];  k4 = scalef * kern[4];  k5 = scalef * kern[5];                              \
  k6 = scalef * kern[6];  k7 = scalef * kern[7];  k8 = scalef * kern[8];                              \
  k9 = scalef * kern[9];  k10 = scalef * kern[10];  k11 = scalef * kern[11];                          \
  k12 = scalef * kern[12];  k13 = scalef * kern[13];  k14 = scalef * kern[14];                        \
  k15 = scalef * kern[15];  k16 = scalef * kern[16];  k17 = scalef * kern[17];                        \
  k18 = scalef * kern[18];  k19 = scalef * kern[19];  k20 = scalef * kern[20];                        \
  k21 = scalef * kern[21];  k22 = scalef * kern[22];  k23 = scalef * kern[23];                        \
  k24 = scalef * kern[24];
/*****************************************************************************************************/
#define GET_SRC_DST_PARAMETERS(type)                                                                  \
  hgt = mlib_ImageGetHeight(src);                                                                     \
  wid = mlib_ImageGetWidth(src);                                                                      \
  nchannel = mlib_ImageGetChannels(src);                                                              \
  sll = mlib_ImageGetStride(src) / sizeof(type);                                                      \
  dll = mlib_ImageGetStride(dst) / sizeof(type);                                                      \
  adr_src = (type *)mlib_ImageGetData(src);                                                           \
  adr_dst = (type *)mlib_ImageGetData(dst);
/*****************************************************************************************************/
#define FIRST_CONVOLVE_THREE_PIXELS(shift)                                                            \
  p04 = row0[col]; p14 = row1[col]; p24 = row2[col]; col += shift;                                    \
  p05 = row0[col]; p15 = row1[col]; p25 = row2[col]; col += shift;                                    \
  p06 = row0[col]; p16 = row1[col]; p26 = row2[col]; col += shift;                                    \
                                                                                                      \
  sum1 = sum2 = sum3 = 0.;                                                                            \
                                                                                                      \
  sum1 += p00 * k0;  sum2 += p01 * k0;  sum3 += p02 * k0;                                             \
  sum1 += p01 * k1;  sum2 += p02 * k1;  sum3 += p03 * k1;                                             \
  sum1 += p02 * k2;  sum2 += p03 * k2;  sum3 += p04 * k2;                                             \
  sum1 += p03 * k3;  sum2 += p04 * k3;  sum3 += p05 * k3;                                             \
  sum1 += p04 * k4;  sum2 += p05 * k4;  sum3 += p06 * k4;                                             \
  sum1 += p10 * k5;  sum2 += p11 * k5;  sum3 += p12 * k5;                                             \
  sum1 += p11 * k6;  sum2 += p12 * k6;  sum3 += p13 * k6;                                             \
  sum1 += p12 * k7;  sum2 += p13 * k7;  sum3 += p14 * k7;                                             \
  sum1 += p13 * k8;  sum2 += p14 * k8;  sum3 += p15 * k8;                                             \
  sum1 += p14 * k9;  sum2 += p15 * k9;  sum3 += p16 * k9;                                             \
  sum1 += p20 * k10; sum2 += p21 * k10; sum3 += p22 * k10;                                            \
  sum1 += p21 * k11; sum2 += p22 * k11; sum3 += p23 * k11;                                            \
  sum1 += p22 * k12; sum2 += p23 * k12; sum3 += p24 * k12;                                            \
  sum1 += p23 * k13; sum2 += p24 * k13; sum3 += p25 * k13;                                            \
  sum1 += p24 * k14; sum2 += p25 * k14; sum3 += p26 * k14;                                            \
                                                                                                      \
  *d_outbuf++ = sum1;                                                                                 \
  *d_outbuf++ = sum2;                                                                                 \
  *d_outbuf++ = sum3;                                                                                 \
                                                                                                      \
  p00 = p03; p01 = p04; p02 = p05; p03 = p06; /* shift 4 last row0 element */                         \
  p10 = p13; p11 = p14; p12 = p15; p13 = p16; /* shift 4 last row1 element */                         \
  p20 = p23; p21 = p24; p22 = p25; p23 = p26; /* shift 4 last row2 element */
/*****************************************************************************************************/
#define SECOND_CONVOLVE_THREE_PIXELS(dst, shift, format)                                              \
  p04 = row3[col]; p14 = row4[col]; col += shift;                                                     \
  p05 = row3[col]; p15 = row4[col]; col += shift;                                                     \
  p06 = row3[col]; p16 = row4[col]; col += shift;                                                     \
                                                                                                      \
  sum1 = *d_outbuf++;                                                                                 \
  sum2 = *d_outbuf++;                                                                                 \
  sum3 = *d_outbuf++;                                                                                 \
                                                                                                      \
  sum1 += p00 * k15;  sum2 += p01 * k15;  sum3 += p02 * k15;                                          \
  sum1 += p01 * k16;  sum2 += p02 * k16;  sum3 += p03 * k16;                                          \
  sum1 += p02 * k17;  sum2 += p03 * k17;  sum3 += p04 * k17;                                          \
  sum1 += p03 * k18;  sum2 += p04 * k18;  sum3 += p05 * k18;                                          \
  sum1 += p04 * k19;  sum2 += p05 * k19;  sum3 += p06 * k19;                                          \
  sum1 += p10 * k20;  sum2 += p11 * k20;  sum3 += p12 * k20;                                          \
  sum1 += p11 * k21;  sum2 += p12 * k21;  sum3 += p13 * k21;                                          \
  sum1 += p12 * k22;  sum2 += p13 * k22;  sum3 += p14 * k22;                                          \
  sum1 += p13 * k23;  sum2 += p14 * k23;  sum3 += p15 * k23;                                          \
  sum1 += p14 * k24;  sum2 += p15 * k24;  sum3 += p16 * k24;                                          \
                                                                                                      \
  *dst = SAT_##format(sum1); dst += shift;                                                            \
  *dst = SAT_##format(sum2); dst += shift;                                                            \
  *dst = SAT_##format(sum3); dst += shift;                                                            \
                                                                                                      \
  p00 = p03; p01 = p04; p02 = p05; p03 = p06; /* shift 4 last row3 element */                         \
  p10 = p13; p11 = p14; p12 = p15; p13 = p16; /* shift 4 last row4 element */                         \

/*****************************************************************************************************/
#define FIRST_CONVOLVE_ONE_PIXEL(shift)                                                               \
  p04 = row0[col];                                                                                    \
  p14 = row1[col];                                                                                    \
  p24 = row2[col]; col += shift;                                                                      \
  sum1 = 0.;                                                                                          \
  sum1 += p00 * k0;                         /* sum of prods */                                        \
  sum1 += p01 * k1;                                                                                   \
  sum1 += p02 * k2;                                                                                   \
  sum1 += p03 * k3;                                                                                   \
  sum1 += p04 * k4;                                                                                   \
  sum1 += p10 * k5;                                                                                   \
  sum1 += p11 * k6;                                                                                   \
  sum1 += p12 * k7;                                                                                   \
  sum1 += p13 * k8;                                                                                   \
  sum1 += p14 * k9;                                                                                   \
  sum1 += p20 * k10;                                                                                  \
  sum1 += p21 * k11;                                                                                  \
  sum1 += p22 * k12;                                                                                  \
  sum1 += p23 * k13;                                                                                  \
  sum1 += p24 * k14;                                                                                  \
  *d_outbuf++ = sum1;                                                                                 \
  p00 = p01; p01 = p02; p02 = p03; p03 = p04; /* shift 4 last row0 element */                         \
  p10 = p11; p11 = p12; p12 = p13; p13 = p14; /* shift 4 last row1 element */                         \
  p20 = p21; p21 = p22; p22 = p23; p23 = p24; /* shift 4 last row2 element */
/*****************************************************************************************************/
#define SECOND_CONVOLVE_ONE_PIXEL(dst, shift, format)                                                 \
  p04 = row3[col];                                                                                    \
  p14 = row4[col]; col += shift;                                                                      \
  sum1 = *d_outbuf++;                                                                                 \
  sum1 += p00 * k15;                        /* sum of prods */                                        \
  sum1 += p01 * k16;                                                                                  \
  sum1 += p02 * k17;                                                                                  \
  sum1 += p03 * k18;                                                                                  \
  sum1 += p04 * k19;                                                                                  \
  sum1 += p10 * k20;                                                                                  \
  sum1 += p11 * k21;                                                                                  \
  sum1 += p12 * k22;                                                                                  \
  sum1 += p13 * k23;                                                                                  \
  sum1 += p14 * k24;                                                                                  \
                                                                                                      \
  *dst = SAT_##format(sum1); dst += shift;                                                                                 \
  p00 = p01; p01 = p02; p02 = p03; p03 = p04; /* shift 4 last row3 element */                         \
  p10 = p11; p11 = p12; p12 = p13; p13 = p14; /* shift 4 last row4 element */
/*****************************************************************************************************/
#define LOAD_ONE_ROW_EXT(number)                                                                      \
  for (col = 0; col < swid; col++) {                                                                  \
    rowbuf[number][col+dx_l] = (mlib_s32)(*sa);                                                       \
    sa += nchannel;                                                                                   \
  }                                                                                                   \
  for (col = 0; col < dx_l; col++) {                                                                  \
    rowbuf[number][col] = rowbuf[number][dx_l];                                                       \
  }                                                                                                   \
  for (col = 0; col < dx_r; col++) {                 /* extend edges */                               \
    rowbuf[number][swid + dx_l + col] = rowbuf[number][swid + dx_l - 1];                              \
  }
/*****************************************************************************************************/
#ifdef MLIB_TEST

mlib_status mlib_c_ImageConv5x5( mlib_image *dst,
                                 mlib_image *src,
                                 mlib_s32   *kernel,
                                 mlib_s32   scale,
                                 mlib_s32   cmask,
                                 mlib_edge  edge)

#else
mlib_status mlib_ImageConv5x5( mlib_image *dst,
                               mlib_image *src,
                               mlib_s32   *kernel,
                               mlib_s32   scale,
                               mlib_s32   cmask,
                               mlib_edge  edge)

#endif

{
  mlib_status stat;
  mlib_image tmp_src[1], tmp_dst[1];
  mlib_s32   slb, dlb,      /* line bytes (y stride) */
             src_wid, src_hgt,
             dst_wid, dst_hgt,
             src_nch, dst_nch;
  mlib_type  src_dtype, dst_dtype;
  mlib_s32   zero[4] = {0, 0, 0, 0};
  mlib_s32   dx, dy, dxd, dxs, dyd, dys, dw, dh;
  mlib_s32   dx_l, dx_r, dy_t, dy_b, conv_edge;
  mlib_u8    *sa, *da;

  if (dst == NULL || src == NULL || kernel == NULL)
     return MLIB_FAILURE;

  MLIB_IMAGE_TYPE_EQUAL(dst, src);
  MLIB_IMAGE_CHAN_EQUAL(dst, src);

  MLIB_IMAGE_GET_ALL_PARAMS(dst, dst_dtype, dst_nch, dst_wid, dst_hgt, dlb, da);
  MLIB_IMAGE_GET_ALL_PARAMS(src, src_dtype, src_nch, src_wid, src_hgt, slb, sa);

  dw = (src_wid < dst_wid) ? src_wid : dst_wid;
  dh = (src_hgt < dst_hgt) ? src_hgt : dst_hgt;
  dx = (src_wid - dst_wid);
  dy = (src_hgt - dst_hgt);
  dxs = ((dx > 0) ? (dx+1)>>1 : 0);
  dxd = ((dx > 0) ? 0 : (-dx)>>1);
  dys = ((dy > 0) ? (dy+1)>>1 : 0);
  dyd = ((dy > 0) ? 0 : (-dy)>>1);

  dx_l = LEFT - dxs;
  dx_r = dxs - dx + RIGHT;
  dy_t = TOP - dys;
  dy_b = dys - dy + BOTTOM;
  dx_l = ((dx_l < 0) ? 0 : dx_l);
  dx_r = ((dx_r < 0) ? 0 : (dx_r > RIGHT ? RIGHT : dx_r));
  dy_t = ((dy_t < 0) ? 0 : dy_t);
  dy_b = ((dy_b < 0) ? 0 : (dy_b > BOTTOM ? BOTTOM : dy_b));

  conv_edge = dx_l + dx_r + dy_t + dy_b;
  if (conv_edge == 0) edge = MLIB_EDGE_DST_NO_WRITE;
  if (src_nch == 1) cmask = 1;
  switch (src_dtype) {
  case MLIB_BYTE:
    if (scale < 16 || scale > 31)
      stat = MLIB_FAILURE;
    else {
      stat = MLIB_SUCCESS;
      mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, (dw + LEFT + RIGHT - dx_l - dx_r),
                                  (dh + TOP + BOTTOM - dy_t - dy_b), dlb,
                                  (da + dlb*(dyd + dy_t - TOP) + dst_nch*(dxd + dx_l - LEFT)));
      mlib_ImageSet(tmp_src, dst_dtype, dst_nch, (dw + LEFT + RIGHT - dx_l - dx_r),
                                  (dh + TOP + BOTTOM - dy_t - dy_b), slb,
                                  (sa + slb*(dys + dy_t - TOP) + dst_nch*(dxs + dx_l - LEFT)));
    switch (edge) {
      case MLIB_EDGE_DST_FILL_ZERO:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_8nw(tmp_dst, tmp_src, kernel, scale, cmask);
        if (stat != MLIB_SUCCESS) break;
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + dst_nch*dxd));
        if ((dx_l + dx_r) > dw) {
          dx_l = dw; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh) {
          dy_t = dh; dy_b = 0;
        }
        stat = mlib_c_ImageConvClearEdge(tmp_dst, dx_l, dx_r, dy_t, dy_b, zero, cmask);
        break;
      case MLIB_EDGE_DST_COPY_SRC:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_8nw(tmp_dst, tmp_src, kernel, scale, cmask);
        if (stat != MLIB_SUCCESS) break;
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + dst_nch*dxd));
        mlib_ImageSet(tmp_src, dst_dtype, dst_nch, dw, dh, slb,
                                    (sa + slb*dys + dst_nch*dxs));
        if ((dx_l + dx_r) > dw) {
          dx_l = dw; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh) {
          dy_t = dh; dy_b = 0;
        }
        stat = mlib_c_ImageConvCopyEdge(tmp_dst, tmp_src, dx_l, dx_r, dy_t, dy_b, cmask);
        break;
      case MLIB_EDGE_SRC_EXTEND:
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + dst_nch*dxd));
        mlib_ImageSet(tmp_src, dst_dtype, dst_nch, dw, dh, slb,
                                    (sa + slb*(dys + dy_t - TOP) + dst_nch*(dxs + dx_l - LEFT)));
        stat = mlib_c_conv5x5_8ext(tmp_dst, tmp_src, dx_l, dx_r, dy_t, dy_b, kernel, scale, cmask);
        break;
      case MLIB_EDGE_DST_NO_WRITE:
      default:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_8nw(tmp_dst, tmp_src, kernel, scale, cmask);
        break;
      }
    }
    break;
  case MLIB_SHORT:  /* convert wid from pixels to bytes */
    if (scale < 17 || scale > 32 )
      stat = MLIB_FAILURE;
    else {
      stat = MLIB_SUCCESS;
      mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, (dw + LEFT + RIGHT - dx_l - dx_r),
                                  (dh + TOP + BOTTOM - dy_t - dy_b), dlb,
                                  (da + dlb*(dyd + dy_t - TOP) + 2*dst_nch*(dxd + dx_l - LEFT)));
      mlib_ImageSet(tmp_src, dst_dtype, dst_nch, (dw + LEFT + RIGHT - dx_l - dx_r),
                                  (dh + TOP + BOTTOM - dy_t - dy_b), slb,
                                  (sa + slb*(dys + dy_t - TOP) + 2*dst_nch*(dxs + dx_l - LEFT)));
    switch (edge) {
      case MLIB_EDGE_DST_FILL_ZERO:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_16nw(tmp_dst, tmp_src, kernel, scale, cmask);
        if (stat != MLIB_SUCCESS) break;
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + 2*dst_nch*dxd));
        if ((dx_l + dx_r) > dw) {
          dx_l = dw; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh) {
          dy_t = dh; dy_b = 0;
        }
        stat = mlib_c_ImageConvClearEdge(tmp_dst, dx_l, dx_r, dy_t, dy_b, zero, cmask);
        break;
      case MLIB_EDGE_DST_COPY_SRC:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_16nw(tmp_dst, tmp_src, kernel, scale, cmask);
        if (stat != MLIB_SUCCESS) break;
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + 2*dst_nch*dxd));
        mlib_ImageSet(tmp_src, dst_dtype, dst_nch, dw, dh, slb,
                                    (sa + slb*dys + 2*dst_nch*dxs));
        if ((dx_l + dx_r) > dw) {
          dx_l = dw; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh) {
          dy_t = dh; dy_b = 0;
        }
        stat = mlib_c_ImageConvCopyEdge(tmp_dst, tmp_src, dx_l, dx_r, dy_t, dy_b, cmask);
        break;
      case MLIB_EDGE_SRC_EXTEND:
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + 2*dst_nch*dxd));
        mlib_ImageSet(tmp_src, dst_dtype, dst_nch, dw, dh, slb,
                                    (sa + slb*(dys + dy_t - TOP) + 2*dst_nch*(dxs + dx_l - LEFT)));
        stat = mlib_c_conv5x5_16ext(tmp_dst, tmp_src, dx_l, dx_r, dy_t, dy_b, kernel, scale, cmask);
        break;
      case MLIB_EDGE_DST_NO_WRITE:
      default:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_16nw(tmp_dst, tmp_src, kernel, scale, cmask);
        break;
      }
    }
    break;
  case MLIB_INT:  /* convert wid from pixels to bytes */
    if (scale < 0)
      stat = MLIB_FAILURE;
    else {
      stat = MLIB_SUCCESS;
      mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, (dw + LEFT + RIGHT - dx_l - dx_r),
                                  (dh + TOP + BOTTOM - dy_t - dy_b), dlb,
                                  (da + dlb*(dyd + dy_t - TOP) + 4*dst_nch*(dxd + dx_l - LEFT)));
      mlib_ImageSet(tmp_src, dst_dtype, dst_nch, (dw + LEFT + RIGHT - dx_l - dx_r),
                                  (dh + TOP + BOTTOM - dy_t - dy_b), slb,
                                  (sa + slb*(dys + dy_t - TOP) + 4*dst_nch*(dxs + dx_l - LEFT)));
    switch (edge) {
      case MLIB_EDGE_DST_FILL_ZERO:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_32nw(tmp_dst, tmp_src, kernel, scale, cmask);
        if (stat != MLIB_SUCCESS) break;
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + 4*dst_nch*dxd));
        if ((dx_l + dx_r) > dw) {
          dx_l = dw; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh) {
          dy_t = dh; dy_b = 0;
        }
        stat = mlib_c_ImageConvClearEdge(tmp_dst, dx_l, dx_r, dy_t, dy_b, zero, cmask);
        break;
      case MLIB_EDGE_DST_COPY_SRC:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_32nw(tmp_dst, tmp_src, kernel, scale, cmask);
        if (stat != MLIB_SUCCESS) break;
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + 4*dst_nch*dxd));
        mlib_ImageSet(tmp_src, dst_dtype, dst_nch, dw, dh, slb,
                                    (sa + slb*dys + 4*dst_nch*dxs));
        if ((dx_l + dx_r) > dw) {
          dx_l = dw; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh) {
          dy_t = dh; dy_b = 0;
        }
        stat = mlib_c_ImageConvCopyEdge(tmp_dst, tmp_src, dx_l, dx_r, dy_t, dy_b, cmask);
        break;
      case MLIB_EDGE_SRC_EXTEND:
        mlib_ImageSet(tmp_dst, dst_dtype, dst_nch, dw, dh, dlb,
                                    (da + dlb*dyd + 4*dst_nch*dxd));
        mlib_ImageSet(tmp_src, dst_dtype, dst_nch, dw, dh, slb,
                                    (sa + slb*(dys + dy_t - TOP) + 4*dst_nch*(dxs + dx_l - LEFT)));
        stat = mlib_c_conv5x5_32ext(tmp_dst, tmp_src, dx_l, dx_r, dy_t, dy_b, kernel, scale, cmask);
        break;
      case MLIB_EDGE_DST_NO_WRITE:
      default:
        if (((dx_l + dx_r) < dw) && (dy_t + dy_b) < dh)
          stat = mlib_c_conv5x5_32nw(tmp_dst, tmp_src, kernel, scale, cmask);
        break;
      }
    }
    break;
  default:
    stat = MLIB_FAILURE;
    break;
  }
  return ( stat );
}

/*****************************************************************************************************/

mlib_status mlib_c_conv5x5_32nw( mlib_image *dst,        /* source image */
                                 mlib_image *src,        /* destination image */
                                 mlib_s32 *kern,         /* convolution kernel */
                                 mlib_s32 scalef_expon,  /* scalefactor=2**N */
                                 mlib_s32 cmask)
{
  DEFINE_IDENTIFICATORS(mlib_s32)

  scalef = 1.0;

  LOAD_KERNEL_INTO_DOUBLE()

  GET_SRC_DST_PARAMETERS(mlib_s32)

  /* allocate row bufs: */

  doutbuf = (double *)mlib_malloc(wid*sizeof(double));
  if (doutbuf == NULL) {
    return( MLIB_FAILURE );
  }

  num_blocks = (wid - 4)/3;                      /* (wid-4): no output on 2 first, last columns */
  leftover_cols = (wid - 4) - (num_blocks * 3);
  hgt -= 4;                                      /* boundaries: no output for 2 first & last rows */

  testchan = 1;
  for ( chan = nchannel-1; chan >= 0; chan--) {
    if ((cmask & testchan) == 0) {
      testchan <<= 1;
      continue;
    }
    testchan <<= 1;
    sl = adr_src + chan;
    dl = da = adr_dst + chan;
    dl = da += (dll + nchannel) * 2;

  /*
   * set up rowbufs 1, 2, 3, 4;
   * rowbuf 0 is always reloaded inside the loop:
   */

    row1 = sl;
    sl += sll;
    row2 = sl;
    sl += sll;
    row3 = sl;
    sl += sll;
    row4 = sl;
    sl += sll;

    for ( row = 0; row < hgt; row++ ) {

      row0 = row1;                               /* former row 1 is now row 0 */
      row1 = row2;                               /* former row 2 is now row 1 */
      row2 = row3;                               /* former row 3 is now row 2 */
      row3 = row4;                               /* former row 4 is now row 3 */
      row4 = sl;

      col = 0;
      d_outbuf = doutbuf;

      p00 = row0[col]; p10 = row1[col]; p20 = row2[col]; col += nchannel;
      p01 = row0[col]; p11 = row1[col]; p21 = row2[col]; col += nchannel;
      p02 = row0[col]; p12 = row1[col]; p22 = row2[col]; col += nchannel;
      p03 = row0[col]; p13 = row1[col]; p23 = row2[col]; col += nchannel;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        FIRST_CONVOLVE_THREE_PIXELS(nchannel)
      }

      if (leftover_cols) {
        FIRST_CONVOLVE_ONE_PIXEL(nchannel)
        if (leftover_cols > 1) {
          FIRST_CONVOLVE_ONE_PIXEL(nchannel)
        }
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row3[col]; p10 = row4[col]; col += nchannel;
      p01 = row3[col]; p11 = row4[col]; col += nchannel;
      p02 = row3[col]; p12 = row4[col]; col += nchannel;
      p03 = row3[col]; p13 = row4[col]; col += nchannel;

      /* Process along row, getting 3 output points per loop iteration: */
#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        SECOND_CONVOLVE_THREE_PIXELS(da, nchannel, S32)
      }

      if (leftover_cols) {
        SECOND_CONVOLVE_ONE_PIXEL(da, nchannel, S32)
        if (leftover_cols > 1) {
          SECOND_CONVOLVE_ONE_PIXEL(da, nchannel, S32)
        }
      }

      sl = sl + sll;                             /* set src ptrs to next row */
      dl = da = dl + dll;                        /* set dst ptrs to next row */
    }
  }

  mlib_free( doutbuf );
  return( MLIB_SUCCESS );
}

/*****************************************************************************************************/

mlib_status mlib_c_conv5x5_32ext( mlib_image *dst,        /* source image */
                                  mlib_image *src,        /* destination image */
                                  mlib_s32   dx_l,
                                  mlib_s32   dx_r,
                                  mlib_s32   dy_t,
                                  mlib_s32   dy_b,
                                  mlib_s32   *kern,         /* convolution kernel */
                                  mlib_s32   scalef_expon,  /* scalefactor=2**N */
                                  mlib_s32   cmask)
{
  DEFINE_IDENTIFICATORS(mlib_s32)
  mlib_s32 i, dend;

  scalef = 1.0;

  LOAD_KERNEL_INTO_DOUBLE()

  GET_SRC_DST_PARAMETERS(mlib_s32)

  /* allocate row bufs: */

  doutbuf = (mlib_d64 *)mlib_malloc(wid*sizeof(mlib_d64));
  if (doutbuf == NULL) {
    return( MLIB_FAILURE );
  }

  dend = (dx_r > wid) ? wid : dx_r;
  num_blocks = (wid - dend)/3;
  leftover_cols = (wid - dend) - (num_blocks * 3);

  testchan = 1;
  for ( chan = nchannel-1; chan >= 0; chan--) {
    if ((cmask & testchan) == 0) {
      testchan <<= 1;
      continue;
    }
    testchan <<= 1;
    sl = adr_src + chan;
    dl = da = adr_dst + chan;

  /*
   * set up rowbufs 1, 2, 3, 4;
   * rowbuf 0 is always reloaded inside the loop:
   */

    row = 0;
    row1 = sl; row++;
    if ((row > dy_t) && (row < hgt + TOP + BOTTOM - dy_b)) sl += sll;
    row2 = sl; row++;
    if ((row > dy_t) && (row < hgt + TOP + BOTTOM - dy_b)) sl += sll;
    row3 = sl; row++;
    if ((row > dy_t) && (row < hgt + TOP + BOTTOM - dy_b)) sl += sll;
    row4 = sl; row++;
    if ((row > dy_t) && (row < hgt + TOP + BOTTOM - dy_b)) sl += sll;

    for ( row = 0; row < hgt; row++ ) {

      row0 = row1;                               /* former row 1 is now row 0 */
      row1 = row2;                               /* former row 2 is now row 1 */
      row2 = row3;                               /* former row 3 is now row 2 */
      row3 = row4;                               /* former row 4 is now row 3 */
      row4 = sl;

      col = 0;
      d_outbuf = doutbuf;
      i = 0;

      p00 = row0[col]; p10 = row1[col]; p20 = row2[col]; i++;
      if ((i > dx_l) && (i < wid + LEFT + RIGHT - dx_r)) col += nchannel;
      p01 = row0[col]; p11 = row1[col]; p21 = row2[col]; i++;
      if ((i > dx_l) && (i < wid + LEFT + RIGHT - dx_r)) col += nchannel;
      p02 = row0[col]; p12 = row1[col]; p22 = row2[col]; i++;
      if ((i > dx_l) && (i < wid + LEFT + RIGHT - dx_r)) col += nchannel;
      p03 = row0[col]; p13 = row1[col]; p23 = row2[col]; col += nchannel;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        FIRST_CONVOLVE_THREE_PIXELS(nchannel)
      }

      if (leftover_cols) {
        FIRST_CONVOLVE_ONE_PIXEL(nchannel)
        if (leftover_cols > 1) {
          FIRST_CONVOLVE_ONE_PIXEL(nchannel)
        }
      }

      for (i = 0; i < dend; i++) {
        col -= nchannel;
        FIRST_CONVOLVE_ONE_PIXEL(nchannel)
      }

      col = 0;
      d_outbuf = doutbuf;
      i = 0;

      p00 = row3[col]; p10 = row4[col]; i++;
      if ((i > dx_l) && (i < wid + LEFT + RIGHT - dx_r)) col += nchannel;
      p01 = row3[col]; p11 = row4[col]; i++;
      if ((i > dx_l) && (i < wid + LEFT + RIGHT - dx_r)) col += nchannel;
      p02 = row3[col]; p12 = row4[col]; i++;
      if ((i > dx_l) && (i < wid + LEFT + RIGHT - dx_r)) col += nchannel;
      p03 = row3[col]; p13 = row4[col]; p23 = row2[col]; col += nchannel;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        SECOND_CONVOLVE_THREE_PIXELS(da, nchannel, S32)
      }

      if (leftover_cols) {
        SECOND_CONVOLVE_ONE_PIXEL(da, nchannel, S32)
        if (leftover_cols > 1) {
          SECOND_CONVOLVE_ONE_PIXEL(da, nchannel, S32)
        }
      }

      for (i = 0; i < dend; i++) {
        col -= nchannel;
        SECOND_CONVOLVE_ONE_PIXEL(da, nchannel, S32)
      }

      if (row < hgt - dy_b - 1) sl = sl + sll;   /* set src ptrs to next row */
      dl = da = dl + dll;                        /* set dst ptrs to next row */
    }
  }

  mlib_free( doutbuf );
  return( MLIB_SUCCESS );
}

/*****************************************************************************************************/

mlib_status mlib_c_conv5x5_16nw( mlib_image *dst,        /* source image */
                                 mlib_image *src,        /* destination image */
                                 mlib_s32 *kern,         /* convolution kernel */
                                 mlib_s32 scalef_expon,  /* scalefactor=2**N */
                                 mlib_s32 cmask)
{
  DEFINE_IDENTIFICATORS(mlib_s16)
  DEFINE_DOP_IDENTIFICATORS(mlib_s16)

#ifndef MLIB_USE_FTOI_CLAMPING
  scalef = 1.0;
#else
  scalef = 65536.0;
#endif

  LOAD_KERNEL_INTO_DOUBLE()

  GET_SRC_DST_PARAMETERS(mlib_s16)

  /* allocate row bufs: */

  rowbuf[0] = (mlib_s32 *)mlib_malloc(6 * wid*sizeof(mlib_s32));
  if (rowbuf[0] == NULL) {
    return( MLIB_FAILURE );
  }
  doutbuf = (mlib_d64 *)mlib_malloc(wid*sizeof(mlib_d64));
  if (doutbuf == NULL) {
    mlib_free( rowbuf[0] );
    return( MLIB_FAILURE );
  }
  rowbuf[1] = rowbuf[0] + wid ;
  rowbuf[2] = rowbuf[1] + wid ;
  rowbuf[3] = rowbuf[2] + wid ;
  rowbuf[4] = rowbuf[3] + wid ;
  outbuf    = rowbuf[4] + wid ;

  num_blocks = (wid - 4)/3;                      /* (wid-4): no output on 2 first, last columns */
  leftover_cols = (wid - 4) - (num_blocks * 3);

  hgt -= 4;                                      /* boundaries: no output for 2 first & last rows */

  testchan = 1;
  for ( chan = nchannel-1; chan >= 0; chan--) {
    if ((cmask & testchan) == 0) {
      testchan <<= 1;
      continue;
    }
    testchan <<= 1;
    sl = sa = adr_src + chan;
    dl = da = adr_dst + chan;
    dl = da += (dll + nchannel) * 2;

  /*
   * set up rowbufs 1, 2, 3, 4;
   * rowbuf 0 is always reloaded inside the loop:
   */
    for ( row = 1; row <= 4; row++ ) {           /* set up 2 line buffers */
      prow = rowbuf[row];                        /* ptr to int dst */
      for ( col = 0; col < wid; col++ ) {        /* copy shorts -> int buffer */
        *prow++ = (int)*sa;
        sa += nchannel;
      }
      sl = sa = sl + sll;                        /* set src ptrs to next row */
    }

    row0 = rowbuf[0];                            /* set up line buf ptrs */
    row1 = rowbuf[1];
    row2 = rowbuf[2];
    row3 = rowbuf[3];
    row4 = rowbuf[4];

    for ( row = 0; row < hgt; row++ ) {

      pout = outbuf;
      prow = row0;                               /* save ptr to oldest row (load new row in this buf) */
      row0 = row1;                               /* former row 1 is now row 0 */
      row1 = row2;                               /* former row 2 is now row 1 */
      row2 = row3;                               /* former row 3 is now row 2 */
      row3 = row4;                               /* former row 4 is now row 3 */
      row4 = prow;                               /* former row0 buf is new row 4; fill with new data */

      for ( col = 0; col < wid; col++ ) {        /* copy next row to int buffer */
        *prow++ = (int)*sa;
        sa += nchannel;
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row0[col]; p10 = row1[col]; p20 = row2[col]; col++;
      p01 = row0[col]; p11 = row1[col]; p21 = row2[col]; col++;
      p02 = row0[col]; p12 = row1[col]; p22 = row2[col]; col++;
      p03 = row0[col]; p13 = row1[col]; p23 = row2[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        FIRST_CONVOLVE_THREE_PIXELS(1)
      }

      if (leftover_cols) {
        FIRST_CONVOLVE_ONE_PIXEL(1)
        if (leftover_cols > 1) {
          FIRST_CONVOLVE_ONE_PIXEL(1)
        }
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row3[col]; p10 = row4[col]; col++;
      p01 = row3[col]; p11 = row4[col]; col++;
      p02 = row3[col]; p12 = row4[col]; col++;
      p03 = row3[col]; p13 = row4[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        SECOND_CONVOLVE_THREE_PIXELS(pout, 1, S16)
      }

      if (leftover_cols) {
        SECOND_CONVOLVE_ONE_PIXEL(pout, 1, S16)
        if (leftover_cols > 1) {
          SECOND_CONVOLVE_ONE_PIXEL(pout, 1, S16)
        }
      }

      pout = outbuf;
      for ( col = 0; col < wid - 4; col++ ) {    /* copy int outbuf to short dest buf */
        isum = *pout++;
#ifndef MLIB_USE_FTOI_CLAMPING
        *da = isum;
#else
        *da = (isum >> 16);
#endif
        da += nchannel;
      }
      sl = sa = sl + sll;                        /* set src ptrs to next row */
      dl = da = dl + dll;                        /* set dst ptrs to next row */
    }
  }

  mlib_free( rowbuf[0] );
  mlib_free( doutbuf );
  return( MLIB_SUCCESS );
}

/*****************************************************************************************************/

mlib_status mlib_c_conv5x5_16ext( mlib_image *dst,        /* source image */
                                  mlib_image *src,        /* destination image */
                                  mlib_s32   dx_l,
                                  mlib_s32   dx_r,
                                  mlib_s32   dy_t,
                                  mlib_s32   dy_b,
                                  mlib_s32   *kern,         /* convolution kernel */
                                  mlib_s32   scalef_expon,  /* scalefactor=2**N */
                                  mlib_s32   cmask)
{
  DEFINE_IDENTIFICATORS(mlib_s16)
  DEFINE_DOP_IDENTIFICATORS(mlib_s16)
  mlib_s32 swid;

#ifndef MLIB_USE_FTOI_CLAMPING
  scalef = 1.0;
#else
  scalef = 65536.0;
#endif

  LOAD_KERNEL_INTO_DOUBLE()

  GET_SRC_DST_PARAMETERS(mlib_s16)

  swid = wid + LEFT + RIGHT;

  /* allocate row bufs: */

  rowbuf[0] = (mlib_s32 *)mlib_malloc(6 * swid*sizeof(mlib_s32));
  if (rowbuf[0] == NULL) {
    return( MLIB_FAILURE );
  }
  doutbuf = (mlib_d64 *)mlib_malloc(swid*sizeof(mlib_d64));
  if (doutbuf == NULL) {
    mlib_free( rowbuf[0] );
    return( MLIB_FAILURE );
  }
  rowbuf[1] = rowbuf[0] + swid ;
  rowbuf[2] = rowbuf[1] + swid ;
  rowbuf[3] = rowbuf[2] + swid ;
  rowbuf[4] = rowbuf[3] + swid ;
  outbuf    = rowbuf[4] + swid ;

  num_blocks = (wid)/3;
  leftover_cols = (wid) - (num_blocks * 3);

  swid -= (dx_l + dx_r);

  testchan = 1;
  for ( chan = nchannel - 1; chan >= 0; chan--) {
    if ((cmask & testchan) == 0) {
      testchan <<= 1;
      continue;
    }
    testchan <<= 1;
    sl = sa = adr_src + chan;
    dl = da = adr_dst + chan;

  /*
   * set up rowbufs 1, 2, 3, 4;
   * rowbuf 0 is always reloaded inside the loop:
   */

    for (row = 1; row <= (TOP + BOTTOM); row++) {
      LOAD_ONE_ROW_EXT(row)
      if ((row > dy_t) && (row < hgt + TOP + BOTTOM - dy_b)) {
        sa = (sl += sll);
      } else  sa = sl;
    }

    row0 = rowbuf[0];                            /* set up line buf ptrs */
    row1 = rowbuf[1];
    row2 = rowbuf[2];
    row3 = rowbuf[3];
    row4 = rowbuf[4];

    for ( row = 0; row < hgt; row++ ) {

      pout = outbuf;
      prow = row0;                               /* save ptr to oldest row (load new row in this buf) */
      row0 = row1;                               /* former row 1 is now row 0 */
      row1 = row2;                               /* former row 2 is now row 1 */
      row2 = row3;                               /* former row 3 is now row 2 */
      row3 = row4;                               /* former row 4 is now row 3 */
      row4 = prow;                               /* former row0 buf is new row 4; fill with new data */

      if (row < hgt - dy_b) {
        prow += dx_l;
        for (col = 0; col < swid; col++) {       /* copy next row to int buffer */
          *prow++ = (mlib_s32)(*sa);
          sa += nchannel;
        }

        for (col = 0; col < dx_l; col++) row4[col] = row4[dx_l];
        for (col = 0; col < dx_r; col++) row4[swid + dx_l + col] = row4[swid + dx_l - 1];
      } else  row4 = row3;

      col = 0;
      d_outbuf = doutbuf;

      p00 = row0[col]; p10 = row1[col]; p20 = row2[col]; col++;
      p01 = row0[col]; p11 = row1[col]; p21 = row2[col]; col++;
      p02 = row0[col]; p12 = row1[col]; p22 = row2[col]; col++;
      p03 = row0[col]; p13 = row1[col]; p23 = row2[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        FIRST_CONVOLVE_THREE_PIXELS(1)
      }

      if (leftover_cols) {
        FIRST_CONVOLVE_ONE_PIXEL(1)
        if (leftover_cols > 1) {
          FIRST_CONVOLVE_ONE_PIXEL(1)
        }
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row3[col]; p10 = row4[col]; col++;
      p01 = row3[col]; p11 = row4[col]; col++;
      p02 = row3[col]; p12 = row4[col]; col++;
      p03 = row3[col]; p13 = row4[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        SECOND_CONVOLVE_THREE_PIXELS(pout, 1, S16)
      }

      if (leftover_cols) {
        SECOND_CONVOLVE_ONE_PIXEL(pout, 1, S16)
        if (leftover_cols > 1) {
          SECOND_CONVOLVE_ONE_PIXEL(pout, 1, S16)
        }
      }

      pout = outbuf;
      for ( col = 0; col < wid; col++ ) {       /* copy int outbuf to short dest buf */
        isum = *pout++;
#ifndef MLIB_USE_FTOI_CLAMPING
        *da = isum;
#else
        *da = (isum >> 16);
#endif
        da += nchannel;
      }
      sl = sa = sl + sll;                        /* set src ptrs to next row */
      dl = da = dl + dll;                        /* set dst ptrs to next row */
    }
  }

  mlib_free( rowbuf[0] );
  mlib_free( doutbuf );
  return( MLIB_SUCCESS );
}

/*****************************************************************************************************/

mlib_status mlib_c_conv5x5_8nw( mlib_image *dst,        /* source image */
                                mlib_image *src,        /* destination image */
                                mlib_s32 *kern,         /* convolution kernel */
                                mlib_s32 scalef_expon,  /* scalefactor=2**N */
                                mlib_s32 cmask)
{
  DEFINE_IDENTIFICATORS(mlib_u8)
  DEFINE_DOP_IDENTIFICATORS(mlib_u8)
#ifdef MLIB_USE_FTOI_CLAMPING
  mlib_d64 sat = (mlib_d64)0x7F800000;
#endif

#ifndef MLIB_USE_FTOI_CLAMPING
  scalef = 1.0;
#else
  scalef = 16777216.0;
#endif

  LOAD_KERNEL_INTO_DOUBLE()

  GET_SRC_DST_PARAMETERS(mlib_u8)

  /* allocate row bufs: */

  rowbuf[0] = (mlib_s32 *)mlib_malloc(6 * wid*sizeof(mlib_s32));
  if (rowbuf[0] == NULL) {
    return( MLIB_FAILURE );
  }
  doutbuf = (mlib_d64 *)mlib_malloc(wid*sizeof(mlib_d64));
  if (doutbuf == NULL) {
    mlib_free( rowbuf[0] );
    return( MLIB_FAILURE );
  }
  rowbuf[1] = rowbuf[0] + wid ;
  rowbuf[2] = rowbuf[1] + wid ;
  rowbuf[3] = rowbuf[2] + wid ;
  rowbuf[4] = rowbuf[3] + wid ;
  outbuf    = rowbuf[4] + wid ;

  num_blocks = (wid - 4)/3;                      /* (wid-4): no output on 2 first, last columns */
  leftover_cols = (wid - 4) - (num_blocks * 3);

  hgt -= 4;                                      /* boundaries: no output for 2 first & last rows */

  testchan = 1;
  for ( chan = nchannel-1; chan >= 0; chan--) {
    if ((cmask & testchan) == 0) {
      testchan <<= 1;
      continue;
    }
    testchan <<= 1;
    sl = sa = adr_src + chan;
    dl = da = adr_dst + chan;
    dl = da += (dll + nchannel) * 2;

  /*
   * set up rowbufs 1, 2, 3, 4;
   * rowbuf 0 is always reloaded inside the loop:
   */
    for ( row = 1; row <= 4; row++ ) {           /* set up 2 line buffers */
      prow = rowbuf[row];                        /* ptr to int dst */
      for ( col = 0; col < wid; col++ ) {        /* copy shorts -> int buffer */
        *prow++ = (int)*sa;
        sa += nchannel;
      }
      sl = sa = sl + sll;                        /* set src ptrs to next row */
    }

    row0 = rowbuf[0];                            /* set up line buf ptrs */
    row1 = rowbuf[1];
    row2 = rowbuf[2];
    row3 = rowbuf[3];
    row4 = rowbuf[4];

    for ( row = 0; row < hgt; row++ ) {

      pout = outbuf;
      prow = row0;                               /* save ptr to oldest row (load new row in this buf) */
      row0 = row1;                               /* former row 1 is now row 0 */
      row1 = row2;                               /* former row 2 is now row 1 */
      row2 = row3;                               /* former row 3 is now row 2 */
      row3 = row4;                               /* former row 4 is now row 3 */
      row4 = prow;                               /* former row0 buf is new row 4; fill with new data */

      for ( col = 0; col < wid; col++ ) {        /* copy next row to int buffer */
        *prow++ = (int)*sa;
        sa += nchannel;
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row0[col]; p10 = row1[col]; p20 = row2[col]; col++;
      p01 = row0[col]; p11 = row1[col]; p21 = row2[col]; col++;
      p02 = row0[col]; p12 = row1[col]; p22 = row2[col]; col++;
      p03 = row0[col]; p13 = row1[col]; p23 = row2[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        FIRST_CONVOLVE_THREE_PIXELS(1)
      }

      if (leftover_cols) {
        FIRST_CONVOLVE_ONE_PIXEL(1)
        if (leftover_cols > 1) {
          FIRST_CONVOLVE_ONE_PIXEL(1)
        }
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row3[col]; p10 = row4[col]; col++;
      p01 = row3[col]; p11 = row4[col]; col++;
      p02 = row3[col]; p12 = row4[col]; col++;
      p03 = row3[col]; p13 = row4[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        SECOND_CONVOLVE_THREE_PIXELS(pout, 1, U8)
      }

      if (leftover_cols) {
        SECOND_CONVOLVE_ONE_PIXEL(pout, 1, U8)
        if (leftover_cols > 1) {
          SECOND_CONVOLVE_ONE_PIXEL(pout, 1, U8)
        }
      }

      pout = outbuf;
      for ( col = 0; col < wid - 4; col++ ) {    /* copy int outbuf to short dest buf */
        isum = *pout++;
#ifndef MLIB_USE_FTOI_CLAMPING
        *da = isum;
#else
        *da = ((isum >> 24) ^ 0x80);
#endif
        da += nchannel;
      }
      sl = sa = sl + sll;                        /* set src ptrs to next row */
      dl = da = dl + dll;                        /* set dst ptrs to next row */
    }
  }

  mlib_free( rowbuf[0] );
  return( MLIB_SUCCESS );
}

/*****************************************************************************************************/

mlib_status mlib_c_conv5x5_8ext( mlib_image *dst,        /* source image */
                                 mlib_image *src,        /* destination image */
                                 mlib_s32   dx_l,
                                 mlib_s32   dx_r,
                                 mlib_s32   dy_t,
                                 mlib_s32   dy_b,
                                 mlib_s32   *kern,         /* convolution kernel */
                                 mlib_s32   scalef_expon,  /* scalefactor=2**N */
                                 mlib_s32   cmask)
{
  DEFINE_IDENTIFICATORS(mlib_u8)
  DEFINE_DOP_IDENTIFICATORS(mlib_u8)
  mlib_s32 swid;
#ifdef MLIB_USE_FTOI_CLAMPING
  mlib_d64 sat = (mlib_d64)0x7F800000;
#endif

#ifndef MLIB_USE_FTOI_CLAMPING
  scalef = 1.0;
#else
  scalef = 16777216.0;
#endif

  LOAD_KERNEL_INTO_DOUBLE()

  GET_SRC_DST_PARAMETERS(mlib_u8)

  swid = wid + LEFT + RIGHT;

  /* allocate row bufs: */

  rowbuf[0] = (mlib_s32 *)mlib_malloc(6 * swid*sizeof(mlib_s32));
  if (rowbuf[0] == NULL) {
    return( MLIB_FAILURE );
  }
  doutbuf = (mlib_d64 *)mlib_malloc(swid*sizeof(mlib_d64));
  if (doutbuf == NULL) {
    mlib_free( rowbuf[0] );
    return( MLIB_FAILURE );
  }
  rowbuf[1] = rowbuf[0] + swid ;
  rowbuf[2] = rowbuf[1] + swid ;
  rowbuf[3] = rowbuf[2] + swid ;
  rowbuf[4] = rowbuf[3] + swid ;
  outbuf    = rowbuf[4] + swid ;

  num_blocks = (wid)/3;
  leftover_cols = (wid) - (num_blocks * 3);

  swid -= (dx_l + dx_r);

  testchan = 1;
  for ( chan = nchannel - 1; chan >= 0; chan--) {
    if ((cmask & testchan) == 0) {
      testchan <<= 1;
      continue;
    }
    testchan <<= 1;
    sl = sa = adr_src + chan;
    dl = da = adr_dst + chan;

  /*
   * set up rowbufs 1, 2, 3, 4;
   * rowbuf 0 is always reloaded inside the loop:
   */

    for (row = 1; row <= (TOP + BOTTOM); row++) {
      LOAD_ONE_ROW_EXT(row)
      if ((row > dy_t) && (row < hgt + TOP + BOTTOM - dy_b)) {
        sa = (sl += sll);
      } else  sa = sl;
    }

    row0 = rowbuf[0];                            /* set up line buf ptrs */
    row1 = rowbuf[1];
    row2 = rowbuf[2];
    row3 = rowbuf[3];
    row4 = rowbuf[4];

    for ( row = 0; row < hgt; row++ ) {

      pout = outbuf;
      prow = row0;                               /* save ptr to oldest row (load new row in this buf) */
      row0 = row1;                               /* former row 1 is now row 0 */
      row1 = row2;                               /* former row 2 is now row 1 */
      row2 = row3;                               /* former row 3 is now row 2 */
      row3 = row4;                               /* former row 4 is now row 3 */
      row4 = prow;                               /* former row0 buf is new row 4; fill with new data */

      if (row < hgt - dy_b) {
        prow += dx_l;
        for (col = 0; col < swid; col++) {       /* copy next row to int buffer */
          *prow++ = (mlib_s32)(*sa);
          sa += nchannel;
        }

        for (col = 0; col < dx_l; col++) row4[col] = row4[dx_l];
        for (col = 0; col < dx_r; col++) row4[swid + dx_l + col] = row4[swid + dx_l - 1];
      } else  row4 = row3;

      col = 0;
      d_outbuf = doutbuf;

      p00 = row0[col]; p10 = row1[col]; p20 = row2[col]; col++;
      p01 = row0[col]; p11 = row1[col]; p21 = row2[col]; col++;
      p02 = row0[col]; p12 = row1[col]; p22 = row2[col]; col++;
      p03 = row0[col]; p13 = row1[col]; p23 = row2[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        FIRST_CONVOLVE_THREE_PIXELS(1)
      }

      if (leftover_cols) {
        FIRST_CONVOLVE_ONE_PIXEL(1)
        if (leftover_cols > 1) {
          FIRST_CONVOLVE_ONE_PIXEL(1)
        }
      }

      col = 0;
      d_outbuf = doutbuf;

      p00 = row3[col]; p10 = row4[col]; col++;
      p01 = row3[col]; p11 = row4[col]; col++;
      p02 = row3[col]; p12 = row4[col]; col++;
      p03 = row3[col]; p13 = row4[col]; col++;

      /* Process along row, getting 3 output points per loop iteration: */

#ifdef __SUNPRO_C
#pragma pipeloop(0)
#endif /* __SUNPRO_C */
      for ( block = 0; block < num_blocks; block++ ) {
        SECOND_CONVOLVE_THREE_PIXELS(pout, 1, U8)
      }

      if (leftover_cols) {
        SECOND_CONVOLVE_ONE_PIXEL(pout, 1, U8)
        if (leftover_cols > 1) {
          SECOND_CONVOLVE_ONE_PIXEL(pout, 1, U8)
        }
      }

      pout = outbuf;
      for ( col = 0; col < wid; col++ ) {    /* copy int outbuf to short dest buf */
        isum = *pout++;
#ifndef MLIB_USE_FTOI_CLAMPING
        *da = isum;
#else
        *da = ((isum >> 24) ^ 0x80);
#endif
        da += nchannel;
      }
      sl = sa = sl + sll;                        /* set src ptrs to next row */
      dl = da = dl + dll;                        /* set dst ptrs to next row */
    }
  }

  mlib_free( rowbuf[0] );
  return( MLIB_SUCCESS );
}

/*****************************************************************************************************/
