/*
 * @(#)mlib_c_ImageConvMxN.c	1.8 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


#ifdef _MSC_VER
#pragma warning(disable:4068 4244 4101)
#endif /*_MSC_VER*/

/*
 * @(#)mlib_c_ImageConvMxN.c	4.7	99/04/28 SMI
 *
 * Copyright (c) 1996 - 1999, Sun Microsystems, Inc.
 */

/* Copyright (C) 1996, 1999, Sun Microsystems, Inc. */

#ifdef __SUNPRO_C
#pragma ident	"@(#)mlib_c_ImageConvMxN.c	1.24	99/04/23 SMI"
#endif /* __SUNPRO_C */

/*
 * FUNCTION
 *      mlib_ImageConvMxN - image convolution with edge condition
 *
 * SYNOPSIS
 *      mlib_status mlib_ImageConvMxN(mlib_image *dst,
 *                                    mlib_image *src,
 *                                    mlib_s32   *kernel,
 *                                    mlib_s32   m,
 *                                    mlib_s32   n,
 *                                    mlib_s32   dm,
 *                                    mlib_s32   dn,
 *                                    mlib_s32   scale,
 *                                    mlib_s32   cmask,
 *                                    mlib_edge  edge)
 *
 * ARGUMENTS
 *      dst       Pointer to destination image.
 *      src       Pointer to source image.
 *      m         Kernel width (m must be larger than 1).
 *      n         Kernel height (n must be larger than 1).
 *      dm, dn    Position of key element in convolution kernel.
 *      kernel    Pointer to convolution kernel.
 *      scale     The scaling factor to convert the input integer
 *                coefficients into floating-point coefficients:
 *                floating-point coefficient = integer coefficient * 2^(-scale)
 *      cmask     Channel mask to indicate the channels to be convolved.
 *                Each bit of which represents a channel in the image. The
 *                channels corresponded to 1 bits are those to be processed.
 *      edge      Type of edge condition.
 *
 * DESCRIPTION
 *      2-D convolution, MxN kernel.
 *
 *      The center of the source image is mapped to the center of the
 *      destination image.
 *      The unselected channels are not overwritten. If both src and dst have
 *      just one channel, cmask is ignored.
 *
 *      The edge condition can be one of the following:
 *              MLIB_EDGE_DST_NO_WRITE  (default)
 *              MLIB_EDGE_DST_FILL_ZERO
 *              MLIB_EDGE_DST_COPY_SRC
 *              MLIB_EDGE_SRC_EXTEND
 *
 * RESTRICTION
 *      The src and the dst must be the same type and have same number
 *      of channels (1, 2, 3, or 4).
 *      m > 1.   n > 1.
 *      0 <= dm < m. 0 <= dn < n.
 *      For data type MLIB_BYTE - 16 <= scale <= 31 (to be compatible with VIS
 *      version).
 *      For data type MLIB_SHORT - 17 <= scale <= 32 (to be compatible with VIS
 *      version).
 *      For data type MLIB_INT - scale >= 0.
 */

/***************************************************************/

#include <stdlib.h>
#include <mlib_image.h>
#include <mlib_ImageCheck.h>
#include <mlib_SysMath.h>

/***************************************************************/

#ifdef MLIB_TEST
#define MLIB_IMAGE(F) mlib_c_Image##F
#else
#define MLIB_IMAGE(F) mlib_Image##F
#endif

/***************************************************************/

mlib_image *mlib_ImageSet(mlib_image *image, 
                          mlib_type  type, 
                          mlib_s32   channels,
                          mlib_s32   width, 
                          mlib_s32   height, 
                          mlib_s32   stride, 
                          void       *data);

mlib_status mlib_c_ImageConvCopyEdge(mlib_image *dst, 
                                     mlib_image *src, 
                                     mlib_s32   dx_l,
                                     mlib_s32   dx_r, 
                                     mlib_s32   dy_t, 
                                     mlib_s32   dy_b, 
                                     mlib_s32   cmask);
                                     
mlib_status mlib_c_ImageConvClearEdge(mlib_image *img, 
                                      mlib_s32   dx_l, 
                                      mlib_s32   dx_r,
                                      mlib_s32   dy_t, 
                                      mlib_s32   dy_b, 
                                      mlib_s32   *color, 
                                      mlib_s32   cmask);

/***************************************************************/

void mlib_ImageConvMxN_U8(mlib_image *dst,
                          mlib_image *src,
                          mlib_d64   *dkernel,
                          mlib_s32   m,
                          mlib_s32   n,
                          mlib_u8    *acmask,
                          mlib_d64   *dsa);

void mlib_ImageConvMxN_U8_ext(mlib_image *dst,
                              mlib_image *src,
                              mlib_d64   *dkernel,
                              mlib_s32   m,
                              mlib_s32   n,
                              mlib_s32   dx_l,
                              mlib_s32   dx_r,
                              mlib_s32   dy_t,
                              mlib_s32   dy_b,
                              mlib_u8    *acmask,
                              mlib_d64   *dsa);

void mlib_ImageConvMxN_S16(mlib_image *dst,
                           mlib_image *src,
                           mlib_d64   *dkernel,
                           mlib_s32   m,
                           mlib_s32   n,
                           mlib_u8    *acmask,
                           mlib_d64   *dsa);

void mlib_ImageConvMxN_S16_ext(mlib_image *dst,
                               mlib_image *src,
                               mlib_d64   *dkernel,
                               mlib_s32   m,
                               mlib_s32   n,
                               mlib_s32   dx_l,
                               mlib_s32   dx_r,
                               mlib_s32   dy_t,
                               mlib_s32   dy_b,
                               mlib_u8    *acmask,
                               mlib_d64   *dsa);

void mlib_ImageConvMxN_S32(mlib_image *dst,
                           mlib_image *src,
                           mlib_d64   *dkernel,
                           mlib_s32   m,
                           mlib_s32   n,
                           mlib_u8    *acmask,
                           mlib_d64   *dsa);

void mlib_ImageConvMxN_S32_ext(mlib_image *dst,
                               mlib_image *src,
                               mlib_d64   *dkernel,
                               mlib_s32   m,
                               mlib_s32   n,
                               mlib_s32   dx_l,
                               mlib_s32   dx_r,
                               mlib_s32   dy_t,
                               mlib_s32   dy_b,
                               mlib_u8    *acmask,
                               mlib_d64   *dsa);

/***************************************************************/

mlib_status MLIB_IMAGE(ConvMxN)(mlib_image *dst,
                                mlib_image *src,
                                mlib_s32   *kernel,
                                mlib_s32   m,
                                mlib_s32   n,
                                mlib_s32   dm,
                                mlib_s32   dn,
                                mlib_s32   scale,
                                mlib_s32   cmask,
                                mlib_edge  edge)

{
  mlib_image dst1[1], src1[1];
  mlib_type  dst_dtype, src_dtype;
  mlib_d64   dspace[1024], *dsa = dspace;
  mlib_d64   akernel[256], *dkernel = akernel, fscale = 1.f;
  mlib_s32   scale_exp = scale;
  mlib_u8    acmask[4];
  
  if (dst == NULL || src == NULL || kernel == NULL)
    return MLIB_FAILURE;

  MLIB_IMAGE_TYPE_EQUAL(dst, src);
  MLIB_IMAGE_CHAN_EQUAL(dst, src);

  switch(mlib_ImageGetType(dst)) {
    case MLIB_BYTE: 
      if (scale < 16 || scale > 31) return MLIB_FAILURE;
      break;
    case MLIB_SHORT:
      if (scale < 17 || scale > 32 ) return MLIB_FAILURE;
      break;
    case MLIB_INT:
      if (scale < 0) return MLIB_FAILURE;
  }
  
  while (scale_exp > 30) {
    fscale /= (1 << 30);
    scale_exp -= 30;
  }
  fscale /= (1 << scale_exp);
    
 {
    mlib_u8   *da, *sa;
    mlib_s32  dx, dy, dxs, dxd, dys, dyd, dw, dh;
    mlib_s32  dx_l, dx_r, dy_t, dy_b, dw1, dh1;
    mlib_s32  src_wid, src_hgt, dst_wid, dst_hgt, src_nch, dst_nch=0, dlb, slb;
    mlib_s32  i, nchmul = dst_nch;

    MLIB_IMAGE_GET_ALL_PARAMS(dst, dst_dtype, dst_nch, dst_wid, dst_hgt, dlb,
      da);
    MLIB_IMAGE_GET_ALL_PARAMS(src, src_dtype, src_nch, src_wid, src_hgt, slb,
      sa);
    
    if (m*n > 256) dkernel = mlib_malloc(m*n*sizeof(mlib_d64));
    for(i = 0; i < m*n; i++) dkernel[i] = kernel[i]*fscale;
    
    nchmul = dst_nch << ((dst_dtype == MLIB_BYTE) ? 0 : 
                         (dst_dtype == MLIB_SHORT) ? 1 : 2);
    cmask   = (dst_nch == 1) ? 1 : cmask;

    for(i = 0; i < dst_nch; i++) 
      acmask[i] = (cmask & (1 << (dst_nch - i - 1)));

    dw1 = (src_wid < dst_wid) ? src_wid : dst_wid;
    dh1 = (src_hgt < dst_hgt) ? src_hgt : dst_hgt;
    dx  = (src_wid - dst_wid);
    dy  = (src_hgt - dst_hgt);
    dxs = ((dx > 0) ? (dx+1)>>1 : 0);
    dxd = ((dx > 0) ? 0 : (-dx)>>1);
    dys = ((dy > 0) ? (dy+1)>>1 : 0);
    dyd = ((dy > 0) ? 0 : (-dy)>>1);

    dx_l = dm - dxs;
    dx_r = dxs - dx + (m - dm - 1);
    dy_t = dn - dys;
    dy_b = dys - dy + (n - dn - 1);
    dx_l = ((dx_l < 0) ? 0 : dx_l);
    dx_r = ((dx_r < 0) ? 0 : (dx_r > (m - dm - 1) ? (m - dm - 1) : dx_r));
    dy_t = ((dy_t < 0) ? 0 : dy_t);
    dy_b = ((dy_b < 0) ? 0 : (dy_b > (n - dn - 1) ? (n - dn - 1) : dy_b));
    
    dw   = dw1 - dx_l - dx_r;
    dh   = dh1 - dy_t - dy_b;
    
    if (edge != MLIB_EDGE_DST_NO_WRITE) {
      mlib_s32 color[4] = {0, 0, 0, 0};
      
      if (edge != MLIB_EDGE_SRC_EXTEND) {
        if ((dx_l + dx_r) > dw1) {
          dx_l = dw1; dx_r = 0;
        }
        if ((dy_t + dy_b) > dh1) {
          dy_t = dh1; dy_b = 0;
        }
      }

      switch(edge) {
        case MLIB_EDGE_DST_FILL_ZERO:
          if (dw1 <= 0 || dh1 <= 0) break;
          
          mlib_ImageSet(dst1, dst_dtype, dst_nch, dw1, dh1, dlb,
            da + nchmul*dxd + dyd*dlb);
          mlib_c_ImageConvClearEdge(dst1, dx_l, dx_r, dy_t, dy_b, color, cmask);
          break;
        case MLIB_EDGE_DST_COPY_SRC:
          if (dw1 <= 0 || dh1 <= 0) break;
          
          mlib_ImageSet(dst1, dst_dtype, dst_nch, dw1, dh1, dlb,
            da + nchmul*dxd + dyd*dlb);
          mlib_ImageSet(src1, src_dtype, src_nch, dw1, dh1, slb,
            sa + nchmul*dxs + dys*slb);
          mlib_c_ImageConvCopyEdge(dst1, src1, dx_l, dx_r, dy_t, dy_b, cmask);
          break;
        case MLIB_EDGE_SRC_EXTEND:
          if (dw1 <= 0 || dh1 <= 0) {
            if (dkernel != akernel) mlib_free(dkernel);
            return MLIB_SUCCESS;
          }
          
          mlib_ImageSet(dst1, dst_dtype, dst_nch, dw1, dh1, dlb,
            da + nchmul*(dxd) + (dyd)*dlb);
          mlib_ImageSet(src1, src_dtype, src_nch, dw1, dh1, slb,
            sa + nchmul*(dxs + dx_l - dm) + (dys + dy_t - dn)*slb);

          if (1024 < 3*dw1 + m) dsa  = mlib_malloc(sizeof(mlib_d64)*(3*dw1+m));

          switch(dst_dtype) {
            case MLIB_BYTE:
              mlib_ImageConvMxN_U8_ext(dst1, src1, dkernel, m, n, 
                                       dx_l, dx_r, dy_t, dy_b, acmask, dsa);
              break;
            case MLIB_SHORT:
              mlib_ImageConvMxN_S16_ext(dst1, src1, dkernel, m, n, 
                                        dx_l, dx_r, dy_t, dy_b, acmask, dsa);
              break;
            case MLIB_INT:
              mlib_ImageConvMxN_S32_ext(dst1, src1, dkernel, m, n, 
                                        dx_l, dx_r, dy_t, dy_b, acmask, dsa);
          }
          if (dsa != dspace) mlib_free(dsa);
          if (dkernel != akernel) mlib_free(dkernel);
          
          return MLIB_SUCCESS;
      }
    }
    
    dxd += dx_l;
    dyd += dy_t;
    dxs += dx_l - dm;
    dys += dy_t - dn;

    if (((dx_l + dx_r) >= dw1) || (dy_t + dy_b) >= dh1) return MLIB_SUCCESS;
    if ((dw <= 0) || (dh <= 0)) return MLIB_SUCCESS;

    da     += (nchmul*dxd + dyd*dlb);
    sa     += (nchmul*dxs + dys*slb);

    mlib_ImageSet(dst1, dst_dtype, dst_nch, dw, dh, dlb, da);
    mlib_ImageSet(src1, src_dtype, src_nch, dw+m, dh+n, slb, sa);

    if (1024 < 3*dw + m) dsa  = mlib_malloc(sizeof(mlib_d64)*(3*dw+m));
 }

  switch(dst_dtype) {
    case MLIB_BYTE:
      mlib_ImageConvMxN_U8(dst1, src1, dkernel, m, n, acmask, dsa);
      break;
    case MLIB_SHORT:
      mlib_ImageConvMxN_S16(dst1, src1, dkernel, m, n, acmask, dsa);
      break;
    case MLIB_INT:
      mlib_ImageConvMxN_S32(dst1, src1, dkernel, m, n, acmask, dsa);
  }
  
  if (dsa != dspace) mlib_free(dsa);
  if (dkernel != akernel) mlib_free(dkernel);

  return MLIB_SUCCESS;
}

/***************************************************************/
