/*
 * @(#)AreaSpiroBlaster.java	1.3 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

import java.awt.geom.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Arrays;

public class AreaSpiroBlaster {
    public static int TRANS_X    = 0;
    public static int TRANS_Y    = 1;
    public static int TRANS_XY   = 2;
    public static int TRANSLATES = 3;
    public static int SCALE_X  = 4;
    public static int SCALE_Y  = 5;
    public static int SCALE_XY = 6;
    public static int SCALES   = 7;
    public static int SPIRO_INTERIOR = 8;
    public static int SPIRO_EXTERIOR = 9;
    public static int SPIRO_EDGE     = 10;
    public static int SPIROS         = 11;
    public static int MINI_SPIRO_INTERIOR = 12;
    public static int MINI_SPIRO_EXTERIOR = 13;
    public static int MINI_SPIRO_EDGE     = 14;
    public static int MINI_SPIROS         = 15;
    public static int ARCSAME   = 16;
    public static int ARCDIFF   = 17;
    public static int ARCS      = 18;

    public static int NUMTESTS = 19;

    public static String[] testnames = {
	"transx",        "sliding ellipses offset in X",
	"transy",        "sliding ellipses offset in Y",
	"transxy",       "sliding ellipses offset in X & Y",
	"translates",    "sliding ellipses at all offsets",
	"scalex",        "ellipses scaled in X",
	"scaley",        "ellipses scaled in Y",
	"scalexy",       "ellipses scaled in X & Y",
	"scales",        "ellipses scaled in all directions",
	"spiroint",      "lots of ellipses rotating around interior point",
	"spiroext",      "lots of ellipses rotating around exterior point",
	"spiroedge",     "lots of ellipses rotating around edge point",
	"spiros",        "lots of ellipses rotating around all points",
	"minispiroint",  "a few ellipses rotating around interior point",
	"minispiroext",  "a few ellipses rotating around exterior point",
	"minispiroedge", "a few ellipses rotating around edge point",
	"minispiros",    "a few ellipses rotating around all points",
	"arcsame",       "rotating pie arcs with same radius",
	"arcdiff",       "rotating pie arcs with different radius",
	"arcs",          "all rotating pie arcs",
    };

    public static boolean verbose = false;

    public static boolean DoAdd = false;
    public static boolean DoSub = false;
    public static boolean DoXor = false;
    public static boolean DoInt = false;

    public static AreaSpiroCanvas AddCanvas;
    public static AreaSpiroCanvas SubCanvas;
    public static AreaSpiroCanvas XorCanvas;
    public static AreaSpiroCanvas IntCanvas;

    public static long DisplayDelay = 0;
    public static boolean DisplayAA = false;

    public static void usage(String complaint) {
	if (complaint != null) {
	    System.err.println(complaint);
	}
	String usagestr[] = {
	    "java AreaSpiroBlaster [options|testnames|opnames]*",
	    "  where [options] is a list of any of:",
	    "    -verbose      print info for each test",
	    "    -display      display results on screen",
	    "    -delay=<ms>   minimum delay between result displays",
	    "    -aa           display with Antia-Aliasing",
	    "  where [opnames] is a list of any of:",
	    "    doadd         perform add operation",
	    "    dosub         perform subtract operation",
	    "    doxor         perform exclusive OR operation",
	    "    doint         perform intersect operation",
	    "  where [testnames] is a list of any of:",
	};
	for (int i = 0; i < usagestr.length; i++) {
	    System.err.println(usagestr[i]);
	}
	int maxlen = 0;
	for (int i = 0; i < testnames.length; i += 2) {
	    maxlen = Math.max(maxlen, testnames[i].length());
	}
	char[] padchars = new char[maxlen+4];
	Arrays.fill(padchars, ' ');
	String pad = new String(padchars);
	for (int i = 0; i < testnames.length; i += 2) {
	    String name = testnames[i+0];
	    String desc = testnames[i+1];
	    System.err.println("    "+name+pad.substring(name.length())+desc);
	}
    }

    public static void main(String argv[]) {
	boolean dotest[] = new boolean[NUMTESTS];
	boolean testspecified = false;
	boolean display = false;
	boolean DoTemp = false;
    options:
	for (int i = 0; i < argv.length; i++) {
	    String arg = argv[i].toLowerCase();
	    for (int j = 0; j < testnames.length; j += 2) {
		if (arg.equals(testnames[j])) {
		    testspecified = true;
		    dotest[j/2] = true;
		    continue options;
		}
	    }
	    if (arg.equals("-verbose")) {
		verbose = true;
	    } else if (arg.equals("-display")) {
		display = true;
	    } else if (arg.startsWith("-delay=")) {
		try {
		    DisplayDelay = Long.parseLong(arg.substring(7));
		} catch (NumberFormatException e) {
		    usage("bad delay number: "+argv[i]);
		    System.exit(1);
		}
	    } else if (arg.equals("-aa")) {
		DisplayAA = true;
	    } else if (arg.equals("doadd")) {
		DoAdd = true;
	    } else if (arg.equals("dosub")) {
		DoSub = true;
	    } else if (arg.equals("doxor")) {
		DoXor = true;
	    } else if (arg.equals("doint")) {
		DoInt = true;
	    } else if (arg.equals("temp")) {
		DoTemp = true;
	    } else {
		usage("unrecognized argument: "+argv[i]);
		System.exit(1);
	    }
	}
	if (!testspecified) {
	    // Nothing specified, set all to true
	    for (int i = 0; i < dotest.length; i++) {
		dotest[i] = true;
	    }
	}
	if (!DoAdd && !DoSub && !DoXor && !DoInt) {
	    DoAdd = DoSub = DoXor = DoInt = true;
	}
	if (display) {
	    if (DoAdd) {
		AddCanvas = new AreaSpiroCanvas();
		AddCanvas.show("Add", 0, 0);
	    }
	    if (DoSub) {
		SubCanvas = new AreaSpiroCanvas();
		SubCanvas.show("Subtract", 1, 0);
	    }
	    if (DoXor) {
		XorCanvas = new AreaSpiroCanvas();
		XorCanvas.show("Exclusive Or", 0, 1);
	    }
	    if (DoInt) {
		IntCanvas = new AreaSpiroCanvas();
		IntCanvas.show("Intersect", 1, 1);
	    }
	}
	Thread.currentThread().setPriority(Thread.MIN_PRIORITY+1);
	if (DoTemp) {
	    testSpiro(.25, 3, 10);
	    return;
	}
	if (dotest[TRANS_X] || dotest[TRANSLATES]) {
	    testTranslate(MICROSTEPS, true, false);
	}
	if (dotest[TRANS_Y] || dotest[TRANSLATES]) {
	    testTranslate(MICROSTEPS, false, true);
	}
	if (dotest[TRANS_XY] || dotest[TRANSLATES]) {
	    testTranslate(MICROSTEPS, true, true);
	}
	if (dotest[SCALE_X] || dotest[SCALES]) {
	    testScale(MINISTEPS, true, false);
	}
	if (dotest[SCALE_Y] || dotest[SCALES]) {
	    testScale(MINISTEPS, false, true);
	}
	if (dotest[SCALE_XY] || dotest[SCALES]) {
	    testScale(MINISTEPS, true, true);
	}
	if (dotest[SPIRO_INTERIOR] || dotest[SPIROS]) {
	    testSpiro(-.25, 36, 10);
	}
	if (dotest[SPIRO_EXTERIOR] || dotest[SPIROS]) {
	    testSpiro(.25, 36, 10);
	}
	if (dotest[SPIRO_EDGE] || dotest[SPIROS]) {
	    testSpiro(0, 36, 10);
	}
	if (dotest[MINI_SPIRO_INTERIOR] || dotest[MINI_SPIROS]) {
	    testSpiro(-.25, 12, 30);
	}
	if (dotest[MINI_SPIRO_EXTERIOR] || dotest[MINI_SPIROS]) {
	    testSpiro(.25, 12, 30);
	}
	if (dotest[MINI_SPIRO_EDGE] || dotest[MINI_SPIROS]) {
	    testSpiro(0, 12, 30);
	}
	if (dotest[ARCSAME] || dotest[ARCS]) {
	    testArcs(100, 100, 8);
	}
	if (dotest[ARCDIFF] || dotest[ARCS]) {
	    testArcs(105, 100, 8);
	}
    }

    public static void Add(Area a1, Area a2, boolean saveA1) {
	if (DoAdd) {
	    if (verbose) {
		System.out.println("adding...");
	    }
	    if (saveA1) {
		a1 = new Area(a1);
	    }
	    a1.add(a2);
	    if (AddCanvas != null) {
		AddCanvas.setResult(a1);
	    }
	}
    }

    public static void Sub(Area a1, Area a2, boolean saveA1) {
	if (DoSub) {
	    if (verbose) {
		System.out.println("subtracting...");
	    }
	    if (saveA1) {
		a1 = new Area(a1);
	    }
	    a1.subtract(a2);
	    if (SubCanvas != null) {
		SubCanvas.setResult(a1);
	    }
	}
    }

    public static void Xor(Area a1, Area a2, boolean saveA1) {
	if (DoXor) {
	    if (verbose) {
		System.out.println("xor'ing...");
	    }
	    if (saveA1) {
		a1 = new Area(a1);
	    }
	    a1.exclusiveOr(a2);
	    if (XorCanvas != null) {
		XorCanvas.setResult(a1);
	    }
	}
    }

    public static void Int(Area a1, Area a2, boolean saveA1) {
	if (DoInt) {
	    if (verbose) {
		System.out.println("intersecting...");
	    }
	    if (saveA1) {
		a1 = new Area(a1);
	    }
	    a1.intersect(a2);
	    if (IntCanvas != null) {
		IntCanvas.setResult(a1);
	    }
	}
    }

    public static double MICROSTEPS[] = {
	10, 1, .1, //1E-5, 1E-10, 1E-15
    };
    public static double MINISTEPS[] = {
	10, 2, 1.1, 1, .9, .5, .1
    };

    public static void testTranslate(double vals[], boolean dox, boolean doy) {
	System.out.println("Testing "+vals.length+" "+
			   (dox ? (doy ? "X/Y" : "X") : "Y")+
			   " translations in steps of "+
			   vals[0]+" to "+vals[vals.length - 1]);
	Ellipse2D e1 = new Ellipse2D.Double(100, 100, 200, 200);
	Ellipse2D e2 = new Ellipse2D.Double();
	Area a1 = new Area(e1);
	for (int i = 0; i < vals.length; i++) {
	    double xtrans = dox ? vals[i] : 0;
	    double ytrans = doy ? vals[i] : 0;
	    if (verbose) {
		System.out.println("translation("+xtrans+", "+ytrans+")");
	    }
	    e2.setFrame(100+xtrans, 100+ytrans, 200, 200);
	    Area a2 = new Area(e2);
	    Add(a1, a2, true);
	    Sub(a1, a2, true);
	    Xor(a1, a2, true);
	    Int(a1, a2, true);
	}
    }

    public static void testScale(double vals[], boolean dox, boolean doy) {
	System.out.println("Testing "+vals.length+" "+
			   (dox ? (doy ? "X/Y" : "X") : "Y")+
			   " scales in steps of "+
			   vals[0]+" to "+vals[vals.length - 1]);
	Ellipse2D e1 = new Ellipse2D.Double(100, 100, 200, 200);
	Ellipse2D e2 = new Ellipse2D.Double();
	Area a1 = new Area(e1);
	for (int i = 0; i < vals.length; i++) {
	    double xscale = dox ? vals[i] : 1;
	    double yscale = doy ? vals[i] : 1;
	    if (verbose) {
		System.out.println("scale("+xscale+", "+yscale+")");
	    }
	    e2.setFrame(100, 100, 200*xscale, 200*yscale);
	    Area a2 = new Area(e2);
	    Add(a1, a2, true);
	    Sub(a1, a2, true);
	    Xor(a1, a2, true);
	    Int(a1, a2, true);
	}
    }

    public static void testSpiro(double radius, int num, int degrees) {
	Ellipse2D e1 = new Ellipse2D.Double(200 + radius * 150, 125, 150, 150);
	System.out.println("Testing "+num+" ellipses "+
			   "rotating at radius of "+radius);
	Area AddArea = new Area();
	Area SubArea = new Area(new Rectangle2D.Double(0, 0, 399, 399));
	Area XorArea = new Area();
	Area IntArea = new Area(SubArea);
	AffineTransform at = new AffineTransform();
	int angle = 0;
	for (int i = 0; i < num; i++) {
	    if (verbose) {
		System.out.println("rotating to "+angle+" degrees");
	    }
	    at.setToRotation(Math.toRadians(angle), 200, 200);
	    Area a = new Area(at.createTransformedShape(e1));
	    Add(AddArea, a, false);
	    Sub(SubArea, a, false);
	    Xor(XorArea, a, false);
	    Int(IntArea, a, false);
	    angle += degrees;
	}
    }

    public static void testArcs(double radius1, double radius2, int num) {
	Arc2D arc1 = new Arc2D.Double(Arc2D.PIE);
	arc1.setFrameFromCenter(200, 200, 200-radius1, 200-radius1);
	Arc2D arc2 = new Arc2D.Double(Arc2D.PIE);
	arc2.setFrameFromCenter(200, 200, 200-radius2, 200-radius2);
	System.out.println("Testing arcs with "+num+
			   " rotating angles");
	AffineTransform at = new AffineTransform();
	for (int i = 0; i < num; i++) {
	    int angst1 = i * 360 / num;
	    arc1.setAngleStart(angst1);
	    for (int j = 0; j < num; j++) {
		int angext1 = (j+1) * 360 / num;
		arc1.setAngleExtent(angext1);
		Area a1 = new Area(arc1);
		for (int k = 0; k < num; k++) {
		    int angst2 = k * 360 / num;
		    arc2.setAngleStart(angst2);
		    for (int l = 0; l < num; l++) {
			int angext2 = (l+1) * 360 / num;
			arc2.setAngleExtent(angext2);
			Area a2 = new Area(arc2);
			if (verbose) {
			    System.out.println("arc1 at "+angst1+" +"+angext1);
			    System.out.println("arc2 at "+angst2+" +"+angext2);
			}
			Add(a1, a2, true);
			Sub(a1, a2, true);
			Xor(a1, a2, true);
			Int(a1, a2, true);
		    }
		}
	    }
	}
    }

    public static class AreaSpiroCanvas extends Canvas {
	private static AlphaComposite hilite =
	    AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);

	Shape result;
	long lastdisplay;
	int mouseX = 200;
	int mouseY = 200;

	public Dimension getPreferredSize() {
	    return new Dimension(400, 400);
	}

	public void setResult(Area a) {
	    result = new Area(a);
	    //printShape(result);
	    long now = System.currentTimeMillis();
	    long when = lastdisplay + DisplayDelay;
	    if (now < when) {
		try {
		    Thread.sleep(when - now);
		} catch (InterruptedException e) {
		    DisplayDelay = 0;
		}
		now = when;
	    }
	    lastdisplay = now;
	    repaint();
	}

	public static void printShape(Shape s) {
	    PathIterator pi = s.getPathIterator(null);
	    double[] coords = new double[6];
	    System.out.println("Shape ("+
			       (pi.getWindingRule() == pi.WIND_EVEN_ODD
				? "Even Odd" : "Non Zero")+")");
	    while (!pi.isDone()) {
		int type = pi.currentSegment(coords);
		switch (type) {
		case pi.SEG_MOVETO:
		    System.out.println(" moveTo("+
				       coords[0]+", "+
				       coords[1]+")");
		    break;
		case pi.SEG_LINETO:
		    System.out.println(" lineTo("+
				       coords[0]+", "+
				       coords[1]+")");
		    break;
		case pi.SEG_QUADTO:
		    System.out.println(" quadTo("+
				       coords[0]+", "+
				       coords[1]+", "+
				       coords[2]+", "+
				       coords[3]+")");
		    break;
		case pi.SEG_CUBICTO:
		    System.out.println(" cubicTo("+
				       coords[0]+", "+
				       coords[1]+", "+
				       coords[2]+", "+
				       coords[3]+", "+
				       coords[4]+", "+
				       coords[5]+")");
		    break;
		case pi.SEG_CLOSE:
		    System.out.println(" closePath()");
		    break;
		default:
		    System.out.println(" ????(type = "+type+")");
		    break;
		}
		pi.next();
	    }
	}

	public void paint(Graphics g) {
	    if (result == null) {
		g.drawString("No Result to show", 50, 100);
	    } else {
		Graphics2D g2d = (Graphics2D) g;
		if (DisplayAA) {
		    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					 RenderingHints.VALUE_ANTIALIAS_ON);
		}
		g2d.setColor(Color.blue);
		g2d.fill(result);
		g2d.setColor(Color.green);
		g2d.draw(result);
		g2d.setComposite(hilite);
		boolean ptin = result.contains(mouseX, mouseY);
		boolean shadowin = result.contains(mouseX-5, mouseY-5,
						   10, 10);
		boolean shadowtouch = result.intersects(mouseX-5, mouseY-5,
							10, 10);
		g2d.setColor(shadowtouch
			     ? (shadowin
				? (ptin ? Color.green : Color.red)
				: (ptin ? Color.cyan : Color.yellow))
			     : ((shadowin || ptin)
				? Color.red
				: Color.white));
		g2d.fillRect(mouseX - 5, mouseY - 5, 10, 10);
		g2d.setPaintMode();
		g2d.drawString("shadowin: "+shadowin+", "+
			       "shadowtouch: "+shadowtouch+", "+
			       "ptin: "+ptin,
			       50, 395);
	    }
	}

	public void show(String label, int xpos, int ypos) {
	    Frame f = new Frame(label);
	    f.add(this);
	    f.pack();
	    Dimension d = f.getSize();
	    f.setBounds(d.width * xpos, d.height * ypos, d.width, d.height);
	    f.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    e.getWindow().setVisible(false);
		    if (AreaSpiroCanvas.this == AddCanvas) {
			DoAdd = false;
		    } else if (AreaSpiroCanvas.this == SubCanvas) {
			DoSub = false;
		    } else if (AreaSpiroCanvas.this == XorCanvas) {
			DoXor = false;
		    } else if (AreaSpiroCanvas.this == IntCanvas) {
			DoInt = false;
		    }
		    if (!DoAdd && !DoSub && !DoXor && !DoInt) {
			System.exit(0);
		    }
		}
	    });
	    addMouseMotionListener(new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
		    mouseX = e.getX();
		    mouseY = e.getY();
		    repaint();
		}
		public void mouseMoved(MouseEvent e) {}
	    });
	    f.show();
	}
    }
}
