/*
 * @(#)BasicStrokeOffset.java	1.3 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
 
import java.awt.Frame;
import java.awt.Canvas;
import java.awt.Button;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Scrollbar;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.BasicStroke;
import java.awt.RenderingHints;
import java.awt.geom.GeneralPath;
import java.awt.geom.AffineTransform;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class BasicStrokeOffset extends Canvas {
    float xoff = 0.5f;
    float yoff = 0.5f;
    boolean alpha = true;
    boolean aa = true;
    boolean dblbuf = true;
    boolean scale = true;

    Label label;

    Image offscreen;
    Graphics2D offG;

    public void setXOffset(float off) {
	xoff = off;
	repaint();
    }

    public void setYOffset(float off) {
	yoff = off;
	repaint();
    }

    public void toggleAlpha() {
	alpha = !alpha;
	repaint();
    }

    public void toggleAA() {
	aa = !aa;
	repaint();
    }

    public void toggleDblBuf() {
	dblbuf = !dblbuf;
	repaint();
    }

    public void toggleScale() {
	scale = !scale;
	repaint();
    }

    public void setLabel(Label l) {
	label = l;
    }

    public Dimension getPreferredSize() {
	return new Dimension(300, 300);
    }

    public void update(Graphics g) {
	paint(g);
    }

    public void paint(Graphics g) {
	if (offscreen == null) {
	    offscreen = createImage(300, 300);
	}
	if (dblbuf) {
	    offG = (Graphics2D) offscreen.getGraphics();
	} else {
	    offG = (Graphics2D) g;
	}
	offG.clearRect(0, 0, 300, 300);
	offG.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
			      (aa ? RenderingHints.VALUE_ANTIALIAS_ON
			       : RenderingHints.VALUE_ANTIALIAS_OFF));
	GeneralPath gp = new GeneralPath();
	gp.moveTo(5, 5);
	gp.curveTo(10, 10, 20, 0, 25, 5);
	gp.lineTo(5, 25);
	gp.quadTo(15, 20, 25, 25);
	gp.closePath();
	gp.moveTo(2.5f, 12.5f);
	gp.lineTo(2.5f, 17.5f);
	gp.lineTo(7.5f, 17.5f);
	gp.lineTo(7.5f, 12.5f);
	gp.closePath();
	if (scale) {
	    offG.scale(10, 10);
	} else {
	    AffineTransform t = new AffineTransform();
	    t.scale(10, 10);
	    gp.transform(t);
	}
	float xo = xoff, yo = yoff;
	try {
	    offG.setStroke(new BasicStroke(1, xoff, yoff));
	} catch (NoSuchMethodError e) {
	    offG.setStroke(new BasicStroke(1));
	    xo = 0.5f;
	    yo = 0.5f;
	}
	offG.setColor(new Color(0, 1, 0, alpha ? 0.5f : 1.0f));
	offG.draw(gp);
	offG.setColor(new Color(0, 0, 1, 0.25f));
	offG.fill(gp);
	label.setText("offset = ("+xo+", "+yo+"), "+
		      "alpha "+(alpha ? "on" : "off")+", "+
		      "AA "+(aa ? "on" : "off")+", "+
		      "db "+(dblbuf ? "on" : "off")+", "+
		      "scale "+(scale ? "on" : "off"));

	if (dblbuf) {
	    offG.dispose();
	    g.drawImage(offscreen, 0, 0, 300, 300, null);
	}
    }

    public static void main(String argv[]) {
	Frame f = new Frame("BasicStrokeOffset");
	f.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		System.exit(0);
	    }
	});
	f.setLayout(new BorderLayout());
	final BasicStrokeOffset bso = new BasicStrokeOffset();
	f.add(bso, "Center");
	final Scrollbar sbx =
	    new Scrollbar(Scrollbar.HORIZONTAL, 50, 100, -1000, 1100);
	final Scrollbar sby =
	    new Scrollbar(Scrollbar.VERTICAL,   50, 100, -1000, 1100);
	sbx.addAdjustmentListener(new AdjustmentListener() {
	    public void adjustmentValueChanged(AdjustmentEvent e) {
		bso.setXOffset(e.getValue() / 100.0f);
	    }
	});
	sby.addAdjustmentListener(new AdjustmentListener() {
	    public void adjustmentValueChanged(AdjustmentEvent e) {
		bso.setYOffset(e.getValue() / 100.0f);
	    }
	});
	Button balpha = new Button("Toggle alpha stroke");
	balpha.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		bso.toggleAlpha();
	    }
	});
	Button bAA = new Button("Toggle antialiasing");
	bAA.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		bso.toggleAA();
	    }
	});
	Button bdblbuf = new Button("Toggle double buffering");
	bdblbuf.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		bso.toggleDblBuf();
	    }
	});
	Button bscale = new Button("Toggle scaling");
	bscale.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		bso.toggleScale();
	    }
	});
	Button bresetdef = new Button("Reset to (0.5, 0.5)");
	bresetdef.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		sbx.setValue(50);
		sby.setValue(50);
		bso.setXOffset(0.5f);
		bso.setYOffset(0.5f);
	    }
	});
	Button breset = new Button("Reset to (0.0, 0.0)");
	breset.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		sbx.setValue(0);
		sby.setValue(0);
		bso.setXOffset(0.0f);
		bso.setYOffset(0.0f);
	    }
	});
	Label l = new Label();
	bso.setLabel(l);
	Panel p = new Panel();
	p.setLayout(new GridLayout(0, 2));
	p.add(balpha);
	p.add(bAA);
	p.add(bdblbuf);
	p.add(bscale);
	p.add(breset);
	p.add(bresetdef);
	f.add(p, "North");
	Panel p2 = new Panel();
	p2.setLayout(new GridLayout(0, 1));
	p2.add(sbx);
	p2.add(l);
	f.add(sby, "East");
	f.add(p2, "South");
	f.pack();
	f.show();
    }
}
