/*
 * @(#)DrawIMGPerf.java	1.5 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
 
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Canvas;
import java.awt.Image;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.MediaTracker;
import java.awt.Transparency;
import java.awt.image.MemoryImageSource;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.DirectColorModel;
import java.awt.image.BufferedImage;

public class DrawIMGPerf {

    public static int PREFTIME = 2500;
    public static int NREPS = -1;
    public static int NTESTS = 5;

    public static final int SMALL_SIZE = 20;
    public static final int MEDIUM_SIZE = 100;
    public static final int LARGE_SIZE = 300;

    public static final int RGB_R = 0x00ff0000;
    public static final int RGB_G = 0x0000ff00;
    public static final int RGB_B = 0x000000ff;
    public static final int BGR_R = 0x000000ff;
    public static final int BGR_G = 0x0000ff00;
    public static final int BGR_B = 0x00ff0000;
    public static final int TMASK = 0x01000000;
    public static final int AMASK = 0xff000000;

    public static final String OPAQUE  = "Opaque ";
    public static final String BITMASK = "Bitmask";
    public static final String ALPHA   = "Alpha  ";

    public static final int OUT_DEFAULT      = -1;
    public static final int OUT_SCREEN       = 0;
    public static final int OUT_OFFSCREEN    = 1;
    public static final int OUT_ICM          = 2;
    public static final int OUT_GRAY         = 3;
    public static final int OUT_555          = 4;
    public static final int OUT_565          = 5;
    public static final int OUT_24BIT        = 6;
    public static final int OUT_XRGB         = 7;
    public static final int OUT_XBGR         = 8;

    public static final ColorModel ARGB = ColorModel.getRGBdefault();

    public static final String OutputNames[] = {
	"Screen",
	"Offscreen image",
	"8-bit IndexColorModel image",
	"8-bit Grayscale image",
	"16-bit 555 RGB image",
	"16-bit 565 RGB image",
	"24-bit BGR image",
	"32-bit XRGB image",
	"32-bit XBGR image",
    };

    public static final String usagemsg[] = {
	"usage: java DrawIMGPerf [options (see list below)]",
	"                         TEST CONTROLS (defaults in parentheses)",
	"        ntests=<int>       number of runs per test ("+NTESTS+")",
	"        nreps=<int>        number of fixed reps per run (calibrated)",
	"        preftime=<int>     time in ms to run reps for ("+PREFTIME+")",
	"                         TEST INFORMATION",
	"        list               just list the tests, no runs",
	"        help               print this usage statement and exit",
	"                         SOURCE IMAGE SIZE GROUP",
	"        small              run the 20x20 tests",
	"        medium             run the 100x100 tests",
	"        large              run the 300x300 tests",
	"                         SOURCE IMAGE COLORMODEL TYPE GROUP",
	"        offscr[een]        run the offscreen image tests",
	"        gray               run the gray IndexColorModel tests",
	"        icm                run the 8-bit IndexColorModel tests",
	"        rgb                run the XRGB DirectColorModel tests",
	"        bgr                run the XBGR DirectColorModel tests",
	"        argb               run the ARGB DirectColorModel tests",
	"        dcm                run the XRGB, XBGR and ARGB tests",
	"                         SOURCE IMAGE TRANSPARENCY GROUP",
	"        opaque             run the opaque ColorModel tests",
	"        bitmask|trans*     run the transparent ColorModel tests",
	"        bg*                run the transparent with BG color tests",
	"        alpha              run the translucent ColorModel tests",
	"                         DRAWIMAGE SCALING GROUP",
	"        noscale            run the unscaled tests",
	"        upscale            run the scale by 1.5x tests",
	"        downscale          run the scale by .5x tests",
	"        scale              run the upscale and downscale tests",
	"                         DESTINATIONS",
	"        outscr[een]        draw to the screen",
	"        outoffscr[een]     draw to an offscreen image",
	"        outicm             draw to 8-bit IndexColorModel image",
	"        outgray            draw to grayscale image",
	"        out15|out555       draw to a 16-bit 555 RGB image",
	"        out16|out565       draw to a 16-bit 565 RGB image",
	"        out24              draw to a 24-bit BGR image",
	"        outrgb             draw to a 32-bit XRGB image",
	"        outbgr             draw to a 32-bit XBGR image",
	"    if no choices in a given group are specified, all are chosen",
	"    Only one destination choice can be specified",
    };

    public static void Usage(int exitval) {
	for (int i = 0; i < usagemsg.length; i++) {
	    System.err.println(usagemsg[i]);
	}
	System.exit(exitval);
    }

    public static void main(String argv[]) {
	Frame f = new Frame();
	Canvas c = new Canvas();
	f.setLayout(new BorderLayout());
	f.add("Center", c);
	int testw = (int) (LARGE_SIZE * 1.5 + 305);
	int testh = (int) (LARGE_SIZE * 1.5 + 105);
	c.resize(testw, testh);

	boolean doSmall = false;
	boolean doMedium = false;
	boolean doLarge = false;
	boolean doOffscreen = false;
	boolean doGray = false;
	boolean doIcm = false;
	boolean doRgb = false;
	boolean doBgr = false;
	boolean doArgb = false;
	boolean doOpaque = false;
	boolean doTransparent = false;
	boolean doBGcolor = false;
	boolean doAlpha = false;
	boolean doUpscale = false;
	boolean doDnscale = false;
	boolean doNoscale = false;

	boolean doList = false;

	int output = OUT_DEFAULT;

	for (int i = 0; i < argv.length; i++) {
	    String arg = argv[i].toLowerCase();
	    if (arg.startsWith("ntests=")) {
		NTESTS = Integer.parseInt(arg.substring(7));
	    } else if (arg.startsWith("nreps=")) {
		NREPS = Integer.parseInt(arg.substring(6));
	    } else if (arg.startsWith("preftime=")) {
		PREFTIME = Integer.parseInt(arg.substring(8));
	    } else if (arg.equals("small")) {
		doSmall = true;
	    } else if (arg.equals("medium")) {
		doMedium = true;
	    } else if (arg.equals("large")) {
		doLarge = true;
	    } else if (arg.startsWith("offscr")) {
		doOffscreen = true;
	    } else if (arg.equals("gray")) {
		doGray = true;
	    } else if (arg.equals("icm")) {
		doIcm = true;
	    } else if (arg.equals("rgb")) {
		doRgb = true;
	    } else if (arg.equals("bgr")) {
		doBgr = true;
	    } else if (arg.equals("argb")) {
		doArgb = true;
	    } else if (arg.equals("dcm")) {
		doRgb = doBgr = doArgb = true;
	    } else if (arg.equals("opaque")) {
		doOpaque = true;
	    } else if (arg.equals("alpha")) {
		doAlpha = true;
	    } else if (arg.equals("bg")) {
		doBGcolor = true;
	    } else if (arg.equals("bitmask") || arg.startsWith("trans")) {
		doTransparent = true;
	    } else if (arg.startsWith("out")) {
		int oldout = output;
		String rest = arg.substring(3);
		if (rest.startsWith("scr")) {
		    output = OUT_SCREEN;
		} else if (rest.startsWith("offscr")) {
		    output = OUT_OFFSCREEN;
		} else if (rest.equals("icm")) {
		    output = OUT_ICM;
		} else if (rest.equals("gray")) {
		    output = OUT_GRAY;
		} else if (rest.equals("15") || rest.equals("555")) {
		    output = OUT_555;
		} else if (rest.equals("16") || rest.equals("565")) {
		    output = OUT_565;
		} else if (rest.equals("24")) {
		    output = OUT_24BIT;
		} else if (rest.equals("rgb")) {
		    output = OUT_XRGB;
		} else if (rest.equals("bgr")) {
		    output = OUT_XBGR;
		} else {
		    System.err.println("Unrecognized option: "+arg);
		    Usage(1);
		}
		if (oldout != OUT_DEFAULT) {
		    System.err.println("Only one destination can be "+
				       "specified at a time - ignoring: "+
				       arg);
		    output = oldout;
		}
	    } else if (arg.endsWith("scale")) {
		if (arg.startsWith("no")) {
		    doNoscale = true;
		} else if (arg.startsWith("up")) {
		    doUpscale = true;
		} else if (arg.startsWith("down")) {
		    doDnscale = true;
		} else if (arg.startsWith("scale")) {
		    doUpscale = true;
		    doDnscale = true;
		} else {
		    System.err.println("Unrecognized option: "+arg);
		    Usage(1);
		}
	    } else if (arg.equals("list")) {
		doList = true;
	    } else if (arg.equals("help") || arg.equals("-help")) {
		Usage(0);
	    } else {
		System.err.println("Unrecognized option: "+arg);
		Usage(1);
	    }
	}

	if (!doSmall && !doMedium && !doLarge) {
	    doSmall = doMedium = doLarge = true;
	}
	if (!doOffscreen && !doGray && !doIcm && !doRgb && !doBgr && !doArgb) {
	    doOffscreen = doGray = doIcm = doRgb = doBgr = doArgb = true;
	}
	if (!doOpaque && !doTransparent && !doBGcolor && !doAlpha) {
	    doOpaque = doTransparent = doBGcolor = doAlpha = true;
	}
	if (!doUpscale && !doDnscale && !doNoscale) {
	    doUpscale = doDnscale = doNoscale = true;
	}
	if (output == OUT_DEFAULT) {
	    output = OUT_SCREEN;
	}

	boolean doInt = (doRgb || doBgr || doArgb);

	// Construct the small pixel array.
	byte[] bsmpixels = null;
	int[] ismpixels = null;
	int[] tsmpixels = null;
	int[] asmpixels = null;
	if (doSmall) {
	    bsmpixels = bytearray(SMALL_SIZE, SMALL_SIZE);
	    if (doInt) {
		if (doOpaque) {
		    ismpixels = intarray(SMALL_SIZE, SMALL_SIZE,
					 Transparency.OPAQUE);
		}
		if (doTransparent) {
		    tsmpixels = intarray(SMALL_SIZE, SMALL_SIZE,
					 Transparency.BITMASK);
		}
		if (doAlpha) {
		    asmpixels = intarray(SMALL_SIZE, SMALL_SIZE,
					 Transparency.TRANSLUCENT);
		}
	    }
	}

	// Construct the medium pixel array.
	byte[] bmdpixels = null;
	int[] imdpixels = null;
	int[] tmdpixels = null;
	int[] amdpixels = null;
	if (doMedium) {
	    bmdpixels = bytearray(MEDIUM_SIZE, MEDIUM_SIZE);
	    if (doInt) {
		if (doOpaque) {
		    imdpixels = intarray(MEDIUM_SIZE, MEDIUM_SIZE,
					 Transparency.OPAQUE);
		}
		if (doTransparent) {
		    tmdpixels = intarray(MEDIUM_SIZE, MEDIUM_SIZE,
					 Transparency.BITMASK);
		}
		if (doAlpha) {
		    amdpixels = intarray(MEDIUM_SIZE, MEDIUM_SIZE,
					 Transparency.TRANSLUCENT);
		}
	    }
	}

	// Construct the large pixel array.
	byte[] blgpixels = null;
	int[] ilgpixels = null;
	int[] tlgpixels = null;
	int[] algpixels = null;
	if (doLarge) {
	    blgpixels = bytearray(LARGE_SIZE, LARGE_SIZE);
	    if (doInt) {
		if (doOpaque) {
		    ilgpixels = intarray(LARGE_SIZE, LARGE_SIZE,
					 Transparency.OPAQUE);
		}
		if (doTransparent) {
		    tlgpixels = intarray(LARGE_SIZE, LARGE_SIZE,
					 Transparency.BITMASK);
		}
		if (doAlpha) {
		    algpixels = intarray(LARGE_SIZE, LARGE_SIZE,
					 Transparency.TRANSLUCENT);
		}
	    }
	}

	// Construct the three color models.
	byte[] rmap = new byte[256];
	byte[] gmap = new byte[256];
	byte[] bmap = new byte[256];
	byte[] map = new byte[256];
	for (int i = 0; i < 256; i++) {
	    map[i] = (byte) i;
	    if (i < 216) {
		rmap[i] = (byte) (((i     ) % 6) * 51);
		gmap[i] = (byte) (((i /  6) % 6) * 51);
		bmap[i] = (byte) (((i / 36) % 6) * 51);
	    } else {
		int v = (i - 216);
		byte v2 = (byte) (v % 10);
		v2 = (byte) ((v2 + v2/2 + 1) * 17);
		switch (v / 10) {
		case 0: rmap[i] = v2; break;
		case 1: gmap[i] = v2; break;
		case 2: bmap[i] = v2; break;
		default: rmap[i] = gmap[i] = bmap[i] = v2; break;
		}
	    }
	}

	ColorModel ogray = new IndexColorModel(8, 256, map, map, map);
	ColorModel tgray = new IndexColorModel(8, 256, map, map, map, 0);
	ColorModel agray = new IndexColorModel(8, 256, map, map, map, map);
	ColorModel oicm = new IndexColorModel(8, 256, rmap, gmap, bmap);
	ColorModel ticm = new IndexColorModel(8, 256, rmap, gmap, bmap, 0);
	ColorModel aicm = new IndexColorModel(8, 256, rmap, gmap, bmap, map);

	ColorModel orgb = new DirectColorModel(24, RGB_R, RGB_G, RGB_B);
	ColorModel trgb = new DirectColorModel(25, RGB_R, RGB_G, RGB_B, TMASK);
	ColorModel argb = new DirectColorModel(32, RGB_R, RGB_G, RGB_B, AMASK);

	ColorModel obgr = new DirectColorModel(24, BGR_R, BGR_G, BGR_B);
	ColorModel tbgr = new DirectColorModel(25, BGR_R, BGR_G, BGR_B, TMASK);
	ColorModel abgr = new DirectColorModel(32, BGR_R, BGR_G, BGR_B, AMASK);

	int sizes[] = {SMALL_SIZE, MEDIUM_SIZE, LARGE_SIZE};
	byte[] bytearrays[] = {bsmpixels, bmdpixels, blgpixels};
	int[] opqarrays[] = {ismpixels, imdpixels, ilgpixels};
	int[] transarrays[] = {tsmpixels, tmdpixels, tlgpixels};
	int[] alphaarrays[] = {asmpixels, amdpixels, algpixels};
	Object imgtypes[][] = {
	    { oicm, bytearrays,  null,         Boolean.TRUE  },
	    null,
	    null,
	    null,
	    { ogray, bytearrays, null,         Boolean.FALSE },
	    { tgray, bytearrays, null,         Boolean.FALSE },
	    { tgray, bytearrays, Color.yellow, Boolean.FALSE },
	    { agray, bytearrays, null,         Boolean.FALSE },
	    { oicm, bytearrays,  null,         Boolean.FALSE },
	    { ticm, bytearrays,  null,         Boolean.FALSE },
	    { ticm, bytearrays,  Color.yellow, Boolean.FALSE },
	    { aicm, bytearrays,  null,         Boolean.FALSE },
	    { orgb, opqarrays,   null,         Boolean.FALSE },
	    { trgb, transarrays, null,         Boolean.FALSE },
	    { trgb, transarrays, Color.yellow, Boolean.FALSE },
	    { argb, alphaarrays, null,         Boolean.FALSE },
	    { obgr, opqarrays,   null,         Boolean.FALSE },
	    { tbgr, transarrays, null,         Boolean.FALSE },
	    { tbgr, transarrays, Color.yellow, Boolean.FALSE },
	    { abgr, alphaarrays, null,         Boolean.FALSE },
	    { ARGB, opqarrays,   null,         Boolean.FALSE },
	    { ARGB, transarrays, null,         Boolean.FALSE },
	    { ARGB, transarrays, Color.yellow, Boolean.FALSE },
	    { ARGB, alphaarrays, null,         Boolean.FALSE },
	};
	double scalefactors[] = {1.0, 0.5, 1.5};

	if (!doSmall) sizes[0] = 0;
	if (!doMedium) sizes[1] = 0;
	if (!doLarge) sizes[2] = 0;
	if (!doOffscreen) clear(imgtypes, 0, 1, 4);
	if (!doGray) clear(imgtypes, 4, 1, 8);
	if (!doIcm) clear(imgtypes, 8, 1, 12);
	if (!doRgb) clear(imgtypes, 12, 1, 16);
	if (!doBgr) clear(imgtypes, 16, 1, 20);
	if (!doArgb) clear(imgtypes, 20, 1, 24);
	if (!doOpaque) clear(imgtypes, 0, 4, imgtypes.length);
	if (!doTransparent) clear(imgtypes, 1, 4, imgtypes.length);
	if (!doBGcolor) clear(imgtypes, 2, 4, imgtypes.length);
	if (!doAlpha) clear(imgtypes, 3, 4, imgtypes.length);
	if (!doNoscale) scalefactors[0] = 0.0;
	if (!doDnscale) scalefactors[1] = 0.0;
	if (!doUpscale) scalefactors[2] = 0.0;

	// Construct our test cases.
	DrawTestCase tests[] =
	    new DrawTestCase[scalefactors.length *
			     sizes.length *
			     imgtypes.length + imgtypes.length/3];
	int index = 0;

	for (int i = 0; i < scalefactors.length; i++) {
	    double scale = scalefactors[i];
	    if (scale == 0.0) continue;
	    for (int j = 0; j < sizes.length; j++) {
		int size = sizes[j];
		if (size == 0) continue;
		for (int k = 0; k < imgtypes.length; k++) {
		    Object o[] = imgtypes[k];
		    if (o == null) continue;
		    Object pix = ((Object[]) o[1])[j];
		    if (pix instanceof byte[]) {
			boolean offscr = (o[3] == Boolean.TRUE);
			IndexColorModel icm = (IndexColorModel) o[0];
			tests[index] = new DrawTestCase(c, size, size,
							icm, (byte[]) pix,
							scale, offscr);
		    } else {
			DirectColorModel dcm = (DirectColorModel) o[0];
			tests[index] = new DrawTestCase(c, size, size,
							dcm, (int[]) pix,
							scale);
		    }
		    if (o[2] != null) {
			tests[index] = tests[index].deriveBGversion();
		    }
		    index++;
		}
	    }
	}

	int numtests = index;


	if (doList) {
	    System.out.println(numtests+" tests");
	    for (int i = 0; i < numtests; i++) {
		System.out.println(tests[i]);
	    }
	    System.exit(0);
	}

	// Set ourselves to a high priority.
	Thread.currentThread().setPriority(Thread.MIN_PRIORITY+1);

	f.pack();
	f.show();
	try {Thread.sleep(1000);} catch (InterruptedException e) {}
	Graphics screeng = c.getGraphics();
	screeng.setColor(Color.blue);
	Graphics testg;
	Image img = null;
	ColorModel cm;

	if (output == OUT_SCREEN || output == OUT_OFFSCREEN) {
	    cm = c.getColorModel();
	    if (output == OUT_OFFSCREEN) {
		img = c.createImage(testw, testh);
	    }
	} else {
	    int types[] = {
		BufferedImage.TYPE_BYTE_INDEXED,
		BufferedImage.TYPE_BYTE_GRAY,
		BufferedImage.TYPE_USHORT_555_RGB,
		BufferedImage.TYPE_USHORT_565_RGB,
		BufferedImage.TYPE_3BYTE_BGR,
		BufferedImage.TYPE_INT_RGB,
		BufferedImage.TYPE_INT_BGR,
	    };
	    int type = types[output-OUT_ICM];
	    BufferedImage bimg = new BufferedImage(testw, testh, type);
	    cm = bimg.getColorModel();
	    img = bimg;
	}
	if (img != null) {
	    testg = img.getGraphics();
	} else {
	    testg = screeng;
	}

	// Figure out how deep the screen is.
	System.out.println("Output is going to a "+
			   cm.getPixelSize()+
			   " bit deep destination\n");

	// Run the tests.
	try {Thread.sleep(1000);} catch (InterruptedException e) {}
	for (index = 0; index < numtests; index++) {
	    tests[index].doTest(testg, screeng, img);
	}

	testg.dispose();
	screeng.dispose();

	// Print out a final summary.
	System.out.println("\nAverage pixel rendering speeds on a "+
			   cm.getPixelSize()+
			   " bit destination:");
	System.out.println("Destination: "+OutputNames[output]);
	System.out.println("Java: [version "+
			   System.getProperty("java.version")+"]");
	System.out.println("VM: "+
			   "["+System.getProperty("java.vm.vendor")+"] "+
			   "["+System.getProperty("java.vm.name")+"] "+
			   "[version "+System.getProperty("java.vm.version")+"]");
	System.out.println("OS: "+
			   "["+System.getProperty("os.arch")+"] "+
			   "["+System.getProperty("os.name")+"] "+
			   "[version "+System.getProperty("os.version")+"]");
	System.out.println();
	for (index = 0; index < numtests; index++) {
	    tests[index].summarize(null, true);
	}
	System.exit(0);
    }

    public static byte[] bytearray(int w, int h) {
	int index = 0;
	byte val = 1;
	byte[] pix = new byte[w * h];
	for (int j = 0; j < h; j++) {
	    boolean middlerow = ((j >= (h / 4)) && (j <= ((h * 3) / 4)));
	    for (int i = 0; i < w; i++) {
		if (middlerow && ((i >= (w / 4)) && (i <= ((w * 3) / 4)))) {
		    pix[index++] = 0;
		} else {
		    pix[index++] = val++;
		    if (val == 0) {
			val++;
		    }
		}
	    }
	}
	return pix;
    }

    public static int[] intarray(int w, int h, int trans) {
	int index = 0;
	int[] intarray = new int[w * h];
	for (int j = 0; j < h; j++) {
	    boolean middlerow = ((j >= (h / 4)) && (j <= ((h * 3) / 4)));
	    for (int i = 0; i < w; i++) {
		boolean middlecol = ((i >= (w / 4)) && (i <= ((w * 3) / 4)));
		if (middlerow && middlecol) {
		    if (trans == Transparency.OPAQUE) {
			intarray[index++] = 0xff << 24;
		    } else {
			intarray[index++] = 0;
		    }
		} else {
		    int bpix = 0xff - (index & 0xff);
		    int ipix = bpix << ((2 - ((index / 256) & 1)) * 8);
		    if (trans == Transparency.TRANSLUCENT) {
			ipix |= bpix << 24;
		    } else {
			ipix |= 0xff << 24;
		    }
		    intarray[index++] = ipix;
		}
	    }
	}
	return intarray;
    }

    public static void clear(Object[] array, int start, int inc, int end) {
	while (start < end) {
	    array[start] = null;
	    start += inc;
	}
    }
}

class DrawTestCase {
    long totalms;
    int totalreps;

    int srcW, srcH;
    int dstW, dstH;
    Image img;
    Component comp;
    String desc;

    boolean offscreen;

    private static class BG extends DrawTestCase {
	public BG(DrawTestCase parent) {
	    super(parent);
	}

	public void doReps(Graphics g, Image img, int nreps) {
	    int x = 0;
	    int y = 0;
	    int dx = 3;
	    int dy = 1;
	    int xmax = comp.size().width - dstW;
	    int ymax = comp.size().height - dstH;
	    Color bg = Color.yellow;
	    for (int i = 0; i < nreps; i++) {
		g.drawImage(img, x, y, dstW, dstH, bg, null);
		x += dx;
		if (x > xmax || x < 0) {
		    dx = -dx;
		    x += dx + dx;
		}
		y += dy;
		if (y > ymax || y < 0) {
		    dy = -dy;
		    y += dy + dy;
		}
	    }
	}
    }

    DrawTestCase(DrawTestCase other) {
	this.srcW = other.srcW;
	this.srcH = other.srcH;
	this.dstW = other.dstW;
	this.dstH = other.dstH;
	this.img = other.img;
	this.comp = other.comp;
	this.desc = other.desc + " (BG)";
    }

    public DrawTestCase(Component c, int width, int height,
			IndexColorModel icm, byte[] pixels,
			double scale, boolean offscreen)
    {
	this.srcW = width;
	this.srcH = height;
	this.dstW = (int) (width * scale);
	this.dstH = (int) (height * scale);
	this.comp = c;
	this.img = c.createImage(new MemoryImageSource(width, height, icm,
						       pixels, 0, width));
	this.offscreen = offscreen;
	if (offscreen) {
	    makeDescription("<Off-Screen>");
	} else {
	    makeDescription(icm);
	}
    }

    public DrawTestCase(Component c, int width, int height,
			DirectColorModel dcm, int[] pixels,
			double scale)
    {
	this.srcW = width;
	this.srcH = height;
	this.dstW = (int) (width * scale);
	this.dstH = (int) (height * scale);
	this.comp = c;
	this.img = c.createImage(new MemoryImageSource(width, height, dcm,
						       pixels, 0, width));
	makeDescription(pixels, dcm);
    }

    public DrawTestCase deriveBGversion() {
	return new BG(this);
    }

    private void makeDescription(IndexColorModel icm) {
	String transdesc = DrawIMGPerf.OPAQUE;
	String basetype = "GRAY ";
	for (int i = 0; i < 256; i++) {
	    int rgb = icm.getRGB(i);
	    int alpha = rgb >>> 24;
	    if (alpha != 0xff) {
		if (alpha != 0) {
		    transdesc = DrawIMGPerf.ALPHA;
		} else if (transdesc == DrawIMGPerf.OPAQUE) {
		    transdesc = DrawIMGPerf.BITMASK;
		}
	    }
	    int red = (rgb >> 16) & 0xff;
	    int grn = (rgb >> 8) & 0xff;
	    int blu = (rgb & 0xff);
	    if (red != grn || grn != blu) {
		basetype = "ICM8 ";
	    }
	}
	makeDescription(basetype+transdesc);
    }

    private void makeDescription(int[] pixels, DirectColorModel dcm) {
	if (dcm == DrawIMGPerf.ARGB) {
	    String transdesc = DrawIMGPerf.OPAQUE;
	    for (int i = 0; i < pixels.length; i++) {
		int alpha = pixels[i] >>> 24;
		if (alpha < 255) {
		    if (alpha == 0) {
			transdesc = DrawIMGPerf.BITMASK;
		    } else {
			transdesc = DrawIMGPerf.ALPHA;
			break;
		    }
		}
	    }
	    makeDescription("ARGB "+transdesc);
	} else {
	    makeDescription(dcm);
	}
    }

    private void makeDescription(DirectColorModel dcm) {
	int amask = dcm.getAlphaMask();
	String transdesc;
	if (amask == 0) {
	    transdesc = DrawIMGPerf.OPAQUE;
	} else if ((amask & (amask-1)) == 0) {
	    transdesc = DrawIMGPerf.BITMASK;
	} else {
	    transdesc = DrawIMGPerf.ALPHA;
	}
	makeDescription((dcm.getRedMask() > dcm.getBlueMask()
			 ? "xRgb" : "xBgr")+" "+transdesc);
    }

    private void makeDescription(String transdesc) {
	desc = srcW + "x" + srcH;
	if (srcW < 100) {
	    desc = " "+desc+" ";
	}
	desc = desc + " " + transdesc + " image";
	if ((srcW != dstW) || (srcH != dstH)) {
	    String scaledesc = dstW + "x" + dstH;
	    if (dstW < 100) {
		scaledesc = " "+scaledesc+" ";
	    }
	    desc += " scaled to " + scaledesc;
	}
    }

    public String toString() {
	return "DrawTestCase["+desc+"]";
    }
    
    public void summarize(String prefix, boolean dodesc) {
	String pps = (1000l * totalreps * dstW * dstH / totalms)+"";
	if (prefix == null) {
	    prefix = "          ".substring(Math.min(10, pps.length()));
	}
	System.out.println(prefix+pps+
			   (dodesc ? " "+desc : "")+
			   " pixels per second");
    }

    public void doReps(Graphics g, Image img, int nreps) {
	int x = 0;
	int y = 0;
	int dx = 3;
	int dy = 1;
	int xmax = comp.size().width - dstW;
	int ymax = comp.size().height - dstH;
	for (int i = 0; i < nreps; i++) {
	    g.drawImage(img, x, y, dstW, dstH, null);
	    x += dx;
	    if (x > xmax || x < 0) {
		dx = -dx;
		x += dx + dx;
	    }
	    y += dy;
	    if (y > ymax || y < 0) {
		dy = -dy;
		y += dy + dy;
	    }
	}
    }

    public static boolean idle() {
	try {
	    System.gc();
	    System.gc();
	    Thread.sleep(200);
	} catch (InterruptedException e) {
	    return true;
	}
	return false;
    }

    public void doTest(Graphics testg, Graphics screeng, Image destimg) {
	Image img = this.img;
	long start, end;
	MediaTracker tracker = new MediaTracker(comp);
	if (offscreen) {
	    tracker.addImage(img, 0, srcW, srcH);
	} else {
	    tracker.addImage(img, 0, dstW, dstH);
	}
	try {
	    tracker.waitForAll();
	} catch (InterruptedException e) {
	    System.err.println("Test of a "+desc+" interrupted!");
	    return;
	}
	if (offscreen) {
	    Image offimg = comp.createImage(srcW, srcH);
	    Graphics g2 = offimg.getGraphics();
	    // Just in case - we should have been handed an opaque image...
	    g2.setColor(Color.green);
	    g2.fillRect(0, 0, srcW, srcH);
	    g2.drawImage(img, 0, 0, null);
	    img.flush();
	    img = offimg;
	    offscreen = false;
	}
	testg.setColor(Color.blue);
	testg.fillRect(0, 0, comp.size().width, comp.size().height);
	// Do one rep to prime the pipeline
	doReps(testg, img, 1);
	if (!testg.drawImage(img, 0, 0, dstW, dstH, null)) {
	    System.out.println("image not prepared!!!!!");
	}
	idle();
	int nreps = DrawIMGPerf.NREPS;
	if (nreps < 0) {
	    nreps = 1;
	    int totalreps = 0;
	    comp.getToolkit().sync();
	    long now = start = System.currentTimeMillis();
	    end = start + DrawIMGPerf.PREFTIME;
	    long midtime = (start + end) / 2;
	    while (now < end) {
		doReps(testg, img, nreps);
		comp.getToolkit().sync();
		now = System.currentTimeMillis();
		totalreps += nreps;
		if (now < midtime) {
		    nreps = totalreps;
		} else {
		    nreps = (int) (totalreps * (end - now) / (now - start));
		    if (nreps < 1) {
			nreps = 1;
		    }
		}
	    }
	    nreps = (int) (totalreps * DrawIMGPerf.PREFTIME / (now - start));
	    if (nreps < 1) {
		nreps = 1;
	    }
	}
	if (destimg != null) {
	    screeng.fillRect(0, 0, comp.size().width, comp.size().height);
	    screeng.drawImage(destimg, 0, 0, null);
	}
	for (int j = 0; j < DrawIMGPerf.NTESTS; j++) {
	    if (idle()) {
		break;
	    }
	    comp.getToolkit().sync();
	    start = System.currentTimeMillis();
	    doReps(testg, img, nreps);
	    comp.getToolkit().sync();
	    end = System.currentTimeMillis();
	    System.out.println(nreps+" reps of a "+desc+" rendered in "
			       +(end - start)+"ms");
	    totalms += (end - start);
	    if (destimg != null) {
		screeng.fillRect(0, 0, comp.size().width, comp.size().height);
		screeng.drawImage(destimg, 0, 0, null);
	    }
	}
	img.flush();
	if (totalms == 0) {
	    totalms = 1;
	}
	totalreps = DrawIMGPerf.NTESTS * nreps;
	System.out.println(totalreps+" total reps of a "+desc+" rendered in "
			   +totalms+"ms");
	summarize("average pixel rendering speed == ", false);
	System.out.println();
    }
}
