/*
 * @(#)Pipe.java	1.17 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

import java.applet.Applet;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;


public abstract class Pipe {
    protected Pipeline pipeline;
    protected BufferedImage inImage;
    protected BufferedImage outImage;
    protected WritableRaster oraster;
    protected WritableRaster iraster;
    protected boolean usedRef = false;
    protected Applet applet;
    protected boolean debug = false;

    protected Pipe (Pipeline pipeline, Applet applet, BufferedImage inImage,
                    boolean useRef) {
        this.pipeline = pipeline;
        this.applet = applet;
        ColorModel cm = inImage.getColorModel();
        this.inImage = inImage;
        this.usedRef  = useRef;
        iraster = inImage.getRaster();
        if (usedRef) {
            outImage = new BufferedImage (cm, iraster,
                                          cm.isAlphaPremultiplied(), null);
            oraster = iraster;
        }
        else {
            outImage = new BufferedImage (cm,
                         inImage.getRaster().createCompatibleWritableRaster(),
                         cm.isAlphaPremultiplied(), null);
            oraster = outImage.getRaster();
        }
    }

    public void setImage(BufferedImage bimage) {
        inImage = bimage;
        iraster = inImage.getRaster();
        process();
    }

    public BufferedImage getImage() {
        return outImage;
    }

    protected void createDestRaster() {
        ColorModel cm = outImage.getColorModel();
        oraster = oraster.createCompatibleWritableRaster();
        outImage = new BufferedImage (cm, oraster, cm.isAlphaPremultiplied(),
                                      null);
        usedRef = false;
    }

    protected void addComponent(Container p, GridBagLayout gbl,
                                Component c, int x, int y, int w, int h,
                                double weightX, double weightY) {
        GridBagConstraints gbc = new GridBagConstraints();

        gbc.fill  = GridBagConstraints.BOTH;
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridwidth = h;
        gbc.weightx = weightX;
        gbc.weighty = weightY;
        gbl.setConstraints(c, gbc);
        p.add(c);
    }

    public abstract Container addGui();
    public abstract void process();

}

