/*
 * @(#)ControlsInterface.java	1.6 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.io.*;
import java.util.Vector;
import java.util.StringTokenizer;


/**
 * Selection of different BufferedImage types and antialiasing.
 *
 * @version @(#)ControlsInterface.java	1.6 00/02/02
 * @author Brian Lichtenwalter
 */

public class ControlsInterface extends JPanel implements ActionListener, ListSelectionListener {

    static Font font = new Font("serif", Font.PLAIN, 10);
    private TableModel dataModel;
    private JTable table;

    static Vector data = new Vector();   
    static String[] screenNames = { 
            "On_Screen",
            "INT_RGB", "INT_ARGB", "INT_ARGB_PRE", "INT_BGR",
            "3BYTE_BGR", "4BYTE_ABGR", "4BYTE_ABGR_PRE", "USHORT_565_RGB",
            "USHORT_555_RGB", "BYTE_GRAY", "USHORT_GRAY", "BYTE_BINARY",
            "BYTE_INDEXED"};
    static int index = 0;


    public ControlsInterface() {

        setLayout(new BorderLayout());

        JPanel panel = new JPanel(new BorderLayout());
        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        SoftBevelBorder sbb = new SoftBevelBorder(BevelBorder.RAISED);
        panel.setBorder(new CompoundBorder(eb, sbb));


        File f = new File("CONTROLS.txt");
        if (f == null || !f.exists()) { 
            addData(new Boolean(false), screenNames[0], new Boolean(true));
            for (int i = 1; i < screenNames.length; i++) {
                addData(new Boolean(true), screenNames[i], new Boolean(true));
            }
        } else {
           try {
                BufferedReader reader = new BufferedReader(new FileReader(f));
                String str = null;
                while ((str = reader.readLine()) != null) {
                    StringTokenizer st = new StringTokenizer(str);
                    String imgTypeName = st.nextToken();
                    Boolean antialiasing = new Boolean(st.nextToken());
                    addData(new Boolean(true), imgTypeName, antialiasing);
                }
           } catch (Exception e) { e.printStackTrace(); }
        }

        final String[] names = { " ", "Screen Type", "AntiAlias" };

        dataModel = new AbstractTableModel() {
            public int getColumnCount() { return names.length; }
            public int getRowCount() { return data.size();}
            public Object getValueAt(int row, int col) { 
                    return ((Object[]) data.get(row))[col]; 
            }
            public String getColumnName(int col) {return names[col]; }
            public Class getColumnClass(int c) {
                return getValueAt(0, c).getClass();
            }
            public boolean isCellEditable(int row, int col) {
                return true;
            }
            public void setValueAt(Object aValue, int row, int col) {
                ((Object[]) data.get(row))[col] = aValue; 
            }
        };


        table = new JTable(dataModel);
        table.setFont(font);
        DefaultListSelectionModel lsm = new DefaultListSelectionModel();
        lsm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lsm.addListSelectionListener(this);
        table.setSelectionModel(lsm);
        TableColumn col = table.getColumn(" ");
        col.setMaxWidth(20);
        col.setMinWidth(20);
        col.setWidth(20);
        col = table.getColumn("AntiAlias");
        col.setMaxWidth(60);
        col.setMinWidth(60);
        col.setWidth(60);

        JComboBox combo = new JComboBox();
        for (int i = 0; i < screenNames.length; i++) {
            combo.addItem(screenNames[i]);
        } 
        combo.addActionListener(this);

        col = table.getColumn("Screen Type");
        col.setCellEditor(new DefaultCellEditor(combo));

        DefaultTableCellRenderer colRenderer = new DefaultTableCellRenderer();
        colRenderer.setBackground(Color.pink);
        colRenderer.setToolTipText("Click for combo box");
        col.setCellRenderer(colRenderer);
        table.sizeColumnsToFit(0);

        JScrollPane scrollpane = new JScrollPane(table);
        panel.add("Center",scrollpane);

        JPanel nPanel = new JPanel(new GridLayout(0,3,2,2));
        nPanel.setBorder(new EtchedBorder());
        nPanel.add(makeButton("New", getBackground()));
        nPanel.add(makeButton("Delete", getBackground()));
        nPanel.add(makeButton("Save", getBackground()));
        panel.add("North", nPanel);

        JPanel sPanel = new JPanel(new BorderLayout());
        sPanel.setBorder(new EtchedBorder());
        sPanel.add("West", makeButton("Unselect All", getBackground()));
        panel.add("South", sPanel);

        add(panel);
    }


    public JButton makeButton(String bName, Color c) {
        JButton b = new JButton(bName);
        b.setBackground(c);
        b.setFont(font);
        b.addActionListener(this);
        return b;
    }


    public void actionPerformed(ActionEvent e) {
        String name = e.getActionCommand();
        if (name == "Save") {
            if (data.size() == 0) {
                return;
            }
            PrintWriter out = null;
            try {
                out = new PrintWriter(new BufferedWriter(new FileWriter("CONTROLS.txt")));
                for (int i = 0; i < data.size(); i++) {
                    if (((Boolean) ((Object[]) data.get(i))[0]).booleanValue()) {
                        out.print((String) ((Object[]) data.get(i))[1] + " ");
                        out.println(((Boolean) ((Object[]) data.get(i))[2]).booleanValue() + " ");
                    }
                }
                out.close();
            } catch (IOException ioe) { ioe.printStackTrace(); }
            return;
        }
        if (name == "New") {
            addData(new Boolean(true), screenNames[1], new Boolean(true));
            repaintTestGroup();
        } else if (name == "Delete") {
            if (data.size() == 0) {
                return;
            }
            data.removeElementAt(index);
            index = index >= data.size() ? data.size()-1 : index;
            index = index-1 < 0 ? 0 : index;
	    if (data.size() != 0) {
                repaintTestGroup();
	    }
        } else if (name == "Select All" || name == "Unselect All") {
            JButton b = (JButton) e.getSource();
            b.setSelected(!b.isSelected());
            b.setText(b.isSelected() ? "Select All" : "Unselect All");
            for (int i = 0; i < data.size(); i++) {
                Object[] obj = ((Object[]) data.get(i));
                obj[0] = new Boolean(!b.isSelected());
                data.set(i, (Object) obj);
            }
        }
        table.clearSelection();
        table.addRowSelectionInterval(index, index);
        table.tableChanged(new TableModelEvent(dataModel));
    }

int prevIndex;


    public void valueChanged(ListSelectionEvent e) {
        if (data.size() == 0) {
            return;
        }
        String s = e.toString();
        s = s.substring(s.indexOf('{')+1,s.lastIndexOf('}'));
        prevIndex = index;
        try {
            index = Integer.parseInt(s);
        } catch (Exception ex) { return; }
        if (prevIndex != index) {
            repaintTestGroup();
        }
    }


    private void repaintTestGroup() {
	if (TestProduct.panel != null && TestProduct.group != null) {
	    TestProduct.group.stop();
	    TestProduct.group.start();
        }
    }


    static String getFullName(int i) {
        String name = (String) ((Object[]) data.get(i))[1]; 
        String aa = ((Boolean) ((Object[]) data.get(i))[2]).booleanValue()
                    ? "-aa-" : "-"; 
        return name + aa;
    }


    static boolean getAntiAlias() {
        return ((Boolean) ((Object[]) data.get(index))[2]).booleanValue(); 
    }

 
    static int getScreenIndex() {
        if (data.size() != 0) {
            String s = (String) ((Object[]) data.get(index))[1];
            for (int i = 0; i < ControlsInterface.screenNames.length; i++) {
                if (s.compareTo(ControlsInterface.screenNames[i]) == 0) {
                    return i;
                }
            }
        } 
        return 0;
    }


    static void addData(Boolean selected, 
                        String name, 
                        Boolean antialiasing) 
    {
        Object[] o = { selected, name, antialiasing};
        data.addElement(o);
    }


    public static void main(String s[]) {
        WindowListener l = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        };
        JFrame f = new JFrame("ControlsInterface");
        f.addWindowListener(l);
        f.getContentPane().add("Center", new ControlsInterface());
        f.pack();
        f.setSize(new Dimension(400,300));
        f.show();
    }
}
