/*
 * @(#)ImageComparator.java	1.9 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.io.*;
import java.awt.image.*;
import com.sun.image.codec.jpeg.*;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.RepaintManager;


/**
 * Master and Difference panels.  Compare a master or reference BufferedImage 
 * that's generated from a DataBuffer in a JarFile to a run time BufferedImage.
 *
 * @version @(#)ImageComparator.java	1.9 00/02/02
 * @author Brian Lichtenwalter
 */

public class ImageComparator {

    static Thread mThread, dThread;

    /**
     * Construct the Buffered from the DataBuffer in the jarfile.
     */
    public static class Master extends JPanel implements Runnable {

        public static BufferedImage bimg;
        private Vector vector = new Vector();
        private BufferedReader reader;      
        private int dbsize = 40000;


        public Master(String entryName, String jarfileName) {
            setDoubleBuffered(false);
            JarFile jf = null;
            try {
                jf = new JarFile(new File("jarfiles", jarfileName));
            } catch (Exception ex) {
                ex.printStackTrace();
                vector.addElement(jarfileName + " IOException");
                return;
            }
    
            JarEntry entry = null;
            try {
                entry = jf.getJarEntry(entryName + ".db");
            } catch (Exception ex) {
                ex.printStackTrace();
                vector.addElement(entryName + ".db");
                vector.addElement("Does NOT exist in " + jarfileName);
                return;
            }

            InputStream is = null;
            try {
                is = jf.getInputStream(entry);
            } catch (Exception ex) {
                ex.printStackTrace();
                vector.addElement(entryName + ".db");
                vector.addElement("IO Exception in " + jarfileName);
                return;
            }
    
            try {
                if ((reader = new BufferedReader(new InputStreamReader(is))) == null)
                    return;
            } catch (Exception e) { }
            int type = 1, w = 200, h = 200;
            try {
                type = Integer.parseInt(reader.readLine());
                w = Integer.parseInt(reader.readLine());
                h = Integer.parseInt(reader.readLine());
                dbsize = Integer.parseInt(reader.readLine());
            } catch (Exception e) { }
    
            bimg = new BufferedImage(w, h, type);

            vector.addElement(jarfileName);
            vector.addElement(entryName + ".db");
            mThread = new Thread(this);
            mThread.setName("ImageComparator.Master");
            mThread.start();
        }
    
        public void paint(Graphics g) {
            render(g, this, bimg, vector);
        }

        public void run() {
            DataBuffer db = bimg.getRaster().getDataBuffer();
            for (int i = 0; i < dbsize; i++) {
                try {
                    db.setElem(i, Integer.parseInt(reader.readLine()));
                } catch (Exception e) { break; }
            }
            repaint();
            mThread = null;
        }
    } // End Master class


    /**
     * Take the difference of Master BufferedImage and the Run Time 
     * BufferedImage
     */
    public static class Difference extends JPanel implements Runnable {

        private BufferedImage b1, b2, b3;
        private Vector vector = new Vector();

        public Difference(String entryName) {
            if (ImageComparator.Master.bimg != null) {
                b1 = ImageComparator.Master.bimg;
                TestSurface surface = null;
                try {
                    surface = (TestSurface) Class.forName(entryName).newInstance();
                } catch (Exception e) {
                    e.printStackTrace(); 
                }
                b2 = surface.getRenderedBufferedImage(b1.getType());
                if (b1.getType() != b2.getType()) {
                    vector.addElement("BI_1 type != BI_2 type");
                }
                dThread = new Thread(this);
                dThread.setName("ImageComparator.Difference");
                dThread.start();
            }
        }
    
        public void paint(Graphics g) {
            render(g, this, b3, vector);
        }

        public void run() {
            while (mThread != null) {
                try {
                    dThread.sleep(100);
                } catch (Exception e) {}
            } 
            DataBuffer db1 = b1.getRaster().getDataBuffer();
            DataBuffer db2 = b2.getRaster().getDataBuffer();
            b3 = new BufferedImage(b1.getWidth(), b1.getHeight(), b1.getType());
            DataBuffer db3 = b3.getRaster().getDataBuffer();

            double percent = 0.0;

            for (int i = 0; i < db1.getSize(); i++) {
                if (db1.getElem(i) != db2.getElem(i)) {
                    // What's the best way to represent pixel difference?
                    //  .. db1.getElem(i) - db2.getElem(i) ..
                    //         - or -
                    //  .. db1.getElem(i) ..
                    //         - or -
                    //  .. db2.getElem(i) ..
                    //System.out.println(db1.getElem(i)+" "+db2.getElem(i));
                    db3.setElem(i, db1.getElem(i));
                } else {
                    percent++;
                }
            }
            percent = percent/db1.getSize()*100;
            String s = Double.toString(percent);
            s = (s.length() < 5) ? s : s.substring(0,5);
            s = percent == 0 ? "100" : s;
            vector.addElement("Pixel Comparison " + s + "% equal");
            if (RunInterface.thread != null) {
                RunInterface.print(" : " + s + "% equal");
                RunInterface.percent = percent;
            }
            repaint();
            dThread = null;
        }
    } // End Difference class


    /**
     * Rendering for both the Master and Difference Panels.
     */
    public static void render(Graphics g, 
                              Component cmp, 
                              BufferedImage bimg, 
                              Vector vector) {
        if (!cmp.isShowing()) {
           return;
        }
        Dimension d = cmp.getSize();
        Graphics2D g2 = (Graphics2D) g;
        g2.setBackground(Color.gray);
        g2.clearRect(0,0,d.width,d.height);
        if (bimg == null) {
            g2.setColor(Color.yellow);
            g2.setFont(new Font("Dialog", Font.PLAIN, 14));
            FontMetrics fm = g2.getFontMetrics();
            int ascent = fm.getAscent();
            int descent = fm.getDescent();
            int ssH = 1;
            for (int i = 0; i < vector.size(); i++) {
                ssH += ascent;
                g2.drawString((String) vector.get(i),2,ssH);
                ssH += descent;
            }
            return;
        }
        g2.setColor(Color.black);
        g2.setFont(new Font("Dialog", Font.PLAIN, 10));
        if (vector.size() != 0) {
            FontMetrics fm = g2.getFontMetrics();
            g2.drawString((String) vector.get(0), 2,(int) fm.getAscent());
            if (vector.size() > 1) {
                g2.drawString((String) vector.get(1), 2, d.height-3);
            }
        }
        int x = (d.width - bimg.getWidth()) / 2;
        int y = (d.height - bimg.getHeight()) / 2;
        g2.drawImage(bimg, (x < 0) ? 0 : x, (y < 0) ? 0 : y, null);
    }


    public static boolean doneDrawing() {
        return ((mThread == null) && (dThread == null));
    }
}
