/*
 * @(#)JarFilesInterface.java	1.6 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.util.jar.*;
import java.util.*;
import java.io.*;


/**
 * DataBuffers of the tests are generated and stored into JarFiles.
 *
 * @version @(#)JarFilesInterface.java	1.6 00/02/02
 * @author Brian Lichtenwalter
 */

public class JarFilesInterface extends JPanel implements ItemListener, ActionListener, Runnable {

    private JComboBox sizeCombo;  
    private JarOutputStream jos;
    private Thread thread;
    private JPanel processP;
    private JProgressBar pb = new JProgressBar();
    private JTextArea textarea;
    private Font font = new Font("serif", Font.PLAIN, 10);
    private int imageType;

    static int width = 200, height = 200;
    static JButton jarB;
    static String jtag = "";
    static JTextField tf;


    public JarFilesInterface() {

        setLayout(new GridLayout(0,1));
        setTitleBorder(this, "JarFile Creation of DataBuffers");

	JPanel p1 = new JPanel(new GridBagLayout());
        JLabel label = new JLabel("jarfile tag=");
	label.setMinimumSize(new Dimension(80,22));
        TestProduct.addToGridBag(p1, label, 0, 0, 1, 1, 0.0, 0.0);
        tf = new JTextField(jtag);
	tf.setPreferredSize(new Dimension(110,22));
	tf.setMinimumSize(new Dimension(80,22));
        TestProduct.addToGridBag(p1, tf, 1, 0, 1, 1, 1.0, 0.0);

/*
	sizeCombo = new JComboBox();
	sizeCombo.addItemListener(this);
	sizeCombo.setPreferredSize(new Dimension(130, 24));
	sizeCombo.setAlignmentX(LEFT_ALIGNMENT);
	sizeCombo.addItem("100x100");
	sizeCombo.addItem("150x150");
	sizeCombo.addItem("200x200");
	sizeCombo.addItem("250x250");
	sizeCombo.addItem("300x300");
	sizeCombo.setSelectedIndex(2);
*/

        jarB = new JButton("Generate JarFiles");
        jarB.setBackground(Color.green);
        jarB.addActionListener(this);
        JPanel p7 = new JPanel(new BorderLayout());
        p7.setBorder(new EmptyBorder(5,5,5,5));
        p7.add(jarB);
        TestProduct.addToGridBag(p1, p7, 0, 1, 2, 1, 1.0, 1.0);

        processP = new JPanel(new BorderLayout());
        setTitleBorder(processP, "Controls + Tests Progress");
        processP.add(pb);
        TestProduct.addToGridBag(p1, processP, 0, 2, 2, 1, 1.0, 1.0);

        JPanel p3 = new JPanel(new BorderLayout());
        p3.setBorder(new EmptyBorder(5,0,0,0));
        textarea = new JTextArea("JarFile Creation of DataBuffers\n");
        textarea.setForeground(Color.black);
        textarea.setBackground(Color.lightGray);
        textarea.setEditable(false);
        JScrollPane scroller = new JScrollPane();
        scroller.getViewport().add(textarea);
        p3.add(scroller);
        add(p1);
        add(p3);
    }


    public void setTitleBorder(JPanel p, String name) {
        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        SoftBevelBorder sbb = new SoftBevelBorder(BevelBorder.RAISED);
        CompoundBorder cb = new CompoundBorder(eb, sbb);
        TitledBorder tb = new TitledBorder(cb, name);
        tb.setTitleFont(font);
        p.setBorder(tb);
        p.repaint();
    }


    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand() == "Generate JarFiles") {
            jarB.setText("Stop Generation");
            jarB.setBackground(Color.red);
            start();
        } else if (e.getActionCommand() == "Stop Generation") {
            stop();
        }
    }



    public void itemStateChanged(ItemEvent e) {
        switch (sizeCombo.getSelectedIndex()) {
            case 0 : width = 100; height = 100; break;
            case 1 : width = 150; height = 150; break;
            case 2 : width = 200; height = 200; break;
            case 3 : width = 250; height = 250; break;
            case 4 : width = 300; height = 300; break;
        }
    }


    public void createJarFile(int index) {

        File file = new File("jarfiles");
        if (!file.canWrite()) {
            System.out.println("Can NOT write to jarfiles directory!");
            return;
        }
        if (!file.isDirectory()) {
            file.mkdir();
        }
                    
        String name = ControlsInterface.getFullName(index);
        String tag = tf.getText().trim();
        file = new File(file, name + tag + ".jar");

        try {           
            FileOutputStream os = new FileOutputStream(file);
            jos = new JarOutputStream(os, new Manifest());
            String str = "\nCreated " + file.getPath() + "\n";
            textarea.append(str);
            int y = textarea.getPreferredSize().height;
            textarea.scrollRectToVisible(new Rectangle(0,y,1,1));
            System.out.print(str);
        } catch (Exception e) { e.printStackTrace(); }

        scan((DefaultMutableTreeNode) SelectionInterface.tree2.getModel().getRoot());
        try {
            jos.close();
        } catch (Exception e) { e.printStackTrace(); }
    }


    private void scan(DefaultMutableTreeNode node) {

        if (!node.isLeaf()) {
            TreeModel tm = SelectionInterface.tree2.getModel();
            int num = tm.getChildCount(node);
            for (int i = 0; i < num; i++) {
                DefaultMutableTreeNode leafNode = (DefaultMutableTreeNode) tm.getChild(node, i);
                CheckData cd = (CheckData) leafNode.getUserObject();
                scan(leafNode);
            }
        } else {
            if (thread == null) {
                return;
            }
            CheckData cd = (CheckData) node.getUserObject();
            if (!cd.checked) {
                return;
            }
            setTitleBorder(processP,
                ControlsInterface.screenNames[imageType] + " " +
                cd.path.substring(cd.path.indexOf('/')+1));
            char c = System.getProperty("file.separator").toCharArray()[0];
            String className = cd.path.replace('/','.');
            className = className.replace(c,'.');
            TestSurface surface = null;
            try {
                surface = (TestSurface) Class.forName(className).newInstance();
            } catch (Exception e) {
                e.printStackTrace();
            }
            BufferedImage bimg = 
                  surface.getRenderedBufferedImage(imageType);
            if (thread != null && bimg != null) {
                SaveBIDataBuffer(bimg, className);
            }
            pb.setValue(pb.getValue() + 1);
            pb.repaint();
        }
    }

    //
    // Put DataBuffer into CharArray and store as entry into jar file
    //
    public void SaveBIDataBuffer(BufferedImage bi, String name) {
        CharArrayWriter out = new CharArrayWriter();
        String s = String.valueOf(bi.getType())+"\n";
        out.write(s, 0, s.length());
        out.write(s = String.valueOf(bi.getWidth())+"\n", 0, s.length());
        out.write(s = String.valueOf(bi.getHeight())+"\n", 0, s.length());
        DataBuffer db = bi.getRaster().getDataBuffer();
        out.write(s = String.valueOf(db.getSize())+"\n", 0, s.length());
        for (int i = 0; i < db.getSize() && thread != null; i++) {
            out.write(s = String.valueOf(db.getElem(i))+"\n", 0, s.length());
        }
        out.close();
        putEntry(out.toString().getBytes(), name + ".db");
    }


    private void putEntry(byte[] bytes, String name) {
        try {
            JarEntry entry = new JarEntry(name);
            jos.putNextEntry(entry);
            jos.write(bytes);
            jos.closeEntry();
            String s = entry.getCompressedSize() + " " + entry.getName()+"\n";
            textarea.append(s);
            int y = textarea.getPreferredSize().height;
            textarea.scrollRectToVisible(new Rectangle(0,y,1,1));
            System.out.print(s);
        } catch (Exception ex) { ex.printStackTrace(); }
    }


    public void start() {
        thread = new Thread(this);
        thread.setPriority(Thread.MIN_PRIORITY);
        thread.start();
    }


    public void stop() {
        jarB.setText("Generate JarFiles");
        jarB.setBackground(Color.green);
        jarB.repaint();
        setTitleBorder(processP, "Controls + Tests Progress");
        pb.setValue(pb.getMaximum());
        pb.repaint();
        thread = null;
    }


    public void run() {

        pb.setValue(0);

        // Compute maximum for progress bar.
        SelectionInterface.numSelectedTests = 0;
        SelectionInterface.scanCount((DefaultMutableTreeNode) SelectionInterface.tree2.getModel().getRoot());
        int numSelectedControls = 0;

        Vector data = ControlsInterface.data;

        for (int i = 0; i < data.size(); i++) {
            if (((Boolean) ((Object[]) data.get(i))[0]).booleanValue()) {
                numSelectedControls++;
            }
        }

        int max = SelectionInterface.numSelectedTests * numSelectedControls;
        pb.setMaximum(max);

        for (int i = 0; i < data.size() && thread != null; i++) {
            if (((Boolean) ((Object[]) data.get(i))[0]).booleanValue()) {
                String s = (String)((Object[]) data.get(i))[1];
                for (imageType = 0; imageType < ControlsInterface.screenNames.length; imageType++) {
                    if (s.compareTo(ControlsInterface.screenNames[imageType]) == 0) {
                        break;
                    }
                }
                if (imageType != 0) {
                    createJarFile(i);
                }
            }
        }
        if (TestProduct.exit) {
            System.out.println("All Finished, System Exit.\n");
            System.exit(0);
        }
        if (thread != null) {
            stop();
        }
    }
}
