/*
 * @(#)RunInterface.java	1.9 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.*;
import java.util.Date;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultMutableTreeNode;


/**
 * Controls for running the test suite.
 *
 * @version @(#)RunInterface.java	1.9 00/02/02
 * @author Brian Lichtenwalter
 */

public class RunInterface extends JPanel implements Runnable, ActionListener, ChangeListener
{
    static TitledBorder titledBorder;
    static int delay = 3;
    static JButton runB;
    static Thread thread; 
    static JTextArea textarea;
    static JSlider jslider;
    static PrintWriter fileOut;
    static double percent;
    static boolean toPrinter;
    static String rfile = "results";

    private JButton passB, failB, pauseB;
    private boolean nextTest, pass;
    private JPanel sliderPanel;
    private JProgressBar pb = new JProgressBar();
    private JPanel panel;
    private JLabel label;
    private JTextField tf;
    private int numPass, numFail;
    private int totalPass, totalFail;


    public RunInterface() {

        setLayout(new BorderLayout());
        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        SoftBevelBorder sbb = new SoftBevelBorder(BevelBorder.RAISED);
        setBorder(new CompoundBorder(eb, new BevelBorder(BevelBorder.LOWERED)));

        String s = "Java2D TestProduct\n" + 
                   "java version: " + 
                   System.getProperty("java.version") + "\n" + 
                   System.getProperty("os.name") + " " + 
                   System.getProperty("os.version");
        textarea = new JTextArea(s);
        JScrollPane scroller = new JScrollPane();
        scroller.getViewport().add(textarea);
        textarea.setFont(new Font("TimesRoman", Font.PLAIN, 14));
        textarea.setForeground(Color.black);
        textarea.setBackground(Color.lightGray);
        textarea.setEditable(false);
        textarea.setAutoscrolls(true);
        add("Center", scroller);

        JPanel southPanel = new JPanel(new GridLayout(1,0));

        panel = new JPanel(new GridBagLayout());
        panel.setBorder(new EmptyBorder(5,5,5,5));
        runB = makeButton("Run", Color.lightGray);
        pauseB = makeButton("Pause", Color.yellow);
        passB = makeButton("Pass", Color.green);
        failB = makeButton("Fail", Color.red);
        TestProduct.addToGridBag(panel, runB, 0, 0, 2, 1, 1.0, 1.0);
        label = new JLabel("results file=");
        label.setFont(new Font("serif", Font.PLAIN, 12));
	label.setMinimumSize(new Dimension(70,25));
        TestProduct.addToGridBag(panel, label, 0, 1, 1, 1, 0.0, 0.0);
        tf = new JTextField(rfile);
	tf.setPreferredSize(new Dimension(110,25));
	tf.setMinimumSize(new Dimension(80,25));
        TestProduct.addToGridBag(panel, tf, 1, 1, 1, 1, 1.0, 0.0);

        southPanel.add(panel);


        sliderPanel = new JPanel(new BorderLayout());
        CompoundBorder cb = new CompoundBorder(eb, sbb);
        sliderPanel.setBorder(cb);
        jslider = new JSlider(JSlider.HORIZONTAL, 0, 30, delay);
        jslider.setPaintTicks(true);
        jslider.setMajorTickSpacing(10);
        jslider.setMinorTickSpacing(5);
        titledBorder = new TitledBorder("Delay = " + String.valueOf(delay));
        titledBorder.setTitleFont(new Font("serif", Font.PLAIN, 10));
        jslider.setBorder(titledBorder);
        jslider.addChangeListener(this);
        sliderPanel.add(jslider);

        southPanel.add(sliderPanel);

        add("South", southPanel);
    }


    public void stateChanged(ChangeEvent e) {
        setDelay(jslider.getValue());
    }


    static void setDelay(int d) {
        titledBorder.setTitle("Delay = " + String.valueOf(delay = d));
        jslider.validate();
        jslider.repaint();
    }


    static void print(String s) {
        System.out.print(s);
        textarea.append(s);
        int y = textarea.getPreferredSize().height;
        textarea.scrollRectToVisible(new Rectangle(0,y,1,1));

        if (fileOut != null) {
            fileOut.print(s);
            fileOut.flush();
        }
    }



    public void actionPerformed(ActionEvent e) {
        String title = e.getActionCommand();
        if (title == "Pass") {
            print(" - Pass.");
            numPass++;
            pass = true;
            nextTest = true;
            return;
        } else if (title == "Fail") {
            pass = false;
            numFail++;
            print(" - Fail. ");
            nextTest = true;
            return;
        } else if (title == "Run") {
            panel.removeAll();
            TestProduct.addToGridBag(panel, runB, 0, 0, 1, 1, 1.0, 1.0);
            TestProduct.addToGridBag(panel, pauseB, 1, 0, 1, 1, 1.0, 1.0);
            TestProduct.addToGridBag(panel, passB, 0, 1, 1, 1, 1.0, 1.0);
            TestProduct.addToGridBag(panel, failB, 1, 1, 1, 1, 1.0, 1.0);
            passB.setEnabled(true);
            failB.setEnabled(true);
            runB.setText("Stop");
            nextTest = false;
            start();
        } else if (title == "Stop") {
            runB.setBackground(Color.lightGray);
            panel.removeAll();
            TestProduct.addToGridBag(panel, runB, 0, 0, 2, 1, 1.0, 1.0);
            TestProduct.addToGridBag(panel, label, 0, 1, 1, 1, 0.0, 0.0);
            TestProduct.addToGridBag(panel, tf, 1, 1, 1, 1, 1.0, 0.0);
            stop();
        } else if (title == "Pause" || title == "Resume") {
            pauseB.setSelected(!pauseB.isSelected());
            panel.removeAll();
            if (title == "Pause") {
                pauseB.setText("Resume");
                TestProduct.addToGridBag(panel, pauseB, 0, 0, 1, 1, 1.0, 1.0);
            } else {
                pauseB.setText("Pause");
                TestProduct.addToGridBag(panel, runB, 0, 0, 1, 1, 1.0, 1.0);
                TestProduct.addToGridBag(panel, pauseB, 1, 0, 1, 1, 1.0, 1.0);
                TestProduct.addToGridBag(panel, passB, 0, 1, 1, 1, 1.0, 1.0);
                TestProduct.addToGridBag(panel, failB, 1, 1, 1, 1, 1.0, 1.0);
            }
        }
        panel.validate();
    }

    
    private void pauseCheck() {
        while (pauseB.isSelected()) {
            try {
                thread.sleep(100);
            } catch (Exception e) {}
        } 
    }


    private JButton makeButton(String bName, Color c) {
        JButton b = new JButton(bName);
        b.setBackground(c);
        b.setFont(new Font("Dialog", Font.PLAIN, 10));
        b.addActionListener(this);
        return b;
    }


    private void start() {
        thread = new Thread(this);
        thread.setName("RunInterface");
        thread.start();
    }


    private void stop() {
        thread = null;
    }


    private void scan(DefaultMutableTreeNode node) {

        if (!node.isLeaf()) {
            TreeModel tm = SelectionInterface.tree2.getModel();
            int num = tm.getChildCount(node);
            for (int i = 0; i < num; i++) {
                DefaultMutableTreeNode leafNode = (DefaultMutableTreeNode) tm.getChild(node, i);
                CheckData cd = (CheckData) leafNode.getUserObject();
                scan(leafNode);
            }
        } else {
            if (thread == null) {
                return;
            }
            percent = -1.0;
            CheckData cd = (CheckData) node.getUserObject();
            if (!cd.checked) {
                return;
            }
            pb.setValue(0);
            print("\n " + cd.path);
            TestProduct.panel.removeAll();
            System.gc();
            System.runFinalization();
            TestProduct.group = new TestGroup(cd.path);
            TestProduct.panel.add(TestProduct.group);
            TestProduct.panel.validate();

            nextTest = false;

            TestPanel tp = (TestPanel) TestProduct.group.panel.getComponent(0);


            if (TestProduct.group.panel.getComponentCount() >= 3) {
                while (!ImageComparator.doneDrawing()) {
                    try {
                        thread.sleep(100);
                    } catch (Exception e) { e.printStackTrace(); }
                    pauseCheck();
                }
            }

            // Print the test
            if (toPrinter) {
                tp.toolbar.printB.doClick();
            }

            // Automated mode when the Delay > 0
            for (int l = 0; l < delay && !nextTest && thread != null; l++) { 
                for (int m = 0; m < 10 && !nextTest; m++) {
                    pb.setMaximum(delay);
                    try {
                        thread.sleep(99);
                    } catch (Exception e) { e.printStackTrace(); }
                    pauseCheck();
                 }
                 pb.setValue(pb.getValue() + 1);
            }

            if (!nextTest) { 
                //
                // what is an acceptable percentage for passing the test?
                //
                if (percent > 99.0) {
                    passB.doClick();
                } else {
                    failB.doClick();
                }
            }
        }
    }



    public void run() {

        numPass = numFail = totalPass = totalFail = 0;
        pb.setMinimum(0);
        sliderPanel.add("South", pb);
        validate();

        File file = new File("results");
        if (file.canWrite()) {
            if (!file.isDirectory()) {
                file.mkdir();
            }
            file = new File(file, tf.getText().trim());
            try {
                FileOutputStream fos = new FileOutputStream(file);
                fileOut = new PrintWriter(fos);
            } catch (Exception ex) { 
                System.out.println("Error creating results file");
            }
        }

        String s = "\n\nJava2D TestProduct\n" + 
                   "java version: " + 
                   System.getProperty("java.version") + "\n" + 
                   System.getProperty("os.name") + " " + 
                   System.getProperty("os.version") + "\n" + 
                   new Date().toString() + "\n";
        print(s);

        int saveIndex = ControlsInterface.index;
        for (ControlsInterface.index = 0; ControlsInterface.index < ControlsInterface.data.size() && thread != null; ControlsInterface.index++) {
            if (!((Boolean) ((Object[]) ControlsInterface.data.get(ControlsInterface.index))[0]).booleanValue()) {
                continue;
            }
            String name = ControlsInterface.getFullName(ControlsInterface.index);
            String tag = JarFilesInterface.tf.getText().trim();
            name = name + tag + ".jar";
            file = new File("jarfiles", name);
            if (file == null || !file.exists()) {
                print("\n" + name + " does not exist.");
                continue;
            } 
            print("\n\n" + name);

            scan((DefaultMutableTreeNode) SelectionInterface.tree2.getModel().getRoot());

            print("\nResults PASS : " + numPass);
            print("  FAIL : " + numFail + "\n");
            totalPass += numPass; totalFail += numFail;
            numPass = numFail = 0;
        }
        ControlsInterface.index = saveIndex;
        print("\n\nTotal Results\n");
        print("Tests that passed : " + totalPass + "\n");
        print("Tests that failed : " + totalFail + "\n");
        if (fileOut != null) {
            fileOut.close();
        }
        if (TestProduct.exit) {
            System.out.println("All Finished, System Exit.\n");
            System.exit(0);
        } else {
            runB.setText("Run");
            passB.setEnabled(false);
            failB.setEnabled(false);
            sliderPanel.remove(pb);
            panel.removeAll();
            TestProduct.addToGridBag(panel, runB, 0, 0, 2, 1, 1.0, 1.0);
            TestProduct.addToGridBag(panel, label, 0, 1, 1, 1, 0.0, 0.0);
            TestProduct.addToGridBag(panel, tf, 1, 1, 1, 1, 1.0, 0.0);
            validate();
        }
    }
}
