/*
 * @(#)SelectionInterface.java	1.7 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import java.io.*;


/**
 * Controls for selecting which tests to run.
 *
 * @version @(#)SelectionInterface.java	1.7 00/02/02
 * @author Brian Lichtenwalter
 */

public class SelectionInterface extends JPanel implements ItemListener {

    private JTree tree1;
    private JScrollPane sp;
    private JCheckBox cb;

    static int numSelectedTests;
    static JTree tree2;


    public SelectionInterface() {

        EmptyBorder emptyborder = new EmptyBorder(5,5,5,5);
        BevelBorder bevelborder = new BevelBorder(BevelBorder.LOWERED);
	setLayout(new BorderLayout());
	setBorder(new CompoundBorder(emptyborder, bevelborder));

	DefaultMutableTreeNode top1 = new DefaultMutableTreeNode("tests");

	CheckData cd = new CheckData("tests/", "tests", true);
	DefaultMutableTreeNode top2 = new DefaultMutableTreeNode(cd);

	scanForTests("tests", top1, top2);

	tree1 = new JTree(top1);
	tree1.addMouseListener(ml1);

	tree2 = new JTree(top2);
        tree2.setBackground(Color.lightGray);
	tree2.addMouseListener(ml2);
	tree2.setCellRenderer(new TreeRenderer());

	sp = new JScrollPane();
	sp.getViewport().add(tree1);
	add(sp);

	cb = new JCheckBox("Select Tests For Run", false);
        cb.addItemListener(this);
	add("South", cb);
    }


    DefaultMutableTreeNode tn1, tn2;


    public void scanForTests(String name, 
	    			DefaultMutableTreeNode node1,
				DefaultMutableTreeNode node2) {
	File file = new File(name);
	if (file != null && file.isDirectory()) { 
	    String files[] = file.list();
	    java.util.Arrays.sort(files);
	    for (int i = 0; i < files.length; i++) {
		File leaf = new File(file.getAbsolutePath(), files[i]);	
		String s1 = leaf.getName();
	        String path = leaf.getAbsolutePath();
		if (leaf.isDirectory()) {
	            path = path.substring(path.indexOf("tests"));
	    	    tn1 = new DefaultMutableTreeNode(s1);
	    	    node1.add(tn1);
	    	    CheckData cd = new CheckData(path, s1, true);
	    	    tn2 = new DefaultMutableTreeNode(cd);
		    node2.add(tn2);
		    scanForTests(leaf.getAbsolutePath(), tn1, tn2);
		} else {
		    if (s1.endsWith(".java")) {
		        String s2 = s1.substring(0,s1.indexOf('.'));
		        node1.add(new DefaultMutableTreeNode(s2));
	                path = path.substring(path.indexOf("tests"),
				    path.indexOf('.'));
	    	    	CheckData cd = new CheckData(path, s2, true);
		        node2.add(new DefaultMutableTreeNode(cd));
		    }
		}
	    }
	} 
    }



    public void itemStateChanged(ItemEvent e) {
	sp.getViewport().removeAll();
	sp.getViewport().add(cb.isSelected() ? tree2 : tree1);
	validate();
	repaint();
    }


    /**
     * Mouse listener for Tree to view selected test or test group
     */
    MouseListener ml1 = new MouseAdapter(){
        public void mouseClicked(MouseEvent e){
            if(e.getClickCount() > 1){
		TestProduct.panel.removeAll();
		TestProduct.panel.revalidate();
		System.gc();
		System.runFinalization();
		TreePath treepath = tree1.getSelectionPath();
                if (treepath == null) {
                    return;
                }
		Object[] os = treepath.getPath();

		if (os.length == 1) {
		    return;
		} 

		String s = "tests/";
		
		for (int i = 1; i < os.length; i++) {
		    s += os[i].toString();
		    if ((i+1) != os.length)
			s += "/";
		}

		//System.out.println("Tree, Selected = " + s);
		TestProduct.group = new TestGroup(s);
		TestProduct.panel.add(TestProduct.group);
		TestProduct.panel.validate();
            }
        }
    };


    private void modifySelected(DefaultMutableTreeNode node, boolean b) {
        CheckData cd = (CheckData) node.getUserObject();
	cd.checked = b;
	if (!node.isLeaf()) {
    	    TreeModel tm = tree2.getModel();
	    int num = tm.getChildCount(node);
	    for (int i = 0; i < num; i++) {
	        modifySelected((DefaultMutableTreeNode) tm.getChild(node, i), b);
	    }
	}
    }



    static void scanCount(DefaultMutableTreeNode node) {
        CheckData cd = (CheckData) node.getUserObject();
	if (!node.isLeaf()) {
    	    TreeModel tm = tree2.getModel();
	    int num = tm.getChildCount(node);
	    for (int i = 0; i < num; i++) {
	        scanCount((DefaultMutableTreeNode) tm.getChild(node, i));
	    }
	} else {
	    if (cd.checked) {
		numSelectedTests++;
            }
	}
    }



    /**
     * Mouse listener for Tree to select tests and test groups in run mode.
     */
    MouseListener ml2 = new MouseAdapter(){
        public void mouseClicked(MouseEvent e){
            if (e.getClickCount() == 1) {
		TreePath treepath = tree2.getSelectionPath();
		if (treepath == null)
		    return;
		Object[] os = treepath.getPath();
		int x = e.getX();
		int y = e.getY();
		if ((os.length == 1 && y <= 20) ||
		       (os.length == 2 && x >= 20) ||
		          (os.length == 3 && x >= 40) || 
		              (os.length == 4 && x >= 60) ||
		                  (os.length == 5 && x >= 80)) 
                {


    		    DefaultMutableTreeNode tn = 
			(DefaultMutableTreeNode) treepath.getLastPathComponent();
        	    CheckData cd = (CheckData) tn.getUserObject();
    		    modifySelected(tn, !cd.checked);
		    tree2.treeDidChange();
		}
            }
        }
    };



    class TreeRenderer extends JCheckBox implements TreeCellRenderer {

        public Component getTreeCellRendererComponent(
                JTree tree,
                Object value,       
                boolean selected,    
                boolean expanded,  
                boolean leaf,
                int row,
                boolean hasFocus)
        {
            setBackground(Color.lightGray);
	    DefaultMutableTreeNode tn = (DefaultMutableTreeNode) value;
	    CheckData cd = (CheckData) tn.getUserObject();
	    setText(cd.name);
	    setSelected(cd.checked);
            return this;
        }
    }
}
