/*
 * @(#)TestGroup.java	1.12 01/04/21
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
import javax.swing.border.*;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.net.URL;


/**
 * TestGroup can represent multiple tests or a single test, a panel
 * for a master image and a panel for the difference image.
 *
 * @version @(#)TestGroup.java	1.12 01/04/21
 * @author Brian Lichtenwalter
 */

public class TestGroup extends JPanel implements MouseListener, Runnable {

    public String name;      
    public JPanel panel;

    private JPanel leafPanel;
    private Thread thread;
    private EmptyBorder emptyborder = new EmptyBorder(2,2,2,2);
    private SoftBevelBorder sbb = new SoftBevelBorder(BevelBorder.RAISED);
    private JTabbedPane tabPane;
    private String bugID = null;


    public TestGroup(String name) {

        setLayout(new BorderLayout());

        panel = new JPanel(new GridLayout(0,2));
        panel.setBorder(emptyborder);
        add(panel);
        scanForTests(name, panel);
        fixBorders();
        fixLayout(panel);
    }


    public void scanForTests(String name, JPanel parentPanel) {
        File file = new File(name);
        if (file != null && file.isDirectory()) { 
            String files[] = file.list();
            for (int i = 0; i < files.length; i++) {
                File leafFile = new File(file.getAbsolutePath(), files[i]);     
                String s1 = leafFile.getName();
                if (leafFile.isDirectory()) {
                    leafPanel = new JPanel(new GridLayout(0,2));
                    leafPanel.setBorder(new CompoundBorder(emptyborder, sbb));
                    scanForTests(leafFile.getAbsolutePath(), leafPanel);
                    parentPanel.add(leafPanel);
                    fixLayout(leafPanel);
                } else {
                    if (s1.indexOf('$') == -1 && s1.endsWith(".class")) {
                        s1 = leafFile.getAbsolutePath();
                        s1 = s1.substring(s1.indexOf("tests"),s1.indexOf('.'));
                        this.name = name;
                        TestPanel tp = new TestPanel(s1);
                        tp.surface.addMouseListener(this);
                        parentPanel.add(tp);
                    }
                }
            }
        } else if (file != null) {
            this.name = name;
            TestPanel tp = new TestPanel(name);
            panel.add(tp);
            panel.setBorder(new EmptyBorder(2,2,2,2));
            AddImageComparator();
            if (TestProduct.panel.isShowing()) {
                AddInfoPanel();
            }
        }
    }


    public void repaintTests(JPanel panel) {
        Component cmps[] = panel.getComponents();
        for (int i = 0; i < cmps.length && thread != null; i++) {
            if (cmps[i] instanceof TestPanel) {
                ToolBar tb = ((TestPanel) cmps[i]).toolbar;
                boolean aa = ControlsInterface.getAntiAlias();
                if (aa != tb.aliasB.isSelected()) {
                    tb.aliasB.doClick();
                }
                int screenIndex = ControlsInterface.getScreenIndex();
                if (screenIndex != tb.screenCombo.getSelectedIndex()) {
                    tb.screenCombo.setSelectedIndex(screenIndex);
                }
            } else if (cmps[i] instanceof JPanel) {
                repaintTests((JPanel) cmps[i]);
            }
        }
    }


    public void mouseClicked(MouseEvent e) {
        panel.removeAll();
        panel.setLayout(new BorderLayout());
        panel.validate();

        name = e.getComponent().toString();
        name = name.substring(0, name.indexOf('['));

        panel.add(new TestGroup(name));
        panel.validate();
    }

    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    

    public void AddImageComparator() {
        String cname = ControlsInterface.getFullName(ControlsInterface.index);
        if (cname.startsWith("On")) {
            return;
        }
        if (panel.getComponentCount() == 1) {
            panel.setLayout(new GridLayout(0,2,2,2));
            panel.validate();
        }
        String entryName = name.replace('/','.').replace('\\','.');
        String tag = JarFilesInterface.tf.getText().trim();
        String jarfileName = cname + tag + ".jar";
        File file = new File("jarfiles", jarfileName);
        if (file != null && file.exists()) {
            panel.add(new ImageComparator.Master(entryName, jarfileName));
            panel.add(new ImageComparator.Difference(entryName));
        }
    }


    public void AddInfoPanel() {
        name = name.replace('.', '/');
        File file = new File(name + ".java");
        if (file != null && file.exists()) {
            try {
                BufferedReader br = new BufferedReader(new FileReader(file));
                if (br != null) {
                    String str = null;
                    while ((str = br.readLine()) != null) {
                        if (str.indexOf("@bug") != -1) {
                            bugID = str.substring(8, 15);
                            break;
                        }
                    }
                }
                JToolBar toolbar = new JToolBar(JToolBar.VERTICAL);
                JEditorPane editorPane = new JEditorPane(file.toURL());
                editorPane.setEditable(false);
                JScrollPane scroller = new JScrollPane();
                JViewport viewPort = scroller.getViewport();
                viewPort.add(editorPane);
                if (bugID != null) {
                    toolbar.add(tabPane = new JTabbedPane());
                    tabPane.setFont(new Font("serif", Font.PLAIN, 10));
                    String subname = name.substring(name.lastIndexOf('/')+1);
                    tabPane.addTab(subname + ".java", scroller);
                    new BugLoader().start();
                } else {
                    toolbar.add(scroller);
                }
                panel.add(toolbar);
            } catch (Exception ex) { ex.printStackTrace(); }
        }
    }


    public void fixLayout(JPanel p) {
        if (p.getComponentCount() == 1) {
            Component cmp = p.getComponent(0);
            p.removeAll();
            p.setLayout(new BorderLayout());
            p.validate();
            p.add(cmp);
        }
        p.validate();
    }



    private void fixBorders() {
        int cmpCount = panel.getComponentCount();
        for (int j = 0; j < cmpCount; j++) {
            int top = 2;
            int left = 2;
            if (j == 1)
               left = 0;
            else if (j != 0 && j%2 == 0) {
               top = 0; 
            } else if (j != 0) {
               left = 0; top = 0;
            }
            Component cmp = panel.getComponent(j); 
            if (cmp instanceof JPanel) {
                JPanel p = (JPanel) panel.getComponent(j); 
                EmptyBorder eb = new EmptyBorder(top,left,2,2);
                p.setBorder(new CompoundBorder(eb, sbb));
            }
        }
    }


    public void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.setName("TestGroup " + name + " repaintTests");
            thread.start();
        }
    }


    public synchronized void stop() {
        thread = null;
    }


    public void run() {
        repaintTests(panel);
        thread = null;
    }

    
    class BugLoader extends Thread {
        public BugLoader() {
            setPriority(Thread.MIN_PRIORITY);
        }
        public void run() {
            try {
                String urlName = "http://andorra.eng:8080/cgi-bin/ws.exe"
                                 + "/bugtraq/bug.hts?where=bugid_value%3D"
                                 + bugID;
                JEditorPane ep = new JEditorPane(new URL(urlName));
                ep.setEditable(false);
                JScrollPane sp = new JScrollPane();
                JViewport vp = sp.getViewport();
                vp.add(ep);
                tabPane.addTab(bugID, sp);
                tabPane.revalidate();
            } catch (Exception ex) { }
        }
    }
}
