/*
 * @(#)TestProduct.java	1.5 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * The Java2D feature/functional & regression test suite.
 *
 * @version @(#)TestProduct.java	1.5	00/02/02
 * @author Brian Lichtenwalter
 */
public class TestProduct extends JPanel {

    static JPanel panel;
    static TestGroup group;
    static Intro intro = new Intro();
    static boolean exit;


    public TestProduct() {

	setLayout(new BorderLayout());
	setBackground(Color.lightGray);

	panel = new JPanel(new BorderLayout());
	panel.add(intro);
	add(panel);
    }


    // utility
    static void addToGridBag(JPanel panel, Component comp,
            int x, int y, int w, int h, double weightx, double weighty) {

        GridBagLayout gbl = (GridBagLayout) panel.getLayout();
        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.BOTH;
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = weightx;
        c.weighty = weighty;
        panel.add(comp);
        gbl.setConstraints(comp, c);
    }


    public static void main(String args[]) {

	boolean withGUI = true;
	int tmpDelay = -1;

	for (int i = 0; i < args.length; i++) {
	    if (args[i].startsWith("-h") || args[i].startsWith("-help")) {

System.out.println("\njava TestProduct -run -jtag=winNT.A -rfile=winNT.B -jfgeneration -delay=10 -nogui -print -exit\n");
System.out.println(
            "     -run                 start running\n" +
            "     -jtag=winNT.A        the jarfiles tag for generation & running\n" +
            "     -rfile=winNT.B       the result file\n" + 
            "     -jfgeneration        Generate JarFiles of DataBuffers per Controls.\n" +
            "     -delay=10            the delay between tests.\n" +
            "     -nogui               run w/o any graphic user interface.\n"+
	    "     -print               print test to default printer.\n" +
            "     -exit                explicitly exit; -nogui triggers it.\n");
		System.exit(0);

	    } else if (args[i].startsWith("-jtag=")) {
	        String s = args[i].substring(args[i].indexOf('=')+1);
	        JarFilesInterface.jtag = s;
	    } else if (args[i].startsWith("-rfile=")) {
	        String s = args[i].substring(args[i].indexOf('=')+1);
	        RunInterface.rfile = s;
	    } else if (args[i].startsWith("-delay=")) {
	        String s = args[i].substring(args[i].indexOf('=')+1);
	        RunInterface.delay = Integer.parseInt(s);
		tmpDelay = RunInterface.delay;
	    } else if (args[i].startsWith("-exit")) {
		TestProduct.exit = true;
	    } else if (args[i].startsWith("-nogui")) {
		TestProduct.exit = true;
	    }
	}

	final TestProduct tp = new TestProduct();
	final TabManager tm = new TabManager();

	for (int i = 0; i < args.length; i++) {
	    if (args[i].startsWith("-nogui")) {
		withGUI = false;
		if (tmpDelay == -1) {
		    RunInterface.delay = 0;  // default to 0 in nongui mode
		}
	    } else if (args[i].startsWith("-run")) {
	        TabManager.tabbedPane.setSelectedIndex(1);
	        RunInterface.runB.doClick();
	    } else if (args[i].startsWith("-jfgeneration")) {
	        TabManager.tabbedPane.setSelectedIndex(3);
	        JarFilesInterface.jarB.doClick();
	    } else if (args[i].startsWith("-print")) {
		RunInterface.toPrinter = true;
	    }
	}

	if (withGUI) {
	    JFrame frame = new JFrame("Java2D TestProduct");
	    WindowListener l = new WindowAdapter() {
	        public void windowClosing(WindowEvent e) {System.exit(0);}
	    };
	    frame.addWindowListener(l);
	    frame.getContentPane().add("Center", tp);
	    frame.pack();
	    int X = 30;
	    int WIDTH = 520;
	    frame.setSize(new Dimension(WIDTH,540));
    	    frame.setLocation(X,20);
	    frame.show();

    	    // Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	    frame = new JFrame("Tab Manager");
            frame.getContentPane().add("Center", tm);
            frame.pack();
            frame.setSize(new Dimension(350,400));
    	    frame.setLocation(WIDTH+X+5, 50);
            frame.show();
	} 
    }
}
