/*
 * @(#)TestSurface.java	1.8 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.awt.image.*;
import java.awt.print.*;
import javax.swing.JPanel;
import java.awt.event.*;


/**
 * All TestProduct tests extend this base class.
 */
public abstract class TestSurface extends JPanel implements Printable {

    public Object AntiAlias = RenderingHints.VALUE_ANTIALIAS_ON;
    public Object Rendering = RenderingHints.VALUE_RENDER_SPEED;
    public AlphaComposite composite;
    public Paint texture;
    public String perfStr;            
    public BufferedImage bimg;
    public int imageType;
    public String name;         

    private long startPerf;
    private boolean outputPerfStr;


    public TestSurface() {
        name = this.getClass().getName();
        name = name.substring(name.indexOf(".", 7)+1);
    }


    protected Image getImage(String name) {
        return TestImages.getImage(name, this);
    }


    public int getImageType() {
        return imageType;
    }


    public void setImageType(int imgType) {
        imageType = imgType;
        bimg = null;
    }


    public void setAntiAlias(boolean aa) {
        AntiAlias = aa 
            ? RenderingHints.VALUE_ANTIALIAS_ON
            : RenderingHints.VALUE_ANTIALIAS_OFF;
    }


    public void setRendering(boolean rd) {
        Rendering = rd
            ? RenderingHints.VALUE_RENDER_QUALITY
            : RenderingHints.VALUE_RENDER_SPEED;
    }


    public void setTexture(Object obj) {
        texture = (Paint) obj;
    }


    public void setComposite(boolean cp) {
        composite = cp 
            ? AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f) 
            : null;
    }


    public void setMonitor(boolean ps) {
        outputPerfStr = ps;
    }


    public Graphics2D getGraphics2D(int w, int h, Object obj) {

        Graphics2D g2 = null;
        if (obj instanceof Graphics) {
            g2 = (Graphics2D) obj;
        } else if (obj instanceof BufferedImage) {
            g2 = ((BufferedImage) obj).createGraphics();
        }
        g2.setBackground(Color.white);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, AntiAlias);
        g2.setRenderingHint(RenderingHints.KEY_RENDERING, Rendering);
        g2.clearRect(0, 0, w, h);
        g2.setClip(0, 0, w, h);

        if (texture != null) {
            // set composite to opaque for texture fills
            g2.setComposite(AlphaComposite.SrcOver);
            g2.setPaint(texture);
            g2.fillRect(0, 0, w, h);
        }

        if (composite != null) {
            g2.setComposite(composite);
        }

        return g2;
    }


    public BufferedImage getRenderedBufferedImage(int imgType) {
        int w = JarFilesInterface.width;
        int h = JarFilesInterface.height;
        BufferedImage bi = new BufferedImage(w, h, imgType);
        Graphics2D g2 = getGraphics2D(w, h, bi);
        try {
            render(w, h, g2);
        } catch (Exception ex) { 
            // a good test suite will capture this exception and propagate
            // it to an exception handler and report it.
            ex.printStackTrace(); 
        }
        g2.dispose();
        bi.flush();
        return bi;
    }


    // ...tests extend TestSurface implement this routine...
    public abstract void render(int w, int h, Graphics2D g2);


    public void paint(Graphics g) {
        Dimension d = getSize();
        g.setColor(Color.gray);
        g.fillRect(0, 0, d.width, d.height);
        startPerf = System.currentTimeMillis();
        BufferedImage bimg = null;
        if (imageType > 0 && imageType < 14) {
            bimg = getRenderedBufferedImage(imageType);
            int x = (d.width - bimg.getWidth()) / 2;
            int y = (d.height - bimg.getHeight()) / 2;
            g.drawImage(bimg, x < 0 ? 0 : x, y < 0 ? 0 : y, null);
        } else {
            render(d.width, d.height, getGraphics2D(d.width, d.height, g));
        }
        LogPerformance();
    }


    public int print(Graphics g, PageFormat pf, int pi) throws PrinterException {
        if (pi >= 1) {
            return Printable.NO_SUCH_PAGE;
        }

        Graphics2D g2d = (Graphics2D) g;
        g2d.translate(pf.getImageableX(), pf.getImageableY());
        g2d.translate(pf.getImageableWidth() / 2,
                      pf.getImageableHeight() / 2);
        
        Dimension d = getSize();

        double scale = Math.min(pf.getImageableWidth() / d.width,
                                pf.getImageableHeight() / d.height);
        if (scale < 1.0) {
            g2d.scale(scale, scale);
        }

        g2d.translate(-d.width / 2.0, -d.height / 2.0);

        if (bimg != null) {
            g2d.drawImage(bimg, 0, 0, this);
        } else {
            render(d.width, d.height, getGraphics2D(d.width, d.height, g2d));
        }

        return Printable.PAGE_EXISTS;
    }



    private void LogPerformance() {
        long end = System.currentTimeMillis();
        long rel = (end - startPerf);
        perfStr = name + " " + rel + " ms";
        if (outputPerfStr) {
            System.out.println(perfStr);
        }
    }


    protected static void createFrame(String name, TestSurface surface) {
        Frame f = new Frame("TestProduct - " + name);
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
        f.add("Center", new TestPanel(surface));
        f.pack();
        f.setSize(new Dimension(300,300));
        f.show();
    }
}
