/*
 * @(#)ToolBar.java	1.6 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


import java.awt.*;
import java.awt.event.*;
import java.awt.print.PrinterJob;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


/**
 * ToolBar to control individual demo graphic attributes.  
 *
 * @version @(#)ToolBar.java	1.6 00/02/02
 * @author Brian Lichtenwalter
 */

public class ToolBar extends JPanel implements ActionListener, MouseListener, Runnable {

    private static Font font = new Font("serif", Font.PLAIN, 10);
    private static Color roColor = new Color(204, 204, 255);

    private TestSurface surface;
    private Thread thread;
    private JPanel toolbarPanel;
    private JLabel label;
    private ToggleIcon closedIcon;

    protected boolean focus;

    public JButton toggleB;
    public JButton printB;
    public JComboBox screenCombo;
    public JButton aliasB;
    public JToolBar toolbar;


    public ToolBar(TestSurface surface) {
        this.surface = surface;
        setBackground(Color.gray);
        setLayout(new BorderLayout());

        closedIcon = new ToggleIcon(this, Color.lightGray);
        toggleB = new JButton(closedIcon);
        toggleB.addMouseListener(this);
        toggleB.setSelected(false);
        toggleB.addActionListener(this);
        toggleB.setMargin(new Insets(0,0,-4,0));
        toggleB.setBorderPainted(false);
        toggleB.setFocusPainted(false);
        toggleB.setContentAreaFilled(false);
        toggleB.setRolloverIcon(new ToggleIcon(this, roColor));
        add("North", toggleB);


        toolbar = new JToolBar();
        toolbar.setFloatable(false);

        String s = surface.AntiAlias == RenderingHints.VALUE_ANTIALIAS_ON 
            ? "On" : "Off";
        aliasB = addTool( "A", "Antialiasing " + s, this);

        printB = addTool("print.gif", "Print the Surface", this);

        screenCombo = new JComboBox();
        screenCombo.setLightWeightPopupEnabled(true);
        screenCombo.setPreferredSize(new Dimension(100, 18));
        screenCombo.setFont(font);
        for (int i = 0; i < ControlsInterface.screenNames.length; i++) {
            screenCombo.addItem(ControlsInterface.screenNames[i]);
        } 
        screenCombo.addActionListener(this);
        screenCombo.setSelectedIndex(ControlsInterface.getScreenIndex());

        toolbarPanel = new JPanel(new FlowLayout(FlowLayout.CENTER,5,0));
        toolbarPanel.setBackground(Color.gray);
        toolbarPanel.setLocation(0,6);
        toolbarPanel.setVisible(false);
        toolbarPanel.add(toolbar);
        toolbarPanel.add(screenCombo);
        add(toolbarPanel);

        setPreferredSize(new Dimension(200,6));
    }


    public JButton addTool(String str, 
                           String toolTip,
                           ActionListener al) {
        JButton b = null;
        if (str.indexOf(".") == -1) {
            b = (JButton) toolbar.add(new JButton(str));
            if (toolTip.equals("Rendering Quality") ||
                    toolTip.equals("Antialiasing On") ||
                        toolTip.equals("Texture On")  ||
                            toolTip.equals("Composite On")) {
                b.setBackground(Color.green);
                b.setSelected(true);
            } else {
                b.setBackground(Color.lightGray);
                b.setSelected(false);
            }
        } else {
            Image img = TestImages.getImage(str, this);
            b = (JButton) toolbar.add(new JButton(new ImageIcon(img)));
            b.setSelected(true);
        }
        b.setToolTipText(toolTip);
        b.addActionListener(al);
        return b;
    }


    public void actionPerformed(ActionEvent e) {
        Object obj = e.getSource();
        if (obj.equals(toggleB)) {
            toggleB.setSelected(!toggleB.isSelected());
            if (toggleB.isSelected()) {
                setPreferredSize(new Dimension(200,38));
            } else {
                setPreferredSize(new Dimension(200,6));
            }
            toolbarPanel.setVisible(toggleB.isSelected());
            getParent().validate();
            toggleB.getModel().setRollover(false);
            return;
        }
        if (obj.equals(printB)) {
            start();
            return;
        }
        if (obj instanceof JButton) {
            JButton b = (JButton) obj;
            b.setSelected(!b.isSelected());
            if (b.getIcon() == null) {
                b.setBackground(b.isSelected() ? Color.green : Color.lightGray);
            }
        }
        if (obj.equals(aliasB)) {
            if (aliasB.getToolTipText().equals("Antialiasing On")) {
                aliasB.setToolTipText("Antialiasing Off");
            } else {
                aliasB.setToolTipText("Antialiasing On");
            }
            surface.setAntiAlias(aliasB.isSelected());
        } else if (obj.equals(screenCombo)) {
            surface.setImageType(screenCombo.getSelectedIndex());
        }

        surface.repaint();
    }


    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
        focus = true;
        closedIcon.start();
    }

    public void mouseExited(MouseEvent e) {
        focus = false;
        closedIcon.stop();
    }


    public void start() {
        thread = new Thread(this);
        thread.setPriority(Thread.MAX_PRIORITY);
        thread.setName("Printing " + surface.name);
        thread.start();
    }


    public synchronized void stop() {
        thread = null;
        notifyAll();
    }


    public void run() {
        PrinterJob printJob = PrinterJob.getPrinterJob();
        printJob.setPrintable(surface);
        try {
            printJob.print();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        thread = null;
    }


    static class ToggleIcon implements Icon, Runnable {

        private Color topColor = new Color(153, 153, 204);
        private Color shadowColor = new Color(102, 102, 153);
        private Color backColor = new Color(204, 204, 255);
        private ToolBar toolbar;
        private Thread thread;
        private Color fillColor;


        public ToggleIcon(ToolBar toolbar, Color fillColor) {
            this.toolbar = toolbar;
            this.fillColor = fillColor;
        }

    
        public void paintIcon(Component c, Graphics g, int x, int y ) {
	    int w = getIconWidth();
	    int h = getIconHeight();
	    g.setColor(fillColor);
	    g.fillRect(0, 0, w, h);
            for (; x < w-2; x+=4) {
                g.setColor(Color.white);
                g.fillRect(x, 1, 1, 1);
                g.fillRect(x+2, 3, 1, 1);
                g.setColor(shadowColor);
                g.fillRect(x+1, 2, 1, 1);
                g.fillRect(x+3, 4, 1, 1);
            }
        }

        public int getIconWidth() {
            return toolbar.getSize().width;
        }

        public int getIconHeight() {
            return 6;
        }


        public void start() {
            thread = new Thread(this);
            thread.setPriority(Thread.MIN_PRIORITY);
            thread.setName("ToggleIcon");
            thread.start();
        }


        public synchronized void stop() {
            if (thread != null) {
                thread.interrupt();
            }
            thread = null;
        }


        public void run() {
            try {
               thread.sleep(400);
            } catch (InterruptedException e) { }
            if (toolbar.focus && thread != null) {
                toolbar.toggleB.doClick();
            }
            thread = null;
        }
    }
} // End ToolBar class
