/*
 * @(#)RulesBI.java	1.6 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * @type FUNCTIONAL
 * @test @(#)RulesBI.java	1.6    00/02/02
 * @summary - All the AlphaCompositing Rules.
 */


package tests.AlphaComposite;

import java.awt.*;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import TestSurface;


public class RulesBI extends TestSurface {

    private static AlphaComposite compObjs[] = {
        AlphaComposite.Src,
        AlphaComposite.Src,
        AlphaComposite.SrcOver,
        AlphaComposite.SrcIn,
        AlphaComposite.SrcOut,
        AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.0f),
        AlphaComposite.Clear,
        AlphaComposite.DstOver,
        AlphaComposite.DstIn,
        AlphaComposite.DstOut
    };


    public void render(int w, int h, Graphics2D g2) {

        int PADLEFT  = 15;
        int PADRIGHT = 15;
        int HPAD     = (PADLEFT + PADRIGHT);
        int PADABOVE = 2 + 12;
        int PADBELOW = 2;
        int VPAD     = (PADABOVE + PADBELOW);
        int RECTWIDTH = w/2 - HPAD;
        RECTWIDTH = (RECTWIDTH < 6) ? 6 : RECTWIDTH;
        int RECTHEIGHT = (h-VPAD)/5 - VPAD + 2;
        RECTHEIGHT = (RECTHEIGHT < 6) ? 6 : RECTHEIGHT;
        int PADDEDWIDTH  = (RECTWIDTH  + HPAD);
        int PADDEDHEIGHT = (RECTHEIGHT + VPAD);

        GeneralPath srcpath = new GeneralPath();
        srcpath.moveTo(0,0);
        srcpath.lineTo(RECTWIDTH,0);
        srcpath.lineTo(RECTWIDTH,RECTHEIGHT/2);
        srcpath.lineTo(0,RECTHEIGHT);
        srcpath.closePath();

        GeneralPath dstpath = new GeneralPath();
        dstpath.moveTo(0,0);
        dstpath.lineTo(0,RECTHEIGHT/2);
        dstpath.lineTo(RECTWIDTH,RECTHEIGHT);
        dstpath.lineTo(RECTWIDTH,0);
        dstpath.closePath();

        BufferedImage dBI = new BufferedImage(RECTWIDTH, RECTHEIGHT,
                                    BufferedImage.TYPE_INT_ARGB);
        BufferedImage sBI = new BufferedImage(RECTWIDTH, RECTHEIGHT,
                                    BufferedImage.TYPE_INT_ARGB);

        Graphics2D gD = dBI.createGraphics();
        gD.setRenderingHint(RenderingHints.KEY_ANTIALIASING, AntiAlias);
        Graphics2D gS = sBI.createGraphics();
        gS.setRenderingHint(RenderingHints.KEY_ANTIALIASING, AntiAlias);

        int x = 0, y = 0;

        for (int i = 0; i < compObjs.length; i++) {
            y = (i == 0 || i == 5) ? VPAD+2 : y + PADDEDHEIGHT;
            x = (i >= 5) ? w/2+PADLEFT : PADLEFT;
            g2.translate(x, y);

            gD.setComposite(AlphaComposite.Clear);
            gD.fillRect(0, 0, RECTWIDTH, RECTHEIGHT);
            gD.setComposite(AlphaComposite.Src);
            Paint g =
                    new GradientPaint(0, 0,
                                      new Color(1.0f, 0.0f, 0.0f, 1.0f),
                                      0, RECTHEIGHT,
                                      new Color(1.0f, 0.0f, 0.0f, 0.0f));
            gD.setPaint(g);
            gD.fillRect(0, 0, RECTWIDTH, RECTHEIGHT);

            gS.setComposite(AlphaComposite.Clear);
            gS.fillRect(0, 0, RECTWIDTH, RECTHEIGHT);
            gS.setComposite(AlphaComposite.Src);
            g = new GradientPaint(0, 0,
                                      new Color(0.0f, 0.0f, 1.0f, 1.0f),
                                      RECTWIDTH, 0,
                                      new Color(0.0f, 0.0f, 1.0f, 0.0f));
            gS.setPaint(g);
            gS.fillRect(0, 0, RECTWIDTH, RECTHEIGHT);

            gD.setComposite(compObjs[i]);
            gD.drawImage(sBI, 0, 0, null);

            g2.drawImage(dBI, 0, 0, null);
            g2.setColor(Color.black);
            g2.drawRect(0, 0, RECTWIDTH, RECTHEIGHT);
            g2.translate(-x, -y);
        }

        gD.dispose();
        gS.dispose();
    }



    public static void main(String s[]) {
        createFrame("RulesBI", new RulesBI());
    }
} 
