/*
 * @(#)NegativeTransform.java	1.6 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * @type REGRESSION
 * @test @(#)NegativeTransform.java	1.6    00/02/02
 * @bug 4220924 Clipping in translated Image Graphics isn't correct
 * @summary - 
 * This test exposes a bug in JDK 1.2.  If you get a Graphics from an
 * Image, translate the Graphics and then set clipping on the Graphics,
 * the wrong area of the graphics gets clipped.
 *
 * The strategy used in this test is to translate a Graphics by (-dx, -dy),
 * set a clipping region at (dx, dy), and then draw something at (dx, dy).
 * If the output changes with changes in dx or dy, there is a bug.
 */

package tests.Clipping;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.geom.Rectangle2D;
import TestSurface;


public class NegativeTransform extends TestSurface {

    private int squareSize;

    /**
     * This method draws a red square inside of a blue square.
     * The topleft corner of the blue square is at (0, 0).  In
     * a correctly functioning Graphics, dx and dy should not affect
     * the result of this method at all.
     */
    private void drawIt(Graphics2D g, int dx, int dy) {

        g.translate(-dx, -dy);

        g.setColor(Color.blue);
        g.fillRect(dx, dy, squareSize, squareSize);

        g.setColor(Color.red);

        // This clipping should not affect the appearance of
        // the image, but it does if dx and dy are nonzero.
        g.setClip(new Rectangle2D.Float(dx, dy, squareSize, squareSize));

        g.fillRect(dx, dy, squareSize, squareSize);
    }

    /**
     * Return an Image with the squares draw in it.  (dx, dy) is the
     * amount to translate by - their values should not affect the
     * outcome.
     */
    private BufferedImage makeImage(int dx, int dy) {
        BufferedImage bi = new BufferedImage(squareSize, squareSize, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = (Graphics2D) bi.createGraphics();
        drawIt(g, dx, dy);
        return bi;
    }

    public void render(int w, int h, Graphics2D g2) {

        squareSize = Math.min(w, h)/2 - 20;

        BufferedImage buffer = makeImage(0, 0);
        g2.drawImage(buffer, 10, 10, this);

        buffer = makeImage(60, 40);
        g2.drawImage(buffer, 30 + squareSize, 10, this);
    }


    public static void main(String s[]) {
        createFrame("NegativeTransform", new NegativeTransform());
    }
} 
