/*
 * @(#)ICC_Profiles.java	1.5 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * @type FUNCTIONAL
 * @test @(#)ICC_Profiles.java	1.5    00/02/02
 * @summary - ColorConvertOp with ICC_Profile's on a BufferedImage of a 
 * image, text and colored rectangles.
 */


package tests.Color;

import java.awt.*;
import java.awt.color.ICC_Profile;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.io.IOException;
import java.net.URL;
import TestSurface; 


public class ICC_Profiles extends TestSurface {

    public void render(int w, int h, Graphics2D g2) {
        Image img = getImage("clouds.jpg");
        int iw = img.getWidth(this);
        int ih = img.getHeight(this);

        BufferedImage srcImg = new BufferedImage(iw, ih, BufferedImage.TYPE_INT_RGB);
        Graphics2D srcG = srcImg.createGraphics();
        srcG.setRenderingHints(g2.getRenderingHints());
        srcG.drawImage(img, 0, 0, this);

        Color colors[] = { Color.red, Color.pink, Color.orange, Color.yellow, 
                   Color.green, Color.magenta, Color.cyan, Color.blue,
                   Color.lightGray, Color.gray, Color.darkGray };
        int rw = iw/colors.length;
        int rh = ih/4;
        for (int i = 0; i < colors.length; i++) {
            srcG.setColor(colors[i%colors.length]);
            srcG.fillRect(i*rw, ih-rh, rw, rh);
            srcG.setColor(colors[colors.length-1-i%colors.length]);
            srcG.fillRect(i*rw, 0, rw, rh);
        }


        BufferedImage dstImg;
        try {
            URL url = ICC_Profiles.class.getResource("src.pf");
            ICC_Profile pf1 = ICC_Profile.getInstance(url.getFile());
            url = ICC_Profiles.class.getResource("dest.pf");
            ICC_Profile pf2 = ICC_Profile.getInstance(url.getFile());
            ICC_Profile pfList[] = new ICC_Profile[2];
            pfList[0] = pf1;
            pfList[1] = pf2;
            ColorConvertOp theOp = new ColorConvertOp(pfList, null);

            dstImg = new BufferedImage(iw, ih, BufferedImage.TYPE_INT_RGB);
            theOp.filter(srcImg, dstImg);
        } catch (IOException e) {
            dstImg = null;
        }

        g2.drawImage(srcImg, 10, 10, w/2-20, h-20, this);

        if (dstImg != null) {
            g2.drawImage(dstImg, w/2+10, 10, w/2-20, h-20, this);
        }
    }


    public static void main(String s[]) {
        createFrame("ICC_Profiles", new ICC_Profiles());
    }
}
