/*
 * @(#)Highlighting.java	1.5 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * @type FUNCTIONAL
 * @test @(#)Highlighting.java	1.5    00/02/02
 * @summary - 
 * Highlighting of text showing the caret, the highlight & the character
 * advances.
 */

package tests.Font;

import java.awt.*;
import java.awt.font.TextLayout;
import java.awt.font.TextHitInfo;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import TestSurface;


public class Highlighting extends TestSurface {

    private TextLayout[] layouts;
    private Rectangle2D[]  rects;
    private Font smallF = new Font("serif", Font.PLAIN, 8);
    private FontRenderContext frc;
    private Color colors[] = { Color.red, Color.pink, Color.orange, 
            Color.yellow, Color.green, Color.magenta, Color.cyan, Color.blue};


    public Highlighting() {
        layouts = new TextLayout[3];
        rects = new Rectangle2D[3];
    }


    public void drawTextField(Graphics2D g2, TextLayout tl, Rectangle2D r, String str) {
        float rx = (float) r.getX();
        float ry = (float) r.getY();
        float rw = (float) r.getWidth();
        float rh = (float) r.getHeight();

        AffineTransform at = new AffineTransform();
        at.setToTranslation(rx, ry);

        // draw highlighted shape
        Shape hilite = null;
        for (int i = str.length(); i > 0; i--) {
            hilite = tl.getLogicalHighlightShape(0, i);
            hilite = at.createTransformedShape(hilite);
            g2.setColor(colors[i%colors.length]);
            g2.fill(hilite);
        }

        float hx = (float) hilite.getBounds().getX();
        float hy = (float) hilite.getBounds().getY();
        float hh = (float) hilite.getBounds().getHeight();

        g2.setColor(Color.black);
        tl.draw(g2, rx, ry);
        g2.draw(new Rectangle2D.Float(hx,hy,rw,hh));

        // Display character advances.
        for (TextHitInfo hit = TextHitInfo.trailing(-1); hit != null; hit = tl.getNextRightHit(hit)) {

            g2.draw(at.createTransformedShape(tl.getCaretShape(hit)));
            float[] cInfo = tl.getCaretInfo(hit);
            String s = String.valueOf((int) cInfo[0]);
            TextLayout t = new TextLayout(s,smallF,frc);
            t.draw(g2, (float) rx+cInfo[0], hy+hh+t.getAscent()+1.0f);
        }
    }


    public void render(int w, int h, Graphics2D g2) {

        String text[] = { "ABCDEFGHI", "JKLMNOPQR", "STUVWXYZ" };
        Font f = new Font("serif",Font.PLAIN,w/text[0].length()+4);
        frc = g2.getFontRenderContext();
        for (int i = 0; i < layouts.length; i++ ) {
            layouts[i]  = new TextLayout(text[i], f, frc);
            float rx = (float) (w/2-layouts[i].getBounds().getWidth()/2);
            float ry = (float) (h * ((i+1) * 0.28f));
            float rw = (float) (layouts[i].getBounds().getWidth());
            float rh = (float) (layouts[i].getBounds().getHeight());
            rects[i] = new Rectangle2D.Float(rx, ry, rw, rh);
            drawTextField(g2, layouts[i], rects[i], text[i]);
        }
    }


    public static void main(String argv[]) {
        createFrame("Highlighting", new Highlighting());
    }
}
