/*
 * @(#)Convolve.java	1.5 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * @type FUNCTIONAL
 * @test @(#)Convolve.java	1.5    00/02/02
 * @summary - Images drawn using operators such as ConvolveOp 
 * LowPass & Sharpen, LookupOp and RescaleOp.
 */

package tests.Image;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.AffineTransform;
import TestSurface;


public class Convolve extends TestSurface {

    private BufferedImage bi[];


    public Convolve() {
        bi = new BufferedImage[4];
        String s[] = { "bld.jpg", "boat.gif", "bld.jpg", "boat.gif" };
        for (int i = 0; i < bi.length; i++) {
            Image img = getImage(s[i]);
            int iw = img.getWidth(this);
            int ih = img.getHeight(this);
            bi[i] = new BufferedImage(iw, ih, BufferedImage.TYPE_INT_RGB);
            Graphics2D big = bi[i].createGraphics();
            big.drawImage(img,0,0,this);
        }
    }


    public void render(int w, int h, Graphics2D g2) {

        float[][] data = {{0.1f, 0.1f, 0.1f,    // low-pass
                           0.1f, 0.2f, 0.1f,
                           0.1f, 0.1f, 0.1f},
                          {0.0625f, 0.125f, 0.0625f,  // low-pass
                           0.125f, 0.25f, 0.125f,
                           0.0625f, 0.125f, 0.0625f},
                          {-1.f, -1.f, -1.f,    // high-pass
                           -1.f,  9.f, -1.f,
                           -1.f, -1.f, -1.f},
                          {1.f, -2.f,  1.f,     // high-pass
                          -2.f,  5.f, -2.f,
                           1.f, -2.f,  1.f}};


        String theDesc[] = { "LowPass", "LowPass", 
                             "HighPass", "HighPass" };
        for (int i = 0; i < bi.length; i++) {
            int iw = bi[i].getWidth(this);
            int ih = bi[i].getHeight(this);
            int x = 0, y = 0;

            switch (i) {
                case 0 : x = 5; y = 15; break;
                case 1 : x = w/2+3; y = 15; break;
                case 2 : x = 5; y = h/2+15; break;
                case 3 : x = w/2+3; y = h/2+15;
            }

            AffineTransform at = new AffineTransform();
            at.translate(x, y);
            at.scale((w-14)/2.0/iw, (h-34)/2.0/ih);

            BufferedImage bimg = new BufferedImage(iw,ih,BufferedImage.TYPE_INT_RGB);
            Kernel kernel = new Kernel(3,3,data[i]);
            ConvolveOp cop = new ConvolveOp(kernel,
                                             ConvolveOp.EDGE_NO_OP, null);
            cop.filter(bi[i],bimg);
            BufferedImageOp biop = new AffineTransformOp(at,
                                     AffineTransformOp.TYPE_NEAREST_NEIGHBOR);

            g2.drawImage(bimg,biop,0,0); 
        }
    }


    public static void main(String s[]) {
        createFrame("Convolve", new Convolve());
    }
}
