/*
 * @(#)Filters.java	1.5 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*
 * @type FUNCTIONAL
 * @test @(#)Filters.java	1.5    00/02/02
 * @summary - 
 */

package tests.Image;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import TestSurface;


public class Filters extends TestSurface {

    String filters[] = { "Flip", "Gradient", "Alpha", "RedBlueSwap" };
    String imgName[] = { "box", "fight", "thumbsup", "dukeplug"};
    Image imgs[];
    ImageObserver imgObs[];

    public Filters() {
        imgs = new Image[4];
        imgObs = new ImageObs[4];
        for (int i = 0; i < imgs.length; i++) {
            imgs[i] = getImage(imgName[i] + ".gif");
            imgObs[i] = new ImageObs(this);
            ImageProducer ip = imgs[i].getSource();
            switch (i) {
                case 0 : ip = new FilteredImageSource(ip, new FlipFilter());
                         break;
                case 1 : ip = new FilteredImageSource(ip, new GradientFilter());
                         break;
                case 2 : ip = new FilteredImageSource(ip, new AlphaFilter());
                         break;
                case 3 : ip = new FilteredImageSource(ip, new RedBlueSwapFilter());
            }
            imgs[i] = getToolkit().createImage(ip);
            try {
                MediaTracker tracker = new MediaTracker(this);
                tracker.addImage(imgs[i], 0);
                tracker.waitForID(0);
            } catch (Exception e) { e.printStackTrace(); }
        }
    }


    public void render(int w, int h, Graphics2D g2) {
        for (int i = 0; i < imgs.length; i++) {
            int iw = imgs[i].getWidth(this);
            int ih = imgs[i].getHeight(this);
            int x = 0, y = 0;
            switch (i) {
                case 0 : x = 5; y = 15; break;
                case 1 : x = w/2+3; y = 15; break;
                case 2 : x = 5; y = h/2+15; break;
                case 3 : x = w/2+3; y = h/2+15;
            }
            AffineTransform at = new AffineTransform();
            at.translate(x , y);
            at.scale((w-14)/2.0/iw, (h-34)/2.0/ih);
            g2.drawImage(imgs[i], at, this);
        }
    }

    public static void main(String s[]) {
        createFrame("Filters", new Filters());
    }



class ImageObs implements ImageObserver {
    Component c;

    public ImageObs(Component c) {
        this.c = c;
    }
    
    public boolean imageUpdate(Image img, int infoflags,
                int x, int y, int width, int height) 
    {
        if ((infoflags & ALLBITS) != 0) 
            c.repaint(x, y, width, height);
        return c.isShowing();
    }
}


class FlipFilter extends ImageFilter {
    int width, height;
    boolean flipH = true, flipV = true;

    public void setDimensions(int w, int h) {
        super.setDimensions(width = w, height = h);
    }

    public void setPixels(int srcX, int srcY, int srcW, int srcH,
          ColorModel model, byte pixels[], int srcOff, int srcScan) {
        int s = srcOff;
        byte[] tempBuff = new byte[srcW * srcH];

        for (int y=0; y<srcH; y++) {
            for (int x=0; x<srcW; x++) {
                int d = 0;
                if (flipV) {
                    d += (srcH-y-1)*srcW;
                } else {
                    d += y*srcW;
                }
                if (flipH) {
                    d += (srcW-x-1);
                } else {
                    d += x;
                }
                tempBuff[d] = pixels[s];
                s++;
            }
        }
        
        if (flipH) {
            srcX = width - (srcX+srcW);
        }
        if (flipV) {
            srcY = height - (srcY+srcH);
        }
        super.setPixels(srcX, srcY, srcW, srcH, model, tempBuff, 0, srcScan);
    }
}


class GradientFilter extends RGBImageFilter {
    float[] hsb = new float[3];
    int width, height;

    public void setDimensions(int w, int h) {
        super.setDimensions(w, h);
        width = w;
        height = h;
    }

    public int filterRGB(int x, int y, int rgb) {
        Color c = new Color(rgb);
        Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsb);
        hsb[2] += .5f - (float)x / width;
        hsb[2] += .5f - (float)y / height;
        hsb[2] = Math.max(0.0f, Math.min(1.0f, hsb[2]));
        return Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]);
    }
}

class RedBlueSwapFilter extends RGBImageFilter {
    public RedBlueSwapFilter() {
        canFilterIndexColorModel = true;
    }

    public int filterRGB(int x, int y, int rgb) {
        return ((rgb & 0xff00ff00)
                | ((rgb & 0xff0000) >> 16)
                | ((rgb & 0xff) << 16));
    }
}

class AlphaFilter extends ImageFilter {
    double afactor = 0.5;
    ColorModel transColorModel;

    void setAlphaFilter(double afactor) {
        this.afactor = afactor;
    }

    public void setColorModel(ColorModel model) {
        if (model instanceof IndexColorModel) {
            IndexColorModel icm = (IndexColorModel)model;
            byte[] reds = new byte[icm.getMapSize()];
            byte[] greens = new byte[icm.getMapSize()];
            byte[] blues = new byte[icm.getMapSize()];
            byte[] alphas = new byte[icm.getMapSize()];
            byte[] cmap = new byte[icm.getMapSize() * 4];

            icm.getReds(reds);
            icm.getGreens(greens);
            icm.getBlues(blues);
            icm.getAlphas(alphas);

            int j = 0;
            for (int i=0; i<icm.getMapSize(); i++) {
                cmap[j++] = reds[i];
                cmap[j++] = greens[i];
                cmap[j++] = blues[i];
                cmap[j++] = (byte)((alphas[i]&0xff)*afactor);
            }
            transColorModel = new IndexColorModel(icm.getPixelSize(), 
                icm.getMapSize(), cmap, 0, true, icm.getTransparentPixel());
        } else {
            transColorModel = model;
        }
        consumer.setColorModel(transColorModel);
    }

    public void setPixels(int x, int y, int w, int h,
          ColorModel model, byte pixels[], int off, int scansize) {
        consumer.setPixels(x, y, w, h, transColorModel, pixels, off, scansize);
    }

    public void setPixels(int x, int y, int w, int h,
          ColorModel model, int pixels[], int off, int scansize) {
        consumer.setPixels(x, y, w, h, transColorModel, pixels, off, scansize);
    }
}

    public class imgObs implements ImageObserver 
    {
        Component c;

        public imgObs(Component c) {
            this.c = c;
        }
    
        public boolean imageUpdate(Image img, int infoflags,
                int x, int y, int width, int height) 
        {
            if ((infoflags & ALLBITS) != 0) {
//              System.out.println("ALLBITS" + width + " " + height);
                c.repaint(x, y, width, height);
            }
            return c.isShowing();
        }
    }

}
