/*
 * @(#)ShearTest.java	1.3 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/**
 * @test 1.2 99/02/01
 * @bug 4171820
 * @summary Checks that GradientPaint responds to shearing transforms correctly
 *          The gradients drawn should be parallel to the sides of the
 *          indicated anchor rectangle.
 * @author flar
 */

package tests.Paint;

import java.awt.*;
import java.awt.geom.AffineTransform;
import TestSurface;

public class ShearTest extends TestSurface {

    public static final int GRADW = 30;
    public static final Color faintblue = new Color(0f, 0f, 1.0f, 0.35f);
    public static final Rectangle anchor =
        new Rectangle(-GRADW / 2, -GRADW / 2, GRADW, GRADW);


    public void render(int w, int h, Graphics2D g2) {
	String label = "Slanted";
	boolean cyclic = false; 
        boolean horizontal = false, shearx = true, sheary = false;
        AffineTransform txform = new AffineTransform();
        if (shearx) {
	    txform.shear(-.5, 0);
        }
        if (sheary) {
	    txform.shear(0, -.5);
        }
        int relx = horizontal ? 0 : GRADW / 2;
        int rely = horizontal ? GRADW / 2 : 0;
        GradientPaint grad = new GradientPaint(-relx, -rely, Color.green,
			     relx, rely, Color.yellow, cyclic);

        AffineTransform at = g2.getTransform();
        g2.translate(75, 75);
        g2.transform(txform);
        g2.setPaint(grad);
        g2.fill(g2.getClip());
        g2.setColor(faintblue);
        g2.fill(anchor);
        g2.setTransform(at);

        g2.setColor(Color.black);
        g2.drawRect(0, 0, w-1, h-1);
        g2.drawString(label, 5, h-5);
    }

    public static void main(String s[]) {
        createFrame("ShearTest", new ShearTest());
    }
}
