/*
 * @(#)TextureAnim.java	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
    
import java.awt.Component;
import java.awt.Label;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Canvas;
import java.awt.Panel;
import java.awt.Frame;
import java.awt.FileDialog;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Paint;
import java.awt.Image;
import java.awt.TexturePaint;
import java.awt.GradientPaint;
import java.awt.RenderingHints;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.ImageObserver;
import java.awt.image.BufferedImage;
import java.awt.image.PixelGrabber;
import java.awt.image.ColorModel;
import java.awt.image.WritableRaster;

public class TextureAnim extends Canvas implements Runnable {
    public static final int TEXTURE = 0;
    public static final int GRADIENT = 1;
    public static final int SOLIDFILL = 2;
    public static final int IMAGE = 3;

    BufferedImage offscr;

    Rectangle tilerect;
    Paint tilepaint;
    Dimension tilesize;

    BufferedImage textureimg;
    BufferedImage loadedimg;

    Thread animator;

    boolean bouncesize = false;
    boolean bouncerect = true;
    boolean rotate = false;
    boolean shearx = false;
    boolean sheary = false;
    boolean drawoval = false;
    boolean showanchor = true;
    boolean quality = false;
    boolean dogray = false;
    int texturesize = 100;
    int texturetype = TEXTURE;

    public TextureAnim() {
    }

    public Dimension getPreferredSize() {
	return new Dimension(300, 300);
    }

    public void start() {
	Thread t = new Thread(this);
	t.setPriority(Thread.MIN_PRIORITY);
	animator = t;
	t.start();
    }

    public void stop() {
	animator = null;
    }

    public synchronized void setSize(int size) {
	if (texturesize != size) {
	    makeImage(size);
	    texturesize = size;
	}
	setType(TEXTURE);
    }

    public synchronized void setType(int type) {
	switch (type) {
	case IMAGE:
	    if (loadedimg == null) {
		if (!fetchImage()) {
		    return;
		}
	    }
	    tilepaint = new TexturePaint(loadedimg, tilerect);
	    tilesize = new Dimension(loadedimg.getWidth(),
				     loadedimg.getHeight());
	    break;
	case TEXTURE:
	    if (textureimg == null) {
		makeImage(texturesize);
	    }
	    tilepaint = new TexturePaint(textureimg, tilerect);
	    tilesize = new Dimension(texturesize, texturesize);
	    break;
	case GRADIENT:
	    tilepaint = makeGradient(tilerect);
	    tilesize = new Dimension(texturesize, texturesize);
	    break;
	case SOLIDFILL:
	    tilepaint = Color.black;
	    tilesize = new Dimension(texturesize, texturesize);
	    break;
	}
	texturetype = type;
    }

    public synchronized void doGray(boolean dogray) {
	if (this.dogray == dogray) {
	    return;
	}
	this.dogray = dogray;
	switch (texturetype) {
	case TEXTURE:
	    makeImage(texturesize);
	    tilepaint = new TexturePaint(textureimg, tilerect);
	    break;
	case GRADIENT:
	    tilepaint = makeGradient(tilerect);
	    break;
	}
    }

    public Paint makeGradient(Rectangle r) {
	Color c1 = (dogray ? Color.white : Color.red);
	Color c2 = (dogray ? Color.black : Color.green);
	return new GradientPaint(r.x, r.y+r.height/2, c1,
				 r.x+r.width, r.y+r.height/2, c2,
				 true);
    }

    public synchronized void makeImage(int size) {
	int type;
	textureimg = null;
	if (dogray) {
	    type = BufferedImage.TYPE_BYTE_GRAY;
	} else {
	    textureimg = (BufferedImage) createImage(size, size);
	    if (textureimg == null) {
		// Wait until we are mapped to create the image of the
		// appropriate type...
		return;
	    }
	    type = textureimg.getType();
	    if (type != BufferedImage.TYPE_INT_RGB &&
		type != BufferedImage.TYPE_INT_BGR)
	    {
		type = BufferedImage.TYPE_INT_BGR;
		textureimg = null;
	    }
	}
	if (textureimg == null) {
	    textureimg = new BufferedImage(size, size, type);
	}

	Graphics2D g2d = textureimg.createGraphics();
	g2d.setColor(Color.white);
	g2d.fillRect(0, 0, size, size);
	for (int j = 0; j < size; j++) {
	    float red = j / (float) size;
	    for (int i = 0; i < size; i++) {
		float green = i / (float) size;
		g2d.setColor(new Color(1.0f - red, 1.0f - green, 0.0f, 1.0f));
		g2d.fillRect(i, j, 1, 1);
	    }
	}
    }

    public Frame getFrame() {
	Component c = this;
	while (c != null && !(c instanceof Frame)) {
	    c = c.getParent();
	}
	return (Frame) c;
    }

    static FileDialog fd;

    public boolean fetchImage() {
	if (fd == null) {
	    fd = new FileDialog(getFrame(), "Image file", FileDialog.LOAD);
	}
	fd.show();
	return fetchImage(fd.getDirectory()+fd.getFile());
    }

    public boolean fetchImage(String arg) {
	Image src = getToolkit().getImage(arg);
	PixelGrabber pg = new PixelGrabber(src, 0, 0, -1, -1, false);
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    return false;
	}
	if ((pg.getStatus() & (ImageObserver.ERROR|ImageObserver.ABORT)) != 0) {
	    return false;
	}
	ColorModel cm = pg.getColorModel();
	Object pixels = pg.getPixels();
	int w = pg.getWidth();
	int h = pg.getHeight();
	WritableRaster wr = cm.createCompatibleWritableRaster(w, h);
	wr.setDataElements(0, 0, w, h, pixels);
	boolean isAlphaPremul = cm.isAlphaPremultiplied();
	loadedimg = new BufferedImage(cm, wr, isAlphaPremul, null);
	return true;
    }

    public void run() {
	Thread me = Thread.currentThread();
	if (textureimg == null) {
	    makeImage(100);
	}
	int tilewidth = textureimg.getWidth();
	int tileheight = textureimg.getHeight();
	Dimension dims = getSize();
	AnimVal w = new AnimVal(0, Math.max(200, tilewidth),
				3, 10, tilewidth);
	AnimVal h = new AnimVal(0, Math.max(200, tileheight),
				3, 10, tileheight);
	AnimVal x = new AnimVal(-dims.width/4, dims.width/4 - w.getInt(),
				3, 10, 0);
	AnimVal y = new AnimVal(-dims.height/4, dims.height/4 - h.getInt(),
				3, 10, 0);
	AnimVal rot = new AnimVal(-360, 360, 5, 15, 0);
	AnimVal shx = new AnimVal(-50, 50, 3, 10, 0);
	AnimVal shy = new AnimVal(-50, 50, 3, 10, 0);
	tilerect = new Rectangle(x.getInt(), y.getInt(),
				 w.getInt(), h.getInt());
	if (tilepaint == null) {
	    setType(texturetype);
	}
	Dimension odims = dims;
	Graphics2D gi = offscr == null ? null : offscr.createGraphics();
	AffineTransform at = null;
	if (gi != null) {
	    gi.translate(dims.width / 2, dims.height / 2);
	    at = gi.getTransform();
	}
	double framerate = 0;
	long start = System.currentTimeMillis();
	int numframes = 0;
	while (animator == me) {
	    dims = getSize();
	    if (offscr == null ||
		dims.width != offscr.getWidth() ||
		dims.height != offscr.getHeight())
	    {
		if (gi != null) {
		    gi.dispose();
		    gi = null;
		}
		offscr = (BufferedImage) createImage(dims.width, dims.height);
		x.newlimits(-dims.width/4, dims.width/4 - w.getInt());
		y.newlimits(-dims.height/4, dims.height/4 - h.getInt());
		gi = offscr.createGraphics();
		gi.translate(dims.width / 2, dims.height / 2);
		at = gi.getTransform();
	    }
	    tilewidth = textureimg.getWidth();
	    tileheight = textureimg.getHeight();
	    if (bouncesize) {
		w.anim();
		h.anim();
		x.newlimits(-dims.width/4, dims.width/4 - w.getInt());
		y.newlimits(-dims.height/4, dims.height/4 - h.getInt());
	    } else {
		if (w.getInt() != tilesize.width) {
		    w.set(tilesize.width);
		    x.newlimits(-dims.width/4, dims.width/4 - w.getInt());
		}
		if (h.getInt() != tilesize.height) {
		    h.set(tilesize.height);
		    y.newlimits(-dims.height/4, dims.height/4 - h.getInt());
		}
	    }
	    if (bouncerect) {
		x.anim();
		y.anim();
	    }
	    int X = x.getInt();
	    int Y = y.getInt();
	    int W = w.getInt();
	    int H = h.getInt();
	    if (X != tilerect.x || Y != tilerect.y ||
		W != tilerect.width || H != tilerect.height)
	    {
		synchronized (this) {
		    tilerect.setRect(X, Y, W, H);
		    switch (texturetype) {
		    case IMAGE:
		    case TEXTURE:
			TexturePaint tp = (TexturePaint) tilepaint;
			tilepaint = new TexturePaint(tp.getImage(), tilerect);
			break;
		    case GRADIENT:
			GradientPaint gp = (GradientPaint) tilepaint;
			tilepaint = new GradientPaint(X, Y+H/2,
						      gp.getColor1(),
						      X+W, Y+H/2,
						      gp.getColor2(),
						      true);
			break;
		    }
		}
	    }
	    if (drawoval) {
		gi.setTransform(new AffineTransform());
		gi.setColor(Color.black);
		gi.setColor(Color.gray);
		gi.fillRect(0, 0, dims.width, dims.height);
	    }
	    gi.setTransform(at);
	    if (rotate) {
		rot.anim();
		gi.rotate(Math.toRadians(rot.getFlt()));
	    } else {
		rot.set(0);
	    }
	    if (shearx) {
		shx.anim();
		gi.shear(shx.getFlt()/100, 0.0f);
	    } else {
		shx.set(0);
	    }
	    if (sheary) {
		shy.anim();
		gi.shear(0.0f, shy.getFlt()/100);
	    } else {
		shy.set(0);
	    }
	    gi.setPaint(tilepaint);
	    gi.setRenderingHint(RenderingHints.KEY_RENDERING,
				(quality
				 ? RenderingHints.VALUE_RENDER_QUALITY
				 : RenderingHints.VALUE_RENDER_SPEED));
	    if (drawoval) {
		gi.fillOval(-dims.width/2, -dims.height/2,
			    dims.width, dims.height);
	    } else {
		gi.fillRect(-1000, -1000, 2000, 2000);
	    }
	    if (showanchor) {
		gi.setColor(Color.black);
		gi.setColor(new Color(0f, 0f, 1f, .5f));
		gi.fill(tilerect);
	    }
	    long now = System.currentTimeMillis();
	    numframes++;
	    if (now - start > 1000) {
		framerate = numframes / ((now - start) / 1000.0);
		numframes = 0;
		start = now;
	    }
	    Graphics g = getGraphics();
	    g.drawImage(offscr, 0, 0, null);
	    g.setColor(Color.white);
	    g.drawString((Math.rint(framerate * 10.0) / 10.0) + " fps",
			 5, dims.height - 5);
	    g.dispose();
	}
    }

    public void paint(Graphics g) {
	if (offscr != null) {
	    g.drawImage(offscr, 0, 0, null);
	}
    }

    public static void main(String argv[]) {
	Frame f = new Frame("TextureAnim");
	f.setLayout(new BorderLayout());
	final TextureAnim ta = new TextureAnim();
	if (argv.length > 0) {
	    if (ta.fetchImage(argv[0])) {
		ta.setType(IMAGE);
	    }
	}
	f.add("Center", ta);
	f.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	    public void windowIconified(WindowEvent e) {ta.stop();}
	    public void windowDeiconified(WindowEvent e) {ta.start();}
	});
	f.add("South", new AnimControls(ta));
	f.pack();
	f.show();
	ta.start();
    }

    static class AnimControls extends Panel {
	public static final int BOUNCERECT = 0;
	public static final int BOUNCESIZE = 1;
	public static final int ROTATE = 2;
	public static final int SHEARX = 3;
	public static final int SHEARY = 4;
	public static final int QUALITY = 6;
	public static final int OVAL = 7;
	public static final int ANCHOR = 8;
	public static final int GRAY = 9;

	static class AnimListener implements ItemListener {
	    TextureAnim textureanim;
	    int which;

	    public AnimListener(TextureAnim ta, int which) {
		this.textureanim = ta;
		this.which = which;
	    }

	    public void itemStateChanged(ItemEvent e) {
		boolean val = (e.getStateChange() == ItemEvent.SELECTED);
		switch (which) {
		case BOUNCERECT: textureanim.bouncerect = val; break;
		case BOUNCESIZE: textureanim.bouncesize = val; break;
		case ROTATE: textureanim.rotate = val; break;
		case SHEARX: textureanim.shearx = val; break;
		case SHEARY: textureanim.sheary = val; break;
		case QUALITY: textureanim.quality = val; break;
		case OVAL: textureanim.drawoval = val; break;
		case ANCHOR: textureanim.showanchor = val; break;
		case GRAY: textureanim.doGray(val); break;
		}
	    }
	}

	static class AnimChooser implements ItemListener {
	    TextureAnim textureanim;
	    Choice modechooser;

	    public AnimChooser(TextureAnim ta, Choice ch) {
		textureanim = ta;
		modechooser = ch;
	    }

	    public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
		    String str = (String) e.getItem();
		    int index = str.indexOf(' ');
		    int size = Integer.parseInt(str.substring(index+1));
		    textureanim.setSize(size);
		    textureanim.setType(TEXTURE);
		    modechooser.select(TEXTURE);
		}
	    }
	}

	static class ImageFetcher implements ActionListener {
	    TextureAnim textureanim;
	    Choice modechooser;

	    public ImageFetcher(TextureAnim ta, Choice ch) {
		textureanim = ta;
		modechooser = ch;
	    }

	    public void actionPerformed(ActionEvent e) {
		if (textureanim.fetchImage()) {
		    textureanim.setType(IMAGE);
		    modechooser.select(IMAGE);
		}
	    }
	}

	static class FillChooser implements ItemListener {
	    TextureAnim textureanim;

	    public FillChooser(TextureAnim ta) {
		textureanim = ta;
	    }

	    public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
		    Choice chooser = (Choice) e.getItemSelectable();
		    textureanim.setType(chooser.getSelectedIndex());
		}
	    }
	}

	public AnimControls(TextureAnim ta) {
	    Checkbox toggle;

	    setLayout(new GridLayout(0, 3));

	    toggle = new Checkbox("bounce", true);
	    toggle.addItemListener(new AnimListener(ta, BOUNCERECT));
	    add(toggle);
	    toggle = new Checkbox("resize", false);
	    toggle.addItemListener(new AnimListener(ta, BOUNCESIZE));
	    add(toggle);
	    toggle = new Checkbox("show anchor", true);
	    toggle.addItemListener(new AnimListener(ta, ANCHOR));
	    add(toggle);
	    toggle = new Checkbox("rotate", false);
	    toggle.addItemListener(new AnimListener(ta, ROTATE));
	    add(toggle);
	    toggle = new Checkbox("shearx", false);
	    toggle.addItemListener(new AnimListener(ta, SHEARX));
	    add(toggle);
	    toggle = new Checkbox("sheary", false);
	    toggle.addItemListener(new AnimListener(ta, SHEARY));
	    add(toggle);
	    toggle = new Checkbox("oval", false);
	    toggle.addItemListener(new AnimListener(ta, OVAL));
	    add(toggle);
	    toggle = new Checkbox("gray", false);
	    toggle.addItemListener(new AnimListener(ta, GRAY));
	    add(toggle);
	    toggle = new Checkbox("filter", false);
	    toggle.addItemListener(new AnimListener(ta, QUALITY));
	    add(toggle);

	    Choice modechooser = new Choice();
	    Choice sizechooser = new Choice();

	    Button button = new Button("Load Image...");
	    button.addActionListener(new ImageFetcher(ta, modechooser));
	    add(button);

	    modechooser.add("TexturePaint");
	    modechooser.add("GradientPaint");
	    modechooser.add("SolidFill");
	    modechooser.add("Image");
	    modechooser.addItemListener(new FillChooser(ta));
	    add(modechooser);

	    sizechooser.add("TextureSize: 4");
	    sizechooser.add("TextureSize: 8");
	    sizechooser.add("TextureSize: 16");
	    sizechooser.add("TextureSize: 32");
	    sizechooser.add("TextureSize: 100");
	    sizechooser.addItemListener(new AnimChooser(ta, modechooser));
	    add(sizechooser);

	    sizechooser.select(4);
	    modechooser.select(0);
	}
    }

    static class AnimVal {
	float curval;
	float lowval;
	float highval;
	float currate;
	float lowrate;
	float highrate;

	public AnimVal(int lowval, int highval,
		       int lowrate, int highrate) {
	    this.lowval = lowval;
	    this.highval = highval;
	    this.lowrate = lowrate;
	    this.highrate = highrate;
	    this.curval = randval(lowval, highval);
	    this.currate = randval(lowrate, highrate);
	}

	public AnimVal(int lowval, int highval,
		       int lowrate, int highrate,
		       int pos) {
	    this(lowval, highval, lowrate, highrate);
	    set(pos);
	}

	public float randval(float low, float high) {
	    return (float) (low + Math.random() * (high - low));
	}

	public float getFlt() {
	    return curval;
	}

	public int getInt() {
	    return (int) curval;
	}

	public void anim() {
	    curval += currate;
	    clip();
	}

	public void set(float val) {
	    curval = val;
	}

	public void clip() {
	    if (curval > highval) {
		curval = highval;
		currate = - randval(lowrate, highrate);
	    } else if (curval < lowval) {
		curval = lowval;
		currate = randval(lowrate, highrate);
	    }
	}

	public void newlimits(int lowval, int highval) {
	    if (highval < lowval) {
		this.lowval = highval;
		this.highval = lowval;
	    } else {
		this.lowval = lowval;
		this.highval = highval;
	    }
	    clip();
	}
    }
}
