/*
 * @(#)AreaDemo.java	1.8 00/01/19
 *
 * Copyright 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


/* Revision: AreaDemo.java 1.8 @(#)AreaDemo.java	1.8 
 *
 * @package
 * @copyright
 *  (C) Copyright Taligent, Inc.,  1996 - AllRights Reserved
 *  (C) Copyright IBM Corp.,  1996
 *  This source code and documentation are Taligent Confidential materials. These materials are
 *  provided for internal evaluation purposes only. This notice may not be removed. This
 *  technology is protected by multiple US and international patents.
 *
 *  Taligent is a registered trademark of Taligent, Inc.
 *
 */

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.applet.*;

import java.util.Vector;

public class AreaDemo extends Applet {
    public void init() {
        setLayout(new BorderLayout());
        DrawCanvas dc = new DrawCanvas();
	add("North", new SubpathControls(dc));
        add("Center", dc);
        add("South", new DrawControls(dc));
    }

    public static void main(String args[]) {
        Frame f = new Frame("AreaDemo");
        AreaDemo demo = new AreaDemo();
        demo.init();
        demo.start();

	f.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});
        f.add(demo);
        f.setSize(400, 300);
        f.show();
    }
}

class DrawCanvas extends Canvas implements MouseListener, MouseMotionListener {
    public static final int RECTS = 0;
    public static final int OVALS = 1;
    int mode = RECTS;

    Area theArea = new Area();

    public static final int ADD = 0;
    public static final int SUBTRACT = 1;
    public static final int INTERSECT = 2;
    public static final int XOR = 3;
    int action = ADD;

    int xorig,yorig;
    RectangularShape dragshape;

    SubpathControls pathlistener;
    int highlight;

    public DrawCanvas() {
        setBackground(Color.white);
        action = ADD;
	addMouseListener(this);
    }

    public void setPathListener(SubpathControls pl) {
	pathlistener = pl;
	notifylistener();
    }

    public void notifylistener() {
	if (pathlistener != null) {
	    pathlistener.notify(theArea);
	}
    }

    public void setHighlight(int h) {
	highlight = h;
	repaint();
    }

    public void clearArea() {
        theArea.reset();
	dragshape = null;
	notifylistener();
        repaint();
    }

    public void updateArea() {
	if (dragshape == null) {
	    return;
	}
	Area rhs = new Area(dragshape);
        switch (action) {
          case ADD:
            theArea.add(rhs);
            break;
          case SUBTRACT:
            theArea.subtract(rhs);
            break;
          case INTERSECT:
            theArea.intersect(rhs);
            break;
          case XOR:
            theArea.exclusiveOr(rhs);
            break;
          default:
            throw new IllegalArgumentException();
        }
	notifylistener();
    }

    public void setAreaOperation(int opn) {
        switch (opn) {
          case ADD:
          case SUBTRACT:
          case INTERSECT:
          case XOR:
            action = opn;
            break;
          default:
            throw new IllegalArgumentException();
        }
    }
    public void setDrawMode(int theMode) {
        switch (theMode) {
          case RECTS:
          case OVALS:
            mode = theMode;
            break;
          default:
            throw new IllegalArgumentException();
        }
    }

    public void mouseClicked(MouseEvent e) {
	mouseReleased(e);
    }

    public void mousePressed(MouseEvent e) {
	xorig = e.getX();
	yorig = e.getY();
	if (mode == RECTS) {
	    dragshape = new Rectangle(xorig, yorig, 0, 0);
	} else {
	    dragshape = new Ellipse2D.Float(xorig, yorig, 0, 0);
	}
	addMouseMotionListener(this);
    }

    public void mouseReleased(MouseEvent e) {
	updateArea();
	dragshape = null;
	repaint();
	removeMouseMotionListener(this);
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
	if (dragshape != null) {
	    dragshape.setFrameFromDiagonal(xorig, yorig, e.getX(), e.getY());
	    repaint();
	}
    }

    public void mouseMoved(MouseEvent e) {
	mouseDragged(e);
    }

    public void paint(Graphics g) {
	Graphics2D g2 = (Graphics2D)g;

        g2.setColor(Color.yellow);
	g2.fill(theArea);
        g2.setColor(Color.red);
        g2.draw(theArea);

	if (highlight != 0) {
	    Shape s = extractSubpath(theArea, highlight);
	    g2.setColor(Color.blue);
	    g2.draw(s);
	}

        if (dragshape != null) {
            g2.setColor(Color.green);
            g2.setXORMode(getBackground());
	    g2.draw(dragshape);
            g2.setPaintMode();
        }
    }

    public Shape extractSubpath(Shape s, int count) {
	PathIterator pi = s.getPathIterator(null);
	GeneralPath gp = new GeneralPath();
	float coords[] = new float[6];
	while (!pi.isDone()) {
	    int type = pi.currentSegment(coords);
	    switch (type) {
	    case pi.SEG_MOVETO:
		count--;
		if (count == 0) {
		    gp.moveTo(coords[0], coords[1]);
		}
		break;
	    case pi.SEG_LINETO:
		if (count == 0) {
		    gp.lineTo(coords[0], coords[1]);
		}
		break;
	    case pi.SEG_QUADTO:
		if (count == 0) {
		    gp.quadTo(coords[0], coords[1],
			      coords[2], coords[3]);
		}
		break;
	    case pi.SEG_CUBICTO:
		if (count == 0) {
		    gp.curveTo(coords[0], coords[1],
			       coords[2], coords[3],
			       coords[4], coords[5]);
		}
		break;
	    case pi.SEG_CLOSE:
		if (count == 0) {
		    gp.closePath();
		}
		break;
	    }
	    pi.next();
	}
	return gp;
    }
}

class DrawControls extends Panel {
    DrawCanvas target;

    public DrawControls(DrawCanvas dp) {
        this.target = dp;
        setLayout(new FlowLayout());
	Button b = new Button("Clear");
	b.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		target.clearArea();
	    }
	});
	add(b);
        CheckboxGroup group = new CheckboxGroup();
        addCheckbox("Add", DrawCanvas.ADD, group, true);
        addCheckbox("Sub", DrawCanvas.SUBTRACT, group, false);
        addCheckbox("Int", DrawCanvas.INTERSECT, group, false);
        addCheckbox("XOR", DrawCanvas.XOR, group, false);
        Choice shapes = new Choice();
        shapes.addItem("Rectangle");
        shapes.addItem("Ellipse");
	shapes.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.DESELECTED) {
		    return;
		}
		String item = (String) e.getItem();
		if ("Rectangle".equals(item)) {
		    target.setDrawMode(DrawCanvas.RECTS);
		} else if ("Ellipse".equals(item)) {
		    target.setDrawMode(DrawCanvas.OVALS);
		}
	    }
	});
        add(shapes);
    }

    public void addCheckbox(String label, final int op,
			    CheckboxGroup group, boolean check)
    {
	Checkbox cb = new Checkbox(label, group, check);
	cb.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.DESELECTED) {
		    return;
		}
		target.setAreaOperation(op);
	    }
	});
	add(cb);
    }
}

class SubpathControls extends Panel {
    DrawCanvas target;
    Choice choice;

    public SubpathControls(DrawCanvas dp) {
        this.target = dp;
	choice = new Choice();
	choice.add("Highlight nothing");
	choice.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		target.setHighlight(choice.getSelectedIndex());
	    }
	});
	addButton("Prev", -1);
	add(choice);
	addButton("Next", +1);
	dp.setPathListener(this);
    }

    public void addButton(String label, final int dir) {
	Button b = new Button(label);
	b.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		int sel = choice.getSelectedIndex() + dir;
		if (sel >= 0 && sel < choice.getItemCount()) {
		    target.setHighlight(sel);
		    choice.select(sel);
		}
	    }
	});
	add(b);
    }

    public void notify(Shape s) {
	int numpaths = countSubpaths(s);
	int count = choice.getItemCount() - 1;
	if (count == numpaths) {	
	    return;
	}
	int sel = choice.getSelectedIndex();
	if (sel > numpaths) {
	    sel = numpaths;
	}
	while (count < numpaths) {
	    count++;
	    choice.add("Highlight subpath "+count);
	}
	while (count > numpaths) {
	    choice.remove(count);
	    count--;
	}
	target.setHighlight(sel);
	choice.select(sel);
	validate();
    }

    // Don't need actual values so array can be shared by everyone.
    public static double scratch[] = new double[6];

    public int countSubpaths(Shape s) {
	int numpaths = 0;
	PathIterator pi = s.getPathIterator(null);
	while (!pi.isDone()) {
	    int type = pi.currentSegment(scratch);
	    if (type == pi.SEG_MOVETO) {
		numpaths++;
	    }
	    pi.next();
	}
	return numpaths;
    }
}
