/*
 * @(#)AppleImpl.java	1.6 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

import java.rmi.*;
import java.rmi.server.*;

import java.awt.*;

/**
 * The AppleImpl class implements the behavior of the remote "apple"
 * objects exported by the applet.
 */
public class AppleImpl
    extends UnicastRemoteObject
    implements Apple
{

    String name;
    JuicerDisplay display;

    public AppleImpl(String name, JuicerDisplay display)
	throws RemoteException
    {
	this.name    = name;
	this.display = display;
    }

    /**
     * Receive an array of AppleEvent objects.
     */
    public void notify(AppleEvent[] events)
    {
	try {
	    String threadName = Thread.currentThread().getName();

	    for (int i = 0; i < events.length; ++ i)
		display.append(
		    toString() +
			"'s notify() method: events[" + i + "] = " +
			events[i].toString() + "\n");

	} catch (RuntimeException e) {
	    synchronized (System.err) {
		System.err.print("AppleImpl.notify(): ");
		e.printStackTrace();
	    }
	    throw e;
	}
    }

    public void printlnToAppletDisplay(String s)
    throws RemoteException
    {
        try
        {
	        System.out.println(s);
	        display.append(s + "\n");
        }
        catch (RuntimeException e)
        {
	        synchronized (System.err)
	        {
		        System.err.print("AppleImpl.printlnToAppletDisplay(): ");
		        e.printStackTrace();
	        }
	        throw e;
	    }
    }
    /**
     * Return a newly created and exported orange implementation.
     */
    public Orange newOrange(String name, int numIterations)
	throws RemoteException
    {
	try {
	    String threadName = Thread.currentThread().getName();

	    Orange orange = new OrangeImpl(name, display);

	    //display.appendText(
		//threadName + ": " + toString() +
		//".newOrange(" + name + "): END\n");
        display.append(
		    name + " on iteration #" + numIterations +
		    " called remote method - newOrange.  Orange creation was successful!" + "\n");

	    return orange;
	} catch (RuntimeException e) {
	    synchronized (System.err) {
		System.err.print("AppleImpl.newOrange(): ");
		e.printStackTrace();
	    }
	    throw e;
	}
    }

    public String toString()
    {
	return name;
    }
}
