/*
 * @(#)QueryDialog.java	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package visualBenchmarks;

import java.awt.*;
import java.awt.event.*;
import java.awt.TextArea.*;
import java.lang.*;

/**
 * An extension of the java.awt.Dialog class, tailored for queries.
 * Constructs a <em>modal</em> dialog box with an query and a title. 
 */

public class QueryDialog extends Dialog implements ActionListener
{

	/**
	 * A YES button displayed towards the bottom of the dialog box. The
	 * must either accept or reject to continue execution 
	 */
	protected Button Accept;

	/**
	 * A NO button displayed towards the bottom of the dialog box. The user
	 * must either accept or reject to continue execution
	 */
  	protected Button Reject;
  
  	/**
	 * Region where the query is displayed
	 */
	protected TextArea text;

	/**
	 * A constant indicating acceptance
	 */
	public static int ACCEPT = 1;

	/**
	 * A constant indicating rejection
	 */
	public static int REJECT = 2;

	/**
	 * An integer variable holding the user's respone. This variable is 
	 * tested by the calling component to determine the user's response.
	 */
	public int response;
	
	/**
	 * Constructs a Query Dialog box with a title and a query
	 *
	 * @param parent  the Frame to which the dialog box is attached
	 * @param title   the title for the Query Dialog box
	 * @param message the query
	 */
	public QueryDialog(Frame parent, String title, String message) 
	{
		super(parent, title, true);

		this.setLayout(new BorderLayout(15, 15));   

		text = new TextArea(message,3 ,50,TextArea.SCROLLBARS_NONE );

		//Disable editing of the error message
		text.setEditable(false);

		//Adding the message box to the center causes it to occupy
	 	//all the space left after the other component has been layed 
		//out
		this.add("Center", text);

		// Create and add and OK button.
		Accept = new Button("YES");
		Accept.addActionListener(this);
		Reject = new Button("NO");
		Reject.addActionListener(this);
		Panel p = new Panel();
		p.setLayout(new FlowLayout(FlowLayout.CENTER, 15, 15));
		p.add(Accept);
		p.add(Reject);

		//Center the Dialog box in the parent's window space (approx.)
		setLocation(parent.getSize().width/2,
			 parent.getSize().height/2);

		this.add("South", p);

		// Resize the window to the preferred size of its components.
		this.pack();
	}


	public void actionPerformed(ActionEvent evt)
	{
		String str = evt.getActionCommand();

		if(str.equals("YES"))
		{
			response = ACCEPT;
			this.setVisible(false);
			this.dispose();
		}else if(str.equals("NO")) {
			response = REJECT;
			this.setVisible(false);
			this.dispose();
		}
		
	}
}

