/*
 * @(#)TestInterface.java	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package benchmarks;

import java.util.*;
import java.rmi.*;

/**
 * A set of remote functions that is implemented by the DataCollector and 
 * is invoked by the Clients. 
 */
public interface TestInterface extends Remote {

	/**
	 * A null call with no arguments and returning void.
	 */
	void nullCall() throws RemoteException;

	/**
	 * A remote method wiht one integer argument .
	 */
	void intCall(int IntData) throws RemoteException; 

	/**
	 * A remote method with an Object argument. Serves as a 
	 * general-purpose call for testing invocations with array arguments,
	 * or any other <em>Objects</em>
	 */  
        void objCall(Object obj) throws RemoteException;

	/**
	 * A remote method with two arguments, an Object and an integer.
	 */
	void twoArgCall(Object obj, int IntData) 
		throws RemoteException;

	/**
	 * A remote method returning a remote object. In this case the server
	 * returns a stub for the remote object.
	 */
	Object getRemObj() throws RemoteException;

	/**
	 * This method creates a vector of remote objects on the server side.
	 * This is used by the clients to get a vector on <i>new</i> remote
	 * objects, i.e ones, for which it hasn't seen the class definitions.
	 *
	 * @param n the size of the vector
	 */ 
	void createVector(int n) throws RemoteException;

	/**
	 * A remote method to retreive a slice of the remote objects vector
	 * created by the server. Retreives a slice of size <i>length</i> 
	 * starting from <i>offset</i>
	 */
	Vector getVector(int offset, int length) throws RemoteException; 

	/**
 	 * A remote method to retrieve the entire vector.
	 */ 	
	Vector getVector() throws RemoteException;

	/**
	 * A remote method always throwing up an Exception
	 *
	 * @exception CollectorException always thrown
	 */
	void exceptionCall() throws RemoteException, CollectorException;
}

