/*
 * @(#)bTree.java	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package benchmarks;

import java.io.*;

/**
 * This class is a utility class to create binary tree of arbitrary depth.
 * It is used in the benchmark suite to test remote calls passing graphs of
 * objects. This is basically a test of the overhead in marshalling the 
 * parameters for a remote calls, since the entire graph of objects is 
 * serialized. Thus, if you simple pass the root as a parameter in a remote 
 * call, the entire tree is serialized and passed to the client.
 *
 * @see node
 */

public class bTree implements Serializable{

	private node root;

	private int depth;

	private int currID;

	private node createTree(int depth){

		node currRoot = new node(currID++);

		if (depth == 1) {
			currRoot.setLeft(new node(-1) );
			currRoot.setRight(new node(-1));;
		}else {
	 		currRoot.setLeft(createTree(depth-1));
			currRoot.setRight(createTree(depth-1));
		}

		return currRoot;		

	}


	/**
	 * Constructs a BTree of a specified depth
	 *
	 * @param depth the depth for the BTree.
	 */
	public bTree(int depth ){

		currID = 1;
		this.root = createTree(depth);
		this.depth = depth;

	}

	/**
 	 * Do a pre-order traversal and print out the nodeIDs for the entire
	 * BTree.
	 *
	 * @see #printTree(node)
	 */
	public void printTree() {

		printTree(root);

	}

	/**
	 * Print out a pre-order traversal of the bTree starting from the
	 * specified node.
	 *
	 * @param startNode the node from where to print out the tree
	 * @see #printTree()
	 */
	public void printTree(node startNode ){

		System.out.print(startNode.getID() + ",");

		if ((startNode.getLeft().getID() != -1) && 
			(startNode.getRight().getID() != -1)) {
		
			//Print out the left Tree
			printTree(startNode.getLeft());
		
			//Print out the right Tree
			printTree(startNode.getRight());

		}

	}

	public int getDepth() {

		return depth;

	}

	public node getRoot() {

		return root;

	}

}
