/*
 * @(#)Test.java	1.6 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package javasoft.sqe.harness;

import java.io.PrintStream;

/**
 * This interface is implemented by tests to be run under the test harness
 * of javasoft.sqe.harness. Information about the test is normally contained
 * in the parameters of a special applet in an HTML file that is read by the
 * test harness.
 *
 * A test should also define `main' as follows:
 * <pre>
 * <code>
 * 	public static void main(String[] args) {
 * 	    Test t = new <em>test-class-name</em>();
 * 	    Status s = t.run(args, System.err, System.out);
 * 	    s.exit();
 * 	}
 * </code>
 * </pre>
 * Defining `main' like this means that the test can also be run standalone, 
 * independent of the harness.
 *
 * @author Jonathan J Gibbons
 * @version 1.7 96/10/18 
 */
public interface Test
{
    /**
     * Runs the test embodied by the implementation.
     * @param args 	These are supplied from the `executeArgs'
     *		   	values in the corresponding test description
     *             	and permit an implementation to be used for a variety of tests.
     * @param log  	A stream to which to report messages and errors.
     * @param ref  	A stream to which to write reference output.
     *			The file may subsequently be used to determine if the test 
     *			succeeded by comparing the contents against a golden file.
     */
    public Status run(String[] args, PrintStream log, PrintStream ref);

}
