/*
 * @(#)SimpleSignatureTest.java	1.11 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package crypto;

import java.io.*;
import java.util.*;
import java.security.*;
import java.security.interfaces.*;

import test.*;

/**
 */
public class SimpleSignatureTest extends Test {
    
    public String algorithm;
    public String provider;

    public void run() throws TestException {
	
	try {
	    KeyPairGenerator gen = 
		KeyPairGenerator.getInstance(algorithm, provider);
	    
	    gen.initialize(1024, new SecureRandom());
	    KeyPair pair = gen.generateKeyPair();
	    PublicKey pub = pair.getPublic();
	    PrivateKey priv = pair.getPrivate();

	    byte[] data = new byte[1000];
	    for (int i = 0; i < 1000; i++) {
		data[i] = (byte)i;
	    }

	    Signature signer = Signature.getInstance(algorithm, provider);
	    signer.initSign(priv);
	    signer.update(data);
	    byte[] signature = signer.sign();

	    Signature verifier = Signature.getInstance(algorithm, provider);
	    verifier.initVerify(pub);
	    verifier.update(data);
	    boolean verifies = verifier.verify(signature);

	    System.out.println("public: " + pub + "\n");
	    System.out.println("private: " + priv + "\n");
	    System.out.println("verifies: " + verifies);

	} catch (Exception e) {
	    throw new TestException("unexpected exception", e);
	}
    }
}
		



	    
