/*
 * @(#)ExtGetTest.java	1.9 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

import java.io.InputStream;
import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.cert.CertificateException;
import java.util.Iterator;
import java.util.Set;

import sun.security.util.ObjectIdentifier;
import sun.security.util.BitArray;
import sun.misc.HexDumpEncoder;

/**
 * Usage: ExtGetTest <fileName>
 */
public class ExtGetTest {

    private static void printSet(Set set, X509Certificate cert)
    throws CertificateException {
        for (Iterator i = set.iterator(); i.hasNext();) {
            String attr = (String)i.next();
            System.out.println(attr);
            byte[] extData = cert.getExtensionValue(attr);
            if (extData != null) {
                HexDumpEncoder encoder = new HexDumpEncoder();
                System.out.print(encoder.encodeBuffer(extData));
            }
        }
    }

    public static void main(String[] args) {
            if (args.length != 1) {
                System.out.println("Usage: ExtGetTest <fileName>");
                System.exit(1);
            }
            try {
                InputStream inStrm = new FileInputStream(args[0]);
		CertificateFactory cf = CertificateFactory.getInstance("X509");
                X509Certificate cert
		    = (X509Certificate)cf.generateCertificate(inStrm);
                inStrm.close();

                Set critSet = cert.getCriticalExtensionOIDs();
                System.out.println("Set of critical extensions");
                printSet(critSet, cert);

                Set nonCritSet = cert.getNonCriticalExtensionOIDs();
                System.out.println("Set of non-critical extensions");
                printSet(nonCritSet, cert);

                boolean[] keyUsage = cert.getKeyUsage();
                int pathLen = cert.getBasicConstraints();
                System.out.println("KeyUsage: " + (new BitArray(keyUsage))
                                   + "; BasicCons=" + pathLen);
            } catch (Exception e) {
                e.printStackTrace();
            }
    }
}
