/*
 * @(#)TestLookAndFeel.java	1.3 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package testlnf;

import java.awt.Font;
import java.awt.Color;
import java.awt.SystemColor;
import java.awt.event.KeyEvent;
import java.awt.Insets;
import java.net.URL;
import java.io.Serializable;
import java.awt.Dimension;
import java.util.*;

import javax.swing.LookAndFeel;
import javax.swing.UIDefaults;

/**
 * Implements a test LookAndFeel class which has no visible state.
 * This LookAndFeel is used for non-visual, automated Swing tests.
 * Most of the default values are cloned from BasicLookAndFeel.
 * <p>
 * <strong>Warning:</strong>
 * Serialized objects of this class will not be compatible with
 * future Swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between applications running the same
 * version of Swing.  A future release of Swing will provide support for
 * long term persistence.
 *
 * @version 1.3 02/02/00
 * @author Thomas Ball
 */
public class TestLookAndFeel extends LookAndFeel
{

    public String getName() {
        return "Test";
    }

    public String getID() {
        return "Test";
    }

    public String getDescription() {
        return "A non-visual Look and Feel for testing";
    }

    public boolean isNativeLookAndFeel() {
	return false;
    }

    public boolean isSupportedLookAndFeel() {
        return true;
    }
    
    public UIDefaults getDefaults() {
	UIDefaults table = new UIDefaults();
	initClassDefaults(table);
	return table;
    }

    /**
     * Initialize the uiClassID to TestComponentUI mapping.
     * The JComponent classes define their own uiClassID constants
     * (see AbstractComponent.getUIClassID).  This table must
     * map those constants to a TestComponentUI class of the
     * appropriate type.
     *
     * @see #getDefaults
     */
    protected void initClassDefaults(UIDefaults table)
    {
	String testPackageName = "testlnf.";
	Object[] uiDefaults = {
		   "ButtonUI", testPackageName + "TestButtonUI",
		 "CheckBoxUI", testPackageName + "TestButtonUI",
	 "CheckBoxMenuItemUI", testPackageName + "TestMenuItemUI",
             "ColorChooserUI", testPackageName + "TestColorChooserUI",
		 "ComboBoxUI", testPackageName + "TestComboBoxUI",
	      "DesktopIconUI", testPackageName + "TestDesktopIconUI",
	      "DesktopPaneUI", testPackageName + "TestDesktopPaneUI",
               "EditorPaneUI", testPackageName + "TestTextUI",
	      "FileChooserUI", testPackageName + "TestFileChooserUI",
	    "InternalFrameUI", testPackageName + "TestInternalFrameUI",
		    "LabelUI", testPackageName + "TestLabelUI",
		     "ListUI", testPackageName + "TestListUI",
		     "MenuUI", testPackageName + "TestMenuItemUI",
		  "MenuBarUI", testPackageName + "TestMenuBarUI",
		 "MenuItemUI", testPackageName + "TestMenuItemUI",
	       "OptionPaneUI", testPackageName + "TestOptionPaneUI",
	            "PanelUI", testPackageName + "TestPanelUI",
	    "PasswordFieldUI", testPackageName + "TestTextUI",
		"PopupMenuUI", testPackageName + "TestPopupMenuUI",
       "PopupMenuSeparatorUI", testPackageName + "TestSeparatorUI",
	      "ProgressBarUI", testPackageName + "TestProgressBarUI",
	      "RadioButtonUI", testPackageName + "TestButtonUI",
      "RadioButtonMenuItemUI", testPackageName + "TestMenuItemUI",
		"ScrollBarUI", testPackageName + "TestScrollBarUI",
	       "ScrollPaneUI", testPackageName + "TestScrollPaneUI",
		"SeparatorUI", testPackageName + "TestSeparatorUI",
		   "SliderUI", testPackageName + "TestSliderUI",
		"SplitPaneUI", testPackageName + "TestSplitPaneUI",
	       "TabbedPaneUI", testPackageName + "TestTabbedPaneUI",
		    "TableUI", testPackageName + "TestTableUI",
	      "TableHeaderUI", testPackageName + "TestTableHeaderUI",
		 "TextAreaUI", testPackageName + "TestTextUI",
		"TextFieldUI", testPackageName + "TestTextUI",
		 "TextPaneUI", testPackageName + "TestTextUI",
	     "ToggleButtonUI", testPackageName + "TestButtonUI",
		  "ToolBarUI", testPackageName + "TestToolBarUI",
	 "ToolBarSeparatorUI", testPackageName + "TestSeparatorUI",
		  "ToolTipUI", testPackageName + "TestToolTipUI",
		     "TreeUI", testPackageName + "TestTreeUI",
		 "ViewportUI", testPackageName + "TestViewportUI",
	};

	table.putDefaults(uiDefaults);
    }
}
