/*
 * @(#)jcov.h	1.15 01/04/21
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef _JCOV_H
#define _JCOV_H

#include "typedefs.h"
#include "jvmpi.h"
#include "jcov_hash.h"

#define MAX_PATH_LEN   1024
#define TIMESTAMP_SIZE 24
#define JCOV_FILE_MAJOR_VER 2
#define JCOV_FILE_MINOR_VER 0

typedef struct {
    unsigned short pc; 		/* starting pc for this item                  */
    unsigned char  type;	/* item type                                  */
    unsigned char  instr_type;	/* type of the first instruction in this item */
    unsigned long  where_line;	/* line in source file                        */
    unsigned long  where_pos;	/* position in source file                    */
    unsigned long  count;	/* execution counter                          */
} cov_item_t;

typedef struct {
    jobjectID    id;                        /* class ID                           */
    char         *name;                     /* class name                         */
    char         *src_name;                 /* src file name                      */
    char         timestamp[TIMESTAMP_SIZE]; /* timestamp as generated by compiler */
    int          num_methods;               /* number of methods                  */
    JVMPI_Method *methods;                  /* methods                            */
    int          access_flags;              /* access flags                       */
    char         data_type;                 /* type of gathered jcov data         */
    int          unloaded;                  /* true when class is unloaded        */
} jcov_class_t;

typedef struct {
    jmethodID     id;            /* method ID                                          */
    char          *name;         /* method name                                        */
    char          *signature;    /* method signature                                   */
    int           access_flags;  /* access flags                                       */
    int           *pc_cache;     /* keeps map of pc <-> cov_item correspondence        */
    unsigned int  pc_cache_size; /* size of the map (is equal to method's code length) */
    cov_item_t    *covtable;     /* coverage table                                     */
    unsigned int  covtable_size; /* coverage table size                                */
    jcov_class_t  *class;
} jcov_method_t;

typedef struct {
    char          *name;                     /* class name                             */
    char          *src_name;                 /* src file name                          */
    char          timestamp[TIMESTAMP_SIZE]; /* timestamp as generated by compiler     */
    char          data_type;                 /* type of gathered jcov data             */
    int           access_flags;              /* access flags                           */
    jcov_method_t **method_cache;            /* place to store methods of hooked class */
    int           methods_total;             /* number of methods in cache             */
} jcov_hooked_class_t;

typedef struct {
    JNIEnv      *id;                /* thread ID                                           */
    jcov_hash_t hooked_class_table; /* keeps the hooked class' (name, src_name, timestamp) */
    JVMPI_CallTrace *call_trace;
} jcov_thread_t;

/* range of values of cov_item_t.type -
 * as generated by compiler
 */
enum {
    CT_METHOD = 1,
    CT_FICT_METHOD,
    CT_BLOCK,
    CT_FICT_RET,
    CT_CASE,
    CT_SWITH_WO_DEF,
    CT_BRANCH_TRUE,
    CT_BRANCH_FALSE
};

/* range of values of cov_item_t.instr_type */
enum {
    INSTR_ANY = 0,
    INSTR_IF,
    INSTR_LOOKUPSW,
    INSTR_TABLESW
};


#define CALL(f) (jvmpi_interface->f)
extern JVMPI_Interface *jvmpi_interface;

#endif /* _JCOV_H */
