/*
 * @(#)jcov_hash.h	1.3 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef _JCOV_HASH_H
#define _JCOV_HASH_H

/* Generic support for hash tables */
/* based on src/share/tools/hprof/hprof_hash.h */

typedef struct jcov_bucket_t {
    struct jcov_bucket_t *next;
    void *content;
} jcov_bucket_t;

typedef struct {
    unsigned int n_entries;            /* total buckets in the table */
    unsigned int size;                 /* size of the top-level table */
    jcov_bucket_t **entries;
    unsigned int (*hash_f)(void *);    /* hash function */
    unsigned int (*size_f)(void *);    /* content size */
    int (*compare_f)(void *, void *);  /* comparison function */
} jcov_hash_t;

void jcov_hash_init(jcov_hash_t *table, 
		     int size,
		     unsigned int (*hash_f)(void *),
		     unsigned int (*size_f)(void *),
		     int (*compare_f)(void *, void *));
void * jcov_hash_iterate(jcov_hash_t *table,
			  void * (*f)(void *, void *),
			  void *arg);
void jcov_hash_remove(jcov_hash_t *table,
		       int (*f)(void *, void *),
		       void *arg);
void * jcov_hash_lookup(jcov_hash_t *table, void *new);
void * jcov_hash_put(jcov_hash_t *table, void *new);
#endif /* _JCOV_HASH_H */
