/*
 * @(#)jcov_htables.c	1.4 01/04/21
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <stdlib.h>
#include <string.h>

#include "jvmpi.h"
#include "jcov.h"
#include "jcov_hash.h"
#include "jcov_htables.h"
#include "jcov_setup.h"

static jcov_hash_t class_table_id;
static jcov_hash_t class_table_key;
static jcov_hash_t *method_table;
static jcov_hash_t *method_table_filtered;
static jcov_hash_t thread_table;

/* ====================================*/
/* === functions related to classes ===*/
/* ====================================*/

static unsigned int hash_string(char *str) 
{
    int i;
    unsigned int hash = 0;
    for (i = 0; i < strlen(str); i++) {
        hash = 37*hash + str[i];
    }
    return hash;
}

static unsigned int hash_class_id(void *c) 
{
    return (unsigned int)(*(jcov_class_t**)c)->id;
}

unsigned int hash_hooked_class(void *c) 
{
    jcov_hooked_class_t *class = *(jcov_hooked_class_t**)c;
    return hash_string(class->name);
}

static unsigned int hash_class_key(void *c)
{
    int i; char *tmp;
    unsigned int hash = 0;
    jcov_class_t *class = *(jcov_class_t**)c;
    hash += hash_string(class->name);
    hash += hash_string(class->src_name);
    hash += hash_string(class->timestamp);
    return hash;
}

static unsigned int size_class(void *m)
{
    return sizeof(jcov_class_t*);
}

unsigned int size_hooked_class(void *c)
{
    return sizeof(jcov_hooked_class_t*);
}

int compare_hooked_classes(void *c1, void *c2)
{
    jcov_hooked_class_t *class1 = *(jcov_hooked_class_t**)c1;
    jcov_hooked_class_t *class2 = *(jcov_hooked_class_t**)c2;
    return strcmp(class1->name, class2->name);
}

int compare_hooked_classes_not(void *c1, void *c2)
{
    return !compare_hooked_classes(c1, c2);
}

int compare_class_key(void *c1, void *c2)
{
    int cmp_res;
    jcov_class_t *class1 = *(jcov_class_t**)c1;
    jcov_class_t *class2 = *(jcov_class_t**)c2;

    cmp_res = strcmp(class1->name, class2->name);
    if (cmp_res) {
        return cmp_res;
    }
    cmp_res = strcmp(class1->src_name, class2->src_name);
    if (cmp_res) {
        return cmp_res;
    }
    return strcmp(class1->timestamp, class2->timestamp);
}

static int compare_class_id(void *c1, void *c2)
{
    jcov_class_t *class1 = *(jcov_class_t**)c1;
    jcov_class_t *class2 = *(jcov_class_t**)c2;

    return (int)((long)class1->id - (long)class2->id);
}

static int compare_class_id_not(void *c1, void *c2)
{
    return !compare_class_id(c1, c2);
}

void class_table_iterate(void *(*f)(void *, void *))
{
    void *dummy = NULL;
    jcov_hash_iterate(&class_table_key, f, dummy);
}

void class_key_table_iterate(void *(*f)(void *, void *))
{
    void *dummy = NULL;
    jcov_hash_iterate(&class_table_key, f, dummy);
}

jcov_class_t *lookup_class_by_id(jobjectID class_id)
{
    jcov_class_t **ptr;
    jcov_class_t class_tmp;
    jcov_class_t *class_tmp_addr = &class_tmp;
    class_tmp.id = class_id;
    ptr = (jcov_class_t**)jcov_hash_lookup(&class_table_id, &class_tmp_addr);
    return (ptr ? *ptr : NULL);
}

jcov_hooked_class_t *lookup_hooked_class(jcov_hash_t *table, char *name)
{
    jcov_hooked_class_t **ptr;
    jcov_hooked_class_t class_tmp;
    jcov_hooked_class_t *class_tmp_addr = &class_tmp;
    class_tmp.name = name;
    ptr = (jcov_hooked_class_t**)jcov_hash_lookup(table, &class_tmp_addr);
    return (ptr ? *ptr : NULL);
}

jcov_class_t *lookup_class_by_key(jcov_class_t *class)
{
    jcov_class_t **ptr;
    ptr = (jcov_class_t**)jcov_hash_lookup(&class_table_key, &class);
    return (ptr ? *ptr : NULL);
}

jcov_class_t *put_class_by_id(jcov_class_t **c)
{
    jcov_class_t **ptr;
    ptr = (jcov_class_t**)jcov_hash_put(&class_table_id, c);
    return *ptr;
}

jcov_hooked_class_t *put_hooked_class(jcov_hash_t *table,
                                      jcov_hooked_class_t **c)
{
    jcov_hooked_class_t **ptr;
    ptr = (jcov_hooked_class_t**)jcov_hash_put(table, c);
    return *ptr;
}

jcov_class_t *put_class_by_key(jcov_class_t **c) 
{
    jcov_class_t **ptr;
    ptr = (jcov_class_t**)jcov_hash_put(&class_table_key, c);
    return *ptr;
}

void remove_hooked_class(jcov_hash_t *table, jcov_hooked_class_t *c) 
{
    jcov_hash_remove(table, compare_hooked_classes_not, &c);
}

void remove_class_by_id(jobjectID class_id) 
{
    jcov_class_t class_tmp;
    jcov_class_t *class_tmp_addr = &class_tmp;
    class_tmp.id = class_id;
    jcov_hash_remove(&class_table_id, compare_class_id_not, &class_tmp_addr);
}

int find_method_in_class(jcov_class_t *c, JVMPI_Method *m) 
{
    int i;
    for (i = 0; i < c->num_methods; i++) {
        if (!strcmp((c->methods[i]).method_name, m->method_name) &&
            !strcmp((c->methods[i]).method_signature, m->method_signature)) {
            return i;
        }
    }
    return -1;
}

/* ====================================*/
/* === functions related to methods ===*/
/* ====================================*/

static unsigned int hash_method(void *m)
{
    return (long)(*(jcov_method_t**)m)->id;
}

static unsigned int size_method(void *m)
{
    return sizeof(jcov_method_t*);
}

static int compare_method(void *m1, void *m2)
{
    return (int)((long)(*(jcov_method_t**)m1)->id -
                 (long)(*(jcov_method_t**)m2)->id);
}

static int compare_method_not(void *m1, void *m2)
{
    return !(int)((long)(*(jcov_method_t**)m1)->id -
                  (long)(*(jcov_method_t**)m2)->id);
}

jcov_method_t *lookup_method(jmethodID method_id)
{
    jcov_method_t **ptr;
    jcov_method_t method_tmp;
    jcov_method_t *method_tmp_addr = &method_tmp;
    method_tmp.id = method_id;
    ptr = (jcov_method_t**)jcov_hash_lookup(method_table, &method_tmp_addr);
    return (ptr ? *ptr : NULL);
}

jcov_method_t *lookup_method0(jmethodID method_id)
{
    jcov_method_t **ptr;
    jcov_method_t method_tmp;
    jcov_method_t *method_tmp_addr = &method_tmp;
    method_tmp.id = method_id;
    ptr = (jcov_method_t**)jcov_hash_lookup(method_table_filtered, &method_tmp_addr);
    return (ptr ? *ptr : NULL);
}

jcov_method_t *put_method(jcov_method_t **m)
{
    jcov_method_t **ptr;
    ptr = (jcov_method_t**)jcov_hash_put(method_table, m);
    return *ptr;
}

jcov_method_t *put_method0(jcov_method_t **m)
{
    jcov_method_t **ptr;
    ptr = (jcov_method_t**)jcov_hash_put(method_table_filtered, m);
    return *ptr;
}

void remove_method(jmethodID method_id)
{
    jcov_method_t method_tmp;
    jcov_method_t *method_tmp_addr = &method_tmp;
    method_tmp.id = method_id;
    jcov_hash_remove(method_table, compare_method_not, &method_tmp_addr);
}

int array_lookup_method(JVMPI_Method *key,
                        jcov_method_t **arr,
                        int arr_len,
                        int start_search_index)
{
    int i;
    jcov_method_t *meth;

    for (i = start_search_index; i < arr_len; i++) {
        meth = arr[i];
        if (meth == NULL) continue;
        if (!(strcmp(key->method_name, meth->name)) &&
            !(strcmp(key->method_signature, meth->signature))) {
            return i;
        }
    }
    for (i = 0; i < start_search_index; i++) {
        meth = arr[i];
        if (meth == NULL) continue;
        if (!(strcmp(key->method_name, meth->name)) &&
            !(strcmp(key->method_signature, meth->signature))) {
            return i;
        }
    }
    return -1;
}

/* ====================================*/
/* === functions related to threads ===*/
/* ====================================*/

static unsigned int hash_thread(void *t)
{
    return (unsigned int)(*(jcov_thread_t**)t)->id;
}

static unsigned int size_thread(void *m)
{
    return sizeof(jcov_thread_t*);
}

static int compare_thread(void *t1, void *t2)
{
    return (int)((long)(*(jcov_thread_t**)t1)->id -
                 (long)(*(jcov_thread_t**)t2)->id);
}

static int compare_thread_not(void *t1, void *t2)
{
    return !(int)((long)(*(jcov_thread_t**)t1)->id -
                  (long)(*(jcov_thread_t**)t2)->id);
}

jcov_thread_t *lookup_thread(JNIEnv *thread_id)
{
    jcov_thread_t **ptr;
    jcov_thread_t thread_tmp;
    jcov_thread_t *thread_tmp_addr = &thread_tmp;
    thread_tmp.id = thread_id;
    ptr = (jcov_thread_t**)jcov_hash_lookup(&thread_table, &thread_tmp_addr);
    return (ptr ? *ptr : NULL);
}

jcov_thread_t *put_thread(jcov_thread_t **thread)
{
    jcov_thread_t **ptr;
    ptr = (jcov_thread_t**)jcov_hash_put(&thread_table, thread);
    return *ptr;
}

void remove_thread(JNIEnv *thread_id)
{
    jcov_thread_t thread_tmp;
    jcov_thread_t *thread_tmp_addr = &thread_tmp;
    thread_tmp.id = thread_id;
    jcov_hash_remove(&thread_table, compare_thread_not, &thread_tmp_addr);
}

void thread_table_iterate(void *(*f)(void *, void *))
{
    void *dummy = NULL;
    jcov_hash_iterate(&thread_table, f, dummy);
}

/* ======================*/
/* === initialization ===*/
/* ======================*/

void jcov_htables_init(void)
{
    jcov_hash_init(&class_table_id, 11551, hash_class_id, size_class, compare_class_id);
    jcov_hash_init(&class_table_key, 11551, hash_class_key, size_class, compare_class_key);
    
    method_table = (jcov_hash_t*)jcov_calloc(sizeof(jcov_hash_t));

    jcov_hash_init(method_table, 30031, hash_method, size_method, compare_method);

    if (caller_filter != NULL) {
        method_table_filtered = (jcov_hash_t*)jcov_calloc(sizeof(jcov_hash_t));
        jcov_hash_init(method_table_filtered, 11551, hash_method, size_method, compare_method);
    } else {
        method_table_filtered = method_table;
    }

    jcov_hash_init(&thread_table, 2311, hash_thread, size_thread, compare_thread);
}
