/*
 * @(#)jcov_util.h	1.6 01/04/21
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef _JCOV_UTIL_H
#define _JCOV_UTIL_H

#include <stdlib.h>

#include "jvmpi.h"
#include "jvm.h"
#include "jcov.h"

#define MAX_FILTER_SIZE 256

typedef struct {
    char **incl;
    int  incl_size;
    char **excl;
    int  excl_size;
} jcov_filter_t;

typedef struct {
    unsigned char tag;
    union {
        struct {
            char *bytes;
        } Utf8;
        struct {
            long name_index;
        } Class;
        struct {
            long class_index;
            long name_and_type_index;
        } Methodref;
        struct {
            long class_index;
            long name_and_type_index;
        } InterfaceMethodref;
        struct {
            long name_index;
            long type_index;
        } NameAndType;
#ifdef DEBUG
        struct {
            int val;
        } Integer;
        struct {
            int val;
        } Float;
        struct {
            int lo;
            int hi;
        } Long;
        struct {
            int lo;
            int hi;
        } Double;
        struct {
            int cp_index;
        } String;
        struct {
            long class_index;
            long name_and_type_index;
        } Field;
#endif /* DEBUG */
    } u;
} cp_entry_t;

void *jcov_calloc(unsigned int size);
#define jcov_free(p) if (p != NULL) free(p)
void          jcov_dummy_free(void *);
unsigned char read1bytes(unsigned char **buf, jint *len, int *err_code);
int           read2bytes(unsigned char **buf, jint *len, int *err_code);
long          read4bytes(unsigned char **buf, jint *len, int *err_code);
char         *readUTF8(unsigned char **buf, jint *len, int utflen, int *err_code);
cp_entry_t   *read_next_cp_entry(unsigned char **buf, jint *len, int *err_code);

#ifdef DEBUG
void print_cp_entry(cp_entry_t *cp_entry);
void print_cp(cp_entry_t **cp_entry, int size);
#endif /* DEBUG */

void jcov_free_constant_pool(cp_entry_t **cp, int cp_count);
void jcov_free_hooked_class(jcov_hooked_class_t *c);
void jcov_free_hooked_class_full(jcov_hooked_class_t *c);
void jcov_free_thread(jcov_thread_t *thread);

char *jcov_strdup(const char *str);
void decode_modifiers(char *buf, int access_flags);
char *dummy_src_name(char *class_name);

jcov_filter_t *jcov_filter_new();
void filter_add_incl(jcov_filter_t *filter, char *str);
void filter_add_excl(jcov_filter_t *filter, char *str);
void filter_finalize(jcov_filter_t *filter);
void debug_print_filter(jcov_filter_t *filter);
int string_suits_filter(jcov_filter_t *filter, char *str);

void qsort_strings(char **array, int array_size);
char *search_prefix(const char *key, const char **prefix_array, int array_size);

void jcov_close(FILE **f);
void jcov_remove(const char *filename);

void jcov_util_init(void);

#endif /* _JCOV_UTIL_H */
