/*
 * @(#)X11DrawingSurface.java	1.6 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt;

import sun.awt.PhysicalDrawingSurface;
import java.awt.image.ColorModel;

/**
 * The X11DrawingSurface interface defines the methods needed to
 * be able to render to an X11 window.  Only the commonly needed
 * attributes are returned, other attributes can be queried by
 * calling the X11 XGetWindowAttributes() function.
 *
 * Many of the methods return Java integers which must be cast to the
 * appropriate Windows data types using a cast similar to the pseudocode
 * indicated in the class comments of the various methods.
 *
 * @version 	1.6, 02/02/00
 * @author 	Jim Graham
 */
public interface X11DrawingSurface extends PhysicalDrawingSurface {
    /**
     * Return the X11 Display pointer for the window or pixmap
     * represented by this object.
     * <p>
     * Display *dpy = (Display *) getDisplay();
     */
    public int getDisplay();

    /**
     * Return the X11 Drawable ID for the window or pixmap
     * represented by this object.
     * <p>
     * Drawable d = (Drawable) getDrawable();
     */
    public int getDrawable();

    /**
     * Return the depth of the window or pixmap
     * represented by this object.
     * <p>
     * int depth = getDepth();
     */
    public int getDepth();

    /**
     * Return the X11 Visual ID of the window or pixmap
     * represented by this object.
     * <p>
     * VisualID vid = (VisualID) getVisualID();
     */
    public int getVisualID();

    /**
     * Return the X11 Colormap ID of the window or pixmap
     * represented by this object.
     * <p>
     * Colormap cmapid = (Colormap) getColormapID();
     */
    public int getColormapID();

    /**
     * Return the ColorModel of the window or pixmap
     * represented by this object.
     * <p>
     * Hjava_awt_image_ColorModel *cm =
     *     (Hjava_awt_image_ColorModel *) getColorModel();
     */
    public ColorModel getColorModel();
}
