/*
 * @(#)DrawStringOneBitMono.java	1.3 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.motif;

import java.awt.Font;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.image.ColorModel;
import java.awt.image.Raster;

import sun.java2d.loops.DrawString;
import sun.java2d.loops.ImageData;
import sun.java2d.loops.TextRendering;
import sun.java2d.loops.DrawStringRasterContext;
import sun.awt.font.NativeFontWrapper;
import sun.awt.image.ByteComponentRaster;


// @@ 1BIT MONO
// non anti-aliased string context
class BCRDrawStringOneBitMonoRasterContext
        extends DrawStringRasterContext
{
    final Font font;
    final double [] matrix;
    final float originX, originY;
    final Color color;
    final int clipX, clipY, clipW, clipH;
    final int offset, pixStr, scanStr;
    final byte [] dst;
    BCRDrawStringOneBitMonoRasterContext(
            ByteComponentRaster bcr,
            int rootBand, Color color,
            Font font, AffineTransform tx,
            int clipX, int clipY, int clipW, int clipH)
    {
        this.offset = bcr.getDataOffset(rootBand);
        this.dst = bcr.getDataStorage();
        this.pixStr = bcr.getPixelStride();
        this.scanStr = bcr.getScanlineStride();
        this.font = font;
        this.originX = (float)tx.getTranslateX();
        this.originY = (float)tx.getTranslateY();
        this.matrix = new double[4];
        tx.getMatrix(this.matrix);
        this.color = color;
        this.clipX = clipX;
        this.clipY = clipY;
        this.clipW = clipW;
        this.clipH = clipH;
    }
    public void invoke(String str, float x, float y) {
        NativeFontWrapper.drawStringOneBitMonoRaster(
            str, x + this.originX, y + this.originY,
            this.font, this.matrix,
            1, // monochrome
            this.clipX, this.clipY, this.clipW, this.clipH,
            this.offset, this.dst, this.pixStr, this.scanStr
        );
    }
}


public class DrawStringOneBitMono extends DrawString {
 
    public DrawStringOneBitMono() {
        super(X11OneBitMono.ST_1BIT_MONO);
    }
    public void DrawString(
            ImageData dest,
            String srcData,
            float fX,
            float fY,
            Color fgColor,
            Font fontObject,
            double[] matrix,
            boolean doAntiAlias,
            boolean doFractEnable
    ) {
        // note: non-AntiAliasing only !
        NativeFontWrapper.drawStringOneBitMono(
            srcData, fX, fY,
            fontObject,
            matrix,
            1, // monochrome
            dest.getXOutputArea(),
            dest.getYOutputArea(),
            dest.getDXOutputArea(),
            dest.getDYOutputArea(),
            dest
        );
    }

    public DrawStringRasterContext createCachedContext(
            Raster raster, ColorModel cm, Color color, Font font,
            AffineTransform tx,
            boolean doAntiAlias,
            boolean doFractEnable,
            int clipX, int clipY, int clipW, int clipH) {
        return new BCRDrawStringOneBitMonoRasterContext(
                        (ByteComponentRaster)raster,
                        2, color, font, tx,
                        clipX, clipY, clipW, clipH);
    }
}      

