/*
 * @(#)MCheckboxPeer.java	1.23 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

package sun.awt.motif;

import java.awt.*;
import java.awt.peer.*;
import java.awt.event.*;

public class MCheckboxPeer extends MComponentPeer implements CheckboxPeer {
    private boolean inUpCall = false;
    private boolean inInit=false;

    native void create(MComponentPeer parent);
    native void pSetState(boolean state);
    native boolean pGetState();

    public native void setLabel(String label);
    public native void setCheckboxGroup(CheckboxGroup g);


    void initialize() {
	Checkbox t = (Checkbox)target;
	inInit=true;

	setState(t.getState());
	setCheckboxGroup(t.getCheckboxGroup());
	super.initialize();
	inInit=false;
    }

    public MCheckboxPeer(Checkbox target) {
	super(target);
    }

    public boolean isFocusTraversable() {
	return true;
    }

    public void setState(boolean state) {
	if (inInit) {
		pSetState(state);
	} else if (!inUpCall && (state != pGetState())) {
		pSetState(state);
	    	// 4135725 : do not notify on programatic changes
		// notifyStateChanged(state);
	}
    }

    public Dimension getMinimumSize() {
	String lbl = ((Checkbox)target).getLabel();
	if (lbl != null) {
	    FontMetrics fm = getFontMetrics(target.getFont());
	    return new Dimension(30 + fm.stringWidth(lbl), 
				 Math.max(fm.getHeight() + 8, 15));
	}
	return new Dimension(20, 15);
    }


    void notifyStateChanged(boolean state) {
	Checkbox cb = (Checkbox) target;
	ItemEvent e = new ItemEvent(cb,
			  ItemEvent.ITEM_STATE_CHANGED,
			  cb.getLabel(),
			  state ? ItemEvent.SELECTED : ItemEvent.DESELECTED);
	postEvent(e);
    }


    // NOTE: This method is called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    void action(boolean state) {
	final Checkbox cb = (Checkbox)target;
	final boolean newState = state;
	MToolkit.executeOnEventHandlerThread(cb, new Runnable() {
	    public void run() {
                CheckboxGroup cbg = cb.getCheckboxGroup();
                /* Bugid 4039594. If this is the current Checkbox in 
		 * a CheckboxGroup, then return to prevent deselection. 
		 * Otherwise, it's logical state will be turned off, 
		 * but it will appear on.
                 */
                if ((cbg != null) && (cbg.getSelectedCheckbox() == cb) &&
                    cb.getState()) {
                  inUpCall = false;
		  cb.setState(true);
                  return;
                }
		// All clear - set the new state
	        cb.setState(newState);
		notifyStateChanged(newState);
	    } // run()
        });
    } // action()



    static final int SIZE = 19;
    static final int BORDER = 4;
    static final int SIZ = SIZE - BORDER*2 - 1;

    /*
     * Print the native component by rendering the Motif look ourselves.
     * ToDo(aim): needs to query native motif for more accurate size and
     * color information; need to render check mark.
     */
    public void print(Graphics g) {
        Checkbox cb = (Checkbox)target;
	Dimension d = cb.size();
	Color bg = cb.getBackground();
	Color fg = cb.getForeground();
	Color shadow = bg.darker();
	int x = BORDER;
	int y = ((d.height - SIZE) / 2) + BORDER;

	g.setColor(cb.getState()? shadow : bg);

	if (cb.getCheckboxGroup() != null) {
	    g.fillOval(x, y, SIZ, SIZ);
	    draw3DOval(g, bg, x, y, SIZ, SIZ, !(cb.getState()));
	    if (cb.getState()) {
		g.setColor(fg);
		g.fillOval(x + 3, y + 3, SIZ - 6, SIZ - 6);
	    }
	} else {
	    g.fillRect(x, y, SIZ, SIZ);
	    draw3DRect(g, bg, x, y, SIZ, SIZ, !(cb.getState()));
	    if (cb.getState()) {
		g.setColor(fg);
		g.drawLine(x+1, y+1, x+SIZ-1, y+SIZ-1);
		g.drawLine(x+1, y+SIZ-1, x+SIZ-1, y+1);
	    }
	}
	g.setColor(fg);
	String lbl = cb.getLabel();
	if (lbl != null) {
	    // REMIND: align
	    g.setFont(cb.getFont());
	    FontMetrics fm = g.getFontMetrics();
	    g.drawString(lbl, SIZE,
			 (d.height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
	}

	target.print(g);
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	    return getMinimumSize();
    }

}
