/*
 * @(#)MDragSourceContextPeer.java	1.35 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.motif;

import java.awt.event.InputEvent;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;

import java.awt.peer.ComponentPeer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;

import java.awt.dnd.DragSourceContext;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.InvalidDnDOperationException;

import java.io.IOException;

import java.util.Map;
import java.util.Stack;

import sun.awt.SunToolkit;

import sun.awt.dnd.SunDragSourceContextPeer;

/**
 * @version 1.35, 02/09/01
 * @since 1.2
 */
final class MDragSourceContextPeer extends SunDragSourceContextPeer {

    /**
     * construct a new MDragSourceContextPeer
     */

    MDragSourceContextPeer(DragGestureEvent dge) {
	super(dge);
    }


    /**
     * initiate a DnD operation ...
     */

    public void startDrag(DragSourceContext dsc, Cursor c, Image di, Point ioff) throws InvalidDnDOperationException {
        if (dragDropInProgress) {
            throw new InvalidDnDOperationException();
        }
	dragSourceContext = dsc;
	cursor 		  = c;
	actions           = trigger.getSourceAsDragGestureRecognizer().getSourceActions();

	synchronized(this) {
	    Transferable trans = dragSourceContext.getTransferable();
	    Map formatMap = MDataTransferer.getInstance().
		getFormatsForTransferable(trans, dragSource.getFlavorMap());
	    int[] formats = MDataTransferer.getInstance().
		keysToIntArray(formatMap);
  
	    if (nativeCtxt != 0) {
		throw new InvalidDnDOperationException("drag in progress");
	    }
  
	    try {
		nativeCtxt = startDrag(MToolkit.getNativeContainer
				       (dsc.getComponent()).getPeer(),
				       trigger.getTriggerEvent(),
				       cursor,
				       cursor == null ? 0 : cursor.getType(),
				       dsc.getSourceActions(),
				       formats,
				       formatMap);
	    } catch (Exception e) {
		e.printStackTrace();
		throw new InvalidDnDOperationException("failed to create native peer");
	    }
  
	    if (nativeCtxt == 0) {
		throw new InvalidDnDOperationException("failed to create native peer");
	    }
          
	    MDropTargetContextPeer.setCurrentJVMLocalSourceTransferable(dsc.getTransferable());
	}
        dragDropInProgress = true;
    }

    /**
     *
     */

    public void setCursor(Cursor c) throws InvalidDnDOperationException {
	if (cursor == null || !cursor.equals(c)) {
	    cursor = c;
	    if (nativeCtxt != 0) setCursor(nativeCtxt, c, c == null ? 0 : c.getType());
	}
    }


    /**
     *
     */

    private byte[] convertData(final int format, final Map formatMap) {
        final Stack stack = new Stack();

        Runnable getData = new Runnable () {
		public void run() {
                    byte[] data = null;
                    try {
                        DataFlavor flavor = (DataFlavor)
                            formatMap.get(new Integer(format));
                        
                        // Flavor can be null if some application   
                        // requested data in unsupported format.
                        if (flavor != null) {
                            data = MDataTransferer.getInstance().
                                translateTransferable(
                                    dragSourceContext.getTransferable(), 
                                    flavor, format);
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        try {
                            AWTLockAccess.awtLock();
                            stack.push(data);
                        } finally {
                            AWTLockAccess.awtNotifyAll();
                            AWTLockAccess.awtUnlock();
                        }
                    }
		}
	    };
        
        SunToolkit.executeOnEventHandlerThread(component, getData);
            
        while (stack.empty()) {
            AWTLockAccess.awtWait();
        }
        
        return (byte[])stack.pop();
    }

    /**
     * upcall from native code
     */

    private void dragDropFinished(boolean success, int operations) {
	super.dodragDropFinished(success, operations);
	MDropTargetContextPeer.setCurrentJVMLocalSourceTransferable(null);
        dragDropInProgress = false;
    }

    /**
     * downcall into native code
     */

    private native long startDrag(ComponentPeer nativeCont, 
				  InputEvent nativeTrigger, 
				  Cursor c, int ctype, int actions, 
				  int[] types, Map formatMap);

    /**
     * downcall into native code
     */


    private native void setCursor(long nativeCtxt, Cursor c, int cType);

    protected void startSecondaryDispatch() {
	AWTLockAccess.awtUnlock();
	super.startSecondaryDispatch();
	AWTLockAccess.awtLock();
    }

    /*
     * fields
     */

    private static boolean		     dragDropInProgress   = false;
}
