/*
 * @(#)MDropTargetContextPeer.java	1.41 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.motif;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.InvalidDnDOperationException;

import java.io.InputStream;

import java.util.Map;

import java.io.IOException;
import sun.awt.dnd.SunDropTargetContextPeer;

/**
 * <p>
 * The MDropTargetContextPeer class is the class responsible for handling
 * the interaction between the Motif DnD system and Java.
 * </p>
 *
 * @version 1.41, 02/09/01
 * @since 1.2
 *
 */

final class MDropTargetContextPeer extends SunDropTargetContextPeer {
  
    private long			nativeDropTransfer;         
    
    int					nativeDataAvailable	= 0;
    Object				nativeData		= null;
  
    /**
     * create the peer
     */
  
    static MDropTargetContextPeer createMDropTargetContextPeer() {
        return new MDropTargetContextPeer();
    }

    /**
     * create the peer
     */

    private MDropTargetContextPeer() {
	super();
    }

    /**
     * @return current DataFlavors available
     */
    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public DataFlavor[] getTransferDataFlavors() {

	DataFlavor[]  dfs;
	
	if (local != null) {  
	    return local.getTransferDataFlavors();
	} else {
	    synchronized(lock) {
	        return MDataTransferer.getInstance().
		    getFlavorsForFormatsAsArray(currentT,
						currentDT.getFlavorMap());
	    }
	}
    }

    /**
     * @return if the flavor is supported
     */

    public boolean isDataFlavorSupported(DataFlavor df) {
	if (local != null) {
	    return local.isDataFlavorSupported(df);
	}

	synchronized(lock) {
	    return MDataTransferer.getInstance().
	        getFlavorsForFormats(currentT, currentDT.getFlavorMap()).
	        containsKey(df);
	}
    }

    /**
     * @return the data 
     */

    public Object getTransferData(DataFlavor df) 
	throws UnsupportedFlavorException, IOException, 
	    InvalidDnDOperationException 
    {
	if (local != null) {
	    return local.getTransferData(df);
	}

	Map flavorMap;

	synchronized(lock) {
	    if (dropStatus != STATUS_ACCEPT || dropComplete) {
		throw new InvalidDnDOperationException("No drop current");
	    }

	    flavorMap = MDataTransferer.getInstance().
	        getFlavorsForFormats(currentT, currentDT.getFlavorMap());

	    Integer iFormat = (Integer)flavorMap.get(df);
	    if (iFormat == null) {
		throw new UnsupportedFlavorException(df);
	    }
	    int format = iFormat.intValue();
	
	    if (df.isRepresentationClassRemote() &&
		currentDA != DnDConstants.ACTION_LINK) {
		throw new InvalidDnDOperationException("only ACTION_LINK is permissable for transfer of java.rmi.Remote objects");
	    }

	    if (nativeDropTransfer == 0) {
		nativeDropTransfer = startTransfer(nativeDragContext, format);
	    } else {
		addTransfer (nativeDropTransfer, format);
	    }

	    for (nativeDataAvailable = 0;
		 format != nativeDataAvailable;) {
		try {
		    lock.wait();
		} catch (InterruptedException ie) {
                    // do nothing
                }
            }
          
	    if (nativeData == null) {
                throw new IOException("no native data was transferred");
	    }
      
	    if (nativeData instanceof byte[]) {
                try {
		    return MDataTransferer.getInstance().
			translateBytes((byte[])nativeData,
				       df,
				       format);
		} catch (IOException e) {
                    throw new InvalidDnDOperationException(e.getMessage());
                }
	    } else if (nativeData instanceof InputStream) {
                try {
		    return MDataTransferer.getInstance().
			translateStream((InputStream)nativeData, 
					df,
					format);
		} catch (IOException e) {
		    throw new InvalidDnDOperationException(e.getMessage());
                }
            } else {
		throw new IOException("no native data was transfered");
            }
        }
    }
  
    /**
     * signal drop complete
     */
  
    public void dropComplete(boolean success) {
	synchronized(lock) {
	    if (dropStatus == STATUS_NONE) {
		throw new InvalidDnDOperationException("No Drop pending");
            }
	    
            if (currentDTC != null) currentDTC.removeNotify();
	    
            currentC   = null;
            currentDT  = null;
            currentDTC = null;
            currentT   = null;
            currentA   = DnDConstants.ACTION_NONE;
  
	    synchronized(_globalLock) {
		currentJVMLocalSourceTransferable = null;
            }
  
	    dropStatus   = STATUS_NONE;
            dropComplete = true;
        }
  
	dropDone(nativeDragContext, nativeDropTransfer, local != null,
		 success, currentDA);
  
	currentDA = DnDConstants.ACTION_NONE;
	nativeDragContext = 0;
    }
  
    /**
     * notify transfer complete
     */
  
    private void newData(int format, String type, byte[] data) {
	synchronized (lock) {
	    nativeDataAvailable = format;
            nativeData          = data;
      
	    lock.notify();
        }
    }

    /**
     * notify transfer failed
     */

    private void transferFailed(int format) {
	synchronized(lock) {
	    nativeDataAvailable = format;
	    nativeData          = null;

	    lock.notify();
	}
    }

    /**
     * schedule a native DnD transfer
     */

    private native long startTransfer(long nativeDragContext, int format);

    /**
     * schedule a native DnD data transfer
     */

    private native void addTransfer(long nativeDropTransfer, int format);

    /**
     * signal that drop is completed
     */

    private native void dropDone(long nativeDragContext,
				 long nativeDropTransfer, boolean localTx,
				 boolean success, int dropAction);

    protected void startSecondaryDispatch() {
        AWTLockAccess.awtUnlock();
        super.startSecondaryDispatch();
        AWTLockAccess.awtLock();
    }
}
