/*
 * @(#)MFileDialogPeer.java	1.27 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.motif;

import java.awt.*;
import java.awt.peer.*;
import java.io.*;
import java.awt.datatransfer.*;

public class MFileDialogPeer extends MDialogPeer implements FileDialogPeer {
    native void create(MComponentPeer parent);
    void create(MComponentPeer parent, Object arg) {
	create(parent);
    }
    public MFileDialogPeer(FileDialog target) {
	super(target);
	FileDialog	fdialog = (FileDialog)target;
	String		dir = fdialog.getDirectory();
        String          file = fdialog.getFile();
	FilenameFilter  filter = fdialog.getFilenameFilter();

	insets = new Insets(0, 0, 0, 0);
	setDirectory(dir);
	if (file != null) {
	    setFile(file);
	}

	if (filter != null)
	    setFilenameFilter(filter);
    }
    native void		pReshape(int x, int y, int width, int height);
    native void         pDispose();
    native void		pShow();
    native void		pHide();
    native void		setFileEntry(String dir, String file, String[] ffiles);
    native void insertReplaceFileDialogText(String l);
    public native void	setFont(Font f);

    // NOTE: This method is called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void		handleSelected(final String file) {
        final FileDialog fileDialog = (FileDialog)target;
        MToolkit.executeOnEventHandlerThread(fileDialog, new Runnable() {
            public void run() {
                int index = file.lastIndexOf(java.io.File.separatorChar);/*2509*//*ibm*/
                String dir;

                if (index == -1) {
                    dir = "."+java.io.File.separator;
                    fileDialog.setFile(file);
                } else {
                    dir = file.substring(0, index + 1);
                    fileDialog.setFile(file.substring(index + 1));
                }
                fileDialog.setDirectory(dir);
                fileDialog.hide();
            }
        });
    } // handleSelected()

    // NOTE: This method is called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void		handleCancel() {
        final FileDialog fileDialog = (FileDialog)target;
        MToolkit.executeOnEventHandlerThread(fileDialog, new Runnable() {
            public void run() {
                fileDialog.setFile(null);
                fileDialog.hide();
            }
        });
    } // handleCancel()

    // NOTE: This method is called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void		handleQuit() {
        final FileDialog fileDialog = (FileDialog)target;
        MToolkit.executeOnEventHandlerThread(fileDialog, new Runnable() {
            public void run() {
                fileDialog.setFile(null);
                fileDialog.hide();
            }
        });
    } // handleQuit()

    public  void setDirectory(String dir) {
        String file = ((FileDialog)target).getFile();
	setFileEntry((dir != null) ? dir : "./", (file != null) ? file
		     : "", null);
    }


    public  void setFile(String file) {
        String dir = ((FileDialog)target).getDirectory();
	setFileEntry((dir != null) ? dir : "./", file, null);
    }


    public void	setFilenameFilter(FilenameFilter filter) {
	String d = ((FileDialog)target).getDirectory();
	String f = ((FileDialog)target).getFile();
	File df = new File((d != null) ? d : ".");
	String[] files = df.list(filter);

	if ( files == null ) {
	    files = new String[1];
	    files[0] = "";
	}

	setFileEntry((d != null) ? d : ".", (f != null) ? f : "", files);
    }

    // Called from native widget when paste key is pressed and we
    // already own the selection (prevents Motif from hanging while
    // waiting for the selection)
    //
    public void pasteFromClipboard() {
        Clipboard clipboard = target.getToolkit().getSystemClipboard();
        
	Transferable content = clipboard.getContents(this);
	if (content != null) {
	    try {
		String data = (String)(content.getTransferData(DataFlavor.stringFlavor));
		insertReplaceFileDialogText(data);
	    } catch (Exception e) {       
	    }
        }
    }

// CAVEAT:
// Peer coalescing code turned over the fact that the following functions
// were being inherited from Dialog and were not implemented in awt_FileDialog.c
// Five methods decribed by the peer interface are at fault (setResizable, setTitle,
// toFront, toBack and handleFocusTraversalEvent).  Additionally show has to be overridden
// as it was necessary to add a show function in MDialogPeer for modality flag passing.
// As a result we were winding up in  awt_Dialog.c (now coalesced into awt_TopLevel).
// As Filedialogs are modal and its unclear to me that any of these functions
// can be called while the FD is on-screen let it go.  RJM.

    public void show() {
        // must have our own show or we wind up in pShow for Window. Bad. Very bad.
        setVisible(true);
    }

}
