/*
 * @(#)MFontProperties.java	1.1 00/07/06
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.motif;

import java.io.File;
import java.io.FileInputStream;
import sun.awt.FontProperties;


public class MFontProperties extends FontProperties {

    public MFontProperties(String xosVersion) {
	super(xosVersion);
    }

    /*
     * Verify that the we actually have access to the X fonts
     * named in the properties file.  Heuristic: the serif
     * fonts in all base font properties files (Solaris 2.6)
     * contain optional fonts for English and the target
     * language.  Performance enhancement: there are bitmap
     * versions of English fonts sansserif TrueType fonts which
     * should be faster.
     */

    protected void verifyProperties(File f) {

	try {
	    String font1, font2;
	    if (f.getPath().endsWith("properties")) {
		font1 = getProperty("sansserif.plain.0");
		if (font1 == null)
		    font1 = getProperty("sansserif.0");
		font2 = null;
	    } else {
		font1 = getProperty("serif.plain.0");
		if (font1 == null)
		    font1 = getProperty("serif.0");
		font2 = getProperty("serif.plain.1");
		if (font2 == null)
		    font2 = getProperty("serif.1");
	    }

	    if (!validPropertiesFile(font1, font2)) {
		String path = f.getPath();

		// Remove osVersion before append .5.5.1 or .rf.
		if (path.endsWith(osVersion))
		    path = path.substring(0, 
			    (path.length() - osVersion.length() - 1));

		File fallback = new File(path + ".5.5.1");
		if (!fallback.exists()) {
		    fallback = new File(path + ".rf");
		    if (!fallback.exists()) {
			fallback = null;
		    }
		}

		if (fallback != null) {
		    // Load property file
		    FileInputStream in = new FileInputStream(fallback);
		    clear();
		    load(in);
		    in.close();
		}
	    }
	} catch (Exception e){
	}
    }

    /**
     * Validate a font properties file.  Overridden in X11GraphicsEnvironment.
     */
    private native boolean validPropertiesFile(String font1, String font2);
}
