/*
 * @(#)MScrollbarPeer.java	1.24 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

package sun.awt.motif;

import java.awt.*;
import java.awt.peer.*;
import java.awt.event.AdjustmentEvent;

class MScrollbarPeer extends MComponentPeer implements ScrollbarPeer {
    native void create(MComponentPeer parent);
    
    boolean inUpCall = false;

    MScrollbarPeer(Scrollbar target) {
	super(target);
    }

    native void pSetValues(int value, int visible, int minimum, int maximum);    
    public native void setLineIncrement(int l);
    public native void setPageIncrement(int l);

    public Dimension getMinimumSize() {
	if (((Scrollbar)target).getOrientation() == Scrollbar.VERTICAL) {
	    return new Dimension(18, 50);
	} else {
	    return new Dimension(50, 18);
	}
    }
  
    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    void doUpCall(int value, int type) {
	final Scrollbar sb = (Scrollbar)target;
	final int newValue = value;
	final int newType = type;

	MToolkit.executeOnEventHandlerThread(sb, new Runnable() {
	    public void run() {
		inUpCall = true;
		sb.setValue(newValue);
		notifyValueChanged(newValue,newType);
		inUpCall = false;
	    }
	});
    }
  
    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void lineUp(int value) {
        doUpCall(value, AdjustmentEvent.UNIT_DECREMENT);
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void lineDown(int value) {
	doUpCall(value, AdjustmentEvent.UNIT_INCREMENT);
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void pageUp(int value) {
	doUpCall(value, AdjustmentEvent.BLOCK_DECREMENT);
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void pageDown(int value) {
	doUpCall(value, AdjustmentEvent.BLOCK_INCREMENT);
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void dragAbsolute(int value) {
	doUpCall(value, AdjustmentEvent.TRACK);
    }

    public void print(Graphics g) {
        Scrollbar sb = (Scrollbar)target;
	Dimension d = sb.size();
	Color bg = sb.getBackground();

	boolean horiz = (sb.getOrientation() == Scrollbar.HORIZONTAL);

	drawScrollbar(g, bg, horiz? d.height : d.width, 
		          horiz? d.width : d.height, 
			  sb.getMinimum(), sb.getMaximum(),
			  sb.getValue(), sb.getVisible(),
			  horiz);
        
	target.print(g);
    }
		
    /** 
     * Set the value of the slider in the ScrollBar. 
     */
    public void setValues(int value, int visible, int minimum, int maximum) {
      /* Fix for BugTraq ID 4048060. Prevent unnecessary redrawing of the 
         slider, when the slider is already in the correct position. Since
         the ScrollBar widget now receives the ButtonRelease X event before
         the Java Adjustor event is handled, the slider is already in the
         correct position and does not need to be set again and redrawn,
         when processing the Adjustor event.
      */
      if (! inUpCall) {
        pSetValues(value, visible, minimum, maximum);
        // new consensus - don't send notification on programatic change
        // ( see bug: 4193532 )
        // notifyValueChanged(value, AdjustmentEvent.TRACK);
      }
    }

    void notifyValueChanged(int value, int type) {
	postEvent(new AdjustmentEvent((Scrollbar)target, 
                                     AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                     type, value));
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	    return getMinimumSize();
    }
}
