/*
 * @(#)MToolkit.java	1.137 01/03/20
 *
 *  Copyright 2000-2001 Sun Microsystems, Inc. All rights reserved.
 *  Copyright 2000-2001 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

package sun.awt.motif;

import java.awt.*;
import java.awt.im.InputMethodHighlight;
import java.awt.im.spi.InputMethodDescriptor;
import java.awt.image.*;
import java.awt.peer.*;
import java.awt.datatransfer.Clipboard;
import java.awt.event.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.Properties;
import java.util.Map;

import sun.awt.image.ImageRepresentation;
import sun.awt.AppContext;
import sun.awt.SunToolkit;
import sun.awt.GlobalCursorManager;
 
import java.awt.dnd.DragSource;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.MouseDragGestureRecognizer;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.dnd.peer.DragSourceContextPeer;

import sun.awt.motif.X11InputMethod;
import sun.awt.X11GraphicsConfig;

import sun.java2d.loops.RasterOutputManager;

import sun.awt.motif.MDragSourceContextPeer;

public class MToolkit extends SunToolkit implements Runnable {

    final static boolean useMotifChoiceWidget = true;
    
    // the system Clipboard
    X11Clipboard clipboard;

    /*
     * Note: The MToolkit object depends on the static initializer
     * of X11GraphicsEnvironment to initialize the connection to
     * the X11 server.
     */
    static X11GraphicsConfig config =
	(X11GraphicsConfig) (GraphicsEnvironment.
			     getLocalGraphicsEnvironment().
			     getDefaultScreenDevice().
			     getDefaultConfiguration());

    public MToolkit() {
        super();
	init();
	Thread thread = new Thread(this, "AWT-Motif");
	thread.setPriority(Thread.NORM_PRIORITY + 1);
	thread.start();
    }

    public native void init();
    public native void run();

    /*
     * Create peer objects.
     */

    public ButtonPeer createButton(Button target) {
	ButtonPeer peer = new MButtonPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public TextFieldPeer createTextField(TextField target) {
	TextFieldPeer peer = new MTextFieldPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public LabelPeer createLabel(Label target) {
	LabelPeer peer = new MLabelPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public ListPeer createList(List target) {
	ListPeer peer = new MListPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public CheckboxPeer createCheckbox(Checkbox target) {
	CheckboxPeer peer = new MCheckboxPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public ScrollbarPeer createScrollbar(Scrollbar target) {
	ScrollbarPeer peer = new MScrollbarPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public ScrollPanePeer createScrollPane(ScrollPane target) {
	ScrollPanePeer peer = new MScrollPanePeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public TextAreaPeer createTextArea(TextArea target) {
	TextAreaPeer peer = new MTextAreaPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public ChoicePeer createChoice(Choice target) {
	ChoicePeer peer = null;
	if (useMotifChoiceWidget) {
	    peer = new MChoicePeer(target);
	} else {
	    peer = new MTinyChoicePeer(target);
	}
	targetCreatedPeer(target, peer);
	return peer;
    }

    public FramePeer  createFrame(Frame target) {
	FramePeer peer = new MFramePeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public CanvasPeer createCanvas(Canvas target) {
	CanvasPeer peer = new MCanvasPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public PanelPeer createPanel(Panel target) {
	PanelPeer peer = new MPanelPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public WindowPeer createWindow(Window target) {
	WindowPeer peer = new MWindowPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public DialogPeer createDialog(Dialog target) {
	DialogPeer peer = new MDialogPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public FileDialogPeer createFileDialog(FileDialog target) {
	FileDialogPeer peer = new MFileDialogPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public MenuBarPeer createMenuBar(MenuBar target) {
	MenuBarPeer peer = new MMenuBarPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public MenuPeer createMenu(Menu target) {
	MenuPeer peer = new MMenuPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public PopupMenuPeer createPopupMenu(PopupMenu target) {
	PopupMenuPeer peer = new MPopupMenuPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public MenuItemPeer createMenuItem(MenuItem target) {
	MenuItemPeer peer = new MMenuItemPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public CheckboxMenuItemPeer createCheckboxMenuItem(CheckboxMenuItem target) {
	CheckboxMenuItemPeer peer = new MCheckboxMenuItemPeer(target);
	targetCreatedPeer(target, peer);
	return peer;
    }

    public MEmbeddedFramePeer createEmbeddedFrame(MEmbeddedFrame target, int handle)
    {
        MEmbeddedFramePeer peer = new MEmbeddedFramePeer(target, handle);
        targetCreatedPeer(target, peer);
        return peer;
    }


    public FontPeer getFontPeer(String name, int style){
	/*----------*
        if (!RasterOutputManager.usesPlatformFont()) {
	    return new FontPeer() {};
        } else {
            return new MFontPeer(name, style);
        }
	*----------*/
	return new MFontPeer(name, style);
    }

    static native ColorModel makeColorModel();
    static ColorModel screenmodel;

    static ColorModel getStaticColorModel() {
	if (screenmodel == null) {
	    screenmodel = config.getColorModel ();
	}
	return screenmodel;
    }

    public ColorModel getColorModel() {
	return getStaticColorModel();
    }

    public native int getScreenResolution();
    protected native int getScreenWidth();
    protected native int getScreenHeight();

    public FontMetrics getFontMetrics(Font font) {
        if (!RasterOutputManager.usesPlatformFont()) {
            return super.getFontMetrics(font);
        } else {
            return X11FontMetrics.getFontMetrics(font);
        }
    }

    public native void sync();

    static boolean prepareScrImage(Image img, int w, int h, ImageObserver o) {
	if (w == 0 || h == 0) {
	    return true;
	}

        // Must be an OffScreenImage
        if (!(img instanceof X11Image)) {
            return true;
        }
            
	X11Image ximg = (X11Image) img;
	if (ximg.hasError()) {
	    if (o != null) {
		o.imageUpdate(img, ImageObserver.ERROR|ImageObserver.ABORT,
			      -1, -1, -1, -1);
	    }
	    return false;
	}
	ImageRepresentation ir = ximg.getImageRep();
	return ir.prepare(o);
    }

    static int checkScrImage(Image img, int w, int h, ImageObserver o) {

        if (!(img instanceof X11Image)) {
            return ImageObserver.ALLBITS;
        }
            
	X11Image ximg = (X11Image) img;
	int repbits;
	if (w == 0 || h == 0) {
	    repbits = ImageObserver.ALLBITS;
	} else {
	    repbits = ximg.getImageRep().check(o);
	}
	return ximg.check(o) | repbits;
    }

    public int checkImage(Image img, int w, int h, ImageObserver o) {
	return checkScrImage(img, w, h, o);
    }

    public boolean prepareImage(Image img, int w, int h, ImageObserver o) {
	return prepareScrImage(img, w, h, o);
    }

    public Image createImage(ImageProducer producer) {
	return new X11Image(producer);
    }

    private Constructor printControlPropsConstructor;
    private Constructor printControlAttribsConstructor;

    private class GetConstructorAction implements 
        java.security.PrivilegedAction
    {
        private Class[] args;
        public GetConstructorAction(Class[] args) {
	    this.args = args;
	}
        public Object run() {
	    Constructor constructor = null;
	    try {
	        String nm = System.getProperty("awt.printcontrol");
		if (nm != null) {
		    Class clazz = Class.forName(nm, true,
		        ClassLoader.getSystemClassLoader());
		    constructor = clazz.getConstructor(args);
		} else {
		    constructor = PSPrintControl.class.getConstructor(args);
		}
	    } catch (ClassNotFoundException e) {
	        try {
		    constructor = PSPrintControl.class.getConstructor(args);
		} catch (NoSuchMethodException ee) {
		    throw new InternalError();
		}
	    } catch (NoSuchMethodException e) {
	        try {
		    constructor = PSPrintControl.class.getConstructor(args);
		} catch (NoSuchMethodException ee) {
		    throw new InternalError();
		}
	    }
	    return constructor;
	}
    }

    public PrintJob getPrintJob(final Frame frame, final String doctitle,
				final Properties props) {
	SecurityManager security = System.getSecurityManager();
	if (security != null) {
	    security.checkPrintJobAccess();
	}
	if (printControlPropsConstructor == null) {
	    printControlPropsConstructor = (Constructor)
	        java.security.AccessController.doPrivileged(
		    new GetConstructorAction(new Class[] { Frame.class,
							   String.class,
							   Properties.class
							 }));
	}			
	sun.awt.print.PrintControl printControl = null;
	try {
	    printControl = (sun.awt.print.PrintControl)
	        printControlPropsConstructor.newInstance(
                    new Object[] { frame, doctitle, props });
	} catch (InstantiationException e) {
	    printControl = new PSPrintControl(frame, doctitle, props);
	} catch (IllegalAccessException e) {
	    printControl = new PSPrintControl(frame, doctitle, props);
	} catch (InvocationTargetException e) {
	    Throwable xcp = e.getTargetException();
	    if (xcp instanceof Error) {
	        throw (Error)xcp;
	    } else if (xcp instanceof RuntimeException) {
	        throw (RuntimeException)xcp;
	    } else {
	        printControl = new PSPrintControl(frame, doctitle, props);
	    }
	}
	return printControl.initJob();
    }
    public PrintJob getPrintJob(final Frame frame, final String doctitle,
				final JobAttributes jobAttributes,
				final PageAttributes pageAttributes) {
	SecurityManager security = System.getSecurityManager();
	if (security != null) {
	    security.checkPrintJobAccess();
	}
	if (printControlAttribsConstructor == null) {
	    printControlAttribsConstructor = (Constructor)
	        java.security.AccessController.doPrivileged(
		    new GetConstructorAction(new Class[] { Frame.class,
							   String.class,
							   JobAttributes.class,
							   PageAttributes.class
							 }));
	}			
	sun.awt.print.PrintControl printControl = null;
	try {
	    printControl = (sun.awt.print.PrintControl)
	        printControlAttribsConstructor.newInstance(
                    new Object[] { frame, doctitle, jobAttributes,
			           pageAttributes });
	} catch (InstantiationException e) {
	    printControl = new PSPrintControl(frame, doctitle, jobAttributes,
					      pageAttributes);
	} catch (IllegalAccessException e) {
	    printControl = new PSPrintControl(frame, doctitle, jobAttributes,
					      pageAttributes);
	} catch (InvocationTargetException e) {
	    Throwable xcp = e.getTargetException();
	    if (xcp instanceof Error) {
	        throw (Error)xcp;
	    } else if (xcp instanceof RuntimeException) {
	        throw (RuntimeException)xcp;
	    } else {
	        printControl = new PSPrintControl(frame, doctitle,
						  jobAttributes,
						  pageAttributes);
	    }
	}
	return printControl.initJob();
    }

    public native void beep();

    public  Clipboard getSystemClipboard() {
        SecurityManager security = System.getSecurityManager();
        if (security != null) {
	  security.checkSystemClipboardAccess();
	}
        if (clipboard == null) {
	    clipboard = new X11Clipboard("System");
	}
	return clipboard;
    }
    
    public boolean getLockingKeyState(int key) {
        if (! (key == KeyEvent.VK_CAPS_LOCK || key == KeyEvent.VK_NUM_LOCK ||
               key == KeyEvent.VK_SCROLL_LOCK || key == KeyEvent.VK_KANA_LOCK)) {
            throw new IllegalArgumentException("invalid key for Toolkit.getLockingKeyState");
        }
        return getLockingKeyStateNative(key);
    }
    
    public native boolean getLockingKeyStateNative(int key);

    public native void loadSystemColors(int[] systemColors);

    /**
     * Give native peers the ability to query the native container 
     * given a native component (eg the direct parent may be lightweight).
     */
    public static Container getNativeContainer(Component c) {
	return Toolkit.getNativeContainer(c);
    }

    public static final Object targetToPeer(Object target) {
        return SunToolkit.targetToPeer(target);
    }

    public static final void targetDisposedPeer(Object target, Object peer) {
        SunToolkit.targetDisposedPeer(target, peer);
    }

    public DragSourceContextPeer createDragSourceContextPeer(DragGestureEvent dge) throws InvalidDnDOperationException {
	return new MDragSourceContextPeer(dge);
    }

    public DragGestureRecognizer createDragGestureRecognizer(Class abstractRecognizerClass, DragSource ds, Component c, int srcActions, DragGestureListener dgl) {
	if (MouseDragGestureRecognizer.class.equals(abstractRecognizerClass))
	    return new MMouseDragGestureRecognizer(ds, c, srcActions, dgl);
	else
            return null;
    }


    /**
     * Returns a new input method adapter descriptor for native input methods.
     */
    public InputMethodDescriptor getInputMethodAdapterDescriptor() throws AWTException {
	return new X11InputMethodDescriptor();
    }

    /**
     * Returns a style map for the input method highlight.
     */
    public Map mapInputMethodHighlight(InputMethodHighlight highlight) {
	return X11InputMethod.mapInputMethodHighlight(highlight);
    }

    /**
     * Returns a new custom cursor.
     */
    public Cursor createCustomCursor(Image cursor, Point hotSpot, String name)
        throws IndexOutOfBoundsException {
        return new X11CustomCursor(cursor, hotSpot, name);
    }

    /**
     * Returns the supported cursor size
     */
    public Dimension getBestCursorSize(int preferredWidth, int preferredHeight) {
        return X11CustomCursor.getBestCursorSize(
            preferredWidth, preferredHeight);
    }

    public int getMaximumCursorColors() {
        return 2;  // Black and white.
    }

    private final static String prefix  = "DnD.Cursor.";
    private final static String postfix = ".32x32";

    protected Object lazilyLoadDesktopProperty(String name) {
	if (name.startsWith(prefix)) {
	    String cursorName = name.substring(prefix.length(), name.length()) + postfix;

	    try {
	        return Cursor.getSystemCustomCursor(cursorName);
	    } catch (AWTException awte) {
	 	System.err.println("cannot load system cursor: " + cursorName);
	
		return null;
	    }
	}

	return super.lazilyLoadDesktopProperty(name);
    }

    protected void initializeDesktopProperties() {
	desktopProperties.put("DnD.Autoscroll.initialDelay",     new Integer(50));
	desktopProperties.put("DnD.Autoscroll.interval",         new Integer(50));
	desktopProperties.put("DnD.Autoscroll.cursorHysteresis", new Integer(5));
    }

    public RobotPeer createRobot(Robot target, GraphicsDevice screen) {
	/* 'target' is ununsed for now...*/
	return new MRobotPeer(screen.getDefaultConfiguration());
    }

    public String getDefaultUnicodeEncoding() {
	return "iso-10646-ucs-2";
    }

    public GlobalCursorManager getGlobalCursorManager() {
	return new MGlobalCursorManager();
    }

    /*
     * Get the java.awt.Component object which corresponds to the native
     * window handle argument.  If no Component is found, null is returned.
     * 
     * This method is used by the accessibility software on Solaris.
     */
    public native Component getComponentFromNativeWindowHandle(int hwnd);

    /*
     * Get the native window handle which corresponds to the given peer.
     */
    private native int getNativeWindowHandleFromPeer(MComponentPeer peer);

    /*
     * Get the native window handle which corresponds to the
     * java.awt.Component object.  If the Component is lightweight,
     * its native container's native window handle is returned.
     * 
     * This method is used by the accessibility software on Solaris.
     */
    public int getNativeWindowHandleFromComponent(Component target) {
        // Find the native component's MComponentPeer, and call it for this
        java.awt.peer.ComponentPeer peer = target.getPeer();
        while ((peer != null) && !(peer instanceof MComponentPeer)) {
            target = target.getParent();
            if (target == null)
                return 0;   // No non-lightweight parent found
            peer = target.getPeer();
        }
        if (peer == null)
            return 0;   // No suitable peer found -- maybe not created yet
        return getNativeWindowHandleFromPeer((MComponentPeer)peer);
    }

} // class MToolkit
