/*
 * @(#)MWindowPeer.java	1.31 00/05/19
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.motif;

import java.util.Vector;
import java.awt.*;
import java.awt.peer.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBufferInt;
import java.awt.image.ImageObserver;
import sun.awt.image.ImageRepresentation;
import sun.awt.motif.X11InputMethod;
import sun.awt.motif.MInputMethodControl;
import sun.awt.im.*;

class MWindowPeer extends MPanelPeer implements WindowPeer {
    Insets insets = new Insets( 0, 0, 0, 0 );
    MWindowAttributes winAttr;
    static Vector allWindows = new Vector();
    int         iconWidth  = -1;
    int         iconHeight = -1;

    native void create(MComponentPeer parent);
    native void pShow();
    native void pShowModal( boolean isModal);
    native void pHide();
    native void pReshape(int x, int y, int width, int height);
    native void pDispose();
    native void pSetTitle(String title);
    public native void setState(int state);
    public native int getState();

    public native void setResizable(boolean resizable);
    native void addTextComponentNative();
    native void removeTextComponentNative();
    public native boolean hasTextComponents();
    native void pSetIMMOption(String option);
    native void pSetMenuBar(MMenuBarPeer mbpeer);
    native void setSaveUnder(boolean state);

 
    private static native void initIDs();

    static {
        initIDs();
    }

    // this function is privileged! do not change it to public!
    private static int getInset(final String name, final int def) {
        Integer tmp = (Integer) java.security.AccessController.doPrivileged(
            new sun.security.action.GetIntegerAction(name, def));
        return tmp.intValue();
    }
 
    MWindowPeer() {
	insets = new Insets(0,0,0,0);
	winAttr = new MWindowAttributes();
    }

    MWindowPeer(Window target) {
 
	this();
        init(target);

	allWindows.addElement(this);
    }

    void init( Window target ) {
	if ( winAttr.nativeDecor == true ) {
                insets.top = getInset("awt.frame.topInset", -1);
                insets.left = getInset("awt.frame.leftInset", -1);
                insets.bottom = getInset("awt.frame.bottomInset", -1);
                insets.right = getInset("awt.frame.rightInset", -1);
	}

	super.init(target);
        InputMethodManager imm = InputMethodManager.getInstance();
        String menuString = imm.getTriggerMenuString();
        if (menuString != null)
        {
          pSetIMMOption(menuString);
        }
        pSetTitle(winAttr.title);

        if (winAttr.decorations == MWindowAttributes.AWT_DECOR_ALL) {
          setResizable(winAttr.isResizable);
        }

        setSaveUnder(true);

	Font f = target.getFont();
	if (f == null) {
	    f = new Font("Dialog", Font.PLAIN, 12);
	    target.setFont(f);
	    setFont(f);
	}
	Color c = target.getBackground();
	if (c == null) {
	    target.setBackground(SystemColor.window);
	    setBackground(SystemColor.window);
	}
	c = target.getForeground();
	if (c == null) {
	    target.setForeground(SystemColor.windowText);
	    setForeground(SystemColor.windowText);
	}
    }

    protected void disposeImpl() {
	allWindows.removeElement(this);
	super.disposeImpl();
    }

    public native void toBack();

    public void toFront() {
        pShow();
    }

    /* Handle TAB and Shift-TAB events. */
    public int handleFocusTraversalEvent(KeyEvent e) {

	if (e.getKeyCode() != KeyEvent.VK_TAB ||
	    (e.getSource() instanceof TextArea))
	    return IGNORE_EVENT;

 	if ((e.getModifiers() & ~InputEvent.SHIFT_MASK) > 0)
 	    return IGNORE_EVENT;

	int id = e.getID();
 	if (id == KeyEvent.KEY_RELEASED || id == KeyEvent.KEY_TYPED)
 	    return CONSUME_EVENT;

 	if (e.isShiftDown())
 	    return FOCUS_PREVIOUS;
	else
 	    return FOCUS_NEXT;
    }

    public Insets getInsets() {
	return insets;
    }

    public void handleQuit() {
        postEvent(new WindowEvent((Window)target, WindowEvent.WINDOW_CLOSING));
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void handleIconify() {
        postEvent(new WindowEvent((Window)target, WindowEvent.WINDOW_ICONIFIED));
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void handleDeiconify() {
        postEvent(new WindowEvent((Window)target, WindowEvent.WINDOW_DEICONIFIED));
    }

    /**
     * Called to inform the Window that its size has changed and it
     * should layout its children.
     */
    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void handleResize(int width, int height) {

      // REMIND: Is this secure? Can client code subclass input method?
      if (!tcList.isEmpty() &&
	   !imList.isEmpty()){
	int i;
	for (i = 0; i < imList.size(); i++){
	  ((X11InputMethod)imList.elementAt(i)).configureStatus();
	}	
      }
        postEvent(new ComponentEvent(target, ComponentEvent.COMPONENT_RESIZED));
    }


    /**
     * DEPRECATED:  Replaced by getInsets().
     */
    public Insets insets() {
	return getInsets();
    }

    public void handleMoved(int x, int y) {
//        postEvent(new ComponentEvent(target, ComponentEvent.COMPONENT_MOVED));
    }
    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void handleActivate() {
//    postEvent(new WindowEvent((Window)target, WindowEvent.WINDOW_ACTIVATED));
    }

    // NOTE: This method may be called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    public void handleDeactivate() {
//    postEvent(new WindowEvent((Window)target, WindowEvent.WINDOW_DEACTIVATED));
    }


// relocation of Imm stuff
    private Vector imList = new Vector();
    private Vector tcList = new Vector();

    // NOTE: This method is called by privileged threads.
    //       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
    void notifyIMMOptionChange(){

      // REMIND: IS THIS SECURE??? CAN USER CODE SUBCLASS INPUTMETHODMGR???
      InputMethodManager.getInstance().notifyChangeRequest(target);
    }

    public void addInputMethod(X11InputMethod im) {
	if (!imList.contains(im))
	    imList.addElement(im);
    }

    public void removeInputMethod(X11InputMethod im) {
	if (imList.contains(im))
	    imList.removeElement(im);
    }

    public void addTextComponent(MComponentPeer tc) {
	if (tcList.contains(tc))
	    return;
	if (tcList.isEmpty()){
	    addTextComponentNative();
	    if (!imList.isEmpty()) {
	      for (int i = 0; i < imList.size(); i++) {
		((X11InputMethod)imList.elementAt(i)).reconfigureXIC((MInputMethodControl)this);
	      }
	    }
	}
	tcList.addElement(tc);

    }

    public void removeTextComponent(MComponentPeer tc) {
	if (!tcList.contains(tc))
	    return;
	tcList.removeElement(tc);
	if (tcList.isEmpty()){
	    removeTextComponentNative();
	    if (!imList.isEmpty()) {
	      for (int i = 0; i < imList.size(); i++) {
		((X11InputMethod)imList.elementAt(i)).reconfigureXIC((MInputMethodControl)this);
	      }
	    }
	}
    }


}

