/*
 * @(#)PSPrintJob.java	1.21 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.motif;

import java.awt.*;
import java.io.*;

/**
 * A class which initiates and executes a PostScript print job.
 *
 * @version 	1.21 02/02/00
 * @author 	Amy Fowler
 * @author 	Richard Berlin
 */
public class PSPrintJob extends PrintJob {
    private PrintStream		os;
    private PSPrintStream	stream;
    private boolean             endInvoked = false;

    PSPrintControl              control;

    String  	 title;
    Dimension 	 pageDimension;
    int		 pageResolution;
    boolean 	 lastFirst = false;
    int		 pageCount = 0;

    /**
     * Creates a print job configured to pipe PostScript output
     * to a PrintStream. The print job will call back on the PSPrintControl
     * when it has finished printing.
     */
    public PSPrintJob(PrintStream ps, String title, Dimension pageDim,
		      int pageRes, Dimension paperDim, 
		      PSPrintControl control) {
	this.title = title;
	pageDimension = pageDim;
	pageResolution = pageRes;
	os = ps;
	this.control = control;
	stream = new PSPrintStream(this, os, title, paperDim, lastFirst);
	stream.sendProlog();
    }

    /**
     * Gets a Graphics object that will draw to the next page.
     * The page is flushed to the printer when the graphics
     * object is disposed.
     */
    public Graphics getGraphics() {
	stream.startPage(++pageCount);
	return new PageGraphics(stream, pageCount);
    }

    /**
     * Returns the dimensions of the page in pixels.
     */
    public Dimension getPageDimension() {
	return pageDimension;
    }

    /**
     * Returns the resolution of the page in pixels per inch.
     */
    public int getPageResolution() {
	return pageResolution;
    }

    /**
     * Returns whether the last page will be printed first.
     */
    public boolean lastPageFirst() {
	return lastFirst;
    }

    /**
     * Ends the print job and does cleanup.
     */
    public void end() {
        if (!endInvoked) {
	    stream.close(pageCount);
	    os.close();
	    os = null;

	    if (control != null) {
	        control.finishedPrinting(this);
	    }

	    endInvoked = true;
	}
    }
}

class PageGraphics extends PSGraphics {
    public PageGraphics(PSPrintStream stream, int pageNumber) {
	super(stream, pageNumber);
    }

    protected void disposeImpl() {
        stream.endPage();
	super.disposeImpl();
    }
}
