/*
 * @(#)UPageDialog.java	1.4 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.motif;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Component;
import java.awt.Container;
import java.awt.Choice;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;

import java.awt.print.PageFormat;
import java.awt.print.Paper;

import java.util.ResourceBundle;
import java.util.MissingResourceException;

 /**
  * A class which implements a modal page format setup dialog.
  *
  */
class UPageDialog extends Dialog implements ActionListener, ItemListener {

    public final static int ORIENT_N = 3;  // Number of orientation states;

    public final static int WAITING = 0;   //  waiting for user
    public final static int OK = 1;        // user hit OK
    public final static int CANCELLED = 2; // user cancelled dialog

    private int setupStatus = WAITING;     // current dialog status

    private static ResourceBundle messageRB;

    private int orientation;
    private int oldOrientation;

    private PSPaper[] allPapers;
    private int paperSelection;
    private int oldPaperSelection;
    private Choice paperChoices;

    private CheckboxGroup orientCBG;
    private Checkbox orientCB[] = new Checkbox[ORIENT_N];

    private Button okButton;
    private Button cancelButton;

     
    public UPageDialog(Frame parent, PageFormat format) {
        super(parent, getMsg("dialog.title", "Page Setup"), true);

        Insets noInsets = new Insets(0, 0, 0, 0);
        Insets topInsets = new Insets(10, 0, 0, 0);
        Insets leftInsets = new Insets(0, 40, 0, 0);

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        setLayout(gridbag);

        Font font = getFont();
        int fontSize = font == null ? 12 : font.getSize();
        Font boldFont = new Font("Dialog", Font.BOLD, fontSize);
        setFont(boldFont);

        allPapers = PSPaper.getAllPapers();
        paperChoices = new Choice();

        for( int i = 0; i < allPapers.length; i ++) {
            paperChoices.addItem(allPapers[i].getName());
        }

        Label label = 
              new Label(getMsg("label.papersize","Paper Size:"), Label.RIGHT);
        addComponent(label, this, gridbag, gbc, 0, 0, 1, 0, 1,
                     GridBagConstraints.EAST,
                     GridBagConstraints.NONE,
                     topInsets);
        addComponent(paperChoices, this, gridbag,
                     gbc, 0, 2, 4, 1, 0,
                     GridBagConstraints.CENTER,
                     GridBagConstraints.NONE, noInsets);
        paperChoices.addItemListener(this);

        findPaper(format);
        oldPaperSelection = paperSelection;
        paperChoices.select(paperSelection);

        label = new Label(getMsg("label.orientation", "Orientation:"),
                          Label.RIGHT);
        addComponent(label, this,
                     gridbag, gbc, 0, 4, 1, 0, 1,
                     GridBagConstraints.EAST,
                     GridBagConstraints.NONE,
                     topInsets);

        orientCBG = new CheckboxGroup();

        orientCB[PageFormat.PORTRAIT] =
                  new Checkbox(getMsg("checkbox.portrait", "Portrait"),
                               orientCBG, true);
        addComponent(orientCB[PageFormat.PORTRAIT], this,
                     gridbag, gbc, 0, 5, 2, 0, 1,
                     GridBagConstraints.WEST,
                     GridBagConstraints.NONE, leftInsets);
        orientCB[PageFormat.PORTRAIT].addItemListener(this);

        orientCB[PageFormat.LANDSCAPE] =
                  new Checkbox(getMsg("checkbox.landscape", "Landscape"),
                               orientCBG, false);
        addComponent(orientCB[PageFormat.LANDSCAPE], this,
                     gridbag, gbc, 0, 6, 2, 0, 1,
                     GridBagConstraints.WEST,
                     GridBagConstraints.NONE, leftInsets);
        orientCB[PageFormat.LANDSCAPE].addItemListener(this);

        orientCB[PageFormat.REVERSE_LANDSCAPE] =
                  new Checkbox(getMsg("checkbox.reverselandscape",
                                      "Reverse Landscape"),
                               orientCBG, false);
        addComponent(orientCB[PageFormat.REVERSE_LANDSCAPE], this,
                     gridbag, gbc, 0, 7, 2, 0, 1,
                     GridBagConstraints.WEST,
                     GridBagConstraints.NONE, leftInsets);
        orientCB[PageFormat.REVERSE_LANDSCAPE].addItemListener(this);

        if (format != null) {
            orientation = format.getOrientation();
        }
        else {
            orientation = PageFormat.PORTRAIT;
        }

        oldOrientation = orientation;
        orientCBG.setCurrent(orientCB[orientation]);

        Panel panel = new Panel();
        panel.setLayout(new FlowLayout(FlowLayout.CENTER, 28, 10));
        addComponent(panel, this,
                     gridbag, gbc, 0, 12, 4, 1, 0,
                     GridBagConstraints.CENTER,
                     GridBagConstraints.NONE,
                     noInsets);

        panel.add(okButton = new Button(getMsg("button.ok", "OK")));
        panel.add(cancelButton = new Button(getMsg("button.cancel", "Cancel")));
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setupStatus = CANCELLED;
                // restore old settings on cancel
                orientation = oldOrientation;
                paperSelection = oldPaperSelection;
                setVisible(false);
            }
        });

        pack();
        setResizable(false);
        setModal(true);
    }

    public void itemStateChanged(ItemEvent e) {
        if (e.getSource() instanceof Checkbox) {     // orientation
            for ( int i=0; i < ORIENT_N; i++) {
                if (e.getSource() == orientCB[i]) {
                    orientation = i;
                    return;
                }
            }
        } else if (e.getSource() instanceof Choice) {  // Paper choice
             paperSelection = paperChoices.getSelectedIndex();
             return;
        }
    }


    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == okButton) {
            setupStatus = OK;
            setVisible(false);
        } else if (e.getSource() == cancelButton) {
            setupStatus = CANCELLED;
            // restore old settings on cancel
            orientation = oldOrientation;
            paperSelection = oldPaperSelection;
            setVisible(false);
        }
    }

    /**
     * Return the setup dialogs current state, either
     * WAITING, OK, or CANCEL.
     */
    public int getSetupStatus() {
        return setupStatus;
    }

    /**
     * returns the currently selected paper
     */
    public PSPaper getPaper() {
        return allPapers[paperSelection];
    }

    /**
     * returns the current orientation
     */
    public int getOrientation() {
        return orientation;
    }

    /**
     * find a paper that matches the format
     */
    private void findPaper(PageFormat format) {

        paperSelection = 0;
        if (format == null) {
            return;
        }

        int dPW;             // paper width delta
        int dPH;             // paper height delta
        int dIX;             // imageable area X delta
        int dIY;             // imageable area Y delta
        int dIW;             // imageable area width delta
        int dIH;             // imageable area height delta
        int pTol = 2;        // paper tolerance
        int iTol = 5;        // imageable area tolerance

        for (int i = 0; i < allPapers.length; i++) {
            Paper p = allPapers[i];
            if (format.getOrientation() == PageFormat.PORTRAIT) {
                dPW = (int)Math.abs(format.getWidth()  - p.getWidth());
                dPH = (int)Math.abs(format.getHeight() - p.getHeight());
                dIX = (int)Math.abs(format.getImageableX() - p.getImageableX());
                dIY = (int)Math.abs(format.getImageableY() - p.getImageableY());
                dIW = (int)Math.abs(format.getImageableWidth() - p.getImageableWidth());
                dIH = (int)Math.abs(format.getImageableHeight() - p.getImageableHeight());
            }
            else {
                dPW = (int)Math.abs(format.getHeight() - p.getWidth());
                dPH = (int)Math.abs(format.getWidth() - p.getHeight());
                dIX = (int)Math.abs(format.getImageableY() - p.getImageableX());
                dIY = (int)Math.abs(format.getImageableX() - p.getImageableY());
                dIW = (int)Math.abs(format.getImageableHeight() - p.getImageableWidth());
                dIH = (int)Math.abs(format.getImageableWidth() - p.getImageableHeight());
            }

             /* check for match of paper size AND
              * imageable area size AND
              * imageable area offset
              */
            if (dPW <= pTol && dPH <= pTol &&
                dIX <= iTol && dIY <= iTol &&
                dIW <= iTol && dIH <= iTol) {
                  paperSelection = i;
                  return;
            }
        }

        for (int i = 0; i < allPapers.length; i++) {
            if (format.getOrientation() == PageFormat.PORTRAIT) {
                dPW = (int)Math.abs(format.getWidth() - allPapers[i].getWidth());
                dPH = (int)Math.abs(format.getHeight() - allPapers[i].getHeight());
            }
            else {
                dPW = (int)Math.abs(format.getHeight() - allPapers[i].getWidth());
                dPH = (int)Math.abs(format.getWidth() - allPapers[i].getHeight());
            }

            // check for match of only paper size
            if (dPW <= pTol && dPH <= pTol) {
                paperSelection = i;
                return;
            }
        }
    }


    /**
     * Initialize ResourceBundle
     */
    static void initResource() {
        try {
            messageRB =
              ResourceBundle.getBundle("sun.awt.motif.resources.pagedialog");
        } catch (MissingResourceException e) {
            messageRB = null;
        }
    }

    /**
     * get message string from resource
     */
    static String getMsg(String key, String defaultStr) {
        if (messageRB == null) {
            initResource();
        }

        if (messageRB == null) {
            return defaultStr;
        }

        try {
            return messageRB.getString(key);
        } catch (MissingResourceException e) {
            return defaultStr;
        }
    }

    /**
     * add component to dialog with specified gridbag constraints.
     */
    private void addComponent(Component comp,
                              Container cont,
                              GridBagLayout gridbag,
                              GridBagConstraints gbc,
                              int gridx, int gridy, int gridw,
                              int weightx, int weighty,
                              int anchor, int fill, Insets insets) {
        gbc.gridx = gridx;
        gbc.gridy = gridy;
        gbc.gridwidth = gridw;
        gbc.anchor = anchor;
        gbc.fill = fill;
        gbc.insets = insets;
        gbc.weightx = weightx;
        gbc.weighty = weighty;
        gridbag.setConstraints(comp, gbc);
        cont.add(comp);
    }

}
   /**
    * Extend Paper to include a name
    *
    */
    class PSPaper extends Paper {

        private String mName;

        public PSPaper(String name, double pWidth, double pHeight,
                        double iX, double iY, double iWidth, double iHeight) {

            mName = name;
            setSize(pWidth, pHeight);
            setImageableArea(iX, iY, iWidth, iHeight);
        }

        public String getName() {
            return mName;
        }

        public static PSPaper[] getAllPapers() {
            PSPaper []papers = new PSPaper[7];

            papers[0] = new PSPaper(UPageDialog.getMsg("paper.usletter","US Letter"),
                                 612,  792, 18.0,  9.0, 576.0, 776.0);
            papers[1] = new PSPaper(UPageDialog.getMsg("paper.uslettersmall",
                                           "US Letter Small"),
                                 612,  792, 30.0, 31.0, 552.0, 730.0);
            papers[2] = new PSPaper(UPageDialog.getMsg("paper.uslegal","US Legal"),
                                 612, 1008, 18.0, 15.0, 576.0, 978.0);
            papers[3] = new PSPaper(UPageDialog.getMsg("paper.uslegalsmall",
                                           "US Legal Small"),
                                 612, 1008, 64.0, 54.0, 484.0, 900.0);
            papers[4] = new PSPaper(UPageDialog.getMsg("paper.a4","A4"),
                                 595,  842, 17.0, 22.0, 560.0, 797.0);
            papers[5] = new PSPaper(UPageDialog.getMsg("paper.a4small","A4 Small"),
                                 595,  842, 28.0, 30.0, 538.0, 781.0);
            papers[6] = new PSPaper(UPageDialog.getMsg("paper.b5","B5"),
                                 499,  708, 17.0,  3.0, 482.0, 705.0);

            return papers;
        }
    }
