/*
 * @(#)X11CustomCursor.java	1.13 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.motif;

import sun.awt.CustomCursor;
import java.awt.*;
import java.awt.image.*;
import sun.awt.image.ImageRepresentation;

/**
 * A class to encapsulate a custom image-based cursor.  
 *
 * @see Component#setCursor
 * @version 	1.13 02/02/00
 * @author 	Thomas Ball
 */
public class X11CustomCursor extends CustomCursor {

    public X11CustomCursor(Image cursor, Point hotSpot, String name) 
            throws IndexOutOfBoundsException {
        super(cursor, hotSpot, name);
    }

    /**
     * Returns the supported cursor size
     */
    public static Dimension getBestCursorSize(
        int preferredWidth, int preferredHeight) {

        // Fix for bug 4212593 The Toolkit.createCustomCursor does not 
        //                     check absence of the image of cursor 
        // We use XQueryBestCursor which accepts unsigned ints to obtain
        // the largest cursor size that could be dislpayed 
        Dimension d = new Dimension(Math.abs(preferredWidth), Math.abs(preferredHeight));

        queryBestCursor(d);
        return d;
    }

    private static native void queryBestCursor(Dimension d);

    // convenient function used only in createNativeCursor
    private boolean showForegroundColor(int ocolor, int fcolor, int bcolor) {
	int ro = (ocolor >> 16) & 0x000000ff;
	int go = (ocolor >>  8) & 0x000000ff;
	int bo = (ocolor >>  0) & 0x000000ff;

	int rf = (fcolor >> 16) & 0x000000ff;
	int gf = (fcolor >>  8) & 0x000000ff;
	int bf = (fcolor >>  0) & 0x000000ff;

	int rb = (bcolor >> 16) & 0x000000ff;
	int gb = (bcolor >>  8) & 0x000000ff;
	int bb = (bcolor >>  0) & 0x000000ff;

	return ( (ro-rf)*(ro-rf) + (go-gf)*(go-gf) + (bo-bf)*(bo-bf) 
	       <= (ro-rb)*(ro-rb) + (go-gb)*(go-gb) + (bo-bb)*(bo-bb) ) ? 
	       true : false; 
    }

    protected void createNativeCursor(Image im, int[] pixels, int width, int height,
                                      int xHotSpot, int yHotSpot) {
	final int NUMBER = 20;
	int arrayLen = NUMBER;

	int fc = 0x000000;
	int bc = 0xffffff;
	int colors[] = new int[arrayLen];
	int counts[] = new int[arrayLen];
	int numColors = 0;
	for (int i = 0; i < width; i++) {
	    for (int j = 0; j < height; j++) {
	        int ip = j*width + i;

	        // do not include transparent color
                if ((pixels[ip] & 0xff000000) == 0) continue;

		int clr = (pixels[ip] & 0x00ffffff);
		boolean found = false;
		int k;
		for (k = 0; k < numColors; k++) {
		    if (colors[k] == clr) {
			found = true;
			break;
		    }
		}
		if (found) {
		    counts[k] ++; 
		} else {
		    if (numColors == arrayLen) {
			int oldCor[] = colors;
			int oldCnt[] = counts;
			
			colors = new int[arrayLen + NUMBER];		
			counts = new int[arrayLen + NUMBER];		
			System.arraycopy(oldCor, 0, colors, 0, arrayLen);	
			System.arraycopy(oldCnt, 0, counts, 0, arrayLen);	
			arrayLen += NUMBER;
		    }
		    colors[numColors] = clr;
		    counts[numColors] ++; 
		    numColors ++;
		}
	    }
	}

/*
	for (int j = 0; j < height; j++) {
	    for (int i = 0; i < width; i++) {
	        int ip = j*width + i;
		int clr = (pixels[ip] & 0x00ffffff);
		boolean found = false;
		int k;
		for (k = 0; k < numColors; k++) {
		    if (colors[k] == clr) {
			found = true;
			break;
		    }
		}
	        if (found) {
		   System.out.print(" "+k);
		} else {
		   System.out.print(" a");
		}
            }
	    System.out.println(" ");
	}
	System.out.println(" ");

	System.out.println("Total color is "+numColors);
 */
	// find the two top counts
	int i1 = 0; 
	int i2 = 1;
	int c1 = counts[0];
	int c2 = counts[1];

	if (c1 < c2) {
	   i1 = 1;
	   i2 = 0;
	   int t = c1;
	   c1 = c2;
	   c2 = t;
	}
	for (int i = 2; i < numColors; i++) {
	    if (counts[i] > c1) {
	        i2 = i1;
	        c2 = c1;
		c1 = counts[i];
		i1 = i;
	    } else if (counts[i] > c2) {
		c2 = counts[i];
		i2 = i;
	    }
	}

	fc = colors[i1];
	bc = colors[i2];
        /*
	System.out.println("Foreground color is "+fc + " r="+ 
		((fc>>16) & 0x0000ff)  
		+ " g=" + ((fc>>8) & 0x0000ff) 
		+ " b=" + ((fc>>0) & 0x0000ff) );
	System.out.println("Background color is "+bc + " r="+ 
		((bc>>16) & 0x000000ff)  
		+ " g=" + ((bc>>8) & 0x000000ff) 
		+ " b=" + ((bc>>0) & 0x000000ff) );
          */

	// On Solaris 2.5.x, the above code for cursor of any size runs fine
        // but on Solaris 2.6, the width of a cursor has to be 8 divisible, 
        //   otherwise, the cursor could be displayed as garbaged.
  	// To work around the 2.6 problem, the following code pads any cursor 
	//   with a transparent area to make a new cursor of width 8 multiples.
	// --- Bug 4148455
	int wNByte = (width + 7)/8;
	int tNByte = wNByte * height;
        byte[] xorMask = new byte[tNByte];
        byte[] andMask = new byte[tNByte];

	for (int i = 0; i < width; i++) {
            int omask = 1 << (i % 8);
	    for (int j = 0; j < height; j++) {
	        int ip = j*width + i;
		int ibyte = j*wNByte + i/8;

                if ((pixels[ip] & 0xff000000) != 0) {
                    andMask[ibyte] |= omask;
                }

                // if ((pixels[ip] & 0xffffff) == 0) {
                if (showForegroundColor(pixels[ip], fc, bc)) {
                    xorMask[ibyte] |= omask;
	        }
	    }
	}

        createCursor(xorMask, andMask, 8*wNByte, height, fc, bc, xHotSpot, yHotSpot);
    }

    private native void createCursor(byte[] xorMask, byte[] andMask, 
                                     int width, int height, 
                                     int fcolor, int bcolor, 
                                     int xHotSpot, int yHotSpot);

    static {
        cacheInit();
    }

    private native static void cacheInit();
}
