/*
 * @(#)X11InputMethodDescriptor.java	1.7 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */


package sun.awt.motif;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.im.spi.InputMethod;
import java.awt.im.spi.InputMethodDescriptor;
import java.security.AccessController;
import java.util.Locale;
import sun.security.action.GetPropertyAction;

/**
 * Provides sufficient information about an input method
 * to enable selection and loading of that input method.
 * The input method itself is only loaded when it is actually used.
 * 
 * @since JDK1.3
 */

class X11InputMethodDescriptor implements InputMethodDescriptor {

    private static Locale locale;

    X11InputMethodDescriptor() {
        locale = getSupportedLocale();
    }

    /**
     * @see java.awt.im.spi.InputMethodDescriptor#getAvailableLocales
     */
    public Locale[] getAvailableLocales() {
        Locale[] locales = {locale};
        return locales;
    }
    
    /**
     * @see java.awt.im.spi.InputMethodDescriptor#hasDynamicLocaleList
     */
    public boolean hasDynamicLocaleList() {
        return false;
    }
    
    /**
     * @see java.awt.im.spi.InputMethodDescriptor#getInputMethodDisplayName
     */
    public synchronized String getInputMethodDisplayName(Locale inputLocale, Locale displayLanguage) {
        // We ignore the input locale.
        // When displaying for the default locale, rely on the localized AWT properties;
        // for any other locale, fall back to English.
        String name = "System Input Methods";
        if (Locale.getDefault().equals(displayLanguage)) {
            name = Toolkit.getProperty("AWT.HostInputMethodDisplayName", name);
        }
        return name;
    }
    
    /**
     * @see java.awt.im.spi.InputMethodDescriptor#getInputMethodIcon
     */
    public Image getInputMethodIcon(Locale inputLocale) {
        return null;
    }
    
    /**
     * @see java.awt.im.spi.InputMethodDescriptor#createInputMethod
     */
    public InputMethod createInputMethod() throws Exception {
        return new X11InputMethod();
    }

    /**
     * returns supported locale. Currently this method returns the locale in which 
     * the VM is started since Solaris doesn't provide a way to determine the login locale.
     */
    static Locale getSupportedLocale() {
        if (locale == null) {
	   String language =
	       (String) AccessController.doPrivileged(
	                   new GetPropertyAction("user.language","en"));
	   String country =
	       (String) AccessController.doPrivileged(
	                   new GetPropertyAction("user.region",""));

           /* The user.region property may be of the form country, country_variant,
            * or _variant.  Since the Locale constructor takes the country value as
	    * an unparsed literal, and we don't want to change that behavior, we
	    * must premunge it here into country and variant.  */
	   String variant = "";
	   int i = country.indexOf('_');
	   if (i >= 0) {
	       variant = country.substring(i+1);
	       country = country.substring(0, i);
	   }
	   locale = new Locale(language, country, variant);
	}
	return locale;
    }
}
