/*
 * @(#)TinyCheckboxPeer.java	1.12 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.tiny;

import java.awt.*;
import java.awt.peer.*;

public class TinyCheckboxPeer extends TinyComponentPeer implements CheckboxPeer {
    static final int SIZE = 19;
    static final int BORDER = 4;
    static final int SIZ = SIZE - BORDER*2;

    boolean down;

    /**
     * Create the checkbox.
     */
    public TinyCheckboxPeer(Checkbox target) {
	winCreate((TinyComponentPeer)target.getParent().getPeer());
	init(target);
    }

    /**
     * Minimum size.
     */
    public Dimension getMinimumSize() {
	String lbl = ((Checkbox)target).getLabel();
	if (lbl != null) {
	    FontMetrics fm = getFontMetrics(target.getFont());
	    return new Dimension(SIZE + fm.stringWidth(lbl) + TEXT_XPAD/2, 
		 Math.max(fm.getAscent() + fm.getMaxDescent() + TEXT_YPAD, SIZE));
	}
	return new Dimension(SIZE, SIZE);
    }

    /**
     * Paint the checkbox.
     */
    public void paintComponent(Graphics g) {
	int x = BORDER;
	int y = ((height - SIZE) / 2) + BORDER;
	Color colors[] = getGUIcolors();

	Checkbox cb = (Checkbox)target;

	g.setColor(colors[BACKGROUND_COLOR]);
	g.fillRect(0,0,width,height);

	g.setColor((down || cb.getState()) 
		   ? colors[SHADOW_COLOR] : colors[BACKGROUND_COLOR]);
	if (cb.getCheckboxGroup() != null) {
	    g.fillOval(x, y, SIZ, SIZ);
	    draw3DOval(g, colors, x, y, SIZ, SIZ, !(down || cb.getState()));
	    if (cb.getState()) {
		g.setColor(colors[FOREGROUND_COLOR]);
		g.fillOval(x + 3, y + 3, SIZ - 6, SIZ - 6);
	    }
	} else {
	    g.fillRect(x, y, SIZ, SIZ);
	    draw3DRect(g, colors, x, y, SIZ, SIZ, !(down || cb.getState()));
	    if (cb.getState()) {
		g.setColor(colors[FOREGROUND_COLOR]);
		g.drawLine(x+1, y+1, x+SIZ-1, y+SIZ-1);
		g.drawLine(x+1, y+SIZ-1, x+SIZ-1, y+1);
	    }
	}

	g.setColor(colors[FOREGROUND_COLOR]);
	String lbl = cb.getLabel();
	if (lbl != null) {
	    // REMIND: align
	    g.setFont(cb.getFont());
	    FontMetrics fm = g.getFontMetrics();
	    g.drawString(lbl, SIZE,
			 (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
	}
	target.paint(g);
    }	

    /**
     * Handle events.
     */
    public boolean handleWindowEvent(Event evt) {
	if (!target.isEnabled()) {
	    return false;
	}

	switch (evt.id) {
	case Event.MOUSE_DOWN:
	    down = true;
	    repaint();
	    return true;

	case Event.MOUSE_DRAG:
	    if ((evt.x < 0) || (evt.x > width) || (evt.y < 0) || (evt.y > height)) {
		if (down) {
		    down = false;
		    repaint();
		}
	    } else if (!down) {
		down = true;
		repaint();
	    }
	    return true;

	case Event.MOUSE_UP:
	    if (down) {
		Checkbox cb = (Checkbox)target;
		boolean state = !cb.getState();
		down = false;
		cb.setState(state);
		targetEvent(Event.ACTION_EVENT, new Boolean(state));
	    }
	    return true;
	}
	return false;
    }
   
    public void setLabel(String label) {
	repaint();
    }
    public void setState(boolean state) {
	repaint();
    }
    public void setCheckboxGroup(CheckboxGroup g) {
	repaint();
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	    return getMinimumSize();
    }

}
