/*
 * @(#)TinyScrollbarPeer.java	1.14 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.tiny;

import java.awt.*;
import java.awt.peer.*;

class TinyScrollbarPeer extends TinyComponentPeer implements ScrollbarPeer, TinyScrollbarClient {
    TinyVerticalScrollbar tsb;
    
    /**
     * Create a scrollbar.
     */
    TinyScrollbarPeer(Scrollbar target) {
	winCreate((TinyComponentPeer)target.getParent().getPeer());
	init(target);

	if (target.getOrientation() == Scrollbar.VERTICAL) {
	    tsb = new TinyVerticalScrollbar();
	} else {
	    tsb = new TinyHorizontalScrollbar();
	}
	tsb.sb = this;
	tsb.min = target.getMinimum();
	tsb.max = target.getMaximum();
	tsb.vis = target.getVisibleAmount();
	tsb.val = target.getValue();
	tsb.line = target.getLineIncrement();
	tsb.page = target.getPageIncrement();
    }

    /**
     * Compute the minimum size for the scrollbar.
     */
    public Dimension getMinimumSize() {
	Scrollbar sb = (Scrollbar)target;
	return (sb.getOrientation() == Scrollbar.VERTICAL) ? 
	    new Dimension(15, 50) : new Dimension(50, 15);
    }

    /**
     * Paint the scrollbar.
     */
    public void paintComponent(Graphics g) {
	Scrollbar sb = (Scrollbar)target;
	Color colors[] = getGUIcolors();
	g.setColor(colors[BACKGROUND_COLOR]);
	g.fillRect(0, 0, width, height);
	tsb.paint(g, colors, width, height);
    }

    /**
     * The value has changed.
     */
    public void notifyValue(Object obj, int id, int val) {
	Scrollbar sb = (Scrollbar)target;
	sb.setValue(val);
	targetEvent(id, new Integer(val));
    }

    /**
     * Handle an event.
     */
    public boolean handleWindowEvent(Event evt) {
	Scrollbar sb = (Scrollbar)target;
	return tsb.handleWindowEvent(evt, width, height);
    }

    public void setValue(int value) {
	tsb.val = value;
	repaint();
    }
    public void setValues(int value, int visible, int minimum, int maximum) {
	tsb.min = minimum;
	tsb.max = maximum;
	tsb.vis = visible;
	tsb.val = value;
	repaint();
    }
    public void setLineIncrement(int l) {
	tsb.line = l;
    }
    public void setPageIncrement(int p) {
	tsb.page = p;
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	    return getMinimumSize();
    }

}
