/*
 * @(#)TinyWindowPeer.java	1.14 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.tiny;

import java.util.Vector;
import java.awt.*;
import java.awt.peer.*;
import java.awt.event.*;

class TinyWindowPeer extends TinyPanelPeer implements WindowPeer {
    TinyFrame	frame;

    TinyWindowPeer() {
    }

    TinyWindowPeer(Window target) {
	frame = new TinyWindowFrame(this, null, target.getWarningString(), false);
	init(target);
    }

    void init(Window target) {
	super.init(target);

	if (target.getFont() == null) {
	    target.setFont(new Font("Dialog", Font.PLAIN, 10));
	}
	Color c = target.getBackground();
	if (c == null) {
	    target.setBackground(c = Color.lightGray);
	}
	winBackground(c);
	c = target.getForeground();
	if (c == null) {
	    target.setForeground(c = Color.black);
	}
    }

    public void toFront() {
    	frame.winFront();
    }
    public void toBack() {
    	frame.winBack();
    }

    /* Handle TAB and Shift-TAB events. */
    public int handleFocusTraversalEvent(KeyEvent e) {

	if (e.getKeyCode() != KeyEvent.VK_TAB ||
	    (e.getSource() instanceof TextArea))
	    return IGNORE_EVENT;

 	if ((e.getModifiers() & ~InputEvent.SHIFT_MASK) > 0)
 	    return IGNORE_EVENT;

	int id = e.getID();
 	if (id == KeyEvent.KEY_RELEASED || id == KeyEvent.KEY_TYPED)
 	    return CONSUME_EVENT;

 	if (e.isShiftDown())
 	    return FOCUS_PREVIOUS;
	else
 	    return FOCUS_NEXT;
    }

    public Insets getInsets() {
	return frame.insets();
    }

    public void setBounds(int x, int y, int w, int h) {
	reshape(x, y, w, h);
    }

    /**
     * DEPRECATED:  Replaced by setBounds(int, int, int, int).
     */
    public void reshape(int x, int y, int w, int h) {
	frame.winReshape(x, y, w, h);
    }

    public void show() {
	frame.winShow();
    }
    public void hide() {
	frame.winHide();
    }

    /**
     * DEPRECATED:  Replaced by getInsets().
     */
    public Insets insets() {
	return getInsets();
    }

}

