/*
 * @(#)context_md_linux.h	1.6 00/03/16
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Linux-dependent thread context.  When we move to supporting only
 * 2.2 based systems, we should start using <sys/ucontext.h>, and use
 * the kernel calls getcontext, setcontext etc. just like we do on
 * Solaris.
 */

#ifndef _JAVASOFT_LINUX_CONTEXT_MD_H_
#define _JAVASOFT_LINUX_CONTEXT_MD_H_

#include <errno.h>
#include <setjmp.h>
#include <signal.h>

typedef unsigned int uint_t;

/*-
 * ucontext_t does not exist on linux so we have come up with our
 * version of ucontext_t using jmpbufs and some extras to mimic the
 * limited functionality required for Java threads.
 */

typedef struct gcontext {
    sigjmp_buf	jmpbuf;
#if defined(__i386__)
    char floatbuf[108];
#elif defined(__powerpc__) || defined(__sparc__)
    unsigned int pc;
    unsigned int arg;
#endif
    struct {
	unsigned char *ss_sp;
	unsigned char *ss_size;
    } uc_stack;
} gcontext_t;

/*
 * A linux context.  Everything else is on the stack.
 */

typedef struct {
    unsigned int unix_errno;
    stackp_t current_sp; /* [sbb] leave this in? */
    gcontext_t gcontext;
} context_t;

/* Save state of the current content and call rescheduler */

/* [sbb] added "extern" -- it's the right thing to do */
extern void reschedule(void);

#if defined(__i386__)
#define getcontext(gcontextp)                                  \
({                                                             \
    char *fdata = (char*) (gcontextp)->floatbuf;               \
    __asm__ ("fsave %0"::"m" (*fdata));                        \
    sigsetjmp((gcontextp)->jmpbuf, -1);                        \
    (gcontextp)->uc_stack.ss_sp =                              \
        (unsigned char*) (gcontextp)->jmpbuf->__jmpbuf[JB_SP]; \
    (gcontextp)->uc_stack.ss_size = 0;                         \
})

#define setcontext(gcontextp)                                  \
({                                                             \
    char * fdata;                                              \
    fdata = (char*) (gcontextp)->floatbuf;                     \
    __asm__ ("frstor %0"::"m" (*fdata));                       \
   siglongjmp((gcontextp)->jmpbuf, 1);                         \
})

#define	switchContext(contextp) {			\
    errno = (contextp)->unix_errno;			\
    setcontext(&(contextp)->gcontext);			\
}


#define yieldContext(contextp)   {				\
    unsigned char *dummy;                                       \
    char *fdata;                                                \
    gcontext_t * ucp = &((contextp)->gcontext);                 \
    int ret;                                                    \
    (contextp)->current_sp = &dummy; 				\
    fdata = (char*) (ucp)->floatbuf;                            \
    __asm__ ("fsave %0"::"m" (*fdata));                         \
    ret=sigsetjmp(ucp->jmpbuf, -1);  		                \
    if (ret == 0) {                                             \
        ucp->uc_stack.ss_sp =                                   \
          (unsigned char*)ucp->jmpbuf->__jmpbuf[JB_SP];         \
        ucp->uc_stack.ss_size = 0;                              \
        (contextp)->unix_errno = errno;			        \
        reschedule();						\
    }                                                           \
}

#elif defined(__powerpc__)
#define getcontext(gcontextp) 				           \
({ 							           \
    sigsetjmp((gcontextp)->jmpbuf, -1);  		           \
    (ucontextp)->uc_stack.ss_sp =                                  \
        (unsigned char*) (gcontextp)->jmpbuf->__jmpbuf->__jmpbuf[JB_GPR1]; \
    (gcontextp)->uc_stack.ss_size = 0;                             \
})

#define setcontext(gcontextp)                              \
({                                                         \
     siglongjmp((gcontextp)->jmpbuf,(int) gcontextp);         \
})

#define	switchContext(contextp) {			\
    errno = (contextp)->unix_errno;			\
    setcontext(&(contextp)->gcontext);			\
}

#define yieldContext(contextp)   {				\
    unsigned char *dummy;                                       \
    int ret;                                                    \
    gcontext_t * ucp = &((contextp)->gcontext);                 \
    (contextp)->current_sp = &dummy; 				\
    ret=sigsetjmp(ucp->jmpbuf, -1);  		        \
    if (ret == 0) {                                             \
        ucp->uc_stack.ss_sp =                           \
          (unsigned char*)ucp->jmpbuf->__jmpbuf->__jmpbuf[JB_GPR1]; \
        ucp->uc_stack.ss_size = 0;                         \
        (contextp)->unix_errno = errno;			\
        reschedule();						\
    }                                                           \
}
#elif defined(__sparc__)
#define getcontext(gcontextp)                                            \
({                                                               \
    sigsetjmp((gcontextp)->jmpbuf, -1);                          \
    (gcontextp)->uc_stack.ss_sp =                                  \
        (unsigned char*) (gcontextp)->jmpbuf->__jmpbuf[JB_SP]; \
    (gcontextp)->uc_stack.ss_size = 0;                             \
})

#define setcontext(gcontextp)                              \
({                                                         \
     siglongjmp((gcontextp)->jmpbuf,(int) gcontextp);         \
})

#define       switchContext(contextp) {                       \
    errno = (contextp)->unix_errno;                   \
    setcontext(&(contextp)->gcontext);                        \
}

#define yieldContext(contextp)   {                            \
    unsigned char *dummy;                                       \
    int ret;                                                    \
    gcontext_t * ucp = &((contextp)->gcontext);                 \
    (contextp)->current_sp = &dummy;                          \
    ret=sigsetjmp(ucp->jmpbuf, -1);                   \
    if (ret == 0) {                                             \
        ucp->uc_stack.ss_sp =                           \
          (unsigned char*)ucp->jmpbuf->__jmpbuf[JB_SP]; \
        ucp->uc_stack.ss_size = 0;                         \
        (contextp)->unix_errno = errno;                       \
        reschedule();                                         \
    }                                                           \
}
#elif defined(__mc68000__)
#define getcontext(gcontextp)                                  \
({                                                             \
    sigsetjmp((gcontextp)->jmpbuf, -1);                        \
    (gcontextp)->uc_stack.ss_sp =                              \
        (unsigned char*) (gcontextp)->jmpbuf->__jmpbuf[0].__sp; \
    (gcontextp)->uc_stack.ss_size = 0;                         \
})

#define setcontext(gcontextp)                                  \
({                                                             \
    siglongjmp((gcontextp)->jmpbuf, 1);                        \
})

#define       switchContext(contextp) {                       \
    errno = (contextp)->unix_errno;                   \
    setcontext(&(contextp)->gcontext);                        \
}

#define yieldContext(contextp)   {                            \
    unsigned char *dummy;                                       \
    gcontext_t * ucp = &((contextp)->gcontext);                 \
    int ret;                                                    \
    (contextp)->current_sp = &dummy;                          \
    ret=sigsetjmp(ucp->jmpbuf, -1);                           \
    if (ret == 0) {                                             \
        ucp->uc_stack.ss_sp =                                   \
          (unsigned char*)ucp->jmpbuf->__jmpbuf[0].__sp;        \
        ucp->uc_stack.ss_size = 0;                              \
        (contextp)->unix_errno = errno;                               \
        reschedule();                                         \
    }                                                           \
}


#endif

#endif /* !_JAVASOFT_LINUX_CONTEXT_MD_H_ */

