/*
 * @(#)schedule.h	1.22 00/02/02
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Scheduling functions that other interfaces will need.  Generic to
 * Green threads package.
 */

#ifndef _JAVASOFT_GREEN_SCHEDULE_H_
#define _JAVASOFT_GREEN_SCHEDULE_H_

#include "queue.h"
#include "context.h"
#include "monitor_md.h"

/*
 * The rescheduling flag.  Set by an interrupt routine when
 * a (potentially) different thread should be chosen to run
 * instead of returning to the current thread.
 */
extern volatile int _needReschedule;

/*
 * The scheduling lock.  This is used to arbitrate access by the
 * asynchronous condition variable notification interface.
 *
 * SCHED_LOCK() and SCHED_UNLOCKED() used to be macros, but GCC
 * reordered instructions enough that they had to be changed to
 * functions to guarantee the causal order.
 */
extern int _scheduling_lock;
extern void _sched_lock(void);
extern void _sched_unlock(void);

#define SCHED_LOCK()	(_sched_lock())
#define SCHED_LOCKED()	(_scheduling_lock)
#define SCHED_UNLOCK()	(_sched_unlock())

/*
 * Switch to a new thread.
 */
#define YIELD() {				\
    sys_thread_t *tid = greenThreadSelf();	\
    queueInsert(&runnable_queue, tid);		\
    yieldContext(CONTEXT(tid));			\
}

#endif /* !_JAVASOFT_GREEN_SCHEDULE_H_ */
