/*
 * @(#)malloc.c	1.4 00/01/12
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
#undef VMEMDEBUG

#include <stdlib.h>
#include <malloc.h>
#include <dlfcn.h>
#include <stdio.h>
#include <signal.h>
#include "mutex_md.h"

int malloc_init;
void __java_free(void*);
void *__java_malloc(size_t);
void *__java_calloc(size_t,size_t);
void *__java_valloc(size_t);
void *__java_memalign(size_t,size_t);
void *__java_realloc(void *,size_t);

#define INITVALUE 0xfefefefe
#define INITVALUE2 0xa5a5a5a5
#define PTR_DISP 8
#define SIZE_INCR PTR_DISP /* SIZE_INCR must be at least as large at PTR_DISP */

#define BACK_DISP(ptr, ix) \
	(((long*)(((char *)(ptr)) - PTR_DISP))[(ix)])

#define DO_LASTFREE_CHECK 1


#undef __libc_lock_lock
#undef __libc_unlock_lock
#define __libc_lock_lock(NAME) mutexLock(NAME)
#define __libc_unlock_lock(NAME) mutexUnlock(NAME)


#if 0
static int __libc_malloc_lock_fake[4];
static void *__libc_malloc_lock = &__libc_malloc_lock_fake;
#else
static int __libc_malloc_lock_fake[4];
static mutex_t * __libc_malloc_lock = &__libc_malloc_lock_fake;
#endif


/* keep track of the lowest memory address */
/* we will do a malloc(1) to get this initially */
static void *firstAlloc = 0;


#ifdef DO_LASTFREE_CHECK
#define LASTSIZE 32 /* linuxppc */
static void *last_freed[LASTSIZE];
static int last_free_place=0;
static void *last_retadr[LASTSIZE];
static void *last_retadr2[LASTSIZE];

static int dontDoEnvChecked = 0;
static int dontDoLastCheck = 0;



#define NestLevel _javamalloc_NestLevel

static int NestLevel=0;
inline void NestStart() {
    if (NestLevel) {
	fprintf(stderr, "OUCH: nested memory code, to %d levels.\n",NestLevel);
	fflush(stderr);
	kill(getpid(),SIGABRT);
    }
    NestLevel++;
}

inline void NestEnd() {
    NestLevel--;
}

#if defined(__powerpc__)
static int add_freed_pointer(void *ptr, void *radd, void *radd2)
#else 
static int add_freed_pointer(void *ptr)
#endif
{
	int i;

	if (!dontDoEnvChecked) {
	  dontDoEnvChecked = 1;
	  dontDoLastCheck = ((char *)getenv("DO_NOT_CHECK_MEM") != NULL);
	}

	if (dontDoLastCheck) {
	  return 1;
	}


	for (i = LASTSIZE; i--;) {
		if (last_freed[i] == ptr) {
			return 0;
		}
	}

	last_freed[last_free_place] = ptr;
#if defined(__powerpc__)
        last_retadr[last_free_place] = radd;
        last_retadr2[last_free_place] = radd2;
#endif
	last_free_place++;
	if (last_free_place >= LASTSIZE) last_free_place = 0;

	return 1;
}

static void remove_freed_pointer(void *ptr)
{
	int i;

	if (!dontDoEnvChecked) {
	  dontDoEnvChecked = 1;
	  dontDoLastCheck = ((char *)getenv("DO_NOT_CHECK_MEM") != NULL);
	}

	if (dontDoLastCheck) {
	  return;
	}

	for (i = LASTSIZE; i--;) {
		if (last_freed[i] == ptr) {
		    last_freed[i] = 0;
                    last_retadr[i] = 0;
		    return;
		}
	}
}

#else
inline void NestStart() {}
inline void NestEnd() {}
#endif /* DO_LASTFREE_CHECK */
 

void
free (void *ptr)
{
        /* -- linux debugging code -- */
        static int freeEnvChecked = 0;
	static int doNotFree = 0;

	if (ptr == NULL) return;

	if (!freeEnvChecked) {
	  freeEnvChecked = 1;
	  
	  doNotFree = ((char *)getenv("DO_NOT_FREE") != NULL);
	}

	if (doNotFree) {
	  return;
	}

        /* -- end linux debugging code -- */


#if !defined(__powerpc__)
  #if 0 /* [sbb] 26-Oct-97 10:23:39 no longer needed (?) */
    #if defined(__i386__) && defined(linux)
	/* UGLY HACK */
	/* something in X or Motif is freeing memory in the code block */
	/* this should trap that... */
   	if ((ptr >= (void*)0x40000000 && ptr <= (void*)0x50000000) || (ptr < firstAlloc)) {
      #ifdef DO_LASTFREE_CHECK
		fprintf(stderr, "RC: free: out-of-range: %p\n", ptr);
      #endif
		return;
	}
    #endif
  #endif
#endif /* ! powerpc */
	__libc_lock_lock(__libc_malloc_lock);
	NestStart();

#if defined(steve_debug)

  #if defined(doinit)
	if (BACK_DISP(ptr, 0) != INITVALUE || BACK_DISP(ptr, 1) != INITVALUE2) {
	  fprintf(stderr, "Tried to free bogus memory %x, ignored\n", ptr);
	  NestEnd();
          __libc_unlock_lock(__libc_malloc_lock);
	  return;
	}
  #endif

#endif

#if defined(__powerpc__)
  #ifdef DO_LASTFREE_CHECK
        {
           int retaddr, retaddr2;
           int i;
           __asm__ __volatile__ (
	   "lwz %1,0(1); lwz %0,4(%1); lwz %1,0(%1); lwz %1,4(%1)" : "=r" 
                 (retaddr), "=r" (retaddr2) );

	   /* check against multi-free */
	   if (add_freed_pointer(ptr,retaddr,retaddr2)) {
  #endif
#else /* !powerpc */

  #ifdef DO_LASTFREE_CHECK
	/* (not so) wimpy check against multi-free */
	if (add_freed_pointer(ptr)) {
  #endif
#endif /* powerpc test */
#ifdef VMEMDEBUG
		fprintf(stderr, "freeing at %lX...", ptr);
		fflush(stderr);
#endif

#if defined(steve_debug)
		__java_free (ptr - PTR_DISP);
#else
		__java_free (ptr);
#endif

#ifdef VMEMDEBUG
		fprintf(stderr, "done\n");
#endif
#ifdef DO_LASTFREE_CHECK
  #if !defined(__powerpc__)
	} else {
		fprintf(stderr, "free: double-freed at %lX\n", ptr);
	}
  #else /* powerpc */
	    } else {
               fprintf(stderr, "free: double-freed at %#lX from
                   %#lX,%#lX\n", ptr, retaddr,retaddr2);
	       for (i = LASTSIZE; i--;) {
		  if (last_freed[i] == ptr) {
		     fprintf(stderr,"Free earlier from %#lX,%#lX\n",
                          last_retadr[i],last_retadr2[i]);
	          }
	       }
	    }
	} 
  #endif /* powerpc or not */
#endif
	NestEnd();
	__libc_unlock_lock(__libc_malloc_lock);
}

void *
malloc (size_t size)
{
	void * ret;
	__libc_lock_lock(__libc_malloc_lock);
	NestStart();

#if defined(__i386__) && defined(linux)
	/* keep track of what should be the heap's base, so we know what
	   is a low-memory free, and can ignore it! */
	if (firstAlloc == 0) {
	    firstAlloc = __java_malloc(1);
  #ifdef VMEMDEBUG
	    fprintf(stderr, "firstAlloc: %p\n", firstAlloc);
  #endif
	}
#endif /* x86 linux */

#if defined(steve_debug) 

	ret = (char*)__java_malloc (size + SIZE_INCR);

  #if defined(TEST_ALIGNMENT)
	if (((long)ret) % 8) {
	  fprintf(stderr, " malloc at %x\n", ret);
	}
  #endif

	ret += PTR_DISP;

  #if defined(doinit)
	BACK_DISP(ret, 0) = INITVALUE;
	BACK_DISP(ret, 1) = INITVALUE2;
  #endif
#else
	ret = (char*)__java_malloc (size);
#endif

#ifdef DO_LASTFREE_CHECK
	remove_freed_pointer(ret);
#endif
#ifdef VMEMDEBUG
	fprintf(stderr, "malloced %d at %lX\n", (int)size, (long)ret);
#endif
	NestEnd();
	__libc_unlock_lock(__libc_malloc_lock);
	return ret;
}

void *
valloc (size_t size)
{
	void * ret;
	__libc_lock_lock(__libc_malloc_lock);
	NestStart();

#if defined(steve_debug)
	ret = __java_valloc (size + SIZE_INCR);

  #if defined(TEST_ALIGNMENT)
	if (((long)ret) % 8) {
	  fprintf(stderr, " valloc at %x\n", ret);
	}
  #endif

	ret += PTR_DISP;

  #if defined(doinit)
	BACK_DISP(ret, 0) = INITVALUE;
	BACK_DISP(ret, 1) = INITVALUE2;
  #endif
#else
	ret = __java_valloc (size);
#endif /* steve_debug */

#ifdef DO_LASTFREE_CHECK
	remove_freed_pointer(ret);
#endif

#ifdef VMEMDEBUG
	/*fprintf(stderr, "valloced %d at %lX\n", (int)size, (long)ret);*/
#endif
	NestEnd();
	__libc_unlock_lock(__libc_malloc_lock);
	return ret;
}

void *
calloc (size_t nmemb, size_t size)
{
	void * ret;
#if 0
	ret = malloc(nmemb*size);
	if (ret != NULL)  {
		memset(ret, 0, nmemb*size);
	}
	return ret;
#else /* real code */
	__libc_lock_lock(__libc_malloc_lock);
	NestStart();
  #if !defined(steve_debug)
	ret = __java_calloc (nmemb, size*2);
  #else /* steve_debug */
	{
	  size_t totalSize;
	  totalSize = nmemb * size;
	  ret = __java_malloc(totalSize + SIZE_INCR);

    #if defined(TEST_ALIGNMENT)
 	  if (((long)ret) % 8) {
 	    fprintf(stderr, " calloc at %x\n", ret);
 	  }
    #endif
	  memset(ret, 0, totalSize + SIZE_INCR);
	}

	/* fprintf(stderr, "calloc mem %d size %d \n", nmemb, size); */

	ret += PTR_DISP;

     #if defined(doinit) 
	BACK_DISP(ret,0) = INITVALUE;
	BACK_DISP(ret,1) = INITVALUE2;
     #endif
  #endif /* steve_debug */

  #ifdef DO_LASTFREE_CHECK
	remove_freed_pointer(ret);
  #endif

  #ifdef VMEMDEBUG
	fprintf(stderr, "calloced %d at %lX\n", (int)(nmemb*size), (long)ret);
  #endif
	NestEnd();
	__libc_unlock_lock(__libc_malloc_lock);
	return ret;
#endif /* 0 or not */
}

void *
memalign (size_t alignment, size_t size)
{
	void * ret;
	__libc_lock_lock(__libc_malloc_lock);
	NestStart();
	ret = __java_memalign (alignment, size);

#ifdef DO_LASTFREE_CHECK
	remove_freed_pointer(ret);
#endif

#ifdef VMEMDEBUG
	fprintf(stderr, "memaligned %d at %lX\n", (int)size, (long)ret);
#endif
	NestEnd();
	__libc_unlock_lock(__libc_malloc_lock);
	return ret;
}

void *
realloc (void *ptr, size_t size)
{
	void * ret;
	__libc_lock_lock(__libc_malloc_lock);
	NestStart();

#if defined(steve_debug)
	if (ptr != 0) { 

	  if ((unsigned long)ptr < 2000) {
	    fprintf(stderr, "bad ptr %x", ptr);
	  }

  #if defined(doinit)
	  if (BACK_DISP(ptr,0) != INITVALUE || BACK_DISP(ptr,1) != INITVALUE2) {
	    fprintf(stderr, "Reallocing bogus memory %x\n", ptr);
	  }
  #endif
	} else {
	  ptr += PTR_DISP;		/* compensation for 0 pointers*/
	}

	ret = __java_realloc (ptr - PTR_DISP, size + SIZE_INCR);

  #if defined(TEST_ALIGNMENT) 
	if (((long)ret) % 8) {
	  fprintf(stderr, " relloc at %x\n", ret);
	}
  #endif

	ret += PTR_DISP;

  #if defined(doinit)
	BACK_DISP(ret,0) = INITVALUE;
	BACK_DISP(ret,1) = INITVALUE2;
  #endif
#else
	ret = __java_realloc (ptr, size);
#endif

#ifdef DO_LASTFREE_CHECK
	remove_freed_pointer(ret);
        if (ptr != ret) {
  #if defined(__powerpc__)
           int retaddr, retaddr2;
           int i;
           __asm__ __volatile__ (
	   "lwz %1,0(1); lwz %0,4(%1); lwz %1,0(%1); lwz %1,4(%1)" : "=r"
                  (retaddr), "=r" (retaddr2) );
           add_freed_pointer(ptr,retaddr,retaddr2);
  #else
	   add_freed_pointer(ptr);
  #endif /* ? powerpc */
        }
#endif

#ifdef VMEMDEBUG
	fprintf(stderr, "realloced %d at %lX (ptr = %lX)\n", (int)size,
                    (long)ret, (long)ptr);
#endif

	NestEnd();
	__libc_unlock_lock(__libc_malloc_lock);
	return ret;
}
