/*
 * @(#)schedule.c	1.31 00/03/14
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Thread scheduling utilities and data structures.
 */

#include "hpi_impl.h"

#include "threads_md.h"
#include "queue.h"
#include "schedule.h"
#include <memory.h>

int _scheduling_lock;
volatile int _needReschedule;

gt_queue_t runnable_queue;

void
reschedule()
{
    sys_thread_t *self = greenThreadSelf();
    sys_thread_t *tid;

    sysAssert(SCHED_LOCKED());
    sysAssert(_scheduling_lock == 1);

    tid = queueRemoveHead(&runnable_queue, tid);

    sysAssert(tid->state == RUNNABLE);
    sysAssert(!tid->sigmaskdirty);

    Log2(5, "reschedule: from TID 0x%x to TID 0x%x\n",
    	 self, tid);

    updateSliceStart();

    setCurrentThread(tid);

    if (self->sigmaskdirty) {
	green_sigprocmask(SIG_SETMASK, NULL, &MACHDEP(self)->signalMask);
	self->sigmaskdirty = 0;
    }

    if (memcmp(&MACHDEP(tid)->signalMask, &MACHDEP(self)->signalMask,
	sizeof MACHDEP(tid)->signalMask) != 0)
    {
	sigset_t old;
	green_sigprocmask(SIG_SETMASK, &MACHDEP(tid)->signalMask, &old);
#ifndef __linux__
	sysAssert(memcmp(&MACHDEP(self)->signalMask, &old, sizeof old) == 0);
#endif
    }

    switchContext(CONTEXT(tid));

    /* NOTREACHED */
}
