/*
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#pragma	ident	"@(#)emit.c	1.5	00/05/04 SMI"

/*
 * Wrapper generator
 */

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <strings.h>

#include "javald.h"

static void
emit(FILE *fp, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	(void) vfprintf(fp, fmt, ap);
	va_end(ap);
}

/*
 * this shell function converts relative pathnames into
 * absolute pathnames rooted at the directory that the
 * enclosing script was found in.
 */
static void
emit_prolog(FILE *fp)
{
	char *bind[] = {
		"_D=$(cd `/usr/bin/dirname $0` && print -n $PWD)",
		"",
		"function _",
		"{",
		"\t[[ $1 = ${1##/} ]] && print -n ${_D}/",
		"\tprint -n $1",
		"}",
		0
	};
	char **p = bind;

	while (*p)
		emit(fp, "%s\n", *p++);
}

static void
emit_component(FILE *fp, struct list *l, int islastcomp)
{
	char *dirname;

	if (l == NULL)
		return;
	emit_component(fp, l->next, 0);

	dirname = l->name;
	if (dirname[0] == '/')
		emit(fp, "%s", dirname);
	else
		emit(fp, "$(_ %s)", dirname);

	if (!islastcomp)
		emit(fp, ":");
}

static void
emit_path(FILE *fp, struct list *l)
{
	emit_component(fp, l, 1);
}


/*
 * Set JAVA_HOME, CLASSPATH, and (optionally) LD_LIBRARY_PATH, then
 * exec the JVM
 */
int
emit_script(int fd, int dflg,
    char *java_home, struct list *classlist, struct list *ldliblist,
    char *javaopts, char *classname)
{
	const char llp[] = "LD_LIBRARY_PATH";
	FILE *fp = fdopen(fd, "w");

	emit(fp, "#!/bin/ksh\n#\n");
	emit(fp, "# run-time wrapper for class '%s'\n\n", classname);

	emit_prolog(fp);
	emit(fp, "\n");

	emit(fp, "export JAVA_HOME=%s\n", java_home);

	emit(fp, "export CLASSPATH=");
	emit_path(fp, classlist);
	emit(fp, "\n");

	if (ldliblist) {
		emit(fp, "export %s=", llp);
		emit_path(fp, ldliblist);
		emit(fp, "${%s:+:${%s}}\n", llp, llp);
	}

	if (dflg) {
		emit(fp, "echo JAVA_HOME=${JAVA_HOME}\n");
		if (ldliblist)
			emit(fp, "echo %s=${%s}\n", llp, llp);
		emit(fp, "echo CLASSPATH=${CLASSPATH}\n");
	}

	emit(fp, "exec ${JAVA_HOME}/bin/java ");
	if (javaopts)
		emit(fp, "%s ", javaopts);
	emit(fp, "%s \"$@\"\n", classname);

	return (ferror(fp) == 0 && fclose(fp) == 0);
}
