/*
 * @(#)javai_md.c	1.14 00/03/09
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <dlfcn.h>
#include <stdlib.h>
#include <math.h>

#include "oobj.h"
#include "interpreter.h"
#include "sys_api.h"

#ifdef DEBUG
#define HPI_DLL "libhpi_g.so"
#else
#define HPI_DLL "libhpi.so"
#endif

#ifdef __linux__
int _JVM_native_threads;
#endif

void *
InitializeHPI(void *args)
{
    char buf[MAXPATHLEN];
    Dl_info dlinfo;
    void *hpi_handle;
    GetInterfaceFunc getintf = NULL;
    jint (JNICALL * DLL_Initialize)(GetInterfaceFunc *, void *);
    char *thread_type = getenv("_JVM_THREADS_TYPE");

    if (thread_type) {
	/* Set by .java_wrapper.  Remove for the sake of exec'd VMs. */
	thread_type = strdup(thread_type);
        putenv("_JVM_THREADS_TYPE=");
    } else {
	/* Default thread type for the invocation API. */
        thread_type = strdup("native_threads");
    }
#ifdef __linux__
   _JVM_native_threads = !strcmp("native_threads", thread_type);
#endif

    dladdr((void *)InitializeHPI, &dlinfo);
    strcpy(buf, (char *)dlinfo.dli_fname);
    *(strrchr(buf, '/')) = '\0';	/* get rid of /libjvm.so */
    *(strrchr(buf, '/') + 1) = '\0';	/* get rid of classic    */
    strcat(buf, thread_type);
    strcat(buf, "/" HPI_DLL);
    /* we use RTLD_NOW because of bug 4032715 */
    hpi_handle = dlopen(buf, RTLD_NOW);
    if (hpi_handle == NULL) {
	goto bail;
    }
    DLL_Initialize = (jint (JNICALL *)(GetInterfaceFunc *, void *))
        dlsym(hpi_handle, "DLL_Initialize");
    if (DLL_Initialize == NULL ||
        (*DLL_Initialize)(&getintf, args) < 0) {
        goto bail;
    }
bail:
    free(thread_type);
    return (void *)getintf;
}

props_md_t *
GetPropertiesMD()
{
    static props_md_t sprops = {0};
    char *v; /* tmp var */

    if (sprops.dll_dir) {
        return &sprops;
    }

    /* sysclasspath, java_home, dll_dir */
    {
        char *home_path;
	char *sysclasspath;
	char *dll_path;
	char *rtJar = "/lib/rt.jar";
	char *i18nJar = "/lib/i18n.jar";
	char *sunrsasignJar = "/lib/sunrsasign.jar";
	char *classes = "/classes";
        char buf[MAXPATHLEN];
	Dl_info dlinfo;
	dladdr((void *)GetPropertiesMD, &dlinfo);
	realpath((char *)dlinfo.dli_fname, buf);
	*(strrchr(buf, '/')) = '\0';	/* get rid of /libjvm.so */
	*(strrchr(buf, '/')) = '\0';    /* get rid of classic    */
	dll_path = malloc(strlen(buf) + 1);
	if (dll_path == 0)
	    return NULL;
	strcpy(dll_path, buf);
	sprops.dll_dir = dll_path;

	*(strrchr(buf, '/')) = '\0';
	*(strrchr(buf, '/')) = '\0';

	home_path = malloc(strlen(buf) + 1);
	if (home_path == 0)
	    return NULL;
	strcpy(home_path, buf);
	sprops.java_home = home_path;

	sysclasspath = malloc(strlen(home_path) * 4 +
			      strlen(rtJar) + strlen(i18nJar) +
			      strlen(sunrsasignJar) + strlen(classes) +
			      4);
	if (sysclasspath == 0) {
          return NULL;
	}
	sprintf(sysclasspath, "%s%s:%s%s:%s%s:%s%s",
		home_path, rtJar, home_path, i18nJar,
		home_path, sunrsasignJar, home_path, classes);
	sprops.sysclasspath = sysclasspath;
    }

    /* Where to look for native libraries */
#ifdef __linux__
#define DEFAULT_LD_LIBRARY_PATH "/usr/lib:/lib" /* See ld.so(8) */
#else
#define DEFAULT_LD_LIBRARY_PATH "/usr/lib" /* See ld.so.1(1) */
#endif
    {
	/* Get the user setting of LD_LIBRARY_PATH */
	v = getenv("LD_LIBRARY_PATH");

	if (v == NULL) {
	    v = strdup(DEFAULT_LD_LIBRARY_PATH); /* This is never the case. */
	} else {
	    char *ld_library_path = (char *)
		malloc(sizeof(DEFAULT_LD_LIBRARY_PATH) 	/* enough for '\0' */ +
		       1 				/* colon */ +
		       strlen(v));
	    strcpy(ld_library_path, v);
	    strcat(ld_library_path, ":" DEFAULT_LD_LIBRARY_PATH);
	    v = ld_library_path;
	}
	sprops.library_path = v;
    }

    /* Extensions directories */
    {
	char buf[MAXPATHLEN + 10];
	sprintf(buf, "%s/lib/ext", sprops.java_home);
	sprops.ext_dirs = strdup(buf);
    }

    return &sprops;
}

#ifdef __linux__
/* linuxppc: PORT (addition of routines to do casts properly) */
#if defined(__powerpc__) || defined(__mc68000__)

#include <limits.h>
#include <math.h>
#undef LONG_LONG_MAX
#undef LONG_LONG_MIN
#define LONG_LONG_MAX 0x7fffffffffffffffLL
#define LONG_LONG_MIN 0x8000000000000000LL
#define LL_MAX 0x7fffffffffffffffLL
#define LL_MIN 0x8000000000000000LL
#define LI_MAX 0x7fffffffL
#define LI_MIN 0x80000000L
u_int64_t i64abs(int64_t llv);
u_int64_t i64abs(int64_t llv) {
  if (llv == LL_MIN) {
      return ((u_int64_t) LL_MAX);
  }
  if (llv < 0) {
      return ((u_int64_t) -llv);
  }
  return ((u_int64_t) llv);
}

int32_t float2l(float f)
{
    int64_t lltmp = float2ll(f);
    int32_t ltmp = (int32_t)f;
    if (isnan((double)f)) {return 0;}
    if (((int64_t)ltmp) == lltmp) {
        return ltmp;
    } else {
        if (f < 0) {
            return (int32_t) LI_MIN;
        } else {
            return (int32_t) LI_MAX;
        }
    }
}

int32_t double2l(double f)
{
    int64_t lltmp = double2ll(f);
    int32_t ltmp = (int32_t)f;
    if (isnan(f)) {return 0;}
    if (((int64_t)ltmp) == lltmp) {
        return ltmp;
    } else {
        if (f < 0) {
            return (int32_t) LI_MIN;
        } else {
            return (int32_t) LI_MAX;
        }
    }
}

int64_t float2ll(float f)
{
    int64_t lltmp = (int64_t)f;
    u_int64_t ulltmp = (u_int64_t)fabs((double)f);
    if (isnan((double)f)) {return 0;}
    if (i64abs(lltmp) == ulltmp) { 
       return lltmp;
    } else {
        if (f < 0) {
            lltmp = LL_MIN;
        } else {
            lltmp = LL_MAX;
        }
        return lltmp;
    }
}

int64_t double2ll(double f)
{
    int64_t lltmp = (int64_t)f;
    u_int64_t ulltmp = (u_int64_t)fabs(f);
    if (isnan(f)){return 0;}
    if (i64abs(lltmp) == ulltmp) { 
        return lltmp;
    } else {
        if (f < 0) {
            lltmp = LL_MIN;
        } else {
            lltmp = LL_MAX;
        }
        return lltmp;
    }
}


#endif
#endif

#if !((defined(__powerpc__) && __linux) || (defined(__mc68000__) && __linux__))

int32_t float2l(float f)
{
    int64_t lltmp = (int64_t)f;
    int32_t ltmp = (int32_t)lltmp;
    if (ltmp == lltmp) {
        return ltmp;
    } else {
        if (isnan((double)f)) {return 0;}
        if (f < 0) {
            return 0x80000000;
        } else {
            return 0x7fffffff;
        }
    }
}

int32_t double2l(double f)
{
    int64_t lltmp = (int64_t)f;
    int32_t ltmp = (int32_t)lltmp;
    if (ltmp == lltmp) {
        return ltmp;
    } else {
        if (isnan(f)) {return 0;}
        if (f < 0) {
            return 0x80000000;
        } else {
            return 0x7fffffff;
        }
    }
}

int64_t float2ll(float f)
{
#if defined(i386) || defined(__i386__)
    int64_t lltmp = (int64_t)f;
    if (lltmp != 0x8000000000000000) {
        return lltmp;
    } else {
        if (isnan((double)f)) {return 0;}
        if (f < 0) {
            return 0x8000000000000000;
        } else {
            return 0x7fffffffffffffff;
        }
    }
#endif /* i386 */

#if defined(sparc) || defined(__sparc__)
    if (isnan((double)f)) {
        return 0;
    }
    return (int64_t)f;
#endif /* sparc */
}
#endif 

int64_t double2ll(double f)
{
#if defined(i386) || defined(__i386__)
    int64_t lltmp = (int64_t)f;
    if (lltmp != 0x8000000000000000) {
        return lltmp;
    } else {
        if (isnan(f)) {return 0;}
        if (f < 0) {
            return 0x8000000000000000;
        } else {
            return 0x7fffffffffffffff;
        }
    }
#endif /* i386 */

#if defined(sparc) || defined(__sparc__)
    if (isnan(f)) {
        return 0;
    }
    return (int64_t)f;
#endif /* sparc */
}
