/*
 * @(#)jvm_md.c	1.29 98/03/23
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include "jni.h"
#include "jvm.h"
#include "jvm_md.h"

/*
 * Platform-specific support for java.lang.Runtime
 */

#include "java_lang_Process.h"

/* path in the environment */
static char **PATH = 0;
/* effective uid */
static uid_t uid;
/* effective group id */
static gid_t gid;

static void
parsePath () {
    char *path, *c, *len;
    int count = 0;
    int i;

    /* get uid, gid */
    uid = geteuid();
    gid = getegid();
    if ((path = getenv("PATH")) == 0) {
	return;
    }
    path = strdup(path);
    len = path + strlen(path);
    /* count path elements */
    for (c = path; c < len; c++) {
	if (*c == ':')
	    count++;
    }
    PATH = (char **)malloc((++count+1) * sizeof(char*));

    /* fill it in */
    PATH[0] = path;
    PATH[count] = 0;

    for (i = 1; i < count; i++) {
	c = strchr(path, ':');
	if (c == 0) { /* shouldn't happen */
	    /*	    jio_fprintf(stderr, "processmd.c: wrong count parsing path: i=%d, count=%d\n", i, count); */
	    break;
	}
	*c++ = 0;
	PATH[i] = path = c;
    }
}

/* return 0 if it is executable && readable by this process
 * -1 if no such file, -2 if it cannot be executed, and raise exception
 */
static int
statExecutable(char *exe, struct stat *b) {
    if (stat(exe, b)) { /* doesn't exist */
	return -1;
    }
    if (S_ISDIR(b->st_mode)) {
	/* cannot execute */
	return -2;
    }
    /* check for user permissions */
    if (b->st_uid == uid) {
	return (b->st_mode & S_IXUSR) ? 0 : -2;
    }
    /* check for group permissions */
    if (b->st_gid == gid) {
	return (b->st_mode & S_IXGRP) ? 0 : -2;
    }
		
    /* check for world permissions */
    return b->st_mode & S_IXOTH ? 0 : -2;
}

/* Find the command like a shell would.
 * signal an error for things not executable || not readable || not found.
 */

static char *fullPath(char *part, char *full) {
    char **tmp;
    struct stat b;
    int ret;
    if (*part != '/' && PATH) {
	for (tmp = PATH; *tmp; tmp++) {
	    strcpy(full, *tmp);
	    full[strlen(*tmp)] = '\0';
	    strcat(full, "/");
	    strcat(full, part);
	    ret = statExecutable(full, &b);
	    if (ret == -1) { /* doesn't exist */
		continue;
	    } else if (ret == -2) { /* can't execute */
		jio_snprintf(full, MAXPATHLEN, "%s: cannot execute", part);
		SignalError(0, "java/io/IOException", full);
		return 0;
	    } else {
		return full;
	    }
	}
    } else if (!(ret = statExecutable(part, &b))) {
	return part;
    } else if (ret == -2) { /* cannot execute */
	jio_snprintf(full, MAXPATHLEN, "%s: cannot execute", part);
	SignalError(0, "java/io/IOException", full);
	return 0;
    }

    /* not found if we got here */
    jio_snprintf(full, MAXPATHLEN, "%s: not found", part);
    SignalError(0, "java/io/IOException", full);
    return 0;
}

JNIEXPORT jobject JNICALL
JVM_ExecInternal(JNIEnv *env, jobjectArray cmdarray0, jobjectArray envp0)
{
    HArrayOfString *cmdarray = (HArrayOfString *)DeRef(env, cmdarray0);
    HArrayOfString *envp = (HArrayOfString *)DeRef(env, envp0);
    Hjava_lang_Process *proc = 0;
    char part[MAXPATHLEN+1];
    char *tmp;
    int len;
    char full[MAXPATHLEN+1];
    HString *strh;

    if (cmdarray == 0 || unhand(cmdarray)->body == 0 || 
	obj_length(cmdarray) == 0 ||
	(strh = (HString*)unhand(cmdarray)->body[0]) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    if (PATH == 0) {
	parsePath();
    }
    
    tmp = makePlatformCString(strh);
    len = MAXPATHLEN > strlen(tmp) ? strlen(tmp) : MAXPATHLEN; 
    strncpy(part, tmp, len);
    part[len] = '\0';
    if (fullPath(part, full) == 0) {  /* returns null on invalid executable */
	return 0;
    }

    /*
	We need a way to know which subclass of Process we want to create.
    */
    proc = (Hjava_lang_Process *) execute_java_constructor(EE(),
				  JAVAPKG "UNIXProcess",
				  (ClassClass *) 0, 
				  "([Ljava/lang/String;[Ljava/lang/String;)", 
				  cmdarray, envp);
    return MkRefLocal(env, proc);
}


/*
 * Platform-specific support for java.lang.UNIXProcess
 */

#include "monitor.h"
#include "monitor_md.h"
#ifndef NATIVE
#include "iomgr.h"
#endif

#include "java_lang_UNIXProcess.h"
#include "java_io_FileDescriptor.h"
#include "java_io_FileInputStream.h"

#include <wait.h>
#include <signal.h>
#include <errno.h>

#ifdef NATIVE
static sys_mon_t fork_wait_mon;
static int pforked = 0;
#endif

JNIEXPORT void JNICALL
JVM_Process_Run(JNIEnv *env, jobject this0)
{
    Hjava_lang_UNIXProcess *this = (Hjava_lang_UNIXProcess *)DeRef(env, this0);
    siginfo_t info;
#ifdef NATIVE
    sys_mon_t *mon = &fork_wait_mon;
#else
    sys_mon_t *mon = asyncMon(SYS_ASYNC_MON_CHILD);
#endif
    char *classname = JAVAPKG "UNIXProcess";
    ClassClass *cb = FindClass(0, classname, TRUE);

    if (cb == NULL) {
	SignalError(0, JAVAPKG "NoClassDefFoundError", classname);
	return;
    }

#ifdef NATIVE
    monitorRegister(&fork_wait_mon, "Fork_Wait_monitor");
#endif
    /* reaper thread loops for ever looking out for dead children */
    /* the loop below operates basically as a synchronized block */
    sysMonitorEnter(EE2SysThread(JNIEnv2EE(env)), mon);
    while (1) {
#ifdef NATIVE
        while (pforked == 0) {
            sysMonitorWait(EE2SysThread(JNIEnv2EE(env)), mon, SYS_TIMEOUT_INFINITY);
        }

	pforked--;
	/* a little paranoia checking to make sure we don't underflow */
	if (pforked < 0)
	    pforked = 0;

        sysMonitorExit(EE2SysThread(JNIEnv2EE(env)), mon);

	/* block this reaper thread until the child process exits */
        if (waitid(P_ALL, 0, &info, WEXITED) != 0 || info.si_pid == 0) {
            continue;
        }

	sysMonitorEnter(EE2SysThread(JNIEnv2EE(env)), mon);
#else
	while (waitid(P_ALL, 0, &info, WEXITED|WNOHANG) != 0 ||
	       info.si_pid == 0) {
	    sysMonitorWait(EE2SysThread(JNIEnv2EE(env)), mon, SYS_TIMEOUT_INFINITY);
	}
#endif

	execute_java_static_method(EE(), cb, "deadChild", "(II)V",
                                   info.si_pid, info.si_status);
	/* keep the reference alive */
	if (this == NULL) {
	    sysMonitorExit(EE2SysThread(JNIEnv2EE(env)), mon);
	    SignalError(0, JAVAPKG "NullPointerException", 0);
	    return;
	}
    }
}

JNIEXPORT void JNICALL
JVM_Process_NotifyReaders(JNIEnv *env, jobject this0)
{
#ifndef NATIVE
    Classjava_lang_UNIXProcess *thisptr =
	(Classjava_lang_UNIXProcess *)unhand(DeRef(env, this0));
    Classjava_io_FileDescriptor *outfdptr = unhand(thisptr->stdout_fd);
    Classjava_io_FileDescriptor *errfdptr = unhand(thisptr->stderr_fd);

    if ((outfdptr->fd) > 0) {
	sysMonitorEnter(EE2SysThread(JNIEnv2EE(env)), fdmon[outfdptr->fd]);
	sysMonitorNotifyAll(EE2SysThread(JNIEnv2EE(env)), fdmon[outfdptr->fd]);
	sysMonitorExit(EE2SysThread(JNIEnv2EE(env)), fdmon[outfdptr->fd]);
    }

    if ((errfdptr->fd) > 0) {
	sysMonitorEnter(EE2SysThread(JNIEnv2EE(env)), fdmon[errfdptr->fd]);
	sysMonitorNotifyAll(EE2SysThread(JNIEnv2EE(env)), fdmon[errfdptr->fd]);
	sysMonitorExit(EE2SysThread(JNIEnv2EE(env)), fdmon[errfdptr->fd]);
    }
    /* Keep the reference ***pinned*** */
    if (thisptr == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
    }
#endif
}

JNIEXPORT jint JNICALL
JVM_Process_ForkAndExec(JNIEnv *env, jobject this0, jobjectArray cmdarray0,
		        jobjectArray envp0)
{
    Hjava_lang_UNIXProcess *this =
		(Hjava_lang_UNIXProcess *)DeRef(env, this0);
    HArrayOfString *cmdarray = (HArrayOfString *)DeRef(env, cmdarray0);
    HArrayOfString *envp = (HArrayOfString *)DeRef(env, envp0);
    /* let's do all the setup for the exec before we call fork1 */
    int fdin[2], fdout[2], fderr[2], fdsync[2];
    char **envstr = NULL;
    char fullpath[MAXPATHLEN+1];
    int i;
    char **argv;
    char * savedArgv0;
    int argc;
    int pid = -1; /* this is the error return value */

    /* On error_cleanup, we don't want to close() some random junk. */
    fdsync[0] = fdsync[1] = 
	fdin[0] = fdin[1] = 
	fdout[0] = fdout[1] = 
	fderr[0] = fderr[1] = -1;

    argc = obj_length(cmdarray);
    if (argc == 0) {
	SignalError(0, JAVAPKG "IllegalArgumentException", 0);
	return -1;
    }
#define BODYOF(h)   unhand(h)->body
    /* calloc null terminates for us */
    argv = sysCalloc((argc + 1) * sizeof(char *), 1);
    if (argv == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	goto error_cleanup;
    }

    for (i = 0; i < argc; i++) {
	char * str = makePlatformCString(BODYOF(cmdarray)[i]);
	char * newstr = sysMalloc(strlen(str) + 1);
	if (newstr == 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	    goto error_cleanup;
	}
	strcpy(newstr, str);
	KEEP_POINTER_ALIVE(str);
	argv[i] = newstr;
    }

    savedArgv0 = argv[0];
    argv[0] = fullPath(*argv, fullpath);
    if (argv[0] == NULL) {
	/* fullPath has signalled an exception so we just return */
	goto error_cleanup;
    }

    if (envp != NULL) {
	int len = obj_length(envp);
	int i;
	    
	/* calloc null terminates for us */
	envstr = (char **)sysCalloc((len+1), sizeof(char *));
	if (envstr == 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	    goto error_cleanup;
	}

	for (i = 0; i < len; ++i) {
	    char * str = makePlatformCString(BODYOF(envp)[i]);
            char * newstr = sysMalloc(strlen(str) + 1);
            if (newstr == 0) {
                SignalError(0, JAVAPKG "OutOfMemoryError", 0);
                goto error_cleanup;
            }
            strcpy(newstr, str);
            KEEP_POINTER_ALIVE(str);
            envstr[i] = newstr;
        }
    }
    
    if ((pipe(fdin) < 0) ||   /* stdin */
	(pipe(fdout) < 0) ||  /* stdout */
	(pipe(fderr) < 0) ||  /* stderr */
	(pipe(fdsync) < 0)) { /* fd for synchronization */
	/* make sure we clean up */
 	SignalError(0, "java/io/IOException", strerror(errno));
 	goto error_cleanup;
    }

#ifdef NATIVE
#define CLOSE(fd) close(fd)
#define READ(fd, addr, size) read((fd), (addr), (size))
#else
#define CLOSE(fd) (systable[SYS_CLOSE].addr)(fd)
#define READ(fd, ptr, size) (systable[SYS_READ].addr)((fd), (ptr), (size))
#endif

    if ((pid = fork1()) == 0) {
	/* Child process */
	int i, max_fd;

	/* 0 open for reading, 1 open for writing */
	/* close as we go along to reduce the chance of running out of fds */
	CLOSE(fdsync[1]);
	dup2(fdin[0], 0);
	CLOSE(fdin[0]);
	dup2(fdout[1], 1);
	CLOSE(fdout[1]);
	dup2(fderr[1], 2);
	CLOSE(fderr[1]);

 	{   
 	    int err;
 	    char c;
 	    err = READ(fdsync[0], &c, 1);
 	    CLOSE(fdsync[0]);
 	}

	/* close everything */
	max_fd = sysconf(_SC_OPEN_MAX);
	for (i = 3; i < max_fd; i++) {
	    CLOSE(i);
	}


#undef CLOSE
#undef READ

	if (envstr == NULL) {
	    execv(argv[0], argv);
	} else {
	    execve(argv[0], argv, envstr);
	}

	/*
	 * the exec failed, so there's not much we can do except exit.
	 * We use _exit so that we don't flush file descriptors that
	 * might be shared with our parent, even though we've probably
	 * already closed them at this point
	 */
	_exit(-1);
    } else {
	/* parent process */
	Classjava_lang_UNIXProcess *thisptr = unhand(this);

  	if (pid < 0) {
	    /* make sure we clean up our side of the pipes */
  	    SignalError(0, "java/io/IOException", strerror(errno));
  	    goto error_cleanup;
	}

	/* clean up the child's side of the pipes */
	close(fdin[0]);
	close(fdout[1]);
	close(fderr[1]);
 	close(fdsync[0]);

#ifdef NATIVE
	sysMonitorEnter(EE2SysThread(JNIEnv2EE(env)), &fork_wait_mon);
	pforked++;
	sysMonitorNotify(EE2SysThread(JNIEnv2EE(env)), &fork_wait_mon);
	sysMonitorExit(EE2SysThread(JNIEnv2EE(env)), &fork_wait_mon);
#endif

	unhand(thisptr->stdin_fd)->fd = fdin[1];
	unhand(thisptr->stdout_fd)->fd = fdout[0];
	unhand(thisptr->stderr_fd)->fd = fderr[0];
	unhand(thisptr->sync_fd)->fd = fdsync[1];

	/* keep the reference alive */
	if (this == NULL) {
	    SignalError(0, JAVAPKG "NullPointerException", 0);
	}
    }

 normal_cleanup:
    if (argv) {
	int len = obj_length(cmdarray);
	sysFree(savedArgv0);
	for (i = 1; i < len; i++)
	    sysFree(argv[i]);
	sysFree(argv);
    }
    if (envstr) {
	int len = obj_length(envp);
	for (i = 0; i < len; i++)
	    sysFree(envstr[i]);
	sysFree(envstr);
    }
    return pid;		/* pid will be -1 in the error case */

 error_cleanup:
    close(fdin[0]);
    close(fdin[1]);
    close(fdout[0]);
    close(fdout[1]);
    close(fderr[0]);
    close(fderr[1]);
    close(fdsync[0]);
    close(fdsync[1]);
    goto normal_cleanup;
}

JNIEXPORT void JNICALL
JVM_Process_Destroy(JNIEnv *env, jobject this)
{
    Hjava_lang_UNIXProcess *proc = (Hjava_lang_UNIXProcess *)DeRef(env, this);
    kill(unhand(proc)->pid, SIGTERM);
}

/*
 * Support for Solaris sockets: This looks like it should be done using
 * at least a non-Solaris-specific API...
 */
JNIEXPORT jint JNICALL
JVM_SocketAvailable(jint fd, jint *result) {
    return sysSocketAvailable(fd, (long *)result);
}
