/*
 * @(#)signals_md.c	1.14 00/05/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Solaris 2.X dependent interrupt handling code.
 */


/*
 * Header files.
 */
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>

#if defined(__solaris__) && !defined(SA_SIGINFO)
#error What gives?
#endif

#ifdef SA_SIGINFO  /* should be HAVE_UCONTEXT? */
#include <ucontext.h>
#endif

#ifdef __solaris__
#define DECODE_SIGINFO
#define IMPLICIT_EXCEPTIONS /* let div by 0 happen; recover from sig handler */
#include <sys/trap.h>
#endif

#ifdef __linux__
#define DECODE_SIGINFO
#define MAXSIG NSIG /* that looks right. */
/* [jk] I don't like this, but the information we want is only in
   asm/siginfo.h and not in bits/siginfo.h */
#define SI_FROMUSER(siptr)   ((siptr)->si_code <= 0)
#define NSIGILL  8
#define NSIGFPE  8
#define NSIGSEGV 2
#define NSIGBUS  3
#define NSIGTRAP 2
#define NSIGCLD  6 /* In 2.2.11 this is NSIGCHLD defined without a value */
#define NSIGPOLL 6

#ifdef USE_PTHREADS
#include <pthread.h>
#include <semaphore.h>
#endif
extern int _JVM_native_threads;
#endif

#include "sys_api.h"

#include "oobj.h"
#include "interpreter.h"
#include "threads.h"

static char *signalNames[] = {
    "no signal",
    "SIGHUP    1    hangup",
    "SIGINT    2    interrupt",
    "SIGQUIT   3*   quit",
    "SIGILL    4*   illegal instruction",
    "SIGTRAP   5*   trace trap",
#ifdef __linux__
    "SIGABRT   6*   abort (generated by abort(3) routine, also SIGIOT)",
    "SIGBUS    7*   bus error",
#else
    "SIGABRT   6*   abort (generated by abort(3) routine)",
    "SIGEMT    7*   emulator trap",
#endif
    "SIGFPE    8*   arithmetic exception",
    "SIGKILL   9    kill (cannot be caught, blocked, or ignored)",
#if defined(__linux__) && defined(i386)
    /* Beyond here the signal numbers are different on Linux... */
    "SIGUSR1   10   user defined signal 1",
    "SIGSEGV   11*  segmentation violation",
    "SIGUSR2   12   user defined signal 2",
    "SIGPIPE   13   write on a pipe or other socket with no one to read it",
    "SIGALRM   14   alarm clock",
    "SIGTERM   15   software termination signal",
    "SIGSTKFLT 16   stack fault",
    "SIGCHLD   17@  child status change",
    "SIGCONT   18@  continue after stop",
    "SIGSTOP   19   stop (cannot be caught, blocked, or ignored)",
    "SIGTSTP   20+  stop signal generated from keyboard",
    "SIGTTIN   21+  background read attempted from control terminal",
    "SIGTTOU   22+  background write attempted to control terminal",
    "SIGURG    23@  urgent condition present on socket",
    "SIGXCPU   24   cpu time limit exceeded (see getrlimit(2))",
    "SIGXFSZ   25   file size limit exceeded (see getrlimit(2))",
    "SIGVTALRM 26   virtual time alarm (see getitimer(2))",
    "SIGPROF   27   profiling timer alarm (see getitimer(2))",
    "SIGWINCH  28@  window changed (see termio(4) and win(4S))",
    "SIGPOLL   29   pollable event occurred (also SIGIO)",
    "SIGPWR    30   power-fail restart",
    "SIGUNUSED 31   unused",
    "SIGRTMIN  32   first (high-priority) realtime signal",
    "SIGRTMAX  63   last (lowest-priority) realtime signal"
#else
    "SIGBUS    10*  bus error",
    "SIGSEGV   11*  segmentation violation",
    "SIGSYS    12*  bad argument to system call",
    "SIGPIPE   13   write on a pipe or other socket with no one to read it",
    "SIGALRM   14   alarm clock",
    "SIGTERM   15   software termination signal",
    "SIGUSR1   16   user defined signal 1",
    "SIGUSR2   17   user defined signal 2",
    "SIGCHLD   18@  child status change",
    "SIGPWR    19   power-fail restart",
    "SIGWINCH  20@  window changed (see termio(4) and win(4S))",
    "SIGURG    21@  urgent condition present on socket",
    "SIGPOLL   22   pollable event occurred (also SIGIO)",
    "SIGSTOP   23   stop (cannot be caught, blocked, or ignored)",
    "SIGTSTP   24+  stop signal generated from keyboard",
    "SIGCONT   25@  continue after stop",
    "SIGTTIN   26+  background read attempted from control terminal",
    "SIGTTOU   27+  background write attempted to control terminal",
    "SIGVTALRM 28   virtual time alarm (see getitimer(2))",
    "SIGPROF   29   profiling timer alarm (see getitimer(2))",
    "SIGXCPU   30   cpu time limit exceeded (see getrlimit(2))",
    "SIGXFSZ   31   file size limit exceeded (see getrlimit(2))",
    "SIGWAITING 32  process' lwps are blocked",
    "SIGLWP    33   private to thread library",
    "SIGFREEZE 34   private to checkpoint/resume",
    "SIGTHAW   35   private to checkpoint/resume",
    "SIGRTMIN  36   first (high-priority) realtime signal",
    "SIGRTMAX  43   last (lowest-priority) realtime signal"
#endif
};

#ifdef DECODE_SIGINFO
/*
 * Decode a siginfo_t structure, and print out something interesting
 * about it.  Truncate and succeed if the buffer write overflows.  It's
 * not crucial that this does exactly the right thing.
 */
static char *
decode_si_code(int sig, siginfo_t *info)
{
    char *unknown = "unknown siginfo";
#define MSGMAXLEN 1024
    static char msg[MSGMAXLEN];
    int si_code = info->si_code;

#ifndef __linux__
    if (si_code == SI_NOINFO)
	return "no information";
    else if (SI_FROMUSER(info)) {
#else
    if (SI_FROMUSER(info)) {
#endif
#ifdef __linux__
	char *si_user[] = {
	    "SI_USER", "SI_QUEUE", "SI_TIMER",
	    "SI_MESGQ", "SI_ASYNCIO", "SI_SIGIO"
	};
	if (si_code < SI_SIGIO || si_code > 0) {
	    return unknown;
	}
#else
	char *si_user[] = {
	    "SI_USER", "SI_LWP", "SI_QUEUE",
	    "SI_TIMER", "SI_ASYNCIO", "SI_MESGQ"
	};
	if (si_code < SI_MESGQ || si_code > 0) {
	    return unknown;
	}
#endif
	si_code = abs(si_code);
	jio_snprintf(msg, sizeof(msg), "%s [pid: %ld, uid: %ld]",
#ifdef __linux__
                    si_user[si_code],
                      (long) (int) info->si_pid,
                      (long) (unsigned short int) info->si_uid);
#else
		     si_user[si_code], (long)info->si_pid, (long)info->si_uid);
#endif
	return msg;
    } else {	/* SI_FROMKERNEL */
	switch(sig) {
	    case SIGILL: {
		char *si_code_name[] = {
		    "ILL_ILLOPC", "ILL_ILLOPN", "ILL_ILLADR",
		    "ILL_ILLTRP", "ILL_PRVOPC", "ILL_PRVREG",
		    "ILL_COPROC", "ILL_BADSTK"
		};
		if (si_code > NSIGILL || si_code == 0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%s [addr: 0x%p]\n",
			     si_code_name[si_code-1], info->si_addr);
		break;
	    }
	    case SIGFPE: {
		char *si_code_name[] = {
		    "FPE_INTDIV", "FPE_INTOVF", "FPE_FLTDIV",
		    "FPE_FLTOVF", "FPE_FLTUND", "FPE_FLTRES",
		    "FPE_FLTINV", "FPE_FLTSUB"
		};
		if (si_code > NSIGFPE || si_code == 0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%s [addr: 0x%p]\n",
			     si_code_name[si_code-1], info->si_addr);
		break;
	    }
	    case SIGSEGV: {
		char *si_code_name[] = {
		    "SEGV_MAPERR", "SEGV_ACCERR"
		};
		if (si_code > NSIGSEGV || si_code == 0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%s [addr: 0x%p]\n",
			     si_code_name[si_code-1], info->si_addr);
		break;
	    }
	    case SIGBUS: {
		char *si_code_name[] = {
		    "BUS_ADRALN", "BUS_ADRERR", "BUS_OBJERR"
		};
		if (si_code > NSIGBUS || si_code == 0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%s [addr: 0x%p]\n",
			     si_code_name[si_code-1], info->si_addr);
		break;
	    }
	    case SIGTRAP: {
		char *si_code_name[] = {
		    "TRAP_BRKPT", "TRAP_TRACE"
		};
		if (si_code > NSIGTRAP || si_code == 0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%p\n",
			     si_code_name[si_code-1]);
		break;
	    }
	    case SIGCHLD: {
		char *si_code_name[] = {
		    "CLD_EXITED", "CLD_KILLED", "CLD_DUMPED",
		    "CLD_TRAPPED", "CLD_STOPPED", "CLD_CONTINUED"
		};
		if (si_code > NSIGCLD || si_code ==  0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%s [pid: %ld, status: %ld]\n",
			     si_code_name[si_code-1], (long)info->si_pid,
			     (long)info->si_status);
		break;
	    }
	    case SIGPOLL: {
		char *si_code_name[] = {
		    "POLL_IN", "POLL_OUT", "POLL_MSG",
		    "POLL_ERR", "POLL_PRI", "POLL_HUP"
		};
		if (si_code > NSIGPOLL || si_code == 0) {
		    return unknown;
		}
		jio_snprintf(msg, sizeof(msg), "%s [band: %ld]\n",
			     si_code_name[si_code-1], (long)info->si_band);
		break;
	    }
	    default:
		return unknown;
	}
	return msg;
    }
}
#endif

#ifndef __linux__
/* There is a race condition here. We use a global variable to store the
 * thread ID that raised SIGQUIT. This is no big deal because SIGQUIT is
 * only used for informational purposes.
 */
static sys_thread_t *sigquit_thread = NULL;
#endif

static void UserHandler(int sig, void *siginfo, void *uc)
{
#ifndef __linux__
    if (sig == SIGQUIT) {
        sigquit_thread = sysThreadSelf();
    }
#endif

    sysSignalNotify(sig);

    /* We need to reinstate the signal handler each time... */
    sysSignal(sig, UserHandler);
}

static void
panicHandler(int sig, void *siginfo, void *uc)
{
#ifdef SA_SIGINFO
    siginfo_t *info = siginfo;
#endif
    static int count = 0;

    if (count > 0) {
#if defined(USE_PTHREADS) && defined(__linux__)
        exit(1);
#else
	Abort();
#endif
    }

#if defined(IMPLICIT_EXCEPTIONS) && defined(i386) && !defined(DEBUG)
    if (sig == SIGFPE && info->si_code == FPE_INTDIV) {
	/*
	 * instruction locations inside assembly
	 * interp for divide trapping
	 */
	extern int __idiv_inst5(void);
	extern int __idiv_trap5(void);
	extern int __idiv_inst6(void);
	extern int __idiv_trap6(void);
	extern int __irem_inst5(void);
	extern int __irem_trap5(void);
	extern int __irem_inst6(void);
	extern int __irem_trap6(void);
#ifndef __linux__
	int pc = ((ucontext_t*)uc)->uc_mcontext.gregs[PC];
	if (pc == (int) &__idiv_inst5) {
	    ((ucontext_t*)uc)->uc_mcontext.gregs[PC] = (int) &__idiv_trap5;
	    goto done;
	} else if (pc == (int) &__idiv_inst6) {
	    ((ucontext_t*)uc)->uc_mcontext.gregs[PC] = (int) &__idiv_trap6;
	    goto done;
	} else if (pc == (int) &__irem_inst5) {
	    ((ucontext_t*)uc)->uc_mcontext.gregs[PC] = (int) &__irem_trap5;
	    goto done;
	} else if (pc == (int) &__irem_inst6) {
	    ((ucontext_t*)uc)->uc_mcontext.gregs[PC] = (int) &__irem_trap6;
	    goto done;
	}
#endif
    }
#endif /* IMPLICIT_EXCEPTIONS, i386, !DEBUG */

#ifdef SA_SIGINFO
    if (CompiledCodeSignalHandler(sig, info, uc) == TRUE)
#else
    if (CompiledCodeSignalHandler(sig, siginfo, uc) == TRUE)
#endif
    {
	goto done;
    }

    count += 1;

#if defined(__linux__) && defined(NSIG)
   if (sig > NSIG) {
#else
    if (sig > MAXSIG) {
#endif
	jio_fprintf(stderr, "Unknown signal: %d\n", sig);
    } else {
#if defined(__linux__) && defined(SIGRTMIN) && defined(SIGRTMAX)
        if (sig == SIGRTMAX) {
            jio_fprintf(stderr, "%s\n", signalNames[SIGRTMIN + 1]);
        } else if (sig > SIGRTMIN && sig < SIGRTMAX) {
            jio_fprintf(stderr, "          %d   realtime signal %d", 
                        sig, sig - SIGRTMIN);
        } else {
#endif
	jio_fprintf(stderr, "%s\n", signalNames[sig]);
#if defined(__linux__) && defined(SIGRTMIN) && defined(SIGRTMAX)
        }
#endif
    }

#ifdef DECODE_SIGINFO
    if (info) {
	/* print out signal info */
#ifndef __linux__
	if (info->si_code != SI_NOINFO) {
#endif
	    jio_fprintf(stderr, "    si_signo [%d]: %s\n",
			info->si_signo, signalNames[info->si_signo]);
	    jio_fprintf(stderr, "    si_errno [%d]: %s\n",
			info->si_errno, strerror(info->si_errno));
	    jio_fprintf(stderr, "    si_code [%d]: %s\n",
			info->si_code, decode_si_code(sig, info));
#ifndef __linux__
	}
#endif
        if (sysThreadSelf()) {
            jio_fprintf(stderr, "\tstackpointer=%p\n",
              		sysThreadStackPointer(sysThreadSelf()));
        }
    }
#endif

    HandleSignalInVM(FALSE);

#if defined(__linux__) && defined(USE_PTHREADS)
    exit(1);
#else
    Abort();
#endif

done:
    /* We need to reinstate the signal handler each time... */
    sysSignal(sig, panicHandler);
}

JNIEXPORT void * JNICALL
JVM_RegisterSignal(jint sig, void *handler)
{
    signal_handler_t newHandler =
        handler == (void *)2 ? UserHandler : (signal_handler_t)handler;
    signal_handler_t oldHandler;

    switch (sig) {
    case SIGFPE:
    case SIGSEGV:
    case SIGILL:
        return (void *)-1; /* already used by VM */
	
    case SIGQUIT:
    case SIGABRT:
    case SIGBUS:
#ifdef SIGEMT
    case SIGEMT:
#endif
#ifdef SIGSYS
    case SIGSYS:
#endif
    case SIGTRAP:
    case SIGXCPU:
    case SIGXFSZ:
        if (!reduce_signal_usage)
	    return (void *)-1;
    }

    oldHandler = sysSignal(sig, newHandler);
    if (oldHandler == UserHandler) {
        return (void *)2;
    } else {
        return (void *)oldHandler;
    }
}

JNIEXPORT void JNICALL
JVM_RaiseSignal(jint sig)
{
    sysRaise(sig);
}

JNIEXPORT jint JNICALL
JVM_FindSignal(const char *name)
{
    if (!strcmp(name, "HUP"))  return SIGHUP;
    if (!strcmp(name, "INT"))  return SIGINT;
    if (!strcmp(name, "QUIT"))  return SIGQUIT;
    if (!strcmp(name, "ILL"))   return SIGILL;
    if (!strcmp(name, "TRAP"))  return SIGTRAP;
    if (!strcmp(name, "ABRT"))  return SIGABRT;
#ifdef __linux__
    if (!strcmp(name, "IOT"))  return SIGIOT;
#endif
#ifdef SIGEMT
    if (!strcmp(name, "EMT"))   return SIGEMT;
#endif
    if (!strcmp(name, "FPE"))   return SIGFPE;
    if (!strcmp(name, "KILL"))  return SIGKILL;
    if (!strcmp(name, "BUS"))   return SIGBUS;
    if (!strcmp(name, "SEGV"))  return SIGSEGV;
#ifdef SIGSYS
    if (!strcmp(name, "SYS"))   return SIGSYS;
#endif
    if (!strcmp(name, "PIPE"))  return SIGPIPE;
    if (!strcmp(name, "ALRM"))  return SIGALRM;
    if (!strcmp(name, "TERM"))  return SIGTERM;
#if defined (__linux__) && !defined (SIGSTKFLT)
    if (!strcmp(name, "STKFLT")) return 16;
#elsif defined (__linux__)
    if (!strcmp(name, "STKFLT")) return SIGSTKFLT;
#endif
    if (!strcmp(name, "USR1"))  return SIGUSR1;
    if (!strcmp(name, "USR2"))  return SIGUSR2;
    if (!strcmp(name, "CHLD"))  return SIGCHLD;
#if defined(__linux__) && !defined(SIGPWR)
    if (!strcmp(name, "PWR"))    return 30;
#else
    if (!strcmp(name, "PWR"))   return SIGPWR;
#endif
    if (!strcmp(name, "WINCH")) return SIGWINCH;
    if (!strcmp(name, "URG"))   return SIGURG;
    if (!strcmp(name, "POLL"))  return SIGPOLL;
    if (!strcmp(name, "IO"))    return SIGIO;
    if (!strcmp(name, "STOP"))  return SIGSTOP;
    if (!strcmp(name, "TSTP"))  return SIGTSTP;
    if (!strcmp(name, "CONT"))  return SIGCONT;
    if (!strcmp(name, "TTIN"))  return SIGTTIN;
    if (!strcmp(name, "TTOU"))  return SIGTTOU;
    if (!strcmp(name, "VTALRM")) return SIGVTALRM;
    if (!strcmp(name, "PROF"))  return SIGPROF;
    if (!strcmp(name, "XCPU"))  return SIGXCPU;
    if (!strcmp(name, "XFSZ"))  return SIGXFSZ;
#if defined (__linux__) && !defined(SIGUNUSED)
    if (!strcmp(name, "UNUSED")) return 31;
#elsif defined(__linux__)
    if (!strcmp(name, "UNUSED")) return SIGUNUSED;
#endif
#ifdef SIGSWI
    if (!strcmp(name, "SWI"))    return SIGSWI;
#endif
    return -1;
}

#if defined(__linux__)  && defined(USE_PTHREADS)
/*
 * We cannot support the general signal dispatcher thread on Linux due to
 * problems in LinuxThreads signal handling. Instead, we create a dedicated
 * thread for handling SIGQUIT only, and other registered signal handlers
 * will not be invoked.
 */
static sem_t sigquit_sem;

static void
#ifdef SA_SIGINFO
sigquit_handler(int sig, siginfo_t* info, void* context)
#else
sigquit_handler(int sig)
#endif
{
    psignal(sig , "Received signal");
    /* Notify handler thread that SIGQUIT was received */
#ifdef  DECODE_SIGINFO
        jio_fprintf(stderr, "    si_signo [%d]: %s\n",
                    info->si_signo, signalNames[info->si_signo]);
        jio_fprintf(stderr, "    si_errno [%d]: %s\n",
                    info->si_errno, strerror(info->si_errno));
        jio_fprintf(stderr, "    si_code [%d]: %s\n",
                    info->si_code, decode_si_code(sig, info));
        jio_fprintf(stderr, "    si_code [%d %hu]\n",
                    info->si_pid, info->si_uid);
#endif
      jdk_sem_post(&sigquit_sem);
}

static void sigquit_thread(void *arg)
{
    struct sigaction act;
    sigset_t set;

    /* Initialize SIGQUIT handler semaphore */
    jdk_sem_init(&sigquit_sem, 0, 0);
    /* Set SIGQUIT signal handler */
#ifdef SA_SIGINFO
    act.sa_handler = 0;
    act.sa_sigaction = sigquit_handler;
#else
    act.sa_handler = sigquit_handler;
#endif
    sigfillset(&act.sa_mask);
#ifdef SA_SIGINFO
    act.sa_flags = SA_SIGINFO | SA_RESTART;
#else
    act.sa_flags = SA_RESTART;
#endif
    sigaction(SIGQUIT, &act, 0);
    /* Unblock SIGQUIT in this thread */
    sigemptyset(&set);
    sigaddset(&set, SIGQUIT);
    jdk_pthread_sigmask(SIG_UNBLOCK, &set, 0);
    /* Wait on semaphore that is posted each time a SIGQUIT is received */
    while (1) {
      jdk_sem_wait(&sigquit_sem);
      jio_fprintf(stderr, "SIGQUIT\n");
      HandleSignalInVM(TRUE);
    }
}

#endif

static void sigthread(void *arg)
{
    while (1) {
        int sig = sysSignalWait();
	if (sig == SIGQUIT) {
	    jio_fprintf(stderr, "SIGQUIT\n");
	    
	    HandleSignalInVM(TRUE);
#ifndef __linux__
	    sigquit_thread = NULL;
#endif
	} else {
	    ExecEnv *ee = EE();
	    ClassClass *sigCls = FindStickySystemClass(ee, "sun/misc/Signal",
						       TRUE);
	    if (sigCls == NULL) {
	        exceptionClear(ee);
		continue;
	    }
	    execute_java_static_method(ee, sigCls, "dispatch", "(I)V", sig);
	    exceptionClear(ee);
	}
    }
}

void InitializeSignals()
{
    /* We have to install these signal handlers because Sun JIT relies on them
     * to be delivered.
     */
    sysSignal(SIGFPE, panicHandler);
    sysSignal(SIGSEGV, panicHandler);
    sysSignal(SIGILL, panicHandler);

#ifdef __linux__
    if (!_JVM_native_threads) {
        createSystemThread("Signal dispatcher",
                           MaximumPriority, 0,
                           sigthread, 0);
    }
#else
    createSystemThread("Signal dispatcher", MaximumPriority, 0, 
		       sigthread, 0);
#endif

    if (!reduce_signal_usage) {
#ifdef __linux__
      /* SIGQUIT will be blocked in all threads except SIGQUIT handler */
       if (_JVM_native_threads) {
            sigset_t set;
            pthread_t thr;
            sigemptyset(&set);
            sigaddset(&set, SIGQUIT);
            jdk_pthread_sigmask(SIG_BLOCK, &set, 0);
            /* Create the SIGQUIT handler thread */
            createSystemThread("SIGQUIT handler",
                               MaximumPriority, 0,
                               sigquit_thread, 0);
        } else {
           sysSignal(SIGQUIT, UserHandler);
        }
#else
        sysSignal(SIGQUIT, UserHandler);
#endif

	sysSignal(SIGABRT, panicHandler);
	sysSignal(SIGBUS, panicHandler);
	sysSignal(SIGIOT, panicHandler);
#ifdef SIGEMT
	sysSignal(SIGEMT, panicHandler);
#endif
#ifdef SIGSYS
	sysSignal(SIGSYS, panicHandler);
#endif
	sysSignal(SIGTRAP, panicHandler);
	sysSignal(SIGXCPU, panicHandler);
	sysSignal(SIGXFSZ, panicHandler);
    }
}
