/*
 * @(#)tnf_support.c	1.8 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include <sys/types.h>
#include <tnf/probe.h>

#include "threads.h"
#include "binclasstable.h"

/* Record the name and address of a class.  Used to identify the class
   name given the address of the class. */
static void *write_bin_class(Class *cb, void *dont_use)
{
    TNF_PROBE_2(tnf_atexit_classes, "monitor tnf_atexit", "sunw%debug class names",
        tnf_opaque, class_object, cb,
        tnf_string, class_name, cbName(cb));
    return (void *) NULL;
}
 
void write_mutex_name(void *addr, char *name, int tid)
{
    /* This function is used so that the probe "ee_mutex_name" is unique
       within the vm (i.e., this function is called whenever the name
       and address of a mutex is being recorded as opposed to calling
       a different probe each time.  Different probes with the same name
       are cause some problems with the tnf analysis tools. */
    char name_tid[72]; /* The length must hold "monitorLockMutex" 
			  + thread id + 1 */
    TNF_PROBE_2(ee_mutex_name, "monitor tnf_atexit", "sunw%debug ee mutexes",
        tnf_opaque, lock_addr1, addr,
        tnf_string, lock_name1,
        (sprintf(name_tid, "%s-%d", name, tid), name_tid));
}
 
/* Function passed to iterator to log EE mutexes of thread. */
static int
tnf_EEHelper(sys_thread_t *thr, void *not_used) {
    ExecEnv *ee = sysThread2EE(thr);
    if (EETable_reference(ee)) {
        write_mutex_name((void *)&ee->monitorLockMutex, "monitorLockMutex",
            EE2sysThread(ee)->sys_thread);
        write_mutex_name((void *)&ee->metaLockMutex, "metaLockMutex",
            EE2sysThread(ee)->sys_thread);
        EETable_unreference(ee);
    }
    return 0;
}    

/* Install tnf probes to be executed on exit */
void
tnf_atexit(void) {
    /* log the names of the classes */
    WalkBinClasses(write_bin_class, (void *) NULL);
    /* log all the lock addresses with their names */
    tnf_lock_names();
    /* record the names and address of mutexes in each EE. */
    sysThreadEnumerateOver(tnf_EEHelper, 0);
}        

