/*
 * @(#)util_md.c	1.8 00/05/04
 * 
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

/*
 * Solaris and machine-specific utility code
 */

#include <sys/processor.h>

int getProcessorMhz()
{
    /* Find an on-line processor and return its clock frequency in Mhz. */

    processor_info_t pi;
    processorid_t p = 0;
    while (p_online(p, P_STATUS) != P_ONLINE) p++;
    if (processor_info(p, &pi) == 0) {
        return pi.pi_clock;
    }
    return 0;
}

#ifdef USE_PIC_PROFILERS
/*
 * This code is used on UltraSPARC-based systems for high-precision
 * measurement (using the hstat kernel and/or driver).
 */

#ifdef sparc

# include <stdio.h>
# include "util_md.h"
# include "sys_api.h"

static PICProfiler* profilers = NULL;

static void 
PrintPICProfiler(PICProfiler* profiler) 
{
    int n, i;
    fprintf(stderr, "PIC data for %s:\n", profiler->name);
    if (profiler->calls > profiler->saved_size) {
	n = profiler->saved_size;
	i = profiler->calls % profiler->saved_size;
    } else {
	n = profiler->calls;
	i = 0;
    }
    while (n-- > 0) {
      fprintf(stderr, "%u %u\n", 
	      profiler->saved[i].regs.a, profiler->saved[i].regs.b);
      i= (i + 1) % profiler->saved_size;
    }
    fprintf(stderr,
	    "PIC totals %s: %d calls\nA=%llu B=%llu  A/call=%G B/call=%G\n",
	    profiler->name, profiler->calls,
	    profiler->accum_a, profiler->accum_b,
	    (double)profiler->accum_a / profiler->calls,
	    (double)profiler->accum_b / profiler->calls);
}

static void 
PrintPICProfilers() 
{
    PICProfiler* profiler = profilers;
    while (profiler != NULL) {
	PrintPICProfiler(profiler); profiler = profiler->next;
    }
}

void 
PICprofilerInit(PICProfiler* profiler, char* name, int n_to_save) 
{
    if (profilers == NULL) {
	sysAtexit(&PrintPICProfilers);
    }
    if (profiler->name != NULL) return;
    if (n_to_save == 0) n_to_save= 1;
    profiler->name = name;
    profiler->calls = 0;
    profiler->accum_a = 0;
    profiler->accum_b = 0;
    profiler->saved_size = n_to_save;
    profiler->next_to_save = 0;
    profiler->saved = (pic*)sysCalloc(n_to_save, sizeof(pic));
    profiler->next = profilers;
    profilers = profiler;
}

void 
PICprofilerEndFN(PICProfiler* profiler, pic *end) 
{
    end->regs.a -= profiler->start.regs.a;
    end->regs.b -= profiler->start.regs.b;
    profiler->saved[profiler->next_to_save] = *end;
    profiler->next_to_save =
        (profiler->next_to_save + 1) % profiler->saved_size;
    profiler->accum_a += end->regs.a;
    profiler->accum_b += end->regs.b;
    profiler->calls++;
}
#endif /* sparc */
#endif /* USE_PIC_PROFILERS */

#ifdef __i386

extern long setfpcontrolword(long cw);

long setFPControlWord(long cw)
{
   return setfpcontrolword(cw);
}
#endif
