/*
 * @(#)proc_service_md.c	1.6 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */


/*_MODULE_IMPORT_____________________________________________________
*
*  Description: 
*	This module contains the functions that are used by the
* interface to interact with the process.  These functions
* are normally provided by the debugger.
_____________________________________________________________________*/

#include <stdarg.h>

/* This code is not needed yet and causes 5.5.1 build problems. */
#undef PROC_SERVICE
#ifdef PROC_SERVICE  

#include <dlfcn.h>
#include <stdio.h>
#include <strings.h>
#include <proc_service.h>

#include "util.h"
#include "proc_service_md.h"

/* In SunOS 5.7, the const keyword was removed from the ps_prochandle arg
 *   on most of the prototypes in /usr/include/proc_service.h.
 *   In addition other arguments changed in the read/write prototypes.
 *   This was the best way to deal with this OS /usr/include difference
 *   that I could think of. This allows us to compile on 5.7.
 */
#ifdef __SunOS_5_7 		/* defined by Sun compilers. */
#define PS_PROCHANDLE_PTR_ARG       struct ps_prochandle *
#define PS_SIZE_ARG       	    size_t
#define PS_READ_PTR_ARG       	    void *
#define PS_WRITE_PTR_ARG       	    const void *
#else
#define PS_PROCHANDLE_PTR_ARG const struct ps_prochandle *
#define PS_SIZE_ARG       	    int
#define PS_READ_PTR_ARG       	    char *
#define PS_WRITE_PTR_ARG       	    char *
#endif

#define IMPORT_MAX_BUFFER_SIZE 512

static int td_import_pub_debug = 1;

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_pstop(PS_PROCHANDLE_PTR_ARG ph)

/*_____________________________________________________________________
*	Description:    
*	   Stop a process.  Return after process has
*	been stopped.
*	   tdb will call this function when it is processing
*	a request that requires that the process be stopped.
*	
*	Input:
*	   *ph - process handle defined by debugger
*	
*	Output: 
*	   ps_pstop - return value
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }

   return return_val ;
}


/*_PROCEDURE_PUBLIC__________________________________________________*/

ps_err_e ps_pcontinue(PS_PROCHANDLE_PTR_ARG ph)

/*_____________________________________________________________________
*	Description:    
*	   Continue a process that was stopped 
*	by ps_pstop().
*	   If tdb has stopped a process with ps_pstop()
*	in order to perform some requested operations,
*	it will continue the process with this call.
*
*	Input:
*	   *ph - process handle defined by debugger
*
*	Output: 
*	   ps_pcontinue - return value 
*
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }

   return return_val ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_lstop(PS_PROCHANDLE_PTR_ARG ph, lwpid_t lwpid)

/*_____________________________________________________________________
*	Description:    
*	   Stop an LWP.  Return after LWP has been stopped.
*	   tdb will call this function when it is processing
*	a request that requires that the LWP be stopped.
* 
*	Input: 
*	   *ph - process handle defined by debugger
*	   lwpid - identifier for LWP
* 
*	Output: 
*	   ps_lstop - return value
*
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }
   return return_val ;
}
 

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_lcontinue(PS_PROCHANDLE_PTR_ARG ph, lwpid_t lwpid)

/*_____________________________________________________________________
*	Description:    
*	   Continue an LWP that was stopped by ps_lstop().
*	   If tdb has stopped an LWP with ps_lstop()
*	in order to perform some requested operations,
*	it will continue the LWP with this call.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   lwpid - identifier for LWP
*
*	Output: 
*	   ps_lcontinue - return value
*
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }
   return return_val ;
}


/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_pglobal_lookup(PS_PROCHANDLE_PTR_ARG ph, const char *ld_object_name, 
		const char *ld_symbol_name, paddr_t *ld_symbol_addr)

/*_____________________________________________________________________
*	Description:    
*	   Given a symbol name and load object
*	(e.g., "libthread.so") find the absolute address of symbol. Only
*	global name space is searched.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   *ld_object_name - name of load object
*	   *ld_symbol_name - name of symbol in ld_object_name
*
*	Output: 
*	   *ld_symbol_addr - absolute byte address of symbol
*	      ld_symbol_name
*	   ps_pglobal_lookup - return value
*
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_ERR ;
   void *self_handle ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }
   /*
   *   For testing, use dlopen/dlsym to get addresses.
   */

   self_handle = dlopen( 0, RTLD_LAZY ) ;

   if ( self_handle ) {
      *ld_symbol_addr = (paddr_t) dlsym( self_handle, ld_symbol_name ) ;
      if ( *ld_symbol_addr ) {
	 return_val = PS_OK ;
      }
      else {
	 if ( td_import_pub_debug ) {
	    printf("Symbol not found in ps_pglobal_lookup\n") ;
 	 }
	 return_val = PS_NOSYM ;
      }
      (void) dlclose(self_handle);
   }
   else {
      if ( td_import_pub_debug ) {
         printf("dlopen error in ps_pglobal_lookup\n") ;
      }
      return_val = PS_ERR ;
   }

   return return_val ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_pdread(PS_PROCHANDLE_PTR_ARG ph, paddr_t addr, PS_READ_PTR_ARG buf, PS_SIZE_ARG size)

/*_____________________________________________________________________
*	Description:    
*	   Read from a process's data segment.
* 
*	Input: 
*	   *ph - process handle defined by debugger
*	   addr - address in data segment of process
*	   size - number of bytes to read
*
*	Output: 
*	   *buf - contents of data segment read at addr for size
*	      bytes
*	   ps_pdread - return value
* 
*
_____________________________________________________________________*/
{
   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL 
   	|| (buf == NULL) || (addr == NULL) ) {
      return PS_ERR ;
   }
   /*
   *   Function for testing - direct access of variable address.
   */
#if 1
   {
      int i ;
      char *bytes = (char *) addr ;
      for (i=0; i<size; i++) {
	 ((char*)buf)[i] = *(bytes+i) ;
      }
   }
#else

   memcpy(buf, addr, size) ;
#endif

   return PS_OK ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_pdwrite(PS_PROCHANDLE_PTR_ARG ph, paddr_t addr, PS_WRITE_PTR_ARG buf, PS_SIZE_ARG size)

/*_____________________________________________________________________
*	Description:    
*	   Write to a process's data segment.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   addr - address in data segment of process
*	   *buf - buffer for data to be written
*	   size - number of bytes to be written
*
*	Output: 
*	   ps_pdwrite - return value
*
_____________________________________________________________________*/
{
   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL 
   	|| (buf == NULL) || (addr == NULL) ) {
      return PS_ERR ;
   }
   memcpy((char*)addr, buf, size) ;

   return PS_OK ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_ptread(PS_PROCHANDLE_PTR_ARG ph, paddr_t addr, 
		   PS_READ_PTR_ARG buf, PS_SIZE_ARG size)

/*_____________________________________________________________________
*	Description:    
*	   Read from a process's text segment.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   addr - address in text segment of process
*	   size - number of bytes to read
*
*	Output:
*	   *buf - contents of text segment read at addr for size
*	      bytes
*	   ps_ptread - return value
*
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL 
   	|| (buf == NULL) || (addr == NULL) ) {
      return PS_ERR ;
   }
   return return_val ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/



ps_err_e ps_ptwrite(PS_PROCHANDLE_PTR_ARG ph, paddr_t addr, 
		    PS_WRITE_PTR_ARG buf, PS_SIZE_ARG size)

/*_____________________________________________________________________
*	Description:    
*	   Write bytes to a process's text segment.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   addr - address in text segment of process
*	   *buf - buffer for text to be written
*	   size - number of bytes to be written
*
*	Output: 
*	   ps_ptwrite - return value
*
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL 
   	|| (buf == NULL) || (addr == NULL) ) {
      return PS_ERR ;
   }
   return return_val ;
}



/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_lgetregs(PS_PROCHANDLE_PTR_ARG ph, lwpid_t lid, prgregset_t gregset)

/*_____________________________________________________________________
*	Description:    
*	   Read an LWP's general register set.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   lwpid - identifier for LWP
*
*	Output: 
*	   *gregset - contents of general purpose register set
*	   ps_lgetregs - return value
*
_____________________________________________________________________*/
{
   ps_err_e return_val ;
   int lwpfd ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }

   return PS_ERR ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/


ps_err_e ps_lsetregs(PS_PROCHANDLE_PTR_ARG ph, lwpid_t lid, 
	  const prgregset_t gregset)

/*_____________________________________________________________________
*	Description:    
*	   Write an LWP's general register set.
*
*	Input: 
*	   *ph - process handle defined by debugger
*	   lwpid - identifier for LWP
*	   *gregset - contents of general purpose register set
*			to be written
*
*	Output: 
*	   ps_lsetregs - return value
* _____________________________________________________________________*/ 

{ 
   ps_err_e return_val = PS_ERR ; 
   int lwpfd ;

   if ( ph == NULL || ph == (struct ps_prochandle *)(-1) || ph->pfd == NULL ) {
      return PS_ERR ;
   }

   return PS_ERR ;

}



/*_PROCEDURE_PUBLIC__________________________________________________*/

void ps_plog(const char *fmt, ... )

/*_____________________________________________________________________
*	Description: 
*	   Log a message.  libthread_db will use ps_plog() to
*	write diagnostic messages if logging is turned on.
*	See td_log().
*
*	Input: 	
*	   fmt - printf style format for message
*	   variable number of arguments for message
*
*	Output: 	
*	   none
*
_____________________________________________________________________*/
{
   return ;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/

ps_err_e ps_lgetxregsize( PS_PROCHANDLE_PTR_ARG ph, 
			lwpid_t lid, int *xregsize)

/*_____________________________________________________________________
*	Description:	
*	   Get an LWP's extra register set size.
*
*	Input:		
*	   *ph - debugger defined process handle.
*	   lwpid - LWP identifier.
*
*	Output:		
*	   *xregsize - extra register set size.
*	   ps_lgetxregsize - return value.
*
_____________________________________________________________________*/
{
   return PS_ERR;
}


/*_PROCEDURE_PUBLIC__________________________________________________*/

ps_err_e ps_lgetxregs( PS_PROCHANDLE_PTR_ARG ph, 
			lwpid_t lid, caddr_t xregset)

/*_____________________________________________________________________
*	Description:	
*	   Read an LWP's extra register set.
*
*	Input:		
*	   *ph - debugger defined process handle.
*	   lwpid - LWP identifier.
*
*	Output:		
*	   *xregset - contents of extra register set.
*	   ps_lgetxregs - return value.
*
_____________________________________________________________________*/
{
   return PS_ERR;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/

ps_err_e ps_lsetxregs( PS_PROCHANDLE_PTR_ARG ph, 
			lwpid_t lid, caddr_t xregset)

/*_____________________________________________________________________
*	Description:	
*	   Write an LWP's extra register set.
*
*	Input:		
*	   *ph - debugger defined process handle.
*	   lwpid - LWP identifier.
*	   *xregset - contents of extra register set to be written.
*
*	Output:		
*	   ps_lsetxregs - return value.
*
_____________________________________________________________________*/
{
   return PS_ERR;
}

/*_PROCEDURE_PUBLIC__________________________________________________*/

ps_err_e ps_lgetfpregs(PS_PROCHANDLE_PTR_ARG ph, lwpid_t lid, prfpregset_t *fpregset)

/*_____________________________________________________________________
*	Description:    
*	   Read an LWP's floating point register set.
*
*	Input:
*	   *ph - process handle defined by debugger
*	   lwpid - identifier for LWP
*
*	Output:
*	   *fpregset - contents of floating point register set
*	   ps_lgetfpregs - return value
*
*	Side effects:
*	   none
* _____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;
   int lwpfd ;

   if ( ph == NULL ) {
      return PS_ERR ;
   }

   return PS_ERR ;
}
/*_PROCEDURE_PUBLIC__________________________________________________*/

ps_err_e ps_lsetfpregs(PS_PROCHANDLE_PTR_ARG ph, lwpid_t lid, const prfpregset_t *fpregset)

/*_____________________________________________________________________
*	Description:
*	   Write an LWP's floating point register set.
* 
*	Input:
*	   *ph - process handle defined by debugger
*	   lwpid - identifier for LWP
*	   *fpregset - contents of floating point register set
*		to be written
* 
*	Output:
*	   ps_lsetfpregs - return value
* 
*	Side effects:
*	   Floating point registers of LWP are changed.
* 
_____________________________________________________________________*/
{
   ps_err_e return_val = PS_OK ;
   int lwpfd ;

   if ( ph == NULL ) {
      return PS_ERR ;
   }

   return PS_ERR ;

}
#endif
