/*
 * @(#)SocketInputStream.c	1.25 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "jvm.h"
#include "jni_util.h"
#include "net_util.h"

#include "java_net_SocketInputStream.h"


/************************************************************************
 * SocketInputStream
 */

static jfieldID IO_fd_fdID;

static jfieldID sis_fdID;
static jfieldID sis_implID;

/*
 * Class:     java_net_SocketInputStream
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL 
Java_java_net_SocketInputStream_init(JNIEnv *env, jclass cls) {

    sis_fdID = (*env)->GetFieldID(env, cls, "fd", 
				  "Ljava/io/FileDescriptor;");
    IO_fd_fdID = NET_GetFileDescriptorID(env);
    sis_implID = (*env)->GetFieldID(env, cls, "impl", 
				    "Ljava/net/SocketImpl;");
}

/*
 * Class:     java_net_SocketInputStream
 * Method:    socketRead
 * Signature: ([BII)I
 */
JNIEXPORT jint JNICALL
Java_java_net_SocketInputStream_socketRead(JNIEnv *env, jobject this, 
					   jbyteArray data, jint off, jint len)
{
    char BUF[MAX_BUFFER_LEN];
    char *bufP;

    /* The fd field */
    jobject fdObj = (*env)->GetObjectField(env, this, sis_fdID);
    jint fd, timeout, nread;

    /* The impl field */
    jobject impl = (*env)->GetObjectField(env, this, sis_implID);

    jint datalen, n;

    if (IS_NULL(fdObj)) {
	/* should't this be a NullPointerException? -br */
        JNU_ThrowByName(env, JNU_JAVANETPKG "SocketException", 
			"null fd object");
	return -1;
    } else {
        fd = (*env)->GetIntField(env, fdObj, IO_fd_fdID);
        /* Bug 4086704 - If the Socket associated with this file descriptor
         * was closed (sysCloseFD), the the file descriptor is set to -1.
         */
        if (fd == -1) {
            JNU_ThrowByName(env, "java/net/SocketException", "Socket closed");
            return -1;
        }
    }
    if (IS_NULL(data)) {
	JNU_ThrowNullPointerException(env, "data argument");
        return -1;
    }
    if (IS_NULL(impl)) {
	JNU_ThrowNullPointerException(env, "socket impl");
	return -1;
    } else {
	timeout = (*env)->GetIntField(env, impl, psi_timeoutID);
    }

    datalen = (*env)->GetArrayLength(env, data);

    if (len < 0 || len + off > datalen) {
        JNU_ThrowByName(env, JNU_JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return -1;
    }

    if (len == 0) {
	return 0;
    }

    /*
     * If the read is greater than our stack allocated buffer then
     * we allocate from the heap (up to a limit)
     */
    if (len > MAX_BUFFER_LEN) {
	if (len > MAX_HEAP_BUFFER_LEN) {
	    len = MAX_HEAP_BUFFER_LEN;
	}
	bufP = (char *)malloc((size_t)len);
	if (bufP == NULL) {
	    bufP = BUF;
	    len = MAX_BUFFER_LEN;
	}
    } else {
	bufP = BUF;
    }

    if (timeout) {
	nread = JVM_Timeout(fd, timeout);
	if (nread <= 0) {
            if (nread == 0) {
                JNU_ThrowByName(env, JNU_JAVAIOPKG "InterruptedIOException",
			    "Read timed out");
            } else if (nread == JVM_IO_ERR) {
	        JNU_ThrowByName(env, JNU_JAVANETPKG "SocketException",
			    (errno == EBADF) ? "Socket closed" :
			    strerror(errno));
	    } else if (nread == JVM_IO_INTR) {
	        JNU_ThrowByName(env, JNU_JAVAIOPKG "InterruptedIOException",
			    "Operation interrupted");
	    }
	    if (bufP != BUF) {
	        free(bufP);
	    }
	    return -1;
	}
    }
    nread = JVM_Read(fd, bufP, len);
    if (nread < 0) {
	NET_ThrowCurrent(env, strerror(errno));
	nread = -1;
    } else {
        (*env)->SetByteArrayRegion(env, data, off, nread, (jbyte *)bufP);
    }

    if (bufP != BUF) {
	free(bufP);
    }
    return nread;
}					   
