/*
 * @(#)awt_Component.c	1.118 01/02/09
 *
 *  Copyright 2000-2001 Sun Microsystems, Inc. All rights reserved.
 *  Copyright 2000-2001 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "awt_p.h"
#include "canvas.h"
#include "awt_AWTEvent.h"
#include "VDrawingArea.h"
#include "java_awt_Color.h"
#include "java_awt_Cursor.h"
#include "java_awt_Font.h"
#include "java_awt_Point.h"
#include "java_awt_Component.h"
#include "java_awt_AWTEvent.h"
#include "java_awt_event_KeyEvent.h"
#include "java_awt_event_MouseEvent.h"
#include "sun_awt_motif_MComponentPeer.h"

#include "multi_font.h"
#include "jni.h"
#include "jni_util.h"
#include <jawt.h>

/* CanvasType widgets: Frame, Dialog, Window, Panel, Canvas, 
 *                     &  all lightweights (Component, Container) 
 */
#define IsCanvasTypeWidget(w) \
        XtIsSubclass(w, xmDrawingAreaWidgetClass) ||\
        XtIsSubclass(w, vDrawingAreaClass)

#include "awt_Component.h"
#include "awt_GraphicsEnv.h"

#include "awt_AWTEvent.h"
#include "awt_Cursor.h"

extern struct CursorIDs cursorIDs;
extern struct X11GraphicsConfigIDs x11GraphicsConfigIDs;

/* fieldIDs for Component fields that may be accessed from C */
struct ComponentIDs componentIDs;

/*
 * Class:     java_awt_Component
 * Method:    initIDs
 * Signature: ()V
 */

/* This function gets called from the static initializer for Component.java
   to initialize the fieldIDs for fields that may be accessed from C */

JNIEXPORT void JNICALL
Java_java_awt_Component_initIDs
  (JNIEnv *env, jclass cls)
{
    componentIDs.x = (*env)->GetFieldID(env, cls, "x", "I");
    componentIDs.y = (*env)->GetFieldID(env, cls, "y", "I");
    componentIDs.width = (*env)->GetFieldID(env, cls, "width", "I");
    componentIDs.height = (*env)->GetFieldID(env, cls, "height", "I");
    componentIDs.isPacked = (*env)->GetFieldID(env, cls, "isPacked", "Z");
    componentIDs.peer =
      (*env)->GetFieldID(env, cls, "peer", "Ljava/awt/peer/ComponentPeer;");
    componentIDs.background =
      (*env)->GetFieldID(env, cls, "background", "Ljava/awt/Color;");
    componentIDs.foreground =
      (*env)->GetFieldID(env, cls, "foreground", "Ljava/awt/Color;");
    componentIDs.graphicsConfig =
        (*env)->GetFieldID(env, cls, "graphicsConfig", 
                           "Ljava/awt/GraphicsConfiguration;");
    componentIDs.privateKey =
        (*env)->GetFieldID(env, cls, "privateKey", 
                           "Ljava/lang/Object;");
    componentIDs.name =
      (*env)->GetFieldID(env, cls, "name", "Ljava/lang/String;");

    /* Use _NoClientCode() methods for trusted methods, so that we
     *  know that we are not invoking client code on trusted threads
     */
    componentIDs.getParent = 
      (*env)->GetMethodID(env, cls, "getParent_NoClientCode",
						 "()Ljava/awt/Container;");

    componentIDs.getLocationOnScreen = 
      (*env)->GetMethodID(env, cls, "getLocationOnScreen_NoTreeLock",
						 "()Ljava/awt/Point;");
}

/* fieldIDs for MComponentPeer fields that may be accessed from C */
struct MComponentPeerIDs mComponentPeerIDs;

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    initIDs
 * Signature: ()V
 */

/* This function gets called from the static initializer for
   MComponentPeer.java to initialize the fieldIDs for fields that may
   be accessed from C */

JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_initIDs
  (JNIEnv *env, jclass cls)
{
    mComponentPeerIDs.pData = (*env)->GetFieldID(env, cls, "pData", "J");
    mComponentPeerIDs.target =
      (*env)->GetFieldID(env, cls, "target", "Ljava/awt/Component;");
    mComponentPeerIDs.jniGlobalRef =
      (*env)->GetFieldID(env, cls, "jniGlobalRef", "J");
    mComponentPeerIDs.graphicsConfig =
      (*env)->GetFieldID(env, cls, "graphicsConfig", 
			 "Lsun/awt/X11GraphicsConfig;");
    mComponentPeerIDs.drawState =
      (*env)->GetFieldID(env, cls, "drawState", "I");
}

/* field and method IDs for java.awt.Container. */
struct ContainerIDs containerIDs;

/*
 * Class:     java_awt_Container
 * Method:    initIDs
 * Signature: ()V
 */
/* This function gets called from the static initializer for java.awt.Container
   to initialize the fieldIDs for fields that may be accessed from C */
JNIEXPORT void JNICALL
Java_java_awt_Container_initIDs
  (JNIEnv *env, jclass cls)
{
    containerIDs.layoutMgr = 
      (*env)->GetFieldID(env, cls, "layoutMgr", "Ljava/awt/LayoutManager;");

    containerIDs.getComponents = 
      (*env)->GetMethodID(env, cls, "getComponents_NoClientCode",
						 "()[Ljava/awt/Component;");
    containerIDs.findComponentAt = 
      (*env)->GetMethodID(env, cls, "findComponentAt",
						 "(IIZZ)Ljava/awt/Component;");
}

/*
 * Fix for 4090493.  When Motif computes indicator size, it uses
 * (effectively) XmTextExtents, so the size of the indicator depends
 * on the text of the label.  The side effect is that if the label
 * text is rendered using different platform fonts (for a single Java
 * logical font) the display is inconsistent.  E.g. for 12pt font
 * English label will have a check mark, while Japanese label will
 * not, because underlying X11 fonts have different metrics.
 *
 * The fix is to override Motif calculations for the indicatorSize and
 * compute it ourselves based on the font metrics for all the platform
 * fonts given Java font maps onto.  Every time we set XmNfontList we
 * should set XmNindicatorSize as well.
 * 
 * The logic is in awt_computeIndicatorSize that just compute the
 * arithmetic mean of platform fonts by now.  HIE should take a look
 * at this.
 */

struct changeFontInfo {
    XmFontList fontList;	/* value to set */
    Boolean isMultiFont;	/* only need to compute for multifont */
    struct FontData *fontData;	/* need this to compute indicator size */
    Dimension indSize;		/* computed once by changeFont */

    Boolean    initialized;
    Boolean    error;
    JNIEnv     *env;
    jobject    fObj;
};

static void 
changeFont(Widget w, void *info)
{
    struct changeFontInfo *f = (struct changeFontInfo *)info;
    WidgetClass widgetClass;

    if (f->error)
	return;

    /* Some widgets use no fonts - skip them! */
    widgetClass = XtClass(w);
    if (widgetClass == xmDrawingAreaWidgetClass ||
	widgetClass == xmScrollBarWidgetClass   ||
	widgetClass == xmScrolledWindowWidgetClass)
	return;

    if (!f->initialized) {
	struct FontData *fdata;
	char *err;

	f->initialized = TRUE;

	/*Leak baby leak! */
	fdata = awtJNI_GetFontData(f->env, f->fObj, &err);
	if (fdata == 0) {
	    JNU_ThrowInternalError(f->env, err);
	    f->error = TRUE;
	    return;
	}

	if (awtJNI_IsMultiFont(f->env, f->fObj)) {
	    f->fontList = awtJNI_GetFontList(f->env, f->fObj);
	    f->isMultiFont = TRUE;
	} else {
	    f->fontList = XmFontListCreate(fdata->xfont, "labelFont");
	    f->isMultiFont = FALSE;
	}

	if (f->fontList == NULL) {
	    JNU_ThrowNullPointerException(f->env, "NullPointerException");
	    f->error = TRUE;
	    return;
	}
    }


    /* Fix for 4090493. */
    if (f->isMultiFont && XmIsToggleButton(w)) {
	Dimension indSize;

	/* Compute indicator size if first time through.  Note, that
	   ToggleButtons that are children of menus live in different
	   hieraqrchy (MenuComponent), so we don't check for this case
	   here.  In fact, the only time the XmNfontList is set on
	   MCheckboxMenuItemPeer widget is when it's created. */
	if (f->indSize == 0)
	    f->indSize = awt_computeIndicatorSize(f->fontData);

	XtVaSetValues(w, XmNfontList, f->fontList, NULL);
	if (f->indSize != MOTIF_XmINVALID_DIMENSION)
	    XtVaSetValues(w, XmNindicatorSize, f->indSize, NULL);
    }
    else {			/* usual case */
	XtVaSetValues(w, XmNfontList, f->fontList, NULL);
    }
}

static void 
changeForeground(Widget w, void *fg)
{
    XtVaSetValues(w, XmNforeground, fg, NULL);
}

static void 
changeBackground(Widget w, void *bg)
{
    Pixel fg;

    XtVaGetValues(w, XmNforeground, &fg, NULL);
    XmChangeColor(w, (Pixel) bg);
    XtVaSetValues(w, XmNforeground, fg, NULL);
}

static void 
setTreeTraversal(Widget w, Boolean trav)
{
    WidgetList children;
    int numChildren;
    int i;

    XtVaGetValues(w,
                  XmNchildren, &children,
                  XmNnumChildren, &numChildren,
                  NULL);

    for (i = 0; i < numChildren; i++) {
        if (!XtIsSubclass(children[i], xmScrollBarWidgetClass)) {
            XtVaSetValues(children[i], XmNtraversalOn, trav, NULL);
        }
        if (IsCanvasTypeWidget(children[i])) {
            setTreeTraversal(children[i], trav);
        }
    }
}

AwtGraphicsConfigDataPtr
getGraphicsConfigFromComponentPeer(JNIEnv *env, jobject this) {
    AwtGraphicsConfigDataPtr adata;
    jobject gc_object;
    
    /* GraphicsConfiguration object of MComponentPeer */
    gc_object = (*env)->GetObjectField(env, this,
                                       mComponentPeerIDs.graphicsConfig);

    if (gc_object != 0) {
        adata = (AwtGraphicsConfigDataPtr)
            JNU_GetLongFieldAsPtr(env, gc_object, 
                                  x11GraphicsConfigIDs.aData);
    } else {
        adata = getDefaultConfig(DefaultScreen(awt_display));
    }

    return adata;
}

AwtGraphicsConfigDataPtr
copyGraphicsConfigToPeer(JNIEnv *env, jobject this) {
    
    jobject component_object, gc_object;
    AwtGraphicsConfigDataPtr adata;
    
    /**
     * Copy the GraphicsConfiguration object from Component object to
     * MComponentPeer object.
     */
    component_object = (*env)->GetObjectField(env, this, 
                                              mComponentPeerIDs.target);
    /* GraphicsConfiguration object of Component */
    gc_object = (JNU_CallMethodByName(env, NULL, component_object,
                                      "getGraphicsConfiguration",
                                      "()Ljava/awt/GraphicsConfiguration;")).l;
    
    if (gc_object != 0) {
        /* Set graphicsConfig field of MComponentPeer */
        (*env)->SetObjectField (env, this,
                                mComponentPeerIDs.graphicsConfig,
                                gc_object);
        adata = (AwtGraphicsConfigDataPtr)
            JNU_GetLongFieldAsPtr(env, gc_object, 
                                  x11GraphicsConfigIDs.aData);
    } else {
        /* Component was not constructed with a GraphicsConfiguration
           object */
        adata = getDefaultConfig(DefaultScreen(awt_display));
    }
    
    return adata;
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    getNativeColor
 * Signature  (Ljava/awt/Color;Ljava/awt/GraphicsConfiguration;)I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MComponentPeer_getNativeColor
(JNIEnv *env, jobject this, jobject color, jobject gc_object) {
    AwtGraphicsConfigDataPtr adata;
    adata = (AwtGraphicsConfigDataPtr) JNU_GetLongFieldAsPtr(env, gc_object, 
                                                    x11GraphicsConfigIDs.aData);
    return awtJNI_GetColorForVis(env, color, adata);
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pInitialize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pInitialize
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;
    Widget parent;
    jobject target;
    jobject globalRef;
    EventMask xtMask;
    long awtMask = 0;
    AwtGraphicsConfigDataPtr adata;
    
    globalRef = (jobject)
	JNU_GetLongFieldAsPtr(env, this, mComponentPeerIDs.jniGlobalRef);

    AWT_LOCK();

    adata = copyGraphicsConfigToPeer(env, this);
    
    target = (*env)->GetObjectField(env, this, mComponentPeerIDs.target);
    
    cdata = (struct ComponentData *)
      JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (JNU_IsNull(env, this) || (cdata == 0)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    
    XtVaSetValues(cdata->widget,
                  XmNx, (*env)->GetIntField(env, target, componentIDs.x),
                  XmNy, (*env)->GetIntField(env, target, componentIDs.y),
                  XmNvisual, adata->awt_visInfo.visual,
                  XmNscreen, ScreenOfDisplay(awt_display,
                                  adata->awt_visInfo.screen),
                  NULL);


    /* For all but canvas-style components, pre-process
     * mouse and keyboard events (which means posting them
     * to the Java EventQueue before dispatching them to Xt).  
     * For canvas-style components ONLY pre-process mouse events 
     * because the input-method currently relies on key events 
     * being processed by Xt first.
     */
    awtMask = java_awt_AWTEvent_MOUSE_EVENT_MASK | 
              java_awt_AWTEvent_MOUSE_MOTION_EVENT_MASK;
    xtMask = ExposureMask | FocusChangeMask;

    if (!IsCanvasTypeWidget(cdata->widget)) {
        awtMask |= java_awt_AWTEvent_KEY_EVENT_MASK;
    } else {
        xtMask |= (KeyPressMask | KeyReleaseMask);
    }
    XtAddEventHandler(cdata->widget, xtMask,
                          True, awt_canvas_event_handler, globalRef);
    
    awt_addWidget(cdata->widget, cdata->widget, globalRef, awtMask);

    cdata->repaintPending = RepaintPending_NONE;

    /* If the widget was realized as it was instantiated, we
     * may need to adjust the stacking order since the XmNinsertPosition
     * proc doesn't result in proper stacking if the widget is realized.
     */
    if (XtIsRealized(cdata->widget) &&
        (parent = XtParent(cdata->widget)) != NULL) {
        WidgetList children;
        int num_children;
        Window *windows;
        int num_windows = 0;
        int i;

        XtVaGetValues(parent,
                      XmNnumChildren, &num_children,
                      XmNchildren, &children,
                      NULL);

        windows = (Window *) XtMalloc(num_children * sizeof(Window));
        for (i = 0; i < num_children; i++) {
            if (XtIsRealized(children[i])) {
                windows[num_windows++] = XtWindow(children[i]);
            }
        }
        XRestackWindows(awt_display, windows, num_windows);
        XtFree((char *) windows);
    }
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pShow
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pShow
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    awt_util_show(cdata->widget);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pHide
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pHide
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    awt_util_hide(cdata->widget);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pEnable
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pEnable
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    awt_util_enable(cdata->widget);
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pDisable
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pDisable
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    awt_util_disable(cdata->widget);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pReshape
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pReshape
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h)
{
    struct ComponentData *cdata;
    jint drawState;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    /* Set the draw state */
    drawState = (*env)->GetIntField(env, this,
        mComponentPeerIDs.drawState);
    (*env)->SetIntField(env, this,
        mComponentPeerIDs.drawState,
        drawState | JAWT_LOCK_BOUNDS_CHANGED | JAWT_LOCK_CLIP_CHANGED);
    awt_util_reshape(cdata->widget, x, y, w, h);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pDispose
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pDispose
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    XtUnmanageChild(cdata->widget);

    awt_delWidget(cdata->widget);
    awt_util_consumeAllXEvents(cdata->widget);
    awt_util_cleanupBeforeDestroyWidget(cdata->widget);
    XtDestroyWidget(cdata->widget);

    free((void *) cdata);
    (*env)->SetLongField(env,this,mComponentPeerIDs.pData,0);

    awtJNI_DeleteGlobalRef(env, this);

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pMakeCursorVisible
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pMakeCursorVisible
  (JNIEnv *env, jobject this) 
{
    struct ComponentData *cdata;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
	// need to change, may not be needed
    // awt_util_setCursor(cdata->widget, cdata->cursor);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pGetLocationOnScreen
 * Signature: ()Ljava/awt/Point;
 */
JNIEXPORT jobject JNICALL Java_sun_awt_motif_MComponentPeer_pGetLocationOnScreen
  (JNIEnv *env, jobject this)
{
    struct ComponentData *cdata;
    jobject point = 0;
    jclass clazz;
    jmethodID mid = 0;
    Position rx = 0;
    Position ry = 0;

    AWT_LOCK();
    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return NULL;
    }
    if (!XtIsRealized(cdata->widget)) {
        JNU_ThrowInternalError(env, "widget not visible on screen");
        AWT_UNLOCK();
        return NULL;
    }
    /* Translate the component to the screen coordinate system */
    XtTranslateCoords(cdata->widget, 0, 0, &rx, &ry);

    clazz = (*env)->FindClass(env, "java/awt/Point");
    mid = (*env)->GetMethodID(env, clazz, "<init>", "(II)V");
    if (mid != 0) {
        point = (*env)->NewObject(env, clazz, mid,
                                  (jint) rx, (jint) ry);
    }
    if (((*env)->ExceptionOccurred(env)) || JNU_IsNull(env, point)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return NULL;
    }
    AWT_UNLOCK();
    return point;
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    getParent_NoClientCode
 * Signature: (Ljava/awt/Component)Ljava/awt/Container;
 *
 * NOTE: This method may be called by privileged threads. 
 *       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
 */
JNIEXPORT jobject JNICALL Java_sun_awt_motif_MComponentPeer_getParent_1NoClientCode
  (JNIEnv *env, jclass thisClass, jobject component)
{
    jobject parent = NULL;

    /* getParent is actually getParent_NoClientCode() */
    parent = (*env)->CallObjectMethod(env,component,componentIDs.getParent);
    DASSERT(!((*env)->ExceptionOccurred(env)));
    return parent;
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    getComponents_NoClientCode
 * Signature: (Ljava/awt/Container)[Ljava/awt/Component;
 *               REMIND: Signature is incorrect for returned array value
 *
 * NOTE: This method may be called by privileged threads. 
 *       DO NOT INVOKE CLIENT CODE ON THIS THREAD!
 */
JNIEXPORT jobjectArray JNICALL Java_sun_awt_motif_MComponentPeer_getComponents_1NoClientCode
  (JNIEnv *env, jclass thisClass, jobject container)
{
    jobjectArray contents = NULL;
    contents = (*env)->CallObjectMethod(
				env, container, containerIDs.getComponents);
    DASSERT(!((*env)->ExceptionOccurred(env)));
    return contents;
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pSetForeground
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pSetForeground
  (JNIEnv *env, jobject this, jobject c) 
{
    struct ComponentData *bdata;
    Pixel color;
    AwtGraphicsConfigDataPtr adata;

    if (JNU_IsNull(env, c)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();
    bdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (bdata == 0 || bdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    
    adata = getGraphicsConfigFromComponentPeer(env, this);

    color = awtJNI_GetColorForVis (env, c, adata);
    XtVaSetValues(bdata->widget, XmNforeground, color, NULL);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pSetBackground
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pSetBackground
  (JNIEnv *env, jobject this, jobject c) 
{
    struct ComponentData *bdata;
    Pixel color;
    Pixel fg;
    AwtGraphicsConfigDataPtr adata;

    if (JNU_IsNull(env, c)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();
    bdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (bdata == 0 || bdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }


    adata = getGraphicsConfigFromComponentPeer(env, this);

    color = awtJNI_GetColorForVis (env, c, adata);
    XtVaGetValues(bdata->widget, XmNforeground, &fg, NULL);
    XmChangeColor(bdata->widget, color);
    XtVaSetValues(bdata->widget, XmNforeground, fg, NULL);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pSetScrollbarBackground
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pSetScrollbarBackground
  (JNIEnv *env, jobject this, jobject c) 
{
    struct ComponentData *bdata;
    Pixel color;
    Pixel fg;
    int                 i;
    WidgetList          wlist;
    Cardinal            wlen = 0;


    /* This method propagates the specified background color to the scrollbars in the composite widget.
     * Used to set background scrollbar color in List, TextArea, ScrollPane to its parent.
     */
    if (JNU_IsNull(env, c)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();
    bdata = (struct ComponentData *)
      JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (bdata == 0 || bdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    if (!XtIsComposite(bdata->widget)) {
        AWT_UNLOCK();
	return;
    }
    color = awtJNI_GetColor(env, c);

    XtVaGetValues(bdata->widget,
                  XmNchildren, &wlist,
                  XmNnumChildren, &wlen,
                  NULL);
    if (wlen > 0) {
        for (i=0; i < wlen; i++) {
		if (XtIsSubclass(wlist[i], xmScrollBarWidgetClass)) {
    		    XtVaGetValues(wlist[i], XmNforeground, &fg, NULL);
    		    XmChangeColor(wlist[i], color);
    		    XtVaSetValues(wlist[i], XmNforeground, fg, NULL);
		}
        }
        XtVaGetValues(bdata->widget, XmNforeground, &fg, NULL);
        XmChangeColor(bdata->widget, color);
        XtVaSetValues(bdata->widget, XmNforeground, fg, NULL);
    }

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pSetInnerForeground
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pSetInnerForeground
  (JNIEnv *env, jobject this, jobject c) 
{
    struct ComponentData *bdata;
    Pixel color;

    /* This method propagates the specified foreground color to all its children.
     * It is called to set foreground color in List, TextArea, ScrollPane.
     */
    if (JNU_IsNull(env, c)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();
    bdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (bdata == 0 || bdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    color = awtJNI_GetColor(env, c);
    awt_util_mapChildren(bdata->widget, changeForeground, 1, (void *) color);
    AWT_FLUSH_UNLOCK();
}


/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    isBackgroundInherited
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_motif_MComponentPeer_isBackgroundInherited
  (JNIEnv *env, jobject this) 
{
    jobject target;
    jobject color;

    if (JNU_IsNull(env, this)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return JNI_FALSE;
    }
    AWT_LOCK();

    target = (*env)->GetObjectField(env, this, mComponentPeerIDs.target);
    if (JNU_IsNull(env, target)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return JNI_FALSE;
    }
    color = (*env)->GetObjectField(env, target, componentIDs.background);
    if (JNU_IsNull(env, color)) {
	AWT_UNLOCK();
	return JNI_TRUE;
    } else {
	AWT_UNLOCK();
	return JNI_FALSE;
    }
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    isForegroundInherited
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_motif_MComponentPeer_isForegroundInherited
  (JNIEnv *env, jobject this) 
{
    jobject target;
    jobject color;

    if (JNU_IsNull(env, this)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return JNI_FALSE;
    }
    AWT_LOCK();

    target = (*env)->GetObjectField(env, this, mComponentPeerIDs.target);
    if (JNU_IsNull(env, target)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return JNI_FALSE;
    }
    color = (*env)->GetObjectField(env, target, componentIDs.foreground);
    if (JNU_IsNull(env, color)) {
	AWT_UNLOCK();
	return JNI_TRUE;
    } else {
	AWT_UNLOCK();
	return JNI_FALSE;
    }
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    pSetFont
 * Signature: (Ljava/awt/Font;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_pSetFont
  (JNIEnv *env, jobject this, jobject f) 
{
    struct ComponentData *cdata;

    struct changeFontInfo finfo = { NULL, FALSE, NULL, 0,
				    FALSE, FALSE, NULL, NULL };

    if (JNU_IsNull(env, f)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();

    cdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    finfo.env = env;
    finfo.fObj = f;
    awt_util_mapChildren(cdata->widget, changeFont, 1, (void *)&finfo);
    if (!finfo.error)
	XmFontListFree(finfo.fontList);

    AWT_FLUSH_UNLOCK();
} /* MComponentPeer.pSetFont() */

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    setCursor
 * Signature: (Ljava/awt/Cursor;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_setCursor
  (JNIEnv *env, jobject this, jobject cursor) 
{
    Cursor xcursor;
    struct ComponentData *cdata;

    AWT_LOCK();

    cdata = (struct ComponentData *)
      JNU_GetLongFieldAsPtr(env, this, mComponentPeerIDs.pData);
    if (cdata == 0 || cdata->widget == 0 || JNU_IsNull(env, cursor)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
	// xcursor = (Cursor)(*env)->GetLongField(env, cursor, cursorIDs.pData);
    awt_util_setCursor(cdata->widget, getCursor(env, cursor));

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    nativeHandleEvent
 * Signature: (Ljava/awt/AWTEvent;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_nativeHandleEvent
  (JNIEnv *env, jobject this, jobject event) 
{
    extern void awt_modify_KeyEvent(JNIEnv *env, XEvent * xevent, jobject jevent);
    XEvent *data;
    Widget widget=0;
    Boolean consumed;

    if (JNU_IsNull(env, event)) {
        return;
    }
    AWT_LOCK();

    data = (XEvent *)JNU_GetLongFieldAsPtr(env, event, awtEventIDs.data);

    if (data != 0) {
        consumed =  (*env)->GetBooleanField(env, event, awtEventIDs.consumed);

        /*    
         * Fix for bug 4280561
         *
         * If a menu is up, we must dispatch all XEvents, to allow
         * mouse grabs to be released and prevent server hangs.
         */
        if (consumed) {
            if ((awt_util_focusIsOnMenu(awt_display)) != 0) {
                consumed = False;
            }
        }



        if (!consumed) {
            switch ((*env)->GetIntField(env, event, awtEventIDs.id)) {
                case java_awt_event_KeyEvent_KEY_RELEASED:
                case java_awt_event_KeyEvent_KEY_PRESSED:
                {
                    awt_modify_KeyEvent(env, data, event);
                    break;
                }

                default:
                    break;
            }

            widget = XtWindowToWidget(awt_display, data->xany.window);

            if (!((widget == 0) || (!XtIsObject(widget)) ||
                  (widget->core.being_destroyed))) {
		/* Queue the event to be handled by the AWT-Motif thread */
		awt_put_back_event(env, data);
            }

        }
        AWT_UNLOCK();

        return;
    }
    AWT_UNLOCK();
    return;
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    requestFocus
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_requestFocus
  (JNIEnv *env, jobject this)
{
    struct ComponentData *bdata;
    Boolean result;

    AWT_LOCK();
    bdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env,this,mComponentPeerIDs.pData);
    if (bdata == 0 || bdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    trackInputFocus(bdata->widget);

    /* Motif won't let us set focus to a Panel/Canvas which has
     * traversable children, so temporarily turn off traversal on
     * it's children before requesting the focus.
     */
    if (IsCanvasTypeWidget(bdata->widget)) {
        setTreeTraversal(bdata->widget, FALSE);
    }

    /*
      Fix for bug 4157017: replace XmProcessTraversal with 
      XtSetKeyboardFocus because XmProcessTraversal does not allow
      focus to go to non-visible widgets.
      
      (There is a corresponding change to awt_MToolkit.c:dispatchToWidget)

      I found a last minute problem with this fix i.e. it broke the test
      case for bug 4053856. XmProcessTraversal does something else (that
      XtSetKeyboardFocus does not do) that stops this test case from
      failing. So, as I do not have time to investigate, and having
      both XmProcessTraversal and XtSetKeyboardFocus fixes 4157017 and
      4053856 and should be harmless (reviewer agreed), we have both
      below - XmProcessTraversal AND XtSetKeyboardFocus.
    */
    result = XmProcessTraversal(bdata->widget, XmTRAVERSE_CURRENT); 
    if (!result)
    {
        Widget shell;
        Widget w = bdata->widget;
        XFocusInEvent f;

        shell = w;
        while(shell && !XtIsShell(shell)) {
	    shell = XtParent(shell);
        }
        XtSetKeyboardFocus(shell, w);
    }
    /* end 4157017 */

    if (IsCanvasTypeWidget(bdata->widget)) {
        setTreeTraversal(bdata->widget, TRUE);
    }

    AWT_FLUSH_UNLOCK();
}

Dimension
awt_computeIndicatorSize(struct FontData *fdata)
{
    int height;
    int acc;
    int i;

    if (fdata == (struct FontData *) NULL)
	return MOTIF_XmINVALID_DIMENSION;

    /*
     * If Java font maps into single platform font - there's no
     * problem.  Let Motif use it's usual calculations in this case.
     */
    if (fdata->charset_num == 1)
	return MOTIF_XmINVALID_DIMENSION;

    acc = 0;
    for (i = 0; i < fdata->charset_num; ++i) {
	XFontStruct *xfont = fdata->flist[i].xfont;
	acc += xfont->ascent + xfont->descent;
    }

    height = acc / fdata->charset_num;
    if (height < MOTIF_XmDEFAULT_INDICATOR_DIM)
	height = MOTIF_XmDEFAULT_INDICATOR_DIM;

    return height;
}

Dimension
awt_adjustIndicatorSizeForMenu(Dimension indSize)
{
    if (indSize == 0 || indSize == MOTIF_XmINVALID_DIMENSION)
	return MOTIF_XmINVALID_DIMENSION; /* let motif do the job */

    /* Indicators in menus are smaller.
       2/3 is a magic number from Motif internals. */
    indSize = indSize * 2 / 3;
    if (indSize < MOTIF_XmDEFAULT_INDICATOR_DIM)
	indSize = MOTIF_XmDEFAULT_INDICATOR_DIM;

    return indSize;
}
