/*
 * @(#)awt_DataTransferer.c	1.3 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_p.h"
#include <X11/Intrinsic.h>
#include <X11/Xutil.h>

#include <jni.h>
#include <jni_util.h>

#include <sun_awt_motif_MDataTransferer.h>

/* used to construct string objects for file transfers */
static jclass string;   


/**
 * global function to initialize this client as a Dynamic-only app.
 *
 * gets called once during toolkit initialization.
 */

void awt_initialize_DataTransferer() {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
	/*
   	 * load class java.io.File
	 */

	string = (*env)->FindClass(env, "java/lang/String");

	if (JNU_IsNull(env, string  )) {
	    (*env)->ThrowNew(env,
			     (*env)->FindClass(env, "java/lang/ClassNotFoundException"),
			     "Cant find java/lang/String"
	    );
	}

	string = (*env)->NewGlobalRef(env, string); /* never freed! */
}

/*
 * Class:     sun_awt_motif_MDataTransferer
 * Method:    getAtomForTarget
 * Signature: (Ljava/lang/String;)I
 */

JNIEXPORT jint JNICALL Java_sun_awt_motif_MDataTransferer_getAtomForTarget
  (JNIEnv *env, jclass this, jstring targetString) 
{
    Atom target;
    char *target_str;

    AWT_LOCK();

    if (JNU_IsNull(env, targetString)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return -1;
    }
    target_str = (char *) JNU_GetStringPlatformChars(env, targetString, NULL);
    target = XInternAtom(awt_display, target_str, False);
    JNU_ReleaseStringPlatformChars(env, targetString, (const char *) target_str);

    AWT_UNLOCK();
    return (jint) target;
}

/*
 * Class:     sun_awt_motif_MDataTransferer
 * Method:    getTargetNameForAtom
 * Signature: (I)Ljava/lang/String;
 */

JNIEXPORT jstring JNICALL Java_sun_awt_motif_MDataTransferer_getTargetNameForAtom
  (JNIEnv *env, jclass this, jint atom) 
{
    jstring targetString;
    char *name;

    AWT_LOCK();

    name = XGetAtomName(awt_display, (Atom) atom);
    
    if (name == NULL) {
	JNU_ThrowNullPointerException(env, "NullPointerException");
	AWT_UNLOCK();
	return NULL;
    }

    targetString = (*env)->NewStringUTF(env, (const char *)name);
    if (JNU_IsNull(env, targetString)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return NULL;
    }

    targetString = (*env)->NewGlobalRef(env, targetString);
    XFree (name);

    AWT_UNLOCK();
    return targetString;
}

/*
 * Class:     sun_awt_motif_MDataTransferer
 * Method:    dragQueryFile
 * Signature: ([Ljava/lang/String;)[B
 *
 * This method converts a byte array that came from File most likely from a 
 * drag operation into a String array.
 */

JNIEXPORT jobjectArray JNICALL Java_sun_awt_motif_MDataTransferer_dragQueryFile
  (JNIEnv *env, jclass this, jbyteArray bytes) 
 {
    XTextProperty tp;
    jbyte *value;
    
    char**        strings  = (char **)NULL;
    int           nstrings = 0;
    Status        s;
    jobject       filenames;
    int           i;
    int		  len;
    jboolean	  isCopy=JNI_FALSE;

    value = (*env)->GetByteArrayElements(env, bytes, &isCopy);

    /*
     * If the length of the byte array is 0 just return a null
     */
    len = (*env)->GetArrayLength(env, (jobjectArray)bytes);
    if (len == 0) {
	return NULL;
    }

    tp.encoding = XInternAtom(awt_display, "STRING", False);
    tp.value    = (unsigned char *)value;
    tp.nitems   = len;
    tp.format   = 8;

    /*
     * Convert the byte stream into a list of X11 strings
     */
    if (XTextPropertyToStringList(&tp, &strings, &nstrings) == 0 || nstrings == 0) {
	return NULL;
    }

    (*env)->ReleaseByteArrayElements(env, bytes, value, JNI_ABORT);

    /*
     * Convert the list of X11 strings into an Array of Strings
     */
    if (nstrings == 0) {
      return NULL;
    }

    filenames = (jobjectArray)(*env)->NewObjectArray(env, (jsize)nstrings, 
						     string, (jobject)NULL);

    if (JNU_IsNull(env, filenames) || !JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	XFreeStringList(strings);
	
	(*env)->ExceptionDescribe(env);
	return NULL;
    }
    
    /*
     * Actuall conversion code per X11 String
     */
    for (i = 0; i < nstrings; i++) {
	jobject fileobj;
	jstring str = (*env)->NewStringUTF(env, (const char *)strings[i]);
	if (!JNU_IsNull(env, str))
	    str = (jstring)(*env)->NewGlobalRef(env, str);
	
	
	if (JNU_IsNull(env, str) || !JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    XFreeStringList(strings);
	    
	    (*env)->ExceptionDescribe(env);
	    return NULL;
	}

	(*env)->SetObjectArrayElement(env, filenames, (jsize)i, str    );
	
	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    XFreeStringList(strings);
	    
	    (*env)->ExceptionDescribe(env);
	    return NULL;
	}
    }
    
    /*
     * Clean up and return
     */
    XFreeStringList(strings);
    return filenames;
}

