/*
 * @(#)awt_DrawingSurface.c	1.27 00/08/17
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "awt_p.h"
#include "java_awt_Component.h"
#include "sun_awt_motif_MComponentPeer.h"
#include "sun_awt_motif_MDrawingSurfaceInfo.h"

#include "awt_Component.h"

#include <jni.h>
#include <jni_util.h>
#include <jawt_md.h>

extern struct MComponentPeerIDs mComponentPeerIDs;
extern struct ComponentIDs componentIDs;

/* fieldIDs for MDrawingSurfaceInfo fields that may be accessed from C */
static struct MDrawingSurfaceInfoIDs {
    jfieldID state;
    jfieldID peer;
    jfieldID w;
    jfieldID h;
} mDrawingSurfaceInfoIDs;

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    initIDs
 * Signature: ()V
 */

/* This function gets called from the static initializer for
   MDrawingSurfaceInfo.java to initialize the fieldIDs for fields
   that may be accessed from C */

JNIEXPORT void JNICALL
Java_sun_awt_motif_MDrawingSurfaceInfo_initIDs
  (JNIEnv *env, jclass cls)
{
    mDrawingSurfaceInfoIDs.state =
      (*env)->GetFieldID(env, cls, "state", "I");
    mDrawingSurfaceInfoIDs.peer =
      (*env)->GetFieldID(env, cls, "peer", "Lsun/awt/motif/MComponentPeer;");
    mDrawingSurfaceInfoIDs.w =
      (*env)->GetFieldID(env, cls, "w", "I");
    mDrawingSurfaceInfoIDs.h =
      (*env)->GetFieldID(env, cls, "h", "I");
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    lock
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_lock
  (JNIEnv *env, jobject this) 
{
    jobject hPeer, hComp;
    jint state;
    jint thisW, thisH;
    jint peerW = 0, peerH;

    state = (*env)->GetIntField(env, this, mDrawingSurfaceInfoIDs.state);

    AWT_LOCK();

    hPeer = (*env)->GetObjectField(env, this, mDrawingSurfaceInfoIDs.peer);
    hComp = (*env)->GetObjectField(env, hPeer, mComponentPeerIDs.target);
    
    if (!JNU_IsNull(env, hPeer)) {
        thisW = (*env)->GetIntField(env, this, mDrawingSurfaceInfoIDs.w);
        thisH = (*env)->GetIntField(env, this, mDrawingSurfaceInfoIDs.h);
        peerW = (*env)->GetIntField(env, hComp, componentIDs.width);
        peerH = (*env)->GetIntField(env, hComp, componentIDs.height);

        if (thisW != peerW || thisH != peerH) {
            (*env)->SetIntField(env, this, mDrawingSurfaceInfoIDs.w, peerW);
            (*env)->SetIntField(env, this, mDrawingSurfaceInfoIDs.h, peerH);
            (*env)->SetIntField(env, this, mDrawingSurfaceInfoIDs.state, ++state);
        }
    }
    return state;
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    unlock
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_unlock
  (JNIEnv *env, jobject this) 
{
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    getDisplay
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_getDisplay
  (JNIEnv *env, jobject this) 
{
    return (jint) awt_display;
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    getDrawable
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_getDrawable
  (JNIEnv *env, jobject this) 
{
    Drawable drawable = 0;
    jobject peer;

    AWT_LOCK();

    peer = (*env)->GetObjectField(env, this, mDrawingSurfaceInfoIDs.peer);

    if (!JNU_IsNull(env, peer)) {
        struct CanvasData *cdata;

        cdata = (struct CanvasData *)
	  JNU_GetLongFieldAsPtr(env,peer,mComponentPeerIDs.pData);
        if (cdata != 0) {
            Widget win = cdata->comp.widget;

            drawable = XtWindow(win);
        }
    }

    AWT_UNLOCK();

    if (drawable == 0) {
        JNU_ThrowNullPointerException(env, "unable to determine Drawable for DrawingSurface");
    }
    return (jint) drawable;
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    getDepth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_getDepth
  (JNIEnv *env, jobject this) 
{
    XWindowAttributes attrs;
    Drawable d = Java_sun_awt_motif_MDrawingSurfaceInfo_getDrawable(env, this);

    /* TODO no visual with a Pixmap !! */
    XGetWindowAttributes(awt_display, (Window)d, &attrs);
    return (jint) attrs.depth;
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    getVisualID
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_getVisualID
  (JNIEnv *env, jobject this)
{
    XWindowAttributes attrs;
    Drawable d = Java_sun_awt_motif_MDrawingSurfaceInfo_getDrawable(env, this);

    /* TODO no visual with a Pixmap !! */
    XGetWindowAttributes(awt_display, (Window)d, &attrs);
    return (jint) XVisualIDFromVisual(attrs.visual);
}

/*
 * Class:     sun_awt_motif_MDrawingSurfaceInfo
 * Method:    getColormapID
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_MDrawingSurfaceInfo_getColormapID
  (JNIEnv *env, jobject this)
{
    XWindowAttributes attrs;
    Drawable d = Java_sun_awt_motif_MDrawingSurfaceInfo_getDrawable(env, this);

    /* TODO no visual with a Pixmap !! */
    XGetWindowAttributes(awt_display, (Window)d, &attrs);
    return (jint) attrs.colormap;
}

/*****************************************************************************/
/** New native drawing API begins here                                      **/
/*****************************************************************************/

/*
 * Lock the surface of the target component for native rendering.
 * When finished drawing, the surface must be unlocked with
 * Unlock().  This function returns a bitmask with one or more of the
 * following values:
 *
 * JAWT_LOCK_ERROR - When an error has occurred and the surface could not
 * be locked.
 *
 * JAWT_LOCK_CLIP_CHANGED - When the clip region has changed.
 *
 * JAWT_LOCK_BOUNDS_CHANGED - When the bounds of the surface have changed.
 *
 * JAWT_LOCK_SURFACE_CHANGED - When the surface itself has changed
 */
JNIEXPORT jint JNICALL awt_DrawingSurface_Lock(JAWT_DrawingSurface* ds)
{
    JNIEnv* env;
    jobject target, peer;
    jclass componentClass;
    jint drawState;

    if (ds == NULL) {
#ifdef DEBUG
        fprintf(stderr, "Drawing Surface is NULL\n");
#endif
        return (jint)JAWT_LOCK_ERROR;
    }
    env = ds->env;
    target = ds->target;

    /* Make sure the target is a java.awt.Component */
    componentClass = (*env)->FindClass(env, "java/awt/Component");
    if (!(*env)->IsInstanceOf(env, target, componentClass)) {
#ifdef DEBUG
	    fprintf(stderr, "Target is not a component\n");
#endif
        return (jint)JAWT_LOCK_ERROR;
	}

	if (awt_lock == NULL) {
		return (jint)JAWT_LOCK_ERROR;
	}
    AWT_LOCK();

    /* Get the peer of the target component */
    peer = (*env)->GetObjectField(env, target, componentIDs.peer);
    if (JNU_IsNull(env, peer)) {
#ifdef DEBUG
        fprintf(stderr, "Component peer is NULL\n");
#endif
		AWT_FLUSH_UNLOCK();
        return (jint)JAWT_LOCK_ERROR;
    }

    drawState = (*env)->GetIntField(env, peer, mComponentPeerIDs.drawState);
    (*env)->SetIntField(env, peer, mComponentPeerIDs.drawState, 0);

    return drawState;
}

/*
 * Get the drawing surface info.
 * The value returned may be cached, but the values may change if
 * additional calls to Lock() or Unlock() are made.
 * Lock() must be called before this can return a valid value.
 * Returns NULL if an error has occurred.
 * When finished with the returned value, FreeDrawingSurfaceInfo must be
 * called.
 */
JNIEXPORT JAWT_DrawingSurfaceInfo* JNICALL
awt_DrawingSurface_GetDrawingSurfaceInfo(JAWT_DrawingSurface* ds)
{
    JNIEnv* env;
    jobject target, peer;
    jclass componentClass;
    JAWT_X11DrawingSurfaceInfo* px;
    JAWT_DrawingSurfaceInfo* p;
    struct CanvasData *cdata;
    XWindowAttributes attrs;

    if (ds == NULL) {
#ifdef DEBUG
        fprintf(stderr, "Drawing Surface is NULL\n");
#endif
        return NULL;
    }

    env = ds->env;
    target = ds->target;

    /* Make sure the target is a java.awt.Component */
    componentClass = (*env)->FindClass(env, "java/awt/Component");
    if (!(*env)->IsInstanceOf(env, target, componentClass)) {
#ifdef DEBUG
        fprintf(stderr, "DrawingSurface target must be a component\n");
#endif
        return NULL;
	}

	if (awt_lock == NULL) {
	    return NULL;
	}

    AWT_LOCK();

    /* Get the peer of the target component */
    peer = (*env)->GetObjectField(env, target, componentIDs.peer);
    if (JNU_IsNull(env, peer)) {
#ifdef DEBUG
        fprintf(stderr, "Component peer is NULL\n");
#endif
		AWT_UNLOCK();
        return NULL;
    }

    /* Get the canvas data from the peer */
    cdata = (struct CanvasData *)
        JNU_GetLongFieldAsPtr(env, peer, mComponentPeerIDs.pData);
    if (cdata == NULL) {
#ifdef DEBUG
        fprintf(stderr, "Canvas data is NULL\n");
#endif
		AWT_UNLOCK();
        return NULL;
    }

	AWT_UNLOCK();

    /* Allocate platform-specific data */
    px = (JAWT_X11DrawingSurfaceInfo*)
        malloc(sizeof(JAWT_X11DrawingSurfaceInfo));

    /* Set drawable and display */
    px->drawable = XtWindow(cdata->comp.widget);
    px->display = awt_display;

    /* Get window attributes to set other values */
    XGetWindowAttributes(awt_display, (Window)(px->drawable), &attrs);

    /* Set the other values */
    px->visualID = XVisualIDFromVisual(attrs.visual);
    px->colormapID = attrs.colormap;
    px->depth = attrs.depth;

    /* Allocate and initialize platform-independent data */
    p = (JAWT_DrawingSurfaceInfo*)malloc(sizeof(JAWT_DrawingSurfaceInfo));
    p->platformInfo = px;
    p->ds = ds;
    p->bounds.x = (*env)->GetIntField(env, target, componentIDs.x);
    p->bounds.y = (*env)->GetIntField(env, target, componentIDs.y);
    p->bounds.width = (*env)->GetIntField(env, target, componentIDs.width);
    p->bounds.height = (*env)->GetIntField(env, target, componentIDs.height);
    p->clipSize = 1;
    p->clip = &(p->bounds);

    /* Return our new structure */
    return p;
}

/*
 * Free the drawing surface info.
 */
JNIEXPORT void JNICALL
awt_DrawingSurface_FreeDrawingSurfaceInfo(JAWT_DrawingSurfaceInfo* dsi)
{
    if (dsi == NULL ) {
#ifdef DEBUG
        fprintf(stderr, "Drawing Surface Info is NULL\n");
#endif
    }
    free(dsi->platformInfo);
    free(dsi);
}

/* 
 * Unlock the drawing surface of the target component for native rendering.
 */
JNIEXPORT void JNICALL awt_DrawingSurface_Unlock(JAWT_DrawingSurface* ds)
{
    JNIEnv* env;
    if (ds == NULL) {
#ifdef DEBUG
        fprintf(stderr, "Drawing Surface is NULL\n");
#endif
        return;
    }
    env = ds->env;
    AWT_FLUSH_UNLOCK();
}

JNIEXPORT JAWT_DrawingSurface* JNICALL
    awt_GetDrawingSurface(JNIEnv* env, jobject target)
{
    jclass canvasClass;
    JAWT_DrawingSurface* p;

    /* Make sure the target component is a java.awt.Canvas */
    canvasClass = (*env)->FindClass(env, "java/awt/Canvas");
    if (!(*env)->IsInstanceOf(env, target, canvasClass)) {
#ifdef DEBUG
        fprintf(stderr, 
            "GetDrawingSurface target must be a java.awt.Canvas\n");
#endif
        return NULL;
    }
    
    p = (JAWT_DrawingSurface*)malloc(sizeof(JAWT_DrawingSurface));
    p->env = env;
    p->target = (*env)->NewGlobalRef(env, target);
    p->Lock = awt_DrawingSurface_Lock;
    p->GetDrawingSurfaceInfo = awt_DrawingSurface_GetDrawingSurfaceInfo;
    p->FreeDrawingSurfaceInfo = awt_DrawingSurface_FreeDrawingSurfaceInfo;
    p->Unlock = awt_DrawingSurface_Unlock;
    return p;
}

JNIEXPORT void JNICALL
    awt_FreeDrawingSurface(JAWT_DrawingSurface* ds)
{
    JNIEnv* env;

    if (ds == NULL ) {
#ifdef DEBUG
        fprintf(stderr, "Drawing Surface is NULL\n");
#endif
        return;
    }
    env = ds->env;
    (*env)->DeleteGlobalRef(env, ds->target);
    free(ds);
}

