/*
 * @(#)awt_Graphics.c	1.128 00/08/14
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#define MITSHM

#include "awt_p.h"
#include "color.h"
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include "sun_java2d_loops_GraphicsPrimitive.h"
#include "java_awt_Color.h"
#include "java_awt_Font.h"
#include "java_awt_Rectangle.h"

#include "sun_awt_motif_X11Graphics.h"
#include "java_awt_Canvas.h"
#include "sun_awt_motif_MCanvasPeer.h"
#include "sun_awt_motif_MComponentPeer.h"

#include "img_util_md.h"
#include "img_util.h"

#include "awt_Graphics.h"

#include "awt_Component.h"

#include "ImageData.h"
#include "SpanIterator.h"

#include "multi_font.h"
#include <jni.h>
#include <jni_util.h>
#include <jlong.h>

#include <jdga.h>

#include <dlfcn.h>

#ifdef MITSHM

#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>

extern int XShmQueryExtension();
#endif

#undef max
#define max(x, y) (((int)(x) > (int)(y)) ? (x) : (y))
#undef min
#define min(x, y) (((int)(x) < (int)(y)) ? (x) : (y))

#define INIT_GC0(disp, gdata, this)					\
    do {								\
	if (gdata==0 ||							\
	    (gdata->gc == 0 &&						\
	     !awt_init_gc(env,disp,gdata,this)))			\
	{								\
	    return;							\
	}								\
 	pJDgaInfo->pXRequestSent(env, gdata->dgaDev, gdata->drawable);	\
    } while (0)

#define INIT_GC(disp, gdata, this)					\
    do {								\
	if (gdata==0 ||							\
	    (gdata->gc == 0 &&						\
	     !awt_init_gc(env,disp,gdata,this))) {			\
	    AWT_UNLOCK();						\
	    return;							\
	}								\
	pJDgaInfo->pXRequestSent(env, gdata->dgaDev, gdata->drawable);	\
    } while (0)

#define INIT_GC2(disp, gdata, rval, this)				\
    do {								\
	if (gdata==0 ||							\
	    (gdata->gc == 0 &&						\
	     !awt_init_gc(env,disp,gdata,this))) {			\
	    AWT_UNLOCK();						\
	    return rval;						\
	}								\
	pJDgaInfo->pXRequestSent(env, gdata->dgaDev, gdata->drawable);	\
    } while (0)

#define INIT_GC_NO_DRAW(disp, gdata, this)				\
    do {								\
	if (gdata==0 ||							\
	    (gdata->gc == 0 &&						\
	     !awt_init_gc(env,disp,gdata,this))) {			\
	    AWT_UNLOCK();						\
	    return;							\
	}								\
    } while (0)

#define INIT_GC2_NO_DRAW(disp, gdata, rval, this)			\
    do {								\
	if (gdata==0 ||							\
	    (gdata->gc == 0 &&						\
	     !awt_init_gc(env,disp,gdata,this))) {			\
	    AWT_UNLOCK();						\
	    return rval;						\
	}								\
    } while (0)

#define POLYTEMPSIZE            (256 / sizeof(XPoint))

#define ABS(n)                  (((n) < 0) ? -(n) : (n))


extern struct MComponentPeerIDs mComponentPeerIDs;
static jfieldID gPDataID;
static jfieldID peerID;
static jfieldID fontID;

static JDgaLibInfo DgaLibInfoStub;
static jboolean using_mitshm = False;

JDgaLibInfo sharedDgaInfo;
JDgaLibInfo *pJDgaInfo = &sharedDgaInfo;

static int nativeByteOrder;

int 
awt_init_gc(JNIEnv * env, Display * display, struct GraphicsData *gdata, jobject this)
{
    if (gdata->drawable == 0) {
        jobject peer;
        struct ComponentData *cdata;
        Widget w;

        peer = (*env)->GetObjectField(env,this,peerID);

        if (JNU_IsNull(env, peer)) {
            /* this should never happen */
            return 0;
        }
        cdata = (struct ComponentData *)
	  JNU_GetLongFieldAsPtr(env, peer, mComponentPeerIDs.pData);
        if (cdata == 0) {
            /* native widget for component peer has been destroyed */
            return 0;
        }
        w = cdata->widget;

        if (w != NULL && XtIsRealized(w)) {
            gdata->drawable = XtWindow(w);
        }
        if (gdata->drawable == 0) {
            return 0;
        }
    }

    if (gdata->gc == 0) {
        if ((gdata->gc = XCreateGC(display, gdata->drawable, 0, 0)) != 0) {
            XSetGraphicsExposures(display, gdata->gc, True);
            return 1;
        } else {
            /* Could not allocate a GC for this Graphics. */
            return 0;
        }
    } else {
        /* Already have a GC for this Graphics. */
        return 1;
    }
}

static void
awt_drawArc(JNIEnv * env, jobject this,
            struct GraphicsData *gdata,
            int x, int y, int w, int h,
            int startAngle, int endAngle,
            int filled)
{
    int s, e;

    if (w < 0 || h < 0) {
        return;
    }
    if (gdata == 0) {
        gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
        INIT_GC0(awt_display, gdata, this);
    }
    if (endAngle >= 360 || endAngle <= -360) {
        s = 0;
        e = 360 * 64;
    } else {
        s = (startAngle % 360) * 64;
        e = endAngle * 64;
    }
    x += gdata->originX;
    y += gdata->originY;
    if (filled == 0) {
        XDrawArc(awt_display,
                 gdata->drawable,
                 gdata->gc,
                 x,
                 y,
                 w,
                 h,
                 s,
                 e);
    } else {
        XFillArc(awt_display,
                 gdata->drawable,
                 gdata->gc,
                 x,
                 y,
                 w,
                 h,
                 s,
                 e);
    }
}

static XPoint *
transformPoints(JNIEnv * env, struct GraphicsData *gdata,
                jintArray xcoordsArray, jintArray ycoordsArray,
                XPoint * points, int *pNpoints, int close)
{
    int i;
    int npoints = *pNpoints;
    jint *xcoords, *ycoords;
    long baseX,baseY;

    baseX = gdata->originX;
    baseY = gdata->originY;

    xcoords = (jint *)(*env)->GetPrimitiveArrayCritical(env, xcoordsArray, NULL);
    if (xcoords == NULL) {
        return 0;
    }
    
    ycoords = (jint *)(*env)->GetPrimitiveArrayCritical(env, ycoordsArray, NULL);

    if (ycoords == NULL) {
        (*env)->ReleasePrimitiveArrayCritical(env, xcoordsArray, xcoords,
                                              JNI_ABORT);
        return 0;
    }

    if (close) {
        close = (npoints > 2
                 && (xcoords[npoints - 1] != xcoords[0]
                     || ycoords[npoints - 1] != ycoords[0]));
        if (close) {
            npoints++;
            *pNpoints = npoints;
        }
    }
    if (npoints > POLYTEMPSIZE) {
        points = (XPoint *) malloc(sizeof(XPoint) * npoints);
        if (points == NULL) {
            (*env)->ReleasePrimitiveArrayCritical(env, xcoordsArray, xcoords,
                                                  JNI_ABORT);
            (*env)->ReleasePrimitiveArrayCritical(env, ycoordsArray, ycoords,
                                                  JNI_ABORT);

            return 0;
        }
    }
    i = npoints;
    if (close) {
        --i;
        points[i].x = baseX + xcoords[0];
        points[i].y = baseY + ycoords[0];
    }
    while (--i >= 0) {
        points[i].x = baseX + xcoords[i];
        points[i].y = baseY + ycoords[i];
    }

    (*env)->ReleasePrimitiveArrayCritical(env, xcoordsArray, xcoords,
                                          JNI_ABORT);
    (*env)->ReleasePrimitiveArrayCritical(env, ycoordsArray, ycoords,
                                          JNI_ABORT);

    return points;
}

#ifdef NETSCAPE
#include "nspr/prcpucfg.h"
#endif

static void
awt_XDrawString16(Display * display, Drawable draw,
                  GC gc, int x, int y,
                  XChar2b * string, int length)
{
#if i386 || (defined(NETSCAPE) && defined(IS_LITTLE_ENDIAN))
    XChar2b *str = malloc(sizeof(XChar2b) * length);

    if (str) {
        XChar2b *s, *d, *limit;

        for (s = string, d = str, limit = str + length; d < limit; d++, s++) {
            d->byte1 = s->byte2;
            d->byte2 = s->byte1;
        }
        XDrawString16(display, draw, gc, x, y, str, length);
        free(str);
    }
#else
    XDrawString16(display, draw, gc, x, y, string, length);
#endif
    return;
}


#ifdef MITSHM
int (* prev_handler) (Display *, XErrorEvent *);

int XErrHandler(Display *display, XErrorEvent *xerr) {
    int ret = 0;
    if (xerr->minor_code == X_ShmAttach) {
	using_mitshm = False;
    } else {
	ret = (*prev_handler)(display, xerr);
    }
    return ret;
}

static void InitMITShm(JNIEnv * env) {
    XShmSegmentInfo shminfo;

    AWT_LOCK();
    using_mitshm = False;
    if (!getenv("NO_AWT_MITSHM")) {
	if (XShmQueryExtension(awt_display) ) {
	    shminfo.shmid = shmget(IPC_PRIVATE, 0x10000, IPC_CREAT|0777);
	    if (shminfo.shmid < 0) {
		AWT_UNLOCK();
		return;
	    }
	    shminfo.shmaddr = (char *) shmat(shminfo.shmid, 0, 0);
	    if (shminfo.shmaddr == ((char *) -1)) {
		AWT_UNLOCK();
		return;
	    }
	    shminfo.readOnly = True;
	    using_mitshm = True;

	    XSync(awt_display, False);
	    prev_handler = XSetErrorHandler(XErrHandler);

	    XShmAttach(awt_display, &shminfo);

	    XSync(awt_display, False);
	    (void)XSetErrorHandler(prev_handler);
	    if (using_mitshm) {
	        XShmDetach(awt_display, &shminfo);
	    }
	    shmdt(shminfo.shmaddr);
	    shmctl(shminfo.shmid, IPC_RMID, 0);
	}
    }
    AWT_UNLOCK();
}
#endif /* MITSHM */

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    init
 * Signature: ()Z
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_cacheInit
  (JNIEnv *env, jclass cls)
{
    jclass peerClass;
    void *lib = 0;

    union {
        char c[4];
        int i;
    } endian;
    
    endian.i = 0xff000000;
    nativeByteOrder = (endian.c[0]) ? MSBFirst : LSBFirst;
    
    gPDataID = (*env)->GetFieldID(env, cls, "pData", "J");
    peerID = (*env)->GetFieldID(env,cls,"peer","Lsun/awt/motif/MComponentPeer;");
    fontID = (*env)->GetFieldID(env,cls,"font","Ljava/awt/Font;");
    
    if (getenv("NO_J2D_DGA") == 0) {
    /* we use RTLD_NOW because of bug 4032715 */
#ifdef DEBUG
        lib = dlopen("libsunwjdga_g.so", RTLD_NOW);
#else
        lib = dlopen("libsunwjdga.so", RTLD_NOW);
#endif
    }
 
#ifdef MITSHM
    InitMITShm(env);
#endif /* MITSHM */

    if (lib != 0) {
        int ret = JDGA_UNAVAILABLE;
	void *sym = dlsym(lib, "JDgaLibInit");
	pJDgaInfo->xineramaOn = False;
	if (sym != 0) {
            AWT_LOCK();
            pJDgaInfo->display = awt_display;
	    ret = (*(JDgaLibInitFunc *)sym)(env, pJDgaInfo);
            AWT_UNLOCK();
	}
	if (ret != JDGA_SUCCESS) {
	    dlclose(lib);
	    lib = 0;
	} else if (pJDgaInfo->xineramaOn) {
	    using_mitshm = False;
	}
    }
    if (lib == 0) {
	pJDgaInfo = &DgaLibInfoStub;
    }
    return;
}

jfieldID
getgPDataIDFromX11Graphics() {
    return gPDataID;
}

extern AwtGraphicsConfigDataPtr
getGraphicsConfigFromComponentPeer(JNIEnv *env, jobject this);

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    createFromComponent
 * Signature: (Lsun/awt/motif/MComponentPeer;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_createFromComponent
  (JNIEnv *env, jobject this, jobject canvas) 
{
    struct GraphicsData *gdata;
    AwtGraphicsConfigDataPtr adata =
        getGraphicsConfigFromComponentPeer(env, canvas);

    AWT_LOCK();
    gdata = ZALLOC(GraphicsData);
    JNU_SetLongFieldFromPtr(env,this,gPDataID,gdata);

    if (gdata == 0) {
        JNU_ThrowOutOfMemoryError(env, "OutOfMemoryError");
        AWT_UNLOCK();
        return;
    }
    gdata->gc = 0;
    gdata->drawable = 0;
    gdata->clipset = False;
    gdata->originX = 0;
    gdata->originY = 0;
    gdata->awtData = adata;
    gdata->pScreen = 0;
    if (gdata->awtData == (AwtGraphicsConfigDataPtr) NULL)
	gdata->awtData = getDefaultConfig(DefaultScreen(awt_display));

    AWT_UNLOCK();

}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    createFromGraphics
 * Signature: (Lsun/awt/motif/X11Graphics;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_createFromGraphics
  (JNIEnv *env, jobject this, jobject g) 
{
    struct GraphicsData *odata;
    struct GraphicsData *gdata;

    AWT_LOCK();

    odata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,g,gPDataID);

    if (JNU_IsNull(env, g) || odata == NULL) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    gdata = ZALLOC(GraphicsData);
    JNU_SetLongFieldFromPtr(env,this,gPDataID,gdata);

    if (gdata == 0) {
        JNU_ThrowOutOfMemoryError(env, "OutOfMemoryError");
        AWT_UNLOCK();
        return;
    }
    /* Fix for Bug Id 4261880
     * Fill internal structure before
     * checking for GC
     */
    gdata->originX = odata->originX;
    gdata->originY = odata->originY;
    gdata->awtData = odata->awtData;
    gdata->dgaDev  = odata->dgaDev;
    gdata->clipset = odata->clipset;
    gdata->cliprect = odata->cliprect;
    gdata->drawable = 0;

    INIT_GC_NO_DRAW(awt_display, odata, this);
    gdata->drawable = odata->drawable;
    INIT_GC_NO_DRAW(awt_display, gdata, this);

    /* copy the gc */
    XCopyGC(awt_display, odata->gc,
            GCForeground | GCBackground | GCFont | GCFunction,
            gdata->gc);

    /* Set the clip rect */
    if (gdata->clipset) {
        XSetClipRectangles(awt_display, gdata->gc, 0, 0, &gdata->cliprect, 1, YXBanded);
    }
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    pSetFont
 * Signature: (Ljava/awt/Font;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_pSetFont
  (JNIEnv *env, jobject this, jobject font) 
{
    struct FontData *fdata;
    struct GraphicsData *gdata;
    char *err;

    if (JNU_IsNull(env, font)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();

    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);

    fdata = awtJNI_GetFontData(env, font, &err);
    if (fdata == 0) {
        JNU_ThrowInternalError(env, err);
        AWT_UNLOCK();
        return;
    }
    INIT_GC_NO_DRAW(awt_display, gdata, this);
    if (!awtJNI_IsMultiFont(env, font))
        XSetFont(awt_display, gdata->gc, fdata->xfont->fid);

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    pSetForeground
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_pSetForeground
  (JNIEnv *env, jobject this, jobject c) 
{
    Pixel p1;
    struct GraphicsData *gdata;
    XGCValues xgcval;

    if (JNU_IsNull(env, c)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();

    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC_NO_DRAW(awt_display, gdata, this);
    p1 = awtJNI_GetColorForVis (env, c, gdata->awtData);
    gdata->fgpixel = p1;
    if (gdata->xormode) {
        p1 ^= gdata->xorpixel;
    }
    XSetForeground(awt_display, gdata->gc, p1);
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    setOrigin
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_setOrigin
  (JNIEnv *env, jobject this, jint tx, jint ty) 
{
    struct GraphicsData *gdata;

    AWT_LOCK();

    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC_NO_DRAW(awt_display, gdata, this);
    gdata->originX = tx;
    gdata->originY = ty;

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    dispose
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_disposeImpl
  (JNIEnv *env, jobject this) 
{
    struct GraphicsData *gdata;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    if (gdata == 0) {
        AWT_UNLOCK();
        return;
    }
    if (gdata->gc) {
        XFreeGC(awt_display, gdata->gc);
    }
    if (gdata->monoImage) {
      XDestroyImage(gdata->monoImage);
    }
    if (gdata->pScreen) {
        free(gdata->pScreen);
    }
    free(gdata);
    (*env)->SetLongField(env,this,gPDataID,0);
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    setPaintMode
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_pSetPaintMode
  (JNIEnv *env, jobject this) 
{
    Pixel p1;
    struct GraphicsData *gdata;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC_NO_DRAW(awt_display, gdata, this);
    gdata->xormode = False;
    XSetFunction(awt_display, gdata->gc, GXcopy);
    XSetForeground(awt_display, gdata->gc, gdata->fgpixel);
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    setXORMode
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_pSetXORMode
  (JNIEnv *env, jobject this, jobject c) 
{
    Pixel p1;
    struct GraphicsData *gdata;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    if (JNU_IsNull(env, c)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    INIT_GC_NO_DRAW(awt_display, gdata, this);
    p1 = awtJNI_GetColorForVis (env, c, gdata->awtData);
    gdata->xorpixel = p1;
    p1 ^= gdata->fgpixel;
    gdata->xormode = True;
    XSetFunction(awt_display, gdata->gc, GXxor);
    XSetForeground(awt_display, gdata->gc, p1);
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    getClipBounds
 * Signature: ()Ljava/awt/Rectangle;
 */
JNIEXPORT jobject JNICALL Java_sun_awt_motif_X11Graphics_getClipBounds
  (JNIEnv *env, jobject this) 
{
    struct GraphicsData *gdata;
    jobject clip = NULL;
    int set, x = 0, y = 0, w = 0, h = 0;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);

    if (gdata == 0) {
        JNU_ThrowNullPointerException(env, "Graphics data is null!");
	AWT_UNLOCK();
        return 0;
    }
    /* Deadlock prevention:
     * copy data to stack and then unlock for the constructor.
     */

    set = gdata->clipset;
    if (set) {
        x = gdata->cliprect.x - gdata->originX;
        y = gdata->cliprect.y - gdata->originY;
        w = gdata->cliprect.width;
        h = gdata->cliprect.height;
    }
    AWT_UNLOCK();
    if (set) {
        jclass clazz;
        jmethodID mid;

        clazz = (*env)->FindClass(env, "java/awt/Rectangle");
        mid = (*env)->GetMethodID(env, clazz, "<init>", "(IIII)V");
        if (mid != 0) {
            clip = (*env)->NewObject(env, clazz, mid,
                                     x, y, w, h);

            if ((*env)->ExceptionOccurred(env)) {
                return 0;
            }
        }
    }
    return clip;
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    changeClip
 * Signature: (IIIIZ)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_changeClip
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h, jboolean set) 
{
    struct GraphicsData *gdata;
    int x1, y1, x2, y2;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC_NO_DRAW(awt_display, gdata, this);

    /* compute the union */
    x1 = x;
    y1 = y;
    if (w <= 0 || h <= 0) {
        x2 = x1;
        y2 = y1;
    } else {
        x2 = x1 + w;
        y2 = y1 + h;
    }

    if (!set && gdata->clipset) {
        x1 = max(gdata->cliprect.x, x1);
        y1 = max(gdata->cliprect.y, y1);
        x2 = min(gdata->cliprect.x + gdata->cliprect.width, x2);
        y2 = min(gdata->cliprect.y + gdata->cliprect.height, y2);
        if (x1 > x2) {
            x2 = x1;
        }
        if (y1 > y2) {
            y2 = y1;
        }
    }
    gdata->cliprect.x = (Position) x1;
    gdata->cliprect.y = (Position) y1;
    gdata->cliprect.width = (Dimension) (x2 - x1);
    gdata->cliprect.height = (Dimension) (y2 - y1);
    gdata->clipset = True;

    XSetClipRectangles(awt_display,
                       gdata->gc,
                       0, 0,
                       &gdata->cliprect, 1,
                       YXBanded);
    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    removeClip
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_removeClip
  (JNIEnv *env, jobject this) 
{
    struct GraphicsData *gdata;
    int x1, y1, x2, y2;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC_NO_DRAW(awt_display, gdata, this);

    gdata->clipset = False;
    XSetClipMask(awt_display, gdata->gc, None);

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    setLineAttrs
 * Signature: (III[II)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_setLineAttrs
    (JNIEnv *env, jobject this, jint width, jint caps, jint join,
     jbyteArray dash, jint numdashes, jint dashoff)
{
    struct GraphicsData *gdata;
    int style = LineSolid;
    static int capvalues[] = { CapButt, CapRound, CapProjecting };
    static int joinvalues[] = { JoinMiter, JoinRound, JoinBevel };

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC_NO_DRAW(awt_display, gdata, this);

    if (!JNU_IsNull(env, dash)) {
	unsigned char *dashes;
	int dashcount;
	dashcount = (*env)->GetArrayLength(env, dash);
	if (dashcount < numdashes) {
	    JNU_ThrowArrayIndexOutOfBoundsException(env, "ArrayIndexOutOfBoundsException");
	    AWT_UNLOCK();
	    return;
	}
	dashes = (unsigned char *)
	    (*env)->GetPrimitiveArrayCritical(env, dash, NULL);
	if (dashes != 0) {
	    XSetDashes(awt_display, gdata->gc, dashoff,
                       ( char *) dashes, numdashes);
	    (*env)->ReleasePrimitiveArrayCritical(env, dash, (jchar *) dashes,
						  JNI_ABORT);
	    style = LineOnOffDash;
	}
    }
    XSetLineAttributes(awt_display, gdata->gc,
		       width, style, capvalues[caps], joinvalues[join]);

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devClearRect
 * Signature: (IIII)V
 *
 * ASSUMES: We are not running on a privileged thread
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devClearRect
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h) 
{
    struct GraphicsData *gdata;
    jobject bgh;
    jboolean he;
    Pixel p1;

    DASSERT(!awt_currentThreadIsPrivileged(env));
    
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    if (gdata == 0) {
        AWT_UNLOCK();
        return;
    }
    x += gdata->originX;
    y += gdata->originY;
    INIT_GC(awt_display, gdata, this);
    if (gdata->clipset) {
        long cx, cy, cw, ch;

        cx = gdata->cliprect.x;
        cy = gdata->cliprect.y;
        cw = gdata->cliprect.width;
        ch = gdata->cliprect.height;
        if (x < cx) {
            w = x + w - cx;
            x = cx;
        }
        if (y < cy) {
            h = y + h - cy;
            y = cy;
        }
        if (x + w > cx + cw) {
            w = cx + cw - x;
        }
        if (y + h > cy + ch) {
            h = cy + ch - y;
        }
    }
    if (w <= 0 || h <= 0) {
        AWT_UNLOCK();
        return;
    }

    bgh = (JNU_CallMethodByName(env, &he, this, "getBackground",
                                "()Ljava/awt/Color;")).l;
    
    if (he) {
        AWT_UNLOCK();
        return;
    }
    
    XSetForeground(awt_display, gdata->gc, 
		   awtJNI_GetColorForVis (env, bgh, gdata->awtData));
    if (gdata->xormode) { /* temporarily turn off XOR mode */
        XSetFunction(awt_display, gdata->gc, GXcopy);
    }
    XFillRectangle(awt_display,
                   gdata->drawable, gdata->gc,
                   x, y, w, h);
    p1 = gdata->fgpixel;
    if (gdata->xormode) { /* restore XOR mode */
        XSetFunction(awt_display, gdata->gc, GXxor);
        p1 ^= gdata->xorpixel;
    }
    XSetForeground(awt_display, gdata->gc, p1);
    
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devFillRect
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devFillRect
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h) 
{
    struct GraphicsData *gdata;

    if (w <= 0 || h <= 0) {
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);
    XFillRectangle(awt_display,
                   gdata->drawable,
                   gdata->gc,
                   x + gdata->originX, y + gdata->originY, w, h);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawRect
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawRect
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h) 
{
    struct GraphicsData *gdata;

    if (w < 0 || h < 0) {
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);

    INIT_GC(awt_display, gdata, this);
    XDrawRectangle(awt_display,
                   gdata->drawable,
                   gdata->gc,
                   x + gdata->originX, y + gdata->originY, w, h);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    drawMFChars
 * Signature: ([CIIIILjava/awt/Font;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_drawMFChars
  (JNIEnv *env, jobject this, jcharArray text, jint offset
   , jint length, jint x, jint y, jobject font) 
{
    struct GraphicsData *gdata;
    struct FontData *fdata;
    char *err;

    if (text == NULL || font == NULL) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);

    x += gdata->originX;
    y += gdata->originY;
    awtJNI_DrawMFString(env, text, gdata, font, x, y, offset, length);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    drawSFChars
 * Signature: ([CIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_drawSFChars
  (JNIEnv *env, jobject this, jcharArray text, jint offset, jint length, jint x, jint y) 
{
    ushort *data;
    struct GraphicsData *gdata;
    jstring s;
    int arrayLength;
    jobject font;

    if (JNU_IsNull(env, text)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    arrayLength = (*env)->GetArrayLength(env, text);

    if (offset < 0 || length < 0 || offset + length > arrayLength) {
        JNU_ThrowArrayIndexOutOfBoundsException(env, "ArrayIndexOutOfBoundsException");
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);

    if (length > 1024) {
        length = 1024;
    }

    data = (ushort *) (*env)->GetPrimitiveArrayCritical(env, text, NULL);
    
    if (data == 0) {
        AWT_UNLOCK();
        return;
    }
    
    awt_XDrawString16(awt_display,
                      gdata->drawable,
                      gdata->gc,
                      x + gdata->originX,
                      y + gdata->originY,
                      (XChar2b *) & (data[offset]),
                      length);
    
    if (data)
        (*env)->ReleasePrimitiveArrayCritical(env, text, (jchar *) data, JNI_ABORT);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawBytes
 * Signature: ([BIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawBytes
  (JNIEnv *env, jobject this, jbyteArray text, jint offset, jint length, jint x, jint y) 
{
    ushort *data;
    struct GraphicsData *gdata;
    int arrayLength;

    if (JNU_IsNull(env, text)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    arrayLength = (*env)->GetArrayLength(env, text);

    if (offset < 0 || length < 0 || offset + length > arrayLength) {
        JNU_ThrowArrayIndexOutOfBoundsException(env, "ArrayIndexOutOfBoundsException");
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);

    if (length > 1024) {
        length = 1024;
    }
    data = (ushort *) (*env)->GetPrimitiveArrayCritical(env, text, NULL);

    if (data == 0) {
	AWT_UNLOCK();
        return;
    }

    XDrawString(awt_display,
                gdata->drawable,
                gdata->gc,
                x + gdata->originX,
                y + gdata->originY,
                ( char *) & (data[offset]),
                length);

    (*env)->ReleasePrimitiveArrayCritical(env, text, data, JNI_ABORT);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    drawStringWidth
 * Signature: (Ljava/lang/String;II)I
 */
JNIEXPORT jint JNICALL Java_sun_awt_motif_X11Graphics_drawStringWidth
  (JNIEnv *env, jobject this, jstring text, jint x, jint y) 
{
    ushort *data;
    int length;
    struct GraphicsData *gdata;
    struct FontData *fdata;
    char *err;
    int rval = 0;
    jobject font;

    if (JNU_IsNull(env, text)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return -1;
    }
    AWT_LOCK();

    font = (*env)->GetObjectField(env,this,fontID);
    fdata = awtJNI_GetFontData(env, font, &err);
    if (fdata == 0) {
        JNU_ThrowInternalError(env, err);
        AWT_UNLOCK();
        return -1;
    }
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC2(awt_display, gdata, -1, this);
    length = (*env)->GetStringLength(env, text);

    if (length > 1024) {
        length = 1024;
    }
    data = (ushort *) (*env)->GetStringChars(env, text, NULL);

    awt_XDrawString16(awt_display,
                      gdata->drawable,
                      gdata->gc,
                      x + gdata->originX,
                      y + gdata->originY,
                      (XChar2b *) data,
                      length);

    if (!awtJNI_IsMultiFont(env, font))
        rval = XTextWidth16(fdata->xfont, (XChar2b *) data, length);

    if (data)
        (*env)->ReleaseStringChars(env, text, (const jchar *) data);

    AWT_FLUSH_UNLOCK();

    return rval;
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawLine
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawLine
  (JNIEnv *env, jobject this, jint x1, jint y1, jint x2, jint y2) 
{
    struct GraphicsData *gdata;
    int x;
    int y;

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);
    x = gdata->originX;
    y = gdata->originY;
    XDrawLine(awt_display,
              gdata->drawable,
              gdata->gc,
              x+x1, y+y1,
              x+x2, y+y2);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devCopyArea
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devCopyArea(
   JNIEnv *env, jobject this, jint x, jint y,
   jint width, jint height, jint dx, jint dy) 
{
    struct GraphicsData *gdata;

    if (width <= 0 || height <= 0) {
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);
    x += gdata->originX;
    y += gdata->originY;
    dx += x;
    dy += y;

    XCopyArea(awt_display, gdata->drawable, gdata->drawable, gdata->gc,
              x, y, width, height, dx, dy);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawRoundRect
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawRoundRect
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h, jint arcWidth, jint arcHeight) 
{
    struct GraphicsData *gdata;
    long tx, txw, ty, ty1, ty2, tyh, tx1, tx2;
    long baseX,baseY;

    if (w <= 0 || h <= 0) {
        return;
    }
    arcWidth = ABS(arcWidth);
    arcHeight = ABS(arcHeight);
    if (arcWidth > w)
        arcWidth = w;
    if (arcHeight > h)
        arcHeight = h;
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);

    baseX = gdata->originX;
    baseY = gdata->originY;

    tx = baseX + x;
    txw = baseX + x + w;
    ty = baseY + y;
    tyh = baseY + y + h;

    ty1 = baseY + y + (arcHeight / 2);
    ty2 = baseY + y + h - (arcHeight / 2);
    tx1 = baseX + x + (arcWidth / 2);
    tx2 = baseX + x + w - (arcWidth / 2);

    awt_drawArc(env, this, gdata,
                x, y, arcWidth, arcHeight,
                90, 90, 0);
    awt_drawArc(env, this, gdata,
                x + w - arcWidth, y, arcWidth, arcHeight,
                0, 90, 0);
    awt_drawArc(env, this, gdata,
                x, y + h - arcHeight, arcWidth, arcHeight,
                180, 90, 0);
    awt_drawArc(env, this, gdata,
                x + w - arcWidth, y + h - arcHeight, arcWidth, arcHeight,
                270, 90, 0);

    XDrawLine(awt_display,
              gdata->drawable,
              gdata->gc,
              tx1 + 1, ty, tx2 - 1, ty);
    XDrawLine(awt_display,
              gdata->drawable,
              gdata->gc,
              txw, ty1 + 1, txw, ty2 - 1);

    XDrawLine(awt_display,
              gdata->drawable,
              gdata->gc,
              tx1 + 1, tyh, tx2 - 1, tyh);
    XDrawLine(awt_display,
              gdata->drawable,
              gdata->gc,
              tx, ty1 + 1, tx, ty2 - 1);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devFillRoundRect
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devFillRoundRect
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h, jint arcWidth, jint arcHeight) 
{
    struct GraphicsData *gdata;
    long tx, txw, ty, ty1, ty2, tyh, tx1, tx2;
    long baseX,baseY;

    if (w <= 0 || h <= 0) {
        return;
    }
    arcWidth = ABS(arcWidth);
    arcHeight = ABS(arcHeight);
    if (arcWidth > w)
        arcWidth = w;
    if (arcHeight > h)
        arcHeight = h;
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);

    baseX = gdata->originX;
    baseY = gdata->originY;

    tx = baseX + x;
    txw = baseX + x + w;
    ty = baseY + y;
    tyh = baseY + y + h;

    ty1 = baseY + y + (arcHeight / 2);
    ty2 = baseY + y + h - (arcHeight / 2);
    tx1 = baseX + x + (arcWidth / 2);
    tx2 = baseX + x + w - (arcWidth / 2);

    awt_drawArc(env, this, gdata,
                x, y, arcWidth, arcHeight,
                90, 90, 1);
    awt_drawArc(env, this, gdata,
                x + w - arcWidth, y, arcWidth, arcHeight,
                0, 90, 1);
    awt_drawArc(env, this, gdata,
                x, y + h - arcHeight, arcWidth, arcHeight,
                180, 90, 1);
    awt_drawArc(env, this, gdata,
                x + w - arcWidth, y + h - arcHeight, arcWidth, arcHeight,
                270, 90, 1);

    XFillRectangle(awt_display,
                   gdata->drawable,
                   gdata->gc,
                   tx1, ty, tx2 - tx1, tyh - ty);
    XFillRectangle(awt_display,
                   gdata->drawable,
                   gdata->gc,
                   tx, ty1, tx1 - tx, ty2 - ty1);
    XFillRectangle(awt_display,
                   gdata->drawable,
                   gdata->gc,
                   tx2, ty1, txw - tx2, ty2 - ty1);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawPoly
 * Signature: ([I[IIZ)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawPoly
  (JNIEnv *env, jobject this, jintArray xcoords, jintArray ycoords, jint npoints, jboolean close) 
{
    struct GraphicsData *gdata;
    XPoint pTmp[POLYTEMPSIZE], *points;

    if (JNU_IsNull(env, xcoords) || JNU_IsNull(env, ycoords)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    if ((*env)->GetArrayLength(env, ycoords) < npoints
        || (*env)->GetArrayLength(env, xcoords) < npoints) {
        JNU_ThrowIllegalArgumentException(env, "IllegalArgumentException");
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);
    points = transformPoints(env, gdata,
                             xcoords,
                             ycoords,
                             pTmp, (int *)&npoints, close);
    if (points == 0) {
	AWT_UNLOCK();
        JNU_ThrowOutOfMemoryError(env, "OutOfMemoryError");
        return;
    }
    XDrawLines(awt_display,
               gdata->drawable,
               gdata->gc,
               points,
               npoints,
               CoordModeOrigin);
    AWT_FLUSH_UNLOCK();

    if (points != pTmp) {
        free(points);
    }
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devFillPolygon
 * Signature: ([I[II)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devFillPolygon
  (JNIEnv *env, jobject this, jintArray xcoords, jintArray ycoords, jint npoints, jboolean evenodd) 
{
    struct GraphicsData *gdata;
    XPoint pTmp[POLYTEMPSIZE], *points;

    if (JNU_IsNull(env, xcoords) || JNU_IsNull(env, ycoords)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }
    if ((*env)->GetArrayLength(env, ycoords) < npoints
        || (*env)->GetArrayLength(env, xcoords) < npoints) {
        JNU_ThrowIllegalArgumentException(env, "IllegalArgumentException");
        return;
    }
    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);
    points = transformPoints(env, gdata,
                             xcoords,
                             ycoords,
                             pTmp, (int *)&npoints, FALSE);
    if (points == 0) {
	AWT_UNLOCK();
        JNU_ThrowOutOfMemoryError(env, "OutOfMemoryError");
        return;
    }
    XSetFillRule(awt_display, gdata->gc, evenodd ? EvenOddRule : WindingRule);
    XFillPolygon(awt_display,
                 gdata->drawable,
                 gdata->gc,
                 points,
                 npoints,
                 Complex,
                 CoordModeOrigin);
    AWT_FLUSH_UNLOCK();

    if (points != pTmp) {
        free(points);
    }
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devFillSpans
 * Signature: (Lsun/java2d/pipe/SpanIterator;J)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devFillSpans
  (JNIEnv *env, jobject this, jobject si, jlong pIterator) 
{
    SpanIteratorFuncs *pFuncs = (SpanIteratorFuncs *) jlong_to_ptr(pIterator);
    struct GraphicsData *gdata;
    void *srData;
    jint spanbox[4];

    if (pFuncs == NULL) {
	JNU_ThrowNullPointerException(env, "native iterator not supplied");
	return;
    }

    AWT_LOCK();
    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env,this,gPDataID);
    INIT_GC(awt_display, gdata, this);

    srData = (*pFuncs->open)(env, si);

    while ((*pFuncs->nextSpan)(env, srData, spanbox)) {
	XFillRectangle(awt_display,
		       gdata->drawable,
		       gdata->gc,
		       spanbox[0],
		       spanbox[1],
		       spanbox[2] - spanbox[0],
		       spanbox[3] - spanbox[1]);
    }

    (*pFuncs->close)(env, srData);

    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawOval
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawOval
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h) 
{
    AWT_LOCK();
    awt_drawArc(env, this, 0, x, y, w, h, 0, 360, 0);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devFillOval
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devFillOval
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h) 
{
    AWT_LOCK();
    awt_drawArc(env, this, 0, x, y, w, h, 0, 360, 1);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devDrawArc
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devDrawArc
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h, jint startAngle, jint endAngle) 
{
    AWT_LOCK();
    awt_drawArc(env, this, 0, x, y, w, h, startAngle, endAngle, 0);
    AWT_FLUSH_UNLOCK();
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    devFillArc
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Graphics_devFillArc
  (JNIEnv *env, jobject this, jint x, jint y, jint w, jint h, jint startAngle, jint endAngle)
{
    AWT_LOCK();
    awt_drawArc(env, this, 0, x, y, w, h, startAngle, endAngle, 1);
    AWT_FLUSH_UNLOCK();
}

static JDgaStatus
    GetLockStub(JNIEnv *env, Display *display, void **dgaDev,
		Drawable d, JDgaSurfaceInfo *pSurface,
		jint lox, jint loy, jint hix, jint hiy)
{
    return JDGA_FAILED;
}

static JDgaStatus
    ReleaseLockStub(JNIEnv *env, void *dgaDev, Drawable d)
{
    return JDGA_FAILED;
}

static void
    XRequestSentStub(JNIEnv *env, void *dgaDev, Drawable d)
{
}

static void
    LibDisposeStub(JNIEnv *env)
{
}

static JDgaLibInfo DgaLibInfoStub = {
    0,
    GetLockStub,
    ReleaseLockStub,
    XRequestSentStub,
    LibDisposeStub,
};

int
DGA_DeferredLockViewResources(JNIEnv *env, JDgaLibInfo *pInfo,
                              jobject idData, int *depth, int *byteOrder,
                              ScreenData *pScreen)
{
    struct GraphicsData *gdata;
    jint ret = JDGA_FAILED;

    /* For now, DGA is only available on Sparc platform (big endian) */
    *byteOrder = MSBFirst;
    
    /* AWT_LOCK(); */

    if (pScreen) {
/*
        static char lutDataInitialized = 0;
        static jint lutData[256];
*/
	jint lock;
	jint x, y, w, h;
	JDgaSurfaceInfo surface;

        pScreen->locked = 1;
        gdata = pScreen->gdata;
        x = (*env)->GetIntField(env, idData, gImageData.xViewAreaID);
        y = (*env)->GetIntField(env, idData, gImageData.yViewAreaID);
        w = (*env)->GetIntField(env, idData, gImageData.dxViewAreaID);
        h = (*env)->GetIntField(env, idData, gImageData.dyViewAreaID);
	lock = (*env)->GetIntField(env, idData, gImageData.lockMethodID);

	ret = (*pInfo->pGetLock)(env, awt_display, &(gdata->dgaDev),
                                    gdata->drawable, &surface,
                                    x, y, x+w, y+h);
	if (ret != JDGA_SUCCESS) {
	    /* AWT_UNLOCK(); */
	    return ret;
	}
        
	(*env)->SetIntField(env, idData, gImageData.xDeviceAreaID,
			    surface.window.lox + x);
	(*env)->SetIntField(env, idData, gImageData.yDeviceAreaID,
			    surface.window.loy + y);
	(*env)->SetIntField(env, idData, gImageData.dxDeviceAreaID,
			    w);
	(*env)->SetIntField(env, idData, gImageData.dyDeviceAreaID,
			    h);
	(*env)->SetIntField(env, idData, gImageData.xOutputAreaID,
			    surface.visible.lox);
	(*env)->SetIntField(env, idData, gImageData.yOutputAreaID,
			    surface.visible.loy);
	(*env)->SetIntField(env, idData, gImageData.dxOutputAreaID,
			    surface.visible.hix - surface.visible.lox);
	(*env)->SetIntField(env, idData, gImageData.dyOutputAreaID,
			    surface.visible.hiy - surface.visible.loy);
/*
	printf("DevArea: (%d, %d) [%d x %d]\n",
	       (*env)->GetIntField(env, idData, gImageData.xDeviceAreaID),
	       (*env)->GetIntField(env, idData, gImageData.yDeviceAreaID),
	       (*env)->GetIntField(env, idData, gImageData.dxDeviceAreaID),
	       (*env)->GetIntField(env, idData, gImageData.dyDeviceAreaID));
	printf("OutputArea: (%d, %d) [%d x %d]\n",
	       (*env)->GetIntField(env, idData, gImageData.xOutputAreaID),
	       (*env)->GetIntField(env, idData, gImageData.yOutputAreaID),
	       (*env)->GetIntField(env, idData, gImageData.dxOutputAreaID),
	       (*env)->GetIntField(env, idData, gImageData.dyOutputAreaID));
*/
	pScreen->method = METHOD_DGA;
/*
        if (8 == surface.surfaceDepth) {
            if (!lutDataInitialized) {
                int i;
                for (i=0;i<256;i++) {
                    lutData[i] = 
                        0xff000000
                   | ((gdata->awtData->color_data->awt_Colors[i].r & 0xff)<< 16)
                   | ((gdata->awtData->color_data->awt_Colors[i].g & 0xff)<< 8)
                   | ((gdata->awtData->color_data->awt_Colors[i].b & 0xff));
                }
		memcpy(lutData, gdata->awtData->color_data->awt_icmLUT, 
		       256 * sizeof(lutData[0]));
                lutDataInitialized = 1;
            }
        }
*/

	/* Copy information back to java structure */
	(*env)->SetIntField(env, idData, gImageData.dataID,
			    (jint) surface.basePtr);
	if ((gdata->awtData->awt_depth == 8) &&
	    (gdata->awtData->color_data->awt_icmLUT != NULL)) {
	    (*env)->SetIntField(env, idData, gImageData.lutDataID,
			(jint)(&gdata->awtData->color_data->awt_icmLUT [0]));
	    (*env)->SetIntField(env, idData, gImageData.lutDataLengthID,
			        256);
        }
/*
	else
	   (*env)->SetIntField(env, idData, gImageData.lutDataID,
			    (jint)(&lutData[0]));
	(*env)->SetIntField(env, idData, gImageData.lutDataLengthID,
			    256);
*/
	(*env)->SetIntField(env, idData, gImageData.scanlineStrideID,
			    surface.surfaceScan);
/*
	printf("Dev Ptr: 0x%08x\n",
	       (*env)->GetIntField(env, idData, gImageData.dataID));
	printf("Scanline Stride: %d\n",
	       (*env)->GetIntField(env, idData, gImageData.scanlineStrideID));
*/
        *depth = surface.surfaceDepth;
    }

    return ret;
}

int
XIMAGE_DeferredLockViewResources(JNIEnv *env, jobject idData,
                                 int *depth, int *byteOrder,
                                 ScreenData *pScreen)
{
    struct GraphicsData *gdata = NULL;
    XImage *dst = NULL;
    jint ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;

    /* AWT_LOCK(); */
    if (pScreen) {

	jint lock;
	jint x, y, w, h;

        pScreen->locked = 1;
        gdata = pScreen->gdata;

	x = (*env)->GetIntField(env, idData, gImageData.xViewAreaID);
	y = (*env)->GetIntField(env, idData, gImageData.yViewAreaID);
	w = (*env)->GetIntField(env, idData, gImageData.dxViewAreaID);
	h = (*env)->GetIntField(env, idData, gImageData.dyViewAreaID);
	lock = (*env)->GetIntField(env, idData, gImageData.lockMethodID);

/* REMIND: find out why this isn't needed   */
/* someone above is ignoring translateable  */
/*        x += gdata->originX;              */
/*        y += gdata->originY;              */

	if ((lock & LOCK_TO_READ) != 0) {
	    dst = XGetImage(awt_display, gdata->drawable,
			    x, y, w, h, -1, ZPixmap);
	} else {
	    dst = createImageBuf(gdata->awtData, w, h, True);
	}

	if (dst) {
/*
            static char lutDataInitialized = 0;
            static jint lutData[256];

            if (dst->depth == 8) {
                if (!lutDataInitialized) {
                    int i;
                    for (i=0;i<256;i++) {
                        lutData[i] = 
                              0xff000000
                   | ((gdata->awtData->color_data->awt_Colors[i].r & 0xff)<< 16)
                   | ((gdata->awtData->color_data->awt_Colors[i].g & 0xff)<< 8)
                   | ((gdata->awtData->color_data->awt_Colors[i].b & 0xff));
                    }
		    memcpy(lutData, gdata->awtData->color_data->awt_icmLUT, 
			256 * sizeof(lutData[0]));
                    lutDataInitialized = 1;
                }
            }
*/
            
            pScreen->method = METHOD_XIMAGE;
            ret = sun_awt_motif_X11Graphics_LOCK_OK;

	    pScreen->screenBits = dst;

	    /* Copy information back to java structure */
	    (*env)->SetIntField(env, idData, gImageData.dataID,
                                (jint) dst->data);
	    if ((gdata->awtData->awt_depth == 8) &&
	        (gdata->awtData->color_data->awt_icmLUT != NULL)) {
	        (*env)->SetIntField(env, idData, gImageData.lutDataID,
			(jint)(&gdata->awtData->color_data->awt_icmLUT [0]));
	        (*env)->SetIntField(env, idData,
                                    gImageData.lutDataLengthID, 256);
            }
            
            *depth = dst->depth;
            *byteOrder = dst->byte_order;
	}
    }

    if (ret != sun_awt_motif_X11Graphics_LOCK_OK) {
	if (dst != 0) {
	    dropImageBuf (dst, gdata->awtData->savedXImage);
	}
	/* AWT_UNLOCK(); */
    }
    return ret;
}

#define MAX_DRAWABLE_CACHE 32

typedef struct DrawableMethod {
    Drawable drawable;
    int      method;
} DrawableMethod;

static DrawableMethod dm[MAX_DRAWABLE_CACHE];

static int
getLockMethod(Drawable d) {
    int i;
    DrawableMethod *dp;
    for (i = 0; i < MAX_DRAWABLE_CACHE; i++) {
        dp = &(dm[i]);
        if (dp->drawable == 0 || dp->drawable == d) {
            return dp->method;
        }
    }
    return 0;
}

static void
setLockMethod(Drawable d, int m) {
    static int lastReuse = 0;
    int i;
    DrawableMethod *dp;
    for (i = 0; i < MAX_DRAWABLE_CACHE; i++) {
        dp = &(dm[i]);
        if (dp->drawable == 0 || dp->drawable == d) {
            dp->drawable = d;
            dp->method = m;
            return;
        }
    }
    dm[lastReuse % MAX_DRAWABLE_CACHE].drawable = d;
    dm[lastReuse % MAX_DRAWABLE_CACHE].method   = m;
    lastReuse++;
}

static int
X11DeferredLockViewResources(JNIEnv *env, jobject idData,
                             int *depth, int *byteOrder,
                             ScreenData *pScreen)
{
    int ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;
    int dgaret = JDGA_FAILED;
    int lock;
    Drawable d = pScreen->gdata->drawable;

    if (pScreen->method == 0) {
        pScreen->method = getLockMethod(d);
    }
    
    if (pScreen->method != METHOD_XIMAGE) {
        dgaret = DGA_DeferredLockViewResources(env, pJDgaInfo, idData,
                                            depth, byteOrder, pScreen);
        if (dgaret == JDGA_SUCCESS) {
            setLockMethod(d, METHOD_DGA);
            return sun_awt_motif_X11Graphics_LOCK_OK;
        }
    }
    
    lock = (*env)->GetIntField(env, idData, gImageData.lockMethodID);
    if (lock & LOCK_FASTEST_ONLY) {
        return ret;
    }
    
    if (ret != sun_awt_motif_X11Graphics_LOCK_OK) {
        ret = XIMAGE_DeferredLockViewResources(env, idData, depth,
                                               byteOrder, pScreen);
    }
    
    if (dgaret == JDGA_FAILED && ret == sun_awt_motif_X11Graphics_LOCK_OK) {
        pScreen->method = METHOD_TMP_XIMAGE; /* Temp. DGA lock failure. */
                                             /* Should retry DGA next time */
    } else {
        setLockMethod(d, METHOD_XIMAGE);
    }
    
    return ret;
}

static void getPrimitiveType(JNIEnv *env, int depth, int byteOrder,
			     AwtGraphicsConfigDataPtr awtData,
                             int *byteSwapped) {

    *byteSwapped = 0;
    
    switch (depth) {
    case 8:
        if (awtData->awt_visInfo.class == StaticGray &&
            awtData->awt_num_colors == 256) {
            awtData->primitiveType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_BYTE_GRAY;
        } else if (awtData->awtImage->clrdata.grayscale) {
            awtData->primitiveType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_INDEX8_GRAY;
        } else {
            awtData->primitiveType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_BYTE_INDEXED;
        }
        awtData->bytesPerChannel = 1;
        awtData->pixelStride = 1;
        break;
    case 15:
        if (awtData->awt_visInfo.green_mask == 0x3e0) {
            awtData->primitiveType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_555_RGB;
            if (byteOrder != nativeByteOrder) {
                *byteSwapped = 1;
            }
        }
        awtData->bytesPerChannel = 2;
        awtData->pixelStride = 1;
        break;
    case 16:
        if (awtData->awt_visInfo.green_mask == 0x7e0) {
            awtData->primitiveType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_565_RGB;
            if (byteOrder != nativeByteOrder) {
                *byteSwapped = 1;
            }
        } else if (awtData->awt_visInfo.green_mask == 0x3e0) {
	    /* Fix for 4352984 for Riva128 video board */
	    awtData->primitiveType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_555_RGB;
            if (byteOrder != nativeByteOrder) {
                *byteSwapped = 1;
            }
        }
        awtData->bytesPerChannel = 2;
        awtData->pixelStride = 1;
        break;
    case 24:
        if (awtData->awtImage->wsImageFormat.bits_per_pixel == 24) {
            awtData->primitiveType =
                (awtData->awt_visInfo.red_mask == 0xff) ?
                sun_java2d_loops_GraphicsPrimitive_TYPE_24BIT_RGB:
                sun_java2d_loops_GraphicsPrimitive_TYPE_3BYTE_BGR;
            awtData->bytesPerChannel = 1;
            awtData->pixelStride = 3;
            break;
        }
        /* FALL THROUGH for 32-bit case */
    case 32:
        {
            if (byteOrder == nativeByteOrder) {
                awtData->primitiveType = 
			((awtData->awt_visInfo.red_mask == 0xff)
                        ? sun_java2d_loops_GraphicsPrimitive_TYPE_INT_BGR
                        : sun_java2d_loops_GraphicsPrimitive_TYPE_INT_RGB);
            } else {
                awtData->primitiveType = 
			((awtData->awt_visInfo.red_mask == 0xff)
                        ? sun_java2d_loops_GraphicsPrimitive_TYPE_INT_RGBx
                        : sun_java2d_loops_GraphicsPrimitive_TYPE_INT_BGRx);
            }
            awtData->bytesPerChannel = 4;
            awtData->pixelStride = 1;
        }
        break;
    default:
        {
            char errmsg[32];
            sprintf(errmsg, "Unsupported %d-bit depth\n", depth);
            JNU_ThrowInternalError(env, errmsg);
            break;
        }
    }
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    X11LockViewResources
 * Signature: (Lsun/awt/image/ImageData;)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_motif_X11Graphics_X11LockViewResources(
   JNIEnv *env, jobject this, jobject idData)
{
    struct GraphicsData *gdata;
    ScreenData *pScreen;
    XImage *img;
    int depth;
    int byteOrder;
    jint ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;

    if (JNU_IsNull(env, idData)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
	return ret;
    }

    AWT_LOCK();
    
    gdata =
        (struct GraphicsData *) JNU_GetLongFieldAsPtr(env, this, gPDataID);
    INIT_GC2_NO_DRAW(awt_display, gdata, ret, this);

    /* We have to create the structure. */
    if (gdata->pScreen) {
        pScreen = gdata->pScreen;
    } else {
        gdata->pScreen = pScreen =
            (ScreenData *) calloc(1, sizeof(ScreenData));
    }

    if (pScreen) {
	jint x, y, w, h;
        jint type;
        jint bpChannel;

        pScreen->gdata = gdata;
	x = (*env)->GetIntField(env, idData, gImageData.xViewAreaID);
	y = (*env)->GetIntField(env, idData, gImageData.yViewAreaID);
	w = (*env)->GetIntField(env, idData, gImageData.dxViewAreaID);
	h = (*env)->GetIntField(env, idData, gImageData.dyViewAreaID);

/* REMIND: find out why this isn't needed   */
/* someone above is ignoring translateable  */
/*        x += gdata->originX;              */
/*        y += gdata->originY;              */

        /* Copy information back to java structure */
        (*env)->SetIntField(env, idData, gImageData.numChannelsID, 1);
        (*env)->SetIntField(env, idData, gImageData.xViewAreaID, x);
        (*env)->SetIntField(env, idData, gImageData.yViewAreaID, y);
        (*env)->SetIntField(env, idData, gImageData.dxViewAreaID, w);
        (*env)->SetIntField(env, idData, gImageData.dyViewAreaID, h);
        (*env)->SetIntField(env, idData, gImageData.xDeviceAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.yDeviceAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.dxDeviceAreaID, w);
        (*env)->SetIntField(env, idData, gImageData.dyDeviceAreaID, h);
        (*env)->SetIntField(env, idData, gImageData.xOutputAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.yOutputAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.dxOutputAreaID, w);
        (*env)->SetIntField(env, idData, gImageData.dyOutputAreaID, h);
        /*
        (*env)->SetIntField(env, idData, gImageData.lockFunctionID,
                            (jint)X11DeferredLockViewResources);
			    */
	ret = X11DeferredLockViewResources(env, idData, &depth,
                                           &byteOrder, pScreen);
        if (ret != sun_awt_motif_X11Graphics_LOCK_OK) {
            AWT_UNLOCK();
            return ret;
        }
	if (gdata->awtData->primitiveType == -255)
           getPrimitiveType(env, depth, byteOrder, gdata->awtData,
                            &(gdata->awtData->byteSwapped));
        
        (*env)->SetIntField(env, idData, gImageData.pixelStrideID,
                            gdata->awtData->pixelStride);

        /* swap byte order */
        img = pScreen->screenBits;
        if (img != 0) {
            /* swap byte order */
            if (gdata->awtData->byteSwapped) {
                jint lock =
                    (*env)->GetIntField(env, idData, gImageData.lockMethodID);
                if ((lock & LOCK_TO_READ) != 0) {
                    int lengthInBytes = img->height * img->bytes_per_line;
                    switch (depth) {
                    case 15:
                    case 16:
                        {
                            unsigned short *d = (unsigned short *) img->data;
                            unsigned short t;
                            int i;
                            for (i = 0; i < lengthInBytes/2; i++) {
                                t = *d;
                                *d++ = (t >> 8) | (t << 8);
                            }
                            break;
                        }
                    default: /* all other types should not be swapped as now */
                        break;
                    }
                }
                img->byte_order = nativeByteOrder;
                img->bitmap_bit_order = nativeByteOrder;
            }

            (*env)->SetIntField(env, idData, gImageData.scanlineStrideID,
                    img->bytes_per_line / gdata->awtData->bytesPerChannel);
        }

        (*env)->SetIntField(env, idData, gImageData.typeID, 
					gdata->awtData->primitiveType);
        if (gdata->awtData->primitiveType ==
            sun_java2d_loops_GraphicsPrimitive_TYPE_INDEX8_GRAY) {
                                     /* setup LUT data for INDEX8_GRAY loops */
            static jfieldID rgbfid = 0;
            static jfieldID sizefid = 0;
            jobject icm;
            jintArray rgb;
            jint map_size;

            icm = (*env)->GetObjectField(env, idData, gImageData.colorModelID);
            if (rgbfid == 0) {
                jclass clazz;
                clazz = (*env)->GetObjectClass(env, icm);
                rgbfid = (*env)->GetFieldID(env, clazz, "rgb", "[I");
                sizefid = (*env)->GetFieldID(env, clazz, "map_size", "I");
            }
            rgb = (jintArray)((*env)->GetObjectField(env, icm, rgbfid));
            map_size = (*env)->GetIntField(env, icm, sizefid);
            (*env)->SetObjectField(env, idData, gImageData.lutArrayID, rgb);
            (*env)->SetIntField(env, idData, gImageData.lutDataLengthID,
                                map_size);
        }
        (*env)->SetIntField(env, idData, 
                            gImageData.bytePerChannelID, 
			    gdata->awtData->bytesPerChannel);
        if (depth < 8 && ret == sun_awt_motif_X11Graphics_LOCK_OK) {
            (*env)->SetIntField(env, idData,
                                gImageData.pixelsPerDataUnitID,
                                8/depth);
        } else {
            (*env)->SetIntField(env, idData,
                                gImageData.pixelsPerDataUnitID, 1);
        }
        (*env)->SetIntField(env, idData, gImageData.platformInfoID,
                            (jint)pScreen);
    }

    if (ret != sun_awt_motif_X11Graphics_LOCK_OK) {
	AWT_UNLOCK();
    }

    return ret;
}

void
DGA_UnlockViewResources(JNIEnv *env, JDgaLibInfo *pInfo, void *dgaDev,
                        Drawable d)
{
    if (pInfo == NULL) {
	return;
    }
    (*pInfo->pReleaseLock)(env, dgaDev, d);
}

/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    X11UnLockViewResources
 * Signature: (Lsun/awt/image/ImageData;)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_motif_X11Graphics_X11UnLockViewResources(
   JNIEnv *env, jobject this, jobject idData)
{
    struct GraphicsData *gdata;
    ScreenData *pScreen;
    jint ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;

    if (JNU_IsNull(env, idData)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
	return ret;
    }

    gdata = (struct GraphicsData *) JNU_GetLongFieldAsPtr(env, this, gPDataID);
    INIT_GC2_NO_DRAW(awt_display, gdata, ret, this);

    pScreen = (ScreenData *)gdata->pScreen;

    if (pScreen) {
	if (pScreen->screenBits) {
	    jint x, y, w, h, lock;
            jint xpar;

	    x = (*env)->GetIntField(env, idData, gImageData.xViewAreaID);
	    y = (*env)->GetIntField(env, idData, gImageData.yViewAreaID);
	    w = (*env)->GetIntField(env, idData, gImageData.dxViewAreaID);
	    h = (*env)->GetIntField(env, idData, gImageData.dyViewAreaID);
	    lock = (*env)->GetIntField(env, idData, gImageData.lockMethodID);

/* REMIND: find out why this isn't needed   */
/* someone above is ignoring translateable  */
/*        x += gdata->originX;              */
/*        y += gdata->originY;              */

	    if ((lock & LOCK_TO_WRITE) != 0) {
		if (pScreen->method == METHOD_XIMAGE ||
                    pScreen->method == METHOD_TMP_XIMAGE) {
                    if (pScreen->region != NULL) {
                        XSetRegion(awt_display, gdata->gc, pScreen->region);
                    }
                    if (!pScreen->fUsingRegion || pScreen->region != NULL) {
                        if (gdata->xormode) { /* XOR rendering is done
                                                 in composite loop */
                            XSetFunction(awt_display, gdata->gc, GXcopy);
                        }
                        DoPutImage(awt_display, gdata->drawable,
                                   gdata->gc, pScreen->screenBits,
                                   0, 0, x, y, w, h);
                        if (gdata->xormode) { /* restore XOR mode */
                            XSetFunction(awt_display, gdata->gc, GXxor);
                        }
                    }
                    if (pScreen->region != NULL) {
                        XSetClipMask(awt_display, gdata->gc, None);
                        XDestroyRegion(pScreen->region);
                        pScreen->region = 0;
                    }
		    ret = sun_awt_motif_X11Graphics_UNLOCK_OK;
		}
	    } else {
		ret = sun_awt_motif_X11Graphics_UNLOCK_OK;
	    }
	    dropImageBuf(pScreen->screenBits, gdata->awtData->savedXImage);
            pScreen->fUsingRegion = 0;
            pScreen->screenBits = 0;
	}

	/* blitters only call the deferred locking if the dataID is NULL */
	(*env)->SetIntField(env, idData, gImageData.dataID, 0);
        DGA_UnlockViewResources(env, pJDgaInfo, gdata->dgaDev,
                                gdata->drawable);

    }

    AWT_UNLOCK();

    ret = sun_awt_motif_X11Graphics_UNLOCK_OK;
    return ret;
}

#ifdef MITSHM
static XShmSegmentInfo *
getSharedSegment(int image_size)
{
    XShmSegmentInfo *shminfo;
    
    if (!XShmQueryExtension(awt_display)) {
	return 0;
    }

    shminfo = malloc(sizeof *shminfo);
    if (shminfo == 0) {
	return 0;
    }
    shminfo->shmid = shmget(IPC_PRIVATE, image_size, IPC_CREAT|0777);
    if (shminfo->shmid < 0) {
	free((void *)shminfo);
	return 0;
    }
    shminfo->shmaddr = (char *) shmat(shminfo->shmid, 0, 0);
    if (shminfo->shmaddr == ((char *) -1)) {
	free((void *)shminfo);
	return 0;
    }
    shminfo->readOnly = True;
    XShmAttach (awt_display, shminfo);
    XSync(awt_display, False);
    /*
     * Once the XSync round trip has finished then we know that the
     * server has attached to the shared memory segment and so now
     * we can get rid of the id so that this segment does not stick
     * around after we go away, holding system resources.
     */
    shmctl(shminfo->shmid, IPC_RMID, 0);
    return shminfo;
}
#endif

void
DoPutImage(Display *dsp, Drawable win, GC gc, XImage *img,
	   int sx, int sy, int dx, int dy, int w, int h)
{
#ifdef MITSHM
    if (using_mitshm) {
        if (img->obdata != 0) {
	    XShmPutImage(dsp, win, gc, img, sx, sy, dx, dy, w, h, False);
            XFlush(awt_display);
	    return;
        }
    }
#endif /* MITSHM */
    XPutImage(dsp, win, gc, img, sx, sy, dx, dy, w, h);
    XFlush(awt_display);
}

void
dropImageBuf(XImage *img, XImage *savedXImage)
{
    if (img != savedXImage) {
#ifdef MITSHM
        if (using_mitshm) {
	    XShmSegmentInfo *shminfo = (XShmSegmentInfo *) img->obdata;
	    if (shminfo != 0) {
	        XShmDetach(awt_display, shminfo);
	        shmdt(shminfo->shmaddr);
	        /* shmctl(shminfo->shmid, IPC_RMID, 0); */
	        free((void *)shminfo);
	        XFree(img);
	        return;
            }
	}
#endif /* MITSHM */
	XDestroyImage(img);
    }
}

XImage *
createImageBuf(AwtGraphicsConfigDataPtr awtData, int width, int height, int always)
{
    XImage *img;
    int bpsl;
    int image_size;
    int depth = awtData->awtImage->Depth;
    int bpp = awtData->awtImage->wsImageFormat.bits_per_pixel;
    int slp = awtData->awtImage->wsImageFormat.scanline_pad;
#ifdef MITSHM
    XShmSegmentInfo *shminfo;
#endif /* MITSHM */

    if (awtData->savedXImage != 0) {
        img = awtData->savedXImage;

        if ((width <= img->width) && (width + 64 > img->width)) {
           width = img->width;
           if ((height <= img->height) && (height + 64 > img->height)) {
               /* if this is mitshm, Sync so that previous data gets flushed */
               if (img->obdata != 0) {
                   XSync(awt_display, False);
               }
               return awtData->savedXImage;
           }
        } else if ((height <= img->height) && (height + 64 > img->height)) {
            height = img->height;
	}
    }
    bpsl = paddedwidth(width * bpp, slp) >> 3;
    if (((bpsl << 3) / bpp) < width) {
	return (XImage *) 0;
    }
    img = XCreateImage(awt_display, awtData->awt_visInfo.visual,
		       depth, ZPixmap, 0,
		       0, width, height, 32, bpsl);
    if (img == 0) {
	return (XImage *) 0;
    }
    img->bits_per_pixel = bpp;
    image_size = height * img->bytes_per_line;
    if (image_size / height != img->bytes_per_line) {
	/* Overflow */
	XFree(img);
	return (XImage *) 0;
    }
    if (using_mitshm && (image_size > 0x10000)) {
       /* Only consider using mitshm for large images (~256X256), else the 
        * XSync() required above slows us up more than mitshm speeds us up.
        */
#ifdef MITSHM
        shminfo = getSharedSegment(image_size);
        if (shminfo == 0) {
	    if (!always) {
	        XFree(img);
	        return (XImage *) 0;
	    }
	    img->obdata = (char *) 0;
	    img->data = malloc(image_size);
        } else {
	    img->obdata = (char *) shminfo;
	    img->data = shminfo->shmaddr;
        }
#else /* MITSHM */
        img->data = malloc(image_size);
#endif /* MITSHM */
    } else {
          img->data = malloc(image_size);
    }
    if (img->data == 0) {
	XFree(img);
	return (XImage *) 0;
    }
    if (image_size < (1 << 20)) {
	XImage *tmp = awtData->savedXImage;
	awtData->savedXImage = img;
	if (tmp != 0) {
	    dropImageBuf (tmp, awtData->savedXImage);
	}
    }
    return img;
}

void AddToRegion(Region *regionP, int startX,
                 int startY, int x, int y, int w, int endX, int endY) {
    XRectangle rect;

    if (*regionP == NULL) {
        *regionP = XCreateRegion();
    }
    
    rect.x = startX + x;
    rect.y = startY + y;

    if (endY != startY) {
        if (startX != 0) {
            /* Write out the partial line */
            rect.width=w-startX;
            rect.height=1;
            XUnionRectWithRegion(&rect, *regionP, *regionP);
            /* Set it up for the next rectangle */
            rect.x=x;
            rect.y++;
            startY++;
        }
        if (endY == startY) {
            if (endX != 0) {
                rect.width = endX;
                XUnionRectWithRegion(&rect, *regionP, *regionP);
            }
        }
        else {
            rect.width=w;
            rect.height=endY-startY;
            XUnionRectWithRegion(&rect, *regionP, *regionP);
            if (endX != 0) {
                rect.y += rect.height;
                rect.width = endX;
                rect.height = 1;
                XUnionRectWithRegion(&rect, *regionP, *regionP);
            }
        }
    }
    else {
        rect.width = endX-startX;
        rect.height = 1;
        XUnionRectWithRegion(&rect, *regionP, *regionP);
    }
}

XImage *
createMonoImageBuf( struct GraphicsData *gdata, int width, int height)
{
    XImage *img;
    int bpsl;
    int image_size;
    int slp = gdata->awtData->awtImage->wsImageFormat.scanline_pad;

    if (gdata->monoImage != NULL) {
       img = gdata->monoImage;
       if (width <= img->width && height <= img->height) {
          memset(img->data, 0, img->bytes_per_line*height);
          return img;
       } else {
           XDestroyImage(img);
           img = gdata->monoImage = NULL;
       }
    }

    bpsl = paddedwidth(width, slp) >> 3;
    img = XCreateImage(awt_display, gdata->awtData->awt_visInfo.visual,
                     1, ZPixmap, 0, 0, width, height, 32, bpsl);

    if (img == 0) {
       return (XImage *) 0;
    }
    image_size = height * img->bytes_per_line;
    if (image_size / height != img->bytes_per_line) {
       /* Overflow */
       XFree(img);
       return (XImage *) 0;
    }
    img->data = calloc(image_size, 1);

    if (img->data == 0) {
       XFree(img);
       return (XImage *) 0;
    }
    gdata->monoImage = img;
    return img;
}
 
/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    X11getMonoImageData
 * Signature: (Lsun/awt/image/ImageData;)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_motif_X11Graphics_X11getMonoImageData(
   JNIEnv *env, jobject this, jobject idData)
{
    struct GraphicsData *gdata;
    ScreenData *pScreen;
    XImage *dst = NULL;
    jint ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;

    if (JNU_IsNull(env, idData)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return ret;
    }

    AWT_LOCK();
    
    gdata =
        (struct GraphicsData *) (*env)->GetLongField(env, this, gPDataID);
    INIT_GC2_NO_DRAW(awt_display, gdata, ret, this);

    /* We have to create the structure. */
    if (gdata->pScreen) {
        pScreen = gdata->pScreen;
    } else {
        gdata->pScreen = pScreen =
            (ScreenData *) calloc(1, sizeof(ScreenData));
    }

    if (pScreen) {
        jint x, y, w, h;
        jint type;
        jint bpChannel;

        pScreen->gdata = gdata;
        x = (*env)->GetIntField(env, idData, gImageData.xViewAreaID);
        y = (*env)->GetIntField(env, idData, gImageData.yViewAreaID);
        w = (*env)->GetIntField(env, idData, gImageData.dxViewAreaID);
        h = (*env)->GetIntField(env, idData, gImageData.dyViewAreaID);

/* REMIND: find out why this isn't needed   */
/* someone above is ignoring translateable  */
/*        x += gdata->originX;              */
/*        y += gdata->originY;              */

        /* Copy information back to java structure */
        (*env)->SetIntField(env, idData, gImageData.pixelStrideID, 1);
        (*env)->SetIntField(env, idData, gImageData.numChannelsID, 1);
        (*env)->SetIntField(env, idData, gImageData.xViewAreaID, x);
        (*env)->SetIntField(env, idData, gImageData.yViewAreaID, y);
        (*env)->SetIntField(env, idData, gImageData.dxViewAreaID, w);
        (*env)->SetIntField(env, idData, gImageData.dyViewAreaID, h);
        (*env)->SetIntField(env, idData, gImageData.xDeviceAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.yDeviceAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.dxDeviceAreaID, w);
        (*env)->SetIntField(env, idData, gImageData.dyDeviceAreaID, h);
        (*env)->SetIntField(env, idData, gImageData.xOutputAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.yOutputAreaID, 0);
        (*env)->SetIntField(env, idData, gImageData.dxOutputAreaID, w);
        (*env)->SetIntField(env, idData, gImageData.dyOutputAreaID, h);

      /** ... XXXNONDGAXXX ... ***/
#if 1
        (*env)->SetIntField(env, idData, gImageData.bytePerChannelID, 0);
                          
        (*env)->SetIntField(env, idData, gImageData.pixelsPerDataUnitID, 8);

#else
        (*env)->SetIntField(env, idData,
                            gImageData.bytePerChannelID,
                            gdata->awtData->bytesPerChannel);
        (*env)->SetIntField(env, idData,
                                gImageData.pixelsPerDataUnitID, 1);
#endif
 
        pScreen->locked = 1;
        dst = createMonoImageBuf(gdata, w, h);

        if (dst) {
            ret = sun_awt_motif_X11Graphics_LOCK_OK;

            (*env)->SetIntField(env, idData, gImageData.typeID,
                    sun_java2d_loops_GraphicsPrimitive_TYPE_1BIT_MONO);

            pScreen->screenBits = dst;

            /* Copy information back to java structure */
            (*env)->SetIntField(env, idData, gImageData.dataID,
                                (jint) dst->data);

            (*env)->SetIntField(env, idData, gImageData.scanlineStrideID, 
                              (jint) dst->bytes_per_line);
        } else {
            ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;
        }

    }

    if (ret != sun_awt_motif_X11Graphics_LOCK_OK) {
       AWT_UNLOCK();
    }

    return ret;
}


/*
 * Class:     sun_awt_motif_X11Graphics
 * Method:    X11putMonoImageData
 * Signature: (Lsun/awt/image/ImageData;)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_motif_X11Graphics_X11putMonoImageData(
   JNIEnv *env, jobject this, jobject idData)
{
    struct GraphicsData *gdata;
    ScreenData *pScreen;
    XImage *img;
    jint ret = sun_awt_motif_X11Graphics_LOCK_DATA_STRUCT_PROBLEM;

    if (JNU_IsNull(env, idData)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return ret;
    }

    gdata = (struct GraphicsData *) (*env)->GetLongField(env, this, gPDataID);
    INIT_GC2_NO_DRAW(awt_display, gdata, ret, this);

    pScreen = (ScreenData *)gdata->pScreen;

    if (pScreen) {
       img = pScreen->screenBits;
       if (img != NULL ) {
          jint sx, sy, dx, dy, w, h, i, j;
          sx = sy = 0;

          dx = (*env)->GetIntField(env, idData, gImageData.xViewAreaID);
          dy = (*env)->GetIntField(env, idData, gImageData.yViewAreaID);
          w = (*env)->GetIntField(env, idData, gImageData.dxViewAreaID);
          h = (*env)->GetIntField(env, idData, gImageData.dyViewAreaID);

          // testing has shown that it is faster if we remove the leading
          // and trailing blank scanlines (not true for pixels at right/left)
            {
              int first_scan = -1;
              int last_scan = 0;
              char *data;
              int len = img->bytes_per_line;

              for (i=0; i<h; i++) {
                   data = img->data + (i * len);        
                   for (j=0; j<len; j++) {
                        if (*data++) {
                            if (first_scan < 0) {
                                first_scan = i;
                            }
                            last_scan = i;
                            break;
                        }
                   }
              }
              if (first_scan >= 0) {
                  h = last_scan - first_scan + 1;
                  sy += first_scan;
                  dy += first_scan;
              } else {
                  h = 0;
              }
            }
            if (h > 0) {
                Display *dsp = awt_display;
                GC gc = gdata->gc;

                if ( gdata->awtData->monoPixmap != (Pixmap) 0 ) {
                     if ( gdata->awtData->monoPixmapWidth < w ||
                          gdata->awtData->monoPixmapHeight < h ) {
                          XFreePixmap(dsp, gdata->awtData->monoPixmap);
                          gdata->awtData->monoPixmap = (Pixmap) 0;
                     }
                }
                if ( gdata->awtData->monoPixmap == (Pixmap) 0 ) {
                     gdata->awtData->monoPixmapWidth = w;
                     gdata->awtData->monoPixmapHeight = h;
                     gdata->awtData->monoPixmap = XCreatePixmap (dsp, 
                       RootWindow(dsp, gdata->awtData->awt_visInfo.screen),
                       gdata->awtData->monoPixmapWidth,
                       gdata->awtData->monoPixmapHeight, 1);
                }
                if ( gdata->awtData->monoPixmap != (Pixmap) 0 ) {
                     if ( gdata->awtData->monoPixmapGC == (GC) 0 ) {
                          gdata->awtData->monoPixmapGC = XCreateGC ( dsp,
                          gdata->awtData->monoPixmap, 0, 0 );
                     }

                     img->byte_order = nativeByteOrder;
                     img->bitmap_bit_order = nativeByteOrder;

                     XPutImage(dsp, gdata->awtData->monoPixmap,
                               gdata->awtData->monoPixmapGC,
                               img, sx, sy, 0, 0, w, h);
                     XSetStipple ( dsp, gc, gdata->awtData->monoPixmap );
                     XSetFillStyle ( dsp, gc, FillStippled );
                     XSetTSOrigin ( dsp, gc, dx, dy );
                     XFillRectangle( dsp, gdata->drawable, gc, dx, dy, w, h);
                     XSetFillStyle ( dsp, gc, FillSolid );
                }
            }

            if (gdata->xormode) { /* restore XOR mode */
                XSetFunction(awt_display, gdata->gc, GXxor);
            }
            if (pScreen->region != NULL) {
                XSetClipMask(awt_display, gdata->gc, None);
                XDestroyRegion(pScreen->region);
                pScreen->region = 0;
            }
            if (img != gdata->monoImage) {
                XDestroyImage(img);
            }
            pScreen->fUsingRegion = 0;
            pScreen->screenBits = 0;
      }

      /* blitters only call the deferred locking if the dataID is NULL */
      (*env)->SetIntField(env, idData, gImageData.dataID, 0);

      AWT_UNLOCK();

    }

    ret = sun_awt_motif_X11Graphics_UNLOCK_OK;
    return ret;
}
