/*
 * @(#)awt_GraphicsEnv.c	1.31 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_p.h"
#include "awt.h"
#include "color.h"
#include <sun_awt_X11GraphicsEnvironment.h>
#include <sun_awt_X11GraphicsDevice.h>
#include <sun_awt_X11GraphicsConfig.h>

#include <jni.h>
#include <jni_util.h>
#include <jvm.h>

#include <stdlib.h>

#include "awt_GraphicsEnv.h"

#ifdef NETSCAPE
#include <signal.h>
extern int awt_init_xt;
#endif

#ifdef __linux__
#include <X11/XKBlib.h>
#endif

int awt_numScreens;

static AwtScreenDataPtr x11Screens;

Display *awt_display;

struct X11GraphicsConfigIDs x11GraphicsConfigIDs;

JNIEXPORT void JNICALL
Java_sun_awt_X11GraphicsConfig_initIDs (JNIEnv *env, jclass cls)
{
    x11GraphicsConfigIDs.aData = (*env)->GetFieldID (env, cls, "aData", "J");
} 

/*
 * error handlers
 */

int
xerror_handler(Display * disp, XErrorEvent * err)
{
#ifdef DEBUG
    char msg[128];
    char buf[128];
    char *ev = getenv("NOISY_AWT");

    if (!ev || !ev[0])
        return 0;
    XGetErrorText(disp, err->error_code, msg, sizeof(msg));
    jio_fprintf(stderr, "Xerror %s\n", msg);
    jio_snprintf(buf, sizeof(buf), "%d", err->request_code);
    XGetErrorDatabaseText(disp, "XRequest", buf, "Unknown", msg, sizeof(msg));
    jio_fprintf(stderr, "Major opcode %d (%s)\n", err->request_code, msg);
    if (err->request_code > 128) {
        jio_fprintf(stderr, "Minor opcode %d\n", err->minor_code);
    }
    if (awt_lock) {
        /*SignalError(lockedee->lastpc, lockedee, "fp/ade/gui/GUIException", msg); */
    }
    if (strcasecmp(ev, "abort") == 0) {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

        (*env)->FatalError(env, "xerror_handler abort");
    }
#endif
    return 0;
}

static int 
xioerror_handler(Display * disp)
{
    if (awt_lock) {
        /*SignalError(lockedee->lastpc, lockedee, "fp/ade/gui/GUIException", "I/O error"); */
    }
    return 0;
}

static AwtGraphicsConfigDataPtr
makeDefaultConfig(JNIEnv *env, int screen) {

    int blackpixel;
    int whitepixel;
    Visual *awt_visual;
    int awt_depth;
    Colormap awt_cmap;
    XVisualInfo viTmp, *pVI;
    XVisualInfo awt_visInfo;
    XColor color;
    int numvi;
    AwtGraphicsConfigDataPtr defaultConfig;
    
    
    if (!getenv("FORCEDEFVIS") && XMatchVisualInfo(awt_display, screen,
                                                   24, TrueColor,
                                                   &awt_visInfo)) {
        awt_visual = awt_visInfo.visual;
        awt_depth = awt_visInfo.depth;
        if (awt_visual == DefaultVisual(awt_display, screen)) {
            awt_cmap = DefaultColormap(awt_display, screen);
        } else {
            awt_cmap = XCreateColormap(awt_display, x11Screens[screen].root,
                                       awt_visual, AllocNone);
        }
        color.flags = DoRed | DoGreen | DoBlue;
        color.red = color.green = color.blue = 0x0000;
        XAllocColor(awt_display, awt_cmap, &color);
        blackpixel = color.pixel;
        color.flags = DoRed | DoGreen | DoBlue;
        color.red = color.green = color.blue = 0xffff;
        XAllocColor(awt_display, awt_cmap, &color);
        whitepixel = color.pixel;
    } else {
        awt_visual = DefaultVisual(awt_display, screen);
        awt_depth = DefaultDepth(awt_display, screen);
        awt_cmap = DefaultColormap(awt_display, screen);
        viTmp.visualid = XVisualIDFromVisual(awt_visual);
        viTmp.depth = awt_depth;
        viTmp.screen = screen;
        pVI = XGetVisualInfo(awt_display,
                             VisualIDMask | VisualDepthMask |
                             VisualScreenMask,
                             &viTmp, &numvi);
        if (!pVI) {
            /* This should never happen. */
            JNU_ThrowInternalError(env, "Can't find default visual information");
	    XCloseDisplay(awt_display);
	    awt_display = NULL;
            return NULL;
        }
        awt_visInfo = *pVI;
        XFree(pVI);
        blackpixel = BlackPixel(awt_display, screen);
        whitepixel = WhitePixel(awt_display, screen);
    }

    defaultConfig = ZALLOC (_AwtGraphicsConfigData);
    defaultConfig->primitiveType = -255;
    defaultConfig->awt_depth = awt_depth;
    memcpy (&defaultConfig->awt_visInfo, &awt_visInfo, sizeof (XVisualInfo));
    defaultConfig->awt_cmap = awt_cmap;
    if (!awt_allocate_colors(defaultConfig)) {
        char errmsg[128];
        static char *visnames[] = {"StaticGray", "GrayScale", "StaticColor",
				   "PseudoColor", "TrueColor", "DirectColor"};
	jio_snprintf(errmsg,
		     sizeof(errmsg),
		     "Display type (%s) and depth (%d) not supported.",
		     visnames[awt_visInfo.class], awt_depth);
        JNU_ThrowInternalError(env, errmsg);
	XCloseDisplay(awt_display);
	awt_display = NULL;
        return NULL;
    }

    x11Screens[screen].blackpixel = blackpixel;
    x11Screens[screen].whitepixel = whitepixel;
    return defaultConfig;
}

static void
getAllConfigs (JNIEnv *env, int screen, AwtScreenDataPtr screenDataPtr) {

    int i;
    int n8p=0, n8s=0, n8gs=0, n8sg=0, n1sg=0, nTrue=0;
    int nConfig;
    XVisualInfo *pVI8p, *pVI8s, *pVITrue, *pVI8gs,
                *pVI8sg, *pVI1sg = NULL, viTmp;
    AwtGraphicsConfigDataPtr *graphicsConfigs;
    AwtGraphicsConfigDataPtr defaultConfig;
    int ind;
    char errmsg[128];

    AWT_LOCK ();

    viTmp.screen = screen;
    
    viTmp.depth = 8;
    viTmp.class = PseudoColor;
    viTmp.colormap_size = 256;
    pVI8p = XGetVisualInfo (awt_display, 
                            VisualDepthMask | VisualClassMask |
                            VisualColormapSizeMask | VisualScreenMask, 
                            &viTmp, &n8p);
    
    viTmp.class = TrueColor;
    pVITrue = XGetVisualInfo (awt_display,
                              VisualClassMask |
                              VisualScreenMask,
                              &viTmp, &nTrue);
    
    viTmp.depth = 8;
    viTmp.class = StaticColor;
    pVI8s = XGetVisualInfo (awt_display, VisualDepthMask | VisualClassMask |
                            VisualScreenMask, &viTmp, &n8s);

    viTmp.depth = 8;
    viTmp.class = GrayScale;
    viTmp.colormap_size = 256;
    pVI8gs = XGetVisualInfo (awt_display, 
                             VisualDepthMask | VisualClassMask |
                             VisualColormapSizeMask | VisualScreenMask,
                             &viTmp, &n8gs);
    viTmp.depth = 8;
    viTmp.class = StaticGray;
    viTmp.colormap_size = 256;
    pVI8sg = XGetVisualInfo (awt_display, 
                             VisualDepthMask | VisualClassMask |
                             VisualColormapSizeMask | VisualScreenMask,
                             &viTmp, &n8sg);

/* REMIND.. remove when we have support for the color classes below */
/*     viTmp.depth = 1; */
/*     viTmp.class = StaticGray; */
/*     pVI1sg = XGetVisualInfo (awt_display, VisualDepthMask | VisualClassMask, */
/*                              viTmp, &n1sg); */
    
    nConfig = n8p + n8s + n8gs + n8sg  + n1sg + nTrue + 1;
    graphicsConfigs = (AwtGraphicsConfigDataPtr *)
        calloc(nConfig, sizeof(AwtGraphicsConfigDataPtr));
    if (graphicsConfigs == NULL) {
	JNU_ThrowOutOfMemoryError((JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2),
                                  NULL);
    }

    defaultConfig = screenDataPtr->defaultConfig;
    graphicsConfigs[0] = defaultConfig;
    nConfig = 1; /* reserve index 0 for default config */

    for (i = 0; i < nTrue; i++) {
        if (XVisualIDFromVisual(pVITrue[i].visual) ==
            XVisualIDFromVisual(defaultConfig->awt_visInfo.visual)) {
            continue;
        } else {
            ind = nConfig++;
        }
        graphicsConfigs [ind] = ZALLOC (_AwtGraphicsConfigData);
	graphicsConfigs [ind]->primitiveType = -255;
        graphicsConfigs [ind]->awt_depth = pVITrue [i].depth;
        memcpy (&graphicsConfigs [ind]->awt_visInfo, &pVITrue [i],
                sizeof (XVisualInfo));
    }
 
    for (i = 0; i < n8p; i++) {
        if (XVisualIDFromVisual(pVI8p[i].visual) ==
            XVisualIDFromVisual(defaultConfig->awt_visInfo.visual)) {
            continue;
        } else {
            ind = nConfig++;
        }
        graphicsConfigs [ind] = ZALLOC (_AwtGraphicsConfigData);
	graphicsConfigs [ind]->primitiveType = -255;
        graphicsConfigs [ind]->awt_depth = pVI8p [i].depth;
        memcpy (&graphicsConfigs [ind]->awt_visInfo, &pVI8p [i],
                sizeof (XVisualInfo));
    }
       
    for (i = 0; i < n8s; i++) {
        if (XVisualIDFromVisual(pVI8s[i].visual) ==
            XVisualIDFromVisual(defaultConfig->awt_visInfo.visual)) {
            continue;
        } else {
            ind = nConfig++;
        }
        graphicsConfigs [ind] = ZALLOC (_AwtGraphicsConfigData);
	graphicsConfigs [ind]->primitiveType = -255;
        graphicsConfigs [ind]->awt_depth = pVI8s [i].depth;
        memcpy (&graphicsConfigs [ind]->awt_visInfo, &pVI8s [i],
                sizeof (XVisualInfo));
    }

    for (i = 0; i < n8gs; i++) {
        if (XVisualIDFromVisual(pVI8gs[i].visual) ==
            XVisualIDFromVisual(defaultConfig->awt_visInfo.visual)) {
            continue;
        } else {
            ind = nConfig++;
        }
        graphicsConfigs [ind] = ZALLOC (_AwtGraphicsConfigData);
	graphicsConfigs [ind]->primitiveType = -255;
        graphicsConfigs [ind]->awt_depth = pVI8gs [i].depth;
        memcpy (&graphicsConfigs [ind]->awt_visInfo, &pVI8gs [i],
                sizeof (XVisualInfo));
    }

    for (i = 0; i < n8sg; i++) {
        if (XVisualIDFromVisual(pVI8sg[i].visual) ==
            XVisualIDFromVisual(defaultConfig->awt_visInfo.visual)) {
            continue;
        } else {
            ind = nConfig++;
        }
        graphicsConfigs [ind] = ZALLOC (_AwtGraphicsConfigData);
	graphicsConfigs [ind]->primitiveType = -255;
        graphicsConfigs [ind]->awt_depth = pVI8sg [i].depth;
        memcpy (&graphicsConfigs [ind]->awt_visInfo, &pVI8sg [i],
                sizeof (XVisualInfo));
    }

    for (i = 0; i < n1sg; i++) {
        if (XVisualIDFromVisual(pVI1sg[i].visual) ==
            XVisualIDFromVisual(defaultConfig->awt_visInfo.visual)) {
            continue;
        } else {
            ind = nConfig++;
        }
        graphicsConfigs [ind] = ZALLOC (_AwtGraphicsConfigData);
	graphicsConfigs [ind]->primitiveType = -255;
        graphicsConfigs [ind]->awt_depth = pVI1sg [i].depth;
        memcpy (&graphicsConfigs [ind]->awt_visInfo, &pVI1sg [i],
                sizeof (XVisualInfo));
    }

    if (n8p != 0)
       XFree (pVI8p);
    if (n8s != 0)
       XFree (pVI8s);
    if (n8gs != 0)
       XFree (pVI8gs);
    if (n8sg != 0)
       XFree (pVI8sg);
    if (n1sg != 0)
       XFree (pVI1sg);

    screenDataPtr->numConfigs = nConfig;
    screenDataPtr->configs = graphicsConfigs;

    AWT_UNLOCK ();
}

Display *
awt_init_Display(JNIEnv *env, jobject this)
{
    Display *dpy;
    char errmsg[128];
    AwtGraphicsConfigData aData;
    jclass fontWrapper;
    int i;
#ifdef NETSCAPE
    sigset_t alarm_set, oldset;
#endif

    if (awt_display) {
        return awt_display;
    }

#ifdef NETSCAPE
    /* Disable interrupts during XtOpenDisplay to avoid bugs in unix os select
       code: some unix systems don't implement SA_RESTART properly and
       because of this, select returns with EINTR. Most implementations of
       gethostbyname don't cope with EINTR properly and as a result we get
       stuck (forever) in the gethostbyname code
    */
    sigemptyset(&alarm_set);
    sigaddset(&alarm_set, SIGALRM);
    sigprocmask(SIG_BLOCK, &alarm_set, &oldset);
#endif
    fontWrapper = (*env)->FindClass (env, "sun/awt/font/NativeFontWrapper");
    if (fontWrapper != NULL)
	awt_lock = (*env)->NewGlobalRef (env, fontWrapper);
    else
        awt_lock = (*env)->NewGlobalRef(env, this);
 
#ifdef __linux__
  if (getenv("_AWT_IGNORE_XKB") != NULL &&
      strlen(getenv("_AWT_IGNORE_XKB")) > 0) {
      if (XkbIgnoreExtension(True)) {
	  printf("Ignoring XKB.\n");
      }
  }
#endif

    dpy = awt_display = XOpenDisplay(NULL);
#ifdef NETSCAPE
    sigprocmask(SIG_SETMASK, &oldset, 0);
#endif
    if (!dpy) {
	jio_snprintf(errmsg,
		     sizeof(errmsg),
		     "Can't connect to X11 window server using '%s' as the value of the DISPLAY variable.",
		     (getenv("DISPLAY") == 0) ? ":0.0" : getenv("DISPLAY"));
        JNU_ThrowInternalError(env, errmsg);
        return NULL;
    }

    XSetErrorHandler(xerror_handler);
    XSetIOErrorHandler(xioerror_handler);

    awt_numScreens =  XScreenCount(awt_display);
    
    /* Allocate screen data structure array */
    x11Screens = calloc(awt_numScreens, sizeof(AwtScreenData));
    if (x11Screens == NULL) {
        JNU_ThrowOutOfMemoryError((JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2),
                                  NULL);
    }

    for (i = 0; i < awt_numScreens; i++) {
        x11Screens[i].root = RootWindow(awt_display, i);
        x11Screens[i].defaultConfig = makeDefaultConfig(env, i);
    }
    
    return dpy;
}

/*
 * Class:     sun_awt_X11GraphicsEnvironment
 * Method:    getDefaultScreenNum
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_X11GraphicsEnvironment_getDefaultScreenNum(
JNIEnv *env, jobject this)
{
    return DefaultScreen(awt_display);
}

AwtGraphicsConfigDataPtr
getDefaultConfig(int screen) {
    return x11Screens[screen].defaultConfig;
}

AwtScreenDataPtr
getScreenData(int screen) {
    return &(x11Screens[screen]);
}
 
/*
 * Class:     sun_awt_X11GraphicsEnvironment
 * Method:    initDisplay
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_X11GraphicsEnvironment_initDisplay(JNIEnv *env, jobject this)
{
    (void) awt_init_Display(env, this);
}

/*
 * Class:     sun_awt_X11GraphicsEnvironment
 * Method:    getNumScreens
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_X11GraphicsEnvironment_getNumScreens(JNIEnv *env, jobject this)
{
    return awt_numScreens;
}

/*
 * Class:     sun_awt_X11GraphicsDevice
 * Method:    getDisplay
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_X11GraphicsDevice_getDisplay(JNIEnv *env, jobject this)
{
    return (jint) awt_display;
}

/*
 * Class:     sun_awt_X11GraphicsDevice
 * Method:    getNumConfigs
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_X11GraphicsDevice_getNumConfigs(
JNIEnv *env, jobject this, jint screen)
{
    if (x11Screens[screen].numConfigs == 0) {
	getAllConfigs (env, screen, &(x11Screens[screen]));
    }

    return x11Screens[screen].numConfigs;
}

/*
 * Class:     sun_awt_X11GraphicsDevice
 * Method:    getConfigVisualId
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_X11GraphicsDevice_getConfigVisualId(
JNIEnv *env, jobject this, jint index, jint screen)
{
    int visNum;
 
    if (index == 0) {
        return (x11Screens[screen].defaultConfig->awt_visInfo.visualid);
    } else {
        if (x11Screens[screen].numConfigs == 0) {
            getAllConfigs (env, screen, &(x11Screens[screen]));
        }
        return (x11Screens[screen].configs[index]->awt_visInfo.visualid);
    }
}

/*
 * Class:     sun_awt_X11GraphicsConfig
 * Method:    getXResolution
 * Signature: ()I
 */
JNIEXPORT jdouble JNICALL
Java_sun_awt_X11GraphicsConfig_getXResolution(
JNIEnv *env, jobject this, jint screen)
{
    return ((DisplayWidth(awt_display, screen) * 25.4) /
	    DisplayWidthMM(awt_display, screen));
}

/*
 * Class:     sun_awt_X11GraphicsConfig
 * Method:    getYResolution
 * Signature: ()I
 */
JNIEXPORT jdouble JNICALL
Java_sun_awt_X11GraphicsConfig_getYResolution(
JNIEnv *env, jobject this, jint screen)
{
    return ((DisplayHeight(awt_display, screen) * 25.4) /
	    DisplayHeightMM(awt_display, screen));
}


/*
 * Class:     sun_awt_X11GraphicsConfig
 * Method:    getNumColors
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_X11GraphicsConfig_getNumColors(
JNIEnv *env, jobject this)
{
    AwtGraphicsConfigData *adata;
 
    adata = (AwtGraphicsConfigData *) JNU_GetLongFieldAsPtr(env, this, 
					      x11GraphicsConfigIDs.aData);

    return adata->awt_num_colors;
}

/*
 * Class:     sun_awt_X11GraphicsConfig
 * Method:    init
 * Signature: (I)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_X11GraphicsConfig_init(
JNIEnv *env, jobject this, jint visualNum, jint screen)
{
    
    AwtGraphicsConfigData *adata = NULL;
    AwtScreenData asd = x11Screens[screen];
    int i, n;

    /* If haven't gotten all of the configs yet, do it now. */
    if (asd.numConfigs == 0) {
	getAllConfigs (env, screen, &asd);
    }

    /* Check the graphicsConfig for this visual */
    for (i = 0; i < asd.numConfigs; i++) {
        AwtGraphicsConfigDataPtr agcPtr = asd.configs[i];
	if (agcPtr->awt_visInfo.visualid == visualNum) {
	   adata = agcPtr;
	   break;
	}
    }

    /* If didn't find the visual, throw an exception... */
    if (adata == (AwtGraphicsConfigData *) NULL) {
        JNU_ThrowIllegalArgumentException(env, "Unknown Visual Specified");
	return;
    }

    /*  adata->awt_cmap initialization has been deferred to 
     *  makeColorModel call 
     */

    JNU_SetLongFieldFromPtr(env, this, x11GraphicsConfigIDs.aData, adata);

}
    

    
/*
 * Class:     sun_awt_X11GraphicsConfig
 * Method:    makeColorModel
 * Signature: ()Ljava/awt/image/ColorModel
 */
JNIEXPORT jobject JNICALL
Java_sun_awt_X11GraphicsConfig_makeColorModel(
JNIEnv *env, jobject this)
{
    AwtGraphicsConfigData *adata;
    jobject colorModel;

    /*
     * If awt is not locked yet, return null since the toolkit is not
     * initialized yet.
     */
    if (awt_lock == 0) {
        return NULL;
    }
    
    AWT_LOCK (); 

    adata = (AwtGraphicsConfigData *) JNU_GetLongFieldAsPtr(env, this, 
					      x11GraphicsConfigIDs.aData);

    /* If colormap entry of adata is NULL, need to create it now */
    if (adata->awt_cmap == (Colormap) NULL) {
	awtJNI_CreateColorData (env, adata, 1);
    }

    /* Make Color Model object for this GraphicsConfiguration */
    colorModel = awtJNI_GetColorModel (env, adata);

    AWT_UNLOCK ();

    return colorModel;
}

    
/*
 * Class:     sun_awt_X11GraphicsConfig
 * Method:    getBounds
 * Signature: ()Ljava/awt/Rectangle
 */
JNIEXPORT jobject JNICALL
Java_sun_awt_X11GraphicsConfig_getBounds(JNIEnv *env, jobject this)
{
    jclass clazz;
    jmethodID mid;
    jobject bounds = NULL;
    AwtGraphicsConfigDataPtr adata;

    adata = (AwtGraphicsConfigDataPtr)
        JNU_GetLongFieldAsPtr(env, this, x11GraphicsConfigIDs.aData);
    
    clazz = (*env)->FindClass(env, "java/awt/Rectangle");
    mid = (*env)->GetMethodID(env, clazz, "<init>", "(IIII)V");
    if (mid != 0) {
        bounds = (*env)->NewObject(env, clazz, mid, 0, 0,
                                   DisplayWidth(awt_display,
                                                adata->awt_visInfo.screen),
                                   DisplayHeight(awt_display,
                                                 adata->awt_visInfo.screen));
        
        if ((*env)->ExceptionOccurred(env)) {
            return 0;
        }
    }
    return bounds;
}

