/*
 * @(#)awt_LoadLibrary.c	1.6 00/05/30
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include <stdio.h>
#include <dlfcn.h>
#include <string.h>
#include <stdlib.h>
#include <jni.h>
#include <jni_util.h>
#include <jvm.h>

#include <sys/param.h>
#include <sys/utsname.h>

#include "awt_Plugin.h"

#ifdef DEBUG
#define VERBOSE_AWT_DEBUG
#endif

static void *awtHandle = NULL;

typedef JNIEXPORT jint JNICALL JNI_OnLoad_type(JavaVM *vm, void *reserved);

JNIEXPORT jint JNICALL
JNI_OnLoad(JavaVM *vm, void *reserved)
{
    Dl_info dlinfo;
    char buf[MAXPATHLEN];
    int len;
    char *p;
    JNI_OnLoad_type *JNI_OnLoad_ptr;
    int motifVersion = 1;
    struct utsname name; 
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(vm, JNI_VERSION_1_2);
    void *v;


    /* Has a Motif library been loaded already, e.g. by an application
     * embedding java?   Netscape plugin?
     */

    v = dlsym(RTLD_DEFAULT, "vendorShellWidgetClass");
    if (v != NULL && dladdr(v, &dlinfo)) {

	/*
	 * If we are picking up the vendorShellWigetClass from libXt
	 * instead of libXm (it exists in both), then we are very stuck.
	 * Abort.
	 */
	if (strstr(dlinfo.dli_fname, "libXt.so") != NULL) {
	    fprintf(stderr, "\nRuntime link error - it appears that "
		    "libXt got loaded before libXm,\n"
		    "which is not allowed.\n");
	    JNU_ThrowByName(env, "java/lang/InternalError",
			    "libXt loaded before libXm");
	    return JNI_VERSION_1_2;
	}

	if (strstr(dlinfo.dli_fname, "libXm.so.3") != NULL) {
#ifdef VERBOSE_AWT_DEBUG
	    fprintf(stderr, "Motif 1.2 detected, using that.\n");
#endif
	    motifVersion = 1;
	}
	else if (strstr(dlinfo.dli_fname, "libXm.so.4") != NULL) {
#ifdef VERBOSE_AWT_DEBUG
	    fprintf(stderr, "Motif 2.1 detected, using that.\n");
#endif
	    motifVersion = 2;
	}
    }

    /* Determine desired Motif Version, and set appropriate properties
     * to load the correct version of libmawt.so
     */  
    else {  
        uname(&name); 
    
        if ((strcmp(name.release, "5.5.1") == 0) ||
           (strcmp(name.release, "5.6") == 0)) {
#ifdef VERBOSE_AWT_DEBUG
        fprintf(stderr, "default to Motif 1.2, os is: %s\n",name.release);
#endif
            motifVersion = 1;
        } else if ((strcmp(name.release, "5.7") == 0) ||
           (strcmp(name.release, "5.8") == 0)) {
#ifdef VERBOSE_AWT_DEBUG
            fprintf(stderr, "default to Motif 2.1, os is: %s\n",name.release);
#endif
            motifVersion = 2;
        } else {
#ifdef VERBOSE_AWT_DEBUG
            fprintf(stderr, "default to Motif 2.1, unrecognized os: %s\n",
                    name.release);
#endif
            motifVersion = 2;
        }
        if (getenv("_JAVA_AWT_USE_MOTIF_1_2")) {
#ifdef VERBOSE_AWT_DEBUG
            fprintf(stderr,"_JAVA_AWT_USE_MOTIF_1_2 is set, using Motif 1.2\n");
#endif
            motifVersion = 1;
        } else if (getenv("_JAVA_AWT_USE_MOTIF_2_1")) {
#ifdef VERBOSE_AWT_DEBUG
            fprintf(stderr,"_JAVA_AWT_USE_MOTIF_2_1 is set, using Motif 2.1\n");
#endif
            motifVersion = 2;
        }
 
    }
 
    /* Get address if this library and the directory containing it. */
    dladdr((void *)JNI_OnLoad, &dlinfo);
    realpath((char *)dlinfo.dli_fname, buf);
    len = strlen(buf);
    p = strrchr(buf, '/');
    strcpy(p, (motifVersion != 1) ? "/motif21/libmawt" : "/motif12/libmawt");
#ifdef DEBUG
    strcat(p, "_g");
#endif
    strcat(p, ".so");

    JNU_CallStaticMethodByName(env, NULL, "java/lang/System", "load",
                               "(Ljava/lang/String;)V",
                               JNU_NewStringPlatform(env, buf));

    awtHandle = dlopen(buf, RTLD_LAZY);

    return JNI_VERSION_1_2;
}


/*
 * This entry point must remain in libawt.so as part of a contract
 * with the CDE variant of Java Media Framework. (sdtjmplay)
 * Reflect this call over to the correct libmawt.so.
 */

typedef JNIEXPORT void JNICALL  XsessionWMcommand_type
(JNIEnv *env, jobject this, jobject frame, jstring jcommand);

JNIEXPORT void JNICALL  Java_sun_awt_motif_XsessionWMcommand(JNIEnv *env,
                                                             jobject this,
                                                             jobject frame,
                                                             jstring jcommand)
{
    static XsessionWMcommand_type *XsessionWMcommand = NULL;

    if (XsessionWMcommand == NULL && awtHandle == NULL) {
	return;
    }

    XsessionWMcommand = (XsessionWMcommand_type *)
	dlsym(awtHandle, "Java_sun_awt_motif_XsessionWMcommand");

    if (XsessionWMcommand == NULL)
	return;

    (*XsessionWMcommand)(env, this, frame, jcommand);
}


#define REFLECT_VOID_FUNCTION(name, arglist, paramlist)			\
typedef name##_type arglist;						\
void name arglist							\
{									\
    static name##_type *name##_ptr = NULL;				\
    if (name##_ptr == NULL && awtHandle == NULL) {			\
	return;								\
    }									\
    name##_ptr = (name##_type *)					\
	dlsym(awtHandle, #name);					\
    if (name##_ptr == NULL) {						\
	return;								\
    }									\
    (*name##_ptr)paramlist;						\
}

#define REFLECT_FUNCTION(return_type, name, arglist, paramlist)		\
typedef return_type name##_type arglist;				\
return_type name arglist						\
{									\
    static name##_type *name##_ptr = NULL;				\
    if (name##_ptr == NULL && awtHandle == NULL) {			\
	return NULL;							\
    }									\
    name##_ptr = (name##_type *)					\
	dlsym(awtHandle, #name);					\
    if (name##_ptr == NULL) {						\
	return NULL;							\
    }									\
    return (*name##_ptr)paramlist;					\
}


/*
 * These entry point must remain in libawt.so ***for Java Plugin ONLY***
 * Reflect this call over to the correct libmawt.so.
 */

REFLECT_VOID_FUNCTION(getAwtLockFunctions,
		      (void (**AwtLock)(JNIEnv *), void (**AwtUnlock)(JNIEnv *),
		       void (**AwtNoFlushUnlock)(JNIEnv *), void *reserved), 
		      (AwtLock, AwtUnlock, AwtNoFlushUnlock, reserved))

REFLECT_VOID_FUNCTION(getAwtData,
		      (int *awt_depth, Colormap *awt_cmap, Visual **awt_visual,
		       int *awt_num_colors, void *pReserved),
		      (awt_depth, awt_cmap, awt_visual,
		       awt_num_colors, pReserved))

REFLECT_FUNCTION(Display *, getAwtDisplay, (void), ())
