/*
 * @(#)awt_Menu.c	1.45 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "awt_p.h"
#include "color.h"
#include "java_awt_Menu.h"
#include "sun_awt_motif_MMenuPeer.h"
#include "java_awt_MenuBar.h"
#include "sun_awt_motif_MMenuBarPeer.h"

#include "awt_MenuBar.h"
#include "awt_MenuComponent.h"
#include "awt_MenuItem.h"
#include "awt_Menu.h"

#include "multi_font.h"
#include <jni.h>
#include <jni_util.h>

extern struct MenuComponentIDs menuComponentIDs;
extern struct MenuItemIDs menuItemIDs;
extern struct MMenuItemPeerIDs mMenuItemPeerIDs;
extern struct MMenuBarPeerIDs mMenuBarPeerIDs;

struct MenuIDs menuIDs;

/*
 * Class:     java_awt_Menu
 * Method:    initIDs
 * Signature: ()V
 */

/* This function gets called from the static initializer for
   Menu.java to initialize the fieldIDs for fields that may
   be accessed from C */

JNIEXPORT void JNICALL Java_java_awt_Menu_initIDs
  (JNIEnv *env, jclass cls)
{
    menuIDs.tearOff = (*env)->GetFieldID(env, cls, "tearOff", "Z");
    menuIDs.isHelpMenu = (*env)->GetFieldID(env, cls, "isHelpMenu", "Z");
}

/*
 * Fix for Bug Traq 4251941 - segfault after double tear-off and close
 * Removes the lost callback from menu item on tear-off control re-creation.
 * Only for internal use, to be used from awtTearOffActivatedCallback
 */
static void awtTearOffShellDestroy(Widget widget, XtPointer closure, XtPointer data) {
    if (widget != NULL ) {
	XtSetKeyboardFocus(widget, NULL);
    }
}

/*
 * Fix for Bug Traq 4251941 - segfault after double tear-off and close
 * This callback is added to menu after the creation. 
 * It adds the destroy callback awtTearOffShellDestroy to remove the lost focus callback on destroy
 */
static void awtTearOffActivatedCallback(Widget widget, XtPointer closure, XtPointer data) {
    Widget shell;
    shell = XtParent(widget);
    if (shell != NULL && XtClass(shell) == transientShellWidgetClass) {
	XtAddCallback(shell, XtNdestroyCallback, awtTearOffShellDestroy, widget);
    }
}

/*
 * this is a MMenuPeer instance
 */
static void
awtJNI_CreateMenu(JNIEnv * env, jobject this, Widget menuParent)
{
    int argc;
#define MAX_ARGC 10
    Arg args[MAX_ARGC];
    char *ctitle = NULL;
    struct MenuData *mdata;
    struct FontData *fdata;
    Pixel bg;
    Pixel fg;
    XmFontList fontlist = NULL;
    Widget tearOff;
    XmString mfstr = NULL;
    XmString str = NULL;
    jobject target;
    jobject targetFont;
    jobject label;
    jobject font;
    jboolean IsMultiFont;
    jboolean isTearOff;

    /* perhaps this is unncessary, if awtJNI_CreateMenu is only called
     * from a native method.
     */
    if ((*env)->PushLocalFrame(env, 16) < 0) {
        return;
    }

    fdata = NULL;

    target = (*env)->GetObjectField(env, this, mMenuItemPeerIDs.target);
    if (JNU_IsNull(env, target)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
	(*env)->PopLocalFrame(env, 0);
        return;
    }
    font = JNU_CallMethodByName(env, NULL, target, "getFont_NoClientCode",
				"()Ljava/awt/Font;").l;

    mdata = ZALLOC(MenuData);
    JNU_SetLongFieldFromPtr(env, this, mMenuItemPeerIDs.pData, mdata);
    if (mdata == 0) {
        JNU_ThrowOutOfMemoryError(env, "OutOfMemoryError");
	(*env)->PopLocalFrame(env, 0);
        return;
    }
    targetFont = (*env)->GetObjectField(env, target, menuComponentIDs.font);
    if (!JNU_IsNull(env, targetFont) &&
        (fdata = awtJNI_GetFontData(env, targetFont, 0)) != 0) {
        IsMultiFont = awtJNI_IsMultiFont(env, targetFont);
    } else {
        IsMultiFont = awtJNI_IsMultiFont(env, font);
    }

    label = (*env)->GetObjectField(env, target, menuItemIDs.label);
    if (JNU_IsNull(env, label)) {
        mfstr = XmStringCreateLocalized("");
        ctitle = "";
    } else {
        if (IsMultiFont) {
            mfstr = awtJNI_MakeMultiFontString(env, label, font);
        } else {
            ctitle = (char *) JNU_GetStringPlatformChars(env, label, NULL);
        }
    }

    XtVaGetValues(menuParent, XmNbackground, &bg, NULL);
    XtVaGetValues(menuParent, XmNforeground, &fg, NULL);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, bg);
    argc++;
    XtSetArg(args[argc], XmNforeground, fg);
    argc++;

    XtSetArg(args[argc], XmNlabelFontList,   getMotifFontList());
    argc++;
    XtSetArg(args[argc], XmNbuttonFontList,  getMotifFontList());
    argc++;

    isTearOff = (*env)->GetBooleanField(env, target, menuIDs.tearOff);

    if (isTearOff) {
        XtSetArg(args[argc], XmNtearOffModel, XmTEAR_OFF_ENABLED);
        argc++;
    }

    if (IsMultiFont) {
        DASSERT(!(argc > MAX_ARGC));
        mdata->itemData.comp.widget = XmCreatePulldownMenu(menuParent,
                                                           "",
                                                           args,
                                                           argc);
    } else {
        DASSERT(!(argc > MAX_ARGC));
        mdata->itemData.comp.widget = XmCreatePulldownMenu(menuParent,
                                                           ctitle,
                                                           args,
                                                           argc);
    }

    if (isTearOff) {
        tearOff = XmGetTearOffControl(mdata->itemData.comp.widget);
        XtVaSetValues(tearOff,
                      XmNbackground, bg,
                      XmNforeground, fg,
                      XmNhighlightColor, fg,
                      NULL);
	XtAddCallback(mdata->itemData.comp.widget, XmNtearOffMenuActivateCallback,
		      awtTearOffActivatedCallback, NULL);
    }
    argc = 0;
    XtSetArg(args[argc], XmNsubMenuId, mdata->itemData.comp.widget);
    argc++;

    if (IsMultiFont) {
        XtSetArg(args[argc], XmNlabelString, mfstr);
    } else {
        str = XmStringCreate(ctitle, XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[argc], XmNlabelString, str);
    }
    argc++;
    XtSetArg(args[argc], XmNbackground, bg);
    argc++;
    XtSetArg(args[argc], XmNforeground, fg);
    argc++;

    if (!JNU_IsNull(env, targetFont) && (fdata != 0)) {
        if (IsMultiFont) {
            fontlist = awtJNI_GetFontList(env, targetFont);
        } else {
            fontlist = XmFontListCreate(fdata->xfont, "labelFont");
        }
        XtSetArg(args[argc], XmNfontList, fontlist);
        argc++;
    } else {
        if (IsMultiFont) {
            fontlist = awtJNI_GetFontList(env, font);
            XtSetArg(args[argc], XmNfontList, fontlist);
            argc++;
        }
    }

    if (IsMultiFont) {
        DASSERT(!(argc > MAX_ARGC));
        mdata->comp.widget = XmCreateCascadeButton(menuParent, "", args, argc);
    } else {
        DASSERT(!(argc > MAX_ARGC));
        mdata->comp.widget = XmCreateCascadeButton(menuParent, ctitle, args, argc);
    }

    if ((*env)->GetBooleanField(env, target, menuIDs.isHelpMenu)) {
        XtVaSetValues(menuParent,
                      XmNmenuHelpWidget, mdata->comp.widget,
                      NULL);
    }

    /*
     * Free resources
     */
    if (!JNU_IsNull(env, targetFont)) {
        XmFontListFree(fontlist);
    }

    if (mfstr != NULL) {
      XmStringFree(mfstr); 
      mfstr = NULL;
    }

    if (str) {
      XmStringFree(str);
      str = NULL;
    }

    XtManageChild(mdata->comp.widget);
    XtSetSensitive(mdata->comp.widget,
                   (*env)->GetBooleanField(env, target, menuItemIDs.enabled) ?
		   True : False);

    if (ctitle != NULL && ctitle != "") {
        JNU_ReleaseStringPlatformChars(env, label, (const char *) ctitle);
    }
    (*env)->PopLocalFrame(env, 0);
}


/*
 * Class:     sun_awt_motif_MMenuPeer
 * Method:    createMenu
 * Signature: (Lsun/awt/motif/MMenuBarPeer;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MMenuPeer_createMenu
  (JNIEnv *env, jobject this, jobject parent)
{
    struct ComponentData *mbdata;
    AwtGraphicsConfigDataPtr adata;

    AWT_LOCK();
    if (JNU_IsNull(env, parent)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    mbdata = (struct ComponentData *)
	JNU_GetLongFieldAsPtr(env, parent, mMenuBarPeerIDs.pData);
    if (mbdata == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    awtJNI_CreateMenu(env, this, mbdata->widget);

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MMenuPeer
 * Method:    createSubMenu
 * Signature: (Lsun/awt/motif/MMenuPeer;)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MMenuPeer_createSubMenu
(JNIEnv *env, jobject this, jobject parent)
{
    struct MenuData *mpdata;
    AwtGraphicsConfigDataPtr adata;

    AWT_LOCK();
    if (JNU_IsNull(env, parent)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }
    mpdata = (struct MenuData *)
	JNU_GetLongFieldAsPtr(env, parent, mMenuItemPeerIDs.pData);
    if (mpdata == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    awtJNI_CreateMenu(env, this, mpdata->itemData.comp.widget);

    AWT_UNLOCK();
}

/*
 * Class:     sun_awt_motif_MMenuPeer
 * Method:    pDispose
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_MMenuPeer_pDispose
  (JNIEnv *env, jobject this)
{
    struct MenuData *mdata;

    AWT_LOCK();

    mdata = (struct MenuData *)
	JNU_GetLongFieldAsPtr(env, this, mMenuItemPeerIDs.pData);
    if (mdata == 0) {
        AWT_UNLOCK();
        return;
    }
    XtUnmanageChild(mdata->comp.widget);
    awt_util_consumeAllXEvents(mdata->itemData.comp.widget);
    awt_util_consumeAllXEvents(mdata->comp.widget);
    XtDestroyWidget(mdata->itemData.comp.widget);
    XtDestroyWidget(mdata->comp.widget);
    free((void *) mdata);
    AWT_UNLOCK();
}
