/*
 * @(#)awt_Selection.c	1.39 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_p.h"
#include "awt_DataTransferer.h"
#include "java_awt_datatransfer_Transferable.h"
#include "java_awt_datatransfer_DataFlavor.h"
#include "sun_awt_motif_X11Selection.h"
#include "sun_awt_motif_X11Clipboard.h"
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <sys/utsname.h>

#include <jni.h>
#include <jni_util.h>

static Atom TARGETS;
static Atom COMPOUND_TEXT;

static Widget selection_w = NULL;

Boolean selectionProcessed = FALSE;
Boolean gotData = FALSE;


/* fieldIDs for X11Selection fields that may be accessed from C */
static struct X11SelectionIDs {
    jfieldID holder;
    jfieldID atom;
    jfieldID contents;
    jfieldID targetArray;
    jfieldID data;
    jfieldID selections;
    jfieldID format;
} x11SelectionIDs;

static jclass selectionClazz;
static jmethodID getSelectionsArray;
static jmethodID getFormatsForFlavors;
static jmethodID convertData;

/*
 * Class:     sun_awt_motif_MScrollPanePeer
 * Method:    initIDs
 * Signature: ()V
 */

/* This function gets called from the static initializer for
   X11Selection.java to initialize the fieldIDs for fields that may
   be accessed from C */

JNIEXPORT void JNICALL Java_sun_awt_motif_X11Selection_initIDs
  (JNIEnv *env, jclass cls)
{
    x11SelectionIDs.holder = (*env)->
      GetFieldID(env, cls, "holder","Lsun/awt/motif/X11SelectionHolder;");
    x11SelectionIDs.atom = (*env)->GetFieldID(env, cls, "atom", "I");
    x11SelectionIDs.contents = (*env)->
      GetFieldID(env, cls, "contents","Ljava/awt/datatransfer/Transferable;");
    x11SelectionIDs.targetArray = (*env)->
      GetFieldID(env, cls, "targetArray","[I");
    x11SelectionIDs.data = (*env)->
      GetFieldID(env, cls, "nativeData","Ljava/lang/Object;");
    x11SelectionIDs.selections = (*env)->
      GetStaticFieldID(env, cls, "selections", "Ljava/util/Vector;");
    x11SelectionIDs.format = (*env)->GetFieldID(env, cls, "format", "I");
}

/*
 * Class:     sun_awt_motif_X11Selection
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Selection_init
  (JNIEnv *env, jclass this) 
{
    AWT_LOCK();

    selection_w = XtAppCreateShell("AWTSelection", "XApplication",
                                   topLevelShellWidgetClass,
                                   awt_display,
                                   NULL, 0);

    XtSetMappedWhenManaged(selection_w, False);
    XtRealizeWidget(selection_w);

    TARGETS = XInternAtom(awt_display, "TARGETS", False);
    COMPOUND_TEXT = XInternAtom(awt_display, "COMPOUND_TEXT", False);

    selectionClazz = (*env)->NewGlobalRef(env, this); /* never freed! */

    if (JNU_IsNull(env, selectionClazz)) {
	AWT_UNLOCK();
	return;
    }

    getSelectionsArray = (*env)->GetStaticMethodID(env,
						   selectionClazz,
						   "getSelectionsArray",
						   "()[Ljava/lang/Object;"
						   );

    if (JNU_IsNull(env, getSelectionsArray)) {
	(*env)->ThrowNew(env,
			 (*env)->FindClass(env, "java/lang/NoSuchMethodException"),
			 "getSelectionsArray"
			 );
    }

    getFormatsForFlavors = (*env)->GetMethodID(env,
					       selectionClazz,
					       "getFormatsForFlavors",
					       "()[I"
					       );

    if (JNU_IsNull(env, getFormatsForFlavors)) {
	(*env)->ThrowNew(env,
			 (*env)->FindClass(env, "java/lang/NoSuchMethodException"),
			 "getFormatsForFlavors"
			 );  
    }

    convertData = (*env)->GetMethodID(env,
				  selectionClazz,
				  "convertData",
				  "(I)[B"
				  );

    if (JNU_IsNull(env, convertData)) {
	(*env)->ThrowNew(env,
			 (*env)->FindClass(env, 
					   "java/lang/NoSuchMethodException"),
			 "convertData"
			 );  
    }

    AWT_UNLOCK();
}

static jobject 
getX11Selection(JNIEnv * env, Atom atom)
{
    int i;

    jobjectArray selections;
    int selectionCount;
    jobject selection;

    selections = (jobjectArray) (*env)->CallStaticObjectMethod
	(env, selectionClazz, getSelectionsArray);

    if (JNU_IsNull(env, selections)) {
	return NULL;
    }

    selectionCount = (*env)->GetArrayLength(env, selections);

    for (i = 0; i < selectionCount; i++) {
	selection = (*env)->GetObjectArrayElement(env, selections, (jsize) i);
	if (JNU_IsNull(env, selection) || 
	    !JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    /* Something went wrong just exit gracefully */
	    break;
	}
        if ((*env)->GetIntField(env, selection, x11SelectionIDs.atom)
	    == atom) {
	    return selection;
        }
    }

    return NULL;
}

Boolean 
awtJNI_isSelectionOwner(JNIEnv * env, char *sel_str)
{
    Atom selection;
    jobject x11sel;

    selection = XInternAtom(awt_display, sel_str, False);

    x11sel = getX11Selection(env, selection);
    if (!JNU_IsNull(env, x11sel)) {
        jobject holder;

        holder = (*env)->GetObjectField(env, x11sel, x11SelectionIDs.holder);
        if (!JNU_IsNull(env, holder)) {
            return TRUE;
        }
    }
    return FALSE;
}

static void losingSelectionOwnership(Widget w, Atom * selection);

void 
awtJNI_notifySelectionLost(JNIEnv * env, char *sel_str)
{
    Atom selection;

    selection = XInternAtom(awt_display, sel_str, False);
    losingSelectionOwnership(NULL, &selection);
}

/*
 * Single routine to convert to target FILE_NAME or _DT_FILENAME
 */
Boolean 
convertFileType(jbyteArray data, Atom * type, XtPointer * value, 
		unsigned long *length, int *format)
{
    /*
     * Convert the internal representation to an File Name.
     * The data passed is an array of
     * null separated bytes. Each series of bytes is a string
     * that is then converted to an XString which are then put
     * into an XStringList and put into an XTextProperty for
     * usage in other programs.
     *
     * It would be desireable to have dataConvert to this conversion
     * but it isn't possible to return a byte array that represents
     * the XTextProperty.
     */

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jboolean	  isCopy=JNI_FALSE;
    XTextProperty tp;
    int	          len;
    int		  strings=0;
    int		  i;
    char**	  stringList;
    Status	  s;
    jbyte	  *bytes;
    jbyte	  *start;
    jstring	  string;
    int		  slen;
    char*	  utf;

    /* convert the data to an Array of Byte Elements */
    bytes = (*env)->GetByteArrayElements(env, data, &isCopy);
    if (JNU_IsNull(env, bytes) ||
	!JNU_IsNull(env, (*env)->ExceptionOccurred(env)))
    {
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);

	(*env)->PopLocalFrame(env, 0);

	return False;
    }

    /* Get the length of the area */
    len = (*env)->GetArrayLength(env, data);
    if (len == 0) {
	(*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);

	(*env)->PopLocalFrame(env, 0);

	return False;
    }

    /* 
     * determine the number of lists. The byteArray is null separated list of
     * strings.
     */
    for (i = 0; i < len; i++) {
	if (bytes[i] == '\0') {
	    strings += 1;
	}
    }

    /* Allocate an X11 string list */
    stringList = (char **)XtCalloc(strings, sizeof(char *));
    if (stringList == (char**)NULL) {
	(*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);

	(*env)->PopLocalFrame(env, 0);

	return False;
    }

    for (i = 0; i < strings; i++) {
	if (i == 0) {
	    start = bytes;
	} else {
	    start = &bytes[slen];
	}

	/* 
	 * if start is a NULL we're at the end of the list
	 * We'll just a have null entry on the end of the list
	 */
 
	if (start[0] == '\0') {
	    stringList[i] = NULL;
	    continue;
	}
	/* Determine the start */
	/*   create the String */
	string = (*env)->NewStringUTF(env, (char *)start);
	if (JNU_IsNull(env, string) ||
	    !JNU_IsNull(env, (*env)->ExceptionOccurred(env)))
	{
	    int j;

	    (*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);

	    for (j = 0; j < i; j++) {
		XtFree((void *)stringList[j]);
	    }

	    XtFree((void*)stringList);

	    (*env)->PopLocalFrame(env, 0);

	    return False;
	}

	slen = (*env)->GetStringUTFLength(env, string) + 1;
	utf  = (char *)(*env)->GetStringUTFChars(env, string, &isCopy);

	stringList[i] = (char*)XtCalloc(slen, sizeof(char));

	if (stringList[i] == (char *)NULL) {
	    int j;

	    for (j = 0; j < i; j++) {
		XtFree((void *)stringList[j]);
	    }

	    (*env)->PopLocalFrame(env, 0);

	    return False;
	}

	memcpy((void *)stringList[i], (const void*)utf, slen);

	(*env)->ReleaseStringUTFChars(env, string, (const char *)utf);
    }
	
    (*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
    s = XStringListToTextProperty(stringList, strings, &tp);

    /* free the strings that were created */
    for (i = 0; i < strings; i++) {
	if (stringList[i] != NULL) {
	    XtFree((void*)stringList[i]);
	}
    }

    XtFree((void*)stringList);

    if (s == 0) return False;

    *value = (XtPointer)XtCalloc(tp.nitems, sizeof(char));

    if (*value == (XtPointer)NULL) {
	XFree((void*)tp.value);

	(*env)->PopLocalFrame(env, 0);

	return False;
    }

    memcpy((void *)(*value), (const void *)tp.value, tp.nitems);

    XFree((void*)tp.value);

    *length = tp.nitems;
    *type   = tp.encoding;
    *format = tp.format;
    return True;
}

static Boolean 
provideSelectionData(Widget w, Atom * selection, Atom * target, Atom * type,
                     XtPointer * value, unsigned long *length, int *format)
{
    jobject this;
    jobject contents = 0;
    jobject flavor = NULL;
    jobjectArray flavors;
    jint *targets;
    int i, j, num_flavors, count;
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jclass cls;
    jfieldID fieldID;
    jboolean	  isCopy=JNI_FALSE;

    if ((*env)->PushLocalFrame(env, 16) < 0)
        return FALSE;

    this = getX11Selection(env, *selection);
    if (JNU_IsNull(env, this)) {
        (*env)->PopLocalFrame(env, 0);
        return FALSE;
    }

    if (*target == TARGETS) {
	jintArray formats=NULL;
	jsize conunt;

	formats = (jintArray) (*env)->CallObjectMethod
	    (env, this , getFormatsForFlavors);

	if (JNU_IsNull(env, formats)) {
	    (*env)->PopLocalFrame(env, 0);
	    return FALSE;
	}

	count = (*env)->GetArrayLength(env, formats);
        if (count == 0) {
	    (*env)->PopLocalFrame(env, 0);
            return FALSE;
        }

	targets = (*env)->GetIntArrayElements(env, formats, &isCopy);

        *type = XA_ATOM;
        *format = 32;
	*value = (XtPointer)XtCalloc(count, sizeof(int));
	if (*value == NULL) {
            (*env)->ReleaseIntArrayElements(env, formats, targets, JNI_ABORT);
            (*env)->PopLocalFrame(env, 0);
            return FALSE;
	}
	memcpy((void *)*value, (void *)targets, count * sizeof(int));
	(*env)->ReleaseIntArrayElements(env, formats, targets, JNI_ABORT);
        *length = count;
        (*env)->PopLocalFrame(env, 0);
        return TRUE;
    }

    if (*target == XInternAtom(awt_display, _XA_HOSTNAME, False)) {
	struct utsname name;
	XTextProperty  tp;

	uname(&name);

	XStringListToTextProperty((char **)&name.nodename, 1, &tp);

	*value = (XtPointer)XtCalloc(tp.nitems, sizeof(char));

	memcpy((void *)*value, (const void *)tp.value, tp.nitems);

	XFree((void *)tp.value);

	*type   = tp.encoding;
	*length = tp.nitems + 1;
	*format = tp.format;
    } else if (*target == XInternAtom(awt_display, _XA_FILENAME, False) ||
	       *target == XInternAtom(awt_display, _DT_FILENAME, False)) {

	/*
	 * Convert the internal representation to an File Name.
	 * The data returned from dataConvert is a an array of
	 * null separated bytes. Each series of bytes is a string
	 * that is then converted to an XString which are then put
	 * into an XStringList and put into an XTextProperty for
	 * usage in other programs.
	 *
	 * It would be desireable to have dataConvert to this conversion
	 * but it isn't possible to return a byte array that represents
	 * the XTextProperty.
	 */

 	jbyteArray    data;
	Boolean	      test = False;

	/* assume forthcoming error */
	*type   = XT_CONVERT_FAIL;
	*value  = (XtPointer)NULL;
	*length = 0L;
	*format = 8;


	/*
	 * Get the File name in the form of null separated byte array
	 * SECURITY: OK to call this on privileged thread - peer is secure
	 */
	data = (jbyteArray) (*env)->CallObjectMethod(env,
						     this,
						     convertData,
						     *target);
						
	/* error test */
	if (JNU_IsNull(env, data) || !JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);

    	    (*env)->PopLocalFrame(env, 0);

	    return False;
	}
	
	test = convertFileType(data, type, value, length, format);
	if (test == False) {
	    return False;
	}
    } else {
	jbyteArray data;
	jbyte *bytes;
	jint len;
	Atom targetAtom = *target;

	/* assume forthcoming error */
	*type   = XT_CONVERT_FAIL; 
	*value  = (XtPointer)NULL;
	*length = 0L;
	*format = 8;

	/*
	 * To keep backward compatibility with Kestrel Compound Text support 
	 * we respond to COMPOUND_TEXT target with the text in the default
	 * encoding converted to COMPOUND_TEXT encoding.
	 */
	if (*target == COMPOUND_TEXT) {
	    /* Pass TEXT target to Java to get the text in the default
	       encoding. */
	    targetAtom = XInternAtom(awt_display, "TEXT", False);
	}

	/*
 	 * we get a Buffer object from the X11Selection, then we
 	 * process the Buffer, copying it into a single block of contiguous
	 * memory that we return to Xt.
 	 * 
	 */

	AWT_UNLOCK();

        /* SECURITY: OK to call this on privileged thread - peer is secure */
	data = (*env)->CallObjectMethod(env,
					this,
					convertData,
					targetAtom);
						
	AWT_LOCK();
	
	if (JNU_IsNull(env, data) || 
	    !JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);

    	    (*env)->PopLocalFrame(env, 0);

	    return False;
	}

	len = (*env)->GetArrayLength(env, data);
	if (len == 0) {
	    *type = *target;
	    *format = 8;
	    (*env)->PopLocalFrame(env, 0);

	    return True;
	}

	bytes = (*env)->GetByteArrayElements(env, data, &isCopy);

	if (*target == COMPOUND_TEXT) {
	    XTextProperty text_prop_return;

	    if (XmbTextListToTextProperty(awt_display, &bytes, 1,
					  XCompoundTextStyle,
					  &text_prop_return) == Success) {
		
		*type = text_prop_return.encoding;
		*format = 8;
		*value = (XtPointer) (text_prop_return.value);
		*length = strlen((char *) text_prop_return.value);
		(*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
		(*env)->PopLocalFrame(env, NULL);
		return True;
	    } else {
		(*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
		(*env)->PopLocalFrame(env, NULL);
		return False;
	    }
	}

	*value = (XtPointer)XtCalloc(len, sizeof(char));
	if (*value == NULL) {
            (*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
            (*env)->PopLocalFrame(env, 0);
            return False;
	}
	memcpy((void *)*value, (void *)bytes, len);
	(*env)->ReleaseByteArrayElements(env, data, bytes, JNI_ABORT);
	*type   = *target;
	*length = len;
	*format = 8;
    }

    (*env)->PopLocalFrame(env, 0);

    return True;
}

static char *data;

static void 
transferDone(Widget w, Atom * selection, Atom * target)
{
    free(data);
}

static void 
losingSelectionOwnership(Widget w, Atom * selection)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject this = getX11Selection(env, *selection);

    /*
     * NOTE: this code makes sense until we support multiple system clipboards.
     * PRIMARY selection should be disowned as we clear its contents on the
     * Java side in response to losing CLIPBOARD selection ownership.
     */
    if (*selection == XInternAtom(awt_display, "CLIPBOARD", False)) {
        XtDisownSelection(w, XInternAtom(awt_display, "PRIMARY", False),
                          awt_util_getCurrentServerTime(w));
    }			  

    /* 
     * SECURITY: OK to call this on privileged thread - peer does
     *         not call into client code
     */
    JNU_CallMethodByName(env, NULL, this, "lostSelectionOwnership", "()V");
    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
}

static void 
getSelectionTargets(Widget w, XtPointer client_data, Atom * selection,
		    Atom * type, XtPointer value, unsigned long *length,
		    int *format)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject this = (jobject) client_data;
    jintArray targetArray;
    int count = 0;

    if ((*env)->EnsureLocalCapacity(env, 1) < 0) {
        selectionProcessed = TRUE;
        return;
    }

    /*
     * It is highly unlikely that TARGETS will ever be passed even though that
     * was what was requested. However, XA_ATOM ("ATOM") is likely. 
     * Actually they are the same so treat them as such. See XToolKit 
     *  Intrinsic Manual on XtSelectionCallbackProc for more details on type.
     */
    if (*type == TARGETS || *type == XA_ATOM) {
	int i;
        Atom *targets = (Atom *) value;

        /* Get rid of zero atoms if there are any. */
        for (i = 0; i < *length; i++) {
            if (targets[i] != 0) {
                count++;
            }
        }

        if (count > 0) {
            int j = 0;
            jint* checkedTargets = calloc(count, sizeof(int));

            if (checkedTargets == NULL) {
                JNU_ThrowOutOfMemoryError(env, "");
                (*env)->ExceptionDescribe(env);
                (*env)->ExceptionClear(env);
                selectionProcessed = TRUE;
                return;
            } else {
                for (i = 0; i < *length; i++) {
                    if (targets[i] != 0) {
                        checkedTargets[j++] = targets[i];
                    }
                }

                DASSERT(j == count);
                targetArray = (*env)->NewIntArray(env, count);
                if (JNU_IsNull(env, targetArray)) {
                    selectionProcessed = TRUE;
                    return;
                }
                (*env)->SetIntArrayRegion(env, targetArray, 0, count, 
                                          checkedTargets);
            }
        } else {
            targetArray = (*env)->NewIntArray(env, 0);
            if (JNU_IsNull(env, targetArray)) {
                selectionProcessed = TRUE;
                return;
            }
        }
    } else {
        targetArray = (*env)->NewIntArray(env, 0);
        if (JNU_IsNull(env, targetArray)) {
            selectionProcessed = TRUE;
            return;
        }
    }

    DASSERT(!JNU_IsNull(env, targetArray));
    (*env)->SetObjectField(env, this,
                           x11SelectionIDs.targetArray, targetArray);
    (*env)->DeleteLocalRef(env, targetArray);
    selectionProcessed = TRUE;
}

static void 
getSelectionData(Widget w, XtPointer client_data, Atom * selection,
		 Atom * type, XtPointer value, unsigned long *length,
		 int *format)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject this = (jobject) client_data;
    int count = *length;
    long *atoms;
    int i;
    jstring jstr;
  
    if (*type != None && *length > 0) {
        jbyteArray arry;
  
	if ((*env)->PushLocalFrame(env, 1) < 0) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
	    selectionProcessed = TRUE;
            return;
	}
	
        arry = (*env)->NewByteArray(env, *length);
	
	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
	    (*env)->PopLocalFrame(env, 0);
	    selectionProcessed = TRUE;
	    return;
	}
	
	(*env)->SetByteArrayRegion(env, arry, 0, *length, (jbyte*)value);
	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
	    (*env)->PopLocalFrame(env, 0);
	    selectionProcessed = TRUE;
	    return;
	}

	arry = (*env)->NewGlobalRef(env, arry);

	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
	    (*env)->PopLocalFrame(env, 0);
	    selectionProcessed = TRUE;
	    return;
	}

        (*env)->SetObjectField(env, this, x11SelectionIDs.data, arry);
        (*env)->SetIntField(env, this, x11SelectionIDs.format, (jint) *type);
        (*env)->PopLocalFrame(env, NULL);
        gotData = TRUE;
    }
    selectionProcessed = TRUE;
}

static int 
WaitForSelectionEvent(void *data)
{
    return selectionProcessed;
}

/*
 * Class:     sun_awt_motif_X11Selection
 * Method:    pGetSelectionOwnership
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL
Java_sun_awt_motif_X11Selection_pGetSelectionOwnership
  (JNIEnv *env, jobject this) 
{
    Atom selection;
    Boolean gotit;

    AWT_LOCK();

    selection = (Atom) (*env)->GetIntField(env, this, x11SelectionIDs.atom);

    gotit = XtOwnSelection(selection_w, selection,
			   awt_util_getCurrentServerTime(selection_w),
                           provideSelectionData, losingSelectionOwnership,
			   transferDone);

    AWT_UNLOCK();
    return (gotit ? 1 : 0);
}

/*
 * Class:     sun_awt_motif_X11Selection
 * Method:    pGetTransferTargets
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Selection_pGetTransferTargets
  (JNIEnv *env, jobject this) 
{
    Atom selection;
    jobject gRef;

    AWT_LOCK();

    selection = (Atom) (*env)->GetIntField(env, this, x11SelectionIDs.atom);
    selectionProcessed = FALSE;

    /* We may have callbacks from another thread? Better make */
    /* a global ref to be safe. */
    gRef = (*env)->NewGlobalRef(env, this);

    XtGetSelectionValue(selection_w, selection, TARGETS, getSelectionTargets,
                        (XtPointer) gRef,
			awt_util_getCurrentServerTime(selection_w));

    AWT_FLUSH_UNLOCK();
    awt_MToolkit_modalWait(WaitForSelectionEvent, NULL);
    (*env)->DeleteGlobalRef(env, gRef);
}

/*
 * Class:     sun_awt_motif_X11Selection
 * Method:    pGetTransferData
 * Signature: ([I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_motif_X11Selection_pGetTransferData
  (JNIEnv *env, jobject this, jintArray targets)
{
    Atom selection;
    jint *atoms;
    jobject gRef;
    int count;
    int i;

    AWT_LOCK();

    selection = (Atom) (*env)->GetIntField(env, this, x11SelectionIDs.atom);
    gotData = FALSE;

    atoms = (*env)->GetIntArrayElements(env, targets, NULL);
    if(atoms == NULL) {
        AWT_UNLOCK();
        return;
    }

    count = (*env)->GetArrayLength(env, targets);

    /* We may have callbacks from another thread? Better make */
    /* a global ref to be safe. */
    gRef = (*env)->NewGlobalRef(env, this);

    for (i = 0; i < count && !gotData; i++) {
        selectionProcessed = FALSE;
        XtGetSelectionValue(selection_w, selection, (Atom) atoms[i], 
			    getSelectionData,
                            (XtPointer) gRef, 
			    awt_util_getCurrentServerTime(selection_w));

        AWT_FLUSH_UNLOCK();
        awt_MToolkit_modalWait(WaitForSelectionEvent, NULL);
        AWT_LOCK();

    }

    (*env)->DeleteGlobalRef(env, gRef);
    (*env)->ReleaseIntArrayElements(env, targets, atoms, JNI_ABORT);

    AWT_UNLOCK();
}
