/*
 * @(#)awt_XmDnD.c	1.53 01/02/09
 *
 *  Copyright 2000-2001 Sun Microsystems, Inc. All rights reserved.
 *  Copyright 2000-2001 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include <stdio.h>
#include <string.h>

#include "jvm.h"
#include "jni.h"
#include "jni_util.h"

#include "awt_DataTransferer.h"
#include "awt_XmDnD.h"

#include "awt_p.h"

#include <java_awt_dnd_DnDConstants.h>
#include <sun_awt_dnd_SunDropTargetContextPeer.h>
#include <sun_awt_motif_AWTLockAccess.h>
#include <sun_awt_motif_MComponentPeer.h>
#include <sun_awt_motif_MDragSourceContextPeer.h>
#include <sun_awt_motif_MDropTargetContextPeer.h>

#include <errno.h>

#include <X11/cursorfont.h>
#include <X11/Xutil.h>

#include <sys/utsname.h>

#include "awt_Component.h"
#include "awt_Cursor.h"
#include "awt_AWTEvent.h"

Boolean convertFileType(jbyteArray data, Atom *type, XtPointer *value,
			unsigned long *length, int *format);

extern struct MComponentPeerIDs mComponentPeerIDs;
extern struct CursorIDs cursorIDs;
extern struct ContainerIDs containerIDs;

/* globals */

extern XtAppContext awt_appContext;

/* forwards */

static void awt_XmDropProc(Widget, XtPointer, XmDropProcCallbackStruct*);
static void awt_XmDragProc(Widget, XtPointer, XmDragProcCallbackStruct*);

static void awt_XmTransferProc(Widget, XtPointer, Atom*, Atom*, XtPointer, unsigned long*, int*);

/* for XmDragContext callbacks etc ... */

static Boolean awt_XmConvertProc(Widget, Atom*, Atom*, Atom*, XtPointer*, unsigned long*, int*);

static Boolean awt_XmConvertIncrProc(Widget, Atom*, Atom*, Atom*, XtPointer*, unsigned long*, int*, unsigned long*, XtPointer, XtRequestId*);

static void awt_XmDragEnterProc(Widget, XtPointer, XmDropSiteEnterCallbackStruct*);
static void awt_XmDragMotionProc(Widget, XtPointer, XmDragMotionCallbackStruct*);
static void awt_XmDragLeaveProc(Widget, XtPointer, XmDropSiteLeaveCallbackStruct*);
static void awt_XmDropOperationChangedProc(Widget, XtPointer, XmDropStartCallbackStruct*);
static void awt_XmDragDropFinishProc(Widget, XtPointer, XmDropFinishCallbackStruct*);

static int  DnDConstantsToXm(jint operations);
static jint XmToDnDConstants(int operations);

static void flush_cache(JNIEnv* env);
static void    cacheDropDone(Boolean dropDone);
static Boolean isDropDone();

static void setCursor(JNIEnv* env, Display* d, jobject c, jint type, Time t);

/* JNI static values for methods in MDropTargetContextPeer */

static jclass dTCClazz;

static jmethodID dTCcreate;
static jmethodID dTCenter;
static jmethodID dTCexit;
static jmethodID dTCmotion;
static jmethodID dTCdrop;

static jmethodID dTCnewData;
static jmethodID dTCtxFailed;

/* JNI static values for methods in MDragSourceContextPeer */

static jclass dSCClazz;

static jmethodID dSCenter;
static jmethodID dSCmotion;
static jmethodID dSCchanged;
static jmethodID dSCexit;
static jmethodID dSCconvert;
static jmethodID dSCddfinished;

static int recWindowCount = 0; /*
				* used to simulate when window ids not available
				* provides hints to upper layers regarding 
				* transitions between drop target windows
				*/

/* Cursor Support */

static jclass string; /* used to construct string objects for native cache */
static jclass file;   /* used to construct file objects for transfers */

extern int getModifiers(int); /* in canvas.c */

/**
 * static cache of DropTarget related info.
 */

static struct {
    Widget  	  w;		/* if NULL, cache invalid */

    jobject	  peer;
    jobject	  component;

    jobject	  dtcpeer;

    Widget	  dt;

    jintArray	  targets;
    Cardinal	  nTargets;

    Boolean	  dropDone;
    int		  transfersPending;
    Widget	  transfer;

    int		  completionStatus; // used only on JVM transfers
    jint	  dropAction; // used only on JVM transfers

    Boolean	  flushPending;

    jobject	  formatMap;
} _cache;


/**
 * global function to initialize this client as a Dynamic-only app.
 *
 * gets called once during toolkit initialization.
 */

void awt_initialize_Xm_DnD(Display* dpy) {
	JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
	jclass  clazz;

	XtVaSetValues(XmGetXmDisplay(dpy),
		      XmNdragInitiatorProtocolStyle, XmDRAG_DYNAMIC,
		      XmNdragReceiverProtocolStyle,  XmDRAG_DYNAMIC,
		      NULL
	);

        /* No drop in progress. */
        cacheDropDone(True);

	/*
   	 * load class java.lang.string
	 */

	string = (*env)->FindClass(env, "java/lang/String");

	if (JNU_IsNull(env, string)) {
	    (*env)->ThrowNew(env,
			     (*env)->FindClass(env, "java/lang/ClassNotFoundException"),
			     "Cant find java/lang/String"
	    );
	}

	string = (*env)->NewGlobalRef(env, string); /* never freed! */

	/*
   	 * load class sun.awt.motif.MDropTargetContextPeer and resolve methods
	 */

	dTCClazz = (*env)->FindClass(env, "sun/awt/motif/MDropTargetContextPeer");

	if (JNU_IsNull(env, dTCClazz)) {
	    (*env)->ThrowNew(env,
			     (*env)->FindClass(env, "java/lang/ClassNotFoundException"),
			     "Cant find sun/awt/motif/MDropTargetContextPeer"
	    );
	}

	dTCClazz = (*env)->NewGlobalRef(env, dTCClazz); /* never freed! */

	if (JNU_IsNull(env, dTCClazz)) return;

	dTCcreate = (*env)->GetStaticMethodID(env,
					      dTCClazz,
					      "createMDropTargetContextPeer",
					      "()Lsun/awt/motif/MDropTargetContextPeer;"
		    );

	if (JNU_IsNull(env, dTCcreate)) return;

	dTCenter = (*env)->GetMethodID(env,
				       dTCClazz,
				       "handleEnterMessage",
				        "(Ljava/awt/Component;IIII[IJ)I");


	if (JNU_IsNull(env, dTCenter)) return;

	dTCexit = (*env)->GetMethodID(env,
				      dTCClazz,
				       "handleExitMessage",
				        "(Ljava/awt/Component;J)V"
		    );


	if (JNU_IsNull(env, dTCexit)) return;

	dTCmotion = (*env)->GetMethodID(env,
				       dTCClazz,
				       "handleMotionMessage",
				       "(Ljava/awt/Component;IIII[IJ)I"
		    );


	if (JNU_IsNull(env, dTCmotion)) return;

	dTCdrop = (*env)->GetMethodID(env,
				      dTCClazz,
				      "handleDropMessage",
				       "(Ljava/awt/Component;IIII[IJ)V");

	if (JNU_IsNull(env, dTCdrop)) return;
	/*
         * now load the data method callbacks for MDropTargetContextPeer 
	 */

	dTCnewData = (*env)->GetMethodID(env,
				         dTCClazz,
				         "newData",
				         "(ILjava/lang/String;[B)V"
		     );

	if (JNU_IsNull(env, dTCnewData)) return;

	dTCtxFailed = (*env)->GetMethodID(env,
                                          dTCClazz,
                                          "transferFailed",
                                          "(I)V"
                      );

	if (JNU_IsNull(env, dTCtxFailed)) return;

	/*
   	 * load class sun.awt.motif.MDragSourceContextPeer and resolve methods
	 */

	dSCClazz = (*env)->FindClass(env, "sun/awt/motif/MDragSourceContextPeer");

	if (JNU_IsNull(env, dSCClazz)) {
	    (*env)->ThrowNew(env,
			     (*env)->FindClass(env, "java/lang/ClassNotFoundException"),
			     "Cant find sun/awt/motif/DragSourceContextPeer"
	    );
	}

	dSCClazz = (*env)->NewGlobalRef(env, dSCClazz); /* never freed! */

	if (JNU_IsNull(env, dSCClazz)) return;

	dSCenter = (*env)->GetMethodID(env,
                                       dSCClazz,
                                       "dragEnter",
				       "(III)V");

	if (JNU_IsNull(env, dSCenter)) return;

	dSCmotion = (*env)->GetMethodID(env,
                                        dSCClazz,
                                        "dragMotion",
					"(III)V");

	if (JNU_IsNull(env, dSCmotion)) return;

	dSCchanged = (*env)->GetMethodID(env,
                                         dSCClazz,
                                         "operationChanged",
					 "(III)V");

	if (JNU_IsNull(env, dSCchanged)) return;

	dSCexit = (*env)->GetMethodID(env,
                                      dSCClazz,
                                      "dragExit",
				      "()V"
                  );

	if (JNU_IsNull(env, dSCexit)) return;

	dSCconvert = (*env)->GetMethodID(env,
                                         dSCClazz,
                                         "convertData",
                                         "(ILjava/util/Map;)[B"
                      );

	if (JNU_IsNull(env, dSCconvert)) return;

	dSCddfinished = (*env)->GetMethodID(env,
                                          dSCClazz,
                                          "dragDropFinished",
                                          "(ZI)V"
                        );

	if (JNU_IsNull(env, dSCddfinished)) return;

	/*
	 * load the Cursor stuff
	 */

	clazz = (*env)->FindClass(env, "sun/awt/motif/X11CustomCursor");

	if (JNU_IsNull(env, clazz)) {
	    (*env)->ThrowNew(env,
			     (*env)->FindClass(env, "java/lang/ClassNotFoundException"),
			     "Cant find sun/awt/motif/X11CustomCursor	"
	    );
	}
}

/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    addNativeDropTarget
 * Signature: (Ljava/awt/dnd/DropTarget;)V
 */

JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_addNativeDropTarget (JNIEnv *env, jobject this, jobject droptarget) {
    struct ComponentData* cdata     = (struct ComponentData *)NULL;
           DropSitePtr	  dropsite  = (DropSitePtr)NULL;

    if (JNU_IsNull(env, droptarget)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }

    AWT_LOCK();

    cdata = (struct ComponentData *)
      JNU_GetLongFieldAsPtr(env, this, mComponentPeerIDs.pData);

    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    /* introduce a new Component as a root of a set of DropTargets */

    if ((dropsite = cdata->dsi) == (DropSitePtr)NULL) {
	Arg 	     args[10];
	unsigned int nargs = 0;

	dropsite = cdata->dsi = (DropSitePtr)ZALLOC(DropSiteInfo);

	if (dropsite == (DropSitePtr)NULL) {
	    JNU_ThrowOutOfMemoryError (env, "OutOfMemoryError");
	    AWT_UNLOCK ();
            return;
        }

	/*
         * We mark Composite widgets as COMPOSITE DropSite's by default
         */

	dropsite->component = (*env)->
	  NewGlobalRef(env,
		       (*env)->GetObjectField(env,
					      this,
					      mComponentPeerIDs.target));
	dropsite->isComposite =
	  XtIsSubclass(cdata->widget, compositeWidgetClass);
	for (dropsite->tlw = cdata->widget;
	     XtParent(dropsite->tlw) != NULL;
	     dropsite->tlw = XtParent(dropsite->tlw))
	;

	
#define SetArg(n, v) args[nargs].name = n; args[nargs++].value = (XtArgVal)(v);

	SetArg(XmNanimationStyle,	XmDRAG_UNDER_NONE);
	SetArg(XmNdragProc,		awt_XmDragProc);
	SetArg(XmNdropProc,		awt_XmDropProc);
	SetArg(XmNdropSiteActivity,	XmDROP_SITE_ACTIVE);

	SetArg(XmNdropSiteOperations,	XmDROP_LINK |
					XmDROP_MOVE |
					XmDROP_COPY
	);

	SetArg(XmNimportTargets,	NULL);
	SetArg(XmNnumImportTargets,	0);

	if (dropsite->isComposite) {
	    SetArg(XmNdropSiteType,    XmDROP_SITE_COMPOSITE);
	} else {
	    SetArg(XmNdropSiteType,    XmDROP_SITE_SIMPLE);
	    SetArg(XmNdropRectangles, (XRectangle*)NULL);
	}

#undef  SetArg

	XmDropSiteUnregister(cdata->widget);
	XmDropSiteRegister(cdata->widget, args, nargs);
	XmDropSiteConfigureStackingOrder(cdata->widget, (Widget)NULL, XmABOVE);

	/*
 	 *
	 */

	{
	    jobject ref = (jobject)NULL;

	    Arg	    args[1];
	
	    args[0].name = XmNuserData; args[0].value = (XtArgVal)&ref;

	    XtGetValues(cdata->widget, args, 1);

	    if (ref == (jobject)NULL || JNU_IsNull(env, ref)) {
		args[0].value = (XtArgVal)this;

		XtSetValues(cdata->widget, args, 1);
	    }
	}
    }

    dropsite->dsCnt++;

    AWT_UNLOCK();
}
 
/*
 * Class:     sun_awt_motif_MComponentPeer
 * Method:    removeNativeDropTarget
 * Signature: (Ljava/awt/dnd/DropTarget;)V
 */

JNIEXPORT void JNICALL Java_sun_awt_motif_MComponentPeer_removeNativeDropTarget (JNIEnv *env, jobject this, jobject droptarget) {

    struct ComponentData* cdata;
    DropSitePtr		  dropsite;

    if (JNU_IsNull(env, droptarget)) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return;
    }

    AWT_LOCK();

    cdata = (struct ComponentData *)
      JNU_GetLongFieldAsPtr(env, this, mComponentPeerIDs.pData);

    if (cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    if ((dropsite = cdata->dsi) == (DropSitePtr)NULL) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return;
    }

    dropsite->dsCnt--;
    if (dropsite->dsCnt == 0) {
	XmDropSiteUnregister(cdata->widget);

	(*env)->DeleteGlobalRef(env, dropsite->component);

	free((void *)(cdata->dsi));
	cdata->dsi = (DropSitePtr)NULL;
    }

    AWT_UNLOCK();
}

/**
 *
 */

JNIEXPORT jlong JNICALL Java_sun_awt_motif_MDropTargetContextPeer_startTransfer(JNIEnv *env, jobject this, jlong dragContextVal, jint atom) {
    XmDropTransferEntryRec trec;
    Widget		   dropTransfer;
    Arg		           args[3];
    unsigned int	   nargs = 0;
    jboolean		   isCopy;
    Widget		   dragContext = (Widget)(unsigned long)dragContextVal;
    AWT_LOCK();

    trec.target      = (Atom)atom;
    trec.client_data = (XtPointer)trec.target;
    

#define SetArg(n, v) args[nargs].name = n; args[nargs++].value = (XtArgVal)(v);

    SetArg(XmNdropTransfers,    &trec);
    SetArg(XmNnumDropTransfers, 1    );
    SetArg(XmNtransferProc,     awt_XmTransferProc);

#undef SetArg

    _cache.transfer = dropTransfer = XmDropTransferStart(dragContext, args, nargs);

    _cache.transfersPending++;

    AWT_NOTIFY_ALL();
    AWT_UNLOCK();

    return (jlong)(unsigned long)dropTransfer;
}

/**
 *
 */

JNIEXPORT void JNICALL Java_sun_awt_motif_MDropTargetContextPeer_addTransfer(JNIEnv *env, jobject this, jlong dropTransferVal, jint atom) {
    XmDropTransferEntryRec trec;
    jboolean		   isCopy;
    Widget		   dropTransfer = (Widget)(unsigned long)dropTransferVal;
    trec.target      = (Atom)atom;
    trec.client_data = (XtPointer)trec.target;

    AWT_LOCK();
    
    XmDropTransferAdd(dropTransfer, &trec, 1);

    _cache.transfersPending++;

    AWT_NOTIFY_ALL();
    AWT_UNLOCK();
}

JNIEXPORT jobject JNICALL
Java_sun_awt_dnd_SunDropTargetContextPeer_findComponentAt(JNIEnv *env,
                                                          jobject this,
                                                          jobject container,
                                                          jint x, jint y)
{
    /*
     * Call private version of Container.findComponentAt with the following
     * flags set -- ignoreEnabled = true (i.e., okay to return or recur into
     * disabled Components); ignoreGlassPane = true (i.e., don't return
     * a JRootPane's glass pane as the target Component)
     */
    jobject component =
        (*env)->CallObjectMethod(env, container, containerIDs.findComponentAt,
				 x, y, JNI_TRUE, JNI_TRUE);
    return component;
}

/**
 *
 */

JNIEXPORT void JNICALL Java_sun_awt_motif_MDropTargetContextPeer_dropDone
    (JNIEnv *env, jobject this, jlong dragContextVal, jlong dropTransferVal,
     jboolean isLocal, jboolean success, jint dropAction)
{
    Widget dropTransfer = (Widget)(unsigned long)dropTransferVal;
    Widget dragContext = (Widget)(unsigned long)dragContextVal;

    AWT_LOCK();

    if (_cache.w == (Widget)NULL) {
	AWT_UNLOCK();
	return; 
    }

    if (!isDropDone()) {
	if (dropTransfer != (jlong)NULL) {
            XtVaSetValues(dropTransfer,
		      	  XmNtransferStatus,
		      	  success == JNI_TRUE ? XmTRANSFER_SUCCESS : XmTRANSFER_FAILURE,
		          NULL
	    );
	} else {
	  /*
	   * start a transfer that notifies failure
	   * this causes src side callbacks to be processed.
	   * However, you cannot pass an a success, so the workaround is
	   * to set _cache.transferSuccess to the proper value and read it
	   * on the other side.
	   */


	    Arg arg;

	    /*
	     * this is the workaround code
	     */
	    _cache.transfer = NULL; 
	    _cache.completionStatus = success == JNI_TRUE ? XmTRANSFER_SUCCESS
							  : XmTRANSFER_FAILURE;
	    _cache.dropAction = dropAction;

	    /*
	     * End workaround code
	     */

	    arg.name  = XmNtransferStatus;
	    arg.value = (XtArgVal)(success == JNI_TRUE ? XmTRANSFER_SUCCESS 
						       : XmTRANSFER_FAILURE
			);

	    XmDropTransferStart(dragContext, &arg, 1);
	}

	/*
         * bugid# 4146717
	 *
	 * If this is a local tx, then we never exec the awt_XmTransferProc,
	 * thus we need to flush the cache here as it is our only chance,
	 * otherwise we leave a mess for the next operation to fail on ....
	 *
	 */

	if (isLocal == JNI_TRUE) 
	    flush_cache(env); /* flush now, last chance */
	else
            _cache.flushPending = True; /* flush pending in transfer proc */
    }

    cacheDropDone(True);

    AWT_NOTIFY_ALL();
    AWT_UNLOCK();
}

/**
 *
 */
 
JNIEXPORT void JNICALL Java_sun_awt_motif_MDragSourceContextPeer_setCursor(JNIEnv *env, jobject this, jlong nativeCtxtVal, jobject cursor, jint type) {
    Widget nativeCtxt = (Widget)(unsigned long)nativeCtxtVal;
    Display*		dpy     = XtDisplayOfObject(nativeCtxt);

    AWT_LOCK();

    setCursor(env, dpy, cursor, type, CurrentTime);

    AWT_UNLOCK();
}

/**
 *
 */

JNIEXPORT void JNICALL Java_sun_awt_motif_AWTLockAccess_awtLock(JNIEnv *env, jclass clazz) {
    AWT_LOCK();
}

/**
 *
 */

JNIEXPORT void JNICALL Java_sun_awt_motif_AWTLockAccess_awtUnlock(JNIEnv *env, jclass clazz) {
    AWT_NOTIFY_ALL();
    AWT_UNLOCK();
}

/**
 *
 */

JNIEXPORT void JNICALL 
Java_sun_awt_motif_AWTLockAccess_awtWait(JNIEnv *env, jclass clazz, jint timeout) {
    AWT_WAIT(timeout);
}

/**
 *
 */

JNIEXPORT void JNICALL 
Java_sun_awt_motif_AWTLockAccess_awtNotifyAll(JNIEnv *env, jclass clazz) {
    AWT_NOTIFY_ALL();
}

/**
 *
 */

JNIEXPORT jlong JNICALL Java_sun_awt_motif_MDragSourceContextPeer_startDrag(JNIEnv *env, jobject this, jobject nativeCont, jobject trigger, jobject cursor, jint ctype, jint actions, jintArray formats, jobject formatMap) {
    struct ComponentData* cdata;
    Display*		  dpy;
    Arg			  args[32];
    int			  nargs = 0;
    jobject		  dscp  = (*env)->NewGlobalRef(env, this);
    int			  i;
    Atom*		  targets;
    int			  nTargets;
    Widget		  dc;
    XtCallbackRec	  dsecbr[2];
    XtCallbackRec	  dmcbr[2];
    XtCallbackRec	  occbr[2];
    XtCallbackRec	  dslcbr[2];
    XtCallbackRec	  dscbr[2];
    XtCallbackRec	  ddfcbr[2];
    XEvent*		  event;
    int			  xmActions = DnDConstantsToXm(actions);
    jboolean		  isCopy=JNI_TRUE;

    if (xmActions == XmDROP_NOOP) {
	return 0;
    }

    AWT_LOCK();

    /*
     * Fix for BugTraq ID 4357905.
     * Drop is processed asynchronously on the event dispatch thread.
     * Reject all drag attempts until the current drop is done.
     */      
    if (!isDropDone()) {
        AWT_UNLOCK();
        return (jlong)NULL;
    }

#define SetCB(cbr, cb, cl) cbr[0].callback = (XtCallbackProc)cb; cbr[0].closure = (XtPointer)cl; cbr[1].callback = (XtCallbackProc)NULL; cbr[1].closure = (XtPointer)NULL

#define SetArg(n, v) args[nargs].name = n; args[nargs++].value = (XtArgVal)(v);

    cdata = (struct ComponentData *)
      JNU_GetLongFieldAsPtr(env, nativeCont, mComponentPeerIDs.pData);

    if (JNU_IsNull(env, formats) || cdata == 0 || cdata->widget == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        AWT_UNLOCK();
        return(jlong)NULL;
    }

    event = (XEvent*)JNU_GetLongFieldAsPtr(env, trigger, awtEventIDs.data);

    if (event == (XEvent*)NULL || event->type != ButtonPress) {
	AWT_UNLOCK();
	return (jlong)NULL;
    }

    dpy = XtDisplayOfObject(cdata->widget);

    nTargets = (*env)->GetArrayLength(env, formats);
    targets = (Atom*)(*env)->GetIntArrayElements(env, formats, &isCopy);
    if (JNU_IsNull(env, targets) || !JNU_IsNull(env, ((*env)->ExceptionOccurred(env)))) {
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);
	targets = NULL;
	nTargets = 0;
    }
    
    SetCB(dsecbr, awt_XmDragEnterProc,		  dscp);
    SetCB(dmcbr,  awt_XmDragMotionProc,		  dscp);
    SetCB(occbr,  awt_XmDropOperationChangedProc, dscp);
    SetCB(dslcbr, awt_XmDragLeaveProc,		  dscp);
    SetCB(ddfcbr, awt_XmDragDropFinishProc,	  dscp);

    SetArg(XmNblendModel,    		XmBLEND_NONE          );
    SetArg(XmNclientData,	        dscp		      );
    SetArg(XmNdragOperations,	        xmActions             );
    /* No incremental transfer */
    SetArg(XmNconvertProc,   		awt_XmConvertProc     );
    SetArg(XmNdropSiteEnterCallback,    dsecbr		      );
    SetArg(XmNdragMotionCallback, 	dmcbr		      );
    SetArg(XmNoperationChangedCallback,	occbr		      );
    SetArg(XmNdropSiteLeaveCallback,    dslcbr		      );
    SetArg(XmNdragDropFinishCallback,   ddfcbr		      );
    SetArg(XmNexportTargets,		targets		      );
    SetArg(XmNnumExportTargets,    	nTargets	      );
    SetArg(XmNexportTargets,    	targets               );
    SetArg(XmNnumExportTargets,    	nTargets              );
  
    dc = XmDragStart(cdata->widget, event, args, nargs);
  
    if (dc != (Widget)NULL) {
	setCursor(env, dpy, cursor, ctype, event->xbutton.time);
    }
  
    (*env)->ReleaseIntArrayElements(env, formats, (jint *)targets, JNI_ABORT);

    _cache.formatMap = (*env)->NewGlobalRef(env, formatMap);

    AWT_UNLOCK();

    return (jlong)(unsigned long)dc;

#undef SetArg
#undef SetCB
}

/******************************************************************************/

/**
 *
 */

static void setCursor(JNIEnv* env, Display* dpy, jobject cursor, jint type, Time time) {
    Cursor xcursor = None;

    if (JNU_IsNull(env, cursor)) return;

    XChangeActivePointerGrab(dpy,
			     ButtonPressMask   |
			     ButtonMotionMask  |
			     ButtonReleaseMask |
			     EnterWindowMask   |
			     LeaveWindowMask,
			     getCursor(env, cursor),
	    		     time
    );

    XSync(dpy, False);
}

/**
 * Update the cached targets for this widget
 */

static Boolean updateCachedTargets(JNIEnv* env, Widget dt) {
    Atom*    	  targets  = (Atom*)NULL;
    Cardinal 	  nTargets = (Cardinal)0;
    Arg      	  args[2];
    int		  i;

    /*
     * Get the targets for this component
     */
    args[0].name = XmNexportTargets;    args[0].value = (XtArgVal)&targets;
    args[1].name = XmNnumExportTargets; args[1].value = (XtArgVal)&nTargets;
    XtGetValues(_cache.dt = dt, args, 2);

    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) return False; 

    /*
     * Free the previous targets if there were any
     */
    if (!JNU_IsNull(env, _cache.targets)) {
	(*env)->DeleteGlobalRef(env, _cache.targets);
	_cache.targets = (jintArray)NULL;
    }

    _cache.nTargets = nTargets;

    /*
     * If the widget has targets (atoms) then copy them to the cache
     */
    if (nTargets > 0) {
	_cache.targets = (*env)->NewIntArray(env, nTargets);

	if (!JNU_IsNull(env, _cache.targets)) {
	    _cache.targets = (*env)->NewGlobalRef(env, _cache.targets);
	    if (JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		(*env)->SetIntArrayRegion(env, _cache.targets, 0, nTargets,
					  (jint *) targets);
		if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		    (*env)->DeleteGlobalRef(env, _cache.targets);
		    _cache.nTargets  = (Cardinal)0;
		    _cache.targets = (jintArray)NULL;
		    return False;
		}
		return True;
	    }
	}
    }

    return False;
}


/**
 *
 */

static void flush_cache(JNIEnv* env) {
    _cache.w  = (Widget)NULL;
    _cache.dt = (Widget)NULL;

    (*env)->DeleteGlobalRef(env, _cache.peer);
    _cache.peer = (jobject)NULL;

    (*env)->DeleteGlobalRef(env, _cache.component);
    _cache.component = (jobject)NULL;

   if (_cache.dtcpeer != (jobject)NULL) {
	(*env)->DeleteGlobalRef(env, _cache.dtcpeer);

	_cache.dtcpeer = (jobject)NULL;
   }

   _cache.nTargets  = (Cardinal)0;
   if (_cache.targets != (jintArray)NULL) {
       (*env)->DeleteGlobalRef(env, _cache.targets);
       _cache.targets = (jintArray)NULL;
   }

    _cache.transfersPending = 0;
    _cache.flushPending    = False;
    _cache.transfer         = (Widget)NULL;
    cacheDropDone(True);
}

/**
 *
 */

static void update_cache(JNIEnv* env, Widget w, Widget dt) {
    if(w != _cache.w) {
        struct ComponentData* cdata   = (struct ComponentData *)NULL;
	Arg		      args[1] = {{ XmNuserData, (XtArgVal)&_cache.peer}};

	flush_cache(env);

	if (w == (Widget)NULL) return;

	XtGetValues(w, args, 1);

	if (JNU_IsNull(env, _cache.peer)) {
	    _cache.w = NULL;

            return;
	}

	cdata = (struct ComponentData *) JNU_GetLongFieldAsPtr(env, _cache.peer, mComponentPeerIDs.pData);

        if (cdata         == 0 ||
	    cdata->widget != w ||
	    cdata->dsi    == (DropSitePtr)NULL) {
	    _cache.w = NULL;

            return;
	}

	_cache.w         = w;
	_cache.component = (*env)->NewGlobalRef(env, cdata->dsi->component);
	_cache.peer      = (*env)->NewGlobalRef(env, _cache.peer);
	/* SECURITY: OK to call this on privileged thread - peer is secure */
	_cache.dtcpeer   = (*env)->CallStaticObjectMethod(env,
						          dTCClazz,
						  	  dTCcreate						   );

	_cache.dtcpeer          = (*env)->NewGlobalRef(env, _cache.dtcpeer);

        _cache.transfersPending = 0;
        cacheDropDone(True);
    }

    if (_cache.w != (Widget)NULL) updateCachedTargets(env, dt);
}


/**
 *
 */

static jint convertModifiers(Display* dpy, XEvent* event) {
    jint 	 modifiers = 0;
    unsigned int xMods	   = 0;

    if (event != (XEvent*)NULL) {
	switch (event->type) {
	    case KeyPress:
	    case KeyRelease:
	        modifiers = getModifiers(event->xkey.state);
	        break;

	    case ButtonPress:
	    case ButtonRelease:
	        modifiers = getModifiers(event->xbutton.state);
	        break;
	}
    } else {
	Window dummyw;
	int    dummyi;

	XQueryPointer(dpy,
		      DefaultRootWindow(dpy),
		      &dummyw,
		      &dummyw,
		      &dummyi,
		      &dummyi,
		      &dummyi,
		      &dummyi,
		      &xMods
	);
    }

    return getModifiers(xMods);
}

static void 
cacheDropDone(Boolean dropDone) {
    _cache.dropDone = dropDone;
}

static Boolean
isDropDone() {
    return _cache.dropDone;
}

/**
 *
 */

static jint XmToDnDConstants(int operations) {
    jint src = java_awt_dnd_DnDConstants_ACTION_NONE;

    if (operations & XmDROP_MOVE) src |= java_awt_dnd_DnDConstants_ACTION_MOVE;
    if (operations & XmDROP_COPY) src |= java_awt_dnd_DnDConstants_ACTION_COPY;
    if (operations & XmDROP_LINK) src |= java_awt_dnd_DnDConstants_ACTION_LINK;

    return src;
}

/**
 *
 */

static int DnDConstantsToXm(jint actions) {
    int ret = XmDROP_NOOP;

    if (actions & java_awt_dnd_DnDConstants_ACTION_COPY) ret |= XmDROP_COPY;
    if (actions & java_awt_dnd_DnDConstants_ACTION_MOVE) ret |= XmDROP_MOVE;
    if (actions & java_awt_dnd_DnDConstants_ACTION_LINK) ret |= XmDROP_LINK;

    return ret;
}

/**
 *
 */

static void awt_XmDragProc(Widget w, XtPointer closure, XmDragProcCallbackStruct* cbstruct) {
    JNIEnv* env       = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject component = (jobject)NULL;
    jint    src       = java_awt_dnd_DnDConstants_ACTION_NONE;
    jint    ret       = java_awt_dnd_DnDConstants_ACTION_NONE;

    /*
     * Fix for BugTraq ID 4357905.
     * Drop is processed asynchronously on the event dispatch thread.
     * We reject other drop attempts to protect the SunDTCP context 
     * from being overwritten by an upcall before the drop is done.
     */
    if (!isDropDone()) {
        cbstruct->operation  = XmDROP_NOOP;
        cbstruct->dropSiteStatus = XmINVALID_DROP_SITE;
        return;
    }

    (*env)->PushLocalFrame(env, 0);

    src = XmToDnDConstants(cbstruct->operations);

    update_cache(env, w, cbstruct->dragContext);

    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	flush_cache(env);
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);
	goto wayout;
    }

    switch (cbstruct->reason) {
	case XmCR_DROP_SITE_ENTER_MESSAGE: {

	    /* SECURITY: OK to call this on privileged thread - peer is secure */
	    ret = (*env)->CallIntMethod(env,
					_cache.dtcpeer,
					dTCenter,
					_cache.component,
					cbstruct->x,
					cbstruct->y,
					XmToDnDConstants(cbstruct->operation),
					src,
					_cache.targets,	
					(jlong)(unsigned long)(cbstruct->dragContext)
		   );

	    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		flush_cache(env);
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
	    }
	}
	break;

	case XmCR_DROP_SITE_LEAVE_MESSAGE: {

	    /* SECURITY: OK to call this on privileged thread - peer is secure */
	    (*env)->CallVoidMethod(env,
				   _cache.dtcpeer,
				   dTCexit,
				   _cache.component,
				   (jlong)(unsigned long)(cbstruct->dragContext)
	    );

	    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		flush_cache(env);
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
	    }

	    ret = java_awt_dnd_DnDConstants_ACTION_NONE;

	    /* now cleanup */

	    flush_cache(env);
	}
	break;

	case XmCR_DROP_SITE_MOTION_MESSAGE: {

	    /* SECURITY: OK to call this on privileged thread - peer is secure */
	    ret = (*env)->CallIntMethod(env,
					_cache.dtcpeer,
					dTCmotion,
					_cache.component,
					cbstruct->x,
					cbstruct->y,
					XmToDnDConstants(cbstruct->operation),
					src,
					_cache.targets,
					(jlong)(unsigned long)(cbstruct->dragContext)
		   );

	    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		flush_cache(env);
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
	    }

	}
	break;

	case XmCR_OPERATION_CHANGED: {

	    /* SECURITY: OK to call this on privileged thread - peer is secure */
	    ret = (*env)->CallIntMethod(env,
					_cache.dtcpeer,
					dTCmotion,
					_cache.component,
					cbstruct->x,
					cbstruct->y,
					XmToDnDConstants(cbstruct->operation),
					src, 
					_cache.targets,
					(jlong)(unsigned long)(cbstruct->dragContext)
		   );

	    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		flush_cache(env);
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
	    }

	}
	break;

	default: break;
    }

    wayout: 

    cbstruct->operation  = DnDConstantsToXm(ret) & cbstruct->operations;

    if (ret != java_awt_dnd_DnDConstants_ACTION_NONE) {
        cbstruct->dropSiteStatus = XmVALID_DROP_SITE;
    }  else {
	cbstruct->dropSiteStatus = XmINVALID_DROP_SITE;
    }

    (*env)->PopLocalFrame(env, 0);
}

/**
 *
 */

static void awt_XmDropProc(Widget w, XtPointer closure, XmDropProcCallbackStruct* cbstruct) {
    JNIEnv* env	      = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jint    src       = java_awt_dnd_DnDConstants_ACTION_NONE;
    int     operation = cbstruct->operation;

    jobject dtcpeer = NULL;

    /*
     * Fix for BugTraq ID 4357905.
     * Drop is processed asynchronously on the event dispatch thread.
     * We reject other drop attempts to protect the SunDTCP context 
     * from being overwritten by an upcall before the drop is done.
     */
    if (!isDropDone()) {
        return;
    }

    (*env)->PushLocalFrame(env, 0);


    src = XmToDnDConstants(cbstruct->operations);

    update_cache(env, w, cbstruct->dragContext);

    cacheDropDone(False);

    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);
	(*env)->PopLocalFrame(env, 0);
	return;
    }

    /* SECURITY: OK to call this on privileged thread - peer is secure */
    (*env)->CallVoidMethod(env,
			   _cache.dtcpeer,
			   dTCdrop,
			   _cache.component,
			   cbstruct->x,
			   cbstruct->y,
			   XmToDnDConstants(operation),
			   src,
			   _cache.targets,
			   (jlong)(unsigned long)(cbstruct->dragContext)
			   );

    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	flush_cache(env);
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);
    }
    (*env)->PopLocalFrame(env, 0);
}

/**
 *
 */

static void awt_XmTransferProc(Widget w, XtPointer closure, Atom* selection, Atom* type, XtPointer value, unsigned long* length, int* format) {
    JNIEnv*  env        = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    Atom     req        = (Atom)closure;
    Display* dpy        = XtDisplayOfObject(w);
    jobject  tName = NULL;

    /*
     * Note: this method is only called to transfer data between clients
     * in different JVM's or native apps. For Intra-JVM transfers the peer
     * code shares the sources Transferable with the destination.
     */

    if (_cache.w == (Widget)NULL || _cache.transfer != w) return; /* we have already cleaned up ... */

    (*env)->PushLocalFrame(env, 0);

    if (*type == None || *type == XT_CONVERT_FAIL) {
        /* SECURITY: OK to call this on privileged thread - peer is secure
         */
        (*env)->CallVoidMethod(env,
                               _cache.dtcpeer,
                               dTCtxFailed,
                               (jint)req);
    } else {
        switch (*format) {
        case  8:
        case 16:
        case 32: {
            jbyteArray arry = (*env)->NewByteArray(env, *length);

            if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
                (*env)->ExceptionDescribe(env);
                (*env)->ExceptionClear(env);

                /* SECURITY: OK to call this on privileged thread - peer is secure */
                (*env)->CallVoidMethod(env,
                                       _cache.dtcpeer,
                                       dTCtxFailed,
                                       (jint)req);

                goto wayout;
            }

            (*env)->SetByteArrayRegion(env, arry, 0, *length, (jbyte*)value);
            if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
                (*env)->ExceptionDescribe(env);
                (*env)->ExceptionClear(env);

                /* SECURITY: OK to call this on privileged thread - peer is secure */
                (*env)->CallVoidMethod(env,
                                       _cache.dtcpeer,
                                       dTCtxFailed,
                                       (jint)req);
                goto wayout;
            }

            arry = (*env)->NewGlobalRef(env, arry);

            if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
                (*env)->ExceptionDescribe(env);
                (*env)->ExceptionClear(env);
            }

            {
                char* tn = XGetAtomName(dpy, *type);

                tName = (*env)->NewStringUTF(env, (const char *)tn);

                if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
                    (*env)->ExceptionDescribe(env);
                    (*env)->ExceptionClear(env);
                }

                XFree((void *)tn);
            }

            /* SECURITY: OK to call this on privileged thread - peer is
               secure */
            (*env)->CallVoidMethod(env,
                                   _cache.dtcpeer,
                                   dTCnewData,
                                   (jint)req,
                                   tName,
                                   arry);

            if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
                (*env)->ExceptionDescribe(env);
                (*env)->ExceptionClear(env);
            }
        }

        default:
            break;
        }
    }

    wayout:
    _cache.transfersPending--;
    while (_cache.transfersPending == 0 && !isDropDone()) {
        AWT_WAIT(0);
    }

    if (isDropDone() && _cache.flushPending) {
        flush_cache(env);
    }

    (*env)->PopLocalFrame(env, 0);
}
 
/**
 *
 */

static Boolean awt_XmConvertProc(Widget w, Atom* selection, Atom* target, Atom* type, XtPointer* value, unsigned long* length, int* format) {
    unsigned long maxLength = 0x7fffffffL;
    XtRequestId   rid	    = (XtRequestId)0;
    XtPointer     closure   = (XtPointer)NULL;

    XtVaGetValues(w, XmNclientData, (XtArgVal)&closure, NULL);

    return awt_XmConvertIncrProc(w, selection, target, type, value, length, format, &maxLength, closure, &rid);

}

/**
 *
 */

static Boolean awt_XmConvertIncrProc(Widget w, Atom* selection, Atom* target, Atom* type, XtPointer* value, unsigned long* length, int* format, unsigned long* max_length, XtPointer closure, XtRequestId* receiver_id) {
    JNIEnv*  env  = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject  this = (jobject)closure;
    Display* dpy  = XtDisplayOfObject(w);

    (*env)->PushLocalFrame(env, 0);

    if (*target == XInternAtom(dpy, _XA_DELETE, False)) {

	/*
	 * acknowledge the DELETE target here ... the "delete" semantic
	 * of move will take place after the drop is complete.
	 */

	*type   = XInternAtom(dpy, _XA_NULL, False);
	*length = 0;
	*value  = (XtPointer)NULL;
    } else if (*target == XInternAtom(dpy, _XA_HOSTNAME, False)) {
	struct utsname name;
	char *nodename;
	XTextProperty  tp;

	uname(&name);

	nodename = name.nodename;
	XStringListToTextProperty(&nodename, 1, &tp);

	*value = (XtPointer)XtCalloc(tp.nitems, sizeof(char));

	memcpy((void *)*value, (const void *)tp.value, tp.nitems);

	XFree((void *)tp.value);

	*type   = tp.encoding;
	*length = tp.nitems + 1;
	*format = tp.format;
    } else if (*target == XInternAtom(dpy, _XA_FILENAME, False) ||
	       *target == XInternAtom(dpy, _DT_FILENAME, False)) {

	/*
	 * Convert the internal representation to an File Name.
	 * The data returned from dataConvert is a an array of
	 * null separated bytes. Each series of bytes is a string
	 * that is then converted to an XString which are then put
	 * into an XStringList and put into an XTextProperty for
	 * usage in other programs.
	 *
	 * It would be desireable to have dataConvert to this conversion
	 * but it isn't possible to return a byte array that represents
	 * the XTextProperty.
	 */
	jbyteArray    data;
	Boolean       test = False;
       
	/*  assume forthcoming error */
	*type   = XT_CONVERT_FAIL;
        *value  = (XtPointer)NULL;
        *length = 0L;
        *format = 8;
  
	/* 
	 * Get the File name in the form of null separated byte array
	 * SECURITY: OK to call this on privileged thread - peer is secure
	 */
	data = (jbyteArray) (*env)->CallObjectMethod(env,
						     this,
						     dSCconvert,
						     *target,
						     _cache.formatMap);
                                               
	/* error test */
	if (JNU_IsNull(env, data) ||
	    !JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
            (*env)->ExceptionDescribe(env);
            (*env)->ExceptionClear(env);
            (*env)->PopLocalFrame(env, 0);
            return False;
        }
  
	test = convertFileType(data, type, value, length, format);
	if (test == False) {
            return False;
        }

    } else {
	jbyteArray bytes;
	jbyte* copy;

	*type   = XT_CONVERT_FAIL; /* assume forthcoming error */
	*value  = (XtPointer)NULL;
	*length = 0L;
	*format = 8;

	/*
 	 * we get a Buffer object from the MDragSourceContextPeer, then we
 	 * process the Buffer, copying it into a single block of contiguous
	 * memory that we return to Xt.
 	 * 
	 */

	/* SECURITY: OK to call this on privileged thread - peer is secure */
	bytes = (*env)->CallObjectMethod(env,
					 this,
					 dSCconvert,
					 *target,
					 _cache.formatMap);
	
	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
    	    (*env)->PopLocalFrame(env, 0);
	    return False;
	}

	if (bytes == NULL) {
    	    (*env)->PopLocalFrame(env, 0);
	    return False;
	} else {
            jint    len = (*env)->GetArrayLength(env, bytes);

	    if (len == 0) {
	        *type   = *target;
	     	*format = 8;

    		(*env)->PopLocalFrame(env, 0);
		return True;
	    }

	    copy = (jbyte*)XtMalloc (len * sizeof(jbyte));
	    if (copy == (jbyte*)NULL) {
		JNU_ThrowOutOfMemoryError(env, "OutOfMemoryError");

    		(*env)->PopLocalFrame(env, 0);
 
		return False;
	    }

	    (*env)->GetByteArrayRegion(env, (jbyteArray)bytes, 0, len, copy);

	    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);

		XtFree((void *)copy);

    		(*env)->PopLocalFrame(env, 0);

	        return False;
	    }


	    *value  = (XtPointer)copy;
	    *type   = *target;
	    *length = len;
	    *format = 8;
	}
    }

    (*env)->PopLocalFrame(env, 0);

    return True;
}

/**
 *
 */

static void awt_XmDragEnterProc(Widget w, XtPointer closure,
				XmDropSiteEnterCallbackStruct* cbstruct) {
    JNIEnv*  env   = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject  this  = (jobject)closure;
 
    /*  This should only be valid, but Im leaving this part of the old code */
    jboolean valid = cbstruct->dropSiteStatus == XmVALID_DROP_SITE
	? JNI_TRUE : JNI_FALSE;
  
    if (valid == JNI_TRUE) {
	(*env)->PushLocalFrame(env, 0);
  
	/* SECURITY: OK to call this on privileged thread - peer is secure */
	(*env)->CallVoidMethod(env,
			       this,
			       dSCenter,
			       XmToDnDConstants(cbstruct->operation),
			       XmToDnDConstants(cbstruct->operations),
			       convertModifiers(XtDisplayOfObject(w), 
						cbstruct->event));
	
	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
	}
       
	(*env)->PopLocalFrame(env, 0);
    }
}

/**
 *
 */

static void awt_XmDragMotionProc(Widget w, XtPointer closure,
				 XmDragMotionCallbackStruct* cbstruct) {
    JNIEnv*  env   = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject  this  = (jobject)closure;
 
    /*  This should only be valid, but Im leaving this part of the old code */
    jboolean valid = cbstruct->dropSiteStatus == XmVALID_DROP_SITE
	? JNI_TRUE : JNI_FALSE;
  
    if (valid == JNI_TRUE) {
  
	(*env)->PushLocalFrame(env, 0);
  
	/* SECURITY: OK to call this on privileged thread - peer is secure */
	(*env)->CallVoidMethod(env,
			       this,
			       dSCmotion,
			       XmToDnDConstants(cbstruct->operation),
			       XmToDnDConstants(cbstruct->operations),
			       convertModifiers(XtDisplayOfObject(w), 
						cbstruct->event));
	
	if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
	}
  
       
	cbstruct->operations =  convertModifiers(XtDisplayOfObject(w), 
						 cbstruct->event);
  
	(*env)->PopLocalFrame(env, 0);
    } else {
        (*env)->PushLocalFrame(env, 0);
  
        /* SECURITY: OK to call this on privileged thread - peer is secure */
        (*env)->CallVoidMethod(env, this, dSCexit);
  
        if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	    (*env)->ExceptionDescribe(env);
	    (*env)->ExceptionClear(env);
        }
  
        (*env)->PushLocalFrame(env, 0);
    }
}

/**
 *
 */

static void awt_XmDragLeaveProc(Widget w, XtPointer closure,
				XmDropSiteLeaveCallbackStruct* cbstruct) {
    JNIEnv* env  = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject this = (jobject)closure;
    
    (*env)->PushLocalFrame(env, 0);
  
  
    /* SECURITY: OK to call this on privileged thread - peer is secure */
    (*env)->CallVoidMethod(env, this, dSCexit);
  
    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
  
    (*env)->PushLocalFrame(env, 0);
}

/**
 *
 */

static void awt_XmDropOperationChangedProc(Widget w, XtPointer closure, XmDropStartCallbackStruct* cbstruct) {
    JNIEnv*  env   = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject  this  = (jobject)closure;

    (*env)->PushLocalFrame(env, 0);

    /* SECURITY: OK to call this on privileged thread - peer is secure */
    (*env)->CallVoidMethod(env,
			   this,
			   dSCchanged,
			   XmToDnDConstants(cbstruct->operation),
			   XmToDnDConstants(cbstruct->operations),
			   convertModifiers(XtDisplayOfObject(w), cbstruct->event));

    if (!JNU_IsNull(env, (*env)->ExceptionOccurred(env))) {
	(*env)->ExceptionDescribe(env);
	(*env)->ExceptionClear(env);
    }


    (*env)->PopLocalFrame(env, 0);
}

/**
 *
 */

static void awt_XmDragDropFinishProc(Widget w, XtPointer closure, XmDropFinishCallbackStruct* cbstruct) {
    JNIEnv* env  = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject this = (jobject)closure;
    unsigned char completionStatus = cbstruct->completionStatus;
    jint dropAction = XmToDnDConstants(cbstruct->operation);

    /* cleanup */

    (*env)->PushLocalFrame(env, 0);

    if (_cache.transfer == NULL) {
        completionStatus = _cache.completionStatus;
	dropAction = _cache.dropAction;
    }
  
    /* SECURITY: OK to call this on privileged thread - peer is secure */
    (*env)->CallVoidMethod(env, this, dSCddfinished,
			   completionStatus,
			   dropAction);

    (*env)->DeleteGlobalRef(env, _cache.formatMap);
    _cache.formatMap = NULL;
    (*env)->DeleteGlobalRef(env, this);

    (*env)->PopLocalFrame(env, 0);
}

/******************************************************************************/
 
void sun_awt_motif_MComponentPeer_addNativeDropTarget(void* p1, void* p2, void* p3) { }
void sun_awt_motif_MComponentPeer_removeNativeDropTarget(void* p1, void* p2, void* p3) { }
void sun_awt_motif_MDropTargetContextPeer_dropDone(void* p1, void* p2, void* p3) { }
void sun_awt_motif_MDropTargetContextPeer_startTransfer(void* p1, void* p2, void* p3) { }
void sun_awt_motif_MDropTargetContextPeer_addTransfer(void* p1, void* p2, void* p3) { }
void sun_awt_motif_MDragSourceContextPeer_startDrag(void* p1, void* p2, void* p3) { }
void sun_awt_motif_MDragSourceContextPeer_setCursor(void* p1, void* p2, void* p3) { }
