/*
 * @(#)awt_motif21.c	1.7 00/08/30
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#if MOTIF_VERSION==2
 
#include "awt_motif.h"
#include <Xm/Xm.h>
#include <jvm.h>

#define XmPER_SHELL 0
extern XIC XmImGetXIC(
                                Widget          w,
                        unsigned int    input_policy,
                        ArgList         args,
                        Cardinal        num_args) ;

#define MAXARGS 10
static Arg xic_vlist[MAXARGS];

/* get_im_height: returns height of the input method status area in pixels.
 *
 * This function assumes that if any XIM related information cannot be
 * queried then the app must not have an input method status area in the
 * current locale and returns zero as the status area height
 */
int
awt_motif_getIMStatusHeight(Widget w)
{
    XIC xic = NULL;
    XRectangle *im_rect=NULL;
    int im_height = 0;
    char *ret;
    Widget p=w;

    
    while (!XtIsShell(p))
        p = XtParent(p);
    
    xic = XmImGetXIC(p, XmPER_SHELL, NULL, 0);

    if(xic == NULL) {
        return 0;
    }

    /* finally query the server for the status area geometry */
    xic_vlist[0].name = XNArea;
    xic_vlist[0].value = (XtArgVal)&im_rect;
    xic_vlist[1].name = NULL;
    ret=XGetICValues(xic, XNStatusAttributes, &xic_vlist[0], NULL);
    if (ret == NULL && im_rect != NULL) {
        im_height = im_rect->height;
        XFree(im_rect);
    }
    else {
        im_height = 0;
    }    

#if defined(DEBUG)
    jio_fprintf(stderr,"awt_motif_getIMStatusHeight: Height = %d",im_height);
#endif
    return im_height;
}


static XRectangle geometryRect;
XVaNestedList awt_motif_getXICStatusAreaList(Widget w)
{
    XIC xic;

    XRectangle geomRect ;
    XRectangle *im_rect;
    XFontSet   *im_font;

    Pixel bg ;
    Pixel fg ;
    Dimension height, width ;
    Position x,y ;
    Pixmap bpm, *bpmout ;

    XVaNestedList list = NULL;

    char *ret;
    Widget p=w;
    
    while (!XtIsShell(p)) {
        p = XtParent(p);
    }

    XtVaGetValues(p,
        XmNx, &x,
        XmNy, &y,
        XmNwidth, &width,
        XmNheight, &height,
        XmNbackgroundPixmap, &bpm,
        NULL);



    xic = XmImGetXIC(p, XmPER_SHELL, NULL, 0);
    if(xic == NULL)
    {
#if defined DEBUG
	jio_fprintf(stderr,"Could not get XIC");
#endif
	return list ;
    }

   /* finally query the server for the required attributes area geometry */
    xic_vlist[0].name = XNFontSet ;
    xic_vlist[0].value =  (XtArgVal) &im_font ;
    xic_vlist[1].name = XNArea;
    xic_vlist[1].value = (XtArgVal) &im_rect;
    xic_vlist[2].name = XNBackground ;
    xic_vlist[2].value = (XtArgVal) &bg ;
    xic_vlist[3].name = XNForeground ;
    xic_vlist[3].value = (XtArgVal) &fg ;
    xic_vlist[4].name = NULL;


    if(ret=XGetICValues(xic, XNStatusAttributes, &xic_vlist[0], NULL))
    {
	return list ;
    } else {
        geomRect.x = 0 ;
        geomRect.y = height - im_rect->height ;
        geomRect.width = im_rect->width ;
        geomRect.height = im_rect->height ;
        XFree(im_rect) ;

        bpmout = (Pixmap *)XtMalloc(sizeof(bpm)) ;
        memcpy(bpmout,&bpm,sizeof(bpm));
      
    	list = XVaCreateNestedList(0 ,
			XNFontSet, im_font ,
			XNArea, &geomRect ,
			XNBackground, bg ,   
			XNForeground, fg ,
			XNBackgroundPixmap, bpmout ,
			NULL );
    }    
#if defined(DEBUG)
    jio_fprintf(stderr,"awt_motif_getXICStatusAreaList:\n");
    jio_fprintf(stderr,"XNArea:x=%d,y=%d,width=%d,height=%d\n", \
         geomRect.x,geomRect.y,geomRect.width,geomRect.height);
    jio_fprintf(stderr,"XNBackground=0x%x\n",bg);
    jio_fprintf(stderr,"XNForeground=0x%x\n",fg);
    jio_fprintf(stderr,"XNBackgroundPixmap=0x%x\n",bpm);
#endif
    return list ;

}

#endif /* MOTIF_VERSION==2 */
