/*
 * @(#)awt_util.c	1.82 01/02/13
 *
 *  Copyright 2000-2001 Sun Microsystems, Inc. All rights reserved.
 *  Copyright 2000-2001 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "awt_p.h"
#include "color.h"
#ifdef __linux__
#include <X11/IntrinsicP.h>
#endif
#include <X11/Xatom.h>
#include <X11/Xmd.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <Xm/MenuShell.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/MwmUtil.h>


#include <jni.h>
#include <jni_util.h>


extern int xerror_handler();
static int winmgr_running = 0;
static Atom OLDecorDelAtom = 0;
static Atom MWMHints = 0;
static Atom DTWMHints = 0;
static Atom decor_list[9];

/*
 * The following three funtions are to work around menu problems
 */

/* 
 * test if there is a menu that has the current focus 
 * called from awt_Dialog.c and awt_Component.c 
 */
int
awt_util_focusIsOnMenu(Display *display)
{
  Window window;
  Widget widget;
  int rtr;

  XGetInputFocus(display, &window, &rtr);
  if (window == None) {
    return 0;
  }

  widget = XtWindowToWidget(display, window);
  if (widget == NULL) {
    return 0;
  }

  if (XtIsSubclass(widget, xmMenuShellWidgetClass)) {
    return 1;
  }
  if (XtIsSubclass(widget, xmRowColumnWidgetClass)) {
      unsigned char type;
      XtVaGetValues(widget, XmNrowColumnType, &type, NULL);
      if (type == XmMENU_BAR) {
	  return 1;
      }
  } 
  return 0;
}

static 
void fillButtonEvent(XButtonEvent *ev, int type, Display *display, Window window) {
    ev->type = type;
    ev->display = display;
    ev->window = window;
    ev->send_event = True;

    /* REMIND: multi-screen */ 
    ev->root = RootWindow(display, DefaultScreen(display));
    ev->subwindow = (Window)None;
    ev->time = CurrentTime;
    ev->x = 0;
    ev->y = 0;
    ev->x_root = 0;
    ev->y_root = 0;
    ev->same_screen = True;
    ev->button = Button1;
    ev->state = Button1Mask;
}

/* 
 * generates a mouse press event and a release event 
 * called from awt_Dialog.c
 */
int
awt_util_sendButtonClick(Display *display, Window window)
{
  XButtonEvent ev;
  int status;

  fillButtonEvent(&ev, ButtonPress, display, window);
  status = XSendEvent(display, window, True, ButtonPressMask, (XEvent *)&ev);

  if (status != 0) {
      fillButtonEvent(&ev, ButtonRelease, display, window);
      status = XSendEvent(display, window, False, ButtonReleaseMask,
			  (XEvent *)&ev);
  }
  return status;
}

/* 
 * create a mouse press and a release event, and dispatch them 
 * called from awt_Component.c
 */
void
awt_util_dispatchButtonClick(Display *display, Window window)
{
    XButtonEvent ev;
 
    fillButtonEvent(&ev, ButtonPress, display, window); 
    XtDispatchEvent((XEvent *)&ev);

    fillButtonEvent(&ev, ButtonRelease, display, window);
    XtDispatchEvent((XEvent *)&ev);
}

Widget
awt_util_createWarningWindow(Widget parent, char *warning)
{
    Widget warningWindow;
#ifdef NETSCAPE
    extern Widget FE_MakeAppletSecurityChrome(Widget parent, char* message);
    warningWindow = FE_MakeAppletSecurityChrome(parent, warning);
#else
    Widget label;
    int argc;
#define MAX_ARGC 10
    Arg args[MAX_ARGC];
    int screen = 0;
    int i;
    AwtGraphicsConfigDataPtr adata;
    extern int awt_numScreens;

    Pixel gray;
    Pixel black;

    for (i = 0; i < awt_numScreens; i++) {
        if (ScreenOfDisplay(awt_display, i) == XtScreen(parent)) {
            screen = i;
            break;
        }
    }
    adata = getDefaultConfig(screen);
    
    gray = adata->AwtColorMatch(192, 192, 192, adata);
    black = adata->AwtColorMatch(0, 0, 0, adata);

    argc = 0;
    XtSetArg(args[argc], XmNbackground, gray); argc++;
    XtSetArg(args[argc], XmNmarginHeight, 0); argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0); argc++;
    XtSetArg (args[argc], XmNscreen, XtScreen(parent)); argc++;

    DASSERT(!(argc > MAX_ARGC));
    warningWindow =  XmCreateForm(parent, "main", args, argc);

    XtManageChild(warningWindow);
    label = XtVaCreateManagedWidget(warning,
				    xmLabelWidgetClass, warningWindow,
				    XmNhighlightThickness, 0,
				    XmNbackground, gray,
				    XmNforeground, black,
				    XmNalignment, XmALIGNMENT_CENTER,
				    XmNrecomputeSize, False,
				    NULL);
    XtVaSetValues(label,
		  XmNbottomAttachment, XmATTACH_FORM,
		  XmNtopAttachment, XmATTACH_FORM,
		  XmNleftAttachment, XmATTACH_FORM,
		  XmNrightAttachment, XmATTACH_FORM,
		  NULL);
#endif
    return warningWindow;
}

void
awt_setWidgetGravity(Widget w, int gravity)
{
    XSetWindowAttributes    xattr;
    Window  win = XtWindow(w);

    if (win != 0) {
	xattr.bit_gravity = StaticGravity;
	xattr.win_gravity = StaticGravity;
	XChangeWindowAttributes(XtDisplay(w), win,
				CWBitGravity|CWWinGravity,
				&xattr);
    }
}

void
awt_util_reshape(Widget w, long x, long y, long wd, long ht) 
{
    Widget parent;
    Dimension ww, wh;
    Position wx, wy;
    Boolean move = False;
    Boolean resize = True;
    Boolean mapped_when_managed = False;
    Boolean need_to_unmanage = True;

    if (w == 0) {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        JNU_ThrowNullPointerException(env,"NullPointerException");
	return;
    }
    parent = XtParent(w);

    /* Aim: hack to prevent direct children of scrollpane from
     * being unmanaged during a reshape operation (which results
     * in too many expose events).  
     */
    if (parent != NULL && XtParent(parent) != NULL &&
        XtIsSubclass(XtParent(parent), xmScrolledWindowWidgetClass)) {
        need_to_unmanage = False;
    } 

    XtVaGetValues(w, 
		  XmNwidth, &ww, 
		  XmNheight, &wh, 
		  XmNx, &wx, 
		  XmNy, &wy, 
		  NULL);   

    if (x != wx || y != wy) {
        move = True;
    }
    if (wd != ww || ht != wh) {
        resize = True;
    }
    if (!move && !resize) {
        return;
    }

    if (need_to_unmanage) {    
        if (!resize) {
            mapped_when_managed = w->core.mapped_when_managed;
            w->core.mapped_when_managed = False;
        } 
        XtUnmanageChild(w);
    }
       
    /* GES: AVH's hack: 
     * Motif ignores attempts to move a toplevel window to 0,0.
     * Instead we set the position to 1,1. The expected value is
     * returned by Frame.getBounds() since it uses the internally
     * held rectangle rather than querying the peer.
     * N.B. [pauly, 9/97]  This is only required for wm shells
     * under the Motif Window Manager (MWM), not for any others.
     * Note. Utilizes C short-circuiting if w is not a wm shell.
     */
    if ((x == 0) && (y == 0) &&
        (XtIsSubclass(w, wmShellWidgetClass)) &&
        (XmIsMotifWMRunning(w))) {
	XtVaSetValues(w, XmNx, 1, XmNy, 1, NULL);
    }

    if (move && !resize) {
        XtVaSetValues(w, XmNx, x, XmNy, y, NULL);
    
    } else if (resize && !move) {
        XtVaSetValues(w, 
		      XmNwidth, (wd > 0) ? wd : 1,
		      XmNheight, (ht > 0) ? ht : 1,
		      NULL);

    } else  {
        XtVaSetValues(w,
		  XmNx, x,
		  XmNy, y,
		  XmNwidth, (wd > 0) ? wd : 1,
		  XmNheight, (ht > 0) ? ht : 1,
		  NULL);
    }

    if (need_to_unmanage) {    
        XtManageChild(w);
        if (!resize) {   
            w->core.mapped_when_managed = mapped_when_managed;
        }
    }   
}

void
awt_util_hide(Widget w) 
{
    if (w == 0) {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        JNU_ThrowNullPointerException(env,"NullPointerException");
	return;
    }
    XtSetMappedWhenManaged(w, False);
}

void
awt_util_show(Widget w) 
{
/*
    extern Boolean  scrollBugWorkAround;
*/
    if (w == 0) {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        JNU_ThrowNullPointerException(env,"NullPointerException");
	return;
    }
    XtSetMappedWhenManaged(w, True);
/*
  XXX: causes problems on 2.5
    if (!scrollBugWorkAround) {
	awt_setWidgetGravity(w, StaticGravity);
    }
*/
}

void
awt_util_enable(Widget w) 
{
    XtSetSensitive(w, True);
}

void
awt_util_disable(Widget w) 
{
    XtSetSensitive(w, False);
}

void
awt_util_mapChildren(Widget w, void (*func)(Widget,void *),
		     int applyToCurrent, void *data) {
    WidgetList			wlist;
    Cardinal			wlen = 0;
#ifdef __linux__
    Cardinal			i;
#else
    int				i;
#endif

    /* The widget may have been destroyed by another thread. */
    if ((w == 0) || (!XtIsObject(w)) || (w->core.being_destroyed))
        return;

    if (applyToCurrent != 0) {
	(*func)(w, data);
    }
    if (!XtIsComposite(w)) {
	return;
    }

    XtVaGetValues(w,
		  XmNchildren, &wlist,
		  XmNnumChildren, &wlen,
		  NULL);
    if (wlen > 0) {
	for (i=0; i < wlen; i++) {
	    awt_util_mapChildren(wlist[i], func, 1, data);
	}
    }
}

void
awt_changeAttributes(Display *dpy, Widget w, unsigned long mask,
		     XSetWindowAttributes *xattr)
{
#ifdef __linux__
    Cardinal		i;
#else
    int			i;
#endif
    WidgetList		wlist;
    Cardinal		wlen = 0;

    if (XtWindow(w) && XtIsRealized(w)) {
	XChangeWindowAttributes(dpy,
				XtWindow(w),
				mask,
				xattr);
    } else {
	return;
    }
    XtVaGetValues(w,
		  XmNchildren, &wlist,
		  XmNnumChildren, &wlen,
		  NULL);
    for (i = 0; i < wlen; i++) {
	if (XtWindow(wlist[i]) && XtIsRealized(wlist[i])) {
	    XChangeWindowAttributes(dpy,
				    XtWindow(wlist[i]),
				    mask,
				    xattr);
	}
    }
}

int
awt_util_setCursor(Widget w, Cursor c) {
    static Cursor prevCur = NULL;
    static Widget prevWgt = NULL;

    if (XtIsRealized(w)) {

	unsigned long 		valuemask = 0;
	XSetWindowAttributes	attributes;

	valuemask = CWCursor;
	if (prevWgt != NULL) {
	    attributes.cursor = NULL;
	    XChangeWindowAttributes(awt_display,
                        XtWindow(prevWgt),
                        valuemask,
                        &attributes);
	}

	if (c == NULL) {
	    c = prevCur;
	    prevWgt = w; 
	} else {
	    prevCur = c;
	    prevWgt = NULL;  
	}
	attributes.cursor = c;
	XChangeWindowAttributes(awt_display,
				XtWindow(w),
				valuemask,
				&attributes);				
	XFlush(awt_display);
	return 1;
    } else
        return 0;    
}

/*
  Part fix for bug id 4017222. Return the widget at the given screen coords
  by searching the widget tree beginning at root. This function will return
  null if the pointer is not over the root widget or child of the root widget.

  Additionally, this function will only return a Widget with non-nil XmNuserData.
  In 1.2.1, when the mouse was dragged over a Choice component, this function
  returned the GadgetButton associated with the Choice.  This GadgetButton had
  nil as its XmNuserData.  This lead to a crash when the nil XmNuserData was
  extracted and used as a reference to a peer.  Ooops.
  Now the GadgetButton is not returned and the function goes on to find a widget
  which contains the correct peer reference in XmNuserData. 
*/
Widget
awt_WidgetAtXY(Widget root, Position pointerx, Position pointery) {
  Widget answer = NULL;

  if(!root) return NULL;

  if(XtIsComposite(root)) {
    int i=0;
    WidgetList wl=0;
    Cardinal wlen=0;
 
    XtVaGetValues(root, XmNchildren, &wl, XmNnumChildren, &wlen, NULL);

    if(wlen>0) {
      for(i=0; i<wlen && !answer; i++) {
        answer = awt_WidgetAtXY(wl[i], pointerx, pointery);
      }
    }
  }

  if(!answer) {
    Position wx=0, wy=0;
    Dimension width=0, height=0;
    int lastx=0, lasty=0;
    XtPointer widgetUserData=0;
  
    XtVaGetValues(root, XmNwidth, &width, XmNheight, &height,
		  XmNuserData, &widgetUserData,
		  NULL);

    XtTranslateCoords(root, 0, 0, &wx, &wy);
    lastx = wx + width;
    lasty = wy + height;

    if(pointerx>=wx && pointerx<=lastx && pointery>=wy && pointery<=lasty &&
	   widgetUserData)
        answer = root;
  }

  return answer;
}
#ifdef __linux__


#define MAXARGS 10
static Arg xic_vlist[MAXARGS];
static Arg status_vlist[MAXARGS];
static Arg preedit_vlist[MAXARGS];

#define NO_ARG_VAL -1
#define SEPARATOR_HEIGHT 2

static XFontSet extract_fontset(XmFontList);

/* get_im_height: returns height of the input method status area in pixels.
 *
 * This function assumes that if any XIM related information cannot be
 * queried then the app must not have an input method status area in the
 * current locale and returns zero as the status area height
 */

static XtPointer*
get_im_info_ptr(Widget  w,
                Boolean create)
{
  Widget p;
  XmVendorShellExtObject ve;
  XmWidgetExtData extData;
  XmImShellInfo im_info;
  XmImDisplayInfo xim_info;

  if (w == NULL)
    return NULL;

  p = w;
  while (!XtIsShell(p))
    p = XtParent(p);

  /* Check extension data since app could be attempting to create
   * a text widget as child of menu shell.  This is illegal, and will
   * be detected later, but check here so we don't core dump.
   */
  if ((extData = _XmGetWidgetExtData((Widget)p, XmSHELL_EXTENSION)) == NULL)
    return NULL;

  ve = (XmVendorShellExtObject) extData->widget;

  return &ve->vendor.im_info;
}

static XmImShellInfo
get_im_info(Widget w,
            Boolean create)
{
  XmImShellInfo* ptr = (XmImShellInfo *) get_im_info_ptr(w, create);
  if (ptr != NULL)
    return *ptr;
  else
    return NULL;
}

int
awt_util_getIMStatusHeight(Widget vw)
{
  XmImXICInfo icp,this_icp=NULL;
  XmVendorShellExtObject ve;
  XmWidgetExtData extData;
  int height = 0;
  XRectangle rect;
  XRectangle *rp = NULL;
  XmImShellInfo im_info;
  XVaNestedList set_list, get_list;

  extData = _XmGetWidgetExtData((Widget)vw, XmSHELL_EXTENSION);
  ve = (XmVendorShellExtObject) extData->widget;

  im_info = get_im_info(vw, False);

  if ((im_info == NULL) || (im_info->iclist == NULL)) {
    ve->vendor.im_height = 0;
    return 0;
  }

  set_list = XVaCreateNestedList(0, XNAreaNeeded, (XPointer) &rect, NULL);
  get_list = XVaCreateNestedList(0, XNAreaNeeded, (XPointer) &rp, NULL);

  for (icp = this_icp ? this_icp : im_info->iclist;
       icp != NULL;
       icp = icp->next) {
    if (icp->xic) {
      if (icp->input_style & XIMStatusArea) {
        rect.width = vw->core.width;
        rect.height = 0;

        XSetICValues(icp->xic, XNStatusAttributes, set_list, NULL);
        XGetICValues(icp->xic, XNStatusAttributes, get_list, NULL);

        if (!rp) {  /* fix for bug 4201602 - leob */
           ve->vendor.im_height = 0;
           return 0;
        }
        if ((int) rp->height > height)
          height = rp->height;

        icp->status_width = MIN(rp->width, vw->core.width);
        icp->sp_height = rp->height;
          XFree(rp);
      }
      if (icp->input_style & XIMPreeditArea) {
        rect.width = vw->core.width;
        rect.height = 0;

        XSetICValues(icp->xic, XNPreeditAttributes, set_list, NULL);
        XGetICValues(icp->xic, XNPreeditAttributes, get_list, NULL);

        if ((int) rp->height > height)
          height = rp->height;

        icp->preedit_width = MIN((int) rp->width,
                                 (int) (vw->core.width - icp->status_width));
        if (icp->sp_height < (int) rp->height)
          icp->sp_height = rp->height;
        XFree(rp);
      }
    }

    if (this_icp)
      break;
  }

  XFree(set_list);
  XFree(get_list);

  if (height)
    height += SEPARATOR_HEIGHT;

  ve->vendor.im_height = height;
  return height;
}

#endif /* !linux */

Widget
awt_util_getXICStatusAreaWindow(Widget w)
{
    while (!XtIsShell(w)){
	w = XtParent(w);
    }
    return w;
}

#ifdef __linux__
static XRectangle geometryRect;
XVaNestedList awt_util_getXICStatusAreaList(Widget w)
{
    XIC xic;
    XmImXICInfo icp;
    XmVendorShellExtObject ve;
    XmWidgetExtData extData;
    XmImShellInfo im_info;
    XmFontList fl=NULL;

    XRectangle  *ssgeometry = &geometryRect;
    XRectangle geomRect ;
    XRectangle *im_rect;
    XFontSet   *im_font;

    Pixel bg ;
    Pixel fg ;
    Dimension height, width ;
    Position x,y ;
    Pixmap bpm, *bpmout ;

    XVaNestedList list = NULL;

    char *ret;
    Widget p=w;

    while (!XtIsShell(p)) {
        p = XtParent(p);
    }

    XtVaGetValues(p,
        XmNx, &x,
        XmNy, &y,
        XmNwidth, &width,
        XmNheight, &height,
        XmNbackgroundPixmap, &bpm,
        NULL);

    extData = _XmGetWidgetExtData((Widget) p, XmSHELL_EXTENSION);
    if (extData == NULL) {
        return NULL;
    }
    ve = (XmVendorShellExtObject) extData->widget;
    im_info = get_im_info(w, False);

    if (im_info == NULL) {
        return NULL;
    } else {
        icp = im_info->iclist;
    }

    if (icp) {
        /*
         * We hava at least a textfield/textarea in the frame, use the
         * first one.
         */
        ssgeometry->x = 0;
        ssgeometry->y = height - icp->sp_height;
        ssgeometry->width = icp->status_width;
        ssgeometry->height = icp->sp_height;
        XtVaGetValues(w, XmNbackground, &bg, NULL);
        XtVaGetValues(w, XmNforeground, &fg, NULL);
        XtVaGetValues(w, XmNfontList, &fl, NULL);
        /*
         * use motif TextComponent's resource
         */

        list = XVaCreateNestedList(0,
                        XNFontSet, extract_fontset(fl),
                        XNArea, ssgeometry,
                        XNBackground, bg,
                        XNForeground, fg,
                        NULL);
   }
   return list ;
}



static XFontSet
extract_fontset(XmFontList fl)
{
    XmFontContext context;
    XmFontListEntry next_entry;
    XmFontType type_return;
    XtPointer tmp_font;
    XFontSet first_fs = NULL;
    char *font_tag;

    if (!XmFontListInitFontContext(&context, fl))
	return NULL;

    do {
	next_entry = XmFontListNextEntry(context);
	if (next_entry) {
	    tmp_font = XmFontListEntryGetFont(next_entry, &type_return);
	    if (type_return == XmFONT_IS_FONTSET) {
		font_tag = XmFontListEntryGetTag(next_entry);
		if (!strcmp(font_tag, XmFONTLIST_DEFAULT_TAG)) {
		    XmFontListFreeFontContext(context);
		    XtFree(font_tag);
		    return (XFontSet) tmp_font;
		}
		XtFree(font_tag);
		if (first_fs == NULL)
		    first_fs = (XFontSet) tmp_font;
	    }
	}
    } while (next_entry);

    XmFontListFreeFontContext(context);
    return first_fs;
}
#endif

/* Sets or unsets minimum and maximum size properties of the
   specified shell widget, according to the value of `set'
   parameter (0 means unset)                                  */

void
awt_util_setMinMaxSizeProps(Widget shellW, Boolean set)
{
    Display* disp = XtDisplay(shellW);
    Window win = XtWindow(shellW);
    XSizeHints* hints = XAllocSizeHints();
    long k = 0;

    XGetWMNormalHints(disp, win, hints, &k);
    if (set) hints->flags |= (PMinSize | PMaxSize);

    else hints->flags &= ~(PMinSize | PMaxSize);
    XSetWMNormalHints(disp, win, hints);
    XFree(hints);
}

/* For a specified shell widget, make it resizable by setting its
   minimum and maximum width and height to the screen size range.
   Also, set its window manager decorations and menu accordingly.	*/

void
awt_util_setShellResizable( Widget shellW, Boolean isMapped )
{
    int		screenWidth;
    int		screenHeight;
    Atom	OLDecorAddAtom;
    Atom	OLDecorResizeAtom;
    unsigned	wm;    /* window manager */

    /* Iff shell is mapped, we must unmap and later remap in order to
       effect the changes we make in the window manager decorations.
       N.B.This unmapping / remapping of the shell exposes a bug in
	   X/Motif or the Motif Window Manager.  When you attempt
           to map a widget which is positioned (partially) offscreen,
           the window is relocated to be entirely on screen. Good
           idea.  But if both the x and the y coordinates are less
           than the origin (0,0), the first (re)map will move the
           window to the origin, and any subsequent (re)map will
           relocate the window at some other point on the screen.
           I have written a short Motif test program to discover
           this bug.  Nevertheless, I know of no way to effect the
           changes in the wm decorations other than an unmap and
           remap.  Also, this should occur infrequently and it does
           not cause any real problem.  So for now we'll let it be.     */

    /* Setting the decorations only works under dtwm and mwm.  Decorations
       should be the preferred way to implement non-resizability */

    wm = awt_util_runningWindowManager();
    if ((wm == CDE_WM) || (wm == MOTIF_WM))
    {
        if (isMapped) {
            XtPopdown(shellW);
        }
	XtVaSetValues(shellW,
		XmNmwmDecorations, MWM_DECOR_ALL,
		XmNmwmFunctions, MWM_FUNC_ALL,
		NULL);
        if (isMapped) {
            XtPopup(shellW, XtGrabNone);
        }
    }
    else
    {
	screenWidth = XWidthOfScreen(XDefaultScreenOfDisplay(awt_display));
	screenHeight = XHeightOfScreen(XDefaultScreenOfDisplay(awt_display));
        /*
         * Fix for BugTraq ID 4313607.
         * For Enlightenment WM we must unmap and later remap so that
         * the WM will be able to update decorations.
         */
        if ((wm == ENLIGHTEN_WM) && isMapped) {
            XtPopdown(shellW);
        }
	XtVaSetValues(shellW,
		XmNminWidth, 0,
		XmNmaxWidth, screenWidth,
		XmNminHeight, 0,
		XmNmaxHeight, screenHeight,
		NULL);
	awt_util_setMinMaxSizeProps(shellW, 0);
        if ((wm == ENLIGHTEN_WM) && isMapped) {
            XtPopup(shellW, XtGrabNone);
        }
    }

    /* To set OLWM decorations, we must use the appropriate properties.
     * N.B. It is unfortunate that the OLWM does not allow us to adjust
     * these properties dynamically; the following can not be used here.
     * OLDecorAddAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_ADD", False);
     * OLDecorResizeAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_RESIZE", 
     *                  False);
     * XChangeProperty( XtDisplay(shellW), XtWindow(shellW), OLDecorAddAtom,
     *			XA_ATOM, 32, PropModeReplace,
     *			(unsigned char *)&OLDecorResizeAtom, 1 );
     */
    /* How would we set the appropriate decorations for other WM's ?!	*/
}


/* For a specified shell widget, make it nonresizable by fixing its
   minimum and maximum widths and heights to the values specified.
   Also adjust the window manager decorations and menu accordingly.	*/

#ifdef __linux__
void
awt_util_setShellNotResizable( Widget shellW, long width, long height, Boolean isMapped )
{
    Atom	OLDecorResizeAtom;
    unsigned	wm;    /* window manager */

    /* Iff shell is mapped, we must unmap and later remap in order to
       effect the changes we make in the window manager decorations.
       N.B. See the corresponding comment above in setShellResizable().	*/

    /* Setting the decorations only works under dtwm and mwm.  This should */
    /* be the preferred way to implement non-resizability */

    wm = awt_util_runningWindowManager();
    if ((wm == CDE_WM) || (wm == MOTIF_WM))
    {
	if  (isMapped)
	    XtUnmapWidget(shellW);
	XtVaSetValues(shellW,
		XmNmwmDecorations,
                        MWM_DECOR_BORDER | MWM_DECOR_TITLE | 
			MWM_DECOR_MENU | MWM_DECOR_MINIMIZE,
		XmNmwmFunctions, 
			MWM_FUNC_MOVE | MWM_FUNC_CLOSE | MWM_FUNC_MINIMIZE,
		NULL);
	if  (isMapped)
	    XtMapWidget(shellW);
    }
    else
    {
	XtVaSetValues(shellW,
		XmNminWidth, (XtArgVal)width,
		XmNmaxWidth, (XtArgVal)width,
		XmNminHeight, (XtArgVal)height,
		XmNmaxHeight, (XtArgVal)height,
		NULL);
    }

    /* To set OLWM decorations, we must use the appropriate properties.
     * N.B. It is unfortunate that the OLWM does not allow us to adjust
     * these properties dynamically; the following can not be used here.
     * OLDecorDelAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_DEL", False);
     * OLDecorResizeAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_RESIZE", 
     *                  False);
     * XChangeProperty( XtDisplay(shellW), XtWindow(shellW), OLDecorDelAtom,
     *			XA_ATOM, 32, PropModeReplace,
     *			(unsigned char *)&OLDecorResizeAtom, 1 );
     */
    /* How would we set the appropriate decorations for other WM's ?!	*/
}
#else
void
awt_util_setShellNotResizable( Widget shellW, long width, long height, 
                               Boolean isMapped, Boolean justChangeSize )
{
    Atom	OLDecorResizeAtom;
    unsigned	wm;    /* window manager */

    /* Iff shell is mapped, we must unmap and later remap in order to
       effect the changes we make in the window manager decorations.
       N.B. See the corresponding comment above in setShellResizable().	*/

    /* Setting the decorations only works under dtwm and mwm.  This should */
    /* be the preferred way to implement non-resizability */

    wm = awt_util_runningWindowManager();
    if ((wm == CDE_WM) || (wm == MOTIF_WM))
    {
        /* 
         * Remap shell only when resizability changes.
         * Don't remap after each reshape.
         */
        if (!justChangeSize) {
            if  (isMapped) {
                XtPopdown(shellW);
            }
            XtVaSetValues(shellW,
                          XmNmwmDecorations,
                          MWM_DECOR_BORDER | MWM_DECOR_TITLE | 
                          MWM_DECOR_MENU | MWM_DECOR_MINIMIZE,
                          XmNmwmFunctions, 
                          MWM_FUNC_MOVE | MWM_FUNC_CLOSE | MWM_FUNC_MINIMIZE,
                          NULL);
            if  (isMapped) {
                XtPopup(shellW, XtGrabNone);
            }
        }
    }
    else if ((width > 0) && (height > 0)) 
    {
        /*
         * Fix for BugTraq ID 4313607.
         * For Enlightenment WM we must remap so that the WM will 
         * be able to update decorations on resizability change.
         */
        if ((wm == ENLIGHTEN_WM) && isMapped && !justChangeSize) {
            XtPopdown(shellW);
        }
	XtVaSetValues(shellW,
		XmNminWidth, (XtArgVal)width,
		XmNmaxWidth, (XtArgVal)width,
		XmNminHeight, (XtArgVal)height,
		XmNmaxHeight, (XtArgVal)height,
		NULL);
        if ((wm == ENLIGHTEN_WM) && isMapped && !justChangeSize) {
            XtPopup(shellW, XtGrabNone);
        }
    }

    /* To set OLWM decorations, we must use the appropriate properties.
     * N.B. It is unfortunate that the OLWM does not allow us to adjust
     * these properties dynamically; the following can not be used here.
     * OLDecorDelAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_DEL", False);
     * OLDecorResizeAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_RESIZE", 
     *                  False);
     * XChangeProperty( XtDisplay(shellW), XtWindow(shellW), OLDecorDelAtom,
     *			XA_ATOM, 32, PropModeReplace,
     *			(unsigned char *)&OLDecorResizeAtom, 1 );
     */
    /* How would we set the appropriate decorations for other WM's ?!	*/
}
#endif

/*
** Set the decoration attributes for a top-level openlook window
*/
void setOlDecorHint( Widget shellW, int decor )
{
    PropMwmHints *pMwm;
    Atom        actual_type;
    int         actual_format;
    unsigned long nitems;
    unsigned long bytes_after;
    unsigned char *tmp = NULL;
    int ret;

    if ( MWMHints == NULL )
        MWMHints = XInternAtom( XtDisplay(shellW), "_MOTIF_WM_HINTS", True);
    if ( OLDecorDelAtom == NULL )
        OLDecorDelAtom = XInternAtom( XtDisplay(shellW), "_OL_DECOR_DEL", True);    
    if ( decor_list[0] == NULL )
        decor_list[0] = XInternAtom(XtDisplay(shellW),"_OL_DECOR_RESIZE", True);    
    if ( decor_list[1] == NULL )
        decor_list[1] = XInternAtom(XtDisplay(shellW),"_OL_DECOR_HEADER", True);    
    if ( decor_list[2] == NULL )
        decor_list[2] = XInternAtom(XtDisplay(shellW),"_OL_DECOR_CLOSE", True);    
    if ( decor_list[3] == NULL )
        decor_list[3] = XInternAtom(XtDisplay(shellW),"_OL_DECOR_PIN", True);    
    if ( OLDecorDelAtom != NULL  && decor_list[0] != NULL )
    {
        XChangeProperty( XtDisplay(shellW),
                         XtWindow(shellW),
                         OLDecorDelAtom,
                         XA_ATOM, 32,
                         PropModeReplace,
                         (unsigned char *)decor_list,
                         decor ? 0 : 4 );
        if ( MWMHints != NULL )
        {
            ret = XGetWindowProperty ( XtDisplay(shellW),
			XtWindow( shellW),
			MWMHints,
			0,
			PROP_MOTIF_WM_HINTS_ELEMENTS,
			False,
			AnyPropertyType,
			&actual_type,
			&actual_format,
			&nitems,
			&bytes_after,
			&tmp );

	    if ( ( tmp != NULL ) && ( ret == Success ) ) {
		pMwm = (PropMwmHints *) tmp;
		pMwm->flags = MWM_HINTS_DECORATIONS | MWM_HINTS_FUNCTIONS;
		pMwm->decorations = decor;
	        if( decor & MWM_DECOR_RESIZEH ) {
        	    pMwm->functions |= MWM_FUNC_RESIZE;
    	        } else {
        	    pMwm->functions &= ~MWM_FUNC_RESIZE;
		}
    	        if( decor & MWM_DECOR_TITLE ) {
        	    pMwm->functions |= MWM_FUNC_MOVE;
    	        } else {
        	    pMwm->functions &= ~MWM_FUNC_MOVE;
		}
    	        if( decor & MWM_DECOR_MINIMIZE ) {
        	    pMwm->functions |= MWM_FUNC_MINIMIZE;
    	        } else {
        	    pMwm->functions &= ~MWM_FUNC_MINIMIZE;
		}
    	        if( decor & MWM_DECOR_MAXIMIZE ) {
        	    pMwm->functions |= MWM_FUNC_MAXIMIZE;
    	        } else {
        	    pMwm->functions &= ~MWM_FUNC_MAXIMIZE;
		}
     
		XChangeProperty( XtDisplay(shellW),
			XtWindow(shellW),
			MWMHints,
			actual_type,
			32,
			PropModeReplace,
			(unsigned char *) pMwm,
			PROP_MOTIF_WM_HINTS_ELEMENTS );
	    }
	    if ( tmp != NULL ) {
		XFree(tmp);
	    }
        }
    }    
}



int
awt_util_setWindowDecor( Widget shellW, int decor )
{
    int old_decor = 0;
    PropMwmHints *pMwm, mph;
    Atom        actual_type;
    int         actual_format;
    unsigned long nitems;
    unsigned long bytes_after;
    unsigned char *tmp = NULL;
    int ret;
    Window w;

    w = XtWindow( shellW );
    if ( awt_util_runningWindowManager() == OPENLOOK_WM )
    {   
        setOlDecorHint( shellW, decor );
    } 
    if ( MWMHints == NULL )
        MWMHints = XInternAtom( awt_display, "_MOTIF_WM_HINTS", False);
    if ( DTWMHints == NULL )
        DTWMHints = XInternAtom( awt_display, "_DT_WORKSPACE_HINTS", False);

    ret = XGetWindowProperty ( awt_display,
			       w,
			       MWMHints,
			       0,
			       PROP_MOTIF_WM_HINTS_ELEMENTS,
			       False,
			       AnyPropertyType,
			       &actual_type,
			       &actual_format,
			       &nitems,
			       &bytes_after,
			       &tmp );

    if ( ( tmp == NULL ) || ( ret != Success ) ) 
    {
    ret = XGetWindowProperty ( awt_display,
			       w,
			       DTWMHints,
			       0,
			       PROP_MOTIF_WM_HINTS_ELEMENTS,
			       False,
			       AnyPropertyType,
			       &actual_type,
			       &actual_format,
			       &nitems,
			       &bytes_after,
			       &tmp );

    }
    if ( ( tmp != NULL ) && ( ret == Success ) ) 
    {
#if 0
jio_fprintf(stdout, "Decorations Atom is PRESENT decor = %x\n", decor );
#endif
	pMwm = (PropMwmHints *) tmp;
	old_decor = pMwm->decorations;
        pMwm->decorations = decor;
	pMwm->flags |= MWM_HINTS_DECORATIONS;
 
        XChangeProperty( awt_display,
		         w,
		         MWMHints,
		         actual_type,
		         32,
		         PropModeReplace,
		         (unsigned char *) pMwm,
		         PROP_MOTIF_WM_HINTS_ELEMENTS 
	               );
    }
    else
    {
#if 0
jio_fprintf(stdout, "Decorations Atom is NOT PRESENT we are setting. Decor = %x\n", decor );
#endif
	mph.flags = MWM_HINTS_DECORATIONS;
	mph.functions = 0;
	mph.decorations = decor;
	mph.inputMode = 0;
	mph.status = 0;
        XChangeProperty( awt_display,
		         w,
		         MWMHints,
		         MWMHints,
		         32,
		         PropModeReplace,
		         (unsigned char *) &mph,
		         PROP_MOTIF_WM_HINTS_ELEMENTS 
	               );
        XChangeProperty( awt_display,
                         w,
                         DTWMHints,
                         DTWMHints,
                         32,
                         PropModeReplace,
                         (unsigned char *) &mph,
                         PROP_MOTIF_WM_HINTS_ELEMENTS
                       );
    }
    return old_decor;
}


/* 
** awt_winmgrerr: temporary XError handler setup to determine if 
** 		  any windowow manager is running 
*/
static int
awt_winmgrerr( Display *display, XErrorEvent *error )
{
    if ( error->request_code == X_ChangeWindowAttributes )
    {
	winmgr_running = TRUE;
    }
    return 0;
}

/* If a window mgr implements the insets property 
** Retrun the value of that property in a as Rectangle
*/

unsigned get_awt_TopLevelInsetsProp(  Window w, int *top, int *left, int *bottom, int *right )
{
    Atom           InsetAtom;
    int	           wm;
    int            ret;
    Atom           ret_type;
    int            ret_format;
    unsigned long  ret_count;
    unsigned long  ret_bar;
    Cardinal 	   *prop = NULL;

    /* at the moment enlightenment is the only WM employing an inset property */
    wm = awt_util_runningWindowManager();
    switch ( wm) {
	case ENLIGHTEN_WM:
	   InsetAtom = XInternAtom ( awt_display, "_E_FRAME_SIZE", TRUE );
	   ret = XGetWindowProperty ( awt_display, 
			 	      w, 
			 	      InsetAtom,
			 	      0, 
			 	      4, 
			 	      FALSE, 
			 	      XA_CARDINAL, 
			 	      &ret_type, 
			 	      &ret_format, 
			 	      &ret_count, 
			 	      &ret_bar, 
			 	      (unsigned char **)&prop
			            );
	    /* Prop format is left, right, top, bottom */
	    if (ret == Success && prop != NULL ) {
		*left = prop[0];
		*right = prop[1];
		*top = prop[2];
		*bottom = prop[3];
		return TRUE;
	    }
	    break;
	default:
		break;
    }
    return False;
}

/*
 * Some WMs take the Xy position to be the absolute Position 
 * of the client area, other don't.  This affects the geometry
 * calculation...return yea or nea
 */

unsigned awt_util_wmPositionsShellXY()
{
    unsigned wm = awt_util_runningWindowManager();

    switch ( wm ) {
    case ENLIGHTEN_WM:
	return TRUE;
    default:
	break;
   }
   return FALSE;
}

/* Determine current running Window Manager (Motif, OpenLook, or other)	*/
unsigned awt_util_runningWindowManager()
{
    Atom      WMAtom;
    static int wmgr = WM_YET_TO_BE_DETERMINED;
    unsigned long       nItems;
    unsigned long       bytesAfter;
    Status      status;
    Atom        actualType;
    int         actualFormat;
    unsigned char       *ptr;
    XSetWindowAttributes attrib;
    XtErrorHandler oldHandler;
    const char *vendor_string;


    if ( wmgr != WM_YET_TO_BE_DETERMINED )
    {
	return wmgr;
    }

    vendor_string = ServerVendor( awt_display );
    if ( strstr(vendor_string, "eXcursion") != NULL )
    {
      /* use NO_WM since in all other aspects eXcursion is like not having
      ** a window manager running. ie: It does not reparent top level shells.
      */
            wmgr = NO_WM;
            return wmgr;
    } 

    /* Is any window manager is running? 
    ** It would appear we must make this test first as the winmgr
    ** (at least dtwm) when killed still appears to be running 
    ** presumably as its distinguishing atoms are still lying around
    */
    /* if any window manager is running? */
    XSetErrorHandler(awt_winmgrerr);

    winmgr_running = 0;
    attrib.event_mask = SubstructureRedirectMask;
    XChangeWindowAttributes ( awt_display,
                              DefaultRootWindow(awt_display),
                              CWEventMask,
                              &attrib
                            );
    XSync( awt_display, FALSE );
    (void) XSetErrorHandler(xerror_handler);

    if (!winmgr_running) {
	wmgr = NO_WM;
	attrib.event_mask = 0;
	XChangeWindowAttributes(awt_display,
				DefaultRootWindow(awt_display),
				CWEventMask,
				&attrib);
    } else if ((WMAtom = XInternAtom(awt_display, "ENLIGHTENMENT_COMMS", TRUE))) {
        wmgr = ENLIGHTEN_WM;
#ifdef __linux__
    } else if ((WMAtom = XInternAtom(awt_display, "_ICEWM_WINOPTHINT",
                                     TRUE))) {
        wmgr = ICE_WM;
    } else if ((WMAtom = XInternAtom(awt_display, "_SAWMILL_TIMESTAMP",
                                     TRUE))) {
        wmgr = SAWFISH_WM;
    } else if ((WMAtom = XInternAtom(awt_display, "KWIN_RUNNING",
                                     TRUE))) {
        wmgr = KDE2_WM;
#endif
    } else if ((WMAtom = XInternAtom(awt_display, "_DT_SM_WINDOW_INFO", TRUE))) {
        wmgr = CDE_WM;
    } else  if ((WMAtom = XInternAtom(awt_display, "_MOTIF_WM_INFO", TRUE))) {
        status = XGetWindowProperty (awt_display, 
				     DefaultRootWindow(awt_display),
                                     WMAtom, 0, 1, False,
                                     WMAtom, &actualType,
                                     &actualFormat, &nItems, &bytesAfter,
                                     &ptr);
 
        if (status == Success && actualFormat != 0) {
            XFree((char *) ptr);
            wmgr = MOTIF_WM;
        } else if ( WMAtom = XInternAtom (awt_display,
                           "_SUN_WM_PROTOCOLS" , TRUE) ) {
            wmgr = OPENLOOK_WM;
        } else {
    	    wmgr = OTHER_WM;
        }
    } else if ((WMAtom = XInternAtom(awt_display,"_SUN_WM_PROTOCOLS",
					TRUE))) {
	 	wmgr = OPENLOOK_WM;
    } else  {
    	wmgr = OTHER_WM;
    }
    return(wmgr);
}


/*the caller does have the responsibility to free the memory return
  from this function...*/ 
char* awt_util_makeWMMenuItem(char *target, Atom protocol){
    char        *buf = NULL;
    int         buflen = 0;

    /*a label in a menuitem is not supposed to be a FullOfSpaceString... */
    buflen = strlen(target) * 3;
    buf = (char*)malloc(buflen + 20);
    if (buf == NULL){
	JNU_ThrowOutOfMemoryError((JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2), NULL);
    }
    else{
      int   off = 0;
      char  *ptr = target;
      while ((off < (buflen - 20)) && (*ptr != '\0')){
	if (*ptr == ' '){
	  *(buf + off++) = 0x5c;
	}
	*(buf + off++) = *ptr++;
      }
      sprintf(buf + off, " f.send_msg %ld", protocol);
    }
    return buf;
}

/* 
 * This callback proc is installed via setting the XmNinsertPosition
 * resource on a widget. It ensures that components added
 * to a widget are inserted in the correct z-order position
 * to match up with their peer/target ordering in Container.java
 */
Cardinal 
awt_util_insertCallback(Widget w) 
{
    jobject peer;
    WidgetList children;
    int num_children;
    Widget parent;
    XtPointer userdata;
    Cardinal index;
    long pos;
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    parent = XtParent(w);
    XtVaGetValues(parent,
                  XmNnumChildren, &num_children,
                  XmNchildren, &children,
                  NULL);
    XtVaGetValues(w, XmNuserData, &userdata, NULL);

    index = num_children;         /* default is to add to end */

    if (userdata != NULL) {
        peer = (jobject) userdata;

	// SECURITY: We are running on the privileged toolkit thread.
	//           The peer must *NOT* call into user code
        pos = (long) JNU_CallMethodByName(env
					  ,NULL
					  ,(jobject) peer
					  ,"getZOrderPosition_NoClientCode"
					  ,"()I").i;
        if ((*env)->ExceptionOccurred(env)) {
            (*env)->ExceptionDescribe(env);
            (*env)->ExceptionClear(env);
        }
        index = (Cardinal) (pos != -1 ? pos : num_children);
    }
    return index;
}

void
awt_util_consumeAllXEvents(Widget widget)
{
    /* Remove all queued X Events for the window of the widget. */

#define ALL_EVENTS_MASK 0xFFFF

    XEvent xev;

    XFlush(awt_display);
    while (XCheckWindowEvent(awt_display, XtWindow(widget),
               ALL_EVENTS_MASK, &xev)) ;
}

/**
 * Gets the thread we are currently executing on
 */
jobject
awtJNI_GetCurrentThread(JNIEnv *env) {
    static jclass threadClass = NULL;
    static jmethodID currentThreadMethodID = NULL;

    jobject currentThread = NULL;

    /* Initialize our java identifiers once. Checking before locking 
     * is a huge performance win.
     */
    if (threadClass == NULL) {
	// should enter a monitor here...
	Boolean err = FALSE;
	if (threadClass == NULL) {
	    jclass tc = (*env)->FindClass(env, "java/lang/Thread");
	    threadClass = (*env)->NewGlobalRef(env, tc);
	    if (threadClass != NULL) {
	        currentThreadMethodID = (*env)->GetStaticMethodID(env,
					      threadClass,
					      "currentThread",
					      "()Ljava/lang/Thread;"
		    				);
	    }
	}
	if (currentThreadMethodID == NULL) {
	    threadClass = NULL;
	    err = TRUE;
	}
	if (err) {
	    return NULL;
	}
    } /* threadClass == NULL*/

    currentThread = (*env)->CallStaticObjectMethod(
			env, threadClass, currentThreadMethodID);
    DASSERT(!((*env)->ExceptionOccurred(env)));
    /*JNU_PrintString(env, "getCurrentThread() -> ", JNU_ToString(env,currentThread));*/
    return currentThread;
} /* awtJNI_GetCurrentThread() */

void
awtJNI_ThreadYield(JNIEnv *env) {

    static jclass threadClass = NULL;
    static jmethodID yieldMethodID = NULL;

    /* Initialize our java identifiers once. Checking before locking 
     * is a huge performance win.
     */
    if (threadClass == NULL) {
	// should enter a monitor here...
	Boolean err = FALSE;
	if (threadClass == NULL) {
	    threadClass = (*env)->FindClass(env, "java/lang/Thread");
	    if (threadClass != NULL) {
	        yieldMethodID = (*env)->GetStaticMethodID(env,
					      threadClass,
					      "yield",
					      "()V"
		    				);
	    }
	}
	if (yieldMethodID == NULL) {
	    threadClass = NULL;
	    err = TRUE;
	}
	if (err) {
	    return;
	}
    } /* threadClass == NULL*/

    (*env)->CallStaticVoidMethod(env, threadClass, yieldMethodID);
    DASSERT(!((*env)->ExceptionOccurred(env)));
} /* awtJNI_ThreadYield() */


void
awt_util_cleanupBeforeDestroyWidget(Widget widget)
{
    /* Bug 4017222: Drag processing uses global prevWidget. */
    if (widget == prevWidget) prevWidget = 0;
}

static Boolean timeStampUpdated = False;

static int
isTimeStampUpdated(void* p) {
    return timeStampUpdated;
}

static void
propertyChangeEventHandler(Widget w, XtPointer client_data, 
			   XEvent* event, Boolean* continue_to_dispatch) {
    timeStampUpdated = True;
}

/*
 * If the application doesn't receive events with timestamp for a long time
 * XtLastTimestampProcessed() will return out-of-date value. This may cause 
 * selection handling routines to fail (see BugTraq ID 4085183).
 * This routine is to resolve this problem. It queries the current X server 
 * time by appending a zero-length data to a property as prescribed by 
 * X11 Reference Manual.
 * Note that this is a round-trip request, so it can be slow. If you know 
 * that the Xt timestamp is up-to-date use XtLastTimestampProcessed().
 */
Time 
awt_util_getCurrentServerTime(Widget w) {

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    static Atom _XA_JAVA_TIME_PROPERTY_ATOM = 0;

    if (_XA_JAVA_TIME_PROPERTY_ATOM == 0) {
        XtAddEventHandler(w, PropertyChangeMask, False,
                          propertyChangeEventHandler, NULL);
        _XA_JAVA_TIME_PROPERTY_ATOM = XInternAtom(awt_display, "_SUNW_JAVA_AWT_TIME", False);
    }

    timeStampUpdated = False;
    XChangeProperty(awt_display, XtWindow(w), _XA_JAVA_TIME_PROPERTY_ATOM,
		    XA_ATOM, 32, PropModeAppend, (unsigned char *)"", 0);
    XFlush(awt_display);

    if (awt_currentThreadIsPrivileged(env)) {
        XEvent event;
        XMaskEvent(awt_display, PropertyChangeMask, &event);
        XtDispatchEvent(&event);
    } else {
        awt_MToolkit_modalWait(isTimeStampUpdated, NULL);
    }
    return XtLastTimestampProcessed(awt_display);
}

void awt_util_debug_init() {
#if defined(DEBUG)    
    DTrace_Initialize();
#endif
}

static void awt_util_debug_fini() {
#if defined(DEBUG)
    DTrace_Shutdown();
#endif
}


