/*
 * @(#)canvas.c	1.173 01/03/01
 *
 *  Copyright 2000-2001 Sun Microsystems, Inc. All rights reserved.
 *  Copyright 2000-2001 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "awt_p.h"
#include <sys/time.h> /* timeval */
#define XK_KATAKANA
#include <X11/keysym.h>
#include <Xm/Display.h>
#include <ctype.h>
#include "java_awt_Frame.h"
#include "java_awt_Component.h"
#include "java_awt_AWTEvent.h"
#include "java_awt_event_KeyEvent.h"
#include "java_awt_event_FocusEvent.h"
#include "java_awt_event_MouseEvent.h"
#include "java_awt_event_InputEvent.h"
#include "sun_awt_motif_MComponentPeer.h"
#include "color.h"
#include "canvas.h"
#include "awt_Cursor.h"
#include "VDrawingArea.h"

#include "awt_Component.h"
#include "awt_AWTEvent.h"
#include "awt_Event.h"

#include <jni.h>
#include <jni_util.h>
#include <jvm.h>
#include <jawt.h>

#ifdef NDEBUG	/* NDEBUG overrides DEBUG */
#undef DEBUG
#endif

/*
 * Two Sun keys are not defined in standard keysym.h
 * Key F11, F12 on Sun keyboard somehow have KeySym SunF36, SunF37
 * Need to map these two KeySyms to AWT keys F11, F12 so that
 * pressing F11, F12 will generate F11, F12 events (Bug #4011219, #1264603)
 *
 * echawkes: similar for the dead grave and dead acute (and probably all
 * the other dead keys as well).  
 */
#define XK_SUNFA_Grave 0x1005FF00
#define XK_SUNFA_Acute 0x1005FF03
#define XK_SUNF36      0x1005FF10
#define XK_SUNF37      0x1005FF11

extern struct MComponentPeerIDs mComponentPeerIDs;
extern struct AWTEventIDs awtEventIDs;
extern struct KeyEventIDs keyEventIDs;
extern struct InputEventIDs inputEventIDs;
extern struct ComponentIDs componentIDs;

#ifdef DEBUG
static Boolean debugKeys = False;
#endif

int awt_override_key = -1;

int awt_multiclick_smudge = 4;

static Widget input_focus = NULL;

void trackInputFocus (Widget w) {
  input_focus = w;
}

Widget prevWidget = 0; /* for bug fix 4017222 */

extern void awt_setWidgetGravity(Widget w, int gravity);
extern Boolean scrollBugWorkAround;
extern jobject currentX11InputMethodInstance;
extern Window  currentFocusWindow;
extern Boolean awt_x11inputmethod_lookupString(XKeyPressedEvent *, KeySym *);

/* Used in creation of Java-level InputEvents */
jlong sysTimeMillis();

void awt_post_java_key_event(XtPointer client_data, long id,
                             long eventdata, jlong when, long keycode,
                             jchar keychar, long modifiers);
void awt_post_java_focus_event(XtPointer client_data, long id,
                               long eventdata, Boolean temp);
void awt_post_java_mouse_event(XtPointer client_data, long id,
                               long eventdata, jlong when, long modifiers,
                               int x, int y, int clickcount,
                               Boolean popuptrigger);

/* input method hot key stuff */
Boolean lookForInputMethodSelectionKey(JNIEnv * env);
Boolean checkInputMethodSelectionKey(JNIEnv * env, long keycode, long modifiers);
void popUpInputMethodSelectionMenu(JNIEnv * env, jobject target);
static long selectionKey = java_awt_event_KeyEvent_VK_UNDEFINED;
static long selectionModifiers = 0;

typedef struct KEYMAP_ENTRY {
    long awtKey;
    KeySym x11Key;
    Boolean printable;
} KeymapEntry;

KeymapEntry keymapTable[] =
{
    {java_awt_event_KeyEvent_VK_A, XK_a, TRUE},
    {java_awt_event_KeyEvent_VK_B, XK_b, TRUE},
    {java_awt_event_KeyEvent_VK_C, XK_c, TRUE},
    {java_awt_event_KeyEvent_VK_D, XK_d, TRUE},
    {java_awt_event_KeyEvent_VK_E, XK_e, TRUE},
    {java_awt_event_KeyEvent_VK_F, XK_f, TRUE},
    {java_awt_event_KeyEvent_VK_G, XK_g, TRUE},
    {java_awt_event_KeyEvent_VK_H, XK_h, TRUE},
    {java_awt_event_KeyEvent_VK_I, XK_i, TRUE},
    {java_awt_event_KeyEvent_VK_J, XK_j, TRUE},
    {java_awt_event_KeyEvent_VK_K, XK_k, TRUE},
    {java_awt_event_KeyEvent_VK_L, XK_l, TRUE},
    {java_awt_event_KeyEvent_VK_M, XK_m, TRUE},
    {java_awt_event_KeyEvent_VK_N, XK_n, TRUE},
    {java_awt_event_KeyEvent_VK_O, XK_o, TRUE},
    {java_awt_event_KeyEvent_VK_P, XK_p, TRUE},
    {java_awt_event_KeyEvent_VK_Q, XK_q, TRUE},
    {java_awt_event_KeyEvent_VK_R, XK_r, TRUE},
    {java_awt_event_KeyEvent_VK_S, XK_s, TRUE},
    {java_awt_event_KeyEvent_VK_T, XK_t, TRUE},
    {java_awt_event_KeyEvent_VK_U, XK_u, TRUE},
    {java_awt_event_KeyEvent_VK_V, XK_v, TRUE},
    {java_awt_event_KeyEvent_VK_W, XK_w, TRUE},
    {java_awt_event_KeyEvent_VK_X, XK_x, TRUE},
    {java_awt_event_KeyEvent_VK_Y, XK_y, TRUE},
    {java_awt_event_KeyEvent_VK_Z, XK_z, TRUE},

    {java_awt_event_KeyEvent_VK_ENTER, XK_Return, TRUE},
    {java_awt_event_KeyEvent_VK_ENTER, XK_KP_Enter, TRUE},
    {java_awt_event_KeyEvent_VK_ENTER, XK_Linefeed, TRUE},

    {java_awt_event_KeyEvent_VK_BACK_SPACE, XK_BackSpace, TRUE},
    {java_awt_event_KeyEvent_VK_TAB, XK_Tab, TRUE},
    {java_awt_event_KeyEvent_VK_CANCEL, XK_Cancel, FALSE},
    {java_awt_event_KeyEvent_VK_CLEAR, XK_Clear, FALSE},
    {java_awt_event_KeyEvent_VK_SHIFT, XK_Shift_L, FALSE},
    {java_awt_event_KeyEvent_VK_SHIFT, XK_Shift_R, FALSE},
    {java_awt_event_KeyEvent_VK_CONTROL, XK_Control_L, FALSE},
    {java_awt_event_KeyEvent_VK_CONTROL, XK_Control_R, FALSE},
    {java_awt_event_KeyEvent_VK_ALT, XK_Alt_L, FALSE},
    {java_awt_event_KeyEvent_VK_ALT, XK_Alt_R, FALSE},
    {java_awt_event_KeyEvent_VK_META, XK_Meta_L, FALSE},
    {java_awt_event_KeyEvent_VK_META, XK_Meta_R, FALSE},
    {java_awt_event_KeyEvent_VK_PAUSE, XK_Pause, FALSE},
    {java_awt_event_KeyEvent_VK_CAPS_LOCK, XK_Caps_Lock, FALSE},
    {java_awt_event_KeyEvent_VK_ESCAPE, XK_Escape, TRUE},

    {java_awt_event_KeyEvent_VK_PAGE_UP, XK_Page_Up, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_UP, XK_R9, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_UP, XK_Prior, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_DOWN, XK_Page_Down, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_DOWN, XK_R15, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_DOWN, XK_Next, FALSE},
    {java_awt_event_KeyEvent_VK_END, XK_R13, FALSE},
    {java_awt_event_KeyEvent_VK_END, XK_End, FALSE},
    {java_awt_event_KeyEvent_VK_HOME, XK_Home, FALSE},
    {java_awt_event_KeyEvent_VK_HOME, XK_R7, FALSE},

    {java_awt_event_KeyEvent_VK_LEFT, XK_Left, FALSE},
    {java_awt_event_KeyEvent_VK_UP, XK_Up, FALSE},
    {java_awt_event_KeyEvent_VK_RIGHT, XK_Right, FALSE},
    {java_awt_event_KeyEvent_VK_DOWN, XK_Down, FALSE},
    {java_awt_event_KeyEvent_VK_INSERT, XK_Insert, FALSE},
    {java_awt_event_KeyEvent_VK_HELP, XK_Help, FALSE},

    {java_awt_event_KeyEvent_VK_KP_UP, XK_KP_Up, FALSE},
    {java_awt_event_KeyEvent_VK_KP_DOWN, XK_KP_Down, FALSE},
    {java_awt_event_KeyEvent_VK_KP_RIGHT, XK_KP_Right, FALSE},
    {java_awt_event_KeyEvent_VK_KP_LEFT, XK_KP_Left, FALSE},

    {java_awt_event_KeyEvent_VK_0, XK_0, TRUE},
    {java_awt_event_KeyEvent_VK_1, XK_1, TRUE},
    {java_awt_event_KeyEvent_VK_2, XK_2, TRUE},
    {java_awt_event_KeyEvent_VK_3, XK_3, TRUE},
    {java_awt_event_KeyEvent_VK_4, XK_4, TRUE},
    {java_awt_event_KeyEvent_VK_5, XK_5, TRUE},
    {java_awt_event_KeyEvent_VK_6, XK_6, TRUE},
    {java_awt_event_KeyEvent_VK_7, XK_7, TRUE},
    {java_awt_event_KeyEvent_VK_8, XK_8, TRUE},
    {java_awt_event_KeyEvent_VK_9, XK_9, TRUE},

    {java_awt_event_KeyEvent_VK_SPACE, XK_space, TRUE},
    {java_awt_event_KeyEvent_VK_EXCLAMATION_MARK, XK_exclam, TRUE},
    {java_awt_event_KeyEvent_VK_QUOTEDBL, XK_quotedbl, TRUE},
    {java_awt_event_KeyEvent_VK_NUMBER_SIGN, XK_numbersign, TRUE},
    {java_awt_event_KeyEvent_VK_DOLLAR, XK_dollar, TRUE},
    {java_awt_event_KeyEvent_VK_AMPERSAND, XK_ampersand, TRUE},
    {java_awt_event_KeyEvent_VK_QUOTE, XK_apostrophe, TRUE},
    {java_awt_event_KeyEvent_VK_LEFT_PARENTHESIS, XK_parenleft, TRUE},
    {java_awt_event_KeyEvent_VK_RIGHT_PARENTHESIS, XK_parenright, TRUE},
    {java_awt_event_KeyEvent_VK_ASTERISK, XK_asterisk, TRUE},
    {java_awt_event_KeyEvent_VK_PLUS, XK_plus, TRUE},
    {java_awt_event_KeyEvent_VK_COMMA, XK_comma, TRUE},  
    {java_awt_event_KeyEvent_VK_MINUS, XK_minus, TRUE},
    {java_awt_event_KeyEvent_VK_PERIOD, XK_period, TRUE},
    {java_awt_event_KeyEvent_VK_SLASH, XK_slash, TRUE},

    {java_awt_event_KeyEvent_VK_COLON, XK_colon, TRUE},
    {java_awt_event_KeyEvent_VK_SEMICOLON, XK_semicolon, TRUE},
    {java_awt_event_KeyEvent_VK_LESS, XK_less, TRUE},
    {java_awt_event_KeyEvent_VK_EQUALS, XK_equal, TRUE},
    {java_awt_event_KeyEvent_VK_GREATER, XK_greater, TRUE},

    {java_awt_event_KeyEvent_VK_AT, XK_at, TRUE},

    {java_awt_event_KeyEvent_VK_OPEN_BRACKET, XK_bracketleft, TRUE},
    {java_awt_event_KeyEvent_VK_BACK_SLASH, XK_backslash, TRUE},
    {java_awt_event_KeyEvent_VK_CLOSE_BRACKET, XK_bracketright, TRUE},
    {java_awt_event_KeyEvent_VK_CIRCUMFLEX, XK_asciicircum, TRUE},
    {java_awt_event_KeyEvent_VK_UNDERSCORE, XK_underscore, TRUE},
    {java_awt_event_KeyEvent_VK_BACK_QUOTE, XK_grave, TRUE},

    {java_awt_event_KeyEvent_VK_BRACELEFT, XK_braceleft, TRUE},
    {java_awt_event_KeyEvent_VK_BRACERIGHT, XK_braceright, TRUE},

    {java_awt_event_KeyEvent_VK_INVERTED_EXCLAMATION_MARK, XK_exclamdown, TRUE},

    {java_awt_event_KeyEvent_VK_NUMPAD0, XK_KP_0, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD1, XK_KP_1, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD2, XK_KP_2, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD3, XK_KP_3, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD4, XK_KP_4, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD5, XK_KP_5, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD6, XK_KP_6, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD7, XK_KP_7, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD8, XK_KP_8, TRUE},
    {java_awt_event_KeyEvent_VK_NUMPAD9, XK_KP_9, TRUE},
    {java_awt_event_KeyEvent_VK_MULTIPLY, XK_KP_Multiply, TRUE},
    {java_awt_event_KeyEvent_VK_ADD, XK_KP_Add, TRUE},
    {java_awt_event_KeyEvent_VK_SUBTRACT, XK_KP_Subtract, TRUE},
    {java_awt_event_KeyEvent_VK_DECIMAL, XK_KP_Decimal, TRUE},
    {java_awt_event_KeyEvent_VK_DIVIDE, XK_KP_Divide, TRUE},
    {java_awt_event_KeyEvent_VK_EQUALS, XK_KP_Equal, TRUE},
    {java_awt_event_KeyEvent_VK_INSERT, XK_KP_Insert, FALSE},
    {java_awt_event_KeyEvent_VK_HOME, XK_KP_Home, FALSE},
    {java_awt_event_KeyEvent_VK_END, XK_KP_End, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_UP, XK_KP_Page_Up, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_DOWN, XK_KP_Page_Down, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_UP, XK_KP_Prior, FALSE},
    {java_awt_event_KeyEvent_VK_PAGE_DOWN, XK_KP_Next, FALSE},

    {java_awt_event_KeyEvent_VK_F1, XK_F1, FALSE},
    {java_awt_event_KeyEvent_VK_F2, XK_F2, FALSE},
    {java_awt_event_KeyEvent_VK_F3, XK_F3, FALSE},
    {java_awt_event_KeyEvent_VK_F4, XK_F4, FALSE},
    {java_awt_event_KeyEvent_VK_F5, XK_F5, FALSE},
    {java_awt_event_KeyEvent_VK_F6, XK_F6, FALSE},
    {java_awt_event_KeyEvent_VK_F7, XK_F7, FALSE},
    {java_awt_event_KeyEvent_VK_F8, XK_F8, FALSE},
    {java_awt_event_KeyEvent_VK_F9, XK_F9, FALSE},
    {java_awt_event_KeyEvent_VK_F10, XK_F10, FALSE},
    {java_awt_event_KeyEvent_VK_F11, XK_F11, FALSE},
    {java_awt_event_KeyEvent_VK_F12, XK_F12, FALSE},

    {java_awt_event_KeyEvent_VK_DELETE, XK_Delete, TRUE},
    {java_awt_event_KeyEvent_VK_DELETE, XK_KP_Delete, TRUE},

    {java_awt_event_KeyEvent_VK_NUM_LOCK, XK_Num_Lock, FALSE},
    {java_awt_event_KeyEvent_VK_SCROLL_LOCK, XK_Scroll_Lock, FALSE},
    {java_awt_event_KeyEvent_VK_PRINTSCREEN, XK_Print, FALSE},

    // X11 keysym names for input method related keys don't always match keytop engravings
    // or Java virtual key names, so we here only map constants that we've found on real keyboards.
    {java_awt_event_KeyEvent_VK_ACCEPT, XK_Execute, FALSE}, // Type 5c Japanese keyboard: kakutei
    {java_awt_event_KeyEvent_VK_CONVERT, XK_Kanji, FALSE}, // Type 5c Japanese keyboard: henkan
    {java_awt_event_KeyEvent_VK_INPUT_METHOD_ON_OFF, XK_Henkan_Mode, FALSE}, // Type 5c Japanese keyboard: nihongo
    // VK_KANA_LOCK is handled separately because it generates the same keysym as ALT_GRAPH
    // in spite of its different behavior.

    { java_awt_event_KeyEvent_VK_F11, XK_SUNF36, FALSE },
    { java_awt_event_KeyEvent_VK_F12, XK_SUNF37, FALSE },
    { java_awt_event_KeyEvent_VK_AGAIN, XK_L2, FALSE },
    { java_awt_event_KeyEvent_VK_UNDO, XK_L4, FALSE },
    { java_awt_event_KeyEvent_VK_COPY, XK_L6, FALSE },
    { java_awt_event_KeyEvent_VK_PASTE, XK_L8, FALSE },
    { java_awt_event_KeyEvent_VK_CUT, XK_L10, FALSE },
    { java_awt_event_KeyEvent_VK_FIND, XK_L9, FALSE },
    { java_awt_event_KeyEvent_VK_PROPS, XK_L3, FALSE },
    { java_awt_event_KeyEvent_VK_STOP, XK_L1, FALSE },

    { java_awt_event_KeyEvent_VK_COMPOSE, XK_Multi_key, FALSE },
    { java_awt_event_KeyEvent_VK_ALT_GRAPH, XK_Mode_switch, FALSE },

    /* Dead key mappings (for European keyboards) */
    { java_awt_event_KeyEvent_VK_DEAD_GRAVE, XK_SUNFA_Grave, FALSE },
    { java_awt_event_KeyEvent_VK_DEAD_ACUTE, XK_SUNFA_Acute, FALSE },

    {0, 0, 0}
};

static Boolean
keyboardHasKanaLockKey()
{
    static Boolean haveResult = FALSE;
    static Boolean result = FALSE;

    int minKeyCode, maxKeyCode, keySymsPerKeyCode;
    KeySym *keySyms, *keySymsStart, keySym;
    int i;
    int kanaCount = 0;
    
    // Solaris doesn't let you swap keyboards without rebooting,
    // so there's no need to check for the kana lock key more than once.
    if (haveResult) {
       return result;
    }
    
    // There's no direct way to determine whether the keyboard has
    // a kana lock key. From available keyboard mapping tables, it looks
    // like only keyboards with the kana lock key can produce keysyms
    // for kana characters. So, as an indirect test, we check for those.
    XDisplayKeycodes(awt_display, &minKeyCode, &maxKeyCode);
    keySyms = XGetKeyboardMapping(awt_display, minKeyCode, maxKeyCode - minKeyCode + 1, &keySymsPerKeyCode);
    keySymsStart = keySyms;
    for (i = 0; i < (maxKeyCode - minKeyCode + 1) * keySymsPerKeyCode; i++) {
        keySym = *keySyms++;
        if ((keySym & 0xff00) == 0x0400) {
            kanaCount++;
        }
    }
    XFree(keySymsStart);

    // use a (somewhat arbitrary) minimum so we don't get confused by a stray function key
    result = kanaCount > 10; 
    haveResult = TRUE;
    return result;
}

static void 
keysymToAWTKeyCode(KeySym x11Key, long *keycode, Boolean * printable)
{
    int i;

    // Solaris uses XK_Mode_switch for both the non-locking AltGraph
    // and the locking Kana key, but we want to keep them separate for
    // KeyEvent.
    if (x11Key == XK_Mode_switch && keyboardHasKanaLockKey()) {
        *keycode = java_awt_event_KeyEvent_VK_KANA_LOCK;
        *printable = FALSE;
        return;
    }

    for (i = 0; keymapTable[i].awtKey != 0; i++) {
        if (keymapTable[i].x11Key == x11Key) {
            *keycode = keymapTable[i].awtKey;
            *printable = keymapTable[i].printable;
            return;
        }
    }

    *keycode = 0;
    *printable = FALSE;
/*
  printf("#### No key mapping found : (X11)%x#### \n", x11Key);
*/
}

KeySym 
awt_getX11KeySym(long awtKey)
{
    int i;

    if (awtKey == java_awt_event_KeyEvent_VK_KANA_LOCK && keyboardHasKanaLockKey()) {
        return XK_Mode_switch;
    }

    for (i = 0; keymapTable[i].awtKey != 0; i++) {
        if (keymapTable[i].awtKey == awtKey) {
            return keymapTable[i].x11Key;
        }
    }
/*
  printf("#### No key mapping found: (AWT)%x#### \n", awtKey);
*/
    return 0;
}


#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif

#define INTERSECTS(r1_x1,r1_x2,r1_y1,r1_y2,r2_x1,r2_x2,r2_y1,r2_y2) \
!((r2_x2 <= r1_x1) ||\
  (r2_y2 <= r1_y1) ||\
  (r2_x1 >= r1_x2) ||\
  (r2_y1 >= r1_y2))

    typedef struct COLLAPSE_INFO {
        Window win;
        DamageRect *r;
    } CollapseInfo;

static void
expandDamageRect(DamageRect * drect, XEvent * xev, Boolean debug, char *str)
{
    int x1 = xev->xexpose.x;
    int y1 = xev->xexpose.y;
    int x2 = x1 + xev->xexpose.width;
    int y2 = y1 + xev->xexpose.height;

    /*
      if (debug) {
      printf("   %s: collapsing (%d,%d %dx%d) into (%d,%d %dx%d) ->>",
      str, x1, y1, xev->xexpose.width, xev->xexpose.height,
      drect->x1, drect->y1, drect->x2 - drect->x1, drect->y2 - drect->y1); 
      } 
    */

    drect->x1 = MIN(x1, drect->x1);
    drect->y1 = MIN(y1, drect->y1);
    drect->x2 = MAX(x2, drect->x2);
    drect->y2 = MAX(y2, drect->y2);

    /*
      if (debug) {
      printf("(%d,%d %dx%d) %s\n",
      drect->x1, drect->y1, drect->x2 - drect->x1, drect->y2 - drect->y1);
      }
    */

}

static Bool
checkForExpose(Display * dpy, XEvent * evt, XPointer client_data)
{
    CollapseInfo *cinfo = (CollapseInfo *) client_data;

    if ((evt->type == Expose && evt->xexpose.window == cinfo->win &&
         INTERSECTS(cinfo->r->x1, cinfo->r->x2, cinfo->r->y1, cinfo->r->y2,
                    evt->xexpose.x, 
                    evt->xexpose.x + evt->xexpose.width,
		    evt->xexpose.y,
                    evt->xexpose.y + evt->xexpose.height)) ||
        (evt->type == GraphicsExpose && evt->xgraphicsexpose.drawable == cinfo->win &&
         INTERSECTS(cinfo->r->x1, cinfo->r->x2, cinfo->r->y1, cinfo->r->y2,
                    evt->xgraphicsexpose.x, 
                    evt->xgraphicsexpose.x + evt->xgraphicsexpose.width,
		    evt->xgraphicsexpose.y,
                    evt->xgraphicsexpose.y + evt->xgraphicsexpose.height))) {

        return True;
    }
    return False;
}

/*
 * javaObject is an MComponentPeer instance
 */
static void 
HandleExposeEvent(Widget w, jobject javaObject, XEvent * event)
{
    jobject target;
    jint wdth, hght;

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    switch (event->type) {
        case Expose:
        case GraphicsExpose:
        {
            struct ComponentData *cdata;
            Boolean debug = FALSE;
            jint drawState;

            /* Set the draw state */
            drawState = (*env)->GetIntField(env, javaObject,
                mComponentPeerIDs.drawState);
            (*env)->SetIntField(env, javaObject, mComponentPeerIDs.drawState,
                drawState | JAWT_LOCK_CLIP_CHANGED);
            cdata = (struct ComponentData *)
	      JNU_GetLongFieldAsPtr(env, javaObject, mComponentPeerIDs.pData);
            if (JNU_IsNull(env, javaObject) || (cdata == 0)) {
                return;
            }
            if (event->xexpose.send_event) {
                if (cdata->repaintPending & RepaintPending_REPAINT) {
                    cdata->repaintPending &= ~RepaintPending_REPAINT;

                    JNU_CallMethodByName(env, 
					 NULL,
					 javaObject, 
					 "handleRepaint",
					 "(IIII)V",
					 cdata->repaintRect.x1,
					 cdata->repaintRect.y1,
					 cdata->repaintRect.x2
					 - cdata->repaintRect.x1,
					 cdata->repaintRect.y2
					 - cdata->repaintRect.y1);
                    if ((*env)->ExceptionOccurred(env)) {
                        (*env)->ExceptionDescribe(env);
                        (*env)->ExceptionClear(env);
                    }
                }
                return;
            }
            if ((cdata->repaintPending & RepaintPending_EXPOSE) == 0) {
                cdata->exposeRect.x1 = event->xexpose.x;
                cdata->exposeRect.y1 = event->xexpose.y;
                cdata->exposeRect.x2 = cdata->exposeRect.x1 + event->xexpose.width;
                cdata->exposeRect.y2 = cdata->exposeRect.y1 + event->xexpose.height;
                cdata->repaintPending |= RepaintPending_EXPOSE;
            } else {
                expandDamageRect(&(cdata->exposeRect), event, debug, "1");
            }

            /* Only post Expose/Repaint if we know others arn't following
             * directly in the queue.
             */
            if (event->xexpose.count == 0) {
                int count = 0;
                CollapseInfo cinfo;

                cinfo.win = XtWindow(w);
                cinfo.r = &(cdata->exposeRect);

                /* Do a little more inspecting and collapse further if there
                 * are additional expose events pending on this window where
                 * the damage rects intersect with the current exposeRect.
                 */
                while (1) {
                    XEvent xev;

                    if (XCheckIfEvent(XtDisplay(w), &xev
                                      ,checkForExpose, (XtPointer) & cinfo)) {
                        count = xev.xexpose.count;
                        expandDamageRect(&(cdata->exposeRect), &xev, debug, "2");

                    } else {
                        /* XCheckIfEvent Failed. */
                        break;
                    }
                }

                cdata->repaintPending &= ~RepaintPending_EXPOSE;

		/* Fix for bugtraq id 4262108. Paint events should not be
		 * delivered to components that have one of their 
		 * dimensions equal to zero.
		 */

                if ((*env)->EnsureLocalCapacity(env, 1) < 0) {
                    return;
                }

                target = (*env)->GetObjectField(env, javaObject,
                                                    mComponentPeerIDs.target);
                wdth = (*env)->GetIntField(env, target, componentIDs.width);
                hght = (*env)->GetIntField(env, target, componentIDs.height);
                (*env)->DeleteLocalRef(env, target);

                if ( wdth != 0 && hght != 0) {
	 	    JNU_CallMethodByName(env,
			 		NULL,
			 		javaObject,
					"handleExpose", 
					"(IIII)V",
					cdata->exposeRect.x1, 
					cdata->exposeRect.y1,
					cdata->exposeRect.x2 
					- cdata->exposeRect.x1,
					cdata->exposeRect.y2
					- cdata->exposeRect.y1);
                    if ((*env)->ExceptionOccurred(env)) {
                        (*env)->ExceptionDescribe(env);
                        (*env)->ExceptionClear(env);
                    }
                }
            }
        }
        break;

        default:
            jio_fprintf(stderr, "Got event %d in HandleExposeEvent!\n", event->type);
    }
}

/*
 * The AWT provides an isTemporary() method on focus events to 
 * be able to determine the difference between focus moving between
 * components and a temporary focus change caused by window de-activation
 * or menu postings.  Unfortunately the X11 focus model does not make
 * this distinction easy to determine, so we must track extra state here
 * to determine the difference.
 */
static Widget activated_shell = NULL;
static Widget deactivated_shell = NULL;

Widget 
getAncestorShell(Widget w)
{
    Widget p = w;

    while (!(XtIsSubclass(p, shellWidgetClass))) {
        p = XtParent(p);
    }
    return p;
}

void 
awt_setActivatedShell(Widget w)
{
    activated_shell = w;
}

void 
awt_setDeactivatedShell(Widget w)
{
    deactivated_shell = w;
}

void 
handleFocusEvent(Widget w,
                 XFocusChangeEvent * fevent,
                 void *client_data,
                 Boolean * cont,
                 Boolean passEvent)
{
    if (fevent->type == FocusIn) {
        if (fevent->mode == NotifyNormal &&
            fevent->detail != NotifyPointer && fevent->detail != NotifyVirtual) {
            awt_post_java_focus_event(client_data,
                                      java_awt_event_FocusEvent_FOCUS_GAINED,
                                      0,
                                      FALSE);
        }
    } else {
        /* FocusOut */
        if (fevent->mode == NotifyNormal &&
            fevent->detail != NotifyPointer && fevent->detail != NotifyVirtual) {
            Boolean temp = FALSE;

            /* If the last focus event caused the window to be deactivated,
             * then this focus-out event is temporary.
             */
            if (getAncestorShell(w) == deactivated_shell) {
                temp = TRUE;
                deactivated_shell = NULL;
            } else {
                /* If the event is sent by system then focus-out event is 
		 * temporary.
		 */
                 if (input_focus == w || input_focus == NULL)
		    temp = TRUE;
            }
            awt_post_java_focus_event(client_data,
                                      java_awt_event_FocusEvent_FOCUS_LOST,
                                      0,
                                      temp);
        }
    }
    *cont = TRUE;
}



int
getModifiers(unsigned int state)
{
    int modifiers = 0;

    if (state & ShiftMask) {
        modifiers |= java_awt_event_InputEvent_SHIFT_MASK;
    }
    if (state & ControlMask) {
        modifiers |= java_awt_event_InputEvent_CTRL_MASK;
    }
#ifdef __linux__
    if (state & awt_MetaMask & ~awt_AltMask) {
#else
    if (state & awt_MetaMask) {
#endif
        modifiers |= java_awt_event_InputEvent_META_MASK;
    }
    if (state & awt_AltMask) {
        modifiers |= java_awt_event_InputEvent_ALT_MASK;
    }
    if (state & Button1Mask) {
        modifiers |= java_awt_event_InputEvent_BUTTON1_MASK;
    }
    if ((state & Button2Mask) != 0) {
        modifiers |= java_awt_event_InputEvent_ALT_MASK;
        modifiers |= java_awt_event_InputEvent_BUTTON2_MASK;
    }
    if (state & Button3Mask) {
        modifiers |= java_awt_event_InputEvent_META_MASK;
        modifiers |= java_awt_event_InputEvent_BUTTON3_MASK;
    }
    return modifiers;
}

static void 
modify_Event(XEvent * xevent, long keyCode, jchar keyChar, long modifiers)
{
    KeySym keysym = java_awt_event_KeyEvent_CHAR_UNDEFINED;

    if (xevent->type != KeyPress && xevent->type != KeyRelease)
        return;

    switch (keyCode) {
        case java_awt_event_KeyEvent_VK_ENTER:
        case java_awt_event_KeyEvent_VK_BACK_SPACE:
        case java_awt_event_KeyEvent_VK_TAB:
        case java_awt_event_KeyEvent_VK_ESCAPE:
        case java_awt_event_KeyEvent_VK_ADD:
        case java_awt_event_KeyEvent_VK_SUBTRACT:
        case java_awt_event_KeyEvent_VK_MULTIPLY:
        case java_awt_event_KeyEvent_VK_DIVIDE:
        case java_awt_event_KeyEvent_VK_DECIMAL:
        case java_awt_event_KeyEvent_VK_NUMPAD0:
        case java_awt_event_KeyEvent_VK_NUMPAD1:
        case java_awt_event_KeyEvent_VK_NUMPAD2:
        case java_awt_event_KeyEvent_VK_NUMPAD3:
        case java_awt_event_KeyEvent_VK_NUMPAD4:
        case java_awt_event_KeyEvent_VK_NUMPAD5:
        case java_awt_event_KeyEvent_VK_NUMPAD6:
        case java_awt_event_KeyEvent_VK_NUMPAD7:
        case java_awt_event_KeyEvent_VK_NUMPAD8:
        case java_awt_event_KeyEvent_VK_NUMPAD9:
            keysym = awt_getX11KeySym(keyCode);
            break;
        case java_awt_event_KeyEvent_VK_DELETE:
            /* For some reason XKeysymToKeycode returns incorrect value for 
             * Delete, so we don't want to modify the original event
             */
            break;
        default:
            if (keyChar < 256) {
                keysym = keyChar;
            } else {
                keysym = awt_getX11KeySym(keyCode);
            }
            break;
    }

    if (keysym < 256) {
        if (modifiers & java_awt_event_InputEvent_CTRL_MASK) {
            switch (keysym + 64) {
                case '[':
                case ']':
                case '\\':
                case '_':
                    keysym += 64;
                    break;
                default:
                    if (isalpha(keysym + 'a' - 1)) {
                        keysym += ('a' - 1);
                    }
                    break;
            }
        }
        /*
         * 0xff61 is Unicode value of first XK_kana_fullstop.
         * We need X Keysym to Unicode map in post1.1 release
         * to support more intenational keyboard.
         */
        if (keysym >= 0xff61 && keysym <= 0xff9f) {
            keysym = keysym - 0xff61 + XK_kana_fullstop;
        }
        xevent->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
    }

    if (keysym >= 'A' && keysym <= 'Z') {
        xevent->xkey.state |= ShiftMask;
    }
    if (modifiers & java_awt_event_InputEvent_SHIFT_MASK) {
        xevent->xkey.state |= ShiftMask;
    }
    if (modifiers & java_awt_event_InputEvent_CTRL_MASK) {
        xevent->xkey.state |= ControlMask;
    }
#ifdef __linux__
    if (modifiers &
        java_awt_event_InputEvent_META_MASK &
        ~java_awt_event_InputEvent_ALT_MASK) {
#else
    if (modifiers & java_awt_event_InputEvent_META_MASK) {
#endif
        xevent->xkey.state |= awt_MetaMask;
    }
    if (modifiers & java_awt_event_InputEvent_ALT_MASK) {
        xevent->xkey.state |= awt_AltMask;
    }
    if (modifiers & java_awt_event_InputEvent_BUTTON1_MASK) {
        xevent->xkey.state |= Button1Mask;
    }
    if (modifiers & java_awt_event_InputEvent_BUTTON2_MASK) {
        xevent->xkey.state |= Button2Mask;
    }
    if (modifiers & java_awt_event_InputEvent_BUTTON3_MASK) {
        xevent->xkey.state |= Button3Mask;
    }
}

void 
awt_modify_KeyEvent(JNIEnv *env, XEvent * xevent, jobject jevent)
{
    jint keyCode;
    jchar keyChar;
    jint modifiers;

    keyCode = (*env)->GetIntField(env, jevent, keyEventIDs.keyCode);
    keyChar = (*env)->GetCharField(env, jevent, keyEventIDs.keyChar);
    modifiers = (*env)->GetIntField(env, jevent, inputEventIDs.modifiers);

    modify_Event(xevent, keyCode, keyChar, modifiers);
}

XEvent *
awt_copyXEvent(XEvent * xev)
{
    XEvent *xev_copy = (XEvent *) malloc(sizeof(XEvent));

    if (xev_copy != NULL) {
        memcpy(xev_copy, xev, sizeof(XEvent));
        return xev_copy;
    } else {
        return NULL;
    }
}

#ifdef __linux__
static void 
handleKeyEvent(long keyEventId,
               XEvent * event,
               void *client_data,
               Boolean * cont,
               Boolean passEvent)
{
    int modifiers = getModifiers(event->xkey.state);
    KeySym keysym = 0;
    KeySym keysym_nomods = 0;
    long keycode = 0;
    Modifiers mods = 0;
    Boolean printable = FALSE;

    if (currentX11InputMethodInstance != NULL
        && keyEventId == java_awt_event_KeyEvent_KEY_PRESSED
	&& event->xkey.window == currentFocusWindow) {
	/*
	 * invokes XmbLookupString to get a committed string or keysym,
	 * if any.
	 */
	if (awt_x11inputmethod_lookupString(event, &keysym)) {
	    /* Return on error when the function returns True. */
	    *cont = FALSE;
	    return;
	}
    }
    else {
	/* Now get real keysym which looks at modifiers. */
	XtTranslateKeycode(event->xkey.display, 
	    (KeyCode) event->xkey.keycode, event->xkey.state, &mods, 
	    &keysym);
	if (((event->xkey.state & ShiftMask) ||
	    (event->xkey.state & LockMask)) &&
	    (keysym >= 'a' && keysym <= 'z'))
	{
	    keysym = toupper(keysym);
	}
    }
    /* Get keysym without taking modifiers into account, to map
     * to AWT keyCode table, except for keypad keys when NumLock is on.
     */
    if (IsKeypadKey(keysym) && (event->xkey.state & awt_NumLockMask)) {
        keysym_nomods = keysym;
    } 
    else {
        keysym_nomods = XKeycodeToKeysym(event->xkey.display, event->xkey.keycode, 0);
	if (keysym_nomods == XK_KP_Enter) {  /* Bug 4350175 */
	    keysym = XK_KP_Enter;
        }
    }

#ifdef DEBUG
    if (debugKeys) {
        jio_fprintf(stderr, "---> KeyEvent %d: xkeycode=%x, state=%x xkeysym=%x\n",  
                    event->type, event->xkey.keycode, event->xkey.state, keysym); 
    }
#endif

    if (keysym_nomods == NoSymbol) {
	*cont = TRUE;
	return;
    }

    keysymToAWTKeyCode(keysym_nomods, &keycode, &printable);
    if (keysym_nomods < 256) {
        printable = TRUE;
    } else if (keycode == 0) {
	*cont = TRUE;
	return;
    }

    if ((keysym < 256) && (event->xkey.state & ControlMask)) {
	switch (keysym) {
	    case '[':
	    case ']':
	    case '\\':
	    case '_':
		keysym -= 64;
		break;
	    default:
		if (isalpha(keysym)) {
		    keysym = tolower(keysym) - 'a' + 1;
		}
		break;
	}
    }
    if (keysym >= XK_kana_fullstop && keysym <= XK_semivoicedsound) {
	/*
	 * 0xff61 is Unicode value of first XK_kana_fullstop.
	 * We need X Keysym to Unicode map in post1.1 release
	 * to support more intenational keyboard.
	 */
	keysym = keysym - XK_kana_fullstop + 0xff61;
    }

    /* [jk] Make Shift-Tab work on Linux:
     * Checking for ISO_Left_Tab is needed on all platforms, other
     * Shift-Tab won't work when the display is exported to a Linux
     * machine.
     */
    if (keysym == XK_ISO_Left_Tab) {
	keysym = XK_Tab;
    }

    switch (keysym) {  
	case XK_Return:
	    keysym = XK_Linefeed;     /* fall thru */
	case XK_BackSpace:
	case XK_Tab:
	case XK_Linefeed:
	case XK_Escape:
	case XK_Delete:
	    /* strip off highorder bits defined in xkeysymdef.h */
	    keysym &= 0x007F;
	    break;
	case XK_KP_Decimal:
	    keysym = '.';
	    break;
	case XK_KP_Add:
	    keysym = '+';
	    break;
	case XK_KP_Subtract:
	    keysym = '-';
	    break;
	case XK_KP_Divide:
	    keysym = '/';
	    break;
	case XK_KP_Multiply:
	    keysym = '*';
	    break;
	case XK_KP_Equal:
	    keysym = '=';
	    break;
	case XK_KP_0:
	    keysym = '0';
	    break;
	case XK_KP_1:
	    keysym = '1';
	    break;
	case XK_KP_2:
	    keysym = '2';
	    break;
	case XK_KP_3:
	    keysym = '3';
	    break;
	case XK_KP_4:
	    keysym = '4';
	    break;
	case XK_KP_5:
	    keysym = '5';
	    break;
	case XK_KP_6:
	    keysym = '6';
	    break;
	case XK_KP_7:
	    keysym = '7';
	    break;
	case XK_KP_8:
	    keysym = '8';
	    break;
	case XK_KP_9:
	    keysym = '9';
	    break;
	case XK_KP_Left:  /* Bug 4350175 */
	    keysym = XK_Left;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Up:
	    keysym = XK_Up;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Right:
	    keysym = XK_Right;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Down:
	    keysym = XK_Down;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Home:
	    keysym = XK_Home;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_End:
	    keysym = XK_End;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Page_Up:
	    keysym = XK_Page_Up;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Page_Down:
	    keysym = XK_Page_Down;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Insert:
	    keysym = XK_Insert;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    break;
	case XK_KP_Delete:
	    keysym = XK_Delete;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, keysym);
	    keysym &= 0x007F;
	    break;
	case XK_KP_Enter:
	    keysym = XK_Linefeed;
	    event->xkey.keycode = XKeysymToKeycode(awt_display, XK_Return);
	    keysym &= 0x007F;
	    break;
	default:
	    break;
    }

#ifdef DEBUG
        if (debugKeys) {
            jio_fprintf(stderr, "---> KeyEvent %d: xkeycode=%x, state=%x xkeysym=%x, AWTkeycode=%x\n",
            event->type, event->xkey.keycode, event->xkey.state,
            keysym, keycode);
        }
#endif

    awt_post_java_key_event(client_data, keyEventId,
			    (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
			    ((jlong)(event->xkey.time)), keycode,
			    (jchar) (printable ? keysym : java_awt_event_KeyEvent_CHAR_UNDEFINED),
                            modifiers);

    if (keyEventId == java_awt_event_KeyEvent_KEY_PRESSED) {
        if (printable) {
	    awt_post_java_key_event(client_data,
				    java_awt_event_KeyEvent_KEY_TYPED,
				    (long) 0,
				    ((jlong)(event->xkey.time)), 
  				    java_awt_event_KeyEvent_VK_UNDEFINED, 
				    (jchar) (0xff) & keysym, 
				    modifiers);
        }
    }
}
#else
static void 
handleKeyEvent(long keyEventId,
               XEvent * event,
               void *client_data,
               Boolean * cont,
               Boolean passEvent)
{
    int modifiers = getModifiers(event->xkey.state);
    KeySym keysym = 0;
    long keycode = 0;
    Modifiers mods = 0;
    Boolean printable = FALSE;

    if (currentX11InputMethodInstance != NULL
	&& keyEventId == java_awt_event_KeyEvent_KEY_PRESSED
        && event->xkey.window == currentFocusWindow) {
	/*
	 * invokes XmbLookupString to get a committed string or keysym if any.
	 */
	if (awt_x11inputmethod_lookupString((XKeyPressedEvent*)event, &keysym)) {
	    *cont = FALSE;
	    return;
	}
    } else {
	/* Get keysym without taking modifiers into account first to map
	 * to AWT keyCode table.
	 */
	keysym = XKeycodeToKeysym(event->xkey.display, event->xkey.keycode, 0);
    }

#ifdef DEBUG
    if (debugKeys) {
        jio_fprintf(stderr, "---> KeyEvent %d: xkeycode=%x, state=%x xkeysym=%x\n",  
                    event->type, event->xkey.keycode, event->xkey.state, keysym); 
    }
#endif

    if (keysym == NoSymbol) {
	*cont = TRUE;
	return;
    }

    if (keysym < 256) {
        keysymToAWTKeyCode(keysym, &keycode, &printable);

        /* Now get real keysym which looks at modifiers
         * XtGetActionKeySym() returns wrong value with Kana Lock,
         * so use XtTraslateKeycode().
         */
        XtTranslateKeycode(event->xkey.display, (KeyCode) event->xkey.keycode,
                           event->xkey.state, &mods, &keysym);

        if ((event->xkey.state & ControlMask)) {
            switch (keysym) {
                case '[':
                case ']':
                case '\\':
                case '_':
                    keysym -= 64;
                    break;
                default:
                    if (isalpha(keysym)) {
                        keysym = tolower(keysym) - 'a' + 1;
                    }
                    break;
            }
        }
        if (keysym >= XK_kana_fullstop && keysym <= XK_semivoicedsound) {
            /*
             * 0xff61 is Unicode value of first XK_kana_fullstop.
             * We need X Keysym to Unicode map in post1.1 release
             * to support more intenational keyboard.
             */
            keysym = keysym - XK_kana_fullstop + 0xff61;
        }
        awt_post_java_key_event(client_data, 
          keyEventId,
          (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
          ((jlong)(event->xkey.time)), 
          keycode,
          (jchar) keysym, 
          modifiers);

        if (keyEventId == java_awt_event_KeyEvent_KEY_PRESSED) {
            awt_post_java_key_event(client_data,
              java_awt_event_KeyEvent_KEY_TYPED,
              (long) 0,
              ((jlong)(event->xkey.time)), 
              java_awt_event_KeyEvent_VK_UNDEFINED, 
              (jchar) keysym, 
              modifiers);
        }
    } else {
        if ((IsKeypadKey(keysym) && (event->xkey.state & awt_NumLockMask)) ||
            (event->xkey.state & 32)){ 

	    keysym = XKeycodeToKeysym(event->xkey.display, 
				      event->xkey.keycode, 2);

            switch (keysym) {
                case XK_R13:
                    keysym = XK_KP_1;
                    break;
                case XK_R14:
                    keysym = XK_KP_2;
                    break;
                case XK_R15:
                    keysym = XK_KP_3;
                    break;
                case XK_R10:
                    keysym = XK_KP_4;
                    break;
                case XK_R11:
                    keysym = XK_KP_5;
                    break;
                case XK_R12:
                    keysym = XK_KP_6;
                    break;
                case XK_R7:
                    keysym = XK_KP_7;
                    break;
                case XK_R8:
                    keysym = XK_KP_8;
                    break;
                case XK_R9:
                    keysym = XK_KP_9;
                    break;
                case XK_KP_Insert:
                    keysym = XK_KP_0;
                    break;
                case XK_KP_Delete:
                    keysym = XK_KP_Decimal;
                    break;
                case XK_R4:
                    keysym = XK_KP_Equal;
                    break;
                case XK_R5:
                    keysym = XK_KP_Divide;
                    break;
                case XK_R6:
                    keysym = XK_KP_Multiply;
                    break;
                default:
                    break;
            }
        }
        keysymToAWTKeyCode(keysym, &keycode, &printable);
        if (keycode == 0) {
            *cont = TRUE;
            return;
        }
        switch (keysym) {
            case XK_Return:
            case XK_KP_Enter:
                keysym = XK_Linefeed;     /* fall thru */
            case XK_BackSpace:
            case XK_Tab:
            case XK_Linefeed:
            case XK_Escape:
            case XK_Delete:
                /* strip off highorder bits defined in xkeysymdef.h */
                keysym &= 0x007F;
                break;
            case XK_KP_Decimal:
                keysym = '.';
                break;
            case XK_KP_Add:
                keysym = '+';
                break;
            case XK_KP_Subtract:
                keysym = '-';
                break;
            case XK_KP_Divide:
                keysym = '/';
                break;
            case XK_KP_Multiply:
                keysym = '*';
                break;
            case XK_KP_Equal:
                keysym = '=';
                break;
            case XK_KP_0:
                keysym = '0';
                break;
            case XK_KP_1:
                keysym = '1';
                break;
            case XK_KP_2:
                keysym = '2';
                break;
            case XK_KP_3:
                keysym = '3';
                break;
            case XK_KP_4:
                keysym = '4';
                break;
            case XK_KP_5:
                keysym = '5';
                break;
            case XK_KP_6:
                keysym = '6';
                break;
            case XK_KP_7:
                keysym = '7';
                break;
            case XK_KP_8:
                keysym = '8';
                break;
            case XK_KP_9:
                keysym = '9';
                break;
            default:
                break;
        }
#ifdef DEBUG
        if (debugKeys) {
            jio_fprintf(stderr, "---> KeyEvent %d: xkeycode=%x, state=%x xkeysym=%x, AWTkeycode=%x\n", 
  
                        event->type, event->xkey.keycode, event->xkey.state, 
                        keysym, keycode);  
        }   
#endif

        awt_post_java_key_event(client_data, 
          keyEventId,
          (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
          ((jlong)(event->xkey.time)),
          keycode, 
          (jchar) (printable ? keysym : java_awt_event_KeyEvent_CHAR_UNDEFINED), 
          modifiers);

        if (keyEventId == java_awt_event_KeyEvent_KEY_PRESSED && printable) {
            awt_post_java_key_event(client_data,
              java_awt_event_KeyEvent_KEY_TYPED,
              (long) 0,
              ((jlong)(event->xkey.time)), 
              java_awt_event_KeyEvent_VK_UNDEFINED, 
              (jchar) keysym, 
              modifiers);
        }
    }
}
#endif

static void
translateXY(Widget w, int *xp, int *yp)
{
    Position wx, wy;

    XtVaGetValues(w, XmNx, &wx, XmNy, &wy, NULL);
    *xp += wx;
    *yp += wy;
}


/*
 * Part fix for bug id 4017222. Return the root widget of the Widget parameter.
 */
Widget
getRootWidget(Widget w) {
    if(!w) return NULL;

    if(XtParent(w))
        return getRootWidget(XtParent(w));
    else
        return w;
}

#define ABS(x) ((x) < 0 ? -(x) : (x))

/* This proc is the major AWT engine for processing X events
 * for Java components and is the proc responsible for taking
 * X events and posting their corresponding Java event to the
 * AWT EventQueue.  It is set up to be called both from an Xt
 * event handler And directly from MToolkit.c:dispatchToWidget().
 * For the latter case, the "passEvent" parameter will be true,
 * which means that the event is being posted on teh Java queue
 * BEFORE it is being passed to Xt and so a copy of the X event
 * must be stored within the Java event structure so it can be
 * dispatched to Xt later on.
 */
void 
awt_canvas_handleEvent(Widget w, XtPointer client_data,
                       XEvent * event, struct WidgetInfo *winfo,
                       Boolean * cont, Boolean passEvent)
{
    static int clickCount = 1;
    static XtPointer lastPeer = 0;
    static Time lastTime = 0;
    static int lastx = 0;
    static int lasty = 0;
    static int rbutton = 0;
    static int lastButton = 0;
    Boolean popupTrigger;
    int x, y;
    int modifiers = 0;
    int fullRelease = 0;

    /* Any event handlers which take peer instance pointers as
     * client_data should check to ensure the widget has not been
     * marked as destroyed as a result of a dispose() call on the peer
     * (which can result in the peer instance pointer already haven
     * been gc'd by the time this event is processed)
     */
    if (w->core.being_destroyed) {
        return;
    }
    *cont = FALSE;

    switch (event->type) {
        case SelectionClear:
        case SelectionNotify:
        case SelectionRequest:
            *cont = TRUE;
            break;
        case GraphicsExpose:
        case Expose:
            HandleExposeEvent(w, (jobject) client_data, event);
            break;
        case FocusIn:
        case FocusOut:
            handleFocusEvent(w, (XFocusChangeEvent *) event, client_data, cont, passEvent);
            break;
        case ButtonPress:
            x = event->xbutton.x;
            y = event->xbutton.y;

            if (lastPeer == client_data && 
		lastButton == event->xbutton.button &&
                (event->xbutton.time - lastTime) <= awt_multiclick_time) {
                    clickCount++;
	    } else {
                clickCount = 1;
                lastPeer = client_data;
                lastButton = event->xbutton.button;
                lastx = x;
                lasty = y;
            }
            lastTime = event->xbutton.time;

            /* XXX: Button2Mask and Button3Mask don't get set on ButtonPress */
            /* events properly so we set them here so getModifiers will do */
            /* the right thing. */
            switch (event->xbutton.button) {
		/*
		 * Added case for Button1Mask since it wasn't getting set
		 * either for ButtonPress events.
		 *
		 * The state field represents the state of the pointer
		 * buttons and modifier keys just *prior* to the event
		 * which is why it is never set on ButtonPress.
		 */
                case 1:
                    event->xbutton.state |= Button1Mask;
                    break;
                case 2:
                    event->xbutton.state |= Button2Mask;
                    break;
                case 3:
                    event->xbutton.state |= Button3Mask;
                    break;
                default:
                    break;
            }
            modifiers = getModifiers(event->xbutton.state);
	    
            /* If the widget is a subwidget on a component we need to
             * translate the x,y into the coordinate space of the component.
             */
            if (winfo != NULL && winfo->widget != winfo->origin) {
                translateXY(winfo->widget, &x, &y);
            }
	    
	    /* (4168006) Find out out how many buttons we have
	     * If this is a two button system Right == 2
	     * If this is a three button system Right == 3
	     */
            if ( rbutton == 0 ) {
                unsigned char map[5];
                rbutton = XGetPointerMapping ( awt_display, map, 3 );
            }
	    
            if (event->xbutton.button == rbutton || event->xbutton.button > 2) {
                popupTrigger = True;
            } else {
                popupTrigger = False;
            }
	    
            awt_post_java_mouse_event(client_data,
                                      java_awt_event_MouseEvent_MOUSE_PRESSED,
                                      (passEvent == TRUE) ? 
					   (long) awt_copyXEvent(event) :
					   (long) 0,
                                      ((jlong)(event->xbutton.time)),
                                      modifiers,
                                      x, y,
                                      clickCount,
                                      popupTrigger);

            trackInputFocus(w);

            break;
        case ButtonRelease:
            prevWidget = 0;
            x = event->xbutton.x;
            y = event->xbutton.y;
            modifiers = getModifiers(event->xbutton.state);
	    
	    fullRelease =
	      ((event->xbutton.state & Button1Mask) &&
	       !(event->xbutton.state & Button2Mask) &&
	       !(event->xbutton.state & Button3Mask) &&
	       (event->xbutton.button == Button1)) ||
	      (!(event->xbutton.state & Button1Mask) &&
	       (event->xbutton.state & Button2Mask) &&
	       !(event->xbutton.state & Button3Mask) &&
	       (event->xbutton.button == Button2)) ||
	      (!(event->xbutton.state & Button1Mask) &&
	       !(event->xbutton.state & Button2Mask) &&
	       (event->xbutton.state & Button3Mask) &&
	       (event->xbutton.button == Button3));

            /* If the widget is a subwidget on a component we need to
             * translate the x,y into the coordinate space of the component.
             */
            if (winfo != NULL && winfo->widget != winfo->origin) {
                translateXY(winfo->widget, &x, &y);
            }
	    awt_post_java_mouse_event(client_data,
					java_awt_event_MouseEvent_MOUSE_RELEASED,
					(passEvent == TRUE) ? 
					  (long) awt_copyXEvent(event) : (long) 0,
					((jlong)(event->xbutton.time)),
					modifiers,
					x, y,
					clickCount,
					FALSE);
	      
	    if (lastPeer == client_data) {
                awt_post_java_mouse_event(client_data,
                                          java_awt_event_MouseEvent_MOUSE_CLICKED,
                                          (long) 0,
                                          ((jlong)(event->xbutton.time)),
                                          modifiers,
                                          x, y,
                                          clickCount,
                                          FALSE);
            }

	    if (fullRelease) {
	        updateCursor(client_data, UPDATE_ONLY);
	    }

        break;
        case MotionNotify:
            x = event->xmotion.x;
            y = event->xmotion.y;

            /* If a motion comes in while a multi-click is pending,
             * allow a smudge factor so that moving the mouse by a small
             * amount does not wipe out the multi-click state variables.
             */
            if (!(lastPeer == client_data &&
                  ((event->xmotion.time - lastTime) <= awt_multiclick_time) &&
                  (ABS(lastx - x) < awt_multiclick_smudge &&
                   ABS(lasty - y) < awt_multiclick_smudge))) {
                clickCount = 0;
                lastTime = 0;
                lastPeer = 0;
                lastx = 0;
                lasty = 0;
            }
            modifiers = getModifiers(event->xmotion.state);

            /* If the widget is a subwidget on a component we need to
             * translate the x,y into the coordinate space of the component.
             */
            if (winfo != NULL && winfo->widget != winfo->origin) {
                translateXY(winfo->widget, &x, &y);
            }
            if (event->xmotion.state & (Button1Mask | Button2Mask | Button3Mask)) {
                if (!clickCount) {

            /*
                Fix for bug id 4017222. A button is down, so EnterNotify and
                LeaveNotify events are only being sent to this widget. If
                the pointer has moved over a new widget, manually generate
                MouseEnter and MouseExit and send them to the right widgets.
            */

	        extern Widget awt_WidgetAtXY(Widget root, Position x, Position y);
                extern Widget awt_GetWidgetAtPointer();
                Widget currentWidget=0, topLevelW;
                Position wx=0, wy=0;

                XtTranslateCoords(w, x, y, &wx, &wy);
                /* Get the top level widget underneath the mouse pointer */
                currentWidget = awt_GetWidgetAtPointer();
                /* Get the exact widget at the current XY from the top level */
                currentWidget = awt_WidgetAtXY(currentWidget, wx, wy);
                if(prevWidget && prevWidget!=w && currentWidget!=prevWidget
			&& !prevWidget->core.being_destroyed) {
                    XtPointer userData=0;
                    XtVaGetValues(prevWidget, XmNuserData, &userData, NULL);
                    if (userData) {
			awt_post_java_mouse_event(userData,
			    java_awt_event_MouseEvent_MOUSE_EXITED,
			    (passEvent==TRUE) ? (long)awt_copyXEvent(event) : (long)0,
			    event->xmotion.time,
			    0,
			    x, y,
			    clickCount,
			    FALSE);
                    }
                }
 
                if(currentWidget && currentWidget!=w && currentWidget!=prevWidget) {
                    XtPointer userData=0;
                    XtVaGetValues(currentWidget, XmNuserData, &userData, NULL);
                    if (userData) {
			awt_post_java_mouse_event(userData,
			    java_awt_event_MouseEvent_MOUSE_ENTERED,
			    (passEvent==TRUE) ? (long)awt_copyXEvent(event) : (long)0,
			    event->xmotion.time,
			    0,
			    x, y,
			    clickCount,
			    FALSE);
                    }
					
	        	updateCursor(userData, CACHE_ONLY);
					awt_util_setCursor(currentWidget, NULL);
                }
 
                prevWidget = currentWidget;
                /* end 4017222 */


                awt_post_java_mouse_event(client_data,
                                          java_awt_event_MouseEvent_MOUSE_DRAGGED,
                                          (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
                                          ((jlong)(event->xmotion.time)),
                                          modifiers,
                                          x, y,
                                          clickCount,
                                          FALSE);

            }
            } else {

                awt_post_java_mouse_event(client_data,
                                          java_awt_event_MouseEvent_MOUSE_MOVED,
                                          (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
                                          ((jlong)(event->xmotion.time)),
                                          modifiers,
                                          x, y,
                                          clickCount,
                                          FALSE);
            }
            break;
        case KeyPress:
            clickCount = 0;
            lastTime = 0;
            lastPeer = 0;
            handleKeyEvent(java_awt_event_KeyEvent_KEY_PRESSED,
                           event, client_data, cont, passEvent);
            break;
        case KeyRelease:
            clickCount = 0;
            lastTime = 0;
            lastPeer = 0;
            handleKeyEvent(java_awt_event_KeyEvent_KEY_RELEASED,
                           event, client_data, cont, passEvent);
            break;
        case EnterNotify:
        case LeaveNotify:
/*
  printf("----->%s on %s(%x):mode=%d detail = %d\n",
  event->type == EnterNotify?"EnterNotify":"LeaveNotify",
  XtName(w), w,
  ((XCrossingEvent*)event)->mode, ((XCrossingEvent*)event)->detail);      
*/
        if (event->xcrossing.mode != NotifyNormal ||
                (event->xcrossing.detail == NotifyVirtual &&
                 !XtIsSubclass(w, xmScrolledWindowWidgetClass))) {
                *cont = TRUE;
                return;
            }
            clickCount = 0;
            lastTime = 0;
            lastPeer = 0;
            switch (event->type) {
                case EnterNotify:
                    awt_post_java_mouse_event(client_data,
                                              java_awt_event_MouseEvent_MOUSE_ENTERED,
                                              (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
                                              ((jlong)(event->xcrossing.time)),
                                              0,
                                              event->xcrossing.x, event->xcrossing.y,
                                              clickCount,
                                              FALSE);
            	    if (!(event->xcrossing.state 
			& (Button1Mask | Button2Mask | Button3Mask))) {
		        updateCursor(client_data, CACHE_UPDATE);
		    }

                    break;
                case LeaveNotify:
                    awt_post_java_mouse_event(client_data,
                                              java_awt_event_MouseEvent_MOUSE_EXITED,
                                              (passEvent == TRUE) ? (long) awt_copyXEvent(event) : (long) 0,
                                              ((jlong)(event->xcrossing.time)),
                                              0,
                                              event->xcrossing.x, event->xcrossing.y,
                                              clickCount,
                                              FALSE);
                    break;
            }
            break;

        default:
            break;
    }
}

/*
 * client_data is MComponentPeer subclass
 */
void
awt_canvas_event_handler(Widget w, XtPointer client_data,
                         XEvent * event, Boolean * cont)
{
    awt_canvas_handleEvent(w, client_data, event, NULL, cont, FALSE);
}

void
awt_canvas_reconfigure(struct FrameData *wdata)
{
    Dimension w, h;

    if (wdata->winData.comp.widget == 0 ||
        XtParent(wdata->winData.comp.widget) == 0) {
        return;
    }
    XtVaGetValues(XtParent(wdata->winData.comp.widget), XmNwidth, &w, XmNheight, &h, NULL);
    XtConfigureWidget(wdata->winData.comp.widget,
                      -(wdata->left),
                      -(wdata->top),
                      w + (wdata->left + wdata->right),
                      h + (wdata->top + wdata->bottom),
                      0);
}

static void
Wrap_event_handler(Widget widget,
                   XtPointer client_data,
                   XmDrawingAreaCallbackStruct * call_data)
{
    awt_canvas_reconfigure((struct FrameData *) client_data);
}


Widget
awt_canvas_create(XtPointer this,
                  Widget parent,
		  char *base,
                  long width,
                  long height,
		  Boolean parentIsFrame,
                  struct FrameData *wdata,
                  AwtGraphicsConfigDataPtr awtData)
{
    Widget newCanvas;
    Widget wrap;
#define MAX_ARGC 20
    Arg args[MAX_ARGC];
    int argc;
    char name[128];
    static XtTranslations translationKeyDown = NULL;

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);


    if (parent == 0) {
        JNU_ThrowNullPointerException(env, "NullPointerException");
        return 0;
    }
    if (width == 0) {
        width = 1;
    }
    if (height == 0) {
        height = 1;
    }

    if (wdata != 0) {
        argc = 0;
	if  (!parentIsFrame)
	{
	    XtSetArg(args[argc], XmNwidth, (XtArgVal) width);
	    argc++;
	    XtSetArg(args[argc], XmNheight, (XtArgVal) height);
	    argc++;
	}
        XtSetArg(args[argc], XmNmarginWidth, 0);
        argc++;
        XtSetArg(args[argc], XmNmarginHeight, 0);
        argc++;
        XtSetArg(args[argc], XmNspacing, 0);
        argc++;
        XtSetArg(args[argc], XmNresizePolicy, XmRESIZE_NONE);
        argc++;
        strcpy(name, base);
        strcat(name, "wrap");
        
        DASSERT(!(argc > MAX_ARGC));
        wrap = XmCreateDrawingArea(parent, name, args, argc);
	if  (!parentIsFrame)
	{
	    /* Fixing bugs in frame module (awt_Frame.c).  It will now
	       provide the resize handling for this inner/parent canvas.*/
	    XtAddCallback(wrap, XmNresizeCallback,
			  (XtCallbackProc) Wrap_event_handler, wdata);
	}
        XtManageChild(wrap);
    } else {
        wrap = parent;
    }

    strcpy(name, base);
    strcat(name, "canvas");

    argc = 0;
    XtSetArg(args[argc], XmNspacing, 0);
    argc++;
    if  (!parentIsFrame)
    {
	XtSetArg(args[argc], XmNwidth, (XtArgVal) width);
	argc++;
	XtSetArg(args[argc], XmNheight, (XtArgVal) height);
	argc++;
    }
    XtSetArg(args[argc], XmNmarginHeight, 0);
    argc++;
    XtSetArg(args[argc], XmNmarginWidth, 0);
    argc++;
    XtSetArg(args[argc], XmNresizePolicy, XmRESIZE_NONE);
    argc++;
    XtSetArg(args[argc], XmNuserData, (XtPointer) this);
    argc++;
    /* Fixed 4059430, 3/11/98, robi.khan@eng
     * install insert proc callback so components are ordered correctly
     * when added directly to frame/dialogs/windows 
     */
    XtSetArg(args[argc], XmNinsertPosition, awt_util_insertCallback);
    argc++;

    if (awtData != getDefaultConfig(awtData->awt_visInfo.screen)) {
        XtSetArg (args[argc], XtNvisual, awtData->awt_visInfo.visual); argc++;
        XtSetArg (args[argc], XmNdepth, awtData->awt_depth); argc++;
        XtSetArg (args[argc], XmNscreen,
                  ScreenOfDisplay(awt_display,
                                  awtData->awt_visInfo.screen)); argc++;

        if (awtData->awt_cmap == (Colormap) NULL) {
            awtJNI_CreateColorData (env, awtData, 1);
        }

        XtSetArg (args[argc], XmNcolormap, awtData->awt_cmap); argc++;
    
        DASSERT(!(argc > MAX_ARGC));
        newCanvas = XtCreateWidget(name, vDrawingAreaClass, wrap, 
                                   args, argc);

    } else {
        newCanvas = XmCreateDrawingArea(wrap, name, args, argc);
    }

    XtSetMappedWhenManaged(newCanvas, False);
    XtManageChild(newCanvas);
/*
  XXX: causes problems on 2.5
  if (!scrollBugWorkAround) {
  awt_setWidgetGravity(newCanvas, StaticGravity);
  }
*/
    /* Fixed 4250354 7/28/99 ssi@sparc.spb.su
     * XtParseTranslationTable leaks in old ver of Xtoolkit
     * and result should be deletetd in any case
     * 
     * XtOverrideTranslations(newCanvas,
     *                      XtParseTranslationTable("<KeyDown>:DrawingAreaInput()"));
     */
    if (NULL==translationKeyDown) 
        translationKeyDown=XtParseTranslationTable("<KeyDown>:DrawingAreaInput()");
    XtOverrideTranslations(newCanvas,translationKeyDown);

    XtSetSensitive(newCanvas, True);

    return newCanvas;
}

static void 
messWithGravity(Widget w, int gravity)
{
    extern void awt_changeAttributes(Display * dpy, Widget w,
                                     unsigned long mask,
                                     XSetWindowAttributes * xattr);
    XSetWindowAttributes xattr;

    xattr.bit_gravity = gravity;
    xattr.win_gravity = gravity;

    awt_changeAttributes(XtDisplay(w), w,
                         CWBitGravity | CWWinGravity, &xattr);
}

struct MoveRecord {
    long dx;
    long dy;
};

void
moveWidget(Widget w, void *data)
{
    struct MoveRecord *rec = (struct MoveRecord *) data;

    if (XtIsRealized(w) && XmIsRowColumn(w)) {
        w->core.x -= rec->dx;
        w->core.y -= rec->dy;
    }
}

#if 0
/* Scroll entire contents of window by dx and dy.  Currently only
   dy is supported.  A negative dy means scroll backwards, i.e.,
   contents in window move down. */
void 
awt_canvas_scroll(XtPointer this,
                  struct CanvasData *wdata,
                  long dx,
                  long dy)
{

    Window win;
    XWindowChanges xchgs;
    Window root;
    int x, y;
    unsigned int width, height, junk;
    Display *dpy;
    struct MoveRecord mrec;

    mrec.dx = dx;
    mrec.dy = dy;

    dpy = XtDisplay(wdata->comp.widget);
    win = XtWindow(wdata->comp.widget);

    /* REMIND: consider getting rid of this! */
    XGetGeometry(awt_display,
                 win,
                 &root,
                 &x,
                 &y,
                 &width,
                 &height,
                 &junk,
                 &junk);

    /* we need to actually update the coordinates for manager widgets, */
    /* otherwise the parent won't pass down events to them properly */
    /* after scrolling... */
    awt_util_mapChildren(wdata->comp.widget, moveWidget, 0, &mrec);

    if (dx < 0) {
        /* scrolling backward */

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, NorthWestGravity);
        }
        xchgs.x = x + dx;
        xchgs.y = y;
        xchgs.width = width - dx;
        xchgs.height = height;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY | CWWidth | CWHeight,
                         &xchgs);

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, NorthWestGravity);
        }
        xchgs.x = x;
        xchgs.y = y;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY,
                         &xchgs);

        xchgs.width = width;
        xchgs.height = height;
        XConfigureWindow(awt_display,
                         win,
                         CWWidth | CWHeight,
                         &xchgs);
    } else {
        /* forward scrolling */

        /* make window a little taller */
        xchgs.width = width + dx;
        xchgs.height = height;
        XConfigureWindow(awt_display,
                         win,
                         CWWidth | CWHeight,
                         &xchgs);

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, NorthEastGravity);
        }
        /* move window by amount we're scrolling */
        xchgs.x = x - dx;
        xchgs.y = y;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY,
                         &xchgs);

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, NorthWestGravity);
        }
        /* resize to original size */
        xchgs.x = x;
        xchgs.y = y;
        xchgs.width = width;
        xchgs.height = height;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY | CWWidth | CWHeight,
                         &xchgs);
    }
    /* Because of the weird way we're scrolling this window,
       we have to eat all the exposure events that result from
       scrolling forward, and translate them up by the amount we're
       scrolling by.

       Rather than just eating all the exposures and having the
       java code fill in what it knows is exposed, we do it this
       way.  The reason is that there might be some other exposure
       events caused by overlapping windows on top of us that we
       also need to deal with. */
    {
        XRectangle rect;

        rect.x = -1;
        eatAllExposures(dpy, win, &rect);
        if (rect.x != -1) {         /* we got at least one expose event */
            if (dx > 0) {
                rect.x -= dx;
                rect.width += dx;
            }
/*
  printf("EXPOSE (%d): %d, %d, %d, %d\n",
  dy, rect.x, rect.y, rect.width, rect.height);
*/
            callJavaExpose(this, &rect);
            XSync(awt_display, False);
        }
    }
    if (dy < 0) {
        /* scrolling backward */

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, SouthGravity);
        }
        xchgs.x = x;
        xchgs.y = y + dy;
        xchgs.width = width;
        xchgs.height = height - dy;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY | CWWidth | CWHeight,
                         &xchgs);

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, NorthWestGravity);
        }
        xchgs.x = x;
        xchgs.y = y;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY,
                         &xchgs);

        xchgs.width = width;
        xchgs.height = height;
        XConfigureWindow(awt_display,
                         win,
                         CWWidth | CWHeight,
                         &xchgs);
    } else {
        /* forward scrolling */

        /* make window a little taller */
        xchgs.width = width;
        xchgs.height = height + dy;
        XConfigureWindow(awt_display,
                         win,
                         CWWidth | CWHeight,
                         &xchgs);

        /* move window by amount we're scrolling */
        xchgs.x = x;
        xchgs.y = y - dy;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY,
                         &xchgs);

        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, SouthGravity);
        }
        /* resize to original size */
        xchgs.x = x;
        xchgs.y = y;
        xchgs.width = width;
        xchgs.height = height;
        XConfigureWindow(awt_display,
                         win,
                         CWX | CWY | CWWidth | CWHeight,
                         &xchgs);
        if (scrollBugWorkAround) {
            messWithGravity(wdata->comp.widget, NorthWestGravity);
        }
    }
    /* Because of the weird way we're scrolling this window,
       we have to eat all the exposure events that result from
       scrolling forward, and translate them up by the amount we're
       scrolling by.

       Rather than just eating all the exposures and having the
       java code fill in what it knows is exposed, we do it this
       way.  The reason is that there might be some other exposure
       events caused by overlapping windows on top of us that we
       also need to deal with. */
    {
        XRectangle rect;

        rect.x = -1;
        eatAllExposures(dpy, win, &rect);
        if (rect.x != -1) {         /* we got at least one expose event */
            if (dy > 0) {
                rect.y -= dy;
                rect.height += dy;
            }
            if (dx > 0) {
                rect.x -= dx;
                rect.width += dx;
            }
/*
  printf("EXPOSE (%d): %d, %d, %d, %d\n",
  dy, rect.x, rect.y, rect.width, rect.height);
*/
            callJavaExpose(this, &rect);
            XSync(awt_display, False);
        }
    }
}
#endif

/* 
 * This function is stolen from /src/solaris/hpi/src/system_md.c 
 * It is used in setting the time in Java-level InputEvents 
 */
jlong
sysTimeMillis()
{
    struct timeval t;
    gettimeofday(&t, 0);
    return ((jlong)t.tv_sec) * 1000 + (jlong)(t.tv_usec/1000);
}

/*
 * client_data is MComponentPeer instance
 */
void
awt_post_java_key_event(XtPointer client_data, long id, long eventdata,
                        jlong when, long keycode, jchar keychar, long modifiers)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject peer = (jobject) client_data;
    jobject target;
    static jclass classKeyEvent = NULL;
    static jmethodID mid = NULL;
    char *clsName = "java/awt/event/KeyEvent";
    jobject hEvent;

    if ((*env)->PushLocalFrame(env, 16) < 0)
        return;

    target = (*env)->GetObjectField(env, peer, mComponentPeerIDs.target);

    if (classKeyEvent == NULL) {
        jobject sysClass;

        sysClass = (*env)->FindClass(env, clsName);
        if (sysClass != NULL) {
            /* Make this class 'sticky', we don't want it GC'd */
            classKeyEvent = (*env)->NewGlobalRef(env, sysClass);
            mid = (*env)->GetMethodID(env, classKeyEvent
                                      ,"<init>"
                                      ,"(Ljava/awt/Component;IJIIC)V");
        }
        if (JNU_IsNull(env, classKeyEvent) || mid == 0) {
            JNU_ThrowClassNotFoundException(env, clsName);
	    (*env)->PopLocalFrame(env, 0);
            return;
        }
    }

    /* Check for the Input Method switch hot key.  If it is, pop up an input
     * method selection menu.
     */
    if ((id == java_awt_event_KeyEvent_KEY_PRESSED) &&
        checkInputMethodSelectionKey(env, keycode, modifiers)) {
	popUpInputMethodSelectionMenu(env, target);
	(*env)->PopLocalFrame(env, 0);
        return;
    }

    /* The time field in an X event is usually the number of milliseconds 
     * since the last server reset (see page 621 of Volume 4 of the O'Reilly 
     * Motif series books).  The time field in a Java InputEvent is the UTC time, 
     * so we should use that time instead when creating Java InputEvents.  
     */
    when = sysTimeMillis();
    hEvent = (*env)->NewObject(env, classKeyEvent, mid
                               ,target, id, when, modifiers
                               ,keycode, keychar);

    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
    if (JNU_IsNull(env, hEvent)) {
        JNU_ThrowNullPointerException(env, "NullPointerException: constructor failed.");
	(*env)->PopLocalFrame(env, 0);
        return;
    }
    (*env)->SetLongField(env, hEvent, awtEventIDs.data, (jlong) eventdata);

    #ifdef DEBUG
    if (debugKeys) {
        jio_fprintf(stderr, "native posting event id:%d  keychar:%c\n", (int)id, (char)keychar);
    }
    #endif
    JNU_CallMethodByName(env, NULL, peer, 
			 "postEvent", "(Ljava/awt/AWTEvent;)V", hEvent);
    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
    (*env)->PopLocalFrame(env, 0);
} /* awt_post_java_key_event() */

/*
 * client_data is MComponentPeer instance
 */
void 
awt_post_java_focus_event(XtPointer client_data
                          ,long id
                          ,long eventdata
                          ,Boolean isTemp)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject peer = (jobject) client_data;
    jobject target;
    static jclass classFocusEvent = NULL;
    static jmethodID mid = NULL;
    char *clsName = "java/awt/event/FocusEvent";
    jobject hEvent;

    if ((*env)->PushLocalFrame(env, 16) < 0)
        return;

    target = (*env)->GetObjectField(env, peer, mComponentPeerIDs.target);

    if (classFocusEvent == NULL) {
        jobject sysClass;

        sysClass = (*env)->FindClass(env, clsName);
        if (sysClass != NULL) {
            /* Make this class 'sticky', we don't want it GC'd */
            classFocusEvent = (*env)->NewGlobalRef(env, sysClass);
            mid = (*env)->GetMethodID(env, classFocusEvent
                                      ,"<init>"
                                      ,"(Ljava/awt/Component;IZ)V");
        }
        if (JNU_IsNull(env, classFocusEvent) || mid == 0) {
            JNU_ThrowClassNotFoundException(env, clsName);
	    (*env)->PopLocalFrame(env, 0);
            return;
        }
    }
    hEvent = (*env)->NewObject(env, classFocusEvent, mid
                               ,target, id, isTemp);

    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
    if (JNU_IsNull(env, hEvent)) {
        JNU_ThrowNullPointerException(env, "NullPointerException: constructor failed.");
	(*env)->PopLocalFrame(env, 0);
        return;
    }
    (*env)->SetLongField(env, hEvent, awtEventIDs.data, (jlong) eventdata);

    JNU_CallMethodByName(env, NULL, peer, 
			 "postEvent", "(Ljava/awt/AWTEvent;)V", hEvent);
    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
    (*env)->PopLocalFrame(env, 0);

}

/*
 * client_data is MComponentPeer instance
 */
void
awt_post_java_mouse_event(XtPointer client_data, long id, long eventdata,
                          jlong when, long modifiers, int x, int y, int clickcount,
                          Boolean popuptrigger)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject peer = (jobject) client_data;
    jobject target;
    static jclass classMouseEvent = NULL;
    static jmethodID mid = NULL;
    char *clsName = "java/awt/event/MouseEvent";
    jobject hEvent;

    if ((*env)->PushLocalFrame(env, 16) < 0)
        return;

    target = (*env)->GetObjectField(env, peer, mComponentPeerIDs.target);

    if (classMouseEvent == NULL) {
        jobject sysClass;

        sysClass = (*env)->FindClass(env, clsName);
        if (sysClass != NULL) {
            /* Make this class 'sticky', we don't want it GC'd */
            classMouseEvent = (*env)->NewGlobalRef(env, sysClass);
            mid = (*env)->GetMethodID(env, classMouseEvent
                                      ,"<init>"
                                      ,"(Ljava/awt/Component;IJIIIIZ)V");
        }
        if (JNU_IsNull(env, classMouseEvent) || mid == 0) {
            JNU_ThrowClassNotFoundException(env, clsName);
	    (*env)->PopLocalFrame(env, 0);
            return;
        }
    }

    /* The time field in an X event is usually the number of milliseconds 
     * since the last server reset (see page 621 of Volume 4 of the O'Reilly 
     * Motif series books).  The time field in a Java InputEvent is the UTC time, 
     * so we should use that time instead when creating Java InputEvents.  
     */
    when = sysTimeMillis();
    hEvent = (*env)->NewObject(env, classMouseEvent, mid
                               ,target, id, when, modifiers
                               ,x, y, clickcount, popuptrigger);

    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
    if (JNU_IsNull(env, hEvent)) {
        JNU_ThrowNullPointerException(env, "NullPointerException: constructor failed.");
	(*env)->PopLocalFrame(env, 0);
        return;
    }

    (*env)->SetLongField(env, hEvent, awtEventIDs.data, (jlong) eventdata);

    JNU_CallMethodByName(env, NULL, peer, 
			 "postEvent", "(Ljava/awt/AWTEvent;)V", hEvent);
    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        (*env)->ExceptionClear(env);
    }
    (*env)->PopLocalFrame(env, 0);
}

Boolean lookForInputMethodSelectionKey(JNIEnv * env)
{
    char * selectionKeyString = getenv("JAVA_INPUT_METHOD_SELECTION_KEY");
    char * selectionModifiersString = getenv("JAVA_INPUT_METHOD_SELECTION_KEY_MODIFIERS");
    jboolean hasException = True;
    Boolean enabled = False;
    jvalue jv;

    if (selectionKeyString != NULL) {
	jv = JNU_GetStaticFieldByName(env, &hasException, 
                                      "java/awt/event/KeyEvent",
                                      selectionKeyString, "I");
	enabled = !hasException;
    }

    if (enabled) {
	selectionKey = (long)jv.i;
	
	if (selectionModifiersString != NULL) {
	    char * token = strtok(selectionModifiersString, ",");
	    while (token != NULL) {
		long aMask;
		char *p;

		/* strip blanks */                
		while (*token == ' ') {
		    token++;
		}
		p = strrchr(token, ' ');
		if (p != NULL) {
		    *p = '\0';
		}

		hasException = True;
		jv = JNU_GetStaticFieldByName(env, &hasException,
                                              "java/awt/event/InputEvent",
                                              token, "I");
		if (!hasException) {
		    aMask = (long)jv.i,
		    selectionModifiers |= aMask;
		}
		token = strtok(NULL, ",");
	    }
	}
    }

    return enabled;
}
    

Boolean checkInputMethodSelectionKey(JNIEnv *env, long keycode, long modifiers)
{
    static Boolean examined = False;
    static Boolean enabled = False;

    /* Check for the environment variables at the first time */
    if (!examined) {
	enabled = lookForInputMethodSelectionKey(env);
	examined = True;
    }

    /* Now compare the input key event with the selection key definition */
    return (enabled &&
            (keycode != java_awt_event_KeyEvent_VK_UNDEFINED) &&
            (selectionKey == keycode) &&
            ((modifiers & selectionModifiers) == selectionModifiers));
}


void popUpInputMethodSelectionMenu(JNIEnv * env, jobject target)
{
    jvalue jv;
    jboolean hasException = True;

    /* Get the instance of ExecutableInputMethodManager */
    jv = JNU_CallStaticMethodByName(env, &hasException,
                                    "sun/awt/im/InputMethodManager",
                                    "getInstance",
                                    "()Lsun/awt/im/InputMethodManager;");

    if (!hasException) {
	JNU_CallMethodByName(env, &hasException,
                             jv.l, "notifyChangeRequestByHotKey",
                             "(Ljava/awt/Component;)V", target);
    }
}
