/*
 * @(#)tiny_Font.c	1.12 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "common_exceptions.h"
#include "llni.h"
#include "sun_awt_tiny_TinyFontMetrics.h"
#include "sun_awt_tiny_TinyWindow.h"
#include "tiny.h"

static char 	*defaultfontname = "fixed";
static char 	*defaultfoundry = "misc";
static char	*anyfoundry = "*";
static char	*anystyle = "*-*";
static char	*isolatin1 = "iso8859-1";

static int
FontName(Ijava_lang_String name,
	 char **foundry, char **facename, char **encoding)
{
    char *cname;

    if (LLNI_isNull(name)) {
	return 0;
    }
    cname = javaString2CStringAlloc(name);

    if (strcmp(cname,"Helvetica")==0) {
	*foundry = "adobe";
	*facename = "helvetica";
	*encoding = isolatin1;
    } else if (strcmp(cname,"TimesRoman") == 0) {
	*foundry = "adobe";
	*facename = "times";
	*encoding = isolatin1;
    } else if (strcmp(cname,"Courier") == 0) {
	*foundry = "adobe";
	*facename = "courier";
	*encoding = isolatin1;
    } else if (strcmp(cname,"Dialog") == 0) {
	*foundry = "b&h";
	*facename = "lucida";
	*encoding = isolatin1;
    } else if (strcmp(cname,"DialogInput") == 0) {
	*foundry = "b&h";
	*facename = "lucidatypewriter";
	*encoding = isolatin1;
    } else if (strcmp(cname,"ZapfDingbats") == 0) {
	*foundry = "itc";
	*facename = "zapfdingbats";
	*encoding = "*-*";
    } else {
#ifdef DEBUG
	jio_fprintf(stderr, "Unknown font: %s\n", cname);
#endif
	*foundry = defaultfoundry;
	*facename = defaultfontname;
	*encoding = isolatin1;
    }
    sysFree(cname);
    return 1;
}

    
static char *
Style(long s)
{
    switch (s) {
      case java_awt_Font_ITALIC:
	return "medium-i";
      case java_awt_Font_BOLD:
	return "bold-r";
      case java_awt_Font_BOLD+java_awt_Font_ITALIC:
	return "bold-i";
      case java_awt_Font_PLAIN:
      default:
	return "medium-r";
    }
}

XFontStruct *
awt_getFont(Ijava_awt_Font font)
{
    Display		*display;
    char		fontSpec[1024];
    long		height;
    long		oheight;
    int			above = 0; /* tries above height */
    int			below = 0; /* tries below height */
    char		*foundry;
    char		*name;
    char		*encoding;
    char		*style;
    java_int		fstyle;
    java_int		pData;
    XFontStruct		*xfont;
    bool_t		found;

    if (LLNI_isNull(font)) {
	return 0;
    }
    display = awt_display;

    LLNI_getValField(font, pData, pData);
    xfont = (XFontStruct *)pData;
    if (xfont) {
	return xfont;
    }

    beginLocalRoots1(Ijava_lang_String, fname);
       LLNI_getRefField(font, family, fname);
       found = FontName(fname, &foundry, &name, &encoding);
    endLocalRoots;
    if (!found) {
	return 0;
    }

    LLNI_getValField(font, style, fstyle);
    style = Style(fstyle);
    LLNI_getValField(font, size, height);
    oheight = height;

    while (1) {
	jio_snprintf(fontSpec,sizeof(fontSpec),"-%s-%s-%s-*-*-%d-*-*-*-*-*-%s",
		     foundry,
		     name,
		     style,
		     height,
		     encoding);

	/*fprintf(stderr,"LoadFont: %s\n", fontSpec);*/
	xfont = XLoadQueryFont(display, fontSpec);
	/* XXX: sometimes XLoadQueryFont returns a bogus font structure */
	/* with negative ascent. */
	if (xfont == 0 || xfont->ascent < 0) {
	    if (xfont) {
		XFreeFont(display, xfont);
	    }
	    if (foundry != anyfoundry) {
		/* Try any other foundry before messing with the sizes */
		foundry = anyfoundry;
		continue;
	    }
	    /* We couldn't find the font. We'll try to find an */
	    /* alternate by searching for heights above and below our */
	    /* preferred height. We try for 4 heights above and below. */
	    /* If we still can't find a font we repeat the algorithm */
	    /* using misc-fixed as the font. If we then fail, then we */
	    /* give up and signal an error. */
	    if (above == below) {
		above++;
		height = oheight + above;
	    } else {
		below++;
		if (below > 4) {
		    if (name != defaultfontname || style != anystyle) {
			name = defaultfontname;
			foundry = defaultfoundry;
			height = oheight;
			style = anystyle;
			encoding = isolatin1;
			above = below = 0;
			continue;
		    } else {
			return 0;
		    }
		}
		height = oheight - below;
	    }
	    continue;
	} else {
	    LLNI_setValField(font, pData, (long)xfont);
	    return xfont;
	}
    }
}

void 
sun_awt_tiny_TinyFontMetrics_init(Isun_awt_tiny_TinyFontMetrics this)
{
    XFontStruct	*xfont;
    int		ccount;
    int		i;
    java_int as,ds,ld;
    int off;
    bool_t ok;
    ExecEnv *ee = EE();

    AWT_LOCK();
    beginLocalRoots2(Ijava_awt_Font, font,
		     IArrayOfInt, widths);
       LLNI_getRefField(this, font, font);
       xfont = awt_getFont(font);

       if (xfont == 0) {
	  SignalError(0, JAVAPKG "NullPointerException", 0);
	  AWT_UNLOCK();
	  rootReturnVoid;
       }

       LLNI_beginInconsistent(ee);
          as = xfont->ascent;
	  writeNamedFieldVal32(LLNI_unhand(this), ascent,  as);
	  ds = xfont->descent;
	  writeNamedFieldVal32(LLNI_unhand(this), descent, ds);
	  ld = 1;
	  writeNamedFieldVal32(LLNI_unhand(this), leading, ld);
	  writeNamedFieldVal32(LLNI_unhand(this), height,  as + ds + ld);
	  as = xfont->max_bounds.ascent;
	  writeNamedFieldVal32(LLNI_unhand(this), maxAscent,  as);
	  ds = xfont->max_bounds.descent;
	  writeNamedFieldVal32(LLNI_unhand(this), maxDescent, ds);
	  writeNamedFieldVal32(LLNI_unhand(this), maxHeight,  as + ds + ld);
	  writeNamedFieldVal32(LLNI_unhand(this), maxAdvance,
			       xfont->max_bounds.width);
       LLNI_endInconsistent;

       ok = LLNI_newArray(classArrayOfInt(), 256, (IArrayOfAny)widths);
       LLNI_setRefField(this, widths, widths);
       if (!ok) {
	  OutOfMemoryError();
	  AWT_UNLOCK();
	  rootReturnVoid;
       }
       LLNI_beginInconsistent(ee);
          memset((void*)LLNI_getValArrayBody(widths),0,256*sizeof(java_int));
	  off = xfont->min_char_or_byte2;
	  ccount = xfont->max_char_or_byte2 - xfont->min_char_or_byte2;
	  if (xfont->per_char) {
	     for (i = 0; i <= ccount; i++) {
		writeArrayVal32(LLNI_unhand(widths), off + i,
				(java_int) xfont->per_char[i].width);
	     }
	  } else {
	     for (i = 0; i <= ccount; i++) {
		writeArrayVal32(LLNI_unhand(widths), off + i,
				(java_int) xfont->max_bounds.width);
	     }
	  }
       LLNI_endInconsistent;

    endLocalRoots;
    AWT_UNLOCK();
}
