/*
 * @(#)tiny_Graphics.c	1.30 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "common_exceptions.h"
#include "llni.h"
#include "gclocalroots.h"
#include "java_awt_Rectangle.h"
#include "sun_awt_tiny_TinyGraphics.h"
#include "sun_awt_tiny_TinyWindow.h"
#include "sun_awt_image_ImageRepresentation.h"
#include "tiny.h"

extern XFontStruct *awt_getFont(Font *font);

/*
 * Cache the current Graphics Context in theGC and
 * save it's associated Java object in gc_owner.
 * A weakRoot callback routine takes care of clearing
 * these fields when the Java object is collected.
 */
DeclareGlobalRoot(Isun_awt_tiny_TinyGraphics, gc_owner);

static GC theGC = 0;

static bool_t
tiny_cg_callback (Ijava_lang_Object rootLocation) {
    sysAssert(LLNI_isSameObjectFct((Ijava_lang_Object)gc_owner, rootLocation));
    LLNI_setToNull(gc_owner);
    return FALSE;
}


static GC 
tiny_gc(Isun_awt_tiny_TinyGraphics this)
{
    Window win;
    XGCValues atts;
    int mask = 0;
    bool_t isSame;
    java_int pDrawable, touched;

    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Window) pDrawable;

    if (win == 0) {
	return 0;
    }
    LLNI_isSameObject(this, gc_owner, isSame);
    LLNI_getValField(this, touched, touched);

    if ((theGC == 0) || !isSame || touched) {
	if (theGC == 0) {
	    theGC = XCreateGC(awt_display, win, 0, 0);
	    if (theGC == 0) {
		return 0;
	    }
	}
	if (!LLNI_isNull(gc_owner)) {
	   unregisterWeakGlobalRoot((Ijava_lang_Object) gc_owner);
	   LLNI_setToNull(gc_owner);
	}
	registerWeakGlobalRoot((Ijava_lang_Object) gc_owner, tiny_cg_callback);
	LLNI_copy(gc_owner, this);

	beginLocalRoots4(Ijava_awt_Font,      font,
			 Ijava_awt_Color,     color,
			 Ijava_awt_Color,     xorcolor,
			 Ijava_awt_Rectangle, clip);
	   LLNI_getRefField(this, font, font);
	   if (!LLNI_isNull(font)) {
	      XFontStruct *f = awt_getFont(font);
	      if (f) {
		 atts.font = f->fid;
		 mask |= GCFont;
	      }
	   }
	   LLNI_getRefField(this, color, color);
	   if (!LLNI_isNull(color)) {
	      atts.foreground = awt_getColor(color);
	      mask |= GCForeground;
	   } else {
	      atts.foreground = 0;
	   }
	   LLNI_getRefField(this, xorColor, xorcolor);
	   if (!LLNI_isNull(xorcolor)) {
	      atts.foreground ^= awt_getColor(xorcolor);
	      atts.function = GXxor;
	      mask |= GCFunction | GCForeground;
	   } else {
	      atts.function = GXcopy;
	      mask |= GCFunction;
	   }
	   XChangeGC(awt_display, theGC, mask, &atts);

	   LLNI_getRefField(this, clip, clip);
	   if (!LLNI_isNull(clip)) {
	      XRectangle rect;
	      java_int w, h;
	      LLNI_safeExec(
		 w      = readNamedFieldVal32(LLNI_unhand(clip), width);
		 h      = readNamedFieldVal32(LLNI_unhand(clip), height);
		 rect.x = readNamedFieldVal32(LLNI_unhand(clip), x);
		 rect.y = readNamedFieldVal32(LLNI_unhand(clip), y);
	      );
	      rect.width = (w < 0) ? 0 : w;
	      rect.height = (h < 0) ? 0 : h;
	      XSetClipRectangles(awt_display, theGC, 0, 0, &rect, 1,YXBanded);
	   } else {
	      XSetClipMask(awt_display, theGC, None);
	   }
	endLocalRoots;
	LLNI_setValField(this, touched, 0);
    }

    tiny_flush();
    return theGC;
}

void 
sun_awt_tiny_TinyGraphics_createFromGraphics(Isun_awt_tiny_TinyGraphics this,
					     Isun_awt_tiny_TinyGraphics g)
{
    Window win;
    java_int draw;

    AWT_LOCK();
    LLNI_getValField(g, pDrawable, draw);
    LLNI_setValField(this, pDrawable, draw);
    LLNI_setValField(this, touched, 1);
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_createFromWindow(Isun_awt_tiny_TinyGraphics this,
					   Isun_awt_tiny_TinyWindow window)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    if (LLNI_isNull(window)) {
	SignalError(0, JAVAPKG "NullPointerException", "window");
	AWT_UNLOCK();
	return;
    }
    LLNI_getValField(window, xid, xid);
    win = (Window) xid;
    if (win == 0) {
	AWT_UNLOCK();
	return;
    }

    LLNI_setValField(this, pDrawable, (java_int)win);
    LLNI_setValField(this, touched, 1);
    AWT_UNLOCK();
}

void
sun_awt_tiny_TinyGraphics_imageCreate(Isun_awt_tiny_TinyGraphics this,
				      Isun_awt_image_ImageRepresentation ir)
{
    extern Drawable image_getIRDrawable(Isun_awt_image_ImageRepresentation ir);
    Drawable win;

    AWT_LOCK();
    if (LLNI_isNull(ir)) {
	SignalError(0, JAVAPKG "NullPointerException", "ir");
	AWT_UNLOCK();
	return;
    }

    win = image_getIRDrawable(ir);
    if (win == 0) {
	AWT_UNLOCK();
	return;
    }

    LLNI_setValField(this, pDrawable, (java_int)win);
    LLNI_setValField(this, touched, 1);
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_dispose(Isun_awt_tiny_TinyGraphics this)
{
    bool_t isSame;
    AWT_LOCK();

    LLNI_isSameObject(this, gc_owner, isSame);
    if (isSame) {
	LLNI_setToNull(gc_owner);
	unregisterWeakGlobalRoot((Ijava_lang_Object)gc_owner);
    }
    LLNI_setValField(this, pDrawable, 0);
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_drawLine(Isun_awt_tiny_TinyGraphics this,
				   java_int x1, java_int y1,
				   java_int x2, java_int y2)
{
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }
    
    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XDrawLine(awt_display, win, gc, oX + x1, oY + y1, oX + x2, oY + y2);
    AWT_UNLOCK();
}

void
sun_awt_tiny_TinyGraphics_clearArea(Isun_awt_tiny_TinyGraphics this,
				    java_int x, java_int y,
				    java_int w, java_int h)
{
    Drawable win;
    java_int pDrawable;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    if (win == 0) {
	AWT_UNLOCK();
	return;
    }
    
    XClearArea(awt_display, win, x, y, w, h, 0);
    AWT_UNLOCK();
}
void 
sun_awt_tiny_TinyGraphics_fillRect(Isun_awt_tiny_TinyGraphics this,
				   java_int x, java_int y,
				   java_int w, java_int h)
{
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }
    if (w < 0) {
	w = 0;
    }
    if (h < 0) {
	h = 0;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XFillRectangle(awt_display, win, gc, oX + x, oY + y, w, h);
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_drawString(Isun_awt_tiny_TinyGraphics this,
				     Ijava_lang_String str,
				     java_int x, java_int y)
{
    unicode *dataptr;
    int	length;
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC	gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if (LLNI_isNull(str)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	AWT_UNLOCK();
	return;
    }
    if ((win == 0) || (gc == 0)) {
	AWT_UNLOCK();
	return;
    }

    dataptr = javaString2CUniStringAlloc(str);
    length = javaStringLength(str);

    if (length > 1024) {
	length = 1024;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XDrawString16(awt_display, win, gc, oX + x, oY + y,
		  (XChar2b *)dataptr, length);
    sysFree(dataptr);
    AWT_UNLOCK();
}

void
sun_awt_tiny_TinyGraphics_drawChars(Isun_awt_tiny_TinyGraphics this,
				    IArrayOfChar data,
				    java_int off, java_int len,
				    java_int x, java_int y)
{
    unicode *dataptr;
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC	gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if (LLNI_isNull(data)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	AWT_UNLOCK();
	return;
    }
    if ((win == 0) || (gc == 0)) {
	AWT_UNLOCK();
	return;
    }

    if (off < 0 || len < 0 ||
	off + len > LLNI_getArrayLengthFct((IArrayOfAny)data)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	AWT_UNLOCK();
	return;
    }
    dataptr = javaArrayOfChar2CUniStringAlloc(data);
    if (len > 1024) {
	len = 1024;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XDrawString16(awt_display, win, gc, oX + x, oY + y, 
		  (XChar2b*)dataptr + off, len);
    sysFree(dataptr);
    AWT_UNLOCK();
}

void
sun_awt_tiny_TinyGraphics_drawBytes(Isun_awt_tiny_TinyGraphics this,
				    IArrayOfByte data,
				    java_int off, java_int len,
				    java_int x, java_int y)
{
    char *dataptr;
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    int data_len;
    GC	gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if (LLNI_isNull(data)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	AWT_UNLOCK();
	return;
    }
    if ((win == 0) || (gc == 0)) {
	AWT_UNLOCK();
	return;
    }

    LLNI_getArrayLength((IArrayOfAny)data, data_len);
    if (off < 0 || len < 0 || off + len > data_len) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	AWT_UNLOCK();
	return;
    }
    dataptr = javaArrayOfByte2CStringAlloc(data);
    if (len > 1024) {
	len = 1024;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XDrawString(awt_display, win, gc, oX + x, oY + y, dataptr + off, len);
    sysFree(dataptr);
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_copyArea(Isun_awt_tiny_TinyGraphics this,
				   java_int x, java_int y,
				   java_int w, java_int h,
				   java_int dx, java_int dy)
{
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }
    dx += x;
    dy += y;

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XCopyArea(awt_display, win, win, gc,
	      oX + x, oY + y, w, h,
	      oX + dx, oY + dy);
    AWT_UNLOCK();
}


static XPoint *
transformPoints(Isun_awt_tiny_TinyGraphics this,
		IArrayOfInt xpoints, IArrayOfInt ypoints, long npoints)
{
    static XPoint	*points;
    static int		points_length = 0;
    XPoint		*xp;
    int			i;
    java_int 		oX, oY;
    int 		xpoints_len, ypoints_len;

    LLNI_getArrayLength((IArrayOfAny)xpoints, xpoints_len);
    LLNI_getArrayLength((IArrayOfAny)ypoints, ypoints_len);
    if (ypoints_len < npoints || xpoints_len < npoints) {
	SignalError(0, JAVAPKG "IllegalArgumentException",0);
	return 0;
    }
    if (points_length < npoints) {
	if (points_length != 0) {
	    sysFree((void *)points);
	}
	points = (XPoint *)sysMalloc(sizeof(XPoint) * npoints);
	if (points == 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryError",0);
	    return 0;
	}
	points_length = npoints;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    for (i=0; i < points_length; i++) {
       java_int x, y;
       xp = (points + i);
       LLNI_getValArrayElement(xpoints, i, x);
       LLNI_getValArrayElement(ypoints, i, y);

       xp->x = oX + x;
       xp->y = oY + y;
    }

    return points;
}

void 
sun_awt_tiny_TinyGraphics_fillPolygon(Isun_awt_tiny_TinyGraphics  this,
				      IArrayOfInt xp, IArrayOfInt yp,
				      java_int np)
{
    Drawable win;
    java_int pDrawable;
    GC	gc;
    XPoint *points;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }

    if (points = transformPoints(this, xp, yp, np)) {
	XFillPolygon(awt_display, win, gc, points, np, Complex, CoordModeOrigin);
    }
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_drawArc(Isun_awt_tiny_TinyGraphics this,
				  java_int x, java_int y,
				  java_int w, java_int h,
				  java_int startAngle, java_int endAngle)
{
    long	s, e;
    Drawable win;
    java_int oX, oY;
    java_int pDrawable;
    GC	gc;

    if (w < 0 || h < 0) {
	return;
    }
    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }

    if (endAngle >= 360 || endAngle <= -360) {
	s = 0;
	e = 360 * 64;
    } else {
	s = (startAngle % 360) * 64;
	e = endAngle * 64;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XDrawArc(awt_display, win, gc, 
	     oX + x,
	     oY + y,
	     w,
	     h, 
	     s, e);
    AWT_UNLOCK();
}

void 
sun_awt_tiny_TinyGraphics_fillArc(Isun_awt_tiny_TinyGraphics this,
				  java_int x, java_int y,
				  java_int w, java_int h,
				  java_int startAngle, java_int endAngle)
{
    long	s, e;
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC	gc;

    AWT_LOCK();
    LLNI_getValField(this, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc(this);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }

    if (endAngle >= 360 || endAngle <= -360) {
	s = 0;
	e = 360 * 64;
    } else {
	s = (startAngle % 360) * 64;
	e = endAngle * 64;
    }

    LLNI_getValField(this, originX, oX);
    LLNI_getValField(this, originY, oY);

    XFillArc(awt_display, win, gc, 
	     oX + x,
	     oY + y,
	     w,
	     h, 
	     s, e);
    AWT_UNLOCK();
}

void 
sun_awt_image_ImageRepresentation_imageDraw(
			       Isun_awt_image_ImageRepresentation this,
			       Ijava_awt_Graphics g,
			       java_int x, java_int y, Ijava_awt_Color c)
{
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC	gc;
    int xormode;
    Ijava_awt_Color hxormode;
    unsigned long fgpixel, xorpixel;
    XRectangle clip, *pClip;
    Class *tgclass;

    if (LLNI_isNull((Isun_awt_tiny_TinyGraphics) g)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    LLNI_getClass((Isun_awt_tiny_TinyGraphics) g, tgclass);
    beginLocalRoots1 (Ijava_lang_Class, tgCls);
       if (tgclass != FindClass(0, "sun/awt/tiny/TinyGraphics", TRUE, tgCls)) {
	  SignalError(0, JAVAPKG "IllegalArgumentException",0);
 	  rootReturnVoid;
       }
    endLocalRoots;

    AWT_LOCK();
    LLNI_getValField((Isun_awt_tiny_TinyGraphics) g, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc((Isun_awt_tiny_TinyGraphics) g);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }

    beginLocalRoots3(Isun_awt_tiny_TinyGraphics, tg,
		     Ijava_awt_Color,            ctmp,
		     Ijava_awt_Rectangle,        tgclip);
       LLNI_copy(tg, (Isun_awt_tiny_TinyGraphics) g);
       LLNI_getRefField(tg, xorColor, hxormode);
       xormode = (int)hxormode;
       if (xormode) {
          LLNI_getRefField(tg, color, ctmp);
	  fgpixel = awt_getColor(ctmp);
	  LLNI_getRefField(tg, xorColor, ctmp);
	  xorpixel = awt_getColor(ctmp);
       }
       LLNI_getRefField(tg, clip, tgclip);
       if (!LLNI_isNull(tgclip)) {
          LLNI_safeExec(
	     clip.x      = readNamedFieldVal32(LLNI_unhand(tgclip), x);
	     clip.y      = readNamedFieldVal32(LLNI_unhand(tgclip), y);
	     clip.width  = readNamedFieldVal32(LLNI_unhand(tgclip), width);
	     clip.height = readNamedFieldVal32(LLNI_unhand(tgclip), height);
          );
	  pClip = &clip;
       } else {
	  pClip = 0;
       }

       LLNI_getValField(tg, originX, oX);
       LLNI_getValField(tg, originY, oY);

    endLocalRoots;

    awt_imageDraw(win, gc, this,
		  xormode, xorpixel, fgpixel,
		  oX + x, oY + y, 0, 0, -1, -1, c, pClip);
    AWT_UNLOCK();
}

void 
sun_awt_image_ImageRepresentation_imageStretch
    (Isun_awt_image_ImageRepresentation this,
     Ijava_awt_Graphics g,
     java_int dx1, java_int dy1, java_int dx2, java_int dy2,
     java_int sx1, java_int sy1, java_int sx2, java_int sy2,
     Ijava_awt_Color c)
{
    Drawable win;
    java_int pDrawable;
    java_int oX, oY;
    GC	gc;
    int xormode;
    Ijava_awt_Color hxormode;
    Class *tgclass;
    
    unsigned long fgpixel, xorpixel;
    XRectangle clip, *pClip;
    long w, h;

    if (LLNI_isNull((Isun_awt_tiny_TinyGraphics) g)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    LLNI_getClass((Isun_awt_tiny_TinyGraphics) g, tgclass);
    beginLocalRoots1 (Ijava_lang_Class, tgCls);
       if (tgclass != FindClass(0, "sun/awt/tiny/TinyGraphics", TRUE, tgCls)) {
	  SignalError(0, JAVAPKG "IllegalArgumentException",0);
 	  rootReturnVoid;
       }
    endLocalRoots;

    AWT_LOCK();
    LLNI_getValField((Isun_awt_tiny_TinyGraphics) g, pDrawable, pDrawable);
    win = (Drawable)pDrawable;
    gc = tiny_gc((Isun_awt_tiny_TinyGraphics) g);
    if ((gc == 0) || (win == 0)) {
	AWT_UNLOCK();
	return;
    }
    beginLocalRoots3(Isun_awt_tiny_TinyGraphics, tg,
		     Ijava_awt_Color,            ctmp,
		     Ijava_awt_Rectangle,        tgclip);
       LLNI_copy(tg, (Isun_awt_tiny_TinyGraphics) g);
       LLNI_getRefField(tg, xorColor, hxormode);
       xormode = (int)hxormode;
       if (xormode) {
          LLNI_getRefField(tg, color, ctmp);
	  fgpixel = awt_getColor(ctmp);
	  LLNI_getRefField(tg, xorColor, ctmp);
	  xorpixel = awt_getColor(ctmp);
       }
       LLNI_getRefField(tg, clip, tgclip);
       if (!LLNI_isNull(tgclip)) {
          LLNI_safeExec(
	     clip.x      = readNamedFieldVal32(LLNI_unhand(tgclip), x);
	     clip.y      = readNamedFieldVal32(LLNI_unhand(tgclip), y);
	     clip.width  = readNamedFieldVal32(LLNI_unhand(tgclip), width);
	     clip.height = readNamedFieldVal32(LLNI_unhand(tgclip), height);
          );
	  pClip = &clip;
       } else {
	  pClip = 0;
       }

       w = dx2 - dx1;
       h = dy2 - dy1;
       LLNI_getValField(tg, originX, oX);
       LLNI_getValField(tg, originY, oY);

    endLocalRoots;

    if (w == sx2 - sx1 && h == sy2 - sy1) {
	/* Make sure rectangles aren't upside down. */
	if (w < 0) {
	    dx1 = dx2;
	    sx1 = sx2;
	    w = -w;
	}
	if (h < 0) {
	    dy1 = dy2;
	    sy1 = sy2;
	    h = -h;
	}
	awt_imageDraw(win, gc, this,
		      xormode, xorpixel, fgpixel,
		      oX + dx1, oY + dy1, sx1, sy1, w, h,
		      c, pClip);
    } else {
	awt_imageStretch(win, gc, this,
			 xormode, xorpixel, fgpixel,
			 oX + dx1, oY + dy1, oX + dx2, oY + dy2,
			 sx1, sy1, sx2, sy2,
			 c, pClip); /* was &clip - ross 2/97 */
    }
    AWT_UNLOCK();
}
