/*
 * @(#)tiny_Window.c	1.22 00/05/04
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "llni.h"
#include "java_awt_Frame.h"
#include "sun_awt_image_ImageRepresentation.h"
#include <X11/cursorfont.h>
#include "tiny.h"

/*
 * Create a window. The window is initially unmapped.
 * The window is a toplevel window when the parent is null.
 */

void 
sun_awt_tiny_TinyWindow_winCreate(Isun_awt_tiny_TinyWindow this,
				  Isun_awt_tiny_TinyWindow parent,
				  java_int or)
{
    XSetWindowAttributes atts;
    Window win, pwin;
    Atom WM_DELETE_WINDOW;
    java_int xid;

    AWT_LOCK();
    /* make sure it wasn't already created */
    LLNI_getValField(this, xid, xid);
    if (xid != 0) {
	SignalError(0, JAVAPKG "IllegalArgumentException", "win");
	AWT_UNLOCK();
	return;
    }
    tiny_flush();

    /* get the parent X window id */
    if (!LLNI_isNull(parent)) {
        LLNI_getValField(parent, xid, xid);
	pwin = (Window)xid;
    } else {
	pwin = awt_root; 
    }

    memset(&atts, 0, sizeof(XSetWindowAttributes));
    
    /* initialize window attributes */
    atts.event_mask = KeyPressMask | KeyReleaseMask | FocusChangeMask |
		      ButtonPressMask | ButtonReleaseMask |
             	      EnterWindowMask | LeaveWindowMask |
		      PointerMotionMask | ButtonMotionMask |
		      ExposureMask | StructureNotifyMask;
    atts.override_redirect = (0 != or) ? True : False;
    atts.border_pixel = 0;
    atts.colormap = awt_cmap;
    
    /* create the window */
    win = XCreateWindow(awt_display, pwin, 0, 0, 1, 1, 0,
			awt_depth, InputOutput, awt_visual,
			CWOverrideRedirect | CWEventMask | CWBorderPixel | CWColormap, 
			&atts);

    /* make sure it worked */
    if (win == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", "win");
	AWT_UNLOCK();
	return;
    }

    /* set the window variables */
    LLNI_setValField(this, xid, (int)win);
    LLNI_setRefField(this, parent, parent);


    /* register the window for events */
    tiny_register(this);
    
    WM_DELETE_WINDOW = XInternAtom(awt_display, "WM_DELETE_WINDOW", False);
    XSetWMProtocols(awt_display, win, &WM_DELETE_WINDOW, 1);
 
    AWT_UNLOCK();
}

/**
 * set IconImage when using the windowManager
 */
void 
sun_awt_tiny_TinyWindow_winSetIconImage(Isun_awt_tiny_TinyWindow this,
					Isun_awt_image_ImageRepresentation ir)
{
    Window		win;
    XWMHints 		*hints;
    java_int		xid;

    if (LLNI_isNull(ir)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    AWT_LOCK();
    
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush;
	hints = XAllocWMHints();
	hints->flags = IconPixmapHint; 
	hints->icon_pixmap = image_getIRDrawable(ir);
	XSetWMHints(awt_display, win, hints);
	XFree(hints);
    }
    AWT_UNLOCK();
}
				
/**
 * Set the title of the window.
 * Only to be used when override_redirect = false.
 */
void
sun_awt_tiny_TinyWindow_winSetTitle(Isun_awt_tiny_TinyWindow this,
				    Ijava_lang_String l)
{
    char *c[1];
    Window win;
    XTextProperty text_prop;
    XICCEncodingStyle style;
    java_int xid;

    AWT_LOCK();

    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush;
	
	if (l==0) {
	    c[0] = "";
    	} else {
	    c[0] = javaString2CStringAlloc(l);
	}
    	XmbTextListToTextProperty(awt_display, c, 1, style, &text_prop); 
	XSetWMName(awt_display, win, &text_prop);
	sysFree(c[0]);
        XFree(text_prop.value);
    }
    AWT_UNLOCK();
}

/**
 * Reshape a window. Note: X windows can't be smaller than 1x1 pixel.
 */
void 
sun_awt_tiny_TinyWindow_winReshape(Isun_awt_tiny_TinyWindow this, java_int x, java_int y, java_int w, java_int h)
{
    Window win;
    XSizeHints *sizehints;
    java_int xid;
    AWT_LOCK();
    
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
    
	if (w <= 0) {
	    w = 1;
	}
	if (h <= 0) {
	    h = 1;
	}
	LLNI_beginInconsistent(EE());
	   writeNamedFieldVal32(LLNI_unhand(this), x,      x);
	   writeNamedFieldVal32(LLNI_unhand(this), y,      y);
	   writeNamedFieldVal32(LLNI_unhand(this), width,  w);
	   writeNamedFieldVal32(LLNI_unhand(this), height, h);
	LLNI_endInconsistent;
	
	sizehints = XAllocSizeHints();
	sizehints->x = x;
	sizehints->y = y;
	sizehints->flags = USPosition | PPosition;

	XSetWMNormalHints(awt_display, win, sizehints);
	XFree((char*)sizehints);
	XMoveResizeWindow(awt_display, win, x, y, w, h);
    }
    AWT_UNLOCK();
}

/**
 * Dispose of the window. Further use of it will result in
 * an exception.
 */
void 
sun_awt_tiny_TinyWindow_winDispose(Isun_awt_tiny_TinyWindow this)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
	tiny_unregister(this);
	XDestroyWindow(awt_display, win);
	LLNI_setValField(this, xid, 0);
    }
    AWT_UNLOCK();
}

/**
 * Show a window.
 */
void 
sun_awt_tiny_TinyWindow_winShow(Isun_awt_tiny_TinyWindow this)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
	XMapWindow(awt_display, win);
    }
    AWT_UNLOCK();
}

/**
 * Hide a window.
 */
void 
sun_awt_tiny_TinyWindow_winHide(Isun_awt_tiny_TinyWindow this)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
	XUnmapWindow(awt_display, win);
    }
    AWT_UNLOCK();
}

/**
 * Bring a window to the front.
 */
void 
sun_awt_tiny_TinyWindow_winFront(Isun_awt_tiny_TinyWindow this)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
	XRaiseWindow(awt_display, win);
    }
    AWT_UNLOCK();
}

/**
 * Send a window to the back.
 */
void 
sun_awt_tiny_TinyWindow_winBack(Isun_awt_tiny_TinyWindow this)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
	XLowerWindow(awt_display, win);
    }
    AWT_UNLOCK();
}

/**
 * Set the background color of the window.
 */
void 
sun_awt_tiny_TinyWindow_winBackground(Isun_awt_tiny_TinyWindow this,
				      Ijava_awt_Color col)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	XSetWindowAttributes atts;

	tiny_flush();
	atts.background_pixel = awt_getColor(col);
	XChangeWindowAttributes(awt_display, win, CWBackPixel, &atts);
    }
    AWT_UNLOCK();
}

extern int tm;
/**
 * Request the input focus
 */
void 
sun_awt_tiny_TinyWindow_winFocus(Isun_awt_tiny_TinyWindow this)
{
    Window win;
    java_int xid;

    AWT_LOCK();
    LLNI_getValField(this, xid, xid);
    if ((win = (Window)xid) != 0) {
	tiny_flush();
	XSetInputFocus(awt_display, win, RevertToPointerRoot, CurrentTime);
    }
    AWT_UNLOCK();
}

void
sun_awt_tiny_TinyWindow_winSetCursor(Isun_awt_tiny_TinyWindow this, java_int cursorType)
{
    unsigned long 		valuemask = 0;
    XSetWindowAttributes	attributes;
    Cursor			cursor;
    java_int xid;
    
    switch (cursorType) {
      case java_awt_Frame_DEFAULT_CURSOR:
	cursorType = -1;
	break;
      case java_awt_Frame_CROSSHAIR_CURSOR:
	cursorType = XC_crosshair;
	break;
      case java_awt_Frame_TEXT_CURSOR:
	cursorType = XC_xterm;
	break;
      case java_awt_Frame_WAIT_CURSOR:
	cursorType = XC_watch;
	break;
      case java_awt_Frame_SW_RESIZE_CURSOR:
	cursorType = XC_bottom_left_corner;
	break;
      case java_awt_Frame_NW_RESIZE_CURSOR:
	cursorType = XC_top_left_corner;
	break;
      case java_awt_Frame_SE_RESIZE_CURSOR:
	cursorType = XC_bottom_right_corner;
	break;
      case java_awt_Frame_NE_RESIZE_CURSOR:
	cursorType = XC_top_right_corner;
	break;
      case java_awt_Frame_S_RESIZE_CURSOR:
	cursorType = XC_bottom_side;
	break;
      case java_awt_Frame_N_RESIZE_CURSOR:
	cursorType = XC_top_side;
	break;
      case java_awt_Frame_W_RESIZE_CURSOR:
	cursorType = XC_left_side;
	break;
      case java_awt_Frame_E_RESIZE_CURSOR:
	cursorType = XC_right_side;
	break;
    case java_awt_Frame_HAND_CURSOR:
	cursorType = XC_hand2;
	break;
      case java_awt_Frame_MOVE_CURSOR:
	cursorType = XC_fleur;
	break;
      default:
	AWT_UNLOCK();
	return;
    }
    if (cursorType != -1) {
	cursor = XCreateFontCursor(awt_display, cursorType);
    } else {
	cursor = None;
    }
    valuemask = CWCursor;
    attributes.cursor = cursor;
    LLNI_getValField(this, xid, xid);
    XChangeWindowAttributes(awt_display,
			    (Window)xid,
			    valuemask,
			    &attributes);
}
