/*
 * @(#)Win32GraphicsEnvironment.java	1.10 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt;

import sun.java2d.SunGraphicsEnvironment;
import java.awt.GraphicsDevice;
import java.awt.Toolkit;
import java.awt.print.PrinterJob;
import sun.awt.windows.WPrinterJob;

/**
 * This is an implementation of a GraphicsEnvironment object for the
 * default local GraphicsEnvironment used by the JavaSoft JDK in Win32
 * environments.
 *
 * @see GraphicsDevice
 * @see GraphicsConfiguration
 * @version 10 Feb 1997
 */

public class Win32GraphicsEnvironment extends SunGraphicsEnvironment {
    static {
	java.security.AccessController.doPrivileged(
		    new sun.security.action.LoadLibraryAction("awt"));
	initDisplay();
    }

    private static native void initDisplay();

    public Win32GraphicsEnvironment() {
    }

    protected native int getNumScreens();
//     protected int getNumScreens() {
//         return 1;
//     }

    protected GraphicsDevice makeScreenDevice(int screennum) {
        return new Win32GraphicsDevice(screennum);
    }

    /* Fix: 4175044
    *  allow platform to override version extension
    *  for font.properties files
    */
    protected String getOsVersion() {
        String osname = System.getProperty("os.name");
        String osversion = System.getProperty("os.version");
        if (osname.equals("Windows NT")) {
            return "NT" + osversion;
        }
        return osversion;
    }

    /**
     * Gets a <code>PrinterJob</code> object suitable for the
     * the current platform.
     * @return    a <code>PrinterJob</code> object.
     * @see       java.awt.PrinterJob
     * @since     JDK1.2
     */
    public PrinterJob getPrinterJob() {
	SecurityManager security = System.getSecurityManager();
	if (security != null) {
	    security.checkPrintJobAccess();
	}

	return new WPrinterJob();
    }
}
