/*
 * @(#)TextRenderingGDI.java	1.6 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.windows;

import java.awt.Color;
import java.awt.Font;
import java.awt.font.GlyphVector;

import sun.java2d.loops.GraphicsPrimitive;
import sun.java2d.loops.GraphicsPrimitives;
import sun.java2d.loops.ImageData;
import sun.java2d.loops.DrawString;
import sun.java2d.loops.DrawGlyphVector;
import sun.java2d.loops.DrawChars;
import sun.java2d.loops.DrawBytes;

/**
 * TextRenderingGDI collection
 * Basically, a collection of components which permit painting from
 * font renderer onto printers.
 */

public class TextRenderingGDI implements GraphicsPrimitives {
    final static int[] ST_GDI = { GraphicsPrimitive.TYPE_CUSTOM };
    final static int[] lockFlags = { GraphicsPrimitive.LOCK_FOR_READ |
				     GraphicsPrimitive.LOCK_FOR_NOOP };

    public final GraphicsPrimitive[] getPrimitives() {
	GraphicsPrimitive[] primitives = {
	    new DrawCharsGDI(),
	    new DrawStringGDI(),
	    new DrawBytesGDI(),
	    new DrawGlyphVectorGDI()
	};
	return primitives;
    }
}

class DrawCharsGDI extends DrawChars {

    private synchronized native void drawCharsGDI(
	char [] srcData, int srcOffset, int length,
	float fX,
	float fY,
	Font fontObject,
	double[] matrix,
	int fgColor,
	int cx, int cy, int cw, int ch,
	ImageData dest
    );
    
    DrawCharsGDI() {
	super(TextRenderingGDI.ST_GDI, TextRenderingGDI.lockFlags);
    }

    public void DrawChars(
	ImageData dest,
	char [] srcData, int offset, int length,
	float fX,
	float fY,
	Color fgColor,
	Font fontObject,
	double[] matrix,
	boolean doAntiAlias,
	boolean doFractEnable
    ) {
	drawCharsGDI(
 	    srcData, offset, length, fX, fY,
	    fontObject,
	    matrix, fgColor.getRGB(), 
	    dest.getXOutputArea(),
	    dest.getYOutputArea(),
	    dest.getDXOutputArea(),
	    dest.getDYOutputArea(),
	    dest
	);
    }
}

class DrawStringGDI extends DrawString {

    private synchronized native void drawStringGDI(
	String srcData,
	float fX,
	float fY,
	Font fontObject,
	double[] matrix,
	int fgColor,
	int cx, int cy, int cw, int ch,
	ImageData dest
    );

    DrawStringGDI() {
	super(TextRenderingGDI.ST_GDI, TextRenderingGDI.lockFlags);
    }

    public void DrawString(
	ImageData dest,
	String srcData,
	float fX,
	float fY,
	Color fgColor,
	Font fontObject,
	double[] matrix,
	boolean doAntiAlias,
	boolean doFractEnable
    ) {
        drawStringGDI(
	    srcData, fX, fY,
	    fontObject,
	    matrix, fgColor.getRGB(),
	    dest.getXOutputArea(),
	    dest.getYOutputArea(),
	    dest.getDXOutputArea(),
	    dest.getDYOutputArea(),
	    dest
	);
    }
}

class DrawBytesGDI extends DrawBytes {

    private synchronized native void drawBytesGDI(
	byte [] srcData, int srcOffset, int length,
	float fX,
	float fY,
	Font fontObject,
	double[] matrix,
	int fgColor,
	int cx, int cy, int cw, int ch,
	ImageData dest
    );

    DrawBytesGDI() {
	super(TextRenderingGDI.ST_GDI, TextRenderingGDI.lockFlags);
    }

    public void DrawBytes(
	ImageData dest,
	byte [] srcData, int offset, int length,
	float fX,
	float fY,
	Color fgColor,
	Font fontObject,
	double[] matrix,
	boolean doAntiAlias,
	boolean doFractEnable
    ) {
	drawBytesGDI(
	    srcData, offset, length, fX, fY,
	    fontObject,
	    matrix, fgColor.getRGB(),
	    dest.getXOutputArea(),
	    dest.getYOutputArea(),
	    dest.getDXOutputArea(),
	    dest.getDYOutputArea(),
	    dest
	);
    }
}

class DrawGlyphVectorGDI extends DrawGlyphVector {

    private synchronized native void drawGlyphVectorGDI(
	int [] srcGlyphs,
	float [] positionsXY,
	float fX,
	float fY,
	Font fontObject,
	double[] glyphTX, double[] devTX,
	int fgColor,
	int cx, int cy, int cw, int ch,
	ImageData dest
    );

    DrawGlyphVectorGDI() {
	super(TextRenderingGDI.ST_GDI, TextRenderingGDI.lockFlags);
    }

    public void DrawGlyphVector(
	ImageData dest,
	GlyphVector gv,
	float fX,
	float fY,
	Color fgColor,
	Font fontObject,
	double[] glyphTX, double[] devTX,
	boolean doAntiAlias,
	boolean doFractEnable
    ) {
	int numGlyphs = gv.getNumGlyphs();
	drawGlyphVectorGDI(
	    gv.getGlyphCodes(0, numGlyphs, null),
	    gv.getGlyphPositions(0, numGlyphs, null),
	    fX, fY,
	    fontObject,
	    glyphTX, devTX, fgColor.getRGB(),
	    dest.getXOutputArea(),
	    dest.getYOutputArea(),
	    dest.getDXOutputArea(),
	    dest.getDYOutputArea(),
	    dest
	);
    }
}
