/*
 * @(#)WActiveEvent.java	1.8 01/02/09
 *
 * Copyright 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.windows;

import java.awt.*;

class WActiveEvent extends AWTEvent implements ActiveEvent {
    private WObjectPeer peer;
    private long pData;

    public WActiveEvent(Object target, WObjectPeer peer, long pData) {
        // The 'target' reference we pass here identifies the
        // event as associated with a particular instance. That
        // way the event can be removed by a call to removeSourceEvents.
        super(target, 0);
	this.peer = peer;
        this.pData = pData;
        if (peer instanceof WWindowPeer) {
            ((WWindowPeer)peer).pendingFocusEvents.addElement(this);
        }
    }

    private native void dispatchImpl(long pData);
    private native void finalizeImpl(long pData);

    public void dispatch() {
        /* Only dispatch if the native component still exists */

        if (!peer.isDisposed()) {
	    synchronized (peer) {
	        if (!peer.isDisposed()) {
		    if (peer instanceof WWindowPeer) {
			WWindowPeer wp = (WWindowPeer)peer;
			if (wp.pendingFocusEvents.contains(this)) {
			    wp.grantDefaultFocus = true;
			    wp.pendingFocusEvents.removeElement(this);
			} else {
			    wp.grantDefaultFocus = false;
			}
		    }
		    dispatchImpl(pData);
		}
	    }
	}
    }

    protected void finalize() throws Throwable {
	finalizeImpl(pData);
        super.finalize();
    }
}
