/*
 * @(#)WCheckboxPeer.java	1.17 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.windows;

import java.awt.*;
import java.awt.peer.*;
import java.awt.event.ItemEvent;

public class WCheckboxPeer extends WComponentPeer implements CheckboxPeer {

    // CheckboxPeer implementation

    public native void setState(boolean state);
    public native void setCheckboxGroup(CheckboxGroup g);
    public native void setLabel(String label);

    public Dimension getMinimumSize() {
	String lbl = ((Checkbox)target).getLabel();
	if (lbl != null) {
	    FontMetrics fm = getFontMetrics(((Checkbox)target).getFont());
	    return new Dimension(30 + fm.stringWidth(lbl),
				 Math.max(fm.getHeight() + 8, 15));
	}
	return new Dimension(20, 15);
    }
    public boolean isFocusTraversable() {
	return true;
    }

    // Toolkit & peer internals

    WCheckboxPeer(Checkbox target) {
	super(target);
    }

    native void create(WComponentPeer parent);

    void initialize() {
	Checkbox t = (Checkbox)target;
	setState(t.getState());
	setCheckboxGroup(t.getCheckboxGroup());

	Color bg = ((Component)target).getBackground();
	if (bg != null) {
	    setBackground(bg);
	}

	super.initialize();
    }

    public boolean shouldClearRectBeforePaint() {
        return false;
    }

    // native callbacks

    void handleAction(final boolean state) {
	final Checkbox cb = (Checkbox)this.target;
	WToolkit.executeOnEventHandlerThread(cb, new Runnable() {
	    public void run() {
		cb.setState(state);
		postEvent(new ItemEvent(cb, ItemEvent.ITEM_STATE_CHANGED,
                                cb.getLabel(),
                                state? ItemEvent.SELECTED : ItemEvent.DESELECTED));
	    }
	});
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	    return getMinimumSize();
    }

}
