/*
 * @(#)WDragSourceContextPeer.java	1.19 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.windows;

import java.awt.event.InputEvent;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;

import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragSourceContext;
import java.awt.dnd.InvalidDnDOperationException;

import java.io.IOException;

import java.util.Map;
import java.util.Stack;

import sun.awt.SunToolkit;

import sun.awt.dnd.SunDragSourceContextPeer;

/**
 * <p>
 * TBC
 * </p>
 *
 * @version 1.19, 02/09/01
 * @since 1.2
 *
 */

final class WDragSourceContextPeer extends SunDragSourceContextPeer {

    /**
     * construct a new WDragSourceContextPeer. package private
     */

    WDragSourceContextPeer(DragGestureEvent dge) {
    	super(dge);
    }


    /**
     * initiate a DnD operation ...
     */

    public void startDrag(DragSourceContext dsc, Cursor c, Image di,
			  Point ioff) throws InvalidDnDOperationException {
	dragSourceContext = dsc;
	cursor 		  = c;
	actions           = trigger.getSourceAsDragGestureRecognizer().getSourceActions();

	synchronized(this) {
	    Transferable trans = dragSourceContext.getTransferable();
	    Map formatMap = WDataTransferer.getInstance().
	        getFormatsForTransferable(trans, dragSource.getFlavorMap());
	    int[] formats = WDataTransferer.getInstance().keysToIntArray(formatMap);

	    if (nativeCtxt != 0) {
		throw new InvalidDnDOperationException("drag in progress");
	    }

	    try {
	        nativeCtxt = (long)createDragSource(trigger.getTriggerEvent(),
						    actions,
						    formats,
						    formatMap);
	    } catch (Exception e) {
		e.printStackTrace();
		throw new InvalidDnDOperationException("failed to create native peer");
	    }

	    if (nativeCtxt == 0) 
		throw new InvalidDnDOperationException("failed to create native peer");
	    WDropTargetContextPeer.setCurrentJVMLocalSourceTransferable(trans);

	    try {

		/*
		 * create a new thread (later) to call into the modal windows
		 * code ... 
		 */

	    	((Thread)(new Thread() {

		    /**
  		     * run
		     */

	            public void run() {
		    	try {
			    doDragDrop(WDragSourceContextPeer.this.nativeCtxt,
				       WDragSourceContextPeer.this.cursor
			    );
		        } catch (Exception e) {
		    	    e.printStackTrace();
			} finally {
			    release(WDragSourceContextPeer.this.nativeCtxt);

			    WDragSourceContextPeer.this.nativeCtxt = 0;

			    WDropTargetContextPeer.setCurrentJVMLocalSourceTransferable(null);
		        }
		    }
	        })).start();
	    } catch (Exception e) {
	        e.printStackTrace();

	        release(nativeCtxt);
	        nativeCtxt = 0;

	        throw new InvalidDnDOperationException("failed to start Thread");
	    }
	}
    }

    /**
     *
     */

    public void setCursor(Cursor c) throws InvalidDnDOperationException {
	if (cursor == null || !cursor.equals(c)) {
	    cursor = c;
	    if (nativeCtxt != 0) setNativeCursor(nativeCtxt, cursor);
	}
    }


    /**
     *
     */

    private byte[] convertData(final int format, final Map formatMap) {
        final Stack stack = new Stack();

        Runnable getData = new Runnable () {
		public void run() {
                    byte[] data = null;
                    try {
                        DataFlavor flavor = (DataFlavor)
                            formatMap.get(new Integer(format));
                        
                        // Flavor can be null if some application   
                        // requested data in unsupported format.
                        if (flavor != null) {
                            data = WDataTransferer.getInstance().
                                translateTransferable(
                                    dragSourceContext.getTransferable(), 
                                    flavor, format);
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        try {
                            syncLock.lock();
                            stack.push(data);
                        } finally {
                            quitSecondaryDispatch();
                            syncLock.unlock();
                        }
                    }
		}
	    };
        
        syncLock.lock();
        
        SunToolkit.executeOnEventHandlerThread(component, getData);
            
        while (stack.empty()) {
            syncLock.unlock();
            startSecondaryDispatch();
            syncLock.lock();
        }
        
        syncLock.unlock();
        
        return (byte[])stack.pop();
   }

    /**
     * upcall from native code
     */

    void dragDropFinished(boolean success, int action, int xhint, int yhint, boolean diwcr) {
	super.dodragDropFinished(success, action);
	// this is used by the native code to synthesize a DROPFILES structure
	// when DnD'ing files.

	dropScreenXHint   = xhint;
	dropScreenYHint   = yhint;
	droppedInWindowCR = diwcr;
    }

    /**
     * called by dispatch methods (drag*). Allows extensions of this class to 
     * perform additional operations in beginning a dispatch by overriding 
     * this method.
     *
     * For instance WDragSourceContextPeer will override
     * this method by adding a call to WToolkit.startSecondaryEventLoop();
     */
    protected void startSecondaryDispatch() {
	WToolkit.startSecondaryEventLoop();
    }
    
    /**
     * called by run. Allows extensions of this class to 
     * perform additional operations in ending a dispatch by overriding 
     * this method.
     *
     * For instance WDragSourceContextPeer will override
     * this method by adding a call to WToolkit.quitSecondaryEventLoop();
     */
    protected void quitSecondaryDispatch() {
	WToolkit.quitSecondaryEventLoop();
    }

    /**
     * upcall from native code
     */

    private int getDropXHint() { return dropScreenXHint; }

    /**
     * upcall from native code
     */

    private int getDropYHint() { return dropScreenYHint; }

    /**
     * upcall from native code
     */

    private boolean getDroppedInWindowCR() { return droppedInWindowCR; }

    /**
     * downcall into native code
     */

    native long createDragSource(InputEvent nativeTrigger, int actions,
				int[] formats, Map formatMap);

    /**
     * downcall into native code
     */

    native void doDragDrop(long nativeCtxt, Cursor cursor);

    /**
     * downcall into native code
     */

    private static native void setNativeCursor(long nativeCtxt, Cursor c);

    /**
     * downcall into native code
     */

    private native void release(long nativeCtxt);

    /**
     * initialize native code
     */

    static native void Initialize();

    /*
     * fields
     */

    private int			     	     dropScreenXHint;
    private int			     	     dropScreenYHint;
    private boolean			     droppedInWindowCR;
}


