/*
 * @(#)WPrintGraphics.java	1.25 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.windows;

import java.awt.*;
import java.awt.geom.AffineTransform;
import sun.java2d.OutputManager;
import sun.java2d.loops.ImageData;
import sun.java2d.loops.RasterOutputManager;
import sun.java2d.loops.GraphicsPrimitiveMgr;
import sun.java2d.loops.GraphicsPrimitive;
import java.awt.font.GlyphVector;
import java.text.AttributedCharacterIterator;
import sun.java2d.SunGraphics2D;

/**
 * WPrintGraphics is an object that encapsulates a print graphics context
 * for a page.
 *
 * @version 1.25, 02/02/00
 */

class WPrintGraphics extends WGraphics implements PrintGraphics {

    final static String newDevID = "WPrintGraphics";

    static {
	OutputManager manager = RasterOutputManager.getManager();
	GraphicsPrimitiveMgr.register(new TextRenderingGDI(), newDevID);
    }

    private WPrintJob job;
    private int pageNum;
    private boolean hasPageNum;
    boolean drawingText = false;
    boolean drawingScaledImage = false;
    WPrintGraphicsWrapper wrapper;

    WPrintGraphics(WPrintControl control, WPrintJob job) {
        super(control, job);
	this.job = job;

	// BugId 4258648 -- clearRect() should render a rectangle in the
	// background color of the surface. For PrintGraphics, we assume
	// the surface is a white sheet of paper. This is consistent with
	// the Solaris implementation. Also set the foreground Color to
	// black. This is also consistent with Solaris.
	setColor(Color.black);
	setBackground(Color.white);
    }

    WPrintGraphics(int hdc) {
	super(hdc);
	setForPrinting(true);
	setColor(Color.black);
	setBackground(Color.white);
    }

    public PrintJob getPrintJob() {
        return job;
    }

    /**
     * Create a new WPrintGraphics Object based on this one.
     */
    public Object clone() {
        WPrintGraphics g = (WPrintGraphics) super.clone();
	g.hasPageNum = false;
	g.pageNum = 0;
	return g;
    }

    /**
     * Create a new WPrintGraphics Object based on this one.
     * The WPrintGraphics will be the master object for one page of the
     * document.
     */
    Graphics create(int pageNum) {
        WPrintGraphics g = (WPrintGraphics) create();
	g.hasPageNum = true;
	g.pageNum = pageNum;
	return g;
    }

    final public String getDevID() {
	return newDevID;
    }

    protected void disposeImpl() {
        if (hasPageNum && job != null) {
	    job.flushPage(pageNum);
	}
	super.disposeImpl();
    }

    public ImageData lock(
	int xView, int yView, int dxView, int dyView, int lockFlags)
    {
	ImageData imgData =
	    super.lock(xView, yView, dxView, dyView, lockFlags);
        if (imgData != null) {
            imgData.setResRatio(1.0);
            imgData.setScaleFactorX(1.0);
            imgData.setScaleFactorY(1.0);
            if (drawingText) {
                imgData.setOriginX((int) transform.getTranslateX());
                imgData.setOriginY((int) transform.getTranslateY());
                imgData.setResRatio(wrapper.getRatio());
                imgData.setType( GraphicsPrimitive.TYPE_CUSTOM );
            } else if (drawingScaledImage) {
                imgData.setScaleFactorX(wrapper.getScaleFactorX());
                imgData.setScaleFactorY(wrapper.getScaleFactorY());
            }
        }
	return imgData;
    }

    void preScale(double sx, double sy) {
        transform.preConcatenate(AffineTransform.getScaleInstance(sx, sy));
        invalidateTransform();
    }
}
