/*
 * @(#)WPrintGraphicsWrapper.java	1.9 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.windows;

import java.awt.*;
import java.awt.image.ImageObserver;
import java.text.AttributedCharacterIterator;
import java.awt.geom.AffineTransform;
import sun.awt.Graphics2Delegate;

/*
 * An implementation of java.awt.PrintGraphics, that delegates all of
 * its work to another instance of sun.awt.windows.WPrintGraphics.
 * @version
 */
public class WPrintGraphicsWrapper extends Graphics
    implements PrintGraphics, Graphics2Delegate
{
    private WPrintGraphics target;	// The delegate Graphics for 72dpi
    private double ratio = 1.0;
    private double sx = 1.0;
    private double sy = 1.0;
    WPrintJob pjob;

    public PrintJob getPrintJob() {
	return pjob;
    }

    public WPrintGraphicsWrapper(WPrintGraphics target,
				 WPrintJob pjob) {
	this.target = target;
        this.target.wrapper = this;
	this.pjob = pjob;
	if (pjob != null) {
	    ratio = pjob.truePageResolution / (double) pjob.pageResolution;
	}
    }

    public Graphics create() {
        return new WPrintGraphicsWrapper((WPrintGraphics)target.create(), pjob);
    }

    Graphics create(int pageNum) {
        return new WPrintGraphicsWrapper((WPrintGraphics)target.create(pageNum), pjob);
    }

    public Graphics getTarget() {
	return target;
    }

    public double getRatio() {
        return ratio;
    }

    public double getScaleFactorX() {
        return sx;
    }

    public double getScaleFactorY() {
        return sy;
    }

    public void translate(int x, int y) {
	target.translate(x, y);
    }

    public Color getColor() {
	return target.getColor();
    }

    public void setColor(Color c) {
	target.setColor(c);
    }

    public void setBackground(Color c) {
        target.setBackground(c);
    }

    public void setPaintMode() {
	target.setPaintMode();
    }

    public void setXORMode(Color c1) {
	target.setXORMode(c1);
    }

    public Font getFont() {
	return target.getFont();
    }

    public void setFont(Font font) {
	target.setFont(font);
    }

    public FontMetrics getFontMetrics(Font f) {
	return target.getFontMetrics(f);
    }

    public Rectangle getClipBounds() {
	return target.getClipBounds();
    }

    public void clipRect(int x, int y, int width, int height) {
	target.clipRect(x, y, width, height);
    }

    public void setClip(int x, int y, int width, int height) {
	target.setClip(x, y, width, height);
    }

    public Shape getClip() {
	return target.getClip();
    }

    public void setClip(Shape clip) {
	target.setClip(clip);
    }

    public void copyArea(int x, int y, int width, int height, int dx, int dy) {
	target.copyArea(x, y, width, height, dx, dy);
    }

    public void drawLine(int x1, int y1, int x2, int y2) {
	target.drawLine(x1, y1, x2, y2);
    }

    public void fillRect(int x, int y, int width, int height) {
	target.fillRect(x, y, width, height);
    }

    public void drawRect(int x, int y, int width, int height) {
	target.drawRect(x, y, width, height);
    }

    public void clearRect(int x, int y, int width, int height) {
	target.clearRect(x, y, width, height);
    }

    public void drawRoundRect(int x, int y, int width, int height,
			      int arcWidth, int arcHeight) {
	target.drawRoundRect(x, y, width, height, arcWidth, arcHeight);
    }

    public void fillRoundRect(int x, int y, int width, int height,
			      int arcWidth, int arcHeight) {
	target.fillRoundRect(x, y, width, height, arcWidth, arcHeight);
    }

    public void draw3DRect(int x, int y, int width, int height, 
			   boolean raised) {
	target.draw3DRect(x, y, width, height, raised);
    }

    public void fill3DRect(int x, int y, int width, int height, 
			   boolean raised) {
	target.fill3DRect(x, y, width, height, raised);
    }

    public void drawOval(int x, int y, int width, int height) {
	target.drawOval(x, y, width, height);
    }

    public void fillOval(int x, int y, int width, int height) {
	target.fillOval(x, y, width, height);
    }

    public void drawArc(int x, int y, int width, int height,
			int startAngle, int arcAngle) {
	target.drawArc(x, y, width, height, startAngle, arcAngle);
    }

    public void fillArc(int x, int y, int width, int height,
			int startAngle, int arcAngle) {
	target.fillArc(x, y, width, height, startAngle, arcAngle);
    }

    public void drawPolyline(int xPoints[], int yPoints[], int nPoints) {
	target.drawPolyline(xPoints, yPoints, nPoints);
    }

    public void drawPolygon(int xPoints[], int yPoints[], int nPoints) {
	target.drawPolygon(xPoints, yPoints, nPoints);
    }

    public void fillPolygon(int xPoints[], int yPoints[], int nPoints) {
	target.fillPolygon(xPoints, yPoints, nPoints);
    }

    public void drawString(String str, int x, int y) {
        AffineTransform t = target.getTransform();
        target.scale(ratio, ratio);
        try {
            target.drawingText = true;
            target.drawString(str, x, y);
        } finally {
            target.drawingText = false;
        }
        target.setTransform(t);
    }

    public void drawString(AttributedCharacterIterator iterator,
                                int x, int y) {
        AffineTransform t = target.getTransform();
        target.scale(ratio, ratio);
        try {
            target.drawingText = true;
            target.drawString(iterator, x, y);
        } finally {
            target.drawingText = false;
        }
        target.setTransform(t);
    }

    public void drawChars(char data[], int offset, int length, int x, int y) {
        AffineTransform t = target.getTransform();
        target.scale(ratio, ratio);
        try {
            target.drawingText = true;
            target.drawChars(data, offset, length, x, y);
        } finally {
            target.drawingText = false;
        }
        target.setTransform(t);
    }

    public void drawBytes(byte data[], int offset, int length, int x, int y) {
        AffineTransform t = target.getTransform();
        target.scale(ratio, ratio);
        try {
            target.drawingText = true;
            target.drawBytes(data, offset, length, x, y);
        } finally {
            target.drawingText = false;
        }
        target.setTransform(t);
    }

    interface BooleanFunction {
        public boolean invoke();
    }

    private boolean drawScaledImageHelper(final Image img, 
                                          final int dstX, final int dstY,
                                          final int dstW, final int dstH, 
                                          final int srcW, final int srcH, 
                                          final BooleanFunction drawImage) {
        /*
         * Fix for bug 4257962.
         * Scale up the image here and scale it back down in _dib_unlock
         * to prevent the 2D renderer from artificially pixelating the image.
         */
        boolean b = true;
        if (dstW > 0 && dstH > 0) {
            sx = Math.min(srcW/(double)dstW, ratio);
            sy = Math.min(srcH/(double)dstH, ratio);
        }
        if (sx <= 1.0 && sy <= 1.0) {
            sx = sy = 1.0;
            return drawImage.invoke();
        }
        AffineTransform t = target.getTransform();
        /*
         * When we scale the image back down by call to ::StretchDIBits in _dib_unlock
         * we actually preconcatenate the inversed scale transform. So we have to
         * _preconcatenate_ the direct scale transform here.
         */
        target.preScale(sx, sy);
        /*
         * Since image scaling scales memory consumption accordingly we will draw 
         * the scaled image by pieces. Each piece will be a rectangular cell which
         * size is equal to the page dimension at 72dpi. We assume that 2d quarantees 
         * successful printing (without OutOfMemoryError) of a full-page image 
         * at a default resolution 72dpi.
         */
        Rectangle bounds = getBoundsForPrinting(dstX, dstY, dstW, dstH);
        Dimension cell = getCellDimension();
        try {
            target.drawingScaledImage = true;
            for (int originX = bounds.x; originX < bounds.x + bounds.width; 
                     originX += cell.width) {
                for (int originY = bounds.y; originY < bounds.y + bounds.height; 
                         originY += cell.height) {
                    target.setDevClip(originX, originY, cell.width, cell.height);
                    b = b && drawImage.invoke();
                }
            }
        } finally {
            target.drawingScaledImage = false;
            Dimension page = pjob.getPageDimension();
            target.setDevClip(0, 0, page.width, page.height);
            target.setTransform(t);
            sx = sy = 1.0;
        }
        return b;
    }

    private Rectangle getBoundsForPrinting(int x, int y, int w, int h) {

        // Get the bounding box of the src and transform the corners
        double[] pts = {x, y, x + w, y, x + w, y + h, x, y + h};
        target.getTransform().transform(pts, 0, pts, 0, 4);

        // Get the min, max of the dst
        double maxX = pts[0];
        double maxY = pts[1];
        double minX = pts[0];
        double minY = pts[1];
        for (int i=2; i < 8; i+=2) {
            if (pts[i] > maxX) {
                maxX = pts[i];
            } else if (pts[i] < minX) {
                minX = pts[i];
            }
            if (pts[i+1] > maxY) {
                maxY = pts[i+1];
            } else if (pts[i+1] < minY) {
                minY = pts[i+1];
            }
        }

        // Intersect with a scaled page
        Rectangle r = new Rectangle(0, 0, 0, 0);
        Dimension d = pjob.getPageDimension();
        minX = Math.max(minX, 0);
        minY = Math.max(minY, 0);
        maxX = Math.min(maxX, d.width * sx);
        maxY = Math.min(maxY, d.height * sy);
        if (maxX >= minX && maxY >= minY) {
            r = new Rectangle((int)Math.floor(minX), (int)Math.floor(minY),
                              (int)Math.ceil(maxX - minX), (int)Math.ceil(maxY - minY));
        }
        return r;
    }

    private Dimension getCellDimension() {
        Dimension d = pjob.getPageDimension();
        int res = pjob.getPageResolution();
        return new Dimension((int)Math.floor(d.width * 72 / (double)res),
                             (int)Math.floor(d.height * 72 / (double)res));
    }

    public boolean drawImage(Image img, int x, int y,
			     ImageObserver observer) {
	return target.drawImage(img, x, y, observer);
    }

    public boolean drawImage(final Image img, 
                             final int x, final int y,
                             final int width, final int height,
                             final ImageObserver observer) {
        return drawScaledImageHelper(img, x, y, width, height, 
                                     img.getWidth(observer), img.getHeight(observer),
                                     new BooleanFunction() {
            public boolean invoke() {
                return target.drawImage(img, x, y, width, height, observer);
            }
        });
    }

    public boolean drawImage(Image img, int x, int y,
			     Color bgcolor, ImageObserver observer) {
	return target.drawImage(img, x, y, bgcolor, observer);
    }

    public boolean drawImage(final Image img, 
                             final int x, final int y, 
                             final int width, final int height,
                             final Color bgcolor, final ImageObserver observer) {
        return drawScaledImageHelper(img, x, y, width, height, 
                                     img.getWidth(observer), img.getHeight(observer),
                                     new BooleanFunction() {
            public boolean invoke() {
                return target.drawImage(img, x, y, width, height, bgcolor, observer);
            }
        });
    }

    public boolean drawImage(final Image img,
                             final int dx1, final int dy1, final int dx2, final int dy2,
                             final int sx1, final int sy1, final int sx2, final int sy2,
                             final ImageObserver observer) {
        return drawScaledImageHelper(img, dx1, dy1, dx2 - dx1, dy2 - dy1, 
                                     sx2 - sx1, sy2 - sy1,
                                     new BooleanFunction() {
            public boolean invoke() {
                return target.drawImage(img, dx1, dy1, dx2, dy2,
                                        sx1, sy1, sx2, sy2, observer);
            }
        });
    }

    public boolean drawImage(final Image img,
                             final int dx1, final int dy1, final int dx2, final int dy2,
                             final int sx1, final int sy1, final int sx2, final int sy2,
                             final Color bgcolor, final ImageObserver observer) {
        return drawScaledImageHelper(img, dx1, dy1, dx2 - dx1, dy2 - dy1, 
                                     sx2 - sx1, sy2 - sy1,
                                     new BooleanFunction() {
            public boolean invoke() {
                return target.drawImage(img, dx1, dy1, dx2, dy2,
                                        sx1, sy1, sx2, sy2, bgcolor, observer);
            }
        });
    }

    public void dispose() {
	target.dispose();
    }

    public void finalize() {
	// We don't want Graphics.finalize() called, because that gives
	// us an extra dispose().
    }

    public String toString() {
	return getClass().getName() + "[target=" + target.toString() + "]";
    }

    public Rectangle getClipRect() {
	return target.getClipBounds();
    }

    public boolean hitClip(int x, int y, int width, int height) {
	return target.hitClip(x, y, width, height);
    }

    public Rectangle getClipBounds(Rectangle r) {
	return target.getClipBounds(r);
    }
}
