/*
 * @(#)WPrintJob.java	1.22 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.windows;

import java.awt.*;
import java.util.Properties;

/**
 * A class which initiates and executes a Win32 print job.
 *
 * @version 1.22, 02/02/00
 * @author Erik Gilbert
 * @author David Mendenhall
 */
public class WPrintJob extends PrintJob {
    private Dimension              pageDimension = new Dimension(612, 792);
    int                            pageResolution = 72;
    int                            truePageResolution = 300;
    private boolean                lastFirst = false;
    private int                    pageCount = 0;
    private WPrintGraphics         graphics = null;
    private WPrintGraphicsWrapper  gw = null;

    static {
        initIDs();
    }

    static WPrintJob createPrintJob(WPrintControl control) {
        WPrintJob job = new WPrintJob(control);
  
	if (job.cancelled()) {
	    return null;
	}
	else {
            return job;
	}
    }

    private WPrintJob(WPrintControl control) {
        graphics = new WPrintGraphics(control, this);
	if (graphics.pData != 0) {
	    gw = graphics.wrapper = new WPrintGraphicsWrapper(graphics, this);
	}
	else {
	    graphics.dispose();
	    graphics = null;
	}
    }

    private boolean cancelled() {
        return (graphics == null);
    }

    private native void newPage();

    /**
     * Gets a Graphics object that will draw to the next page.
     * The page is flushed to the printer when the graphics
     * object is disposed.
     */
    public synchronized Graphics getGraphics() {
        if (graphics == null) {
	    return null;
	}
	newPage();
	Graphics g = gw.create(++pageCount);

	// Fix for bug #4080817 -- Null pointer exception from drawString()
	// on a Graphics object of PrintJob.
	// The problem here was that no default font was set on the graphics
	// object returned.
	// "Dialog, plain, 12 points" matches the default font for Frame.
	g.setFont(new Font("Dialog", Font.PLAIN, 12));
        g.setColor(Color.black) ; // default Win32 GDI text printing colour

	return g;
    }

    /**
     * Returns the dimensions of the page in pixels.
     */
    public Dimension getPageDimension() {
	return pageDimension;
    }

    /**
     * Returns the resolution of the page in pixels per inch.
     */
    public int getPageResolution() {
	return pageResolution;
    }

    /**
     * Returns true if the last page should be printed first.
     */
    public boolean lastPageFirst() {
	return lastFirst;
    }

    private native void flushPageImpl();

    synchronized void flushPage(int pageNum) {
        // if (pageNum < pageCount), then we already flushed page <pageNum>
        // as part of a call to newPage()
        if (pageNum == pageCount && graphics != null) {
	    flushPageImpl();
	}
    }

    /**
     * Ends the print job and does cleanup.
     */
    private native void endImpl();

    public synchronized void end() {
        if (graphics != null) {
	    endImpl();
	    graphics.dispose();
	    graphics = null;
	}
    }

    /**
     * Initialize JNI field IDs
     */
    private static native void initIDs();
}
