/*
 * @(#)WScrollbarPeer.java	1.21 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.windows;

import java.awt.*;
import java.awt.peer.*;
import java.awt.event.AdjustmentEvent;

class WScrollbarPeer extends WComponentPeer implements ScrollbarPeer {

    // ComponentPeer overrides

    public Dimension getMinimumSize() {
	if (((Scrollbar)target).getOrientation() == Scrollbar.VERTICAL) {
	    return new Dimension(15, 50);
	}
	else {
	    return new Dimension(50, 15);
	}
    }

    // ScrollbarPeer implementation

    public native void setValues(int value, int visible,
				 int minimum, int maximum);
    public native void setLineIncrement(int l);
    public native void setPageIncrement(int l);

    // Toolkit & peer internals

    WScrollbarPeer(Scrollbar target) {
	super(target);
    }

    native void create(WComponentPeer parent);

    void initialize() {
	Scrollbar sb = (Scrollbar)target;
	setValues(sb.getValue(), sb.getVisibleAmount(),
		  sb.getMinimum(), sb.getMaximum());
	super.initialize();
    }

    // native callbacks
    //!CQ need name update--fix awt_scrollbar.cpp also
    //!CQ also, why direct callbacks & not sendEvents()?

    void lineUp(final int value) {
        final Scrollbar sb = (Scrollbar)target;
	WToolkit.executeOnEventHandlerThread(sb, new Runnable() {
	    public void run() {
		sb.setValue(value);
	        postEvent(new AdjustmentEvent(sb, 
                                     AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                     AdjustmentEvent.UNIT_DECREMENT, value));
	    }
	});
    }

    void lineDown(final int value) {
        final Scrollbar sb = (Scrollbar)target;
	WToolkit.executeOnEventHandlerThread(sb, new Runnable() {
	    public void run() {
		sb.setValue(value);
	        postEvent(new AdjustmentEvent(sb, 
                                      AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                      AdjustmentEvent.UNIT_INCREMENT, value));
	    }
	});
    }

    void pageUp(final int value) {
        final Scrollbar sb = (Scrollbar)target;
	WToolkit.executeOnEventHandlerThread(sb, new Runnable() {
	    public void run() {
		sb.setValue(value);
	        postEvent(new AdjustmentEvent(sb, 
                                      AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                      AdjustmentEvent.BLOCK_DECREMENT, value));
	    }
	});
    }

    void pageDown(final int value) {
        final Scrollbar sb = (Scrollbar)target;
	WToolkit.executeOnEventHandlerThread(sb, new Runnable() {
	    public void run() {
		sb.setValue(value);
	        postEvent(new AdjustmentEvent(sb, 
                                      AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                      AdjustmentEvent.BLOCK_INCREMENT, value));
	    }
	});
    }

    void dragBegin(final int value) {
// New event model event -- not supported yet.
//	((Scrollbar)target).setValue(value);
//	postEvent(new Event(target, Event.SCROLL_BEGIN, new Integer(value)));
    }

    void dragAbsolute(final int value) {
        final Scrollbar sb = (Scrollbar)target;
	WToolkit.executeOnEventHandlerThread(sb, new Runnable() {
	    public void run() {
		sb.setValue(value);
	        postEvent(new AdjustmentEvent(sb, 
                                      AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                      AdjustmentEvent.TRACK, value));
	    }
	});
    }

    void dragEnd(final int value) {
// New event model event -- not supported yet.
//	((Scrollbar)target).setValue(value);
//	postEvent(new Event(target, Event.SCROLL_END, new Integer(value)));
    }

    public boolean shouldClearRectBeforePaint() {
        return false;
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	return getMinimumSize();
    }

}
