/*
 * @(#)signals_md.c	1.7 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Win32 dependent interrupt handling code.
 */


/*
 * Header files.
 */
#include <windows.h>
#include <signal.h>
#include "sys_api.h"

#include "oobj.h"
#include "interpreter.h"

/*
 * This function is included primarily as a debugging aid. If Java is
 * running in a console window, then pressing <CTRL-BREAK> will cause
 * the current state of all active threads and monitors to be written
 * to the console window.
 */
static void
UserHandler(int sig, void *siginfo, void *context)
{
    sysSignalNotify(sig);
    /* We need to reinstate the signal handler each time... */
    sysSignal(sig, UserHandler);
}


#ifndef DEBUG
/* Actual instruction locations inside assembly interp for divide trapping */
extern _idiv_inst5;
extern _idiv_trap5;
extern _idiv_inst6;
extern _idiv_trap6;
extern _irem_inst5;
extern _irem_trap5;
extern _irem_inst6;
extern _irem_trap6;
#endif /*ifndef DEBUG*/

static long WINAPI
JavaVMUnhandledExceptionFilter(LPEXCEPTION_POINTERS lpEP)
{
    long code = lpEP->ExceptionRecord->ExceptionCode;
    if ((code == EXCEPTION_INT_OVERFLOW) ||
	(code == EXCEPTION_INT_DIVIDE_BY_ZERO)) {
#ifndef DEBUG
	PCONTEXT pcon = lpEP->ContextRecord;
	if (pcon->Eip == (DWORD)&_idiv_inst5) {
	    pcon->Eip =  (DWORD)&_idiv_trap5;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
	if (pcon->Eip == (DWORD)&_idiv_inst6) {
	    pcon->Eip =  (DWORD)&_idiv_trap6;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
	if (pcon->Eip == (DWORD)&_irem_inst5) {
	    pcon->Eip =  (DWORD)&_irem_trap5;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
	if (pcon->Eip == (DWORD)&_irem_inst6) {
	    pcon->Eip =  (DWORD)&_irem_trap6;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
#endif /*ifndef DEBUG*/
	return EXCEPTION_CONTINUE_SEARCH;
    }
    return EXCEPTION_CONTINUE_SEARCH;
}

JNIEXPORT void * JNICALL
JVM_RegisterSignal(jint sig, void * handler)
{
    signal_handler_t newHandler =
        handler == (void *)2 ? UserHandler : (signal_handler_t)handler;
    signal_handler_t oldHandler;

    switch (sig) {
    case SIGFPE:
        return (void *)-1; /* already used by VM */
	
    case SIGBREAK:
        if (!reduce_signal_usage)
	    return (void *)-1;
    }

    oldHandler = sysSignal(sig, newHandler);
    if (oldHandler == UserHandler) {
        return (void *)2;
    } else {
        return (void *)oldHandler;
    }
}

JNIEXPORT void JNICALL
JVM_RaiseSignal(jint sig)
{
    sysRaise(sig);
}

JNIEXPORT jint JNICALL
JVM_FindSignal(const char *name)
{
    if (!strcmp(name, "ABRT"))  return SIGABRT;
    if (!strcmp(name, "FPE"))   return SIGFPE;
    if (!strcmp(name, "SEGV"))  return SIGSEGV;
    if (!strcmp(name, "INT"))   return SIGINT;
    if (!strcmp(name, "TERM"))  return SIGTERM;
    if (!strcmp(name, "ILL"))  return SIGILL;
    return -1;
}

static void sigthread(void *arg)
{
    while (1) {
        int sig = sysSignalWait();
	if (sig == SIGBREAK) {
	    HandleSignalInVM(TRUE);
	} else {
	    ExecEnv *ee = EE();
	    ClassClass *sigCls = FindStickySystemClass(ee, "sun/misc/Signal", TRUE);
	    if (sigCls == NULL) {
	        exceptionClear(ee);
		continue;
	    }
	    execute_java_static_method(ee, sigCls, "dispatch", "(I)V", sig);
	    exceptionClear(ee);
	}
    }
}

/* The Win32 C runtime library maps all console control events other than ^C
 * into SIGBREAK, which makes it impossible to distinguish ^BREAK from close,
 * logoff, and shutdown events.  We therefore install our own console handler
 * that raises SIGTERM for the latter cases.
 */
static BOOL WINAPI
consoleHandler(DWORD event)
{
    switch(event) {
    case CTRL_C_EVENT:
	sysRaise(SIGINT);
	return TRUE;
	break;
    case CTRL_BREAK_EVENT:
	sysRaise(SIGBREAK);
	return TRUE;
	break;
    case CTRL_CLOSE_EVENT:
    case CTRL_LOGOFF_EVENT:
    case CTRL_SHUTDOWN_EVENT:
	sysRaise(SIGTERM);
	return TRUE;
	break;
    default:
	break;
    }
    return FALSE;
}

void InitializeSignals()
{
    if (!reduce_signal_usage) {
        createSystemThread("Signal dispatcher", MaximumPriority, 0,
			   sigthread, 0);
	sysSignal(SIGBREAK, UserHandler);
	SetConsoleCtrlHandler(consoleHandler, TRUE);
    }
    SetUnhandledExceptionFilter(JavaVMUnhandledExceptionFilter);
}
