/*
 * @(#)PLATFORM_API_WinOS_MidiIn.c	1.7 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*****************************************************************************/
/*
**	PLATFORM_API_WinOS_MidiIn.c
**
**	This provides platform specfic MIDI input functions for Windows 95/NT. 
**	This implementation does not interface with the HAE engine at all.
**  This interface is for Windows 95/NT and uses the midiIn API to receive 
**	MIDI messages through the multimedia system.
**
**	Overview:
**
**	History	-
**	06.24.99	Created
*/
/*****************************************************************************/


#include <windows.h>
#include <mmsystem.h>
#include <windowsx.h>

#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include "PlatformMidi.h"


#if USE_PLATFORM_MIDI_IN == TRUE

// linked list of all MIDI messages received.  messages are added to this list as they
// are received by MIDI_IN_PutMessage, and removed as they are read by MIDI_IN_GetMessage.
static MIDI_MESSAGE	*midiInQueue = NULL;

// very simple locking mechanism.  midiInLock gets incremented when someone wants to change
// the event queue.
int midiInLock = 0;

static void AddToQueue(MIDI_MESSAGE *pMessage)
{

	static MIDI_MESSAGE	* lastMessage = NULL;

	// wait for lock to clear
	while (midiInLock) {  }
					  
	// we're changing the queue
	midiInLock++;

	//fprintf(stderr, "> AddToQueue: midiInQueue: %d, lastMessage: %d, pMessage: %d\n", midiInQueue, lastMessage, pMessage);

	if (midiInQueue == NULL)
	{
		midiInQueue = pMessage;
		lastMessage = midiInQueue;
	}
	else
	{
		lastMessage->pNext = pMessage;
		lastMessage = pMessage;
	}

	// we're done changing the queue
	midiInLock--;

	//fprintf(stderr, "< AddToQueue: midiInQueue: %d, lastMessage: %d\n", midiInQueue, lastMessage);
}

static MIDI_MESSAGE * GetFromQueue()
{
	MIDI_MESSAGE *pMessage;

	//fprintf(stderr, "> GetFromQueue(): midiInQueue: %d\n", midiInQueue);
	
	// wait for lock to clear
	while (midiInLock) {  }
					 
	// we're changing the queue
	midiInLock++;

	pMessage = midiInQueue;

	if (pMessage != NULL)
	{
		midiInQueue = pMessage->pNext;
	}

	// we're done changing the queue
	midiInLock--;

	//fprintf(stderr, "< GetFromQueue(): midiInQueue: %d, pMessage %d\n", midiInQueue, pMessage);
	return pMessage;
}


/* 
 * Callback from the MIDI device for all messages.
 */
void CALLBACK MIDI_IN_PutMessage( HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2 )
{
	
	//fprintf(stderr, "\nMIDI_IN_PutMessage, hMidiIn: %d, wMsg: %d, dwInstance: %d, dwParam1: %d, dwParam2: %d\n\n", hMidiIn, wMsg, dwInstance, dwParam1, dwParam2);
	
	switch( wMsg )
	{

	case MIM_OPEN:

		//fprintf(stderr, "MIDI_IN_PutMessage: MIM_OPEN!\n");
		break;

	case MIM_CLOSE:
		
		//fprintf(stderr, "MIDI_IN_PutMessage: MIM_CLOSE!\n");
		break;

	case MIM_MOREDATA:
	case MIM_DATA:
		{
			MIDI_MESSAGE * pMessage = malloc( sizeof(MIDI_MESSAGE) );
			// fprintf(stderr, "MIDI_IN_PutMessage: MIM_MOREDATA or MIM_DATA!\n");

/*
//			{
//				union 
//				{
//					DWORD dwData;
//					unsigned char bData[4];
//				} u;
//  
//				u.dwData = dwParam1;
//				fprintf(stderr, "fukme status: %u, data1: %u, data2: %u\n", u.bData[0], u.bData[1], u.bData[2]);
//			}
*/
	
			pMessage->deviceHandle = (long)hMidiIn;
			pMessage->objectHandle = (long)dwInstance;
			pMessage->type = SHORT_MESSAGE;
			pMessage->length = 3;	// max!  status + 2 data bytes
			pMessage->pData = (unsigned char *)&dwParam1;
			pMessage->timestamp = (int)dwParam2;
			pMessage->pNext = NULL;

			AddToQueue(pMessage);
			break;
		}

	case MIM_LONGDATA:

			//fprintf(stderr, "MIDI_IN_PutMessage: MIM_LONGDATA!\n");
			break;

	case MIM_ERROR:

			//fprintf(stderr, "ERROR: MIDI_IN_PutMessage: MIM_ERROR!\n");
			break;

	case MIM_LONGERROR:

			//fprintf(stderr, "ERROR: MIDI_IN_PutMessage: MIM_LONGERROR!\n");
			break;

	default:

			//fprintf(stderr, "ERROR: MIDI_IN_PutMessage: unknown message %d!\n", wMsg);
			break;

	} // switch (wMsg)
}


// PLATFORM_MIDI_IN method implementations


long	MIDI_IN_GetNumDevices(void) 
{
		return (long)midiInGetNumDevs();
}

	
long	MIDI_IN_GetDeviceName(long deviceID, char *name, unsigned long nameLength)
{
	MIDIINCAPS midiInCaps;

	if( midiInGetDevCaps( deviceID, &midiInCaps, sizeof(MIDIINCAPS) ) != MMSYSERR_NOERROR )
	{
		return (FALSE);
	}
	else
	{
		strncpy(name, midiInCaps.szPname, nameLength-1);
		name[nameLength-1] = 0; 
	}

	return (TRUE);
}
	

long	MIDI_IN_GetDeviceVendor(long deviceID, char *name, unsigned long nameLength)
{
	/* $$kk: 06.24.99: need to implement */	
	return (FALSE);
}


long	MIDI_IN_GetDeviceDescription(long deviceID, char *name, unsigned long nameLength)
{
	/* $$kk: 06.24.99: need to implement */	
	return (FALSE);
}

	
long	MIDI_IN_GetDeviceVersion(long deviceID, char *name, unsigned long nameLength)
{
	/* $$kk: 06.24.99: need to implement */	
	return (FALSE);
}

	
long	MIDI_IN_OpenDevice(long deviceID, long objectHandle)
{
	UINT err = MMSYSERR_NOERROR;
	HMIDIIN deviceHandle = 0;

	//fprintf(stderr,	"MIDI_IN_OpenDevice: objectHandle: %d\n", objectHandle);
	
	err = midiInOpen( &deviceHandle, (UINT)deviceID, (DWORD)&(MIDI_IN_PutMessage), (DWORD)objectHandle, CALLBACK_FUNCTION );

	if ( (err != MMSYSERR_NOERROR) || (!deviceHandle) )
	{
		//fprintf(stderr, "ERROR: MIDI_IN_OpenDevice: midiInOpen returned an error: %d\n", err);
		deviceHandle = 0;
	}
	else
	{
		//fprintf(stderr, "MIDI_IN_OpenDevice: midiInOpen succeeded\n");
	}	

	return (long)deviceHandle;
}

	
long	MIDI_IN_CloseDevice(long deviceHandle)
{
	MMRESULT err = midiInClose((HMIDIIN)deviceHandle);

	if (err != MMSYSERR_NOERROR)
	{
		//fprintf(stderr, "ERROR: MIDI_IN_CloseDevice: midiInClose returned an error: %d\n", err);
		return 0; // failure
	}
	else
	{
		//fprintf(stderr, "MIDI_IN_CloseDevice: midiInClose succeeded\n");
		return 1; // success
	}
}

	
long	MIDI_IN_StartDevice(long deviceHandle)
{
	// $$kk: 06.24.99: do thread stuff here!!

	MMRESULT err = midiInStart((HMIDIIN)deviceHandle);

	if (err != MMSYSERR_NOERROR)
	{
		//fprintf(stderr, "ERROR: MIDI_IN_StartDevice: midiInStart returned an error: %d\n", err);
		return 0; // failure
	}
	else
	{
		//fprintf(stderr, "MIDI_IN_StartDevice: midiInStart succeeded\n");
		return 1; // success
	}
}

	
long	MIDI_IN_StopDevice(long deviceHandle)
{
	// $$kk: 06.24.99: do thread stuff here!!

	MIDI_MESSAGE * msg = midiInQueue;	
	MMRESULT err = midiInStop((HMIDIIN)deviceHandle);

	// wait for lock to clear
	while (midiInLock) {  }
					  
	// we're changing the queue
	midiInLock++;

	// clear all the events with this device handle from the queue
	while (midiInQueue != NULL)
	{
		if (midiInQueue->deviceHandle == deviceHandle)
		{
			midiInQueue = msg->pNext;
		}
		else
		{
			break;
		}

		msg = midiInQueue;
		while ( (msg = msg->pNext) != NULL)
		{
			while (msg->pNext->deviceHandle == deviceHandle)
			{
				msg->pNext = msg->pNext->pNext;
			}
		}
	}

	// we're done changing the queue
	midiInLock--;

	if (err != MMSYSERR_NOERROR)
	{
		//fprintf(stderr, "ERROR: MIDI_IN_StopDevice: midiInStop returned an error: %d\n", err);
		return 0; // failure
	}
	else
	{
		//fprintf(stderr, "MIDI_IN_StopDevice: midiInStop succeeded\n");
		return 1; // success
	}
}


// read the next message from the queue
MIDI_MESSAGE * MIDI_IN_GetMessage()
{
	return GetFromQueue();
}

#endif // USE_PLATFORM_MIDI_IN
