/*
 * @(#)PLATFORM_API_WinOS_MidiOut.c	1.5 00/02/02
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
/*****************************************************************************/
/*
**	PLATFORM_API_WinOS_MidiOut.c
**
**	This provides platform specfic MIDI output functions for Windows 95/NT. 
**	This implementation does not interface with the HAE engine at all.
**  This interface is for Windows 95/NT and uses the midiOut API to send 
**	MIDI messages through the multimedia system.
**
**	Overview:
**
**	History	-
**	06.24.99	Created
*/
/*****************************************************************************/


#include <windows.h>
#include <mmsystem.h>
#include <windowsx.h>

#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include "PlatformMidi.h"


#if USE_PLATFORM_MIDI_OUT == TRUE


// PLATFORM_MIDI_OUT method implementations


long	MIDI_OUT_GetNumDevices(void) 
{
	// add one for the MIDI_MAPPER
	// we want to return it first so it'll be the default, so we
	// decrement each deviceID for these methods....
	return (long)(midiOutGetNumDevs() + 1);
}

	
long	MIDI_OUT_GetDeviceName(long deviceID, char *name, unsigned long nameLength)
{
	MIDIOUTCAPS midiOutCaps;

	if (deviceID == 0)
	{
		deviceID = MIDI_MAPPER;
	}
	else
	{
		deviceID -= 1;
	}

	if( midiOutGetDevCaps( deviceID, &midiOutCaps, sizeof(MIDIOUTCAPS) ) != MMSYSERR_NOERROR )
	{
		//fprintf(stderr, "ERROR: MIDI_OUT_GetDeviceName: failed to get MIDIOUTCAPS for deviceID %d\n", deviceID);
		return (FALSE);
	}
	else
	{
		strncpy(name, midiOutCaps.szPname, nameLength-1);
		name[nameLength-1] = 0; 
	}

	return (TRUE);
}
	

long	MIDI_OUT_GetDeviceVendor(long deviceID, char *name, unsigned long nameLength)
{
	/* $$kk: 06.24.99: need to implement */	
	return (FALSE);
}


long	MIDI_OUT_GetDeviceDescription(long deviceID, char *name, unsigned long nameLength)
{
	MIDIOUTCAPS midiOutCaps;
	char *desc;

	if (deviceID == 0)
	{
		deviceID = MIDI_MAPPER;
	}
	else
	{
		deviceID -= 1;
	}

	if( midiOutGetDevCaps( deviceID, &midiOutCaps, sizeof(MIDIOUTCAPS) ) != MMSYSERR_NOERROR )
	{
		//fprintf(stderr, "ERROR: MIDI_OUT_GetDeviceName: failed to get MIDIOUTCAPS for deviceID %d\n", deviceID);
		return (FALSE);
	}
	else
	{
		int tech = (int)midiOutCaps.wTechnology;

		//fprintf(stdout, "midiOutCaps.szPname: %s\n", midiOutCaps.szPname);

		switch(tech) 
		{
		case MOD_MIDIPORT:
			//fprintf(stdout, "MIDI_OUT_GetDeviceName: MOD_MIDIPORT\n");
			desc = "External MIDI Port";
			break;
		case MOD_SQSYNTH:
			//fprintf(stdout, "MIDI_OUT_GetDeviceName: MOD_SQSYNTH\n");
			desc = "Internal square wave synthesizer";
			break;
		case MOD_FMSYNTH:
			//fprintf(stdout, "MIDI_OUT_GetDeviceName: MOD_FMSYNTH\n");
			desc = "Internal FM synthesizer";
			break;
		case MOD_SYNTH:
			//fprintf(stdout, "MIDI_OUT_GetDeviceName: MOD_SYNTH\n");
			desc = "Internal synthesizer (generic)";
			break;
		case MOD_MAPPER:
			//fprintf(stdout, "MIDI_OUT_GetDeviceName: MOD_MAPPER\n");
			desc = "Windows MIDI_MAPPER driver";
			break;
		default:
			//fprintf(stdout, "MIDI_OUT_GetDeviceName: UNKNOWN TECHNOLOGY VALUE\n");
			return (FALSE);
		}

		strncpy(name, desc, nameLength-1);
		name[nameLength-1] = 0; 
	}

	return (TRUE);
}

	
long	MIDI_OUT_GetDeviceVersion(long deviceID, char *name, unsigned long nameLength)
{
	/* $$kk: 06.24.99: need to implement */	
	return (FALSE);
}

	
long	MIDI_OUT_OpenDevice(long deviceID, long objectContext)
{
	UINT err = MMSYSERR_NOERROR;
	HMIDIOUT deviceHandle = 0;

	if (deviceID == 0)
	{
		deviceID = MIDI_MAPPER;
	}
	else
	{
		deviceID -= 1;
	}

	//fprintf(stdout, ">> MIDI_OUT_OpenDevice: deviceID: %d, objectContext: %d\n", deviceID, objectContext);

	err = midiOutOpen( &deviceHandle, (UINT)deviceID, (DWORD)0, (DWORD)objectContext, (DWORD)0L );

	if ( (err != MMSYSERR_NOERROR) || (!deviceHandle) )
	{
		//fprintf(stdout, "ERROR: MIDI_OUT_OpenDevice: midiOutOpen returned an error: %d\n", err);
		deviceHandle = 0;
	}
	else
	{
		//fprintf(stdout, "<< MIDI_OUT_OpenDevice: midiOutOpen succeeded, returning %d\n", (long)deviceHandle);
	}	

	return (long)deviceHandle;
}

	
long	MIDI_OUT_CloseDevice(long deviceHandle)
{
	MMRESULT err;
	int channel;
	
	//fprintf(stdout, ">> MIDI_OUT_CloseDevice: deviceHandle: %d\n", deviceHandle);

	// issue a "SUSTAIN OFF" message to each MIDI channel, 0 to 15.
	// "CONTROL CHANGE" is 176, "SUSTAIN CONTROLLER" is 64, and the value is 0.
	// $$kk: 07.16.99: MidiOutReset is supposed to do this.  it does turn notes off for each channel, but 
	// does not seem to turn off the sustain controller, which appears to override the all-notes-off message.
	// $$kk: 07.16.99: is this right or should it be 1 to 16??
	// $$kk: 07.16.99: it's right!
	for (channel = 0; channel < 16; channel++) 
	{
		if (!MIDI_OUT_ProcessShortMessage(deviceHandle, (unsigned char)(176 + channel), (unsigned char)64, (unsigned char)0, (unsigned long)-1))
		{
			//fprintf(stdout, "ERROR: MIDI_OUT_CloseDevice: MIDI_OUT_ProcessShortMessage(SUSTAIN OFF, channel %d) failed\n", channel);
		}
	}


	err = midiOutReset((HMIDIOUT)deviceHandle);
	//fprintf(stdout, "MIDI_OUT_CloseDevice: midiOutReset returned %d\n", err);

	if (err == MMSYSERR_NOERROR)
	{
		err = midiOutClose((HMIDIOUT)deviceHandle);
		//fprintf(stdout, "MIDI_OUT_CloseDevice: midiOutClose returned %d\n", err);
	}
	else
	{
		//fprintf(stdout, "ERROR: MIDI_OUT_CloseDevice: midiOutReset returned an error: %d\n", err);
	}

	if (err != MMSYSERR_NOERROR)
	{
		//fprintf(stdout, "<< ERROR: MIDI_OUT_CloseDevice: midiOutClose returned an error: %d\n", err);
		return 0; // failure
	}
	else
	{
		//fprintf(stdout, "<< MIDI_OUT_CloseDevice: midiOutClose succeeded\n");
		return 1; // success
	}
}


long	MIDI_OUT_ProcessShortMessage(long deviceHandle, unsigned char status, unsigned char data1, unsigned char data2, unsigned long timestamp) 
{
	MMRESULT err = MMSYSERR_NOERROR;

	union 
	{
		DWORD dwData;
		char bData[4];
	} u;

	//fprintf(stdout, "MIDI_OUT_ProcessShortMessage: deviceHandle: %d, status: %d, data1: %d, data2: %d\n", deviceHandle, status, data1, data2);

	/* Construct the MIDI message */
	u.bData[0] = status;
	u.bData[1] = data1;
	u.bData[2] = data2;
	u.bData[3] = 0;

	/* Send the message */
	err = midiOutShortMsg((HMIDIOUT)deviceHandle, u.dwData);

	return ( (err == MMSYSERR_NOERROR) ? 1 : 0  );
}
	
#endif // USE_PLATFORM_MIDI_OUT
