/*
 * @(#)java_props_md.c	1.21 01/04/21
 *
 * Copyright 1994-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include <shlobj.h>
#include <objidl.h>
#include <locale.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include "locale_str.h"
#include "java_props.h"

#ifndef VER_PLATFORM_WIN32_WINDOWS
#define VER_PLATFORM_WIN32_WINDOWS 1
#endif

#define SHELL_KEY "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders"

/* 
 * Code to figure out the user's home directory using the registry
*/
static char *
getHomeFromRegistry()
{
    HKEY key;
    int rc;
    DWORD type;
    char *p;
    char path[MAX_PATH+1];
    int size = MAX_PATH+1;

    rc = RegOpenKeyEx(HKEY_CURRENT_USER, SHELL_KEY, 0, KEY_READ, &key);
    if (rc != ERROR_SUCCESS) {
	// Shell folder doesn't exist??!!
	return NULL;
    }

    path[0] = 0;
    rc = RegQueryValueEx(key, "Desktop", 0, &type, path, &size);
    if (rc != ERROR_SUCCESS || type != REG_SZ) {
        return NULL;
    }

    /* Get the parent of Desktop directory */
    p = strrchr(path, '\\');
    if (p == NULL) {
	return NULL;
    }
    *p = '\0';
    return strdup(path);
}

/*
 * Code to figure out the user's home directory using shell32.dll
 */
typedef HRESULT (WINAPI *GetSpecialFolderType)(HWND, int, LPITEMIDLIST *);
typedef BOOL (WINAPI *GetPathFromIDListType)(LPCITEMIDLIST, LPSTR);

char *
getHomeFromShell32()
{
    HMODULE lib = LoadLibrary("SHELL32.DLL");
    GetSpecialFolderType do_get_folder;
    GetPathFromIDListType do_get_path;
    HRESULT rc;
    LPITEMIDLIST item_list = 0;
    char *p;
    char path[MAX_PATH+1];
    int size = MAX_PATH+1;

    if (lib == 0) {
	// We can't load the library !!??
	return NULL;
    }

    do_get_folder = (GetSpecialFolderType)GetProcAddress(lib, "SHGetSpecialFolderLocation");
    do_get_path = (GetPathFromIDListType)GetProcAddress(lib, "SHGetPathFromIDListA");

    if (do_get_folder == 0 || do_get_path == 0) {
	// the library doesn't hold the right functions !!??
	return NULL;
    }

    rc = (*do_get_folder)(NULL, CSIDL_DESKTOPDIRECTORY, &item_list);
    if (!SUCCEEDED(rc)) {
	// we can't find the shell folder.
	return NULL;
    }

    path[0] = 0;
    (*do_get_path)(item_list, path);

    /* Get the parent of Desktop directory */
    p = strrchr(path, '\\');
    if (p) {
        *p = 0;
    }

    /*
     * We've been successful.  Note that we don't free the memory allocated
     * by ShGetSpecialFolderLocation.  We only ever come through here once,
     * and only if the registry lookup failed, so it's just not worth it. 
     *
     * We also don't unload the SHELL32 DLL.  We've paid the hit for loading
     * it and we may need it again later.
     */

    return strdup(path);
}
typedef BOOL (WINAPI *ProcPresentType)(DWORD);

java_props_t *
GetJavaProperties()
{
    static java_props_t sprops = {0};
    char *v; /* tmp var */

    if (sprops.user_dir) {
        return &sprops;
    }

    /* AWT properties */
    sprops.awt_toolkit = "sun.awt.windows.WToolkit";

    /* tmp dir */
    {
        char tmpdir[MAX_PATH + 1];
        /* we might want to check that this succeed */
        GetTempPath(MAX_PATH + 1, tmpdir);
        sprops.tmp_dir = strdup(tmpdir);
    }

    /* Printing properties */
    sprops.printerJob = "sun.awt.windows.WPrinterJob";

    /* Java2D properties */
    sprops.graphics_env = "sun.awt.Win32GraphicsEnvironment";

    {
        char buf[max(512, MAX_PATH+1)];
        char *path = getenv("JAVA_FONTS");
        if (path != 0) {
            if (sprintf(buf, "%s", path) != -1) {
                v = buf; /* sysNativePath(buf); Is this necessary? */
            } else {
                v = "";
            }
        } else {
            v = "";
        }
        sprops.font_dir = strdup(v);
    }

    /* OS properties */
    {
        char buf[100];
        OSVERSIONINFO ver;    
	ver.dwOSVersionInfoSize = sizeof(ver);
	GetVersionEx(&ver);
	switch (ver.dwPlatformId) {
	case VER_PLATFORM_WIN32s:
	    sprops.os_name = "Windows 3.1";
	    break;
	case VER_PLATFORM_WIN32_WINDOWS:
           if (ver.dwMajorVersion == 4) {
                switch (ver.dwMinorVersion) {
                case 0:
                    sprops.os_name = "Windows 95";
                    break;
                case 10:
                    sprops.os_name = "Windows 98";
                    break;
                case 90:
                    sprops.os_name = "Windows Me";
                    break;
                default:
                    sprops.os_name = "Windows 9X (unknown)";
                }
            } else {
                sprops.os_name = "Windows 9X (unknown)";
            }
            break;
	case VER_PLATFORM_WIN32_NT:
            if (ver.dwMajorVersion <= 4) {
	        sprops.os_name = "Windows NT";
            } else if (ver.dwMajorVersion == 5) {
                sprops.os_name = "Windows 2000";
            } else {
                sprops.os_name = "Windows NT (unknown)";
            }
	    break;
        default:
            sprops.os_name = "Windows (unknown)";
            break;
	}
	sprintf(buf, "%d.%d", ver.dwMajorVersion, ver.dwMinorVersion);
	sprops.os_version = strdup(buf);
#if _X86_
        sprops.os_arch = "x86";
#else
        sprops.os_arch = "unknown";
#endif
    }

    /* Endianness of platform */
    {
        unsigned int endianTest = 0xff000000; 
        if (((char*)(&endianTest))[0] != 0) {
            sprops.cpu_endian = "big";
        } else {
            sprops.cpu_endian = "little";
        }
    }

    /* ISA list */
    {
        int procType = 0;
        SYSTEM_INFO info;
        HMODULE lib;
        ProcPresentType proc_present;

        GetSystemInfo(&info);
        
        if (strcmp(sprops.os_name, "Windows NT") == 0) {
            boolean mmx = 0;
            char list[100];
            list[0] = 0;

            lib = LoadLibrary("KERNEL32");
            proc_present = (ProcPresentType)GetProcAddress(lib,
                                               "IsProcessorFeaturePresent");
            if (proc_present) {
                mmx = (*proc_present)(PF_MMX_INSTRUCTIONS_AVAILABLE);
                procType = info.wProcessorLevel;
                switch (procType) {
                    case 6:
                        if (mmx) {
                            strcat(list, "pentium_pro+mmx pentium_pro ");
                        } else {
                            strcat(list, "pentium_pro ");
                        }
                        /* FALL THROUGH */
                    case 5:
                        if (mmx) {
                            strcat(list, "pentium+mmx pentium ");
                        } else {
                            strcat(list, "pentium ");
                        }
                        /* FALL THROUGH */
                    case 4:
                        strcat(list, "i486 ");
                        /* FALL THROUGH */
                    case 3:
                        strcat(list, "i386");
                }
                sprops.cpu_isalist = strdup(list);
            }
        } else {
            procType = info.dwProcessorType;
            switch(procType) {
                case PROCESSOR_INTEL_386:
                    sprops.cpu_isalist = "i386";
                    break;
                case PROCESSOR_INTEL_486:
                    sprops.cpu_isalist = "i486 i386";
                    break;
                case PROCESSOR_INTEL_PENTIUM:
                    sprops.cpu_isalist = "pentium i486 i386";
                    break;
            }
        }
    }

    /*
     * User name
     * We try to avoid calling GetUserName as it turns out to
     * be surprisingly expensive on NT.  It pulls in an extra
     * 100 K of footprint.
     */
    {
        char *uname = getenv("USERNAME");
        if (uname != NULL && strlen(uname) > 0) {
	    sprops.user_name = strdup(uname);
        } else {
            char buf[100];
            int buflen = sizeof(buf);
            sprops.user_name =
                GetUserName(buf, &buflen) ? strdup(buf) : "unknown";
	}
    }

    /*
     * Home directory/
     *
     * We first look under a standard registry key.  If that fails we
     * fall back on using a SHELL32.DLL API.  If that fails we use a
     * default value. 
     *
     * Note: To save space we want to avoid loading SHELL32.DLL
     * unless really necessary.  However if we do load it, we leave it
     * in memory, as it may be needed again later.
     *
     * The normal result is that for a given user name XXX:
     *     On multi-user NT, user.home gets set to c:\winnt\profiles\XXX.
     *     On multi-user Win95, user.home gets set to c:\windows\profiles\XXX.
     *     On single-user Win95, user.home gets set to c:\windows.
     */
    {
	char *homep = getHomeFromRegistry();
	if (homep == NULL) {
	    homep = getHomeFromShell32();
	    if (homep == NULL) {
                homep = "C:\\";
	    }
        }
        sprops.user_home = homep;
    }

    /*
     *  user.language
     *  user.region (if user's environmant specify this)
     *  file.encoding
     *  file.encoding.pkg
     */
    {
        /*
         * query the system for the current system default locale
         * (which is a Windows LCID value)
         */
        LCID systemLocale = GetThreadLocale();

        /*
         * binary-search our list of LCID values.  If we don't find the 
	 * one we're looking for, mask out the country code and try again 
	 * with just the language code
         */
        {
            int index = -1;
            int tries = 0;
            char* localeID;
            do {
                int lo, hi, mid;
                lo = 0;
                hi = sizeof(localeIDMap) / sizeof(LCIDtoLocale);
                while (index == -1 && lo < hi) {
                    mid = (lo + hi) / 2;
                    if (localeIDMap[mid].winID == systemLocale)
                        index = mid;
                    else if (localeIDMap[mid].winID > systemLocale)
                        hi = mid;
                    else
                        lo = mid + 1;
                }
                systemLocale &= 0x03ff;
                ++tries;
            } while (index == -1 && tries < 2);

            /*
             * if we didn't find the LCID that the system returned to us,
	     * we don't have a Java locale ID that corresponds to it. 
	     * Fall back on en_US.
             */
            if (index == -1) {
                sprops.language = "en";
                sprops.region = "US";
            }

            /* otherwise, look up the corresponding Java locale ID from the list
             * of Java locale IDs and set up the system properties accordingly.
             * [Two notes: Some of the Java IDs we look up may contain not only
             * a language and country, but also a variant.  If this is the case,
             * we leave the variant with the country (i.e., country_variant) and
             * store it in user.region.  This works because code has been added
             * to Locale for euro support to recognize a user.region value of
             * the form country_variant or _variant during initialization of the
             * default locale from system properties.  We don't use a new
             * property ("user.variant") because we don't want to make the VM to
             * Java property-based communication pipe any wider than it is
             * already.  Second, we are indeed stomping over the actual static
             * tables here to split the locale ID into language and country
             * codes.  Since this function is the only one that uses that table,
             * and it's only called once at system init time, this should be
             * okay, but if either assumption changes, we'll have to re-think
             * this.]  --rtg 5/4/98 --liu 7/27/98 */
            else {
                char* lang;
                char* ctry;

                localeID = localeIDMap[index].javaID;
                lang = localeID;
                ctry = lang;

                sprops.language = lang;
                while (*ctry != '_' && *ctry != 0)
                    ++ctry;

                if (*ctry == '_') {
                    *ctry++ = 0;
                }
                sprops.language = lang;
                sprops.region = ctry;
            }
        }

        {
            char *fallback_encoding = NULL;
            char locale_name[128];
            int i;
            if (*sprops.region != 0)
                sprintf(locale_name, "%s_%s", sprops.language, sprops.region);
            else
                strcpy(locale_name, sprops.language);

            for (i = 0; strcmp(encoding_names[i], ""); i += 2) {
                if (!strcmp(encoding_names[i], locale_name)) {
                    sprops.encoding = encoding_names[i + 1];
                    break;
                }
                else if (fallback_encoding == NULL &&
                         !strcmp(encoding_names[i], sprops.language)) {
                    fallback_encoding = encoding_names[i + 1];
                }
            }

            if (sprops.encoding == NULL)
                sprops.encoding = fallback_encoding;

	    /* If we can't deduce an encoding, fall back to Cp1252 */
            if (sprops.encoding == NULL)
                sprops.encoding = "Cp1252";
        }
    }

    sprops.unicode_encoding = "UnicodeLittle";
    /* User TIMEZONE */
    {
	/*
	 * We defer setting up timezone until it's actually necessary.
	 * Refer to TimeZone.getDefault(). However, the system
	 * property is necessary to be able to be set by the command
	 * line interface -D. Here temporarily set a null string to
	 * timezone.
	 */
	sprops.timezone = "";
    }

    /* Current directory */
    {
        char buf[MAX_PATH];
        GetCurrentDirectory(sizeof(buf), buf);
        sprops.user_dir = strdup(buf);
    }

    sprops.file_separator = "\\";
    sprops.path_separator = ";";
    sprops.line_separator = "\r\n";

    return &sprops;
}
