/*
 * @(#)SocketInputStream.c	1.24 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include <winsock.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>

#include "java_net_PlainSocketImpl.h"
#include "java_net_SocketImpl.h"
#include "java_net_SocketInputStream.h"
#include "java_io_FileDescriptor.h"

#include "net_util.h"
#include "jni_util.h"

/*************************************************************************
 * SocketInputStream
 */

static jfieldID IO_fd_fdID;

jfieldID sis_fdID;
jfieldID sis_implID;

/*
 * Class:     java_net_SocketInputStream
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL 
Java_java_net_SocketInputStream_init(JNIEnv *env, jclass cls) {
    sis_fdID = (*env)->GetFieldID(env, cls, "fd", "Ljava/io/FileDescriptor;");
    sis_implID = (*env)->GetFieldID(env, cls, "impl", "Ljava/net/SocketImpl;");
    IO_fd_fdID = NET_GetFileDescriptorID(env);
}

/*
 * Class:     java_net_SocketInputStream
 * Method:    socketRead
 * Signature: ([BII)I
 */
JNIEXPORT jint JNICALL
Java_java_net_SocketInputStream_socketRead(JNIEnv *env, jobject this, 
					   jbyteArray data, jint off, jint len){
    char *bufP;
    char BUF[MAX_BUFFER_LEN];

    jobject fdObj = (*env)->GetObjectField(env, this, sis_fdID);
    jint fd, timeout;

    jobject impl = (*env)->GetObjectField(env, this, sis_implID);

    jint datalen, nread;

    if (IS_NULL(fdObj)) {
        JNU_ThrowByName(env, JNU_JAVANETPKG "SocketException", "socket closed");
	return -1;
    } else {
        fd = (*env)->GetIntField(env, fdObj, IO_fd_fdID); 
    }
    if (fd == -1) {
	NET_ThrowSocketException(env, "Socket closed");
	return -1;
    }
    if (IS_NULL(data)) {
	JNU_ThrowNullPointerException(env, "data argument");
        return -1;
    }
    if (IS_NULL(impl)) {
	JNU_ThrowNullPointerException(env, "socket impl");
	return -1;
    } else {
	timeout = (*env)->GetIntField(env, impl, psi_timeoutID);
    }
    datalen = (*env)->GetArrayLength(env, data);
    
    if (len < 0 || len + off > datalen) {
        JNU_ThrowByName(env, JNU_JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return 0;
    } 
    
    if (len == 0) {
	return 0;
    }
    
    /*
     * If the caller buffer is large than our stack buffer then we allocate
     * from the heap (up to a limit). If memory is exhausted we always use
     * the stack buffer.
     */
    if (len <= MAX_BUFFER_LEN) {
	bufP = BUF;
    } else {
	if (len > MAX_HEAP_BUFFER_LEN) {
	    len = MAX_HEAP_BUFFER_LEN;
	}
	bufP = (char *)malloc((size_t)len);
	if (bufP == NULL) {
	    /* allocation failed so use stack buffer */
	    bufP = BUF;
	    len = MAX_BUFFER_LEN;
	}
    }

    if (timeout) {
        int ret = JVM_Timeout(fd, timeout);
	if (ret <= 0) {
            if (ret == 0) {
 	        JNU_ThrowByName(env, JNU_JAVAIOPKG "InterruptedIOException",
			        "Read timed out");
	    } else if (ret == JVM_IO_ERR) {
	        JNU_ThrowByName(env, JNU_JAVANETPKG "SocketException", "socket closed");
	    } else if (ret == JVM_IO_INTR) {
	        JNU_ThrowByName(env, JNU_JAVAIOPKG "InterruptedIOException",
			        "Operation interrupted");
	    }
	    if (bufP != BUF) {
		free(bufP);
	    }
	    return -1;
	}
    }
    nread = JVM_Recv(fd, bufP, len, 0);
    if (nread > 0) {
	(*env)->SetByteArrayRegion(env, data, off, nread, (jbyte *)bufP);
    } else {
	if (nread < 0) {
	    NET_ThrowCurrent(env, "JVM_recv in socket input stream read");
	}
    }
    if (bufP != BUF) {
	free(bufP);
    }
    return nread;
}
