/*
 * @(#)SocketOutputStream.c	1.20 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include <winsock.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>

#include "java_net_SocketOutputStream.h"

#include "net_util.h"
#include "jni_util.h"

/************************************************************************
 * SocketOutputStream
 */
static jfieldID IO_fd_fdID;

jfieldID sos_fdID;

/*
 * Class:     java_net_SocketOutputStream
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_java_net_SocketOutputStream_init(JNIEnv *env, jclass cls) {

    sos_fdID = (*env)->GetFieldID(env, cls, "fd", "Ljava/io/FileDescriptor;");
    IO_fd_fdID = NET_GetFileDescriptorID(env);
}

/*
 * Class:     java_net_SocketOutputStream
 * Method:    socketWrite
 * Signature: ([BII)V
 */
JNIEXPORT void JNICALL
Java_java_net_SocketOutputStream_socketWrite(JNIEnv *env, jobject this, 
					     jbyteArray data,
					     jint off, jint len) {
    char *bufP;
    char BUF[MAX_BUFFER_LEN];
    int buflen;

    /* The fd field */
    jobject fdObj = (*env)->GetObjectField(env, this, sos_fdID);
    int fd;

    jint datalen, n;

    if (IS_NULL(fdObj)) {
	JNU_ThrowByName(env, JNU_JAVANETPKG "SocketException", "Socket closed");
	return;
    } else {
	fd = (*env)->GetIntField(env, fdObj, IO_fd_fdID);
    }
    if (IS_NULL(data)) {
	JNU_ThrowNullPointerException(env, "data argument");
	return;
    }

    datalen = (*env)->GetArrayLength(env, data);

    if ((len < 0) || (off < 0) || (len + off > datalen)) {
        JNU_ThrowByName(env, JNU_JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return;
    }

    /*
     * Use stack allocate buffer if possible. For large sizes we allocate
     * an intermediate buffer from the heap (up to a maximum). If heap is
     * unavailable just use our stack buffer.
     */
    if (len <= MAX_BUFFER_LEN) {
	bufP = BUF;
	buflen = MAX_BUFFER_LEN;
    } else {
	buflen = min(MAX_HEAP_BUFFER_LEN, len);
	bufP = (char *)malloc((size_t)buflen);
	if (bufP == NULL) {
	    bufP = BUF;
	    buflen = MAX_BUFFER_LEN;
	}
    }

    while(len > 0) {
	int loff = 0;
	int chunkLen = min(buflen, len);
	int llen = chunkLen;
	(*env)->GetByteArrayRegion(env, data, off, chunkLen, (jbyte *)bufP);
      
	while(llen > 0) {
	    int n = JVM_Send(fd, bufP + loff, llen, 0);
	    if (n > 0) {
		llen -= n;
		loff += n;
		continue;
	    }
	    NET_ThrowCurrent(env, "socket write error");
	    if (bufP != BUF) {
		free(bufP);
	    }
	    return;
	}
	len -= chunkLen;
	off += chunkLen;
    }

    if (bufP != BUF) {
	free(bufP);
    }
}







