/*
 * @(#)TimeZone_md.c	1.8 00/04/06
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "TimeZone_md.h"

#define MAX_ZONE_CHAR 64
#define VALUE_UNKNOWN 0
#define VALUE_KEY 1
#define VALUE_MAPID 2

#define NT_TZ_KEY "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones"
#define WIN_TZ_KEY "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Time Zones"
#define MAPPINGS_FILE "\\lib\\tzmappings"

/*
 * code to figure out current timezone setting on windows by registry.
 */
static int getWinTimeZone(char *winZoneName, char *winMapID) 
{
    TIME_ZONE_INFORMATION tzi;
    DWORD gtzi;
    int iRet, onlyMapID;
    HANDLE hKey, hSubKey = NULL;
    LONG ret;
    DWORD dwSubKeys, i;
    ULONG ulType;
    TCHAR szStdName[MAX_ZONE_CHAR];
    TCHAR szName[MAX_ZONE_CHAR];
    TCHAR szValue[MAX_ZONE_CHAR];
    DWORD dwStdName = sizeof(szStdName);
    DWORD dwName = sizeof(szName);
    DWORD dwValue = sizeof(szValue);

    // Get current selected time zone name. This returns current control
    // panel, Date & Time, setting. And convert string from WC to MBS.
    gtzi = GetTimeZoneInformation(&tzi);
    if (gtzi == -1) {
        return VALUE_UNKNOWN;
    }
    iRet = WideCharToMultiByte(CP_ACP, 0, tzi.StandardName, -1,
                               szStdName, dwStdName, NULL, NULL);
    if (iRet == 0) {
        return VALUE_UNKNOWN;
    }

    // First try to see this is NT Machine. Then, 95/98 keys
    ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, NT_TZ_KEY, 0, KEY_READ, (PHKEY)&hKey);
    if (ret != ERROR_SUCCESS) {
        ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, WIN_TZ_KEY, 0, KEY_READ, (PHKEY)&hKey);
        // Failed here means, cannot access both 95/98, NT key. Nothing to do.
        if (ret != ERROR_SUCCESS) {
            return VALUE_UNKNOWN;
        }
    }

    // Count up sub keys, actual zone info, to enumerate
    ret = RegQueryInfoKey(hKey, NULL, NULL, NULL, &dwSubKeys,
                          NULL, NULL, NULL, NULL, NULL, NULL, NULL);
    if (ret != ERROR_SUCCESS) {
        RegCloseKey(hKey);
        return VALUE_UNKNOWN;
    }

    // Looking into "Std" value of each sub keys and find match to
    // current control panel setting.
    onlyMapID = 0;
    for(i = 0; i < dwSubKeys; ++i) {
        dwName = sizeof(szName);
        dwValue = sizeof(szValue);
        ret = RegEnumKeyEx(hKey, i, szName, &dwName, NULL, NULL, NULL, NULL);
        if (ret != ERROR_SUCCESS) {
            RegCloseKey(hKey);
            return VALUE_UNKNOWN;
        }
        ret = RegOpenKeyEx(hKey, szName, 0, KEY_READ, (PHKEY)&hSubKey);
        if (ret != ERROR_SUCCESS) {
            RegCloseKey(hKey);
            return VALUE_UNKNOWN;
        }
        ret = RegQueryValueEx(hSubKey, "Std", NULL, &ulType,
                              szValue, &dwValue);
        if (ret != ERROR_SUCCESS) {
            // NT4SP3 will fail here. Since they don't have "Std" key in 
            // time zone info.
            hSubKey = NULL;
            onlyMapID = 1;
            strcpy(winZoneName, szStdName);
            break;
        }
        if (strcmp(szValue, szStdName) == 0) {
            // found matched record, terminate search
            strcpy(winZoneName, szName);
            break;
        }
        RegCloseKey(hSubKey);
    }

    // We also need "MapID" value to eliminate duplicate key name.
    if (hSubKey == NULL) {
        ret = RegOpenKeyEx(hKey, szStdName, 0, KEY_READ, (PHKEY)&hSubKey);
        if (ret != ERROR_SUCCESS) {
            RegCloseKey(hKey);
            return VALUE_UNKNOWN;
        }
    }
    dwValue = sizeof(szValue);
    ret = RegQueryValueEx(hSubKey, "MapID", NULL, &ulType, szValue, &dwValue);
    RegCloseKey(hKey);
    RegCloseKey(hSubKey);
    if (ret == ERROR_SUCCESS) {
        strcpy(winMapID, szValue);
    } else {
        return VALUE_UNKNOWN;
    }

    return (onlyMapID == 1) ? VALUE_MAPID : VALUE_KEY;
}

#define TZ_WIN_NAME 0
#define TZ_MAPID 1
#define TZ_REGION 2
#define TZ_JAVA_NAME 3
#define TZ_ITEMS 4

/*
 * Matching win timezone name in tzName to Java TimeZone ID.
 * The key for mathing is diffent by value_type.
 * VALUE_KEY: exact matching of key
 * VALUE_MAPID: find all matched Map ID and determine by region
 */
static char *matchJavaTZ(const char *java_home_dir, int value_type, char *tzName,
                         char *mapID, char *javaTZName, const char *region) 
{
    char mapFileName[_MAX_PATH + 1];
    FILE *fp;
    char lineBuff[MAX_ZONE_CHAR * 4];
    char *itemBuff[4];
    char defBuff[MAX_ZONE_CHAR];
    int found = 0;
    int line;
    int IDmatched = 0;

    defBuff[0] = '\0';

    strcpy(mapFileName, java_home_dir);
    strcat(mapFileName, MAPPINGS_FILE);
    if ((fp = fopen(mapFileName, "r")) == NULL) {
        jio_fprintf(stderr, "can't open %s.\n", mapFileName);
        return NULL;
    }

    line = 0;
    while((fgets(lineBuff, sizeof(lineBuff), fp) != NULL) && !found) {
        char *start, *idx;
        int item = 0;

        line++;
        start = idx = lineBuff;
        if (*idx == '#' || *idx == '\n')
            continue;

        while((*idx != '\n') && (*idx != '\0')) {
            if (*idx == ':') {
                *idx = '\0';
                itemBuff[item++] = start;
                ++idx;
                start = idx;
            } else {
                ++idx;
            }
        }

        if (item < TZ_ITEMS 
            || (strlen(itemBuff[TZ_JAVA_NAME]) > MAX_ZONE_CHAR)) { 
            // Line does not have complete item. Should be broken file.
            jio_fprintf(stderr, "tzmappings: Illegal format at line %d.\n", line);
            break;
        }

        if (strcmp(mapID, itemBuff[TZ_MAPID]) == 0) {
            IDmatched = 1;
            if (strcmp(itemBuff[TZ_WIN_NAME], tzName) == 0) {
                // if WIN_NAME is matched. This is the one.
                strcpy (javaTZName, itemBuff[TZ_JAVA_NAME]);
                found = 1;
            } else {
                // if MAPID is matched, find best fit region.
                if (strcmp(itemBuff[TZ_REGION], "NULL") == 0) {
                    strcpy(defBuff, itemBuff[TZ_JAVA_NAME]);
                }
                if (strcmp(itemBuff[TZ_REGION], region) == 0) {
                    strcpy(javaTZName, itemBuff[TZ_JAVA_NAME]);
		    if (value_type == VALUE_MAPID) {
		      found = 1;
		    }
                }
            }
        } else if (IDmatched == 1) {
            // we found, matched MapID, but couldn't get name matched.
            break;
        }
    }

    fclose(fp);
    if ((defBuff[0] != '\0') && !found) {
        strcpy(javaTZName, defBuff);
        found = 1;
    }
    return ((found) ? javaTZName : NULL);
}

/*
 * Gets the platform defined TimeZone ID. Quering win32 registry
 * and found matching from "tzmappings" file.
 */
char *findJavaTZ_md(const char *java_home_dir, const char *region)
{
    char winZoneName[MAX_ZONE_CHAR];
    char winMapID[10];
    char *tmpBuff;
    char *std_timezone = NULL;

    int result = getWinTimeZone(winZoneName, winMapID);
    if (result != VALUE_UNKNOWN) {
        tmpBuff = malloc(MAX_ZONE_CHAR);
        if (tmpBuff == NULL) {
            return NULL;
        }
        std_timezone = matchJavaTZ(java_home_dir, result, winZoneName, winMapID,
                                   tmpBuff, region);
        if (std_timezone == NULL) {
            free(tmpBuff);
        }
    }

    return std_timezone;
}
