/*
 * @(#)TextRenderingGDI.cpp	1.7 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "img_util_md.h"
#include <stdio.h>
#include <FontGlue.h>
#include <GlyphVector.h>
#include <assert.h>

#ifdef DEBUG
#define VERIFY(exp) assert(exp)
#else
#define VERIFY(exp) (exp)
#endif

#define OPAQUE_IMAGEDATA_ARGS(SRC_TYPE) \
	(JNIEnv *env, jobject joSelf, SRC_TYPE theString, \
	jfloat x, jfloat y, jobject theFont, jdoubleArray matrix, \
	jint fgColor, jint cx, jint cy, jint cw, jint ch, \
	jobject imageData)

#define OPAQUE_IMAGEDATA_ARRAY_ARGS(SRC_TYPE) \
	(JNIEnv *env, jobject joSelf, SRC_TYPE theArray, \
	jint offsetSrc, jint count, jfloat x, jfloat y, \
	jobject theFont, jdoubleArray matrix, jint fgColor, \
	jint cx, jint cy, jint cw, jint ch, jobject imageData)

#define OPAQUE_GLYPH_IMAGEDATA_ARGS(SRC_TYPE) \
	(JNIEnv *env, jobject joSelf, SRC_TYPE theString, \
	jfloatArray positions, jfloat x, jfloat y, \
	jobject theFont, jdoubleArray matrix, jdoubleArray devTX, \
	jint fgColor, jint cx, jint cy, jint cw, jint ch, \
	jobject imageData)

#define IMAGEDATA_REF(X, Y, MATRIX, AAHINT, FMHINT) \
	gv.positionGlyphsWithImages(X, Y, MATRIX, AAHINT, FMHINT); \

#define GLYPH_IMAGEDATA_REF(X, Y, MATRIX, AAHINT, FMHINT) \
	gv.setDevTX(devTX); \
	gv.setPositions(positions); \
	IMAGEDATA_REF(X, Y, MATRIX, AAHINT, FMHINT);

#define OPAQUE_TEXT_RENDER_IMAGEDATA(SRC_TYPE, ARGS, IMAGE_REF) \
    ARGS(SRC_TYPE) { \
	fontObject *fo = getFontPtr(env, theFont); \
	if (fo) { \
	    GlyphVector gv(env, theString, matrix, false, false, fo); \
	    if (gv.needShaping()) { \
		JNU_ThrowByName(env, "sun/awt/font/ShapingException", \
			"Shaping may be needed"); \
		return; \
	    } \
	    IMAGE_REF(x, y, matrix, false, false); \
	    TextRenderingGDI(env, imageData, gv, cx, cy, cw, ch, fgColor); \
	} \
    }

#define OPAQUE_TEXT_ARRAY_RENDER_IMAGEDATA(SRC_TYPE, ARGS, IMAGE_REF) \
    ARGS(SRC_TYPE) { \
	fontObject *fo = getFontPtr(env, theFont); \
	if (fo) { \
	    GlyphVector gv(env, theArray, offsetSrc, count, \
		matrix, false, false, fo); \
	    if (gv.needShaping()) { \
		JNU_ThrowByName(env, "sun/awt/font/ShapingException", \
			"Shaping may be needed"); \
		return; \
	    } \
	    IMAGE_REF(x, y, matrix, false, false); \
	    TextRenderingGDI (env, imageData, gv, cx, cy, cw, ch, fgColor); \
	} \
    }

#define OPAQUE_TEXT_TO_IMAGEDATA(SRC_TYPE) \
    OPAQUE_TEXT_RENDER_IMAGEDATA(SRC_TYPE, OPAQUE_IMAGEDATA_ARGS, \
		IMAGEDATA_REF)

#define OPAQUE_TEXT_ARRAY_TO_IMAGEDATA(SRC_TYPE) \
    OPAQUE_TEXT_ARRAY_RENDER_IMAGEDATA(SRC_TYPE, OPAQUE_IMAGEDATA_ARRAY_ARGS, \
		IMAGEDATA_REF)

#define OPAQUE_GLYPHS_TO_IMAGEDATA(SRC_TYPE) \
	 OPAQUE_TEXT_RENDER_IMAGEDATA(SRC_TYPE, OPAQUE_GLYPH_IMAGEDATA_ARGS, \
		GLYPH_IMAGEDATA_REF)

void TextRenderingGDI (JNIEnv * env, jobject imageData, GlyphVector& gv,
		jfloat cx, jfloat cy, jfloat cw, jfloat ch, jint colorData)
{ 
    int originX, originY;
    double ratio;
    getOriginFromImageData(env, imageData, &originX, &originY);
    ratio = getResRatioFromImageData(env, imageData);

    HDC hdc = *((HDC *)getDeviceInfoFromImageData(env, imageData));

    HBRUSH cbr;
    VERIFY(cbr = ::CreateSolidBrush(PALETTERGB((colorData>>16)&0xff,
			(colorData>>8)&0xff, (colorData)&0xff)));
    RECT theRect;

    SIZE s;
    VERIFY(::GetWindowExtEx(hdc, &s));
    int newResolution = s.cx * ratio;

    // recalculate the clip region
    int clipLeft    = (int)(cx * ratio - originX * (ratio -1) + 0.5);
    int clipRight   = (int)((cx + cw) * ratio - originX * (ratio -1) + 0.5);
    int clipTop     = (int)(cy * ratio - originY * (ratio -1) + 0.5);
    int clipBottom  = (int)((cy + ch) * ratio - originY * (ratio -1) + 0.5);

    // transform window origin and extent
    int restore;
    VERIFY(restore = ::SaveDC(hdc));
    VERIFY(::OffsetWindowOrgEx(hdc, (int)(originX * (1 - ratio) + 0.5),
			 (int)(originY * (1 - ratio) + 0.5), NULL));
    VERIFY(::SetWindowExtEx(hdc, newResolution, newResolution, NULL));

    int totalGlyphs = gv.getNumGlyphs();
    for (int glyphCounter = 0; glyphCounter < totalGlyphs; glyphCounter++) {
	float xPos, yPos;
	const void *pixelPtr;
	const UInt8 * pixels;
	unsigned int rowBytes;
	int width, height;
	gv.getGlyphPosition(glyphCounter, xPos, yPos);
	gv.getGlyphBlitInfo(glyphCounter, pixelPtr, rowBytes, width, height);
	pixels = (const UInt8 *)pixelPtr;
	if (!pixels)
	    continue;

	/* if any clipping required, modify parameters now */
	int left   = (int)xPos;
	int right  = left + width;
	int top    = (int)yPos;
	int bottom = top + height;
	if (left < clipLeft) {
	    pixels += clipLeft - left;
	     left = clipLeft;
	}
	if (top < clipTop) {
	    pixels += (clipTop - top) * rowBytes;
	    top = clipTop;
	}
	if (right > clipRight) {
	    right = clipRight;
	}
	if (bottom > clipBottom) {
	    bottom = clipBottom;
	}
	width = right - left;
	height = bottom - top;

	if ((width <= 0) || (height <= 0)) {
	    continue;
	}

	while (height--) {
	    theRect.top = top++;
	    theRect.bottom = top;
	    theRect.left = theRect.right = 0;
	    for (int i=0; i < width; i++) {
		if (pixels[i]) {
		    if (theRect.left == theRect.right) {
			theRect.left = left + i;
			theRect.right = theRect.left + 1;
		    } else {
			theRect.right++;
		    }
		} else if (theRect.left != theRect.right) {
		    VERIFY(::FillRect(hdc, &theRect, cbr));
		    theRect.left = theRect.right = 0;
		}
	    }
	    if (theRect.left != theRect.right) {
		VERIFY(::FillRect(hdc, &theRect, cbr));
	    }
	    pixels += rowBytes;
	}
    }

    VERIFY(::RestoreDC(hdc, restore));
    VERIFY(::DeleteObject(cbr));
}

extern "C" {

/*
 * Class:     sun_awt_windows_DrawCharsGDI
 * Method:    drawCharsGDI
 * Signature: ([CIIFFLjava/awt/Font;[FIIIIILsun/java2d/loops/ImageData;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_DrawCharsGDI_drawCharsGDI
    OPAQUE_TEXT_ARRAY_TO_IMAGEDATA(jcharArray)

/*
 * Class:     sun_awt_windows_DrawStringGDI
 * Method:    drawStringGDI
 * Signature: (Ljava/lang/String;FFLjava/awt/Font;[FIIIIILsun/java2d/loops/ImageData;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_DrawStringGDI_drawStringGDI
    OPAQUE_TEXT_TO_IMAGEDATA(jstring)

/*
 * Class:     sun_awt_windows_DrawBytesGDI
 * Method:    drawBytesGDI
 * Signature: ([BIIFFLjava/awt/Font;[FIIIIILsun/java2d/loops/ImageData;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_DrawBytesGDI_drawBytesGDI
    OPAQUE_TEXT_ARRAY_TO_IMAGEDATA(jbyteArray)

/*
 * Class:     sun_awt_windows_DrawGlyphVectorGDI
 * Method:    drawGlyphVectorGDI
 * Signature: ([I[FFFLjava/awt/Font;[FIIIIILsun/java2d/loops/ImageData;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_DrawGlyphVectorGDI_drawGlyphVectorGDI
OPAQUE_GLYPHS_TO_IMAGEDATA(jintArray)

} // end of extern "C"
