/*
 * @(#)awt_AWTEvent.cpp	1.9 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_AWTEvent.h"
#include "awt.h"

/************************************************************************
 * AwtAWTEvent fields
 */

jfieldID AwtAWTEvent::dataID;
jfieldID AwtAWTEvent::idID;
jfieldID AwtAWTEvent::consumedID;

/************************************************************************
 * AwtEvent native methods
 */

extern "C" {

JNIEXPORT void JNICALL 
Java_java_awt_AWTEvent_initIDs(JNIEnv *env, jclass cls) {
    TRY;

    AwtAWTEvent::dataID = env->GetFieldID(cls, "data", "J");
    AwtAWTEvent::idID = env->GetFieldID(cls, "id", "I");
    AwtAWTEvent::consumedID = env->GetFieldID(cls, "consumed", "Z");

    DASSERT(AwtAWTEvent::dataID != NULL);
    DASSERT(AwtAWTEvent::idID != NULL);
    DASSERT(AwtAWTEvent::consumedID != NULL);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     java_awt_AWTEvent
 * Method:    freeNativeData
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_java_awt_AWTEvent_freeNativeData(JNIEnv *env, jobject event)
{
    TRY_NO_VERIFY;

    JNI_CHECK_NULL_RETURN(event, "null AWTEvent");

    jlong data = (env)->GetLongField(event, AwtAWTEvent::dataID);
    if (data != 0) {
	MSG* pMsg = (MSG*)data;
	(env)->SetLongField(event, AwtAWTEvent::dataID, 0);
	delete pMsg;
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     java_awt_AWTEvent
 * Method:    copyDataFieldInto
 * Signature: (Ljava/awt/AWTEvent;)V
 *
 * Allocates a copy of the data instance variable and copies it
 * from thisEvent into thatEvent. If data is null, thatEvent's
 * data field is also set to NULL.
 */
JNIEXPORT void JNICALL
Java_java_awt_AWTEvent_copyDataFieldInto
  (JNIEnv *env, jobject thisEvent, jobject thatEvent)
{
    TRY;

    JNI_CHECK_NULL_RETURN(thatEvent, "null AWTEvent");

    MSG *pThisData = (MSG *)(env)->GetLongField(thisEvent, AwtAWTEvent::dataID);
    MSG *pThatData = (MSG *)(env)->GetLongField(thatEvent, AwtAWTEvent::dataID);
    if ((pThisData == NULL) && (pThatData == NULL))
	return; // They're both NULL; we're done
    if (pThatData != NULL) {
	// pThatData points to a MSG struct that should be deleted
	delete pThatData;
	pThatData = NULL;
    }
    if (pThisData != NULL) {
	// Allocate a new MSG struct and copy it into pThatData
        try {
	    pThatData = new MSG();
	} catch (std::bad_alloc&) {
	    env->SetLongField(thatEvent, AwtAWTEvent::dataID, NULL);
	    throw;
	}

	memcpy(pThatData, pThisData, sizeof(MSG));
    }
    (env)->SetLongField(thatEvent, AwtAWTEvent::dataID, (jlong)pThatData);

    CATCH_BAD_ALLOC;
} /* AWTEvent_copyDataFieldInto() */

} /* extern "C" */
