/*
 * @(#)awt_ActiveEvent.cpp	1.10 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_ActiveEvent.h"
#include "awt_Object.h"

#include <sun_awt_windows_WActiveEvent.h>

CriticalSection AwtActiveEvent::sm_lock;


void AwtActiveEvent::SendEvent(AwtObject *poster)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    CriticalSection::Lock l(sm_lock);
    if (env->EnsureLocalCapacity(2) < 0) {
        return;
    }

    static jclass wActiveEventCls;
    if (wActiveEventCls == NULL) {
        jclass wActiveEventClsLocal =
	    env->FindClass("sun/awt/windows/WActiveEvent");
	if (!wActiveEventClsLocal) {
	    /* exception already thrown */
	    return;
	}
	wActiveEventCls = (jclass)env->NewGlobalRef(wActiveEventClsLocal);
	env->DeleteLocalRef(wActiveEventClsLocal);
    }

    static jmethodID wActiveEventConst;
    if (wActiveEventConst == NULL) {
        wActiveEventConst = 
            env->GetMethodID(wActiveEventCls, "<init>",
		     "(Ljava/lang/Object;Lsun/awt/windows/WObjectPeer;J)V");
	DASSERT(wActiveEventConst);
    }

    jobject target = poster->GetTarget(env);
    jobject wActiveEvent = env->NewObject(wActiveEventCls, wActiveEventConst,
					  target, poster->GetPeer(env),
					  (jlong) Clone());
    DASSERT(!safe_ExceptionOccurred(env));
    DASSERT(wActiveEvent != NULL);

    poster->SendEvent(wActiveEvent);
    env->DeleteLocalRef(target);
    env->DeleteLocalRef(wActiveEvent);
}

/************************************************************************
 * WActiveEvent native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WActiveEvent
 * Method:    dispatchImpl
 * Signature: (J)V
 */

JNIEXPORT void JNICALL
Java_sun_awt_windows_WActiveEvent_dispatchImpl(JNIEnv *env, jobject self,
					       jlong pData)
{
    TRY;

    DASSERT(!::IsBadReadPtr( (const void *) pData, sizeof(AwtActiveEvent)));
    ((AwtActiveEvent *) pData)->Dispatch();

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WActiveEvent
 * Method:    finalizeImpl
 * Signature: (J)V
 */

JNIEXPORT void JNICALL
Java_sun_awt_windows_WActiveEvent_finalizeImpl(JNIEnv *env, jobject self,
					       jlong pData)
{
    TRY_NO_VERIFY;

    DASSERT(!::IsBadReadPtr( (const void *) pData, sizeof(AwtActiveEvent)));
    delete ((AwtActiveEvent *) pData);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */
