/*
 * @(#)awt_ActiveEvent.h	1.5 00/02/02
 *
 * Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef AWT_ACTIVE_EVENT_H
#define AWT_ACTIVE_EVENT_H

#include "awt_Toolkit.h"
class AwtObject;

/************************************************************************
 * AwtActiveEvent class
 * This class is used to schedule native code which should be run
 * synchronized on the EventQueue.
 *
 * To use this class:
 *   * Declare and define a public subclass of AwtActiveEvent which
 *     defines Clone(void) and Dispatch(void). Clone(void) should
 *     allocate storage with 'new'. You may want to make this class
 *     a C++ equivalent of a Java inner class. See
 *     AwtFrame::AwtNativeActivationEvent for an example.
 *   * Instantiate an instance of your subclass. (You are responsible
 *     for deleting this instance.)
 *   * Call SendEvent(AwtObject *) on your instance. Pass as a parameter
 *     the AwtObject whose Java peer you want used to post the event to
 *     the AWT EventQueue. SendEvent will create a copy of the event
 *     instance using the Clone(void) method you defined (you are _not_
 *     responsible for deleting this copy) and a WActiveEvent Java object
 *     which will manage it. The WActiveEvent will then be posted to the
 *     EventQueue.
 *   * When the Event Dispatch Thread calls WActiveEvent.dispatch(),
 *     your Dispatch(void) method will be invoked.
 *   * The AwtActiveEvent copy is deleted during finalization of the
 *     WActiveEvent instance.
 * You can call SendEvent(AwtObject *) multiple times on the same
 * object without a problem.
 *
 * -- david.mendenhall@eng 16Apr1998
 */

class AwtActiveEvent {

public:
    AwtActiveEvent() {}
    virtual ~AwtActiveEvent() {}

    virtual AwtActiveEvent *Clone() = 0;
    virtual void Dispatch() = 0;
    virtual void SendEvent(AwtObject *poster);

private:
    static CriticalSection sm_lock;
};


#endif
