/*
 * @(#)awt_Canvas.cpp	1.62 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_Toolkit.h"
#include "awt_Canvas.h"
#include "awt_image.h"
#include "awt_Graphics.h"
#include "awt_Win32GraphicsConfig.h"

/* IMPORTANT! Read the README.JNI file for notes on JNI converted AWT code.
 */

/************************************************************************
 * AwtCanvas methods
 */

AwtCanvas::AwtCanvas() {
    m_eraseBackground = FALSE;
}

AwtCanvas::~AwtCanvas() {
}

const char* AwtCanvas::GetClassName() {
  return "SunAwtCanvas";
}

/*
 * Create a new AwtCanvas object and window.
 */
AwtCanvas* AwtCanvas::Create(jobject self, jobject hParent)
{
    TRY;
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    jobject target = NULL;
    AwtCanvas *canvas = NULL;

    try {
	if (env->EnsureLocalCapacity(1) < 0) {
	    return NULL;
	}

	AwtComponent* parent;

	JNI_CHECK_NULL_GOTO(hParent, "null hParent", done);

	parent = (AwtComponent*)
	    (env->GetIntField(hParent, AwtObject::pDataID));
	JNI_CHECK_NULL_GOTO(parent, "null parent", done);

	target = env->GetObjectField(self, AwtObject::targetID);
	JNI_CHECK_NULL_GOTO(target, "null target", done);

	canvas = new AwtCanvas();

	{
	    jint x = env->GetIntField(target, AwtComponent::xID);
	    jint y = env->GetIntField(target, AwtComponent::yID);
	    jint width = env->GetIntField(target, AwtComponent::widthID);
	    jint height = env->GetIntField(target, AwtComponent::heightID);

	    canvas->CreateHWnd(env, "",
			       WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, 0,
			       x, y, width, height,
			       parent->GetHWnd(),
			       NULL,
			       ::GetSysColor(COLOR_WINDOWTEXT),
			       ::GetSysColor(COLOR_WINDOW),
			       self);

        // Set the pixel format of the HWND if a GraphicsConfiguration
        // was provided to the Canvas constructor.

        jclass canvasClass = env->FindClass("java/awt/Canvas");
        if ( env->IsInstanceOf( target, canvasClass ) ) {

            // Get GraphicsConfig from our target
            jobject graphicsConfig = env->GetObjectField(target,
                AwtComponent::graphicsConfigID);
            if (graphicsConfig != NULL) {

                jclass win32cls = env->FindClass("sun/awt/Win32GraphicsConfig");
                DASSERT (win32cls != NULL);

                if ( env->IsInstanceOf( graphicsConfig, win32cls ) ) {
                    // Get the visual ID member from our GC
                    jint visual = env->GetIntField(graphicsConfig,
                          AwtWin32GraphicsConfig::win32GCVisualID);
                    if (visual > 0) {
                        HDC hdc = ::GetDC(canvas->m_hwnd);
                        // Set our pixel format
                        PIXELFORMATDESCRIPTOR pfd;
                        BOOL ret = ::SetPixelFormat(hdc, (int)visual, &pfd);
                        ::ReleaseDC(canvas->m_hwnd, hdc);
                        //Since a GraphicsConfiguration was specified, we should
                        //throw an exception if the PixelFormat couldn't be set.
                        if (ret == FALSE) {
                            DASSERT(!safe_ExceptionOccurred(env)); 
                            jclass excCls = env->FindClass(
                             "java/lang/RuntimeException");
                            DASSERT(excCls);
                            env->ExceptionClear();
                            env->ThrowNew(excCls,
                             "\nUnable to set Pixel format on Canvas");
                            env->DeleteLocalRef(target);
                            return canvas;
                        }
                    }
                }
            }
        }
    }
    } catch (...) {
        env->DeleteLocalRef(target);
	throw;
    }

done:
    env->DeleteLocalRef(target);
    return canvas;
    CATCH_BAD_ALLOC_RET(0);
}


/*
 * This routine is duplicated in AwtWindow.
 */
MsgRouting AwtCanvas::WmEraseBkgnd(HDC hDC, BOOL& didErase)
{
    m_eraseBackground = TRUE;
    didErase = TRUE;
    RECT	rc;
    ::GetClipBox(hDC, &rc);
    ::FillRect(hDC, &rc, this->GetBackgroundBrush());
    return mrConsume;
}

/*
 * This routine is duplicated in AwtWindow.
 */
MsgRouting AwtCanvas::WmPaint(HDC)
{
    DTRACE_PRINTLN("AwtCanvas::WmPaint Begin");
    PAINTSTRUCT ps;
    AWT_DUMP_UPDATE_RECTANGLE("AwtCanvas::WmPaint update rect", GetHWnd());
    HDC hDC = ::BeginPaint(GetHWnd(), &ps);// the passed-in HDC is ignored.
    DTRACE_PRINTLN1("AwtCanvas::WmPaint hdc=%x", hDC);
    DASSERT(hDC);
    RECT& r = ps.rcPaint;

    if ((r.right-r.left) > 0 && (r.bottom-r.top) > 0 &&
        m_peerObject != NULL && m_callbacksEnabled) {
        DoCallback("handleExpose",      // BUGID 4070488 repaint
                   "(IIII)V", r.left, r.top,
                   r.right-r.left, r.bottom-r.top);
    }
    m_eraseBackground = FALSE;
    VERIFY(::EndPaint(GetHWnd(), &ps));
    DTRACE_PRINTLN("AwtCanvas::WmPaint End");
    return mrConsume;
}

MsgRouting AwtCanvas::WmMouseDown(UINT flags, int x, int y, int button)
{
    // fixed 4035189: We capture the mouse on button down, since capturing
    // in WM_MOUSEMOVE doesn't work if mouse is already outside window
    // rkhan 4/28/98
    DASSERT(::GetCapture() == NULL || ::GetCapture() == GetHWnd()); // don't want to interfere with other controls
    if (::GetCapture() != GetHWnd()) {
	::SetCapture(GetHWnd());
    }
    return AwtComponent::WmMouseDown(flags, x, y, button);
}

MsgRouting AwtCanvas::WmMouseUp(UINT flags, int x, int y, int button)
{
    BOOL	isMouseCaptured = (::GetCapture() == GetHWnd());
    BOOL	areButtonsReleased = (flags & ALL_MK_BUTTONS) == 0;

    if ( isMouseCaptured && areButtonsReleased ) {
    // user has released all buttons, so release the capture
	::ReleaseCapture();
    }
    return AwtComponent::WmMouseUp(flags, x, y, button);
}

MsgRouting AwtCanvas::HandleEvent(MSG *msg, BOOL synthetic)
{
    if (msg->message == WM_LBUTTONDOWN || msg->message == WM_LBUTTONDBLCLK) {
	/*
	 * Fix for BugTraq ID 4041703: keyDown not being invoked.
	 * Give the focus to a Canvas or Panel if it doesn't have heavyweight 
	 * subcomponents so that they will behave the same way as on Solaris 
	 * providing a possibility of giving keyboard focus to an empty Applet. 
	 * Since ScrollPane doesn't receive focus on mouse press on Solaris, 
	 * HandleEvent() is overriden there to do nothing with focus.
	 */
	if (::GetWindow(GetHWnd(), GW_CHILD) == NULL) {
	    ::SetFocus(GetHWnd());
	}
    }
    return AwtComponent::HandleEvent(msg, synthetic);
}

/************************************************************************
 * WCanvasPeer native methods
 */

extern "C" {

JNIEXPORT void JNICALL
Java_sun_awt_windows_WCanvasPeer_create(JNIEnv *env, jobject self,
					jobject parent)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(parent);
    AwtToolkit::CreateComponent(self, parent,
				(AwtToolkit::ComponentFactory)
				AwtCanvas::Create);
    JNI_CHECK_PEER_CREATION_RETURN(self);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */

