/*
 * @(#)awt_Clipboard.cpp	1.22 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_Clipboard.h"
#include "awt_Toolkit.h"
#include <shlobj.h>
#include <sun_awt_windows_WClipboard.h>
#include <sun_awt_windows_FullyRenderedTransferable.h>


/************************************************************************
 * AwtClipboard fields
 */

jmethodID AwtClipboard::lostSelectionOwnershipMID;
jobject AwtClipboard::theCurrentClipboard;

/* This flag is set while we call EmptyClipboard to indicate to
   WM_DESTROYCLIPBOARD handler that we are not losing ownership */
BOOL AwtClipboard::isGettingOwnership = FALSE;

#define GALLOCFLG (GMEM_DDESHARE | GMEM_MOVEABLE | GMEM_ZEROINIT)

/************************************************************************
 * AwtClipboard methods
 */

void AwtClipboard::LostOwnership(JNIEnv *env) {
    if (theCurrentClipboard != NULL) {
        env->CallVoidMethod(theCurrentClipboard, lostSelectionOwnershipMID);
        DASSERT(!safe_ExceptionOccurred(env));
	env->DeleteGlobalRef(theCurrentClipboard);
        theCurrentClipboard = NULL;
    }
}

extern "C" {

/************************************************************************
 * WClipboard native methods
 */

/*
 * Class:     sun_awt_windows_WClipboard
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WClipboard_init(JNIEnv *env, jclass cls) 
{
    TRY;

    AwtClipboard::lostSelectionOwnershipMID = 
        env->GetMethodID(cls, "lostSelectionOwnershipImpl", "()V");
    DASSERT(AwtClipboard::lostSelectionOwnershipMID != NULL);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WClipboard
 * Method:    openClipboard
 * Signature: (Lsun/awt/windows/WClipboard;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WClipboard_openClipboard(JNIEnv *env, jclass cls,
					      jobject newOwner)
{
    TRY;

    DASSERT(::GetOpenClipboardWindow() != AwtToolkit::GetInstance().GetHWnd());

    if (!::OpenClipboard(AwtToolkit::GetInstance().GetHWnd())) {
        JNU_ThrowByName(env, "java/lang/IllegalStateException",
			"cannot open system clipboard");
	return;
    }
    if (newOwner != NULL) {
        AwtClipboard::GetOwnership();
	if (AwtClipboard::theCurrentClipboard != NULL) {
	    env->DeleteGlobalRef(AwtClipboard::theCurrentClipboard);
	}
	AwtClipboard::theCurrentClipboard = env->NewGlobalRef(newOwner);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WClipboard
 * Method:    closeClipboard
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WClipboard_closeClipboard(JNIEnv *env, jclass cls)
{
    TRY;

    DASSERT(::GetOpenClipboardWindow() == AwtToolkit::GetInstance().GetHWnd());

    VERIFY(::CloseClipboard());

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WClipboard
 * Method:    publishClipboardData
 * Signature: (I[B)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WClipboard_publishClipboardData(JNIEnv *env,
						     jobject self,
						     jint format,
						     jbyteArray bytes)
{
    TRY;

    DASSERT(::GetOpenClipboardWindow() == AwtToolkit::GetInstance().GetHWnd());

    jint nBytes = env->GetArrayLength(bytes);

    // We have to prepend the DROPFILES structure here because WDataTransferer
    // doesn't.
    HGLOBAL hglobal = ::GlobalAlloc(GALLOCFLG, nBytes + ((format == CF_HDROP)
							    ? sizeof(DROPFILES)
							    : 0));
    if (hglobal == NULL) {
	throw std::bad_alloc();
    }
    char *dataout = (char *)::GlobalLock(hglobal);
    
    if (format == CF_HDROP) {
        DROPFILES *dropfiles = (DROPFILES *)dataout;
	dropfiles->pFiles = sizeof(DROPFILES);
	dropfiles->fWide = FALSE; // good guess!
        dataout += sizeof(DROPFILES);
    }

    env->GetByteArrayRegion(bytes, 0, nBytes, (jbyte *)dataout);
    ::GlobalUnlock(hglobal);

    VERIFY(::SetClipboardData(format, hglobal));

    CATCH_BAD_ALLOC;
}

/************************************************************************
 * FullyRenderedTransferable native methods
 */

/*
 * Class:     sun_awt_windows_FullyRenderedTransferable
 * Method:    getClipboardFormats
 * Signature: ()[I
 */
JNIEXPORT jintArray JNICALL
Java_sun_awt_windows_FullyRenderedTransferable_getClipboardFormats(JNIEnv *env,
								  jobject self)
{
    TRY;

    DASSERT(::GetOpenClipboardWindow() == AwtToolkit::GetInstance().GetHWnd());

    int nFormats = ::CountClipboardFormats();
    jintArray formats = env->NewIntArray(nFormats);
    if (formats == NULL) {
        throw std::bad_alloc();
    }
    jboolean isCopy;
    jint *iFormats = env->GetIntArrayElements(formats, &isCopy),
        *saveFormats = iFormats;
    UINT num = 0;

    for (int i = 0; i < nFormats; i++, iFormats++) {
        *iFormats = num = ::EnumClipboardFormats(num);
    }
    
    env->ReleaseIntArrayElements(formats, saveFormats, 0);

    return formats;

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_FullyRenderedTransferable
 * Method:    getClipboardBytes
 * Signature: (I)[B
 */
JNIEXPORT jbyteArray JNICALL
Java_sun_awt_windows_FullyRenderedTransferable_getClipboardBytes(JNIEnv *env,
								 jobject self,
								 jint format)
{
    TRY;

    DASSERT(::GetOpenClipboardWindow() == AwtToolkit::GetInstance().GetHWnd());

    HANDLE handle = ::GetClipboardData(format);
    if (handle == NULL) {
        JNU_ThrowIOException(env, "system clipboard data unavailable");
	return NULL;
    }

    DWORD size = ::GlobalSize(handle);
    DASSERT(size != 0 || ::GetLastError() == 0);

    jbyteArray bytes = env->NewByteArray(size);
    if (bytes == NULL) {
	throw std::bad_alloc();
    }

    LPVOID data = ::GlobalLock(handle);
    env->SetByteArrayRegion(bytes, 0, size, (jbyte *)data);
    ::GlobalUnlock(handle);

    return bytes;

    CATCH_BAD_ALLOC_RET(NULL);
}

} /* extern "C" */
