/*
 * @(#)awt_DataTransferer.cpp	1.4 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt.h"
#include "awt_dlls.h"
#include "awt_TextComponent.h"
#include <shlobj.h>
#include <sun_awt_windows_WDataTransferer.h>

#define GALLOCFLG (GMEM_DDESHARE | GMEM_MOVEABLE | GMEM_ZEROINIT)

extern "C" {

/*
 * Class:     sun_awt_windows_WDataTransferer
 * Method:    registerClipboardFormat
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDataTransferer_registerClipboardFormat(JNIEnv *env,
							     jobject obj,
							     jstring str)
{
    TRY;

    jboolean isCopy;
    const char *cStr = env->GetStringUTFChars(str, &isCopy);
    jint value = ::RegisterClipboardFormat(cStr);
    env->ReleaseStringUTFChars(str, cStr);

    return value;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WDataTransferer
 * Method:    getClipboardFormatName
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL
Java_sun_awt_windows_WDataTransferer_getClipboardFormatName(JNIEnv *env,
							    jobject obj,
							    jint format)
{
    TRY;

    char *buf = new char[512]; // perhaps a bad idea to limit ourselves to 512
    VERIFY(::GetClipboardFormatName(format, buf, 512));
    jstring name = env->NewStringUTF(buf);
    delete [] buf;
    if (name == NULL) {
	throw std::bad_alloc();
    }
    return name;

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WDataTransferer
 * Method:    draqQueryFile
 * Signature: ([B)[Ljava/lang/String;
 */
JNIEXPORT jobjectArray JNICALL
Java_sun_awt_windows_WDataTransferer_dragQueryFile(JNIEnv *env, jobject obj,
						   jbyteArray bytes)
{
    TRY;

    /*
     * Fix for the BugTraq ID 4327064 - inter-jvm DnD crashes the droping jvm.
     * On Win9X DragQueryFile() doesn't accept a pointer to the local help as
     * the first argument, so we should dump the bits into global memory.
     */
    UINT size = env->GetArrayLength(bytes);
    HGLOBAL hglobal = NULL;
    jbyte *bBytes = NULL;
    HDROP hdrop = NULL;
    LPTSTR buffer = NULL;

    hglobal = ::GlobalAlloc(GALLOCFLG, size);

    if (hglobal == NULL) {
        throw std::bad_alloc();
    }
    
    try {

        bBytes = (jbyte*)::GlobalLock(hglobal);
        env->GetByteArrayRegion(bytes, 0, size, bBytes);

        hdrop = (HDROP)bBytes;

        load_shell_procs();

	UINT nFilenames = (*do_drag_query_file)(hdrop, (UINT)-1, NULL, 0);

	jclass str_clazz = env->FindClass("java/lang/String");
	DASSERT(str_clazz != NULL);
	jobjectArray filenames = env->NewObjectArray(nFilenames, str_clazz,
						     NULL);
	if (filenames == NULL) {
	    throw std::bad_alloc();
	}

	UINT bufsize = 512;
	buffer = (LPTSTR)safe_Malloc(bufsize);
    
	for (UINT i = 0; i < nFilenames; i++) {
	    UINT size = (*do_drag_query_file)(hdrop, i, NULL, 0);
	    if (size > bufsize) {
	        bufsize = size;
		buffer = (LPTSTR)safe_Realloc(buffer, bufsize);
	    }
	    (*do_drag_query_file)(hdrop, i, buffer, bufsize);
	    jstring name = JNU_NewStringPlatform(env, buffer);
	    env->SetObjectArrayElement(filenames, i, name);
	}

	free(buffer);
        ::GlobalUnlock(hglobal);
        ::GlobalFree(hglobal);
	return filenames;

    } catch (std::bad_alloc&) {
        free(buffer);
        ::GlobalUnlock(hglobal);
        ::GlobalFree(hglobal);
        throw;
    }

    CATCH_BAD_ALLOC_RET(NULL);
}

}
