/*
 * @(#)awt_DnDDS.cpp	1.24 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "jlong.h"
#include "awt_DnDDS.h"
#include "awt_Cursor.h"
#include "awt_Toolkit.h"
#include "awt_Component.h"

#include <java_awt_event_InputEvent.h>
#include <sun_awt_windows_WDragSourceContextPeer.h>

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <shlobj.h>

#define GALLOCFLG (GMEM_DDESHARE | GMEM_MOVEABLE | GMEM_ZEROINIT)

extern "C" {
DWORD __cdecl convertActionsToDROPEFFECT(jint actions);
jint  __cdecl convertDROPEFFECTToActions(DWORD effects);

DWORD __cdecl mapModsToDROPEFFECT(DWORD, DWORD);
}

/*
 * static initialization
 */

jclass AwtDragSource::dSCClazz    = (jclass)NULL; // the DragSourceContext
jclass AwtDragSource::awtIEClazz  = (jclass)NULL; // InputEvent

jmethodID AwtDragSource::dSCdragenter  = (jmethodID)NULL;
jmethodID AwtDragSource::dSCdragmotion = (jmethodID)NULL;
jmethodID AwtDragSource::dSCopschanged = (jmethodID)NULL;
jmethodID AwtDragSource::dSCdragexit   = (jmethodID)NULL;
jmethodID AwtDragSource::dSCddfinish   = (jmethodID)NULL;

jmethodID AwtDragSource::dSCgetdxh     = (jmethodID)NULL;
jmethodID AwtDragSource::dSCgetdyh     = (jmethodID)NULL;
jmethodID AwtDragSource::dSCgetdiwcr   = (jmethodID)NULL;

jmethodID AwtDragSource::dSCconvertdata  = (jmethodID)NULL;

jfieldID AwtDragSource::awtIEmods = (jfieldID)NULL;

BOOL AwtDragSource:: m_isOldCursorCustom = FALSE;



/**
 * static initialization
 */

void AwtDragSource::InitAwtDragSource() {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    // first load the class ...

    dSCClazz = env->FindClass("sun/awt/windows/WDragSourceContextPeer");

    if (JNU_IsNull(env, dSCClazz)) return;

    dSCClazz = (jclass)env->NewGlobalRef(dSCClazz); // never freed

    // now resolve upcalls ...

    dSCdragenter = env->GetMethodID(dSCClazz, "dragEnter", "(III)V");

    if (JNU_IsNull(env, dSCdragenter)) return;

    dSCdragmotion = env->GetMethodID(dSCClazz, "dragMotion", "(III)V");

    if (JNU_IsNull(env, dSCdragmotion)) return;

    dSCopschanged = env->GetMethodID(dSCClazz, "operationChanged", "(III)V");

    if (JNU_IsNull(env, dSCopschanged)) return;

    dSCdragexit = env->GetMethodID(dSCClazz, "dragExit", "()V");

    if (JNU_IsNull(env, dSCdragexit)) return;

    dSCddfinish = env->GetMethodID(dSCClazz, "dragDropFinished", "(ZIIIZ)V");

    if (JNU_IsNull(env, dSCddfinish)) return;

    dSCgetdxh = env->GetMethodID(dSCClazz, "getDropXHint", "()I");

    if (JNU_IsNull(env, dSCgetdxh)) return;

    dSCgetdyh = env->GetMethodID(dSCClazz, "getDropYHint", "()I");

    if (JNU_IsNull(env, dSCgetdyh)) return;

    dSCgetdiwcr = env->GetMethodID(dSCClazz, "getDroppedInWindowCR", "()Z");

    if (JNU_IsNull(env, dSCgetdiwcr)) return;

    dSCconvertdata = env->GetMethodID(dSCClazz, "convertData", "(ILjava/util/Map;)[B");

    if (JNU_IsNull(env, dSCconvertdata)) return;

    awtIEClazz = (jclass)env->FindClass("java/awt/event/InputEvent");

    if (JNU_IsNull(env, awtIEClazz)) return;

    awtIEClazz = (jclass)env->NewGlobalRef(awtIEClazz);

    awtIEmods = env->GetFieldID(awtIEClazz, "modifiers", "I");

    if (JNU_IsNull(env, awtIEmods)) return;
}

/**
 * StartDrag
 */

void AwtDragSource::StartDrag(AwtDragSource* self, jobject cursor) {
    JNIEnv* 		   env   	 = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    HCURSOR		   initialCursor = self->SetCursor(cursor);

    AwtToolkit::GetInstance().WaitForSingleObject(self->m_mutex);

    AwtToolkit::GetInstance().InvokeFunctionLater((void (*)(void *))&AwtDragSource::_DoDragDrop, (void *)self);

    self->WaitUntilSignalled(FALSE);

    if (initialCursor != NULL) {
      if( TRUE == m_isOldCursorCustom ) {
	AwtToolkit::GetInstance().NotifyCustomCursor();
      } else {
	AwtToolkit::GetInstance().NotifySystemCursor();
      }
      ::SetCursor(initialCursor);
    }
}

/**
 * DoDragDrop - called from message pump thread
 */

void AwtDragSource::_DoDragDrop(void* param) {
    AwtDragSource* dragSource   = (AwtDragSource*)param;
    DWORD          effects      = DROPEFFECT_NONE;
    JNIEnv*        env          = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject        peer         = env->NewLocalRef(dragSource->GetPeer());
    POINT          client       = { 0, 0 };
    POINT          drop         = { 0, 0 };
    jboolean       dropInClient = JNI_FALSE;

    HRESULT        res;

    // StartDrag has caused dragSource->m_mutex to be held by our thread now

    res = ::DoDragDrop(dragSource,
		       dragSource,
		       dragSource->m_effects = convertActionsToDROPEFFECT(dragSource->m_actions),
		       &effects
	  );

    // for later HDROP processing ... probably wrong ... but a good guess
    // the x,y and dropInClient boolean are used to fill in the DROPFILES
    // structs later ...

    if (res == DRAGDROP_S_DROP && effects != DROPEFFECT_NONE) {

	::GetCursorPos(&drop);

	HWND win = ::WindowFromPoint(client = drop);

	if (win != (HWND)NULL || win != ::GetDesktopWindow()) {
	    ::ScreenToClient(win, &client);

	    RECT rect;

	    ::GetClientRect(win, &rect);

	    dropInClient = (client.x < rect.left  ||
	  		    client.x > rect.right ||
	     		    client.y < rect.top   ||
	     		    client.y > rect.bottom) ? JNI_FALSE : JNI_TRUE;
	}
    }

    env->CallVoidMethod(peer,
			dSCddfinish,
			res     == DRAGDROP_S_DROP &&
			effects != DROPEFFECT_NONE,
			convertDROPEFFECTToActions(effects),
			(jint)client.x,
			(jint)client.y,
			dropInClient ? JNI_TRUE : JNI_FALSE
		);

    env->DeleteLocalRef(peer);

    dragSource->Signal();
}

/**
 * constructor
 */

AwtDragSource::AwtDragSource(JNIEnv* env, jobject peer, jobject trigger,
			     jint actions, jintArray formats,
			     jobject formatMap) {
    m_peer	= env->NewGlobalRef(peer);

    m_refs	= 0;

    m_actions   = actions;

    m_ntypes    = 0;

    m_trigger   = env->NewGlobalRef(trigger);
    m_initmods	= env->GetIntField(trigger, awtIEmods);
    m_lastmods	= m_initmods;

    m_cancel    = JNI_FALSE;

    m_startCond  = JNI_FALSE;

    m_droptarget   = NULL;
    m_enterpending = FALSE;

    m_cursor     = NULL;

    m_mutex      = ::CreateMutex(NULL, FALSE, NULL);

    m_formatMap  = env->NewGlobalRef(formatMap);

    LoadCache(formats);

    AddRef();
}

/**
 * destructor
 */

AwtDragSource::~AwtDragSource() {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    env->DeleteGlobalRef(m_peer);
    env->DeleteGlobalRef(m_trigger);
    env->DeleteGlobalRef(m_formatMap);

    ::CloseHandle(m_mutex);

    UnloadCache();
}

/**
 * _compar
 *
 * compare format's then tymed's .... only one tymed bit may be set
 * at any time in a FORMATETC in the cache.
 */

int AwtDragSource::_compar(const void* first, const void* second) {
    FORMATETC *fp = (FORMATETC *)first;
    FORMATETC *sp = (FORMATETC *)second;
    int      r  = fp->cfFormat - sp->cfFormat;

    return r != 0 ? r : fp->tymed - sp->tymed;
}

/**
 * LoadCache
 */

void AwtDragSource::LoadCache(jintArray formats) {
    JNIEnv*      env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    unsigned int items;

    if (m_types != (FORMATETC *)NULL) {
        UnloadCache();
    }

    items = m_ntypes = env->GetArrayLength(formats);

    if (m_ntypes == 0) {
        return;
    }

    // we allocate twice the number of actual types in order to represent
    // them both as FORMATETC's for HGLOBAL's and ISTREAM TYMED's ...

    m_ntypes *= 2;
    try {
        m_types = (FORMATETC *)safe_Calloc(sizeof(FORMATETC), m_ntypes);
    } catch (std::bad_alloc&) {
	m_ntypes = 0;
	throw;
    }

    jboolean isCopy;
    jint *iFormats = env->GetIntArrayElements(formats, &isCopy),
        *saveFormats = iFormats;

    for (unsigned int i = 0; i < items; i++, iFormats++) {
	unsigned int idx = i * 2;

	m_types[idx].cfFormat = (unsigned short)*iFormats;
	m_types[idx].dwAspect = DVASPECT_CONTENT;
	m_types[idx].lindex   = -1;
	m_types[idx].tymed    = TYMED_HGLOBAL; // for now ...

	// now make a copy, but with a TYMED of ISTREAM

	idx++;
	memcpy(m_types + idx, m_types + idx - 1, sizeof(FORMATETC));
	m_types[idx].tymed = TYMED_ISTREAM;
    }

    env->ReleaseIntArrayElements(formats, saveFormats, 0);

    // sort them in ascending order of format
    qsort((void *)m_types, (size_t)m_ntypes, (size_t)sizeof(FORMATETC),
	  _compar);
}

/**
 * UnloadCache
 */

void AwtDragSource::UnloadCache() {
    if (m_ntypes == 0) {
        return;
    }

    free((void*)m_types);
    m_ntypes = 0;
    m_types  = (FORMATETC *)NULL;
}

/**
 * SetCursor
 */

HCURSOR AwtDragSource::SetCursor(jobject cursor) {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (JNU_IsNull(env, cursor)) {
        return ::GetCursor();
    }

    m_isOldCursorCustom = FALSE;
    if( NULL != m_cursor ) {
      m_isOldCursorCustom = m_cursor->IsCustomCursor();
    }

    jlong pData = env->GetLongField(cursor, AwtCursor::pDataID);
    m_cursor = (AwtCursor *)jlong_to_ptr(pData);

    if (m_cursor == NULL) {
        m_cursor = AwtCursor::CreateSystemCursor(cursor);
    }

    if( TRUE == m_cursor->IsCustomCursor() ) {
      AwtToolkit::GetInstance().NotifyCustomCursor();
    } else {
      AwtToolkit::GetInstance().NotifySystemCursor();
    }

    return ::SetCursor(m_cursor->GetHCursor());
}

/**
 * MatchFormatEtc
 */

HRESULT __stdcall 
AwtDragSource::MatchFormatEtc(FORMATETC __RPC_FAR *pFormatEtcIn,
			      FORMATETC *cacheEnt) {
    TRY;

    if ((pFormatEtcIn->tymed & (TYMED_HGLOBAL | TYMED_ISTREAM)) == 0) {
	return DV_E_TYMED;
    } else if (pFormatEtcIn->lindex != -1) {
        return DV_E_LINDEX;
    } else if (pFormatEtcIn->dwAspect != DVASPECT_CONTENT) {
        return DV_E_DVASPECT;
    }

    FORMATETC tmp;
    memcpy(&tmp, pFormatEtcIn, sizeof(FORMATETC));

    BOOL search = TRUE;

    if ((pFormatEtcIn->tymed & TYMED_HGLOBAL) != 0) {
        tmp.tymed = TYMED_HGLOBAL;
    } else if ((pFormatEtcIn->tymed & TYMED_ISTREAM) != 0) {
        tmp.tymed = TYMED_ISTREAM;
    } else {
        search = FALSE;
    }

    if (search) {
    	FORMATETC *cp = (FORMATETC *)bsearch((const void *)&tmp,
					     (const void *)m_types,
					     (size_t)      m_ntypes,
					     (size_t)      sizeof(FORMATETC),
					                   _compar
					     );
	if (cp != (FORMATETC *)NULL) {
	    if (cacheEnt != (FORMATETC *)NULL) {
	        memcpy(cacheEnt, cp, sizeof(FORMATETC));
	    }
	    return S_OK;
	}
    }

    return DV_E_FORMATETC;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * QueryInterface
 */

HRESULT __stdcall AwtDragSource::QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject) {
    TRY;

    if (riid == IID_IUnknown) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IUnknown*)(IDropSource*)this;
	AddRef();
	return S_OK;
    } else if (riid == IID_IDropSource) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IDropSource*)this;
	AddRef();
	return S_OK;
    } else if (riid == IID_IDataObject) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IDataObject*)this;
	AddRef();
	return S_OK;
    } else {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)NULL;
	return E_NOINTERFACE;
    }

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * AddRef
 */

ULONG __stdcall AwtDragSource::AddRef() {
    return (ULONG)++m_refs;
}

/**
 * Release
 */

ULONG __stdcall AwtDragSource::Release() {
    int refs;

    if ((refs = --m_refs) == 0) delete this;

    return (ULONG)refs;
}

/**
 * QueryContinueDrag
 */

HRESULT __stdcall  AwtDragSource::QueryContinueDrag(BOOL fEscapeKeyPressed, DWORD grfKeyState) {
    TRY;

    JNIEnv* env       = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jint    mb        = 0;
    jint    initialmb = m_initmods & (java_awt_event_InputEvent_BUTTON1_MASK |
				      java_awt_event_InputEvent_SHIFT_MASK   |
				      java_awt_event_InputEvent_CTRL_MASK    |
				      java_awt_event_InputEvent_ALT_MASK     |
				      java_awt_event_InputEvent_META_MASK
				     );

    if (m_cancel || fEscapeKeyPressed)
	return DRAGDROP_S_CANCEL;

    if ((grfKeyState & MK_LBUTTON) || (::GetAsyncKeyState(VK_LBUTTON) & 0xff00))
	mb    = LEFT_BUTTON;
    else if ((grfKeyState & MK_MBUTTON) || (::GetAsyncKeyState(VK_MBUTTON) & 0xff00))
	mb = MIDDLE_BUTTON;
    else if ((grfKeyState & MK_RBUTTON) || (::GetAsyncKeyState(VK_RBUTTON) & 0xff00))
	mb = RIGHT_BUTTON;

    jint modifiers = AwtComponent::GetJavaModifiers(0, mb);

    if (!m_startCond)
	m_startCond = (modifiers & initialmb) == initialmb;
    else if ((modifiers & initialmb) != initialmb) {
	return DRAGDROP_S_DROP;
    } else if (m_lastmods != (modifiers = AwtComponent::GetJavaModifiers(grfKeyState, mb))) {
	env->CallVoidMethod(m_peer, dSCopschanged, convertDROPEFFECTToActions(mapModsToDROPEFFECT(m_effects, grfKeyState)), m_actions, modifiers);
    }

    m_lastmods = modifiers;

   if (m_cancel)
	return DRAGDROP_S_CANCEL;
   else {
     if( TRUE == m_cursor->IsCustomCursor() ) {
       AwtToolkit::GetInstance().NotifyCustomCursor();
     } else {
       AwtToolkit::GetInstance().NotifySystemCursor();
     }
     ::SetCursor(m_cursor->GetHCursor());

     return S_OK;
   }

   CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * GiveFeedback
 */

HRESULT __stdcall  AwtDragSource::GiveFeedback(DWORD dwEffect) {
    TRY;

    JNIEnv* env       = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jint    modifiers = 0;
    int	    mb;
    SHORT   mods      = 0;


    m_actions = convertDROPEFFECTToActions(dwEffect);

    if (::GetAsyncKeyState(VK_LBUTTON) & 0xff00) {
	mb    = LEFT_BUTTON;
	mods |= MK_LBUTTON;
    } else if (::GetAsyncKeyState(VK_MBUTTON) & 0xff00) {
	mb    = MIDDLE_BUTTON;
	mods |= MK_MBUTTON;
    } else if (::GetAsyncKeyState(VK_RBUTTON) & 0xff00) {
	mb    = RIGHT_BUTTON;
	mods |= MK_RBUTTON;
    }

    if (::GetAsyncKeyState(VK_SHIFT)   & 0xff00)
	mods |= MK_SHIFT;
    if (::GetAsyncKeyState(VK_CONTROL) & 0xff00)
	mods |= MK_CONTROL;
    if (::GetAsyncKeyState(VK_MENU) & 0xff00)
	mods |= MK_ALT;
	
    modifiers = AwtComponent::GetJavaModifiers(mods, mb);

    POINT curs;

    ::GetCursorPos(&curs);

    HWND win = ::WindowFromPoint(curs);

    int invalid = (dwEffect == DROPEFFECT_NONE);

    if (m_droptarget != NULL) {
	if (!invalid) {
	    env->CallVoidMethod(m_peer,
				m_enterpending ? dSCdragenter : dSCdragmotion,
				m_effects = convertDROPEFFECTToActions(mapModsToDROPEFFECT(dwEffect, mods)),
				m_actions,
				modifiers
	    );

	    m_droptarget   = win;
            m_enterpending = FALSE;
	} else {
	    if (!m_enterpending)
	        env->CallVoidMethod(m_peer, dSCdragexit);

	    m_droptarget = (HWND)NULL;
	}

        m_enterpending = FALSE;
    } else if (!invalid) {
	m_droptarget   = win;
	m_enterpending = TRUE;
    }
 
    if( TRUE == m_cursor->IsCustomCursor() ) {
      AwtToolkit::GetInstance().NotifyCustomCursor();
    } else {
      AwtToolkit::GetInstance().NotifySystemCursor();
    }
    ::SetCursor(m_cursor->GetHCursor());

    if (m_cursor->IsCustomCursor()) {
	return S_OK; 
    } else {
	return DRAGDROP_S_USEDEFAULTCURSORS;
    }

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}


/**
 * GetData
 */

HRESULT __stdcall AwtDragSource::GetData(FORMATETC __RPC_FAR *pFormatEtc,
					 STGMEDIUM __RPC_FAR *pmedium) {
    TRY;

    HRESULT res = MatchFormatEtc(pFormatEtc, (FORMATETC *)NULL);
    if (res != S_OK) {
        return res;
    }

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    jbyteArray bytes = (jbyteArray)env->CallObjectMethod(m_peer,
							 dSCconvertdata,
							 pFormatEtc->cfFormat,
							 m_formatMap);
    if (bytes == NULL) {
        return E_UNEXPECTED;
    }
    if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
        env->ExceptionDescribe();
	env->ExceptionClear();

	return E_UNEXPECTED;
    }

    jint nBytes = env->GetArrayLength(bytes);

    if (pFormatEtc->cfFormat != CF_HDROP &&
	    (pFormatEtc->tymed & TYMED_ISTREAM) != 0) {
        ADSIStreamProxy *istream = new ADSIStreamProxy(this, bytes, nBytes);

	if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
	    return E_UNEXPECTED;
	}

	pmedium->tymed = TYMED_ISTREAM;
	pmedium->pstm = istream;
	pmedium->pUnkForRelease = (IUnknown *)istream;

	return S_OK;
    } else if ((pFormatEtc->tymed & TYMED_HGLOBAL) != 0) {
        HGLOBAL copy = ::GlobalAlloc(GALLOCFLG, nBytes +
				     ((pFormatEtc->cfFormat == CF_HDROP)
				          ? sizeof(DROPFILES)
				          : 0));
	if (copy == NULL) {
	    throw std::bad_alloc();
	}

	char *dataout = (char *)::GlobalLock(copy);

	if (pFormatEtc->cfFormat == CF_HDROP) {
	    DROPFILES *dropfiles = (DROPFILES *)dataout;
	    dropfiles->pFiles = sizeof(DROPFILES);
	    dropfiles->pt.x = (LONG)env->CallIntMethod(m_peer, dSCgetdxh);
	    dropfiles->pt.y = (LONG)env->CallIntMethod(m_peer, dSCgetdyh);
	    dropfiles->fNC = env->CallBooleanMethod(m_peer, dSCgetdiwcr);
	    dropfiles->fWide = FALSE; // good guess!
	    dataout += sizeof(DROPFILES);
	}

	env->GetByteArrayRegion(bytes, 0, nBytes, (jbyte *)dataout);
	::GlobalUnlock(copy);

	pmedium->tymed = TYMED_HGLOBAL;
	pmedium->hGlobal = copy;
	pmedium->pUnkForRelease = (IUnknown *)NULL;

	return S_OK;
    }
    
    return DV_E_TYMED;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * GetDataHere
 */

HRESULT __stdcall AwtDragSource::GetDataHere(FORMATETC __RPC_FAR *pFormatEtc,
					     STGMEDIUM __RPC_FAR *pmedium) {
    TRY;

    if (pmedium->pUnkForRelease != (IUnknown *)NULL) {
        return E_INVALIDARG;
    }

    HRESULT res = MatchFormatEtc(pFormatEtc, NULL);
    if (res != S_OK) {
        return res;
    }

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    jbyteArray bytes = (jbyteArray)env->CallObjectMethod(m_peer,
							 dSCconvertdata,
							 pFormatEtc->cfFormat,
							 m_formatMap);
    if (bytes == NULL) {
        return E_UNEXPECTED;
    }
    if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
        env->ExceptionDescribe();
	env->ExceptionClear();

	return E_UNEXPECTED;
    }

    jint nBytes = env->GetArrayLength(bytes);

    if ((pFormatEtc->tymed & TYMED_HGLOBAL) != 0) {
        DWORD mBytes = ::GlobalSize(pmedium->hGlobal);

	if (nBytes + ((pFormatEtc->cfFormat == CF_HDROP)
		        ? sizeof(DROPFILES) : 0) > mBytes) {
	    return STG_E_MEDIUMFULL;
	}

	char *dataout = (char *)::GlobalLock(pmedium->hGlobal);

	if (pFormatEtc->cfFormat == CF_HDROP) {
	    DROPFILES *dropfiles = (DROPFILES *)dataout;
	    dropfiles->pFiles = sizeof(DROPFILES);
	    dropfiles->pt.x = (LONG)env->CallIntMethod(m_peer, dSCgetdxh);
	    dropfiles->pt.y = (LONG)env->CallIntMethod(m_peer, dSCgetdyh);
	    dropfiles->fNC = env->CallBooleanMethod(m_peer, dSCgetdiwcr);
	    dropfiles->fWide = FALSE; // good guess!
	    dataout += sizeof(DROPFILES);
	}

	env->GetByteArrayRegion(bytes, 0, nBytes, (jbyte *)dataout);
	::GlobalUnlock(pmedium->hGlobal);

	return S_OK;
    } else if (pFormatEtc->cfFormat != CF_HDROP && 
	           (pFormatEtc->tymed & TYMED_ISTREAM) != 0) {
        jboolean isCopy;
	jbyte *bBytes = env->GetByteArrayElements(bytes, &isCopy);

	ULONG act;
	HRESULT res = pmedium->pstm->Write((const void *)bBytes, (ULONG)nBytes,
					   &act);

	env->ReleaseByteArrayElements(bytes, bBytes, JNI_ABORT);

	return res;
    }
    
    return DV_E_TYMED;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * QueryGetData
 */

HRESULT __stdcall  AwtDragSource::QueryGetData(FORMATETC __RPC_FAR *pFormatEtc) {
    TRY;

    return MatchFormatEtc(pFormatEtc, (FORMATETC *)NULL);

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}


/**
 * GetCanonicalFormatEtc
 */

HRESULT __stdcall  AwtDragSource::GetCanonicalFormatEtc(FORMATETC __RPC_FAR *pFormatEtcIn, FORMATETC __RPC_FAR *pFormatEtcOut) {
    TRY;

    HRESULT   res = MatchFormatEtc(pFormatEtcIn, (FORMATETC *)NULL);

    if (res != S_OK) return res;

    *pFormatEtcOut = *pFormatEtcIn;

    pFormatEtcOut->ptd = NULL;

    return DATA_S_SAMEFORMATETC;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * SetData
 */

HRESULT __stdcall AwtDragSource::SetData(FORMATETC __RPC_FAR *pFormatEtc, STGMEDIUM __RPC_FAR *pmedium, BOOL fRelease) {
    return E_NOTIMPL;
}

/**
 * EnumFormatEtc
 */

HRESULT __stdcall  AwtDragSource::EnumFormatEtc(DWORD dwDirection, IEnumFORMATETC *__RPC_FAR *ppenumFormatEtc) {
    TRY;

    *ppenumFormatEtc = new ADSIEnumFormatEtc(this);
    return S_OK;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * DAdvise
 */

HRESULT __stdcall  AwtDragSource::DAdvise(FORMATETC __RPC_FAR *pFormatEtc, DWORD advf, IAdviseSink __RPC_FAR *pAdvSink, DWORD __RPC_FAR *pdwConnection) {
    return E_NOTIMPL;
}

/**
 * DUnadvise
 */

HRESULT __stdcall  AwtDragSource::DUnadvise(DWORD dwConnection) {
    return OLE_E_ADVISENOTSUPPORTED;
}

/**
 * EnumAdvise
 */

HRESULT __stdcall  AwtDragSource::EnumDAdvise(IEnumSTATDATA __RPC_FAR *__RPC_FAR *ppenumAdvise) {
    return OLE_E_ADVISENOTSUPPORTED;
}

/**
 * Constructor
 */

AwtDragSource::ADSIEnumFormatEtc::ADSIEnumFormatEtc(AwtDragSource* parent) {
    m_parent = parent;
    m_idx    = 0;

    m_refs   = 0;

    m_parent->AddRef();

    AddRef();
}

/**
 * Destructor
 */

AwtDragSource::ADSIEnumFormatEtc::~ADSIEnumFormatEtc() {
    m_parent->Release();
}

/**
 * QueryInterface
 */

HRESULT __stdcall  AwtDragSource::ADSIEnumFormatEtc::QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject) {
    TRY;

    if (riid == IID_IUnknown) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IUnknown*)this;
	AddRef();
	return S_OK;
    } else if (riid == IID_IEnumFORMATETC) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IEnumFORMATETC*)this;
	AddRef();
	return S_OK;
    } else {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)NULL;
	return E_NOINTERFACE;
    }

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * AddRef
 */

ULONG __stdcall  AwtDragSource::ADSIEnumFormatEtc::AddRef(void) {
    return (ULONG)++m_refs;
}

/**
 * Release
 */

ULONG __stdcall  AwtDragSource::ADSIEnumFormatEtc::Release(void) {
    int refs;

    if ((refs = --m_refs) == 0) delete this;

    return (ULONG)refs;
}

/**
 * Next
 */

HRESULT _stdcall AwtDragSource::ADSIEnumFormatEtc::Next(ULONG celt, FORMATETC __RPC_FAR *rgelt, ULONG __RPC_FAR *pceltFetched) {
    TRY;

    unsigned int len = m_parent->getNTypes();
    unsigned int i;

    for (i = 0; i < celt && m_idx < len; i++, m_idx++) {
	FORMATETC fetc = m_parent->getType(m_idx);
	rgelt[i] = fetc;
    }

    if (pceltFetched != NULL) *pceltFetched = i;

    return i == celt ? S_OK : S_FALSE;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Skip
 */

HRESULT __stdcall  AwtDragSource::ADSIEnumFormatEtc::Skip(ULONG celt) {
    TRY;

    unsigned int len = m_parent->getNTypes();
    unsigned int tmp = m_idx + celt;

    if (tmp < len) {
	m_idx = tmp;

        return S_OK;
    } else {
	m_idx = len;

        return S_FALSE;
    }

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Reset
 */

HRESULT __stdcall  AwtDragSource::ADSIEnumFormatEtc::Reset(void) {
    m_idx = 0;

    return S_OK;
}

/**
 * Clone
 */

HRESULT __stdcall  AwtDragSource::ADSIEnumFormatEtc::Clone(IEnumFORMATETC  __RPC_FAR *__RPC_FAR *ppenum) {
    TRY;

    *ppenum = new ADSIEnumFormatEtc(m_parent);
    (*ppenum)->Skip(m_idx);
    return S_OK;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * constructor
 */

AwtDragSource::ADSIStreamProxy::ADSIStreamProxy(AwtDragSource* parent, jbyteArray buffer, jint blen) {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    m_parent = parent;

    m_buffer = (signed char *)safe_Calloc(sizeof(signed char), m_blen = blen);

    env->GetByteArrayRegion(buffer, 0, blen, m_buffer);

    if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) return;

    m_off     = 0;

    m_cloneof = (ADSIStreamProxy*)NULL;
    
    m_refs    = 0;

    FILETIME now;

    ::CoFileTimeNow(&now);

    m_statstg.pwcsName		= (LPWSTR)NULL;
    m_statstg.type		= STGTY_STREAM;
    m_statstg.cbSize.HighPart 	= 0;
    m_statstg.cbSize.LowPart  	= m_blen;
    m_statstg.mtime		= now;
    m_statstg.ctime 		= now;
    m_statstg.atime 		= now;
    m_statstg.grfMode 		= STGM_READ;
    m_statstg.grfLocksSupported = FALSE;
    m_statstg.clsid 	 	= CLSID_NULL;
    m_statstg.grfStateBits 	= 0;
    m_statstg.reserved 	 	= 0;

    m_parent->AddRef();

    AddRef();
}

/**
 * constructor (clone)
 */

AwtDragSource::ADSIStreamProxy::ADSIStreamProxy(ADSIStreamProxy* cloneof) {
    m_cloneof = cloneof;

    m_parent  = cloneof->m_parent;

    m_buffer  = cloneof->m_buffer;
    m_blen    = cloneof->m_blen;
    m_off     = cloneof->m_off;

    m_statstg = cloneof->m_statstg;

    m_refs    = 0;

    m_parent->AddRef();
    m_cloneof->AddRef();
}

/**
 * destructor
 */

AwtDragSource::ADSIStreamProxy::~ADSIStreamProxy() {
    if (m_cloneof == (ADSIStreamProxy*)NULL)
	free((void *)m_buffer);
    else {
	m_cloneof->Release();
    }

    m_parent->Release();
}

/**
 * QueryInterface
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject) {
    TRY;

    if (riid == IID_IUnknown) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IUnknown*)this;
	AddRef();
	return S_OK;
    } else if (riid == IID_IStream) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IStream*)this;
	AddRef();
	return S_OK;
    } else {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)NULL;
	return E_NOINTERFACE;
    }

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * AddRef
 */

ULONG __stdcall  AwtDragSource::ADSIStreamProxy::AddRef(void) {
    return (ULONG)++m_refs;
}

/**
 * Release
 */

ULONG __stdcall  AwtDragSource::ADSIStreamProxy::Release(void) {
    int refs;

    if ((refs = --m_refs) == 0) delete this;

    return (ULONG)refs;
}

/**
 * Read
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Read(void __RPC_FAR *pv, ULONG cb, ULONG __RPC_FAR *pcbRead) {
    TRY;

    unsigned int rem  = m_blen - m_off;
    int          read = cb > rem ? rem : cb;
    
    if (read > 0) memmove(pv, (void *)(m_buffer + m_off), read);

    m_off += read;

    if (pcbRead != (ULONG __RPC_FAR *)NULL) {
	*pcbRead = read;
    }

    FILETIME now; ::CoFileTimeNow(&now); m_statstg.atime = now;

    return S_OK;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Write
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Write(const void __RPC_FAR *pv, ULONG cb, ULONG __RPC_FAR *pcbWritten) {
    TRY;

    if (pcbWritten != (ULONG __RPC_FAR *)NULL) {
	*pcbWritten = 0;
    }

    FILETIME now; ::CoFileTimeNow(&now); m_statstg.atime = now;

    return STG_E_CANTSAVE; // dont support writing

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Seek
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Seek(LARGE_INTEGER dlibMove, DWORD dwOrigin, ULARGE_INTEGER __RPC_FAR *plibNewPosition) {
    TRY;
  
    if (dlibMove.HighPart != 0) return STG_E_INVALIDPOINTER;

    if (plibNewPosition != (ULARGE_INTEGER __RPC_FAR *)NULL) {
	plibNewPosition->HighPart = 0;
	plibNewPosition->LowPart  = 0;
    }

    switch (dwOrigin) {
	case STREAM_SEEK_SET: {
	    if (dlibMove.HighPart != 0 || dlibMove.LowPart >= m_blen) return STG_E_INVALIDPOINTER;

	    m_off = dlibMove.LowPart;
	}
	break;

	case STREAM_SEEK_CUR:
	case STREAM_SEEK_END: {
	    if (dlibMove.HighPart > 0) return STG_E_INVALIDPOINTER;

	    long newoff = (dwOrigin == STREAM_SEEK_END ? m_blen : m_off) + dlibMove.LowPart;

	    if (newoff < 0 || newoff >= (long)m_blen)
		return STG_E_INVALIDPOINTER;
	    else
		m_off = newoff;
	}
	break;

	default: return STG_E_INVALIDFUNCTION;
    }

    if (plibNewPosition != (ULARGE_INTEGER __RPC_FAR *)NULL)
	plibNewPosition->LowPart = m_off;

    FILETIME now; ::CoFileTimeNow(&now); m_statstg.atime = now;

    return S_OK;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * SetSize
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::SetSize(ULARGE_INTEGER libNewSize) {
    return STG_E_INVALIDFUNCTION;
}

/**
 * CopyTo
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::CopyTo(IStream __RPC_FAR *pstm, ULARGE_INTEGER cb, ULARGE_INTEGER __RPC_FAR *pcbRead, ULARGE_INTEGER __RPC_FAR *pcbWritten) {
    TRY;

    ULONG written = 0;

    pcbWritten->HighPart = (ULONG)0;
    pcbWritten->LowPart  = (ULONG)0;

    pcbRead->HighPart     = (ULONG)0;

    unsigned int rem     = m_blen - m_off;
    int          ovrflow = cb.LowPart >= rem;


    if (cb.HighPart != 0) return STG_E_INVALIDPOINTER;

    ULONG nbytes = pcbRead->LowPart = (ULONG)(ovrflow ? rem : cb.LowPart);

    HRESULT res = pstm->Write((const void *)(m_buffer + m_off), nbytes, &written);

    pcbWritten->LowPart = written;

    FILETIME now; ::CoFileTimeNow(&now); m_statstg.atime = now;

    return res;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Commit
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Commit(DWORD grfCommitFlags) {
    return S_OK;
}

/**
 * Revert
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Revert() {
    return S_OK;
}

/**
 * LockRegion
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::LockRegion(ULARGE_INTEGER libOffset, ULARGE_INTEGER cb, DWORD dwLockType) {
    return STG_E_INVALIDFUNCTION;
}

/**
 * UnlockRegion
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::UnlockRegion(ULARGE_INTEGER libOffset, ULARGE_INTEGER cb, DWORD dwLockType) {
    return STG_E_INVALIDFUNCTION;
}

/**
 * Stat
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Stat(STATSTG __RPC_FAR *pstatstg, DWORD grfStatFlag) {
    TRY;

    *pstatstg = m_statstg;

    FILETIME now; ::CoFileTimeNow(&now); m_statstg.atime = now;

    return S_OK;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Clone
 */

HRESULT __stdcall  AwtDragSource::ADSIStreamProxy::Clone(IStream __RPC_FAR *__RPC_FAR *ppstm) {
    TRY;

    *ppstm = new ADSIStreamProxy(this);
    return S_OK;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/*****************************************************************************/

extern "C" {

/**
 * Initialize
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDragSourceContextPeer_Initialize(JNIEnv* env, jclass cls) {
    TRY;

    AwtDragSource::InitAwtDragSource();

    CATCH_BAD_ALLOC;
}

/**
 * createDragSource
 */

JNIEXPORT jlong JNICALL
Java_sun_awt_windows_WDragSourceContextPeer_createDragSource(
    JNIEnv* env, jobject self, jobject trigger, jint actions,
    jintArray formats, jobject formatMap)
{
    TRY;

    AwtDragSource* ds = new AwtDragSource(env, self, trigger, actions,
					  formats, formatMap);
    ::CoLockObjectExternal((IDropSource*)ds, TRUE, FALSE);

    return (jlong)ds;

    CATCH_BAD_ALLOC_RET(0);
}

/**
 * doDragDrop
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDragSourceContextPeer_doDragDrop(JNIEnv* env, jobject self, jlong nativeCtxt, jobject cursor) {
    TRY;

    AwtDragSource::StartDrag((AwtDragSource*)nativeCtxt, cursor);

    CATCH_BAD_ALLOC;
}

/**
 * setNativeCursor
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDragSourceContextPeer_setNativeCursor(JNIEnv* env, jclass self, jlong nativeCtxt, jobject cursor) {
    TRY;

    ((AwtDragSource*)nativeCtxt)->SetCursor(cursor);

    CATCH_BAD_ALLOC;
}

/**
 * Release
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDragSourceContextPeer_release(JNIEnv* env, jobject self, jlong nativeCtxt) { 
    TRY_NO_VERIFY;

    AwtDragSource* dragSource = (AwtDragSource*)nativeCtxt;
    
    ::CoLockObjectExternal((IDropSource*)dragSource, FALSE, FALSE);

    dragSource->Release();

    CATCH_BAD_ALLOC;
}

} /* extern "C" */
