/*
 * @(#)awt_DnDDT.cpp	1.31 01/02/09
 *
 * Copyright 1997-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt.h"
#include "awt_DnDDT.h"
#include "awt_DnDDS.h"
#include "awt_Toolkit.h"
#include "awt_dlls.h"
#include "awt_Container.h"

#include <java_awt_dnd_DnDConstants.h>
#include <sun_awt_dnd_SunDropTargetContextPeer.h>
#include <sun_awt_windows_WDropTargetContextPeer.h>

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <shellapi.h>

// forwards

extern "C" {
DWORD __cdecl convertActionsToDROPEFFECT(jint actions);
jint  __cdecl convertDROPEFFECTToActions(DWORD effects);

DWORD __cdecl mapModsToDROPEFFECT(DWORD, DWORD);
} // extern "C"


/**
 * static initialization
 */

jclass AwtDropTarget::dTCClazz = (jclass)NULL;

jclass AwtDropTarget::file   = (jclass)NULL;

jmethodID AwtDropTarget::dTCgetp   = (jmethodID)NULL;
jmethodID AwtDropTarget::dTCenter  = (jmethodID)NULL;
jmethodID AwtDropTarget::dTCexit   = (jmethodID)NULL;
jmethodID AwtDropTarget::dTCmotion = (jmethodID)NULL;
jmethodID AwtDropTarget::dTCdrop   = (jmethodID)NULL;
 
jmethodID AwtDropTarget::dTCgetfs  = (jmethodID)NULL;
jmethodID AwtDropTarget::dTCgetis  = (jmethodID)NULL;
jmethodID AwtDropTarget::dTCfilei  = (jmethodID)NULL;

void AwtDropTarget::InitAwtDropTarget() {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    
    /*
     * load class java.io.File
     */
    
    file = env->FindClass("java/io/File");
    
    if (JNU_IsNull(env, AwtDropTarget::file)) {
        env->ThrowNew(env->FindClass("java/lang/ClassNotFoundException"),
    		      "Cant find java/io/File"
        );
    }
    
    file = (jclass)env->NewGlobalRef(file); /* never freed! */
    
    dTCfilei = env->GetMethodID(file,
				"<init>",
				"(Ljava/lang/String;)V"
	       );

    if (JNU_IsNull(env, dTCfilei)) return;

    /*
     * load class sun.awt.motif.WDropTargetContextPeer and resolve methods
     */
    
    dTCClazz = env->FindClass("sun/awt/windows/WDropTargetContextPeer");
    
    if (JNU_IsNull(env, dTCClazz)) {
        env->ThrowNew(env->FindClass("java/lang/ClassNotFoundException"),
    		      "Cant find sun/awt/windows/WDropTargetContextPeer"
        );
    }
    
    dTCClazz = (jclass)env->NewGlobalRef(dTCClazz); /* never freed! */
    
    if (JNU_IsNull(env, dTCClazz)) return;
    
    dTCgetp = env->GetStaticMethodID(dTCClazz,
    				    "getWDropTargetContextPeer",
    				    "()Lsun/awt/windows/WDropTargetContextPeer;"
    	        );
    
    if (JNU_IsNull(env, dTCgetp)) return;
    
    dTCenter = env->GetMethodID(dTCClazz,
    			        "handleEnterMessage",
				"(Ljava/awt/Component;IIII[IJ)I"
    	       );
    
    if (JNU_IsNull(env, dTCenter)) return;
    
    dTCexit = env->GetMethodID(dTCClazz,
    			       "handleExitMessage",
    			       "(Ljava/awt/Component;J)V"
    	      );
    
    
    if (JNU_IsNull(env, dTCexit)) return;
    
    dTCmotion = env->GetMethodID(dTCClazz,
    			         "handleMotionMessage",
				 "(Ljava/awt/Component;IIII[IJ)I"
    	        );
    
    
    if (JNU_IsNull(env, dTCmotion)) return;
    
    dTCdrop = env->GetMethodID(dTCClazz,
    			       "handleDropMessage",
			       "(Ljava/awt/Component;IIII[IJ)V"
    	      );
    
    if (JNU_IsNull(env, dTCdrop)) return;

    dTCgetfs = env->GetStaticMethodID(dTCClazz,
				      "getFileStream",
				      "(Ljava/lang/String;I)Ljava/io/FileInputStream;"
   	       );

    if (JNU_IsNull(env, dTCgetfs)) return;

    dTCgetis = env->GetStaticMethodID(dTCClazz,
				      "getIStream",
				      "(I)Ljava/lang/Object;"
   	       );

    if (JNU_IsNull(env, dTCgetis)) return;
}

/**
 * constructor
 */

AwtDropTarget::AwtDropTarget(JNIEnv* env, AwtComponent* component) {

    m_component     = component;
    m_window	    = component->GetHWnd();
    m_refs          = 0U;
    m_target	    = env->NewGlobalRef(component->GetTarget(env));
    m_registered    = 0;
    m_dataObject    = NULL;
    m_enumFormatEtc = NULL;
    m_formats	    = NULL;
    m_nformats	    = 0;
    m_dtcp    	    = NULL;
    m_cfFormats	    = NULL;
    m_mutex  	    = ::CreateMutex(NULL, FALSE, NULL);

    AddRef();
}

/**
 * destructor
 */

AwtDropTarget::~AwtDropTarget() {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    env->DeleteGlobalRef(m_target);
    env->DeleteGlobalRef(m_dtcp);

    ::CloseHandle(m_mutex);

    UnloadCache();
}

/**
 * QueryInterface
 */

HRESULT __stdcall AwtDropTarget::QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject) {
    TRY;

    if (riid == IID_IUnknown) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IUnknown*)this;
	AddRef();
	return S_OK;
    } else if (riid == IID_IDropTarget) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IDropTarget*)this;
	AddRef();
	return S_OK;
    } else if (riid == IID_IAdviseSink) {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)(IAdviseSink*)this;
	AddRef();
	return S_OK;
    } else {
	*ppvObject = (void __RPC_FAR *__RPC_FAR)NULL;
	return E_NOINTERFACE;
    }

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * AddRef
 */

ULONG __stdcall AwtDropTarget::AddRef() {
    return (ULONG)++m_refs;
}

/**
 * Release
 */

ULONG __stdcall AwtDropTarget::Release() {
    int refs;

    if ((refs = --m_refs) == 0) delete this;

    return (ULONG)refs;
}

/**
 * DragEnter
 */

HRESULT __stdcall AwtDropTarget::DragEnter(IDataObject __RPC_FAR *pDataObj, DWORD grfKeyState, POINTL pt, DWORD __RPC_FAR *pdwEffect) {
    TRY;

    JNIEnv*    env       = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    HRESULT    ret       = S_OK;
    DWORD      retEffect = DROPEFFECT_NONE;
    jobject    dtcp = NULL;

    dtcp = env->CallStaticObjectMethod(dTCClazz, dTCgetp);
    if (dtcp) {
	env->DeleteGlobalRef(m_dtcp);
	m_dtcp = env->NewGlobalRef(dtcp);
	env->DeleteLocalRef(dtcp);
    }

    if (JNU_IsNull(env, m_dtcp) || !JNU_IsNull(env, safe_ExceptionOccurred(env))) {
	return ret;
    }

    LoadCache(pDataObj);

    if (m_nformats > 0) {
        POINT cp;
	RECT  wr;

	::GetWindowRect(m_window, &wr);
	
    	cp.x = pt.x - wr.left;
	cp.y = pt.y - wr.top;

	jint actions = env->CallIntMethod(m_dtcp,
			 	          dTCenter, 
					  m_target,
					  (jint)cp.x,
					  (jint)cp.y,
					  ::convertDROPEFFECTToActions(mapModsToDROPEFFECT(*pdwEffect, grfKeyState)),
					  ::convertDROPEFFECTToActions(*pdwEffect),
					  m_cfFormats,
					  (jlong)this
		        );

	try {
	    if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
	        env->ExceptionDescribe();
		env->ExceptionClear();
		actions = java_awt_dnd_DnDConstants_ACTION_NONE;
	    }
	} catch (std::bad_alloc&) {
	    retEffect = ::convertActionsToDROPEFFECT(actions);
	    *pdwEffect = retEffect;
	    throw;
	}

	retEffect = ::convertActionsToDROPEFFECT(actions);
    }

    *pdwEffect = retEffect;

    return ret;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * DragOver
 */

HRESULT __stdcall AwtDropTarget::DragOver(DWORD grfKeyState, POINTL pt, DWORD __RPC_FAR *pdwEffect) {
    TRY;

    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    HRESULT ret = S_OK;
    POINT   cp;
    RECT    wr;
    jint    actions;
	
    ::GetWindowRect(m_window, &wr);

    cp.x = pt.x - wr.left;
    cp.y = pt.y - wr.top;

    actions = env->CallIntMethod(m_dtcp,
			      	 dTCmotion,
				 m_target,
			         (jint)cp.x,
			         (jint)cp.y,
				 ::convertDROPEFFECTToActions(mapModsToDROPEFFECT(*pdwEffect, grfKeyState)),
			         ::convertDROPEFFECTToActions(*pdwEffect),
			         m_cfFormats,
			         (jlong)this
	      );

    try {
        if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
	    env->ExceptionDescribe();
	    env->ExceptionClear();
	    actions = java_awt_dnd_DnDConstants_ACTION_NONE;
	}
    } catch (std::bad_alloc&) {
        *pdwEffect = ::convertActionsToDROPEFFECT(actions);
	throw;
    }

    *pdwEffect = ::convertActionsToDROPEFFECT(actions);

    return ret;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * DragLeave
 */

HRESULT __stdcall AwtDropTarget::DragLeave() {
    TRY_NO_VERIFY;

    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    HRESULT ret = S_OK;
    
    env->CallVoidMethod(m_dtcp, dTCexit, m_target, (jlong)this);

    try {
        if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
	    env->ExceptionDescribe();
	    env->ExceptionClear();
	}
    } catch (std::bad_alloc&) {
        DragCleanup();
	throw;
    }

    DragCleanup();

    return ret;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * Drop
 */

HRESULT __stdcall AwtDropTarget::Drop(IDataObject __RPC_FAR *pDataObj, DWORD grfKeyState, POINTL pt, DWORD __RPC_FAR *pdwEffect) {
    TRY;

    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    HRESULT ret = S_OK;
    POINT   cp;
    RECT    wr;
	
    LoadCache(pDataObj);

    if (m_nformats == 0) {
	*pdwEffect = DROPEFFECT_NONE;
	DragCleanup();
	return ret;
    }

    ::GetWindowRect(m_window, &wr);

    cp.x = pt.x - wr.left;
    cp.y = pt.y - wr.top;

    m_dropActions = java_awt_dnd_DnDConstants_ACTION_NONE;

    env->CallVoidMethod(m_dtcp,
		        dTCdrop,
		        m_target,
		        (jint)cp.x,
			(jint)cp.y,
			::convertDROPEFFECTToActions(mapModsToDROPEFFECT(*pdwEffect, grfKeyState)),
			::convertDROPEFFECTToActions(*pdwEffect),
			m_cfFormats,
			(jlong)this
    );

    try {
        if (!JNU_IsNull(env, safe_ExceptionOccurred(env))) {
	    env->ExceptionDescribe();
	    env->ExceptionClear();
	    ret = E_FAIL;
	}
    } catch (std::bad_alloc&) {
        AwtToolkit::GetInstance().MessageLoop();
	*pdwEffect = ::convertActionsToDROPEFFECT(m_dropActions);
	DragCleanup();
	throw;
    }

    AwtToolkit::GetInstance().MessageLoop();

    *pdwEffect = ::convertActionsToDROPEFFECT(m_dropActions);

    DragCleanup();

    return ret;

    CATCH_BAD_ALLOC_RET(E_OUTOFMEMORY);
}

/**
 * DoDropDone
 */

void AwtDropTarget::DoDropDone(jint actions) {
    DropDoneRec ddr = { this, actions };

    AwtToolkit::GetInstance().InvokeFunction(_DropDone, &ddr);
}

/**
 * _DropDone
 */

void AwtDropTarget::_DropDone(void* param) {
    DropDonePtr ddrp = (DropDonePtr)param;

    (ddrp->dropTarget)->DropDone(ddrp->action);
}

/**
 * DropDone
 */

void AwtDropTarget::DropDone(jint actions) {
    m_dropActions = actions;
    AwtToolkit::GetInstance().QuitMessageLoop(AwtToolkit::EXIT_ENCLOSING_LOOP);
}

/**
 * DoRegisterTarget
 */

void AwtDropTarget::_RegisterTarget(void* param) {
    RegisterTargetPtr rtrp = (RegisterTargetPtr)param;

    rtrp->dropTarget->RegisterTarget(rtrp->show);
}

/**
 * RegisterTarget
 */

void AwtDropTarget::RegisterTarget(WORD show) {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    HRESULT res;
 
    if (!AwtToolkit::IsMainThread()) {
	RegisterTargetRec rtr = { this, show };

	AwtToolkit::GetInstance().InvokeFunction(_RegisterTarget, &rtr);

	return;
    }

    if (show) {
        res = ::RegisterDragDrop(m_window, (IDropTarget*)this);
    } else {
	res = ::RevokeDragDrop(m_window);
    }

    if (res == S_OK) m_registered = show;
}

/**
 * DoGetData
 */

jobject AwtDropTarget::DoGetData(jint format) {
    jobject    ret = (jobject)NULL;
    GetDataRec gdr = { this, format, &ret };

    AwtToolkit::GetInstance().WaitForSingleObject(m_mutex);

    AwtToolkit::GetInstance().InvokeFunctionLater(_GetData, &gdr);

    WaitUntilSignalled(FALSE);

    return ret;
}

/**
 * _GetData
 */

void AwtDropTarget::_GetData(void* param) {
    GetDataPtr gdrp = (GetDataPtr)param;

    *(gdrp->ret) = gdrp->dropTarget->GetData(gdrp->format);

    gdrp->dropTarget->Signal();
}


/**
 * GetData
 *
 * Returns the data object being transferred.
 */

jobject AwtDropTarget::GetData(jint fmt) {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(1) < 0) {
        return (jobject)NULL;
    }

    FORMATETC format = { (unsigned short)fmt };

    FORMATETC *cpp = (FORMATETC *)bsearch((const void *)&format,
					  (const void *)m_formats,
					  (size_t)      m_nformats,
					  (size_t)      sizeof(FORMATETC),
					                _compar
					  );

    if (cpp == (FORMATETC *)NULL) {
	return (jobject)NULL;
    }

    memcpy(&format, cpp, sizeof(FORMATETC));
    STGMEDIUM stgmedium;

    format.tymed |= TYMED_HGLOBAL | TYMED_FILE | TYMED_ISTREAM;

    if (m_dataObject->GetData(&format, &stgmedium) != S_OK) {
	return (jobject)NULL;
    }

    switch (stgmedium.tymed) {
	case TYMED_HGLOBAL: {
	    DWORD size = ::GlobalSize(stgmedium.hGlobal);
	    if (size == 0 && ::GetLastError() != 0) {
	        ::SetLastError(0); // clear error
		::ReleaseStgMedium(&stgmedium);
		return (jobject)NULL; // failed
	    }

	    jbyteArray bytes = env->NewByteArray(size);
	    if (bytes == NULL) {
		::ReleaseStgMedium(&stgmedium);
		throw std::bad_alloc();
	    }

	    LPVOID data = ::GlobalLock(stgmedium.hGlobal);
	    env->SetByteArrayRegion(bytes, 0, size, (jbyte *)data);
	    ::GlobalUnlock(stgmedium.hGlobal);
	    ::ReleaseStgMedium(&stgmedium);

	    jobject global = env->NewGlobalRef(bytes);
	    env->DeleteLocalRef(bytes);
	    return global;
	}

	case TYMED_FILE: {
	    jobject local = JNU_NewStringPlatform(env, (const char *)
						  stgmedium.lpszFileName);
	    jstring fileName = (jstring)env->NewGlobalRef(local);
	    env->DeleteLocalRef(local);

	    STGMEDIUM *stgm = NULL;
	    try {
	        stgm = (STGMEDIUM *)safe_Malloc(sizeof(STGMEDIUM));
	    } catch (std::bad_alloc&) {
	        env->DeleteGlobalRef(fileName);
		::ReleaseStgMedium(&stgmedium);
		throw;
	    }
	    memcpy(stgm, &stgmedium, sizeof(STGMEDIUM));

	    local = env->CallStaticObjectMethod(dTCClazz, dTCgetfs, fileName,
						stgm);
	    jobject stream = env->NewGlobalRef(local);
	    env->DeleteLocalRef(local);

	    try {
	        if (JNU_IsNull(env, stream) ||
		        !JNU_IsNull(env, safe_ExceptionOccurred(env))) {
		    env->DeleteGlobalRef(fileName);
		    free((void*)stgm);
		    ::ReleaseStgMedium(&stgmedium);
		    return (jobject)NULL;
		}
	    } catch (std::bad_alloc&) {
	        env->DeleteGlobalRef(fileName);
		free((void*)stgm);
		::ReleaseStgMedium(&stgmedium);
		throw;
	    }

	    return stream;
	}

	case TYMED_ISTREAM: {
	   WDTCPIStreamWrapper* istream = new WDTCPIStreamWrapper(&stgmedium);

	   jobject local = env->CallStaticObjectMethod(dTCClazz, dTCgetis,
						       (jint)istream);
	   jobject stream = env->NewGlobalRef(local);
	   env->DeleteLocalRef(local);

	   try {
	       if (JNU_IsNull(env, stream) ||
		       !JNU_IsNull(env, safe_ExceptionOccurred(env))) {
		   istream->Close();
		   return (jobject)NULL;
	       }
	   } catch (std::bad_alloc&) {
	       istream->Close();
	       throw;
	   }

	   return stream;
	}

	case TYMED_GDI:
	case TYMED_MFPICT:
	case TYMED_ISTORAGE:
	case TYMED_ENHMF:
	default:
	    ::ReleaseStgMedium(&stgmedium);
	    return (jobject)NULL;
    }
}

/**
 *
 */

int __cdecl AwtDropTarget::_compar(const void* first, const void* second) {
    FORMATETC *fp = (FORMATETC *)first;
    FORMATETC *sp = (FORMATETC *)second;

    return fp->cfFormat - sp->cfFormat;
}

const unsigned int AwtDropTarget::CACHE_INCR = 16;

void AwtDropTarget::LoadCache(IDataObject* pDataObj) {
    JNIEnv*      env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    unsigned int cnt = 0;
    HRESULT  	 res;

    if (m_dataObject != (IDataObject*)NULL) UnloadCache();

    (m_dataObject = pDataObj)->AddRef();

    res = m_dataObject->EnumFormatEtc(DATADIR_GET, &m_enumFormatEtc);

    if (res != S_OK) {
        m_dataObject->Release();
        m_dataObject    = (IDataObject*)NULL;
        m_enumFormatEtc = (IEnumFORMATETC*)NULL;
        return;
    }

    for (;;) {
	
	FORMATETC tmp;
	ULONG	  actual = 1;

	res = m_enumFormatEtc->Next((ULONG)1, &tmp, &actual);

	if (res == S_FALSE) break;

	if (!(tmp.cfFormat  >= 1                &&
	      tmp.ptd       == NULL	        &&
	      tmp.lindex    == -1   	        &&
	      tmp.dwAspect  == DVASPECT_CONTENT &&
	      (tmp.tymed    == TYMED_HGLOBAL    ||
	       tmp.tymed    == TYMED_FILE       ||
	       tmp.tymed    == TYMED_ISTREAM
	      ) // but not ISTORAGE , GDI, MFPICT, ENHMF
	     )
 	) continue;

	if (m_dataObject->QueryGetData(&tmp) != S_OK) continue;

	if (m_nformats % CACHE_INCR == 0) {
	    m_formats = (FORMATETC *)safe_Realloc(m_formats,
						  (CACHE_INCR + m_nformats) *
						  sizeof(FORMATETC));
	}

	memcpy(m_formats + m_nformats, &tmp, sizeof(FORMATETC));

	m_nformats++;
    }

    if (m_nformats > 0) {
	qsort((void*)m_formats, m_nformats, sizeof(FORMATETC),
	      AwtDropTarget::_compar);
    }

    if (m_cfFormats != NULL) {
        env->DeleteGlobalRef(m_cfFormats);
    }
    jintArray l_cfFormats = env->NewIntArray(m_nformats);
    if (l_cfFormats == NULL) {
        throw std::bad_alloc();
    }
    m_cfFormats = (jintArray)env->NewGlobalRef(l_cfFormats);
    env->DeleteLocalRef(l_cfFormats);

    jboolean isCopy;
    jint *icfFormats = env->GetIntArrayElements(m_cfFormats, &isCopy),
        *saveFormats = icfFormats;

    for (unsigned int i = 0; i < m_nformats; i++, icfFormats++) {
        *icfFormats = m_formats[i].cfFormat;
    }

    env->ReleaseIntArrayElements(m_cfFormats, saveFormats, 0);
}

/**
 * UnloadCache
 */

void AwtDropTarget::UnloadCache() {
    if (m_dataObject == (IDataObject*)NULL) return;

    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    free((void*)m_formats);
    m_formats  = (FORMATETC *)NULL;
    m_nformats = 0;

    env->DeleteGlobalRef(m_cfFormats);
    m_cfFormats = NULL;

    m_enumFormatEtc->Release();
    m_enumFormatEtc = (IEnumFORMATETC*)NULL;

    m_dataObject->Release();
    m_dataObject = (IDataObject*)NULL;
}

/**
 * DragCleanup
 */

void AwtDropTarget::DragCleanup(void) {
    UnloadCache();
}

/*****************************************************************************/

jclass WDTCPIStreamWrapper::javaIOExceptionClazz = (jclass)NULL;

/**
 * construct a wrapper
 */

WDTCPIStreamWrapper::WDTCPIStreamWrapper(STGMEDIUM* stgmedium) {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    m_stgmedium = *stgmedium;
    m_istream   = stgmedium->pstm;
    m_mutex     = ::CreateMutex(NULL, FALSE, NULL);

    if (javaIOExceptionClazz == (jclass)NULL) {
	javaIOExceptionClazz = env->FindClass("java/io/IOException");

        if (JNU_IsNull(env, javaIOExceptionClazz)) {
            env->ThrowNew(env->FindClass("java/lang/ClassNotFoundException"),
    		          "Cant find java/io/IOException"
            );
    	}
    }
}

/**
 * destroy a wrapper
 */

WDTCPIStreamWrapper::~WDTCPIStreamWrapper() {
    ::CloseHandle(m_mutex);

    ::ReleaseStgMedium(&m_stgmedium);
}

/**
 * return available data
 */

jint WDTCPIStreamWrapper::DoAvailable(WDTCPIStreamWrapper* istream) {
    WDTCPIStreamWrapperRec iswr = { istream, 0 };

    AwtToolkit::GetInstance().WaitForSingleObject(istream->m_mutex);

    AwtToolkit::GetInstance().InvokeFunctionLater( _Available, &iswr);

    istream->WaitUntilSignalled(FALSE);

    return iswr.ret;
}

/**
 * return available data
 */

void WDTCPIStreamWrapper::_Available(void *param) {
    WDTCPIStreamWrapperPtr iswrp = (WDTCPIStreamWrapperPtr)param;

    iswrp->ret = (iswrp->istream)->Available();

    iswrp->istream->Signal();
}

/**
 * return available data
 */

jint WDTCPIStreamWrapper::Available() {
    JNIEnv* env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (m_istream->Stat(&m_statstg, STATFLAG_NONAME) != S_OK) {
	env->ThrowNew(javaIOExceptionClazz, "IStream::Stat() failed");
	return 0;
    }

    if (m_statstg.cbSize.QuadPart > 0x7ffffffL) {
	env->ThrowNew(javaIOExceptionClazz, "IStream::Stat() cbSize > 0x7ffffff");
	return 0;
    }

    return (jint)m_statstg.cbSize.LowPart;
}

/**
 * read 1 byte
 */

jint WDTCPIStreamWrapper::DoRead(WDTCPIStreamWrapper* istream) {
    WDTCPIStreamWrapperRec iswr = { istream, 0 };

    AwtToolkit::GetInstance().WaitForSingleObject(istream->m_mutex);

    AwtToolkit::GetInstance().InvokeFunctionLater(_Read, &iswr);

    istream->WaitUntilSignalled(FALSE);

    return iswr.ret;
}

/**
 * read 1 byte
 */

void WDTCPIStreamWrapper::_Read(void* param) {
    WDTCPIStreamWrapperPtr iswrp = (WDTCPIStreamWrapperPtr)param;

    iswrp->ret = (iswrp->istream)->Read();

    iswrp->istream->Signal();
}

/**
 * read 1 byte
 */

jint WDTCPIStreamWrapper::Read() {
    JNIEnv* env    = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jint    b      = 0;
    ULONG   actual = 0;
    HRESULT res;

    switch (res = m_istream->Read((void *)&b, (ULONG)1, &actual)) {
	case S_FALSE:
	    return (jint)-1;

	case S_OK:
	    return (jint)(actual == 0 ? -1 : b);

	default:
	    env->ThrowNew(javaIOExceptionClazz, "IStream::Read failed");
    }
    return (jint)-1;
}

/**
 * read Buffer
 */

jint WDTCPIStreamWrapper::DoReadBytes(WDTCPIStreamWrapper* istream, jbyteArray array, jint off, jint len) {
    WDTCPIStreamWrapperReadBytesRec iswrbr = { istream, 0, array, off, len };

    AwtToolkit::GetInstance().WaitForSingleObject(istream->m_mutex);

    AwtToolkit::GetInstance().InvokeFunctionLater(_ReadBytes, &iswrbr);

    istream->WaitUntilSignalled(FALSE);

    return iswrbr.ret;
}

/**
 * read buffer
 */

void WDTCPIStreamWrapper::_ReadBytes(void*  param) {
    WDTCPIStreamWrapperReadBytesPtr iswrbrp =
        (WDTCPIStreamWrapperReadBytesPtr)param;

    iswrbrp->ret = (iswrbrp->istream)->ReadBytes(iswrbrp->array,
						 iswrbrp->off,
						 iswrbrp->len);
    iswrbrp->istream->Signal();
}

/**
 * read buffer
 */

jint WDTCPIStreamWrapper::ReadBytes(jbyteArray buf, jint off, jint len) {
    JNIEnv*  env     = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jboolean isCopy  = JNI_FALSE;
    ULONG    actual  = 0;
    jbyte*   local   = env->GetByteArrayElements(buf, &isCopy);
    HRESULT  res;

    switch (res = m_istream->Read((void *)(local + off), (ULONG)len, &actual)) {
	case S_FALSE:
	case S_OK: {
	    int eof = (actual == 0);

       	    env->ReleaseByteArrayElements(buf, local, !eof ? 0 : JNI_ABORT);
	    return (jint)(!eof ? actual : -1);
	}

	default:
       	    env->ReleaseByteArrayElements(buf, local, JNI_ABORT);
	    env->ThrowNew(javaIOExceptionClazz, "IStream::Read failed");
    }

    return (jint)-1;
}

/**
 * close
 */

void WDTCPIStreamWrapper::DoClose(WDTCPIStreamWrapper* istream) {
    AwtToolkit::GetInstance().InvokeFunctionLater(_Close, istream);
}

/**
 * close
 */

void WDTCPIStreamWrapper::_Close(void* param) {
    ((WDTCPIStreamWrapper*)param)->Close();
}

/**
 * close
 */

void WDTCPIStreamWrapper::Close() {
    delete this;
}

/*****************************************************************************/

extern "C" {

/**
 * awt_dnd_initialize: initial DnD system
 */

void awt_dnd_initialize() {
    ::OleInitialize((LPVOID)NULL);
}

/**
 * awt_dnd_uninitialize: deactivate DnD system
 */

void awt_dnd_uninitialize() {
    ::OleUninitialize();
}

/**
 * convertActionsToDROPEFFECT
 */

DWORD convertActionsToDROPEFFECT(jint actions) {
    DWORD effects = DROPEFFECT_NONE;

    if (actions & java_awt_dnd_DnDConstants_ACTION_LINK) effects |= DROPEFFECT_LINK;
    if (actions & java_awt_dnd_DnDConstants_ACTION_MOVE) effects |= DROPEFFECT_MOVE;
    if (actions & java_awt_dnd_DnDConstants_ACTION_COPY) effects |= DROPEFFECT_COPY;
    return effects;
}

/**
 * convertDROPEFFECTToAction
 */

jint convertDROPEFFECTToActions(DWORD effects) {
    jint actions = java_awt_dnd_DnDConstants_ACTION_NONE;

    if (effects & DROPEFFECT_LINK) actions |= java_awt_dnd_DnDConstants_ACTION_LINK;
    if (effects & DROPEFFECT_MOVE) actions |= java_awt_dnd_DnDConstants_ACTION_MOVE;
    if (effects & DROPEFFECT_COPY) actions |= java_awt_dnd_DnDConstants_ACTION_COPY;

    return actions;
}

/**
 * map keyboard modifiers to a DROPEFFECT
 */

DWORD mapModsToDROPEFFECT(DWORD effects, DWORD mods) {
    DWORD ret = DROPEFFECT_NONE;

    switch (mods & (MK_CONTROL | MK_SHIFT)) {
	case MK_CONTROL:
	    ret = DROPEFFECT_COPY;
	break;

	case MK_CONTROL | MK_SHIFT:
	    ret = DROPEFFECT_LINK;
	break;

	default:
	    ret = DROPEFFECT_MOVE;
    }

    return ret & effects;
}

/**
 *
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDropTargetContextPeer_Initialize(JNIEnv* env, jclass cls) {
    TRY;

    AwtDropTarget::InitAwtDropTarget();

    CATCH_BAD_ALLOC;
}

/**
 * downcall to fetch data ... gets scheduled on message thread
 */

JNIEXPORT jobject JNICALL Java_sun_awt_windows_WDropTargetContextPeer_getData(JNIEnv* env, jobject self, jlong dropTarget, jint format) {
    TRY;

    return ((AwtDropTarget*)dropTarget)->DoGetData(format);

    CATCH_BAD_ALLOC_RET(NULL);
}

/**
 * downcall to signal drop done ... gets scheduled on message thread
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDropTargetContextPeer_dropDone(JNIEnv* env, jobject self, jlong dropTarget, jint actions) {
    TRY_NO_VERIFY;

    ((AwtDropTarget*)dropTarget)->DoDropDone(actions);

    CATCH_BAD_ALLOC;
}

JNIEXPORT jobject JNICALL
Java_sun_awt_dnd_SunDropTargetContextPeer_findComponentAt(
    JNIEnv *env, jobject, jobject container, jint x, jint y)
{
    TRY;
        
    /*
     * Call private version of Container.findComponentAt with the following
     * flags set -- ignoreEnabled = true (i.e., okay to return or recur into
     * disabled Components); ignoreGlassPane = true (i.e., don't return
     * a JRootPane's glass pane as the target Component)
     */
    JNI_CHECK_NULL_RETURN_NULL(container, "null container");
    jobject comp =
        env->CallObjectMethod(container, AwtContainer::findComponentAtMID,
                              x, y, JNI_TRUE, JNI_TRUE);
    return comp;

    CATCH_BAD_ALLOC_RET(NULL);
}

/**
 * downcall to free up storage medium for FileStream
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDropTargetContextPeerFileStream_freeStgMedium(JNIEnv* env, jobject self, jint stgmedium) {
    TRY;

    ::ReleaseStgMedium((STGMEDIUM*)stgmedium);

    free((void*)stgmedium);

    CATCH_BAD_ALLOC;
}

/**
 * 
 */

JNIEXPORT jint JNICALL Java_sun_awt_windows_WDropTargetContextPeerIStream_Available(JNIEnv* env, jobject self, jobject istream) {
    TRY;

    return WDTCPIStreamWrapper::DoAvailable((WDTCPIStreamWrapper*)istream);

    CATCH_BAD_ALLOC_RET(0);
}

/**
 * 
 */

JNIEXPORT jint JNICALL Java_sun_awt_windows_WDropTargetContextPeerIStream_Read(JNIEnv* env, jobject self, jobject istream) {
    TRY;

    return WDTCPIStreamWrapper::DoRead((WDTCPIStreamWrapper*)istream);

    CATCH_BAD_ALLOC_RET(0);
}

/**
 * 
 */

JNIEXPORT jint JNICALL Java_sun_awt_windows_WDropTargetContextPeerIStream_ReadBytes(JNIEnv* env, jobject self, jobject istream, jbyteArray buf, jint off, jint len) {
    TRY;

    return WDTCPIStreamWrapper::DoReadBytes((WDTCPIStreamWrapper*)istream, buf, off, len);

    CATCH_BAD_ALLOC_RET(0);
}

/**
 * 
 */

JNIEXPORT void JNICALL Java_sun_awt_windows_WDropTargetContextPeerIStream_Close(JNIEnv* env, jobject self, jobject istream) {
    TRY_NO_VERIFY;

    WDTCPIStreamWrapper::DoClose((WDTCPIStreamWrapper*)istream);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */
