/*
 * @(#)awt_DrawingSurface.cpp	1.21 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#define _JNI_IMPLEMENTATION_
#include "awt_DrawingSurface.h"

#include "awt_Component.h"
#include "awt_image.h"
#include "awt_Graphics.h"
#include "sun_awt_windows_WDrawingSurfaceInfo.h"


/************************************************************************
 * AwtDrawingSurface fields
 */

jfieldID AwtDrawingSurface::pDataID;
jfieldID AwtDrawingSurface::stateID;
jfieldID AwtDrawingSurface::wID;
jfieldID AwtDrawingSurface::hID;
jfieldID AwtDrawingSurface::peerID = 0;


/************************************************************************
 * WDrawingSurfaceInfo native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_initIDs(JNIEnv *env, jclass cls) 
{
    TRY;

    AwtDrawingSurface::pDataID = env->GetFieldID(cls, "pData", "I");
    AwtDrawingSurface::stateID = env->GetFieldID(cls, "state", "I");
    AwtDrawingSurface::wID = env->GetFieldID(cls, "w", "I");
    AwtDrawingSurface::hID = env->GetFieldID(cls, "h", "I");
    AwtDrawingSurface::peerID = 
	env->GetFieldID(cls, "peer", "Lsun/awt/windows/WComponentPeer;");

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    lock
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_lock(JNIEnv *env, jobject self) 
{
    TRY;

    //jobject imgrep;

    /* REMIND check peer self? */
    AwtSharedDC *pSharedDC = NULL;
    jobject hComp = env->GetObjectField(self, AwtDrawingSurface::peerID);

    char *str = NULL;
    long state = env->GetIntField(self, AwtDrawingSurface::stateID);

    if (hComp != NULL) {
	AwtComponent *pComp = (AwtComponent *)(JNI_GET_PDATA(hComp));
	jobject hJComp = env->GetObjectField(hComp, AwtObject::targetID);

	if (pComp == NULL || hJComp == NULL) {
	    str = "null component";
	} else {
	    if (::IsWindow(pComp->GetHWnd()) == FALSE) {
		str = "disposed component";
	    } else {
		pSharedDC = pComp->GetSharedDC();
		int w = env->GetIntField(self, AwtDrawingSurface::wID);
		int h = env->GetIntField(self, AwtDrawingSurface::hID);
		int hJCompW = env->GetIntField(hJComp, AwtComponent::widthID);
		int hJCompH = env->GetIntField(hJComp, AwtComponent::heightID);

		if (w != hJCompW || h != hJCompH) {
		    env->SetIntField(self, AwtDrawingSurface::wID, hJCompW);
		    env->SetIntField(self, AwtDrawingSurface::hID, hJCompH);

		    int state = env->GetIntField(self, 
						 AwtDrawingSurface::stateID);
		    env->SetIntField(self, AwtDrawingSurface::stateID, 
				     state++);
		}
	    }
	}
//     } else if ((imgrep = env->GetObjectField(self,AwtDrawingSurface::imgrepID))
// 	       != NULL) {

// 	/* REMIND !!! DEREF */
// 	ImageRep *imgrepHdl = (ImageRep *)DeRef(env, imgrep);
// 	AwtImage *pImage = AwtImage::GetpImage(imgrepHdl);

//         pSharedDC = pImage->m_pSharedDC;
    } else {
	pSharedDC = NULL;
    }

    if (pSharedDC == NULL) {
	JNU_ThrowNullPointerException(env, str);
	return state;
    }

    pSharedDC->Lock();
    env->SetIntField(self, AwtDrawingSurface::pDataID, (jint)pSharedDC);

    return state;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    unlock
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_unlock(JNIEnv *env, jobject self)
{
    TRY_NO_VERIFY;

    AwtSharedDC *pSharedDC = (AwtSharedDC *)(env->GetIntField(self, AwtDrawingSurface::pDataID));
    if (pSharedDC == NULL) {
	JNU_ThrowNullPointerException(env, "null DC");
        return;
    }
    pSharedDC->Unlock();
    pSharedDC->Release(); 
    env->SetIntField(self, AwtDrawingSurface::pDataID, 0);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    getHWnd
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_getHWnd(JNIEnv *env, jobject self) 
{
    TRY;

    AwtSharedDC *pSharedDC = (AwtSharedDC *)(env->GetIntField(self, AwtDrawingSurface::pDataID));
    if (pSharedDC == NULL) {
	JNU_ThrowNullPointerException(env, "null DC");
	return NULL;
    }
    
    return (long) pSharedDC->GetHWnd();

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    getHBitmap
 * Signature: ()I
 */
JNIEXPORT jint JNICALL 
Java_sun_awt_windows_WDrawingSurfaceInfo_getHBitmap(JNIEnv *env, jobject self) 
{
//     TRY;
//
//     jobject imgrep = env->GetObjectField(self, AwtDrawingSurface::imgrepID);
//     if (imgrep == NULL) {
//         return NULL;
//     }
//     ImageRep *imgrepHdl = (ImageRep *)(DeRef(env, imgrep));
//     AwtImage *pImage = AwtImage::GetpImage(imgrepHdl);
//     return (long) pImage->m_hBitmap;
//
//     CATCH_BAD_ALLOC_RET(NULL);
    return NULL;
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    getPBits
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_getPBits(JNIEnv *env, jobject self)
{
//     TRY;
//
//     jobject imgrep = env->GetObjectField(self, AwtDrawingSurface::imgrepID);
//     if (imgrep == NULL) {
//         return NULL;
//     }
//     ImageRep *imgrepHdl = (ImageRep *)(DeRef(env, imgrep));
//     AwtImage *pImage = AwtImage::GetpImage(imgrepHdl);
//     return (long) pImage->m_hBitmap;
//
//     CATCH_BAD_ALLOC_RET(NULL);
    return NULL;
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    getHDC
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_getHDC(JNIEnv *env, jobject self)
{
    TRY;

    AwtSharedDC *pSharedDC = (AwtSharedDC *)(env->GetIntField(self, AwtDrawingSurface::pDataID));
    if (pSharedDC == NULL) {
	JNU_ThrowNullPointerException(env, "null DC");
        return NULL;
    }
    // return a new HDC, not the one in use by shared dc so
    // we don't have to worry about saving/restoring the dc state
    return (long) ::GetDCEx(pSharedDC->GetHWnd(), NULL, DCX_CACHE|DCX_CLIPCHILDREN|DCX_CLIPSIBLINGS);

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    getDepth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_getDepth(JNIEnv *env, jobject self)
{
    TRY;

    return AwtImage::GetDepth();

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WDrawingSurfaceInfo
 * Method:    getHPalette
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WDrawingSurfaceInfo_getHPalette(JNIEnv *env, 
						     jobject self)
{
    return (long)AwtImage::m_hPal;
}

} /* extern "C" */




/*****************************************************************************/
/** New native drawing API begins here                                      **/
/*****************************************************************************/

/* DSI */

jint JAWTDrawingSurfaceInfo::Init(JAWTDrawingSurface* parent)
{
    TRY;

    JNIEnv* env = parent->env;
    jobject target = parent->target;
    if (JNU_IsNull(env, target)) {
        DTRACE_PRINTLN("NULL target");
        return JAWT_LOCK_ERROR;
    }
    if (AwtComponent::peerID == 0) {
        return JAWT_LOCK_ERROR;
    }
    jobject peer = env->GetObjectField(target, AwtComponent::peerID);
    if (JNU_IsNull(env, peer)) {
        DTRACE_PRINTLN("NULL peer");
        return JAWT_LOCK_ERROR;
    }
    pComponent = reinterpret_cast<AwtComponent*>(JNI_GET_PDATA(peer));
    DASSERT(pComponent != NULL);
    pDC = pComponent->GetSharedDC();
    platformInfo = this;
    ds = parent;
    bounds.x = env->GetIntField(target, AwtComponent::xID);
    bounds.y = env->GetIntField(target, AwtComponent::yID);
    bounds.width = env->GetIntField(target, AwtComponent::widthID);
    bounds.height = env->GetIntField(target, AwtComponent::heightID);
    // return a new HDC, not the one in use by shared dc so
    // we don't have to worry about saving/restoring the dc state
    hdc = ::GetDCEx(pDC->GetHWnd(), NULL, DCX_CACHE|DCX_CLIPCHILDREN|DCX_CLIPSIBLINGS);
    clipSize = 1;
    clip = &bounds;
    // TODO : Get clip
    hwnd = pComponent->GetHWnd();
    hpalette = AwtImage::m_hPal;
    return 0;

    CATCH_BAD_ALLOC_RET(JAWT_LOCK_ERROR);
}

/* Drawing Surface */

JAWTDrawingSurface::JAWTDrawingSurface(JNIEnv* pEnv, jobject rTarget)
{
    TRY_NO_VERIFY;

    env = pEnv;
    target = env->NewGlobalRef(rTarget);
    Lock = LockSurface;
    GetDrawingSurfaceInfo = GetDSI;
    FreeDrawingSurfaceInfo = FreeDSI;
    Unlock = UnlockSurface;

    CATCH_BAD_ALLOC;
}

JAWTDrawingSurface::~JAWTDrawingSurface()
{
    TRY_NO_VERIFY;

    env->DeleteGlobalRef(target);

    CATCH_BAD_ALLOC;
}

JAWT_DrawingSurfaceInfo* JNICALL JAWTDrawingSurface::GetDSI
    (JAWT_DrawingSurface* ds)
{
    TRY;

    if (ds == NULL) {
        DTRACE_PRINTLN("Drawing Surface is NULL");
        return NULL;
    }
    JAWTDrawingSurface* pds = static_cast<JAWTDrawingSurface*>(ds);
    return &(pds->info);
    
    CATCH_BAD_ALLOC_RET(NULL);
}

void JNICALL JAWTDrawingSurface::FreeDSI
    (JAWT_DrawingSurfaceInfo* dsi)
{
    TRY_NO_VERIFY;

	DASSERTMSG(dsi != NULL, "Drawing Surface Info is NULL\n");

    JAWTDrawingSurfaceInfo* jdsi = static_cast<JAWTDrawingSurfaceInfo*>(dsi);
    ::ReleaseDC(jdsi->pDC->GetHWnd(), jdsi->hdc);

    CATCH_BAD_ALLOC;
}

jint JNICALL JAWTDrawingSurface::LockSurface
    (JAWT_DrawingSurface* ds)
{
    TRY;

    if (ds == NULL) {
        DTRACE_PRINTLN("Drawing Surface is NULL");
        return JAWT_LOCK_ERROR;
    }
    JAWTDrawingSurface* pds = static_cast<JAWTDrawingSurface*>(ds);
    jint val = pds->info.Init(pds);
	if ((val & JAWT_LOCK_ERROR) != 0) {
		return val;
	}
    AwtSharedDC* pSharedDC = pds->info.pDC;
    pSharedDC->Lock();
	AwtComponent* pComponent = pds->info.pComponent;
    val = pComponent->GetDrawState();
    pComponent->SetDrawState(static_cast<jint>(0));
    return val;

    CATCH_BAD_ALLOC_RET(JAWT_LOCK_ERROR);
}

void JNICALL JAWTDrawingSurface::UnlockSurface
    (JAWT_DrawingSurface* ds)
{
    TRY_NO_VERIFY;

    if (ds == NULL) {
        DTRACE_PRINTLN("Drawing Surface is NULL");
        return;
    }
    JAWTDrawingSurface* pds = static_cast<JAWTDrawingSurface*>(ds);
    AwtSharedDC* pSharedDC = pds->info.pDC;
    pSharedDC->Unlock();
    pSharedDC->Release();

    CATCH_BAD_ALLOC;
}

/* C exports */

extern "C" JNIEXPORT JAWT_DrawingSurface* JNICALL DSGetDrawingSurface
    (JNIEnv* env, jobject target)
{
    TRY;

    // Make sure the target component is a java.awt.Canvas
    jclass canvasClass = env->FindClass("java/awt/Canvas");
    if (!env->IsInstanceOf(target, canvasClass)) {
        DTRACE_PRINTLN(
            "GetDrawingSurface target must be a java.awt.Canvas");
        return NULL;
    }

    return new JAWTDrawingSurface(env, target);

    CATCH_BAD_ALLOC_RET(NULL);
}

extern "C" JNIEXPORT void JNICALL DSFreeDrawingSurface
    (JAWT_DrawingSurface* ds)
{
    TRY_NO_VERIFY;

    if (ds == NULL) {
        DTRACE_PRINTLN("Drawing Surface is NULL");
    }
    delete ds;

    CATCH_BAD_ALLOC;
}

