/*
 * @(#)awt_Frame.cpp	1.104 01/02/14
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_Toolkit.h"
#include "awt_Frame.h"
#include "awt_MenuBar.h"
#include "awt_Dialog.h"
#include "awt_IconCursor.h"

#include <sun_awt_windows_WEmbeddedFrame.h>
#include <sun_awt_windows_WEmbeddedFramePeer.h>


/* IMPORTANT! Read the README.JNI file for notes on JNI converted AWT code.
 */

/************************************************************************
 * inner class AwtNativeActivationEvent methods
 */

class AwtFrame::AwtNativeActivationEvent : public AwtActiveEvent
{
public:
    AwtNativeActivationEvent(AwtFrame *frame, HWND prevWnd)
        : AwtActiveEvent(),
	  m_frame(frame),
	  m_prevWnd(prevWnd) {}
    virtual ~AwtNativeActivationEvent() {}
    virtual AwtActiveEvent *Clone() { 
        return new AwtNativeActivationEvent(m_frame, m_prevWnd);
    }
    virtual void Dispatch() {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
	if (env->EnsureLocalCapacity(2) < 0) {
	    return;
	}

	jobject peer =
	  JNU_CallMethodByName(env, NULL, m_frame->GetPeer(env),
			       "getFocusPeer",
			       "()Lsun/awt/windows/WComponentPeer;").l;

	DASSERT(!safe_ExceptionOccurred(env));
	if (peer == NULL) {
	    // Set default focus.
	    JNU_CallMethodByName(env, NULL, m_frame->GetPeer(env),
				 "setDefaultFocus", "()V");
	    DASSERT(!safe_ExceptionOccurred(env));
	} else {
	    AwtComponent* c = (AwtComponent *)JNI_GET_PDATA(peer);
	    if (c != NULL) {
		if (AwtComponent::sm_focusFixEnabled == 1) {
		    ::PostMessage(c->GetHWnd(), WM_AWT_COMPONENT_SETFOCUS,
			       (WPARAM) m_frame->GetHWnd(), 
			       (LPARAM) m_prevWnd);
		} else {
		    ::SendMessage(c->GetHWnd(), WM_AWT_COMPONENT_SETFOCUS,
			       (WPARAM) m_frame->GetHWnd(), 
			       (LPARAM) m_prevWnd);
		}
	    }
	}

	env->DeleteLocalRef(peer);
    }

private:
    AwtFrame *m_frame;
    HWND m_prevWnd;
};

/************************************************************************
 * AwtFrame fields
 */

jfieldID AwtFrame::handleID;
jfieldID AwtFrame::stateID;


/************************************************************************
 * AwtFrame methods
 */

AwtFrame::AwtFrame() {
    m_hIcon = NULL;
    m_parentWnd = NULL;
    menuBar = NULL;
    m_isEmbedded = FALSE;
    m_extraHeight = 0;
    m_ignoreWmSize = FALSE;
    m_isMenuDropped = FALSE;
    m_isInputMethodWindow = FALSE;
}

AwtFrame::~AwtFrame() {
    if (m_hIcon != NULL) {
        ::DestroyIcon(m_hIcon);
    }
}

const char* AwtFrame::GetClassName() {
  return "SunAwtFrame";
}

/*
 * Create a new AwtFrame object and window.
 */
AwtFrame* AwtFrame::Create(jobject self, jobject parent)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(1) < 0) {
	return NULL;
    }

    int pData;
    HWND hwndParent = NULL;
    AwtFrame* frame;
    jclass cls = NULL;
    jclass inputMethodWindowCls = NULL;
    jobject target = NULL;

    try {
        target = env->GetObjectField(self, AwtObject::targetID);
	JNI_CHECK_NULL_GOTO(target, "target", done);

	if (parent != NULL) {
	    JNI_CHECK_PEER_GOTO(parent, done);
	    {
	        AwtFrame* parent = (AwtFrame *)pData;
		hwndParent = parent->GetHWnd();
	    }
	}

	frame = new AwtFrame();
	
	{
	    /*
	     * A variation on Netscape's hack for embedded frames: the client
	     * area of the browser is a Java Frame for parenting purposes, but
	     * really a Windows child window
	     */
	    cls = env->FindClass("sun/awt/EmbeddedFrame");
	    if (cls == NULL) {
	        return NULL;
	    }
	    jint handle;
	    jboolean isEmbeddedInstance = env->IsInstanceOf(target, cls);
	    jboolean isEmbedded = FALSE;

	    if (isEmbeddedInstance) {
	        handle = env->GetIntField(target, AwtFrame::handleID);
		if (handle != 0) {
		    isEmbedded = TRUE;
		}
	    }
	    frame->m_isEmbedded = isEmbedded;
	    
	    if (isEmbedded) {
	        hwndParent = (HWND)handle;
		RECT rect;
		::GetClientRect(hwndParent, &rect);
		frame->RegisterClass();
		DWORD exStyle = WS_EX_NOPARENTNOTIFY;

		if (GetRTL()) {
		    exStyle |= WS_EX_RIGHT | WS_EX_LEFTSCROLLBAR;
		    if (GetRTLReadingOrder())
		        exStyle |= WS_EX_RTLREADING;
		}

		frame->m_hwnd = ::CreateWindowEx(exStyle,
						 frame->GetClassName(), "",
						 WS_CHILD | WS_CLIPCHILDREN,
						 0, 0,
						 rect.right, rect.bottom,
						 hwndParent, NULL,
						 AwtToolkit::GetInstance().GetModuleHandle(),
						 NULL);
		frame->LinkObjects(env, self);
		frame->SubclassHWND();

		// Update target's dimensions to reflect this embedded window.
		::GetClientRect(frame->m_hwnd, &rect);
		::MapWindowPoints(frame->m_hwnd, hwndParent, (LPPOINT)&rect,
				  2);

		env->SetIntField(target, AwtComponent::xID, rect.left);
		env->SetIntField(target, AwtComponent::yID, rect.top);
		env->SetIntField(target, AwtComponent::widthID,
				 rect.right-rect.left);
		env->SetIntField(target, AwtComponent::heightID,
				 rect.bottom-rect.top);
	    } else {
	        jint state = env->GetIntField(target, AwtFrame::stateID);
		DWORD exStyle;
		DWORD style;
	        
		// for input method windows, use minimal decorations
		inputMethodWindowCls = env->FindClass("sun/awt/im/InputMethodWindow");
	        if ((inputMethodWindowCls != NULL) && env->IsInstanceOf(target, inputMethodWindowCls)) {
		    exStyle = WS_EX_PALETTEWINDOW;
		    style = WS_CLIPCHILDREN;
		    frame->m_isInputMethodWindow = TRUE;
		} else {
		    exStyle = WS_EX_WINDOWEDGE;
		    style = WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN |
			  ((state == java_awt_Frame_ICONIFIED) ? WS_ICONIC : 0);
		}

		if (GetRTL()) {
		    exStyle |= WS_EX_RIGHT | WS_EX_LEFTSCROLLBAR;
		    if (GetRTLReadingOrder())
		        exStyle |= WS_EX_RTLREADING;
		}

		frame->CreateHWnd(env, "",
		                  style,
				  exStyle,
				  0, 0, 0, 0,
				  hwndParent,
				  NULL,
				  ::GetSysColor(COLOR_WINDOWTEXT),
				  ::GetSysColor(COLOR_WINDOWFRAME),
				  self);

		jint x = env->GetIntField(target, AwtComponent::xID);
		jint y = env->GetIntField(target, AwtComponent::yID);
		jint width = env->GetIntField(target, AwtComponent::widthID);
		jint height = env->GetIntField(target, AwtComponent::heightID);

		/* 
		 * Reshape here instead of during create, so that a
		 * WM_NCCALCSIZE is sent. 
		 */
		frame->Reshape(x, y, width, height);
	    }
	    
	    AwtDialog::SetDisabledLevelToGreatest(frame);
	}
    } catch (...) {
        env->DeleteLocalRef(target);
	env->DeleteLocalRef(cls);
	env->DeleteLocalRef(inputMethodWindowCls);
	throw;
    }

done:
    env->DeleteLocalRef(target);
    env->DeleteLocalRef(cls);
    env->DeleteLocalRef(inputMethodWindowCls);

    return frame;
}

/*
 * Override AwtComponent's Reshape to keep this frame in place if it is
 * an embedded 'frame'
 */
void AwtFrame::Reshape(int x, int y, int width, int height)
{
    if (IsEmbedded()) {
	x = y = 0;
    }
    AwtWindow::Reshape(x, y, width, height);
}

/* Show the frame, and activate it. */
void AwtFrame::Show()
{
    if (isIconic()) {
        SendMessage(WM_AWT_COMPONENT_SHOW, SW_SHOWMINIMIZED);
    } else {
        if (m_isInputMethodWindow) {
	    // Don't activate input methow window 
            SendMessage(WM_AWT_COMPONENT_SHOW, SW_SHOWNA);
        } else {
	    SendMessage(WM_AWT_COMPONENT_SHOW, SW_SHOW);
        }
    }
}

MsgRouting AwtFrame::WmSize(int type, int w, int h)
{
    if (m_ignoreWmSize) {
        return mrDoDefault;
    }
    else {
        return AwtWindow::WmSize(type, w, h);
    }
}

MsgRouting AwtFrame::WmActivate(UINT nState, BOOL fMinimized)
{
    if (nState == WA_INACTIVE) {
        SendWindowEvent(java_awt_event_WindowEvent_WINDOW_DEACTIVATED);
        return mrDoDefault;
    }
    else {
        SendWindowEvent(java_awt_event_WindowEvent_WINDOW_ACTIVATED);
	AwtNativeActivationEvent(this, ::GetFocus()).SendEvent(this);
	return mrConsume;
    }
}

MsgRouting AwtFrame::WmEnterMenuLoop(BOOL isTrackPopupMenu)
{
    if ( !isTrackPopupMenu ) {
	m_isMenuDropped = TRUE;
    }
    return mrDoDefault;
}

MsgRouting AwtFrame::WmExitMenuLoop(BOOL isTrackPopupMenu)
{
    if ( !isTrackPopupMenu ) {
	m_isMenuDropped = FALSE;
    }
    return mrDoDefault;
}

AwtMenuBar* AwtFrame::GetMenuBar()
{
    return menuBar;
}

void AwtFrame::SetMenuBar(AwtMenuBar* mb)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(2) < 0) {
        return;
    }

    jobject frame = GetTarget(env);
    jclass frameClass = env->GetObjectClass(frame);

    jmethodID resizeMID = env->GetMethodID(frameClass, "resize", "(II)V");

    menuBar = mb;
    if (mb == NULL) {
        // Remove existing menu bar, if any.
        ::SetMenu(GetHWnd(), NULL);

	if (m_extraHeight != 0) {
	    RECT outside;
	    ::GetWindowRect(GetHWnd(), &outside);

//	    m_ignoreWmSize = TRUE;
	    ::MoveWindow( GetHWnd(),
			  outside.left,
			  outside.top,
			  outside.right - outside.left,
			  outside.bottom - m_extraHeight - outside.top,
			  TRUE);
//	    m_ignoreWmSize = FALSE;
/*
	    env->CallVoidMethod(frame, resizeMID, outside.right - outside.left,
				outside.bottom - m_extraHeight - outside.top);
*/
	    m_extraHeight = 0;
	}
    } else {
        if (menuBar->GetHMenu() != NULL) {
            // Save the size of original window and client rectangle
            RECT outside;
            RECT inside;
            ::GetWindowRect(GetHWnd(), &outside);
            ::GetClientRect(GetHWnd(), &inside);
            ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&inside, 2);

	    CriticalSection::Lock l(menuBar->GetLock());
/*
            CriticalSection& l = menuBar->GetLock();
	    l.Enter();
*/
            ::SetMenu(GetHWnd(), menuBar->GetHMenu());

            // Get the size of the new client rectangle with the menu bar
            RECT insidewithmenu;
            ::GetClientRect(GetHWnd(), &insidewithmenu);
            ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&insidewithmenu, 2);

            // If the top of the client rectangle intrudes below the
            // bottom of the window rectangle, we have no choice but to
            // make the window bigger so that insets are computed correctly.
            if (insidewithmenu.top > inside.top && m_extraHeight == 0) {
		RECT exposed = { outside.left, outside.bottom, outside.right,
				 outside.bottom + insidewithmenu.top - 
				 inside.top };

//		m_ignoreWmSize = TRUE;
	        ::MoveWindow( GetHWnd(),
			      outside.left,
			      outside.top,
			      outside.right - outside.left,
			      exposed.bottom - outside.top,
			      TRUE);
//		m_ignoreWmSize = FALSE;
		
		m_extraHeight = insidewithmenu.top - inside.top;
/*		
		l.Leave();

		env->CallVoidMethod(frame, resizeMID, 
				    outside.right - outside.left,
				    exposed.bottom - outside.top);
	    }
	    else {
	        l.Leave();
*/
	    }
        }
    }

    env->DeleteLocalRef(frameClass);
    env->DeleteLocalRef(frame);
}

MsgRouting AwtFrame::WmDrawItem(UINT ctrlId, DRAWITEMSTRUCT far& drawInfo)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    // if the item to be redrawn is the menu bar, then do it
    AwtMenuBar* awtMenubar = GetMenuBar();
    if (drawInfo.CtlType == ODT_MENU && (awtMenubar != NULL) &&
        (::GetMenu( GetHWnd() ) == (HMENU)drawInfo.hwndItem) )
	{
		awtMenubar->DrawItem(drawInfo);
		return mrConsume;
    }

	return AwtComponent::WmDrawItem(ctrlId, drawInfo);
}

MsgRouting AwtFrame::WmMeasureItem(UINT ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
	JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
	AwtMenuBar* awtMenubar = GetMenuBar();
	if ((measureInfo.CtlType == ODT_MENU) && (awtMenubar != NULL))
	{
		// AwtMenu instance is stored in itemData. Use it to check if this
		// menu is the menu bar.
		AwtMenu * pMenu = (AwtMenu *) measureInfo.itemData;
		DASSERT(pMenu != NULL);
		if ( pMenu == awtMenubar )
		{
			HWND hWnd = GetHWnd();
			HDC hDC = ::GetDC(hWnd);
			DASSERT(hDC != NULL);
			awtMenubar->MeasureItem(hDC, measureInfo);
			VERIFY(::ReleaseDC(hWnd, hDC));
			return mrConsume;
		}
	}

	return AwtComponent::WmMeasureItem(ctrlId, measureInfo);
}

static
HBITMAP create_mask(int hW,int hH)
{
    HBITMAP     hbmRC = NULL;
    int         numBytes = (hW * hH + 7) / 8;
    BYTE *pMask = new BYTE[numBytes];
    memset(pMask, 0, numBytes);
    hbmRC = ::CreateBitmap(hW, hH, 1, 1, pMask);
    ::GdiFlush();
    delete [] pMask;
    return hbmRC;
}

void AwtFrame::MakeSetIcon(JNIEnv *env, jintArray intRasterData,
			   jbyteArray byteMaskData, int nSS, int nW, int nH)
{
    HBITMAP hMask = NULL;
    BYTE* pMask = NULL;

    if (::IsIconic(GetHWnd())) {
        ::InvalidateRect(GetHWnd(), NULL, TRUE);
    }
    if (m_hIcon != NULL) {
        ::DestroyIcon(m_hIcon);
        m_hIcon = NULL;
    }

    if (intRasterData != NULL) {
        if (byteMaskData != NULL) {
            try {
                pMask = (BYTE*) env->GetPrimitiveArrayCritical(byteMaskData, 0);
                JNI_CHECK_NULL_RETURN(pMask, "byteMaskData data");
                hMask = ::CreateBitmap(nW, nH, 1, 1, pMask);
            }
            catch(...) {
                if (pMask != NULL) {
                    env->ReleasePrimitiveArrayCritical(byteMaskData, pMask, 0);
                }
                throw;
            }
            env->ReleasePrimitiveArrayCritical(byteMaskData, pMask, 0);
        }
        else {
            hMask = create_mask(nW, nH);
        }

	int *pINTRGBImageBuffer = NULL;
	HBITMAP hColor = NULL;
	try {
	    pINTRGBImageBuffer = (int *)
	        env->GetPrimitiveArrayCritical(intRasterData, 0);

	    JNI_CHECK_NULL_RETURN(pINTRGBImageBuffer, "intRasterData data");

	    hColor = create_BMP(m_parentWnd, pINTRGBImageBuffer,
				    nSS, nW, nH);
	} catch (...) {
	    if (pINTRGBImageBuffer != NULL) {
	        env->ReleasePrimitiveArrayCritical(intRasterData,
						   pINTRGBImageBuffer, 0);
	    }
	    throw;
	}

	env->ReleasePrimitiveArrayCritical(intRasterData, pINTRGBImageBuffer,
					   0);
        if (hMask && hColor) {
            ICONINFO icnInfo;
            memset(&icnInfo, 0, sizeof(ICONINFO));
            icnInfo.hbmMask = hMask;
            icnInfo.hbmColor = hColor;
            icnInfo.fIcon = TRUE;
            m_hIcon = ::CreateIconIndirect(&icnInfo);

            destroy_BMP(hColor);
            destroy_BMP(hMask);
        }
    }

    // If m_hIcon is NULL, Windows will use class default icon.
    SendMessage(WM_SETICON, ICON_SMALL, (LPARAM)m_hIcon);
    SendMessage(WM_SETICON, ICON_BIG,   (LPARAM)m_hIcon);

    // set icon on owned windows
    ::EnumThreadWindows(AwtToolkit::MainThread(),
			(WNDENUMPROC)OwnedSetIcon, (LPARAM)this);
}

// set icon on windows owned by this one
BOOL CALLBACK AwtFrame::OwnedSetIcon(HWND hWndDlg, LPARAM lParam)
{
    TRY;

    AwtFrame *frame = (AwtFrame *)lParam;
    HWND hwndDlgOwner = ::GetWindow(hWndDlg, GW_OWNER);

    while (hwndDlgOwner != NULL) {
	if (hwndDlgOwner == frame->GetHWnd()) {
	    break;
	}
        hwndDlgOwner = ::GetWindow(hwndDlgOwner, GW_OWNER);
    }

    if (hwndDlgOwner == NULL)
	return TRUE;

    // if owned window has an icon, change it to match the frame
    if (::SendMessage(hWndDlg, WM_GETICON, ICON_SMALL, NULL) != NULL) {
	HWND hWndOwner = frame->GetHWnd();

	HICON hBigIcon = frame->m_hIcon;
	//    = (HICON)::SendMessage(hWndOwner, WM_GETICON, ICON_BIG, NULL);
	if (hBigIcon == NULL) {
	    hBigIcon = (HICON)::GetClassLong(hWndOwner, GCL_HICON);
	}
	// We know that AwtFrame has at least class' big icon.
	DASSERT(hBigIcon != NULL);

	HICON hSmallIcon
	    = (HICON)::SendMessage(hWndOwner, WM_GETICON, ICON_SMALL, NULL);
	if (hSmallIcon == NULL) {
	    // If we just use the default AWT (big) icon for
	    // ICON_SMALL, it looks pretty jagged, so use the one
	    // autoconverted by windows.
	    hSmallIcon = (HICON)::GetClassLong(hWndOwner, GCL_HICONSM);
	}
	if (hSmallIcon == NULL) {
	    hSmallIcon = hBigIcon;
	}
	// We know that AwtFrame has at least class' small icon
	// (autoconverted by windows, as we use WNDCLASS that doesn't
	// have a slot for small icon).
	DASSERT(hSmallIcon != NULL);

	::SendMessage(hWndDlg, WM_SETICON, ICON_SMALL, (LPARAM)hSmallIcon);
	::SendMessage(hWndDlg, WM_SETICON, ICON_BIG,   (LPARAM)hBigIcon);
    }

    return TRUE;

    CATCH_BAD_ALLOC_RET(FALSE);
}


MsgRouting AwtFrame::WmSysCommand(UINT uCmdType, UINT xPos, UINT yPos)
{
    if (uCmdType == (SYSCOMMAND_IMM & 0xFFF0)){
      JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
      if (env->EnsureLocalCapacity(2) < 0) {
	return mrConsume;
      }
      JNU_CallMethodByName(env, NULL, m_peerObject, "notifyIMMOptionChange", 
			   "()V");
      DASSERT(!safe_ExceptionOccurred(env));  
      return mrConsume;
    }
    return AwtWindow::WmSysCommand(uCmdType, xPos, yPos);
}

LONG AwtFrame::WinThreadExecProc(ExecuteArgs * args)
{
    switch( args->cmdId ) {
	case FRAME_SETMENUBAR:
	{
    	    jobject  mbPeer = (jobject)args->param1;

	    // cancel any currently dropped down menus
	    if (m_isMenuDropped) {
		SendMessage(WM_CANCELMODE);
	    }

	    if (mbPeer == NULL) {
		// Remove existing menu bar, if any
		SetMenuBar(NULL);
	    } else {
		JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
		AwtMenuBar* menuBar = (AwtMenuBar *)JNI_GET_PDATA(mbPeer);
		SetMenuBar(menuBar);
	    }
	    DrawMenuBar();
	    break;
	}

	default:
	    AwtWindow::WinThreadExecProc(args);
	    break;
    }

    return 0L;
}

/************************************************************************
 * WFramePeer native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_java_awt_Frame_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    AwtFrame::stateID = env->GetFieldID(cls, "state", "I");
    DASSERT(AwtFrame::stateID != NULL);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    setState
 * Signature: (I)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WFramePeer_setState(JNIEnv *env, jobject self,
                                                jint state)
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN(self);

    AwtFrame* w = (AwtFrame*)pData;
    DASSERT(!IsBadReadPtr(w, sizeof(AwtFrame)));

    if (::IsWindowVisible(w->GetHWnd())) {
        if (state == java_awt_Frame_NORMAL) {
            ::ShowWindow(w->GetHWnd(), SW_RESTORE);
	} else {
            ::ShowWindow(w->GetHWnd(), SW_MINIMIZE);
	}
    } else { 
        if (state == java_awt_Frame_NORMAL) {
           w->setIconic(FALSE);
	} else {
           w->setIconic(TRUE);
        }
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    getState
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WFramePeer_getState(JNIEnv *env, jobject self)
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN_NULL(self);

    AwtFrame* w = (AwtFrame*)pData;
    DASSERT(!IsBadReadPtr(w, sizeof(AwtFrame)));

    if (w->isIconic()) {
	return java_awt_Frame_ICONIFIED;
    } else {
	return java_awt_Frame_NORMAL;
    }

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    setMenuBar0
 * Signature: (Lsun/awt/windows/WMenuBarPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WFramePeer_setMenuBar0(JNIEnv *env, jobject self,
					    jobject mbPeer)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    AwtObject::WinThreadExec(self, AwtFrame::FRAME_SETMENUBAR, (LPARAM)mbPeer);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    create
 * Signature: (Lsun/awt/windows/WComponentPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WFramePeer_createAwtFrame(JNIEnv *env, jobject self,
                                               jobject parent)
{
    TRY;

    AwtToolkit::CreateComponent(self, parent,
				(AwtToolkit::ComponentFactory)
				AwtFrame::Create);
    int pData;
    JNI_CHECK_PEER_CREATION_RETURN(self);

    CATCH_BAD_ALLOC;
}
/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    getSysIconHeight
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WFramePeer_getSysIconHeight(JNIEnv *env, jclass self)
{
    TRY;

    return ::GetSystemMetrics(SM_CYICON);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    getSysIconWidth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WFramePeer_getSysIconWidth(JNIEnv *env, jclass self)
{
    TRY;

    return ::GetSystemMetrics(SM_CXICON);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    getSysMenuHeight
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WFramePeer_getSysMenuHeight(JNIEnv *env, jclass self)
{
    TRY;

    return ::GetSystemMetrics(SM_CYMENUSIZE);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    setIconImageFromIntRasterData
 * Signature: ([I)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WFramePeer_setIconImageFromIntRasterData
    (JNIEnv *env, jobject self, jintArray intRasterData,
     jbyteArray byteMaskData, jint scanStride, jint nW, jint nH)
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN(self);
    // ok to pass null intRasterData: default AWT icon

    AwtFrame* frame = (AwtFrame *)pData;
    frame->MakeSetIcon(env, intRasterData, byteMaskData, (int)scanStride,
                       (int)nW, (int)nH);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    pSetIMMOption
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WFramePeer_pSetIMMOption(JNIEnv *env, jobject self,
					       jstring option) 
{
    TRY;

    jint pData;
    const char *coption;
    char *empty = "InputMethod";
    JNI_CHECK_PEER_RETURN(self);
    JNI_CHECK_NULL_RETURN(option, "IMMOption argument");

    AwtFrame* frame = (AwtFrame *)pData;

    if (JNU_IsNull(env, option)) {
        coption = empty;
    } else {
        coption = JNU_GetStringPlatformChars(env, option, NULL);
        if (coption == NULL) {
	    throw std::bad_alloc();
        }
    }

    HMENU hSysMenu = ::GetSystemMenu(frame->GetHWnd(), FALSE);
    ::AppendMenu(hSysMenu,  MF_STRING, SYSCOMMAND_IMM, coption);

    if (coption != empty)
        JNU_ReleaseStringPlatformChars(env, option, coption);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */


/************************************************************************
 * WEmbeddedFrame native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WFramePeer
 * Method:    initIDs
 * Signature: (Lsun/awt/windows/WMenuBarPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WEmbeddedFrame_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    AwtFrame::handleID = env->GetFieldID(cls, "handle", "I");
    DASSERT(AwtFrame::handleID != NULL);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WEmbeddedFrame
 * Method:    setThisPeer
 * Signature: (Lsun/awt/windows/WComponentPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WEmbeddedFrame_setThisPeer(JNIEnv *env, jobject self,
                                                jobject lpeer)
{
    TRY;

    jclass cls;
    jfieldID fid;

    cls = env->GetObjectClass(self);
    fid = env->GetFieldID(cls, "peer", "Ljava/awt/peer/ComponentPeer;");
    env->SetObjectField(self, fid, lpeer);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */


/************************************************************************
 * WEmbeddedFramePeer native methods
 */

extern "C" {

JNIEXPORT void JNICALL
Java_sun_awt_windows_WEmbeddedFramePeer_create(JNIEnv *env, jobject self,
					       jobject parent)
{
    TRY;

    JNI_CHECK_NULL_RETURN(self, "peer");
    AwtToolkit::CreateComponent(self, parent,
				(AwtToolkit::ComponentFactory)
				AwtFrame::Create);
    int pData;
    JNI_CHECK_PEER_CREATION_RETURN(self);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */
