/*
 * @(#)awt_Frame.h	1.42 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef AWT_FRAME_H
#define AWT_FRAME_H

#include "awt_Window.h"
#include "awt_ActiveEvent.h"
#include "awt_MenuBar.h" //add for multifont

#include "java_awt_Frame.h"
#include "sun_awt_windows_WFramePeer.h"
#include <awt_image.h>


/************************************************************************
 * AwtFrame class
 */

class AwtFrame : public AwtWindow {
public:
    enum FrameExecIds {
	FRAME_SETMENUBAR
    };

    /* int handle field for sun.awt.windows.WEmbeddedFrame */
    static jfieldID handleID;

    /* int state field for java.awt.Frame */
    static jfieldID stateID;

    AwtFrame();
    virtual ~AwtFrame();

    virtual const char* GetClassName();
    
    /* Create a new AwtFrame.  This must be run on the main thread. */
    static AwtFrame* Create(jobject self, jobject parent);

    /* Returns whether this frame is embedded in an external native frame. */
    INLINE BOOL IsEmbedded() { return m_isEmbedded; }

    void Reshape(int x, int y, int width, int height);
    void Show();

    INLINE void DrawMenuBar() { VERIFY(::DrawMenuBar(GetHWnd())); }

    void MakeSetIcon(JNIEnv *env, jintArray intRasterData,
                     jbyteArray byteMaskData, int ss, int w, int h);

    /*for WmDrawItem and WmMeasureItem method */
    AwtMenuBar* GetMenuBar();
    void SetMenuBar(AwtMenuBar*);

    MsgRouting WmSize(int type, int w, int h);
    MsgRouting WmActivate(UINT nState, BOOL fMinimized);
    MsgRouting WmDrawItem(UINT ctrlId, DRAWITEMSTRUCT far& drawInfo);
    MsgRouting WmMeasureItem(UINT ctrlId, MEASUREITEMSTRUCT far& measureInfo);
    MsgRouting WmEnterMenuLoop(BOOL isTrackPopupMenu);
    MsgRouting WmExitMenuLoop(BOOL isTrackPopupMenu);
    
    void PushModality();
    void PopModality();

    virtual MsgRouting WmSysCommand(UINT uCmdType, UINT xPos, UINT yPos);

    LONG WinThreadExecProc(ExecuteArgs * args);

private:
    static BOOL CALLBACK OwnedSetIcon(HWND hWnd, LPARAM lParam);
  /* The frame's icon and its size.  If NULL, just paint the HotJava
       logo. */
    HICON m_hIcon;

    /* The frame's embedding parent (if any) */
    HWND m_parentWnd;

    /* The frame's menubar. */
    AwtMenuBar* menuBar;

    /* The frame is an EmbeddedFrame. */
    BOOL m_isEmbedded;

    /* height added by SetMenuBar */
    int m_extraHeight;

    /* used so that calls to ::MoveWindow in SetMenuBar don't propogate
       because they are immediately followed by calls to Component.resize */
    BOOL m_ignoreWmSize;

    /* tracks whether or not menu on this frame is dropped down */
    BOOL m_isMenuDropped;

    /* The frame is an InputMethodWindow */
    BOOL m_isInputMethodWindow;

//inner classes
    class AwtNativeActivationEvent;
    friend class AwtNativeActivationEvent;
};

#endif /* AWT_FRAME_H */
