/*
 * @(#)awt_Graphics.cpp	1.161 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt.h"
#include "awt_Color.h"
#include "awt_Component.h"
#include "awt_Dimension.h"
#include "awt_Frame.h"   /* frames for inset calculation */
#include "awt_Graphics.h"
#include "awt_Rectangle.h"
#include "awt_Unicode.h"
#include "awt_Window.h"
#include "awt_dlls.h"
#include "awt_Win32GraphicsConfig.h"
#include "awt_Win32GraphicsDevice.h"
#include "awt_Win32GraphicsEnv.h"

#include <jlong.h>

# include <math.h>                /* for cos(), sin(), etc */
# include <limits.h>              /* for LONG_MIN, LONG_MAX, etc */

#ifndef     _Included_sun_awt_windows_WGraphics
#include    "sun_awt_windows_WGraphics.h"
#endif

#ifndef     _Included_sun_java2d_loops_ImageData
#include    "sun_java2d_loops_ImageData.h"
#endif

#ifndef     _Included_sun_java2d_loops_GraphicsPrimitive
#include    "sun_java2d_loops_GraphicsPrimitive.h"
#endif

#ifndef     _Included_ImageData
#include    <ImageData.h>
#endif


/* IMPORTANT! Read the README.JNI file for notes on JNI converted AWT code.
 */

#define POLYTEMPSIZE	(512 / sizeof(POINT))

extern CriticalSection imageLock;
extern unsigned char awt_reds[256];
extern unsigned char awt_greens[256];
extern unsigned char awt_blues[256];
/************************************************************************
 * AwtGraphics fields
 */

jfieldID AwtGraphics::foregroundID       = 0;
jfieldID AwtGraphics::pDataID            = 0;

jmethodID AwtGraphics::getFontMID        = 0;

/************************************************************************
 * AwtGraphics methods
 */

long AwtGraphics::m_DDRefCount = 0;
HINSTANCE AwtGraphics::m_hLibDDraw = NULL;              // DirectDraw Library handle
LPDIRECTDRAW        AwtGraphics::m_DDObject = NULL;     // DirectDraw COM object
LPDIRECTDRAWSURFACE AwtGraphics::m_lpDDSPrimary = NULL; // DirectDraw primary surface
BOOL AwtGraphics::m_fDDFailed = FALSE;                  // one-way ratchet ... once failed, never try again


AwtGraphics::AwtGraphics()
{
    CriticalSection::Lock l(imageLock);

    m_hDC = NULL;
    m_javaObject = NULL;
    m_pSharedDC = NULL;
    m_bPenSet = FALSE;
    m_hPen = NULL;
    m_color = RGB(0, 0, 0);
    m_xorcolor = RGB(255, 255, 255);
    m_bBrushSet = FALSE;
    m_hBrush = NULL;
    m_pFont = NULL;
    m_bPaletteSelected = FALSE;
    ::SetRectEmpty(&m_rectClip);
    m_bClipping = FALSE;
    m_originX = 0;
    m_originY = 0;
    m_rop = R2_COPYPEN;
    VERIFY((m_hClipRgn = ::CreateRectRgnIndirect(&m_rectClip)) != NULL);
    m_isPrinter = FALSE;
    m_supportsBitBlt = TRUE;
    m_DDRefCount++;
    m_cachedDIB = NULL;
    memset(&m_cachedDIBbi, 0, sizeof(m_cachedDIBbi));
    m_cachedDIBnUsage = 0;
    m_cachedDIBpImage = NULL;

    m_nProtectCount = 0;

    m_scaleFactorX = 1.0;
    m_scaleFactorY = 1.0;
}

AwtGraphics::~AwtGraphics()
{
    CriticalSection::Lock l(imageLock);
    SetJavaObject((JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2), NULL);

    if (!--m_DDRefCount) {
        ReleaseDDResources();
    }
    if (m_pSharedDC != NULL) {
        m_pSharedDC->Detach(this);
        m_pSharedDC->Release();
        m_pSharedDC = NULL;
    }

    if (m_hPen != NULL) {
        m_hPen->Release();
        m_hPen = NULL;
    }

    if (m_hBrush != NULL) {
        m_hBrush->Release();
        m_hBrush = NULL;
    }

    VERIFY(::DeleteObject(m_hClipRgn));
    if (m_cachedDIB) {
        ::DeleteObject(m_cachedDIB);
        m_cachedDIB = NULL;
    }
    m_hClipRgn = NULL;
}

AwtGraphics * AwtGraphics::CreateFromComponent(AwtGraphicsCreate *pagc)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject wGraphics = pagc->wGraphics;
    AwtComponent *pWindow = (AwtComponent *)pagc->source;

#ifdef DEBUG
// Enable this call to track down GDIFlush() failures on NT 3.51.
//    VERIFY(::GdiSetBatchLimit(1));
#endif

    CriticalSection::Lock l(imageLock);
    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->SetJavaObject(env, wGraphics);
    pGraphics->m_pSharedDC = pWindow->GetSharedDC();
    pGraphics->m_pSharedDC->Attach();
    pGraphics->SetColor(pWindow->GetBackgroundColor());

    return pGraphics;
}

AwtGraphics *AwtGraphics::CreateCopy(AwtGraphicsCreate *pagc)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    jobject wGraphics = pagc->wGraphics;
    AwtGraphics *pSource = (AwtGraphics *)pagc->source;

    AwtGraphics *pGraphics;
    AwtComponent* pWnd = pSource->GetComponent();

    if (pWnd != NULL) {
        AwtGraphics::AwtGraphicsCreate agc = { wGraphics, pWnd };
	pGraphics = AwtGraphics::CreateFromComponent(&agc);
        if (pGraphics == NULL) {
            return NULL;
        }
    } else {
        CriticalSection::Lock l(imageLock);
        pGraphics = new AwtGraphics();

	DASSERT(JNU_IsInstanceOfByName(env, wGraphics,
				      "sun/awt/windows/WGraphics") > 0);
        pGraphics->SetJavaObject(env, wGraphics);

        pGraphics->m_pSharedDC = pSource->m_pSharedDC;
        pGraphics->m_pSharedDC->AddRef();
        if (pGraphics->m_pSharedDC != NULL) {
            pGraphics->m_pSharedDC->Attach();
        }
    }

    pGraphics->m_isPrinter = pSource->m_isPrinter;
    pGraphics->m_supportsBitBlt = pSource->m_supportsBitBlt;
    if (pSource->m_bClipping) {
        int x = pSource->m_rectClip.left;
        int y = pSource->m_rectClip.top;
        int w = pSource->m_rectClip.right - x;
        int h = pSource->m_rectClip.bottom - y;
        pGraphics->ChangeClip(x, y, w, h, TRUE);
    }
    pGraphics->m_originX = pSource->m_originX;
    pGraphics->m_originY = pSource->m_originY;
    pGraphics->SetRop(pSource->m_rop, pSource->m_color, RGB(0, 0, 0));
    pGraphics->SetColor(pSource->m_color);
    pGraphics->SetFont(pSource->m_pFont);
    return pGraphics;
}

AwtGraphics *AwtGraphics::CreateFromHDC(AwtGraphicsCreate *pagc)
{
    jobject wGraphics = pagc->wGraphics;
    HDC hdc = (HDC)pagc->source;

    CriticalSection::Lock l(imageLock);
    AwtGraphics *pGraphics = new AwtGraphics();

    try {
        pGraphics->m_pSharedDC = new AwtSharedDC(hdc);
    } catch (std::bad_alloc&) {
        delete pGraphics;
	throw;
    }

    pGraphics->m_pSharedDC->Attach();

    // Initialize pen and brush to match AwtGraphics settings.
    ::SelectObject(hdc, ::GetStockObject(NULL_PEN));
    ::SelectObject(hdc, ::GetStockObject(NULL_BRUSH));

    return pGraphics;
}

void AwtGraphics::ValidateDC()
{
    m_hDC = m_pSharedDC->GrabDC(this);
    if (m_hDC == NULL) {
        return;
    }
    if (::SetTextColor(m_hDC, m_color) == CLR_INVALID) {
        VERIFY(::SetTextColor(m_hDC, m_color) != CLR_INVALID);
    }
    if (m_pFont != NULL) {
        VERIFY(::SelectObject(m_hDC, m_pFont->GetHFont()) != NULL);
    }
    VERIFY(::SetROP2(m_hDC, m_rop));
    VERIFY(::SetStretchBltMode(m_hDC, COLORONCOLOR));
    if (!m_isPrinter && AwtImage::m_hPal != NULL) {
        VERIFY(::SelectPalette(m_hDC, AwtImage::m_hPal, FALSE));
        ::RealizePalette(m_hDC);
        AwtToolkit::GetInstance().adjustPalette(m_hDC);
        m_bPaletteSelected = TRUE;

    } else {
        m_bPaletteSelected = FALSE;
    }
    if (m_bClipping) {
        VERIFY(LogicalSelectClipRgn(m_hDC, m_hClipRgn) != ERROR);
    }
}

void AwtGraphics::InvalidateDC() {
    DTRACE_PRINTLN("AwtGraphics::InvalidateDC");
    ::GdiFlush();
    SetPenColor(FALSE);
    SetBrushColor(FALSE);
    ::SelectObject(m_hDC, ::GetStockObject(SYSTEM_FONT));
    if (m_bPaletteSelected) {
        ::SelectObject(m_hDC, ::GetStockObject(DEFAULT_PALETTE));
        m_bPaletteSelected = FALSE;
    }
    if (m_bClipping) {
        LogicalSelectClipRgn(m_hDC, NULL);
    }
    m_hDC = NULL;
}

static COLORREF XorResult(COLORREF fgcolor, COLORREF xorcolor) {
    COLORREF c;
    if (AwtImage::m_clrdata.bitsperpixel > 8) {
        c = (fgcolor & 0xff000000) | ((fgcolor ^ xorcolor) & 0x00ffffff);
    } else {
        c = PALETTEINDEX(GetNearestPaletteIndex(AwtImage::m_hPal, fgcolor) ^
                         GetNearestPaletteIndex(AwtImage::m_hPal, xorcolor));
    }
    return c;
}

void AwtGraphics::SetColor(COLORREF color)
{
    if (AwtImage::m_clrdata.grayscale) {
        COLORREF gcolor, gxorcolor;
        int g;

        g = (int) (.299 * (color & 0xFF) + .587 * ((color >> 8) & 0xFF) +
            .114 * ((color >> 16) & 0xFF) + 0.5);
        // gcolor = g | (g << 8) | (g << 16);
	gcolor = PALETTERGB(g, g, g);
        if (m_rop == R2_XORPEN) {
            g = (int) (.299 * (m_xorcolor & 0xFF) +
                .587 * ((m_xorcolor >> 8) & 0xFF) +
                .114 * ((m_xorcolor >> 16) & 0xFF) + 0.5);
            // gxorcolor = g | (g << 8) | (g << 16);
	    gxorcolor = PALETTERGB(g, g, g);
            ChangeColor(XorResult(gcolor, gxorcolor));
        } else {
            ChangeColor(gcolor);
        }
        return;
    }
    if (m_rop == R2_XORPEN) {
        ChangeColor(XorResult(color, m_xorcolor));
    } else {
        ChangeColor(color);
    }
}

void AwtGraphics::ChangeColor(COLORREF color)
{
    if (m_color != color) {
        m_color = color;
        if (m_hDC != NULL) {
            VERIFY(::SetTextColor(m_hDC, color) != CLR_INVALID);
        }
        if (m_hPen != NULL) {
            if (m_hDC != NULL) {
                SetPenColor(FALSE);
            }
            m_hPen->Release();
            m_hPen = NULL;
        }
        if (m_hBrush != NULL) {
            if (m_hDC != NULL) {
                SetBrushColor(FALSE);
            }
            m_hBrush->Release();
            m_hBrush = NULL;
        }
    }
}

void AwtGraphics::SetJavaObject(JNIEnv *env, jobject obj)
{
    if (obj == NULL && m_javaObject != NULL) {
	/* we are disposing of this, and need to clean up m_javaObject */
        jobject localObj = env->NewLocalRef(m_javaObject);
	if (localObj != NULL) {
            env->SetIntField(localObj, AwtGraphics::pDataID, 0);
	    env->DeleteLocalRef(localObj);
	}
	env->DeleteWeakGlobalRef(m_javaObject);

    } else if (obj != NULL) {
	/* we are setting m_javaObject */
	DASSERT(JNU_IsInstanceOfByName(env, obj,
				      "sun/awt/windows/WGraphics") > 0);
	m_javaObject = env->NewWeakGlobalRef(obj);
    }
}

void AwtGraphics::SetFont(AwtFont *pFont)
{
    if (m_pFont != pFont) {
        if (m_hDC != NULL) {
            VERIFY(::SelectObject(m_hDC, pFont->GetHFont()) != NULL);
        }
        m_pFont = pFont;
    }
}

void AwtGraphics::SetRop(int rop, COLORREF fgcolor, COLORREF color)
{
    if (m_rop != rop) {
        if (m_hDC != NULL) {
            VERIFY(::SetROP2(m_hDC, rop));
        }
        m_rop = rop;
    }
    if (AwtImage::m_clrdata.grayscale) {
        COLORREF gfgcolor, gcolor;
        int g;

        g = (int) (.299 * (fgcolor & 0xFF) + .587 * ((fgcolor >> 8) & 0xFF) +
            .114 * ((fgcolor >> 16) & 0xFF) + 0.5);
        // gfgcolor = g | (g << 8) | (g << 16);
	gfgcolor = PALETTERGB(g, g, g);
        if (m_rop == R2_XORPEN) {
            g = (int) (.299 * (color & 0xFF) +
                .587 * ((color >> 8) & 0xFF) +
                .114 * ((color >> 16) & 0xFF) + 0.5);
            // gcolor = g | (g << 8) | (g << 16);
	    gcolor = PALETTERGB(g, g, g);
            ChangeColor(XorResult(gfgcolor, gcolor));
            m_xorcolor = color;
        } else {
            ChangeColor(gfgcolor);
        }
        return;
    }
    if (m_rop == R2_XORPEN) {
        ChangeColor(XorResult(fgcolor, color));
        m_xorcolor = color;
    } else {
        ChangeColor(fgcolor);
    }
}

void AwtGraphics::DrawLine(JNIEnv* env, long nX1, long nY1, long nX2, long nY2)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    nX1 += m_originX;
    nY1 += m_originY;
    nX2 += m_originX;
    nY2 += m_originY;
    if (m_supportsBitBlt) {
        // Bug #4055623 (Performance enhancements)
        if (nX1 == nX2 || nY1 == nY2) {
	    // vertical line; use PatBlt to draw it
	    SetBrushColor(TRUE);
	    if (nX1 > nX2) {
	        long t = nX1;
		nX1 = nX2;
		nX2 = t;
	    }
	    if (nY1 > nY2) {
	        long t = nY1;
		nY1 = nY2;
		nY2 = t;
	    }
	    VERIFY(::PatBlt(hDC, nX1, nY1, nX2 - nX1 + 1, nY2 - nY1 + 1,
			    ((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));
	    return;
	}
	// you come here when the line is not horizontal or vertical
	
	SetPenColor(TRUE);
	VERIFY(::MoveToEx(hDC, nX1, nY1, NULL));
	VERIFY(::LineTo(hDC, nX2, nY2));
	// Paint the end-point.
	SetBrushColor(TRUE);
	VERIFY(::PatBlt(hDC, nX2, nY2, 1, 1,
			((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));
    } else {
        SetPenColor(TRUE);
	VERIFY(::MoveToEx(hDC, nX1, nY1, NULL));
	VERIFY(::LineTo(hDC, nX2 + 1, nY2 + 1));
    }
}

void AwtGraphics::ClearRect(JNIEnv* env, COLORREF bgcolor, long nX, long nY, long nW, long nH)
{
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    nX += m_originX;
    nY += m_originY;

    RECT rect;
    ::SetRect(&rect, nX, nY, nX + nW, nY + nH);

    if (m_isPrinter) {
        VERIFY(::FillRect(hDC, &rect,
			  (HBRUSH)AwtBrush::Get(bgcolor)->GetHandle()));
    } else {
        COLORREF oldcolor;
	VERIFY((oldcolor = ::SetBkColor(hDC, bgcolor)) != CLR_INVALID);

	VERIFY(::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL));
	
	VERIFY(::SetBkColor(hDC, oldcolor) != CLR_INVALID);
    }
}

void AwtGraphics::FillRect(JNIEnv* env, long nX, long nY, long nW, long nH)
{
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    SetBrushColor(TRUE);
    // Bug #4047723 (FillRect apears wrong (was using the wrong DC handle))
    // Bug #4055623 (Performance enhancements)
    VERIFY(::PatBlt(hDC, nX + m_originX, nY + m_originY, nW, nH,
		    ((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));
}

void AwtGraphics::RoundRect(long nX, long nY, long nW, long nH,
			    long nArcW, long nArcH)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    nX += m_originX;
    nY += m_originY;
    ::RoundRect(hDC, nX, nY, nX+nW+1, nY+nH+1, nArcW, nArcH);
}

void AwtGraphics::FillRoundRect(long nX, long nY, long nW, long nH,
				long nArcW, long nArcH)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    nX += m_originX;
    nY += m_originY;
    ::RoundRect(hDC, nX, nY, nX+nW+1, nY+nH+1, nArcW, nArcH);
}

void AwtGraphics::Ellipse(long nX, long nY, long nW, long nH)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    nX += m_originX;
    nY += m_originY;
    VERIFY(::Ellipse(hDC, nX, nY, nX+nW+1, nY+nH+1));
}

void AwtGraphics::FillEllipse(long nX, long nY, long nW, long nH)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    nX += m_originX;
    nY += m_originY;
    VERIFY(::Ellipse(hDC, nX, nY, nX+nW+1, nY+nH+1));
}

void AngleToCoord(long nAngle, long nW, long nH, long *nX, long *nY)
{
    const double pi = 3.1415926535;
    const double toRadians = 2 * pi / 360;

    *nX = (long)(cos((double)nAngle * toRadians) * nW);
    *nY = -(long)(sin((double)nAngle * toRadians) * nH);
}

void AwtGraphics::Arc(long nX, long nY, long nW, long nH,
		      long nStartAngle, long nEndAngle)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW <= 0 || nH <= 0 || nEndAngle == 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    nX += m_originX;
    nY += m_originY;
    long nSX, nSY, nDX, nDY;
    if (nEndAngle >= 360 || nEndAngle <= -360) {
        nSX = nDX = nX + nW;
        nSY = nDY = nY + nH / 2;
    } else {
        BOOL bEndAngleNeg = nEndAngle < 0;

        nEndAngle += nStartAngle;
        if (bEndAngleNeg) {
            // swap angles
            long tmp = nStartAngle;
            nStartAngle = nEndAngle;
            nEndAngle = tmp;
        }
        AngleToCoord(nStartAngle, nW, nH, &nSX, &nSY);
        nSX += nX + nW/2;
        nSY += nY + nH/2;
        AngleToCoord(nEndAngle, nW, nH, &nDX, &nDY);
        nDX += nX + nW/2;
        nDY += nY + nH/2;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    VERIFY(::Arc(hDC, nX, nY, nX+nW+1, nY+nH+1,
		 nSX, nSY, nDX, nDY));
}

void AwtGraphics::FillArc(long nX, long nY, long nW, long nH,
			  long nStartAngle, long nEndAngle)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW <= 0 || nH <= 0 || nEndAngle == 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    nX += m_originX;
    nY += m_originY;
    long nSX, nSY, nDX, nDY;
    if (nEndAngle >= 360 || nEndAngle <= -360) {
        nSX = nDX = nX + nW;
        nSY = nDY = nY + nH / 2;
    } else {
        BOOL bEndAngleNeg = nEndAngle < 0;

        nEndAngle += nStartAngle;
        if (bEndAngleNeg) {
            // swap angles
            long tmp = nStartAngle;
            nStartAngle = nEndAngle;
            nEndAngle = tmp;
        }
        AngleToCoord(nStartAngle, nW, nH, &nSX, &nSY);
        nSX += nX + nW/2;
        nSY += nY + nH/2;
        AngleToCoord(nEndAngle, nW, nH, &nDX, &nDY);
        nDX += nX + nW/2;
        nDY += nY + nH/2;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    VERIFY(::Pie(hDC, nX, nY, nX+nW+1, nY+nH+1,
		 nSX, nSY, nDX, nDY));
}

void AwtGraphics::ChangePen(BOOL bSetPen)
{
    if (bSetPen) {
        if (m_hPen == NULL) {
            m_hPen = AwtPen::Get(m_color);
            m_bPenSet = FALSE;
        }
        if (!m_bPenSet) {
            ::SelectObject(m_hDC, m_hPen->GetHandle());
            m_bPenSet = TRUE;
        }
    } else if (m_bPenSet) {
        ::SelectObject(m_hDC, ::GetStockObject(NULL_PEN));
        m_bPenSet = FALSE;
    }
}

void AwtGraphics::ChangeBrush(BOOL bSetBrush)
{
    if (bSetBrush) {
        if (m_hBrush == NULL) {
            m_hBrush = AwtBrush::Get(m_color);
            m_bBrushSet = FALSE;
        }
        if (!m_bBrushSet) {
            ::SelectObject(m_hDC, m_hBrush->GetHandle());
            m_bBrushSet = TRUE;
        }
    } else if (m_bBrushSet) {
        ::SelectObject(m_hDC, ::GetStockObject(NULL_BRUSH));
        m_bBrushSet = FALSE;
    }
}

POINT* AwtGraphics::TransformPoly(long* xpoints, long* ypoints,
				  POINT* points, int *pNpoints, BOOL close)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    int		i;
    int		npoints = *pNpoints;

    if (close) {
	close = (npoints > 2
		 && (xpoints[npoints - 1] != xpoints[0]
		     || ypoints[npoints - 1] != ypoints[0]));
	if (close) {
	    npoints++;
	    *pNpoints = npoints;
	}
    }
    if (npoints > POLYTEMPSIZE) {
	points = (POINT*)safe_Malloc(sizeof(POINT) * npoints);
    }
    i = npoints;
    if (close) {
	--i;
	points[i].x = xpoints[0] + m_originX;
	points[i].y = ypoints[0] + m_originY;
    }
    while (--i >= 0) {
	points[i].x = xpoints[i] + m_originX;
	points[i].y = ypoints[i] + m_originY;
    }

    return points;
}

BOOL AwtGraphics::Polyline(long* xPoints, long* yPoints,
			   int nPoints, BOOL close)
{
    POINT tmpPts[POLYTEMPSIZE], *pPoints;

    pPoints = TransformPoly(xPoints, yPoints, tmpPts, &nPoints, close);
    if (pPoints == NULL) {
	return FALSE;
    }

    HDC hDC = GetDC();
    if (hDC != NULL) {
        AwtGraphics::AutoReleaseDC ardc(this);

	SetPenColor(TRUE);
	SetBrushColor(FALSE);
	VERIFY(::Polyline(hDC, pPoints, nPoints));
    }
    if (pPoints != tmpPts) {
	free(pPoints);
    }
    return TRUE;
}

BOOL AwtGraphics::FillPolygon(long* xPoints, long* yPoints, int nPoints,
			      BOOL evenodd)
{
    POINT tmpPts[POLYTEMPSIZE], *pPoints;

    pPoints = TransformPoly(xPoints, yPoints, tmpPts, &nPoints, FALSE);
    if (pPoints == NULL) {
	return FALSE;
    }

    HDC hDC = GetDC();
    if (hDC != NULL) {
        AwtGraphics::AutoReleaseDC ardc(this);

	SetPenColor(FALSE);
	SetBrushColor(TRUE);
	::SetPolyFillMode(hDC, evenodd ? ALTERNATE : WINDING);
	VERIFY(::Polygon(hDC, pPoints, nPoints));
    }
    if (pPoints != tmpPts) {
	free(pPoints);
    }
    return TRUE;
}

void AwtGraphics::FillSpans(JNIEnv *env, jobject si, SpanIteratorFuncs *pFuncs)
{
    jint spanbox[4];

    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    SetBrushColor(TRUE);
    // Bug #4047723 (FillRect apears wrong (was using the wrong DC handle))
    // Bug #4055623 (Performance enhancements)
    DWORD rop = (m_rop == R2_XORPEN) ? PATINVERT : PATCOPY;
    void *srData = (*pFuncs->open)(env, si);
    while ((*pFuncs->nextSpan)(env, srData, spanbox)) {
	VERIFY(::PatBlt(hDC, spanbox[0], spanbox[1],
			spanbox[2] - spanbox[0],
			spanbox[3] - spanbox[1], rop));
    }
    (*pFuncs->close)(env, srData);
}

void AwtGraphics::DrawRect(JNIEnv* env, long nX, long nY, long nW, long nH)
{
    nX += m_originX;
    nY += m_originY;

    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    // Bug #4055623 (Performance enhancements)
    if (m_supportsBitBlt) {
        SetBrushColor(TRUE);
        DWORD rop = (m_rop == R2_XORPEN) ? PATINVERT : PATCOPY;
	if (nW == 0 || nH == 0) {
	    VERIFY(::PatBlt(hDC, nX, nY, nW+1, nH+1, rop));
	} else {
	    VERIFY(::PatBlt(hDC, nX, nY, nW+1, 1, rop));
	    VERIFY(::PatBlt(hDC, nX+nW, nY, 1, nH+1, rop));
	    VERIFY(::PatBlt(hDC, nX, nY, 1, nH+1, rop));
	    VERIFY(::PatBlt(hDC, nX, nY+nH, nW+1, 1, rop));
	}
    } else {
        SetPenColor(TRUE);
	SetBrushColor(FALSE);
	VERIFY(::Rectangle(hDC, nX, nY, nX + nW + 1, nY + nH + 1));
    }
}

long AwtGraphics::DrawStringWidth(wchar_t *pzString, long nX, long nY)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    HDC hDC = GetDC();
    if (hDC == NULL) {
        return 0;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    nX += m_originX;
    nY += m_originY;
    if (m_pFont != NULL && m_pFont->GetAscent() < 0) {
        AwtFont::SetupAscent(m_pFont);
    }
    if (m_rop == R2_XORPEN) {
        BeginPath(hDC);
    }

    if (m_isPrinter) {
        // Microsoft recommended hack: use ExtTextOutW to force each
        // printed character to start on an exact (screen unit) pixel
        // boundary to avoid clipping the edges of printed text.
        int length = wcslen(pzString);
	int *charwidths = new int[length];

        SIZE size;
        for (int i=0; i<length; i++) {
            GetTextExtentPoint32W(hDC, &pzString[i], 1, &size);
            charwidths[i] = size.cx;
        }
        VERIFY(::ExtTextOutW(hDC,
                             nX, nY - (m_pFont ? m_pFont->GetAscent() : 0),
                             0, NULL,
                             pzString, length,
                             charwidths));
        delete[] charwidths;
    } else {
        VERIFY(::TextOutW(hDC,
                          nX, nY -(m_pFont ? m_pFont->GetAscent() : 0),
                          pzString, wcslen(pzString)));
    }
    if (m_rop == R2_XORPEN) {
        EndPath(hDC);
        SetBrushColor(TRUE);
        SetPenColor(FALSE);
        FillPath(hDC);
    }

    SIZE size;
    VERIFY(::GetTextExtentPoint32W(hDC, pzString, wcslen(pzString), &size));
    return size.cx;
}

long AwtGraphics::DrawStringWidth(jstring string, jobject font,
				  long nX, long nY)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    HDC hDC = GetDC();
    if (hDC == NULL) {
        return 0;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    nX += m_originX;
    nY += m_originY;
    if (m_pFont != NULL && m_pFont->GetAscent() < 0) {
        AwtFont::SetupAscent(m_pFont);
    }
    if (m_rop == R2_XORPEN) {
        BeginPath(hDC);
    }

    SIZE size =
      AwtFont::drawMFStringSize(hDC, font, string,
				nX, nY - (m_pFont ? m_pFont->GetAscent() : 0));
    if (m_rop == R2_XORPEN) {
        EndPath(hDC);
        SetBrushColor(TRUE);
        SetPenColor(FALSE);
        FillPath(hDC);
    }

    return size.cx;
}

void AwtGraphics::CopyArea(long nDstX, long nDstY,
			   long nX, long nY,
			   long nW, long nH)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    long dX = nDstX - nX;
    long dY = nDstY - nY;
    nDstX = nDstX + m_originX;
    nDstY = nDstY + m_originY;
    RECT rectSrc, rectDst;
    ::SetRect(&rectDst, nDstX, nDstY, nDstX+nW, nDstY+nH);
    if (m_bClipping) {
	// ScrollDC uses the clip to limit both reading of source pixels
	// and writing of destination pixels.  Our CopyArea operation
	// wants to use the clip only for writing pixels, not reading
	// them.  So, we need to manually clip the destination of the
	// operation and perform the ScrollDC operation with no clipping
	// installed in the DC.  Note that this design only works for
	// simple rectangular clips and copy areas, but could be
	// extended to iterate over more complicated regions.
	if (!::IntersectRect(&rectDst, &rectDst, &m_rectClip)) {
	    return;
	}
        LogicalSelectClipRgn(hDC, NULL);
    }
    ::CopyRect(&rectSrc, &rectDst);
    ::OffsetRect(&rectSrc, -dX, -dY);

    HRGN rgnUpdate = ::CreateRectRgn(0, 0, 0, 0);
    VERIFY(::ScrollDC(hDC, dX, dY, &rectSrc, NULL, rgnUpdate, NULL));
    if (m_bClipping) {
        VERIFY(LogicalSelectClipRgn(hDC, m_hClipRgn) != ERROR);
    }

    // ScrollDC invalidates the part of the source rectangle that
    // is outside of the destination rectangle on the assumption
    // that you wanted to "move" the pixels from source to dest,
    // and so now you will want to paint new pixels in the source.
    // Since our copyarea operation involves no such semantics we
    // are only interested in the part of the update region that
    // corresponds to unavailable source pixels - i.e. the part
    // that falls within the destination rectangle.
    HRGN rgnDst = ::CreateRectRgnIndirect(&rectDst);
    int result = ::CombineRgn(rgnUpdate, rgnUpdate, rgnDst, RGN_AND);

    // Invalidate the exposed area.
    HWND hWnd;
    if (result != NULLREGION && (hWnd = GetHWnd()) != NULL) {
	// REMIND: must somehow send exposures for bitmaps...
	::InvalidateRgn(hWnd, rgnUpdate, TRUE);
    }
    ::DeleteObject(rgnUpdate);
    ::DeleteObject(rgnDst);
}

// Translate logical units to device units (needed by SelectClipRgn)
int LogicalSelectClipRgn(HDC hDC, HRGN hClipRgn)
{
    SIZE wExt, vExt;
    POINT vOrg;
    HRGN hDevRgn;
    RECT rectClip;

    if (hClipRgn == NULL) {
        hDevRgn = hClipRgn;
    } else {
        GetWindowExtEx(hDC, &wExt);
        GetViewportExtEx(hDC, &vExt);
	GetViewportOrgEx(hDC, &vOrg);
        VERIFY(::GetRgnBox(hClipRgn, &rectClip) != ERROR);

        hDevRgn = CreateRectRgn((rectClip.left   * vExt.cx) / wExt.cx + vOrg.x,
				(rectClip.top    * vExt.cy) / wExt.cy + vOrg.y,
				(rectClip.right  * vExt.cx) / wExt.cx + vOrg.x,
				(rectClip.bottom * vExt.cy) / wExt.cy + vOrg.y
				);
    }
    int ret = ::SelectClipRgn(hDC, hDevRgn);
    if (hDevRgn != NULL) {
        ::DeleteObject(hDevRgn);
    }

    return ret;
}

void AwtGraphics::ChangeClip(long nX, long nY, long nW, long nH, BOOL set)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    int x1, y1, x2, y2;
    x1 = nX;
    y1 = nY;
    if (nW <= 0 || nH <= 0) {
        x2 = x1;
        y2 = y1;
    } else {
        x2 = x1 + nW;
        y2 = y1 + nH;
    }
    if (!m_bClipping ||
        m_rectClip.left != x1 || m_rectClip.top != y1 ||
        m_rectClip.right != x2 || m_rectClip.bottom != y2)
    {
        RECT rectTmp;
        ::SetRect(&rectTmp, x1, y1, x2, y2);
        if (!set && m_bClipping) {
            ::IntersectRect(&m_rectClip, &m_rectClip, &rectTmp);
        } else {
            VERIFY(::CopyRect(&m_rectClip, &rectTmp));
        }
        ::SetRectRgn(m_hClipRgn,
                     m_rectClip.left,
                     m_rectClip.top,
                     m_rectClip.right,
                     m_rectClip.bottom);
    }
    m_bClipping = TRUE;
    VERIFY(LogicalSelectClipRgn(hDC, m_hClipRgn) != ERROR);
}

void AwtGraphics::RemoveClip()
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    m_bClipping = FALSE;
    LogicalSelectClipRgn(hDC, NULL);
}

RECT* AwtGraphics::GetClipRect()
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    if (m_bClipping) {
        RECT* rect = new RECT;
        ::CopyRect(rect, &m_rectClip);
	::OffsetRect(rect, -m_originX, -m_originY);
        return rect;
    } else {
        return NULL;
    }
}

BOOL AwtGraphics::CanUseDirectDraw(void) {
    if (m_isPrinter) {
        return false;
    }
    if (!m_fDDFailed && !m_hLibDDraw && !m_DDObject) {
        m_fDDFailed = TRUE;
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        jclass clazz = env->FindClass("sun/awt/windows/WGraphics");
        if (clazz != NULL) {
            jmethodID mid = env->GetStaticMethodID(clazz, "checkNoDDraw",
                            "()Z");
            if (mid != NULL) {
                if (env->CallStaticBooleanMethod(clazz, mid)) {
                    return false;
                }
            }
        }
        UINT saveMode = ::SetErrorMode(SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
        m_hLibDDraw = ::LoadLibrary("ddraw.dll");
        ::SetErrorMode(saveMode);
        if (m_hLibDDraw) {
            typedef HRESULT (WINAPI *FnDDCreateFunc)(GUID FAR * lpGUID,LPDIRECTDRAW FAR * lplpDD,IUnknown FAR * pUnkOuter);
            FnDDCreateFunc ddCreate = (FnDDCreateFunc)::GetProcAddress(m_hLibDDraw,"DirectDrawCreate");
            if (ddCreate) {
                HRESULT ddResult = (*ddCreate)(NULL,&m_DDObject,NULL);
                if (DD_OK == ddResult) {
                    ddResult = m_DDObject->SetCooperativeLevel( NULL, DDSCL_NORMAL);
                    if (DD_OK == ddResult) {
                        DDSURFACEDESC           ddsd;
                        // Create the primary surface
                        ddsd.dwSize = sizeof( ddsd );
                        ddsd.dwFlags = DDSD_CAPS;
                        ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
                        ddResult = m_DDObject->CreateSurface( &ddsd, &m_lpDDSPrimary, NULL );
                        if (DD_OK == ddResult) {
                            m_fDDFailed = FALSE;
                        }
                    }
                }
            }
            if (m_fDDFailed) {
                ReleaseDDResources();
            }
        }
    }
    return !m_fDDFailed && m_hLibDDraw && m_DDObject;
}

void AwtGraphics::ReleaseDDResources(void) {
/* ATTN @@
    if (m_lpDDSPrimary) {
        m_lpDDSPrimary->Release();
        m_lpDDSPrimary = NULL;
    }
    if (m_DDObject) {
        m_DDObject->Release();
        m_DDObject = NULL;
    }
    if (m_hLibDDraw) {
        ::FreeLibrary(m_hLibDDraw);
        m_hLibDDraw = NULL;
    }
*/
}

void AwtGraphics::Cleanup()
{
//  LOCK CS
    CriticalSection::Lock l(imageLock);
    m_fDDFailed = TRUE;
    if (m_lpDDSPrimary) {
        m_lpDDSPrimary->Release();
        m_lpDDSPrimary = NULL;
    }
    if (m_DDObject) {
        m_DDObject->Release();
        m_DDObject = NULL;
    }
    if (m_hLibDDraw) {
        ::FreeLibrary(m_hLibDDraw);
        m_hLibDDraw = NULL;
    }
}

void AwtGraphics::PrintComponent(AwtComponent* c)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    HDC hdc = GetDC();
    if (hdc == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    ::OffsetWindowOrgEx(hdc, -m_originX, -m_originY, NULL);
    c->SendMessage(WM_PRINT, (WPARAM)hdc,
		   PRF_CHECKVISIBLE | PRF_CLIENT | PRF_NONCLIENT);
    ::OffsetWindowOrgEx(hdc, m_originX, m_originY, NULL);
}

BOOL
AwtGraphics::SimpleOutputArea(RECT& rClip, const RECT& rView) {
    int nComplexity = ::GetClipBox(m_VideoInfo.hDC, &rClip);
    switch (nComplexity) {
    case SIMPLEREGION:
        if (    !::IntersectRect(&rClip, &rClip, &rView)
                || ::IsRectEmpty(&rClip)) {
            return FALSE;
        }
        break;
    case COMPLEXREGION:
        {
            // if complex user/system clip, more detailed testing required
            // check to see if the view itself has a complex clip.
            // ::GetClipBox is only API which returns overlapped window status
            // so we set the rView as our clip, and then see if resulting
            // clip is complex.
            // REMIND: we can cache the CreateRectRgnIndirect result,
            // and only override with ::SetRectRgn
            HRGN rgnSave = ::CreateRectRgnIndirect(&rView);
            int  clipStatus = ::GetClipRgn(m_VideoInfo.hDC, rgnSave);
            if (-1 == clipStatus) {
                ::DeleteObject(rgnSave);
                return FALSE;
            }
            HRGN rgnView = ::CreateRectRgnIndirect(&rView);
            ::SelectClipRgn(m_VideoInfo.hDC, rgnView);
            nComplexity = ::GetClipBox(m_VideoInfo.hDC, &rClip);
            ::SelectClipRgn(m_VideoInfo.hDC, clipStatus? rgnSave: NULL);
            ::DeleteObject(rgnSave);
            ::DeleteObject(rgnView);
            if (SIMPLEREGION != nComplexity) {
                return FALSE;
            }
        }
        break;
    case NULLREGION:
    case ERROR:
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL
AwtGraphics::SimpleViewArea(RECT& rClip, const RECT& rView) {
    int nComplexity = ::GetClipBox(m_VideoInfo.hDC, &rClip);
    switch (nComplexity) {
    case SIMPLEREGION:
    case COMPLEXREGION:
        if (    !::IntersectRect(&rClip, &rClip, &rView)
                || ::IsRectEmpty(&rClip)) {
            return FALSE;
        }
        break;
    case NULLREGION:
    case ERROR:
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL
AwtGraphics::CheckMonitorArea(RECT rView) {
  BOOL retCode = TRUE;

  if (AwtWin32GraphicsEnv::GetNumScreens() > 1) {
    
    MONITOR_INFO miInfo;
    RECT rSect = {0,0,0,0};
    POINT ptOrig = {0,0};
    
    ::ClientToScreen(GetHWnd(), &ptOrig);
    ::LPtoDP(m_VideoInfo.hDC, (LPPOINT)&rView, 2);
    ::OffsetRect(&rView,ptOrig.x, ptOrig.y);
    
    retCode = FALSE;

    memset((void*)(&miInfo), 0, sizeof(MONITOR_INFO));
    miInfo.dwSize = sizeof(MONITOR_INFO);

    ::GetMonitorInfo(AwtWin32GraphicsEnv::GetPrimaryMHND(), &miInfo);

    ::IntersectRect(&rSect, &rView, &(miInfo.rMonitor));

    if( ::IsRectEmpty(&rSect) == FALSE ) {
      if( ::EqualRect(&rSect, &rView) == TRUE ) {
	retCode = TRUE;
      }
    }
  }
  return retCode;
}
BOOL
AwtGraphics::CheckCursorArea(RECT rView) {
  AwtToolkit& tk = AwtToolkit::GetInstance();

  BOOL retCode = TRUE;

  RECT rSect = {0,0,0,0};
  RECT rCurs = {0,0,0,0};

  _get_curs_rect(rCurs);

  ::IntersectRect(&rSect,&rView,&rCurs);

  if( FALSE == ::IsRectEmpty(&rSect) ) {
    
    if( TRUE == tk.IsCustomCursor() ) {
      retCode = FALSE;
    }
    if( TRUE == tk.AreThemesInstalled() ) {
      retCode = FALSE;
    }
  }

  return retCode;
}

void
AwtGraphics::GetLUT(VideoInfo& vi) {
    AwtToolkit& tk = AwtToolkit::GetInstance();
    if (m_VideoInfo.m_palSeed == tk.m_palSeed) {
        return;
    }
    m_VideoInfo.m_palSeed = tk.m_palSeed;
    int palLimit = tk.m_palEntryCount;
    m_VideoInfo.lLutDataLength = tk.m_palEntryCount;

    unsigned long *pEntries = m_VideoInfo.uLutData;
    LPPALETTEENTRY palEntries = (LPPALETTEENTRY)tk.m_palEntries;
    for (int i=0 ; i < palLimit; i++ ) {
        pEntries[i] =  0xff000000
                        | (palEntries[i].peRed << 16)
                        | (palEntries[i].peGreen << 8)
                        | (palEntries[i].peBlue);
    }
}

void
AwtGraphics::PreparePixelFormat(void) {
    switch (m_VideoInfo.sdPrimary.ddpfPixelFormat.dwRGBBitCount) {
    case 8:
        m_VideoInfo.nBytePerChannel = 1;
        m_VideoInfo.nNumChannels    = 1;
        m_VideoInfo.nPixelStride    = 1;
        switch(AwtImage::m_clrdata.grayscale) {
        case GS_NOTGRAY:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_BYTE_INDEXED;
            break;
        case GS_INDEXGRAY:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_INDEX8_GRAY;
            break;
        case GS_STATICGRAY:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_BYTE_GRAY;
            break;
        }
        // REMIND: cache these calculations as much as possible
        GetLUT(m_VideoInfo);
        break;
    case 16:
        m_VideoInfo.nBytePerChannel = 2;
        m_VideoInfo.nNumChannels    = 1;
        m_VideoInfo.nPixelStride    = 1;
        switch (m_VideoInfo.sdPrimary.ddpfPixelFormat.dwGBitMask) {
        case 0x07e0:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_565_RGB;
            break;
        case 0x03e0:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_555_RGB;
            break;
        default:
            break;
        }
        break;
    case 24:
        m_VideoInfo.nBytePerChannel = 1;
        m_VideoInfo.nNumChannels    = 3;
        m_VideoInfo.nPixelStride    = 3;
        switch (m_VideoInfo.sdPrimary.ddpfPixelFormat.dwBBitMask) {
        case 0x000000ff:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_3BYTE_BGR;
            break;
        case 0x00ff0000:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_24BIT_RGB;
            break;
        default:
            break;
        }
        break;
    case 32:
        m_VideoInfo.nBytePerChannel = 4;
        m_VideoInfo.nNumChannels    = 1;
        m_VideoInfo.nPixelStride    = 1;
        switch (m_VideoInfo.sdPrimary.ddpfPixelFormat.dwBBitMask) {
        case 0x000000ff:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_INT_RGB;
            break;
        case 0x0000ff00:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_INT_RGBx;
            break;
        case 0x00ff0000:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_INT_BGR;
            break;
        case 0xff000000:
            m_VideoInfo.nType =
                sun_java2d_loops_GraphicsPrimitive_TYPE_INT_BGRx;
            break;
        default:
            break;
        }
        break;
    default:
        m_VideoInfo.nNumChannels    = 1;
        m_VideoInfo.nBytePerChannel = 1;
        m_VideoInfo.nPixelStride    = 1;
        break;
    }

    if (m_VideoInfo.nType) {
        m_VideoInfo.vpData =
                (unsigned char *)(m_VideoInfo.sdPrimary.lpSurface);
        m_VideoInfo.nScanlineStride =
                (m_VideoInfo.sdPrimary.lPitch / m_VideoInfo.nBytePerChannel);
    }
}

int AwtGraphics::_dd_lock(const RECT& rView, RECT& rDevice, RECT& rOutput)
{
    RECT rClip;
    m_VideoInfo.wVideoMethod = METHOD_INVALID;
    // 0.1uS
    if (!CanUseDirectDraw()) {
      return sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
    }

    // 5uS
    if (!SimpleOutputArea(rClip, rView)) {
      return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }

    if (!CheckCursorArea(rView)) {
      return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }

    if (!CheckMonitorArea(rView)) {
      return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }

    // _shield_cursor(rView);    -- Left commented as an illustration
    //                           -- of the problem with deadlock (dfeld)

    // enter direct draw locking...
    // 0.1uS
    memset(&m_VideoInfo.sdPrimary,0,sizeof(DDSURFACEDESC));
    m_VideoInfo.sdPrimary.dwSize = sizeof(m_VideoInfo.sdPrimary);

    // 12uS
    HRESULT hrDDResult;
    do {
        hrDDResult = m_lpDDSPrimary->Lock(NULL,&m_VideoInfo.sdPrimary,0,NULL);
    } while (hrDDResult == DDERR_WASSTILLDRAWING);

    if (DD_OK != hrDDResult) {
	// _un_shield_cursor(rView);    -- Left commented as an illustration
	//                              -- of the problem with deadlock (dfeld)
        return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }

#define TESTBITS(a,f) ((a)&(f) == (f))
    if ( !(m_VideoInfo.sdPrimary.dwFlags & DDSD_ALL)
            && !TESTBITS( m_VideoInfo.sdPrimary.dwFlags,
                        DDSD_CAPS | DDSD_HEIGHT |
                        DDSD_WIDTH | DDSD_PITCH |
                        DDSD_LPSURFACE | DDSD_PIXELFORMAT) ) {

        m_lpDDSPrimary->Unlock(NULL);
	// _un_shield_cursor(rView);    -- Left commented as an illustration
	//                              -- of the problem with deadlock (dfeld)
        return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }
#undef  TESTBITS

    m_VideoInfo.nType = 0;
    // 0.2uS
    PreparePixelFormat();

    int channelStride = m_VideoInfo.nPixelStride * m_VideoInfo.nBytePerChannel;
    if (!m_VideoInfo.nType ||
            (channelStride && m_VideoInfo.sdPrimary.lPitch % channelStride)) {
        m_lpDDSPrimary->Unlock(NULL);
	// _un_shield_cursor(rView);    -- Left commented as an illustration
	//                              -- of the problem with deadlock (dfeld)
        return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }

    // 0.6uS
    POINT pOrigin = {0,0};
    ::ClientToScreen(GetHWnd(), &pOrigin);

    rDevice = rView;
    ::LPtoDP(m_VideoInfo.hDC,(LPPOINT)&rDevice,2);
    ::OffsetRect(&rDevice, pOrigin.x, pOrigin.y);

    rOutput = rClip;
    ::LPtoDP(m_VideoInfo.hDC,(LPPOINT)&rOutput,2);
    ::OffsetRect(&rOutput, pOrigin.x, pOrigin.y);

    m_VideoInfo.wVideoMethod = METHOD_DDRAW;

    return sun_awt_windows_WGraphics_LOCK_OK;
}

int AwtGraphics::_dd_unlock()
{
    int nRC = sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
    if (m_lpDDSPrimary) {
        //  REMIND: For now only one surface rect is used.
        m_lpDDSPrimary->Unlock(NULL);

	// _un_shield_cursor(rView); -- Left commented as an illustration
	//                           -- of the problem with deadlock (dfeld)

        nRC = sun_awt_windows_WGraphics_UNLOCK_OK;
    }
    return nRC;
}

void AwtGraphics::_get_curs_rect(RECT& rRect) {
  POINT ptPos;
  HWND hW = GetHWnd();
  
  rRect.left = rRect.right = 0;
  rRect.top = rRect.bottom = 0;
  
  if( NULL != hW ) {
    ::GetCursorPos(&ptPos);
    
    int cX = ::GetSystemMetrics(SM_CXCURSOR);
    int cY = ::GetSystemMetrics(SM_CYCURSOR);
    
    ::ScreenToClient(hW,&ptPos);
    
    rRect.left = ptPos.x - cX;
    rRect.right = ptPos.x + cX;
    rRect.top = ptPos.y - cY;
    rRect.bottom = ptPos.y + cY;
  }
  
}

HBITMAP AwtGraphics::getDIBSection(HDC hDC, BITMAPINFO *bi,
                                   int nUsage, void **pImage)
{
    if (m_cachedDIB) {
        AwtToolkit& tk = AwtToolkit::GetInstance();
	/**/
	if ((m_VideoInfo.m_palSeed == tk.m_palSeed)
	    && (nUsage == m_cachedDIBnUsage)
            && (m_cachedDIBbi.bmiHeader.biWidth >= bi->bmiHeader.biWidth)
            && (-m_cachedDIBbi.bmiHeader.biHeight >= -bi->bmiHeader.biHeight)
	    && (m_cachedDIBbi.bmiHeader.biBitCount == bi->bmiHeader.biBitCount)
	    && (m_cachedDIBbi.bmiHeader.biCompression == bi->bmiHeader.biCompression)) {
	  *bi = m_cachedDIBbi;
	  *pImage = m_cachedDIBpImage;
	  return m_cachedDIB;
	}
	/**/
        ::DeleteObject(m_cachedDIB);
        m_cachedDIB = NULL;
        memset(&m_cachedDIBbi, 0, sizeof(m_cachedDIBbi));
        m_cachedDIBnUsage = 0;
        m_cachedDIBpImage = NULL;
    }
    if (!m_cachedDIB) {
        m_cachedDIB = ::CreateDIBSection(hDC, bi, nUsage, pImage, NULL, 0);
        if (m_cachedDIB) {
            m_cachedDIBbi = *bi;
            m_cachedDIBnUsage = nUsage;
            m_cachedDIBpImage = *pImage;
        } else {
            memset(&m_cachedDIBbi, 0, sizeof(m_cachedDIBbi));
            m_cachedDIBnUsage = 0;
            m_cachedDIBpImage = NULL;
        }
    }
    return m_cachedDIB;
}

int AwtGraphics::_dib_lock(const RECT& rView, RECT& rDevice, RECT& rOutput)
{
    int nRC = sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;

    m_VideoInfo.wVideoMethod = METHOD_INVALID;
    RECT rClip;
    if (!SimpleViewArea(rClip, rView)) {
//        return sun_awt_windows_WGraphics_LOCK_GEOMETRY_PROBLEM;
    }

    rDevice = rView;
    rOutput = rClip;
    if (!m_isPrinter) {
        ::LPtoDP(m_VideoInfo.hDC,(LPPOINT)&rDevice,2);
        ::LPtoDP(m_VideoInfo.hDC,(LPPOINT)&rOutput,2);
    } else {
        // else ... deal with printer in final output phase.
        rOutput = rView;
    }

    HBITMAP     hOldMap;

    // Fix for 4302507
    // nType, set in this function, determines the rendering path
    // later on in Java code.  nType is determined using the pixel
    // depth, nScreenBPP.  Erroneous values of nScreenBPP were leading
    // to incorrect values for nType, causing this bug.
    //
    // Previously, nScreenBPP was retrieved from a DC for the primary
    // monitor.  This is not sufficient for proper operation in a
    // multiscreen environment.  In the case of a multiscreen system,
    // the GraphicsDevice from the corresponding Component is gotten,
    // and then a DC is retrieved, from which the correct screen depth
    // can be determined.
    int         nScreenBPP;
    AwtComponent* comp = NULL;
    comp = GetComponent();

    if (AwtWin32GraphicsEnv::GetNumScreens() > 1 && comp != NULL) {
        JNIEnv *env = (JNIEnv*)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        DASSERT(env);
        jobject peer = comp->GetPeer(env);
        DASSERT(peer);
        jobject gc = env->CallObjectMethod(peer, AwtComponent::peerGetGCMID); 
        DASSERT(gc);

        jint pfdIdx = env->GetIntField(gc,
                                       AwtWin32GraphicsConfig::win32GCVisualID);
        if (0 == pfdIdx) {
        // We're running on an OS with broken PixelFormat calls (i.e. 98).
        // Thus, all GCs are using the ColorModel for the default screen.
        // In effect, we DON'T want to execute the 4302507 fix.
            nScreenBPP  =   ::GetDeviceCaps(m_VideoInfo.hDC, BITSPIXEL);
        }
        else {

            jobject gd = env->GetObjectField(gc,
                                         AwtWin32GraphicsConfig::win32DeviceID);
            DASSERT(gd);

            jint scrn = env->GetIntField(gd, AwtWin32GraphicsDevice::screenID);
            HDC dcFromScrn = AwtWin32GraphicsEnv::GetDCFromScreen(scrn);
            DASSERT(dcFromScrn);
            nScreenBPP = ::GetDeviceCaps(dcFromScrn, BITSPIXEL);
            ::DeleteDC(dcFromScrn);
        }
    } else {
        nScreenBPP  =   ::GetDeviceCaps(m_VideoInfo.hDC, BITSPIXEL);
    }

    int         nUsage      =   DIB_RGB_COLORS;

    int         nType                   = 0;
    WORD        nCoordinateOffset       = 0;
    WORD        nBytePerChannel         = 0;
    WORD        nNumChannels            = 0;
    WORD        nPixelStride            = 0;
    DWORD       dwBitPerPixel           = nScreenBPP;

    int         nW                      = rOutput.right - rOutput.left;

    nW += (4 - nW%4);

    memset(&(m_VideoInfo.bmhHeader),0,sizeof(Bitmapheader));
//  REMINDER:
//  Form header for each screen resolution case.

    m_VideoInfo.bmhHeader.bmiHeader.biSize             = sizeof(BITMAPINFOHEADER);
    if (sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_NOOP == 
	(m_VideoInfo.nLockMethod & 
	 sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_NOOP))
    {
	m_VideoInfo.bmhHeader.bmiHeader.biWidth        = 1;
	m_VideoInfo.bmhHeader.bmiHeader.biHeight       = 1;
    } else {
	m_VideoInfo.bmhHeader.bmiHeader.biWidth            = nW;
	m_VideoInfo.bmhHeader.bmiHeader.biHeight           = -(rOutput.bottom - rOutput.top);
    }
    m_VideoInfo.bmhHeader.bmiHeader.biPlanes           = 1;

    m_VideoInfo.bmhHeader.bmiHeader.biBitCount         = nScreenBPP;
    m_VideoInfo.bmhHeader.bmiHeader.biCompression      = BI_RGB;

    m_VideoInfo.bmhHeader.bmiHeader.biSizeImage        = 0;
    m_VideoInfo.bmhHeader.bmiHeader.biXPelsPerMeter    = 0;
    m_VideoInfo.bmhHeader.bmiHeader.biYPelsPerMeter    = 0;
    m_VideoInfo.bmhHeader.bmiHeader.biClrUsed          = 0;
    m_VideoInfo.bmhHeader.bmiHeader.biClrImportant     = 0;

//          FORMAT SPECIFICATIONS
    if(m_isPrinter) {
        m_VideoInfo.bmhHeader.bmiHeader.biBitCount = 24;
        nBytePerChannel = 1;
        nNumChannels    = 3;
        nPixelStride    = 3;
        m_VideoInfo.bmhHeader.bmiHeader.biCompression = BI_RGB;
        nUsage =   DIB_RGB_COLORS;
        nType  = sun_java2d_loops_GraphicsPrimitive_TYPE_3BYTE_BGR;
    } else {
        switch( nScreenBPP ) {
        case 8:
            nBytePerChannel = 1;
            nNumChannels    = 1;
            nPixelStride    = 1;
            switch(AwtImage::m_clrdata.grayscale) {
            case GS_NOTGRAY:
                nType = sun_java2d_loops_GraphicsPrimitive_TYPE_BYTE_INDEXED;
                break;
            case GS_INDEXGRAY:
                nType = sun_java2d_loops_GraphicsPrimitive_TYPE_INDEX8_GRAY;
                break;
            case GS_STATICGRAY:
                nType = sun_java2d_loops_GraphicsPrimitive_TYPE_BYTE_GRAY;
                break;
            }
            m_VideoInfo.bmhHeader.bmiHeader.biCompression      = BI_RGB;     //----
            {
                for (int k=0; k<256; k++) {
                    ((short *)&m_VideoInfo.bmhHeader.dwMasks)[k] = k;
                }
            }
            nUsage = DIB_PAL_COLORS;
            break;
        case 16:
            nBytePerChannel = 2;
            nNumChannels    = 1;
            nPixelStride    = 1;
            m_VideoInfo.bmhHeader.bmiHeader.biCompression      = BI_BITFIELDS;   //----
            if (16 == AwtImage::GetDepth()) {
                m_VideoInfo.bmhHeader.dwMasks[0]     = 0x0000F800;
                m_VideoInfo.bmhHeader.dwMasks[1]     = 0x000007E0;
                m_VideoInfo.bmhHeader.dwMasks[2]     = 0x0000001F;
                nType  = sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_565_RGB;
            } else {
                m_VideoInfo.bmhHeader.dwMasks[0]     = 0x00007C00; // Has to be set, unless just white area
                m_VideoInfo.bmhHeader.dwMasks[1]     = 0x000003E0; // is shown up
                m_VideoInfo.bmhHeader.dwMasks[2]     = 0x0000001F;
                nType  = sun_java2d_loops_GraphicsPrimitive_TYPE_SHORT_555_RGB;
            }
            nUsage          =   DIB_RGB_COLORS;
            break;
        default:
            // usually, printer at 1bpp
            m_VideoInfo.bmhHeader.bmiHeader.biBitCount = 24;
	    /* NO BREAK */
        case 24:
            nBytePerChannel = 1;
            nNumChannels    = 3;
            nPixelStride    = 3;
            m_VideoInfo.bmhHeader.bmiHeader.biCompression      = BI_RGB;       //----
            nUsage          =   DIB_RGB_COLORS;
            nType  = sun_java2d_loops_GraphicsPrimitive_TYPE_3BYTE_BGR;
            break;
        case 32:
            nBytePerChannel = 4;
            nNumChannels    = 1;
            nPixelStride    = 1;
            m_VideoInfo.bmhHeader.bmiHeader.biCompression      = BI_BITFIELDS; //----
            m_VideoInfo.bmhHeader.dwMasks[0]     = 0x00FF0000;
            m_VideoInfo.bmhHeader.dwMasks[1]     = 0x0000FF00;
            m_VideoInfo.bmhHeader.dwMasks[2]     = 0x000000FF;
            nUsage          =   DIB_RGB_COLORS;
            nType  = sun_java2d_loops_GraphicsPrimitive_TYPE_INT_RGB;
            break;
        }
    }
//          END of FORMAT SPECIFICATIONS

    m_VideoInfo.nType              =   nType;

    m_VideoInfo.nScanlineStride    =   nW * nNumChannels;
    m_VideoInfo.nPixelStride       =   nPixelStride;
    m_VideoInfo.nNumChannels       =   nNumChannels;
    m_VideoInfo.nBytePerChannel    =   nBytePerChannel;

    m_VideoInfo.wVideoMethod       =   METHOD_DIB;

    nRC                             = sun_awt_windows_WGraphics_LOCK_OK;

    if( m_VideoInfo.hDC) {
        m_VideoInfo.hbmpBitmap = getDIBSection(m_VideoInfo.hDC,
                                        (BITMAPINFO*)&(m_VideoInfo.bmhHeader),
                                        nUsage,
                                        (void**)&(m_VideoInfo.ptrImageData));
        if (m_VideoInfo.hbmpBitmap) {
            int scanStride = m_VideoInfo.bmhHeader.bmiHeader.biWidth * nNumChannels;
            int xElements = (int)rOutput.left * nPixelStride;
            int yElements = (int)rOutput.top * scanStride;

	    int yCorrection = 0;
	    /*
	      Win9X and WinNT are processing bottom-top images differently.
	      This is to eliminate this difference.
	     */
	    if( IS_WIN95 ) {
	      yCorrection = (-m_VideoInfo.bmhHeader.bmiHeader.biHeight) -
		(rOutput.bottom - rOutput.top);
	    }

	    yElements -= (yCorrection * scanStride);

            m_VideoInfo.nScanlineStride = scanStride;
            // the offset here looks counterintuitive at first, but
            // it is a conscious trade-off between putting more state
            // data into ImageData, or other peformance loss
            // the issue is that the top left of the DIB is (0,0) and
            // all of the blitters are expecting (xOutput,yOutput)
            // which we are preserving for the actual screen rectangle
            m_VideoInfo.vpData =
                m_VideoInfo.ptrImageData
                - nBytePerChannel * (xElements + yElements);

            if (!m_isPrinter && (8 == nScreenBPP)) {
                GetLUT(m_VideoInfo);
            }
	    if (sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_NOOP ==
		(m_VideoInfo.nLockMethod & sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_NOOP)) {
		return nRC;
	    }
            if (m_isPrinter && (nType == sun_java2d_loops_GraphicsPrimitive_TYPE_3BYTE_BGR)) {
                unsigned char *rowPtr = m_VideoInfo.ptrImageData;
                int height  = rOutput.bottom - rOutput.top;
                int width   = rOutput.right - rOutput.left;
                for (int i=0; i < height; i++) {
                    unsigned char *pixPtr = rowPtr;
                    for (int j=0; j < width; j++) {
                        COLORREF cref = ::GetPixel(m_VideoInfo.hDC, rOutput.left + j, rOutput.top + i);
                        pixPtr[j*3 + 0] = GetBValue(cref);
                        pixPtr[j*3 + 1] = GetGValue(cref);
                        pixPtr[j*3 + 2] = GetRValue(cref);
                    }
                    rowPtr += scanStride;
                }
            } else if ( (sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_READ
                    == (m_VideoInfo.nLockMethod & sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_READ)) )
	    {
                HDC hMemDC;

		if( NULL != (hMemDC = ::CreateCompatibleDC(m_VideoInfo.hDC)) )
                {
                    hOldMap = (HBITMAP)::SelectObject(hMemDC, m_VideoInfo.hbmpBitmap);
                    // REMIND: returns BLACK with printer device ... research needed
		    if( FALSE == ::BitBlt(  hMemDC,0,yCorrection,
                                            rOutput.right - rOutput.left,
					    rOutput.bottom - rOutput.top,
					    m_VideoInfo.hDC, rOutput.left, rOutput.top,
					    SRCCOPY) )
		    {
			nRC = sun_awt_windows_WGraphics_LOCK_NOT_LOCKED;
                    }
                    ::GdiFlush();

                    ::SelectObject(hMemDC, hOldMap);
                    ::DeleteDC(hMemDC);
                }
            }
        }
    }
    return nRC;
}

int
AwtGraphics::_dib_unlock()
{
    int nRC = sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
    int needWrite = m_VideoInfo.nLockMethod;
    needWrite &= sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_WRITE;

    if( sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_NOOP ==
	(sun_java2d_loops_GraphicsPrimitive_LOCK_FOR_NOOP & 
	 m_VideoInfo.nLockMethod) ) {
      needWrite = 0;
      nRC = sun_awt_windows_WGraphics_UNLOCK_OK;
    }

    if (    m_VideoInfo.ptrImageData
            && needWrite
            && m_VideoInfo.hDC) {
        int nUsage = DIB_RGB_COLORS;
        // write the image pixels to the screen.
        if (8 == m_VideoInfo.bmhHeader.bmiHeader.biBitCount ) {
            nUsage = DIB_PAL_COLORS;
        }
        int width  = m_VideoInfo.rOutput.right - m_VideoInfo.rOutput.left;
        int height = m_VideoInfo.rOutput.bottom - m_VideoInfo.rOutput.top;
        if (!m_isPrinter) {
            if (::SetDIBitsToDevice(
                    m_VideoInfo.hDC,
                    m_VideoInfo.rOutput.left, m_VideoInfo.rOutput.top,
                    width, height,
                    0, 0,
                    0, height,
                    m_VideoInfo.ptrImageData,
                    (BITMAPINFO *)&(m_VideoInfo.bmhHeader),
                    nUsage)) {
                ::GdiFlush();
                nRC = sun_awt_windows_WGraphics_UNLOCK_OK;
            }
        } else {
            ::GdiFlush();
            /*
             * Fix for bug id 4257962.
             * We prescaled the transformation matrix before drawing the image
             * in WPrintGraphicsWrapper.drawImage(), now prescale the image back down.
             */
            if (::StretchDIBits(
                    m_VideoInfo.hDC,
                    (int)ceil(m_VideoInfo.rOutput.left / GetScaleFactorX()),
                    (int)ceil(m_VideoInfo.rOutput.top / GetScaleFactorY()),
                    (int)ceil(width / GetScaleFactorX()), 
                    (int)ceil(height / GetScaleFactorY()),
                    0, 0, width, height,
                    m_VideoInfo.ptrImageData,
                    (BITMAPINFO *)&(m_VideoInfo.bmhHeader),
                    nUsage,
                    SRCCOPY)) {
                ::GdiFlush();
                nRC = sun_awt_windows_WGraphics_UNLOCK_OK;
            }
        }
    }
    m_VideoInfo.ptrImageData = NULL;
    return nRC;
}

int
AwtGraphics::_gdi_lock(const RECT& rView, RECT& rDevice, RECT& rOutput)
{
    return sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
}

int
AwtGraphics::_gdi_unlock()
{
    return sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
}

int
AwtGraphics::LockScreen(const RECT& rView, RECT& rDevice, RECT& rOutput)
{
    // REMIND: rearchitect to remove incestuous coupling of
    // onscreen/offscreen state management
    AwtToolkit& tk = AwtToolkit::GetInstance();
    tk.m_offscreen = false;
    m_VideoInfo.hDC = this->GetDC();
    try {
        int nRC = _dd_lock(rView, rDevice, rOutput);
	if (sun_awt_windows_WGraphics_LOCK_OK != nRC) {
            if (( sun_java2d_loops_GraphicsPrimitive_LOCK_FASTEST_ONLY &
                 m_VideoInfo.nLockMethod) && !m_isPrinter ) {
                if (m_VideoInfo.hDC) {
                    m_VideoInfo.hDC = NULL;
                    this->ReleaseDC();
                }
                return nRC;
            }
	    tk.m_offscreen = true;
	    nRC = _dib_lock(rView, rDevice, rOutput);
	    if (sun_awt_windows_WGraphics_LOCK_OK != nRC) {
	        nRC = _gdi_lock(rView, rDevice, rOutput);
	    }
	}
	m_VideoInfo.rOutput = rOutput;
	return nRC;
    } catch (...) {
        if (m_VideoInfo.hDC) {
	    m_VideoInfo.hDC = NULL;
	    this->ReleaseDC();
	}
	throw;
    }
}

int
AwtGraphics::UnLockScreen()
{
    int nRC = sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;

    try {
        switch(m_VideoInfo.wVideoMethod) {
	case METHOD_DDRAW:
	    nRC = _dd_unlock();
	    break;
	case METHOD_DIB:
	    nRC = _dib_unlock();
	    break;
	case METHOD_GDI:
            nRC = _gdi_unlock();
	    break;
	default:
	    break;
	}
    } catch (...) {
        // call ReleaseDC unconditionally
        if (m_VideoInfo.hDC) {
	    m_VideoInfo.hDC = NULL;
	    this->ReleaseDC();
	}
	throw;
    }

    if (m_VideoInfo.hDC) {
        m_VideoInfo.hDC = NULL;
        this->ReleaseDC();
    }
    return nRC;
}

/************************************************************************
 * AwtSharedDC methods
 */

AwtSharedDC::AwtSharedDC(AwtComponent* pComponent) {
    DASSERT( !::IsBadWritePtr(pComponent, sizeof(AwtComponent)) );
    m_pComponent = pComponent;
    m_hDC = NULL;
    Init();
}

AwtSharedDC::AwtSharedDC(HDC hDC) {
#ifdef DEBUG
    DWORD type = ::GetObjectType(hDC);
    DASSERT(type == OBJ_DC || type == OBJ_ENHMETADC || type == OBJ_MEMDC ||
	    type == OBJ_METADC);
#endif
    m_pComponent = NULL;
    m_hDC = hDC;
    Init();
}

void AwtSharedDC::Init() {
    DTRACE_PRINTLN("AwtSharedDC::Init");
    m_useCount = 0;
    m_attachCount = 0;
    m_pOwner = NULL;
    AddRef();
#ifdef DEBUG
    m_lockCount = 0;
#endif
}

AwtSharedDC::~AwtSharedDC() {
    CriticalSection::Lock l(m_lockDraw);

    DASSERT(m_useCount == 0 && m_attachCount == 0);
    /* Delete passed in HDC */
    if (m_pComponent == NULL) {
        VERIFY(::DeleteDC(m_hDC));
    }
}

/*
 * Called when a Graphics is about to draw into the DC. We
 * may need to clean up DC attribs set by the current owner
 * of the DC before letting the caller use it
 */
HDC AwtSharedDC::GrabDC(AwtGraphics *pNewOwner)
{
    DASSERT( !::IsBadWritePtr(pNewOwner, sizeof(AwtGraphics)) );

    if (m_pOwner != NULL) {
        m_pOwner->InvalidateDC();
    }
    if (m_hDC != NULL) {
        m_pOwner = pNewOwner;
    }

    return this->GetDC();
}

/*
 * Called by source component when it is destroyed
 * to indicate the DC can no longer be used or called
 * when the last graphics using it detaches
 */
void AwtSharedDC::CloseDC() {
    if (m_hDC == NULL) {
	return; // already closed
    }

    DASSERT( !::IsBadWritePtr(m_pComponent, sizeof(AwtComponent)) );
    
    DASSERT(::GetObjectType(m_hDC) == OBJ_DC);
    if (m_pOwner != NULL) {
	m_pOwner->InvalidateDC();
	m_pOwner = NULL;
    }
    DTRACE_PRINTLN2("%x: AwtSharedDC::Detach releasing HDC %x", this, m_hDC);
//    VERIFY(::ReleaseDC(m_pComponent->GetHWnd(), m_hDC) != 0);
    m_pComponent->SendMessage(WM_AWT_RELEASEDC, (WPARAM)m_hDC);
    m_hDC = NULL;
}

HDC AwtSharedDC::GetDC() {    
    return m_hDC;
}

/* 
 * Called when a graphics object wants to make use of the DC-- at this point
 * we go and get the DC handle from the window
 */
void AwtSharedDC::Attach() {
    // first attach gets the DC
    if (m_attachCount == 0 && m_pComponent != NULL) {
	DASSERT(m_pComponent != NULL && m_hDC == NULL);
//        m_hDC = ::GetDCEx(m_pComponent->GetHWnd(), NULL, DCX_CACHE|DCX_CLIPCHILDREN|DCX_CLIPSIBLINGS);
	m_hDC = (HDC)m_pComponent->SendMessage(WM_AWT_GETDC);
	::SelectObject(m_hDC, ::GetStockObject(NULL_PEN));
	::SelectObject(m_hDC, ::GetStockObject(NULL_BRUSH));
	DTRACE_PRINTLN2("%x: AwtSharedDC::Attach getting HDC %x", this, m_hDC);
    }

    m_attachCount++;
}

/* 
 * Called when a graphics object is done using the DC for drawing
 */
void AwtSharedDC::Detach(AwtGraphics *pOldOwner) {
    DASSERT( !::IsBadWritePtr(pOldOwner, sizeof(AwtGraphics)) );
    /* A separate lock is used here because graphic objects sharing a
     * DC may be freed while one has the sharedDC locked.  For example,
     * drawing text involves an upcall to create a new multifont string,
     * which may invoke a synchronous GC if the alloc fails.  A lock is
     * needed because two threads may be detaching at the same time, and
     * since we can't lock the object, we'll serialize access to this
     * method. */
    CriticalSection::Lock l(m_lockSelf);
    
    if (m_pOwner == pOldOwner) {
        // Release this Device Context and leave it "ownerless"
        m_pOwner->InvalidateDC();
        m_pOwner = NULL;
    }
    
    DASSERT(m_attachCount > 0);
    m_attachCount--;

    // last detach releases the DC
    if (m_attachCount == 0 && m_pComponent != NULL) {
	CloseDC();
    }
}

/*
 * Increase ref count for this object-- use this when
 * making copies of the shared dc pointer
 */
void AwtSharedDC::AddRef() {
    DASSERT(m_useCount >= 0 && m_useCount < 10000);
    m_useCount++;
    DTRACE_PRINTLN1("AwtSharedDC::AddRef %d", m_useCount);
}

/*
 * Decreases ref count for this object-- use this when
 * caller is done with the shared dc
 */
void AwtSharedDC::Release() {
    DASSERT(m_useCount > 0 && m_useCount < 10000);
    m_useCount--;
    DTRACE_PRINTLN1("AwtSharedDC::Release %d", m_useCount);
    if (m_useCount == 0) {
        delete this;
    }
}

/*
 * Lock/unlock this shared DC.  This is necessary so that one
 * thread isn't executing GDI calls when another thread has the
 * DC.
 */
void AwtSharedDC::Lock() {
    m_lockDraw.Enter();
#ifdef DEBUG
    DASSERT(m_lockCount >= 0);
    ++m_lockCount;
#endif
}
void AwtSharedDC::Unlock() {
#ifdef DEBUG
    DASSERT(m_lockCount > 0);
    --m_lockCount;
#endif
    m_lockDraw.Leave();
}

/************************************************************************
 * WGraphics native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    createFromComponent
 * Signature: (Lsun/awt/windows/WComponentPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_createFromComponent(JNIEnv *env, jobject self,
						   jobject component)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.createFromComponent(%x)", self, component);
    if (component == NULL) {
	JNU_ThrowNullPointerException(env, "component argument");
	return;
    }
    int pData = JNI_GET_PDATA(component);
    if (pData == NULL) {
	JNU_ThrowNullPointerException(env, "component argument pData");
	return;
    }

    AwtComponent* comp = (AwtComponent*)pData;
    if (::IsWindow(comp->GetHWnd()) == FALSE) {
	JNU_ThrowNullPointerException(env, "disposed component");
	return;
    }

    AwtGraphics::AwtGraphicsCreate agc = { self, (void *)comp };
    AwtGraphics* graphics = (AwtGraphics *)AwtToolkit::GetInstance().
        InvokeFunction((void*(*)(void*))AwtGraphics::CreateFromComponent,
		       &agc);

    if (graphics == NULL && !AwtToolkit::GetInstance().IsDisposed()) {
	JNU_ThrowInternalError(env, "getGraphics not implemented "
			       "for this component");
	return;
    } else if (graphics != NULL) {
        SET_GRAPHICS_PDATA(self, (jint)graphics);
    }

    CATCH_BAD_ALLOC;
}

JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_createFromGraphics(JNIEnv *env, jobject self,
						  jobject wGraphics)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.createFromGraphics(%x)", self, wGraphics);
    if (wGraphics == NULL) {
	JNU_ThrowNullPointerException(env, "wGraphics argument");
	return;
    }
    int pData = GET_GRAPHICS_PDATA(wGraphics);
    if (pData == NULL) {
        JNU_ThrowNullPointerException(env, "wGraphics argument pData");
	return;
    }
    AwtGraphics* g = (AwtGraphics*)pData;
    AwtGraphics::AwtGraphicsCreate agc = { self, (void *)g };
    AwtGraphics* graphics = (AwtGraphics *)AwtToolkit::GetInstance().
        InvokeFunction((void*(*)(void*))AwtGraphics::CreateCopy,
		       &agc);

    if (graphics == NULL && !AwtToolkit::GetInstance().IsDisposed()) {
	JNU_ThrowInternalError(env,
			       "getGraphics implemented "
			       "only for Canvas Components");
	return;
    } else if (graphics != NULL) {
        SET_GRAPHICS_PDATA(self, (jint)graphics);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    createFromHDC
 * Signature: (I)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_createFromHDC(JNIEnv *env, jobject self,
					     jint hdc)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.createFromHDC(%x)", self, hdc);
    if (hdc == 0) {
	JNU_ThrowNullPointerException(env, "hdc argument");
	return;
    }

    AwtGraphics::AwtGraphicsCreate agc = { self, (void *)hdc };
    AwtGraphics *pGraphics = (AwtGraphics *)AwtToolkit::GetInstance().
        InvokeFunction((void*(*)(void*))AwtGraphics::CreateFromHDC,
		       &agc);

    if (pGraphics != NULL) {
        SET_GRAPHICS_PDATA(self, (jint)pGraphics);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    getVisBounds
 * Signature: (ILjava/awt/Rectangle;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_getVisBounds(JNIEnv *env, jobject self,
					    jint hdc, jobject rectangle)
{
    TRY;

    DTRACE_PRINTLN3("%x: WGraphics.getVisBounds(%x, %x)", self, hdc, rectangle);
    if (hdc == 0) {
	JNU_ThrowNullPointerException(env, "hdc argument");
	return;
    }
    if (rectangle == NULL) {
	JNU_ThrowNullPointerException(env, "rectangle argument");
	return;
    }

    RECT rect;
    VERIFY(::GetClipBox((HDC)hdc, &rect));

    (env)->SetIntField(rectangle, AwtRectangle::xID, rect.left);
    (env)->SetIntField(rectangle, AwtRectangle::yID, rect.top);
    (env)->SetIntField(rectangle, AwtRectangle::widthID,
		       rect.right-rect.left);
    (env)->SetIntField(rectangle, AwtRectangle::heightID,
		       rect.bottom-rect.top);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    pSetFont
 * Signature: (Ljava/awt/Font;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_pSetFont(JNIEnv *env, jobject self,
					jobject font)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.pSetFont(%x)", self, font);
    if (font == NULL) {
	JNU_ThrowNullPointerException(env, "font argument");
	return;
    }

    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	AwtFont* awtFont = AwtFont::GetFont(env, font);
	graphics->SetFont(awtFont);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    pSetForeground
 * Signature: (I)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_pSetForeground(JNIEnv *env, jobject  self,
					      jint value)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.pSetForeground(%x)", self, value);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	COLORREF color = PALETTERGB((value>>16)&0xff,
				    (value>>8)&0xff,
				    (value)&0xff);
	graphics->SetColor(color);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    setForPrinting
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_setForPrinting(JNIEnv *env, jobject self,
					      jboolean b)
{
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);
    graphics->SetIsPrinter(b);
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    setOrigin
 * Signature: (II)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_setOrigin(JNIEnv *env, jobject  self,
					 jint tx, jint ty)
{
    TRY;

    DTRACE_PRINTLN3("%x: WGraphics.setOrigin(%d, %d)", self, tx, ty);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->SetOrigin(tx, ty);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    dispose
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_disposeImpl(JNIEnv *env, jobject self)
{
    TRY_NO_VERIFY;

    DTRACE_PRINTLN1("%x: WGraphics.dispose()", self);
    AwtGraphics* pGraphics =
	(AwtGraphics*)env->GetIntField(self, AwtGraphics::pDataID);
    if (pGraphics != NULL) {
        AwtToolkit::GetInstance().SendMessage(WM_AWT_DISPOSE,
					      (WPARAM)pGraphics, 0);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    setPaintMode
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_pSetPaintMode(JNIEnv *env, jobject self)
{
    TRY;

    DTRACE_PRINTLN1("%x: WGraphics.setPaintMode()", self);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    jobject foreground = env->GetObjectField(self, AwtGraphics::foregroundID);
    if (foreground == 0) {
	JNU_ThrowNullPointerException(env, "foreground field of WGraphics");
	return;
    }

    jint value = env->CallIntMethod(foreground, AwtColor::getRGBMID);
    DASSERT(value != 0);

    COLORREF fgcolor = PALETTERGB((value>>16)&0xff,
				  (value>>8)&0xff,
				  (value)&0xff);
    if (graphics != NULL) {
	graphics->SetRop(R2_COPYPEN, fgcolor, RGB(0, 0, 0));
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    setXORMode
 * Signature: (Ljava/awt/Color;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_pSetXORMode(JNIEnv *env, jobject self,
					  jobject color)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.setXORMode(%x)", self, color);
    jobject foreground = env->GetObjectField(self, AwtGraphics::foregroundID);
    if (color == 0 || foreground == 0) {
	JNU_ThrowNullPointerException(env, "null color or foreground");
	return;
    }
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    jint value = env->CallIntMethod(color, AwtColor::getRGBMID);
    COLORREF awtColor = PALETTERGB((value>>16)&0xff,
				   (value>>8)&0xff,
				   (value)&0xff);
    value = env->CallIntMethod(foreground, AwtColor::getRGBMID);
    COLORREF awtForeground = PALETTERGB((value>>16)&0xff,
					(value>>8)&0xff,
					(value)&0xff);

    if (graphics != NULL) {
	graphics->SetRop(R2_XORPEN, awtForeground, awtColor);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    getClipBounds
 * Signature: ()Ljava/awt/Rectangle;
 */
JNIEXPORT jobject JNICALL
Java_sun_awt_windows_WGraphics_getClipBounds(JNIEnv *env, jobject self)
{
    TRY;

    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    jobject rectangle = NULL;

    if (graphics != NULL) {
	RECT* rect = graphics->GetClipRect();
	if (rect != NULL) {
	    rectangle = JNU_NewObjectByName(env,"java/awt/Rectangle", "()V");
	    if (rectangle == NULL) {
		return NULL;
	    }
	    DASSERT(!safe_ExceptionOccurred(env));
	    (env)->SetIntField(rectangle, AwtRectangle::xID, rect->left);
	    (env)->SetIntField(rectangle, AwtRectangle::yID, rect->top);
	    (env)->SetIntField(rectangle, AwtRectangle::widthID,
			       rect->right-rect->left);
	    (env)->SetIntField(rectangle, AwtRectangle::heightID,
			       rect->bottom-rect->top);

	    delete rect;
	}
    }
    return rectangle;

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    changeClip
 * Signature: (IIIIZ)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_changeClip(JNIEnv *env, jobject self,
					  jint x, jint y, jint w, jint h,
					  jboolean set)
{
    TRY;

    DTRACE_PRINTLN6("%x: WGraphics.changeClip(%d, %d, %d, %d, %d)",
	       self, x, y, w, h, set);

    CriticalSection::Lock l(imageLock);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->ChangeClip(x, y, w, h, set != 0);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    removeClip
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_removeClip(JNIEnv *env, jobject self)
{
    TRY;

    DTRACE_PRINTLN1("%x: WGraphics.removeClip()", self);

    CriticalSection::Lock l(imageLock);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->RemoveClip();
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    clearRect
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devClearRect(JNIEnv *env, jobject self,
					 jint x, jint y, jint w, jint h)
{
    TRY;

    DTRACE_PRINTLN5("%x: WGraphics.clearRect(%d, %d, %d, %d)", self, x, y, w, h);

    AwtGraphics *graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);
    if (graphics == NULL) {
	return;
    }

    jobject background = (JNU_CallMethodByName(env, NULL, self,
					       "getBackground",
					       "()Ljava/awt/Color;")).l;

    jint value = 0;		// defaults to black if background is null
    if (background != 0) {
	value = env->CallIntMethod(background, AwtColor::getRGBMID);
    }

    COLORREF bgcolor = PALETTERGB((value>>16)&0xff,
                                  (value>>8)&0xff,
                                  (value)&0xff);
    graphics->ClearRect(env, bgcolor, x, y, w, h);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    fillRect
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devFillRect(JNIEnv *env, jobject self,
					jint x, jint y, jint w, jint h)
{
    TRY;

#define TILE_SIZE 64
    static unsigned char solidalpha[TILE_SIZE * TILE_SIZE];

    if (w <= 0 || h <= 0) {
    	return;
    }
    CriticalSection::Lock l(imageLock);

    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
        graphics->FillRect(env, x, y, w, h);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawRect
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawRect(JNIEnv *env, jobject self,
					jint x, jint y, jint w, jint h)
{
    TRY;

    DTRACE_PRINTLN5("%x: WGraphics.drawRect(%d, %d, %d, %d)", self, x, y, w, h);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
        graphics->DrawRect(env, x, y, w, h);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawStringWidth
 * Signature: (Ljava/lang/String;II)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WGraphics_drawStringWidth(JNIEnv *env, jobject self,
					       jstring text, jint x, jint y)
{
    TRY;

    DTRACE_PRINTLN4("%x: WGraphics.drawStringWidth(%S, %d, %d)",
	       self, TO_WSTRING(text), x, y);

    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (text == NULL) {
	JNU_ThrowNullPointerException(env, "text argument");
	return NULL;
    }
    if (graphics == NULL) {
	return -1;
    }

    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jobject font = JNI_GET_JAVAFONT(self);
    if (IsMultiFont(env, font)) {
	return graphics->DrawStringWidth(text, font, x, y);
    } else {
	return graphics->DrawStringWidth(TO_WSTRING(text), x, y);
    }

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawCharsWidth
 * Signature: ([CIIII)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WGraphics_drawCharsWidth(JNIEnv *env, jobject self,
					      jcharArray text,
					      jint offset, jint length,
					      jint x, jint y)
{
    TRY;

    DTRACE_PRINTLN6("%x: WGraphics.drawCharsWidth(%x, %d, %d, %d, %d)",
	       self, text, offset, length, x, y);

    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);
    DASSERT(graphics);

    if (graphics == NULL || text == NULL) {
	if (text == NULL) {
	    JNU_ThrowNullPointerException(env, "text argument");
	}
	return -1;
    }
    if (offset < 0 || length < 0 ||
	(offset + length) > (env->GetArrayLength(text))) {
	JNU_ThrowArrayIndexOutOfBoundsException(env, "off/len arguments");
	return -1;
    }

    jint result = NULL;
    jchar *textPtr = new jchar[length + 1];
    env->GetCharArrayRegion(text, offset, length, textPtr);

    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jobject font = JNI_GET_JAVAFONT(self);

    if (IsMultiFont(env, font)) {
	jstring jstr = env->NewString(textPtr, length);
	result = graphics->DrawStringWidth(jstr, font, x, y);

    } else {
	textPtr[length] = '0';
	result = graphics->DrawStringWidth(textPtr, x, y);
    }
    delete[] textPtr;
    return result;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawBytesWidth
 * Signature: ([BIIII)I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WGraphics_drawBytesWidth(JNIEnv *env, jobject self,
					      jbyteArray text,
					      jint offset, jint length,
					      jint x, jint y)
{
    TRY;

    DTRACE_PRINTLN6("%x: WGraphics.drawBytesWidth(%x, %d, %d, %d, %d)",
	       self, text, offset, length, x, y);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics == NULL || text == NULL) {
	if (text == NULL) {
	    JNU_ThrowNullPointerException(env, "text argument");
	}
	return -1;
    }
    if (offset < 0 || length < 0 ||
	offset + length > env->GetArrayLength(text)) {
	JNU_ThrowArrayIndexOutOfBoundsException(env, "off/len arguments");
	return -1;
    }

    jbyte *textPtr = new jbyte[length + 1];

    env->GetByteArrayRegion(text, offset, length, textPtr);
    textPtr[length] = 0;
    jint result = graphics->DrawStringWidth(A2W((char *)textPtr), x, y);
    delete[] textPtr;

    return result;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawLine
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawLine(JNIEnv *env, jobject self,
					jint x1, jint y1, jint x2, jint y2)
{
    TRY;

    DTRACE_PRINTLN5("%x: WGraphics.drawLine(%d, %d, %d, %d)",
	       self, x1, y1, x2, y2);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->DrawLine(env, x1, y1, x2, y2);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    copyArea
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devCopyArea(JNIEnv *env, jobject self,
					jint x, jint y,
					jint width, jint height,
					jint dx, jint dy)
{
    TRY;

    DTRACE_PRINTLN7("%x: WGraphics.copyArea(%d, %d, %d, %d, %d, %d)",
	       self, x, y, width, height, dx, dy);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->CopyArea(dx+x, dy+y, x, y, width, height);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawRoundRect
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawRoundRect(JNIEnv *env, jobject self,
					     jint x, jint y, jint w, jint h,
					     jint arcWidth, jint arcHeight)
{
    TRY;

    DTRACE_PRINTLN7("%x: WGraphics.drawRoundRect(%d, %d, %d, %d, %d, %d)",
	       self, x, y, w, h, arcWidth, arcHeight);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->RoundRect(x, y, w, h, arcWidth, arcHeight);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    fillRoundRect
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devFillRoundRect(JNIEnv *env, jobject self,
					     jint x, jint y, jint w, jint h,
					     jint arcWidth, jint arcHeight)
{
    TRY;

    DTRACE_PRINTLN7("%x: WGraphics.fillRoundRect(%d, %d, %d, %d, %d, %d)",
	       self, x, y, w, h, arcWidth, arcHeight);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->FillRoundRect(x, y, w, h, arcWidth, arcHeight);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawPolygon
 * Signature: ([I[II)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawPolygon(JNIEnv *env, jobject self,
					   jintArray xpoints,
					   jintArray ypoints,
					   jint npoints)
{
    TRY;

    DTRACE_PRINTLN4("%x: WGraphics.drawPolygon(%x, %x, %d)",
	       self, xpoints, ypoints, npoints);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics == NULL || xpoints == NULL || ypoints == NULL) {
	JNU_ThrowNullPointerException(env, "arguments || graphics");
	return;
    }
    int xpointsLen = env->GetArrayLength(xpoints);
    int ypointsLen = env->GetArrayLength(ypoints);

    if (ypointsLen < npoints || xpointsLen < npoints) {
	JNU_ThrowArrayIndexOutOfBoundsException(env,
						"{xpoints,ypoints}, npoints");
	return;
    }
    jint *xpointsPtr = NULL;
    try {
        xpointsPtr = (jint *)env->GetPrimitiveArrayCritical(xpoints, 0);

	jint *ypointsPtr = NULL;
	try {
	    ypointsPtr = (jint *)env->GetPrimitiveArrayCritical(ypoints, 0);
	    int result = graphics->Polyline(xpointsPtr, ypointsPtr, npoints,
					    TRUE);
	} catch (...) {
	    if (ypointsPtr != NULL) {
	        env->ReleasePrimitiveArrayCritical(ypoints, ypointsPtr, 0);
	    }
	    throw;
	}

	env->ReleasePrimitiveArrayCritical(ypoints, ypointsPtr, 0);
    } catch (...) {
        if (xpointsPtr != NULL) {
	    env->ReleasePrimitiveArrayCritical(xpoints, xpointsPtr, 0);
	}
	throw;
    }

    env->ReleasePrimitiveArrayCritical(xpoints, xpointsPtr, 0);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawPolyline
 * Signature: ([I[II)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawPolyline(JNIEnv *env, jobject self,
					    jintArray xpoints,
					    jintArray ypoints,
					    jint npoints)
{
    TRY;

    DTRACE_PRINTLN4("%x: WGraphics.drawPolygon(%x, %x, %d)",
	       self, xpoints, ypoints, npoints);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics == NULL || xpoints == NULL || ypoints == NULL) {
	JNU_ThrowNullPointerException(env, "arguments || graphics");
	return;
    }
    int xpointsLen = env->GetArrayLength(xpoints);
    int ypointsLen = env->GetArrayLength(ypoints);

    if (ypointsLen < npoints || xpointsLen < npoints) {
	JNU_ThrowArrayIndexOutOfBoundsException(env,
						"{xpoints,ypoints}, npoints");
	return;
    }
    jint *xpointsPtr = NULL;
    try {
        xpointsPtr = (jint *)env->GetPrimitiveArrayCritical(xpoints, 0);

	jint *ypointsPtr = NULL;
	try {
	    ypointsPtr = (jint *)env->GetPrimitiveArrayCritical(ypoints, 0);
	    // This line is the only difference between this and drawPolygon
	    // try code sharing later -br.
	    int result = graphics->Polyline(xpointsPtr, ypointsPtr, npoints,
					    FALSE);
	} catch (...) {
	    if (ypointsPtr != NULL) {
	        env->ReleasePrimitiveArrayCritical(ypoints, ypointsPtr, 0);
	    }
	    throw;
	}

	env->ReleasePrimitiveArrayCritical(ypoints, ypointsPtr, 0);
    } catch (...) {
        if (xpointsPtr != NULL) {
	    env->ReleasePrimitiveArrayCritical(xpoints, xpointsPtr, 0);
	}
	throw;
    }

    env->ReleasePrimitiveArrayCritical(xpoints, xpointsPtr, 0);

    CATCH_BAD_ALLOC;
}


/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    fillPolygon
 * Signature: ([I[II)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devFillPolygon(JNIEnv *env, jobject self,
					      jintArray xpoints,
					      jintArray ypoints,
					      jint npoints,
					      jboolean evenodd)
{
    TRY;

    DTRACE_PRINTLN4("%x: WGraphics.fillPolygon(%x, %x, %d)",
	       self, xpoints, ypoints, npoints);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics == NULL || xpoints == NULL || ypoints == NULL) {
	JNU_ThrowNullPointerException(env, "arguments || graphics");
	return;
    }
    int xpointsLen = env->GetArrayLength(xpoints);
    int ypointsLen = env->GetArrayLength(ypoints);

    if (ypointsLen < npoints || xpointsLen < npoints) {
	JNU_ThrowArrayIndexOutOfBoundsException(env,
						"{xpoints,ypoints}, npoints");
	return;
    }
    jint *xpointsPtr = NULL;
    try {
        xpointsPtr = (jint *)env->GetPrimitiveArrayCritical(xpoints, 0);
	jint *ypointsPtr = NULL;

	try {
	    ypointsPtr = (jint *)env->GetPrimitiveArrayCritical(ypoints, 0);
	    int result = graphics->FillPolygon(xpointsPtr, ypointsPtr, npoints,
					       evenodd);
	} catch (...) {
	    if (ypointsPtr != NULL) {
	        env->ReleasePrimitiveArrayCritical(ypoints, ypointsPtr, 0);
	    }
	    throw;
	}

	env->ReleasePrimitiveArrayCritical(ypoints, ypointsPtr, 0);
    } catch (...) {
        if (xpointsPtr != NULL) {
	    env->ReleasePrimitiveArrayCritical(xpoints, xpointsPtr, 0);
	}
	throw;
    }

    env->ReleasePrimitiveArrayCritical(xpoints, xpointsPtr, 0);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    devFillSpans
 * Signature: (Lsun/java2d/pipe/SpanIterator;J)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devFillSpans(JNIEnv *env, jobject self,
					    jobject si,
					    jlong pIterator) 
{
    TRY;

    DTRACE_PRINTLN3("%x: WGraphics.fillSpans(%x, %d)",
	       self, si, (int) pIterator);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);
    SpanIteratorFuncs *pFuncs = (SpanIteratorFuncs *) jlong_to_ptr(pIterator);

    if (graphics == NULL || pFuncs == NULL) {
	JNU_ThrowNullPointerException(env, "native iterator || graphics");
	return;
    }

    graphics->FillSpans(env, si, pFuncs);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawOval
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawOval(JNIEnv *env, jobject self,
					jint x, jint y, jint w, jint h){
    TRY;

    DTRACE_PRINTLN5("%x: WGraphics.drawOval(%d, %d, %d, %d)", self, x, y, w, h);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->Ellipse(x, y, w, h);
#ifdef DEBUG
    } else {
	DebugBreak();
#endif
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    fillOval
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devFillOval(JNIEnv *env, jobject self,
					jint x, jint y, jint w, jint h){
    TRY;

    DTRACE_PRINTLN5("%x: WGraphics.fillOval(%d, %d, %d, %d)", self, x, y, w, h);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->FillEllipse(x, y, w, h);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    drawArc
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devDrawArc(JNIEnv *env, jobject self,
				       jint x, jint y, jint w, jint h,
				       jint startAngle, jint endAngle)
{
    TRY;

    DTRACE_PRINTLN7("%x: WGraphics.drawArc(%d, %d, %d, %d %d %d)",
	       self, x, y, w, h, startAngle, endAngle);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
	graphics->Arc(x, y, w, h, startAngle, endAngle);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    fillArc
 * Signature: (IIIIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devFillArc(JNIEnv *env, jobject self,
				       jint x, jint y, jint w, jint h,
				       jint startAngle, jint endAngle)
{
    TRY;

    DTRACE_PRINTLN7("%x: WGraphics.fillArc(%d, %d, %d, %d %d %d)",
               self, x, y, w, h, startAngle, endAngle);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);

    if (graphics != NULL) {
        graphics->FillArc(x, y, w, h, startAngle, endAngle);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    devPrint
 * Signature: (Lsun/awt/windows/WComponentPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_devPrint(JNIEnv *env, jobject self,
					jobject peer)
{
    TRY;

    DTRACE_PRINTLN2("%x: WGraphics.print(%x)", self, peer);
    AwtGraphics* graphics = (AwtGraphics *)GET_GRAPHICS_PDATA(self);
    if (graphics != NULL) {
	int pData;
	JNI_CHECK_PEER_RETURN(peer);
        AwtComponent* c = (AwtComponent*)pData;
	c->SendMessage(WM_AWT_PRINT_COMPONENT, (WPARAM)graphics,
		       MAKELPARAM(graphics->GetOX(), graphics->GetOY()));
    }

    CATCH_BAD_ALLOC;
}

static void
setImageDataFields(JNIEnv *env, jobject joImageData, VideoInfo& vi,
                                        RECT& rDevice, RECT& rOutput)
{
    env->SetIntField(joImageData, gImageData.dataID, (jint)(vi.vpData));
    env->SetIntField(joImageData, gImageData.lutDataID, (jint)(vi.uLutData));
    env->SetIntField(joImageData, gImageData.typeID, (jint)(vi.nType));
    env->SetIntField(joImageData, gImageData.lutDataLengthID,
                                                (jint)(vi.lLutDataLength));
    env->SetIntField(joImageData, gImageData.grayInverseLutDataID,
                     (jint)(AwtToolkit::GetInstance().m_grayInverseLUT));
    env->SetIntField(joImageData, gImageData.pixelStrideID,
                                                (jint)(vi.nPixelStride));
    env->SetIntField(joImageData, gImageData.scanlineStrideID,
                                                (jint)(vi.nScanlineStride));
    env->SetIntField(joImageData, gImageData.numChannelsID,
                                                (jint)(vi.nNumChannels));
    env->SetIntField(joImageData, gImageData.bytePerChannelID,
                                                (jint)(vi.nBytePerChannel));
    if (vi.sdPrimary.ddpfPixelFormat.dwRGBBitCount < 8 &&
            vi.sdPrimary.ddpfPixelFormat.dwRGBBitCount > 0) {
        env->SetIntField(joImageData, gImageData.pixelsPerDataUnitID,
                            8/vi.sdPrimary.ddpfPixelFormat.dwRGBBitCount);
    } else {
        env->SetIntField(joImageData, gImageData.pixelsPerDataUnitID, 1);
    }

    env->SetIntField(joImageData, gImageData.xDeviceAreaID,
                                                (jint)(rDevice.left));
    env->SetIntField(joImageData, gImageData.yDeviceAreaID,
                                                (jint)(rDevice.top));
    env->SetIntField(joImageData,
                        gImageData.dxDeviceAreaID,
                        (jint)(rDevice.right - rDevice.left));
    env->SetIntField(joImageData,
                        gImageData.dyDeviceAreaID,
                        (jint)(rDevice.bottom - rDevice.top));

    env->SetIntField(joImageData, gImageData.xOutputAreaID,
                                                (jint)(rOutput.left));
    env->SetIntField(joImageData, gImageData.yOutputAreaID,
                                                (jint)(rOutput.top));
    env->SetIntField(joImageData,
                        gImageData.dxOutputAreaID,
                        (jint)(rOutput.right - rOutput.left));
    env->SetIntField(joImageData,
                        gImageData.dyOutputAreaID,
                        (jint)(rOutput.bottom - rOutput.top));
    env->SetIntField(joImageData,gImageData.platformInfoID, (jint)(&vi));
    env->SetIntField(joImageData,gImageData.deviceInfoID, (jint)(&(vi.hDC)));
}

JNIEXPORT jint JNICALL
Java_sun_awt_windows_WGraphics_W32LockViewResources(
    JNIEnv *env, jobject joSelf, jobject joImageData,
    jint xView, jint yView, jint dxView, jint dyView,
    jint lockFlags)
{
    AwtGraphics *pGraphics = NULL;

    try {
        int nRC = sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
	imageLock.Enter();

//  Get AwtGraphics object
	pGraphics =
	    (AwtGraphics*)env->GetIntField(joSelf, AwtGraphics::pDataID);
	if (pGraphics) {
	    RECT rView, rDevice, rOutput;

	    pGraphics->m_VideoInfo.nLockMethod = lockFlags;
	    rView.left  = xView;
	    rView.top   = yView;
	    rView.right = xView + dxView;
	    rView.bottom= yView + dyView;

// Perform request
#if 0
	    for (int xx=0;xx<10;xx++) {
	        rView.left = -1;
		pGraphics->LockScreen(rView, rDevice, rOutput);
		pGraphics->UnLockScreen();
		rView.left = xView;
	    }
#endif
	    nRC = pGraphics->LockScreen(rView, rDevice, rOutput);

	    if (sun_awt_windows_WGraphics_LOCK_OK == nRC ) {
// Copy information back to java structure
	        setImageDataFields(env, joImageData, pGraphics->m_VideoInfo,
				   rDevice, rOutput);
	    }
	}
	if (sun_awt_windows_WGraphics_LOCK_OK != nRC ) {
	    imageLock.Leave();
	}
	return (jint)nRC;

    } catch (std::bad_alloc&) {
        imageLock.Leave();
	if (pGraphics) {
	    pGraphics->UnLockScreen();
	}
	handle_bad_alloc();
	return sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
    }
}

JNIEXPORT jint JNICALL Java_sun_awt_windows_WGraphics_W32UnLockViewResources
  (JNIEnv *env, jobject joSelf, jobject joImageData)
{
    try {
        int nRC = sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;

//  Get AwtGraphics object
	AwtGraphics* pGraphics =
	    (AwtGraphics*)env->GetIntField(joSelf,AwtGraphics::pDataID);
	if (pGraphics) {
            double sx, sy;
            getScaleFactorFromImageData(env, joImageData, &sx, &sy);
            pGraphics->SetScaleFactorX(sx);
            pGraphics->SetScaleFactorY(sy);
	    nRC = pGraphics->UnLockScreen();
            pGraphics->SetScaleFactorX(1.0);
            pGraphics->SetScaleFactorY(1.0);
	}
	imageLock.Leave();
	return (jint)nRC;

    } catch (std::bad_alloc&) {
        imageLock.Leave();
	handle_bad_alloc();
	return sun_awt_windows_WGraphics_LOCK_DATA_STRUCT_PROBLEM;
    }
}

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    AwtGraphics::foregroundID = env->GetFieldID(cls, "foregroundColor", "Ljava/awt/Color;");
    AwtGraphics::pDataID = env->GetFieldID(cls, "pData", "I");

    AwtGraphics::getFontMID = env->GetMethodID(cls, "getFont", "()Ljava/awt/Font;");

    DASSERT(AwtGraphics::foregroundID != NULL);
    DASSERT(AwtGraphics::pDataID != NULL);

    DASSERT(AwtGraphics::getFontMID != NULL);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */
