/*
 * @(#)awt_Graphics.h	1.115 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
#ifndef _AWTGRAPHICS_H_
#define _AWTGRAPHICS_H_

#include "awt.h"
#include "awt_image.h"
#include "awt_Brush.h"
#include "awt_Component.h"
#include "awt_Pen.h"
#include "awt_Toolkit.h"

#include <ddraw.h>
#include <sun_java2d_loops_GraphicsPrimitive.h>

#include "SpanIterator.h"

extern int LogicalSelectClipRgn(HDC hDC, HRGN hClipRgn);

class AwtFont;

/************************************************************************
 * AwtSharedDC class
 */

class AwtSharedDC
{
public:
    AwtSharedDC(AwtComponent* pComponent);
    AwtSharedDC(HDC hdc);

    INLINE AwtComponent* GetComponent() {
        return m_pComponent;
    }

    INLINE HWND GetHWnd() {
        return (m_pComponent == NULL) ? NULL : m_pComponent->GetHWnd();
    }

    HDC GetDC();
    HDC GrabDC(class AwtGraphics *pNewOwner);
    void CloseDC();

    void AddRef();
    void Release();

    void Attach();
    void Detach(class AwtGraphics *pOldOwner);

    void Lock();
    void Unlock();

    /* Returns the mutex for this SharedDC. */
private:
    ~AwtSharedDC();
    /* Get rid of the Windows DC object */
    void Init();

    int			 m_attachCount;
    int			 m_useCount;
    class AwtGraphics	*m_pOwner;
    HDC			 m_hDC;
    AwtComponent	*m_pComponent;
    CriticalSection      m_lockDraw;
    CriticalSection 	 m_lockSelf;

#ifdef DEBUG
    int                  m_lockCount;
#endif
};


/************************************************************************
 * AwtGraphics class
 */

#define METHOD_INVALID  0x00
#define METHOD_DDRAW    0x01
#define METHOD_DIB      0x02
#define METHOD_GDI      0x03

typedef struct tagBitmapheader  {
    BITMAPINFOHEADER bmiHeader;
    DWORD            dwMasks[256];
}   Bitmapheader, *LPBITMAPHEADER;

class VideoInfo {
public :
    // common
    HDC hDC;

    // DDraw
    DDSURFACEDESC       sdPrimary;
    WORD                wVideoMethod;

    // DIB
    Bitmapheader    bmhHeader;
    HBITMAP         hbmpBitmap;
    HPALETTE        hPalette;
    unsigned char*  ptrImageData;

    // GDI ... nil

    //      Lock method
    int     nLockMethod;        //  Do we lock for read or write?

    //      Image related
    void*   vpData;             //  Address of data buffer
    int     nType;              //  Java image type.

    int     m_palSeed;
    unsigned long uLutData[256];//  Color lookup table
    long    lLutDataLength;     //  Length of CLUT

    int     nPixelStride;       //  Distance between pixels in pixel elements
    int     nScanlineStride;    //  Distance between pixel rows in pixelel
    int     nNumChannels;       //  Number of channels
    int     nBytePerChannel;    //  Bytes per channel
    int     nXSize;             //  X size
    int     nYSize;             //  Y size

    RECT    rOutput;

public:
    ~VideoInfo() {
    }
    VideoInfo() {
        memset(this, 0, sizeof(VideoInfo));
    }
};

class AwtGraphics : public AwtObject
{
public:
    /* java.awt.Color foreground field for sun.awt.windows.WGraphics */
    static jfieldID foregroundID;

    /* sun.awt.windows.WGraphics fields */
    static jfieldID pDataID;

    /* java.awt.Font getFont method for sun.awt.windows.WGraphics */
    static jmethodID getFontMID;

    AwtGraphics();
    virtual ~AwtGraphics();

   /*
    * Access methods
    */
    INLINE int GetRop() { return m_rop; }

   /*
    * Operations
    *
    * Creates a graphics context on or from 'source'. These functions should
    * always be called on the Toolkit thread.
    */
    struct AwtGraphicsCreate {
        jobject wGraphics;
        void *source;
    };
    static AwtGraphics *CreateFromComponent(AwtGraphicsCreate *pagc);
    static AwtGraphics *CreateCopy(AwtGraphicsCreate *pagc);
    static AwtGraphics *CreateFromHDC(AwtGraphicsCreate *pagc);

    INLINE AwtComponent* GetComponent() {
        if (m_pSharedDC == NULL) {return NULL;}
        return m_pSharedDC->GetComponent();
    }

    INLINE HWND GetHWnd() {
        if (m_pSharedDC == NULL) {return NULL;}
        return m_pSharedDC->GetHWnd();
    }

    INLINE HDC GetDC() {
        if (m_pSharedDC != NULL) {
            m_pSharedDC->Lock();
	    try {
	        if (m_hDC == NULL) {
		    ValidateDC();
		    if (m_hDC == NULL) {
		        m_pSharedDC->Unlock();
			return NULL;
		    }
		} else if (m_bClipping) {
		    VERIFY(LogicalSelectClipRgn(m_hDC, m_hClipRgn) != ERROR);
		}
	    } catch (...) {
	        m_pSharedDC->Unlock();
		throw;
	    }
	    return m_hDC;
        }
        return NULL;
    }

    INLINE void ReleaseDC() {
        if (m_pSharedDC != NULL) {
	    try {
	        if (m_pSharedDC->GetComponent() != NULL) {
		    // Bug #4055623 - Performance enhancements
		    if (m_bClipping) {
		        LogicalSelectClipRgn(m_hDC, NULL);
		    }
		}
	    } catch (...) {
	        m_pSharedDC->Unlock();
		throw;
	    }
            m_pSharedDC->Unlock();
        }
    }

    class AutoReleaseDC {
    public:
        INLINE AutoReleaseDC(AwtGraphics *graphics) : m_graphics(graphics) {}
	INLINE ~AutoReleaseDC() { m_graphics->ReleaseDC(); }
    private:
	AwtGraphics *m_graphics;
    };

    void ValidateDC();
    void InvalidateDC();

    /* Sets the m_color instance variable to the absolute COLORREF */
    void ChangeColor(COLORREF color);

    /*
     * Calculates the appropriate new COLORREF according to the
     * current ROP and calls ChangeColor on the new COLORREF
     */
    void SetColor(COLORREF color);

    void SetFont(AwtFont *pFont);

    INLINE void SetIsPrinter(BOOL b) { m_isPrinter = b; }

    void SetOrigin(jint tx, jint ty) {
	m_originX = tx;
	m_originY = ty;
    }

    INLINE AwtFont* GetFont() { return m_pFont; }

    void SetRop(int rop, COLORREF fgcolor, COLORREF color);

    void ChangeClip(long nX, long nY, long nW, long nH, BOOL set);
    void RemoveClip();

    /* Returns the clipping rectangle.  If there is none, return NULL. */
    RECT* GetClipRect();

    /*
     * Draws a line using the foreground color between points (nX1, nY1)
     * and (nX2, nY2).  The end-point is painted.
     */
    void DrawLine(JNIEnv* env, long nX1, long nY1, long nX2, long nY2);

    POINT* TransformPoly(long* xpoints, long* ypoints, POINT *points,
			 int *pNpoints, BOOL close);

    BOOL Polyline(long* xpoints, long* ypoints, int nPoints, BOOL close);

    BOOL FillPolygon(long* xpoints, long* ypoints, int nPoints, BOOL evenodd);

    void FillSpans(JNIEnv *env, jobject si, SpanIteratorFuncs *span);

    void ClearRect(JNIEnv* env, COLORREF bgcolor, long nX, long nY, long nW, long nH);

    void FillRect(JNIEnv* env, long nX, long nY, long nW, long nH);

    void DrawRect(JNIEnv* env, long nX, long nY, long nW, long nH);

    void RoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH);

    void FillRoundRect(long nX, long nY, long nW, long nH, long nArcW,
                       long nArcH);

    void Ellipse(long nX, long nY, long nW, long nH);

    void FillEllipse(long nX, long nY, long nW, long nH);

    /*
     * Converts an angle given in 1/64ths of a degree to a pair of
     * coordinates that intersects the line produced by the angle.
     */
    void AngleToCoords(long nAngle, long *nX, long *nY);

    void Arc(long nX, long nY, long nW, long nH, long nStartAngle,
             long nEndAngle);

    void FillArc(long nX, long nY, long nW, long nH, long nStartAngle,
                 long nEndAngle);

    INLINE void SetBrushColor(BOOL bSetBrush) {
        if (bSetBrush != m_bBrushSet) {
            ChangeBrush(bSetBrush);
        }
    };
    void ChangeBrush(BOOL bSetBrush);
    INLINE void SetPenColor(BOOL bSetPen) {
        if (bSetPen != m_bPenSet) {
            ChangePen(bSetPen);
        }
    };
    void ChangePen(BOOL bSetPen);

    /*
     * Draws the string at the specified point.  (nX, nY) designates
     * the reference point of the string - i.e. the top of the string
     * plus the ascent.  The width of the string is returned.
     */
    long DrawStringWidth(wchar_t *pzString, long nX, long nY);
    long DrawStringWidth(jstring string, jobject font, long nX, long nY);

    void CopyArea(long nDstX, long nDstY, long nX, long nY, long nW, long nH);

    /* Delete any AwtGraphics objects which were not disposed. */
    static void Cleanup();

    void PrintComponent(AwtComponent* c);

    void SetJavaObject(JNIEnv *env, jobject obj);
    int GetOX() {
	return m_originX;
    }
    int GetOY() {
	return m_originY;
    }

// internal methods
    BOOL CanUseDirectDraw(void);
    void ReleaseDDResources(void);

// Variables
public:
    HDC m_hDC;
    class AwtSharedDC *m_pSharedDC;

protected:
    // TRUE iff the underlying DC was obtained from ::PrintDlg, FALSE
    // otherwise
    BOOL m_isPrinter;
    BOOL m_supportsBitBlt;

private:
    BOOL m_bPaletteSelected;
    RECT m_rectClip;
    HRGN m_hClipRgn;
    BOOL m_bClipping;

    jint m_originX;
    jint m_originY;

    AwtFont *m_pFont;

    COLORREF m_color;
    COLORREF m_xorcolor;
    BOOL m_bBrushSet;
    AwtBrush* m_hBrush;
    BOOL m_bPenSet;
    AwtPen* m_hPen;
    int m_rop;

    jobject m_javaObject;

    HBITMAP     m_cachedDIB;
    BITMAPINFO  m_cachedDIBbi;
    int         m_cachedDIBnUsage;
    void        *m_cachedDIBpImage;

    // members which can be shared across multiple instances
    static long m_DDRefCount;                   // number of instances of this class
    static HINSTANCE m_hLibDDraw;               // DirectDraw Library handle
    static LPDIRECTDRAW m_DDObject;             // DirectDraw COM object
    static LPDIRECTDRAWSURFACE m_lpDDSPrimary;  // DirectDraw primary surface
    static BOOL m_fDDFailed;                    // one-way ratchet ... once failed, never try again

private:
    BOOL CheckCursorArea(RECT rView);
    BOOL CheckMonitorArea(RECT rView);
    BOOL SimpleOutputArea(RECT& rClip, const RECT& rView);
    BOOL SimpleViewArea(RECT& rClip, const RECT& rView);
    void PreparePixelFormat(void);
    void GetLUT(VideoInfo& vi);
    HBITMAP getDIBSection( HDC hDC, BITMAPINFO *bi,
                           int nUsage, void **pImage);
    int _dd_lock(const RECT&, RECT&, RECT&);
    int _dd_unlock();
    int _gdi_lock(const RECT&, RECT&, RECT&);
    int _gdi_unlock();
    int _dib_lock(const RECT&, RECT&, RECT&);
    int _dib_unlock();

public:
    VideoInfo m_VideoInfo;

    int  LockScreen(const RECT&, RECT&, RECT&);
    int	 UnLockScreen();

 private:
    int m_nProtectCount;
    void _get_curs_rect(RECT&);

 public:
    INLINE void SetScaleFactorX(double sx) { m_scaleFactorX = sx; }
    INLINE void SetScaleFactorY(double sy) { m_scaleFactorY = sy; }
    INLINE double GetScaleFactorX() { return m_scaleFactorX; }
    INLINE double GetScaleFactorY() { return m_scaleFactorY; }

 private:
    double m_scaleFactorX;
    double m_scaleFactorY;
};


/************************************************************************
 * Miscellaneous Macros
 */

extern int LogicalSelectClipRgn(HDC hDC, HRGN hClipRgn);

#define GET_GRAPHICS_PDATA(obj) (env->GetIntField(obj, AwtGraphics::pDataID))

#define SET_GRAPHICS_PDATA(obj, val) \
    (env->SetIntField(obj, AwtGraphics::pDataID, val))

#define GET_JAVAFONT(g) \
    (Hjava_awt_Font *)execute_java_dynamic_method \
        (EE(), (HObject *)(g), "getFont", "()Ljava/awt/Font;")

#define JNI_GET_JAVAFONT(wGraphics) \
    env->CallObjectMethod(wGraphics, AwtGraphics::getFontMID)

#endif /* AWTGRAPHICS_H */
