/*
 * @(#)awt_MasterPrinter.cpp	1.9 00/04/06
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_MasterPrinter.h"
#include "awt_Dimension.h"
#include "awt_Font.h"
#include "awt_PrintControl.h"
#include "awt_PrintJob.h"
#include "awt_PrintDialog.h"

#include <winspool.h>

#define ROUNDTOINT(x) ((int)((x)+0.5))
static const int DEFAULT_RES = 72;
static const double TENTHS_MM_TO_POINTS = 3.527777778;


/************************************************************************
 * AwtMasterPrinterGraphics
 */

AwtMasterPrinterGraphics::AwtMasterPrinterGraphics()
    : AwtGraphics()
{
    m_isPrinter = TRUE;
    m_printingPage = FALSE;
    m_printingJob = FALSE;
    m_fullBleed = FALSE;
    m_printResolution = 0;
    m_hPrinter = NULL;
}

AwtMasterPrinterGraphics::~AwtMasterPrinterGraphics()
{
    // End the final page and the document -- if they were already
    // ended, these commands will be ignored
    EndPage();
    EndPrintJob();
}

BOOL CALLBACK PrintDlgHook(HWND hDlg, UINT iMsg, UINT wParam, LONG lParam)
{
    TRY;

    if (iMsg == WM_INITDIALOG) {
	SetForegroundWindow(hDlg);
	return FALSE;
    }
    return FALSE;

    CATCH_BAD_ALLOC_RET(TRUE);
}

BOOL AwtMasterPrinterGraphics::CreateDevModeAndDevNames(PRINTDLG *ppd,
							LPTSTR pPrinterName,
							LPTSTR pPortName)
{
    DWORD cbNeeded = 0;
    LPBYTE pPrinter = NULL;
    BOOL retval = FALSE;

    try {
        if (!::OpenPrinter(pPrinterName, &m_hPrinter, NULL)) {
	    goto done;
	}
	VERIFY(::GetPrinter(m_hPrinter, 2, NULL, 0, &cbNeeded) == 0);
	if (::GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
	    goto done;
	}
	pPrinter = new BYTE[cbNeeded];
	if (!::GetPrinter(m_hPrinter, 2, pPrinter, cbNeeded, &cbNeeded)) {
	    goto done;
	}
	PRINTER_INFO_2 *info2 = (PRINTER_INFO_2 *)pPrinter;

	// Create DEVMODE, if it exists.
	if (info2->pDevMode != NULL) {
	    size_t devmodeSize =
	        sizeof(DEVMODE) + info2->pDevMode->dmDriverExtra;
	    ppd->hDevMode = ::GlobalAlloc(GHND, devmodeSize);
	    if (ppd->hDevMode == NULL) {
	        throw std::bad_alloc();
	    }
	    DEVMODE *devmode = (DEVMODE *)::GlobalLock(ppd->hDevMode);
	    DASSERT(!::IsBadWritePtr(devmode, devmodeSize));
	    memcpy(devmode, info2->pDevMode, devmodeSize);
	    VERIFY(::GlobalUnlock(ppd->hDevMode) == 0);
	    DASSERT(::GetLastError() == NO_ERROR);
	}

	// Create DEVNAMES.
	if (IS_NT) {
	    if (pPortName != NULL) {
	        info2->pPortName = pPortName;
	    } else if (info2->pPortName != NULL) {
	        // pPortName may specify multiple ports. We only want one.
	        info2->pPortName = strtok(info2->pPortName, ",");
	    }
	}

	size_t lenDriverName = ((info2->pDriverName != NULL)
				    ? strlen(info2->pDriverName)
				    : 0) + 1;
	size_t lenPrinterName = ((pPrinterName != NULL)
				     ? strlen(pPrinterName)
				     : 0) + 1;
	size_t lenOutputName = ((info2->pPortName != NULL)
				    ? strlen(info2->pPortName)
				    : 0) + 1;
	
	ppd->hDevNames = ::GlobalAlloc(GHND,
				       sizeof(DEVNAMES) +
				       lenDriverName +
				       lenPrinterName +
				       lenOutputName
				       );
	if (ppd->hDevNames == NULL) {
	    throw std::bad_alloc();
	}
	DEVNAMES *devnames =
	    (DEVNAMES *)::GlobalLock(ppd->hDevNames);
	DASSERT(!IsBadWritePtr(devnames, sizeof(DEVNAMES) + lenDriverName +
			       lenPrinterName + lenOutputName));
	char *lpcDevnames = (char *)devnames;
	devnames->wDriverOffset = sizeof(DEVNAMES);
	devnames->wDeviceOffset = sizeof(DEVNAMES) + lenDriverName;
	devnames->wOutputOffset =
	    sizeof(DEVNAMES) + lenDriverName + lenPrinterName;
	if (info2->pDriverName != NULL) {
	    strcpy(lpcDevnames + devnames->wDriverOffset,
		   info2->pDriverName);
	} else {
	    *(lpcDevnames + devnames->wDriverOffset) = '\0';
	}
	if (pPrinterName != NULL) {
	    strcpy(lpcDevnames + devnames->wDeviceOffset,
		   pPrinterName);
	} else {
	    *(lpcDevnames + devnames->wDeviceOffset) = '\0';
	}
	if (info2->pPortName != NULL) {
	    strcpy(lpcDevnames + devnames->wOutputOffset,
		   info2->pPortName);
	} else {
	    *(lpcDevnames + devnames->wOutputOffset) = '\0';
	}
	VERIFY(::GlobalUnlock(ppd->hDevNames) == 0);
	DASSERT(::GetLastError() == NO_ERROR);
    } catch (std::bad_alloc&) {
        if (ppd->hDevNames != NULL) {
	    VERIFY(::GlobalFree(ppd->hDevNames) == NULL);
	    ppd->hDevNames = NULL;
	}
	if (ppd->hDevMode != NULL) {
	    VERIFY(::GlobalFree(ppd->hDevMode) == NULL);
	    ppd->hDevMode = NULL;
	}
	delete [] pPrinter;
	VERIFY(::ClosePrinter(m_hPrinter));
	m_hPrinter = NULL;
	throw;
    }

    retval = TRUE;

done:
    delete [] pPrinter;
    if (m_hPrinter) {
        VERIFY(::ClosePrinter(m_hPrinter));
	m_hPrinter = NULL;
    }
    return retval;
}

BOOL AwtMasterPrinterGraphics::StartPrintJob(jobject printCtrl, jobject pj)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(3) < 0)
        return FALSE;

    HWND hwndOwner = NULL;
    jobject dialogOwner =
        env->GetObjectField(printCtrl, AwtPrintControl::dialogOwnerPeerID);
    if (dialogOwner != NULL) {
        AwtComponent *dialogOwnerComp =
	    (AwtComponent *)JNI_GET_PDATA(dialogOwner);
	hwndOwner = dialogOwnerComp->GetHWnd();
	env->DeleteLocalRef(dialogOwner);
	dialogOwner = NULL;
    }

    jobject mdh = NULL;
    jobject dest = NULL;
    jobject select = NULL;
    jobject dialog = NULL;
    jstring docname = NULL;
    char *lpcNames = NULL, *pDriverName = NULL, *pPrinterName = NULL;
    BOOL bypassNativeDialog = FALSE;
    BOOL freeOutputFileName = FALSE;
    jint maxPage = 0;

    PRINTDLG pd;
    DOCINFO di;
    DEVNAMES *devnames = NULL;
    DEVMODE *devmode = NULL;



    // Get default PRINTDLG, DEVNAMES, and DEVMODE values for the specified
    // printer.

    memset(&pd, 0, sizeof(PRINTDLG));
    pd.lStructSize = sizeof(PRINTDLG);

    // If the user didn't specify a printer, then this call returns the
    // name of the default printer.
    jstring printerName = (jstring)
        env->CallObjectMethod(printCtrl, AwtPrintControl::getPrinterID);

    if (printerName != NULL) {
        LPTSTR foundPrinter = NULL;
	LPTSTR foundPort = NULL;
	DWORD cbBuf = 0;

	VERIFY(AwtPrintControl::FindPrinter(NULL, NULL, &cbBuf, NULL, NULL));
	LPBYTE buffer = new BYTE[cbBuf];
	if (AwtPrintControl::FindPrinter(printerName, buffer, &cbBuf,
					 &foundPrinter, &foundPort)) {
	    if (!CreateDevModeAndDevNames(&pd, foundPrinter, foundPort)) {
	        delete [] buffer;
		JNU_ThrowInternalError(env, "Unable to open printer");
	        goto done;
	    }
	    DASSERT(pd.hDevNames != NULL);
	} else {
	    delete [] buffer;
	    JNU_ThrowIllegalArgumentException(
                env, "Invalid value for property printer");
	    return FALSE;
	}

	delete [] buffer;
    } else {
        // There is no default printer. This means that there are no
        // printers installed at all.
        dialog = env->CallObjectMethod(printCtrl,
				       AwtPrintControl::getDialogID);

	if (env->IsSameObject(dialog, AwtPrintControl::NATIVE)) {
	    // If we were to display the native print dialog, instead
	    // display a native warning message by calling PrintDlg
	    // without the PD_RETURNDEFAULT flag set.
	    AwtPrintDialog::PrintDlg(&pd);
	} else {
	    JNU_ThrowInternalError(env, "No default printer installed");
	}

	env->DeleteLocalRef(dialog); dialog = NULL;
	goto done;
    }



    // Now, set-up the struct for the real calls to ::PrintDlg and ::CreateDC

    pd.hwndOwner = hwndOwner;
    pd.Flags = PD_ENABLEPRINTHOOK;
    pd.lpfnPrintHook = (LPPRINTHOOKPROC)PrintDlgHook;

    mdh = env->CallObjectMethod(printCtrl, AwtPrintControl::getMDHID);
    if (env->IsSameObject(mdh,
		      AwtPrintControl::SEPARATE_DOCUMENTS_COLLATED_COPIES)) {
        pd.Flags |= PD_COLLATE;
    }
    env->DeleteLocalRef(mdh); mdh = NULL;
    dest = env->CallObjectMethod(printCtrl, AwtPrintControl::getDestID);
    if (env->IsSameObject(dest, AwtPrintControl::FILE)) {
        pd.Flags |= PD_PRINTTOFILE;
    }
    env->DeleteLocalRef(dest); dest = NULL;
    if (!env->CallBooleanMethod(printCtrl,
				AwtPrintControl::getPrintToFileEnabledID)) {
        pd.Flags |= PD_DISABLEPRINTTOFILE;
    }
    dialog = env->CallObjectMethod(printCtrl, AwtPrintControl::getDialogID);
    if (!env->IsSameObject(dialog, AwtPrintControl::NATIVE)) {
        // Don't set PD_RETURNDEFAULT because PrintDlg will return an error
        // if PD_RETRUNDEFAULT is set, but hDevMode or hDevNames is non-null
        bypassNativeDialog = TRUE;
    }
    env->DeleteLocalRef(dialog); dialog = NULL;
    select = env->CallObjectMethod(printCtrl, AwtPrintControl::getSelectID);
    if (env->IsSameObject(select, AwtPrintControl::ALL)) {
        pd.Flags |= PD_ALLPAGES;
    } else if (env->IsSameObject(select, AwtPrintControl::RANGE)) {
        pd.Flags |= PD_PAGENUMS;
    } else {
        pd.Flags |= PD_SELECTION;
    }
    env->DeleteLocalRef(select); select = NULL;

    pd.nFromPage = (WORD)env->CallIntMethod(printCtrl, 
                                            AwtPrintControl::getFromPageID);
    pd.nToPage = (WORD)env->CallIntMethod(printCtrl, 
                                          AwtPrintControl::getToPageID);
    pd.nMinPage = (WORD)env->CallIntMethod(printCtrl,
                                           AwtPrintControl::getMinPageID);
    maxPage = env->CallIntMethod(printCtrl, AwtPrintControl::getMaxPageID);
    pd.nMaxPage = (maxPage <= (jint)((WORD)-1)) ? (WORD)maxPage : (WORD)-1;

    pd.nCopies = (WORD)env->CallIntMethod(printCtrl,
                                          AwtPrintControl::getCopiesID);



    // Set-up DEVMODE struct, if it exists. Win16 printer drivers do not
    // support DEVMODE.

    if (pd.hDevMode != NULL) {
        devmode = (DEVMODE *)::GlobalLock(pd.hDevMode);
	DASSERT(!IsBadWritePtr(devmode, sizeof(DEVMODE)));

        devmode->dmFields |= DM_COPIES | DM_COLLATE | DM_ORIENTATION |
                             DM_PAPERSIZE | DM_PRINTQUALITY | DM_COLOR |
                             DM_DUPLEX;

	// We have to set this copies field because its value could override
	// the value of pd.nCopies in the native print dialog. We have to
	// reset it to 1 before we call ::CreateDC, however.
        devmode->dmCopies = pd.nCopies;

        devmode->dmCollate = (pd.Flags & PD_COLLATE) ? DMCOLLATE_TRUE
                                                     : DMCOLLATE_FALSE;

        jobject orient = env->CallObjectMethod(printCtrl,
                                               AwtPrintControl::getOrientID);
        if (env->IsSameObject(orient, AwtPrintControl::PORTRAIT)) {
            devmode->dmOrientation = DMORIENT_PORTRAIT;
        } else if (env->IsSameObject(orient,
                                     AwtPrintControl::LANDSCAPE)) {
            devmode->dmOrientation = DMORIENT_LANDSCAPE;
        }
        env->DeleteLocalRef(orient); orient = NULL;

	short paperSize =
	    env->CallCharMethod(printCtrl, AwtPrintControl::getWin32MediaID);
	if (paperSize > 0) {
	    devmode->dmPaperSize = paperSize;
	}

        jobject quality =
	    env->CallObjectMethod(printCtrl, AwtPrintControl::getQualityID);
        if (env->IsSameObject(quality, AwtPrintControl::HIGH)) {
            devmode->dmPrintQuality = DMRES_HIGH;
        } else if (env->IsSameObject(quality,
                                     AwtPrintControl::NORMAL)) {
            devmode->dmPrintQuality = DMRES_MEDIUM;
        } else {
            devmode->dmPrintQuality = DMRES_DRAFT;
        }
        env->DeleteLocalRef(quality); quality = NULL;

        jobject color = env->CallObjectMethod(printCtrl,
                                              AwtPrintControl::getColorID);
        if (env->IsSameObject(color, AwtPrintControl::COLOR)) {
            devmode->dmColor = DMCOLOR_COLOR;
        } else {
            devmode->dmColor = DMCOLOR_MONOCHROME;
        }
        env->DeleteLocalRef(color); color = NULL;

	jobject sides = env->CallObjectMethod(printCtrl,
					      AwtPrintControl::getSidesID);
	if (env->IsSameObject(sides, AwtPrintControl::ONE_SIDED)) {
	    devmode->dmDuplex = DMDUP_SIMPLEX;
	} else if (env->IsSameObject(sides,
				     AwtPrintControl::TWO_SIDED_LONG_EDGE)) {
	    devmode->dmDuplex = DMDUP_VERTICAL;
	} else {
	    devmode->dmDuplex = DMDUP_HORIZONTAL;
	}
	env->DeleteLocalRef(sides); sides = NULL;

        ::GlobalUnlock(pd.hDevMode);
	devmode = NULL;
    }



    // Show native print dialog if requested

    if (!bypassNativeDialog) {
        if (!AwtPrintDialog::PrintDlg(&pd)) {
	    // user cancelled printing
	    goto done;
	}
    }



    // Open the printer and create a DC for it. We always do this explicitly
    // so that we have full control of the DEVNAMES and DEVMODE structs.

    DASSERT(pd.hDevNames != NULL);

    devnames = (DEVNAMES *)::GlobalLock(pd.hDevNames);
    DASSERT(!IsBadReadPtr(devnames, sizeof(DEVNAMES)));

    lpcNames = (char *)devnames;
    pDriverName = (IS_NT) 
        ? lpcNames + devnames->wDriverOffset
        : NULL;
    pPrinterName = lpcNames + devnames->wDeviceOffset;

    // Corresponding call to ClosePrinter in EndPrintJob
    if (!::OpenPrinter(pPrinterName, &m_hPrinter, NULL)) {
        goto done;
    }

    // Update the DEVMODE structure for the default printer
    // CreateDC will fail if we pass a DEVMODE for the printer which is
    // not the current DEVMODE for that printer
    if (pd.hDevMode != NULL) {
        devmode = (DEVMODE *)::GlobalLock(pd.hDevMode);
	DASSERT(!IsBadWritePtr(devmode, sizeof(DEVMODE)));

	// Reset copies to 1. Multiple copies should always be rendered
	// by the Java application.
	if (devmode->dmFields & DM_COPIES) {
	    if (devmode->dmCopies > pd.nCopies) {
	        pd.nCopies = devmode->dmCopies;
	    }
	    devmode->dmCopies = 1;
	}

	if (::DocumentProperties(NULL, m_hPrinter, pPrinterName,
				 devmode, devmode, 
				 DM_IN_BUFFER | DM_OUT_BUFFER) != IDOK) {
	    ::GlobalUnlock(pd.hDevMode);
	    devmode = NULL;
	    goto done;
	}
    }
	    
    pd.hDC = ::CreateDC(pDriverName, pPrinterName, NULL, devmode);
    if (pd.hDC == NULL) {
        goto done;
    }

    if (pd.hDevMode != NULL) {
        ::GlobalUnlock(pd.hDevMode);
	devmode = NULL;
    }
    ::GlobalUnlock(pd.hDevNames);
    devnames = NULL;



    // Setup DOCINFO struct

    memset(&di, 0, sizeof(DOCINFO));
    di.cbSize = sizeof(DOCINFO);

    docname = (jstring)
        env->GetObjectField(printCtrl, AwtPrintControl::doctitleID);
    if (docname != NULL) {
        JavaStringBuffer docbuf(env, docname);
	di.lpszDocName = strdup((char *)docbuf);
    }
    env->DeleteLocalRef(docname); // docname = NULL;

    if (pd.Flags & PD_PRINTTOFILE) {
        jstring filename =
	    (jstring)env->CallObjectMethod(printCtrl,
					   AwtPrintControl::getFileNameID);
	if (filename != NULL) {
	    JavaStringBuffer filenamebuf(env, filename);
	    di.lpszOutput = strdup((char *)filenamebuf);
	    freeOutputFileName = TRUE;
	} else if (pd.hDevNames != NULL && !bypassNativeDialog) {
	    devnames = (DEVNAMES *)::GlobalLock(pd.hDevNames);
	    DASSERT(!IsBadReadPtr(devnames, sizeof(DEVNAMES)));
	    char *lpcNames = (char *)devnames;
	    di.lpszOutput = strdup(lpcNames + devnames->wOutputOffset);
	    ::GlobalUnlock(pd.hDevNames);
	    devnames = NULL;
	    freeOutputFileName = TRUE;
	} else {
	    di.lpszOutput = "FILE:";
	}
	env->DeleteLocalRef(filename);
    }



    // Call StartDoc. If the call succeeds, we really are going to print.

    if (::StartDoc(pd.hDC, &di) <= 0) {
        // If ::GetLastError() returns ERROR_CANCELLED or
        // ERROR_PRINT_CANCELLED, the user cancelled the operation.
        DWORD errorCode = ::GetLastError();
	if (errorCode != ERROR_CANCELLED &&
	        errorCode != ERROR_PRINT_CANCELLED) {
	    JNU_ThrowInternalError(env, "printing StartDoc failed");
	}
    } else {
        m_printingJob = TRUE;
            
	jobject origin = NULL;
	int res = 0;
	jobject pageDimension = NULL;

	// are we offsetting the origin?
	origin = env->CallObjectMethod(printCtrl,
				       AwtPrintControl::getOriginID);
	m_fullBleed = env->IsSameObject(origin,
					AwtPrintControl::PHYSICAL);
	env->DeleteLocalRef(origin); origin = NULL;

	// We support any print resolution because we emulate all of
	// them.
	m_printResolution =
	    env->CallIntMethod(printCtrl, AwtPrintControl::getResID);

	// Set pageResolution and pageDimension
	int iResolutionX = ::GetDeviceCaps(pd.hDC, LOGPIXELSX);
	int iResolutionY = ::GetDeviceCaps(pd.hDC, LOGPIXELSY);
	int iPhysicalX = ::GetDeviceCaps(pd.hDC, PHYSICALWIDTH);
	int iPhysicalY = ::GetDeviceCaps(pd.hDC, PHYSICALHEIGHT);
	int iPrintableX = ::GetDeviceCaps(pd.hDC, HORZRES);
	int iPrintableY = ::GetDeviceCaps(pd.hDC, VERTRES);

	int iExportX, iExportY;
	if (m_fullBleed) {
	    iExportX = iPhysicalX;
	    iExportY = iPhysicalY;
	} else {
	    iExportX = iPrintableX;
	    iExportY = iPrintableY;
	}

	pageDimension =
	    env->GetObjectField(pj, AwtPrintJob::pageDimensionID);
	env->SetIntField(pj, AwtPrintJob::truePageResolutionID,
			 iResolutionX);
	env->SetIntField(pj, AwtPrintJob::pageResolutionID, 
			 m_printResolution);
	env->SetIntField(pageDimension, AwtDimension::widthID,
			 ROUNDTOINT((iExportX/(double)iResolutionX) *
				    m_printResolution));
	env->SetIntField(pageDimension, AwtDimension::heightID,
			 ROUNDTOINT((iExportY/(double)iResolutionY) *
				    m_printResolution));
	env->DeleteLocalRef(pageDimension); pageDimension = NULL;

	// Does the printer support BitBlt? Plotters don't.
	m_supportsBitBlt =
	    (::GetDeviceCaps(pd.hDC, RASTERCAPS) & RC_BITBLT) != 0;

	m_pSharedDC = new AwtSharedDC(pd.hDC);
	m_pSharedDC->Attach();
	
	// update Attributes objects
	if (pd.hDevMode != NULL) {
	    devmode = (DEVMODE *)::GlobalLock(pd.hDevMode);
	    DASSERT(!IsBadReadPtr(devmode, sizeof(DEVMODE)));
	}
	env->CallVoidMethod(printCtrl, AwtPrintControl::setMDHID,
            (pd.Flags & PD_COLLATE)
	        ? AwtPrintControl::SEPARATE_DOCUMENTS_COLLATED_COPIES
		: AwtPrintControl::SEPARATE_DOCUMENTS_UNCOLLATED_COPIES);
	if (devmode != NULL && devmode->dmFields & DM_COLOR) {
	  env->CallVoidMethod(printCtrl,
               AwtPrintControl::setColorID,
                (devmode->dmColor == DMCOLOR_COLOR) 
                    ? AwtPrintControl::COLOR
                    : AwtPrintControl::MONOCHROME);
	}
	env->CallVoidMethod(printCtrl,
			    AwtPrintControl::setDestID,
			    (pd.Flags & PD_PRINTTOFILE)
				? AwtPrintControl::FILE
				: AwtPrintControl::PRINTER);
	// Set printer even when printing to a file because specific
	// printers could produce different output files (e.g., PCL v.
	// PostScript) or support different paper sizes
	if (pd.hDevNames != NULL) {
	    devnames = (DEVNAMES*)::GlobalLock(pd.hDevNames);
	    DASSERT(!IsBadReadPtr(devnames, sizeof(DEVNAMES)));
	    char *lpcNames = (char *)devnames;
	    char *buf = strdup(lpcNames + devnames->wDeviceOffset);
	    if (buf != NULL) {
	        env->CallVoidMethod(printCtrl,
				    AwtPrintControl::setPrinterID,
				    AwtFont::multibyte2javaString(env, buf));
		free(buf);
	    }
	    ::GlobalUnlock(pd.hDevNames);
	    devnames = NULL;
	}
	switch (pd.Flags & (PD_ALLPAGES | PD_PAGENUMS | PD_SELECTION)) {
	  case PD_ALLPAGES:
	    env->CallVoidMethod(printCtrl,
				AwtPrintControl::setSelectID,
				AwtPrintControl::ALL);
	    break;
	  case PD_PAGENUMS:
	    env->CallVoidMethod(printCtrl,
				AwtPrintControl::setSelectID,
				AwtPrintControl::RANGE);
	    break;
	  case PD_SELECTION:
	    env->CallVoidMethod(printCtrl,
				AwtPrintControl::setSelectID,
				AwtPrintControl::SELECTION);
	    break;
	}
	if (devmode != NULL && devmode->dmFields & DM_DUPLEX) {
	    jobject sides;
	    if (devmode->dmDuplex == DMDUP_SIMPLEX) {
	        sides = AwtPrintControl::ONE_SIDED;
	    } else if (devmode->dmDuplex == DMDUP_HORIZONTAL) {
	        sides = AwtPrintControl::TWO_SIDED_SHORT_EDGE;
	    } else {
	        sides = AwtPrintControl::TWO_SIDED_LONG_EDGE;
	    }
	    env->CallVoidMethod(printCtrl,
				AwtPrintControl::setSidesID, sides);
	}
	env->CallVoidMethod(printCtrl,
			    AwtPrintControl::setCopiesID,
			    pd.nCopies);
	if (devmode != NULL && devmode->dmFields & DM_ORIENTATION) {
	    env->CallVoidMethod(printCtrl,
				AwtPrintControl::setOrientID,
				(devmode->dmOrientation == DMORIENT_PORTRAIT) 
				    ? AwtPrintControl::PORTRAIT
				    : AwtPrintControl::LANDSCAPE);
	}
	if (devmode != NULL && devmode->dmFields & DM_PAPERSIZE) {
	    short paperSize = devmode->dmPaperSize;
	    if (paperSize >= DMPAPER_FIRST &&
		paperSize <= DMPAPER_FANFOLD_LGL_GERMAN) {
	        env->CallVoidMethod(printCtrl,
				    AwtPrintControl::setWin32MediaID,
				    paperSize, 0, 0);
	    } else if (paperSize == 0 ||
		       paperSize > DMPAPER_FANFOLD_LGL_GERMAN) {
	        short length = ROUNDTOINT((iPhysicalY/(double)iResolutionY) *
					  DEFAULT_RES * TENTHS_MM_TO_POINTS);
		short width = ROUNDTOINT((iPhysicalX/(double)iResolutionX) *
					 DEFAULT_RES * TENTHS_MM_TO_POINTS);
		env->CallVoidMethod(printCtrl,
				    AwtPrintControl::setWin32MediaID,
				    paperSize, width, length);
	    }
	}
	if (devmode != NULL && devmode->dmFields & DM_PRINTQUALITY) {
	    switch (devmode->dmPrintQuality) {
	      case DMRES_HIGH:
		env->CallVoidMethod(printCtrl,
				    AwtPrintControl::setQualityID,
				    AwtPrintControl::HIGH);
		break;
	      case DMRES_MEDIUM:
		env->CallVoidMethod(printCtrl,
				    AwtPrintControl::setQualityID,
				    AwtPrintControl::NORMAL);
		break;
	      case DMRES_LOW:
	      case DMRES_DRAFT:
		  env->CallVoidMethod(printCtrl,
				      AwtPrintControl::setQualityID,
				      AwtPrintControl::DRAFT);
		  break;
	    }
	}
	// Set toPage before fromPage because fromPage can never
	// exceed toPage.
	env->CallVoidMethod(printCtrl, AwtPrintControl::setToPageID,
			    pd.nToPage);
	env->CallVoidMethod(printCtrl, AwtPrintControl::setFromPageID,
			    pd.nFromPage);
	if (devmode != NULL) {
	    ::GlobalUnlock(pd.hDevMode);
	    devmode = NULL;
	}
    }
    free((void *)(di.lpszDocName));
    if (freeOutputFileName) {
        free((void *)(di.lpszOutput));
    }

done:
    if (pd.hDevMode != NULL) {
        ::GlobalFree(pd.hDevMode);
    }
    if (pd.hDevNames != NULL) {
        ::GlobalFree(pd.hDevNames);
    }

    return m_printingJob;
}

void AwtMasterPrinterGraphics::StartPage()
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    AwtGraphics::AutoReleaseDC ardc(this);

    if (::StartPage(hDC) <= 0) {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        JNU_ThrowInternalError(env, "printing StartPage failed");
    }
    else {
        VERIFY(::SetMapMode(hDC, MM_ISOTROPIC) > 0);

        VERIFY(::SetWindowExtEx(hDC, m_printResolution, m_printResolution,
				NULL));
        VERIFY(::SetViewportExtEx(hDC,
                                  ::GetDeviceCaps(hDC, LOGPIXELSX),
                                  ::GetDeviceCaps(hDC, LOGPIXELSY),
                                  NULL));
        if (m_fullBleed) {
            VERIFY(::SetViewportOrgEx(hDC,
                                      -::GetDeviceCaps(hDC, PHYSICALOFFSETX),
                                      -::GetDeviceCaps(hDC, PHYSICALOFFSETY),
                                      NULL));
        }

        // Initialize pen and brush to match AwtGraphics settings.
        VERIFY(::SelectObject(hDC, ::GetStockObject(NULL_PEN)));
        VERIFY(::SelectObject(hDC, ::GetStockObject(NULL_BRUSH)));

        // NETIVA FIX  (4078973) - printer DC's default to transparent just
        // like screen DC's
        VERIFY(::SetBkMode(hDC, TRANSPARENT));
        // NETIVA FIX - end

        m_printingPage = TRUE;
    }
}

void AwtMasterPrinterGraphics::EndPage()
{
    if (m_printingPage) {
        HDC hDC = GetDC();
        if (hDC == NULL) {
            return;
	}
	AwtGraphics::AutoReleaseDC ardc(this);

        if (::EndPage(hDC) <= 0) {
	    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
	    JNU_ThrowInternalError(env, "printing EndPage failed");
        }
	else {
	    m_printingPage = FALSE;
	}
    }
}

void AwtMasterPrinterGraphics::EndPrintJob()
{
    if (m_printingJob) {
        HDC hDC = GetDC();
	if (hDC == NULL) {
	    return;
	}
	AwtGraphics::AutoReleaseDC ardc(this);
	JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    
	if (::EndDoc(hDC) <= 0) {
	    JNU_ThrowInternalError(env, "printing EndDoc failed");
	} else if (m_hPrinter != NULL && ::ClosePrinter(m_hPrinter) == 0) {
	    JNU_ThrowInternalError(env, "printing EndDoc failed");
	} else {
	    m_printingJob = FALSE;
	    m_hPrinter = NULL;
	}
    }
}

void AwtMasterPrinterGraphics::CreateAndStart(void *lpvcass)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    AwtMasterPrinterGraphics::CreateAndStartStruct *lpcass =
        (AwtMasterPrinterGraphics::CreateAndStartStruct *)lpvcass;

    AwtMasterPrinterGraphics *pGraphics = new AwtMasterPrinterGraphics();
    pGraphics->SetJavaObject(env, lpcass->self);
    SET_GRAPHICS_PDATA(lpcass->self, (jint)pGraphics);

    if (!pGraphics->StartPrintJob(lpcass->control, lpcass->job)) {
        delete pGraphics;
    }

    jthrowable xcp = safe_ExceptionOccurred(env);
    if (xcp != NULL) {
        env->ExceptionClear();
        lpcass->xcp = (jthrowable)env->NewGlobalRef(xcp);
	env->DeleteLocalRef(xcp);
    }
}

/************************************************************************
 * WGraphics native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WGraphics
 * Method:    createFromPrintJob
 * Signature: (Lsun/awt/windows/WPrintJob;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WGraphics_createFromPrintJob(JNIEnv *env, jobject self,
						  jobject control,
						  jobject job)
{
    TRY;

    DTRACE_PRINTLN3("%x: WGraphics.createFromPrintJob(%x)", self, control,
		    job);
    if (control == NULL) {
	JNU_ThrowNullPointerException(env, "print control argument");
	return;
    }
    if (job == NULL) {
	JNU_ThrowNullPointerException(env, "print job argument");
	return;
    }

    AwtMasterPrinterGraphics::CreateAndStartStruct cass = 
        { self, control, job, NULL };
    AwtToolkit::GetInstance().InvokeFunction(
        AwtMasterPrinterGraphics::CreateAndStart, &cass);
    if (cass.xcp != NULL) {
        env->Throw(cass.xcp);
	env->DeleteGlobalRef(cass.xcp);
    }

    CATCH_BAD_ALLOC;
}

} /* extern "C" */
