/*
 * @(#)awt_Pen.cpp	1.7 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_Pen.h"

GDIHashtable AwtPen::cache("Pen cache", DeleteAwtPen);

AwtPen::AwtPen(COLORREF color) {
    SetColor(color);
    HPEN pen = ::CreatePen(PS_SOLID, 1, color);
    /*
     * Fix for BugTraq ID 4191297.
     * If GDI resource creation failed flush all GDIHashtables
     * to destroy unreferenced GDI resources.
     */
    if (pen == NULL) {
        cache.flushAll();
        pen = ::CreatePen(PS_SOLID, 1, color);
    }
    DASSERT(pen != NULL);
    SetHandle(pen);
}

AwtPen* AwtPen::Get(COLORREF color) {

    CriticalSection::Lock l(cache.getManagerLock());

    AwtPen* obj = (AwtPen*)cache.get((void*)color);
    if (obj == NULL) {
        obj = new AwtPen(color);
        VERIFY(cache.put((void*)color, obj) == NULL);
    }
    obj->IncrRefCount();
    return obj;
}

void AwtPen::ReleaseInCache() {

    CriticalSection::Lock l(cache.getManagerLock());

    if (DecrRefCount() == 0) {
        cache.release((void*)GetColor()); 
    }
}

void AwtPen::DeleteAwtPen(void* pPen) {
    delete (AwtPen*)pPen;
}
