/*
 * @(#)awt_PrintJob.cpp	1.51 01/02/12
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <math.h>
#include <windef.h>
#include <wtypes.h>
#include <winuser.h>
#include <commdlg.h>
#include <winspool.h>

#include "awt.h"
#include "awt_PrintJob.h"
#include "awt_Toolkit.h"
#include "awt_Component.h"
#include "awt_Font.h"
#include "awt_MasterPrinter.h"
#include "awt_PrintDialog.h"

#include <sun_awt_windows_WPrintJob.h>
#include <sun_awt_windows_WPrinterJob.h>

#define DEBUG_PRINTING	1

/* Round 'num' to the nearest integer and return
 * the result as a long.
 */
#define ROUND_TO_LONG(num)    ((long) floor((num) + 0.5))

/* Round 'num' to the nearest integer and return
 * the result as an int.
 */
#define ROUND_TO_INT(num)     ((int) floor((num) + 0.5))

/************************************************************************
 * AwtPrintJob fields
 */

jfieldID AwtPrintJob::pageDimensionID;
jfieldID AwtPrintJob::pageResolutionID;
jfieldID AwtPrintJob::truePageResolutionID;
jfieldID AwtPrintJob::graphicsID;


/************************************************************************
 * WPrintJob native methods
 */   

extern "C" {

/*** Private Constants ***/

static char *kJavaIntStr = "I";

/* 2D printing uses 3 byte BGR pixels in Raster printing */
static int J2DRasterBPP = 3;

/*
 * Class Names
 */
static const char *PRINTEREXCEPTION_STR = "java/awt/print/PrinterException";

/*
 * The following strings are the names of instance variables in WPrintJob2D.
 */
static const char *PRINTPAPERSIZE_STR = "mPrintPaperSize"; // The paper size
static const char *PRINTHDMODE_STR = "mPrintHDevMode";   // The print devmode
static const char *PRINTDNAME_STR = "mPrintHDevNames";  // The print devnames
static const char *PRINTDC_STR = "mPrintDC";	// The print device context.
static const char *XRES_STR = "mPrintXRes";	// The x dpi.
static const char *YRES_STR = "mPrintYRes";	// The y dpi.
static const char *PHYSX_STR = "mPrintPhysX";   // pixel x of printable area
static const char *PHYSY_STR = "mPrintPhysY";   // pixel y of printable area
static const char *PHYSW_STR = "mPrintWidth";   // pixel wid of printable area
static const char *PHYSH_STR = "mPrintHeight";  // pixel hgt of printable area
static const char *PAGEW_STR = "mPageWidth";    // pixel wid of page
static const char *PAGEH_STR = "mPageHeight";   // pixel hgt of page

static const char *DRIVER_COPIES_STR = "driverDoesMultipleCopies";
static const char *DRIVER_COLLATE_STR = "driverDoesCollation";
static const char *USER_COLLATE_STR = "userRequestedCollation";

static const char *SETRANGE_STR = "setPageRange";
static const char *SETRANGE_SIG = "(II)V";
/*
 * Methods and fields in the interface awt.print.Pageable.
 */

// public int getNumberOfPages()

static const char *GETNUMPAGES_STR = "getNumberOfPages";
static const char *GETNUMPAGES_SIG = "()I";

// public int getCopies()

static const char *GETCOPIES_STR = "getCopies";
static const char *GETCOPIES_SIG = "()I";

// public void setCopies()

static const char *SETCOPIES_STR = "setCopies";
static const char *SETCOPIES_SIG = "(I)V";

// public String getJobName()

static const char *GETJOBNAME_STR = "getJobName";
static const char *GETJOBNAME_SIG = "()Ljava/lang/String;";

/*
 * Methods and fields in awt.print.PageFormat.
 */

// public Paper getPaper()
static const char *GETPAPER_STR = "getPaper";
static const char *GETPAPER_SIG = "()Ljava/awt/print/Paper;";

// public void setPaper(Paper paper)
static const char *SETPAPER_STR = "setPaper";
static const char *SETPAPER_SIG = "(Ljava/awt/print/Paper;)V";

// public int getOrientation()
static const char *GETORIENT_STR = "getOrientation";
static const char *GETORIENT_SIG = "()I";

// public void setOrientation(int orientation)
static const char *SETORIENT_STR = "setOrientation";
static const char *SETORIENT_SIG = "(I)V";

// public static final int PORTRAIT = 0;
static const char *PORTRAIT_STR = "PORTRAIT";
static const char *PORTRAIT_SIG = "I";

// public static final int LANDSCAPE = 1;
static const char *LANDSCAPE_STR = "LANDSCAPE";
static const char *LANDSCAPE_SIG = "I";

/*
 * Methods in awt.print.Paper.
 */

// public void setSize(double width, double height)
static const char *SETSIZE_STR = "setSize";
static const char *SETSIZE_SIG = "(DD)V";

// protected void setImageableArea(double x, double y, double width,
//						    double height)
static const char *SETIMAGEABLE_STR = "setImageableArea";
static const char *SETIMAGEABLE_SIG = "(DDDD)V";

// public double getWidth()
static const char *GETWIDTH_STR = "getWidth";
static const char *GETWIDTH_SIG = "()D";

// public double getHeight()
static const char *GETHEIGHT_STR = "getHeight";
static const char *GETHEIGHT_SIG = "()D";

// public double getImageableX()
static const char *GETIMG_X_STR = "getImageableX";
static const char *GETIMG_X_SIG = "()D";

// public double getImageableY()
static const char *GETIMG_Y_STR = "getImageableY";
static const char *GETIMG_Y_SIG = "()D";

// public double getImageableWidth()
static const char *GETIMG_W_STR = "getImageableWidth";
static const char *GETIMG_W_SIG = "()D";

// public double getImageableHeight()
static const char *GETIMG_H_STR = "getImageableHeight";
static const char *GETIMG_H_SIG = "()D";

/* Multiply a Window's MM_HIENGLISH value 
 * (1000th of an inch) by this number to
 * get a value in 72nds of an inch.
 */
static const double HIENGLISH_TO_POINTS = (72.0 / 1000.0);

/* Multiply a Window's MM_HIMETRIC value
 * (100ths of a millimeter) by this
 * number to get a value in 72nds of an inch.
 */
static const double HIMETRIC_TO_POINTS = (72.0 / 2540.0);

/* Multiply a Window's MM_LOMETRIC value
 * (10ths of a millimeter) by this
 * number to get a value in 72nds of an inch.
 */
static const double LOMETRIC_TO_POINTS = (72.0 / 254.0);

/* Multiply a measurement in 1/72's of an inch by this
 * value to convert it to Window's MM_HIENGLISH
 * (1000th of an inch) units.
 */
static const double POINTS_TO_HIENGLISH = (1000.0 / 72.0);

/* Multiply a measurement in 1/72's of an inch by this
 * value to convert it to Window's MM_HIMETRIC
 * (100th of an millimeter) units.
 */
static const double POINTS_TO_HIMETRIC = (2540.0 / 72.0);

/* Multiply a measurement in 1/72's of an inch by this
 * value to convert it to Window's MM_LOMETRIC
 * (10th of an millimeter) units.
 */
static const double POINTS_TO_LOMETRIC = (254.0 / 72.0);

/*** Private Macros ***/

/* A Page Setup paint hook passes a word describing the
   orientation and type of page being displayed in the
   dialog. These macros break the word down into meaningful
   values.
*/
#define PRINTER_TYPE_MASK   (0x0003)
#define PORTRAIT_MASK	    (0x0004)
#define ENVELOPE_MASK	    (0x0008)

#define IS_ENVELOPE(param)  (((param) & ENVELOPE_MASK) != 0)
#define IS_PORTRAIT(param)  (((param) & PORTRAIT_MASK) != 0)

/*	If the Pagable does not know the number of pages in the document,
	then we limit the print dialog to this number of pages.
*/
#define MAX_UNKNOWN_PAGES 9999

/* When making a font that is already at least bold,
 * bolder then we increase the LOGFONT lfWeight field
 * by this amount.
 */
#define EMBOLDEN_WEIGHT   (100)

/* The lfWeight field of a GDI LOGFONT structure should not
 * exceed this value.
 */
#define MAX_FONT_WEIGHT   (1000)

/*** Private Variable Types ***/

typedef struct {
    jdouble x;
    jdouble y;
    jdouble width;
    jdouble height;
} RectDouble;

/*** Private Prototypes ***/

static UINT CALLBACK pageDlgHook(HWND hDlg, UINT msg,
				 UINT wParam, LONG lParam);
static UINT CALLBACK printDlgHook(HWND hDlg, UINT msg,
				  UINT wParam, LONG lParam);
static void initPrinter(JNIEnv *env, jobject self);
static HDC getDefaultPrinterDC(JNIEnv *env, jobject printerJob);
static void pageFormatToSetup(JNIEnv *env, jobject job, jobject page, 
                              PAGESETUPDLG *setup, HDC hDC);
static WORD getOrientationFromDevMode(JNIEnv *env, jobject self);
static void setOrientationInDevMode(HGLOBAL hDevMode, jboolean isPortrait);
static int bitsToDevice(HDC printDC, jbyte *image, long destX, long destY,
			long width, long height);
static void retrievePaperInfo(const PAGESETUPDLG *setup, POINT *paperSize,
				RECT *margins, jboolean *isPortrait);
static void setRange(JNIEnv *env, jobject self, jint from, jint to);
static jint getNumPages(JNIEnv *env, jobject doc);
static jint getCopies(JNIEnv *env, jobject printerJob);
static void setCopies(JNIEnv *env, jobject printerJob, int copies);
static LPTSTR getJobName(JNIEnv *env, jobject printerJob);
static jobject getPaper(JNIEnv *env, jobject page);
static void setPaper(JNIEnv *env, jobject page, jobject paper);
static jboolean isPortraitOrientation(JNIEnv *env, jobject page);
static jboolean isLandscapeOrientation(JNIEnv *env, jobject page);
static void setOrientation(JNIEnv *env, jobject page, jboolean isPortrait);
static void getPaperValues(JNIEnv *env, jobject paper, RectDouble *paperSize,
			  RectDouble *margins, BOOL widthAsMargin=TRUE);
static void setPaperValues(JNIEnv *env, jobject paper, const POINT *paperSize,
			    const RECT *margins, int units);
static long convertFromPoints(double value, int units);
static double convertToPoints(long value, int units);
static void setCapabilities(JNIEnv *env, jobject self, HDC printDC);
static HDC getPrintDC(JNIEnv *env, jobject self);
static void setPrintDC(JNIEnv *env, jobject self, HDC printDC);
static HGLOBAL getPrintHDMode(JNIEnv *env, jobject self);
static void setPrintHDMode(JNIEnv *env, jobject self, HGLOBAL hGlobal);
static HGLOBAL getPrintHDName(JNIEnv *env, jobject self);
static void setPrintHDName(JNIEnv *env, jobject self, HGLOBAL hGlobal);
static inline WORD getPrintPaperSize(JNIEnv *env, jobject self);
static inline void setPrintPaperSize(JNIEnv *env, jobject self, WORD sz);
static jint getIntField(JNIEnv *env, jobject self, const char *fieldName);
static void setIntField(JNIEnv *env, jobject self,
			    const char *fieldName, jint value);
static jfieldID getIdOfIntField(JNIEnv *env, jobject self,
			    const char *fieldName);
static void setBooleanField(JNIEnv *env, jobject self,
                            const char *fieldName, jboolean value);

static jbyte *findNonWhite(jbyte *image, long sy, long width, long height,
                           long scanLineStride, long *numLinesP);
static jbyte *findWhite(jbyte *image, long sy, long width, long height, 
                           long scanLineStride, long *numLines);
static void dumpDevMode(HGLOBAL hDevMode);
static void dumpPrinterCaps(HANDLE hDevNames);
static void throwPrinterException(JNIEnv *env, DWORD err);
static void matchPaperSize(HDC printDC, HGLOBAL hDevMode, HGLOBAL hDevNames,
                           double origWid, double origHgt,
                           double* newHgt, double *newWid,
                           WORD* paperSize);

/***********************************************************************/

static jboolean jFontToWFontW(JNIEnv *env, HDC printDC, jstring fontName,
                        jfloat fontSize, jboolean isBold, jboolean isItalic,
                        jint rotation);
static jboolean jFontToWFontA(JNIEnv *env, HDC printDC, jstring fontName,
                        jfloat fontSize, jboolean isBold, jboolean isItalic,
                        jint rotation);

static int CALLBACK fontEnumProcW(ENUMLOGFONTEXW  *lpelfe,
                                 NEWTEXTMETRICEX *lpntme,
                                 int FontType,
                                 LPARAM lParam);
static int CALLBACK fontEnumProcA(ENUMLOGFONTEXA  *logfont,
                                  NEWTEXTMETRICEX  *lpntme,
                                  int FontType,
                                  LPARAM lParam);

static int embolden(int currentWeight);

/*** Code ***/

/*
 * Class:     sun_awt_windows_WPrintJob
 * Method:    end
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrintJob_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    AwtPrintJob::pageDimensionID =
      env->GetFieldID(cls, "pageDimension", "Ljava/awt/Dimension;");
    AwtPrintJob::pageResolutionID =
      env->GetFieldID(cls, "pageResolution", "I");
    AwtPrintJob::truePageResolutionID =
      env->GetFieldID(cls, "truePageResolution", "I");
    AwtPrintJob::graphicsID = 
        env->GetFieldID(cls, "graphics", "Lsun/awt/windows/WPrintGraphics;");

    DASSERT(AwtPrintJob::pageDimensionID != NULL);
    DASSERT(AwtPrintJob::pageResolutionID != NULL);
    DASSERT(AwtPrintJob::truePageResolutionID != NULL);
    DASSERT(AwtPrintJob::graphicsID != NULL);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrintJob
 * Method:    _flushPage
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrintJob_flushPageImpl(JNIEnv *env, jobject self)
{
    TRY;

    if (env->EnsureLocalCapacity(1) < 0) {
        return;
    }
    jobject hWGraphics = env->GetObjectField(self, AwtPrintJob::graphicsID);
    AwtMasterPrinterGraphics* pGraphics = (AwtMasterPrinterGraphics*)
        (env->GetIntField(hWGraphics, AwtGraphics::pDataID));

    if (pGraphics != NULL) {
        // if there is currently no page in progress, EndPage is ignored
        pGraphics->EndPage();
    }

    env->DeleteLocalRef(hWGraphics);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrintJob
 * Method:    newPage
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrintJob_newPage(JNIEnv *env, jobject self)
{
    TRY;

    if (env->EnsureLocalCapacity(1) < 0) {
        return;
    }
    jobject hWGraphics = env->GetObjectField(self, AwtPrintJob::graphicsID);
    AwtMasterPrinterGraphics* pGraphics = (AwtMasterPrinterGraphics*)
        (env->GetIntField(hWGraphics, AwtGraphics::pDataID));

    if (pGraphics != NULL) {
        // if there is currently no page in progress, EndPage is ignored
        pGraphics->EndPage();
	pGraphics->StartPage();
    }

    env->DeleteLocalRef(hWGraphics);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrintJob
 * Method:    _end
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrintJob_endImpl(JNIEnv *env, jobject self)
{
    TRY;

    if (env->EnsureLocalCapacity(1) < 0) {
        return;
    }
    jobject hWGraphics = env->GetObjectField(self, AwtPrintJob::graphicsID);
    AwtMasterPrinterGraphics* pGraphics = (AwtMasterPrinterGraphics*)
        (env->GetIntField(hWGraphics, AwtGraphics::pDataID));

    if (pGraphics != NULL) {
        // End the final page and the document -- if they were already
        // ended, these commands will be ignored. Don't dispose of the
        // awt_Graphics object, though. That's done during finalization
        // of the WGraphics object.
        pGraphics->EndPage();
        pGraphics->EndPrintJob();
    }

    env->DeleteLocalRef(hWGraphics);

    CATCH_BAD_ALLOC;
}

/************************************************************************
 * WPrinterJob native methods
 */

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    pageSetup
 * Signature: (Ljava/awt/print/PageFormat;Ljava/awt/print/Printable;)Z
 *
 * 11/28/97 Note: Currently unused, the 'painter' parameter is intended one
 * day to allow the creation of an application drawn page preview within
 * the page setup dialog.
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_windows_WPrinterJob_pageSetup
  (JNIEnv *env, jobject self, jobject page, jobject painter) {

    TRY;

    jboolean doIt = JNI_FALSE; // Assume the user will cancel the dialog.
    PAGESETUPDLG setup;
    memset(&setup, 0, sizeof(setup));

    setup.lStructSize = sizeof(setup);
    setup.hwndOwner = NULL;
    setup.hDevMode = NULL;
    setup.hDevNames = NULL;
    setup.Flags = PSD_RETURNDEFAULT | PSD_DEFAULTMINMARGINS;
    // setup.ptPaperSize = 
    // setup.rtMinMargin =
    // setup.rtMargin = 
    setup.hInstance = NULL;
    setup.lCustData = 0;
    setup.lpfnPageSetupHook = pageDlgHook;
    setup.lpfnPagePaintHook = NULL;
    setup.lpPageSetupTemplateName = NULL;
    setup.hPageSetupTemplate = NULL;

    BOOL privateDM = FALSE;

    /* Because the return default flag is set, this first call
     * will not display the dialog but will return default values, inc
     * including hDevMode, hDevName, ptPaperSize, and rtMargin values.
     * We can use the devmode to set the orientation of the page
     * and the size of the page.
     * The units used by the user is also needed.
     */
    if (getPrintHDMode(env,self) == NULL || getPrintHDName(env,self) == NULL) {
        (void)AwtPrintDialog::PageSetupDlg(&setup);
        privateDM = TRUE;
    } else {
        int measure = PSD_INTHOUSANDTHSOFINCHES;
        int sz = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IMEASURE, NULL, 0);
        if (sz > 0) {
            char *str = (char*)safe_Malloc(sizeof(char) * sz);
            sz = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IMEASURE, str, sz);
            if ((sz > 0) && (str != NULL)) {
                if (strcmp("0", str) == 0) {
                   measure = PSD_INHUNDREDTHSOFMILLIMETERS;
                }
                free((char *)str);
            }
        }
        setup.Flags |= measure;
        setup.hDevMode = getPrintHDMode(env, self);
        setup.hDevNames = getPrintHDName(env, self);
    }

    /* Move page size and orientation from the PageFormat object
     * into the Windows setup structure so that the format can
     * be displayed in the dialog.
    */
    pageFormatToSetup(env, self, page, &setup, getPrintDC(env, self));

    setup.lpfnPageSetupHook = pageDlgHook;
    setup.Flags = PSD_ENABLEPAGESETUPHOOK | PSD_DISABLEPRINTER
		  | PSD_MARGINS;


    if (AwtPrintDialog::PageSetupDlg(&setup)) {

	jobject paper = getPaper(env, page);

	int units = setup.Flags & PSD_INTHOUSANDTHSOFINCHES ?
						MM_HIENGLISH :
						MM_HIMETRIC;
	POINT paperSize;
	RECT margins;
	jboolean isPortrait;

	/* Get the Windows paper and margins description.
	*/
	retrievePaperInfo(&setup, &paperSize, &margins, &isPortrait);

	/* Convert the Windows' paper and margins description
	 * and place them into a Paper instance.
	 */
	setPaperValues(env, paper, &paperSize, &margins, units);

	/* Put the updated Paper instance and the orientation into
	 * the PageFormat.
	 */
	setPaper(env, page, paper);

	setOrientation(env, page, isPortrait);

        if (setup.hDevMode != NULL) {
            DEVMODE *devmode = (DEVMODE *)::GlobalLock(setup.hDevMode);
            if (devmode != NULL) {
                if (devmode->dmFields & DM_PAPERSIZE) {
                    setPrintPaperSize(env, self, devmode->dmPaperSize);
                }
            }
            ::GlobalUnlock(setup.hDevMode);
        }

           
	doIt = JNI_TRUE;
    }


    if (privateDM == TRUE) {
        if (setup.hDevMode != NULL) {
            ::GlobalFree(setup.hDevMode);
        }
        if (setup.hDevNames != NULL) {
            ::GlobalFree(setup.hDevNames);
        }
    } else {
        HGLOBAL oldG = getPrintHDMode(env, self);
        if (setup.hDevMode != oldG) {
            if (oldG != NULL) {
                ::GlobalFree(oldG);
            }
            setPrintHDMode(env, self, setup.hDevMode);
        }

        oldG = getPrintHDName(env, self);
        if (setup.hDevNames != oldG) {
            if (oldG != NULL) {
                ::GlobalFree(oldG);
            }
            setPrintHDName(env, self, setup.hDevNames);
        }
    }
    
    return doIt;

    CATCH_BAD_ALLOC_RET(0);
}


/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    jobSetup
 * Signature: (Ljava/awt/print/Pageable;)Z
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_windows_WPrinterJob_jobSetup
  (JNIEnv *env, jobject self, jobject doc, jboolean canPrintToFile) {
    TRY;

    jboolean doIt = JNI_FALSE;
    PRINTDLG job;

    /* If the number of pages in the document is bigger than 
     * Window's can use then we'll truncate it - pretty
     * unlikely.
     */
    jint numPages = getNumPages(env, doc);

    unsigned short winNumPages = numPages < (long )USHRT_MAX
				 ? numPages
				 : USHRT_MAX; 

    job.lStructSize = sizeof(job);
    job.hwndOwner = NULL;
    // dc, devmode, and devnames, should all be null or all initialised.
    job.hDevMode = getPrintHDMode(env, self);
    job.hDevNames = getPrintHDName(env, self);
    job.hDC = getPrintDC(env, self);
    job.Flags = PD_ENABLEPRINTHOOK | PD_NOSELECTION | PD_RETURNDC;

    if (!canPrintToFile) {
        job.Flags |= PD_DISABLEPRINTTOFILE;
    }

    job.nFromPage = 1;
    job.nToPage = (winNumPages <= 0) ? 1 : winNumPages;
    job.nMinPage = 1;

    /* Make sure the maximum page number is not less than the minimum.
     */
    job.nMaxPage = winNumPages >= job.nMinPage ? winNumPages : job.nMinPage;
    
    job.nCopies = (WORD) getCopies(env, self);
    job.hInstance = NULL;
    job.lCustData = NULL;
    job.lpfnPrintHook = printDlgHook;
    job.lpfnSetupHook = NULL;
    job.lpPrintTemplateName = NULL;
    job.lpSetupTemplateName = NULL;
    job.hPrintTemplate = NULL;
    job.hSetupTemplate = NULL;
	
    if (AwtPrintDialog::PrintDlg(&job)) {

        /* Drivers indicate support for multiple copies via the DEVMODE
         * DM_COPIES flag, and for collated copies via the DM_COLLATE flag.
         * When we support API for collation, then we will need to inspect
         * the DM_COLLATE flag. Until then we can allow the drivers
         * which support it to do so without our knowledge.
         * There's no benefit in knowing unless we need to use the info.
         * The drivers which don't support it will set the PD_COLLATE flag,
         * which we do check, and then render the copies from the application.
         */
        int devWillDoCopies = FALSE;
        int devWillDoCollation = FALSE;
        int userRequestedCollation = (job.Flags & PD_COLLATE) != 0;

        int nCopies = job.nCopies;

        if (job.hDevMode != NULL) {
            DEVMODE *devmode = (DEVMODE *)::GlobalLock(job.hDevMode);
            if (devmode != NULL) {
                if ((devmode->dmFields & DM_COPIES) && (nCopies == 1)) {
                    // takes precedence over job.nCopies if !=1
                    devWillDoCopies = TRUE;
                    nCopies = devmode->dmCopies;
                }
            }
            ::GlobalUnlock(job.hDevMode);
        }
        setCopies(env, self, nCopies);

        setBooleanField(env, self, DRIVER_COPIES_STR,
                        (devWillDoCopies ? JNI_TRUE : JNI_FALSE));
        setBooleanField(env, self, DRIVER_COLLATE_STR,
                        (devWillDoCollation ? JNI_TRUE : JNI_FALSE));
        setBooleanField(env, self, USER_COLLATE_STR,
                        (userRequestedCollation ? JNI_TRUE : JNI_FALSE));

	setCapabilities(env, self, job.hDC);

	switch (job.Flags & (PD_SELECTION | PD_PAGENUMS)) {

	case PD_ALLPAGES:
	    setRange(env, self, 0, numPages - 1);
	    break;

	case PD_PAGENUMS:
	    setRange(env, self, job.nFromPage - 1, job.nToPage - 1);
	    break;

	/* We disable the selection button in the Print dialog
	 * so we should never see that case. There are only the
	 * three cases aso anything else means something is very
	 * wrong.
	 */
	case PD_SELECTION:
	default:
	    assert(FALSE);
	    break;
	}

	doIt = JNI_TRUE;
    
        /* repeated calls to printDialog should not leak handles */

        HDC oldDC = getPrintDC(env, self);
        if (job.hDC != oldDC) {
            if (oldDC != NULL) {
                ::DeleteDC(oldDC);
            }
            setPrintDC(env, self, job.hDC);
        }

        HGLOBAL oldG = getPrintHDMode(env, self);
        if (job.hDevMode != oldG) {
            if (oldG != NULL) {
                ::GlobalFree(oldG);
            }
            setPrintHDMode(env, self, job.hDevMode);
        }

        oldG = getPrintHDName(env, self);
        if (job.hDevNames != oldG) {
            if (oldG != NULL) {
                ::GlobalFree(oldG);
            }
            setPrintHDName(env, self, job.hDevNames);
        }
    }
    return doIt;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    getDefaultPage
 * Signature: (Ljava/awt/print/PageFormat;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob_getDefaultPage(JNIEnv *env, jobject self,
                                                jobject page) {

    /* REMIND: this method currently does not take "current printer"
     * into account: it returns a default page for the system default
     * printer. This may not always be what we want.
     */

    TRY;

    PAGESETUPDLG setup;
    memset(&setup, 0, sizeof(setup));
    setup.lStructSize = sizeof(setup);
    setup.Flags = PSD_RETURNDEFAULT | PSD_DEFAULTMINMARGINS;

    /* Because the return default flag is set, this call
     * will not display the dialog but will return default
     * hDevMode, hDevName, ptPaperSize, and rtMargin values.
     * In particular we want the hDevMode so that we can obtain
     * the orientation of the page.
     */
    (void)AwtPrintDialog::PageSetupDlg(&setup);

    /* Get the Windows paper and margins description
     * Convert them and put then into a Paper instance.
     * Put the updated Paper instance and the orientation into
     * the PageFormat.
     * Note: getPaper() returns a clone, which is what we want, and
     * its the clone we update and set back into the PageFormat.
     */

    int units = setup.Flags & PSD_INTHOUSANDTHSOFINCHES ?
                                            MM_HIENGLISH :
                                            MM_HIMETRIC;
    POINT paperSize;
    RECT margins;
    jboolean isPortrait;

    retrievePaperInfo(&setup, &paperSize, &margins, &isPortrait);
    jobject paper = getPaper(env, page);
    setPaperValues(env, paper, &paperSize, &margins, units);
    setPaper(env, page, paper);
    setOrientation(env, page, isPortrait);

    if (setup.hDevMode != NULL) {
        ::GlobalFree(setup.hDevMode);
    }
    if (setup.hDevNames != NULL) {
        ::GlobalFree(setup.hDevNames);
    }

    CATCH_BAD_ALLOC;

}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    validatePaper
 * Signature: (Ljava/awt/print/Paper;Ljava/awt/print/Paper;)V
 *
 * Query the current or default printer to find all paper sizes it
 * supports and find the closest matching to the origPaper.
 * For the matching size, validate the margins and printable area
 * against the printer's capabilities.
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob_validatePaper(JNIEnv *env, jobject self,
                                         jobject origPaper, jobject newPaper) {
    TRY;

    /* If the print dialog has been displayed or a DC has otherwise
     * been created, use that. Else get a DC for the default printer
     * which we discard before returning.
     */
     
    HDC printDC = getPrintDC(env, self); 
    HGLOBAL hDevMode = getPrintHDMode(env, self); 
    HGLOBAL hDevNames = getPrintHDName(env, self); 
    BOOL privateDC = FALSE;

    if (printDC == NULL) {
        PRINTDLG pd;
        memset(&pd, 0, sizeof(PRINTDLG));
        pd.lStructSize = sizeof(PRINTDLG);
        pd.Flags = PD_RETURNDEFAULT | PD_RETURNDC;

        if (AwtPrintDialog::PrintDlg(&pd)) {
            printDC = pd.hDC;
            hDevMode = pd.hDevMode;
            hDevNames = pd.hDevNames;
            privateDC = TRUE;
        }
    }

    if (printDC == NULL) {
       return;
    }

    /* We try to mitigate the effects of floating point rounding errors
     * by only setting a value if it would differ from the value in the
     * target by at least 0.10 points = 1/720 inches.
     * eg if the values present in the target are close to the calculated
     * values then we accept the target.
     */
    const double epsilon = 0.10;

    jdouble paperWidth, paperHeight;
    WORD dmPaperSize = getPrintPaperSize(env, self);

    double ix, iy, iw, ih, pw, ph;

    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());
    jmethodID getID;

    jclass paperClass = env->GetObjectClass(origPaper);
    getID = env->GetMethodID(paperClass, GETWIDTH_STR, GETWIDTH_SIG);
    pw = env->CallDoubleMethod(origPaper, getID);
    getID = env->GetMethodID(paperClass, GETHEIGHT_STR, GETHEIGHT_SIG);
    ph = env->CallDoubleMethod(origPaper, getID);
    getID = env->GetMethodID(paperClass, GETIMG_X_STR, GETIMG_X_SIG);
    ix = env->CallDoubleMethod(origPaper, getID);
    getID = env->GetMethodID(paperClass, GETIMG_Y_STR, GETIMG_Y_SIG);
    iy = env->CallDoubleMethod(origPaper, getID);
    getID = env->GetMethodID(paperClass, GETIMG_W_STR, GETIMG_W_SIG);
    iw = env->CallDoubleMethod(origPaper, getID);
    getID = env->GetMethodID(paperClass, GETIMG_H_STR, GETIMG_H_SIG);
    ih = env->CallDoubleMethod(origPaper, getID);

    matchPaperSize(printDC, hDevMode, hDevNames, pw, ph, 
                   &paperWidth, &paperHeight, &dmPaperSize);

    /* Validate margins and imageable area */

    // pixels per inch in x and y direction
    jint xPixelRes = GetDeviceCaps(printDC, LOGPIXELSX);
    jint yPixelRes = GetDeviceCaps(printDC, LOGPIXELSY);

    // x & y coord of printable area in pixels
    jint xPixelOrg = GetDeviceCaps(printDC, PHYSICALOFFSETX);
    jint yPixelOrg = GetDeviceCaps(printDC, PHYSICALOFFSETY);

    // width & height of printable area in pixels
    jint imgPixelWid = GetDeviceCaps(printDC, HORZRES);
    jint imgPixelHgt = GetDeviceCaps(printDC, VERTRES);

    // if the values were obtained from a rotated device, swap.
    if (getOrientationFromDevMode(env, self) == DMORIENT_LANDSCAPE) {
      jint tmp;

      tmp = xPixelRes;
      xPixelRes = yPixelRes;
      yPixelRes = tmp;

      tmp = xPixelOrg;
      xPixelOrg = yPixelOrg;
      yPixelOrg = tmp;
     
      tmp = imgPixelWid;
      imgPixelWid = imgPixelHgt;
      imgPixelHgt = tmp;
    }

    // page imageable area in 1/72"
    jdouble imgX = (jdouble)((xPixelOrg * 72)/(jdouble)xPixelRes);
    jdouble imgY = (jdouble)((yPixelOrg * 72)/(jdouble)yPixelRes);
    jdouble imgWid = (jdouble)((imgPixelWid * 72)/(jdouble)xPixelRes);
    jdouble imgHgt = (jdouble)((imgPixelHgt * 72)/(jdouble)yPixelRes);

    /* Check each of the individual values is within range.
     * Then make sure imageable area is placed within imageable area.
     * Allow for a small floating point error in the comparisons
     */ 

    if (ix < 0.0 ) {
        ix = 0.0;
    }
    if (iy < 0.0 ) {
        iy = 0.0;
    }
    if (iw < 0.0) {
        iw = 0.0;
    }
    if (ih < 0.0) {
        ih = 0.0;
    }
    if ((ix + epsilon) < imgX) {
         ix = imgX;
    }
    if ((iy + epsilon) < imgY) {
         iy = imgY;
    }
    if (iw + epsilon > imgWid) {
        iw = imgWid;
    }
    if (ih + epsilon > imgHgt) {
        ih = imgHgt;
    }
    if ((ix + iw + epsilon) > (imgX+imgWid)) {
        ix = (imgX+imgWid) - iw;
    }
    if ((iy + ih + epsilon) > (imgY+imgHgt)) {
        iy = (imgY+imgHgt) - ih;
    }

    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jmethodID setSizeID = env->GetMethodID(paperClass,
                                        SETSIZE_STR, SETSIZE_SIG);
    jmethodID setImageableID = env->GetMethodID(paperClass,
                                        SETIMAGEABLE_STR, SETIMAGEABLE_SIG);

    env->CallVoidMethod(newPaper, setSizeID, paperWidth, paperHeight);
    env->CallVoidMethod(newPaper, setImageableID, ix, iy, iw, ih);

    /* Free any resources allocated */
    if (privateDC == TRUE) {
        if (printDC != NULL) {
            ::DeleteDC(printDC);
        }
        if (hDevMode != NULL) {
            ::GlobalFree(hDevMode);
        }
        if (hDevNames != NULL) {
            ::GlobalFree(hDevNames);
        }
    }

    CATCH_BAD_ALLOC;
}


static void initPrinter(JNIEnv *env, jobject self) {

    HDC printDC = getPrintDC(env, self);

    /*
     * The print device context will be NULL if the
     * user never okayed a print dialog. This
     * will happen most often when the java application
     * decides not to present a print dialog to the user.
     * We create a device context for the default printer.
     */
    if (printDC == NULL) {
	printDC = getDefaultPrinterDC(env, self);
	if (printDC){
	    setPrintDC(env, self, printDC);
	    setCapabilities(env, self, printDC);
	}
    }
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    initPrinter
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob_initPrinter(JNIEnv *env, jobject self) {
    TRY;

    initPrinter(env, self);

    CATCH_BAD_ALLOC;
}


/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    startDoc
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob__1startDoc(JNIEnv *env, jobject self) {
    TRY;

    DWORD err = 0;

    initPrinter(env, self);
    HDC printDC = getPrintDC(env, self);

    /* We do our own rotation, so device must be in portrait mode */
    HGLOBAL hDevMode = getPrintHDMode(env, self);
    if (printDC != NULL && hDevMode != NULL) {
        DEVMODE *devmode = (DEVMODE *)::GlobalLock(hDevMode);
        if (devmode != NULL) {
                devmode->dmFields |= DM_ORIENTATION;
                devmode->dmOrientation = DMORIENT_PORTRAIT;
                ::ResetDC(printDC, devmode);
        }
        ::GlobalUnlock(hDevMode);
    }

    if (printDC){
	DOCINFO docInfo;
	memset(&docInfo, 0, sizeof(DOCINFO));

	docInfo.cbSize = sizeof (DOCINFO);
	docInfo.lpszDocName = getJobName(env, self);
	docInfo.lpszOutput = (LPTSTR)NULL;
	docInfo.fwType = 0;

	err = ::StartDoc(printDC, &docInfo);
        free((void*)docInfo.lpszDocName);
	if (err <= 0) {
	    err = GetLastError();
	} else {
	    err = 0;
	}
    }
    else {
        jclass printerException = env->FindClass(PRINTEREXCEPTION_STR);
        env->ThrowNew(printerException, "No printer found.");
    }

    if (err) {
	throwPrinterException(env, err);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    endDoc
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob_endDoc(JNIEnv *env, jobject self) {
    TRY;

    HDC printDC = getPrintDC(env, self);

    if (printDC != NULL){
	::EndDoc(printDC);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    abortDoc
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob_abortDoc(JNIEnv *env, jobject self) {
    TRY;

    HDC printDC = getPrintDC(env, self);

    if (printDC != NULL){
	 ::AbortDoc(printDC);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    deleteDC
 * Signature: ()V
 * Called from finalize, not before, so repeated print() calls can be made.
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WPrinterJob_deleteDC(JNIEnv *env, jobject self) {
    TRY_NO_VERIFY;

    HDC printDC = getPrintDC(env, self);
    if (printDC != NULL){
        setPrintDC(env, self, NULL);
        ::DeleteDC(printDC);
    }

    HGLOBAL printHDMode = getPrintHDMode(env, self);
    if (printHDMode != NULL){
         setPrintHDMode(env, self, NULL);
         ::GlobalFree(printHDMode);
    }

    HGLOBAL printHDName = getPrintHDName(env, self);
    if (printHDName != NULL){
         setPrintHDName(env, self, NULL);
         ::GlobalFree(printHDName);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    deviceStartPage
 * Signature: (Ljava/awt/print/PageFormat;Ljava/awt/print/Printable;I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_deviceStartPage
(JNIEnv *env, jobject self, jobject format, jobject painter, jint pageIndex) {
    TRY;

    HDC printDC = getPrintDC(env, self);

    if (printDC != NULL){

        HGLOBAL hDevMode = getPrintHDMode(env, self);
        HGLOBAL hDevNames = getPrintHDName(env, self);
        WORD dmPaperSize = getPrintPaperSize(env, self);
        if (hDevMode != NULL && hDevNames != NULL) {
	   LONG retval = 0;
            RectDouble paperSize;
            RectDouble margins;
            jobject paper = getPaper(env, format);
            getPaperValues(env, paper, &paperSize, &margins);
            double paperWidth, paperHeight;
            matchPaperSize(printDC, hDevMode, hDevNames,
                           paperSize.width,  paperSize.height,
                           &paperWidth, &paperHeight, &dmPaperSize);

            DEVMODE *devmode = (DEVMODE *)::GlobalLock(hDevMode);
            if (devmode != NULL) {
               if (dmPaperSize == 0) {
                  devmode->dmFields |= DM_PAPERLENGTH | DM_PAPERWIDTH
                    | DM_PAPERSIZE;
                  devmode->dmPaperSize = DMPAPER_USER;
                  devmode->dmPaperWidth = 
                    (short)(convertFromPoints(paperSize.width, MM_LOMETRIC));
                  devmode->dmPaperLength = 
                    (short)(convertFromPoints(paperSize.height, MM_LOMETRIC));

		 
		  // sync with public devmode settings
		  {
		    DEVNAMES *devnames = (DEVNAMES *)::GlobalLock(hDevNames);
		    if (devnames != NULL) {
		      
		      LPSTR lpdevnames = (LPSTR)devnames;
		      LPSTR printerName = strdup((LPSTR)(lpdevnames+devnames->wDeviceOffset));
						
		      HANDLE hPrinter;
		      if (::OpenPrinter(printerName, &hPrinter, NULL)== TRUE) {

			// Need to call DocumentProperties to update change
			// in paper setting because some drivers do not update
			// it with a simple call to ResetDC.
			retval = ::DocumentProperties(NULL, hPrinter,printerName, 
					     devmode, devmode, 
					     DM_IN_BUFFER|DM_OUT_BUFFER);			 
			
			::ClosePrinter(hPrinter);	
			free ((char*)printerName);
		      }
		    }
		    
		    ::GlobalUnlock(hDevNames);
		  }
	       }
	       // if DocumentProperties() fail
	       if (retval < 0) {
		  ::GlobalUnlock(hDevMode);		 
		  return;
	       }
	       HDC res = ::ResetDC(printDC, devmode);
            }
            ::GlobalUnlock(hDevMode);
        }

	::StartPage(printDC);

        /* The origin for a glyph will be along the left
         * edge of its bnounding box at the base line.
         * The coincides with the Java text glyph origin.
         */
        ::SetTextAlign(printDC, TA_LEFT | TA_BASELINE);

        /* The background mode is used when GDI draws text,
         * hatched brushes and poen that are not solid.
         * We set the mode to transparentso that when
         * drawing text only the glyphs themselves are
         * drawn. The boundingbox of the string is not
         * erased to the background color.
         */
        ::SetBkMode(printDC, TRANSPARENT);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    deviceEndPage
 * Signature: (Ljava/awt/print/PageFormat;Ljava/awt/print/Printable;I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_deviceEndPage
(JNIEnv *env, jobject self, jobject format, jobject painter, jint pageIndex) {
    TRY;

    HDC printDC = getPrintDC(env, self);

    if (printDC != NULL){
	::EndPage(printDC);
    }

    CATCH_BAD_ALLOC;
}


/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    printBand
 * Signature: ([BIIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_printBand
  (JNIEnv *env, jobject self, jbyteArray imageArray, jint x, jint y,
					jint width, jint height) {
    TRY;

    HDC printDC = getPrintDC(env, self);

    jbyte *image = NULL;
    try {
        long scanLineStride = J2DRasterBPP * width;
        image = (jbyte *)env->GetPrimitiveArrayCritical(imageArray, 0);
	jbyte *startImage =  image + (scanLineStride * (height - 1));
	jbyte *endImage = NULL;
	long startY = 0;
	long numLines = 0;

        do {
            startImage = findNonWhite(startImage, startY, width, height,
                                      scanLineStride, &numLines);

            if (startImage != NULL) {
                startY += numLines;
                endImage = findWhite(startImage, startY, width, height,
                                     scanLineStride, &numLines);
                bitsToDevice(printDC, endImage, x, y + startY, width,
                             numLines);
                startImage = endImage - scanLineStride;
                startY += numLines;
            }
        } while (startY < height && startImage != NULL);

    } catch (...) {
        if (image != NULL) {
	    env->ReleasePrimitiveArrayCritical(imageArray, image, 0);
	}
	throw;
    }

    env->ReleasePrimitiveArrayCritical(imageArray, image, 0);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    beginPath
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_beginPath
(JNIEnv *env , jobject self, jint printDC) {
    TRY;
 
    (void) ::BeginPath((HDC) printDC);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    endPath
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_endPath
(JNIEnv *env, jobject self, jint printDC) {
    TRY;

    (void) ::EndPath((HDC) printDC);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    fillPath
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_fillPath
(JNIEnv *env, jobject self, jint printDC) {
    TRY;

    (void) ::FillPath((HDC) printDC);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    closeFigure
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_closeFigure
(JNIEnv *env, jobject self, jint printDC) {
    TRY;

    (void) ::CloseFigure((HDC) printDC);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    lineTo
 * Signature: (IFF)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_lineTo
(JNIEnv *env, jobject self, jint printDC, jfloat x, jfloat y) {
    TRY;

    (void) ::LineTo((HDC) printDC, ROUND_TO_LONG(x), ROUND_TO_LONG(y));

    CATCH_BAD_ALLOC;
}


/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    moveTo
 * Signature: (IFF)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_moveTo
(JNIEnv *env, jobject self, jint printDC, jfloat x, jfloat y) {
    TRY;

    (void) ::MoveToEx((HDC) printDC, ROUND_TO_LONG(x), ROUND_TO_LONG(y), NULL);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    polyBezierTo
 * Signature: (IFFFFFF)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_polyBezierTo
(JNIEnv *env, jobject self, jint printDC,
 jfloat control1x, jfloat control1y,
 jfloat control2x, jfloat control2y,
 jfloat endX, jfloat endY) {

    TRY;

    POINT points[3];

    points[0].x = ROUND_TO_LONG(control1x);
    points[0].y = ROUND_TO_LONG(control1y);
    points[1].x = ROUND_TO_LONG(control2x);
    points[1].y = ROUND_TO_LONG(control2y);
    points[2].x = ROUND_TO_LONG(endX);
    points[2].y = ROUND_TO_LONG(endY);

    (void) ::PolyBezierTo((HDC) printDC, points, 3);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    setPolyFillMode
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_setPolyFillMode
(JNIEnv *env, jobject self, jint printDC, jint fillRule) {
    TRY;

    (void) ::SetPolyFillMode((HDC) printDC, fillRule);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    selectSolidBrush
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_selectSolidBrush
(JNIEnv *env, jobject self, jint printDC, jint red, jint green, jint blue) {

    TRY;

    HBRUSH colorBrush = ::CreateSolidBrush(RGB(red, green, blue));
    HBRUSH oldBrush = (HBRUSH)::SelectObject((HDC) printDC, colorBrush);
    DeleteObject(oldBrush);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    getPenX
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_sun_awt_windows_WPrinterJob_getPenX
(JNIEnv *env, jobject self, jint printDC) {

    TRY;

    POINT where;
    ::GetCurrentPositionEx((HDC) printDC, &where);

    return (jint) where.x;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    getPenY
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_sun_awt_windows_WPrinterJob_getPenY
(JNIEnv *env, jobject self, jint printDC) {

    TRY;

    POINT where;
    ::GetCurrentPositionEx((HDC) printDC, &where);

    return (jint) where.y;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    selectClipPath
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_selectClipPath
(JNIEnv *env, jobject self, jint printDC) {

    TRY;

    ::SelectClipPath((HDC) printDC, RGN_COPY);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    setLogicalFont
 * Signature: (java/awt/Font;I)Z
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_windows_WPrinterJob_setLogicalFont
 (JNIEnv *env, jobject self, jobject font, jint rotation)
{
    jboolean didSetFont = JNI_FALSE;

    AwtFont* awtFont = AwtFont::GetFont(env, font, rotation);
    if (awtFont != NULL) {
        didSetFont = JNI_TRUE;
    }

    return didSetFont;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    setFont
 * Signature: (ILjava/lang/String;FZZI)Z
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_windows_WPrinterJob_setFont
  (JNIEnv *env, jobject self, jint printDC, jstring fontName,
   jfloat fontSize, jboolean isBold, jboolean isItalic, jint rotation)
{
    jboolean didSetFont = JNI_FALSE;

    if (IS_NT) {
        didSetFont = jFontToWFontW(env, (HDC) printDC,
                               fontName,
                               fontSize,
                               isBold,
                               isItalic,
                               rotation);
    } else {
        didSetFont = jFontToWFontA(env, (HDC) printDC,
                               fontName,
                               fontSize,
                               isBold,
                               isItalic,
                               rotation);
    }

    return didSetFont;
}

/**
 * Try to convert a java font to a GDI font. On entry, 'printDC',
 * is the device context we want to draw into. 'fontName' is
 * the name of the font to be matched and 'fontSize' is the
 * size of the font in device coordinates. If there is an
 * equivalent GDI font then this function sets that font
 * into 'printDC' and returns a 'true'. If there is no equivalent
 * font then 'false' is returned.
 */
static jboolean jFontToWFontA(JNIEnv *env, HDC printDC, jstring fontName,
                        jfloat fontSize, jboolean isBold, jboolean isItalic,
                        jint rotation)
{
    LOGFONTA lf;
    LOGFONTA matchedLogFont;
    BOOL foundFont = false;     // Assume we didn't find a matching GDI font.

    memset(&matchedLogFont, 0, sizeof(matchedLogFont));

    WCHAR* name = TO_WSTRING(fontName);
    size_t nameLen = wcslen(name);

    /* Some fontnames of Non-ASCII fonts like 'MS Minchou' are themselves
     * Non-ASCII.  They are assumed to be written in Unicode.
     * Hereby, they are converted into platform codeset.
     */
    int destLen = WideCharToMultiByte(CP_ACP,   // convert to ASCII code page
                                      0,        // flags
                                      name,     // Unicode string
                                      nameLen,  // Unicode length
                                      lf.lfFaceName, // Put ASCII string here
                                      sizeof(lf.lfFaceName) - 1, // max len
                                      NULL, // default handling of unmappables
                                      NULL);// do not care if def char is used

    /* If WideCharToMultiByte succeeded then the number
     * of bytes it copied into the face name buffer will
     * be creater than zero and we just need to NULL terminate
     * the string. If there was an error then the number of
     * bytes copied is zero and we can not match the font.
     */
    if (destLen > 0) {

        DASSERT(destLen < sizeof(lf.lfFaceName));
        lf.lfFaceName[destLen] = '\0';
        lf.lfCharSet = DEFAULT_CHARSET;
        lf.lfPitchAndFamily = 0;

        foundFont = !EnumFontFamiliesExA((HDC) printDC, &lf,
                                        (FONTENUMPROCA) fontEnumProcA,
                                        (long) &matchedLogFont, 0);
    }


    if (foundFont) {

        /* Build a font of the requested size with no
         * width modifications. A negative font height
         * tells GDI that we want that values absolute
         * value as the font's point size. If the font
         * is successfully built then set it as the current
         * GDI font.
         */
        matchedLogFont.lfHeight = -ROUND_TO_LONG(fontSize);
        matchedLogFont.lfWidth = 0;
        matchedLogFont.lfEscapement = rotation;
        matchedLogFont.lfOrientation = rotation;
        matchedLogFont.lfUnderline = 0;
        matchedLogFont.lfStrikeOut = 0;

        /* Force bold or italic if requested. The font name
           such as Arial Bold may have already set a weight
           so here we just try to increase it.
        */
        if (isBold) {
            matchedLogFont.lfWeight = embolden(matchedLogFont.lfWeight);
        }

        if (isItalic) {
            matchedLogFont.lfItalic = 0xff;     // TRUE
        }

        HFONT font = CreateFontIndirectA(&matchedLogFont);
        if (font) {
            HFONT oldFont = (HFONT)::SelectObject(printDC, font);
            if (oldFont != NULL) {
                ::DeleteObject(oldFont);
            } else {
                foundFont = false;
            }
        } else {
            foundFont = false;
        }

    }

    return foundFont ? JNI_TRUE : JNI_FALSE;
}

/**
 * Try to convert a java font to a GDI font. On entry, 'printDC',
 * is the device context we want to draw into. 'fontName' is
 * the name of the font to be matched and 'fontSize' is the
 * size of the font in device coordinates. If there is an
 * equivalent GDI font then this function sets that font
 * into 'printDC' and returns a 'true'. If there is no equivalent
 * font then 'false' is returned.
 */
static jboolean jFontToWFontW(JNIEnv *env, HDC printDC, jstring fontName,
                        jfloat fontSize, jboolean isBold, jboolean isItalic,
                        jint rotation)
{
    LOGFONTW lf;
    LOGFONTW matchedLogFont;
    BOOL foundFont = false;     // Assume we didn't find a matching GDI font.

    memset(&matchedLogFont, 0, sizeof(matchedLogFont));

    /* Describe the GDI fonts we want enumerated. We
     * simply supply the java font name and let GDI
     * do the matching. If the java font name is
     * longer than the GDI maximum font lenght then
     * we can't convert the font.
     */
    WCHAR* name = TO_WSTRING(fontName);
    size_t nameLen = wcslen(name);

    if (nameLen < (sizeof(lf.lfFaceName) / sizeof(lf.lfFaceName[0]))) {

        wcscpy(lf.lfFaceName, name);

        lf.lfCharSet = DEFAULT_CHARSET;
        lf.lfPitchAndFamily = 0;

        foundFont = !EnumFontFamiliesExW((HDC) printDC, &lf,
                                        (FONTENUMPROCW) fontEnumProcW,
                                        (long) &matchedLogFont, 0);
    }

    if (foundFont) {

        /* Build a font of the requested size with no
         * width modifications. A negative font height
         * tells GDI that we want that values absolute
         * value as the font's point size. If the font
         * is successfully built then set it as the current
         * GDI font.
         */
        matchedLogFont.lfHeight = -ROUND_TO_LONG(fontSize);
        matchedLogFont.lfWidth = 0;
        matchedLogFont.lfEscapement = rotation;
        matchedLogFont.lfOrientation = rotation;
        matchedLogFont.lfUnderline = 0;
        matchedLogFont.lfStrikeOut = 0;

        /* Force bold or italic if requested. The font name
         * such as Arial Bold may have already set a weight
         * so here we just try to increase it.
         */
        if (isBold) {
            matchedLogFont.lfWeight = embolden(matchedLogFont.lfWeight);
        }

        if (isItalic) {
            matchedLogFont.lfItalic = 0xff;     // TRUE
        }

        //Debug: dumpLogFont(&matchedLogFont);

        HFONT font = CreateFontIndirectW(&matchedLogFont);
        if (font) {
            HFONT oldFont = (HFONT)::SelectObject(printDC, font);
            if (oldFont != NULL) {
                ::DeleteObject(oldFont);
            } else {
                foundFont = false;
            }
        } else {
            foundFont = false;
        }
    }

    return foundFont ? JNI_TRUE : JNI_FALSE;
}


/**
 * Invoked by GDI as a result of the EnumFontFamiliesExW
 * call this routine choses a GDI font that matches
 * a Java font. When a match is found then function
 * returns a zero result to terminate the EnumFontFamiliesExW
 * call. The information about the chosen font is copied into
 * the LOGFONTW structure pointed to by 'lParam'.
 */
static int CALLBACK fontEnumProcW(ENUMLOGFONTEXW *logfont,// logical-font data
                    NEWTEXTMETRICEX *lpntme,              // physical-font data
                    int FontType,                         // type of font
                    LPARAM lParam)
{
    LOGFONTW *matchedLogFont = (LOGFONTW *) lParam;
    int stop = 0;          // Take the first style found.

    if (matchedLogFont != NULL) {
        *matchedLogFont = logfont->elfLogFont;
    }

    return stop;
}

/**
 * Invoked by GDI as a result of the EnumFontFamiliesExA
 * call this routine choses a GDI font that matches
 * a Java font. When a match is found then function
 * returns a zero result to terminate the EnumFontFamiliesExA
 * call. The information about the chosen font is copied into
 * the LOGFONTA structure pointed to by 'lParam'.
 */
static int CALLBACK fontEnumProcA(ENUMLOGFONTEXA *logfont,// logical-font data
                    NEWTEXTMETRICEX *lpntme,              // physical-font data
                    int FontType,                         // type of font
                    LPARAM lParam)
{
    LOGFONTA *matchedLogFont = (LOGFONTA *) lParam;
    int stop = 0;          // Take the first style found.

    if (matchedLogFont != NULL) {
        *matchedLogFont = logfont->elfLogFont;
    }

    return stop;
}

/**
 * Given the weight of a font from a GDI LOGFONT
 * structure, return a new weight indicating a
 * bolder font.
 */
static int embolden(int currentWeight)
{

    /* If the font is less than bold then make
     * it bold. In real life this will mean making
     * a FW_NORMAL font bold.
     */
    if (currentWeight < FW_BOLD) {
        currentWeight = FW_BOLD;

    /* If the font is already bold or bolder
     * then just increase the weight. This will
     * not be visible with GDI in Win95 or NT4.
     */
    } else {
        currentWeight += EMBOLDEN_WEIGHT;
        if (currentWeight > MAX_FONT_WEIGHT) {
            currentWeight = MAX_FONT_WEIGHT;
        }
    }

    return currentWeight;
}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    setTextColor
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_setTextColor
(JNIEnv *env, jobject self, jint printDC, jint red, jint green, jint blue) {

    (void) ::SetTextColor( (HDC) printDC, RGB(red, green, blue));

}

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    textOut
 * Signature: (ILjava/lang/String;[FFFLjava/awt/Font;)V
 *
 * Generate GDI text calls for the unicode string
 * <code>text</code> into the device context
 * <code>printDC</code>. The text string is
 * positioned at <code>x</code>, <code>y</code>.
 * The positioning of each glyph in the string
 * is determined by windows.
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_textOut
  (JNIEnv *env, jobject self, jint printDC, jstring text, jfloat x, jfloat y,
   jobject font)
{

    long posX = ROUND_TO_LONG(x);
    long posY = ROUND_TO_LONG(y);

    // if font is supplied, its taken to be a logical font.
    if (font != NULL) {
        AwtFont::drawMFString((HDC)printDC, font, text, posX, posY);
        return;
    } 

    LPWSTR wText = TO_WSTRING(text);
    size_t strLen = wcslen(wText);


    BOOL drawn = ::ExtTextOutW( (HDC) printDC,
                    posX, posY,     // starting position for the text
                    0,              // supply x and y advances
                    NULL,           // optional clipping-opaquing rectangle
                    wText, strLen,  // the Unicode text to draw
                    NULL);          // intercharacter advances or NULL

}

/**
 * Scans a 24 bit RGB DIB image looking for the first non-white line.
 * On entry, 'image' points at the bottom of the DIB, which
 * is where the first scan line is. *'numLinesP', on entry, is
 * the number of scan lines in the image while 'width' is
 * the number of 24 bit pixels on each line. If a non-white
 * line is found in the DIB, then a pointer to the first,
 * working from the bottom, non-white scan line is returned.
 * and the number of remaining scan lines is returned in  *'numLinesP'.
 * Pixels are 3 byte BGR triples, so any byte that is not 0xff indicates
 * its a component of a non-white pixel. So we don't need to combine bytes
 * into pixels. Simply scan the image looking for any byte that is not 0xff
 */
static jbyte *findNonWhite(jbyte *image, long sy, long width, long height,
                          long scanLineStride, long *numLinesP) {

    long found = -1;
    long numLines = 0;
    jbyte *startLine = image;
    unsigned char *inLine;
    const unsigned char cc = (unsigned char)0xff;

    assert(image != NULL);
    assert(0 <= sy && sy < height);
    assert(0 < width);
    assert(0 < height);
    assert(numLinesP != NULL);

    for (numLines = 0; sy < height; numLines++, sy++) {

	inLine = (unsigned char*)startLine;

	for (long colcomp = 0; colcomp < scanLineStride; colcomp++) {
	    if (*inLine++ != cc) {
		found = sy;
		break;
	    }
	}

	if(found != -1) {
	    break;
	}

	startLine -= scanLineStride;
    }

    *numLinesP = numLines;

    return found == -1 ? NULL : startLine;
}

/* Find the 1st scanline that's entirely white.
 * The starting scanline pointed to by 'image' may be part way through the DIB.
 * If an all white scanline is found, the return value points to the beginning
 * of the last scanline with a non-white pixel. If no all white scanlines
 * are found, the starting scanline is returned.
 * '*numLinesP' returns the number of non-white scan lines.
 * Skip the 1st scanline as its always non-white. The DIB is bottom-up.
 */
static jbyte *findWhite(jbyte *image, long sy, long width, long height, 
                        long scanLineStride, long *numLinesP) {

    long numLines;
    jbyte *startLine = image;
    unsigned char *inLine;
    jbyte *found = NULL;
    long white;
    const unsigned char cc = (unsigned char)0xff;

    assert(image != NULL);
    assert(0 <= sy);
    assert(0 < width);
    assert(0 < height);
    assert(numLinesP != NULL);

    ++sy;
    for(numLines = 1; sy < height; numLines++, sy++) {

	startLine -= scanLineStride;
	inLine = (unsigned char*)startLine;
        white = 1;

	for (long colcomp = 0; colcomp < scanLineStride; colcomp++) {
	    if (*inLine++ != cc) {
                white = 0;
		break;
	    }
	}

	if (white != 0) {
	   found = startLine + scanLineStride;
	   break;
	}
    }

    *numLinesP = numLines;

    return found == NULL ? startLine : found;

}

#if 0
/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    drawImageIntRGB
 * Signature: (I[IFFFFFFFFII)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_drawImageIntRGB
  (JNIEnv *env, jobject self,
   jint printDC, jintArray image,
   jfloat destX, jfloat destY,
   jfloat destWidth, jfloat destHeight,
   jfloat srcX, jfloat srcY,
   jfloat srcWidth, jfloat srcHeight,
   jint srcBitMapWidth, jint srcBitMapHeight) {

    int result = 0;

    assert(printDC != NULL);
    assert(image != NULL);
    assert(srcX >= 0);
    assert(srcY >= 0);
    assert(srcWidth > 0);
    assert(srcHeight > 0);
    assert(srcBitMapWidth > 0);
    assert(srcBitMapHeight > 0);


    static int alphaMask =  0xff000000;
    static int redMask =    0x00ff0000;
    static int greenMask =  0x0000ff00;
    static int blueMask =   0x000000ff;

    struct {
        BITMAPV4HEADER header;
        DWORD          masks[256];
    } dib;



    memset(&dib,0,sizeof(dib));
    dib.header.bV4Size = sizeof(dib.header);
    dib.header.bV4Width = srcBitMapWidth;
    dib.header.bV4Height = -srcBitMapHeight;    // Top down DIB
    dib.header.bV4Planes = 1;
    dib.header.bV4BitCount = 32;
    dib.header.bV4V4Compression = BI_BITFIELDS;
    dib.header.bV4SizeImage = 0;        // It's the default size.
    dib.header.bV4XPelsPerMeter = 0;
    dib.header.bV4YPelsPerMeter = 0;
    dib.header.bV4ClrUsed = 0;
    dib.header.bV4ClrImportant = 0;
    dib.header.bV4RedMask = redMask;
    dib.header.bV4GreenMask = greenMask;
    dib.header.bV4BlueMask = blueMask;
    dib.header.bV4AlphaMask = alphaMask;
    dib.masks[0] = redMask;
    dib.masks[1] = greenMask;
    dib.masks[2] = blueMask;
    dib.masks[3] = alphaMask;

    jint *imageBits = NULL;

    try {
        imageBits = (jint *)env->GetPrimitiveArrayCritical(image, 0);

        if (printDC){
            result = ::StretchDIBits( (HDC) printDC,
                                             ROUND_TO_LONG(destX),
                                             ROUND_TO_LONG(destY),
                                             ROUND_TO_LONG(destWidth),
                                             ROUND_TO_LONG(destHeight),
                                             ROUND_TO_LONG(srcX),
                                             ROUND_TO_LONG(srcY),
                                             ROUND_TO_LONG(srcWidth),
                                             ROUND_TO_LONG(srcHeight),
                                             imageBits,
                                             (BITMAPINFO *)&dib,
                                             DIB_RGB_COLORS,
                                             SRCCOPY);

        }
    } catch (...) {
        if (imageBits != NULL) {
            env->ReleasePrimitiveArrayCritical(image, imageBits, 0);
        }
        throw;
    }

    env->ReleasePrimitiveArrayCritical(image, imageBits, 0);

}
#else

/*
 * Class:     sun_awt_windows_WPrinterJob
 * Method:    drawImage3ByteBGR
 * Signature: (I[BFFFFFFFF)V
 */
JNIEXPORT void JNICALL Java_sun_awt_windows_WPrinterJob_drawImage3ByteBGR
  (JNIEnv *env, jobject self,
   jint printDC, jbyteArray image,
   jfloat destX, jfloat destY,
   jfloat destWidth, jfloat destHeight,
   jfloat srcX, jfloat srcY,
   jfloat srcWidth, jfloat srcHeight) {

    int result = 0;

    assert(printDC != NULL);
    assert(image != NULL);
    assert(srcX >= 0);
    assert(srcY >= 0);
    assert(srcWidth > 0);
    assert(srcHeight > 0);

    BITMAPINFOHEADER header;

    memset(&header, 0, sizeof(header));
    header.biSize = sizeof(header);
    header.biWidth = ROUND_TO_LONG(srcWidth);
    header.biHeight = ROUND_TO_LONG(srcHeight); 
    header.biPlanes = 1;
    header.biBitCount = 24;
    header.biCompression = BI_RGB;
    header.biSizeImage = 0;        // It's the default size.
    header.biXPelsPerMeter = 0;
    header.biYPelsPerMeter = 0;
    header.biClrUsed = 0;
    header.biClrImportant = 0;
   

    jint *imageBits = NULL;

    try {
        imageBits = (jint *)env->GetPrimitiveArrayCritical(image, 0);

	/* get width in bytes */
	long imgWidthByteSz = ROUND_TO_LONG(srcWidth) * 3;
	
	int padBytes = 0;
	/* do we need to pad the image to make it DWORD aligned? */
	if ((imgWidthByteSz % sizeof(DWORD)) != 0)
	  padBytes = sizeof(DWORD) - (imgWidthByteSz % sizeof(DWORD));

	long newImgSize = (imgWidthByteSz+padBytes) * ROUND_TO_LONG(srcHeight);
	jbyte* alignedImage = (jbyte*) safe_Malloc(newImgSize);

	if (alignedImage != NULL) {
	  memset(alignedImage, 0xff, newImgSize);
	  
	  /* our image is for top-down, we need to copy it in reverse order */
	  jbyte* imgLinePtr = alignedImage;
	  for (long i=ROUND_TO_LONG(srcHeight)-1; i>=0; i--) {
	    memcpy(imgLinePtr, (jbyte*)imageBits+(i*imgWidthByteSz), 
		   imgWidthByteSz);
	    imgLinePtr += (imgWidthByteSz + padBytes);
	  }

	  if (printDC){
            result = ::StretchDIBits( (HDC) printDC,
				      ROUND_TO_LONG(destX),
				      ROUND_TO_LONG(destY),
				      ROUND_TO_LONG(destWidth),
				      ROUND_TO_LONG(destHeight),
				      ROUND_TO_LONG(srcX),
				      ROUND_TO_LONG(srcY),
				      ROUND_TO_LONG(srcWidth),
				      ROUND_TO_LONG(srcHeight),
				      alignedImage,
				      (BITMAPINFO *)&header,
				      DIB_RGB_COLORS,
				      SRCCOPY);

	    free(alignedImage);

	  }
	} /* if (alignedImage != NULL) */
    } catch (...) {
        if (imageBits != NULL) {
            env->ReleasePrimitiveArrayCritical(image, imageBits, 0);
        }
        throw;
    }

    env->ReleasePrimitiveArrayCritical(image, imageBits, 0);

}
#endif

static int bitsToDevice(HDC printDC, jbyte *image, long destX, long destY,
			long width, long height) {
    int result = 0;

    assert(printDC != NULL);
    assert(image != NULL);
    assert(destX >= 0);
    assert(destY >= 0);
    assert(width > 0);
    assert(height > 0);

    struct {
	BITMAPINFOHEADER bmiHeader; 
        DWORD*             bmiColors;
    } bitMapHeader;

    memset(&bitMapHeader,0,sizeof(bitMapHeader));
    bitMapHeader.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bitMapHeader.bmiHeader.biWidth = width; 
    bitMapHeader.bmiHeader.biHeight = height; // does -height work ever?
    bitMapHeader.bmiHeader.biPlanes = 1; 
    bitMapHeader.bmiHeader.biBitCount = 24;
    bitMapHeader.bmiHeader.biCompression = BI_RGB; 
    bitMapHeader.bmiHeader.biSizeImage = 0;	// It's the default size. 
    bitMapHeader.bmiHeader.biXPelsPerMeter = 0;
    bitMapHeader.bmiHeader.biYPelsPerMeter = 0;
    bitMapHeader.bmiHeader.biClrUsed = 0;
    bitMapHeader.bmiHeader.biClrImportant = 0;
    bitMapHeader.bmiColors = NULL;

    if (printDC){
	result = ::SetDIBitsToDevice(printDC,
				destX,	// left of dest rect
				destY,	// top of dest rect	
				width,	// width of dest rect
				height,	// height of dest rect
				0,	// left of source rect
				0,	// top of source rect
				0,	// line number of 1st source scan line
				height,	// number of source scan lines
				image,	// points to the DIB
				(BITMAPINFO *)&bitMapHeader,
				DIB_RGB_COLORS);
    }

    return result;
}

/**
 * Called by the Page Setup dialog this routine makes sure the
 * print dialog becomes the front most window.
 */
static UINT CALLBACK pageDlgHook(HWND hDlg, UINT msg,
				 UINT wParam, LONG lParam) {
    TRY;

    if (msg == WM_INITDIALOG) {
	SetForegroundWindow(hDlg);
    }

    return (UINT) FALSE;

    CATCH_BAD_ALLOC_RET(TRUE);
}


/**
 * Called by the Print dialog, this routine makes sure the
 * print dialog becoes the front most window.
 */
static UINT CALLBACK printDlgHook(HWND hDlg, UINT msg,
				  UINT wParam, LONG lParam)
{
    TRY;

    if (msg == WM_INITDIALOG) {
	SetForegroundWindow(hDlg);
    }

    return (UINT) FALSE;

    CATCH_BAD_ALLOC_RET(TRUE);
}


/**
 *	Create and return a printer device context for the
 *	default printer. If there is no default printer then
 *	return NULL. This fn is used when printing is invoked
 *      and no user dialog was created. So despite its name, it
 *      needs to return a DC which reflects all the applications
 *      settings which the driver might support.
 *      The number of copies is the most important setting.
 */
static HDC getDefaultPrinterDC(JNIEnv *env, jobject printerJob) {
    HDC printDC = NULL;

    int devWillDoCopies = FALSE;
    PRINTDLG pd;
    memset(&pd, 0, sizeof(PRINTDLG));
    pd.lStructSize = sizeof(PRINTDLG);
    pd.Flags = PD_RETURNDEFAULT | PD_RETURNDC;
    
    if (AwtPrintDialog::PrintDlg(&pd)) {
        printDC = pd.hDC;

        /* Find out how many copies the driver can do, and use driver's
         * dmCopies if requested number is within that limit
         */
        int maxCopies = 1;
        int nCopies = getCopies(env, printerJob);

        if (pd.hDevNames != NULL) {
            DEVNAMES *devnames = (DEVNAMES *)::GlobalLock(pd.hDevNames);

            if (devnames != NULL) {
                LPSTR lpdevnames = (LPSTR)devnames;
                LPSTR printer = (LPSTR)(lpdevnames+devnames->wDeviceOffset);
                LPSTR port = (LPSTR)(lpdevnames+devnames->wOutputOffset);
                // if DeviceCapabilities fails, return value is -1
                maxCopies = (int)::DeviceCapabilities(printer, port, DC_COPIES, 
                                                      NULL, NULL);
            }
            ::GlobalUnlock(pd.hDevNames);
        }

        if ((maxCopies >= nCopies) && (pd.hDevMode != NULL)) {
            DEVMODE *devmode = (DEVMODE *)::GlobalLock(pd.hDevMode);

            if (devmode != NULL) {

                if ((devmode->dmFields & DM_COPIES) && (nCopies > 1)) {
                    devmode->dmCopies = nCopies;
                    HDC tmpDC = ::ResetDC(pd.hDC, devmode);
                    if (tmpDC != NULL) {
                        printDC = tmpDC;
                        devWillDoCopies = TRUE;
                    } 
                }
            }
            ::GlobalUnlock(pd.hDevMode);
        }

        /* Not pretty that this is set in a separate place then the DC */
        if (pd.hDevMode != NULL) {
            setPrintHDMode(env, printerJob, pd.hDevMode);
        }
        if (pd.hDevNames != NULL) {
            setPrintHDName(env, printerJob, pd.hDevNames);
        }

        setBooleanField(env, printerJob, DRIVER_COPIES_STR, 
                        (devWillDoCopies ? JNI_TRUE : JNI_FALSE));
        setBooleanField(env, printerJob, DRIVER_COLLATE_STR, JNI_FALSE);
        setBooleanField(env, printerJob, USER_COLLATE_STR, JNI_FALSE);

    }

    return printDC;
}


/**
 * Move the description of the page's size and orientation
 * from the PageFormat object 'page' into the structure,
 * 'setup' used by Windows to display the Page Setup dialog.
 */
static void pageFormatToSetup(JNIEnv *env, jobject job,
                              jobject page, PAGESETUPDLG *setup, HDC hDC) {
    RectDouble paperSize;
    RectDouble margins;

    /* Move the orientation from PageFormat to Windows.
     */
    jboolean isPortrait = isPortraitOrientation(env, page);
    setOrientationInDevMode(setup->hDevMode, isPortrait);

    int units = (setup->Flags & PSD_INTHOUSANDTHSOFINCHES)
						? MM_HIENGLISH
						: MM_HIMETRIC;
    jobject paper = getPaper(env, page);
    getPaperValues(env, paper, &paperSize, &margins);
    // Setting the paper size appears to be a futile exercise, as its not one
    // of the values you can initialise - its an out-only arg. Margins are OK.
    // set it into the DEVMODE if there is one ..
    setup->ptPaperSize.x = convertFromPoints(paperSize.width, units);
    setup->ptPaperSize.y = convertFromPoints(paperSize.height, units);

    if (setup->hDevMode != NULL) {

        double paperWidth, paperHeight;
        WORD dmPaperSize = getPrintPaperSize(env, job);
        matchPaperSize(hDC, setup->hDevMode, setup->hDevNames,
                       paperSize.width,  paperSize.height,
                       &paperWidth, &paperHeight, &dmPaperSize);

        DEVMODE *devmode = (DEVMODE *)::GlobalLock(setup->hDevMode);
        if (devmode != NULL) {
            if (dmPaperSize != 0) {
                devmode->dmFields |= DM_PAPERSIZE;
                devmode->dmPaperSize = dmPaperSize;
            } else {
             devmode->dmFields |= DM_PAPERLENGTH | DM_PAPERWIDTH
              | DM_PAPERSIZE;
            devmode->dmPaperSize = DMPAPER_USER;
            devmode->dmPaperWidth = 
              (short)(convertFromPoints(paperSize.width, MM_LOMETRIC));
            devmode->dmPaperLength = 
              (short)(convertFromPoints(paperSize.height, MM_LOMETRIC));  
            }
        }
        ::GlobalUnlock(setup->hDevMode);
    }

    // When setting up these values, account for the orientation of the Paper
    // in the PageFormat. In the margins Rect when in portrait mode,
    // width is really right margin, height is really bottom margin.
    if (isPortrait) {
        setup->rtMargin.left = (short)convertFromPoints(margins.x, units);
        setup->rtMargin.top  = (short)convertFromPoints(margins.y, units);
        setup->rtMargin.right = (short)convertFromPoints(margins.width, units);
        setup->rtMargin.bottom = (short)convertFromPoints(margins.height, units);
    } else if (isLandscapeOrientation(env, page)) {
        setup->rtMargin.left = (short)convertFromPoints(margins.height, units);
        setup->rtMargin.top  = (short)convertFromPoints(margins.x, units);
        setup->rtMargin.right = (short)convertFromPoints(margins.y, units);
        setup->rtMargin.bottom = (short)convertFromPoints(margins.width, units);
    } else { // reverse landscape
        setup->rtMargin.left = (short)convertFromPoints(margins.y, units);
        setup->rtMargin.top  = (short)convertFromPoints(margins.width, units);
        setup->rtMargin.right = (short)convertFromPoints(margins.height, units);
        setup->rtMargin.bottom = (short)convertFromPoints(margins.x, units);
    }

    // Set page size here.
}


/**
 * Return an array of POINTS describing the paper sizes supported
 * by the driver identified by 'deviceName' and 'portName'.
 * If there is an error, then NULL is returned.
 */
static POINT *getPaperSizeList(const char *deviceName, const char *portName) {
    DWORD numPaperSizes;
    POINT *paperSizes = NULL;

    numPaperSizes = DeviceCapabilities(deviceName, portName,
				       DC_PAPERSIZE, NULL, NULL);
    
    if (numPaperSizes > 0) {
	paperSizes = (POINT *)safe_Malloc(sizeof(*paperSizes) * numPaperSizes);

	DWORD result = DeviceCapabilities(deviceName, portName,
					  DC_PAPERSIZE, (char *) paperSizes,
					  NULL);
	if (result == -1) {
	    free((char *) paperSizes);
	    paperSizes = NULL;
	}
    }

    return paperSizes;
}

/**
 * Get the orientation of the paper described by the printer
 * handle to a device mode structure 'hDevMode'.
 */
static WORD getOrientationFromDevMode(JNIEnv *env, jobject self) {

    WORD orient = DMORIENT_PORTRAIT;

    HGLOBAL hDevMode = getPrintHDMode(env, self);
    if (hDevMode != NULL) {
        LPDEVMODE devMode = (LPDEVMODE) GlobalLock(hDevMode);
        if ((devMode != NULL) && (devMode->dmFields & DM_ORIENTATION)) {
            orient = devMode->dmOrientation;
        }
        GlobalUnlock(hDevMode);
    }
    return orient;
}

/**
 * Set the orientation of the paper described by the printer
 * handle to a device mode structure 'hDevMode'.
 */
static void setOrientationInDevMode(HGLOBAL hDevMode, jboolean isPortrait) {

    if (hDevMode != NULL) {
	LPDEVMODE devMode = (LPDEVMODE) GlobalLock(hDevMode);
	if (devMode != NULL) {
	    devMode->dmOrientation = isPortrait
				    ? DMORIENT_PORTRAIT
				    : DMORIENT_LANDSCAPE;
	    devMode->dmFields |= DM_ORIENTATION;
	}
        GlobalUnlock(hDevMode);
    }
}

/**
 * Return the paper size and margins for the page
 * adjusted to take into account the portrait or
 * landscape orientation of the page. On entry,
 * 'setup' is a filled in structure as returned
 * by PageSetupDlg(). 'paperSize', 'margins',
 * and 'isPortrait' all point to caller allocated
 * space while will be filled in by this routine
 * with the size, in unknown Windows units, of
 * the paper, of the margins, and an indicator
 * whether the page is in portrait or landscape
 * orientation, respectively.
 */
static void retrievePaperInfo(const PAGESETUPDLG *setup, POINT *paperSize,
					RECT *margins, jboolean *isPortrait) {
    int orientationKnown = FALSE;

    *paperSize = setup->ptPaperSize;
    *isPortrait = TRUE;

    /* Usually the setup dialog will tell us the
     * orientation of the page, but it may not.
     */
    if (setup->hDevMode != NULL) {
	DEVMODE *devMode = (DEVMODE *)GlobalLock(setup->hDevMode);
	if(devMode->dmFields & DM_ORIENTATION){
	    orientationKnown = TRUE;
	    *isPortrait = devMode->dmOrientation == DMORIENT_PORTRAIT;
	}
	GlobalUnlock(setup->hDevMode);
    }

    /* The driver didn't tell us the paper orientation
     * so we declare it portrait if the paper
     * is longer than it is wide. Square paper is
     * declared to be portait.
     */
    if (orientationKnown == FALSE){
	*isPortrait = paperSize->x < paperSize->y;
    }

    /* The Paper class expresses the page size in
     * portait mode while Windows returns the paper
     * size adjusted for the orientation. If the
     * orientation is landscape then we want to
     * flip the width and height to get a portait
     * description of the page.
     */
    if (*isPortrait == FALSE){
	long hold = paperSize->x;
	paperSize->x = paperSize->y;
	paperSize->y = hold;

	margins->left = setup->rtMargin.top;
	margins->right = setup->rtMargin.bottom;
	margins->top = setup->rtMargin.right;
	margins->bottom = setup->rtMargin.left;

    } else {
	*margins = setup->rtMargin;
    }

}

/**
 * Set the range of pages to be printed. Both 'from', the first
 * page and 'to', the last page, are zero based indices.
 */
static void setRange(JNIEnv *env, jobject self, jint from, jint to) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jclass wPrintJob2DClass = env->GetObjectClass(self);
    jmethodID setRangeID = env->GetMethodID(wPrintJob2DClass, SETRANGE_STR,
						SETRANGE_SIG);
    env->CallVoidMethod(self, setRangeID, from, to);
}

/**
 * Return the number of pages in a Pageable document.
 */
static jint getNumPages(JNIEnv *env, jobject doc)
{
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jint numPages;
    jclass pageableClass = env->GetObjectClass(doc);
    jmethodID getNumPagesID = env->GetMethodID(pageableClass, GETNUMPAGES_STR,
					       GETNUMPAGES_SIG);
    numPages = env->CallIntMethod(doc, getNumPagesID);
    if (numPages < 0) numPages = MAX_UNKNOWN_PAGES;

    return numPages;
}

/**
 * Return the number of copies to be printed for a printerJob.
 */
static jint getCopies(JNIEnv *env, jobject printerJob)
{
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());


    jclass printerJobClass = env->GetObjectClass(printerJob);
    jmethodID getCopiesID = env->GetMethodID(printerJobClass, GETCOPIES_STR,
					     GETCOPIES_SIG);
    jint copies = env->CallIntMethod(printerJob, getCopiesID);

    return copies;
}

/**
 * Set the number of copies to be printed for a printerJob.
 */
static void setCopies(JNIEnv *env, jobject printerJob, int copies)
{
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());


    jclass printerJobClass = env->GetObjectClass(printerJob);
    jmethodID setCopiesID = env->GetMethodID(printerJobClass, SETCOPIES_STR,
                                             SETCOPIES_SIG);
    env->CallVoidMethod(printerJob, setCopiesID, copies);
}

/**
 * Return the job name to be specified. Caller should free the string.
 */
static LPTSTR getJobName(JNIEnv *env, jobject printerJob)
{
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());


    jclass printerJobClass = env->GetObjectClass(printerJob);
    jmethodID getJobNameID = env->GetMethodID(printerJobClass, GETJOBNAME_STR,
					     GETJOBNAME_SIG);
    jstring jobname = (jstring)env->CallObjectMethod(printerJob, getJobNameID);
    if (jobname != NULL) {
       char* tmp = (char *)JNU_GetStringPlatformChars(env, jobname, NULL);
       if (tmp) {
           char* jname = strdup(tmp);
           JNU_ReleaseStringPlatformChars(env, jobname, (const char *) tmp);
           return jname; 
       }
    }
    return strdup("Java Printing"); 
}

/**
 * Return a copy of the Paper object attached to the
 * PageFormat object 'page.'
 */
static jobject getPaper(JNIEnv *env, jobject page) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());


    jclass pageClass = env->GetObjectClass(page);
    jmethodID getPaperID = env->GetMethodID(pageClass, GETPAPER_STR,
							GETPAPER_SIG);
 
    return env->CallObjectMethod(page, getPaperID);
}

/**
 * Set the Paper object for a PageFormat instance.
 * 'paper' is the new Paper object that must be
 * set into 'page'.
 */
static void setPaper(JNIEnv *env, jobject page, jobject paper) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jclass pageClass = env->GetObjectClass(page);
    jmethodID setPaperID = env->GetMethodID(pageClass, SETPAPER_STR,
							SETPAPER_SIG);
    env->CallVoidMethod(page, setPaperID, paper);
}

/**
 * Return true if the PageFormat object 'page' describes a
 * page in the portrait orientation.
 */
static jboolean isPortraitOrientation(JNIEnv *env, jobject page) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jclass pageClass = env->GetObjectClass(page);
    jmethodID getOrientID = env->GetMethodID(pageClass, GETORIENT_STR,
							GETORIENT_SIG);
   
    jfieldID fieldID = env->GetStaticFieldID(pageClass, PORTRAIT_STR,
							PORTRAIT_SIG);

    jint portrait = env->GetStaticIntField(pageClass, fieldID);

    jint orientation = env->CallIntMethod(page, getOrientID);	
    
    /* We'll return true if the orientation is PORTRAIT. If
     * another PORTRAIT orientation is added, such as
     * PORTRAIT_LANDSCAPE, then this test will need to be
     * extended.
     */
    return orientation == portrait;

}

/**
 * Return true if the PageFormat object 'page' describes a
 * page in the landscape orientation.
 */
static jboolean isLandscapeOrientation(JNIEnv *env, jobject page) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jclass pageClass = env->GetObjectClass(page);
    jmethodID getOrientID = env->GetMethodID(pageClass, GETORIENT_STR,
                                                        GETORIENT_SIG);

    jfieldID fieldID = env->GetStaticFieldID(pageClass, LANDSCAPE_STR,
                                                        LANDSCAPE_SIG);

    jint landscape = env->GetStaticIntField(pageClass, fieldID);

    jint orientation = env->CallIntMethod(page, getOrientID);

    return orientation == landscape;

}

static void setOrientation(JNIEnv *env, jobject page, jboolean isPortrait) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jclass pageClass = env->GetObjectClass(page);
    jmethodID setOrientID = env->GetMethodID(pageClass, SETORIENT_STR,
							SETORIENT_SIG);

    /* We'll either set portrait or landscape, but we need to
     * get the orientation constants out of the PageFormat
     * class. We assume that the two constants have the same
     * signature - PORTRAIT_SIG.
     */
    const char *fieldName = isPortrait ? PORTRAIT_STR : LANDSCAPE_STR;
    jfieldID fieldID = env->GetStaticFieldID(pageClass, fieldName,
							PORTRAIT_SIG);
    jint orientation = env->GetStaticIntField(pageClass, fieldID);

    env->CallVoidMethod(page, setOrientID, orientation);	
}

/**
 * Pull the paper size and margins out of the paper object and
 * return them in points.
 */
static void getPaperValues(JNIEnv *env, jobject paper, RectDouble *paperSize,
			  RectDouble *margins, BOOL widthAsMargin) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jmethodID getID;
    
    paperSize->x = 0;
    paperSize->y = 0;

    jclass paperClass = env->GetObjectClass(paper);

    getID = env->GetMethodID(paperClass, GETWIDTH_STR, GETWIDTH_SIG);
    paperSize->width = env->CallDoubleMethod(paper, getID);

    getID = env->GetMethodID(paperClass, GETHEIGHT_STR, GETHEIGHT_SIG);
    paperSize->height = env->CallDoubleMethod(paper, getID);

    getID = env->GetMethodID(paperClass, GETIMG_X_STR, GETIMG_X_SIG);
    margins->x = env->CallDoubleMethod(paper, getID);
    if (margins-> x < 0 ) {
        margins-> x = 0;
    }

    getID = env->GetMethodID(paperClass, GETIMG_Y_STR, GETIMG_Y_SIG);
    margins->y = env->CallDoubleMethod(paper, getID);
    if (margins-> y < 0 ) {
        margins-> y = 0;
    }

    getID = env->GetMethodID(paperClass, GETIMG_W_STR, GETIMG_W_SIG);
    if (widthAsMargin) {
        margins->width = paperSize->width - margins->x 
                                      - env->CallDoubleMethod(paper, getID);
    } else {
        margins->width = env->CallDoubleMethod(paper, getID);
    }

    if (margins->width < 0) {
        margins->width = 0;
    }

    getID = env->GetMethodID(paperClass, GETIMG_H_STR, GETIMG_H_SIG);
    if (widthAsMargin) {
        margins->height = paperSize->height - margins->y
                                        - env->CallDoubleMethod(paper, getID);
    } else {
        margins->height = env->CallDoubleMethod(paper, getID);
    }

    if (margins->height < 0) {
        margins->height = 0;
    }

}

/**
 * Given a RECT specifying the margins
 * for the page and an indication of whether
 * the units are 1000ths of an inch (MM_HIENGLISH)
 * or 100ths of a millimeter (MM_HIMETRIC),
 * convert the margins to 72nds of an inch
 * and set them into the PageFormat insance provided.
 */
static void setPaperValues(JNIEnv *env, jobject paper, const POINT *paperSize,
					 const RECT *margins, int units) {
    // Because this function may call client Java code,
    // we can't run it on the toolkit thread.
    DASSERT(AwtToolkit::MainThread() != ::GetCurrentThreadId());

    jclass paperClass = env->GetObjectClass(paper);
    jmethodID setSizeID = env->GetMethodID(paperClass,
					SETSIZE_STR, SETSIZE_SIG);
    jmethodID setImageableID = env->GetMethodID(paperClass,
					SETIMAGEABLE_STR, SETIMAGEABLE_SIG);
								    
    /* Set the physical size of the paper.
     */
    jdouble paperWidth = convertToPoints(paperSize->x, units);
    jdouble paperHeight = convertToPoints(paperSize->y, units);
    env->CallVoidMethod(paper, setSizeID, paperWidth, paperHeight);

    /* Set the margins of the paper. In Windows' margin RECT,
     * the right and bottom parts of the structure are not
     * really the right and bottom of the imageable rectangle,
     * but rather the right and bottom margins.
     */
    jdouble x = convertToPoints(margins->left, units);
    jdouble y = convertToPoints(margins->top, units);
    long intWidth = paperSize->x - margins->left - margins->right;
    long intHeight = paperSize->y - margins->top - margins->bottom;
    jdouble width = convertToPoints(intWidth, units);
    jdouble height = convertToPoints(intHeight, units);
    env->CallVoidMethod(paper, setImageableID, x, y, width, height);

}

/**
 * Convert 'value' a measurement in 1/72's of an inch to
 * the units specified by 'units' - either MM_HIENGLISH
 * MM_HIMETRIC, or MM_LOMETRIC. The converted value is returned as
 * a long.
 */
static long convertFromPoints(double value, int units) {
    double conversion = 0;

    switch (units){
     case MM_HIENGLISH:
	conversion = POINTS_TO_HIENGLISH;
	break;

     case MM_HIMETRIC:
	conversion = POINTS_TO_HIMETRIC;
	break;

     case MM_LOMETRIC:
	conversion = POINTS_TO_LOMETRIC;
	break;

     default:
	assert(FALSE);	// Unsupported unit.
    }

    return (long) (value * conversion);
}

/**
 * Convert a measurement, 'value', from the units
 * specified by 'units', either MM_HIENGLISH or
 * MM_HIMETRIC to 1/72's of an inch and returned
 * as a double.
 */
static double convertToPoints(long value, int units) {
    double convertedValue = (double)value;

    switch (units){
    case MM_HIENGLISH:
	//convertedValue *= HIENGLISH_TO_POINTS;
        // this order of calculation is for bug 4191615
        convertedValue = (convertedValue*72.0) / 1000.0;
	break;

    case MM_HIMETRIC:
	convertedValue *= HIMETRIC_TO_POINTS;
	break;

    case MM_LOMETRIC:
	convertedValue *= LOMETRIC_TO_POINTS;
	break;

    default:
	assert(FALSE);	// Unsupported unit.
    }

    //Need to round off to the precision of the initial value. FIX.

    return convertedValue;
}

/**
 *	Ask the printer device context, 'printDC' about
 *	its capabilities and set these into the WPrintJob2D
 *	object 'self'.
 */
static void setCapabilities(JNIEnv *env, jobject self, HDC printDC) {

    // width of page in pixels
    jint pageWid = GetDeviceCaps(printDC, PHYSICALWIDTH);
    setIntField(env, self, PAGEW_STR, pageWid);

    // height of page in pixels
    jint pageHgt = GetDeviceCaps(printDC, PHYSICALHEIGHT);
    setIntField(env, self, PAGEH_STR, pageHgt);

    // x scaling factor of printer
    jint xsf = GetDeviceCaps(printDC, SCALINGFACTORX);

    // x scaling factor of printer
    jint ysf = GetDeviceCaps(printDC, SCALINGFACTORY);

    if (getOrientationFromDevMode(env, self) == DMORIENT_LANDSCAPE) {
        // pixels per inch in x direction
        jint xRes = GetDeviceCaps(printDC, LOGPIXELSY);
        setIntField(env, self, XRES_STR, xRes);

        // pixels per inch in y direction
        jint yRes = GetDeviceCaps(printDC, LOGPIXELSX);
        setIntField(env, self, YRES_STR, yRes);

        // x coord of printable area in pixels
        jint xOrg = GetDeviceCaps(printDC, PHYSICALOFFSETY);
        setIntField(env, self, PHYSX_STR, xOrg);

        // y coord of printable area in pixels
        jint yOrg = GetDeviceCaps(printDC, PHYSICALOFFSETX);
        setIntField(env, self, PHYSY_STR, yOrg);

        // width of printable area in pixels
        jint printWid = GetDeviceCaps(printDC, VERTRES);
        setIntField(env, self, PHYSW_STR, printWid);

        // height of printable area in pixels
        jint printHgt = GetDeviceCaps(printDC, HORZRES);
        setIntField(env, self, PHYSH_STR, printHgt);

    } else {
        // pixels per inch in x direction
        jint xRes = GetDeviceCaps(printDC, LOGPIXELSX);
        setIntField(env, self, XRES_STR, xRes);

        // pixels per inch in y direction
        jint yRes = GetDeviceCaps(printDC, LOGPIXELSY);
        setIntField(env, self, YRES_STR, yRes);

        // x coord of printable area in pixels
        jint xOrg = GetDeviceCaps(printDC, PHYSICALOFFSETX);
        setIntField(env, self, PHYSX_STR, xOrg);

        // y coord of printable area in pixels
        jint yOrg = GetDeviceCaps(printDC, PHYSICALOFFSETY);
        setIntField(env, self, PHYSY_STR, yOrg);

        // width of printable area in pixels
        jint printWid = GetDeviceCaps(printDC, HORZRES);
        setIntField(env, self, PHYSW_STR, printWid);

        // height of printable area in pixels
        jint printHgt = GetDeviceCaps(printDC, VERTRES);
        setIntField(env, self, PHYSH_STR, printHgt);
    }
}



static inline HDC getPrintDC(JNIEnv *env, jobject self) {
    return (HDC)getIntField(env, self, PRINTDC_STR);	
}

static inline void setPrintDC(JNIEnv *env, jobject self, HDC printDC) {
    setIntField(env, self, PRINTDC_STR, (jint)printDC);
}

static inline HGLOBAL getPrintHDMode(JNIEnv *env, jobject self) {
    return (HGLOBAL)getIntField(env, self, PRINTHDMODE_STR);
}

static inline void setPrintHDMode(JNIEnv *env, jobject self, HGLOBAL hGlobal) {
    setIntField(env, self, PRINTHDMODE_STR, (jint)hGlobal);
}

static inline HGLOBAL getPrintHDName(JNIEnv *env, jobject self) {
    return (HGLOBAL)getIntField(env, self, PRINTDNAME_STR);
}

static inline void setPrintHDName(JNIEnv *env, jobject self, HGLOBAL hGlobal) {
    setIntField(env, self, PRINTDNAME_STR, (jint)hGlobal);
}

static inline WORD getPrintPaperSize(JNIEnv *env, jobject self) {
    return (WORD)getIntField(env, self, PRINTPAPERSIZE_STR);
}

static inline void setPrintPaperSize(JNIEnv *env, jobject self, WORD sz) {
    setIntField(env, self, PRINTPAPERSIZE_STR, (jint)sz);
}

/**
 *	Return the java int value of the field 'fieldName' in the
 *	java instance 'self'.
 */
static jint getIntField(JNIEnv *env, jobject self, const char *fieldName) {
    jfieldID fieldId = getIdOfIntField(env, self, fieldName);
    return env->GetIntField(self, fieldId);	
}

/**
 *	Set the int field named 'fieldName' of the java instance
 *	'self' to the value 'value'.
 */
static void setIntField(JNIEnv *env, jobject self, const char *fieldName,
								jint value) {
    jfieldID fieldId = getIdOfIntField(env, self, fieldName);
    env->SetIntField(self, fieldId, value);
}

/**
 *	Return the field id of the java instance 'self' of the
 *	java int field named 'fieldName'.
 */
static jfieldID getIdOfIntField(JNIEnv *env, jobject self,
						const char *fieldName) {
    jclass myClass = env->GetObjectClass(self);
    jfieldID fieldId = env->GetFieldID(myClass, fieldName, kJavaIntStr);
    assert(fieldId != 0);

    return fieldId;

}

static void setBooleanField(JNIEnv *env, jobject self, const char *fieldName,
                                                              jboolean value) {
    jclass myClass = env->GetObjectClass(self);
    jfieldID fieldId = env->GetFieldID(myClass, fieldName, "Z");
    DASSERT(fieldId != 0);
    env->SetIntField(self, fieldId, value);
}

/**
 *  Throw a PrinterException with a string describing
 *  the Window's system error 'err'.
 */
static void throwPrinterException(JNIEnv *env, DWORD err) {
    char errStr[256];
    jclass printerException = env->FindClass(PRINTEREXCEPTION_STR);

    errStr[0] = '\0';
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		  NULL,
		  err,
		  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		  (LPTSTR) errStr,
		  sizeof(errStr),
		  NULL );


    env->ThrowNew(printerException, errStr);
}


/* 
 * Finds the closest matching paper size for the printer.
 * Parameters are in 72ndths of an inch.
 * paperSize is the win32 integer identifier for a paper size.
 * Requires an initialised set of printer device structures.
 * Updates the printDC to specify the matched paper size.
 * If the passed in paper size is non-zero, its taken to be a windows
 * paper size "name", and we check that paper size against the paper
 * we are matching and prefer that name over other names which also match
 * the size.
 */
static void matchPaperSize(HDC printDC, HGLOBAL hDevMode, HGLOBAL hDevNames,
                           double origWid, double origHgt, 
                           double* newWid, double *newHgt,
                           WORD* paperSize) {

    const double epsilon = 0.50;
    const double tolerance = (1.0 * 72.0);  // # inches * 72

    *newWid = origWid;
    *newHgt = origHgt;

   /* 1st check if the DC/Devmode has as its current papersize a paper
    * which matches the paper specified. If yes, then we can skip hunting
    * for the match and in the process we avoid finding a "name" for
    * the paper size which isn't the one the user specified in the page
    * setup dialog. For example "11x17" is also "Ledger".
    */
    if (printDC != NULL) {
    // pixels per inch in x and y direction
    jint xPixelRes = GetDeviceCaps(printDC, LOGPIXELSX);
    jint yPixelRes = GetDeviceCaps(printDC, LOGPIXELSY);

    // width and height of page in pixels
    jint pagePixelWid = GetDeviceCaps(printDC, PHYSICALWIDTH);
    jint pagePixelHgt = GetDeviceCaps(printDC, PHYSICALHEIGHT);

    // page size in 1/72"
    jdouble paperWidth = (jdouble)((pagePixelWid * 72)/(jdouble)xPixelRes);
    jdouble paperHeight = (jdouble)((pagePixelHgt * 72)/(jdouble)yPixelRes);

    if ((fabs(origWid - paperWidth) < epsilon) &&
       (fabs(origHgt - paperHeight) < epsilon) &&
          (*paperSize == 0)) {

        *newWid = origWid;
        *newHgt = origHgt;

        if (hDevMode != NULL) {
            DEVMODE *devmode = (DEVMODE *)::GlobalLock(hDevMode);
            if (devmode != NULL && (devmode->dmFields & DM_PAPERSIZE)) {
             *paperSize = devmode->dmPaperSize;
                ::ResetDC(printDC, devmode);
            }
            ::GlobalUnlock(hDevMode);
        }
        return;
    }
    }

    /* begin trying to match papers */

    LPSTR printer = NULL, port = NULL;
    if (hDevNames != NULL) {
        DEVNAMES *devnames = (DEVNAMES *)::GlobalLock(hDevNames);
        if (devnames != NULL) {
            LPSTR lpdevnames = (LPSTR)devnames;
            printer = strdup((LPSTR)(lpdevnames+devnames->wDeviceOffset));
            port = strdup((LPSTR)(lpdevnames+devnames->wOutputOffset));
        }
        ::GlobalUnlock(hDevNames);
    }

    int numPaperSizes = 0;
    WORD *papers = NULL;
    POINT *paperSizes = NULL;

    numPaperSizes = (int)DeviceCapabilities(printer, port, DC_PAPERSIZE,
                                            NULL, NULL);
    if (numPaperSizes > 0) {
        papers = (WORD*)safe_Malloc(sizeof(WORD) * numPaperSizes);
        paperSizes = (POINT *)safe_Malloc(sizeof(*paperSizes) * numPaperSizes);

        DWORD result1 = DeviceCapabilities(printer, port,
                                          DC_PAPERS, (char *) papers, NULL);
        DWORD result2 = DeviceCapabilities(printer, port,
                                          DC_PAPERSIZE, (char *) paperSizes,
                                          NULL);

        if (result1 == -1 || result2 == -1 ) {
            free((char *) papers);
            papers = NULL;
            free((char *) paperSizes);
            paperSizes = NULL;
        }
    }

    double closestWid = 0.0;
    double closestHgt = 0.0;
    WORD   closestMatch = 0;

    if (paperSizes != NULL) {

        /* Paper sizes are in 0.1mm units. Convert to 1/72"
         * For each paper size, compute the difference from the paper size
         * passed in. Use a least-squares difference, so paper much different
         * in x or y should score poorly
         */
        double diffw = origWid;
        double diffh = origHgt;
        double least_square = diffw * diffw + diffh * diffh;
        double tmp_ls;
        double widpts, hgtpts;

        for (int i=0;i<numPaperSizes;i++) {
            widpts = paperSizes[i].x * LOMETRIC_TO_POINTS;
            hgtpts = paperSizes[i].y * LOMETRIC_TO_POINTS;

            if ((fabs(origWid - widpts) < epsilon) &&
              (fabs(origHgt - hgtpts) < epsilon)) {

		if ((*paperSize == 0) || ((*paperSize !=0) && 
                                        (papers[i]==*paperSize))) {

                closestWid = origWid;
                closestHgt = origHgt;
                closestMatch = papers[i];
                break;
		}
	    }

	    diffw = fabs(widpts - origWid);
            diffh = fabs(hgtpts - origHgt);
            tmp_ls = diffw * diffw + diffh * diffh;
            if ((diffw < tolerance) && (diffh < tolerance) &&
                (tmp_ls < least_square)) {
              least_square = tmp_ls;
	      closestWid = widpts;
	      closestHgt = hgtpts;
	      closestMatch = papers[i];
            }
        }
    }
    
    *newWid = closestWid;
    *newHgt = closestHgt;
    *paperSize = closestMatch;

    /* At this point we have the paper which is the closest match
     * We now need to select the paper into the DEVMODE, and
     * get a DC which matches so we can get the margins.
     */
    if ((printDC != NULL) && (hDevMode != NULL) && (closestMatch != 0)) {
        DEVMODE *devmode = (DEVMODE *)::GlobalLock(hDevMode);
        if (devmode != NULL) {
            devmode->dmFields |= DM_PAPERSIZE;
            devmode->dmPaperSize = closestMatch;
            ::ResetDC(printDC, devmode);
        }
        ::GlobalUnlock(hDevMode);
    }

    if (printer != NULL) {
        free((char *)printer);
    }
    if (port != NULL) {
        free((char *)port);
    }
    if (papers != NULL) {
        free((char *)papers);
    }
    if (paperSizes != NULL) {
        free((char *)paperSizes);
    }

}

} /* extern "C" */
