/*
 * @(#)awt_TextComponent.cpp	1.37 00/04/06
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_Toolkit.h"
#include "awt_TextComponent.h"

#include "awt_Canvas.h"

#include "jni.h"
#include "awt_Font.h"


/************************************************************************
 * AwtTextComponent fields
 */

/* java.awt.TextComponent fields */
jfieldID AwtTextComponent::canAccessClipboardID;


/************************************************************************
 * AwtTextComponent methods
 */

AwtTextComponent::AwtTextComponent() {
    m_synthetic = FALSE;
//    javaEventsMask = 0;    // accessibility support 
}

const char* AwtTextComponent::GetClassName() {
    return "EDIT";  /* System provided edit control class */
}

/* Set a suitable font to IME against the component font. */
void AwtTextComponent::SetFont(AwtFont* font)
{
    DASSERT(font != NULL);
    if (font->GetAscent() < 0) {
	AwtFont::SetupAscent(font);
    }

    int index = font->GetInputHFontIndex();
    if (index < 0)
	/* In this case, user cannot get any suitable font for input. */
        index = 0;	

    //im --- changed for over the spot composing
    m_hFont = font->GetHFont(index);
    SendMessage(WM_SETFONT, (WPARAM)m_hFont, MAKELPARAM(FALSE, 0));
    SendMessage(EM_SETMARGINS, EC_LEFTMARGIN | EC_RIGHTMARGIN,
                MAKELPARAM(1, 1)); 
    VERIFY(::InvalidateRect(GetHWnd(), NULL, TRUE));
    //im --- end

}

/*
 * Returns a copy of pStr where \n have been converted to \r\n.
 * If the result != 'pStr', 'pStr' has been freed.
 */
WCHAR *AwtTextComponent::AddCR(WCHAR *pStr, int nStrLen)
{
    int nNewlines = 0;
    int i, nResultIx, nLen;
    WCHAR *result;

    /* check to see if there are any LF's */
    if (wcschr(pStr, L'\n') == NULL) {
        return pStr;
    }

    /* count newlines */
    for (i=0; pStr[i] != 0; i++) {
        if (pStr[i] == L'\n') {
            nNewlines++;
        }
    }
    nLen = i;

    if (nLen + nNewlines + 1 > nStrLen) {
        result = new WCHAR[nLen + nNewlines + 1];
    } else {
        result = pStr;
    }
    nResultIx = nLen + nNewlines;
    for (i=nLen; i>=0; i--) {
        result[nResultIx--] = pStr[i];
        if (pStr[i] == L'\n') {
            result[nResultIx--] = L'\r';
        }
    }
    DASSERT(nResultIx == -1);

    if (pStr != result) {
        /* We realloc'd, so delete old buffer. */
        delete[] pStr;
    }
    return result;
}

char *AwtTextComponent::AddCR(char *pStr, int nStrLen)
{
    int nNewlines = 0;
    int i, nResultIx, nLen;
    char *result;

    /* check to see if there are any LF's */
    if (strchr(pStr, '\n') == NULL) {
        return pStr;
    }

    /* count newlines */
    for (i=0; pStr[i] != 0; i++) {
        if (pStr[i] == '\n') {
            nNewlines++;
        }
    }
    nLen = i;

    if (nLen + nNewlines + 1 > nStrLen) {
        result = new char[nLen + nNewlines + 1];
    } else {
        result = pStr;
    }
    nResultIx = nLen + nNewlines;
    for (i=nLen; i>=0; i--) {
        result[nResultIx--] = pStr[i];
        if (pStr[i] == '\n') {
            result[nResultIx--] = '\r';
        }
    }
    DASSERT(nResultIx == -1);

    if (pStr != result) {
        /* We realloc'd, so delete old buffer. */
        delete[] pStr;
    }
    return result;
}

/* Converts \r\n pairs to \n. */
int AwtTextComponent::RemoveCR(WCHAR *pStr)
{
    int i, nLen = 0;

    if (pStr) {
        /* check to see if there are any CR's */
        if (wcschr(pStr, L'\r') == NULL) {
            return wcslen(pStr);
        }

        for (i=0; pStr[i] != 0; i++) {
            if (pStr[i] == L'\r' && pStr[i + 1] == L'\n') {
                i++;
            }
            pStr[nLen++] = pStr[i];
        }
        pStr[nLen] = 0;
    }
    return nLen;
}

/* Have to override this method from AwtComponent to fix
 * bug 4118621. 
 */
void AwtTextComponent::Reshape(int x, int y, int w, int h)
{
    long start, end;
    AwtComponent::Reshape(x, y, w, h);
    SendMessage(EM_GETSEL, (WPARAM)&start, (LPARAM)&end);
    SendMessage(EM_SETSEL, 0, 0);
    SendMessage(EM_SCROLLCARET);
    SendMessage(EM_SETSEL, start, end);
    SendMessage(EM_SCROLLCARET);
}

/*
 * This routine is safe for converting multibyte strings, because
 * control characters such as \n and \r are never part of a multibyte
 * character sequence.
 */
int AwtTextComponent::RemoveCR(char *pStr)
{
    int i, nLen = 0;

    if (pStr) {
        /* check to see if there are any CR's */
        if (strchr(pStr, '\r') == NULL) {
            return strlen(pStr);
        }

        for (i=0; pStr[i] != 0; i++) {
            if (pStr[i] == '\r' && pStr[i + 1] == '\n') {
                i++;
            }
            pStr[nLen++] = pStr[i];
        }
        pStr[nLen] = 0;
    }
    return nLen;
}

MsgRouting
AwtTextComponent::WmNotify(UINT notifyCode)
{
    if (notifyCode == EN_CHANGE) {
      DoCallback("valueChanged", "()V");
    } 
    return mrDoDefault;
}

/*
 * Store the 'synthetic' parameter so that the WM_PASTE security check
 * happens only for synthetic events.
 */
MsgRouting
AwtTextComponent::HandleEvent(MSG *msg, BOOL synthetic)
{
    MsgRouting returnVal;

    m_synthetic = synthetic;
    returnVal = AwtComponent::HandleEvent(msg, synthetic);
    m_synthetic = FALSE;
    return returnVal;
}

/*
 * If this Paste is occuring because of a synthetic Java event (e.g.,
 * a synthesized <CTRL>-V KeyEvent), then verify that the TextComponent
 * has permission to access the Clipboard before pasting. If permission
 * is denied, we should throw a SecurityException, but currently do not
 * because when we detect the security violation, we are in the Toolkit
 * thread, not the thread which dispatched the illegal event.
 */
MsgRouting
AwtTextComponent::WmPaste()
{
    if (m_synthetic) {
        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
	if (env->EnsureLocalCapacity(1) < 0) {
	    return mrConsume;
	}
	jobject target = GetTarget(env);
	jboolean canAccessClipboard = 
	    env->GetBooleanField(target,
				 AwtTextComponent::canAccessClipboardID);
	env->DeleteLocalRef(target);
	return (canAccessClipboard) ? mrDoDefault : mrConsume;
    }
    else {
        return mrDoDefault;
    }
}

//im --- override to over the spot composition
void AwtTextComponent::SetCompositionWindow()
{
    HIMC hIMC = ImmGetContext(m_hwnd);
    COMPOSITIONFORM cf = { CFS_POINT, {0,0}, {0,0,0,0} };
    GetCaretPos(&(cf.ptCurrentPos));
    ImmSetCompositionWindow(hIMC, &cf);

    LOGFONT lf;
    GetObject(m_hFont, sizeof(LOGFONT), &lf);
    ImmSetCompositionFont(hIMC, &lf);
}
//im --- end

static long getJavaSelPos(AwtTextComponent *c, long orgPos)
{
    long mblen;
    long pos = 0;
    long cur = 0;
    char *mbbuf;

    if ((mblen = c->GetTextLength()) == 0)
        return 0;
    mbbuf = new char[mblen + 1];
    c->GetText(mbbuf, mblen + 1);

    while (cur < orgPos && pos++ < mblen) {
        if (IsDBCSLeadByte(mbbuf[cur])) {
	    cur++;
	} else if (mbbuf[cur] == '\r' && mbbuf[cur + 1] == '\n') {
	    cur++;
	}
	cur++;
    }
    delete[] mbbuf;
    return pos;
}

static long getWin32SelPos(AwtTextComponent *c, long orgPos)
{
    long mblen;
    long pos = 0;
    long cur = 0;
    char *mbbuf;

    if ((mblen = c->GetTextLength()) == 0)
       return 0;
    mbbuf = new char[mblen + 1];
    c->GetText(mbbuf, mblen + 1);

    while (cur < orgPos && pos < mblen) {
        if (IsDBCSLeadByte(mbbuf[pos])) {
	    pos++;
	} else if (mbbuf[pos] == '\r' && mbbuf[pos + 1] == '\n') {
	    pos++;
	}
	pos++;
	cur++;
    }
    delete[] mbbuf;
    return pos;
}

/************************************************************************
 * WTextComponentPeer native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    getText
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL
Java_sun_awt_windows_WTextComponentPeer_getText(JNIEnv *env, jobject self)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN_NULL(self);

    AwtTextComponent* c = (AwtTextComponent*)pData;

    BOOL isNT = IS_NT;
    int len;
    /* before wrapper function of GetWindowTextLengthW is made, 
     * we have to distinguish the running environment
     */
    if (isNT) {
	len = ::GetWindowTextLengthW(c->GetHWnd());
    } else {
        len = ::GetWindowTextLengthA(c->GetHWnd());
    }
    
    jstring js;
    if (len == 0) {
        /* Make java null string */
        jchar *jc = new jchar[0];
	js = env->NewString(jc, 0);
	delete [] jc;
    } else {
	/*
	 * before wrapper function of GetWindowText is made so that 2
	 * types of AwtTextComponent::GetText are unified, we have to
	 * distinguish the running environment 
	 */
        if (isNT) {
	    WCHAR* buf = new WCHAR[len + 1];
            c->GetText(buf, len + 1);
            AwtTextComponent::RemoveCR(buf);
            js = env->NewString(buf, wcslen(buf));
            delete [] buf;
        } else {
#ifdef WIN32
            DASSERT(IS_WIN95);
#endif
	    /*
	     * Text in a TextComponent is written in platform-specific
	     * codeset because it may be output from IME, which
	     * doesn't emit unicode.  
	     */
            char* buffer = new char[len + 1];	/* Including null terminater */
            c->GetText(buffer, len + 1);
            AwtTextComponent::RemoveCR(buffer);
	    js = AwtFont::multibyte2javaString(env, buffer);/* convert to unicode. */
            delete[] buffer;
        }
    }
    return js;

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    setText
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WTextComponentPeer_setText(JNIEnv *env, jobject self,
						jstring text)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    AwtTextComponent* c = (AwtTextComponent*)pData;
    /*
     * before wrapper function of SetWindowText is made, two versions
     * of SetText will be unified.  Before that we have to distinguish
     * the running environment.
     */
    if (IS_NT) {
    	int length = env->GetStringLength(text);
        WCHAR* buffer = new WCHAR[length + 1];
	env->GetStringRegion(text, 0, length, buffer);
	buffer[length] = 0;
        buffer = AwtTextComponent::AddCR(buffer, length + 1);
        c->SetText(buffer);
        delete[] buffer;
    } else {
#ifdef WIN32
        DASSERT(IS_WIN95);
#endif
	int length = env->GetStringLength(text);
        char* buffer = new char[length * 2 + 1];
	AwtFont::javaString2multibyte(env, text, buffer, length);
        buffer = AwtTextComponent::AddCR(buffer, length);
        c->SetText(buffer);

        delete[] buffer;
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    getSelectionStart
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WTextComponentPeer_getSelectionStart(JNIEnv *env,
							  jobject self)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN_NULL(self);
    AwtComponent* c = (AwtComponent*)pData;
    long start;
    c->SendMessage(EM_GETSEL, (WPARAM)&start);
    return getJavaSelPos((AwtTextComponent*)c, start);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    getSelectionEnd
 * Signature: ()I
 */
JNIEXPORT jint JNICALL 
Java_sun_awt_windows_WTextComponentPeer_getSelectionEnd(JNIEnv *env,
							jobject self)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN_NULL(self);
    AwtComponent* c = (AwtComponent*)pData;
    long end;
    c->SendMessage(EM_GETSEL, 0, (LPARAM)&end);
    return getJavaSelPos((AwtTextComponent*)c, end);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    select
 * Signature: (II)V
 */
JNIEXPORT void JNICALL 
Java_sun_awt_windows_WTextComponentPeer_select(JNIEnv *env, jobject self,
					       jint start, jint end)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    AwtComponent* c = (AwtComponent*)pData;

    c->SendMessage(EM_SETSEL, 
		   getWin32SelPos((AwtTextComponent*)c, start),
		   getWin32SelPos((AwtTextComponent*)c, end));
    c->SendMessage(EM_SCROLLCARET);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    enableEditing
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WTextComponentPeer_enableEditing(JNIEnv *env, 
						      jobject self,
						      jboolean on)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    AwtTextComponent* c = (AwtTextComponent*)pData;
    c->SendMessage(EM_SETREADONLY, !on);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WTextComponentPeer_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    cls = env->FindClass("java/awt/TextComponent");
    if (cls != NULL) {
        AwtTextComponent::canAccessClipboardID = 
	    env->GetFieldID(cls, "canAccessClipboard", "Z");
	DASSERT(AwtTextComponent::canAccessClipboardID != NULL);
    }

    CATCH_BAD_ALLOC;
}

//
// Accessibility support
//

/* To be fully implemented in a future release
 * 
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    getIndexAtPoint
 * Signature: (II)I
 *
JNIEXPORT jlong JNICALL
Java_sun_awt_windows_WTextComponentPeer_filterEvents(JNIEnv *env, jobject self, jlong mask)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN_NULL(self);
    AwtTextComponent* c = (AwtTextComponent*)pData;

    jlong oldMask = c->javaEventsMask;
    c->javaEventsMask = mask;

    return oldMask;

    CATCH_BAD_ALLOC_RET(0);
}
*/

// [[[FIXME]]] need to switch to rich edit field; look for EN_SELCHANGE event instead
/*
 * Handle WmKeyDown to catch keystrokes which may move the caret,
 * and fire events as appropriate when that happens, if they are wanted
 *
 * Note: mouse clicks come through WmKeyDown as well (do they??!?!)
 *
MsgRouting AwtTextComponent::WmKeyDown(UINT wkey, UINT repCnt, 
                                   UINT flags, BOOL system) {

    printf("AwtTextComponent::WmKeyDown called\r\n");

    
    // NOTE: WmKeyDown won't be processed 'till well after we return
    //       so we need to modify the values based on the keystroke
    //
    static long oldStart = -1;
    static long oldEnd = -1;

    // most keystrokes can move the caret
    // so we'll simply check to see if the caret has moved!
    if (javaEventsMask & (jlong) java_awt_TextComponent_textSelectionMask) {
        long start;
        long end;
        SendMessage(EM_GETSEL, (WPARAM)&start, (LPARAM)&end);
        if (start != oldStart || end != oldEnd) {  
 
            printf("  -> calling TextComponent.selectionValuesChanged()\r\n");
            printf("  -> old = (%d, %d); new = (%d, %d)\r\n", 
                    oldStart, oldEnd, start, end);

            DoCallback("selectionValuesChanged", "(II)V", start, end); // let Java-side track details...
            oldStart = start;
            oldEnd = end;
        }
    }
    
    return AwtComponent::WmKeyDown(wkey, repCnt, flags, system);
}
*/

/* To be fully implemented in a future release
 *
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    getIndexAtPoint
 * Signature: (II)I
 *
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WTextComponentPeer_getIndexAtPoint(JNIEnv *env, jobject self, jint x, jint y)
{
    TRY;

    int pData;
//    JNI_CHECK_PEER_RETURN_VAL(self, -1);   [[[FIXME]]] Peter Korn -> should return -1 here
    JNI_CHECK_PEER_RETURN_NULL(self);
    AwtTextComponent* c = (AwtTextComponent*)pData;
    int indicies = c->SendMessage(EM_CHARFROMPOS, (WPARAM) 0, (LPARAM) MAKELPARAM(x, y));
    int index = LOWORD(indicies);   // index into the line the (x,y) coord is on
    int lineIndex = c->SendMessage(EM_LINEINDEX, HIWORD(indicies));  // index of start of line
    return (index + lineIndex);

    CATCH_BAD_ALLOC_RET(-1);
}
*/

/* To be fully implemented in a future release
 *
 * Class:     sun_awt_windows_WTextComponentPeer
 * Method:    getCharacterBounds
 * Signature: (I)Ljava/awt/Rectangle;
 *
JNIEXPORT jobject JNICALL
Java_sun_awt_windows_WTextComponentPeer_getCharacterBounds(JNIEnv *env, jobject self, jint i)
{

    //  loop through lines with EM_LINELENGTH?  e.g.:
    //     line = 0; ttl = 0;   // index is passed in as 'i' above
    //     while (ttl < index) {
    //        ttl += SendMessage(EM_LINELENGTH, line++);
    //     } 
    //     line-- (decrement back again)
    //  alternately, we could use EM_LINEINDEX to the same effect; perhaps slightly cleaner:
    //     computedIndex = 0; line = 0;
    //     while (computedIndex < index) {
    //        computedIndex = SendMessage(EM_LINEINDEX, 1 + line++);
    //     }
    //     line--;

    // EM_POSFROMCHAR  - convert char index into a Point
    // wParam = (LPPOINT) lpPoint;        // address of structure 
		                                  // receiving character position 
    // lParam = (LPARAM) wCharIndex;      // zero-based index of character 
    //
    // still need to turn the above into a Rect somehow...
    // (use font metrics on font info for letter to get height?  use
    // getLineHeight type of message?).

    // WM_GETFONT - get the font struct for the window control
    // wParam = lParam = 0
    // returns an HFONT 
    //  -or-
    // GetTextMetrics(hDC) to get the text info for the font selected
    // into the hDC of the control (tmHeight is what we want in the
    // TEXTMETRIC struct).
    // also GetCharWidth32() with the char at the index in question to get
    // the width of that char
    // *** Can't use GetTextMetrics/GetCharWidth32, as we don't have an hDC!! ***

    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN_NULL(self);
    AwtComponent* c = (AwtComponent*)pData;
/*
    int line = 0;
    int lineIndex = 0;
    while (lineIndex < i) {
        lineIndex = c->SendMessage(EM_LINEINDEX, 1 + line++);
    }
    line--;     // line is now the line which contains our character at position 'i'
    int offsetIndex = i - lineIndex;    // offsetIndex is now distance in on the line
* /
    POINT p;

    c->SendMessage(EM_POSFROMCHAR, (WPARAM) &p, (LPARAM) i);    // x coord is meaningful; y may not be

    // need to calculate charWidth, charHeight, and set p.y to something meangful

    jint charWidth;
    jint charHeight;

/*
    HFONT font = c->SendMessage(WM_GETFONT);
    if (GetCharWidth32(c->hdc, i, i, &charWidth) != 0) {        // [[[FIXME]]] need to get hDC!

        JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
        jobject rect = JNU_NewObjectByName(env, "java/awt/Rectangle", "(IIII)V",
                                           (jint) p.x, (jint) p.y, charWidth, charHeight);

        return rect;
    }
* /
    return (jobject) 0;

    CATCH_BAD_ALLOC_RET(0);
}
*/

} /* extern "C" */
