/*
 * @(#)awt_TextComponent.h	1.29 00/04/06
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef AWT_TEXTCOMPONENT_H
#define AWT_TEXTCOMPONENT_H

#include "awt_Component.h"

#include "java_awt_TextComponent.h"
#include "sun_awt_windows_WTextComponentPeer.h"


/************************************************************************
 * AwtTextComponent class
 */

class AwtTextComponent : public AwtComponent {
public:
    /* java.awt.TextComponent canAccessClipboard field ID */
    static jfieldID canAccessClipboardID;

    AwtTextComponent();

    virtual const char* GetClassName();

    // Part of the fix for 4118621
    virtual void Reshape(int x, int y, int w, int h);

    static WCHAR *AddCR(WCHAR *pStr, int nStrLen);
    static int RemoveCR(WCHAR *pStr);

    static char *AddCR(char *pStr, int nStrLen);
    static int RemoveCR(char *pStr);

    INLINE void SetText(const char* text) { 
        ::SetWindowText(GetHWnd(), text); 
    }
    INLINE void SetText(const WCHAR* text) {
        ::SetWindowTextW(GetHWnd(), text); 
    }

    INLINE int GetText(char* buffer, int size) { 
        return ::GetWindowText(GetHWnd(), buffer, size); 
    }
    INLINE int GetText(WCHAR* buffer, int size) {
        return ::GetWindowTextW(GetHWnd(), buffer, size); 
    }

    /*
     * For TextComponents that contains WCHAR strings or messages with
     * WCHAR parameters.  
     */
    INLINE LRESULT SendMessageW(UINT msg, WPARAM wParam = 0, LPARAM lParam = 0)
    {
        DASSERT(GetHWnd());
        return ::SendMessageW(GetHWnd(), msg, wParam, lParam);
    }

    void SetFont(AwtFont* font);

    /*
     * Windows message handler functions
     */
    MsgRouting WmNotify(UINT notifyCode);
    MsgRouting HandleEvent(MSG *msg, BOOL synthetic);
    MsgRouting WmPaste();

/*  To be fully implemented in a future release

    MsgRouting WmKeyDown(UINT wkey, UINT repCnt, 
                         UINT flags, BOOL system);  // accessibility support
*/


    //im --- for over the spot composition
    void SetCompositionWindow(); 

    INLINE HWND GetDBCSEditHandle() { return GetHWnd(); }

private:
    HFONT m_hFont;
    //im --- end

    // Used to prevent untrusted code from synthesizing a WM_PASTE message
    // by posting a <CTRL>-V KeyEvent
    BOOL    m_synthetic; 

    //
    // Accessibility support
    //
//public:
//    jlong javaEventsMask;
};

#endif /* AWT_TEXTCOMPONENT_H */
