/*
 * @(#)awt_Toolkit.cpp	1.132 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "stdhdrs.h"
#include <signal.h>

#if defined(_DEBUG) && defined(_MSC_VER) && _MSC_VER >= 1000
#include <crtdbg.h>
#endif

#include "awt_AWTEvent.h"
#include "awt_Component.h"
#include "awt_Canvas.h"
#include "awt_Clipboard.h"
#include "awt_Frame.h"
#include "awt_Dialog.h"
#include "awt_Font.h"
#include "awt_Cursor.h"
#include "awt_Graphics.h"
#include "awt_InputEvent.h"
#include "awt_KeyEvent.h"
#include "awt_List.h"
#include "awt_PopupMenu.h"
#include "awt_Toolkit.h"
#include "awt_DesktopProperties.h"
#include "awt_FileDialog.h"
#include "CmdIDList.h"
#include "awt_new.h"
#include <awt_image.h>

#include <awt_DnDDT.h>
#include <awt_DnDDS.h>

#include <java_awt_Toolkit.h>
#include <java_awt_event_InputMethodEvent.h>
#include <java_awt_peer_ComponentPeer.h>
#include <sun_awt_windows_WEmbeddedFrame.h>
#include <sun_awt_windows_WEmbeddedFramePeer.h>
#include "awt_Win32GraphicsEnv.h"

extern CriticalSection imageLock;
extern "C" void awt_dnd_initialize();
extern "C" void awt_dnd_uninitialize();
extern "C" BOOL g_bUserHasChangedInputLang;

/************************************************************************
 * Utilities
 */

/* Initialize the Java VM instance variable when the library is 
   first loaded */
JavaVM *jvm;

JNIEXPORT jint JNICALL
JNI_OnLoad(JavaVM *vm, void *reserved)
{
    TRY;

    jvm = vm;
    return JNI_VERSION_1_2;

    CATCH_BAD_ALLOC_RET(0);
}

#define IDT_AWT_MOUSECHECK 0x101

static const char* szAwtToolkitClassName = "SunAwtToolkit";

UINT AwtToolkit::GetAsyncMouseKeyState()
{
    static BOOL mbSwapped = ::GetSystemMetrics(SM_SWAPBUTTON);
    UINT mouseKeyState = 0;

    if (HIBYTE(::GetAsyncKeyState(VK_CONTROL)))
        mouseKeyState |= MK_CONTROL;
    if (HIBYTE(::GetAsyncKeyState(VK_SHIFT)))
        mouseKeyState |= MK_SHIFT;
    if (HIBYTE(::GetAsyncKeyState(VK_LBUTTON)))
        mouseKeyState |= (mbSwapped ? MK_RBUTTON : MK_LBUTTON);
    if (HIBYTE(::GetAsyncKeyState(VK_RBUTTON)))
        mouseKeyState |= (mbSwapped ? MK_LBUTTON : MK_RBUTTON);
    if (HIBYTE(::GetAsyncKeyState(VK_MBUTTON)))
        mouseKeyState |= MK_MBUTTON;
    return mouseKeyState;
}

//
// Normal ::GetKeyboardState call only works if current thread has
// a message pump, so provide a way for other threads to get
// the keyboard state
//
void AwtToolkit::GetAsyncKeyboardState(PBYTE keyboardState)
{
    CriticalSection::Lock	l(AwtToolkit::GetInstance().m_lockKB);
    DASSERT(!IsBadWritePtr(keyboardState, KB_STATE_SIZE));
    memcpy(keyboardState, AwtToolkit::GetInstance().m_lastKeyboardState,
	   KB_STATE_SIZE);
}

/************************************************************************
 * Exported functions
 */

extern "C" BOOL APIENTRY DllMain(HANDLE hInstance, DWORD ul_reason_for_call, 
                                 LPVOID)
{
    // Don't use the TRY and CATCH_BAD_ALLOC_RET macros if we're detaching
    // the library. Doing so causes awt.dll to call back into the VM during
    // shutdown. This crashes the HotSpot VM.
    if (ul_reason_for_call == DLL_PROCESS_ATTACH) {
        TRY;
	AwtToolkit::GetInstance().SetModuleHandle((HMODULE)hInstance);
	CATCH_BAD_ALLOC_RET(FALSE);
    }

    return TRUE;
}

/************************************************************************
 * AwtToolkit fields
 */

AwtToolkit AwtToolkit::theInstance;

/* ids for WToolkit fields accessed from native code */
jmethodID AwtToolkit::windowsSettingChangeMID;
jmethodID AwtToolkit::displayChangeMID;
/* ids for Toolkit methods */
jmethodID AwtToolkit::getDefaultToolkitMID;
jmethodID AwtToolkit::getFontMetricsMID;


/************************************************************************
 * JavaStringBuffer method
 */

JavaStringBuffer::JavaStringBuffer(JNIEnv *env, jstring jstr) {
    if (jstr != NULL) {
        int length = env->GetStringLength(jstr);
        buffer = new char[length * 2 + 1];
        AwtFont::javaString2multibyte(env, jstr, buffer, length);
    } else {
        buffer = new char[1];
        buffer[0] = 0;
    }
}


/************************************************************************
 * AwtToolkit methods
 */

AwtToolkit::AwtToolkit() {
    m_localPump = FALSE;
    m_mainThreadId = 0;
    m_toolkitHWnd = NULL;
    m_verbose = FALSE;
    m_isActive = TRUE;
    m_isDisposed = FALSE;

    m_verifyComponents = FALSE;
    m_breakOnError = FALSE;

    m_breakMessageLoop = FALSE;
    m_messageLoopResult = 0;

    m_lastMouseOver = NULL;
    m_mouseDown = FALSE;

    m_hGetMessageHook = 0;
    m_timer = 0;

    m_cmdIDs = new AwtCmdIDList();
    m_pModalDialog = NULL;
    m_peer = NULL;
    m_dllHandle = NULL;

    // initialize kb state array
    ::GetKeyboardState(m_lastKeyboardState);

    m_palEntryCount = 0;
    m_palSeed = 0;
    memset(m_palEntries, 0, 256 * sizeof(PALETTEENTRY));

    m_inverseDesktopLUT = NULL;
    m_inverseDesktopLUTSize = 0;
    m_inverseOffscreenLUT = NULL;
    m_inverseOffscreenLUTSize = 0;
    m_grayInverseLUT = NULL;
    m_offscreen = FALSE;

    // Fix for 4302507
    // Palette initialization has been moved into Win32GraphicsEnv static
    // initializer.
}

AwtToolkit::~AwtToolkit() {
/*
 *  The code has been moved to AwtToolkit::Dispose() method.
 */
}

HWND AwtToolkit::CreateToolkitWnd(const char* name)
{
    HWND hwnd = CreateWindow(
	szAwtToolkitClassName,
	(LPCTSTR)name,	                  /* window name */
	WS_DISABLED,			  /* window style */
	-1, -1,				  /* position of window */
	0, 0,				  /* width and height */
	NULL, NULL, 			  /* hWndParent and hWndMenu */
	GetModuleHandle(),
	NULL);				  /* lpParam */
    DASSERT(hwnd != NULL);
    return hwnd;
}

BOOL AwtToolkit::Initialize(BOOL localPump) {
    AwtToolkit& tk = AwtToolkit::GetInstance();

    if (!tk.m_isActive || tk.m_mainThreadId != 0) {
	/* Already initialized. */
	return FALSE;
    }

    /* Register this toolkit's helper window */
    VERIFY(tk.RegisterClass() != NULL);

    // Set up operator new/malloc out of memory handler.
    NewHandler::init();

#ifdef DEBUG
    // Turn on Runtime.runFinalizersOnExit so that we get a more accurate
    // leak report
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    DASSERT(env != NULL);
    jclass cls = env->FindClass("java/lang/Runtime");
    DASSERT(cls != NULL);
    jmethodID runFinalizersOnExitID =
        env->GetStaticMethodID(cls, "runFinalizersOnExit", "(Z)V");
    DASSERT(runFinalizersOnExitID != NULL);
    env->CallStaticVoidMethod(cls, runFinalizersOnExitID, JNI_TRUE);
#endif

	//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
	// Bugs 4032109, 4047966, and 4071991 to fix AWT
	//	crash in 16 color display mode.  16 color mode is supported.  Less
	//	than 16 color is not.
	// creighto@eng.sun.com 1997-10-07
	//
	// Check for at least 16 colors
    HDC hDC = ::GetDC(NULL);
	if ((::GetDeviceCaps(hDC, BITSPIXEL) * ::GetDeviceCaps(hDC, PLANES)) < 4) {
		::MessageBox(NULL,
			     "Sorry, but this release of Java requires at least 16 colors",
			     "AWT Initialization Error",
			     MB_ICONHAND | MB_APPLMODAL);	
		::DeleteDC(hDC);
		JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
		JNU_ThrowByName(env, "java/lang/InternalError",
				"unsupported screen depth");
		return FALSE;
	}
    ::ReleaseDC(NULL, hDC);
	///////////////////////////////////////////////////////////////////////////

    tk.m_localPump = localPump;
    tk.m_mainThreadId = ::GetCurrentThreadId();

    /* 
     * Create the one-and-only toolkit window.  This window isn't
     * displayed, but is used to route messages to this thread.  
     */
    tk.m_toolkitHWnd = tk.CreateToolkitWnd("theAwtToolkitWindow");
    DASSERT(tk.m_toolkitHWnd != NULL);

    /*
     * Setup a GetMessage filter to watch all messages coming out of our 
     * queue from PreProcessMsg().
     */
    tk.m_hGetMessageHook = ::SetWindowsHookEx(WH_GETMESSAGE,
					      (HOOKPROC)GetMessageFilter,
					      0, tk.m_mainThreadId);

    awt_dnd_initialize();

    return TRUE;
}

BOOL AwtToolkit::Dispose() {
    DTRACE_PRINTLN("In AwtToolkit::Dispose()");

    AwtToolkit& tk = AwtToolkit::GetInstance();

    if (!tk.m_isActive || tk.m_mainThreadId != ::GetCurrentThreadId()) {
        return FALSE;
    }

    tk.m_isActive = FALSE;

    awt_dnd_uninitialize();

    AwtObjectList::Cleanup();
    AwtFont::Cleanup();
    AwtGraphics::Cleanup();

    HWND toolkitHWndToDestroy = tk.m_toolkitHWnd;
    tk.m_toolkitHWnd = 0;
    VERIFY(::DestroyWindow(toolkitHWndToDestroy) != NULL);

    tk.UnregisterClass();

    ::UnhookWindowsHookEx(tk.m_hGetMessageHook);

    tk.m_mainThreadId = 0;

    delete [] tk.m_inverseDesktopLUT;
    delete [] tk.m_inverseOffscreenLUT;

    AwtImage::DisposePalette();
    delete tk.m_cmdIDs;

    tk._end_cursors();

    tk.m_isDisposed = TRUE;

    return TRUE;
}

ATOM AwtToolkit::RegisterClass() {
    WNDCLASS  wc;

    wc.style         = 0;
    wc.lpfnWndProc   = (WNDPROC)WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = AwtToolkit::GetInstance().GetModuleHandle(),
    wc.hIcon         = NULL;
    wc.hCursor       = NULL;
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = szAwtToolkitClassName;

    ATOM ret = ::RegisterClass(&wc);
    DASSERT(ret != NULL);
    return ret;
}

void AwtToolkit::UnregisterClass() {
    VERIFY(::UnregisterClass(szAwtToolkitClassName, AwtToolkit::GetInstance().GetModuleHandle()));
}

/*
 * Structure holding the information to create a component. This packet is 
 * sent to the toolkit window.
 */
struct ComponentCreatePacket {
    void* hComponent;
    void* hParent;
    void (*factory)(void*, void*);
};

/*
 * Create an AwtXxxx component using a given factory function
 * Implemented by sending a message to the toolkit window to invoke the 
 * factory function from that thread
 */
void AwtToolkit::CreateComponent(void* component, void* parent, 
				 ComponentFactory compFactory)
{
    ComponentCreatePacket ccp = { component, parent, compFactory };
    AwtToolkit::GetInstance().SendMessage(WM_AWT_COMPONENT_CREATE, 0,
					  (long)&ccp);
}

/*
 * Destroy an HWND that was created in the toolkit thread. Can be used on 
 * Components and the toolkit window itself.
 */
void AwtToolkit::DestroyComponent(AwtComponent* comp)
{
    AwtToolkit& tk = AwtToolkit::GetInstance();
    if (comp == tk.m_lastMouseOver) {
        tk.m_lastMouseOver = NULL;
    }

    /* Don't filter any post-destroy messages, such as WM_NCDESTROY */
    comp->UnsubclassHWND();

    tk.SendMessage(WM_AWT_DESTROY_WINDOW, (WPARAM)comp->GetHWnd(), 0);
}

/*
 * An AwtToolkit window is just a means of routing toolkit messages to here.
 */
LRESULT CALLBACK AwtToolkit::WndProc(HWND hWnd, UINT message, 
                                     WPARAM wParam, LPARAM lParam)
{
    TRY;

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    /*
     * Awt widget creation messages are routed here so that all
     * widgets are created on the main thread.  Java allows widgets
     * to live beyond their creating thread -- by creating them on
     * the main thread, a widget can always be properly disposed.
     */
    switch (message) {
      case WM_AWT_EXECUTE_SYNC: {
	  AwtObject * 			object = (AwtObject *)wParam;
	  AwtObject::ExecuteArgs *	args = (AwtObject::ExecuteArgs *)lParam;
	  DASSERT(!IsBadReadPtr(object, sizeof(AwtObject)) );
	  DASSERT(!IsBadReadPtr(args, sizeof(AwtObject::ExecuteArgs)) );
	  return object->WinThreadExecProc(args);
      }
      case WM_AWT_COMPONENT_CREATE: {
          ComponentCreatePacket* ccp = (ComponentCreatePacket*)lParam;
          DASSERT(ccp->factory != NULL);
          DASSERT(ccp->hComponent != NULL);
          (*ccp->factory)(ccp->hComponent, ccp->hParent);
          return 0;
      }
      case WM_AWT_DESTROY_WINDOW: {
          /* Destroy widgets from this same thread that created them */
          VERIFY(::DestroyWindow((HWND)wParam) != NULL);
          return 0;
      }
      case WM_AWT_DISPOSE: {
          AwtObject *p = (AwtObject *)wParam;
          delete p;
          return 0;
      }
      case WM_SYSCOLORCHANGE:
      case WM_SETTINGCHANGE: {
	  /* upcall to WToolkit when user changes configuration */
	  jobject peer = AwtToolkit::GetInstance().m_peer;
	  if (peer != NULL) {
	      env->CallVoidMethod(peer, AwtToolkit::windowsSettingChangeMID);
	  }
	  return 0;
      }
      case WM_TIMER: {
          // Create an artifical MouseExit message if the mouse left to 
          // a non-java window (bad mouse!)
          POINT pt;
	  AwtToolkit& tk = AwtToolkit::GetInstance();
	  VERIFY(::GetCursorPos(&pt));
	  
	  HWND hWndOver = ::WindowFromPoint(pt);
	  AwtComponent * last_M;
	  if ( AwtComponent::GetComponent(hWndOver) == NULL && tk.m_lastMouseOver != NULL ) {
	      last_M = tk.m_lastMouseOver;
	      // translate point from screen to target window
	      MapWindowPoints(HWND_DESKTOP, last_M->GetHWnd(), &pt, 1);
	      last_M->SendMessage(WM_AWT_MOUSEEXIT,
				  GetAsyncMouseKeyState(), 
				  POINTTOPOINTS(pt));
	      tk.m_lastMouseOver = 0;
	      ::KillTimer(tk.m_toolkitHWnd, tk.m_timer);
	      tk.m_timer = 0;
	  }
          return 0;
      }
      case WM_AWT_POPUPMENU_SHOW: {
          AwtPopupMenu* popup = (AwtPopupMenu*)wParam;
          popup->Show(env, (jobject)lParam);
          return 0;
      }
      case WM_DESTROYCLIPBOARD: {
	  if (!AwtClipboard::IsGettingOwnership())
	      AwtClipboard::LostOwnership((JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2));
          return 0;
      }
      case WM_AWT_LIST_SETMULTISELECT: {
          AwtList* list = (AwtList*)wParam;
          list->SetMultiSelect(lParam);
          return 0;
      }

      // Special awt message to call Imm APIs.
      // ImmXXXX() API must be used in the main thread.
      // In other thread these APIs does not work correctly even if
      // it returs with no error. (This restriction is not documented)
      // So we must use thse messages to call these APIs in main thread.
      case WM_AWT_CREATECONTEXT: {
          return ImmCreateContext();
      }
      case WM_AWT_DESTROYCONTEXT: {
	  ImmDestroyContext((HIMC)wParam);
          return 0;
      }
      case WM_AWT_ASSOCIATECONTEXT: {
          ImmAssociateContext((HWND)wParam, (HIMC)lParam);
          return 0;
      }
      case WM_AWT_ENDCOMPOSITION: {
	  /*right now we just cancel the composition string
	  may need to commit it in the furture
	  Changed to commit it according to the flag 10/29/98*/
          ImmNotifyIME((HIMC)wParam, NI_COMPOSITIONSTR,
	               (lParam ? CPS_COMPLETE : CPS_CANCEL), 0);
          return 0;
      }
      case WM_AWT_SETCONVERSIONSTATUS: {
	  DWORD cmode;
	  DWORD smode;
	  ImmGetConversionStatus((HIMC)wParam, (LPDWORD)&cmode, (LPDWORD)&smode);
	  ImmSetConversionStatus((HIMC)wParam, (DWORD)LOWORD(lParam), smode);
	  return 0;
      }
      case WM_AWT_GETCONVERSIONSTATUS: {
	  DWORD cmode;
	  DWORD smode;
	  ImmGetConversionStatus((HIMC)wParam, (LPDWORD)&cmode, (LPDWORD)&smode);
	  return cmode;
      }
      case WM_AWT_ACTIVATEKEYBOARDLAYOUT: {
          if (wParam && g_bUserHasChangedInputLang) {
	      // Input language has been changed since the last WInputMethod.getNativeLocale()
	      // call.  So let's honor the user's selection.
	      // Note: we need to check this flag inside the toolkit thread to synchronize access
	      // to the flag.
	      return FALSE;
	  }
	 
          if (lParam == (LPARAM)::GetKeyboardLayout(0)) {
	      // already active
	      return FALSE;
	  }
	 
	  // Since ActivateKeyboardLayout does not post WM_INPUTLANGCHANGEREQUEST,
	  // we explicitly need to do the same thing here.
	  static BYTE keyboardState[AwtToolkit::KB_STATE_SIZE];
	  AwtToolkit::GetAsyncKeyboardState(keyboardState);
	  WORD ignored;
	  ::ToAscii(VK_SPACE, ::MapVirtualKey(VK_SPACE, 0),
		    keyboardState, &ignored, 0);
		    
          return (LRESULT)(::ActivateKeyboardLayout((HKL)lParam, 0) != 0);
      }

      /*
       * send this message via ::SendMessage() and the MPT will acquire the
       * HANDLE synchronized with the sender's thread. The HANDLE must be
       * signalled or deadlock may occur between the MPT and the caller.
       */

      case WM_AWT_WAIT_FOR_SINGLE_OBJECT: {
	return ::WaitForSingleObject((HANDLE)lParam, INFINITE);
      }
      case WM_AWT_INVOKE_METHOD: {
	return (LRESULT)(*(void*(*)(void*))wParam)((void *)lParam);
      }
      case WM_AWT_INVOKE_VOID_METHOD: {
	return (LRESULT)(*(void*(*)(void))wParam)();
      }

      case WM_AWT_SETOPENSTATUS: {
	  ImmSetOpenStatus((HIMC)wParam, (BOOL)lParam);
	  return 0;
      }
      case WM_AWT_GETOPENSTATUS: {
	  return (DWORD)ImmGetOpenStatus((HIMC)wParam);
      }
      case WM_DISPLAYCHANGE: {
	  AwtCursor::DirtyAllCustomCursors();

      // Reinitialize screens, in case screens have been added, removed
      AwtWin32GraphicsEnv::DisposeScreens(env);
      AwtWin32GraphicsEnv::InitScreens(env);
  
      // Reset color parameters for the new display configuration(s)
      AwtWin32GraphicsEnv::SetColorParams(env);

	  //Call WToolkit.displayChanged()
	  jclass clazz = env->FindClass("sun/awt/windows/WToolkit");
	  env->CallStaticVoidMethod(clazz, AwtToolkit::displayChangeMID);
	  break;
      }
      case WM_AWT_SETCURSOR: {
	  ::SetCursor((HCURSOR)wParam);
	  return TRUE;
      }
      /* Session management */
      case WM_QUERYENDSESSION: {
	  /* Shut down cleanly */
	  raise(SIGTERM);
	  return TRUE;
      }
      case WM_ENDSESSION: {
	  // Keep pumping messages until the shutdown sequence halts the VM,
	  // or we exit the MessageLoop because of a WM_QUIT message
	  AwtToolkit& tk = AwtToolkit::GetInstance();
	  tk.MessageLoop();

	  // Dispose here instead of in eventLoop so that we don't have
	  // to return from the WM_ENDSESSION handler.
	  tk.Dispose();

	  // Never return. The VM will halt the process.
	  hang_if_shutdown();

	  // Should never get here.
	  DASSERT(FALSE);
      }

    }
    return DefWindowProc(hWnd, message, wParam, lParam);

    CATCH_BAD_ALLOC_RET(0);
}

LRESULT CALLBACK AwtToolkit::GetMessageFilter(int code,
					      WPARAM wParam, LPARAM lParam)
{
    TRY;

    if (code >= 0 && wParam == PM_REMOVE && lParam != 0) {
       if (AwtToolkit::GetInstance().PreProcessMsg(*(MSG*)lParam) !=
	       mrPassAlong) {
           /* PreProcessMsg() wants us to eat it */
           ((MSG*)lParam)->message = WM_NULL;  
       }
    }
    return ::CallNextHookEx(AwtToolkit::GetInstance().m_hGetMessageHook, code, 
			    wParam, lParam);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * The main message loop
 */

const int AwtToolkit::EXIT_ENCLOSING_LOOP      = 0;
const int AwtToolkit::EXIT_ALL_ENCLOSING_LOOPS = -1;

UINT AwtToolkit::MessageLoop()
{
    DTRACE_PRINTLN("AWT event loop started");

    m_messageLoopResult = 0;
    while (!m_breakMessageLoop) {            
        ::WaitMessage();               /* allow system to go idle */
        PumpWaitingMessages();         /* pumps any waiting messages */
    }
    if (m_messageLoopResult == EXIT_ALL_ENCLOSING_LOOPS)
	::PostQuitMessage(EXIT_ALL_ENCLOSING_LOOPS);
    m_breakMessageLoop = FALSE;

    DTRACE_PRINTLN("AWT event loop ended");

    return m_messageLoopResult;
}

/*
 * Exit the enclosing message loop(s).
 *
 * The message will be ignored if Windows is currently is in an internal
 * message loop (such as a scroll bar drag). So we first send IDCANCEL and
 * WM_CANCELMODE messages to every Window on the thread.
 */
static BOOL CALLBACK CancelAllThreadWindows(HWND hWnd, LPARAM)
{
    TRY;

    ::SendMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDCANCEL, 0), (LPARAM)hWnd);
    ::SendMessage(hWnd, WM_CANCELMODE, 0, 0);

    return TRUE;

    CATCH_BAD_ALLOC_RET(FALSE);
}
void AwtToolkit::QuitMessageLoop(int status) {
    if (status == EXIT_ALL_ENCLOSING_LOOPS) {
        ::EnumThreadWindows(MainThread(), (WNDENUMPROC)CancelAllThreadWindows,
			    0);
    }
    ::PostMessage(this->GetHWnd(), WM_QUIT, status, 0);
}

/*
 * Called by the message loop to pump the message queue when there are
 * messages waiting. Can also be called anywhere to pump messages.
 */
BOOL AwtToolkit::PumpWaitingMessages()
{
    MSG  msg;
    BOOL foundOne = FALSE;

    while (::PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {
        foundOne = TRUE;
        if (msg.message == WM_QUIT) {
            m_breakMessageLoop = TRUE;
            m_messageLoopResult = msg.wParam;
            if (m_messageLoopResult == EXIT_ALL_ENCLOSING_LOOPS)
		::PostQuitMessage(msg.wParam);  // make sure all loops exit
            break;
        }
        else if (msg.message != WM_NULL) {
	    /*
	     * The AWT in standalone mode (that is, dynamically loaded from the
	     * Java VM) doesn't have any translation tables to worry about, so
	     * TranslateAccelerator isn't called.
	     */

            ::TranslateMessage(&msg);
            ::DispatchMessage(&msg);
        }
    }
    return foundOne;
}

/*
 * Perform pre-processing on a message before it is translated &
 * dispatched.  Returns true to eat the message
 */
BOOL AwtToolkit::PreProcessMsg(MSG& msg)
{
    /*
     * Offer preprocessing first to the target component, then call out to
     * specific mouse and key preprocessor methods
     */
    AwtComponent* p = AwtComponent::GetComponent(msg.hwnd);
    if (p && p->PreProcessMsg(msg) == mrConsume)
        return TRUE;

    if (msg.message >= WM_MOUSEFIRST && msg.message <= WM_MOUSELAST ||
        msg.message >= WM_NCMOUSEMOVE && msg.message <= WM_NCMBUTTONDBLCLK) {
        if (PreProcessMouseMsg(p, msg))
            return TRUE;
    }
    else if (msg.message >= WM_KEYFIRST && msg.message <= WM_KEYLAST) {
        if (PreProcessKeyMsg(p, msg))
            return TRUE;
    }
    return FALSE;
}

BOOL AwtToolkit::PreProcessMouseMsg(AwtComponent* p, MSG& msg)
{
    WPARAM mouseWParam;
    LPARAM mouseLParam;
    if (msg.message >= WM_MOUSEFIRST && msg.message <= WM_MOUSELAST) {
        mouseWParam = msg.wParam;
        mouseLParam = msg.lParam;
    } else {
        mouseWParam = GetAsyncMouseKeyState();
    }

    /*
     * Get the window under the mouse, as it will be different if its
     * captured.
     */
    DWORD dwCurPos = ::GetMessagePos();
    DWORD dwScreenPos = dwCurPos;
    POINT curPos;
    curPos.x = LOWORD(dwCurPos);
    curPos.y = HIWORD(dwCurPos);
    HWND hWndFromPoint = ::WindowFromPoint(curPos);
    AwtComponent* mouseComp = 
        AwtComponent::GetComponent(hWndFromPoint);
    // If the point under the mouse isn't in the client area,
    // ignore it to maintain compatibility with Solaris (#4095172)
    RECT windowRect;
    ::GetClientRect(hWndFromPoint, &windowRect);
    POINT topLeft;
    topLeft.x = 0;
    topLeft.y = 0;
    ::ClientToScreen(hWndFromPoint, &topLeft);
    windowRect.top += topLeft.y;
    windowRect.bottom += topLeft.y;
    windowRect.left += topLeft.x;
    windowRect.right += topLeft.x;
    if ((curPos.y < windowRect.top) ||
        (curPos.y >= windowRect.bottom) ||
        (curPos.x < windowRect.left) ||
        (curPos.x >= windowRect.right)) {
        mouseComp = NULL;
	hWndFromPoint = NULL;
    }

    /*
     * Look for mouse transitions between windows & create 
     * MouseExit & MouseEnter messages 
     */
    if (mouseComp != m_lastMouseOver) {
	/*
	 * Send the messages right to the windows so that they are in 
	 * the right sequence.
	 */
        if (m_lastMouseOver) {
            dwCurPos = dwScreenPos;
            curPos.x = LOWORD(dwCurPos);
            curPos.y = HIWORD(dwCurPos);
            ::MapWindowPoints(HWND_DESKTOP, m_lastMouseOver->GetHWnd(),
                              &curPos, 1);
            mouseLParam = MAKELPARAM((WORD)curPos.x, (WORD)curPos.y);
            m_lastMouseOver->SendMessage(WM_AWT_MOUSEEXIT, mouseWParam, 
					 mouseLParam);
        }
        if (mouseComp) {
                dwCurPos = dwScreenPos;
                curPos.x = LOWORD(dwCurPos);
                curPos.y = HIWORD(dwCurPos);
                ::MapWindowPoints(HWND_DESKTOP, mouseComp->GetHWnd(),
                                  &curPos, 1);
                mouseLParam = MAKELPARAM((WORD)curPos.x, (WORD)curPos.y);
            mouseComp->SendMessage(WM_AWT_MOUSEENTER, mouseWParam,
				   mouseLParam);
        }
        m_lastMouseOver = mouseComp;
    }

    /*
     * Make sure we get at least one last chance to check for transitions 
     * before we sleep
     */
    if (m_lastMouseOver && !m_timer) {
        m_timer = ::SetTimer(m_toolkitHWnd, IDT_AWT_MOUSECHECK, 200, 0);
    }
    return FALSE;  /* Now go ahead and process current message as usual */
}

BOOL AwtToolkit::PreProcessKeyMsg(AwtComponent* p, MSG& msg)
{
    // get keyboard state for use in AwtToolkit::GetAsyncKeyboardState
    CriticalSection::Lock	l(m_lockKB);
    ::GetKeyboardState(m_lastKeyboardState);
    return FALSE;
}

UINT AwtToolkit::CreateCmdID(AwtObject* object) 
{ 
    return m_cmdIDs->Add(object);
}

void AwtToolkit::RemoveCmdID(UINT id)
{
    m_cmdIDs->Remove(id);
}

AwtObject* AwtToolkit::LookupCmdID(UINT id)
{
    return m_cmdIDs->Lookup(id);
}

HICON AwtToolkit::GetAwtIcon()
{
    return ::LoadIcon(GetModuleHandle(), "AWT_ICON");
}

void AwtToolkit::SetHeapCheck(long flag) {
    if (flag) {
        printf("heap checking not supported with this build\n");
    }
}

/* monitor state of screen palette to allow generation of inverse LUT for
 * drawing directly to the framebuffer
 */
void AwtToolkit::adjustPalette(HDC aHDC) {
    PALETTEENTRY pe[256];
    int numEntries = 0;
    HWND dtWND = NULL;
    HDC mainDC;
    int prevNumEntries;
    LPPALETTEENTRY palEntries;

    if (aHDC) {
        mainDC = aHDC;
        prevNumEntries = m_palEntryOffscreenCount;
        palEntries = m_palOffscreenEntries;
    } else {
        dtWND = ::GetDesktopWindow();
        mainDC = ::GetDC(dtWND);
        prevNumEntries = m_palEntryCount;
        palEntries = m_palEntries;
    }
    if (!mainDC) {
        return;
    }

    if (8 != ::GetDeviceCaps(mainDC, BITSPIXEL)) {
        goto Exit;
    }

    if (!aHDC) {
        numEntries = AwtImage::FetchPaletteEntries(mainDC, pe);
    } else {
        numEntries = ::GetPaletteEntries(AwtImage::m_hPal, 0, 256, pe);
    }

//    fprintf(stderr,"New Inverse LUT Entries %d %d\n",numEntries, prevNumEntries);
//    fprintf(stderr,"New Inverse LUT memcmp %d\n", memcmp(pe, palEntries, prevNumEntries * sizeof(PALETTEENTRY)));
    if (    (numEntries == prevNumEntries)
            && (0 == memcmp(pe, palEntries, prevNumEntries * sizeof(PALETTEENTRY)))) {
        goto Exit;
    }

    {
	CriticalSection::Lock l1(imageLock); // to prevent adjusting palette
					     // while inverse Lut is in use
					     // by rendering loops
	CriticalSection::Lock l2(m_paletteLock);
	numEntries = (numEntries > 256)? 256: numEntries;
	memcpy(palEntries, pe, numEntries * sizeof(PALETTEENTRY));
	if (!aHDC) {
	    GenerateInverseLUT(pe, numEntries, m_inverseDesktopLUT, m_inverseDesktopLUTSize);
	    m_palEntryCount = numEntries;
	    m_palSeed++;
	} else {
	    GenerateInverseLUT(pe, numEntries, m_inverseOffscreenLUT, m_inverseOffscreenLUTSize);
	    m_palEntryOffscreenCount = numEntries;
	}
	if (AwtImage::m_clrdata.grayscale == GS_INDEXGRAY) {
	    m_palEntryCount = numEntries;
	    GenerateGrayInverseLUT();
	}
    }
Exit:    
    if (!aHDC) {
        ::ReleaseDC(dtWND, mainDC);
    }
}

/*
 * state info needed for breadth-first recursion of color cube from
 * initial palette entries within the cube
 */

struct CubeStateInfo {
    unsigned int depth;
    unsigned int maxDepth;

    unsigned char *usedFlags;
    unsigned int  activeEntries;
    unsigned short *rgb;
    unsigned char *indices;
    unsigned char *iLUT;
};

#define INSERTNEW(state, rgb, index) do {                           \
        if (!state.usedFlags[rgb]) {                                \
            state.usedFlags[rgb] = 1;                               \
            state.iLUT[rgb] = index;                                \
            state.rgb[state.activeEntries] = rgb;                   \
            state.indices[state.activeEntries] = index;             \
            state.activeEntries++;                                  \
        }                                                           \
} while (0);
    

#define ACTIVATE(code, mask, delta, state, index) do {              \
    if (((rgb & mask) + delta) <= mask) {                           \
        rgb += delta;                                               \
        INSERTNEW(state, rgb, index);                               \
        rgb -= delta;                                               \
    }                                                               \
    if ((rgb & mask) >= delta) {                                    \
        rgb -= delta;                                               \
        INSERTNEW(state, rgb, index);                               \
        rgb += delta;                                               \
    }                                                               \
} while (0);

/* REMIND: does not deal well with bifurcation which happens when two
 * palette entries map to the same cube vertex
 */

static int
recurseLevel(CubeStateInfo *priorState) {
//    fprintf(stderr,"Level Recurse %d, entries %d\n", priorState->depth, priorState->activeEntries);
    CubeStateInfo currentState;
    memcpy(&currentState, priorState, sizeof(CubeStateInfo));
    currentState.rgb = (unsigned short *)
        safe_Malloc(6 * sizeof(unsigned short) * priorState->activeEntries);
    
    try {
        currentState.indices = (unsigned char *)
	    safe_Malloc(6 * sizeof(unsigned char) * priorState->activeEntries);
    } catch (std::bad_alloc&) {
        free(currentState.rgb);
	currentState.rgb = NULL;
	throw;
    }
    
    currentState.depth++;
    if (currentState.depth > priorState->maxDepth) {
        priorState->maxDepth = currentState.depth;
    }
    currentState.activeEntries = 0;
    for (int i = priorState->activeEntries - 1; i >= 0; i--) {
        unsigned short rgb = priorState->rgb[i];
        unsigned char  index = priorState->indices[i];
        ACTIVATE(rgb, 0x7c00, 0x0400, currentState, index);
        ACTIVATE(rgb, 0x03e0, 0x0020, currentState, index);
        ACTIVATE(rgb, 0x001f, 0x0001, currentState, index);
    }
    if (currentState.activeEntries) {
        try {
	  recurseLevel(&currentState);
	} catch (std::bad_alloc&) {
	    free(currentState.rgb);
	    free(currentState.indices);
	    throw;
        }
    }
    
    if (currentState.maxDepth > priorState->maxDepth) {
        priorState->maxDepth = currentState.maxDepth;
    }
    
    free(currentState.rgb);
    free(currentState.indices);
    return  1;
}

/*
 * REMIND: take core inversedLUT calculation to the shared tree and
 * recode the functions (Win32)awt_Image:initCubemap(),
 * (Win32)awt_Image:make_cubemap(), (Win32)AwtToolkit::GenerateInverseLUT(),
 * (Solaris)color:initCubemap() to call the shared codes.
 */
void AwtToolkit::GenerateInverseLUT(
        LPPALETTEENTRY aPalEntries, int aNumPalEntries,
        unsigned char *& aInverseLUT, int& aLutSize)
{
//    fprintf(stderr,"Generating Inverse LUT\n");
    aLutSize = 32*32*32;         // 2^5 * 2^5 * 2^5
    unsigned char *useFlags;
    unsigned char *newILut;

#ifdef DEBUG
    try {
#endif
        newILut = new unsigned char[aLutSize];
#ifdef DEBUG
    } catch (std::bad_alloc&) {
        fprintf(stderr,
		"Out of memory in AwtToolkit::GenerateInverseLUT1\n");
	throw;
    }
#endif

    unsigned char *tempLUT = aInverseLUT;
        
    try {
        useFlags = (unsigned char *)safe_Calloc(aLutSize, 1);
    } catch (std::bad_alloc&) {
        delete [] newILut;
#ifdef DEBUG
	fprintf(stderr,
		"Out of memory in AwtToolkit::GenerateInverseLUT2\n");
#endif
	throw;
    }
      
    // use existing palette as seed values
    CubeStateInfo currentState;
    currentState.depth          = 0;
    currentState.maxDepth       = 0;
    currentState.usedFlags      = useFlags;
    currentState.activeEntries  = 0;
    currentState.iLUT           = newILut;

    try {
        currentState.rgb = (unsigned short *)
	    safe_Malloc(256 * sizeof(unsigned short));
    } catch (std::bad_alloc&) {
        delete [] newILut;
	free(useFlags);
#ifdef DEBUG
	fprintf(stderr,
		"Out of memory in AwtToolkit::GenerateInverseLUT3\n");
#endif
	throw;
    }

    try {
        currentState.indices = (unsigned char *)
	    safe_Malloc(256 * sizeof(unsigned char));
    } catch (std::bad_alloc&) {
        delete [] newILut;
	free(useFlags);
	free(currentState.rgb);
#ifdef DEBUG
	fprintf(stderr,
		"Out of memory in AwtToolkit::GenerateInverseLUT4\n");
#endif
	throw;
    }

    for (int i = 0; i < 128; i++) {
        unsigned short rgb;
	rgb = ((aPalEntries[i].peRed & 0xf8) << 7)
	    | ((aPalEntries[i].peGreen & 0xf8) << 2)
	    | (aPalEntries[i].peBlue >> 3);
	INSERTNEW(currentState, rgb, i);
	rgb = ((aPalEntries[255-i].peRed & 0xf8) << 7)
	    | ((aPalEntries[255-i].peGreen & 0xf8) << 2)
	    | (aPalEntries[255-i].peBlue >> 3);
	INSERTNEW(currentState, rgb, 255-i);
    }

    try {
        recurseLevel(&currentState);
    } catch (std::bad_alloc&) {
        delete [] newILut;
	free(useFlags);
	free(currentState.rgb);
	free(currentState.indices);
#ifdef DEBUG
	fprintf(stderr,
		"Out of memory in AwtToolkit::GenerateInverseLUT5\n");
#endif
	throw;
    }

    free(useFlags);
    free(currentState.rgb);
    free(currentState.indices);

    aInverseLUT = newILut;  // atomic insertion operation
    delete [] tempLUT;
//        fprintf(stderr,"Inverse LUT maxdepth %d\n",currentState.maxDepth);
}

void AwtToolkit::GenerateGrayInverseLUT(void) {
    int *newILUT = new int[256];
    int *tempILUT = m_grayInverseLUT;
    int grayvals[256];
    int grayindices[256];
    int gcount;
    int mindist, besti, d;
    PALETTEENTRY *pPalEntries;

    /* REMIND: should a more sophisticated algorithm be used to
       map inverse grays, e.g. best luminance match?
    */
    pPalEntries = m_palEntries;
    gcount = 0;
    for (int i = 0; i < m_palEntryCount; i++, pPalEntries++) {
        if ((pPalEntries->peRed == pPalEntries->peGreen) &&
	        (pPalEntries->peGreen == pPalEntries->peBlue)) {
	    grayvals[gcount] = (unsigned char) pPalEntries->peRed;
	    grayindices[gcount++] = i;
	}
    }

    for (int g = 0; g < 256; g++) {
        int *pg = grayvals;
	mindist = 256;
	besti = 0;
	for (int i = 0 ; i < gcount; i++, pg++) {
	    d = *pg - g;
	    if (d < 0)
	        d = -d;
	    if (d < mindist) {
	        besti = i;
		if (d == 0) {
		    break;
		}
		mindist = d;
	    }
	}
	newILUT[g] = grayindices[besti];
    }
    m_grayInverseLUT = newILUT;  // atomic insertion operation
    delete [] tempILUT;
}

/* cursor support 
 */
void AwtToolkit::_begin_cursors() {
  m_nCustomCursor = FALSE;
  _detect_themes();
}
void AwtToolkit::_end_cursors() {
  m_nCustomCursor = FALSE;
}
void AwtToolkit::_detect_themes() {
  HKEY regKey = NULL;
  
  m_nHasThemes = FALSE;
  
  if( ERROR_SUCCESS == ::RegOpenKeyEx(
				      HKEY_CURRENT_USER,
				      "Control Panel\\Cursors",
				      0,
				      KEY_READ,
				      &regKey ) ) {
    char nBuffer[0xFF];
    DWORD buffLen = 0xFF;
    
    if( ERROR_SUCCESS == ::RegQueryValueEx(
					   regKey,
					   "Arrow",
					   0,0,(LPBYTE)nBuffer,
					   &buffLen) ) {
      HCURSOR hC = ::LoadCursorFromFile(nBuffer);
      if( NULL != hC ) {
	m_nHasThemes = TRUE;
	::DestroyCursor(hC);
      }
    }
    ::RegCloseKey(regKey);
  }
}

void throw_if_shutdown(void) throw (awt_toolkit_shutdown)
{
    AwtToolkit::GetInstance().VerifyActive();
}
void hang_if_shutdown(void)
{
    try {
        AwtToolkit::GetInstance().VerifyActive();
    } catch (awt_toolkit_shutdown&) {
        // Never return. The VM will halt the process.
        ::WaitForSingleObject(::CreateEvent(NULL, TRUE, FALSE, NULL),
                              INFINITE);
        // Should never get here.
        DASSERT(FALSE);
    }
}

/************************************************************************
 * Toolkit native methods
 */

extern "C" {
  
/*
 * Class:     java_awt_Toolkit
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL 
Java_java_awt_Toolkit_initIDs(JNIEnv *env, jclass cls) {
    TRY;

    AwtToolkit::getDefaultToolkitMID = 
	env->GetStaticMethodID(cls,"getDefaultToolkit","()Ljava/awt/Toolkit;");
    AwtToolkit::getFontMetricsMID =
	env->GetMethodID(cls, "getFontMetrics", 
			 "(Ljava/awt/Font;)Ljava/awt/FontMetrics;");
    DASSERT(AwtToolkit::getDefaultToolkitMID != NULL);
    DASSERT(AwtToolkit::getFontMetricsMID != NULL);

    CATCH_BAD_ALLOC;
}

} /* extern "C" */

/************************************************************************
 * SunToolkit native methods
 */

extern "C" {
  
/*
 * Class:     sun_awt_SunToolkit
 * Method:    getPrivateKey
 * Signature: (Ljava/lang/Object;)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL
Java_sun_awt_SunToolkit_getPrivateKey(JNIEnv *env, jclass cls, jobject obj)
{
    jobject key = obj;

    // get global reference of java/awt/Component class (run only once)
    static jclass componentCls = NULL;
    if (componentCls == NULL) {
        jclass componentClsLocal = env->FindClass("java/awt/Component");
        DASSERT(componentClsLocal != NULL);
        if (componentClsLocal == NULL) {
            /* exception already thrown */
            return key;
        }
        componentCls = (jclass)env->NewGlobalRef(componentClsLocal);
        env->DeleteLocalRef(componentClsLocal);
    }

    // get global reference of java/awt/MenuComponent class (run only once)
    static jclass menuComponentCls = NULL;
    if (menuComponentCls == NULL) {
        jclass menuComponentClsLocal = env->FindClass("java/awt/MenuComponent");
        DASSERT(menuComponentClsLocal != NULL);
        if (menuComponentClsLocal == NULL) {
            /* exception already thrown */
            return key;
        }
        menuComponentCls = (jclass)env->NewGlobalRef(menuComponentClsLocal);
        env->DeleteLocalRef(menuComponentClsLocal);
    }

    /*
     * Fix for BugTraq ID 4254701.
     * Don't use Components and MenuComponents as keys in hash maps.
     * We use private keys instead.
     */
    if (env->IsInstanceOf(obj, componentCls)) {
        key = env->GetObjectField(obj, AwtComponent::privateKeyID);
    } else if (env->IsInstanceOf(obj, menuComponentCls)) {
        key = env->GetObjectField(obj, AwtMenuItem::privateKeyID);
    }
    return key;
}

} /* extern "C" */

/************************************************************************
 * WToolkit native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    AwtToolkit::windowsSettingChangeMID =
	env->GetMethodID(cls, "windowsSettingChange", "()V");
    DASSERT(AwtToolkit::windowsSettingChangeMID != 0);

    AwtToolkit::displayChangeMID =
    env->GetStaticMethodID(cls, "displayChange", "()V");
    DASSERT(AwtToolkit::displayChangeMID != 0);

    CATCH_BAD_ALLOC;
}


/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    embeddedInit
 * Signature: ()V
 */
JNIEXPORT jboolean JNICALL
Java_sun_awt_windows_WToolkit_embeddedInit(JNIEnv *env, jclass cls)
{
    TRY;

    return AwtToolkit::GetInstance().Initialize(FALSE);

    CATCH_BAD_ALLOC_RET(JNI_FALSE);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    embeddedDispose
 * Signature: ()V
 */
JNIEXPORT jboolean JNICALL
Java_sun_awt_windows_WToolkit_embeddedDispose(JNIEnv *env, jclass cls)
{
    TRY;

    BOOL retval = AwtToolkit::GetInstance().Dispose();
    AwtToolkit::GetInstance().SetPeer(env, NULL);
    return retval;

    CATCH_BAD_ALLOC_RET(JNI_FALSE);
}


/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT jboolean JNICALL
Java_sun_awt_windows_WToolkit_init(JNIEnv *env, jobject self)
{
    TRY;

    AwtToolkit::GetInstance().SetPeer(env, self);

    // This call will fail if the Toolkit was already initialized.
    // In that case, we don't want to start another message pump.
    return AwtToolkit::GetInstance().Initialize(TRUE);

    CATCH_BAD_ALLOC_RET(FALSE);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    eventLoop
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_eventLoop(JNIEnv *env, jobject self)
{
    TRY;

    DASSERT(AwtToolkit::GetInstance().localPump());

    AwtToolkit::GetInstance().MessageLoop();

    AwtToolkit::GetInstance().Dispose();

    /*
     * IMPORTANT NOTES:
     *   The AwtToolkit has been destructed by now.
     * DO NOT CALL any method of AwtToolkit!!!
     */

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    shutdown
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_shutdown(JNIEnv *env, jobject self)
{
    TRY;

    AwtToolkit& tk = AwtToolkit::GetInstance();

    tk.QuitMessageLoop(AwtToolkit::EXIT_ALL_ENCLOSING_LOOPS);
    
    while (!tk.IsDisposed()) {
        Sleep(100);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    startSecondaryEventLoop
 * Signature: ()V;
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_startSecondaryEventLoop(
    JNIEnv *env,
    jclass)
{
    TRY;

    DASSERT(AwtToolkit::MainThread() == ::GetCurrentThreadId());

    AwtToolkit::GetInstance().MessageLoop();

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    quitSecondaryEventLoop
 * Signature: ()V;
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_quitSecondaryEventLoop(
    JNIEnv *env,
    jclass)
{
    TRY;

    AwtToolkit::GetInstance().QuitMessageLoop(AwtToolkit::EXIT_ENCLOSING_LOOP);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    makeColorModel
 * Signature: ()Ljava/awt/image/ColorModel;
 */
JNIEXPORT jobject JNICALL 
Java_sun_awt_windows_WToolkit_makeColorModel(JNIEnv *env, jclass cls) 
{
    TRY;

    return AwtImage::getColorModel(env);

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getScreenResolution
 * Signature: ()I
 */
JNIEXPORT jint JNICALL 
Java_sun_awt_windows_WToolkit_getScreenResolution(JNIEnv *env, jobject self) 
{
    TRY;

    HWND hWnd = ::GetDesktopWindow();
    HDC hDC = ::GetDC(hWnd);
    long result = ::GetDeviceCaps(hDC, LOGPIXELSX);
    ::ReleaseDC(hWnd, hDC);
    return result;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getMaximumCursorColors
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WToolkit_getMaximumCursorColors(JNIEnv *env, jobject self)
{
    TRY;

    HDC hIC = ::CreateIC("DISPLAY", NULL, NULL, NULL);

    int nColor = 256;
    switch (::GetDeviceCaps(hIC, BITSPIXEL) * ::GetDeviceCaps(hIC, PLANES)) {
	case 1:		nColor = 2; 		break;
	case 4:		nColor = 16; 		break;
	case 8:		nColor = 256; 		break;
        case 16:        nColor = 65536;         break;
	case 24:	nColor = 16777216; 	break;
    }
    ::DeleteDC(hIC);
    return nColor;

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getScreenWidth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL 
Java_sun_awt_windows_WToolkit_getScreenWidth(JNIEnv *env, jobject self)
{
    TRY;

    return ::GetSystemMetrics(SM_CXSCREEN);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getScreenHeight
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WToolkit_getScreenHeight(JNIEnv *env, jobject self)
{
    TRY;

    return ::GetSystemMetrics(SM_CYSCREEN);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    sync
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_sync(JNIEnv *env, jobject self)
{
    TRY;

    VERIFY(::GdiFlush());

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    beep
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_beep(JNIEnv *env, jobject self)
{
    TRY;

    VERIFY(::MessageBeep(MB_OK));

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getLockingKeyStateNative
 * Signature: (I)B
 */
JNIEXPORT jboolean JNICALL
Java_sun_awt_windows_WToolkit_getLockingKeyStateNative(JNIEnv *env, jobject self, jint javaKey)
{
    TRY;

    UINT windowsKey, modifiers;
    AwtComponent::JavaKeyToWindowsKey(javaKey, &windowsKey, &modifiers);
    
    if (windowsKey == 0) {
        JNU_ThrowByName(env, "java/lang/UnsupportedOperationException", "Keyboard doesn't have requested key");
        return JNI_FALSE;
    }
    
    // low order bit in keyboardState indicates whether the key is toggled 
    BYTE keyboardState[AwtToolkit::KB_STATE_SIZE];
    AwtToolkit::GetAsyncKeyboardState(keyboardState);
    return keyboardState[windowsKey] & 0x01;

    CATCH_BAD_ALLOC_RET(JNI_FALSE);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    setLockingKeyStateNative
 * Signature: (IB)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_setLockingKeyStateNative(JNIEnv *env, jobject self, jint javaKey, jboolean state)
{
    TRY;

    UINT windowsKey, modifiers;
    AwtComponent::JavaKeyToWindowsKey(javaKey, &windowsKey, &modifiers);
    
    if (windowsKey == 0) {
        JNU_ThrowByName(env, "java/lang/UnsupportedOperationException", "Keyboard doesn't have requested key");
        return;
    }

    // if the key isn't in the desired state yet, simulate key events to get there
    // low order bit in keyboardState indicates whether the key is toggled 
    BYTE keyboardState[AwtToolkit::KB_STATE_SIZE];
    AwtToolkit::GetAsyncKeyboardState(keyboardState);
    if ((keyboardState[windowsKey] & 0x01) != state) {
        ::keybd_event(windowsKey, 0, 0, 0);
        ::keybd_event(windowsKey, 0, KEYEVENTF_KEYUP, 0);
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    loadSystemColors
 * Signature: ([I)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_loadSystemColors(JNIEnv *env, jobject self,
					       jintArray colors)
{
    TRY;

    static int indexMap[] = {
        COLOR_DESKTOP, /* DESKTOP */
        COLOR_ACTIVECAPTION, /* ACTIVE_CAPTION */
        COLOR_CAPTIONTEXT, /* ACTIVE_CAPTION_TEXT */
        COLOR_ACTIVEBORDER, /* ACTIVE_CAPTION_BORDER */
        COLOR_INACTIVECAPTION, /* INACTIVE_CAPTION */
        COLOR_INACTIVECAPTIONTEXT, /* INACTIVE_CAPTION_TEXT */
        COLOR_INACTIVEBORDER, /* INACTIVE_CAPTION_BORDER */
        COLOR_WINDOW, /* WINDOW */
        COLOR_WINDOWFRAME, /* WINDOW_BORDER */
        COLOR_WINDOWTEXT, /* WINDOW_TEXT */
        COLOR_MENU, /* MENU */
        COLOR_MENUTEXT, /* MENU_TEXT */
        COLOR_3DFACE, /* TEXT */
        COLOR_BTNTEXT, /* TEXT_TEXT */
        COLOR_HIGHLIGHT, /* TEXT_HIGHLIGHT */
        COLOR_HIGHLIGHTTEXT, /* TEXT_HIGHLIGHT_TEXT */
        COLOR_GRAYTEXT, /* TEXT_INACTIVE_TEXT */
        COLOR_3DFACE, /* CONTROL */
        COLOR_BTNTEXT, /* CONTROL_TEXT */
        COLOR_3DLIGHT, /* CONTROL_HIGHLIGHT */
        COLOR_3DHILIGHT, /* CONTROL_LT_HIGHLIGHT */
        COLOR_3DSHADOW, /* CONTROL_SHADOW */
        COLOR_3DDKSHADOW, /* CONTROL_DK_SHADOW */
        COLOR_SCROLLBAR, /* SCROLLBAR */
        COLOR_INFOBK, /* INFO */
        COLOR_INFOTEXT, /* INFO_TEXT */
    };

    jint colorLen = env->GetArrayLength(colors);
    jint* colorsPtr = NULL;
    try {
        colorsPtr = (jint *)env->GetPrimitiveArrayCritical(colors, 0);
	for (int i = 0; i < sizeof indexMap && i < colorLen; i++) {
	    colorsPtr[i] = DesktopColor2RGB(indexMap[i]);
	}
    } catch (...) {
        if (colorsPtr != NULL) {
	    env->ReleasePrimitiveArrayCritical(colors, colorsPtr, 0);
	}
	throw;
    }

    env->ReleasePrimitiveArrayCritical(colors, colorsPtr, 0);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getComponentFromNativeWindowHandle
 * Signature: (I)Ljava/awt/Component;
 */
JNIEXPORT jobject JNICALL 
Java_sun_awt_windows_WToolkit_getComponentFromNativeWindowHandle(JNIEnv *env,
                                        jobject self, jint hwnd)
{
    TRY;

    HWND hWnd = (HWND)hwnd;
    if (!::IsWindow(hWnd))
        return NULL;

    AwtComponent* comp = AwtComponent::GetComponent(hWnd);
    if (comp == NULL)
        return NULL;

    return comp->GetTarget(env);

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WToolkit
 * Method:    getNativeWindowHandleFromPeer
 * Signature: (Lsun/awt/windows/WComponentPeer;)I
 */
JNIEXPORT jint JNICALL 
Java_sun_awt_windows_WToolkit_getNativeWindowHandleFromPeer(
                        JNIEnv *env, jobject self, jobject peer)
{
    TRY;

    return (jint)(((AwtComponent*)JNI_GET_PDATA(peer))->GetHWnd());

    CATCH_BAD_ALLOC_RET(0);
}

JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_pushModality(JNIEnv *env, jobject self)
{
    TRY;

    JNI_CHECK_NULL_RETURN(self, "null peer");
    AwtDialog::ModalDisable(NULL);

    CATCH_BAD_ALLOC;
}

JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_popModality(JNIEnv *env, jobject self)
{
    TRY;

    JNI_CHECK_NULL_RETURN(self, "null peer");
    AwtDialog::ModalEnable(NULL);
    // shouldn't be necessary since containing app will
    // reenable it's parent window when it's dialog goes
    // away
//    AwtDialog::ModalNextWindowToFront(GetHWnd());

    CATCH_BAD_ALLOC;
}

JNIEXPORT void JNICALL
Java_sun_awt_windows_WToolkit_finalize(JNIEnv *env, jobject self)
{
#ifdef DEBUG
    TRY_NO_VERIFY;

    // If this method was called, that means runFinalizersOnExit is turned
    // on and the VM is exiting cleanly. We should signal the debug memory
    // manager to generate a leaks report.
    AwtDebugSupport::GenerateLeaksReport();

    CATCH_BAD_ALLOC;
#endif
}

} /* extern "C" */

/* Convert a Windows desktop color index into an RGB value. */
COLORREF DesktopColor2RGB(int colorIndex) {
    long sysColor = ::GetSysColor(colorIndex);
    return ((GetRValue(sysColor)<<16) | (GetGValue(sysColor)<<8) | 
	    (GetBValue(sysColor)) | 0xff000000);
}

