/*
 * @(#)awt_Win32GraphicsConfig.cpp	1.17 01/02/09
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include "awt.h"
#include <sun_awt_Win32GraphicsConfig.h>
#include "awt_Win32GraphicsConfig.h"
#include "awt_Win32GraphicsEnv.h"
#include "awt_Canvas.h"

//Info for building a ColorModel
#include "awt_image.h" 
#include "java_awt_image_DataBuffer.h"

//Local utility functions
static int shiftToMask(int numBits, int shift);

/*
 * AwtWin32GraphicsConfig fields
 */

jfieldID AwtWin32GraphicsConfig::win32GCVisualID;
jfieldID AwtWin32GraphicsConfig::win32DeviceID;

/*
 * Class:     sun_awt_Win32GraphicsConfig
 * Method:    initIDs
 * Signature: ()V
 */

JNIEXPORT void JNICALL
Java_sun_awt_Win32GraphicsConfig_initIDs
    (JNIEnv *env, jclass thisCls)
{
    TRY;
    AwtWin32GraphicsConfig::win32GCVisualID = env->GetFieldID(thisCls,
	 "visual", "I");
    DASSERT(AwtWin32GraphicsConfig::win32GCVisualID);
    
    AwtWin32GraphicsConfig::win32DeviceID= env->GetFieldID(thisCls,
       "screen", "Lsun/awt/Win32GraphicsDevice;");
    DASSERT(AwtWin32GraphicsConfig::win32DeviceID);

	CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_Win32GraphicsConfig
 * Method:    makeColorModel
 * Signature: ()Ljava/awt/image/ColorModel
 */

JNIEXPORT jobject JNICALL
    Java_sun_awt_Win32GraphicsConfig_makeColorModel 
    (JNIEnv *env, jobject thisPtr, jint screen)
{
    TRY;
    jclass cmClazz = NULL;
    jmethodID mID;
    jobject retClrMdl = NULL;
    jint numBits, rmask, gmask, bmask, amask;
    PIXELFORMATDESCRIPTOR pfd;
    jint pfdIdx;
    unsigned int bits;
    HDC hDC = NULL;
    
    try {
        pfdIdx = env->GetIntField(thisPtr,
                                  AwtWin32GraphicsConfig::win32GCVisualID);
        //If GDI PixelFormat calls fail, a pfdIdx of 0 is used and PixelFormats
        //are not used.
        if (pfdIdx == 0) {
            return AwtImage::getColorModel(env);
        }
        
        hDC = AwtWin32GraphicsEnv::GetDCFromScreen(screen);
        int result = ::DescribePixelFormat(hDC, (int)pfdIdx,
                                           sizeof(PIXELFORMATDESCRIPTOR), &pfd);

        //Create ColorModel based on values in PFD
	if (pfd.cColorBits == 24) {
	    retClrMdl = JNU_NewObjectByName(env,
					    "sun/awt/Win32ColorModel24",
					    "()V");
            if (retClrMdl == NULL) {
                throw std::bad_alloc();
            }
	}
	else if (pfd.iPixelType == PFD_TYPE_RGBA || pfd.cColorBits > 8) {
            //DirectColorModel
            cmClazz = env->FindClass("java/awt/image/DirectColorModel");
            mID = env->GetMethodID(cmClazz, "<init>", "(IIIII)V");
            rmask = shiftToMask(pfd.cRedBits, pfd.cRedShift);
            gmask = shiftToMask(pfd.cGreenBits, pfd.cGreenShift);
            bmask = shiftToMask(pfd.cBlueBits, pfd.cBlueShift);

            /* Fix for 4263603:
             * Alpha in PixelFormats should not be included by default in 
             * ColorModels.  Hardware translucency will be handled elsewhere.
             */
            amask = 0;

            numBits = 0;
            bits = (rmask | gmask | bmask | amask);
            while (bits) {
                numBits++;
                bits >>= 1;
            }

            retClrMdl = env->NewObject(cmClazz,  //DirectColorModel
                                       mID,     //Constructor
                                       numBits, //Bits used per pixel
                                       rmask,   //Mask for red bits
                                       gmask,   //Mask for green bits
                                       bmask,   //Mask for blue bits
                                       amask    //Mask for alpha bits
                                       );
            if (retClrMdl == NULL) {
                throw std::bad_alloc();
            }
        }
        else if (pfd.iPixelType == PFD_TYPE_COLORINDEX && pfd.cColorBits == 8) {
            //IndexColorModel
            //This code is based on (shamelessly copied from) AwtImage.getColorModel()
            jintArray hRGB = env->NewIntArray(256);
            unsigned int *rgb = NULL, *rgbP = NULL;
            RGBQUAD *c, *cBase;
            jboolean hasalpha = JNI_FALSE;
            int i;

            /* Create the LUT from the color map */
            cBase = g_colorInfo[COLOR8].m_pPaletteColors;
            DASSERT(cBase);
            try {
                rgb = (unsigned int *) env->GetPrimitiveArrayCritical(hRGB, 0);
                if (rgb == NULL) {
                    if (hDC != NULL) {
                        ::ReleaseDC(NULL, hDC);
                        hDC = NULL;
                    }
                    return NULL;
                }
                rgbP = rgb;

                //Another potential multiscreen problem            
                if (AwtImage::m_clrdata.grayscale == GS_INDEXGRAY) {

                    /* For IndexColorModel, pretend first 10 colors and last
                       10 colors are transparent black.  This makes 
                       ICM.allgrayopaque true.
                    */
                    hasalpha = JNI_TRUE;
                    for (i=0; i < 10; i++) {
                        *rgbP++ = 0x00000000;
                    }
                    for (i=10, c = cBase + 10; i < 246; c++ /*heh*/, i++) {
                        *rgbP++ = 0xff000000 |
                            ((c->rgbRed&0xff)<<16)  |
                            ((c->rgbGreen&0xff)<<8) |
                            (c->rgbBlue&0xff);
                    }
                    for (i=246; i < 256; i++) {
                        *rgbP++ = 0x00000000;
                    }
                } else {
                    for (i=0, c = cBase; i < 256; c++, i++) {
                        *rgbP++ = 0xff000000 |
                            ((c->rgbRed&0xff)<<16)  |
                            ((c->rgbGreen&0xff)<<8) |
                            (c->rgbBlue&0xff);
                    }
                }
            } catch (...) {
                env->ReleasePrimitiveArrayCritical(hRGB, rgb, JNI_COMMIT);
                throw;
            }

            env->ReleasePrimitiveArrayCritical(hRGB, rgb, JNI_COMMIT);

            /* Construct a new color model */
            retClrMdl = JNU_NewObjectByName(env, 
                                            "java/awt/image/IndexColorModel",
                                            "(II[IIZII)V",
                                            8, 256,
                                            hRGB, 0, 
                                            hasalpha,
                                            -1,
                                            java_awt_image_DataBuffer_TYPE_BYTE);

            if (retClrMdl == NULL) {
                throw std::bad_alloc();
            }
        }
    } //try
    catch(std::bad_alloc&) {
        if (hDC != NULL) {
            ::ReleaseDC(NULL, hDC);
            hDC = NULL;
        }
        throw;
    }

    ::ReleaseDC(NULL, hDC);
    hDC = NULL;
    return retClrMdl;
	CATCH_BAD_ALLOC_RET(0);
}

/*
 *  shiftToMask:
 *  This function converts between cXXXBits and cXXXShift
 *  fields in the Windows GDI PIXELFORMATDESCRIPTOR and the mask
 *  fields passed to the DirectColorModel constructor.
 */
inline int shiftToMask(int numBits, int shift) {
    int mask = 0xFFFFFFFF;

    //Shift in numBits 0s
    mask = mask << numBits;
    mask = ~mask;
    //shift left by value of shift
    mask = mask << shift;
    return mask;
}

/*
 * Class:     sun_awt_Win32GraphicsConfig
 * Method:    getBounds
 * Signature: ()Ljava/awt/Rectangle
 */
JNIEXPORT jobject JNICALL
    Java_sun_awt_Win32GraphicsConfig_getBounds(JNIEnv *env, jobject thisobj,
                                               jint screen)
{
    jclass clazz;
    jmethodID mid;
    jobject bounds = NULL;
    
    clazz = env->FindClass("java/awt/Rectangle");
    mid = env->GetMethodID(clazz, "<init>", "(IIII)V");
    if (mid != 0) {
        RECT    rRW     = {0,0,0,0};
        if( TRUE == ::MonitorBounds(AwtWin32GraphicsEnv::GetMHNDFromScreen(
                                                              screen), &rRW) ) {
            bounds = env->NewObject(clazz, mid,
                                    rRW.left, rRW.top,
                                    rRW.right - rRW.left,
                                    rRW.bottom - rRW.top);
            if (safe_ExceptionOccurred(env)) {
                return 0;
            }
        }
    }
    return bounds;
}
