/*
 * @(#)awt_Win32GraphicsDevice.cpp	1.14 01/02/09
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include <jni.h>
#include <awt.h>
#include <sun_awt_Win32GraphicsDevice.h>
#include "awt_Canvas.h"
#include "awt_Image.h"
#include "awt_Win32GraphicsEnv.h"
#include "awt_Win32GraphicsDevice.h"

jfieldID AwtWin32GraphicsDevice::screenID;

    const DWORD REQUIRED_FLAGS = (   //Flags which must be set in
     PFD_SUPPORT_GDI |               //in the PixelFormatDescriptor.
     PFD_DRAW_TO_WINDOW);            //Used to choose the default config
                                     //and to check formats in
                                     //isPixFmtSupported()

/*
 * Class:     sun_awt_Win32GraphicsDevice
 * Method:    initIDs
 * Signature: ()V
 */

JNIEXPORT void JNICALL
Java_sun_awt_Win32GraphicsDevice_initIDs
    (JNIEnv *env, jclass thisCls)
{
    TRY;
    AwtWin32GraphicsDevice::screenID = env->GetFieldID(thisCls,
       "screen", "I");
    DASSERT(AwtWin32GraphicsDevice::screenID);
      CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_Win32GraphicsDevice
 * Method:    getMaxConfigs
 * Signature: ()I
 */

JNIEXPORT jint JNICALL Java_sun_awt_Win32GraphicsDevice_getMaxConfigs
    (JNIEnv* jniEnv, jobject theThis, jint screen) {
	TRY;
    HDC hDC = AwtWin32GraphicsEnv::GetDCFromScreen(screen);

    PIXELFORMATDESCRIPTOR pfd;
    int max = ::DescribePixelFormat(hDC, 1, sizeof(PIXELFORMATDESCRIPTOR),
        &pfd);
    if (hDC != NULL) {
        ::ReleaseDC(NULL, hDC);
        hDC = NULL;
    }
    //If ::DescribePixelFormat() fails, max = 0
    //In this case, we return 1 config with visual number 0
    if (max == 0) {
        max = 1;
    }
    return (jint)max;
	CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_Win32GraphicsDevice
 * Method:    isPixFmtSupported
 * Signature: (I)Z
 */

JNIEXPORT jboolean JNICALL Java_sun_awt_Win32GraphicsDevice_isPixFmtSupported
    (JNIEnv* env, jobject theThis, jint pixFmtID, jint screen) {
	TRY;
    jboolean suppColor = JNI_TRUE;
    HDC hDC = AwtWin32GraphicsEnv::GetDCFromScreen(screen);
    
    PIXELFORMATDESCRIPTOR pfd;
    int max = ::DescribePixelFormat(hDC, (int)pixFmtID,
        sizeof(PIXELFORMATDESCRIPTOR), &pfd);
    DASSERT(max);

    //Check for supported ColorModel
    if ((pfd.cColorBits < 8) ||
       ((pfd.cColorBits == 8) && (pfd.iPixelType != PFD_TYPE_COLORINDEX))) {
        //Note: this still allows for PixelFormats with > 8 color bits
        //which use COLORINDEX instead of RGB.  This seems to work fine,
        //although issues may crop up involving PFD_NEED_PALETTE, which 
        //is not currently taken into account.
        //If changes are made, they should also be reflected in
        //getDefaultPixID.
        suppColor = JNI_FALSE;
    }

    if (hDC != NULL) {
        ::ReleaseDC(NULL, hDC);
        hDC = NULL;
    }
    return (((pfd.dwFlags & REQUIRED_FLAGS) == REQUIRED_FLAGS) && suppColor) ?
     JNI_TRUE : JNI_FALSE;
	CATCH_BAD_ALLOC_RET(FALSE);
}

/*
 * Class:     sun_awt_Win32GraphicsDevice
 * Method:    getDefaultPixID
 * Signature: ()I
 */

JNIEXPORT jint JNICALL Java_sun_awt_Win32GraphicsDevice_getDefaultPixID
    (JNIEnv* env, jobject theThis, jint screen) {
	TRY;
    int pixFmtID = 0;
    HDC hDC = AwtWin32GraphicsEnv::GetDCFromScreen(screen);
    char ebuf[8];

    // This is a workaround for Win2K problem where it 
    // doesn't return the right pixel fmt id on Dome-like 256 
    // gray boards which resulted in slow performance due to 
    // the wrong loop used.
    // We default to the old non-pixid based mechanism in this case.
    if ((AwtImage::m_clrdata.grayscale == GS_STATICGRAY) &&
	GetEnvironmentVariable("NOPIXID", ebuf, 0)) {
	if (hDC != NULL) {
	     ::ReleaseDC(NULL, hDC);
	     hDC = NULL;
	}
	return 0;
    }
       
    PIXELFORMATDESCRIPTOR pfd = {
        sizeof(PIXELFORMATDESCRIPTOR),
        1,               //version
        REQUIRED_FLAGS,  //flags
        0,               //iPixelType
        0,               //cColorBits
        0,0,0,0,0,0,0,0, //cRedBits, cRedShift, green, blue, alpha
        0,0,0,0,0,       //cAccumBits, cAccumRedBits, green, blue, alpha
        0,0,0,0,0,0,0,0  //etc.
    };
    
    //If 8-bit mode, must use Indexed mode
    if (8 == ::GetDeviceCaps(hDC, BITSPIXEL)) {
        pfd.iPixelType = PFD_TYPE_COLORINDEX;
    }

    pixFmtID = ::ChoosePixelFormat(hDC, &pfd);
    if (pixFmtID == 0) {
        //Return 0 if GDI call fails.
        if (hDC != NULL) {
            ::ReleaseDC(NULL, hDC);
            hDC = NULL;
        }
        return pixFmtID;
    }

    if (JNI_FALSE == Java_sun_awt_Win32GraphicsDevice_isPixFmtSupported(
     env, theThis, pixFmtID, screen)) {
        jclass excClass = env->FindClass("java/lang/RuntimeException");
        DASSERT(excClass);
        env->ThrowNew(excClass, "\nUnable to create a suitable default GraphicsConfiguration.  Try changing your Display Settings.");
        if (hDC != NULL) {
            ::ReleaseDC(NULL, hDC);
            hDC = NULL;
        }
        return 0;
    }

    ::ReleaseDC(NULL, hDC);
    hDC = NULL;
    return (jint)pixFmtID; 
	CATCH_BAD_ALLOC_RET(0);
}
  
/*
 * Class:     sun_awt_Win32GraphicsDevice
 * Method:    resetComponentGC
 * Signature: (Ljava/awt/Component;)V;
 */
JNIEXPORT void JNICALL
Java_sun_awt_Win32GraphicsDevice_resetComponentGC(
 JNIEnv* env, jobject theThis, jobject comp)
{
	TRY;
    jclass compCls;
    jmethodID resetGCID;

    compCls = env->GetObjectClass(comp);
    DASSERT(compCls);
    resetGCID = env->GetMethodID(compCls, "resetGC", "()V");
    DASSERT(resetGCID);

    env->CallVoidMethod(comp, resetGCID);
	CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_Win32GraphicsDevice
 * Method:    resetPeerGC
 * Signature: (Ljava/awt/ComponentPeer;)V;
 */
JNIEXPORT void JNICALL
Java_sun_awt_Win32GraphicsDevice_resetPeerGC(
 JNIEnv* env, jobject theThis, jobject peer)
{
	TRY;
    jclass peerCls;
    jmethodID resetGCID;

    peerCls = env->GetObjectClass(peer);
    DASSERT(peerCls);
    resetGCID = env->GetMethodID(peerCls, "resetLocalGC", "()V");
    DASSERT(resetGCID);

    env->CallVoidMethod(peer, resetGCID);
	CATCH_BAD_ALLOC;
}

