/*
 * @(#)awt_Win32GraphicsEnv.cpp	1.7 01/02/09
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include <windows.h>
#include <jni.h>
#include <awt.h>
#include <sun_awt_Win32GraphicsEnvironment.h>
#include "awt_Canvas.h"
#include "awt_image.h"
#include "awt_Toolkit.h"
#include "awt_Win32GraphicsEnv.h"

MHND *AwtWin32GraphicsEnv::monHds = NULL;
int AwtWin32GraphicsEnv::awt_numScreens = 1;
int AwtWin32GraphicsEnv::first256Screen = -1;
int AwtWin32GraphicsEnv::awt_primaryScreen = 0;

/*
 * Allocate and initialize the array of monitor handles.
 */
void AwtWin32GraphicsEnv::InitScreens(JNIEnv *env) {
    awt_numScreens = ::CountMonitors();
    AwtWin32GraphicsEnv::monHds = (MHND *)safe_Malloc(
                                                 awt_numScreens * sizeof(MHND));
    if (awt_numScreens != ::CollectMonitors(AwtWin32GraphicsEnv::monHds,
                                            awt_numScreens)) {
        JNU_ThrowInternalError(env, "Failed to get all monitor handles.");
    }

    MONITOR_INFO miInfo;
    memset((void*)(&miInfo), 0, sizeof(MONITOR_INFO));
    miInfo.dwSize = sizeof(MONITOR_INFO);
    for (int i = 0; i < awt_numScreens; i++) {
        ::GetMonitorInfo(monHds[i], &miInfo);
        if (miInfo.dwFlags & MONITOR_INFO_FLAG_PRIMARY) {
            awt_primaryScreen = i;
            break;
        }
    }
    first256Screen = FindFirst256Screen();
}

/*
 * Free the array of monitor handles
 */
void AwtWin32GraphicsEnv::DisposeScreens(JNIEnv *env) {
    awt_numScreens = 0;
    awt_primaryScreen = 0;
    free(AwtWin32GraphicsEnv::monHds);
}

/*
 * Called during GraphicsEnvironment initialization and after a display
 * change.  Initializes color parameters (AwtImage::InitializePalette),
 * and calls BuildPalette for the proper screen, if necessary.
 * Added for 4302507
 */
void AwtWin32GraphicsEnv::SetColorParams(JNIEnv *env) {
    HDC primDC = ::GetDC(NULL);
    AwtImage::InitializePalette(primDC);
    // This should be called in either case to initialize
    // m_inverseDesktopLUT
    AwtToolkit::GetInstance().adjustPalette(NULL);
    if (NeedToBuildPalette()) {
        // Don't need to check primary display becaust palette would
        // already be built for it if its in 256 color mode.
        DASSERT(GetNumScreens() > 1);
        int palettedScr = GetFirst256Screen();
        DASSERT(palettedScr > 0);
        HDC palettedDC = GetDCFromScreen(palettedScr);
        DASSERT(palettedDC);
        AwtImage::BuildPalettes(palettedDC);
        AwtToolkit::GetInstance().adjustPalette(palettedDC);
        ::DeleteDC(palettedDC);
    }
    else {
        AwtToolkit::GetInstance().adjustPalette(primDC);
    }
    ::ReleaseDC(NULL, primDC);
}
  
/*
 * Returns true if the color palette for indexed-drawing needs to be built.
 * Added for 4302507
 */
bool AwtWin32GraphicsEnv::NeedToBuildPalette(void) {
    //Only build the palette if it hasn't been initialized yet.
    //Otherwise, another thread using the palette could have parts of it
    //overwritten in the middle of drawing.

    if (g_colorInfo[COLOR8].m_pPaletteColors == 0 &&
        (GetFirst256Screen() > -1)) {
        return TRUE;
    }
    return FALSE;
}

/*
 * Added to AwtWin32GraphicsEnv for 4302507
 */
HDC AwtWin32GraphicsEnv::GetDCFromScreen(int screen) {
    DASSERT(screen < GetNumScreens());
    return MakeDCFromMonitor(AwtWin32GraphicsEnv::monHds[screen]);
}

/*
 * Added to AwtWin32GraphicsEnv for 4302507
 */
MHND AwtWin32GraphicsEnv::GetMHNDFromScreen(int screen) {
    DASSERT(screen < GetNumScreens());
    return AwtWin32GraphicsEnv::monHds[screen];
}

/*
 * Returns the index (0-based) of the first screen requiring a palette, or -1
 * if no such screen exists.
 * This screen should be used to build the color palettes.
 * Called upon screen (re)initialization.
 * Added for 4302507
 */
int AwtWin32GraphicsEnv::FindFirst256Screen() {
    int numScreens = GetNumScreens();
    HDC tempDC = NULL;
    int tempBPP;
    for (int loop = 0; loop < numScreens; loop++) {
        tempDC = GetDCFromScreen(loop);
        DASSERT(tempDC);
        tempBPP = ::GetDeviceCaps(tempDC, BITSPIXEL);
        ::DeleteDC(tempDC);
        if (tempBPP <= 8) {
            return loop;
        }
    }
    // Didn't find any 256-color screens
    return -1;
}

/*
 * Class:     sun_awt_Win32GraphicsEnvironment
 * Method:    initDisplay
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_Win32GraphicsEnvironment_initDisplay(JNIEnv *env,
                                                  jclass thisClass)
{
    AwtWin32GraphicsEnv::InitScreens(env);
    AwtWin32GraphicsEnv::SetColorParams(env);
}

/*
 * Class:     sun_awt_Win32GraphicsEnvironment
 * Method:    getNumScreens
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_Win32GraphicsEnvironment_getNumScreens(JNIEnv *env,
                                                    jobject thisobj)
{
    return AwtWin32GraphicsEnv::GetNumScreens();
}
