/*
 * @(#)awt_Window.cpp	1.81 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt_Component.h"
#include "awt_Container.h"
#include "awt_Frame.h"
#include "awt_Graphics.h"
#include "awt_Insets.h"
#include "awt_Panel.h"
#include "awt_Toolkit.h"
#include "awt_Window.h"

#include "java_awt_Insets.h"
#include <java_awt_Container.h>
#include <java_awt_event_ComponentEvent.h>

/* IMPORTANT! Read the README.JNI file for notes on JNI converted AWT code.
 */

/************************************************************************
 * AwtWindow fields
 */

jfieldID AwtWindow::warningStringID;

/************************************************************************
 * AwtWindow class methods
 */

AwtWindow::AwtWindow() {
    m_iconic = FALSE;
    m_eraseBackground = FALSE;
    m_resizing = FALSE;
    m_sizePt.x = m_sizePt.y = 0;
    VERIFY(::SetRectEmpty(&m_insets));
    VERIFY(::SetRectEmpty(&m_old_insets));
    VERIFY(::SetRectEmpty(&m_warningRect));
}

AwtWindow::~AwtWindow() {
}

const char* AwtWindow::GetClassName() {
  return "SunAwtWindow";
}

void AwtWindow::FillClassInfo(WNDCLASS *lpwc)
{
    AwtComponent::FillClassInfo(lpwc);
    /*
     * This line causes bug #4189244 (Swing Popup menu is not being refreshed (cleared) under a Dialog)
     * so it's comment out (son@sparc.spb.su)
     *
     * lpwc->style     |= CS_SAVEBITS; // improve pull-down menu performance
     */
    lpwc->cbWndExtra = DLGWINDOWEXTRA;
}


/* Create a new AwtWindow object and window.   */
AwtWindow* AwtWindow::Create(jobject self, jobject parent)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    jobject target = NULL;
    AwtWindow* window = NULL;

    try {
        if (env->EnsureLocalCapacity(1) < 0) {
	    return NULL;
	}

	jint pData;
	AwtWindow* awtParent;
	JNI_CHECK_PEER_GOTO(parent, done);

	awtParent = (AwtWindow*)pData;
	JNI_CHECK_NULL_GOTO(awtParent, "null awtParent", done);

	target = env->GetObjectField(self, AwtObject::targetID);
	JNI_CHECK_NULL_GOTO(target, "null target", done);

	window = new AwtWindow();

	{
	    DWORD style = WS_CLIPCHILDREN | WS_POPUP;
	    DWORD exStyle = 0;
	    if (GetRTL()) {
	        exStyle |= WS_EX_RIGHT | WS_EX_LEFTSCROLLBAR;
		if (GetRTLReadingOrder())
		    exStyle |= WS_EX_RTLREADING;
	    }
	    window->CreateHWnd(env, "",
			       style, exStyle,
			       0, 0, 0, 0,
			       awtParent->GetHWnd(),
			       NULL,
			       ::GetSysColor(COLOR_WINDOWTEXT),
			       ::GetSysColor(COLOR_WINDOW),
			       self);

	    jint x = env->GetIntField(target, AwtComponent::xID);
	    jint y = env->GetIntField(target, AwtComponent::yID);
	    jint width = env->GetIntField(target, AwtComponent::widthID);
	    jint height = env->GetIntField(target, AwtComponent::heightID);

	    /* 
	     * Reshape here instead of during create, so that a WM_NCCALCSIZE
	     * is sent. 
	     */
	    window->Reshape(x, y, width, height);
	}
    } catch (...) {
        env->DeleteLocalRef(target);
	throw;
    }

done:
    env->DeleteLocalRef(target);
    return window;
}

//
// Override AwtComponent's Reshape to handle minimized/maximized
// windows, fixes 4065534 (robi.khan@eng).
// NOTE: See also AwtWindow::WmSize
//
void AwtWindow::Reshape(int x, int y, int width, int height) 
{
    HWND    hWndSelf = GetHWnd();

    if ( IsIconic(hWndSelf)) {
    // normal AwtComponent::Reshape will not work for iconified windows so...
	WINDOWPLACEMENT wp;
	POINT	    ptMinPosition = {x,y};
	POINT	    ptMaxPosition = {0,0};
	RECT	    rcNormalPosition = {x,y,x+width,y+height};
	RECT	    rcWorkspace;
	HWND	    hWndDesktop = GetDesktopWindow();

	// SetWindowPlacement takes workspace coordinates, but
	// if taskbar is at top of screen, workspace coords !=
	// screen coords, so offset by workspace origin
	VERIFY(::SystemParametersInfo(SPI_GETWORKAREA, 0, (PVOID)&rcWorkspace, 0));
	::OffsetRect(&rcNormalPosition, -rcWorkspace.left, -rcWorkspace.top);

	// set the window size for when it is not-iconified
	wp.length = sizeof(wp);
	wp.flags = WPF_SETMINPOSITION;
	wp.showCmd = SW_SHOWNA;
	wp.ptMinPosition = ptMinPosition;
	wp.ptMaxPosition = ptMaxPosition;
	wp.rcNormalPosition = rcNormalPosition;
	SetWindowPlacement(hWndSelf, &wp);
	return;
    }
    
    if (IsZoomed(hWndSelf)) {
    // setting size of maximized window, we remove the
    // maximized state bit (matches Motif behaviour)
    // (calling ShowWindow(SW_RESTORE) would fire an
    //  activation event which we don't want)
	LONG	style = GetStyle();
	DASSERT(style & WS_MAXIMIZE);
	style ^= WS_MAXIMIZE;
	SetStyle(style);
    }

    AwtComponent::Reshape(x, y, width, height);
}

/*
 * Get and return the insets for this window (container, really).
 * Calculate & cache them while we're at it, for use by AwtGraphics
 */
BOOL AwtWindow::UpdateInsets(jobject insets)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    DASSERT(GetPeer(env) != NULL);
    if (env->EnsureLocalCapacity(2) < 0) {
	return FALSE;
    }

    // fix 4167248 : don't update insets when frame is iconified
    // to avoid bizarre window/client rectangles
    if (m_iconic)
	return FALSE;

    /*
     * Code to calculate insets. Stores results in frame's data
     * members, and in the peer's Inset object.
     */
    RECT outside;
    RECT inside;
    int extraBottomInsets = 0;

    ::GetWindowRect(GetHWnd(), &outside);

    jobject target = GetTarget(env);
    jstring warningString = 
      (jstring)(env)->GetObjectField(target, AwtWindow::warningStringID);
    if (warningString != NULL) {
        ::CopyRect(&inside, &outside);
	DefWindowProc(WM_NCCALCSIZE, FALSE, (LPARAM)&inside);
        /*
         * Fix for BugTraq ID 4304024.
         * Calculate client rectangle in client coordinates. 
         */
	VERIFY(::OffsetRect(&inside, -inside.left, -inside.top));
	extraBottomInsets = ::GetSystemMetrics(SM_CYCAPTION) +
	    ((GetStyle() & WS_THICKFRAME) ? 2 : -2);
    } else {
        ::GetClientRect(GetHWnd(), &inside);
    }
    env->DeleteLocalRef(target);
    env->DeleteLocalRef(warningString);

    /* Update our inset member */
    if (outside.right - outside.left > 0 && outside.bottom - outside.top > 0) {
        ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&inside, 2);
        m_insets.top = inside.top - outside.top;
        m_insets.bottom = outside.bottom - inside.bottom + extraBottomInsets;
        m_insets.left = inside.left - outside.left;
        m_insets.right = outside.right - inside.right;
    } else {
	m_insets.top = -1;
    }
    if (m_insets.left < 0 || m_insets.top < 0 ||
	m_insets.right < 0 || m_insets.bottom < 0)
    {
	/* This window hasn't been sized yet -- use system metrics. */
        jobject target = GetTarget(env);
        if (JNU_IsInstanceOfByName(env, target, "java/awt/Frame") > 0 ||
            JNU_IsInstanceOfByName(env, target, "java/awt/Dialog") > 0) {
            /* Get outer frame sizes. */
            long style = GetStyle();
            if (style & WS_THICKFRAME) {
                m_insets.left = m_insets.right = 
                    ::GetSystemMetrics(SM_CXSIZEFRAME);
                m_insets.top = m_insets.bottom = 
                    ::GetSystemMetrics(SM_CYSIZEFRAME);
            } else {
                m_insets.left = m_insets.right = 
                    ::GetSystemMetrics(SM_CXDLGFRAME);
                m_insets.top = m_insets.bottom = 
                    ::GetSystemMetrics(SM_CYDLGFRAME);
            }

            /* Add in title. */
            m_insets.top += ::GetSystemMetrics(SM_CYCAPTION);

            /* Add in menuBar, if any. */
            if (JNU_IsInstanceOfByName(env, target, "java/awt/Frame") > 0 &&
                ((AwtFrame*)this)->GetMenuBar()) {
                m_insets.top += ::GetSystemMetrics(SM_CYMENU);
            }
        }
	m_insets.bottom += extraBottomInsets;
	env->DeleteLocalRef(target);
    }

    jobject peer = GetPeer(env);
    /* Get insets into our peer directly */
    jobject peerInsets = (env)->GetObjectField(peer, AwtPanel::insets_ID);
    DASSERT(!safe_ExceptionOccurred(env));
    if (peerInsets != NULL) { // may have been called during creation
	(env)->SetIntField(peerInsets, AwtInsets::topID, m_insets.top);
	(env)->SetIntField(peerInsets, AwtInsets::bottomID, 
			   m_insets.bottom);
	(env)->SetIntField(peerInsets, AwtInsets::leftID, m_insets.left);
	(env)->SetIntField(peerInsets, AwtInsets::rightID, m_insets.right);
    }
    /* Get insets into the Inset object (if any) that was passed */
    if (insets != NULL) {
	(env)->SetIntField(insets, AwtInsets::topID, m_insets.top);
	(env)->SetIntField(insets, AwtInsets::bottomID, m_insets.bottom);
	(env)->SetIntField(insets, AwtInsets::leftID, m_insets.left);
	(env)->SetIntField(insets, AwtInsets::rightID, m_insets.right);
    }
    env->DeleteLocalRef(peerInsets);

    BOOL	insetsChanged = !::EqualRect( &m_old_insets, &m_insets );	
    ::CopyRect( &m_old_insets, &m_insets );
    return insetsChanged;
}


/*
 * Although this function sends ComponentEvents, it needs to be defined
 * here because only top-level windows need to have move and resize
 * events fired from native code.  All contained windows have these events
 * fired from common Java code.
 */
void AwtWindow::SendComponentEvent(long eventId)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    static jclass classEvent = NULL;
    if (classEvent == NULL) {
        if (env->PushLocalFrame(1) < 0)
	    return;
	classEvent = env->FindClass("java/awt/event/ComponentEvent");
	if (classEvent != NULL) {
	    classEvent = (jclass)env->NewGlobalRef(classEvent);
	}
	env->PopLocalFrame(0);
    }
    static jmethodID eventInitMID = NULL;
    if (eventInitMID == NULL) {
	eventInitMID = env->GetMethodID(classEvent, "<init>",
					"(Ljava/awt/Component;I)V");
	if (eventInitMID == NULL) {
	    return;
	}
    }
    if (env->EnsureLocalCapacity(2) < 0) {
	return;
    }
    jobject target = GetTarget(env);
    jobject event = env->NewObject(classEvent, eventInitMID, 
				   target, eventId);
    DASSERT(!safe_ExceptionOccurred(env));
    DASSERT(event != NULL);
    SendEvent(event);

    env->DeleteLocalRef(target);
    env->DeleteLocalRef(event);
}

void AwtWindow::SendWindowEvent(long eventId)
{
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);

    static jclass wClassEvent = NULL;

    if (wClassEvent == NULL) {
        if (env->PushLocalFrame(1) < 0)
	    return;
	wClassEvent = env->FindClass("java/awt/event/WindowEvent");
	if (wClassEvent != NULL) {
	    wClassEvent = (jclass)env->NewGlobalRef(wClassEvent);
	}
	env->PopLocalFrame(0);
        if (wClassEvent == NULL) {
            return;
        }
    }
    static jmethodID wEventInitMID = NULL;
    if (wEventInitMID == NULL) {
	wEventInitMID = env->GetMethodID(wClassEvent, "<init>",
					 "(Ljava/awt/Window;I)V");
        DASSERT(wEventInitMID);
	if (wEventInitMID == NULL) {
	    return;
	}
    }
    if (env->EnsureLocalCapacity(2) < 0)
        return;
    jobject target = GetTarget(env);
    jobject event = env->NewObject(wClassEvent, wEventInitMID, 
				   target, eventId);
    DASSERT(!safe_ExceptionOccurred(env));
    DASSERT(event != NULL);
    SendEvent(event);

    env->DeleteLocalRef(target);
    env->DeleteLocalRef(event);
}

MsgRouting AwtWindow::WmCreate()
{
    return mrDoDefault;  
}

MsgRouting AwtWindow::WmClose()
{
    SendWindowEvent(java_awt_event_WindowEvent_WINDOW_CLOSING);

    /* Rely on above notification to handle quitting as needed */
    return mrConsume;  
}

MsgRouting AwtWindow::WmDestroy()
{
    SendWindowEvent(java_awt_event_WindowEvent_WINDOW_CLOSED);
    return AwtComponent::WmDestroy();
}

/*
 * Override AwtComponent's move handling to first update the
 * java AWT target's position fields directly, since Windows
 * and below can be resized from outside of java (by user)
 */
MsgRouting AwtWindow::WmMove(int x, int y)
{
    if ( IsIconic(GetHWnd()) ) {
    // fixes 4065534 (robi.khan@eng)
    // if a window is iconified we don't want to update
    // it's target's position since minimized Win32 windows
    // move to -32000, -32000 for whatever reason
    // NOTE: See also AwtWindow::Reshape
	return mrDoDefault;
    }

    /* Update the java AWT target component's fields directly */
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(1) < 0) {
	return mrConsume;
    }
    jobject peer = GetPeer(env);
    jobject target = env->GetObjectField(peer, AwtObject::targetID);

    RECT rect;
    ::GetWindowRect(GetHWnd(), &rect);

    jint targX = (env)->GetIntField(target, AwtComponent::xID);
    jint targY = (env)->GetIntField(target, AwtComponent::yID);
    if (targX != rect.left || targY != rect.top) {
	(env)->SetIntField(target, AwtComponent::xID, rect.left);
	(env)->SetIntField(target, AwtComponent::yID, rect.top);
        SendComponentEvent(java_awt_event_ComponentEvent_COMPONENT_MOVED);
    }

    env->DeleteLocalRef(target);
    return AwtComponent::WmMove(x, y);
}

/*
 * Override AwtComponent's size handling to first update the 
 * java AWT target's dimension fields directly, since Windows
 * and below can be resized from outside of java (by user)
 */
MsgRouting AwtWindow::WmSize(int type, int w, int h)
{
    if (type == SIZE_MINIMIZED) {
        SendWindowEvent(java_awt_event_WindowEvent_WINDOW_ICONIFIED);
        m_iconic = TRUE;
	return mrDoDefault;
    }
    else if ((type == SIZE_RESTORED || type == SIZE_MAXIMIZED) && m_iconic) {
        SendWindowEvent(java_awt_event_WindowEvent_WINDOW_DEICONIFIED);
        m_iconic = FALSE;
    } 
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(1) < 0)
        return mrDoDefault;
    jobject target = GetTarget(env);
    // fix 4167248 : ensure the insets are up-to-date before using
    BOOL insetsChanged = UpdateInsets(NULL);
    jint width = (env)->GetIntField(target, AwtComponent::widthID);
    jint height = (env)->GetIntField(target, AwtComponent::heightID);
    int newWidth = w + m_insets.left + m_insets.right;
    int newHeight = h + m_insets.top + m_insets.bottom;

    if (width != newWidth || height != newHeight || insetsChanged) {
        (env)->SetIntField(target, AwtComponent::widthID, newWidth);
	(env)->SetIntField(target, AwtComponent::heightID, newHeight);
	if (!m_resizing) {
	    SendComponentEvent(java_awt_event_ComponentEvent_COMPONENT_RESIZED);
	}
    }
    env->DeleteLocalRef(target);
    return AwtComponent::WmSize(type, w, h);
}

MsgRouting AwtWindow::WmPaint(HDC)
{
    HRGN rgn = ::CreateRectRgn(0,0,1,1);
    int updated = ::GetUpdateRgn(GetHWnd(), rgn, FALSE);

    if (updated == COMPLEXREGION || updated == SIMPLEREGION) {
        /*
         * Now remove all update regions from this window -- do it
         * here instead of after the Java upcall, in case any new
         * updating is requested.  
         */
        ::ValidateRgn(GetHWnd(), NULL);

        if (m_peerObject != NULL && m_callbacksEnabled) {
            ::OffsetRgn(rgn, m_insets.left, m_insets.top);

            int size = ::GetRegionData(rgn, 0, NULL);
            char* buffer = new char[size];
            memset(buffer, 0, size);
            LPRGNDATA rgndata = (LPRGNDATA)buffer;
            rgndata->rdh.dwSize = sizeof(RGNDATAHEADER);
            rgndata->rdh.iType = RDH_RECTANGLES;
            VERIFY(::GetRegionData(rgn, size, rgndata));

            RECT* r = (RECT*)(buffer + rgndata->rdh.dwSize);
            for (DWORD i=0; i<rgndata->rdh.nCount; i++) {
                if ((r->right-r->left) > 0 && (r->bottom-r->top) > 0) {
                    DoCallback("handleExpose", "(IIII)V", r->left, r->top,
                               r->right-r->left, r->bottom-r->top);
                }
                r++;
            }

            delete [] buffer;
        }
    } else if (updated == NULLREGION) {
        ::ValidateRgn(GetHWnd(), NULL);
    }

    ::DeleteObject((HGDIOBJ)rgn);
    m_eraseBackground = FALSE;
    return mrConsume;
}

MsgRouting AwtWindow::WmSysCommand(UINT uCmdType, UINT xPos, UINT yPos)
{
    if (uCmdType == SC_SIZE) {
        m_resizing = TRUE;
    }
    return mrDoDefault;
}

MsgRouting AwtWindow::WmExitSizeMove()
{
    if (m_resizing) {
        SendComponentEvent(java_awt_event_ComponentEvent_COMPONENT_RESIZED);
        m_resizing = FALSE;
    }
    return mrDoDefault;
}

MsgRouting AwtWindow::WmSettingChange(WORD wFlag, LPCTSTR pszSection)
{
    if (wFlag == SPI_SETNONCLIENTMETRICS) {
    // user changed window metrics in
    // Control Panel->Display->Appearance
    // which may cause window insets to change
	UpdateInsets(NULL);
	return mrConsume;
    }
    return mrDoDefault;
}

MsgRouting AwtWindow::WmNcCalcSize(BOOL fCalcValidRects, 
                                   LPNCCALCSIZE_PARAMS lpncsp, UINT& retVal)
{
    MsgRouting mrRetVal = mrDoDefault;

    if (fCalcValidRects == FALSE) {
        return mrDoDefault;
    }
    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(2) < 0) {
	return mrConsume;
    }
    jobject target = GetTarget(env);
    jstring warningString = 
      (jstring)(env)->GetObjectField(target, AwtWindow::warningStringID);
    if (warningString != NULL) {
	RECT r;
	::CopyRect(&r, &lpncsp->rgrc[0]);
	retVal = DefWindowProc(WM_NCCALCSIZE, fCalcValidRects, (LPARAM)lpncsp);

	/* Adjust non-client area for warning banner space. */
	m_warningRect.left = lpncsp->rgrc[0].left;
	m_warningRect.right = lpncsp->rgrc[0].right;
	m_warningRect.bottom = lpncsp->rgrc[0].bottom;
	m_warningRect.top = 
            m_warningRect.bottom - ::GetSystemMetrics(SM_CYCAPTION);
        if (GetStyle() & WS_THICKFRAME) {
	    m_warningRect.top -= 2;
        } else {
	    m_warningRect.top += 2;
        }

        lpncsp->rgrc[0].bottom = (m_warningRect.top >= lpncsp->rgrc[0].top)
            ? m_warningRect.top
            : lpncsp->rgrc[0].top;

	/* Convert to window-relative coordinates. */
	::OffsetRect(&m_warningRect, -r.left, -r.top);

        /* Notify target of Insets change. */
        UpdateInsets(NULL);

	mrRetVal = mrConsume;
    } else {
	// WM_NCCALCSIZE is usually in response to a resize, but
	// also can be triggered by SetWindowPos(SWP_FRAMECHANGED),
	// which means the insets will have changed - rnk 4/7/1998
	retVal = DefWindowProc(WM_NCCALCSIZE, fCalcValidRects, (LPARAM)lpncsp);
	UpdateInsets(NULL);
	mrRetVal = mrConsume;
    }
    env->DeleteLocalRef(target);
    env->DeleteLocalRef(warningString);
    return mrRetVal;
}

MsgRouting AwtWindow::WmNcPaint(HRGN hrgn)
{
    DefWindowProc(WM_NCPAINT, (WPARAM)hrgn, 0);

    JNIEnv *env = (JNIEnv *)JNU_GetEnv(jvm, JNI_VERSION_1_2);
    if (env->EnsureLocalCapacity(2) < 0) {
	return mrConsume;
    }
    jobject target = GetTarget(env);
    jstring warningString =
      (jstring)(env)->GetObjectField(target, AwtWindow::warningStringID);
    if (warningString != NULL) {
	RECT r;
        ::CopyRect(&r, &m_warningRect);
	HDC hDC = ::GetWindowDC(GetHWnd());
	DASSERT(hDC);
	int iSaveDC = ::SaveDC(hDC);
	VERIFY(::SelectClipRgn(hDC, NULL) != NULL);
	VERIFY(::FillRect(hDC, &m_warningRect, (HBRUSH)::GetStockObject(BLACK_BRUSH)));

        if (GetStyle() & WS_THICKFRAME) {
            /* draw edge */
            VERIFY(::DrawEdge(hDC, &r, EDGE_RAISED, BF_TOP));
            r.top += 2;
            VERIFY(::DrawEdge(hDC, &r, EDGE_SUNKEN, BF_RECT));
            ::InflateRect(&r, -2, -2);
        }

	/* draw warning text */
	LPWSTR text = TO_WSTRING(warningString);
	VERIFY(::SetBkColor(hDC, ::GetSysColor(COLOR_BTNFACE)) != CLR_INVALID);
	VERIFY(::SetTextColor(hDC, ::GetSysColor(COLOR_BTNTEXT)) != CLR_INVALID);
	VERIFY(::SelectObject(hDC, ::GetStockObject(DEFAULT_GUI_FONT)) != NULL);
	VERIFY(::SetTextAlign(hDC, TA_LEFT | TA_BOTTOM) != GDI_ERROR);
	VERIFY(::ExtTextOutW(hDC, r.left+2, r.bottom-1,
			     ETO_CLIPPED | ETO_OPAQUE,
			     &r, text, wcslen(text), NULL));
	VERIFY(::RestoreDC(hDC, iSaveDC));
	::ReleaseDC(GetHWnd(), hDC);
    }

    env->DeleteLocalRef(target);
    env->DeleteLocalRef(warningString);
    return mrConsume;
}

/*
MsgRouting AwtWindow::WmNcHitTest(UINT x, UINT y, UINT& retVal)
{
    POINT pt = { x, y };
    ::ScreenToClient(GetHWnd(), &pt);
    if (::PtInRect(&m_warningRect, pt)) {
        retVal = HTNOWHERE;
    } else {
        retVal = DefWindowProc(WM_NCHITTEST, 0, MAKELPARAM(x, y));
    }
    return mrConsume;
}
*/

/*
 * Fix for BugTraq ID 4041703: keyDown not being invoked.
 * This method overrides AwtCanvas::HandleEvent() since 
 * an empty Window always receives the focus on the activation
 * so we don't have to modify the behavior.
 */
MsgRouting AwtWindow::HandleEvent(MSG *msg, BOOL synthetic)
{
    return AwtComponent::HandleEvent(msg, synthetic);
}

BOOL CALLBACK InvalidateChildRect(HWND hWnd, LPARAM)
{
    TRY;

    ::InvalidateRect(hWnd, NULL, TRUE);
    return TRUE;

    CATCH_BAD_ALLOC_RET(FALSE);
}

void AwtWindow::Invalidate(RECT* r)
{
    ::InvalidateRect(GetHWnd(), NULL, TRUE);
    ::EnumChildWindows(GetHWnd(), (WNDENUMPROC)InvalidateChildRect, 0);
}

void AwtWindow::SetResizable(BOOL isResizable)
{
    if (IsEmbedded()) {
	return;
    }
    long style = GetStyle();
    static const long SizeStyles = WS_THICKFRAME|WS_MAXIMIZEBOX;

    if (isResizable) {
	style |= SizeStyles;
    } else {
	style &= ~SizeStyles;
    }
    SetStyle(style);
    RedrawNonClient();
}

// SetWindowPos flags to cause frame edge to be recalculated
static const UINT SwpFrameChangeFlags =
    SWP_FRAMECHANGED | /* causes WM_NCCALCSIZE to be called */
    SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER |
    SWP_NOACTIVATE | SWP_NOCOPYBITS | 
    SWP_NOREPOSITION | SWP_NOSENDCHANGING;

//
// Forces WM_NCCALCSIZE to be called to recalculate
// window border (updates insets) without redrawing it
//
void AwtWindow::RecalcNonClient()
{
    ::SetWindowPos(GetHWnd(), (HWND) NULL, 0, 0, 0, 0, SwpFrameChangeFlags|SWP_NOREDRAW);
}

//
// Forces WM_NCCALCSIZE to be called to recalculate
// window border (updates insets) and redraws border to match
//
void AwtWindow::RedrawNonClient()
{
    ::SetWindowPos(GetHWnd(), (HWND) NULL, 0, 0, 0, 0, SwpFrameChangeFlags|SWP_ASYNCWINDOWPOS);
}

extern "C" {

/*
 * Class:     java_awt_Window
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_java_awt_Window_initIDs(JNIEnv *env, jclass cls)
{
    TRY;

    AwtWindow::warningStringID =
        env->GetFieldID(cls, "warningString", "Ljava/lang/String;");

    CATCH_BAD_ALLOC;
}

} /* extern "C" */


/************************************************************************
 * WindowPeer native methods
 */

extern "C" {

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    toFront
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer_toFront(JNIEnv *env, jobject self)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    AwtWindow* w = (AwtWindow*)pData;

    ::SetWindowPos(w->GetHWnd(), HWND_TOP, 0,0,0,0,
                   SWP_NOMOVE|SWP_NOSIZE);
    ::SetForegroundWindow(w->GetHWnd());

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    toBack
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer_toBack(JNIEnv *env, jobject self)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    AwtWindow* w = (AwtWindow*)pData;

    ::SetWindowPos(w->GetHWnd(), HWND_BOTTOM, 0,0,0,0,
                   SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    _setTitle
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer__1setTitle(JNIEnv *env, jobject self,
					    jstring title)
{
    TRY;

    int pData;
    JNI_CHECK_PEER_RETURN(self);
    JNI_CHECK_NULL_RETURN(title, "null title");

    AwtWindow* w = (AwtWindow*)pData;

    if (IS_NT) {
	int length = env->GetStringLength(title);
	WCHAR *buffer = new WCHAR[length + 1];
	env->GetStringRegion(title, 0, length, buffer);
	buffer[length] = L'\0';
	VERIFY(::SetWindowTextW(w->GetHWnd(), buffer));
	delete[] buffer;
    } else {
	int length = env->GetStringLength(title);
	char *buffer = new char[length * 2 + 1];
	AwtFont::javaString2multibyte(env, title, buffer, length);	
	VERIFY(::SetWindowText(w->GetHWnd(), buffer));
	delete[] buffer;
    }

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    _setResizable
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer__1setResizable(JNIEnv *env, jobject self,
						jboolean resizable)
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN(self);

    AwtWindow* w = (AwtWindow*)pData;
    DASSERT(!IsBadReadPtr(w, sizeof(AwtWindow)));
    w->SetResizable(resizable != 0);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    create
 * Signature: (Lsun/awt/windows/WComponentPeer;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer_createAwtWindow(JNIEnv *env, jobject self, 
                                                 jobject parent)
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN(parent);
    AwtToolkit::CreateComponent(self, parent,
				(AwtToolkit::ComponentFactory)
				AwtWindow::Create);
    JNI_CHECK_PEER_CREATION_RETURN(self);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    updateInsets
 * Signature: (Ljava/awt/Insets;)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer_updateInsets(JNIEnv *env, jobject self,
					      jobject insets)
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN(self);
    JNI_CHECK_NULL_RETURN(insets, "null insets");

    AwtWindow* w = (AwtWindow*)pData;
    w->UpdateInsets(insets);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    getContainerElement
 * Signature: (Ljava/awt/Container;I)Ljava/awt/Component;
 */
JNIEXPORT jobject JNICALL
Java_sun_awt_windows_WWindowPeer_getContainerElement(JNIEnv *env, jobject self,
						     jobject container, 
						     jint index)
{
    TRY;

    static jfieldID Container_ncomponents_FID = 0;
    static jfieldID Container_component_FID = 0;

    JNI_CHECK_NULL_RETURN_NULL(container, "null container");
 
    if ((env)->GetIntField(container, AwtContainer::ncomponentsID) > 0) {
	jobjectArray comps = 
	    (jobjectArray)(env)->GetObjectField(container, 
						AwtContainer::componentID);
        return env->GetObjectArrayElement(comps, index);
    }
    return NULL;

    CATCH_BAD_ALLOC_RET(NULL);
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    reshapeFrame
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL
Java_sun_awt_windows_WWindowPeer_reshapeFrame(JNIEnv *env, jobject self,
					jint x, jint y, jint w, jint h) 
{
    TRY;

    jint pData;
    JNI_CHECK_PEER_RETURN(self);

    if (env->EnsureLocalCapacity(1) < 0) {
	return;
    }

    AwtFrame* p = (AwtFrame *)pData;

    jobject target = env->GetObjectField(self, AwtObject::targetID);
    JNI_CHECK_NULL_RETURN(target, "null target");

    // enforce tresholds before sending the event
    int minWidth = ::GetSystemMetrics(SM_CXMIN);
    int minHeight = ::GetSystemMetrics(SM_CYMIN);
    if (w < minWidth)
	env->SetIntField(target, AwtComponent::widthID,
			 w = minWidth);
    if (h < minHeight)
	env->SetIntField(target, AwtComponent::heightID,
			 h = minHeight);

    env->DeleteLocalRef(target);

    RECT *r = new RECT;
    ::SetRect(r, x, y, x + w, y + h);
    p->SendMessage(WM_AWT_RESHAPE_COMPONENT, 0, (LPARAM)r);

    CATCH_BAD_ALLOC;
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    getSysMinWidth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WWindowPeer_getSysMinWidth(JNIEnv *env, jclass self)
{
    TRY;

    return ::GetSystemMetrics(SM_CXMIN);

    CATCH_BAD_ALLOC_RET(0);
}

/*
 * Class:     sun_awt_windows_WWindowPeer
 * Method:    getSysMinHeight
 * Signature: ()I
 */
JNIEXPORT jint JNICALL
Java_sun_awt_windows_WWindowPeer_getSysMinHeight(JNIEnv *env, jclass self)
{
    TRY;

    return ::GetSystemMetrics(SM_CYMIN);

    CATCH_BAD_ALLOC_RET(0);
}

} /* extern "C" */
