/*
 * @(#)awt_Window.h	1.36 00/02/02
 *
 * Copyright 1996-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#ifndef AWT_WINDOW_H
#define AWT_WINDOW_H

#include "awt_Canvas.h"

#include "java_awt_Window.h"
#include "sun_awt_windows_WWindowPeer.h"


/************************************************************************
 * AwtWindow class
 */

class AwtWindow : public AwtCanvas {
public:

    /* java.awt.Window field ids */
    static jfieldID warningStringID;
    static jfieldID screenID; /* screen number passed over from WindowPeer */
    
    AwtWindow();
    virtual ~AwtWindow();

    virtual const char* GetClassName();
    void FillClassInfo(WNDCLASS *lpwc);

    static AwtWindow* Create(jobject self, jobject parent);

    /* Update the insets for this Window (container), its peer &
     * optional other
     */
    BOOL UpdateInsets(jobject insets = 0);

    /* Subtract inset values from a window origin. (2 versions) */
    INLINE void SubtractInsetPoint(long& x, long& y) {
	x -= m_insets.left;
	y -= m_insets.top;
    }
    INLINE void SubtractInsetPoint(int& x, int& y) {
	x -= m_insets.left;
	y -= m_insets.top;
    }

    /* Returns whether this window is in iconified state. */
    INLINE BOOL isIconic() { return m_iconic; }
    INLINE void setIconic(BOOL b) { m_iconic = b; }

    virtual void GetInsets(RECT* rect) {
        VERIFY(::CopyRect(rect, &m_insets));
    }

    /* to make embedded frames easier */
    virtual BOOL IsEmbedded() { return FALSE;}

    /* We can hold children */
    virtual BOOL IsContainer() { return TRUE;}

    virtual void Invalidate(RECT* r);
    virtual void Reshape(int x, int y, int width, int height);
    virtual void SetResizable(BOOL isResizable);
    virtual void RecalcNonClient();
    virtual void RedrawNonClient();

    /* Post events to the EventQueue */
    void SendComponentEvent(long eventId);
    void SendWindowEvent(long eventId);

    /*
     * Windows message handler functions
     */
    virtual MsgRouting WmCreate();
    virtual MsgRouting WmClose();
    virtual MsgRouting WmDestroy();
    virtual MsgRouting WmMove(int x, int y);
    virtual MsgRouting WmSize(int type, int w, int h);
    virtual MsgRouting WmPaint(HDC hDC);
    virtual MsgRouting WmSysCommand(UINT uCmdType, UINT xPos, UINT yPos);
    virtual MsgRouting WmExitSizeMove();
    virtual MsgRouting WmSettingChange(WORD wFlag, LPCTSTR pszSection);
    virtual MsgRouting WmNcCalcSize(BOOL fCalcValidRects,
				    LPNCCALCSIZE_PARAMS lpncsp, UINT& retVal);
    virtual MsgRouting WmNcPaint(HRGN hrgn);
    // virtual MsgRouting WmNcHitTest(UINT x, UINT y, UINT& retVal);

    virtual MsgRouting HandleEvent(MSG *msg, BOOL synthetic);

private:
    BOOL m_iconic;          /* are we in an iconic state (resize tracking) */
    RECT m_insets;          /* a cache of the insets being used */
    RECT m_old_insets;      /* help determine if insets change */
    BOOL m_resizing;        /* in the middle of a resizing operation */
    POINT m_sizePt;         /* the last value of WM_SIZE */
    RECT m_warningRect;     /* The window's warning banner area, if any. */
};

#endif /* AWT_WINDOW_H */
