/*
 * @(#)awt_dlls.cpp	1.12 01/02/09
 *
 * Copyright 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

#include "awt.h"
#include "awt_dlls.h"

/*
 * To reduce memory footprint we don't statically link to COMDLG32.DLL
 * and SHELL32.  Instead we programatically load them only when they are
 * needed.
 */

PrintDlgType AwtCommDialog::do_print_dlg;
PageSetupDlgType AwtCommDialog::do_page_setup_dlg;
GetOpenFileNameType AwtCommDialog::get_open_file_name;
GetSaveFileNameType AwtCommDialog::get_save_file_name;
GetExtendedErrorType AwtCommDialog::get_dlg_extended_error;
DragQueryFileType do_drag_query_file;
GetPathFromIDListType get_path_from_idlist;

/***********************************************************************/

/*
 * Load the COMDLG32.dll and get pointers to various procedures.
 */

DWORD
AwtCommDialog::CommDlgExtendedError(VOID) {
    AwtCommDialog::load_comdlg_procs();
    return (DWORD)AwtToolkit::GetInstance().
        InvokeFunction((void *(*)(void))
                     AwtCommDialog::GetExtendedErrorWrapper);
}

void
AwtCommDialog::load_comdlg_procs()
{
    static int initialized;
    HMODULE lib = NULL;
    if (initialized) {
	return;
    }
    lib = LoadLibrary("COMDLG32.DLL");
    do_print_dlg = (PrintDlgType)GetProcAddress(lib, "PrintDlgA");
    do_page_setup_dlg = (PageSetupDlgType)GetProcAddress(lib, "PageSetupDlgA");
    get_open_file_name = (GetOpenFileNameType)GetProcAddress(lib, "GetOpenFileNameA");
    get_save_file_name = (GetSaveFileNameType)GetProcAddress(lib, "GetSaveFileNameA");
    get_dlg_extended_error = (GetExtendedErrorType)GetProcAddress(lib, "CommDlgExtendedError");
    initialized = 1;
}

/***********************************************************************/

/*
 * Load the SHELL32.dll and get pointers to various procedures. 
 */

void
load_shell_procs()
{
    static int initialized;
    HMODULE lib = NULL;
    if (initialized) {
	return;
    }
    lib = LoadLibrary("SHELL32.DLL");
    do_drag_query_file = (DragQueryFileType)GetProcAddress(lib, "DragQueryFileA");
	get_path_from_idlist = (GetPathFromIDListType)GetProcAddress(lib,
		"SHGetPathFromIDListA");
    initialized = 1;
}

/***********************************************************************/

GetFileVersionInfoSizeType get_file_version_info_size;
GetFileVersionInfoType get_file_version_info;
VerQueryValueType do_ver_query_value;

/*
 * Load the VERSION.dll and get pointers to various procedures. 
 */

void
load_version_procs()
{
    static int initialized;
    HMODULE lib = NULL;
    if (initialized) {
        return;
    }
    lib = LoadLibrary("VERSION.DLL");
    get_file_version_info_size = 
        (GetFileVersionInfoSizeType)GetProcAddress(lib, "GetFileVersionInfoSizeA");
    get_file_version_info = 
        (GetFileVersionInfoType)GetProcAddress(lib, "GetFileVersionInfoA");
    do_ver_query_value = 
        (VerQueryValueType)GetProcAddress(lib, "VerQueryValueA");
    initialized = 1;
}

/***********************************************************************/

#define MAX_KNOWN_VERSION 4

/*
 * We are going to use an undocumented procedure RSRC32.DLL.
 * The safest will be to use it only for a finite set of known versions.
 */

DWORD known_versions[MAX_KNOWN_VERSION][2] = {
    { 0x00040000, 0x000003B6 },    // WIN95\RETAIL
                                   // WIN95\RETAIL\UPGRADE
                                   // WIN95\OSR2
                                   // WIN95\OSR25
                                   // WIN95\international\RETAIL
                                   // WIN95\international\OSR2

    { 0x0004000A, 0x00000672 },    // WIN98\international\win98beta3

    { 0x0004000A, 0x000007CE },    // WIN98\RETAIL
                                   // WIN98\international\RETAIL
                                   // WIN98\SE

    { 0x0004005A, 0x00000BB8 }     // WIN98ME
};

GetFreeSystemResourcesType get_free_system_resources = NULL;

/*
 * Load the RSRC32.dll, check that it is a known version
 * and get the pointer to the undocumented procedure. 
 */

void
load_rsrc32_procs() 
{
    static int initialized;
    if (initialized) {
        return;
    }
    if (IS_NT) {
        // 4310028: Only load library on non-NT systems. The load
        // will always fail anyways. However, if a Win 9x OS is
        // also installed on the system, and the user's path
        // includes C:\WINDOWS\SYSTEM, or the equivalent, a really
        // ugly and annoying warning dialog will appear.
        initialized = 1;
	return;
    }
    HMODULE lib = LoadLibrary("RSRC32.DLL");
    if (lib != NULL) {
        char      szFullPath[_MAX_PATH];
        DWORD     dwVerHnd = 0;
        DWORD     dwVersionInfoSize;
        LPBYTE    lpVersionInfo;
        LPVOID    lpBuffer;
        UINT      uLength = 0;

        /*
         * We use undocumented procedure exported by RSRC32.DLL, so the
         * safest will be to check the library's version and only attempt 
         * to get the procedure address if it's a known version.
         */
        if (::GetModuleFileName(lib, szFullPath, sizeof(szFullPath))) {
            load_version_procs();
            dwVersionInfoSize = (*get_file_version_info_size)(szFullPath, &dwVerHnd);
            if (dwVersionInfoSize) { 
                lpVersionInfo = new BYTE[dwVersionInfoSize];
                (*get_file_version_info)(szFullPath, dwVerHnd, 
                                         dwVersionInfoSize, lpVersionInfo);
                if ((*do_ver_query_value)(lpVersionInfo, "\\", &lpBuffer, &uLength)) {
                    VS_FIXEDFILEINFO *lpvsFixedFileInfo = (VS_FIXEDFILEINFO *)lpBuffer; 
                    DWORD dwFileVersionMS = lpvsFixedFileInfo->dwFileVersionMS; 
                    DWORD dwFileVersionLS = lpvsFixedFileInfo->dwFileVersionLS;
                    for (int i = 0; i < MAX_KNOWN_VERSION; i++) {
                        if ((known_versions[i][0] == dwFileVersionMS) &&
                            (known_versions[i][1] == dwFileVersionLS)) {
                            get_free_system_resources = 
                                (GetFreeSystemResourcesType)
                                ::GetProcAddress(lib, "_MyGetFreeSystemResources32@4");
                            break;
                        }
                    }
                }
                delete[] lpVersionInfo;
            }
        }
    }
    initialized = 1;
}
