/*
 * @(#)ProbeObject.java	1.5 00/02/02
 *
 * Copyright 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package sun.awt.robot.probe;

/**
 * Base class for all ProbeObjects.  These are generated by the MUT
 * at a probe point and sent to ProbeMonitor for dispersal.
 *
 * @see ProbeMonitor
 *
 * @author David Herron <tt>&lt;David.Herron@eng.sun.com&gt;</tt>
 */
public abstract class ProbeObject extends java.util.EventObject {

  static long count = 0;  // Count of ProbeObject's created - for sequence numbering
  static Object countLock = new Object();

  /**
   * Initialize the object.
   *
   * @param source The <i>source</i> from which this object sprang.
   */
  public ProbeObject(Object source) {
    //    super(source);
    this(source, "");
  }

  /**
   * Full initialization of the object.
   *
   * @param source The <i>source</i> from which this object sprang.
   * @param type The <i>type</i> of this ProbeObject.
   * @param info Miscellaneous information.
   */
  public ProbeObject(Object source, String info) {
    super(source);
    this.source    = source;
    this.otherInfo = info;
    this.when      = System.currentTimeMillis();
    synchronized(ProbeObject.countLock) {
      this.sequence  = ProbeObject.count++;
    }
  }


  /**
   * Contains the misc information attached to this object.
   */
  public String otherInfo;

  /**
   * Timestamp of when this event occurred.
   * @see java.lang.System#currentTimeMillis
   */
  public long when;

  /**
   * Counter of which ProbeObject this is.  Each ProbeObject gets
   * a unique number.
   */
  public long sequence;

  protected String paramString() { return ""; }

  /**
   * Generate an informational string about this object.
   */
  public String toString() {
    return this.getClass().getName()
      + ":[source=" + source
      + "(" + new Long(when).toString() + "," + new Long(sequence).toString() + "):"
      + "info=[" + otherInfo + "] params=[" + paramString() + "]];";
  }
}
