/*
 * @(#)ProbeStatus.java	1.5 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package sun.awt.robot.probe;

/**
 * Probe for specifying status information.  As the test executes
 * various status information of individual steps in the test
 * are interesting to know about.  This encapsulates the information
 * that is expected to be known, and the ProbeStatusBuffer is
 * a Probe Listener that is tuned to capturing these things.
 *
 * <p>
 *
 * @see ProbeObjectListener
 */
public class ProbeStatus extends ProbeObject {
	
  /**
   * This status is a SUCCESS indicator.  That is, the test
   * in question worked and there's nothing to snarq about.
   */
  public static final int SUCCESS = 1;
	
  /**
   * This status is a FAIL indicator.  That is, the test
   * in question failed and bells and alarms need to be raised.
   */
  public static final int FAIL    = 2;
	
  /**
   * This status is a WARN indicator.  That is, something
   * questionable happened and we're not sure if it is a
   * failure or what.  More investigation needed.
   */
  public static final int WARN    = 3;
	
  /**
   * This status indicates something less serious
   * than WARN happened.  Something a little smelly
   * happened that's probably okay but we're not sure.
   */
  public static final int KVETCH  = 4;
	
  /**
   * This status gives some other kind of information.
   * Something happened and that's all we know.
   */
  public static final int OTHER   = 5;

  public String statusName() {
    switch (status) {
    case ProbeStatus.SUCCESS: return "ProbeStatus.SUCCESS";
    case ProbeStatus.FAIL:    return "ProbeStatus.FAIL";
    case ProbeStatus.WARN:    return "ProbeStatus.WARN";
    case ProbeStatus.KVETCH:  return "ProbeStatus.KVETCH";
    case ProbeStatus.OTHER:   return "ProbeStatus.OTHER";
    }
    return "Unknown status " + new Integer(status).toString();
  }
	
  /**
   * The <i>status</i> carried by this event.
   * @see #SUCCESS
   * @see #FAIL
   * @see #WARN
   * @see #KVETCH
   * @see #OTHER
   */
  public int    status;
	
  /**
   * Initialization.
   * @param source Where this status indicator is coming from.
   * @param status The status being indicated.
   * @param message Other information this message can indicate.
   */
  public ProbeStatus(Object source,
                     int status, String message) {
     super(source, message);
     this.status = status;
  }

  protected String paramString() { return statusName(); }

}
